@@ -2,28 +2,28 @@
 /* vim:expandtab:shiftwidth=4:tabstop=4:
  */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Sun Microsystems, Inc.
- * Portions created by Sun Microsystems are Copyright (C) 2002 Sun
- * Microsystems, Inc. All Rights Reserved.
+ * The Initial Developer of the Original Code is
+ * Sun Microsystems, Inc.
+ * Portions created by the Initial Developer are Copyright (C) 2002
+ * the Initial Developer. All Rights Reserved.
  *
- * Original Author: Bolian Yin (bolian.yin@sun.com)
- *
- * Contributor(s): John Sun (john.sun@sun.com)
+ * Contributor(s):
+ *   Bolian Yin (bolian.yin@sun.com)
+ *   John Sun (john.sun@sun.com)
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -31,11 +31,11 @@
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -43,6 +43,8 @@
 #include "nsAccessibleWrap.h"
 #include "nsAppRootAccessible.h"
 #include "nsString.h"
+#include "prprf.h"
+#include "nsRoleMap.h"
 
 #include "nsMaiInterfaceComponent.h"
 #include "nsMaiInterfaceAction.h"
@@ -51,7 +53,14 @@
 #include "nsMaiInterfaceSelection.h"
 #include "nsMaiInterfaceValue.h"
 #include "nsMaiInterfaceHypertext.h"
+#include "nsMaiInterfaceHyperlinkImpl.h"
 #include "nsMaiInterfaceTable.h"
+#include "nsXPCOMStrings.h"
+#include "nsComponentManagerUtils.h"
+#include "nsIPersistentProperties2.h"
+#include "nsMaiInterfaceDocument.h"
+
+extern "C" GType g_atk_hyperlink_impl_type; //defined in nsAppRootAccessible.cpp
 
 /* MaiAtkObject */
 
@@ -67,6 +76,69 @@ enum {
   LAST_SIGNAL
 };
 
+enum MaiInterfaceType {
+    MAI_INTERFACE_COMPONENT, /* 0 */
+    MAI_INTERFACE_ACTION,
+    MAI_INTERFACE_VALUE,
+    MAI_INTERFACE_EDITABLE_TEXT,
+    MAI_INTERFACE_HYPERTEXT,
+    MAI_INTERFACE_HYPERLINK_IMPL,
+    MAI_INTERFACE_SELECTION,
+    MAI_INTERFACE_TABLE,
+    MAI_INTERFACE_TEXT,
+    MAI_INTERFACE_DOCUMENT /* 9 */
+};
+
+static GType GetAtkTypeForMai(MaiInterfaceType type)
+{
+  switch (type) {
+    case MAI_INTERFACE_COMPONENT:
+      return ATK_TYPE_COMPONENT;
+    case MAI_INTERFACE_ACTION:
+      return ATK_TYPE_ACTION;
+    case MAI_INTERFACE_VALUE:
+      return ATK_TYPE_VALUE;
+    case MAI_INTERFACE_EDITABLE_TEXT:
+      return ATK_TYPE_EDITABLE_TEXT;
+    case MAI_INTERFACE_HYPERTEXT:
+      return ATK_TYPE_HYPERTEXT;
+    case MAI_INTERFACE_HYPERLINK_IMPL:
+       return g_atk_hyperlink_impl_type;
+    case MAI_INTERFACE_SELECTION:
+      return ATK_TYPE_SELECTION;
+    case MAI_INTERFACE_TABLE:
+      return ATK_TYPE_TABLE;
+    case MAI_INTERFACE_TEXT:
+      return ATK_TYPE_TEXT;
+    case MAI_INTERFACE_DOCUMENT:
+      return ATK_TYPE_DOCUMENT;
+  }
+  return G_TYPE_INVALID;
+}
+
+static const GInterfaceInfo atk_if_infos[] = {
+    {(GInterfaceInitFunc)componentInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL}, 
+    {(GInterfaceInitFunc)actionInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL},
+    {(GInterfaceInitFunc)valueInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL},
+    {(GInterfaceInitFunc)editableTextInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL},
+    {(GInterfaceInitFunc)hypertextInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL},
+    {(GInterfaceInitFunc)hyperlinkImplInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL},
+    {(GInterfaceInitFunc)selectionInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL},
+    {(GInterfaceInitFunc)tableInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL},
+    {(GInterfaceInitFunc)textInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL},
+    {(GInterfaceInitFunc)documentInterfaceInitCB,
+     (GInterfaceFinalizeFunc) NULL, NULL}
+};
+
 /**
  * This MaiAtkObject is a thin wrapper, in the MAI namespace, for AtkObject
  */
@@ -102,15 +174,16 @@ static void finalizeCB(GObject *aObj);
 static const gchar*        getNameCB (AtkObject *aAtkObj);
 static const gchar*        getDescriptionCB (AtkObject *aAtkObj);
 static AtkRole             getRoleCB(AtkObject *aAtkObj);
+static AtkAttributeSet*    getAttributesCB(AtkObject *aAtkObj);
 static AtkObject*          getParentCB(AtkObject *aAtkObj);
 static gint                getChildCountCB(AtkObject *aAtkObj);
 static AtkObject*          refChildCB(AtkObject *aAtkObj, gint aChildIndex);
 static gint                getIndexInParentCB(AtkObject *aAtkObj);
 static AtkStateSet*        refStateSetCB(AtkObject *aAtkObj);
+static AtkRelationSet*     refRelationSetCB(AtkObject *aAtkObj);
 
 /* the missing atkobject virtual functions */
 /*
-  static AtkRelationSet*     refRelationSetCB(AtkObject *aAtkObj);
   static AtkLayer            getLayerCB(AtkObject *aAtkObj);
   static gint                getMdiZorderCB(AtkObject *aAtkObj);
   static void                SetNameCB(AtkObject *aAtkObj,
@@ -143,9 +216,8 @@ static AtkStateSet*        refStateSetCB(AtkObject *aAtkObj);
 */
 G_END_DECLS
 
-static GType GetMaiAtkType(const PRUint32 & interfaceCount,
-                           MaiInterface **interfaces);
-static const char * GetUniqueMaiAtkTypeName(void);
+static GType GetMaiAtkType(PRUint16 interfacesBits);
+static const char * GetUniqueMaiAtkTypeName(PRUint16 interfacesBits);
 
 static gpointer parent_class = NULL;
 
@@ -182,9 +254,7 @@ PRInt32 nsAccessibleWrap::mAccWrapDeleted = 0;
 nsAccessibleWrap::nsAccessibleWrap(nsIDOMNode* aNode,
                                    nsIWeakReference *aShell)
     : nsAccessible(aNode, aShell),
-      mMaiAtkObject(nsnull),
-      mInterfaces(nsnull),
-      mInterfaceCount(0)
+      mAtkObject(nsnull)
 {
 #ifdef MAI_LOGGING
     ++mAccWrapCreated;
@@ -204,43 +274,36 @@ nsAccessibleWrap::~nsAccessibleWrap()
                    (void*)this, mAccWrapDeleted,
                    (mAccWrapCreated-mAccWrapDeleted)));
 
-    if (mMaiAtkObject) {
-        MAI_ATK_OBJECT(mMaiAtkObject)->accWrap = nsnull;
-        g_object_unref(mMaiAtkObject);
+    if (mAtkObject) {
+        if (IS_MAI_OBJECT(mAtkObject)) {
+            MAI_ATK_OBJECT(mAtkObject)->accWrap = nsnull;
+        }
+        g_object_unref(mAtkObject);
     }
-    if (mInterfaces) {
-        for (int index = 0; index < MAI_INTERFACE_NUM; ++index)
-            delete mInterfaces[index];
-        delete [] mInterfaces;
-    }
-}
-
-NS_IMETHODIMP nsAccessibleWrap::GetExtState(PRUint32 *aState)
-{
-    PRUint32 state;
-    nsAccessible::GetState(&state);
-    if (!(state & STATE_INVISIBLE))
-      *aState |= STATE_SHOWING;
-    return NS_OK;
 }
 
 NS_IMETHODIMP nsAccessibleWrap::GetNativeInterface(void **aOutAccessible)
 {
     *aOutAccessible = nsnull;
 
-    if (!mMaiAtkObject) {
-        CreateMaiInterfaces();
-        mMaiAtkObject =
+    if (!IsEmbeddedObject(this)) {
+      // We don't create ATK objects for nsIAccessible plain text leaves
+      return NS_ERROR_FAILURE;
+    }
+    if (!mAtkObject) {
+        GType type = GetMaiAtkType(CreateMaiInterfaces());
+        NS_ENSURE_TRUE(type, NS_ERROR_FAILURE);
+        mAtkObject =
             NS_REINTERPRET_CAST(AtkObject *,
-                                g_object_new(GetMaiAtkType(mInterfaceCount, mInterfaces), NULL));
-        NS_ENSURE_TRUE(mMaiAtkObject, NS_ERROR_OUT_OF_MEMORY);
+                                g_object_new(type, NULL));
+        NS_ENSURE_TRUE(mAtkObject, NS_ERROR_OUT_OF_MEMORY);
 
-        atk_object_initialize(mMaiAtkObject, this);
-        mMaiAtkObject->role = ATK_ROLE_INVALID;
-        mMaiAtkObject->layer = ATK_LAYER_INVALID;
+        atk_object_initialize(mAtkObject, this);
+        mAtkObject->role = ATK_ROLE_INVALID;
+        mAtkObject->layer = ATK_LAYER_INVALID;
     }
 
-    *aOutAccessible = mMaiAtkObject;
+    *aOutAccessible = mAtkObject;
     return NS_OK;
 }
 
@@ -253,47 +316,32 @@ nsAccessibleWrap::GetAtkObject(void)
 }
 
 /* private */
-nsresult
+PRUint16
 nsAccessibleWrap::CreateMaiInterfaces(void)
 {
-    typedef MaiInterface * MaiInterfacePointer;
-    if (!mInterfaces) {
-        mInterfaces = new MaiInterfacePointer[MAI_INTERFACE_NUM];
-        for (PRUint16 index = 0; index < MAI_INTERFACE_NUM; ++index) {
-            mInterfaces[index] = nsnull;
-        }
-        NS_ENSURE_TRUE(mInterfaces, NS_ERROR_OUT_OF_MEMORY);
-    }
+    PRUint16 interfacesBits = 0;
+    
     // Add Interfaces for each nsIAccessible.ext interfaces
 
-    nsresult rv;
-
     // the Component interface are supported by all nsIAccessible
-    MaiInterfaceComponent *maiInterfaceComponent =
-        new MaiInterfaceComponent(this);
-    NS_ENSURE_TRUE(maiInterfaceComponent, NS_ERROR_OUT_OF_MEMORY);
-    rv = AddMaiInterface(maiInterfaceComponent);
-    NS_ENSURE_SUCCESS(rv, rv);
+    interfacesBits |= 1 << MAI_INTERFACE_COMPONENT;
 
     // Add Action interface if the action count is more than zero.
     PRUint8 actionCount = 0;
-    rv = GetNumActions(&actionCount);
+    nsresult rv = GetNumActions(&actionCount);
     if (NS_SUCCEEDED(rv) && actionCount > 0) {
-        MaiInterfaceAction *maiInterfaceAction = new MaiInterfaceAction(this);
-        NS_ENSURE_TRUE(maiInterfaceAction, NS_ERROR_OUT_OF_MEMORY);
-        rv = AddMaiInterface(maiInterfaceAction);
-        NS_ENSURE_SUCCESS(rv, rv);
+       interfacesBits |= 1 << MAI_INTERFACE_ACTION; 
     }
 
+    PRUint32 accRole;
+    GetRole(&accRole);
+
     //nsIAccessibleText
     nsCOMPtr<nsIAccessibleText> accessInterfaceText;
     QueryInterface(NS_GET_IID(nsIAccessibleText),
                    getter_AddRefs(accessInterfaceText));
     if (accessInterfaceText) {
-        MaiInterfaceText *maiInterfaceText = new MaiInterfaceText(this);
-        NS_ENSURE_TRUE(maiInterfaceText, NS_ERROR_OUT_OF_MEMORY);
-        rv = AddMaiInterface(maiInterfaceText);
-        NS_ENSURE_SUCCESS(rv, rv);
+        interfacesBits |= 1 << MAI_INTERFACE_TEXT;
     }
 
     //nsIAccessibleEditableText
@@ -301,11 +349,7 @@ nsAccessibleWrap::CreateMaiInterfaces(void)
     QueryInterface(NS_GET_IID(nsIAccessibleEditableText),
                    getter_AddRefs(accessInterfaceEditableText));
     if (accessInterfaceEditableText) {
-        MaiInterfaceEditableText *maiInterfaceEditableText =
-            new MaiInterfaceEditableText(this);
-        NS_ENSURE_TRUE(maiInterfaceEditableText, NS_ERROR_OUT_OF_MEMORY);
-        rv = AddMaiInterface(maiInterfaceEditableText);
-        NS_ENSURE_SUCCESS(rv, rv);
+        interfacesBits |= 1 << MAI_INTERFACE_EDITABLE_TEXT;
     }
 
     //nsIAccessibleSelection
@@ -313,11 +357,7 @@ nsAccessibleWrap::CreateMaiInterfaces(void)
     QueryInterface(NS_GET_IID(nsIAccessibleSelectable),
                    getter_AddRefs(accessInterfaceSelection));
     if (accessInterfaceSelection) {
-        MaiInterfaceSelection *maiInterfaceSelection =
-            new MaiInterfaceSelection(this);
-        NS_ENSURE_TRUE(maiInterfaceSelection, NS_ERROR_OUT_OF_MEMORY);
-        rv = AddMaiInterface(maiInterfaceSelection);
-        NS_ENSURE_SUCCESS(rv, rv);
+        interfacesBits |= 1 << MAI_INTERFACE_SELECTION;
     }
 
     //nsIAccessibleValue
@@ -325,10 +365,7 @@ nsAccessibleWrap::CreateMaiInterfaces(void)
     QueryInterface(NS_GET_IID(nsIAccessibleValue),
                    getter_AddRefs(accessInterfaceValue));
     if (accessInterfaceValue) {
-        MaiInterfaceValue *maiInterfaceValue = new MaiInterfaceValue(this);
-        NS_ENSURE_TRUE(maiInterfaceValue, NS_ERROR_OUT_OF_MEMORY);
-        rv = AddMaiInterface(maiInterfaceValue);
-        NS_ENSURE_SUCCESS(rv, rv);
+       interfacesBits |= 1 << MAI_INTERFACE_VALUE; 
     }
 
     //nsIAccessibleHypertext
@@ -337,58 +374,41 @@ nsAccessibleWrap::CreateMaiInterfaces(void)
     QueryInterface(NS_GET_IID(nsIAccessibleHyperText),
                    getter_AddRefs(accessInterfaceHypertext));
     if (accessInterfaceHypertext) {
-        rv = accessInterfaceHypertext->GetLinks(&linkCount);
+        nsresult rv = accessInterfaceHypertext->GetLinks(&linkCount);
         if (NS_SUCCEEDED(rv) && (linkCount > 0)) {
-            MaiInterfaceHypertext *maiInterfaceHypertext =
-                new MaiInterfaceHypertext(this, mWeakShell);
-            NS_ENSURE_TRUE(maiInterfaceHypertext, NS_ERROR_OUT_OF_MEMORY);
-            rv = AddMaiInterface(maiInterfaceHypertext);
-            NS_ENSURE_SUCCESS(rv, rv);
+            interfacesBits |= 1 << MAI_INTERFACE_HYPERTEXT;
         }
     }
 
+    //nsIAccessibleHyperLink
+    nsCOMPtr<nsIAccessibleHyperLink> accessInterfaceHyperlink;
+    QueryInterface(NS_GET_IID(nsIAccessibleHyperLink),
+                   getter_AddRefs(accessInterfaceHyperlink));
+    if (accessInterfaceHyperlink) {
+       interfacesBits |= 1 << MAI_INTERFACE_HYPERLINK_IMPL;
+    }
+
     //nsIAccessibleTable
     nsCOMPtr<nsIAccessibleTable> accessInterfaceTable;
     QueryInterface(NS_GET_IID(nsIAccessibleTable),
                    getter_AddRefs(accessInterfaceTable));
     if (accessInterfaceTable) {
-        MaiInterfaceTable *maiInterfaceTable = new MaiInterfaceTable(this);
-        NS_ENSURE_TRUE(maiInterfaceTable, NS_ERROR_OUT_OF_MEMORY);
-        rv = AddMaiInterface(maiInterfaceTable);
-        NS_ENSURE_SUCCESS(rv, rv);
+        interfacesBits |= 1 << MAI_INTERFACE_TABLE;
     }
 
-    return rv;
-}
-
-nsresult
-nsAccessibleWrap::AddMaiInterface(MaiInterface *aMaiIface)
-{
-    NS_ENSURE_ARG_POINTER(aMaiIface);
-    MaiInterfaceType aMaiIfaceType = aMaiIface->GetType();
-
-    if ((aMaiIfaceType <= MAI_INTERFACE_INVALID) ||
-        (aMaiIfaceType >= MAI_INTERFACE_NUM))
-        return NS_ERROR_FAILURE;
-
-    // if same type of If has been added, release previous one
-    if (mInterfaces[aMaiIfaceType]) {
-        delete mInterfaces[aMaiIfaceType];
+    //nsIAccessibleDocument
+    nsCOMPtr<nsIAccessibleDocument> accessInterfaceDocument;
+    QueryInterface(NS_GET_IID(nsIAccessibleDocument),
+                              getter_AddRefs(accessInterfaceDocument));
+    if (accessInterfaceDocument) {
+        interfacesBits |= 1 << MAI_INTERFACE_DOCUMENT;
     }
-    mInterfaces[aMaiIfaceType] = aMaiIface;
-    mInterfaceCount++;
-    return NS_OK;
-}
-MaiInterface *
-nsAccessibleWrap::GetMaiInterface(PRInt16 aIfaceType)
-{
-    NS_ENSURE_TRUE(aIfaceType > MAI_INTERFACE_INVALID, nsnull);
-    NS_ENSURE_TRUE(aIfaceType < MAI_INTERFACE_NUM, nsnull);
-    return mInterfaces[aIfaceType];
+
+    return interfacesBits;
 }
 
 static GType
-GetMaiAtkType(const PRUint32 & interfaceCount, MaiInterface **interfaces)
+GetMaiAtkType(PRUint16 interfacesBits)
 {
     GType type;
     static const GTypeInfo tinfo = {
@@ -404,33 +424,51 @@ GetMaiAtkType(const PRUint32 & interfaceCount, MaiInterface **interfaces)
         NULL /* value table */
     };
 
-    if (interfaceCount == 0)
-        return MAI_TYPE_ATK_OBJECT;
+    /*
+     * The members we use to register GTypes are GetAtkTypeForMai
+     * and atk_if_infos, which are constant values to each MaiInterface
+     * So we can reuse the registered GType when having
+     * the same MaiInterface types.
+     */
+    const char *atkTypeName = GetUniqueMaiAtkTypeName(interfacesBits);
+    type = g_type_from_name(atkTypeName);
+    if (type) {
+        return type;
+    }
 
+    /*
+     * gobject limits the number of types that can directly derive from any
+     * given object type to 4095.
+     */
+    static PRUint16 typeRegCount = 0;
+    if (typeRegCount++ >= 4095) {
+        return G_TYPE_INVALID;
+    }
     type = g_type_register_static(MAI_TYPE_ATK_OBJECT,
-                                  GetUniqueMaiAtkTypeName(),
+                                  atkTypeName,
                                   &tinfo, GTypeFlags(0));
 
-    for (int index = 0; index < MAI_INTERFACE_NUM; index++) {
-        if (!interfaces[index])
-            continue;
+    for (PRUint32 index = 0; index < NS_ARRAY_LENGTH(atk_if_infos); index++) {
+      if (interfacesBits & (1 << index)) {
         g_type_add_interface_static(type,
-                                    interfaces[index]->GetAtkType(),
-                                    interfaces[index]->GetInterfaceInfo());
+                                    GetAtkTypeForMai((MaiInterfaceType)index),
+                                    &atk_if_infos[index]);
+      }
     }
+
     return type;
 }
 
 static const char *
-GetUniqueMaiAtkTypeName(void)
+GetUniqueMaiAtkTypeName(PRUint16 interfacesBits)
 {
-#define MAI_ATK_TYPE_NAME_LEN (30)     /* 10+sizeof(gulong)*8/4+1 < 30 */
+#define MAI_ATK_TYPE_NAME_LEN (30)     /* 10+sizeof(PRUint16)*8/4+1 < 30 */
 
-    static PRUint32 atkTypeNameIndex = 0;
     static gchar namePrefix[] = "MaiAtkType";   /* size = 10 */
     static gchar name[MAI_ATK_TYPE_NAME_LEN + 1];
 
-    sprintf(name, "%s%x", namePrefix, atkTypeNameIndex++);
+    PR_snprintf(name, MAI_ATK_TYPE_NAME_LEN, "%s%x", namePrefix,
+                interfacesBits);
     name[MAI_ATK_TYPE_NAME_LEN] = '\0';
 
     MAI_LOG_DEBUG(("MaiWidget::LastedTypeName=%s\n", name));
@@ -444,7 +482,6 @@ The following nsIAccessible states aren't translated, just ignored.
   STATE_READONLY:      The object is designated read-only.
   STATE_HOTTRACKED:    Means its appearance has changed to indicate mouse
                        over it.
-  STATE_DEFAULT:       Represents the default button in a window.
   STATE_FLOATING:      Not supported yet.
   STATE_MARQUEED:      Indicate scrolling or moving text or graphics.
   STATE_ANIMATED:
@@ -464,7 +501,6 @@ Returned AtkStatusSet never contain the following AtkStates.
   ATK_STATE_ARMED:     Indicates that the object is armed.
   ATK_STATE_DEFUNCT:   Indicates the user interface object corresponding to
                        thus object no longer exists.
-  ATK_STATE_EDITABLE:  Indicates the user can change the contents of the object.
   ATK_STATE_HORIZONTAL:Indicates the orientation of this object is horizontal.
   ATK_STATE_ICONIFIED:
   ATK_STATE_OPAQUE:     Indicates the object paints every pixel within its
@@ -474,7 +510,7 @@ Returned AtkStatusSet never contain the following AtkStates.
 ******************************************************************************/
 
 void
-nsAccessibleWrap::TranslateStates(PRUint32 aState, void *aAtkStateSet)
+nsAccessibleWrap::TranslateStates(PRUint32 aState, PRUint32 aExtState, void *aAtkStateSet)
 {
     if (!aAtkStateSet)
         return;
@@ -522,41 +558,59 @@ nsAccessibleWrap::TranslateStates(PRUint32 aState, void *aAtkStateSet)
         atk_state_set_add_state (state_set, ATK_STATE_SENSITIVE);
     }
 
+    if (aState & nsIAccessible::STATE_INVALID)
+        atk_state_set_add_state (state_set, ATK_STATE_INVALID_ENTRY);
+
+    if (aState & nsIAccessible::STATE_DEFAULT)
+        atk_state_set_add_state (state_set, ATK_STATE_DEFAULT);
+
+    if (aState & nsIAccessible::STATE_REQUIRED)
+        atk_state_set_add_state (state_set, ATK_STATE_REQUIRED);
+
+    if (aState & nsIAccessible::STATE_TRAVERSED)
+        atk_state_set_add_state (state_set, ATK_STATE_VISITED);
+
+    if (aState & nsIAccessible::STATE_ANIMATED)
+        atk_state_set_add_state (state_set, ATK_STATE_ANIMATED);
+
     // The following state is
     // Extended state flags (for now non-MSAA, for Java and Gnome/ATK support)
-    // This is only the states that there isn't already a mapping for in MSAA
-    // See www.accessmozilla.org/article.php?sid=11 for information on the
-    // mappings between accessibility API state
-    if (aState & nsIAccessible::STATE_INVALID)
-        atk_state_set_add_state (state_set, ATK_STATE_INVALID);
+    if (aExtState & nsIAccessible::EXT_STATE_SELECTABLE_TEXT)
+        atk_state_set_add_state (state_set, ATK_STATE_SELECTABLE_TEXT);
 
-    if (aState & nsIAccessible::STATE_ACTIVE)
+    if (aExtState & nsIAccessible::EXT_STATE_ACTIVE)
         atk_state_set_add_state (state_set, ATK_STATE_ACTIVE);
 
-    if (aState & nsIAccessible::STATE_EXPANDABLE)
+    if (aExtState & nsIAccessible::EXT_STATE_EXPANDABLE)
         atk_state_set_add_state (state_set, ATK_STATE_EXPANDABLE);
 
-    if (aState & nsIAccessible::STATE_MODAL)
+    if (aExtState & nsIAccessible::EXT_STATE_MODAL)
         atk_state_set_add_state (state_set, ATK_STATE_MODAL);
 
-    if (aState & nsIAccessible::STATE_MULTI_LINE)
+    if (aExtState & nsIAccessible::EXT_STATE_MULTI_LINE)
         atk_state_set_add_state (state_set, ATK_STATE_MULTI_LINE);
 
-    if (aState & nsIAccessible::STATE_SENSITIVE)
+    if (aExtState & nsIAccessible::EXT_STATE_SENSITIVE)
         atk_state_set_add_state (state_set, ATK_STATE_SENSITIVE);
 
-    if (aState & nsIAccessible::STATE_SHOWING)
+    if (aExtState & nsIAccessible::EXT_STATE_SHOWING)
         atk_state_set_add_state (state_set, ATK_STATE_SHOWING);
 
-    if (aState & nsIAccessible::STATE_SINGLE_LINE)
+    if (aExtState & nsIAccessible::EXT_STATE_SINGLE_LINE)
         atk_state_set_add_state (state_set, ATK_STATE_SINGLE_LINE);
 
-    if (aState & nsIAccessible::STATE_TRANSIENT)
+    if (aExtState & nsIAccessible::EXT_STATE_TRANSIENT)
         atk_state_set_add_state (state_set, ATK_STATE_TRANSIENT);
 
-    if (aState & nsIAccessible::STATE_VERTICAL)
+    if (aExtState & nsIAccessible::EXT_STATE_VERTICAL)
         atk_state_set_add_state (state_set, ATK_STATE_VERTICAL);
 
+    if (aExtState & nsIAccessible::EXT_STATE_EDITABLE)
+        atk_state_set_add_state (state_set, ATK_STATE_EDITABLE);
+
+    if (aExtState & nsIAccessible::EXT_STATE_DEFUNCT)
+        atk_state_set_add_state (state_set, ATK_STATE_DEFUNCT);
+
 }
 
 PRBool nsAccessibleWrap::IsValidObject()
@@ -580,7 +634,9 @@ classInitCB(AtkObjectClass *aClass)
     aClass->ref_child = refChildCB;
     aClass->get_index_in_parent = getIndexInParentCB;
     aClass->get_role = getRoleCB;
+    aClass->get_attributes = getAttributesCB;
     aClass->ref_state_set = refStateSetCB;
+    aClass->ref_relation_set = refRelationSetCB;
 
     aClass->initialize = initializeCB;
 
@@ -656,7 +712,7 @@ classInitCB(AtkObjectClass *aClass)
 void
 initializeCB(AtkObject *aAtkObj, gpointer aData)
 {
-    NS_ASSERTION((MAI_IS_ATK_OBJECT(aAtkObj)), "Invalid AtkObject");
+    NS_ASSERTION((IS_MAI_OBJECT(aAtkObj)), "Invalid AtkObject");
     NS_ASSERTION(aData, "Invalid Data to init AtkObject");
     if (!aAtkObj || !aData)
         return;
@@ -684,7 +740,7 @@ initializeCB(AtkObject *aAtkObj, gpointer aData)
 void
 finalizeCB(GObject *aObj)
 {
-    if (!MAI_IS_ATK_OBJECT(aObj))
+    if (!IS_MAI_OBJECT(aObj))
         return;
     NS_ASSERTION(MAI_ATK_OBJECT(aObj)->accWrap == nsnull, "AccWrap NOT null");
 
@@ -716,10 +772,10 @@ getNameCB(AtkObject *aAtkObj)
     NS_ENSURE_SUCCESS(rv, nsnull);
 
     if (uniName.Length() > 0) {
-        NS_ConvertUTF8toUCS2 objName(aAtkObj->name);
+        NS_ConvertUTF8toUTF16 objName(aAtkObj->name);
         if (!uniName.Equals(objName)) {
             atk_object_set_name(aAtkObj,
-                                NS_ConvertUCS2toUTF8(uniName).get());
+                                NS_ConvertUTF16toUTF8(uniName).get());
         }
     }
     return aAtkObj->name;
@@ -743,7 +799,7 @@ getDescriptionCB(AtkObject *aAtkObj)
         len = uniDesc.Length();
         if (len > 0) {
             atk_object_set_description(aAtkObj,
-                                       NS_ConvertUCS2toUTF8(uniDesc).get());
+                                       NS_ConvertUTF16toUTF8(uniDesc).get());
         }
     }
     return aAtkObj->description;
@@ -758,32 +814,65 @@ getRoleCB(AtkObject *aAtkObj)
         nsAccessibleWrap *accWrap =
             NS_REINTERPRET_CAST(MaiAtkObject*, aAtkObj)->accWrap;
 
-        PRUint32 accRole;
-        nsresult rv = accWrap->GetRole(&accRole);
+        PRUint32 accRole, atkRole;
+        nsresult rv = accWrap->GetFinalRole(&accRole);
         NS_ENSURE_SUCCESS(rv, ATK_ROLE_INVALID);
 
-        //the cross-platform Accessible object returns the same value for
-        //both "ROLE_MENUITEM" and "ROLE_MENUPOPUP"
-        if (accRole == nsIAccessible::ROLE_MENUITEM) {
-            PRInt32 childCount = 0;
-            accWrap->GetChildCount(&childCount);
-            if (childCount > 0)
-                accRole = nsIAccessible::ROLE_MENUPOPUP;
-        }
-        else if (accRole == nsIAccessible::ROLE_LINK) {
-            //ATK doesn't have role-link now
-            //register it on runtime
-            static AtkRole linkRole = (AtkRole)0;
-            if (linkRole == 0) {
-                linkRole = atk_role_register("hyper link");
-            }
-            accRole = linkRole;
-        }
-        aAtkObj->role = NS_STATIC_CAST(AtkRole, accRole);
+        atkRole = atkRoleMap[accRole]; // map to the actual value
+        NS_ASSERTION(atkRoleMap[nsIAccessible::ROLE_LAST_ENTRY] ==
+                     kROLE_ATK_LAST_ENTRY, "ATK role map skewed");
+        aAtkObj->role = NS_STATIC_CAST(AtkRole, atkRole);
     }
     return aAtkObj->role;
 }
 
+AtkAttributeSet *
+GetAttributeSet(nsIAccessible* aAccessible)
+{
+    AtkAttributeSet *objAttributeSet = nsnull;
+    nsCOMPtr<nsIPersistentProperties> attributes;
+    aAccessible->GetAttributes(getter_AddRefs(attributes));
+
+    if (attributes) {
+        nsCOMPtr<nsISimpleEnumerator> propEnum;
+        nsresult rv = attributes->Enumerate(getter_AddRefs(propEnum));
+        NS_ENSURE_SUCCESS(rv, nsnull);
+
+        PRBool hasMore;
+        while (NS_SUCCEEDED(propEnum->HasMoreElements(&hasMore)) && hasMore) {
+            nsCOMPtr<nsISupports> sup;
+            rv = propEnum->GetNext(getter_AddRefs(sup));
+            nsCOMPtr<nsIPropertyElement> propElem(do_QueryInterface(sup));
+            NS_ENSURE_TRUE(propElem, nsnull);
+
+            nsCAutoString name;
+            rv = propElem->GetKey(name);
+            NS_ENSURE_SUCCESS(rv, nsnull);
+
+            nsAutoString value;
+            rv = propElem->GetValue(value);
+            NS_ENSURE_SUCCESS(rv, nsnull);
+
+            AtkAttribute *objAttribute = (AtkAttribute *)g_malloc(sizeof(AtkAttribute));
+            objAttribute->name = g_strdup(name.get());
+            objAttribute->value = g_strdup(NS_ConvertUTF16toUTF8(value).get());
+            objAttributeSet = g_slist_prepend(objAttributeSet, objAttribute);
+        }
+    }
+
+    return objAttributeSet;
+}
+
+AtkAttributeSet *
+getAttributesCB(AtkObject *aAtkObj)
+{
+    NS_ENSURE_SUCCESS(CheckMaiAtkObject(aAtkObj), nsnull);
+    nsAccessibleWrap *accWrap =
+        NS_REINTERPRET_CAST(MaiAtkObject*, aAtkObj)->accWrap;
+
+    return GetAttributeSet(accWrap);
+}
+
 AtkObject *
 getParentCB(AtkObject *aAtkObj)
 {
@@ -814,20 +903,55 @@ getChildCountCB(AtkObject *aAtkObj)
         NS_REINTERPRET_CAST(MaiAtkObject*, aAtkObj)->accWrap;
 
     PRInt32 count = 0;
-    accWrap->GetChildCount(&count);
+    nsCOMPtr<nsIAccessibleHyperText> hyperText;
+    accWrap->QueryInterface(NS_GET_IID(nsIAccessibleHyperText), getter_AddRefs(hyperText));
+    if (hyperText) {
+        // If HyperText, then number of links matches number of children
+        hyperText->GetLinks(&count);
+    }
+    else {
+        nsCOMPtr<nsIAccessibleText> accText;
+        accWrap->QueryInterface(NS_GET_IID(nsIAccessibleText), getter_AddRefs(accText));
+        if (!accText) {    // Accessible text that is not a HyperText has no children
+            accWrap->GetChildCount(&count);
+        }
+    }
     return count;
 }
 
 AtkObject *
 refChildCB(AtkObject *aAtkObj, gint aChildIndex)
 {
+    // aChildIndex should not be less than zero
+    if (aChildIndex < 0) {
+      return nsnull;
+    }
+
+    // XXX Fix this so it is not O(n^2) to walk through the children!
+    // Either we can cache the last accessed child so that we can just GetNextSibling()
+    // or we should cache an array of children in each nsAccessible
+    // (instead of mNextSibling on the children)
     NS_ENSURE_SUCCESS(CheckMaiAtkObject(aAtkObj), nsnull);
     nsAccessibleWrap *accWrap =
         NS_REINTERPRET_CAST(MaiAtkObject*, aAtkObj)->accWrap;
 
     nsresult rv;
     nsCOMPtr<nsIAccessible> accChild;
-    rv = accWrap->GetChildAt(aChildIndex, getter_AddRefs(accChild));
+    nsCOMPtr<nsIAccessibleHyperText> hyperText;
+    accWrap->QueryInterface(NS_GET_IID(nsIAccessibleHyperText), getter_AddRefs(hyperText));
+    if (hyperText) {
+        // If HyperText, then number of links matches number of children
+        nsCOMPtr<nsIAccessibleHyperLink> hyperLink;
+        rv = hyperText->GetLink(aChildIndex, getter_AddRefs(hyperLink));
+        accChild = do_QueryInterface(hyperLink);
+    }
+    else {
+        nsCOMPtr<nsIAccessibleText> accText;
+        accWrap->QueryInterface(NS_GET_IID(nsIAccessibleText), getter_AddRefs(accText));
+        if (!accText) {  // Accessible Text that is not HyperText has no children
+            rv = accWrap->GetChildAt(aChildIndex, getter_AddRefs(accChild));
+        }
+    }
 
     if (NS_FAILED(rv) || !accChild)
         return nsnull;
@@ -850,12 +974,41 @@ refChildCB(AtkObject *aAtkObj, gint aChildIndex)
 gint
 getIndexInParentCB(AtkObject *aAtkObj)
 {
+    // We don't use nsIAccessible::GetIndexInParent() because
+    // for ATK we don't want to include text leaf nodes as children
     NS_ENSURE_SUCCESS(CheckMaiAtkObject(aAtkObj), -1);
     nsAccessibleWrap *accWrap =
         NS_REINTERPRET_CAST(MaiAtkObject*, aAtkObj)->accWrap;
 
-    PRInt32 currentIndex = -1;
-    accWrap->GetIndexInParent(&currentIndex);
+    nsCOMPtr<nsIAccessible> parent;
+    accWrap->GetParent(getter_AddRefs(parent));
+    if (!parent) {
+        return -1; // No parent
+    }
+
+    nsCOMPtr<nsIAccessible> sibling;
+    parent->GetFirstChild(getter_AddRefs(sibling));
+    if (!sibling) {
+        return -1;  // Error, parent has no children
+    }
+
+    PRInt32 currentIndex = 0;
+
+    while (sibling != NS_STATIC_CAST(nsIAccessible*, accWrap)) {
+      NS_ASSERTION(sibling, "Never ran into the same child that we started from");
+
+      if (!sibling) {
+          return -1;
+      }
+      if (nsAccessible::IsEmbeddedObject(sibling)) {
+        ++ currentIndex;
+      }
+
+      nsCOMPtr<nsIAccessible> tempAccessible;
+      sibling->GetNextSibling(getter_AddRefs(tempAccessible));
+      sibling.swap(tempAccessible);
+    }
+
     return currentIndex;
 }
 
@@ -870,24 +1023,70 @@ refStateSetCB(AtkObject *aAtkObj)
         NS_REINTERPRET_CAST(MaiAtkObject*, aAtkObj)->accWrap;
 
     PRUint32 accState = 0;
-    nsresult rv = accWrap->GetState(&accState);
+    nsresult rv = accWrap->GetFinalState(&accState);
     NS_ENSURE_SUCCESS(rv, state_set);
 
-    rv = accWrap->GetExtState(&accState);
+    PRUint32 accExtState = 0;
+    rv = accWrap->GetExtState(&accExtState);
     NS_ENSURE_SUCCESS(rv, state_set);
 
-    if (accState == 0)
+    if ((accState == 0) && (accExtState == 0))
       return state_set;
 
-    nsAccessibleWrap::TranslateStates(accState, state_set);
+    nsAccessibleWrap::TranslateStates(accState, accExtState, state_set);
     return state_set;
 }
 
+AtkRelationSet *
+refRelationSetCB(AtkObject *aAtkObj)
+{
+    AtkRelationSet *relation_set = nsnull;
+    relation_set = ATK_OBJECT_CLASS(parent_class)->ref_relation_set(aAtkObj);
+
+    NS_ENSURE_SUCCESS(CheckMaiAtkObject(aAtkObj), relation_set);
+    nsAccessibleWrap *accWrap =
+        NS_REINTERPRET_CAST(MaiAtkObject*, aAtkObj)->accWrap;
+
+    AtkObject *accessible_array[1];
+    AtkRelation* relation;
+    
+    PRUint32 relationType[] = {nsIAccessible::RELATION_LABELLED_BY,
+                               nsIAccessible::RELATION_LABEL_FOR,
+                               nsIAccessible::RELATION_NODE_CHILD_OF,
+                               nsIAccessible::RELATION_CONTROLLED_BY,
+                               nsIAccessible::RELATION_CONTROLLER_FOR,
+                               nsIAccessible::RELATION_EMBEDS,
+                               nsIAccessible::RELATION_FLOWS_TO,
+                               nsIAccessible::RELATION_FLOWS_FROM,
+                               nsIAccessible::RELATION_DESCRIBED_BY,
+                               nsIAccessible::RELATION_DESCRIPTION_FOR,
+                               };
+
+    for (PRUint32 i = 0; i < NS_ARRAY_LENGTH(relationType); i++) {
+        relation = atk_relation_set_get_relation_by_type(relation_set, NS_STATIC_CAST(AtkRelationType, relationType[i]));
+        if (relation) {
+            atk_relation_set_remove(relation_set, relation);
+        }
+
+        nsIAccessible* accRelated;
+        nsresult rv = accWrap->GetAccessibleRelated(relationType[i], &accRelated);
+        if (NS_SUCCEEDED(rv) && accRelated) {
+            accessible_array[0] = NS_STATIC_CAST(nsAccessibleWrap*, accRelated)->GetAtkObject();
+            relation = atk_relation_new(accessible_array, 1,
+                                        NS_STATIC_CAST(AtkRelationType, relationType[i]));
+            atk_relation_set_add(relation_set, relation);
+            g_object_unref(relation);
+        }
+    }
+
+    return relation_set;
+}
+
 // Check if aAtkObj is a valid MaiAtkObject
 nsresult
 CheckMaiAtkObject(AtkObject *aAtkObj)
 {
-    NS_ENSURE_ARG(MAI_IS_ATK_OBJECT(aAtkObj));
+    NS_ENSURE_ARG(IS_MAI_OBJECT(aAtkObj));
     nsAccessibleWrap * tmpAccWrap = MAI_ATK_OBJECT(aAtkObj)->accWrap;
     if (tmpAccWrap == nsnull)
         return NS_ERROR_INVALID_POINTER;
@@ -902,7 +1101,7 @@ CheckMaiAtkObject(AtkObject *aAtkObj)
 // for it.
 nsAccessibleWrap *GetAccessibleWrap(AtkObject *aAtkObj)
 {
-    NS_ENSURE_TRUE(MAI_IS_ATK_OBJECT(aAtkObj), nsnull);
+    NS_ENSURE_TRUE(IS_MAI_OBJECT(aAtkObj), nsnull);
     nsAccessibleWrap * tmpAccWrap = MAI_ATK_OBJECT(aAtkObj)->accWrap;
     NS_ENSURE_TRUE(tmpAccWrap != nsnull, nsnull);
     NS_ENSURE_TRUE(tmpAccWrap->GetAtkObject() == aAtkObj, nsnull);
