@@ -2,28 +2,29 @@
 /* vim:expandtab:shiftwidth=4:tabstop=4:
  */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Sun Microsystems, Inc.
- * Portions created by Sun Microsystems are Copyright (C) 2002 Sun
- * Microsystems, Inc. All Rights Reserved.
+ * The Initial Developer of the Original Code is
+ * Sun Microsystems, Inc.
+ * Portions created by the Initial Developer are Copyright (C) 2002
+ * the Initial Developer. All Rights Reserved.
  *
- * Original Author: Bolian Yin (bolian.yin@sun.com)
- *
- * Contributor(s): John Sun (john.sun@sun.com)
+ * Contributor(s):
+ *   Bolian Yin (bolian.yin@sun.com)
+ *   John Sun (john.sun@sun.com)
+ *   Ginn Chen (ginn.chen@sun.com)
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -31,15 +32,17 @@
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsMai.h"
+#include "nsAutoPtr.h"
+#include "nsRootAccessible.h"
 #include "nsDocAccessibleWrap.h"
 #include "nsAccessibleEventData.h"
 
@@ -70,13 +73,13 @@ static char * sAtkPropertyNameArray[PROP_LAST] = {
     "accessible_table_summary"
 };
 
-static  AtkStateType TranslateAState(PRUint32 aState);
+static  AtkStateType TranslateAState(PRUint32 aState, PRUint32 aExtState);
 
-NS_IMPL_ISUPPORTS_INHERITED2(nsDocAccessibleWrap, nsDocAccessible, nsIAccessibleText, nsIAccessibleEditableText)
+NS_IMPL_ISUPPORTS_INHERITED1(nsDocAccessibleWrap, nsDocAccessible, nsIAccessibleText)
 
 nsDocAccessibleWrap::nsDocAccessibleWrap(nsIDOMNode *aDOMNode,
                                          nsIWeakReference *aShell): 
-  nsDocAccessible(aDOMNode, aShell), nsAccessibleEditableText(aDOMNode)
+  nsDocAccessible(aDOMNode, aShell), mActivated(PR_FALSE)
 {
 }
 
@@ -105,10 +108,14 @@ NS_IMETHODIMP nsDocAccessibleWrap::FireToolkitEvent(PRUint32 aEvent,
 
     switch (aEvent) {
     case nsIAccessibleEvent::EVENT_FOCUS:
+      {
         MAI_LOG_DEBUG(("\n\nReceived: EVENT_FOCUS\n"));
-        atk_focus_tracker_notify(accWrap->GetAtkObject());
+        nsRefPtr<nsRootAccessible> rootAccWrap = accWrap->GetRootAccessible();
+        if (rootAccWrap && rootAccWrap->mActivated) {
+          atk_focus_tracker_notify(accWrap->GetAtkObject());
+        }
         rv = NS_OK;
-        break;
+      } break;
 
     case nsIAccessibleEvent::EVENT_STATE_CHANGE:
         AtkStateChange *pAtkStateChange;
@@ -134,7 +141,7 @@ NS_IMETHODIMP nsDocAccessibleWrap::FireToolkitEvent(PRUint32 aEvent,
             pAtkStateChange->enable = !pAtkStateChange->enable;
             break;
         default:
-            atkState = TranslateAState(pAtkStateChange->state);
+            atkState = TranslateAState(pAtkStateChange->state, pAtkStateChange->extState);
         }
 
         atk_object_notify_state_change(accWrap->GetAtkObject(),
@@ -147,8 +154,9 @@ NS_IMETHODIMP nsDocAccessibleWrap::FireToolkitEvent(PRUint32 aEvent,
          * Need handle them separately.
          */
     case nsIAccessibleEvent::EVENT_ATK_PROPERTY_CHANGE :
+      {
         AtkPropertyChange *pAtkPropChange;
-        AtkPropertyValues values;
+        AtkPropertyValues values = { NULL };
 
         MAI_LOG_DEBUG(("\n\nReceived: EVENT_ATK_PROPERTY_CHANGE\n"));
         if (!aEventData)
@@ -193,6 +201,22 @@ NS_IMETHODIMP nsDocAccessibleWrap::FireToolkitEvent(PRUint32 aEvent,
                                                  pAtkPropChange->newvalue));
             rv = NS_OK;
             break;
+        case PROP_VALUE:
+            {
+              // Old value not used for anything other than state change events
+              nsCOMPtr<nsIAccessibleValue> accValue(do_QueryInterface(aAccessible, &rv));
+              if (!NS_SUCCEEDED(rv)) {
+                break;
+              }
+              double newValue;
+              rv = accValue->GetCurrentValue(&newValue);
+              if (!NS_SUCCEEDED(rv)) {
+                break;
+              }
+              g_value_init(&values.new_value, G_TYPE_DOUBLE);
+              g_value_set_double(&values.new_value, newValue);
+            }
+            break;
   
             //Perhaps need more cases in the future
         default:
@@ -202,12 +226,14 @@ NS_IMETHODIMP nsDocAccessibleWrap::FireToolkitEvent(PRUint32 aEvent,
             g_value_set_pointer (&values.new_value, pAtkPropChange->newvalue);
             rv = NS_OK;
         }
-        if (NS_SUCCEEDED(rv))
-            g_signal_emit_by_name(accWrap->GetAtkObject(),
-                                  g_strconcat("property_change::",
-                                              values.property_name),
+        if (NS_SUCCEEDED(rv)) {
+            char *signal_name = g_strconcat("property_change::",
+                                            values.property_name, NULL);
+            g_signal_emit_by_name(accWrap->GetAtkObject(), signal_name,
                                   &values, NULL);
-
+            g_free (signal_name);
+        }
+      }
         break;
 
     case nsIAccessibleEvent::EVENT_ATK_SELECTION_CHANGE:
@@ -347,6 +373,7 @@ NS_IMETHODIMP nsDocAccessibleWrap::FireToolkitEvent(PRUint32 aEvent,
 
     case nsIAccessibleEvent::EVENT_ATK_LINK_SELECTED:
         MAI_LOG_DEBUG(("\n\nReceived: EVENT_ATK_LINK_SELECTED\n"));
+        atk_focus_tracker_notify(accWrap->GetAtkObject());
         g_signal_emit_by_name(accWrap->GetAtkObject(),
                               "link_selected",
                               // Selected link index 
@@ -398,32 +425,37 @@ NS_IMETHODIMP nsDocAccessibleWrap::FireToolkitEvent(PRUint32 aEvent,
          */
     case nsIAccessibleEvent::EVENT_MENUSTART:
         MAI_LOG_DEBUG(("\n\nReceived: EVENT_MENUSTART\n"));
-        atk_focus_tracker_notify(accWrap->GetAtkObject());
-        g_signal_emit_by_name(accWrap->GetAtkObject(),
-                              "selection_changed");
         rv = NS_OK;
         break;
 
     case nsIAccessibleEvent::EVENT_MENUEND:
         MAI_LOG_DEBUG(("\n\nReceived: EVENT_MENUEND\n"));
-        g_signal_emit_by_name(accWrap->GetAtkObject(),
-                              "selection_changed");
         rv = NS_OK;
         break;
 
     case nsIAccessibleEvent::EVENT_ATK_WINDOW_ACTIVATE:
+      {
         MAI_LOG_DEBUG(("\n\nReceived: EVENT_ATK_WINDOW_ACTIVATED\n"));
-        g_signal_emit(accWrap->GetAtkObject(),
-                      g_signal_lookup ("activate", MAI_TYPE_ATK_OBJECT), 0);
+        nsDocAccessibleWrap *accDocWrap =
+          NS_STATIC_CAST(nsDocAccessibleWrap *, aAccessible);
+        accDocWrap->mActivated = PR_TRUE;
+        AtkObject *accessible = accWrap->GetAtkObject();
+        guint id = g_signal_lookup ("activate", MAI_TYPE_ATK_OBJECT);
+        g_signal_emit(accessible, id, 0);
         rv = NS_OK;
-        break;
+      } break;
 
     case nsIAccessibleEvent::EVENT_ATK_WINDOW_DEACTIVATE:
+      {
         MAI_LOG_DEBUG(("\n\nReceived: EVENT_ATK_WINDOW_DEACTIVATED\n"));
-        g_signal_emit(accWrap->GetAtkObject(),
-                      g_signal_lookup ("deactivate", MAI_TYPE_ATK_OBJECT), 0);
+        nsDocAccessibleWrap *accDocWrap =
+          NS_STATIC_CAST(nsDocAccessibleWrap *, aAccessible);
+        accDocWrap->mActivated = PR_FALSE;
+        AtkObject *accessible = accWrap->GetAtkObject();
+        guint id = g_signal_lookup ("deactivate", MAI_TYPE_ATK_OBJECT);
+        g_signal_emit(accessible, id, 0);
         rv = NS_OK;
-        break;
+      } break;
 
     default:
         // Don't transfer others
@@ -436,7 +468,7 @@ NS_IMETHODIMP nsDocAccessibleWrap::FireToolkitEvent(PRUint32 aEvent,
 
 /* static */
 AtkStateType
-TranslateAState(PRUint32 aState)
+TranslateAState(PRUint32 aState, PRUint32 aExtState)
 {
     switch (aState) {
     case nsIAccessible::STATE_SELECTED:
@@ -475,62 +507,50 @@ TranslateAState(PRUint32 aState)
     case nsIAccessible::STATE_READONLY:
         return !ATK_STATE_EDITABLE;
 #endif
+    }
 
-        // The following state is
-        // Extended state flags (for non-MSAA, for Java and Gnome/ATK support)
-        // They are only the states that are not already  mapped in MSAA
-        // See www.accessmozilla.org/article.php?sid=11 for information on the
-        // mappings between accessibility API state
-
-    case nsIAccessible::STATE_ACTIVE:
+    // The following state is
+    // Extended state flags (for non-MSAA, for Java and Gnome/ATK support)
+    switch (aExtState) {
+    case nsIAccessible::EXT_STATE_ACTIVE:
         return ATK_STATE_ACTIVE;
-    case nsIAccessible::STATE_EXPANDABLE:
+    case nsIAccessible::EXT_STATE_EXPANDABLE:
         return ATK_STATE_EXPANDABLE;
 #if 0
         // Need change definitions in nsIAccessible.idl to avoid
         // duplicate value
-    case nsIAccessible::STATE_MODAL:
+    case nsIAccessible::EXT_STATE_MODAL:
         return ATK_STATE_MODAL;
 #endif
-    case nsIAccessible::STATE_MULTI_LINE:
+    case nsIAccessible::EXT_STATE_MULTI_LINE:
         return ATK_STATE_MULTI_LINE;
-    case nsIAccessible::STATE_SENSITIVE:
+    case nsIAccessible::EXT_STATE_SENSITIVE:
         return ATK_STATE_SENSITIVE;
-    case nsIAccessible::STATE_SHOWING:
+    case nsIAccessible::EXT_STATE_SHOWING:
         return ATK_STATE_SHOWING;
-    case nsIAccessible::STATE_SINGLE_LINE:
+    case nsIAccessible::EXT_STATE_SINGLE_LINE:
         return ATK_STATE_SINGLE_LINE;
-    case nsIAccessible::STATE_TRANSIENT:
+    case nsIAccessible::EXT_STATE_TRANSIENT:
         return ATK_STATE_TRANSIENT;
-    case nsIAccessible::STATE_VERTICAL:
+    case nsIAccessible::EXT_STATE_VERTICAL:
         return ATK_STATE_VERTICAL;
-    default:
-        return ATK_STATE_INVALID;
     }
+    return ATK_STATE_INVALID;
 }
 
-NS_IMETHODIMP nsDocAccessibleWrap::Shutdown()
+NS_IMETHODIMP nsDocAccessibleWrap::FireDocLoadingEvent(PRBool aIsFinished)
 {
-    nsAccessibleEditableText::ShutdownEditor();
-    return nsDocAccessible::Shutdown();
-}
-
-NS_IMETHODIMP nsDocAccessibleWrap::GetRole(PRUint32 *_retval)
-{
-    PRBool isEditable;
-    GetIsEditable(&isEditable);
-
-    if (isEditable)
-        *_retval = ROLE_TEXT;
-    else
-        *_retval = ROLE_HTML_CONTAINER;
-
-    return NS_OK;
-}
-
-void nsDocAccessibleWrap::CheckForEditor()
-{
-    nsDocAccessible::CheckForEditor();
-    if (mEditor)
-        SetEditor(mEditor); // set editor for nsAccessibleEditableText
+  if (!mDocument || !mWeakShell)
+    return NS_OK;  // Document has been shut down
+
+  if (!aIsFinished) {
+    // Load has been verified, it will occur, about to commence
+    AtkChildrenChange childrenData;
+    childrenData.index = -1;
+    childrenData.child = 0;
+    childrenData.add = PR_FALSE;
+    FireToolkitEvent(nsIAccessibleEvent::EVENT_REORDER, this, &childrenData);
+  }
+
+  return nsDocAccessible::FireDocLoadingEvent(aIsFinished);
 }
