@@ -37,12 +37,20 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsAccessibleWrap.h"
+#include "nsAccessibilityAtoms.h"
 #include "nsIAccessibleSelectable.h"
 #include "nsIAccessibleWin32Object.h"
-#include "nsArray.h"
+#include "nsIMutableArray.h"
 #include "nsIDOMDocument.h"
+#include "nsIFrame.h"
+#include "nsIScrollableFrame.h"
+#include "nsINameSpaceManager.h"
+#include "nsINodeInfo.h"
 #include "nsIPrefService.h"
 #include "nsIServiceManager.h"
+#include "nsTextFormatter.h"
+#include "nsIView.h"
+#include "nsRoleMap.h"
 
 // for the COM IEnumVARIANT solution in get_AccSelection()
 #define _ATLBASE_IMPL
@@ -51,7 +59,7 @@ extern CComModule _Module;
 #define _ATLCOM_IMPL
 #include <atlcom.h>
 
-/* For documentation of the accessibility architecture, 
+/* For documentation of the accessibility architecture,
  * see http://lxr.mozilla.org/seamonkey/source/accessible/accessible-docs.html
  */
 
@@ -62,7 +70,7 @@ static gAccessibles = 0;
 #endif
 
 CComModule _Module;
-  
+
 EXTERN_C GUID CDECL CLSID_Accessible =
 { 0x61044601, 0xa811, 0x4e2b, { 0xbb, 0xba, 0x17, 0xbf, 0xab, 0xd3, 0x29, 0xd7 } };
 
@@ -71,9 +79,9 @@ EXTERN_C GUID CDECL CLSID_Accessible =
  */
 
 //-----------------------------------------------------
-// construction 
+// construction
 //-----------------------------------------------------
-nsAccessibleWrap::nsAccessibleWrap(nsIDOMNode* aNode, nsIWeakReference *aShell): 
+nsAccessibleWrap::nsAccessibleWrap(nsIDOMNode* aNode, nsIWeakReference *aShell):
   nsAccessible(aNode, aShell), mEnumVARIANTPosition(0)
 {
 }
@@ -106,8 +114,8 @@ STDMETHODIMP nsAccessibleWrap::QueryInterface(REFIID iid, void** ppv)
   if (IID_IUnknown == iid || IID_IDispatch == iid || IID_IAccessible == iid)
     *ppv = NS_STATIC_CAST(IAccessible*, this);
   else if (IID_IEnumVARIANT == iid && !gIsEnumVariantSupportDisabled) {
-    PRInt32 numChildren;
-    GetChildCount(&numChildren);
+    long numChildren;
+    get_accChildCount(&numChildren);
     if (numChildren > 0)  // Don't support this interface for leaf elements
       *ppv = NS_STATIC_CAST(IEnumVARIANT*, this);
   }
@@ -117,7 +125,7 @@ STDMETHODIMP nsAccessibleWrap::QueryInterface(REFIID iid, void** ppv)
 
   if (NULL == *ppv)
     return nsAccessNodeWrap::QueryInterface(iid, ppv);
-    
+
   (NS_REINTERPRET_CAST(IUnknown*, *ppv))->AddRef();
   return S_OK;
 }
@@ -133,8 +141,8 @@ STDMETHODIMP nsAccessibleWrap::AccessibleObjectFromWindow(HWND hwnd,
                                                           void **ppvObject)
 {
   // open the dll dynamically
-  if (!gmAccLib) 
-    gmAccLib =::LoadLibrary("OLEACC.DLL");  
+  if (!gmAccLib)
+    gmAccLib =::LoadLibrary("OLEACC.DLL");
 
   if (gmAccLib) {
     if (!gmAccessibleObjectFromWindow)
@@ -154,7 +162,7 @@ STDMETHODIMP nsAccessibleWrap::NotifyWinEvent(DWORD event,
 {
   if (gmNotifyWinEvent)
     return gmNotifyWinEvent(event, hwnd, idObjectType, idObject);
-    
+
   return E_FAIL;
 }
 
@@ -164,19 +172,66 @@ STDMETHODIMP nsAccessibleWrap::get_accParent( IDispatch __RPC_FAR *__RPC_FAR *pp
   if (!mWeakShell)
     return E_FAIL;  // We've been shut down
 
-  nsCOMPtr<nsIAccessible> xpParentAccessible;
+  nsIFrame *frame = GetFrame();
+  HWND hwnd = 0;
+  if (frame) {
+    nsIView *view = frame->GetViewExternal();
+    if (view) {
+      // This code is essentially our implementation of WindowFromAccessibleObject,
+      // because MSAA iterates get_accParent() until it sees an object of ROLE_WINDOW
+      // to know where the window for a given accessible is. We must expose the native
+      // window accessible that MSAA creates for us. This must be done for the document
+      // object as well as any layout that creates its own window (e.g. via overflow: scroll)
+      nsIWidget *widget = view->GetWidget();
+      if (widget) {
+        hwnd = (HWND)widget->GetNativeData(NS_NATIVE_WINDOW);
+        NS_ASSERTION(hwnd, "No window handle for window");
+        nsIView *rootView;
+        view->GetViewManager()->GetRootView(rootView);
+        if (rootView == view) {
+          // If the current object has a widget but was created by an
+          // outer object with its own outer window, then
+          // we want the native accessible for that outer window
+          hwnd = ::GetParent(hwnd);
+          NS_ASSERTION(hwnd, "No window handle for window");
+        }
+      }
+      else {
+        // If a frame is a scrollable frame, then it has one window for the client area,
+        // not an extra parent window for just the scrollbars
+        nsIScrollableFrame *scrollFrame = nsnull;
+        CallQueryInterface(frame, &scrollFrame);
+        if (scrollFrame) {
+          hwnd = (HWND)scrollFrame->GetScrolledFrame()->GetWindow()->GetNativeData(NS_NATIVE_WINDOW);
+          NS_ASSERTION(hwnd, "No window handle for window");
+        }
+      }
+    }
 
-  GetParent(getter_AddRefs(xpParentAccessible));
-  if (xpParentAccessible) {
-    *ppdispParent = NativeAccessible(xpParentAccessible);
-    return S_OK;
+    if (hwnd && SUCCEEDED(AccessibleObjectFromWindow(hwnd, OBJID_WINDOW, IID_IAccessible,
+                                              (void**)ppdispParent))) {
+      return S_OK;
+    }
   }
 
-  return E_FAIL;
+  nsCOMPtr<nsIAccessible> xpParentAccessible;
+  GetParent(getter_AddRefs(xpParentAccessible));
+  NS_ASSERTION(xpParentAccessible, "No parent accessible where we're not direct child of window");
+  if (!xpParentAccessible) {
+    return E_UNEXPECTED;
+  }
+  *ppdispParent = NativeAccessible(xpParentAccessible);
+
+  return S_OK;
 }
 
 STDMETHODIMP nsAccessibleWrap::get_accChildCount( long __RPC_FAR *pcountChildren)
 {
+  *pcountChildren = 0;
+  if (MustPrune(this)) {
+    return NS_OK;
+  }
+
   PRInt32 numChildren;
   GetChildCount(&numChildren);
   *pcountChildren = numChildren;
@@ -184,7 +239,7 @@ STDMETHODIMP nsAccessibleWrap::get_accChildCount( long __RPC_FAR *pcountChildren
   return S_OK;
 }
 
-STDMETHODIMP nsAccessibleWrap::get_accChild( 
+STDMETHODIMP nsAccessibleWrap::get_accChild(
       /* [in] */ VARIANT varChild,
       /* [retval][out] */ IDispatch __RPC_FAR *__RPC_FAR *ppdispChild)
 {
@@ -200,13 +255,15 @@ STDMETHODIMP nsAccessibleWrap::get_accChild(
   }
 
   nsCOMPtr<nsIAccessible> childAccessible;
-  GetChildAt(varChild.lVal - 1, getter_AddRefs(childAccessible));
-  *ppdispChild = NativeAccessible(childAccessible);
+  if (!MustPrune(this)) {
+    GetChildAt(varChild.lVal - 1, getter_AddRefs(childAccessible));
+    *ppdispChild = NativeAccessible(childAccessible);
+  }
 
   return (*ppdispChild)? S_OK: E_FAIL;
 }
 
-STDMETHODIMP nsAccessibleWrap::get_accName( 
+STDMETHODIMP nsAccessibleWrap::get_accName(
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ BSTR __RPC_FAR *pszName)
 {
@@ -217,16 +274,19 @@ STDMETHODIMP nsAccessibleWrap::get_accName(
     nsAutoString name;
     if (NS_FAILED(xpAccessible->GetName(name)))
       return S_FALSE;
-
-    *pszName = ::SysAllocString(name.get());
+    if (!name.IsVoid()) {
+      *pszName = ::SysAllocString(name.get());
+    }
+#ifdef DEBUG_A11Y
     NS_ASSERTION(mIsInitialized, "Access node was not initialized");
+#endif
   }
 
   return S_OK;
 }
 
 
-STDMETHODIMP nsAccessibleWrap::get_accValue( 
+STDMETHODIMP nsAccessibleWrap::get_accValue(
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ BSTR __RPC_FAR *pszValue)
 {
@@ -244,8 +304,109 @@ STDMETHODIMP nsAccessibleWrap::get_accValue(
   return S_OK;
 }
 
+NS_IMETHODIMP nsAccessibleWrap::GetDescription(nsAString& aDescription)
+{
+  // For items that are a choice in a list of choices,
+  // use MSAA description field to shoehorn positional info, it's becoming
+  // a defacto standard use for the field.
+  // Tree items override this, because they provide the current level as well
 
-STDMETHODIMP nsAccessibleWrap::get_accDescription( 
+  aDescription.Truncate();
+  PRUint32 currentRole;
+  nsresult rv = GetFinalRole(&currentRole);
+  if (NS_FAILED(rv) ||
+      (currentRole != ROLE_LISTITEM && currentRole != ROLE_MENUITEM &&
+       currentRole != ROLE_RADIOBUTTON && currentRole != ROLE_PAGETAB &&
+       currentRole != ROLE_OUTLINEITEM)) {
+    nsAutoString description;
+    nsAccessible::GetDescription(description);
+    if (!description.IsEmpty()) {
+      // Signal to screen readers that this description is speakable
+      // and is not a formatted positional information description
+      // Don't localize the "Description: " part of this string, it will be
+      // parsed out by assistive technologies.
+      aDescription = NS_LITERAL_STRING("Description: ") + description;
+    }
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIAccessible> parent;
+  GetParent(getter_AddRefs(parent));
+  if (!parent) {
+    return NS_ERROR_FAILURE;
+  }
+
+  PRInt32 indexInParent = 0, numSiblings = 0;
+
+  nsCOMPtr<nsIAccessible> sibling, nextSibling;
+  parent->GetFirstChild(getter_AddRefs(sibling));
+  NS_ENSURE_TRUE(sibling, NS_ERROR_FAILURE);
+
+  PRBool foundCurrent = PR_FALSE;
+  PRUint32 siblingRole;
+  while (sibling) {
+    sibling->GetFinalRole(&siblingRole);
+    if (siblingRole == currentRole) {
+      ++ numSiblings;
+      if (!foundCurrent) {
+        ++ indexInParent;
+        if (sibling == this) {
+          foundCurrent = PR_TRUE;
+        }
+      }
+    }
+    sibling->GetNextSibling(getter_AddRefs(nextSibling));
+    sibling = nextSibling;
+  }
+
+  // Don't localize the string "of" -- that's just the format of this string.
+  // The AT will parse the relevant numbers out and add its own localization.
+  if (currentRole == ROLE_OUTLINEITEM) {
+    PRUint32 level = 1;
+    nsCOMPtr<nsIAccessible> nextParent;
+    while (parent) {
+      parent->GetFinalRole(&currentRole);
+      if (currentRole != ROLE_GROUPING) {
+        break;
+      }
+      ++level;
+      parent->GetParent(getter_AddRefs(nextParent));
+      parent.swap(nextParent);
+    }
+
+    // Count the number of tree item children
+    PRInt32 numChildren = 0;
+    nsCOMPtr<nsIAccessible> groupSibling;
+    GetNextSibling(getter_AddRefs(groupSibling));
+    if (groupSibling) {
+      groupSibling->GetFinalRole(&currentRole);
+      if (currentRole == ROLE_GROUPING) {
+        // Accessible that groups child tree items
+        nsCOMPtr<nsIAccessible> child;
+        groupSibling->GetFirstChild(getter_AddRefs(child));
+        while (child) {
+          child->GetFinalRole(&currentRole);
+          numChildren += (currentRole == ROLE_OUTLINEITEM);
+          nsCOMPtr<nsIAccessible> nextChild;
+          child->GetNextSibling(getter_AddRefs(nextChild));
+          child.swap(nextChild);
+        }
+      }
+    }
+
+    // This must be a DHTML tree item -- XUL tree items impl GetDescription()
+    nsTextFormatter::ssprintf(aDescription,
+                              NS_LITERAL_STRING("L%d, %d of %d with %d").get(),
+                              level, indexInParent, numSiblings, numChildren);
+  }
+  else {
+    nsTextFormatter::ssprintf(aDescription, NS_LITERAL_STRING("%d of %d").get(),
+                              indexInParent, numSiblings);
+  }
+  return NS_OK;
+}
+
+STDMETHODIMP nsAccessibleWrap::get_accDescription(
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ BSTR __RPC_FAR *pszDescription)
 {
@@ -263,12 +424,11 @@ STDMETHODIMP nsAccessibleWrap::get_accDescription(
   return S_OK;
 }
 
-STDMETHODIMP nsAccessibleWrap::get_accRole( 
+STDMETHODIMP nsAccessibleWrap::get_accRole(
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ VARIANT __RPC_FAR *pvarRole)
 {
   VariantInit(pvarRole);
-  pvarRole->vt = VT_I4;
 
   nsCOMPtr<nsIAccessible> xpAccessible;
   GetXPAccessibleFor(varChild, getter_AddRefs(xpAccessible));
@@ -276,15 +436,51 @@ STDMETHODIMP nsAccessibleWrap::get_accRole(
   if (!xpAccessible)
     return E_FAIL;
 
-  PRUint32 role = 0;
-  if (NS_FAILED(xpAccessible->GetRole(&role)))
+  PRUint32 xpRole = 0, msaaRole = 0;
+  if (NS_FAILED(xpAccessible->GetFinalRole(&xpRole)))
     return E_FAIL;
+  msaaRole = msaaRoleMap[xpRole];
+  NS_ASSERTION(msaaRoleMap[nsIAccessible::ROLE_LAST_ENTRY] == ROLE_MSAA_LAST_ENTRY,
+               "MSAA role map skewed");
 
-  pvarRole->lVal = role;
-  return S_OK;
+  // -- Try enumerated role
+  if (msaaRole != USE_ROLE_STRING) {
+    pvarRole->vt = VT_I4;
+    pvarRole->lVal = msaaRole;  // Normal enumerated role
+    return S_OK;
+  }
+
+  // -- Try BSTR role
+  // Could not map to known enumerated MSAA role like ROLE_BUTTON
+  // Use BSTR role to expose role attribute or tag name + namespace
+  nsCOMPtr<nsIDOMNode> domNode;
+  nsCOMPtr<nsIAccessNode> accessNode(do_QueryInterface(xpAccessible));
+  NS_ASSERTION(accessNode, "No accessnode for accessible");
+  accessNode->GetDOMNode(getter_AddRefs(domNode));
+  nsIContent *content = GetRoleContent(domNode);
+  NS_ASSERTION(content, "No content for accessible");
+  if (content && content->IsNodeOfType(nsINode::eELEMENT)) {
+    nsAutoString roleString;
+    if (msaaRole != ROLE_SYSTEM_CLIENT && !GetRoleAttribute(content, roleString)) {
+      nsINodeInfo *nodeInfo = content->NodeInfo();
+      nodeInfo->GetName(roleString);
+      nsAutoString nameSpaceURI;
+      nodeInfo->GetNamespaceURI(nameSpaceURI);
+      if (!nameSpaceURI.IsEmpty()) {
+        // Only append name space if different from that of current document
+        roleString += NS_LITERAL_STRING(", ") + nameSpaceURI;
+      }
+    }
+    if (!roleString.IsEmpty()) {
+      pvarRole->vt = VT_BSTR;
+      pvarRole->bstrVal = ::SysAllocString(roleString.get());
+      return S_OK;
+    }
+  }
+  return E_FAIL;
 }
 
-STDMETHODIMP nsAccessibleWrap::get_accState( 
+STDMETHODIMP nsAccessibleWrap::get_accState(
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ VARIANT __RPC_FAR *pvarState)
 {
@@ -298,7 +494,7 @@ STDMETHODIMP nsAccessibleWrap::get_accState(
     return E_FAIL;
 
   PRUint32 state;
-  if (NS_FAILED(xpAccessible->GetState(&state)))
+  if (NS_FAILED(xpAccessible->GetFinalState(&state)))
     return E_FAIL;
 
   pvarState->lVal = state;
@@ -307,7 +503,7 @@ STDMETHODIMP nsAccessibleWrap::get_accState(
 }
 
 
-STDMETHODIMP nsAccessibleWrap::get_accHelp( 
+STDMETHODIMP nsAccessibleWrap::get_accHelp(
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ BSTR __RPC_FAR *pszHelp)
 {
@@ -315,7 +511,7 @@ STDMETHODIMP nsAccessibleWrap::get_accHelp(
   return S_FALSE;
 }
 
-STDMETHODIMP nsAccessibleWrap::get_accHelpTopic( 
+STDMETHODIMP nsAccessibleWrap::get_accHelpTopic(
       /* [out] */ BSTR __RPC_FAR *pszHelpFile,
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ long __RPC_FAR *pidTopic)
@@ -325,7 +521,7 @@ STDMETHODIMP nsAccessibleWrap::get_accHelpTopic(
   return E_NOTIMPL;
 }
 
-STDMETHODIMP nsAccessibleWrap::get_accKeyboardShortcut( 
+STDMETHODIMP nsAccessibleWrap::get_accKeyboardShortcut(
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ BSTR __RPC_FAR *pszKeyboardShortcut)
 {
@@ -344,16 +540,37 @@ STDMETHODIMP nsAccessibleWrap::get_accKeyboardShortcut(
   return S_FALSE;
 }
 
-STDMETHODIMP nsAccessibleWrap::get_accFocus( 
+STDMETHODIMP nsAccessibleWrap::get_accFocus(
       /* [retval][out] */ VARIANT __RPC_FAR *pvarChild)
 {
-  // Unless we manage a child's focus we return VT_EMPTY.
-  // Since documents manage the child focus in Mozilla, we 
-  // only return a focused IAccessible in 
-  // nsDocAccessibleWrap::get_accFocus)()
+  // VT_EMPTY:    None. This object does not have the keyboard focus itself
+  //              and does not contain a child that has the keyboard focus.
+  // VT_I4:       lVal is CHILDID_SELF. The object itself has the keyboard focus.
+  // VT_I4:       lVal contains the child ID of the child element with the keyboard focus.
+  // VT_DISPATCH: pdispVal member is the address of the IDispatch interface
+  //              for the child object with the keyboard focus.
+
+  if (!mDOMNode) {
+    return E_FAIL; // This node is shut down
+  }
 
   VariantInit(pvarChild);
-  pvarChild->vt = VT_EMPTY;
+
+  // Return the current IAccessible child that has focus
+  nsCOMPtr<nsIAccessible> focusedAccessible;
+  GetFocusedChild(getter_AddRefs(focusedAccessible));
+  if (focusedAccessible == this) {
+    pvarChild->vt = VT_I4;
+    pvarChild->lVal = CHILDID_SELF;
+  }
+  else if (focusedAccessible) {
+    pvarChild->vt = VT_DISPATCH;
+    pvarChild->pdispVal = NativeAccessible(focusedAccessible);
+  }
+  else {
+    pvarChild->vt = VT_EMPTY;   // No focus or focus is not a child
+  }
+
   return S_OK;
 }
 
@@ -367,12 +584,12 @@ STDMETHODIMP nsAccessibleWrap::get_accFocus(
   *  of options selected, unless there are none selected in which case we return an empty
   *  VARIANT.
   *
-  * The typedefs at the beginning set up the structure that will contain an array 
-  *  of the IAccessibles. It implements the IEnumVARIANT interface, allowing us to 
+  * The typedefs at the beginning set up the structure that will contain an array
+  *  of the IAccessibles. It implements the IEnumVARIANT interface, allowing us to
   *  use it to return the IAccessibles in the VARIANT.
   *
-  * We get the selected options from the select's accessible object and then put create 
-  *  IAccessible objects for them and put those in the CComObject<EnumeratorType> 
+  * We get the selected options from the select's accessible object and then put create
+  *  IAccessible objects for them and put those in the CComObject<EnumeratorType>
   *  object. Then we put the CComObject<EnumeratorType> object in the VARIANT and return.
   *
   * returns a VT_EMPTY VARIANT if:
@@ -413,7 +630,7 @@ STDMETHODIMP nsAccessibleWrap::get_accSelection(VARIANT __RPC_FAR *pvarChildren)
       // 1) Populate an array to store in the enumeration
       for (PRUint32 i = 0 ; i < length ; i++) {
         nsCOMPtr<nsIAccessible> tempAccess;
-        selectedOptions->QueryElementAt(i, NS_GET_IID(nsIAccessible), 
+        selectedOptions->QueryElementAt(i, NS_GET_IID(nsIAccessible),
                                         getter_AddRefs(tempAccess));
         if (tempAccess) {
           optionArray[i] = NativeAccessible(tempAccess);
@@ -436,7 +653,7 @@ STDMETHODIMP nsAccessibleWrap::get_accSelection(VARIANT __RPC_FAR *pvarChildren)
   return S_OK;
 }
 
-STDMETHODIMP nsAccessibleWrap::get_accDefaultAction( 
+STDMETHODIMP nsAccessibleWrap::get_accDefaultAction(
       /* [optional][in] */ VARIANT varChild,
       /* [retval][out] */ BSTR __RPC_FAR *pszDefaultAction)
 {
@@ -454,7 +671,7 @@ STDMETHODIMP nsAccessibleWrap::get_accDefaultAction(
   return S_OK;
 }
 
-STDMETHODIMP nsAccessibleWrap::accSelect( 
+STDMETHODIMP nsAccessibleWrap::accSelect(
       /* [in] */ long flagsSelect,
       /* [optional][in] */ VARIANT varChild)
 {
@@ -470,8 +687,14 @@ STDMETHODIMP nsAccessibleWrap::accSelect(
     if (flagsSelect & SELFLAG_TAKESELECTION)
       xpAccessible->TakeSelection();
 
+    if (flagsSelect & SELFLAG_ADDSELECTION)
+      xpAccessible->SetSelected(PR_TRUE);
+
     if (flagsSelect & SELFLAG_REMOVESELECTION)
-      xpAccessible->RemoveSelection();
+      xpAccessible->SetSelected(PR_FALSE);
+
+    if (flagsSelect & SELFLAG_EXTENDSELECTION)
+      xpAccessible->ExtendSelection();
 
     return S_OK;
   }
@@ -479,7 +702,7 @@ STDMETHODIMP nsAccessibleWrap::accSelect(
   return E_FAIL;
 }
 
-STDMETHODIMP nsAccessibleWrap::accLocation( 
+STDMETHODIMP nsAccessibleWrap::accLocation(
       /* [out] */ long __RPC_FAR *pxLeft,
       /* [out] */ long __RPC_FAR *pyTop,
       /* [out] */ long __RPC_FAR *pcxWidth,
@@ -501,10 +724,10 @@ STDMETHODIMP nsAccessibleWrap::accLocation(
     return S_OK;
   }
 
-  return E_FAIL;  
+  return E_FAIL;
 }
 
-STDMETHODIMP nsAccessibleWrap::accNavigate( 
+STDMETHODIMP nsAccessibleWrap::accNavigate(
       /* [in] */ long navDir,
       /* [optional][in] */ VARIANT varStart,
       /* [retval][out] */ VARIANT __RPC_FAR *pvarEndUpAt)
@@ -515,16 +738,21 @@ STDMETHODIMP nsAccessibleWrap::accNavigate(
     return E_FAIL;
 
   VariantInit(pvarEndUpAt);
+  PRUint32 xpRelation = 0;
 
   switch(navDir) {
-    case NAVDIR_DOWN: 
+    case NAVDIR_DOWN:
       xpAccessibleStart->GetAccessibleBelow(getter_AddRefs(xpAccessibleResult));
       break;
     case NAVDIR_FIRSTCHILD:
-      xpAccessibleStart->GetFirstChild(getter_AddRefs(xpAccessibleResult));
+      if (!MustPrune(xpAccessibleStart)) {
+        xpAccessibleStart->GetFirstChild(getter_AddRefs(xpAccessibleResult));
+      }
       break;
     case NAVDIR_LASTCHILD:
-      xpAccessibleStart->GetLastChild(getter_AddRefs(xpAccessibleResult));
+      if (!MustPrune(xpAccessibleStart)) {
+        xpAccessibleStart->GetLastChild(getter_AddRefs(xpAccessibleResult));
+      }
       break;
     case NAVDIR_LEFT:
       xpAccessibleStart->GetAccessibleToLeft(getter_AddRefs(xpAccessibleResult));
@@ -541,18 +769,44 @@ STDMETHODIMP nsAccessibleWrap::accNavigate(
     case NAVDIR_UP:
       xpAccessibleStart->GetAccessibleAbove(getter_AddRefs(xpAccessibleResult));
       break;
+    // MSAA relationship extensions to accNavigate
+    case NAVRELATION_CONTROLLED_BY:    xpRelation = RELATION_CONTROLLED_BY;    break;
+    case NAVRELATION_CONTROLLER_FOR:   xpRelation = RELATION_CONTROLLER_FOR;   break;
+    case NAVRELATION_LABEL_FOR:        xpRelation = RELATION_LABEL_FOR;        break;
+    case NAVRELATION_LABELLED_BY:      xpRelation = RELATION_LABELLED_BY;      break;
+    case NAVRELATION_MEMBER_OF:        xpRelation = RELATION_MEMBER_OF;        break;
+    case NAVRELATION_NODE_CHILD_OF:    xpRelation = RELATION_NODE_CHILD_OF;    break;
+    case NAVRELATION_FLOWS_TO:         xpRelation = RELATION_FLOWS_TO;         break;
+    case NAVRELATION_FLOWS_FROM:       xpRelation = RELATION_FLOWS_FROM;       break;
+    case NAVRELATION_SUBWINDOW_OF:     xpRelation = RELATION_SUBWINDOW_OF;     break;
+    case NAVRELATION_EMBEDS:           xpRelation = RELATION_EMBEDS;           break;
+    case NAVRELATION_EMBEDDED_BY:      xpRelation = RELATION_EMBEDDED_BY;      break;
+    case NAVRELATION_POPUP_FOR:        xpRelation = RELATION_POPUP_FOR;        break;
+    case NAVRELATION_PARENT_WINDOW_OF: xpRelation = RELATION_PARENT_WINDOW_OF; break;
+    case NAVRELATION_DEFAULT_BUTTON:   xpRelation = RELATION_DEFAULT_BUTTON;   break;
+    case NAVRELATION_DESCRIBED_BY:     xpRelation = RELATION_DESCRIBED_BY;     break;
+    case NAVRELATION_DESCRIPTION_FOR:  xpRelation = RELATION_DESCRIPTION_FOR;  break;
+  }
+
+  pvarEndUpAt->vt = VT_EMPTY;
+
+  if (xpRelation) {
+    nsresult rv = GetAccessibleRelated(xpRelation,
+                                       getter_AddRefs(xpAccessibleResult));
+    if (rv == NS_ERROR_NOT_IMPLEMENTED) {
+      return E_NOTIMPL;
+    }
   }
 
   if (xpAccessibleResult) {
     pvarEndUpAt->pdispVal = NativeAccessible(xpAccessibleResult);
     pvarEndUpAt->vt = VT_DISPATCH;
     return NS_OK;
-  } 
-  pvarEndUpAt->vt = VT_EMPTY;
+  }
   return E_FAIL;
 }
 
-STDMETHODIMP nsAccessibleWrap::accHitTest( 
+STDMETHODIMP nsAccessibleWrap::accHitTest(
       /* [in] */ long xLeft,
       /* [in] */ long yTop,
       /* [retval][out] */ VARIANT __RPC_FAR *pvarChild)
@@ -565,7 +819,12 @@ STDMETHODIMP nsAccessibleWrap::accHitTest(
   xLeft = xLeft;
   yTop = yTop;
 
-  GetChildAtPoint(xLeft, yTop, getter_AddRefs(xpAccessible));
+  if (MustPrune(this)) {
+    xpAccessible = this;
+  }
+  else {
+    GetChildAtPoint(xLeft, yTop, getter_AddRefs(xpAccessible));
+  }
 
   // if we got a child
   if (xpAccessible) {
@@ -595,7 +854,7 @@ STDMETHODIMP nsAccessibleWrap::accHitTest(
   return S_OK;
 }
 
-STDMETHODIMP nsAccessibleWrap::accDoDefaultAction( 
+STDMETHODIMP nsAccessibleWrap::accDoDefaultAction(
       /* [optional][in] */ VARIANT varChild)
 {
   nsCOMPtr<nsIAccessible> xpAccessible;
@@ -607,14 +866,14 @@ STDMETHODIMP nsAccessibleWrap::accDoDefaultAction(
   return S_OK;
 }
 
-STDMETHODIMP nsAccessibleWrap::put_accName( 
+STDMETHODIMP nsAccessibleWrap::put_accName(
       /* [optional][in] */ VARIANT varChild,
       /* [in] */ BSTR szName)
 {
   return E_NOTIMPL;
 }
 
-STDMETHODIMP nsAccessibleWrap::put_accValue( 
+STDMETHODIMP nsAccessibleWrap::put_accValue(
       /* [optional][in] */ VARIANT varChild,
       /* [in] */ BSTR szValue)
 {
@@ -671,8 +930,10 @@ nsAccessibleWrap::Next(ULONG aNumElementsRequested, VARIANT FAR* pvar, ULONG FAR
 
   for (long childIndex = 0; pvar[*aNumElementsFetched].vt == VT_DISPATCH; ++childIndex) {
     PRBool wasAccessibleFetched = PR_FALSE;
-    nsAccessibleWrap *msaaAccessible = 
+    nsAccessibleWrap *msaaAccessible =
       NS_STATIC_CAST(nsAccessibleWrap*, pvar[*aNumElementsFetched].pdispVal);
+    if (!msaaAccessible)
+      break;
     if (childIndex >= mEnumVARIANTPosition) {
       if (++*aNumElementsFetched >= aNumElementsRequested)
         break;
@@ -694,7 +955,7 @@ nsAccessibleWrap::Skip(ULONG aNumElements)
 
   PRInt32 numChildren;
   GetChildCount(&numChildren);
-  
+
   if (mEnumVARIANTPosition > numChildren)
   {
     mEnumVARIANTPosition = numChildren;
@@ -703,7 +964,7 @@ nsAccessibleWrap::Skip(ULONG aNumElements)
   return NOERROR;
 }
 
-STDMETHODIMP 
+STDMETHODIMP
 nsAccessibleWrap::Reset(void)
 {
   mEnumVARIANTPosition = 0;
@@ -730,10 +991,10 @@ nsAccessibleWrap::Clone(IEnumVARIANT FAR* FAR* ppenum)
 
   return NOERROR;
 }
-        
+
 
 // For IDispatch support
-STDMETHODIMP 
+STDMETHODIMP
 nsAccessibleWrap::GetTypeInfoCount(UINT *p)
 {
   *p = 0;
@@ -748,7 +1009,7 @@ STDMETHODIMP nsAccessibleWrap::GetTypeInfo(UINT i, LCID lcid, ITypeInfo **ppti)
 }
 
 // For IDispatch support
-STDMETHODIMP 
+STDMETHODIMP
 nsAccessibleWrap::GetIDsOfNames(REFIID riid, LPOLESTR *rgszNames,
                            UINT cNames, LCID lcid, DISPID *rgDispId)
 {
@@ -767,11 +1028,11 @@ STDMETHODIMP nsAccessibleWrap::Invoke(DISPID dispIdMember, REFIID riid,
 NS_IMETHODIMP nsAccessibleWrap::GetNativeInterface(void **aOutAccessible)
 {
   *aOutAccessible = NS_STATIC_CAST(IAccessible*, this);
-  NS_STATIC_CAST(IAccessible*, this)->AddRef();
+  NS_ADDREF_THIS();
   return NS_OK;
 }
 
-        
+
 //------- Helper methods ---------
 
 IDispatch *nsAccessibleWrap::NativeAccessible(nsIAccessible *aXPAccessible)
@@ -782,8 +1043,8 @@ IDispatch *nsAccessibleWrap::NativeAccessible(nsIAccessible *aXPAccessible)
     accObject->GetHwnd(&hwnd);
     if (hwnd) {
       IDispatch *retval = nsnull;
-      AccessibleObjectFromWindow(NS_REINTERPRET_CAST(HWND, hwnd), 
-        OBJID_CLIENT, IID_IAccessible, (void **) &retval);
+      AccessibleObjectFromWindow(NS_REINTERPRET_CAST(HWND, hwnd),
+        OBJID_WINDOW, IID_IAccessible, (void **) &retval);
       return retval;
     }
   }
@@ -804,7 +1065,10 @@ void nsAccessibleWrap::GetXPAccessibleFor(const VARIANT& aVarChild, nsIAccessibl
   // if its us real easy - this seems to always be the case
   if (aVarChild.lVal == CHILDID_SELF) {
     *aXPAccessible = NS_STATIC_CAST(nsIAccessible*, this);
-  } 
+  }
+  else if (MustPrune(this)) {
+    return;
+  }
   else {
     // XXX It is rare to use a VARIANT with a child num
     // so optimizing this is not a priority
