@@ -38,7 +38,8 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsXULSelectAccessible.h"
-#include "nsArray.h"
+#include "nsAccessibilityService.h"
+#include "nsIMutableArray.h"
 #include "nsIContent.h"
 #include "nsIDOMXULMenuListElement.h"
 #include "nsIDOMXULMultSelectCntrlEl.h"
@@ -47,11 +48,14 @@
 #include "nsIDOMXULTextboxElement.h"
 #include "nsIPresShell.h"
 #include "nsIServiceManager.h"
+#include "nsCaseTreatment.h"
 
 /**
   * Selects, Listboxes and Comboboxes, are made up of a number of different
   *  widgets, some of which are shared between the two. This file contains
   *  all of the widgets for both of the Selects, for XUL only.
+  *  (except nsXULRadioGroupAccessible which inherits
+  *   nsXULSelectableAccessible so that it supports nsIAccessibleSelectable)
   *
   *  Listbox:
   *     - nsXULListboxAccessible
@@ -77,47 +81,44 @@ nsAccessibleWrap(aDOMNode, aShell)
 {
 }
 
-NS_IMETHODIMP nsXULSelectableAccessible::GetName(nsAString& _retval)
+NS_IMETHODIMP nsXULSelectableAccessible::GetName(nsAString& aName)
 {
-  return GetXULName(_retval);
+  return GetXULName(aName, PR_FALSE);
 }
 
 NS_IMPL_ISUPPORTS_INHERITED1(nsXULSelectableAccessible, nsAccessible, nsIAccessibleSelectable)
 
 NS_IMETHODIMP nsXULSelectableAccessible::ChangeSelection(PRInt32 aIndex, PRUint8 aMethod, PRBool *aSelState)
 {
+  nsCOMPtr<nsIAccessible> childAcc;
+  GetChildAt(aIndex, getter_AddRefs(childAcc));
+  nsCOMPtr<nsIAccessNode> accNode = do_QueryInterface(childAcc);
+  NS_ENSURE_TRUE(accNode, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDOMNode> childNode;
+  accNode->GetDOMNode(getter_AddRefs(childNode));
+  nsCOMPtr<nsIDOMXULSelectControlItemElement> item(do_QueryInterface(childNode));
+  NS_ENSURE_TRUE(item, NS_ERROR_FAILURE);
+
   *aSelState = PR_FALSE;
 
-  nsCOMPtr<nsIDOMXULMultiSelectControlElement> xulMultiSelect(do_QueryInterface(mDOMNode));
-  if (xulMultiSelect) {
-    nsCOMPtr<nsIDOMNodeList> nodeList;
-    xulMultiSelect->GetChildNodes(getter_AddRefs(nodeList));
-    if (nodeList) {
-      nsCOMPtr<nsIDOMNode> node;
-      nodeList->Item(aIndex, getter_AddRefs(node));
-      nsCOMPtr<nsIDOMXULSelectControlItemElement> item(do_QueryInterface(node));
-      item->GetSelected(aSelState);
-      if (eSelection_Add == aMethod && !(*aSelState))
-        xulMultiSelect->AddItemToSelection(item);
-      else if (eSelection_Remove == aMethod && (*aSelState))
-        xulMultiSelect->RemoveItemFromSelection(item);
-    }
-    return NS_OK;
-  }
-
   nsCOMPtr<nsIDOMXULSelectControlElement> xulSelect(do_QueryInterface(mDOMNode));
-  if (xulSelect) {
-    nsresult rv = NS_OK;
-    PRInt32 selIndex;
-    xulSelect->GetSelectedIndex(&selIndex);
-    if (selIndex == aIndex)
-      *aSelState = PR_TRUE;
-    if (eSelection_Add == aMethod && !(*aSelState))
-      rv = xulSelect->SetSelectedIndex(aIndex);
-    else if (eSelection_Remove == aMethod && (*aSelState)) {
-      rv = xulSelect->SetSelectedIndex(-1);
-    }
-    return rv;
+  nsCOMPtr<nsIDOMXULMultiSelectControlElement> xulMultiSelect(do_QueryInterface(mDOMNode));
+
+  if (xulSelect || xulMultiSelect) {
+      item->GetSelected(aSelState);
+      if (eSelection_Add == aMethod && !(*aSelState)) {
+        if (xulMultiSelect)
+          return xulMultiSelect->AddItemToSelection(item);
+        else if (xulSelect)
+          return xulSelect->SetSelectedItem(item);
+      }
+      else if (eSelection_Remove == aMethod && (*aSelState)) {
+        if (xulMultiSelect)
+          return xulMultiSelect->RemoveItemFromSelection(item);
+        else if (xulSelect)
+          return xulSelect->SetSelectedIndex(-1);
+      }
   }
 
   return NS_ERROR_FAILURE;
@@ -132,10 +133,9 @@ NS_IMETHODIMP nsXULSelectableAccessible::GetSelectedChildren(nsIArray **_retval)
   if (!accService)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIMutableArray> selectedAccessibles;
-  NS_NewArray(getter_AddRefs(selectedAccessibles));
-  if (!selectedAccessibles)
-    return NS_ERROR_OUT_OF_MEMORY;
+  nsCOMPtr<nsIMutableArray> selectedAccessibles =
+    do_CreateInstance(NS_ARRAY_CONTRACTID);
+  NS_ENSURE_STATE(selectedAccessibles);
 
   // For XUL multi-select control
   nsCOMPtr<nsIDOMXULMultiSelectControlElement> xulMultiSelect(do_QueryInterface(mDOMNode));
@@ -185,8 +185,8 @@ NS_IMETHODIMP nsXULSelectableAccessible::RefSelection(PRInt32 aIndex, nsIAccessi
     nsCOMPtr<nsIAccessible> tempAccess;
     accService->GetAccessibleInWeakShell(tempDOMNode, mWeakShell, getter_AddRefs(tempAccess));
     *_retval = tempAccess;
-    NS_ADDREF(*_retval);
-    return NS_OK;
+    NS_IF_ADDREF(*_retval);
+    return *_retval ? NS_OK : NS_ERROR_FAILURE;
   }
 
   return NS_ERROR_FAILURE;
@@ -344,9 +344,7 @@ nsIFrame* nsXULSelectOptionAccessible::GetBoundsFrame()
         if (!presShell) {
           return nsnull;
         }
-        nsIFrame *menuListFrame = nsnull;
-        presShell->GetPrimaryFrameFor(menuListContent, &menuListFrame);
-        return menuListFrame;
+        return presShell->GetPrimaryFrameFor(menuListContent);
       }
       break;
     }
@@ -368,26 +366,16 @@ nsXULSelectableAccessible(aDOMNode, aShell)
 {
 }
 
-/**
-  * Let Accessible count them up
-  */
-NS_IMETHODIMP nsXULListboxAccessible::GetChildCount(PRInt32 *_retval)
-{
-  return nsAccessible::GetChildCount(_retval);
-}
-
 /**
   * As a nsXULListboxAccessible we can have the following states:
   *     STATE_FOCUSED
   *     STATE_READONLY
   *     STATE_FOCUSABLE
   */
-NS_IMETHODIMP nsXULListboxAccessible::GetState(PRUint32 *_retval)
+NS_IMETHODIMP nsXULListboxAccessible::GetState(PRUint32 *aState)
 {
   // Get focus status from base class
-  nsAccessible::GetState(_retval);
-
-  *_retval |= STATE_FOCUSABLE;
+  nsAccessible::GetState(aState);
 
 // see if we are multiple select if so set ourselves as such
   nsCOMPtr<nsIDOMElement> element (do_QueryInterface(mDOMNode));
@@ -395,11 +383,9 @@ NS_IMETHODIMP nsXULListboxAccessible::GetState(PRUint32 *_retval)
     nsAutoString selType;
     element->GetAttribute(NS_LITERAL_STRING("seltype"), selType);
     if (!selType.IsEmpty() && selType.EqualsLiteral("multiple"))
-        *_retval |= STATE_MULTISELECTABLE;
+      *aState |= STATE_MULTISELECTABLE | STATE_EXTSELECTABLE;
   }
 
-  *_retval |= STATE_FOCUSABLE ;
-
   return NS_OK;
 }
 
@@ -532,9 +518,15 @@ nsXULSelectableAccessible(aDOMNode, aShell)
 }
 
 /** We are a combobox */
-NS_IMETHODIMP nsXULComboboxAccessible::GetRole(PRUint32 *_retval)
+NS_IMETHODIMP nsXULComboboxAccessible::GetRole(PRUint32 *aRole)
 {
-  *_retval = ROLE_COMBOBOX;
+  nsCOMPtr<nsIContent> content = do_QueryInterface(mDOMNode);
+  if (!content) {
+    return NS_ERROR_FAILURE;
+  }
+  *aRole = content->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::type,
+                                NS_LITERAL_STRING("autocomplete"), eIgnoreCase) ?
+                                ROLE_AUTOCOMPLETE : ROLE_COMBOBOX;
   return NS_OK;
 }
 
@@ -585,3 +577,86 @@ NS_IMETHODIMP nsXULComboboxAccessible::GetValue(nsAString& _retval)
   }
   return NS_ERROR_FAILURE;
 }
+
+NS_IMETHODIMP nsXULComboboxAccessible::GetDescription(nsAString& aDescription)
+{
+  // Use description of currently focused option
+  aDescription.Truncate();
+  nsCOMPtr<nsIDOMXULMenuListElement> menuList(do_QueryInterface(mDOMNode));
+  if (!menuList) {
+    return NS_ERROR_FAILURE;  // Shut down
+  }
+  nsCOMPtr<nsIDOMXULSelectControlItemElement> focusedOption;
+  menuList->GetSelectedItem(getter_AddRefs(focusedOption));
+  nsCOMPtr<nsIDOMNode> focusedOptionNode(do_QueryInterface(focusedOption));
+  if (focusedOptionNode) {
+    nsCOMPtr<nsIAccessibilityService> accService = 
+      do_GetService("@mozilla.org/accessibilityService;1");
+    NS_ENSURE_TRUE(accService, NS_ERROR_FAILURE);
+    nsCOMPtr<nsIAccessible> focusedOptionAccessible;
+    accService->GetAccessibleInWeakShell(focusedOptionNode, mWeakShell, 
+                                        getter_AddRefs(focusedOptionAccessible));
+    NS_ENSURE_TRUE(focusedOptionAccessible, NS_ERROR_FAILURE);
+    return focusedOptionAccessible->GetDescription(aDescription);
+  }
+  return NS_OK;
+}
+
+void nsXULComboboxAccessible::CacheChildren()
+{
+  if (!mWeakShell) {
+    // This node has been shut down
+    mAccChildCount = eChildCountUninitialized;
+    return;
+  }
+
+  if (mAccChildCount != eChildCountUninitialized) {
+    return;
+  }
+
+  // Set menugenerated="true" on the menupopup node to generate the
+  // sub-menu items if they have not been generated
+  PRUint32 childIndex, numChildren = 0;
+  nsCOMPtr<nsIDOMNode> childNode;
+  nsCOMPtr<nsIDOMNodeList> nodeList;
+  mDOMNode->GetChildNodes(getter_AddRefs(nodeList));
+  if (nodeList && NS_OK == nodeList->GetLength(&numChildren)) {
+    for (childIndex = 0; childIndex < numChildren; childIndex++) {
+      nodeList->Item(childIndex, getter_AddRefs(childNode));
+      nsCOMPtr<nsIContent> content = do_QueryInterface(childNode);
+      if (content->NodeInfo()->Equals(nsAccessibilityAtoms::menupopup, kNameSpaceID_XUL)) {
+        break;
+      }
+    }
+
+    if (childIndex < numChildren) {
+      nsCOMPtr<nsIDOMElement> element(do_QueryInterface(childNode));
+      if (element) {
+        nsAutoString attr;
+        element->GetAttribute(NS_LITERAL_STRING("menugenerated"), attr);
+        if (!attr.Equals(NS_LITERAL_STRING("true"))) {
+          element->SetAttribute(NS_LITERAL_STRING("menugenerated"), NS_LITERAL_STRING("true"));
+        }
+      }
+    }
+  }
+  nsXULSelectableAccessible::CacheChildren();
+}
+
+NS_IMETHODIMP
+nsXULComboboxAccessible::GetAllowsAnonChildAccessibles(PRBool *aAllowsAnonChildren)
+{
+  nsCOMPtr<nsIContent> content = do_QueryInterface(mDOMNode);
+
+  if (content->NodeInfo()->Equals(nsAccessibilityAtoms::textbox, kNameSpaceID_XUL)) {
+    // autocomplete textbox also uses nsXULComboboxAccessible and we need walk
+    // anonymous children
+    *aAllowsAnonChildren = PR_TRUE;
+  } else {
+    // Argument of PR_FALSE indicates we don't walk anonymous children for
+    // menuitems
+    *aAllowsAnonChildren = PR_FALSE;
+  }
+  return NS_OK;
+}
+
