@@ -1,12 +1,12 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* vim:set ts=4 sw=4 sts=4 et cin: */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -15,7 +15,7 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -23,16 +23,16 @@
  * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -43,15 +43,14 @@
 */
 
 #include "nsChromeProtocolHandler.h"
+#include "nsChromeRegistry.h"
 #include "nsCOMPtr.h"
 #include "nsContentCID.h"
 #include "nsCRT.h"
-#include "nsEventQueueUtils.h"
+#include "nsThreadUtils.h"
 #include "nsIChannel.h"
 #include "nsIChromeRegistry.h"
 #include "nsIComponentManager.h"
-#include "nsIEventQueue.h"
-#include "nsIEventQueueService.h"
 #include "nsIFastLoadService.h"
 #include "nsIFile.h"
 #include "nsIFileURL.h"
@@ -65,23 +64,19 @@
 #include "nsIServiceManager.h"
 #include "nsIStandardURL.h"
 #include "nsIStreamListener.h"
-#ifdef MOZ_XUL
-#include "nsIXULPrototypeCache.h"
-#include "nsIXULPrototypeDocument.h"
-#endif
 #include "nsNetUtil.h"
 #include "nsXPIDLString.h"
 #include "nsString.h"
 #include "prlog.h"
 
-//----------------------------------------------------------------------
-
 #ifdef MOZ_XUL
-static NS_DEFINE_CID(kXULPrototypeCacheCID,      NS_XULPROTOTYPECACHE_CID);
+#include "nsIXULPrototypeCache.h"
+#include "nsIXULPrototypeDocument.h"
 #endif
 
-// This comes from nsChromeRegistry.cpp
-extern nsIChromeRegistry* gChromeRegistry;
+//----------------------------------------------------------------------
+
+static NS_DEFINE_CID(kXULPrototypeCacheCID,      NS_XULPROTOTYPECACHE_CID);
 
 //----------------------------------------------------------------------
 //
@@ -112,6 +107,7 @@ protected:
     ~nsCachedChromeChannel();
 
     nsCOMPtr<nsIURI>            mURI;
+    nsCOMPtr<nsIURI>            mOriginalURI;
     nsCOMPtr<nsILoadGroup>      mLoadGroup;
     nsCOMPtr<nsIStreamListener> mListener;
     nsCOMPtr<nsISupports>       mContext;
@@ -119,16 +115,12 @@ protected:
     nsCOMPtr<nsISupports>       mOwner;
     nsresult                    mStatus;
 
-    static NS_HIDDEN_(nsresult)
-    PostLoadEvent(nsCachedChromeChannel* aChannel, PLHandleEventProc aHandler);
-
-    static void* PR_CALLBACK HandleLoadEvent(PLEvent* aEvent);
-    static void PR_CALLBACK DestroyLoadEvent(PLEvent* aEvent);
-
 #ifdef PR_LOGGING
     static PRLogModuleInfo* gLog;
 #endif
 
+    void HandleLoadEvent();
+
 public:
     nsCachedChromeChannel(nsIURI* aURI);
 
@@ -146,22 +138,18 @@ public:
     NS_IMETHOD GetLoadFlags(nsLoadFlags *);
     NS_IMETHOD SetLoadFlags(nsLoadFlags);
 
-// nsIChannel
+    // nsIChannel
     NS_DECL_NSICHANNEL
-
 };
 
 #ifdef PR_LOGGING
 PRLogModuleInfo* nsCachedChromeChannel::gLog;
 #endif
 
-NS_IMPL_ISUPPORTS2(nsCachedChromeChannel,
-                   nsIChannel,
-                   nsIRequest)
+NS_IMPL_ISUPPORTS2(nsCachedChromeChannel, nsIChannel, nsIRequest)
 
 nsCachedChromeChannel::nsCachedChromeChannel(nsIURI* aURI)
     : mURI(aURI)
-    , mLoadGroup(nsnull)
     , mLoadFlags(nsIRequest::LOAD_NORMAL)
     , mStatus(NS_OK)
 {
@@ -183,7 +171,7 @@ nsCachedChromeChannel::~nsCachedChromeChannel()
 NS_IMETHODIMP
 nsCachedChromeChannel::GetOriginalURI(nsIURI* *aOriginalURI)
 {
-    *aOriginalURI = mURI;
+    *aOriginalURI = mOriginalURI ? mOriginalURI : mURI;
     NS_ADDREF(*aOriginalURI);
     return NS_OK;
 }
@@ -191,10 +179,7 @@ nsCachedChromeChannel::GetOriginalURI(nsIURI* *aOriginalURI)
 NS_IMETHODIMP
 nsCachedChromeChannel::SetOriginalURI(nsIURI* aOriginalURI)
 {
-    // don't stp on a uri if we already have one there...this is a work around fix
-    // for Bug #34769.
-    if (!mURI)
-        mURI = aOriginalURI;
+    mOriginalURI = aOriginalURI;
     return NS_OK;
 }
 
@@ -226,10 +211,13 @@ nsCachedChromeChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctxt)
     LOG(("nsCachedChromeChannel[%p]: posting load event for %p",
         this, listener));
 
+    nsCOMPtr<nsIRunnable> event =
+        NS_NEW_RUNNABLE_METHOD(nsCachedChromeChannel, this, HandleLoadEvent);
+
     // Queue an event to ourselves to let the stack unwind before
     // calling OnStartRequest(). This allows embedding to occur
     // before we fire OnStopRequest().
-    rv = PostLoadEvent(this, HandleLoadEvent);
+    rv = NS_DispatchToCurrentThread(event);
     if (NS_FAILED(rv))
         return rv;
 
@@ -353,78 +341,42 @@ nsCachedChromeChannel::SetContentLength(PRInt32 aContentLength)
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-nsresult
-nsCachedChromeChannel::PostLoadEvent(nsCachedChromeChannel* aChannel,
-                                     PLHandleEventProc aHandler)
-{
-    nsCOMPtr<nsIEventQueue> queue;
-    nsresult rv = NS_GetCurrentEventQ(getter_AddRefs(queue));
-    if (NS_FAILED(rv))
-        return rv;
-
-    PLEvent* event = new PLEvent;
-    if (! event)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    PL_InitEvent(event,
-                 aChannel,
-                 aHandler,
-                 DestroyLoadEvent);
-    NS_ADDREF(aChannel);
-
-    rv = queue->PostEvent(event);
-    if (NS_FAILED(rv))
-        PL_DestroyEvent(event);
-    return rv;
-}
-
-void* PR_CALLBACK
-nsCachedChromeChannel::HandleLoadEvent(PLEvent* aEvent)
+void
+nsCachedChromeChannel::HandleLoadEvent()
 {
     // Fire the OnStartRequest() for the cached chrome channel, then
-    // queue another event to trigger the OnStopRequest()...
-    nsCachedChromeChannel* channel = (nsCachedChromeChannel*) aEvent->owner;
+    // trigger the OnStopRequest()...
 
     // If the load has been cancelled, then just bail now. We won't
     // send On[Start|Stop]Request().
-    if (NS_FAILED(channel->mStatus))
-        return nsnull;
+    if (NS_FAILED(mStatus))
+        return;
 
     LOG(("nsCachedChromeChannel[%p]: firing OnStartRequest for %p",
-        channel, channel->mListener.get()));
+        this, mListener.get()));
 
-    (void) channel->mListener->OnStartRequest(channel, channel->mContext);
+    mListener->OnStartRequest(this, mContext);
 
     LOG(("nsCachedChromeChannel[%p]: firing OnStopRequest for %p",
-        channel, channel->mListener.get()));
+        this, mListener.get()));
 
-    (void) channel->mListener->OnStopRequest(channel, channel->mContext,
-                                             channel->mStatus);
+    mListener->OnStopRequest(this, mContext, mStatus);
 
-    if (channel->mLoadGroup) {
+    if (mLoadGroup) {
         LOG(("nsCachedChromeChannel[%p]: removing self from load group %p",
-            channel, channel->mLoadGroup.get()));
-
-        (void) channel->mLoadGroup->RemoveRequest(channel, nsnull, channel->mStatus);
+            this, mLoadGroup.get()));
+        mLoadGroup->RemoveRequest(this, nsnull, mStatus);
     }
 
-    channel->mListener = nsnull;
-    channel->mContext  = nsnull;
-
-    return nsnull;
-}
-
-void PR_CALLBACK
-nsCachedChromeChannel::DestroyLoadEvent(PLEvent* aEvent)
-{
-    nsCachedChromeChannel* channel = (nsCachedChromeChannel*) aEvent->owner;
-    NS_RELEASE(channel);
-    delete aEvent;
+    mListener = nsnull;
+    mContext  = nsnull;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsChromeProtocolHandler, nsIProtocolHandler, nsISupportsWeakReference)
+NS_IMPL_THREADSAFE_ISUPPORTS2(nsChromeProtocolHandler,
+                              nsIProtocolHandler,
+                              nsISupportsWeakReference)
 
 ////////////////////////////////////////////////////////////////////////////////
 // nsIProtocolHandler methods:
@@ -464,50 +416,32 @@ nsChromeProtocolHandler::NewURI(const nsACString &aSpec,
                                 nsIURI *aBaseURI,
                                 nsIURI **result)
 {
-    NS_PRECONDITION(result, "Null out param");
-    
     nsresult rv;
 
-    *result = nsnull;
-
     // Chrome: URLs (currently) have no additional structure beyond that provided
     // by standard URLs, so there is no "outer" given to CreateInstance
 
-    nsCOMPtr<nsIStandardURL> url(do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv));
+    nsCOMPtr<nsIStandardURL> surl(do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = surl->Init(nsIStandardURL::URLTYPE_STANDARD, -1, aSpec, aCharset, aBaseURI);
     if (NS_FAILED(rv))
         return rv;
 
-    rv = url->Init(nsIStandardURL::URLTYPE_STANDARD, -1, aSpec, aCharset, aBaseURI);
-    if (NS_FAILED(rv))
-        return rv;
-
-    nsCOMPtr<nsIURI> uri(do_QueryInterface(url, &rv));
-    if (NS_FAILED(rv))
-        return rv;
+    nsCOMPtr<nsIURL> url(do_QueryInterface(surl, &rv));
+    NS_ENSURE_SUCCESS(rv, rv);
 
     // Canonify the "chrome:" URL; e.g., so that we collapse
     // "chrome://navigator/content/" and "chrome://navigator/content"
     // and "chrome://navigator/content/navigator.xul".
 
-    // Try the global cache first.
-    nsCOMPtr<nsIChromeRegistry> reg = gChromeRegistry;
-
-    // If that fails, the service has not been instantiated yet; let's
-    // do that now.
-    if (!reg) {
-        reg = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
-        if (NS_FAILED(rv))
-            return rv;
-    }
-
-    NS_ASSERTION(reg, "Must have a chrome registry by now");
-    
-    rv = reg->Canonify(uri);
+    rv = nsChromeRegistry::Canonify(url);
     if (NS_FAILED(rv))
         return rv;
 
-    *result = uri;
-    NS_ADDREF(*result);
+    surl->SetMutable(PR_FALSE);
+
+    NS_ADDREF(*result = url);
     return NS_OK;
 }
 
@@ -515,42 +449,42 @@ NS_IMETHODIMP
 nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
                                     nsIChannel* *aResult)
 {
+    nsresult rv;
+
     NS_ENSURE_ARG_POINTER(aURI);
     NS_PRECONDITION(aResult, "Null out param");
     
 #ifdef DEBUG
     // Check that the uri we got is already canonified
     nsresult debug_rv;
-    nsCOMPtr<nsIChromeRegistry> debugReg(do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &debug_rv));
+    nsCOMPtr<nsIURI> debugClone;
+    debug_rv = aURI->Clone(getter_AddRefs(debugClone));
     if (NS_SUCCEEDED(debug_rv)) {
-        nsCOMPtr<nsIURI> debugClone;
-        debug_rv = aURI->Clone(getter_AddRefs(debugClone));
+        nsCOMPtr<nsIURL> debugURL (do_QueryInterface(debugClone));
+        debug_rv = nsChromeRegistry::Canonify(debugURL);
         if (NS_SUCCEEDED(debug_rv)) {
-            debug_rv = debugReg->Canonify(debugClone);
+            PRBool same;
+            debug_rv = aURI->Equals(debugURL, &same);
             if (NS_SUCCEEDED(debug_rv)) {
-                PRBool same;
-                debug_rv = aURI->Equals(debugClone, &same);
-                if (NS_SUCCEEDED(debug_rv)) {
-                    NS_ASSERTION(same, "Non-canonified chrome uri passed to nsChromeProtocolHandler::NewChannel!");
-                }
+                NS_ASSERTION(same, "Non-canonified chrome uri passed to nsChromeProtocolHandler::NewChannel!");
             }
-                
         }
     }
 #endif
 
-    nsresult rv;
     nsCOMPtr<nsIChannel> result;
 
 #ifdef MOZ_XUL
     // Check the prototype cache to see if we've already got the
     // document in the cache.
-    nsCOMPtr<nsIXULPrototypeCache> cache =
-             do_GetService(kXULPrototypeCacheCID, &rv);
-    if (NS_FAILED(rv)) return rv;
-
+    nsCOMPtr<nsIXULPrototypeCache> cache
+        (do_GetService(kXULPrototypeCacheCID));
     nsCOMPtr<nsIXULPrototypeDocument> proto;
-    cache->GetPrototype(aURI, getter_AddRefs(proto));
+
+    if (cache)
+        cache->GetPrototype(aURI, getter_AddRefs(proto));
+    else
+        NS_WARNING("Unable to obtain the XUL prototype cache!");
 
     // Same comment as nsXULDocument::StartDocumentLoad and
     // nsXULDocument::ResumeWalk
@@ -575,39 +509,37 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
         if (! result)
             return NS_ERROR_OUT_OF_MEMORY;
     }
-    else
-#endif
-        {
+    else {
+#endif // MOZ_XUL
         // Miss. Resolve the chrome URL using the registry and do a
         // normal necko load.
         //nsXPIDLCString oldSpec;
         //aURI->GetSpec(getter_Copies(oldSpec));
         //printf("*************************** %s\n", (const char*)oldSpec);
 
-        nsCOMPtr<nsIChromeRegistry> reg = gChromeRegistry;
-        if (!reg) {
-            reg = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
-            if (NS_FAILED(rv)) return rv;
+        if (!nsChromeRegistry::gChromeRegistry) {
+            // We don't actually want this ref, we just want the service to
+            // initialize if it hasn't already.
+            nsCOMPtr<nsIChromeRegistry> reg (do_GetService(NS_CHROMEREGISTRY_CONTRACTID));
         }
 
-        nsCAutoString spec;
-        rv = reg->ConvertChromeURL(aURI, spec);
+        NS_ENSURE_TRUE(nsChromeRegistry::gChromeRegistry, NS_ERROR_FAILURE);
+
+        nsCOMPtr<nsIURI> resolvedURI;
+        rv = nsChromeRegistry::gChromeRegistry->ConvertChromeURL(aURI, getter_AddRefs(resolvedURI));
         if (NS_FAILED(rv)) {
 #ifdef DEBUG
+            nsCAutoString spec;
             aURI->GetSpec(spec);
             printf("Couldn't convert chrome URL: %s\n", spec.get());
 #endif
             return rv;
         }
 
-        nsCOMPtr<nsIIOService> ioServ(do_GetIOService(&rv));
-        if (NS_FAILED(rv)) return rv;
+        nsCOMPtr<nsIIOService> ioServ (do_GetIOService(&rv));
+        NS_ENSURE_SUCCESS(rv, rv);
 
-        nsCOMPtr<nsIURI> chromeURI;
-        rv = ioServ->NewURI(spec, nsnull, nsnull, getter_AddRefs(chromeURI));
-        if (NS_FAILED(rv)) return rv;
-
-        rv = ioServ->NewChannelFromURI(chromeURI, getter_AddRefs(result));
+        rv = ioServ->NewChannelFromURI(resolvedURI, getter_AddRefs(result));
         if (NS_FAILED(rv)) return rv;
 
         // XXX Will be removed someday when we handle remote chrome.
@@ -642,14 +574,12 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
         rv = result->SetOriginalURI(aURI);
         if (NS_FAILED(rv)) return rv;
 
-        // Get a system principal for xul files and set the owner
+        // Get a system principal for content files and set the owner
         // property of the result
         nsCOMPtr<nsIURL> url = do_QueryInterface(aURI);
-        nsCAutoString fileExtension;
-        rv = url->GetFileExtension(fileExtension);
-        if (PL_strcasecmp(fileExtension.get(), "xul") == 0 ||
-            PL_strcasecmp(fileExtension.get(), "html") == 0 ||
-            PL_strcasecmp(fileExtension.get(), "xml") == 0)
+        nsCAutoString path;
+        rv = url->GetPath(path);
+        if (StringBeginsWith(path, NS_LITERAL_CSTRING("/content/")))
         {
             nsCOMPtr<nsIScriptSecurityManager> securityManager =
                      do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
@@ -663,6 +593,7 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
             result->SetOwner(owner);
         }
 
+#ifdef MOZ_XUL
         // Track FastLoad file dependencies.
         //
         // This is harder than it ought to be!  While an nsResChannel "is-a"
@@ -700,14 +631,13 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
 
                 if (file) {
                     rv = fastLoadServ->AddDependency(file);
-#ifdef MOZ_XUL
                     if (NS_FAILED(rv))
                         cache->AbortFastLoads();
-#endif
                 }
             }
         }
     }
+#endif
 
     *aResult = result;
     NS_ADDREF(*aResult);
