@@ -37,23 +37,19 @@
 #ifndef nsIDocument_h___
 #define nsIDocument_h___
 
-#include "nsISupports.h"
-#include "nsEvent.h"
-#include "nsString.h"
-#include "nsCOMArray.h"
-#include "nsIDocumentObserver.h"
+#include "nsINode.h"
+#include "nsStringGlue.h"
+#include "nsIDocumentObserver.h" // for nsUpdateType
 #include "nsCOMPtr.h"
 #include "nsIURI.h"
 #include "nsIBindingManager.h"
 #include "nsWeakPtr.h"
 #include "nsIWeakReferenceUtils.h"
 #include "nsILoadGroup.h"
-#include "nsReadableUtils.h"
 #include "nsCRT.h"
 #include "mozFlushType.h"
-#include "nsPropertyTable.h"
+#include "nsIAtom.h"
 
-class nsIAtom;
 class nsIContent;
 class nsPresContext;
 class nsIPresShell;
@@ -65,6 +61,7 @@ class nsIStyleSheet;
 class nsIStyleRule;
 class nsIViewManager;
 class nsIScriptGlobalObject;
+class nsPIDOMWindow;
 class nsIDOMEvent;
 class nsIDeviceContext;
 class nsIParser;
@@ -86,40 +83,66 @@ class nsNodeInfoManager;
 class nsICSSLoader;
 class nsHTMLStyleSheet;
 class nsIHTMLCSSStyleSheet;
+class nsILayoutHistoryState;
+class nsIVariant;
+class nsIDOMUserDataHandler;
+template<class E> class nsCOMArray;
+class nsIDocumentObserver;
 
 // IID for the nsIDocument interface
-// c59c70e5-28d1-494b-9f8e-c18368d09ebc
 #define NS_IDOCUMENT_IID      \
-{ 0x9f670164, 0xc446, 0x11d8, \
-  { 0x84, 0xe1, 0x00, 0x0a, 0x95, 0xdc, 0x23, 0x4c } }
-
-// The base value for the content ID counter.
-// This counter is used by the document to 
-// assign a monotonically increasing ID to each content
-// object it creates
-#define NS_CONTENT_ID_COUNTER_BASE 10000
-
+{ 0x3a5ce97e, 0x4f41, 0x42ef, \
+ { 0xa7, 0xc9, 0xcc, 0x3d, 0x4e, 0x7a, 0x3d, 0x00 } }
 
 // Flag for AddStyleSheet().
 #define NS_STYLESHEET_FROM_CATALOG                (1 << 0)
 
-
 //----------------------------------------------------------------------
 
-// Document interface
-class nsIDocument : public nsISupports
+// Document interface.  This is implemented by all document objects in
+// Gecko.
+class nsIDocument : public nsINode
 {
 public:
-  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IDOCUMENT_IID)
+  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IDOCUMENT_IID)
   NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
 
+#ifdef MOZILLA_INTERNAL_API
   nsIDocument()
-    : mCharacterSet(NS_LITERAL_CSTRING("ISO-8859-1")),
-      mNextContentID(NS_CONTENT_ID_COUNTER_BASE),
-      mNodeInfoManager(nsnull)
+    : nsINode(nsnull),
+      mCharacterSet(NS_LITERAL_CSTRING("ISO-8859-1")),
+      mNodeInfoManager(nsnull),
+      mIsInitialDocumentInWindow(PR_FALSE),
+      mPartID(0)
   {
+    mParentPtrBits |= PARENT_BIT_INDOCUMENT;
   }
-
+#endif
+  
+  /**
+   * Let the document know that we're starting to load data into it.
+   * @param aCommand The parser command
+   *                 XXXbz It's odd to have that here.
+   * @param aChannel The channel the data will come from
+   * @param aLoadGroup The loadgroup this document should use from now on.
+   *                   Note that the document might not be the only thing using
+   *                   this loadgroup.
+   * @param aContainer The container this document is in.  This may be null.
+   *                   XXXbz maybe we should make it more explicit (eg make the
+   *                   container an nsIWebNavigation or nsIDocShell or
+   *                   something)?
+   * @param [out] aDocListener the listener to pump data from the channel into.
+   *                           Generally this will be the parser this document
+   *                           sets up, or some sort of data-handler for media
+   *                           documents.
+   * @param aReset whether the document should call Reset() on itself.  If this
+   *               is false, the document will NOT set its principal to the
+   *               channel's owner, will not clear any event listeners that are
+   *               already set on it, etc.
+   * @param aSink The content sink to use for the data.  If this is null and
+   *              the document needs a content sink, it will create one based
+   *              on whatever it knows about the data it's going to load.
+   */  
   virtual nsresult StartDocumentLoad(const char* aCommand,
                                      nsIChannel* aChannel,
                                      nsILoadGroup* aLoadGroup,
@@ -127,13 +150,13 @@ public:
                                      nsIStreamListener **aDocListener,
                                      PRBool aReset,
                                      nsIContentSink* aSink = nsnull) = 0;
-
   virtual void StopDocumentLoad() = 0;
 
   /**
-   * Return the title of the document. May return null.
+   * Return the title of the document.  This will return a void string
+   * if there is no title for this document).
    */
-  const nsAFlatString& GetDocumentTitle() const
+  const nsString& GetDocumentTitle() const
   {
     return mDocumentTitle;
   }
@@ -145,15 +168,11 @@ public:
   {
     return mDocumentURI;
   }
-  void SetDocumentURI(nsIURI* aURI)
-  {
-    mDocumentURI = aURI;
-  }
 
   /**
-   * Return the principal responsible for this document.
+   * Set the URI for the document.
    */
-  virtual nsIPrincipal* GetPrincipal() = 0;
+  virtual void SetDocumentURI(nsIURI* aURI) = 0;
 
   /**
    * Set the principal responsible for this document.
@@ -194,7 +213,7 @@ public:
    * will trigger a startDocumentLoad if necessary to answer the
    * question.
    */
-  const nsAFlatCString& GetDocumentCharacterSet() const
+  const nsCString& GetDocumentCharacterSet() const
   {
     return mCharacterSet;
   }
@@ -242,7 +261,7 @@ public:
    */
   void GetContentLanguage(nsAString& aContentLanguage) const
   {
-    CopyASCIItoUCS2(mContentLanguage, aContentLanguage);
+    CopyASCIItoUTF16(mContentLanguage, aContentLanguage);
   }
 
   // The state BidiEnabled should persist across multiple views
@@ -269,13 +288,43 @@ public:
   }
 
   /**
-   * Return the Line Breaker for the document
+   * Ask this document whether it's the initial document in its window.
    */
-  virtual nsILineBreaker* GetLineBreaker() = 0;
-  virtual void SetLineBreaker(nsILineBreaker* aLineBreaker) = 0;
-  virtual nsIWordBreaker* GetWordBreaker() = 0;
-  virtual void SetWordBreaker(nsIWordBreaker* aWordBreaker) = 0;
+  PRBool IsInitialDocument() const
+  {
+    return mIsInitialDocumentInWindow;
+  }
+  
+  /**
+   * Tell this document that it's the initial document in its window.  See
+   * comments on mIsInitialDocumentInWindow for when this should be called.
+   */
+  void SetIsInitialDocument(PRBool aIsInitialDocument)
+  {
+    mIsInitialDocumentInWindow = aIsInitialDocument;
+  }
+  
 
+  /**
+   * Get the bidi options for this document.
+   * @see nsBidiUtils.h
+   */
+  PRUint32 GetBidiOptions() const
+  {
+    return mBidiOptions;
+  }
+
+  /**
+   * Set the bidi options for this document.  This just sets the bits;
+   * callers are expected to take action as needed if they want this
+   * change to actually change anything immediately.
+   * @see nsBidiUtils.h
+   */
+  void SetBidiOptions(PRUint32 aBidiOptions)
+  {
+    mBidiOptions = aBidiOptions;
+  }
+  
   /**
    * Access HTTP header data (this may also get set from other
    * sources, like HTML META tags).
@@ -295,6 +344,7 @@ public:
   virtual PRBool DeleteShell(nsIPresShell* aShell) = 0;
   virtual PRUint32 GetNumberOfShells() const = 0;
   virtual nsIPresShell *GetShellAt(PRUint32 aIndex) const = 0;
+  virtual void SetShellsHidden(PRBool aHide) = 0;
 
   /**
    * Return the parent document of this document. Will return null
@@ -337,15 +387,6 @@ public:
   {
     return mRootContent;
   }
-  virtual void SetRootContent(nsIContent* aRoot) = 0;
-
-  /** 
-   * Get the direct children of the document - content in
-   * the prolog, the root content and content in the epilog.
-   */
-  virtual nsIContent *GetChildAt(PRUint32 aIndex) const = 0;
-  virtual PRInt32 IndexOf(nsIContent* aPossibleChild) const = 0;
-  virtual PRUint32 GetChildCount() const = 0;
 
   /**
    * Accessors to the collection of stylesheets owned by this document.
@@ -420,12 +461,21 @@ public:
   virtual PRInt32 GetNumberOfCatalogStyleSheets() const = 0;
   virtual nsIStyleSheet* GetCatalogStyleSheetAt(PRInt32 aIndex) const = 0;
   virtual void AddCatalogStyleSheet(nsIStyleSheet* aSheet) = 0;
+  virtual void EnsureCatalogStyleSheet(const char *aStyleSheetURI) = 0;
 
   /**
-   * Get this document's CSSLoader.  May return null in error
-   * conditions (OOM)
+   * Get this document's CSSLoader.  This is guaranteed to not return null.
    */
-  virtual nsICSSLoader* GetCSSLoader() = 0;
+  nsICSSLoader* CSSLoader() const {
+    return mCSSLoader;
+  }
+
+  /**
+   * Get the channel that was passed to StartDocumentLoad or Reset for this
+   * document.  Note that this may be null in some cases (eg if
+   * StartDocumentLoad or Reset were never called)
+   */
+  virtual nsIChannel* GetChannel() const = 0;
 
   /**
    * Get this document's attribute stylesheet.  May return null if
@@ -440,13 +490,35 @@ public:
   virtual nsIHTMLCSSStyleSheet* GetInlineStyleSheet() const = 0;
   
   /**
-   * Set the object from which a document can get a script context.
-   * This is the context within which all scripts (during document 
-   * creation and during event handling) will run.
+   * Get/set the object from which a document can get a script context
+   * and scope. This is the context within which all scripts (during
+   * document creation and during event handling) will run. Note that
+   * this is the *inner* window object.
    */
   virtual nsIScriptGlobalObject* GetScriptGlobalObject() const = 0;
   virtual void SetScriptGlobalObject(nsIScriptGlobalObject* aGlobalObject) = 0;
 
+  /**
+   * Get the object that is used as the scope for all of the content
+   * wrappers whose owner document is this document. Unlike the script global
+   * object, this will only return null when the global object for this
+   * document is truly gone. Use this object when you're trying to find a
+   * content wrapper in XPConnect.
+   */
+  virtual nsIScriptGlobalObject* GetScopeObject() = 0;
+
+  /**
+   * Return the window containing the document (the outer window).
+   */
+  virtual nsPIDOMWindow *GetWindow() = 0;
+
+  /**
+   * Return the inner window used as the script compilation scope for
+   * this document. If you're not absolutely sure you need this, use
+   * GetWindow().
+   */
+  virtual nsPIDOMWindow *GetInnerWindow() = 0;
+
   /**
    * Get the script loader for this document
    */ 
@@ -477,27 +549,11 @@ public:
   virtual void EndUpdate(nsUpdateType aUpdateType) = 0;
   virtual void BeginLoad() = 0;
   virtual void EndLoad() = 0;
-  virtual void CharacterDataChanged(nsIContent* aContent, PRBool aAppend) = 0;
   // notify that one or two content nodes changed state
   // either may be nsnull, but not both
   virtual void ContentStatesChanged(nsIContent* aContent1,
                                     nsIContent* aContent2,
                                     PRInt32 aStateMask) = 0;
-  virtual void AttributeWillChange(nsIContent* aChild,
-                                   PRInt32 aNameSpaceID,
-                                   nsIAtom* aAttribute) = 0;
-  virtual void AttributeChanged(nsIContent* aChild,
-                                PRInt32 aNameSpaceID,
-                                nsIAtom* aAttribute,
-                                PRInt32 aModType) = 0;
-  virtual void ContentAppended(nsIContent* aContainer,
-                               PRInt32 aNewIndexInContainer) = 0;
-  virtual void ContentInserted(nsIContent* aContainer,
-                               nsIContent* aChild,
-                               PRInt32 aIndexInContainer) = 0;
-  virtual void ContentRemoved(nsIContent* aContainer,
-                              nsIContent* aChild,
-                              PRInt32 aIndexInContainer) = 0;
 
   // Observation hooks for style data to propagate notifications
   // to document observers
@@ -509,10 +565,12 @@ public:
   virtual void StyleRuleRemoved(nsIStyleSheet* aStyleSheet,
                                 nsIStyleRule* aStyleRule) = 0;
 
-  virtual nsresult HandleDOMEvent(nsPresContext* aPresContext,
-                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
-                                  PRUint32 aFlags,
-                                  nsEventStatus* aEventStatus) = 0;
+  /**
+   * Notify document of pending attribute change
+   */
+  virtual void AttributeWillChange(nsIContent* aChild,
+                                   PRInt32 aNameSpaceID,
+                                   nsIAtom* aAttribute) = 0;
 
   /**
    * Flush notifications for this document and its parent documents
@@ -520,12 +578,7 @@ public:
    */
   virtual void FlushPendingNotifications(mozFlushType aType) = 0;
 
-  PRInt32 GetAndIncrementContentID()
-  {
-    return mNextContentID++;
-  }
-
-  nsIBindingManager* GetBindingManager() const
+  nsIBindingManager* BindingManager() const
   {
     return mBindingManager;
   }
@@ -539,15 +592,17 @@ public:
     return mNodeInfoManager;
   }
 
+  /**
+   * Reset the document using the given channel and loadgroup.  This works
+   * like ResetToURI, but also sets the document's channel to aChannel.
+   */
   virtual void Reset(nsIChannel* aChannel, nsILoadGroup* aLoadGroup) = 0;
+
   /**
    * Reset this document to aURI and aLoadGroup.  aURI must not be null.
    */
   virtual void ResetToURI(nsIURI *aURI, nsILoadGroup* aLoadGroup) = 0;
 
-  virtual void AddReference(void *aKey, nsISupports *aReference) = 0;
-  virtual already_AddRefed<nsISupports> RemoveReference(void *aKey) = 0;
-
   /**
    * Set the container (docshell) for this document.
    */
@@ -571,12 +626,14 @@ public:
   virtual nsIScriptEventManager* GetScriptEventManager() = 0;
 
   /**
-   * Set and get XML declaration. Notice that if version is empty,
-   * there can be no XML declaration (it is a required part).
+   * Set and get XML declaration. If aVersion is null there is no declaration.
+   * aStandalone takes values -1, 0 and 1 indicating respectively that there
+   * was no standalone parameter in the declaration, that it was given as no,
+   * or that it was given as yes.
    */
-  virtual void SetXMLDeclaration(const nsAString& aVersion,
-                                 const nsAString& aEncoding,
-                                 const nsAString& Standalone) = 0;
+  virtual void SetXMLDeclaration(const PRUnichar *aVersion,
+                                 const PRUnichar *aEncoding,
+                                 const PRInt32 aStandalone) = 0;
   virtual void GetXMLDeclaration(nsAString& aVersion,
                                  nsAString& aEncoding,
                                  nsAString& Standalone) = 0;
@@ -590,6 +647,11 @@ public:
 
   virtual nsresult AddXMLEventsContent(nsIContent * aXMLEventsElement) = 0;
 
+  virtual PRBool IsLoadedAsData()
+  {
+    return PR_FALSE;
+  }
+
   /**
    * Create an element with the specified name, prefix and namespace ID.
    * If aDocumentDefaultType is true we create an element of the default type
@@ -619,20 +681,135 @@ public:
    */
   virtual PRInt32 GetDefaultNamespaceID() const = 0;
 
-  virtual void* GetProperty(nsIAtom *aPropertyName,
-                            nsresult *aStatus = nsnull) const = 0;
-
-  virtual nsresult SetProperty(nsIAtom            *aPropertyName,
-                               void               *aValue,
-                               NSPropertyDtorFunc  aDtor = nsnull) = 0;
-
-  virtual nsresult DeleteProperty(nsIAtom *aPropertyName) = 0;
-
-  virtual void* UnsetProperty(nsIAtom  *aPropertyName,
-                              nsresult *aStatus = nsnull) = 0;
-
   nsPropertyTable* PropertyTable() { return &mPropertyTable; }
 
+  /**
+   * Sets the ID used to identify this part of the multipart document
+   */
+  void SetPartID(PRUint32 aID) {
+    mPartID = aID;
+  }
+
+  /**
+   * Return the ID used to identify this part of the multipart document
+   */
+  PRUint32 GetPartID() const {
+    return mPartID;
+  }
+
+  /**
+   * Sanitize the document by resetting all input elements and forms that have
+   * autocomplete=off to their default values.
+   */
+  virtual nsresult Sanitize() = 0;
+
+  /**
+   * Enumerate all subdocuments.
+   * The enumerator callback should return PR_TRUE to continue enumerating, or
+   * PR_FALSE to stop.
+   */
+  typedef PRBool (*nsSubDocEnumFunc)(nsIDocument *aDocument, void *aData);
+  virtual void EnumerateSubDocuments(nsSubDocEnumFunc aCallback,
+                                     void *aData) = 0;
+
+  /**
+   * Check whether it is safe to cache the presentation of this document
+   * and all of its subdocuments. This method checks the following conditions
+   * recursively:
+   *  - Some document types, such as plugin documents, cannot be safely cached.
+   *  - If there are any pending requests, we don't allow the presentation
+   *    to be cached.  Ideally these requests would be suspended and resumed,
+   *    but that is difficult in some cases, such as XMLHttpRequest.
+   *  - If there are any beforeunload or unload listeners, we must fire them
+   *    for correctness, but this likely puts the document into a state where
+   *    it would not function correctly if restored.
+   *
+   * |aNewRequest| should be the request for a new document which will
+   * replace this document in the docshell.  The new document's request
+   * will be ignored when checking for active requests.  If there is no
+   * request associated with the new document, this parameter may be null.
+   */
+  virtual PRBool CanSavePresentation(nsIRequest *aNewRequest) = 0;
+
+  /**
+   * Notify the document that its associated ContentViewer is being destroyed.
+   * This releases circular references so that the document can go away.
+   * Destroy() is only called on documents that have a content viewer.
+   */
+  virtual void Destroy() = 0;
+
+  /**
+   * Get the layout history state that should be used to save and restore state
+   * for nodes in this document.  This may return null; if that happens state
+   * saving and restoration is not possible.
+   */
+  virtual already_AddRefed<nsILayoutHistoryState> GetLayoutHistoryState() const = 0;
+
+  /**
+   * Methods that can be used to prevent onload firing while an event that
+   * should block onload is posted.  onload is guaranteed to not fire until
+   * either all calls to BlockOnload() have been matched by calls to
+   * UnblockOnload() or the load has been stopped altogether (by the user
+   * pressing the Stop button, say).
+   */
+  virtual void BlockOnload() = 0;
+  /**
+   * @param aFireSync whether to fire onload synchronously.  If false,
+   * onload will fire asynchronously after all onload blocks have been
+   * removed.  It will NOT fire from inside UnblockOnload.  If true,
+   * onload may fire from inside UnblockOnload.
+   */
+  virtual void UnblockOnload(PRBool aFireSync) = 0;
+
+  /**
+   * Notification that the page has been shown, for documents which are loaded
+   * into a DOM window.  This corresponds to the completion of document load,
+   * or to the page's presentation being restored into an existing DOM window.
+   * This notification fires applicable DOM events to the content window.  See
+   * nsIDOMPageTransitionEvent.idl for a description of the |aPersisted|
+   * parameter.
+   */
+  virtual void OnPageShow(PRBool aPersisted) = 0;
+
+  /**
+   * Notification that the page has been hidden, for documents which are loaded
+   * into a DOM window.  This corresponds to the unloading of the document, or
+   * to the document's presentation being saved but removed from an existing
+   * DOM window.  This notification fires applicable DOM events to the content
+   * window.  See nsIDOMPageTransitionEvent.idl for a description of the
+   * |aPersisted| parameter.
+   */
+  virtual void OnPageHide(PRBool aPersisted) = 0;
+  
+  /*
+   * We record the set of links in the document that are relevant to
+   * style.
+   */
+  /**
+   * Notification that an element is a link with a given URI that is
+   * relevant to style.
+   */
+  virtual void AddStyleRelevantLink(nsIContent* aContent, nsIURI* aURI) = 0;
+  /**
+   * Notification that an element is a link and its URI might have been
+   * changed or the element removed. If the element is still a link relevant
+   * to style, then someone must ensure that AddStyleRelevantLink is
+   * (eventually) called on it again.
+   */
+  virtual void ForgetLink(nsIContent* aContent) = 0;
+  /**
+   * Notification that the visitedness state of a URI has been changed
+   * and style related to elements linking to that URI should be updated.
+   */
+  virtual void NotifyURIVisitednessChanged(nsIURI* aURI) = 0;
+
+  /**
+   * Resets and removes a box object from the document's box object cache
+   *
+   * @param aElement canonical nsIContent pointer of the box object's element
+   */
+  virtual void ClearBoxObjectFor(nsIContent *aContent) = 0;
+
 protected:
   ~nsIDocument()
   {
@@ -660,26 +837,41 @@ protected:
   // such element exists.
   nsIContent* mRootContent;
 
-  // A content ID counter used to give a monotonically increasing ID
-  // to the content objects in the document's content model
-  PRInt32 mNextContentID;
-
   nsCOMPtr<nsIBindingManager> mBindingManager;
   nsNodeInfoManager* mNodeInfoManager; // [STRONG]
 
+  nsICSSLoader* mCSSLoader; // [STRONG; not a COMPtr to avoid
+                            // including nsICSSLoader.h; the ownership
+                            // is managed by nsDocument]
+
   // Table of element properties for this document.
   nsPropertyTable mPropertyTable;
 
   // True if BIDI is enabled.
-  PRBool mBidiEnabled;
+  PRPackedBool mBidiEnabled;
 
-  nsXPIDLCString mContentLanguage;
+  // True if this document is the initial document for a window.  This should
+  // basically be true only for documents that exist in newly-opened windows or
+  // documents created to satisfy a GetDocument() on a window when there's no
+  // document in it.
+  PRPackedBool mIsInitialDocumentInWindow;
+
+  // The bidi options for this document.  What this bitfield means is
+  // defined in nsBidiUtils.h
+  PRUint32 mBidiOptions;
+
+  nsCString mContentLanguage;
   nsCString mContentType;
 
   // The document's security info
   nsCOMPtr<nsISupports> mSecurityInfo;
+
+  // if this document is part of a multipart document,
+  // the ID can be used to distinguish it from the other parts.
+  PRUint32 mPartID;
 };
 
+NS_DEFINE_STATIC_IID_ACCESSOR(nsIDocument, NS_IDOCUMENT_IID)
 
 /**
  * Helper class to automatically handle batching of document updates.  This
@@ -713,6 +905,13 @@ private:
   nsUpdateType mUpdateType;
 };
 
+#define MOZ_AUTO_DOC_UPDATE_PASTE2(tok,line) tok##line
+#define MOZ_AUTO_DOC_UPDATE_PASTE(tok,line) \
+  MOZ_AUTO_DOC_UPDATE_PASTE2(tok,line)
+#define MOZ_AUTO_DOC_UPDATE(doc,type,notify) \
+  mozAutoDocUpdate MOZ_AUTO_DOC_UPDATE_PASTE(_autoDocUpdater_, __LINE__) \
+  (doc,type,notify)
+
 // XXX These belong somewhere else
 nsresult
 NS_NewHTMLDocument(nsIDocument** aInstancePtrResult);
@@ -730,13 +929,19 @@ NS_NewImageDocument(nsIDocument** aInstancePtrResult);
 
 nsresult
 NS_NewDocumentFragment(nsIDOMDocumentFragment** aInstancePtrResult,
-                       nsIDocument* aOwnerDocument);
+                       nsNodeInfoManager *aNodeInfoManager);
+
+// Note: it's the caller's responsibility to create or get aPrincipal as needed
+// -- this method will not attempt to get a principal based on aDocumentURI.
+// Also, both aDocumentURI and aBaseURI must not be null.
 nsresult
 NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
                   const nsAString& aNamespaceURI, 
                   const nsAString& aQualifiedName, 
                   nsIDOMDocumentType* aDoctype,
-                  nsIURI* aBaseURI);
+                  nsIURI* aDocumentURI,
+                  nsIURI* aBaseURI,
+                  nsIPrincipal* aPrincipal);
 nsresult
 NS_NewPluginDocument(nsIDocument** aInstancePtrResult);
 
