@@ -36,11 +36,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-/*
- * nsContentIterator.cpp: Implementation of the nsContentIterator object.
- * This ite
- */
-
 #include "nsISupports.h"
 #include "nsIDOMNodeList.h"
 #include "nsIContentIterator.h"
@@ -576,7 +571,7 @@ nsresult nsContentIterator::RebuildIndexStack()
 {
   // Make sure we start at the right indexes on the stack!  Build array up
   // to common parent of start and end.  Perhaps it's too many entries, but
-  // thats far better than too few.
+  // that's far better than too few.
   nsIContent* parent;
   nsIContent* current;
 
@@ -1116,7 +1111,10 @@ nsContentIterator::PositionAt(nsIContent* aCurNode)
   if (!oldParentStack.SizeTo(mIndexes.Count()+1))
     return NS_ERROR_FAILURE;
 
-  // plus one for the node we're currently on.
+  // We want to loop mIndexes.Count() + 1 times here, because we want to make
+  // sure we include mCommonParent in the oldParentStack, for use in the next
+  // for loop, and mIndexes only has entries for nodes from tempNode up through
+  // an ancestor of tempNode that's a child of mCommonParent.
   for (PRInt32 i = mIndexes.Count()+1; i > 0 && tempNode; i--)
   {
     // Insert at head since we're walking up
@@ -1131,7 +1129,7 @@ nsContentIterator::PositionAt(nsIContent* aCurNode)
     {
       // The position was moved to a parent of the current position. 
       // All we need to do is drop some indexes.  Shortcut here.
-      mIndexes.RemoveElementsAt(mIndexes.Count() - (oldParentStack.Count()+1),
+      mIndexes.RemoveElementsAt(mIndexes.Count() - oldParentStack.Count(),
                                 oldParentStack.Count());
       mIsDone = PR_FALSE;
       return NS_OK;
@@ -1159,11 +1157,12 @@ nsContentIterator::PositionAt(nsIContent* aCurNode)
       // ok, the parent IS on the old stack!  Rework things.
       // we want newIndexes to replace all nodes equal to or below the match
       // Note that index oldParentStack.Count()-1 is the last node, which is
-      // one BELOW the last index in the mIndexes stack.
+      // one BELOW the last index in the mIndexes stack.  In other words, we
+      // want to remove elements starting at index (indx+1).
       PRInt32 numToDrop = oldParentStack.Count()-(1+indx);
       if (numToDrop > 0)
         mIndexes.RemoveElementsAt(mIndexes.Count() - numToDrop,numToDrop);
-      mIndexes.InsertElementsAt(newIndexes,mIndexes.Count());
+      mIndexes.AppendElements(newIndexes);
 
       break;
     }
@@ -1326,6 +1325,13 @@ nsresult nsContentSubtreeIterator::Init(nsIDOMRange* aRange)
     return NS_ERROR_FAILURE;
   cEndP = do_QueryInterface(endParent);
   aRange->GetEndOffset(&endIndx);
+
+  if (!cStartP || !cEndP)
+  {
+    // XXX Hack to account for the fact that not everything QIs to nsIContent.
+    // See bug 302775
+    return NS_ERROR_FAILURE;
+  }
   
   // short circuit when start node == end node
   if (startParent == endParent)
