@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=78: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -35,6 +36,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * nsBaseContentList is a basic list of content nodes; nsContentList
+ * is a commonly used NodeList implementation (used for
+ * getElementsByTagName, some properties on nsIDOMHTMLDocument, etc).
+ */
+
 #include "nsContentList.h"
 #include "nsIContent.h"
 #include "nsIDOMNode.h"
@@ -49,12 +56,20 @@
 
 // Form related includes
 #include "nsIDOMHTMLFormElement.h"
-#include "nsIContentList.h"
 
 #include "pldhash.h"
 
+#ifdef DEBUG_CONTENT_LIST
+#include "nsIContentIterator.h"
+nsresult
+NS_NewPreContentIterator(nsIContentIterator** aInstancePtrResult);
+#define ASSERT_IN_SYNC AssertInSync()
+#else
+#define ASSERT_IN_SYNC PR_BEGIN_MACRO PR_END_MACRO
+#endif
 
-static nsIContentList *gCachedContentList;
+
+static nsContentList *gCachedContentList;
 
 nsBaseContentList::nsBaseContentList()
 {
@@ -62,8 +77,6 @@ nsBaseContentList::nsBaseContentList()
 
 nsBaseContentList::~nsBaseContentList()
 {
-  // mElements only has weak references to the content objects so we
-  // don't need to do any cleanup here.
 }
 
 
@@ -90,8 +103,8 @@ nsBaseContentList::GetLength(PRUint32* aLength)
 NS_IMETHODIMP
 nsBaseContentList::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 {
-  nsISupports *tmp = NS_REINTERPRET_CAST(nsISupports *,
-                                         mElements.SafeElementAt(aIndex));
+  nsISupports *tmp = mElements.SafeObjectAt(aIndex);
+
   if (!tmp) {
     *aReturn = nsnull;
 
@@ -104,15 +117,13 @@ nsBaseContentList::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 void
 nsBaseContentList::AppendElement(nsIContent *aContent)
 {
-  // Shouldn't hold a reference since we'll be told when the content
-  // leaves the document or the document will be destroyed.
-  mElements.AppendElement(aContent);
+  mElements.AppendObject(aContent);
 }
 
 void
 nsBaseContentList::RemoveElement(nsIContent *aContent)
 {
-  mElements.RemoveElement(aContent);
+  mElements.RemoveObject(aContent);
 }
 
 PRInt32
@@ -160,47 +171,6 @@ nsFormContentList::nsFormContentList(nsIDOMHTMLFormElement *aForm,
   }
 }
 
-nsFormContentList::~nsFormContentList()
-{
-  Reset();
-}
-
-void
-nsFormContentList::AppendElement(nsIContent *aContent)
-{
-  NS_ADDREF(aContent);
-
-  nsBaseContentList::AppendElement(aContent);
-}
-
-void
-nsFormContentList::RemoveElement(nsIContent *aContent)
-{
-  PRInt32 i = mElements.IndexOf(aContent);
-
-  if (i >= 0) {
-    nsIContent *content = NS_STATIC_CAST(nsIContent *, mElements.ElementAt(i));
-
-    NS_RELEASE(content);
-
-    mElements.RemoveElementAt(i);
-  }
-}
-
-void
-nsFormContentList::Reset()
-{
-  PRInt32 i, length = mElements.Count();
-
-  for (i = 0; i < length; i++) {
-    nsIContent *content = NS_STATIC_CAST(nsIContent *, mElements.ElementAt(i));
-
-    NS_RELEASE(content);
-  }
-
-  nsBaseContentList::Reset();
-}
-
 // Hashtable for storing nsContentLists
 static PLDHashTable gContentListHashTable;
 
@@ -236,12 +206,12 @@ ContentListHashtableMatchEntry(PLDHashTable *table,
   return list1->Equals(*list2);
 }
 
-nsresult
-NS_GetContentList(nsIDocument* aDocument, nsIAtom* aMatchAtom,
-                  PRInt32 aMatchNameSpaceId, nsIContent* aRootContent,
-                  nsIContentList** aInstancePtrResult)
+already_AddRefed<nsContentList>
+NS_GetContentList(nsINode* aRootNode, nsIAtom* aMatchAtom,
+                  PRInt32 aMatchNameSpaceId)
 {
-  *aInstancePtrResult = nsnull;
+  NS_ASSERTION(aRootNode, "content list has to have a root");
+
   nsContentList* list = nsnull;
 
   static PLDHashTableOps hash_table_ops =
@@ -271,8 +241,8 @@ NS_GetContentList(nsIDocument* aDocument, nsIAtom* aMatchAtom,
   ContentListHashEntry *entry = nsnull;
   // First we look in our hashtable.  Then we create a content list if needed
   if (gContentListHashTable.ops) {
-    nsContentListKey hashKey(aDocument, aMatchAtom,
-                             aMatchNameSpaceId, aRootContent);
+    nsContentListKey hashKey(aRootNode, aMatchAtom,
+                             aMatchNameSpaceId);
     
     // A PL_DHASH_ADD is equivalent to a PL_DHASH_LOOKUP for cases
     // when the entry is already in the hashtable.
@@ -287,8 +257,8 @@ NS_GetContentList(nsIDocument* aDocument, nsIAtom* aMatchAtom,
   if (!list) {
     // We need to create a ContentList and add it to our new entry, if
     // we have an entry
-    list = new nsContentList(aDocument, aMatchAtom,
-                             aMatchNameSpaceId, aRootContent);
+    list = new nsContentList(aRootNode, aMatchAtom,
+                             aMatchNameSpaceId);
     if (entry) {
       if (list)
         entry->mContentList = list;
@@ -296,11 +266,10 @@ NS_GetContentList(nsIDocument* aDocument, nsIAtom* aMatchAtom,
         PL_DHashTableRawRemove(&gContentListHashTable, entry);
     }
 
-    NS_ENSURE_TRUE(list, NS_ERROR_OUT_OF_MEMORY);
+    NS_ENSURE_TRUE(list, nsnull);
   }
 
-  *aInstancePtrResult = list;
-  NS_ADDREF(*aInstancePtrResult);
+  NS_ADDREF(list);
 
   // Hold on to the last requested content list to avoid having it be
   // removed from the cache immediately when it's released. Avoid
@@ -314,75 +283,63 @@ NS_GetContentList(nsIDocument* aDocument, nsIAtom* aMatchAtom,
     NS_ADDREF(gCachedContentList);
   }
 
-  return NS_OK;
+  return list;
 }
 
 
 // nsContentList implementation
 
-nsContentList::nsContentList(nsIDocument *aDocument,
+nsContentList::nsContentList(nsINode* aRootNode,
                              nsIAtom* aMatchAtom,
                              PRInt32 aMatchNameSpaceId,
-                             nsIContent* aRootContent,
                              PRBool aDeep)
   : nsBaseContentList(),
-    nsContentListKey(aDocument, aMatchAtom, aMatchNameSpaceId, aRootContent),
+    nsContentListKey(aRootNode, aMatchAtom, aMatchNameSpaceId),
     mFunc(nsnull),
     mData(nsnull),
     mState(LIST_DIRTY),
-    mDeep(aDeep)
+    mDeep(aDeep),
+    mFuncMayDependOnAttr(PR_FALSE)
 {
-  NS_ASSERTION(mDeep || mRootContent, "Must have root content for non-deep list!");
-  if (nsLayoutAtoms::wildcard == mMatchAtom) {
+  NS_ASSERTION(mRootNode, "Must have root");
+  if (nsLayoutAtoms::_asterix == mMatchAtom) {
     mMatchAll = PR_TRUE;
   }
   else {
     mMatchAll = PR_FALSE;
   }
-  Init(aDocument);
+  mRootNode->AddMutationObserver(this);
 }
 
-nsContentList::nsContentList(nsIDocument *aDocument,
+nsContentList::nsContentList(nsINode* aRootNode,
                              nsContentListMatchFunc aFunc,
                              const nsAString& aData,
-                             nsIContent* aRootContent,
                              PRBool aDeep,
                              nsIAtom* aMatchAtom,
-                             PRInt32 aMatchNameSpaceId)
+                             PRInt32 aMatchNameSpaceId,
+                             PRBool aFuncMayDependOnAttr)
   : nsBaseContentList(),
-    nsContentListKey(aDocument, aMatchAtom, aMatchNameSpaceId, aRootContent),
+    nsContentListKey(aRootNode, aMatchAtom, aMatchNameSpaceId),
     mFunc(aFunc),
     mData(&EmptyString()),
     mMatchAll(PR_FALSE),
     mState(LIST_DIRTY),
-    mDeep(aDeep)
+    mDeep(aDeep),
+    mFuncMayDependOnAttr(aFuncMayDependOnAttr)
 {
-  NS_ASSERTION(mDeep || mRootContent, "Must have root content for non-deep list!");
+  NS_ASSERTION(mRootNode, "Must have root");
   if (!aData.IsEmpty()) {
     mData = new nsString(aData);
     // If this fails, fail silently
   }
-  Init(aDocument);
-}
-
-void nsContentList::Init(nsIDocument *aDocument)
-{
-  // We don't reference count the reference to the document
-  // If the document goes away first, we'll be informed and we
-  // can drop our reference.
-  // If we go away first, we'll get rid of ourselves from the
-  // document's observer list.
-  mDocument = aDocument;
-  if (mDocument) {
-    mDocument->AddObserver(this);
-  }
+  mRootNode->AddMutationObserver(this);
 }
 
 nsContentList::~nsContentList()
 {
   RemoveFromHashtable();
-  if (mDocument) {
-    mDocument->RemoveObserver(this);
+  if (mRootNode) {
+    mRootNode->RemoveMutationObserver(this);
   }
   
   if (mData && mData != &EmptyString()) {
@@ -395,7 +352,7 @@ nsContentList::~nsContentList()
 // QueryInterface implementation for nsContentList
 NS_INTERFACE_MAP_BEGIN(nsContentList)
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLCollection)
-  NS_INTERFACE_MAP_ENTRY(nsIContentList)
+  NS_INTERFACE_MAP_ENTRY(nsIMutationObserver)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(ContentList)
 NS_INTERFACE_MAP_END_INHERITING(nsBaseContentList)
 
@@ -407,17 +364,12 @@ NS_IMPL_RELEASE_INHERITED(nsContentList, nsBaseContentList)
 nsISupports *
 nsContentList::GetParentObject()
 {
-  if (mRootContent) {
-    return mRootContent;
-  }
-
-  return mDocument;
+  return mRootNode;
 }
   
 PRUint32
 nsContentList::Length(PRBool aDoFlush)
 {
-  CheckDocumentExistence();
   BringSelfUpToDate(aDoFlush);
     
   return mElements.Count();
@@ -426,45 +378,45 @@ nsContentList::Length(PRBool aDoFlush)
 nsIContent *
 nsContentList::Item(PRUint32 aIndex, PRBool aDoFlush)
 {
-  CheckDocumentExistence();
-
-  if (mDocument && aDoFlush) {
-    // Flush pending content changes Bug 4891.
-    mDocument->FlushPendingNotifications(Flush_ContentAndNotify);
+  if (mRootNode && aDoFlush) {
+    // XXX sXBL/XBL2 issue
+    nsIDocument* doc = mRootNode->GetCurrentDoc();
+    if (doc) {
+      // Flush pending content changes Bug 4891.
+      doc->FlushPendingNotifications(Flush_ContentAndNotify);
+    }
   }
 
   if (mState != LIST_UP_TO_DATE)
     PopulateSelf(aIndex+1);
 
-  NS_ASSERTION(!mDocument || mState != LIST_DIRTY,
+  ASSERT_IN_SYNC;
+  NS_ASSERTION(!mRootNode || mState != LIST_DIRTY,
                "PopulateSelf left the list in a dirty (useless) state!");
 
-  return NS_STATIC_CAST(nsIContent *, mElements.SafeElementAt(aIndex));
+  return mElements.SafeObjectAt(aIndex);
 }
 
 nsIContent *
 nsContentList::NamedItem(const nsAString& aName, PRBool aDoFlush)
 {
-  CheckDocumentExistence();
-
   BringSelfUpToDate(aDoFlush);
     
   PRInt32 i, count = mElements.Count();
 
+  // Typically IDs and names are atomized
+  nsCOMPtr<nsIAtom> name = do_GetAtom(aName);
+  NS_ENSURE_TRUE(name, nsnull);
+
   for (i = 0; i < count; i++) {
-    nsIContent *content = NS_STATIC_CAST(nsIContent *,
-                                         mElements.ElementAt(i));
-    if (content) {
-      nsAutoString name;
-      // XXX Should it be an EqualsIgnoreCase?
-      if (((content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::name,
-                             name) == NS_CONTENT_ATTR_HAS_VALUE) &&
-           aName.Equals(name)) ||
-          ((content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::id,
-                             name) == NS_CONTENT_ATTR_HAS_VALUE) &&
-           aName.Equals(name))) {
-        return content;
-      }
+    nsIContent *content = mElements[i];
+    // XXX Should this pass eIgnoreCase?
+    if (content &&
+        (content->AttrValueIs(kNameSpaceID_None, nsHTMLAtoms::name,
+                              name, eCaseMatters) ||
+         content->AttrValueIs(kNameSpaceID_None, nsHTMLAtoms::id,
+                              name, eCaseMatters))) {
+      return content;
     }
   }
 
@@ -474,18 +426,41 @@ nsContentList::NamedItem(const nsAString& aName, PRBool aDoFlush)
 PRInt32
 nsContentList::IndexOf(nsIContent *aContent, PRBool aDoFlush)
 {
-  CheckDocumentExistence();
   BringSelfUpToDate(aDoFlush);
     
   return mElements.IndexOf(aContent);
 }
 
 void
-nsContentList::RootDestroyed()
+nsContentList::NodeWillBeDestroyed(const nsINode* aNode)
 {
   // We shouldn't do anything useful from now on
-  DisconnectFromDocument();  // This dirties us so we lose all state
-  mRootContent = nsnull;
+
+  RemoveFromHashtable();
+  mRootNode = nsnull;
+
+  // We will get no more updates, so we can never know we're up to
+  // date
+  mState = LIST_DIRTY;
+
+  // Call Reset() to prevent getting bogus (or eveng dangling) items since
+  // PopulateSelf() will be a no-op after mRootNode was nulled out above
+  Reset();
+}
+
+// static
+void
+nsContentList::OnDocumentDestroy(nsIDocument *aDocument)
+{
+  // If our content list cache holds a list used for a document that's
+  // now being destroyed, free the cache to prevent the list from
+  // staying around until the next use of content lists ends up
+  // replacing what's in the cache.
+
+  if (gCachedContentList && gCachedContentList->mRootNode &&
+      gCachedContentList->mRootNode->GetOwnerDoc() == aDocument) {
+    NS_RELEASE(gCachedContentList);
+  }
 }
 
 NS_IMETHODIMP
@@ -531,22 +506,27 @@ nsContentList::AttributeChanged(nsIDocument *aDocument, nsIContent* aContent,
 {
   NS_PRECONDITION(aContent, "Must have a content node to work with");
   
-  if (!mFunc || mState == LIST_DIRTY || IsContentAnonymous(aContent)) {
+  if (!mFunc || !mFuncMayDependOnAttr || mState == LIST_DIRTY ||
+      !MayContainRelevantNodes(aContent->GetNodeParent()) ||
+      IsContentAnonymous(aContent)) {
     // Either we're already dirty or this notification doesn't affect
     // whether we might match aContent.
     return;
   }
   
-  if (MayContainRelevantNodes(aContent->GetParent())) {
-    if (Match(aContent)) {
-      // We may now match a new node.  Just dirty ourselves.
+  if (Match(aContent)) {
+    if (mElements.IndexOf(aContent) == -1) {
+      // We match aContent now, and it's not in our list already.  Just dirty
+      // ourselves; this is simpler than trying to figure out where to insert
+      // aContent.
       mState = LIST_DIRTY;
-    } else {
-      // We no longer match aContent.  Remove it from our list.  If
-      // it's already not there, this is a no-op, which is fine.
-      // Either way, no change of mState is required here.
-      mElements.RemoveElement(aContent);
     }
+  } else {
+    // We no longer match aContent.  Remove it from our list.  If it's
+    // already not there, this is a no-op (though a potentially
+    // expensive one).  Either way, no change of mState is required
+    // here.
+    mElements.RemoveObject(aContent);
   }
 }
 
@@ -562,7 +542,8 @@ nsContentList::ContentAppended(nsIDocument *aDocument, nsIContent* aContainer,
    * aContainer is anonymous from our point of view, we know that we can't
    * possibly be matching any of the kids.
    */
-  if (mState == LIST_DIRTY || IsContentAnonymous(aContainer)) 
+  if (mState == LIST_DIRTY || IsContentAnonymous(aContainer) ||
+      !MayContainRelevantNodes(aContainer))
     return;
 
   /*
@@ -576,31 +557,20 @@ nsContentList::ContentAppended(nsIDocument *aDocument, nsIContent* aContainer,
   
   PRInt32 count = aContainer->GetChildCount();
 
-  if (count > 0 && MayContainRelevantNodes(aContainer)) {
+  if (count > 0) {
     PRInt32 ourCount = mElements.Count();
     PRBool appendToList = PR_FALSE;
     if (ourCount == 0) {
       appendToList = PR_TRUE;
     } else {
-      nsIContent* ourLastContent =
-        NS_STATIC_CAST(nsIContent*, mElements.ElementAt(ourCount - 1));
+      nsIContent* ourLastContent = mElements[ourCount - 1];
       /*
        * We want to append instead of invalidating if the first thing
        * that got appended comes after ourLastContent.
        */
-      nsCOMPtr<nsIDOM3Node> ourLastDOM3Node(do_QueryInterface(ourLastContent));
-      if (ourLastDOM3Node) {
-        nsCOMPtr<nsIDOMNode> newNode =
-          do_QueryInterface(aContainer->GetChildAt(aNewIndexInContainer));
-        NS_ASSERTION(newNode, "Content being inserted is not a node.... why?");
-
-        PRUint16 comparisonFlags;
-        nsresult rv =
-          ourLastDOM3Node->CompareDocumentPosition(newNode, &comparisonFlags);
-        if (NS_SUCCEEDED(rv) && 
-            (comparisonFlags & nsIDOM3Node::DOCUMENT_POSITION_FOLLOWING)) {
-          appendToList = PR_TRUE;
-        }
+      if (nsContentUtils::PositionIsBefore(ourLastContent,
+                                           aContainer->GetChildAt(aNewIndexInContainer))) {
+        appendToList = PR_TRUE;
       }
     }
     
@@ -618,6 +588,7 @@ nsContentList::ContentAppended(nsIDocument *aDocument, nsIContent* aContainer,
         }
       }
  
+      ASSERT_IN_SYNC;
       return;
     }
 
@@ -636,8 +607,10 @@ nsContentList::ContentAppended(nsIDocument *aDocument, nsIContent* aContainer,
      */
     for (i = aNewIndexInContainer; i <= count-1; ++i) {
       PRUint32 limit = PRUint32(-1);
-      PopulateWith(aContainer->GetChildAt(i), PR_TRUE, limit);
+      PopulateWith(aContainer->GetChildAt(i), limit);
     }
+
+    ASSERT_IN_SYNC;
   }
 }
 
@@ -650,11 +623,14 @@ nsContentList::ContentInserted(nsIDocument *aDocument,
   // Note that aContainer can be null here if we are inserting into
   // the document itself; any attempted optimizations to this method
   // should deal with that.
-  if (mState == LIST_DIRTY || IsContentAnonymous(aChild))
-    return;
-
-  if (MayContainRelevantNodes(aContainer) && MatchSelf(aChild))
+  if (mState != LIST_DIRTY &&
+      MayContainRelevantNodes(NODE_FROM(aContainer, aDocument)) &&
+      !IsContentAnonymous(aChild) &&
+      MatchSelf(aChild)) {
     mState = LIST_DIRTY;
+  }
+
+  ASSERT_IN_SYNC;
 }
  
 void
@@ -666,28 +642,14 @@ nsContentList::ContentRemoved(nsIDocument *aDocument,
   // Note that aContainer can be null here if we are removing from
   // the document itself; any attempted optimizations to this method
   // should deal with that.
-  if (mState != LIST_DIRTY) {
-    if (MayContainRelevantNodes(aContainer)) {
-      if (!IsContentAnonymous(aChild) && MatchSelf(aChild)) {
-        mState = LIST_DIRTY;
-      }
-      return;
-    }
+  if (mState != LIST_DIRTY &&
+      MayContainRelevantNodes(NODE_FROM(aContainer, aDocument)) &&
+      !IsContentAnonymous(aChild) &&
+      MatchSelf(aChild)) {
+    mState = LIST_DIRTY;
   }
 
-  // Even if aChild is anonymous from our point of view, it could
-  // contain our root (eg say our root is an anonymous child of
-  // aChild).
-  if (ContainsRoot(aChild)) {
-    DisconnectFromDocument();
-  }
-}
-
-void
-nsContentList::DocumentWillBeDestroyed(nsIDocument *aDocument)
-{
-  DisconnectFromDocument();
-  Reset();
+  ASSERT_IN_SYNC;
 }
 
 PRBool
@@ -701,14 +663,17 @@ nsContentList::Match(nsIContent *aContent)
   }
 
   if (mMatchAtom) {
-    if (!aContent->IsContentOfType(nsIContent::eELEMENT)) {
+    if (!aContent->IsNodeOfType(nsINode::eELEMENT)) {
       return PR_FALSE;
     }
 
-    nsINodeInfo *ni = aContent->GetNodeInfo();
-    NS_ASSERTION(ni, "Element without nodeinfo!");
+    nsINodeInfo *ni = aContent->NodeInfo();
 
     if (mMatchNameSpaceId == kNameSpaceID_Unknown) {
+      return (mMatchAll || ni->QualifiedNameEquals(mMatchAtom));
+    }
+
+    if (mMatchNameSpaceId == kNameSpaceID_Wildcard) {
       return (mMatchAll || ni->Equals(mMatchAtom));
     }
 
@@ -719,24 +684,11 @@ nsContentList::Match(nsIContent *aContent)
   return PR_FALSE;
 }
 
-void
-nsContentList::CheckDocumentExistence()
-{
-  if (!mDocument && mRootContent) {
-    
-    mDocument = mRootContent->GetDocument();
-    if (mDocument) {
-      mDocument->AddObserver(this);
-      mState = LIST_DIRTY;
-    }
-  }
-}
-
 PRBool 
 nsContentList::MatchSelf(nsIContent *aContent)
 {
   NS_PRECONDITION(aContent, "Can't match null stuff, you know");
-  NS_PRECONDITION(mDeep || aContent->GetParent() == mRootContent,
+  NS_PRECONDITION(mDeep || aContent->GetNodeParent() == mRootNode,
                   "MatchSelf called on a node that we can't possibly match");
   
   if (Match(aContent))
@@ -756,52 +708,45 @@ nsContentList::MatchSelf(nsIContent *aContent)
   return PR_FALSE;
 }
 
-void 
-nsContentList::PopulateWith(nsIContent *aContent, PRBool aIncludeRoot,
-                            PRUint32 & aElementsToAppend)
+void
+nsContentList::PopulateWith(nsIContent *aContent, PRUint32& aElementsToAppend)
 {
-  NS_PRECONDITION(mDeep || aContent == mRootContent ||
-                  aContent->GetParent() == mRootContent,
+  NS_PRECONDITION(mDeep || aContent->GetNodeParent() == mRootNode,
                   "PopulateWith called on nodes we can't possibly match");
-  NS_PRECONDITION(mDeep || aIncludeRoot || aContent == mRootContent,
-                  "Bogus root passed to PopulateWith in non-deep list");
-  NS_PRECONDITION(!aIncludeRoot || aContent != mRootContent,
-                  "We should never be trying to match mRootContent");
-  
-  if (aIncludeRoot) {
-    if (Match(aContent)) {
-      mElements.AppendElement(aContent);
-      --aElementsToAppend;
-      if (aElementsToAppend == 0)
-        return;
-    }
+  NS_PRECONDITION(aContent != mRootNode,
+                  "We should never be trying to match mRootNode");
+
+  if (Match(aContent)) {
+    mElements.AppendObject(aContent);
+    --aElementsToAppend;
+    if (aElementsToAppend == 0)
+      return;
   }
 
-  // Don't recurse down if we're not doing a deep match and we're
-  // already looking at kids of the root.
-  if (!mDeep && aIncludeRoot)
+  // Don't recurse down if we're not doing a deep match.
+  if (!mDeep)
     return;
   
   PRUint32 i, count = aContent->GetChildCount();
 
   for (i = 0; i < count; i++) {
-    PopulateWith(aContent->GetChildAt(i), PR_TRUE, aElementsToAppend);
+    PopulateWith(aContent->GetChildAt(i), aElementsToAppend);
     if (aElementsToAppend == 0)
       return;
   }
 }
 
 void 
-nsContentList::PopulateWithStartingAfter(nsIContent *aStartRoot,
-                                         nsIContent *aStartChild,
+nsContentList::PopulateWithStartingAfter(nsINode *aStartRoot,
+                                         nsINode *aStartChild,
                                          PRUint32 & aElementsToAppend)
 {
-  NS_PRECONDITION(mDeep || aStartRoot == mRootContent ||
-                  (aStartRoot->GetParent() == mRootContent &&
+  NS_PRECONDITION(mDeep || aStartRoot == mRootNode ||
+                  (aStartRoot->GetNodeParent() == mRootNode &&
                    aStartChild == nsnull),
                   "Bogus aStartRoot or aStartChild");
 
-  if (mDeep || aStartRoot == mRootContent) {
+  if (mDeep || aStartRoot == mRootNode) {
 #ifdef DEBUG
     PRUint32 invariant = aElementsToAppend + mElements.Count();
 #endif
@@ -814,7 +759,7 @@ nsContentList::PopulateWithStartingAfter(nsIContent *aStartRoot,
 
     PRUint32 childCount = aStartRoot->GetChildCount();
     for ( ; ((PRUint32)i) < childCount; ++i) {
-      PopulateWith(aStartRoot->GetChildAt(i), PR_TRUE, aElementsToAppend);
+      PopulateWith(aStartRoot->GetChildAt(i), aElementsToAppend);
     
       NS_ASSERTION(aElementsToAppend + mElements.Count() == invariant,
                    "Something is awry in PopulateWith!");
@@ -825,10 +770,14 @@ nsContentList::PopulateWithStartingAfter(nsIContent *aStartRoot,
 
   // We want to make sure we don't move up past our root node. So if
   // we're there, don't move to the parent.
-  if (aStartRoot == mRootContent)
+  if (aStartRoot == mRootNode)
     return;
   
-  nsIContent* parent = aStartRoot->GetParent();
+  // We could call GetParent() here to avoid walking children of the
+  // document node. However they should be very few in number and we
+  // might want to walk them in the future so it's unnecessary to have
+  // this be the only thing that prevents it
+  nsINode* parent = aStartRoot->GetNodeParent();
   
   if (parent)
     PopulateWithStartingAfter(parent, aStartRoot, aElementsToAppend);
@@ -837,6 +786,12 @@ nsContentList::PopulateWithStartingAfter(nsIContent *aStartRoot,
 void 
 nsContentList::PopulateSelf(PRUint32 aNeededLength)
 {
+  if (!mRootNode) {
+    return;
+  }
+
+  ASSERT_IN_SYNC;
+
   if (mState == LIST_DIRTY) {
     Reset();
   }
@@ -849,93 +804,21 @@ nsContentList::PopulateSelf(PRUint32 aNeededLength)
 #ifdef DEBUG
   PRUint32 invariant = elementsToAppend + mElements.Count();
 #endif
-  if (count != 0) {
-    PopulateWithStartingAfter(NS_STATIC_CAST(nsIContent*,
-                                             mElements.ElementAt(count - 1)),
-                              nsnull,
-                              elementsToAppend);
-    NS_ASSERTION(elementsToAppend + mElements.Count() == invariant,
-                 "Something is awry in PopulateWithStartingAfter!");
-  } else if (mRootContent) {
-    PopulateWith(mRootContent, PR_FALSE, elementsToAppend);
-    NS_ASSERTION(elementsToAppend + mElements.Count() == invariant,
-                 "Something is awry in PopulateWith!");
-  }
-  else if (mDocument) {
-    nsIContent *root = mDocument->GetRootContent();
-    if (root) {
-      PopulateWith(root, PR_TRUE, elementsToAppend);
-      NS_ASSERTION(elementsToAppend + mElements.Count() == invariant,
-                   "Something is awry in PopulateWith!");
-    }
-  }
 
-  if (mDocument) {
-    if (elementsToAppend != 0)
-      mState = LIST_UP_TO_DATE;
-    else
-      mState = LIST_LAZY;
-  } else {
-    // No document means we have to stay on our toes since we don't
-    // get content notifications.
-    mState = LIST_DIRTY;
-  }
-}
+  // If we already have nodes start searching at the last one, otherwise
+  // start searching at the root.
+  nsINode* startRoot = count == 0 ? mRootNode : mElements[count - 1];
 
-PRBool
-nsContentList::MayContainRelevantNodes(nsIContent* aContainer) 
-{
-  if (!mRootContent) {
-#ifdef DEBUG
-    // aContainer can be null when ContentInserted/ContentRemoved are
-    // called, but we still want to return PR_TRUE in such cases if
-    // mRootContent is null.  We could pass the document into this
-    // method instead of trying to get it from aContainer, but that
-    // seems a little pointless just to run this debug-only integrity
-    // check.
-    if (aContainer) { 
-      NS_ASSERTION(aContainer->GetDocument() == mDocument,
-                   "We should not get in here if aContainer is in some _other_ document!");
-    }
-#endif
-    return PR_TRUE;
-  }
+  PopulateWithStartingAfter(startRoot, nsnull, elementsToAppend);
+  NS_ASSERTION(elementsToAppend + mElements.Count() == invariant,
+               "Something is awry in PopulateWith!");
 
-  if (!aContainer) {
-    return PR_FALSE;
-  }
+  if (elementsToAppend != 0)
+    mState = LIST_UP_TO_DATE;
+  else
+    mState = LIST_LAZY;
 
-  if (!mDeep) {
-    // We only care about cases when aContainer is our root content node.
-    return aContainer == mRootContent;
-  }
-  
-  return nsContentUtils::ContentIsDescendantOf(aContainer, mRootContent);
-}
-
-PRBool
-nsContentList::ContainsRoot(nsIContent* aContent)
-{
-  if (!mRootContent || !aContent) {
-    return PR_FALSE;
-  }
-
-  return nsContentUtils::ContentIsDescendantOf(mRootContent, aContent);
-}
-
-void 
-nsContentList::DisconnectFromDocument()
-{
-  if (mDocument) {
-    // Our key will change... Best remove ourselves before that happens.
-    RemoveFromHashtable();
-    mDocument->RemoveObserver(this);
-    mDocument = nsnull;
-  }
-
-  // We will get no more updates, so we can never know we're up to
-  // date
-  mState = LIST_DIRTY;
+  ASSERT_IN_SYNC;
 }
 
 void
@@ -962,15 +845,20 @@ nsContentList::RemoveFromHashtable()
 void
 nsContentList::BringSelfUpToDate(PRBool aDoFlush)
 {
-  if (mDocument && aDoFlush) {
-    // Flush pending content changes Bug 4891.
-    mDocument->FlushPendingNotifications(Flush_ContentAndNotify);
+  if (mRootNode && aDoFlush) {
+    // XXX sXBL/XBL2 issue
+    nsIDocument* doc = mRootNode->GetCurrentDoc();
+    if (doc) {
+      // Flush pending content changes Bug 4891.
+      doc->FlushPendingNotifications(Flush_ContentAndNotify);
+    }
   }
 
   if (mState != LIST_UP_TO_DATE)
     PopulateSelf(PRUint32(-1));
     
-  NS_ASSERTION(!mDocument || mState == LIST_UP_TO_DATE,
+  ASSERT_IN_SYNC;
+  NS_ASSERTION(!mRootNode || mState == LIST_UP_TO_DATE,
                "PopulateSelf dod not bring content list up to date!");
 }
 
@@ -988,34 +876,93 @@ nsContentList::IsContentAnonymous(nsIContent* aContent)
    * that node that it can see are not anonymous from our point of view.
    */
 
-  if (!mRootContent) {
+  if (!mRootNode->IsNodeOfType(nsINode::eCONTENT)) {
     /**
-     * If we have no root, that means we correspond to a list gotten off the
-     * "document" object, so will always contain only nodes that the document
-     * can see, which means only nodes with a null bindingParent.
+     * The root isn't an nsIContent, that means we correspond to a list gotten
+     * off the "document" object, so will always contain only nodes that the
+     * document can see, which means only nodes with a null bindingParent.
      */
     return aContent->GetBindingParent() != nsnull;
   }
 
   /**
-   * If mRootContent and aContent have _different_ bindingParents, then there
+   * If mRootNode and aContent have _different_ bindingParents, then there
    * are two possibilities:
    *
    * 1) aContent or one of its ancestors is an anonymous child of a descendant
-   *    of mRootContent (or of mRootContent itself).
-   * 2) mRootContent is not an ancestor of aContent.
+   *    of mRootNode (or of mRootNode itself).
+   * 2) mRootNode is not an ancestor of aContent.
    *
    * In either case, we don't want to be matching aContent or any of its
    * descendants.
    *
-   * On the other hand, if mRootContent and aContent have the same
+   * On the other hand, if mRootNode and aContent have the same
    * bindingParent then they are part of the same binding (or native anonymous
-   * content chunk) and then aContent may be a descendant of mRootContent and
+   * content chunk) and then aContent may be a descendant of mRootNode and
    * we may want to match it.
    *
    * So we return true if the binding parents don't match; if the binding
    * parents are the same, the checks we normally do to determine whether we
    * match a node should be done.
    */
-  return mRootContent->GetBindingParent() != aContent->GetBindingParent();
+  return NS_STATIC_CAST(nsIContent*, mRootNode)->GetBindingParent() !=
+         aContent->GetBindingParent();
 }
+
+#ifdef DEBUG_CONTENT_LIST
+void
+nsContentList::AssertInSync()
+{
+  if (mState == LIST_DIRTY) {
+    return;
+  }
+
+  if (!mRootNode) {
+    NS_ASSERTION(mElements.Count() == 0 && mState == LIST_DIRTY,
+                 "Empty iterator isn't quite empty?");
+    return;
+  }
+
+  // XXX This code will need to change if nsContentLists can ever match
+  // elements that are outside of the document element.
+  nsIContent *root;
+  if (mRootNode->IsNodeOfType(nsINode::eDOCUMENT)) {
+    root = NS_STATIC_CAST(nsIDocument*, mRootNode)->GetRootContent();
+  }
+  else {
+    root = NS_STATIC_CAST(nsIContent*, mRootNode);
+  }
+
+  nsCOMPtr<nsIContentIterator> iter;
+  if (mDeep) {
+    NS_NewPreContentIterator(getter_AddRefs(iter));
+    iter->Init(root);
+    iter->First();
+  }
+
+  PRInt32 cnt = 0, index = 0;
+  while (PR_TRUE) {
+    if (cnt == mElements.Count() && mState == LIST_LAZY) {
+      break;
+    }
+
+    nsIContent *cur = mDeep ? iter->GetCurrentNode() :
+                              mRootNode->GetChildAt(index++);
+    if (!cur) {
+      break;
+    }
+
+    if (Match(cur)) {
+      NS_ASSERTION(cnt < mElements.Count() && mElements[cnt] == cur,
+                   "Elements is out of sync");
+      ++cnt;
+    }
+
+    if (mDeep) {
+      iter->Next();
+    }
+  }
+
+  NS_ASSERTION(cnt == mElements.Count(), "Too few elements");
+}
+#endif
