@@ -35,6 +35,11 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * Base class for the XML and HTML content sinks, which construct a
+ * DOM based on information from the parser.
+ */
+
 #include "nsContentSink.h"
 #include "nsIScriptLoader.h"
 #include "nsIDocument.h"
@@ -44,8 +49,6 @@
 #include "nsINodeInfo.h"
 #include "nsIDocShell.h"
 #include "nsIDocShellTreeItem.h"
-#include "nsIWebNavigation.h"
-#include "nsIRefreshURI.h"
 #include "nsCPrefetchService.h"
 #include "nsIURI.h"
 #include "nsNetUtil.h"
@@ -63,13 +66,11 @@
 #include "nsHTMLAtoms.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIPrincipal.h"
-#include "nsIScriptSecurityManager.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsNetCID.h"
 #include "nsICookieService.h"
 #include "nsIPrompt.h"
-#include "nsIServiceManagerUtils.h"
-#include "nsICharsetConverterManager.h"
+#include "nsServiceManagerUtils.h"
 #include "nsContentUtils.h"
 #include "nsParserUtils.h"
 #include "nsCRT.h"
@@ -183,10 +184,8 @@ nsContentSink::Init(nsIDocument* aDoc,
   nsresult rv = loader->AddObserver(proxy);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  mCSSLoader = aDoc->GetCSSLoader();
+  mCSSLoader = aDoc->CSSLoader();
 
-  // XXX this presumes HTTP header info is already set in document
-  // XXX if it isn't we need to set it here...
   ProcessHTTPHeaders(aChannel);
 
   mNodeInfoManager = aDoc->NodeInfoManager();
@@ -195,8 +194,9 @@ nsContentSink::Init(nsIDocument* aDoc,
 }
 
 NS_IMETHODIMP
-nsContentSink::StyleSheetLoaded(nsICSSStyleSheet* aSheet, 
-                                PRBool aDidNotify)
+nsContentSink::StyleSheetLoaded(nsICSSStyleSheet* aSheet,
+                                PRBool aWasAlternate,
+                                nsresult aStatus)
 {
   return NS_OK;
 }
@@ -246,7 +246,7 @@ nsContentSink::ScriptAvailable(nsresult aResult,
       //     script load, assuming that that error code means that the user
       //     stopped the load through some action (like clicking a link). See
       //     http://bugzilla.mozilla.org/show_bug.cgi?id=243392.
-      mParser->ContinueParsing();
+      mParser->ContinueInterruptedParsing();
     }
   }
 
@@ -273,11 +273,11 @@ nsContentSink::ScriptEvaluated(nsresult aResult,
   mScriptElements.RemoveObjectAt(count - 1); 
 
   if (NS_SUCCEEDED(aResult)) {
-    PostEvaluateScript();
+    PostEvaluateScript(aElement);
   }
 
   if (mParser && mParser->IsParserEnabled() && aWasPending) {
-    mParser->ContinueParsing();
+    mParser->ContinueInterruptedParsing();
   }
 
   return NS_OK;
@@ -292,27 +292,16 @@ nsContentSink::ProcessHTTPHeaders(nsIChannel* aChannel)
     return NS_OK;
   }
 
-  static const char *const headers[] = {
-    "link",
-    "default-style",
-    "content-style-type",
-    "content-language",
-    // add more http headers if you need
-    // XXXbz don't add content-location support without reading bug
-    // 238654 and its dependencies/dups first.
-    0
-  };
+  // Note that the only header we care about is the "link" header, since we
+  // have all the infrastructure for kicking off stylesheet loads.
   
-  const char *const *name = headers;
-  nsCAutoString tmp;
+  nsCAutoString linkHeader;
   
-  while (*name) {
-    nsresult rv = httpchannel->GetResponseHeader(nsDependentCString(*name), tmp);
-    if (NS_SUCCEEDED(rv) && !tmp.IsEmpty()) {
-      nsCOMPtr<nsIAtom> key = do_GetAtom(*name);
-      ProcessHeaderData(key, NS_ConvertASCIItoUCS2(tmp));
-    }
-    ++name;
+  nsresult rv = httpchannel->GetResponseHeader(NS_LITERAL_CSTRING("link"),
+                                               linkHeader);
+  if (NS_SUCCEEDED(rv) && !linkHeader.IsEmpty()) {
+    ProcessHeaderData(nsHTMLAtoms::link,
+                      NS_ConvertASCIItoUTF16(linkHeader));
   }
   
   return NS_OK;
@@ -327,29 +316,11 @@ nsContentSink::ProcessHeaderData(nsIAtom* aHeader, const nsAString& aValue,
 
   mDocument->SetHeaderData(aHeader, aValue);
 
-  NS_ENSURE_TRUE(mDocShell, NS_ERROR_FAILURE);
-
-  // see if we have a refresh "header".
-  if (aHeader == nsHTMLAtoms::refresh) {
-    // first get our baseURI
-
-    nsCOMPtr<nsIURI> baseURI;
-    nsCOMPtr<nsIWebNavigation> webNav = do_QueryInterface(mDocShell);
-    rv = webNav->GetCurrentURI(getter_AddRefs(baseURI));
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-
-    nsCOMPtr<nsIRefreshURI> reefer = do_QueryInterface(mDocShell);
-    if (reefer) {
-      rv = reefer->SetupRefreshURIFromHeader(baseURI,
-                                             NS_ConvertUCS2toUTF8(aValue));
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-    }
-  }
-  else if (aHeader == nsHTMLAtoms::setcookie) {
+  if (aHeader == nsHTMLAtoms::setcookie) {
+    // Note: Necko already handles cookies set via the channel.  We can't just
+    // call SetCookie on the channel because we want to do some security checks
+    // here and want to use the prompt associated to our current window, not
+    // the window where the channel was dispatched.
     nsCOMPtr<nsICookieService> cookieServ =
       do_GetService(NS_COOKIESERVICE_CONTRACTID, &rv);
     if (NS_FAILED(rv)) {
@@ -361,24 +332,11 @@ nsContentSink::ProcessHeaderData(nsIAtom* aHeader, const nsAString& aValue,
     // We use the original codebase in case the codebase was changed
     // by SetDomain
 
-    nsIPrincipal *docPrincipal = mDocument->GetPrincipal();
-    if (!docPrincipal) {
-      return NS_ERROR_FAILURE;
-    }
-
-    nsCOMPtr<nsIPrincipal> systemPrincipal;
-    nsContentUtils::GetSecurityManager()->
-      GetSystemPrincipal(getter_AddRefs(systemPrincipal));
-    NS_ASSERTION(systemPrincipal, "No system principal");
-    
-    if (docPrincipal == systemPrincipal) {
-      // Document's principal is not a codebase, so we can't set cookies
-      return NS_OK;
-    }
-
+    // Note that a non-codebase principal (eg the system principal) will return
+    // a null URI.
     nsCOMPtr<nsIURI> codebaseURI;
-    rv = docPrincipal->GetURI(getter_AddRefs(codebaseURI));
-    NS_ENSURE_SUCCESS(rv, rv);
+    rv = mDocument->NodePrincipal()->GetURI(getter_AddRefs(codebaseURI));
+    NS_ENSURE_TRUE(codebaseURI, rv);
 
     nsCOMPtr<nsIPrompt> prompt;
     nsCOMPtr<nsIDOMWindowInternal> window (do_QueryInterface(mDocument->GetScriptGlobalObject()));
@@ -393,7 +351,7 @@ nsContentSink::ProcessHeaderData(nsIAtom* aHeader, const nsAString& aValue,
 
     rv = cookieServ->SetCookieString(codebaseURI,
                                      prompt,
-                                     NS_ConvertUCS2toUTF8(aValue).get(),
+                                     NS_ConvertUTF16toUTF8(aValue).get(),
                                      channel);
     if (NS_FAILED(rv)) {
       return rv;
@@ -404,6 +362,7 @@ nsContentSink::ProcessHeaderData(nsIAtom* aHeader, const nsAString& aValue,
   }
   else if (aHeader == nsHTMLAtoms::msthemecompatible) {
     // Disable theming for the presshell if the value is no.
+    // XXXbz don't we want to support this as an HTTP header too?
     nsAutoString value(aValue);
     if (value.LowerCaseEqualsLiteral("no")) {
       nsIPresShell* shell = mDocument->GetShellAt(0);
@@ -412,7 +371,9 @@ nsContentSink::ProcessHeaderData(nsIAtom* aHeader, const nsAString& aValue,
       }
     }
   }
-  else if (mParser) {
+  // Don't report "refresh" headers back to necko, since our document handles
+  // them
+  else if (aHeader != nsHTMLAtoms::refresh && mParser) {
     // we also need to report back HTTP-EQUIV headers to the channel
     // so that it can process things like pragma: no-cache or other
     // cache-control headers. Ideally this should also be the way for
@@ -425,7 +386,7 @@ nsContentSink::ProcessHeaderData(nsIAtom* aHeader, const nsAString& aValue,
         const char* header;
         (void)aHeader->GetUTF8String(&header);
         (void)httpChannel->SetResponseHeader(nsDependentCString(header),
-                                             NS_ConvertUCS2toUTF8(aValue),
+                                             NS_ConvertUTF16toUTF8(aValue),
                                              PR_TRUE);
       }
     }
@@ -621,9 +582,9 @@ nsContentSink::ProcessLinkHeader(nsIContent* aElement,
 
 nsresult
 nsContentSink::ProcessLink(nsIContent* aElement,
-                           const nsAString& aHref, const nsAString& aRel,
-                           const nsAString& aTitle, const nsAString& aType,
-                           const nsAString& aMedia)
+                           const nsSubstring& aHref, const nsSubstring& aRel,
+                           const nsSubstring& aTitle, const nsSubstring& aType,
+                           const nsSubstring& aMedia)
 {
   // XXX seems overkill to generate this string array
   nsStringArray linkTypes;
@@ -647,11 +608,11 @@ nsContentSink::ProcessLink(nsIContent* aElement,
 
 nsresult
 nsContentSink::ProcessStyleLink(nsIContent* aElement,
-                                const nsAString& aHref,
+                                const nsSubstring& aHref,
                                 PRBool aAlternate,
-                                const nsAString& aTitle,
-                                const nsAString& aType,
-                                const nsAString& aMedia)
+                                const nsSubstring& aTitle,
+                                const nsSubstring& aType,
+                                const nsSubstring& aMedia)
 {
   if (aAlternate && aTitle.IsEmpty()) {
     // alternates must have title return without error, for now
@@ -676,42 +637,15 @@ nsContentSink::ProcessStyleLink(nsIContent* aElement,
     return NS_OK;
   }
 
-  if (!aAlternate) {
-    // possibly preferred sheet
-
-    if (!aTitle.IsEmpty()) {
-      nsAutoString preferredStyle;
-      mDocument->GetHeaderData(nsHTMLAtoms::headerDefaultStyle,
-                               preferredStyle);
-      if (preferredStyle.IsEmpty()) {
-        mDocument->SetHeaderData(nsHTMLAtoms::headerDefaultStyle, aTitle);
-      }
-    }
-  }
-
-  PRBool blockParser = kBlockByDefault;
-  if (aAlternate) {
-    blockParser = PR_FALSE;
-  }
-
-  // NOTE: no longer honoring the important keyword to indicate
-  // blocking as it is proprietary and unnecessary since all
-  // non-alternate will block the parser now -mja
-#if 0
-  if (linkTypes.IndexOf("important") != -1) {
-    blockParser = PR_TRUE;
-  }
-#endif
-
-  PRBool doneLoading;
   nsIParser* parser = nsnull;
-  if (blockParser) {
+  if (kBlockByDefault) {
     parser = mParser;
   }
-  rv = mCSSLoader->LoadStyleLink(aElement, url, aTitle, aMedia,
-                                 parser, doneLoading, this);
-
-  if (NS_SUCCEEDED(rv) && blockParser && !doneLoading) {
+  
+  PRBool isAlternate;
+  rv = mCSSLoader->LoadStyleLink(aElement, url, aTitle, aMedia, aAlternate,
+                                 parser, this, &isAlternate);
+  if (NS_SUCCEEDED(rv) && parser && !isAlternate) {
     rv = NS_ERROR_HTMLPARSER_BLOCK;
   }
 
@@ -793,53 +727,6 @@ nsContentSink::PrefetchHref(const nsAString &aHref, PRBool aExplicit)
 }
 
 
-// Convert the ref from document charset to unicode.
-static nsresult
-CharsetConvRef(const nsACString& aDocCharset,
-               const nsCString& aRefInDocCharset,
-               nsString& aRefInUnicode)
-{
-  nsresult rv;
-
-  nsCOMPtr <nsIAtom> docCharsetAtom;
-  nsCOMPtr<nsICharsetConverterManager> ccm =
-    do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIUnicodeDecoder> decoder;
-  rv = ccm->GetUnicodeDecoder(PromiseFlatCString(aDocCharset).get(),
-                              getter_AddRefs(decoder));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  PRInt32 srcLen = aRefInDocCharset.Length();
-  PRInt32 dstLen;
-  rv = decoder->GetMaxLength(aRefInDocCharset.get(), srcLen, &dstLen);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  PRUnichar *ustr = (PRUnichar *)nsMemory::Alloc((dstLen + 1) *
-                                                 sizeof(PRUnichar));
-  if (!ustr) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  rv = decoder->Convert(aRefInDocCharset.get(), &srcLen, ustr, &dstLen);
-  if (NS_SUCCEEDED(rv)) {
-    ustr[dstLen] = 0;
-    aRefInUnicode.Assign(ustr, dstLen);
-  }
-
-  nsMemory::Free(ustr);
-
-  return rv;
-}
-
 PRBool
 nsContentSink::ScrollToRef(PRBool aReallyScroll)
 {
@@ -862,7 +749,7 @@ nsContentSink::ScrollToRef(PRBool aReallyScroll)
   nsresult rv = NS_ERROR_FAILURE;
   // We assume that the bytes are in UTF-8, as it says in the spec:
   // http://www.w3.org/TR/html4/appendix/notes.html#h-B.2.1
-  NS_ConvertUTF8toUCS2 ref(unescapedRef);
+  NS_ConvertUTF8toUTF16 ref(unescapedRef);
 
   PRInt32 i, ns = mDocument->GetNumberOfShells();
   for (i = 0; i < ns; i++) {
@@ -882,7 +769,7 @@ nsContentSink::ScrollToRef(PRBool aReallyScroll)
       if (NS_FAILED(rv)) {
         const nsACString &docCharset = mDocument->GetDocumentCharacterSet();
 
-        rv = CharsetConvRef(docCharset, unescapedRef, ref);
+        rv = nsContentUtils::ConvertStringFromCharset(docCharset, unescapedRef, ref);
 
         if (NS_SUCCEEDED(rv) && !ref.IsEmpty())
           rv = shell->GoToAnchor(ref, aReallyScroll);
@@ -948,7 +835,10 @@ nsContentSink::StartLayout(PRBool aIsFrameset)
 
       // Resize-reflow this time
       nsRect r = shell->GetPresContext()->GetVisibleArea();
-      shell->InitialReflow(r.width, r.height);
+      nsresult rv = shell->InitialReflow(r.width, r.height);
+      if (NS_FAILED(rv)) {
+        return;
+      }
 
       // Now trigger a refresh
       RefreshIfEnabled(shell->GetViewManager());
