@@ -35,10 +35,14 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * Implementation of DOM Core's nsIDOMAttr node.
+ */
+
 #include "nsDOMAttribute.h"
 #include "nsGenericElement.h"
 #include "nsIContent.h"
-#include "nsITextContent.h"
+#include "nsContentCreatorFunctions.h"
 #include "nsINameSpaceManager.h"
 #include "nsDOMError.h"
 #include "nsContentUtils.h"
@@ -46,13 +50,20 @@
 #include "nsDOMString.h"
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
+#include "nsIDOM3Attr.h"
+#include "nsIDOMUserDataHandler.h"
+#include "nsEventDispatcher.h"
+#include "nsGkAtoms.h"
+#include "nsCOMArray.h"
+#include "nsNodeUtils.h"
 
 //----------------------------------------------------------------------
+PRBool nsDOMAttribute::sInitialized;
 
-nsDOMAttribute::nsDOMAttribute(nsIContent* aContent, nsINodeInfo *aNodeInfo,
-                               const nsAString& aValue)
-  : nsIAttribute(aContent, aNodeInfo), mValue(aValue), mChild(nsnull),
-    mChildList(nsnull)
+nsDOMAttribute::nsDOMAttribute(nsDOMAttributeMap *aAttrMap,
+                               nsINodeInfo       *aNodeInfo,
+                               const nsAString   &aValue)
+  : nsIAttribute(aAttrMap, aNodeInfo), mValue(aValue), mChildList(nsnull)
 {
   NS_ABORT_IF_FALSE(mNodeInfo, "We must get a nodeinfo here!");
 
@@ -63,8 +74,10 @@ nsDOMAttribute::nsDOMAttribute(nsIContent* aContent, nsINodeInfo *aNodeInfo,
 
 nsDOMAttribute::~nsDOMAttribute()
 {
-  NS_IF_RELEASE(mChild);
-  NS_IF_RELEASE(mChildList);
+  if (mChildList) {
+    mChildList->DropReference();
+    NS_RELEASE(mChildList);
+  }
 }
 
 
@@ -72,67 +85,130 @@ nsDOMAttribute::~nsDOMAttribute()
 NS_INTERFACE_MAP_BEGIN(nsDOMAttribute)
   NS_INTERFACE_MAP_ENTRY(nsIDOMAttr)
   NS_INTERFACE_MAP_ENTRY(nsIAttribute)
+  NS_INTERFACE_MAP_ENTRY(nsINode)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMGCParticipant)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNode)
   NS_INTERFACE_MAP_ENTRY(nsIDOM3Node)
+  NS_INTERFACE_MAP_ENTRY(nsIDOM3Attr)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMAttr)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(Attr)
 NS_INTERFACE_MAP_END
 
 
 NS_IMPL_ADDREF(nsDOMAttribute)
-NS_IMPL_RELEASE(nsDOMAttribute)
+NS_IMPL_RELEASE_WITH_DESTROY(nsDOMAttribute,
+                             nsNodeUtils::LastRelease(this, PR_TRUE))
 
+// nsIDOMGCParticipant methods
+nsIDOMGCParticipant*
+nsDOMAttribute::GetSCCIndex()
+{
+  nsIContent *owner = GetContentInternal();
+
+  return owner ? owner->GetSCCIndex() : this;
+}
+
+void
+nsDOMAttribute::AppendReachableList(nsCOMArray<nsIDOMGCParticipant>& aArray)
+{
+  NS_ASSERTION(GetContentInternal() == nsnull,
+               "shouldn't be an SCC index if we're in an element");
+
+  // This node is the root of a subtree that's been removed from the
+  // document (since AppendReachableList is only called on SCC index
+  // nodes).  The document is reachable from it (through
+  // .ownerDocument), but it's not reachable from the document.
+  aArray.AppendObject(GetOwnerDoc());
+}
+
+void
+nsDOMAttribute::SetMap(nsDOMAttributeMap *aMap)
+{
+  if (mAttrMap && !aMap && sInitialized) {
+    // We're breaking a relationship with content and not getting a new one,
+    // need to locally cache value. GetValue() does that.
+    GetValue(mValue);
+  }
+  
+  mAttrMap = aMap;
+}
+
+nsIContent*
+nsDOMAttribute::GetContent() const
+{
+  return GetContentInternal();
+}
 
 nsresult
+nsDOMAttribute::SetOwnerDocument(nsIDocument* aDocument)
+{
+  NS_ASSERTION(aDocument, "Missing document");
+
+  nsIDocument *doc = GetOwnerDoc();
+  NS_ASSERTION(doc != aDocument, "bad call to nsDOMAttribute::SetOwnerDocument");
+  if (doc) {
+    doc->PropertyTable()->DeleteAllPropertiesFor(this);
+  }
+
+  nsCOMPtr<nsINodeInfo> newNodeInfo;
+  nsresult rv = aDocument->NodeInfoManager()->
+    GetNodeInfo(mNodeInfo->NameAtom(), mNodeInfo->GetPrefixAtom(),
+                mNodeInfo->NamespaceID(), getter_AddRefs(newNodeInfo));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ASSERTION(newNodeInfo, "GetNodeInfo lies");
+  mNodeInfo.swap(newNodeInfo);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
 nsDOMAttribute::GetName(nsAString& aName)
 {
   mNodeInfo->GetQualifiedName(aName);
   return NS_OK;
 }
 
-nsresult
+NS_IMETHODIMP
 nsDOMAttribute::GetValue(nsAString& aValue)
 {
-  if (mContent) {
-    nsAutoString tmpValue;
-    nsresult attrResult = mContent->GetAttr(mNodeInfo->NamespaceID(),
-                                            mNodeInfo->NameAtom(),
-                                            tmpValue);
-    if (NS_CONTENT_ATTR_NOT_THERE != attrResult) {
-      mValue = tmpValue;
-    }
+  nsIContent* content = GetContentInternal();
+  if (content) {
+    content->GetAttr(mNodeInfo->NamespaceID(), mNodeInfo->NameAtom(), aValue);
+  }
+  else {
+    aValue = mValue;
   }
-
-  aValue = mValue;
 
   return NS_OK;
 }
 
-nsresult
+NS_IMETHODIMP
 nsDOMAttribute::SetValue(const nsAString& aValue)
 {
-  nsresult result = NS_OK;
-  if (mContent) {
-    result = mContent->SetAttr(mNodeInfo->NamespaceID(), mNodeInfo->NameAtom(),
-                               mNodeInfo->GetPrefixAtom(), aValue, PR_TRUE);
+  nsresult rv = NS_OK;
+  nsIContent* content = GetContentInternal();
+  if (content) {
+    rv = content->SetAttr(mNodeInfo->NamespaceID(),
+                          mNodeInfo->NameAtom(),
+                          mNodeInfo->GetPrefixAtom(),
+                          aValue,
+                          PR_TRUE);
+  }
+  else {
+    mValue = aValue;
   }
-  mValue=aValue;
 
-  return result;
+  return rv;
 }
 
-nsresult
+NS_IMETHODIMP
 nsDOMAttribute::GetSpecified(PRBool* aSpecified)
 {
   NS_ENSURE_ARG_POINTER(aSpecified);
 
-  if (!mContent) {
-    *aSpecified = PR_FALSE;
-    return NS_OK;
-  }
-
-  *aSpecified = mContent->HasAttr(mNodeInfo->NamespaceID(),
-                                  mNodeInfo->NameAtom());
+  nsIContent* content = GetContentInternal();
+  *aSpecified = content && content->HasAttr(mNodeInfo->NamespaceID(),
+                                            mNodeInfo->NameAtom());
 
   return NS_OK;
 }
@@ -142,8 +218,9 @@ nsDOMAttribute::GetOwnerElement(nsIDOMElement** aOwnerElement)
 {
   NS_ENSURE_ARG_POINTER(aOwnerElement);
 
-  if (mContent) {
-    return CallQueryInterface(mContent, aOwnerElement);
+  nsIContent* content = GetContentInternal();
+  if (content) {
+    return CallQueryInterface(content, aOwnerElement);
   }
 
   *aOwnerElement = nsnull;
@@ -203,18 +280,11 @@ nsDOMAttribute::GetChildNodes(nsIDOMNodeList** aChildNodes)
 NS_IMETHODIMP
 nsDOMAttribute::HasChildNodes(PRBool* aHasChildNodes)
 {
-  *aHasChildNodes = PR_FALSE;
-  if (mChild) {
-    *aHasChildNodes = PR_TRUE;
-  }
-  else if (mContent) {
-    nsAutoString value;
+  PRBool hasChild;
+  nsresult rv = EnsureChildState(PR_FALSE, hasChild);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    GetValue(value);
-    if (!value.IsEmpty()) {
-      *aHasChildNodes = PR_TRUE;
-    }
-  }
+  *aHasChildNodes = hasChild;
 
   return NS_OK;
 }
@@ -232,31 +302,17 @@ nsDOMAttribute::HasAttributes(PRBool* aHasAttributes)
 NS_IMETHODIMP
 nsDOMAttribute::GetFirstChild(nsIDOMNode** aFirstChild)
 {
-  nsAutoString value;
-  nsresult result;
+  *aFirstChild = nsnull;
 
-  result = GetValue(value);
-  if (NS_OK != result) {
-    return result;
-  }
-  if (!value.IsEmpty()) {
-    if (!mChild) {
-      nsCOMPtr<nsITextContent> content;
+  PRBool hasChild;
+  nsresult rv = EnsureChildState(PR_TRUE, hasChild);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-      result = NS_NewTextNode(getter_AddRefs(content));
-      if (NS_FAILED(result)) {
-        return result;
-      }
-      // XXX We should be setting |this| as the parent of the textnode!
-      result = CallQueryInterface(content, &mChild);
-    }
-    mChild->SetData(value);
-    result = CallQueryInterface(mChild, aFirstChild);
+  if (mChild) {
+    CallQueryInterface(mChild, aFirstChild);
   }
-  else {
-    *aFirstChild = nsnull;
-  }
-  return result;
+  
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -295,47 +351,47 @@ nsDOMAttribute::GetAttributes(nsIDOMNamedNodeMap** aAttributes)
 NS_IMETHODIMP
 nsDOMAttribute::InsertBefore(nsIDOMNode* aNewChild, nsIDOMNode* aRefChild, nsIDOMNode** aReturn)
 {
-  return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
 nsDOMAttribute::ReplaceChild(nsIDOMNode* aNewChild, nsIDOMNode* aOldChild, nsIDOMNode** aReturn)
 {
-  return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
 nsDOMAttribute::RemoveChild(nsIDOMNode* aOldChild, nsIDOMNode** aReturn)
 {
-  return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
 nsDOMAttribute::AppendChild(nsIDOMNode* aNewChild, nsIDOMNode** aReturn)
 {
-  return NS_ERROR_DOM_NO_MODIFICATION_ALLOWED_ERR;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP
-nsDOMAttribute::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
+nsresult
+nsDOMAttribute::Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const
 {
-  nsDOMAttribute* newAttr;
+  nsAutoString value;
+  NS_CONST_CAST(nsDOMAttribute*, this)->GetValue(value);
 
-  if (mContent) {
-    nsAutoString value;
-    mContent->GetAttr(mNodeInfo->NamespaceID(),
-                      mNodeInfo->NameAtom(), value);
-    newAttr = new nsDOMAttribute(nsnull, mNodeInfo, value);
-  }
-  else {
-    newAttr = new nsDOMAttribute(nsnull, mNodeInfo, mValue);
-  }
-
-  if (!newAttr) {
+  *aResult = new nsDOMAttribute(nsnull, aNodeInfo, value);
+  if (!*aResult) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  return CallQueryInterface(newAttr, aReturn);
+  NS_ADDREF(*aResult);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMAttribute::CloneNode(PRBool aDeep, nsIDOMNode** aResult)
+{
+  return nsNodeUtils::CloneNodeImpl(this, aDeep, aResult);
 }
 
 NS_IMETHODIMP
@@ -343,21 +399,9 @@ nsDOMAttribute::GetOwnerDocument(nsIDOMDocument** aOwnerDocument)
 {
   *aOwnerDocument = nsnull;
 
-  nsresult rv = NS_OK;
-  if (mContent) {
-    nsCOMPtr<nsIDOMNode> node = do_QueryInterface(mContent, &rv);
-    if (NS_SUCCEEDED(rv)) {
-      rv = node->GetOwnerDocument(aOwnerDocument);
-    }
-  }
-  else {
-    nsIDocument *document = mNodeInfo->GetDocument();
-    if (document) {
-      rv = CallQueryInterface(document, aOwnerDocument);
-    }
-  }
+  nsIDocument *document = GetOwnerDoc();
 
-  return rv;
+  return document ? CallQueryInterface(document, aOwnerDocument) : NS_OK;
 }
 
 NS_IMETHODIMP
@@ -376,28 +420,38 @@ nsDOMAttribute::GetPrefix(nsAString& aPrefix)
 NS_IMETHODIMP
 nsDOMAttribute::SetPrefix(const nsAString& aPrefix)
 {
+  // XXX: Validate the prefix string!
+
   nsCOMPtr<nsINodeInfo> newNodeInfo;
   nsCOMPtr<nsIAtom> prefix;
 
   if (!aPrefix.IsEmpty()) {
     prefix = do_GetAtom(aPrefix);
+    if (!prefix) {
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+  }
+
+  if (!nsContentUtils::IsValidNodeName(mNodeInfo->NameAtom(), prefix,
+                                       mNodeInfo->NamespaceID())) {
+    return NS_ERROR_DOM_NAMESPACE_ERR;
   }
 
   nsresult rv = nsContentUtils::PrefixChanged(mNodeInfo, prefix,
                                               getter_AddRefs(newNodeInfo));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (mContent) {
+  nsIContent* content = GetContentInternal();
+  if (content) {
     nsIAtom *name = mNodeInfo->NameAtom();
     PRInt32 nameSpaceID = mNodeInfo->NamespaceID();
 
     nsAutoString tmpValue;
-    rv = mContent->GetAttr(nameSpaceID, name, tmpValue);
-    if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
-      mContent->UnsetAttr(nameSpaceID, name, PR_TRUE);
+    if (content->GetAttr(nameSpaceID, name, tmpValue)) {
+      content->UnsetAttr(nameSpaceID, name, PR_TRUE);
 
-      mContent->SetAttr(newNodeInfo->NamespaceID(), newNodeInfo->NameAtom(),
-                        newNodeInfo->GetPrefixAtom(), tmpValue, PR_TRUE);
+      content->SetAttr(newNodeInfo->NamespaceID(), newNodeInfo->NameAtom(),
+                       newNodeInfo->GetPrefixAtom(), tmpValue, PR_TRUE);
     }
   }
 
@@ -425,7 +479,8 @@ nsDOMAttribute::IsSupported(const nsAString& aFeature,
                             const nsAString& aVersion,
                             PRBool* aReturn)
 {
-  return nsGenericElement::InternalIsSupported(aFeature, aVersion, aReturn);
+  return nsGenericElement::InternalIsSupported(NS_STATIC_CAST(nsIDOMAttr*, this), 
+                                               aFeature, aVersion, aReturn);
 }
 
 NS_IMETHODIMP
@@ -433,7 +488,7 @@ nsDOMAttribute::GetBaseURI(nsAString &aURI)
 {
   aURI.Truncate();
   nsresult rv = NS_OK;
-  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(mContent));
+  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(GetContentInternal()));
   if (node)
     rv = node->GetBaseURI(aURI);
   return rv;
@@ -444,117 +499,11 @@ nsDOMAttribute::CompareDocumentPosition(nsIDOMNode* aOther,
                                         PRUint16* aReturn)
 {
   NS_ENSURE_ARG_POINTER(aOther);
-  NS_PRECONDITION(aReturn, "Must have an out parameter");
 
-  PRUint16 mask = 0;
+  nsCOMPtr<nsINode> other = do_QueryInterface(aOther);
+  NS_ENSURE_TRUE(other, NS_ERROR_DOM_NOT_SUPPORTED_ERR);
 
-  nsCOMPtr<nsIDOMElement> el;
-  GetOwnerElement(getter_AddRefs(el));
-  if (!el) {
-    // If we have no owner element then there is no common container node,
-    // (of course there isn't if we have no container!) and the order is
-    // then based upon order between the root container of each node that
-    // is in no container. In this case, the result is disconnected
-    // and implementation-dependent.
-    mask |= (nsIDOM3Node::DOCUMENT_POSITION_DISCONNECTED |
-             nsIDOM3Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC);
-
-    *aReturn = mask;
-
-    return NS_OK;
-  }
-
-  // Check to see if the other node is also an attribute
-  PRUint16 nodeType = 0;
-  aOther->GetNodeType(&nodeType);
-  if (nodeType == nsIDOMNode::ATTRIBUTE_NODE) {
-    nsCOMPtr<nsIDOMAttr> otherAttr(do_QueryInterface(aOther));
-    nsCOMPtr<nsIDOMElement> otherEl;
-    otherAttr->GetOwnerElement(getter_AddRefs(otherEl));
-    if (el == otherEl) {
-      PRBool sameNode = PR_FALSE;
-      IsSameNode(aOther, &sameNode);
-      if (!sameNode) {
-        // If neither of the two determining nodes is a child node and
-        // nodeType is the same for both determining nodes, then an
-        // implementation-dependent order between the determining nodes
-        // is returned.
-        mask |= nsIDOM3Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC;
-      }
-
-      // If the two nodes being compared are the same node,
-      // then no flags are set on the return.
-    }
-
-    *aReturn = mask;
-
-    return NS_OK;
-  }
-
-  PRBool sameNode = PR_FALSE;
-
-  if (nodeType == nsIDOMNode::TEXT_NODE ||
-      nodeType == nsIDOMNode::CDATA_SECTION_NODE ||
-      nodeType == nsIDOMNode::ENTITY_REFERENCE_NODE) {
-    // XXXcaa we really should check the other node's parentNode
-    // against ourselves.  But since we can't, we should walk our
-    // child node list to see if it's a descendant.  But wait!
-    // We suck so bad that we cannot even do that, since we store
-    // only one text node per attribute, even if there are multiple.
-    // So technically, we could walk the child nodes list, but it
-    // would not make sense really to walk it for only one thing.
-    // How lame.  So.... it seems the only option that we DO have
-    // is to get our one and only child and compare it against the
-    // other node.  As such, that is exactly what we'll do.
-    // *Sigh*  These silly hacks are quite disgusting, really....
-
-    nsCOMPtr<nsIDOMNode> ourOnlyChild;
-    GetFirstChild(getter_AddRefs(ourOnlyChild));
-
-    nsCOMPtr<nsIDOM3Node> longLostRelative(do_QueryInterface(aOther));
-    NS_ASSERTION(longLostRelative, "All our data nodes support DOM3Node");
-
-    longLostRelative->IsSameNode(ourOnlyChild, &sameNode);
-    if (sameNode) {
-      // Woohoo!  We found our long lost relative and it's our child!
-      // Throw a party!  Celebrate by returning that it is contained
-      // and following this node.
-
-      mask |= (nsIDOM3Node::DOCUMENT_POSITION_CONTAINED_BY |
-               nsIDOM3Node::DOCUMENT_POSITION_FOLLOWING);
-
-      *aReturn = mask;
-      return NS_OK;
-    }
-
-    // Sigh.  The other node isn't our child, but it still may be
-    // related to us.  Fall through so we can keep looking.
-  }
-
-
-  // The other node isn't an attribute, or a child.
-  // Compare position relative to this attribute's owner element.
-
-  nsCOMPtr<nsIDOM3Node> parent(do_QueryInterface(el));
-  parent->IsSameNode(aOther, &sameNode);
-  if (sameNode) {
-    // If the other node contains us, then it precedes us.
-    mask |= (nsIDOM3Node::DOCUMENT_POSITION_CONTAINS |
-             nsIDOM3Node::DOCUMENT_POSITION_PRECEDING);
-
-    *aReturn = mask;
-    return NS_OK;
-  }
-
-  PRUint16 parentMask = 0;
-  parent->CompareDocumentPosition(aOther, &parentMask);
-
-  // We already established earlier that the node is not contained
-  // by this attribute.  So if it is contained by our owner element,
-  // unset the flag.
-  mask |= parentMask & ~nsIDOM3Node::DOCUMENT_POSITION_CONTAINED_BY;
-
-  *aReturn = mask;
+  *aReturn = nsContentUtils::ComparePosition(other, this);
   return NS_OK;
 }
 
@@ -562,46 +511,10 @@ NS_IMETHODIMP
 nsDOMAttribute::IsSameNode(nsIDOMNode* aOther,
                            PRBool* aReturn)
 {
-  PRBool sameNode = PR_FALSE;
+  NS_ASSERTION(aReturn, "IsSameNode() called with aReturn == nsnull!");
+  
+  *aReturn = SameCOMIdentity(NS_STATIC_CAST(nsIDOMNode*, this), aOther);
 
-  // XXXcaa Comparing pointers on two attributes is not yet reliable.
-  // When bug 93614 is fixed, this should be changed to simple pointer
-  // comparisons. But for now, check owner elements and node names.
-  PRUint16 otherType = 0;
-  aOther->GetNodeType(&otherType);
-  if (nsIDOMNode::ATTRIBUTE_NODE == otherType) {
-    nsCOMPtr<nsIDOMElement> nodeOwner;
-    GetOwnerElement(getter_AddRefs(nodeOwner));
-    nsCOMPtr<nsIDOMAttr> other(do_QueryInterface(aOther));
-    nsCOMPtr<nsIDOMElement> otherOwner;
-    other->GetOwnerElement(getter_AddRefs(otherOwner));
-
-    // Do these attributes belong to the same element?
-    if (nodeOwner == otherOwner) {
-      PRBool ci = PR_FALSE;
-      nsCOMPtr<nsIContent> content(do_QueryInterface(nodeOwner));
-      // Check to see if we're in HTML.
-      if (content->IsContentOfType(nsIContent::eHTML)) {
-        nsINodeInfo *ni = content->GetNodeInfo();
-        if (ni) {
-          // If there is no namespace, we're in HTML (as opposed to XHTML)
-          // and we'll need to compare node names case insensitively.
-          ci = ni->NamespaceEquals(kNameSpaceID_None);
-        }
-      }
-
-      nsAutoString nodeName;
-      nsAutoString otherName;
-      GetNodeName(nodeName);
-      aOther->GetNodeName(otherName);
-      // Compare node names
-      sameNode = ci ? nodeName.Equals(otherName,
-                                      nsCaseInsensitiveStringComparator())
-                    : nodeName.Equals(otherName);
-    }
-  }
-
-  *aReturn = sameNode;
   return NS_OK;
 }
 
@@ -609,18 +522,46 @@ NS_IMETHODIMP
 nsDOMAttribute::IsEqualNode(nsIDOMNode* aOther,
                             PRBool* aReturn)
 {
-  NS_NOTYETIMPLEMENTED("nsDocument::IsEqualNode()");
+  NS_ENSURE_ARG_POINTER(aOther);
 
-  return NS_ERROR_NOT_IMPLEMENTED;
+  *aReturn = PR_FALSE;
+
+  // Node type check by QI.  We also reuse this later.
+  nsCOMPtr<nsIAttribute> aOtherAttr = do_QueryInterface(aOther);
+  if (!aOtherAttr) {
+    return NS_OK;
+  }
+
+  // Prefix, namespace URI, local name, node name check.
+  if (!mNodeInfo->Equals(aOtherAttr->NodeInfo())) {
+    return NS_OK;
+  }
+
+  // Value check
+  nsAutoString ourValue, otherValue;
+  nsresult rv = GetValue(ourValue);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = aOther->GetNodeValue(otherValue);
+  NS_ENSURE_SUCCESS(rv, rv);
+  if (!ourValue.Equals(otherValue))
+    return NS_OK;
+
+  // Checks not needed:  Child nodes, attributes.
+
+  *aReturn = PR_TRUE;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsDOMAttribute::IsDefaultNamespace(const nsAString& aNamespaceURI,
                                    PRBool* aReturn)
 {
-  NS_NOTYETIMPLEMENTED("nsDOMAttribute::IsDefaultNamespace()");
-
-  return NS_ERROR_NOT_IMPLEMENTED;
+  *aReturn = PR_FALSE;
+  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(GetContentInternal()));
+  if (node) {
+    return node->IsDefaultNamespace(aNamespaceURI, aReturn);
+  }
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -641,28 +582,64 @@ nsDOMAttribute::GetFeature(const nsAString& aFeature,
                            const nsAString& aVersion,
                            nsISupports** aReturn)
 {
-  NS_NOTYETIMPLEMENTED("nsDocument::GetFeature()");
-
-  return NS_ERROR_NOT_IMPLEMENTED;
+  return nsGenericElement::InternalGetFeature(NS_STATIC_CAST(nsIDOMAttr*, this), 
+                                              aFeature, aVersion, aReturn);
 }
 
 NS_IMETHODIMP
-nsDOMAttribute::SetUserData(const nsAString& aKey,
-                            nsIVariant* aData,
+nsDOMAttribute::SetUserData(const nsAString& aKey, nsIVariant* aData,
                             nsIDOMUserDataHandler* aHandler,
-                            nsIVariant** aReturn)
+                            nsIVariant** aResult)
 {
-  NS_NOTYETIMPLEMENTED("nsDocument::SetUserData()");
+  nsCOMPtr<nsIAtom> key = do_GetAtom(aKey);
+  if (!key) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
 
-  return NS_ERROR_NOT_IMPLEMENTED;
+  return nsContentUtils::SetUserData(this, key, aData, aHandler, aResult);
 }
 
 NS_IMETHODIMP
-nsDOMAttribute::GetUserData(const nsAString& aKey,
-                            nsIVariant** aReturn)
+nsDOMAttribute::GetUserData(const nsAString& aKey, nsIVariant** aResult)
 {
-  NS_NOTYETIMPLEMENTED("nsDocument::GetUserData()");
+  nsIDocument *document = GetOwnerDoc();
+  NS_ENSURE_TRUE(document, NS_ERROR_FAILURE);
 
+  nsCOMPtr<nsIAtom> key = do_GetAtom(aKey);
+  if (!key) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  *aResult = NS_STATIC_CAST(nsIVariant*, GetProperty(DOM_USER_DATA, key));
+  NS_IF_ADDREF(*aResult);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMAttribute::GetIsId(PRBool* aReturn)
+{
+  nsIContent* content = GetContentInternal();
+  if (!content)
+  {
+    *aReturn = PR_FALSE;
+    return NS_OK;
+  }
+
+  nsIAtom* idAtom = content->GetIDAttributeName();
+  if (!idAtom)
+  {
+    *aReturn = PR_FALSE;
+    return NS_OK;
+  }
+
+  *aReturn = mNodeInfo->Equals(idAtom, kNameSpaceID_None);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMAttribute::GetSchemaTypeInfo(nsIDOM3TypeInfo** aReturn)
+{
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -670,13 +647,11 @@ NS_IMETHODIMP
 nsDOMAttribute::LookupPrefix(const nsAString& aNamespaceURI,
                              nsAString& aPrefix)
 {
-  aPrefix.Truncate();
-
-  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(mContent));
-  if (node) {
+  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(GetContentInternal()));
+  if (node)
     return node->LookupPrefix(aNamespaceURI, aPrefix);
-  }
 
+  SetDOMStringToNull(aPrefix);
   return NS_OK;
 }
 
@@ -684,12 +659,131 @@ NS_IMETHODIMP
 nsDOMAttribute::LookupNamespaceURI(const nsAString& aNamespacePrefix,
                                    nsAString& aNamespaceURI)
 {
-  aNamespaceURI.Truncate();
-  nsresult rv = NS_OK;
-  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(mContent));
+  nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(GetContentInternal()));
   if (node)
-    rv = node->LookupNamespaceURI(aNamespacePrefix, aNamespaceURI);
-  return rv;
+    return node->LookupNamespaceURI(aNamespacePrefix, aNamespaceURI);
+
+  SetDOMStringToNull(aNamespaceURI);
+  return NS_OK;
+}
+
+PRBool
+nsDOMAttribute::IsNodeOfType(PRUint32 aFlags) const
+{
+    return !(aFlags & ~eATTRIBUTE);
+}
+
+PRUint32
+nsDOMAttribute::GetChildCount() const
+{
+  PRBool hasChild;
+  EnsureChildState(PR_FALSE, hasChild);
+
+  return hasChild ? 1 : 0;
+}
+
+nsIContent *
+nsDOMAttribute::GetChildAt(PRUint32 aIndex) const
+{
+  // Don't need to check result of EnsureChildState since mChild will be null.
+  PRBool hasChild;
+  EnsureChildState(PR_TRUE, hasChild);
+
+  return aIndex == 0 && hasChild ? mChild.get() : nsnull;
+}
+  
+PRInt32
+nsDOMAttribute::IndexOf(nsINode* aPossibleChild) const
+{
+  // No need to call EnsureChildState here. If we don't already have a child
+  // then aPossibleChild can't possibly be our child.
+  if (!aPossibleChild || aPossibleChild != mChild) {
+    return -1;
+  }
+
+  PRBool hasChild;
+  EnsureChildState(PR_FALSE, hasChild);
+  return hasChild ? 0 : -1;
+}
+
+nsresult
+nsDOMAttribute::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
+                              PRBool aNotify)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+nsresult
+nsDOMAttribute::AppendChildTo(nsIContent* aKid, PRBool aNotify)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+nsresult
+nsDOMAttribute::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+nsresult
+nsDOMAttribute::PreHandleEvent(nsEventChainPreVisitor& aVisitor)
+{
+  // We don't support event dispatching to attributes yet.
+  aVisitor.mCanHandle = PR_FALSE;
+  return NS_OK;
+}
+
+nsresult
+nsDOMAttribute::PostHandleEvent(nsEventChainPostVisitor& aVisitor)
+{
+  return NS_OK;
+}
+
+nsresult
+nsDOMAttribute::DispatchDOMEvent(nsEvent* aEvent, nsIDOMEvent* aDOMEvent,
+                                 nsPresContext* aPresContext,
+                                 nsEventStatus* aEventStatus)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+nsresult
+nsDOMAttribute::EnsureChildState(PRBool aSetText, PRBool &aHasChild) const
+{
+  aHasChild = PR_FALSE;
+
+  nsDOMAttribute* mutableThis = NS_CONST_CAST(nsDOMAttribute*, this);
+
+  nsAutoString value;
+  mutableThis->GetValue(value);
+
+  if (!mChild && !value.IsEmpty()) {
+    nsresult rv = NS_NewTextNode(getter_AddRefs(mutableThis->mChild),
+                                 mNodeInfo->NodeInfoManager());
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // XXX We should be setting |this| as the parent of the textnode!
+  }
+
+  aHasChild = !value.IsEmpty();
+
+  if (aSetText && aHasChild) {
+    mChild->SetText(value, PR_TRUE);
+  }
+
+  return NS_OK;
+}
+
+void
+nsDOMAttribute::Initialize()
+{
+  sInitialized = PR_TRUE;
+}
+
+void
+nsDOMAttribute::Shutdown()
+{
+  sInitialized = PR_FALSE;
 }
 
 //----------------------------------------------------------------------
@@ -708,14 +802,7 @@ nsAttributeChildList::~nsAttributeChildList()
 NS_IMETHODIMP
 nsAttributeChildList::GetLength(PRUint32* aLength)
 {
-  *aLength = 0;
-  if (mAttribute) {
-    nsAutoString value;
-    mAttribute->GetValue(value);
-    if (!value.IsEmpty()) {
-      *aLength = 1;
-    }
-  }
+  *aLength = mAttribute ? mAttribute->GetChildCount() : 0;
 
   return NS_OK;
 }
