@@ -34,6 +34,11 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+
+/*
+ * Base class for all our document implementations.
+ */
+
 #ifndef nsDocument_h___
 #define nsDocument_h___
 
@@ -44,6 +49,7 @@
 #include "nsWeakReference.h"
 #include "nsWeakPtr.h"
 #include "nsVoidArray.h"
+#include "nsHashSets.h"
 #include "nsIDOMXMLDocument.h"
 #include "nsIDOM3Document.h"
 #include "nsIDOMDocumentView.h"
@@ -55,38 +61,45 @@
 #include "nsStubDocumentObserver.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIDOM3EventTarget.h"
+#include "nsIDOMNSEventTarget.h"
 #include "nsIDOMStyleSheetList.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIContent.h"
 #include "nsIEventListenerManager.h"
-#include "nsGenericDOMNodeList.h"
 #include "nsIDOM3Node.h"
 #include "nsIPrincipal.h"
+#include "nsIParser.h"
 #include "nsIBindingManager.h"
 #include "nsINodeInfo.h"
 #include "nsIDOMDocumentEvent.h"
 #include "nsIDOM3DocumentEvent.h"
 #include "nsCOMArray.h"
 #include "nsHashtable.h"
-#include "nsIWordBreakerFactory.h"
-#include "nsILineBreakerFactory.h"
+#include "nsInterfaceHashtable.h"
+#include "nsIBoxObject.h"
+#include "nsPIBoxObject.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIURI.h"
 #include "nsScriptLoader.h"
 #include "nsICSSLoader.h"
-#include "nsIDOMXPathEvaluator.h"
 #include "nsIRadioGroupContainer.h"
 #include "nsIScriptEventManager.h"
+#include "nsILayoutHistoryState.h"
+#include "nsIRequest.h"
+#include "nsILoadGroup.h"
+#include "nsTObserverArray.h"
+#include "nsStubMutationObserver.h"
 
 // Put these here so all document impls get them automatically
 #include "nsHTMLStyleSheet.h"
 #include "nsIHTMLCSSStyleSheet.h"
 
 #include "nsStyleSet.h"
-
+#include "nsXMLEventsManager.h"
 #include "pldhash.h"
-
+#include "nsAttrAndChildArray.h"
+#include "nsDOMAttributeMap.h"
 
 #define XML_DECLARATION_BITS_DECLARATION_EXISTS   (1 << 0)
 #define XML_DECLARATION_BITS_ENCODING_EXISTS      (1 << 1)
@@ -99,11 +112,102 @@ class nsDOMStyleSheetList;
 class nsIOutputStream;
 class nsDocument;
 class nsIDTD;
-class nsXPathDocumentTearoff;
 class nsIRadioVisitor;
 class nsIFormControl;
-class nsXMLEventsManager;
 struct nsRadioGroupStruct;
+class nsOnloadBlocker;
+class nsUnblockOnloadEvent;
+struct PLEvent;
+class nsChildContentList;
+
+PR_BEGIN_EXTERN_C
+/* Note that these typedefs declare functions, not pointer to
+   functions.  That's the only way in which they differ from
+   PLHandleEventProc and PLDestroyEventProc. */
+typedef void*
+(PR_CALLBACK EventHandlerFunc)(PLEvent* self);
+typedef void
+(PR_CALLBACK EventDestructorFunc)(PLEvent* self);
+PR_END_EXTERN_C
+
+/**
+ * Hashentry using a PRUint32 key and a cheap set of nsIContent* owning
+ * pointers for the value.
+ *
+ * @see nsTHashtable::EntryType for specification
+ */
+class nsUint32ToContentHashEntry : public PLDHashEntryHdr
+{
+  public:
+    typedef const PRUint32& KeyType;
+    typedef const PRUint32* KeyTypePointer;
+
+    nsUint32ToContentHashEntry(const KeyTypePointer key) :
+      mValue(*key), mValOrHash(nsnull) { }
+    nsUint32ToContentHashEntry(const nsUint32ToContentHashEntry& toCopy) :
+      mValue(toCopy.mValue), mValOrHash(toCopy.mValOrHash)
+    {
+      // Pathetic attempt to not die: clear out the other mValOrHash so we're
+      // effectively stealing it. If toCopy is destroyed right after this,
+      // we'll be OK.
+      NS_CONST_CAST(nsUint32ToContentHashEntry&, toCopy).mValOrHash = nsnull;
+      NS_ERROR("Copying not supported. Fasten your seat belt.");
+    }
+    ~nsUint32ToContentHashEntry() { Destroy(); }
+
+    KeyType GetKey() const { return mValue; }
+    KeyTypePointer GetKeyPointer() const { return &mValue; }
+
+    PRBool KeyEquals(KeyTypePointer aKey) const { return mValue == *aKey; }
+
+    static KeyTypePointer KeyToPointer(KeyType aKey) { return &aKey; }
+    static PLDHashNumber HashKey(KeyTypePointer aKey) { return *aKey; }
+    enum { ALLOW_MEMMOVE = PR_TRUE };
+
+    // Content set methods
+    nsresult PutContent(nsIContent* aContent);
+
+    void RemoveContent(nsIContent* aContent);
+
+    struct Visitor {
+      virtual void Visit(nsIContent* aContent) = 0;
+    };
+    void VisitContent(Visitor* aVisitor);
+
+    PRBool IsEmpty() { return mValOrHash == nsnull; }
+
+  private:
+    typedef unsigned long PtrBits;
+    typedef nsTHashtable<nsISupportsHashKey> HashSet;
+    /** Get the hash pointer (or null if we're not a hash) */
+    HashSet* GetHashSet()
+    {
+      return (PtrBits(mValOrHash) & 0x1) ? nsnull : (HashSet*)mValOrHash;
+    }
+    /** Find out whether it is an nsIContent (returns weak) */
+    nsIContent* GetContent()
+    {
+      return (PtrBits(mValOrHash) & 0x1)
+             ? (nsIContent*)(PtrBits(mValOrHash) & ~0x1)
+             : nsnull;
+    }
+    /** Set the single element, adding a reference */
+    nsresult SetContent(nsIContent* aVal)
+    {
+      NS_IF_ADDREF(aVal);
+      mValOrHash = (void*)(PtrBits(aVal) | 0x1);
+      return NS_OK;
+    }
+    /** Initialize the hash */
+    nsresult InitHashSet(HashSet** aSet);
+
+    void Destroy();
+
+  private:
+    const PRUint32 mValue;
+    /** A hash or nsIContent ptr, depending on the lower bit (0=hash, 1=ptr) */
+    void* mValOrHash;
+};
 
 
 class nsDocHeaderData
@@ -124,26 +228,6 @@ public:
   nsDocHeaderData*  mNext;
 };
 
-// Represents the children of a document (prolog, epilog and
-// document element)
-class nsDocumentChildNodes : public nsGenericDOMNodeList
-{
-public:
-  nsDocumentChildNodes(nsIDocument* aDocument);
-  ~nsDocumentChildNodes();
-
-  NS_IMETHOD    GetLength(PRUint32* aLength);
-  NS_IMETHOD    Item(PRUint32 aIndex, nsIDOMNode** aReturn);
-
-  void DropReference();
-
-protected:
-  nsDocumentChildNodes(); // Not implemented
-
-  nsIDocument* mDocument;
-};
-
-
 class nsDOMStyleSheetList : public nsIDOMStyleSheetList,
                             public nsStubDocumentObserver
 {
@@ -156,7 +240,7 @@ public:
   NS_DECL_NSIDOMSTYLESHEETLIST
 
   // nsIDocumentObserver
-  virtual void DocumentWillBeDestroyed(nsIDocument *aDocument);
+  virtual void NodeWillBeDestroyed(const nsINode *aNode);
   virtual void StyleSheetAdded(nsIDocument *aDocument,
                                nsIStyleSheet* aStyleSheet,
                                PRBool aDocumentSheet);
@@ -167,9 +251,19 @@ public:
 protected:
   PRInt32       mLength;
   nsIDocument*  mDocument;
-  void*         mScriptObject;
 };
 
+class nsOnloadBlocker : public nsIRequest
+{
+public:
+  nsOnloadBlocker() {}
+
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIREQUEST
+
+private:
+  ~nsOnloadBlocker() {}
+};
 
 // Base class for our document implementations.
 //
@@ -194,8 +288,10 @@ class nsDocument : public nsIDocument,
                    public nsSupportsWeakReference,
                    public nsIDOMEventReceiver,
                    public nsIDOM3EventTarget,
+                   public nsIDOMNSEventTarget,
                    public nsIScriptObjectPrincipal,
-                   public nsIRadioGroupContainer
+                   public nsIRadioGroupContainer,
+                   public nsStubMutationObserver
 {
 public:
   NS_DECL_ISUPPORTS
@@ -203,21 +299,21 @@ public:
   virtual void Reset(nsIChannel *aChannel, nsILoadGroup *aLoadGroup);
   virtual void ResetToURI(nsIURI *aURI, nsILoadGroup *aLoadGroup);
 
+  // StartDocumentLoad is pure virtual so that subclasses must override it.
+  // The nsDocument StartDocumentLoad does some setup, but does NOT set
+  // *aDocListener; this is the job of subclasses.
   virtual nsresult StartDocumentLoad(const char* aCommand,
                                      nsIChannel* aChannel,
                                      nsILoadGroup* aLoadGroup,
                                      nsISupports* aContainer,
                                      nsIStreamListener **aDocListener,
                                      PRBool aReset = PR_TRUE,
-                                     nsIContentSink* aContentSink = nsnull);
+                                     nsIContentSink* aContentSink = nsnull) = 0;
 
   virtual void StopDocumentLoad();
 
-  /**
-   * Return the principal responsible for this document.
-   */
-  virtual nsIPrincipal* GetPrincipal();
-
+  virtual void SetDocumentURI(nsIURI* aURI);
+  
   /**
    * Set the principal responsible for this document.
    */
@@ -258,14 +354,6 @@ public:
    */
   virtual void RemoveCharSetObserver(nsIObserver* aObserver);
 
-  /**
-   * Return the Line Breaker for the document
-   */
-  virtual nsILineBreaker* GetLineBreaker();
-  virtual void SetLineBreaker(nsILineBreaker* aLineBreaker);
-  virtual nsIWordBreaker* GetWordBreaker();
-  virtual void SetWordBreaker(nsIWordBreaker* aWordBreaker);
-
   /**
    * Access HTTP header data (this may also get set from other sources, like
    * HTML META tags).
@@ -286,22 +374,13 @@ public:
   virtual PRBool DeleteShell(nsIPresShell* aShell);
   virtual PRUint32 GetNumberOfShells() const;
   virtual nsIPresShell *GetShellAt(PRUint32 aIndex) const;
+  virtual void SetShellsHidden(PRBool aHide);
 
   virtual nsresult SetSubDocumentFor(nsIContent *aContent,
                                      nsIDocument* aSubDoc);
   virtual nsIDocument* GetSubDocumentFor(nsIContent *aContent) const;
   virtual nsIContent* FindContentForSubDocument(nsIDocument *aDocument) const;
 
-  virtual void SetRootContent(nsIContent* aRoot);
-
-  /**
-   * Get the direct children of the document - content in
-   * the prolog, the root content and content in the epilog.
-   */
-  virtual nsIContent *GetChildAt(PRUint32 aIndex) const;
-  virtual PRInt32 IndexOf(nsIContent* aPossibleChild) const;
-  virtual PRUint32 GetChildCount() const;
-
   /**
    * Get the style sheets owned by this document.
    * These are ordered, highest priority last
@@ -324,13 +403,17 @@ public:
   virtual PRInt32 GetNumberOfCatalogStyleSheets() const;
   virtual nsIStyleSheet* GetCatalogStyleSheetAt(PRInt32 aIndex) const;
   virtual void AddCatalogStyleSheet(nsIStyleSheet* aSheet);
+  virtual void EnsureCatalogStyleSheet(const char *aStyleSheetURI);
 
+  virtual nsIChannel* GetChannel() const {
+    return mChannel;
+  }
 
   /**
    * Get this document's attribute stylesheet.  May return null if
    * there isn't one.
    */
-  nsHTMLStyleSheet* GetAttributeStyleSheet() const {
+  virtual nsHTMLStyleSheet* GetAttributeStyleSheet() const {
     return mAttrStyleSheet;
   }
 
@@ -338,7 +421,7 @@ public:
    * Get this document's inline style sheet.  May return null if there
    * isn't one
    */
-  nsIHTMLCSSStyleSheet* GetInlineStyleSheet() const {
+  virtual nsIHTMLCSSStyleSheet* GetInlineStyleSheet() const {
     return mStyleAttrStyleSheet;
   }
   
@@ -350,11 +433,28 @@ public:
   virtual nsIScriptGlobalObject* GetScriptGlobalObject() const;
   virtual void SetScriptGlobalObject(nsIScriptGlobalObject* aGlobalObject);
 
+  virtual nsIScriptGlobalObject* GetScopeObject();
+
+  /**
+   * Return the window containing the document (the outer window).
+   */
+  virtual nsPIDOMWindow *GetWindow();
+
+  /**
+   * Return the inner window used as the script compilation scope for
+   * this document. If you're not absolutely sure you need this, use
+   * GetWindow().
+   */
+  virtual nsPIDOMWindow *GetInnerWindow();
+
   /**
    * Get the script loader for this document
    */
   virtual nsIScriptLoader* GetScriptLoader();
 
+  virtual void AddMutationObserver(nsIMutationObserver* aObserver);
+  virtual void RemoveMutationObserver(nsIMutationObserver* aMutationObserver);
+
   /**
    * Add a new observer of document change notifications. Whenever
    * content is changed, appended, inserted or removed the observers are
@@ -374,8 +474,6 @@ public:
   virtual void EndUpdate(nsUpdateType aUpdateType);
   virtual void BeginLoad();
   virtual void EndLoad();
-  virtual void CharacterDataChanged(nsIContent* aContent,
-                                    PRBool aAppend);
   virtual void ContentStatesChanged(nsIContent* aContent1,
                                     nsIContent* aContent2,
                                     PRInt32 aStateMask);
@@ -383,18 +481,6 @@ public:
   virtual void AttributeWillChange(nsIContent* aChild,
                                    PRInt32 aNameSpaceID,
                                    nsIAtom* aAttribute);
-  virtual void AttributeChanged(nsIContent* aChild,
-                                PRInt32 aNameSpaceID,
-                                nsIAtom* aAttribute,
-                                PRInt32 aModType);
-  virtual void ContentAppended(nsIContent* aContainer,
-                               PRInt32 aNewIndexInContainer);
-  virtual void ContentInserted(nsIContent* aContainer,
-                               nsIContent* aChild,
-                               PRInt32 aIndexInContainer);
-  virtual void ContentRemoved(nsIContent* aContainer,
-                              nsIContent* aChild,
-                              PRInt32 aIndexInContainer);
 
   virtual void StyleRuleChanged(nsIStyleSheet* aStyleSheet,
                                 nsIStyleRule* aOldStyleRule,
@@ -405,29 +491,48 @@ public:
                                 nsIStyleRule* aStyleRule);
 
   virtual void FlushPendingNotifications(mozFlushType aType);
-  virtual void AddReference(void *aKey, nsISupports *aReference);
-  virtual already_AddRefed<nsISupports> RemoveReference(void *aKey);
   virtual nsIScriptEventManager* GetScriptEventManager();
-  virtual void SetXMLDeclaration(const nsAString& aVersion,
-                               const nsAString& aEncoding,
-                               const nsAString& Standalone);
+  virtual void SetXMLDeclaration(const PRUnichar *aVersion,
+                                 const PRUnichar *aEncoding,
+                                 const PRInt32 aStandalone);
   virtual void GetXMLDeclaration(nsAString& aVersion,
                                  nsAString& aEncoding,
                                  nsAString& Standalone);
   virtual PRBool IsScriptEnabled();
 
-  virtual nsresult HandleDOMEvent(nsPresContext* aPresContext,
-                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
-                                  PRUint32 aFlags,
-                                  nsEventStatus* aEventStatus);
+  virtual void OnPageShow(PRBool aPersisted);
+  virtual void OnPageHide(PRBool aPersisted);
+
+  // nsINode
+  virtual PRBool IsNodeOfType(PRUint32 aFlags) const;
+  virtual nsIContent *GetChildAt(PRUint32 aIndex) const;
+  virtual PRInt32 IndexOf(nsINode* aPossibleChild) const;
+  virtual PRUint32 GetChildCount() const;
+  virtual nsresult InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
+                                 PRBool aNotify);
+  virtual nsresult AppendChildTo(nsIContent* aKid, PRBool aNotify);
+  virtual nsresult RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
+  virtual nsresult PreHandleEvent(nsEventChainPreVisitor& aVisitor);
+  virtual nsresult PostHandleEvent(nsEventChainPostVisitor& aVisitor);
+  virtual nsresult DispatchDOMEvent(nsEvent* aEvent, nsIDOMEvent* aDOMEvent,
+                                    nsPresContext* aPresContext,
+                                    nsEventStatus* aEventStatus);
+  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const
+  {
+    return NS_ERROR_NOT_IMPLEMENTED;
+  }
 
   // nsIRadioGroupContainer
   NS_IMETHOD WalkRadioGroup(const nsAString& aName,
-                            nsIRadioVisitor* aVisitor);
+                            nsIRadioVisitor* aVisitor,
+                            PRBool aFlushContent);
   NS_IMETHOD SetCurrentRadioButton(const nsAString& aName,
                                    nsIDOMHTMLInputElement* aRadio);
   NS_IMETHOD GetCurrentRadioButton(const nsAString& aName,
                                    nsIDOMHTMLInputElement** aRadio);
+  NS_IMETHOD GetPositionInGroup(nsIDOMHTMLInputElement *aRadio,
+                                PRInt32 *aPositionIndex,
+                                PRInt32 *aItemsInGroup);
   NS_IMETHOD GetNextRadioButton(const nsAString& aName,
                                 const PRBool aPrevious,
                                 nsIDOMHTMLInputElement*  aFocusedRadio,
@@ -441,6 +546,10 @@ public:
   nsresult GetRadioGroup(const nsAString& aName,
                          nsRadioGroupStruct **aRadioGroup);
 
+  // nsIDOMGCParticipant interface methods
+  virtual nsIDOMGCParticipant* GetSCCIndex();
+  virtual void AppendReachableList(nsCOMArray<nsIDOMGCParticipant>& aArray);
+
   // nsIDOMNode
   NS_DECL_NSIDOMNODE
 
@@ -488,7 +597,8 @@ public:
                                    const nsIID& aIID);
   NS_IMETHOD RemoveEventListenerByIID(nsIDOMEventListener *aListener,
                                       const nsIID& aIID);
-  NS_IMETHOD GetListenerManager(nsIEventListenerManager** aInstancePtrResult);
+  NS_IMETHOD GetListenerManager(PRBool aCreateIfNotFound,
+                                nsIEventListenerManager** aResult);
   NS_IMETHOD HandleEvent(nsIDOMEvent *aEvent);
   NS_IMETHOD GetSystemEventGroup(nsIDOMEventGroup** aGroup);
 
@@ -498,9 +608,11 @@ public:
   // nsIDOM3EventTarget
   NS_DECL_NSIDOM3EVENTTARGET
 
+  // nsIDOMNSEventTarget
+  NS_DECL_NSIDOMNSEVENTTARGET
+
   // nsIScriptObjectPrincipal
-  // virtual nsIPrincipal* GetPrincipal();
-  // Already declared in nsIDocument
+  virtual nsIPrincipal* GetPrincipal();
 
   virtual nsresult Init();
   
@@ -511,26 +623,39 @@ public:
                               PRBool aDocumentDefaultType,
                               nsIContent **aResult);
 
-  virtual NS_HIDDEN_(void*) GetProperty(nsIAtom  *aPropertyName,
-                                        nsresult *aStatus = nsnull) const;
+  virtual NS_HIDDEN_(nsresult) Sanitize();
 
-  virtual NS_HIDDEN_(nsresult) SetProperty(nsIAtom            *aPropertyName,
-                                           void               *aValue,
-                                           NSPropertyDtorFunc  aDtor = nsnull);
+  virtual NS_HIDDEN_(void) EnumerateSubDocuments(nsSubDocEnumFunc aCallback,
+                                                 void *aData);
 
-  virtual NS_HIDDEN_(nsresult) DeleteProperty(nsIAtom *aPropertyName);
+  virtual NS_HIDDEN_(PRBool) CanSavePresentation(nsIRequest *aNewRequest);
+  virtual NS_HIDDEN_(void) Destroy();
+  virtual NS_HIDDEN_(already_AddRefed<nsILayoutHistoryState>) GetLayoutHistoryState() const;
 
-  virtual NS_HIDDEN_(void*) UnsetProperty(nsIAtom  *aPropertyName,
-                                          nsresult *aStatus = nsnull);
+  virtual NS_HIDDEN_(void) BlockOnload();
+  virtual NS_HIDDEN_(void) UnblockOnload(PRBool aFireSync);
+
+  virtual NS_HIDDEN_(void) AddStyleRelevantLink(nsIContent* aContent, nsIURI* aURI);
+  virtual NS_HIDDEN_(void) ForgetLink(nsIContent* aContent);
+  virtual NS_HIDDEN_(void) NotifyURIVisitednessChanged(nsIURI* aURI);
+
+  NS_HIDDEN_(void) ClearBoxObjectFor(nsIContent* aContent);
 
 protected:
 
+  void DispatchContentLoadedEvents();
+
   void RetrieveRelevantHeaders(nsIChannel *aChannel);
 
   static PRBool TryChannelCharset(nsIChannel *aChannel,
                                   PRInt32& aCharsetSource,
                                   nsACString& aCharset);
   
+  void UpdateLinkMap();
+  // Call this before the document does something that will unbind all content.
+  // That will stop us from resolving URIs for all links as they are removed.
+  void DestroyLinkMap();
+
   nsresult doCreateShell(nsPresContext* aContext,
                          nsIViewManager* aViewManager, nsStyleSet* aStyleSet,
                          nsCompatibility aCompatMode,
@@ -543,59 +668,86 @@ protected:
   nsresult CreateElement(nsINodeInfo *aNodeInfo, PRInt32 aElementType,
                          nsIContent** aResult);
 
+  // Return whether all the presshells for this document are safe to flush
+  PRBool IsSafeToFlush() const;
+  
   virtual PRInt32 GetDefaultNamespaceID() const
   {
     return kNameSpaceID_None;
   };
 
-  nsDocument();
+  // Dispatch an event to the ScriptGlobalObject for this document
+  void DispatchEventToWindow(nsEvent *aEvent);
+
+#define NS_DOCUMENT_NOTIFY_OBSERVERS(func_, params_)                  \
+  NS_OBSERVER_ARRAY_NOTIFY_OBSERVERS(mObservers, nsIDocumentObserver, \
+                                     func_, params_);
+  
+#ifdef DEBUG
+  void VerifyRootContentState();
+#endif
+
+  nsDocument(const char* aContentType);
   virtual ~nsDocument();
 
+  void LastRelease();
+
   nsCString mReferrer;
-  nsCString mLastModified;
-  nsCOMPtr<nsIPrincipal> mPrincipal;
+  nsString mLastModified;
 
   nsVoidArray mCharSetObservers;
 
   PLDHashTable *mSubDocuments;
 
-  nsSmallVoidArray mPresShells;
-
   // Array of owning references to all children
-  nsCOMArray<nsIContent> mChildren;
+  nsAttrAndChildArray mChildren;
+
+  // Pointer to our parser if we're currently in the process of being
+  // parsed into.
+  nsCOMPtr<nsIParser> mParser;
 
   nsCOMArray<nsIStyleSheet> mStyleSheets;
   nsCOMArray<nsIStyleSheet> mCatalogSheets;
 
-  // Basically always has at least 1 entry
-  nsAutoVoidArray mObservers;
+  // Array of observers
+  nsTObserverArray<nsIDocumentObserver> mObservers;
+
+  // The document's script global object, the object from which the
+  // document can get its script context and scope. This is the
+  // *inner* window object.
   nsCOMPtr<nsIScriptGlobalObject> mScriptGlobalObject;
+
+  // Weak reference to the scope object (aka the script global object)
+  // that, unlike mScriptGlobalObject, is never unset once set. This
+  // is a weak reference to avoid leaks due to circular references.
+  nsWeakPtr mScopeObject;
+
   nsCOMPtr<nsIEventListenerManager> mListenerManager;
   nsCOMPtr<nsIDOMStyleSheetList> mDOMStyleSheets;
   nsCOMPtr<nsIScriptLoader> mScriptLoader;
   nsDocHeaderData* mHeaderData;
-  nsCOMPtr<nsILineBreaker> mLineBreaker;
-  nsCOMPtr<nsIWordBreaker> mWordBreaker;
 
-  nsRefPtr<nsDocumentChildNodes> mChildNodes;
+  nsRefPtr<nsChildContentList> mChildNodes;
 
   nsHashtable mRadioGroups;
 
+  // True if the document has been detached from its content viewer.
+  PRPackedBool mIsGoingAway:1;
   // True if the document is being destroyed.
-  PRPackedBool mIsGoingAway;
+  PRPackedBool mInDestructor:1;
+  // True if the document "page" is not hidden
+  PRPackedBool mVisible:1;
 
-  // True if the document is being destroyed.
-  PRPackedBool mInDestructor;
+  PRPackedBool mShellsAreHidden:1;
 
   PRUint8 mXMLDeclarationBits;
 
   PRUint8 mDefaultElementType;
 
-  nsSupportsHashtable* mBoxObjectTable;
+  nsInterfaceHashtable<nsISupportsHashKey, nsPIBoxObject> *mBoxObjectTable;
 
-  nsSupportsHashtable mContentWrapperHash;
-
-  nsCOMPtr<nsICSSLoader> mCSSLoader;
+  // The channel that got passed to StartDocumentLoad(), if any
+  nsCOMPtr<nsIChannel> mChannel;
   nsRefPtr<nsHTMLStyleSheet> mAttrStyleSheet;
   nsCOMPtr<nsIHTMLCSSStyleSheet> mStyleAttrStyleSheet;
   nsRefPtr<nsXMLEventsManager> mXMLEventsManager;
@@ -605,15 +757,40 @@ protected:
   nsString mBaseTarget;
 
 private:
-  nsresult IsAllowedAsChild(PRUint16 aNodeType, nsIContent* aRefContent);
+  friend class nsUnblockOnloadEvent;
+
+  void PostUnblockOnloadEvent();
+  void DoUnblockOnload();
+
+  /**
+   * See if aDocument is a child of this.  If so, return the frame element in
+   * this document that holds currentDoc (or an ancestor).
+   */
+  already_AddRefed<nsIDOMElement>
+    CheckAncestryAndGetFrame(nsIDocument* aDocument) const;
 
   // These are not implemented and not supported.
   nsDocument(const nsDocument& aOther);
   nsDocument& operator=(const nsDocument& aOther);
 
-  nsXPathDocumentTearoff* mXPathDocument;
+  nsSmallVoidArray mPresShells;
+
+  nsCOMPtr<nsISupports> mXPathEvaluatorTearoff;
+
+  // The layout history state that should be used by nodes in this
+  // document.  We only actually store a pointer to it when:
+  // 1)  We have no script global object.
+  // 2)  We haven't had Destroy() called on us yet.
+  nsCOMPtr<nsILayoutHistoryState> mLayoutHistoryState;
+
+  PRUint32 mOnloadBlockCount;
+  nsCOMPtr<nsIRequest> mOnloadBlocker;
+  
+  // A map from unvisited URI hashes to content elements
+  nsTHashtable<nsUint32ToContentHashEntry> mLinkMap;
+  // URIs whose visitedness has changed while we were hidden
+  nsCOMArray<nsIURI> mVisitednessChangedURIs;
 };
 
 
-
 #endif /* nsDocument_h___ */
