@@ -36,6 +36,11 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * Object that can be used to serialize selections, ranges, or nodes
+ * to strings in a gazillion different ways.
+ */
+ 
 #include "nsIDocumentEncoder.h"
 
 #include "nscore.h"
@@ -61,10 +66,9 @@
 #include "nsIDOMDocument.h"
 #include "nsICharsetConverterManager.h"
 #include "nsHTMLAtoms.h"
-#include "nsITextContent.h"
+#include "nsIContent.h"
 #include "nsIEnumerator.h"
 #include "nsISelectionPrivate.h"
-#include "nsIFrameSelection.h"
 #include "nsISupportsArray.h"
 #include "nsIParserService.h"
 #include "nsIScriptContext.h"
@@ -81,44 +85,23 @@ enum nsRangeIterationDirection {
   kDirectionIn = 1
 };
 
-#ifdef XP_MAC
-#pragma mark -
-#pragma mark  nsDocumentEncoder declaration 
-#pragma mark -
-#endif
-
 class nsDocumentEncoder : public nsIDocumentEncoder
 {
 public:
-  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IDOCUMENT_ENCODER_IID)
-
   nsDocumentEncoder();
   virtual ~nsDocumentEncoder();
 
-  NS_IMETHOD Init(nsIDocument* aDocument, const nsAString& aMimeType, PRUint32 aFlags);
-
-  /* Interfaces for addref and release and queryinterface */
   NS_DECL_ISUPPORTS
 
-  // Inherited methods from nsIDocumentEncoder
-  NS_IMETHOD SetSelection(nsISelection* aSelection);
-  NS_IMETHOD SetRange(nsIDOMRange* aRange);
-  NS_IMETHOD SetNode(nsIDOMNode* aNode);
-  NS_IMETHOD SetWrapColumn(PRUint32 aWC);
-  NS_IMETHOD SetCharset(const nsACString& aCharset);
-  NS_IMETHOD GetMimeType(nsAString& aMimeType);
-  NS_IMETHOD EncodeToStream(nsIOutputStream* aStream);
-  NS_IMETHOD EncodeToString(nsAString& aOutputString);
-  NS_IMETHOD EncodeToStringWithContext(nsAString& aEncodedString, 
-                                       nsAString& aContextString,
-                                       nsAString& aInfoString);
-  NS_IMETHOD SetNodeFixup(nsIDocumentEncoderNodeFixup *aFixup);
-                                       
+  NS_DECL_NSIDOCUMENTENCODER
+
 protected:
+  void Initialize();
   nsresult SerializeNodeStart(nsIDOMNode* aNode, PRInt32 aStartOffset,
                               PRInt32 aEndOffset, nsAString& aStr);
   nsresult SerializeToStringRecursive(nsIDOMNode* aNode,
-                                      nsAString& aStr);
+                                      nsAString& aStr,
+                                      PRBool aDontSerializeRoot);
   nsresult SerializeNodeEnd(nsIDOMNode* aNode, nsAString& aStr);
   nsresult SerializeRangeToString(nsIDOMRange *aRange,
                                   nsAString& aOutputString);
@@ -163,13 +146,9 @@ protected:
   nsAutoVoidArray   mEndOffsets;
   PRPackedBool      mHaltRangeHint;  
   PRPackedBool      mIsCopying;  // Set to PR_TRUE only while copying
+  PRPackedBool      mNodeIsContainer;
 };
 
-#ifdef XP_MAC
-#pragma mark  nsDocumentEncoder implementation 
-#pragma mark -
-#endif
-
 NS_IMPL_ADDREF(nsDocumentEncoder)
 NS_IMPL_RELEASE(nsDocumentEncoder)
 
@@ -180,14 +159,21 @@ NS_INTERFACE_MAP_END
 
 nsDocumentEncoder::nsDocumentEncoder()
 {
-
+  Initialize();
   mMimeType.AssignLiteral("text/plain");
 
+}
+
+void nsDocumentEncoder::Initialize()
+{
   mFlags = 0;
   mWrapColumn = 72;
   mStartDepth = 0;
   mEndDepth = 0;
+  mStartRootIndex = 0;
+  mEndRootIndex = 0;
   mHaltRangeHint = PR_FALSE;
+  mNodeIsContainer = PR_FALSE;
 }
 
 nsDocumentEncoder::~nsDocumentEncoder()
@@ -195,14 +181,17 @@ nsDocumentEncoder::~nsDocumentEncoder()
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::Init(nsIDocument* aDocument,
+nsDocumentEncoder::Init(nsIDOMDocument* aDocument,
                         const nsAString& aMimeType,
                         PRUint32 aFlags)
 {
   if (!aDocument)
     return NS_ERROR_INVALID_ARG;
 
-  mDocument = aDocument;
+  Initialize();
+
+  mDocument = do_QueryInterface(aDocument);
+  NS_ENSURE_TRUE(mDocument, NS_ERROR_FAILURE);
 
   mMimeType = aMimeType;
 
@@ -236,10 +225,19 @@ nsDocumentEncoder::SetRange(nsIDOMRange* aRange)
 NS_IMETHODIMP
 nsDocumentEncoder::SetNode(nsIDOMNode* aNode)
 {
+  mNodeIsContainer = PR_FALSE;
   mNode = aNode;
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDocumentEncoder::SetContainerNode(nsIDOMNode *aContainer)
+{
+  mNodeIsContainer = PR_TRUE;
+  mNode = aContainer;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsDocumentEncoder::SetCharset(const nsACString& aCharset)
 {
@@ -283,14 +281,8 @@ nsDocumentEncoder::SerializeNodeStart(nsIDOMNode* aNode, PRInt32 aStartOffset,
     case nsIDOMNode::ELEMENT_NODE:
     {
       nsCOMPtr<nsIDOMElement> element = do_QueryInterface(node);
-      // Because FixupNode() may have done a shallow copy of aNode
-      // we need to tell the serializer if the original had children.
-      // Some serializers (notably XML) need this information 
-      // in order to handle empty tags properly.
-      PRBool hasChildren;
-      mSerializer->AppendElementStart(element, 
-                                      NS_SUCCEEDED(aNode->HasChildNodes(&hasChildren)) && hasChildren,
-                                      aStr);
+      nsCOMPtr<nsIDOMElement> originalElement = do_QueryInterface(aNode);
+      mSerializer->AppendElementStart(element, originalElement, aStr);
       break;
     }
     case nsIDOMNode::TEXT_NODE:
@@ -350,10 +342,14 @@ nsDocumentEncoder::SerializeNodeEnd(nsIDOMNode* aNode,
 
 nsresult
 nsDocumentEncoder::SerializeToStringRecursive(nsIDOMNode* aNode,
-                                              nsAString& aStr)
+                                              nsAString& aStr,
+                                              PRBool aDontSerializeRoot)
 {
-  nsresult rv = SerializeNodeStart(aNode, 0, -1, aStr);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv = NS_OK;
+  if (!aDontSerializeRoot) {
+    rv = SerializeNodeStart(aNode, 0, -1, aStr);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
   PRBool hasChildren = PR_FALSE;
 
@@ -373,13 +369,15 @@ nsDocumentEncoder::SerializeToStringRecursive(nsIDOMNode* aNode,
       rv = childNodes->Item(index, getter_AddRefs(child));
       NS_ENSURE_SUCCESS(rv, rv);
 
-      rv = SerializeToStringRecursive(child, aStr);
+      rv = SerializeToStringRecursive(child, aStr, PR_FALSE);
       NS_ENSURE_SUCCESS(rv, rv);     
     }
   }
 
-  rv = SerializeNodeEnd(aNode, aStr);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (!aDontSerializeRoot) {
+    rv = SerializeNodeEnd(aNode, aStr);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
   return FlushText(aStr, PR_FALSE);
 }
@@ -410,7 +408,9 @@ ConvertAndWrite(const nsAString& aString,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCAutoString charXferString;
-  charXferString.SetCapacity(charLength);
+  if (!EnsureStringLength(charXferString, charLength))
+    return NS_ERROR_OUT_OF_MEMORY;
+
   char* charXferBuf = charXferString.BeginWriting();
   nsresult convert_rv = NS_OK;
 
@@ -659,7 +659,7 @@ nsDocumentEncoder::SerializeRangeNodes(nsIDOMRange* aRange,
   {
     // node is completely contained in range.  Serialize the whole subtree
     // rooted by this node.
-    rv = SerializeToStringRecursive(aNode, aString);
+    rv = SerializeToStringRecursive(aNode, aString, PR_FALSE);
     NS_ENSURE_SUCCESS(rv, rv);
   }
   else
@@ -738,7 +738,7 @@ nsDocumentEncoder::SerializeRangeNodes(nsIDOMRange* aRange,
         if ((j==startOffset) || (j==endOffset-1))
           rv = SerializeRangeNodes(aRange, childAsNode, aString, aDepth+1);
         else
-          rv = SerializeToStringRecursive(childAsNode, aString);
+          rv = SerializeToStringRecursive(childAsNode, aString, PR_FALSE);
 
         NS_ENSURE_SUCCESS(rv, rv);
       }
@@ -915,7 +915,7 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
 
       mRange = nsnull;
   } else if (mNode) {
-    rv = SerializeToStringRecursive(mNode, aOutputString);
+    rv = SerializeToStringRecursive(mNode, aOutputString, mNodeIsContainer);
     mNode = nsnull;
   } else {
     nsCOMPtr<nsIDOMDocument> domdoc(do_QueryInterface(mDocument));
@@ -924,7 +924,7 @@ nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
     if (NS_SUCCEEDED(rv)) {
       nsCOMPtr<nsIDOMNode> doc(do_QueryInterface(mDocument));
 
-      rv = SerializeToStringRecursive(doc, aOutputString);
+      rv = SerializeToStringRecursive(doc, aOutputString, PR_FALSE);
     }
   }
 
@@ -972,9 +972,9 @@ nsDocumentEncoder::EncodeToStream(nsIOutputStream* aStream)
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::EncodeToStringWithContext(nsAString& aEncodedString, 
-                                             nsAString& aContextString,
-                                             nsAString& aInfoString)
+nsDocumentEncoder::EncodeToStringWithContext(nsAString& aContextString,
+                                             nsAString& aInfoString,
+                                             nsAString& aEncodedString)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -999,13 +999,6 @@ NS_NewTextEncoder(nsIDocumentEncoder** aResult)
  return NS_OK;
 }
 
-
-#ifdef XP_MAC
-#pragma mark -
-#pragma mark  nsDocumentEncoder declaration 
-#pragma mark -
-#endif
-
 class nsHTMLCopyEncoder : public nsDocumentEncoder
 {
 public:
@@ -1013,13 +1006,13 @@ public:
   nsHTMLCopyEncoder();
   virtual ~nsHTMLCopyEncoder();
 
-  NS_IMETHOD Init(nsIDocument* aDocument, const nsAString& aMimeType, PRUint32 aFlags);
+  NS_IMETHOD Init(nsIDOMDocument* aDocument, const nsAString& aMimeType, PRUint32 aFlags);
 
   // overridden methods from nsDocumentEncoder
   NS_IMETHOD SetSelection(nsISelection* aSelection);
-  NS_IMETHOD EncodeToStringWithContext(nsAString& aEncodedString, 
-                                       nsAString& aContextString,
-                                       nsAString& aInfoString);
+  NS_IMETHOD EncodeToStringWithContext(nsAString& aContextString,
+                                       nsAString& aInfoString,
+                                       nsAString& aEncodedString);
 
 protected:
 
@@ -1047,11 +1040,6 @@ protected:
   PRBool mIsTextWidget;
 };
 
-#ifdef XP_MAC
-#pragma mark  nsDocumentEncoder implementation 
-#pragma mark -
-#endif
-
 nsHTMLCopyEncoder::nsHTMLCopyEncoder()
 {
   mIsTextWidget = PR_FALSE;
@@ -1062,15 +1050,19 @@ nsHTMLCopyEncoder::~nsHTMLCopyEncoder()
 }
 
 NS_IMETHODIMP
-nsHTMLCopyEncoder::Init(nsIDocument* aDocument,
+nsHTMLCopyEncoder::Init(nsIDOMDocument* aDocument,
                         const nsAString& aMimetype,
                         PRUint32 aFlags)
 {
   if (!aDocument)
     return NS_ERROR_INVALID_ARG;
 
+  mIsTextWidget = PR_FALSE;
+  Initialize();
+
   mIsCopying = PR_TRUE;
-  mDocument = aDocument;
+  mDocument = do_QueryInterface(aDocument);
+  NS_ENSURE_TRUE(mDocument, NS_ERROR_FAILURE);
 
   mMimeType.AssignLiteral("text/html");
   
@@ -1144,7 +1136,8 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
   
   // also consider ourselves in a text widget if we can't find an html document
   nsCOMPtr<nsIHTMLDocument> htmlDoc = do_QueryInterface(mDocument);
-  if (!htmlDoc) mIsTextWidget = PR_TRUE;
+  if (!htmlDoc || mDocument->IsCaseSensitive())
+    mIsTextWidget = PR_TRUE;
   
   // normalize selection if we are not in a widget
   if (mIsTextWidget) 
@@ -1197,9 +1190,9 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
 }
 
 NS_IMETHODIMP
-nsHTMLCopyEncoder::EncodeToStringWithContext(nsAString& aEncodedString, 
-                                             nsAString& aContextString,
-                                             nsAString& aInfoString)
+nsHTMLCopyEncoder::EncodeToStringWithContext(nsAString& aContextString,
+                                             nsAString& aInfoString,
+                                             nsAString& aEncodedString)
 {
   nsresult rv = EncodeToString(aEncodedString);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1440,8 +1433,7 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
       rv = GetNodeLocation(node, address_of(parent), &offset);
       NS_ENSURE_SUCCESS(rv, rv);
       if (offset == -1) return NS_OK; // we hit generated content; STOP
-      nsIParserService *parserService =
-        nsContentUtils::GetParserServiceWeakRef();
+      nsIParserService *parserService = nsContentUtils::GetParserService();
       if (!parserService)
         return NS_ERROR_OUT_OF_MEMORY;
       while ((IsFirstNode(node)) && (!IsRoot(parent)) && (parent != common))
@@ -1451,11 +1443,8 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
           nsCOMPtr<nsIContent> content = do_QueryInterface(parent);
           if (content)
           {
-            PRInt32 id;
-            parserService->HTMLAtomTagToId(content->Tag(), &id);
-
             PRBool isBlock = PR_FALSE;
-            parserService->IsBlock(id, isBlock);
+            parserService->IsBlock(parserService->HTMLAtomTagToId(content->Tag()), isBlock);
             if (isBlock)
             {
               bResetPromotion = PR_FALSE;
@@ -1526,8 +1515,7 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
       rv = GetNodeLocation(node, address_of(parent), &offset);
       NS_ENSURE_SUCCESS(rv, rv);
       if (offset == -1) return NS_OK; // we hit generated content; STOP
-      nsIParserService *parserService =
-        nsContentUtils::GetParserServiceWeakRef();
+      nsIParserService *parserService = nsContentUtils::GetParserService();
       if (!parserService)
         return NS_ERROR_OUT_OF_MEMORY;
       while ((IsLastNode(node)) && (!IsRoot(parent)) && (parent != common))
@@ -1537,11 +1525,8 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
           nsCOMPtr<nsIContent> content = do_QueryInterface(parent);
           if (content)
           {
-            PRInt32 id;
-            parserService->HTMLAtomTagToId(content->Tag(), &id);
-
             PRBool isBlock = PR_FALSE;
-            parserService->IsBlock(id, isBlock);
+            parserService->IsBlock(parserService->HTMLAtomTagToId(content->Tag()), isBlock);
             if (isBlock)
             {
               bResetPromotion = PR_FALSE;
@@ -1739,12 +1724,8 @@ nsHTMLCopyEncoder::IsLastNode(nsIDOMNode *aNode)
 PRBool
 nsHTMLCopyEncoder::IsEmptyTextContent(nsIDOMNode* aNode)
 {
-  PRBool result = PR_FALSE;
-  nsCOMPtr<nsITextContent> tc(do_QueryInterface(aNode));
-  if (tc) {
-    result = tc->IsOnlyWhitespace();
-  }
-  return result;
+  nsCOMPtr<nsIContent> cont = do_QueryInterface(aNode);
+  return cont && cont->TextIsOnlyWhitespace();
 }
 
 nsresult NS_NewHTMLCopyTextEncoder(nsIDocumentEncoder** aResult); // make mac compiler happy
