@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=78: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -36,7 +37,11 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "nsIFrameLoader.h"
+/*
+ * Class for managing loading of a subframe (creation of the docshell,
+ * handling of loads in it, recursion-checking).
+ */
+
 #include "nsIDOMHTMLIFrameElement.h"
 #include "nsIDOMHTMLFrameElement.h"
 #include "nsIDOMWindow.h"
@@ -55,11 +60,11 @@
 #include "nsIDocShellTreeOwner.h"
 #include "nsIDocShellLoadInfo.h"
 #include "nsIBaseWindow.h"
-#include "nsIWebShell.h"
 #include "nsContentUtils.h"
 #include "nsUnicharUtils.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptSecurityManager.h"
+#include "nsFrameLoader.h"
 
 #include "nsIURI.h"
 #include "nsIURL.h"
@@ -73,7 +78,8 @@
 //             itself.  Note that "#foo" on the end of URL doesn't affect
 //             whether it's considered identical, but "?foo" or ";foo" are
 //             considered and compared.
-#define MAX_SAME_URL_CONTENT_FRAMES 3
+// Bug 228829: Limit this to 1, like IE does.
+#define MAX_SAME_URL_CONTENT_FRAMES 1
 
 // Bug 8065: Limit content frame depth to some reasonable level. This
 // does not count chrome frames when determining depth, nor does it
@@ -84,84 +90,13 @@
 // we'd need to re-institute a fixed version of bug 98158.
 #define MAX_DEPTH_CONTENT_FRAMES 10
 
-
-class nsFrameLoader : public nsIFrameLoader
-{
-public:
-  nsFrameLoader();
-  virtual ~nsFrameLoader();
-
-  // nsISupports
-  NS_DECL_ISUPPORTS
-
-  // nsIFrameLoader
-  NS_IMETHOD Init(nsIContent *aOwner);
-  NS_IMETHOD LoadFrame();
-  NS_IMETHOD GetDocShell(nsIDocShell **aDocShell);
-  NS_IMETHOD Destroy();
-
-protected:
-  nsresult EnsureDocShell();
-  void GetURL(nsAString& aURL);
-
-  nsCOMPtr<nsIDocShell> mDocShell;
-
-  nsIContent *mOwnerContent; // WEAK
-};
-
-nsresult
-NS_NewFrameLoader(nsIFrameLoader **aFrameLoader)
-{
-  *aFrameLoader = new nsFrameLoader();
-  NS_ENSURE_TRUE(*aFrameLoader, NS_ERROR_OUT_OF_MEMORY);
-
-  NS_ADDREF(*aFrameLoader);
-
-  return NS_OK;
-}
-
-nsFrameLoader::nsFrameLoader()
-  : mOwnerContent(nsnull)
-{
-}
-
-nsFrameLoader::~nsFrameLoader()
-{
-  Destroy();
-}
-
-
-// QueryInterface implementation for nsFrameLoader
-NS_INTERFACE_MAP_BEGIN(nsFrameLoader)
-  NS_INTERFACE_MAP_ENTRY(nsIFrameLoader)
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
-NS_INTERFACE_MAP_END
-
-
-NS_IMPL_ADDREF(nsFrameLoader)
-NS_IMPL_RELEASE(nsFrameLoader)
-
-NS_IMETHODIMP
-nsFrameLoader::Init(nsIContent *aOwner)
-{
-  mOwnerContent = aOwner; // WEAK
-
-  return NS_OK;
-}
+NS_IMPL_ISUPPORTS1(nsFrameLoader, nsIFrameLoader)
 
 NS_IMETHODIMP
 nsFrameLoader::LoadFrame()
 {
   NS_ENSURE_TRUE(mOwnerContent, NS_ERROR_NOT_INITIALIZED);
 
-  nsresult rv = EnsureDocShell();
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsIDocument* doc = mOwnerContent->GetDocument();
-  if (!doc) {
-    return NS_OK;
-  }
-
   nsAutoString src;
   GetURL(src);
 
@@ -171,155 +106,111 @@ nsFrameLoader::LoadFrame()
     src.AssignLiteral("about:blank");
   }
 
-  // Make an absolute URI
-  nsIURI *base_uri = doc->GetBaseURI();
+  nsIDocument* doc = mOwnerContent->GetOwnerDoc();
+  if (!doc) {
+    return NS_OK;
+  }
 
-  const nsACString &doc_charset = doc->GetDocumentCharacterSet();
+  nsCOMPtr<nsIURI> base_uri = mOwnerContent->GetBaseURI();
+  const nsAFlatCString &doc_charset = doc->GetDocumentCharacterSet();
+  const char *charset = doc_charset.IsEmpty() ? nsnull : doc_charset.get();
 
   nsCOMPtr<nsIURI> uri;
-  rv = NS_NewURI(getter_AddRefs(uri), src,
-                 doc_charset.IsEmpty() ? nsnull :
-                 PromiseFlatCString(doc_charset).get(), base_uri);
+  nsresult rv = NS_NewURI(getter_AddRefs(uri), src, charset, base_uri);
+
+  // If the URI was malformed, try to recover by loading about:blank.
+  if (rv == NS_ERROR_MALFORMED_URI) {
+    rv = NS_NewURI(getter_AddRefs(uri), NS_LITERAL_STRING("about:blank"),
+                   charset);
+  }
+
+  NS_ENSURE_SUCCESS(rv, rv);
+  return LoadURI(uri);
+}
+
+NS_IMETHODIMP
+nsFrameLoader::LoadURI(nsIURI* aURI)
+{
+  NS_PRECONDITION(aURI, "Null URI?");
+  if (!aURI)
+    return NS_ERROR_INVALID_POINTER;
+
+  nsIDocument* doc = mOwnerContent->GetOwnerDoc();
+  if (!doc) {
+    return NS_OK;
+  }
+
+  nsresult rv = EnsureDocShell();
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
   mDocShell->CreateLoadInfo(getter_AddRefs(loadInfo));
   NS_ENSURE_TRUE(loadInfo, NS_ERROR_FAILURE);
 
-  // Check for security
+  // Check for security.  The fun part is trying to figure out what principals
+  // to use.  The way I figure it, if we're doing a LoadFrame() accidentally
+  // (eg someone created a frame/iframe node, we're being parsed, XUL iframes
+  // are being reframed, etc.) then we definitely want to use the node
+  // principal of mOwnerContent for security checks.  If, on the other hand,
+  // someone's setting the src on our owner content, or created it via script,
+  // or whatever, then they can clearly access it... and we should still use
+  // the principal of mOwnerContent.  I don't think that leads to privilege
+  // escalation, and it's reasonably guaranteed to not lead to XSS issues
+  // (since caller can already access mOwnerContent in this case).  So just use
+  // the principal of mOwnerContent no matter what.  If script wants to run
+  // things with its own permissions, which differ from those of mOwnerContent
+  // (which means the script is privileged in some way) it should set
+  // window.location instead.
   nsIScriptSecurityManager *secMan = nsContentUtils::GetSecurityManager();
 
-  // Get referring URL
-  nsCOMPtr<nsIURI> referrer;
-  nsCOMPtr<nsIPrincipal> principal;
-  rv = secMan->GetSubjectPrincipal(getter_AddRefs(principal));
-  NS_ENSURE_SUCCESS(rv, rv);
+  // Get our principal
+  nsIPrincipal* principal = mOwnerContent->NodePrincipal();
 
-  // If we were called from script, get the referring URL from the script
-
-  if (principal) {
-    // Pass the script principal to the docshell
-
-    loadInfo->SetOwner(principal);
-  } else {
-    // We're not being called form script, tell the docshell
-    // to inherit an owner from the current document.
-
-    loadInfo->SetInheritOwner(PR_TRUE);
-    principal = doc->GetPrincipal();
-  }
-
-  if (!principal) {
-    return NS_ERROR_FAILURE;
-  }
-  
   // Check if we are allowed to load absURL
-  rv = secMan->CheckLoadURIWithPrincipal(principal, uri,
+  rv = secMan->CheckLoadURIWithPrincipal(principal, aURI,
                                          nsIScriptSecurityManager::STANDARD);
   if (NS_FAILED(rv)) {
     return rv; // We're not
   }
 
+  // Bail out if this is an infinite recursion scenario
+  rv = CheckForRecursiveLoad(aURI);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  // We'll use our principal, not that of the document loaded inside us.  This
+  // is very important; needed to prevent XSS attacks on documents loaded in
+  // subframes!  But only use our principal if our docshell's type is the same
+  // as the type of our ownerDocument's docshell.  Note that we could try
+  // checking GetSameTypeParent() on mDocShell, but that might break if we ever
+  // support docshells loaded inside disconnected nodes...
+  nsCOMPtr<nsISupports> container = doc->GetContainer();
+  nsCOMPtr<nsIDocShellTreeItem> parentItem = do_QueryInterface(container);
+  nsCOMPtr<nsIDocShellTreeItem> ourItem = do_QueryInterface(mDocShell);
+  NS_ASSERTION(ourItem, "Must have item");
+  if (parentItem) {
+    PRInt32 parentType;
+    rv = parentItem->GetItemType(&parentType);
+    PRInt32 ourType;
+    nsresult rv2 = ourItem->GetItemType(&ourType);
+    if (NS_SUCCEEDED(rv) && NS_SUCCEEDED(rv2) && ourType == parentType) {
+      loadInfo->SetOwner(principal);
+    }
+  }
+
+  nsCOMPtr<nsIURI> referrer;
   rv = principal->GetURI(getter_AddRefs(referrer));
   NS_ENSURE_SUCCESS(rv, rv);
 
   loadInfo->SetReferrer(referrer);
 
-  // Bug 136580: Check for recursive frame loading
-  // pre-grab these for speed
-  nsCAutoString prepath;
-  nsCAutoString filepath;
-  nsCAutoString query;
-  nsCAutoString param;
-  rv = uri->GetPrePath(prepath);
-  NS_ENSURE_SUCCESS(rv,rv);
-  nsCOMPtr<nsIURL> aURL(do_QueryInterface(uri, &rv)); // QI can fail
-  if (NS_SUCCEEDED(rv)) {
-    rv = aURL->GetFilePath(filepath);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = aURL->GetQuery(query);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = aURL->GetParam(param);
-    NS_ENSURE_SUCCESS(rv,rv);
-  } else {
-    // Not a URL, so punt and just take the whole path.  Note that if you
-    // have a self-referential-via-refs non-URL (can't happen via nsSimpleURI,
-    // but could in theory with an external protocol handler) frameset it will
-    // recurse down to the depth limit before stopping, but it will stop.
-    rv = uri->GetPath(filepath);
-    NS_ENSURE_SUCCESS(rv,rv);
-  }
-
-  PRInt32 matchCount = 0;
-  nsCOMPtr<nsIDocShellTreeItem> treeItem = do_QueryInterface(mDocShell);
-  nsCOMPtr<nsIDocShellTreeItem> parentAsItem;
-  treeItem->GetParent(getter_AddRefs(parentAsItem));
-  while (parentAsItem) {
-    // Only interested in checking for recursion in content
-    PRInt32 parentType;
-    parentAsItem->GetItemType(&parentType);
-    if (parentType != nsIDocShellTreeItem::typeContent) {
-      break; // Not content
-    }
-    // Check the parent URI with the URI we're loading
-    nsCOMPtr<nsIWebNavigation> parentAsNav(do_QueryInterface(parentAsItem));
-    if (parentAsNav) {
-      // Does the URI match the one we're about to load?
-      nsCOMPtr<nsIURI> parentURI;
-      parentAsNav->GetCurrentURI(getter_AddRefs(parentURI));
-      if (parentURI) {
-        // Bug 98158/193011: We need to ignore data after the #
-        // Note that this code is back-stopped by the maximum-depth checks.
-        
-        // Check prepath (foo://blah@bar:port/) and filepath
-        // (/dir/dir/file.ext), but not # extensions.
-        // There's no easy way to get the URI without extension
-        // directly, so check prepath, filepath and query/param (if any)
-
-        // Note that while in theory a CGI could return different data for
-        // the same query string, the spec states that it shouldn't, so
-        // we'll compare queries (and params).
-        nsCAutoString parentPrePath;
-        nsCAutoString parentFilePath;
-        nsCAutoString parentQuery;
-        nsCAutoString parentParam;
-        rv = parentURI->GetPrePath(parentPrePath);
-        NS_ENSURE_SUCCESS(rv,rv);
-        nsCOMPtr<nsIURL> parentURL(do_QueryInterface(parentURI, &rv)); // QI can fail
-        if (NS_SUCCEEDED(rv)) {
-          rv = parentURL->GetFilePath(parentFilePath);
-          NS_ENSURE_SUCCESS(rv,rv);
-          rv = parentURL->GetQuery(parentQuery);
-          NS_ENSURE_SUCCESS(rv,rv);
-          rv = parentURL->GetParam(parentParam);
-          NS_ENSURE_SUCCESS(rv,rv);
-        } else {
-          rv = uri->GetPath(filepath);
-          NS_ENSURE_SUCCESS(rv,rv);
-        }
-        
-        // filepath will often not match; test it first
-        if (filepath.Equals(parentFilePath) &&
-            query.Equals(parentQuery) &&
-            prepath.Equals(parentPrePath) &&
-            param.Equals(parentParam))
-        {
-          matchCount++;
-          if (matchCount >= MAX_SAME_URL_CONTENT_FRAMES) {
-            NS_WARNING("Too many nested content frames have the same url (recursion?) so giving up");
-            return NS_ERROR_UNEXPECTED;
-          }
-        }
-      }
-    }
-    nsIDocShellTreeItem* temp = parentAsItem;
-    temp->GetParent(getter_AddRefs(parentAsItem));
-  }
-  
   // Kick off the load...
-  rv = mDocShell->LoadURI(uri, loadInfo, nsIWebNavigation::LOAD_FLAGS_NONE,
+  rv = mDocShell->LoadURI(aURI, loadInfo, nsIWebNavigation::LOAD_FLAGS_NONE,
                           PR_FALSE);
-  NS_ASSERTION(NS_SUCCEEDED(rv), "failed to load URL");
+#ifdef DEBUG
+  if (NS_FAILED(rv)) {
+    NS_WARNING("Failed to load the URL");
+  }
+#endif
 
   return rv;
 }
@@ -356,6 +247,21 @@ nsFrameLoader::Destroy()
     mOwnerContent = nsnull;
   }
 
+  // Let the tree owner know we're gone.
+  if (mIsTopLevelContent) {
+    nsCOMPtr<nsIDocShellTreeItem> ourItem = do_QueryInterface(mDocShell);
+    if (ourItem) {
+      nsCOMPtr<nsIDocShellTreeItem> parentItem;
+      ourItem->GetParent(getter_AddRefs(parentItem));
+      nsCOMPtr<nsIDocShellTreeOwner> owner = do_GetInterface(parentItem);
+      nsCOMPtr<nsIDocShellTreeOwner_MOZILLA_1_8_BRANCH> owner2 =
+        do_QueryInterface(owner);
+      if (owner2) {
+        owner2->ContentShellRemoved(ourItem);
+      }
+    }
+  }
+  
   // Let our window know that we are gone
   nsCOMPtr<nsPIDOMWindow> win_private(do_GetInterface(mDocShell));
   if (win_private) {
@@ -369,7 +275,13 @@ nsFrameLoader::Destroy()
   }
 
   mDocShell = nsnull;
+  return NS_OK;
+}
 
+NS_IMETHODIMP
+nsFrameLoader::GetDepthTooGreat(PRBool* aDepthTooGreat)
+{
+  *aDepthTooGreat = mDepthTooGreat;
   return NS_OK;
 }
 
@@ -391,37 +303,6 @@ nsFrameLoader::EnsureDocShell()
   nsCOMPtr<nsIWebNavigation> parentAsWebNav =
     do_GetInterface(doc->GetScriptGlobalObject());
 
-  // Bug 8065: Don't exceed some maximum depth in content frames
-  // (MAX_DEPTH_CONTENT_FRAMES)
-  PRInt32 depth = 0;
-
-  if (parentAsWebNav) {
-    nsCOMPtr<nsIDocShellTreeItem> parentAsItem =
-      do_QueryInterface(parentAsWebNav);
-
-    while (parentAsItem) {
-      ++depth;
-
-      if (depth >= MAX_DEPTH_CONTENT_FRAMES) {
-        NS_WARNING("Too many nested content frames so giving up");
-
-        return NS_ERROR_UNEXPECTED; // Too deep, give up!  (silently?)
-      }
-
-      // Only count depth on content, not chrome.
-      // If we wanted to limit total depth, skip the following check:
-      PRInt32 parentType;
-      parentAsItem->GetItemType(&parentType);
-
-      if (nsIDocShellTreeItem::typeContent == parentType) {
-        nsIDocShellTreeItem* temp = parentAsItem;
-        temp->GetParent(getter_AddRefs(parentAsItem));
-      } else {
-        break; // we have exited content, stop counting, depth is OK!
-      }
-    }
-  }
-
   // Create the docshell...
   mDocShell = do_CreateInstance("@mozilla.org/webshell;1");
   NS_ENSURE_TRUE(mDocShell, NS_ERROR_FAILURE);
@@ -431,16 +312,14 @@ nsFrameLoader::EnsureDocShell()
   NS_ENSURE_TRUE(docShellAsItem, NS_ERROR_FAILURE);
   nsAutoString frameName;
 
-  // Don't use mOwnerContent->GetNameSpaceID() here since it returns
-  // kNameSpaceID_XHTML for both HTML and XHTML, see bug 183683.
-  nsINodeInfo* ni = mOwnerContent->GetNodeInfo();
-  if (ni && ni->NamespaceID() == kNameSpaceID_XHTML) {
+  PRInt32 namespaceID = mOwnerContent->GetNameSpaceID();
+  if (namespaceID == kNameSpaceID_XHTML) {
     mOwnerContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::id, frameName);
   } else {
     mOwnerContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, frameName);
     // XXX if no NAME then use ID, after a transition period this will be
     // changed so that XUL only uses ID too (bug 254284).
-    if (frameName.IsEmpty() && ni && ni->NamespaceID() == kNameSpaceID_XUL) {
+    if (frameName.IsEmpty() && namespaceID == kNameSpaceID_XUL) {
       mOwnerContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::id, frameName);
     }
   }
@@ -455,6 +334,9 @@ nsFrameLoader::EnsureDocShell()
 
   nsCOMPtr<nsIDocShellTreeNode> parentAsNode(do_QueryInterface(parentAsWebNav));
   if (parentAsNode) {
+    // Note: This logic duplicates a lot of logic in
+    // nsSubDocumentFrame::AttributeChanged.  We should fix that.
+
     nsCOMPtr<nsIDocShellTreeItem> parentAsItem =
       do_QueryInterface(parentAsNode);
 
@@ -464,28 +346,17 @@ nsFrameLoader::EnsureDocShell()
     nsAutoString value;
     PRBool isContent = PR_FALSE;
 
-    if (mOwnerContent->IsContentOfType(nsIContent::eXUL)) {
+    if (mOwnerContent->IsNodeOfType(nsINode::eXUL)) {
       mOwnerContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::type, value);
     }
 
     // we accept "content" and "content-xxx" values.
-    // at time of writing, we expect "xxx" to be "primary", but
-    // someday it might be an integer expressing priority
+    // at time of writing, we expect "xxx" to be "primary" or "targetable", but
+    // someday it might be an integer expressing priority or something else.
 
-    if (value.Length() >= 7) {
-      // Lowercase the value, ContentShellAdded() further down relies
-      // on it being lowercased.
-      ToLowerCase(value);
-
-      nsAutoString::const_char_iterator start, end;
-      value.BeginReading(start);
-      value.EndReading(end);
-
-      nsAutoString::const_char_iterator iter(start + 7);
-
-      isContent = Substring(start, iter).EqualsLiteral("content") &&
-                  (iter == end || *iter == '-');
-    }
+    isContent = value.LowerCaseEqualsLiteral("content") ||
+      StringBeginsWith(value, NS_LITERAL_STRING("content-"),
+                       nsCaseInsensitiveStringComparator());
 
     if (isContent) {
       // The web shell's type is content.
@@ -501,28 +372,29 @@ nsFrameLoader::EnsureDocShell()
 
     parentAsNode->AddChild(docShellAsItem);
 
-    if (isContent) {
+    if (parentType == nsIDocShellTreeItem::typeChrome && isContent) {
+      mIsTopLevelContent = PR_TRUE;
+      
+      // XXXbz why is this in content code, exactly?  We should handle
+      // this some other way.....  Not sure how yet.
       nsCOMPtr<nsIDocShellTreeOwner> parentTreeOwner;
       parentAsItem->GetTreeOwner(getter_AddRefs(parentTreeOwner));
+      nsCOMPtr<nsIDocShellTreeOwner_MOZILLA_1_8_BRANCH> owner2 =
+        do_QueryInterface(parentTreeOwner);
 
-      if (parentTreeOwner) {
-        PRBool is_primary = parentType == nsIDocShellTreeItem::typeChrome &&
-                            value.EqualsLiteral("content-primary");
+      PRBool is_primary = value.LowerCaseEqualsLiteral("content-primary");
 
+      if (owner2) {
+        PRBool is_targetable = is_primary ||
+          value.LowerCaseEqualsLiteral("content-targetable");
+        owner2->ContentShellAdded2(docShellAsItem, is_primary, is_targetable,
+                                   value);
+      } else if (parentTreeOwner) {
         parentTreeOwner->ContentShellAdded(docShellAsItem, is_primary,
                                            value.get());
       }
     }
 
-    // connect the container...
-    nsCOMPtr<nsIWebShell> webShell(do_QueryInterface(mDocShell));
-    nsCOMPtr<nsIWebShellContainer> outerContainer =
-      do_QueryInterface(parentAsWebNav);
-
-    if (outerContainer) {
-      webShell->SetContainer(outerContainer);
-    }
-
     // Make sure all shells have links back to the content element
     // in the nearest enclosing chrome shell.
     nsCOMPtr<nsIChromeEventHandler> chromeEventHandler;
@@ -532,8 +404,8 @@ nsFrameLoader::EnsureDocShell()
       // enclosing chrome shell.
 
       chromeEventHandler = do_QueryInterface(mOwnerContent);
-      NS_WARN_IF_FALSE(chromeEventHandler,
-                       "This mContent should implement this.");
+      NS_ASSERTION(chromeEventHandler,
+                   "This mContent should implement this.");
     } else {
       nsCOMPtr<nsIDocShell> parentShell(do_QueryInterface(parentAsNode));
 
@@ -573,7 +445,7 @@ nsFrameLoader::EnsureDocShell()
 }
 
 void
-nsFrameLoader::GetURL(nsAString& aURI)
+nsFrameLoader::GetURL(nsString& aURI)
 {
   aURI.Truncate();
 
@@ -584,3 +456,93 @@ nsFrameLoader::GetURL(nsAString& aURI)
   }
 }
 
+nsresult
+nsFrameLoader::CheckForRecursiveLoad(nsIURI* aURI)
+{
+  mDepthTooGreat = PR_FALSE;
+  
+  NS_PRECONDITION(mDocShell, "Must have docshell here");
+  
+  nsCOMPtr<nsIDocShellTreeItem> treeItem = do_QueryInterface(mDocShell);
+  NS_ASSERTION(treeItem, "docshell must be a treeitem!");
+  
+  PRInt32 ourType;
+  nsresult rv = treeItem->GetItemType(&ourType);
+  if (NS_SUCCEEDED(rv) && ourType != nsIDocShellTreeItem::typeContent) {
+    // No need to do recursion-protection here XXXbz why not??  Do we really
+    // trust people not to screw up with non-content docshells?
+    return NS_OK;
+  }
+
+  // Bug 8065: Don't exceed some maximum depth in content frames
+  // (MAX_DEPTH_CONTENT_FRAMES)
+  nsCOMPtr<nsIDocShellTreeItem> parentAsItem;
+  treeItem->GetSameTypeParent(getter_AddRefs(parentAsItem));
+  PRInt32 depth = 0;
+  while (parentAsItem) {
+    ++depth;
+    
+    if (depth >= MAX_DEPTH_CONTENT_FRAMES) {
+      mDepthTooGreat = PR_TRUE;
+      NS_WARNING("Too many nested content frames so giving up");
+
+      return NS_ERROR_UNEXPECTED; // Too deep, give up!  (silently?)
+    }
+
+    nsCOMPtr<nsIDocShellTreeItem> temp;
+    temp.swap(parentAsItem);
+    temp->GetSameTypeParent(getter_AddRefs(parentAsItem));
+  }
+  
+  // Bug 136580: Check for recursive frame loading
+  // pre-grab these for speed
+  nsCOMPtr<nsIURI> cloneURI;
+  rv = aURI->Clone(getter_AddRefs(cloneURI));
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  // Bug 98158/193011: We need to ignore data after the #
+  nsCOMPtr<nsIURL> cloneURL(do_QueryInterface(cloneURI)); // QI can fail
+  if (cloneURL) {
+    rv = cloneURL->SetRef(EmptyCString());
+    NS_ENSURE_SUCCESS(rv,rv);
+  }
+
+  PRInt32 matchCount = 0;
+  treeItem->GetSameTypeParent(getter_AddRefs(parentAsItem));
+  while (parentAsItem) {
+    // Check the parent URI with the URI we're loading
+    nsCOMPtr<nsIWebNavigation> parentAsNav(do_QueryInterface(parentAsItem));
+    if (parentAsNav) {
+      // Does the URI match the one we're about to load?
+      nsCOMPtr<nsIURI> parentURI;
+      parentAsNav->GetCurrentURI(getter_AddRefs(parentURI));
+      if (parentURI) {
+        nsCOMPtr<nsIURI> parentClone;
+        rv = parentURI->Clone(getter_AddRefs(parentClone));
+        NS_ENSURE_SUCCESS(rv, rv);
+        nsCOMPtr<nsIURL> parentURL(do_QueryInterface(parentClone));
+        if (parentURL) {
+          rv = parentURL->SetRef(EmptyCString());
+          NS_ENSURE_SUCCESS(rv,rv);
+        }
+
+        PRBool equal;
+        rv = cloneURI->Equals(parentClone, &equal);
+        NS_ENSURE_SUCCESS(rv, rv);
+        
+        if (equal) {
+          matchCount++;
+          if (matchCount >= MAX_SAME_URL_CONTENT_FRAMES) {
+            NS_WARNING("Too many nested content frames have the same url (recursion?) so giving up");
+            return NS_ERROR_UNEXPECTED;
+          }
+        }
+      }
+    }
+    nsCOMPtr<nsIDocShellTreeItem> temp;
+    temp.swap(parentAsItem);
+    temp->GetSameTypeParent(getter_AddRefs(parentAsItem));
+  }
+
+  return NS_OK;
+}
