@@ -34,6 +34,12 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+
+/*
+ * Base class for DOM Core's nsIDOMComment, nsIDOMDocumentType, nsIDOMText,
+ * nsIDOMCDATASection, and nsIDOMProcessingInstruction nodes.
+ */
+
 #include "nsGenericDOMDataNode.h"
 #include "nsGenericElement.h"
 #include "nsIDocument.h"
@@ -53,43 +59,59 @@
 #include "nsIDOMText.h"
 #include "nsCOMPtr.h"
 #include "nsDOMString.h"
+#include "nsLayoutAtoms.h"
+#include "nsIDOMUserDataHandler.h"
+#include "nsChangeHint.h"
+#include "nsEventDispatcher.h"
+#include "nsCOMArray.h"
+#include "nsNodeUtils.h"
 
 #include "pldhash.h"
 #include "prprf.h"
 
-nsGenericDOMDataNode::nsGenericDOMDataNode(nsIDocument *aDocument)
-  : mDocument(aDocument)
+nsGenericDOMDataNode::nsDataSlots::nsDataSlots(PtrBits aFlags)
+  : nsINode::nsSlots(aFlags),
+    mBindingParent(nsnull)
+{
+}
+
+nsGenericDOMDataNode::nsDataSlots::~nsDataSlots()
+{
+  if (mChildNodes) {
+    mChildNodes->DropReference();
+  }
+}
+
+nsGenericDOMDataNode::nsGenericDOMDataNode(nsINodeInfo *aNodeInfo)
+  : nsIContent(aNodeInfo)
 {
 }
 
 nsGenericDOMDataNode::~nsGenericDOMDataNode()
 {
-  if (CouldHaveEventListenerManager()) {
-    PL_DHashTableOperate(&nsGenericElement::sEventListenerManagersHash,
-                         this, PL_DHASH_REMOVE);
-  }
-
-  if (CouldHaveRangeList()) {
-    PL_DHashTableOperate(&nsGenericElement::sRangeListsHash,
-                         this, PL_DHASH_REMOVE);
-  }
+  NS_PRECONDITION(!IsInDoc(),
+                  "Please remove this from the document properly");
 }
 
 
 NS_IMPL_ADDREF(nsGenericDOMDataNode)
-NS_IMPL_RELEASE(nsGenericDOMDataNode)
+NS_IMPL_RELEASE_WITH_DESTROY(nsGenericDOMDataNode,
+                             nsNodeUtils::LastRelease(this, PR_TRUE))
 
 NS_INTERFACE_MAP_BEGIN(nsGenericDOMDataNode)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIContent)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMGCParticipant)
   NS_INTERFACE_MAP_ENTRY_TEAROFF(nsIDOMEventReceiver,
                                  nsDOMEventRTTearoff::Create(this))
   NS_INTERFACE_MAP_ENTRY_TEAROFF(nsIDOMEventTarget,
                                  nsDOMEventRTTearoff::Create(this))
   NS_INTERFACE_MAP_ENTRY_TEAROFF(nsIDOM3EventTarget,
                                  nsDOMEventRTTearoff::Create(this))
+  NS_INTERFACE_MAP_ENTRY_TEAROFF(nsIDOMNSEventTarget,
+                                 nsDOMEventRTTearoff::Create(this))
   NS_INTERFACE_MAP_ENTRY(nsIContent)
-  // No nsITextContent since all subclasses might not want that.
   NS_INTERFACE_MAP_ENTRY_TEAROFF(nsIDOM3Node, new nsNode3Tearoff(this))
+  NS_INTERFACE_MAP_ENTRY(nsINode)
 NS_INTERFACE_MAP_END
 
 
@@ -108,96 +130,58 @@ nsGenericDOMDataNode::SetNodeValue(const nsAString& aNodeValue)
 nsresult
 nsGenericDOMDataNode::GetParentNode(nsIDOMNode** aParentNode)
 {
-  nsresult rv = NS_OK;
+  *aParentNode = nsnull;
+  nsINode *parent = GetNodeParent();
 
-  nsIContent *parent = GetParent();
-  if (parent) {
-    rv = CallQueryInterface(parent, aParentNode);
-  }
-  else if (mDocument) {
-    rv = CallQueryInterface(mDocument, aParentNode);
-  }
-  else {
-    *aParentNode = nsnull;
-  }
-
-  NS_ASSERTION(NS_SUCCEEDED(rv), "Must be a DOM Node");
-
-  return rv;
+  return parent ? CallQueryInterface(parent, aParentNode) : NS_OK;
 }
 
 nsresult
 nsGenericDOMDataNode::GetPreviousSibling(nsIDOMNode** aPrevSibling)
 {
-  nsresult rv = NS_OK;
+  *aPrevSibling = nsnull;
 
-  nsIContent *sibling = nsnull;
-  nsIContent *parent = GetParent();
-  if (parent) {
-    PRInt32 pos = parent->IndexOf(this);
-    if (pos > 0) {
-      sibling = parent->GetChildAt(pos - 1);
-    }
-  }
-  else if (mDocument) {
-    PRInt32 pos = mDocument->IndexOf(this);
-    if (pos > 0) {
-      sibling = mDocument->GetChildAt(pos - 1);
-    }
+  nsINode *parent = GetNodeParent();
+  if (!parent) {
+    return NS_OK;
   }
 
-  if (sibling) {
-    rv = CallQueryInterface(sibling, aPrevSibling);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "Must be a DOM Node");
-  } else {
-    *aPrevSibling = nsnull;
-  }
+  PRInt32 pos = parent->IndexOf(this);
+  nsIContent *sibling = parent->GetChildAt(pos - 1);
 
-  return rv;
+  return sibling ? CallQueryInterface(sibling, aPrevSibling) : NS_OK;
 }
 
 nsresult
 nsGenericDOMDataNode::GetNextSibling(nsIDOMNode** aNextSibling)
 {
-  nsresult rv = NS_OK;
+  *aNextSibling = nsnull;
 
-  nsIContent *sibling = nsnull;
-  nsIContent *parent = GetParent();
-  if (parent) {
-    PRInt32 pos = parent->IndexOf(this);
-    if (pos > -1) {
-      sibling = parent->GetChildAt(pos + 1);
-    }
-  }
-  else if (mDocument) {
-    PRInt32 pos = mDocument->IndexOf(this);
-    if (pos > -1) {
-      sibling = mDocument->GetChildAt(pos + 1);
-    }
+  nsINode *parent = GetNodeParent();
+  if (!parent) {
+    return NS_OK;
   }
 
-  if (sibling) {
-    rv = CallQueryInterface(sibling, aNextSibling);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "Must be a DOM Node");
-  } else {
-    *aNextSibling = nsnull;
-  }
+  PRInt32 pos = parent->IndexOf(this);
+  nsIContent *sibling = parent->GetChildAt(pos + 1);
 
-  return rv;
+  return sibling ? CallQueryInterface(sibling, aNextSibling) : NS_OK;
 }
 
 nsresult
 nsGenericDOMDataNode::GetChildNodes(nsIDOMNodeList** aChildNodes)
 {
-  // XXX Since we believe this won't be done very often, we won't
-  // burn another slot in the data node and just create a new
-  // (empty) childNodes list every time we're asked.
-  nsChildContentList* list = new nsChildContentList(nsnull);
-  if (!list) {
-    return NS_ERROR_OUT_OF_MEMORY;
+  *aChildNodes = nsnull;
+  nsDataSlots *slots = GetDataSlots();
+  NS_ENSURE_TRUE(slots, NS_ERROR_OUT_OF_MEMORY);
+
+  if (!slots->mChildNodes) {
+    slots->mChildNodes = new nsChildContentList(this);
+    NS_ENSURE_TRUE(slots->mChildNodes, NS_ERROR_OUT_OF_MEMORY);
   }
 
-  return CallQueryInterface(list, aChildNodes);
+  NS_ADDREF(*aChildNodes = slots->mChildNodes);
+  return NS_OK;
 }
 
 nsresult
@@ -254,7 +238,8 @@ nsGenericDOMDataNode::IsSupported(const nsAString& aFeature,
                                   const nsAString& aVersion,
                                   PRBool* aReturn)
 {
-  return nsGenericElement::InternalIsSupported(aFeature, aVersion, aReturn);
+  return nsGenericElement::InternalIsSupported(NS_STATIC_CAST(nsIContent*, this),
+                                               aFeature, aVersion, aReturn);
 }
 
 nsresult
@@ -312,7 +297,7 @@ nsGenericDOMDataNode::LookupNamespaceURI(const nsAString& aNamespacePrefix,
 // Implementation of nsIDOMCharacterData
 
 nsresult
-nsGenericDOMDataNode::GetData(nsAString& aData)
+nsGenericDOMDataNode::GetData(nsAString& aData) const
 {
   if (mText.Is2b()) {
     aData.Assign(mText.Get2b(), mText.GetLength());
@@ -323,7 +308,7 @@ nsGenericDOMDataNode::GetData(nsAString& aData)
     const char *data = mText.Get1b();
 
     if (data) {
-      CopyASCIItoUCS2(Substring(data, data + mText.GetLength()), aData);
+      CopyASCIItoUTF16(Substring(data, data + mText.GetLength()), aData);
     } else {
       aData.Truncate();
     }
@@ -339,13 +324,11 @@ nsGenericDOMDataNode::SetData(const nsAString& aData)
   // we can lie and say we are deleting all the text, since in a total
   // text replacement we should just collapse all the ranges.
 
-  nsVoidArray *rangeList = LookupRangeList();
+  const nsVoidArray *rangeList = GetRangeList();
   if (rangeList) {
     nsRange::TextOwnerChanged(this, rangeList, 0, mText.GetLength(), 0);
   }
 
-  nsCOMPtr<nsITextContent> textContent = do_QueryInterface(this);
-
   SetText(aData, PR_TRUE);
 
   return NS_OK;
@@ -382,7 +365,7 @@ nsGenericDOMDataNode::SubstringData(PRUint32 aStart, PRUint32 aCount,
     // terminated strings.
 
     const char *data = mText.Get1b() + aStart;
-    CopyASCIItoUCS2(Substring(data, data + amount), aReturn);
+    CopyASCIItoUTF16(Substring(data, data + amount), aReturn);
   }
 
   return NS_OK;
@@ -393,39 +376,40 @@ nsGenericDOMDataNode::SubstringData(PRUint32 aStart, PRUint32 aCount,
 nsresult
 nsGenericDOMDataNode::AppendData(const nsAString& aData)
 {
-#if 1
-  PRInt32 length = 0;
+  // Apparently this is called often enough that we don't want to just simply
+  // call SetText like ReplaceData does. See bug 77585 and comment in
+  // ReplaceData.
+  
+  // FIXME, but 330872: We can't call BeginUpdate here because it confuses the
+  // poor little nsHTMLContentSink.
+  // mozAutoDocUpdate updateBatch(document, UPDATE_CONTENT_MODEL, PR_TRUE);
 
-  // See bugzilla bug 77585.
-  if (mText.Is2b() || (!IsASCII(aData))) {
-    nsAutoString old_data;
-    mText.AppendTo(old_data);
-    length = old_data.Length();
-    // XXXjag We'd like to just say |old_data + aData|, but due
-    // to issues with dependent concatenation and sliding (sub)strings
-    // we'll just have to copy for now. See bug 121841 for details.
-    old_data.Append(aData);
-    SetText(old_data, PR_FALSE);
-  } else {
-    // We know aData and the current data is ASCII, so use a
-    // nsC*String, no need for any fancy unicode stuff here.
-    nsCAutoString old_data;
-    mText.AppendTo(old_data);
-    length = old_data.Length();
-    LossyAppendUTF16toASCII(aData, old_data);
-    SetText(old_data.get(), old_data.Length(), PR_FALSE);
+  PRBool haveMutationListeners =
+    nsContentUtils::HasMutationListeners(this,
+      NS_EVENT_BITS_MUTATION_CHARACTERDATAMODIFIED);
+
+  nsCOMPtr<nsIAtom> oldValue;
+  if (haveMutationListeners) {
+    oldValue = GetCurrentValueAtom();
+  }
+    
+  mText.Append(aData);
+
+  SetBidiStatus();
+
+  if (haveMutationListeners) {
+    nsMutationEvent mutation(PR_TRUE, NS_MUTATION_CHARACTERDATAMODIFIED);
+
+    mutation.mPrevAttrValue = oldValue;
+    mutation.mNewAttrValue = GetCurrentValueAtom();
+
+    nsEventDispatcher::Dispatch(this, nsnull, &mutation);
   }
 
-  // Trigger a reflow
-  nsIDocument *document = GetCurrentDoc();
-  if (document) {
-    document->CharacterDataChanged(this, PR_TRUE);
-  }
+  // Notify observers
+  nsNodeUtils::CharacterDataChanged(this, PR_TRUE);
 
   return NS_OK;
-#else
-  return ReplaceData(mText.GetLength(), 0, aData);
-#endif
 }
 
 nsresult
@@ -438,8 +422,7 @@ nsGenericDOMDataNode::InsertData(PRUint32 aOffset,
 nsresult
 nsGenericDOMDataNode::DeleteData(PRUint32 aOffset, PRUint32 aCount)
 {
-  nsAutoString empty;
-  return ReplaceData(aOffset, aCount, empty);
+  return ReplaceData(aOffset, aCount, EmptyString());
 }
 
 nsresult
@@ -452,6 +435,15 @@ nsGenericDOMDataNode::ReplaceData(PRUint32 aOffset, PRUint32 aCount,
     return NS_ERROR_DOM_INDEX_SIZE_ERR;
   }
 
+  // Fast path (hit by editor when typing at the end of the paragraph, for
+  // example): aOffset == textLength (so just doing an append; note that in
+  // this case any value of aCount would just get converted to 0 by the very
+  // next if block).  Call AppendData so that we pass PR_TRUE for our aAppend
+  // arg to CharacterDataChanged.
+  if (aOffset == textLength) {
+    return AppendData(aData);
+  }
+
   // Allocate new buffer
   PRUint32 endOffset = aOffset + aCount;
   if (endOffset > textLength) {
@@ -466,7 +458,7 @@ nsGenericDOMDataNode::ReplaceData(PRUint32 aOffset, PRUint32 aCount,
   }
 
   // inform any enclosed ranges of change
-  nsVoidArray *rangeList = LookupRangeList();
+  const nsVoidArray *rangeList = GetRangeList();
   if (rangeList) {
     nsRange::TextOwnerChanged(this, rangeList, aOffset, endOffset, dataLength);
   }
@@ -493,45 +485,6 @@ nsGenericDOMDataNode::ReplaceData(PRUint32 aOffset, PRUint32 aCount,
 
 //----------------------------------------------------------------------
 
-nsresult
-nsGenericDOMDataNode::GetListenerManager(nsIEventListenerManager **aResult)
-{
-  nsCOMPtr<nsIEventListenerManager> listener_manager;
-  LookupListenerManager(getter_AddRefs(listener_manager));
-
-  if (listener_manager) {
-    *aResult = listener_manager;
-    NS_ADDREF(*aResult);
-
-    return NS_OK;
-  }
-
-  if (!nsGenericElement::sEventListenerManagersHash.ops) {
-    nsresult rv = nsGenericElement::InitHashes();
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  nsresult rv = NS_NewEventListenerManager(aResult);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Add a mapping to the hash table
-  EventListenerManagerMapEntry *entry =
-    NS_STATIC_CAST(EventListenerManagerMapEntry *,
-                   PL_DHashTableOperate(&nsGenericElement::
-                                        sEventListenerManagersHash, this,
-                                        PL_DHASH_ADD));
-
-  entry->mListenerManager = *aResult;
-
-  entry->mListenerManager->SetListenerTarget(this);
-
-  SetHasEventListenerManager();
-
-  return NS_OK;
-}
-
-//----------------------------------------------------------------------
-
 // Implementation of nsIContent
 
 #ifdef DEBUG
@@ -545,12 +498,12 @@ nsGenericDOMDataNode::ToCString(nsAString& aBuf, PRInt32 aOffset,
 
     while (cp < end) {
       PRUnichar ch = *cp++;
-      if (ch == '\r') {
-        aBuf.AppendLiteral("\\r");
-      } else if (ch == '\n') {
-        aBuf.AppendLiteral("\\n");
-      } else if (ch == '\t') {
-        aBuf.AppendLiteral("\\t");
+      if (ch == '&') {
+        aBuf.AppendLiteral("&amp;");
+      } else if (ch == '<') {
+        aBuf.AppendLiteral("&lt;");
+      } else if (ch == '>') {
+        aBuf.AppendLiteral("&gt;");
       } else if ((ch < ' ') || (ch >= 127)) {
         char buf[10];
         PR_snprintf(buf, sizeof(buf), "\\u%04x", ch);
@@ -565,12 +518,12 @@ nsGenericDOMDataNode::ToCString(nsAString& aBuf, PRInt32 aOffset,
 
     while (cp < end) {
       PRUnichar ch = *cp++;
-      if (ch == '\r') {
-        aBuf.AppendLiteral("\\r");
-      } else if (ch == '\n') {
-        aBuf.AppendLiteral("\\n");
-      } else if (ch == '\t') {
-        aBuf.AppendLiteral("\\t");
+      if (ch == '&') {
+        aBuf.AppendLiteral("&amp;");
+      } else if (ch == '<') {
+        aBuf.AppendLiteral("&lt;");
+      } else if (ch == '>') {
+        aBuf.AppendLiteral("&gt;");
       } else if ((ch < ' ') || (ch >= 127)) {
         char buf[10];
         PR_snprintf(buf, sizeof(buf), "\\u%04x", ch);
@@ -583,49 +536,169 @@ nsGenericDOMDataNode::ToCString(nsAString& aBuf, PRInt32 aOffset,
 }
 #endif
 
-nsIDocument*
-nsGenericDOMDataNode::GetDocument() const
+/**
+ * See comment for nsGenericElement::GetSCCIndex
+ */
+nsIDOMGCParticipant*
+nsGenericDOMDataNode::GetSCCIndex()
 {
-  return GetCurrentDoc();
-}
-
-void
-nsGenericDOMDataNode::SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                                  PRBool aCompileEventHandlers)
-{
-  mDocument = aDocument;
-  if (mDocument && mText.IsBidi()) {
-    aDocument->SetBidiEnabled(PR_TRUE);
+  // This is an optimized way of walking nsIDOMNode::GetParentNode to
+  // the top of the tree.
+  nsINode *top = GetCurrentDoc();
+  if (!top) {
+    top = this;
+    nsINode *parent;
+    while ((parent = top->GetNodeParent())) {
+      top = parent;
+    }
   }
+
+  return top;
 }
 
 void
-nsGenericDOMDataNode::SetParent(nsIContent* aParent)
+nsGenericDOMDataNode::AppendReachableList(nsCOMArray<nsIDOMGCParticipant>& aArray)
 {
-  PtrBits new_bits = NS_REINTERPRET_CAST(PtrBits, aParent);
+  NS_ASSERTION(GetCurrentDoc() == nsnull,
+               "shouldn't be an SCC index if we're in a doc");
 
-  new_bits |= mParentPtrBits & nsIContent::kParentBitMask;
-
-  mParentPtrBits = new_bits;
+  // This node is the root of a subtree that's been removed from the
+  // document (since AppendReachableList is only called on SCC index
+  // nodes).  The document is reachable from it (through
+  // .ownerDocument), but it's not reachable from the document.
+  aArray.AppendObject(GetOwnerDoc());
 }
 
-PRBool
-nsGenericDOMDataNode::IsNativeAnonymous() const
+nsresult
+nsGenericDOMDataNode::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                                 nsIContent* aBindingParent,
+                                 PRBool aCompileEventHandlers)
 {
-  nsIContent* parent = GetParent();
-  return parent && parent->IsNativeAnonymous();
+  NS_PRECONDITION(aParent || aDocument, "Must have document if no parent!");
+  // XXXbz XUL elements are confused about their current doc when they're
+  // cloned, so we don't assert if aParent is a XUL element and aDocument is
+  // null, even if aParent->GetCurrentDoc() is non-null
+  //  NS_PRECONDITION(!aParent || aDocument == aParent->GetCurrentDoc(),
+  //                  "aDocument must be current doc of aParent");
+  NS_PRECONDITION(!aParent ||
+                  (aParent->IsNodeOfType(eXUL) && aDocument == nsnull) ||
+                  aDocument == aParent->GetCurrentDoc(),
+                  "aDocument must be current doc of aParent");
+  NS_PRECONDITION(!GetCurrentDoc() && !IsInDoc(),
+                  "Already have a document.  Unbind first!");
+  // Note that as we recurse into the kids, they'll have a non-null parent.  So
+  // only assert if our parent is _changing_ while we have a parent.
+  NS_PRECONDITION(!GetParent() || aParent == GetParent(),
+                  "Already have a parent.  Unbind first!");
+  NS_PRECONDITION(!GetBindingParent() ||
+                  aBindingParent == GetBindingParent() ||
+                  (!aBindingParent && aParent &&
+                   aParent->GetBindingParent() == GetBindingParent()),
+                  "Already have a binding parent.  Unbind first!");
+
+  if (!aBindingParent && aParent) {
+    aBindingParent = aParent->GetBindingParent();
+  }
+
+  // First set the binding parent
+  if (aBindingParent) {
+    nsDataSlots *slots = GetDataSlots();
+    NS_ENSURE_TRUE(slots, NS_ERROR_OUT_OF_MEMORY);
+
+    slots->mBindingParent = aBindingParent; // Weak, so no addref happens.
+  }
+
+  // Set parent
+  if (aParent) {
+    mParentPtrBits =
+      NS_REINTERPRET_CAST(PtrBits, aParent) | PARENT_BIT_PARENT_IS_CONTENT;
+  }
+  else {
+    mParentPtrBits = NS_REINTERPRET_CAST(PtrBits, aDocument);
+  }
+
+  nsresult rv = NS_OK;
+  nsCOMPtr<nsIDocument> oldOwnerDocument = GetOwnerDoc();
+  nsIDocument *newOwnerDocument;
+  nsNodeInfoManager* nodeInfoManager;
+
+  // XXXbz sXBL/XBL2 issue!
+
+  // Set document
+  if (aDocument) {
+    // XXX See the comment in nsGenericElement::BindToTree
+    mParentPtrBits |= PARENT_BIT_INDOCUMENT;
+    if (mText.IsBidi()) {
+      aDocument->SetBidiEnabled(PR_TRUE);
+    }
+
+    newOwnerDocument = aDocument;
+    nodeInfoManager = newOwnerDocument->NodeInfoManager();
+  } else {
+    newOwnerDocument = aParent->GetOwnerDoc();
+    nodeInfoManager = aParent->NodeInfo()->NodeInfoManager();
+  }
+
+  if (mNodeInfo->NodeInfoManager() != nodeInfoManager) {
+    nsCOMPtr<nsINodeInfo> newNodeInfo;
+    // optimize common cases
+    nsIAtom* name = mNodeInfo->NameAtom();
+    if (name == nsLayoutAtoms::textTagName) {
+      newNodeInfo = nodeInfoManager->GetTextNodeInfo();
+      NS_ENSURE_TRUE(newNodeInfo, NS_ERROR_OUT_OF_MEMORY);
+    }
+    else if (name == nsLayoutAtoms::commentTagName) {
+      newNodeInfo = nodeInfoManager->GetCommentNodeInfo();
+      NS_ENSURE_TRUE(newNodeInfo, NS_ERROR_OUT_OF_MEMORY);
+    }
+    else {
+      rv = nodeInfoManager->GetNodeInfo(name,
+                                        mNodeInfo->GetPrefixAtom(),
+                                        mNodeInfo->NamespaceID(),
+                                        getter_AddRefs(newNodeInfo));
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
+    mNodeInfo.swap(newNodeInfo);
+  }
+
+  if (oldOwnerDocument && oldOwnerDocument != newOwnerDocument &&
+      HasProperties()) {
+    nsPropertyTable *oldTable = oldOwnerDocument->PropertyTable();
+    if (newOwnerDocument) {
+      nsPropertyTable *newTable = newOwnerDocument->PropertyTable();
+
+      oldTable->TransferOrDeleteAllPropertiesFor(this, newTable);
+    }
+    else {
+      oldTable->DeleteAllPropertiesFor(this);
+    }
+  }
+
+  NS_POSTCONDITION(aDocument == GetCurrentDoc(), "Bound to wrong document");
+  NS_POSTCONDITION(aParent == GetParent(), "Bound to wrong parent");
+  NS_POSTCONDITION(aBindingParent == GetBindingParent(),
+                   "Bound to wrong binding parent");
+
+  return NS_OK;
 }
 
 void
-nsGenericDOMDataNode::SetNativeAnonymous(PRBool aAnonymous)
+nsGenericDOMDataNode::UnbindFromTree(PRBool aDeep, PRBool aNullParent)
 {
-  // XXX Need to fix this to do something - bug 165110
-}
+  nsIDocument *document = GetCurrentDoc();
+  if (document) {
+    // Notify XBL- & nsIAnonymousContentCreator-generated
+    // anonymous content that the document is changing.
+    // This is needed to update the insertion point.
+    document->BindingManager()->ChangeDocumentFor(this, document, nsnull);
+  }
 
-void
-nsGenericDOMDataNode::GetNameSpaceID(PRInt32* aID) const
-{
-  *aID = kNameSpaceID_None;
+  mParentPtrBits = aNullParent ? 0 : mParentPtrBits & ~PARENT_BIT_INDOCUMENT;
+
+  nsDataSlots *slots = GetExistingDataSlots();
+  if (slots) {
+    slots->mBindingParent = nsnull;
+  }
 }
 
 nsIAtom *
@@ -634,12 +707,6 @@ nsGenericDOMDataNode::GetIDAttributeName() const
   return nsnull;
 }
 
-nsIAtom *
-nsGenericDOMDataNode::GetClassAttributeName() const
-{
-  return nsnull;
-}
-
 already_AddRefed<nsINodeInfo>
 nsGenericDOMDataNode::GetExistingAttrNameFromQName(const nsAString& aStr) const
 {
@@ -661,13 +728,13 @@ nsGenericDOMDataNode::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr,
   return NS_OK;
 }
 
-nsresult
+PRBool
 nsGenericDOMDataNode::GetAttr(PRInt32 aNameSpaceID, nsIAtom *aAttr,
                               nsAString& aResult) const
 {
   aResult.Truncate();
 
-  return NS_CONTENT_ATTR_NOT_THERE;
+  return PR_FALSE;
 }
 
 PRBool
@@ -676,15 +743,10 @@ nsGenericDOMDataNode::HasAttr(PRInt32 aNameSpaceID, nsIAtom *aAttribute) const
   return PR_FALSE;
 }
 
-nsresult
-nsGenericDOMDataNode::GetAttrNameAt(PRUint32 aIndex, PRInt32* aNameSpaceID,
-                                    nsIAtom** aName, nsIAtom** aPrefix) const
+const nsAttrName*
+nsGenericDOMDataNode::GetAttrNameAt(PRUint32 aIndex) const
 {
-  *aNameSpaceID = kNameSpaceID_None;
-  *aName = nsnull;
-  *aPrefix = nsnull;
-
-  return NS_ERROR_ILLEGAL_VALUE;
+  return nsnull;
 }
 
 PRUint32
@@ -694,105 +756,26 @@ nsGenericDOMDataNode::GetAttrCount() const
 }
 
 nsresult
-nsGenericDOMDataNode::HandleDOMEvent(nsPresContext* aPresContext,
-                                     nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
-                                     PRUint32 aFlags,
-                                     nsEventStatus* aEventStatus)
+nsGenericDOMDataNode::PreHandleEvent(nsEventChainPreVisitor& aVisitor)
 {
-  nsresult ret = NS_OK;
-  nsIDOMEvent* domEvent = nsnull;
-
-  PRBool externalDOMEvent = PR_FALSE;
-
-  if (NS_EVENT_FLAG_INIT & aFlags) {
-    if (!aDOMEvent) {
-      aDOMEvent = &domEvent;
-    } else {
-      externalDOMEvent = PR_TRUE;
-    }
-
-    aEvent->flags |= aFlags;
-    aFlags &= ~(NS_EVENT_FLAG_CANT_BUBBLE | NS_EVENT_FLAG_CANT_CANCEL);
-    aFlags |= NS_EVENT_FLAG_BUBBLE | NS_EVENT_FLAG_CAPTURE;
-  }
-
-  nsIContent *parent = GetParent();
-
-  //Capturing stage evaluation
-  if (NS_EVENT_FLAG_CAPTURE & aFlags) {
-    //Initiate capturing phase.  Special case first call to document
-    if (parent) {
-      parent->HandleDOMEvent(aPresContext, aEvent, aDOMEvent,
-                             aFlags & NS_EVENT_CAPTURE_MASK, aEventStatus);
-    }
-    else {
-      nsIDocument *document = GetCurrentDoc();
-      if (document) {
-        document->HandleDOMEvent(aPresContext, aEvent, aDOMEvent,
-                                 aFlags & NS_EVENT_CAPTURE_MASK,
-                                 aEventStatus);
-      }
-    }
-  }
-
-  nsCOMPtr<nsIEventListenerManager> listener_manager;
-  LookupListenerManager(getter_AddRefs(listener_manager));
-
-  //Local handling stage
-  //Check for null ELM, check if we're a non-bubbling event in the bubbling state (bubbling state
-  //is indicated by the presence of the NS_EVENT_FLAG_BUBBLE flag and not the NS_EVENT_FLAG_INIT), and check 
-  //if we're a no content dispatch event
-  if (listener_manager &&
-       !(NS_EVENT_FLAG_CANT_BUBBLE & aEvent->flags && NS_EVENT_FLAG_BUBBLE & aFlags && !(NS_EVENT_FLAG_INIT & aFlags)) &&
-       !(aEvent->flags & NS_EVENT_FLAG_NO_CONTENT_DISPATCH)) {
-    aEvent->flags |= aFlags;
-    listener_manager->HandleEvent(aPresContext, aEvent, aDOMEvent, nsnull,
-                                  aFlags, aEventStatus);
-    aEvent->flags &= ~aFlags;
-  }
-
-  //Bubbling stage
-  if (NS_EVENT_FLAG_BUBBLE & aFlags && parent) {
-    ret = parent->HandleDOMEvent(aPresContext, aEvent, aDOMEvent,
-                                 aFlags & NS_EVENT_BUBBLE_MASK, aEventStatus);
-  }
-
-  if (NS_EVENT_FLAG_INIT & aFlags) {
-    // We're leaving the DOM event loop so if we created a DOM event,
-    // release here.
-
-    if (!externalDOMEvent && *aDOMEvent) {
-      if (0 != (*aDOMEvent)->Release()) {
-        // Okay, so someone in the DOM loop (a listener, JS object)
-        // still has a ref to the DOM Event but the internal data
-        // hasn't been malloc'd.  Force a copy of the data here so the
-        // DOM Event is still valid.
-
-        nsCOMPtr<nsIPrivateDOMEvent> privateEvent =
-          do_QueryInterface(*aDOMEvent);
-
-        if (privateEvent) {
-          privateEvent->DuplicatePrivateData();
-        }
-      }
-    }
-
-    aDOMEvent = nsnull;
-  }
-
-  return ret;
+  return nsGenericElement::doPreHandleEvent(this, aVisitor);
 }
 
-PRUint32
-nsGenericDOMDataNode::ContentID() const
+nsresult
+nsGenericDOMDataNode::PostHandleEvent(nsEventChainPostVisitor& /*aVisitor*/)
 {
-  return 0;
+  return NS_OK;
 }
 
-nsINodeInfo *
-nsGenericDOMDataNode::GetNodeInfo() const
+nsresult
+nsGenericDOMDataNode::DispatchDOMEvent(nsEvent* aEvent,
+                                       nsIDOMEvent* aDOMEvent,
+                                       nsPresContext* aPresContext,
+                                       nsEventStatus* aEventStatus)
 {
-  return nsnull;
+  return nsEventDispatcher::DispatchDOMEvent(NS_STATIC_CAST(nsINode*, this),
+                                             aEvent, aDOMEvent,
+                                             aPresContext, aEventStatus);
 }
 
 PRUint32
@@ -808,21 +791,14 @@ nsGenericDOMDataNode::GetChildAt(PRUint32 aIndex) const
 }
 
 PRInt32
-nsGenericDOMDataNode::IndexOf(nsIContent* aPossibleChild) const
+nsGenericDOMDataNode::IndexOf(nsINode* aPossibleChild) const
 {
   return -1;
 }
 
 nsresult
 nsGenericDOMDataNode::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                    PRBool aNotify, PRBool aDeepSetDocument)
-{
-  return NS_OK;
-}
-
-nsresult
-nsGenericDOMDataNode::AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                    PRBool aDeepSetDocument)
+                                    PRBool aNotify)
 {
   return NS_OK;
 }
@@ -833,103 +809,28 @@ nsGenericDOMDataNode::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
   return NS_OK;
 }
 
-nsresult
-nsGenericDOMDataNode::RangeAdd(nsIDOMRange* aRange)
+// virtual
+PRBool
+nsGenericDOMDataNode::MayHaveFrame() const
 {
-  // lazy allocation of range list
-
-  if (!nsGenericElement::sRangeListsHash.ops) {
-    nsresult rv = nsGenericElement::InitHashes();
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  RangeListMapEntry *entry =
-    NS_STATIC_CAST(RangeListMapEntry *,
-                   PL_DHashTableOperate(&nsGenericElement::sRangeListsHash,
-                                        this, PL_DHASH_ADD));
-
-  if (!entry) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  nsVoidArray *range_list = entry->mRangeList;
-
-  if (!range_list) {
-    range_list = new nsAutoVoidArray();
-
-    if (!range_list) {
-      PL_DHashTableRawRemove(&nsGenericElement::sRangeListsHash, entry);
-
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-
-    entry->mRangeList = range_list;
-
-    SetHasRangeList();
-  } else {
-    // Make sure we don't add a range that is already
-    // in the list!
-    PRInt32 i = range_list->IndexOf(aRange);
-
-    if (i >= 0) {
-      // Range is already in the list, so there is nothing to do!
-
-      return NS_OK;
-    }
-  }
-
-  // dont need to addref - this call is made by the range object itself
-  PRBool rv = range_list->AppendElement(aRange);
-
-  return rv ? NS_OK : NS_ERROR_FAILURE;
-}
-
-
-void
-nsGenericDOMDataNode::RangeRemove(nsIDOMRange* aRange)
-{
-  if (!CouldHaveRangeList()) {
-    return;
-  }
-
-  RangeListMapEntry *entry =
-    NS_STATIC_CAST(RangeListMapEntry *,
-                   PL_DHashTableOperate(&nsGenericElement::sRangeListsHash,
-                                        this, PL_DHASH_LOOKUP));
-
-  // Don't need to release: this call is made by the range object itself.
-  if (entry && PL_DHASH_ENTRY_IS_BUSY(entry) &&
-      entry->mRangeList->RemoveElement(aRange) &&
-      entry->mRangeList->Count() == 0) {
-    PL_DHashTableRawRemove(&nsGenericElement::sRangeListsHash, entry);
-  }
-}
-
-const nsVoidArray *
-nsGenericDOMDataNode::GetRangeList() const
-{
-  return LookupRangeList();
+  nsIContent* parent = GetParent();
+  return parent && parent->MayHaveFrame();
 }
 
 nsIContent *
 nsGenericDOMDataNode::GetBindingParent() const
 {
-  nsIContent* parent = GetParent();
-  return parent ? parent->GetBindingParent() : nsnull;
-}
-
-nsresult
-nsGenericDOMDataNode::SetBindingParent(nsIContent* aParent)
-{
-  return NS_OK;
+  nsDataSlots *slots = GetExistingDataSlots();
+  return slots ? slots->mBindingParent : nsnull;
 }
 
 PRBool
-nsGenericDOMDataNode::IsContentOfType(PRUint32 aFlags) const
+nsGenericDOMDataNode::IsNodeOfType(PRUint32 aFlags) const
 {
-  return PR_FALSE;
+  return !(aFlags & ~(eCONTENT | eDATA_NODE));
 }
 
+
 #ifdef DEBUG
 void
 nsGenericDOMDataNode::List(FILE* out, PRInt32 aIndent) const
@@ -953,8 +854,9 @@ nsGenericDOMDataNode::GetBaseURI() const
   }
 
   nsIURI *uri;
-  if (mDocument) {
-    NS_IF_ADDREF(uri = mDocument->GetBaseURI());
+  nsIDocument *doc = GetOwnerDoc();
+  if (doc) {
+    NS_IF_ADDREF(uri = doc->GetBaseURI());
   }
   else {
     uri = nsnull;
@@ -963,6 +865,12 @@ nsGenericDOMDataNode::GetBaseURI() const
   return uri;
 }
 
+nsINode::nsSlots*
+nsGenericDOMDataNode::CreateSlots()
+{
+  return new nsDataSlots(mFlagsOrSlots);
+}
+
 //----------------------------------------------------------------------
 
 // Implementation of the nsIDOMText interface
@@ -989,11 +897,11 @@ nsGenericDOMDataNode::SplitText(PRUint32 aOffset, nsIDOMText** aReturn)
   }
 
   /*
-   * Use CloneContent() for creating the new node so that the new node is of
-   * same class as this node!
+   * Use Clone for creating the new node so that the new node is of same class
+   * as this node!
    */
 
-  nsCOMPtr<nsITextContent> newContent = CloneContent(PR_FALSE, nsnull);
+  nsCOMPtr<nsIContent> newContent = CloneDataNode(mNodeInfo, PR_FALSE);
   if (!newContent) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -1007,20 +915,21 @@ nsGenericDOMDataNode::SplitText(PRUint32 aOffset, nsIDOMText** aReturn)
 
     nsCOMPtr<nsIContent> content(do_QueryInterface(newContent));
 
-    parent->InsertChildAt(content, index+1, PR_TRUE, PR_FALSE);
+    parent->InsertChildAt(content, index+1, PR_TRUE);
   }
 
-  // XXX Shouldn't we handle the case where this is a child of the document?
+  // No need to handle the case of document being the parent since text
+  // isn't allowed as direct child of documents
 
   return CallQueryInterface(newContent, aReturn);
 }
 
 //----------------------------------------------------------------------
 
-// Implementation of the nsITextContent interface
+// Implementation of the nsIContent interface text functions
 
 const nsTextFragment *
-nsGenericDOMDataNode::Text()
+nsGenericDOMDataNode::GetText()
 {
   return &mText;
 }
@@ -1031,7 +940,7 @@ nsGenericDOMDataNode::TextLength()
   return mText.GetLength();
 }
 
-void
+nsresult
 nsGenericDOMDataNode::SetText(const PRUnichar* aBuffer,
                               PRUint32 aLength,
                               PRBool aNotify)
@@ -1039,27 +948,28 @@ nsGenericDOMDataNode::SetText(const PRUnichar* aBuffer,
   if (!aBuffer) {
     NS_ERROR("Null buffer passed to SetText()!");
 
-    return;
+    return NS_ERROR_NULL_POINTER;
   }
 
   nsIDocument *document = GetCurrentDoc();
   mozAutoDocUpdate updateBatch(document, UPDATE_CONTENT_MODEL, aNotify);
 
-  PRBool haveMutationListeners =
-    document && nsGenericElement::HasMutationListeners(this, NS_EVENT_BITS_MUTATION_CHARACTERDATAMODIFIED);
+  PRBool haveMutationListeners = aNotify &&
+    nsContentUtils::HasMutationListeners(this,
+      NS_EVENT_BITS_MUTATION_CHARACTERDATAMODIFIED);
 
   nsCOMPtr<nsIAtom> oldValue;
   if (haveMutationListeners) {
     oldValue = GetCurrentValueAtom();
   }
     
+  // XXX Add OOM checking to this
   mText.SetTo(aBuffer, aLength);
 
   SetBidiStatus();
 
   if (haveMutationListeners) {
-    nsCOMPtr<nsIDOMEventTarget> node(do_QueryInterface(this));
-    nsMutationEvent mutation(NS_MUTATION_CHARACTERDATAMODIFIED, node);
+    nsMutationEvent mutation(PR_TRUE, NS_MUTATION_CHARACTERDATAMODIFIED);
 
     mutation.mPrevAttrValue = oldValue;
     if (aLength > 0) {
@@ -1069,127 +979,37 @@ nsGenericDOMDataNode::SetText(const PRUnichar* aBuffer,
         do_GetAtom(Substring(aBuffer, aBuffer + aLength));
     }
 
-    nsEventStatus status = nsEventStatus_eIgnore;
-    HandleDOMEvent(nsnull, &mutation, nsnull,
-                   NS_EVENT_FLAG_INIT, &status);
+    nsEventDispatcher::Dispatch(this, nsnull, &mutation);
   }
 
   // Trigger a reflow
-  if (aNotify && document) {
-    document->CharacterDataChanged(this, PR_FALSE);
-  }
-}
-
-void
-nsGenericDOMDataNode::SetText(const char* aBuffer, PRUint32 aLength,
-                              PRBool aNotify)
-{
-  if (!aBuffer) {
-    NS_ERROR("Null buffer passed to SetText()!");
-
-    return;
+  if (aNotify) {
+    nsNodeUtils::CharacterDataChanged(this, PR_FALSE);
   }
 
-  nsIDocument *document = GetCurrentDoc();
-  mozAutoDocUpdate updateBatch(document, UPDATE_CONTENT_MODEL, aNotify);
-
-  PRBool haveMutationListeners =
-    document && nsGenericElement::HasMutationListeners(this, NS_EVENT_BITS_MUTATION_CHARACTERDATAMODIFIED);
-
-  nsCOMPtr<nsIAtom> oldValue;
-  if (haveMutationListeners) {
-    oldValue = GetCurrentValueAtom();
-  }
-    
-  mText.SetTo(aBuffer, aLength);
-
-  if (haveMutationListeners) {
-    nsCOMPtr<nsIDOMEventTarget> node(do_QueryInterface(this));
-    nsMutationEvent mutation(NS_MUTATION_CHARACTERDATAMODIFIED, node);
-
-    mutation.mPrevAttrValue = oldValue;
-    if (aLength > 0) {
-      // Must use Substring() since nsDependentCString() requires null
-      // terminated strings.
-      mutation.mNewAttrValue =
-        do_GetAtom(Substring(aBuffer, aBuffer + aLength));
-    }
-
-    nsEventStatus status = nsEventStatus_eIgnore;
-    HandleDOMEvent(nsnull, &mutation, nsnull,
-                   NS_EVENT_FLAG_INIT, &status);
-  }
-
-  // Trigger a reflow
-  if (aNotify && document) {
-    document->CharacterDataChanged(this, PR_FALSE);
-  }
-}
-
-void
-nsGenericDOMDataNode::SetText(const nsAString& aStr,
-                              PRBool aNotify)
-{
-  nsIDocument *document = GetCurrentDoc();
-  mozAutoDocUpdate updateBatch(document, UPDATE_CONTENT_MODEL, aNotify);
-
-  PRBool haveMutationListeners =
-    document && nsGenericElement::HasMutationListeners(this, NS_EVENT_BITS_MUTATION_CHARACTERDATAMODIFIED);
-
-  nsCOMPtr<nsIAtom> oldValue;
-  if (haveMutationListeners) {
-    oldValue = GetCurrentValueAtom();
-  }
-    
-  mText = aStr;
-
-  SetBidiStatus();
-
-  if (haveMutationListeners) {
-    nsCOMPtr<nsIDOMEventTarget> node(do_QueryInterface(this));
-    nsMutationEvent mutation(NS_MUTATION_CHARACTERDATAMODIFIED, node);
-
-    mutation.mPrevAttrValue = oldValue;
-    if (!aStr.IsEmpty())
-      mutation.mNewAttrValue = do_GetAtom(aStr);
-    nsEventStatus status = nsEventStatus_eIgnore;
-    HandleDOMEvent(nsnull, &mutation, nsnull,
-                   NS_EVENT_FLAG_INIT, &status);
-  }
-
-  // Trigger a reflow
-  if (aNotify && document) {
-    document->CharacterDataChanged(this, PR_FALSE);
-  }
+  return NS_OK;
 }
 
 PRBool
-nsGenericDOMDataNode::IsOnlyWhitespace()
+nsGenericDOMDataNode::TextIsOnlyWhitespace()
 {
-  nsTextFragment& frag = mText;
-  if (frag.Is2b()) {
-    const PRUnichar* cp = frag.Get2b();
-    const PRUnichar* end = cp + frag.GetLength();
+  if (mText.Is2b()) {
+    // The fragment contains non-8bit characters and such characters
+    // are never considered whitespace.
+    return PR_FALSE;
+  }
 
-    while (cp < end) {
-      PRUnichar ch = *cp++;
+  const char* cp = mText.Get1b();
+  const char* end = cp + mText.GetLength();
 
-      if (!XP_IS_SPACE(ch)) {
-        return PR_FALSE;
-      }
+  while (cp < end) {
+    char ch = *cp;
+
+    if (!XP_IS_SPACE(ch)) {
+      return PR_FALSE;
     }
-  } else {
-    const char* cp = frag.Get1b();
-    const char* end = cp + frag.GetLength();
 
-    while (cp < end) {
-      PRUnichar ch = PRUnichar(*(unsigned char*)cp);
-      ++cp;
-
-      if (!XP_IS_SPACE(ch)) {
-        return PR_FALSE;
-      }
-    }
+    ++cp;
   }
 
   return PR_TRUE;
@@ -1201,46 +1021,6 @@ nsGenericDOMDataNode::AppendTextTo(nsAString& aResult)
   mText.AppendTo(aResult);
 }
 
-void
-nsGenericDOMDataNode::LookupListenerManager(nsIEventListenerManager **aListenerManager) const
-{
-  *aListenerManager = nsnull;
-
-  if (!CouldHaveEventListenerManager()) {
-    return;
-  }
-
-  EventListenerManagerMapEntry *entry =
-    NS_STATIC_CAST(EventListenerManagerMapEntry *,
-                   PL_DHashTableOperate(&nsGenericElement::
-                                        sEventListenerManagersHash, this,
-                                        PL_DHASH_LOOKUP));
-
-  if (PL_DHASH_ENTRY_IS_BUSY(entry)) {
-    *aListenerManager = entry->mListenerManager;
-    NS_ADDREF(*aListenerManager);
-  }
-}
-
-nsVoidArray *
-nsGenericDOMDataNode::LookupRangeList() const
-{
-  if (!CouldHaveRangeList()) {
-    return nsnull;
-  }
-
-  RangeListMapEntry *entry =
-    NS_STATIC_CAST(RangeListMapEntry *,
-                   PL_DHashTableOperate(&nsGenericElement::sRangeListsHash,
-                                        this, PL_DHASH_LOOKUP));
-
-  if (PL_DHASH_ENTRY_IS_BUSY(entry)) {
-    return entry->mRangeList;
-  }
-
-  return nsnull;
-}
-
 void nsGenericDOMDataNode::SetBidiStatus()
 {
   nsIDocument *document = GetCurrentDoc();
@@ -1264,11 +1044,54 @@ nsGenericDOMDataNode::GetCurrentValueAtom()
   return NS_NewAtom(val);
 }
 
-already_AddRefed<nsITextContent> 
-nsGenericDOMDataNode::CloneContent(PRBool aCloneText,
-                                   nsIDocument *aOwnerDocument)
+nsIAtom*
+nsGenericDOMDataNode::GetID() const
+{
+  return nsnull;
+}
+
+const nsAttrValue*
+nsGenericDOMDataNode::GetClasses() const
+{
+  return nsnull;
+}
+
+NS_IMETHODIMP
+nsGenericDOMDataNode::WalkContentStyleRules(nsRuleWalker* aRuleWalker)
+{
+  return NS_OK;
+}
+
+nsICSSStyleRule*
+nsGenericDOMDataNode::GetInlineStyleRule()
+{
+  return nsnull;
+}
+
+NS_IMETHODIMP
+nsGenericDOMDataNode::SetInlineStyleRule(nsICSSStyleRule* aStyleRule,
+                                         PRBool aNotify)
+{
+  NS_NOTREACHED("How come we're setting inline style on a non-element?");
+  return NS_ERROR_UNEXPECTED;
+}
+
+NS_IMETHODIMP_(PRBool)
+nsGenericDOMDataNode::IsAttributeMapped(const nsIAtom* aAttribute) const
+{
+  return PR_FALSE;
+}
+
+nsChangeHint
+nsGenericDOMDataNode::GetAttributeChangeHint(const nsIAtom* aAttribute,
+                                             PRInt32 aModType) const
+{
+  NS_NOTREACHED("Shouldn't be calling this!");
+  return nsChangeHint(0);
+}
+
+nsIAtom*
+nsGenericDOMDataNode::GetClassAttributeName() const
 {
-  NS_ERROR("Huh, this shouldn't be called!");
-
   return nsnull;
 }
