@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -35,6 +36,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * nsIContentSerializer implementation that can be used with an
+ * nsIDocumentEncoder to convert an HTML (not XHTML!) DOM to an HTML
+ * string that could be parsed into more or less the original DOM.
+ */
+
 #include "nsHTMLContentSerializer.h"
 
 #include "nsIDOMElement.h"
@@ -57,8 +64,9 @@
 #include "nsCRT.h"
 #include "nsIParserService.h"
 #include "nsContentUtils.h"
-#include "nsILineBreakerFactory.h"
 #include "nsLWBrkCIID.h"
+#include "nsIScriptElement.h"
+#include "nsAttrName.h"
 
 #define kIndentStr NS_LITERAL_STRING("  ")
 #define kLessThan NS_LITERAL_STRING("<")
@@ -66,7 +74,6 @@
 #define kEndTag NS_LITERAL_STRING("</")
 
 static const char kMozStr[] = "moz";
-static NS_DEFINE_CID(kLWBrkCID, NS_LWBRK_CID);
 
 static const PRInt32 kLongLineLen = 128;
 
@@ -162,20 +169,6 @@ nsHTMLContentSerializer::AppendText(nsIDOMText* aText,
     nsCOMPtr<nsIDOMDocument> domDoc;
     aText->GetOwnerDocument(getter_AddRefs(domDoc));
     nsCOMPtr<nsIDocument> document = do_QueryInterface(domDoc);
-    if (document) {
-      mLineBreaker = document->GetLineBreaker();
-    }
-
-    if (!mLineBreaker) {
-      nsresult rv;
-      nsCOMPtr<nsILineBreakerFactory> lf(do_GetService(kLWBrkCID, &rv));
-      if (NS_SUCCEEDED(rv)) {
-        rv = lf->GetBreaker(EmptyString(), getter_AddRefs(mLineBreaker));
-        // Ignore result value.
-        // If we are unable to obtain a line breaker,
-        // we will use our simple fallback logic.
-      }
-    }
   }
 
   nsAutoString data;
@@ -189,6 +182,12 @@ nsHTMLContentSerializer::AppendText(nsIDOMText* aText,
   if (mPreLevel > 0) {
     AppendToStringConvertLF(data, aStr);
   }
+  else if (mFlags & nsIDocumentEncoder::OutputRaw) {
+    PRInt32 lastNewlineOffset = data.RFindChar('\n');
+    AppendToString(data, aStr);
+    if (lastNewlineOffset != kNotFound)
+      mColPos = data.Length() - lastNewlineOffset;
+  }
   else if (!mDoFormat) {
     PRInt32 lastNewlineOffset = kNotFound;
     PRBool hasLongLines = HasLongLines(data, lastNewlineOffset);
@@ -202,12 +201,6 @@ nsHTMLContentSerializer::AppendText(nsIDOMText* aText,
       AppendToStringConvertLF(data, aStr);
     }
   }
-  else if (mFlags & nsIDocumentEncoder::OutputRaw) {
-    PRInt32 lastNewlineOffset = data.RFindChar('\n');
-    AppendToString(data, aStr);
-    if (lastNewlineOffset != kNotFound)
-      mColPos = data.Length() - lastNewlineOffset;
-  }
   else {
     AppendToStringWrapped(data, aStr, PR_FALSE);
   }
@@ -349,47 +342,39 @@ void nsHTMLContentSerializer::AppendWrapped_NonWhitespaceSequence(
         // we must wrap
 
         PRBool foundWrapPosition = PR_FALSE;
+        nsILineBreaker *lineBreaker = nsContentUtils::LineBreaker();
 
-        if (mLineBreaker) { // we have a line breaker helper object
-          PRUint32 wrapPosition;
-          PRBool needMoreText;
-          nsresult rv;
+        PRInt32 wrapPosition;
 
-          rv = mLineBreaker->Prev(aSequenceStart,
-                                  (aEnd - aSequenceStart),
-                                  (aPos - aSequenceStart) + 1,
-                                  &wrapPosition,
-                                  &needMoreText);
-          if (NS_SUCCEEDED(rv) && !needMoreText && wrapPosition > 0) {
+        wrapPosition = lineBreaker->Prev(aSequenceStart,
+                                         (aEnd - aSequenceStart),
+                                         (aPos - aSequenceStart) + 1);
+        if (wrapPosition != NS_LINEBREAKER_NEED_MORE_TEXT) {
+          foundWrapPosition = PR_TRUE;
+        }
+        else {
+          wrapPosition = lineBreaker->Next(aSequenceStart,
+                                           (aEnd - aSequenceStart),
+                                           (aPos - aSequenceStart));
+          if (wrapPosition != NS_LINEBREAKER_NEED_MORE_TEXT) {
             foundWrapPosition = PR_TRUE;
           }
-          else {
-            rv = mLineBreaker->Next(aSequenceStart,
-                                    (aEnd - aSequenceStart),
-                                    (aPos - aSequenceStart),
-                                    &wrapPosition,
-                                    &needMoreText);
-            if (NS_SUCCEEDED(rv) && !needMoreText && wrapPosition > 0) {
-              foundWrapPosition = PR_TRUE;
-            }
-          }
-
-          if (foundWrapPosition) {
-            if (mAddSpace) {
-              aOutputStr.Append(PRUnichar(' '));
-              mAddSpace = PR_FALSE;
-            }
-
-            aOutputStr.Append(aSequenceStart, wrapPosition);
-            aOutputStr.Append(mLineBreak);
-            aPos = aSequenceStart + wrapPosition;
-            mColPos = 0;
-            aMayIgnoreStartOfLineWhitespaceSequence = PR_TRUE;
-            mMayIgnoreLineBreakSequence = PR_TRUE;
-          }
         }
 
-        if (!mLineBreaker || !foundWrapPosition) {
+        if (foundWrapPosition) {
+          if (mAddSpace) {
+            aOutputStr.Append(PRUnichar(' '));
+            mAddSpace = PR_FALSE;
+          }
+
+          aOutputStr.Append(aSequenceStart, wrapPosition);
+          aOutputStr.Append(mLineBreak);
+          aPos = aSequenceStart + wrapPosition;
+          mColPos = 0;
+          aMayIgnoreStartOfLineWhitespaceSequence = PR_TRUE;
+          mMayIgnoreLineBreakSequence = PR_TRUE;
+        }
+        else {
           // try some simple fallback logic
           // go forward up to the next whitespace position,
           // in the worst case this will be all the rest of the data
@@ -476,6 +461,7 @@ nsHTMLContentSerializer::IsJavaScript(nsIAtom* aAttrNameAtom, const nsAString& a
               || (aAttrNameAtom == nsLayoutAtoms::onfocus)     || (aAttrNameAtom == nsLayoutAtoms::onkeydown)
               || (aAttrNameAtom == nsLayoutAtoms::onkeypress)  || (aAttrNameAtom == nsLayoutAtoms::onkeyup)
               || (aAttrNameAtom == nsLayoutAtoms::onload)      || (aAttrNameAtom == nsLayoutAtoms::onmousedown)
+              || (aAttrNameAtom == nsLayoutAtoms::onpageshow)  || (aAttrNameAtom == nsLayoutAtoms::onpagehide)
               || (aAttrNameAtom == nsLayoutAtoms::onmousemove) || (aAttrNameAtom == nsLayoutAtoms::onmouseout)
               || (aAttrNameAtom == nsLayoutAtoms::onmouseover) || (aAttrNameAtom == nsLayoutAtoms::onmouseup)
               || (aAttrNameAtom == nsLayoutAtoms::onreset)     || (aAttrNameAtom == nsLayoutAtoms::onselect)
@@ -529,7 +515,7 @@ nsHTMLContentSerializer::EscapeURI(const nsAString& aURI, nsAString& aEscapedURI
       NS_ENSURE_SUCCESS(rv, rv);
     }
     else {
-      escapedURI.Adopt(nsEscape(NS_ConvertUCS2toUTF8(part).get(), url_Path));
+      escapedURI.Adopt(nsEscape(NS_ConvertUTF16toUTF8(part).get(), url_Path));
     }
     AppendASCIItoUTF16(escapedURI, aEscapedURI);
 
@@ -547,7 +533,7 @@ nsHTMLContentSerializer::EscapeURI(const nsAString& aURI, nsAString& aEscapedURI
       NS_ENSURE_SUCCESS(rv, rv);
     }
     else {
-      escapedURI.Adopt(nsEscape(NS_ConvertUCS2toUTF8(part).get(), url_Path));
+      escapedURI.Adopt(nsEscape(NS_ConvertUTF16toUTF8(part).get(), url_Path));
     }
     AppendASCIItoUTF16(escapedURI, aEscapedURI);
   }
@@ -563,8 +549,6 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
   nsresult rv;
   PRUint32 index, count;
   nsAutoString nameStr, valueStr;
-  PRInt32 namespaceID;
-  nsCOMPtr<nsIAtom> attrName, attrPrefix;
 
   count = aContent->GetAttrCount();
 
@@ -575,10 +559,9 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
   // index is unsigned, hence index >= 0 is always true.
   for (index = count; index > 0; ) {
     --index;
-    aContent->GetAttrNameAt(index, 
-                            &namespaceID,
-                            getter_AddRefs(attrName),
-                            getter_AddRefs(attrPrefix));
+    const nsAttrName* name = aContent->GetAttrNameAt(index);
+    PRInt32 namespaceID = name->NamespaceID();
+    nsIAtom* attrName = name->LocalName();
 
     // Filter out any attribute starting with [-|_]moz
     const char* sharedName;
@@ -653,18 +636,19 @@ nsHTMLContentSerializer::SerializeAttributes(nsIContent* aContent,
 
 NS_IMETHODIMP
 nsHTMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
-                                            PRBool aHasChildren,
+                                            nsIDOMElement *aOriginalElement,
                                             nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
-  
+
   nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
   if (!content) return NS_ERROR_FAILURE;
-  
+
   // The _moz_dirty attribute is emitted by the editor to
   // indicate that this element should be pretty printed
   // even if we're not in pretty printing mode
-  PRBool hasDirtyAttr = HasDirtyAttr(content);
+  PRBool hasDirtyAttr = content->HasAttr(kNameSpaceID_None,
+                                         nsLayoutAtoms::mozdirty);
 
   nsIAtom *name = content->Tag();
 
@@ -736,7 +720,7 @@ nsHTMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   }
 
   if (mIsCopying && name == nsHTMLAtoms::li) {
-    mIsFirstChildOfOL = IsFirstChildOfOL(aElement);
+    mIsFirstChildOfOL = IsFirstChildOfOL(aOriginalElement);
     if (mIsFirstChildOfOL){
       // If OL is parent of this LI, serialize attributes in different manner.
       SerializeLIValueAttribute(aElement, aStr);
@@ -774,10 +758,23 @@ nsHTMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
   nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
   if (!content) return NS_ERROR_FAILURE;
 
-  PRBool hasDirtyAttr = HasDirtyAttr(content);
+  PRBool hasDirtyAttr = content->HasAttr(kNameSpaceID_None,
+                                         nsLayoutAtoms::mozdirty);
 
   nsIAtom *name = content->Tag();
 
+  if (name == nsHTMLAtoms::script) {
+    nsCOMPtr<nsIScriptElement> script = do_QueryInterface(aElement);
+    NS_ASSERTION(script, "What kind of weird script element is this?");
+
+    if (script->IsMalformed()) {
+      // We're looking at a malformed script tag. This means that the end tag
+      // was missing in the source. Imitate that here by not serializing the end
+      // tag.
+      return NS_OK;
+    }
+  }
+
   if (name == nsHTMLAtoms::pre ||
       name == nsHTMLAtoms::script ||
       name == nsHTMLAtoms::style) {
@@ -795,14 +792,13 @@ nsHTMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
     }
   }
   
-  nsIParserService* parserService = nsContentUtils::GetParserServiceWeakRef();
+  nsIParserService* parserService = nsContentUtils::GetParserService();
 
   if (parserService && (name != nsHTMLAtoms::style)) {
     PRBool isContainer;
-    PRInt32 id;
 
-    parserService->HTMLAtomTagToId(name, &id);
-    parserService->IsContainer(id, isContainer);
+    parserService->IsContainer(parserService->HTMLAtomTagToId(name),
+                               isContainer);
     if (!isContainer) return NS_OK;
   }
 
@@ -912,8 +908,7 @@ nsHTMLContentSerializer::AppendToString(const nsAString& aStr,
                   nsIDocumentEncoder::OutputEncodeLatin1Entities |
                   nsIDocumentEncoder::OutputEncodeHTMLEntities   |
                   nsIDocumentEncoder::OutputEncodeW3CEntities)) {
-      nsIParserService* parserService =
-        nsContentUtils::GetParserServiceWeakRef();
+      nsIParserService* parserService = nsContentUtils::GetParserService();
 
       if (!parserService) {
         NS_ERROR("Can't get parser service");
@@ -933,6 +928,8 @@ nsHTMLContentSerializer::AppendToString(const nsAString& aStr,
            iter != done_reading; 
            iter.advance(PRInt32(advanceLength))) {
         PRUint32 fragmentLength = iter.size_forward();
+        PRUint32 lengthReplaced = 0; // the number of UTF-16 codepoints
+                                     //  replaced by a particular entity
         const PRUnichar* c = iter.get();
         const PRUnichar* fragmentStart = c;
         const PRUnichar* fragmentEnd = c + fragmentLength;
@@ -965,10 +962,26 @@ nsHTMLContentSerializer::AppendToString(const nsAString& aStr,
           }
           else if (val > 127 && 
                    mFlags & nsIDocumentEncoder::OutputEncodeW3CEntities &&
-                   mEntityConverter &&
-                   NS_SUCCEEDED(mEntityConverter->ConvertToEntity(val,
-                                nsIEntityConverter::entityW3C, &fullEntityText))) {
-            break;
+                   mEntityConverter) {
+            if (IS_HIGH_SURROGATE(val) &&
+                c + 1 < fragmentEnd &&
+                IS_LOW_SURROGATE(*(c + 1))) {
+              PRUint32 valUTF32 = SURROGATE_TO_UCS4(val, *(++c));
+              if (NS_SUCCEEDED(mEntityConverter->ConvertUTF32ToEntity(valUTF32,
+                               nsIEntityConverter::entityW3C, &fullEntityText))) {
+                lengthReplaced = 2;
+                break;
+              }
+              else {
+                advanceLength++;
+              }
+            }
+            else if (NS_SUCCEEDED(mEntityConverter->ConvertToEntity(val,
+                                  nsIEntityConverter::entityW3C, 
+                                  &fullEntityText))) {
+              lengthReplaced = 1;
+              break;
+            }
           }
         }
 
@@ -983,7 +996,7 @@ nsHTMLContentSerializer::AppendToString(const nsAString& aStr,
         else if (fullEntityText) {
           AppendASCIItoUTF16(fullEntityText, aOutputStr);
           nsMemory::Free(fullEntityText);
-          advanceLength++;
+          advanceLength += lengthReplaced;
         }
       }
     } else {
@@ -1022,21 +1035,6 @@ nsHTMLContentSerializer::AppendToStringConvertLF(const nsAString& aStr,
 }
 
 PRBool
-nsHTMLContentSerializer::HasDirtyAttr(nsIContent* aContent)
-{
-  nsAutoString val;
-
-  if (NS_CONTENT_ATTR_NOT_THERE != aContent->GetAttr(kNameSpaceID_None,
-                                                     nsLayoutAtoms::mozdirty,
-                                                     val)) {
-    return PR_TRUE;
-  }
-  else {
-    return PR_FALSE;
-  }
-}
-
-PRBool 
 nsHTMLContentSerializer::LineBreakBeforeOpen(nsIAtom* aName, 
                                              PRBool aHasDirtyAttr)
 {
@@ -1056,15 +1054,11 @@ nsHTMLContentSerializer::LineBreakBeforeOpen(nsIAtom* aName,
     return PR_TRUE;
   }
   else {
-    nsIParserService* parserService =
-      nsContentUtils::GetParserServiceWeakRef();
+    nsIParserService* parserService = nsContentUtils::GetParserService();
     
     if (parserService) {
       PRBool res;
-      PRInt32 id;
-
-      parserService->HTMLAtomTagToId(aName, &id);
-      parserService->IsBlock(id, res);
+      parserService->IsBlock(parserService->HTMLAtomTagToId(aName), res);
       return res;
     }
   }
@@ -1157,15 +1151,11 @@ nsHTMLContentSerializer::LineBreakAfterClose(nsIAtom* aName,
     return PR_TRUE;
   }
   else {
-    nsIParserService* parserService =
-      nsContentUtils::GetParserServiceWeakRef();
+    nsIParserService* parserService = nsContentUtils::GetParserService();
     
     if (parserService) {
       PRBool res;
-      PRInt32 id;
-
-      parserService->HTMLAtomTagToId(aName, &id);
-      parserService->IsBlock(id, res);
+      parserService->IsBlock(parserService->HTMLAtomTagToId(aName), res);
       return res;
     }
   }
@@ -1326,13 +1316,15 @@ nsHTMLContentSerializer::SerializeLIValueAttribute(nsIDOMElement* aElement,
 PRBool
 nsHTMLContentSerializer::IsFirstChildOfOL(nsIDOMElement* aElement){
   nsCOMPtr<nsIDOMNode> node = do_QueryInterface(aElement);
-  nsIDOMNode* parentNode;
   nsAutoString parentName;
-  node->GetParentNode(&parentNode);
-  if (parentNode)
-    parentNode->GetNodeName(parentName);
-  else
-    return PR_FALSE;
+  {
+    nsCOMPtr<nsIDOMNode> parentNode;
+    node->GetParentNode(getter_AddRefs(parentNode));
+    if (parentNode)
+      parentNode->GetNodeName(parentName);
+    else
+      return PR_FALSE;
+  }
   
   if (parentName.LowerCaseEqualsLiteral("ol")) {
     olState defaultOLState(0, PR_FALSE);
