@@ -3,9 +3,9 @@
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Mozilla Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
@@ -15,13 +15,13 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Boris Zbarsky <bzbarsky@mit.edu>.
  * Portions created by the Initial Developer are Copyright (C) 2003
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *
+ *   Christian Biesinger <cbiesinger@web.de>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -29,37 +29,41 @@
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * A base class which implements nsIImageLoadingContent and can be
+ * subclassed by various content nodes that want to provide image
+ * loading functionality (eg <img>, <object>, etc).
+ */
+
 #include "nsImageLoadingContent.h"
+#include "nsAutoPtr.h"
 #include "nsContentErrors.h"
 #include "nsIContent.h"
 #include "nsIDocument.h"
-#include "nsINodeInfo.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMWindow.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 #include "nsContentPolicyUtils.h"
 #include "nsIURI.h"
 #include "nsILoadGroup.h"
 #include "imgIContainer.h"
 #include "gfxIImageFrame.h"
 #include "imgILoader.h"
-#include "plevent.h"
-#include "nsIEventQueueService.h"
-#include "nsIEventQueue.h"
+#include "nsThreadUtils.h"
 #include "nsNetUtil.h"
 
 #include "nsPresContext.h"
 #include "nsIPresShell.h"
+#include "nsIEventStateManager.h"
 #include "nsGUIEvent.h"
-#include "nsDummyLayoutRequest.h"
 
 #include "nsIChannel.h"
 #include "nsIStreamListener.h"
@@ -69,6 +73,10 @@
 #include "nsIDOMNode.h"
 
 #include "nsContentUtils.h"
+#include "nsIContentPolicy.h"
+#include "nsContentPolicyUtils.h"
+#include "nsEventDispatcher.h"
+#include "nsDOMClassInfo.h"
 
 #ifdef DEBUG_chb
 static void PrintReqURL(imgIRequest* req) {
@@ -90,51 +98,59 @@ static void PrintReqURL(imgIRequest* req) {
 }
 #endif /* DEBUG_chb */
 
-// Statics
-imgILoader* nsImageLoadingContent::sImgLoader = nsnull;
-nsIIOService* nsImageLoadingContent::sIOService = nsnull;
-
 
 nsImageLoadingContent::nsImageLoadingContent()
   : mObserverList(nsnull),
+    mImageBlockingStatus(nsIContentPolicy::ACCEPT),
+    mRootRefCount(0),
     mLoadingEnabled(PR_TRUE),
-    mImageIsBlocked(PR_FALSE),
-    mHaveHadObserver(PR_FALSE)
+    mStartingLoad(PR_FALSE),
+    mLoading(PR_FALSE),
+    // mBroken starts out true, since an image without a URI is broken....
+    mBroken(PR_TRUE),
+    mUserDisabled(PR_FALSE),
+    mSuppressed(PR_FALSE)
 {
-  if (!sImgLoader)
+  if (!nsContentUtils::GetImgLoader()) {
     mLoadingEnabled = PR_FALSE;
+  }
 }
 
-nsImageLoadingContent::~nsImageLoadingContent()
+void
+nsImageLoadingContent::DestroyImageLoadingContent()
 {
   // Cancel our requests so they won't hold stale refs to us
   if (mCurrentRequest) {
     mCurrentRequest->Cancel(NS_ERROR_FAILURE);
+    mCurrentRequest = nsnull;
   }
   if (mPendingRequest) {
     mPendingRequest->Cancel(NS_ERROR_FAILURE);
+    mPendingRequest = nsnull;
   }
+
+  // This can actually fire for multipart/x-mixed-replace, since if the
+  // load is canceled between parts (e.g., by cancelling the load
+  // group), we won't get any notification.  See bug 321054 comment 31
+  // and bug 339610.  *If* that multipart/x-mixed-replace image has
+  // event handlers, we won't even get to this warning; we'll leak
+  // instead.
+  NS_WARN_IF_FALSE(mRootRefCount == 0,
+                   "unbalanced handler preservation refcount");
+  if (mRootRefCount != 0) {
+    mRootRefCount = 1;
+    UnpreserveLoadHandlers();
+  }
+}
+
+nsImageLoadingContent::~nsImageLoadingContent()
+{
+  NS_ASSERTION(!mCurrentRequest && !mPendingRequest,
+               "DestroyImageLoadingContent not called");
   NS_ASSERTION(!mObserverList.mObserver && !mObserverList.mNext,
                "Observers still registered?");
 }
 
-void
-nsImageLoadingContent::Initialize()
-{
-  // If this fails, NS_NewURI will try to get the service itself
-  CallGetService("@mozilla.org/network/io-service;1", &sIOService);
-
-  // Ignore failure and just don't load images
-  CallGetService("@mozilla.org/image/loader;1", &sImgLoader);
-}
-
-void
-nsImageLoadingContent::Shutdown()
-{
-  NS_IF_RELEASE(sImgLoader);
-  NS_IF_RELEASE(sIOService);
-}
-
 // Macro to call some func on each observer.  This handles observers
 // removing themselves.
 #define LOOP_OVER_OBSERVERS(func_)                                       \
@@ -164,6 +180,13 @@ nsImageLoadingContent::FrameChanged(imgIContainer* aContainer,
 /*
  * imgIDecoderObserver impl
  */
+NS_IMETHODIMP
+nsImageLoadingContent::OnStartRequest(imgIRequest* aRequest)
+{
+  LOOP_OVER_OBSERVERS(OnStartRequest(aRequest));
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsImageLoadingContent::OnStartDecode(imgIRequest* aRequest)
 {
@@ -176,6 +199,10 @@ nsImageLoadingContent::OnStartContainer(imgIRequest* aRequest,
                                         imgIContainer* aContainer)
 {
   LOOP_OVER_OBSERVERS(OnStartContainer(aRequest, aContainer));
+
+  // Have to check for state changes here, since we might have been in
+  // the LOADING state before.
+  UpdateImageState(PR_TRUE);
   return NS_OK;    
 }
 
@@ -223,16 +250,38 @@ nsImageLoadingContent::OnStopDecode(imgIRequest* aRequest,
 
   if (aRequest == mPendingRequest) {
     mCurrentRequest->Cancel(NS_ERROR_IMAGE_SRC_CHANGED);
-    mCurrentRequest = mPendingRequest;
+    mPendingRequest.swap(mCurrentRequest);
     mPendingRequest = nsnull;
   }
 
+  // XXXldb What's the difference between when OnStopDecode and OnStopRequest
+  // fire?  Should we do this work there instead?  Should they just be the
+  // same?
+
   if (NS_SUCCEEDED(aStatus)) {
     FireEvent(NS_LITERAL_STRING("load"));
   } else {
     FireEvent(NS_LITERAL_STRING("error"));
   }
 
+  // Have to check for state changes here (for example, the new load could
+  // have resulted in a broken image).  Note that we don't want to do this
+  // async, unlike the event, because while this is waiting to happen our
+  // state could change yet again, and then we'll get confused about our
+  // state.
+  UpdateImageState(PR_TRUE);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageLoadingContent::OnStopRequest(imgIRequest* aRequest, PRBool aLastPart)
+{
+  LOOP_OVER_OBSERVERS(OnStopRequest(aRequest, aLastPart));
+
+  if (aLastPart)
+    UnpreserveLoadHandlers();
+
   return NS_OK;
 }
 
@@ -250,16 +299,17 @@ nsImageLoadingContent::GetLoadingEnabled(PRBool *aLoadingEnabled)
 NS_IMETHODIMP
 nsImageLoadingContent::SetLoadingEnabled(PRBool aLoadingEnabled)
 {
-  if (sImgLoader)
+  if (nsContentUtils::GetImgLoader()) {
     mLoadingEnabled = aLoadingEnabled;
+  }
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsImageLoadingContent::GetImageBlocked(PRBool* aBlocked)
+nsImageLoadingContent::GetImageBlockingStatus(PRInt16* aStatus)
 {
-  NS_PRECONDITION(aBlocked, "Null out param");
-  *aBlocked = mImageIsBlocked;
+  NS_PRECONDITION(aStatus, "Null out param");
+  *aStatus = mImageBlockingStatus;
   return NS_OK;
 }
 
@@ -268,8 +318,6 @@ nsImageLoadingContent::AddObserver(imgIDecoderObserver* aObserver)
 {
   NS_ENSURE_ARG_POINTER(aObserver);
 
-  mHaveHadObserver = PR_TRUE;
-  
   if (!mObserverList.mObserver) {
     mObserverList.mObserver = aObserver;
     // Don't touch the linking of the list!
@@ -371,11 +419,16 @@ nsImageLoadingContent::GetRequestType(imgIRequest* aRequest,
 NS_IMETHODIMP
 nsImageLoadingContent::GetCurrentURI(nsIURI** aURI)
 {
-  if (mCurrentRequest)
+  if (mCurrentRequest) {
     return mCurrentRequest->GetURI(aURI);
+  }
 
-  NS_IF_ADDREF(*aURI = mCurrentURI);
-  return NS_OK;
+  if (!mCurrentURI) {
+    *aURI = nsnull;
+    return NS_OK;
+  }
+  
+  return NS_EnsureSafeToReturn(mCurrentURI, aURI);
 }
 
 NS_IMETHODIMP
@@ -386,59 +439,123 @@ nsImageLoadingContent::LoadImageWithChannel(nsIChannel* aChannel,
   
   NS_ENSURE_ARG_POINTER(aChannel);
 
-  if (!sImgLoader)
+  if (!nsContentUtils::GetImgLoader()) {
     return NS_ERROR_NULL_POINTER;
+  }
 
   // XXX what should we do with content policies here, if anything?
   // Shouldn't that be done before the start of the load?
+  // XXX what about shouldProcess?
   
   nsCOMPtr<nsIDocument> doc = GetOurDocument();
   if (!doc) {
     // Don't bother
     return NS_OK;
   }
+
+  PreserveLoadHandlers();
+
+  // Null out our mCurrentURI, in case we have no image requests right now.
+  mCurrentURI = nsnull;
   
-  CancelImageRequests(NS_ERROR_IMAGE_SRC_CHANGED, PR_FALSE);
+  CancelImageRequests(NS_ERROR_IMAGE_SRC_CHANGED, PR_FALSE,
+                      nsIContentPolicy::ACCEPT);
 
   nsCOMPtr<imgIRequest> & req = mCurrentRequest ? mPendingRequest : mCurrentRequest;
 
-  return sImgLoader->LoadImageWithChannel(aChannel, this, doc, aListener, getter_AddRefs(req));
+  nsresult rv = nsContentUtils::GetImgLoader()->
+    LoadImageWithChannel(aChannel, this, doc, aListener, getter_AddRefs(req));
+
+  // Make sure our state is up to date
+  UpdateImageState(PR_TRUE);
+
+  if (NS_FAILED(rv))
+    UnpreserveLoadHandlers();
+
+  return rv;
 }
 
-// XXX This should be a protected method, not an interface method!!!
-NS_IMETHODIMP
-nsImageLoadingContent::ImageURIChanged(const nsAString& aNewURI) {
-  return ImageURIChanged(NS_ConvertUCS2toUTF8(aNewURI));
+NS_IMETHODIMP nsImageLoadingContent::ForceReload()
+{
+  nsCOMPtr<nsIURI> currentURI;
+  GetCurrentURI(getter_AddRefs(currentURI));
+  if (!currentURI) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  return LoadImage(currentURI, PR_TRUE, PR_TRUE, nsnull, nsIRequest::VALIDATE_ALWAYS);
 }
 
 /*
  * Non-interface methods
  */
-nsresult
-nsImageLoadingContent::ImageURIChanged(const nsACString& aNewURI)
-{
-  if (!mLoadingEnabled) {
-    return NS_OK;
-  }
 
+nsresult
+nsImageLoadingContent::LoadImage(const nsAString& aNewURI,
+                                 PRBool aForce,
+                                 PRBool aNotify)
+{
   // First, get a document (needed for security checks and the like)
-  nsCOMPtr<nsIDocument> doc = GetOurDocument();
+  nsIDocument* doc = GetOurDocument();
   if (!doc) {
     // No reason to bother, I think...
     return NS_OK;
   }
 
+  nsCOMPtr<nsIURI> imageURI;
+  nsresult rv = StringToURI(aNewURI, doc, getter_AddRefs(imageURI));
+  NS_ENSURE_SUCCESS(rv, rv);
+  // XXXbiesi fire onerror if that failed?
+
+  NS_TryToSetImmutable(imageURI);
+
+  return LoadImage(imageURI, aForce, aNotify, doc);
+}
+
+nsresult
+nsImageLoadingContent::LoadImage(nsIURI* aNewURI,
+                                 PRBool aForce,
+                                 PRBool aNotify,
+                                 nsIDocument* aDocument,
+                                 nsLoadFlags aLoadFlags)
+{
+  if (!mLoadingEnabled) {
+    FireEvent(NS_LITERAL_STRING("error"));
+    return NS_OK;
+  }
+
+  NS_ASSERTION(!aDocument || aDocument == GetOurDocument(),
+               "Bogus document passed in");
+  // First, get a document (needed for security checks and the like)
+  if (!aDocument) {
+    aDocument = GetOurDocument();
+    if (!aDocument) {
+      // No reason to bother, I think...
+      return NS_OK;
+    }
+  }
+
+
   nsresult rv;   // XXXbz Should failures in this method fire onerror?
 
-  nsCOMPtr<nsIURI> imageURI;
-  rv = StringToURI(aNewURI, doc, getter_AddRefs(imageURI));
-  NS_ENSURE_SUCCESS(rv, rv);
+  // Skip the URI equality check if our current image was blocked.  If
+  // that happened, we really do want to try loading again.
+  if (!aForce && NS_CP_ACCEPTED(mImageBlockingStatus)) {
+    nsCOMPtr<nsIURI> currentURI;
+    GetCurrentURI(getter_AddRefs(currentURI));
+    PRBool equal;
+    if (currentURI &&
+        NS_SUCCEEDED(currentURI->Equals(aNewURI, &equal)) &&
+        equal) {
+      // Nothing to do here.
+      return NS_OK;
+    }
+  }
+
+  // From this point on, our state could change before return, so make
+  // sure to notify if it does.
+  AutoStateChanger changer(this, aNotify);
 
-  // Remember the URL of this request, in case someone asks us for it later
-  // But this only matters if we are affecting the current request
-  if (!mCurrentRequest)
-    mCurrentURI = imageURI;
-  
   // If we'll be loading a new image, we want to cancel our existing
   // requests; the question is what reason to pass in.  If everything
   // is going smoothly, that reason should be
@@ -447,88 +564,132 @@ nsImageLoadingContent::ImageURIChanged(const nsACString& aNewURI)
   // content policy or security manager, we will want to cancel with
   // the error code from those.
 
-  // TODOtw: figure out whether we should show alternate text
-  //     (CanLoadImage can tell us this, since Content Policy tells it)
-  PRBool loadImage = nsContentUtils::CanLoadImage(imageURI, this, doc);
+  PRInt16 newImageStatus;
+  PRBool loadImage = nsContentUtils::CanLoadImage(aNewURI, this, aDocument,
+                                                  &newImageStatus);
+  NS_ASSERTION(loadImage || !NS_CP_ACCEPTED(newImageStatus),
+               "CanLoadImage lied");
 
   nsresult cancelResult = loadImage ? NS_ERROR_IMAGE_SRC_CHANGED
                                     : NS_ERROR_IMAGE_BLOCKED;
 
-  mImageIsBlocked = !loadImage;
+  CancelImageRequests(cancelResult, PR_FALSE, newImageStatus);
 
-  CancelImageRequests(cancelResult, PR_FALSE);
-
-  if (mImageIsBlocked) {
+  // Remember the URL of this request, in case someone asks us for it later.
+  // But this only matters if we are affecting the current request.  Need to do
+  // this after CancelImageRequests, since that affects the value of
+  // mCurrentRequest.
+  if (!mCurrentRequest) {
+    mCurrentURI = aNewURI;
+  }
+  
+  if (!loadImage) {
     // Don't actually load anything!  This was blocked by CanLoadImage.
+    FireEvent(NS_LITERAL_STRING("error"));
     return NS_OK;
   }
 
+  PreserveLoadHandlers();
+
   nsCOMPtr<imgIRequest> & req = mCurrentRequest ? mPendingRequest : mCurrentRequest;
 
-  // It may be that one of our frames has replaced itself with alt text... This
-  // would only have happened if our mCurrentRequest had issues, and we would
-  // have set it to null by now in that case.  Have to save that information
-  // here, since LoadImage may clobber the value of mCurrentRequest.  On the
-  // other hand, if we've never had an observer, we know there aren't any frames
-  // that have changed to alt text on us yet.
-  PRBool mayNeedReframe = mHaveHadObserver && !mCurrentRequest;
-  
-  rv = nsContentUtils::LoadImage(imageURI, doc, doc->GetDocumentURI(),
-                                 this, nsIRequest::LOAD_NORMAL,
+  rv = nsContentUtils::LoadImage(aNewURI, aDocument,
+                                 aDocument->GetDocumentURI(),
+                                 this, aLoadFlags,
                                  getter_AddRefs(req));
+  if (NS_FAILED(rv)) {
+    FireEvent(NS_LITERAL_STRING("error"));
+    UnpreserveLoadHandlers();
+    return NS_OK;
+  }
+
   // If we now have a current request, we don't need to store the URI, since
   // we can get it off the request. Release it.
   if (mCurrentRequest) {
     mCurrentURI = nsnull;
   }
 
-  if (!mayNeedReframe) {
-    // We're all set
-    return NS_OK;
+  return NS_OK;
+}
+
+PRInt32
+nsImageLoadingContent::ImageState() const
+{
+  return
+    (mBroken * NS_EVENT_STATE_BROKEN) |
+    (mUserDisabled * NS_EVENT_STATE_USERDISABLED) |
+    (mSuppressed * NS_EVENT_STATE_SUPPRESSED) |
+    (mLoading * NS_EVENT_STATE_LOADING);
+}
+
+void
+nsImageLoadingContent::UpdateImageState(PRBool aNotify)
+{
+  if (mStartingLoad) {
+    // Ignore this call; we'll update our state when the state changer is
+    // destroyed.  Need this to work around the fact that some libpr0n stuff is
+    // actually sync and hence we can get OnStopDecode called while we're still
+    // under LoadImage, and OnStopDecode doesn't know anything about
+    // aNotify
+    return;
+  }
+  
+  nsCOMPtr<nsIContent> thisContent = do_QueryInterface(this);
+  if (!thisContent) {
+    return;
   }
 
-  // Only continue if we have a parent and a document -- that would mean we're
-  // a useful chunk of the content model and _may_ have a frame.  This should
-  // eliminate things like SetAttr calls during the parsing process, as well as
-  // things like setting src on |new Image()|-type things.
-  nsCOMPtr<nsIContent> thisContent = do_QueryInterface(this, &rv);
-  NS_ENSURE_TRUE(thisContent, rv);
+  PRInt32 oldState = ImageState();
 
-  if (!thisContent->GetDocument() || !thisContent->GetParent()) {
-    return NS_OK;
-  }
-
-  // OK, now for each PresShell, see whether we have a frame -- this tends to
-  // be expensive, which is why it's the last check....  If we have a frame
-  // and it's not of the right type, reframe it.
-  PRInt32 numShells = doc->GetNumberOfShells();
-  for (PRInt32 i = 0; i < numShells; ++i) {
-    nsIPresShell *shell = doc->GetShellAt(i);
-    if (shell) {
-      nsIFrame* frame = nsnull;
-      shell->GetPrimaryFrameFor(thisContent, &frame);
-      if (frame) {
-        // XXXbz I don't like this one bit... we really need a better way of
-        // doing the CantRenderReplacedElement stuff.. In particular, it needs
-        // to be easily detectable.  For example, I suspect that this code will
-        // fail for <object> in the current CantRenderReplacedElement
-        // implementation...
-        nsIAtom* frameType = frame->GetType();
-        if (frameType != nsLayoutAtoms::imageFrame &&
-            frameType != nsLayoutAtoms::imageControlFrame &&
-            frameType != nsLayoutAtoms::objectFrame) {
-          shell->RecreateFramesFor(thisContent);
-        }
-      }
+  mLoading = mBroken = mUserDisabled = mSuppressed = PR_FALSE;
+  
+  // If we were blocked by server-based content policy, we claim to be
+  // suppressed.  If we were blocked by type-based content policy, we claim to
+  // be user-disabled.  Otherwise, claim to be broken.
+  if (mImageBlockingStatus == nsIContentPolicy::REJECT_SERVER) {
+    mSuppressed = PR_TRUE;
+  } else if (mImageBlockingStatus == nsIContentPolicy::REJECT_TYPE) {
+    mUserDisabled = PR_TRUE;
+  } else if (!mCurrentRequest) {
+    // No current request means error, since we weren't disabled or suppressed
+    mBroken = PR_TRUE;
+  } else {
+    PRUint32 currentLoadStatus;
+    nsresult rv = mCurrentRequest->GetImageStatus(&currentLoadStatus);
+    if (NS_FAILED(rv) || (currentLoadStatus & imgIRequest::STATUS_ERROR)) {
+      mBroken = PR_TRUE;
+    } else if (!(currentLoadStatus & imgIRequest::STATUS_SIZE_AVAILABLE)) {
+      mLoading = PR_TRUE;
     }
   }
 
-  return NS_OK;
+  if (aNotify) {
+    nsIDocument* doc = thisContent->GetCurrentDoc();
+    if (doc) {
+      NS_ASSERTION(thisContent->IsInDoc(), "Something is confused");
+      PRInt32 changedBits = oldState ^ ImageState();
+      if (changedBits) {
+        mozAutoDocUpdate upd(doc, UPDATE_CONTENT_STATE, PR_TRUE);
+        doc->ContentStatesChanged(thisContent, nsnull, changedBits);
+      }
+    }
+  }
+}
+
+void
+nsImageLoadingContent::CancelImageRequests(PRBool aNotify)
+{
+  // Make sure to null out mCurrentURI here, so we no longer look like an image
+  mCurrentURI = nsnull;
+  CancelImageRequests(NS_BINDING_ABORTED, PR_TRUE, nsIContentPolicy::ACCEPT);
+  NS_ASSERTION(!mStartingLoad, "Whence a state changer here?");
+  UpdateImageState(aNotify);
 }
 
 void
 nsImageLoadingContent::CancelImageRequests(nsresult aReason,
-                                           PRBool   aEvenIfSizeAvailable)
+                                           PRBool   aEvenIfSizeAvailable,
+                                           PRInt16  aNewImageStatus)
 {
   // Cancel the pending request, if any
   if (mPendingRequest) {
@@ -541,13 +702,50 @@ nsImageLoadingContent::CancelImageRequests(nsresult aReason,
   if (mCurrentRequest) {
     PRUint32 loadStatus = imgIRequest::STATUS_ERROR;
     mCurrentRequest->GetImageStatus(&loadStatus);
+
+    NS_ASSERTION(NS_CP_ACCEPTED(mImageBlockingStatus),
+                 "Have current request but blocked image?");
     
     if (aEvenIfSizeAvailable ||
         !(loadStatus & imgIRequest::STATUS_SIZE_AVAILABLE)) {
+      // The new image is going to become the current request.  Make sure to
+      // set mImageBlockingStatus _before_ we cancel the request... if we set
+      // it after, things that are watching the mCurrentRequest will get wrong
+      // data.
+      mImageBlockingStatus = aNewImageStatus;
       mCurrentRequest->Cancel(aReason);
       mCurrentRequest = nsnull;
     }
+  } else {
+    // No current request so the new image status will become the
+    // status of the current request
+    mImageBlockingStatus = aNewImageStatus;
   }
+
+  // Note that the only way we could have avoided setting the image blocking
+  // status above is if we have a current request and have kept it as the
+  // current request.  In that case, we want to leave our old status, since the
+  // status corresponds to the current request.  Even if we plan to do a
+  // pending request load, having an mCurrentRequest means that our current
+  // status is not a REJECT_* status, and doing the load shouldn't change that.
+  // XXXbz there is an issue here if different ACCEPT statuses are used, but...
+}
+
+nsresult
+nsImageLoadingContent::UseAsPrimaryRequest(imgIRequest* aRequest,
+                                           PRBool aNotify)
+{
+  // Use an AutoStateChanger so that the clone call won't
+  // automatically notify from inside OnStopDecode.
+  NS_PRECONDITION(aRequest, "Must have a request here!");
+  AutoStateChanger changer(this, aNotify);
+  mCurrentURI = nsnull;
+  PreserveLoadHandlers();
+  CancelImageRequests(NS_BINDING_ABORTED, PR_TRUE, nsIContentPolicy::ACCEPT);
+
+  NS_ASSERTION(!mCurrentRequest, "We should not have a current request now");
+
+  return aRequest->Clone(this, getter_AddRefs(mCurrentRequest));
 }
 
 nsIDocument*
@@ -560,7 +758,7 @@ nsImageLoadingContent::GetOurDocument()
 }
 
 nsresult
-nsImageLoadingContent::StringToURI(const nsACString& aSpec,
+nsImageLoadingContent::StringToURI(const nsAString& aSpec,
                                    nsIDocument* aDocument,
                                    nsIURI** aURI)
 {
@@ -573,79 +771,67 @@ nsImageLoadingContent::StringToURI(const nsACString& aSpec,
   nsCOMPtr<nsIURI> baseURL = thisContent->GetBaseURI();
 
   // (2) Get the charset
-  const nsACString &charset = aDocument->GetDocumentCharacterSet();
+  const nsAFlatCString &charset = aDocument->GetDocumentCharacterSet();
 
   // (3) Construct the silly thing
   return NS_NewURI(aURI,
                    aSpec,
-                   charset.IsEmpty() ? nsnull : PromiseFlatCString(charset).get(),
+                   charset.IsEmpty() ? nsnull : charset.get(),
                    baseURL,
-                   sIOService);
+                   nsContentUtils::GetIOService());
 }
 
 
 /**
- * Struct used to dispatch events
+ * Class used to dispatch events
  */
-MOZ_DECL_CTOR_COUNTER(ImageEvent)
 
-class ImageEvent : public PLEvent,
-                   public nsDummyLayoutRequest
+class nsImageLoadingContent::Event : public nsRunnable
 {
 public:
-  ImageEvent(nsPresContext* aPresContext, nsIContent* aContent,
-             const nsAString& aMessage, nsILoadGroup *aLoadGroup)
-    : nsDummyLayoutRequest(nsnull),
-      mPresContext(aPresContext),
+  Event(nsPresContext* aPresContext, nsImageLoadingContent* aContent,
+             const nsAString& aMessage, nsIDocument* aDocument)
+    : mPresContext(aPresContext),
       mContent(aContent),
       mMessage(aMessage),
-      mLoadGroup(aLoadGroup)
+      mDocument(aDocument)
   {
-    MOZ_COUNT_CTOR(ImageEvent);
   }
-  ~ImageEvent()
+  ~Event()
   {
-    MOZ_COUNT_DTOR(ImageEvent);
+    mDocument->UnblockOnload(PR_TRUE);
+    mContent->UnpreserveLoadHandlers();
   }
+
+  NS_IMETHOD Run();
   
   nsCOMPtr<nsPresContext> mPresContext;
-  nsCOMPtr<nsIContent> mContent;
+  nsRefPtr<nsImageLoadingContent> mContent;
   nsString mMessage;
-  nsCOMPtr<nsILoadGroup> mLoadGroup;
+  // Need to hold on to the document in case our event outlives document
+  // teardown... Wantto be able to get back to the document even if the
+  // prescontext and content can't.
+  nsCOMPtr<nsIDocument> mDocument;
 };
 
-PR_STATIC_CALLBACK(void*)
-HandleImagePLEvent(PLEvent* aEvent)
+NS_IMETHODIMP
+nsImageLoadingContent::Event::Run()
 {
-  ImageEvent* evt = NS_STATIC_CAST(ImageEvent*, aEvent);
-  nsEventStatus estatus = nsEventStatus_eIgnore;
   PRUint32 eventMsg;
 
-  if (evt->mMessage.EqualsLiteral("load")) {
-    eventMsg = NS_IMAGE_LOAD;
+  if (mMessage.EqualsLiteral("load")) {
+    eventMsg = NS_LOAD;
   } else {
-    eventMsg = NS_IMAGE_ERROR;
+    eventMsg = NS_LOAD_ERROR;
   }
 
-  nsEvent event(eventMsg);
-  evt->mContent->HandleDOMEvent(evt->mPresContext, &event, nsnull,
-                                NS_EVENT_FLAG_INIT, &estatus);
+  nsCOMPtr<nsIContent> ourContent = do_QueryInterface(mContent);
 
-  evt->mLoadGroup->RemoveRequest(evt, nsnull, NS_OK);
+  nsEvent event(PR_TRUE, eventMsg);
+  event.flags |= NS_EVENT_FLAG_CANT_BUBBLE;
+  nsEventDispatcher::Dispatch(ourContent, mPresContext, &event);
 
-  return nsnull;
-}
-
-PR_STATIC_CALLBACK(void)
-DestroyImagePLEvent(PLEvent* aEvent)
-{
-  ImageEvent* evt = NS_STATIC_CAST(ImageEvent*, aEvent);
-
-  // We're reference counted, and we hold a strong reference to
-  // ourselves while we're a 'live' PLEvent. Now that the PLEvent is
-  // destroyed, release ourselves.
-
-  NS_RELEASE(evt);
+  return NS_OK;
 }
 
 nsresult
@@ -660,49 +846,53 @@ nsImageLoadingContent::FireEvent(const nsAString& aEventType)
     // no use to fire events if there is no document....
     return NS_OK;
   }                                                                             
-  nsresult rv;
-  nsCOMPtr<nsIEventQueueService> eventQService =
-    do_GetService("@mozilla.org/event-queue-service;1", &rv);
-  NS_ENSURE_TRUE(eventQService, rv);
 
-  nsCOMPtr<nsIEventQueue> eventQ;
-  // Use the UI thread event queue (though we should not be getting called from
-  // off the UI thread in any case....)
-  rv = eventQService->GetSpecialEventQueue(nsIEventQueueService::UI_THREAD_EVENT_QUEUE,
-                                           getter_AddRefs(eventQ));
-  NS_ENSURE_TRUE(eventQ, rv);
-
-  nsCOMPtr<nsILoadGroup> loadGroup = document->GetDocumentLoadGroup();
+  // We should not be getting called from off the UI thread...
+  NS_ASSERTION(NS_IsMainThread(), "should be on the main thread");
 
   nsIPresShell *shell = document->GetShellAt(0);
-  NS_ENSURE_TRUE(shell, NS_ERROR_FAILURE);
-
-  nsPresContext *presContext = shell->GetPresContext();
-  NS_ENSURE_TRUE(presContext, NS_ERROR_FAILURE);
-
-  nsCOMPtr<nsIContent> ourContent = do_QueryInterface(this);
-  
-  ImageEvent* evt = new ImageEvent(presContext, ourContent, aEventType,
-                                   loadGroup);
+  nsPresContext *presContext = shell ? shell->GetPresContext() : nsnull;
 
+  nsCOMPtr<nsIRunnable> evt =
+      new nsImageLoadingContent::Event(presContext, this, aEventType, document);
   NS_ENSURE_TRUE(evt, NS_ERROR_OUT_OF_MEMORY);
 
-  PL_InitEvent(evt, this, ::HandleImagePLEvent, ::DestroyImagePLEvent);
+  // Block onload for our event.  Since we unblock in the event destructor, we
+  // want to block now, even if posting will fail.
+  document->BlockOnload();
+  PreserveLoadHandlers();
+  
+  return NS_DispatchToCurrentThread(evt);
+}
 
-  // The event will own itself while it's in the event queue, once
-  // removed, it will release itself, and if there are no other
-  // references, it will be deleted.
-  NS_ADDREF(evt);
-
-  rv = eventQ->PostEvent(evt);
+void
+nsImageLoadingContent::PreserveLoadHandlers()
+{
+  ++mRootRefCount;
+  NS_LOG_ADDREF(&mRootRefCount, mRootRefCount,
+                "nsImageLoadingContent::mRootRefCount", sizeof(mRootRefCount));
+  if (mRootRefCount == 1) {
+    nsCOMPtr<nsIDOMGCParticipant> part = do_QueryInterface(this);
+    nsresult rv = nsDOMClassInfo::SetExternallyReferenced(part);
+    // The worst that will happen if we ignore this failure is that
+    // onload or onerror will fail to fire.  I suppose we could fire
+    // onerror now as a result of that, but the only reason it would
+    // actually fail is out-of-memory, and it seems silly to bother and
+    // unlikely to work in that case.
+    NS_ASSERTION(NS_SUCCEEDED(rv), "ignoring failure to root participant");
+  }
+}
 
-  if (NS_SUCCEEDED(rv)) {
-    // Add the dummy request (the ImageEvent) to the load group only
-    // after all the early returns here!
-    loadGroup->AddRequest(evt, nsnull);
-  } else {
-    PL_DestroyEvent(evt);
+void
+nsImageLoadingContent::UnpreserveLoadHandlers()
+{
+  NS_ASSERTION(mRootRefCount != 0,
+               "load handler preservation refcount underflow");
+  --mRootRefCount;
+  NS_LOG_RELEASE(&mRootRefCount, mRootRefCount,
+                 "nsImageLoadingContent::mRootRefCount");
+  if (mRootRefCount == 0) {
+    nsCOMPtr<nsIDOMGCParticipant> part = do_QueryInterface(this);
+    nsDOMClassInfo::UnsetExternallyReferenced(part);
   }
-
-  return rv;
 }
