@@ -37,8 +37,13 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * nsIContentSerializer implementation that can be used with an
+ * nsIDocumentEncoder to convert a DOM into plaintext in a nice way
+ * (eg for copy/paste as plaintext).
+ */
+
 #include "nsPlainTextSerializer.h"
-#include "nsILineBreakerFactory.h"
 #include "nsLWBrkCIID.h"
 #include "nsIServiceManager.h"
 #include "nsHTMLAtoms.h"
@@ -46,7 +51,6 @@
 #include "nsIDOMCDATASection.h"
 #include "nsIDOMElement.h"
 #include "nsINameSpaceManager.h"
-#include "nsITextContent.h"
 #include "nsTextFragment.h"
 #include "nsContentUtils.h"
 #include "nsReadableUtils.h"
@@ -54,8 +58,6 @@
 #include "nsCRT.h"
 #include "nsIParserService.h"
 
-static NS_DEFINE_CID(kLWBrkCID, NS_LWBRK_CID);
-
 #define PREF_STRUCTS "converter.html2txt.structs"
 #define PREF_HEADER_STRATEGY "converter.html2txt.header_strategy"
 
@@ -104,6 +106,7 @@ nsPlainTextSerializer::nsPlainTextSerializer()
   mHeaderStrategy = 1 /*indent increasingly*/;   // ditto
   mQuotesPreformatted = PR_FALSE;                // ditto
   mDontWrapAnyQuotes = PR_FALSE;                 // ditto
+  mHasWrittenCiteBlockquote = PR_FALSE;
   mSpanLevel = 0;
   for (PRInt32 i = 0; i <= 6; i++) {
     mHeaderCounter[i] = 0;
@@ -162,22 +165,14 @@ nsPlainTextSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
   }
 #endif
 
-  NS_ENSURE_TRUE(nsContentUtils::GetParserServiceWeakRef(),
-                 NS_ERROR_UNEXPECTED);
+  NS_ENSURE_TRUE(nsContentUtils::GetParserService(), NS_ERROR_UNEXPECTED);
 
-  nsresult rv;
-  
   mFlags = aFlags;
   mWrapColumn = aWrapColumn;
 
   // Only create a linebreaker if we will handle wrapping.
   if (MayWrap()) {
-    nsCOMPtr<nsILineBreakerFactory> lf(do_GetService(kLWBrkCID, &rv));
-    if (NS_SUCCEEDED(rv)) {
-      nsAutoString lbarg;
-      rv = lf->GetBreaker(lbarg, getter_AddRefs(mLineBreaker));
-      if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-    }
+    mLineBreaker = nsContentUtils::LineBreaker();
   }
 
   // Set the line break character:
@@ -310,26 +305,25 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
   PRInt32 length = 0;
   nsAutoString textstr;
 
-  nsCOMPtr<nsITextContent> content = do_QueryInterface(aText);
-  if (!content) return NS_ERROR_FAILURE;
+  nsCOMPtr<nsIContent> content = do_QueryInterface(aText);
+  const nsTextFragment* frag;
+  if (!content || !(frag = content->GetText())) {
+    return NS_ERROR_FAILURE;
+  }
   
-  const nsTextFragment* frag = content->Text();
+  PRInt32 endoffset = (aEndOffset == -1) ? frag->GetLength() : aEndOffset;
+  NS_ASSERTION(aStartOffset <= endoffset, "A start offset is beyond the end of the text fragment!");
 
-  if (frag) {
-    PRInt32 endoffset = (aEndOffset == -1) ? frag->GetLength() : aEndOffset;
-    NS_ASSERTION(aStartOffset <= endoffset, "A start offset is beyond the end of the text fragment!");
+  length = endoffset - aStartOffset;
+  if (length <= 0) {
+    return NS_OK;
+  }
 
-    length = endoffset - aStartOffset;
-    if (length <= 0) {
-      return NS_OK;
-    }
-
-    if (frag->Is2b()) {
-      textstr.Assign(frag->Get2b() + aStartOffset, length);
-    }
-    else {
-      textstr.AssignWithConversion(frag->Get1b()+aStartOffset, length);
-    }
+  if (frag->Is2b()) {
+    textstr.Assign(frag->Get2b() + aStartOffset, length);
+  }
+  else {
+    textstr.AssignWithConversion(frag->Get1b()+aStartOffset, length);
   }
 
   mOutputString = &aStr;
@@ -384,7 +378,7 @@ nsPlainTextSerializer::AppendCDATASection(nsIDOMCDATASection* aCDATASection,
 
 NS_IMETHODIMP
 nsPlainTextSerializer::AppendElementStart(nsIDOMElement *aElement,
-                                          PRBool aHasChildren,
+                                          nsIDOMElement *aOriginalElement,
                                           nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
@@ -403,8 +397,7 @@ nsPlainTextSerializer::AppendElementStart(nsIDOMElement *aElement,
     rv = DoOpenContainer(nsnull, id);
   }
   else {
-    nsAutoString empty;
-    rv = DoAddLeaf(nsnull, id, empty);
+    rv = DoAddLeaf(nsnull, id, EmptyString());
   }
 
   mContent = 0;
@@ -467,22 +460,23 @@ nsPlainTextSerializer::OpenContainer(const nsIParserNode& aNode)
 {
   PRInt32 type = aNode.GetNodeType();
 
+  if (type == eHTMLTag_head) {
+    mInHead = PR_TRUE;
+    return NS_OK;
+  }
+
   return DoOpenContainer(&aNode, type);
 }
 
 NS_IMETHODIMP 
 nsPlainTextSerializer::CloseContainer(const nsHTMLTag aTag)
 {
-  return DoCloseContainer(aTag);
-}
+  if (aTag == eHTMLTag_head) {
+    mInHead = PR_FALSE;
+    return NS_OK;
+  }
 
-NS_IMETHODIMP 
-nsPlainTextSerializer::AddHeadContent(const nsIParserNode& aNode)
-{
-  OpenHead(aNode);
-  nsresult rv = AddLeaf(aNode);
-  CloseHead();
-  return rv;
+  return DoCloseContainer(aTag);
 }
 
 NS_IMETHODIMP 
@@ -512,80 +506,13 @@ nsPlainTextSerializer::AddLeaf(const nsIParserNode& aNode)
   }
 }
 
-NS_IMETHODIMP
-nsPlainTextSerializer::OpenHTML(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
 NS_IMETHODIMP 
-nsPlainTextSerializer::CloseHTML()
-{
-  return CloseContainer(eHTMLTag_html);
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::OpenHead(const nsIParserNode& aNode)
+nsPlainTextSerializer::OpenHead()
 {
   mInHead = PR_TRUE;
   return NS_OK;
 }
 
-NS_IMETHODIMP 
-nsPlainTextSerializer::CloseHead()
-{
-  mInHead = PR_FALSE;
-  return NS_OK;
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::OpenBody(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::CloseBody()
-{
-  return CloseContainer(eHTMLTag_body);
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::OpenForm(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::CloseForm()
-{
-  return CloseContainer(eHTMLTag_form);
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::OpenMap(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::CloseMap()
-{
-  return CloseContainer(eHTMLTag_map);
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::OpenFrameset(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
-NS_IMETHODIMP 
-nsPlainTextSerializer::CloseFrameset()
-{
-  return CloseContainer(eHTMLTag_frameset);
-}
-
 NS_IMETHODIMP
 nsPlainTextSerializer::IsEnabled(PRInt32 aTag, PRBool* aReturn)
 {
@@ -630,7 +557,21 @@ nsPlainTextSerializer::DoOpenContainer(const nsIParserNode* aNode, PRInt32 aTag)
     return NS_OK;
   }
 
-  if (mLineBreakDue)
+  // Reset this so that <blockquote type=cite> doesn't affect the whitespace
+  // above random <pre>s below it.
+  mHasWrittenCiteBlockquote = mHasWrittenCiteBlockquote && aTag == eHTMLTag_pre;
+
+  PRBool isInCiteBlockquote = PR_FALSE;
+
+  // XXX special-case <blockquote type=cite> so that we don't add additional
+  // newlines before the text.
+  if (aTag == eHTMLTag_blockquote) {
+    nsAutoString value;
+    nsresult rv = GetAttributeValue(aNode, nsHTMLAtoms::type, value);
+    isInCiteBlockquote = NS_SUCCEEDED(rv) && value.EqualsIgnoreCase("cite");
+  }
+
+  if (mLineBreakDue && !isInCiteBlockquote)
     EnsureVerticalSpace(mFloatingLines);
 
   // Check if this tag's content that should not be output
@@ -705,8 +646,17 @@ nsPlainTextSerializer::DoOpenContainer(const nsIParserNode* aNode, PRInt32 aTag)
     return NS_OK;
   }
 
-  if (type == eHTMLTag_p || type == eHTMLTag_pre) {
-    EnsureVerticalSpace(1); // Should this be 0 in unformatted case?
+  if (type == eHTMLTag_p)
+    EnsureVerticalSpace(1);
+  else if (type == eHTMLTag_pre) {
+    if (GetLastBool(mIsInCiteBlockquote))
+      EnsureVerticalSpace(0);
+    else if (mHasWrittenCiteBlockquote) {
+      EnsureVerticalSpace(0);
+      mHasWrittenCiteBlockquote = PR_FALSE;
+    }
+    else
+      EnsureVerticalSpace(1);
   }
   else if (type == eHTMLTag_tr) {
     PushBool(mHasWrittenCellsForRow, PR_FALSE);
@@ -776,8 +726,8 @@ nsPlainTextSerializer::DoOpenContainer(const nsIParserNode* aNode, PRInt32 aTag)
     }
     else {
       static char bulletCharArray[] = "*o+#";
-      NS_ASSERTION(mULCount > 0, "mULCount should be greater than 0 here");
-      char bulletChar = bulletCharArray[(mULCount - 1) % 4];
+      PRUint32 index = mULCount > 0 ? (mULCount - 1) : 3;
+      char bulletChar = bulletCharArray[index % 4];
       mInIndentString.Append(PRUnichar(bulletChar));
     }
     
@@ -797,19 +747,14 @@ nsPlainTextSerializer::DoOpenContainer(const nsIParserNode* aNode, PRInt32 aTag)
     ++mSpanLevel;
   }
   else if (type == eHTMLTag_blockquote) {
-    EnsureVerticalSpace(1);
-
-    nsAutoString value;
-    nsresult rv = GetAttributeValue(aNode, nsHTMLAtoms::type, value);
-
-    PRBool isInCiteBlockquote =
-      NS_SUCCEEDED(rv) && value.LowerCaseEqualsLiteral("cite");
     // Push
     PushBool(mIsInCiteBlockquote, isInCiteBlockquote);
     if (isInCiteBlockquote) {
+      EnsureVerticalSpace(0);
       mCiteQuoteLevel++;
     }
     else {
+      EnsureVerticalSpace(1);
       mIndent += kTabSize; // Check for some maximum value?
     }
   }
@@ -960,7 +905,7 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
     mLineBreakDue = PR_TRUE;
   } 
   else if (type == eHTMLTag_pre) {
-    mFloatingLines = 1;
+    mFloatingLines = GetLastBool(mIsInCiteBlockquote) ? 0 : 1;
     mLineBreakDue = PR_TRUE;
   }
   else if (type == eHTMLTag_ul) {
@@ -1004,12 +949,13 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
 
     if (isInCiteBlockquote) {
       mCiteQuoteLevel--;
+      mFloatingLines = 0;
+      mHasWrittenCiteBlockquote = PR_TRUE;
     }
     else {
       mIndent -= kTabSize;
+      mFloatingLines = 1;
     }
-
-    mFloatingLines = 1;
     mLineBreakDue = PR_TRUE;
   }
   else if (IsBlockLevel(aTag)
@@ -1101,6 +1047,11 @@ nsPlainTextSerializer::DoAddLeaf(const nsIParserNode *aNode, PRInt32 aTag,
   if (!DoOutput()) {
     return NS_OK;
   }
+
+  if (aTag != eHTMLTag_whitespace && aTag != eHTMLTag_newline) {
+    // Make sure to reset this, since it's no longer true.
+    mHasWrittenCiteBlockquote = PR_FALSE;
+  }
   
   if (mLineBreakDue)
     EnsureVerticalSpace(mFloatingLines);
@@ -1130,8 +1081,7 @@ nsPlainTextSerializer::DoAddLeaf(const nsIParserNode *aNode, PRInt32 aTag,
     Write(aText);
   }
   else if (type == eHTMLTag_entity) {
-    nsIParserService* parserService =
-      nsContentUtils::GetParserServiceWeakRef();
+    nsIParserService* parserService = nsContentUtils::GetParserService();
     if (parserService) {
       nsAutoString str(aText);
       PRInt32 entity;
@@ -1289,12 +1239,13 @@ nsPlainTextSerializer::Output(nsString& aString)
     mStartedOutput = PR_TRUE;
   }
 
-  // First, replace all nbsp characters with spaces,
-  // which the unicode encoder won't do for us.
-  static PRUnichar nbsp = 160;
-  static PRUnichar space = ' ';
-  aString.ReplaceChar(nbsp, space);
-
+  if (!(mFlags & nsIDocumentEncoder::OutputPersistNBSP)) {
+    // First, replace all nbsp characters with spaces,
+    // which the unicode encoder won't do for us.
+    static PRUnichar nbsp = 160;
+    static PRUnichar space = ' ';
+    aString.ReplaceChar(nbsp, space);
+  }
   mOutputString->Append(aString);
 }
 
@@ -1377,10 +1328,7 @@ nsPlainTextSerializer::AddToLine(const PRUnichar * aLineFragment,
     PRUint32 bonuswidth = (mWrapColumn > 20) ? 4 : 0;
 
     // XXX: Should calculate prefixwidth with GetUnicharStringWidth
-    while(mCurrentLineWidth+prefixwidth > mWrapColumn+bonuswidth) {
-      // Must wrap. Let's find a good place to do that.
-      nsresult result = NS_OK;
-      
+    while(mCurrentLineWidth+prefixwidth > mWrapColumn+bonuswidth) {      
       // We go from the end removing one letter at a time until
       // we have a reasonable width
       PRInt32 goodSpace = mCurrentLine.Length();
@@ -1392,20 +1340,16 @@ nsPlainTextSerializer::AddToLine(const PRUnichar * aLineFragment,
 
       goodSpace++;
       
-      PRBool oNeedMoreText;
-      if (nsnull != mLineBreaker) {
-        result = mLineBreaker->Prev(mCurrentLine.get(), 
-                                    mCurrentLine.Length(), goodSpace,
-                                    (PRUint32 *) &goodSpace, &oNeedMoreText);
-        if (oNeedMoreText) {
-          goodSpace = -1;
-        }
-        else if (nsCRT::IsAsciiSpace(mCurrentLine.CharAt(goodSpace-1))) {
+      if (mLineBreaker) {
+        goodSpace = mLineBreaker->Prev(mCurrentLine.get(), 
+                                    mCurrentLine.Length(), goodSpace);
+        if (goodSpace != NS_LINEBREAKER_NEED_MORE_TEXT &&
+            nsCRT::IsAsciiSpace(mCurrentLine.CharAt(goodSpace-1))) {
           --goodSpace;    // adjust the position since line breaker returns a position next to space
         }
       }
       // fallback if the line breaker is unavailable or failed
-      if (nsnull == mLineBreaker || NS_FAILED(result)) {
+      if (!mLineBreaker) {
         goodSpace = mWrapColumn-prefixwidth;
         while (goodSpace >= 0 &&
                !nsCRT::IsAsciiSpace(mCurrentLine.CharAt(goodSpace))) {
@@ -1414,18 +1358,18 @@ nsPlainTextSerializer::AddToLine(const PRUnichar * aLineFragment,
       }
       
       nsAutoString restOfLine;
-      if (goodSpace < 0) {
+      if (goodSpace == NS_LINEBREAKER_NEED_MORE_TEXT) {
         // If we don't found a good place to break, accept long line and
         // try to find another place to break
         goodSpace=(prefixwidth>mWrapColumn+1)?1:mWrapColumn-prefixwidth+1;
-        result = NS_OK;
-        if (nsnull != mLineBreaker) {
-          result = mLineBreaker->Next(mCurrentLine.get(), 
-                                      mCurrentLine.Length(), goodSpace,
-                                      (PRUint32 *) &goodSpace, &oNeedMoreText);
+        if (mLineBreaker) {
+          goodSpace = mLineBreaker->Next(mCurrentLine.get(), 
+                                      mCurrentLine.Length(), goodSpace);
+          if (goodSpace == NS_LINEBREAKER_NEED_MORE_TEXT)
+            goodSpace = mCurrentLine.Length();
         }
         // fallback if the line breaker is unavailable or failed
-        if (nsnull == mLineBreaker || NS_FAILED(result)) {
+        if (!mLineBreaker) {
           goodSpace=(prefixwidth>mWrapColumn)?1:mWrapColumn-prefixwidth;
           while (goodSpace < linelength &&
                  !nsCRT::IsAsciiSpace(mCurrentLine.CharAt(goodSpace))) {
@@ -1574,7 +1518,7 @@ nsPlainTextSerializer::OutputQuotesAndIndent(PRBool stripTrailingSpaces /* = PR_
     }
     if (!mCurrentLine.IsEmpty()) {
       /* Better don't output a space here, if the line is empty,
-         in case a recieving f=f-aware UA thinks, this were a flowed line,
+         in case a receiving f=f-aware UA thinks, this were a flowed line,
          which it isn't - it's just empty.
          (Flowed lines may be joined with the following one,
          so the empty line may be lost completely.) */
@@ -1628,7 +1572,7 @@ nsPlainTextSerializer::Write(const nsAString& aString)
 {
 #ifdef DEBUG_wrapping
   printf("Write(%s): wrap col = %d\n",
-         NS_ConvertUCS2toUTF8(aString).get(), mWrapColumn);
+         NS_ConvertUTF16toUTF8(aString).get(), mWrapColumn);
 #endif
 
   PRInt32 bol = 0;
@@ -1649,18 +1593,18 @@ nsPlainTextSerializer::Write(const nsAString& aString)
 
     // This mustn't be mixed with intelligent wrapping without clearing
     // the mCurrentLine buffer before!!!
-    NS_WARN_IF_FALSE(mCurrentLine.IsEmpty(),
+    NS_ASSERTION(mCurrentLine.IsEmpty(),
                  "Mixed wrapping data and nonwrapping data on the same line");
     if (!mCurrentLine.IsEmpty()) {
       FlushLine();
     }
-    
+
     // Put the mail quote "> " chars in, if appropriate.
     // Have to put it in before every line.
     while(bol<totLen) {
-      if(mAtFirstColumn) {
-        OutputQuotesAndIndent();
-      }
+      PRBool outputQuotes = mAtFirstColumn;
+      PRBool atFirstColumn = mAtFirstColumn;
+      PRBool outputLineBreak = PR_FALSE;
 
       // Find one of '\n' or '\r' using iterators since nsAString
       // doesn't have the old FindCharInSet function.
@@ -1692,20 +1636,19 @@ nsPlainTextSerializer::Write(const nsAString& aString)
             mInWhitespace = PR_FALSE;
           }
         }
-        Output(stringpart);
+        mCurrentLine.Assign(stringpart);
         mEmptyLines=-1;
-        mAtFirstColumn = mAtFirstColumn && (totLen-bol)==0;
+        atFirstColumn = mAtFirstColumn && (totLen-bol)==0;
         bol = totLen;
       } 
       else {
         // There is a newline
         nsAutoString stringpart(Substring(aString, bol, newline-bol));
         mInWhitespace = PR_TRUE;
-        Output(stringpart);
-        // and write the newline
-        Output(mLineBreak);
+        mCurrentLine.Assign(stringpart);
+        outputLineBreak = PR_TRUE;
         mEmptyLines=0;
-        mAtFirstColumn = PR_TRUE;
+        atFirstColumn = PR_TRUE;
         bol = newline+1;
         if('\r' == *iter && bol < totLen && '\n' == *++iter) {
           // There was a CRLF in the input. This used to be illegal and
@@ -1714,8 +1657,22 @@ nsPlainTextSerializer::Write(const nsAString& aString)
           bol++;
         }
       }
+
+      if(outputQuotes) {
+        // Note: this call messes with mAtFirstColumn
+        OutputQuotesAndIndent();
+      }
+
+      Output(mCurrentLine);
+      if (outputLineBreak) {
+        Output(mLineBreak);
+      }
+      mAtFirstColumn = atFirstColumn;
     }
 
+    // Reset mCurrentLine.
+    mCurrentLine.Truncate();
+
 #ifdef DEBUG_wrapping
     printf("No wrapping: newline is %d, totLen is %d\n",
            newline, totLen);
@@ -1731,7 +1688,6 @@ nsPlainTextSerializer::Write(const nsAString& aString)
   // If needed, strip out all "end of lines"
   // and multiple whitespace between words
   PRInt32 nextpos;
-  nsAutoString tempstr;
   const PRUnichar * offsetIntoBuffer = nsnull;
   
   while (bol < totLen) {    // Loop over lines
@@ -1812,8 +1768,7 @@ nsPlainTextSerializer::GetAttributeValue(const nsIParserNode* aNode,
                                          nsString& aValueRet)
 {
   if (mContent) {
-    if (NS_CONTENT_ATTR_NOT_THERE != mContent->GetAttr(kNameSpaceID_None,
-                                                       aName, aValueRet)) {
+    if (mContent->GetAttr(kNameSpaceID_None, aName, aValueRet)) {
       return NS_OK;
     }
   }
@@ -1842,7 +1797,7 @@ PRBool
 nsPlainTextSerializer::IsCurrentNodeConverted(const nsIParserNode* aNode)
 {
   nsAutoString value;
-  nsresult rv = GetAttributeValue(aNode, nsHTMLAtoms::kClass, value);
+  nsresult rv = GetAttributeValue(aNode, nsHTMLAtoms::_class, value);
   return (NS_SUCCEEDED(rv) &&
           (value.EqualsIgnoreCase("moz-txt", 7) ||
            value.EqualsIgnoreCase("\"moz-txt", 8)));
@@ -1853,17 +1808,14 @@ nsPlainTextSerializer::IsCurrentNodeConverted(const nsIParserNode* aNode)
 PRInt32
 nsPlainTextSerializer::GetIdForContent(nsIContent* aContent)
 {
-  if (!aContent->IsContentOfType(nsIContent::eHTML)) {
+  if (!aContent->IsNodeOfType(nsINode::eHTML)) {
     return eHTMLTag_unknown;
   }
 
-  nsIParserService* parserService = nsContentUtils::GetParserServiceWeakRef();
+  nsIParserService* parserService = nsContentUtils::GetParserService();
 
-  PRInt32 id;
-  nsresult rv = parserService->HTMLAtomTagToId(aContent->Tag(), &id);
-  NS_ASSERTION(NS_SUCCEEDED(rv), "Can't map HTML tag to id!");
-
-  return id;
+  return parserService ? parserService->HTMLAtomTagToId(aContent->Tag()) :
+                         eHTMLTag_unknown;
 }
 
 /**
@@ -1875,7 +1827,7 @@ nsPlainTextSerializer::IsBlockLevel(PRInt32 aId)
 {
   PRBool isBlock = PR_FALSE;
 
-  nsIParserService* parserService = nsContentUtils::GetParserServiceWeakRef();
+  nsIParserService* parserService = nsContentUtils::GetParserService();
   if (parserService) {
     parserService->IsBlock(aId, isBlock);
   }
@@ -1891,7 +1843,7 @@ nsPlainTextSerializer::IsContainer(PRInt32 aId)
 {
   PRBool isContainer = PR_FALSE;
 
-  nsIParserService* parserService = nsContentUtils::GetParserServiceWeakRef();
+  nsIParserService* parserService = nsContentUtils::GetParserService();
   if (parserService) {
     parserService->IsContainer(aId, isContainer);
   }
@@ -1925,7 +1877,7 @@ nsPlainTextSerializer::IsInPre()
 }
 
 /**
- * This method is required only to indentify LI's inside OL.
+ * This method is required only to identify LI's inside OL.
  * Returns TRUE if we are inside an OL tag and FALSE otherwise.
  */
 PRBool
