@@ -36,7 +36,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 /*
- * nsRange.cpp: Implementation of the nsIDOMRange object.
+ * Implementation of the DOM nsIDOMRange object.
  */
 
 #include "nscore.h"
@@ -50,60 +50,20 @@
 #include "nsIDOMDocumentFragment.h"
 #include "nsIContent.h"
 #include "nsIDocument.h"
-#include "nsVoidArray.h"
 #include "nsIDOMText.h"
 #include "nsDOMError.h"
 #include "nsIContentIterator.h"
 #include "nsIDOMNodeList.h"
-#include "nsIParser.h"
-#include "nsIComponentManager.h"
-#include "nsParserCIID.h"
-#include "nsIFragmentContentSink.h"
-#include "nsIContentSink.h"
-#include "nsIEnumerator.h"
-#include "nsIScriptSecurityManager.h"
-#include "nsIScriptGlobalObject.h"
-#include "nsIScriptContext.h"
-#include "nsIHTMLDocument.h"
-#include "nsCRT.h"
-
-#include "nsIJSContextStack.h"
-// XXX Temporary inclusion to deal with fragment parsing
-#include "nsHTMLParts.h"
 
 #include "nsContentUtils.h"
 
-static NS_DEFINE_CID(kCParserCID, NS_PARSER_CID);
-
-PRMonitor*   nsRange::mMonitor = nsnull;
-nsVoidArray* nsRange::mStartAncestors = nsnull;      
-nsVoidArray* nsRange::mEndAncestors = nsnull;        
-nsVoidArray* nsRange::mStartAncestorOffsets = nsnull; 
-nsVoidArray* nsRange::mEndAncestorOffsets = nsnull;  
-
 nsresult NS_NewContentIterator(nsIContentIterator** aInstancePtrResult);
 nsresult NS_NewContentSubtreeIterator(nsIContentIterator** aInstancePtrResult);
 
-
-
-#ifdef XP_MAC
-#pragma mark -
-#pragma mark  utility functions (some exposed through nsRangeUtils, below
-#pragma mark -
-#endif
-
-
 /******************************************************
  * stack based utilty class for managing monitor
  ******************************************************/
 
-class nsAutoRangeLock
-{
-  public:
-    nsAutoRangeLock()  { nsRange::Lock(); }
-    ~nsAutoRangeLock() { nsRange::Unlock(); }
-};
-
 // NS_ERROR_DOM_NOT_OBJECT_ERR is not the correct one to throw, but spec doesn't say
 // what is
 #define VALIDATE_ACCESS(node_)                                                     \
@@ -296,15 +256,6 @@ PRBool GetNodeBracketPoints(nsIContent* aNode,
   return PR_TRUE;
 }
 
-
-
-#ifdef XP_MAC
-#pragma mark -
-#pragma mark  class nsRangeUtils
-#pragma mark -
-#endif
-
-
 /******************************************************
  * non members
  ******************************************************/
@@ -364,13 +315,6 @@ nsRangeUtils::CompareNodeToRange(nsIContent* aNode, nsIDOMRange* aRange,
                                      outNodeAfter);
 }
 
-#ifdef XP_MAC
-#pragma mark -
-#pragma mark  class nsRange
-#pragma mark -
-#endif
-
-
 /******************************************************
  * non members
  ******************************************************/
@@ -409,27 +353,6 @@ nsRange::~nsRange()
   // note that "nsCOMPtr<nsIDOMmNode>()" is the moral equivalent of null
 } 
 
-// for layout module destructor
-void nsRange::Shutdown()
-{
-  if (mMonitor) {
-    PR_DestroyMonitor(mMonitor);
-    mMonitor = nsnull;
-  }
-
-  delete mStartAncestors;      
-  mStartAncestors = nsnull;      
-
-  delete mEndAncestors;        
-  mEndAncestors = nsnull;        
-
-  delete mStartAncestorOffsets; 
-  mStartAncestorOffsets = nsnull; 
-
-  delete mEndAncestorOffsets;  
-  mEndAncestorOffsets = nsnull;  
-}
-
 /******************************************************
  * nsISupports
  ******************************************************/
@@ -702,55 +625,34 @@ nsresult nsRange::DoSetRange(nsIDOMNode* aStartN, PRInt32 aStartOffset,
 PRBool nsRange::IsIncreasing(nsIDOMNode* aStartN, PRInt32 aStartOffset,
                              nsIDOMNode* aEndN, PRInt32 aEndOffset)
 {
-  PRInt32 startIdx = 0;
-  PRInt32 endIdx = 0;
-  PRInt32 commonNodeStartOffset = 0;
-  PRInt32 commonNodeEndOffset = 0;
-  
   // no trivial cases please
   if (!aStartN || !aEndN) 
     return PR_FALSE;
   
   // check common case first
-  if (aStartN==aEndN)
+  if (aStartN == aEndN)
   {
-    if (aStartOffset>aEndOffset) 
-      return PR_FALSE;
-    else 
-      return PR_TRUE;
-  }
-  
-  // thread safety - need locks around here to end of routine to protect use of static members
-  nsAutoRangeLock lock;
-  
-  // lazy allocation of static arrays
-  if (!mStartAncestors)
-  {
-    mStartAncestors = new nsAutoVoidArray();
-    if (!mStartAncestors) return NS_ERROR_OUT_OF_MEMORY;
-    mStartAncestorOffsets = new nsAutoVoidArray();
-    if (!mStartAncestorOffsets) return NS_ERROR_OUT_OF_MEMORY;
-    mEndAncestors = new nsAutoVoidArray();
-    if (!mEndAncestors) return NS_ERROR_OUT_OF_MEMORY;
-    mEndAncestorOffsets = new nsAutoVoidArray();
-    if (!mEndAncestorOffsets) return NS_ERROR_OUT_OF_MEMORY;
+    return aStartOffset <= aEndOffset;
   }
 
-  // refresh ancestor data
-  mStartAncestors->Clear();
-  mStartAncestorOffsets->Clear();
-  mEndAncestors->Clear();
-  mEndAncestorOffsets->Clear();
-
-  nsContentUtils::GetAncestorsAndOffsets(aStartN, aStartOffset,
-                                         mStartAncestors, mStartAncestorOffsets);
-
-  nsContentUtils::GetAncestorsAndOffsets(aEndN, aEndOffset,
-                                         mEndAncestors, mEndAncestorOffsets);
+  nsCOMPtr<nsIContent> startCont = do_QueryInterface(aStartN);
+  nsCOMPtr<nsIContent> endCont = do_QueryInterface(aEndN);
 
+  nsAutoVoidArray startAncestors, endAncestors;
+  nsIContent* node = startCont;
+  while (node) {
+    startAncestors.AppendElement(node);
+    node = node->GetParent();
+  }
+  node = endCont;
+  while (node) {
+    endAncestors.AppendElement(node);
+    node = node->GetParent();
+  }
+  
   // Get the number of ancestors, adjusting for zero-based counting.
-  startIdx = mStartAncestors->Count() - 1;
-  endIdx   = mEndAncestors->Count() - 1;
+  PRInt32 startIdx = startAncestors.Count() - 1;
+  PRInt32 endIdx = endAncestors.Count() - 1;
 
   // Ensure that we actually have ancestors to iterate through
   if (startIdx < 0) {
@@ -771,14 +673,22 @@ PRBool nsRange::IsIncreasing(nsIDOMNode* aStartN, PRInt32 aStartOffset,
     // numStartAncestors will only be <0 if one endpoint's node is the
     // common ancestor of the other
   } while (startIdx >= 0 && endIdx >= 0 &&
-           mStartAncestors->ElementAt(startIdx) == mEndAncestors->ElementAt(endIdx));
+           startAncestors.FastElementAt(startIdx) ==
+           endAncestors.FastElementAt(endIdx));
   // now back up one and that's the last common ancestor from the root,
   // or the first common ancestor from the leaf perspective
   ++startIdx;
   ++endIdx;
   // both indexes are now >= 0
-  commonNodeStartOffset = NS_PTR_TO_INT32(mStartAncestorOffsets->ElementAt(startIdx));
-  commonNodeEndOffset   = NS_PTR_TO_INT32(mEndAncestorOffsets->ElementAt(endIdx));
+  
+  
+  PRInt32 commonNodeStartOffset = startIdx == 0 ? aStartOffset :
+    NS_STATIC_CAST(nsIContent*, startAncestors.FastElementAt(startIdx))->IndexOf(
+      NS_STATIC_CAST(nsIContent*, startAncestors.FastElementAt(startIdx - 1)));
+
+  PRInt32 commonNodeEndOffset = endIdx == 0 ? aEndOffset :
+    NS_STATIC_CAST(nsIContent*, endAncestors.FastElementAt(endIdx))->IndexOf(
+      NS_STATIC_CAST(nsIContent*, endAncestors.FastElementAt(endIdx - 1)));
 
   if (commonNodeStartOffset > commonNodeEndOffset) {
     return PR_FALSE;
@@ -792,16 +702,9 @@ PRBool nsRange::IsIncreasing(nsIDOMNode* aStartN, PRInt32 aStartOffset,
   // of both endpoints.  In this case, we compare the depth of the ancestor tree to determine
   // the ordering.
 
-  if (startIdx == endIdx) {
-    // whoa nelly. this shouldn't happen.
-    NS_NOTREACHED("nsRange::IsIncreasing");
-  }
+  NS_ASSERTION(startIdx != endIdx, "whoa nelly. this shouldn't happen");
 
-  if (startIdx < endIdx) {
-    return PR_TRUE;
-  }
-
-  return PR_FALSE;
+  return startIdx < endIdx;
 }
 
 PRInt32 nsRange::IndexOf(nsIDOMNode* aChildNode)
@@ -972,10 +875,74 @@ nsresult nsRange::GetCommonAncestorContainer(nsIDOMNode** aCommonParent)
   return nsContentUtils::GetCommonAncestor(mStartParent, mEndParent, aCommonParent);
 }
 
+nsresult nsRange::IsValidBoundary(nsIDOMNode* aNode)
+{
+  if (!aNode) {
+    // DOM 2 Range specification doesn't define the error code for this case,
+    // but this is the best one we have.
+    return NS_ERROR_DOM_RANGE_BAD_BOUNDARYPOINTS_ERR;
+  }
+
+  PRUint16 nodeType = 0;
+  aNode->GetNodeType(&nodeType);
+  switch (nodeType) {
+    case nsIDOMNode::DOCUMENT_TYPE_NODE:
+    case nsIDOMNode::ENTITY_NODE:
+    case nsIDOMNode::NOTATION_NODE:
+      return NS_ERROR_DOM_RANGE_INVALID_NODE_TYPE_ERR;
+    case nsIDOMNode::DOCUMENT_NODE:
+    case nsIDOMNode::DOCUMENT_FRAGMENT_NODE:
+    case nsIDOMNode::ATTRIBUTE_NODE:
+      return NS_OK;
+    default:
+      break;
+  }
+
+  nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
+  if (!content) {
+    // DOM 2 Range specification doesn't define the error code for this case,
+    // but this is the best one we have.
+    return NS_ERROR_DOM_RANGE_BAD_BOUNDARYPOINTS_ERR;
+  }
+
+  // Elements etc. must be in document or in document fragment,
+  // text nodes in document, in document fragment or in attribute.
+  if (content->IsInDoc()) {
+    return NS_OK;
+  }
+
+  nsINode* parent = content->GetNodeParent();
+  if (parent) {
+    if (parent->IsNodeOfType(nsINode::eATTRIBUTE)) {
+      return NS_OK;
+    }
+
+    do {
+      if (parent->IsNodeOfType(nsINode::eDOCUMENT_FRAGMENT)) {
+        return NS_OK;
+      }
+      parent = parent->GetNodeParent();
+    } while(parent);
+  }
+
+#ifdef DEBUG_smaug
+  nsAutoString name;
+  content->Tag()->ToString(name);
+  printf("nsRange::IsValidBoundary: node is not a valid boundary point [%s]\n",
+         NS_ConvertUTF16toUTF8(name).get());
+#endif
+
+  // DOM 2 Range specification doesn't define the error code for this case,
+  // but this is the best one we have.
+  return NS_ERROR_DOM_RANGE_BAD_BOUNDARYPOINTS_ERR;
+}
+
 nsresult nsRange::SetStart(nsIDOMNode* aParent, PRInt32 aOffset)
 {
   VALIDATE_ACCESS(aParent);
-  
+  nsresult rv = IsValidBoundary(aParent);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   PRInt32 len = GetNodeLength(aParent);
   if ( (aOffset < 0) || (len < 0) || (aOffset > len) )
     return NS_ERROR_DOM_INDEX_SIZE_ERR;
@@ -1024,7 +991,9 @@ nsresult nsRange::SetStartAfter(nsIDOMNode* aSibling)
 nsresult nsRange::SetEnd(nsIDOMNode* aParent, PRInt32 aOffset)
 {
   VALIDATE_ACCESS(aParent);
-  
+  nsresult rv = IsValidBoundary(aParent);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   PRInt32 len = GetNodeLength(aParent);
   if ( (aOffset < 0) || (len < 0) || (aOffset > len) )
     return NS_ERROR_DOM_INDEX_SIZE_ERR;
@@ -1148,12 +1117,6 @@ nsresult nsRange::SelectNodeContents(nsIDOMNode* aN)
   return DoSetRange(theNode,0,theNode,indx);
 }
 
-#ifdef XP_MAC
-#pragma mark -
-#pragma mark  class RangeSubtreeIterator
-#pragma mark -
-#endif
-
 // The Subtree Content Iterator only returns subtrees that are
 // completely within a given range. It doesn't return a CharacterData
 // node that contains either the start or end point of the range.
@@ -1386,7 +1349,7 @@ RangeSubtreeIterator::Prev()
 }
 
 
-// CollapseRangeAfterDelete() is a utiltiy method that is used by
+// CollapseRangeAfterDelete() is a utility method that is used by
 // DeleteContents() and ExtractContents() to collapse the range
 // in the correct place, under the range's root container (the
 // range end points common container) as outlined by the Range spec:
@@ -1588,8 +1551,10 @@ nsresult nsRange::DeleteContents()
 
       node->GetParentNode(getter_AddRefs(parent));
 
-      res = parent->RemoveChild(node, getter_AddRefs(tmpNode));
-      if (NS_FAILED(res)) return res;
+      if (parent) {
+        res = parent->RemoveChild(node, getter_AddRefs(tmpNode));
+        if (NS_FAILED(res)) return res;
+      }
     }
   }
 
@@ -1754,6 +1719,13 @@ nsresult nsRange::CloneContents(nsIDOMDocumentFragment** aReturn)
   res = mStartParent->GetOwnerDocument(getter_AddRefs(document));
   if (NS_FAILED(res)) return res;
 
+  if (!document) {
+    document = do_QueryInterface(mStartParent);
+  }
+
+  NS_ASSERTION(document, "CloneContents needs a document to continue.");
+  if (!document) return NS_ERROR_FAILURE;
+
   // Create a new document fragment in the context of this document,
   // which might be null
 
@@ -1761,7 +1733,8 @@ nsresult nsRange::CloneContents(nsIDOMDocumentFragment** aReturn)
 
   nsCOMPtr<nsIDocument> doc(do_QueryInterface(document));
 
-  res = NS_NewDocumentFragment(getter_AddRefs(clonedFrag), doc);
+  res = NS_NewDocumentFragment(getter_AddRefs(clonedFrag),
+                               doc->NodeInfoManager());
   if (NS_FAILED(res)) return res;
 
   nsCOMPtr<nsIDOMNode> commonCloneAncestor(do_QueryInterface(clonedFrag));
@@ -1962,11 +1935,25 @@ nsresult nsRange::InsertNode(nsIDOMNode* aN)
     nsCOMPtr<nsIDOMNode> tSCParentNode;
     res = tStartContainer->GetParentNode(getter_AddRefs(tSCParentNode));
     if(NS_FAILED(res)) return res;
+    
+    PRBool isCollapsed;
+    res = GetCollapsed(&isCollapsed);
+    if(NS_FAILED(res)) return res;
+
+    PRInt32 tEndOffset;
+    GetEndOffset(&tEndOffset);
 
     nsCOMPtr<nsIDOMText> secondPart;
     res = startTextNode->SplitText(tStartOffset, getter_AddRefs(secondPart));
     if (NS_FAILED(res)) return res;
     
+    // SplitText collapses the range; fix that (bug 253609)
+    if (!isCollapsed)
+    {
+      res = SetEnd(secondPart, tEndOffset - tStartOffset);
+      if(NS_FAILED(res)) return res;
+    }
+    
     nsCOMPtr<nsIDOMNode> tResultNode;
     return tSCParentNode->InsertBefore(aN, secondPart, getter_AddRefs(tResultNode));
   }  
@@ -2270,7 +2257,7 @@ nsresult nsRange::OwnerChildReplaced(nsIContent* aParentNode, PRInt32 aOffset, n
   
 
 nsresult
-nsRange::TextOwnerChanged(nsIContent* aTextNode, nsVoidArray *aRangeList,
+nsRange::TextOwnerChanged(nsIContent* aTextNode, const nsVoidArray *aRangeList,
                           PRInt32 aStartChanged, PRInt32 aEndChanged,
                           PRInt32 aReplaceLength)
 {
@@ -2329,196 +2316,13 @@ nsRange::TextOwnerChanged(nsIContent* aTextNode, nsVoidArray *aRangeList,
 
 // nsIDOMNSRange interface
 NS_IMETHODIMP    
-nsRange::CreateContextualFragment(const nsAString& aFragment, 
+nsRange::CreateContextualFragment(const nsAString& aFragment,
                                   nsIDOMDocumentFragment** aReturn)
 {
-  if (!mIsPositioned) {
-    return NS_ERROR_FAILURE;
-  }
-
-  // Create a new parser for this entire operation
-  nsresult result;
-  nsCOMPtr<nsIParser> parser = do_CreateInstance(kCParserCID, &result);
-  NS_ENSURE_SUCCESS(result, result);
-
-  nsCOMPtr<nsIDocument> document;
-  nsCOMPtr<nsIDOMDocument> domDocument;
-
-  result = mStartParent->GetOwnerDocument(getter_AddRefs(domDocument));
-  if (domDocument && NS_SUCCEEDED(result)) {
-    document = do_QueryInterface(domDocument, &result);
-  }
-
-  nsVoidArray tagStack;
-  nsCOMPtr<nsIDOMNode> parent = mStartParent;
-  while (parent && 
-         (parent != domDocument) && 
-         NS_SUCCEEDED(result)) {
-    PRUint16 nodeType;
-    
-    parent->GetNodeType(&nodeType);
-    if (nsIDOMNode::ELEMENT_NODE == nodeType) {
-      PRInt32 namespaceID;
-      nsAutoString tagName, uriStr;
-      parent->GetNodeName(tagName);
-
-      // see if we need to add xmlns declarations
-      nsCOMPtr<nsIContent> content( do_QueryInterface(parent) );
-      PRUint32 count = content->GetAttrCount();
-      if (count > 0) {
-        PRUint32 index;
-        nsAutoString nameStr, prefixStr, valueStr;
-        nsCOMPtr<nsIAtom> attrName, attrPrefix;
-
-        for (index = 0; index < count; index++) {
-    
-          content->GetAttrNameAt(index,
-                                &namespaceID,
-                                getter_AddRefs(attrName),
-                                getter_AddRefs(attrPrefix));
-    
-          if (namespaceID == kNameSpaceID_XMLNS) {
-            content->GetAttr(namespaceID, attrName, uriStr);
-
-            // really want something like nsXMLContentSerializer::SerializeAttr()
-            tagName.Append(NS_LITERAL_STRING(" xmlns")); // space important
-            if (attrPrefix) {
-              tagName.Append(PRUnichar(':'));
-              attrName->ToString(nameStr);
-              tagName.Append(nameStr);
-            }
-            tagName.Append(NS_LITERAL_STRING("=\"") + uriStr + NS_LITERAL_STRING("\""));
-          }
-        }
-      }
-
-      // XXX Wish we didn't have to allocate here
-      PRUnichar* name = ToNewUnicode(tagName);
-      if (name) {
-        tagStack.AppendElement(name);
-        nsCOMPtr<nsIDOMNode> temp = parent;
-        result = temp->GetParentNode(getter_AddRefs(parent));
-      }
-      else {
-        result = NS_ERROR_OUT_OF_MEMORY;
-      }
-    }
-    else {
-      nsCOMPtr<nsIDOMNode> temp = parent;
-      result = temp->GetParentNode(getter_AddRefs(parent));
-    }
-  }
-
-  if (NS_SUCCEEDED(result)) {
-    nsCAutoString contentType;
-    PRBool bCaseSensitive = PR_TRUE;
-    if (document) {
-      nsAutoString buf;
-      document->GetContentType(buf);
-      CopyUCS2toASCII(buf, contentType);
-      bCaseSensitive = document->IsCaseSensitive();
-    }
-    else {
-      contentType.AssignLiteral("text/xml");
-    }
-
-    nsCOMPtr<nsIHTMLDocument> htmlDoc(do_QueryInterface(domDocument));
-    PRBool bHTML = htmlDoc && !bCaseSensitive;
-    nsCOMPtr<nsIFragmentContentSink> sink;
-    if (bHTML) {
-      result = NS_NewHTMLFragmentContentSink(getter_AddRefs(sink));
-    } else {
-      result = NS_NewXMLFragmentContentSink(getter_AddRefs(sink));
-    }
-    if (NS_SUCCEEDED(result)) {
-      sink->SetTargetDocument(document);
-      nsCOMPtr<nsIContentSink> contentsink( do_QueryInterface(sink) );
-      parser->SetContentSink(contentsink);
-
-      // If there's no JS or system JS running,
-      // push the current document's context on the JS context stack
-      // so that event handlers in the fragment do not get 
-      // compiled with the system principal.
-      nsCOMPtr<nsIJSContextStack> ContextStack;
-      if (document) {
-        nsCOMPtr<nsIPrincipal> sysPrin;
-        nsCOMPtr<nsIPrincipal> subjectPrin;
-
-        nsIScriptSecurityManager *secMan = nsContentUtils::GetSecurityManager();
-
-        // Just to compare, not to use!
-        result = secMan->GetSystemPrincipal(getter_AddRefs(sysPrin));
-        if (NS_SUCCEEDED(result))
-            result = secMan->GetSubjectPrincipal(getter_AddRefs(subjectPrin));
-        // If there's no subject principal, there's no JS running, so we're in system code.
-        // (just in case...null subject principal will probably never happen)
-        if (NS_SUCCEEDED(result) &&
-           (!subjectPrin || sysPrin.get() == subjectPrin.get())) {
-          nsIScriptGlobalObject *globalObj = document->GetScriptGlobalObject();
-          JSContext* cx = nsnull;
-
-          if (globalObj) {
-            nsIScriptContext *scriptContext = globalObj->GetContext();
-
-            if (scriptContext) {
-              cx = (JSContext*)scriptContext->GetNativeContext();
-            }
-          }
-
-          if (cx) {
-            ContextStack = do_GetService("@mozilla.org/js/xpc/ContextStack;1");
-            if (ContextStack) {
-              result = ContextStack->Push(cx);
-            }
-          }
-        }
-      }
-
-      nsDTDMode mode = eDTDMode_autodetect;
-      nsCOMPtr<nsIHTMLDocument> htmlDoc(do_QueryInterface(domDocument));
-      if (bHTML) {
-        switch (htmlDoc->GetCompatibilityMode()) {
-          case eCompatibility_NavQuirks:
-            mode = eDTDMode_quirks;
-            break;
-          case eCompatibility_AlmostStandards:
-            mode = eDTDMode_almost_standards;
-            break;
-          case eCompatibility_FullStandards:
-            mode = eDTDMode_full_standards;
-            break;
-          default:
-            NS_NOTREACHED("unknown mode");
-            break;
-        }
-      } else {
-        mode = eDTDMode_full_standards;
-      }
-      result = parser->ParseFragment(aFragment, (void*)0,
-                                     tagStack,
-                                     !bHTML, contentType, mode);
-
-      if (ContextStack) {
-        JSContext *notused;
-        ContextStack->Pop(&notused);
-      }
-
-      if (NS_SUCCEEDED(result)) {
-        sink->GetFragment(aReturn);
-      }
-    }
-  }
-
-  // XXX Ick! Delete strings we allocated above.
-  PRInt32 count = tagStack.Count();
-  for (PRInt32 i = 0; i < count; i++) {
-    PRUnichar* str = (PRUnichar*)tagStack.ElementAt(i);
-    if (str) {
-      nsCRT::free(str);
-    }
-  }
-
-  return result;
+  return
+    mIsPositioned
+    ? nsContentUtils::CreateContextualFragment(mStartParent, aFragment, aReturn)
+    : NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
@@ -2558,25 +2362,3 @@ nsRange::SetBeforeAndAfter(PRBool aBefore, PRBool aAfter)
   mBeforeGenContent = aAfter;
   return NS_OK;
 }
-
-nsresult
-nsRange::Lock()
-{
-  if (!mMonitor)
-    mMonitor = ::PR_NewMonitor();
-
-  if (mMonitor)
-    PR_EnterMonitor(mMonitor);
-
-  return NS_OK;
-}
-
-nsresult
-nsRange::Unlock()
-{
-  if (mMonitor)
-    PR_ExitMonitor(mMonitor);
-
-  return NS_OK;
-}
-
