@@ -35,6 +35,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+/*
+ * nsIContentSerializer implementation that can be used with an
+ * nsIDocumentEncoder to convert an XML DOM to an XML string that
+ * could be parsed into more or less the original DOM.
+ */
+
 #include "nsXMLContentSerializer.h"
 
 #include "nsHTMLAtoms.h"
@@ -48,7 +54,6 @@
 #include "nsIContent.h"
 #include "nsIDocument.h"
 #include "nsINameSpaceManager.h"
-#include "nsITextContent.h"
 #include "nsTextFragment.h"
 #include "nsString.h"
 #include "prprf.h"
@@ -56,6 +61,7 @@
 #include "nsCRT.h"
 #include "nsContentUtils.h"
 #include "nsLayoutAtoms.h"
+#include "nsAttrName.h"
 
 typedef struct {
   nsString mPrefix;
@@ -101,10 +107,11 @@ nsXMLContentSerializer::AppendTextData(nsIDOMNode* aNode,
                                        PRBool aTranslateEntities,
                                        PRBool aIncrColumn)
 {
-  nsCOMPtr<nsITextContent> content(do_QueryInterface(aNode));
-  if (!content) return NS_ERROR_FAILURE;
-
-  const nsTextFragment* frag = content->Text();
+  nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
+  const nsTextFragment* frag;
+  if (!content || !(frag = content->GetText())) {
+    return NS_ERROR_FAILURE;
+  }
 
   PRInt32 endoffset = (aEndOffset == -1) ? frag->GetLength() : aEndOffset;
   PRInt32 length = endoffset - aStartOffset;
@@ -125,7 +132,7 @@ nsXMLContentSerializer::AppendTextData(nsIDOMNode* aNode,
                    aTranslateEntities, aIncrColumn);
   }
   else {
-    AppendToString(NS_ConvertASCIItoUCS2(frag->Get1b()+aStartOffset, length),
+    AppendToString(NS_ConvertASCIItoUTF16(frag->Get1b()+aStartOffset, length),
                    aStr, aTranslateEntities, aIncrColumn);
   }
 
@@ -335,33 +342,66 @@ PRBool
 nsXMLContentSerializer::ConfirmPrefix(nsAString& aPrefix,
                                       const nsAString& aURI,
                                       nsIDOMElement* aElement,
-                                      PRBool aMustHavePrefix)
+                                      PRBool aIsAttribute)
 {
-  if (aPrefix.EqualsLiteral(kXMLNS) ||
-      (aPrefix.EqualsLiteral("xml") &&
-       aURI.EqualsLiteral("http://www.w3.org/XML/1998/namespace"))) {
-    return PR_FALSE;
-  }
-  if (aURI.IsEmpty()) {
-    aPrefix.Truncate();
+  if (aPrefix.EqualsLiteral(kXMLNS)) {
     return PR_FALSE;
   }
 
+  if (aURI.EqualsLiteral("http://www.w3.org/XML/1998/namespace")) {
+    // The prefix must be xml for this namespace. We don't need to declare it,
+    // so always just set the prefix to xml.
+    aPrefix.AssignLiteral("xml");
+
+    return PR_FALSE;
+  }
+
+  PRBool mustHavePrefix;
+  if (aIsAttribute) {
+    if (aURI.IsEmpty()) {
+      // Attribute in the null namespace.  This just shouldn't have a prefix.
+      // And there's no need to push any namespace decls
+      aPrefix.Truncate();
+      return PR_FALSE;
+    }
+
+    // Attribute not in the null namespace -- must have a prefix
+    mustHavePrefix = PR_TRUE;
+  } else {
+    // Not an attribute, so doesn't _have_ to have a prefix
+    mustHavePrefix = PR_FALSE;
+  }
+
+  // Keep track of the closest prefix that's bound to aURI and whether we've
+  // found such a thing.  closestURIMatch holds the prefix, and uriMatch
+  // indicates whether we actually have one.
   nsAutoString closestURIMatch;
   PRBool uriMatch = PR_FALSE;
 
+  // Also keep track of whether we've seen aPrefix already.  If we have, that
+  // means that it's already bound to a URI different from aURI, so even if we
+  // later (so in a more outer scope) see it bound to aURI we can't reuse it.
+  PRBool haveSeenOurPrefix = PR_FALSE;
+
   PRInt32 count = mNameSpaceStack.Count();
   PRInt32 index = count - 1;
   while (index >= 0) {
     NameSpaceDecl* decl = (NameSpaceDecl*)mNameSpaceStack.ElementAt(index);
     // Check if we've found a prefix match
     if (aPrefix.Equals(decl->mPrefix)) {
-      
-      // If the URI's match, we don't have to add a namespace decl
-      if (aURI.Equals(decl->mURI)) {
-        return PR_FALSE;
+
+      // If the URIs match and aPrefix is not bound to any other URI, we can
+      // use aPrefix
+      if (!haveSeenOurPrefix && aURI.Equals(decl->mURI)) {
+        // Just use our uriMatch stuff.  That will deal with an empty aPrefix
+        // the right way.  We can break out of the loop now, though.
+        uriMatch = PR_TRUE;
+        closestURIMatch = aPrefix;
+        break;
       }
 
+      haveSeenOurPrefix = PR_TRUE;      
+
       // If they don't, and either:
       // 1) We have a prefix (so we'd be redeclaring this prefix to point to a
       //    different namespace) or
@@ -372,14 +412,19 @@ nsXMLContentSerializer::ConfirmPrefix(nsAString& aPrefix,
       // URIs when |decl| doesn't have aElement as its owner.  In that case we
       // can simply push the new namespace URI as the default namespace for
       // aElement.
-      if (!aPrefix.IsEmpty() ||
-          (decl->mPrefix.IsEmpty() && decl->mOwner == aElement)) {
+      if (!aPrefix.IsEmpty() || decl->mOwner == aElement) {
+        NS_ASSERTION(!aURI.IsEmpty(),
+                     "Not allowed to add a xmlns attribute with an empty "
+                     "namespace name unless it declares the default "
+                     "namespace.");
+
         GenerateNewPrefix(aPrefix);
         // Now we need to validate our new prefix/uri combination; check it
         // against the full namespace stack again.  Note that just restarting
         // the while loop is ok, since we haven't changed aURI, so the
-        // closestURIMatch state is not affected.
+        // closestURIMatch and uriMatch state is not affected.
         index = count - 1;
+        haveSeenOurPrefix = PR_FALSE;
         continue;
       }
     }
@@ -406,33 +451,41 @@ nsXMLContentSerializer::ConfirmPrefix(nsAString& aPrefix,
   }
 
   // At this point the following invariants hold:
-  // 1) There is nothing on the namespace stack that matches the pair
-  //    (aPrefix, aURI)
+  // 1) The prefix in closestURIMatch is mapped to aURI in our scope if
+  //    uriMatch is set.
   // 2) There is nothing on the namespace stack that has aPrefix as the prefix
   //    and a _different_ URI, except for the case aPrefix.IsEmpty (and
   //    possible default namespaces on ancestors)
-  // 3) The prefix in closestURIMatch is mapped to aURI in our scope if
-  //    uriMatch is set.
   
   // So if uriMatch is set it's OK to use the closestURIMatch prefix.  The one
   // exception is when closestURIMatch is actually empty (default namespace
   // decl) and we must have a prefix.
-  if (uriMatch && (!aMustHavePrefix || !closestURIMatch.IsEmpty())) {
+  if (uriMatch && (!mustHavePrefix || !closestURIMatch.IsEmpty())) {
     aPrefix.Assign(closestURIMatch);
     return PR_FALSE;
   }
   
-  // At this point, if aPrefix is empty (which means we never had a prefix to
-  // start with) and we must have a prefix, just generate a new prefix and then
-  // send it back through the namespace stack checks to make sure it's OK.
-  if (aPrefix.IsEmpty() && aMustHavePrefix) {
-    GenerateNewPrefix(aPrefix);
-    return ConfirmPrefix(aPrefix, aURI, aElement, aMustHavePrefix);
-  }
-  // else we will just set aURI as the new default namespace URI
+  if (aPrefix.IsEmpty()) {
+    // At this point, aPrefix is empty (which means we never had a prefix to
+    // start with).  If we must have a prefix, just generate a new prefix and
+    // then send it back through the namespace stack checks to make sure it's
+    // OK.
+    if (mustHavePrefix) {
+      GenerateNewPrefix(aPrefix);
+      return ConfirmPrefix(aPrefix, aURI, aElement, aIsAttribute);
+    }
 
-  // Indicate that we need to create a namespace decl for the
-  // final prefix
+    // One final special case.  If aPrefix is empty and we never saw an empty
+    // prefix (default namespace decl) on the namespace stack and we're in the
+    // null namespace there is no reason to output an |xmlns=""| here.  It just
+    // makes the output less readable.
+    if (!haveSeenOurPrefix && aURI.IsEmpty()) {
+      return PR_FALSE;
+    }
+  }
+
+  // Now just set aURI as the new default namespace URI.  Indicate that we need
+  // to create a namespace decl for the final prefix
   return PR_TRUE;
 }
 
@@ -529,7 +582,7 @@ nsXMLContentSerializer::SerializeAttr(const nsAString& aPrefix,
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
-                                           PRBool aHasChildren,
+                                           nsIDOMElement *aOriginalElement,
                                            nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
@@ -545,21 +598,18 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   aElement->GetLocalName(tagLocalName);
   aElement->GetNamespaceURI(tagNamespaceURI);
 
-  PRInt32 namespaceID;
-    
   PRUint32 index, count;
   nsAutoString nameStr, prefixStr, uriStr, valueStr;
-  nsCOMPtr<nsIAtom> attrName, attrPrefix;
 
   count = content->GetAttrCount();
 
   // First scan for namespace declarations, pushing each on the stack
+  PRUint32 skipAttr = count;
   for (index = 0; index < count; index++) {
     
-    content->GetAttrNameAt(index,
-                           &namespaceID,
-                           getter_AddRefs(attrName),
-                           getter_AddRefs(attrPrefix));
+    const nsAttrName* name = content->GetAttrNameAt(index);
+    PRInt32 namespaceID = name->NamespaceID();
+    nsIAtom *attrName = name->LocalName();
     
     if (namespaceID == kNameSpaceID_XMLNS ||
         // Also push on the stack attrs named "xmlns" in the null
@@ -568,15 +618,29 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
         // XXXbz what if we have both "xmlns" in the null namespace and "xmlns"
         // in the xmlns namespace?
         (namespaceID == kNameSpaceID_None &&
-         attrName == nsLayoutAtoms::xmlnsNameSpace)) {
+         attrName == nsLayoutAtoms::xmlns)) {
       content->GetAttr(namespaceID, attrName, uriStr);
 
-      if (!attrPrefix) {
-        // Default NS attribute does not have prefix (and the name is "xmlns")
-        PushNameSpaceDecl(EmptyString(), uriStr, aElement);
-      } else {
+      if (!name->GetPrefix()) {
+        if (tagNamespaceURI.IsEmpty() && !uriStr.IsEmpty()) {
+          // If the element is in no namespace we need to add a xmlns
+          // attribute to declare that. That xmlns attribute must not have a
+          // prefix (see http://www.w3.org/TR/REC-xml-names/#dt-prefix), ie it
+          // must declare the default namespace. We just found an xmlns
+          // attribute that declares the default namespace to something
+          // non-empty. We're going to ignore this attribute, for children we
+          // will detect that we need to add it again and attributes aren't
+          // affected by the default namespace.
+          skipAttr = index;
+        }
+        else {
+          // Default NS attribute does not have prefix (and the name is "xmlns")
+          PushNameSpaceDecl(EmptyString(), uriStr, aOriginalElement);
+        }
+      }
+      else {
         attrName->ToString(nameStr);
-        PushNameSpaceDecl(nameStr, uriStr, aElement);
+        PushNameSpaceDecl(nameStr, uriStr, aOriginalElement);
       }
     }
   }
@@ -585,7 +649,8 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
     
   MaybeAddNewline(aStr);
 
-  addNSAttr = ConfirmPrefix(tagPrefix, tagNamespaceURI, aElement, PR_FALSE);
+  addNSAttr = ConfirmPrefix(tagPrefix, tagNamespaceURI, aOriginalElement,
+                            PR_FALSE);
   // Serialize the qualified name of the element
   AppendToString(NS_LITERAL_STRING("<"), aStr);
   if (!tagPrefix.IsEmpty()) {
@@ -604,17 +669,21 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
       // Serialize namespace decl
       SerializeAttr(xmlnsStr, tagPrefix, tagNamespaceURI, aStr, PR_TRUE);
     }
-    PushNameSpaceDecl(tagPrefix, tagNamespaceURI, aElement);
+    PushNameSpaceDecl(tagPrefix, tagNamespaceURI, aOriginalElement);
   }
 
   // Now serialize each of the attributes
   // XXX Unfortunately we need a namespace manager to get
   // attribute URIs.
   for (index = 0; index < count; index++) {
-    content->GetAttrNameAt(index,
-                           &namespaceID,
-                           getter_AddRefs(attrName),
-                           getter_AddRefs(attrPrefix));
+    if (skipAttr == index) {
+        continue;
+    }
+
+    const nsAttrName* name = content->GetAttrNameAt(index);
+    PRInt32 namespaceID = name->NamespaceID();
+    nsIAtom* attrName = name->LocalName();
+    nsIAtom* attrPrefix = name->GetPrefix();
 
     if (attrPrefix) {
       attrPrefix->ToString(prefixStr);
@@ -625,10 +694,8 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
 
     addNSAttr = PR_FALSE;
     if (kNameSpaceID_XMLNS != namespaceID) {
-      nsContentUtils::GetNSManagerWeakRef()->GetNameSpaceURI(namespaceID,
-                                                             uriStr);
-      addNSAttr = ConfirmPrefix(prefixStr, uriStr, aElement,
-                                namespaceID != kNameSpaceID_None);
+      nsContentUtils::NameSpaceManager()->GetNameSpaceURI(namespaceID, uriStr);
+      addNSAttr = ConfirmPrefix(prefixStr, uriStr, aOriginalElement, PR_TRUE);
     }
     
     content->GetAttr(namespaceID, attrName, valueStr);
@@ -641,9 +708,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
       continue;
 
     if (namespaceID == kNameSpaceID_None) {
-      PRInt32 elementNsID;
-      content->GetNameSpaceID(&elementNsID);
-      if (elementNsID == kNameSpaceID_XHTML) {
+      if (content->GetNameSpaceID() == kNameSpaceID_XHTML) {
         if (IsShorthandAttr(attrName, content->Tag()) &&
             valueStr.IsEmpty()) {
           valueStr = nameStr;
@@ -656,12 +721,14 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
       NS_ASSERTION(!prefixStr.IsEmpty(),
                    "Namespaced attributes must have a prefix");
       SerializeAttr(xmlnsStr, prefixStr, uriStr, aStr, PR_TRUE);
-      PushNameSpaceDecl(prefixStr, uriStr, aElement);
+      PushNameSpaceDecl(prefixStr, uriStr, aOriginalElement);
     }
   }
 
   // We don't output a separate end tag for empty element
-  if (!aHasChildren) {
+  PRBool hasChildren;
+  if (NS_FAILED(aOriginalElement->HasChildNodes(&hasChildren)) ||
+      !hasChildren) {
     AppendToString(NS_LITERAL_STRING("/>"), aStr);    
     MaybeFlagNewline(aElement);
   } else {
