@@ -53,20 +53,29 @@
 #include "nsMutationEvent.h"
 #include "nsContentUtils.h"
 #include "nsIURI.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsIScriptError.h"
 
 static const char* const sEventNames[] = {
   "mousedown", "mouseup", "click", "dblclick", "mouseover",
   "mouseout", "mousemove", "contextmenu", "keydown", "keyup", "keypress",
   "focus", "blur", "load", "beforeunload", "unload", "abort", "error",
   "submit", "reset", "change", "select", "input", "paint" ,"text",
-  "popupshowing", "popupshown", "popuphiding", "popuphidden", "close", "command", "broadcast", "commandupdate",
+  "compositionstart", "compositionend", "popupshowing", "popupshown",
+  "popuphiding", "popuphidden", "close", "command", "broadcast", "commandupdate",
   "dragenter", "dragover", "dragexit", "dragdrop", "draggesture", "resize",
   "scroll","overflow", "underflow", "overflowchanged",
   "DOMSubtreeModified", "DOMNodeInserted", "DOMNodeRemoved", 
   "DOMNodeRemovedFromDocument", "DOMNodeInsertedIntoDocument",
   "DOMAttrModified", "DOMCharacterDataModified",
-  "popupBlocked", "DOMActivate", "DOMFocusIn", "DOMFocusOut"
-}; 
+  "DOMActivate", "DOMFocusIn", "DOMFocusOut",
+  "pageshow", "pagehide"
+#ifdef MOZ_SVG
+ ,
+  "SVGLoad", "SVGUnload", "SVGAbort", "SVGError", "SVGResize", "SVGScroll",
+  "SVGZoom"
+#endif // MOZ_SVG
+};
 
 static char *sPopupAllowedEvents;
 
@@ -105,7 +114,7 @@ nsDOMEvent::nsDOMEvent(nsPresContext* aPresContext, nsEvent* aEvent)
           ...
         }
      */
-    mEvent = new nsEvent();
+    mEvent = new nsEvent(PR_FALSE, 0);
     mEvent->time = PR_Now();
   }
 
@@ -115,10 +124,7 @@ nsDOMEvent::nsDOMEvent(nsPresContext* aPresContext, nsEvent* aEvent)
     mTmpRealOriginalTarget = mExplicitOriginalTarget;
     nsCOMPtr<nsIContent> content = do_QueryInterface(mExplicitOriginalTarget);
     if (content) {
-      if (content->IsNativeAnonymous()) {
-        mExplicitOriginalTarget = nsnull;
-      }
-      if (content->GetBindingParent()) {
+      if (content->IsNativeAnonymous() || content->GetBindingParent()) {
         mExplicitOriginalTarget = nsnull;
       }
     }
@@ -129,7 +135,7 @@ nsDOMEvent::~nsDOMEvent()
 {
   NS_ASSERT_OWNINGTHREAD(nsDOMEvent);
 
-  if (mEventIsInternal) {
+  if (mEventIsInternal && mEvent) {
     delete mEvent->userType;
     delete mEvent;
   }
@@ -167,51 +173,20 @@ NS_METHOD nsDOMEvent::GetType(nsAString& aType)
 
 NS_METHOD nsDOMEvent::GetTarget(nsIDOMEventTarget** aTarget)
 {
-  if (nsnull != mTarget) {
-    *aTarget = mTarget;
-    NS_ADDREF(*aTarget);
-    return NS_OK;
+  if (mEvent->target) {
+    return CallQueryInterface(mEvent->target, aTarget);
   }
-  
   *aTarget = nsnull;
-
-  nsCOMPtr<nsIContent> targetContent;  
-
-  if (mPresContext) {
-    mPresContext->EventStateManager()->
-      GetEventTargetContent(mEvent, getter_AddRefs(targetContent));
-  }
-  
-  if (targetContent) {
-    mTarget = do_QueryInterface(targetContent);
-    if (mTarget) {
-      *aTarget = mTarget;
-      NS_ADDREF(*aTarget);
-    }
-  }
-  else {
-    //Always want a target.  Use document if nothing else.
-    nsIPresShell *presShell;
-    if (mPresContext && (presShell = mPresContext->GetPresShell())) {
-      nsIDocument *doc = presShell->GetDocument();
-      if (doc) {
-        mTarget = do_QueryInterface(doc);
-        if (mTarget) {
-          *aTarget = mTarget;
-          NS_ADDREF(*aTarget);
-        }      
-      }
-    }
-  }
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsDOMEvent::GetCurrentTarget(nsIDOMEventTarget** aCurrentTarget)
 {
-  *aCurrentTarget = mCurrentTarget;
-  NS_IF_ADDREF(*aCurrentTarget);
+  if (mEvent->currentTarget) {
+    return CallQueryInterface(mEvent->currentTarget, aCurrentTarget);
+  }
+  *aCurrentTarget = nsnull;
   return NS_OK;
 }
 
@@ -266,18 +241,17 @@ nsDOMEvent::GetTmpRealOriginalTarget(nsIDOMEventTarget** aRealEventTarget)
 NS_IMETHODIMP
 nsDOMEvent::GetOriginalTarget(nsIDOMEventTarget** aOriginalTarget)
 {
-  if (!mOriginalTarget)
-    return GetTarget(aOriginalTarget);
+  if (mEvent->originalTarget) {
+    return CallQueryInterface(mEvent->originalTarget, aOriginalTarget);
+  }
 
-  *aOriginalTarget = mOriginalTarget;
-  NS_IF_ADDREF(*aOriginalTarget);
-  return NS_OK;
+  return GetTarget(aOriginalTarget);
 }
 
 NS_IMETHODIMP
 nsDOMEvent::HasOriginalTarget(PRBool* aResult)
 {
-  *aResult = (mOriginalTarget != nsnull);
+  *aResult = !!(mEvent->originalTarget);
   return NS_OK;
 }
 
@@ -285,9 +259,9 @@ NS_IMETHODIMP
 nsDOMEvent::SetTrusted(PRBool aTrusted)
 {
   if (aTrusted) {
-    mEvent->internalAppFlags |= NS_APP_EVENT_FLAG_TRUSTED;
+    mEvent->flags |= NS_EVENT_FLAG_TRUSTED;
   } else {
-    mEvent->internalAppFlags &= ~NS_APP_EVENT_FLAG_TRUSTED;
+    mEvent->flags &= ~NS_EVENT_FLAG_TRUSTED;
   }
 
   return NS_OK;
@@ -296,21 +270,19 @@ nsDOMEvent::SetTrusted(PRBool aTrusted)
 NS_IMETHODIMP
 nsDOMEvent::GetEventPhase(PRUint16* aEventPhase)
 {
-  if (mEvent->flags & NS_EVENT_FLAG_CAPTURE) {
-    if (mEvent->flags & NS_EVENT_FLAG_BUBBLE) {
-      *aEventPhase = nsIDOMEvent::AT_TARGET;
-    }
-    else {
-      *aEventPhase = nsIDOMEvent::CAPTURING_PHASE;
-    }
-  }
-  else if (mEvent->flags & NS_EVENT_FLAG_BUBBLE) {
+  // Note, remember to check that this works also
+  // if or when Bug 235441 is fixed.
+  if (mEvent->currentTarget == mEvent->target ||
+      ((mEvent->flags & NS_EVENT_FLAG_CAPTURE) &&
+       (mEvent->flags & NS_EVENT_FLAG_BUBBLE))) {
+    *aEventPhase = nsIDOMEvent::AT_TARGET;
+  } else if (mEvent->flags & NS_EVENT_FLAG_CAPTURE) {
+    *aEventPhase = nsIDOMEvent::CAPTURING_PHASE;
+  } else if (mEvent->flags & NS_EVENT_FLAG_BUBBLE) {
     *aEventPhase = nsIDOMEvent::BUBBLING_PHASE;
-  }
-  else {
+  } else {
     *aEventPhase = 0;
   }
-
   return NS_OK;
 }
 
@@ -342,28 +314,51 @@ nsDOMEvent::StopPropagation()
   return NS_OK;
 }
 
+static void
+ReportUseOfDeprecatedMethod(nsEvent* aEvent, nsIDOMEvent* aDOMEvent,
+                            const char* aWarning)
+{
+  nsCOMPtr<nsIDocument> doc;
+  nsCOMPtr<nsINode> node = do_QueryInterface(aEvent->currentTarget);
+  if (node) {
+    doc = node->GetOwnerDoc();
+  } else {
+    nsCOMPtr<nsPIDOMWindow> window = do_QueryInterface(aEvent->currentTarget);
+    if (window) {
+      doc = do_QueryInterface(window->GetExtantDocument());
+    }
+  }
+
+  nsAutoString type;
+  aDOMEvent->GetType(type);
+  const PRUnichar *strings[] = { type.get() };
+  nsContentUtils::ReportToConsole(nsContentUtils::eDOM_PROPERTIES,
+                                  aWarning,
+                                  strings, NS_ARRAY_LENGTH(strings),
+                                  doc ? doc->GetDocumentURI() : nsnull,
+                                  EmptyString(), 0, 0,
+                                  nsIScriptError::warningFlag,
+                                  "DOM Events");
+}
+
 NS_IMETHODIMP
 nsDOMEvent::PreventBubble()
 {
-  if (mEvent->flags & NS_EVENT_FLAG_BUBBLE || mEvent->flags & NS_EVENT_FLAG_INIT) {
-    mEvent->flags |= NS_EVENT_FLAG_STOP_DISPATCH;
-  }
+  ReportUseOfDeprecatedMethod(mEvent, this, "UseOfPreventBubbleWarning");
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsDOMEvent::PreventCapture()
 {
-  if (mEvent->flags & NS_EVENT_FLAG_CAPTURE) {
-    mEvent->flags |= NS_EVENT_FLAG_STOP_DISPATCH;
-  }
+  ReportUseOfDeprecatedMethod(mEvent, this, "UseOfPreventCaptureWarning");
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsDOMEvent::GetIsTrusted(PRBool *aIsTrusted)
 {
-  *aIsTrusted = (mEvent->internalAppFlags & NS_APP_EVENT_FLAG_TRUSTED) != 0;
+  *aIsTrusted = NS_IS_TRUSTED_EVENT(mEvent);
 
   return NS_OK;
 }
@@ -380,7 +375,7 @@ nsDOMEvent::PreventDefault()
 nsresult
 nsDOMEvent::SetEventType(const nsAString& aEventTypeArg)
 {
-  nsCOMPtr<nsIAtom> atom= do_GetAtom(NS_LITERAL_STRING("on") + aEventTypeArg);
+  nsCOMPtr<nsIAtom> atom = do_GetAtom(NS_LITERAL_STRING("on") + aEventTypeArg);
   mEvent->message = NS_USER_DEFINED_EVENT;
 
   if (mEvent->eventStructType == NS_MOUSE_EVENT) {
@@ -407,6 +402,11 @@ nsDOMEvent::SetEventType(const nsAString& aEventTypeArg)
       mEvent->message = NS_KEY_UP;
     else if (atom == nsLayoutAtoms::onkeypress)
       mEvent->message = NS_KEY_PRESS;
+  } else if (mEvent->eventStructType == NS_COMPOSITION_EVENT) {
+    if (atom == nsLayoutAtoms::oncompositionstart)
+      mEvent->message = NS_COMPOSITION_START;
+    else if (atom == nsLayoutAtoms::oncompositionend)
+      mEvent->message = NS_COMPOSITION_END;
   } else if (mEvent->eventStructType == NS_EVENT) {
     if (atom == nsLayoutAtoms::onfocus)
       mEvent->message = NS_FOCUS_CONTENT;
@@ -421,13 +421,13 @@ nsDOMEvent::SetEventType(const nsAString& aEventTypeArg)
     else if (atom == nsLayoutAtoms::onselect)
       mEvent->message = NS_FORM_SELECTED;
     else if (atom == nsLayoutAtoms::onload)
-      mEvent->message = NS_PAGE_LOAD;
+      mEvent->message = NS_LOAD;
     else if (atom == nsLayoutAtoms::onunload)
       mEvent->message = NS_PAGE_UNLOAD;
     else if (atom == nsLayoutAtoms::onabort)
       mEvent->message = NS_IMAGE_ABORT;
     else if (atom == nsLayoutAtoms::onerror)
-      mEvent->message = NS_IMAGE_ERROR;
+      mEvent->message = NS_LOAD_ERROR;
   } else if (mEvent->eventStructType == NS_MUTATION_EVENT) {
     if (atom == nsLayoutAtoms::onDOMAttrModified)
       mEvent->message = NS_MUTATION_ATTRMODIFIED;
@@ -450,7 +450,33 @@ nsDOMEvent::SetEventType(const nsAString& aEventTypeArg)
       mEvent->message = NS_UI_FOCUSIN;
     else if (atom == nsLayoutAtoms::onDOMFocusOut)
       mEvent->message = NS_UI_FOCUSOUT;
+    else if (atom == nsLayoutAtoms::oninput)
+      mEvent->message = NS_FORM_INPUT;
+  } else if (mEvent->eventStructType == NS_PAGETRANSITION_EVENT) {
+    if (atom == nsLayoutAtoms::onpageshow)
+      mEvent->message = NS_PAGE_SHOW;
+    else if (atom == nsLayoutAtoms::onpagehide)
+      mEvent->message = NS_PAGE_HIDE;
   }
+#ifdef MOZ_SVG
+  else if (mEvent->eventStructType == NS_SVG_EVENT) {
+    if (atom == nsLayoutAtoms::onSVGLoad)
+      mEvent->message = NS_SVG_LOAD;
+    else if (atom == nsLayoutAtoms::onSVGUnload)
+      mEvent->message = NS_SVG_UNLOAD;
+    else if (atom == nsLayoutAtoms::onSVGAbort)
+      mEvent->message = NS_SVG_ABORT;
+    else if (atom == nsLayoutAtoms::onSVGError)
+      mEvent->message = NS_SVG_ERROR;
+    else if (atom == nsLayoutAtoms::onSVGResize)
+      mEvent->message = NS_SVG_RESIZE;
+    else if (atom == nsLayoutAtoms::onSVGScroll)
+      mEvent->message = NS_SVG_SCROLL;
+  } else if (mEvent->eventStructType == NS_SVGZOOM_EVENT) {
+    if (atom == nsLayoutAtoms::onSVGZoom)
+      mEvent->message = NS_SVG_ZOOM;
+  }
+#endif // MOZ_SVG
 
   if (mEvent->message == NS_USER_DEFINED_EVENT)
     mEvent->userType = new nsStringKey(aEventTypeArg);
@@ -461,35 +487,385 @@ nsDOMEvent::SetEventType(const nsAString& aEventTypeArg)
 NS_IMETHODIMP
 nsDOMEvent::InitEvent(const nsAString& aEventTypeArg, PRBool aCanBubbleArg, PRBool aCancelableArg)
 {
+  // Make sure this event isn't already being dispatched.
+  NS_ENSURE_TRUE(!NS_IS_EVENT_IN_DISPATCH(mEvent), NS_ERROR_INVALID_ARG);
+
+  if (NS_IS_TRUSTED_EVENT(mEvent)) {
+    // Ensure the caller is permitted to dispatch trusted DOM events.
+
+    PRBool enabled = PR_FALSE;
+    nsContentUtils::GetSecurityManager()->
+      IsCapabilityEnabled("UniversalBrowserWrite", &enabled);
+
+    if (!enabled) {
+      SetTrusted(PR_FALSE);
+    }
+  }
+
   NS_ENSURE_SUCCESS(SetEventType(aEventTypeArg), NS_ERROR_FAILURE);
-  mEvent->flags |= aCanBubbleArg ? NS_EVENT_FLAG_NONE : NS_EVENT_FLAG_CANT_BUBBLE;
-  mEvent->flags |= aCancelableArg ? NS_EVENT_FLAG_NONE : NS_EVENT_FLAG_CANT_CANCEL;
+
+  if (aCanBubbleArg) {
+    mEvent->flags &= ~NS_EVENT_FLAG_CANT_BUBBLE;
+  } else {
+    mEvent->flags |= NS_EVENT_FLAG_CANT_BUBBLE;
+  }
+
+  if (aCancelableArg) {
+    mEvent->flags &= ~NS_EVENT_FLAG_CANT_CANCEL;
+  } else {
+    mEvent->flags |= NS_EVENT_FLAG_CANT_CANCEL;
+  }
+
+  // Clearing the old targets, so that the event is targeted correctly when
+  // re-dispatching it.
+  mEvent->target = nsnull;
+  mEvent->originalTarget = nsnull;
+
+  // Unset the NS_EVENT_FLAG_STOP_DISPATCH_IMMEDIATELY bit (which is
+  // set at the end of event dispatch) so that this event can be
+  // dispatched.
+  mEvent->flags &= ~NS_EVENT_FLAG_STOP_DISPATCH_IMMEDIATELY;
+
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::DuplicatePrivateData()
 {
-  //XXX Write me!
+  // FIXME! Simplify this method and make it somehow easily extendable,
+  //        Bug 329127
+  
+  NS_ASSERTION(mEvent, "No nsEvent for nsDOMEvent duplication!");
+  if (mEventIsInternal) {
+    return NS_OK;
+  }
+
+  nsEvent* newEvent = nsnull;
+  PRUint32 msg = mEvent->message;
+  PRBool isInputEvent = PR_FALSE;
+
+  switch (mEvent->eventStructType) {
+    case NS_EVENT:
+    {
+      newEvent = new nsEvent(PR_FALSE, msg);
+      break;
+    }
+    case NS_GUI_EVENT:
+    {
+      // Not copying widget, it is a weak reference.
+      newEvent = new nsGUIEvent(PR_FALSE, msg, nsnull);
+      break;
+    }
+    case NS_SIZE_EVENT:
+    {
+      nsSizeEvent* sizeEvent = new nsSizeEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(sizeEvent, NS_ERROR_OUT_OF_MEMORY);
+      sizeEvent->mWinWidth = NS_STATIC_CAST(nsSizeEvent*, mEvent)->mWinWidth;
+      sizeEvent->mWinHeight = NS_STATIC_CAST(nsSizeEvent*, mEvent)->mWinHeight;
+      newEvent = sizeEvent;
+      break;
+    }
+    case NS_SIZEMODE_EVENT:
+    {
+      newEvent = new nsSizeModeEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      NS_STATIC_CAST(nsSizeModeEvent*, newEvent)->mSizeMode =
+        NS_STATIC_CAST(nsSizeModeEvent*, mEvent)->mSizeMode;
+      break;
+    }
+    case NS_ZLEVEL_EVENT:
+    {
+      nsZLevelEvent* zLevelEvent = new nsZLevelEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(zLevelEvent, NS_ERROR_OUT_OF_MEMORY);
+      nsZLevelEvent* oldZLevelEvent = NS_STATIC_CAST(nsZLevelEvent*, mEvent);
+      zLevelEvent->mPlacement = oldZLevelEvent->mPlacement;
+      zLevelEvent->mImmediate = oldZLevelEvent->mImmediate;
+      zLevelEvent->mAdjusted = oldZLevelEvent->mAdjusted;
+      newEvent = zLevelEvent;
+      break;
+    }
+    case NS_PAINT_EVENT:
+    {
+      newEvent = new nsPaintEvent(PR_FALSE, msg, nsnull);
+      break;
+    }
+    case NS_SCROLLBAR_EVENT:
+    {
+      newEvent = new nsScrollbarEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      NS_STATIC_CAST(nsScrollbarEvent*, newEvent)->position =
+        NS_STATIC_CAST(nsScrollbarEvent*, mEvent)->position;
+      break;
+    }
+    case NS_INPUT_EVENT:
+    {
+      newEvent = new nsInputEvent(PR_FALSE, msg, nsnull);
+      isInputEvent = PR_TRUE;
+      break;
+    }
+    case NS_KEY_EVENT:
+    {
+      nsKeyEvent* keyEvent = new nsKeyEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(keyEvent, NS_ERROR_OUT_OF_MEMORY);
+      nsKeyEvent* oldKeyEvent = NS_STATIC_CAST(nsKeyEvent*, mEvent);
+      isInputEvent = PR_TRUE;
+      keyEvent->keyCode = oldKeyEvent->keyCode;
+      keyEvent->charCode = oldKeyEvent->charCode;
+      keyEvent->isChar = oldKeyEvent->isChar;
+      newEvent = keyEvent;
+      break;
+    }
+    case NS_MOUSE_EVENT:
+    {
+      nsMouseEvent* oldMouseEvent = NS_STATIC_CAST(nsMouseEvent*, mEvent);
+      nsMouseEvent* mouseEvent =
+        new nsMouseEvent(PR_FALSE, msg, nsnull, oldMouseEvent->reason);
+      NS_ENSURE_TRUE(mouseEvent, NS_ERROR_OUT_OF_MEMORY);
+      isInputEvent = PR_TRUE;
+      mouseEvent->clickCount = oldMouseEvent->clickCount;
+      mouseEvent->acceptActivation = oldMouseEvent->acceptActivation;
+      mouseEvent->relatedTarget = oldMouseEvent->relatedTarget;
+      newEvent = mouseEvent;
+      break;
+    }
+    case NS_MENU_EVENT:
+    {
+      newEvent = new nsMenuEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      NS_STATIC_CAST(nsMenuEvent*, newEvent)->mCommand =
+        NS_STATIC_CAST(nsMenuEvent*, mEvent)->mCommand;
+      break;
+    }
+    case NS_SCRIPT_ERROR_EVENT:
+    {
+      newEvent = new nsScriptErrorEvent(PR_FALSE, msg);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      NS_STATIC_CAST(nsScriptErrorEvent*, newEvent)->lineNr =
+        NS_STATIC_CAST(nsScriptErrorEvent*, mEvent)->lineNr;
+      break;
+    }
+    case NS_TEXT_EVENT:
+    {
+      newEvent = new nsTextEvent(PR_FALSE, msg, nsnull);
+      isInputEvent = PR_TRUE;
+      break;
+    }
+    case NS_COMPOSITION_EVENT:
+    {
+      newEvent = new nsCompositionEvent(PR_FALSE, msg, nsnull);
+      isInputEvent = PR_TRUE;
+      break;
+    }
+    case NS_RECONVERSION_EVENT:
+    {
+      newEvent = new nsReconversionEvent(PR_FALSE, msg, nsnull);
+      isInputEvent = PR_TRUE;
+      break;
+    }
+    case NS_MOUSE_SCROLL_EVENT:
+    {
+      nsMouseScrollEvent* mouseScrollEvent =
+        new nsMouseScrollEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(mouseScrollEvent, NS_ERROR_OUT_OF_MEMORY);
+      isInputEvent = PR_TRUE;
+      nsMouseScrollEvent* oldMouseScrollEvent =
+        NS_STATIC_CAST(nsMouseScrollEvent*, mEvent);
+      mouseScrollEvent->scrollFlags = oldMouseScrollEvent->scrollFlags;
+      mouseScrollEvent->delta = oldMouseScrollEvent->delta;
+      mouseScrollEvent->relatedTarget = oldMouseScrollEvent->relatedTarget;
+      newEvent = mouseScrollEvent;
+      break;
+    }
+    case NS_SCROLLPORT_EVENT:
+    {
+      newEvent = new nsScrollPortEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      NS_STATIC_CAST(nsScrollPortEvent*, newEvent)->orient =
+        NS_STATIC_CAST(nsScrollPortEvent*, mEvent)->orient;
+      break;
+    }
+    case NS_MUTATION_EVENT:
+    {
+      nsMutationEvent* mutationEvent = new nsMutationEvent(PR_FALSE, msg);
+      NS_ENSURE_TRUE(mutationEvent, NS_ERROR_OUT_OF_MEMORY);
+      nsMutationEvent* oldMutationEvent =
+        NS_STATIC_CAST(nsMutationEvent*, mEvent);
+      mutationEvent->mRelatedNode = oldMutationEvent->mRelatedNode;
+      mutationEvent->mAttrName = oldMutationEvent->mAttrName;
+      mutationEvent->mPrevAttrValue = oldMutationEvent->mPrevAttrValue;
+      mutationEvent->mNewAttrValue = oldMutationEvent->mNewAttrValue;
+      mutationEvent->mAttrChange = oldMutationEvent->mAttrChange;
+      newEvent = mutationEvent;
+      break;
+    }
+    case NS_ACCESSIBLE_EVENT:
+    {
+      newEvent = new nsAccessibleEvent(PR_FALSE, msg, nsnull);
+      isInputEvent = PR_TRUE;
+      break;
+    }
+    case NS_FORM_EVENT:
+    {
+      newEvent = new nsFormEvent(PR_FALSE, msg);
+      break;
+    }
+    case NS_FOCUS_EVENT:
+    {
+      newEvent = new nsFocusEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      NS_STATIC_CAST(nsFocusEvent*, newEvent)->isMozWindowTakingFocus =
+        NS_STATIC_CAST(nsFocusEvent*, mEvent)->isMozWindowTakingFocus;
+      break;
+    }
+    case NS_POPUP_EVENT:
+    {
+      newEvent = new nsInputEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      isInputEvent = PR_TRUE;
+      newEvent->eventStructType = NS_POPUP_EVENT;
+      break;
+    }
+    case NS_APPCOMMAND_EVENT:
+    {
+      newEvent = new nsAppCommandEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      isInputEvent = PR_TRUE;
+      NS_STATIC_CAST(nsAppCommandEvent*, newEvent)->appCommand =
+        NS_STATIC_CAST(nsAppCommandEvent*, mEvent)->appCommand;
+      break;
+    }
+    case NS_POPUPBLOCKED_EVENT:
+    {
+      NS_WARNING("nsPopupBlockedEvent should never be an external event!");
+      newEvent = new nsPopupBlockedEvent(PR_FALSE, msg);
+      break;
+    }
+    case NS_BEFORE_PAGE_UNLOAD_EVENT:
+    {
+      newEvent = new nsBeforePageUnloadEvent(PR_FALSE, msg);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      NS_STATIC_CAST(nsBeforePageUnloadEvent*, newEvent)->text =
+        NS_STATIC_CAST(nsBeforePageUnloadEvent*, mEvent)->text;
+      break;
+    }
+    case NS_UI_EVENT:
+    {
+      newEvent = new nsUIEvent(PR_FALSE, msg,
+                               NS_STATIC_CAST(nsUIEvent*, mEvent)->detail);
+      break;
+    }
+    case NS_QUERYCARETRECT_EVENT:
+    {
+      newEvent = new nsQueryCaretRectEvent(PR_FALSE, msg, nsnull);
+      isInputEvent = PR_TRUE;
+      break;
+    }
+    case NS_PAGETRANSITION_EVENT:
+    {
+      newEvent =
+        new nsPageTransitionEvent(PR_FALSE, msg,
+                                  ((nsPageTransitionEvent*) mEvent)->persisted);
+      break;
+    }
+#ifdef MOZ_SVG
+    case NS_SVG_EVENT:
+    {
+      newEvent = new nsEvent(PR_FALSE, msg);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      newEvent->eventStructType = NS_SVG_EVENT;
+      break;
+    }
+    case NS_SVGZOOM_EVENT:
+    {
+      newEvent = new nsGUIEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      newEvent->eventStructType = NS_SVGZOOM_EVENT;
+      break;
+    }
+#endif // MOZ_SVG
+    case NS_XUL_COMMAND_EVENT:
+    {
+      newEvent = new nsXULCommandEvent(PR_FALSE, msg, nsnull);
+      NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+      isInputEvent = PR_TRUE;
+      newEvent->eventStructType = NS_XUL_COMMAND_EVENT;
+       NS_STATIC_CAST(nsXULCommandEvent*, newEvent)->sourceEvent =
+         NS_STATIC_CAST(nsXULCommandEvent*, mEvent)->sourceEvent;
+      break;
+    }
+    default:
+    {
+      NS_WARNING("Unknown event type!!!");
+      return NS_ERROR_FAILURE;
+    }
+  }
+
+  NS_ENSURE_TRUE(newEvent, NS_ERROR_OUT_OF_MEMORY);
+
+  if (isInputEvent) {
+    nsInputEvent* oldInputEvent = NS_STATIC_CAST(nsInputEvent*, mEvent);
+    nsInputEvent* newInputEvent = NS_STATIC_CAST(nsInputEvent*, newEvent);
+    newInputEvent->isShift = oldInputEvent->isShift;
+    newInputEvent->isControl = oldInputEvent->isControl;
+    newInputEvent->isAlt = oldInputEvent->isAlt;
+    newInputEvent->isMeta = oldInputEvent->isMeta;
+  }
+
+  newEvent->target                 = mEvent->target;
+  newEvent->currentTarget          = mEvent->currentTarget;
+  newEvent->originalTarget         = mEvent->originalTarget;
+  newEvent->flags                  = mEvent->flags;
+  newEvent->time                   = mEvent->time;
+  newEvent->refPoint               = mEvent->refPoint;
+
+  mEvent = newEvent;
+  mPresContext = nsnull;
+  mEventIsInternal = PR_TRUE;
 
-  //XXX And when you do, make sure to copy over the event target here, too!
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::SetTarget(nsIDOMEventTarget* aTarget)
 {
-  mTarget = aTarget;
+#ifdef DEBUG
+  {
+    nsCOMPtr<nsPIDOMWindow> win = do_QueryInterface(aTarget);
+
+    NS_ASSERTION(!win || !win->IsInnerWindow(),
+                 "Uh, inner window set as event target!");
+  }
+#endif
+
+  mEvent->target = aTarget;
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::SetCurrentTarget(nsIDOMEventTarget* aCurrentTarget)
 {
-  mCurrentTarget = aCurrentTarget;
+#ifdef DEBUG
+  {
+    nsCOMPtr<nsPIDOMWindow> win = do_QueryInterface(aCurrentTarget);
+
+    NS_ASSERTION(!win || !win->IsInnerWindow(),
+                 "Uh, inner window set as event target!");
+  }
+#endif
+
+  mEvent->currentTarget = aCurrentTarget;
   return NS_OK;
 }
 
 NS_METHOD nsDOMEvent::SetOriginalTarget(nsIDOMEventTarget* aOriginalTarget)
 {
-  mOriginalTarget = aOriginalTarget;
+#ifdef DEBUG
+  {
+    nsCOMPtr<nsPIDOMWindow> win = do_QueryInterface(aOriginalTarget);
+
+    NS_ASSERTION(!win || !win->IsInnerWindow(),
+                 "Uh, inner window set as event target!");
+  }
+#endif
+
+  mEvent->originalTarget = aOriginalTarget;
   return NS_OK;
 }
 
@@ -504,28 +880,6 @@ nsDOMEvent::IsDispatchStopped(PRBool* aIsDispatchStopped)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsDOMEvent::IsHandled(PRBool* aIsHandled)
-{
-  if (mEvent->internalAppFlags | NS_APP_EVENT_FLAG_HANDLED) {
-    *aIsHandled = PR_TRUE;
-  } else {
-    *aIsHandled = PR_FALSE;
-  }
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsDOMEvent::SetHandled(PRBool aHandled)
-{
-  if(aHandled)
-    mEvent->internalAppFlags |= NS_APP_EVENT_FLAG_HANDLED;
-  else
-    mEvent->internalAppFlags &= ~NS_APP_EVENT_FLAG_HANDLED;
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsDOMEvent::GetInternalNSEvent(nsEvent** aNSEvent)
 {
@@ -592,9 +946,11 @@ nsDOMEvent::GetEventPopupControlState(nsEvent *aEvent)
       case NS_FORM_SELECTED :
         if (::PopupAllowedForEvent("select"))
           abuse = openControlled;
+        break;
       case NS_FORM_CHANGE :
         if (::PopupAllowedForEvent("change"))
           abuse = openControlled;
+        break;
       }
     }
     break;
@@ -625,7 +981,7 @@ nsDOMEvent::GetEventPopupControlState(nsEvent *aEvent)
     }
     break;
   case NS_KEY_EVENT :
-    if (aEvent->internalAppFlags & NS_APP_EVENT_FLAG_TRUSTED) {
+    if (NS_IS_TRUSTED_EVENT(aEvent)) {
       PRUint32 key = NS_STATIC_CAST(nsKeyEvent *, aEvent)->keyCode;
       switch(aEvent->message) {
       case NS_KEY_PRESS :
@@ -650,7 +1006,7 @@ nsDOMEvent::GetEventPopupControlState(nsEvent *aEvent)
     }
     break;
   case NS_MOUSE_EVENT :
-    if (aEvent->internalAppFlags & NS_APP_EVENT_FLAG_TRUSTED) {
+    if (NS_IS_TRUSTED_EVENT(aEvent)) {
       switch(aEvent->message) {
       case NS_MOUSE_LEFT_BUTTON_UP :
         if (::PopupAllowedForEvent("mouseup"))
@@ -677,7 +1033,7 @@ nsDOMEvent::GetEventPopupControlState(nsEvent *aEvent)
     break;
   case NS_SCRIPT_ERROR_EVENT :
     switch(aEvent->message) {
-    case NS_SCRIPT_ERROR :
+    case NS_LOAD_ERROR :
       // Any error event will allow popups, if enabled in the pref.
       if (::PopupAllowedForEvent("error"))
         abuse = openControlled;
@@ -731,6 +1087,8 @@ nsDOMEvent::Shutdown()
   }
 }
 
+// To be called ONLY by nsDOMEvent::GetType (which has the additional
+// logic for handling user-defined events).
 // static
 const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
 {
@@ -763,15 +1121,17 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_keydown];
   case NS_KEY_PRESS:
     return sEventNames[eDOMEvents_keypress];
+  case NS_COMPOSITION_START:
+    return sEventNames[eDOMEvents_compositionstart];
+  case NS_COMPOSITION_END:
+    return sEventNames[eDOMEvents_compositionend];
   case NS_FOCUS_CONTENT:
     return sEventNames[eDOMEvents_focus];
   case NS_BLUR_CONTENT:
     return sEventNames[eDOMEvents_blur];
   case NS_XUL_CLOSE:
     return sEventNames[eDOMEvents_close];
-  case NS_PAGE_LOAD:
-  case NS_IMAGE_LOAD:
-  case NS_SCRIPT_LOAD:
+  case NS_LOAD:
     return sEventNames[eDOMEvents_load];
   case NS_BEFORE_PAGE_UNLOAD:
     return sEventNames[eDOMEvents_beforeunload];
@@ -779,8 +1139,7 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_unload];
   case NS_IMAGE_ABORT:
     return sEventNames[eDOMEvents_abort];
-  case NS_IMAGE_ERROR:
-  case NS_SCRIPT_ERROR:
+  case NS_LOAD_ERROR:
     return sEventNames[eDOMEvents_error];
   case NS_FORM_SUBMIT:
     return sEventNames[eDOMEvents_submit];
@@ -853,9 +1212,34 @@ const char* nsDOMEvent::GetEventName(PRUint32 aEventType)
     return sEventNames[eDOMEvents_DOMFocusIn];
   case NS_UI_FOCUSOUT:
     return sEventNames[eDOMEvents_DOMFocusOut];
+  case NS_PAGE_SHOW:
+    return sEventNames[eDOMEvents_pageshow];
+  case NS_PAGE_HIDE:
+    return sEventNames[eDOMEvents_pagehide];
+#ifdef MOZ_SVG
+  case NS_SVG_LOAD:
+    return sEventNames[eDOMEvents_SVGLoad];
+  case NS_SVG_UNLOAD:
+    return sEventNames[eDOMEvents_SVGUnload];
+  case NS_SVG_ABORT:
+    return sEventNames[eDOMEvents_SVGAbort];
+  case NS_SVG_ERROR:
+    return sEventNames[eDOMEvents_SVGError];
+  case NS_SVG_RESIZE:
+    return sEventNames[eDOMEvents_SVGResize];
+  case NS_SVG_SCROLL:
+    return sEventNames[eDOMEvents_SVGScroll];
+  case NS_SVG_ZOOM:
+    return sEventNames[eDOMEvents_SVGZoom];
+#endif // MOZ_SVG
   default:
     break;
   }
+  // XXXldb We can hit this case for nsEvent objects that we didn't
+  // create and that are not user defined events since this function and
+  // SetEventType are incomplete.  (But fixing that requires fixing the
+  // arrays in nsEventListenerManager too, since the events for which
+  // this is a problem generally *are* created by nsDOMEvent.)
   return nsnull;
 }
 
