@@ -46,16 +46,15 @@
 #include "nsHTMLAtoms.h"
 #include "nsIHTMLDocument.h"
 #include "nsIFormControl.h"
-#include "nsIScriptGlobalObject.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsDOMError.h"
-#include "nsHTMLValue.h"
-#include "nsGenericElement.h"
+#include "nsGenericHTMLElement.h"
 #include "nsISaveAsCharset.h"
 
 // JBK added for submit move from content frame
 #include "nsIFile.h"
 #include "nsDirectoryServiceDefs.h"
+#include "nsStringStream.h"
 #include "nsIFormProcessor.h"
 #include "nsIURI.h"
 #include "nsNetUtil.h"
@@ -75,7 +74,6 @@
 //end
 
 static NS_DEFINE_CID(kFormProcessorCID, NS_FORMPROCESSOR_CID);
-static NS_DEFINE_CID(kCharsetAliasCID, NS_CHARSETALIAS_CID);
 
 /**
  * Helper superclass implementation of nsIFormSubmission, providing common
@@ -112,7 +110,7 @@ public:
   // nsIFormSubmission
   //
   virtual nsresult SubmitTo(nsIURI* aActionURI, const nsAString& aTarget,
-                            nsIContent* aSource, nsPresContext* aPresContext,
+                            nsIContent* aSource, nsILinkHandler* aLinkHandler,
                             nsIDocShell** aDocShell, nsIRequest** aRequest);
 
   /**
@@ -141,28 +139,29 @@ protected:
    * @param aSource the HTML element the name/value is associated with
    * @param aName the name that will be submitted
    * @param aValue the value that will be submitted
-   * @return the new value (or null if the value was not changed)
+   * @param the processed value that will be sent to the server. [OUT]
    */
-  nsString* ProcessValue(nsIDOMHTMLElement* aSource,
-                         const nsAString& aName, const nsAString& aValue);
+  nsresult ProcessValue(nsIDOMHTMLElement* aSource, const nsAString& aName, 
+                        const nsAString& aValue, nsAString& aResult);
 
   // Encoding Helpers
   /**
-   * Encode a Unicode string to bytes using the encoder (or just do ToNewCString
+   * Encode a Unicode string to bytes using the encoder (or just copy the input
    * if there is no encoder).
    * @param aStr the string to encode
-   * @return a char* pointing to the encoded bytes (use nsMemory::Free to free)
+   * @param aResult the encoded string [OUT]
+   * @throws an error if UnicodeToNewBytes fails
    */
-  char* EncodeVal(const nsAString& aStr);
+  nsresult EncodeVal(const nsAString& aStr, nsACString& aResult);
   /**
    * Encode a Unicode string to bytes using an encoder.  (Used by EncodeVal)
    * @param aStr the string to encode
-   * @param aLen the length of aStr
    * @param aEncoder the encoder to encode the bytes with (cannot be null)
-   * @return a char* pointing to the encoded bytes (use nsMemory::Free to free)
+   * @param aOut the encoded string [OUT] 
+   * @throws an error if the encoder fails
    */
-  char* UnicodeToNewBytes(const PRUnichar* aStr, PRUint32 aLen,
-                          nsISaveAsCharset* aEncoder);
+  nsresult UnicodeToNewBytes(const nsAString& aStr, nsISaveAsCharset* aEncoder,
+                             nsACString& aOut);
 
   /** The name of the encoder charset */
   nsCString mCharset;
@@ -185,18 +184,16 @@ public:
    *        builds.
    * @param aCharset the returned charset [OUT]
    */
-  static void GetSubmitCharset(nsIHTMLContent* aForm,
+  static void GetSubmitCharset(nsGenericHTMLElement* aForm,
                                PRUint8 aCtrlsModAtSubmit,
                                nsACString& aCharset);
   /**
    * Get the encoder for a form (suitable to pass in to the constructor).
    * @param aForm the form in question
-   * @param aPresContext the pres context in which we are submitting
    * @param aCharset the charset of the form
    * @param aEncoder the returned encoder [OUT]
    */
-  static nsresult GetEncoder(nsIHTMLContent* aForm,
-                             nsPresContext* aPresContext,
+  static nsresult GetEncoder(nsGenericHTMLElement* aForm,
                              const nsACString& aCharset,
                              nsISaveAsCharset** aEncoder);
   /**
@@ -207,7 +204,7 @@ public:
    *        exist on the form, so *make sure you provide a default value*.)
    *        [OUT]
    */
-  static void GetEnumAttr(nsIHTMLContent* aForm,
+  static void GetEnumAttr(nsGenericHTMLElement* aForm,
                           nsIAtom* aAtom, PRInt32* aValue);
 };
 
@@ -222,8 +219,8 @@ public:
  *        layout/html/forms/src/HtmlProperties.js
  */
 static nsresult
-SendJSWarning(nsIHTMLContent* aContent,
-              const nsAFlatString& aWarningName);
+SendJSWarning(nsIContent* aContent,
+              const char* aWarningName);
 /**
  * Send a warning to the JS console
  * @param aContent the content the warning is about
@@ -232,8 +229,8 @@ SendJSWarning(nsIHTMLContent* aContent,
  * @param aWarningArg1 an argument to replace a %S in the warning
  */
 static nsresult
-SendJSWarning(nsIHTMLContent* aContent,
-              const nsAFlatString& aWarningName,
+SendJSWarning(nsIContent* aContent,
+              const char* aWarningName,
               const nsAFlatString& aWarningArg1);
 /**
  * Send a warning to the JS console
@@ -244,8 +241,8 @@ SendJSWarning(nsIHTMLContent* aContent,
  * @param aWarningArgsLen the number of strings in the array
  */
 static nsresult
-SendJSWarning(nsIHTMLContent* aContent,
-              const nsAFlatString& aWarningName,
+SendJSWarning(nsIContent* aContent,
+              const char* aWarningName,
               const PRUnichar** aWarningArgs, PRUint32 aWarningArgsLen);
 
 
@@ -338,8 +335,8 @@ nsFSURLEncoded::AddNameValuePair(nsIDOMHTMLElement* aSource,
   if (!mWarnedFileControl) {
     nsCOMPtr<nsIFormControl> formControl = do_QueryInterface(aSource);
     if (formControl->GetType() == NS_FORM_INPUT_FILE) {
-      nsCOMPtr<nsIHTMLContent> content = do_QueryInterface(aSource);
-      SendJSWarning(content, NS_LITERAL_STRING("ForgotFileEnctypeWarning"));
+      nsCOMPtr<nsIContent> content = do_QueryInterface(aSource);
+      SendJSWarning(content, "ForgotFileEnctypeWarning");
       mWarnedFileControl = PR_TRUE;
     }
   }
@@ -347,26 +344,29 @@ nsFSURLEncoded::AddNameValuePair(nsIDOMHTMLElement* aSource,
   //
   // Let external code process (and possibly change) value
   //
-  nsString* processedValue = ProcessValue(aSource, aName, aValue);
-
-  //
-  // Encode name
-  //
-  nsCString convName;
-  nsresult rv = URLEncode(aName, convName);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsAutoString processedValue;
+  nsresult rv = ProcessValue(aSource, aName, aValue, processedValue);
 
   //
   // Encode value
   //
   nsCString convValue;
-  if (processedValue) {
-    rv = URLEncode(*processedValue, convValue);
-  } else {
+  if (NS_SUCCEEDED(rv)) {
+    rv = URLEncode(processedValue, convValue);
+  }
+  else {
     rv = URLEncode(aValue, convValue);
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
+  //
+  // Encode name
+  //
+  nsCAutoString convName;
+  rv = URLEncode(aName, convName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+
   //
   // Append data to string
   //
@@ -377,8 +377,6 @@ nsFSURLEncoded::AddNameValuePair(nsIDOMHTMLElement* aSource,
                   + NS_LITERAL_CSTRING("=") + convValue;
   }
 
-  delete processedValue;
-
   return NS_OK;
 }
 
@@ -447,7 +445,27 @@ HandleMailtoSubject(nsCString& aPath) {
       aPath.Append('?');
     }
 
-    aPath.AppendLiteral("subject=Form%20Post%20From%20Mozilla&");
+    // Get the default subject
+    nsXPIDLString brandName;
+    nsresult rv =
+      nsContentUtils::GetLocalizedString(nsContentUtils::eBRAND_PROPERTIES,
+                                         "brandShortName", brandName);
+    if (NS_FAILED(rv))
+      return;
+    const PRUnichar *formatStrings[] = { brandName.get() };
+    nsXPIDLString subjectStr;
+    rv = nsContentUtils::FormatLocalizedString(
+                                           nsContentUtils::eFORMS_PROPERTIES,
+                                           "DefaultFormSubject",
+                                           formatStrings,
+                                           NS_ARRAY_LENGTH(formatStrings),
+                                           subjectStr);
+    if (NS_FAILED(rv))
+      return;
+    aPath.AppendLiteral("subject=");
+    nsCString subjectStrEscaped;
+    aPath.Append(NS_EscapeURL(NS_ConvertUTF16toUTF8(subjectStr), esc_Query,
+                              subjectStrEscaped));
   }
 }
 
@@ -552,22 +570,20 @@ nsFSURLEncoded::GetEncodedSubmission(nsIURI* aURI,
 nsresult
 nsFSURLEncoded::URLEncode(const nsAString& aStr, nsCString& aEncoded)
 {
-  char* inBuf = EncodeVal(aStr);
-
-  if (!inBuf)
-    inBuf = ToNewCString(aStr);
-
-  NS_ENSURE_TRUE(inBuf, NS_ERROR_OUT_OF_MEMORY);
-
   // convert to CRLF breaks
-  char* convertedBuf = nsLinebreakConverter::ConvertLineBreaks(inBuf,
-                           nsLinebreakConverter::eLinebreakAny,
-                           nsLinebreakConverter::eLinebreakNet);
-  nsMemory::Free(inBuf);
+  PRUnichar* convertedBuf =
+    nsLinebreakConverter::ConvertUnicharLineBreaks(PromiseFlatString(aStr).get(),
+                                                   nsLinebreakConverter::eLinebreakAny,
+                                                   nsLinebreakConverter::eLinebreakNet);
+  NS_ENSURE_TRUE(convertedBuf, NS_ERROR_OUT_OF_MEMORY);
 
-  char* escapedBuf = nsEscape(convertedBuf, url_XPAlphas);
+  nsCAutoString encodedBuf;
+  nsresult rv = EncodeVal(nsDependentString(convertedBuf), encodedBuf);
   nsMemory::Free(convertedBuf);
+  NS_ENSURE_SUCCESS(rv, rv);
 
+  char* escapedBuf = nsEscape(encodedBuf.get(), url_XPAlphas);
+  NS_ENSURE_TRUE(escapedBuf, NS_ERROR_OUT_OF_MEMORY);
   aEncoded.Adopt(escapedBuf);
 
   return NS_OK;
@@ -704,36 +720,31 @@ nsFSMultipartFormData::ProcessAndEncode(nsIDOMHTMLElement* aSource,
   //
   // Let external code process (and possibly change) value
   //
-  nsString* processedValue = ProcessValue(aSource, aName, aValue);
-
-  //
-  // Get name
-  //
-  char * encodedVal = EncodeVal(aName);
-  if (!encodedVal) {
-    delete processedValue;
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  aProcessedName.Adopt(encodedVal);
+  nsAutoString processedValue;
+  nsresult rv = ProcessValue(aSource, aName, aValue, processedValue);
 
   //
   // Get value
   //
-  if (processedValue) {
-    encodedVal = EncodeVal(*processedValue);
-    delete processedValue;
+  nsCAutoString encodedVal;
+  if (NS_SUCCEEDED(rv)) {
+    rv = EncodeVal(processedValue, encodedVal);
   } else {
-    encodedVal = EncodeVal(aValue);
+    rv = EncodeVal(aValue, encodedVal);
   }
-  if (!encodedVal) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  aProcessedValue.Adopt(encodedVal);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  //
+  // Get name
+  //
+  rv  = EncodeVal(aName, aProcessedName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
 
   //
   // Convert linebreaks in value
   //
-  aProcessedValue.Adopt(nsLinebreakConverter::ConvertLineBreaks(aProcessedValue.get(),
+  aProcessedValue.Adopt(nsLinebreakConverter::ConvertLineBreaks(encodedVal.get(),
                         nsLinebreakConverter::eLinebreakAny,
                         nsLinebreakConverter::eLinebreakNet));
   return NS_OK;
@@ -747,7 +758,7 @@ nsFSMultipartFormData::AddNameValuePair(nsIDOMHTMLElement* aSource,
                                         const nsAString& aName,
                                         const nsAString& aValue)
 {
-  nsCString nameStr;
+  nsCAutoString nameStr;
   nsCString valueStr;
   nsresult rv = ProcessAndEncode(aSource, aName, aValue, nameStr, valueStr);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -755,6 +766,9 @@ nsFSMultipartFormData::AddNameValuePair(nsIDOMHTMLElement* aSource,
   //
   // Make MIME block for name/value pair
   //
+  // XXX: name parameter should be encoded per RFC 2231
+  // RFC 2388 specifies that RFC 2047 be used, but I think it's not 
+  // consistent with MIME standard.
   mPostDataChunk += NS_LITERAL_CSTRING("--") + mBoundary
                  + NS_LITERAL_CSTRING(CRLF)
                  + NS_LITERAL_CSTRING("Content-Disposition: form-data; name=\"")
@@ -772,8 +786,8 @@ nsFSMultipartFormData::AddNameFilePair(nsIDOMHTMLElement* aSource,
                                        const nsACString& aContentType,
                                        PRBool aMoreFilesToCome)
 {
-  nsCString nameStr;
-  nsCString filenameStr;
+  nsCAutoString nameStr;
+  nsCAutoString filenameStr;
   nsresult rv = ProcessAndEncode(aSource, aName, aFilename, nameStr, filenameStr);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -788,6 +802,9 @@ nsFSMultipartFormData::AddNameFilePair(nsIDOMHTMLElement* aSource,
     mPostDataChunk +=
           NS_LITERAL_CSTRING("Content-Transfer-Encoding: binary" CRLF);
   }
+  // XXX: name/filename parameter should be encoded per RFC 2231
+  // RFC 2388 specifies that RFC 2047 be used, but I think it's not 
+  // consistent with the MIME standard.
   mPostDataChunk +=
          NS_LITERAL_CSTRING("Content-Disposition: form-data; name=\"")
        + nameStr + NS_LITERAL_CSTRING("\"; filename=\"")
@@ -957,16 +974,16 @@ nsFSTextPlain::AddNameValuePair(nsIDOMHTMLElement* aSource,
   //
   // Let external code process (and possibly change) value
   //
-  nsString* processedValue = ProcessValue(aSource, aName, aValue);
+  nsString processedValue;
+  nsresult rv = ProcessValue(aSource, aName, aValue, processedValue);
 
   // XXX This won't work well with a name like "a=b" or "a\nb" but I suppose
   // text/plain doesn't care about that.  Parsers aren't built for escaped
   // values so we'll have to live with it.
-  if (processedValue) {
-    mBody.Append(aName + NS_LITERAL_STRING("=") + *processedValue +
+  if (NS_SUCCEEDED(rv)) {
+    mBody.Append(aName + NS_LITERAL_STRING("=") + processedValue +
                  NS_LITERAL_STRING(CRLF));
 
-    delete processedValue;
   } else {
     mBody.Append(aName + NS_LITERAL_STRING("=") + aValue +
                  NS_LITERAL_STRING(CRLF));
@@ -1016,9 +1033,11 @@ nsFSTextPlain::GetEncodedSubmission(nsIURI* aURI,
     HandleMailtoSubject(path);
 
     // Append the body to and force-plain-text args to the mailto line
+    char* escapedBuf = nsEscape(NS_ConvertUTF16toUTF8(mBody).get(),
+                                url_XAlphas);
+    NS_ENSURE_TRUE(escapedBuf, NS_ERROR_OUT_OF_MEMORY);
     nsCString escapedBody;
-    escapedBody.Adopt(nsEscape(NS_ConvertUTF16toUTF8(mBody).get(),
-                               url_XAlphas));
+    escapedBody.Adopt(escapedBuf);
 
     path += NS_LITERAL_CSTRING("&force-plain-text=Y&body=") + escapedBody;
 
@@ -1071,15 +1090,15 @@ NS_INTERFACE_MAP_END
 // submission
 
 static nsresult
-SendJSWarning(nsIHTMLContent* aContent,
-               const nsAFlatString& aWarningName)
+SendJSWarning(nsIContent* aContent,
+               const char* aWarningName)
 {
   return SendJSWarning(aContent, aWarningName, nsnull, 0);
 }
 
 static nsresult
-SendJSWarning(nsIHTMLContent* aContent,
-               const nsAFlatString& aWarningName,
+SendJSWarning(nsIContent* aContent,
+               const char* aWarningName,
                const nsAFlatString& aWarningArg1)
 {
   const PRUnichar* formatStrings[1] = { aWarningArg1.get() };
@@ -1087,71 +1106,30 @@ SendJSWarning(nsIHTMLContent* aContent,
 }
 
 static nsresult
-SendJSWarning(nsIHTMLContent* aContent,
-              const nsAFlatString& aWarningName,
+SendJSWarning(nsIContent* aContent,
+              const char* aWarningName,
               const PRUnichar** aWarningArgs, PRUint32 aWarningArgsLen)
 {
-  nsresult rv = NS_OK;
-
-  //
   // Get the document URL to use as the filename
-  //
-  nsCAutoString documentURISpec;
 
   nsIDocument* document = aContent->GetDocument();
+  nsIURI *documentURI = nsnull;
   if (document) {
-    nsIURI *documentURI = document->GetDocumentURI();
+    documentURI = document->GetDocumentURI();
     NS_ENSURE_TRUE(documentURI, NS_ERROR_UNEXPECTED);
-    documentURI->GetPath(documentURISpec);
   }
 
-  //
-  // Get the error string
-  //
-  nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-  nsCOMPtr<nsIStringBundle> bundle;
-  rv = bundleService->CreateBundle(
-      "chrome://communicator/locale/layout/HtmlForm.properties",
-      getter_AddRefs(bundle));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsXPIDLString warningStr;
-  if (aWarningArgsLen > 0) {
-    bundle->FormatStringFromName(aWarningName.get(),
-                                 aWarningArgs, aWarningArgsLen,
-                                 getter_Copies(warningStr));
-  } else {
-    bundle->GetStringFromName(aWarningName.get(), getter_Copies(warningStr));
-  }
-
-  //
-  // Create the error
-  //
-  nsCOMPtr<nsIScriptError>
-      scriptError(do_CreateInstance(NS_SCRIPTERROR_CONTRACTID));
-  NS_ENSURE_TRUE(scriptError, NS_ERROR_UNEXPECTED);
-
-  rv = scriptError->Init(warningStr.get(),
-                         NS_ConvertUTF8toUTF16(documentURISpec).get(),
-                         nsnull, (uintN)0,
-                         0, nsIScriptError::warningFlag,
-                         "HTML");
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  //
-  // Send the error to the console
-  //
-  nsCOMPtr<nsIConsoleService>
-      consoleService(do_GetService(NS_CONSOLESERVICE_CONTRACTID));
-  NS_ENSURE_TRUE(consoleService, NS_ERROR_UNEXPECTED);
-
-  return consoleService->LogMessage(scriptError);
+  return nsContentUtils::ReportToConsole(nsContentUtils::eFORMS_PROPERTIES,
+                                         aWarningName,
+                                         aWarningArgs, aWarningArgsLen,
+                                         documentURI,
+                                         EmptyString(), 0, 0,
+                                         nsIScriptError::warningFlag,
+                                         "HTML");
 }
 
 nsresult
-GetSubmissionFromForm(nsIHTMLContent* aForm,
-                      nsPresContext* aPresContext,
+GetSubmissionFromForm(nsGenericHTMLElement* aForm,
                       nsIFormSubmission** aFormSubmission)
 {
   nsresult rv = NS_OK;
@@ -1159,10 +1137,12 @@ GetSubmissionFromForm(nsIHTMLContent* aForm,
   //
   // Get all the information necessary to encode the form data
   //
+  nsIDocument* doc = aForm->GetCurrentDoc();
+  NS_ASSERTION(doc, "Should have doc if we're building submission!");
 
   // Get BIDI options
   PRUint8 ctrlsModAtSubmit = 0;
-  PRUint32 bidiOptions = aPresContext->GetBidi();
+  PRUint32 bidiOptions = doc->GetBidiOptions();
   ctrlsModAtSubmit = GET_BIDI_OPTION_CONTROLSTEXTMODE(bidiOptions);
 
   // Get encoding type (default: urlencoded)
@@ -1179,8 +1159,7 @@ GetSubmissionFromForm(nsIHTMLContent* aForm,
 
   // Get unicode encoder
   nsCOMPtr<nsISaveAsCharset> encoder;
-  nsFormSubmission::GetEncoder(aForm, aPresContext, charset,
-                               getter_AddRefs(encoder));
+  nsFormSubmission::GetEncoder(aForm, charset, getter_AddRefs(encoder));
 
   // Get form processor
   nsCOMPtr<nsIFormProcessor> formProcessor =
@@ -1207,7 +1186,7 @@ GetSubmissionFromForm(nsIHTMLContent* aForm,
         enctype == NS_FORM_ENCTYPE_TEXTPLAIN) {
       nsAutoString enctypeStr;
       aForm->GetAttr(kNameSpaceID_None, nsHTMLAtoms::enctype, enctypeStr);
-      SendJSWarning(aForm, NS_LITERAL_STRING("ForgotPostWarning"), PromiseFlatString(enctypeStr));
+      SendJSWarning(aForm, "ForgotPostWarning", PromiseFlatString(enctypeStr));
     }
     *aFormSubmission = new nsFSURLEncoded(charset, encoder,
                                           formProcessor, bidiOptions, method);
@@ -1225,7 +1204,7 @@ GetSubmissionFromForm(nsIHTMLContent* aForm,
 
 nsresult
 nsFormSubmission::SubmitTo(nsIURI* aActionURI, const nsAString& aTarget,
-                           nsIContent* aSource, nsPresContext* aPresContext,
+                           nsIContent* aSource, nsILinkHandler* aLinkHandler,
                            nsIDocShell** aDocShell, nsIRequest** aRequest)
 {
   nsresult rv;
@@ -1240,20 +1219,19 @@ nsFormSubmission::SubmitTo(nsIURI* aActionURI, const nsAString& aTarget,
   //
   // Actually submit the data
   //
-  nsILinkHandler *handler = aPresContext->GetLinkHandler();
-  NS_ENSURE_TRUE(handler, NS_ERROR_FAILURE);
+  NS_ENSURE_ARG_POINTER(aLinkHandler);
 
-  return handler->OnLinkClickSync(aSource, eLinkVerb_Replace,
-                                  aActionURI,
-                                  PromiseFlatString(aTarget).get(),
-                                  postDataStream, nsnull,
-                                  aDocShell, aRequest);
+  return aLinkHandler->OnLinkClickSync(aSource, eLinkVerb_Replace,
+                                       aActionURI,
+                                       PromiseFlatString(aTarget).get(),
+                                       postDataStream, nsnull,
+                                       aDocShell, aRequest);
 }
 
 // JBK moved from nsFormFrame - bug 34297
 // static
 void
-nsFormSubmission::GetSubmitCharset(nsIHTMLContent* aForm,
+nsFormSubmission::GetSubmitCharset(nsGenericHTMLElement* aForm,
                                    PRUint8 aCtrlsModAtSubmit,
                                    nsACString& oCharset)
 {
@@ -1261,18 +1239,15 @@ nsFormSubmission::GetSubmitCharset(nsIHTMLContent* aForm,
 
   nsresult rv = NS_OK;
   nsAutoString acceptCharsetValue;
-  nsHTMLValue value;
-  rv = aForm->GetHTMLAttribute(nsHTMLAtoms::acceptcharset, value);
-  if (rv == NS_CONTENT_ATTR_HAS_VALUE && value.GetUnit() == eHTMLUnit_String) {
-    value.GetStringValue(acceptCharsetValue);
-  }
+  aForm->GetAttr(kNameSpaceID_None, nsHTMLAtoms::acceptcharset,
+                 acceptCharsetValue);
 
   PRInt32 charsetLen = acceptCharsetValue.Length();
   if (charsetLen > 0) {
     PRInt32 offset=0;
     PRInt32 spPos=0;
     // get charset from charsets one by one
-    nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID, &rv));
+    nsCOMPtr<nsICharsetAlias> calias(do_GetService(NS_CHARSETALIAS_CONTRACTID, &rv));
     if (NS_FAILED(rv)) {
       return;
     }
@@ -1284,8 +1259,9 @@ nsFormSubmission::GetSubmitCharset(nsIHTMLContent* aForm,
           nsAutoString uCharset;
           acceptCharsetValue.Mid(uCharset, offset, cnt);
 
-          nsCAutoString charset; charset.AssignWithConversion(uCharset);
-          if (NS_SUCCEEDED(calias->GetPreferred(charset, oCharset)))
+          if (NS_SUCCEEDED(calias->
+                           GetPreferred(NS_LossyConvertUTF16toASCII(uCharset),
+                                        oCharset)))
             return;
         }
         offset = spPos + 1;
@@ -1326,8 +1302,7 @@ nsFormSubmission::GetSubmitCharset(nsIHTMLContent* aForm,
 // JBK moved from nsFormFrame - bug 34297
 // static
 nsresult
-nsFormSubmission::GetEncoder(nsIHTMLContent* aForm,
-                             nsPresContext* aPresContext,
+nsFormSubmission::GetEncoder(nsGenericHTMLElement* aForm,
                              const nsACString& aCharset,
                              nsISaveAsCharset** aEncoder)
 {
@@ -1335,8 +1310,18 @@ nsFormSubmission::GetEncoder(nsIHTMLContent* aForm,
   nsresult rv = NS_OK;
 
   nsCAutoString charset(aCharset);
-  if(charset.EqualsLiteral("ISO-8859-1"))
+  // canonical name is passed so that we just have to check against
+  // *our* canonical names listed in charsetaliases.properties
+  if (charset.EqualsLiteral("ISO-8859-1")) {
     charset.AssignLiteral("windows-1252");
+  }
+
+  // use UTF-8 for UTF-16* and UTF-32* (per WHATWG and existing practice of
+  // MS IE/Opera). 
+  if (StringBeginsWith(charset, NS_LITERAL_CSTRING("UTF-16")) || 
+      StringBeginsWith(charset, NS_LITERAL_CSTRING("UTF-32"))) {
+    charset.AssignLiteral("UTF-8");
+  }
 
   rv = CallCreateInstance( NS_SAVEASCHARSET_CONTRACTID, aEncoder);
   NS_ASSERTION(NS_SUCCEEDED(rv), "create nsISaveAsCharset failed");
@@ -1353,16 +1338,14 @@ nsFormSubmission::GetEncoder(nsIHTMLContent* aForm,
 }
 
 // i18n helper routines
-char*
-nsFormSubmission::UnicodeToNewBytes(const PRUnichar* aStr, PRUint32 aLen,
-                                    nsISaveAsCharset* aEncoder)
+nsresult
+nsFormSubmission::UnicodeToNewBytes(const nsAString& aStr, 
+                                    nsISaveAsCharset* aEncoder,
+                                    nsACString& aOut)
 {
-  nsresult rv = NS_OK;
-
   PRUint8 ctrlsModAtSubmit = GET_BIDI_OPTION_CONTROLSTEXTMODE(mBidiOptions);
   PRUint8 textDirAtSubmit = GET_BIDI_OPTION_DIRECTION(mBidiOptions);
   //ahmed 15-1
-  nsAutoString temp;
   nsAutoString newBuffer;
   //This condition handle the RTL,LTR for a logical file
   if (ctrlsModAtSubmit == IBMBIDI_CONTROLSTEXTMODE_VISUAL
@@ -1371,8 +1354,6 @@ nsFormSubmission::UnicodeToNewBytes(const PRUnichar* aStr, PRUint32 aLen,
     Conv_06_FE_WithReverse(nsString(aStr),
                            newBuffer,
                            textDirAtSubmit);
-    aStr = newBuffer.get();
-    aLen=newBuffer.Length();
   }
   else if (ctrlsModAtSubmit == IBMBIDI_CONTROLSTEXTMODE_LOGICAL
           && mCharset.Equals(NS_LITERAL_CSTRING("IBM864"),
@@ -1380,19 +1361,16 @@ nsFormSubmission::UnicodeToNewBytes(const PRUnichar* aStr, PRUint32 aLen,
     //For 864 file, When it is logical, if LTR then only convert
     //If RTL will mak a reverse for the buffer
     Conv_FE_06(nsString(aStr), newBuffer);
-    aStr = newBuffer.get();
-    temp = newBuffer;
-    aLen=newBuffer.Length();
     if (textDirAtSubmit == 2) { //RTL
     //Now we need to reverse the Buffer, it is by searching the buffer
-      PRUint32 loop = aLen;
-      PRUint32 z;
-      for (z=0; z<=aLen; z++) {
-        temp.SetCharAt((PRUnichar)aStr[loop], z);
-        loop--;
-      }
+      PRInt32 len = newBuffer.Length();
+      PRUint32 z = 0;
+      nsAutoString temp;
+      temp.SetLength(len);
+      while (--len >= 0)
+        temp.SetCharAt(newBuffer.CharAt(len), z++);
+      newBuffer = temp;
     }
-    aStr = temp.get();
   }
   else if (ctrlsModAtSubmit == IBMBIDI_CONTROLSTEXTMODE_VISUAL
           && mCharset.Equals(NS_LITERAL_CSTRING("IBM864"),
@@ -1400,87 +1378,74 @@ nsFormSubmission::UnicodeToNewBytes(const PRUnichar* aStr, PRUint32 aLen,
                   && textDirAtSubmit == IBMBIDI_TEXTDIRECTION_RTL) {
 
     Conv_FE_06(nsString(aStr), newBuffer);
-    aStr = newBuffer.get();
-    temp = newBuffer;
-    aLen=newBuffer.Length();
     //Now we need to reverse the Buffer, it is by searching the buffer
-    PRUint32 loop = aLen;
-    PRUint32 z;
-    for (z=0; z<=aLen; z++) {
-      temp.SetCharAt((PRUnichar)aStr[loop], z);
-      loop--;
-    }
-    aStr = temp.get();
+    PRInt32 len = newBuffer.Length();
+    PRUint32 z = 0;
+    nsAutoString temp;
+    temp.SetLength(len);
+    while (--len >= 0)
+      temp.SetCharAt(newBuffer.CharAt(len), z++);
+    newBuffer = temp;
+  }
+  else {
+    newBuffer = aStr;
   }
 
-  
-  char* res = nsnull;
-  if(aStr && aStr[0]) {
-    rv = aEncoder->Convert(aStr, &res);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "conversion failed");
-  } 
-  if(! res)
-    res = nsCRT::strdup("");
-  return res;
+  nsXPIDLCString res;
+  if (!newBuffer.IsEmpty()) {
+    aOut.Truncate();
+    nsresult rv = aEncoder->Convert(newBuffer.get(), getter_Copies(res));
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
+  aOut = res;
+  return NS_OK;
 }
 
 
 // static
 void
-nsFormSubmission::GetEnumAttr(nsIHTMLContent* aContent,
+nsFormSubmission::GetEnumAttr(nsGenericHTMLElement* aContent,
                               nsIAtom* atom, PRInt32* aValue)
 {
-  nsHTMLValue value;
-  if (aContent->GetHTMLAttribute(atom, value) == NS_CONTENT_ATTR_HAS_VALUE) {
-    if (eHTMLUnit_Enumerated == value.GetUnit()) {
-      *aValue = value.GetIntValue();
-    }
+  const nsAttrValue* value = aContent->GetParsedAttr(atom);
+  if (value && value->Type() == nsAttrValue::eEnum) {
+    *aValue = value->GetEnumValue();
   }
 }
 
-char*
-nsFormSubmission::EncodeVal(const nsAString& aStr)
+nsresult
+nsFormSubmission::EncodeVal(const nsAString& aStr, nsACString& aOut)
 {
-  char* retval;
-  if (mEncoder) {
-    retval = UnicodeToNewBytes(PromiseFlatString(aStr).get(), aStr.Length(),
-                               mEncoder);
-  } else {
-    retval = ToNewCString(aStr);
-  }
+  NS_ASSERTION(mEncoder, "Encoder not available. Losing data !");
+  if (mEncoder)
+    return UnicodeToNewBytes(aStr, mEncoder, aOut);
 
-  return retval;
+  // fall back to UTF-8
+  CopyUTF16toUTF8(aStr, aOut);
+  return NS_OK;
 }
 
-nsString*
+nsresult
 nsFormSubmission::ProcessValue(nsIDOMHTMLElement* aSource,
-                               const nsAString& aName, const nsAString& aValue)
+                               const nsAString& aName, const nsAString& aValue,
+                               nsAString& aResult) 
 {
   // Hijack _charset_ (hidden inputs only) for internationalization (bug 18643)
   if (aName.EqualsLiteral("_charset_")) {
     nsCOMPtr<nsIFormControl> formControl = do_QueryInterface(aSource);
-    if (formControl) {
-      if (formControl->GetType() == NS_FORM_INPUT_HIDDEN) {
-        return new NS_ConvertASCIItoUTF16(mCharset);
-      }
+    if (formControl && formControl->GetType() == NS_FORM_INPUT_HIDDEN) {
+        CopyASCIItoUTF16(mCharset, aResult);
+        return NS_OK;
     }
   }
 
-  nsString* retval = nsnull;
+  nsresult rv = NS_OK;
+  aResult = aValue;
   if (mFormProcessor) {
-    // XXX We need to change the ProcessValue interface to take nsAString
-    nsString tmpNameStr(aName);
-    retval = new nsString(aValue);
-    if (!retval) {
-      return nsnull;
-    }
-
-#ifdef DEBUG
-    nsresult rv =
-#endif
-    mFormProcessor->ProcessValue(aSource, tmpNameStr, *retval);
+    rv = mFormProcessor->ProcessValue(aSource, aName, aResult);
     NS_ASSERTION(NS_SUCCEEDED(rv), "Unable to Notify form process observer");
   }
 
-  return retval;
+  return rv;
 }
