@@ -44,14 +44,13 @@
 #include "nsIFormControl.h"
 #include "nsIDOMNSHTMLFrameElement.h"
 #include "nsIChromeEventHandler.h"
-#include "nsIFrameLoader.h"
+#include "nsFrameLoader.h"
 
 class nsIDOMAttr;
 class nsIDOMEventListener;
 class nsIDOMNodeList;
 class nsIFrame;
 class nsMappedAttributes;
-class nsIHTMLContent;
 class nsIStyleRule;
 class nsISupportsArray;
 class nsChildContentList;
@@ -60,12 +59,17 @@ class nsIDOMCSSStyleDeclaration;
 class nsIURI;
 class nsIFormControlFrame;
 class nsIForm;
-class nsIPresState;
+class nsPresState;
 class nsIScrollableView;
 class nsILayoutHistoryState;
-class nsHTMLValue;
+class nsIEditor;
 struct nsRect;
 struct nsSize;
+struct nsRuleData;
+
+typedef void (*nsMapRuleToAttributesFunc)(const nsMappedAttributes* aAttributes, 
+                                          nsRuleData* aData);
+
 
 /**
  * A common superclass for HTML elements
@@ -78,12 +82,17 @@ public:
   {
   }
 
+  /** Typesafe, non-refcounting cast from nsIContent.  Cheaper than QI. **/
+  static nsGenericHTMLElement* FromContent(nsIContent *aContent)
+  {
+    if (aContent->IsNodeOfType(eHTML))
+      return NS_STATIC_CAST(nsGenericHTMLElement*, aContent);
+    return nsnull;
+  }
+
   /** Call on shutdown to release globals */
   static void Shutdown();
 
-  // nsISupports
-  NS_IMETHOD QueryInterface(REFNSIID aIID, void** aInstancePtr);
-
   /**
    * Handle QI for the standard DOM interfaces (DOMNode, DOMElement,
    * DOMHTMLElement) and handles tearoffs for other standard interfaces.
@@ -96,7 +105,7 @@ public:
                              void **aInstancePtr);
 
   // From nsGenericElement
-  nsresult CopyInnerTo(nsGenericElement* aDest, PRBool aDeep);
+  nsresult CopyInnerTo(nsGenericElement* aDest) const;
 
   // Implementation for nsIDOMNode
   NS_METHOD GetNodeName(nsAString& aNodeName);
@@ -146,12 +155,15 @@ public:
   nsresult GetClientHeight(PRInt32* aClientHeight);
   nsresult GetClientWidth(PRInt32* aClientWidth);
   nsresult ScrollIntoView(PRBool aTop);
-  // Declare Focus(), Blur(), GetTabIndex() and SetTabIndex() such that classes that 
-  // inherit interfaces with those methods properly override them
+  // Declare Focus(), Blur(), GetTabIndex(), SetTabIndex(), GetSpellcheck() and
+  // SetSpellcheck() such that classes that inherit interfaces with those 
+  // methods properly override them
   NS_IMETHOD Focus();
   NS_IMETHOD Blur();
   NS_IMETHOD GetTabIndex(PRInt32 *aTabIndex);
   NS_IMETHOD SetTabIndex(PRInt32 aTabIndex);
+  NS_IMETHOD GetSpellcheck(PRBool* aSpellcheck);
+  NS_IMETHOD SetSpellcheck(PRBool aSpellcheck);
 
   /**
    * Get the frame's offset information for offsetTop/Left/Width/Height.
@@ -172,88 +184,51 @@ public:
   static const nsSize GetClientAreaSize(nsIFrame *aFrame);
 
   // Implementation for nsIContent
-  virtual void SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                           PRBool aCompileEventHandlers);
-  virtual void GetNameSpaceID(PRInt32* aID) const;
-  nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                   const nsAString& aValue, PRBool aNotify)
-  {
-    return SetAttr(aNameSpaceID, aName, nsnull, aValue, aNotify);
-  }
-  virtual nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsIAtom* aPrefix,
-                           const nsAString& aValue,
-                           PRBool aNotify);
-  virtual nsresult GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                           nsAString& aResult) const;
+  virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                              nsIContent* aBindingParent,
+                              PRBool aCompileEventHandlers);
   virtual nsresult UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                              PRBool aNotify);
 #ifdef DEBUG
   virtual void List(FILE* out, PRInt32 aIndent) const;
   virtual void DumpContent(FILE* out, PRInt32 aIndent,PRBool aDumpAll) const;
 #endif
-  virtual PRBool IsContentOfType(PRUint32 aFlags) const;
+  virtual PRBool IsNodeOfType(PRUint32 aFlags) const;
   virtual void RemoveFocus(nsPresContext *aPresContext);
   virtual PRBool IsFocusable(PRInt32 *aTabIndex = nsnull);
 
-  /**
-   * Standard anchor HandleDOMEvent, used by A, AREA and LINK (parameters
-   * are the same as HandleDOMEvent)
-   */
-  nsresult HandleDOMEventForAnchors(nsPresContext* aPresContext,
-                                    nsEvent* aEvent,
-                                    nsIDOMEvent** aDOMEvent,
-                                    PRUint32 aFlags,
-                                    nsEventStatus* aEventStatus);
+  nsresult PostHandleEventForAnchors(nsEventChainPostVisitor& aVisitor);
 
   // Used by A, AREA, LINK, and STYLE.
   // Callers must hold a reference to nsHTMLUtils's global reference count.
   nsresult GetHrefURIForAnchors(nsIURI** aURI);
 
-  // Implementation for nsIHTMLContent
-  NS_IMETHOD GetHTMLAttribute(nsIAtom* aAttribute, nsHTMLValue& aValue) const;
-  NS_IMETHOD GetID(nsIAtom** aResult) const;
+  // HTML element methods
+  void Compact() { mAttrsAndChildren.Compact(); }
+  const nsAttrValue* GetParsedAttr(nsIAtom* aAttr) const
+  {
+    return mAttrsAndChildren.GetAttr(aAttr);
+  }
+  virtual nsMapRuleToAttributesFunc GetAttributeMappingFunction() const;
+
   virtual const nsAttrValue* GetClasses() const;
   virtual nsIAtom *GetIDAttributeName() const;
   virtual nsIAtom *GetClassAttributeName() const;
-  NS_IMETHOD_(PRBool) HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const;
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
   virtual nsICSSStyleRule* GetInlineStyleRule();
   NS_IMETHOD SetInlineStyleRule(nsICSSStyleRule* aStyleRule, PRBool aNotify);
   already_AddRefed<nsIURI> GetBaseURI() const;
 
-  //----------------------------------------
-  /**
-   * Turn an attribute value into string based on the type of attribute
-   * (does not need to do standard types such as string, integer,
-   * color ...).  Called by GetAttr().
-   *
-   * @param aAttribute the attribute to convert
-   * @param aValue the value to convert
-   * @param aResult the string [OUT]
-   * @return NS_CONTENT_ATTR_HAS_VALUE if the value was successfully converted
-   *         NS_CONTENT_ATTR_NOT_THERE if the value could not be converted
-   * @see nsGenericHTMLElement::GetAttr
-   */
-  NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
-                               const nsHTMLValue& aValue,
-                               nsAString& aResult) const;
-
-  /**
-   * Convert an attribute string value to attribute type based on the type of
-   * attribute.  Called by SetAttr().
-   *
-   * @param aAttribute to attribute to convert
-   * @param aValue the string value to convert
-   * @param aResult the nsAttrValue [OUT]
-   * @return PR_TRUE if the parsing was successful, PR_FALSE otherwise
-   * @see nsGenericHTMLElement::SetAttr
-   */
-  virtual PRBool ParseAttribute(nsIAtom* aAttribute,
+  virtual PRBool ParseAttribute(PRInt32 aNamespaceID,
+                                nsIAtom* aAttribute,
                                 const nsAString& aValue,
                                 nsAttrValue& aResult);
 
   NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* aAttribute) const;
-  NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
+  virtual PRBool SetMappedAttribute(nsIDocument* aDocument,
+                                    nsIAtom* aName,
+                                    nsAttrValue& aValue,
+                                    nsresult* aRetval);
 
   /**
    * Get the base target for any links within this piece
@@ -286,22 +261,6 @@ public:
     return GetFormControlFrameFor(this, doc, aFlushContent);
   }
 
-  /**
-   * Get the primary frame for this content (see GetPrimaryFrameFor)
-   *
-   * @param aFlushContent whether to flush the content sink
-   * @return the primary frame
-   */
-  nsIFrame* GetPrimaryFrame(PRBool aFlushContent)
-  {
-    nsIDocument* doc = GetCurrentDoc();
-    if (!doc) {
-      return nsnull;
-    }
-
-    return GetPrimaryFrameFor(this, doc, aFlushContent);
-  }
-
   //----------------------------------------
 
   /**
@@ -323,15 +282,6 @@ public:
    */
   PRBool ParseDivAlignValue(const nsAString& aString,
                             nsAttrValue& aResult) const;
-  /**
-   * Convert a div align value to string
-   *
-   * @param aValue the value to convert
-   * @param aResult the resulting string
-   * @return whether the value was converted
-   */
-  PRBool DivAlignValueToString(const nsHTMLValue& aValue,
-                               nsAString& aResult) const;
 
   /**
    * Convert a table halign string to value (left/right/center/char/justify)
@@ -342,15 +292,6 @@ public:
    */
   PRBool ParseTableHAlignValue(const nsAString& aString,
                                nsAttrValue& aResult) const;
-  /**
-   * Convert a table halign value to string
-   *
-   * @param aValue the value to convert
-   * @param aResult the resulting string
-   * @return whether the value was converted
-   */
-  PRBool TableHAlignValueToString(const nsHTMLValue& aValue,
-                                  nsAString& aResult) const;
 
   /**
    * Convert a table cell halign string to value
@@ -361,15 +302,6 @@ public:
    */
   PRBool ParseTableCellHAlignValue(const nsAString& aString,
                                    nsAttrValue& aResult) const;
-  /**
-   * Convert a table cell halign value to string
-   *
-   * @param aValue the value to convert
-   * @param aResult the resulting string
-   * @return whether the value was converted
-   */
-  PRBool TableCellHAlignValueToString(const nsHTMLValue& aValue,
-                                      nsAString& aResult) const;
 
   /**
    * Convert a table valign string to value (left/right/center/char/justify/
@@ -381,36 +313,6 @@ public:
    */
   static PRBool ParseTableVAlignValue(const nsAString& aString,
                                       nsAttrValue& aResult);
-  /**
-   * Convert a table valign value to string
-   *
-   * @param aValue the value to convert
-   * @param aResult the resulting string
-   * @return whether the value was converted
-   */
-  static PRBool TableVAlignValueToString(const nsHTMLValue& aValue,
-                                         nsAString& aResult);
-
-  /**
-   * Convert an align value to string (left/right/texttop/baseline/center/
-   * bottom/top/middle/absbottom/abscenter/absmiddle)
-   *
-   * @param aValue the value to convert
-   * @param aResult the resulting string
-   * @return whether the value was converted
-   */
-  static PRBool AlignValueToString(const nsHTMLValue& aValue,
-                                   nsAString& aResult);
-
-  /**
-   * Convert a valign value to string
-   *
-   * @param aValue the value to convert
-   * @param aResult the resulting string
-   * @return whether the value was converted
-   */
-  static PRBool VAlignValueToString(const nsHTMLValue& aValue,
-                                    nsAString& aResult);
 
   /**
    * Convert an image attribute to value (width, height, hspace, vspace, border)
@@ -432,15 +334,6 @@ public:
    */
   static PRBool ParseFrameborderValue(const nsAString& aString,
                                       nsAttrValue& aResult);
-  /**
-   * Convert a frameborder value to string
-   *
-   * @param aValue the value to convert
-   * @param aResult the resulting string
-   * @return whether the value was converted
-   */
-  static PRBool FrameborderValueToString(const nsHTMLValue& aValue,
-                                         nsAString& aResult);
 
   /**
    * Convert a scrolling string to value (yes/no/on/off/scroll/noscroll/auto)
@@ -451,15 +344,6 @@ public:
    */
   static PRBool ParseScrollingValue(const nsAString& aString,
                                     nsAttrValue& aResult);
-  /**
-   * Convert a scrolling value to string
-   *
-   * @param aValue the value to convert
-   * @param aResult the resulting string
-   * @return whether the value was converted
-   */
-  static PRBool ScrollingValueToString(const nsHTMLValue& aValue,
-                                       nsAString& aResult);
 
   /**
    * Create the style struct from the style attr.  Used when an element is first
@@ -500,6 +384,7 @@ public:
   static const MappedAttributeEntry sImageAlignAttributeMap[];
   static const MappedAttributeEntry sDivAlignAttributeMap[];
   static const MappedAttributeEntry sBackgroundAttributeMap[];
+  static const MappedAttributeEntry sBackgroundColorAttributeMap[];
   static const MappedAttributeEntry sScrollingAttributeMap[];
   
   /**
@@ -550,6 +435,26 @@ public:
    */
   static void MapImageSizeAttributesInto(const nsMappedAttributes* aAttributes,
                                          nsRuleData* aData);
+  /**
+   * Helper to map the background attribute
+   * into a style struct.
+   *
+   * @param aAttributes the list of attributes to map
+   * @param aData the returned rule data [INOUT]
+   * @see GetAttributeMappingFunction
+   */
+  static void MapBackgroundInto(const nsMappedAttributes* aAttributes,
+                                nsRuleData* aData);
+  /**
+   * Helper to map the bgcolor attribute
+   * into a style struct.
+   *
+   * @param aAttributes the list of attributes to map
+   * @param aData the returned rule data [INOUT]
+   * @see GetAttributeMappingFunction
+   */
+  static void MapBGColorInto(const nsMappedAttributes* aAttributes,
+                             nsRuleData* aData);
   /**
    * Helper to map the background attributes (currently background and bgcolor)
    * into a style struct.
@@ -570,19 +475,6 @@ public:
    */
   static void MapScrollingAttributeInto(const nsMappedAttributes* aAttributes,
                                         nsRuleData* aData);
-  /**
-   * Get the primary frame for a piece of content.
-   *
-   * @param aContent the content to get the primary frame for
-   * @param aDocument the document for this content
-   * @param aFlushContent whether to flush the content sink, which creates
-   *        frames for content that do not already have it.  EXPENSIVE.
-   * @return the primary frame
-   */
-  static nsIFrame* GetPrimaryFrameFor(nsIContent* aContent,
-                                      nsIDocument* aDocument,
-                                      PRBool aFlushContent);
-
   /**
    * Get the primary form control frame for a piece of content.  Same as
    * GetPrimaryFrameFor, except it QI's to nsIFormControlFrame.
@@ -603,8 +495,8 @@ public:
    * @param aContent the content to get presentation state for.
    * @param aPresState the presentation state (out param)
    */
-  static nsresult GetPrimaryPresState(nsIHTMLContent* aContent,
-                                      nsIPresState** aPresState);
+  static nsresult GetPrimaryPresState(nsGenericHTMLElement* aContent,
+                                      nsPresState** aPresState);
   /**
    * Get the layout history object *and* generate the key for a particular
    * piece of content.
@@ -613,19 +505,19 @@ public:
    * @param aState the history state object (out param)
    * @param aKey the key (out param)
    */
-  static nsresult GetLayoutHistoryAndKey(nsIHTMLContent* aContent,
+  static nsresult GetLayoutHistoryAndKey(nsGenericHTMLElement* aContent,
                                          nsILayoutHistoryState** aState,
                                          nsACString& aKey);
   /**
    * Restore the state for a form control.  Ends up calling
    * nsIFormControl::RestoreState().
    *
-   * @param aContent an nsIHTMLContent* pointing to the form control
+   * @param aContent an nsGenericHTMLElement* pointing to the form control
    * @param aControl an nsIFormControl* pointing to the form control
    * @return PR_FALSE if RestoreState() was not called, the return
    *         value of RestoreState() otherwise.
    */
-  static PRBool RestoreFormControlState(nsIHTMLContent* aContent,
+  static PRBool RestoreFormControlState(nsGenericHTMLElement* aContent,
                                         nsIFormControl* aControl);
 
   /**
@@ -637,9 +529,13 @@ public:
   // Form Helper Routines
   /**
    * Find an ancestor of this content node which is a form (could be null)
-   * @param aForm the form ancestore [OUT]
+   * @param aCurrentForm the current form for this node.  If this is
+   *        non-null, and no ancestor form is found, and the current form is in
+   *        a connected subtree with the node, the current form will be
+   *        returned.  This is needed to handle cases when HTML elements have a
+   *        current form that they're not descendants of.
    */
-  already_AddRefed<nsIDOMHTMLFormElement> FindForm();
+  already_AddRefed<nsIDOMHTMLFormElement> FindForm(nsIForm* aCurrentForm = nsnull);
 
   /**
    * See if the document being tested has nav-quirks mode enabled.
@@ -647,30 +543,6 @@ public:
    */
   static PRBool InNavQuirksMode(nsIDocument* aDoc);
 
-  /**
-   * Set attribute and (if needed) notify documentobservers and fire off
-   * mutation events.
-   *
-   * @param aNamespaceID  namespace of attribute
-   * @param aAttribute    local-name of attribute
-   * @param aPrefix       aPrefix of attribute
-   * @param aOldValue     previous value of attribute. Only needed if
-   *                      aFireMutation is true.
-   * @param aParsedValue  parsed new value of attribute
-   * @param aModification is this a attribute-modification or addition. Only
-   *                      needed if aFireMutation or aNotify is true.
-   * @param aFireMutation should mutation-events be fired?
-   * @param aNotify       should we notify document-observers?
-   */
-  nsresult SetAttrAndNotify(PRInt32 aNamespaceID,
-                            nsIAtom* aAttribute,
-                            nsIAtom* aPrefix,
-                            const nsAString& aOldValue,
-                            nsAttrValue& aParsedValue,
-                            PRBool aModification,
-                            PRBool aFireMutation,
-                            PRBool aNotify);
- 
   // Helper functions for <a> and <area>
   static nsresult SetProtocolInHrefString(const nsAString &aHref,
                                           const nsAString &aProtocol,
@@ -743,17 +615,15 @@ protected:
    */
   PRBool IsEventName(nsIAtom* aName);
 
-  virtual const nsAttrName* InternalGetExistingAttrNameFromQName(const nsAString& aStr) const;
+  virtual nsresult AfterSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
+                                const nsAString* aValue, PRBool aNotify);
 
-  /**
-   * ReplaceContentsWithText will take the aText string and make sure
-   * that the only child of |this| is a textnode which corresponds to
-   * that string.
-   *
-   * @param aText the text to put in
-   * @param aNotify whether to notify the document of child adds/removes
-   */
-  nsresult ReplaceContentsWithText(const nsAString& aText, PRBool aNotify);
+  virtual nsresult
+    GetEventListenerManagerForAttr(nsIEventListenerManager** aManager,
+                                   nsISupports** aTarget,
+                                   PRBool* aDefer);
+
+  virtual const nsAttrName* InternalGetExistingAttrNameFromQName(const nsAString& aStr) const;
 
   /**
    * Helper method for NS_IMPL_STRING_ATTR macro.
@@ -799,7 +669,7 @@ protected:
    * @param aAttr    name of attribute.
    * @param aValue   Boolean value of attribute.
    */
-  NS_HIDDEN_(nsresult) GetBoolAttr(nsIAtom* aAttr, PRBool* aValue);
+  NS_HIDDEN_(nsresult) GetBoolAttr(nsIAtom* aAttr, PRBool* aValue) const;
 
   /**
    * Helper method for NS_IMPL_BOOL_ATTR macro.
@@ -840,16 +710,58 @@ protected:
    * isn't a relative URI the value of the attribute is returned as is. Only
    * works for attributes in null namespace.
    *
+   * @param aAttr      name of attribute.
+   * @param aBaseAttr  name of base attribute.
+   * @param aResult    result value [out]
+   */
+  NS_HIDDEN_(nsresult) GetURIAttr(nsIAtom* aAttr, nsIAtom* aBaseAttr, nsAString& aResult);
+
+  /**
+   * This method works like GetURIAttr, except that it supports multiple
+   * URIs separated by whitespace (one or more U+0020 SPACE characters).
+   *
+   * Gets the absolute URI values of an attribute, by resolving any relative
+   * URIs in the attribute against the baseuri of the element. If a substring
+   * isn't a relative URI, the substring is returned as is. Only works for
+   * attributes in null namespace.
+   *
    * @param aAttr    name of attribute.
    * @param aResult  result value [out]
    */
-  NS_HIDDEN_(nsresult) GetURIAttr(nsIAtom* aAttr, nsAString& aResult);
+  NS_HIDDEN_(nsresult) GetURIListAttr(nsIAtom* aAttr, nsAString& aResult);
 
   /**
    * Helper method to recreate all frames for this content, if there
    * are any.
    */
   void RecreateFrames();
+
+  /**
+   * Locate an nsIEditor rooted at this content node, if there is one.
+   */
+  NS_HIDDEN_(nsresult) GetEditor(nsIEditor** aEditor);
+  NS_HIDDEN_(nsresult) GetEditorInternal(nsIEditor** aEditor);
+
+  /**
+   * Locates the nsIEditor associated with this node.  In general this is
+   * equivalent to GetEditorInternal(), but for designmode or contenteditable,
+   * this may need to get an editor that's not actually on this element's
+   * associated TextControlFrame.  This is used by the spellchecking routines
+   * to get the editor affected by changing the spellcheck attribute on this
+   * node.
+   */
+  virtual already_AddRefed<nsIEditor> GetAssociatedEditor();
+
+  /**
+   * Returns true if this is the current document's body element
+   */
+  PRBool IsCurrentBodyElement();
+
+  /**
+   * Ensures all editors associated with a subtree are synced, for purposes of
+   * spellchecking.
+   */
+  static void SyncEditorsOnSubtree(nsIContent* content);
 };
 
 
@@ -867,17 +779,19 @@ public:
 
   NS_IMETHOD QueryInterface(REFNSIID aIID, void** aInstancePtr);
 
-  virtual PRBool IsContentOfType(PRUint32 aFlags) const;
+  virtual PRBool IsNodeOfType(PRUint32 aFlags) const;
 
   // nsIFormControl
   NS_IMETHOD GetForm(nsIDOMHTMLFormElement** aForm);
   NS_IMETHOD SetForm(nsIDOMHTMLFormElement* aForm,
-                     PRBool aRemoveFromForm = PR_TRUE);
+                     PRBool aRemoveFromForm,
+                     PRBool aNotify);
+
   NS_IMETHOD SaveState()
   {
     return NS_OK;
   }
-  virtual PRBool RestoreState(nsIPresState* aState)
+  virtual PRBool RestoreState(nsPresState* aState)
   {
     return PR_FALSE;
   }
@@ -885,20 +799,18 @@ public:
   {
     return PR_TRUE;
   }
+  
+  virtual PRBool IsSubmitControl() const;
 
   // nsIContent
-  virtual void SetParent(nsIContent *aParent);
-  virtual void SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                           PRBool aCompileEventHandlers);
-
-  nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                   const nsAString& aValue, PRBool aNotify)
-  {
-    return SetAttr(aNameSpaceID, aName, nsnull, aValue, aNotify);
-  }
-  virtual nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                           nsIAtom* aPrefix, const nsAString& aValue,
-                           PRBool aNotify);
+  virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                              nsIContent* aBindingParent,
+                              PRBool aCompileEventHandlers);
+  virtual void UnbindFromTree(PRBool aDeep = PR_TRUE,
+                              PRBool aNullParent = PR_TRUE);
+  virtual nsresult UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                             PRBool aNotify);
+  virtual PRUint32 GetDesiredIMEState();
 
 protected:
   /**
@@ -909,6 +821,19 @@ protected:
    */
   void FindAndSetForm();
 
+  virtual nsresult BeforeSetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                                 const nsAString* aValue, PRBool aNotify);
+
+  virtual nsresult AfterSetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                                const nsAString* aValue, PRBool aNotify);
+
+  /**
+   * Returns true if the control can be disabled
+   */
+  PRBool CanBeDisabled() const;
+
+  virtual PRInt32 IntrinsicState() const;
+
   /** The form that contains this control */
   nsIForm* mForm;
 };
@@ -921,8 +846,8 @@ protected:
 
 class nsGenericHTMLFrameElement : public nsGenericHTMLElement,
                                   public nsIDOMNSHTMLFrameElement,
-                                  public nsIChromeEventHandler,
-                                  public nsIFrameLoaderOwner
+                                  public nsIFrameLoaderOwner,
+                                  public nsIChromeEventHandler
 {
 public:
   nsGenericHTMLFrameElement(nsINodeInfo *aNodeInfo)
@@ -931,6 +856,7 @@ public:
   }
   virtual ~nsGenericHTMLFrameElement();
 
+  // nsISupports
   NS_IMETHOD QueryInterface(REFNSIID aIID, void** aInstancePtr);
 
   // nsIDOMNSHTMLFrameElement
@@ -940,14 +866,15 @@ public:
   NS_DECL_NSICHROMEEVENTHANDLER
 
   // nsIFrameLoaderOwner
-  NS_IMETHOD GetFrameLoader(nsIFrameLoader **aFrameLoader);
-  NS_IMETHOD SetFrameLoader(nsIFrameLoader *aFrameLoader);
+  NS_DECL_NSIFRAMELOADEROWNER
 
   // nsIContent
   virtual PRBool IsFocusable(PRInt32 *aTabIndex = nsnull);
-  virtual void SetParent(nsIContent *aParent);
-  virtual void SetDocument(nsIDocument *aDocument, PRBool aDeep,
-                           PRBool aCompileEventHandlers);
+  virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                              nsIContent* aBindingParent,
+                              PRBool aCompileEventHandlers);
+  virtual void UnbindFromTree(PRBool aDeep = PR_TRUE,
+                              PRBool aNullParent = PR_TRUE);
   nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                    const nsAString& aValue, PRBool aNotify)
   {
@@ -977,14 +904,14 @@ protected:
  * A macro to implement the NS_NewHTMLXXXElement() functions.
  */
 #define NS_IMPL_NS_NEW_HTML_ELEMENT(_elementName)                            \
-nsIHTMLContent*                                                              \
+nsGenericHTMLElement*                                                        \
 NS_NewHTML##_elementName##Element(nsINodeInfo *aNodeInfo, PRBool aFromParser)\
 {                                                                            \
   return new nsHTML##_elementName##Element(aNodeInfo);                       \
 }
 
 #define NS_IMPL_NS_NEW_HTML_ELEMENT_CHECK_PARSER(_elementName)               \
-nsIHTMLContent*                                                              \
+nsGenericHTMLElement*                                                        \
 NS_NewHTML##_elementName##Element(nsINodeInfo *aNodeInfo, PRBool aFromParser)\
 {                                                                            \
   return new nsHTML##_elementName##Element(aNodeInfo, aFromParser);          \
@@ -1073,7 +1000,7 @@ NS_NewHTML##_elementName##Element(nsINodeInfo *aNodeInfo, PRBool aFromParser)\
   NS_IMETHODIMP                                                     \
   _class::Get##_method(nsAString& aValue)                           \
   {                                                                 \
-    return GetURIAttr(nsHTMLAtoms::_atom, aValue);                  \
+    return GetURIAttr(nsHTMLAtoms::_atom, nsnull, aValue);          \
   }                                                                 \
   NS_IMETHODIMP                                                     \
   _class::Set##_method(const nsAString& aValue)                     \
@@ -1081,6 +1008,18 @@ NS_NewHTML##_elementName##Element(nsINodeInfo *aNodeInfo, PRBool aFromParser)\
     return SetAttrHelper(nsHTMLAtoms::_atom, aValue);               \
   }
 
+#define NS_IMPL_URI_ATTR_WITH_BASE(_class, _method, _atom, _base_atom)       \
+  NS_IMETHODIMP                                                              \
+  _class::Get##_method(nsAString& aValue)                                    \
+  {                                                                          \
+    return GetURIAttr(nsHTMLAtoms::_atom, nsHTMLAtoms::_base_atom, aValue);  \
+  }                                                                          \
+  NS_IMETHODIMP                                                              \
+  _class::Set##_method(const nsAString& aValue)                              \
+  {                                                                          \
+    return SetAttrHelper(nsHTMLAtoms::_atom, aValue);                        \
+  }
+
 /**
  * QueryInterface() implementation helper macros
  */
@@ -1147,17 +1086,20 @@ NS_NewHTML##_elementName##Element(nsINodeInfo *aNodeInfo, PRBool aFromParser)\
 // Element class factory methods
 
 #define NS_DECLARE_NS_NEW_HTML_ELEMENT(_elementName)              \
-nsIHTMLContent*                                                   \
+nsGenericHTMLElement*                                             \
 NS_NewHTML##_elementName##Element(nsINodeInfo *aNodeInfo,         \
                                   PRBool aFromParser = PR_FALSE);
 
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Shared)
+NS_DECLARE_NS_NEW_HTML_ELEMENT(SharedList)
+NS_DECLARE_NS_NEW_HTML_ELEMENT(SharedObject)
+
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Anchor)
-NS_DECLARE_NS_NEW_HTML_ELEMENT(Applet)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Area)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(BR)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Body)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Button)
+NS_DECLARE_NS_NEW_HTML_ELEMENT(Canvas)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Mod)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Div)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(FieldSet)
@@ -1178,7 +1120,6 @@ NS_DECLARE_NS_NEW_HTML_ELEMENT(Legend)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Link)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Map)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Meta)
-NS_DECLARE_NS_NEW_HTML_ELEMENT(SharedList)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Object)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(OptGroup)
 NS_DECLARE_NS_NEW_HTML_ELEMENT(Option)
