@@ -39,7 +39,6 @@
 #include "nsCOMPtr.h"
 #include "nsIDOMHTMLBodyElement.h"
 #include "nsIDOMEventReceiver.h"
-#include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
 #include "nsStyleConsts.h"
@@ -57,6 +56,7 @@
 #include "nsRuleData.h"
 #include "nsIFrame.h"
 #include "nsIDocShell.h"
+#include "nsIEditorDocShell.h"
 #include "nsCOMPtr.h"
 #include "nsIView.h"
 #include "nsLayoutAtoms.h"
@@ -96,7 +96,7 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLElement::)
+  NS_FORWARD_NSIDOMNODE(nsGenericHTMLElement::)
 
   // nsIDOMElement
   NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLElement::)
@@ -107,14 +107,17 @@ public:
   // nsIDOMHTMLBodyElement
   NS_DECL_NSIDOMHTMLBODYELEMENT
 
-  virtual PRBool ParseAttribute(nsIAtom* aAttribute,
+  virtual PRBool ParseAttribute(PRInt32 aNamespaceID,
+                                nsIAtom* aAttribute,
                                 const nsAString& aValue,
                                 nsAttrValue& aResult);
-  virtual void SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                           PRBool aCompileEventHandlers);
-  NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
+  virtual void UnbindFromTree(PRBool aDeep = PR_TRUE,
+                              PRBool aNullParent = PR_TRUE);
+  virtual nsMapRuleToAttributesFunc GetAttributeMappingFunction() const;
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
   NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* aAttribute) const;
+  virtual already_AddRefed<nsIEditor> GetAssociatedEditor();
+  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
 
 protected:
   BodyRule* mContentStyleRule;
@@ -139,8 +142,6 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
   if (!aData || (aData->mSID != eStyleStruct_Margin) || !aData->mMarginData || !mPart)
     return NS_OK; // We only care about margins.
 
-  nsHTMLValue value;
-  
   PRInt32 bodyMarginWidth  = -1;
   PRInt32 bodyMarginHeight = -1;
   PRInt32 bodyTopMargin = -1;
@@ -153,11 +154,12 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
   nsCompatibility mode = aData->mPresContext->CompatibilityMode();
 
 
+  const nsAttrValue* value;
   if (mPart->GetAttrCount() > 0) {
     // if marginwidth/marginheight are set, reflect them as 'margin'
-    mPart->GetHTMLAttribute(nsHTMLAtoms::marginwidth, value);
-    if (eHTMLUnit_Integer == value.GetUnit()) {
-      bodyMarginWidth = value.GetIntValue();
+    value = mPart->GetParsedAttr(nsHTMLAtoms::marginwidth);
+    if (value && value->Type() == nsAttrValue::eInteger) {
+      bodyMarginWidth = value->GetIntegerValue();
       if (bodyMarginWidth < 0) bodyMarginWidth = 0;
       nsCSSRect& margin = aData->mMarginData->mMargin;
       if (margin.mLeft.GetUnit() == eCSSUnit_Null)
@@ -166,9 +168,9 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
         margin.mRight.SetFloatValue((float)bodyMarginWidth, eCSSUnit_Pixel);
     }
 
-    mPart->GetHTMLAttribute(nsHTMLAtoms::marginheight, value);
-    if (eHTMLUnit_Integer == value.GetUnit()) {
-      bodyMarginHeight = value.GetIntValue();
+    value = mPart->GetParsedAttr(nsHTMLAtoms::marginheight);
+    if (value && value->Type() == nsAttrValue::eInteger) {
+      bodyMarginHeight = value->GetIntegerValue();
       if (bodyMarginHeight < 0) bodyMarginHeight = 0;
       nsCSSRect& margin = aData->mMarginData->mMargin;
       if (margin.mTop.GetUnit() == eCSSUnit_Null)
@@ -179,9 +181,9 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
 
     if (eCompatibility_NavQuirks == mode){
       // topmargin (IE-attribute)
-      mPart->GetHTMLAttribute(nsHTMLAtoms::topmargin, value);
-      if (eHTMLUnit_Integer == value.GetUnit()) {
-        bodyTopMargin = value.GetIntValue();
+      value = mPart->GetParsedAttr(nsHTMLAtoms::topmargin);
+      if (value && value->Type() == nsAttrValue::eInteger) {
+        bodyTopMargin = value->GetIntegerValue();
         if (bodyTopMargin < 0) bodyTopMargin = 0;
         nsCSSRect& margin = aData->mMarginData->mMargin;
         if (margin.mTop.GetUnit() == eCSSUnit_Null)
@@ -189,9 +191,9 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
       }
 
       // bottommargin (IE-attribute)
-      mPart->GetHTMLAttribute(nsHTMLAtoms::bottommargin, value);
-      if (eHTMLUnit_Integer == value.GetUnit()) {
-        bodyBottomMargin = value.GetIntValue();
+      value = mPart->GetParsedAttr(nsHTMLAtoms::bottommargin);
+      if (value && value->Type() == nsAttrValue::eInteger) {
+        bodyBottomMargin = value->GetIntegerValue();
         if (bodyBottomMargin < 0) bodyBottomMargin = 0;
         nsCSSRect& margin = aData->mMarginData->mMargin;
         if (margin.mBottom.GetUnit() == eCSSUnit_Null)
@@ -199,9 +201,9 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
       }
 
       // leftmargin (IE-attribute)
-      mPart->GetHTMLAttribute(nsHTMLAtoms::leftmargin, value);
-      if (eHTMLUnit_Integer == value.GetUnit()) {
-        bodyLeftMargin = value.GetIntValue();
+      value = mPart->GetParsedAttr(nsHTMLAtoms::leftmargin);
+      if (value && value->Type() == nsAttrValue::eInteger) {
+        bodyLeftMargin = value->GetIntegerValue();
         if (bodyLeftMargin < 0) bodyLeftMargin = 0;
         nsCSSRect& margin = aData->mMarginData->mMargin;
         if (margin.mLeft.GetUnit() == eCSSUnit_Null)
@@ -209,9 +211,9 @@ BodyRule::MapRuleInfoInto(nsRuleData* aData)
       }
 
       // rightmargin (IE-attribute)
-      mPart->GetHTMLAttribute(nsHTMLAtoms::rightmargin, value);
-      if (eHTMLUnit_Integer == value.GetUnit()) {
-        bodyRightMargin = value.GetIntValue();
+      value = mPart->GetParsedAttr(nsHTMLAtoms::rightmargin);
+      if (value && value->Type() == nsAttrValue::eInteger) {
+        bodyRightMargin = value->GetIntegerValue();
         if (bodyRightMargin < 0) bodyRightMargin = 0;
         nsCSSRect& margin = aData->mMarginData->mMargin;
         if (margin.mRight.GetUnit() == eCSSUnit_Null)
@@ -305,7 +307,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLBodyElement, nsGenericHTMLElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
 
-NS_IMPL_DOM_CLONENODE(nsHTMLBodyElement)
+NS_IMPL_ELEMENT_CLONE(nsHTMLBodyElement)
 
 
 NS_IMPL_URI_ATTR(nsHTMLBodyElement, Background, background)
@@ -317,8 +319,7 @@ nsHTMLBodyElement::Get##func_(nsAString& aColor)                    \
   aColor.Truncate();                                                \
   nsAutoString color;                                               \
   nscolor attrColor;                                                \
-  if (NS_CONTENT_ATTR_NOT_THERE ==                                  \
-      GetAttr(kNameSpaceID_None, nsHTMLAtoms::attr_, color)) {      \
+  if (!GetAttr(kNameSpaceID_None, nsHTMLAtoms::attr_, color)) {     \
                                                                     \
     nsPresContext *presContext = GetPresContext();                  \
     if (presContext) {                                              \
@@ -353,22 +354,16 @@ nsHTMLBodyElement::GetBgColor(nsAString& aBgColor)
 
   nsAutoString attr;
   nscolor bgcolor;
-  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::bgcolor, attr);
 
   // If we don't have an attribute, find the actual color used for
   // (generally from the user agent style sheet) for compatibility
-  if (rv == NS_CONTENT_ATTR_NOT_THERE) {
-    nsIDocument *document = GetCurrentDoc();
-    if (document) {
-      // Make sure the style is up-to-date, since we need it
-      document->FlushPendingNotifications(Flush_Style);
-
-      nsIFrame* frame = GetPrimaryFrameFor(this, document, PR_FALSE);
+  if (!GetAttr(kNameSpaceID_None, nsHTMLAtoms::bgcolor, attr)) {
+    // Make sure the style is up-to-date, since we need it
+    nsIFrame* frame = GetPrimaryFrame(Flush_Style);
     
-      if (frame) {
-        bgcolor = frame->GetStyleBackground()->mBackgroundColor;
-        NS_RGBToHex(bgcolor, aBgColor);
-      }
+    if (frame) {
+      bgcolor = frame->GetStyleBackground()->mBackgroundColor;
+      NS_RGBToHex(bgcolor, aBgColor);
     }
   }
   else if (NS_ColorNameToRGB(attr, &bgcolor)) {
@@ -391,41 +386,44 @@ nsHTMLBodyElement::SetBgColor(const nsAString& aBgColor)
 }
 
 PRBool
-nsHTMLBodyElement::ParseAttribute(nsIAtom* aAttribute,
+nsHTMLBodyElement::ParseAttribute(PRInt32 aNamespaceID,
+                                  nsIAtom* aAttribute,
                                   const nsAString& aValue,
                                   nsAttrValue& aResult)
 {
-  if (aAttribute == nsHTMLAtoms::bgcolor ||
-      aAttribute == nsHTMLAtoms::text ||
-      aAttribute == nsHTMLAtoms::link ||
-      aAttribute == nsHTMLAtoms::alink ||
-      aAttribute == nsHTMLAtoms::vlink) {
-    return aResult.ParseColor(aValue, GetOwnerDoc());
-  }
-  if (aAttribute == nsHTMLAtoms::marginwidth ||
-      aAttribute == nsHTMLAtoms::marginheight ||
-      aAttribute == nsHTMLAtoms::topmargin ||
-      aAttribute == nsHTMLAtoms::bottommargin ||
-      aAttribute == nsHTMLAtoms::leftmargin ||
-      aAttribute == nsHTMLAtoms::rightmargin) {
-    return aResult.ParseIntWithBounds(aValue, 0);
+  if (aNamespaceID == kNameSpaceID_None) {
+    if (aAttribute == nsHTMLAtoms::bgcolor ||
+        aAttribute == nsHTMLAtoms::text ||
+        aAttribute == nsHTMLAtoms::link ||
+        aAttribute == nsHTMLAtoms::alink ||
+        aAttribute == nsHTMLAtoms::vlink) {
+      return aResult.ParseColor(aValue, GetOwnerDoc());
+    }
+    if (aAttribute == nsHTMLAtoms::marginwidth ||
+        aAttribute == nsHTMLAtoms::marginheight ||
+        aAttribute == nsHTMLAtoms::topmargin ||
+        aAttribute == nsHTMLAtoms::bottommargin ||
+        aAttribute == nsHTMLAtoms::leftmargin ||
+        aAttribute == nsHTMLAtoms::rightmargin) {
+      return aResult.ParseIntWithBounds(aValue, 0);
+    }
   }
 
-  return nsGenericHTMLElement::ParseAttribute(aAttribute, aValue, aResult);
+  return nsGenericHTMLElement::ParseAttribute(aNamespaceID, aAttribute, aValue,
+                                              aResult);
 }
 
 void
-nsHTMLBodyElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                               PRBool aCompileEventHandlers)
+nsHTMLBodyElement::UnbindFromTree(PRBool aDeep, PRBool aNullParent)
 {
-  nsIDocument *document = GetCurrentDoc();
-  if (aDocument != document && mContentStyleRule) {
+  if (mContentStyleRule) {
     mContentStyleRule->mPart = nsnull;
 
-    // destroy old style rule since the sheet will probably change
+    // destroy old style rule
     NS_RELEASE(mContentStyleRule);
   }
-  nsGenericHTMLElement::SetDocument(aDocument, aDeep, aCompileEventHandlers);
+
+  nsGenericHTMLElement::UnbindFromTree(aDeep, aNullParent);  
 }
 
 static 
@@ -474,11 +472,10 @@ void MapAttributesIntoRule(const nsMappedAttributes* aAttributes, nsRuleData* aD
   nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
 }
 
-NS_IMETHODIMP
-nsHTMLBodyElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const
+nsMapRuleToAttributesFunc
+nsHTMLBodyElement::GetAttributeMappingFunction() const
 {
-  aMapRuleFunc = &MapAttributesIntoRule;
-  return NS_OK;
+  return &MapAttributesIntoRule;
 }
 
 NS_IMETHODIMP
@@ -523,3 +520,32 @@ nsHTMLBodyElement::IsAttributeMapped(const nsIAtom* aAttribute) const
 
   return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
 }
+
+already_AddRefed<nsIEditor>
+nsHTMLBodyElement::GetAssociatedEditor()
+{
+  nsIEditor* editor = nsnull;
+  if (NS_SUCCEEDED(GetEditorInternal(&editor)) && editor) {
+    return editor;
+  }
+
+  // Make sure this is the actual body of the document
+  if (!IsCurrentBodyElement()) {
+    return nsnull;
+  }
+
+  // For designmode, try to get document's editor
+  nsPresContext* presContext = GetPresContext();
+  if (!presContext) {
+    return nsnull;
+  }
+
+  nsCOMPtr<nsISupports> container = presContext->GetContainer();
+  nsCOMPtr<nsIEditorDocShell> editorDocShell = do_QueryInterface(container);
+  if (!editorDocShell) {
+    return nsnull;
+  }
+
+  editorDocShell->GetEditor(&editor);
+  return editor;
+}
