@@ -37,7 +37,6 @@
 #include "nsIDOMHTMLImageElement.h"
 #include "nsIDOMNSHTMLImageElement.h"
 #include "nsIDOMEventReceiver.h"
-#include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsImageLoadingContent.h"
 #include "nsHTMLAtoms.h"
@@ -51,7 +50,6 @@
 #include "nsIDOMWindowInternal.h"
 #include "nsIDOMDocument.h"
 #include "nsIScriptContext.h"
-#include "nsIScriptGlobalObject.h"
 #include "nsIURL.h"
 #include "nsIIOService.h"
 #include "nsIURL.h"
@@ -79,6 +77,7 @@
 #include "nsIView.h"
 #include "nsImageMapUtils.h"
 #include "nsIDOMHTMLMapElement.h"
+#include "nsEventDispatcher.h"
 
 // XXX nav attrs: suppress
 
@@ -96,7 +95,7 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLElement::)
+  NS_FORWARD_NSIDOMNODE(nsGenericHTMLElement::)
 
   // nsIDOMElement
   NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLElement::)
@@ -115,20 +114,17 @@ public:
                         PRUint32 argc, jsval *argv);
 
   // nsIContent
-  virtual PRBool ParseAttribute(nsIAtom* aAttribute,
+  virtual PRBool ParseAttribute(PRInt32 aNamespaceID,
+                                nsIAtom* aAttribute,
                                 const nsAString& aValue,
                                 nsAttrValue& aResult);
-  NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
-                               const nsHTMLValue& aValue,
-                               nsAString& aResult) const;
   virtual nsChangeHint GetAttributeChangeHint(const nsIAtom* aAttribute,
                                               PRInt32 aModType) const;
   NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* aAttribute) const;
-  NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
-  virtual nsresult HandleDOMEvent(nsPresContext* aPresContext,
-                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
-                                  PRUint32 aFlags,
-                                  nsEventStatus* aEventStatus);
+  virtual nsMapRuleToAttributesFunc GetAttributeMappingFunction() const;
+
+  virtual nsresult PreHandleEvent(nsEventChainPreVisitor& aVisitor);
+
   PRBool IsFocusable(PRInt32 *aTabIndex = nsnull);
 
   // SetAttr override.  C++ is stupid, so have to override both
@@ -145,9 +141,12 @@ public:
   // XXXbz What about UnsetAttr?  We don't seem to unload images when
   // that happens...
 
-  virtual void SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                           PRBool aCompileEventHandlers);  
-  virtual void SetParent(nsIContent* aParent);  
+  virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                              nsIContent* aBindingParent,
+                              PRBool aCompileEventHandlers);
+
+  virtual PRInt32 IntrinsicState() const;
+  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
 
 protected:
   void GetImageFrame(nsIImageFrame** aImageFrame);
@@ -155,7 +154,7 @@ protected:
   nsSize GetWidthHeight();
 };
 
-nsIHTMLContent*
+nsGenericHTMLElement*
 NS_NewHTMLImageElement(nsINodeInfo *aNodeInfo, PRBool aFromParser)
 {
   /*
@@ -186,6 +185,7 @@ nsHTMLImageElement::nsHTMLImageElement(nsINodeInfo *aNodeInfo)
 
 nsHTMLImageElement::~nsHTMLImageElement()
 {
+  DestroyImageLoadingContent();
 }
 
 
@@ -204,7 +204,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLImageElement, nsGenericHTMLElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
 
-NS_IMPL_DOM_CLONENODE(nsHTMLImageElement)
+NS_IMPL_ELEMENT_CLONE(nsHTMLImageElement)
 
 
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, Name, name)
@@ -227,7 +227,7 @@ nsHTMLImageElement::GetImageFrame(nsIImageFrame** aImageFrame)
   if (!GetParent())
     return;
 
-  nsIFrame* frame = GetPrimaryFrame(PR_TRUE);
+  nsIFrame* frame = GetPrimaryFrame(Flush_Frames);
 
   if (frame) {
     CallQueryInterface(frame, aImageFrame);
@@ -280,8 +280,7 @@ nsHTMLImageElement::GetXY()
   document->FlushPendingNotifications(Flush_Layout);
 
   // Get the Frame for this image
-  nsIFrame* frame = nsnull;
-  presShell->GetPrimaryFrameFor(this, &frame);
+  nsIFrame* frame = presShell->GetPrimaryFrameFor(this);
 
   if (!frame) {
     return point;
@@ -289,7 +288,7 @@ nsHTMLImageElement::GetXY()
 
   nsPoint origin(0, 0);
   nsIView* parentView;
-  nsresult rv = frame->GetOffsetFromView(context, origin, &parentView);
+  nsresult rv = frame->GetOffsetFromView(origin, &parentView);
   if (NS_FAILED(rv)) {
     return point;
   }
@@ -341,7 +340,7 @@ nsHTMLImageElement::GetWidthHeight()
 
   if (imageFrame) {
     CallQueryInterface(imageFrame, &frame);
-    NS_WARN_IF_FALSE(frame,"Should not happen - image frame is not frame");
+    NS_ASSERTION(frame,"Should not happen - image frame is not frame");
   }
 
   if (frame) {
@@ -364,22 +363,22 @@ nsHTMLImageElement::GetWidthHeight()
       size.height = NSTwipsToIntPixels(size.height, t2p);
     }
   } else {
-    nsHTMLValue value;
+    const nsAttrValue* value;
     nsCOMPtr<imgIContainer> image;
     if (mCurrentRequest) {
       mCurrentRequest->GetImage(getter_AddRefs(image));
     }
 
-    if (GetHTMLAttribute(nsHTMLAtoms::width,
-                         value) == NS_CONTENT_ATTR_HAS_VALUE) {
-      size.width = value.GetIntValue();
+    if ((value = GetParsedAttr(nsHTMLAtoms::width)) &&
+        value->Type() == nsAttrValue::eInteger) {
+      size.width = value->GetIntegerValue();
     } else if (image) {
       image->GetWidth(&size.width);
     }
 
-    if (GetHTMLAttribute(nsHTMLAtoms::height,
-                         value) == NS_CONTENT_ATTR_HAS_VALUE) {
-      size.height = value.GetIntValue();
+    if ((value = GetParsedAttr(nsHTMLAtoms::height)) &&
+        value->Type() == nsAttrValue::eInteger) {
+      size.height = value->GetIntegerValue();
     } else if (image) {
       image->GetHeight(&size.height);
     }
@@ -425,38 +424,27 @@ nsHTMLImageElement::SetWidth(PRInt32 aWidth)
 }
 
 PRBool
-nsHTMLImageElement::ParseAttribute(nsIAtom* aAttribute,
+nsHTMLImageElement::ParseAttribute(PRInt32 aNamespaceID,
+                                   nsIAtom* aAttribute,
                                    const nsAString& aValue,
                                    nsAttrValue& aResult)
 {
-  if (aAttribute == nsHTMLAtoms::align) {
-    return ParseAlignValue(aValue, aResult);
-  }
-  if (aAttribute == nsHTMLAtoms::src) {
-    static const char* kWhitespace = " \n\r\t\b";
-    aResult.SetTo(nsContentUtils::TrimCharsInSet(kWhitespace, aValue));
-    return PR_TRUE;
-  }
-  if (ParseImageAttribute(aAttribute, aValue, aResult)) {
-    return PR_TRUE;
-  }
-
-  return nsGenericHTMLElement::ParseAttribute(aAttribute, aValue, aResult);
-}
-
-NS_IMETHODIMP
-nsHTMLImageElement::AttributeToString(nsIAtom* aAttribute,
-                                      const nsHTMLValue& aValue,
-                                      nsAString& aResult) const
-{
-  if (aAttribute == nsHTMLAtoms::align) {
-    if (eHTMLUnit_Enumerated == aValue.GetUnit()) {
-      VAlignValueToString(aValue, aResult);
-      return NS_CONTENT_ATTR_HAS_VALUE;
+  if (aNamespaceID == kNameSpaceID_None) {
+    if (aAttribute == nsHTMLAtoms::align) {
+      return ParseAlignValue(aValue, aResult);
+    }
+    if (aAttribute == nsHTMLAtoms::src) {
+      static const char* kWhitespace = " \n\r\t\b";
+      aResult.SetTo(nsContentUtils::TrimCharsInSet(kWhitespace, aValue));
+      return PR_TRUE;
+    }
+    if (ParseImageAttribute(aAttribute, aValue, aResult)) {
+      return PR_TRUE;
     }
   }
 
-  return nsGenericHTMLElement::AttributeToString(aAttribute, aValue, aResult);
+  return nsGenericHTMLElement::ParseAttribute(aNamespaceID, aAttribute, aValue,
+                                              aResult);
 }
 
 static void
@@ -497,34 +485,28 @@ nsHTMLImageElement::IsAttributeMapped(const nsIAtom* aAttribute) const
 }
 
 
-NS_IMETHODIMP
-nsHTMLImageElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const
+nsMapRuleToAttributesFunc
+nsHTMLImageElement::GetAttributeMappingFunction() const
 {
-  aMapRuleFunc = &MapAttributesIntoRule;
-  return NS_OK;
+  return &MapAttributesIntoRule;
 }
 
 
 nsresult
-nsHTMLImageElement::HandleDOMEvent(nsPresContext* aPresContext,
-                                   nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
-                                   PRUint32 aFlags,
-                                   nsEventStatus* aEventStatus)
+nsHTMLImageElement::PreHandleEvent(nsEventChainPreVisitor& aVisitor)
 {
   // If we are a map and get a mouse click, don't let it be handled by
   // the Generic Element as this could cause a click event to fire
   // twice, once by the image frame for the map and once by the Anchor
   // element. (bug 39723)
-  if (NS_MOUSE_LEFT_CLICK == aEvent->message) {
+  if (NS_MOUSE_LEFT_CLICK == aVisitor.mEvent->message) {
     PRBool isMap = PR_FALSE;
     GetIsMap(&isMap);
     if (isMap) {
-      *aEventStatus = nsEventStatus_eConsumeNoDefault;
+      aVisitor.mEventStatus = nsEventStatus_eConsumeNoDefault;
     }
   }
-
-  return nsGenericHTMLElement::HandleDOMEvent(aPresContext, aEvent, aDOMEvent,
-                                              aFlags, aEventStatus);
+  return nsGenericHTMLElement::PreHandleEvent(aVisitor);
 }
 
 PRBool
@@ -565,11 +547,11 @@ nsHTMLImageElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                             nsIAtom* aPrefix, const nsAString& aValue,
                             PRBool aNotify)
 {
-  // If we plan to call ImageURIChanged, we want to do it first so that the
+  // If we plan to call LoadImage, we want to do it first so that the
   // image load kicks off _before_ the reflow triggered by the SetAttr.  But if
   // aNotify is false, we are coming from the parser or some such place; we'll
-  // get our parent set after all the attributes have been set, so we'll do the
-  // image load from SetParent.  Skip the ImageURIChanged call in that case.
+  // get bound after all the attributes have been set, so we'll do the
+  // image load from BindToTree.  Skip the LoadImage call in that case.
   if (aNotify &&
       aNameSpaceID == kNameSpaceID_None && aName == nsHTMLAtoms::src) {
 
@@ -582,7 +564,9 @@ nsHTMLImageElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
 
     nsCOMPtr<imgIRequest> oldCurrentRequest = mCurrentRequest;
 
-    ImageURIChanged(aValue);
+    // Force image loading here, so that we'll try to load the image from
+    // network if it's set to be not cacheable...
+    LoadImage(aValue, PR_TRUE, aNotify);
 
     if (mCurrentRequest && !mPendingRequest &&
         oldCurrentRequest != mCurrentRequest) {
@@ -601,37 +585,33 @@ nsHTMLImageElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                                        aNotify);
 }
 
-void
-nsHTMLImageElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                                PRBool aCompileEventHandlers)
+nsresult
+nsHTMLImageElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                               nsIContent* aBindingParent,
+                               PRBool aCompileEventHandlers)
 {
-  PRBool documentChanging = aDocument && aDocument != GetCurrentDoc();
-  
-  nsGenericHTMLElement::SetDocument(aDocument, aDeep, aCompileEventHandlers);
-  if (documentChanging && GetParent()) {
-    // Our base URI may have changed; claim that our URI changed, and the
-    // nsImageLoadingContent will decide whether a new image load is warranted.
-    nsAutoString uri;
-    nsresult result = GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, uri);
-    if (result == NS_CONTENT_ATTR_HAS_VALUE) {
-      ImageURIChanged(uri);
-    }
+  nsresult rv = nsGenericHTMLElement::BindToTree(aDocument, aParent,
+                                                 aBindingParent,
+                                                 aCompileEventHandlers);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Our base URI may have changed; claim that our URI changed, and the
+  // nsImageLoadingContent will decide whether a new image load is warranted.
+  nsAutoString uri;
+  if (GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, uri)) {
+    // Note: no need to notify here; since we're just now being bound
+    // we don't have any frames or anything yet.
+    LoadImage(uri, PR_FALSE, PR_FALSE);
   }
+
+  return rv;
 }
 
-void
-nsHTMLImageElement::SetParent(nsIContent* aParent)
+PRInt32
+nsHTMLImageElement::IntrinsicState() const
 {
-  nsGenericHTMLElement::SetParent(aParent);
-  if (aParent && IsInDoc()) {
-    // Our base URI may have changed; claim that our URI changed, and the
-    // nsImageLoadingContent will decide whether a new image load is warranted.
-    nsAutoString uri;
-    nsresult result = GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, uri);
-    if (result == NS_CONTENT_ATTR_HAS_VALUE) {
-      ImageURIChanged(uri);
-    }
-  }
+  return nsGenericHTMLElement::IntrinsicState() |
+    nsImageLoadingContent::ImageState();
 }
 
 NS_IMETHODIMP
