@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
+ *   Mats Palmgren <mats.palmgren@bredband.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -41,7 +42,6 @@
 #include "nsIDOMHTMLOptGroupElement.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMEventReceiver.h"
-#include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
 #include "nsStyleConsts.h"
@@ -49,7 +49,6 @@
 #include "nsIFormControl.h"
 #include "nsIForm.h"
 #include "nsIDOMText.h"
-#include "nsITextContent.h"
 #include "nsIDOMNode.h"
 #include "nsGenericElement.h"
 #include "nsIDOMHTMLCollection.h"
@@ -67,8 +66,9 @@
 #include "nsCOMPtr.h"
 #include "nsLayoutAtoms.h"
 #include "nsIEventStateManager.h"
-#include "nsXULAtoms.h"
+#include "nsIDocument.h"
 #include "nsIDOMDocument.h"
+#include "nsContentCreatorFunctions.h"
 
 /**
  * Implementation of &lt;option&gt;
@@ -87,7 +87,7 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLElement::)
+  NS_FORWARD_NSIDOMNODE(nsGenericHTMLElement::)
 
   // nsIDOMElement
   NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLElement::)
@@ -112,45 +112,38 @@ public:
   NS_IMETHOD SetSelectedInternal(PRBool aValue, PRBool aNotify);
 
   // nsIContent
-  nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                   const nsAString& aValue, PRBool aNotify)
+  virtual PRInt32 IntrinsicState() const;
+
+  virtual nsresult UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
+                             PRBool aNotify)
   {
-    return SetAttr(aNameSpaceID, aName, nsnull, aValue, aNotify);
+    nsresult rv = nsGenericHTMLElement::UnsetAttr(aNameSpaceID, aAttribute,
+                                                  aNotify);
+
+    AfterSetAttr(aNameSpaceID, aAttribute, nsnull, aNotify);
+    return rv;
   }
-  virtual nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                           nsIAtom* aPrefix, const nsAString& aValue,
-                           PRBool aNotify);
-  virtual nsresult InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                 PRBool aNotify, PRBool aDeepSetDocument);
-  virtual nsresult AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                 PRBool aDeepSetDocument);
-  virtual nsresult RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
+  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
 
 protected:
-  /**
-   * Get the primary frame associated with this content
-   * @return the primary frame associated with this content
-   */
-  nsIFormControlFrame *GetSelectFrame() const;
-
   /**
    * Get the select content element that contains this option, this
    * intentionally does not return nsresult, all we care about is if
    * there's a select associated with this option or not.
    * @param aSelectElement the select element (out param)
    */
-  void GetSelect(nsIDOMHTMLSelectElement **aSelectElement) const;
+  nsIContent* GetSelect();
 
   /**
-   * Notify the frame that the option text or value or label has changed.
+   * Called when an attribute has just been changed
    */
-  void NotifyTextChanged();
-
+  virtual nsresult AfterSetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                                const nsAString* aValue, PRBool aNotify);
   PRPackedBool mIsInitialized;
   PRPackedBool mIsSelected;
 };
 
-nsIHTMLContent*
+nsGenericHTMLElement*
 NS_NewHTMLOptionElement(nsINodeInfo *aNodeInfo, PRBool aFromParser)
 {
   /*
@@ -202,7 +195,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLOptionElement, nsGenericHTMLElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
 
-NS_IMPL_DOM_CLONENODE(nsHTMLOptionElement)
+NS_IMPL_ELEMENT_CLONE(nsHTMLOptionElement)
 
 
 NS_IMETHODIMP
@@ -211,10 +204,7 @@ nsHTMLOptionElement::GetForm(nsIDOMHTMLFormElement** aForm)
   NS_ENSURE_ARG_POINTER(aForm);
   *aForm = nsnull;
 
-  nsCOMPtr<nsIDOMHTMLSelectElement> selectElement;
-  GetSelect(getter_AddRefs(selectElement));
-
-  nsCOMPtr<nsIFormControl> selectControl(do_QueryInterface(selectElement));
+  nsCOMPtr<nsIFormControl> selectControl = do_QueryInterface(GetSelect());
 
   if (selectControl) {
     selectControl->GetForm(aForm);
@@ -232,7 +222,7 @@ nsHTMLOptionElement::SetSelectedInternal(PRBool aValue, PRBool aNotify)
   if (aNotify) {
     nsIDocument* document = GetCurrentDoc();
     if (document) {
-      mozAutoDocUpdate(document, UPDATE_CONTENT_STATE, aNotify);
+      mozAutoDocUpdate upd(document, UPDATE_CONTENT_STATE, aNotify);
       document->ContentStatesChanged(this, nsnull, NS_EVENT_STATE_CHECKED);
     }
   }
@@ -250,10 +240,9 @@ nsHTMLOptionElement::SetValue(const nsAString& aValue)
 NS_IMETHODIMP
 nsHTMLOptionElement::GetValue(nsAString& aValue)
 {
-  nsresult rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, aValue);
   // If the value attr is there, that is *exactly* what we use.  If it is
   // not, we compress whitespace .text.
-  if (NS_CONTENT_ATTR_NOT_THERE == rv) {
+  if (!GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, aValue)) {
     GetText(aValue);
   }
 
@@ -290,9 +279,7 @@ nsHTMLOptionElement::SetSelected(PRBool aValue)
 {
   // Note: The select content obj maintains all the PresState
   // so defer to it to get the answer
-  nsCOMPtr<nsIDOMHTMLSelectElement> selectElement;
-  GetSelect(getter_AddRefs(selectElement));
-  nsCOMPtr<nsISelectElement> selectInt(do_QueryInterface(selectElement));
+  nsCOMPtr<nsISelectElement> selectInt = do_QueryInterface(GetSelect());
   if (selectInt) {
     PRInt32 index;
     GetIndex(&index);
@@ -320,9 +307,8 @@ nsHTMLOptionElement::GetIndex(PRInt32* aIndex)
   *aIndex = -1; // -1 indicates the index was not found
 
   // Get our containing select content object.
-  nsCOMPtr<nsIDOMHTMLSelectElement> selectElement;
-
-  GetSelect(getter_AddRefs(selectElement));
+  nsCOMPtr<nsIDOMHTMLSelectElement> selectElement =
+    do_QueryInterface(GetSelect());
 
   if (selectElement) {
     // Get the options from the select object.
@@ -368,27 +354,10 @@ nsHTMLOptionElement::GetAttributeChangeHint(const nsIAtom* aAttribute,
 NS_IMETHODIMP
 nsHTMLOptionElement::GetText(nsAString& aText)
 {
-  PRUint32 i, numNodes = GetChildCount();
-
-  aText.Truncate();
-
   nsAutoString text;
-  for (i = 0; i < numNodes; i++) {
-    nsCOMPtr<nsIDOMText> domText(do_QueryInterface(GetChildAt(i)));
-
-    if (domText) {
-      nsresult rv = domText->GetData(text);
-      if (NS_FAILED(rv)) {
-        aText.Truncate();
-        return rv;
-      }
-
-      aText.Append(text);
-    }
-  }
+  nsContentUtils::GetNodeTextContent(this, PR_FALSE, text);
 
   // XXX No CompressWhitespace for nsAString.  Sad.
-  text = aText;
   text.CompressWhitespace(PR_TRUE, PR_TRUE);
   aText = text;
 
@@ -398,147 +367,81 @@ nsHTMLOptionElement::GetText(nsAString& aText)
 NS_IMETHODIMP
 nsHTMLOptionElement::SetText(const nsAString& aText)
 {
-  PRUint32 i, numNodes = GetChildCount();
-  PRBool usedExistingTextNode = PR_FALSE;  // Do we need to create a text node?
-  nsresult rv = NS_OK;
-
-  for (i = 0; i < numNodes; i++) {
-    nsCOMPtr<nsIDOMText> domText(do_QueryInterface(GetChildAt(i)));
-
-    if (domText) {
-      rv = domText->SetData(aText);
-
-      if (NS_SUCCEEDED(rv)) {
-        // If we used an existing node, the notification will not happen (the
-        // notification typically happens in AppendChildTo).
-        NotifyTextChanged();
-        usedExistingTextNode = PR_TRUE;
-      }
-
-      break;
-    }
-  }
-
-  if (!usedExistingTextNode) {
-    nsCOMPtr<nsITextContent> text;
-    rv = NS_NewTextNode(getter_AddRefs(text));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    text->SetText(aText, PR_TRUE);
-
-    rv = AppendChildTo(text, PR_TRUE, PR_FALSE);
-  }
-
-  return rv;
+  return nsContentUtils::SetNodeTextContent(this, aText, PR_TRUE);
 }
 
-void
-nsHTMLOptionElement::NotifyTextChanged()
+PRInt32
+nsHTMLOptionElement::IntrinsicState() const
 {
-  // No need to flush here, if there's no frame yet we don't need to
-  // force it to be created just to update the selection in it.
-  nsIFormControlFrame* fcFrame = GetSelectFrame();
-
-  if (fcFrame) {
-    nsISelectControlFrame* selectFrame = nsnull;
-
-    CallQueryInterface(fcFrame, &selectFrame);
-
-    if (selectFrame) {
-      selectFrame->OnOptionTextChanged(this);
-    }
-  }
-}
-
-nsresult
-nsHTMLOptionElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                             nsIAtom* aPrefix, const nsAString& aValue,
-                             PRBool aNotify)
-{
-  nsresult rv = nsGenericHTMLElement::SetAttr(aNameSpaceID, aName, aPrefix,
-                                              aValue, aNotify);
-  if (NS_SUCCEEDED(rv) && aNotify && aName == nsHTMLAtoms::label &&
-      aNameSpaceID == kNameSpaceID_None) {
-    // XXX Why does this only happen to the combobox?  and what about
-    // when the text gets set and label is blank?
-    NotifyTextChanged();
+  PRInt32 state = nsGenericHTMLElement::IntrinsicState();
+  // Nasty hack because we need to call an interface method, and one that
+  // toggles some of our hidden internal state at that!  Would that we could
+  // use |mutable|.
+  PRBool selected;
+  NS_CONST_CAST(nsHTMLOptionElement*, this)->GetSelected(&selected);
+  if (selected) {
+    state |= NS_EVENT_STATE_CHECKED;
   }
 
-  return rv;
-}
-
-//
-// Override nsIContent children changing methods so we can detect when our text
-// is changing
-//
-nsresult
-nsHTMLOptionElement::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                   PRBool aNotify, PRBool aDeepSetDocument)
-{
-  nsresult rv = nsGenericHTMLElement::InsertChildAt(aKid, aIndex, aNotify,
-                                                    aDeepSetDocument);
-  NotifyTextChanged();
-  return rv;
-}
-
-nsresult
-nsHTMLOptionElement::AppendChildTo(nsIContent* aKid, PRBool aNotify, PRBool aDeepSetDocument)
-{
-  nsresult rv = nsGenericHTMLElement::AppendChildTo(aKid, aNotify,
-                                                    aDeepSetDocument);
-  NotifyTextChanged();
-  return rv;
-}
-
-nsresult
-nsHTMLOptionElement::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
-{
-  nsresult rv = nsGenericHTMLElement::RemoveChildAt(aIndex, aNotify);
-  NotifyTextChanged();
-  return rv;
-}
-
-
-// Options don't have frames - get the select content node
-// then call nsGenericHTMLElement::GetFormControlFrameFor()
-
-nsIFormControlFrame *
-nsHTMLOptionElement::GetSelectFrame() const
-{
-  if (!GetParent()) {
-    return nsnull;
+  // Also calling a non-const interface method (for :default)
+  NS_CONST_CAST(nsHTMLOptionElement*, this)->GetDefaultSelected(&selected);
+  if (selected) {
+    state |= NS_EVENT_STATE_DEFAULT;
   }
 
-  nsIDocument* currentDoc = GetCurrentDoc();
-  if (!currentDoc) {
-    return nsnull;
+  PRBool disabled;
+  GetBoolAttr(nsHTMLAtoms::disabled, &disabled);
+  if (disabled) {
+    state |= NS_EVENT_STATE_DISABLED;
+    state &= ~NS_EVENT_STATE_ENABLED;
+  } else {
+    state &= ~NS_EVENT_STATE_DISABLED;
+    state |= NS_EVENT_STATE_ENABLED;
   }
 
-  nsCOMPtr<nsIDOMHTMLSelectElement> selectElement;
-
-  GetSelect(getter_AddRefs(selectElement));
-
-  nsCOMPtr<nsIContent> selectContent(do_QueryInterface(selectElement));
-
-  if (!selectContent) {
-    return nsnull;
-  }
-
-  return GetFormControlFrameFor(selectContent, currentDoc, PR_FALSE);
+  return state;
 }
 
 // Get the select content element that contains this option
-void
-nsHTMLOptionElement::GetSelect(nsIDOMHTMLSelectElement **aSelectElement) const
+nsIContent*
+nsHTMLOptionElement::GetSelect()
 {
-  *aSelectElement = nsnull;
-
-  for (nsIContent* parent = GetParent(); parent; parent = parent->GetParent()) {
-    CallQueryInterface(parent, aSelectElement);
-    if (*aSelectElement) {
+  nsIContent* parent = this;
+  while ((parent = parent->GetParent()) &&
+         parent->IsNodeOfType(eHTML)) {
+    if (parent->Tag() == nsHTMLAtoms::select) {
+      return parent;
+    }
+    if (parent->Tag() != nsHTMLAtoms::optgroup) {
       break;
     }
   }
+  
+  return nsnull;
+}
+
+nsresult
+nsHTMLOptionElement::AfterSetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                                  const nsAString* aValue, PRBool aNotify)
+{
+  if (aNotify && aNameSpaceID == kNameSpaceID_None &&
+      (aName == nsHTMLAtoms::disabled || aName == nsHTMLAtoms::selected)) {
+    PRInt32 states;
+    if (aName == nsHTMLAtoms::disabled) {
+      states = NS_EVENT_STATE_DISABLED | NS_EVENT_STATE_ENABLED;
+    } else {
+      states = NS_EVENT_STATE_DEFAULT;
+    }
+    
+    nsIDocument* document = GetCurrentDoc();
+    if (document) {
+      mozAutoDocUpdate upd(document, UPDATE_CONTENT_STATE, PR_TRUE);
+      document->ContentStatesChanged(this, nsnull, states);
+    }
+  }
+
+  return nsGenericHTMLElement::AfterSetAttr(aNameSpaceID, aName, aValue,
+                                            aNotify);
 }
 
 NS_IMETHODIMP    
@@ -554,9 +457,9 @@ nsHTMLOptionElement::Initialize(JSContext* aContext,
     JSString* jsstr = JS_ValueToString(aContext, argv[0]);
     if (jsstr) {
       // Create a new text node and append it to the option
-      nsCOMPtr<nsITextContent> textContent;
-
-      result = NS_NewTextNode(getter_AddRefs(textContent));
+      nsCOMPtr<nsIContent> textContent;
+      result = NS_NewTextNode(getter_AddRefs(textContent),
+                              mNodeInfo->NodeInfoManager());
       if (NS_FAILED(result)) {
         return result;
       }
@@ -566,7 +469,7 @@ nsHTMLOptionElement::Initialize(JSContext* aContext,
                            JS_GetStringLength(jsstr),
                            PR_FALSE);
       
-      result = AppendChildTo(textContent, PR_FALSE, PR_FALSE);
+      result = AppendChildTo(textContent, PR_FALSE);
       if (NS_FAILED(result)) {
         return result;
       }
