@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -36,12 +37,10 @@
  * ***** END LICENSE BLOCK ***** */
 #include "nsIDOMHTMLScriptElement.h"
 #include "nsIDOMEventReceiver.h"
-#include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
 #include "nsStyleConsts.h"
 #include "nsPresContext.h"
-#include "nsITextContent.h"
 #include "nsIDocument.h"
 #include "nsIScriptLoader.h"
 #include "nsIScriptLoaderObserver.h"
@@ -55,10 +54,13 @@
 #include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIXPConnect.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 #include "nsIScriptEventHandler.h"
 #include "nsIDOMDocument.h"
-
+#include "nsEventDispatcher.h"
+#include "nsContentErrors.h"
+#include "nsIArray.h"
+#include "nsDOMJSUtils.h"
 
 //
 // Helper class used to support <SCRIPT FOR=object EVENT=handler ...>
@@ -143,7 +145,7 @@ nsresult nsHTMLScriptEventHandler::ParseEventString(const nsAString &aValue)
   }
 
   // Javascript expects all argument names to be ASCII.
-  NS_LossyConvertUCS2toASCII sig(Substring(next, end));
+  NS_LossyConvertUTF16toASCII sig(Substring(next, end));
 
   // Store each (comma separated) argument in mArgNames
   mArgNames.ParseString(sig.get(), ",");
@@ -208,12 +210,13 @@ nsHTMLScriptEventHandler::Invoke(nsISupports *aTargetObject,
   // Get the script context...
   nsCOMPtr<nsIDOMDocument> domdoc;
   nsCOMPtr<nsIScriptContext> scriptContext;
+  nsIScriptGlobalObject *sgo = nsnull;
 
   mOuter->GetOwnerDocument(getter_AddRefs(domdoc));
 
   nsCOMPtr<nsIDocument> doc(do_QueryInterface(domdoc));
   if (doc) {
-    nsIScriptGlobalObject *sgo = doc->GetScriptGlobalObject();
+    sgo = doc->GetScriptGlobalObject();
     if (sgo) {
       scriptContext = sgo->GetContext();
     }
@@ -226,9 +229,10 @@ nsHTMLScriptEventHandler::Invoke(nsISupports *aTargetObject,
   // wrap the target object...
   JSContext *cx = (JSContext *)scriptContext->GetNativeContext();
   JSObject *scriptObject = nsnull;
+  JSObject *scope = sgo->GetGlobalJSObject();
 
   nsCOMPtr<nsIXPConnectJSObjectHolder> holder;
-  nsContentUtils::XPConnect()->WrapNative(cx, ::JS_GetGlobalObject(cx),
+  nsContentUtils::XPConnect()->WrapNative(cx, scope,
                                           aTargetObject,
                                           NS_GET_IID(nsISupports),
                                           getter_AddRefs(holder));
@@ -296,10 +300,17 @@ nsHTMLScriptEventHandler::Invoke(nsISupports *aTargetObject,
     return rv;
   }
 
+  // Create an nsIArray for the args (the JS context will efficiently
+  // re-fetch the jsvals from this object)
+  nsCOMPtr<nsIArray> argarray;
+  rv = NS_CreateJSArgv(cx, aArgCount, (jsval *)aArgs, getter_AddRefs(argarray));
+  if (NS_FAILED(rv))
+    return rv;
+
   // Invoke the event handler script...
-  jsval dummy;
-  return scriptContext->CallEventHandler(scriptObject, (JSObject *)funcObject,
-                                         aArgCount, (jsval *)aArgs, &dummy);
+  nsCOMPtr<nsIVariant> ret;
+  return scriptContext->CallEventHandler(aTargetObject, scope, funcObject,
+                                         argarray, getter_AddRefs(ret));
 }
 
 
@@ -309,14 +320,14 @@ class nsHTMLScriptElement : public nsGenericHTMLElement,
                             public nsIScriptElement
 {
 public:
-  nsHTMLScriptElement(nsINodeInfo *aNodeInfo);
+  nsHTMLScriptElement(nsINodeInfo *aNodeInfo, PRBool aFromParser);
   virtual ~nsHTMLScriptElement();
 
   // nsISupports
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLElement::)
+  NS_FORWARD_NSIDOMNODE(nsGenericHTMLElement::)
 
   // nsIDOMElement
   NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLElement::)
@@ -337,6 +348,8 @@ public:
   virtual void GetScriptCharset(nsAString& charset); 
   virtual void SetScriptLineNumber(PRUint32 aLineNumber);
   virtual PRUint32 GetScriptLineNumber();
+  virtual void SetIsMalformed();
+  virtual PRBool IsMalformed();
 
   // nsIContent
   nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
@@ -347,15 +360,19 @@ public:
   virtual nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                            nsIAtom* aPrefix, const nsAString& aValue,
                            PRBool aNotify);
-  virtual void SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                           PRBool aCompileEventHandlers);
+
+  virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                              nsIContent* aBindingParent,
+                              PRBool aCompileEventHandlers);
   virtual nsresult InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                 PRBool aNotify, PRBool aDeepSetDocument);
-  virtual nsresult AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                 PRBool aDeepSetDocument);
+                                 PRBool aNotify);
 
   virtual nsresult GetInnerHTML(nsAString& aInnerHTML);
   virtual nsresult SetInnerHTML(const nsAString& aInnerHTML);
+  virtual void DoneAddingChildren(PRBool aHaveNotified);
+  virtual PRBool IsDoneAddingChildren();
+
+  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
 
 protected:
   PRBool IsOnloadEventForWindow();
@@ -363,9 +380,11 @@ protected:
   PRUint32 mLineNumber;
   PRPackedBool mIsEvaluated;
   PRPackedBool mEvaluating;
+  PRPackedBool mDoneAddingChildren;
+  PRPackedBool mMalformed;
 
   // Pointer to the script handler helper object (OWNING reference)
-  nsHTMLScriptEventHandler *mScriptEventHandler;
+  nsCOMPtr<nsHTMLScriptEventHandler> mScriptEventHandler;
 
   /**
    * Processes the script if it's in the document-tree and links to or
@@ -384,21 +403,22 @@ protected:
 };
 
 
-NS_IMPL_NS_NEW_HTML_ELEMENT(Script)
+NS_IMPL_NS_NEW_HTML_ELEMENT_CHECK_PARSER(Script)
 
 
-nsHTMLScriptElement::nsHTMLScriptElement(nsINodeInfo *aNodeInfo)
-  : nsGenericHTMLElement(aNodeInfo)
+nsHTMLScriptElement::nsHTMLScriptElement(nsINodeInfo *aNodeInfo,
+                                         PRBool aFromParser)
+  : nsGenericHTMLElement(aNodeInfo),
+    mLineNumber(0),
+    mIsEvaluated(PR_FALSE),
+    mEvaluating(PR_FALSE),
+    mDoneAddingChildren(!aFromParser),
+    mMalformed(PR_FALSE)
 {
-  mLineNumber = 0;
-  mIsEvaluated = PR_FALSE;
-  mEvaluating = PR_FALSE;
-  mScriptEventHandler = nsnull;
 }
 
 nsHTMLScriptElement::~nsHTMLScriptElement()
 {
-  NS_IF_RELEASE(mScriptEventHandler);
 }
 
 
@@ -438,22 +458,28 @@ nsHTMLScriptElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
   return rv;
 }
 
-void
-nsHTMLScriptElement::SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                                 PRBool aCompileEventHandlers)
+nsresult
+nsHTMLScriptElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                                nsIContent* aBindingParent,
+                                PRBool aCompileEventHandlers)
 {
-  nsGenericHTMLElement::SetDocument(aDocument, aDeep, aCompileEventHandlers);
+  nsresult rv = nsGenericHTMLElement::BindToTree(aDocument, aParent,
+                                                 aBindingParent,
+                                                 aCompileEventHandlers);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   if (aDocument) {
     MaybeProcessScript();
   }
+
+  return rv;
 }
 
 nsresult
 nsHTMLScriptElement::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                   PRBool aNotify, PRBool aDeepSetDocument)
+                                   PRBool aNotify)
 {
-  nsresult rv = nsGenericHTMLElement::InsertChildAt(aKid, aIndex, aNotify,
-                                                    aDeepSetDocument);
+  nsresult rv = nsGenericHTMLElement::InsertChildAt(aKid, aIndex, aNotify);
   if (NS_SUCCEEDED(rv) && aNotify) {
     MaybeProcessScript();
   }
@@ -462,39 +488,27 @@ nsHTMLScriptElement::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
 }
 
 nsresult
-nsHTMLScriptElement::AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                   PRBool aDeepSetDocument)
+nsHTMLScriptElement::Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const
 {
-  nsresult rv = nsGenericHTMLElement::AppendChildTo(aKid, aNotify,
-                                                    aDeepSetDocument);
-  if (NS_SUCCEEDED(rv) && aNotify) {
-    MaybeProcessScript();
-  }
+  *aResult = nsnull;
 
-  return rv;
-}
-
-nsresult
-nsHTMLScriptElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
-{
-  *aReturn = nsnull;
-
-  nsHTMLScriptElement* it = new nsHTMLScriptElement(mNodeInfo);
+  nsHTMLScriptElement* it = new nsHTMLScriptElement(aNodeInfo, PR_FALSE);
   if (!it) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  nsCOMPtr<nsIDOMNode> kungFuDeathGrip(it);
-
-  CopyInnerTo(it, aDeep);
+  nsCOMPtr<nsINode> kungFuDeathGrip = it;
+  nsresult rv = CopyInnerTo(it);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // The clone should be marked evaluated if we are.  It should also be marked
   // evaluated if we're evaluating, to handle the case when this script node's
   // script clones the node.
   it->mIsEvaluated = mIsEvaluated || mEvaluating;
   it->mLineNumber = mLineNumber;
+  it->mMalformed = mMalformed;
 
-  kungFuDeathGrip.swap(*aReturn);
+  kungFuDeathGrip.swap(*aResult);
 
   return NS_OK;
 }
@@ -502,14 +516,14 @@ nsHTMLScriptElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 NS_IMETHODIMP
 nsHTMLScriptElement::GetText(nsAString& aValue)
 {
-  GetContentsAsText(aValue);
+  nsContentUtils::GetNodeTextContent(this, PR_FALSE, aValue);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsHTMLScriptElement::SetText(const nsAString& aValue)
 {
-  return ReplaceContentsWithText(aValue, PR_TRUE);
+  return nsContentUtils::SetNodeTextContent(this, aValue, PR_TRUE);
 }
 
 
@@ -518,19 +532,32 @@ NS_IMPL_BOOL_ATTR(nsHTMLScriptElement, Defer, defer)
 NS_IMPL_URI_ATTR(nsHTMLScriptElement, Src, src)
 NS_IMPL_STRING_ATTR(nsHTMLScriptElement, Type, type)
 NS_IMPL_STRING_ATTR(nsHTMLScriptElement, HtmlFor, _for)
-NS_IMPL_STRING_ATTR(nsHTMLScriptElement, Event, _event)
+NS_IMPL_STRING_ATTR(nsHTMLScriptElement, Event, event)
 
 nsresult
 nsHTMLScriptElement::GetInnerHTML(nsAString& aInnerHTML)
 {
-  GetContentsAsText(aInnerHTML);
+  nsContentUtils::GetNodeTextContent(this, PR_FALSE, aInnerHTML);
   return NS_OK;
 }
 
 nsresult
 nsHTMLScriptElement::SetInnerHTML(const nsAString& aInnerHTML)
 {
-  return ReplaceContentsWithText(aInnerHTML, PR_TRUE);
+  return nsContentUtils::SetNodeTextContent(this, aInnerHTML, PR_TRUE);
+}
+
+void
+nsHTMLScriptElement::DoneAddingChildren(PRBool aHaveNotified)
+{
+  mDoneAddingChildren = PR_TRUE;
+  MaybeProcessScript();
+}
+
+PRBool
+nsHTMLScriptElement::IsDoneAddingChildren()
+{
+  return mDoneAddingChildren;
 }
 
 // variation of this code in nsSVGScriptElement - check if changes
@@ -548,7 +575,7 @@ nsHTMLScriptElement::ScriptAvailable(nsresult aResult,
 {
   if (!aIsInline && NS_FAILED(aResult)) {
     nsEventStatus status = nsEventStatus_eIgnore;
-    nsScriptErrorEvent event(NS_SCRIPT_ERROR);
+    nsScriptErrorEvent event(PR_TRUE, NS_LOAD_ERROR);
 
     event.lineNr = aLineNo;
 
@@ -558,11 +585,12 @@ nsHTMLScriptElement::ScriptAvailable(nsresult aResult,
     nsCAutoString spec;
     aURI->GetSpec(spec);
 
-    NS_ConvertUTF8toUCS2 fileName(spec);
+    NS_ConvertUTF8toUTF16 fileName(spec);
     event.fileName = fileName.get();
 
     nsCOMPtr<nsPresContext> presContext = GetPresContext();
-    HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
+    nsEventDispatcher::Dispatch(NS_STATIC_CAST(nsIContent*, this), presContext,
+                                &event, nsnull, &status);
   }
 
   return NS_OK;
@@ -581,10 +609,16 @@ nsHTMLScriptElement::ScriptEvaluated(nsresult aResult,
   nsresult rv = NS_OK;
   if (!aIsInline) {
     nsEventStatus status = nsEventStatus_eIgnore;
-    nsEvent event(NS_SUCCEEDED(aResult) ? NS_SCRIPT_LOAD : NS_SCRIPT_ERROR);
+    PRUint32 type = NS_SUCCEEDED(aResult) ? NS_LOAD : NS_LOAD_ERROR;
+    nsEvent event(PR_TRUE, type);
+    if (type == NS_LOAD) {
+      // Load event doesn't bubble.
+      event.flags |= NS_EVENT_FLAG_CANT_BUBBLE;
+    }
+
     nsCOMPtr<nsPresContext> presContext = GetPresContext();
-    rv = HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT,
-                        &status);
+    nsEventDispatcher::Dispatch(NS_STATIC_CAST(nsIContent*, this), presContext,
+                                &event, nsnull, &status);
   }
 
   return rv;
@@ -634,13 +668,25 @@ nsHTMLScriptElement::GetScriptLineNumber()
   return mLineNumber;
 }
 
+void
+nsHTMLScriptElement::SetIsMalformed()
+{
+  mMalformed = PR_TRUE;
+}
+
+PRBool
+nsHTMLScriptElement::IsMalformed()
+{
+  return mMalformed;
+}
+
 // variation of this code in nsSVGScriptElement - check if changes
 // need to be transfered when modifying
 
 void
 nsHTMLScriptElement::MaybeProcessScript()
 {
-  if (mIsEvaluated || mEvaluating || !IsInDoc() || !GetParent()) {
+  if (mIsEvaluated || mEvaluating || !mDoneAddingChildren || !IsInDoc()) {
     return;
   }
 
@@ -668,15 +714,13 @@ nsHTMLScriptElement::MaybeProcessScript()
         return;
       }
 
-      NS_ADDREF(mScriptEventHandler);
-
       // The script-loader will make sure that the script is not evaluated
       // right away.
     }
 
     if (mScriptEventHandler) {
       nsAutoString event_val;
-      GetAttr(kNameSpaceID_None, nsHTMLAtoms::_event, event_val);
+      GetAttr(kNameSpaceID_None, nsHTMLAtoms::event, event_val);
       mScriptEventHandler->ParseEventString(event_val);
     }
   }
