@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
+ *   Mats Palmgren <mats.palmgren@bredband.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -42,12 +43,12 @@
 #include "nsIDOMNSXBLFormControl.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMEventReceiver.h"
-#include "nsIHTMLContent.h"
-#include "nsITextContent.h"
+#include "nsContentCreatorFunctions.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
 #include "nsStyleConsts.h"
 #include "nsPresContext.h"
+#include "nsLayoutUtils.h"
 #include "nsMappedAttributes.h"
 #include "nsIForm.h"
 #include "nsIFormSubmission.h"
@@ -62,13 +63,14 @@
 #include "nsIDOMHTMLOptionsCollection.h"
 #include "nsIDOMNSHTMLOptionCollectn.h"
 #include "nsGUIEvent.h"
+#include "nsIPrivateDOMEvent.h"
 #include "nsIBoxObject.h"
 #include "nsIDOMNSDocument.h"
 #include "nsIDOMDocumentEvent.h"
 
 // PresState
 #include "nsXPCOM.h"
-#include "nsIPresState.h"
+#include "nsPresState.h"
 #include "nsIComponentManager.h"
 #include "nsCheapSets.h"
 
@@ -76,11 +78,13 @@
 #include "nsIDocument.h"
 #include "nsIPresShell.h"
 #include "nsIFormControlFrame.h"
+#include "nsIComboboxControlFrame.h"
+#include "nsIListControlFrame.h"
 #include "nsIFrame.h"
 
 #include "nsDOMError.h"
 #include "nsRuleData.h"
-
+#include "nsEventDispatcher.h"
 
 class nsHTMLSelectElement;
 
@@ -113,18 +117,45 @@ public:
    * @param aOption the option to insert
    * @param aIndex the index to insert at
    */
-  nsresult InsertElementAt(nsIDOMHTMLOptionElement* aOption, PRInt32 aIndex);
+  PRBool InsertOptionAt(nsIDOMHTMLOptionElement* aOption, PRInt32 aIndex)
+  {
+    return mElements.InsertObjectAt(aOption, aIndex);
+  }
+
   /**
    * Remove an option
    * @param aIndex the index of the option to remove
    */
-  nsresult RemoveElementAt(PRInt32 aIndex);
+  void RemoveOptionAt(PRInt32 aIndex)
+  {
+    mElements.RemoveObjectAt(aIndex);
+  }
+
   /**
    * Get the option at the index
    * @param aIndex the index
    * @param aReturn the option returned [OUT]
    */
-  nsIDOMHTMLOptionElement *ItemAsOption(PRInt32 aIndex);
+  nsIDOMHTMLOptionElement *ItemAsOption(PRInt32 aIndex)
+  {
+    return mElements.SafeObjectAt(aIndex);
+  }
+
+  /**
+   * Clears out all options
+   */
+  void Clear()
+  {
+    mElements.Clear();
+  }
+
+  /**
+   * Append an option to end of array
+   */
+  PRBool AppendOption(nsIDOMHTMLOptionElement* aOption)
+  {
+    return mElements.AppendObject(aOption);
+  }
 
   /**
    * Drop the reference to the select.  Called during select destruction.
@@ -200,7 +231,7 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLFormElement::)
+  NS_FORWARD_NSIDOMNODE(nsGenericHTMLFormElement::)
 
   // nsIDOMElement
   NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLFormElement::)
@@ -218,42 +249,41 @@ public:
   NS_DECL_NSIDOMNSXBLFORMCONTROL
 
   // nsIContent
-  virtual nsresult InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                 PRBool aNotify, PRBool aDeepSetDocument);
-  virtual nsresult AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                 PRBool aDeepSetDocument);
-  virtual nsresult RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
-
-  virtual nsresult HandleDOMEvent(nsPresContext* aPresContext,
-                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
-                                  PRUint32 aFlags,
-                                  nsEventStatus* aEventStatus);
+  virtual nsresult PreHandleEvent(nsEventChainPreVisitor& aVisitor);
 
   virtual void SetFocus(nsPresContext* aPresContext);
   virtual PRBool IsFocusable(PRInt32 *aTabIndex = nsnull);
+  virtual nsresult InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
+                                 PRBool aNotify);
+  virtual nsresult RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
 
   // Overriden nsIFormControl methods
-  NS_IMETHOD_(PRInt32) GetType() { return NS_FORM_SELECT; }
+  NS_IMETHOD_(PRInt32) GetType() const { return NS_FORM_SELECT; }
   NS_IMETHOD Reset();
   NS_IMETHOD SubmitNamesValues(nsIFormSubmission* aFormSubmission,
                                nsIContent* aSubmitElement);
   NS_IMETHOD SaveState();
-  virtual PRBool RestoreState(nsIPresState* aState);
+  virtual PRBool RestoreState(nsPresState* aState);
 
   // nsISelectElement
   NS_DECL_NSISELECTELEMENT
 
-  virtual void DoneAddingChildren();
+  virtual nsresult UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
+                             PRBool aNotify);
+  
+  virtual void DoneAddingChildren(PRBool aHaveNotified);
   virtual PRBool IsDoneAddingChildren();
 
-  virtual PRBool ParseAttribute(nsIAtom* aAttribute,
+  virtual PRBool ParseAttribute(PRInt32 aNamespaceID,
+                                nsIAtom* aAttribute,
                                 const nsAString& aValue,
                                 nsAttrValue& aResult);
-  NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
+  virtual nsMapRuleToAttributesFunc GetAttributeMappingFunction() const;
   virtual nsChangeHint GetAttributeChangeHint(const nsIAtom* aAttribute,
                                               PRInt32 aModType) const;
   NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* aAttribute) const;
 
+  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
 
 protected:
   // Helper Methods
@@ -396,8 +426,34 @@ protected:
    */
   void DispatchDOMEvent(const nsAString& aName);
 
+  /**
+   * Is this a combobox?
+   */
+  PRBool IsCombobox() {
+    PRBool isMultiple = PR_TRUE;
+    PRInt32 size = 1;
+    GetSize(&size);
+    GetMultiple(&isMultiple);
+    return !isMultiple && size <= 1;
+  }
+
+  /**
+   * Helper method for dispatching ContentReset notifications to list
+   * and combo box frames.
+   */
+  void DispatchContentReset();
+
+  /**
+   * Rebuilds the options array from scratch as a fallback in error cases.
+   */
+  void RebuildOptionsArray();
+
+#ifdef DEBUG
+  void VerifyOptionsArray();
+#endif
+
   /** The options[] array */
-  nsHTMLOptionCollection* mOptions;
+  nsRefPtr<nsHTMLOptionCollection> mOptions;
   /** false if the parser is in the middle of adding children. */
   PRBool    mIsDoneAddingChildren;
   /** The number of non-options as children of the select */
@@ -413,7 +469,7 @@ protected:
    * The temporary restore state in case we try to restore before parser is
    * done adding options
    */
-  nsSelectState* mRestoreState;
+  nsRefPtr<nsSelectState> mRestoreState;
 };
 
 
@@ -434,22 +490,20 @@ nsHTMLSelectElement::nsHTMLSelectElement(nsINodeInfo *aNodeInfo,
     mIsDoneAddingChildren(!aFromParser),
     mNonOptionChildren(0),
     mOptGroupCount(0),
-    mSelectedIndex(-1),
-    mRestoreState(nsnull)
+    mSelectedIndex(-1)
 {
+  // FIXME: Bug 328908, set mOptions in an Init function and get rid of null
+  // checks.
+
   // DoneAddingChildren() will be called later if it's from the parser,
   // otherwise it is
-
-  NS_IF_ADDREF(mOptions);
 }
 
 nsHTMLSelectElement::~nsHTMLSelectElement()
 {
   if (mOptions) {
     mOptions->DropReference();
-    NS_RELEASE(mOptions);
   }
-  NS_IF_RELEASE(mRestoreState);
 }
 
 // ISupports
@@ -472,7 +526,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_END
 // nsIDOMHTMLSelectElement
 
 
-NS_IMPL_DOM_CLONENODE(nsHTMLSelectElement)
+NS_IMPL_ELEMENT_CLONE(nsHTMLSelectElement)
 
 
 NS_IMETHODIMP
@@ -481,35 +535,58 @@ nsHTMLSelectElement::GetForm(nsIDOMHTMLFormElement** aForm)
   return nsGenericHTMLFormElement::GetForm(aForm);
 }
 
-
-// nsIContent
 nsresult
-nsHTMLSelectElement::AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                   PRBool aDeepSetDocument)
+nsHTMLSelectElement::InsertChildAt(nsIContent* aKid,
+                                   PRUint32 aIndex,
+                                   PRBool aNotify)
 {
-  WillAddOptions(aKid, this, GetChildCount());
+  PRUint32 prevOptGroups = mOptGroupCount;
 
-  // Actually perform the append
-  return nsGenericHTMLFormElement::AppendChildTo(aKid, aNotify,
-                                                 aDeepSetDocument);
-}
+  nsresult rv = WillAddOptions(aKid, this, aIndex);
+  PRBool rebuild = NS_FAILED(rv);
 
-nsresult
-nsHTMLSelectElement::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                   PRBool aNotify, PRBool aDeepSetDocument)
-{
-  WillAddOptions(aKid, this, aIndex);
+  rv = nsGenericHTMLFormElement::InsertChildAt(aKid, aIndex, aNotify);
+  if (rebuild || NS_FAILED(rv)) {
+    RebuildOptionsArray();
+    return rv;
+  }
 
-  return nsGenericHTMLFormElement::InsertChildAt(aKid, aIndex, aNotify,
-                                                 aDeepSetDocument);
+  if (mOptGroupCount && !prevOptGroups) {
+    // FIXME: Bug 328907, get rid of this event
+    DispatchDOMEvent(NS_LITERAL_STRING("selectHasGroups"));
+  }
+
+#ifdef DEBUG
+  VerifyOptionsArray();
+#endif
+
+  return NS_OK;
 }
 
 nsresult
 nsHTMLSelectElement::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
 {
-  WillRemoveOptions(this, aIndex);
+  PRUint32 prevOptGroups = mOptGroupCount;
 
-  return nsGenericHTMLFormElement::RemoveChildAt(aIndex, aNotify);
+  nsresult rv = WillRemoveOptions(this, aIndex);
+  PRBool rebuild = NS_FAILED(rv);
+
+  rv = nsGenericHTMLFormElement::RemoveChildAt(aIndex, aNotify);
+  if (rebuild || NS_FAILED(rv)) {
+    RebuildOptionsArray();
+    return rv;
+  }
+
+  if (!mOptGroupCount && prevOptGroups) {
+    // FIXME: Bug 328907, get rid of this event
+    DispatchDOMEvent(NS_LITERAL_STRING("selectHasNoGroups"));
+  }
+
+#ifdef DEBUG
+  VerifyOptionsArray();
+#endif
+
+  return NS_OK;
 }
 
 
@@ -589,22 +666,18 @@ nsHTMLSelectElement::PrintOptions(nsIContent* aOptions, PRInt32 tabs)
   if (elem) {
     nsAutoString s;
     elem->GetTagName(s);
-    printf("<%s>\n", NS_ConvertUCS2toUTF8(s).get());
+    printf("<%s>\n", NS_ConvertUTF16toUTF8(s).get());
   } else {
     printf(">>text\n");
   }
 
   // Recurse down into optgroups
-  //
-  // I *would* put a restriction in here to only search under
-  // optgroups (and not, for example, <P></P>), but it really
-  // doesn't *hurt* to search under other stuff and it's more
-  // efficient in the normal only-optgroup-and-option case
-  // (one less QueryInterface).
-  PRUint32 numChildren = aOptions->GetChildCount();
+  if (IsOptGroup(aOptions)) {
+    PRUint32 numChildren = aOptions->GetChildCount();
 
-  for (PRUint32 i = 0; i < numChildren; ++i) {
-    PrintOptions(aOptions->GetChildAt(i), tabs + 1);
+    for (PRUint32 i = 0; i < numChildren; ++i) {
+      PrintOptions(aOptions->GetChildAt(i), tabs + 1);
+    }
   }
 
   return NS_OK;
@@ -654,10 +727,8 @@ nsHTMLSelectElement::RemoveOptionsFromList(nsIContent* aOptions,
 
 static PRBool IsOptGroup(nsIContent *aContent)
 {
-  nsINodeInfo *ni = aContent->GetNodeInfo();
-
-  return (ni && ni->Equals(nsHTMLAtoms::optgroup) &&
-          aContent->IsContentOfType(nsIContent::eHTML));
+  return (aContent->NodeInfo()->Equals(nsHTMLAtoms::optgroup) &&
+          aContent->IsNodeOfType(nsINode::eHTML));
 }
 
 // If the document is such that recursing over these options gets us
@@ -675,7 +746,7 @@ nsHTMLSelectElement::InsertOptionsIntoListRecurse(nsIContent* aOptions,
 
   nsCOMPtr<nsIDOMHTMLOptionElement> optElement(do_QueryInterface(aOptions));
   if (optElement) {
-    nsresult rv = mOptions->InsertElementAt(optElement, *aInsertIndex);
+    nsresult rv = mOptions->InsertOptionAt(optElement, *aInsertIndex);
     NS_ENSURE_SUCCESS(rv, rv);
     (*aInsertIndex)++;
     return NS_OK;
@@ -687,24 +758,16 @@ nsHTMLSelectElement::InsertOptionsIntoListRecurse(nsIContent* aOptions,
     mNonOptionChildren++;
   }
 
+  // Recurse down into optgroups
   if (IsOptGroup(aOptions)) {
     mOptGroupCount++;
-    DispatchDOMEvent(NS_LITERAL_STRING("selectHasGroups"));
-  }
 
-  // Recurse down into optgroups
-  //
-  // I *would* put a restriction in here to only search under
-  // optgroups (and not, for example, <P></P>), but it really
-  // doesn't *hurt* to search under other stuff and it's more
-  // efficient in the normal only-optgroup-and-option case
-  // (one less QueryInterface).
-  PRUint32 numChildren = aOptions->GetChildCount();
-
-  for (PRUint32 i = 0; i < numChildren; ++i) {
-    nsresult rv = InsertOptionsIntoListRecurse(aOptions->GetChildAt(i),
-                                               aInsertIndex, aDepth+1);
-    NS_ENSURE_SUCCESS(rv, rv);
+    PRUint32 numChildren = aOptions->GetChildCount();
+    for (PRUint32 i = 0; i < numChildren; ++i) {
+      nsresult rv = InsertOptionsIntoListRecurse(aOptions->GetChildAt(i),
+                                                 aInsertIndex, aDepth+1);
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
   }
 
   return NS_OK;
@@ -725,8 +788,11 @@ nsHTMLSelectElement::RemoveOptionsFromListRecurse(nsIContent* aOptions,
 
   nsCOMPtr<nsIDOMHTMLOptionElement> optElement(do_QueryInterface(aOptions));
   if (optElement) {
-    nsresult rv = mOptions->RemoveElementAt(aRemoveIndex);
-    NS_ENSURE_SUCCESS(rv, rv);
+    if (mOptions->ItemAsOption(aRemoveIndex) != optElement) {
+      NS_ERROR("wrong option at index");
+      return NS_ERROR_UNEXPECTED;
+    }
+    mOptions->RemoveOptionAt(aRemoveIndex);
     (*aNumRemoved)++;
     return NS_OK;
   }
@@ -736,28 +802,18 @@ nsHTMLSelectElement::RemoveOptionsFromListRecurse(nsIContent* aOptions,
     mNonOptionChildren--;
   }
 
-  if (mOptGroupCount) {
-    if (IsOptGroup(aOptions)) {
-      mOptGroupCount--;
-      DispatchDOMEvent(NS_LITERAL_STRING("selectHasNoGroups"));
-    }
-  }
-
   // Recurse down deeper for options
-  //
-  // I *would* put a restriction in here to only search under
-  // optgroups (and not, for example, <P></P>), but it really
-  // doesn't *hurt* to search under other stuff and it's more
-  // efficient in the normal only-optgroup-and-option case
-  // (one less QueryInterface).
-  PRUint32 numChildren = aOptions->GetChildCount();
+  if (mOptGroupCount && IsOptGroup(aOptions)) {
+    mOptGroupCount--;
 
-  for (PRUint32 i = 0; i < numChildren; ++i) {
-    nsresult rv = RemoveOptionsFromListRecurse(aOptions->GetChildAt(i),
-                                               aRemoveIndex,
-                                               aNumRemoved,
-                                               aDepth + 1);
-    NS_ENSURE_SUCCESS(rv, rv);
+    PRUint32 numChildren = aOptions->GetChildCount();
+    for (PRUint32 i = 0; i < numChildren; ++i) {
+      nsresult rv = RemoveOptionsFromListRecurse(aOptions->GetChildAt(i),
+                                                 aRemoveIndex,
+                                                 aNumRemoved,
+                                                 aDepth + 1);
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
   }
 
   return NS_OK;
@@ -813,9 +869,8 @@ NS_IMETHODIMP
 nsHTMLSelectElement::WillRemoveOptions(nsIContent* aParent,
                                        PRInt32 aContentIndex)
 {
-  nsresult rv = NS_OK;
-
   PRInt32 level = GetContentDepth(aParent);
+  NS_ASSERTION(level >= 0, "getting notified by unexpected content");
   if (level == -1) {
     return NS_ERROR_FAILURE;
   }
@@ -833,11 +888,12 @@ nsHTMLSelectElement::WillRemoveOptions(nsIContent* aParent,
       ind = GetFirstOptionIndex(currentKid);
     }
     if (ind != -1) {
-      rv = RemoveOptionsFromList(currentKid, ind, level);
+      nsresult rv = RemoveOptionsFromList(currentKid, ind, level);
+      NS_ENSURE_SUCCESS(rv, rv);
     }
   }
 
-  return rv;
+  return NS_OK;
 }
 
 PRInt32
@@ -1059,16 +1115,16 @@ nsHTMLSelectElement::SetLength(PRUint32 aLength)
     nsContentUtils::NameChanged(mNodeInfo, nsHTMLAtoms::option,
                                 getter_AddRefs(nodeInfo));
 
-    nsCOMPtr<nsIHTMLContent> element = NS_NewHTMLOptionElement(nodeInfo);
+    nsCOMPtr<nsIContent> element = NS_NewHTMLOptionElement(nodeInfo);
     if (!element) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    nsCOMPtr<nsITextContent> text;
-    rv = NS_NewTextNode(getter_AddRefs(text));
+    nsCOMPtr<nsIContent> text;
+    rv = NS_NewTextNode(getter_AddRefs(text), mNodeInfo->NodeInfoManager());
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = element->AppendChildTo(text, PR_FALSE, PR_FALSE);
+    rv = element->AppendChildTo(text, PR_FALSE);
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIDOMNode> node(do_QueryInterface(element));
@@ -1106,8 +1162,19 @@ nsHTMLSelectElement::GetSelectedIndex(PRInt32* aValue)
 NS_IMETHODIMP
 nsHTMLSelectElement::SetSelectedIndex(PRInt32 aIndex)
 {
-  return SetOptionsSelectedByIndex(aIndex, aIndex, PR_TRUE,
-                                   PR_TRUE, PR_TRUE, PR_TRUE, nsnull);
+  PRInt32 oldSelectedIndex = mSelectedIndex;
+
+  nsresult rv = SetOptionsSelectedByIndex(aIndex, aIndex, PR_TRUE,
+                                          PR_TRUE, PR_TRUE, PR_TRUE, nsnull);
+
+  if (NS_SUCCEEDED(rv)) {
+    nsISelectControlFrame* selectFrame = GetSelectFrame();
+    if (selectFrame) {
+      rv = selectFrame->OnSetSelectedIndex(oldSelectedIndex, mSelectedIndex);
+    }
+  }
+
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -1471,47 +1538,13 @@ nsHTMLSelectElement::GetValue(nsAString& aValue)
 
     rv = Item(selectedIndex, getter_AddRefs(node));
 
-    if (NS_SUCCEEDED(rv) && node) {
-      nsCOMPtr<nsIHTMLContent> option = do_QueryInterface(node);
-
-      if (option) {
-        nsHTMLValue value;
-        // first check to see if value is there and has a value
-        rv = option->GetHTMLAttribute(nsHTMLAtoms::value, value);
-
-        if (NS_CONTENT_ATTR_HAS_VALUE == rv) {
-          if (eHTMLUnit_String == value.GetUnit()) {
-            value.GetStringValue(aValue);
-          } else {
-            aValue.SetLength(0);
-          }
-
-          return NS_OK;
-        }
-#if 0 // temporary for bug 4050
-        // first check to see if label is there and has a value
-        rv = option->GetHTMLAttribute(nsHTMLAtoms::label, value);
-
-        if (NS_CONTENT_ATTR_HAS_VALUE == rv) {
-          if (eHTMLUnit_String == value.GetUnit()) {
-            value.GetStringValue(aValue);
-          } else {
-            aValue.SetLength(0);
-          }
-
-          return NS_OK;
-        }
-#endif
-        nsCOMPtr<nsIDOMHTMLOptionElement> optElement(do_QueryInterface(node));
-        if (optElement) {
-          optElement->GetText(aValue);
-        }
-
-        return NS_OK;
-      }
+    nsCOMPtr<nsIDOMHTMLOptionElement> option = do_QueryInterface(node);
+    if (NS_SUCCEEDED(rv) && option) {
+      return option->GetValue(aValue);
     }
   }
 
+  aValue.Truncate(0);
   return rv;
 }
 
@@ -1554,13 +1587,15 @@ nsHTMLSelectElement::SetValue(const nsAString& aValue)
 NS_IMPL_BOOL_ATTR(nsHTMLSelectElement, Disabled, disabled)
 NS_IMPL_BOOL_ATTR(nsHTMLSelectElement, Multiple, multiple)
 NS_IMPL_STRING_ATTR(nsHTMLSelectElement, Name, name)
-NS_IMPL_INT_ATTR(nsHTMLSelectElement, Size, size)
+NS_IMPL_INT_ATTR_DEFAULT_VALUE(nsHTMLSelectElement, Size, size, 0)
 NS_IMPL_INT_ATTR_DEFAULT_VALUE(nsHTMLSelectElement, TabIndex, tabindex, 0)
 
 NS_IMETHODIMP
 nsHTMLSelectElement::Blur()
 {
-  SetElementFocus(PR_FALSE);
+  if (ShouldFocus(this)) {
+    SetElementFocus(PR_FALSE);
+  }
 
   return NS_OK;
 }
@@ -1586,16 +1621,13 @@ nsHTMLSelectElement::SetFocus(nsPresContext* aPresContext)
     return;
   }
 
-  aPresContext->EventStateManager()->SetContentState(this,
-                                                     NS_EVENT_STATE_FOCUS);
-
-  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
-
-  if (formControlFrame) {
-    formControlFrame->SetFocus(PR_TRUE, PR_TRUE);
-    formControlFrame->ScrollIntoView(aPresContext);
-    // Could call SelectAll(aPresContext) here to automatically
-    // select text when we receive focus.
+  nsIEventStateManager *esm = aPresContext->EventStateManager();
+  if (esm->SetContentState(this, NS_EVENT_STATE_FOCUS)) {
+    nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
+    if (formControlFrame) {
+      formControlFrame->SetFocus(PR_TRUE, PR_TRUE);
+      nsLayoutUtils::ScrollIntoView(formControlFrame);
+    }
   }
 }
 
@@ -1628,11 +1660,7 @@ PRBool
 nsHTMLSelectElement::CheckSelectSomething()
 {
   if (mIsDoneAddingChildren) {
-    PRInt32 size = 1;
-    GetSize(&size);
-    PRBool isMultiple;
-    GetMultiple(&isMultiple);
-    if (mSelectedIndex < 0 && !isMultiple && size <= 1) {
+    if (mSelectedIndex < 0 && IsCombobox()) {
       return SelectSomething();
     }
   }
@@ -1663,6 +1691,37 @@ nsHTMLSelectElement::SelectSomething()
   return PR_FALSE;
 }
 
+nsresult
+nsHTMLSelectElement::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
+                               PRBool aNotify)
+{
+  if (aNotify && aNameSpaceID == kNameSpaceID_None &&
+      aAttribute == nsHTMLAtoms::multiple) {
+    // We're changing from being a multi-select to a single-select.
+    // Make sure we only have one option selected before we do that.
+    // Note that this needs to come before we really unset the attr,
+    // since SetOptionsSelectedByIndex does some bail-out type
+    // optimization for cases when the select is not multiple that
+    // would lead to only a single option getting deselected.
+    if (mSelectedIndex >= 0) {
+      SetSelectedIndex(mSelectedIndex);
+    }
+  }
+
+  nsresult rv = nsGenericHTMLFormElement::UnsetAttr(aNameSpaceID, aAttribute,
+                                                    aNotify);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (aNotify && aNameSpaceID == kNameSpaceID_None &&
+      aAttribute == nsHTMLAtoms::multiple) {
+    // We might have become a combobox; make sure _something_ gets
+    // selected in that case
+    CheckSelectSomething();
+  }
+
+  return rv;
+}
+
 PRBool
 nsHTMLSelectElement::IsDoneAddingChildren()
 {
@@ -1670,7 +1729,7 @@ nsHTMLSelectElement::IsDoneAddingChildren()
 }
 
 void
-nsHTMLSelectElement::DoneAddingChildren()
+nsHTMLSelectElement::DoneAddingChildren(PRBool aHaveNotified)
 {
   mIsDoneAddingChildren = PR_TRUE;
 
@@ -1680,7 +1739,7 @@ nsHTMLSelectElement::DoneAddingChildren()
   // content, restore the rest of the options proper-like
   if (mRestoreState) {
     RestoreStateTo(mRestoreState);
-    NS_RELEASE(mRestoreState);
+    mRestoreState = nsnull;
   }
 
   // Notify the frame
@@ -1697,14 +1756,16 @@ nsHTMLSelectElement::DoneAddingChildren()
 }
 
 PRBool
-nsHTMLSelectElement::ParseAttribute(nsIAtom* aAttribute,
+nsHTMLSelectElement::ParseAttribute(PRInt32 aNamespaceID,
+                                    nsIAtom* aAttribute,
                                     const nsAString& aValue,
                                     nsAttrValue& aResult)
 {
-  if (aAttribute == nsHTMLAtoms::size) {
+  if (aAttribute == nsHTMLAtoms::size && kNameSpaceID_None == aNamespaceID) {
     return aResult.ParseIntWithBounds(aValue, 0);
   }
-  return nsGenericHTMLElement::ParseAttribute(aAttribute, aValue, aResult);
+  return nsGenericHTMLElement::ParseAttribute(aNamespaceID, aAttribute, aValue,
+                                              aResult);
 }
 
 static void
@@ -1739,23 +1800,19 @@ nsHTMLSelectElement::IsAttributeMapped(const nsIAtom* aAttribute) const
   return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
 }
 
-NS_IMETHODIMP
-nsHTMLSelectElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const
+nsMapRuleToAttributesFunc
+nsHTMLSelectElement::GetAttributeMappingFunction() const
 {
-  aMapRuleFunc = &MapAttributesIntoRule;
-
-  return NS_OK;
+  return &MapAttributesIntoRule;
 }
 
 
 nsresult
-nsHTMLSelectElement::HandleDOMEvent(nsPresContext* aPresContext,
-                                    nsEvent* aEvent,
-                                    nsIDOMEvent** aDOMEvent,
-                                    PRUint32 aFlags,
-                                    nsEventStatus* aEventStatus)
+nsHTMLSelectElement::PreHandleEvent(nsEventChainPreVisitor& aVisitor)
 {
+  aVisitor.mCanHandle = PR_FALSE;
   // Do not process any DOM events if the element is disabled
+  // XXXsmaug This is not the right thing to do. But what is?
   PRBool disabled;
   nsresult rv = GetDisabled(&disabled);
   if (NS_FAILED(rv) || disabled) {
@@ -1779,18 +1836,15 @@ nsHTMLSelectElement::HandleDOMEvent(nsPresContext* aPresContext,
 
   // Must notify the frame that the blur event occurred
   // NOTE: At this point EventStateManager has not yet set the
-  /// new content as having focus so this content is still considered
+  // new content as having focus so this content is still considered
   // the focused element. So the ComboboxControlFrame tracks the focus
   // at a class level (Bug 32920)
-  if ((nsEventStatus_eIgnore == *aEventStatus) &&
-      !(aFlags & NS_EVENT_FLAG_CAPTURE) && !(aFlags & NS_EVENT_FLAG_SYSTEM_EVENT) &&
-      (aEvent->message == NS_BLUR_CONTENT) && formControlFrame) {
+  if (nsEventStatus_eIgnore == aVisitor.mEventStatus &&
+      (aVisitor.mEvent->message == NS_BLUR_CONTENT) && formControlFrame) {
     formControlFrame->SetFocus(PR_FALSE, PR_TRUE);
   }
 
-  return nsGenericHTMLFormElement::HandleDOMEvent(aPresContext, aEvent,
-                                                  aDOMEvent, aFlags,
-                                                  aEventStatus);
+  return nsGenericHTMLElement::PreHandleEvent(aVisitor);
 }
 
 // nsIFormControl
@@ -1798,11 +1852,10 @@ nsHTMLSelectElement::HandleDOMEvent(nsPresContext* aPresContext,
 NS_IMETHODIMP
 nsHTMLSelectElement::SaveState()
 {
-  nsSelectState* state = new nsSelectState();
+  nsRefPtr<nsSelectState> state = new nsSelectState();
   if (!state) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
-  NS_ADDREF(state);
 
   PRUint32 len;
   GetLength(&len);
@@ -1820,21 +1873,19 @@ nsHTMLSelectElement::SaveState()
     }
   }
 
-  nsCOMPtr<nsIPresState> presState;
-  nsresult rv = GetPrimaryPresState(this, getter_AddRefs(presState));
+  nsPresState *presState = nsnull;
+  nsresult rv = GetPrimaryPresState(this, &presState);
   if (presState) {
     rv = presState->SetStatePropertyAsSupports(NS_LITERAL_STRING("selecteditems"),
                                            state);
     NS_ASSERTION(NS_SUCCEEDED(rv), "selecteditems set failed!");
   }
 
-  NS_RELEASE(state);
-
   return rv;
 }
 
 PRBool
-nsHTMLSelectElement::RestoreState(nsIPresState* aState)
+nsHTMLSelectElement::RestoreState(nsPresState* aState)
 {
   // Get the presentation state object to retrieve our stuff out of.
   nsCOMPtr<nsISupports> state;
@@ -1845,10 +1896,7 @@ nsHTMLSelectElement::RestoreState(nsIPresState* aState)
 
     // Don't flush, if the frame doesn't exist yet it doesn't care if
     // we're reset or not.
-    nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_FALSE);
-    if (formControlFrame) {
-      formControlFrame->OnContentReset();
-    }
+    DispatchContentReset();
   }
 
   return PR_FALSE;
@@ -1872,7 +1920,6 @@ nsHTMLSelectElement::RestoreStateTo(nsSelectState* aNewSelected)
 {
   if (!mIsDoneAddingChildren) {
     mRestoreState = aNewSelected;
-    NS_ADDREF(mRestoreState);
     return;
   }
 
@@ -1934,13 +1981,7 @@ nsHTMLSelectElement::Reset()
   //
   // If nothing was selected and it's not multiple, select something
   //
-  PRInt32 size = 1;
-  GetSize(&size);
-
-  PRBool isMultiple = PR_FALSE;
-  GetMultiple(&isMultiple);
-
-  if (numSelected == 0 && !isMultiple && size <= 1) {
+  if (numSelected == 0 && IsCombobox()) {
     SelectSomething();
   }
 
@@ -1950,10 +1991,7 @@ nsHTMLSelectElement::Reset()
   // Don't flush, if there's no frame yet it won't care about us being
   // reset even if we forced it to be created now.
   //
-  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_FALSE);
-  if (formControlFrame) {
-    formControlFrame->OnContentReset();
-  }
+  DispatchContentReset();
 
   return NS_OK;
 }
@@ -1977,9 +2015,8 @@ nsHTMLSelectElement::SubmitNamesValues(nsIFormSubmission* aFormSubmission,
   // Get the name (if no name, no submit)
   //
   nsAutoString name;
-  rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, name);
-  if (NS_FAILED(rv) || rv == NS_CONTENT_ATTR_NOT_THERE) {
-    return rv;
+  if (!GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, name)) {
+    return NS_OK;
   }
 
   //
@@ -2029,19 +2066,84 @@ nsHTMLSelectElement::GetHasOptGroups(PRBool* aHasGroups)
 void
 nsHTMLSelectElement::DispatchDOMEvent(const nsAString& aName)
 {
-  nsCOMPtr<nsIDOMDocumentEvent> domDoc = do_QueryInterface(GetOwnerDoc());
-  if (domDoc) {
-    nsCOMPtr<nsIDOMEvent> selectEvent;
-    domDoc->CreateEvent(NS_LITERAL_STRING("Events"), getter_AddRefs(selectEvent));
-    if (selectEvent) {
-      selectEvent->InitEvent(aName, PR_TRUE, PR_TRUE);
-      nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(NS_STATIC_CAST(nsIDOMNode*, this)));
-      PRBool noDefault;
-      target->DispatchEvent(selectEvent, &noDefault);
+  nsContentUtils::DispatchTrustedEvent(GetOwnerDoc(),
+                                       NS_STATIC_CAST(nsIContent*, this),
+                                       aName, PR_TRUE, PR_TRUE);
+}
+
+void nsHTMLSelectElement::DispatchContentReset() {
+  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_FALSE);
+  if (formControlFrame) {
+    // Only dispatch content reset notification if this is a list control
+    // frame or combo box control frame.
+    if (IsCombobox()) {
+      nsIComboboxControlFrame* comboFrame = nsnull;
+      CallQueryInterface(formControlFrame, &comboFrame);
+      if (comboFrame) {
+        comboFrame->OnContentReset();
+      }
+    } else {
+      nsIListControlFrame* listFrame = nsnull;
+      CallQueryInterface(formControlFrame, &listFrame);
+      if (listFrame) {
+        listFrame->OnContentReset();
+      }
     }
   }
 }
 
+static void
+AddOptionsRecurse(nsIContent* aRoot, nsHTMLOptionCollection* aArray)
+{
+  nsIContent* child;
+  for(PRUint32 i = 0; (child = aRoot->GetChildAt(i)); ++i) {
+    nsCOMPtr<nsIDOMHTMLOptionElement> opt = do_QueryInterface(child);
+    if (opt) {
+      // If we fail here, then at least we've tried our best
+      aArray->AppendOption(opt);
+    }
+    else if (IsOptGroup(child)) {
+      AddOptionsRecurse(child, aArray);
+    }
+  }
+}
+
+void
+nsHTMLSelectElement::RebuildOptionsArray()
+{
+  mOptions->Clear();
+  AddOptionsRecurse(this, mOptions);
+}
+
+#ifdef DEBUG
+
+static void
+VerifyOptionsRecurse(nsIContent* aRoot, PRInt32& aIndex,
+                     nsHTMLOptionCollection* aArray)
+{
+  nsIContent* child;
+  for(PRUint32 i = 0; (child = aRoot->GetChildAt(i)); ++i) {
+    nsCOMPtr<nsIDOMHTMLOptionElement> opt = do_QueryInterface(child);
+    if (opt) {
+      NS_ASSERTION(opt == aArray->ItemAsOption(aIndex++),
+                   "Options collection broken");
+    }
+    else if (IsOptGroup(child)) {
+      VerifyOptionsRecurse(child, aIndex, aArray);
+    }
+  }
+}
+
+void
+nsHTMLSelectElement::VerifyOptionsArray()
+{
+  PRInt32 aIndex = 0;
+  VerifyOptionsRecurse(this, aIndex, mOptions);
+}
+
+
+#endif
+
 //----------------------------------------------------------------------
 //
 // nsHTMLOptionCollection implementation
@@ -2139,31 +2241,44 @@ NS_IMETHODIMP
 nsHTMLOptionCollection::SetOption(PRInt32 aIndex,
                                   nsIDOMHTMLOptionElement *aOption)
 {
+  if (aIndex < 0 || !mSelect) {
+    return NS_OK;
+  }
+  
+  // if the new option is null, just remove this option.  Note that it's safe
+  // to pass a too-large aIndex in here.
+  if (!aOption) {
+    mSelect->Remove(aIndex);
+
+    // We're done.
+    return NS_OK;
+  }
+
   nsresult rv = NS_OK;
 
-  // If the indx is within range
-  if (mSelect && aIndex >= 0 && aIndex <= mElements.Count()) {
-    // if the new option is null, remove this option
-    if (!aOption) {
-      mSelect->Remove(aIndex);
+  // Now we're going to be setting an option in our collection
+  if (aIndex > mElements.Count()) {
+    // Fill our array with blank options up to (but not including, since we're
+    // about to change it) aIndex, for compat with other browsers.
+    rv = SetLength(aIndex);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
 
-      // We're done.
-      return NS_OK;
-    }
+  NS_ASSERTION(aIndex <= mElements.Count(), "SetLength lied");
+  
+  nsCOMPtr<nsIDOMNode> ret;
+  if (aIndex == mElements.Count()) {
+    rv = mSelect->AppendChild(aOption, getter_AddRefs(ret));
+  } else {
+    // Find the option they're talking about and replace it
+    // hold a strong reference to follow COM rules.
+    nsCOMPtr<nsIDOMHTMLOptionElement> refChild = mElements.SafeObjectAt(aIndex);
+    NS_ENSURE_TRUE(refChild, NS_ERROR_UNEXPECTED);
 
-    nsCOMPtr<nsIDOMNode> ret;
-    if (aIndex == mElements.Count()) {
-      rv = mSelect->AppendChild(aOption, getter_AddRefs(ret));
-    } else {
-      // Find the option they're talking about and replace it
-      nsIDOMHTMLOptionElement *refChild = mElements.ObjectAt(aIndex);
-      NS_ENSURE_TRUE(refChild, NS_ERROR_UNEXPECTED);
-
-      nsCOMPtr<nsIDOMNode> parent;
-      refChild->GetParentNode(getter_AddRefs(parent));
-      if (parent) {
-        rv = parent->ReplaceChild(aOption, refChild, getter_AddRefs(ret));
-      }
+    nsCOMPtr<nsIDOMNode> parent;
+    refChild->GetParentNode(getter_AddRefs(parent));
+    if (parent) {
+      rv = parent->ReplaceChild(aOption, refChild, getter_AddRefs(ret));
     }
   }
 
@@ -2196,12 +2311,6 @@ nsHTMLOptionCollection::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
   return NS_OK;
 }
 
-nsIDOMHTMLOptionElement *
-nsHTMLOptionCollection::ItemAsOption(PRInt32 aIndex)
-{
-  return mElements.SafeObjectAt(aIndex);
-}
-
 NS_IMETHODIMP
 nsHTMLOptionCollection::NamedItem(const nsAString& aName,
                                   nsIDOMNode** aReturn)
@@ -2215,14 +2324,10 @@ nsHTMLOptionCollection::NamedItem(const nsAString& aName,
     nsCOMPtr<nsIContent> content = do_QueryInterface(mElements.ObjectAt(i));
 
     if (content) {
-      nsAutoString name;
-
-      if (((content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::name,
-                             name) == NS_CONTENT_ATTR_HAS_VALUE) &&
-           aName.Equals(name)) ||
-          ((content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::id,
-                             name) == NS_CONTENT_ATTR_HAS_VALUE) &&
-           aName.Equals(name))) {
+      if (content->AttrValueIs(kNameSpaceID_None, nsHTMLAtoms::name, aName,
+                               eCaseMatters) ||
+          content->AttrValueIs(kNameSpaceID_None, nsHTMLAtoms::id, aName,
+                               eCaseMatters)) {
         rv = CallQueryInterface(content, aReturn);
 
         break;
@@ -2239,16 +2344,3 @@ nsHTMLOptionCollection::GetSelect(nsIDOMHTMLSelectElement **aReturn)
   NS_IF_ADDREF(*aReturn = mSelect);
   return NS_OK;
 }
-
-nsresult
-nsHTMLOptionCollection::InsertElementAt(nsIDOMHTMLOptionElement* aOption,
-                                        PRInt32 aIndex)
-{
-  return mElements.InsertObjectAt(aOption, aIndex);
-}
-
-nsresult
-nsHTMLOptionCollection::RemoveElementAt(PRInt32 aIndex)
-{
-  return mElements.RemoveObjectAt(aIndex);
-}
