@@ -42,7 +42,6 @@
 #include "nsDOMError.h"
 #include "nsContentList.h"
 #include "nsGenericDOMHTMLCollection.h"
-#include "nsIHTMLContent.h"
 #include "nsMappedAttributes.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
@@ -71,7 +70,7 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLElement::)
+  NS_FORWARD_NSIDOMNODE(nsGenericHTMLElement::)
 
   // nsIDOMElement
   NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLElement::)
@@ -82,20 +81,20 @@ public:
   // nsIDOMHTMLTableElement
   NS_DECL_NSIDOMHTMLTABLEELEMENT
 
-  virtual PRBool ParseAttribute(nsIAtom* aAttribute,
+  virtual PRBool ParseAttribute(PRInt32 aNamespaceID,
+                                nsIAtom* aAttribute,
                                 const nsAString& aValue,
                                 nsAttrValue& aResult);
-  NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
-                               const nsHTMLValue& aValue,
-                               nsAString& aResult) const;
-  NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
+  virtual nsMapRuleToAttributesFunc GetAttributeMappingFunction() const;
   NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* aAttribute) const;
 
+  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
+
 protected:
   already_AddRefed<nsIDOMHTMLTableSectionElement> GetSection(nsIAtom *aTag);
 
-  nsCOMPtr<nsIContentList> mTBodies;
-  TableRowsCollection *mRows;
+  nsRefPtr<nsContentList> mTBodies;
+  nsRefPtr<TableRowsCollection> mRows;
 };
 
 
@@ -110,6 +109,8 @@ public:
   TableRowsCollection(nsHTMLTableElement *aParent);
   virtual ~TableRowsCollection();
 
+  nsresult Init();
+
   NS_IMETHOD    GetLength(PRUint32* aLength);
   NS_IMETHOD    Item(PRUint32 aIndex, nsIDOMNode** aReturn);
   NS_IMETHOD    NamedItem(const nsAString& aName,
@@ -118,6 +119,8 @@ public:
   NS_IMETHOD    ParentDestroyed();
 
 protected:
+  // Those rows that are not in table sections
+  nsRefPtr<nsContentList> mOrphanRows;  
   nsHTMLTableElement * mParent;
 };
 
@@ -136,18 +139,81 @@ TableRowsCollection::~TableRowsCollection()
   // reference for us.
 }
 
+nsresult
+TableRowsCollection::Init()
+{
+  mOrphanRows = new nsContentList(mParent,
+                                  nsHTMLAtoms::tr,
+                                  mParent->NodeInfo()->NamespaceID(),
+                                  PR_FALSE);
+  return mOrphanRows ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+}
+
+// Macro that can be used to avoid copy/pasting code to iterate over the
+// rowgroups.  _code should be the code to execute for each rowgroup.  The
+// rowgroup's rows will be in the nsIDOMHTMLCollection* named "rows".  Note
+// that this may be null at any time.  This macro assumes an nsresult named
+// |rv| is in scope.
+#define DO_FOR_EACH_ROWGROUP(_code)                                  \
+  PR_BEGIN_MACRO                                                     \
+    if (mParent) {                                                   \
+      /* THead */                                                    \
+      nsCOMPtr<nsIDOMHTMLTableSectionElement> rowGroup;              \
+      rv = mParent->GetTHead(getter_AddRefs(rowGroup));              \
+      NS_ENSURE_SUCCESS(rv, rv);                                     \
+      nsCOMPtr<nsIDOMHTMLCollection> rows;                           \
+      if (rowGroup) {                                                \
+        rowGroup->GetRows(getter_AddRefs(rows));                     \
+        do { /* gives scoping */                                     \
+          _code                                                      \
+        } while (0);                                                 \
+      }                                                              \
+      nsCOMPtr<nsIDOMHTMLCollection> _tbodies;                       \
+      /* TBodies */                                                  \
+      rv = mParent->GetTBodies(getter_AddRefs(_tbodies));            \
+      NS_ENSURE_SUCCESS(rv, rv);                                     \
+      if (_tbodies) {                                                \
+        nsCOMPtr<nsIDOMNode> _node;                                  \
+        PRUint32 _tbodyIndex = 0;                                    \
+        rv = _tbodies->Item(_tbodyIndex, getter_AddRefs(_node));     \
+        NS_ENSURE_SUCCESS(rv, rv);                                   \
+        while (_node) {                                              \
+          rowGroup = do_QueryInterface(_node);                       \
+          if (rowGroup) {                                            \
+            rowGroup->GetRows(getter_AddRefs(rows));                 \
+            do { /* gives scoping */                                 \
+              _code                                                  \
+            } while (0);                                             \
+          }                                                          \
+          rv = _tbodies->Item(++_tbodyIndex, getter_AddRefs(_node)); \
+          NS_ENSURE_SUCCESS(rv, rv);                                 \
+        }                                                            \
+      }                                                              \
+      /* orphan rows */                                              \
+      rows = mOrphanRows;                                            \
+      do { /* gives scoping */                                       \
+        _code                                                        \
+      } while (0);                                                   \
+      /* TFoot */                                                    \
+      rv = mParent->GetTFoot(getter_AddRefs(rowGroup));              \
+      NS_ENSURE_SUCCESS(rv, rv);                                     \
+      rows = nsnull;                                                 \
+      if (rowGroup) {                                                \
+        rowGroup->GetRows(getter_AddRefs(rows));                     \
+        do { /* gives scoping */                                     \
+          _code                                                      \
+        } while (0);                                                 \
+      }                                                              \
+    }                                                                \
+  PR_END_MACRO
+
 static PRUint32
-CountRowsInRowGroup(nsIDOMHTMLTableSectionElement* aRowGroup)
+CountRowsInRowGroup(nsIDOMHTMLCollection* rows)
 {
   PRUint32 length = 0;
   
-  if (aRowGroup) {
-    nsCOMPtr<nsIDOMHTMLCollection> rows;
-    aRowGroup->GetRows(getter_AddRefs(rows));
-    
-    if (rows) {
-      rows->GetLength(&length);
-    }
+  if (rows) {
+    rows->GetLength(&length);
   }
   
   return length;
@@ -159,40 +225,12 @@ CountRowsInRowGroup(nsIDOMHTMLTableSectionElement* aRowGroup)
 NS_IMETHODIMP 
 TableRowsCollection::GetLength(PRUint32* aLength)
 {
-  NS_ENSURE_ARG_POINTER(aLength);
-
   *aLength=0;
   nsresult rv = NS_OK;
 
-  if (mParent) {
-    // count the rows in the thead, tfoot, and all tbodies
-    nsCOMPtr<nsIDOMHTMLTableSectionElement> rowGroup;
-
-    mParent->GetTHead(getter_AddRefs(rowGroup));
-    *aLength += CountRowsInRowGroup(rowGroup);
-
-    mParent->GetTFoot(getter_AddRefs(rowGroup));
-    *aLength += CountRowsInRowGroup(rowGroup);
-
-    nsCOMPtr<nsIDOMHTMLCollection> tbodies;
-
-    mParent->GetTBodies(getter_AddRefs(tbodies));
-
-    if (tbodies) {
-      nsCOMPtr<nsIDOMNode> node;
-
-      PRUint32 theIndex = 0;
-
-      tbodies->Item(theIndex, getter_AddRefs(node));
-
-      while (node) {
-        rowGroup = do_QueryInterface(node);
-        *aLength += CountRowsInRowGroup(rowGroup);
-        
-        tbodies->Item(++theIndex, getter_AddRefs(node));
-      }
-    }
-  }
+  DO_FOR_EACH_ROWGROUP(
+    *aLength += CountRowsInRowGroup(rows);
+  );
 
   return rv;
 }
@@ -200,7 +238,7 @@ TableRowsCollection::GetLength(PRUint32* aLength)
 // Returns the number of items in the row group, only if *aItem ends
 // up null.  Otherwise, returns 0.
 static PRUint32
-GetItemOrCountInRowGroup(nsIDOMHTMLTableSectionElement* aRowGroup,
+GetItemOrCountInRowGroup(nsIDOMHTMLCollection* rows,
                          PRUint32 aIndex, nsIDOMNode** aItem)
 {
   NS_PRECONDITION(aItem, "Null out param");
@@ -208,86 +246,62 @@ GetItemOrCountInRowGroup(nsIDOMHTMLTableSectionElement* aRowGroup,
   *aItem = nsnull;
   PRUint32 length = 0;
   
-  if (aRowGroup) {
-    nsCOMPtr<nsIDOMHTMLCollection> rows;
-    aRowGroup->GetRows(getter_AddRefs(rows));
-    
-    if (rows) {
-      rows->Item(aIndex, aItem);
-      if (!*aItem) {
-        rows->GetLength(&length);
-      }
+  if (rows) {
+    rows->Item(aIndex, aItem);
+    if (!*aItem) {
+      rows->GetLength(&length);
     }
   }
   
   return length;
 }
+
 // increments aReturn refcnt by 1
 NS_IMETHODIMP 
 TableRowsCollection::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 {
   *aReturn = nsnull;
   nsresult rv = NS_OK;
- 
-  if (mParent) {
-    nsCOMPtr<nsIDOMHTMLTableSectionElement> rowGroup;
 
-    // check the thead
-    mParent->GetTHead(getter_AddRefs(rowGroup));
-
-    PRUint32 count = GetItemOrCountInRowGroup(rowGroup, aIndex, aReturn);
+  DO_FOR_EACH_ROWGROUP(
+    PRUint32 count = GetItemOrCountInRowGroup(rows, aIndex, aReturn);
     if (*aReturn) {
       return NS_OK; 
     }
 
     NS_ASSERTION(count <= aIndex, "GetItemOrCountInRowGroup screwed up");
     aIndex -= count;
-    
-    // check the tbodies
-    nsCOMPtr<nsIDOMHTMLCollection> tbodies;
-
-    mParent->GetTBodies(getter_AddRefs(tbodies));
-
-    if (tbodies) {
-      nsCOMPtr<nsIDOMNode> node;
-      PRUint32 theIndex = 0;
-
-      tbodies->Item(theIndex, getter_AddRefs(node));
-
-      while (node) {
-        rowGroup = do_QueryInterface(node);
-        count = GetItemOrCountInRowGroup(rowGroup, aIndex, aReturn);
-        if (*aReturn) {
-          return NS_OK; 
-        }
-
-        NS_ASSERTION(count <= aIndex, "GetItemOrCountInRowGroup screwed up");
-        aIndex -= count;
-
-        tbodies->Item(++theIndex, getter_AddRefs(node));
-      }
-    }
-
-    // check the tfoot
-    mParent->GetTFoot(getter_AddRefs(rowGroup));
-
-    GetItemOrCountInRowGroup(rowGroup, aIndex, aReturn);
-  }
+  );
 
   return rv;
 }
 
+static nsresult
+GetNamedItemInRowGroup(nsIDOMHTMLCollection* aRows,
+                       const nsAString& aName, nsIDOMNode** aNamedItem)
+{
+  if (aRows) {
+    return aRows->NamedItem(aName, aNamedItem);
+  }
+
+  *aNamedItem = nsnull;
+  return NS_OK;
+}
+
 NS_IMETHODIMP 
 TableRowsCollection::NamedItem(const nsAString& aName,
                                nsIDOMNode** aReturn)
 {
-  NS_ENSURE_ARG_POINTER(aReturn);
-
-  // FIXME: Implement this!
-
   *aReturn = nsnull;
-
-  return NS_OK;
+  nsresult rv = NS_OK;
+  DO_FOR_EACH_ROWGROUP(
+    rv = GetNamedItemInRowGroup(rows, aName, aReturn);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (*aReturn) {
+      return rv;
+    }
+  );
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -309,18 +323,12 @@ NS_IMPL_NS_NEW_HTML_ELEMENT(Table)
 nsHTMLTableElement::nsHTMLTableElement(nsINodeInfo *aNodeInfo)
   : nsGenericHTMLElement(aNodeInfo)
 {
-  mRows=nsnull;
 }
 
 nsHTMLTableElement::~nsHTMLTableElement()
 {
-  if (mTBodies) {
-    mTBodies->RootDestroyed();
-  }
-
   if (mRows) {
     mRows->ParentDestroyed();
-    NS_RELEASE(mRows);
   }
 }
 
@@ -336,7 +344,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLTableElement, nsGenericHTMLElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
 
-NS_IMPL_DOM_CLONENODE(nsHTMLTableElement)
+NS_IMPL_ELEMENT_CLONE(nsHTMLTableElement)
 
 
 // the DOM spec says border, cellpadding, cellSpacing are all "wstring"
@@ -405,7 +413,7 @@ nsHTMLTableElement::GetSection(nsIAtom *aTag)
 
     section = do_QueryInterface(child);
 
-    if (section && child->GetNodeInfo()->Equals(aTag)) {
+    if (section && child->NodeInfo()->Equals(aTag)) {
       nsIDOMHTMLTableSectionElement *result = section;
       NS_ADDREF(result);
 
@@ -427,6 +435,13 @@ nsHTMLTableElement::GetTHead(nsIDOMHTMLTableSectionElement** aValue)
 NS_IMETHODIMP
 nsHTMLTableElement::SetTHead(nsIDOMHTMLTableSectionElement* aValue)
 {
+  nsCOMPtr<nsIContent> content(do_QueryInterface(aValue));
+  NS_ENSURE_TRUE(content, NS_ERROR_DOM_HIERARCHY_REQUEST_ERR);
+
+  if (!content->NodeInfo()->Equals(nsHTMLAtoms::thead)) {
+    return NS_ERROR_DOM_HIERARCHY_REQUEST_ERR;
+  }
+  
   nsresult rv = DeleteTHead();
   if (NS_FAILED(rv)) {
     return rv;
@@ -457,6 +472,13 @@ nsHTMLTableElement::GetTFoot(nsIDOMHTMLTableSectionElement** aValue)
 NS_IMETHODIMP
 nsHTMLTableElement::SetTFoot(nsIDOMHTMLTableSectionElement* aValue)
 {
+  nsCOMPtr<nsIContent> content(do_QueryInterface(aValue));
+  NS_ENSURE_TRUE(content, NS_ERROR_DOM_HIERARCHY_REQUEST_ERR);
+
+  if (!content->NodeInfo()->Equals(nsHTMLAtoms::tfoot)) {
+    return NS_ERROR_DOM_HIERARCHY_REQUEST_ERR;
+  }
+  
   nsresult rv = DeleteTFoot();
   if (NS_SUCCEEDED(rv)) {
     if (aValue) {
@@ -476,7 +498,11 @@ nsHTMLTableElement::GetRows(nsIDOMHTMLCollection** aValue)
     mRows = new TableRowsCollection(this);
     NS_ENSURE_TRUE(mRows, NS_ERROR_OUT_OF_MEMORY);
 
-    NS_ADDREF(mRows); // this table's reference, released in the destructor
+    nsresult rv = mRows->Init();
+    if (NS_FAILED(rv)) {
+      mRows = nsnull;
+      return rv;
+    }
   }
 
   *aValue = mRows;
@@ -490,16 +516,16 @@ nsHTMLTableElement::GetTBodies(nsIDOMHTMLCollection** aValue)
 {
   if (!mTBodies) {
     // Not using NS_GetContentList because this should not be cached
-    mTBodies = new nsContentList(GetDocument(),
+    mTBodies = new nsContentList(this,
                                  nsHTMLAtoms::tbody,
                                  mNodeInfo->NamespaceID(),
-                                 this,
                                  PR_FALSE);
 
     NS_ENSURE_TRUE(mTBodies, NS_ERROR_OUT_OF_MEMORY);
   }
 
-  return CallQueryInterface(mTBodies, aValue);
+  NS_ADDREF(*aValue = mTBodies);
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -523,7 +549,7 @@ nsHTMLTableElement::CreateTHead(nsIDOMHTMLElement** aValue)
     nsContentUtils::NameChanged(mNodeInfo, nsHTMLAtoms::thead,
                                 getter_AddRefs(nodeInfo));
 
-    nsCOMPtr<nsIHTMLContent> newHead = NS_NewHTMLTableSectionElement(nodeInfo);
+    nsCOMPtr<nsIContent> newHead = NS_NewHTMLTableSectionElement(nodeInfo);
 
     if (newHead) {
       nsCOMPtr<nsIDOMNode> child;
@@ -579,10 +605,10 @@ nsHTMLTableElement::CreateTFoot(nsIDOMHTMLElement** aValue)
     nsContentUtils::NameChanged(mNodeInfo, nsHTMLAtoms::tfoot,
                                 getter_AddRefs(nodeInfo));
 
-    nsCOMPtr<nsIHTMLContent> newFoot = NS_NewHTMLTableSectionElement(nodeInfo);
+    nsCOMPtr<nsIContent> newFoot = NS_NewHTMLTableSectionElement(nodeInfo);
 
     if (newFoot) {
-      rv = AppendChildTo(newFoot, PR_TRUE, PR_FALSE);
+      rv = AppendChildTo(newFoot, PR_TRUE);
       CallQueryInterface(newFoot, aValue);
     }
   }
@@ -625,10 +651,10 @@ nsHTMLTableElement::CreateCaption(nsIDOMHTMLElement** aValue)
     nsContentUtils::NameChanged(mNodeInfo, nsHTMLAtoms::caption,
                                 getter_AddRefs(nodeInfo));
 
-    nsCOMPtr<nsIHTMLContent> newCaption = NS_NewHTMLTableCaptionElement(nodeInfo);
+    nsCOMPtr<nsIContent> newCaption = NS_NewHTMLTableCaptionElement(nodeInfo);
 
     if (newCaption) {
-      rv = AppendChildTo(newCaption, PR_TRUE, PR_FALSE);
+      rv = AppendChildTo(newCaption, PR_TRUE);
       CallQueryInterface(newCaption, aValue);
     }
   }
@@ -701,7 +727,7 @@ nsHTMLTableElement::InsertRow(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
     nsContentUtils::NameChanged(mNodeInfo, nsHTMLAtoms::tr,
                                 getter_AddRefs(nodeInfo));
 
-    nsCOMPtr<nsIHTMLContent> newRow = NS_NewHTMLTableRowElement(nodeInfo);
+    nsCOMPtr<nsIContent> newRow = NS_NewHTMLTableRowElement(nodeInfo);
 
     if (newRow) {
       nsCOMPtr<nsIDOMNode> newRowNode(do_QueryInterface(newRow));
@@ -733,11 +759,12 @@ nsHTMLTableElement::InsertRow(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
     PRUint32 childCount = GetChildCount();
     for (PRUint32 i = 0; i < childCount; ++i) {
       nsIContent* child = GetChildAt(i);
-      nsINodeInfo* childInfo = child->GetNodeInfo();
-      if (childInfo &&
-          (childInfo->Equals(nsHTMLAtoms::thead, namespaceID) ||
-           childInfo->Equals(nsHTMLAtoms::tbody, namespaceID) ||
-           childInfo->Equals(nsHTMLAtoms::tfoot, namespaceID))) {
+      nsINodeInfo *childInfo = child->NodeInfo();
+      nsIAtom *localName = childInfo->NameAtom();
+      if (childInfo->NamespaceID() == namespaceID &&
+          (localName == nsHTMLAtoms::thead ||
+           localName == nsHTMLAtoms::tbody ||
+           localName == nsHTMLAtoms::tfoot)) {
         rowGroup = do_QueryInterface(child);
         NS_ASSERTION(rowGroup, "HTML node did not QI to nsIDOMNode");
         break;
@@ -749,11 +776,11 @@ nsHTMLTableElement::InsertRow(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
       nsContentUtils::NameChanged(mNodeInfo, nsHTMLAtoms::tbody,
                                   getter_AddRefs(nodeInfo));
 
-      nsCOMPtr<nsIHTMLContent> newRowGroup
-        = NS_NewHTMLTableSectionElement(nodeInfo);
+      nsCOMPtr<nsIContent> newRowGroup =
+        NS_NewHTMLTableSectionElement(nodeInfo);
 
       if (newRowGroup) {
-        rv = AppendChildTo(newRowGroup, PR_TRUE, PR_FALSE);
+        rv = AppendChildTo(newRowGroup, PR_TRUE);
 
         rowGroup = do_QueryInterface(newRowGroup);
       }
@@ -764,7 +791,7 @@ nsHTMLTableElement::InsertRow(PRInt32 aIndex, nsIDOMHTMLElement** aValue)
       nsContentUtils::NameChanged(mNodeInfo, nsHTMLAtoms::tr,
                                   getter_AddRefs(nodeInfo));
 
-      nsCOMPtr<nsIHTMLContent> newRow = NS_NewHTMLTableRowElement(nodeInfo);
+      nsCOMPtr<nsIContent> newRow = NS_NewHTMLTableRowElement(nodeInfo);
       if (newRow) {
         nsCOMPtr<nsIDOMNode> firstRow;
 
@@ -835,7 +862,7 @@ nsHTMLTableElement::DeleteRow(PRInt32 aValue)
   return parent->RemoveChild(row, getter_AddRefs(deleted_row));
 }
 
-static const nsHTMLValue::EnumTable kFrameTable[] = {
+static const nsAttrValue::EnumTable kFrameTable[] = {
   { "void",   NS_STYLE_TABLE_FRAME_NONE },
   { "above",  NS_STYLE_TABLE_FRAME_ABOVE },
   { "below",  NS_STYLE_TABLE_FRAME_BELOW },
@@ -848,7 +875,7 @@ static const nsHTMLValue::EnumTable kFrameTable[] = {
   { 0 }
 };
 
-static const nsHTMLValue::EnumTable kRulesTable[] = {
+static const nsAttrValue::EnumTable kRulesTable[] = {
   { "none",   NS_STYLE_TABLE_RULES_NONE },
   { "groups", NS_STYLE_TABLE_RULES_GROUPS },
   { "rows",   NS_STYLE_TABLE_RULES_ROWS },
@@ -857,7 +884,7 @@ static const nsHTMLValue::EnumTable kRulesTable[] = {
   { 0 }
 };
 
-static const nsHTMLValue::EnumTable kLayoutTable[] = {
+static const nsAttrValue::EnumTable kLayoutTable[] = {
   { "auto",   NS_STYLE_TABLE_LAYOUT_AUTO },
   { "fixed",  NS_STYLE_TABLE_LAYOUT_FIXED },
   { 0 }
@@ -865,94 +892,69 @@ static const nsHTMLValue::EnumTable kLayoutTable[] = {
 
 
 PRBool
-nsHTMLTableElement::ParseAttribute(nsIAtom* aAttribute,
+nsHTMLTableElement::ParseAttribute(PRInt32 aNamespaceID,
+                                   nsIAtom* aAttribute,
                                    const nsAString& aValue,
                                    nsAttrValue& aResult)
 {
   /* ignore summary, just a string */
-  if (aAttribute == nsHTMLAtoms::cellspacing ||
-      aAttribute == nsHTMLAtoms::cellpadding) {
-    return aResult.ParseSpecialIntValue(aValue, PR_TRUE, PR_FALSE);
-  }
-  if (aAttribute == nsHTMLAtoms::cols) {
-    return aResult.ParseIntWithBounds(aValue, 0);
-  }
-  if (aAttribute == nsHTMLAtoms::border) {
-    if (!aResult.ParseIntWithBounds(aValue, 0)) {
-      // XXX this should really be NavQuirks only to allow non numeric value
-      aResult.SetTo(1, nsAttrValue::eInteger);
+  if (aNamespaceID == kNameSpaceID_None) {
+    if (aAttribute == nsHTMLAtoms::cellspacing ||
+        aAttribute == nsHTMLAtoms::cellpadding) {
+      return aResult.ParseSpecialIntValue(aValue, PR_TRUE, PR_FALSE);
     }
-
-    return PR_TRUE;
-  }
-  if (aAttribute == nsHTMLAtoms::height) {
-    return aResult.ParseSpecialIntValue(aValue, PR_TRUE, PR_FALSE);
-  }
-  if (aAttribute == nsHTMLAtoms::width) {
-    if (aResult.ParseSpecialIntValue(aValue, PR_TRUE, PR_FALSE)) {
-      // treat 0 width as auto
-      nsAttrValue::ValueType type = aResult.Type();
-      if ((type == nsAttrValue::eInteger && aResult.GetIntegerValue() == 0) ||
-          (type == nsAttrValue::ePercent && aResult.GetPercentValue() == 0.0f)) {
-        return PR_FALSE;
+    if (aAttribute == nsHTMLAtoms::cols) {
+      return aResult.ParseIntWithBounds(aValue, 0);
+    }
+    if (aAttribute == nsHTMLAtoms::border) {
+      if (!aResult.ParseIntWithBounds(aValue, 0)) {
+        // XXX this should really be NavQuirks only to allow non numeric value
+        aResult.SetTo(1);
       }
-    }
-    return PR_TRUE;
-  }
-  if (aAttribute == nsHTMLAtoms::align) {
-    return ParseTableHAlignValue(aValue, aResult);
-  }
-  if (aAttribute == nsHTMLAtoms::bgcolor ||
-           aAttribute == nsHTMLAtoms::bordercolor) {
-    return aResult.ParseColor(aValue, GetOwnerDoc());
-  }
-  if (aAttribute == nsHTMLAtoms::frame) {
-    return aResult.ParseEnumValue(aValue, kFrameTable);
-  }
-  if (aAttribute == nsHTMLAtoms::layout) {
-    return aResult.ParseEnumValue(aValue, kLayoutTable);
-  }
-  if (aAttribute == nsHTMLAtoms::rules) {
-    return aResult.ParseEnumValue(aValue, kRulesTable);
-  }
-  if (aAttribute == nsHTMLAtoms::hspace ||
-           aAttribute == nsHTMLAtoms::vspace) {
-    return aResult.ParseIntWithBounds(aValue, 0);
-  }
 
-  return nsGenericHTMLElement::ParseAttribute(aAttribute, aValue, aResult);
-}
-
-NS_IMETHODIMP
-nsHTMLTableElement::AttributeToString(nsIAtom* aAttribute,
-                               const nsHTMLValue& aValue,
-                               nsAString& aResult) const
-{
-  /* ignore summary, just a string */
-  /* ignore attributes that are of standard types border, cellpadding,
-     cellspacing, cols, height, width, background, bgcolor */
-  if (aAttribute == nsHTMLAtoms::align) {
-    if (TableHAlignValueToString(aValue, aResult)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
+      return PR_TRUE;
     }
-  }
-  else if (aAttribute == nsHTMLAtoms::frame) {
-    if (aValue.EnumValueToString(kFrameTable, aResult)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
+    if (aAttribute == nsHTMLAtoms::height) {
+      return aResult.ParseSpecialIntValue(aValue, PR_TRUE, PR_FALSE);
     }
-  }
-  else if (aAttribute == nsHTMLAtoms::layout) {
-    if (aValue.EnumValueToString(kLayoutTable, aResult)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
+    if (aAttribute == nsHTMLAtoms::width) {
+      if (aResult.ParseSpecialIntValue(aValue, PR_TRUE, PR_FALSE)) {
+        // treat 0 width as auto
+        nsAttrValue::ValueType type = aResult.Type();
+        if ((type == nsAttrValue::eInteger &&
+             aResult.GetIntegerValue() == 0) ||
+            (type == nsAttrValue::ePercent &&
+             aResult.GetPercentValue() == 0.0f)) {
+          return PR_FALSE;
+        }
+      }
+      return PR_TRUE;
     }
-  }
-  else if (aAttribute == nsHTMLAtoms::rules) {
-    if (aValue.EnumValueToString(kRulesTable, aResult)) {
-      return NS_CONTENT_ATTR_HAS_VALUE;
+    
+    if (aAttribute == nsHTMLAtoms::align) {
+      return ParseTableHAlignValue(aValue, aResult);
+    }
+    if (aAttribute == nsHTMLAtoms::bgcolor ||
+        aAttribute == nsHTMLAtoms::bordercolor) {
+      return aResult.ParseColor(aValue, GetOwnerDoc());
+    }
+    if (aAttribute == nsHTMLAtoms::frame) {
+      return aResult.ParseEnumValue(aValue, kFrameTable);
+    }
+    if (aAttribute == nsHTMLAtoms::layout) {
+      return aResult.ParseEnumValue(aValue, kLayoutTable);
+    }
+    if (aAttribute == nsHTMLAtoms::rules) {
+      return aResult.ParseEnumValue(aValue, kRulesTable);
+    }
+    if (aAttribute == nsHTMLAtoms::hspace ||
+        aAttribute == nsHTMLAtoms::vspace) {
+      return aResult.ParseIntWithBounds(aValue, 0);
     }
   }
 
-  return nsGenericHTMLElement::AttributeToString(aAttribute, aValue, aResult);
+  return nsGenericHTMLElement::ParseAttribute(aNamespaceID, aAttribute, aValue,
+                                              aResult);
 }
 
 static void 
@@ -962,6 +964,72 @@ MapTableFrameInto(const nsMappedAttributes* aAttributes,
   if (!aData->mMarginData)
     return;
 
+  // 0 out the sides that we want to hide based on the frame attribute
+  const nsAttrValue* frameValue = aAttributes->GetAttr(nsHTMLAtoms::frame);
+
+  if (frameValue && frameValue->Type() == nsAttrValue::eEnum) {
+    // adjust the border style based on the value of frame
+    switch (frameValue->GetEnumValue())
+    {
+    case NS_STYLE_TABLE_FRAME_NONE:
+      if (aData->mMarginData->mBorderStyle.mLeft.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mRight.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mTop.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mBottom.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      break;
+    case NS_STYLE_TABLE_FRAME_ABOVE:
+      if (aData->mMarginData->mBorderStyle.mLeft.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mRight.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mBottom.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      break;
+    case NS_STYLE_TABLE_FRAME_BELOW: 
+      if (aData->mMarginData->mBorderStyle.mLeft.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mRight.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mTop.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      break;
+    case NS_STYLE_TABLE_FRAME_HSIDES:
+      if (aData->mMarginData->mBorderStyle.mLeft.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mRight.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      break;
+    case NS_STYLE_TABLE_FRAME_LEFT:
+      if (aData->mMarginData->mBorderStyle.mRight.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mTop.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mBottom.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      break;
+    case NS_STYLE_TABLE_FRAME_RIGHT:
+      if (aData->mMarginData->mBorderStyle.mLeft.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mTop.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mBottom.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      break;
+    case NS_STYLE_TABLE_FRAME_VSIDES:
+      if (aData->mMarginData->mBorderStyle.mTop.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      if (aData->mMarginData->mBorderStyle.mBottom.GetUnit() == eCSSUnit_Null)
+        aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
+      break;
+    // BOX and BORDER are ignored, the caller has already set all the border sides
+    // any illegal value is also ignored
+    }
+  }
+
   // set up defaults
   if (aData->mMarginData->mBorderStyle.mLeft.GetUnit() == eCSSUnit_Null)
     aData->mMarginData->mBorderStyle.mLeft.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
@@ -972,53 +1040,11 @@ MapTableFrameInto(const nsMappedAttributes* aAttributes,
   if (aData->mMarginData->mBorderStyle.mBottom.GetUnit() == eCSSUnit_Null)
     aData->mMarginData->mBorderStyle.mBottom.SetIntValue(aBorderStyle, eCSSUnit_Enumerated);
 
-  // 0 out the sides that we want to hide based on the frame attribute
-  const nsAttrValue* frameValue = aAttributes->GetAttr(nsHTMLAtoms::frame);
-
-  if (frameValue && frameValue->Type() == nsAttrValue::eEnum) {
-    // adjust the border style based on the value of frame
-    switch (frameValue->GetEnumValue())
-    {
-    case NS_STYLE_TABLE_FRAME_NONE:
-      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      break;
-    case NS_STYLE_TABLE_FRAME_ABOVE:
-      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      break;
-    case NS_STYLE_TABLE_FRAME_BELOW: 
-      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      break;
-    case NS_STYLE_TABLE_FRAME_HSIDES:
-      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      break;
-    case NS_STYLE_TABLE_FRAME_LEFT:
-      aData->mMarginData->mBorderStyle.mRight.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      break;
-    case NS_STYLE_TABLE_FRAME_RIGHT:
-      aData->mMarginData->mBorderStyle.mLeft.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      break;
-    case NS_STYLE_TABLE_FRAME_VSIDES:
-      aData->mMarginData->mBorderStyle.mTop.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      aData->mMarginData->mBorderStyle.mBottom.SetIntValue(NS_STYLE_BORDER_STYLE_NONE, eCSSUnit_Enumerated);
-      break;
-    // BOX and BORDER are ignored, the caller has already set all the border sides
-    // any illegal value is also ignored
-    }
-  }
 }
 
+// XXX The two callsites care about the two different halves of this
+// function, so split it, probably by just putting it in inline at the
+// callsites.
 static void 
 MapTableBorderInto(const nsMappedAttributes* aAttributes,
                    nsRuleData* aData, PRUint8 aBorderStyle)
@@ -1174,7 +1200,6 @@ MapAttributesIntoRule(const nsMappedAttributes* aAttributes,
   }
   else if (aData->mSID == eStyleStruct_Padding) {
     const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
-  
     if (readDisplay->mDisplay == NS_STYLE_DISPLAY_TABLE_CELL) {
       const nsAttrValue* value = aAttributes->GetAttr(nsHTMLAtoms::cellpadding);
       if (value) {
@@ -1238,15 +1263,19 @@ MapAttributesIntoRule(const nsMappedAttributes* aAttributes,
       nsGenericHTMLElement::MapCommonAttributesInto(aAttributes, aData);
   }
   else if (aData->mSID == eStyleStruct_Border) {
-    if (!aData->mStyleContext) return;
     const nsStyleTableBorder* tableStyle = aData->mStyleContext->GetStyleTableBorder();
     const nsStyleDisplay* readDisplay = aData->mStyleContext->GetStyleDisplay();
     if (readDisplay->mDisplay == NS_STYLE_DISPLAY_TABLE_CELL) {
       if (NS_STYLE_BORDER_SEPARATE == tableStyle->mBorderCollapse) {
-        // Set the cell's border from the table in the separate border model. If there is a border
-        // on the table, then the mapping to rules=all will take care of borders in the collapsing model.
+        // Set the cell's border from the table in the separate border
+        // model. If there is a border on the table, then the mapping to
+        // rules=all will take care of borders in the collapsing model.
+        // But if rules="none", we don't want to do this.
         const nsAttrValue* value = aAttributes->GetAttr(nsHTMLAtoms::border);
-        if (value &&
+        const nsAttrValue* rulesValue = aAttributes->GetAttr(nsHTMLAtoms::rules);
+        if ((!rulesValue || rulesValue->Type() != nsAttrValue::eEnum ||
+             rulesValue->GetEnumValue() != NS_STYLE_TABLE_RULES_NONE) &&
+            value &&
             ((value->Type() == nsAttrValue::eInteger &&
               value->GetIntegerValue() > 0) ||
              value->IsEmptyString())) {
@@ -1299,7 +1328,7 @@ MapAttributesIntoRule(const nsMappedAttributes* aAttributes,
         if (aData->mMarginData->mBorderColor.mBottom.GetUnit() == eCSSUnit_Null)
           aData->mMarginData->mBorderColor.mBottom.SetColorValue(color);
 
-        borderStyle = NS_STYLE_BORDER_STYLE_OUTSET; // use css outset
+        borderStyle = NS_STYLE_BORDER_STYLE_SOLID; // compat, see bug 349655
       }
       else if (NS_STYLE_BORDER_COLLAPSE == tableStyle->mBorderCollapse) {
         // make the color grey
@@ -1357,9 +1386,8 @@ nsHTMLTableElement::IsAttributeMapped(const nsIAtom* aAttribute) const
   return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
 }
 
-NS_IMETHODIMP
-nsHTMLTableElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const
+nsMapRuleToAttributesFunc
+nsHTMLTableElement::GetAttributeMappingFunction() const
 {
-  aMapRuleFunc = &MapAttributesIntoRule;
-  return NS_OK;
+  return &MapAttributesIntoRule;
 }
