@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set sw=2 ts=2 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -38,7 +39,10 @@
 #include "nsIDOMHTMLTextAreaElement.h"
 #include "nsIDOMNSHTMLTextAreaElement.h"
 #include "nsITextControlElement.h"
+#include "nsIDOMNSEditableElement.h"
 #include "nsIControllers.h"
+#include "nsIFocusController.h"
+#include "nsPIDOMWindow.h"
 #include "nsContentCID.h"
 #include "nsCOMPtr.h"
 #include "nsIComponentManager.h"
@@ -47,7 +51,6 @@
 #include "nsIForm.h"
 #include "nsIFormSubmission.h"
 #include "nsIDOMEventReceiver.h"
-#include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
 #include "nsStyleConsts.h"
@@ -64,18 +67,21 @@
 #include "nsIPrivateDOMEvent.h"
 #include "nsGUIEvent.h"
 #include "nsLinebreakConverter.h"
-#include "nsIPresState.h"
+#include "nsPresState.h"
 #include "nsIDOMText.h"
 #include "nsReadableUtils.h"
-#include "nsITextContent.h"
+#include "nsEventDispatcher.h"
+#include "nsLayoutUtils.h"
 
 static NS_DEFINE_CID(kXULControllersCID,  NS_XULCONTROLLERS_CID);
 
+#define NS_NO_CONTENT_DISPATCH (1 << 0)
 
 class nsHTMLTextAreaElement : public nsGenericHTMLFormElement,
                               public nsIDOMHTMLTextAreaElement,
                               public nsIDOMNSHTMLTextAreaElement,
-                              public nsITextControlElement
+                              public nsITextControlElement,
+                              public nsIDOMNSEditableElement
 {
 public:
   nsHTMLTextAreaElement(nsINodeInfo *aNodeInfo, PRBool aFromParser = PR_FALSE);
@@ -85,7 +91,7 @@ public:
   NS_DECL_ISUPPORTS_INHERITED
 
   // nsIDOMNode
-  NS_FORWARD_NSIDOMNODE_NO_CLONENODE(nsGenericHTMLFormElement::)
+  NS_FORWARD_NSIDOMNODE(nsGenericHTMLFormElement::)
 
   // nsIDOMElement
   NS_FORWARD_NSIDOMELEMENT(nsGenericHTMLFormElement::)
@@ -99,13 +105,16 @@ public:
   // nsIDOMNSHTMLTextAreaElement
   NS_DECL_NSIDOMNSHTMLTEXTAREAELEMENT
 
+  // nsIDOMNSEditableElement
+  NS_FORWARD_NSIDOMNSEDITABLEELEMENT(nsGenericHTMLElement::)
+
   // nsIFormControl
-  NS_IMETHOD_(PRInt32) GetType() { return NS_FORM_TEXTAREA; }
+  NS_IMETHOD_(PRInt32) GetType() const { return NS_FORM_TEXTAREA; }
   NS_IMETHOD Reset();
   NS_IMETHOD SubmitNamesValues(nsIFormSubmission* aFormSubmission,
                                nsIContent* aSubmitElement);
   NS_IMETHOD SaveState();
-  virtual PRBool RestoreState(nsIPresState* aState);
+  virtual PRBool RestoreState(nsPresState* aState);
 
   // nsITextControlElemet
   NS_IMETHOD TakeTextFrameValue(const nsAString& aValue);
@@ -113,29 +122,27 @@ public:
 
   // nsIContent
   virtual nsresult InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                 PRBool aNotify, PRBool aDeepSetDocument);
-  virtual nsresult AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                 PRBool aDeepSetDocument);
+                                 PRBool aNotify);
   virtual nsresult RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
-  virtual PRBool ParseAttribute(nsIAtom* aAttribute,
+  virtual PRBool ParseAttribute(PRInt32 aNamespaceID,
+                                nsIAtom* aAttribute,
                                 const nsAString& aValue,
                                 nsAttrValue& aResult);
-  NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
+  virtual nsMapRuleToAttributesFunc GetAttributeMappingFunction() const;
   virtual nsChangeHint GetAttributeChangeHint(const nsIAtom* aAttribute,
                                               PRInt32 aModType) const;
   NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* aAttribute) const;
-  virtual nsresult HandleDOMEvent(nsPresContext* aPresContext,
-                                  nsEvent* aEvent, nsIDOMEvent** aDOMEvent,
-                                  PRUint32 aFlags,
-                                  nsEventStatus* aEventStatus);
+
+  virtual nsresult PreHandleEvent(nsEventChainPreVisitor& aVisitor);
+  virtual nsresult PostHandleEvent(nsEventChainPostVisitor& aVisitor);
+
   virtual void SetFocus(nsPresContext* aPresContext);
 
-  virtual nsresult GetInnerHTML(nsAString& aInnerHTML);
-  virtual nsresult SetInnerHTML(const nsAString& aInnerHTML);
-
-  virtual void DoneAddingChildren();
+  virtual void DoneAddingChildren(PRBool aHaveNotified);
   virtual PRBool IsDoneAddingChildren();
 
+  virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
+
 protected:
   nsCOMPtr<nsIControllers> mControllers;
   /** The current value.  This is null if the frame owns the value. */
@@ -195,6 +202,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLTextAreaElement,
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLTextAreaElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSHTMLTextAreaElement)
   NS_INTERFACE_MAP_ENTRY(nsITextControlElement)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMNSEditableElement)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLTextAreaElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
@@ -202,7 +210,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_END
 // nsIDOMHTMLTextAreaElement
 
 
-NS_IMPL_DOM_CLONENODE(nsHTMLTextAreaElement)
+NS_IMPL_ELEMENT_CLONE(nsHTMLTextAreaElement)
 
 
 NS_IMETHODIMP
@@ -217,7 +225,9 @@ nsHTMLTextAreaElement::GetForm(nsIDOMHTMLFormElement** aForm)
 NS_IMETHODIMP
 nsHTMLTextAreaElement::Blur()
 {
-  SetElementFocus(PR_FALSE);
+  if (ShouldFocus(this)) {
+    SetElementFocus(PR_FALSE);
+  }
 
   return NS_OK;
 }
@@ -239,24 +249,38 @@ nsHTMLTextAreaElement::SetFocus(nsPresContext* aPresContext)
     return;
 
   // first see if we are disabled or not. If disabled then do nothing.
-  nsAutoString disabled;
-
-  if (NS_CONTENT_ATTR_HAS_VALUE ==
-      nsGenericHTMLFormElement::GetAttr(kNameSpaceID_None,
-                                        nsHTMLAtoms::disabled, disabled)) {
+  if (HasAttr(kNameSpaceID_None, nsHTMLAtoms::disabled)) {
     return;
   }
 
-  aPresContext->EventStateManager()->SetContentState(this,
-                                                     NS_EVENT_STATE_FOCUS);
+  // We can't be focus'd if we aren't in a document
+  nsIDocument* doc = GetCurrentDoc();
+  if (!doc)
+    return;
 
-  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
+  // If the window is not active, do not allow the focus to bring the
+  // window to the front.  We update the focus controller, but do
+  // nothing else.
+  nsPIDOMWindow* win = doc->GetWindow();
+  if (win) {
+    nsIFocusController *focusController = win->GetRootFocusController();
+    PRBool isActive = PR_FALSE;
+    focusController->GetActive(&isActive);
+    if (!isActive) {
+      focusController->SetFocusedWindow(win);
+      focusController->SetFocusedElement(this);
 
-  if (formControlFrame) {
-    formControlFrame->SetFocus(PR_TRUE, PR_TRUE);
-    formControlFrame->ScrollIntoView(aPresContext);
-    // Could call SelectAll(aPresContext) here to automatically
-    // select text when we receive focus.
+      return;
+    }
+  }
+
+  nsIEventStateManager *esm = aPresContext->EventStateManager();
+  if (esm->SetContentState(this, NS_EVENT_STATE_FOCUS)) {
+    nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
+    if (formControlFrame) {
+      formControlFrame->SetFocus(PR_TRUE, PR_TRUE);
+      nsLayoutUtils::ScrollIntoView(formControlFrame);
+    }
   }
 }
 
@@ -266,13 +290,31 @@ nsHTMLTextAreaElement::Select()
   nsresult rv = NS_OK;
 
   // first see if we are disabled or not. If disabled then do nothing.
-  nsAutoString disabled;
-  if (NS_CONTENT_ATTR_HAS_VALUE ==
-      nsGenericHTMLFormElement::GetAttr(kNameSpaceID_None,
-                                        nsHTMLAtoms::disabled, disabled)) {
+  if (HasAttr(kNameSpaceID_None, nsHTMLAtoms::disabled)) {
     return rv;
   }
 
+  // We can't be focus'd if we aren't in a document
+  nsIDocument* doc = GetCurrentDoc();
+  if (!doc)
+    return rv;
+
+  // If the window is not active, do not allow the focus to bring the
+  // window to the front.  We update the focus controller, but do
+  // nothing else.
+  nsPIDOMWindow* win = doc->GetWindow();
+  if (win) {
+    nsIFocusController *focusController = win->GetRootFocusController();
+    PRBool isActive = PR_FALSE;
+    focusController->GetActive(&isActive);
+    if (!isActive) {
+      focusController->SetFocusedWindow(win);
+      focusController->SetFocusedElement(this);
+
+      return rv;
+    }
+  }
+
   // XXX Bug?  We have to give the input focus before contents can be
   // selected
 
@@ -280,17 +322,19 @@ nsHTMLTextAreaElement::Select()
   nsCOMPtr<nsPresContext> presContext = GetPresContext();
 
   nsEventStatus status = nsEventStatus_eIgnore;
-  nsGUIEvent event(NS_FORM_SELECTED);
-  rv = HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
+  nsGUIEvent event(PR_TRUE, NS_FORM_SELECTED, nsnull);
+  nsEventDispatcher::Dispatch(NS_STATIC_CAST(nsIContent*, this), presContext,
+                              &event, nsnull, &status);
 
   // If the DOM event was not canceled (e.g. by a JS event handler
   // returning false)
   if (status == nsEventStatus_eIgnore) {
     PRBool shouldFocus = ShouldFocus(this);
 
-    if (shouldFocus) {
-      presContext->EventStateManager()->SetContentState(this,
-                                                        NS_EVENT_STATE_FOCUS);
+    if (shouldFocus &&
+        !presContext->EventStateManager()->SetContentState(this,
+                                                           NS_EVENT_STATE_FOCUS)) {
+      return rv; // We ended up unfocused, e.g. due to a DOM event handler.
     }
 
     nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
@@ -314,8 +358,7 @@ nsHTMLTextAreaElement::SelectAll(nsPresContext* aPresContext)
   nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_TRUE);
 
   if (formControlFrame) {
-    formControlFrame->SetProperty(aPresContext, nsHTMLAtoms::select,
-                                  EmptyString());
+    formControlFrame->SetFormProperty(nsHTMLAtoms::select, EmptyString());
   }
 
   return NS_OK;
@@ -352,7 +395,7 @@ nsHTMLTextAreaElement::GetValueInternal(nsAString& aValue, PRBool aIgnoreWrap)
   // No need to flush here, if there is no frame yet for this textarea
   // there won't be a value in it we don't already have even if we
   // force the frame to be created.
-  nsIFrame* primaryFrame = GetPrimaryFrame(PR_FALSE);
+  nsIFrame* primaryFrame = GetPrimaryFrame();
   nsITextControlFrame* textControlFrame = nsnull;
   if (primaryFrame) {
     CallQueryInterface(primaryFrame, &textControlFrame);
@@ -407,8 +450,7 @@ nsHTMLTextAreaElement::SetValueInternal(const nsAString& aValue,
     textControlFrame->OwnsValue(&frameOwnsValue);
   }
   if (frameOwnsValue) {
-    formControlFrame->SetProperty(GetPresContext(),
-                                  nsHTMLAtoms::value, aValue);
+    formControlFrame->SetFormProperty(nsHTMLAtoms::value, aValue);
   }
   else {
     if (mValue) {
@@ -444,37 +486,27 @@ nsHTMLTextAreaElement::SetValueChanged(PRBool aValueChanged)
 NS_IMETHODIMP
 nsHTMLTextAreaElement::GetDefaultValue(nsAString& aDefaultValue)
 {
-  GetContentsAsText(aDefaultValue);
+  nsContentUtils::GetNodeTextContent(this, PR_FALSE, aDefaultValue);
   return NS_OK;
 }  
 
 NS_IMETHODIMP
 nsHTMLTextAreaElement::SetDefaultValue(const nsAString& aDefaultValue)
 {
-  return ReplaceContentsWithText(aDefaultValue, PR_TRUE);
-}
-
-nsresult
-nsHTMLTextAreaElement::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                     PRBool aNotify, PRBool aDeepSetDocument)
-{
-  nsresult rv;
-  rv = nsGenericHTMLFormElement::InsertChildAt(aKid, aIndex, aNotify,
-                                               aDeepSetDocument);
-  if (!mValueChanged) {
+  nsresult rv = nsContentUtils::SetNodeTextContent(this, aDefaultValue, PR_TRUE);
+  if (NS_SUCCEEDED(rv) && !mValueChanged) {
     Reset();
   }
   return rv;
 }
 
 nsresult
-nsHTMLTextAreaElement::AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                     PRBool aDeepSetDocument)
+nsHTMLTextAreaElement::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
+                                     PRBool aNotify)
 {
   nsresult rv;
-  rv = nsGenericHTMLFormElement::AppendChildTo(aKid, aNotify,
-                                               aDeepSetDocument);
-  if (!mValueChanged) {
+  rv = nsGenericHTMLFormElement::InsertChildAt(aKid, aIndex, aNotify);
+  if (!mValueChanged && mDoneAddingChildren) {
     Reset();
   }
   return rv;
@@ -486,23 +518,29 @@ nsHTMLTextAreaElement::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
   nsresult rv;
   rv = nsGenericHTMLFormElement::RemoveChildAt(aIndex, aNotify);
   if (!mValueChanged) {
+    NS_ASSERTION(mDoneAddingChildren,
+                 "The HTML content sink shouldn't call this");
     Reset();
   }
   return rv;
 }
 
 PRBool
-nsHTMLTextAreaElement::ParseAttribute(nsIAtom* aAttribute,
+nsHTMLTextAreaElement::ParseAttribute(PRInt32 aNamespaceID,
+                                      nsIAtom* aAttribute,
                                       const nsAString& aValue,
                                       nsAttrValue& aResult)
 {
-  if (aAttribute == nsHTMLAtoms::cols) {
-    return aResult.ParseIntWithBounds(aValue, 0);
+  if (aNamespaceID == kNameSpaceID_None) {
+    if (aAttribute == nsHTMLAtoms::cols) {
+      return aResult.ParseIntWithBounds(aValue, 0);
+    }
+    if (aAttribute == nsHTMLAtoms::rows) {
+      return aResult.ParseIntWithBounds(aValue, 0);
+    }
   }
-  if (aAttribute == nsHTMLAtoms::rows) {
-    return aResult.ParseIntWithBounds(aValue, 0);
-  }
-  return nsGenericHTMLElement::ParseAttribute(aAttribute, aValue, aResult);
+  return nsGenericHTMLElement::ParseAttribute(aNamespaceID, aAttribute, aValue,
+                                              aResult);
 }
 
 static void
@@ -537,22 +575,17 @@ nsHTMLTextAreaElement::IsAttributeMapped(const nsIAtom* aAttribute) const
   return FindAttributeDependence(aAttribute, map, NS_ARRAY_LENGTH(map));
 }
 
-NS_IMETHODIMP
-nsHTMLTextAreaElement::GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const
+nsMapRuleToAttributesFunc
+nsHTMLTextAreaElement::GetAttributeMappingFunction() const
 {
-  aMapRuleFunc = &MapAttributesIntoRule;
-  return NS_OK;
+  return &MapAttributesIntoRule;
 }
 
-
 nsresult
-nsHTMLTextAreaElement::HandleDOMEvent(nsPresContext* aPresContext,
-                                      nsEvent* aEvent,
-                                      nsIDOMEvent** aDOMEvent,
-                                      PRUint32 aFlags,
-                                      nsEventStatus* aEventStatus)
+nsHTMLTextAreaElement::PreHandleEvent(nsEventChainPreVisitor& aVisitor)
 {
   // Do not process any DOM events if the element is disabled
+  aVisitor.mCanHandle = PR_FALSE;
   PRBool disabled;
   nsresult rv = GetDisabled(&disabled);
   if (NS_FAILED(rv) || disabled) {
@@ -573,44 +606,55 @@ nsHTMLTextAreaElement::HandleDOMEvent(nsPresContext* aPresContext,
     }
   }
 
-  PRBool isSelectEvent = (aEvent->message == NS_FORM_SELECTED);
   // Don't dispatch a second select event if we are already handling
   // one.
-  if (isSelectEvent && mHandlingSelect) {
-    return NS_OK;
+  if (aVisitor.mEvent->message == NS_FORM_SELECTED) {
+    if (mHandlingSelect) {
+      return NS_OK;
+    }
+    mHandlingSelect = PR_TRUE;
   }
 
   // If NS_EVENT_FLAG_NO_CONTENT_DISPATCH is set we will not allow content to handle
   // this event.  But to allow middle mouse button paste to work we must allow 
   // middle clicks to go to text fields anyway.
-  PRBool noContentDispatch = aEvent->flags & NS_EVENT_FLAG_NO_CONTENT_DISPATCH;
-  if (aEvent->message == NS_MOUSE_MIDDLE_CLICK) {
-    aEvent->flags &= ~NS_EVENT_FLAG_NO_CONTENT_DISPATCH;
+  if (aVisitor.mEvent->flags & NS_EVENT_FLAG_NO_CONTENT_DISPATCH)
+    aVisitor.mItemFlags |= NS_NO_CONTENT_DISPATCH;
+  if (aVisitor.mEvent->message == NS_MOUSE_MIDDLE_CLICK) {
+    aVisitor.mEvent->flags &= ~NS_EVENT_FLAG_NO_CONTENT_DISPATCH;
   }
 
-  if (isSelectEvent) {
-    mHandlingSelect = PR_TRUE;
-  }
+  return nsGenericHTMLElement::PreHandleEvent(aVisitor);
+}
 
-  rv = nsGenericHTMLFormElement::HandleDOMEvent(aPresContext, aEvent,
-                                                aDOMEvent, aFlags,
-                                                aEventStatus);
-
-  if (isSelectEvent) {
+nsresult
+nsHTMLTextAreaElement::PostHandleEvent(nsEventChainPostVisitor& aVisitor)
+{
+  if (aVisitor.mEvent->message == NS_FORM_SELECTED) {
     mHandlingSelect = PR_FALSE;
   }
 
   // Reset the flag for other content besides this text field
-  aEvent->flags |= noContentDispatch ? NS_EVENT_FLAG_NO_CONTENT_DISPATCH : NS_EVENT_FLAG_NONE;
+  aVisitor.mEvent->flags |= (aVisitor.mItemFlags & NS_NO_CONTENT_DISPATCH)
+    ? NS_EVENT_FLAG_NO_CONTENT_DISPATCH : NS_EVENT_FLAG_NONE;
 
-  return rv;
+  return NS_OK;
 }
 
 void
-nsHTMLTextAreaElement::DoneAddingChildren()
+nsHTMLTextAreaElement::DoneAddingChildren(PRBool aHaveNotified)
 {
+  if (!mValueChanged) {
+    if (!mDoneAddingChildren) {
+      // Reset now that we're done adding children if the content sink tried to
+      // sneak some text in without calling AppendChildTo.
+      Reset();
+    }
+
+    RestoreFormControlState(this, this);
+  }
+
   mDoneAddingChildren = PR_TRUE;
-  RestoreFormControlState(this, this);
 }
 
 PRBool
@@ -619,19 +663,6 @@ nsHTMLTextAreaElement::IsDoneAddingChildren()
   return mDoneAddingChildren;
 }
 
-nsresult
-nsHTMLTextAreaElement::GetInnerHTML(nsAString& aInnerHTML)
-{
-  GetContentsAsText(aInnerHTML);
-  return NS_OK;
-}
-
-nsresult
-nsHTMLTextAreaElement::SetInnerHTML(const nsAString& aInnerHTML)
-{
-  return ReplaceContentsWithText(aInnerHTML, PR_TRUE);
-}
-
 // Controllers Methods
 
 NS_IMETHODIMP
@@ -767,7 +798,6 @@ nsHTMLTextAreaElement::Reset()
     GetDefaultValue(resetVal);
     rv = SetValue(resetVal);
     NS_ENSURE_SUCCESS(rv, rv);
-    formControlFrame->OnContentReset();
   }
   SetValueChanged(PR_FALSE);
   return NS_OK;
@@ -792,9 +822,8 @@ nsHTMLTextAreaElement::SubmitNamesValues(nsIFormSubmission* aFormSubmission,
   // Get the name (if no name, no submit)
   //
   nsAutoString name;
-  rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, name);
-  if (NS_FAILED(rv) || rv == NS_CONTENT_ATTR_NOT_THERE) {
-    return rv;
+  if (!GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, name)) {
+    return NS_OK;
   }
 
   //
@@ -819,8 +848,8 @@ nsHTMLTextAreaElement::SaveState()
 
   // Only save if value != defaultValue (bug 62713)
   if (mValueChanged) {
-    nsCOMPtr<nsIPresState> state;
-    rv = GetPrimaryPresState(this, getter_AddRefs(state));
+    nsPresState *state = nsnull;
+    rv = GetPrimaryPresState(this, &state);
     if (state) {
       nsAutoString value;
       GetValueInternal(value, PR_TRUE);
@@ -839,7 +868,7 @@ nsHTMLTextAreaElement::SaveState()
 }
 
 PRBool
-nsHTMLTextAreaElement::RestoreState(nsIPresState* aState)
+nsHTMLTextAreaElement::RestoreState(nsPresState* aState)
 {
   nsAutoString value;
 #ifdef DEBUG
