@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set sw=2 ts=2 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -20,6 +21,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Robert Sayre <sayrer@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -40,7 +42,6 @@
 #include "nsIHTMLContentSink.h"
 #include "nsIParser.h"
 #include "nsIParserService.h"
-#include "nsIHTMLContent.h"
 #include "nsHTMLAtoms.h"
 #include "nsHTMLTokens.h"
 #include "nsGenericHTMLElement.h"
@@ -49,8 +50,6 @@
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMDocumentFragment.h"
 #include "nsVoidArray.h"
-#include "nsITextContent.h"
-#include "nsINameSpace.h"
 #include "nsINameSpaceManager.h"
 #include "nsIDocument.h"
 #include "nsINodeInfo.h"
@@ -61,6 +60,10 @@
 #include "nsEscape.h"
 #include "nsNodeInfoManager.h"
 #include "nsContentCreatorFunctions.h"
+#include "nsNetUtil.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsContentSink.h"
+#include "nsTHashtable.h"
 
 //
 // XXX THIS IS TEMPORARY CODE
@@ -84,31 +87,19 @@ public:
   NS_IMETHOD WillInterrupt(void);
   NS_IMETHOD WillResume(void);
   NS_IMETHOD SetParser(nsIParser* aParser);
-  virtual void FlushContent(PRBool aNotify) { }
+  virtual void FlushPendingNotifications(mozFlushType aType) { }
   NS_IMETHOD SetDocumentCharset(nsACString& aCharset) { return NS_OK; }
   virtual nsISupports *GetTarget() { return mTargetDocument; }
 
   // nsIHTMLContentSink
   NS_IMETHOD BeginContext(PRInt32 aID);
   NS_IMETHOD EndContext(PRInt32 aID);
-  NS_IMETHOD SetTitle(const nsString& aValue);
-  NS_IMETHOD OpenHTML(const nsIParserNode& aNode);
-  NS_IMETHOD CloseHTML();
-  NS_IMETHOD OpenHead(const nsIParserNode& aNode);
-  NS_IMETHOD CloseHead();
-  NS_IMETHOD OpenBody(const nsIParserNode& aNode);
-  NS_IMETHOD CloseBody();
-  NS_IMETHOD OpenForm(const nsIParserNode& aNode);
-  NS_IMETHOD CloseForm();
-  NS_IMETHOD OpenFrameset(const nsIParserNode& aNode);
-  NS_IMETHOD CloseFrameset();
+  NS_IMETHOD OpenHead();
   NS_IMETHOD IsEnabled(PRInt32 aTag, PRBool* aReturn) {
     *aReturn = PR_TRUE;
     return NS_OK;
   }
   NS_IMETHOD_(PRBool) IsFormOnStack() { return PR_FALSE; }
-  NS_IMETHOD OpenMap(const nsIParserNode& aNode);
-  NS_IMETHOD CloseMap();
   NS_IMETHOD WillProcessTokens(void) { return NS_OK; }
   NS_IMETHOD DidProcessTokens(void) { return NS_OK; }
   NS_IMETHOD WillProcessAToken(void) { return NS_OK; }
@@ -116,7 +107,6 @@ public:
   NS_IMETHOD NotifyTagObservers(nsIParserNode* aNode) { return NS_OK; }
   NS_IMETHOD OpenContainer(const nsIParserNode& aNode);
   NS_IMETHOD CloseContainer(const nsHTMLTag aTag);
-  NS_IMETHOD AddHeadContent(const nsIParserNode& aNode);
   NS_IMETHOD AddLeaf(const nsIParserNode& aNode);
   NS_IMETHOD AddComment(const nsIParserNode& aNode);
   NS_IMETHOD AddProcessingInstruction(const nsIParserNode& aNode);
@@ -125,16 +115,19 @@ public:
   // nsIFragmentContentSink
   NS_IMETHOD GetFragment(nsIDOMDocumentFragment** aFragment);
   NS_IMETHOD SetTargetDocument(nsIDocument* aDocument);
+  NS_IMETHOD WillBuildContent();
+  NS_IMETHOD DidBuildContent();
+  NS_IMETHOD IgnoreFirstContainer();
 
   nsIContent* GetCurrentContent();
   PRInt32 PushContent(nsIContent *aContent);
   nsIContent* PopContent();
 
-  nsresult AddAttributes(const nsIParserNode& aNode,
-                         nsIContent* aContent);
+  virtual nsresult AddAttributes(const nsIParserNode& aNode,
+                                 nsIContent* aContent);
 
   nsresult AddText(const nsAString& aString);
-  nsresult AddTextToContent(nsIContent* aContent,const nsString& aText);
+  nsresult AddTextToContent(nsIContent* aContent, const nsAString& aText);
   nsresult FlushText();
 
   void ProcessBaseTag(nsIContent* aContent);
@@ -142,13 +135,14 @@ public:
 
   nsresult Init();
 
-  PRBool mHitSentinel;
-  PRBool mSeenBody;
+  PRPackedBool mAllContent;
+  PRPackedBool mProcessing;
+  PRPackedBool mSeenBody;
+  PRPackedBool mIgnoreContainer;
+  PRPackedBool mIgnoreNextCloseHead;
 
-  nsIContent* mRoot;
-  nsIParser* mParser;
-  nsIDOMHTMLFormElement* mCurrentForm;
-  nsIHTMLContent* mCurrentMap;
+  nsCOMPtr<nsIContent> mRoot;
+  nsCOMPtr<nsIParser> mParser;
 
   nsVoidArray* mContentStack;
 
@@ -156,8 +150,8 @@ public:
   PRInt32 mTextLength;
   PRInt32 mTextSize;
 
-  nsString mBaseHREF;
-  nsString mBaseTarget;
+  nsCOMPtr<nsIURI> mBaseHref;
+  nsCOMPtr<nsIAtom> mBaseTarget;
 
   nsCOMPtr<nsIDocument> mTargetDocument;
   nsRefPtr<nsNodeInfoManager> mNodeInfoManager;
@@ -194,22 +188,16 @@ NS_NewHTMLFragmentContentSink(nsIFragmentContentSink** aResult)
 }
 
 nsHTMLFragmentContentSink::nsHTMLFragmentContentSink(PRBool aAllContent)
+  : mAllContent(aAllContent),
+    mProcessing(aAllContent),
+    mSeenBody(!aAllContent),
+    mIgnoreContainer(PR_FALSE),
+    mIgnoreNextCloseHead(PR_FALSE),
+    mContentStack(nsnull),
+    mText(nsnull),
+    mTextLength(0),
+    mTextSize(0)
 {
-  if (aAllContent) {
-    mHitSentinel = PR_TRUE;
-    mSeenBody = PR_FALSE;
-  } else {
-    mHitSentinel = PR_FALSE;
-    mSeenBody = PR_TRUE;
-  }
-  mRoot = nsnull;
-  mParser = nsnull;
-  mCurrentForm = nsnull;
-  mCurrentMap = nsnull;
-  mContentStack = nsnull;
-  mText = nsnull;
-  mTextLength = 0;
-  mTextSize = 0;
 }
 
 nsHTMLFragmentContentSink::~nsHTMLFragmentContentSink()
@@ -217,10 +205,6 @@ nsHTMLFragmentContentSink::~nsHTMLFragmentContentSink()
   // Should probably flush the text buffer here, just to make sure:
   //FlushText();
 
-  NS_IF_RELEASE(mRoot);
-  NS_IF_RELEASE(mParser);
-  NS_IF_RELEASE(mCurrentForm);
-  NS_IF_RELEASE(mCurrentMap);
   if (nsnull != mContentStack) {
     // there shouldn't be anything here except in an error condition
     PRInt32 indx = mContentStack->Count();
@@ -252,13 +236,15 @@ nsHTMLFragmentContentSink::WillBuildModel(void)
     return NS_OK;
   }
 
-  NS_ASSERTION(mTargetDocument, "Need a document!");
+  NS_ASSERTION(mNodeInfoManager, "Need a nodeinfo manager!");
 
   nsCOMPtr<nsIDOMDocumentFragment> frag;
-  nsresult rv = NS_NewDocumentFragment(getter_AddRefs(frag), mTargetDocument);
+  nsresult rv = NS_NewDocumentFragment(getter_AddRefs(frag), mNodeInfoManager);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  return CallQueryInterface(frag, &mRoot);
+  mRoot = do_QueryInterface(frag, &rv);
+  
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -268,7 +254,7 @@ nsHTMLFragmentContentSink::DidBuildModel(void)
 
   // Drop our reference to the parser to get rid of a circular
   // reference.
-  NS_IF_RELEASE(mParser);
+  mParser = nsnull;
 
   return NS_OK;
 }
@@ -288,9 +274,7 @@ nsHTMLFragmentContentSink::WillResume(void)
 NS_IMETHODIMP
 nsHTMLFragmentContentSink::SetParser(nsIParser* aParser)
 {
-  NS_IF_RELEASE(mParser);
   mParser = aParser;
-  NS_IF_ADDREF(mParser);
 
   return NS_OK;
 }
@@ -308,140 +292,65 @@ nsHTMLFragmentContentSink::EndContext(PRInt32 aID)
 }
 
 NS_IMETHODIMP
-nsHTMLFragmentContentSink::SetTitle(const nsString& aValue)
-{
-  NS_ENSURE_TRUE(mNodeInfoManager, NS_ERROR_NOT_INITIALIZED);
-
-  nsresult result=NS_OK;
-
-  nsCOMPtr<nsINodeInfo> nodeInfo;
-  result = mNodeInfoManager->GetNodeInfo(nsHTMLAtoms::title, nsnull,
-                                         kNameSpaceID_None,
-                                         getter_AddRefs(nodeInfo));
-  if(NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIHTMLContent> content = NS_NewHTMLTitleElement(nodeInfo);
-
-    if (!content) {
-      result = NS_ERROR_OUT_OF_MEMORY;
-    } else {
-      nsIContent *parent = GetCurrentContent();
-
-      if (nsnull == parent) {
-        parent = mRoot;
-      }
-
-      result=parent->AppendChildTo(content, PR_FALSE, PR_FALSE);
-
-      if (NS_SUCCEEDED(result)) {
-        result=AddTextToContent(content,aValue);
-      }
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::OpenHTML(const nsIParserNode& aNode)
+nsHTMLFragmentContentSink::OpenHead()
 {
+  mIgnoreNextCloseHead = PR_TRUE;
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::CloseHTML()
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::OpenHead(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::CloseHead()
-{
-  return CloseContainer(eHTMLTag_head);
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::OpenBody(const nsIParserNode& aNode)
-{
-  // Ignore repeated BODY elements. The DTD is just sending them
-  // to us for compatibility reasons that don't apply here.
-  if (!mSeenBody) {
-    mSeenBody = PR_TRUE;
-    return OpenContainer(aNode);
-  }
-  else {
-    return NS_OK;
-  }
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::CloseBody()
-{
-  return CloseContainer(eHTMLTag_body);
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::OpenForm(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::CloseForm()
-{
-  return CloseContainer(eHTMLTag_form);
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::OpenFrameset(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::CloseFrameset()
-{
-  return CloseContainer(eHTMLTag_frameset);
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::OpenMap(const nsIParserNode& aNode)
-{
-  return OpenContainer(aNode);
-}
-
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::CloseMap()
-{
-  return CloseContainer(eHTMLTag_map);
-}
-
 void
 nsHTMLFragmentContentSink::ProcessBaseTag(nsIContent* aContent)
 {
   nsAutoString value;
-  if (NS_CONTENT_ATTR_HAS_VALUE == aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::href, value)) {
-    mBaseHREF = value;
+  if (aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::href, value)) {
+    nsCOMPtr<nsIURI> baseHrefURI;
+    nsresult rv = 
+      nsContentUtils::NewURIWithDocumentCharset(getter_AddRefs(baseHrefURI),
+                                                value, mTargetDocument,
+                                                nsnull);
+    if (NS_FAILED(rv))
+      return;
+
+    nsIScriptSecurityManager *securityManager =
+      nsContentUtils::GetSecurityManager();
+
+    NS_ASSERTION(aContent->NodePrincipal() == mTargetDocument->NodePrincipal(),
+                 "How'd that happpen?");
+    
+    rv = securityManager->
+      CheckLoadURIWithPrincipal(mTargetDocument->NodePrincipal(), baseHrefURI,
+                                nsIScriptSecurityManager::STANDARD);
+    if (NS_SUCCEEDED(rv)) {
+      mBaseHref = baseHrefURI;
+    }
   }
-  if (NS_CONTENT_ATTR_HAS_VALUE == aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::target, value)) {
-    mBaseTarget = value;
+  if (aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::target, value)) {
+    mBaseTarget = do_GetAtom(value);
   }
 }
 
 void
 nsHTMLFragmentContentSink::AddBaseTagInfo(nsIContent* aContent)
 {
-  if (aContent) {
-    if (!mBaseHREF.IsEmpty()) {
-      aContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::_baseHref, mBaseHREF, PR_FALSE);
+  if (!aContent) {
+    return;
+  }
+
+  nsresult rv;
+  if (mBaseHref) {
+    rv = aContent->SetProperty(nsHTMLAtoms::htmlBaseHref, mBaseHref,
+                               nsPropertyTable::SupportsDtorFunc, PR_TRUE);
+    if (NS_SUCCEEDED(rv)) {
+      // circumvent nsDerivedSafe
+      NS_ADDREF(NS_STATIC_CAST(nsIURI*, mBaseHref));
     }
-    if (!mBaseTarget.IsEmpty()) {
-      aContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::_baseTarget, mBaseTarget, PR_FALSE);
+  }
+  if (mBaseTarget) {
+    rv = aContent->SetProperty(nsHTMLAtoms::htmlBaseTarget, mBaseTarget,
+                               nsPropertyTable::SupportsDtorFunc, PR_TRUE);
+    if (NS_SUCCEEDED(rv)) {
+      // circumvent nsDerivedSafe
+      NS_ADDREF(NS_STATIC_CAST(nsIAtom*, mBaseTarget));
     }
   }
 }
@@ -451,15 +360,22 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
 {
   NS_ENSURE_TRUE(mNodeInfoManager, NS_ERROR_NOT_INITIALIZED);
 
-  nsAutoString tag;
   nsresult result = NS_OK;
 
-  tag.Assign(aNode.GetText());
-
-  if (nsHTMLAtoms::endnote->Equals(tag)) {
-    mHitSentinel = PR_TRUE;
+  if (aNode.GetNodeType() == eHTMLTag_html) {
+    return NS_OK;
   }
-  else if (mHitSentinel) {
+
+  // Ignore repeated BODY elements. The DTD is just sending them
+  // to us for compatibility reasons that don't apply here.
+  if (aNode.GetNodeType() == eHTMLTag_body) {
+    if (mSeenBody) {
+      return NS_OK;
+    }
+    mSeenBody = PR_TRUE;
+  }
+
+  if (mProcessing && !mIgnoreContainer) {
     FlushText();
 
     nsHTMLTag nodeType = nsHTMLTag(aNode.GetNodeType());
@@ -473,20 +389,15 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
                                       kNameSpaceID_None,
                                       getter_AddRefs(nodeInfo));
     } else {
-      nsIParserService* parserService =
-        nsContentUtils::GetParserServiceWeakRef();
+      nsIParserService* parserService = nsContentUtils::GetParserService();
       if (!parserService)
         return NS_ERROR_OUT_OF_MEMORY;
 
-      const PRUnichar *name = nsnull;
-
-      parserService->HTMLIdToStringTag(nodeType, &name);
+      nsIAtom *name = parserService->HTMLIdToAtomTag(nodeType);
       NS_ASSERTION(name, "This should not happen!");
 
-      result =
-        mNodeInfoManager->GetNodeInfo(nsDependentString(name), nsnull,
-                                      kNameSpaceID_None,
-                                      getter_AddRefs(nodeInfo));
+      result = mNodeInfoManager->GetNodeInfo(name, nsnull, kNameSpaceID_None,
+                                             getter_AddRefs(nodeInfo));
     }
 
     NS_ENSURE_SUCCESS(result, result);
@@ -502,7 +413,7 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
           parent = mRoot;
         }
 
-        parent->AppendChildTo(content, PR_FALSE, PR_FALSE);
+        parent->AppendChildTo(content, PR_FALSE);
         PushContent(content);
       }
     }
@@ -517,6 +428,9 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
       // XXX if navigator_quirks_mode (only body in html supports background)
       AddBaseTagInfo(content); 
   }
+  else if (mProcessing && mIgnoreContainer) {
+    mIgnoreContainer = PR_FALSE;
+  }
 
   return result;
 }
@@ -524,7 +438,15 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
 NS_IMETHODIMP
 nsHTMLFragmentContentSink::CloseContainer(const nsHTMLTag aTag)
 {
-  if (mHitSentinel && (nsnull != GetCurrentContent())) {
+  if (aTag == eHTMLTag_html) {
+    return NS_OK;
+  }
+  if (mIgnoreNextCloseHead && aTag == eHTMLTag_head) {
+    mIgnoreNextCloseHead = PR_FALSE;
+    return NS_OK;
+  }
+
+  if (mProcessing && (nsnull != GetCurrentContent())) {
     nsIContent* content;
     FlushText();
     content = PopContent();
@@ -533,12 +455,6 @@ nsHTMLFragmentContentSink::CloseContainer(const nsHTMLTag aTag)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLFragmentContentSink::AddHeadContent(const nsIParserNode& aNode)
-{
-  return AddLeaf(aNode);
-}
-
 NS_IMETHODIMP
 nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
 {
@@ -555,8 +471,7 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
         nsCOMPtr<nsIContent> content;
         nsHTMLTag nodeType = nsHTMLTag(aNode.GetNodeType());
 
-        nsIParserService* parserService =
-          nsContentUtils::GetParserServiceWeakRef();
+        nsIParserService* parserService = nsContentUtils::GetParserService();
         if (!parserService)
           return NS_ERROR_OUT_OF_MEMORY;
 
@@ -568,14 +483,12 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
                                           kNameSpaceID_None,
                                           getter_AddRefs(nodeInfo));
         } else {
-          const PRUnichar *name = nsnull;
-          result = parserService->HTMLIdToStringTag(nodeType, &name);
+          nsIAtom *name = parserService->HTMLIdToAtomTag(nodeType);
           NS_ASSERTION(name, "This should not happen!");
 
-          result =
-            mNodeInfoManager->GetNodeInfo(nsDependentString(name), nsnull,
-                                          kNameSpaceID_None,
-                                          getter_AddRefs(nodeInfo));
+          result = mNodeInfoManager->GetNodeInfo(name, nsnull,
+                                                 kNameSpaceID_None,
+                                                 getter_AddRefs(nodeInfo));
         }
 
         NS_ENSURE_SUCCESS(result, result);
@@ -592,27 +505,11 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
                 parent = mRoot;
               }
 
-              parent->AppendChildTo(content, PR_FALSE, PR_FALSE);
+              parent->AppendChildTo(content, PR_FALSE);
             }
           }
 
-          if(nodeType == eHTMLTag_script    ||
-             nodeType == eHTMLTag_style     ||
-             nodeType == eHTMLTag_textarea  ||
-             nodeType == eHTMLTag_xmp) {
-
-            // Create a text node holding the content
-            nsCOMPtr<nsIDTD> dtd;
-            mParser->GetDTD(getter_AddRefs(dtd));
-            NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
-
-            nsAutoString skippedContent;
-            PRInt32 lineNo = 0;
-
-            dtd->CollectSkippedContent(nodeType, skippedContent, lineNo);
-            result=AddTextToContent(content, skippedContent);
-          }
-          else if (nodeType == eHTMLTag_img || nodeType == eHTMLTag_frame
+          if (nodeType == eHTMLTag_img || nodeType == eHTMLTag_frame
               || nodeType == eHTMLTag_input)    // elements with 'SRC='
               AddBaseTagInfo(content);
           else if (nodeType == eHTMLTag_base)
@@ -646,28 +543,22 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
 NS_IMETHODIMP
 nsHTMLFragmentContentSink::AddComment(const nsIParserNode& aNode)
 {
-  nsIContent *comment;
-  nsIDOMComment *domComment;
+  nsCOMPtr<nsIContent> comment;
   nsresult result = NS_OK;
 
   FlushText();
 
-  result = NS_NewCommentNode(&comment);
+  result = NS_NewCommentNode(getter_AddRefs(comment), mNodeInfoManager);
   if (NS_SUCCEEDED(result)) {
-    result = CallQueryInterface(comment, &domComment);
-    if (NS_SUCCEEDED(result)) {
-      domComment->AppendData(aNode.GetText());
-      NS_RELEASE(domComment);
+    comment->SetText(aNode.GetText(), PR_FALSE);
 
-      nsIContent *parent = GetCurrentContent();
+    nsIContent *parent = GetCurrentContent();
 
-      if (nsnull == parent) {
-        parent = mRoot;
-      }
-
-      parent->AppendChildTo(comment, PR_FALSE, PR_FALSE);
+    if (nsnull == parent) {
+      parent = mRoot;
     }
-    NS_RELEASE(comment);
+
+    parent->AppendChildTo(comment, PR_FALSE);
   }
 
   return NS_OK;
@@ -713,6 +604,33 @@ nsHTMLFragmentContentSink::SetTargetDocument(nsIDocument* aTargetDocument)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsHTMLFragmentContentSink::WillBuildContent()
+{
+  mProcessing = PR_TRUE;
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLFragmentContentSink::DidBuildContent()
+{
+  if (!mAllContent) {
+    FlushText();
+    DidBuildModel(); // Release our ref to the parser now.
+    mProcessing = PR_FALSE;
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLFragmentContentSink::IgnoreFirstContainer()
+{
+  mIgnoreContainer = PR_TRUE;
+  return NS_OK;
+}
+
 nsIContent*
 nsHTMLFragmentContentSink::GetCurrentContent()
 {
@@ -796,19 +714,19 @@ nsHTMLFragmentContentSink::AddText(const nsAString& aString)
 }
 
 nsresult
-nsHTMLFragmentContentSink::AddTextToContent(nsIContent* aContent,const nsString& aText) {
+nsHTMLFragmentContentSink::AddTextToContent(nsIContent* aContent, const nsAString& aText) {
   NS_ASSERTION(aContent !=nsnull, "can't add text w/o a content");
 
   nsresult result=NS_OK;
 
   if(aContent) {
     if (!aText.IsEmpty()) {
-      nsCOMPtr<nsITextContent> text;
-      result = NS_NewTextNode(getter_AddRefs(text));
+      nsCOMPtr<nsIContent> text;
+      result = NS_NewTextNode(getter_AddRefs(text), mNodeInfoManager);
       if (NS_SUCCEEDED(result)) {
         text->SetText(aText, PR_TRUE);
 
-        result = aContent->AppendChildTo(text, PR_FALSE, PR_FALSE);
+        result = aContent->AppendChildTo(text, PR_FALSE);
       }
     }
   }
@@ -822,8 +740,8 @@ nsHTMLFragmentContentSink::FlushText()
     return NS_OK;
   }
 
-  nsCOMPtr<nsITextContent> content;
-  nsresult rv = NS_NewTextNode(getter_AddRefs(content));
+  nsCOMPtr<nsIContent> content;
+  nsresult rv = NS_NewTextNode(getter_AddRefs(content), mNodeInfoManager);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Set the text in the text node
@@ -836,7 +754,7 @@ nsHTMLFragmentContentSink::FlushText()
     parent = mRoot;
   }
 
-  rv = parent->AppendChildTo(content, PR_FALSE, PR_FALSE);
+  rv = parent->AppendChildTo(content, PR_FALSE);
 
   mTextLength = 0;
 
@@ -887,8 +805,8 @@ nsHTMLFragmentContentSink::AddAttributes(const nsIParserNode& aNode,
       nsContentUtils::TrimCharsInSet(kWhitespace, aNode.GetValueAt(i));
 
     if (nodeType == eHTMLTag_a && keyAtom == nsHTMLAtoms::name) {
-      NS_ConvertUCS2toUTF8 cname(v);
-      NS_ConvertUTF8toUCS2 uv(nsUnescape(cname.BeginWriting()));
+      NS_ConvertUTF16toUTF8 cname(v);
+      NS_ConvertUTF8toUTF16 uv(nsUnescape(cname.BeginWriting()));
 
       // Add attribute to content
       aContent->SetAttr(kNameSpaceID_None, keyAtom, uv, PR_FALSE);
@@ -900,3 +818,361 @@ nsHTMLFragmentContentSink::AddAttributes(const nsIParserNode& aNode,
 
   return NS_OK;
 }
+
+// nsHTMLParanoidFragmentSink
+
+// Find the whitelist of allowed elements and attributes in
+// nsContentSink.h We share it with nsHTMLParanoidFragmentSink
+
+class nsHTMLParanoidFragmentSink : public nsHTMLFragmentContentSink
+{
+public:
+  nsHTMLParanoidFragmentSink();
+
+  static nsresult Init();
+  static void Cleanup();
+
+  // nsISupports
+  NS_DECL_ISUPPORTS_INHERITED
+
+  NS_IMETHOD OpenContainer(const nsIParserNode& aNode);
+  NS_IMETHOD CloseContainer(const nsHTMLTag aTag);
+  NS_IMETHOD AddLeaf(const nsIParserNode& aNode);
+  NS_IMETHOD AddComment(const nsIParserNode& aNode);
+  NS_IMETHOD AddProcessingInstruction(const nsIParserNode& aNode);
+
+  nsresult AddAttributes(const nsIParserNode& aNode,
+                         nsIContent* aContent);
+protected:
+  nsresult NameFromType(const nsHTMLTag aTag,
+                        nsIAtom **aResult);
+
+  nsresult NameFromNode(const nsIParserNode& aNode,
+                        nsIAtom **aResult);
+  
+  PRBool mSkip; // used when we descend into <style> or <script>
+
+  // Use nsTHashTable as a hash set for our whitelists
+  static nsTHashtable<nsISupportsHashKey>* sAllowedTags;
+  static nsTHashtable<nsISupportsHashKey>* sAllowedAttributes;
+};
+
+nsTHashtable<nsISupportsHashKey>* nsHTMLParanoidFragmentSink::sAllowedTags;
+nsTHashtable<nsISupportsHashKey>* nsHTMLParanoidFragmentSink::sAllowedAttributes;
+
+nsHTMLParanoidFragmentSink::nsHTMLParanoidFragmentSink():
+  nsHTMLFragmentContentSink(PR_FALSE), mSkip(PR_FALSE)
+{
+}
+
+nsresult
+nsHTMLParanoidFragmentSink::Init()
+{
+  nsresult rv = NS_ERROR_FAILURE;
+  
+  if (sAllowedTags) {
+    return NS_OK;
+  }
+
+  PRUint32 size = NS_ARRAY_LENGTH(kDefaultAllowedTags);
+  sAllowedTags = new nsTHashtable<nsISupportsHashKey>();
+  if (sAllowedTags) {
+    rv = sAllowedTags->Init(size);
+    for (PRUint32 i = 0; i < size && NS_SUCCEEDED(rv); i++) {
+      if (!sAllowedTags->PutEntry(*kDefaultAllowedTags[i])) {
+        rv = NS_ERROR_OUT_OF_MEMORY;
+      }
+    }
+  }
+
+  size = NS_ARRAY_LENGTH(kDefaultAllowedAttributes);
+  sAllowedAttributes = new nsTHashtable<nsISupportsHashKey>();
+  if (sAllowedAttributes && NS_SUCCEEDED(rv)) {
+    rv = sAllowedAttributes->Init(size);
+    for (PRUint32 i = 0; i < size && NS_SUCCEEDED(rv); i++) {
+      if (!sAllowedAttributes->PutEntry(*kDefaultAllowedAttributes[i])) {
+        rv = NS_ERROR_OUT_OF_MEMORY;
+      }
+    }
+  }
+
+  if (NS_FAILED(rv)) {
+    NS_WARNING("Failed to populate whitelist hash sets");
+    Cleanup();
+    return rv; 
+  }
+
+  return rv;
+}
+
+void
+nsHTMLParanoidFragmentSink::Cleanup()
+{
+  if (sAllowedTags) {
+    delete sAllowedTags;
+    sAllowedTags = nsnull;
+  }
+  
+  if (sAllowedAttributes) {
+    delete sAllowedAttributes;
+    sAllowedAttributes = nsnull;
+  }
+}
+
+nsresult
+NS_NewHTMLParanoidFragmentSink(nsIFragmentContentSink** aResult)
+{
+  nsHTMLParanoidFragmentSink* it = new nsHTMLParanoidFragmentSink();
+  if (!it) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+  nsresult rv = nsHTMLParanoidFragmentSink::Init();
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ADDREF(*aResult = it);
+  
+  return NS_OK;
+}
+
+void
+NS_HTMLParanoidFragmentSinkShutdown()
+{
+  nsHTMLParanoidFragmentSink::Cleanup();
+}
+
+NS_IMPL_ISUPPORTS_INHERITED0(nsHTMLParanoidFragmentSink,
+                             nsHTMLFragmentContentSink)
+
+nsresult
+nsHTMLParanoidFragmentSink::NameFromType(const nsHTMLTag aTag,
+                                         nsIAtom **aResult)
+{
+  nsIParserService* parserService = nsContentUtils::GetParserService();
+  if (!parserService) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+  NS_IF_ADDREF(*aResult = parserService->HTMLIdToAtomTag(aTag));
+  
+  return NS_OK;
+}
+
+nsresult
+nsHTMLParanoidFragmentSink::NameFromNode(const nsIParserNode& aNode,
+                                         nsIAtom **aResult)
+{
+  nsresult rv;
+  eHTMLTags type = (eHTMLTags)aNode.GetNodeType();
+  
+  *aResult = nsnull;
+  if (type == eHTMLTag_userdefined) {
+    nsCOMPtr<nsINodeInfo> nodeInfo;
+    rv =
+      mNodeInfoManager->GetNodeInfo(aNode.GetText(), nsnull,
+                                    kNameSpaceID_None,
+                                    getter_AddRefs(nodeInfo));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_IF_ADDREF(*aResult = nodeInfo->NameAtom());
+  } else {
+    rv = NameFromType(type, aResult);
+  }
+  return rv;
+}
+
+// nsHTMLFragmentContentSink
+
+nsresult
+nsHTMLParanoidFragmentSink::AddAttributes(const nsIParserNode& aNode,
+                                          nsIContent* aContent)
+{
+  PRInt32 ac = aNode.GetAttributeCount();
+
+  if (ac == 0) {
+    return NS_OK;
+  }
+
+  nsCAutoString k;
+  nsHTMLTag nodeType = nsHTMLTag(aNode.GetNodeType());
+
+  nsresult rv;
+  // use this to check for safe URIs in the few attributes that allow them
+  nsIScriptSecurityManager* secMan = nsContentUtils::GetSecurityManager();
+  nsCOMPtr<nsIURI> baseURI;
+
+  for (PRInt32 i = ac - 1; i >= 0; i--) {
+    rv = NS_OK;
+    const nsAString& key = aNode.GetKeyAt(i);
+    CopyUTF16toUTF8(key, k);
+    ToLowerCase(k);
+
+    nsCOMPtr<nsIAtom> keyAtom = do_GetAtom(k);
+
+    // not an allowed attribute
+    if (!sAllowedAttributes || !sAllowedAttributes->GetEntry(keyAtom)) {
+      continue;
+    }
+
+    // Get value and remove mandatory quotes
+    static const char* kWhitespace = "\n\r\t\b";
+    const nsAString& v =
+      nsContentUtils::TrimCharsInSet(kWhitespace, aNode.GetValueAt(i));
+
+    // check the attributes we allow that contain URIs
+    if (IsAttrURI(keyAtom)) {
+      if (!baseURI) {
+        baseURI = aContent->GetBaseURI();
+      }
+      nsCOMPtr<nsIURI> attrURI;
+      rv = NS_NewURI(getter_AddRefs(attrURI), v, nsnull, baseURI);
+      if (NS_SUCCEEDED(rv)) {
+        rv = secMan->CheckLoadURIWithPrincipal(mTargetDocument->NodePrincipal(),
+                                               attrURI,
+                                               nsIScriptSecurityManager::DISALLOW_SCRIPT_OR_DATA);
+      }
+    }
+    
+    // skip to the next attribute if we encountered issues with the
+    // current value
+    if (NS_FAILED(rv)) {
+      continue;
+    }
+
+    if (nodeType == eHTMLTag_a && keyAtom == nsHTMLAtoms::name) {
+      NS_ConvertUTF16toUTF8 cname(v);
+      NS_ConvertUTF8toUTF16 uv(nsUnescape(cname.BeginWriting()));
+      // Add attribute to content
+      aContent->SetAttr(kNameSpaceID_None, keyAtom, uv, PR_FALSE);
+    } else {
+      // Add attribute to content
+      aContent->SetAttr(kNameSpaceID_None, keyAtom, v, PR_FALSE);
+    }
+
+    if (nodeType == eHTMLTag_a || 
+        nodeType == eHTMLTag_form ||
+        nodeType == eHTMLTag_img ||
+        nodeType == eHTMLTag_map ||
+        nodeType == eHTMLTag_q ||
+        nodeType == eHTMLTag_blockquote ||
+        nodeType == eHTMLTag_input) {
+      AddBaseTagInfo(aContent);
+    }
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLParanoidFragmentSink::OpenContainer(const nsIParserNode& aNode)
+{
+  nsresult rv = NS_OK;
+  
+  // bail if it's a script or style, or we're already inside one of those
+  eHTMLTags type = (eHTMLTags)aNode.GetNodeType();
+  if (type == eHTMLTag_script || type == eHTMLTag_style) {
+    mSkip = PR_TRUE;
+    return rv;
+  }
+
+  nsCOMPtr<nsIAtom> name;
+  rv = NameFromNode(aNode, getter_AddRefs(name));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // not on whitelist
+  if (!sAllowedTags || !sAllowedTags->GetEntry(name)) {
+    return NS_OK;
+  }
+
+  return nsHTMLFragmentContentSink::OpenContainer(aNode);
+}
+
+NS_IMETHODIMP
+nsHTMLParanoidFragmentSink::CloseContainer(const nsHTMLTag aTag)
+{
+  nsresult rv = NS_OK;
+
+  if (mSkip) {
+    mSkip = PR_FALSE;
+    return rv;
+  }
+
+  nsCOMPtr<nsIAtom> name;
+  rv = NameFromType(aTag, getter_AddRefs(name));
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  // not on whitelist
+  if (!sAllowedTags || !sAllowedTags->GetEntry(name)) {
+    return NS_OK;
+  }
+
+  return nsHTMLFragmentContentSink::CloseContainer(aTag);
+}
+
+NS_IMETHODIMP
+nsHTMLParanoidFragmentSink::AddLeaf(const nsIParserNode& aNode)
+{
+  NS_ENSURE_TRUE(mNodeInfoManager, NS_ERROR_NOT_INITIALIZED);
+  
+  nsresult rv = NS_OK;
+
+#ifndef MOZILLA_1_8_BRANCH
+  if (mSkip) {
+    return rv;
+  }
+#endif
+  
+  if (aNode.GetTokenType() == eToken_start) {
+    nsCOMPtr<nsIAtom> name;
+    rv = NameFromNode(aNode, getter_AddRefs(name));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+#ifdef MOZILLA_1_8_BRANCH
+    // we have to do this on the branch for some late 90s reason
+    if (name == nsHTMLAtoms::script || name == nsHTMLAtoms::style) {
+      nsCOMPtr<nsIDTD> dtd;
+      mParser->GetDTD(getter_AddRefs(dtd));
+      NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+
+      nsAutoString skippedContent;
+      PRInt32 lineNo = 0;                
+      dtd->CollectSkippedContent(nodeType, skippedContent, lineNo);
+    }
+#endif
+
+    // We will process base tags, but we won't include them
+    // in the output
+    if (name == nsHTMLAtoms::base) {
+      nsCOMPtr<nsIContent> content;
+      nsCOMPtr<nsINodeInfo> nodeInfo;
+      nsIParserService* parserService = nsContentUtils::GetParserService();
+      if (!parserService)
+        return NS_ERROR_OUT_OF_MEMORY;
+      rv = mNodeInfoManager->GetNodeInfo(name, nsnull,
+                                         kNameSpaceID_None,
+                                         getter_AddRefs(nodeInfo));
+      NS_ENSURE_SUCCESS(rv, rv);
+      NS_NewHTMLElement(getter_AddRefs(content), nodeInfo);
+      NS_ENSURE_SUCCESS(rv, rv);
+      AddAttributes(aNode, content);
+      ProcessBaseTag(content);
+      return NS_OK;
+    }
+
+    if (!sAllowedTags || !sAllowedTags->GetEntry(name)) {
+      return NS_OK;
+    }
+  }
+
+  return nsHTMLFragmentContentSink::AddLeaf(aNode);
+}
+
+NS_IMETHODIMP
+nsHTMLParanoidFragmentSink::AddComment(const nsIParserNode& aNode)
+{
+  // no comments
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLParanoidFragmentSink::AddProcessingInstruction(const nsIParserNode& aNode)
+{
+  // no PIs
+  return NS_OK;
+}
