@@ -52,7 +52,7 @@
 #include "imgIRequest.h"
 #include "imgILoader.h"
 #include "imgIContainer.h"
-#include "imgIDecoderObserver.h"
+#include "nsStubImageDecoderObserver.h"
 #include "nsIPresShell.h"
 #include "nsPresContext.h"
 #include "nsIScrollableView.h"
@@ -66,6 +66,8 @@
 #include "nsPIDOMWindow.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMNSHTMLElement.h"
+#include "nsContentErrors.h"
+#include "ImageErrors.h"
 
 #define AUTOMATIC_IMAGE_RESIZING_PREF "browser.enable_automatic_image_resizing"
 
@@ -84,7 +86,7 @@ public:
 
 class nsImageDocument : public nsMediaDocument,
                         public nsIImageDocument,
-                        public imgIDecoderObserver,
+                        public nsStubImageDecoderObserver,
                         public nsIDOMEventListener
 {
 public:
@@ -104,12 +106,12 @@ public:
                                      nsIContentSink*     aSink = nsnull);
 
   virtual void SetScriptGlobalObject(nsIScriptGlobalObject* aScriptGlobalObject);
+  virtual void Destroy();
 
   NS_DECL_NSIIMAGEDOCUMENT
 
-  NS_DECL_IMGIDECODEROBSERVER
-
-  NS_DECL_IMGICONTAINEROBSERVER
+  // imgIDecoderObserver (override nsStubImageDecoderObserver)
+  NS_IMETHOD OnStartContainer(imgIRequest* aRequest, imgIContainer* aImage);
 
   // nsIDOMEventListener
   NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent);
@@ -118,7 +120,7 @@ public:
 protected:
   nsresult CreateSyntheticDocument();
 
-  nsresult CheckOverflowing();
+  nsresult CheckOverflowing(PRBool changeState);
 
   void UpdateTitleAndCharset();
 
@@ -127,16 +129,22 @@ protected:
                   (float)mVisibleHeight / mImageHeight);
   }
 
-  nsCOMPtr<nsIHTMLContent>      mImageContent;
+  nsCOMPtr<nsIContent>          mImageContent;
 
   PRInt32                       mVisibleWidth;
   PRInt32                       mVisibleHeight;
   PRInt32                       mImageWidth;
   PRInt32                       mImageHeight;
 
-  PRPackedBool                  mImageResizingEnabled;
+  PRPackedBool                  mResizeImageByDefault;
   PRPackedBool                  mImageIsOverflowing;
+  // mImageIsResized is true if the image is currently resized
   PRPackedBool                  mImageIsResized;
+  // mShouldResize is true if the image should be resized when it doesn't fit
+  // mImageIsResized cannot be true when this is false, but mImageIsResized
+  // can be false when this is true
+  PRPackedBool                  mShouldResize;
+  PRPackedBool                  mFirstResize;
 };
 
 NS_IMPL_ADDREF_INHERITED(ImageListener, nsMediaDocumentStreamListener)
@@ -184,7 +192,8 @@ ImageListener::OnStartRequest(nsIRequest* request, nsISupports *ctxt)
                                              domWindow->GetFrameElementInternal(),
                                              mimeType,
                                              nsnull,
-                                             &decision);
+                                             &decision,
+                                             nsContentUtils::GetContentPolicy());
                                                
   if (NS_FAILED(rv) || NS_CP_REJECTED(decision)) {
     request->Cancel(NS_ERROR_CONTENT_BLOCKED);
@@ -213,6 +222,26 @@ ImageListener::OnStopRequest(nsIRequest* request, nsISupports *ctxt,
     imageLoader->RemoveObserver(imgDoc);
   }
 
+  // |status| is NS_IMAGELIB_ERROR_LOAD_ABORTED if the image was found in
+  // the cache (bug 177747 comment 51).
+  if (status == NS_IMAGELIB_ERROR_LOAD_ABORTED) {
+    status = NS_OK;
+  }
+
+  // mImageContent can be null if the document is already destroyed
+  if (NS_FAILED(status) && imgDoc->mStringBundle && imgDoc->mImageContent) {
+    nsCAutoString src;
+    imgDoc->mDocumentURI->GetSpec(src);
+    NS_ConvertUTF8toUTF16 srcString(src);
+    const PRUnichar* formatString[] = { srcString.get() };
+    nsXPIDLString errorMsg;
+    NS_NAMED_LITERAL_STRING(str, "InvalidImage");
+    imgDoc->mStringBundle->FormatStringFromName(str.get(), formatString, 1,
+                                                getter_Copies(errorMsg));
+    
+    imgDoc->mImageContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::alt, errorMsg, PR_FALSE);
+  }
+
   return nsMediaDocumentStreamListener::OnStopRequest(request, ctxt, status);
 }
 
@@ -250,8 +279,10 @@ nsImageDocument::Init()
   nsresult rv = nsMediaDocument::Init();
   NS_ENSURE_SUCCESS(rv, rv);
 
-  mImageResizingEnabled =
+  mResizeImageByDefault =
     nsContentUtils::GetBoolPref(AUTOMATIC_IMAGE_RESIZING_PREF);
+  mShouldResize = mResizeImageByDefault;
+  mFirstResize = PR_TRUE;
 
   return NS_OK;
 }
@@ -283,47 +314,56 @@ nsImageDocument::StartDocumentLoad(const char*         aCommand,
 }
 
 void
-nsImageDocument::SetScriptGlobalObject(nsIScriptGlobalObject* aScriptGlobalObject)
+nsImageDocument::Destroy()
 {
-  if (!aScriptGlobalObject) {
-    if (mImageResizingEnabled) {
-      nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(mImageContent);
-      target->RemoveEventListener(NS_LITERAL_STRING("click"), this, PR_FALSE);
-
-      target = do_QueryInterface(mScriptGlobalObject);
-      target->RemoveEventListener(NS_LITERAL_STRING("resize"), this, PR_FALSE);
-      target->RemoveEventListener(NS_LITERAL_STRING("keypress"), this,
-                                  PR_FALSE);
-    }
+  if (mImageContent) {
+    // Remove our event listener from the image content.
+    nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(mImageContent);
+    target->RemoveEventListener(NS_LITERAL_STRING("click"), this, PR_FALSE);
 
     // Break reference cycle with mImageContent, if we have one
     nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mImageContent);
     if (imageLoader) {
       imageLoader->RemoveObserver(this);
     }
-    
+
     mImageContent = nsnull;
   }
 
+  nsMediaDocument::Destroy();
+}
+
+void
+nsImageDocument::SetScriptGlobalObject(nsIScriptGlobalObject* aScriptGlobalObject)
+{
+  // If the script global object is changing, we need to unhook our event
+  // listeners on the window.
+  nsCOMPtr<nsIDOMEventTarget> target;
+  if (mScriptGlobalObject &&
+      aScriptGlobalObject != mScriptGlobalObject) {
+    target = do_QueryInterface(mScriptGlobalObject);
+    target->RemoveEventListener(NS_LITERAL_STRING("resize"), this, PR_FALSE);
+    target->RemoveEventListener(NS_LITERAL_STRING("keypress"), this,
+                                PR_FALSE);
+  }
+
   // Set the script global object on the superclass before doing
   // anything that might require it....
   nsHTMLDocument::SetScriptGlobalObject(aScriptGlobalObject);
 
   if (aScriptGlobalObject) {
-    // Create synthetic document
-    nsresult rv = CreateSyntheticDocument();
-    if (NS_FAILED(rv)) {
-      return;
-    }
+    if (!mRootContent) {
+      // Create synthetic document
+      nsresult rv = CreateSyntheticDocument();
+      NS_ASSERTION(NS_SUCCEEDED(rv), "failed to create synthetic document");
 
-    if (mImageResizingEnabled) {
-      nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(mImageContent);
+      target = do_QueryInterface(mImageContent);
       target->AddEventListener(NS_LITERAL_STRING("click"), this, PR_FALSE);
-
-      target = do_QueryInterface(aScriptGlobalObject);
-      target->AddEventListener(NS_LITERAL_STRING("resize"), this, PR_FALSE);
-      target->AddEventListener(NS_LITERAL_STRING("keypress"), this, PR_FALSE);
     }
+
+    target = do_QueryInterface(aScriptGlobalObject);
+    target->AddEventListener(NS_LITERAL_STRING("resize"), this, PR_FALSE);
+    target->AddEventListener(NS_LITERAL_STRING("keypress"), this, PR_FALSE);
   }
 }
 
@@ -331,7 +371,7 @@ nsImageDocument::SetScriptGlobalObject(nsIScriptGlobalObject* aScriptGlobalObjec
 NS_IMETHODIMP
 nsImageDocument::GetImageResizingEnabled(PRBool* aImageResizingEnabled)
 {
-  *aImageResizingEnabled = mImageResizingEnabled;
+  *aImageResizingEnabled = PR_TRUE;
   return NS_OK;
 }
 
@@ -349,20 +389,32 @@ nsImageDocument::GetImageIsResized(PRBool* aImageIsResized)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsImageDocument::GetImageRequest(imgIRequest** aImageRequest)
+{
+  nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mImageContent);
+  if (imageLoader) {
+    return imageLoader->GetRequest(nsIImageLoadingContent::CURRENT_REQUEST,
+                                   aImageRequest);
+  }
+
+  *aImageRequest = nsnull;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsImageDocument::ShrinkToFit()
 {
-  if (mImageResizingEnabled) {
-    nsCOMPtr<nsIDOMHTMLImageElement> image = do_QueryInterface(mImageContent);
-    image->SetWidth(NSToCoordFloor(GetRatio() * mImageWidth));
+  nsCOMPtr<nsIDOMHTMLImageElement> image = do_QueryInterface(mImageContent);
+  image->SetWidth(NSToCoordFloor(GetRatio() * mImageWidth));
+  
+  mImageContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::style,
+                         NS_LITERAL_STRING("cursor: -moz-zoom-in"), PR_TRUE);
+  
+  mImageIsResized = PR_TRUE;
+  
+  UpdateTitleAndCharset();
 
-    mImageContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::style,
-                           NS_LITERAL_STRING("cursor: -moz-zoom-in"), PR_TRUE);
-
-    mImageIsResized = PR_TRUE;
-
-    UpdateTitleAndCharset();
-  }
   return NS_OK;
 }
 
@@ -396,8 +448,8 @@ nsImageDocument::RestoreImageTo(PRInt32 aX, PRInt32 aY)
     return NS_OK;
 
   nsRect portRect = view->View()->GetBounds();
-  view->ScrollTo((nscoord)(aX / (ratio * mImageWidth) * PR_MAX(0, scrolledSize.width - portRect.width) + 0.5),
-                 (nscoord)(aY / (ratio * mImageHeight) * PR_MAX(0, scrolledSize.height - portRect.height) + 0.5),
+  view->ScrollTo(NSToCoordRound((aX/ratio)*context->PixelsToTwips() - portRect.width/2),
+                 NSToCoordRound((aY/ratio)*context->PixelsToTwips() - portRect.height/2),
                  NS_VMREFRESH_IMMEDIATE);
   return NS_OK;
 }
@@ -405,21 +457,19 @@ nsImageDocument::RestoreImageTo(PRInt32 aX, PRInt32 aY)
 NS_IMETHODIMP
 nsImageDocument::RestoreImage()
 {
-  if (mImageResizingEnabled) {
-    mImageContent->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::width, PR_TRUE);
-
-    if (mImageIsOverflowing) {
-      mImageContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::style,
-                             NS_LITERAL_STRING("cursor: -moz-zoom-out"), PR_TRUE);
-    }
-    else {
-      mImageContent->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::style, PR_TRUE);
-    }
-
-    mImageIsResized = PR_FALSE;
-
-    UpdateTitleAndCharset();
+  mImageContent->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::width, PR_TRUE);
+  
+  if (mImageIsOverflowing) {
+    mImageContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::style,
+                           NS_LITERAL_STRING("cursor: -moz-zoom-out"), PR_TRUE);
   }
+  else {
+    mImageContent->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::style, PR_TRUE);
+  }
+  
+  mImageIsResized = PR_FALSE;
+  
+  UpdateTitleAndCharset();
 
   return NS_OK;
 }
@@ -427,91 +477,39 @@ nsImageDocument::RestoreImage()
 NS_IMETHODIMP
 nsImageDocument::ToggleImageSize()
 {
-  if (mImageResizingEnabled) {
-    if (mImageIsResized) {
-      RestoreImage();
-    }
-    else if (mImageIsOverflowing) {
-      ShrinkToFit();
-    }
+  mShouldResize = PR_TRUE;
+  if (mImageIsResized) {
+    mShouldResize = PR_FALSE;
+    RestoreImage();
+  }
+  else if (mImageIsOverflowing) {
+    ShrinkToFit();
   }
 
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsImageDocument::OnStartDecode(imgIRequest* aRequest)
-{
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsImageDocument::OnStartContainer(imgIRequest* aRequest, imgIContainer* aImage)
 {
   aImage->GetWidth(&mImageWidth);
   aImage->GetHeight(&mImageHeight);
-  if (mImageResizingEnabled) {
-    CheckOverflowing();
-  }
+  CheckOverflowing(mResizeImageByDefault);
   UpdateTitleAndCharset();
 
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsImageDocument::OnStartFrame(imgIRequest* aRequest, gfxIImageFrame* aFrame)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsImageDocument::OnDataAvailable(imgIRequest* aRequest,
-                                 gfxIImageFrame* aFrame,
-                                 const nsRect* aRect)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsImageDocument::OnStopFrame(imgIRequest* aRequest,
-                             gfxIImageFrame* aFrame)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsImageDocument::OnStopContainer(imgIRequest* aRequest,
-                                 imgIContainer* aImage)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsImageDocument::OnStopDecode(imgIRequest* aRequest,
-                              nsresult status,
-                              const PRUnichar* statusArg)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsImageDocument::FrameChanged(imgIContainer* aContainer,
-                              gfxIImageFrame* aFrame,
-                              nsRect* aDirtyRect)
-{
-  return NS_OK;
-}
-
-
 NS_IMETHODIMP
 nsImageDocument::HandleEvent(nsIDOMEvent* aEvent)
 {
   nsAutoString eventType;
   aEvent->GetType(eventType);
   if (eventType.EqualsLiteral("resize")) {
-    CheckOverflowing();
+    CheckOverflowing(PR_FALSE);
   }
   else if (eventType.EqualsLiteral("click")) {
+    mShouldResize = PR_TRUE;
     if (mImageIsResized) {
       PRInt32 x = 0, y = 0;
       nsCOMPtr<nsIDOMMouseEvent> event(do_QueryInterface(aEvent));
@@ -525,6 +523,7 @@ nsImageDocument::HandleEvent(nsIDOMEvent* aEvent)
         x -= left;
         y -= top;
       }
+      mShouldResize = PR_FALSE;
       RestoreImageTo(x, y);
     }
     else if (mImageIsOverflowing) {
@@ -537,12 +536,14 @@ nsImageDocument::HandleEvent(nsIDOMEvent* aEvent)
     keyEvent->GetCharCode(&charCode);
     // plus key
     if (charCode == 0x2B) {
+      mShouldResize = PR_FALSE;
       if (mImageIsResized) {
         RestoreImage();
       }
     }
     // minus key
     else if (charCode == 0x2D) {
+      mShouldResize = PR_TRUE;
       if (mImageIsOverflowing) {
         ShrinkToFit();
       }
@@ -559,7 +560,7 @@ nsImageDocument::CreateSyntheticDocument()
   nsresult rv = nsMediaDocument::CreateSyntheticDocument();
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIHTMLContent> body = do_QueryInterface(mBodyContent);
+  nsCOMPtr<nsIContent> body = do_QueryInterface(mBodyContent);
   if (!body) {
     NS_WARNING("no body on image document!");
     return NS_ERROR_FAILURE;
@@ -575,36 +576,26 @@ nsImageDocument::CreateSyntheticDocument()
   if (!mImageContent) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
-  mImageContent->SetDocument(this, PR_FALSE, PR_TRUE);
   nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mImageContent);
   NS_ENSURE_TRUE(imageLoader, NS_ERROR_UNEXPECTED);
 
   nsCAutoString src;
   mDocumentURI->GetSpec(src);
 
-  NS_ConvertUTF8toUCS2 srcString(src);
+  NS_ConvertUTF8toUTF16 srcString(src);
   // Make sure not to start the image load from here...
   imageLoader->SetLoadingEnabled(PR_FALSE);
   mImageContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::src, srcString, PR_FALSE);
+  mImageContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::alt, srcString, PR_FALSE);
 
-  if (mStringBundle) {
-    const PRUnichar* formatString[1] = { srcString.get() };
-    nsXPIDLString errorMsg;
-    NS_NAMED_LITERAL_STRING(str, "InvalidImage");
-    mStringBundle->FormatStringFromName(str.get(), formatString, 1,
-                                        getter_Copies(errorMsg));
-
-    mImageContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::alt, errorMsg, PR_FALSE);
-  }
-
-  body->AppendChildTo(mImageContent, PR_FALSE, PR_FALSE);
+  body->AppendChildTo(mImageContent, PR_FALSE);
   imageLoader->SetLoadingEnabled(PR_TRUE);
 
   return NS_OK;
 }
 
 nsresult
-nsImageDocument::CheckOverflowing()
+nsImageDocument::CheckOverflowing(PRBool changeState)
 {
   nsIPresShell *shell = GetShellAt(0);
   if (!shell) {
@@ -615,6 +606,11 @@ nsImageDocument::CheckOverflowing()
   nsRect visibleArea = context->GetVisibleArea();
 
   nsCOMPtr<nsIContent> content = do_QueryInterface(mBodyContent);
+  if (!content) {
+    NS_WARNING("no body on image document!");
+    return NS_ERROR_FAILURE;
+  }
+
   nsRefPtr<nsStyleContext> styleContext =
     context->StyleSet()->ResolveStyleFor(content, nsnull);
 
@@ -636,12 +632,15 @@ nsImageDocument::CheckOverflowing()
   mImageIsOverflowing =
     mImageWidth > mVisibleWidth || mImageHeight > mVisibleHeight;
 
-  if (mImageIsOverflowing) {
-    ShrinkToFit();
-  }
-  else if (mImageIsResized) {
-    RestoreImage();
+  if (changeState || mShouldResize || mFirstResize) {
+    if (mImageIsOverflowing && (changeState || mShouldResize)) {
+      ShrinkToFit();
+    }
+    else if (mImageIsResized || mFirstResize) {
+      RestoreImage();
+    }
   }
+  mFirstResize = PR_FALSE;
 
   return NS_OK;
 }
@@ -701,7 +700,7 @@ nsImageDocument::UpdateTitleAndCharset()
   {
     "ImageTitleWithNeitherDimensionsNorFile",
     "ImageTitleWithoutDimensions",
-    "ImageTitleWithDimesions",
+    "ImageTitleWithDimensions",
     "ImageTitleWithDimensionsAndFile",
   };
 
@@ -718,14 +717,14 @@ NS_NewImageDocument(nsIDocument** aResult)
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
+  NS_ADDREF(doc);
   nsresult rv = doc->Init();
 
   if (NS_FAILED(rv)) {
-    delete doc;
-    return rv;
+    NS_RELEASE(doc);
   }
 
-  NS_ADDREF(*aResult = doc);
+  *aResult = doc;
 
-  return NS_OK;
+  return rv;
 }
