@@ -61,6 +61,7 @@
 #include "nsIScriptError.h"
 #include "nsNodeInfoManager.h"
 #include "nsINodeInfo.h"
+#include "nsIPrincipal.h"
 
 nsresult
 NS_NewXBLContentSink(nsIXMLContentSink** aResult,
@@ -170,10 +171,11 @@ nsXBLContentSink::FlushText(PRBool aCreateTextNode,
   }
 
   nsIContent* content = GetCurrentContent();
-  if (content && (content->GetNodeInfo()->NamespaceEquals(kNameSpaceID_XBL) || (
-      content->GetNodeInfo()->NamespaceEquals(kNameSpaceID_XUL) &&
-      content->Tag() != nsXULAtoms::label &&
-      content->Tag() != nsXULAtoms::description))) {
+  if (content &&
+      (content->NodeInfo()->NamespaceEquals(kNameSpaceID_XBL) ||
+       (content->NodeInfo()->NamespaceEquals(kNameSpaceID_XUL) &&
+        content->Tag() != nsXULAtoms::label &&
+        content->Tag() != nsXULAtoms::description))) {
 
     PRBool isWS = PR_TRUE;
     if (mTextLength > 0) {
@@ -201,36 +203,34 @@ nsXBLContentSink::FlushText(PRBool aCreateTextNode,
 
 NS_IMETHODIMP
 nsXBLContentSink::ReportError(const PRUnichar* aErrorText, 
-                              const PRUnichar* aSourceText)
+                              const PRUnichar* aSourceText,
+                              nsIScriptError *aError,
+                              PRBool *_retval)
 {
+  NS_PRECONDITION(aError && aSourceText && aErrorText, "Check arguments!!!");
+
+  // XXX FIXME This function overrides and calls on
+  // nsXMLContentSink::ReportError, and probably should die.  See bug 347826.
+
   // XXX We should make sure the binding has no effect, but that it also
   // gets destroyed properly without leaking.  Perhaps we should even
   // ensure that the content that was bound is displayed with no
   // binding.
 
-  // Report the error to the error console.
-  nsCOMPtr<nsIConsoleService> consoleService =
-    do_GetService(NS_CONSOLESERVICE_CONTRACTID);
-  if (consoleService) {
-    // XXX It would be nice if the parser didn't pre-format it for us,
-    // because then we could create a instance of nsIScriptError and the
-    // error console would format this much more nicely for us.
-    // However, that would require duplicating even more code between
-    // the XML content sink and the XUL content sink.
-
-    consoleService->LogStringMessage(aErrorText);
-  }
-
 #ifdef DEBUG
   // Report the error to stderr.
   fprintf(stderr,
           "\n%s\n%s\n\n",
-          NS_LossyConvertUCS2toASCII(aErrorText).get(),
-          NS_LossyConvertUCS2toASCII(aSourceText).get());
+          NS_LossyConvertUTF16toASCII(aErrorText).get(),
+          NS_LossyConvertUTF16toASCII(aSourceText).get());
 #endif
 
   // Most of what this does won't be too useful, but whatever...
-  return nsXMLContentSink::ReportError(aErrorText, aSourceText);
+  // nsXMLContentSink::ReportError will handle the console logging.
+  return nsXMLContentSink::ReportError(aErrorText, 
+                                       aSourceText, 
+                                       aError,
+                                       _retval);
 }
 
 nsresult
@@ -250,6 +250,7 @@ nsXBLContentSink::ReportUnexpectedElement(nsIAtom* aElementName,
                                          "UnexpectedElement",
                                          params, NS_ARRAY_LENGTH(params),
                                          mDocumentURI,
+                                         EmptyString() /* source line */,
                                          aLineNumber, 0 /* column number */,
                                          nsIScriptError::errorFlag,
                                          "XBL Content Sink");
@@ -296,54 +297,64 @@ nsXBLContentSink::HandleEndElement(const PRUnichar *aName)
   FlushText();
 
   if (mState != eXBL_InDocument) {
-    nsCOMPtr<nsIAtom> nameSpacePrefix, tagAtom;
+    PRInt32 nameSpaceID;
+    nsCOMPtr<nsIAtom> prefix, localName;
+    nsContentUtils::SplitExpatName(aName, getter_AddRefs(prefix),
+                                   getter_AddRefs(localName), &nameSpaceID);
 
-    SplitXMLName(nsDependentString(aName), getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(tagAtom));
-
-    PRInt32 nameSpaceID = GetNameSpaceId(nameSpacePrefix);
     if (nameSpaceID == kNameSpaceID_XBL) {
-      if (mState == eXBL_InHandlers) {
-        if (tagAtom == nsXBLAtoms::handlers) {
+      if (mState == eXBL_Error) {
+        // Check whether we've opened this tag before; we may not have if
+        // it was a real XBL tag before the error occured.
+        if (!GetCurrentContent()->NodeInfo()->Equals(localName,
+                                                     nameSpaceID)) {
+          // OK, this tag was never opened as far as the XML sink is
+          // concerned.  Just drop the HandleEndElement
+          return NS_OK;
+        }
+      }
+      else if (mState == eXBL_InHandlers) {
+        if (localName == nsXBLAtoms::handlers) {
           mState = eXBL_InBinding;
           mHandler = nsnull;
         }
-        else if (tagAtom == nsXBLAtoms::handler)
+        else if (localName == nsXBLAtoms::handler)
           mSecondaryState = eXBL_None;
         return NS_OK;
       }
       else if (mState == eXBL_InResources) {
-        if (tagAtom == nsXBLAtoms::resources)
+        if (localName == nsXBLAtoms::resources)
           mState = eXBL_InBinding;
         return NS_OK;
       }
       else if (mState == eXBL_InImplementation) {
-        if (tagAtom == nsXBLAtoms::implementation)
+        if (localName == nsXBLAtoms::implementation)
           mState = eXBL_InBinding;
-        else if (tagAtom == nsXBLAtoms::property) {
+        else if (localName == nsXBLAtoms::property) {
           mSecondaryState = eXBL_None;
           mProperty = nsnull;
         }
-        else if (tagAtom == nsXBLAtoms::method) {
+        else if (localName == nsXBLAtoms::method) {
           mSecondaryState = eXBL_None;
           mMethod = nsnull;
         }
-        else if (tagAtom == nsXBLAtoms::field) {
+        else if (localName == nsXBLAtoms::field) {
           mSecondaryState = eXBL_None;
           mField = nsnull;
         }
-        else if (tagAtom == nsXBLAtoms::constructor ||
-                 tagAtom == nsXBLAtoms::destructor)
+        else if (localName == nsXBLAtoms::constructor ||
+                 localName == nsXBLAtoms::destructor)
           mSecondaryState = eXBL_None;
-        else if (tagAtom == nsXBLAtoms::getter ||
-                 tagAtom == nsXBLAtoms::setter)
+        else if (localName == nsXBLAtoms::getter ||
+                 localName == nsXBLAtoms::setter)
           mSecondaryState = eXBL_InProperty;
-        else if (tagAtom == nsXBLAtoms::parameter ||
-                 tagAtom == nsXBLAtoms::body)
+        else if (localName == nsXBLAtoms::parameter ||
+                 localName == nsXBLAtoms::body)
           mSecondaryState = eXBL_InMethod;
         return NS_OK;
       }
-      else if (mState == eXBL_InBindings && tagAtom == nsXBLAtoms::bindings) {
+      else if (mState == eXBL_InBindings &&
+               localName == nsXBLAtoms::bindings) {
         mState = eXBL_InDocument;
       }
       
@@ -351,7 +362,7 @@ nsXBLContentSink::HandleEndElement(const PRUnichar *aName)
       if (NS_FAILED(rv))
         return rv;
 
-      if (mState == eXBL_InBinding && tagAtom == nsXBLAtoms::binding) {
+      if (mState == eXBL_InBinding && localName == nsXBLAtoms::binding) {
         mState = eXBL_InBindings;
         if (mBinding) {  // See comment in HandleStartElement()
           mBinding->Initialize();
@@ -375,6 +386,11 @@ nsXBLContentSink::HandleCDataSection(const PRUnichar *aData,
   return nsXMLContentSink::HandleCDataSection(aData, aLength);
 }
 
+#define ENSURE_XBL_STATE(_cond)                                                       \
+  PR_BEGIN_MACRO                                                                      \
+    if (!(_cond)) { ReportUnexpectedElement(aTagName, aLineNumber); return PR_TRUE; } \
+  PR_END_MACRO
+
 PRBool 
 nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts, 
                                   PRUint32 aAttsCount, 
@@ -386,145 +402,147 @@ nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts,
     return PR_TRUE;
   }
   
-  PRBool ret = PR_TRUE;
-  if (aNameSpaceID == kNameSpaceID_XBL) {
-    if (aTagName == nsXBLAtoms::bindings) {
-      if (mState != eXBL_InDocument) {
-        ReportUnexpectedElement(aTagName, aLineNumber);
-        return PR_TRUE;
-      }
-      
-      NS_NewXBLDocumentInfo(mDocument, &mDocInfo);
-      if (!mDocInfo) {
-        mState = eXBL_Error;
-        return PR_TRUE;
-      }
-
-      mDocument->GetBindingManager()->PutXBLDocumentInfo(mDocInfo);
-
-      nsIURI *uri = mDocument->GetDocumentURI();
-      
-      PRBool isChrome = PR_FALSE;
-      PRBool isRes = PR_FALSE;
-
-      uri->SchemeIs("chrome", &isChrome);
-      uri->SchemeIs("resource", &isRes);
-      mIsChromeOrResource = isChrome || isRes;
-      
-      nsIXBLDocumentInfo* info = mDocInfo;
-      NS_RELEASE(info); // We keep a weak ref. We've created a cycle between doc/binding manager/doc info.
-      mState = eXBL_InBindings; 
-    }
-    else if (aTagName == nsXBLAtoms::binding) {
-      if (mState != eXBL_InBindings) {
-        ReportUnexpectedElement(aTagName, aLineNumber);
-        return PR_TRUE;
-      }
-      mState = eXBL_InBinding;
-    }
-    else if (aTagName == nsXBLAtoms::handlers) {
-      if (mState != eXBL_InBinding || !mBinding) {
-        ReportUnexpectedElement(aTagName, aLineNumber);
-        return PR_TRUE;
-      }
-      mState = eXBL_InHandlers;
-      ret = PR_FALSE; // The XML content sink should not do anything with <handlers>.
-    }
-    else if (aTagName == nsXBLAtoms::handler) {
-      if (mState != eXBL_InHandlers) {
-        ReportUnexpectedElement(aTagName, aLineNumber);
-        return PR_TRUE;
-      }
-      mSecondaryState = eXBL_InHandler;
-      ConstructHandler(aAtts, aLineNumber);
-      ret = PR_FALSE;
-    }
-    else if (aTagName == nsXBLAtoms::resources) {
-      if (mState != eXBL_InBinding || !mBinding) {
-        ReportUnexpectedElement(aTagName, aLineNumber);
-        return PR_TRUE;
-      }
-      mState = eXBL_InResources;
-      ret = PR_FALSE; // The XML content sink should ignore all <resources>.
-    }
-    else if (mState == eXBL_InResources) {
-      NS_ASSERTION(mBinding, "Must have binding here");
-      if (aTagName == nsXBLAtoms::stylesheet || aTagName == nsXBLAtoms::image)
-        ConstructResource(aAtts, aTagName);
-      ret = PR_FALSE; // The XML content sink should ignore everything within a <resources> block.
-    }
-    else if (aTagName == nsXBLAtoms::implementation) {
-      if (mState != eXBL_InBinding || !mBinding) {
-        ReportUnexpectedElement(aTagName, aLineNumber);
-        return PR_TRUE;
-      }
-      mState = eXBL_InImplementation;
-      ConstructImplementation(aAtts);
-      ret = PR_FALSE; // The XML content sink should ignore the <implementation>.
-    }
-    else if (mState == eXBL_InImplementation) {
-      NS_ASSERTION(mBinding, "Must have binding here");
-      if (aTagName == nsXBLAtoms::constructor) {
-        mSecondaryState = eXBL_InConstructor;
-        nsXBLProtoImplAnonymousMethod* newMethod =
-          new nsXBLProtoImplAnonymousMethod();
-        if (newMethod) {
-          newMethod->SetLineNumber(aLineNumber);
-          mBinding->SetConstructor(newMethod);
-          AddMember(newMethod);
-        }
-      }
-      else if (aTagName == nsXBLAtoms::destructor) {
-        mSecondaryState = eXBL_InDestructor;
-        nsXBLProtoImplAnonymousMethod* newMethod =
-          new nsXBLProtoImplAnonymousMethod();
-        if (newMethod) {
-          newMethod->SetLineNumber(aLineNumber);
-          mBinding->SetDestructor(newMethod);
-          AddMember(newMethod);
-        }
-      }
-      else if (aTagName == nsXBLAtoms::field) {
-        mSecondaryState = eXBL_InField;
-        ConstructField(aAtts, aLineNumber);
-      }
-      else if (aTagName == nsXBLAtoms::property) {
-        mSecondaryState = eXBL_InProperty;
-        ConstructProperty(aAtts);
-      }
-      else if (aTagName == nsXBLAtoms::getter) {
-        if (mSecondaryState == eXBL_InProperty && mProperty) {
-          mProperty->SetGetterLineNumber(aLineNumber);
-        }        
-        mSecondaryState = eXBL_InGetter;
-      }
-      else if (aTagName == nsXBLAtoms::setter) {
-        if (mSecondaryState == eXBL_InProperty && mProperty) {
-          mProperty->SetSetterLineNumber(aLineNumber);
-        }
-        mSecondaryState = eXBL_InSetter;
-      }
-      else if (aTagName == nsXBLAtoms::method) {
-        mSecondaryState = eXBL_InMethod;
-        ConstructMethod(aAtts);
-      }
-      else if (aTagName == nsXBLAtoms::parameter)
-        ConstructParameter(aAtts);
-      else if (aTagName == nsXBLAtoms::body) {
-        if (mSecondaryState == eXBL_InMethod && mMethod) {
-          // stash away the line number
-          mMethod->SetLineNumber(aLineNumber);
-        }
-        mSecondaryState = eXBL_InBody;
-      }
-
-      ret = PR_FALSE; // Ignore everything we encounter inside an <implementation> block.
-    }
+  if (aNameSpaceID != kNameSpaceID_XBL) {
+    // Construct non-XBL nodes
+    return PR_TRUE;
   }
 
-  return ret;
+  PRBool ret = PR_TRUE;
+  if (aTagName == nsXBLAtoms::bindings) {
+    ENSURE_XBL_STATE(mState == eXBL_InDocument);
+      
+    NS_NewXBLDocumentInfo(mDocument, &mDocInfo);
+    if (!mDocInfo) {
+      mState = eXBL_Error;
+      return PR_TRUE;
+    }
+
+    mDocument->BindingManager()->PutXBLDocumentInfo(mDocInfo);
+
+    nsIURI *uri = mDocument->GetDocumentURI();
+      
+    PRBool isChrome = PR_FALSE;
+    PRBool isRes = PR_FALSE;
+
+    uri->SchemeIs("chrome", &isChrome);
+    uri->SchemeIs("resource", &isRes);
+    mIsChromeOrResource = isChrome || isRes;
+      
+    nsIXBLDocumentInfo* info = mDocInfo;
+    NS_RELEASE(info); // We keep a weak ref. We've created a cycle between doc/binding manager/doc info.
+    mState = eXBL_InBindings;
+  }
+  else if (aTagName == nsXBLAtoms::binding) {
+    ENSURE_XBL_STATE(mState == eXBL_InBindings);
+    mState = eXBL_InBinding;
+  }
+  else if (aTagName == nsXBLAtoms::handlers) {
+    ENSURE_XBL_STATE(mState == eXBL_InBinding && mBinding);
+    mState = eXBL_InHandlers;
+    ret = PR_FALSE;
+  }
+  else if (aTagName == nsXBLAtoms::handler) {
+    ENSURE_XBL_STATE(mState == eXBL_InHandlers);
+    mSecondaryState = eXBL_InHandler;
+    ConstructHandler(aAtts, aLineNumber);
+    ret = PR_FALSE;
+  }
+  else if (aTagName == nsXBLAtoms::resources) {
+    ENSURE_XBL_STATE(mState == eXBL_InBinding && mBinding);
+    mState = eXBL_InResources;
+    // Note that this mState will cause us to return false, so no need
+    // to set ret to false.
+  }
+  else if (aTagName == nsXBLAtoms::stylesheet || aTagName == nsXBLAtoms::image) {
+    ENSURE_XBL_STATE(mState == eXBL_InResources);
+    NS_ASSERTION(mBinding, "Must have binding here");
+    ConstructResource(aAtts, aTagName);
+  }
+  else if (aTagName == nsXBLAtoms::implementation) {
+    ENSURE_XBL_STATE(mState == eXBL_InBinding && mBinding);
+    mState = eXBL_InImplementation;
+    ConstructImplementation(aAtts);
+    // Note that this mState will cause us to return false, so no need
+    // to set ret to false.
+  }
+  else if (aTagName == nsXBLAtoms::constructor) {
+    ENSURE_XBL_STATE(mState == eXBL_InImplementation &&
+                     mSecondaryState == eXBL_None);
+    NS_ASSERTION(mBinding, "Must have binding here");
+      
+    mSecondaryState = eXBL_InConstructor;
+    nsXBLProtoImplAnonymousMethod* newMethod =
+      new nsXBLProtoImplAnonymousMethod();
+    if (newMethod) {
+      newMethod->SetLineNumber(aLineNumber);
+      mBinding->SetConstructor(newMethod);
+      AddMember(newMethod);
+    }
+  }
+  else if (aTagName == nsXBLAtoms::destructor) {
+    ENSURE_XBL_STATE(mState == eXBL_InImplementation &&
+                     mSecondaryState == eXBL_None);
+    NS_ASSERTION(mBinding, "Must have binding here");
+    mSecondaryState = eXBL_InDestructor;
+    nsXBLProtoImplAnonymousMethod* newMethod =
+      new nsXBLProtoImplAnonymousMethod();
+    if (newMethod) {
+      newMethod->SetLineNumber(aLineNumber);
+      mBinding->SetDestructor(newMethod);
+      AddMember(newMethod);
+    }
+  }
+  else if (aTagName == nsXBLAtoms::field) {
+    ENSURE_XBL_STATE(mState == eXBL_InImplementation &&
+                     mSecondaryState == eXBL_None);
+    NS_ASSERTION(mBinding, "Must have binding here");
+    mSecondaryState = eXBL_InField;
+    ConstructField(aAtts, aLineNumber);
+  }
+  else if (aTagName == nsXBLAtoms::property) {
+    ENSURE_XBL_STATE(mState == eXBL_InImplementation &&
+                     mSecondaryState == eXBL_None);
+    NS_ASSERTION(mBinding, "Must have binding here");
+    mSecondaryState = eXBL_InProperty;
+    ConstructProperty(aAtts);
+  }
+  else if (aTagName == nsXBLAtoms::getter) {
+    ENSURE_XBL_STATE(mSecondaryState == eXBL_InProperty && mProperty);
+    NS_ASSERTION(mState == eXBL_InImplementation, "Unexpected state");
+    mProperty->SetGetterLineNumber(aLineNumber);
+    mSecondaryState = eXBL_InGetter;
+  }
+  else if (aTagName == nsXBLAtoms::setter) {
+    ENSURE_XBL_STATE(mSecondaryState == eXBL_InProperty && mProperty);
+    NS_ASSERTION(mState == eXBL_InImplementation, "Unexpected state");
+    mProperty->SetSetterLineNumber(aLineNumber);
+    mSecondaryState = eXBL_InSetter;
+  }
+  else if (aTagName == nsXBLAtoms::method) {
+    ENSURE_XBL_STATE(mState == eXBL_InImplementation &&
+                     mSecondaryState == eXBL_None);
+    NS_ASSERTION(mBinding, "Must have binding here");
+    mSecondaryState = eXBL_InMethod;
+    ConstructMethod(aAtts);
+  }
+  else if (aTagName == nsXBLAtoms::parameter) {
+    ENSURE_XBL_STATE(mSecondaryState == eXBL_InMethod && mMethod);
+    NS_ASSERTION(mState == eXBL_InImplementation, "Unexpected state");
+    ConstructParameter(aAtts);
+  }
+  else if (aTagName == nsXBLAtoms::body) {
+    ENSURE_XBL_STATE(mSecondaryState == eXBL_InMethod && mMethod);
+    NS_ASSERTION(mState == eXBL_InImplementation, "Unexpected state");
+    // stash away the line number
+    mMethod->SetLineNumber(aLineNumber);
+    mSecondaryState = eXBL_InBody;
+  }
+
+  return ret && mState != eXBL_InResources && mState != eXBL_InImplementation;
 }
 
+#undef ENSURE_XBL_STATE
+
 nsresult
 nsXBLContentSink::ConstructBinding()
 {
@@ -553,12 +571,29 @@ nsXBLContentSink::ConstructBinding()
   return rv;
 }
 
+static PRBool
+FindValue(const PRUnichar **aAtts, nsIAtom *aAtom, const PRUnichar **aResult)
+{
+  nsCOMPtr<nsIAtom> prefix, localName;
+  for (; *aAtts; aAtts += 2) {
+    PRInt32 nameSpaceID;
+    nsContentUtils::SplitExpatName(aAtts[0], getter_AddRefs(prefix),
+                                   getter_AddRefs(localName), &nameSpaceID);
+
+    // Is this attribute one of the ones we care about?
+    if (nameSpaceID == kNameSpaceID_None && localName == aAtom) {
+      *aResult = aAtts[1];
+
+      return PR_TRUE;
+    }
+  }
+
+  return PR_FALSE;
+}
 
 void
 nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts, PRUint32 aLineNumber)
 {
-  nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
-
   const PRUnichar* event          = nsnull;
   const PRUnichar* modifiers      = nsnull;
   const PRUnichar* button         = nsnull;
@@ -568,42 +603,45 @@ nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts, PRUint32 aLineNumber
   const PRUnichar* phase          = nsnull;
   const PRUnichar* command        = nsnull;
   const PRUnichar* action         = nsnull;
+  const PRUnichar* group          = nsnull;
   const PRUnichar* preventdefault = nsnull;
+  const PRUnichar* allowuntrusted = nsnull;
 
+  nsCOMPtr<nsIAtom> prefix, localName;
   for (; *aAtts; aAtts += 2) {
-    // Get upper-cased key
+    PRInt32 nameSpaceID;
+    nsContentUtils::SplitExpatName(aAtts[0], getter_AddRefs(prefix),
+                                   getter_AddRefs(localName), &nameSpaceID);
 
-    SplitXMLName(nsDependentString(aAtts[0]), getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(nameAtom));
-
-    if (nameSpacePrefix || nameAtom == nsLayoutAtoms::xmlnsNameSpace) {
+    if (nameSpaceID != kNameSpaceID_None) {
       continue;
     }
 
     // Is this attribute one of the ones we care about?
-    if (nameAtom == nsXBLAtoms::event)
+    if (localName == nsXBLAtoms::event)
       event = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::modifiers)
+    else if (localName == nsXBLAtoms::modifiers)
       modifiers = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::button)
+    else if (localName == nsXBLAtoms::button)
       button = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::clickcount)
+    else if (localName == nsXBLAtoms::clickcount)
       clickcount = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::keycode)
+    else if (localName == nsXBLAtoms::keycode)
       keycode = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::key || nameAtom == nsXBLAtoms::charcode)
+    else if (localName == nsXBLAtoms::key || localName == nsXBLAtoms::charcode)
       charcode = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::phase)
+    else if (localName == nsXBLAtoms::phase)
       phase = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::command)
+    else if (localName == nsXBLAtoms::command)
       command = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::action)
+    else if (localName == nsXBLAtoms::action)
       action = aAtts[1];
-    else if (nameAtom == nsXBLAtoms::preventdefault)
+    else if (localName == nsXBLAtoms::group)
+      group = aAtts[1];
+    else if (localName == nsXBLAtoms::preventdefault)
       preventdefault = aAtts[1];
-    else {
-       // Nope, it's some irrelevant attribute. Ignore it and move on.
-     }
+    else if (localName == nsXBLAtoms::allowuntrusted)
+      allowuntrusted = aAtts[1];
   }
 
   if (command && !mIsChromeOrResource)
@@ -611,23 +649,29 @@ nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts, PRUint32 aLineNumber
     // shorthand syntax.
     return; // Don't even make this handler.
 
-  // All of our pointers are now filled in.  Construct our handler with all of these
-  // parameters.
+  // All of our pointers are now filled in. Construct our handler with all of
+  // these parameters.
   nsXBLPrototypeHandler* newHandler;
   newHandler = new nsXBLPrototypeHandler(event, phase, action, command,
                                          keycode, charcode, modifiers, button,
-                                         clickcount, preventdefault, mBinding);
+                                         clickcount, group, preventdefault,
+                                         allowuntrusted, mBinding);
 
   if (newHandler) {
     newHandler->SetLineNumber(aLineNumber);
     
     // Add this handler to our chain of handlers.
-    if (mHandler)
-      mHandler->SetNextHandler(newHandler); // Already have a chain. Just append to the end.
-    else
-      mBinding->SetPrototypeHandlers(newHandler); // We're the first handler in the chain.
-
-    mHandler = newHandler; // Adjust our mHandler pointer to point to the new last handler in the chain.
+    if (mHandler) {
+      // Already have a chain. Just append to the end.
+      mHandler->SetNextHandler(newHandler);
+    }
+    else {
+      // We're the first handler in the chain.
+      mBinding->SetPrototypeHandlers(newHandler);
+    }
+    // Adjust our mHandler pointer to point to the new last handler in the
+    // chain.
+    mHandler = newHandler;
   }
 }
 
@@ -638,23 +682,9 @@ nsXBLContentSink::ConstructResource(const PRUnichar **aAtts,
   if (!mBinding)
     return;
 
-  nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
-  for (; *aAtts; aAtts += 2) {
-    // Get upper-cased key
-    const nsDependentString key(aAtts[0]);
-
-    SplitXMLName(key, getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(nameAtom));
-
-    if (nameSpacePrefix || nameAtom == nsLayoutAtoms::xmlnsNameSpace) {
-      continue;
-    }
-
-    // Is this attribute one of the ones we care about?
-    if (key.EqualsLiteral("src")) {
-      mBinding->AddResource(aResourceType, nsDependentString(aAtts[1]));
-      break;
-    }
+  const PRUnichar* src = nsnull;
+  if (FindValue(aAtts, nsHTMLAtoms::src, &src)) {
+    mBinding->AddResource(aResourceType, nsDependentString(src));
   }
 }
 
@@ -668,24 +698,38 @@ nsXBLContentSink::ConstructImplementation(const PRUnichar **aAtts)
     return;
 
   const PRUnichar* name = nsnull;
-  
-  nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
-  for (; *aAtts; aAtts +=2) {
-    // Get upper-cased key
 
-    SplitXMLName(nsDependentString(aAtts[0]), getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(nameAtom));
+  nsCOMPtr<nsIAtom> prefix, localName;
+  for (; *aAtts; aAtts += 2) {
+    PRInt32 nameSpaceID;
+    nsContentUtils::SplitExpatName(aAtts[0], getter_AddRefs(prefix),
+                                   getter_AddRefs(localName), &nameSpaceID);
 
-    if (nameSpacePrefix || nameAtom == nsLayoutAtoms::xmlnsNameSpace) {
+    if (nameSpaceID != kNameSpaceID_None) {
       continue;
     }
 
     // Is this attribute one of the ones we care about?
-    if (nameAtom == nsXBLAtoms::name) {
+    if (localName == nsXBLAtoms::name) {
       name = aAtts[1];
     }
-    else if (nameAtom == nsXBLAtoms::implements) {
-      mBinding->ConstructInterfaceTable(nsDependentString(aAtts[1]));
+    else if (localName == nsXBLAtoms::implements) {
+      // Only allow implementation of interfaces via XBL if the principal of
+      // our XBL document has UniversalXPConnect privileges.  No principal
+      // means no privs!
+      
+      // XXX this api is so badly tied to JS it's not even funny.  We don't
+      // have a concept of enabling capabilities on a per-principal basis,
+      // but only on a per-principal-and-JS-stackframe basis!  So for now
+      // this is basically equivalent to testing that we have the system
+      // principal, since there is no JS stackframe in sight here...
+      PRBool hasUniversalXPConnect;
+      nsresult rv = mDocument->NodePrincipal()->
+        IsCapabilityEnabled("UniversalXPConnect", nsnull,
+                            &hasUniversalXPConnect);
+      if (NS_SUCCEEDED(rv) && hasUniversalXPConnect) {
+        mBinding->ConstructInterfaceTable(nsDependentString(aAtts[1]));
+      }
     }
   }
 
@@ -695,76 +739,79 @@ nsXBLContentSink::ConstructImplementation(const PRUnichar **aAtts)
 void
 nsXBLContentSink::ConstructField(const PRUnichar **aAtts, PRUint32 aLineNumber)
 {
-  nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
-
   const PRUnichar* name     = nsnull;
   const PRUnichar* readonly = nsnull;
-  
-  for (; *aAtts; aAtts += 2) {
-    // Get upper-cased key
-    SplitXMLName(nsDependentString(aAtts[0]), getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(nameAtom));
 
-    if (nameSpacePrefix || nameAtom == nsLayoutAtoms::xmlnsNameSpace)
+  nsCOMPtr<nsIAtom> prefix, localName;
+  for (; *aAtts; aAtts += 2) {
+    PRInt32 nameSpaceID;
+    nsContentUtils::SplitExpatName(aAtts[0], getter_AddRefs(prefix),
+                                   getter_AddRefs(localName), &nameSpaceID);
+
+    if (nameSpaceID != kNameSpaceID_None) {
       continue;
+    }
 
     // Is this attribute one of the ones we care about?
-    if (nameAtom == nsXBLAtoms::name) {
+    if (localName == nsXBLAtoms::name) {
       name = aAtts[1];
     }
-    else if (nameAtom == nsXBLAtoms::readonly) {
+    else if (localName == nsXBLAtoms::readonly) {
       readonly = aAtts[1];
     }
   }
 
-  // All of our pointers are now filled in.  Construct our field with all of these
-  // parameters.
-  mField = new nsXBLProtoImplField(name, readonly);
-  if (mField) {
-    mField->SetLineNumber(aLineNumber);
-    AddMember(mField);
+  if (name) {
+    // All of our pointers are now filled in. Construct our field with all of
+    // these parameters.
+    mField = new nsXBLProtoImplField(name, readonly);
+    if (mField) {
+      mField->SetLineNumber(aLineNumber);
+      AddMember(mField);
+    }
   }
 }
 
 void
 nsXBLContentSink::ConstructProperty(const PRUnichar **aAtts)
 {
-  nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
-
   const PRUnichar* name     = nsnull;
   const PRUnichar* readonly = nsnull;
   const PRUnichar* onget    = nsnull;
   const PRUnichar* onset    = nsnull;
-  
-  for (; *aAtts; aAtts += 2) {
-    // Get upper-cased key
-    SplitXMLName(nsDependentString(aAtts[0]), getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(nameAtom));
 
-    if (nameSpacePrefix || nameAtom == nsLayoutAtoms::xmlnsNameSpace) {
+  nsCOMPtr<nsIAtom> prefix, localName;
+  for (; *aAtts; aAtts += 2) {
+    PRInt32 nameSpaceID;
+    nsContentUtils::SplitExpatName(aAtts[0], getter_AddRefs(prefix),
+                                   getter_AddRefs(localName), &nameSpaceID);
+
+    if (nameSpaceID != kNameSpaceID_None) {
       continue;
     }
 
     // Is this attribute one of the ones we care about?
-    if (nameAtom == nsXBLAtoms::name) {
+    if (localName == nsXBLAtoms::name) {
       name = aAtts[1];
     }
-    else if (nameAtom == nsXBLAtoms::readonly) {
+    else if (localName == nsXBLAtoms::readonly) {
       readonly = aAtts[1];
     }
-    else if (nameAtom == nsXBLAtoms::onget) {
+    else if (localName == nsXBLAtoms::onget) {
       onget = aAtts[1];
     }
-    else if (nameAtom == nsXBLAtoms::onset) {
+    else if (localName == nsXBLAtoms::onset) {
       onset = aAtts[1];
     }
   }
 
-  // All of our pointers are now filled in.  Construct our property with all of these
-  // parameters.
-  mProperty = new nsXBLProtoImplProperty(name, onget, onset, readonly);
-  if (mProperty) {
-    AddMember(mProperty);
+  if (name) {
+    // All of our pointers are now filled in. Construct our property with all of
+    // these parameters.
+    mProperty = new nsXBLProtoImplProperty(name, onget, onset, readonly);
+    if (mProperty) {
+      AddMember(mProperty);
+    }
   }
 }
 
@@ -773,22 +820,9 @@ nsXBLContentSink::ConstructMethod(const PRUnichar **aAtts)
 {
   mMethod = nsnull;
 
-  nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
-
-  for(; *aAtts; aAtts += 2) {
-    // Get upper-cased key
-    SplitXMLName(nsDependentString(aAtts[0]), getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(nameAtom));
-
-    if (nameSpacePrefix || nameAtom == nsLayoutAtoms::xmlnsNameSpace) {
-      continue;
-    }
-
-    // Is this attribute one of the ones we care about?
-    if (nameAtom == nsXBLAtoms::name) {
-      mMethod = new nsXBLProtoImplMethod(aAtts[1]);
-      break;
-    }
+  const PRUnichar* name = nsnull;
+  if (FindValue(aAtts, nsXBLAtoms::name, &name)) {
+    mMethod = new nsXBLProtoImplMethod(name);
   }
 
   if (mMethod) {
@@ -802,22 +836,9 @@ nsXBLContentSink::ConstructParameter(const PRUnichar **aAtts)
   if (!mMethod)
     return;
 
-  nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
-
-  for (; *aAtts; aAtts += 2) {
-    // Get upper-cased key
-    SplitXMLName(nsDependentString(aAtts[0]), getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(nameAtom));
-
-    if (nameSpacePrefix || nameAtom == nsLayoutAtoms::xmlnsNameSpace) {
-      continue;
-    }
-
-    // Is this attribute one of the ones we care about?
-    if (nameAtom == nsXBLAtoms::name) {
-      mMethod->AddParameter(nsDependentString(aAtts[1]));
-      break;
-    }
+  const PRUnichar* name = nsnull;
+  if (FindValue(aAtts, nsXBLAtoms::name, &name)) {
+    mMethod->AddParameter(nsDependentString(name));
   }
 }
 
@@ -841,6 +862,9 @@ nsXBLContentSink::CreateElement(const PRUnichar** aAtts, PRUint32 aAttsCount,
     return NS_ERROR_OUT_OF_MEMORY;
 
   prototype->mNodeInfo = aNodeInfo;
+  // XXX - we need to do exactly what the XUL content-sink does (eg,
+  // look for 'type', 'version' etc attributes)
+  prototype->mScriptTypeID = nsIProgrammingLanguage::JAVASCRIPT;
 
   AddAttributesToXULPrototype(aAtts, aAttsCount, prototype);
 
@@ -860,7 +884,7 @@ nsresult
 nsXBLContentSink::AddAttributes(const PRUnichar** aAtts,
                                 nsIContent* aContent)
 {
-  if (aContent->IsContentOfType(nsIContent::eXUL))
+  if (aContent->IsNodeOfType(nsINode::eXUL))
     return NS_OK; // Nothing to do, since the proto already has the attrs.
 
   return nsXMLContentSink::AddAttributes(aAtts, aContent);
@@ -887,38 +911,20 @@ nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts,
   aElement->mNumAttributes = aAttsCount;
 
   // Copy the attributes into the prototype
-  nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
+  nsCOMPtr<nsIAtom> prefix, localName;
 
   PRUint32 i;  
   for (i = 0; i < aAttsCount; ++i) {
-    const nsDependentString key(aAtts[i * 2]);
-
-    SplitXMLName(key, getter_AddRefs(nameSpacePrefix),
-                 getter_AddRefs(nameAtom));
-
     PRInt32 nameSpaceID;
-
-    if (nameSpacePrefix)
-        nameSpaceID = GetNameSpaceId(nameSpacePrefix);
-    else {
-      if (nameAtom == nsLayoutAtoms::xmlnsNameSpace)
-        nameSpaceID = kNameSpaceID_XMLNS;
-      else
-        nameSpaceID = kNameSpaceID_None;
-    }
-
-    if (kNameSpaceID_Unknown == nameSpaceID) {
-      nameSpaceID = kNameSpaceID_None;
-      nameAtom = do_GetAtom(key);
-      nameSpacePrefix = nsnull;
-    } 
+    nsContentUtils::SplitExpatName(aAtts[i * 2], getter_AddRefs(prefix),
+                                   getter_AddRefs(localName), &nameSpaceID);
 
     if (nameSpaceID == kNameSpaceID_None) {
-      attrs[i].mName.SetTo(nameAtom);
+      attrs[i].mName.SetTo(localName);
     }
     else {
       nsCOMPtr<nsINodeInfo> ni;
-      mNodeInfoManager->GetNodeInfo(nameAtom, nameSpacePrefix, nameSpaceID,
+      mNodeInfoManager->GetNodeInfo(localName, prefix, nameSpaceID,
                                     getter_AddRefs(ni));
       attrs[i].mName.SetTo(ni);
     }
