@@ -44,7 +44,6 @@
 #include "nsXBLWindowKeyHandler.h"
 #include "nsXBLWindowDragHandler.h"
 #include "nsIInputStream.h"
-#include "nsINameSpace.h"
 #include "nsINameSpaceManager.h"
 #include "nsHashtable.h"
 #include "nsIURI.h"
@@ -63,21 +62,20 @@
 #include "nsContentCID.h"
 #include "nsXMLDocument.h"
 #include "nsHTMLAtoms.h"
-#include "nsSupportsArray.h"
-#include "nsITextContent.h"
 #include "nsIMemory.h"
 #include "nsIObserverService.h"
 #include "nsIDOMNodeList.h"
 #include "nsXBLContentSink.h"
-#include "nsIXBLBinding.h"
+#include "nsXBLBinding.h"
 #include "nsXBLPrototypeBinding.h"
 #include "nsIXBLDocumentInfo.h"
 #include "nsXBLAtoms.h"
 #include "nsXULAtoms.h"
 #include "nsCRT.h"
 #include "nsContentUtils.h"
-#include "nsISyncLoadDOMService.h"
+#include "nsSyncLoadService.h"
 #include "nsIDOM3Node.h"
+#include "nsContentPolicyUtils.h"
 
 #include "nsIPresShell.h"
 #include "nsIDocumentObserver.h"
@@ -91,9 +89,6 @@
 #include "nsIDOMLoadListener.h"
 #include "nsIDOMEventGroup.h"
 
-// Static IIDs/CIDs. Try to minimize these.
-static NS_DEFINE_CID(kXMLDocumentCID,             NS_XMLDOCUMENT_CID);
-
 static PRBool IsChromeOrResourceURI(nsIURI* aURI)
 {
   PRBool isChrome = PR_FALSE;
@@ -104,14 +99,6 @@ static PRBool IsChromeOrResourceURI(nsIURI* aURI)
   return PR_FALSE;
 }
 
-static PRBool IsResourceURI(nsIURI* aURI)
-{
-  PRBool isResource = PR_FALSE;
-  if (NS_SUCCEEDED(aURI->SchemeIs("resource", &isResource)) && isResource)
-      return PR_TRUE; 
-  return PR_FALSE;
-}
-
 // Individual binding requests.
 class nsXBLBindingRequest
 {
@@ -133,7 +120,9 @@ public:
 
   void DocumentLoaded(nsIDocument* aBindingDoc)
   {
-    nsCOMPtr<nsIDocument> doc = mBoundElement->GetDocument();
+    // We only need the document here to cause frame construction, so
+    // we need the current doc, not the owner doc.
+    nsIDocument* doc = mBoundElement->GetCurrentDoc();
     if (!doc)
       return;
 
@@ -144,9 +133,9 @@ public:
     if (!ready)
       return;
 
-    // XXX Deal with layered bindings.
+    // XXX Deal with layered bindings.  For example, mBoundElement may be anonymous content.
     // Now do a ContentInserted notification to cause the frames to get installed finally,
-    nsCOMPtr<nsIContent> parent = mBoundElement->GetParent();
+    nsIContent* parent = mBoundElement->GetParent();
     PRInt32 index = 0;
     if (parent)
       index = parent->IndexOf(mBoundElement);
@@ -162,8 +151,7 @@ public:
     // will happen.
     nsIPresShell *shell = doc->GetShellAt(0);
     if (shell) {
-      nsIFrame* childFrame;
-      shell->GetPrimaryFrameFor(mBoundElement, &childFrame);
+      nsIFrame* childFrame = shell->GetPrimaryFrameFor(mBoundElement);
       if (!childFrame) {
         // Check to see if it's in the undisplayed content map.
         nsStyleContext* sc =
@@ -228,11 +216,11 @@ public:
   NS_DECL_NSIREQUESTOBSERVER
 
   NS_IMETHOD Load(nsIDOMEvent* aEvent);
-  NS_IMETHOD BeforeUnload(nsIDOMEvent* aEvent) { return NS_OK; };
-  NS_IMETHOD Unload(nsIDOMEvent* aEvent) { return NS_OK; };
-  NS_IMETHOD Abort(nsIDOMEvent* aEvent) { return NS_OK; };
-  NS_IMETHOD Error(nsIDOMEvent* aEvent) { return NS_OK; };
-  NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent) { return NS_OK; };
+  NS_IMETHOD BeforeUnload(nsIDOMEvent* aEvent) { return NS_OK; }
+  NS_IMETHOD Unload(nsIDOMEvent* aEvent) { return NS_OK; }
+  NS_IMETHOD Abort(nsIDOMEvent* aEvent) { return NS_OK; }
+  NS_IMETHOD Error(nsIDOMEvent* aEvent) { return NS_OK; }
+  NS_IMETHOD HandleEvent(nsIDOMEvent* aEvent) { return NS_OK; }
 
 #ifdef MOZ_XUL
   static nsIXULPrototypeCache* gXULCache;
@@ -382,13 +370,13 @@ nsXBLStreamListener::Load(nsIDOMEvent* aEvent)
     // ready.
     if (count > 0) {
       nsXBLBindingRequest* req = (nsXBLBindingRequest*)mBindingRequests.ElementAt(0);
-      nsIDocument* document = req->mBoundElement->GetDocument();
+      nsIDocument* document = req->mBoundElement->GetCurrentDoc();
       if (document)
         document->FlushPendingNotifications(Flush_ContentAndNotify);
     }
 
     // Remove ourselves from the set of pending docs.
-    nsIBindingManager *bindingManager = doc->GetBindingManager();
+    nsIBindingManager *bindingManager = doc->BindingManager();
     nsIURI* documentURI = mBindingDocument->GetDocumentURI();
     bindingManager->RemoveLoadingDocListener(documentURI);
 
@@ -399,7 +387,7 @@ nsXBLStreamListener::Load(nsIDOMEvent* aEvent)
 
     // Put our doc info in the doc table.
     nsCOMPtr<nsIXBLDocumentInfo> info;
-    nsIBindingManager *xblDocBindingManager = mBindingDocument->GetBindingManager();
+    nsIBindingManager *xblDocBindingManager = mBindingDocument->BindingManager();
     xblDocBindingManager->GetXBLDocumentInfo(documentURI, getter_AddRefs(info));
     xblDocBindingManager->RemoveXBLDocumentInfo(info); // Break the self-imposed cycle.
     if (!info) {
@@ -425,17 +413,6 @@ nsXBLStreamListener::Load(nsIDOMEvent* aEvent)
       nsXBLBindingRequest* req = (nsXBLBindingRequest*)mBindingRequests.ElementAt(i);
       req->DocumentLoaded(mBindingDocument);
     }
-
-    // All reqs normally have the same binding doc.  Force a synchronous
-    // reflow on this binding doc to deal with the fact that iframes
-    // don't construct or load their subdocs until they get a reflow.
-    // XXXbz this is still needed for <xul:iframe>.  We need to fix that.
-    if (count > 0) {
-      nsXBLBindingRequest* req = (nsXBLBindingRequest*)mBindingRequests.ElementAt(0);
-      nsIDocument* document = req->mBoundElement->GetDocument();
-      if (document)
-        document->FlushPendingNotifications(Flush_Layout);
-    }
   }
   
   for (i = 0; i < count; i++) {
@@ -520,36 +497,32 @@ nsXBLService::~nsXBLService(void)
 // onto the element.
 NS_IMETHODIMP
 nsXBLService::LoadBindings(nsIContent* aContent, nsIURI* aURL, PRBool aAugmentFlag,
-                           nsIXBLBinding** aBinding, PRBool* aResolveStyle) 
+                           nsXBLBinding** aBinding, PRBool* aResolveStyle) 
 { 
   *aBinding = nsnull;
   *aResolveStyle = PR_FALSE;
 
   nsresult rv;
 
-  nsCOMPtr<nsIDocument> document = aContent->GetDocument();
+  nsCOMPtr<nsIDocument> document = aContent->GetOwnerDoc();
 
   // XXX document may be null if we're in the midst of paint suppression
   if (!document)
     return NS_OK;
 
-  nsIBindingManager *bindingManager = document->GetBindingManager();
+  nsIBindingManager *bindingManager = document->BindingManager();
   
-  nsCOMPtr<nsIXBLBinding> binding;
-  bindingManager->GetBinding(aContent, getter_AddRefs(binding));
+  nsXBLBinding *binding = bindingManager->GetBinding(aContent);
   if (binding && !aAugmentFlag) {
-    nsCOMPtr<nsIXBLBinding> styleBinding;
-    binding->GetFirstStyleBinding(getter_AddRefs(styleBinding));
+    nsXBLBinding *styleBinding = binding->GetFirstStyleBinding();
     if (styleBinding) {
-      PRBool marked = PR_FALSE;
-      binding->MarkedForDeath(&marked);
-      if (marked) {
+      if (binding->MarkedForDeath()) {
         FlushStyleBindings(aContent);
         binding = nsnull;
       }
       else {
         // See if the URIs match.
-        nsIURI* uri = styleBinding->BindingURI();
+        nsIURI* uri = styleBinding->PrototypeBinding()->BindingURI();
         PRBool equal;
         if (NS_SUCCEEDED(uri->Equals(aURL, &equal)) && equal)
           return NS_OK;
@@ -566,16 +539,40 @@ nsXBLService::LoadBindings(nsIContent* aContent, nsIURI* aURL, PRBool aAugmentFl
 
   // Not everything with a chrome URI has a system principal.  See bug 160042.
   if (NS_FAILED(rv) || !isChrome) {
-    nsIScriptSecurityManager *secMan = nsContentUtils::GetSecurityManager();
-
-    rv = secMan->
-      CheckLoadURIWithPrincipal(document->GetPrincipal(), aURL,
+    rv = nsContentUtils::GetSecurityManager()->
+      CheckLoadURIWithPrincipal(document->NodePrincipal(), aURL,
                                 nsIScriptSecurityManager::ALLOW_CHROME);
     if (NS_FAILED(rv))
       return rv;
   }
-  nsCOMPtr<nsIXBLBinding> newBinding;
-  if (NS_FAILED(rv = GetBinding(aContent, aURL, getter_AddRefs(newBinding)))) {
+
+  // Content policy check.  We have to be careful to not pass aContent as the
+  // context here.  Otherwise, if there is a JS-implemented content policy, we
+  // will attempt to wrap the content node, which will try to load XBL bindings
+  // for it, if any.  Since we're not done loading this binding yet, that will
+  // reenter this method and we'll end up creating a binding and then
+  // immediately clobbering it in our table.  That makes things very confused,
+  // leading to misbehavior and crashes.
+  PRInt16 decision = nsIContentPolicy::ACCEPT;
+  rv = NS_CheckContentLoadPolicy(nsIContentPolicy::TYPE_OTHER,
+                                 aURL,
+                                 docURI,
+                                 document,        // context
+                                 EmptyCString(),  // mime guess
+                                 nsnull,          // extra
+                                 &decision,
+                                 nsContentUtils::GetContentPolicy());
+
+  if (NS_SUCCEEDED(rv) && !NS_CP_ACCEPTED(decision))
+    rv = NS_ERROR_NOT_AVAILABLE;
+
+  if (NS_FAILED(rv))
+    return rv;
+
+  PRBool ready;
+  nsRefPtr<nsXBLBinding> newBinding;
+  if (NS_FAILED(rv = GetBinding(aContent, aURL, PR_FALSE, &ready,
+                                getter_AddRefs(newBinding)))) {
     return rv;
   }
 
@@ -590,27 +587,25 @@ nsXBLService::LoadBindings(nsIContent* aContent, nsIURI* aURL, PRBool aAugmentFl
   }
 
   if (aAugmentFlag) {
-    nsCOMPtr<nsIXBLBinding> baseBinding;
-    nsCOMPtr<nsIXBLBinding> nextBinding = newBinding;
+    nsXBLBinding *baseBinding;
+    nsXBLBinding *nextBinding = newBinding;
     do {
       baseBinding = nextBinding;
-      baseBinding->GetBaseBinding(getter_AddRefs(nextBinding));
+      nextBinding = baseBinding->GetBaseBinding();
       baseBinding->SetIsStyleBinding(PR_FALSE);
     } while (nextBinding);
 
     // XXX Handle adjusting the prototype chain! We need to somehow indicate to
     // InstallImplementation that the whole chain should just be whacked and rebuilt.
     // We are becoming the new binding.
-    bindingManager->SetBinding(aContent, newBinding);
     baseBinding->SetBaseBinding(binding);
+    bindingManager->SetBinding(aContent, newBinding);
   }
   else {
     // We loaded a style binding.  It goes on the end.
     if (binding) {
       // Get the last binding that is in the append layer.
-      nsCOMPtr<nsIXBLBinding> rootBinding;
-      binding->GetRootBinding(getter_AddRefs(rootBinding));
-      rootBinding->SetBaseBinding(newBinding);
+      binding->RootBinding()->SetBaseBinding(newBinding);
     }
     else {
       // Install the binding on the content node.
@@ -628,13 +623,15 @@ nsXBLService::LoadBindings(nsIContent* aContent, nsIURI* aURL, PRBool aAugmentFl
   newBinding->InstallEventHandlers();
 
   // Set up our properties
-  newBinding->InstallImplementation();
+  rv = newBinding->InstallImplementation();
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Figure out if we need to execute a constructor.
-  newBinding->GetFirstBindingWithConstructor(aBinding);
+  *aBinding = newBinding->GetFirstBindingWithConstructor();
+  NS_IF_ADDREF(*aBinding);
 
   // Figure out if we have any scoped sheets.  If so, we do a second resolve.
-  newBinding->HasStyleSheets(aResolveStyle);
+  *aResolveStyle = newBinding->HasStyleSheets();
   
   return NS_OK; 
 }
@@ -642,20 +639,18 @@ nsXBLService::LoadBindings(nsIContent* aContent, nsIURI* aURL, PRBool aAugmentFl
 nsresult
 nsXBLService::FlushStyleBindings(nsIContent* aContent)
 {
-  nsCOMPtr<nsIDocument> document = aContent->GetDocument();
+  nsCOMPtr<nsIDocument> document = aContent->GetOwnerDoc();
 
   // XXX doc will be null if we're in the midst of paint suppression.
   if (! document)
     return NS_OK;
 
-  nsIBindingManager *bindingManager = document->GetBindingManager();
+  nsIBindingManager *bindingManager = document->BindingManager();
   
-  nsCOMPtr<nsIXBLBinding> binding;
-  bindingManager->GetBinding(aContent, getter_AddRefs(binding));
+  nsXBLBinding *binding = bindingManager->GetBinding(aContent);
   
   if (binding) {
-    nsCOMPtr<nsIXBLBinding> styleBinding;
-    binding->GetFirstStyleBinding(getter_AddRefs(styleBinding));
+    nsXBLBinding *styleBinding = binding->GetFirstStyleBinding();
 
     if (styleBinding) {
       // Clear out the script references.
@@ -674,15 +669,13 @@ NS_IMETHODIMP
 nsXBLService::ResolveTag(nsIContent* aContent, PRInt32* aNameSpaceID,
                          nsIAtom** aResult)
 {
-  nsIDocument* document = aContent->GetDocument();
+  nsIDocument* document = aContent->GetOwnerDoc();
   if (document) {
-    nsIBindingManager *bindingManager = document->GetBindingManager();
-  
-    if (bindingManager)
-      return bindingManager->ResolveTag(aContent, aNameSpaceID, aResult);
+    return document->BindingManager()->ResolveTag(aContent, aNameSpaceID,
+                                                  aResult);
   }
 
-  aContent->GetNameSpaceID(aNameSpaceID);
+  *aNameSpaceID = aContent->GetNameSpaceID();
   NS_ADDREF(*aResult = aContent->Tag());
 
   return NS_OK;
@@ -706,9 +699,9 @@ nsXBLService::GetXBLDocumentInfo(nsIURI* aURI, nsIContent* aBoundElement, nsIXBL
 
   if (!*aResult) {
     // The second line of defense is the binding manager's document table.
-    nsIDocument* boundDocument = aBoundElement->GetDocument();
+    nsIDocument* boundDocument = aBoundElement->GetOwnerDoc();
     if (boundDocument)
-      boundDocument->GetBindingManager()->GetXBLDocumentInfo(aURI, aResult);
+      boundDocument->BindingManager()->GetXBLDocumentInfo(aURI, aResult);
   }
   return NS_OK;
 }
@@ -729,7 +722,8 @@ nsXBLService::AttachGlobalKeyHandler(nsIDOMEventReceiver* aReceiver)
   nsCOMPtr<nsIDOMEventReceiver> rec = aReceiver;
   nsCOMPtr<nsIContent> contentNode(do_QueryInterface(aReceiver));
   if (contentNode) {
-    nsCOMPtr<nsIDocument> doc = contentNode->GetDocument();
+    // Only attach if we're really in a document
+    nsCOMPtr<nsIDocument> doc = contentNode->GetCurrentDoc();
     if (doc)
       rec = do_QueryInterface(doc); // We're a XUL keyset. Attach to our document.
   }
@@ -828,27 +822,22 @@ nsXBLService::FlushMemory()
 
 // Internal helper methods ////////////////////////////////////////////////////////////////
 
-nsresult nsXBLService::GetBinding(nsIContent* aBoundElement, 
-                                  nsIURI* aURI, 
-                                  nsIXBLBinding** aResult)
-{
-  PRBool dummy;
-  return GetBindingInternal(aBoundElement, aURI, PR_FALSE, &dummy, aResult);
-}
-
 NS_IMETHODIMP nsXBLService::BindingReady(nsIContent* aBoundElement, 
                                          nsIURI* aURI, 
                                          PRBool* aIsReady)
 {
-  return GetBindingInternal(aBoundElement, aURI, PR_TRUE, aIsReady, nsnull);
+  return GetBinding(aBoundElement, aURI, PR_TRUE, aIsReady, nsnull);
 }
 
-NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement, 
-                                               nsIURI* aURI, 
-                                               PRBool aPeekOnly,
-                                               PRBool* aIsReady, 
-                                               nsIXBLBinding** aResult)
+nsresult
+nsXBLService::GetBinding(nsIContent* aBoundElement, nsIURI* aURI, 
+                         PRBool aPeekOnly, PRBool* aIsReady, 
+                         nsXBLBinding** aResult)
 {
+  NS_ASSERTION(aPeekOnly || aResult,
+               "Must have non-null out param if not just peeking to see "
+               "whether the binding is ready");
+  
   if (aResult)
     *aResult = nsnull;
 
@@ -872,7 +861,7 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
   if (ref.IsEmpty())
     return NS_ERROR_FAILURE;
   
-  nsCOMPtr<nsIDocument> boundDocument = aBoundElement->GetDocument();
+  nsCOMPtr<nsIDocument> boundDocument = aBoundElement->GetOwnerDoc();
 
   nsCOMPtr<nsIXBLDocumentInfo> docInfo;
   LoadBindingDocumentInfo(aBoundElement, boundDocument, aURI, PR_FALSE,
@@ -905,12 +894,12 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
   }
 
   // If our prototype already has a base, then don't check for an "extends" attribute.
-  nsCOMPtr<nsIXBLBinding> baseBinding;
+  nsRefPtr<nsXBLBinding> baseBinding;
   PRBool hasBase = protoBinding->HasBasePrototype();
   nsXBLPrototypeBinding* baseProto = protoBinding->GetBasePrototype();
   if (baseProto) {
-    if (NS_FAILED(GetBindingInternal(aBoundElement, baseProto->BindingURI(),
-                                     aPeekOnly, aIsReady, getter_AddRefs(baseBinding))))
+    if (NS_FAILED(GetBinding(aBoundElement, baseProto->BindingURI(),
+                             aPeekOnly, aIsReady, getter_AddRefs(baseBinding))))
       return NS_ERROR_FAILURE; // We aren't ready yet.
   }
   else if (hasBase) {
@@ -963,10 +952,8 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
               //child->UnsetAttr(kNameSpaceID_None, nsXBLAtoms::extends, PR_FALSE);
             }
 
-            PRInt32 nameSpaceID;
-
-            nsContentUtils::GetNSManagerWeakRef()->GetNameSpaceID(nameSpace,
-                                                                  &nameSpaceID);
+            PRInt32 nameSpaceID =
+              nsContentUtils::NameSpaceManager()->GetNameSpaceID(nameSpace);
 
             nsCOMPtr<nsIAtom> tagName = do_GetAtom(display);
             protoBinding->SetBaseTag(nameSpaceID, tagName);
@@ -984,12 +971,12 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
                     doc->GetBaseURI());
         NS_ENSURE_SUCCESS(rv, rv);
         
-        if (NS_FAILED(GetBindingInternal(aBoundElement, bindingURI, aPeekOnly,
-                                         aIsReady, getter_AddRefs(baseBinding))))
+        if (NS_FAILED(GetBinding(aBoundElement, bindingURI, aPeekOnly,
+                                 aIsReady, getter_AddRefs(baseBinding))))
           return NS_ERROR_FAILURE; // Binding not yet ready or an error occurred.
         if (!aPeekOnly) {
           // Make sure to set the base prototype.
-          baseBinding->GetPrototypeBinding(&baseProto);
+          baseProto = baseBinding->PrototypeBinding();
           protoBinding->SetBasePrototype(baseProto);
           child->UnsetAttr(kNameSpaceID_None, nsXBLAtoms::extends, PR_FALSE);
           child->UnsetAttr(kNameSpaceID_None, nsXBLAtoms::display, PR_FALSE);
@@ -1001,9 +988,13 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
   *aIsReady = PR_TRUE;
   if (!aPeekOnly) {
     // Make a new binding
-    NS_NewXBLBinding(protoBinding, aResult);
+    nsXBLBinding *newBinding = new nsXBLBinding(protoBinding);
+    NS_ENSURE_TRUE(newBinding, NS_ERROR_OUT_OF_MEMORY);
+
     if (baseBinding)
-      (*aResult)->SetBaseBinding(baseBinding);
+      newBinding->SetBaseBinding(baseBinding);
+
+    NS_ADDREF(*aResult = newBinding);
   }
 
   return NS_OK;
@@ -1053,20 +1044,20 @@ nsXBLService::LoadBindingDocumentInfo(nsIContent* aBoundElement,
     NS_ENSURE_SUCCESS(rv, rv);
 
     if (aBoundDocument) {
-      bindingManager = aBoundDocument->GetBindingManager();
+      bindingManager = aBoundDocument->BindingManager();
       bindingManager->GetXBLDocumentInfo(documentURI, getter_AddRefs(info));
     }
 
     nsINodeInfo *ni = nsnull;
     if (aBoundElement)
-      ni = aBoundElement->GetNodeInfo();
+      ni = aBoundElement->NodeInfo();
 
     if (!info && bindingManager &&
         (!ni || !(ni->Equals(nsXULAtoms::scrollbar, kNameSpaceID_XUL) ||
                   ni->Equals(nsXULAtoms::thumb, kNameSpaceID_XUL) ||
                   ((ni->Equals(nsHTMLAtoms::input) ||
                     ni->Equals(nsHTMLAtoms::select)) &&
-                   aBoundElement->IsContentOfType(nsIContent::eHTML)))) &&
+                   aBoundElement->IsNodeOfType(nsINode::eHTML)))) &&
         !aForceSyncLoad) {
       // The third line of defense is to investigate whether or not the
       // document is currently being loaded asynchronously.  If so, there's no
@@ -1101,7 +1092,7 @@ nsXBLService::LoadBindingDocumentInfo(nsIContent* aBoundElement,
                            bindingURL, aForceSyncLoad, getter_AddRefs(document));
    
       if (document) {
-        nsIBindingManager *xblDocBindingManager = document->GetBindingManager();
+        nsIBindingManager *xblDocBindingManager = document->BindingManager();
         xblDocBindingManager->GetXBLDocumentInfo(documentURI, getter_AddRefs(info));
         if (!info) {
           NS_ERROR("An XBL file is malformed.  Did you forget the XBL namespace on the bindings tag?");
@@ -1149,43 +1140,36 @@ nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoun
   if (aBoundDocument)
     loadGroup = aBoundDocument->GetDocumentLoadGroup();
 
-  nsINodeInfo *ni = nsnull;
-  if (aBoundElement)
-    ni = aBoundElement->GetNodeInfo();
-
-  if (ni && (ni->Equals(nsXULAtoms::scrollbar, kNameSpaceID_XUL) ||
-             ni->Equals(nsXULAtoms::thumb, kNameSpaceID_XUL) || 
-             (ni->Equals(nsHTMLAtoms::select) &&
-              aBoundElement->IsContentOfType(nsIContent::eHTML))) ||
-      IsResourceURI(aDocumentURI))
+  // We really shouldn't have to force a sync load for anything here... could
+  // we get away with not doing that?  Not sure.
+  if (IsChromeOrResourceURI(aDocumentURI))
     aForceSyncLoad = PR_TRUE;
 
-  if(!aForceSyncLoad) {
-    // Create the XML document
-    nsCOMPtr<nsIDocument> doc = do_CreateInstance(kXMLDocumentCID, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
+  // Create document and contentsink and set them up.
+  nsCOMPtr<nsIDocument> doc;
+  rv = NS_NewXMLDocument(getter_AddRefs(doc));
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsIChannel> channel;
-    rv = NS_NewChannel(getter_AddRefs(channel), aDocumentURI, nsnull, loadGroup);
-    if (NS_FAILED(rv)) return rv;
+  nsCOMPtr<nsIXMLContentSink> xblSink;
+  rv = NS_NewXBLContentSink(getter_AddRefs(xblSink), doc, aDocumentURI, nsnull);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsIStreamListener> listener;
-    nsCOMPtr<nsIXMLContentSink> xblSink;
-    NS_NewXBLContentSink(getter_AddRefs(xblSink), doc, aDocumentURI, nsnull);
-    if (!xblSink)
-      return NS_ERROR_FAILURE;
+  // Open channel
+  nsCOMPtr<nsIChannel> channel;
+  rv = NS_NewChannel(getter_AddRefs(channel), aDocumentURI, nsnull, loadGroup);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    if (NS_FAILED(rv = doc->StartDocumentLoad("loadAsInteractiveData", 
-                                              channel, 
-                                              loadGroup, 
-                                              nsnull, 
-                                              getter_AddRefs(listener),
-                                              PR_TRUE,
-                                              xblSink))) {
-      NS_ERROR("Failure to init XBL doc prior to load.");
-      return rv;
-    }
+  nsCOMPtr<nsIStreamListener> listener;
+  rv = doc->StartDocumentLoad("loadAsInteractiveData",
+                              channel,
+                              loadGroup,
+                              nsnull,
+                              getter_AddRefs(listener),
+                              PR_TRUE,
+                              xblSink);
+  NS_ENSURE_SUCCESS(rv, rv);
 
+  if (!aForceSyncLoad) {
     // We can be asynchronous
     nsXBLStreamListener* xblListener = new nsXBLStreamListener(this, listener, aBoundDocument, doc);
     NS_ENSURE_TRUE(xblListener,NS_ERROR_OUT_OF_MEMORY);
@@ -1196,7 +1180,7 @@ nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoun
     // Add ourselves to the list of loading docs.
     nsIBindingManager *bindingManager;
     if (aBoundDocument)
-      bindingManager = aBoundDocument->GetBindingManager();
+      bindingManager = aBoundDocument->BindingManager();
     else
       bindingManager = nsnull;
 
@@ -1215,24 +1199,16 @@ nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoun
   }
 
   // Now do a blocking synchronous parse of the file.
-
-  nsCOMPtr<nsIDOMDocument> domDoc;
-  nsCOMPtr<nsISyncLoadDOMService> loader =
-    do_GetService("@mozilla.org/content/syncload-dom-service;1", &rv);
+  nsCOMPtr<nsIInputStream> in;
+  rv = channel->Open(getter_AddRefs(in));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // Open channel
-  nsCOMPtr<nsIChannel> channel;
-  rv = NS_NewChannel(getter_AddRefs(channel), aDocumentURI, nsnull, loadGroup);
+  rv = nsSyncLoadService::PushSyncStreamToListener(in, listener, channel);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = loader->LoadLocalXBLDocument(channel, getter_AddRefs(domDoc));
-  if (rv == NS_ERROR_FILE_NOT_FOUND) {
-      return NS_OK;
-  }
-  NS_ENSURE_SUCCESS(rv, rv);
+  doc.swap(*aResult);
 
-  return CallQueryInterface(domDoc, aResult);
+  return NS_OK;
 }
 
 // Creation Routine ///////////////////////////////////////////////////////////////////////
