@@ -1,10 +1,10 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ----- BEGIN LICENSE BLOCK -----
+/* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Mozilla Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
  * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
@@ -14,27 +14,27 @@
  *
  * The Original Code is the Mozilla XTF project.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Alex Fritze.
  * Portions created by the Initial Developer are Copyright (C) 2004
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *    Alex Fritze <alex@croczilla.com> (original author)
+ *   Alex Fritze <alex@croczilla.com> (original author)
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
- * ----- END LICENSE BLOCK ----- */
+ * ***** END LICENSE BLOCK ***** */
 
 #include "nsXTFElementWrapper.h"
 #include "nsIXTFElement.h"
@@ -49,10 +49,21 @@
 #include "nsIEventListenerManager.h"
 #include "nsIDOMEvent.h"
 #include "nsGUIEvent.h"
+#include "nsContentUtils.h"
+#include "nsIXTFService.h"
+#include "nsDOMAttributeMap.h"
+#include "nsUnicharUtils.h"
+#include "nsLayoutAtoms.h"
+#include "nsEventDispatcher.h"
+#include "nsIProgrammingLanguage.h"
+#include "nsIXPConnect.h"
 
 nsXTFElementWrapper::nsXTFElementWrapper(nsINodeInfo* aNodeInfo)
     : nsXTFElementWrapperBase(aNodeInfo),
-      mNotificationMask(0)
+      mNotificationMask(0),
+      mIntrinsicState(0),
+      mTmpAttrName(nsLayoutAtoms::_asterix) // XXX this is a hack, but names
+                                            // have to have a value
 {
 }
 
@@ -97,14 +108,16 @@ nsXTFElementWrapper::QueryInterface(REFNSIID aIID, void** aInstancePtr)
   }
   else {
     // try to get get the interface from our wrapped element:
-    void *innerPtr = nsnull;
-    QueryInterfaceInner(aIID, &innerPtr);
+    nsCOMPtr<nsISupports> inner;
+    QueryInterfaceInner(aIID, getter_AddRefs(inner));
 
-    if (innerPtr)
-      return NS_NewXTFInterfaceAggregator(aIID,
-                                          NS_STATIC_CAST(nsISupports*, innerPtr),
-                                          NS_STATIC_CAST(nsIContent*, this),
-                                          aInstancePtr);
+    if (inner) {
+      rv = NS_NewXTFInterfaceAggregator(aIID, inner,
+                                        NS_STATIC_CAST(nsIContent*, this),
+                                        aInstancePtr);
+
+      return rv;
+    }
   }
 
   return NS_ERROR_NO_INTERFACE;
@@ -113,51 +126,88 @@ nsXTFElementWrapper::QueryInterface(REFNSIID aIID, void** aInstancePtr)
 //----------------------------------------------------------------------
 // nsIContent methods:
 
-void
-nsXTFElementWrapper::SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                                 PRBool aCompileEventHandlers)
+nsresult
+nsXTFElementWrapper::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                                nsIContent* aBindingParent,
+                                PRBool aCompileEventHandlers)
 {
-  // XXX For some reason we often get 2 SetDocument notifications with
-  // identical aDocument (one when expat encounters the element and
-  // another when the element is appended to its parent). We want to
-  // make sure that we only route notifications if the document has
-  // actually changed.
-  bool docReallyChanged = false;
-  if (aDocument!=GetCurrentDoc()) docReallyChanged = true;
-  
+  // XXXbz making up random order for the notifications... Perhaps
+  // this api should more closely match BindToTree/UnbindFromTree?
+  nsCOMPtr<nsIDOMElement> domParent;
+  if (aParent != GetParent()) {
+    domParent = do_QueryInterface(aParent);
+  }
+
   nsCOMPtr<nsIDOMDocument> domDocument;
-  if (docReallyChanged &&
-      mNotificationMask & (nsIXTFElement::NOTIFY_WILL_CHANGE_DOCUMENT |
-                           nsIXTFElement::NOTIFY_DOCUMENT_CHANGED))
+  if (aDocument &&
+      (mNotificationMask & (nsIXTFElement::NOTIFY_WILL_CHANGE_DOCUMENT |
+                            nsIXTFElement::NOTIFY_DOCUMENT_CHANGED))) {
     domDocument = do_QueryInterface(aDocument);
-  
-  if (docReallyChanged &&
-      (mNotificationMask & nsIXTFElement::NOTIFY_WILL_CHANGE_DOCUMENT))
+  }
+
+  if (domDocument &&
+      (mNotificationMask & (nsIXTFElement::NOTIFY_WILL_CHANGE_DOCUMENT))) {
     GetXTFElement()->WillChangeDocument(domDocument);
-  nsXTFElementWrapperBase::SetDocument(aDocument, aDeep, aCompileEventHandlers);
-  if (docReallyChanged &&
-      (mNotificationMask & nsIXTFElement::NOTIFY_DOCUMENT_CHANGED))
+  }
+
+  if (domParent &&
+      (mNotificationMask & (nsIXTFElement::NOTIFY_WILL_CHANGE_PARENT))) {
+    GetXTFElement()->WillChangeParent(domParent);
+  }
+
+  nsresult rv = nsXTFElementWrapperBase::BindToTree(aDocument, aParent,
+                                                    aBindingParent,
+                                                    aCompileEventHandlers);
+
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (domDocument &&
+      (mNotificationMask & (nsIXTFElement::NOTIFY_DOCUMENT_CHANGED))) {
     GetXTFElement()->DocumentChanged(domDocument);
+  }
+
+  if (domParent &&
+      (mNotificationMask & (nsIXTFElement::NOTIFY_PARENT_CHANGED))) {
+    GetXTFElement()->ParentChanged(domParent);
+  }
+
+  return rv;  
 }
 
 void
-nsXTFElementWrapper::SetParent(nsIContent* aParent)
+nsXTFElementWrapper::UnbindFromTree(PRBool aDeep, PRBool aNullParent)
 {
-  nsCOMPtr<nsIDOMElement> domParent;
-  if (mNotificationMask & (nsIXTFElement::NOTIFY_WILL_CHANGE_PARENT |
-                           nsIXTFElement::NOTIFY_PARENT_CHANGED))
-    domParent = do_QueryInterface(aParent);
-  
-  if (mNotificationMask & nsIXTFElement::NOTIFY_WILL_CHANGE_PARENT)
-    GetXTFElement()->WillChangeParent(domParent);
-  nsXTFElementWrapperBase::SetParent(aParent);
-  if (mNotificationMask & nsIXTFElement::NOTIFY_PARENT_CHANGED)
-    GetXTFElement()->ParentChanged(domParent);
+  // XXXbz making up random order for the notifications... Perhaps
+  // this api should more closely match BindToTree/UnbindFromTree?
+  PRBool inDoc = IsInDoc();
+  if (inDoc &&
+      (mNotificationMask & nsIXTFElement::NOTIFY_WILL_CHANGE_DOCUMENT)) {
+    GetXTFElement()->WillChangeDocument(nsnull);
+  }
+
+  PRBool parentChanged = aNullParent && GetParent();
+
+  if (parentChanged &&
+      (mNotificationMask & nsIXTFElement::NOTIFY_WILL_CHANGE_PARENT)) {
+    GetXTFElement()->WillChangeParent(nsnull);
+  }
+
+  nsXTFElementWrapperBase::UnbindFromTree(aDeep, aNullParent);
+
+  if (parentChanged &&
+      (mNotificationMask & nsIXTFElement::NOTIFY_PARENT_CHANGED)) {
+    GetXTFElement()->ParentChanged(nsnull);
+  }
+
+  if (inDoc &&
+      (mNotificationMask & nsIXTFElement::NOTIFY_DOCUMENT_CHANGED)) {
+    GetXTFElement()->DocumentChanged(nsnull);
+  }
 }
 
 nsresult
 nsXTFElementWrapper::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                   PRBool aNotify, PRBool aDeepSetDocument)
+                                   PRBool aNotify)
 {
   nsresult rv;
 
@@ -168,32 +218,13 @@ nsXTFElementWrapper::InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
   
   if (mNotificationMask & nsIXTFElement::NOTIFY_WILL_INSERT_CHILD)
     GetXTFElement()->WillInsertChild(domKid, aIndex);
-  rv = nsXTFElementWrapperBase::InsertChildAt(aKid, aIndex, aNotify, aDeepSetDocument);
+  rv = nsXTFElementWrapperBase::InsertChildAt(aKid, aIndex, aNotify);
   if (mNotificationMask & nsIXTFElement::NOTIFY_CHILD_INSERTED)
     GetXTFElement()->ChildInserted(domKid, aIndex);
   
   return rv;
 }
 
-nsresult
-nsXTFElementWrapper::AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                   PRBool aDeepSetDocument)
-{
-  nsresult rv;
-
-  nsCOMPtr<nsIDOMNode> domKid;
-  if (mNotificationMask & (nsIXTFElement::NOTIFY_WILL_APPEND_CHILD |
-                           nsIXTFElement::NOTIFY_CHILD_APPENDED))
-    domKid = do_QueryInterface(aKid);
-  
-  if (mNotificationMask & nsIXTFElement::NOTIFY_WILL_APPEND_CHILD)
-    GetXTFElement()->WillAppendChild(domKid);
-  rv = nsXTFElementWrapperBase::AppendChildTo(aKid, aNotify, aDeepSetDocument);
-  if (mNotificationMask & nsIXTFElement::NOTIFY_CHILD_APPENDED)
-    GetXTFElement()->ChildAppended(domKid);
-  return rv;
-}
-
 nsresult
 nsXTFElementWrapper::RemoveChildAt(PRUint32 aIndex, PRBool aNotify)
 {
@@ -242,22 +273,14 @@ nsXTFElementWrapper::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
   return rv;
 }
 
-nsresult
+PRBool
 nsXTFElementWrapper::GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, 
                              nsAString& aResult) const
 {
   if (aNameSpaceID==kNameSpaceID_None && HandledByInner(aName)) {
     // XXX we don't do namespaced attributes yet
-    if (aNameSpaceID != kNameSpaceID_None) {
-      NS_WARNING("getattr: xtf elements don't do namespaced attribs yet!");
-      return NS_CONTENT_ATTR_NOT_THERE;
-    }
     nsresult rv = mAttributeHandler->GetAttribute(aName, aResult);
-    if (NS_FAILED(rv)) return rv;
-    if (aResult.IsVoid()) return NS_CONTENT_ATTR_NOT_THERE;
-    if (aResult.IsEmpty()) return NS_CONTENT_ATTR_NO_VALUE;
-    
-    return NS_CONTENT_ATTR_HAS_VALUE;
+    return NS_SUCCEEDED(rv) && !aResult.IsVoid();
   }
   else { // try wrapper
     return nsXTFElementWrapperBase::GetAttr(aNameSpaceID, aName, aResult);
@@ -277,6 +300,91 @@ nsXTFElementWrapper::HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const
   }
 }
 
+PRBool
+nsXTFElementWrapper::AttrValueIs(PRInt32 aNameSpaceID,
+                                 nsIAtom* aName,
+                                 const nsAString& aValue,
+                                 nsCaseTreatment aCaseSensitive) const
+{
+  NS_ASSERTION(aName, "Must have attr name");
+  NS_ASSERTION(aNameSpaceID != kNameSpaceID_Unknown, "Must have namespace");
+
+  if (aNameSpaceID == kNameSpaceID_None && HandledByInner(aName)) {
+    nsAutoString ourVal;
+    if (!GetAttr(aNameSpaceID, aName, ourVal)) {
+      return PR_FALSE;
+    }
+    return aCaseSensitive == eCaseMatters ?
+      aValue.Equals(ourVal) :
+      aValue.Equals(ourVal, nsCaseInsensitiveStringComparator());
+  }
+
+  return nsXTFElementWrapperBase::AttrValueIs(aNameSpaceID, aName, aValue,
+                                              aCaseSensitive);
+}
+
+PRBool
+nsXTFElementWrapper::AttrValueIs(PRInt32 aNameSpaceID,
+                                 nsIAtom* aName,
+                                 nsIAtom* aValue,
+                                 nsCaseTreatment aCaseSensitive) const
+{
+  NS_ASSERTION(aName, "Must have attr name");
+  NS_ASSERTION(aNameSpaceID != kNameSpaceID_Unknown, "Must have namespace");
+  NS_ASSERTION(aValue, "Null value atom");
+
+  if (aNameSpaceID == kNameSpaceID_None && HandledByInner(aName)) {
+    nsAutoString ourVal;
+    if (!GetAttr(aNameSpaceID, aName, ourVal)) {
+      return PR_FALSE;
+    }
+    if (aCaseSensitive == eCaseMatters) {
+      return aValue->Equals(ourVal);
+    }
+    nsAutoString val;
+    aValue->ToString(val);
+    return val.Equals(ourVal, nsCaseInsensitiveStringComparator());
+  }
+
+  return nsXTFElementWrapperBase::AttrValueIs(aNameSpaceID, aName, aValue,
+                                              aCaseSensitive);
+}
+
+PRInt32
+nsXTFElementWrapper::FindAttrValueIn(PRInt32 aNameSpaceID,
+                                     nsIAtom* aName,
+                                     AttrValuesArray* aValues,
+                                     nsCaseTreatment aCaseSensitive) const
+{
+  NS_ASSERTION(aName, "Must have attr name");
+  NS_ASSERTION(aNameSpaceID != kNameSpaceID_Unknown, "Must have namespace");
+  NS_ASSERTION(aValues, "Null value array");
+  
+  if (aNameSpaceID == kNameSpaceID_None && HandledByInner(aName)) {
+    nsAutoString ourVal;
+    if (!GetAttr(aNameSpaceID, aName, ourVal)) {
+      return ATTR_MISSING;
+    }
+    
+    for (PRInt32 i = 0; aValues[i]; ++i) {
+      if (aCaseSensitive == eCaseMatters) {
+        if ((*aValues[i])->Equals(ourVal)) {
+          return i;
+        }
+      } else {
+        nsAutoString val;
+        (*aValues[i])->ToString(val);
+        if (val.Equals(ourVal, nsCaseInsensitiveStringComparator())) {
+          return i;
+        }
+      }
+    }
+    return ATTR_VALUE_NO_MATCH;
+  }
+
+  return nsXTFElementWrapperBase::FindAttrValueIn(aNameSpaceID, aName, aValues,
+                                                  aCaseSensitive);
+}
 
 nsresult
 nsXTFElementWrapper::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr, 
@@ -293,7 +401,17 @@ nsXTFElementWrapper::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr,
       NS_WARNING("setattr: xtf elements don't do namespaced attribs yet!");
       return NS_ERROR_FAILURE;
     }  
+    nsDOMSlots *slots = GetExistingDOMSlots();
+    if (slots && slots->mAttributeMap) {
+      slots->mAttributeMap->DropAttribute(aNameSpaceID, aAttr);
+    }
     rv = mAttributeHandler->RemoveAttribute(aAttr);
+
+    // XXX if the RemoveAttribute() call fails, we might end up having removed
+    // the attribute from the attribute map even though the attribute is still
+    // on the element
+    // https://bugzilla.mozilla.org/show_bug.cgi?id=296205
+
     // XXX mutation events?
   }
   else { // try wrapper
@@ -306,9 +424,8 @@ nsXTFElementWrapper::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr,
   return rv;
 }
 
-nsresult
-nsXTFElementWrapper::GetAttrNameAt(PRUint32 aIndex, PRInt32* aNameSpaceID,
-                                   nsIAtom** aName, nsIAtom** aPrefix) const
+const nsAttrName*
+nsXTFElementWrapper::GetAttrNameAt(PRUint32 aIndex) const
 {
   PRUint32 innerCount=0;
   if (mAttributeHandler) {
@@ -316,13 +433,15 @@ nsXTFElementWrapper::GetAttrNameAt(PRUint32 aIndex, PRInt32* aNameSpaceID,
   }
   
   if (aIndex < innerCount) {
-    *aNameSpaceID = kNameSpaceID_None;
-    *aPrefix = nsnull;
-    return mAttributeHandler->GetAttributeNameAt(aIndex, aName);
+    nsCOMPtr<nsIAtom> localName;
+    nsresult rv = mAttributeHandler->GetAttributeNameAt(aIndex, getter_AddRefs(localName));
+    NS_ENSURE_SUCCESS(rv, nsnull);
+
+    NS_CONST_CAST(nsXTFElementWrapper*, this)->mTmpAttrName.SetTo(localName);
+    return &mTmpAttrName;
   }
   else { // wrapper handles attrib
-    return nsXTFElementWrapperBase::GetAttrNameAt(aIndex - innerCount, aNameSpaceID,
-                                                  aName, aPrefix);
+    return nsXTFElementWrapperBase::GetAttrNameAt(aIndex - innerCount);
   }
 }
 
@@ -338,7 +457,14 @@ nsXTFElementWrapper::GetAttrCount() const
 }
 
 void
-nsXTFElementWrapper::DoneAddingChildren()
+nsXTFElementWrapper::BeginAddingChildren()
+{
+  if (mNotificationMask & nsIXTFElement::NOTIFY_BEGIN_ADDING_CHILDREN)
+    GetXTFElement()->BeginAddingChildren();
+}
+
+void
+nsXTFElementWrapper::DoneAddingChildren(PRBool aHaveNotified)
 {
   if (mNotificationMask & nsIXTFElement::NOTIFY_DONE_ADDING_CHILDREN)
     GetXTFElement()->DoneAddingChildren();
@@ -359,6 +485,49 @@ nsXTFElementWrapper::GetExistingAttrNameFromQName(const nsAString& aStr) const
   return nodeInfo;
 }
 
+PRInt32
+nsXTFElementWrapper::IntrinsicState() const
+{
+  return nsXTFElementWrapperBase::IntrinsicState() | mIntrinsicState;
+}
+
+nsresult
+nsXTFElementWrapper::Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const
+{
+  *aResult = nsnull;
+  nsCOMPtr<nsIContent> it;
+  nsContentUtils::GetXTFService()->CreateElement(getter_AddRefs(it),
+                                                 aNodeInfo);
+  if (!it)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  nsXTFElementWrapper* wrapper =
+    NS_STATIC_CAST(nsXTFElementWrapper*,
+                   NS_STATIC_CAST(nsIContent*, it.get()));
+  nsresult rv = CopyInnerTo(wrapper);
+
+  if (NS_SUCCEEDED(rv)) {
+    if (mAttributeHandler) {
+      PRUint32 innerCount = 0;
+      mAttributeHandler->GetAttributeCount(&innerCount);
+      for (PRUint32 i = 0; i < innerCount; ++i) {
+        nsCOMPtr<nsIAtom> attrName;
+        mAttributeHandler->GetAttributeNameAt(i, getter_AddRefs(attrName));
+        if (attrName) {
+          nsAutoString value;
+          if (NS_SUCCEEDED(mAttributeHandler->GetAttribute(attrName, value)))
+            it->SetAttr(kNameSpaceID_None, attrName, value, PR_TRUE);
+        }
+      }
+    }
+    NS_ADDREF(*aResult = it);
+  }
+
+  // XXX CloneState should take |const nIDOMElement*|
+  wrapper->CloneState(NS_CONST_CAST(nsXTFElementWrapper*, this));
+  return rv;
+}
+
 //----------------------------------------------------------------------
 // nsIDOMElement methods:
 
@@ -438,10 +607,14 @@ nsXTFElementWrapper::GetInterfaces(PRUint32 *count, nsIID * **array)
 
 /* nsISupports getHelperForLanguage (in PRUint32 language); */
 NS_IMETHODIMP 
-nsXTFElementWrapper::GetHelperForLanguage(PRUint32 language, nsISupports **_retval)
+nsXTFElementWrapper::GetHelperForLanguage(PRUint32 language,
+                                          nsISupports** aHelper)
 {
-  *_retval = nsnull;
-  return NS_OK;
+  *aHelper = nsnull;
+  nsCOMPtr<nsIClassInfo> ci = 
+    do_QueryInterface(nsContentUtils::GetClassInfoInstance(eDOMClassInfo_Element_id));
+  return
+    ci ? ci->GetHelperForLanguage(language, aHelper) : NS_ERROR_NOT_AVAILABLE;
 }
 
 /* readonly attribute string contractID; */
@@ -566,46 +739,89 @@ nsXTFElementWrapper::HandledByInner(nsIAtom *attr) const
 }
 
 nsresult
-nsXTFElementWrapper::HandleDOMEvent(nsPresContext* aPresContext,
-                                    nsEvent* aEvent,
-                                    nsIDOMEvent** aDOMEvent,
-                                    PRUint32 aFlags,
-                                    nsEventStatus* aEventStatus)
+nsXTFElementWrapper::PostHandleEvent(nsEventChainPostVisitor& aVisitor)
 {
-  nsresult rv = nsXTFElementWrapperBase::HandleDOMEvent(aPresContext, aEvent,
-                                                        aDOMEvent, aFlags,
-                                                        aEventStatus);
-
-  if (NS_FAILED(rv) ||
-      (nsEventStatus_eIgnore != *aEventStatus) ||
-      !(mNotificationMask & nsIXTFElement::NOTIFY_HANDLE_DEFAULT) ||
-      (aFlags & (NS_EVENT_FLAG_SYSTEM_EVENT | NS_EVENT_FLAG_CAPTURE)))
+  nsresult rv = NS_OK;
+  if (aVisitor.mEventStatus == nsEventStatus_eConsumeNoDefault) {
     return rv;
+  }
 
-  nsIDOMEvent* domEvent = nsnull;
-  if (!aDOMEvent)
-    aDOMEvent = &domEvent;
-
-  if (!*aDOMEvent) {
+  if (!aVisitor.mDOMEvent) {
     // We haven't made a DOMEvent yet.  Force making one now.
-    nsCOMPtr<nsIEventListenerManager> listenerManager;
-    if (NS_FAILED(rv = GetListenerManager(getter_AddRefs(listenerManager))))
-      return rv;
-
-    nsAutoString empty;
-    if (NS_FAILED(rv = listenerManager->CreateEvent(aPresContext, aEvent,
-                                                    empty, aDOMEvent)))
+    if (NS_FAILED(rv = nsEventDispatcher::CreateEvent(aVisitor.mPresContext,
+                                                      aVisitor.mEvent,
+                                                      EmptyString(),
+                                                      &aVisitor.mDOMEvent)))
       return rv;
   }
-  if (!*aDOMEvent)
+  if (!aVisitor.mDOMEvent)
     return NS_ERROR_FAILURE;
   
   PRBool defaultHandled = PR_FALSE;
-  nsIXTFElement * xtfElement = GetXTFElement();
+  nsIXTFElement* xtfElement = GetXTFElement();
   if (xtfElement)
-    rv = xtfElement->HandleDefault(*aDOMEvent, &defaultHandled);
+    rv = xtfElement->HandleDefault(aVisitor.mDOMEvent, &defaultHandled);
   if (defaultHandled)
-    *aEventStatus = nsEventStatus_eConsumeNoDefault;
+    aVisitor.mEventStatus = nsEventStatus_eConsumeNoDefault;
   return rv;
 }
 
+NS_IMETHODIMP
+nsXTFElementWrapper::SetIntrinsicState(PRInt32 aNewState)
+{
+  nsIDocument *doc = GetCurrentDoc();
+  PRInt32 bits = mIntrinsicState ^ aNewState;
+  
+  if (!doc || !bits)
+    return NS_OK;
+
+  mIntrinsicState = aNewState;
+  mozAutoDocUpdate upd(doc, UPDATE_CONTENT_STATE, PR_TRUE);
+  doc->ContentStatesChanged(this, nsnull, bits);
+
+  return NS_OK;
+}
+
+// nsXTFStyleableElementWrapper
+
+nsXTFStyledElementWrapper::nsXTFStyledElementWrapper(nsINodeInfo* aNodeInfo)
+: nsXTFElementWrapper(aNodeInfo)
+{
+}
+
+nsIAtom *
+nsXTFStyledElementWrapper::GetClassAttributeName() const
+{
+  return mClassAttributeName;
+}
+
+const nsAttrValue*
+nsXTFStyledElementWrapper::GetClasses() const
+{
+  const nsAttrValue* val = nsnull;
+  nsIAtom* clazzAttr = GetClassAttributeName();
+  if (clazzAttr) {
+    val = mAttrsAndChildren.GetAttr(clazzAttr);
+    // This is possibly the first time we need any classes.
+    if (val && val->Type() == nsAttrValue::eString) {
+      nsAutoString value;
+      val->ToString(value);
+      nsAttrValue newValue;
+      newValue.ParseAtomArray(value);
+      NS_CONST_CAST(nsAttrAndChildArray*, &mAttrsAndChildren)->
+        SetAndTakeAttr(clazzAttr, newValue);
+    }
+  }
+  return val;
+}
+
+nsresult
+nsXTFStyledElementWrapper::SetClassAttributeName(nsIAtom* aName)
+{
+  // The class attribute name can be set only once
+  if (mClassAttributeName || !aName)
+    return NS_ERROR_FAILURE;
+  
+  mClassAttributeName = aName;
+  return NS_OK;
+}
