@@ -24,6 +24,7 @@
  *   Peter Annema <disttsc@bart.nl>
  *   Mike Shaver <shaver@mozilla.org>
  *   Ben Goodger <ben@netscape.com>
+ *   Mark Hammond <mhammond@skippinet.com.au>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -51,27 +52,22 @@
 // XXX because nsIEventListenerManager has broken includes
 #include "nsIDOMEvent.h"
 #include "nsIServiceManager.h"
-#include "nsHTMLValue.h"
 #include "nsIAtom.h"
 #include "nsINodeInfo.h"
 #include "nsIControllers.h"
 #include "nsICSSParser.h"
-#include "nsICSSStyleRule.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIDOM3EventTarget.h"
 #include "nsIDOMXULElement.h"
 #include "nsIDOMXULMultSelectCntrlEl.h"
 #include "nsIEventListenerManager.h"
-#include "nsINameSpace.h"
 #include "nsIRDFCompositeDataSource.h"
 #include "nsIRDFResource.h"
 #include "nsIScriptObjectOwner.h"
-#include "nsIStyledContent.h"
 #include "nsIBindingManager.h"
-#include "nsIXBLBinding.h"
 #include "nsIURI.h"
-#include "nsIXULContent.h"
+#include "nsIXMLContent.h"
 #include "nsIXULPrototypeCache.h"
 #include "nsIXULTemplateBuilder.h"
 #include "nsIBoxObject.h"
@@ -83,6 +79,7 @@
 #include "nsXULAtoms.h"
 #include "nsAutoPtr.h"
 #include "nsGenericElement.h"
+#include "nsDOMScriptObjectHolder.h"
 
 class nsIDocument;
 class nsIRDFService;
@@ -93,6 +90,7 @@ class nsString;
 class nsVoidArray;
 class nsIDocShell;
 class nsDOMAttributeMap;
+class nsICSSStyleRule;
 
 class nsIObjectInputStream;
 class nsIObjectOutputStream;
@@ -114,8 +112,6 @@ static NS_DEFINE_CID(kCSSParserCID, NS_CSSPARSER_CID);
 
  */
 
-MOZ_DECL_CTOR_COUNTER(nsXULPrototypeAttribute)
-
 class nsXULPrototypeAttribute
 {
 public:
@@ -131,8 +127,14 @@ public:
 
     nsAttrName mName;
     nsAttrValue mValue;
+    // mEventHandler is only valid for the language ID specified in the
+    // containing nsXULPrototypeElement.  We would ideally use
+    // nsScriptObjectHolder, but want to avoid the extra lang ID.
     void* mEventHandler;
 
+    // Containing element must tell us the langID so we can cleanup.
+    void Finalize(PRUint32 aLangID);
+
 #ifdef XUL_PROTOTYPE_ATTRIBUTE_METERING
     /**
       If enough attributes, on average, are event handlers, it pays to keep
@@ -203,10 +205,10 @@ public:
 
     virtual ~nsXULPrototypeNode() {}
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext,
+                               nsIScriptGlobalObject* aGlobal,
                                const nsCOMArray<nsINodeInfo> *aNodeInfos) = 0;
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
-                                 nsIScriptContext* aContext,
+                                 nsIScriptGlobalObject* aGlobal,
                                  nsIURI* aDocumentURI,
                                  const nsCOMArray<nsINodeInfo> *aNodeInfos) = 0;
 
@@ -241,13 +243,17 @@ public:
           mNumChildren(0),
           mChildren(nsnull),
           mNumAttributes(0),
-          mAttributes(nsnull)
+          mAttributes(nsnull),
+          mScriptTypeID(nsIProgrammingLanguage::UNKNOWN)
     {
         NS_LOG_ADDREF(this, 1, ClassName(), ClassSize());
     }
 
     virtual ~nsXULPrototypeElement()
     {
+        PRUint32 i;
+        for (i = 0; i < mNumAttributes; i++)
+            mAttributes[i].Finalize(mScriptTypeID);
         delete[] mAttributes;
         delete[] mChildren;
     }
@@ -261,9 +267,8 @@ public:
     {
       if (mChildren) {
         for (PRInt32 i = mNumChildren-1; i >= 0; i--) {
-          if (! mChildren[i])
-            break;
-          mChildren[i]->ReleaseSubtree();
+          if (mChildren[i])
+            mChildren[i]->ReleaseSubtree();
         }
       }
 
@@ -271,10 +276,10 @@ public:
     }
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext,
+                               nsIScriptGlobalObject* aGlobal,
                                const nsCOMArray<nsINodeInfo> *aNodeInfos);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
-                                 nsIScriptContext* aContext,
+                                 nsIScriptGlobalObject* aGlobal,
                                  nsIURI* aDocumentURI,
                                  const nsCOMArray<nsINodeInfo> *aNodeInfos);
 
@@ -288,6 +293,11 @@ public:
     PRUint32                 mNumAttributes;
     nsXULPrototypeAttribute* mAttributes;         // [OWNER]
 
+    // The language ID can not be set on a per-node basis, but is tracked
+    // so that the language ID from the originating root can be used
+    // (eg, when a node from an overlay ends up in our document, that node
+    // must use its original script language, not our document's default.
+    PRUint32                 mScriptTypeID;
     static void ReleaseGlobals()
     {
         NS_IF_RELEASE(sCSSParser);
@@ -308,14 +318,12 @@ protected:
     static nsICSSParser* sCSSParser;
 };
 
-struct JSRuntime;
-struct JSObject;
 class nsXULDocument;
 
 class nsXULPrototypeScript : public nsXULPrototypeNode
 {
 public:
-    nsXULPrototypeScript(PRUint16 aLineNo, const char *aVersion);
+    nsXULPrototypeScript(PRUint32 aLangID, PRUint32 aLineNo, PRUint32 version);
     virtual ~nsXULPrototypeScript();
 
 #ifdef NS_BUILD_REFCNT_LOGGING
@@ -324,29 +332,29 @@ public:
 #endif
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext,
+                               nsIScriptGlobalObject* aGlobal,
                                const nsCOMArray<nsINodeInfo> *aNodeInfos);
     nsresult SerializeOutOfLine(nsIObjectOutputStream* aStream,
-                                nsIScriptContext* aContext);
+                                nsIScriptGlobalObject* aGlobal);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
-                                 nsIScriptContext* aContext,
+                                 nsIScriptGlobalObject* aGlobal,
                                  nsIURI* aDocumentURI,
                                  const nsCOMArray<nsINodeInfo> *aNodeInfos);
     nsresult DeserializeOutOfLine(nsIObjectInputStream* aInput,
-                                  nsIScriptContext* aContext);
+                                  nsIScriptGlobalObject* aGlobal);
 
     nsresult Compile(const PRUnichar* aText, PRInt32 aTextLength,
-                     nsIURI* aURI, PRUint16 aLineNo,
+                     nsIURI* aURI, PRUint32 aLineNo,
                      nsIDocument* aDocument,
                      nsIXULPrototypeDocument* aPrototypeDocument);
 
     nsCOMPtr<nsIURI>         mSrcURI;
-    PRUint16                 mLineNo;
+    PRUint32                 mLineNo;
     PRPackedBool             mSrcLoading;
     PRPackedBool             mOutOfLine;
     nsXULDocument*           mSrcLoadWaiters;   // [OWNER] but not COMPtr
-    JSObject*                mJSObject;
-    const char*              mLangVersion;
+    PRUint32                 mLangVersion;
+    nsScriptObjectHolder     mScriptObject;
 
     static void ReleaseGlobals()
     {
@@ -383,10 +391,10 @@ public:
 #endif
 
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext,
+                               nsIScriptGlobalObject* aGlobal,
                                const nsCOMArray<nsINodeInfo> *aNodeInfos);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
-                                 nsIScriptContext* aContext,
+                                 nsIScriptGlobalObject* aGlobal,
                                  nsIURI* aDocumentURI,
                                  const nsCOMArray<nsINodeInfo> *aNodeInfos);
 
@@ -401,11 +409,44 @@ public:
 
  */
 
-class nsXULElement : public nsIXULContent,
+#define XUL_ELEMENT_LAZY_STATE_OFFSET NODE_TYPE_SPECIFIC_BITS_OFFSET
+
+class nsXULElement : public nsGenericElement,
                      public nsIDOMXULElement,
                      public nsIScriptEventHandlerOwner,
                      public nsIChromeEventHandler
 {
+public:
+    /**
+     * These flags are used to maintain bookkeeping information for partially-
+     * constructed content.
+     *
+     *   eChildrenMustBeRebuilt
+     *     The element's children are invalid or unconstructed, and should
+     *     be reconstructed.
+     *
+     *   eTemplateContentsBuilt
+     *     Child content that is built from a XUL template has been
+     *     constructed. 
+     *
+     *   eContainerContentsBuilt
+     *     Child content that is built by following the ``containment''
+     *     property in a XUL template has been built.
+     */
+    enum LazyState {
+        eChildrenMustBeRebuilt  = 0x1,
+        eTemplateContentsBuilt  = 0x2,
+        eContainerContentsBuilt = 0x4
+    };
+
+    /** Typesafe, non-refcounting cast from nsIContent.  Cheaper than QI. **/
+    static nsXULElement* FromContent(nsIContent *aContent)
+    {
+        if (aContent->IsNodeOfType(eXUL))
+            return NS_STATIC_CAST(nsXULElement*, aContent);
+        return nsnull;
+    }
+
 public:
     static nsIXBLService* GetXBLService() {
         if (!gXBLService)
@@ -418,9 +459,7 @@ public:
     }
 
 protected:
-    static nsrefcnt             gRefCnt;
     // pseudo-constants
-    static nsIRDFService*       gRDFService;
     static nsIXBLService*       gXBLService;
     static nsICSSOMFactory*     gCSSOMFactory;
 
@@ -430,69 +469,50 @@ public:
            PRBool aIsScriptable, nsIContent** aResult);
 
     // nsISupports
-    NS_DECL_ISUPPORTS
+    NS_DECL_ISUPPORTS_INHERITED
 
-    // nsIContent (from nsIStyledContent)
-    nsIDocument *GetDocument() const
-    {
-        return mDocument;
-    }
-    virtual void SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                             PRBool aCompileEventHandlers);
-    PRBool IsInDoc() const
-    {
-        return !!mDocument;
-    }
-    nsIDocument *GetOwnerDoc() const
-    {
-        return mDocument ? mDocument : NodeInfo()->GetDocument();
-    }
-    virtual void SetParent(nsIContent* aParent);
-    virtual PRBool IsNativeAnonymous() const;
-    virtual void SetNativeAnonymous(PRBool aAnonymous);
+    // nsINode
     virtual PRUint32 GetChildCount() const;
     virtual nsIContent *GetChildAt(PRUint32 aIndex) const;
-    virtual PRInt32 IndexOf(nsIContent* aPossibleChild) const;
+    virtual PRInt32 IndexOf(nsINode* aPossibleChild) const;
+    virtual nsresult PreHandleEvent(nsEventChainPreVisitor& aVisitor);
     virtual nsresult InsertChildAt(nsIContent* aKid, PRUint32 aIndex,
-                                   PRBool aNotify, PRBool aDeepSetDocument);
-    virtual nsresult AppendChildTo(nsIContent* aKid, PRBool aNotify,
-                                   PRBool aDeepSetDocument);
+                                   PRBool aNotify);
+
+    // nsIContent
+    virtual nsresult BindToTree(nsIDocument* aDocument, nsIContent* aParent,
+                                nsIContent* aBindingParent,
+                                PRBool aCompileEventHandlers);
+    virtual void UnbindFromTree(PRBool aDeep = PR_TRUE,
+                                PRBool aNullParent = PR_TRUE);
+    virtual void SetNativeAnonymous(PRBool aAnonymous);
     virtual nsresult RemoveChildAt(PRUint32 aIndex, PRBool aNotify);
-    virtual void GetNameSpaceID(PRInt32* aNameSpeceID) const;
-    virtual nsIAtom *Tag() const;
-    virtual nsINodeInfo *GetNodeInfo() const;
     virtual nsIAtom *GetIDAttributeName() const;
     virtual nsIAtom *GetClassAttributeName() const;
-    virtual already_AddRefed<nsINodeInfo> GetExistingAttrNameFromQName(const nsAString& aStr) const;
-    nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     const nsAString& aValue, PRBool aNotify)
-    {
-      return SetAttr(aNameSpaceID, aName, nsnull, aValue, aNotify);
-    }
-    virtual nsresult SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsIAtom* aPrefix,
-                             const nsAString& aValue, PRBool aNotify);
-    virtual nsresult GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                             nsAString& aResult) const;
+    virtual PRBool GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+                           nsAString& aResult) const;
     virtual PRBool HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
+    virtual PRBool AttrValueIs(PRInt32 aNameSpaceID, nsIAtom* aName,
+                               const nsAString& aValue,
+                               nsCaseTreatment aCaseSensitive) const;
+    virtual PRBool AttrValueIs(PRInt32 aNameSpaceID, nsIAtom* aName,
+                               nsIAtom* aValue,
+                               nsCaseTreatment aCaseSensitive) const;
+    virtual PRInt32 FindAttrValueIn(PRInt32 aNameSpaceID,
+                                    nsIAtom* aName,
+                                    AttrValuesArray* aValues,
+                                    nsCaseTreatment aCaseSensitive) const;
     virtual nsresult UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                                PRBool aNotify);
-    virtual nsresult GetAttrNameAt(PRUint32 aIndex, PRInt32* aNameSpaceID,
-                                   nsIAtom** aName, nsIAtom** aPrefix) const;
+    virtual const nsAttrName* GetAttrNameAt(PRUint32 aIndex) const;
     virtual PRUint32 GetAttrCount() const;
+
 #ifdef DEBUG
     virtual void List(FILE* out, PRInt32 aIndent) const;
     virtual void DumpContent(FILE* out, PRInt32 aIndent,PRBool aDumpAll) const
     {
     }
 #endif
-    virtual nsresult HandleDOMEvent(nsPresContext* aPresContext,
-                              nsEvent* aEvent,
-                              nsIDOMEvent** aDOMEvent,
-                              PRUint32 aFlags,
-                              nsEventStatus* aEventStatus);
-
-    virtual PRUint32 ContentID() const;
-    virtual void SetContentID(PRUint32 aID);
 
     virtual nsresult RangeAdd(nsIDOMRange* aRange);
     virtual void RangeRemove(nsIDOMRange* aRange);
@@ -501,28 +521,14 @@ public:
     virtual void RemoveFocus(nsPresContext* aPresContext);
 
     virtual nsIContent *GetBindingParent() const;
-    virtual nsresult SetBindingParent(nsIContent* aParent);
-    virtual PRBool IsContentOfType(PRUint32 aFlags) const;
-    virtual already_AddRefed<nsIURI> GetBaseURI() const;
-    virtual nsresult GetListenerManager(nsIEventListenerManager** aResult);
+    virtual PRBool IsNodeOfType(PRUint32 aFlags) const;
     virtual PRBool IsFocusable(PRInt32 *aTabIndex = nsnull);
-    virtual void* GetProperty(nsIAtom  *aPropertyName,
-                              nsresult *aStatus = nsnull) const;
-    virtual nsresult SetProperty(nsIAtom            *aPropertyName,
-                                 void               *aValue,
-                                 NSPropertyDtorFunc  aDtor);
-    virtual nsresult DeleteProperty(nsIAtom *aPropertyName);
-    virtual void* UnsetProperty(nsIAtom  *aPropertyName,
-                                nsresult *aStatus = nsnull);
+    virtual nsIAtom* GetID() const;
+    virtual const nsAttrValue* GetClasses() const;
 
     // nsIXMLContent
     NS_IMETHOD MaybeTriggerAutoLink(nsIDocShell *aShell);
 
-    // nsIStyledContent
-    NS_IMETHOD GetID(nsIAtom** aResult) const;
-    virtual const nsAttrValue* GetClasses() const;
-    NS_IMETHOD_(PRBool) HasClass(nsIAtom* aClass, PRBool aCaseSensitive) const;
-
     NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
     virtual nsICSSStyleRule* GetInlineStyleRule();
     NS_IMETHOD SetInlineStyleRule(nsICSSStyleRule* aStyleRule, PRBool aNotify);
@@ -530,42 +536,47 @@ public:
                                                 PRInt32 aModType) const;
     NS_IMETHOD_(PRBool) IsAttributeMapped(const nsIAtom* aAttribute) const;
 
-    // nsIXULContent
-    NS_IMETHOD_(PRUint32) PeekChildCount() const;
-    NS_IMETHOD SetLazyState(LazyState aFlags);
-    NS_IMETHOD ClearLazyState(LazyState aFlags);
-    NS_IMETHOD GetLazyState(LazyState aFlag, PRBool& aValue);
-    NS_IMETHOD AddScriptEventListener(nsIAtom* aName, const nsAString& aValue);
+    // XUL element methods
+    PRUint32 PeekChildCount() const
+    { return mAttrsAndChildren.ChildCount(); }
+    void SetLazyState(LazyState aFlags)
+    { SetFlags(aFlags << XUL_ELEMENT_LAZY_STATE_OFFSET); }
+    void ClearLazyState(LazyState aFlags)
+    { UnsetFlags(aFlags << XUL_ELEMENT_LAZY_STATE_OFFSET); }
+    PRBool GetLazyState(LazyState aFlag)
+    { return GetFlags() & (aFlag << XUL_ELEMENT_LAZY_STATE_OFFSET); }
 
-    // nsIDOMNode (from nsIDOMElement)
-    NS_DECL_NSIDOMNODE
+    // nsIDOMNode
+    NS_FORWARD_NSIDOMNODE(nsGenericElement::)
 
     // nsIDOMElement
-    NS_DECL_NSIDOMELEMENT
+    NS_FORWARD_NSIDOMELEMENT(nsGenericElement::)
 
     // nsIDOMXULElement
     NS_DECL_NSIDOMXULELEMENT
 
     // nsIScriptEventHandlerOwner
     nsresult CompileEventHandler(nsIScriptContext* aContext,
-                                 void* aTarget,
+                                 nsISupports* aTarget,
                                  nsIAtom *aName,
                                  const nsAString& aBody,
                                  const char* aURL,
                                  PRUint32 aLineNo,
-                                 void** aHandler);
-    nsresult GetCompiledEventHandler(nsIAtom *aName, void** aHandler);
+                                 nsScriptObjectHolder &aHandler);
+    nsresult GetCompiledEventHandler(nsIAtom *aName,
+                                     nsScriptObjectHolder &aHandler);
 
     // nsIChromeEventHandler
     NS_DECL_NSICHROMEEVENTHANDLER
 
+    virtual nsresult Clone(nsINodeInfo *aNodeInfo, nsINode **aResult) const;
+
+    nsresult GetStyle(nsIDOMCSSStyleDeclaration** aStyle);
 
 protected:
-    nsXULElement();
-    nsresult Init();
+    nsXULElement(nsINodeInfo* aNodeInfo);
     virtual ~nsXULElement(void);
 
-
     // Implementation methods
     nsresult EnsureContentsGenerated(void) const;
 
@@ -574,26 +585,22 @@ protected:
     static nsresult
     ExecuteJSCode(nsIDOMElement* anElement, nsEvent* aEvent);
 
-    static PRBool IsAncestor(nsIDOMNode* aParentNode, nsIDOMNode* aChildNode);
-
     // Helper routine that crawls a parent chain looking for a tree element.
     NS_IMETHOD GetParentTree(nsIDOMXULMultiSelectControlElement** aTreeElement);
 
     nsresult AddPopupListener(nsIAtom* aName);
 
-    nsIContent* GetParent() const {
-        // Override nsIContent::GetParent to be more efficient internally,
-        // we don't use the low 2 bits of mParentPtrBits for anything.
- 
-        return NS_REINTERPRET_CAST(nsIContent *, mParentPtrBits);
-    }
+    class nsXULSlots : public nsGenericElement::nsDOMSlots
+    {
+    public:
+       nsXULSlots(PtrBits aFlags);
+       virtual ~nsXULSlots();
+    };
+
+    virtual nsINode::nsSlots* CreateSlots();
 
-protected:
     // Required fields
-    nsXULPrototypeElement*              mPrototype;
-    nsIDocument*                        mDocument;
-    nsAttrAndChildArray                 mAttrsAndChildren;   // [OWNER]
-    nsCOMPtr<nsIEventListenerManager>   mListenerManager;    // [OWNER]
+    nsRefPtr<nsXULPrototypeElement>     mPrototype;
 
     /**
      * The nearest enclosing content node with a binding
@@ -601,39 +608,37 @@ protected:
      */
     nsIContent*                         mBindingParent;
 
-    /**
-     * Lazily instantiated if/when object is mutated. mAttributes are
-     * lazily copied from the prototype when changed.
-     */
-    struct Slots {
-        Slots();
-        ~Slots();
-
-        nsCOMPtr<nsINodeInfo>               mNodeInfo;           // [OWNER]
-        nsCOMPtr<nsIControllers>            mControllers;        // [OWNER]
-        nsRefPtr<nsDOMCSSDeclaration>       mDOMStyle;           // [OWNER]
-        nsRefPtr<nsDOMAttributeMap>         mAttributeMap;       // [OWNER]
-        nsRefPtr<nsChildContentList>        mChildNodes;         // [OWNER]
-        unsigned                            mLazyState : 3;
-        unsigned                            mHasProperties : 1;
-    };
-
-    friend struct Slots;
-    Slots* mSlots;
-
-    /**
-     * Ensure that we've got an mSlots object, creating a Slots object
-     * if necessary.
-     */
-    nsresult EnsureSlots();
-
     /**
      * Abandon our prototype linkage, and copy all attributes locally
      */
     nsresult MakeHeavyweight();
 
+    /**
+     * Get the attr info for the given namespace ID and attribute name.
+     * The namespace ID must not be kNameSpaceID_Unknown and the name
+     * must not be null.
+     */
+    virtual nsAttrInfo GetAttrInfo(PRInt32 aNamespaceID, nsIAtom* aName) const;
+
     const nsAttrValue* FindLocalOrProtoAttr(PRInt32 aNameSpaceID,
-                                            nsIAtom *aName) const;
+                                            nsIAtom *aName) const {
+        return nsXULElement::GetAttrInfo(aNameSpaceID, aName).mValue;
+    }
+
+    virtual nsresult BeforeSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
+                                   const nsAString* aValue, PRBool aNotify);
+    virtual nsresult AfterSetAttr(PRInt32 aNamespaceID, nsIAtom* aName,
+                                  const nsAString* aValue, PRBool aNotify);
+
+    virtual PRBool ParseAttribute(PRInt32 aNamespaceID,
+                                  nsIAtom* aAttribute,
+                                  const nsAString& aValue,
+                                  nsAttrValue& aResult);
+
+    virtual nsresult
+      GetEventListenerManagerForAttr(nsIEventListenerManager** aManager,
+                                     nsISupports** aTarget,
+                                     PRBool* aDefer);
   
     /**
      * Return our prototype's attribute, if one exists.
@@ -650,29 +655,25 @@ protected:
 
     nsresult HideWindowChrome(PRBool aShouldHide);
 
-    
-    nsresult SetAttrAndNotify(PRInt32 aNamespaceID,
-                              nsIAtom* aAttribute,
-                              nsIAtom* aPrefix,
-                              const nsAString& aOldValue,
-                              nsAttrValue& aParsedValue,
-                              PRBool aModification,
-                              PRBool aFireMutation,
-                              PRBool aNotify);
-
     const nsAttrName* InternalGetExistingAttrNameFromQName(const nsAString& aStr) const;
 
 protected:
-    // Internal accessors. These shadow the 'Slots', and return
-    // appropriate default values if there are no slots defined in the
-    // delegate.
-    nsINodeInfo     *NodeInfo() const    { return mSlots ? mSlots->mNodeInfo          : mPrototype->mNodeInfo; }
-    nsIControllers  *Controllers() const { return mSlots ? mSlots->mControllers.get() : nsnull; }
+    // Internal accessor. This shadows the 'Slots', and returns
+    // appropriate value.
+    nsIControllers *Controllers() {
+      nsDOMSlots* slots = GetExistingDOMSlots();
+      return slots ? slots->mControllers : nsnull; 
+    }
 
     void UnregisterAccessKey(const nsAString& aOldValue);
+    PRBool BoolAttrIsTrue(nsIAtom* aName);
 
     friend nsresult
     NS_NewXULElement(nsIContent** aResult, nsINodeInfo *aNodeInfo);
+
+    static already_AddRefed<nsXULElement>
+    Create(nsXULPrototypeElement* aPrototype, nsINodeInfo *aNodeInfo,
+           PRBool aIsScriptable);
 };
 
 
