@@ -57,7 +57,6 @@
 #include "nsContentCID.h"
 #include "nsContentUtils.h"
 
-#include "nsIScriptGlobalObject.h"
 #include "nsIScriptContext.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIDOMXULDocument.h"
@@ -67,7 +66,7 @@
 #include "nsIDOMNSUIEvent.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMNSEvent.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 #include "nsIPrincipal.h"
 #include "nsIScriptSecurityManager.h"
 
@@ -78,8 +77,11 @@
 #include "nsPresContext.h"
 #include "nsIPresShell.h"
 #include "nsIEventStateManager.h"
+#include "nsIFocusController.h"
+#include "nsPIDOMWindow.h"
 
 #include "nsIFrame.h"
+#include "nsIMenuFrame.h"
 
 // on win32 and os/2, context menus come up on mouse up. On other platforms,
 // they appear on mouse down. Certain bits of code care about this difference.
@@ -126,7 +128,6 @@ protected:
 
     virtual nsresult LaunchPopup(nsIDOMEvent* anEvent);
     virtual nsresult LaunchPopup(PRInt32 aClientX, PRInt32 aClientY) ;
-    virtual void ClosePopup();
 
 private:
 
@@ -137,7 +138,7 @@ private:
     nsIDOMElement* mElement;               // Weak ref. The element will go away first.
 
     // The popup that is getting shown on top of mElement.
-    nsIDOMElement* mPopupContent; 
+    nsCOMPtr<nsIPopupBoxObject> mPopup;
 
     // The type of the popup
     XULPopupType popupType;
@@ -147,13 +148,15 @@ private:
 ////////////////////////////////////////////////////////////////////////
       
 XULPopupListenerImpl::XULPopupListenerImpl(void)
-  : mElement(nsnull), mPopupContent(nsnull)
+  : mElement(nsnull)
 {
 }
 
 XULPopupListenerImpl::~XULPopupListenerImpl(void)
 {
-  ClosePopup();
+  if (mPopup) {
+    mPopup->HidePopup();
+  }
   
 #ifdef DEBUG_REFS
     --gInstanceCount;
@@ -237,15 +240,12 @@ XULPopupListenerImpl::PreLaunchPopup(nsIDOMEvent* aMouseEvent)
       // The user wants his contextmenus.  Let's make sure that this is a website
       // and not chrome since there could be places in chrome which don't want
       // contextmenus.
-      nsCOMPtr<nsIDocument> doc;
-      nsCOMPtr<nsIPrincipal> prin;
-      nsContentUtils::GetDocumentAndPrincipal(targetNode,
-                                              getter_AddRefs(doc),
-                                              getter_AddRefs(prin));
-      if (prin) {
+      nsCOMPtr<nsINode> node = do_QueryInterface(targetNode);
+      if (node) {
         nsCOMPtr<nsIPrincipal> system;
-        nsContentUtils::GetSecurityManager()->GetSystemPrincipal(getter_AddRefs(system));
-        if (prin != system) {
+        nsContentUtils::GetSecurityManager()->
+          GetSystemPrincipal(getter_AddRefs(system));
+        if (node->NodePrincipal() != system) {
           // This isn't chrome.  Cancel the preventDefault() and
           // let the event go forth.
           preventDefault = PR_FALSE;
@@ -282,7 +282,9 @@ XULPopupListenerImpl::PreLaunchPopup(nsIDOMEvent* aMouseEvent)
   }
 
   // Store clicked-on node in xul document for context menus and menu popups.
+  // CLEAR THE POPUP EVENT BEFORE THIS FUNCTION EXITS
   xulDocument->SetPopupNode( targetNode );
+  xulDocument->SetTrustedPopupEvent( aMouseEvent );
 
   nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(aMouseEvent));
 
@@ -293,31 +295,24 @@ XULPopupListenerImpl::PreLaunchPopup(nsIDOMEvent* aMouseEvent)
       if (button == 0) {
         // Time to launch a popup menu.
         LaunchPopup(aMouseEvent);
-
-        if (nsevent) {
-            nsevent->PreventBubble();
-        }
-
+        aMouseEvent->StopPropagation();
         aMouseEvent->PreventDefault();
       }
       break;
     case eXULPopupType_context:
 
-    // Time to launch a context menu
+      // Time to launch a context menu
 #ifndef NS_CONTEXT_MENU_IS_MOUSEUP
-    // If the context menu launches on mousedown,
-    // we have to fire focus on the content we clicked on
-    FireFocusOnTargetContent(targetNode);
+      // If the context menu launches on mousedown,
+      // we have to fire focus on the content we clicked on
+      FireFocusOnTargetContent(targetNode);
 #endif
-    LaunchPopup(aMouseEvent);
-
-    if (nsevent) {
-        nsevent->PreventBubble();
-    }
-
-    aMouseEvent->PreventDefault();
-    break;
+      LaunchPopup(aMouseEvent);
+      aMouseEvent->StopPropagation();
+      aMouseEvent->PreventDefault();
+      break;
   }
+  xulDocument->SetTrustedPopupEvent(nsnull);
   return NS_OK;
 }
 
@@ -329,10 +324,10 @@ XULPopupListenerImpl::FireFocusOnTargetContent(nsIDOMNode* aTargetNode)
   rv = aTargetNode->GetOwnerDocument(getter_AddRefs(domDoc));
   if(NS_SUCCEEDED(rv) && domDoc)
   {
-    nsCOMPtr<nsIDocument> tempdoc = do_QueryInterface(domDoc);
+    nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
 
     // Get nsIDOMElement for targetNode
-    nsIPresShell *shell = tempdoc->GetShellAt(0);
+    nsIPresShell *shell = doc->GetShellAt(0);
     if (!shell)
       return NS_ERROR_FAILURE;
 
@@ -340,8 +335,7 @@ XULPopupListenerImpl::FireFocusOnTargetContent(nsIDOMNode* aTargetNode)
     nsCOMPtr<nsPresContext> context = shell->GetPresContext();
  
     nsCOMPtr<nsIContent> content = do_QueryInterface(aTargetNode);
-    nsIFrame* targetFrame;
-    shell->GetPrimaryFrameFor(content, &targetFrame);
+    nsIFrame* targetFrame = shell->GetPrimaryFrameFor(content);
     if (!targetFrame) return NS_ERROR_FAILURE;
       
     PRBool suppressBlur = PR_FALSE;
@@ -354,10 +348,8 @@ XULPopupListenerImpl::FireFocusOnTargetContent(nsIDOMNode* aTargetNode)
     nsIFrame* currFrame = targetFrame;
     // Look for the nearest enclosing focusable frame.
     while (currFrame) {
-        const nsStyleUserInterface* ui = currFrame->GetStyleUserInterface();
-        if ((ui->mUserFocus != NS_STYLE_USER_FOCUS_IGNORE) &&
-            (ui->mUserFocus != NS_STYLE_USER_FOCUS_NONE)) 
-        {
+        PRInt32 tabIndexUnused;
+        if (currFrame->IsFocusable(&tabIndexUnused, PR_TRUE)) {
           newFocus = currFrame->GetContent();
           nsCOMPtr<nsIDOMElement> domElement(do_QueryInterface(newFocus));
           if (domElement) {
@@ -370,9 +362,26 @@ XULPopupListenerImpl::FireFocusOnTargetContent(nsIDOMNode* aTargetNode)
     nsCOMPtr<nsIContent> focusableContent = do_QueryInterface(element);
     nsIEventStateManager *esm = context->EventStateManager();
 
-    if (focusableContent)
+    if (focusableContent) {
+      // Lock to scroll by SetFocus. See bug 309075.
+      nsCOMPtr<nsIFocusController> focusController = nsnull;
+      PRBool isAlreadySuppressed = PR_FALSE;
+      nsPIDOMWindow *ourWindow = doc->GetWindow();
+      if (ourWindow) {
+        focusController = ourWindow->GetRootFocusController();
+        if (focusController) {
+          focusController->GetSuppressFocusScroll(&isAlreadySuppressed);
+          if (!isAlreadySuppressed)
+            focusController->SetSuppressFocusScroll(PR_TRUE);
+        }
+      }
+
       focusableContent->SetFocus(context);
-    else if (!suppressBlur)
+
+      // Unlock scroll if it's needed.
+      if (focusController && !isAlreadySuppressed)
+        focusController->SetSuppressFocusScroll(PR_FALSE);
+    } else if (!suppressBlur)
       esm->SetContentState(nsnull, NS_EVENT_STATE_FOCUS);
 
     esm->SetContentState(focusableContent, NS_EVENT_STATE_ACTIVE);
@@ -380,30 +389,6 @@ XULPopupListenerImpl::FireFocusOnTargetContent(nsIDOMNode* aTargetNode)
   return rv;
 }
 
-//
-// ClosePopup
-//
-// Do everything needed to shut down the popup.
-//
-// NOTE: This routine is safe to call even if the popup is already closed.
-//
-void
-XULPopupListenerImpl :: ClosePopup ( )
-{
-  if ( mPopupContent ) {
-    nsCOMPtr<nsIDOMXULElement> popupElement(do_QueryInterface(mPopupContent));
-    nsCOMPtr<nsIBoxObject> boxObject;
-    if (popupElement)
-      popupElement->GetBoxObject(getter_AddRefs(boxObject));
-    nsCOMPtr<nsIPopupBoxObject> popupObject(do_QueryInterface(boxObject));
-    if (popupObject)
-      popupObject->HidePopup();
-
-    mPopupContent = nsnull;  // release the popup
-  }
-
-} // ClosePopup
-
 //
 // LaunchPopup
 //
@@ -560,9 +545,8 @@ XULPopupListenerImpl::LaunchPopup(PRInt32 aClientX, PRInt32 aClientY)
           list->Item(ctr, getter_AddRefs(node));
           nsCOMPtr<nsIContent> childContent(do_QueryInterface(node));
 
-          nsINodeInfo *ni = childContent->GetNodeInfo();
-
-          if (ni && ni->Equals(nsXULAtoms::menupopup, kNameSpaceID_XUL)) {
+          if (childContent->NodeInfo()->Equals(nsXULAtoms::menupopup,
+                                               kNameSpaceID_XUL)) {
             popupContent = do_QueryInterface(childContent);
             break;
           }
@@ -580,34 +564,48 @@ XULPopupListenerImpl::LaunchPopup(PRInt32 aClientX, PRInt32 aClientY)
   if ( !popupContent )
     return NS_OK;
 
+  // Submenus can't be used as context menus or popups, bug 288763.
+  // Similar code also in nsXULTooltipListener::GetTooltipFor.
+  nsCOMPtr<nsIContent> popup = do_QueryInterface(popupContent);
+  nsIContent* parent = popup->GetParent();
+  if (parent) {
+    nsIDocument* doc = parent->GetCurrentDoc();
+    nsIPresShell* presShell = doc ? doc->GetShellAt(0) : nsnull;
+    nsIFrame* frame = presShell ? presShell->GetPrimaryFrameFor(parent) : nsnull;
+    if (frame) {
+      nsIMenuFrame* menu = nsnull;
+      CallQueryInterface(frame, &menu);
+      NS_ENSURE_FALSE(menu, NS_OK);
+    }
+  }
+
   // We have some popup content. Obtain our window.
-  nsCOMPtr<nsIDOMWindowInternal> domWindow =
-    do_QueryInterface(document->GetScriptGlobalObject());
+  nsPIDOMWindow *domWindow = document->GetWindow();
 
   if (domWindow) {
     // Find out if we're anchored.
-    mPopupContent = popupContent.get();
-
     nsAutoString anchorAlignment;
-    mPopupContent->GetAttribute(NS_LITERAL_STRING("popupanchor"), anchorAlignment);
+    popupContent->GetAttribute(NS_LITERAL_STRING("popupanchor"), anchorAlignment);
 
     nsAutoString popupAlignment;
-    mPopupContent->GetAttribute(NS_LITERAL_STRING("popupalign"), popupAlignment);
+    popupContent->GetAttribute(NS_LITERAL_STRING("popupalign"), popupAlignment);
 
     PRInt32 xPos = aClientX, yPos = aClientY;
 
-    ConvertPosition(mPopupContent, anchorAlignment, popupAlignment, yPos);
+    ConvertPosition(popupContent, anchorAlignment, popupAlignment, yPos);
     if (!anchorAlignment.IsEmpty() && !popupAlignment.IsEmpty())
       xPos = yPos = -1;
 
     nsCOMPtr<nsIBoxObject> popupBox;
-    nsCOMPtr<nsIDOMXULElement> xulPopupElt(do_QueryInterface(mPopupContent));
+    nsCOMPtr<nsIDOMXULElement> xulPopupElt(do_QueryInterface(popupContent));
     xulPopupElt->GetBoxObject(getter_AddRefs(popupBox));
     nsCOMPtr<nsIPopupBoxObject> popupBoxObject(do_QueryInterface(popupBox));
-    if (popupBoxObject)
-      popupBoxObject->ShowPopup(mElement, mPopupContent, xPos, yPos, 
+    if (popupBoxObject) {
+      mPopup = popupBoxObject;
+      popupBoxObject->ShowPopup(mElement, popupContent, xPos, yPos, 
                                 type.get(), anchorAlignment.get(), 
                                 popupAlignment.get());
+    }
   }
 
   return NS_OK;
