@@ -69,7 +69,9 @@ class nsIXULPrototypeScript;
 #include "nsIObjectOutputStream.h"
 #include "nsXULElement.h"
 #endif
-
+#include "nsURIHashKey.h"
+#include "nsInterfaceHashtable.h"
+ 
 struct JSObject;
 struct PRLogModuleInfo;
 
@@ -103,33 +105,25 @@ public:
                                        PRBool aReset = PR_TRUE,
                                        nsIContentSink* aSink = nsnull);
 
-    virtual nsIPrincipal* GetPrincipal();
-
-    virtual void SetPrincipal(nsIPrincipal *aPrincipal);
-
     virtual void SetContentType(const nsAString& aContentType);
 
     virtual void EndLoad();
 
-    virtual void ContentAppended(nsIContent* aContainer,
+    // nsIMutationObserver interface
+    virtual void ContentAppended(nsIDocument* aDocument,
+                                 nsIContent* aContainer,
                                  PRInt32 aNewIndexInContainer);
-
-    virtual void ContentInserted(nsIContent* aContainer,
+    virtual void ContentInserted(nsIDocument* aDocument,
+                                 nsIContent* aContainer,
                                  nsIContent* aChild,
                                  PRInt32 aIndexInContainer);
-
-    virtual void ContentRemoved(nsIContent* aContainer,
+    virtual void ContentRemoved(nsIDocument* aDocument,
+                                nsIContent* aContainer,
                                 nsIContent* aChild,
                                 PRInt32 aIndexInContainer);
-
-    virtual void AttributeChanged(nsIContent* aElement, PRInt32 aNameSpaceID,
-                                  nsIAtom* aAttribute, PRInt32 aModType);
-
-    virtual nsresult HandleDOMEvent(nsPresContext* aPresContext,
-                                    nsEvent* aEvent,
-                                    nsIDOMEvent** aDOMEvent,
-                                    PRUint32 aFlags,
-                                    nsEventStatus* aEventStatus);
+    virtual void AttributeChanged(nsIDocument* aDocument, nsIContent* aElement,
+                                  PRInt32 aNameSpaceID, nsIAtom* aAttribute,
+                                  PRInt32 aModType);
 
     // nsIXULDocument interface
     NS_IMETHOD AddElementForID(const nsAString& aID, nsIContent* aElement);
@@ -148,7 +142,7 @@ public:
     NS_IMETHOD GetTemplateBuilderFor(nsIContent* aContent,
                                      nsIXULTemplateBuilder** aResult);
     NS_IMETHOD OnPrototypeLoadDone();
-    NS_IMETHOD OnHide();
+    PRBool OnDocumentParserError();
 
     // nsIDOMNode interface overrides
     NS_IMETHOD CloneNode(PRBool deep, nsIDOMNode **_retval);
@@ -205,6 +199,10 @@ protected:
                            nsIPrincipal* aDocumentPrincipal,
                            nsIParser** aResult);
 
+    nsresult 
+    LoadOverlayInternal(nsIURI* aURI, PRBool aIsDynamic, PRBool* aShouldReturn,
+                        PRBool* aFailureFromContent);
+
     nsresult ApplyPersistentAttributes();
     nsresult ApplyPersistentAttributesToElements(nsIRDFResource* aResource,
                                                  nsISupportsArray* aElements);
@@ -264,6 +262,7 @@ protected:
     PRPackedBool               mIsFastLoad;
     PRPackedBool               mApplyingPersistedAttrs;
     PRPackedBool               mIsWritingFastLoad;
+    PRPackedBool               mDocumentLoaded;
     nsCOMPtr<nsIDOMXULCommandDispatcher>     mCommandDispatcher; // [OWNER] of the focus tracker
 
     // Maintains the template builders that have been attached to
@@ -348,10 +347,17 @@ protected:
      * Execute the precompiled script object scoped by this XUL document's
      * containing window object, and using its associated script context.
      */
-    nsresult ExecuteScript(JSObject* aScriptObject);
+    nsresult ExecuteScript(nsIScriptContext *aContext, void* aScriptObject);
+
+    /**
+     * Helper method for the above that uses aScript to find the appropriate
+     * script context and object.
+     */
+    nsresult ExecuteScript(nsXULPrototypeScript *aScript);
 
     /**
      * Create a delegate content model element from a prototype.
+     * Note that the resulting content node is not bound to any tree
      */
     nsresult CreateElementFromPrototype(nsXULPrototypeElement* aPrototype,
                                         nsIContent** aResult);
@@ -375,12 +381,6 @@ protected:
      */
     nsXULPrototypeScript* mCurrentScriptProto;
 
-    /**
-     * A "dummy" channel that is used as a placeholder to signal document load
-     * completion.
-     */
-    nsCOMPtr<nsIRequest> mPlaceHolderRequest;
-        
     /**
      * Check if a XUL template builder has already been hooked up.
      */
@@ -393,11 +393,6 @@ protected:
     static nsresult
     CreateTemplateBuilder(nsIContent* aElement);
 
-    /**
-     * Do hookup for <xul:observes> tag
-     */
-    nsresult HookupObserver(nsIContent* aElement);
-
     /**
      * Add the current prototype's style sheets to the document.
      */
@@ -441,7 +436,7 @@ protected:
         nsCOMPtr<nsIContent> mOverlay; // [OWNER]
         PRBool mResolved;
 
-        nsresult Merge(nsIContent* aTargetNode, nsIContent* aOverlayNode);
+        nsresult Merge(nsIContent* aTargetNode, nsIContent* aOverlayNode, PRBool aNotify);
 
     public:
         OverlayForwardReference(nsXULDocument* aDocument, nsIContent* aOverlay)
@@ -470,10 +465,19 @@ protected:
 
     friend class TemplateBuilderHookup;
 
-    static
+    // The out params of FindBroadcaster only have values that make sense when
+    // the method returns NS_FINDBROADCASTER_FOUND.  In all other cases, the
+    // values of the out params should not be relied on (though *aListener and
+    // *aBroadcaster do need to be released if non-null, of course).
     nsresult
-    CheckBroadcasterHookup(nsXULDocument* aDocument,
-                           nsIContent* aElement,
+    FindBroadcaster(nsIContent* aElement,
+                    nsIDOMElement** aListener,
+                    nsString& aBroadcasterID,
+                    nsString& aAttribute,
+                    nsIDOMElement** aBroadcaster);
+
+    nsresult
+    CheckBroadcasterHookup(nsIContent* aElement,
                            PRBool* aNeedsHookup,
                            PRBool* aDidResolve);
 
@@ -484,7 +488,7 @@ protected:
 
     static
     nsresult
-    InsertElement(nsIContent* aParent, nsIContent* aChild);
+    InsertElement(nsIContent* aParent, nsIContent* aChild, PRBool aNotify);
 
     static 
     nsresult
@@ -574,6 +578,10 @@ protected:
      */
     PLDHashTable* mBroadcasterMap;
 
+    nsInterfaceHashtable<nsURIHashKey,nsIObserver> mOverlayLoadObservers;
+    nsInterfaceHashtable<nsURIHashKey,nsIObserver> mPendingOverlayLoadNotifications;
+    
+    PRBool mInitialLayoutComplete;
 private:
     // helpers
 
