@@ -23,6 +23,7 @@
  *   Chris Waterson <waterson@netscape.com>
  *   L. David Baron <dbaron@dbaron.org>
  *   Ben Goodger <ben@netscape.com>
+ *   Mark Hammond <mhammond@skippinet.com.au>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -54,8 +55,10 @@
 #include "nsIScriptGlobalObjectOwner.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIScriptSecurityManager.h"
+#include "nsIScriptRuntime.h"
 #include "nsIServiceManager.h"
-#include "nsISupportsArray.h"
+#include "nsISupportsArray.h" // deprecated array interface.
+#include "nsIArray.h"         // blessed array interface.
 #include "nsIURI.h"
 #include "nsIXULDocument.h"
 #include "nsIXULPrototypeDocument.h"
@@ -67,10 +70,10 @@
 #include "nsIScriptError.h"
 #include "nsIDOMScriptObjectFactory.h"
 #include "nsDOMCID.h"
-#include "nsArray.h"
 #include "nsNodeInfoManager.h"
 #include "nsContentUtils.h"
 
+#include "nsDOMJSUtils.h" // for GetScriptContextFromJSContext
 
 static NS_DEFINE_CID(kDOMScriptObjectFactoryCID,
                      NS_DOM_SCRIPT_OBJECT_FACTORY_CID);
@@ -86,24 +89,17 @@ public:
     NS_DECL_ISUPPORTS
 
     // nsIScriptGlobalObject methods
-    virtual void SetContext(nsIScriptContext *aContext);
-    virtual nsIScriptContext *GetContext();
-    virtual nsresult SetNewDocument(nsIDOMDocument *aDocument,
-                                    PRBool aRemoveEventListeners,
-                                    PRBool aClearScope);
-    virtual void SetDocShell(nsIDocShell *aDocShell);
-    virtual nsIDocShell *GetDocShell();
-    virtual void SetOpenerWindow(nsIDOMWindowInternal *aOpener);
     virtual void SetGlobalObjectOwner(nsIScriptGlobalObjectOwner* aOwner);
     virtual nsIScriptGlobalObjectOwner *GetGlobalObjectOwner();
-    virtual nsresult HandleDOMEvent(nsPresContext* aPresContext, 
-                                    nsEvent* aEvent, 
-                                    nsIDOMEvent** aDOMEvent,
-                                    PRUint32 aFlags,
-                                    nsEventStatus* aEventStatus);
-    virtual JSObject *GetGlobalJSObject();
-    virtual void OnFinalize(JSObject *aObject);
+    virtual void OnFinalize(PRUint32 aLangID, void *aGlobal);
     virtual void SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts);
+    virtual nsresult SetNewArguments(nsIArray *aArguments);
+
+    virtual void *GetScriptGlobal(PRUint32 lang);
+    virtual nsresult EnsureScriptEnvironment(PRUint32 aLangID);
+
+    virtual nsIScriptContext *GetScriptContext(PRUint32 lang);
+    virtual nsresult SetScriptContext(PRUint32 language, nsIScriptContext *ctx);
 
     // nsIScriptObjectPrincipal methods
     virtual nsIPrincipal* GetPrincipal();
@@ -111,11 +107,11 @@ public:
 protected:
     virtual ~nsXULPDGlobalObject();
 
-    nsCOMPtr<nsIScriptContext> mScriptContext;
-    JSObject *mJSObject;    // XXX JS language rabies bigotry badness
-
     nsIScriptGlobalObjectOwner* mGlobalObjectOwner; // weak reference
 
+    nsCOMPtr<nsIScriptContext>  mScriptContexts[NS_STID_ARRAY_UBOUND];
+    void *                      mScriptGlobals[NS_STID_ARRAY_UBOUND];
+
     static JSClass gSharedGlobalClass;
 };
 
@@ -205,7 +201,7 @@ nsXULPDGlobalObject_finalize(JSContext *cx, JSObject *obj)
     nsCOMPtr<nsIScriptGlobalObject> sgo(do_QueryInterface(nativeThis));
 
     if (sgo) {
-        sgo->OnFinalize(obj);
+        sgo->OnFinalize(nsIProgrammingLanguage::JAVASCRIPT, obj);
     }
 
     // The addref was part of JSObject construction
@@ -224,7 +220,7 @@ nsXULPDGlobalObject_resolve(JSContext *cx, JSObject *obj, jsval id)
 
 JSClass nsXULPDGlobalObject::gSharedGlobalClass = {
     "nsXULPrototypeScript compilation scope",
-    JSCLASS_HAS_PRIVATE | JSCLASS_PRIVATE_IS_NSISUPPORTS,
+    JSCLASS_HAS_PRIVATE | JSCLASS_PRIVATE_IS_NSISUPPORTS | JSCLASS_GLOBAL_FLAGS,
     JS_PropertyStub,  JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
     JS_EnumerateStub, nsXULPDGlobalObject_resolve,  JS_ConvertStub,
     nsXULPDGlobalObject_finalize
@@ -266,8 +262,8 @@ nsXULPrototypeDocument::Init()
 nsXULPrototypeDocument::~nsXULPrototypeDocument()
 {
     if (mGlobalObject) {
-        mGlobalObject->SetContext(nsnull); // remove circular reference
-        mGlobalObject->SetGlobalObjectOwner(nsnull); // just in case
+        // cleaup cycles etc.
+        mGlobalObject->SetGlobalObjectOwner(nsnull);
     }
     
     if (mRoot)
@@ -355,12 +351,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
 {
     nsresult rv;
 
-    PRUint32 version;
-    rv = aStream->Read32(&version);
-    if (version != XUL_FASTLOAD_FILE_VERSION)
-        return NS_ERROR_FAILURE;
-
-    rv |= aStream->ReadObject(PR_TRUE, getter_AddRefs(mURI));
+    rv = aStream->ReadObject(PR_TRUE, getter_AddRefs(mURI));
 
     PRUint32 referenceCount;
     nsCOMPtr<nsIURI> referenceURI;
@@ -404,10 +395,6 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     if (! mRoot)
        return NS_ERROR_OUT_OF_MEMORY;
 
-    nsIScriptContext *scriptContext = mGlobalObject->GetContext();
-    NS_ASSERTION(scriptContext != nsnull,
-                 "no prototype script context!");
-
     // nsINodeInfo table
     nsCOMArray<nsINodeInfo> nodeInfos;
 
@@ -430,7 +417,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     if ((nsXULPrototypeNode::Type)type != nsXULPrototypeNode::eType_Element)
         return NS_ERROR_FAILURE;
 
-    rv |= mRoot->Deserialize(aStream, scriptContext, mURI, &nodeInfos);
+    rv |= mRoot->Deserialize(aStream, mGlobalObject, mURI, &nodeInfos);
     rv |= NotifyLoadDone();
 
     return rv;
@@ -487,9 +474,7 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
 {
     nsresult rv;
 
-    rv = aStream->Write32(XUL_FASTLOAD_FILE_VERSION);
-    
-    rv |= aStream->WriteCompoundObject(mURI, NS_GET_IID(nsIURI), PR_TRUE);
+    rv = aStream->WriteCompoundObject(mURI, NS_GET_IID(nsIURI), PR_TRUE);
     
     PRUint32 referenceCount;
     nsCOMPtr<nsIURI> referenceURI;
@@ -543,10 +528,8 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     nsIScriptGlobalObject* globalObject = GetScriptGlobalObject();
     NS_ENSURE_TRUE(globalObject, NS_ERROR_UNEXPECTED);
 
-    nsIScriptContext *scriptContext = globalObject->GetContext();
-
     if (mRoot)
-        rv |= mRoot->Serialize(aStream, scriptContext, &nodeInfos);
+        rv |= mRoot->Serialize(aStream, globalObject, &nodeInfos);
  
     return rv;
 }
@@ -779,9 +762,9 @@ nsXULPrototypeDocument::GetScriptGlobalObject()
 //
 
 nsXULPDGlobalObject::nsXULPDGlobalObject()
-    : mJSObject(nsnull),
-      mGlobalObjectOwner(nsnull)
+    :  mGlobalObjectOwner(nsnull)
 {
+  memset(mScriptGlobals, 0, sizeof(mScriptGlobals));
 }
 
 
@@ -803,82 +786,129 @@ NS_INTERFACE_MAP_END
 // nsIScriptGlobalObject methods
 //
 
-void
-nsXULPDGlobalObject::SetContext(nsIScriptContext *aContext)
+nsresult
+nsXULPDGlobalObject::SetScriptContext(PRUint32 lang_id, nsIScriptContext *aScriptContext)
 {
-    mScriptContext = aContext;
+  // almost a clone of nsGlobalWindow
+  nsresult rv;
+
+  PRBool ok = NS_STID_VALID(lang_id);
+  NS_ASSERTION(ok, "Invalid programming language ID requested");
+  NS_ENSURE_TRUE(ok, NS_ERROR_INVALID_ARG);
+  PRUint32 lang_ndx = NS_STID_INDEX(lang_id);
+
+  if (!aScriptContext)
+    NS_WARNING("Possibly early removal of script object, see bug #41608");
+  else {
+    // should probably assert the context is clean???
+    aScriptContext->WillInitializeContext();
+    // NOTE: We init this context with a NULL global - this is subtly
+    // different than nsGlobalWindow which passes 'this'
+    rv = aScriptContext->InitContext(nsnull);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
+  nsIScriptContext *existing;
+  existing = mScriptContexts[lang_ndx];
+  NS_ASSERTION(!aScriptContext || !existing, "Bad call to SetContext()!");
+
+  if (existing)
+    existing->SetOwner(nsnull);
+  void *script_glob = nsnull;
+
+  if (aScriptContext) {
+    aScriptContext->DidInitializeContext();
+    script_glob = aScriptContext->GetNativeGlobal();
+    NS_ASSERTION(script_glob, "GetNativeGlobal returned NULL!");
+  }
+  mScriptContexts[lang_ndx] = aScriptContext;
+  mScriptGlobals[lang_ndx] = script_glob;
+  return NS_OK;
 }
 
-
-nsIScriptContext *
-nsXULPDGlobalObject::GetContext()
-{
-    // This whole fragile mess is predicated on the fact that
-    // GetContext() will be called before GetScriptObject() is.
-    if (! mScriptContext) {
-        nsCOMPtr<nsIDOMScriptObjectFactory> factory =
-            do_GetService(kDOMScriptObjectFactoryCID);
-        NS_ENSURE_TRUE(factory, nsnull);
-
-        nsresult rv =
-            factory->NewScriptContext(nsnull, getter_AddRefs(mScriptContext));
-        if (NS_FAILED(rv))
-            return nsnull;
-
-        JSContext *cx = (JSContext *)mScriptContext->GetNativeContext();
-
-        mJSObject = ::JS_NewObject(cx, &gSharedGlobalClass, nsnull, nsnull);
-        if (!mJSObject)
-            return nsnull;
-
-        ::JS_SetGlobalObject(cx, mJSObject);
-
-        // Add an owning reference from JS back to us. This'll be
-        // released when the JSObject is finalized.
-        ::JS_SetPrivate(cx, mJSObject, this);
-        NS_ADDREF(this);
-    }
-
-    return mScriptContext;
-}
-
-
 nsresult
-nsXULPDGlobalObject::SetNewDocument(nsIDOMDocument *aDocument,
-                                    PRBool aRemoveEventListeners,
-                                    PRBool aClearScope)
+nsXULPDGlobalObject::EnsureScriptEnvironment(PRUint32 lang_id)
 {
-    NS_NOTREACHED("waaah!");
-    return NS_ERROR_UNEXPECTED;
+  PRBool ok = NS_STID_VALID(lang_id);
+  NS_ASSERTION(ok, "Invalid programming language ID requested");
+  NS_ENSURE_TRUE(ok, NS_ERROR_INVALID_ARG);
+  PRUint32 lang_ndx = NS_STID_INDEX(lang_id);
+
+  if (mScriptContexts[lang_ndx] == nsnull) {
+    nsresult rv;
+    NS_ASSERTION(mScriptGlobals[lang_ndx] == nsnull, "Have global without context?");
+
+    nsCOMPtr<nsIScriptRuntime> languageRuntime;
+    rv = NS_GetScriptRuntimeByID(lang_id, getter_AddRefs(languageRuntime));
+    NS_ENSURE_SUCCESS(rv, nsnull);
+
+    nsCOMPtr<nsIScriptContext> ctxNew;
+    rv = languageRuntime->CreateContext(getter_AddRefs(ctxNew));
+    // For JS, we have to setup a special global object.  We do this then
+    // attach it as the global for this context.  Then, ::SetScriptContext
+    // will re-fetch the global and set it up in our language globals array.
+    if (lang_id == nsIProgrammingLanguage::JAVASCRIPT) {
+      // some special JS specific code we should abstract
+      JSContext *cx = (JSContext *)ctxNew->GetNativeContext();
+      JSAutoRequest ar(cx);
+      JSObject *newGlob = ::JS_NewObject(cx, &gSharedGlobalClass, nsnull, nsnull);
+      if (!newGlob)
+        return nsnull;
+
+      ::JS_SetGlobalObject(cx, newGlob);
+
+      // Add an owning reference from JS back to us. This'll be
+      // released when the JSObject is finalized.
+      ::JS_SetPrivate(cx, newGlob, this);
+      NS_ADDREF(this);
+    }
+
+    NS_ENSURE_SUCCESS(rv, nsnull);
+    rv = SetScriptContext(lang_id, ctxNew);
+    NS_ENSURE_SUCCESS(rv, nsnull);
+  }
+  return NS_OK;
 }
 
-
-void
-nsXULPDGlobalObject::SetDocShell(nsIDocShell *aDocShell)
+nsIScriptContext *
+nsXULPDGlobalObject::GetScriptContext(PRUint32 lang_id)
 {
-    NS_NOTREACHED("waaah!");
-}
-
-
-nsIDocShell *
-nsXULPDGlobalObject::GetDocShell()
-{
-    NS_WARNING("waaah!");
-
+  // This global object creates a context on demand - do that now.
+  nsresult rv = EnsureScriptEnvironment(lang_id);
+  if (NS_FAILED(rv)) {
+    NS_ERROR("Failed to setup script language");
     return nsnull;
+  }
+  // Note that EnsureScriptEnvironment has validated lang_id
+  return mScriptContexts[NS_STID_INDEX(lang_id)];
 }
 
-
-void
-nsXULPDGlobalObject::SetOpenerWindow(nsIDOMWindowInternal *aOpener)
+void *
+nsXULPDGlobalObject::GetScriptGlobal(PRUint32 lang_id)
 {
-    NS_NOTREACHED("waaah!");
+  PRBool ok = NS_STID_VALID(lang_id);
+  NS_ASSERTION(ok, "Invalid programming language ID requested");
+  NS_ENSURE_TRUE(ok, nsnull);
+  PRUint32 lang_ndx = NS_STID_INDEX(lang_id);
+
+  NS_ASSERTION(mScriptContexts[lang_ndx] != nsnull, "Querying for global before setting up context?");
+  return mScriptGlobals[lang_ndx];
 }
 
 
 void
 nsXULPDGlobalObject::SetGlobalObjectOwner(nsIScriptGlobalObjectOwner* aOwner)
 {
+    if (aOwner == nsnull) {
+        // this means we should cleanup
+        PRUint32 lang_ndx;
+        NS_STID_FOR_INDEX(lang_ndx) {
+            if (mScriptContexts[lang_ndx]) {
+                mScriptContexts[lang_ndx]->FinalizeContext();
+                mScriptContexts[lang_ndx] = nsnull;
+            }
+        }
+    }
     mGlobalObjectOwner = aOwner; // weak reference
 }
 
@@ -890,40 +920,13 @@ nsXULPDGlobalObject::GetGlobalObjectOwner()
 }
 
 
-nsresult
-nsXULPDGlobalObject::HandleDOMEvent(nsPresContext* aPresContext, 
-                                       nsEvent* aEvent, 
-                                       nsIDOMEvent** aDOMEvent,
-                                       PRUint32 aFlags,
-                                       nsEventStatus* aEventStatus)
-{
-    NS_NOTREACHED("waaah!");
-    return NS_ERROR_UNEXPECTED;
-}
-
-JSObject *
-nsXULPDGlobalObject::GetGlobalJSObject()
-{
-    // The prototype document has its own special secret script object
-    // that can be used to compile scripts and event handlers.
-
-    if (!mScriptContext)
-        return nsnull;
-
-    JSContext* cx = NS_REINTERPRET_CAST(JSContext*,
-                                        mScriptContext->GetNativeContext());
-    if (!cx)
-        return nsnull;
-
-    return ::JS_GetGlobalObject(cx);
-}
-
 void
-nsXULPDGlobalObject::OnFinalize(JSObject *aObject)
+nsXULPDGlobalObject::OnFinalize(PRUint32 aLangID, void *aObject)
 {
-    NS_ASSERTION(aObject == mJSObject, "Wrong object finalized!");
-
-    mJSObject = nsnull;
+    NS_ASSERTION(NS_STID_VALID(aLangID), "Invalid language ID");
+    NS_ASSERTION(aObject == mScriptGlobals[NS_STID_INDEX(aLangID)],
+                 "Wrong object finalized!");
+    mScriptGlobals[NS_STID_INDEX(aLangID)] = nsnull;
 }
 
 void
@@ -932,6 +935,12 @@ nsXULPDGlobalObject::SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts)
     // We don't care...
 }
 
+nsresult
+nsXULPDGlobalObject::SetNewArguments(nsIArray *aArguments)
+{
+    NS_NOTREACHED("waaah!");
+    return NS_ERROR_UNEXPECTED;
+}
 
 //----------------------------------------------------------------------
 //
