@@ -49,14 +49,16 @@
 #include "nsIComponentManager.h"
 #include "nsLDAPConnection.h"
 #include "nsLDAPMessage.h"
-#include "nsIEventQueueService.h"
+#include "nsThreadUtils.h"
 #include "nsIConsoleService.h"
 #include "nsIDNSService.h"
+#include "nsIDNSRecord.h"
 #include "nsIRequestObserver.h"
 #include "nsIProxyObjectManager.h"
-#include "nsEventQueueUtils.h"
 #include "nsNetError.h"
 #include "nsLDAPOperation.h"
+#include "nsILDAPErrors.h"
+#include "nsIClassInfoImpl.h"
 
 const char kConsoleServiceContractId[] = "@mozilla.org/consoleservice;1";
 const char kDNSServiceContractId[] = "@mozilla.org/network/dns-service;1";
@@ -90,10 +92,15 @@ nsLDAPConnection::~nsLDAPConnection()
 // since converting to the strong reference isn't MT safe.
 //
 NS_IMPL_THREADSAFE_ADDREF(nsLDAPConnection)
-NS_IMPL_THREADSAFE_QUERY_INTERFACE3(nsLDAPConnection,
-                                    nsILDAPConnection,
-                                    nsISupportsWeakReference,
-                                    nsIDNSListener)
+NS_INTERFACE_MAP_BEGIN(nsLDAPConnection)
+  NS_INTERFACE_MAP_ENTRY(nsILDAPConnection)
+  NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
+  NS_INTERFACE_MAP_ENTRY(nsIDNSListener)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsILDAPConnection)
+  NS_IMPL_QUERY_CLASSINFO(nsLDAPConnection)
+NS_INTERFACE_MAP_END_THREADSAFE
+NS_IMPL_CI_INTERFACE_GETTER3(nsLDAPConnection, nsILDAPConnection, 
+                             nsISupportsWeakReference, nsIDNSListener)
 
 nsrefcnt
 nsLDAPConnection::Release(void)
@@ -186,11 +193,10 @@ nsLDAPConnection::Init(const char *aHost, PRInt32 aPort, PRBool aSSL,
         return NS_ERROR_FAILURE;
     }
 
-    nsCOMPtr<nsIEventQueue> curEventQ;
-    rv = NS_GetCurrentEventQ(getter_AddRefs(curEventQ));
-    if (NS_FAILED(rv)) {
+    nsCOMPtr<nsIThread> curThread = do_GetCurrentThread();
+    if (!curThread) {
         NS_ERROR("nsLDAPConnection::Init(): couldn't "
-                 "get current event queue");
+                 "get current thread");
         return NS_ERROR_FAILURE;
     }
     // Do the pre-resolve of the hostname, using the DNS service. This
@@ -222,7 +228,7 @@ nsLDAPConnection::Init(const char *aHost, PRInt32 aPort, PRBool aSSL,
     if (spacePos != kNotFound)
       mDNSHost.Truncate(spacePos);
 
-    rv = pDNSService->AsyncResolve(mDNSHost, 0, this, curEventQ, 
+    rv = pDNSService->AsyncResolve(mDNSHost, 0, this, curThread, 
                                    getter_AddRefs(mDNSRequest));
 
     if (NS_FAILED(rv)) {
@@ -277,7 +283,7 @@ nsLDAPConnection::Close()
   // Init listener (if still there).
   //
   if (mDNSRequest) {
-      mDNSRequest->Cancel();
+      mDNSRequest->Cancel(NS_ERROR_ABORT);
       mDNSRequest = 0;
   }
   mInitListener = 0;
@@ -466,7 +472,10 @@ nsLDAPConnection::InvokeMessageCallback(LDAPMessage *aMsgHandle,
     nsCOMPtr<nsILDAPOperation> operation;
     nsCOMPtr<nsILDAPMessageListener> listener;
 
+#if defined(DEBUG)
+    // We only want this being logged for debug builds so as not to affect performance too much.
     PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, ("InvokeMessageCallback entered\n"));
+#endif
 
     // get the message id corresponding to this operation
     //
@@ -593,7 +602,7 @@ CheckLDAPOperationResult(nsHashKey *aKey, void *aData, void* aClosure)
     LDAPMessage *msgHandle;
     nsCOMPtr<nsILDAPMessage> msg;
     PRBool operationFinished = PR_TRUE;
-    struct timeval timeout = { 1, 0 }; 
+    struct timeval timeout = { 0, 0 }; 
     PRIntervalTime sleepTime = PR_MillisecondsToInterval(40);
 
     // we need to access some of the connection loop's objects
@@ -621,16 +630,11 @@ CheckLDAPOperationResult(nsHashKey *aKey, void *aData, void* aClosure)
         case 0: // timeout
 
             // the connection may not exist yet.  sleep for a while
-            // and try again
-            //
-        PR_LOG(gLDAPLogModule, PR_LOG_WARNING, ("ldap_result() timed out.\n"));
-
-            // The sleep here is to avoid a problem where the LDAP
-            // Connection/thread isn't ready quite yet, and we want to
-            // avoid a very busy loop.
+            // to avoid a problem where the LDAP connection/thread isn't 
+            // ready quite yet, and we want to avoid a very busy loop.
             //
             PR_Sleep(sleepTime);
-        return PR_TRUE;
+            return PR_TRUE;
 
         case -1: // something went wrong 
 
@@ -714,34 +718,37 @@ CheckLDAPOperationResult(nsHashKey *aKey, void *aData, void* aClosure)
             // initialize the message, using a protected method not available
             // through nsILDAPMessage (which is why we need the raw pointer)
             //
-        rv = rawMsg->Init(loop->mRawConn, msgHandle);
+            rv = rawMsg->Init(loop->mRawConn, msgHandle);
 
             switch (rv) {
 
-            case NS_OK: 
-              {
+            case NS_OK: {
                 PRInt32 errorCode;
                 rawMsg->GetErrorCode(&errorCode);
-                if (errorCode == LDAP_PROTOCOL_ERROR)
-                {
-                  // maybe a version error, e.g., using v3 on a v2 server.
-                  // if we're using v3, try v2.
-                  if (loop->mRawConn->mVersion == nsILDAPConnection::VERSION3)
-                  {
+                // maybe a version error, e.g., using v3 on a v2 server.
+                // if we're using v3, try v2.
+                //
+                if (errorCode == LDAP_PROTOCOL_ERROR && 
+                   loop->mRawConn->mVersion == nsILDAPConnection::VERSION3) {
                     nsCAutoString password;
                     loop->mRawConn->mVersion = nsILDAPConnection::VERSION2;
-                    ldap_set_option(loop->mRawConn->mConnectionHandle, LDAP_OPT_PROTOCOL_VERSION, &loop->mRawConn->mVersion);
-                    nsCOMPtr <nsILDAPOperation> operation = NS_STATIC_CAST(nsILDAPOperation *, NS_STATIC_CAST(nsISupports *, aData));
-                    // we pass in an empty password to tell the operation that it
-                    // should use the cached password.
-                    operation->SimpleBind(password);
-                    operationFinished = PR_FALSE;
-                    // we don't want to notify callers that we're done...
-                    return PR_TRUE;
-                  }
+                    ldap_set_option(loop->mRawConn->mConnectionHandle,
+                          LDAP_OPT_PROTOCOL_VERSION, &loop->mRawConn->mVersion);
+                    nsCOMPtr <nsILDAPOperation> operation = 
+                      NS_STATIC_CAST(nsILDAPOperation *, 
+                          NS_STATIC_CAST(nsISupports *, aData));
+                    // we pass in an empty password to tell the operation that 
+                    // it should use the cached password.
+                    //
+                    rv = operation->SimpleBind(password);
+                    if (NS_SUCCEEDED(rv)) {
+                        operationFinished = PR_FALSE;
+                        // we don't want to notify callers that we're done...
+                        return PR_TRUE;
+                    }
                 }
-              }
-              break;
+            }
+            break;
 
             case NS_ERROR_LDAP_DECODING_ERROR:
                 consoleSvc->LogStringMessage(
@@ -863,7 +870,7 @@ nsLDAPConnectionLoop::Run(void)
 }
 
 NS_IMETHODIMP
-nsLDAPConnection::OnLookupComplete(nsIDNSRequest *aRequest,
+nsLDAPConnection::OnLookupComplete(nsICancelable *aRequest,
                                    nsIDNSRecord  *aRecord,
                                    nsresult       aStatus)
 {    
@@ -996,9 +1003,8 @@ nsLDAPConnection::OnLookupComplete(nsIDNSRequest *aRequest,
         // all locking etc. in nsLDAPConnection::Release().
         //
         mRunnable = new nsLDAPConnectionLoop();
-        NS_ADDREF(mRunnable);
-        rv = mRunnable->Init();
-        if (NS_FAILED(rv)) {
+        NS_IF_ADDREF(mRunnable);
+        if (!mRunnable || NS_FAILED(mRunnable->Init())) {
             rv = NS_ERROR_OUT_OF_MEMORY;
         } else {
             // Here we keep a weak reference in the runnable object to the
@@ -1015,13 +1021,13 @@ nsLDAPConnection::OnLookupComplete(nsIDNSRequest *aRequest,
             mRunnable->mWeakConn = do_GetWeakReference(conn);
 
             // kick off a thread for result listening and marshalling
-            // XXXdmose - should this be JOINABLE?
             //
-            rv = NS_NewThread(getter_AddRefs(mThread), mRunnable, 0,
-                          PR_UNJOINABLE_THREAD);
+            rv = NS_NewThread(getter_AddRefs(mThread), mRunnable);
             if (NS_FAILED(rv)) {
                 rv = NS_ERROR_NOT_AVAILABLE;
             }
+            // XXX(darin): We need to shutdown this thread at some point.
+            //             Otherwise, it will stick around until shutdown.
         }
     }
 
