@@ -22,6 +22,7 @@
  *
  * Contributor(s):
  *   Adam Lock <adamlock@netscape.com>
+ *   Jeff Walden <jwalden+code@mit.edu>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -44,6 +45,7 @@
 #include "nsCRT.h"
 
 #include "nsIPrefService.h"
+#include "nsIPrefLocalizedString.h"
 #include "nsIPlatformCharset.h"
 #include "nsILocalFile.h"
 
@@ -252,6 +254,10 @@ nsDefaultURIFixup::CreateFixupURI(const nsACString& aStringURI, PRUint32 aFixupF
         // Just try to create an URL out of it
         rv = NS_NewURI(aURI, uriString,
                        bUseNonDefaultCharsetForURI ? GetCharsetForUrlBar() : nsnull);
+
+        if (!*aURI && rv != NS_ERROR_MALFORMED_URI) {
+            return rv;
+        }
     }
     
     if (*aURI) {
@@ -262,8 +268,8 @@ nsDefaultURIFixup::CreateFixupURI(const nsACString& aStringURI, PRUint32 aFixupF
 
     // See if it is a keyword
     // Test whether keywords need to be fixed up
+    PRBool fixupKeywords = PR_FALSE;
     if (aFixupFlags & FIXUP_FLAG_ALLOW_KEYWORD_LOOKUP) {
-        PRBool fixupKeywords = PR_FALSE;
         if (mPrefBranch)
         {
             NS_ENSURE_SUCCESS(mPrefBranch->GetBoolPref("keyword.enabled", &fixupKeywords), NS_ERROR_FAILURE);
@@ -332,9 +338,67 @@ nsDefaultURIFixup::CreateFixupURI(const nsACString& aStringURI, PRUint32 aFixupF
         MakeAlternateURI(*aURI);
     }
 
+    // If we still haven't been able to construct a valid URI, try to force a
+    // keyword match.  This catches search strings with '.' or ':' in them.
+    if (!*aURI && fixupKeywords)
+    {
+        KeywordToURI(aStringURI, aURI);
+        if(*aURI)
+            return NS_OK;
+    }
+
     return rv;
 }
 
+static nsresult MangleKeywordIntoURI(const char *aKeyword, const char *aURL,
+                                     nsCString& query)
+{
+    query = (*aKeyword == '?') ? (aKeyword + 1) : aKeyword;
+    query.Trim(" "); // pull leading/trailing spaces.
+
+    // encode
+    char * encQuery = nsEscape(query.get(), url_XPAlphas);
+    if (!encQuery) return NS_ERROR_OUT_OF_MEMORY;
+    query.Adopt(encQuery);
+
+    // prepend the query with the keyword url
+    // XXX this url should come from somewhere else
+    query.Insert(aURL, 0);
+    return NS_OK;
+}
+
+NS_IMETHODIMP nsDefaultURIFixup::KeywordToURI(const nsACString& aKeyword,
+                                              nsIURI **aURI)
+{
+    *aURI = nsnull;
+    NS_ENSURE_STATE(mPrefBranch);
+
+    nsXPIDLCString url;
+    nsCOMPtr<nsIPrefLocalizedString> keywordURL;
+    mPrefBranch->GetComplexValue("keyword.URL", 
+                                 NS_GET_IID(nsIPrefLocalizedString),
+                                 getter_AddRefs(keywordURL));
+
+    if (keywordURL) {
+        nsXPIDLString wurl;
+        keywordURL->GetData(getter_Copies(wurl));
+        CopyUTF16toUTF8(wurl, url);
+    } else {
+        // Fall back to a non-localized pref, for backwards compat
+        mPrefBranch->GetCharPref("keyword.URL", getter_Copies(url));
+    }
+
+    // if we can't find a keyword.URL keywords won't work.
+    if (url.IsEmpty())
+        return NS_ERROR_NOT_AVAILABLE;
+
+    nsCAutoString spec;
+    nsresult rv = MangleKeywordIntoURI(PromiseFlatCString(aKeyword).get(),
+                                       url.get(), spec);
+    if (NS_FAILED(rv)) return rv;
+
+    return NS_NewURI(aURI, spec);
+}
 
 PRBool nsDefaultURIFixup::MakeAlternateURI(nsIURI *aURI)
 {
@@ -466,7 +530,7 @@ nsresult nsDefaultURIFixup::ConvertFileToStringURI(const nsACString& aIn,
     {
         attemptFixup = PR_TRUE;
     }
-#elif XP_UNIX
+#elif defined(XP_UNIX) || defined(XP_BEOS)
     // Check if it starts with / (UNIX)
     if(aIn.First() == '/')
     {
@@ -513,10 +577,10 @@ nsresult nsDefaultURIFixup::ConvertFileToStringURI(const nsACString& aIn,
         // to stop using AssignWithConversion and do things correctly.  See bug
         // 58866 for what happens if we remove this
         // PossiblyByteExpandedFileName check.
-        NS_ConvertUTF8toUCS2 in(aIn);
+        NS_ConvertUTF8toUTF16 in(aIn);
         if (PossiblyByteExpandedFileName(in)) {
           // removes high byte
-          rv = NS_NewNativeLocalFile(NS_LossyConvertUCS2toASCII(in), PR_FALSE, getter_AddRefs(filePath));
+          rv = NS_NewNativeLocalFile(NS_LossyConvertUTF16toASCII(in), PR_FALSE, getter_AddRefs(filePath));
         }
         else {
           // input is unicode
@@ -695,44 +759,31 @@ nsresult nsDefaultURIFixup::KeywordURIFixup(const nsACString & aURIString,
     // These are keyword formatted strings
     // "what is mozilla"
     // "what is mozilla?"
-    // "?mozilla"
-    // "?What is mozilla"
+    // "docshell site:mozilla.org" - has no dot/colon in the first space-separated substring
+    // "?mozilla" - anything that begins with a question mark
+    // "?site:mozilla.org docshell"
 
     // These are not keyword formatted strings
-    // "www.blah.com" - anything with a dot in it 
-    // "nonQualifiedHost:80" - anything with a colon in it
+    // "www.blah.com" - first space-separated substring contains a dot, doesn't start with "?"
+    // "www.blah.com stuff"
+    // "nonQualifiedHost:80" - first space-separated substring contains a colon, doesn't start with "?"
+    // "nonQualifiedHost:80 args"
     // "nonQualifiedHost?"
     // "nonQualifiedHost?args"
     // "nonQualifiedHost?some args"
 
-    if(aURIString.FindChar('.') == -1 && aURIString.FindChar(':') == -1)
+    PRInt32 dotLoc   = aURIString.FindChar('.');
+    PRInt32 colonLoc = aURIString.FindChar(':');
+    PRInt32 spaceLoc = aURIString.FindChar(' ');
+    PRInt32 qMarkLoc = aURIString.FindChar('?');
+
+    if ((dotLoc == kNotFound || (spaceLoc > 0 && spaceLoc < dotLoc)) &&
+        (colonLoc == kNotFound || (spaceLoc > 0 && spaceLoc < colonLoc)) &&
+        (spaceLoc > 0 && (qMarkLoc == kNotFound || spaceLoc < qMarkLoc)) ||
+        qMarkLoc == 0)
     {
-        PRInt32 qMarkLoc = aURIString.FindChar('?');
-        PRInt32 spaceLoc = aURIString.FindChar(' ');
-
-        PRBool keyword = PR_FALSE;
-        if(qMarkLoc == 0)
-        keyword = PR_TRUE;
-        else if((spaceLoc > 0) && ((qMarkLoc == -1) || (spaceLoc < qMarkLoc)))
-        keyword = PR_TRUE;
-
-        if(keyword)
-        {
-            nsCAutoString keywordSpec("keyword:");
-            char *utf8Spec = ToNewCString(aURIString); // aURIString is UTF-8
-            if(utf8Spec)
-            {
-                char* escapedUTF8Spec = nsEscape(utf8Spec, url_Path);
-                if(escapedUTF8Spec) 
-                {
-                    keywordSpec.Append(escapedUTF8Spec);
-                    NS_NewURI(aURI, keywordSpec.get(), nsnull);
-                    nsMemory::Free(escapedUTF8Spec);
-                } // escapedUTF8Spec
-                nsMemory::Free(utf8Spec);
-            } // utf8Spec
-        } // keyword 
-    } // FindChar
+        KeywordToURI(aURIString, aURI);
+    }
 
     if(*aURI)
         return NS_OK;
