@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=4 sw=4 et tw=80:
  *
  * ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
@@ -48,6 +49,7 @@
 #include "nsIContentViewer.h"
 #include "nsIPrefBranch.h"
 #include "nsVoidArray.h"
+#include "nsInterfaceHashtable.h"
 #include "nsIScriptContext.h"
 #include "nsITimer.h"
 
@@ -56,7 +58,7 @@
 #include "nsIContentViewerContainer.h"
 #include "nsIDeviceContext.h"
 
-#include "nsIDocumentLoader.h"
+#include "nsDocLoader.h"
 #include "nsIURILoader.h"
 #include "nsIEditorDocShell.h"
 
@@ -70,6 +72,8 @@
 #include "nsCOMPtr.h"
 #include "nsPoint.h" // mCurrent/mDefaultScrollbarPreferences
 #include "nsString.h"
+#include "nsAutoPtr.h"
+#include "nsThreadUtils.h"
 
 // Threshold value in ms for META refresh based redirects
 #define REFRESH_REDIRECT_TIMER 15000
@@ -99,51 +103,12 @@
 #include "nsIHttpChannel.h"
 #include "nsDocShellTransferableHooks.h"
 #include "nsIAuthPromptProvider.h"
-
-
-#define MAKE_LOAD_TYPE(type, flags) ((type) | ((flags) << 16))
+#include "nsISecureBrowserUI.h"
+#include "nsIObserver.h"
+#include "nsDocShellLoadTypes.h"
 
 /* load commands were moved to nsIDocShell.h */
-
-/* load types are legal combinations of load commands and flags 
- *  
- * NOTE:
- *  Remember to update the IsValidLoadType function below if you change this
- *  enum to ensure bad flag combinations will be rejected.
- */
-enum LoadType {
-    LOAD_NORMAL = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_NORMAL, nsIWebNavigation::LOAD_FLAGS_NONE),
-    LOAD_NORMAL_REPLACE = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_NORMAL, nsIWebNavigation::LOAD_FLAGS_REPLACE_HISTORY),
-    LOAD_HISTORY = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_HISTORY, nsIWebNavigation::LOAD_FLAGS_NONE),
-    LOAD_RELOAD_NORMAL = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_RELOAD, nsIWebNavigation::LOAD_FLAGS_NONE),
-    LOAD_RELOAD_BYPASS_CACHE = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_RELOAD, nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE),
-    LOAD_RELOAD_BYPASS_PROXY = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_RELOAD, nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY),
-    LOAD_RELOAD_BYPASS_PROXY_AND_CACHE = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_RELOAD, nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY),
-    LOAD_LINK = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_NORMAL, nsIWebNavigation::LOAD_FLAGS_IS_LINK),
-    LOAD_REFRESH = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_NORMAL, nsIWebNavigation::LOAD_FLAGS_IS_REFRESH),
-    LOAD_RELOAD_CHARSET_CHANGE = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_RELOAD, nsIWebNavigation::LOAD_FLAGS_CHARSET_CHANGE),
-    LOAD_BYPASS_HISTORY = MAKE_LOAD_TYPE(nsIDocShell::LOAD_CMD_NORMAL, nsIWebNavigation::LOAD_FLAGS_BYPASS_HISTORY)
-    // NOTE: Adding a new value? Remember to update IsValidLoadType!
-};
-static inline PRBool IsValidLoadType(PRUint32 aLoadType)
-{
-    switch (aLoadType)
-    {
-    case LOAD_NORMAL:
-    case LOAD_NORMAL_REPLACE:
-    case LOAD_HISTORY:
-    case LOAD_RELOAD_NORMAL:
-    case LOAD_RELOAD_BYPASS_CACHE:
-    case LOAD_RELOAD_BYPASS_PROXY:
-    case LOAD_RELOAD_BYPASS_PROXY_AND_CACHE:
-    case LOAD_LINK:
-    case LOAD_REFRESH:
-    case LOAD_RELOAD_CHARSET_CHANGE:
-    case LOAD_BYPASS_HISTORY:
-        return PR_TRUE;
-    }
-    return PR_FALSE;
-}
+/* load types were moved to nsDocShellLoadTypes.h */
 
 /* internally used ViewMode types */
 enum ViewMode {
@@ -179,7 +144,8 @@ protected:
 //***    nsDocShell
 //*****************************************************************************
 
-class nsDocShell : public nsIDocShell,
+class nsDocShell : public nsDocLoader,
+                   public nsIDocShell,
                    public nsIDocShellTreeItem, 
                    public nsIDocShellTreeNode,
                    public nsIDocShellHistory,
@@ -189,14 +155,13 @@ class nsDocShell : public nsIDocShell,
                    public nsITextScroll, 
                    public nsIDocCharset, 
                    public nsIContentViewerContainer,
-                   public nsIInterfaceRequestor,
                    public nsIScriptGlobalObjectOwner,
                    public nsIRefreshURI,
                    public nsIWebProgressListener,
                    public nsIEditorDocShell,
                    public nsIWebPageDescriptor,
                    public nsIAuthPromptProvider,
-                   public nsSupportsWeakReference
+                   public nsIObserver
 {
 friend class nsDSURIContentListener;
 
@@ -204,7 +169,9 @@ public:
     // Object Management
     nsDocShell();
 
-    NS_DECL_ISUPPORTS
+    virtual nsresult Init();
+
+    NS_DECL_ISUPPORTS_INHERITED
 
     NS_DECL_NSIDOCSHELL
     NS_DECL_NSIDOCSHELLTREEITEM
@@ -222,23 +189,38 @@ public:
     NS_DECL_NSIEDITORDOCSHELL
     NS_DECL_NSIWEBPAGEDESCRIPTOR
     NS_DECL_NSIAUTHPROMPTPROVIDER
+    NS_DECL_NSIOBSERVER
+
+    NS_IMETHOD Stop() {
+        // Need this here because otherwise nsIWebNavigation::Stop
+        // overrides the docloader's Stop()
+        return nsDocLoader::Stop();
+    }
+
+    // Need to implement (and forward) nsISecurityEventSink, because
+    // nsIWebProgressListener has methods with identical names...
+    NS_FORWARD_NSISECURITYEVENTSINK(nsDocLoader::)
 
-    nsresult SetLoadCookie(nsISupports * aCookie);
-    nsresult GetLoadCookie(nsISupports ** aResult);
     nsDocShellInfoLoadType ConvertLoadTypeToDocShellLoadInfo(PRUint32 aLoadType);
     PRUint32 ConvertDocShellLoadInfoToLoadType(nsDocShellInfoLoadType aDocShellLoadType);
 
     // nsIScriptGlobalObjectOwner methods
     virtual nsIScriptGlobalObject* GetScriptGlobalObject();
+
+    // Restores a cached presentation from history (mLSHE).
+    // This method swaps out the content viewer and simulates loads for
+    // subframes.  It then simulates the completion of the toplevel load.
+    nsresult RestoreFromHistory();
+
 protected:
     // Object Management
     virtual ~nsDocShell();
-    NS_IMETHOD DestroyChildren();
+    virtual void DestroyChildren();
 
     // Content Viewer Management
     NS_IMETHOD EnsureContentViewer();
     NS_IMETHOD EnsureDeviceContext();
-    NS_IMETHOD CreateAboutBlankContentViewer();
+    nsresult CreateAboutBlankContentViewer();
     NS_IMETHOD CreateContentViewer(const char * aContentType, 
         nsIRequest * request, nsIStreamListener ** aContentHandler);
     NS_IMETHOD NewContentViewerObj(const char * aContentType, 
@@ -246,25 +228,56 @@ protected:
         nsIStreamListener ** aContentHandler, nsIContentViewer ** aViewer);
     NS_IMETHOD SetupNewViewer(nsIContentViewer * aNewViewer);
 
+    void SetupReferrerFromChannel(nsIChannel * aChannel);
+    
     NS_IMETHOD GetEldestPresContext(nsPresContext** aPresContext);
-    void GetCurrentDocumentOwner(nsISupports ** aOwner);
+
+    // Get the principal that we'll set on the channel if we're inheriting.  If
+    // aConsiderCurrentDocument is true, we try to use the current document if
+    // at all possible.  If that fails, we fall back on the parent document.
+    // If that fails too, we force creation of a content viewer and use the
+    // resulting principal.  If aConsiderCurrentDocument is false, we just look
+    // at the parent.
+    nsIPrincipal* GetInheritedPrincipal(PRBool aConsiderCurrentDocument);
+
+    // Actually open a channel and perform a URI load.  Note: whatever owner is
+    // passed to this function will be set on the channel.  Callers who wish to
+    // not have an owner on the channel should just pass null.
     virtual nsresult DoURILoad(nsIURI * aURI,
                                nsIURI * aReferrer,
+                               PRBool aSendReferrer,
                                nsISupports * aOwner,
                                const char * aTypeHint,
                                nsIInputStream * aPostData,
                                nsIInputStream * aHeadersData,
                                PRBool firstParty,
                                nsIDocShell ** aDocShell,
-                               nsIRequest ** aRequest);
+                               nsIRequest ** aRequest,
+                               PRBool aIsNewWindowTarget);
     NS_IMETHOD AddHeadersToChannel(nsIInputStream * aHeadersData, 
                                   nsIChannel * aChannel);
     virtual nsresult DoChannelLoad(nsIChannel * aChannel,
                                    nsIURILoader * aURILoader);
-    NS_IMETHOD ScrollIfAnchor(nsIURI * aURI, PRBool * aWasAnchor, PRUint32 aLoadType, nscoord *cx, nscoord *cy);
-    NS_IMETHOD OnLoadingSite(nsIChannel * aChannel);
+    NS_IMETHOD ScrollIfAnchor(nsIURI * aURI, PRBool * aWasAnchor,
+                              PRUint32 aLoadType, nscoord *cx, nscoord *cy);
 
-    NS_IMETHOD OnNewURI(nsIURI * aURI, nsIChannel * aChannel, PRUint32 aLoadType);
+    // Returns PR_TRUE if would have called FireOnLocationChange,
+    // but did not because aFireOnLocationChange was false on entry.
+    // In this case it is the caller's responsibility to ensure
+    // FireOnLocationChange is called.
+    // In all other cases PR_FALSE is returned.
+    PRBool OnLoadingSite(nsIChannel * aChannel,
+                         PRBool aFireOnLocationChange,
+                         PRBool aAddToGlobalHistory = PR_TRUE);
+
+    // Returns PR_TRUE if would have called FireOnLocationChange,
+    // but did not because aFireOnLocationChange was false on entry.
+    // In this case it is the caller's responsibility to ensure
+    // FireOnLocationChange is called.
+    // In all other cases PR_FALSE is returned.
+    PRBool OnNewURI(nsIURI * aURI, nsIChannel * aChannel, PRUint32 aLoadType,
+                    PRBool aFireOnLocationChange,
+                    PRBool aAddToGlobalHistory = PR_TRUE);
 
     virtual void SetReferrerURI(nsIURI * aURI);
 
@@ -276,14 +289,75 @@ protected:
 
     NS_IMETHOD LoadHistoryEntry(nsISHEntry * aEntry, PRUint32 aLoadType);
     NS_IMETHOD PersistLayoutHistoryState();
-    NS_IMETHOD CloneAndReplace(nsISHEntry * srcEntry, PRUint32 aCloneID,
-        nsISHEntry * areplaceEntry, nsISHEntry ** destEntry);
+
+    // Clone a session history tree for subframe navigation.
+    // The tree rooted at |aSrcEntry| will be cloned into |aDestEntry|, except
+    // for the entry with id |aCloneID|, which will be replaced with
+    // |aReplaceEntry|.  |aSrcShell| is a (possibly null) docshell which
+    // corresponds to |aSrcEntry| via its mLSHE or mOHE pointers, and will
+    // have that pointer updated to point to the cloned history entry.
+    static nsresult CloneAndReplace(nsISHEntry *aSrcEntry,
+                                    nsDocShell *aSrcShell,
+                                    PRUint32 aCloneID,
+                                    nsISHEntry *aReplaceEntry,
+                                    nsISHEntry **aDestEntry);
+
+    // Child-walking callback for CloneAndReplace
+    static nsresult CloneAndReplaceChild(nsISHEntry *aEntry,
+                                         nsDocShell *aShell,
+                                         PRInt32 aChildIndex, void *aData);
+
     nsresult GetRootSessionHistory(nsISHistory ** aReturn);
     nsresult GetHttpChannel(nsIChannel * aChannel, nsIHttpChannel ** aReturn);
     PRBool ShouldDiscardLayoutState(nsIHttpChannel * aChannel);
-    
+
+    // Determine whether this docshell corresponds to the given history entry,
+    // via having a pointer to it in mOSHE or mLSHE.
+    PRBool HasHistoryEntry(nsISHEntry *aEntry) const
+    {
+        return aEntry && (aEntry == mOSHE || aEntry == mLSHE);
+    }
+
+    // Update any pointers (mOSHE or mLSHE) to aOldEntry to point to aNewEntry
+    void SwapHistoryEntries(nsISHEntry *aOldEntry, nsISHEntry *aNewEntry);
+
+    // Call this method to swap in a new history entry to m[OL]SHE, rather than
+    // setting it directly.  This completes the navigation in all docshells
+    // in the case of a subframe navigation.
+    void SetHistoryEntry(nsCOMPtr<nsISHEntry> *aPtr, nsISHEntry *aEntry);
+
+    // Child-walking callback for SetHistoryEntry
+    static nsresult SetChildHistoryEntry(nsISHEntry *aEntry,
+                                         nsDocShell *aShell,
+                                         PRInt32 aEntryIndex, void *aData);
+
+    // Callback prototype for WalkHistoryEntries.
+    // aEntry is the child history entry, aShell is its corresponding docshell,
+    // aChildIndex is the child's index in its parent entry, and aData is
+    // the opaque pointer passed to WalkHistoryEntries.
+    typedef nsresult (*WalkHistoryEntriesFunc)(nsISHEntry *aEntry,
+                                               nsDocShell *aShell,
+                                               PRInt32 aChildIndex,
+                                               void *aData);
+
+    // For each child of aRootEntry, find the corresponding docshell which is
+    // a child of aRootShell, and call aCallback.  The opaque pointer aData
+    // is passed to the callback.
+    static nsresult WalkHistoryEntries(nsISHEntry *aRootEntry,
+                                       nsDocShell *aRootShell,
+                                       WalkHistoryEntriesFunc aCallback,
+                                       void *aData);
+
+    // overridden from nsDocLoader, this provides more information than the
+    // normal OnStateChange with flags STATE_REDIRECTING
+    virtual void OnRedirectStateChange(nsIChannel* aOldChannel,
+                                       nsIChannel* aNewChannel,
+                                       PRUint32 aRedirectFlags,
+                                       PRUint32 aStateFlags);
+
     // Global History
-    nsresult AddToGlobalHistory(nsIURI * aURI, PRBool aRedirect, nsIURI * aReferrer);
+    nsresult AddToGlobalHistory(nsIURI * aURI, PRBool aRedirect,
+                                nsIChannel * aChannel);
 
     // Helper Routines
     NS_IMETHOD GetPromptAndStringBundle(nsIPrompt ** aPrompt,
@@ -291,14 +365,18 @@ protected:
     NS_IMETHOD GetChildOffset(nsIDOMNode * aChild, nsIDOMNode * aParent,
         PRInt32 * aOffset);
     NS_IMETHOD GetRootScrollableView(nsIScrollableView ** aOutScrollView);
-    NS_IMETHOD EnsureContentListener();
     NS_IMETHOD EnsureScriptEnvironment();
     NS_IMETHOD EnsureEditorData();
     nsresult   EnsureTransferableHookData();
     NS_IMETHOD EnsureFind();
-    NS_IMETHOD RefreshURIFromQueue();
-    NS_IMETHOD DisplayLoadError(nsresult aError, nsIURI *aURI, const PRUnichar *aURL);
-    NS_IMETHOD LoadErrorPage(nsIURI *aURI, const PRUnichar *aURL, const PRUnichar *aPage, const PRUnichar *aDescription);
+    nsresult   RefreshURIFromQueue();
+    NS_IMETHOD DisplayLoadError(nsresult aError, nsIURI *aURI,
+                                const PRUnichar *aURL,
+                                nsIChannel* aFailedChannel = nsnull);
+    NS_IMETHOD LoadErrorPage(nsIURI *aURI, const PRUnichar *aURL,
+                             const PRUnichar *aPage,
+                             const PRUnichar *aDescription,
+                             nsIChannel* aFailedChannel);
     PRBool IsPrintingOrPP(PRBool aDisplayErrorDialog = PR_TRUE);
 
     nsresult SetBaseUrlForWyciwyg(nsIContentViewer * aContentViewer);
@@ -314,12 +392,6 @@ protected:
       return t_sec;
     }
 
-
-
-    virtual nsresult FindTarget(const PRUnichar *aTargetName,
-                                PRBool *aIsNewWindow,
-                                nsIDocShell **aResult);
-
     PRBool IsFrame();
 
     //
@@ -332,7 +404,93 @@ protected:
 
     nsresult CheckLoadingPermissions();
 
+    // Security checks to prevent frameset spoofing.  See comments at
+    // implementation sites.
+    static PRBool CanAccessItem(nsIDocShellTreeItem* aTargetItem,
+                                nsIDocShellTreeItem* aAccessingItem,
+                                PRBool aConsiderOpener = PR_TRUE);
+    static PRBool ValidateOrigin(nsIDocShellTreeItem* aOriginTreeItem,
+                                 nsIDocShellTreeItem* aTargetTreeItem);
+
+    // Returns PR_TRUE if would have called FireOnLocationChange,
+    // but did not because aFireOnLocationChange was false on entry.
+    // In this case it is the caller's responsibility to ensure
+    // FireOnLocationChange is called.
+    // In all other cases PR_FALSE is returned.
+    PRBool SetCurrentURI(nsIURI *aURI, nsIRequest *aRequest,
+                         PRBool aFireOnLocationChange);
+
+    // The following methods deal with saving and restoring content viewers
+    // in session history.
+
+    // mContentViewer points to the current content viewer associated with
+    // this docshell.  When loading a new document, the content viewer is
+    // either destroyed or stored into a session history entry.  To make sure
+    // that destruction happens in a controlled fashion, a given content viewer
+    // is always owned in exactly one of these ways:
+    //   1) The content viewer is active and owned by a docshell's
+    //      mContentViewer.
+    //   2) The content viewer is still being displayed while we begin loading
+    //      a new document.  The content viewer is owned by the _new_
+    //      content viewer's mPreviousViewer, and has a pointer to the
+    //      nsISHEntry where it will eventually be stored.  The content viewer
+    //      has been close()d by the docshell, which detaches the document from
+    //      the window object.
+    //   3) The content viewer is cached in session history.  The nsISHEntry
+    //      has the only owning reference to the content viewer.  The viewer
+    //      has released its nsISHEntry pointer to prevent circular ownership.
+    //
+    // When restoring a content viewer from session history, open() is called
+    // to reattach the document to the window object.  The content viewer is
+    // then placed into mContentViewer and removed from the history entry.
+    // (mContentViewer is put into session history as described above, if
+    // applicable).
+
+    // Determines whether we can safely cache the current mContentViewer in
+    // session history.  This checks a number of factors such as cache policy,
+    // pending requests, and unload handlers.
+    // |aLoadType| should be the load type that will replace the current
+    // presentation.  |aNewRequest| should be the request for the document to
+    // be loaded in place of the current document, or null if such a request
+    // has not been created yet. |aNewDocument| should be the document that will
+    // replace the current document.
+    PRBool CanSavePresentation(PRUint32 aLoadType,
+                               nsIRequest *aNewRequest,
+                               nsIDocument *aNewDocument);
+
+    // Captures the state of the supporting elements of the presentation
+    // (the "window" object, docshell tree, meta-refresh loads, and security
+    // state) and stores them on |mOSHE|.
+    nsresult CaptureState();
+
+    // Begin the toplevel restore process for |aSHEntry|.
+    // This simulates a channel open, and defers the real work until
+    // RestoreFromHistory is called from a PLEvent.
+    nsresult RestorePresentation(nsISHEntry *aSHEntry, PRBool *aRestoring);
+
+    // Call BeginRestore(nsnull, PR_FALSE) for each child of this shell.
+    nsresult BeginRestoreChildren();
+
+    // Check whether aURI should inherit our security context
+    static nsresult URIInheritsSecurityContext(nsIURI* aURI, PRBool* aResult);
+
+    // Check whether aURI is about:blank
+    static PRBool IsAboutBlank(nsIURI* aURI);
+    
 protected:
+    // Override the parent setter from nsDocLoader
+    virtual nsresult SetDocLoaderParent(nsDocLoader * aLoader);
+
+    // Event type dispatched by RestorePresentation
+    class RestorePresentationEvent : public nsRunnable {
+    public:
+        NS_DECL_NSIRUNNABLE
+        RestorePresentationEvent(nsDocShell *ds) : mDocShell(ds) {}
+        void Revoke() { mDocShell = nsnull; }
+    private:
+        nsDocShell *mDocShell;
+    };
+
     PRPackedBool               mAllowSubframes;
     PRPackedBool               mAllowPlugins;
     PRPackedBool               mAllowJavascript;
@@ -342,7 +500,9 @@ protected:
     PRPackedBool               mHasFocus;
     PRPackedBool               mCreatingDocument; // (should be) debugging only
     PRPackedBool               mUseErrorPages;
+    PRPackedBool               mObserveErrorPages;
     PRPackedBool               mAllowAuth;
+    PRPackedBool               mAllowKeywordFixup;
 
     PRPackedBool               mFiredUnloadEvent;
 
@@ -355,17 +515,16 @@ protected:
 
     PRPackedBool               mIsBeingDestroyed;
 
-    // Disallow popping up new windows with target=
-    PRPackedBool               mDisallowPopupWindows;
-
-    // Validate window targets to prevent frameset spoofing
-    PRPackedBool               mValidateOrigin;
-
     PRPackedBool               mIsExecutingOnLoadHandler;
 
     // Indicates that a DocShell in this "docshell tree" is printing
     PRPackedBool               mIsPrintingOrPP;
 
+    // Indicates to CreateContentViewer() that it is safe to cache the old
+    // presentation of the page, and to SetupNewViewer() that the old viewer
+    // should be passed a SHEntry to save itself into.
+    PRPackedBool               mSavingOldViewer;
+
     PRUint32                   mAppType;
 
     // Offset in the parent's child list.
@@ -386,22 +545,22 @@ protected:
      * session history entries.
      */
     nsCString                  mContentTypeHint;
-    nsVoidArray                mChildren;
     nsCOMPtr<nsISupportsArray> mRefreshURIList;
-    nsDSURIContentListener *   mContentListener;
+    nsCOMPtr<nsISupportsArray> mSavedRefreshURIList;
+    nsRefPtr<nsDSURIContentListener> mContentListener;
     nsRect                     mBounds; // Dimensions of the docshell
     nsCOMPtr<nsIContentViewer> mContentViewer;
     nsCOMPtr<nsIDocumentCharsetInfo> mDocumentCharsetInfo;
     nsCOMPtr<nsIDeviceContext> mDeviceContext;
     nsCOMPtr<nsIWidget>        mParentWidget;
     nsCOMPtr<nsIPrefBranch>    mPrefs;
+
+    // mCurrentURI should be marked immutable on set if possible.
     nsCOMPtr<nsIURI>           mCurrentURI;
     nsCOMPtr<nsIURI>           mReferrerURI;
     nsCOMPtr<nsIScriptGlobalObject> mScriptGlobal;
-    nsCOMPtr<nsIScriptContext> mScriptContext;
     nsCOMPtr<nsISHistory>      mSessionHistory;
     nsCOMPtr<nsIGlobalHistory2> mGlobalHistory;
-    nsCOMPtr<nsISupports>      mLoadCookie; // the load cookie associated with the window context.
     nsCOMPtr<nsIWebBrowserFind> mFind;
     nsPoint                    mDefaultScrollbarPref; // persistent across doc loads
     // Reference to the SHEntry for this docshell until the page is destroyed.
@@ -411,17 +570,32 @@ protected:
     // Somebody give me better name
     nsCOMPtr<nsISHEntry>       mLSHE;
 
+    // Holds a weak pointer to a RestorePresentationEvent object if any that
+    // holds a weak pointer back to us.  We use this pointer to possibly revoke
+    // the event whenever necessary.
+    nsRevocableEventPtr<RestorePresentationEvent> mRestorePresentationEvent;
+
+    // hash of session storages, keyed by domain
+    nsInterfaceHashtable<nsCStringHashKey, nsIDOMStorage> mStorages;
+
+    // Index into the SHTransaction list, indicating the previous and current
+    // transaction at the time that this DocShell begins to load
+    PRInt32                    mPreviousTransIndex;
+    PRInt32                    mLoadedTransIndex;
+
     // Editor stuff
     nsDocShellEditorData*      mEditorData;          // editor data, if any
 
     // Transferable hooks/callbacks
     nsCOMPtr<nsIClipboardDragDropHookList>  mTransferableHookData;
 
+    // Secure browser UI object
+    nsCOMPtr<nsISecureBrowserUI> mSecurityUI;
+
     // WEAK REFERENCES BELOW HERE.
     // Note these are intentionally not addrefd.  Doing so will create a cycle.
     // For that reasons don't use nsCOMPtr.
 
-    nsIDocShellTreeItem *      mParent;  // Weak Reference
     nsIDocShellTreeOwner *     mTreeOwner; // Weak Reference
     nsIChromeEventHandler *    mChromeEventHandler; //Weak Reference
 
