@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim:set ts=2 sw=2 sts=2 cin et: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -38,13 +39,11 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsDocShell.h"
-#include "nsIWebShell.h"
 #include "nsWebShell.h"
-#include "nsIWebBrowserChrome.h"
+#include "nsIWebBrowserChrome2.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIWebProgress.h"
-#include "nsIDocumentLoader.h"
 #include "nsIContentViewer.h"
 #include "nsIDocumentViewer.h"
 #include "nsIMarkupDocumentViewer.h"
@@ -53,20 +52,14 @@
 #include "nsIStreamListener.h"
 #include "nsIPrompt.h"
 #include "nsNetUtil.h"
-#include "nsIDNSService.h"
-#include "nsISocketProvider.h"
 #include "nsIRefreshURI.h"
-#include "nsIScriptGlobalObject.h"
-#include "nsIScriptGlobalObjectOwner.h"
 #include "nsIDOMEvent.h"
 #include "nsPresContext.h"
 #include "nsIComponentManager.h"
-#include "nsIEventQueueService.h"
 #include "nsCRT.h"
 #include "nsVoidArray.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
-#include "plevent.h"
 #include "prprf.h"
 #include "nsIPluginHost.h"
 #include "nsplugin.h"
@@ -106,23 +99,28 @@
 #include "nsISHistoryInternal.h"
 
 #include "nsIHttpChannel.h"
+#include "nsIHttpChannelInternal.h"
 #include "nsIUploadChannel.h"
 #include "nsISeekableStream.h"
+#include "nsStreamUtils.h"
+#include "nsThreadUtils.h"
 
 #include "nsILocaleService.h"
 #include "nsIStringBundle.h"
 
 #include "nsICachingChannel.h"
 
-//XXX for nsIPostData; this is wrong; we shouldn't see the nsIDocument type
 #include "nsIDocument.h"
 #include "nsITextToSubURI.h"
 
 #include "nsIExternalProtocolService.h"
 #include "nsCExternalHandlerService.h"
 
-// Used in the fixup code
-#include "prnetdb.h"
+#include "nsIIDNService.h"
+
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
+#include "nsITimer.h"
 
 #ifdef NS_DEBUG
 /**
@@ -148,16 +146,353 @@ static PRLogModuleInfo* gLogModule = PR_NewLogModule("webshell");
 #define WEB_TRACE(_bit,_args)
 #endif
 
+//-----------------------------------------------------------------------------
+
+#define PREF_PINGS_ENABLED           "browser.send_pings"
+#define PREF_PINGS_MAX_PER_LINK      "browser.send_pings.max_per_link"
+#define PREF_PINGS_REQUIRE_SAME_HOST "browser.send_pings.require_same_host"
+
+// Check prefs to see if pings are enabled and if so what restrictions might
+// be applied.
+//
+// @param maxPerLink
+//   This parameter returns the number of pings that are allowed per link click
+//
+// @param requireSameHost
+//   This parameter returns PR_TRUE if pings are restricted to the same host as
+//   the document in which the click occurs.  If the same host restriction is
+//   imposed, then we still allow for pings to cross over to different
+//   protocols and ports for flexibility and because it is not possible to send
+//   a ping via FTP.
+//
+// @returns
+//   PR_TRUE if pings are enabled and PR_FALSE otherwise.
+//
+static PRBool
+PingsEnabled(PRInt32 *maxPerLink, PRBool *requireSameHost)
+{
+  PRBool allow = PR_FALSE;
+
+  *maxPerLink = 1;
+  *requireSameHost = PR_TRUE;
+
+  nsCOMPtr<nsIPrefBranch> prefs =
+      do_GetService(NS_PREFSERVICE_CONTRACTID);
+  if (prefs) {
+    PRBool val;
+    if (NS_SUCCEEDED(prefs->GetBoolPref(PREF_PINGS_ENABLED, &val)))
+      allow = val;
+    if (allow) {
+      prefs->GetIntPref(PREF_PINGS_MAX_PER_LINK, maxPerLink);
+      prefs->GetBoolPref(PREF_PINGS_REQUIRE_SAME_HOST, requireSameHost);
+    }
+  }
+
+  return allow;
+}
+
+typedef void (* ForEachPingCallback)(void *closure, nsIContent *content,
+                                     nsIURI *uri, nsIIOService *ios);
+
+static void
+ForEachPing(nsIContent *content, ForEachPingCallback callback, void *closure)
+{
+  // NOTE: Using nsIDOMNSHTMLAnchorElement2::GetPing isn't really worth it here
+  //       since we'd still need to parse the resulting string.  Instead, we
+  //       just parse the raw attribute.  It might be nice if the content node
+  //       implemented an interface that exposed an enumeration of nsIURIs.
+
+  // Make sure we are dealing with either an <A> or <AREA> element in the HTML
+  // or XHTML namespace.
+  if (!content->IsNodeOfType(nsINode::eHTML))
+    return;
+  nsIAtom *nameAtom = content->Tag();
+  if (!nameAtom->EqualsUTF8(NS_LITERAL_CSTRING("a")) &&
+      !nameAtom->EqualsUTF8(NS_LITERAL_CSTRING("area")))
+    return;
+
+  nsCOMPtr<nsIAtom> pingAtom = do_GetAtom("ping");
+  if (!pingAtom)
+    return;
+
+  nsAutoString value;
+  content->GetAttr(kNameSpaceID_None, pingAtom, value);
+  if (value.IsEmpty())
+    return;
+
+  nsCOMPtr<nsIIOService> ios = do_GetIOService();
+  if (!ios)
+    return;
+
+  nsIDocument *doc = content->GetOwnerDoc();
+  if (!doc)
+    return;
+
+  // value contains relative URIs split on spaces (U+0020)
+  const PRUnichar *start = value.BeginReading();
+  const PRUnichar *end   = value.EndReading();
+  const PRUnichar *iter  = start;
+  for (;;) {
+    if (iter < end && *iter != ' ') {
+      ++iter;
+    } else {  // iter is pointing at either end or a space
+      while (*start == ' ' && start < iter)
+        ++start;
+      if (iter != start) {
+        nsCOMPtr<nsIURI> uri, baseURI = content->GetBaseURI();
+        ios->NewURI(NS_ConvertUTF16toUTF8(Substring(start, iter)),
+                    doc->GetDocumentCharacterSet().get(),
+                    baseURI, getter_AddRefs(uri));
+        if (uri) {
+          // Ignore non-HTTP(S) pings:
+          PRBool match;
+          if ((NS_SUCCEEDED(uri->SchemeIs("http", &match)) && match) ||
+              (NS_SUCCEEDED(uri->SchemeIs("https", &match)) && match)) {
+            callback(closure, content, uri, ios);
+          }
+        }
+      }
+      start = iter = iter + 1;
+      if (iter >= end)
+        break;
+    }
+  }
+}
+
 //----------------------------------------------------------------------
 
-static PRBool
-IsOffline()
+// We wait this many milliseconds before killing the ping channel...
+#define PING_TIMEOUT 10000
+
+static void
+OnPingTimeout(nsITimer *timer, void *closure)
 {
-    PRBool offline = PR_TRUE;
-    nsCOMPtr<nsIIOService> ios(do_GetIOService());
-    if (ios)
-        ios->GetOffline(&offline);
-    return offline;
+  nsILoadGroup *loadGroup = NS_STATIC_CAST(nsILoadGroup *, closure);
+  loadGroup->Cancel(NS_ERROR_ABORT);
+  loadGroup->Release();
+}
+
+// Check to see if two URIs have the same host or not
+static PRBool
+IsSameHost(nsIURI *uri1, nsIURI *uri2)
+{
+  nsCAutoString host1, host2;
+  uri1->GetAsciiHost(host1);
+  uri2->GetAsciiHost(host2);
+  return host1.Equals(host2);
+}
+
+class nsPingListener : public nsIStreamListener
+                     , public nsIInterfaceRequestor
+                     , public nsIChannelEventSink
+{
+public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIREQUESTOBSERVER
+  NS_DECL_NSISTREAMLISTENER
+  NS_DECL_NSIINTERFACEREQUESTOR
+  NS_DECL_NSICHANNELEVENTSINK
+
+  nsPingListener(PRBool requireSameHost)
+    : mRequireSameHost(requireSameHost)
+  {}
+
+private:
+  PRBool mRequireSameHost;
+};
+
+NS_IMPL_ISUPPORTS4(nsPingListener, nsIStreamListener, nsIRequestObserver,
+                   nsIInterfaceRequestor, nsIChannelEventSink)
+
+NS_IMETHODIMP
+nsPingListener::OnStartRequest(nsIRequest *request, nsISupports *context)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPingListener::OnDataAvailable(nsIRequest *request, nsISupports *context,
+                                nsIInputStream *stream, PRUint32 offset,
+                                PRUint32 count)
+{
+  PRUint32 result;
+  return stream->ReadSegments(NS_DiscardSegment, nsnull, count, &result);
+}
+
+NS_IMETHODIMP
+nsPingListener::OnStopRequest(nsIRequest *request, nsISupports *context,
+                              nsresult status)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPingListener::GetInterface(const nsIID &iid, void **result)
+{
+  if (iid.Equals(NS_GET_IID(nsIChannelEventSink))) {
+    NS_ADDREF_THIS();
+    *result = (nsIChannelEventSink *) this;
+    return NS_OK;
+  }
+
+  return NS_ERROR_NO_INTERFACE;
+}
+
+NS_IMETHODIMP
+nsPingListener::OnChannelRedirect(nsIChannel *oldChan, nsIChannel *newChan,
+                                  PRUint32 flags)
+{
+  if (!mRequireSameHost)
+    return NS_OK;
+
+  nsCOMPtr<nsIURI> oldURI, newURI;
+  oldChan->GetURI(getter_AddRefs(oldURI));
+  newChan->GetURI(getter_AddRefs(newURI));
+  NS_ENSURE_STATE(oldURI && newURI);
+
+  if (!IsSameHost(oldURI, newURI))
+    return NS_ERROR_ABORT;
+
+  return NS_OK;
+}
+
+struct SendPingInfo {
+  PRInt32 numPings;
+  PRInt32 maxPings;
+  PRBool  requireSameHost;
+  nsIURI *referrer;
+};
+
+static void
+SendPing(void *closure, nsIContent *content, nsIURI *uri, nsIIOService *ios)
+{
+  SendPingInfo *info = NS_STATIC_CAST(SendPingInfo *, closure);
+  if (info->numPings >= info->maxPings)
+    return;
+
+  if (info->requireSameHost) {
+    // Make sure the referrer and the given uri share the same origin.  We
+    // only require the same hostname.  The scheme and port may differ.
+    if (!IsSameHost(uri, info->referrer))
+      return;
+  }
+
+  nsIDocument *doc = content->GetOwnerDoc();
+  if (!doc)
+    return;
+
+  nsCOMPtr<nsIChannel> chan;
+  ios->NewChannelFromURI(uri, getter_AddRefs(chan));
+  if (!chan)
+    return;
+
+  // Don't bother caching the result of this URI load.
+  chan->SetLoadFlags(nsIRequest::INHIBIT_CACHING);
+
+  nsCOMPtr<nsIHttpChannel> httpChan = do_QueryInterface(chan);
+  if (!httpChan)
+    return;
+
+  // This is needed in order for 3rd-party cookie blocking to work.
+  nsCOMPtr<nsIHttpChannelInternal> httpInternal = do_QueryInterface(httpChan);
+  if (httpInternal)
+    httpInternal->SetDocumentURI(doc->GetDocumentURI());
+
+  if (info->referrer)
+    httpChan->SetReferrer(info->referrer);
+
+  httpChan->SetRequestMethod(NS_LITERAL_CSTRING("POST"));
+
+  // Remove extraneous request headers (to reduce request size)
+  httpChan->SetRequestHeader(NS_LITERAL_CSTRING("accept"),
+                             EmptyCString(), PR_FALSE);
+  httpChan->SetRequestHeader(NS_LITERAL_CSTRING("accept-language"),
+                             EmptyCString(), PR_FALSE);
+  httpChan->SetRequestHeader(NS_LITERAL_CSTRING("accept-charset"),
+                             EmptyCString(), PR_FALSE);
+  httpChan->SetRequestHeader(NS_LITERAL_CSTRING("accept-encoding"),
+                             EmptyCString(), PR_FALSE);
+
+  nsCOMPtr<nsIUploadChannel> uploadChan = do_QueryInterface(httpChan);
+  if (!uploadChan)
+    return;
+
+  // To avoid sending an unnecessary Content-Type header, we encode the
+  // closing portion of the headers in the POST body.
+  NS_NAMED_LITERAL_CSTRING(uploadData, "Content-Length: 0\r\n\r\n");
+
+  nsCOMPtr<nsIInputStream> uploadStream;
+  NS_NewPostDataStream(getter_AddRefs(uploadStream), PR_FALSE,
+                       uploadData, 0);
+  if (!uploadStream)
+    return;
+
+  uploadChan->SetUploadStream(uploadStream, EmptyCString(), -1);
+
+  // The channel needs to have a loadgroup associated with it, so that we can
+  // cancel the channel and any redirected channels it may create.
+  nsCOMPtr<nsILoadGroup> loadGroup =
+      do_CreateInstance(NS_LOADGROUP_CONTRACTID);
+  if (!loadGroup)
+    return;
+  chan->SetLoadGroup(loadGroup);
+
+  // Construct a listener that merely discards any response.  If successful at
+  // opening the channel, then it is not necessary to hold a reference to the
+  // channel.  The networking subsystem will take care of that for us.
+  nsCOMPtr<nsIStreamListener> listener =
+      new nsPingListener(info->requireSameHost);
+  if (!listener)
+    return;
+
+  // Observe redirects as well:
+  nsCOMPtr<nsIInterfaceRequestor> callbacks = do_QueryInterface(listener);
+  NS_ASSERTION(callbacks, "oops");
+  loadGroup->SetNotificationCallbacks(callbacks);
+
+  chan->AsyncOpen(listener, nsnull);
+
+  // Even if AsyncOpen failed, we still count this as a successful ping.  It's
+  // possible that AsyncOpen may have failed after triggering some background
+  // process that may have written something to the network.
+  info->numPings++;
+
+  // Prevent ping requests from stalling and never being garbage collected...
+  nsCOMPtr<nsITimer> timer =
+      do_CreateInstance(NS_TIMER_CONTRACTID);
+  if (timer) {
+    nsresult rv = timer->InitWithFuncCallback(OnPingTimeout, loadGroup,
+                                              PING_TIMEOUT,
+                                              nsITimer::TYPE_ONE_SHOT);
+    if (NS_SUCCEEDED(rv)) {
+      // When the timer expires, the callback function will release this
+      // reference to the loadgroup.
+      NS_STATIC_CAST(nsILoadGroup *, loadGroup.get())->AddRef();
+      loadGroup = 0;
+    }
+  }
+  
+  // If we failed to setup the timer, then we should just cancel the channel
+  // because we won't be able to ensure that it goes away in a timely manner.
+  if (loadGroup)
+    chan->Cancel(NS_ERROR_ABORT);
+}
+
+// Spec: http://whatwg.org/specs/web-apps/current-work/#ping
+static void
+DispatchPings(nsIContent *content, nsIURI *referrer)
+{
+  SendPingInfo info;
+
+  if (!PingsEnabled(&info.maxPings, &info.requireSameHost))
+    return;
+  if (info.maxPings == 0)
+    return;
+
+  info.numPings = 0;
+  info.referrer = referrer;
+
+  ForEachPing(content, SendPing, &info);
 }
 
 //----------------------------------------------------------------------
@@ -170,10 +505,9 @@ nsWebShell::nsWebShell() : nsDocShell()
   ++gNumberOfWebShells;
 #endif
 #ifdef DEBUG
-    printf("++WEBSHELL == %ld\n", gNumberOfWebShells);
+    printf("++WEBSHELL %p == %ld\n", (void*) this, gNumberOfWebShells);
 #endif
 
-  mThread = nsnull;
   InitFrameData();
   mItemType = typeContent;
   mCharsetReloadState = eCharsetReloadInit;
@@ -183,31 +517,11 @@ nsWebShell::~nsWebShell()
 {
    Destroy();
 
-  // Stop any pending document loads and destroy the loader...
-  if (mDocLoader) {
-    mDocLoader->Stop();
-    mDocLoader->SetContainer(nsnull);
-    mDocLoader->Destroy();
-    mDocLoader = nsnull;
-  }
-  // Cancel any timers that were set for this loader.
-  CancelRefreshURITimers();
-
   ++mRefCnt; // following releases can cause this destructor to be called
              // recursively if the refcount is allowed to remain 0
 
   mContentViewer=nsnull;
   mDeviceContext=nsnull;
-  NS_IF_RELEASE(mContainer);
-
-  if (mScriptGlobal) {
-    mScriptGlobal->SetDocShell(nsnull);
-    mScriptGlobal = nsnull;
-  }
-  if (mScriptContext) {
-    mScriptContext->SetOwner(nsnull);
-    mScriptContext = nsnull;
-  }
 
   InitFrameData();
 
@@ -216,7 +530,7 @@ nsWebShell::~nsWebShell()
   --gNumberOfWebShells;
 #endif
 #ifdef DEBUG
-  printf("--WEBSHELL == %ld\n", gNumberOfWebShells);
+  printf("--WEBSHELL %p == %ld\n", (void*) this, gNumberOfWebShells);
 #endif
 }
 
@@ -254,9 +568,7 @@ NS_IMPL_ADDREF_INHERITED(nsWebShell, nsDocShell)
 NS_IMPL_RELEASE_INHERITED(nsWebShell, nsDocShell)
 
 NS_INTERFACE_MAP_BEGIN(nsWebShell)
-   NS_INTERFACE_MAP_ENTRY(nsIWebShell)
    NS_INTERFACE_MAP_ENTRY(nsIWebShellServices)
-   NS_INTERFACE_MAP_ENTRY(nsIWebShellContainer)
    NS_INTERFACE_MAP_ENTRY(nsILinkHandler)
    NS_INTERFACE_MAP_ENTRY(nsIClipboardCommands)
 NS_INTERFACE_MAP_END_INHERITING(nsDocShell)
@@ -264,73 +576,19 @@ NS_INTERFACE_MAP_END_INHERITING(nsDocShell)
 NS_IMETHODIMP
 nsWebShell::GetInterface(const nsIID &aIID, void** aInstancePtr)
 {
-   NS_ENSURE_ARG_POINTER(aInstancePtr);
-   nsresult rv = NS_OK;
+   NS_PRECONDITION(aInstancePtr, "null out param");
+
    *aInstancePtr = nsnull;
 
-   if(aIID.Equals(NS_GET_IID(nsILinkHandler)))
-      {
-      // Note: If we ever allow for registering other link handlers,
-      // we need to make sure that link handler implementations take
-      // the necessary precautions to prevent the security compromise
-      // that is blocked by nsWebSell::OnLinkClickSync().
-
-      *aInstancePtr = NS_STATIC_CAST(nsILinkHandler*, this);
-      NS_ADDREF((nsISupports*)*aInstancePtr);
-      return NS_OK;
-      }
-   else if(aIID.Equals(NS_GET_IID(nsIScriptGlobalObjectOwner)))
-      {
-      *aInstancePtr = NS_STATIC_CAST(nsIScriptGlobalObjectOwner*, this);
-      NS_ADDREF((nsISupports*)*aInstancePtr);
-      return NS_OK;
-      }
-   else if(aIID.Equals(NS_GET_IID(nsIScriptGlobalObject)))
-      {
-      NS_ENSURE_SUCCESS(EnsureScriptEnvironment(), NS_ERROR_FAILURE);
-      *aInstancePtr = mScriptGlobal;
-      NS_ADDREF((nsISupports*)*aInstancePtr);
-      return NS_OK;
-      }
-   else if(aIID.Equals(NS_GET_IID(nsIDOMWindowInternal)) ||
-           aIID.Equals(NS_GET_IID(nsIDOMWindow)))
-
-      {
-      NS_ENSURE_SUCCESS(EnsureScriptEnvironment(), NS_ERROR_FAILURE);
-      NS_ENSURE_SUCCESS(mScriptGlobal->QueryInterface(aIID, aInstancePtr),
-                        NS_ERROR_FAILURE);
-      return NS_OK;
-      }
-   else if(aIID.Equals(NS_GET_IID(nsICommandManager)))
+   if(aIID.Equals(NS_GET_IID(nsICommandManager)))
       {
       NS_ENSURE_SUCCESS(EnsureCommandHandler(), NS_ERROR_FAILURE);
-      NS_ENSURE_SUCCESS(mCommandManager->QueryInterface(NS_GET_IID(nsICommandManager),
-         aInstancePtr), NS_ERROR_FAILURE);
+      *aInstancePtr = mCommandManager;
+      NS_ADDREF((nsISupports*) *aInstancePtr);
       return NS_OK;
       }
 
-   if (!*aInstancePtr || NS_FAILED(rv))
-     return nsDocShell::GetInterface(aIID,aInstancePtr);
-   else
-     return rv;
-}
-
-NS_IMETHODIMP
-nsWebShell::SetContainer(nsIWebShellContainer* aContainer)
-{
-  NS_IF_RELEASE(mContainer);
-  mContainer = aContainer;
-  NS_IF_ADDREF(mContainer);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsWebShell::GetContainer(nsIWebShellContainer*& aResult)
-{
-  aResult = mContainer;
-  NS_IF_ADDREF(mContainer);
-  return NS_OK;
+   return nsDocShell::GetInterface(aIID, aInstancePtr);
 }
 
 nsEventStatus PR_CALLBACK
@@ -340,19 +598,6 @@ nsWebShell::HandleEvent(nsGUIEvent *aEvent)
 }
 
 
-/**
- * Document Load methods
- */
-NS_IMETHODIMP
-nsWebShell::GetDocumentLoader(nsIDocumentLoader*& aResult)
-{
-  aResult = mDocLoader;
-  if (!mDocLoader)
-    return NS_ERROR_FAILURE;
-  NS_ADDREF(aResult);
-  return NS_OK;
-}
-
 //----------------------------------------------------------------------
 // Web Shell Services API
 
@@ -420,14 +665,15 @@ nsWebShell::SetRendering(PRBool aRender)
 
 // WebShell link handling
 
-struct OnLinkClickEvent : public PLEvent {
+class OnLinkClickEvent : public nsRunnable {
+public:
   OnLinkClickEvent(nsWebShell* aHandler, nsIContent* aContent,
                    nsLinkVerb aVerb, nsIURI* aURI,
-                   const PRUnichar* aTargetSpec, nsIInputStream* aPostDataStream = 0, 
+                   const PRUnichar* aTargetSpec,
+                   nsIInputStream* aPostDataStream = 0, 
                    nsIInputStream* aHeadersDataStream = 0);
-  ~OnLinkClickEvent();
 
-  void HandleEvent() {
+  NS_IMETHOD Run() {
     nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(mHandler->mScriptGlobal));
     nsAutoPopupStatePusher popupStatePusher(window, mPopupState);
 
@@ -435,9 +681,11 @@ struct OnLinkClickEvent : public PLEvent {
                               mTargetSpec.get(), mPostDataStream,
                               mHeadersDataStream,
                               nsnull, nsnull);
+    return NS_OK;
   }
 
-  nsWebShell*              mHandler;
+private:
+  nsRefPtr<nsWebShell>     mHandler;
   nsCOMPtr<nsIURI>         mURI;
   nsString                 mTargetSpec;
   nsCOMPtr<nsIInputStream> mPostDataStream;
@@ -447,16 +695,6 @@ struct OnLinkClickEvent : public PLEvent {
   PopupControlState        mPopupState;
 };
 
-static void PR_CALLBACK HandlePLEvent(OnLinkClickEvent* aEvent)
-{
-  aEvent->HandleEvent();
-}
-
-static void PR_CALLBACK DestroyPLEvent(OnLinkClickEvent* aEvent)
-{
-  delete aEvent;
-}
-
 OnLinkClickEvent::OnLinkClickEvent(nsWebShell* aHandler,
                                    nsIContent *aContent,
                                    nsLinkVerb aVerb,
@@ -464,34 +702,17 @@ OnLinkClickEvent::OnLinkClickEvent(nsWebShell* aHandler,
                                    const PRUnichar* aTargetSpec,
                                    nsIInputStream* aPostDataStream,
                                    nsIInputStream* aHeadersDataStream)
+  : mHandler(aHandler)
+  , mURI(aURI)
+  , mTargetSpec(aTargetSpec)
+  , mPostDataStream(aPostDataStream)
+  , mHeadersDataStream(aHeadersDataStream)
+  , mContent(aContent)
+  , mVerb(aVerb)
 {
-  mHandler = aHandler;
-  NS_ADDREF(aHandler);
-  mURI = aURI;
-  mTargetSpec.Assign(aTargetSpec);
-  mPostDataStream = aPostDataStream;
-  mHeadersDataStream = aHeadersDataStream;
-  mContent = aContent;
-  mVerb = aVerb;
-
   nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(mHandler->mScriptGlobal));
 
   mPopupState = window->GetPopupControlState();
-
-  PL_InitEvent(this, nsnull,
-               (PLHandleEventProc) ::HandlePLEvent,
-               (PLDestroyEventProc) ::DestroyPLEvent);
-
-  nsCOMPtr<nsIEventQueue> eventQueue;
-  aHandler->GetEventQueue(getter_AddRefs(eventQueue));
-  NS_ASSERTION(eventQueue, "no event queue");
-  if (eventQueue)
-    eventQueue->PostEvent(this);
-}
-
-OnLinkClickEvent::~OnLinkClickEvent()
-{
-  NS_IF_RELEASE(mHandler);
 }
 
 //----------------------------------------
@@ -504,26 +725,11 @@ nsWebShell::OnLinkClick(nsIContent* aContent,
                         nsIInputStream* aPostDataStream,
                         nsIInputStream* aHeadersDataStream)
 {
-  OnLinkClickEvent* ev;
-
-  ev = new OnLinkClickEvent(this, aContent, aVerb, aURI,
-                            aTargetSpec, aPostDataStream, aHeadersDataStream);
-  if (!ev) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  return NS_OK;
-}
-
-nsresult
-nsWebShell::GetEventQueue(nsIEventQueue **aQueue)
-{
-  NS_ENSURE_ARG_POINTER(aQueue);
-  *aQueue = 0;
-
-  nsCOMPtr<nsIEventQueueService> eventService(do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID));
-  if (eventService)
-    eventService->GetThreadEventQueue(mThread, aQueue);
-  return *aQueue ? NS_OK : NS_ERROR_FAILURE;
+  NS_ASSERTION(NS_IsMainThread(), "wrong thread");
+  nsCOMPtr<nsIRunnable> ev =
+      new OnLinkClickEvent(this, aContent, aVerb, aURI, aTargetSpec,
+                           aPostDataStream, aHeadersDataStream);
+  return NS_DispatchToCurrentThread(ev);
 }
 
 NS_IMETHODIMP
@@ -557,19 +763,15 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
   nsCOMPtr<nsIDOMNode> node(do_QueryInterface(aContent));
   NS_ENSURE_TRUE(node, NS_ERROR_UNEXPECTED);
 
-  PRBool isJS = PR_FALSE;
-  PRBool isData = PR_FALSE;
-
-  aURI->SchemeIs("javascript", &isJS);
-  aURI->SchemeIs("data", &isData);
-
-  if (isJS || isData) {
+  PRBool inherit;
+  nsresult rv = URIInheritsSecurityContext(aURI, &inherit);
+  if (NS_FAILED(rv) || inherit) {
     nsCOMPtr<nsIDocument> sourceDoc = aContent->GetDocument();
 
     if (!sourceDoc) {
       // The source is in a 'zombie' document, or not part of a
-      // document any more. Don't let it execute any javascript in the
-      // new document.
+      // document any more. Don't let it perform loads in this docshell.
+      // XXXbz why only for the inherit case?
 
       return NS_OK;
     }
@@ -579,8 +781,8 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
     NS_ENSURE_TRUE(presShell, NS_ERROR_FAILURE);
 
     if (presShell->GetDocument() != sourceDoc) {
-      // The source is not in the current document, don't let it
-      // execute any javascript in the current document.
+      // The source is not in the current document, don't let it load anything
+      // that would inherit the principals of the current document.
 
       return NS_OK;
     }
@@ -621,25 +823,29 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
 
   switch(aVerb) {
     case eLinkVerb_New:
+      NS_ASSERTION(target.IsEmpty(), "Losing window name information");
       target.AssignLiteral("_blank");
       // Fall into replace case
     case eLinkVerb_Undefined:
       // Fall through, this seems like the most reasonable action
     case eLinkVerb_Replace:
       {
-        return InternalLoad(aURI,               // New URI
-                            referer,            // Referer URI
-                            nsnull,             // No onwer
-                            PR_TRUE,            // Inherit owner from document
-                            target.get(),       // Window target
-                            NS_LossyConvertUCS2toASCII(typeHint).get(),
-                            aPostDataStream,    // Post data stream
-                            aHeadersDataStream, // Headers stream
-                            LOAD_LINK,          // Load type
-                            nsnull,             // No SHEntry
-                            PR_TRUE,            // first party site
-                            aDocShell,          // DocShell out-param
-                            aRequest);          // Request out-param
+        rv = InternalLoad(aURI,               // New URI
+                          referer,            // Referer URI
+                          nsnull,             // No onwer
+                          INTERNAL_LOAD_FLAGS_INHERIT_OWNER, // Inherit owner from document
+                          target.get(),       // Window target
+                          NS_LossyConvertUTF16toASCII(typeHint).get(),
+                          aPostDataStream,    // Post data stream
+                          aHeadersDataStream, // Headers stream
+                          LOAD_LINK,          // Load type
+                          nsnull,             // No SHEntry
+                          PR_TRUE,            // first party site
+                          aDocShell,          // DocShell out-param
+                          aRequest);          // Request out-param
+        if (NS_SUCCEEDED(rv)) {
+          DispatchPings(aContent, referer);
+        }
       }
       break;
     case eLinkVerb_Embed:
@@ -647,8 +853,9 @@ nsWebShell::OnLinkClickSync(nsIContent *aContent,
       //          in NS 4.x
     default:
       NS_ABORT_IF_FALSE(0,"unexpected link verb");
-      return NS_ERROR_UNEXPECTED;
+      rv = NS_ERROR_UNEXPECTED;
   }
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -656,27 +863,40 @@ nsWebShell::OnOverLink(nsIContent* aContent,
                        nsIURI* aURI,
                        const PRUnichar* aTargetSpec)
 {
-  nsCOMPtr<nsIWebBrowserChrome> browserChrome(do_GetInterface(mTreeOwner));
+  nsCOMPtr<nsIWebBrowserChrome2> browserChrome2 = do_GetInterface(mTreeOwner);
   nsresult rv = NS_ERROR_FAILURE;
 
-  if (browserChrome)  {
-    nsCOMPtr<nsITextToSubURI> textToSubURI = do_GetService(NS_ITEXTTOSUBURI_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
+  nsCOMPtr<nsIWebBrowserChrome> browserChrome;
+  if (!browserChrome2) {
+    browserChrome = do_GetInterface(mTreeOwner);
+    if (!browserChrome)
+      return rv;
+  }
 
-    // use url origin charset to unescape the URL
-    nsCAutoString charset;
-    rv = aURI->GetOriginCharset(charset);
-    NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsITextToSubURI> textToSubURI =
+      do_GetService(NS_ITEXTTOSUBURI_CONTRACTID, &rv);
+  if (NS_FAILED(rv))
+    return rv;
 
-    nsCAutoString spec;
-    rv = aURI->GetSpec(spec);
-    NS_ENSURE_SUCCESS(rv, rv);
+  // use url origin charset to unescape the URL
+  nsCAutoString charset;
+  rv = aURI->GetOriginCharset(charset);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    nsAutoString uStr;
-    rv = textToSubURI->UnEscapeURIForUI(charset, spec, uStr);    
+  nsCAutoString spec;
+  rv = aURI->GetSpec(spec);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-    if (NS_SUCCEEDED(rv))
-      rv = browserChrome->SetStatus(nsIWebBrowserChrome::STATUS_LINK, uStr.get());
+  nsAutoString uStr;
+  rv = textToSubURI->UnEscapeURIForUI(charset, spec, uStr);    
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (browserChrome2) {
+    nsCOMPtr<nsIDOMElement> element = do_QueryInterface(aContent);
+    rv = browserChrome2->SetStatusWithContext(nsIWebBrowserChrome::STATUS_LINK,
+                                              uStr, element);
+  } else {
+    rv = browserChrome->SetStatus(nsIWebBrowserChrome::STATUS_LINK, uStr.get());
   }
   return rv;
 }
@@ -738,12 +958,17 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
   else 
     mCharsetReloadState = eCharsetReloadInit;
 
+  // Save a pointer to the currently-loading history entry.
+  // nsDocShell::EndPageLoad will clear mLSHE, but we may need this history
+  // entry further down in this method.
+  nsCOMPtr<nsISHEntry> loadingSHE = mLSHE;
+  
   //
   // one of many safeguards that prevent death and destruction if
   // someone is so very very rude as to bring this window down
   // during this load handler.
   //
-  nsCOMPtr<nsIWebShell> kungFuDeathGrip(this);
+  nsCOMPtr<nsIDocShell> kungFuDeathGrip(this);
   nsDocShell::EndPageLoad(aProgress, channel, aStatus);
 
   // Test if this is the top frame or a subframe
@@ -758,18 +983,22 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
   //
   // If the page load failed, then deal with the error condition...
   // Errors are handled as follows:
-  //   1. Check to see if it a file not found error.
+  //   1. Check to see if it's a file not found error or bad content
+  //      encoding error.
   //   2. Send the URI to a keyword server (if enabled)
   //   3. If the error was DNS failure, then add www and .com to the URI
   //      (if appropriate).
   //   4. Throw an error dialog box...
   //
 
-  if(url && NS_FAILED(aStatus)) {
-    if (aStatus == NS_ERROR_FILE_NOT_FOUND) {
-      DisplayLoadError(aStatus, url, nsnull);
+  if (url && NS_FAILED(aStatus))
+  {
+    if (aStatus == NS_ERROR_FILE_NOT_FOUND ||
+        aStatus == NS_ERROR_INVALID_CONTENT_ENCODING)
+    {
+      DisplayLoadError(aStatus, url, nsnull, channel);
       return NS_OK;
-    }  
+    }
 
     if (sURIFixup)
     {
@@ -784,7 +1013,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
       //
       // First try keyword fixup
       //
-      if (aStatus == NS_ERROR_UNKNOWN_HOST)  
+      if (aStatus == NS_ERROR_UNKNOWN_HOST && mAllowKeywordFixup)
       {
         PRBool keywordsEnabled = PR_FALSE;
 
@@ -813,19 +1042,28 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
             keywordsEnabled = PR_FALSE;
         }
 
-        // Don't perform fixup on an IP address
-        PRNetAddr addr;
-        if(PR_StringToNetAddr(host.get(), &addr) == PR_SUCCESS) {
-            keywordsEnabled = PR_FALSE;
-        }
-
-        if(keywordsEnabled && (-1 == dotLoc)) {
+        if(keywordsEnabled && (kNotFound == dotLoc)) {
           // only send non-qualified hosts to the keyword server
-          nsCAutoString keywordSpec("keyword:");
-          keywordSpec += host;
-
-          NS_NewURI(getter_AddRefs(newURI),
-                    keywordSpec, nsnull);
+          //
+          // If this string was passed through nsStandardURL by chance, then it
+          // may have been converted from UTF-8 to ACE, which would result in a
+          // completely bogus keyword query.  Here we try to recover the
+          // original Unicode value, but this is not 100% correct since the
+          // value may have been normalized per the IDN normalization rules.
+          //
+          // Since we don't have access to the exact original string that was
+          // entered by the user, this will just have to do.
+          //
+          PRBool isACE;
+          nsCAutoString utf8Host;
+          nsCOMPtr<nsIIDNService> idnSrv =
+              do_GetService(NS_IDNSERVICE_CONTRACTID);
+          if (idnSrv &&
+              NS_SUCCEEDED(idnSrv->IsACE(host, &isACE)) && isACE &&
+              NS_SUCCEEDED(idnSrv->ConvertACEtoUTF8(host, utf8Host)))
+            sURIFixup->KeywordToURI(utf8Host, getter_AddRefs(newURI));
+          else
+            sURIFixup->KeywordToURI(host, getter_AddRefs(newURI));
         } // end keywordsEnabled
       }
 
@@ -880,7 +1118,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
         {
           nsCAutoString newSpec;
           newURI->GetSpec(newSpec);
-          NS_ConvertUTF8toUCS2 newSpecW(newSpec);
+          NS_ConvertUTF8toUTF16 newSpecW(newSpec);
 
           // This seems evil, since it is modifying the original URL
           rv = url->SetSpec(newSpec);
@@ -906,7 +1144,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
          aStatus == NS_ERROR_UNKNOWN_PROXY_HOST || 
          aStatus == NS_ERROR_PROXY_CONNECTION_REFUSED) &&
             (isTopFrame || mUseErrorPages)) {
-      DisplayLoadError(aStatus, url, nsnull);
+      DisplayLoadError(aStatus, url, nsnull, channel);
     }
     // Errors to be shown for any frame
     else if (aStatus == NS_ERROR_NET_TIMEOUT ||
@@ -914,7 +1152,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
              aStatus == NS_ERROR_UNKNOWN_SOCKET_TYPE ||
              aStatus == NS_ERROR_NET_INTERRUPT ||
              aStatus == NS_ERROR_NET_RESET) {
-      DisplayLoadError(aStatus, url, nsnull);
+      DisplayLoadError(aStatus, url, nsnull, channel);
     }
     else if (aStatus == NS_ERROR_DOCUMENT_NOT_CACHED) {
       /* A document that was requested to be fetched *only* from
@@ -928,7 +1166,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
       nsCAutoString method;
       if (httpChannel)
         httpChannel->GetRequestMethod(method);
-      if (method.Equals("POST") && !IsOffline()) {
+      if (method.Equals("POST") && !NS_IsOffline()) {
         nsCOMPtr<nsIPrompt> prompter;
         PRBool repost;
         nsCOMPtr<nsIStringBundle> stringBundle;
@@ -970,9 +1208,25 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
 
             if (rootSH && (mLoadType & LOAD_CMD_HISTORY)) {
               nsCOMPtr<nsISHistoryInternal> shInternal(do_QueryInterface(rootSH));
-              if (shInternal)
-               shInternal->UpdateIndex();
+              if (shInternal) {
+                rootSH->GetIndex(&mPreviousTransIndex);
+                shInternal->UpdateIndex();
+                rootSH->GetIndex(&mLoadedTransIndex);
+#ifdef DEBUG_PAGE_CACHE
+                printf("Previous index: %d, Loaded index: %d\n\n",
+                       mPreviousTransIndex, mLoadedTransIndex);
+#endif
+              }
             }
+
+            // Make it look like we really did honestly finish loading the
+            // history page we were loading, since the "reload" load we're
+            // about to kick off will reload our current history entry.  This
+            // is a bit of a hack, and if the force-load fails I think we'll
+            // end up being confused about what page we're on... but we would
+            // anyway, since we've updated the session history index above.
+            SetHistoryEntry(&mOSHE, loadingSHE);
+            
             /* The user does want to repost the data to the server.
              * Initiate a new load again.
              */
@@ -998,7 +1252,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
             InternalLoad(url,                               // URI
                          referrer,                          // Referring URI
                          nsnull,                            // Owner
-                         PR_TRUE,                           // Inherit owner
+                         INTERNAL_LOAD_FLAGS_INHERIT_OWNER, // Inherit owner
                          nsnull,                            // No window target
                          nsnull,                            // No type hint
                          inputStream,                       // Post data stream
@@ -1012,7 +1266,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
         }
       }
       else {
-        DisplayLoadError(aStatus, url, nsnull);
+        DisplayLoadError(aStatus, url, nsnull, channel);
       }
     }
   } // if we have a host
@@ -1183,45 +1437,17 @@ nsWebShell::SelectNone(void)
 
 NS_IMETHODIMP nsWebShell::Create()
 {
-  // Remember the current thread (in current and forseeable implementations,
-  // it'll just be the unique UI thread)
-  //
-  // Since this call must be made on the UI thread, we know the Event Queue
-  // will be associated with the current thread...
-  //
-  mThread = PR_GetCurrentThread();
-
+  if (mPrefs) {
+    // We've already been created
+    return NS_OK;
+  }
+  
   WEB_TRACE(WEB_TRACE_CALLS,
             ("nsWebShell::Init: this=%p", this));
 
-  // HACK....force the uri loader to give us a load cookie for this webshell...then get its
-  // doc loader and store it...as more of the docshell lands, we'll be able to get rid
-  // of this hack...
-  nsresult rv;
-  nsCOMPtr<nsIURILoader> uriLoader = do_GetService(NS_URI_LOADER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = uriLoader->GetDocumentLoaderForContext(this,
-                                              getter_AddRefs(mDocLoader));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIContentViewerContainer> shellAsContainer;
-  CallQueryInterface(this, NS_STATIC_CAST(nsIContentViewerContainer**, getter_AddRefs(shellAsContainer)));
-  // Set the webshell as the default IContentViewerContainer for the loader...
-  mDocLoader->SetContainer(shellAsContainer);
-
   return nsDocShell::Create();
 }
 
-NS_IMETHODIMP nsWebShell::Destroy()
-{
-  nsDocShell::Destroy();
-
-  SetContainer(nsnull);
-
-  return NS_OK;
-}
-
 #ifdef DEBUG
 unsigned long nsWebShell::gNumberOfWebShells = 0;
 #endif
