@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -43,14 +44,19 @@
 #include "nsIXPCScriptable.h"
 #include "jsapi.h"
 #include "nsIScriptSecurityManager.h"
+#include "nsIScriptContext.h"
+#include "nsDOMJSUtils.h" // for GetScriptContextFromJSContext
 
 class nsIDOMWindow;
 class nsIDOMNSHTMLOptionCollection;
 class nsIPluginInstance;
 class nsIForm;
+class nsIDOMNode;
 class nsIDOMNodeList;
 class nsIDOMDocument;
+class nsIDOMGCParticipant;
 class nsIHTMLDocument;
+class nsGlobalWindow;
 
 struct nsDOMClassInfoData;
 
@@ -127,13 +133,130 @@ public:
 
   static nsresult WrapNative(JSContext *cx, JSObject *scope,
                              nsISupports *native, const nsIID& aIID,
-                             jsval *vp);
+                             jsval *vp,
+                             // aHolder keeps the jsval alive while
+                             // there's a ref to it
+                             nsIXPConnectJSObjectHolder** aHolder);
   static nsresult ThrowJSException(JSContext *cx, nsresult aResult);
 
   static nsresult InitDOMJSClass(JSContext *cx, JSObject *obj);
 
   static JSClass sDOMJSClass;
 
+  /**
+   * Get our JSClass pointer for the XPCNativeWrapper class
+   */
+  static const JSClass* GetXPCNativeWrapperClass() {
+    return sXPCNativeWrapperClass;
+  }
+  
+  /**
+   * Set our JSClass pointer for the XPCNativeWrapper class
+   */
+  static void SetXPCNativeWrapperClass(JSClass* aClass) {
+    NS_ASSERTION(!sXPCNativeWrapperClass,
+                 "Double set of sXPCNativeWrapperClass");
+    sXPCNativeWrapperClass = aClass;
+  }
+
+  static PRBool ObjectIsNativeWrapper(JSContext* cx, JSObject* obj)
+  {
+#ifdef DEBUG
+    {
+      nsIScriptContext *scx = GetScriptContextFromJSContext(cx);
+
+      NS_PRECONDITION(!scx || !scx->IsContextInitialized() ||
+                      sXPCNativeWrapperClass,
+                      "Must know what the XPCNativeWrapper class is!");
+    }
+#endif
+
+    return sXPCNativeWrapperClass &&
+      ::JS_GetClass(cx, obj) == sXPCNativeWrapperClass;
+  }
+
+  /**
+   * Note that the XPConnect wrapper should be protected from garbage
+   * collection as long as the GC participant is reachable.
+   *
+   * A preservation with a given key overwrites any previous
+   * preservation with that key.
+   *
+   * No strong references are held as a result of this function call, so
+   * the caller is responsible for calling |ReleaseWrapper| sometime
+   * before |aParticipant|'s destructor runs.
+   *
+   * aRootWhenExternallyReferenced should be true if the preservation is
+   * for an event handler that could be triggered by the participant
+   * being externally referenced by a network load.
+   */
+  static nsresult PreserveWrapper(void* aKey,
+                                  nsIXPConnectJSObjectHolder* (*aKeyToWrapperFunc)(void* aKey),
+                                  nsIDOMGCParticipant *aParticipant,
+                                  PRBool aRootWhenExternallyReferenced);
+
+
+  /**
+   * Easier way to call the above just for DOM nodes (and better, since
+   * we get the performance benefits of having the same identity function).
+   * The call to |PreserveWrapper| is made with |aKey| == |aWrapper|.
+   *
+   * The caller need not call |ReleaseWrapper| since the node's
+   * wrapper's scriptable helper does so in its finalize callback.
+   */
+  static nsresult PreserveNodeWrapper(nsIXPConnectWrappedNative *aWrapper,
+                                      PRBool aRootWhenExternallyReferenced =
+                                        PR_FALSE);
+
+  /**
+   * Undoes the effects of any prior |PreserveWrapper| calls made with
+   * |aKey|.
+   */
+  static void ReleaseWrapper(void* aKey);
+
+  /**
+   * Mark all preserved wrappers reachable from |aDOMNode| via DOM APIs.
+   */
+  static void MarkReachablePreservedWrappers(nsIDOMGCParticipant *aParticipant,
+                                             JSContext *cx, void *arg);
+
+  /**
+   * Add/remove |aParticipant| from the table of externally referenced
+   * participants.  Does not maintain a count, so an object should not
+   * be added when it is already in the table.
+   *
+   * The table of externally referenced participants is a list of
+   * participants that should be marked at GC-time **if they are a
+   * participant in the preserved wrapper table added with
+   * aRootWhenExternallyReferenced**, whether or not they are reachable
+   * from marked participants.  This should be used for participants
+   * that hold onto scripts (typically onload or onerror handlers) that
+   * can be triggered at the end of a currently-ongoing operation
+   * (typically a network request) and that could thus expose the
+   * participant to script again in the future even though it is not
+   * otherwise reachable.
+   *
+   * The caller is responsible for ensuring that the GC participant is
+   * alive while it is in this table; the table does not own a reference.
+   *
+   * UnsetExternallyReferenced must be called exactly once for every
+   * successful SetExternallyReferenced call, and in no other cases.
+   */
+  static nsresult SetExternallyReferenced(nsIDOMGCParticipant *aParticipant);
+  static void UnsetExternallyReferenced(nsIDOMGCParticipant *aParticipant);
+
+  /**
+   * Classify the wrappers for use by |MarkReachablePreservedWrappers|
+   * during the GC.  Returns false to indicate failure (out-of-memory).
+   */
+  static PRBool BeginGCMark(JSContext *cx);
+
+  /**
+   * Clean up data structures (and strong references) created by
+   * |BeginGCMark|.
+   */
+  static void EndGCMark();
+
 protected:
   const nsDOMClassInfoData* mData;
 
@@ -206,7 +329,6 @@ protected:
   static jsval sParent_id;
   static jsval sScrollbars_id;
   static jsval sLocation_id;
-  static jsval sComponents_id;
   static jsval sConstructor_id;
   static jsval s_content_id;
   static jsval sContent_id;
@@ -246,6 +368,8 @@ protected:
   static jsval sOnload_id;
   static jsval sOnbeforeunload_id;
   static jsval sOnunload_id;
+  static jsval sOnpageshow_id;
+  static jsval sOnpagehide_id;
   static jsval sOnabort_id;
   static jsval sOnerror_id;
   static jsval sOnpaint_id;
@@ -269,23 +393,53 @@ protected:
   static jsval sAdd_id;
   static jsval sAll_id;
   static jsval sTags_id;
+  static jsval sAddEventListener_id;
+  static jsval sBaseURIObject_id;
+  static jsval sNodePrincipal_id;
+  static jsval sDocumentURIObject_id;
 
   static const JSClass *sObjectClass;
+  static const JSClass *sXPCNativeWrapperClass;
 
+public:
   static PRBool sDoSecurityCheckInAddProperty;
 };
 
 typedef nsDOMClassInfo nsDOMGenericSH;
 
+// Scriptable helper for implementations of nsIDOMGCParticipant that
+// need a mark callback.
+class nsDOMGCParticipantSH : public nsDOMGenericSH
+{
+protected:
+  nsDOMGCParticipantSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
+  {
+  }
+
+  virtual ~nsDOMGCParticipantSH()
+  {
+  }
+
+public:
+  NS_IMETHOD Finalize(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                      JSObject *obj);
+  NS_IMETHOD Mark(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                  JSObject *obj, void *arg, PRUint32 *_retval);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsDOMGCParticipantSH(aData);
+  }
+};
 
 // EventProp scriptable helper, this class should be the base class of
 // all objects that should support things like
 // obj.onclick=function{...}
 
-class nsEventReceiverSH : public nsDOMGenericSH
+class nsEventReceiverSH : public nsDOMGCParticipantSH
 {
 protected:
-  nsEventReceiverSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
+  nsEventReceiverSH(nsDOMClassInfoData* aData) : nsDOMGCParticipantSH(aData)
   {
   }
 
@@ -308,10 +462,15 @@ protected:
     return PR_FALSE;
   }
 
+  static JSBool JS_DLL_CALLBACK AddEventListenerHelper(JSContext *cx,
+                                                       JSObject *obj,
+                                                       uintN argc, jsval *argv,
+                                                       jsval *rval);
+
   nsresult RegisterCompileHandler(nsIXPConnectWrappedNative *wrapper,
                                   JSContext *cx, JSObject *obj, jsval id,
                                   PRBool compile, PRBool remove,
-                                  PRBool *did_compile);
+                                  PRBool *did_define);
 
 public:
   NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
@@ -338,7 +497,7 @@ protected:
   {
   }
 
-  static nsresult GlobalResolve(nsISupports *aNative, JSContext *cx,
+  static nsresult GlobalResolve(nsGlobalWindow *aWin, JSContext *cx,
                                 JSObject *obj, JSString *str, PRUint32 flags,
                                 PRBool *did_resolve);
 
@@ -356,11 +515,17 @@ public:
   NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                         JSObject *obj, jsval id, PRUint32 flags,
                         JSObject **objp, PRBool *_retval);
+  NS_IMETHOD NewEnumerate(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                          JSObject *obj, PRUint32 enum_op, jsval *statep,
+                          jsid *idp, PRBool *_retval);
   NS_IMETHOD Finalize(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                       JSObject *obj);
-
-  static nsresult OnDocumentChanged(JSContext *cx, JSObject *obj,
-                                    nsIDOMWindow *window);
+  NS_IMETHOD Equality(nsIXPConnectWrappedNative *wrapper, JSContext * cx,
+                      JSObject * obj, jsval val, PRBool *bp);
+  NS_IMETHOD OuterObject(nsIXPConnectWrappedNative *wrapper, JSContext * cx,
+                         JSObject * obj, JSObject * *_retval);
+  NS_IMETHOD InnerObject(nsIXPConnectWrappedNative *wrapper, JSContext * cx,
+                         JSObject * obj, JSObject * *_retval);
 
   static JSBool JS_DLL_CALLBACK GlobalScopePolluterNewResolve(JSContext *cx,
                                                               JSObject *obj,
@@ -374,10 +539,8 @@ public:
   static JSBool JS_DLL_CALLBACK SecurityCheckOnSetProp(JSContext *cx,
                                                        JSObject *obj, jsval id,
                                                        jsval *vp);
-  static JSObject *GetInvalidatedGlobalScopePolluter(JSContext *cx,
-                                                     JSObject *obj);
+  static void InvalidateGlobalScopePolluter(JSContext *cx, JSObject *obj);
   static nsresult InstallGlobalScopePolluter(JSContext *cx, JSObject *obj,
-                                             JSObject *oldPolluter,
                                              nsIHTMLDocument *doc);
 
   static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
@@ -405,6 +568,9 @@ public:
                          JSObject *obj, jsval id, PRUint32 mode,
                          jsval *vp, PRBool *_retval);
 
+  NS_IMETHOD PreCreate(nsISupports *nativeObj, JSContext *cx,
+                       JSObject *globalObj, JSObject **parentObj);
+
   static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
     return new nsLocationSH(aData);
@@ -412,6 +578,30 @@ public:
 };
 
 
+// Navigator scriptable helper
+
+class nsNavigatorSH : public nsDOMGenericSH
+{
+protected:
+  nsNavigatorSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
+  {
+  }
+
+  virtual ~nsNavigatorSH()
+  {
+  }
+
+public:
+  NS_IMETHOD PreCreate(nsISupports *nativeObj, JSContext *cx,
+                       JSObject *globalObj, JSObject **parentObj);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsNavigatorSH(aData);
+  }
+};
+
+
 // DOM Node helper, this class deals with setting the parent for the
 // wrappers
 
@@ -426,11 +616,32 @@ protected:
   {
   }
 
+  // Helper to check whether a capability is enabled
+  PRBool IsCapabilityEnabled(const char* aCapability);
+
+  inline PRBool IsPrivilegedScript() {
+    return IsCapabilityEnabled("UniversalXPConnect");
+  }
+
+  // Helper to define a void property with JSPROP_SHARED; this can do all the
+  // work so it's safe to just return whatever it returns.  |obj| is the object
+  // we're defining on, |id| is the name of the prop.  This must be a string
+  // jsval.  |objp| is the out param if we define successfully.
+  nsresult DefineVoidProp(JSContext* cx, JSObject* obj, jsval id,
+                          JSObject** objp);
+
 public:
   NS_IMETHOD PreCreate(nsISupports *nativeObj, JSContext *cx,
                        JSObject *globalObj, JSObject **parentObj);
   NS_IMETHOD AddProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                          JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                        JSObject *obj, jsval id, PRUint32 flags,
+                        JSObject **objp, PRBool *_retval);
+  NS_IMETHOD GetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD SetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
   NS_IMETHOD GetFlags(PRUint32 *aFlags);
 
   static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
@@ -478,6 +689,9 @@ protected:
   }
   
 public:
+  NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                        JSObject *obj, jsval id, PRUint32 flags,
+                        JSObject **objp, PRBool *_retval);
   NS_IMETHOD Enumerate(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                        JSObject *obj, PRBool *_retval);
   
@@ -674,6 +888,8 @@ public:
   NS_IMETHOD SetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                          JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
   NS_IMETHOD GetFlags(PRUint32* aFlags);
+  NS_IMETHOD PostCreate(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                        JSObject *obj);
 
   static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
@@ -810,6 +1026,39 @@ public:
 };
 
 
+// HTML[I]FrameElement helper
+
+class nsHTMLFrameElementSH : public nsHTMLElementSH
+{
+protected:
+  nsHTMLFrameElementSH(nsDOMClassInfoData* aData) : nsHTMLElementSH(aData)
+  {
+  }
+
+  virtual ~nsHTMLFrameElementSH()
+  {
+  }
+
+public:
+  NS_IMETHOD GetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD SetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD AddProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD DelProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                        JSObject *obj, jsval id, PRUint32 flags,
+                        JSObject **objp, PRBool *_retval);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsHTMLFrameElementSH(aData);
+  }
+};
+
+
 // HTMLSelectElement helper
 
 class nsHTMLSelectElementSH : public nsHTMLElementSH
@@ -1284,6 +1533,63 @@ public:
 };
 #endif
 
+// WebApps Storage helpers
+
+class nsStorageSH : public nsNamedArraySH
+{
+protected:
+  nsStorageSH(nsDOMClassInfoData* aData) : nsNamedArraySH(aData)
+  {
+  }
+
+  virtual ~nsStorageSH()
+  {
+  }
+
+  NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                        JSObject *obj, jsval id, PRUint32 flags,
+                        JSObject **objp, PRBool *_retval);
+  NS_IMETHOD SetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD DelProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD NewEnumerate(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                          JSObject *obj, PRUint32 enum_op, jsval *statep,
+                          jsid *idp, PRBool *_retval);
+
+  // Override nsNamedArraySH::GetNamedItem()
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
+                                nsISupports **aResult);
+
+public:
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsStorageSH(aData);
+  }
+};
+
+class nsStorageListSH : public nsNamedArraySH
+{
+protected:
+  nsStorageListSH(nsDOMClassInfoData* aData) : nsNamedArraySH(aData)
+  {
+  }
+
+  virtual ~nsStorageListSH()
+  {
+  }
+
+  // Override nsNamedArraySH::GetNamedItem()
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
+                                nsISupports **aResult);
+
+public:
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsStorageListSH(aData);
+  }
+};
+
 
 // Event handler 'this' translator class, this is called by XPConnect
 // when a "function interface" (nsIDOMEventListener) is called, this
@@ -1310,6 +1616,72 @@ public:
   NS_DECL_NSIXPCFUNCTIONTHISTRANSLATOR
 };
 
+class nsDOMConstructorSH : public nsDOMGenericSH
+{
+protected:
+  nsDOMConstructorSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
+  {
+  }
+
+  virtual ~nsDOMConstructorSH()
+  {
+  }
+
+public:
+  NS_IMETHOD Construct(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                       JSObject *obj, PRUint32 argc, jsval *argv,
+                       jsval *vp, PRBool *_retval);
+
+  NS_IMETHOD HasInstance(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval val, PRBool *bp,
+                         PRBool *_retval);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsDOMConstructorSH(aData);
+  }
+};
+
+class nsNonDOMObjectSH : public nsDOMGenericSH
+{
+protected:
+  nsNonDOMObjectSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
+  {
+  }
+
+  virtual ~nsNonDOMObjectSH()
+  {
+  }
+
+public:
+  NS_IMETHOD GetFlags(PRUint32 *aFlags);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsNonDOMObjectSH(aData);
+  }
+};
+
+// Need this to override GetFlags() on nsNodeSH
+class nsAttributeSH : public nsNodeSH
+{
+protected:
+  nsAttributeSH(nsDOMClassInfoData* aData) : nsNodeSH(aData)
+  {
+  }
+
+  virtual ~nsAttributeSH()
+  {
+  }
+
+public:
+  NS_IMETHOD GetFlags(PRUint32 *aFlags);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsAttributeSH(aData);
+  }
+};
 
 void InvalidateContextAndWrapperCache();
 
