@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -40,7 +41,6 @@
 #include "nsGlobalWindow.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIScriptContext.h"
-#include "nsIWebShell.h"
 #include "nsIDocShell.h"
 #include "nsIDocShellLoadInfo.h"
 #include "nsIWebNavigation.h"
@@ -73,6 +73,36 @@
 #include "nsITextToSubURI.h"
 #include "nsContentUtils.h"
 
+static nsresult
+GetContextFromStack(nsIJSContextStack *aStack, JSContext **aContext)
+{
+  nsCOMPtr<nsIJSContextStackIterator>
+    iterator(do_CreateInstance("@mozilla.org/js/xpc/ContextStackIterator;1"));
+  NS_ENSURE_TRUE(iterator, NS_ERROR_FAILURE);
+
+  nsresult rv = iterator->Reset(aStack);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  PRBool done;
+  while (NS_SUCCEEDED(iterator->Done(&done)) && !done) {
+    rv = iterator->Prev(aContext);
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Broken iterator implementation");
+
+    // Consider a null context the end of the line.
+    if (!*aContext) {
+      break;
+    }
+
+    if (nsJSUtils::GetDynamicScriptContext(*aContext)) {
+      return NS_OK;
+    }
+  }
+
+  *aContext = nsnull;
+
+  return NS_OK;
+}
+
 static nsresult
 GetDocumentCharacterSetForURI(const nsAString& aHref, nsACString& aCharset)
 {
@@ -85,7 +115,7 @@ GetDocumentCharacterSetForURI(const nsAString& aHref, nsACString& aCharset)
 
   JSContext *cx;
 
-  rv = stack->Peek(&cx);
+  rv = GetContextFromStack(stack, &cx);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (cx) {
@@ -107,18 +137,18 @@ GetDocumentCharacterSetForURI(const nsAString& aHref, nsACString& aCharset)
   return NS_OK;
 }
 
-LocationImpl::LocationImpl(nsIDocShell *aDocShell)
+nsLocation::nsLocation(nsIDocShell *aDocShell)
 {
-  mDocShell = aDocShell; // Weak Reference
+  mDocShell = do_GetWeakReference(aDocShell);
 }
 
-LocationImpl::~LocationImpl()
+nsLocation::~nsLocation()
 {
 }
 
 
-// QueryInterface implementation for LocationImpl
-NS_INTERFACE_MAP_BEGIN(LocationImpl)
+// QueryInterface implementation for nsLocation
+NS_INTERFACE_MAP_BEGIN(nsLocation)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSLocation)
   NS_INTERFACE_MAP_ENTRY(nsIDOMLocation)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMLocation)
@@ -126,20 +156,32 @@ NS_INTERFACE_MAP_BEGIN(LocationImpl)
 NS_INTERFACE_MAP_END
 
 
-NS_IMPL_ADDREF(LocationImpl)
-NS_IMPL_RELEASE(LocationImpl)
+NS_IMPL_ADDREF(nsLocation)
+NS_IMPL_RELEASE(nsLocation)
 
 void
-LocationImpl::SetDocShell(nsIDocShell *aDocShell)
+nsLocation::SetDocShell(nsIDocShell *aDocShell)
 {
-   mDocShell = aDocShell; // Weak Reference
+   mDocShell = do_GetWeakReference(aDocShell);
+}
+
+nsIDocShell *
+nsLocation::GetDocShell()
+{
+  nsCOMPtr<nsIDocShell> docshell(do_QueryReferent(mDocShell));
+  return docshell;
 }
 
 nsresult
-LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
+nsLocation::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
 {
   *aLoadInfo = nsnull;
 
+  nsCOMPtr<nsIDocShell> docShell(do_QueryReferent(mDocShell));
+  if (!docShell) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
   nsresult result;
   // Get JSContext from stack.
   nsCOMPtr<nsIJSContextStack>
@@ -150,54 +192,55 @@ LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
 
   JSContext *cx;
 
-  if (NS_FAILED(stack->Peek(&cx)))
+  if (NS_FAILED(GetContextFromStack(stack, &cx)))
     return NS_ERROR_FAILURE;
 
-  if (!cx) {
+  nsCOMPtr<nsISupports> owner;
+  nsCOMPtr<nsIURI> sourceURI;
+
+  if (cx) {
     // No cx means that there's no JS running, or at least no JS that
     // was run through code that properly pushed a context onto the
     // context stack (as all code that runs JS off of web pages
-    // does). Going further from here will crash, so lets not do
-    // that...
+    // does). We won't bother with security checks in this case, but
+    // we need to create the loadinfo etc.
 
-    return NS_OK;
+    // Get security manager.
+    nsCOMPtr<nsIScriptSecurityManager>
+      secMan(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &result));
+
+    if (NS_FAILED(result))
+      return NS_ERROR_FAILURE;
+
+    // Check to see if URI is allowed.
+    result = secMan->CheckLoadURIFromScript(cx, aURI);
+
+    if (NS_FAILED(result))
+      return result;
+
+    // Now get the principal to use when loading the URI
+    nsCOMPtr<nsIPrincipal> principal;
+    if (NS_FAILED(secMan->GetSubjectPrincipal(getter_AddRefs(principal))) ||
+        !principal)
+      return NS_ERROR_FAILURE;
+    owner = do_QueryInterface(principal);
+
+    GetSourceURL(cx, getter_AddRefs(sourceURI));
   }
 
-  // Get security manager.
-  nsCOMPtr<nsIScriptSecurityManager>
-    secMan(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &result));
-
-  if (NS_FAILED(result))
-    return NS_ERROR_FAILURE;
-
-  // Check to see if URI is allowed.
-  result = secMan->CheckLoadURIFromScript(cx, aURI);
-
-  if (NS_FAILED(result))
-    return result;
-
   // Create load info
   nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
-  mDocShell->CreateLoadInfo(getter_AddRefs(loadInfo));
+  docShell->CreateLoadInfo(getter_AddRefs(loadInfo));
   NS_ENSURE_TRUE(loadInfo, NS_ERROR_FAILURE);
 
-  // Now get the principal to use when loading the URI
-  nsCOMPtr<nsIPrincipal> principal;
-  if (NS_FAILED(secMan->GetSubjectPrincipal(getter_AddRefs(principal))) ||
-      !principal)
-    return NS_ERROR_FAILURE;
-  nsCOMPtr<nsISupports> owner = do_QueryInterface(principal);
   loadInfo->SetOwner(owner);
 
   // now set the referrer on the loadinfo
-  nsCOMPtr<nsIURI> sourceURI;
-  GetSourceURL(cx, getter_AddRefs(sourceURI));
   if (sourceURI) {
     loadInfo->SetReferrer(sourceURI);
   }
-  
-  *aLoadInfo = loadInfo.get();
-  NS_ADDREF(*aLoadInfo);
+
+  loadInfo.swap(*aLoadInfo);
 
   return NS_OK;
 }
@@ -206,19 +249,21 @@ LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
 // anything that would allow a relative uri.
 
 nsresult
-LocationImpl::FindUsableBaseURI(nsIURI * aBaseURI, nsIDocShell * aParent, nsIURI ** aUsableURI)
+nsLocation::FindUsableBaseURI(nsIURI * aBaseURI, nsIDocShell * aParent,
+                              nsIURI ** aUsableURI)
 {
   if (!aBaseURI || !aParent)
     return NS_ERROR_FAILURE;
   NS_ENSURE_ARG_POINTER(aUsableURI);
-    
+
   *aUsableURI = nsnull;
   nsresult rv = NS_OK;    
   nsCOMPtr<nsIDocShell> parentDS = aParent;
   nsCOMPtr<nsIURI> baseURI = aBaseURI;
-  nsCOMPtr<nsIIOService> ioService(do_GetService(NS_IOSERVICE_CONTRACTID, &rv));
+  nsCOMPtr<nsIIOService> ioService =
+    do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
 
-  while(NS_SUCCEEDED(rv) && baseURI && ioService) {
+  while(NS_SUCCEEDED(rv) && baseURI) {
     // Check if the current base uri supports relative uris.
     // We make this check by looking at the protocol flags of
     // the protocol handler. If the protocol flags has URI_NORELATIVE,
@@ -260,13 +305,13 @@ LocationImpl::FindUsableBaseURI(nsIURI * aBaseURI, nsIDocShell * aParent, nsIURI
 
 
 nsresult
-LocationImpl::GetURI(nsIURI** aURI, PRBool aGetInnermostURI)
+nsLocation::GetURI(nsIURI** aURI, PRBool aGetInnermostURI)
 {
   *aURI = nsnull;
 
   nsresult rv;
-
-  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mDocShell, &rv));
+  nsCOMPtr<nsIDocShell> docShell(do_QueryReferent(mDocShell));
+  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(docShell, &rv));
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -298,7 +343,7 @@ LocationImpl::GetURI(nsIURI** aURI, PRBool aGetInnermostURI)
 }
 
 nsresult
-LocationImpl::GetWritableURI(nsIURI** aURI)
+nsLocation::GetWritableURI(nsIURI** aURI)
 {
   *aURI = nsnull;
 
@@ -313,25 +358,31 @@ LocationImpl::GetWritableURI(nsIURI** aURI)
 }
 
 nsresult
-LocationImpl::SetURI(nsIURI* aURI)
+nsLocation::SetURI(nsIURI* aURI, PRBool aReplace)
 {
-  if (mDocShell) {
+  nsCOMPtr<nsIDocShell> docShell(do_QueryReferent(mDocShell));
+  if (docShell) {
     nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
-    nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mDocShell));
+    nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(docShell));
 
     if(NS_FAILED(CheckURL(aURI, getter_AddRefs(loadInfo))))
       return NS_ERROR_FAILURE;
 
-    webNav->Stop(nsIWebNavigation::STOP_CONTENT);
-    return mDocShell->LoadURI(aURI, loadInfo,
-                              nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
+    if (aReplace) {
+      loadInfo->SetLoadType(nsIDocShellLoadInfo::loadStopContentAndReplace);
+    } else {
+      loadInfo->SetLoadType(nsIDocShellLoadInfo::loadStopContent);
+    }
+
+    return docShell->LoadURI(aURI, loadInfo,
+                             nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHash(nsAString& aHash)
+nsLocation::GetHash(nsAString& aHash)
 {
   aHash.SetLength(0);
 
@@ -374,7 +425,7 @@ LocationImpl::GetHash(nsAString& aHash)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHash(const nsAString& aHash)
+nsLocation::SetHash(const nsAString& aHash)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
@@ -384,25 +435,15 @@ LocationImpl::SetHash(const nsAString& aHash)
   nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
 
   if (url) {
-    url->SetRef(NS_ConvertUCS2toUTF8(aHash));
-
-    if (mDocShell) {
-      nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
-
-      if (NS_SUCCEEDED(CheckURL(url, getter_AddRefs(loadInfo))))
-        // We're not calling nsIWebNavigation->Stop, we don't want to
-        // stop the load when we're just scrolling to a named anchor
-        // in the document. See bug 114975.
-        mDocShell->LoadURI(url, loadInfo,
-                           nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
-    }
+    url->SetRef(NS_ConvertUTF16toUTF8(aHash));
+    SetURI(url);
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHost(nsAString& aHost)
+nsLocation::GetHost(nsAString& aHost)
 {
   aHost.Truncate();
 
@@ -425,7 +466,7 @@ LocationImpl::GetHost(nsAString& aHost)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHost(const nsAString& aHost)
+nsLocation::SetHost(const nsAString& aHost)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result;
@@ -433,7 +474,7 @@ LocationImpl::SetHost(const nsAString& aHost)
   result = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
-    uri->SetHostPort(NS_ConvertUCS2toUTF8(aHost));
+    uri->SetHostPort(NS_ConvertUTF16toUTF8(aHost));
     SetURI(uri);
   }
 
@@ -441,7 +482,7 @@ LocationImpl::SetHost(const nsAString& aHost)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHostname(nsAString& aHostname)
+nsLocation::GetHostname(nsAString& aHostname)
 {
   aHostname.Truncate();
 
@@ -464,7 +505,7 @@ LocationImpl::GetHostname(nsAString& aHostname)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHostname(const nsAString& aHostname)
+nsLocation::SetHostname(const nsAString& aHostname)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result;
@@ -472,7 +513,7 @@ LocationImpl::SetHostname(const nsAString& aHostname)
   result = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
-    uri->SetHost(NS_ConvertUCS2toUTF8(aHostname));
+    uri->SetHost(NS_ConvertUTF16toUTF8(aHostname));
     SetURI(uri);
   }
 
@@ -480,7 +521,7 @@ LocationImpl::SetHostname(const nsAString& aHostname)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHref(nsAString& aHref)
+nsLocation::GetHref(nsAString& aHref)
 {
   aHref.Truncate();
 
@@ -503,15 +544,8 @@ LocationImpl::GetHref(nsAString& aHref)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHref(const nsAString& aHref)
+nsLocation::SetHref(const nsAString& aHref)
 {
-  if (!aHref.IsEmpty() && aHref.First() == PRUnichar('#')) {
-    // Special-case anchor loads so that we don't stop content
-    // Note that SetHash (or more precisely nsIURL::SetRef) deals with
-    // the leading '#'.
-    return SetHash(aHref);
-  }
-  
   nsAutoString oldHref;
   nsresult rv = NS_OK;
 
@@ -524,7 +558,7 @@ LocationImpl::SetHref(const nsAString& aHref)
 
   JSContext *cx;
 
-  if (NS_FAILED(stack->Peek(&cx)))
+  if (NS_FAILED(GetContextFromStack(stack, &cx)))
     return NS_ERROR_FAILURE;
 
   if (cx) {
@@ -547,8 +581,8 @@ LocationImpl::SetHref(const nsAString& aHref)
 }
 
 nsresult
-LocationImpl::SetHrefWithContext(JSContext* cx, const nsAString& aHref,
-                                 PRBool aReplace)
+nsLocation::SetHrefWithContext(JSContext* cx, const nsAString& aHref,
+                               PRBool aReplace)
 {
   nsCOMPtr<nsIURI> base;
 
@@ -563,14 +597,16 @@ LocationImpl::SetHrefWithContext(JSContext* cx, const nsAString& aHref,
 }
 
 nsresult
-LocationImpl::SetHrefWithBase(const nsAString& aHref,
-                              nsIURI* aBase, PRBool aReplace)
+nsLocation::SetHrefWithBase(const nsAString& aHref, nsIURI* aBase,
+                            PRBool aReplace)
 {
   nsresult result;
   nsCOMPtr<nsIURI> newUri, baseURI;
 
+  nsCOMPtr<nsIDocShell> docShell(do_QueryReferent(mDocShell));
+
   // Try to make sure the base url is something that will be useful. 
-  result = FindUsableBaseURI(aBase,  mDocShell, getter_AddRefs(baseURI));
+  result = FindUsableBaseURI(aBase,  docShell, getter_AddRefs(baseURI));
   if (!baseURI)  {
     // If nothing useful was found, just use what you have.
     baseURI = aBase;
@@ -582,15 +618,7 @@ LocationImpl::SetHrefWithBase(const nsAString& aHref,
   else
     result = NS_NewURI(getter_AddRefs(newUri), aHref, nsnull, baseURI);
 
-  if (newUri && mDocShell) {
-    nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
-    nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mDocShell));
-
-    nsresult rv = CheckURL(newUri, getter_AddRefs(loadInfo));
-
-    if(NS_FAILED(rv))
-      return rv;
-     
+  if (newUri) {
     /* Check with the scriptContext if it is currently processing a script tag.
      * If so, this must be a <script> tag with a location.href in it.
      * we want to do a replace load, in such a situation. 
@@ -608,7 +636,7 @@ LocationImpl::SetHrefWithBase(const nsAString& aHref,
     if (stack) {
       JSContext *cx;
 
-      result = stack->Peek(&cx);
+      result = GetContextFromStack(stack, &cx);
       if (cx) {
         nsIScriptContext *scriptContext =
           nsJSUtils::GetDynamicScriptContext(cx);
@@ -618,27 +646,21 @@ LocationImpl::SetHrefWithBase(const nsAString& aHref,
             // Now check to make sure that the script is running in our window,
             // since we only want to replace if the location is set by a
             // <script> tag in the same window.  See bug 178729.
-            nsCOMPtr<nsIScriptGlobalObject> ourGlobal(do_GetInterface(mDocShell));
+            nsCOMPtr<nsIScriptGlobalObject> ourGlobal(do_GetInterface(docShell));
             inScriptTag = (ourGlobal == scriptContext->GetGlobalObject());
           }
         }  
       } //cx
     }  // stack
 
-    if (aReplace || inScriptTag) {
-      loadInfo->SetLoadType(nsIDocShellLoadInfo::loadNormalReplace);
-    }
-
-    webNav->Stop(nsIWebNavigation::STOP_CONTENT);
-    return mDocShell->LoadURI(newUri, loadInfo,
-                              nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
+    return SetURI(newUri, aReplace || inScriptTag);
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::GetPathname(nsAString& aPathname)
+nsLocation::GetPathname(nsAString& aPathname)
 {
   aPathname.Truncate();
 
@@ -662,7 +684,7 @@ LocationImpl::GetPathname(nsAString& aPathname)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetPathname(const nsAString& aPathname)
+nsLocation::SetPathname(const nsAString& aPathname)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
@@ -670,7 +692,7 @@ LocationImpl::SetPathname(const nsAString& aPathname)
   result = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
-    uri->SetPath(NS_ConvertUCS2toUTF8(aPathname));
+    uri->SetPath(NS_ConvertUTF16toUTF8(aPathname));
     SetURI(uri);
   }
 
@@ -678,7 +700,7 @@ LocationImpl::SetPathname(const nsAString& aPathname)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetPort(nsAString& aPort)
+nsLocation::GetPort(nsAString& aPort)
 {
   aPort.SetLength(0);
 
@@ -689,20 +711,23 @@ LocationImpl::GetPort(nsAString& aPort)
 
   if (uri) {
     PRInt32 port;
-    uri->GetPort(&port);
+    result = uri->GetPort(&port);
 
-    if (-1 != port) {
+    if (NS_SUCCEEDED(result) && -1 != port) {
       nsAutoString portStr;
       portStr.AppendInt(port);
       aPort.Append(portStr);
     }
+
+    // Don't propagate this exception to caller
+    result = NS_OK;
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::SetPort(const nsAString& aPort)
+nsLocation::SetPort(const nsAString& aPort)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
@@ -711,7 +736,7 @@ LocationImpl::SetPort(const nsAString& aPort)
 
   if (uri) {
     // perhaps use nsReadingIterators at some point?
-    NS_ConvertUCS2toUTF8 portStr(aPort);
+    NS_ConvertUTF16toUTF8 portStr(aPort);
     const char *buf = portStr.get();
     PRInt32 port = -1;
 
@@ -732,7 +757,7 @@ LocationImpl::SetPort(const nsAString& aPort)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetProtocol(nsAString& aProtocol)
+nsLocation::GetProtocol(nsAString& aProtocol)
 {
   aProtocol.SetLength(0);
 
@@ -756,7 +781,7 @@ LocationImpl::GetProtocol(nsAString& aProtocol)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetProtocol(const nsAString& aProtocol)
+nsLocation::SetProtocol(const nsAString& aProtocol)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
@@ -764,7 +789,7 @@ LocationImpl::SetProtocol(const nsAString& aProtocol)
   result = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
-    uri->SetScheme(NS_ConvertUCS2toUTF8(aProtocol));
+    uri->SetScheme(NS_ConvertUTF16toUTF8(aProtocol));
     SetURI(uri);
   }
 
@@ -772,7 +797,7 @@ LocationImpl::SetProtocol(const nsAString& aProtocol)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetSearch(nsAString& aSearch)
+nsLocation::GetSearch(nsAString& aSearch)
 {
   aSearch.SetLength(0);
 
@@ -798,7 +823,7 @@ LocationImpl::GetSearch(nsAString& aSearch)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetSearch(const nsAString& aSearch)
+nsLocation::SetSearch(const nsAString& aSearch)
 {
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
@@ -807,7 +832,7 @@ LocationImpl::SetSearch(const nsAString& aSearch)
 
   nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
   if (url) {
-    result = url->SetQuery(NS_ConvertUCS2toUTF8(aSearch));
+    result = url->SetQuery(NS_ConvertUTF16toUTF8(aSearch));
     SetURI(uri);
   }
 
@@ -815,10 +840,11 @@ LocationImpl::SetSearch(const nsAString& aSearch)
 }
 
 NS_IMETHODIMP
-LocationImpl::Reload(PRBool aForceget)
+nsLocation::Reload(PRBool aForceget)
 {
   nsresult rv;
-  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mDocShell));
+  nsCOMPtr<nsIDocShell> docShell(do_QueryReferent(mDocShell));
+  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(docShell));
 
   if (webNav) {
     PRUint32 reloadFlags = nsIWebNavigation::LOAD_FLAGS_NONE;
@@ -835,7 +861,6 @@ LocationImpl::Reload(PRBool aForceget)
       rv = NS_OK;
     }
   } else {
-    NS_ASSERTION(0, "nsIWebNavigation interface is not available!");
     rv = NS_ERROR_FAILURE;
   }
 
@@ -843,7 +868,7 @@ LocationImpl::Reload(PRBool aForceget)
 }
 
 NS_IMETHODIMP
-LocationImpl::Reload()
+nsLocation::Reload()
 {
   nsCOMPtr<nsIXPCNativeCallContext> ncc;
   nsresult rv = nsContentUtils::XPConnect()->
@@ -853,7 +878,8 @@ LocationImpl::Reload()
   if (!ncc)
     return NS_ERROR_NOT_AVAILABLE;
 
-  nsCOMPtr<nsPIDOMWindow> window(do_GetInterface(mDocShell));
+  nsCOMPtr<nsIDocShell> docShell(do_QueryReferent(mDocShell));
+  nsCOMPtr<nsPIDOMWindow> window(do_GetInterface(docShell));
 
   if (window && window->IsHandlingResizeEvent()) {
     // location.reload() was called on a window that is handling a
@@ -892,6 +918,7 @@ LocationImpl::Reload()
     rv = ncc->GetJSContext(&cx);
     NS_ENSURE_SUCCESS(rv, rv);
 
+    JSAutoRequest ar(cx);
     JS_ValueToBoolean(cx, argv[0], &force_get);
   }
 
@@ -899,7 +926,7 @@ LocationImpl::Reload()
 }
 
 NS_IMETHODIMP
-LocationImpl::Replace(const nsAString& aUrl)
+nsLocation::Replace(const nsAString& aUrl)
 {
   nsresult rv = NS_OK;
 
@@ -910,7 +937,7 @@ LocationImpl::Replace(const nsAString& aUrl)
   if (stack) {
     JSContext *cx;
 
-    rv = stack->Peek(&cx);
+    rv = GetContextFromStack(stack, &cx);
     NS_ENSURE_SUCCESS(rv, rv);
     if (cx) {
       return SetHrefWithContext(cx, aUrl, PR_TRUE);
@@ -931,7 +958,7 @@ LocationImpl::Replace(const nsAString& aUrl)
 }
 
 NS_IMETHODIMP
-LocationImpl::Assign(const nsAString& aUrl)
+nsLocation::Assign(const nsAString& aUrl)
 {
   nsAutoString oldHref;
   nsresult result = NS_OK;
@@ -952,13 +979,13 @@ LocationImpl::Assign(const nsAString& aUrl)
 }
 
 NS_IMETHODIMP
-LocationImpl::ToString(nsAString& aReturn)
+nsLocation::ToString(nsAString& aReturn)
 {
   return GetHref(aReturn);
 }
 
 nsresult
-LocationImpl::GetSourceDocument(JSContext* cx, nsIDocument** aDocument)
+nsLocation::GetSourceDocument(JSContext* cx, nsIDocument** aDocument)
 {
   // XXX Code duplicated from nsHTMLDocument
   // XXX Tom said this reminded him of the "Six Degrees of
@@ -991,7 +1018,7 @@ LocationImpl::GetSourceDocument(JSContext* cx, nsIDocument** aDocument)
 }
 
 nsresult
-LocationImpl::GetSourceBaseURL(JSContext* cx, nsIURI** sourceURL)
+nsLocation::GetSourceBaseURL(JSContext* cx, nsIURI** sourceURL)
 {
   nsCOMPtr<nsIDocument> doc;
   nsresult rv = GetSourceDocument(cx, getter_AddRefs(doc));
@@ -1005,7 +1032,7 @@ LocationImpl::GetSourceBaseURL(JSContext* cx, nsIURI** sourceURL)
 }
 
 nsresult
-LocationImpl::GetSourceURL(JSContext* cx, nsIURI** sourceURL)
+nsLocation::GetSourceURL(JSContext* cx, nsIURI** sourceURL)
 {
   nsCOMPtr<nsIDocument> doc;
   nsresult rv = GetSourceDocument(cx, getter_AddRefs(doc));
