@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set ts=2 sw=4 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -43,6 +44,7 @@
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "nsJSProtocolHandler.h"
+#include "nsStringStream.h"
 #include "nsNetUtil.h"
 
 #include "nsIComponentManager.h"
@@ -56,23 +58,20 @@
 #include "nsIScriptSecurityManager.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
-#include "nsIStringStream.h"
 #include "nsIWindowMediator.h"
-#include "nsIDOMWindowInternal.h"
+#include "nsPIDOMWindow.h"
 #include "nsIDOMDocument.h"
 #include "nsIJSConsoleService.h"
 #include "nsIConsoleService.h"
 #include "nsXPIDLString.h"
 #include "prprf.h"
 #include "nsEscape.h"
-#include "nsIJSContextStack.h"
 #include "nsIWebNavigation.h"
 #include "nsIDocShell.h"
 #include "nsIContentViewer.h"
-
-static NS_DEFINE_CID(kSimpleURICID, NS_SIMPLEURI_CID);
-static NS_DEFINE_CID(kWindowMediatorCID, NS_WINDOWMEDIATOR_CID);
-
+#include "nsIXPConnect.h"
+#include "nsContentUtils.h"
+#include "nsJSUtils.h"
 
 
 class nsJSThunk : public nsIInputStream
@@ -116,34 +115,6 @@ nsresult nsJSThunk::Init(nsIURI* uri)
     return NS_OK;
 }
 
-static void
-GetInterfaceFromChannel(nsIChannel* aChannel,
-                        const nsIID &aIID,
-                        void **aResult)
-{
-    NS_PRECONDITION(aChannel, "Must have a channel");
-    NS_PRECONDITION(aResult, "Null out param");
-    *aResult = nsnull;
-
-    // Get an interface requestor from the channel callbacks.
-    nsCOMPtr<nsIInterfaceRequestor> callbacks;
-    aChannel->GetNotificationCallbacks(getter_AddRefs(callbacks));
-    if (callbacks) {
-        callbacks->GetInterface(aIID, aResult);
-    }
-    if (!*aResult) {
-        // Try the loadgroup
-        nsCOMPtr<nsILoadGroup> loadGroup;
-        aChannel->GetLoadGroup(getter_AddRefs(loadGroup));
-        if (loadGroup) {
-            loadGroup->GetNotificationCallbacks(getter_AddRefs(callbacks));
-            if (callbacks) {
-                callbacks->GetInterface(aIID, aResult);
-            }
-        }
-    }
-}
-
 nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel)
 {
     nsresult rv;
@@ -157,8 +128,7 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel)
 
     // The the global object owner from the channel
     nsCOMPtr<nsIScriptGlobalObjectOwner> globalOwner;
-    GetInterfaceFromChannel(aChannel, NS_GET_IID(nsIScriptGlobalObjectOwner),
-                            getter_AddRefs(globalOwner));
+    NS_QueryNotificationCallbacks(aChannel, globalOwner);
     NS_ASSERTION(globalOwner, 
                  "Unable to get an nsIScriptGlobalObjectOwner from the "
                  "channel!");
@@ -176,14 +146,26 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel)
         return NS_ERROR_FAILURE;
     }
 
-    JSObject *globalJSObject = global->GetGlobalJSObject();
+    nsCOMPtr<nsPIDOMWindow> win(do_QueryInterface(global));
+
+    // Make sure we create a new inner window if one doesn't already exist (see
+    // bug 306630).
+    nsPIDOMWindow *innerWin = win->EnsureInnerWindow();
+
+    if (!innerWin) {
+        return NS_ERROR_UNEXPECTED;
+    }
+
+    nsCOMPtr<nsIScriptGlobalObject> innerGlobal = do_QueryInterface(innerWin);
+
+    JSObject *globalJSObject = innerGlobal->GetGlobalJSObject();
 
     nsCOMPtr<nsIDOMWindow> domWindow(do_QueryInterface(global, &rv));
     if (NS_FAILED(rv)) {
         return NS_ERROR_FAILURE;
     }
 
-    // If mURI is just "javascript:", we bring up the JavaScript console
+    // If mURI is just "javascript:", we bring up the Error console
     // and return NS_ERROR_DOM_RETVAL_UNDEFINED.
     if (script.IsEmpty()) {
         rv = BringUpConsole(domWindow);
@@ -191,16 +173,6 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel)
         return NS_ERROR_DOM_RETVAL_UNDEFINED;
     }
 
-    // Now get the DOM Document.  Accessing the document will create one
-    // if necessary.  So, basically, this call ensures that a document gets
-    // created -- if necessary.
-    nsCOMPtr<nsIDOMDocument> doc;
-    rv = domWindow->GetDocument(getter_AddRefs(doc));
-    NS_ASSERTION(doc, "No DOMDocument!");
-    if (NS_FAILED(rv)) {
-        return NS_ERROR_FAILURE;
-    }
-
     nsCOMPtr<nsIScriptContext> scriptContext = global->GetContext();
     if (!scriptContext)
         return NS_ERROR_FAILURE;
@@ -225,16 +197,16 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel)
     if (NS_FAILED(rv))
         return rv;
 
+    PRBool useSandbox = PR_TRUE;
+
     if (owner) {
         principal = do_QueryInterface(owner, &rv);
         NS_ASSERTION(principal, "Channel's owner is not a principal");
         if (!principal)
             return NS_ERROR_FAILURE;
 
-        //-- Don't run if the script principal is different from the
-        //   principal of the context, with two exceptions: we allow
-        //   the script to run if the script has the system principal
-        //   or the context is about:blank.
+        //-- Don't run if the script principal is different from the principal
+        //   of the context, unless the script has the system principal.
         nsCOMPtr<nsIPrincipal> objectPrincipal;
         rv = securityManager->GetObjectPrincipal(
                                 (JSContext*)scriptContext->GetNativeContext(),
@@ -248,52 +220,97 @@ nsresult nsJSThunk::EvaluateScript(nsIChannel *aChannel)
         if (principal != systemPrincipal) {
             rv = securityManager->CheckSameOriginPrincipal(principal,
                                                            objectPrincipal);
-            if (NS_FAILED(rv)) {
-                nsCOMPtr<nsIConsoleService> console =
-                    do_GetService("@mozilla.org/consoleservice;1");
-                if (console) {
-                    // XXX Localize me!
-                    console->LogStringMessage(
-                        NS_LITERAL_STRING("Attempt to load a javascript: URL from one host\nin a window displaying content from another host\nwas blocked by the security manager.").get());
-                }
-
-                return NS_ERROR_DOM_RETVAL_UNDEFINED;
+            if (NS_SUCCEEDED(rv)) {
+                useSandbox = PR_FALSE;
             }
+        } else {
+            useSandbox = PR_FALSE;
         }
     }
-    else {
-        // No owner from channel, use the current URI to generate a principal
-        rv = securityManager->GetCodebasePrincipal(mURI, 
-                                                   getter_AddRefs(principal));
-        if (NS_FAILED(rv) || !principal) {
-            return NS_ERROR_FAILURE;
-        }
-    }
+
+    nsString result;
+    PRBool isUndefined;
 
     // Finally, we have everything needed to evaluate the expression.
-    nsString result;
-    PRBool bIsUndefined;
 
-    rv = scriptContext->EvaluateString(NS_ConvertUTF8toUCS2(script),
-                                       globalJSObject, // obj
-                                       principal,
-                                       url.get(),      // url
-                                       1,              // line no
-                                       nsnull,
-                                       &result,
-                                       &bIsUndefined);
+    if (useSandbox) {
+        // No owner from channel, or we have a principal
+        // mismatch. Evaluate the javascript URL in a sandbox to
+        // prevent it from accessing data it doesn't have permissions
+        // to access.
+
+        nsIXPConnect *xpc = nsContentUtils::XPConnect();
+
+        JSContext *cx = (JSContext*)scriptContext->GetNativeContext();
+        nsCOMPtr<nsIXPConnectJSObjectHolder> sandbox;
+        rv = xpc->CreateSandbox(cx, principal, getter_AddRefs(sandbox));
+        NS_ENSURE_SUCCESS(rv, rv);
+
+        jsval rval = JSVAL_VOID;
+        nsAutoGCRoot root(&rval, &rv);
+        if (NS_FAILED(rv)) {
+            return rv;
+        }
+
+        rv = xpc->EvalInSandboxObject(NS_ConvertUTF8toUTF16(script), cx,
+                                      sandbox, &rval);
+
+        // Propagate and report exceptions that happened in the
+        // sandbox.
+        if (JS_IsExceptionPending(cx)) {
+            JS_ReportPendingException(cx);
+        }
+
+        isUndefined = rval == JSVAL_VOID;
+
+        if (!isUndefined && NS_SUCCEEDED(rv)) {
+            JSAutoRequest ar(cx);
+
+            JSString *str = JS_ValueToString(cx, rval);
+            if (!str) {
+                // Report any pending exceptions.
+                if (JS_IsExceptionPending(cx)) {
+                    JS_ReportPendingException(cx);
+                }
+
+                // We don't know why this failed, so just use a
+                // generic error code. It'll be translated to a
+                // different one below anyways.
+                rv = NS_ERROR_FAILURE;
+            } else {
+                result = nsDependentJSString(str);
+            }
+        }
+    } else {
+        // No need to use the sandbox, evaluate the script directly in
+        // the given scope.
+        rv = scriptContext->EvaluateString(NS_ConvertUTF8toUTF16(script),
+                                           globalJSObject, // obj
+                                           principal,
+                                           url.get(),      // url
+                                           1,              // line no
+                                           nsnull,
+                                           &result,
+                                           &isUndefined);
+    }
 
     if (NS_FAILED(rv)) {
         rv = NS_ERROR_MALFORMED_URI;
     }
-    else if (bIsUndefined) {
+    else if (isUndefined) {
         rv = NS_ERROR_DOM_RETVAL_UNDEFINED;
     }
     else {
-        // NS_NewStringInputStream calls ToNewCString
-        // XXXbe this should not decimate! pass back UCS-2 to necko
-        rv = NS_NewStringInputStream(getter_AddRefs(mInnerStream), result);
+        PRUint32 resultUTF8len;
+        char *resultUTF8 = ToNewUTF8String(result, &resultUTF8len);
+        if (resultUTF8)
+            rv = NS_NewByteInputStream(getter_AddRefs(mInnerStream),
+                                       resultUTF8, resultUTF8len,
+                                       NS_ASSIGNMENT_ADOPT);
+        else
+            rv = NS_ERROR_OUT_OF_MEMORY;
     }
+
     return rv;
 }
 
@@ -303,7 +320,7 @@ nsresult nsJSThunk::BringUpConsole(nsIDOMWindow *aDomWindow)
 
     // First, get the Window Mediator service.
     nsCOMPtr<nsIWindowMediator> windowMediator =
-        do_GetService(kWindowMediatorCID, &rv);
+        do_GetService(NS_WINDOWMEDIATOR_CONTRACTID, &rv);
 
     if (NS_FAILED(rv)) return rv;
 
@@ -328,7 +345,8 @@ nsresult nsJSThunk::BringUpConsole(nsIDOMWindow *aDomWindow)
 
 ////////////////////////////////////////////////////////////////////////////////
 
-class nsJSChannel : public nsIChannel
+class nsJSChannel : public nsIChannel,
+                    public nsIStreamListener
 {
 public:
     nsJSChannel();
@@ -336,6 +354,8 @@ public:
     NS_DECL_ISUPPORTS
     NS_DECL_NSIREQUEST
     NS_DECL_NSICHANNEL
+    NS_DECL_NSIREQUESTOBSERVER
+    NS_DECL_NSISTREAMLISTENER
 
     nsresult Init(nsIURI *aURI);
 
@@ -349,6 +369,7 @@ protected:
 
 protected:
     nsCOMPtr<nsIChannel>    mStreamChannel;
+    nsCOMPtr<nsIStreamListener> mListener;  // Our final listener
 
     nsLoadFlags             mLoadFlags;
 
@@ -372,8 +393,7 @@ nsresult nsJSChannel::StopAll()
 {
     nsresult rv = NS_ERROR_UNEXPECTED;
     nsCOMPtr<nsIWebNavigation> webNav;
-    GetInterfaceFromChannel(mStreamChannel, NS_GET_IID(nsIWebNavigation),
-                            getter_AddRefs(webNav));
+    NS_QueryNotificationCallbacks(mStreamChannel, webNav);
 
     NS_ASSERTION(webNav, "Can't get nsIWebNavigation from channel!");
     if (webNav) {
@@ -400,7 +420,8 @@ nsresult nsJSChannel::Init(nsIURI *aURI)
     // If the resultant script evaluation actually does return a value, we
     // treat it as html.
     rv = NS_NewInputStreamChannel(getter_AddRefs(channel), aURI, mIOThunk,
-                                  NS_LITERAL_CSTRING("text/html"));
+                                  NS_LITERAL_CSTRING("text/html"),
+                                  NS_LITERAL_CSTRING("UTF-8"));
     if (NS_FAILED(rv)) return rv;
 
     rv = mIOThunk->Init(aURI);
@@ -422,6 +443,8 @@ NS_INTERFACE_MAP_BEGIN(nsJSChannel)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIChannel)
     NS_INTERFACE_MAP_ENTRY(nsIRequest)
     NS_INTERFACE_MAP_ENTRY(nsIChannel)
+    NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
+    NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
 NS_INTERFACE_MAP_END
 
 //
@@ -506,7 +529,12 @@ nsJSChannel::Open(nsIInputStream **aResult)
 NS_IMETHODIMP
 nsJSChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports *aContext)
 {
-    return InternalOpen(PR_TRUE, aListener, aContext, nsnull);
+    mListener = aListener;
+    nsresult rv = InternalOpen(PR_TRUE, this, aContext, nsnull);
+    if (NS_FAILED(rv)) {
+        mListener = nsnull;
+    }
+    return rv;
 }
 
 nsresult
@@ -564,8 +592,7 @@ nsJSChannel::InternalOpen(PRBool aIsAsync, nsIStreamListener *aListener,
             // ok. If so, stop all pending network loads.
 
             nsCOMPtr<nsIDocShell> docShell;
-            GetInterfaceFromChannel(mStreamChannel, NS_GET_IID(nsIDocShell),
-                                    getter_AddRefs(docShell));
+            NS_QueryNotificationCallbacks(mStreamChannel, docShell);
             if (docShell) {
                 nsCOMPtr<nsIContentViewer> cv;
                 docShell->GetContentViewer(getter_AddRefs(cv));
@@ -711,6 +738,41 @@ nsJSChannel::SetContentLength(PRInt32 aContentLength)
     return mStreamChannel->SetContentLength(aContentLength);
 }
 
+NS_IMETHODIMP
+nsJSChannel::OnStartRequest(nsIRequest* aRequest,
+                            nsISupports* aContext)
+{
+    NS_ENSURE_TRUE(aRequest == mStreamChannel, NS_ERROR_UNEXPECTED);
+
+    return mListener->OnStartRequest(this, aContext);    
+}
+
+NS_IMETHODIMP
+nsJSChannel::OnDataAvailable(nsIRequest* aRequest,
+                             nsISupports* aContext, 
+                             nsIInputStream* aInputStream,
+                             PRUint32 aOffset,
+                             PRUint32 aCount)
+{
+    NS_ENSURE_TRUE(aRequest == mStreamChannel, NS_ERROR_UNEXPECTED);
+
+    return mListener->OnDataAvailable(this, aContext, aInputStream, aOffset,
+                                      aCount);
+}
+
+NS_IMETHODIMP
+nsJSChannel::OnStopRequest(nsIRequest* aRequest,
+                           nsISupports* aContext,
+                           nsresult aStatus)
+{
+    NS_ENSURE_TRUE(aRequest == mStreamChannel, NS_ERROR_UNEXPECTED);
+
+    // Make sure to drop our ref to mListener
+    nsCOMPtr<nsIStreamListener> listener;
+    listener.swap(mListener);
+    
+    return listener->OnStopRequest(this, aContext, aStatus);
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -765,7 +827,7 @@ nsJSProtocolHandler::EnsureUTF8Spec(const nsAFlatCString &aSpec, const char *aCh
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!IsASCII(uStr))
-    NS_EscapeURL(NS_ConvertUCS2toUTF8(uStr), esc_AlwaysCopy | esc_OnlyNonASCII, aUTF8Spec);
+    NS_EscapeURL(NS_ConvertUTF16toUTF8(uStr), esc_AlwaysCopy | esc_OnlyNonASCII, aUTF8Spec);
 
   return NS_OK;
 }
@@ -790,7 +852,7 @@ nsJSProtocolHandler::GetDefaultPort(PRInt32 *result)
 NS_IMETHODIMP
 nsJSProtocolHandler::GetProtocolFlags(PRUint32 *result)
 {
-    *result = URI_NORELATIVE | URI_NOAUTH;
+    *result = URI_NORELATIVE | URI_NOAUTH | URI_HAS_NO_SECURITY_CONTEXT;
     return NS_OK;
 }
 
@@ -807,7 +869,7 @@ nsJSProtocolHandler::NewURI(const nsACString &aSpec,
     // CreateInstance.
 
     nsIURI* url;
-    rv = CallCreateInstance(kSimpleURICID, &url);
+    rv = CallCreateInstance(NS_SIMPLEURI_CONTRACTID, &url);
 
     if (NS_FAILED(rv))
         return rv;
