@@ -56,6 +56,7 @@
 #include "nsUnicharUtils.h"
 #include "nsHTMLCSSUtils.h"
 #include "nsColor.h"
+#include "nsAttrName.h"
 
 static
 void ProcessBValue(const nsAString * aInputString, nsAString & aOutputString,
@@ -474,7 +475,7 @@ nsHTMLCSSUtils::SetCSSProperty(nsIDOMElement *aElement, nsIAtom * aProperty, con
       result = mHTMLEditor->DoTransaction(txn);
     }
   }
-  // The transaction system (if any) has taken ownwership of txn
+  // The transaction system (if any) has taken ownership of txn
   NS_IF_RELEASE(txn);
   return result;
 }
@@ -508,7 +509,7 @@ nsHTMLCSSUtils::RemoveCSSProperty(nsIDOMElement *aElement, nsIAtom * aProperty,
       result = mHTMLEditor->DoTransaction(txn);
     }
   }
-  // The transaction system (if any) has taken ownwership of txn
+  // The transaction system (if any) has taken ownership of txn
   NS_IF_RELEASE(txn);
   return result;
 }
@@ -565,11 +566,11 @@ nsHTMLCSSUtils::GetCSSInlinePropertyBase(nsIDOMNode *aNode, nsIAtom *aProperty,
   switch (aStyleType) {
     case COMPUTED_STYLE_TYPE:
       if (element && aViewCSS) {
-        nsAutoString empty, value, propString;
+        nsAutoString value, propString;
         nsCOMPtr<nsIDOMCSSStyleDeclaration> cssDecl;
         aProperty->ToString(propString);
         // Get the all the computed css styles attached to the element node
-        res = aViewCSS->GetComputedStyle(element, empty, getter_AddRefs(cssDecl));
+        res = aViewCSS->GetComputedStyle(element, EmptyString(), getter_AddRefs(cssDecl));
         if (NS_FAILED(res)) return res;
         // from these declarations, get the one we want and that one only
         res = cssDecl->GetPropertyValue(propString, value);
@@ -614,9 +615,11 @@ nsHTMLCSSUtils::GetDefaultViewCSS(nsIDOMNode *aNode, nsIDOMViewCSS **aViewCSS)
       // from the document, get the abtractView
       res = documentView->GetDefaultView(getter_AddRefs(abstractView));
       if (NS_FAILED(res)) return res;
-      // from the abstractView, get the CSS view
-      CallQueryInterface(abstractView, aViewCSS);
-      return NS_OK;
+      if (abstractView) {
+        // from the abstractView, get the CSS view
+        CallQueryInterface(abstractView, aViewCSS);
+        return NS_OK;
+      }
     }
   }
   *aViewCSS = nsnull;
@@ -658,14 +661,7 @@ nsHTMLCSSUtils::RemoveCSSInlineStyle(nsIDOMNode *aNode, nsIAtom *aProperty, cons
     }
     else if (1 == attrCount) {
       // incredible hack in case the only remaining attribute is a _moz_dirty...
-      PRInt32 nameSpaceID;
-      nsCOMPtr<nsIAtom> attrName, prefix;
-      res = content->GetAttrNameAt(0, &nameSpaceID, getter_AddRefs(attrName),
-                                   getter_AddRefs(prefix));
-      if (NS_FAILED(res)) return res;
-      nsAutoString attrString, tmp;
-      attrName->ToString(attrString);
-      if (attrString.EqualsLiteral("_moz_dirty")) {
+      if (content->GetAttrNameAt(0)->Equals(nsEditProperty::mozdirty)) {
         res = mHTMLEditor->RemoveContainer(aNode);
         if (NS_FAILED(res)) return res;
       }
