@@ -48,6 +48,7 @@
 #include "nsITableEditor.h"
 #include "nsIEditorMailSupport.h"
 #include "nsIEditorStyleSheets.h"
+#include "nsITextServicesDocument.h"
 
 #include "nsEditor.h"
 #include "nsIDOMElement.h"
@@ -60,7 +61,6 @@
 
 #include "nsEditProperty.h"
 #include "nsHTMLCSSUtils.h"
-#include "nsIParserService.h"
 
 #include "nsVoidArray.h"
 
@@ -83,6 +83,7 @@ class TypeInState;
 class nsIContentFilter;
 class nsIURL;
 class nsIRangeUtils;
+class nsILinkHandler;
 
 /**
  * The HTML editor implementation.<br>
@@ -179,11 +180,6 @@ public:
 
   NS_IMETHOD GetNextElementByTagName(nsIDOMElement *aCurrentElement, const nsAString *aTagName, nsIDOMElement **aReturn);
 
-  /* ------------ nsIEditorIMESupport overrides -------------- */
-  
-  NS_IMETHOD SetCompositionString(const nsAString& aCompositionString, nsIPrivateTextRangeList* aTextRangeList,nsTextEventReply* aReply);
-  NS_IMETHOD GetReconversionString(nsReconversionEventReply* aReply);
-
   /* ------------ nsIEditorStyleSheets methods -------------- */
 
   NS_IMETHOD AddStyleSheet(const nsAString & aURL);
@@ -326,10 +322,6 @@ public:
   /** Internal, static version */
   static nsresult NodeIsBlockStatic(nsIDOMNode *aNode, PRBool *aIsBlock);
 
-
-  /** we override this here to install event listeners */
-  NS_IMETHOD PostCreate();
-
   NS_IMETHOD GetFlags(PRUint32 *aFlags);
   NS_IMETHOD SetFlags(PRUint32 aFlags);
 
@@ -349,6 +341,9 @@ public:
   /** returns PR_TRUE if aParentTag can contain a child of type aChildTag */
   virtual PRBool TagCanContainTag(const nsAString& aParentTag, const nsAString& aChildTag);
   
+  /** returns PR_TRUE if aNode is a container */
+  virtual PRBool IsContainer(nsIDOMNode *aNode);
+
   /** make the given selection span the entire document */
   NS_IMETHOD SelectEntireDocument(nsISelection *aSelection);
 
@@ -371,7 +366,8 @@ public:
                            PRUint32             aLength);
 
   /* ------------ nsICSSLoaderObserver -------------- */
-  NS_IMETHOD StyleSheetLoaded(nsICSSStyleSheet*aSheet, PRBool aNotify);
+  NS_IMETHOD StyleSheetLoaded(nsICSSStyleSheet*aSheet, PRBool aWasAlternate,
+                              nsresult aStatus);
 
   /* ------------ Utility Routines, not part of public API -------------- */
   NS_IMETHOD TypedText(const nsAString& aString, PRInt32 aAction);
@@ -443,12 +439,10 @@ protected:
 
   NS_IMETHOD  InitRules();
 
-  /** install the event listeners for the editor 
-    * used to be part of Init, but now broken out into a separate method
-    * called by PostCreate, giving the caller the chance to interpose
-    * their own listeners before we install our own backstops.
-    */
-  NS_IMETHOD InstallEventListeners();
+  // Create the event listeners for the editor to install
+  virtual nsresult CreateEventListeners();
+
+  virtual void RemoveEventListeners();
 
   /** returns the layout object (nsIFrame in the real world) for aNode
     * @param aNode          the content to get a frame for
@@ -629,8 +623,10 @@ protected:
                                         const nsAString & aContextStr,
                                         const nsAString & aInfoStr,
                                         nsCOMPtr<nsIDOMNode> *outFragNode,
-                                        PRInt32 *outRangeStartHint,
-                                        PRInt32 *outRangeEndHint);
+                                        nsCOMPtr<nsIDOMNode> *outStartNode,
+                                        nsCOMPtr<nsIDOMNode> *outEndNode,
+                                        PRInt32 *outStartOffset,
+                                        PRInt32 *outEndOffset);
   nsresult   ParseFragment(const nsAString & aStr, nsVoidArray &aTagStack,
                            nsIDocument* aTargetDoc,
                            nsCOMPtr<nsIDOMNode> *outNode);
@@ -658,9 +654,6 @@ protected:
                                      PRInt32 aHighWaterMark);
   nsIDOMNode* GetArrayEndpoint(PRBool aEnd, nsCOMArray<nsIDOMNode>& aNodeArray);
 
-  /** simple utility to handle any error with event listener allocation or registration */
-  void HandleEventListenerError();
-
   /* small utility routine to test if a break node is visible to user */
   PRBool   IsVisBreak(nsIDOMNode *aNode);
 
@@ -745,8 +738,6 @@ protected:
   nsresult GetFirstEditableLeaf( nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *aOutFirstLeaf);
   nsresult GetLastEditableLeaf( nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *aOutLastLeaf);
 
-  nsresult GetDOMEventReceiver(nsIDOMEventReceiver **aEventReceiver);
-
   //XXX Kludge: Used to suppress spurious drag/drop events (bug 50703)
   PRBool   mIgnoreSpuriousDragEvent;
 
@@ -768,21 +759,12 @@ protected:
 
   TypeInState*         mTypeInState;
 
-  nsCOMPtr<nsIAtom>    mBoldAtom;
-  nsCOMPtr<nsIAtom>    mItalicAtom;
-  nsCOMPtr<nsIAtom>    mUnderlineAtom;
-  nsCOMPtr<nsIAtom>    mFontAtom;
-  nsCOMPtr<nsIAtom>    mLinkAtom;
-
   nsCOMPtr<nsIDOMNode> mCachedNode;
-  
-  PRBool   mCachedBoldStyle;
-  PRBool   mCachedItalicStyle;
-  PRBool   mCachedUnderlineStyle;
-  nsString mCachedFontName;
 
-  // Used to disable HTML formatting commands during HTML source editing
-  PRBool   mCanEditHTML;
+  PRPackedBool mCRInParagraphCreatesParagraph;
+
+  PRPackedBool mCSSAware;
+  nsHTMLCSSUtils *mHTMLCSSUtils;
 
   // Used by GetFirstSelectedCell and GetNextSelectedCell
   PRInt32  mSelectedCellIndex;
@@ -790,19 +772,15 @@ protected:
   nsString mLastStyleSheetURL;
   nsString mLastOverrideStyleSheetURL;
 
-  PRBool mCSSAware;
-  nsHTMLCSSUtils *mHTMLCSSUtils;
-
   // Maintain a list of associated style sheets and their urls.
   nsStringArray mStyleSheetURLs;
   nsCOMArray<nsICSSStyleSheet> mStyleSheets;
-  PRInt32 mNumStyleSheets;
   
   // an array for holding default style settings
   nsVoidArray mDefaultStyles;
 
-  // Maintain a static parser service ...
-  static nsIParserService* sParserService;
+   // for real-time spelling
+   nsCOMPtr<nsITextServicesDocument> mTextServices;
 
   // And a static range utils service
   static nsIRangeUtils* sRangeHelper;
@@ -814,9 +792,15 @@ public:
 protected:
 
   /* ANONYMOUS UTILS */
+  void     RemoveListenerAndDeleteRef(const nsAString& aEvent,
+                                      nsIDOMEventListener* aListener,
+                                      PRBool aUseCapture,
+                                      nsIDOMElement* aElement,
+                                      nsIContent* aParentContent,
+                                      nsIPresShell* aShell);
   void     DeleteRefToAnonymousNode(nsIDOMElement* aElement,
                                     nsIContent * aParentContent,
-                                    nsIDocumentObserver * aDocObserver);
+                                    nsIPresShell* aShell);
   nsresult GetElementOrigin(nsIDOMElement * aElement, PRInt32 & aX, PRInt32 & aY);
   nsresult GetPositionAndDimensions(nsIDOMElement * aElement,
                                     PRInt32 & aX, PRInt32 & aY,
@@ -866,7 +850,6 @@ protected:
   nsCOMPtr<nsIDOMElement> mResizedObject;
 
   nsCOMPtr<nsIDOMEventListener>  mMouseMotionListenerP;
-  nsCOMPtr<nsIDOMEventListener>  mMutationListenerP;
   nsCOMPtr<nsISelectionListener> mSelectionListenerP;
   nsCOMPtr<nsIDOMEventListener>  mResizeEventListenerP;
 
@@ -964,6 +947,8 @@ protected:
   void     AddMouseClickListener(nsIDOMElement * aElement);
   void     RemoveMouseClickListener(nsIDOMElement * aElement);
 
+  nsCOMPtr<nsILinkHandler> mLinkHandler;
+
 public:
 
 // friends
