@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=4 sw=2 et tw=78: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -55,12 +56,6 @@
 #include "nsIPrefService.h"
 #include "nsILookAndFeel.h"
 #include "nsPresContext.h"
-#ifdef USE_HACK_REPAINT
-// for repainting hack only
-#include "nsIView.h"
-#include "nsIViewManager.h"
-// end repainting hack only
-#endif
 
 // Drag & Drop, Clipboard
 #include "nsIServiceManager.h"
@@ -155,7 +150,14 @@ nsTextEditorKeyListener::KeyPress(nsIDOMEvent* aKeyEvent)
   {
     if (flags & nsIPlaintextEditor::eEditorReadonlyMask || 
         flags & nsIPlaintextEditor::eEditorDisabledMask) 
+    {
+      // consume backspace for disabled and readonly textfields, to prevent
+      // back in history, which could be confusing to users
+      if (keyCode == nsIDOMKeyEvent::DOM_VK_BACK_SPACE)
+        aKeyEvent->PreventDefault();
+
       return NS_OK;
+    }
   }
   else
     return NS_ERROR_FAILURE;  // Editor unable to handle this.
@@ -275,20 +277,31 @@ nsTextEditorMouseListener::HandleEvent(nsIDOMEvent* aEvent)
   return NS_OK;
 }
 
-
-
 nsresult
 nsTextEditorMouseListener::MouseClick(nsIDOMEvent* aMouseEvent)
 {
-  nsCOMPtr<nsIDOMMouseEvent> mouseEvent ( do_QueryInterface(aMouseEvent) );
-  nsCOMPtr<nsIDOMNSEvent> nsEvent ( do_QueryInterface(aMouseEvent) );
+  nsCOMPtr<nsIDOMMouseEvent> mouseEvent = do_QueryInterface(aMouseEvent);
+  nsCOMPtr<nsIDOMNSEvent> nsevent = do_QueryInterface(aMouseEvent);
   PRBool isTrusted = PR_FALSE;
-  if (!mouseEvent || !nsEvent ||
-      NS_FAILED(nsEvent->GetIsTrusted(&isTrusted)) || !isTrusted) {
-    //non-ui, or non-trusted evet passed in.  bad things.
+  if (!mouseEvent || !nsevent ||
+      NS_FAILED(nsevent->GetIsTrusted(&isTrusted)) || !isTrusted) {
+    // Non-ui or non-trusted event passed in. Bad things.
     return NS_OK;
   }
 
+  nsresult rv;
+  nsCOMPtr<nsIDOMNSUIEvent> nsuiEvent = do_QueryInterface(aMouseEvent);
+  if (!nsuiEvent)
+    return NS_ERROR_NULL_POINTER;
+
+  PRBool preventDefault;
+  rv = nsuiEvent->GetPreventDefault(&preventDefault);
+  if (NS_FAILED(rv) || preventDefault)
+  {
+    // We're done if 'preventdefault' is true (see for example bug 70698).
+    return rv;
+  }
+
   nsCOMPtr<nsIEditor> editor = do_QueryInterface(mEditor);
   if (!editor) { return NS_OK; }
 
@@ -303,7 +316,6 @@ nsTextEditorMouseListener::MouseClick(nsIDOMEvent* aMouseEvent)
   // middle-mouse click (paste);
   if (button == 1)
   {
-    nsresult rv;
     nsCOMPtr<nsIPrefBranch> prefBranch =
       do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv) && prefBranch)
@@ -313,9 +325,6 @@ nsTextEditorMouseListener::MouseClick(nsIDOMEvent* aMouseEvent)
       if (NS_SUCCEEDED(rv) && doMiddleMousePaste)
       {
         // Set the selection to the point under the mouse cursor:
-        nsCOMPtr<nsIDOMNSUIEvent> nsuiEvent (do_QueryInterface(aMouseEvent));
-        if (!nsuiEvent)
-          return NS_ERROR_NULL_POINTER;
         nsCOMPtr<nsIDOMNode> parent;
         if (NS_FAILED(nsuiEvent->GetRangeParent(getter_AddRefs(parent))))
           return NS_ERROR_NULL_POINTER;
@@ -349,14 +358,9 @@ nsTextEditorMouseListener::MouseClick(nsIDOMEvent* aMouseEvent)
         else
           editor->Paste(clipboard);
 
-        // Prevent the event from bubbling up to be possibly handled
+        // Prevent the event from propagating up to be possibly handled
         // again by the containing window:
-        nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(mouseEvent));
-
-        if (nsevent) {
-          nsevent->PreventBubble();
-        }
-
+        mouseEvent->StopPropagation();
         mouseEvent->PreventDefault();
 
         // We processed the event, whether drop/paste succeeded or not
@@ -370,6 +374,10 @@ nsTextEditorMouseListener::MouseClick(nsIDOMEvent* aMouseEvent)
 nsresult
 nsTextEditorMouseListener::MouseDown(nsIDOMEvent* aMouseEvent)
 {
+  nsCOMPtr<nsIEditorIMESupport> imeEditor = do_QueryInterface(mEditor);
+  if (!imeEditor)
+    return NS_OK;
+  imeEditor->ForceCompositionEnd();
   return NS_OK;
 }
 
@@ -514,15 +522,18 @@ nsTextEditorDragListener::DragGesture(nsIDOMEvent* aDragEvent)
 nsresult
 nsTextEditorDragListener::DragEnter(nsIDOMEvent* aDragEvent)
 {
-  if (mPresShell)
+  if (!mCaret)
   {
-    if (!mCaret)
+    nsCOMPtr<nsIPresShell> presShell = do_QueryReferent(mPresShell);
+    if (presShell)
     {
       mCaret = do_CreateInstance("@mozilla.org/layout/caret;1");
       if (mCaret)
       {
-        mCaret->Init(mPresShell);
+        mCaret->Init(presShell);
         mCaret->SetCaretReadOnly(PR_TRUE);
+
+        mOtherCaret = presShell->SetCaret(mCaret);
       }
       mCaretDrawn = PR_FALSE;
     }
@@ -624,7 +635,15 @@ nsTextEditorDragListener::DragDrop(nsIDOMEvent* aMouseEvent)
       mCaretDrawn = PR_FALSE;
     }
     mCaret->SetCaretVisible(PR_FALSE);    // hide it, so that it turns off its timer
-    mCaret = nsnull;      // release it
+
+    nsCOMPtr<nsIPresShell> presShell = do_QueryReferent(mPresShell);
+    if (presShell)
+    {
+      NS_ASSERTION(mOtherCaret, "Where'd my other caret go?");
+      mCaret = presShell->SetCaret(mOtherCaret);
+    }
+
+    mOtherCaret = mCaret = nsnull;
   }
 
   if (!mEditor)
@@ -649,11 +668,7 @@ nsTextEditorDragListener::DragDrop(nsIDOMEvent* aMouseEvent)
     return NS_OK;
   }
 
-  //some day we want to use another way to stop this from bubbling.
-  nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(aMouseEvent));
-  if (nsevent)
-    nsevent->PreventBubble();
-
+  aMouseEvent->StopPropagation();
   aMouseEvent->PreventDefault();
   return mEditor->InsertFromDrop(aMouseEvent);
 }
@@ -843,6 +858,24 @@ nsTextEditorCompositionListener::HandleQueryReconversion(nsIDOMEvent* aReconvers
   return mEditor->GetReconversionString(eventReply);
 }
 
+nsresult
+nsTextEditorCompositionListener::HandleQueryCaretRect(nsIDOMEvent* aQueryCaretRectEvent)
+{
+#ifdef DEBUG_IME
+  printf("nsTextEditorCompositionListener::HandleQueryCaretRect\n");
+#endif
+  nsCOMPtr<nsIPrivateCompositionEvent> pCompositionEvent = do_QueryInterface(aQueryCaretRectEvent);
+  if (!pCompositionEvent)
+    return NS_ERROR_FAILURE;
+
+  nsQueryCaretRectEventReply* eventReply;
+  nsresult rv = pCompositionEvent->GetQueryCaretRectReply(&eventReply);
+  if (NS_FAILED(rv))
+    return rv;
+
+  return mEditor->GetQueryCaretRect(eventReply);
+}
+
 /*
  * Factory functions
  */
@@ -991,6 +1024,7 @@ IsTargetFocused(nsIDOMEventTarget* aTarget)
 nsresult
 nsTextEditorFocusListener::Focus(nsIDOMEvent* aEvent)
 {
+  NS_ENSURE_ARG(aEvent);
   // It's possible for us to receive a focus when we're really not focused.
   // This happens, for example, when an onfocus handler that's hooked up
   // before this listener focuses something else.  In that case, all of the
@@ -1008,10 +1042,7 @@ nsTextEditorFocusListener::Focus(nsIDOMEvent* aEvent)
   // turn on selection and caret
   if (mEditor)
   {
-    nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(aEvent));
-    if (nsevent) {
-      nsevent->PreventBubble();
-    }
+    aEvent->StopPropagation();
 
     PRUint32 flags;
     mEditor->GetFlags(&flags);
@@ -1024,25 +1055,11 @@ nsTextEditorFocusListener::Focus(nsIDOMEvent* aEvent)
         editor->GetSelectionController(getter_AddRefs(selCon));
         if (selCon)
         {
-          if (! (flags & nsIPlaintextEditor::eEditorReadonlyMask)) {
-            selCon->SetCaretEnabled(PR_TRUE);
-          }
-
+          const PRBool kIsReadonly = (flags & nsIPlaintextEditor::eEditorReadonlyMask) != 0;
+          selCon->SetCaretReadOnly(kIsReadonly);
+          selCon->SetCaretEnabled(PR_TRUE);
           selCon->SetDisplaySelection(nsISelectionController::SELECTION_ON);
-#ifdef USE_HACK_REPAINT
-  // begin hack repaint
-          nsIViewManager* viewmgr = ps->GetViewManager();
-          if (viewmgr) {
-            nsIView* view;
-            viewmgr->GetRootView(view);         // views are not refCounted
-            if (view) {
-              viewmgr->UpdateView(view,NS_VMREFRESH_IMMEDIATE);
-            }
-          }
-  // end hack repaint
-#else
           selCon->RepaintSelection(nsISelectionController::SELECTION_NORMAL);
-#endif
         }
       }
     }
@@ -1057,19 +1074,16 @@ nsTextEditorFocusListener::Focus(nsIDOMEvent* aEvent)
 nsresult
 nsTextEditorFocusListener::Blur(nsIDOMEvent* aEvent)
 {
+  NS_ENSURE_ARG(aEvent);
   // turn off selection and caret
   if (mEditor)
   {
-    nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(aEvent));
-    if (nsevent) {
-      nsevent->PreventBubble();
-    }
+    aEvent->StopPropagation();
 
     // when imeEditor exists, call ForceCompositionEnd() to tell
     // the input focus is leaving first
     nsCOMPtr<nsIEditorIMESupport> imeEditor = do_QueryInterface(mEditor);
     if (imeEditor) {
-      imeEditor->ForceCompositionEnd();
       imeEditor->NotifyIMEOnBlur();
     }
 
@@ -1097,21 +1111,7 @@ nsTextEditorFocusListener::Blur(nsIDOMEvent* aEvent)
           selCon->SetDisplaySelection(nsISelectionController::SELECTION_DISABLED);
         }
 
-#ifdef USE_HACK_REPAINT
-// begin hack repaint
-        nsIViewManager* viewmgr = ps->GetViewManager();
-        if (viewmgr) 
-        {
-          nsIView* view;
-          viewmgr->GetRootView(view);         // views are not refCounted
-          if (view) {
-            viewmgr->UpdateView(view,NS_VMREFRESH_IMMEDIATE);
-          }
-        }
-// end hack repaint
-#else
         selCon->RepaintSelection(nsISelectionController::SELECTION_NORMAL);
-#endif
       }
     }
   }
