@@ -47,7 +47,7 @@
 #include "nsIDOMMouseEvent.h"
 #include "nsISelection.h"
 #include "nsCRT.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 
 #include "nsIDOMRange.h"
 #include "nsIDOMNSRange.h"
@@ -63,7 +63,7 @@
 
 // Misc
 #include "nsEditorUtils.h"
-
+#include "nsContentCID.h"
 
 NS_IMETHODIMP nsPlaintextEditor::PrepareTransferable(nsITransferable **transferable)
 {
@@ -131,7 +131,7 @@ NS_IMETHODIMP nsPlaintextEditor::InsertTextFromTransferable(nsITransferable *aTr
       rv = InsertTextAt(stuffToPaste, aDestinationNode, aDestOffset, aDoDeleteSelection);
     }
   }
-  nsCRT::free(bestFlavor);
+  NS_Free(bestFlavor);
       
   // Try to scroll the selection into view if the paste/drop succeeded
   if (NS_SUCCEEDED(rv))
@@ -408,9 +408,7 @@ NS_IMETHODIMP nsPlaintextEditor::DoDrag(nsIDOMEvent *aDragEvent)
   rv = dragService->InvokeDragSession(domnode, transferableArray, nsnull, flags);
   if (NS_FAILED(rv)) return rv;
 
-  nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(aDragEvent));
-  if (nsevent)
-    nsevent->PreventBubble();
+  aDragEvent->StopPropagation();
 
   return rv;
 }
@@ -492,12 +490,9 @@ NS_IMETHODIMP nsPlaintextEditor::CanPaste(PRInt32 aSelectionType, PRBool *aCanPa
 nsresult
 nsPlaintextEditor::SetupDocEncoder(nsIDocumentEncoder **aDocEncoder)
 {
-  nsCOMPtr<nsIDOMDocument> domdoc;
-  nsresult rv = GetDocument(getter_AddRefs(domdoc));
+  nsCOMPtr<nsIDOMDocument> domDoc;
+  nsresult rv = GetDocument(getter_AddRefs(domDoc));
   if (NS_FAILED(rv)) return rv;
-	
-  nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
-  if (!doc) return NS_ERROR_FAILURE;
 
   // find out if we're a plaintext control or not
   PRUint32 editorFlags = 0;
@@ -522,7 +517,7 @@ nsPlaintextEditor::SetupDocEncoder(nsIDocumentEncoder **aDocEncoder)
   if (!encoder)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  rv = encoder->Init(doc, mimeType, docEncoderFlags);
+  rv = encoder->Init(domDoc, mimeType, docEncoderFlags);
   if (NS_FAILED(rv)) return rv;
     
   /* get the selection to be dragged */
