@@ -61,18 +61,22 @@
 #include "nsIWebBrowserPersist.h"
 #include "nsIClipboardCommands.h"
 #include "nsIProfile.h"
-#include "nsIPrintOptions.h"
-#include "nsIWebBrowserPrint.h"
 #include "nsIWidget.h"
 #include "nsIWebBrowserFocus.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsIComponentRegistrar.h"
 
+#ifdef NS_PRINTING
+#include "nsIPrintOptions.h"
+#include "nsIWebBrowserPrint.h"
+#endif
+
 #include "nsIDOMWindow.h"
 #include "nsIDOMHTMLAnchorElement.h"
 #include "nsIDOMNSDocument.h"
 
 #include "nsEmbedAPI.h"
+#include "nsEmbedCID.h"
 
 #define HACK_NON_REENTRANCY
 #ifdef HACK_NON_REENTRANCY
@@ -88,6 +92,7 @@ static HANDLE s_hHackedNonReentrancy = NULL;
 static NS_DEFINE_CID(kPromptServiceCID, NS_PROMPTSERVICE_CID);
 static NS_DEFINE_CID(kHelperAppLauncherDialogCID, NS_HELPERAPPLAUNCHERDIALOG_CID);
 
+#ifdef NS_PRINTING
 class PrintListener : public nsIWebProgressListener
 {
     PRBool mComplete;
@@ -100,6 +105,7 @@ public:
 
     void WaitForComplete();
 };
+#endif
 
 class SimpleDirectoryProvider :
     public nsIDirectoryServiceProvider
@@ -180,7 +186,7 @@ CMozillaBrowser::CMozillaBrowser()
     mBrowserHelperListCount = 0;
 
     // Name of the default profile to use
-    mProfileName.AssignLiteral("MozillaControl");
+    mProfileName.Assign(NS_LITERAL_STRING("MozillaControl"));
 
     // Initialise the web browser
     Initialize();
@@ -442,7 +448,7 @@ LRESULT CMozillaBrowser::OnCreate(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL&
                 ios->GetProtocolHandler(kDesignModeScheme, getter_AddRefs(ph));
                 if (ph &&
                     NS_SUCCEEDED(ph->GetScheme(phScheme)) &&
-                    phScheme.LowerCaseEqualsASCII(kDesignModeScheme))
+                    phScheme.Equals(NS_LITERAL_CSTRING(kDesignModeScheme)))
                 {
                     Navigate(const_cast<BSTR>(kDesignModeURL), NULL, NULL, NULL, NULL);
                 }
@@ -565,6 +571,7 @@ LRESULT CMozillaBrowser::OnPageSetup(WORD wNotifyCode, WORD wID, HWND hWndCtl, B
     NG_TRACE_METHOD(CMozillaBrowser::OnPageSetup);
 
 
+#ifdef NS_PRINTING
     nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
     nsCOMPtr<nsIPrintSettings> printSettings;
     if(!print ||
@@ -576,6 +583,7 @@ LRESULT CMozillaBrowser::OnPageSetup(WORD wNotifyCode, WORD wID, HWND hWndCtl, B
     // show the page setup dialog
     CPageSetupDlg dlg(printSettings);
     dlg.DoModal();
+#endif
 
     return 0;
 }
@@ -608,24 +616,14 @@ LRESULT CMozillaBrowser::OnSaveAs(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL
     memset(&SaveFileName, 0, sizeof(SaveFileName));
     SaveFileName.lStructSize = sizeof(SaveFileName);
     SaveFileName.hwndOwner = m_hWnd;
-    SaveFileName.hInstance = NULL;
     SaveFileName.lpstrFilter = "Web Page, HTML Only (*.htm;*.html)\0*.htm;*.html\0Text File (*.txt)\0*.txt\0"; 
-    SaveFileName.lpstrCustomFilter = NULL; 
-    SaveFileName.nMaxCustFilter = NULL; 
     SaveFileName.nFilterIndex = 1; 
     SaveFileName.lpstrFile = szFile; 
     SaveFileName.nMaxFile = sizeof(szFile); 
     SaveFileName.lpstrFileTitle = szFileTitle;
     SaveFileName.nMaxFileTitle = sizeof(szFileTitle); 
-    SaveFileName.lpstrInitialDir = NULL; 
-    SaveFileName.lpstrTitle = NULL; 
     SaveFileName.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT; 
-    SaveFileName.nFileOffset = NULL; 
-    SaveFileName.nFileExtension = NULL; 
     SaveFileName.lpstrDefExt = "htm"; 
-    SaveFileName.lCustData = NULL; 
-    SaveFileName.lpfnHook = NULL; 
-    SaveFileName.lpTemplateName = NULL; 
 
     //Get the title of the current web page to set as the default filename.
     char szTmp[_MAX_FNAME] = "untitled";
@@ -788,8 +786,8 @@ LRESULT CMozillaBrowser::OnViewSource(WORD wNotifyCode, WORD wID, HWND hWndCtl,
     nsCAutoString aURI;
     uri->GetSpec(aURI);
 
-    nsAutoString strURI(NS_LITERAL_STRING("view-source:"));
-    AppendUTF8toUTF16(aURI, strURI);
+    NS_ConvertUTF8toUTF16 strURI(aURI);
+    strURI.Insert(NS_LITERAL_STRING("view-source:"), 0);
 
     // Ask the client to create a window to view the source in
     CIPtr(IDispatch) spDispNew;
@@ -934,18 +932,27 @@ LRESULT CMozillaBrowser::OnLinkCopyShortcut(WORD wNotifyCode, WORD wID, HWND hWn
     {
         EmptyClipboard();
 
+        NS_ConvertUTF16toUTF8 curi(uri);
+        const char *stringText;
+        PRUint32 stringLen = NS_CStringGetData(curi,
+                                               &stringText);
+
         // CF_TEXT
-        HGLOBAL hmemText = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, uri.Length() + 1);
+        HGLOBAL hmemText = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT,
+                                       stringLen + 1);
         char *pszText = (char *) GlobalLock(hmemText);
-        uri.ToCString(pszText, uri.Length() + 1);
+        strncpy(pszText, stringText, stringLen);
+        pszText[stringLen] = '\0';
         GlobalUnlock(hmemText);
         SetClipboardData(CF_TEXT, hmemText);
 
         // UniformResourceLocator - CFSTR_SHELLURL
         const UINT cfShellURL = RegisterClipboardFormat(CFSTR_SHELLURL);
-        HGLOBAL hmemURL = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT, uri.Length() + 1);
+        HGLOBAL hmemURL = GlobalAlloc(GMEM_MOVEABLE | GMEM_ZEROINIT,
+                                      stringLen + 1);
         char *pszURL = (char *) GlobalLock(hmemURL);
-        uri.ToCString(pszURL, uri.Length() + 1);
+        strncpy(pszText, stringText, stringLen);
+        pszText[stringLen] = '\0';
         GlobalUnlock(hmemURL);
         SetClipboardData(cfShellURL, hmemURL);
 
@@ -1056,7 +1063,7 @@ HRESULT CMozillaBrowser::Initialize()
 
         rv = registrar->RegisterFactory(kPromptServiceCID,
             "Prompt Service",
-            "@mozilla.org/embedcomp/prompt-service;1",
+            NS_PROMPTSERVICE_CONTRACTID,
             promptFactory);
         if (NS_FAILED(rv)) return rv;
 
@@ -1491,6 +1498,7 @@ HRESULT CMozillaBrowser::UnloadBrowserHelpers()
 // Print document
 HRESULT CMozillaBrowser::PrintDocument(BOOL promptUser)
 {
+#ifdef NS_PRINTING
     // Print the contents
     nsCOMPtr<nsIWebBrowserPrint> browserAsPrint = do_GetInterface(mWebBrowser);
     NS_ASSERTION(browserAsPrint, "No nsIWebBrowserPrint!");
@@ -1525,7 +1533,7 @@ HRESULT CMozillaBrowser::PrintDocument(BOOL promptUser)
         printSettings->SetPrintSilent(oldPrintSilent);
     }
     mPrefBranch->SetBoolPref(kShowPrintProgressPref, oldShowPrintProgress);
-
+#endif
     return S_OK;
 }
 
@@ -2042,15 +2050,15 @@ NS_IMETHODIMP SimpleDirectoryProvider::GetFile(const char *prop, PRBool *persist
     nsCOMPtr<nsILocalFile> localFile;
     nsresult rv = NS_ERROR_FAILURE;
 
-    if (nsCRT::strcmp(prop, NS_APP_APPLICATION_REGISTRY_DIR) == 0)
+    if (strcmp(prop, NS_APP_APPLICATION_REGISTRY_DIR) == 0)
     {
         localFile = mApplicationRegistryDir;
     }
-    else if (nsCRT::strcmp(prop, NS_APP_APPLICATION_REGISTRY_FILE) == 0)
+    else if (strcmp(prop, NS_APP_APPLICATION_REGISTRY_FILE) == 0)
     {
         localFile = mApplicationRegistryFile;
     }
-    else if (nsCRT::strcmp(prop, NS_APP_USER_PROFILES_ROOT_DIR) == 0)
+    else if (strcmp(prop, NS_APP_USER_PROFILES_ROOT_DIR) == 0)
     {
         localFile = mUserProfileDir;
     }
@@ -2066,6 +2074,8 @@ NS_IMETHODIMP SimpleDirectoryProvider::GetFile(const char *prop, PRBool *persist
 // PrintListener implementation
 
 
+#ifdef NS_PRINTING
+
 NS_IMPL_ISUPPORTS1(PrintListener, nsIWebProgressListener)
 
 PrintListener::PrintListener() : mComplete(PR_FALSE)
@@ -2147,3 +2157,4 @@ NS_IMETHODIMP PrintListener::OnSecurityChange(nsIWebProgress *aWebProgress, nsIR
 {
     return NS_OK;
 }
+#endif
