@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,7 +14,7 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -22,16 +22,16 @@
  * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -746,18 +746,18 @@ const char kDirServiceContractID[] = "@mozilla.org/file/directory_service;1";
   }
 }
 
--(NSMenu*)getContextMenu
+-(NSMenu*)contextMenu
 {
   if ([[self superview] conformsToProtocol:@protocol(CHBrowserContainer)])
   {
     id<CHBrowserContainer> browserContainer = [self superview];
-  	return [browserContainer getContextMenu];
+  	return [browserContainer contextMenu];
   }
   
   return nil;
 }
 
--(NSWindow*)getNativeWindow
+-(NSWindow*)nativeWindow
 {
   NSWindow* window = [self window];
   if (window)
@@ -768,12 +768,12 @@ const char kDirServiceContractID[] = "@mozilla.org/file/directory_service;1";
   if (mWindow)
     return mWindow;
   
-  // Finally, see if our parent responds to the getNativeWindow selector,
+  // Finally, see if our parent responds to the nativeWindow selector,
   // and if they do, let them handle it.
   if ([[self superview] conformsToProtocol:@protocol(CHBrowserContainer)])
   {
     id<CHBrowserContainer> browserContainer = [self superview];
-    return [browserContainer getNativeWindow];
+    return [browserContainer nativeWindow];
   }
     
   return nil;
