@@ -39,9 +39,11 @@
 #include <nsIWebProgress.h>
 #include "nsIWidget.h"
 #include "nsCRT.h"
-
-// for NS_APPSHELL_CID
-#include <nsWidgetsCID.h>
+#include "nsNetUtil.h"
+#include "nsIWebBrowserStream.h"
+#include "nsIWebBrowserFocus.h"
+#include "nsIDirectoryService.h"
+#include "nsAppDirectoryServiceDefs.h"
 
 // for do_GetInterface
 #include <nsIInterfaceRequestor.h>
@@ -52,7 +54,8 @@
 #include <nsIWindowWatcher.h>
 
 #include <nsILocalFile.h>
-#include <nsEmbedAPI.h>
+
+#include "nsXULAppAPI.h"
 
 // all of the crap that we need for event listeners
 // and when chrome windows finish loading
@@ -67,9 +70,6 @@
 // for the focus hacking we need to do
 #include <nsIFocusController.h>
 
-// for profiles
-#include <nsProfileDirServiceProvider.h>
-
 // app component registration
 #include <nsIGenericFactory.h>
 #include <nsIComponentRegistrar.h>
@@ -81,7 +81,6 @@
 #include "EmbedContentListener.h"
 #include "EmbedEventListener.h"
 #include "EmbedWindowCreator.h"
-#include "EmbedStream.h"
 #ifdef MOZ_WIDGET_GTK2
 #include "GtkPromptService.h"
 #else
@@ -94,25 +93,75 @@
 #include "nsIDOMDocument.h"
 #endif
 
-#ifdef _BUILD_STATIC_BIN
-#include "nsStaticComponent.h"
-nsresult PR_CALLBACK
-gtk_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count);
-#endif
-
-static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
-
 PRUint32     EmbedPrivate::sWidgetCount = 0;
+
+char        *EmbedPrivate::sPath        = nsnull;
 char        *EmbedPrivate::sCompPath    = nsnull;
-nsIAppShell *EmbedPrivate::sAppShell    = nsnull;
 nsVoidArray *EmbedPrivate::sWindowList  = nsnull;
-char        *EmbedPrivate::sProfileDir  = nsnull;
-char        *EmbedPrivate::sProfileName = nsnull;
-nsIPref     *EmbedPrivate::sPrefs       = nsnull;
+nsILocalFile *EmbedPrivate::sProfileDir  = nsnull;
+nsISupports  *EmbedPrivate::sProfileLock = nsnull;
 GtkWidget   *EmbedPrivate::sOffscreenWindow = 0;
 GtkWidget   *EmbedPrivate::sOffscreenFixed  = 0;
+
 nsIDirectoryServiceProvider *EmbedPrivate::sAppFileLocProvider = nsnull;
-nsProfileDirServiceProvider *EmbedPrivate::sProfileDirServiceProvider = nsnull;
+
+class GTKEmbedDirectoryProvider : public nsIDirectoryServiceProvider2
+{
+public:
+  NS_DECL_ISUPPORTS_INHERITED
+  NS_DECL_NSIDIRECTORYSERVICEPROVIDER
+  NS_DECL_NSIDIRECTORYSERVICEPROVIDER2
+};
+
+static const GTKEmbedDirectoryProvider kDirectoryProvider;
+
+NS_IMPL_QUERY_INTERFACE2(GTKEmbedDirectoryProvider,
+                         nsIDirectoryServiceProvider,
+                         nsIDirectoryServiceProvider2)
+
+NS_IMETHODIMP_(nsrefcnt)
+GTKEmbedDirectoryProvider::AddRef()
+{
+  return 1;
+}
+
+NS_IMETHODIMP_(nsrefcnt)
+GTKEmbedDirectoryProvider::Release()
+{
+  return 1;
+}
+
+NS_IMETHODIMP
+GTKEmbedDirectoryProvider::GetFile(const char *aKey, PRBool *aPersist,
+                                   nsIFile* *aResult)
+{
+  if (EmbedPrivate::sAppFileLocProvider) {
+    nsresult rv = EmbedPrivate::sAppFileLocProvider->GetFile(aKey, aPersist,
+                                                             aResult);
+    if (NS_SUCCEEDED(rv))
+      return rv;
+  }
+
+  if (EmbedPrivate::sProfileDir && !strcmp(aKey, NS_APP_USER_PROFILE_50_DIR)) {
+    *aPersist = PR_TRUE;
+    return EmbedPrivate::sProfileDir->Clone(aResult);
+  }
+
+  return NS_ERROR_FAILURE;
+}
+
+NS_IMETHODIMP
+GTKEmbedDirectoryProvider::GetFiles(const char *aKey,
+                                    nsISimpleEnumerator* *aResult)
+{
+  nsCOMPtr<nsIDirectoryServiceProvider2>
+    dp2(do_QueryInterface(EmbedPrivate::sAppFileLocProvider));
+
+  if (!dp2)
+    return NS_ERROR_FAILURE;
+
+  return dp2->GetFiles(aKey, aResult);
+}
 
 #define NS_PROMPTSERVICE_CID \
  {0x95611356, 0xf583, 0x46f5, {0x81, 0xff, 0x4b, 0x3e, 0x01, 0x62, 0xc6, 0x19}}
@@ -149,7 +198,6 @@ EmbedPrivate::EmbedPrivate(void)
   mProgress         = nsnull;
   mContentListener  = nsnull;
   mEventListener    = nsnull;
-  mStream           = nsnull;
   mChromeMask       = nsIWebBrowserChrome::CHROME_ALL;
   mIsChrome         = PR_FALSE;
   mChromeLoaded     = PR_FALSE;
@@ -240,7 +288,7 @@ EmbedPrivate::Realize(PRBool *aAlreadyRealized)
   // create the offscreen window if we have to
   EnsureOffscreenWindow();
 
-  // Have we ever been initialized before?  If so then just reparetn
+  // Have we ever been initialized before?  If so then just reparent
   // from the offscreen window.
   if (mMozWindowWidget) {
     gtk_widget_reparent(mMozWindowWidget, GTK_WIDGET(mOwningWidget));
@@ -270,7 +318,7 @@ EmbedPrivate::Realize(PRBool *aAlreadyRealized)
   nsCOMPtr<nsIWeakReference> weakRef;
   supportsWeak->GetWeakReference(getter_AddRefs(weakRef));
   webBrowser->AddWebBrowserListener(weakRef,
-				    nsIWebProgressListener::GetIID());
+				    NS_GET_IID(nsIWebProgressListener));
 
   // set ourselves as the parent uri content listener
   nsCOMPtr<nsIURIContentListener> uriListener;
@@ -355,7 +403,7 @@ EmbedPrivate::Destroy(void)
   nsCOMPtr<nsIWeakReference> weakRef;
   supportsWeak->GetWeakReference(getter_AddRefs(weakRef));
   webBrowser->RemoveWebBrowserListener(weakRef,
-				       nsIWebProgressListener::GetIID());
+				       NS_GET_IID(nsIWebProgressListener));
   weakRef = nsnull;
   supportsWeak = nsnull;
 
@@ -373,7 +421,7 @@ EmbedPrivate::Destroy(void)
   DetachListeners();
   if (mEventReceiver)
     mEventReceiver = nsnull;
-  
+
   // destroy our child window
   mWindow->ReleaseChildren();
 
@@ -405,12 +453,18 @@ EmbedPrivate::SetURI(const char *aURI)
 void
 EmbedPrivate::LoadCurrentURI(void)
 {
-  if (mURI.Length())
+  if (mURI.Length()) {
+    nsCOMPtr<nsPIDOMWindow> piWin;
+    GetPIDOMWindow(getter_AddRefs(piWin));
+
+    nsAutoPopupStatePusher popupStatePusher(piWin, openAllowed);
+
     mNavigation->LoadURI(mURI.get(),                        // URI string
                          nsIWebNavigation::LOAD_FLAGS_NONE, // Load flags
                          nsnull,                            // Referring URI
                          nsnull,                            // Post data
                          nsnull);                           // extra headers
+  }
 }
 
 void
@@ -447,7 +501,7 @@ EmbedPrivate::ApplyChromeMask()
          if (scrollbars) {
 
             scrollbars->SetVisible
-               (mChromeMask & nsIWebBrowserChrome::CHROME_SCROLLBARS ? 
+               (mChromeMask & nsIWebBrowserChrome::CHROME_SCROLLBARS ?
                 PR_TRUE : PR_FALSE);
          }
       }
@@ -470,53 +524,44 @@ EmbedPrivate::PushStartup(void)
 {
   // increment the number of widgets
   sWidgetCount++;
-  
+
   // if this is the first widget, fire up xpcom
   if (sWidgetCount == 1) {
     nsresult rv;
+
     nsCOMPtr<nsILocalFile> binDir;
-    
     if (sCompPath) {
       rv = NS_NewNativeLocalFile(nsDependentCString(sCompPath), 1, getter_AddRefs(binDir));
       if (NS_FAILED(rv))
 	return;
     }
 
-#ifdef _BUILD_STATIC_BIN
-    // Initialize XPCOM's module info table
-    NSGetStaticModuleInfo = gtk_getModuleInfo;
-#endif
+    const char *grePath = sPath;
 
-    rv = NS_InitEmbedding(binDir, sAppFileLocProvider);
+    if (!grePath)
+      grePath = getenv("MOZILLA_FIVE_HOME");
+
+    if (!grePath)
+      return;
+
+    nsCOMPtr<nsILocalFile> greDir;
+    rv = NS_NewNativeLocalFile(nsDependentCString(grePath), PR_TRUE,
+                               getter_AddRefs(greDir));
     if (NS_FAILED(rv))
       return;
 
-    // we no longer need a reference to the DirectoryServiceProvider
-    if (sAppFileLocProvider) {
-      NS_RELEASE(sAppFileLocProvider);
-      sAppFileLocProvider = nsnull;
-    }
+    rv = XRE_InitEmbedding(greDir, binDir,
+                           NS_CONST_CAST(GTKEmbedDirectoryProvider*,
+                                         &kDirectoryProvider),
+                           nsnull, nsnull);
+    if (NS_FAILED(rv))
+      return;
 
-    rv = StartupProfile();
-    NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "Warning: Failed to start up profiles.\n");
+    if (sProfileDir)
+      XRE_NotifyProfile();
 
     rv = RegisterAppComponents();
-    NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "Warning: Failed to register app components.\n");
-
-    // XXX startup appshell service?
-    // XXX create offscreen window for appshell service?
-    // XXX remove X prop from offscreen window?
-    
-    nsCOMPtr<nsIAppShell> appShell;
-    appShell = do_CreateInstance(kAppShellCID);
-    if (!appShell) {
-      NS_WARNING("Failed to create appshell in EmbedPrivate::PushStartup!\n");
-      return;
-    }
-    sAppShell = appShell.get();
-    NS_ADDREF(sAppShell);
-    sAppShell->Create(0, nsnull);
-    sAppShell->Spinup();
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Warning: Failed to register app components.\n");
   }
 }
 
@@ -529,22 +574,29 @@ EmbedPrivate::PopStartup(void)
 
     // destroy the offscreen window
     DestroyOffscreenWindow();
-    
-    // shut down the profiles
-    ShutdownProfile();
 
-    if (sAppShell) {
-      // Shutdown the appshell service.
-      sAppShell->Spindown();
-      NS_RELEASE(sAppShell);
-      sAppShell = 0;
+    // we no longer need a reference to the DirectoryServiceProvider
+    if (sAppFileLocProvider) {
+      NS_RELEASE(sAppFileLocProvider);
+      sAppFileLocProvider = nsnull;
     }
 
     // shut down XPCOM/Embedding
-    NS_TermEmbedding();
+    XRE_TermEmbedding();
   }
 }
 
+/* static */
+void EmbedPrivate::SetPath(const char *aPath)
+{
+  if (sPath)
+    free(sPath);
+  if (aPath)
+    sPath = strdup(aPath);
+  else
+    sPath = nsnull;
+}
+
 /* static */
 void
 EmbedPrivate::SetCompPath(const char *aPath)
@@ -571,28 +623,44 @@ void
 EmbedPrivate::SetProfilePath(const char *aDir, const char *aName)
 {
   if (sProfileDir) {
-    nsMemory::Free(sProfileDir);
-    sProfileDir = nsnull;
+    if (sWidgetCount) {
+      NS_ERROR("Cannot change profile directory during run.");
+      return;
+    }
+
+    NS_RELEASE(sProfileDir);
+    NS_RELEASE(sProfileLock);
   }
 
-  if (sProfileName) {
-    nsMemory::Free(sProfileName);
-    sProfileName = nsnull;
+  nsresult rv =
+    NS_NewNativeLocalFile(nsDependentCString(aDir), PR_TRUE, &sProfileDir);
+
+  if (NS_SUCCEEDED(rv) && aName)
+    rv = sProfileDir->AppendNative(nsDependentCString(aName));
+
+  if (NS_SUCCEEDED(rv))
+    rv = XRE_LockProfileDirectory(sProfileDir, &sProfileLock);
+
+  if (NS_SUCCEEDED(rv)) {
+    if (sWidgetCount)
+      XRE_NotifyProfile();
+
+    return;
   }
 
-  if (aDir)
-    sProfileDir = (char *)nsMemory::Clone(aDir, strlen(aDir) + 1);
-  
-  if (aName)
-    sProfileName = (char *)nsMemory::Clone(aName, strlen(aDir) + 1);
+  NS_WARNING("Failed to lock profile.");
+
+  // Failed
+  NS_IF_RELEASE(sProfileDir);
+  NS_IF_RELEASE(sProfileLock);
 }
 
-void 
-EmbedPrivate::SetDirectoryServiceProvider(nsIDirectoryServiceProvider * appFileLocProvider) 
+void
+EmbedPrivate::SetDirectoryServiceProvider(nsIDirectoryServiceProvider * appFileLocProvider)
 {
   if (sAppFileLocProvider)
     NS_RELEASE(sAppFileLocProvider);
-  
+
   if (appFileLocProvider) {
     sAppFileLocProvider = appFileLocProvider;
     NS_ADDREF(sAppFileLocProvider);
@@ -602,48 +670,47 @@ EmbedPrivate::SetDirectoryServiceProvider(nsIDirectoryServiceProvider * appFileL
 nsresult
 EmbedPrivate::OpenStream(const char *aBaseURI, const char *aContentType)
 {
-  nsresult rv;
+  nsCOMPtr<nsIWebBrowser> webBrowser;
+  mWindow->GetWebBrowser(getter_AddRefs(webBrowser));
 
-  if (!mStream) {
-    mStream = new EmbedStream();
-    mStreamGuard = do_QueryInterface(mStream);
-    mStream->InitOwner(this);
-    rv = mStream->Init();
-    if (NS_FAILED(rv))
-      return rv;
-  }
+  nsCOMPtr<nsIWebBrowserStream> wbStream = do_QueryInterface(webBrowser);
+  if (!wbStream) return NS_ERROR_FAILURE;
 
-  rv = mStream->OpenStream(aBaseURI, aContentType);
+  nsCOMPtr<nsIURI> uri;
+  nsresult rv = NS_NewURI(getter_AddRefs(uri), aBaseURI);
+  if (NS_FAILED(rv))
+    return rv;
+
+  rv = wbStream->OpenStream(uri, nsDependentCString(aContentType));
   return rv;
 }
 
 nsresult
-EmbedPrivate::AppendToStream(const char *aData, PRInt32 aLen)
+EmbedPrivate::AppendToStream(const PRUint8 *aData, PRUint32 aLen)
 {
-  if (!mStream)
-    return NS_ERROR_FAILURE;
-
   // Attach listeners to this document since in some cases we don't
   // get updates for content added this way.
   ContentStateChange();
 
-  return mStream->AppendToStream(aData, aLen);
+  nsCOMPtr<nsIWebBrowser> webBrowser;
+  mWindow->GetWebBrowser(getter_AddRefs(webBrowser));
+
+  nsCOMPtr<nsIWebBrowserStream> wbStream = do_QueryInterface(webBrowser);
+  if (!wbStream) return NS_ERROR_FAILURE;
+
+  return wbStream->AppendToStream(aData, aLen);
 }
 
 nsresult
 EmbedPrivate::CloseStream(void)
 {
-  nsresult rv;
+  nsCOMPtr<nsIWebBrowser> webBrowser;
+  mWindow->GetWebBrowser(getter_AddRefs(webBrowser));
 
-  if (!mStream)
-    return NS_ERROR_FAILURE;
-  rv = mStream->CloseStream();
+  nsCOMPtr<nsIWebBrowserStream> wbStream = do_QueryInterface(webBrowser);
+  if (!wbStream) return NS_ERROR_FAILURE;
 
-  // release
-  mStream = 0;
-  mStreamGuard = 0;
-
-  return rv;
+  return wbStream->CloseStream();
 }
 
 /* static */
@@ -683,7 +750,7 @@ EmbedPrivate::ContentStateChange(void)
 
   if (!mEventReceiver)
     return;
-  
+
   AttachListeners();
 
 }
@@ -698,7 +765,7 @@ EmbedPrivate::ContentFinishedLoading(void)
     // get the web browser
     nsCOMPtr<nsIWebBrowser> webBrowser;
     mWindow->GetWebBrowser(getter_AddRefs(webBrowser));
-    
+
     // get the content DOM window for that web browser
     nsCOMPtr<nsIDOMWindow> domWindow;
     webBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
@@ -706,7 +773,7 @@ EmbedPrivate::ContentFinishedLoading(void)
       NS_WARNING("no dom window in content finished loading\n");
       return;
     }
-    
+
     // resize the content
     domWindow->SizeToContent();
 
@@ -719,6 +786,7 @@ EmbedPrivate::ContentFinishedLoading(void)
   }
 }
 
+#ifdef MOZ_WIDGET_GTK
 // handle focus in and focus out events
 void
 EmbedPrivate::TopLevelFocusIn(void)
@@ -753,6 +821,7 @@ EmbedPrivate::TopLevelFocusOut(void)
   if (focusController)
     focusController->SetActive(PR_FALSE);
 }
+#endif /* MOZ_WIDGET_GTK */
 
 void
 EmbedPrivate::ChildFocusIn(void)
@@ -760,6 +829,21 @@ EmbedPrivate::ChildFocusIn(void)
   if (mIsDestroyed)
     return;
 
+#ifdef MOZ_WIDGET_GTK2
+  nsresult rv;
+  nsCOMPtr<nsIWebBrowser> webBrowser;
+  rv = mWindow->GetWebBrowser(getter_AddRefs(webBrowser));
+  if (NS_FAILED(rv))
+    return;
+
+  nsCOMPtr<nsIWebBrowserFocus> webBrowserFocus(do_QueryInterface(webBrowser));
+  if (!webBrowserFocus)
+    return;
+  
+  webBrowserFocus->Activate();
+#endif /* MOZ_WIDGET_GTK2 */
+
+#ifdef MOZ_WIDGET_GTK
   nsCOMPtr<nsPIDOMWindow> piWin;
   GetPIDOMWindow(getter_AddRefs(piWin));
 
@@ -767,6 +851,7 @@ EmbedPrivate::ChildFocusIn(void)
     return;
 
   piWin->Activate();
+#endif /* MOZ_WIDGET_GTK */
 }
 
 void
@@ -775,6 +860,21 @@ EmbedPrivate::ChildFocusOut(void)
   if (mIsDestroyed)
     return;
 
+#ifdef MOZ_WIDGET_GTK2
+  nsresult rv;
+  nsCOMPtr<nsIWebBrowser> webBrowser;
+  rv = mWindow->GetWebBrowser(getter_AddRefs(webBrowser));
+  if (NS_FAILED(rv))
+	  return;
+
+  nsCOMPtr<nsIWebBrowserFocus> webBrowserFocus(do_QueryInterface(webBrowser));
+  if (!webBrowserFocus)
+	  return;
+  
+  webBrowserFocus->Deactivate();
+#endif /* MOZ_WIDGET_GTK2 */
+
+#ifdef MOZ_WIDGET_GTK
   nsCOMPtr<nsPIDOMWindow> piWin;
   GetPIDOMWindow(getter_AddRefs(piWin));
 
@@ -788,7 +888,7 @@ EmbedPrivate::ChildFocusOut(void)
   nsIFocusController *focusController = piWin->GetRootFocusController();
   if (focusController)
     focusController->SetActive(PR_TRUE);
-
+#endif /* MOZ_WIDGET_GTK */
 }
 
 // Get the event listener for the chrome event handler.
@@ -902,7 +1002,7 @@ EmbedPrivate::GetPIDOMWindow(nsPIDOMWindow **aPIWin)
   nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
   // and the root window for that DOM window
   *aPIWin = domWindowPrivate->GetPrivateRoot();
-  
+
   if (*aPIWin) {
     NS_ADDREF(*aPIWin);
     return NS_OK;
@@ -942,61 +1042,6 @@ EmbedPrivate::GetAtkObjectForCurrentDocument()
 }
 #endif /* MOZ_ACCESSIBILITY_ATK */
 
-/* static */
-nsresult
-EmbedPrivate::StartupProfile(void)
-{
-  // initialize profiles
-  if (sProfileDir && sProfileName) {
-    nsresult rv;
-    nsCOMPtr<nsILocalFile> profileDir;
-    NS_NewNativeLocalFile(nsDependentCString(sProfileDir), PR_TRUE,
-                          getter_AddRefs(profileDir));
-    if (!profileDir)
-      return NS_ERROR_FAILURE;
-    rv = profileDir->AppendNative(nsDependentCString(sProfileName));
-    if (NS_FAILED(rv))
-      return NS_ERROR_FAILURE;
-
-    nsCOMPtr<nsProfileDirServiceProvider> locProvider;
-    NS_NewProfileDirServiceProvider(PR_TRUE, getter_AddRefs(locProvider));
-    if (!locProvider)
-      return NS_ERROR_FAILURE;
-    rv = locProvider->Register();
-    if (NS_FAILED(rv))
-      return rv;
-    rv = locProvider->SetProfileDir(profileDir);
-    if (NS_FAILED(rv))
-      return rv;
-    // Keep a ref so we can shut it down.
-    NS_ADDREF(sProfileDirServiceProvider = locProvider);
-
-    // get prefs
-    nsCOMPtr<nsIPref> pref;
-    pref = do_GetService(NS_PREF_CONTRACTID);
-    if (!pref)
-      return NS_ERROR_FAILURE;
-    sPrefs = pref.get();
-    NS_ADDREF(sPrefs);
-  }
-  return NS_OK;
-}
-
-/* static */
-void
-EmbedPrivate::ShutdownProfile(void)
-{
-  if (sProfileDirServiceProvider) {
-    sProfileDirServiceProvider->Shutdown();
-    NS_RELEASE(sProfileDirServiceProvider);
-    sProfileDirServiceProvider = 0;
-  }
-  if (sPrefs) {
-    NS_RELEASE(sPrefs);
-    sPrefs = 0;
-  }
-}
-
 /* static */
 nsresult
 EmbedPrivate::RegisterAppComponents(void)
@@ -1005,6 +1050,10 @@ EmbedPrivate::RegisterAppComponents(void)
   nsresult rv = NS_GetComponentRegistrar(getter_AddRefs(cr));
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsCOMPtr<nsIComponentManager> cm;
+  rv = NS_GetComponentManager (getter_AddRefs (cm));
+  NS_ENSURE_SUCCESS (rv, rv);
+
   for (int i = 0; i < sNumAppComps; ++i) {
     nsCOMPtr<nsIGenericFactory> componentFactory;
     rv = NS_NewGenericFactory(getter_AddRefs(componentFactory),
@@ -1017,11 +1066,18 @@ EmbedPrivate::RegisterAppComponents(void)
     rv = cr->RegisterFactory(sAppComps[i].mCID, sAppComps[i].mDescription,
                              sAppComps[i].mContractID, componentFactory);
     NS_ASSERTION(NS_SUCCEEDED(rv), "Unable to register factory for component");
+
+    // Call the registration hook of the component, if any
+    if (sAppComps[i].mRegisterSelfProc) {
+      rv = sAppComps[i].mRegisterSelfProc(cm, nsnull, nsnull, nsnull,
+                                          &(sAppComps[i]));
+      NS_ASSERTION(NS_SUCCEEDED(rv), "Unable to self-register component");
+    }
   }
 
   return rv;
 }
-			     
+
 /* static */
 void
 EmbedPrivate::EnsureOffscreenWindow(void)
