@@ -1,24 +1,40 @@
-/*
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- * 
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- * 
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
  * The Original Code is mozilla.org code.
- * 
- * The Initial Developer of the Original Code is Christopher Blizzard.
- * Portions created by Christopher Blizzard are Copyright (C)
- * Christopher Blizzard.  All Rights Reserved.
- * 
+ *
+ * The Initial Developer of the Original Code is
+ * Christopher Blizzard. Portions created by Christopher Blizzard are Copyright (C) Christopher Blizzard.  All Rights Reserved.
+ * Portions created by the Initial Developer are Copyright (C) 2001
+ * the Initial Developer. All Rights Reserved.
+ *
  * Contributor(s):
  *   Christopher Blizzard <blizzard@mozilla.org>
  *   Brian Edmond <briane@qnx.com>
- */
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 #include <nsIDocShell.h>
 #include <nsIURI.h>
@@ -58,6 +74,10 @@
 #include "nsIWebBrowserPrint.h"
 #include "nsIClipboardCommands.h"
 
+// for the clipboard input group setting
+#include "nsClipboard.h"
+#include "nsWidgetsCID.h"
+
 // for the focus hacking we need to do
 #include <nsIFocusController.h>
 
@@ -80,10 +100,12 @@
 extern char *g_Print_Left_Header_String, *g_Print_Right_Header_String, *g_Print_Left_Footer_String, *g_Print_Right_Footer_String;
 
 static const char sWatcherContractID[] = "@mozilla.org/embedcomp/window-watcher;1";
+static NS_DEFINE_CID(kCClipboardCID, NS_CLIPBOARD_CID);
 
 nsIAppShell *EmbedPrivate::sAppShell    = nsnull;
 nsIPref     *EmbedPrivate::sPrefs       = nsnull;
 nsVoidArray *EmbedPrivate::sWindowList  = nsnull;
+nsClipboard *EmbedPrivate::sClipboard   = nsnull;
 
 EmbedPrivate::EmbedPrivate(void)
 {
@@ -103,6 +125,13 @@ EmbedPrivate::EmbedPrivate(void)
   		sWindowList = new nsVoidArray();
 	}
 	sWindowList->AppendElement(this);
+	if( !sClipboard ) {
+		nsresult rv;
+		nsCOMPtr<nsClipboard> s;
+		s = do_GetService( kCClipboardCID, &rv );
+		sClipboard = ( nsClipboard * ) s;
+		if( NS_FAILED( rv ) ) sClipboard = 0;
+	}
 }
 
 EmbedPrivate::~EmbedPrivate()
@@ -232,7 +261,7 @@ EmbedPrivate::Setup()
 	supportsWeak = do_QueryInterface(mProgressGuard);
 	nsCOMPtr<nsIWeakReference> weakRef;
 	supportsWeak->GetWeakReference(getter_AddRefs(weakRef));
-	webBrowser->AddWebBrowserListener(weakRef, nsIWebProgressListener::GetIID());
+	webBrowser->AddWebBrowserListener(weakRef, NS_GET_IID(nsIWebProgressListener));
 
 	// set ourselves as the parent uri content listener
 	nsCOMPtr<nsIURIContentListener> uriListener;
@@ -297,7 +326,7 @@ EmbedPrivate::Destroy(void)
   nsCOMPtr<nsIWeakReference> weakRef;
   supportsWeak->GetWeakReference(getter_AddRefs(weakRef));
   webBrowser->RemoveWebBrowserListener(weakRef,
-				       nsIWebProgressListener::GetIID());
+				       NS_GET_IID(nsIWebProgressListener));
   weakRef = nsnull;
   supportsWeak = nsnull;
 
@@ -483,30 +512,50 @@ EmbedPrivate::CanGoForward()
 }
 
 void
-EmbedPrivate::Cut()
+EmbedPrivate::Cut(int ig)
 {
 	nsCOMPtr<nsIClipboardCommands> clipboard(do_GetInterface(mWindow->mWebBrowser));
-	if (clipboard)
+	if (clipboard) {
+		//
+		// Pass Voyager input group to clipboard functions.
+		// Using Ctrl-C/V does not do this, only Edit->Copy/Paste.
+		//
+		if (sClipboard)
+			sClipboard->SetInputGroup(ig);
 	    clipboard->CutSelection();
+		}
 }
 
 void
-EmbedPrivate::Copy()
+EmbedPrivate::Copy(int ig)
 {
 	nsCOMPtr<nsIClipboardCommands> clipboard(do_GetInterface(mWindow->mWebBrowser));
-	if (clipboard)
+	if (clipboard) {
+		//
+		// Pass Voyager input group to clipboard functions.
+		// Using Ctrl-C/V does not do this, only Edit->Copy/Paste.
+		//
+		if (sClipboard)
+			sClipboard->SetInputGroup(ig);
 	    clipboard->CopySelection();
+		}
 }
 
 void
-EmbedPrivate::Paste()
+EmbedPrivate::Paste(int ig)
 {
 	nsCOMPtr<nsIClipboardCommands> clipboard(do_GetInterface(mWindow->mWebBrowser));
-	if (clipboard)
+	if (clipboard) {
+		//
+		// Pass Voyager input group to clipboard functions.
+		// Using Ctrl-C/V does not do this, only Edit->Copy/Paste.
+		//
+		if (sClipboard)
+			sClipboard->SetInputGroup(ig);
 	    clipboard->Paste();
+		}
 }
 
-#include <nsIWebShell.h>
 void
 EmbedPrivate::SelectAll()
 {
@@ -917,13 +966,13 @@ static void mozilla_set_default_pref( nsIPref *pref )
 */
 
 	/* translation set = Western European (ISO 8859-1) */
-	pref->SetUnicharPref( "intl.charset.default", NS_ConvertASCIItoUCS2("iso8859-1").get());
+	pref->SetUnicharPref( "intl.charset.default", NS_ConvertASCIItoUTF16("iso8859-1").get());
 
 	/* HTML Options */
-	pref->SetUnicharPref( "browser.visited_color", NS_ConvertASCIItoUCS2("#008080").get() );
-	pref->SetUnicharPref( "browser.anchor_color", NS_ConvertASCIItoUCS2("#0000ff").get() );
-	pref->SetUnicharPref( "browser.display.foreground_color", NS_ConvertASCIItoUCS2("#000000").get() );
-	pref->SetUnicharPref( "browser.display.background_color", NS_ConvertASCIItoUCS2("#ffffff").get() );
+	pref->SetUnicharPref( "browser.visited_color", NS_ConvertASCIItoUTF16("#008080").get() );
+	pref->SetUnicharPref( "browser.anchor_color", NS_ConvertASCIItoUTF16("#0000ff").get() );
+	pref->SetUnicharPref( "browser.display.foreground_color", NS_ConvertASCIItoUTF16("#000000").get() );
+	pref->SetUnicharPref( "browser.display.background_color", NS_ConvertASCIItoUTF16("#ffffff").get() );
 
 	pref->SetCharPref( "font.name.serif.x-western", "serif" );
 	pref->SetCharPref( "font.name.sans-serif.x-western", "sans-serif" );
@@ -940,7 +989,6 @@ static void mozilla_set_default_pref( nsIPref *pref )
 //	pref->SetIntPref( "browser.cache.check_doc_frequency", 2 );
 	pref->SetBoolPref( "browser.cache.disk.enable", PR_TRUE );
 	pref->SetIntPref( "browser.cache.disk.capacity", 5000 );
-	pref->SetIntPref( "network.http.connect.timeout", 2400 );
 	pref->SetIntPref( "network.http.max-connections", 4 );
 	pref->SetIntPref( "network.proxy.http_port", 80 );
 	pref->SetIntPref( "network.proxy.ftp_port", 80 );
