@@ -1,25 +1,41 @@
-/*
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- * 
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
- * 
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
  * The Original Code is mozilla.org code.
- * 
- * The Initial Developer of the Original Code is Christopher Blizzard.
- * Portions created by Christopher Blizzard are Copyright (C)
- * Christopher Blizzard.  All Rights Reserved.
- * 
+ *
+ * The Initial Developer of the Original Code is
+ * Christopher Blizzard. Portions created by Christopher Blizzard are Copyright (C) Christopher Blizzard.  All Rights Reserved.
+ * Portions created by the Initial Developer are Copyright (C) 2001
+ * the Initial Developer. All Rights Reserved.
+ *
  * Contributor(s):
  *   Christopher Blizzard <blizzard@mozilla.org>
  *   Ramiro Estrugo <ramiro@eazel.com>
- *	 Brian Edmond <briane@qnx.com>
- */
+ *   Brian Edmond <briane@qnx.com>
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 #include <stdlib.h>
 
 #include "nsCWebBrowser.h"
@@ -63,8 +79,8 @@
 #include "nsIURIFixup.h"
 #include "nsPIDOMWindow.h"
 
-#include "nsIEventQueueService.h"
 #include "nsIServiceManager.h"
+#include "nsIComponentRegistrar.h"
 #include "nsUnknownContentTypeHandler.h"
 
 #include "EmbedPrivate.h"
@@ -334,24 +350,25 @@ mozilla_modify( PtWidget_t *widget, PtArg_t const *argt, PtResourceRec_t const *
 			char translation[1024];
 			strcpy( translation, (char*)argt->value );
 			fix_translation_string( translation );
- 			pref->SetUnicharPref( "intl.charset.default", NS_ConvertASCIItoUCS2( translation ).get());
+ 			pref->SetUnicharPref( "intl.charset.default", NS_ConvertASCIItoUTF16( translation ).get());
 			pref->SavePrefFile( nsnull );
 			}
 			break;
 
-		case Pt_ARG_MOZ_COMMAND:
+		case Pt_ARG_MOZ_COMMAND: {
+			PtWebClient2Command_t *wdata = ( PtWebClient2Command_t * ) argt->len;
 			switch ((int)(argt->value)) 
 			{
 				case Pt_MOZ_COMMAND_CUT: {
-					moz->EmbedRef->Cut();
+					moz->EmbedRef->Cut(wdata?wdata->ClipboardInfo.input_group:1);
 					}
 					break;
 				case Pt_MOZ_COMMAND_COPY: {
-					moz->EmbedRef->Copy();
+					moz->EmbedRef->Copy(wdata?wdata->ClipboardInfo.input_group:1);
 					}
 					break;
 				case Pt_MOZ_COMMAND_PASTE: {
-					moz->EmbedRef->Paste();
+					moz->EmbedRef->Paste(wdata?wdata->ClipboardInfo.input_group:1);
 					}
 					break;
 				case Pt_MOZ_COMMAND_SELECTALL: {
@@ -364,9 +381,8 @@ mozilla_modify( PtWidget_t *widget, PtArg_t const *argt, PtResourceRec_t const *
 					break;
 
 				case Pt_MOZ_COMMAND_FIND: {
-					PtWebClient2Command_t *wdata = ( PtWebClient2Command_t * ) argt->len;
 					nsCOMPtr<nsIWebBrowserFind> finder( do_GetInterface( moz->EmbedRef->mWindow->mWebBrowser ) );
-					finder->SetSearchString( NS_ConvertASCIItoUCS2(wdata->FindInfo.string).get() );
+					finder->SetSearchString( NS_ConvertASCIItoUTF16(wdata->FindInfo.string).get() );
 					finder->SetMatchCase( wdata->FindInfo.flags & Pt_WEB_FIND_MATCH_CASE );
 					finder->SetFindBackwards( wdata->FindInfo.flags & Pt_WEB_FIND_GO_BACKWARDS );
 					finder->SetWrapFind( wdata->FindInfo.flags & Pt_WEB_FIND_START_AT_TOP );
@@ -378,7 +394,6 @@ mozilla_modify( PtWidget_t *widget, PtArg_t const *argt, PtResourceRec_t const *
 					}
 
 				case Pt_MOZ_COMMAND_SAVEAS: {
-					PtWebClient2Command_t *wdata = ( PtWebClient2Command_t * ) argt->len;
 					char *dirname = ( char * ) calloc( 1, strlen( wdata->SaveasInfo.filename + 7 ) );
 					if( dirname ) {
 						sprintf( dirname, "%s_files", wdata->SaveasInfo.filename );
@@ -387,6 +402,7 @@ mozilla_modify( PtWidget_t *widget, PtArg_t const *argt, PtResourceRec_t const *
 						}
 					break;
 					}
+				}
 			}
 			break;
 
@@ -402,7 +418,7 @@ mozilla_modify( PtWidget_t *widget, PtArg_t const *argt, PtResourceRec_t const *
 			if( unknown->response == Pt_WEB_RESPONSE_CANCEL ) {
 				EmbedDownload *d = FindDownload( moz, unknown->download_ticket );
 				if( d ) {
-					if( d->mLauncher ) d->mLauncher->Cancel(); /* this will also call the EmbedDownload destructor */
+					if( d->mLauncher ) d->mLauncher->Cancel(NS_BINDING_ABORTED); /* this will also call the EmbedDownload destructor */
 					else if( d->mPersist ) d->mPersist->CancelSave(); /* this will also call the EmbedDownload destructor */
 					else delete d; /* just in case neither d->mLauncher or d->mPersist was set */
 					}
@@ -514,7 +530,7 @@ mozilla_get_info( PtWidget_t *widget, PtArg_t const *argt, PtResourceRec_t const
 			PRUnichar *charset = nsnull;
 			static char encoding[256];
 			pref->GetLocalizedUnicharPref( "intl.charset.default", &charset );
-			strcpy( encoding, NS_ConvertUCS2toUTF8(charset).get() );
+			strcpy( encoding, NS_ConvertUTF16toUTF8(charset).get() );
 			*(char**)argt->value = encoding;
 			}
 			break;
@@ -565,22 +581,22 @@ mozilla_set_pref( PtWidget_t *widget, char *option, char *value )
 
 /* HTML Options */
 	if( !strcmp( option, "A:visited color" ) ) {
-		pref->SetUnicharPref( "browser.visited_color", NS_ConvertASCIItoUCS2(value).get() );
+		pref->SetUnicharPref( "browser.visited_color", NS_ConvertASCIItoUTF16(value).get() );
 		}
 	else if( !strcmp( option, "A:link color" ) ) {
-		pref->SetUnicharPref( "browser.anchor_color", NS_ConvertASCIItoUCS2(value).get() );
+		pref->SetUnicharPref( "browser.anchor_color", NS_ConvertASCIItoUTF16(value).get() );
 		}
 
 /* the mozserver already has A:link color == browser.anchor_color for this */
 //	else if( !strcmp( option, "A:active color" ) ) {
-//		pref->SetUnicharPref( "browser.anchor_color", NS_ConvertASCIItoUCS2(value).get() );
+//		pref->SetUnicharPref( "browser.anchor_color", NS_ConvertASCIItoUTF16(value).get() );
 //		}
 
 	else if( !strcmp( option, "BODY color" ) ) {
-		pref->SetUnicharPref( "browser.display.foreground_color", NS_ConvertASCIItoUCS2(value).get() );
+		pref->SetUnicharPref( "browser.display.foreground_color", NS_ConvertASCIItoUTF16(value).get() );
 		}
 	else if( !strcmp( option, "BODY background" ) ) {
-		pref->SetUnicharPref( "browser.display.background_color", NS_ConvertASCIItoUCS2(value).get() );
+		pref->SetUnicharPref( "browser.display.background_color", NS_ConvertASCIItoUTF16(value).get() );
 		}
 	else if( !strcmp( option, "bIgnoreDocumentAttributes" ) )
 		pref->SetBoolPref( "browser.display.use_document_colors", stricmp( value, "TRUE" ) ? PR_FALSE : PR_TRUE );
@@ -603,7 +619,7 @@ mozilla_set_pref( PtWidget_t *widget, char *option, char *value )
 		char *font_default = NULL;
 		char preference[256];
 
-		pref->CopyCharPref( "font.default", &font_default );
+		pref->CopyCharPref( "font.default.x-western", &font_default );
 		if( !font_default ) font_default = "serif";
 
 		sprintf( preference, "font.name.%s.x-western", font_default );
@@ -660,8 +676,6 @@ mozilla_set_pref( PtWidget_t *widget, char *option, char *value )
 		pref->SetIntPref( "network.proxy.gopher_port", atoi(value) );
 
 /* TCP/IP options */
-	else if( !strcmp( option, "socket_timeout" ) )
-		pref->SetIntPref( "network.http.connect.timeout", atoi( value ) );
 	else if( !strcmp( option, "max_connections" ) )
 		pref->SetIntPref( "network.http.max-connections", atoi( value ) );
 
@@ -848,7 +862,7 @@ static void mozilla_get_pref( PtWidget_t *widget, char *option, char *value ) {
 		char *font_default = NULL, *font;
 		char preference[256];
 
-		pref->CopyCharPref( "font.default", &font_default );
+		pref->CopyCharPref( "font.default.x-western", &font_default );
 		if( !font_default ) font_default = "serif";
 
 		sprintf( preference, "font.name.%s.x-western", font_default );
@@ -915,11 +929,6 @@ static void mozilla_get_pref( PtWidget_t *widget, char *option, char *value ) {
 		}
 
 /* TCP/IP options */
-  else if( !strcmp( option, "socket_timeout" ) ) {
-		int n;
-		pref->GetIntPref( "network.http.connect.timeout", &n );
-		sprintf( value, "%d", n );
-		}
   else if( !strcmp( option, "max_connections" ) ) {
 		int n;
 		pref->GetIntPref( "network.http.max-connections", &n );
