@@ -67,7 +67,6 @@
 #include <nsProfileDirServiceProvider.h>
 #include <nsIGenericFactory.h>
 #include <nsIComponentRegistrar.h>
-#include <nsIPref.h>
 #include <nsVoidArray.h>
 #include <nsIDOMDocument.h>
 #include <nsIDOMBarProp.h>
@@ -209,7 +208,7 @@ QGeckoEmbedPrivate::init()
     nsCOMPtr<nsIWeakReference> weakRef;
     supportsWeak->GetWeakReference(getter_AddRefs(weakRef));
     webBrowser->AddWebBrowserListener(weakRef,
-                                      nsIWebProgressListener::GetIID());
+                                      NS_GET_IID(nsIWebProgressListener));
 
     // set ourselves as the parent uri content listener
     webBrowser->SetParentURIContentListener(contentListener);
@@ -417,9 +416,7 @@ QGeckoEmbed::reload(ReloadFlags flags)
 bool
 QGeckoEmbed::domKeyDownEvent(nsIDOMKeyEvent *keyEvent)
 {
-    qDebug("key event start");
     emit domKeyDown(keyEvent);
-    qDebug("key event stop");
     return false;
 }
 
@@ -434,7 +431,6 @@ bool
 QGeckoEmbed::domKeyUpEvent(nsIDOMKeyEvent *keyEvent)
 {
     emit domKeyUp(keyEvent);
-    qDebug("key release event stop");
     return false;
 }
 
