@@ -55,7 +55,6 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIWebBrowserChrome.h"
-#include "nsIWebShell.h"
 #include "nsPIDOMWindow.h"
 #include "nsIFocusController.h"
 #include "nsIDOMWindowInternal.h"
@@ -72,8 +71,8 @@
 #include "nsIURI.h"
 #include "nsIWebBrowserPersist.h"
 #include "nsCWebBrowserPersist.h"
-#include "nsIWebBrowserPrint.h"
 #include "nsIServiceManager.h"
+#include "nsAutoPtr.h"
 
 // for painting the background window
 #include "nsIRenderingContext.h"
@@ -81,7 +80,10 @@
 #include "nsILookAndFeel.h"
 
 // Printing Includes
+#ifdef NS_PRINTING
+#include "nsIWebBrowserPrint.h"
 #include "nsIContentViewer.h"
+#endif
 
 // PSM2 includes
 #include "nsISecureBrowserUI.h"
@@ -92,7 +94,6 @@
 #include "nsIEventSink.h"
 #endif
 
-static NS_DEFINE_CID(kWebShellCID, NS_WEB_SHELL_CID);
 static NS_DEFINE_IID(kWindowCID, NS_WINDOW_CID);
 static NS_DEFINE_CID(kChildCID, NS_CHILD_CID);
 static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
@@ -106,6 +107,7 @@ nsWebBrowser::nsWebBrowser() : mDocShellTreeOwner(nsnull),
    mInitInfo(nsnull),
    mContentType(typeContentWrapper),
    mActivating(PR_FALSE),
+   mShouldEnableHistory(PR_TRUE),
    mParentNativeWindow(nsnull),
    mProgressListener(nsnull),
    mBackgroundColor(0),
@@ -114,7 +116,6 @@ nsWebBrowser::nsWebBrowser() : mDocShellTreeOwner(nsnull),
    mPersistFlags(nsIWebBrowserPersist::PERSIST_FLAGS_NONE),
    mStream(nsnull),
    mParentWidget(nsnull),
-   mParent(nsnull),
    mListenerArray(nsnull)
 #if (defined(XP_MAC) || defined(XP_MACOSX)) && !defined(MOZ_WIDGET_COCOA)
    , mTopLevelWidget(nsnull)
@@ -187,6 +188,7 @@ NS_INTERFACE_MAP_BEGIN(nsWebBrowser)
     NS_INTERFACE_MAP_ENTRY(nsIInterfaceRequestor)
     NS_INTERFACE_MAP_ENTRY(nsIWebBrowserSetup)
     NS_INTERFACE_MAP_ENTRY(nsIWebBrowserPersist)
+    NS_INTERFACE_MAP_ENTRY(nsICancelable)
     NS_INTERFACE_MAP_ENTRY(nsIWebBrowserFocus)
     NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener)
     NS_INTERFACE_MAP_ENTRY(nsIWebBrowserStream)
@@ -210,20 +212,22 @@ NS_IMETHODIMP nsWebBrowser::GetInterface(const nsIID& aIID, void** aSink)
 #endif
 
    if (mDocShell) {
+#ifdef NS_PRINTING
        if (aIID.Equals(NS_GET_IID(nsIWebBrowserPrint))) {
            nsCOMPtr<nsIContentViewer> viewer;
            mDocShell->GetContentViewer(getter_AddRefs(viewer));
-           if (viewer) {
-               nsCOMPtr<nsIWebBrowserPrint> webBrowserPrint(do_QueryInterface(viewer));
-               nsIWebBrowserPrint* print = (nsIWebBrowserPrint*)webBrowserPrint.get();
-               NS_ASSERTION(print, "This MUST support this interface!");
-               NS_ADDREF(print);
-               *aSink = print;
-               return NS_OK;
-           }
-       } else {
-           return mDocShellAsReq->GetInterface(aIID, aSink);
+           if (!viewer)
+               return NS_NOINTERFACE;
+
+           nsCOMPtr<nsIWebBrowserPrint> webBrowserPrint(do_QueryInterface(viewer));
+           nsIWebBrowserPrint* print = (nsIWebBrowserPrint*)webBrowserPrint.get();
+           NS_ASSERTION(print, "This MUST support this interface!");
+           NS_ADDREF(print);
+           *aSink = print;
+           return NS_OK;
        }
+#endif
+       return mDocShellAsReq->GetInterface(aIID, aSink);
    }
 
    return NS_NOINTERFACE;
@@ -237,13 +241,13 @@ NS_IMETHODIMP nsWebBrowser::GetInterface(const nsIID& aIID, void** aSink)
 //  - nsIWebProgressListener
 NS_IMETHODIMP nsWebBrowser::AddWebBrowserListener(nsIWeakReference *aListener, const nsIID& aIID)
 {           
-    nsresult rv = NS_ERROR_INVALID_ARG;
     NS_ENSURE_ARG_POINTER(aListener);
 
+    nsresult rv = NS_OK;
     if (!mWebProgress) {
         // The window hasn't been created yet, so queue up the listener. They'll be
         // registered when the window gets created.
-        nsWebBrowserListenerState *state = nsnull;
+        nsAutoPtr<nsWebBrowserListenerState> state;
         NS_NEWXPCOM(state, nsWebBrowserListenerState);
         if (!state) return NS_ERROR_OUT_OF_MEMORY;
 
@@ -252,10 +256,17 @@ NS_IMETHODIMP nsWebBrowser::AddWebBrowserListener(nsIWeakReference *aListener, c
 
         if (!mListenerArray) {
             NS_NEWXPCOM(mListenerArray, nsVoidArray);
-            if (!mListenerArray) return NS_ERROR_OUT_OF_MEMORY;
+            if (!mListenerArray) {
+                return NS_ERROR_OUT_OF_MEMORY;
+            }
         }
 
-        if (!mListenerArray->AppendElement(state)) return NS_ERROR_OUT_OF_MEMORY;
+        if (!mListenerArray->AppendElement(state)) {
+            return NS_ERROR_OUT_OF_MEMORY;
+        }
+
+        // We're all set now; don't delete |state| after this point
+        state.forget();
     } else {
         nsCOMPtr<nsISupports> supports(do_QueryReferent(aListener));
         if (!supports) return NS_ERROR_INVALID_ARG;
@@ -288,9 +299,9 @@ NS_IMETHODIMP nsWebBrowser::BindListener(nsISupports *aListener, const nsIID& aI
 
 NS_IMETHODIMP nsWebBrowser::RemoveWebBrowserListener(nsIWeakReference *aListener, const nsIID& aIID)
 {
-    nsresult rv = NS_ERROR_INVALID_ARG;
     NS_ENSURE_ARG_POINTER(aListener);
 
+    nsresult rv = NS_OK;
     if (!mWebProgress) {
         // if there's no-one to register the listener w/, and we don't have a queue going,
         // the the called is calling Remove before an Add which doesn't make sense.
@@ -403,13 +414,13 @@ NS_IMETHODIMP nsWebBrowser::GetContainerWindow(nsIWebBrowserChrome** aTopWindow)
 {
    NS_ENSURE_ARG_POINTER(aTopWindow);
 
-   if(mDocShellTreeOwner)
-      *aTopWindow = mDocShellTreeOwner->mWebBrowserChrome;
-   else
+   if(mDocShellTreeOwner) {
+      *aTopWindow = mDocShellTreeOwner->GetWebBrowserChrome().get();
+   } else {
       *aTopWindow = nsnull;
-   NS_IF_ADDREF(*aTopWindow);
+   }
 
-   return NS_OK;   
+   return NS_OK;
 }
 
 NS_IMETHODIMP nsWebBrowser::SetContainerWindow(nsIWebBrowserChrome* aTopWindow)
@@ -521,41 +532,14 @@ NS_IMETHODIMP nsWebBrowser::SetItemType(PRInt32 aItemType)
 
 NS_IMETHODIMP nsWebBrowser::GetParent(nsIDocShellTreeItem** aParent)
 {
-   NS_ENSURE_ARG_POINTER(aParent);
-
-   *aParent = mParent;
-   NS_IF_ADDREF(*aParent);
-
+   *aParent = nsnull;
    return NS_OK;
 }
 
-NS_IMETHODIMP nsWebBrowser::SetParent(nsIDocShellTreeItem* aParent)
-{
-  // null aParent is ok
-   
-  /* Note this doesn't do an addref on purpose.  This is because the parent
-   is an implied lifetime.  We don't want to create a cycle by refcounting
-   the parent.*/
-  mParent = aParent;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsWebBrowser::GetSameTypeParent(nsIDocShellTreeItem** aParent)
 {
-   NS_ENSURE_ARG_POINTER(aParent);
    *aParent = nsnull;
 
-   if(!mParent)
-      return NS_OK;
-      
-   PRInt32  parentType;
-   NS_ENSURE_SUCCESS(mParent->GetItemType(&parentType), NS_ERROR_FAILURE);
-
-   if(typeContentWrapper == parentType)
-      {
-      *aParent = mParent;
-      NS_ADDREF(*aParent);
-      }                   
    return NS_OK;
 }
 
@@ -593,13 +577,15 @@ NS_IMETHODIMP nsWebBrowser::GetSameTypeRootTreeItem(nsIDocShellTreeItem** aRootT
 }
 
 NS_IMETHODIMP nsWebBrowser::FindItemWithName(const PRUnichar *aName, 
-   nsISupports* aRequestor, nsIDocShellTreeItem **_retval)
+   nsISupports* aRequestor, nsIDocShellTreeItem* aOriginalRequestor,
+   nsIDocShellTreeItem **_retval)
 {
    NS_ENSURE_STATE(mDocShell);
    NS_ASSERTION(mDocShellTreeOwner, "This should always be set when in this situation");
 
    return mDocShellAsItem->FindItemWithName(aName, 
-      NS_STATIC_CAST(nsIDocShellTreeOwner*, mDocShellTreeOwner), _retval);
+      NS_STATIC_CAST(nsIDocShellTreeOwner*, mDocShellTreeOwner),
+      aOriginalRequestor, _retval);
 }
 
 NS_IMETHODIMP nsWebBrowser::GetTreeOwner(nsIDocShellTreeOwner** aTreeOwner)
@@ -804,6 +790,7 @@ NS_IMETHODIMP nsWebBrowser::SetProperty(PRUint32 aId, PRUint32 aValue)
            NS_ENSURE_STATE(mDocShell);
            NS_ENSURE_TRUE((aValue == PR_TRUE || aValue == PR_FALSE), NS_ERROR_INVALID_ARG);
            rv = EnableGlobalHistory(aValue);
+           mShouldEnableHistory = aValue;
         }
         break;
     case nsIWebBrowserSetup::SETUP_FOCUS_DOC_BEFORE_CONTENT:
@@ -913,7 +900,7 @@ NS_IMETHODIMP nsWebBrowser::SetPersistFlags(PRUint32 aPersistFlags)
     mPersistFlags = aPersistFlags;
     if (mPersist)
     {
-        rv = SetPersistFlags(mPersistFlags);
+        rv = mPersist->SetPersistFlags(mPersistFlags);
         mPersist->GetPersistFlags(&mPersistFlags);
     }
     return rv;
@@ -1106,6 +1093,18 @@ NS_IMETHODIMP nsWebBrowser::CancelSave()
     return NS_OK;
 }
 
+/* void cancel(nsresult aReason); */
+NS_IMETHODIMP nsWebBrowser::Cancel(nsresult aReason)
+{
+    if (mPersist)
+    {
+        return mPersist->Cancel(aReason);
+    }
+    return NS_OK;
+}
+
+
+
 
 //*****************************************************************************
 // nsWebBrowser::nsIBaseWindow
@@ -1156,7 +1155,7 @@ NS_IMETHODIMP nsWebBrowser::Create()
                               nsnull, nsnull, nsnull, &widgetInit);  
       }
 
-   nsCOMPtr<nsIDocShell> docShell(do_CreateInstance(kWebShellCID));
+   nsCOMPtr<nsIDocShell> docShell(do_CreateInstance("@mozilla.org/webshell;1"));
    NS_ENSURE_SUCCESS(SetDocShell(docShell), NS_ERROR_FAILURE);
 
    // get the system default window background colour
@@ -1218,9 +1217,9 @@ NS_IMETHODIMP nsWebBrowser::Create()
    NS_ENSURE_TRUE(mInitInfo->sessionHistory, NS_ERROR_FAILURE);
    mDocShellAsNav->SetSessionHistory(mInitInfo->sessionHistory);
    
-   // Hook up global history. Do not fail if we can't - just assert.
-   nsresult rv = EnableGlobalHistory(PR_TRUE);
-   NS_ASSERTION(NS_SUCCEEDED(rv), "EnableGlobalHistory() failed");
+   // Hook up global history. Do not fail if we can't - just warn.
+   nsresult rv = EnableGlobalHistory(mShouldEnableHistory);
+   NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "EnableGlobalHistory() failed");
 
    NS_ENSURE_SUCCESS(mDocShellAsWin->Create(), NS_ERROR_FAILURE);
 
@@ -1230,8 +1229,12 @@ NS_IMETHODIMP nsWebBrowser::Create()
    rv = GetContentDOMWindow(getter_AddRefs(domWindow));
    if (NS_SUCCEEDED(rv))
    {
-       mSecurityUI = do_CreateInstance(NS_SECURE_BROWSER_UI_CONTRACTID, &rv);
-       if (NS_SUCCEEDED(rv))mSecurityUI->Init(domWindow);
+       // this works because the implementation of nsISecureBrowserUI
+       // (nsSecureBrowserUIImpl) gets a docShell from the domWindow,
+       // and calls docShell->SetSecurityUI(this);
+       nsCOMPtr<nsISecureBrowserUI> securityUI =
+           do_CreateInstance(NS_SECURE_BROWSER_UI_CONTRACTID, &rv);
+       if (NS_SUCCEEDED(rv)) securityUI->Init(domWindow);
    }
 
    mDocShellTreeOwner->AddToWatcher(); // evil twin of Remove in SetDocShell(0)
@@ -1801,19 +1804,14 @@ NS_IMETHODIMP nsWebBrowser::Activate(void)
     if (mWWatch)
       mWWatch->SetActiveWindow(win);
 
-    /* Activate the window itself. Do this only if the PresShell has
-       been created, since DOMWindow->Activate asserts otherwise.
-       (This method can be called during window creation before
-       the PresShell exists. For ex, Windows apps responding to
-       WM_ACTIVATE). */
-    NS_ENSURE_STATE(mDocShell);
-    nsCOMPtr<nsIPresShell> presShell;
-    mDocShell->GetPresShell(getter_AddRefs(presShell));
-    if(presShell) {
-      nsCOMPtr<nsPIDOMWindow> privateDOMWindow = do_QueryInterface(win);
-      if(privateDOMWindow)
-        privateDOMWindow->Activate();
-    }
+    /* Activate the window itself. Note that this method can be called during
+       window creation before the PresShell exists (for ex, Windows apps
+       responding to WM_ACTIVATE), which case nsGlobalWindow::Activate()
+       will return early.
+    */
+    nsCOMPtr<nsPIDOMWindow> privateDOMWindow = do_QueryInterface(win);
+    if (privateDOMWindow)
+      privateDOMWindow->Activate();
   }
 
   mActivating = PR_FALSE;
@@ -1828,12 +1826,6 @@ NS_IMETHODIMP nsWebBrowser::Deactivate(void)
      This seems harmless and maybe safer, but we have no real evidence
      either way just yet. */
 
-  NS_ENSURE_STATE(mDocShell);
-  nsCOMPtr<nsIPresShell> presShell;
-  mDocShell->GetPresShell(getter_AddRefs(presShell));
-  if(!presShell)
-    return NS_OK;
-
   nsCOMPtr<nsIDOMWindow> domWindow;
   GetContentDOMWindow(getter_AddRefs(domWindow));
   if (domWindow) {
@@ -1924,8 +1916,8 @@ NS_IMETHODIMP nsWebBrowser::SetFocusedElement(nsIDOMElement * aFocusedElement)
 // nsWebBrowser::nsIWebBrowserStream
 //*****************************************************************************   
 
-/* void openStream (in string aBaseURI, in string aContentType); */
-NS_IMETHODIMP nsWebBrowser::OpenStream(const char *aBaseURI, const char *aContentType)
+/* void openStream(in nsIURI aBaseURI, in ACString aContentType); */
+NS_IMETHODIMP nsWebBrowser::OpenStream(nsIURI *aBaseURI, const nsACString& aContentType)
 {
   nsresult rv;
 
@@ -1941,8 +1933,9 @@ NS_IMETHODIMP nsWebBrowser::OpenStream(const char *aBaseURI, const char *aConten
   return mStream->OpenStream(aBaseURI, aContentType);
 }
 
-/* void appendStream (in string aData, in long aLen); */
-NS_IMETHODIMP nsWebBrowser::AppendToStream(const char *aData, PRInt32 aLen)
+/* void appendToStream([const, array, size_is(aLen)] in octet aData,
+ * in unsigned long aLen); */
+NS_IMETHODIMP nsWebBrowser::AppendToStream(const PRUint8 *aData, PRUint32 aLen)
 {
   if (!mStream)
     return NS_ERROR_FAILURE;
