@@ -31,6 +31,8 @@
 #ifdef MOZ_REQUIRE_CURRENT_SDK
 #undef WINVER
 #define WINVER 0x0500
+#undef _WIN32_WINNT
+#define _WIN32_WINNT 0x0500
 #endif
 
 /* -------------------------------------------------------------------
@@ -67,7 +69,10 @@ WIN_LIBS=                                       \
 #include "nsIPrintSettingsWin.h"
 #include "nsUnitConversion.h"
 #include "nsIPrintOptions.h"
-#include "nsGfxCIID.h"
+#include "nsWidgetsCID.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+
 static NS_DEFINE_IID(kPrinterEnumeratorCID, NS_PRINTER_ENUMERATOR_CID);
 
 #include "nsRect.h"
@@ -84,8 +89,6 @@ static NS_DEFINE_IID(kPrinterEnumeratorCID, NS_PRINTER_ENUMERATOR_CID);
 // This is for extending the dialog
 #include <dlgs.h>
 
-static NS_DEFINE_CID(kStringBundleServiceCID,  NS_STRINGBUNDLESERVICE_CID);
-
 // For PrintDlgEx
 // needed because there are unicode/ansi versions of this routine
 // and we need to make sure we get the correct one.
@@ -353,7 +356,7 @@ GetLocalizedBundle(const char * aPropFileName, nsIStringBundle** aStrBundle)
 
   // Create bundle
   nsCOMPtr<nsIStringBundleService> stringService = 
-    do_GetService(kStringBundleServiceCID, &rv);
+    do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv) && stringService) {
     rv = stringService->CreateBundle(aPropFileName, aStrBundle);
   }
@@ -828,6 +831,23 @@ static PRUnichar * GetDefaultPrinterNameFromGlobalPrinters()
   return printerName;
 }
 
+// Determine whether we have a completely native dialog
+// or whether we cshould extend it
+static PRBool ShouldExtendPrintDialog()
+{
+  nsresult rv;
+  nsCOMPtr<nsIPrefService> prefs =
+    do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, PR_TRUE);
+  nsCOMPtr<nsIPrefBranch> prefBranch;
+  rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+  NS_ENSURE_SUCCESS(rv, PR_TRUE);
+
+  PRBool result;
+  rv = prefBranch->GetBoolPref("print.extend_native_print_dialog", &result);
+  NS_ENSURE_SUCCESS(rv, PR_TRUE);
+  return result;
+}
 
 //------------------------------------------------------------------
 // Displays the native Print Dialog
@@ -865,7 +885,7 @@ ShowNativePrintDialog(HWND              aHWnd,
   pDevNames->wOutputOffset = sizeof(DEVNAMES)+len+1;
   pDevNames->wDefault      = 0;
   char* device = &(((char*)pDevNames)[pDevNames->wDeviceOffset]);
-  strcpy(device, NS_ConvertUCS2toUTF8(printerName).get());
+  strcpy(device, NS_ConvertUTF16toUTF8(printerName).get());
   ::GlobalUnlock(hDevNames);
 
   // Create a Moveable Memory Object that holds a new DevMode
@@ -877,7 +897,7 @@ ShowNativePrintDialog(HWND              aHWnd,
 #ifdef UNICODE
   hGlobalDevMode = CreateGlobalDevModeAndInit(printerName, aPrintSettings);
 #else 
-  hGlobalDevMode = CreateGlobalDevModeAndInit(NS_CONST_CAST(char*, NS_ConvertUCS2toUTF8(printerName).get()), aPrintSettings);
+  hGlobalDevMode = CreateGlobalDevModeAndInit(NS_CONST_CAST(char*, NS_ConvertUTF16toUTF8(printerName).get()), aPrintSettings);
 #endif
 
   // Prepare to Display the Print Dialog
@@ -900,19 +920,6 @@ ShowNativePrintDialog(HWND              aHWnd,
   }
   aPrintSettings->GetHowToEnableFrameUI(&howToEnableFrameUI);
 
-  // Determine whether we have a completely native dialog
-  // or whether we cshould extend it
-  // true  - do only the native
-  // false - extend the dialog
-  PRPackedBool doExtend = PR_FALSE;
-  nsCOMPtr<nsIStringBundle> strBundle;
-  if (NS_SUCCEEDED(GetLocalizedBundle(PRINTDLG_PROPERTIES, getter_AddRefs(strBundle)))) {
-    nsAutoString doExtendStr;
-    if (NS_SUCCEEDED(GetLocalizedString(strBundle, "extend", doExtendStr))) {
-      doExtend = doExtendStr.EqualsLiteral("true");
-    }
-  }
-
   prntdlg.nFromPage           = 0xFFFF;
   prntdlg.nToPage             = 0xFFFF;
   prntdlg.nMinPage            = 1;
@@ -926,7 +933,7 @@ ShowNativePrintDialog(HWND              aHWnd,
   prntdlg.hInstance           = NULL;
   prntdlg.lpPrintTemplateName = NULL;
 
-  if (!doExtend) {
+  if (!ShouldExtendPrintDialog()) {
     prntdlg.lCustData         = NULL;
     prntdlg.lpfnPrintHook     = NULL;
   } else {
@@ -1184,7 +1191,7 @@ ShowNativePrintDialogEx(HWND              aHWnd,
 #ifdef UNICODE
     hGlobalDevMode = CreateGlobalDevModeAndInit(printerName, aPrintSettings);
 #else 
-    hGlobalDevMode = CreateGlobalDevModeAndInit(NS_CONST_CAST(char*, NS_ConvertUCS2toUTF8(printerName).get()), aPrintSettings);
+    hGlobalDevMode = CreateGlobalDevModeAndInit(NS_CONST_CAST(char*, NS_ConvertUTF16toUTF8(printerName).get()), aPrintSettings);
 #endif
   }
 
@@ -1210,19 +1217,6 @@ ShowNativePrintDialogEx(HWND              aHWnd,
     aPrintSettings->GetHowToEnableFrameUI(&howToEnableFrameUI);
   }
 
-  // Determine whether we have a completely native dialog
-  // or whether we cshould extend it
-  // true  - do only the native
-  // false - extend the dialog
-  PRPackedBool doExtend = PR_FALSE;
-  nsCOMPtr<nsIStringBundle> strBundle;
-  if (NS_SUCCEEDED(GetLocalizedBundle(PRINTDLG_PROPERTIES, getter_AddRefs(strBundle)))) {
-    nsAutoString doExtendStr;
-    if (NS_SUCCEEDED(GetLocalizedString(strBundle, "extend", doExtendStr))) {
-      doExtend = doExtendStr.LowerCaseEqualsLiteral("true");
-    }
-  }
-
   // At the moment we can only support one page range
   // from all the documentation I can find, it appears that this 
   // will get cleanup automatically when the struct goes away
@@ -1238,7 +1232,7 @@ ShowNativePrintDialogEx(HWND              aHWnd,
   prntdlg.nMaxPage       = 0xFFFF;
   prntdlg.nCopies        = 1;
 
-  if (doExtend) {
+  if (ShouldExtendPrintDialog()) {
     // lLcalize the Property Sheet (Tab) title
     char* pTitle = NULL;
     nsString optionsStr;
