@@ -42,6 +42,7 @@
 #include "nsNetUtil.h"
 #include "nsILineInputStream.h"
 #include "nsAppDirectoryServiceDefs.h"
+#include "nsVoidArray.h"
 #include "prprf.h"
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -207,17 +208,17 @@ nsresult nsPermissionManager::Init()
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
-  // Cache the permissions file
-  rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mPermissionsFile));
-  if (NS_SUCCEEDED(rv)) {
-    rv = mPermissionsFile->AppendNative(NS_LITERAL_CSTRING(kPermissionsFileName));
-  }
-
   // Clear the array of type strings
   memset(mTypeArray, nsnull, sizeof(mTypeArray));
 
-  // Ignore an error. That is not a problem. No cookperm.txt usually.
-  Read();
+  // Cache the permissions file
+  rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mPermissionsFile));
+  if (NS_SUCCEEDED(rv)) {
+    mPermissionsFile->AppendNative(NS_LITERAL_CSTRING(kPermissionsFileName));
+
+    // Ignore an error. That is not a problem. No cookperm.txt usually.
+    Read();
+  }
 
   mObserverService = do_GetService("@mozilla.org/observer-service;1", &rv);
   if (NS_SUCCEEDED(rv)) {
@@ -233,8 +234,9 @@ nsPermissionManager::Add(nsIURI     *aURI,
                          const char *aType,
                          PRUint32    aPermission)
 {
-  NS_ASSERTION(aURI, "could not get uri");
+  NS_ENSURE_ARG_POINTER(aURI);
   NS_ENSURE_ARG_POINTER(aType);
+
   nsresult rv;
 
   nsCAutoString host;
@@ -251,6 +253,7 @@ nsPermissionManager::Add(nsIURI     *aURI,
 
   mChangedList = PR_TRUE;
   LazyWrite();
+
   return NS_OK;
 }
 
@@ -285,6 +288,13 @@ nsPermissionManager::AddInternal(const nsAFlatCString &aHost,
   PRUint32 oldPermission = entry->GetPermission(aTypeIndex);
   entry->SetPermission(aTypeIndex, aPermission);
 
+  // If no more types are present, remove the entry
+  // Can happen if this add() is resetting the permission to default.
+  if (entry->PermissionsAreEmpty()) {
+    mHostTable.RawRemoveEntry(entry);
+    --mHostCount;
+  }
+
   // check whether we are deleting, adding, or changing a permission,
   // so we can notify observers. this would be neater to do in Add(),
   // but we need to do it here because we only know what type of notification
@@ -366,8 +376,7 @@ nsPermissionManager::TestPermission(nsIURI     *aURI,
                                     const char *aType,
                                     PRUint32   *aPermission)
 {
-  NS_ASSERTION(aURI, "could not get uri");
-  NS_ASSERTION(aPermission, "no permission pointer");
+  NS_ENSURE_ARG_POINTER(aURI);
   NS_ENSURE_ARG_POINTER(aType);
 
   // set the default
@@ -493,9 +502,9 @@ NS_IMETHODIMP nsPermissionManager::Observe(nsISupports *aSubject, const char *aT
     // Re-get the permissions file
     rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mPermissionsFile));
     if (NS_SUCCEEDED(rv)) {
-      rv = mPermissionsFile->AppendNative(NS_LITERAL_CSTRING(kPermissionsFileName));
+      mPermissionsFile->AppendNative(NS_LITERAL_CSTRING(kPermissionsFileName));
+      Read();
     }
-    Read();
   }
 
   return rv;
@@ -620,7 +629,6 @@ nsPermissionManager::Read()
     NS_ENSURE_SUCCESS(rv, rv);
 
     rv = NS_NewLocalFileInputStream(getter_AddRefs(fileInputStream), oldPermissionsFile);
-    NS_ENSURE_SUCCESS(rv, rv);
 
     readingOldFile = PR_TRUE;
 
@@ -631,6 +639,11 @@ nsPermissionManager::Read()
      * with a=0, b=1 etc
      */
   }
+  // An error path is expected when cookperm.txt is not found either, or when
+  // creating the stream failed for another reason.
+  if (NS_FAILED(rv))
+    return rv;
+
 
   nsCOMPtr<nsILineInputStream> lineInputStream = do_QueryInterface(fileInputStream, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -687,7 +700,7 @@ nsPermissionManager::Read()
         }
 
         PRUint32 type;
-        if (!PR_sscanf(buffer.get() + 1, "%u", &type) || type >= NUMBER_OF_TYPES) {
+        if (PR_sscanf(buffer.get() + 1, "%u", &type) != 1 || type >= NUMBER_OF_TYPES) {
           continue;
         }
 
@@ -744,6 +757,10 @@ nsPermissionManager::Read()
           type = 10*type + (c-'0');
           c = permissionString.CharAt(++index);
         }
+
+        if (type >= NUMBER_OF_TYPES)
+          continue; // invalid type for this permission entry
+
         if (index >= permissionString.Length())
           continue; // bad format for this permission entry
 
@@ -954,8 +971,6 @@ nsPermissionManager::Write()
 nsresult
 nsPermissionManager::GetHost(nsIURI *aURI, nsACString &aResult)
 {
-  NS_ASSERTION(aURI, "could not get uri");
-
   aURI->GetHost(aResult);
 
   // If there is no host, use the scheme, and prepend "scheme:",
@@ -968,5 +983,6 @@ nsPermissionManager::GetHost(nsIURI *aURI, nsACString &aResult)
     }
     aResult = NS_LITERAL_CSTRING("scheme:") + aResult;
   }
+
   return NS_OK;
 }
