@@ -42,12 +42,14 @@ extern "C" {
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 }
 
+#include "nsServiceManagerUtils.h"
+#include "nsComponentManagerUtils.h"
 #include "nsIGenericFactory.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIPrefService.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "nsIObserver.h"
-#include "nsEventQueueUtils.h"
+#include "nsThreadUtils.h"
 #include "nsProxyRelease.h"
 #include "nsIAuthPrompt.h"
 #include "nsIStringBundle.h"
@@ -55,13 +57,14 @@ extern "C" {
 #include "nsIURL.h"
 #include "nsMimeTypes.h"
 #include "nsNetUtil.h"
+#include "nsINetUtil.h"
 #include "nsAutoPtr.h"
 #include "nsError.h"
-#include "nsEscape.h"
-#include "nsCRT.h"
 #include "netCore.h"
 #include "prlog.h"
 #include "prtime.h"
+#include "prprf.h"
+#include "plstr.h"
 
 #define MOZ_GNOMEVFS_SCHEME              "moz-gnomevfs"
 #define MOZ_GNOMEVFS_SUPPORTED_PROTOCOLS "network.gnomevfs.supported-protocols"
@@ -141,30 +144,6 @@ MapGnomeVFSResult(GnomeVFSResult result)
 
 //-----------------------------------------------------------------------------
 
-// Query the channel for a nsIAuthPrompt reference (use the canonical method of
-// searching the channel's notification callbacks first, and its loadgroup's
-// callbacks seconds).  XXX This should really live somewhere else.
-static void
-GetAuthPromptFromChannel(nsIChannel *channel, nsIAuthPrompt **result)
-{
-  nsCOMPtr<nsIInterfaceRequestor> cbs;
-  channel->GetNotificationCallbacks(getter_AddRefs(cbs));
-  if (cbs)
-    CallGetInterface(cbs.get(), result);
-
-  if (!*result)
-  {
-    nsCOMPtr<nsILoadGroup> loadGroup;
-    channel->GetLoadGroup(getter_AddRefs(loadGroup));
-    if (loadGroup)
-    {
-      loadGroup->GetNotificationCallbacks(getter_AddRefs(cbs));
-      if (cbs)
-        CallGetInterface(cbs.get(), result);
-    }
-  }
-}
-
 static void
 ProxiedAuthCallback(gconstpointer in,
                     gsize         in_size,
@@ -185,7 +164,7 @@ ProxiedAuthCallback(gconstpointer in,
     return;
 
   nsCOMPtr<nsIAuthPrompt> prompt;
-  GetAuthPromptFromChannel(channel, getter_AddRefs(prompt));
+  NS_QueryNotificationCallbacks(channel, prompt);
 
   // If no auth prompt, then give up.  We could failover to using the
   // WindowWatcher service, but that might defeat a consumer's purposeful
@@ -231,20 +210,25 @@ ProxiedAuthCallback(gconstpointer in,
     return;
 
   // Construct the single signon key.  Altering the value of this key will
-  // cause peoples remembered passwords to be forgotten.  Think carefully
+  // cause people's remembered passwords to be forgotten.  Think carefully
   // before changing the way this key is constructed.
-  nsAutoString key, dispHost, realm;
-  AppendUTF8toUTF16(scheme + NS_LITERAL_CSTRING("://") + hostPort, dispHost);
+  nsAutoString key, realm;
+
+  NS_ConvertUTF8toUTF16 dispHost(scheme);
+  dispHost.Append(NS_LITERAL_STRING("://"));
+  dispHost.Append(NS_ConvertUTF8toUTF16(hostPort));
+
   key = dispHost;
   if (authIn->realm)
   {
     // We assume the realm string is ASCII.  That might be a bogus assumption,
     // but we have no idea what encoding GnomeVFS is using, so for now we'll
     // limit ourselves to ISO-Latin-1.  XXX What is a better solution?
-    realm.Append(PRUnichar('"'));
-    realm.AppendWithConversion(authIn->realm);
-    realm.Append(PRUnichar('"'));
-    key += NS_LITERAL_STRING(" ") + realm;
+    realm.Append('"');
+    realm.Append(NS_ConvertASCIItoUTF16(authIn->realm));
+    realm.Append('"');
+    key.Append(' ');
+    key.Append(realm);
   }
 
   // Construct the message string...
@@ -263,7 +247,7 @@ ProxiedAuthCallback(gconstpointer in,
   if (!bundle)
     return;
 
-  nsXPIDLString message;
+  nsString message;
   if (!realm.IsEmpty())
   {
     const PRUnichar *strings[] = { realm.get(), dispHost.get() };
@@ -276,7 +260,7 @@ ProxiedAuthCallback(gconstpointer in,
     bundle->FormatStringFromName(NS_LITERAL_STRING("EnterUserPasswordFor").get(),
                                  strings, 1, getter_Copies(message));
   }
-  if (!message)
+  if (message.IsEmpty())
     return;
 
   // Prompt the user...
@@ -306,31 +290,20 @@ ProxiedAuthCallback(gconstpointer in,
   nsMemory::Free(pass);
 }
 
-struct nsGnomeVFSAuthParams
+struct nsGnomeVFSAuthCallbackEvent : public nsRunnable
 {
   gconstpointer in;
   gsize         in_size;
   gpointer      out;
   gsize         out_size;
   gpointer      callback_data;
+
+  NS_IMETHOD Run() {
+    ProxiedAuthCallback(in, in_size, out, out_size, callback_data);
+    return NS_OK;
+  }
 };
 
-PR_STATIC_CALLBACK(void *)
-AuthCallbackEventHandler(PLEvent *ev)
-{
-  nsGnomeVFSAuthParams *params = (nsGnomeVFSAuthParams *) ev->owner;
-  ProxiedAuthCallback(params->in, params->in_size,
-                      params->out, params->out_size,
-                      params->callback_data);
-  return nsnull;
-}
-
-PR_STATIC_CALLBACK(void)
-AuthCallbackEventDestructor(PLEvent *ev)
-{
-  // ignored
-}
-
 static void
 AuthCallback(gconstpointer in,
              gsize         in_size,
@@ -338,30 +311,20 @@ AuthCallback(gconstpointer in,
              gsize         out_size,
              gpointer      callback_data)
 {
-  // Need to proxy this callback over to the main thread.  This code is greatly
-  // simplified by the fact that we are making a synchronous callback.  E.g., we
-  // don't need to allocate the PLEvent on the heap.
+  // Need to proxy this callback over to the main thread.  Synchronous dispatch
+  // is required in order to provide data to the GnomeVFS callback.
 
-  nsCOMPtr<nsIEventQueue> eventQ;
-  NS_GetMainEventQ(getter_AddRefs(eventQ));
-  if (!eventQ)
-    return;
+  nsRefPtr<nsGnomeVFSAuthCallbackEvent> ev = new nsGnomeVFSAuthCallbackEvent();
+  if (!ev)
+    return;  // OOM
 
-  nsGnomeVFSAuthParams params;
-  params.in = in;
-  params.in_size = in_size;
-  params.out = out;
-  params.out_size = out_size;
-  params.callback_data = callback_data;
+  ev->in = in;
+  ev->in_size = in_size;
+  ev->out = out;
+  ev->out_size = out_size;
+  ev->callback_data = callback_data;
 
-  PLEvent ev;
-  PL_InitEvent(&ev, &params,
-      AuthCallbackEventHandler,
-      AuthCallbackEventDestructor);
-
-  void *result;
-  if (NS_FAILED(eventQ->PostSynchronousEvent(&ev, &result)))
-    PL_DestroyEvent(&ev);
+  NS_DispatchToMainThread(ev, NS_DISPATCH_SYNC);
 }
 
 //-----------------------------------------------------------------------------
@@ -509,8 +472,9 @@ nsGnomeVFSInputStream::DoOpen()
       mDirListPtr = mDirList;
 
       // Write base URL (make sure it ends with a '/')
-      mDirBuf.Append(NS_LITERAL_CSTRING("300: ") + mSpec);
-      if (mSpec.Last() != '/')
+      mDirBuf.Append("300: ");
+      mDirBuf.Append(mSpec);
+      if (mSpec.get()[mSpec.Length() - 1] != '/')
         mDirBuf.Append('/');
       mDirBuf.Append('\n');
 
@@ -582,12 +546,14 @@ nsGnomeVFSInputStream::DoRead(char *aBuf, PRUint32 aCount, PRUint32 *aCountRead)
         mDirBuf.Assign("201: ");
 
         // The "filename" field
-        char *escName = nsEscape(info->name, url_Path);
-        if (escName)
-        {
+        nsCString escName;
+        nsCOMPtr<nsINetUtil> nu = do_GetService(NS_NETUTIL_CONTRACTID);
+        if (nu) {
+          nu->EscapeString(nsDependentCString(info->name),
+                           nsINetUtil::ESCAPE_URL_PATH, escName);
+
           mDirBuf.Append(escName);
           mDirBuf.Append(' ');
-          nsMemory::Free(escName);
         }
 
         // The "content-length" field
@@ -613,13 +579,13 @@ nsGnomeVFSInputStream::DoRead(char *aBuf, PRUint32 aCount, PRUint32 *aCountRead)
         switch (info->type)
         {
           case GNOME_VFS_FILE_TYPE_REGULAR:
-            mDirBuf.AppendLiteral("FILE ");
+            mDirBuf.Append("FILE ");
             break;
           case GNOME_VFS_FILE_TYPE_DIRECTORY:
-            mDirBuf.AppendLiteral("DIRECTORY ");
+            mDirBuf.Append("DIRECTORY ");
             break;
           case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
-            mDirBuf.AppendLiteral("SYMBOLIC-LINK ");
+            mDirBuf.Append("SYMBOLIC-LINK ");
             break;
           default:
             break;
@@ -642,32 +608,25 @@ nsGnomeVFSInputStream::DoRead(char *aBuf, PRUint32 aCount, PRUint32 *aCountRead)
 }
 
 // This class is used to implement SetContentTypeOfChannel.
-class nsGnomeVFSSetContentTypeEvent : public PLEvent
+class nsGnomeVFSSetContentTypeEvent : public nsRunnable
 {
   public:
     nsGnomeVFSSetContentTypeEvent(nsIChannel *channel, const char *contentType)
-      : mContentType(contentType)
+      : mChannel(channel), mContentType(contentType)
     {
-      // stash channel reference in owner field.  no AddRef here!  see note
+      // stash channel reference in mChannel.  no AddRef here!  see note
       // in SetContentTypeOfchannel.
-      PL_InitEvent(this, channel, EventHandler, EventDestructor);
     }
 
-    PR_STATIC_CALLBACK(void *) EventHandler(PLEvent *ev)
+    NS_IMETHOD Run()
     {
-      nsGnomeVFSSetContentTypeEvent *self = (nsGnomeVFSSetContentTypeEvent *) ev;
-      ((nsIChannel *) self->owner)->SetContentType(self->mContentType);
-      return nsnull;
-    }
-
-    PR_STATIC_CALLBACK(void) EventDestructor(PLEvent *ev)
-    {
-      nsGnomeVFSSetContentTypeEvent *self = (nsGnomeVFSSetContentTypeEvent *) ev;
-      delete self;
+      mChannel->SetContentType(mContentType);
+      return NS_OK;
     }
 
   private: 
-    nsCString mContentType;
+    nsIChannel *mChannel;
+    nsCString   mContentType;
 };
 
 nsresult
@@ -680,12 +639,7 @@ nsGnomeVFSInputStream::SetContentTypeOfChannel(const char *contentType)
   // thread's event queue to protect us against memory corruption.
 
   nsresult rv;
-  nsCOMPtr<nsIEventQueue> eventQ;
-  rv = NS_GetMainEventQ(getter_AddRefs(eventQ));
-  if (NS_FAILED(rv))
-    return rv;
-
-  nsGnomeVFSSetContentTypeEvent *ev =
+  nsCOMPtr<nsIRunnable> ev =
       new nsGnomeVFSSetContentTypeEvent(mChannel, contentType);
   if (!ev)
   {
@@ -693,9 +647,7 @@ nsGnomeVFSInputStream::SetContentTypeOfChannel(const char *contentType)
   }
   else
   {
-    rv = eventQ->PostEvent(ev);
-    if (NS_FAILED(rv))
-      PL_DestroyEvent(ev);
+    rv = NS_DispatchToMainThread(ev);
   }
   return rv;
 }
@@ -722,14 +674,13 @@ nsGnomeVFSInputStream::Close()
 
   if (mChannel)
   {
-    nsresult rv;
+    nsresult rv = NS_OK;
 
-    nsCOMPtr<nsIEventQueue> eventQ;
-    rv = NS_GetMainEventQ(getter_AddRefs(eventQ));
-    if (NS_SUCCEEDED(rv))
-      rv = NS_ProxyRelease(eventQ, mChannel);
+    nsCOMPtr<nsIThread> thread = do_GetMainThread();
+    if (thread)
+      rv = NS_ProxyRelease(thread, mChannel);
 
-    NS_ASSERTION(NS_SUCCEEDED(rv), "leaking channel reference");
+    NS_ASSERTION(thread && NS_SUCCEEDED(rv), "leaking channel reference");
     mChannel = nsnull;
   }
 
@@ -759,6 +710,8 @@ nsGnomeVFSInputStream::Read(char *aBuf,
 {
   *aCountRead = 0;
 
+  if (mStatus == NS_BASE_STREAM_CLOSED)
+    return NS_OK;
   if (NS_FAILED(mStatus))
     return mStatus;
 
@@ -820,7 +773,7 @@ class nsGnomeVFSProtocolHandler : public nsIProtocolHandler
     void   InitSupportedProtocolsPref(nsIPrefBranch *prefs);
     PRBool IsSupportedProtocol(const nsCString &spec);
 
-    nsXPIDLCString mSupportedProtocols;
+    nsCString mSupportedProtocols;
 };
 
 NS_IMPL_ISUPPORTS2(nsGnomeVFSProtocolHandler, nsIProtocolHandler, nsIObserver)
@@ -841,7 +794,7 @@ nsGnomeVFSProtocolHandler::Init()
     }
   }
 
-  nsCOMPtr<nsIPrefBranchInternal> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
   if (prefs)
   {
     InitSupportedProtocolsPref(prefs);
@@ -857,37 +810,43 @@ nsGnomeVFSProtocolHandler::InitSupportedProtocolsPref(nsIPrefBranch *prefs)
   // read preferences
   nsresult rv = prefs->GetCharPref(MOZ_GNOMEVFS_SUPPORTED_PROTOCOLS,
                                    getter_Copies(mSupportedProtocols));
-  if (NS_SUCCEEDED(rv))
+  if (NS_SUCCEEDED(rv)) {
     mSupportedProtocols.StripWhitespace();
+    ToLowerCase(mSupportedProtocols);
+  }
   else
-    mSupportedProtocols.AssignLiteral("smb:,sftp:"); // use defaults
+    mSupportedProtocols.Assign("smb:,sftp:"); // use defaults
 
   LOG(("gnomevfs: supported protocols \"%s\"\n", mSupportedProtocols.get()));
 }
 
 PRBool
-nsGnomeVFSProtocolHandler::IsSupportedProtocol(const nsCString &spec)
+nsGnomeVFSProtocolHandler::IsSupportedProtocol(const nsCString &aSpec)
 {
-  PRInt32 colon = spec.FindChar(':');
-  if (colon == kNotFound)
+  const char *specString = aSpec.get();
+  const char *colon = strchr(specString, ':');
+  if (!colon)
     return PR_FALSE;
 
+  PRUint32 length = colon - specString + 1;
+
   // <scheme> + ':'
-  const nsCSubstring &scheme = StringHead(spec, colon + 1);
+  nsCString scheme(specString, length);
 
-  nsACString::const_iterator begin, end, iter;
-  mSupportedProtocols.BeginReading(begin);
-  mSupportedProtocols.EndReading(end);
+  char *found = PL_strcasestr(mSupportedProtocols.get(), scheme.get());
+  if (!found)
+    return PR_FALSE;
 
-  iter = begin;
-  return CaseInsensitiveFindInReadable(scheme, iter, end) &&
-      (iter == begin || *(--iter) == ',');
+  if (found[length] != ',' && found[length] != '\0')
+    return PR_FALSE;
+
+  return PR_TRUE;
 }
 
 NS_IMETHODIMP
 nsGnomeVFSProtocolHandler::GetScheme(nsACString &aScheme)
 {
-  aScheme.AssignLiteral(MOZ_GNOMEVFS_SCHEME);
+  aScheme.Assign(MOZ_GNOMEVFS_SCHEME);
   return NS_OK;
 }
 
@@ -912,8 +871,7 @@ nsGnomeVFSProtocolHandler::NewURI(const nsACString &aSpec,
                                   nsIURI *aBaseURI,
                                   nsIURI **aResult)
 {
-  const nsPromiseFlatCString &flatSpec = PromiseFlatCString(aSpec);
-
+  const nsCString flatSpec(aSpec);
   LOG(("gnomevfs: NewURI [spec=%s]\n", flatSpec.get()));
 
   if (!aBaseURI)
@@ -964,6 +922,7 @@ nsGnomeVFSProtocolHandler::NewURI(const nsACString &aSpec,
 NS_IMETHODIMP
 nsGnomeVFSProtocolHandler::NewChannel(nsIURI *aURI, nsIChannel **aResult)
 {
+  NS_ENSURE_ARG_POINTER(aURI);
   nsresult rv;
 
   nsCAutoString spec;
