@@ -41,18 +41,17 @@
 
 //-------- global variables
 var helpBrowser;
+var helpExternal;
 var helpSearchPanel;
 var emptySearch;
-var emptySearchText
-var emptySearchLink
+var emptySearchText = "No search items found.";
+var emptySearchLink = "about:blank";
 var helpTocPanel;
 var helpIndexPanel;
 var helpGlossaryPanel;
 
 // Namespaces
 const NC = "http://home.netscape.com/NC-rdf#";
-const SN = "rdf:http://www.w3.org/1999/02/22-rdf-syntax-ns#";
-const XML = "http://www.w3.org/XML/1998/namespace#"
 const MAX_LEVEL = 40; // maximum depth of recursion in search datasources.
 
 // Resources
@@ -60,6 +59,7 @@ const RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Comp
 const RDF_ROOT = RDF.GetResource("urn:root");
 const NC_PANELLIST = RDF.GetResource(NC + "panellist");
 const NC_PANELID = RDF.GetResource(NC + "panelid");
+const NC_PLATFORM = RDF.GetResource(NC + "platform");
 const NC_EMPTY_SEARCH_TEXT = RDF.GetResource(NC + "emptysearchtext");
 const NC_EMPTY_SEARCH_LINK = RDF.GetResource(NC + "emptysearchlink");
 const NC_DATASOURCES = RDF.GetResource(NC + "datasources");
@@ -88,6 +88,9 @@ const defaultHelpFile = "chrome://help/locale/mozillahelp.rdf";
 var defaultTopic = "welcome"; 
 var searchDatasources = "rdf:null";
 var searchDS = null;
+var platform = /mac/i.test(navigator.platform) ? "mac" :
+               /win/i.test(navigator.platform) ? "win" :
+               /os\/2/i.test(navigator.platform) ? "os/2" : "unix";
 
 const NSRESULT_RDF_SYNTAX_ERROR = 0x804e03f7; 
 
@@ -107,6 +110,31 @@ function displayTopic(topic) {
   loadURI(uri);
 }
 
+var helpContentListener = {
+  onStartURIOpen: function(aURI) {
+    return false;
+  },
+  doContent: function(aContentType, aIsContentPreferred, aRequest, aContentHandler) {
+    throw Components.results.NS_ERROR_UNEXPECTED;
+  },
+  isPreferred: function(aContentType, aDesiredContentType) {
+    return false;
+  },
+  canHandleContent: function(aContentType, aIsContentPreferred, aDesiredContentType) {
+    return false;
+  },
+  loadCookie: null,
+  parentContentListener: null,
+  QueryInterface: function (aIID) {
+    if (aIID.equals(Components.interfaces.nsIURIContentListener) ||
+        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
+        aIID.equals(Components.interfaces.nsISupports))
+      return this;
+    Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
+    return null;
+  }
+};
+
 // Initialize the Help window
 function init() {
   //cache panel references.
@@ -115,6 +143,13 @@ function init() {
   helpIndexPanel = document.getElementById("help-index-panel");
   helpGlossaryPanel = document.getElementById("help-glossary-panel");
   helpBrowser = document.getElementById("help-content");
+  helpExternal = document.getElementById("help-external");
+  helpExternal.docShell.useErrorPages = false;
+  helpExternal
+    .docShell
+    .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+    .getInterface(Components.interfaces.nsIURIContentListener)
+    .parentContentListener = helpContentListener;
 
   // Get the help content pack, base URL, and help topic
   var helpTopic = defaultTopic;
@@ -128,22 +163,47 @@ function init() {
 
   displayTopic(helpTopic);
 
-  // Initalize History.
-  var sessionHistory =  Components.classes["@mozilla.org/browser/shistory;1"]
-                                  .createInstance(Components.interfaces.nsISHistory);
-
   window.XULBrowserWindow = new nsHelpStatusHandler();
 
   //Start the status handler.
   window.XULBrowserWindow.init();
 
   // Hook up UI through Progress Listener
-  const interfaceRequestor = helpBrowser.docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
-  const webProgress = interfaceRequestor.getInterface(Components.interfaces.nsIWebProgress);
-  webProgress.addProgressListener(window.XULBrowserWindow, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
+  helpBrowser.addProgressListener(window.XULBrowserWindow, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
+  helpExternal.addProgressListener(window.XULBrowserWindow, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
 
   //Always show the Table of Contents sidebar at startup.
   showPanel('help-toc');
+
+  // alwaysRaised only works on Mac & Win.
+  if (!/Win|Mac/.test(navigator.platform)) {
+    var toggleSeparator = document.getElementById("context-sep-selectall");
+    var toggleOnTop = document.getElementById("context-zlevel");
+    toggleOnTop.hidden = true;
+    toggleSeparator.hidden = true;
+  }
+}
+
+function contentClick(event) {
+  // is this a left click on a link?
+  if (event.shiftKey || event.ctrlKey || event.altKey || event.metaKey || event.button != 0)
+    return true;
+
+  // is this a link?
+  var target = event.target;
+  while (!(target instanceof HTMLAnchorElement))
+    if (!(target = target.parentNode))
+      return true;
+
+  // is this an internal link?
+  if (target.href.lastIndexOf("chrome:", 0) == 0)
+    return true;
+
+  const loadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_IS_LINK;
+  try {
+    helpExternal.webNavigation.loadURI(target.href, loadFlags, null, null, null);
+  } catch (e) {}
+  return false;
 }
 
 function loadHelpRDF() {
@@ -165,24 +225,31 @@ function loadHelpRDF() {
       var panelDefs = helpFileDS.GetTarget(RDF_ROOT, NC_PANELLIST, true);      
       RDFContainer.Init(helpFileDS, panelDefs);
       var iterator = RDFContainer.GetElements();
-        while (iterator.hasMoreElements()) {
+      while (iterator.hasMoreElements()) {
         var panelDef = iterator.getNext();
-        var panelID = getAttribute(helpFileDS, panelDef, NC_PANELID, null);        
 
-        var datasources = getAttribute(helpFileDS, panelDef, NC_DATASOURCES, "rdf:none");
+        var panelPlatforms = getAttribute(helpFileDS, panelDef, NC_PLATFORM, platform);
+        panelPlatforms = panelPlatforms.split(/\s+/);
+
+        if (panelPlatforms.indexOf(platform) == -1)
+          continue; // ignore datasources for other platforms.
+
+        var panelID = getAttribute(helpFileDS, panelDef, NC_PANELID, null);
+
+        var datasources = getAttribute(helpFileDS, panelDef, NC_DATASOURCES, "rdf:null");
         datasources = normalizeLinks(helpBaseURI, datasources);
         // cache additional datsources to augment search datasources.
         if (panelID == "search") {
-	       emptySearchText = getAttribute(helpFileDS, panelDef, NC_EMPTY_SEARCH_TEXT, "No search items found.");
-	       emptySearchLink = getAttribute(helpFileDS, panelDef, NC_EMPTY_SEARCH_LINK, "about:blank");
-          searchDatasources = datasources;
-          datasources = "rdf:null"; // but don't try to display them yet!
+          emptySearchText = getAttribute(helpFileDS, panelDef, NC_EMPTY_SEARCH_TEXT, emptySearchText);
+          emptySearchLink = getAttribute(helpFileDS, panelDef, NC_EMPTY_SEARCH_LINK, emptySearchLink);
+          searchDatasources += " " + datasources;
+          continue; // but don't try to display them yet!
         }  
 
         // cache toc datasources for use by ID lookup.
         var tree = document.getElementById("help-" + panelID + "-panel");
         loadDatabasesBlocking(datasources);
-        tree.setAttribute("datasources", datasources);
+        tree.setAttribute("datasources", tree.getAttribute("datasources") + " " + datasources);
       }  
     }
     catch (e) {
@@ -373,7 +440,7 @@ nsHelpStatusHandler.prototype =
 
   setJSStatus : function(status) {},
   setJSDefaultStatus : function(status) {},
-  setOverLink : function(link) {}
+  setOverLink : function(link, context) {}
 }
 
 function UpdateBackForwardButtons()
@@ -460,14 +527,19 @@ function doFind() {
   // clear any previous results.
   clearDatabases(searchTree.database);
 
-  // split search string into separate terms and compile into regexp's
-  RE = findText.value.split(/\s+/);
-  for (var i=0; i < RE.length; ++i) {
-    if (RE[i] == "")
-      continue;
-    RE[i] = new RegExp(RE[i], "i");
+    // if the search string is empty or contains only whitespace, purge the results tree and return
+    RE = findText.value.match(/\S+/g);
+    if (!RE) {
+      searchTree.builder.rebuild();
+      return;
+    }
+
+    // compile the search string, which has already been split up above, into regexps
+    for (var i=0; i < RE.length; ++i) {
+      RE[i] = new RegExp(RE[i], "i");
   }
- emptySearch = true;
+  emptySearch = true;
+
   // search TOC
   var resultsDS =  Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"].createInstance(Components.interfaces.nsIRDFDataSource);
   var tree = document.getElementById("help-toc-panel");
@@ -502,12 +574,6 @@ function doFind() {
   searchTree.builder.rebuild();
 }
 
-function doEnabling() {
-  var findButton = document.getElementById("findButton");
-  var findTextbox = document.getElementById("findText");
-  findButton.disabled = !findTextbox.value;
-}
-
 function clearDatabases(compositeDataSource) {
   var enumDS = compositeDataSource.GetDataSources()
   while (enumDS.hasMoreElements()) {
@@ -609,7 +675,7 @@ function getAttribute(datasource, resource, attributeResourceName, defaultValue)
   var literal = datasource.GetTarget(resource, attributeResourceName, true);
   return literal instanceof Components.interfaces.nsIRDFLiteral ? literal.Value : defaultValue;
 }
-// Write debug string to javascript console.
+// Write debug string to error console.
 function log(aText) {
   CONSOLE_SERVICE.logStringMessage(aText);
 }
@@ -651,3 +717,29 @@ function showRelativePanel(goForward) {
     selectedIndex = sidebarButtons.length - 1;
   sidebarButtons[selectedIndex].doCommand();
 }
+
+// getXulWin - Returns the current Help window as a nsIXULWindow.
+function getXulWin() {
+  window.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
+  var webnav = window.getInterface(Components.interfaces.nsIWebNavigation);
+  var dsti = webnav.QueryInterface(Components.interfaces.nsIDocShellTreeItem);
+  var treeowner = dsti.treeOwner;
+  var ifreq = treeowner.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
+
+  return ifreq.getInterface(Components.interfaces.nsIXULWindow);
+}
+
+// toggleZLevel - Toggles whether or not the window will always appear on top.
+// element is the DOM node that persists the checked state
+function toggleZLevel(element) {
+  var xulwin = getXulWin();
+
+  // Now we can flip the zLevel, and set the attribute so that it persists correctly
+  if (xulwin.zLevel > xulwin.normalZ) {
+    xulwin.zLevel = xulwin.normalZ;
+    element.setAttribute("checked", "false");
+  } else {
+    xulwin.zLevel = xulwin.raisedZ;
+    element.setAttribute("checked", "true");
+  }
+}
