@@ -51,6 +51,14 @@ const NS_NET_STATUS_SENDING_TO = NS_ERROR_MODULE_NETWORK + 5;
 const NS_NET_STATUS_RECEIVING_FROM = NS_ERROR_MODULE_NETWORK + 6;
 const NS_NET_STATUS_CONNECTING_TO = NS_ERROR_MODULE_NETWORK + 7;
 
+// Security Constants.
+const STATE_IS_BROKEN = 1;
+const STATE_IS_SECURE = 2;
+const STATE_IS_INSECURE = 3;
+
+const STATE_SECURE_LOW = 1;
+const STATE_SECURE_HIGH = 2;
+
 const nsIScriptableInputStream = Components.interfaces.nsIScriptableInputStream;
 
 const nsIBinaryInputStream = Components.interfaces.nsIBinaryInputStream;
@@ -96,9 +104,17 @@ function toSOutputStream(stream, binary)
 
 function CBSConnection (binary)
 {
-    var sockServiceClass =
+    /* Since 2003-01-17 18:14, Mozilla has had this contract ID for the STS.
+     * Prior to that it didn't have one, so we also include the CID for the
+     * STS back then - DO NOT UPDATE THE ID if it changes in Mozilla.
+     */
+    const sockClassByName =
+        Components.classes["@mozilla.org/network/socket-transport-service;1"];
+    const sockClassByID =
         Components.classesByID["{c07e81e0-ef12-11d2-92b6-00105a1b0d64}"];
 
+    var sockServiceClass = (sockClassByName || sockClassByID);
+
     if (!sockServiceClass)
         throw ("Couldn't get socket service class.");
 
@@ -109,51 +125,111 @@ function CBSConnection (binary)
     this._sockService = sockService.QueryInterface
         (Components.interfaces.nsISocketTransportService);
 
-    this.wrappedJSObject = this;
-    this.binaryMode = binary || false;
+    /* Note: as part of the mess from bug 315288 and bug 316178, ChatZilla now
+     *       uses the *binary* stream interfaces for all network
+     *       communications.
+     *
+     *       However, these interfaces do not exist prior to 1999-11-05. To
+     *       make matters worse, an incompatible change to the "readBytes"
+     *       method of this interface was made on 2003-03-13; luckly, this
+     *       change also added a "readByteArray" method, which we will check
+     *       for below, to determin if we can use the binary streams.
+     */
 
-    //if (!ASSERT(!this.binaryMode || jsenv.HAS_WORKING_BINARY_STREAMS,
-    //            "Unable to use binary streams in this build."))
-    //    return null;
+    // We want to check for working binary streams only the first time.
+    if (CBSConnection.prototype.workingBinaryStreams == -1)
+    {
+        CBSConnection.prototype.workingBinaryStreams = false;
+
+        if (typeof nsIBinaryInputStream != "undefined")
+        {
+            var isCls = Components.classes["@mozilla.org/binaryinputstream;1"];
+            var inputStream = isCls.createInstance(nsIBinaryInputStream);
+            if ("readByteArray" in inputStream)
+                CBSConnection.prototype.workingBinaryStreams = true;
+        }
+    }
+
+    this.wrappedJSObject = this;
+    if (typeof binary != "undefined")
+        this.binaryMode = binary;
+    else
+        this.binaryMode = this.workingBinaryStreams;
+
+    if (!ASSERT(!this.binaryMode || this.workingBinaryStreams,
+                "Unable to use binary streams in this build."))
+    {
+        return null;
+    }
+
+    // Shut up a misguided strict warning.
+    return this;
 }
 
-CBSConnection.prototype.connect =
-function bc_connect(host, port, bind, tcp_flag, isSecure, observer)
-{
-    if (typeof tcp_flag == "undefined")
-        tcp_flag = false;
+CBSConnection.prototype.workingBinaryStreams = -1;
 
+CBSConnection.prototype.connect =
+function bc_connect(host, port, config, observer)
+{
     this.host = host.toLowerCase();
     this.port = port;
-    this.bind = bind;
-    this.tcp_flag = tcp_flag;
+
+    if (typeof config != "object")
+        config = {};
 
     // Lets get a transportInfo for this
-    var cls =
-        Components.classes["@mozilla.org/network/protocol-proxy-service;1"];
-    var pps = cls.getService(Components.interfaces.nsIProtocolProxyService);
-
+    var pps = getService("@mozilla.org/network/protocol-proxy-service;1",
+                         "nsIProtocolProxyService");
     if (!pps)
         throw ("Couldn't get protocol proxy service");
 
-    var ios = Components.classes["@mozilla.org/network/io-service;1"].
-      getService(Components.interfaces.nsIIOService);
-    var spec = "irc://" + host + ':' + port;
-    var uri = ios.newURI(spec,null,null);
-    var info = pps.examineForProxy(uri);
+    var ios = getService("@mozilla.org/network/io-service;1", "nsIIOService");
+
+    function getProxyFor(uri)
+    {
+        uri = ios.newURI(uri, null, null);
+        // As of 2005-03-25, 'examineForProxy' was replaced by 'resolve'.
+        if ("resolve" in pps)
+            return pps.resolve(uri, 0);
+        if ("examineForProxy" in pps)
+            return pps.examineForProxy(uri);
+        return null;
+    };
+
+    var proxyInfo = null;
+    var usingHTTPCONNECT = false;
+    if ("proxy" in config)
+    {
+        /* Force Necko to supply the HTTP proxy info if desired. For none,
+         * force no proxy. Other values will get default treatment.
+         */
+        if (config.proxy == "http")
+            proxyInfo = getProxyFor("http://" + host + ":" + port);
+        else if (config.proxy != "none")
+            proxyInfo = getProxyFor("irc://" + host + ":" + port);
+
+        /* Since the proxy info is opaque, we need to check that we got
+         * something for our HTTP proxy - we can't just check proxyInfo.type.
+         */
+        usingHTTPCONNECT = ((config.proxy == "http") && proxyInfo);
+    }
+    else
+    {
+        proxyInfo = getProxyFor("irc://" + host + ":" + port);
+    }
 
     if (jsenv.HAS_STREAM_PROVIDER)
     {
-        if (isSecure)
+        if (("isSecure" in config) && config.isSecure)
         {
             this._transport = this._sockService.
-                              createTransportOfType("ssl", host, port, info,
-                                                    0, 0);
+                              createTransportOfType("ssl", host, port,
+                                                    proxyInfo, 0, 0);
         }
         else
         {
             this._transport = this._sockService.
-                              createTransport(host, port, info, 0, 0);
+                              createTransport(host, port, proxyInfo, 0, 0);
         }
         if (!this._transport)
             throw ("Error creating transport.");
@@ -185,15 +261,16 @@ function bc_connect(host, port, bind, tcp_flag, isSecure, observer)
     else
     {
         /* use new necko interfaces */
-        if (isSecure)
+        if (("isSecure" in config) && config.isSecure)
         {
             this._transport = this._sockService.
-                              createTransport(["ssl"], 1, host, port, info);
+                              createTransport(["ssl"], 1, host, port,
+                                              proxyInfo);
         }
         else
         {
             this._transport = this._sockService.
-                              createTransport(null, 0, host, port, info);
+                              createTransport(null, 0, host, port, proxyInfo);
         }
         if (!this._transport)
             throw ("Error creating transport.");
@@ -223,6 +300,12 @@ function bc_connect(host, port, bind, tcp_flag, isSecure, observer)
     this.connectDate = new Date();
     this.isConnected = true;
 
+    // Bootstrap the connection if we're proxying via an HTTP proxy.
+    if (usingHTTPCONNECT)
+    {
+        this.sendData("CONNECT " + host + ":" + port + " HTTP/1.1\r\n\r\n");
+    }
+
     return this.isConnected;
 
 }
@@ -260,8 +343,6 @@ function bc_accept(transport, observer)
     this._transport = transport;
     this.host = this._transport.host.toLowerCase();
     this.port = this._transport.port;
-    //this.bind = bind;
-    //this.tcp_flag = tcp_flag;
 
     if (jsenv.HAS_STREAM_PROVIDER)
     {
@@ -318,7 +399,7 @@ function bc_accept(transport, observer)
     this.isConnected = true;
 
     // Clean up listening socket.
-    this._serverSock.close();
+    this.close();
 
     return this.isConnected;
 }
@@ -442,6 +523,62 @@ function bc_senddatanow(str)
     return rv;
 }
 
+/* getSecurityState returns an array containing information about the security
+ * of the connection. The array always has at least one item, which contains a
+ * value from the STATE_IS_* enumeration at the top of this file. Iff this is
+ * STATE_IS_SECURE, the array has a second item indicating the level of
+ * security - a value from the STATE_SECURE_* enumeration.
+ *
+ * STATE_IS_BROKEN is returned if any errors occur, and STATE_IS_INSECURE is
+ * returned for disconnected sockets.
+ */
+CBSConnection.prototype.getSecurityState =
+function bc_getsecuritystate()
+{
+    if (!this.isConnected || !this._transport.securityInfo)
+        return [STATE_IS_INSECURE];
+
+    try
+    {
+        var sslSp = Components.interfaces.nsISSLStatusProvider;
+        var sslStatus = Components.interfaces.nsISSLStatus;
+
+        // Get the actual SSL Status
+        sslSp = this._transport.securityInfo.QueryInterface(sslSp);
+        sslStatus = sslSp.SSLStatus.QueryInterface(sslStatus);
+        // Store appropriate status
+        if (!("keyLength" in sslStatus) || !sslStatus.keyLength)
+            return [STATE_IS_BROKEN];
+        else if (sslStatus.keyLength >= 90)
+            return [STATE_IS_SECURE, STATE_SECURE_HIGH];
+        else
+            return [STATE_IS_SECURE, STATE_SECURE_LOW];
+    }
+    catch (ex)
+    {
+        // Something goes wrong -> broken security icon
+        dd("Exception getting certificate for connection: " + ex.message);
+        return [STATE_IS_BROKEN];
+    }
+}
+
+CBSConnection.prototype.getCertificate =
+function bc_getcertificate()
+{
+    if (!this.isConnected || !this._transport.securityInfo)
+        return null;
+
+    var sslSp = Components.interfaces.nsISSLStatusProvider;
+    var sslStatus = Components.interfaces.nsISSLStatus;
+
+    // Get the actual SSL Status
+    sslSp = this._transport.securityInfo.QueryInterface(sslSp);
+    sslStatus = sslSp.SSLStatus.QueryInterface(sslStatus);
+
+    // return the certificate
+    return sslStatus.serverCert;
+}
+
 function _notimpl ()
 {
     throw "Not Implemented.";
@@ -548,6 +685,9 @@ function sl_dataavail (request, ctxt, inStr, sourceOffset, count)
         return;
     }
 
+    if (!("_sInputStream" in ctxt))
+        ctxt._sInputStream = toSInputStream(inStr, false);
+
     if (this._observer)
         this._observer.onStreamDataAvailable(request, inStr, sourceOffset,
                                              count);
