@@ -40,7 +40,7 @@
 var utils = new Object();
 
 var DEBUG = true;
-var dd;
+var dd, warn, TEST, ASSERT;
 
 if (DEBUG) {
     var _dd_pfx = "";
@@ -91,8 +91,22 @@ if (DEBUG) {
                  _dd_lastDumpWasOpen = false;
              }
          }
+    warn = function (msg) { dd("** WARNING " + msg + " **"); }
+    TEST = ASSERT = function _assert(expr, msg) {
+                 if (!expr) {
+                     var m = "** ASSERTION FAILED: " + msg + " **\n" +
+                             getStackTrace();
+                     try {
+                         alert(m);
+                     } catch(ex) {}
+                     dd(m);
+                     return false;
+                 } else {
+                     return true;
+                 }
+             }
 } else {
-    dd = function (){};
+    dd = warn = TEST = ASSERT = function (){};
 }
 
 var jsenv = new Object();
@@ -106,6 +120,7 @@ jsenv.HAS_DOCUMENT = (typeof document == "object");
 jsenv.HAS_NSPR_EVENTQ = jsenv.HAS_DOCUMENT;
 jsenv.HAS_STREAM_PROVIDER = ("nsIStreamProvider" in Components.interfaces);
 jsenv.HAS_SERVER_SOCKETS = ("nsIServerSocket" in Components.interfaces);
+jsenv.HAS_THREAD_MANAGER = ("nsIThreadManager" in Components.interfaces);
 
 function dumpObject (o, pfx, sep)
 {
@@ -260,7 +275,7 @@ function ecmaUnescape(str)
 {
     function replaceEscapes(seq)
     {
-        var ary = seq.match(/([\da-f]{1,2})(.*)|u([\da-f]{1,4})/);
+        var ary = seq.match(/([\da-f]{1,2})(.*)|u([\da-f]{1,4})/i);
         if (!ary)
             return "<ERROR>";
 
@@ -302,11 +317,18 @@ function replaceVars(str, vars)
     return str.replace(/(\$\w[\w\d\-]+)/g, doReplace);
 }
 
-function formatException (ex)
+function formatException(ex)
 {
-    if (ex instanceof Error)
-        return getMsg (MSG_FMT_JSEXCEPTION, [ex.name, ex.message, ex.fileName,
-                                             ex.lineNumber]);
+    if (isinstance(ex, Error))
+    {
+        return getMsg(MSG_FMT_JSEXCEPTION, [ex.name, ex.message, ex.fileName,
+                                            ex.lineNumber]);
+    }
+    if ((typeof ex == "object") && ("filename" in ex))
+    {
+        return getMsg(MSG_FMT_JSEXCEPTION, [ex.name, ex.message, ex.filename,
+                                            ex.lineNumber]);
+    }
 
     return String(ex);
 }
@@ -322,8 +344,21 @@ function Clone (obj)
 {
     var robj = new Object();
 
-    for (var p in obj)
-        robj[p] = obj[p];
+    if ("__proto__" in obj)
+    {
+        // Special clone for Spidermonkey.
+        for (var p in obj)
+        {
+            if (obj.hasOwnProperty(p))
+                robj[p] = obj[p];
+        }
+        robj.__proto__ = obj.__proto__;
+    }
+    else
+    {
+        for (var p in obj)
+            robj[p] = obj[p];
+    }
 
     return robj;
 
@@ -345,7 +380,7 @@ function Copy(source, dest, overwrite)
 
 /*
  * matches a real object against one or more pattern objects.
- * if you pass an array of pattern objects, |negate| controls wether to check
+ * if you pass an array of pattern objects, |negate| controls whether to check
  * if the object matches ANY of the patterns, or NONE of the patterns.
  */
 function matchObject (o, pattern, negate)
@@ -354,7 +389,7 @@ function matchObject (o, pattern, negate)
 
     function _match (o, pattern)
     {
-        if (pattern instanceof Function)
+        if (isinstance(pattern, Function))
             return pattern(o);
 
         for (p in pattern)
@@ -370,7 +405,7 @@ function matchObject (o, pattern, negate)
                 */
             val = o[p];
 
-            if (pattern[p] instanceof Function)
+            if (isinstance(pattern[p], Function))
             {
                 if (!pattern[p](val))
                     return false;
@@ -389,7 +424,7 @@ function matchObject (o, pattern, negate)
 
     }
 
-    if (!(pattern instanceof Array))
+    if (!isinstance(pattern, Array))
         return Boolean (negate ^ _match(o, pattern));
 
     for (var i in pattern)
@@ -506,7 +541,7 @@ function newObject(contractID, iface)
 
     var rv;
     var cls = Components.classes[contractID];
-    
+
     if (!cls)
         return null;
 
@@ -540,7 +575,7 @@ function getService(contractID, iface)
 
     var rv;
     var cls = Components.classes[contractID];
-    
+
     if (!cls)
         return null;
 
@@ -647,14 +682,26 @@ function formatDateOffset (offset, format)
     if (!format)
     {
         var ary = new Array();
-        if (days > 0)
-            ary.push (getMsg(MSG_DAYS, days));
-        if (hours > 0)
-            ary.push (getMsg(MSG_HOURS, hours));
-        if (minutes > 0)
-            ary.push (getMsg(MSG_MINUTES, minutes));
-        if (seconds > 0 || offset == 0)
-            ary.push (getMsg(MSG_SECONDS, seconds));
+
+        if (days == 1)
+            ary.push(MSG_DAY);
+        else if (days > 0)
+            ary.push(getMsg(MSG_DAYS, days));
+
+        if (hours == 1)
+            ary.push(MSG_HOUR);
+        else if (hours > 0)
+            ary.push(getMsg(MSG_HOURS, hours));
+
+        if (minutes == 1)
+            ary.push(MSG_MINUTE);
+        else if (minutes > 0)
+            ary.push(getMsg(MSG_MINUTES, minutes));
+
+        if (seconds == 1)
+            ary.push(MSG_SECOND);
+        else if (seconds > 0 || offset == 0)
+            ary.push(getMsg(MSG_SECONDS, seconds));
 
         format = ary.join(", ");
     }
@@ -892,9 +939,24 @@ function getFileFromURLSpec(url)
 {
     const nsIFileProtocolHandler = Components.interfaces.nsIFileProtocolHandler;
 
-    var service = getService("@mozilla.org/network/io-service;1", 
+    var service = getService("@mozilla.org/network/io-service;1",
                              "nsIIOService");
-    
+
+    /* In sept 2002, bug 166792 moved this method to the nsIFileProtocolHandler
+     * interface, but we need to support older versions too. */
+    if ("getFileFromURLSpec" in service)
+        return service.getFileFromURLSpec(url);
+
+    /* In builds before 2002-08-15, there is no getFileFromURLSpec at all.
+     * Instead, we have nsIIOservice.initFileFromURLSpec(nsIFile, string).
+     */
+    if ("initFileFromURLSpec" in service)
+    {
+        var file = newObject("@mozilla.org/file/local;1", "nsILocalFile");
+        service.initFileFromURLSpec(file, url);
+        return file;
+    }
+
     var handler = service.getProtocolHandler("file");
     handler = handler.QueryInterface(nsIFileProtocolHandler);
     return handler.getFileFromURLSpec(url);
@@ -955,6 +1017,66 @@ function confirm(msg, parent, title)
     return ps.confirm (parent, title, msg);
 }
 
+function confirmEx(msg, buttons, defaultButton, checkText,
+                   checkVal, parent, title)
+{
+    /* Note that on versions before Mozilla 0.9, using 3 buttons,
+     * the revert or dontsave button, or custom button titles will NOT work.
+     *
+     * The buttons should be listed in the 'accept', 'cancel' and 'extra' order,
+     * and the exact button order is host app- and platform-dependant.
+     * For example, on Windows this is usually [button 1] [button 3] [button 2],
+     * and on Linux [button 3] [button 2] [button 1].
+     */
+    var PROMPT_CTRID = "@mozilla.org/embedcomp/prompt-service;1";
+    var nsIPromptService = Components.interfaces.nsIPromptService;
+    var ps = Components.classes[PROMPT_CTRID].getService(nsIPromptService);
+
+    var buttonConstants = {
+        ok: ps.BUTTON_TITLE_OK,
+        cancel: ps.BUTTON_TITLE_CANCEL,
+        yes: ps.BUTTON_TITLE_YES,
+        no: ps.BUTTON_TITLE_NO,
+        save: ps.BUTTON_TITLE_SAVE,
+        revert: ps.BUTTON_TITLE_REVERT,
+        dontsave: ps.BUTTON_TITLE_DONT_SAVE
+    };
+    var buttonFlags = 0;
+    var buttonText = [null, null, null];
+
+    if (!isinstance(buttons, Array))
+        throw "buttons parameter must be an Array";
+    if ((buttons.length < 1) || (buttons.length > 3))
+        throw "the buttons array must have 1, 2 or 3 elements";
+
+    for (var i = 0; i < buttons.length; i++)
+    {
+        var buttonFlag = ps.BUTTON_TITLE_IS_STRING;
+        if ((buttons[i][0] == "!") && (buttons[i].substr(1) in buttonConstants))
+            buttonFlag = buttonConstants[buttons[i].substr(1)];
+        else
+            buttonText[i] = buttons[i];
+
+        buttonFlags += ps["BUTTON_POS_" + i] * buttonFlag;
+    }
+
+    // ignore anything but a proper number
+    var defaultIsNumber = (typeof defaultButton == "number");
+    if (defaultIsNumber && arrayHasElementAt(buttons, defaultButton))
+        buttonFlags += ps["BUTTON_POS_" + defaultButton + "_DEFAULT"];
+
+    if (!parent)
+        parent = window;
+    if (!title)
+        title = MSG_CONFIRM;
+    if (!checkVal)
+        checkVal = new Object();
+
+    var rv = ps.confirmEx(parent, title, msg, buttonFlags, buttonText[0],
+                          buttonText[1], buttonText[2], checkText, checkVal);
+    return rv;
+}
+
 function prompt(msg, initial, parent, title)
 {
     var PROMPT_CTRID = "@mozilla.org/embedcomp/prompt-service;1";
@@ -964,7 +1086,7 @@ function prompt(msg, initial, parent, title)
         parent = window;
     if (!title)
         title = MSG_PROMPT;
-    rv = { value: initial };
+    var rv = { value: initial };
 
     if (!ps.prompt (parent, title, msg, rv, null, {value: null}))
         return null;
@@ -981,7 +1103,7 @@ function promptPassword(msg, initial, parent, title)
         parent = window;
     if (!title)
         title = MSG_PROMPT;
-    rv = { value: initial };
+    var rv = { value: initial };
 
     if (!ps.promptPassword (parent, title, msg, rv, null, {value: null}))
         return null;
@@ -989,6 +1111,15 @@ function promptPassword(msg, initial, parent, title)
     return rv.value;
 }
 
+function viewCert(cert, parent)
+{
+    var cd = getService("@mozilla.org/nsCertificateDialogs;1",
+                        "nsICertificateDialogs");
+    if (!parent)
+        parent = window;
+    cd.viewCert(parent, cert);
+}
+
 function getHostmaskParts(hostmask)
 {
     var rv;
@@ -1017,6 +1148,59 @@ function getHostmaskParts(hostmask)
     return rv;
 }
 
+function makeMaskRegExp(text)
+{
+    function escapeChars(c)
+    {
+        if (c == "*")
+            return ".*";
+        if (c == "?")
+            return ".";
+        return "\\" + c;
+    }
+    // Anything that's not alpha-numeric gets escaped.
+    // "*" and "?" are 'escaped' to ".*" and ".".
+    // Optimisation; * translates as 'match all'.
+    return new RegExp("^" + text.replace(/[^\w\d]/g, escapeChars) + "$", "i");
+}
+
+function hostmaskMatches(user, mask)
+{
+    // Need to match .nick, .user, and .host.
+    if (!("nickRE" in mask))
+    {
+        // We cache all the regexp objects, but use null if the term is
+        // just "*", so we can skip having the object *and* the .match
+        // later on.
+        if (mask.nick == "*")
+            mask.nickRE = null;
+        else
+            mask.nickRE = makeMaskRegExp(mask.nick);
+
+        if (mask.user == "*")
+            mask.userRE = null;
+        else
+            mask.userRE = makeMaskRegExp(mask.user);
+
+        if (mask.host == "*")
+            mask.hostRE = null;
+        else
+            mask.hostRE = makeMaskRegExp(mask.host);
+    }
+
+    var lowerNick;
+    if (user.TYPE == "IRCChanUser")
+        lowerNick = user.parent.parent.toLowerCase(user.unicodeName);
+    else
+        lowerNick = user.parent.toLowerCase(user.unicodeName);
+
+    if ((!mask.nickRE || lowerNick.match(mask.nickRE)) &&
+        (!mask.userRE || user.name.match(mask.userRE)) &&
+        (!mask.hostRE || user.host.match(mask.hostRE)))
+        return true;
+    return false;
+}
+
 function isinstance(inst, base)
 {
     /* Returns |true| if |inst| was constructed by |base|. Not 100% accurate,
@@ -1024,6 +1208,69 @@ function isinstance(inst, base)
      * 254067 which makes instanceof fail if the two sides are 'from'
      * different windows (something we don't care about).
      */
-    return (inst && inst.constructor && base &&
-            (inst.constructor.name == base.name));
+    return (inst && base &&
+            ((inst instanceof base) ||
+             (inst.constructor && (inst.constructor.name == base.name))));
+}
+
+function scaleNumberBy1024(number)
+{
+    var scale = 0;
+    while ((number >= 1000) && (scale < 6))
+    {
+        scale++;
+        number /= 1024;
+    }
+
+    return [scale, number];
+}
+
+function getSISize(size)
+{
+    var data = scaleNumberBy1024(size);
+
+    if (data[1] < 10)
+        data[1] = data[1].toFixed(2);
+    else if (data[1] < 100)
+        data[1] = data[1].toFixed(1);
+    else
+        data[1] = data[1].toFixed(0);
+
+    return getMsg(MSG_SI_SIZE, [data[1], getMsg("msg.si.size." + data[0])]);
+}
+
+function getSISpeed(speed)
+{
+    var data = scaleNumberBy1024(speed);
+
+    if (data[1] < 10)
+        data[1] = data[1].toFixed(2);
+    else if (data[1] < 100)
+        data[1] = data[1].toFixed(1);
+    else
+        data[1] = data[1].toFixed(0);
+
+    return getMsg(MSG_SI_SPEED, [data[1], getMsg("msg.si.speed." + data[0])]);
+}
+
+// Returns -1 if version 1 is newer, +1 if version 2 is newer, and 0 for same.
+function compareVersions(ver1, ver2)
+{
+    var ver1parts = ver1.split(".");
+    var ver2parts = ver2.split(".");
+
+    while ((ver1parts.length > 0) && (ver2parts.length > 0))
+    {
+        if (ver1parts[0] < ver2parts[0])
+            return 1;
+        if (ver1parts[0] > ver2parts[0])
+            return -1;
+        ver1parts.shift();
+        ver2parts.shift();
+    }
+    if (ver1parts.length > 0)
+        return -1;
+    if (ver2parts.length > 0)
+        return 1;
+    return 0;
 }
