@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,25 +14,25 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * Mitesh Shah <mitesh@netscape.com>
+ *   Mitesh Shah <mitesh@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -44,11 +44,11 @@
 #include "nsIURI.h"
 #include "nsIHttpChannel.h"
 #include "nsIFileStreams.h"
+#include "nsThreadUtils.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "prmem.h"
 #include "nsIProfile.h"
 #include "nsIObserverService.h"
-#include "nsIEventQueueService.h"
 #include "nsLiteralString.h"
 #include "nsIPromptService.h"
 #include "nsIServiceManager.h"
@@ -353,19 +353,11 @@ nsresult nsAutoConfig::downloadAutoConfig()
 
         firstTime = PR_FALSE;
     
-        // Getting an event queue. If we start an AsyncOpen, the thread
+        // Getting the current thread. If we start an AsyncOpen, the thread
         // needs to wait before the reading of autoconfig is done
 
-        nsCOMPtr<nsIEventQueueService> service = 
-            do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID, &rv);
-        if (NS_FAILED(rv)) 
-            return rv;
-
-        nsCOMPtr<nsIEventQueue> currentThreadQ;
-        rv = service->GetThreadEventQueue(NS_CURRENT_THREAD,
-                                          getter_AddRefs(currentThreadQ));
-        if (NS_FAILED(rv)) 
-            return rv;
+        nsCOMPtr<nsIThread> thread = do_GetCurrentThread();
+        NS_ENSURE_STATE(thread);
     
         /* process events until we're finished. AutoConfig.jsc reading needs
            to be finished before the browser starts loading up
@@ -375,18 +367,8 @@ nsresult nsAutoConfig::downloadAutoConfig()
            that mLoaded will be set to true in any case (success/failure)
         */
         
-        while (!mLoaded) {
-
-            PRBool isEventPending;
-            rv = currentThreadQ->PendingEvents(&isEventPending);
-            if (NS_FAILED(rv)) 
-                return rv;        
-            if (isEventPending) {
-                rv = currentThreadQ->ProcessPendingEvents();
-                if (NS_FAILED(rv)) 
-                    return rv;        
-            }
-        }
+        while (!mLoaded)
+            NS_ENSURE_STATE(NS_ProcessNextEvent(thread));
         
         PRInt32 minutes = 0;
         rv = mPrefBranch->GetIntPref("autoadmin.refresh_interval", 
@@ -536,30 +518,28 @@ nsresult nsAutoConfig::getEmailAddr(nsACString & emailAddr)
     
     rv = mPrefBranch->GetCharPref("mail.accountmanager.defaultaccount", 
                                   getter_Copies(prefValue));
-    // Checking prefValue and its length.  Since by default the preference 
-    // is set to nothing
-    PRUint32 len;
-    if (NS_SUCCEEDED(rv) && (len = strlen(prefValue)) > 0) {
-        emailAddr = NS_LITERAL_CSTRING("mail.account.") + 
-            nsDependentCString(prefValue, len) + NS_LITERAL_CSTRING(".identities");
+    
+    if (NS_SUCCEEDED(rv) && !prefValue.IsEmpty()) {
+        emailAddr = NS_LITERAL_CSTRING("mail.account.") +
+            prefValue + NS_LITERAL_CSTRING(".identities");
         rv = mPrefBranch->GetCharPref(PromiseFlatCString(emailAddr).get(),
                                       getter_Copies(prefValue));
-        if (NS_FAILED(rv) || (len = strlen(prefValue)) == 0) 
-            return rv;
-        emailAddr = NS_LITERAL_CSTRING("mail.identity.") + 
-            nsDependentCString(prefValue, len) + NS_LITERAL_CSTRING(".useremail");
+        if (NS_FAILED(rv) || prefValue.IsEmpty())
+            return PromptForEMailAddress(emailAddr);
+        emailAddr = NS_LITERAL_CSTRING("mail.identity.") +
+            prefValue + NS_LITERAL_CSTRING(".useremail");
         rv = mPrefBranch->GetCharPref(PromiseFlatCString(emailAddr).get(),
                                       getter_Copies(prefValue));
-        if (NS_FAILED(rv)  || (len = strlen(prefValue)) == 0) 
-            return rv;
-        emailAddr = nsDependentCString(prefValue, len);
+        if (NS_FAILED(rv)  || prefValue.IsEmpty())
+            return PromptForEMailAddress(emailAddr);
+        emailAddr = prefValue;
     }
     else {
         // look for 4.x pref in case we just migrated.
         rv = mPrefBranch->GetCharPref("mail.identity.useremail", 
                                   getter_Copies(prefValue));
         if (NS_SUCCEEDED(rv) && !prefValue.IsEmpty())
-          emailAddr = prefValue;
+            emailAddr = prefValue;
         else if (NS_FAILED(PromptForEMailAddress(emailAddr))  && (!mCurrProfile.IsEmpty()))
             emailAddr = mCurrProfile;
     }
@@ -594,7 +574,7 @@ nsresult nsAutoConfig::PromptForEMailAddress(nsACString &emailAddress)
     if (!success)
       return NS_ERROR_FAILURE;
     NS_ENSURE_SUCCESS(rv, rv);
-    CopyUCS2toASCII(emailResult, emailAddress);
+    LossyCopyUTF16toASCII(emailResult, emailAddress);
     return NS_OK;
 }
 
