@@ -41,7 +41,6 @@
 #include "nsIServiceManager.h"
 #include "nsIGenericFactory.h"
 #include "nsIWebBrowserChrome.h"
-#include "nsIDocumentLoader.h"
 #include "nsCURILoader.h"
 #include "nsNetUtil.h"
 #include "nsIURL.h"
@@ -50,15 +49,13 @@
 #include "nsIDocShellTreeOwner.h"
 #include "nsIEditorDocShell.h"
 #include "nsISimpleEnumerator.h"
-#include "nsIDOMWindow.h"
-#include "nsIDOMWindowInternal.h"
 #include "nsPIDOMWindow.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMNSUIEvent.h"
 #include "nsIChromeEventHandler.h"
 #include "nsIDOMNSEvent.h"
 #include "nsIPrefBranch.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "nsIPrefService.h"
 #include "nsString.h"
 #include "nsCRT.h"
@@ -81,12 +78,10 @@
 #include "nsISelectionPrivate.h"
 #include "nsISelectElement.h"
 #include "nsILink.h"
-#include "nsITextContent.h"
 #include "nsTextFragment.h"
 #include "nsILookAndFeel.h"
 
 #include "nsICaret.h"
-#include "nsIScriptGlobalObject.h"
 #include "nsIDOMKeyEvent.h"
 #include "nsIDocShellTreeItem.h"
 #include "nsIWebNavigation.h"
@@ -115,7 +110,7 @@
 
 
 NS_INTERFACE_MAP_BEGIN(nsTypeAheadFind)
-  NS_INTERFACE_MAP_ENTRY(nsITypeAheadFind)
+  NS_INTERFACE_MAP_ENTRY(nsISuiteTypeAheadFind)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
   NS_INTERFACE_MAP_ENTRY(nsITimerCallback)
   NS_INTERFACE_MAP_ENTRY(nsIScrollPositionListener)
@@ -132,7 +127,6 @@ NS_IMPL_ADDREF(nsTypeAheadFind)
 NS_IMPL_RELEASE(nsTypeAheadFind)
 
 static NS_DEFINE_IID(kRangeCID, NS_RANGE_CID);
-static NS_DEFINE_CID(kStringBundleServiceCID,  NS_STRINGBUNDLESERVICE_CID);
 static NS_DEFINE_CID(kFrameTraversalCID, NS_FRAMETRAVERSAL_CID);
 static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
 
@@ -171,7 +165,7 @@ nsTypeAheadFind::~nsTypeAheadFind()
   RemoveDocListeners();
   mTimer = nsnull;
 
-  nsCOMPtr<nsIPrefBranchInternal> prefInternal(do_GetService(NS_PREFSERVICE_CONTRACTID));
+  nsCOMPtr<nsIPrefBranch2> prefInternal(do_GetService(NS_PREFSERVICE_CONTRACTID));
   if (prefInternal) {
     prefInternal->RemoveObserver("accessibility.typeaheadfind", this);
     prefInternal->RemoveObserver("accessibility.browsewithcaret", this);
@@ -185,7 +179,7 @@ nsTypeAheadFind::Init()
   NS_ENSURE_SUCCESS(rv, rv);
 
 
-  nsCOMPtr<nsIPrefBranchInternal> prefInternal(do_GetService(NS_PREFSERVICE_CONTRACTID));
+  nsCOMPtr<nsIPrefBranch2> prefInternal(do_GetService(NS_PREFSERVICE_CONTRACTID));
   mSearchRange = do_CreateInstance(kRangeCID);
   mStartPointRange = do_CreateInstance(kRangeCID);
   mEndPointRange = do_CreateInstance(kRangeCID);
@@ -288,7 +282,7 @@ nsTypeAheadFind::PrefsReset()
 
       // Initialize string bundle
       nsCOMPtr<nsIStringBundleService> stringBundleService =
-        do_GetService(kStringBundleServiceCID);
+        do_GetService(NS_STRINGBUNDLE_CONTRACTID);
 
       if (stringBundleService)
         stringBundleService->CreateBundle(TYPEAHEADFIND_BUNDLE_URL,
@@ -328,7 +322,7 @@ nsTypeAheadFind::PrefsReset()
   }
   mNotFoundSoundURL = soundStr;
 
-  PRBool isTimeoutEnabled;
+  PRBool isTimeoutEnabled = PR_FALSE;
   prefBranch->GetBoolPref("accessibility.typeaheadfind.enabletimeout",
                           &isTimeoutEnabled);
   PRInt32 timeoutLength = 0;
@@ -501,7 +495,7 @@ nsTypeAheadFind::UseInWindow(nsIDOMWindow *aDOMWin)
     return NS_OK;
   }
 
-  nsCOMPtr<nsIPresShell> oldPresShell(do_QueryReferent(mFocusedWeakShell));
+  nsCOMPtr<nsIPresShell> oldPresShell(GetPresShell());
 
   if (!oldPresShell || oldPresShell != presShell) {
     CancelFind();
@@ -556,7 +550,7 @@ nsTypeAheadFind::HandleEvent(nsIDOMEvent* aEvent)
     nsCOMPtr<nsIDOMEventTarget> eventTarget;
     event->GetOriginalTarget(getter_AddRefs(eventTarget));
     nsCOMPtr<nsIDocument> doc(do_QueryInterface(eventTarget));
-    nsCOMPtr<nsIPresShell> focusedShell(do_QueryReferent(mFocusedWeakShell));
+    nsCOMPtr<nsIPresShell> focusedShell(GetPresShell());
     if (!focusedShell || !doc) {
       return NS_ERROR_FAILURE;
     }
@@ -726,7 +720,8 @@ nsTypeAheadFind::KeyPress(nsIDOMEvent* aEvent)
     return NS_OK;
   }
 
-  aEvent->StopPropagation();  // We're using this key, no one else should
+  // We're using this key, no one else should
+  aEvent->PreventDefault();
 
   return HandleChar(charCode);
 }
@@ -876,7 +871,7 @@ nsTypeAheadFind::HandleChar(PRUnichar aChar)
     return NS_ERROR_FAILURE;
   }
 
-  aChar = ToLowerCase(NS_STATIC_CAST(PRUnichar, aChar));
+  aChar = ToLowerCase(aChar);
   PRInt32 bufferLength = mTypeAheadBuffer.Length();
 
   mIsFirstVisiblePreferred = PR_FALSE;
@@ -933,7 +928,7 @@ nsTypeAheadFind::HandleChar(PRUnichar aChar)
       // If not, make sure the selection is in sync with the focus, so we can 
       // start our search from there.
       nsCOMPtr<nsIContent> focusedContent;
-      nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mFocusedWeakShell));
+      nsCOMPtr<nsIPresShell> presShell(GetPresShell());
       NS_ENSURE_TRUE(presShell, NS_OK);
       nsPresContext *presContext = presShell->GetPresContext();
       NS_ENSURE_TRUE(presContext, NS_OK);
@@ -1211,14 +1206,20 @@ nsTypeAheadFind::HandleQueryReconversion(nsIDOMEvent* aCompositionEvent)
 }
 
 
+NS_IMETHODIMP
+nsTypeAheadFind::HandleQueryCaretRect(nsIDOMEvent* aCompositionEvent)
+{
+  return NS_OK;
+}
+
+
 nsresult
 nsTypeAheadFind::FindItNow(nsIPresShell *aPresShell,
                            PRBool aIsRepeatingSameChar, PRBool aIsLinksOnly,
                            PRBool aIsFirstVisiblePreferred)
 {
   nsCOMPtr<nsIPresShell> presShell(aPresShell);
-  nsCOMPtr<nsIPresShell> startingPresShell =
-    do_QueryReferent(mFocusedWeakShell);
+  nsCOMPtr<nsIPresShell> startingPresShell = GetPresShell();
 
   if (!presShell) {
     presShell = startingPresShell;  // this is the current document
@@ -1355,10 +1356,11 @@ nsTypeAheadFind::FindItNow(nsIPresShell *aPresShell,
         // Get selection controller and selection for new frame/iframe
         GetSelection(presShell, getter_AddRefs(mFocusedDocSelCon), 
                      getter_AddRefs(mFocusedDocSelection));
-        if (!mFocusedDocSelection || !mFocusedDocSelCon) {
-          // Apparently these can go away even though presshell/prescontext exist
-          return NS_ERROR_FAILURE;
-        }
+      }
+ 
+     if (!mFocusedDocSelection || !mFocusedDocSelCon) {
+        // Apparently these can go away even though presshell/prescontext exist
+        return NS_ERROR_FAILURE;
       }
 
       // Select the found text and focus it
@@ -1518,8 +1520,7 @@ nsTypeAheadFind::GetSearchContainers(nsISupports *aContainer,
   // Consider current selection as null if
   // it's not in the currently focused document
   nsCOMPtr<nsIDOMRange> currentSelectionRange;
-  nsCOMPtr<nsIPresShell> selectionPresShell =
-    do_QueryReferent(mFocusedWeakShell);
+  nsCOMPtr<nsIPresShell> selectionPresShell = GetPresShell();
 
   if (aCanUseDocSelection && selectionPresShell == presShell && mFocusedDocSelection) {
     mFocusedDocSelection->GetRangeAt(0, getter_AddRefs(currentSelectionRange));
@@ -1589,19 +1590,16 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
   }
   origContent = startContent;
 
-  if (startContent->IsContentOfType(nsIContent::eELEMENT)) {
+  if (startContent->IsNodeOfType(nsINode::eELEMENT)) {
     nsIContent *childContent = startContent->GetChildAt(startOffset);
     if (childContent) {
       startContent = childContent;
     }
   }
   else if (startOffset > 0) {
-    nsCOMPtr<nsITextContent> textContent(do_QueryInterface(startContent));
-
-    if (textContent) {
+    const nsTextFragment *textFrag = startContent->GetText();
+    if (textFrag) {
       // look for non whitespace character before start offset
-      const nsTextFragment *textFrag = textContent->Text();
-
       for (PRInt32 index = 0; index < startOffset; index++) {
         if (!XP_IS_SPACE(textFrag->CharAt(index))) {
           *aIsStartingLink = PR_FALSE;  // not at start of a node
@@ -1624,7 +1622,7 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
     // Keep testing while textContent is equal to something,
     // eventually we'll run out of ancestors
 
-    if (startContent->IsContentOfType(nsIContent::eHTML)) {
+    if (startContent->IsNodeOfType(nsINode::eHTML)) {
       nsCOMPtr<nsILink> link(do_QueryInterface(startContent));
       if (link) {
         // Check to see if inside HTML link
@@ -1636,9 +1634,9 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
       // Any xml element can be an xlink
       *aIsInsideLink = startContent->HasAttr(kNameSpaceID_XLink, hrefAtom);
       if (*aIsInsideLink) {
-        nsAutoString xlinkType;
-        startContent->GetAttr(kNameSpaceID_XLink, typeAtom, xlinkType);
-        if (!xlinkType.EqualsLiteral("simple")) {
+        if (!startContent->AttrValueIs(kNameSpaceID_XLink, typeAtom,
+                                       NS_LITERAL_STRING("simple"),
+                                       eCaseMatters)) {
           *aIsInsideLink = PR_FALSE;  // Xlink must be type="simple"
         }
 
@@ -1650,11 +1648,8 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
     nsCOMPtr<nsIContent> parent = startContent->GetParent();
     if (parent) {
       nsIContent *parentsFirstChild = parent->GetChildAt(0);
-      nsCOMPtr<nsITextContent> textContent =
-        do_QueryInterface(parentsFirstChild);
-
       // We don't want to look at a whitespace-only first child
-      if (textContent && textContent->IsOnlyWhitespace())
+      if (parentsFirstChild && parentsFirstChild->TextIsOnlyWhitespace())
         parentsFirstChild = parent->GetChildAt(1);
 
       if (parentsFirstChild != startContent) {
@@ -1694,13 +1689,13 @@ nsTypeAheadFind::ScrollPositionDidChange(nsIScrollableView *aScrollableView,
 
 NS_IMETHODIMP
 nsTypeAheadFind::NotifySelectionChanged(nsIDOMDocument *aDoc,
-                                        nsISelection *aSel, short aReason)
+                                        nsISelection *aSel, PRInt16 aReason)
 {
   if (!mIsFindingText) {
     if (mRepeatingMode != eRepeatingNone) {
       // Selection had changed color for Type Ahead Find's version of Accel+G
       // We change it back when the selection changes from someone else
-      nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mFocusedWeakShell));
+      nsCOMPtr<nsIPresShell> presShell(GetPresShell());
       SetSelectionLook(presShell, PR_FALSE, PR_FALSE);
     }
     CancelFind();
@@ -1710,7 +1705,7 @@ nsTypeAheadFind::NotifySelectionChanged(nsIDOMDocument *aDoc,
 }
 
 
-// ---------------- nsITypeAheadFind --------------------
+// ---------------- nsISuiteTypeAheadFind --------------------
 
 NS_IMETHODIMP
 nsTypeAheadFind::FindNext(PRBool aFindBackwards, nsISupportsInterfacePointer *aCallerWindowSupports)
@@ -1729,7 +1724,7 @@ nsTypeAheadFind::FindNext(PRBool aFindBackwards, nsISupportsInterfacePointer *aC
   // with the top level content pres shell window where find next is happening
   // If they're different, exit so that webbrowswerfind can handle FindNext()
 
-  nsCOMPtr<nsIPresShell> typeAheadPresShell(do_QueryReferent(mFocusedWeakShell));
+  nsCOMPtr<nsIPresShell> typeAheadPresShell(GetPresShell());
   NS_ENSURE_TRUE(typeAheadPresShell, NS_OK);
 
   nsPresContext *presContext = typeAheadPresShell->GetPresContext();
@@ -1862,7 +1857,7 @@ nsTypeAheadFind::StartNewFind(nsIDOMWindow *aWindow, PRBool aLinksOnly)
     mIsFindingText = PR_TRUE;  // Turn off side effects from selection listener
     mFocusedDocSelection->CollapseToStart();
     mIsFindingText = PR_FALSE;
-    nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mFocusedWeakShell));
+    nsCOMPtr<nsIPresShell> presShell(GetPresShell());
     SetSelectionLook(presShell, PR_TRUE, PR_TRUE);
   }
   DisplayStatus(PR_TRUE, nsnull, PR_FALSE);
@@ -2026,7 +2021,7 @@ nsTypeAheadFind::CancelFind()
   if (mIsTypeAheadOn || mRepeatingMode != eRepeatingNone) {
     mTypeAheadBuffer.Truncate();
     DisplayStatus(PR_FALSE, nsnull, PR_TRUE); // Clear status
-    nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mFocusedWeakShell));
+    nsCOMPtr<nsIPresShell> presShell(GetPresShell());
     SetSelectionLook(presShell, PR_FALSE, PR_FALSE);
   }
 
@@ -2207,7 +2202,7 @@ nsTypeAheadFind::SetSelectionLook(nsIPresShell *aPresShell,
 void
 nsTypeAheadFind::RemoveDocListeners()
 {
-  nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mFocusedWeakShell));
+  nsCOMPtr<nsIPresShell> presShell(GetPresShell());
   nsIViewManager* vm = nsnull;
 
   if (presShell) {
@@ -2401,7 +2396,7 @@ nsTypeAheadFind::IsTargetContentOkay(nsIContent *aContent)
     return PR_FALSE;
   }
 
-  if (aContent->IsContentOfType(nsIContent::eHTML_FORM_CONTROL)) {
+  if (aContent->IsNodeOfType(nsINode::eHTML_FORM_CONTROL)) {
     nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(aContent));
     PRInt32 controlType = formControl->GetType();
     if (controlType == NS_FORM_SELECT || 
@@ -2415,7 +2410,7 @@ nsTypeAheadFind::IsTargetContentOkay(nsIContent *aContent)
       return PR_FALSE;
     }
   }
-  else if (aContent->IsContentOfType(nsIContent::eHTML)) {
+  else if (aContent->IsNodeOfType(nsINode::eHTML)) {
     // Test for isindex, a deprecated kind of text field. We're using a string 
     // compare because <isindex> is not considered a form control, so it does 
     // not support nsIFormControl or eHTML_FORM_CONTROL, and it's not worth 
@@ -2472,7 +2467,7 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
     return NS_OK;
   }
 
-  nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(doc->GetScriptGlobalObject()));
+  nsIDOMWindow *domWin = doc->GetWindow();
   nsCOMPtr<nsIDOMWindow> topContentWin;
   GetStartWindow(domWin, getter_AddRefs(topContentWin));
 
@@ -2483,7 +2478,7 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
     return NS_OK;
   }
 
-  nsCOMPtr<nsIPresShell> lastShell(do_QueryReferent(mFocusedWeakShell));
+  nsCOMPtr<nsIPresShell> lastShell(GetPresShell());
 
   if (lastShell != presShell || topContentWin != mFocusedWindow) {
     GetAutoStart(topContentWin, &mIsFindAllowedInWindow);
@@ -2562,8 +2557,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
     return PR_FALSE;
   }
 
-  nsIFrame *frame = nsnull;
-  aPresShell->GetPrimaryFrameFor(content, &frame);
+  nsIFrame *frame = aPresShell->GetPrimaryFrameFor(content);
   if (!frame) {
     // No frame! Not visible then.
 
@@ -2590,9 +2584,9 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
     if (startRangeOffset < endFrameOffset) {
       break;
     }
-    nsIFrame *nextInFlowFrame = frame->GetNextInFlow();
-    if (nextInFlowFrame) {
-      frame = nextInFlowFrame;
+    nsIFrame *nextContinuationFrame = frame->GetNextContinuation();
+    if (nextContinuationFrame) {
+      frame = nextContinuationFrame;
     }
     else {
       break;
@@ -2619,7 +2613,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
 
   if (!aGetTopVisibleLeaf) {
     nsRect relFrameRect = frame->GetRect();
-    frame->GetOffsetFromView(aPresContext, frameOffset, &containingView);
+    frame->GetOffsetFromView(frameOffset, &containingView);
     if (!containingView) {
       // no view -- not visible
 
@@ -2645,8 +2639,13 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
   nsCOMPtr<nsIBidirectionalEnumerator> frameTraversal;
   nsCOMPtr<nsIFrameTraversal> trav(do_CreateInstance(kFrameTraversalCID));
   if (trav) {
-    trav->NewFrameTraversal(getter_AddRefs(frameTraversal), LEAF,
-                            aPresContext, frame);
+    trav->NewFrameTraversal(getter_AddRefs(frameTraversal),
+                            aPresContext, frame,
+                            eLeaf,
+                            PR_FALSE, // aVisual
+                            PR_FALSE, // aLockInScrollView
+                            PR_FALSE  // aFollowOOFs
+                            );
   }
 
   if (!frameTraversal) {
@@ -2664,7 +2663,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
     }
 
     nsRect relFrameRect = frame->GetRect();
-    frame->GetOffsetFromView(aPresContext, frameOffset, &containingView);
+    frame->GetOffsetFromView(frameOffset, &containingView);
     if (containingView) {
       relFrameRect.x = frameOffset.x;
       relFrameRect.y = frameOffset.y;
@@ -2716,7 +2715,7 @@ nsTypeAheadFind::DisplayStatus(PRBool aSuccess, nsIContent *aFocusedContent,
   // pres shell -> pres context -> container -> tree item ->
   // tree owner -> browser chrome
 
-  nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mFocusedWeakShell));
+  nsCOMPtr<nsIPresShell> presShell(GetPresShell());
   if (!presShell) {
     return;
   }
@@ -2922,7 +2921,7 @@ nsTypeAheadController::DoCommand(const char *aCommand)
   EnsureContentWindow(domWinInternal, getter_AddRefs(startContentWin));
   NS_ENSURE_TRUE(startContentWin, NS_ERROR_FAILURE);
 
-  nsCOMPtr<nsITypeAheadFind> typeAhead = 
+  nsCOMPtr<nsISuiteTypeAheadFind> typeAhead = 
     do_GetService(NS_TYPEAHEADFIND_CONTRACTID);
   NS_ENSURE_TRUE(typeAhead, NS_ERROR_FAILURE);
 
@@ -3005,3 +3004,21 @@ nsTypeAheadController::EnsureContentWindow(nsIDOMWindowInternal *aFocusedWin,
   NS_IF_ADDREF(*aStartContentWin);
   return NS_OK;
 }
+
+already_AddRefed<nsIPresShell>
+nsTypeAheadFind::GetPresShell()
+{
+  if (!mFocusedWeakShell)
+    return nsnull;
+
+  nsIPresShell *shell = nsnull;
+  CallQueryReferent(mFocusedWeakShell.get(), &shell);
+  if (shell) {
+    nsPresContext *pc = shell->GetPresContext();
+    if (!pc || !nsCOMPtr<nsISupports>(pc->GetContainer())) {
+      NS_RELEASE(shell);
+    }
+  }
+
+  return shell;
+}
