@@ -48,15 +48,11 @@
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIContent.h"
 #include "nsIDocument.h"
-#include "nsIDocumentLoader.h"
 #include "nsCURILoader.h"
-#include "nsIWebShell.h"
 #include "nsIDOMHTMLInputElement.h"
 #include "nsIFormControl.h"
 #include "nsIDocShell.h"
-#include "nsIDOMWindowInternal.h"
-#include "nsIInterfaceRequestor.h"
-#include "nsIInterfaceRequestorUtils.h"
+#include "nsPIDOMWindow.h"
 #include "nsIPrompt.h"
 #include "nsIChannel.h"
 #include "nsIWindowWatcher.h"
@@ -65,9 +61,10 @@
 #include "nsUnicharUtils.h"
 #include "nsReadableUtils.h"
 #include "nsICategoryManager.h"
+#include "nsNetUtil.h"
+#include "nsEmbedCID.h"
 
 // for making the leap from nsIDOMWindowInternal -> nsIPresShell
-#include "nsIScriptGlobalObject.h"
 
 static NS_DEFINE_IID(kDocLoaderServiceCID, NS_DOCUMENTLOADER_SERVICE_CID);
 
@@ -88,19 +85,20 @@ nsWalletlibService::~nsWalletlibService()
   SI_ClearUserData();
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS5(nsWalletlibService,
+NS_IMPL_THREADSAFE_ISUPPORTS6(nsWalletlibService,
                               nsIWalletService,
                               nsIObserver,
                               nsIFormSubmitObserver,
                               nsIWebProgressListener,
+                              nsIPromptFactory,
                               nsISupportsWeakReference)
 
-NS_IMETHODIMP nsWalletlibService::WALLET_PreEdit(nsAutoString& walletList) {
+NS_IMETHODIMP nsWalletlibService::WALLET_PreEdit(nsAString& walletList) {
   ::WLLT_PreEdit(walletList);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWalletlibService::WALLET_PostEdit(nsAutoString walletList) {
+NS_IMETHODIMP nsWalletlibService::WALLET_PostEdit(const nsAString & walletList) {
   ::WLLT_PostEdit(walletList);
   return NS_OK;
 }
@@ -120,9 +118,8 @@ nsWalletlibService::WALLET_RequestToCapture(nsIDOMWindowInternal* aWin,
                                             PRUint32* status)
 {
 
-  nsCOMPtr<nsIScriptGlobalObject> scriptGlobalObject;
-  scriptGlobalObject = do_QueryInterface(aWin);
-  nsIDocShell *docShell = scriptGlobalObject->GetDocShell();
+  nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(aWin));
+  nsIDocShell *docShell = window->GetDocShell();
 
   nsCOMPtr<nsIPresShell> presShell;
   if(docShell)
@@ -147,9 +144,8 @@ nsWalletlibService::WALLET_Prefill(PRBool quick,
                                    nsIDOMWindowInternal* aWin,
                                    PRBool* status)
 {
-  nsCOMPtr<nsIScriptGlobalObject> scriptGlobalObject;
-  scriptGlobalObject = do_QueryInterface(aWin);
-  nsIDocShell *docShell = scriptGlobalObject->GetDocShell();
+  nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(aWin));
+  nsIDocShell *docShell = window->GetDocShell();
 
   nsCOMPtr<nsIPresShell> presShell;
   if(docShell)
@@ -158,7 +154,7 @@ nsWalletlibService::WALLET_Prefill(PRBool quick,
   return ::WLLT_Prefill(presShell, quick, aWin);
 }
 
-NS_IMETHODIMP nsWalletlibService::WALLET_PrefillReturn(nsAutoString results){
+NS_IMETHODIMP nsWalletlibService::WALLET_PrefillReturn(const nsAString & results){
   ::WLLT_PrefillReturn(results);
   return NS_OK;
 }
@@ -174,33 +170,23 @@ NS_IMETHODIMP nsWalletlibService::WALLET_InitReencryptCallback(nsIDOMWindowInter
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWalletlibService::SI_RemoveUser(const char *key, const PRUnichar *userName) {
-  ::SINGSIGN_RemoveUser(key, userName, PR_TRUE);
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsWalletlibService::SI_StorePassword(const char *key, const PRUnichar *userName, const PRUnichar *password) {
-  ::SINGSIGN_StorePassword(key, userName, password);
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsWalletlibService::WALLET_GetNopreviewListForViewer(nsAutoString& aNopreviewList){
+NS_IMETHODIMP nsWalletlibService::WALLET_GetNopreviewListForViewer(nsAString& aNopreviewList){
   ::WLLT_GetNopreviewListForViewer(aNopreviewList);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWalletlibService::WALLET_GetNocaptureListForViewer(nsAutoString& aNocaptureList){
+NS_IMETHODIMP nsWalletlibService::WALLET_GetNocaptureListForViewer(nsAString& aNocaptureList){
   ::WLLT_GetNocaptureListForViewer(aNocaptureList);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWalletlibService::WALLET_GetPrefillListForViewer(nsAutoString& aPrefillList){
+NS_IMETHODIMP nsWalletlibService::WALLET_GetPrefillListForViewer(nsAString& aPrefillList){
   ::WLLT_GetPrefillListForViewer(aPrefillList);
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWalletlibService::SI_SignonViewerReturn(nsAutoString results){
-  ::SINGSIGN_SignonViewerReturn(results);
+NS_IMETHODIMP nsWalletlibService::SI_SignonViewerReturn(const nsAString& results){
+  ::Wallet_SignonViewerReturn(results);
   return NS_OK;
 }
 
@@ -220,7 +206,7 @@ NS_IMETHODIMP nsWalletlibService::Observe(nsISupports *aSubject, const char *aTo
     if (uri) {
       nsCAutoString spec;
       if (NS_SUCCEEDED(uri->GetSpec(spec)))
-        SI_StorePassword(spec.get(), nsnull, someData);
+        SINGSIGN_StorePassword(spec.get(), EmptyString().get(), someData);
     }
   }
   else if (!nsCRT::strcmp(aTopic, "login-failed")) {
@@ -229,9 +215,8 @@ NS_IMETHODIMP nsWalletlibService::Observe(nsISupports *aSubject, const char *aTo
     nsCOMPtr<nsIURI> uri = do_QueryInterface(aSubject);
     if (uri) {
       nsCAutoString spec;
-      uri->GetSpec(spec);
       if (NS_SUCCEEDED(uri->GetSpec(spec)))
-        SINGSIGN_RemoveUserAfterLoginFailure(spec.get(), nsnull, PR_TRUE);
+        SINGSIGN_RemoveUserAfterLoginFailure(spec.get(), EmptyString().get(), PR_TRUE);
     }
   }
   return NS_OK;
@@ -243,6 +228,9 @@ NS_IMETHODIMP nsWalletlibService::Notify(nsIContent* formNode, nsIDOMWindowInter
   if (!formNode) {
     return NS_ERROR_FAILURE;
   }
+
+  NS_ENSURE_TRUE(window, NS_OK);
+
   ::WLLT_OnSubmit(formNode, window);
   return NS_OK;
 }
@@ -313,7 +301,7 @@ nsresult nsWalletlibService::Init()
            do_GetService("@mozilla.org/observer-service;1", &rv);
   if (NS_SUCCEEDED(rv) && svc) {
     // Register as an observer of form submission
-    svc->AddObserver(this, NS_FORMSUBMIT_SUBJECT, PR_TRUE);
+    svc->AddObserver(this, NS_EARLYFORMSUBMIT_SUBJECT, PR_TRUE);
     // Register as an observer of profile changes
     svc->AddObserver(this, "profile-before-change", PR_TRUE);
     // Register as an observer for login
@@ -321,19 +309,17 @@ nsresult nsWalletlibService::Init()
     svc->AddObserver(this, "login-failed", PR_TRUE);
   }
   else
-    NS_ASSERTION(PR_FALSE, "Could not get nsIObserverService");
+    NS_ERROR("Could not get nsIObserverService");
 
   // Get the global document loader service...
-  nsCOMPtr<nsIDocumentLoader> docLoaderService =
+  nsCOMPtr<nsIWebProgress> progress =
            do_GetService(kDocLoaderServiceCID, &rv);
-  if (NS_SUCCEEDED(rv) && docLoaderService) {
-    nsCOMPtr<nsIWebProgress> progress(do_QueryInterface(docLoaderService, &rv));
-    if (NS_SUCCEEDED(rv))
-        (void) progress->AddProgressListener((nsIWebProgressListener*)this,
-                                        nsIWebProgress::NOTIFY_STATE_DOCUMENT);
+  if (NS_SUCCEEDED(rv)) {
+    (void) progress->AddProgressListener((nsIWebProgressListener*)this,
+                                    nsIWebProgress::NOTIFY_STATE_DOCUMENT);
   }
   else
-    NS_ASSERTION(PR_FALSE, "Could not get nsIDocumentLoader");
+    NS_ERROR("Could not get nsIWebProgress");
 
   /* initialize the expire-master-password feature */
   nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
@@ -448,14 +434,11 @@ nsWalletlibService::OnStateChange(nsIWebProgress* aWebProgress,
                               if (nameString) {
                                 nsAutoString value;
                                 PRUnichar* valueString = NULL;
-                                nsCOMPtr<nsIInterfaceRequestor> interfaces;
                                 nsCOMPtr<nsIPrompt> prompter;
 
                                 nsCOMPtr<nsIChannel> channel = do_QueryInterface(aRequest);
                                 if (channel)
-                                  channel->GetNotificationCallbacks(getter_AddRefs(interfaces));
-                                if (interfaces)
-                                  interfaces->GetInterface(NS_GET_IID(nsIPrompt), getter_AddRefs(prompter));
+                                  NS_QueryNotificationCallbacks(channel, prompter);
                                 if (!prompter) {
                                   nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
                                   if (wwatch)
@@ -554,6 +537,60 @@ nsWalletlibService::WALLET_Decrypt (const char *crypt, PRUnichar **text) {
   return rv;
 }
 
+NS_IMETHODIMP
+nsWalletlibService::GetPrompt(nsIDOMWindow* aParent,
+                              const nsIID& aIID,
+                              void** _retval) {
+  if (!aIID.Equals(NS_GET_IID(nsIAuthPrompt2))) {
+    NS_WARNING("Wallet asked for unexpected interface");
+    return NS_NOINTERFACE;
+  }
+
+  // NOTE: It is important to return the specific return value here. The
+  // caller cares.
+  nsresult rv;
+  nsCOMPtr<nsIPromptService2> service =
+    do_GetService(NS_PROMPTSERVICE_CONTRACTID, &rv);
+  if (NS_FAILED(rv))
+    return rv;
+
+  nsWalletAuthPromptWrapper* wrapper =
+    new nsWalletAuthPromptWrapper(service, aParent);
+  if (!wrapper)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(wrapper);
+  *_retval = NS_STATIC_CAST(nsIAuthPrompt2*, wrapper);
+  return NS_OK;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// nsWalletAuthPromptWrapper
+
+NS_IMPL_ISUPPORTS1(nsWalletAuthPromptWrapper, nsIAuthPrompt2)
+
+NS_IMETHODIMP
+nsWalletAuthPromptWrapper::PromptAuth(nsIChannel* aChannel,
+                                      PRUint32 aLevel,
+                                      nsIAuthInformation* aAuthInfo,
+                                      PRBool* retval)
+{
+  return SINGSIGN_PromptAuth(mService, mParent, aChannel,
+                             aLevel, aAuthInfo, retval);
+}
+
+NS_IMETHODIMP
+nsWalletAuthPromptWrapper::AsyncPromptAuth(nsIChannel*,
+                                           nsIAuthPromptCallback*,
+                                           nsISupports*,
+                                           PRUint32,
+                                           nsIAuthInformation*,
+                                           nsICancelable**)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+
 
 ////////////////////////////////////////////////////////////////////////////////
 // nsSingleSignOnPrompt
@@ -576,7 +613,7 @@ nsSingleSignOnPrompt::Prompt(const PRUnichar *dialogTitle, const PRUnichar *text
   nsresult rv;
   rv = SINGSIGN_Prompt(
     dialogTitle, text, defaultText, result,
-    NS_ConvertUCS2toUTF8(passwordRealm).get(), mPrompt, _retval, savePassword);
+    NS_ConvertUTF16toUTF8(passwordRealm).get(), mPrompt, _retval, savePassword);
   return rv;
 }
 
@@ -588,7 +625,7 @@ nsSingleSignOnPrompt::PromptUsernameAndPassword(const PRUnichar *dialogTitle, co
   nsresult rv;
   rv = SINGSIGN_PromptUsernameAndPassword(
     dialogTitle, text, user, pwd,
-    NS_ConvertUCS2toUTF8(passwordRealm).get(), mPrompt, _retval, savePassword);
+    NS_ConvertUTF16toUTF8(passwordRealm).get(), mPrompt, _retval, savePassword);
   return rv;
 }
 
@@ -600,7 +637,7 @@ nsSingleSignOnPrompt::PromptPassword(const PRUnichar *dialogTitle, const PRUnich
   nsresult rv;
   rv = SINGSIGN_PromptPassword(
     dialogTitle, text, pwd,
-    NS_ConvertUCS2toUTF8(passwordRealm).get(), mPrompt, _retval, savePassword);
+    NS_ConvertUTF16toUTF8(passwordRealm).get(), mPrompt, _retval, savePassword);
   return rv;
 }
 
