@@ -1,21 +1,24 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 // vim:expandtab:ts=4 sw=4:
-/*
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla.
  *
- * The Initial Developer of the Original Code is Oracle Corporation.
- * Portions created by Oracle Corporation are  Copyright (C) 2004
- * by Oracle Corporation.  All Rights Reserved.
+ * The Initial Developer of the Original Code is
+ * Oracle Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 2004
+ * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *   Mike Shaver <shaver@off.net> (original author)
@@ -34,9 +37,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "nsWebDAVInternal.h"
+
 #include "nsIHttpChannel.h"
 #include "nsIIOService.h"
 #include "nsNetUtil.h"
+#include "nsIURL.h"
 
 #include "nsIDOM3Node.h"
 #include "nsIDOMDocument.h"
@@ -46,12 +52,11 @@
 
 #include "nsIDocument.h"
 #include "nsIDocumentEncoder.h"
+#include "nsContentCID.h"
 
 #include "nsIWebDAVResource.h"
 #include "nsIWebDAVListener.h"
 
-#include "nsWebDAVInternal.h"
-
 #include "nsString.h"
 
 #include "nsISupportsPrimitives.h"
@@ -70,9 +75,10 @@ public:
 
     OperationStreamListener(nsIWebDAVResource *resource,
                             nsIWebDAVOperationListener *listener,
+                            nsISupports *closure,
                             nsIOutputStream *outstream,
                             PRUint32 mode) :
-        mResource(resource), mListener(listener), 
+        mResource(resource), mListener(listener), mClosure(closure), 
         mOutputStream(outstream), mOperation(mode) { }
 
     virtual ~OperationStreamListener() { }
@@ -80,13 +86,14 @@ public:
 protected:
     virtual nsresult SignalCompletion(PRUint32 status)
     {    
-        mListener->OnOperationComplete(status, mResource, mOperation);
+        mListener->OnOperationComplete(status, mResource, mOperation,
+                                       mClosure);
         if (mOutputStream)
             return mOutputStream->Flush();
         return NS_OK;
     }
 
-    virtual void SignalDetail(PRUint32 statusCode, nsACString &resource,
+    virtual void SignalDetail(PRUint32 statusCode, const nsACString &resource,
                               nsISupports *detail);
 
     virtual nsresult ProcessResponse(nsIDOMElement *responseElt);
@@ -97,6 +104,7 @@ protected:
 
     nsCOMPtr<nsIWebDAVResource>          mResource;
     nsCOMPtr<nsIWebDAVOperationListener> mListener;
+    nsCOMPtr<nsISupports>                mClosure;
     nsCOMPtr<nsIOutputStream>            mOutputStream;
     PRUint32                             mOperation;
     nsCString                            mBody;
@@ -115,6 +123,7 @@ NS_IMETHODIMP
 OperationStreamListener::OnStartRequest(nsIRequest *aRequest,
                                         nsISupports *aContext)
 {
+    LOG(("OperationStreamListener::OnStartRequest() entered"));
     mBody.Truncate();
     return NS_OK;
 }
@@ -127,6 +136,8 @@ OperationStreamListener::OnStopRequest(nsIRequest *aRequest,
     PRUint32 status, rv;
     nsCOMPtr<nsIHttpChannel> channel = do_QueryInterface(aContext);
 
+    LOG(("OperationStreamListener::OnStopRequest() entered"));
+
     rv = channel ? channel->GetResponseStatus(&status) : NS_ERROR_UNEXPECTED;
 
     if (NS_FAILED(rv))
@@ -184,6 +195,7 @@ OperationStreamListener::OnDataAvailable(nsIRequest *aRequest,
                                          nsIInputStream *aInputStream,
                                          PRUint32 offset, PRUint32 count)
 {
+    LOG(("OperationStreamListener::OnDataAvailable() entered"));
     PRUint32 result;
     nsCOMPtr<nsIHttpChannel> channel = do_QueryInterface(aContext);
     if (!channel)
@@ -203,10 +215,32 @@ OperationStreamListener::OnDataAvailable(nsIRequest *aRequest,
 
 void
 OperationStreamListener::SignalDetail(PRUint32 statusCode,
-                                      nsACString &resource,
+                                      const nsACString &resource,
                                       nsISupports *detail)
 {
-    mListener->OnOperationDetail(statusCode, resource, mOperation, detail);
+    nsCOMPtr<nsIURL> resourceURL, detailURL;
+    nsCOMPtr<nsIURI> detailURI;
+    if (NS_FAILED(mResource->GetResourceURL(getter_AddRefs(resourceURL))))
+        return;
+    if (resource.IsEmpty()) {
+        detailURL = resourceURL;
+    } else {
+        // if this URL is relative, resolve it.
+        nsresult rv;
+        nsCAutoString resolvedSpec;
+        rv = resourceURL->Resolve(resource, resolvedSpec);
+
+        // XXX better error handling
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Failed to resolve remote URL!");
+
+        if (NS_FAILED(resourceURL->Clone(getter_AddRefs(detailURI))) ||
+            !(detailURL = do_QueryInterface(detailURI)) ||
+            NS_FAILED(detailURI->SetSpec(resolvedSpec))) {
+            return;
+        }
+    }
+    mListener->OnOperationDetail(statusCode, detailURL, mOperation, detail,
+                                 mClosure);
 }
 
 nsresult
@@ -236,6 +270,9 @@ OperationStreamListener::StatusAndHrefFromResponse(nsIDOMElement *responseElt,
     nsAutoString statusString;
     rv = NS_WD_ElementTextChildValue(responseElt, NS_LITERAL_STRING("status"),
                                      statusString);
+    // XXX if we don't find a status element (or hit any other parse error,
+    // for that matter) we need to signal this back to the caller
+    //
     NS_ENSURE_SUCCESS(rv, rv);
 
     PRInt32 res = 0;
@@ -257,8 +294,9 @@ public:
 
     PropfindStreamListener(nsIWebDAVResource *resource,
                            nsIWebDAVOperationListener *listener,
+                           nsISupports *closure,
                            PRBool isPropname) :
-        OperationStreamListener(resource, listener, nsnull,
+        OperationStreamListener(resource, listener, closure, nsnull,
                                 isPropname ?
                                 (PRUint32)nsIWebDAVOperationListener::GET_PROPERTY_NAMES :
                                 (PRUint32)nsIWebDAVOperationListener::GET_PROPERTIES) { }
@@ -333,11 +371,8 @@ PropfindStreamListener::PropertiesFromPropElt(nsIDOMElement *propElt,
         nsCOMPtr<nsIDocumentEncoder> encoder =
           do_CreateInstance(NS_DOC_ENCODER_CONTRACTID_BASE "text/xml", &rv);
 
-        nsCOMPtr<nsIDocument> baseDoc = do_QueryInterface(mXMLDoc, &rv);
-        NS_ENSURE_SUCCESS(rv, rv);
-
         // This method will fail if no document
-        rv = encoder->Init(baseDoc, NS_LITERAL_STRING("text/xml"),
+        rv = encoder->Init(mXMLDoc, NS_LITERAL_STRING("text/xml"),
                            nsIDocumentEncoder::OutputEncodeBasicEntities);
         NS_ENSURE_SUCCESS(rv, rv);
         
@@ -405,22 +440,116 @@ PropfindStreamListener::ProcessResponse(nsIDOMElement *responseElt)
     return NS_OK;
 }
 
+
+class ReportStreamListener : public OperationStreamListener
+{
+public:
+    NS_DECL_ISUPPORTS_INHERITED
+
+    ReportStreamListener(nsIWebDAVResource *resource,
+                         nsIWebDAVOperationListener *listener,
+                         nsISupports *closure) :
+        OperationStreamListener(resource, listener, closure, nsnull,
+                                nsIWebDAVOperationListener::REPORT) { }
+    virtual ~ReportStreamListener() { }
+
+protected:
+    virtual nsresult ProcessResponse(nsIDOMElement *responseElt);
+};
+
+NS_IMPL_ISUPPORTS_INHERITED0(ReportStreamListener, OperationStreamListener)
+
+nsresult
+ReportStreamListener::ProcessResponse(nsIDOMElement *responseElt)
+{
+    nsCAutoString href;
+    PRUint32 statusCode;
+    nsresult rv = StatusAndHrefFromResponse(responseElt, href, &statusCode);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    LOG(("response for %s: %d", href.get(), statusCode));
+
+    SignalDetail(statusCode, href, responseElt);
+    return NS_OK;
+}
+
+
+class GetToStringStreamListener : public OperationStreamListener
+{
+public:
+    NS_DECL_ISUPPORTS_INHERITED
+
+    GetToStringStreamListener(nsIWebDAVResource *resource,
+                              nsIWebDAVOperationListener *listener,
+                              nsISupports *closure) :
+        OperationStreamListener(resource, listener, closure, nsnull,
+                                nsIWebDAVOperationListener::GET_TO_STRING)
+    { }
+
+    virtual ~GetToStringStreamListener() { }
+protected:
+    NS_IMETHOD OnStopRequest(nsIRequest *aRequest,
+                                   nsISupports *aContext,
+                                   nsresult aStatusCode);
+};
+
+NS_IMPL_ISUPPORTS_INHERITED0(GetToStringStreamListener, OperationStreamListener)
+
+NS_IMETHODIMP
+GetToStringStreamListener::OnStopRequest(nsIRequest *aRequest,
+                                         nsISupports *aContext,
+                                         nsresult aStatusCode)
+{
+    PRUint32 status, rv;
+    nsCOMPtr<nsIHttpChannel> channel = do_QueryInterface(aContext);
+
+    LOG(("OperationStreamListener::OnStopRequest() entered"));
+
+    rv = channel ? channel->GetResponseStatus(&status) : NS_ERROR_UNEXPECTED;
+
+    if (NS_FAILED(rv))
+        return SignalCompletion(rv);
+
+    if (status != 200)
+        return SignalCompletion(status);
+
+    nsCOMPtr<nsISupportsCString>
+        suppString(do_CreateInstance("@mozilla.org/supports-cstring;1",
+                                     &rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+    suppString->SetData(mBody);
+
+    SignalDetail(status, nsCAutoString(""), suppString);
+    SignalCompletion(status);
+    return NS_OK;
+}
+
 nsIStreamListener *
 NS_WD_NewPropfindStreamListener(nsIWebDAVResource *resource,
                                 nsIWebDAVOperationListener *listener,
+                                nsISupports *closure,
                                 PRBool isPropname)
 {
-    return new PropfindStreamListener(resource, listener, isPropname);
+    return new PropfindStreamListener(resource, listener, closure, isPropname);
+}
+
+nsIStreamListener *
+NS_WD_NewReportStreamListener(nsIWebDAVResource *resource,
+                              nsIWebDAVOperationListener *listener,
+                              nsISupports *closure)
+{
+    return new ReportStreamListener(resource, listener, closure);
 }
 
 nsresult
 NS_WD_NewOperationStreamListener(nsIWebDAVResource *resource,
                                  nsIWebDAVOperationListener *listener,
+                                 nsISupports *closure,
                                  PRUint32 operation,
                                  nsIStreamListener **streamListener)
 {
     nsCOMPtr<nsIRequestObserver> osl = 
-        new OperationStreamListener(resource, listener, nsnull,
+        new OperationStreamListener(resource, listener, closure, nsnull,
                                     operation);
     if (!osl)
         return NS_ERROR_OUT_OF_MEMORY;
@@ -430,13 +559,28 @@ NS_WD_NewOperationStreamListener(nsIWebDAVResource *resource,
 nsresult
 NS_WD_NewGetOperationRequestObserver(nsIWebDAVResource *resource,
                                      nsIWebDAVOperationListener *listener,
+                                     nsISupports *closure,
                                      nsIOutputStream *outstream,
                                      nsIRequestObserver **observer)
 {
     nsCOMPtr<nsIRequestObserver> osl = 
-        new OperationStreamListener(resource, listener, outstream,
+        new OperationStreamListener(resource, listener, closure, outstream,
                                     nsIWebDAVOperationListener::GET);
     if (!osl)
         return NS_ERROR_OUT_OF_MEMORY;
     return CallQueryInterface(osl, observer);
 }
+
+nsresult
+NS_WD_NewGetToStringOperationRequestObserver(nsIWebDAVResource *resource,
+                                             nsIWebDAVOperationListener *listener,
+                                             nsISupports *closure,
+                                             nsIStreamListener **streamListener)
+{
+    nsCOMPtr<nsIRequestObserver> osl = 
+        new GetToStringStreamListener(resource, listener, closure);
+    if (!osl)
+        return NS_ERROR_OUT_OF_MEMORY;
+    return CallQueryInterface(osl, streamListener);
+}
+
