@@ -1,24 +1,28 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 // vim:expandtab:ts=4 sw=4:
-/*
- * The contents of this file are subject to the Mozilla Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla.
  *
- * The Initial Developer of the Original Code is Oracle Corporation.
- * Portions created by Oracle Corporation are  Copyright (C) 2004
- * by Oracle Corporation.  All Rights Reserved.
+ * The Initial Developer of the Original Code is
+ * Oracle Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 2004
+ * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *   Mike Shaver <shaver@off.net> (original author)
+ *   Gary van der Merwe <garyvdm@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -34,22 +38,20 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#if defined(MOZ_LOGGING)
-#define FORCE_PR_LOG
-#endif
-
 #include "nsWebDAVInternal.h"
 
 #include "nsIWebDAVService.h"
 #include "nsWebDAVServiceCID.h"
 
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
+#include "nsIClassInfoImpl.h"
 
 #include "nsIHttpChannel.h"
 #include "nsIIOService.h"
 #include "nsNetUtil.h"
 #include "nsIStorageStream.h"
 #include "nsIUploadChannel.h"
+#include "nsIURL.h"
 
 #include "nsContentCID.h"
 
@@ -62,6 +64,7 @@
 
 #include "nsIDocument.h"
 #include "nsIDocumentEncoder.h"
+#include "nsContentCID.h"
 
 #include "nsIDOMParser.h"
 
@@ -78,6 +81,7 @@ public:
 protected:
     nsresult EnsureIOService();
     nsresult ChannelFromResource(nsIWebDAVResource *resource,
+                                 nsIInterfaceRequestor *notificationCallbacks,
                                  nsIHttpChannel** channel,
                                  nsIURI ** resourceURI = 0);
 
@@ -88,12 +92,13 @@ protected:
     nsresult PropfindInternal(nsIWebDAVResource *resource, PRUint32 propCount,
                               const char **properties, PRBool withDepth,
                               nsIWebDAVOperationListener *listener,
-                              PRBool namesOnly);
+                              nsIInterfaceRequestor *notificationCallbacks,
+                              nsISupports *closure, PRBool namesOnly);
 
-    nsresult SendPropfindDocumentToChannel(nsIDocument *doc,
-                                           nsIHttpChannel *channel,
-                                           nsIStreamListener *listener,
-                                           PRBool withDepth);
+    nsresult SendDocumentToChannel(nsIDocument *doc, nsIHttpChannel *channel,
+                                   const char *const method,
+                                   nsIStreamListener *listener,
+                                   PRBool withDepth);
     nsCOMPtr<nsIIOService> mIOService; // XXX weak?
     nsAutoString mDAVNSString; // "DAV:"
 };
@@ -120,16 +125,17 @@ nsWebDAVService::EnsureIOService()
 }
 
 nsresult
-nsWebDAVService::SendPropfindDocumentToChannel(nsIDocument *doc,
-                                               nsIHttpChannel *channel,
-                                               nsIStreamListener *listener,
-                                               PRBool withDepth)
+nsWebDAVService::SendDocumentToChannel(nsIDocument *doc,
+                                       nsIHttpChannel *channel, 
+                                       const char *const method,
+                                       nsIStreamListener *listener,
+                                       PRBool withDepth)
 {
     nsCOMPtr<nsIStorageStream> storageStream;
     // Why do I have to pick values for these?  I just want to store some data
     // for stream access!  (And how would script set these?)
-    nsresult rv = NS_NewStorageStream(4 * 1024, 256 * 1024,
-                             getter_AddRefs(storageStream));
+    nsresult rv = NS_NewStorageStream(4096, PR_UINT32_MAX,
+                                      getter_AddRefs(storageStream));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIOutputStream> storageOutputStream;
@@ -141,7 +147,10 @@ nsWebDAVService::SendPropfindDocumentToChannel(nsIDocument *doc,
         do_CreateInstance(NS_DOC_ENCODER_CONTRACTID_BASE "text/xml", &rv);
     NS_ENSURE_SUCCESS(rv, rv);
     
-    rv = encoder->Init(doc, NS_LITERAL_STRING("text/xml"),
+    nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(doc);
+    NS_ASSERTION(domDoc, "Need a document");
+
+    rv = encoder->Init(domDoc, NS_LITERAL_STRING("text/xml"),
                        nsIDocumentEncoder::OutputEncodeBasicEntities);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -152,6 +161,7 @@ nsWebDAVService::SendPropfindDocumentToChannel(nsIDocument *doc,
     storageOutputStream->Close();
 
     // You gotta really want it.
+#ifdef PR_LOGGING
     if (PR_LOG_TEST(gDAVLog, 5)) {
         nsCOMPtr<nsIInputStream> logInputStream;
         rv = storageStream->NewInputStream(0, getter_AddRefs(logInputStream));
@@ -168,6 +178,7 @@ nsWebDAVService::SendPropfindDocumentToChannel(nsIDocument *doc,
         
         delete [] buf;
     }
+#endif
 
     nsCOMPtr<nsIInputStream> inputStream;
     rv = storageStream->NewInputStream(0, getter_AddRefs(inputStream));
@@ -179,7 +190,7 @@ nsWebDAVService::SendPropfindDocumentToChannel(nsIDocument *doc,
                                         NS_LITERAL_CSTRING("text/xml"), -1);
     NS_ENSURE_SUCCESS(rv, rv);
     
-    channel->SetRequestMethod(NS_LITERAL_CSTRING("PROPFIND"));
+    channel->SetRequestMethod(nsDependentCString(method));
 
     if (withDepth) {
         channel->SetRequestHeader(NS_LITERAL_CSTRING("Depth"),
@@ -189,13 +200,15 @@ nsWebDAVService::SendPropfindDocumentToChannel(nsIDocument *doc,
                                   NS_LITERAL_CSTRING("0"), PR_FALSE);
     }
 
+#ifdef PR_LOGGING
     if (LOG_ENABLED()) {
         nsCOMPtr<nsIURI> uri;
         channel->GetURI(getter_AddRefs(uri));
         nsCAutoString spec;
         uri->GetSpec(spec);
-        LOG(("PROPFIND starting for %s", spec.get()));
+        LOG(("%s starting for %s", method, spec.get()));
     }
+#endif
 
     return channel->AsyncOpen(listener, channel);
 }
@@ -214,17 +227,21 @@ nsWebDAVService::CreatePropfindDocument(nsIURI *resourceURI,
 
     nsCOMPtr<nsIPrivateDOMImplementation> 
         privImpl(do_QueryInterface(implementation));
-    privImpl->Init(resourceURI);
+    // XXXbz I doubt this is right, but I have no idea what this code is doing
+    // or why it's creating documents without a useful principal... so I'm just
+    // going to make the fact that those documents have no principal very
+    // explicit, and if this causes issues then someone familiar with this code
+    // should figure out what principals this _should_ be using.
+    privImpl->Init(resourceURI, resourceURI, nsnull);
 
     nsCOMPtr<nsIDOMDocument> doc;
-    nsAutoString emptyString;
-    rv = implementation->CreateDocument(mDAVNSString, emptyString, nsnull,
+    rv = implementation->CreateDocument(mDAVNSString, EmptyString(), nsnull,
                                         getter_AddRefs(doc));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIDocument> baseDoc = do_QueryInterface(doc);
-    baseDoc->SetXMLDeclaration(NS_LITERAL_STRING("1.0"), emptyString,
-                               emptyString);
+    baseDoc->SetXMLDeclaration(NS_LITERAL_STRING("1.0").get(),
+                               EmptyString().get(), -1);
     baseDoc->SetDocumentURI(resourceURI);
 
     nsCOMPtr<nsIDOMElement> elt;
@@ -243,42 +260,52 @@ nsWebDAVService::CreatePropfindDocument(nsIURI *resourceURI,
 }
 
 nsresult
-nsWebDAVService::ChannelFromResource(nsIWebDAVResource *resource,
-                                     nsIHttpChannel **channel,
-                                     nsIURI **resourceURI)
+nsWebDAVService::ChannelFromResource(nsIWebDAVResource *aResource,
+                                     nsIInterfaceRequestor *notificationCallbacks,
+                                     nsIHttpChannel **aChannel,
+                                     nsIURI **aResourceURI)
 {
     ENSURE_IO_SERVICE();
 
-    nsCAutoString spec;
+    nsCOMPtr<nsIURL> resourceURL;
 
-    nsresult rv = resource->GetUrlSpec(spec);
-    NS_ENSURE_SUCCESS(rv, rv);
-    if (spec.IsEmpty()) {
-        NS_ASSERTION(0, "non-empty spec!");
-        return NS_ERROR_INVALID_ARG;
-    }
-
-    nsCOMPtr<nsIURI> uri;
-    rv = mIOService->NewURI(spec, nsnull, nsnull, getter_AddRefs(uri));
+    nsresult rv = aResource->GetResourceURL(getter_AddRefs(resourceURL));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIChannel> baseChannel;
-    rv = mIOService->NewChannelFromURI(uri, getter_AddRefs(baseChannel));
+    rv = mIOService->NewChannelFromURI(resourceURL, getter_AddRefs(baseChannel));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (resourceURI) {
-        *resourceURI = uri.get();
-        NS_ADDREF(*resourceURI);
+    rv = baseChannel->SetNotificationCallbacks(notificationCallbacks);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsLoadFlags loadFlags;
+    rv = baseChannel->GetLoadFlags(&loadFlags);
+    if (NS_SUCCEEDED(rv)) {
+        rv = baseChannel->SetLoadFlags(loadFlags | 
+                                       nsIRequest::VALIDATE_ALWAYS);
+        NS_ASSERTION(NS_SUCCEEDED(rv),
+                     "nsWebDavService::ChannelFromResource(): "
+                     "Couldn't set loadflags on channel");
     }
 
-    return CallQueryInterface(baseChannel, channel);
+    rv = CallQueryInterface(baseChannel, aChannel);
+
+    if (NS_SUCCEEDED(rv) && aResourceURI) {
+        *aResourceURI = resourceURL.get();
+        NS_ADDREF(*aResourceURI);
+    }
+
+    return rv;
 }
 
 nsWebDAVService::nsWebDAVService() :
     mDAVNSString(NS_LITERAL_STRING("DAV:"))
 
 {
+#ifdef PR_LOGGING
     gDAVLog = PR_NewLogModule("webdav");
+#endif
 }
 
 nsWebDAVService::~nsWebDAVService()
@@ -288,14 +315,18 @@ nsWebDAVService::~nsWebDAVService()
 
 NS_IMETHODIMP
 nsWebDAVService::LockResources(PRUint32 count, nsIWebDAVResource **resources,
-                               nsIWebDAVOperationListener *listener)
+                               nsIWebDAVOperationListener *listener,
+                               nsIInterfaceRequestor *notificationCallbacks,
+                               nsISupports *closure)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
 nsWebDAVService::UnlockResources(PRUint32 count, nsIWebDAVResource **resources,
-                                 nsIWebDAVOperationListener *listener)
+                                 nsIWebDAVOperationListener *listener,
+                                 nsIInterfaceRequestor *notificationCallbacks,
+                                 nsISupports *closure)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -303,10 +334,12 @@ nsWebDAVService::UnlockResources(PRUint32 count, nsIWebDAVResource **resources,
 NS_IMETHODIMP
 nsWebDAVService::GetResourcePropertyNames(nsIWebDAVResource *resource,
                                           PRBool withDepth,
-                                          nsIWebDAVOperationListener *listener)
+                                          nsIWebDAVOperationListener *listener,
+                                          nsIInterfaceRequestor *notificationCallbacks,
+                                          nsISupports *closure)
 {
     return PropfindInternal(resource, 0, nsnull, withDepth,
-                            listener, PR_TRUE);
+                            listener, notificationCallbacks, closure, PR_TRUE);
 }
 
 NS_IMETHODIMP
@@ -314,10 +347,12 @@ nsWebDAVService::GetResourceProperties(nsIWebDAVResource *resource,
                                        PRUint32 propCount,
                                        const char **properties,
                                        PRBool withDepth,
-                                       nsIWebDAVOperationListener *listener)
+                                       nsIWebDAVOperationListener *listener,
+                                       nsIInterfaceRequestor *notificationCallbacks,
+                                       nsISupports *closure)
 {
     return PropfindInternal(resource, propCount, properties, withDepth,
-                            listener, PR_FALSE);
+                            listener, notificationCallbacks, closure, PR_FALSE);
 }
 
 nsresult
@@ -326,6 +361,8 @@ nsWebDAVService::PropfindInternal(nsIWebDAVResource *resource,
                                   const char **properties,
                                   PRBool withDepth,
                                   nsIWebDAVOperationListener *listener,
+                                  nsIInterfaceRequestor *notificationCallbacks,
+                                  nsISupports *closure,
                                   PRBool namesOnly)
 {
     nsresult rv;
@@ -335,7 +372,7 @@ nsWebDAVService::PropfindInternal(nsIWebDAVResource *resource,
 
     nsCOMPtr<nsIURI> resourceURI;
     nsCOMPtr<nsIHttpChannel> channel;
-    rv = ChannelFromResource(resource, getter_AddRefs(channel),
+    rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel),
                              getter_AddRefs(resourceURI));
     if (NS_FAILED(rv))
         return rv;
@@ -379,12 +416,13 @@ nsWebDAVService::PropfindInternal(nsIWebDAVResource *resource,
             RFindInReadable(NS_LITERAL_CSTRING(" "), start, end);
 
             if (start == end) {
-                nsCAutoString msg(NS_LITERAL_CSTRING("Illegal property name ") +
-                                  fullpropName);
+                nsCAutoString msg(NS_LITERAL_CSTRING("Illegal property name ")
+                                  + fullpropName + NS_LITERAL_CSTRING("\n"));
                 NS_WARNING(msg.get());
                 return NS_ERROR_INVALID_ARG;
             }
 
+#ifdef PR_LOGGING
             if (LOG_ENABLED()) {
                 nsACString::const_iterator s = start;
                 
@@ -393,6 +431,7 @@ nsWebDAVService::PropfindInternal(nsIWebDAVResource *resource,
                 
                 LOG(("prop ns: '%s', name: '%s'", propNamespace.get(), propName.get()));
             }
+#endif
 
             NS_ConvertASCIItoUTF16 propNamespace(nsDependentCSubstring(saveStart, start)),
                 propName(nsDependentCSubstring(++start, saveEnd));
@@ -405,29 +444,33 @@ nsWebDAVService::PropfindInternal(nsIWebDAVResource *resource,
     }
 
     nsCOMPtr<nsIStreamListener> streamListener = 
-        NS_WD_NewPropfindStreamListener(resource, listener, namesOnly);
+        NS_WD_NewPropfindStreamListener(resource, listener, closure,
+                                        namesOnly);
 
     if (!streamListener)
         return NS_ERROR_OUT_OF_MEMORY;
     
     nsCOMPtr<nsIDocument> requestBaseDoc = do_QueryInterface(requestDoc);
-    return SendPropfindDocumentToChannel(requestBaseDoc, channel,
-                                         streamListener, withDepth);
+    return SendDocumentToChannel(requestBaseDoc, channel, "PROPFIND",
+                                 streamListener, withDepth);
 }
 
 NS_IMETHODIMP
 nsWebDAVService::GetResourceOptions(nsIWebDAVResource *resource,
-                                    nsIWebDAVOperationListener *listener)
+                                    nsIWebDAVOperationListener *listener,
+                                    nsIInterfaceRequestor *notificationCallbacks,
+                                    nsISupports *closure)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsWebDAVService::Get(nsIWebDAVResource *resource, nsIStreamListener *listener)
+nsWebDAVService::Get(nsIWebDAVResource *resource, nsIStreamListener *listener,
+                     nsIInterfaceRequestor *notificationCallbacks)
 {
     nsresult rv;
     nsCOMPtr<nsIHttpChannel> channel;
-    rv = ChannelFromResource(resource, getter_AddRefs(channel));
+    rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel));
     if (NS_FAILED(rv))
         return rv;
 
@@ -437,12 +480,15 @@ nsWebDAVService::Get(nsIWebDAVResource *resource, nsIStreamListener *listener)
 NS_IMETHODIMP
 nsWebDAVService::GetToOutputStream(nsIWebDAVResource *resource,
                                    nsIOutputStream *stream,
-                                   nsIWebDAVOperationListener *listener)
+                                   nsIWebDAVOperationListener *listener,
+                                   nsIInterfaceRequestor *notificationCallbacks,
+                                   nsISupports *closure)
 {
     nsCOMPtr<nsIRequestObserver> getObserver;
     nsresult rv;
 
-    rv = NS_WD_NewGetOperationRequestObserver(resource, listener, stream,
+    rv = NS_WD_NewGetOperationRequestObserver(resource, listener, closure, 
+                                              stream,
                                               getter_AddRefs(getObserver));
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -451,17 +497,40 @@ nsWebDAVService::GetToOutputStream(nsIWebDAVResource *resource,
                                     stream, getObserver);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    return Get(resource, streamListener);
+    return Get(resource, streamListener, notificationCallbacks);
+}
+
+NS_IMETHODIMP
+nsWebDAVService::GetToString(nsIWebDAVResource *resource,
+                             nsIWebDAVOperationListener *listener,
+                             nsIInterfaceRequestor *notificationCallbacks,
+                             nsISupports *closure)
+{
+    nsCOMPtr<nsIStreamListener> getListener;
+    nsresult rv;
+
+    rv = NS_WD_NewGetToStringOperationRequestObserver(resource, listener,
+                                                      closure,
+                                                      getter_AddRefs(getListener));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsIHttpChannel> channel;
+    rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    return channel->AsyncOpen(getListener, channel);
 }
 
 NS_IMETHODIMP
 nsWebDAVService::Put(nsIWebDAVResource *resource,
                      const nsACString& contentType, nsIInputStream *data,
-                     nsIWebDAVOperationListener *listener)
+                     nsIWebDAVOperationListener *listener,
+                     nsIInterfaceRequestor *notificationCallbacks,
+                     nsISupports *closure)
 {
     nsCOMPtr<nsIHttpChannel> channel;
 
-    nsresult rv = ChannelFromResource(resource, getter_AddRefs(channel));
+    nsresult rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel));
     NS_ENSURE_SUCCESS(rv, rv);
     
     nsCOMPtr<nsIUploadChannel> uploadChannel = do_QueryInterface(channel, &rv);
@@ -471,13 +540,14 @@ nsWebDAVService::Put(nsIWebDAVResource *resource,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIStreamListener> streamListener;
-    rv = NS_WD_NewOperationStreamListener(resource, listener,
+    rv = NS_WD_NewOperationStreamListener(resource, listener, closure,
                                           nsIWebDAVOperationListener::PUT,
                                           getter_AddRefs(streamListener));
     NS_ENSURE_SUCCESS(rv, rv);
 
     channel->SetRequestMethod(NS_LITERAL_CSTRING("PUT"));
 
+#ifdef PR_LOGGING
     if (LOG_ENABLED()) {
         nsCOMPtr<nsIURI> uri;
         channel->GetURI(getter_AddRefs(uri));
@@ -485,26 +555,49 @@ nsWebDAVService::Put(nsIWebDAVResource *resource,
         uri->GetSpec(spec);
         LOG(("PUT starting for %s", spec.get()));
     }
+#endif
 
     return channel->AsyncOpen(streamListener, channel);
 }
 
+NS_IMETHODIMP
+nsWebDAVService::PutFromString(nsIWebDAVResource *resource,
+                               const nsACString& contentType,
+                               const nsACString& data,
+                               nsIWebDAVOperationListener *listener,
+                               nsIInterfaceRequestor *notificationCallbacks,
+                               nsISupports *closure)
+{
+    nsresult rv;
+    nsCOMPtr<nsIStringInputStream> dataStream = 
+        do_CreateInstance("@mozilla.org/io/string-input-stream;1", &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    dataStream->SetData(PromiseFlatCString(data).get(),
+                        data.Length());
+    return Put(resource, contentType, dataStream, listener, 
+               notificationCallbacks, closure);
+}
+
 NS_IMETHODIMP
 nsWebDAVService::Remove(nsIWebDAVResource *resource,
-                        nsIWebDAVOperationListener *listener)
+                        nsIWebDAVOperationListener *listener,
+                        nsIInterfaceRequestor *notificationCallbacks,
+                        nsISupports *closure)
 {
     nsCOMPtr<nsIHttpChannel> channel;
-    nsresult rv = ChannelFromResource(resource, getter_AddRefs(channel));
+    nsresult rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIStreamListener> streamListener;
-    rv = NS_WD_NewOperationStreamListener(resource, listener,
+    rv = NS_WD_NewOperationStreamListener(resource, listener, closure,
                                           nsIWebDAVOperationListener::REMOVE,
                                           getter_AddRefs(streamListener));
     NS_ENSURE_SUCCESS(rv, rv);
 
     channel->SetRequestMethod(NS_LITERAL_CSTRING("DELETE"));
 
+#ifdef PR_LOGGING
     if (LOG_ENABLED()) {
         nsCOMPtr<nsIURI> uri;
         channel->GetURI(getter_AddRefs(uri));
@@ -512,26 +605,30 @@ nsWebDAVService::Remove(nsIWebDAVResource *resource,
         uri->GetSpec(spec);
         LOG(("DELETE starting for %s", spec.get()));
     }
+#endif
 
     return channel->AsyncOpen(streamListener, channel);
 }
 
 NS_IMETHODIMP
 nsWebDAVService::MakeCollection(nsIWebDAVResource *resource,
-                                nsIWebDAVOperationListener *listener)
+                                nsIWebDAVOperationListener *listener,
+                                nsIInterfaceRequestor *notificationCallbacks,
+                                nsISupports *closure)
 {
     nsCOMPtr<nsIHttpChannel> channel;
-    nsresult rv = ChannelFromResource(resource, getter_AddRefs(channel));
+    nsresult rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIStreamListener> streamListener;
-    rv = NS_WD_NewOperationStreamListener(resource, listener,
+    rv = NS_WD_NewOperationStreamListener(resource, listener, closure,
                                           nsIWebDAVOperationListener::MAKE_COLLECTION,
                                           getter_AddRefs(streamListener));
     NS_ENSURE_SUCCESS(rv, rv);
 
     channel->SetRequestMethod(NS_LITERAL_CSTRING("MKCOL"));
 
+#ifdef PR_LOGGING
     if (LOG_ENABLED()) {
         nsCOMPtr<nsIURI> uri;
         channel->GetURI(getter_AddRefs(uri));
@@ -539,6 +636,7 @@ nsWebDAVService::MakeCollection(nsIWebDAVResource *resource,
         uri->GetSpec(spec);
         LOG(("MKCOL starting for %s", spec.get()));
     }
+#endif
 
     return channel->AsyncOpen(streamListener, channel);
 }
@@ -547,14 +645,16 @@ NS_IMETHODIMP
 nsWebDAVService::MoveTo(nsIWebDAVResource *resource,
                         const nsACString &destination,
                         PRBool overwrite,
-                        nsIWebDAVOperationListener *listener)
+                        nsIWebDAVOperationListener *listener,
+                        nsIInterfaceRequestor *notificationCallbacks,
+                        nsISupports *closure)
 {
     nsCOMPtr<nsIHttpChannel> channel;
-    nsresult rv = ChannelFromResource(resource, getter_AddRefs(channel));
+    nsresult rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIStreamListener> streamListener;
-    rv = NS_WD_NewOperationStreamListener(resource, listener,
+    rv = NS_WD_NewOperationStreamListener(resource, listener, closure,
                                           nsIWebDAVOperationListener::COPY,
                                           getter_AddRefs(streamListener));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -574,6 +674,7 @@ nsWebDAVService::MoveTo(nsIWebDAVResource *resource,
     channel->SetRequestHeader(NS_LITERAL_CSTRING("Destination"),
                               destination, PR_FALSE);
 
+#ifdef PR_LOGGING
     if (LOG_ENABLED()) {
         nsCOMPtr<nsIURI> uri;
         channel->GetURI(getter_AddRefs(uri));
@@ -582,6 +683,7 @@ nsWebDAVService::MoveTo(nsIWebDAVResource *resource,
         LOG(("MOVE starting for %s -> %s", spec.get(),
              nsCAutoString(destination).get()));
     }
+#endif
 
     return channel->AsyncOpen(streamListener, channel);
 }
@@ -590,15 +692,17 @@ NS_IMETHODIMP
 nsWebDAVService::CopyTo(nsIWebDAVResource *resource,
                         const nsACString &destination,
                         PRBool recursive, PRBool overwrite,
-                        nsIWebDAVOperationListener *listener)
+                        nsIWebDAVOperationListener *listener,
+                        nsIInterfaceRequestor *notificationCallbacks,
+                        nsISupports *closure)
 
 {
     nsCOMPtr<nsIHttpChannel> channel;
-    nsresult rv = ChannelFromResource(resource, getter_AddRefs(channel));
+    nsresult rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIStreamListener> streamListener;
-    rv = NS_WD_NewOperationStreamListener(resource, listener,
+    rv = NS_WD_NewOperationStreamListener(resource, listener, closure,
                                           nsIWebDAVOperationListener::COPY,
                                           getter_AddRefs(streamListener));
     NS_ENSURE_SUCCESS(rv, rv);
@@ -625,6 +729,7 @@ nsWebDAVService::CopyTo(nsIWebDAVResource *resource,
     channel->SetRequestHeader(NS_LITERAL_CSTRING("Destination"),
                               destination, PR_FALSE);
 
+#ifdef PR_LOGGING
     if (LOG_ENABLED()) {
         nsCOMPtr<nsIURI> uri;
         channel->GetURI(getter_AddRefs(uri));
@@ -633,10 +738,44 @@ nsWebDAVService::CopyTo(nsIWebDAVResource *resource,
         LOG(("COPY starting for %s -> %s", spec.get(),
              nsCAutoString(destination).get()));
     }
+#endif
 
     return channel->AsyncOpen(streamListener, channel);
 }
 
+
+NS_IMETHODIMP
+nsWebDAVService::Report(nsIWebDAVResource *resource, nsIDOMDocument *query, 
+                        PRBool withDepth, 
+                        nsIWebDAVOperationListener *listener, 
+                        nsIInterfaceRequestor *notificationCallbacks,
+                        nsISupports *closure)
+{
+    nsresult rv;
+
+    NS_ENSURE_ARG(resource);
+    NS_ENSURE_ARG(query);
+    NS_ENSURE_ARG(listener);
+
+    nsCOMPtr<nsIDocument> queryDoc = do_QueryInterface(query, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    nsCOMPtr<nsIURI> resourceURI;
+    nsCOMPtr<nsIHttpChannel> channel;
+    rv = ChannelFromResource(resource, notificationCallbacks, getter_AddRefs(channel),
+                             getter_AddRefs(resourceURI));
+    if (NS_FAILED(rv))
+        return rv;
+
+    nsCOMPtr<nsIStreamListener> streamListener = 
+        NS_WD_NewReportStreamListener(resource, listener, closure);
+    if (!streamListener)
+        return NS_ERROR_OUT_OF_MEMORY;
+
+    return SendDocumentToChannel(queryDoc, channel, "REPORT", streamListener,
+                                 withDepth);
+}
+
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsWebDAVService)
 
 NS_DECL_CLASSINFO(nsWebDAVService)
