@@ -81,18 +81,16 @@ static PRInt32 GetXftDPI(void);
 
 #include <X11/Xatom.h>
 
-#include "nsDeviceContextSpecG.h"
+#include "nsIDeviceContextSpec.h"
 
 static PRInt32 GetOSDPI(void);
 
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-
 #define GDK_DEFAULT_FONT1 "-*-helvetica-medium-r-*--*-120-*-*-*-*-iso8859-1"
 #define GDK_DEFAULT_FONT2 "-*-fixed-medium-r-*-*-*-120-*-*-*-*-*-*"
 
 #ifdef MOZ_WIDGET_GTK
 // this is specific to gtk 1.2
-extern GdkFont *default_font;
+extern NS_IMPORT_(GdkFont *) default_font;
 #endif /* MOZ_WIDGET_GTK */
 
 /**
@@ -147,19 +145,15 @@ nsDeviceContextGTK::nsDeviceContextGTK()
   mDepth = 0 ;
   mNumCells = 0;
 
-  mWidthFloat = 0.0f;
-  mHeightFloat = 0.0f;
-  mWidth = -1;
-  mHeight = -1;
   mDeviceWindow = nsnull;
 }
 
 nsDeviceContextGTK::~nsDeviceContextGTK()
 {
   nsresult rv;
-  nsCOMPtr<nsIPref> prefs = do_GetService(kPrefCID, &rv);
+  nsCOMPtr<nsIPref> prefs = do_GetService(NS_PREF_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv)) {
-    prefs->UnregisterCallback("browser.display.screen_resolution",
+    prefs->UnregisterCallback("layout.css.dpi",
                               prefChanged, (void *)this);
   }
 }
@@ -220,11 +214,8 @@ NS_IMETHODIMP nsDeviceContextGTK::Init(nsNativeWidget aNativeWidget)
   nsCOMPtr<nsIScreen> screen;
   mScreenManager->GetPrimaryScreen ( getter_AddRefs(screen) );
   if ( screen ) {
-    PRInt32 x, y, width, height, depth;
-    screen->GetRect ( &x, &y, &width, &height );
+    PRInt32 depth;
     screen->GetPixelDepth ( &depth );
-    mWidthFloat = float(width);
-    mHeightFloat = float(height);
     mDepth = NS_STATIC_CAST ( PRUint32, depth );
   }
     
@@ -234,7 +225,7 @@ NS_IMETHODIMP nsDeviceContextGTK::Init(nsNativeWidget aNativeWidget)
     initialized = 1;
 
     // Set prefVal the value of the preference
-    // "browser.display.screen_resolution"
+    // "layout.css.dpi"
     // or -1 if we can't get it.
     // If it's negative, we pretend it's not set.
     // If it's 0, it means force use of the operating system's logical
@@ -242,13 +233,13 @@ NS_IMETHODIMP nsDeviceContextGTK::Init(nsNativeWidget aNativeWidget)
     // If it's positive, we use it as the logical resolution
     nsresult res;
 
-    nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res));
+    nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &res));
     if (NS_SUCCEEDED(res) && prefs) {
-      res = prefs->GetIntPref("browser.display.screen_resolution", &prefVal);
+      res = prefs->GetIntPref("layout.css.dpi", &prefVal);
       if (NS_FAILED(res)) {
         prefVal = -1;
       }
-      prefs->RegisterCallback("browser.display.screen_resolution", prefChanged,
+      prefs->RegisterCallback("layout.css.dpi", prefChanged,
                               (void *)this);
     }
 
@@ -303,22 +294,18 @@ NS_IMETHODIMP nsDeviceContextGTK::CreateRenderingContext(nsIRenderingContext *&a
   }
 #endif
 
-  nsIRenderingContext *pContext;
   nsresult             rv;
-  nsDrawingSurfaceGTK  *surf;
-  GtkWidget *w;
-
-  w = (GtkWidget*)mWidget;
+  GtkWidget *w = (GtkWidget*)mWidget;
 
   // to call init for this, we need to have a valid nsDrawingSurfaceGTK created
-  pContext = new nsRenderingContextGTK();
+  nsIRenderingContext* pContext = new nsRenderingContextGTK();
 
   if (nsnull != pContext)
   {
     NS_ADDREF(pContext);
 
     // create the nsDrawingSurfaceGTK
-    surf = new nsDrawingSurfaceGTK();
+    nsDrawingSurfaceGTK* surf = new nsDrawingSurfaceGTK();
 
     if (surf && w)
       {
@@ -456,20 +443,21 @@ NS_IMETHODIMP nsDeviceContextGTK::GetDeviceSurfaceDimensions(PRInt32 &aWidth, PR
   }
 #endif
 
-  if (mWidth == -1)
-    mWidth = NSToIntRound(mWidthFloat * mDevUnitsToAppUnits);
+  PRInt32 width = 0, height = 0;
 
-  if (mHeight == -1)
-    mHeight = NSToIntRound(mHeightFloat * mDevUnitsToAppUnits);
+  nsCOMPtr<nsIScreen> screen;
+  mScreenManager->GetPrimaryScreen(getter_AddRefs(screen));
+  if (screen) {
+    PRInt32 x, y;
+    screen->GetRect(&x, &y, &width, &height);
+  }
 
-  aWidth = mWidth;
-  aHeight = mHeight;
+  aWidth = NSToIntRound(float(width) * mDevUnitsToAppUnits);
+  aHeight = NSToIntRound(float(height) * mDevUnitsToAppUnits);
 
   return NS_OK;
 }
 
-
-
 NS_IMETHODIMP nsDeviceContextGTK::GetRect(nsRect &aRect)
 {
   // if we have an initialized widget for this device context, use it
@@ -538,7 +526,10 @@ NS_IMETHODIMP nsDeviceContextGTK::GetDeviceContextFor(nsIDeviceContextSpec *aDev
                                                        nsIDeviceContext *&aContext)
 {
   nsresult                 rv;
+
+#if 0
   PrintMethod              method;
+
   nsDeviceContextSpecGTK  *spec = NS_STATIC_CAST(nsDeviceContextSpecGTK *, aDevice);
   
   rv = spec->GetPrintMethod(method);
@@ -568,8 +559,11 @@ NS_IMETHODIMP nsDeviceContextGTK::GetDeviceContextFor(nsIDeviceContextSpec *aDev
   }
   else
 #endif /* USE_XPRINT */
+#endif
 #ifdef USE_POSTSCRIPT
-  if (method == pmPostScript) { // PostScript
+//  if (method == pmPostScript) // PostScript
+  {
+
     // default/PS
     static NS_DEFINE_CID(kCDeviceContextPS, NS_DEVICECONTEXTPS_CID);
   
@@ -592,8 +586,8 @@ NS_IMETHODIMP nsDeviceContextGTK::GetDeviceContextFor(nsIDeviceContextSpec *aDev
                               (void **)&aContext);
     return rv;
   }
+
 #endif /* USE_POSTSCRIPT */
-  
   NS_WARNING("no print module created.");
   return NS_ERROR_UNEXPECTED;
 }
@@ -658,27 +652,7 @@ nsDeviceContextGTK::SetDPI(PRInt32 aPrefDPI)
   return NS_OK;
 }
 
-int nsDeviceContextGTK::prefChanged(const char *aPref, void *aClosure)
-{
-  nsDeviceContextGTK *context = (nsDeviceContextGTK*)aClosure;
-  nsresult rv;
-  
-  if (nsCRT::strcmp(aPref, "browser.display.screen_resolution")==0) {
-    PRInt32 dpi;
-    nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &rv));
-    rv = prefs->GetIntPref(aPref, &dpi);
-    if (NS_SUCCEEDED(rv))
-      context->SetDPI(dpi);
-
-    // If this pref changes, we have to clear our cache of stored system
-    // fonts.
-    ClearCachedSystemFonts();
-  }
-
-  return 0;
-}
-
-void nsDeviceContextGTK::ClearCachedSystemFonts()
+static void DoClearCachedSystemFonts()
 {
   //clear our cache of stored system fonts
   if (gSystemFonts) {
@@ -687,6 +661,33 @@ void nsDeviceContextGTK::ClearCachedSystemFonts()
   }
 }
 
+NS_IMETHODIMP
+nsDeviceContextGTK::ClearCachedSystemFonts()
+{
+  DoClearCachedSystemFonts();
+  return NS_OK;
+}
+
+int nsDeviceContextGTK::prefChanged(const char *aPref, void *aClosure)
+{
+  nsDeviceContextGTK *context = (nsDeviceContextGTK*)aClosure;
+  nsresult rv;
+  
+  if (nsCRT::strcmp(aPref, "layout.css.dpi")==0) {
+    PRInt32 dpi;
+    nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
+    rv = prefs->GetIntPref(aPref, &dpi);
+    if (NS_SUCCEEDED(rv))
+      context->SetDPI(dpi);
+
+    // If this pref changes, we have to clear our cache of stored system
+    // fonts.
+    DoClearCachedSystemFonts();
+  }
+
+  return 0;
+}
+
 #define DEFAULT_TWIP_FONT_SIZE 240
 
 nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
@@ -1224,5 +1225,6 @@ xlfd_from_pango_font_description(GtkWidget *aWidget,
     g_free(subfont_charsets);
   }
   g_free(spec);
+  g_object_unref(font);
 }
 #endif /* MOZ_WIDGET_GTK2 && MOZ_ENABLE_COREXFONTS */
