@@ -36,11 +36,12 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include <strings.h>
 #include "nsFont.h"
 #include "nsIDeviceContext.h"
 #include "nsICharsetConverterManager.h"
 #include "nsIPref.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 
 #define PANGO_ENABLE_BACKEND
 #define PANGO_ENABLE_ENGINE
@@ -52,8 +53,8 @@
 
 #include "nsUnicharUtils.h"
 #include "nsQuickSort.h"
+#include "nsFontConfigUtils.h"
 
-#include <pango/pangoxft.h>
 #include <fontconfig/fontconfig.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -80,25 +81,6 @@ static int                         gNumInstances;
 static NS_DEFINE_CID(kCharsetConverterManagerCID,
                      NS_ICHARSETCONVERTERMANAGER_CID);
 
-struct MozPangoLangGroup {
-    const char *mozLangGroup;
-    const char *PangoLang;
-};
-
-static const MozPangoLangGroup MozPangoLangGroups[] = {
-    { "x-western",      "en" },
-    { "x-central-euro", "pl" },
-    { "x-cyrillic",     "ru" },
-    { "x-baltic",       "lv" },
-    { "x-devanagari",   "hi" },
-    { "x-tamil",        "ta" },
-    { "x-unicode",      0    },
-    { "x-user-def",     0    },
-};
-
-#define NUM_PANGO_LANG_GROUPS (sizeof (MozPangoLangGroups) / \
-                               sizeof (MozPangoLangGroups[0]))
-
 #ifdef DEBUG
 #define DUMP_PRUNICHAR(ustr, ulen) for (PRUint32 llen=0;llen<ulen;llen++) \
                                       printf("0x%x ", ustr[llen]); \
@@ -115,11 +97,7 @@ static const MozPangoLangGroup MozPangoLangGroups[] = {
 
 // Static function decls
 
-static PRBool IsASCIIFontName  (const nsString& aName);
-static int    FFRECountHyphens (nsACString &aFFREName);
-
 static PangoLanguage *GetPangoLanguage(nsIAtom *aLangGroup);
-static const MozPangoLangGroup* FindPangoLangGroup (nsACString &aLangGroup);
 
 static void   FreeGlobals    (void);
 
@@ -144,6 +122,7 @@ nsFontMetricsPango::nsFontMetricsPango()
     mRTLPangoContext = nsnull;
     mPangoAttrList = nsnull;
     mIsRTL = PR_FALSE;
+    mPangoSpaceWidth = 0;
 
     static PRBool initialized = PR_FALSE;
     if (initialized)
@@ -156,8 +135,6 @@ nsFontMetricsPango::nsFontMetricsPango()
 
 nsFontMetricsPango::~nsFontMetricsPango()
 {
-    delete mFont;
-
     if (mDeviceContext)
         mDeviceContext->FontMetricsDeleted(this);
 
@@ -188,13 +165,13 @@ NS_IMETHODIMP
 nsFontMetricsPango::Init(const nsFont& aFont, nsIAtom* aLangGroup,
                          nsIDeviceContext *aContext)
 {
-    mFont = new nsFont(aFont);
+    mFont = aFont;
     mLangGroup = aLangGroup;
 
     // Hang on to the device context
     mDeviceContext = aContext;
     
-    mPointSize = NSTwipsToFloatPoints(mFont->size);
+    mPointSize = NSTwipsToFloatPoints(mFont.size);
 
     // Make sure to clamp the pixel size to something reasonable so we
     // don't make the X server blow up.
@@ -202,7 +179,7 @@ nsFontMetricsPango::Init(const nsFont& aFont, nsIAtom* aLangGroup,
     mPointSize = PR_MIN(screenPixels * FONT_MAX_FONT_SCALE, mPointSize);
 
     // enumerate over the font names passed in
-    mFont->EnumerateFamilies(nsFontMetricsPango::EnumFontCallback, this);
+    mFont.EnumerateFamilies(nsFontMetricsPango::EnumFontCallback, this);
 
     nsCOMPtr<nsIPref> prefService;
     prefService = do_GetService(NS_PREF_CONTRACTID);
@@ -210,10 +187,14 @@ nsFontMetricsPango::Init(const nsFont& aFont, nsIAtom* aLangGroup,
         return NS_ERROR_FAILURE;
         
     nsXPIDLCString value;
+    const char* langGroup;
+    mLangGroup->GetUTF8String(&langGroup);
 
     // Set up the default font name if it's not set
     if (!mGenericFont) {
-        prefService->CopyCharPref("font.default", getter_Copies(value));
+        nsCAutoString name("font.default.");
+        name.Append(langGroup);
+        prefService->CopyCharPref(name.get(), getter_Copies(value));
 
         if (value.get())
             mDefaultFont = value.get();
@@ -233,10 +214,6 @@ nsFontMetricsPango::Init(const nsFont& aFont, nsIAtom* aLangGroup,
             name.Append("variable");
 
         name.Append(char('.'));
-
-        const char* langGroup;
-        mLangGroup->GetUTF8String(&langGroup);
-
         name.Append(langGroup);
 
         PRInt32 minimumInt = 0;
@@ -302,12 +279,11 @@ nsFontMetricsPango::CacheFontMetrics(void)
 
     // Get our font face
     FT_Face face;
+    face = pango_fc_font_lock_face(fcfont);
+    if (!face)
+    	return NS_ERROR_NOT_AVAILABLE;
+    	
     TT_OS2 *os2;
-    XftFont *xftFont = pango_xft_font_get_font(PANGO_FONT(fcfont));
-    if (!xftFont)
-        return NS_ERROR_NOT_AVAILABLE;
-
-    face = XftLockFace(xftFont);
     os2 = (TT_OS2 *) FT_Get_Sfnt_Table(face, ft_sfnt_os2);
 
     // mEmHeight (size in pixels of EM height)
@@ -319,10 +295,12 @@ nsFontMetricsPango::CacheFontMetrics(void)
     mEmHeight = PR_MAX(1, nscoord(size * f));
 
     // mMaxAscent
-    mMaxAscent = nscoord(xftFont->ascent * f);
+    val = MOZ_FT_TRUNC(face->size->metrics.ascender);
+    mMaxAscent = NSToIntRound(val * f);
 
     // mMaxDescent
-    mMaxDescent = nscoord(xftFont->descent * f);
+    val = -MOZ_FT_TRUNC(face->size->metrics.descender);
+    mMaxDescent = NSToIntRound(val * f);
 
     nscoord lineHeight = mMaxAscent + mMaxDescent;
 
@@ -342,7 +320,20 @@ nsFontMetricsPango::CacheFontMetrics(void)
     mEmDescent = mEmHeight - mEmAscent;
 
     // mMaxAdvance
-    mMaxAdvance = nscoord(xftFont->max_advance_width * f);
+    val = MOZ_FT_TRUNC(face->size->metrics.max_advance);
+    mMaxAdvance = NSToIntRound(val * f);
+    // X may screw up if we try to measure/draw more than 32767 pixels in
+    // one operation.
+    mMaxStringLength = (PRInt32)floor(32767.0/val);
+    mMaxStringLength = PR_MAX(1, mMaxStringLength);
+
+    // mPangoSpaceWidth
+    PangoLayout *layout = pango_layout_new(mPangoContext);
+    pango_layout_set_text(layout, " ", 1);
+    int pswidth, psheight;
+    pango_layout_get_size(layout, &pswidth, &psheight);
+    mPangoSpaceWidth = pswidth;
+    g_object_unref(layout);
 
     // mSpaceWidth (width of a space)
     nscoord tmpWidth;
@@ -357,17 +348,16 @@ nsFontMetricsPango::CacheFontMetrics(void)
     mAveCharWidth = tmpWidth;
 
     // mXHeight (height of an 'x' character)
-    PRUnichar xUnichar('x');
-    XGlyphInfo extents;
-    if (FcCharSetHasChar(xftFont->charset, xUnichar)) {
-        XftTextExtents16(GDK_DISPLAY(), xftFont, &xUnichar, 1, &extents);
-        mXHeight = extents.height;
+    if (pango_fc_font_has_char(fcfont, 'x')) {
+        PangoRectangle rect;
+        PangoGlyph glyph = pango_fc_font_get_glyph (fcfont, 'x');
+        pango_font_get_glyph_extents (PANGO_FONT (fcfont), glyph, &rect, NULL);
+        mXHeight = NSToIntRound(rect.height * f / PANGO_SCALE);
     }
     else {
         // 56% of ascent, best guess for non-true type or asian fonts
-        mXHeight = nscoord(((float)mMaxAscent) * 0.56);
+        mXHeight = nscoord(((float)mMaxAscent) * 0.56 * f);
     }
-    mXHeight = nscoord(mXHeight * f);
 
     // mUnderlineOffset (offset for underlines)
     val = CONVERT_DESIGN_UNITS_TO_PIXELS(face->underline_position,
@@ -377,7 +367,8 @@ nsFontMetricsPango::CacheFontMetrics(void)
     }
     else {
         mUnderlineOffset =
-            -NSToIntRound(PR_MAX(1, floor(0.1 * xftFont->height + 0.5)) * f);
+            -NSToIntRound(PR_MAX(1, floor(0.1 *
+                MOZ_FT_TRUNC(face->size->metrics.height) + 0.5)) * f);
     }
 
     // mUnderlineSize (thickness of an underline)
@@ -388,7 +379,8 @@ nsFontMetricsPango::CacheFontMetrics(void)
     }
     else {
         mUnderlineSize =
-            NSToIntRound(PR_MAX(1, floor(0.05 * xftFont->height + 0.5)) * f);
+            NSToIntRound(PR_MAX(1,
+               floor(0.05 * MOZ_FT_TRUNC(face->size->metrics.height) + 0.5)) * f);
     }
 
     // mSuperscriptOffset
@@ -419,7 +411,7 @@ nsFontMetricsPango::CacheFontMetrics(void)
     // mStrikeoutSize
     mStrikeoutSize = mUnderlineSize;
 
-    XftUnlockFace(xftFont);
+    pango_fc_font_unlock_face(fcfont);
 
     /*
     printf("%i\n", mXHeight);
@@ -451,13 +443,6 @@ nsFontMetricsPango::Destroy()
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsFontMetricsPango::GetFont(const nsFont *&aFont)
-{
-    aFont = mFont;
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsFontMetricsPango::GetLangGroup(nsIAtom** aLangGroup)
 {
@@ -484,17 +469,18 @@ nsFontMetricsPango::GetWidth(const char* aString, PRUint32 aLength,
 
     pango_layout_set_text(layout, aString, aLength);
 
+    if (mPangoSpaceWidth)
+        FixupSpaceWidths(layout, aString);
+
     int width, height;
 
     pango_layout_get_size(layout, &width, &height);
 
-    width /= PANGO_SCALE;
-
     g_object_unref(layout);
 
     float f;
     f = mDeviceContext->DevUnitsToAppUnits();
-    aWidth = NSToCoordRound(width * f);
+    aWidth = NSToCoordRound(width * f / PANGO_SCALE);
 
     //    printf("GetWidth (char *) %d\n", aWidth);
 
@@ -525,13 +511,12 @@ nsFontMetricsPango::GetWidth(const PRUnichar* aString, PRUint32 aLength,
     gint width, height;
 
     pango_layout_set_text(layout, text, strlen(text));
+    FixupSpaceWidths(layout, text);
     pango_layout_get_size(layout, &width, &height);
 
-    width /= PANGO_SCALE;
-
     float f;
     f = mDeviceContext->DevUnitsToAppUnits();
-    aWidth = NSToCoordRound(width * f);
+    aWidth = NSToCoordRound(width * f / PANGO_SCALE);
 
     //    printf("GetWidth %d\n", aWidth);
 
@@ -572,6 +557,7 @@ nsFontMetricsPango::GetTextDimensions(const PRUnichar* aString,
         
 
     pango_layout_set_text(layout, text, strlen(text));
+    FixupSpaceWidths(layout, text);
 
     // Get the logical extents
     PangoLayoutLine *line;
@@ -586,9 +572,9 @@ nsFontMetricsPango::GetTextDimensions(const PRUnichar* aString,
     float P2T;
     P2T = mDeviceContext->DevUnitsToAppUnits();
 
-    aDimensions.width = NSToCoordRound(rect.width / PANGO_SCALE * P2T);
-    aDimensions.ascent = NSToCoordRound(PANGO_ASCENT(rect) / PANGO_SCALE * P2T);
-    aDimensions.descent = NSToCoordRound(PANGO_DESCENT(rect) / PANGO_SCALE * P2T);
+    aDimensions.width = NSToCoordRound(rect.width * P2T / PANGO_SCALE);
+    aDimensions.ascent = NSToCoordRound(PANGO_ASCENT(rect) * P2T / PANGO_SCALE);
+    aDimensions.descent = NSToCoordRound(PANGO_DESCENT(rect) * P2T / PANGO_SCALE);
 
     //    printf("GetTextDimensions %d %d %d\n", aDimensions.width,
     //aDimensions.ascent, aDimensions.descent);
@@ -714,6 +700,7 @@ nsFontMetricsPango::DrawString(const char *aString, PRUint32 aLength,
     PangoLayout *layout = pango_layout_new(mPangoContext);
 
     pango_layout_set_text(layout, aString, aLength);
+    FixupSpaceWidths(layout, aString);
 
     int x = aX;
     int y = aY;
@@ -777,6 +764,7 @@ nsFontMetricsPango::DrawString(const PRUnichar* aString, PRUint32 aLength,
     }
 
     pango_layout_set_text(layout, text, strlen(text));
+    FixupSpaceWidths(layout, text);
 
     aContext->GetTranMatrix()->TransformCoord(&x, &y);
 
@@ -835,39 +823,33 @@ nsFontMetricsPango::GetBoundingMetrics(const PRUnichar *aString,
         NS_WARNING("nsFontMetricsPango::GetBoundingMetrics invalid unicode to follow");
         DUMP_PRUNICHAR(aString, aLength)
 #endif
-        aBoundingMetrics.leftBearing = 0;
-        aBoundingMetrics.rightBearing = 0;
-        aBoundingMetrics.width = 0;
-        aBoundingMetrics.ascent = 0;
-        aBoundingMetrics.descent = 0;
+        aBoundingMetrics.Clear();
 
         rv = NS_ERROR_FAILURE;
         goto loser;
     }
 
-    pango_layout_set_text(layout, text, strlen(text));
+    pango_layout_set_text(layout, text, -1);
+    FixupSpaceWidths(layout, text);
 
-    // Get the logical extents
     PangoLayoutLine *line;
     if (pango_layout_get_line_count(layout) != 1) {
         printf("Warning: more than one line!\n");
     }
     line = pango_layout_get_line(layout, 0);
 
-    // Get the ink extents
-    PangoRectangle rect;
-    pango_layout_line_get_extents(line, NULL, &rect);
+    // Get the ink and logical extents
+    PangoRectangle ink, logical;
+    pango_layout_line_get_extents(line, &ink, &logical);
 
     float P2T;
     P2T = mDeviceContext->DevUnitsToAppUnits();
 
-    aBoundingMetrics.leftBearing =
-        NSToCoordRound(rect.x / PANGO_SCALE * P2T);
-    aBoundingMetrics.rightBearing =
-        NSToCoordRound(rect.width / PANGO_SCALE * P2T);
-    aBoundingMetrics.width = NSToCoordRound((rect.x + rect.width) / PANGO_SCALE * P2T);
-    aBoundingMetrics.ascent = NSToCoordRound(rect.y / PANGO_SCALE * P2T);
-    aBoundingMetrics.descent = NSToCoordRound(rect.height / PANGO_SCALE * P2T);
+    aBoundingMetrics.leftBearing  = NSToCoordRound(PANGO_LBEARING(ink) * P2T / PANGO_SCALE);
+    aBoundingMetrics.rightBearing = NSToCoordRound(PANGO_RBEARING(ink) * P2T / PANGO_SCALE);
+    aBoundingMetrics.ascent       = NSToCoordRound(PANGO_ASCENT(ink)   * P2T / PANGO_SCALE);
+    aBoundingMetrics.descent      = NSToCoordRound(PANGO_DESCENT(ink)  * P2T / PANGO_SCALE);
+    aBoundingMetrics.width        = NSToCoordRound(logical.width       * P2T / PANGO_SCALE);
 
  loser:
     g_free(text);
@@ -889,7 +871,7 @@ nsFontMetricsPango::SetRightToLeftText(PRBool aIsRTL)
 {
     if (aIsRTL) {
         if (!mRTLPangoContext) {
-            mRTLPangoContext = pango_xft_get_context(GDK_DISPLAY(), 0);
+            mRTLPangoContext = gdk_pango_context_get();
             pango_context_set_base_dir(mRTLPangoContext, PANGO_DIRECTION_RTL);
 
             gdk_pango_context_set_colormap(mRTLPangoContext, gdk_rgb_get_cmap());
@@ -906,13 +888,241 @@ nsFontMetricsPango::SetRightToLeftText(PRBool aIsRTL)
     return NS_OK;
 }
 
+PRBool
+nsFontMetricsPango::GetRightToLeftText()
+{
+    return mIsRTL;
+}
+
+nsresult
+nsFontMetricsPango::GetClusterInfo(const PRUnichar *aText,
+                                   PRUint32 aLength,
+                                   PRUint8 *aClusterStarts)
+{
+    nsresult rv = NS_OK;
+    PangoLogAttr *attrs = NULL;
+    gint n_attrs = 0;
+    PangoLayout *layout = pango_layout_new(mPangoContext);
+    
+    // Convert the incoming UTF-16 to UTF-8
+    gchar *text = g_utf16_to_utf8(aText, aLength, NULL, NULL, NULL);
+
+    if (!text) {
+#ifdef DEBUG
+        NS_WARNING("nsFontMetricsPango::GetWidth invalid unicode to follow");
+        DUMP_PRUNICHAR(aText, aLength)
+#endif
+        rv = NS_ERROR_FAILURE;
+        goto loser;
+    }
+
+    // Set up the pango layout
+    pango_layout_set_text(layout, text, strlen(text));
+    FixupSpaceWidths(layout, text);
+
+    // Convert back to UTF-16 while filling in the cluster info
+    // structure.
+    pango_layout_get_log_attrs(layout, &attrs, &n_attrs);
+
+    for (PRUint32 pos = 0; pos < aLength; pos++) {
+        if (IS_HIGH_SURROGATE(aText[pos])) {
+            aClusterStarts[pos] = 1;
+            pos++;
+        }
+        else {
+            aClusterStarts[pos] = attrs[pos].is_cursor_position;
+        }
+    }
+
+ loser:
+    if (attrs)
+        g_free(attrs);
+    if (text)
+        g_free(text);
+    if (layout)
+        g_object_unref(layout);
+
+    return rv;
+}
+
+PRInt32
+nsFontMetricsPango::GetPosition(const PRUnichar *aText, PRUint32 aLength,
+                                nsPoint aPt)
+{
+    int trailing = 0;
+    int inx = 0;
+    const gchar *curChar;
+    PRInt32 retval = 0;
+
+    float f = mDeviceContext->AppUnitsToDevUnits();
+    
+    PangoLayout *layout = pango_layout_new(mPangoContext);
+    PRUint32 localX = (PRUint32)(aPt.x * PANGO_SCALE * f);
+    PRUint32 localY = (PRUint32)(aPt.y * PANGO_SCALE * f);
+
+    // Convert the incoming UTF-16 to UTF-8
+    gchar *text = g_utf16_to_utf8(aText, aLength, NULL, NULL, NULL);
+
+    if (!text) {
+#ifdef DEBUG
+        NS_WARNING("nsFontMetricsPango::GetWidth invalid unicode to follow");
+        DUMP_PRUNICHAR(aText, aLength)
+#endif
+        retval = -1;
+        goto loser;
+    }
+
+    // Set up the pango layout
+    pango_layout_set_text(layout, text, strlen(text));
+    FixupSpaceWidths(layout, text);
+    
+    pango_layout_xy_to_index(layout, localX, localY,
+                             &inx, &trailing);
+
+    // Convert the index back to the utf-16 index
+    curChar = text;
+
+    for (PRUint32 curOffset=0; curOffset < aLength;
+         curOffset++, curChar = g_utf8_find_next_char(curChar, NULL)) {
+
+        // Check for a match before checking for a surrogate pair
+        if (curChar - text == inx) {
+            retval = curOffset;
+            break;
+        }
+
+        if (IS_HIGH_SURROGATE(aText[curOffset]))
+            curOffset++;
+    }
+
+    // If there was a trailing result, advance the index pointer the
+    // number of characters equal to the trailing result.
+    while (trailing) {
+        retval++;
+        // Yes, this can make aInx > length to indicate the end of the
+        // string.
+        if (retval < (PRInt32)aLength && IS_HIGH_SURROGATE(aText[retval]))
+            retval++;
+        trailing--;
+    }
+
+ loser:
+    if (text)
+        g_free(text);
+    if (layout)
+        g_object_unref(layout);
+
+    return retval;
+}
+
+nsresult
+nsFontMetricsPango::GetRangeWidth(const PRUnichar *aText,
+                                  PRUint32 aLength,
+                                  PRUint32 aStart,
+                                  PRUint32 aEnd,
+                                  PRUint32 &aWidth)
+{
+    nsresult rv = NS_OK;
+    PRUint32 utf8Start = 0;
+    PRUint32 utf8End = 0;
+
+    aWidth = 0;
+
+    // Convert the incoming UTF-16 to UTF-8
+    gchar *text = g_utf16_to_utf8(aText, aLength, NULL, NULL, NULL);
+    gchar *curChar = text;
+
+    if (!text) {
+#ifdef DEBUG
+        NS_WARNING("nsFontMetricsPango::GetWidth invalid unicode to follow");
+        DUMP_PRUNICHAR(aText, aLength)
+#endif
+        rv = NS_ERROR_FAILURE;
+        goto loser;
+    }
+
+    // Convert the utf16 offsets into utf8 offsets
+    for (PRUint32 curOffset = 0; curOffset < aLength;
+         curOffset++, curChar = g_utf8_find_next_char(curChar, NULL)) {
+
+        if (curOffset == aStart)
+            utf8Start = curChar - text;
+
+        if (curOffset == aEnd)
+            utf8End = curChar - text;
+        
+        if (IS_HIGH_SURROGATE(aText[curOffset]))
+            curOffset++;
+    }
+
+    // Special case where the end index is the same as the length
+    if (aLength == aEnd)
+        utf8End = strlen(text);
+
+    rv = GetRangeWidth(text, strlen(text), utf8Start, utf8End, aWidth);
+
+ loser:
+    if (text)
+        g_free(text);
+
+    return rv;
+}
+
+nsresult
+nsFontMetricsPango::GetRangeWidth(const char *aText,
+                                  PRUint32 aLength,
+                                  PRUint32 aStart,
+                                  PRUint32 aEnd,
+                                  PRUint32 &aWidth)
+{
+    nsresult rv = NS_OK;
+    int *ranges = NULL;
+    int n_ranges = 0;
+    float f;
+
+    aWidth = 0;
+
+    PangoLayout *layout = pango_layout_new(mPangoContext);
+
+    if (!aText) {
+        rv = NS_ERROR_FAILURE;
+        goto loser;
+    }
+
+    pango_layout_set_text(layout, aText, aLength);
+    FixupSpaceWidths(layout, aText);
+
+    PangoLayoutLine *line;
+    if (pango_layout_get_line_count(layout) != 1) {
+        printf("Warning: more than one line!\n");
+    }
+    line = pango_layout_get_line(layout, 0);
+
+    pango_layout_line_get_x_ranges(line, aStart, aEnd, &ranges, &n_ranges);
+
+    aWidth = (ranges[((n_ranges - 1) * 2) + 1] - ranges[0]);
+
+    f = mDeviceContext-> DevUnitsToAppUnits();
+    aWidth = nscoord(aWidth * f / PANGO_SCALE);
+
+ loser:
+    if (ranges)
+        g_free(ranges);
+    if (layout)
+        g_object_unref(layout);
+
+    return rv;
+}
+
 /* static */
 PRUint32
 nsFontMetricsPango::GetHints(void)
 {
     return (NS_RENDERING_HINT_BIDI_REORDERING |
             NS_RENDERING_HINT_ARABIC_SHAPING | 
-            NS_RENDERING_HINT_FAST_MEASURE);
+            NS_RENDERING_HINT_FAST_MEASURE |
+            NS_RENDERING_HINT_REORDER_SPACED_TEXT |
+            NS_RENDERING_HINT_TEXT_CLUSTERS);
 }
 
 /* static */
@@ -920,13 +1130,11 @@ nsresult
 nsFontMetricsPango::FamilyExists(nsIDeviceContext *aDevice,
                                  const nsString &aName)
 {
-    if (!IsASCIIFontName(aName))
-        return NS_ERROR_FAILURE;
-
-    NS_ConvertUCS2toUTF8 name(aName);
+    // fontconfig family name is always in UTF-8
+    NS_ConvertUTF16toUTF8 name(aName);
 
     nsresult rv = NS_ERROR_FAILURE;
-    PangoContext *context = pango_xft_get_context(GDK_DISPLAY(), 0);
+    PangoContext *context = gdk_pango_context_get();
     PangoFontFamily **familyList;
     int n;
 
@@ -971,7 +1179,7 @@ nsFontMetricsPango::RealizeFont(void)
 
     // If there's a generic add a pref for the generic if there's one
     // set.
-    if (mGenericFont && !mFont->systemFont) {
+    if (mGenericFont && !mFont.systemFont) {
         nsCString name;
         name += "font.name.";
         name += mGenericFont->get();
@@ -991,7 +1199,7 @@ nsFontMetricsPango::RealizeFont(void)
 
             // we ignore prefs that have three hypens since they are X
             // style prefs.
-            if (FFRECountHyphens(value) < 3) {
+            if (NS_FFRECountHyphens(value) < 3) {
                 nsCString tmpstr;
                 tmpstr.Append(value);
 
@@ -1002,7 +1210,7 @@ nsFontMetricsPango::RealizeFont(void)
     }
 
     // Add the generic if there is one.
-    if (mGenericFont && !mFont->systemFont) {
+    if (mGenericFont && !mFont.systemFont) {
         familyList.Append(mGenericFont->get());
         familyList.Append(',');
     }
@@ -1017,17 +1225,21 @@ nsFontMetricsPango::RealizeFont(void)
 
     // Set the style
     pango_font_description_set_style(mPangoFontDesc,
-                                     CalculateStyle(mFont->style));
+                                     CalculateStyle(mFont.style));
 
     // Set the weight
     pango_font_description_set_weight(mPangoFontDesc,
-                                      CalculateWeight(mFont->weight));
+                                      CalculateWeight(mFont.weight));
 
     // Now that we have the font description set up, create the
     // context.
-    mLTRPangoContext = pango_xft_get_context(GDK_DISPLAY(), 0);
+    mLTRPangoContext = gdk_pango_context_get();
     mPangoContext = mLTRPangoContext;
 
+    // Make sure to set the base direction to LTR - if layout needs to
+    // render RTL text it will use ::SetRightToLeftText()
+    pango_context_set_base_dir(mPangoContext, PANGO_DIRECTION_LTR);
+
     // Set the color map so we can draw later.
     gdk_pango_context_set_colormap(mPangoContext, gdk_rgb_get_cmap());
 
@@ -1045,13 +1257,12 @@ PRBool
 nsFontMetricsPango::EnumFontCallback(const nsString &aFamily,
                                      PRBool aIsGeneric, void *aData)
 {
-    // make sure it's an ascii name, if not then return and continue
-    // enumerating
-    if (!IsASCIIFontName(aFamily))
-        return PR_TRUE;
+    NS_ConvertUTF16toUTF8 name(aFamily);
 
-    nsCAutoString name;
-    name.AssignWithConversion(aFamily.get());
+    // The newest fontconfig does the full Unicode case folding so that 
+    // we're being lazy here by calling |ToLowerCase| after converting
+    // to UTF-8  assuming that in virtually all cases, we just have to
+    // fold [A-Z].  (bug 223653). 
     ToLowerCase(name);
     nsFontMetricsPango *metrics = (nsFontMetricsPango *)aData;
     metrics->mFontList.AppendCString(name);
@@ -1136,13 +1347,11 @@ nsFontMetricsPango::DrawStringSlowly(const gchar *aText,
         }
 
         /*        printf("    rendering at X coord %d\n", aX + offset); */
-
-        gdk_draw_glyphs(aDrawable, aGC, layoutRun->item->analysis.font,
-                        aX + (gint)(offset / PANGO_SCALE), aY, layoutRun->glyphs);
-
         offset += tmpOffset;
     }
 
+    gdk_draw_layout_line(aDrawable, aGC, aX, aY, aLine);
+
     delete[] utf8spacing;
 }
 
@@ -1294,35 +1503,24 @@ nsFontMetricsPango::GetTextDimensionsInternal(const gchar*        aString,
     return NS_OK;
 }
 
-/* static */
-PRBool
-IsASCIIFontName(const nsString& aName)
+void
+nsFontMetricsPango::FixupSpaceWidths (PangoLayout *aLayout,
+                                      const char *aString)
 {
-    PRUint32 len = aName.Length();
-    const PRUnichar* str = aName.get();
-    for (PRUint32 i = 0; i < len; i++) {
-        /*
-         * X font names are printable ASCII, ignore others (for now)
-         */
-        if ((str[i] < 0x20) || (str[i] > 0x7E)) {
-            return PR_FALSE;
+    PangoLayoutLine *line = pango_layout_get_line(aLayout, 0);
+
+    gint curRun = 0;
+
+    for (GSList *tmpList = line->runs; tmpList && tmpList->data;
+         tmpList = tmpList->next, curRun++) {
+        PangoLayoutRun *layoutRun = (PangoLayoutRun *)tmpList->data;
+
+        for (gint i=0; i < layoutRun->glyphs->num_glyphs; i++) {
+            gint thisOffset = (gint)layoutRun->glyphs->log_clusters[i] + layoutRun->item->offset;
+            if (aString[thisOffset] == ' ')
+                layoutRun->glyphs->glyphs[i].geometry.width = mPangoSpaceWidth;
         }
     }
-  
-    return PR_TRUE;
-}
-
-/* static */
-int
-FFRECountHyphens (nsACString &aFFREName)
-{
-    int h = 0;
-    PRInt32 hyphen = 0;
-    while ((hyphen = aFFREName.FindChar('-', hyphen)) >= 0) {
-        ++h;
-        ++hyphen;
-    }
-    return h;
 }
 
 /* static */
@@ -1335,8 +1533,8 @@ GetPangoLanguage(nsIAtom *aLangGroup)
 
     // see if the lang group needs to be translated from mozilla's
     // internal mapping into fontconfig's
-    const struct MozPangoLangGroup *langGroup;
-    langGroup = FindPangoLangGroup(cname);
+    const MozGtkLangGroup *langGroup;
+    langGroup = NS_FindFCLangGroup(cname);
 
     // if there's no lang group, just use the lang group as it was
     // passed to us
@@ -1345,26 +1543,12 @@ GetPangoLanguage(nsIAtom *aLangGroup)
     // safe.
     if (!langGroup)
         return pango_language_from_string(cname.get());
-    else if (langGroup->PangoLang) 
-        return pango_language_from_string(langGroup->PangoLang);
+    else if (langGroup->Lang) 
+        return pango_language_from_string((char *) langGroup->Lang);
 
     return pango_language_from_string("en");
 }
 
-/* static */
-const MozPangoLangGroup*
-FindPangoLangGroup (nsACString &aLangGroup)
-{
-    for (unsigned int i=0; i < NUM_PANGO_LANG_GROUPS; ++i) {
-        if (aLangGroup.Equals(MozPangoLangGroups[i].mozLangGroup,
-                              nsCaseInsensitiveCStringComparator())) {
-            return &MozPangoLangGroups[i];
-        }
-    }
-
-    return nsnull;
-}
-
 /* static */
 void
 FreeGlobals(void)
@@ -1377,7 +1561,7 @@ CalculateStyle(PRUint8 aStyle)
 {
     switch(aStyle) {
     case NS_FONT_STYLE_ITALIC:
-        return PANGO_STYLE_OBLIQUE;
+        return PANGO_STYLE_ITALIC;
         break;
     case NS_FONT_STYLE_OBLIQUE:
         return PANGO_STYLE_OBLIQUE;
@@ -1459,7 +1643,7 @@ EnumFontsPango(nsIAtom* aLangGroup, const char* aGeneric,
     if (!pat)
         goto end;
 
-    os = FcObjectSetBuild(FC_FAMILY, FC_FOUNDRY, 0);
+    os = FcObjectSetBuild(FC_FAMILY, FC_FOUNDRY, NULL);
     if (!os)
         goto end;
 
@@ -1522,7 +1706,6 @@ EnumFontsPango(nsIAtom* aLangGroup, const char* aGeneric,
 
     for (int i=0; i < fs->nfont; ++i) {
         char *family;
-        PRUnichar *name;
 
         // if there's no family, just move to the next iteration
         if (FcPatternGetString (fs->fonts[i], FC_FAMILY, 0,
@@ -1530,18 +1713,12 @@ EnumFontsPango(nsIAtom* aLangGroup, const char* aGeneric,
             continue;
         }
 
-        name = NS_STATIC_CAST(PRUnichar *,
-                              nsMemory::Alloc ((strlen (family) + 1)
-                                               * sizeof (PRUnichar)));
+        // fontconfig always returns family names in UTF-8
+        PRUnichar* name =  UTF8ToNewUnicode(nsDependentCString(family));
 
         if (!name)
             goto end;
 
-        PRUnichar *r = name;
-        for (char *f = family; *f; ++f)
-            *r++ = *f;
-        *r = '\0';
-
         array[narray++] = name;
     }
 
