@@ -43,7 +43,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsISupportsUtils.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 #include "nsIPref.h"
 #include "nsFontMetricsXft.h"
 #include "prenv.h"
@@ -85,6 +85,9 @@ public:
     nsFontXft(FcPattern *aPattern, FcPattern *aFontName);
     virtual ~nsFontXft() = 0;
 
+    // Callers outside of FindFont and DoMatch (which deal directly with
+    // mLoadedFonts) do not need to call this; they can access mXftFont
+    // directly since it is guaranteed to be non-null.
     XftFont   *GetXftFont (void);
     virtual nsresult GetTextExtents32 (const FcChar32 *aString, PRUint32 aLen, 
                                        XGlyphInfo &aGlyphInfo);
@@ -319,8 +322,6 @@ nsFontMetricsXft::nsFontMetricsXft(): mMiniFont(nsnull)
 
 nsFontMetricsXft::~nsFontMetricsXft()
 {
-    delete mFont;
-
     if (mDeviceContext)
         mDeviceContext->FontMetricsDeleted(this);
 
@@ -352,14 +353,14 @@ NS_IMETHODIMP
 nsFontMetricsXft::Init(const nsFont& aFont, nsIAtom* aLangGroup,
                        nsIDeviceContext *aContext)
 {
-    mFont = new nsFont(aFont);
+    mFont = aFont;
     mLangGroup = aLangGroup;
 
     // Hang onto the device context
     mDeviceContext = aContext;
 
     float app2dev = mDeviceContext->AppUnitsToDevUnits();
-    mPixelSize = NSTwipsToFloatPixels(mFont->size, app2dev);
+    mPixelSize = NSTwipsToFloatPixels(mFont.size, app2dev);
 
     // Make sure to clamp the pixel size to something reasonable so we
     // don't make the X server blow up.
@@ -367,7 +368,7 @@ nsFontMetricsXft::Init(const nsFont& aFont, nsIAtom* aLangGroup,
     mPixelSize = PR_MIN(screenPixels * FONT_MAX_FONT_SCALE, mPixelSize);
 
     // enumerate over the font names passed in
-    mFont->EnumerateFamilies(nsFontMetricsXft::EnumFontCallback, this);
+    mFont.EnumerateFamilies(nsFontMetricsXft::EnumFontCallback, this);
 
     nsCOMPtr<nsIPref> prefService;
     prefService = do_GetService(NS_PREF_CONTRACTID);
@@ -375,10 +376,14 @@ nsFontMetricsXft::Init(const nsFont& aFont, nsIAtom* aLangGroup,
         return NS_ERROR_FAILURE;
         
     nsXPIDLCString value;
+    const char* langGroup;
+    mLangGroup->GetUTF8String(&langGroup);
 
     // Set up the default font name if it's not set
     if (!mGenericFont) {
-        prefService->CopyCharPref("font.default", getter_Copies(value));
+        nsCAutoString name("font.default.");
+        name.Append(langGroup);
+        prefService->CopyCharPref(name.get(), getter_Copies(value));
 
         if (value.get())
             mDefaultFont = value.get();
@@ -398,10 +403,6 @@ nsFontMetricsXft::Init(const nsFont& aFont, nsIAtom* aLangGroup,
             name.Append("variable");
 
         name.Append(char('.'));
-
-        const char* langGroup;
-        mLangGroup->GetUTF8String(&langGroup);
-
         name.Append(langGroup);
 
         PRInt32 minimum = 0;
@@ -455,13 +456,6 @@ nsFontMetricsXft::Destroy()
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsFontMetricsXft::GetFont(const nsFont *&aFont)
-{
-    aFont = mFont;
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsFontMetricsXft::GetLangGroup(nsIAtom** aLangGroup)
 {
@@ -486,9 +480,8 @@ nsFontMetricsXft::GetWidth(const char* aString, PRUint32 aLength,
 {
     NS_TIMELINE_MARK_FUNCTION("GetWidth");
 
-    XftFont *font = mWesternFont->GetXftFont();
-    if (!font)
-        return NS_ERROR_NOT_AVAILABLE;
+    XftFont *font = mWesternFont->mXftFont;
+    NS_ASSERTION(font, "FindFont returned a bad font");
 
     XGlyphInfo glyphInfo;
 
@@ -742,6 +735,48 @@ nsFontMetricsXft::SetRightToLeftText(PRBool aIsRTL)
     return NS_OK;
 }
 
+PRBool
+nsFontMetricsXft::GetRightToLeftText()
+{
+    return PR_FALSE;
+}
+
+nsresult
+nsFontMetricsXft::GetClusterInfo(const PRUnichar *aText,
+                                 PRUint32 aLength,
+                                 PRUint8 *aClusterStarts)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+PRInt32
+nsFontMetricsXft::GetPosition(const PRUnichar *aText,
+                              PRUint32 aLength,
+                              nsPoint aPt)
+{
+    return -1;
+}
+
+nsresult
+nsFontMetricsXft::GetRangeWidth(const PRUnichar *aText,
+                                PRUint32 aLength,
+                                PRUint32 aStart,
+                                PRUint32 aEnd,
+                                PRUint32 &aWidth)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+nsresult
+nsFontMetricsXft::GetRangeWidth(const char *aText,
+                                PRUint32 aLength,
+                                PRUint32 aStart,
+                                PRUint32 aEnd,
+                                PRUint32 &aWidth)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 PRUint32
 nsFontMetricsXft::GetHints(void)
 {
@@ -777,14 +812,12 @@ nsFontMetricsXft::CacheFontMetrics(void)
     f = mDeviceContext->DevUnitsToAppUnits();
     
     // Get our font face
-    FT_Face face;
-    TT_OS2 *os2;
-    XftFont *xftFont = mWesternFont->GetXftFont();
-    if (!xftFont)
-        return NS_ERROR_NOT_AVAILABLE;
+    XftFont *xftFont = mWesternFont->mXftFont;
+    NS_ASSERTION(xftFont, "FindFont returned a bad font");
 
-    face = XftLockFace(xftFont);
-    os2 = (TT_OS2 *) FT_Get_Sfnt_Table(face, ft_sfnt_os2);
+    FT_Face face = XftLockFace(xftFont);
+    if (!face)
+        return NS_ERROR_NOT_AVAILABLE;
 
     // mEmHeight (size in pixels of EM height)
     int size;
@@ -819,6 +852,10 @@ nsFontMetricsXft::CacheFontMetrics(void)
 
     // mMaxAdvance
     mMaxAdvance = nscoord(xftFont->max_advance_width * f);
+    // X may screw up if we try to measure/draw more than 32767 pixels in
+    // one operation.
+    mMaxStringLength = (PRInt32)floor(32767.0/xftFont->max_advance_width);
+    mMaxStringLength = PR_MAX(1, mMaxStringLength);
 
     // mSpaceWidth (width of a space)
     gint rawWidth;
@@ -865,6 +902,8 @@ nsFontMetricsXft::CacheFontMetrics(void)
             NSToIntRound(PR_MAX(1, floor(0.05 * xftFont->height + 0.5)) * f);
     }
 
+    TT_OS2 *os2 = (TT_OS2 *) FT_Get_Sfnt_Table(face, ft_sfnt_os2);
+
     // mSuperscriptOffset
     if (os2 && os2->ySuperscriptYOffset) {
         val = CONVERT_DESIGN_UNITS_TO_PIXELS(os2->ySuperscriptYOffset,
@@ -929,9 +968,13 @@ nsFontMetricsXft::FindFont(PRUint32 aChar)
         return nsnull;
     }
 
+    PRBool removeFirstFont = PR_FALSE;
     nsFontXft *font = (nsFontXft *)mLoadedFonts.ElementAt(0);
-    if (font->HasChar(aChar))
-        return font;
+    if (font->HasChar(aChar)) {
+        if (font->GetXftFont())
+            return font;
+        removeFirstFont = PR_TRUE;
+    }
 
     // We failed to find the character in the best-match font, so load
     // _all_ matching fonts if we haven't already done so.
@@ -939,12 +982,26 @@ nsFontMetricsXft::FindFont(PRUint32 aChar)
     if (mMatchType == eBestMatch)
         DoMatch(PR_TRUE);
 
-    // Now check the remaining fonts
+    PRInt32 i = 1;
+    if (removeFirstFont) {
+        // The first font was bad, so remove it (see below).  But do this
+        // after |DoMatch| since otherwise it will get re-added.
+        mLoadedFonts.RemoveElementAt(0);
+        i = 0;
+    }
 
-    for (PRInt32 i = 1, end = mLoadedFonts.Count(); i < end; ++i) {
+    // Now check the remaining fonts
+    for (; i < mLoadedFonts.Count(); ++i) {
         nsFontXft *font = (nsFontXft *)mLoadedFonts.ElementAt(i);
-        if (font->HasChar(aChar))
-            return font;
+        if (font->HasChar(aChar)) {
+            if (font->GetXftFont())
+                return font;
+            // This is a bad font, so remove it from mLoadedFonts.  This
+            // could happen if it's in fc.cache-1 but the font doesn't exist
+            // (https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=111973)
+            // or isn't readable.
+            mLoadedFonts.RemoveElementAt(i--);
+        }
     }
 
     // If we got this far, none of the fonts support this character.
@@ -1008,56 +1065,27 @@ nsFontMetricsXft::SetupFCPattern(void)
 
     // If there's a generic add a pref for the generic if there's one
     // set.
-    if (mGenericFont && !mFont->systemFont) {
-        nsCString name;
-        name += "font.name.";
-        name += mGenericFont->get();
-        name += ".";
-
-        nsString langGroup;
-        mLangGroup->ToString(langGroup);
-
-        name.AppendWithConversion(langGroup);
-
-        nsCOMPtr<nsIPref> pref;
-        pref = do_GetService(NS_PREF_CONTRACTID);
-        if (pref) {
-            nsresult rv;
-            nsXPIDLCString value;
-            rv = pref->GetCharPref(name.get(), getter_Copies(value));
-
-            // we ignore prefs that have three hypens since they are X
-            // style prefs.
-            if (NS_FFRECountHyphens(value) < 3) {
-                nsCString tmpstr;
-                tmpstr.Append(value);
-
-                if (PR_LOG_TEST(gXftFontLoad, PR_LOG_DEBUG)) {
-                    printf("\tadding generic font from preferences: %s\n",
-                           tmpstr.get());
-                }
-
-                NS_AddFFRE(mPattern, &tmpstr, PR_FALSE);
-            }
-        }
+    if (mGenericFont && !mFont.systemFont) {
+        NS_AddGenericFontFromPref(mGenericFont, mLangGroup, mPattern,
+                                  gXftFontLoad);
     }
 
     // Add the generic if there is one.
-    if (mGenericFont && !mFont->systemFont)
+    if (mGenericFont && !mFont.systemFont)
         NS_AddFFRE(mPattern, mGenericFont, PR_FALSE);
 
     if (PR_LOG_TEST(gXftFontLoad, PR_LOG_DEBUG)) {
         // generic font
-        if (mGenericFont && !mFont->systemFont) {
+        if (mGenericFont && !mFont.systemFont) {
             printf("\tadding generic family: %s\n", mGenericFont->get());
         }
 
         // pixel size
-        printf("\tpixel,twip size: %f,%d\n", mPixelSize, mFont->size);
+        printf("\tpixel,twip size: %f,%d\n", mPixelSize, mFont.size);
 
         // slant type
         printf("\tslant: ");
-        switch(mFont->style) {
+        switch(mFont.style) {
         case NS_FONT_STYLE_ITALIC:
             printf("italic\n");
             break;
@@ -1071,7 +1099,7 @@ nsFontMetricsXft::SetupFCPattern(void)
 
         // weight
         printf("\tweight: (orig,calc) %d,%d\n",
-               mFont->weight, NS_CalculateWeight(mFont->weight));
+               mFont.weight, NS_CalculateWeight(mFont.weight));
 
     }        
 
@@ -1083,11 +1111,11 @@ nsFontMetricsXft::SetupFCPattern(void)
 
     // Add the slant type
     FcPatternAddInteger(mPattern, FC_SLANT,
-                        NS_CalculateSlant(mFont->style));
+                        NS_CalculateSlant(mFont.style));
 
     // Add the weight
     FcPatternAddInteger(mPattern, FC_WEIGHT,
-                        NS_CalculateWeight(mFont->weight));
+                        NS_CalculateWeight(mFont.weight));
 
     // Set up the default substitutions for this font
     FcConfigSubstitute(0, mPattern, FcMatchPattern);
@@ -1231,9 +1259,8 @@ nsFontMetricsXft::SetupMiniFont(void)
 
     FcPattern *pattern = nsnull;
     XftFont *font = nsnull;
-    XftFont *xftFont = mWesternFont->GetXftFont();
-    if (!xftFont)
-        return NS_ERROR_NOT_AVAILABLE;
+    XftFont *xftFont = mWesternFont->mXftFont;
+    NS_ASSERTION(xftFont, "FindFont returned a bad font");
 
     mMiniFontAscent = xftFont->ascent;
     mMiniFontDescent = xftFont->descent;
@@ -1250,7 +1277,7 @@ nsFontMetricsXft::SetupMiniFont(void)
     FcPatternAddInteger(pattern, FC_PIXEL_SIZE, int(0.5 * mPixelSize));
 
     FcPatternAddInteger(pattern, FC_WEIGHT,
-                        NS_CalculateWeight(mFont->weight));
+                        NS_CalculateWeight(mFont.weight));
 
     FcConfigSubstitute(0, pattern, FcMatchPattern);
     XftDefaultSubstitute(GDK_DISPLAY(), DefaultScreen(GDK_DISPLAY()),
@@ -1666,10 +1693,8 @@ nsresult
 nsFontMetricsXft::FamilyExists(nsIDeviceContext *aDevice,
                                const nsString &aName)
 {
-    if (!NS_IsASCIIFontName(aName))
-        return NS_ERROR_FAILURE;
-
-    NS_ConvertUCS2toUTF8 name(aName);
+    // fontconfig family name is always in UTF-8
+    NS_ConvertUTF16toUTF8 name(aName);
 
     FcFontSet *set = nsnull;
     FcObjectSet *os = nsnull;
@@ -1682,7 +1707,7 @@ nsFontMetricsXft::FamilyExists(nsIDeviceContext *aDevice,
     
     // Build a list of familes and walk the list looking to see if we
     // have it.
-    os = FcObjectSetBuild(FC_FAMILY, 0);
+    os = FcObjectSetBuild(FC_FAMILY, NULL);
     if (!os)
         goto end;
 
@@ -1722,13 +1747,12 @@ PRBool
 nsFontMetricsXft::EnumFontCallback(const nsString &aFamily, PRBool aIsGeneric,
                                    void *aData)
 {
-    // make sure it's an ascii name, if not then return and continue
-    // enumerating
-    if (!NS_IsASCIIFontName(aFamily))
-        return PR_TRUE;
+    NS_ConvertUTF16toUTF8 name(aFamily);
 
-    nsCAutoString name;
-    name.AssignWithConversion(aFamily.get());
+    // The newest fontconfig does the full Unicode case folding so that 
+    // we're being lazy here by calling |ToLowerCase| after converting
+    // to UTF-8  assuming that in virtually all cases, we just have to
+    // fold [A-Z].  (bug 223653). 
     ToLowerCase(name);
     nsFontMetricsXft *metrics = (nsFontMetricsXft *)aData;
     metrics->mFontList.AppendCString(name);
@@ -1927,8 +1951,7 @@ nsresult
 nsFontXft::GetTextExtents32(const FcChar32 *aString, PRUint32 aLen, 
                             XGlyphInfo &aGlyphInfo)
 {
-    if (!mXftFont && !GetXftFont())
-            return NS_ERROR_NOT_AVAILABLE;
+    NS_PRECONDITION(mXftFont, "FindFont should not return bad fonts");
 
     // NS_CONST_CAST needed for older versions of Xft
     XftTextExtents32(GDK_DISPLAY(), mXftFont,
@@ -1970,18 +1993,14 @@ nsFontXft::GetBoundingMetrics32(const FcChar32*    aString,
 PRInt16
 nsFontXft::GetMaxAscent(void)
 {
-    if (!mXftFont && !GetXftFont())
-            return 0;
-
+    NS_PRECONDITION(mXftFont, "FindFont should not return bad fonts");
     return mXftFont->ascent;
 }
 
 PRInt16
 nsFontXft::GetMaxDescent(void)
 {
-    if (!mXftFont && !GetXftFont())
-            return 0;
-
+    NS_PRECONDITION(mXftFont, "FindFont should not return bad fonts");
     return mXftFont->descent;
 }
 
@@ -1995,11 +2014,9 @@ nsFontXft::CharToGlyphIndex(FcChar32 aChar)
 nsresult
 nsFontXft::DrawStringSpec(FcChar32 *aString, PRUint32 aLen, void *aData)
 {
+    NS_PRECONDITION(mXftFont, "FindFont should not return bad fonts");
     DrawStringData *data = (DrawStringData *)aData;
 
-    if (!mXftFont && !GetXftFont())
-            return NS_ERROR_NOT_AVAILABLE;
-
     FcChar32 *pstr = aString;
     const FcChar32 *end = aString + aLen;
 
@@ -2058,6 +2075,8 @@ nsresult
 nsFontXftCustom::GetTextExtents32(const FcChar32 *aString, PRUint32 aLen, 
                                   XGlyphInfo &aGlyphInfo)
 {
+    NS_PRECONDITION(mXftFont, "FindFont should not return bad fonts");
+
     nsAutoFcChar32Buffer buffer;
     nsresult rv;
     PRUint32 destLen = aLen;
@@ -2071,9 +2090,6 @@ nsFontXftCustom::GetTextExtents32(const FcChar32 *aString, PRUint32 aLen,
       
     FcChar32 *str = buffer.get();
 
-    if (!mXftFont && !GetXftFont())
-            return NS_ERROR_NOT_AVAILABLE;
-
     // short cut for the common case
     if (isWide) { 
         XftTextExtents32(GDK_DISPLAY(), mXftFont, str, destLen, &aGlyphInfo);
@@ -2117,6 +2133,8 @@ nsresult
 nsFontXftCustom::DrawStringSpec(FcChar32* aString, PRUint32 aLen,
                                 void* aData)
 {
+    NS_PRECONDITION(mXftFont, "FindFont should not return bad fonts");
+
     nsresult rv = NS_OK;
     nsAutoFcChar32Buffer buffer;
     PRUint32 destLen = aLen;
@@ -2126,9 +2144,6 @@ nsFontXftCustom::DrawStringSpec(FcChar32* aString, PRUint32 aLen,
                              isWide, buffer);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (!mXftFont && !GetXftFont())
-            return NS_ERROR_NOT_AVAILABLE;
-
     if (!isWide) {
         // For some narrow fonts(Mathematica, Symbol, and MTExtra),  
         // what we get back  after the conversion  is in the encoding 
@@ -2149,8 +2164,7 @@ nsFontXftCustom::DrawStringSpec(FcChar32* aString, PRUint32 aLen,
 nsresult
 nsFontXftCustom::SetFT_FaceCharmap(void)
 {
-    if (!mXftFont && !GetXftFont())
-            return NS_ERROR_NOT_AVAILABLE;
+    NS_PRECONDITION(mXftFont, "FindFont should not return bad fonts");
 
     if (mFT_Face)
         return NS_OK;
@@ -2174,6 +2188,14 @@ nsAutoDrawSpecBuffer::Draw(nscoord x, nscoord y, XftFont *font, FT_UInt glyph)
     if (mSpecPos >= BUFFER_LEN-1)
         Flush();
 
+    // Make sure the coordinates fit into the 16-bit x and y fields of
+    // XftGlyphFontSpec
+    NS_ASSERTION(sizeof(short) == sizeof(mSpecBuffer[mSpecPos].x), "Unexpected coordinate type");
+    if ((short)x != x || (short)y != y){
+        NS_WARNING("ignoring coordinate overflow");
+        return;
+    }
+
     mSpecBuffer[mSpecPos].x = x;
     mSpecBuffer[mSpecPos].y = y;
     mSpecBuffer[mSpecPos].font = font;
@@ -2236,7 +2258,7 @@ EnumFontsXft(nsIAtom* aLangGroup, const char* aGeneric,
     if (!pat)
         goto end;
 
-    os = FcObjectSetBuild(FC_FAMILY, FC_FOUNDRY, 0);
+    os = FcObjectSetBuild(FC_FAMILY, FC_FOUNDRY, NULL);
     if (!os)
         goto end;
 
@@ -2250,11 +2272,6 @@ EnumFontsXft(nsIAtom* aLangGroup, const char* aGeneric,
     if (!fs)
         goto end;
 
-    if (!fs->nfont) {
-        rv = NS_OK;
-        goto end;
-    }
-
     // Fontconfig supports 3 generic fonts, "serif", "sans-serif", and
     // "monospace", slightly different from CSS's 5.
     if (!aGeneric)
@@ -2299,7 +2316,6 @@ EnumFontsXft(nsIAtom* aLangGroup, const char* aGeneric,
 
     for (int i=0; i < fs->nfont; ++i) {
         char *family;
-        PRUnichar *name;
 
         // if there's no family, just move to the next iteration
         if (FcPatternGetString (fs->fonts[i], FC_FAMILY, 0,
@@ -2307,18 +2323,12 @@ EnumFontsXft(nsIAtom* aLangGroup, const char* aGeneric,
             continue;
         }
 
-        name = NS_STATIC_CAST(PRUnichar *,
-                              nsMemory::Alloc ((strlen (family) + 1)
-                                               * sizeof (PRUnichar)));
+        // fontconfig always returns family names in UTF-8
+        PRUnichar* name =  UTF8ToNewUnicode(nsDependentCString(family));
 
         if (!name)
             goto end;
 
-        PRUnichar *r = name;
-        for (char *f = family; *f; ++f)
-            *r++ = *f;
-        *r = '\0';
-
         array[narray++] = name;
     }
 
