@@ -205,11 +205,19 @@ NS_IMETHODIMP nsRenderingContextGTK::CommonInit()
   return NS_OK;
 }
 
+void*
+nsRenderingContextGTK::GetNativeGraphicData(GraphicDataType aType)
+{
+  if (aType == NATIVE_GDK_DRAWABLE)
+    return mSurface->GetDrawable();
+  return nsnull;
+}
+
 NS_IMETHODIMP nsRenderingContextGTK::GetHints(PRUint32& aResult)
 {
   PRUint32 result = 0;
 
-  // Most X servers implement 8 bit text rendering alot faster than
+  // Most X servers implement 8 bit text rendering a lot faster than
   // XChar2b rendering. In addition, we can avoid the PRUnichar to
   // XChar2b conversion. So we set this bit...
   result |= NS_RENDERING_HINT_FAST_8BIT_TEXT;
@@ -1211,21 +1219,8 @@ nsRenderingContextGTK::GetWidth(PRUnichar aC, nscoord& aWidth,
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::GetWidth(const nsString& aString,
-                                nscoord& aWidth, PRInt32* aFontID)
-{
-  return GetWidth(aString.get(), aString.Length(), aWidth, aFontID);
-}
-
-NS_IMETHODIMP
-nsRenderingContextGTK::GetWidth(const char* aString, nscoord& aWidth)
-{
-  return GetWidth(aString, strlen(aString), aWidth);
-}
-
-NS_IMETHODIMP
-nsRenderingContextGTK::GetWidth(const char* aString, PRUint32 aLength,
-                                nscoord& aWidth)
+nsRenderingContextGTK::GetWidthInternal(const char* aString, PRUint32 aLength,
+                                        nscoord& aWidth)
 {
   if (0 == aLength) {
     aWidth = 0;
@@ -1238,8 +1233,8 @@ nsRenderingContextGTK::GetWidth(const char* aString, PRUint32 aLength,
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::GetWidth(const PRUnichar* aString, PRUint32 aLength,
-                                nscoord& aWidth, PRInt32* aFontID)
+nsRenderingContextGTK::GetWidthInternal(const PRUnichar* aString, PRUint32 aLength,
+                                        nscoord& aWidth, PRInt32* aFontID)
 {
   if (0 == aLength) {
     aWidth = 0;
@@ -1252,8 +1247,8 @@ nsRenderingContextGTK::GetWidth(const PRUnichar* aString, PRUint32 aLength,
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::GetTextDimensions(const char* aString, PRUint32 aLength,
-                                         nsTextDimensions& aDimensions)
+nsRenderingContextGTK::GetTextDimensionsInternal(const char* aString, PRUint32 aLength,
+                                                 nsTextDimensions& aDimensions)
 {
   mFontMetrics->GetMaxAscent(aDimensions.ascent);
   mFontMetrics->GetMaxDescent(aDimensions.descent);
@@ -1261,25 +1256,25 @@ nsRenderingContextGTK::GetTextDimensions(const char* aString, PRUint32 aLength,
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::GetTextDimensions(const PRUnichar* aString,
-                                         PRUint32 aLength,
-                                         nsTextDimensions& aDimensions, 
-                                         PRInt32* aFontID)
+nsRenderingContextGTK::GetTextDimensionsInternal(const PRUnichar* aString,
+                                                 PRUint32 aLength,
+                                                 nsTextDimensions& aDimensions, 
+                                                 PRInt32* aFontID)
 {
   return mFontMetrics->GetTextDimensions(aString, aLength, aDimensions,
                                          aFontID, this);
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::GetTextDimensions(const char*       aString,
-                                         PRInt32           aLength,
-                                         PRInt32           aAvailWidth,
-                                         PRInt32*          aBreaks,
-                                         PRInt32           aNumBreaks,
-                                         nsTextDimensions& aDimensions,
-                                         PRInt32&          aNumCharsFit,
-                                         nsTextDimensions& aLastWordDimensions,
-                                         PRInt32*          aFontID)
+nsRenderingContextGTK::GetTextDimensionsInternal(const char*       aString,
+                                                 PRInt32           aLength,
+                                                 PRInt32           aAvailWidth,
+                                                 PRInt32*          aBreaks,
+                                                 PRInt32           aNumBreaks,
+                                                 nsTextDimensions& aDimensions,
+                                                 PRInt32&          aNumCharsFit,
+                                                 nsTextDimensions& aLastWordDimensions,
+                                                 PRInt32*          aFontID)
 {
   return mFontMetrics->GetTextDimensions(aString, aLength, aAvailWidth,
                                          aBreaks, aNumBreaks, aDimensions,
@@ -1288,15 +1283,15 @@ nsRenderingContextGTK::GetTextDimensions(const char*       aString,
                                          this);
 }
 NS_IMETHODIMP
-nsRenderingContextGTK::GetTextDimensions(const PRUnichar*  aString,
-                                         PRInt32           aLength,
-                                         PRInt32           aAvailWidth,
-                                         PRInt32*          aBreaks,
-                                         PRInt32           aNumBreaks,
-                                         nsTextDimensions& aDimensions,
-                                         PRInt32&          aNumCharsFit,
-                                         nsTextDimensions& aLastWordDimensions,
-                                         PRInt32*          aFontID)
+nsRenderingContextGTK::GetTextDimensionsInternal(const PRUnichar*  aString,
+                                                 PRInt32           aLength,
+                                                 PRInt32           aAvailWidth,
+                                                 PRInt32*          aBreaks,
+                                                 PRInt32           aNumBreaks,
+                                                 nsTextDimensions& aDimensions,
+                                                 PRInt32&          aNumCharsFit,
+                                                 nsTextDimensions& aLastWordDimensions,
+                                                 PRInt32*          aFontID)
 {
   return mFontMetrics->GetTextDimensions(aString, aLength, aAvailWidth,
                                          aBreaks, aNumBreaks, aDimensions,
@@ -1306,34 +1301,24 @@ nsRenderingContextGTK::GetTextDimensions(const PRUnichar*  aString,
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::DrawString(const char *aString, PRUint32 aLength,
-                                  nscoord aX, nscoord aY,
-                                  const nscoord* aSpacing)
+nsRenderingContextGTK::DrawStringInternal(const char *aString, PRUint32 aLength,
+                                          nscoord aX, nscoord aY,
+                                          const nscoord* aSpacing)
 {
   return mFontMetrics->DrawString(aString, aLength, aX, aY, aSpacing,
                                   this, mSurface);
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::DrawString(const PRUnichar* aString, PRUint32 aLength,
-                                  nscoord aX, nscoord aY,
-                                  PRInt32 aFontID,
-                                  const nscoord* aSpacing)
+nsRenderingContextGTK::DrawStringInternal(const PRUnichar* aString, PRUint32 aLength,
+                                          nscoord aX, nscoord aY,
+                                          PRInt32 aFontID,
+                                          const nscoord* aSpacing)
 {
   return mFontMetrics->DrawString(aString, aLength, aX, aY, aFontID,
                                   aSpacing, this, mSurface);
 }
 
-NS_IMETHODIMP
-nsRenderingContextGTK::DrawString(const nsString& aString,
-                                  nscoord aX, nscoord aY,
-                                  PRInt32 aFontID,
-                                  const nscoord* aSpacing)
-{
-  return DrawString(aString.get(), aString.Length(),
-                    aX, aY, aFontID, aSpacing);
-}
-
 NS_IMETHODIMP
 nsRenderingContextGTK::CopyOffScreenBits(nsIDrawingSurface* aSrcSurf,
                                          PRInt32 aSrcX, PRInt32 aSrcY,
@@ -1385,7 +1370,7 @@ nsRenderingContextGTK::CopyOffScreenBits(nsIDrawingSurface* aSrcSurf,
     mTranMatrix->TransformCoord(&drect.x, &drect.y, &drect.width, &drect.height);
 
 #if 0
-  // XXX impliment me
+  // XXX implement me
   if (aCopyFlags & NS_COPYBITS_USE_SOURCE_CLIP_REGION)
   {
     // we should use the source clip region if this flag is used...
@@ -1413,27 +1398,22 @@ nsRenderingContextGTK::CopyOffScreenBits(nsIDrawingSurface* aSrcSurf,
   return NS_OK;
 }
 
-NS_IMETHODIMP nsRenderingContextGTK::RetrieveCurrentNativeGraphicData(PRUint32 * ngd)
-{
-  return NS_OK;
-}
-
 #ifdef MOZ_MATHML
 
 NS_IMETHODIMP
-nsRenderingContextGTK::GetBoundingMetrics(const char*        aString, 
-                                          PRUint32           aLength,
-                                          nsBoundingMetrics& aBoundingMetrics)
+nsRenderingContextGTK::GetBoundingMetricsInternal(const char*        aString, 
+                                                  PRUint32           aLength,
+                                                  nsBoundingMetrics& aBoundingMetrics)
 {
   return mFontMetrics->GetBoundingMetrics(aString, aLength, aBoundingMetrics,
                                           this);
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::GetBoundingMetrics(const PRUnichar*   aString, 
-                                          PRUint32           aLength,
-                                          nsBoundingMetrics& aBoundingMetrics,
-                                          PRInt32*           aFontID)
+nsRenderingContextGTK::GetBoundingMetricsInternal(const PRUnichar*   aString, 
+                                                  PRUint32           aLength,
+                                                  nsBoundingMetrics& aBoundingMetrics,
+                                                  PRInt32*           aFontID)
 {
   return mFontMetrics->GetBoundingMetrics(aString, aLength, aBoundingMetrics,
                                           aFontID, this);
@@ -1446,17 +1426,60 @@ NS_IMETHODIMP nsRenderingContextGTK::SetRightToLeftText(PRBool aIsRTL)
   return mFontMetrics->SetRightToLeftText(aIsRTL);
 }
 
+NS_IMETHODIMP nsRenderingContextGTK::GetRightToLeftText(PRBool* aIsRTL)
+{
+  *aIsRTL = mFontMetrics->GetRightToLeftText();
+  return NS_OK;
+}
+
+PRInt32 nsRenderingContextGTK::GetMaxStringLength()
+{
+  if (!mFontMetrics)
+    return 1;
+  return mFontMetrics->GetMaxStringLength();
+}
+
+NS_IMETHODIMP nsRenderingContextGTK::GetClusterInfo(const PRUnichar *aText,
+                                                    PRUint32 aLength,
+                                                    PRUint8 *aClusterStarts)
+{
+  return mFontMetrics->GetClusterInfo(aText, aLength, aClusterStarts);
+}
+
+PRInt32 nsRenderingContextGTK::GetPosition(const PRUnichar *aText, PRUint32 aLength,
+                                           nsPoint aPt)
+{
+  return mFontMetrics->GetPosition(aText, aLength, aPt);
+}
+
+NS_IMETHODIMP nsRenderingContextGTK::GetRangeWidth(const PRUnichar *aText, PRUint32 aLength,
+                                                   PRUint32 aStart, PRUint32 aEnd,
+                                                   PRUint32 &aWidth)
+{
+  return mFontMetrics->GetRangeWidth(aText, aLength, aStart, aEnd, aWidth);
+}
+
+NS_IMETHODIMP nsRenderingContextGTK::GetRangeWidth(const char *aText, PRUint32 aLength,
+                                                   PRUint32 aStart, PRUint32 aEnd,
+                                                   PRUint32 &aWidth)
+{
+  return mFontMetrics->GetRangeWidth(aText, aLength, aStart, aEnd, aWidth);
+}
+
 NS_IMETHODIMP nsRenderingContextGTK::DrawImage(imgIContainer *aImage, const nsRect & aSrcRect, const nsRect & aDestRect)
 {
   UpdateGC();
   return nsRenderingContextImpl::DrawImage(aImage, aSrcRect, aDestRect);
 }
 
-NS_IMETHODIMP nsRenderingContextGTK::GetBackbuffer(const nsRect &aRequestedSize, const nsRect &aMaxSize, nsIDrawingSurface* &aBackbuffer)
+NS_IMETHODIMP nsRenderingContextGTK::GetBackbuffer(const nsRect &aRequestedSize,
+                                                   const nsRect &aMaxSize,
+                                                   PRBool aForBlending,
+                                                   nsIDrawingSurface* &aBackbuffer)
 {
   // Do not cache the backbuffer. On GTK it is more efficient to allocate
   // the backbuffer as needed and it doesn't cause a performance hit. @see bug 95952
-  return AllocateBackbuffer(aRequestedSize, aMaxSize, aBackbuffer, PR_FALSE);
+  return AllocateBackbuffer(aRequestedSize, aMaxSize, aBackbuffer, PR_FALSE, 0);
 }
  
 NS_IMETHODIMP nsRenderingContextGTK::ReleaseBackbuffer(void) {
