@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
+ *   Mark Mentovai <mark@moxienet.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -37,6 +38,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsIInterfaceRequestorUtils.h" 
+#include "nsIServiceManager.h"
 #include "nsRenderingContextMac.h"
 #include "nsDeviceContextMac.h"
 #include "nsFontMetricsMac.h"
@@ -190,7 +192,7 @@ void nsRenderingContextMac::SelectDrawingSurface(nsDrawingSurfaceMac* aSurface,
 		return;
 
   NS_ASSERTION(ValidateDrawingState(), "Bad drawing state");
-	
+
 	// if surface is changing, be extra conservative about graphic state changes.
 	if (mCurrentSurface != aSurface)
 	{
@@ -593,7 +595,7 @@ NS_IMETHODIMP nsRenderingContextMac::SetClipRect(const nsRect& aRect, nsClipComb
 	Rect macRect;
 	::SetRect(&macRect, trect.x, trect.y, trect.x + trect.width, trect.y + trect.height);
 
-	RgnHandle rectRgn = sNativeRegionPool.GetNewRegion();
+	StRegionFromPool rectRgn;
 	RgnHandle clipRgn = mGS->mClipRegion;
 	if (!clipRgn || !rectRgn)
 		return NS_ERROR_OUT_OF_MEMORY;
@@ -618,7 +620,6 @@ NS_IMETHODIMP nsRenderingContextMac::SetClipRect(const nsRect& aRect, nsClipComb
 		::SectRgn(rectRgn, mGS->mMainRegion, clipRgn);
 		break;
 	}
-	sNativeRegionPool.ReleaseRegion(rectRgn);
 
 	{
 		StPortSetter setter(mPort);
@@ -767,17 +768,16 @@ NS_IMETHODIMP nsRenderingContextMac::GetColor(nscolor &aColor) const
 
 NS_IMETHODIMP nsRenderingContextMac::SetLineStyle(nsLineStyle aLineStyle)
 {
-	// note: the line style must be saved in the nsGraphicState like font, color, etc...
-  mLineStyle = aLineStyle;
-	return NS_OK;
+  mGS->mLineStyle = aLineStyle;
+  return NS_OK;
 }
 
 //------------------------------------------------------------------------
 
 NS_IMETHODIMP nsRenderingContextMac::GetLineStyle(nsLineStyle &aLineStyle)
 {
-  aLineStyle = mLineStyle;
-	return NS_OK;
+  aLineStyle = mGS->mLineStyle;
+  return NS_OK;
 }
 
 
@@ -843,12 +843,56 @@ NS_IMETHODIMP nsRenderingContextMac::GetCurrentTransform(nsTransform2D *&aTransf
 
 
 #pragma mark -
+
+// 0 1 2 3 4 5 6 7
+// *   *   *   *  
+//   *   *   *   *
+// *   *   *   *  
+//   *   *   *   *
+// *   *   *   *  
+//   *   *   *   *
+// *   *   *   *  
+//   *   *   *   *
+static const Pattern dottedPattern = {0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55};
+
+// 0 1 2 3 4 5 6 7
+// * * * *        
+//   * * * *      
+//     * * * *    
+//       * * * *  
+//         * * * *
+// *         * * *
+// * *         * *
+// * * *         *
+static const Pattern dashedPattern = {0xf0,0x78,0x3c,0x1e,0x0f,0x87,0xc3,0xe1};
+
 //------------------------------------------------------------------------
 
 NS_IMETHODIMP nsRenderingContextMac::DrawLine(nscoord aX0, nscoord aY0, nscoord aX1, nscoord aY1)
 {
+  if (mGS->mLineStyle == nsLineStyle_kNone)
+    return NS_OK;
+
   SetupPortState();
 
+  PenState savedPenState;
+  RGBColor savedBGColor;
+  if (mGS->mLineStyle == nsLineStyle_kDotted ||
+      mGS->mLineStyle == nsLineStyle_kDashed) {
+    ::GetPenState(&savedPenState);
+    ::GetBackColor(&savedBGColor);
+
+    ::PenMode(transparent);
+    if (mGS->mLineStyle == nsLineStyle_kDashed)
+      ::PenPat(&dashedPattern);
+    else
+      ::PenPat(&dottedPattern);
+    RGBColor invertedForeColor;
+    ::GetForeColor(&invertedForeColor);
+    ::InvertColor(&invertedForeColor);
+    ::RGBBackColor(&invertedForeColor);
+  }
+
 	mGS->mTMatrix.TransformCoord(&aX0,&aY0);
 	mGS->mTMatrix.TransformCoord(&aX1,&aY1);
 
@@ -861,35 +905,66 @@ NS_IMETHODIMP nsRenderingContextMac::DrawLine(nscoord aX0, nscoord aY0, nscoord
 	if (diffY)
 		diffY -= (diffY > 0 ? 1 : -1);
 
-	// draw line
-	::MoveTo(aX0, aY0);
-	::Line(diffX, diffY);
+  ::MoveTo(aX0, aY0);
+  ::Line(diffX, diffY);
+
+  if (mGS->mLineStyle == nsLineStyle_kDotted ||
+      mGS->mLineStyle == nsLineStyle_kDashed) {
+    ::PenMode(savedPenState.pnMode);
+    ::PenPat(&savedPenState.pnPat);
+    ::RGBBackColor(&savedBGColor);
+  }
 
 	return NS_OK;
 }
 
-
 //------------------------------------------------------------------------
 
 NS_IMETHODIMP nsRenderingContextMac::DrawPolyline(const nsPoint aPoints[], PRInt32 aNumPoints)
 {
-	SetupPortState();
+  if (mGS->mLineStyle == nsLineStyle_kNone)
+    return NS_OK;
 
-	PRUint32   i;
-	PRInt32    x,y;
+  SetupPortState();
 
-	x = aPoints[0].x;
-	y = aPoints[0].y;
-	mGS->mTMatrix.TransformCoord((PRInt32*)&x,(PRInt32*)&y);
-	::MoveTo(x,y);
+  PenState savedPenState;
+  RGBColor savedBGColor;
+  if (mGS->mLineStyle == nsLineStyle_kDotted ||
+      mGS->mLineStyle == nsLineStyle_kDashed) {
+    ::GetPenState(&savedPenState);
+    ::GetBackColor(&savedBGColor);
 
-	for (i = 1; i < aNumPoints; i++){
-		x = aPoints[i].x;
-		y = aPoints[i].y;
+    ::PenMode(transparent);
+    if (mGS->mLineStyle == nsLineStyle_kDashed)
+      ::PenPat(&dashedPattern);
+    else
+      ::PenPat(&dottedPattern);
+    RGBColor invertedForeColor;
+    ::GetForeColor(&invertedForeColor);
+    ::InvertColor(&invertedForeColor);
+    ::RGBBackColor(&invertedForeColor);
+  }
 
-		mGS->mTMatrix.TransformCoord((PRInt32*)&x,(PRInt32*)&y);
-		::LineTo(x,y);
-	}
+  PRInt32 x,y;
+
+  x = aPoints[0].x;
+  y = aPoints[0].y;
+  mGS->mTMatrix.TransformCoord((PRInt32*)&x,(PRInt32*)&y);
+
+  ::MoveTo(x,y);
+  for (PRInt32 i = 1; i < aNumPoints; i++) {
+    x = aPoints[i].x;
+    y = aPoints[i].y;
+    mGS->mTMatrix.TransformCoord((PRInt32*)&x,(PRInt32*)&y);
+    ::LineTo(x,y);
+  }
+
+  if (mGS->mLineStyle == nsLineStyle_kDotted ||
+      mGS->mLineStyle == nsLineStyle_kDashed) {
+    ::PenMode(savedPenState.pnMode);
+    ::PenPat(&savedPenState.pnPat);
+    ::RGBBackColor(&savedBGColor);
+  }
 
 	return NS_OK;
 }
@@ -975,7 +1050,6 @@ NS_IMETHODIMP nsRenderingContextMac::DrawPolygon(const nsPoint aPoints[], PRInt3
 {
 	SetupPortState();
 
-	PRUint32   i;
 	PolyHandle thepoly;
 	PRInt32    x,y;
 
@@ -989,7 +1063,7 @@ NS_IMETHODIMP nsRenderingContextMac::DrawPolygon(const nsPoint aPoints[], PRInt3
 	mGS->mTMatrix.TransformCoord((PRInt32*)&x,(PRInt32*)&y);
 	::MoveTo(x,y);
 
-	for (i = 1; i < aNumPoints; i++) {
+	for (PRInt32 i = 1; i < aNumPoints; i++) {
 		x = aPoints[i].x;
 		y = aPoints[i].y;
 		
@@ -1011,7 +1085,6 @@ NS_IMETHODIMP nsRenderingContextMac::FillPolygon(const nsPoint aPoints[], PRInt3
 {
 	SetupPortState();
 
-	PRUint32   i;
 	PolyHandle thepoly;
 	PRInt32    x,y;
 
@@ -1025,7 +1098,7 @@ NS_IMETHODIMP nsRenderingContextMac::FillPolygon(const nsPoint aPoints[], PRInt3
 	mGS->mTMatrix.TransformCoord((PRInt32*)&x,(PRInt32*)&y);
 	::MoveTo(x,y);
 
-	for (i = 1; i < aNumPoints; i++) {
+	for (PRInt32 i = 1; i < aNumPoints; i++) {
 		x = aPoints[i].x;
 		y = aPoints[i].y;
 		mGS->mTMatrix.TransformCoord((PRInt32*)&x,(PRInt32*)&y);
@@ -1120,7 +1193,7 @@ NS_IMETHODIMP nsRenderingContextMac::DrawArc(nscoord aX, nscoord aY, nscoord aWi
 
 	mGS->mTMatrix.TransformCoord(&x,&y,&w,&h);
 	::SetRect(&therect, pinToShort(x), pinToShort(y), pinToShort(x + w), pinToShort(y + h));
-	::FrameArc(&therect,aStartAngle,aEndAngle);
+	::FrameArc(&therect, (short)aStartAngle, (short)aEndAngle);
 
 	return NS_OK;
 }
@@ -1150,11 +1223,18 @@ NS_IMETHODIMP nsRenderingContextMac::FillArc(nscoord aX, nscoord aY, nscoord aWi
 
 	mGS->mTMatrix.TransformCoord(&x,&y,&w,&h);
 	::SetRect(&therect, pinToShort(x), pinToShort(y), pinToShort(x + w), pinToShort(y + h));
-	::PaintArc(&therect,aStartAngle,aEndAngle);
+	::PaintArc(&therect, (short)aStartAngle, (short)aEndAngle);
 
 	return NS_OK;
 }
 
+PRInt32 nsRenderingContextMac::GetMaxStringLength()
+{
+  if (!mGS->mFontMetrics)
+    return 1;
+  return NS_STATIC_CAST(nsFontMetricsMac*, mGS->mFontMetrics)->GetMaxStringLength();
+}
+
 #pragma mark -
 //------------------------------------------------------------------------
 
@@ -1184,22 +1264,8 @@ NS_IMETHODIMP nsRenderingContextMac::GetWidth(PRUnichar ch, nscoord &aWidth, PRI
 
 //------------------------------------------------------------------------
 
-NS_IMETHODIMP nsRenderingContextMac::GetWidth(const nsString& aString, nscoord &aWidth, PRInt32 *aFontID)
-{
-	return GetWidth(aString.get(), aString.Length(), aWidth, aFontID);
-}
-
-//------------------------------------------------------------------------
-
-NS_IMETHODIMP nsRenderingContextMac::GetWidth(const char *aString, nscoord &aWidth)
-{
-	return GetWidth(aString, strlen(aString), aWidth);
-}
-
-//------------------------------------------------------------------------
-
 NS_IMETHODIMP
-nsRenderingContextMac::GetWidth(const char* aString, PRUint32 aLength, nscoord& aWidth)
+nsRenderingContextMac::GetWidthInternal(const char* aString, PRUint32 aLength, nscoord& aWidth)
 {
 	SetupPortState();
 
@@ -1218,7 +1284,7 @@ nsRenderingContextMac::GetWidth(const char* aString, PRUint32 aLength, nscoord&
 
 //------------------------------------------------------------------------
 
-NS_IMETHODIMP nsRenderingContextMac::GetWidth(const PRUnichar *aString, PRUint32 aLength, nscoord &aWidth, PRInt32 *aFontID)
+NS_IMETHODIMP nsRenderingContextMac::GetWidthInternal(const PRUnichar *aString, PRUint32 aLength, nscoord &aWidth, PRInt32 *aFontID)
 {
 	SetupPortState();
 	
@@ -1238,8 +1304,8 @@ NS_IMETHODIMP nsRenderingContextMac::GetWidth(const PRUnichar *aString, PRUint32
 //------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsRenderingContextMac::GetTextDimensions(const char* aString, PRUint32 aLength,
-                                        nsTextDimensions& aDimensions)
+nsRenderingContextMac::GetTextDimensionsInternal(const char* aString, PRUint32 aLength,
+                                                 nsTextDimensions& aDimensions)
 {
   nsresult rv= GetWidth(aString, aLength, aDimensions.width);
   if (NS_SUCCEEDED(rv) && (mGS->mFontMetrics))
@@ -1251,8 +1317,8 @@ nsRenderingContextMac::GetTextDimensions(const char* aString, PRUint32 aLength,
 }
 
 NS_IMETHODIMP
-nsRenderingContextMac::GetTextDimensions(const PRUnichar* aString, PRUint32 aLength,
-                                         nsTextDimensions& aDimensions, PRInt32* aFontID)
+nsRenderingContextMac::GetTextDimensionsInternal(const PRUnichar* aString, PRUint32 aLength,
+                                                 nsTextDimensions& aDimensions, PRInt32* aFontID)
 {
   SetupPortState();
   
@@ -1272,9 +1338,9 @@ nsRenderingContextMac::GetTextDimensions(const PRUnichar* aString, PRUint32 aLen
 #pragma mark -
 //------------------------------------------------------------------------
 
-NS_IMETHODIMP nsRenderingContextMac::DrawString(const char *aString, PRUint32 aLength,
-                                         nscoord aX, nscoord aY,
-                                         const nscoord* aSpacing)
+NS_IMETHODIMP nsRenderingContextMac::DrawStringInternal(const char *aString, PRUint32 aLength,
+                                                        nscoord aX, nscoord aY,
+                                                        const nscoord* aSpacing)
 {
 	SetupPortState();
 
@@ -1299,7 +1365,7 @@ NS_IMETHODIMP nsRenderingContextMac::DrawString(const char *aString, PRUint32 aL
 		{
 			mGS->mTMatrix.ScaleXCoords(aSpacing, aLength, spacing);
 			PRInt32 currentX = x;
-			for (PRInt32 i = 0; i < aLength; i++)
+			for (PRUint32 i = 0; i < aLength; i++)
 			{
 				::DrawChar(aString[i]);
 				currentX += spacing[i];
@@ -1319,9 +1385,9 @@ NS_IMETHODIMP nsRenderingContextMac::DrawString(const char *aString, PRUint32 aL
 
 
 //------------------------------------------------------------------------
-NS_IMETHODIMP nsRenderingContextMac::DrawString(const PRUnichar *aString, PRUint32 aLength,
-                                         nscoord aX, nscoord aY, PRInt32 aFontID,
-                                         const nscoord* aSpacing)
+NS_IMETHODIMP nsRenderingContextMac::DrawStringInternal(const PRUnichar *aString, PRUint32 aLength,
+                                                        nscoord aX, nscoord aY, PRInt32 aFontID,
+                                                        const nscoord* aSpacing)
 {
 	SetupPortState();
 
@@ -1343,24 +1409,10 @@ NS_IMETHODIMP nsRenderingContextMac::DrawString(const PRUnichar *aString, PRUint
 	return rv;        
 }
 
-//------------------------------------------------------------------------
-
-NS_IMETHODIMP nsRenderingContextMac::DrawString(const nsString& aString,
-                                         nscoord aX, nscoord aY, PRInt32 aFontID,
-                                         const nscoord* aSpacing)
-{
- 	return DrawString(aString.get(), aString.Length(), aX, aY, aFontID, aSpacing);
-}
-
 
 #pragma mark -
 //------------------------------------------------------------------------
 
-NS_IMETHODIMP nsRenderingContextMac::RetrieveCurrentNativeGraphicData(PRUint32 * ngd)
-{
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsRenderingContextMac::InvertRect(const nsRect& aRect)
 {
 	return InvertRect(aRect.x, aRect.y, aRect.width, aRect.height);
@@ -1394,43 +1446,24 @@ nsRenderingContextMac::FlushRect(const nsRect& aRect)
 NS_IMETHODIMP
 nsRenderingContextMac::FlushRect(nscoord aX, nscoord aY, nscoord aWidth, nscoord aHeight)
 {
-#ifdef MOZ_WIDGET_COCOA
-    if (mPort) {
-        SetupPortState();
-
-        nscoord x,y,w,h;
-        Rect	therect;
-    
-        x = aX;
-        y = aY;
-        w = aWidth;
-        h = aHeight;
-    
-        mGS->mTMatrix.TransformCoord(&x, &y, &w, &h);
-        RgnHandle rgn = ::NewRgn();
-        ::SetRectRgn(rgn, pinToShort(x), pinToShort(y), pinToShort(x + w), pinToShort(y + h));
-        ::QDFlushPortBuffer(mPort, rgn);
-        ::DisposeRgn(rgn);
-    }
-#endif
-    return NS_OK;
+  return NS_OK;
 }
 
 #ifdef MOZ_MATHML
 
 NS_IMETHODIMP
-nsRenderingContextMac::GetBoundingMetrics(const char*        aString, 
-                                          PRUint32           aLength,
-                                          nsBoundingMetrics& aBoundingMetrics)
+nsRenderingContextMac::GetBoundingMetricsInternal(const char*        aString, 
+                                                  PRUint32           aLength,
+                                                  nsBoundingMetrics& aBoundingMetrics)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsRenderingContextMac::GetBoundingMetrics(const PRUnichar*   aString, 
-                                          PRUint32           aLength,
-                                          nsBoundingMetrics& aBoundingMetrics,
-                                          PRInt32*           aFontID)
+nsRenderingContextMac::GetBoundingMetricsInternal(const PRUnichar*   aString, 
+                                                  PRUint32           aLength,
+                                                  nsBoundingMetrics& aBoundingMetrics,
+                                                  PRInt32*           aFontID)
 {
   SetupPortState();
   
@@ -1458,6 +1491,29 @@ nsRenderingContextMac::SetRightToLeftText(PRBool aIsRTL)
 	return NS_OK;
 }
 
+NS_IMETHODIMP
+nsRenderingContextMac::GetRightToLeftText(PRBool* aIsRTL)
+{
+  *aIsRTL = mRightToLeftText;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsRenderingContextMac::DrawImage(imgIContainer *aImage, const nsRect & aSrcRect, const nsRect & aDestRect)
+{
+  SetupPortState();
+  return nsRenderingContextImpl::DrawImage(aImage, aSrcRect, aDestRect);
+}
+
+NS_IMETHODIMP
+nsRenderingContextMac::DrawTile(imgIContainer *aImage,
+                    nscoord aXImageStart, nscoord aYImageStart,
+                    const nsRect * aTargetRect)
+{
+  SetupPortState();
+  return nsRenderingContextImpl::DrawTile(aImage, aXImageStart, aYImageStart, aTargetRect);
+}
+
 #pragma mark -
 
 // override to set the port back to the window port
@@ -1479,15 +1535,25 @@ nsRenderingContextMac::UseBackbuffer(PRBool* aUseBackbuffer)
 
 
 PRBool
-nsRenderingContextMac::OnJaguar()
+nsRenderingContextMac::OnTigerOrLater()
 {
   static PRBool sInitVer = PR_FALSE;
-  static PRBool sOnJaguar = PR_FALSE;
+  static PRBool sOnTigerOrLater = PR_FALSE;
   if (!sInitVer) {
     long version;
     OSErr err = ::Gestalt(gestaltSystemVersion, &version);
-    sOnJaguar = (err == noErr && version >= 0x00001020);
+    sOnTigerOrLater = ((err == noErr) && (version >= 0x00001040));
     sInitVer = PR_TRUE;
   }
-  return sOnJaguar;
+  return sOnTigerOrLater;
+}
+
+void*
+nsRenderingContextMac::GetNativeGraphicData(GraphicDataType aType)
+{
+  if (aType == NATIVE_MAC_THING) {
+    return mPort;
+  }
+
+  return nsnull;
 }
