@@ -34,6 +34,7 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+
 #include "nsUnicodeRenderingToolkit.h"
 #include "nsUnicodeFontMappingMac.h"
 #include "nsUnicodeFallbackCache.h"
@@ -53,11 +54,7 @@
 
 #include <FixMath.h>
 
-
-#define BAD_FONT_NUM -1
-#define BAD_SCRIPT 0x7F
 #define STACK_TRESHOLD 1000
-static NS_DEFINE_CID(kSaveAsCharsetCID, NS_SAVEASCHARSET_CID);
 
 //#define DISABLE_TEC_FALLBACK
 //#define DISABLE_PRECOMPOSEHANGUL_FALLBACK
@@ -130,7 +127,7 @@ static NS_DEFINE_CID(kSaveAsCharsetCID, NS_SAVEASCHARSET_CID);
   )
 #define BAD_TEXT_ENCODING 0xFFFFFFFF
 
-// all the chracter should not be drawn if there are no glyph
+// all the character should not be drawn if there are no glyph
 #define IS_ZERO_WIDTH_CHAR(c) ( \
   IN_RANGE(c, 0x200b, 0x200f) || \
   IN_RANGE(c, 0x202a, 0x202e) \
@@ -141,14 +138,14 @@ static NS_DEFINE_CID(kSaveAsCharsetCID, NS_SAVEASCHARSET_CID);
 #define IN_ARABIC_PRESENTATION_B(a) ((0xfe70 <= (a)) && ((a) <= 0xfeff))
 #define IN_ARABIC_PRESENTATION_A_OR_B(a) (IN_ARABIC_PRESENTATION_A(a) || IN_ARABIC_PRESENTATION_B(a))
 
-// we should not ues TEC fallback for characters in latin, greek and cyrillic script
-// because Japanese, Chinese and Korean font have these chracters. If we let them 
-// render in the TEC fallback process, then we will use a Japanese/korean/chinese font
-// to render it even the current font have a glyph in it
+// We should not use TEC fallback for characters in Latin, Greek and Cyrillic scripts
+// because Japanese, Chinese and Korean fonts have these characters. If we let them 
+// render in the TEC fallback process, then we would use a Japanese/Korean/Chinese font
+// to render it even if the current font has a glyph in it.
 // if we skip the TEC fallback, then the ATSUI fallback will try to use the glyph 
 // in the font first (TEC or TEC fallback are using QuickDraw, which can only use 
-// the glyphs that in the font script's encodign. but a lot of TrueType font
-// have houndred more glyph in additional to the font scripts
+// the glyphs that are in the font script's encoding. But a lot of TrueType fonts
+// have hundreds more glyphs in addition to the font scripts.
 #define IS_LATIN(c)  ( IN_RANGE(c, 0x0000, 0x024F) || IN_RANGE(c, 0x1e00, 0x1eff) )
 #define IS_GREEK(c)  ( IN_RANGE(c, 0x0370, 0x03FF) || IN_RANGE(c, 0x1f00, 0x1fff) )
 #define IS_CYRILLIC(c)  IN_RANGE(c, 0x0400, 0x04ff)
@@ -280,8 +277,9 @@ PRBool nsUnicodeRenderingToolkit::TECFallbackGetBoundingMetrics(
             ::TextFont(scriptFallbackFonts[fallbackScript]);
             GetScriptTextBoundingMetrics(buf, outLen, fallbackScript, oBoundingMetrics);
             ::TextFont(fontNum);
+            return PR_TRUE;
         }
-        return PR_TRUE;
+        return PR_FALSE;
     }
     
     for(fallbackScript = 0; fallbackScript < 32; fallbackScript++)
@@ -348,8 +346,8 @@ PRBool nsUnicodeRenderingToolkit :: TECFallbackDrawChar(
         ::TextFont(scriptFallbackFonts[fallbackScript]);
         DrawScriptText(buf, outLen, x, y, oWidth);
         ::TextFont(origFontNum);
+        return PR_TRUE;
     }
-    return PR_TRUE;
   }
   return PR_FALSE;
 }
@@ -432,43 +430,39 @@ nsUnicodeRenderingToolkit::ATSUIFallbackGetDimensions(
                                             aBold, aItalic, aColor);
     }
     if (NS_SUCCEEDED(res)) 
-    {
       return PR_TRUE;
-    }
   }
+
   if (IN_ARABIC_PRESENTATION_A_OR_B(*aCharPt))
   {      
     PRUnichar isolated;
     if (NS_SUCCEEDED( FormAorBIsolated(*aCharPt, info, &isolated))) 
     {
-      if(NS_SUCCEEDED(ATSUIFallbackGetDimensions(&isolated, oDim, origFontNum, 
-                                                 aSize, aBold, aItalic, aColor))) 
-      {
+      if (ATSUIFallbackGetDimensions(&isolated, oDim, origFontNum, 
+                                     aSize, aBold, aItalic, aColor))
          return PR_TRUE;
-      }
     }                                                 
   }
 
   // we know some ATSUI font do not have bold, turn it off and try again
   if (aBold) 
   {
-	  if (NS_SUCCEEDED(ATSUIFallbackGetDimensions(aCharPt, oDim, origFontNum, 
-	                                              aSize, PR_FALSE, aItalic, aColor))) 
-	  {
+	  if (ATSUIFallbackGetDimensions(aCharPt, oDim, origFontNum, 
+	                                 aSize, PR_FALSE, aItalic, aColor))
 	     return PR_TRUE;
-	  }
   }
+
   // we know some ATSUI font do not have italic, turn it off and try again
   if (aItalic) 
   {
-	  if (NS_SUCCEEDED(ATSUIFallbackGetDimensions(aCharPt, oDim, origFontNum, 
-	                                              aSize, PR_FALSE, PR_FALSE, aColor))) 
-	  {
+	  if (ATSUIFallbackGetDimensions(aCharPt, oDim, origFontNum, 
+	                                 aSize, PR_FALSE, PR_FALSE, aColor))
 	     return PR_TRUE;
-	  }
   }
+
   return PR_FALSE;
 }
+
 //------------------------------------------------------------------------
 
 #ifdef MOZ_MATHML
@@ -507,41 +501,36 @@ nsUnicodeRenderingToolkit::ATSUIFallbackGetBoundingMetrics(
                                              aBold, aItalic, aColor);
     }
     if (NS_SUCCEEDED(res))
-    {
       return PR_TRUE;
-    }
   }
+
   if (IN_ARABIC_PRESENTATION_A_OR_B(*aCharPt))
   {  
     PRUnichar isolated;
-    if (NS_SUCCEEDED( FormAorBIsolated(*aCharPt, info, &isolated))) 
+    if (NS_SUCCEEDED(FormAorBIsolated(*aCharPt, info, &isolated)))
     {
-      if(NS_SUCCEEDED(ATSUIFallbackGetBoundingMetrics(&isolated, oBoundingMetrics, origFontNum, 
-                                                      aSize, aBold, aItalic, aColor))) 
-      {
+      if (ATSUIFallbackGetBoundingMetrics(&isolated, oBoundingMetrics, origFontNum, 
+                                          aSize, aBold, aItalic, aColor))
         return PR_TRUE;
-      }
     }                                                 
   }
 
   // we know some ATSUI font do not have bold, turn it off and try again
   if (aBold)
   {
-    if (NS_SUCCEEDED(ATSUIFallbackGetBoundingMetrics(aCharPt, oBoundingMetrics, origFontNum, 
-                                                     aSize, PR_FALSE, aItalic, aColor))) 
-    {
+    if (ATSUIFallbackGetBoundingMetrics(aCharPt, oBoundingMetrics, origFontNum, 
+                                        aSize, PR_FALSE, aItalic, aColor))
       return PR_TRUE;
-    }
   }
+
   // we know some ATSUI font do not have italic, turn it off and try again
   if (aItalic) 
   {
-    if (NS_SUCCEEDED(ATSUIFallbackGetBoundingMetrics(aCharPt, oBoundingMetrics, origFontNum, 
-                                                     aSize, PR_FALSE, PR_FALSE, aColor))) 
-    {
+    if (ATSUIFallbackGetBoundingMetrics(aCharPt, oBoundingMetrics, origFontNum, 
+                                        aSize, PR_FALSE, PR_FALSE, aColor))
       return PR_TRUE;
-    }
   }
+
   return PR_FALSE;
 }
 #endif // MOZ_MATHML
@@ -582,35 +571,33 @@ PRBool nsUnicodeRenderingToolkit :: ATSUIFallbackDrawChar(
     if (NS_SUCCEEDED(res))
       return PR_TRUE;
   }
+
   if (IN_ARABIC_PRESENTATION_A_OR_B(*aCharPt))
   {      
     PRUnichar isolated;
-    if (NS_SUCCEEDED( FormAorBIsolated(*aCharPt, info, &isolated))) {
-      if (NS_SUCCEEDED(ATSUIFallbackDrawChar(&isolated, x, y, oWidth, origFontNum, 
-                                             aSize, aBold, aItalic, aColor))) 
-      {
+    if (NS_SUCCEEDED(FormAorBIsolated(*aCharPt, info, &isolated))) {
+      if (ATSUIFallbackDrawChar(&isolated, x, y, oWidth, origFontNum, 
+                                aSize, aBold, aItalic, aColor))
          return PR_TRUE;
-      }
     }                                                 
   }
+
   // we know some ATSUI font do not have bold, turn it off and try again
   if (aBold)
   {
-    if (NS_SUCCEEDED(ATSUIFallbackDrawChar(aCharPt, x, y, oWidth, origFontNum, 
-                                          aSize, PR_FALSE, aItalic, aColor))) 
-    {
+    if (ATSUIFallbackDrawChar(aCharPt, x, y, oWidth, origFontNum, 
+                              aSize, PR_FALSE, aItalic, aColor))
        return PR_TRUE;
-    }
   }
+
   // we know some ATSUI font do not have italic, turn it off and try again
   if (aItalic)
   {
-    if (NS_SUCCEEDED(ATSUIFallbackDrawChar(aCharPt, x, y, oWidth, origFontNum, 
-                                           aSize, PR_FALSE, PR_FALSE, aColor))) 
-    {
+    if (ATSUIFallbackDrawChar(aCharPt, x, y, oWidth, origFontNum, 
+                              aSize, PR_FALSE, PR_FALSE, aColor))
        return PR_TRUE;
-    }
   }
+
   return PR_FALSE;
 }
 
@@ -692,7 +679,7 @@ PRBool nsUnicodeRenderingToolkit :: LoadTransliterator()
 		return PR_TRUE;
 		
 	nsresult res;
-    mTrans = do_CreateInstance(kSaveAsCharsetCID, &res);
+    mTrans = do_CreateInstance(NS_SAVEASCHARSET_CONTRACTID, &res);
     if ( NS_SUCCEEDED(res) )
     {
        res = mTrans->Init("x-mac-roman",
@@ -833,12 +820,12 @@ PRBool nsUnicodeRenderingToolkit :: TransliterateFallbackDrawChar(
 #define NCount (VCount * TCount)
 static void UnicodePrecomposedHangulTo4EUCKR(PRUnichar in, char *out)
 {
-        static PRUint8 lMap[LCount] = {
+        static const PRUint8 lMap[LCount] = {
           0xa1, 0xa2, 0xa4, 0xa7, 0xa8, 0xa9, 0xb1, 0xb2, 0xb3, 0xb5,
           0xb6, 0xb7, 0xb8, 0xb9, 0xba, 0xbb, 0xbc, 0xbd, 0xbe
         };
 
-        static PRUint8 tMap[TCount] = {
+        static const PRUint8 tMap[TCount] = {
           0xd4, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa9, 0xaa, 
           0xab, 0xac, 0xad, 0xae, 0xaf, 0xb0, 0xb1, 0xb2, 0xb4, 0xb5, 
           0xb6, 0xb7, 0xb8, 0xba, 0xbb, 0xbc, 0xbd, 0xbe
@@ -1245,10 +1232,9 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
       nsTextDimensions& oDim)
 {
   oDim.Clear();
-  if(aLength == 0) 
+  if(aLength == 0 || fontNum == IGNORABLE_FONT_NUM) 
     return NS_OK;
   NS_PRECONDITION(BAD_FONT_NUM != fontNum, "illegal font num");
-  short textWidth = 0;
   PRUint32 processLen = 0;
   char *heapBuf = nsnull;
   PRUint32 heapBufSize = 0;
@@ -1319,8 +1305,7 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
           ((processLen+1) < aLength) &&
           IS_LOW_SURROGATE(*(aString+1)))
       {
-         const nsFont *font;
-         mGS->mFontMetrics->GetFont(font);
+         const nsFont *font = &mGS->mFontMetrics->Font();
          fallbackDone = SurrogateGetDimensions(aString, segDim, fontNum, 
                                                font->size, 
                                                (font->weight > NS_FONT_WEIGHT_NORMAL), 
@@ -1350,8 +1335,7 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
       // Fallback by using ATSUI
       if (!fallbackDone)  
       {
-        const nsFont *font;
-        mGS->mFontMetrics->GetFont(font);
+        const nsFont *font = &mGS->mFontMetrics->Font();
         fallbackDone = ATSUIFallbackGetDimensions(aString, segDim, fontNum, 
                                                   font->size, 
                                                   (font->weight > NS_FONT_WEIGHT_NORMAL), 
@@ -1434,7 +1418,7 @@ nsUnicodeRenderingToolkit::GetTextSegmentBoundingMetrics(
       nsBoundingMetrics& oBoundingMetrics)
 {
   oBoundingMetrics.Clear();
-  if(aLength == 0) 
+  if(aLength == 0 || fontNum == IGNORABLE_FONT_NUM) 
     return NS_OK;
   NS_PRECONDITION(BAD_FONT_NUM != fontNum, "illegal font num");
   PRBool firstTime = PR_TRUE;
@@ -1506,8 +1490,7 @@ nsUnicodeRenderingToolkit::GetTextSegmentBoundingMetrics(
           ((processLen+1) < aLength) &&
           IS_LOW_SURROGATE(*(aString+1)) )
       {
-         const nsFont *font;
-         mGS->mFontMetrics->GetFont(font);
+         const nsFont *font = &mGS->mFontMetrics->Font();
          fallbackDone = SurrogateGetBoundingMetrics(aString, segBoundingMetrics, fontNum, 
                                                     font->size, 
                                                     (font->weight > NS_FONT_WEIGHT_NORMAL), 
@@ -1536,8 +1519,7 @@ nsUnicodeRenderingToolkit::GetTextSegmentBoundingMetrics(
       // Fallback by using ATSUI
       if (!fallbackDone)  
       {
-        const nsFont *font;
-        mGS->mFontMetrics->GetFont(font);
+        const nsFont *font = &mGS->mFontMetrics->Font();
         fallbackDone = ATSUIFallbackGetBoundingMetrics(aString, segBoundingMetrics, fontNum, 
                                                   font->size, 
                                                   (font->weight > NS_FONT_WEIGHT_NORMAL), 
@@ -1581,7 +1563,7 @@ nsresult nsUnicodeRenderingToolkit :: DrawTextSegment(
 			short fontNum, nsUnicodeFontMappingMac& fontMapping, 
 			PRInt32 x, PRInt32 y, PRUint32& oWidth)
 {
-	if(aLength == 0) {
+	if(aLength == 0 || fontNum == IGNORABLE_FONT_NUM) {
 		oWidth = 0;
 		return NS_OK;
 	}	
@@ -1648,8 +1630,7 @@ nsresult nsUnicodeRenderingToolkit :: DrawTextSegment(
           ((processLen+1) < aLength) &&
           IS_LOW_SURROGATE(*(aString+1)) )
       {
-         const nsFont *font;
-         mGS->mFontMetrics->GetFont(font);
+         const nsFont *font = &mGS->mFontMetrics->Font();
          fallbackDone = SurrogateDrawChar(aString, x, y, thisWidth, fontNum, 
                                           font->size, 
                                           (font->weight > NS_FONT_WEIGHT_NORMAL), 
@@ -1677,8 +1658,7 @@ nsresult nsUnicodeRenderingToolkit :: DrawTextSegment(
 #ifndef DISABLE_ATSUI_FALLBACK  
 		  // Fallback by using ATSUI
 		  if(! fallbackDone)  {
-		  	const nsFont *font;
-			  mGS->mFontMetrics->GetFont(font);
+		  	const nsFont *font = &mGS->mFontMetrics->Font();
 		  	fallbackDone = ATSUIFallbackDrawChar(aString, x, y, thisWidth, fontNum, 
 									  		font->size, 
 									  		(font->weight > NS_FONT_WEIGHT_NORMAL), 
