@@ -68,9 +68,6 @@
 
 #define NS_REPLACEMENT_CHAR  PRUnichar(0x003F) // question mark
 
-static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-static NS_DEFINE_CID(kLocaleServiceCID, NS_LOCALESERVICE_CID); 
 
 #define NS_MAX_FONT_WEIGHT 900
 #define NS_MIN_FONT_WEIGHT 100
@@ -214,7 +211,7 @@ static nsFontCleanupObserver *gFontCleanupObserver;
 static nsresult
 InitGlobals(void)
 {
-  CallGetService(kPrefCID, &gPref);
+  CallGetService(NS_PREF_CONTRACTID, &gPref);
   if (!gPref) {
     FreeGlobals();
     return NS_ERROR_FAILURE;
@@ -304,15 +301,17 @@ InitGlobals(void)
 
   gSystemCodePage = ::WinQueryCp(HMQ_CURRENT);
 
+  nsresult res;
+
   if (!nsFontMetricsOS2::gGlobalFonts) {
-    nsresult res = nsFontMetricsOS2::InitializeGlobalFonts();
+    res = nsFontMetricsOS2::InitializeGlobalFonts();
     if (NS_FAILED(res)) {
       FreeGlobals();
       return res;
     }
   }
 
-  nsresult res = gPref->GetBoolPref("browser.display.substitute_vector_fonts",
+  res = gPref->GetBoolPref("browser.display.substitute_vector_fonts",
                                     &nsFontMetricsOS2::gSubstituteVectorFonts);
   NS_ASSERTION( NS_SUCCEEDED(res), "Could not get pref 'browser.display.substitute_vector_fonts'" );
 
@@ -541,7 +540,7 @@ nsFontMetricsOS2::SetFontHandle(HPS aPS, GlobalFontEntry* aEntry,
       if (gSubstituteVectorFonts &&
           GetVectorSubstitute(aPS, aEntry->GetKey(), alias))
       {
-        strcpy(fattrs->szFacename, NS_LossyConvertUCS2toASCII(alias).get());
+        strcpy(fattrs->szFacename, NS_LossyConvertUTF16toASCII(alias).get());
         fattrs->fsFontUse = FATTR_FONTUSE_OUTLINE | FATTR_FONTUSE_TRANSFORMABLE;
         fattrs->fsSelection &= ~(FM_SEL_BOLD | FM_SEL_ITALIC);
       }
@@ -657,10 +656,11 @@ nsFontMetricsOS2::LoadFont(HPS aPS, const nsAString& aFontname)
     FONTMETRICS* pMetrics = getMetrics(lFonts, facename.get(), aPS);
 
     if (lFonts > 0) {
-      nsAutoString familyname;
-      CopyASCIItoUCS2(nsDependentCString(pMetrics[0].szFamilyname),
-                      familyname);
-      GlobalFontEntry* globalEntry = gGlobalFonts->GetEntry(familyname);
+      nsAutoChar16Buffer familyname;
+      MultiByteToWideChar(0, pMetrics[0].szFamilyname,
+                          strlen(pMetrics[0].szFamilyname), familyname, len);
+      nsAutoString name(familyname.get());
+      GlobalFontEntry* globalEntry = gGlobalFonts->GetEntry(name);
       if (globalEntry) {
         // Look through metrics for one that matches given facename
         nsMiniMetrics* metrics = globalEntry->mMetrics;
@@ -724,7 +724,7 @@ PR_STATIC_CALLBACK(PLDHashOperator)
 DebugOutputEnumFunc(GlobalFontEntry* aEntry, void* aData)
 {
   printf("---------------------------------------------------------------------\n");
-  printf(" [[]] %s\n", NS_LossyConvertUCS2toASCII(aEntry->GetKey()).get());
+  printf(" [[]] %s\n", NS_LossyConvertUTF16toASCII(aEntry->GetKey()).get());
   nsMiniMetrics* metrics = aEntry->mMetrics;
   while (metrics) {
     printf("  %32s", metrics->szFacename);
@@ -774,6 +774,7 @@ nsFontMetricsOS2::InitializeGlobalFonts()
   lRemFonts = GFX (::GpiQueryFonts(ps, QF_PUBLIC, NULL, &lNumFonts,
                                    sizeof (FONTMETRICS), pFontMetrics),
                    GPI_ALTERROR);
+  ::WinReleasePS(ps);
 
   for (int i = 0; i < lNumFonts; i++) {
     FONTMETRICS* font = &(pFontMetrics[i]);
@@ -805,19 +806,21 @@ nsFontMetricsOS2::InitializeGlobalFonts()
      //           'italic', 'oblique', 'regular').  If the Facename contains
      //           one of these indicators, then we will create the sort key
      //           based on the Familyname.  Otherwise, use the Facename.
-    nsAutoString fontptr;
+    char* f;
     if (PL_strcasestr(font->szFacename, "bold") != nsnull ||
         PL_strcasestr(font->szFacename, "italic") != nsnull ||
         PL_strcasestr(font->szFacename, "oblique") != nsnull ||
         PL_strcasestr(font->szFacename, "regular") != nsnull ||
         PL_strcasestr(font->szFacename, "-normal") != nsnull)
     {
-      CopyASCIItoUCS2(nsDependentCString(NS_STATIC_CAST(char*, font->szFamilyname)),
-                      fontptr);
+      f = NS_STATIC_CAST(char*, font->szFamilyname);
     } else {
-      CopyASCIItoUCS2(nsDependentCString(NS_STATIC_CAST(char*, font->szFacename)),
-                      fontptr);
+      f = NS_STATIC_CAST(char*, font->szFacename);
     }
+    nsAutoChar16Buffer fontname;
+    PRInt32 len;
+    MultiByteToWideChar(0, f, strlen(f), fontname, len);
+    nsAutoString fontptr(fontname.get());
 
     // The fonts in gBadDBCSFontMapping do not display well in non-Chinese
     //   systems.  Map them to a more intelligible name.
@@ -828,10 +831,9 @@ nsFontMetricsOS2::InitializeGlobalFonts()
           (gSystemCodePage != 950))
       {
         for (int i = 0; gBadDBCSFontMapping[i].mName != nsnull; i++) {
-          if (fontptr.Equals(NS_ConvertASCIItoUCS2(gBadDBCSFontMapping[i].mName),
-                            nsCaseInsensitiveStringComparator()))
+          if (strcmp(f, gBadDBCSFontMapping[i].mName) == 0)
           {
-            CopyASCIItoUCS2(nsDependentCString(gBadDBCSFontMapping[i].mWinName),
+            CopyASCIItoUTF16(nsDependentCString(gBadDBCSFontMapping[i].mWinName),
                             fontptr);
             break;
           }
@@ -868,8 +870,6 @@ nsFontMetricsOS2::InitializeGlobalFonts()
     globalEntry->mCodePage = font->usCodePage;
   }
 
-  ::WinReleasePS(ps);
-
 #ifdef DEBUG_pedemonte
   gGlobalFonts->EnumerateEntries(DebugOutputEnumFunc, nsnull);
   fflush(stdout);
@@ -1040,7 +1040,7 @@ nsFontMetricsOS2::FindLocalFont(HPS aPS, PRUint32 aChar)
     }
 #ifdef DEBUG_FONT_SELECTION
     printf(" FindLocalFont(): attempting to load %s\n",
-           NS_LossyConvertUCS2toASCII(*winName).get());
+           NS_LossyConvertUTF16toASCII(*winName).get());
 #endif
     nsFontOS2* font = LoadFont(aPS, *winName);
     if (font && font->HasGlyph(aPS, aChar)) {
@@ -1057,14 +1057,14 @@ nsFontMetricsOS2::LoadGenericFont(HPS aPS, PRUint32 aChar, const nsAString& aNam
     // woah, this seems bad
     const nsACString& fontName =
       nsDependentCString(((nsFontOS2*)mLoadedFonts[i])->mFattrs.szFacename);
-    if (aName.Equals(NS_ConvertASCIItoUCS2(fontName),
+    if (aName.Equals(NS_ConvertASCIItoUTF16(fontName),
                      nsCaseInsensitiveStringComparator()))
       return nsnull;
 
   }
 #ifdef DEBUG_FONT_SELECTION
   printf(" LoadGenericFont(): attempting to load %s\n",
-         NS_LossyConvertUCS2toASCII(aName).get());
+         NS_LossyConvertUTF16toASCII(aName).get());
 #endif
   nsFontOS2* font = LoadFont(aPS, aName);
   if (font && font->HasGlyph(aPS, aChar)) {
@@ -1162,7 +1162,7 @@ nsFontMetricsOS2::FindGenericFont(HPS aPS, PRUint32 aChar)
     }
 
     AppendGenericFontFromPref(font.name, langGroup, 
-                              NS_ConvertUCS2toUTF8(mGeneric).get());
+                              NS_ConvertUTF16toUTF8(mGeneric).get());
   }
 
   // Iterate over the list of names using the callback mechanism of nsFont...
@@ -1191,8 +1191,8 @@ nsFontMetricsOS2::FindPrefFont(HPS aPS, PRUint32 aChar)
     nsAutoString langGroup;
     gUsersLocale->ToString(langGroup);
     AppendGenericFontFromPref(font.name, 
-                              NS_ConvertUCS2toUTF8(langGroup).get(), 
-                              NS_ConvertUCS2toUTF8(mGeneric).get());
+                              NS_ConvertUTF16toUTF8(langGroup).get(), 
+                              NS_ConvertUTF16toUTF8(mGeneric).get());
   }
   // Try the pref of the user's system lang group
   // For example, if the os language is Simplified Chinese, 
@@ -1202,8 +1202,8 @@ nsFontMetricsOS2::FindPrefFont(HPS aPS, PRUint32 aChar)
     nsAutoString langGroup;
     gSystemLocale->ToString(langGroup);
     AppendGenericFontFromPref(font.name, 
-                              NS_ConvertUCS2toUTF8(langGroup).get(), 
-                              NS_ConvertUCS2toUTF8(mGeneric).get());
+                              NS_ConvertUTF16toUTF8(langGroup).get(), 
+                              NS_ConvertUTF16toUTF8(mGeneric).get());
   }
 
   // Also try all the default pref fonts enlisted from other languages
@@ -1211,7 +1211,7 @@ nsFontMetricsOS2::FindPrefFont(HPS aPS, PRUint32 aChar)
     nsIAtom* langGroup = NS_NewAtom(gCharsetInfo[i].mLangGroup); 
     if((gUsersLocale != langGroup) && (gSystemLocale != langGroup)) {
       AppendGenericFontFromPref(font.name, gCharsetInfo[i].mLangGroup, 
-                                NS_ConvertUCS2toUTF8(mGeneric).get());
+                                NS_ConvertUTF16toUTF8(mGeneric).get());
     }
     NS_IF_RELEASE(langGroup);
   }
@@ -1339,7 +1339,11 @@ nsFontMetricsOS2::RealizeFont()
 
   // set a fallback generic font if the font-family list didn't have one
   if (mGeneric.IsEmpty()) {
-    rv = gPref->CopyUnicharPref("font.default", getter_Copies(value));
+    const char* langGroup = nsnull;
+    mLangGroup->GetUTF8String(&langGroup);
+    pref.Assign("font.default.");
+    pref.Append(langGroup);
+    rv = gPref->CopyUnicharPref(pref.get(), getter_Copies(value));
     if (NS_SUCCEEDED(rv)) {
       mGeneric.Assign(value);
     }
@@ -1533,12 +1537,6 @@ NS_IMETHODIMP nsFontMetricsOS2::GetMaxAdvance( nscoord &aAdvance)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFontMetricsOS2::GetFont( const nsFont *&aFont)
-{
-  aFont = &mFont;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsFontMetricsOS2::GetFontHandle( nsFontHandle &aHandle)
 {
   aHandle = mFontHandle;
@@ -1593,7 +1591,7 @@ nsFontMetricsOS2::LoadUnicodeFont(HPS aPS, const nsAString& aName)
 {
 #ifdef DEBUG_FONT_SELECTION
   printf(" LoadUnicodeFont(): attempting to load %s\n",
-         NS_LossyConvertUCS2toASCII(aName).get());
+         NS_LossyConvertUTF16toASCII(aName).get());
 #endif
   nsFontOS2* font = LoadFont(aPS, aName);
   if (font) {
@@ -1629,7 +1627,7 @@ nsFontMetricsOS2::FindUnicodeFont(HPS aPS)
   nsCAutoString pref, generic;
   nsXPIDLString value;
 
-  generic.Assign(NS_ConvertUCS2toUTF8(mGeneric));
+  generic.Assign(NS_ConvertUTF16toUTF8(mGeneric));
 
   pref.Assign("font.name.");
   pref.Append(generic);
@@ -1685,8 +1683,11 @@ nsFontMetricsOS2::FindWesternFont()
   // Create a 'western' font by making a copy of the currently selected font
   // and changing the codepage 1252
   nsFontOS2* font = new nsFontOS2();
-  *font = *mFontHandle;
+  font->mFattrs = mFontHandle->mFattrs;
   font->mFattrs.usCodePage = 1252;
+  font->mCharbox = mFontHandle->mCharbox;
+  font->mMaxAscent = mFontHandle->mMaxAscent;
+  font->mMaxDescent = mFontHandle->mMaxDescent;
   font->mConvertCodePage = 1252;
   mLoadedFonts.AppendElement(font);
   mWesternFont = font;
@@ -1973,7 +1974,7 @@ nsFontMetricsOS2FT::FindPrefFont(HPS aPS, PRUint32 aChar)
   // be identified instead of a single language (eg. CJK and latin). In this case we have to 
   // try every language in the set. gUserLocale and gSystemLocale provide some hints about 
   // which one should be tried first. This is important for CJK font, since the glyph for single 
-  // char varies dramatically in different langauges. For latin languages, their glyphs are 
+  // char varies dramatically in different languages. For latin languages, their glyphs are 
   // similar. In fact, they almost always share identical fonts. It will be a waste of time to 
   // figure out which one comes first. As a final fallback, unicode preference is always tried. 
 
@@ -1981,15 +1982,15 @@ nsFontMetricsOS2FT::FindPrefFont(HPS aPS, PRUint32 aChar)
   if (unicodeRange < kRangeSpecificItemNum) {
     // a single language is identified
     AppendGenericFontFromPref(font.name, LangGroupFromUnicodeRange(unicodeRange), 
-                              NS_ConvertUCS2toUTF8(mGeneric).get());
+                              NS_ConvertUTF16toUTF8(mGeneric).get());
   } else if (kRangeSetLatin == unicodeRange) { 
     // Character is from a latin language set, so try western and central european
     // If mLangGroup is western or central european, this most probably will not be
     // used, but is here as a fallback scenario.    
     AppendGenericFontFromPref(font.name, "x-western",
-                              NS_ConvertUCS2toUTF8(mGeneric).get());
+                              NS_ConvertUTF16toUTF8(mGeneric).get());
     AppendGenericFontFromPref(font.name, "x-central-euro",
-                              NS_ConvertUCS2toUTF8(mGeneric).get());
+                              NS_ConvertUTF16toUTF8(mGeneric).get());
   } else if (kRangeSetCJK == unicodeRange) { 
     // CJK, we have to be careful about the order, use locale info as hint
     
@@ -1998,7 +1999,7 @@ nsFontMetricsOS2FT::FindPrefFont(HPS aPS, PRUint32 aChar)
       nsCAutoString usersLocaleLangGroup;
       gUsersLocale->ToUTF8String(usersLocaleLangGroup);
       AppendGenericFontFromPref(font.name, usersLocaleLangGroup.get(), 
-                                NS_ConvertUCS2toUTF8(mGeneric).get());
+                                NS_ConvertUTF16toUTF8(mGeneric).get());
     }
     
     // then system locale (os language)
@@ -2006,30 +2007,30 @@ nsFontMetricsOS2FT::FindPrefFont(HPS aPS, PRUint32 aChar)
       nsCAutoString systemLocaleLangGroup;
       gSystemLocale->ToUTF8String(systemLocaleLangGroup);
       AppendGenericFontFromPref(font.name, systemLocaleLangGroup.get(), 
-                                NS_ConvertUCS2toUTF8(mGeneric).get());
+                                NS_ConvertUTF16toUTF8(mGeneric).get());
     }
 
     // try all other languages in this set.
     if (mLangGroup != gJA && gUsersLocale != gJA && gSystemLocale != gJA)
       AppendGenericFontFromPref(font.name, "ja",
-                                NS_ConvertUCS2toUTF8(mGeneric).get());
+                                NS_ConvertUTF16toUTF8(mGeneric).get());
     if (mLangGroup != gZHCN && gUsersLocale != gZHCN && gSystemLocale != gZHCN)
       AppendGenericFontFromPref(font.name, "zh-CN",
-                                NS_ConvertUCS2toUTF8(mGeneric).get());
+                                NS_ConvertUTF16toUTF8(mGeneric).get());
     if (mLangGroup != gZHTW && gUsersLocale != gZHTW && gSystemLocale != gZHTW)
       AppendGenericFontFromPref(font.name, "zh-TW",
-                                NS_ConvertUCS2toUTF8(mGeneric).get());
+                                NS_ConvertUTF16toUTF8(mGeneric).get());
     if (mLangGroup != gZHHK && gUsersLocale != gZHHK && gSystemLocale != gZHHK)
       AppendGenericFontFromPref(font.name, "zh-HK",
-                                NS_ConvertUCS2toUTF8(mGeneric).get());
+                                NS_ConvertUTF16toUTF8(mGeneric).get());
     if (mLangGroup != gKO && gUsersLocale != gKO && gSystemLocale != gKO)
       AppendGenericFontFromPref(font.name, "ko",
-                                NS_ConvertUCS2toUTF8(mGeneric).get());
+                                NS_ConvertUTF16toUTF8(mGeneric).get());
   } 
 
   // always try unicode as fallback
   AppendGenericFontFromPref(font.name, "x-unicode",
-                            NS_ConvertUCS2toUTF8(mGeneric).get());
+                            NS_ConvertUTF16toUTF8(mGeneric).get());
   
   // use the font list to find font
   GenericFontEnumContext context = {aPS, aChar, nsnull, this};
@@ -2277,12 +2278,56 @@ nsFontMetricsOS2FT::ResolveBackwards(HPS                  aPS,
   count = mLoadedFonts.Count();
 
   // see if one of our loaded fonts can represent the current character
-  currFont = LocateFont(aPS, *currChar, count);
+  if (IS_LOW_SURROGATE(*currChar) && (currChar-1) > lastChar && IS_HIGH_SURROGATE(*(currChar-1))) {
+    currFont = LocateFont(aPS, SURROGATE_TO_UCS4(*(currChar-1), *currChar), count);
+    currChar -= 2;
+  }
+  else {
+    currFont = LocateFont(aPS, *currChar, count);
+    --currChar;
+  }
+
+  //This if block is meant to speedup the process in normal situation, when
+  //most characters can be found in first font
+  if (currFont == mLoadedFonts[0]) {
+    while (currChar > lastChar && (currFont->HasGlyph(aPS, *currChar)))
+      --currChar;
+    fontSwitch.mFont = currFont;
+    if (!(*aFunc)(&fontSwitch, currChar+1, firstChar - currChar, aData))
+      return NS_OK;
+    if (currChar == lastChar)
+      return NS_OK;
+    // continue with the next substring, re-using the available loaded fonts
+    firstChar = currChar;
+    if (IS_LOW_SURROGATE(*currChar) && (currChar-1) > lastChar && IS_HIGH_SURROGATE(*(currChar-1))) {
+      currFont = LocateFont(aPS, SURROGATE_TO_UCS4(*(currChar-1), *currChar), count);
+      currChar -= 2;
+    }
+    else {
+      currFont = LocateFont(aPS, *currChar, count);
+      --currChar;
+    }
+  }
 
   // see if we can keep the same font for adjacent characters
-  while (--currChar > lastChar) {
-    nextFont = LocateFont(aPS, *currChar, count);
-    if (nextFont != currFont) {
+  PRInt32 lastCharLen;
+  PRUint32 codepoint;
+
+  while (currChar > lastChar) {
+    if (IS_LOW_SURROGATE(*currChar) && (currChar-1) > lastChar && IS_HIGH_SURROGATE(*(currChar-1))) {
+      codepoint =  SURROGATE_TO_UCS4(*(currChar-1), *currChar);
+      nextFont = LocateFont(aPS, codepoint, count);
+      lastCharLen = 2;
+    }
+    else {
+      codepoint = *currChar;
+      nextFont = LocateFont(aPS, codepoint, count);
+      lastCharLen = 1;
+    }
+    if (nextFont != currFont ||
+        /* render right-to-left characters outside the BMP one by one, because
+           OS/2 doesn't reorder them. */
+        codepoint > 0xFFFF) {
       // We have a substring that can be represented with the same font, and
       // we are about to switch fonts, it is time to notify our caller.
       fontSwitch.mFont = currFont;
@@ -2292,6 +2337,7 @@ nsFontMetricsOS2FT::ResolveBackwards(HPS                  aPS,
       firstChar = currChar;
       currFont = nextFont; // use the font found earlier for the char
     }
+    currChar -= lastCharLen;
   }
 
   //do it for last part of the string
@@ -2393,7 +2439,7 @@ nsFontOS2FT::GetWidth(HPS aPS, const PRUnichar* aString, PRUint32 aLength)
     if (rc == FALSE)
 #endif /* use_expanded_freetype_funcs */
     {
-      NS_ConvertUCS2toUTF8 str(Substring(aString, aString + aLength));
+      NS_ConvertUTF16toUTF8 str(Substring(aString, aString + aLength));
       rc = GetTextExtentPoint32(aPS, (const char*)str.get(), str.Length(),
                                 &size);
     }
@@ -2445,7 +2491,7 @@ nsFontOS2FT::DrawString(HPS aPS, nsDrawingSurfaceOS2* aSurface,
     if (rc == GPI_ERROR)
 #endif /* use_expanded_freetype_funcs */
     {
-      NS_ConvertUCS2toUTF8 str(Substring(aString, aString + aLength));
+      NS_ConvertUTF16toUTF8 str(Substring(aString, aString + aLength));
       ExtTextOut(aPS, ptl.x, ptl.y, 0, NULL, (const char*)str.get(),
                  str.Length(), NULL);
     }
@@ -2550,7 +2596,7 @@ nsFontOS2Substitute::GetWidth(HPS aPS, const PRUnichar* aString,
 
   SIZEL size;
   PRUnichar* string = buffer.get();
-  NS_ConvertUCS2toUTF8 str(Substring(string, string + aLength));
+  NS_ConvertUTF16toUTF8 str(Substring(string, string + aLength));
   BOOL rc = GetTextExtentPoint32(aPS, (const char*)str.get(), str.Length(),
                                  &size);
   if (rc == TRUE) {
@@ -2572,7 +2618,7 @@ nsFontOS2Substitute::DrawString(HPS aPS, nsDrawingSurfaceOS2* aSurface,
   POINTL ptl = { aX, aY };
   aSurface->NS2PM(&ptl, 1);
   PRUnichar* string = buffer.get();
-  NS_ConvertUCS2toUTF8 str(Substring(string, string + aLength));
+  NS_ConvertUTF16toUTF8 str(Substring(string, string + aLength));
   ExtTextOut(aPS, ptl.x, ptl.y, 0, NULL, (const char*)str.get(), str.Length(), NULL);
 }
 #endif /* use_freetype */
