@@ -72,12 +72,13 @@
 #include "nsIPersistentProperties2.h"
 #include "nsCRT.h"
 #include "nsFontMetricsPS.h"
-#include "nsPaperPS.h"
 
 #ifndef NS_BUILD_ID
 #include "nsBuildID.h"
 #endif /* !NS_BUILD_ID */
 
+#include "nsPrintfCString.h"
+
 #include "prenv.h"
 #include "prprf.h"
 #include "prerror.h"
@@ -108,8 +109,6 @@ class fpCString : public nsCAutoString {
 #define NS_RGB_TO_GRAY(r,g,b) ((int(r) * 77 + int(g) * 150 + int(b) * 29) / 256)
 #define NS_IS_BOLD(x) (((x) >= 401) ? 1 : 0) 
 
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 
 /* 
  * global
@@ -200,7 +199,7 @@ nsPostScriptObj::nsPostScriptObj() :
 {
   PR_LOG(nsPostScriptObjLM, PR_LOG_DEBUG, ("nsPostScriptObj::nsPostScriptObj()\n"));
 
-  CallGetService(kPrefCID, &gPrefs);
+  CallGetService(NS_PREF_CONTRACTID, &gPrefs);
 
   gLangGroups = new nsHashtable();
 }
@@ -225,20 +224,14 @@ nsPostScriptObj::~nsPostScriptObj()
   }
 
   if (nsnull != mPrintContext){
-    if (nsnull != mPrintContext->prInfo){
-      delete mPrintContext->prInfo;
-    }
-    if (nsnull != mPrintContext->prSetup){
-      delete mPrintContext->prSetup;
-    }
+    delete mPrintContext->prInfo;
+    delete mPrintContext->prSetup;
     delete mPrintContext;
     mPrintContext = nsnull;
   }
 
-  if (nsnull != mPrintSetup) {
-	  delete mPrintSetup;
-	  mPrintSetup = nsnull;
-  }
+  delete mPrintSetup;
+  mPrintSetup = nsnull;
 
   NS_IF_RELEASE(gPrefs);
 
@@ -268,7 +261,6 @@ nsPostScriptObj::Init( nsIDeviceContextSpecPS *aSpec )
 {
   PRBool      isGray,
               isFirstPageFirst;
-  int         landscape;
 
   PrintInfo* pi = new PrintInfo(); 
   mPrintSetup = new PrintSetup();
@@ -279,8 +271,8 @@ nsPostScriptObj::Init( nsIDeviceContextSpecPS *aSpec )
     mPrintSetup->color = PR_TRUE;              // Image output 
     mPrintSetup->deep_color = PR_TRUE;         // 24 bit color output 
     mPrintSetup->reverse = 0;                  // Output order, 0 is acsending 
+    mPrintSetup->num_copies = 1;
     if ( aSpec != nsnull ) {
-      aSpec->GetCopies(mPrintSetup->num_copies);
       aSpec->GetGrayscale( isGray );
       if ( isGray == PR_TRUE ) {
         mPrintSetup->color = PR_FALSE; 
@@ -302,34 +294,25 @@ nsPostScriptObj::Init( nsIDeviceContextSpecPS *aSpec )
     memset(mPrintContext, 0, sizeof(struct PSContext_));
     memset(pi, 0, sizeof(struct PrintInfo_));
 
-    /* Find PS paper size record by name */
+    /* Get the paper name to write into the ps output */
     aSpec->GetPaperName(&(mPrintSetup->paper_name));
-    nsPaperSizePS paper;
-    if (!paper.Find(mPrintSetup->paper_name))
+
+    /* Get the paper size */
+    rv = aSpec->GetPageSizeInTwips(&mPrintSetup->width, &mPrintSetup->height);
+    if ( NS_FAILED(rv) || mPrintSetup->width <= 0 || mPrintSetup->height <= 0 ) {
       return NS_ERROR_GFX_PRINTER_PAPER_SIZE_NOT_SUPPORTED;
-
-    aSpec->GetLandscape( landscape );
-    mPrintSetup->width = NS_MILLIMETERS_TO_TWIPS(paper.Width_mm());
-    mPrintSetup->height = NS_MILLIMETERS_TO_TWIPS(paper.Height_mm());
-
-    if (landscape) {
-      nscoord temp = mPrintSetup->width;
-      mPrintSetup->width = mPrintSetup->height;
-      mPrintSetup->height = temp;
     }
 
 #ifdef DEBUG
     printf("\nPaper Width = %d twips (%gmm) Height = %d twips (%gmm)\n",
-        mPrintSetup->width, paper.Width_mm(),
-        mPrintSetup->height, paper.Height_mm());
+        mPrintSetup->width, NS_TWIPS_TO_MILLIMETERS(mPrintSetup->width),
+        mPrintSetup->height, NS_TWIPS_TO_MILLIMETERS(mPrintSetup->height));
 #endif
     mPrintSetup->header = "header";
     mPrintSetup->footer = "footer";
     mPrintSetup->sizes = nsnull;
 
-    mPrintSetup->landscape = (landscape) ? PR_TRUE : PR_FALSE; // Rotated output 
-    //mPrintSetup->landscape = PR_FALSE;
-
+    aSpec->GetLandscape(mPrintSetup->landscape);
     mPrintSetup->underline = PR_TRUE;             // underline links 
     mPrintSetup->scale_images = PR_TRUE;          // Scale unsized images which are too big 
     mPrintSetup->scale_pre = PR_FALSE;		        // do the pre-scaling thing 
@@ -374,6 +357,13 @@ nsPostScriptObj::Init( nsIDeviceContextSpecPS *aSpec )
     }
 }
 
+void
+nsPostScriptObj::SetNumCopies(int aNumCopies)
+{
+  NS_PRECONDITION(mPrintSetup, "mPrintSetup must not be NULL");
+  mPrintSetup->num_copies = aNumCopies;
+}
+
 /** ---------------------------------------------------
  *  See documentation in nsPostScriptObj.h
  *	@update 2/1/99 dwc
@@ -404,37 +394,31 @@ nsPostScriptObj::initialize_translation(PrintSetup* pi)
  *  @param aHandle File handle which receives the prolog
  */
 void 
-nsPostScriptObj::write_prolog(FILE *aHandle)
+nsPostScriptObj::write_prolog(FILE *aHandle, PRBool aFTPEnable)
 {
-  int i;
   FILE *f = aHandle;
 
-  nscoord paper_width = mPrintContext->prSetup->width;
-  nscoord paper_height = mPrintContext->prSetup->height;
-  const char *orientation;
+  float fWidth = NSTwipsToFloatPoints(mPrintContext->prSetup->width);
+  float fHeight = NSTwipsToFloatPoints(mPrintContext->prSetup->height);
 
-  if (paper_height < paper_width) {
-    // prSetup->width and height have been swapped, indicating landscape.
-    // The bounding box etc. must be in terms of the default PS coordinate
-    // system.
-    nscoord temp = paper_width;
-    paper_width = paper_height;
-    paper_height = temp;
-    orientation = "Landscape";
-  }
-  else
-    orientation = "Portrait";
+  // PostScript comments marked with %% are document structuring conventions
+  // (DSC) comments. See Adobe specification 5001 at
+  // <http://partners.adobe.com/public/developer/ps/index_specs.html>.
 
   fprintf(f, "%%!PS-Adobe-3.0\n");
   fprintf(f, "%%%%BoundingBox: 0 0 %s %s\n",
-    fpCString(NSTwipsToFloatPoints(paper_width)).get(),
-    fpCString(NSTwipsToFloatPoints(paper_height)).get());
+    fpCString(NSToCoordRound(fWidth)).get(),
+    fpCString(NSToCoordRound(fHeight)).get());
+  fprintf(f, "%%%%HiResBoundingBox: 0 0 %s %s\n",
+    fpCString(fWidth).get(),
+    fpCString(fHeight).get());
 
   fprintf(f, "%%%%Creator: Mozilla PostScript module (%s/%lu)\n",
              "rv:" MOZILLA_VERSION, (unsigned long)NS_BUILD_ID);
   fprintf(f, "%%%%DocumentData: Clean8Bit\n");
   fprintf(f, "%%%%DocumentPaperSizes: %s\n", mPrintSetup->paper_name);
-  fprintf(f, "%%%%Orientation: %s\n", orientation);
+  fprintf(f, "%%%%Orientation: %s\n",
+    mPrintContext->prSetup->landscape ? "Landscape" : "Portrait");
   fprintf(f, "%%%%Pages: %d\n", (int) mPageNumber - 1);
 
   fprintf(f, "%%%%PageOrder: %s\n",
@@ -452,32 +436,75 @@ nsPostScriptObj::write_prolog(FILE *aHandle)
   // general comments: Mozilla-specific 
   fputs("% MozillaCharsetName: iso-8859-1\n\n", f);
     
-    // now begin prolog 
-  fprintf(f, "%%%%BeginProlog\n");
+  fputs("%%BeginProlog\n", f);
 
-  // Tell the printer what size paper it should use
-  fprintf(f,
-    "/setpagedevice where\n"			// Test for the feature
-    "{ pop 1 dict\n"				// Set up a dictionary
-    "  dup /PageSize [ %s %s ] put\n"		// Paper dimensions
-    "  setpagedevice\n"				// Install settings
-    "} if\n", 
-    fpCString(NSTwipsToFloatPoints(paper_width)).get(),
-    fpCString(NSTwipsToFloatPoints(paper_height)).get());
+  // The first number following "Gecko_procset" is a version number. If you change
+  // any of the PS between the following BeginResource and EndResource lines, you
+  // must increment the version.
+  fputs("%%BeginResource: procset Gecko_procset 1.0 0\n", f);
 
+  // Procedure to select and scale a font, using selectfont if available. See
+  // Adobe Technical note 5048. Note msf args are backwards from selectfont.
+  fprintf(f, "%s",
+    "/Msf /selectfont where\n"
+    "  { pop { exch selectfont } }\n"
+    "  { { findfont exch scalefont setfont } }\n"
+    "  ifelse\n"
+    "  bind def\n");
+
+  // Procedure to stroke a rectangle. Coordinates are rounded
+  // to device pixel boundaries. See Adobe Technical notes 5111 and
+  // 5126 and the "Scan Conversion" section of the PS Language
+  // Reference for background.
+  fprintf(f, "%s",
+    "/Mrect { % x y w h Mrect -\n"
+    "  2 index add\n"		// x y w h+y
+    "  4 1 roll\n"		// h+y x y w
+    "  2 index add\n"		// h+y x y w+x
+    "  4 1 roll\n"		// w+x h+y x y
+    "  transform round .1 add exch round .1 add exch itransform\n"
+    "  4 -2 roll\n"		// x' y' w+x h+x
+    "  transform round .1 sub exch round .1 sub exch itransform\n"
+    "  2 index sub\n"		// x' y' w'+x h'
+    "  4 1 roll\n"		// h' x' y' w'+x
+    "  2 index sub\n"		// h' x' y' w'
+    "  4 1 roll\n"		// w' h' x' y'
+    "  moveto\n"		// w' h'
+    "  dup 0 exch rlineto\n"	// w' h'
+    "  exch 0 rlineto\n"	// h'
+    "  neg 0 exch rlineto\n"	// -
+    "  closepath\n"
+    "} bind def\n"
+
+    // Setstrokeadjust, or null if not supported
+    "/Msetstrokeadjust /setstrokeadjust where\n"
+    "  { pop /setstrokeadjust } { /pop } ifelse\n"
+    "  load def\n"
+    );
+  fputs("%%EndResource\n", f);    // Gecko_procset
+
+  if (aFTPEnable) {
+    return;
+  }
+
+  // Write the AFM font support.
+  // The first number following "Gecko_afm_procset" is a version number. If you change
+  // any of the PS between the following BeginResource and EndResource lines, you
+  // must increment the version.
+  fputs("%%BeginResource: procset Gecko_afm_procset 1.0 0\n", f);
   fprintf(f, "[");
-  for (i = 0; i < 256; i++){
-	  if (*isotab[i] == '\0'){
-      fprintf(f, " /.notdef");
-    }else{
-	    fprintf(f, " /%s", isotab[i]);
+  for (int i = 0; i < 256; i++) {
+    if (*isotab[i] == '\0') {
+      fputs(" /.notdef", f);
+    }
+    else {
+      fprintf(f, " /%s", isotab[i]);
     }
 
     if (( i % 6) == 5){
       fprintf(f, "\n");
     }
   }
-
   fprintf(f, "] /isolatin1encoding exch def\n");
 
   // Procedure to reencode a font
@@ -492,22 +519,6 @@ nsPostScriptObj::write_prolog(FILE *aHandle)
       "  definefont pop\n"
       "} bind def\n");
 
-  // Procedure to select and scale a font, using selectfont if available. See
-  // Adobe Technical note 5048. Note msf args are backwards from selectfont.
-  fprintf(f, "%s",
-    "/Msf /selectfont where\n"
-    "  { pop { exch selectfont } }\n"
-    "  { { findfont exch scalefont setfont } }\n"
-    "  ifelse\n"
-    "  bind def\n");
-
-  for(i=0;i<NUM_AFM_FONTS;i++){
-    fprintf(f, 
-      "/F%d /%s Mfr\n"
-      "/f%d { dup /csize exch def /F%d Msf } bind def\n",
-      i, gSubstituteFonts[i].mPSName, i, i);
-  }
-
   fprintf(f, "%s",
     // Unicode glyph dictionary
     "/UniDict\n"
@@ -1772,43 +1783,18 @@ nsPostScriptObj::write_prolog(FILE *aHandle)
     "  /unicodeshow2 { real_unicodeshow_native } bind def\n"
     "} bind def\n"
     );
-
-    // Procedure to stroke a rectangle. Coordinates are rounded
-    // to device pixel boundaries. See Adobe Technical notes 5111 and
-    // 5126 and the "Scan Conversion" section of the PS Language
-    // Reference for background.
-  fprintf(f, "%s",
-    "/Mrect { % x y w h Mrect -\n"
-    "  2 index add\n"		// x y w h+y
-    "  4 1 roll\n"		// h+y x y w
-    "  2 index add\n"		// h+y x y w+x
-    "  4 1 roll\n"		// w+x h+y x y
-    "  transform round .1 add exch round .1 add exch itransform\n"
-    "  4 -2 roll\n"		// x' y' w+x h+x
-    "  transform round .1 sub exch round .1 sub exch itransform\n"
-    "  2 index sub\n"		// x' y' w'+x h'
-    "  4 1 roll\n"		// h' x' y' w'+x
-    "  2 index sub\n"		// h' x' y' w'
-    "  4 1 roll\n"		// w' h' x' y'
-    "  moveto\n"		// w' h'
-    "  dup 0 exch rlineto\n"	// w' h'
-    "  exch 0 rlineto\n"	// h'
-    "  neg 0 exch rlineto\n"	// -
-    "  closepath\n"
-    "} bind def\n"
-
-    // Setstrokeadjust, or null if not supported
-    "/Msetstrokeadjust /setstrokeadjust where\n"
-    "  { pop /setstrokeadjust } { /pop } ifelse\n"
-    "  load def\n"
-
-    "\n"
-    );
-
   // setup prolog for each langgroup
   initlanggroup(f);
+  fputs("%%EndResource\n", f);  // Gecko_afm_procset
 
-  fprintf(f, "%%%%EndProlog\n");
+  for(int i = 0;i < NUM_AFM_FONTS; i++) {
+    fprintf(f, 
+      "%%%%BeginResource: font Gecko_%s\n"
+      "/F%d /%s Mfr\n"
+      "/f%d { dup /csize exch def /F%d Msf } bind def\n"
+      "%%%%EndResource\n",
+      gSubstituteFonts[i].mPSName, i, gSubstituteFonts[i].mPSName, i, i);
+  }
 }
 
 /** ---------------------------------------------------
@@ -1821,9 +1807,27 @@ nsresult
 nsPostScriptObj::write_script(FILE *aDestHandle)
 {
   NS_PRECONDITION(aDestHandle, "Handle must not be NULL");
+
+  // Close out the prolog
+  fprintf(aDestHandle, "%%%%EndProlog\n");
+
+  // Begin the script section. Set the correct paper size.
+  fputs("%%BeginSetup\n", aDestHandle);
+  fprintf(aDestHandle,
+    "%%%%BeginFeature: *PageSize %s\n"
+    "/setpagedevice where\n"			// Test for the feature
+    "{ pop 1 dict\n"
+    "  dup /PageSize [ %s %s ] put\n"		// Paper dimensions
+    "  setpagedevice\n"				// Install settings
+    "} if\n"
+    "%%%%EndFeature\n",
+    mPrintSetup->paper_name,
+    fpCString(NSTwipsToFloatPoints(mPrintContext->prSetup->width)).get(),
+    fpCString(NSTwipsToFloatPoints(mPrintContext->prSetup->height)).get());
+  fputs("%%EndSetup\n", aDestHandle);
+
   char buf[BUFSIZ];
   size_t readAmt;
-
   rewind(mScriptFP);
   while ((readAmt = fread(buf, 1, sizeof buf, mScriptFP))) {
     size_t writeAmt = fwrite(buf, 1, readAmt, aDestHandle);
@@ -1844,9 +1848,12 @@ nsPostScriptObj::begin_page()
 {
   fprintf(mScriptFP, "%%%%Page: %d %d\n", mPageNumber, mPageNumber);
   fprintf(mScriptFP, "%%%%BeginPageSetup\n");
-  if(mPrintSetup->num_copies != 1) {
-    fprintf(mScriptFP, "1 dict dup /NumCopies %d put setpagedevice\n",
-      mPrintSetup->num_copies);
+  if(mPrintSetup->num_copies > 1) {
+    fprintf(mScriptFP, 
+      "/setpagedevice where\n"
+      "{ pop 1 dict dup /NumCopies %d put setpagedevice }\n"
+      "{ userdict /#copies %d put } ifelse\n",
+      mPrintSetup->num_copies, mPrintSetup->num_copies);
   }
   fprintf(mScriptFP,"/pagelevel save def\n");
   // Rescale the coordinate system from points to twips.
@@ -2039,8 +2046,35 @@ nsPostScriptObj::show(const PRUnichar* txt, int len,
   fprintf(mScriptFP, ") %sunicodeshow\n", align);
 }
 
+#if defined(MOZ_ENABLE_FREETYPE2) || defined(MOZ_ENABLE_XFT)
+void 
+nsPostScriptObj::show(const PRUnichar* aTxt, int aLen,
+                      const nsAFlatString& aCharList, PRUint16 aSubFontIdx)
+{
+  int i;
+  fputc('<', mScriptFP);
 
+  const PRUint16 subFontSize = nsPSFontGenerator::kSubFontSize;
 
+  // the character repertoire of a subfont (255 characters max)
+  const nsAString& repertoire = 
+        Substring(aCharList, aSubFontIdx * subFontSize,
+                  PR_MIN(subFontSize, 
+                  aCharList.Length() - aSubFontIdx * subFontSize));
+
+  for (i = 0; i < aLen; i++) {
+    // XXX This is a little inefficient, but printing is not perf. critical. 
+    NS_ASSERTION(repertoire.FindChar(aTxt[i]) != kNotFound,
+        "character is not covered by this subfont");
+      
+    // Type 1 encoding vector has 256 slots, but the 0-th slot is 
+    // reserved for /.notdef so that we use the 1st through 255th slots
+    // for actual characters (hence '+ 1')
+    fprintf(mScriptFP, "%02x", repertoire.FindChar(aTxt[i]) + 1); 
+  }
+  fputs("> show\n", mScriptFP);
+}
+#endif
 
 /** ---------------------------------------------------
  *  See documentation in nsPostScriptObj.h
@@ -2270,7 +2304,7 @@ nsPostScriptObj::translate(nscoord x, nscoord y)
    *    @param iRect    Rectangle describing the portion of the image's
    *                    coordinate space covered by the image pixel data.
    */
-
+   
 void
 nsPostScriptObj::draw_image(nsIImage *anImage,
     const nsRect& sRect, const nsRect& iRect, const nsRect& dRect)
@@ -2280,6 +2314,17 @@ nsPostScriptObj::draw_image(nsIImage *anImage,
     return;
   }
 
+  PRBool hasAlpha = anImage->GetHasAlphaMask();
+  PRInt32 bytesPerRow = anImage->GetLineStride();
+  
+  PRUint8 *rowCopy = nsnull;
+  if (hasAlpha) {
+    rowCopy = new PRUint8[bytesPerRow];
+    if (!rowCopy) {
+      return;
+    }
+  }
+
   anImage->LockImagePixels(PR_FALSE);
   PRUint8 *theBits = anImage->GetBits();
 
@@ -2287,14 +2332,34 @@ nsPostScriptObj::draw_image(nsIImage *anImage,
   // There's nothing to print, so just return.
   if (!theBits || (0 == iRect.width) || (0 == iRect.height)) {
     anImage->UnlockImagePixels(PR_FALSE);
+    delete[] rowCopy;
     return;
   }
-
+  
   // Save the current graphic state and define a PS variable that
-  // can hold one line of pixel data.
-  fprintf(mScriptFP, "gsave\n/rowdata %d string def\n",
-     mPrintSetup->color ? iRect.width * 3 : iRect.width);
- 
+  // can hold pixel data.
+  // If !hasAlpha, we read the image one line at a time
+  PRInt32 pixCountBytes = mPrintSetup->color ? iRect.width * 3 : iRect.width;
+  PRInt32 alphaCountBytes = hasAlpha ? (iRect.width + 7)/8 : 0;
+  fprintf(mScriptFP, "gsave\n"
+                     "/rowdata %d string def\n", pixCountBytes + alphaCountBytes);
+  if (hasAlpha) {
+    // Determine whether we can support explicit mask
+    fputs("/useExplicitMask false def\n"
+          "/languagelevel where\n"
+          "{pop languagelevel\n"
+          " 3 eq\n"
+          " {/useExplicitMask true def} if\n"
+          "} if\n"
+          "/makedict {"
+          " counttomark 2 idiv\n"
+          " dup dict\n"
+          " begin\n"
+          "  {def} repeat\n"
+          "  pop\n"
+          "  currentdict\n"
+          " end } def\n", mScriptFP);
+  }
   // Translate the coordinate origin to the corner of the rectangle where
   // the image should appear, set up a clipping region, and scale the
   // coordinate system to the image's final size.
@@ -2303,10 +2368,6 @@ nsPostScriptObj::draw_image(nsIImage *anImage,
   clip();
   fprintf(mScriptFP, "%d %d scale\n", dRect.width, dRect.height);
 
-  // Describe how the pixel data is to be interpreted: pixels per row,
-  // rows, and bits per pixel (per component in color).
-  fprintf(mScriptFP, "%d %d 8 ", iRect.width, iRect.height);
-
   // Output the transformation matrix for the image. This is a bit tricky
   // to understand. PS image-drawing operations involve two transformation
   // matrices: (1) A Transformation matrix associated with the image
@@ -2347,25 +2408,121 @@ nsPostScriptObj::draw_image(nsIImage *anImage,
     tmTY += tmSY;
     tmSY = -tmSY;
   }
-  fprintf(mScriptFP, "[ %d 0 0 %d %d %d ]\n", tmSX, tmSY, tmTX, tmTY);
-
-  // Output the data-reading procedure and the appropriate image command.
-  fputs(" { currentfile rowdata readhexstring pop }", mScriptFP);
-  if (mPrintSetup->color)
+  
+  if (hasAlpha) {
+    const char* decodeMatrix;
+    fprintf(mScriptFP, " useExplicitMask {\n");
+    if (mPrintSetup->color) {
+      fprintf(mScriptFP, " /DeviceRGB setcolorspace\n");
+      decodeMatrix = "0 1 0 1 0 1";
+    } else {
+      fprintf(mScriptFP, " /DeviceGray setcolorspace\n");
+      decodeMatrix = "0 1";
+    }
+    fprintf(mScriptFP, "mark /ImageType 3\n"
+                       "  /DataDict mark\n"
+                       "   /ImageType 1 /Width %d /Height %d\n"
+                       "   /ImageMatrix [ %d 0 0 %d %d %d ]\n"
+                       "   /DataSource { currentfile rowdata readhexstring pop }\n"
+                       "   /BitsPerComponent 8\n"
+                       "   /Decode [%s]\n"
+                       "  makedict\n"
+                       "  /MaskDict mark\n"
+                       "   /ImageType 1 /Width %d /Height %d\n"
+                       "   /ImageMatrix [ %d 0 0 %d %d %d ]\n"
+                       "   /BitsPerComponent 1\n"
+                       "   /Decode [1 0]\n"
+                       "  makedict\n"
+                       "  /InterleaveType 2\n" // interleave by row
+                       " makedict image}\n"
+                       "{", iRect.width, iRect.height, tmSX, tmSY, tmTX, tmTY,
+                       decodeMatrix,
+                       iRect.width, iRect.height, tmSX, tmSY, tmTX, tmTY);
+  }
+  // output the runtime fallback "no mask" code
+  fprintf(mScriptFP, " %d %d 8 [ %d 0 0 %d %d %d ]\n",
+                     iRect.width, iRect.height, tmSX, tmSY, tmTX, tmTY);
+  if (hasAlpha) {
+    fprintf(mScriptFP, " { currentfile rowdata readhexstring pop %d %d getinterval }\n",
+                       /*index*/ alphaCountBytes, /*index*/ pixCountBytes);
+  } else {
+    fprintf(mScriptFP, " { currentfile rowdata readhexstring pop }\n");
+  }
+  if (mPrintSetup->color) {
     fputs(" false 3 colorimage\n", mScriptFP);
-  else
+  } else {
     fputs(" image\n", mScriptFP);
-
+  }
+  if (hasAlpha) {
+    fputs("} ifelse\n", mScriptFP);
+  }
+  
+  PRUint8* alphaBits;
+  PRInt32 alphaBytesPerRow;
+  PRInt32 alphaDepth;
+  if (hasAlpha) {
+    anImage->LockImagePixels(PR_TRUE);
+    alphaBits = anImage->GetAlphaBits();
+    alphaBytesPerRow = anImage->GetAlphaLineStride();
+    alphaDepth = anImage->GetAlphaDepth();
+  }
+  
   // Output the image data. The entire image is written, even
   // if it's partially clipped in the document.
   int outputCount = 0;
-  PRInt32 bytesPerRow = anImage->GetLineStride();
 
-  for (nscoord y = 0; y < iRect.height; y++) {
-    // calculate the starting point for this row of pixels
-    PRUint8 *row = theBits                // Pixel buffer start
-      + y * bytesPerRow;                  // Rows already output
+  nscoord y;
+  for (y = 0; y < iRect.height; y++) {
+    PRUint8 *row = theBits + y * bytesPerRow;
 
+    // output alpha first, if any
+    if (hasAlpha) {
+      // Copy the row so that we can set any transparent pixels to white
+      memcpy(rowCopy, row, bytesPerRow);
+      row = rowCopy;
+        
+      PRUint8 *alphaRow = alphaBits + y * alphaBytesPerRow;
+      PRUint8 v = 0;
+      for (nscoord x = 0; x < iRect.width; x++) {
+        PRUint8 alpha;
+        if (alphaDepth == 8) {
+          alpha = alphaRow[x];
+        } else {
+          PRUint8 mask = 0x80 >> (x & 7);
+          // Test the x'th bit of the alphaRow
+          alpha = (alphaRow[x >> 3] & mask) ? 255 : 0;
+        }
+        
+        if (alpha >= 128) {
+          PRUint8 mask = 0x80 >> (x & 7);
+          // set the correct bit for the Postscript mask
+          v |= mask;
+        } else {
+          // Many tools set fully transparent pixels to black (perhaps
+          // because that compresses better). So for PS printers that
+          // don't support transparency, we'll get the image on an ugly
+          // black background which is especially bad on printers.
+          // So if the pixel is fully transparent and fully black, then
+          // we set it to white --- still wrong, but less bad.
+          PRUint8 *pixel = row + (x * 3);
+          if (alpha == 0 && pixel[0] == 0 && pixel[1] == 0 &&
+              pixel[2] == 0) {
+            pixel[0] = pixel[1] = pixel[2] = 0xFF;
+          }
+        }
+        if ((x & 7) == 7 || x == iRect.width - 1) {
+          outputCount += fprintf(mScriptFP, "%02x", v);
+          if (outputCount >= 72) {
+            fputc('\n', mScriptFP);
+            outputCount = 0;
+          }
+          v = 0;
+        }
+      }
+      fputc('\n', mScriptFP);
+      outputCount = 0;
+    }
+    
     for (nscoord x = 0; x < iRect.width; x++) {
       PRUint8 *pixel = row + (x * 3);
       if (mPrintSetup->color)
@@ -2380,12 +2537,18 @@ nsPostScriptObj::draw_image(nsIImage *anImage,
         outputCount = 0;
       }
     }
+    fputc('\n', mScriptFP);
+    outputCount = 0;
+  }
+  if (hasAlpha) {
+    anImage->UnlockImagePixels(PR_TRUE);
   }
   anImage->UnlockImagePixels(PR_FALSE);
-
+  
   // Free the PS data buffer and restore the previous graphics state.
-  fputs("\n/rowdata where { /rowdata undef } if\n", mScriptFP);
-  fputs("grestore\n", mScriptFP);
+  fputs("/undef where { pop /rowdata where { /rowdata undef } if } if\n"
+        "grestore\n", mScriptFP);
+  delete[] rowCopy;
 }
 
 
@@ -2418,9 +2581,11 @@ float greyBrightness;
 }
 
 
-void nsPostScriptObj::setfont(const nsCString aFontName, PRUint32 aHeight)
+void nsPostScriptObj::setfont(const nsCString& aFontName, PRUint32 aHeight,
+                              PRInt32 aSubFont)
 {
-  fprintf(mScriptFP, "%d /%s Msf\n", aHeight, aFontName.get());
+  fprintf(mScriptFP, "%d /%s.Set%d Msf\n", aHeight, aFontName.get(),
+          aSubFont);
 }
 
 /** ---------------------------------------------------
@@ -2691,7 +2856,7 @@ static void PrefEnumCallback(const char *aName, void *aClosure)
     if (psnativecode) {
       nsAutoString str;
       nsCOMPtr<nsICharsetConverterManager> ccMain =
-        do_GetService(kCharsetConverterManagerCID, &res);
+        do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &res);
       
       if (NS_SUCCEEDED(res)) {
         res = ccMain->GetUnicodeEncoder(psnativecode.get(), &linfo->mEncoder);
