@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -24,18 +24,17 @@
  *   Zack Rusin <zack@kde.org>
  *   Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
  *
- *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -340,23 +339,23 @@ NS_IMETHODIMP nsDeviceContextSpecQt::Init(nsIPrintSettings *aPS)
     aPS->GetMarginRight(&dright);
 
     if (printfile)
-      strcpy(mPath,    NS_ConvertUCS2toUTF8(printfile).get());
+      strcpy(mPath,    NS_ConvertUTF16toUTF8(printfile).get());
     if (command)
-      strcpy(mCommand, NS_ConvertUCS2toUTF8(command).get());
+      strcpy(mCommand, NS_ConvertUTF16toUTF8(command).get());
     if (printer)
-      strcpy(mPrinter, NS_ConvertUCS2toUTF8(printer).get());
+      strcpy(mPrinter, NS_ConvertUTF16toUTF8(printer).get());
     if (papername)
-      strcpy(mPaperName, NS_ConvertUCS2toUTF8(papername).get());
+      strcpy(mPaperName, NS_ConvertUTF16toUTF8(papername).get());
 
     DO_PR_DEBUG_LOG(("margins:   %5.2f,%5.2f,%5.2f,%5.2f\n", dtop, dleft, dbottom, dright));
     DO_PR_DEBUG_LOG(("printRange %d\n",   printRange));
     DO_PR_DEBUG_LOG(("fromPage   %d\n",   fromPage));
     DO_PR_DEBUG_LOG(("toPage     %d\n",   toPage));
     DO_PR_DEBUG_LOG(("tofile     %d\n",   tofile));
-    DO_PR_DEBUG_LOG(("printfile  '%s'\n", printfile? NS_ConvertUCS2toUTF8(printfile).get():"<NULL>"));
-    DO_PR_DEBUG_LOG(("command    '%s'\n", command? NS_ConvertUCS2toUTF8(command).get():"<NULL>"));
-    DO_PR_DEBUG_LOG(("printer    '%s'\n", printer? NS_ConvertUCS2toUTF8(printer).get():"<NULL>"));
-    DO_PR_DEBUG_LOG(("papername  '%s'\n", papername? NS_ConvertUCS2toUTF8(papername).get():"<NULL>"));
+    DO_PR_DEBUG_LOG(("printfile  '%s'\n", printfile? NS_ConvertUTF16toUTF8(printfile).get():"<NULL>"));
+    DO_PR_DEBUG_LOG(("command    '%s'\n", command? NS_ConvertUTF16toUTF8(command).get():"<NULL>"));
+    DO_PR_DEBUG_LOG(("printer    '%s'\n", printer? NS_ConvertUTF16toUTF8(printer).get():"<NULL>"));
+    DO_PR_DEBUG_LOG(("papername  '%s'\n", papername? NS_ConvertUTF16toUTF8(papername).get():"<NULL>"));
 
     mTop         = dtop;
     mBottom      = dbottom;
@@ -623,7 +622,7 @@ NS_IMETHODIMP nsPrinterEnumeratorQt::GetDefaultPrinterName(PRUnichar **aDefaultP
 
   GlobalPrinters::GetInstance()->GetDefaultPrinterName(aDefaultPrinterName);
 
-  DO_PR_DEBUG_LOG(("GetDefaultPrinterName(): default printer='%s'.\n", NS_ConvertUCS2toUTF8(*aDefaultPrinterName).get()));
+  DO_PR_DEBUG_LOG(("GetDefaultPrinterName(): default printer='%s'.\n", NS_ConvertUTF16toUTF8(*aDefaultPrinterName).get()));
   return NS_OK;
 }
 
@@ -645,8 +644,8 @@ NS_IMETHODIMP nsPrinterEnumeratorQt::InitPrintSettingsFromPrinter(const PRUnicha
 
   nsXPIDLCString fullPrinterName, /* Full name of printer incl. driver-specific prefix */
                  printerName;     /* "Stripped" name of printer */
-  fullPrinterName.Assign(NS_ConvertUCS2toUTF8(aPrinterName));
-  printerName.Assign(NS_ConvertUCS2toUTF8(aPrinterName));
+  fullPrinterName.Assign(NS_ConvertUTF16toUTF8(aPrinterName));
+  printerName.Assign(NS_ConvertUTF16toUTF8(aPrinterName));
   DO_PR_DEBUG_LOG(("printerName='%s'\n", printerName.get()));
 
   PrintMethod type = pmInvalid;
@@ -683,7 +682,7 @@ NS_IMETHODIMP nsPrinterEnumeratorQt::InitPrintSettingsFromPrinter(const PRUnicha
       filename.Assign("mozilla.ps");
   }
   DO_PR_DEBUG_LOG(("Setting default filename to '%s'\n", filename.get()));
-  aPrintSettings->SetToFileName(NS_ConvertUTF8toUCS2(filename).get());
+  aPrintSettings->SetToFileName(NS_ConvertUTF8toUTF16(filename).get());
 
   aPrintSettings->SetIsInitializedFromPrinter(PR_TRUE);
 #ifdef USE_XPRINT
@@ -785,7 +784,7 @@ NS_IMETHODIMP nsPrinterEnumeratorQt::InitPrintSettingsFromPrinter(const PRUnicha
       aPrintSettings->SetPaperSizeUnit(nsIPrintSettings::kPaperSizeMillimeters);
       aPrintSettings->SetPaperWidth(total_width);
       aPrintSettings->SetPaperHeight(total_height);
-      aPrintSettings->SetPaperName(NS_ConvertUTF8toUCS2(papername).get());
+      aPrintSettings->SetPaperName(NS_ConvertUTF8toUTF16(papername).get());
 
 #ifdef SET_PRINTER_FEATURES_VIA_PREFS
       int i;
@@ -890,7 +889,7 @@ NS_IMETHODIMP nsPrinterEnumeratorQt::InitPrintSettingsFromPrinter(const PRUnicha
         aPrintSettings->SetPaperSizeUnit(nsIPrintSettings::kPaperSizeInches);
         aPrintSettings->SetPaperWidth(PSPaperSizeRec_FullPaperWidth(default_paper));
         aPrintSettings->SetPaperHeight(PSPaperSizeRec_FullPaperHeight(default_paper));
-        aPrintSettings->SetPaperName(NS_ConvertUTF8toUCS2(default_paper->name).get());
+        aPrintSettings->SetPaperName(NS_ConvertUTF8toUTF16(default_paper->name).get());
       }
       else {
         DO_PR_DEBUG_LOG(("Unknown paper size '%s' given.\n", papername.get()));
@@ -916,7 +915,7 @@ NS_IMETHODIMP nsPrinterEnumeratorQt::InitPrintSettingsFromPrinter(const PRUnicha
     nsXPIDLCString command;
     if (NS_SUCCEEDED(CopyPrinterCharPref(pPrefs, "postscript", printerName, "print_command", getter_Copies(command)))) {
       DO_PR_DEBUG_LOG(("setting default print command to '%s'\n", command.get()));
-      aPrintSettings->SetPrintCommand(NS_ConvertUTF8toUCS2(command).get());
+      aPrintSettings->SetPrintCommand(NS_ConvertUTF8toUTF16(command).get());
     }
 
 #ifdef SET_PRINTER_FEATURES_VIA_PREFS
@@ -955,7 +954,7 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
     int i;
     for(  i = 0 ; i < mGlobalNumPrinters ; i++ )
     {
-      mGlobalPrinterList->AppendString(nsString(NS_ConvertASCIItoUCS2(plist[i].name)));
+      mGlobalPrinterList->AppendString(nsString(NS_ConvertASCIItoUTF16(plist[i].name)));
     }
 
     XpuFreePrinterList(plist);
@@ -996,8 +995,8 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
         added_default_printer = PR_TRUE;
 
       mGlobalPrinterList->AppendString(
-        nsString(NS_ConvertASCIItoUCS2(NS_POSTSCRIPT_DRIVER_NAME)) +
-        nsString(NS_ConvertASCIItoUCS2(name)));
+        nsString(NS_ConvertASCIItoUTF16(NS_POSTSCRIPT_DRIVER_NAME)) +
+        nsString(NS_ConvertASCIItoUTF16(name)));
       mGlobalNumPrinters++;
     }
 
@@ -1009,7 +1008,7 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
   if (!added_default_printer)
   {
     mGlobalPrinterList->AppendString(
-      nsString(NS_ConvertASCIItoUCS2(NS_POSTSCRIPT_DRIVER_NAME "default")));
+      nsString(NS_ConvertASCIItoUTF16(NS_POSTSCRIPT_DRIVER_NAME "default")));
     mGlobalNumPrinters++;
   }
 #endif /* USE_POSTSCRIPT */
@@ -1023,11 +1022,9 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
 //----------------------------------------------------------------------
 void GlobalPrinters::FreeGlobalPrinters()
 {
-  if (mGlobalPrinterList) {
-    delete mGlobalPrinterList;
-    mGlobalPrinterList = nsnull;
-    mGlobalNumPrinters = 0;
-  }
+  delete mGlobalPrinterList;
+  mGlobalPrinterList = nsnull;
+  mGlobalNumPrinters = 0;
 }
 
 void
