@@ -37,15 +37,13 @@
 
 #include "nsDeviceContextWin.h"
 #include "nsRenderingContextWin.h"
-#include "nsDeviceContextSpecWin.h"
-#include "nsIPref.h"
 #include "nsIServiceManager.h"
 #include "nsCOMPtr.h"
 #include "nsIScreenManager.h"
 #include "nsIScreen.h"
 #include "nsGfxCIID.h"
 #include "nsReadableUtils.h"
-
+#include "nsISupportsPrimitives.h"
 #include "nsString.h"
 
 #if defined(DEBUG_rods) && defined(MOZ_LAYOUTDEBUG)
@@ -54,13 +52,6 @@
 
 #define DOC_TITLE_LENGTH      64
 
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-
-PRBool nsDeviceContextWin::gRound = PR_FALSE;
-PRUint32 nsDeviceContextWin::sNumberOfScreens = 0;
-
-static char* nav4rounding = "font.size.nav4rounding";
-
 #include "prlog.h"
 #ifdef PR_LOGGING 
 PRLogModuleInfo * kGfxPrintingLogMod = PR_NewLogModule("printing-gfx");
@@ -84,17 +75,6 @@ nsDeviceContextWin :: nsDeviceContextWin()
   mSpec = nsnull;
   mCachedClientRect = PR_FALSE;
   mCachedFullRect = PR_FALSE;
-
-  nsresult res = NS_ERROR_FAILURE;
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res));
-  if (NS_SUCCEEDED(res)) {
-    static PRBool roundingInitialized = PR_FALSE;
-    if (!roundingInitialized) {
-      roundingInitialized = PR_TRUE;
-      PrefChanged(nav4rounding, this);
-    }
-    prefs->RegisterCallback(nav4rounding, PrefChanged, this);
-  }
 }
 
 nsDeviceContextWin :: ~nsDeviceContextWin()
@@ -114,12 +94,6 @@ nsDeviceContextWin :: ~nsDeviceContextWin()
   }
 
   NS_IF_RELEASE(mSpec);
-
-  nsresult res = NS_ERROR_FAILURE;
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res));
-  if (NS_SUCCEEDED(res)) {
-    prefs->UnregisterCallback(nav4rounding, PrefChanged, this);
-  }
 }
 
 NS_IMETHODIMP nsDeviceContextWin :: Init(nsNativeWidget aWidget)
@@ -184,8 +158,6 @@ void nsDeviceContextWin :: CommonInit(HDC aDC)
     // screen objects. We'll save the result 
     nsresult ignore;
     mScreenManager = do_GetService("@mozilla.org/gfx/screenmanager;1", &ignore);   
-    if ( !sNumberOfScreens )
-      mScreenManager->GetNumberOfScreens(&sNumberOfScreens);
   } // if this dc is not a print device
   mTwipsToPixels = 1.0 / mPixelsToTwips;
 
@@ -196,29 +168,25 @@ void nsDeviceContextWin :: CommonInit(HDC aDC)
 void
 nsDeviceContextWin :: ComputeClientRectUsingScreen ( nsRect* outRect )
 {
-  // if we have more than one screen, we always need to recompute the clientRect
+  // we always need to recompute the clientRect
   // because the window may have moved onto a different screen. In the single
   // monitor case, we only need to do the computation if we haven't done it
   // once already, and remember that we have because we're assured it won't change.
-  if ( sNumberOfScreens > 1 || !mCachedClientRect ) {
-    nsCOMPtr<nsIScreen> screen;
-    FindScreen ( getter_AddRefs(screen) );
-    if ( screen ) {
-      PRInt32 x, y, width, height;
-      screen->GetAvailRect ( &x, &y, &width, &height );
-    
-      // convert to device units
-      outRect->y = NSToIntRound(y * mDevUnitsToAppUnits);
-      outRect->x = NSToIntRound(x * mDevUnitsToAppUnits);
-      outRect->width = NSToIntRound(width * mDevUnitsToAppUnits);
-      outRect->height = NSToIntRound(height * mDevUnitsToAppUnits);
+  nsCOMPtr<nsIScreen> screen;
+  FindScreen ( getter_AddRefs(screen) );
+  if ( screen ) {
+    PRInt32 x, y, width, height;
+    screen->GetAvailRect ( &x, &y, &width, &height );
+  
+    // convert to device units
+    outRect->y = NSToIntRound(y * mDevUnitsToAppUnits);
+    outRect->x = NSToIntRound(x * mDevUnitsToAppUnits);
+    outRect->width = NSToIntRound(width * mDevUnitsToAppUnits);
+    outRect->height = NSToIntRound(height * mDevUnitsToAppUnits);
 
-      mCachedClientRect = PR_TRUE;
-      mClientRect = *outRect;
-    }
-  } // if we need to recompute the client rect
-  else
-    *outRect = mClientRect;
+    mCachedClientRect = PR_TRUE;
+    mClientRect = *outRect;
+  }
 
 } // ComputeClientRectUsingScreen
 
@@ -230,29 +198,21 @@ nsDeviceContextWin :: ComputeFullAreaUsingScreen ( nsRect* outRect )
   // because the window may have moved onto a different screen. In the single
   // monitor case, we only need to do the computation if we haven't done it
   // once already, and remember that we have because we're assured it won't change.
-  if ( sNumberOfScreens > 1 || !mCachedFullRect ) {
-    nsCOMPtr<nsIScreen> screen;
-    FindScreen ( getter_AddRefs(screen) );
-    if ( screen ) {
-      PRInt32 x, y, width, height;
-      screen->GetRect ( &x, &y, &width, &height );
-    
-      // convert to device units
-      outRect->y = NSToIntRound(y * mDevUnitsToAppUnits);
-      outRect->x = NSToIntRound(x * mDevUnitsToAppUnits);
-      outRect->width = NSToIntRound(width * mDevUnitsToAppUnits);
-      outRect->height = NSToIntRound(height * mDevUnitsToAppUnits);
+  nsCOMPtr<nsIScreen> screen;
+  FindScreen ( getter_AddRefs(screen) );
+  if ( screen ) {
+    PRInt32 x, y, width, height;
+    screen->GetRect ( &x, &y, &width, &height );
+  
+    // convert to device units
+    outRect->y = NSToIntRound(y * mDevUnitsToAppUnits);
+    outRect->x = NSToIntRound(x * mDevUnitsToAppUnits);
+    outRect->width = NSToIntRound(width * mDevUnitsToAppUnits);
+    outRect->height = NSToIntRound(height * mDevUnitsToAppUnits);
 
-      mWidth = width;
-      mHeight = height;
-      mCachedFullRect = PR_TRUE;
-    }
-  } // if we need to recompute the client rect
-  else {
-      outRect->y = 0;
-      outRect->x = 0;
-      outRect->width = NSToIntRound(mWidth * mDevUnitsToAppUnits);
-      outRect->height = NSToIntRound(mHeight * mDevUnitsToAppUnits);
+    mWidth = width;
+    mHeight = height;
+    mCachedFullRect = PR_TRUE;
   }
  
 } // ComputeFullRectUsingScreen
@@ -266,14 +226,6 @@ nsDeviceContextWin :: ComputeFullAreaUsingScreen ( nsRect* outRect )
 void
 nsDeviceContextWin :: FindScreen ( nsIScreen** outScreen )
 {
-  // optimize for the case where we only have one monitor.
-  if ( !mPrimaryScreen && mScreenManager )
-    mScreenManager->GetPrimaryScreen ( getter_AddRefs(mPrimaryScreen) );  
-  if ( sNumberOfScreens == 1 ) {
-    NS_IF_ADDREF(*outScreen = mPrimaryScreen.get());
-    return;
-  }
-  
   // now then, if we have more than one screen, we need to find which screen this
   // window is on.
   HWND window = NS_REINTERPRET_CAST(HWND, mWidget);
@@ -417,28 +369,50 @@ nsresult nsDeviceContextWin::CopyLogFontToNSFont(HDC* aHDC, const LOGFONT* ptrLo
   // any value when going to a printer, for example mPixleScale is
   // 6.25 when going to a 600dpi printer.
   // round, but take into account whether it is negative
-  LONG logHeight = LONG((float(ptrLogFont->lfHeight) * mPixelScale) + (ptrLogFont->lfHeight < 0 ? -0.5 : 0.5)); // round up
-  int pointSize = -MulDiv(logHeight, 72, ::GetDeviceCaps(*aHDC, LOGPIXELSY));
+  float pixelHeight = -ptrLogFont->lfHeight;
+  if (pixelHeight < 0) {
+    HFONT hFont = ::CreateFontIndirect(ptrLogFont);
+    if (!hFont)
+      return NS_ERROR_OUT_OF_MEMORY;
+    HGDIOBJ hObject = ::SelectObject(*aHDC, hFont);
+    TEXTMETRIC tm;
+    ::GetTextMetrics(*aHDC, &tm);
+    ::SelectObject(*aHDC, hObject);
+    ::DeleteObject(hFont);
+    pixelHeight = tm.tmAscent;
+  }
+
+  float pointSize = pixelHeight * mPixelScale * 72 / ::GetDeviceCaps(*aHDC, LOGPIXELSY);
 
   // we have problem on Simplified Chinese system because the system report
   // the default font size is 8. but if we use 8, the text display very
   // Ugly. force it to be at 9 on that system (cp936), but leave other sizes alone.
-  if ((pointSize == 8) && 
+  if ((pointSize < 9) && 
       (936 == ::GetACP())) 
     pointSize = 9;
 
-  aFont->size = NSIntPointsToTwips(pointSize);
+  aFont->size = NSFloatPointsToTwips(pointSize);
   return NS_OK;
 }
 
 nsresult nsDeviceContextWin :: GetSysFontInfo(HDC aHDC, nsSystemFontID anID, nsFont* aFont) const
 {
-  NONCLIENTMETRICS ncm;
   HGDIOBJ hGDI;
 
   LOGFONT logFont;
   LOGFONT* ptrLogFont = NULL;
 
+#ifdef WINCE
+  hGDI = ::GetStockObject(SYSTEM_FONT);
+  if (hGDI == NULL)
+    return NS_ERROR_UNEXPECTED;
+  
+  if (::GetObject(hGDI, sizeof(logFont), &logFont) > 0)
+    ptrLogFont = &logFont;
+#else
+
+  NONCLIENTMETRICS ncm;
+
   BOOL status;
   if (anID == eSystemFont_Icon) 
   {
@@ -513,6 +487,8 @@ nsresult nsDeviceContextWin :: GetSysFontInfo(HDC aHDC, nsSystemFontID anID, nsF
       break;
   } // switch 
 
+#endif // WINCE
+
   if (nsnull == ptrLogFont)
   {
     return NS_ERROR_FAILURE;
@@ -604,8 +580,13 @@ NS_IMETHODIMP nsDeviceContextWin :: CheckFontExistence(const nsString& aFontName
                                    logFont.lfFaceName, sizeof(logFont.lfFaceName), nsnull, nsnull);
 
   // somehow the WideCharToMultiByte failed, let's try the old code
-  if(0 == outlen)
-    aFontName.ToCString(logFont.lfFaceName, LF_FACESIZE);
+  if(0 == outlen) {
+    nsFixedCString logFontStr(logFont.lfFaceName, LF_FACESIZE);
+    LossyCopyUTF16toASCII(aFontName, logFontStr);
+    if (logFontStr.get() != logFont.lfFaceName) {
+      return NS_ERROR_FAILURE; // the font name is too large
+    }
+  }
 
   ::EnumFontFamiliesEx(hdc, &logFont, (FONTENUMPROC)fontcallback, (LPARAM)&isthere, 0);
 
@@ -631,10 +612,14 @@ NS_IMETHODIMP nsDeviceContextWin::GetPaletteInfo(nsPaletteInfo& aPaletteInfo)
   aPaletteInfo.numReserved = mPaletteInfo.numReserved;
 
   if (NULL == mPaletteInfo.palette) {
+#ifndef WINCE
     HWND    hwnd = (HWND)mWidget;
     HDC     hdc = ::GetDC(hwnd);
     mPaletteInfo.palette = ::CreateHalftonePalette(hdc);  
     ::ReleaseDC(hwnd, hdc);                                                     
+#else
+    mPaletteInfo.palette = (HPALETTE) GetStockObject(DEFAULT_PALETTE);
+#endif
   }
 
   aPaletteInfo.palette = mPaletteInfo.palette;
@@ -720,23 +705,13 @@ NS_IMETHODIMP nsDeviceContextWin :: GetDeviceContextFor(nsIDeviceContextSpec *aD
   }
 
   devConWin->mSpec = aDevice;
-  NS_ADDREF(aDevice);
- 
-  char*   devicename;
-  char*   drivername;
 
-  nsDeviceContextSpecWin* devSpecWin = NS_STATIC_CAST(nsDeviceContextSpecWin*, aDevice);
-  devSpecWin->GetDeviceName(devicename); // sets pointer do not free
-  devSpecWin->GetDriverName(drivername); // sets pointer do not free
+  NS_ADDREF(aDevice);
 
   HDC dc = NULL;
-  LPDEVMODE devmode;
-  devSpecWin->GetDevMode(devmode);
-  NS_ASSERTION(devmode, "DevMode can't be NULL here");
-  if (devmode) {
-    dc = ::CreateDC(drivername, devicename, NULL, devmode);
-  }
-
+  nsCOMPtr<nsISupportsVoid> supVoid = do_QueryInterface(aDevice);
+  supVoid->GetData((void**)&dc);
+ 
   return devConWin->Init(dc, this); // take ownership of the DC
 }
 
@@ -905,21 +880,7 @@ NS_IMETHODIMP nsDeviceContextWin :: EndPage(void)
   return NS_OK;
 }
 
-int
-nsDeviceContextWin :: PrefChanged(const char* aPref, void* aClosure)
-{
-  nsresult res = NS_ERROR_FAILURE;
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res));
-  if (NS_SUCCEEDED(res)) {
-    prefs->GetBoolPref(nav4rounding, &gRound);
-    nsDeviceContextWin* deviceContext = (nsDeviceContextWin*) aClosure;
-    deviceContext->FlushFontCache();
-  }
-
-  return 0;
-}
-
-char* 
+char*
 nsDeviceContextWin :: GetACPString(const nsAString& aStr)
 {
    int acplen = aStr.Length() * 2 + 1;
