@@ -119,7 +119,7 @@ public:
 GraphicsState :: GraphicsState()
 {
   mNext = nsnull;
-  mMatrix.SetToIdentity();  
+  mMatrix.SetToIdentity();
   mLocalClip.x = mLocalClip.y = mLocalClip.width = mLocalClip.height = 0;
   mClipRegion = NULL;
   mColor = NS_RGB(0, 0, 0);
@@ -162,6 +162,7 @@ GraphicsState :: ~GraphicsState()
   mDottedPen = NULL;
 }
 
+// gIsWIN95 includes Win98 and WinME too
 #define NOT_SETUP 0x33
 static PRBool gIsWIN95 = NOT_SETUP;
 
@@ -177,12 +178,16 @@ static HPEN   gStockWhitePen = (HPEN)::GetStockObject(WHITE_PEN);
 
 nsRenderingContextWin :: nsRenderingContextWin()
 {
+#ifdef WINCE
+  gIsWIN95 = PR_FALSE;
+  gBidiInfo = DONT_INIT;
+#else
   // The first time in we initialize gIsWIN95 flag
   if (NOT_SETUP == gIsWIN95) {
     OSVERSIONINFO os;
     os.dwOSVersionInfoSize = sizeof(os);
     ::GetVersionEx(&os);
-    // XXX This may need tweaking for win98
+    // gIsWIN95 must be true for Win98 and WinME too
     if (VER_PLATFORM_WIN32_NT == os.dwPlatformId) {
       gIsWIN95 = PR_FALSE;
     }
@@ -206,6 +211,7 @@ nsRenderingContextWin :: nsRenderingContextWin()
       }
     }
   }
+#endif
 
   mDC = NULL;
   mMainDC = NULL;
@@ -432,15 +438,19 @@ nsresult nsRenderingContextWin :: SetupDC(HDC aOldDC, HDC aNewDC)
   ::SetTextColor(aNewDC, PALETTERGB_COLORREF(mColor));
   mCurrTextColor = mCurrentColor;
   ::SetBkMode(aNewDC, TRANSPARENT);
+#ifndef WINCE
   ::SetPolyFillMode(aNewDC, WINDING);
+#endif
   // Temporary fix for bug 135226 until we do better decode-time
   // dithering and paletized storage of images.
   nsPaletteInfo palInfo;
   mContext->GetPaletteInfo(palInfo);
+#ifndef WINCE
   if (palInfo.isPaletteDevice)
     ::SetStretchBltMode(aNewDC, HALFTONE);
   else
     ::SetStretchBltMode(aNewDC, COLORONCOLOR);                                  
+#endif
 
   ::SetTextAlign(aNewDC, TA_BASELINE);
 
@@ -532,7 +542,9 @@ NS_IMETHODIMP nsRenderingContextWin :: LockDrawingSurface(PRInt32 aX, PRInt32 aY
     if(palInfo.isPaletteDevice && palInfo.palette){
       ::SelectPalette(mDC,(HPALETTE)palInfo.palette,PR_TRUE);
       ::RealizePalette(mDC);
+#ifndef WINCE
       ::UpdateColors(mDC);                                                      
+#endif
     }
   }
 
@@ -558,8 +570,10 @@ NS_IMETHODIMP nsRenderingContextWin :: UnlockDrawingSurface(void)
     mCurrTextColor = mCurrentColor;
 
     ::SetBkMode(mDC, TRANSPARENT);
+#ifndef WINCE
     ::SetPolyFillMode(mDC, WINDING);
     ::SetStretchBltMode(mDC, COLORONCOLOR);
+#endif
 
     mOrigSolidBrush = (HBRUSH)::SelectObject(mDC, mCurrBrush);
     mOrigFont = (HFONT)::SelectObject(mDC, mCurrFont);
@@ -628,7 +642,9 @@ nsRenderingContextWin :: GetHints(PRUint32& aResult)
 {
   PRUint32 result = 0;
   
+#ifndef WINCE
   result |= NS_RENDERING_HINT_FAST_MEASURE;
+#endif
 
   if (gIsWIN95)
     result |= NS_RENDERING_HINT_FAST_8BIT_TEXT;
@@ -636,10 +652,12 @@ nsRenderingContextWin :: GetHints(PRUint32& aResult)
   if (NOT_SETUP == gBidiInfo) {
     InitBidiInfo();
   }
+#ifndef WINCE
   if (GCP_REORDER == (gBidiInfo & GCP_REORDER) )
     result |= NS_RENDERING_HINT_BIDI_REORDERING;
   if (GCP_GLYPHSHAPE == (gBidiInfo & GCP_GLYPHSHAPE) )
     result |= NS_RENDERING_HINT_ARABIC_SHAPING;
+#endif
   
   aResult = result;
 
@@ -1055,6 +1073,7 @@ NS_IMETHODIMP nsRenderingContextWin :: DrawLine(nscoord aX0, nscoord aY0, nscoor
 
   SetupPen();
 
+#ifndef WINCE
   if (nsLineStyle_kDotted == mCurrLineStyle)
   {
     lineddastruct dda_struct;
@@ -1066,6 +1085,7 @@ NS_IMETHODIMP nsRenderingContextWin :: DrawLine(nscoord aX0, nscoord aY0, nscoor
     LineDDA((int)(aX0),(int)(aY0),(int)(aX1),(int)(aY1),(LINEDDAPROC) LineDDAFunc,(long)&dda_struct);
   }
   else
+#endif
   {
     ::MoveToEx(mDC, (int)(aX0), (int)(aY0), NULL);
     ::LineTo(mDC, (int)(aX1), (int)(aY1));
@@ -1270,6 +1290,14 @@ NS_IMETHODIMP nsRenderingContextWin :: FillPolygon(const nsPoint aPoints[], PRIn
   return NS_OK;
 }
 
+void*
+nsRenderingContextWin::GetNativeGraphicData(GraphicDataType aType)
+{
+  if (aType == NATIVE_WINDOWS_DC)
+    return mDC;
+
+  return nsnull;
+}
 
 NS_IMETHODIMP nsRenderingContextWin :: DrawEllipse(const nsRect& aRect)
 {
@@ -1346,7 +1374,9 @@ NS_IMETHODIMP nsRenderingContextWin :: DrawArc(nscoord aX, nscoord aY, nscoord a
   ey = (PRInt32)(cy - distance * sin(anglerad));
 
   // this just makes it consitent, on windows 95 arc will always draw CC, nt this sets direction
+#ifndef WINCE
   ::SetArcDirection(mDC, AD_COUNTERCLOCKWISE);
+#endif
 
   ::Arc(mDC, aX, aY, aX + aWidth, aY + aHeight, sx, sy, ex, ey); 
 
@@ -1387,13 +1417,21 @@ NS_IMETHODIMP nsRenderingContextWin :: FillArc(nscoord aX, nscoord aY, nscoord a
 
   // this just makes it consistent, on windows 95 arc will always draw CC,
   // on NT this sets direction
+#ifndef WINCE
   ::SetArcDirection(mDC, AD_COUNTERCLOCKWISE);
-
+#endif
   ::Pie(mDC, aX, aY, aX + aWidth, aY + aHeight, sx, sy, ex, ey); 
 
   return NS_OK;
 }
 
+// This must be called to clamp string lengths to 8K for Win95/98/ME.
+inline void CheckLength(PRUint32 *aLength)
+{
+  if (gIsWIN95 && *aLength > 8192)
+    *aLength = 8192;
+}
+
 NS_IMETHODIMP nsRenderingContextWin :: GetWidth(char ch, nscoord& aWidth)
 {
   char buf[1];
@@ -1408,14 +1446,9 @@ NS_IMETHODIMP nsRenderingContextWin :: GetWidth(PRUnichar ch, nscoord &aWidth, P
   return GetWidth(buf, 1, aWidth, aFontID);
 }
 
-NS_IMETHODIMP nsRenderingContextWin :: GetWidth(const char* aString, nscoord& aWidth)
-{
-  return GetWidth(aString, strlen(aString), aWidth);
-}
-
-NS_IMETHODIMP nsRenderingContextWin :: GetWidth(const char* aString,
-                                                PRUint32 aLength,
-                                                nscoord& aWidth)
+NS_IMETHODIMP nsRenderingContextWin :: GetWidthInternal(const char* aString,
+                                                        PRUint32 aLength,
+                                                        nscoord& aWidth)
 {
 
   if (nsnull != mFontMetrics)
@@ -1427,6 +1460,7 @@ NS_IMETHODIMP nsRenderingContextWin :: GetWidth(const char* aString,
       return mFontMetrics->GetSpaceWidth(aWidth);
     }
 
+    CheckLength(&aLength);
     SetupFontAndColor();
     nscoord pxWidth = mCurrFontWin->GetWidth(mDC, aString, aLength);
     aWidth = NSToCoordRound(float(pxWidth) * mP2T);
@@ -1437,11 +1471,6 @@ NS_IMETHODIMP nsRenderingContextWin :: GetWidth(const char* aString,
     return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP nsRenderingContextWin :: GetWidth(const nsString& aString, nscoord& aWidth, PRInt32 *aFontID)
-{
-  return GetWidth(aString.get(), aString.Length(), aWidth, aFontID);
-}
-
 struct GetWidthData {
   HDC   mDC;    // IN
   HFONT mFont;  // IN/OUT (running)
@@ -1466,13 +1495,14 @@ do_GetWidth(const nsFontSwitch* aFontSwitch,
   return PR_TRUE; // don't stop till the end
 }
 
-NS_IMETHODIMP nsRenderingContextWin :: GetWidth(const PRUnichar *aString,
-                                                PRUint32 aLength,
-                                                nscoord &aWidth,
-                                                PRInt32 *aFontID)
+NS_IMETHODIMP nsRenderingContextWin :: GetWidthInternal(const PRUnichar *aString,
+                                                        PRUint32 aLength,
+                                                        nscoord &aWidth,
+                                                        PRInt32 *aFontID)
 {
   if (!mFontMetrics) return NS_ERROR_FAILURE;
 
+  CheckLength(&aLength);
   SetupFontAndColor();
 
   nsFontMetricsWin* metrics = (nsFontMetricsWin*)mFontMetrics;
@@ -1492,20 +1522,21 @@ NS_IMETHODIMP nsRenderingContextWin :: GetWidth(const PRUnichar *aString,
 }
 
 NS_IMETHODIMP
-nsRenderingContextWin::GetTextDimensions(const char*       aString,
-                                         PRInt32           aLength,
-                                         PRInt32           aAvailWidth,
-                                         PRInt32*          aBreaks,
-                                         PRInt32           aNumBreaks,
-                                         nsTextDimensions& aDimensions,
-                                         PRInt32&          aNumCharsFit,
-                                         nsTextDimensions& aLastWordDimensions,
-                                         PRInt32*          aFontID)
+nsRenderingContextWin::GetTextDimensionsInternal(const char*       aString,
+                                                 PRInt32           aLength,
+                                                 PRInt32           aAvailWidth,
+                                                 PRInt32*          aBreaks,
+                                                 PRInt32           aNumBreaks,
+                                                 nsTextDimensions& aDimensions,
+                                                 PRInt32&          aNumCharsFit,
+                                                 nsTextDimensions& aLastWordDimensions,
+                                                 PRInt32*          aFontID)
 {
   NS_PRECONDITION(aBreaks[aNumBreaks - 1] == aLength, "invalid break array");
 
   if (nsnull != mFontMetrics) {
     // Setup the font and foreground color
+    CheckLength((PRUint32*)&aLength);
     SetupFontAndColor();
 
     // If we need to back up this state represents the last place we could
@@ -1664,7 +1695,7 @@ struct BreakGetTextDimensionsData {
   // Remember the fonts that we use so that we can deal with
   // line-breaking in-between fonts later. mOffsets[0] is also used
   // to initialize the current offset from where to start measuring
-  nsVoidArray* mFonts;   // OUT
+  nsVoidArray* mFonts;   // IN/OUT
   nsVoidArray* mOffsets; // IN/OUT
 
 };
@@ -1690,10 +1721,10 @@ do_BreakGetTextDimensions(const nsFontSwitch* aFontSwitch,
   PRInt32 numCharsFit = data->mNumCharsFit;
   nscoord width = data->mWidth;
   PRInt32 start = (PRInt32)(aSubstring - pstr);
-  PRInt32 i = start + aSubstringLength;
+  PRInt32 end = start + aSubstringLength;
   PRBool allDone = PR_FALSE;
 
-  while (start < i) {
+  while (start < end) {
     // Estimate how many characters will fit. Do that by dividing the
     // available space by the average character width
     PRInt32 estimatedNumChars = data->mEstimatedNumChars;
@@ -1713,9 +1744,9 @@ do_BreakGetTextDimensions(const nsFontSwitch* aFontSwitch,
 
     // Avoid scanning the break array in the case where we think all
     // the text should fit
-    if (i <= estimatedBreakOffset) {
+    if (end <= estimatedBreakOffset) {
       // Everything should fit
-      numChars = i - start;
+      numChars = end - start;
     }
     else {
       // Find the nearest place to break that is less than or equal to
@@ -1735,20 +1766,20 @@ do_BreakGetTextDimensions(const nsFontSwitch* aFontSwitch,
       if (start < data->mBreaks[breakIndex]) {
         // The text crosses at least one segment boundary so measure to the
         // break point just before the estimated break offset
-        numChars = PR_MIN(data->mBreaks[breakIndex] - start, (PRInt32)aSubstringLength);
+        numChars = PR_MIN(data->mBreaks[breakIndex], end) - start;
       } 
       else {
         // See whether there is another segment boundary between this one
         // and the end of the text
-        if ((breakIndex < (data->mNumBreaks - 1)) && (data->mBreaks[breakIndex] < i)) {
+        if ((breakIndex < (data->mNumBreaks - 1)) && (data->mBreaks[breakIndex] < end)) {
           ++breakIndex;
-          numChars = PR_MIN(data->mBreaks[breakIndex] - start, (PRInt32)aSubstringLength);
+          numChars = PR_MIN(data->mBreaks[breakIndex], end) - start;
         }
         else {
-          NS_ASSERTION(i != data->mBreaks[breakIndex], "don't expect to be at segment boundary");
+          NS_ASSERTION(end != data->mBreaks[breakIndex], "don't expect to be at segment boundary");
 
           // The text is all within the same segment
-          numChars = i - start;
+          numChars = end - start;
 
           // Remember we're in the middle of a segment and not between
           // two segments
@@ -1812,17 +1843,15 @@ do_BreakGetTextDimensions(const nsFontSwitch* aFontSwitch,
 
       // We can't just revert to the previous break state. Find the break
       // index just before the end of the text
-      i = start + numChars;
-      if (breakIndex == -1) {
-        breakIndex = 0;
-        if (data->mBreaks[breakIndex] < i) {
-          while ((breakIndex + 1 < data->mNumBreaks) && (data->mBreaks[breakIndex + 1] < i)) {
-            ++breakIndex;
-          }
+      end = start + numChars;
+      breakIndex = 0;
+      if (data->mBreaks[breakIndex] < end) {
+        while ((breakIndex + 1 < data->mNumBreaks) && (data->mBreaks[breakIndex + 1] < end)) {
+          ++breakIndex;
         }
       }
 
-      if ((0 == breakIndex) && (i <= data->mBreaks[0])) {
+      if ((0 == breakIndex) && (end <= data->mBreaks[0])) {
         // There's no place to back up to, so even though the text doesn't fit
         // return it anyway
         numCharsFit += numChars;
@@ -1851,21 +1880,54 @@ do_BreakGetTextDimensions(const nsFontSwitch* aFontSwitch,
       // all the way back to the first word
       width += twWidth;
       while ((breakIndex >= 0) && (width > data->mAvailWidth)) {
-        twWidth = 0;
         start = data->mBreaks[breakIndex];
-        numChars = i - start;
+        numChars = end - start;
+        numCharsFit = start;
         if ((1 == numChars) && (pstr[start] == ' ')) {
-          twWidth = data->mSpaceWidth;
+          width -= data->mSpaceWidth;
         }
-        else if (numChars > 0) {
+        else if (pstr + start >= aSubstring) {
+          // The entire fragment to chop is within the current font.
           pxWidth = fontWin->GetWidth(data->mDC, &pstr[start], numChars);
-          twWidth = NSToCoordRound(float(pxWidth) * data->mP2T);
+          width -= NSToCoordRound(float(pxWidth) * data->mP2T);
+        }
+        else {
+          // The fragment that we want to chop extends back into previous fonts.
+          // We need to reverse into previous fonts. Fortunately,
+          // data->mFonts[] and data->mOffsets[] tell us which fonts are used
+          // and when. 
+          end = data->mNumCharsFit; // same as aSubstring - pstr
+          data->mNumCharsFit = numCharsFit; // has got shorter...
+          PRInt32 k = data->mFonts->Count() - 1;
+          for ( ; k >= 0 && start < end; --k, end -= numChars) {
+            fontWin = (nsFontWin*)data->mFonts->ElementAt(k);
+            const PRUnichar* ps = (const PRUnichar*)data->mOffsets->ElementAt(k);
+            if (ps < pstr + start)
+              ps = pstr + start;
+
+            numChars = pstr + end - ps;
+            NS_ASSERTION(numChars > 0, "empty string");
+
+            data->mFont = fontWin->mFont;
+            ::SelectObject(data->mDC, data->mFont);
+            pxWidth = fontWin->GetWidth(data->mDC, ps, numChars);
+            data->mWidth -= NSToCoordRound(float(pxWidth) * data->mP2T);
+
+            // By construction, mFonts[k] is the last font, and
+            // mOffsets[k+1] is the last offset.
+            data->mFonts->RemoveElementAt(k);
+            data->mOffsets->RemoveElementAt(k+1);
+          }
+
+          // We are done, update the data now because we won't do it later.
+          // The |if (data->mNumCharsFit != numCharsFit)| won't apply below
+          data->mFonts->AppendElement(fontWin);
+          data->mOffsets->AppendElement((void*)&pstr[numCharsFit]);
+          break;
         }
 
-        width -= twWidth;
-        numCharsFit = start;
         --breakIndex;
-        i = start;
+        end = start;
       }
     }
 
@@ -1873,7 +1935,7 @@ do_BreakGetTextDimensions(const nsFontSwitch* aFontSwitch,
   }
 
 #ifdef DEBUG_rbs
-  NS_ASSERTION(allDone || start == i, "internal error");
+  NS_ASSERTION(allDone || start == end, "internal error");
   NS_ASSERTION(allDone || data->mNumCharsFit != numCharsFit, "internal error");
 #endif /* DEBUG_rbs */
 
@@ -1894,18 +1956,19 @@ do_BreakGetTextDimensions(const nsFontSwitch* aFontSwitch,
 }
 
 NS_IMETHODIMP
-nsRenderingContextWin::GetTextDimensions(const PRUnichar*  aString,
-                                         PRInt32           aLength,
-                                         PRInt32           aAvailWidth,
-                                         PRInt32*          aBreaks,
-                                         PRInt32           aNumBreaks,
-                                         nsTextDimensions& aDimensions,
-                                         PRInt32&          aNumCharsFit,
-                                         nsTextDimensions& aLastWordDimensions,
-                                         PRInt32*          aFontID)
+nsRenderingContextWin::GetTextDimensionsInternal(const PRUnichar*  aString,
+                                                 PRInt32           aLength,
+                                                 PRInt32           aAvailWidth,
+                                                 PRInt32*          aBreaks,
+                                                 PRInt32           aNumBreaks,
+                                                 nsTextDimensions& aDimensions,
+                                                 PRInt32&          aNumCharsFit,
+                                                 nsTextDimensions& aLastWordDimensions,
+                                                 PRInt32*          aFontID)
 {
   if (!mFontMetrics) return NS_ERROR_FAILURE;
 
+  CheckLength((PRUint32*)&aLength);
   SetupFontAndColor();
 
   nsFontMetricsWin* metrics = (nsFontMetricsWin*)mFontMetrics;
@@ -2056,9 +2119,9 @@ nsRenderingContextWin::GetTextDimensions(const PRUnichar*  aString,
 }
 
 NS_IMETHODIMP
-nsRenderingContextWin::GetTextDimensions(const char*       aString,
-                                         PRUint32          aLength,
-                                         nsTextDimensions& aDimensions)
+nsRenderingContextWin::GetTextDimensionsInternal(const char*       aString,
+                                                 PRUint32          aLength,
+                                                 nsTextDimensions& aDimensions)
 {
   if (!mFontMetrics) {
     aDimensions.Clear();
@@ -2102,14 +2165,15 @@ do_GetTextDimensions(const nsFontSwitch* aFontSwitch,
 }
 
 NS_IMETHODIMP
-nsRenderingContextWin::GetTextDimensions(const PRUnichar*  aString,
-                                         PRUint32          aLength,
-                                         nsTextDimensions& aDimensions,
-                                         PRInt32*          aFontID)
+nsRenderingContextWin::GetTextDimensionsInternal(const PRUnichar*  aString,
+                                                 PRUint32          aLength,
+                                                 nsTextDimensions& aDimensions,
+                                                 PRInt32*          aFontID)
 {
   aDimensions.Clear();
   if (!mFontMetrics) return NS_ERROR_FAILURE;
 
+  CheckLength(&aLength);
   SetupFontAndColor();
 
   nsFontMetricsWin* metrics = (nsFontMetricsWin*)mFontMetrics;
@@ -2130,15 +2194,16 @@ nsRenderingContextWin::GetTextDimensions(const PRUnichar*  aString,
   return NS_OK;
 }
 
-NS_IMETHODIMP nsRenderingContextWin :: DrawString(const char *aString, PRUint32 aLength,
-                                                  nscoord aX, nscoord aY,
-                                                  const nscoord* aSpacing)
+NS_IMETHODIMP nsRenderingContextWin :: DrawStringInternal(const char *aString, PRUint32 aLength,
+                                                          nscoord aX, nscoord aY,
+                                                          const nscoord* aSpacing)
 {
   NS_PRECONDITION(mFontMetrics,"Something is wrong somewhere");
 
   PRInt32 x = aX;
   PRInt32 y = aY;
 
+  CheckLength(&aLength);
   SetupFontAndColor();
 
   INT dxMem[500];
@@ -2228,13 +2293,14 @@ do_DrawString(const nsFontSwitch* aFontSwitch,
   return PR_TRUE; // don't stop till the end
 }
 
-NS_IMETHODIMP nsRenderingContextWin :: DrawString(const PRUnichar *aString, PRUint32 aLength,
-                                                  nscoord aX, nscoord aY,
-                                                  PRInt32 aFontID,
-                                                  const nscoord* aSpacing)
+NS_IMETHODIMP nsRenderingContextWin :: DrawStringInternal(const PRUnichar *aString, PRUint32 aLength,
+                                                          nscoord aX, nscoord aY,
+                                                          PRInt32 aFontID,
+                                                          const nscoord* aSpacing)
 {
   if (!mFontMetrics) return NS_ERROR_FAILURE;
 
+  CheckLength(&aLength);
   SetupFontAndColor();
 
   nsFontMetricsWin* metrics = (nsFontMetricsWin*)mFontMetrics;
@@ -2261,25 +2327,18 @@ NS_IMETHODIMP nsRenderingContextWin :: DrawString(const PRUnichar *aString, PRUi
   return NS_OK;
 }
 
-NS_IMETHODIMP nsRenderingContextWin :: DrawString(const nsString& aString,
-                                                  nscoord aX, nscoord aY,
-                                                  PRInt32 aFontID,
-                                                  const nscoord* aSpacing)
-{
-  return DrawString(aString.get(), aString.Length(), aX, aY, aFontID, aSpacing);
-}
-
 #ifdef MOZ_MATHML
 NS_IMETHODIMP 
-nsRenderingContextWin::GetBoundingMetrics(const char*        aString,
-                                          PRUint32           aLength,
-                                          nsBoundingMetrics& aBoundingMetrics)
+nsRenderingContextWin::GetBoundingMetricsInternal(const char*        aString,
+                                                  PRUint32           aLength,
+                                                  nsBoundingMetrics& aBoundingMetrics)
 {
   NS_PRECONDITION(mFontMetrics,"Something is wrong somewhere");
 
   aBoundingMetrics.Clear();
   if (!mFontMetrics) return NS_ERROR_FAILURE;
 
+  CheckLength(&aLength);
   SetupFontAndColor();
   nsresult rv = mCurrFontWin->GetBoundingMetrics(mDC, aString, aLength, aBoundingMetrics);
 
@@ -2333,14 +2392,15 @@ do_GetBoundingMetrics(const nsFontSwitch* aFontSwitch,
 }
 
 NS_IMETHODIMP
-nsRenderingContextWin::GetBoundingMetrics(const PRUnichar*   aString,
-                                          PRUint32           aLength,
-                                          nsBoundingMetrics& aBoundingMetrics,
-                                          PRInt32*           aFontID)
+nsRenderingContextWin::GetBoundingMetricsInternal(const PRUnichar*   aString,
+                                                  PRUint32           aLength,
+                                                  nsBoundingMetrics& aBoundingMetrics,
+                                                  PRInt32*           aFontID)
 {
   aBoundingMetrics.Clear();
   if (!mFontMetrics) return NS_ERROR_FAILURE;
 
+  CheckLength(&aLength);
   SetupFontAndColor();
 
   nsFontMetricsWin* metrics = (nsFontMetricsWin*)mFontMetrics;
@@ -2369,6 +2429,13 @@ nsRenderingContextWin::GetBoundingMetrics(const PRUnichar*   aString,
 }
 #endif // MOZ_MATHML
 
+PRInt32 nsRenderingContextWin::GetMaxStringLength()
+{
+  if (!mFontMetrics)
+    return 1;
+  return NS_STATIC_CAST(nsFontMetricsWin*, mFontMetrics)->GetMaxStringLength();
+}
+
 NS_IMETHODIMP nsRenderingContextWin :: CopyOffScreenBits(nsIDrawingSurface* aSrcSurf,
                                                          PRInt32 aSrcX, PRInt32 aSrcY,
                                                          const nsRect &aDestBounds,
@@ -2421,7 +2488,9 @@ NS_IMETHODIMP nsRenderingContextWin :: CopyOffScreenBits(nsIDrawingSurface* aSrc
         // road until I can get all this figured out.. and completed correctly.
         // Opened bug #153367 to take care of this palette issue.
         //::RealizePalette(destdc);
+#ifndef WINCE
         ::UpdateColors(mDC);                                                      
+#endif
       }
 
       if (aCopyFlags & NS_COPYBITS_XFORM_SOURCE_VALUES)
@@ -2447,14 +2516,6 @@ NS_IMETHODIMP nsRenderingContextWin :: CopyOffScreenBits(nsIDrawingSurface* aSrc
   return NS_OK;
 }
 
-//~~~
-NS_IMETHODIMP nsRenderingContextWin::RetrieveCurrentNativeGraphicData(PRUint32 * ngd)
-{
-  if(ngd != nsnull)
-    *ngd = (PRUint32)mDC;
-  return NS_OK;
-}
-
 // Small cache of HBRUSH objects
 // Note: the current assumption is that there is only one UI thread so
 // we do not lock, and we do not use TLS
@@ -2661,7 +2722,14 @@ HPEN nsRenderingContextWin :: SetupDottedPen(void)
 {
   if ((mCurrentColor != mCurrPenColor) || (NULL == mCurrPen) || (mCurrPen != mStates->mDottedPen))
   {
-    HPEN  tpen = ::CreatePen(PS_DOT, 0, PALETTERGB_COLORREF(mColor));
+    HPEN  tpen = ::CreatePen(
+#ifndef WINCE
+                             PS_DOT, 
+#else
+                             PS_DASH,
+#endif
+                             0, PALETTERGB_COLORREF(mColor));
+
 
     ::SelectObject(mDC, tpen);
 
@@ -2752,7 +2820,7 @@ NS_IMETHODIMP nsRenderingContextWin :: CreateDrawingSurface(HDC aDC, nsIDrawingS
 /**
  * ConditionRect is used to fix a coordinate overflow problem under WIN95/WIN98.
  * Some operations fail for rectangles whose coordinates have very large
- * absolute values.  Since these values are off the screen, they can be
+ * absolute values.  Since these values are (hopefully) off the screen, they can be
  * truncated to reasonable ones.
  *
  * @param aSrcRect input rectangle
@@ -2765,14 +2833,17 @@ NS_IMETHODIMP nsRenderingContextWin :: CreateDrawingSurface(HDC aDC, nsIDrawingS
 void 
 nsRenderingContextWin::ConditionRect(nsRect& aSrcRect, RECT& aDestRect)
 {
-   // XXX: TODO find the exact values for the and bottom limits. These limits were determined by
-   // printing out the RECT coordinates and noticing when they failed. There must be an offical
-   // document that describes what the coordinate limits are for calls
-   // such as ::FillRect and ::IntersectClipRect under WIN95 which fail when large negative and
-   // position values are passed.
+  // There is no limit in NT class Windows versions (this includes W2K and XP)
+  if (!gIsWIN95)
+  {
+    aDestRect.top = aSrcRect.y;
+    aDestRect.bottom = aSrcRect.y + aSrcRect.height;
+    aDestRect.left = aSrcRect.x;
+    aDestRect.right = aSrcRect.x + aSrcRect.width;
+    return;
+  }
 
-   // The following is for WIN95. If range of legal values for the rectangles passed for 
-   // clipping and drawing is smaller on WIN95 than under WINNT.                              
+  // The following is for WIN95, WIN98 and WINME
   const nscoord kBottomRightLimit = 16384;
   const nscoord kTopLeftLimit = -8192;
 
@@ -2781,22 +2852,22 @@ nsRenderingContextWin::ConditionRect(nsRect& aSrcRect, RECT& aDestRect)
                       : aSrcRect.y;
   aDestRect.bottom = ((aSrcRect.y + aSrcRect.height) > kBottomRightLimit)
                       ? kBottomRightLimit
-                      : (aSrcRect.y+aSrcRect.height);
+                      : (aSrcRect.y + aSrcRect.height);
   aDestRect.left = (aSrcRect.x < kTopLeftLimit)
                       ? kTopLeftLimit
                       : aSrcRect.x;
   aDestRect.right = ((aSrcRect.x + aSrcRect.width) > kBottomRightLimit)
                       ? kBottomRightLimit
-                      : (aSrcRect.x+aSrcRect.width);
+                      : (aSrcRect.x + aSrcRect.width);
 }
 
 
 NS_IMETHODIMP 
-nsRenderingContextWin::GetBackbuffer(const nsRect &aRequestedSize, const nsRect &aMaxSize, nsIDrawingSurface* &aBackbuffer)
+nsRenderingContextWin::GetBackbuffer(const nsRect &aRequestedSize, const nsRect &aMaxSize, PRBool aForBlending, nsIDrawingSurface* &aBackbuffer)
 {
   // Do not cache the backbuffer. On WIN32 it is faster to get allocate
   // the backbuffer as needed. @see bug 95952
-  return AllocateBackbuffer(aRequestedSize, aMaxSize, aBackbuffer, PR_FALSE);
+  return AllocateBackbuffer(aRequestedSize, aMaxSize, aBackbuffer, PR_FALSE, aForBlending ? NS_CREATEDRAWINGSURFACE_FOR_PIXEL_ACCESS : 0);
 }
  
 NS_IMETHODIMP 
@@ -2814,6 +2885,7 @@ nsRenderingContextWin::ReleaseBackbuffer(void) {
 NS_IMETHODIMP
 nsRenderingContextWin::SetRightToLeftText(PRBool aIsRTL)
 {
+#ifndef WINCE
   // Only call SetTextAlign if the new value is different from the
   // current value
   if (aIsRTL != mRightToLeftText) {
@@ -2828,6 +2900,18 @@ nsRenderingContextWin::SetRightToLeftText(PRBool aIsRTL)
   }
 
   mRightToLeftText = aIsRTL;
+#endif
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsRenderingContextWin::GetRightToLeftText(PRBool* aIsRTL)
+{
+#ifndef WINCE
+  *aIsRTL = mRightToLeftText;
+#else
+  *aIsRTL = PR_FALSE;
+#endif
   return NS_OK;
 }
 
@@ -2838,6 +2922,7 @@ nsRenderingContextWin::SetRightToLeftText(PRBool aIsRTL)
 void
 nsRenderingContextWin::InitBidiInfo()
 {
+#ifndef WINCE
   if (NOT_SETUP == gBidiInfo) {
     gBidiInfo = DONT_INIT;
 
@@ -2880,6 +2965,7 @@ nsRenderingContextWin::InitBidiInfo()
       }
     }
   }
+#endif //WINCE
 }
 
 
