@@ -109,8 +109,6 @@ public:
   nsCOMPtr<nsIFontMetrics> mFontMetrics;
 };
 
-MOZ_DECL_CTOR_COUNTER(GraphicsState)
-
 GraphicsState::GraphicsState() :
   mMatrix(nsnull),
   mColor(NS_RGB(0, 0, 0)),
@@ -255,7 +253,7 @@ nsRenderingContextXlib::GetHints(PRUint32& aResult)
 {
   PRUint32 result = 0;
 
-  // Most X servers implement 8 bit text rendering alot faster than
+  // Most X servers implement 8 bit text rendering a lot faster than
   // XChar2b rendering. In addition, we can avoid the PRUnichar to
   // XChar2b conversion. So we set this bit...
   result |= NS_RENDERING_HINT_FAST_8BIT_TEXT;
@@ -678,7 +676,7 @@ nsRenderingContextXlib::SetLineStyle(nsLineStyle aLineStyle)
   PR_LOG(RenderingContextXlibLM, PR_LOG_DEBUG, ("nsRenderingContextXlib::SetLineStyle()\n"));
 
   if (aLineStyle != mCurrentLineStyle) {
-    /* XXX this isnt done in gtk, copy from there when ready
+    /* XXX this isn't done in gtk, copy from there when ready
     switch(aLineStyle)
       { 
       case nsLineStyle_kSolid:
@@ -1334,15 +1332,9 @@ nsRenderingContextXlib::GetWidth(const char* aString, PRUint32 aLength,
     nsXFont *xFont = mCurrentFont->GetXFont();
 #ifdef USE_FREETYPE
     if (mCurrentFont->IsFreeTypeFont()) {
-      PRUnichar unichars[WIDEN_8_TO_16_BUF_SIZE];
-      // need to fix this for long strings
-      PRUint32 len = PR_MIN(aLength, WIDEN_8_TO_16_BUF_SIZE);
-      // convert 7 bit data to unicode
       // this function is only supposed to be called for ascii data
-      for (PRUint32 i=0; i<len; i++) {
-        unichars[i] = (PRUnichar)((unsigned char)aString[i]);
-      }
-      rawWidth = mCurrentFont->GetWidth(unichars, len);
+      rawWidth = mCurrentFont->
+        GetWidth(NS_ConvertASCIItoUTF16(aString, aLength).get(), aLength);
     }
     else
 #endif /* USE_FREETYPE */
@@ -1583,7 +1575,7 @@ struct BreakGetTextDimensionsData {
   // Remember the fonts that we use so that we can deal with
   // line-breaking in-between fonts later. mOffsets[0] is also used
   // to initialize the current offset from where to start measuring
-  nsVoidArray* mFonts;   // OUT
+  nsVoidArray* mFonts;   // IN/OUT
   nsVoidArray* mOffsets; // IN/OUT
 };
 
@@ -1604,10 +1596,10 @@ do_BreakGetTextDimensions(const nsFontSwitchXlib *aFontSwitch,
   PRInt32 numCharsFit = data->mNumCharsFit;
   nscoord width = data->mWidth;
   PRInt32 start = (PRInt32)(aSubstring - pstr);
-  PRInt32 i = start + aSubstringLength;
+  PRInt32 end = start + aSubstringLength;
   PRBool allDone = PR_FALSE;
 
-  while (start < i) {
+  while (start < end) {
     // Estimate how many characters will fit. Do that by dividing the
     // available space by the average character width
     PRInt32 estimatedNumChars = data->mEstimatedNumChars;
@@ -1627,9 +1619,9 @@ do_BreakGetTextDimensions(const nsFontSwitchXlib *aFontSwitch,
 
     // Avoid scanning the break array in the case where we think all
     // the text should fit
-    if (i <= estimatedBreakOffset) {
+    if (end <= estimatedBreakOffset) {
       // Everything should fit
-      numChars = i - start;
+      numChars = end - start;
     }
     else {
       // Find the nearest place to break that is less than or equal to
@@ -1649,20 +1641,20 @@ do_BreakGetTextDimensions(const nsFontSwitchXlib *aFontSwitch,
       if (start < data->mBreaks[breakIndex]) {
         // The text crosses at least one segment boundary so measure to the
         // break point just before the estimated break offset
-        numChars = PR_MIN(data->mBreaks[breakIndex] - start, (PRInt32)aSubstringLength);
+        numChars = PR_MIN(data->mBreaks[breakIndex], end) - start;
       } 
       else {
         // See whether there is another segment boundary between this one
         // and the end of the text
-        if ((breakIndex < (data->mNumBreaks - 1)) && (data->mBreaks[breakIndex] < i)) {
+        if ((breakIndex < (data->mNumBreaks - 1)) && (data->mBreaks[breakIndex] < end)) {
           ++breakIndex;
-          numChars = PR_MIN(data->mBreaks[breakIndex] - start, (PRInt32)aSubstringLength);
+          numChars = PR_MIN(data->mBreaks[breakIndex], end) - start;
         }
         else {
-          NS_ASSERTION(i != data->mBreaks[breakIndex], "don't expect to be at segment boundary");
+          NS_ASSERTION(end != data->mBreaks[breakIndex], "don't expect to be at segment boundary");
 
           // The text is all within the same segment
-          numChars = i - start;
+          numChars = end - start;
 
           // Remember we're in the middle of a segment and not between
           // two segments
@@ -1726,17 +1718,15 @@ do_BreakGetTextDimensions(const nsFontSwitchXlib *aFontSwitch,
 
       // We can't just revert to the previous break state. Find the break
       // index just before the end of the text
-      i = start + numChars;
-      if (breakIndex == -1) {
-        breakIndex = 0;
-        if (data->mBreaks[breakIndex] < i) {
-          while ((breakIndex + 1 < data->mNumBreaks) && (data->mBreaks[breakIndex + 1] < i)) {
-            ++breakIndex;
-          }
+      end = start + numChars;
+      breakIndex = 0;
+      if (data->mBreaks[breakIndex] < end) {
+        while ((breakIndex + 1 < data->mNumBreaks) && (data->mBreaks[breakIndex + 1] < end)) {
+          ++breakIndex;
         }
       }
 
-      if ((0 == breakIndex) && (i <= data->mBreaks[0])) {
+      if ((0 == breakIndex) && (end <= data->mBreaks[0])) {
         // There's no place to back up to, so even though the text doesn't fit
         // return it anyway
         numCharsFit += numChars;
@@ -1765,21 +1755,52 @@ do_BreakGetTextDimensions(const nsFontSwitchXlib *aFontSwitch,
       // all the way back to the first word
       width += twWidth;
       while ((breakIndex >= 0) && (width > data->mAvailWidth)) {
-        twWidth = 0;
         start = data->mBreaks[breakIndex];
-        numChars = i - start;
+        numChars = end - start;
+        numCharsFit = start;
         if ((1 == numChars) && (pstr[start] == ' ')) {
-          twWidth = data->mSpaceWidth;
+          width -= data->mSpaceWidth;
         }
-        else if (numChars > 0) {
+        else if (pstr + start >= aSubstring) {
+          // The entire fragment to chop is within the current font.
           pxWidth = fontXlib->GetWidth(&pstr[start], numChars);
-          twWidth = NSToCoordRound(float(pxWidth) * data->mP2T);
+          width -= NSToCoordRound(float(pxWidth) * data->mP2T);
+        }
+        else {
+          // The fragment that we want to chop extends back into previous fonts.
+          // We need to reverse into previous fonts. Fortunately,
+          // data->mFonts[] and data->mOffsets[] tell us which fonts are used
+          // and when. 
+          end = data->mNumCharsFit; // same as aSubstring - pstr
+          data->mNumCharsFit = numCharsFit; // has got shorter...
+          PRInt32 k = data->mFonts->Count() - 1;
+          for ( ; k >= 0 && start < end; --k, end -= numChars) {
+            fontXlib = (nsFontXlib*)data->mFonts->ElementAt(k);
+            const PRUnichar* ps = (const PRUnichar*)data->mOffsets->ElementAt(k);
+            if (ps < pstr + start)
+              ps = pstr + start;
+
+            numChars = pstr + end - ps;
+            NS_ASSERTION(numChars > 0, "empty string");
+
+            pxWidth = fontXlib->GetWidth(ps, numChars);
+            data->mWidth -= NSToCoordRound(float(pxWidth) * data->mP2T);
+
+            // By construction, mFonts[k] is the last font, and
+            // mOffsets[k+1] is the last offset.
+            data->mFonts->RemoveElementAt(k);
+            data->mOffsets->RemoveElementAt(k+1);
+          }
+
+          // We are done, update the data now because we won't do it later.
+          // The |if (data->mNumCharsFit != numCharsFit)| won't apply below
+          data->mFonts->AppendElement(fontXlib);
+          data->mOffsets->AppendElement((void*)&pstr[numCharsFit]);
+          break;
         }
 
-        width -= twWidth;
-        numCharsFit = start;
         --breakIndex;
-        i = start;
+        end = start;
       }
     }
 
@@ -1787,7 +1808,7 @@ do_BreakGetTextDimensions(const nsFontSwitchXlib *aFontSwitch,
   }
 
 #ifdef DEBUG_rbs
-  NS_ASSERTION(allDone || start == i, "internal error");
+  NS_ASSERTION(allDone || start == end, "internal error");
   NS_ASSERTION(allDone || data->mNumCharsFit != numCharsFit, "internal error");
 #endif /* DEBUG_rbs */
 
@@ -2066,16 +2087,10 @@ nsRenderingContextXlib::DrawString(const char *aString, PRUint32 aLength,
         mTranMatrix->TransformCoord(&xx, &yy);
 #ifdef USE_FREETYPE
         if (mCurrentFont->IsFreeTypeFont()) {
-          PRUnichar unichars[WIDEN_8_TO_16_BUF_SIZE];
-          // need to fix this for long strings
-          PRUint32 len = PR_MIN(aLength, WIDEN_8_TO_16_BUF_SIZE);
-          // convert 7 bit data to unicode
           // this function is only supposed to be called for ascii data
-          for (PRUint32 i=0; i<len; i++) {
-            unichars[i] = (PRUnichar)((unsigned char)aString[i]);
-          }
-          res = mCurrentFont->DrawString(this, mSurface, xx, yy,
-                                         unichars, len);
+          res = mCurrentFont->DrawString(this, mSurface, xx, yy, 
+                            NS_ConvertASCIItoUTF16(aString, aLength).get(),
+                            aLength);
         }
         else
 #endif /* USE_FREETYPE */
@@ -2097,16 +2112,10 @@ nsRenderingContextXlib::DrawString(const char *aString, PRUint32 aLength,
       mTranMatrix->TransformCoord(&x, &y);
 #ifdef USE_FREETYPE
       if (mCurrentFont->IsFreeTypeFont()) {
-        PRUnichar unichars[WIDEN_8_TO_16_BUF_SIZE];
-        // need to fix this for long strings
-        PRUint32 len = PR_MIN(aLength, WIDEN_8_TO_16_BUF_SIZE);
-        // convert 7 bit data to unicode
         // this function is only supposed to be called for ascii data
-        for (PRUint32 i=0; i<len; i++) {
-          unichars[i] = (PRUnichar)((unsigned char)aString[i]);
-        }
-        res = mCurrentFont->DrawString(this, mSurface, x, y,
-                                       unichars, len);
+        res = mCurrentFont->DrawString(this, mSurface, x, y, 
+                          NS_ConvertASCIItoUTF16(aString, aLength).get(),
+                          aLength);
       }
       else
 #endif /* USE_FREETYPE */
@@ -2129,9 +2138,7 @@ nsRenderingContextXlib::DrawString(const char *aString, PRUint32 aLength,
   //will take it's place that will need this code again. MMP
   if (mFontMetrics)
   {
-    const nsFont *font;
-    mFontMetrics->GetFont(font);
-    PRUint8 deco = font->decorations;
+    PRUint8 deco = mFontMetrics->Font().decorations;
 
     if (deco & NS_FONT_DECORATION_OVERLINE)
       DrawLine(aX, aY, aX + aWidth, aY);
@@ -2321,13 +2328,6 @@ nsRenderingContextXlib::CopyOffScreenBits(nsIDrawingSurface* aSrcSurf, PRInt32 a
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsRenderingContextXlib::RetrieveCurrentNativeGraphicData(PRUint32 * ngd)
-{
-  PR_LOG(RenderingContextXlibLM, PR_LOG_DEBUG, ("nsRenderingContextXlib::RetrieveCurrentNativeGraphicData()\n"));
-  return NS_OK;
-}
-
 #ifdef MOZ_MATHML
 static
 void Widen8To16AndGetTextExtents(nsXFont    *xFont,  
@@ -2381,16 +2381,10 @@ nsRenderingContextXlib::GetBoundingMetrics(const char*        aString,
     nsXFont *xFont = mCurrentFont->GetXFont();
 #ifdef USE_FREETYPE
     if (mCurrentFont->IsFreeTypeFont()) {
-      PRUnichar unichars[WIDEN_8_TO_16_BUF_SIZE];
-      // need to fix this for long strings
-      PRUint32 len = PR_MIN(aLength, WIDEN_8_TO_16_BUF_SIZE);
-      // convert 7 bit data to unicode
       // this function is only supposed to be called for ascii data
-      for (PRUint32 i=0; i<len; i++) {
-        unichars[i] = (PRUnichar)((unsigned char)aString[i]);
-      }
-      res = mCurrentFont->GetBoundingMetrics(unichars, len,
-                                            aBoundingMetrics);
+      res = mCurrentFont->GetBoundingMetrics(
+                        NS_ConvertASCIItoUTF16(aString, aLength).get(),
+                        aLength, aBoundingMetrics);
     }
     else
 #endif /* USE_FREETYPE */
@@ -2511,12 +2505,15 @@ nsRenderingContextXlib::DrawImage(imgIContainer *aImage, const nsRect & aSrcRect
 }
 
 NS_IMETHODIMP
-nsRenderingContextXlib::GetBackbuffer(const nsRect &aRequestedSize, const nsRect &aMaxSize, nsIDrawingSurface* &aBackbuffer)
+nsRenderingContextXlib::GetBackbuffer(const nsRect &aRequestedSize,
+                                      const nsRect &aMaxSize,
+                                      PRBool aForBlending,
+                                      nsIDrawingSurface* &aBackbuffer)
 {
   PR_LOG(RenderingContextXlibLM, PR_LOG_DEBUG, ("nsRenderingContextXlib::GetBackbuffer()\n"));
   /* Do not cache the backbuffer. On X11 it is more efficient to allocate
    * the backbuffer as needed and it doesn't cause a performance hit. @see bug 95952 */
-  return AllocateBackbuffer(aRequestedSize, aMaxSize, aBackbuffer, PR_FALSE);
+  return AllocateBackbuffer(aRequestedSize, aMaxSize, aBackbuffer, PR_FALSE, 0);
 }
  
 NS_IMETHODIMP
