@@ -45,7 +45,7 @@
 #include "nsStringBundleTextOverride.h"
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
-#include "nsArray.h"
+#include "nsIMutableArray.h"
 #include "nsArrayEnumerator.h"
 #include "nscore.h"
 #include "nsHashtable.h"
@@ -74,9 +74,6 @@
 #include "nsIStringStream.h"
 #endif
 
-// eventQ
-#include "nsIEventQueueService.h"
-
 #include "prenv.h"
 #include "nsCRT.h"
 
@@ -106,8 +103,8 @@ nsStringBundle::LoadProperties()
   if (mAttemptedLoad) {
     if (mLoaded)
       return NS_OK;
-    else
-      return NS_ERROR_UNEXPECTED;
+      
+    return NS_ERROR_UNEXPECTED;
   }
   
   mAttemptedLoad = PR_TRUE;
@@ -171,7 +168,7 @@ nsStringBundle::GetStringFromID(PRInt32 aID, nsAString& aResult)
   char *s = ToNewCString(aResult);
   printf("\n** GetStringFromID: aResult=%s, len=%d\n", s?s:"null", 
          aResult.Length());
-  delete s;
+  if (s) nsMemory::Free(s);
 #endif /* DEBUG_tao_ */
 
   return rv;
@@ -186,18 +183,19 @@ nsStringBundle::GetStringFromName(const nsAString& aName,
   // try override first
   if (mOverrideStrings) {
     rv = mOverrideStrings->GetStringFromName(mPropertiesURL,
-                                             NS_ConvertUCS2toUTF8(aName),
+                                             NS_ConvertUTF16toUTF8(aName),
                                              aResult);
     if (NS_SUCCEEDED(rv)) return rv;
   }
   
-  rv = mProps->GetStringProperty(NS_ConvertUCS2toUTF8(aName), aResult);
+  rv = mProps->GetStringProperty(NS_ConvertUTF16toUTF8(aName), aResult);
 #ifdef DEBUG_tao_
   char *s = ToNewCString(aResult),
        *ss = ToNewCString(aName);
   printf("\n** GetStringFromName: aName=%s, aResult=%s, len=%d\n", 
          ss?ss:"null", s?s:"null", aResult.Length());
-  delete s;
+  if (s)  nsMemory::Free(s);
+  if (ss) nsMemory::Free(ss);
 #endif /* DEBUG_tao_ */
   return rv;
 }
@@ -221,6 +219,10 @@ nsStringBundle::FormatStringFromName(const PRUnichar *aName,
                                      PRUint32 aLength,
                                      PRUnichar **aResult)
 {
+  NS_ENSURE_ARG_POINTER(aName);
+  NS_ASSERTION(aParams && aLength, "FormatStringFromName() without format parameters: use GetStringFromName() instead");
+  NS_ENSURE_ARG_POINTER(aResult);
+
   nsresult rv;
   rv = LoadProperties();
   if (NS_FAILED(rv)) return rv;
@@ -247,16 +249,22 @@ nsStringBundle::GetStringFromID(PRInt32 aID, PRUnichar **aResult)
   *aResult = nsnull;
   nsAutoString tmpstr;
 
-  nsresult ret = GetStringFromID(aID, tmpstr);
-  if (NS_SUCCEEDED(ret))
-    *aResult = ToNewUnicode(tmpstr);
-  return ret;
+  rv = GetStringFromID(aID, tmpstr);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  *aResult = ToNewUnicode(tmpstr);
+  NS_ENSURE_TRUE(*aResult, NS_ERROR_OUT_OF_MEMORY);
+
+  return NS_OK;
 }
 
 /* void GetStringFromName (in wstring aName, out wstring aResult); */
 NS_IMETHODIMP 
 nsStringBundle::GetStringFromName(const PRUnichar *aName, PRUnichar **aResult)
 {
+  NS_ENSURE_ARG_POINTER(aName);
+  NS_ENSURE_ARG_POINTER(aResult);
+
   nsresult rv;
   rv = LoadProperties();
   if (NS_FAILED(rv)) return rv;
@@ -265,10 +273,21 @@ nsStringBundle::GetStringFromName(const PRUnichar *aName, PRUnichar **aResult)
   *aResult = nsnull;
   nsAutoString tmpstr;
   rv = GetStringFromName(nsDependentString(aName), tmpstr);
-  if (NS_SUCCEEDED(rv))
-    *aResult = ToNewUnicode(tmpstr);
-  
-  return rv;
+  if (NS_FAILED(rv))
+  {
+#if 0
+    // it is not uncommon for apps to request a string name which may not exist
+    // so be quiet about it. 
+    NS_WARNING("String missing from string bundle");
+    printf("  '%s' missing from bundle %s\n", NS_ConvertUTF16toUTF8(aName).get(), mPropertiesURL.get());
+#endif
+    return rv;
+  }
+
+  *aResult = ToNewUnicode(tmpstr);
+  NS_ENSURE_TRUE(*aResult, NS_ERROR_OUT_OF_MEMORY);
+
+  return NS_OK;
 }
 
 nsresult
@@ -280,8 +299,9 @@ nsStringBundle::GetCombinedEnumeration(nsIStringBundleOverride* aOverrideStrings
   
   nsresult rv;
 
-  nsCOMPtr<nsIMutableArray> resultArray;
-  NS_NewArray(getter_AddRefs(resultArray));
+  nsCOMPtr<nsIMutableArray> resultArray =
+    do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // first, append the override elements
   nsCOMPtr<nsISimpleEnumerator> overrideEnumerator;
@@ -289,14 +309,16 @@ nsStringBundle::GetCombinedEnumeration(nsIStringBundleOverride* aOverrideStrings
                                                getter_AddRefs(overrideEnumerator));
   
   PRBool hasMore;
-  overrideEnumerator->HasMoreElements(&hasMore);
+  rv = overrideEnumerator->HasMoreElements(&hasMore);
+  NS_ENSURE_SUCCESS(rv, rv);
   while (hasMore) {
 
     rv = overrideEnumerator->GetNext(getter_AddRefs(supports));
     if (NS_SUCCEEDED(rv))
       resultArray->AppendElement(supports, PR_FALSE);
 
-    overrideEnumerator->HasMoreElements(&hasMore);
+    rv = overrideEnumerator->HasMoreElements(&hasMore);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
   // ok, now we have the override elements in resultArray
@@ -325,7 +347,8 @@ nsStringBundle::GetCombinedEnumeration(nsIStringBundleOverride* aOverrideStrings
         resultArray->AppendElement(propElement, PR_FALSE);
     }
 
-    propEnumerator->HasMoreElements(&hasMore);
+    rv = propEnumerator->HasMoreElements(&hasMore);
+    NS_ENSURE_SUCCESS(rv, rv);
   } while (hasMore);
 
   return resultArray->Enumerate(aResult);
@@ -696,11 +719,7 @@ nsStringBundleService::CreateBundle(const char* aURLSpec,
 {
 #ifdef DEBUG_tao_
   printf("\n++ nsStringBundleService::CreateBundle ++\n");
-  {
-    printf("\n** nsStringBundleService::CreateBundle: %s\n",
-           aURLSpec ? aURLSpec : "null");
-    delete s;
-  }
+  printf("\n** nsStringBundleService::CreateBundle: %s\n", aURLSpec ? aURLSpec : "null");
 #endif
 
   return getStringBundle(aURLSpec,aResult);
@@ -744,7 +763,7 @@ nsStringBundleService::FormatWithBundle(nsIStringBundle* bundle, nsresult aStatu
 
   // first try looking up the error message with the string key:
   if (NS_SUCCEEDED(rv)) {
-    rv = bundle->FormatStringFromName(NS_ConvertASCIItoUCS2(key).get(),
+    rv = bundle->FormatStringFromName(NS_ConvertASCIItoUTF16(key).get(),
                                       (const PRUnichar**)argArray, 
                                       argCount, result);
   }
@@ -781,6 +800,7 @@ nsStringBundleService::FormatStatusMessage(nsresult aStatus,
   // XXX hack for mailnews who has already formatted their messages:
   if (aStatus == NS_OK && aStatusArg) {
     *result = nsCRT::strdup(aStatusArg);
+    NS_ENSURE_TRUE(*result, NS_ERROR_OUT_OF_MEMORY);
     return NS_OK;
   }
 
