@@ -50,17 +50,15 @@
 #include "nsIObserverService.h"
 #include "nsICategoryManager.h"
 #include "nsIJSRuntimeService.h"
-#include "nsIEventQueueService.h"
+#include "nsIThreadInternal.h"
+#include "nsThreadUtils.h"
 #include "nsMemory.h"
 #include "jsdebug.h"
 #include "nsReadableUtils.h"
 #include "nsCRT.h"
 
-/* XXX this stuff is used by NestEventLoop, a temporary hack to be refactored
- * later */
-#include "nsWidgetsCID.h"
+/* XXX DOM dependency */
 #include "nsIScriptContext.h"
-#include "nsIAppShell.h"
 #include "nsIJSContextStack.h"
 
 /*
@@ -108,7 +106,7 @@
 #define AUTOREG_CATEGORY  "xpcom-autoregistration"
 #define APPSTART_CATEGORY "app-startup"
 #define JSD_AUTOREG_ENTRY "JSDebugger Startup Observer"
-#define JSD_STARTUP_ENTRY "JSDebugger Startup Observer,service"
+#define JSD_STARTUP_ENTRY "JSDebugger Startup Observer"
 
 JS_STATIC_DLL_CALLBACK (JSBool)
 jsds_GCCallbackProc (JSContext *cx, JSGCStatus status);
@@ -118,11 +116,10 @@ jsds_GCCallbackProc (JSContext *cx, JSGCStatus status);
  ******************************************************************************/
 
 const char implementationString[] = "Mozilla JavaScript Debugger Service";
-static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
-static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 
 const char jsdServiceCtrID[] = "@mozilla.org/js/jsd/debugger-service;1";
-const char jsdASObserverCtrID[] = "@mozilla.org/js/jsd/app-start-observer;2";
+const char jsdARObserverCtrID[] = "@mozilla.org/js/jsd/app-start-observer;2";
+const char jsdASObserverCtrID[] = "service,@mozilla.org/js/jsd/app-start-observer;2";
 
 #ifdef DEBUG_verbose
 PRUint32 gScriptCount   = 0;
@@ -1017,6 +1014,7 @@ PCMapEntry *
 jsdScript::CreatePPLineMap()
 {    
     JSContext  *cx  = JSD_GetDefaultJSContext (mCx);
+    JSAutoRequest ar(cx);
     JSObject   *obj = JS_NewObject(cx, NULL, NULL, NULL);
     JSFunction *fun = JSD_GetJSFunction (mCx, mScript);
     JSScript   *script;
@@ -1262,18 +1260,19 @@ jsdScript::GetFunctionSource(nsAString & aFunctionSource)
         return NS_ERROR_FAILURE;
     }
     JSFunction *fun = JSD_GetJSFunction (mCx, mScript);
+
+    JSAutoRequest ar(cx);
+
     JSString *jsstr;
     if (fun)
-    {
         jsstr = JS_DecompileFunction (cx, fun, 4);
-    }
-    else
-    {
+    else {
         JSScript *script = JSD_GetJSScript (mCx, mScript);
         jsstr = JS_DecompileScript (cx, script, "ppscript", 4);
     }
     if (!jsstr)
         return NS_ERROR_FAILURE;
+
     aFunctionSource = NS_REINTERPRET_CAST(PRUnichar*, JS_GetStringChars(jsstr));
     return NS_OK;
 }
@@ -1332,6 +1331,30 @@ jsdScript::GetTotalExecutionTime(double *_rval)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+jsdScript::GetMinOwnExecutionTime(double *_rval)
+{
+    ASSERT_VALID_EPHEMERAL;
+    *_rval = JSD_GetScriptMinOwnExecutionTime (mCx, mScript);
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+jsdScript::GetMaxOwnExecutionTime(double *_rval)
+{
+    ASSERT_VALID_EPHEMERAL;
+    *_rval = JSD_GetScriptMaxOwnExecutionTime (mCx, mScript);
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+jsdScript::GetTotalOwnExecutionTime(double *_rval)
+{
+    ASSERT_VALID_EPHEMERAL;
+    *_rval = JSD_GetScriptTotalOwnExecutionTime (mCx, mScript);
+    return NS_OK;
+}
+
 NS_IMETHODIMP
 jsdScript::ClearProfileData()
 {
@@ -1887,6 +1910,9 @@ jsdStackFrame::Eval (const nsAString &bytes, const char *fileName,
     jsval jv;
 
     JSContext *cx = JSD_GetJSContext (mCx, mThreadState);
+
+    JSAutoRequest ar(cx);
+
     estate = JS_SaveExceptionState (cx);
     JS_ClearPendingException (cx);
 
@@ -1902,6 +1928,7 @@ jsdStackFrame::Eval (const nsAString &bytes, const char *fileName,
     }
 
     JS_RestoreExceptionState (cx, estate);
+
     JSDValue *jsdv = JSD_NewValue (mCx, jv);
     if (!jsdv)
         return NS_ERROR_FAILURE;
@@ -2169,18 +2196,20 @@ NS_IMETHODIMP
 jsdValue::GetProperties (jsdIProperty ***propArray, PRUint32 *length)
 {
     ASSERT_VALID_EPHEMERAL;
-    if (!JSD_IsValueObject(mCx, mValue)) {
+    *propArray = nsnull;
+    if (length)
         *length = 0;
-        *propArray = 0;
-        return NS_OK;
-    }
 
-    jsdIProperty **pa_temp;
-    PRUint32 prop_count = JSD_GetCountOfProperties (mCx, mValue);
-    
-    pa_temp = NS_STATIC_CAST(jsdIProperty **,
-                             nsMemory::Alloc(sizeof (jsdIProperty *) * 
-                                             prop_count));
+    PRUint32 prop_count = JSD_IsValueObject(mCx, mValue)
+        ? JSD_GetCountOfProperties (mCx, mValue)
+        : 0;
+    NS_ENSURE_TRUE(prop_count, NS_OK);
+
+    jsdIProperty **pa_temp =
+        NS_STATIC_CAST(jsdIProperty **,
+                       nsMemory::Alloc(sizeof (jsdIProperty *) * 
+                                       prop_count));
+    NS_ENSURE_TRUE(pa_temp, NS_ERROR_OUT_OF_MEMORY);
 
     PRUint32     i    = 0;
     JSDProperty *iter = NULL;
@@ -2205,8 +2234,13 @@ jsdValue::GetProperty (const char *name, jsdIProperty **_rval)
 {
     ASSERT_VALID_EPHEMERAL;
     JSContext *cx = JSD_GetDefaultJSContext (mCx);
+
+    JSAutoRequest ar(cx);
+
     /* not rooting this */
     JSString *jstr_name = JS_NewStringCopyZ (cx, name);
+    if (!jstr_name)
+        return NS_ERROR_OUT_OF_MEMORY;
 
     JSDProperty *prop = JSD_GetValueProperty (mCx, mValue, jstr_name);
     
@@ -2349,7 +2383,7 @@ jsdService::SetInitAtStartup (PRBool state)
     if (state) {
         rv = categoryManager->AddCategoryEntry(AUTOREG_CATEGORY,
                                                JSD_AUTOREG_ENTRY,
-                                               jsdASObserverCtrID,
+                                               jsdARObserverCtrID,
                                                PR_TRUE, PR_TRUE, nsnull);
         if (NS_FAILED(rv))
             return rv;
@@ -2378,6 +2412,7 @@ jsdService::SetInitAtStartup (PRBool state)
 NS_IMETHODIMP
 jsdService::GetFlags (PRUint32 *_rval)
 {
+    ASSERT_VALID_CONTEXT;
     *_rval = JSD_GetContextFlags (mCx);
     return NS_OK;
 }
@@ -2385,6 +2420,7 @@ jsdService::GetFlags (PRUint32 *_rval)
 NS_IMETHODIMP
 jsdService::SetFlags (PRUint32 flags)
 {
+    ASSERT_VALID_CONTEXT;
     JSD_SetContextFlags (mCx, flags);
     return NS_OK;
 }
@@ -2560,6 +2596,9 @@ jsdService::GetPauseDepth(PRUint32 *_rval)
 NS_IMETHODIMP
 jsdService::Pause(PRUint32 *_rval)
 {
+    if (!mCx)
+        return NS_ERROR_NOT_INITIALIZED;
+
     if (++mPauseLevel == 1) {
         JSD_SetErrorReporter (mCx, NULL, NULL);
         JSD_ClearThrowHook (mCx);
@@ -2579,6 +2618,9 @@ jsdService::Pause(PRUint32 *_rval)
 NS_IMETHODIMP
 jsdService::UnPause(PRUint32 *_rval)
 {
+    if (!mCx)
+        return NS_ERROR_NOT_INITIALIZED;
+
     if (mPauseLevel == 0)
         return NS_ERROR_NOT_AVAILABLE;
 
@@ -2913,40 +2955,28 @@ jsdService::WrapValue(jsdIValue **_rval)
 NS_IMETHODIMP
 jsdService::EnterNestedEventLoop (jsdINestCallback *callback, PRUint32 *_rval)
 {
-    nsCOMPtr<nsIAppShell> appShell(do_CreateInstance(kAppShellCID));
-    NS_ENSURE_TRUE(appShell, NS_ERROR_FAILURE);
-    nsCOMPtr<nsIEventQueueService> 
-        eventService(do_GetService(kEventQueueServiceCID));
-    NS_ENSURE_TRUE(eventService, NS_ERROR_FAILURE);
-    
-    appShell->Create(0, nsnull);
-    appShell->Spinup();
-
+    // Nesting event queues is a thing of the past.  Now, we just spin the
+    // current event loop.
+ 
     nsCOMPtr<nsIJSContextStack> 
         stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1"));
     nsresult rv = NS_OK;
     PRUint32 nestLevel = ++mNestedLoopLevel;
     
-    nsCOMPtr<nsIEventQueue> eventQ;
-    if (stack && NS_SUCCEEDED(stack->Push(nsnull)) &&
-        NS_SUCCEEDED(eventService->PushThreadEventQueue(getter_AddRefs(eventQ))))
-    {
-        if (NS_SUCCEEDED(rv) && callback) {
+    nsCOMPtr<nsIThread> thread = do_GetCurrentThread();
+
+    if (stack && NS_SUCCEEDED(stack->Push(nsnull))) {
+        if (callback) {
             Pause(nsnull);
             rv = callback->OnNest();
             UnPause(nsnull);
         }
         
-        while(NS_SUCCEEDED(rv) && mNestedLoopLevel >= nestLevel)
-        {
-            void* data;
-            PRBool isRealEvent;
-            //PRBool processEvent;
-            
-            rv = appShell->GetNativeEvent(isRealEvent, data);
-            if(NS_SUCCEEDED(rv))
-                appShell->DispatchNativeEvent(isRealEvent, data);
-            }
+        while (NS_SUCCEEDED(rv) && mNestedLoopLevel >= nestLevel) {
+            if (!NS_ProcessNextEvent(thread))
+                rv = NS_ERROR_UNEXPECTED;
+        }
+
         JSContext* cx;
         stack->Pop(&cx);
         NS_ASSERTION(cx == nsnull, "JSContextStack mismatch");
@@ -2954,9 +2984,6 @@ jsdService::EnterNestedEventLoop (jsdINestCallback *callback, PRUint32 *_rval)
     else
         rv = NS_ERROR_FAILURE;
     
-    eventService->PopThreadEventQueue(eventQ);
-    appShell->Spindown();
-
     NS_ASSERTION (mNestedLoopLevel <= nestLevel,
                   "nested event didn't unwind properly");
     if (mNestedLoopLevel == nestLevel)
@@ -3290,7 +3317,7 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(jsdASObserver)
 
 static const nsModuleComponentInfo components[] = {
     {"JSDService", JSDSERVICE_CID,    jsdServiceCtrID, jsdServiceConstructor},
-    {"JSDASObserver",  JSDASO_CID, jsdASObserverCtrID, jsdASObserverConstructor}
+    {"JSDASObserver",  JSDASO_CID, jsdARObserverCtrID, jsdASObserverConstructor}
 };
 
 NS_IMPL_NSGETMODULE(JavaScript_Debugger, components)
