@@ -1,4 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=8 sw=4 et tw=78:
  *
  * ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
@@ -63,6 +64,9 @@
 
 #if JS_HAS_SCRIPT_OBJECT
 
+static const char js_script_exec[] = "Script.prototype.exec";
+static const char js_script_compile[] = "Script.prototype.compile";
+
 #if JS_HAS_TOSOURCE
 static JSBool
 script_toSource(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
@@ -176,6 +180,7 @@ script_compile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     str = js_ValueToString(cx, argv[0]);
     if (!str)
         return JS_FALSE;
+    argv[0] = STRING_TO_JSVAL(str);
 
     /* Compile using the caller's scope chain, which js_Invoke passes to fp. */
     fp = cx->fp;
@@ -189,8 +194,12 @@ script_compile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
         argv[1] = OBJECT_TO_JSVAL(scopeobj);
     }
     if (caller) {
-        if (!scopeobj)
-            scopeobj = caller->scopeChain;
+        if (!scopeobj) {
+            scopeobj = js_GetScopeChain(cx, caller);
+            if (!scopeobj)
+                return JS_FALSE;
+            fp->scopeChain = scopeobj;  /* for the compiler's benefit */
+        }
 
         file = caller->script->filename;
         line = js_PCToLineNumber(cx, caller->script, caller->pc);
@@ -201,6 +210,11 @@ script_compile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
         principals = NULL;
     }
 
+    /* Ensure we compile this script with the right (inner) principals. */
+    scopeobj = js_CheckScopeChainValidity(cx, scopeobj, js_script_compile);
+    if (!scopeobj)
+        return JS_FALSE;
+
     /*
      * Compile the new script using the caller's scope chain, a la eval().
      * Unlike jsobj.c:obj_eval, however, we do not set JSFRAME_EVAL in fp's
@@ -209,6 +223,7 @@ script_compile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
      * tested in jsemit.c and jsscan.c to optimize based on identity of run-
      * and compile-time scope.
      */
+    fp->flags |= JSFRAME_SCRIPT_OBJECT;
     script = JS_CompileUCScriptForPrincipals(cx, scopeobj, principals,
                                              JSSTRING_CHARS(str),
                                              JSSTRING_LENGTH(str),
@@ -238,6 +253,7 @@ script_exec(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     JSScript *script;
     JSObject *scopeobj, *parent;
     JSStackFrame *fp, *caller;
+    JSPrincipals *principals;
 
     if (!JS_InstanceOf(cx, obj, &js_ScriptClass, argv))
         return JS_FALSE;
@@ -268,7 +284,7 @@ script_exec(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     caller = JS_GetScriptedCaller(cx, fp);
     if (caller && !caller->varobj) {
         /* Called from a lightweight function. */
-        JS_ASSERT(caller->fun && !(caller->fun->flags & JSFUN_HEAVYWEIGHT));
+        JS_ASSERT(caller->fun && !JSFUN_HEAVYWEIGHT_TEST(caller->fun->flags));
 
         /* Scope chain links from Call object to callee's parent. */
         parent = OBJ_GET_PARENT(cx, JSVAL_TO_OBJECT(caller->argv[-2]));
@@ -283,7 +299,9 @@ script_exec(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
              * Load caller->scopeChain after the conditional js_GetCallObject
              * call above, which resets scopeChain as well as varobj.
              */
-            scopeobj = caller->scopeChain;
+            scopeobj = js_GetScopeChain(cx, caller);
+            if (!scopeobj)
+                return JS_FALSE;
         } else {
             /*
              * Called from native code, so we don't know what scope object to
@@ -297,93 +315,88 @@ script_exec(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
         }
     }
 
+    scopeobj = js_CheckScopeChainValidity(cx, scopeobj, js_script_exec);
+    if (!scopeobj)
+        return JS_FALSE;
+
+    /* Belt-and-braces: check that this script object has access to scopeobj. */
+    principals = script->principals;
+    if (!js_CheckPrincipalsAccess(cx, scopeobj, principals,
+                                  CLASS_ATOM(cx, Script))) {
+        return JS_FALSE;
+    }
+
     return js_Execute(cx, scopeobj, script, caller, JSFRAME_EVAL, rval);
 }
 
 #if JS_HAS_XDR
 
-static JSBool
-XDRAtomListElement(JSXDRState *xdr, JSAtomListElement *ale)
-{
-    jsval value;
-    jsatomid index;
-
-    if (xdr->mode == JSXDR_ENCODE)
-        value = ATOM_KEY(ALE_ATOM(ale));
-
-    index = ALE_INDEX(ale);
-    if (!JS_XDRUint32(xdr, &index))
-        return JS_FALSE;
-    ALE_SET_INDEX(ale, index);
-
-    if (!JS_XDRValue(xdr, &value))
-        return JS_FALSE;
-
-    if (xdr->mode == JSXDR_DECODE) {
-        if (!ALE_SET_ATOM(ale, js_AtomizeValue(xdr->cx, value, 0)))
-            return JS_FALSE;
-    }
-    return JS_TRUE;
-}
-
 static JSBool
 XDRAtomMap(JSXDRState *xdr, JSAtomMap *map)
 {
-    uint32 length;
-    uintN i;
-    JSBool ok;
+    JSContext *cx;
+    uint32 natoms, i, index;
+    JSAtom **atoms;
+
+    cx = xdr->cx;
 
     if (xdr->mode == JSXDR_ENCODE)
-        length = map->length;
+        natoms = (uint32)map->length;
 
-    if (!JS_XDRUint32(xdr, &length))
+    if (!JS_XDRUint32(xdr, &natoms))
         return JS_FALSE;
 
-    if (xdr->mode == JSXDR_DECODE) {
-        JSContext *cx;
-        void *mark;
-        JSAtomList al;
-        JSAtomListElement *ale;
-
-        cx = xdr->cx;
-        mark = JS_ARENA_MARK(&cx->tempPool);
-        ATOM_LIST_INIT(&al);
-        for (i = 0; i < length; i++) {
-            JS_ARENA_ALLOCATE_TYPE(ale, JSAtomListElement, &cx->tempPool);
-            if (!ale ||
-                !XDRAtomListElement(xdr, ale)) {
-                if (!ale)
-                    JS_ReportOutOfMemory(cx);
-                JS_ARENA_RELEASE(&cx->tempPool, mark);
-                return JS_FALSE;
-            }
-            ALE_SET_NEXT(ale, al.list);
-            al.count++;
-            al.list = ale;
-        }
-        ok = js_InitAtomMap(cx, map, &al);
-        JS_ARENA_RELEASE(&cx->tempPool, mark);
-        return ok;
-    }
-
     if (xdr->mode == JSXDR_ENCODE) {
-        JSAtomListElement ale;
-
-        for (i = 0; i < map->length; i++) {
-            ALE_SET_ATOM(&ale, map->vector[i]);
-            ALE_SET_INDEX(&ale, i);
-            if (!XDRAtomListElement(xdr, &ale))
+        atoms = map->vector;
+    } else {
+        if (natoms == 0) {
+            atoms = NULL;
+        } else {
+            atoms = (JSAtom **) JS_malloc(cx, (size_t)natoms * sizeof *atoms);
+            if (!atoms)
                 return JS_FALSE;
+#ifdef DEBUG
+            memset(atoms, 0, (size_t)natoms * sizeof *atoms);
+#endif
         }
+
+        map->vector = atoms;
+        map->length = natoms;
     }
+
+    for (i = 0; i != natoms; ++i) {
+        if (xdr->mode == JSXDR_ENCODE)
+            index = i;
+        if (!JS_XDRUint32(xdr, &index))
+            goto bad;
+
+        /*
+         * Assert that, when decoding, the read index is valid and points to
+         * an unoccupied element of atoms array.
+         */
+        JS_ASSERT(index < natoms);
+        JS_ASSERT(xdr->mode == JSXDR_ENCODE || !atoms[index]);
+        if (!js_XDRAtom(xdr, &atoms[index]))
+            goto bad;
+    }
+
     return JS_TRUE;
+
+  bad:
+    if (xdr->mode == JSXDR_DECODE) {
+        JS_free(cx, atoms);
+        map->vector = NULL;
+        map->length = 0;
+    }
+
+    return JS_FALSE;
 }
 
 JSBool
 js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
 {
     JSContext *cx;
-    JSScript *script, *newscript;
+    JSScript *script, *newscript, *oldscript;
     uint32 length, lineno, depth, magic, nsrcnotes, ntrynotes;
     uint32 prologLength, version;
     JSBool filenameWasSaved;
@@ -402,15 +415,21 @@ js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
      * nsrcnotes and ntrynotes fields to come before everything except magic,
      * length, prologLength, and version, so that srcnote and trynote storage
      * can be allocated as part of the JSScript (along with bytecode storage).
+     *
+     * So far, the magic number has not changed for every jsopcode.tbl change.
+     * We stipulate forward compatibility by requiring old bytecodes never to
+     * change or go away (modulo a few exceptions before the XDR interfaces
+     * evolved, and a few exceptions during active trunk development).  With
+     * the addition of JSOP_STOP to support JS_THREADED_INTERP, we make a new
+     * magic number (_5) so that we know to append JSOP_STOP to old scripts
+     * when deserializing.
      */
     if (xdr->mode == JSXDR_ENCODE)
         magic = JSXDR_MAGIC_SCRIPT_CURRENT;
     if (!JS_XDRUint32(xdr, &magic))
         return JS_FALSE;
-    if (magic != JSXDR_MAGIC_SCRIPT_4 &&
-        magic != JSXDR_MAGIC_SCRIPT_3 &&
-        magic != JSXDR_MAGIC_SCRIPT_2 &&
-        magic != JSXDR_MAGIC_SCRIPT_1) {
+    JS_ASSERT((uint32)JSXDR_MAGIC_SCRIPT_5 - (uint32)JSXDR_MAGIC_SCRIPT_1 == 4);
+    if (magic - (uint32)JSXDR_MAGIC_SCRIPT_1 > 4) {
         if (!hasMagic) {
             JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL,
                                  JSMSG_BAD_SCRIPT_MAGIC);
@@ -463,7 +482,11 @@ js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
     }
 
     if (xdr->mode == JSXDR_DECODE) {
-        script = js_NewScript(cx, length, nsrcnotes, ntrynotes);
+        size_t alloclength = length;
+        if (magic < JSXDR_MAGIC_SCRIPT_5)
+            ++alloclength;      /* add a byte for JSOP_STOP */
+
+        script = js_NewScript(cx, alloclength, nsrcnotes, ntrynotes);
         if (!script)
             return JS_FALSE;
         if (magic >= JSXDR_MAGIC_SCRIPT_2) {
@@ -484,18 +507,32 @@ js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
      * in the (DECODE and magic < _4) case must point at a temporary vector
      * allocated just below.
      */
+    oldscript = xdr->script;
+    xdr->script = script;
     if (!JS_XDRBytes(xdr, (char *)script->code, length * sizeof(jsbytecode)) ||
         !XDRAtomMap(xdr, &script->atomMap)) {
         goto error;
     }
 
-    if (magic < JSXDR_MAGIC_SCRIPT_4) {
-        if (!JS_XDRUint32(xdr, &nsrcnotes))
-            goto error;
+    if (magic < JSXDR_MAGIC_SCRIPT_5) {
         if (xdr->mode == JSXDR_DECODE) {
-            notes = (jssrcnote *) JS_malloc(cx, nsrcnotes * sizeof(jssrcnote));
-            if (!notes)
+            /*
+             * Append JSOP_STOP to old scripts, to relieve the interpreter
+             * from having to bounds-check pc.  Also take care to increment
+             * length, as it is used below and must count all bytecode.
+             */
+            script->code[length++] = JSOP_STOP;
+        }
+
+        if (magic < JSXDR_MAGIC_SCRIPT_4) {
+            if (!JS_XDRUint32(xdr, &nsrcnotes))
                 goto error;
+            if (xdr->mode == JSXDR_DECODE) {
+                notes = (jssrcnote *)
+                        JS_malloc(cx, nsrcnotes * sizeof(jssrcnote));
+                if (!notes)
+                    goto error;
+            }
         }
     }
 
@@ -599,6 +636,8 @@ js_XDRScript(JSXDRState *xdr, JSScript **scriptp, JSBool *hasMagic)
         tn->length = (ptrdiff_t) catchLength;
         tn->catchStart = (ptrdiff_t) catchStart;
     }
+
+    xdr->script = oldscript;
     return JS_TRUE;
 
   error:
@@ -704,6 +743,7 @@ script_thaw(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     str = js_ValueToString(cx, argv[0]);
     if (!str)
         return JS_FALSE;
+    argv[0] = STRING_TO_JSVAL(str);
 
     /* create new XDR */
     xdr = JS_XDRNewMem(cx, JSXDR_DECODE);
@@ -816,13 +856,19 @@ script_mark(JSContext *cx, JSObject *obj, void *arg)
 
     script = (JSScript *) JS_GetPrivate(cx, obj);
     if (script)
-        js_MarkScript(cx, script, arg);
+        js_MarkScript(cx, script);
     return 0;
 }
 
+#if !JS_HAS_SCRIPT_OBJECT
+const char js_Script_str[] = "Script";
+
+#define JSProto_Script  JSProto_Object
+#endif
+
 JS_FRIEND_DATA(JSClass) js_ScriptClass = {
     js_Script_str,
-    JSCLASS_HAS_PRIVATE,
+    JSCLASS_HAS_PRIVATE | JSCLASS_HAS_CACHED_PROTO(JSProto_Script),
     JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,
     JS_EnumerateStub, JS_ResolveStub,   JS_ConvertStub,   script_finalize,
     NULL,             NULL,             script_call,      NULL,/*XXXbe xdr*/
@@ -839,6 +885,12 @@ Script(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
         obj = js_NewObject(cx, &js_ScriptClass, NULL, NULL);
         if (!obj)
             return JS_FALSE;
+
+        /*
+         * script_compile does not use rval to root its temporaries
+         * so we can use it to root obj.
+         */
+        *rval = OBJECT_TO_JSVAL(obj);
     }
     return script_compile(cx, obj, argc, argv, rval);
 }
@@ -899,7 +951,8 @@ typedef struct ScriptFilenameEntry {
     JSHashEntry         *next;          /* hash chain linkage */
     JSHashNumber        keyHash;        /* key hash function result */
     const void          *key;           /* ptr to filename, below */
-    JSPackedBool        mark;           /* mark flag, for GC */
+    uint32              flags;          /* user-defined filename prefix flags */
+    JSPackedBool        mark;           /* GC mark flag */
     char                filename[3];    /* two or more bytes, NUL-terminated */
 } ScriptFilenameEntry;
 
@@ -925,10 +978,8 @@ static JSHashAllocOps sftbl_alloc_ops = {
 };
 
 JSBool
-js_InitRuntimeScriptState(JSContext *cx)
+js_InitRuntimeScriptState(JSRuntime *rt)
 {
-    JSRuntime *rt = cx->runtime;
-
 #ifdef JS_THREADSAFE
     JS_ASSERT(!rt->scriptFilenameTableLock);
     rt->scriptFilenameTableLock = JS_NEW_LOCK();
@@ -940,17 +991,23 @@ js_InitRuntimeScriptState(JSContext *cx)
         JS_NewHashTable(16, JS_HashString, js_compare_strings, NULL,
                         &sftbl_alloc_ops, NULL);
     if (!rt->scriptFilenameTable) {
-        js_FinishRuntimeScriptState(cx);    /* free lock if threadsafe */
+        js_FinishRuntimeScriptState(rt);    /* free lock if threadsafe */
         return JS_FALSE;
     }
+    JS_INIT_CLIST(&rt->scriptFilenamePrefixes);
     return JS_TRUE;
 }
 
-void
-js_FinishRuntimeScriptState(JSContext *cx)
-{
-    JSRuntime *rt = cx->runtime;
+typedef struct ScriptFilenamePrefix {
+    JSCList     links;      /* circular list linkage for easy deletion */
+    const char  *name;      /* pointer to pinned ScriptFilenameEntry string */
+    size_t      length;     /* prefix string length, precomputed */
+    uint32      flags;      /* user-defined flags to inherit from this prefix */
+} ScriptFilenamePrefix;
 
+void
+js_FinishRuntimeScriptState(JSRuntime *rt)
+{
     if (rt->scriptFilenameTable) {
         JS_HashTableDestroy(rt->scriptFilenameTable);
         rt->scriptFilenameTable = NULL;
@@ -963,62 +1020,219 @@ js_FinishRuntimeScriptState(JSContext *cx)
 #endif
 }
 
+void
+js_FreeRuntimeScriptState(JSRuntime *rt)
+{
+    ScriptFilenamePrefix *sfp;
+
+    if (!rt->scriptFilenameTable)
+        return;
+
+    while (!JS_CLIST_IS_EMPTY(&rt->scriptFilenamePrefixes)) {
+        sfp = (ScriptFilenamePrefix *) rt->scriptFilenamePrefixes.next;
+        JS_REMOVE_LINK(&sfp->links);
+        free(sfp);
+    }
+    js_FinishRuntimeScriptState(rt);
+}
+
 #ifdef DEBUG_brendan
+#define DEBUG_SFTBL
+#endif
+#ifdef DEBUG_SFTBL
 size_t sftbl_savings = 0;
 #endif
 
-const char *
-js_SaveScriptFilename(JSContext *cx, const char *filename)
+static ScriptFilenameEntry *
+SaveScriptFilename(JSRuntime *rt, const char *filename, uint32 flags)
 {
-    JSRuntime *rt = cx->runtime;
     JSHashTable *table;
     JSHashNumber hash;
     JSHashEntry **hep;
     ScriptFilenameEntry *sfe;
+    size_t length;
+    JSCList *head, *link;
+    ScriptFilenamePrefix *sfp;
 
-    JS_ACQUIRE_LOCK(rt->scriptFilenameTableLock);
     table = rt->scriptFilenameTable;
     hash = JS_HashString(filename);
     hep = JS_HashTableRawLookup(table, hash, filename);
     sfe = (ScriptFilenameEntry *) *hep;
-#ifdef DEBUG_brendan
+#ifdef DEBUG_SFTBL
     if (sfe)
         sftbl_savings += strlen(sfe->filename);
 #endif
+
     if (!sfe) {
         sfe = (ScriptFilenameEntry *)
               JS_HashTableRawAdd(table, hep, hash, filename, NULL);
-        if (sfe) {
-            sfe->key = strcpy(sfe->filename, filename);
-            JS_ASSERT(!sfe->mark);
-        }
+        if (!sfe)
+            return NULL;
+        sfe->key = strcpy(sfe->filename, filename);
+        sfe->flags = 0;
+        sfe->mark = JS_FALSE;
     }
-    JS_RELEASE_LOCK(rt->scriptFilenameTableLock);
+
+    /* If saving a prefix, add it to the set in rt->scriptFilenamePrefixes. */
+    if (flags != 0) {
+        /* Search in case filename was saved already; we must be idempotent. */
+        sfp = NULL;
+        length = strlen(filename);
+        for (head = link = &rt->scriptFilenamePrefixes;
+             link->next != head;
+             link = link->next) {
+            /* Lag link behind sfp to insert in non-increasing length order. */
+            sfp = (ScriptFilenamePrefix *) link->next;
+            if (!strcmp(sfp->name, filename))
+                break;
+            if (sfp->length <= length) {
+                sfp = NULL;
+                break;
+            }
+            sfp = NULL;
+        }
+
+        if (!sfp) {
+            /* No such prefix: add one now. */
+            sfp = (ScriptFilenamePrefix *) malloc(sizeof(ScriptFilenamePrefix));
+            if (!sfp)
+                return NULL;
+            JS_INSERT_AFTER(&sfp->links, link);
+            sfp->name = sfe->filename;
+            sfp->length = length;
+            sfp->flags = 0;
+        }
+
+        /*
+         * Accumulate flags in both sfe and sfp: sfe for later access from the
+         * JS_GetScriptedCallerFilenameFlags debug-API, and sfp so that longer
+         * filename entries can inherit by prefix.
+         */
+        sfe->flags |= flags;
+        sfp->flags |= flags;
+    }
+
+    return sfe;
+}
+
+const char *
+js_SaveScriptFilename(JSContext *cx, const char *filename)
+{
+    JSRuntime *rt;
+    ScriptFilenameEntry *sfe;
+    JSCList *head, *link;
+    ScriptFilenamePrefix *sfp;
+
+    rt = cx->runtime;
+    JS_ACQUIRE_LOCK(rt->scriptFilenameTableLock);
+    sfe = SaveScriptFilename(rt, filename, 0);
     if (!sfe) {
+        JS_RELEASE_LOCK(rt->scriptFilenameTableLock);
         JS_ReportOutOfMemory(cx);
         return NULL;
     }
+
+    /*
+     * Try to inherit flags by prefix.  We assume there won't be more than a
+     * few (dozen! ;-) prefixes, so linear search is tolerable.
+     * XXXbe every time I've assumed that in the JS engine, I've been wrong!
+     */
+    for (head = &rt->scriptFilenamePrefixes, link = head->next;
+         link != head;
+         link = link->next) {
+        sfp = (ScriptFilenamePrefix *) link;
+        if (!strncmp(sfp->name, filename, sfp->length)) {
+            sfe->flags |= sfp->flags;
+            break;
+        }
+    }
+    JS_RELEASE_LOCK(rt->scriptFilenameTableLock);
     return sfe->filename;
 }
 
+const char *
+js_SaveScriptFilenameRT(JSRuntime *rt, const char *filename, uint32 flags)
+{
+    ScriptFilenameEntry *sfe;
+
+    /* This may be called very early, via the jsdbgapi.h entry point. */
+    if (!rt->scriptFilenameTable && !js_InitRuntimeScriptState(rt))
+        return NULL;
+
+    JS_ACQUIRE_LOCK(rt->scriptFilenameTableLock);
+    sfe = SaveScriptFilename(rt, filename, flags);
+    JS_RELEASE_LOCK(rt->scriptFilenameTableLock);
+    if (!sfe)
+        return NULL;
+
+    return sfe->filename;
+}
+
+/*
+ * Back up from a saved filename by its offset within its hash table entry.
+ */
+#define FILENAME_TO_SFE(fn) \
+    ((ScriptFilenameEntry *) ((fn) - offsetof(ScriptFilenameEntry, filename)))
+
+/*
+ * The sfe->key member, redundant given sfe->filename but required by the old
+ * jshash.c code, here gives us a useful sanity check.  This assertion will
+ * very likely botch if someone tries to mark a string that wasn't allocated
+ * as an sfe->filename.
+ */
+#define ASSERT_VALID_SFE(sfe)   JS_ASSERT((sfe)->key == (sfe)->filename)
+
+uint32
+js_GetScriptFilenameFlags(const char *filename)
+{
+    ScriptFilenameEntry *sfe;
+
+    sfe = FILENAME_TO_SFE(filename);
+    ASSERT_VALID_SFE(sfe);
+    return sfe->flags;
+}
+
 void
 js_MarkScriptFilename(const char *filename)
 {
     ScriptFilenameEntry *sfe;
 
-    /*
-     * Back up from filename by its offset within its hash table entry.
-     * The sfe->key member, redundant given sfe->filename but required by
-     * the old jshash.c code, here gives us a useful sanity check.  This
-     * assertion will very likely botch if someone tries to mark a string
-     * that wasn't allocated as an sfe->filename.
-     */
-    sfe = (ScriptFilenameEntry *)
-          (filename - offsetof(ScriptFilenameEntry, filename));
-    JS_ASSERT(sfe->key == sfe->filename);
+    sfe = FILENAME_TO_SFE(filename);
+    ASSERT_VALID_SFE(sfe);
     sfe->mark = JS_TRUE;
 }
 
+JS_STATIC_DLL_CALLBACK(intN)
+js_script_filename_marker(JSHashEntry *he, intN i, void *arg)
+{
+    ScriptFilenameEntry *sfe = (ScriptFilenameEntry *) he;
+
+    sfe->mark = JS_TRUE;
+    return HT_ENUMERATE_NEXT;
+}
+
+void
+js_MarkScriptFilenames(JSRuntime *rt, JSBool keepAtoms)
+{
+    JSCList *head, *link;
+    ScriptFilenamePrefix *sfp;
+
+    if (!rt->scriptFilenameTable)
+        return;
+
+    if (keepAtoms) {
+        JS_HashTableEnumerateEntries(rt->scriptFilenameTable,
+                                     js_script_filename_marker,
+                                     rt);
+    }
+    for (head = &rt->scriptFilenamePrefixes, link = head->next;
+         link != head;
+         link = link->next) {
+        sfp = (ScriptFilenamePrefix *) link;
+        js_MarkScriptFilename(sfp->name);
+    }
+}
+
 JS_STATIC_DLL_CALLBACK(intN)
 js_script_filename_sweeper(JSHashEntry *he, intN i, void *arg)
 {
@@ -1033,12 +1247,17 @@ js_script_filename_sweeper(JSHashEntry *he, intN i, void *arg)
 void
 js_SweepScriptFilenames(JSRuntime *rt)
 {
+    if (!rt->scriptFilenameTable)
+        return;
+
     JS_HashTableEnumerateEntries(rt->scriptFilenameTable,
                                  js_script_filename_sweeper,
                                  rt);
-#ifdef DEBUG_brendan
+#ifdef DEBUG_notme
+#ifdef DEBUG_SFTBL
     printf("script filename table savings so far: %u\n", sftbl_savings);
 #endif
+#endif
 }
 
 JSScript *
@@ -1135,8 +1354,8 @@ js_CallNewScriptHook(JSContext *cx, JSScript *script, JSFunction *fun)
     }
 }
 
-void
-js_DestroyScript(JSContext *cx, JSScript *script)
+JS_FRIEND_API(void)
+js_CallDestroyScriptHook(JSContext *cx, JSScript *script)
 {
     JSRuntime *rt;
     JSDestroyScriptHook hook;
@@ -1145,6 +1364,12 @@ js_DestroyScript(JSContext *cx, JSScript *script)
     hook = rt->destroyScriptHook;
     if (hook)
         hook(cx, script, rt->destroyScriptHookData);
+}
+
+void
+js_DestroyScript(JSContext *cx, JSScript *script)
+{
+    js_CallDestroyScriptHook(cx, script);
 
     JS_ClearScriptTraps(cx, script);
     js_FreeAtomMap(cx, &script->atomMap);
@@ -1154,7 +1379,7 @@ js_DestroyScript(JSContext *cx, JSScript *script)
 }
 
 void
-js_MarkScript(JSContext *cx, JSScript *script, void *arg)
+js_MarkScript(JSContext *cx, JSScript *script)
 {
     JSAtomMap *map;
     uintN i, length;
@@ -1164,7 +1389,7 @@ js_MarkScript(JSContext *cx, JSScript *script, void *arg)
     length = map->length;
     vector = map->vector;
     for (i = 0; i < length; i++)
-        GC_MARK_ATOM(cx, vector[i], arg);
+        GC_MARK_ATOM(cx, vector[i]);
 
     if (script->filename)
         js_MarkScriptFilename(script->filename);
@@ -1202,11 +1427,15 @@ js_PCToLineNumber(JSContext *cx, JSScript *script, jsbytecode *pc)
      * Special case: function definition needs no line number note because
      * the function's script contains its starting line number.
      */
-    if (*pc == JSOP_DEFFUN) {
-        atom = GET_ATOM(cx, script, pc);
+    if (*pc == JSOP_DEFFUN ||
+        (*pc == JSOP_LITOPX && pc[1 + LITERAL_INDEX_LEN] == JSOP_DEFFUN)) {
+        atom = js_GetAtom(cx, &script->atomMap,
+                          (*pc == JSOP_DEFFUN)
+                          ? GET_ATOM_INDEX(pc)
+                          : GET_LITERAL_INDEX(pc));
         fun = (JSFunction *) JS_GetPrivate(cx, ATOM_TO_OBJECT(atom));
-        JS_ASSERT(fun->interpreted);
-        return fun->u.script->lineno;
+        JS_ASSERT(FUN_INTERPRETED(fun));
+        return fun->u.i.script->lineno;
     }
 
     /*
@@ -1233,19 +1462,31 @@ js_PCToLineNumber(JSContext *cx, JSScript *script, jsbytecode *pc)
     return lineno;
 }
 
+/* The line number limit is the same as the jssrcnote offset limit. */
+#define SN_LINE_LIMIT   (SN_3BYTE_OFFSET_FLAG << 16)
+
 jsbytecode *
 js_LineNumberToPC(JSScript *script, uintN target)
 {
-    ptrdiff_t offset;
-    uintN lineno;
+    ptrdiff_t offset, best;
+    uintN lineno, bestdiff, diff;
     jssrcnote *sn;
     JSSrcNoteType type;
 
     offset = 0;
+    best = -1;
     lineno = script->lineno;
+    bestdiff = SN_LINE_LIMIT;
     for (sn = SCRIPT_NOTES(script); !SN_IS_TERMINATOR(sn); sn = SN_NEXT(sn)) {
-        if (lineno >= target)
-            break;
+        if (lineno == target)
+            goto out;
+        if (lineno > target) {
+            diff = lineno - target;
+            if (diff < bestdiff) {
+                bestdiff = diff;
+                best = offset;
+            }
+        }
         offset += SN_DELTA(sn);
         type = (JSSrcNoteType) SN_TYPE(sn);
         if (type == SRC_SETLINE) {
@@ -1254,10 +1495,13 @@ js_LineNumberToPC(JSScript *script, uintN target)
             lineno++;
         }
     }
+    if (best >= 0)
+        offset = best;
+out:
     return script->code + offset;
 }
 
-uintN
+JS_FRIEND_API(uintN)
 js_GetScriptLineExtent(JSScript *script)
 {
     uintN lineno;
@@ -1275,3 +1519,43 @@ js_GetScriptLineExtent(JSScript *script)
     }
     return 1 + lineno - script->lineno;
 }
+
+#if JS_HAS_GENERATORS
+
+jsbytecode *
+js_FindFinallyHandler(JSScript *script, jsbytecode *pc)
+{
+    JSTryNote *tn;
+    ptrdiff_t off;
+    JSOp op2;
+
+    tn = script->trynotes;
+    if (!tn)
+        return NULL;
+
+    off = pc - script->main;
+    if (off < 0)
+        return NULL;
+
+    JS_ASSERT(tn->catchStart != 0);
+    do {
+        if ((jsuword)(off - tn->start) < (jsuword)tn->length) {
+            /*
+             * We have a handler: is it the finally one, or a catch handler?
+             *
+             * Catch bytecode begins with:   JSOP_SETSP JSOP_ENTERBLOCK
+             * Finally bytecode begins with: JSOP_SETSP JSOP_(GOSUB|EXCEPTION)
+             */
+            pc = script->main + tn->catchStart;
+            JS_ASSERT(*pc == JSOP_SETSP);
+            op2 = pc[JSOP_SETSP_LENGTH];
+            if (op2 != JSOP_ENTERBLOCK) {
+                JS_ASSERT(op2 == JSOP_GOSUB || op2 == JSOP_EXCEPTION);
+                return pc;
+            }
+        }
+    } while ((++tn)->catchStart != 0);
+    return NULL;
+}
+
+#endif
