@@ -43,12 +43,12 @@
 /* High level class and public functions implementation. */
 
 #include "xpcprivate.h"
+#include "XPCNativeWrapper.h"
 
 NS_IMPL_THREADSAFE_ISUPPORTS2(nsXPConnect,nsIXPConnect,nsISupportsWeakReference)
 
 nsXPConnect* nsXPConnect::gSelf = nsnull;
 JSBool       nsXPConnect::gOnceAliveNowDead = JS_FALSE;
-PRThread*    nsXPConnect::gMainThread = nsnull;
 
 const char XPC_CONTEXT_STACK_CONTRACTID[] = "@mozilla.org/js/xpc/ContextStack;1";
 const char XPC_RUNTIME_CONTRACTID[]       = "@mozilla.org/js/xpc/RuntimeService;1";
@@ -62,7 +62,7 @@ const char XPC_XPCONNECT_CONTRACTID[]     = "@mozilla.org/js/xpc/XPConnect;1";
 
 nsXPConnect::nsXPConnect()
     :   mRuntime(nsnull),
-        mInterfaceInfoManager(nsnull),
+        mInterfaceInfoManager(do_GetService(NS_INTERFACEINFOMANAGER_SERVICE_CONTRACTID)),
         mContextStack(nsnull),
         mDefaultSecurityManager(nsnull),
         mDefaultSecurityManagerFlags(0),
@@ -72,10 +72,6 @@ nsXPConnect::nsXPConnect()
     // then we'll set this up later as needed.
     CreateRuntime();
 
-    nsCOMPtr<nsIInterfaceInfoManager> iim = 
-        dont_AddRef(XPTI_GetInterfaceInfoManager());
-    CallQueryInterface(iim, &mInterfaceInfoManager);
-
     CallGetService(XPC_CONTEXT_STACK_CONTRACTID, &mContextStack);
 
 #ifdef XPC_TOOLS_SUPPORT
@@ -124,7 +120,6 @@ nsXPConnect::~nsXPConnect()
         }
     }
 
-    NS_IF_RELEASE(mInterfaceInfoManager);
     NS_IF_RELEASE(mContextStack);
     NS_IF_RELEASE(mDefaultSecurityManager);
 
@@ -330,19 +325,6 @@ nsXPConnect::CreateRuntime()
     return nsnull != mRuntime;
 }
 
-// static 
-PRThread* 
-nsXPConnect::FindMainThread()
-{
-    nsCOMPtr<nsIThread> t;
-    nsresult rv;
-    rv = nsIThread::GetMainThread(getter_AddRefs(t));
-    NS_ASSERTION(NS_SUCCEEDED(rv) && t, "bad");
-    rv = t->GetPRThread(&gMainThread);
-    NS_ASSERTION(NS_SUCCEEDED(rv) && gMainThread, "bad");
-    return gMainThread;
-}
-
 /***************************************************************************/
 
 typedef PRBool (*InfoTester)(nsIInterfaceInfoManager* manager, const void* data,
@@ -446,6 +428,10 @@ nsXPConnect::InitClasses(JSContext * aJSContext, JSObject * aGlobalJSObj)
     // Initialize any properties IDispatch needs on the global object
     XPCIDispatchExtension::Initialize(ccx, aGlobalJSObj);
 #endif
+
+    if (!XPCNativeWrapper::AttachNewConstructorObject(ccx, aGlobalJSObj))
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
     return NS_OK;
 }
 
@@ -463,12 +449,12 @@ static JSClass xpcTempGlobalClass = {
     JSCLASS_NO_OPTIONAL_MEMBERS
 };
 
-/* nsIXPConnectJSObjectHolder initClassesWithNewWrappedGlobal (in JSContextPtr aJSContext, in nsISupports aCOMObj, in nsIIDRef aIID, in PRBool aCallJS_InitStandardClasses); */
+/* nsIXPConnectJSObjectHolder initClassesWithNewWrappedGlobal (in JSContextPtr aJSContext, in nsISupports aCOMObj, in nsIIDRef aIID, in PRUint32 aFlags); */
 NS_IMETHODIMP
 nsXPConnect::InitClassesWithNewWrappedGlobal(JSContext * aJSContext,
                                              nsISupports *aCOMObj,
                                              const nsIID & aIID,
-                                             PRBool aCallJS_InitStandardClasses,
+                                             PRUint32 aFlags,
                                              nsIXPConnectJSObjectHolder **_retval)
 {
     NS_ASSERTION(aJSContext, "bad param");
@@ -490,18 +476,34 @@ nsXPConnect::InitClassesWithNewWrappedGlobal(JSContext * aJSContext,
        !JS_SetPrototype(aJSContext, tempGlobal, nsnull))
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
-    if(NS_FAILED(InitClasses(aJSContext, tempGlobal)))
-        return UnexpectedFailure(NS_ERROR_FAILURE);
+    if(aFlags & nsIXPConnect::FLAG_SYSTEM_GLOBAL_OBJECT)
+        JS_FlagSystemObject(aJSContext, tempGlobal);
 
     nsCOMPtr<nsIXPConnectJSObjectHolder> holder;
-    if(NS_FAILED(WrapNative(aJSContext, tempGlobal, aCOMObj, aIID,
-                            getter_AddRefs(holder))) || !holder)
-        return UnexpectedFailure(NS_ERROR_FAILURE);
+    {
+        // Scope for our auto-marker; it just needs to keep tempGlobal alive
+        // long enough for InitClasses and WrapNative to do their work
+        AUTO_MARK_JSVAL(ccx, OBJECT_TO_JSVAL(tempGlobal));
+
+        if(NS_FAILED(InitClasses(aJSContext, tempGlobal)))
+            return UnexpectedFailure(NS_ERROR_FAILURE);
+
+        nsresult rv;
+        if(!XPCConvert::NativeInterface2JSObject(ccx, getter_AddRefs(holder),
+                                                 aCOMObj, &aIID, tempGlobal,
+                                                 PR_FALSE, OBJ_IS_GLOBAL, &rv))
+            return UnexpectedFailure(rv);
+
+        NS_ASSERTION(NS_SUCCEEDED(rv) && holder, "Didn't wrap properly");
+    }
 
     JSObject* globalJSObj;
     if(NS_FAILED(holder->GetJSObject(&globalJSObj)) || !globalJSObj)
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
+    if(aFlags & nsIXPConnect::FLAG_SYSTEM_GLOBAL_OBJECT)
+        NS_ASSERTION(JS_IsSystemObject(aJSContext, globalJSObj), "huh?!");
+
     // voodoo to fixup scoping and parenting...
 
     JS_SetParent(aJSContext, globalJSObj, nsnull);
@@ -510,7 +512,7 @@ nsXPConnect::InitClassesWithNewWrappedGlobal(JSContext * aJSContext,
     if(!oldGlobal || oldGlobal == tempGlobal)
         JS_SetGlobalObject(aJSContext, globalJSObj);
 
-    if(aCallJS_InitStandardClasses &&
+    if((aFlags & nsIXPConnect::INIT_JS_STANDARD_CLASSES) &&
        !JS_InitStandardClasses(aJSContext, globalJSObj))
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
@@ -538,6 +540,9 @@ nsXPConnect::InitClassesWithNewWrappedGlobal(JSContext * aJSContext,
     if(!nsXPCComponents::AttachNewComponentsObject(ccx, scope, globalJSObj))
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
+    if (!XPCNativeWrapper::AttachNewConstructorObject(ccx, globalJSObj))
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
     NS_ADDREF(*_retval = holder);
 
     return NS_OK;
@@ -563,9 +568,18 @@ nsXPConnect::WrapNative(JSContext * aJSContext,
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
     nsresult rv;
-    if(!XPCConvert::NativeInterface2JSObject(ccx, _retval,
-                                             aCOMObj, &aIID, aScope, &rv))
+    if(!XPCConvert::NativeInterface2JSObject(ccx, _retval, aCOMObj, &aIID,
+                                             aScope, PR_FALSE,
+                                             OBJ_IS_NOT_GLOBAL, &rv))
         return rv;
+
+#ifdef DEBUG
+    JSObject* returnObj;
+    (*_retval)->GetJSObject(&returnObj);
+    NS_ASSERTION(!XPCNativeWrapper::IsNativeWrapper(aJSContext, returnObj),
+                 "Shouldn't be returning a native wrapper here");
+#endif
+    
     return NS_OK;
 }
 
@@ -712,6 +726,123 @@ nsXPConnect::ReparentWrappedNativeIfFound(JSContext * aJSContext,
                                (XPCWrappedNative**) _retval);
 }
 
+JS_STATIC_DLL_CALLBACK(JSDHashOperator)
+MoveableWrapperFinder(JSDHashTable *table, JSDHashEntryHdr *hdr,
+                      uint32 number, void *arg)
+{
+    // Every element counts.
+    nsVoidArray *va = NS_STATIC_CAST(nsVoidArray *,arg);
+    va->AppendElement(((Native2WrappedNativeMap::Entry*)hdr)->value);
+    return JS_DHASH_NEXT;
+}
+
+/* void reparentScopeAwareWrappers(in JSContextPtr aJSContext, in JSObjectPtr  aOldScope, in JSObjectPtr  aNewScope); */
+NS_IMETHODIMP
+nsXPConnect::ReparentScopeAwareWrappers(JSContext *aJSContext,
+                                        JSObject *aOldScope,
+                                        JSObject *aNewScope)
+{
+    XPCCallContext ccx(NATIVE_CALLER, aJSContext);
+    if(!ccx.IsValid())
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    XPCWrappedNativeScope *oldScope =
+        XPCWrappedNativeScope::FindInJSObjectScope(ccx, aOldScope);
+    if(!oldScope)
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    XPCWrappedNativeScope *newScope =
+        XPCWrappedNativeScope::FindInJSObjectScope(ccx, aNewScope);
+    if(!newScope)
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    {   // scoped lock
+        XPCAutoLock lock(oldScope->GetRuntime()->GetMapLock());
+
+        // First, look through the old scope and find all of the wrappers that
+        // we're going to move.
+
+        Native2WrappedNativeMap *map = oldScope->GetWrappedNativeMap();
+        nsVoidArray wrappersToMove(map->Count());
+        map->Enumerate(MoveableWrapperFinder, &wrappersToMove);
+
+        // Now that we have the wrappers, reparent them to the new scope.
+        for(PRInt32 i = 0, stop = wrappersToMove.Count(); i < stop; ++i)
+        {
+            // First, check to see if this wrapper really needs to be
+            // reparented.
+
+            XPCWrappedNative *wrapper =
+                NS_STATIC_CAST(XPCWrappedNative *, wrappersToMove[i]);
+            nsISupports *identity = wrapper->GetIdentityObject();
+            nsCOMPtr<nsIClassInfo> info(do_QueryInterface(identity));
+
+            // ClassInfo is implemented as singleton objects. If the identity
+            // object here is the same object as returned by the QI, then it
+            // is the singleton classinfo, so we don't need to reparent it.
+            if(SameCOMIdentity(identity, info))
+                info = nsnull;
+
+            if(!info)
+                continue;
+
+            XPCNativeScriptableCreateInfo sciProto;
+            XPCNativeScriptableCreateInfo sciWrapper;
+
+            nsresult rv =
+                XPCWrappedNative::GatherScriptableCreateInfo(identity,
+                                                             info.get(),
+                                                             &sciProto,
+                                                             &sciWrapper);
+            if(NS_FAILED(rv))
+                return NS_ERROR_FAILURE;
+
+            // If the wrapper doesn't want precreate, then we don't need to
+            // worry about reparenting it.
+            if(!sciWrapper.GetFlags().WantPreCreate())
+                continue;
+
+            JSObject *newParent = aOldScope;
+            rv = sciWrapper.GetCallback()->PreCreate(identity, ccx, aOldScope,
+                                                     &newParent);
+            if(NS_FAILED(rv))
+                return rv;
+
+            if(newParent != aOldScope)
+            {
+                // The wrapper returned a new parent. If the new parent is in
+                // a different scope, then we need to reparent it, otherwise,
+                // the old scope is fine.
+
+                XPCWrappedNativeScope *betterScope =
+                    XPCWrappedNativeScope::FindInJSObjectScope(ccx, newParent);
+                if(betterScope == oldScope)
+                    continue;
+
+                NS_ASSERTION(betterScope == newScope, "Weird scope returned");
+            }
+            else
+            {
+                // The old scope still works for this wrapper.
+                continue;
+            }
+
+            // Now, reparent the wrapper, since we know that it wants to be
+            // reparented.
+
+            XPCWrappedNative *junk;
+            rv = XPCWrappedNative::ReparentWrapperIfFound(ccx, oldScope,
+                                                          newScope, aNewScope,
+                                                          wrapper->GetIdentityObject(),
+                                                          &junk);
+            NS_ENSURE_SUCCESS(rv, rv);
+            NS_IF_RELEASE(junk);
+        }
+    }
+
+    return NS_OK;
+}
+
 /* void setSecurityManagerForJSContext (in JSContextPtr aJSContext, in nsIXPCSecurityManager aManager, in PRUint16 flags); */
 NS_IMETHODIMP
 nsXPConnect::SetSecurityManagerForJSContext(JSContext * aJSContext,
@@ -959,6 +1090,149 @@ nsXPConnect::ClearAllWrappedNativeSecurityPolicies()
     return XPCWrappedNativeScope::ClearAllWrappedNativeSecurityPolicies(ccx);
 }
 
+/* void restoreWrappedNativePrototype (in JSContextPtr aJSContext, in JSObjectPtr aScope, in nsIClassInfo aClassInfo, in nsIXPConnectJSObjectHolder aPrototype); */
+NS_IMETHODIMP 
+nsXPConnect::RestoreWrappedNativePrototype(JSContext * aJSContext, 
+                                           JSObject * aScope, 
+                                           nsIClassInfo * aClassInfo, 
+                                           nsIXPConnectJSObjectHolder * aPrototype)
+{
+    XPCCallContext ccx(NATIVE_CALLER, aJSContext);
+    if(!ccx.IsValid())
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    if(!aClassInfo || !aPrototype)
+        return UnexpectedFailure(NS_ERROR_INVALID_ARG);
+
+    JSObject *protoJSObject;
+    nsresult rv = aPrototype->GetJSObject(&protoJSObject);
+    if(NS_FAILED(rv))
+        return UnexpectedFailure(rv);
+
+    if(!IS_PROTO_CLASS(JS_GET_CLASS(ccx, protoJSObject)))
+        return UnexpectedFailure(NS_ERROR_INVALID_ARG);
+
+    XPCWrappedNativeScope* scope =
+        XPCWrappedNativeScope::FindInJSObjectScope(ccx, aScope);
+    if(!scope)
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    XPCWrappedNativeProto *proto =
+        (XPCWrappedNativeProto*)JS_GetPrivate(ccx, protoJSObject);
+    if(!proto)
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    if(scope != proto->GetScope())
+    {
+        NS_ERROR("Attempt to reset prototype to a prototype from a"
+                 "different scope!");
+
+        return UnexpectedFailure(NS_ERROR_INVALID_ARG);
+    }
+
+    XPCNativeScriptableInfo *si = proto->GetScriptableInfo();
+
+    if(si && si->GetFlags().DontSharePrototype())
+        return UnexpectedFailure(NS_ERROR_INVALID_ARG);
+
+    ClassInfo2WrappedNativeProtoMap* map = scope->GetWrappedNativeProtoMap();
+    XPCLock* lock = scope->GetRuntime()->GetMapLock();
+
+    {   // scoped lock
+        XPCAutoLock al(lock);
+
+        XPCWrappedNativeProtoMap* detachedMap =
+            GetRuntime()->GetDetachedWrappedNativeProtoMap();
+
+        // If we're replacing an old proto, make sure to put it on the
+        // map of detached wrapped native protos so that the old proto
+        // gets properly cleaned up, especially during shutdown.
+        XPCWrappedNativeProto *oldProto = map->Find(aClassInfo);
+        if(oldProto)
+        {
+            detachedMap->Add(oldProto);
+
+            // ClassInfo2WrappedNativeProtoMap doesn't ever replace
+            // entries in the map, so now since we know there's an
+            // entry for aClassInfo in the map we have to remove it to
+            // be able to add the new one.
+            map->Remove(aClassInfo);
+
+            // This code should do the right thing even if we're
+            // restoring the current proto, but warn in that case
+            // since doing that is pointless.
+            NS_ASSERTION(proto != oldProto,
+                         "Restoring current prototype, fix caller!");
+        }
+
+        map->Add(aClassInfo, proto);
+
+        // Remove the prototype from the map of detached wrapped
+        // native prototypes now that the prototype is part of a scope
+        // again.
+        detachedMap->Remove(proto);
+    }
+
+    // The global in this scope didn't change, but a prototype did
+    // (most likely the global object's prototype), which means the
+    // scope needs to get a chance to update its cached
+    // Object.prototype pointers etc.
+    scope->SetGlobal(ccx, aScope);
+
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsXPConnect::CreateSandbox(JSContext *cx, nsIPrincipal *principal,
+                           nsIXPConnectJSObjectHolder **_retval)
+{
+#ifdef XPCONNECT_STANDALONE
+    return NS_ERROR_NOT_AVAILABLE;
+#else /* XPCONNECT_STANDALONE */
+    XPCCallContext ccx(NATIVE_CALLER, cx);
+    if(!ccx.IsValid())
+        return UnexpectedFailure(NS_ERROR_FAILURE);
+
+    *_retval = nsnull;
+
+    jsval rval = JSVAL_VOID;
+    AUTO_MARK_JSVAL(ccx, &rval);
+
+    nsresult rv = xpc_CreateSandboxObject(cx, &rval, principal);
+    NS_ASSERTION(NS_FAILED(rv) || !JSVAL_IS_PRIMITIVE(rval),
+                 "Bad return value from xpc_CreateSandboxObject()!");
+
+    if (NS_SUCCEEDED(rv) && !JSVAL_IS_PRIMITIVE(rval)) {
+        *_retval = XPCJSObjectHolder::newHolder(cx, JSVAL_TO_OBJECT(rval));
+        NS_ENSURE_TRUE(*_retval, NS_ERROR_OUT_OF_MEMORY);
+
+        NS_ADDREF(*_retval);
+    }
+
+    return rv;
+#endif /* XPCONNECT_STANDALONE */
+}
+
+NS_IMETHODIMP
+nsXPConnect::EvalInSandboxObject(const nsAString& source, JSContext *cx,
+                                 nsIXPConnectJSObjectHolder *sandbox,
+                                 jsval *rval)
+{
+#ifdef XPCONNECT_STANDALONE
+    return NS_ERROR_NOT_AVAILABLE;
+#else /* XPCONNECT_STANDALONE */
+    if (!sandbox)
+        return NS_ERROR_INVALID_ARG;
+
+    JSObject *obj;
+    nsresult rv = sandbox->GetJSObject(&obj);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    return xpc_EvalInSandbox(cx, obj, source,
+                             NS_ConvertUTF16toUTF8(source).get(), 1, rval);
+#endif /* XPCONNECT_STANDALONE */
+}
+
 /* nsIXPConnectJSObjectHolder getWrappedNativePrototype (in JSContextPtr aJSContext, in JSObjectPtr aScope, in nsIClassInfo aClassInfo); */
 NS_IMETHODIMP 
 nsXPConnect::GetWrappedNativePrototype(JSContext * aJSContext, 
@@ -980,7 +1254,8 @@ nsXPConnect::GetWrappedNativePrototype(JSContext * aJSContext,
 
     AutoMarkingWrappedNativeProtoPtr proto(ccx);
     proto = XPCWrappedNativeProto::GetNewOrUsed(ccx, scope, aClassInfo, 
-                                                &sciProto, JS_FALSE);
+                                                &sciProto, JS_FALSE,
+                                                OBJ_IS_NOT_GLOBAL);
     if(!proto)
         return UnexpectedFailure(NS_ERROR_FAILURE);
 
@@ -1098,7 +1373,7 @@ nsXPConnect::DebugDump(PRInt16 depth)
         XPC_LOG_ALWAYS(("gOnceAliveNowDead is %d", (int)gOnceAliveNowDead));
         XPC_LOG_ALWAYS(("mDefaultSecurityManager @ %x", mDefaultSecurityManager));
         XPC_LOG_ALWAYS(("mDefaultSecurityManagerFlags of %x", mDefaultSecurityManagerFlags));
-        XPC_LOG_ALWAYS(("mInterfaceInfoManager @ %x", mInterfaceInfoManager));
+        XPC_LOG_ALWAYS(("mInterfaceInfoManager @ %x", mInterfaceInfoManager.get()));
         XPC_LOG_ALWAYS(("mContextStack @ %x", mContextStack));
         if(mRuntime)
         {
@@ -1255,6 +1530,26 @@ nsXPConnect::JSToVariant(JSContext* ctx, jsval value, nsIVariant** _retval)
     return NS_OK;
 }
 
+/* void flagSystemFilenamePrefix (in string filenamePrefix); */
+NS_IMETHODIMP 
+nsXPConnect::FlagSystemFilenamePrefix(const char *aFilenamePrefix)
+{
+    NS_PRECONDITION(aFilenamePrefix, "bad param");
+
+    nsIJSRuntimeService* rtsvc = nsXPConnect::GetJSRuntimeService();
+    if(!rtsvc)
+        return NS_ERROR_NOT_INITIALIZED;
+
+    JSRuntime* rt;
+    nsresult rv = rtsvc->GetRuntime(&rt);
+    if(NS_FAILED(rv))
+        return rv;
+
+    if(!JS_FlagScriptFilenamePrefix(rt, aFilenamePrefix, JSFILENAME_SYSTEM))
+        return NS_ERROR_OUT_OF_MEMORY;
+    return NS_OK;
+}
+
 #ifdef DEBUG
 /* These are here to be callable from a debugger */
 JS_BEGIN_EXTERN_C
@@ -1282,5 +1577,41 @@ void DumpJSObject(JSObject* obj)
 {
     xpc_DumpJSObject(obj);
 }
+
+void DumpJSValue(jsval val)
+{
+    printf("Dumping 0x%lx. Value tag is %lu.\n", val, JSVAL_TAG(val));
+    if(JSVAL_IS_NULL(val)) {
+        printf("Value is null\n");
+    }
+    else if(JSVAL_IS_OBJECT(val)) {
+        printf("Value is an object\n");
+        JSObject* obj = JSVAL_TO_OBJECT(val);
+        DumpJSObject(obj);
+    }
+    else if(JSVAL_IS_NUMBER(val)) {
+        printf("Value is a number: ");
+        if(JSVAL_IS_INT(val))
+          printf("Integer %i\n", JSVAL_TO_INT(val));
+        else if(JSVAL_IS_DOUBLE(val))
+          printf("Floating-point value %f\n", *JSVAL_TO_DOUBLE(val));
+    }
+    else if(JSVAL_IS_STRING(val)) {
+        printf("Value is a string: ");
+        JSString* string = JSVAL_TO_STRING(val);
+        char* bytes = JS_GetStringBytes(string);
+        printf("<%s>\n", bytes);
+    }
+    else if(JSVAL_IS_BOOLEAN(val)) {
+        printf("Value is boolean: ");
+        printf(JSVAL_TO_BOOLEAN(val) ? "true" : "false");
+    }
+    else if(JSVAL_IS_VOID(val)) {
+        printf("Value is undefined\n");
+    }
+    else {
+        printf("No idea what this value is.\n");
+    }
+}
 JS_END_EXTERN_C
 #endif
