@@ -43,6 +43,9 @@
 /* Data conversion between native and JavaScript types. */
 
 #include "xpcprivate.h"
+#include "nsString.h"
+#include "XPCNativeWrapper.h"
+#include "nsIAtom.h"
 
 //#define STRICT_CHECK_OF_UNICODE
 #ifdef STRICT_CHECK_OF_UNICODE
@@ -398,8 +401,10 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
 
                     JSString* jsString = JS_NewUCString(cx, p, len);
 
-                    if(!jsString)
+                    if(!jsString) {
+                        nsMemory::Free(p); 
                         return JS_FALSE; 
+                    }
 
                     *d = STRING_TO_JSVAL(jsString);
                 }
@@ -458,9 +463,15 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
                     }
                     // else...
                     
+                    // XXX The OBJ_IS_NOT_GLOBAL here is not really right. In
+                    // fact, this code is depending on the fact that the
+                    // global object will not have been collected, and
+                    // therefore this NativeInterface2JSObject will not end up
+                    // creating a new XPCNativeScriptableShared.
                     nsCOMPtr<nsIXPConnectJSObjectHolder> holder;
                     if(!NativeInterface2JSObject(ccx, getter_AddRefs(holder),
-                                                 iface, iid, scope, pErr))
+                                                 iface, iid, scope, PR_TRUE,
+                                                 OBJ_IS_NOT_GLOBAL, pErr))
                         return JS_FALSE;
 
                     if(holder)
@@ -468,6 +479,11 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
                         JSObject* jsobj;
                         if(NS_FAILED(holder->GetJSObject(&jsobj)))
                             return JS_FALSE;
+#ifdef DEBUG
+                        if(!JS_GetParent(ccx, jsobj))
+                            NS_ASSERTION(JS_GET_CLASS(ccx, jsobj)->flags & JSCLASS_IS_GLOBAL,
+                                         "Why did we recreate this wrapper?");
+#endif
                         *d = OBJECT_TO_JSVAL(jsobj);
                     }
                 }
@@ -964,6 +980,25 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
                 *((nsISupports**)d) = NS_STATIC_CAST(nsIVariant*, variant);
                 return JS_TRUE;
             }
+            else if(iid->Equals(NS_GET_IID(nsIAtom)) &&
+                    JSVAL_IS_STRING(s))
+            {
+                // We're trying to pass a string as an nsIAtom.  Let's atomize!
+                JSString* str = JSVAL_TO_STRING(s);
+                PRUnichar* chars =
+                    NS_REINTERPRET_CAST(PRUnichar*, JS_GetStringChars(str));
+                if (!chars) {
+                    if (pErr)
+                        *pErr = NS_ERROR_XPC_BAD_CONVERT_JS_NULL_REF;
+                    return JS_FALSE;
+                }
+                PRUint32 length = JS_GetStringLength(str);
+                nsIAtom* atom = NS_NewAtom(nsDependentString(chars, length));
+                if (!atom && pErr)
+                    *pErr = NS_ERROR_OUT_OF_MEMORY;
+                *((nsISupports**)d) = atom;
+                return atom != nsnull;                
+            }
             //else ...
 
             if(JSVAL_IS_VOID(s) || JSVAL_IS_NULL(s))
@@ -1005,7 +1040,10 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
                                      nsIXPConnectJSObjectHolder** dest,
                                      nsISupports* src,
                                      const nsID* iid,
-                                     JSObject* scope, nsresult* pErr)
+                                     JSObject* scope,
+                                     PRBool allowNativeWrapper,
+                                     PRBool isGlobal,
+                                     nsresult* pErr)
 {
     NS_ASSERTION(dest, "bad param");
     NS_ASSERTION(iid, "bad param");
@@ -1031,6 +1069,8 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
     // is this a wrapped JS object?
     if(nsXPCWrappedJSClass::IsWrappedJS(src))
     {
+        NS_ASSERTION(!isGlobal, "The global object must be native");
+
         // verify that this wrapper is for the right interface
         nsCOMPtr<nsISupports> wrapper;
         if(NS_FAILED(src->QueryInterface(*iid,(void**)getter_AddRefs(wrapper))))
@@ -1054,11 +1094,97 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
 
         XPCWrappedNative* wrapper;
         nsresult rv = XPCWrappedNative::GetNewOrUsed(ccx, src, xpcscope,
-                                                     iface, &wrapper);
+                                                     iface, isGlobal,
+                                                     &wrapper);
         if(pErr)
             *pErr = rv;
         if(NS_SUCCEEDED(rv) && wrapper)
         {
+            if (allowNativeWrapper && wrapper->GetScope() != xpcscope)
+            {
+                // Cross scope access detected. Check if chrome code
+                // is accessing non-chrome objects, and if so, wrap
+                // the XPCWrappedNative with an XPCNativeWrapper to
+                // prevent user-defined properties from shadowing DOM
+                // properties from chrome code.
+
+                // printf("Wrapped native accessed across scope boundary\n");
+
+                JSScript* script = nsnull;
+                if(ccx.GetXPCContext()->CallerTypeIsJavaScript())
+                {
+                    // Called from JS.  We're going to hand the resulting
+                    // JSObject to said JS, so look for the script we want on
+                    // the stack.
+                    JSContext* cx = ccx;
+                    JSStackFrame* fp = cx->fp;
+                    while(!script && fp)
+                    {
+                        script = fp->script;
+                        fp = fp->down;
+                    }
+                }
+                else if(ccx.GetXPCContext()->CallerTypeIsNative())
+                {
+                    JSObject* callee = ccx.GetCallee();
+                    if(callee && JS_ObjectIsFunction(ccx, callee))
+                    {
+                        // Called from c++, and calling out to |callee|, which
+                        // is a JS function object.  Look for the script for
+                        // this function.
+                        JSFunction* fun =
+                            (JSFunction*) JS_GetPrivate(ccx, callee);
+                        NS_ASSERTION(fun,
+                                     "Must have JSFunction for a Function "
+                                     "object");
+                        script = JS_GetFunctionScript(ccx, fun);
+                    }
+                    // Else we don't know whom we're calling, so don't create
+                    // XPCNativeWrappers.
+                }
+                // else don't create XPCNativeWrappers, since we have
+                // no idea what's calling what here.
+                
+                uint32 flags = script ? JS_GetScriptFilenameFlags(script) : 0;
+                NS_ASSERTION(flags != JSFILENAME_NULL, "null script filename");
+
+                if((flags & JSFILENAME_SYSTEM) &&
+                   !JS_IsSystemObject(ccx, wrapper->GetFlatJSObject()))
+                {
+#ifdef DEBUG_XPCNativeWrapper
+                    {
+                        char *s = wrapper->ToString(ccx);
+                        printf("Content accessed from chrome, wrapping wrapper (%s) in XPCNativeWrapper\n", s);
+                        if (s)
+                            JS_smprintf_free(s);
+                    }
+#endif
+
+                    JSObject *nativeWrapper =
+                        XPCNativeWrapper::GetNewOrUsed(ccx, wrapper);
+
+                    if (nativeWrapper)
+                    {
+                        XPCJSObjectHolder *objHolder =
+                            XPCJSObjectHolder::newHolder(ccx, nativeWrapper);
+
+                        if (objHolder)
+                        {
+                            NS_ADDREF(objHolder);
+                            NS_RELEASE(wrapper);
+
+                            *dest = objHolder;
+                            return JS_TRUE;
+                        }
+                    }
+
+                    // Out of memory or other failure that already
+                    // threw a JS exception.
+                    NS_RELEASE(wrapper);
+                    return JS_FALSE;
+                }
+            }
+
             *dest = NS_STATIC_CAST(nsIXPConnectJSObjectHolder*, wrapper);
             return JS_TRUE;
         }
@@ -1113,6 +1239,12 @@ XPCConvert::JSObject2NativeInterface(XPCCallContext& ccx,
         }
         // else...
         
+        // XXX E4X breaks the world. Don't try wrapping E4X objects!
+        // This hack can be removed (or changed accordingly) when the
+        // DOM <-> E4X bindings are complete, see bug 270553
+        if(JS_TypeOfValue(cx, OBJECT_TO_JSVAL(src)) == JSTYPE_XML)
+            return JS_FALSE;
+
         // Does the JSObject have 'nsISupportness'?
         // XXX hmm, I wonder if this matters anymore with no 
         // oldstyle DOM objects around.
@@ -1162,19 +1294,25 @@ XPCConvert::ConstructException(nsresult rv, const char* message,
     static const char format[] = "\'%s\' when calling method: [%s::%s]";
     const char * msg = message;
     char* sz = nsnull;
+    nsXPIDLString xmsg;
+    nsCAutoString sxmsg;
 
+    nsCOMPtr<nsIScriptError> errorObject = do_QueryInterface(data);
+    if(errorObject) {
+        if (NS_SUCCEEDED(errorObject->GetMessage(getter_Copies(xmsg)))) {
+            CopyUTF16toUTF8(xmsg, sxmsg);
+            msg = sxmsg.get();
+        }
+    }
     if(!msg)
         if(!nsXPCException::NameAndFormatForNSResult(rv, nsnull, &msg) || ! msg)
             msg = "<error>";
-
     if(ifaceName && methodName)
-        sz = JS_smprintf(format, msg, ifaceName, methodName);
-    else
-        sz = (char*) msg; // I promise to play nice after casting away const
+        msg = sz = JS_smprintf(format, msg, ifaceName, methodName);
 
-    nsresult res = nsXPCException::NewException(sz, rv, nsnull, data, exceptn);
+    nsresult res = nsXPCException::NewException(msg, rv, nsnull, data, exceptn);
 
-    if(sz && sz != msg)
+    if(sz)
         JS_smprintf_free(sz);
     return res;
 }
@@ -1312,8 +1450,6 @@ XPCConvert::JSValToXPCException(XPCCallContext& ccx,
         }
         else
         {
-            if(JSVAL_TO_DOUBLE(s))
-            {
                 number = *(JSVAL_TO_DOUBLE(s));
                 if(number > 0.0 &&
                    number < (double)0xffffffff &&
@@ -1322,7 +1458,6 @@ XPCConvert::JSValToXPCException(XPCCallContext& ccx,
                     rv = (nsresult) number;
                     if(NS_FAILED(rv))
                         isResult = JS_TRUE;
-                }
             }
         }
 
@@ -1389,9 +1524,12 @@ XPCConvert::JSErrorToXPCException(XPCCallContext& ccx,
         }
 
         data = new nsScriptError();
+        if(!data)
+            return NS_ERROR_OUT_OF_MEMORY;
+
         NS_ADDREF(data);
         data->Init(bestMessage.get(),
-                   NS_ConvertASCIItoUCS2(report->filename).get(),
+                   NS_ConvertASCIItoUTF16(report->filename).get(),
                    (const PRUnichar *)report->uclinebuf, report->lineno,
                    report->uctokenptr - report->uclinebuf, report->flags,
                    "XPConnect JavaScript");
@@ -1401,16 +1539,13 @@ XPCConvert::JSErrorToXPCException(XPCCallContext& ccx,
 
     if(data)
     {
-        char* formattedMsg;
-        if(NS_FAILED(data->ToString(&formattedMsg)))
-            formattedMsg = nsnull;
+        nsCAutoString formattedMsg;
+        data->ToString(formattedMsg);
 
         rv = ConstructException(NS_ERROR_XPC_JAVASCRIPT_ERROR_WITH_DETAILS,
-                                formattedMsg, ifaceName, methodName, data,
+                                formattedMsg.get(), ifaceName, methodName, data,
                                 exceptn);
 
-        if(formattedMsg)
-            nsMemory::Free(formattedMsg);
         NS_RELEASE(data);
     }
     else
@@ -1561,7 +1696,8 @@ XPCConvert::NativeArray2JS(XPCCallContext& ccx,
         *pErr = NS_ERROR_XPC_BAD_CONVERT_NATIVE;
 
     JSUint32 i;
-    jsval current;
+    jsval current = JSVAL_NULL;
+    AUTO_MARK_JSVAL(ccx, &current);
 
 #define POPULATE(_t)                                                         \
     PR_BEGIN_MACRO                                                           \
