@@ -42,6 +42,8 @@
 
 #include <stdio.h>
 
+#include "nsComponentManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 #include "nsIXPConnect.h"
 #include "nsIScriptError.h"
 #include "nsIServiceManager.h"
@@ -52,8 +54,9 @@
 #include "nsMemory.h"
 #include "nsIXPCSecurityManager.h"
 #include "nsICategoryManager.h"
-#include "nsString.h"
 #include "nsIVariant.h"
+#include "nsStringAPI.h"
+#include "nsEmbedString.h"
 
 #include "jsapi.h"
 
@@ -113,9 +116,9 @@ Load(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 }
 
 static JSFunctionSpec glob_functions[] = {
-    {"print",           Print,          0},
-    {"load",            Load,           1},
-    {0}
+    {"print",           Print,          0,0,0},
+    {"load",            Load,           1,0,0},
+    {nsnull,nsnull,0,0,0}
 };
 
 static JSClass global_class = {
@@ -515,7 +518,7 @@ TestArgFormatter(JSContext* jscontext, JSObject* glob, nsIXPConnect* xpc)
         goto out;
     }
 
-    if(d_in != d_out)
+    if(!d_in.Equals(d_out))
     {
         printf(" JS to native for %%is returned the wrong value -- FAILED!\n");
         goto out;
@@ -622,13 +625,11 @@ static void ShowXPCException()
                         nsCOMPtr<nsIScriptError> report = do_QueryInterface(data);
                         if(report)
                         {
-                            char* str2;
-                            rv = report->ToString(&str2);
-                            if(NS_SUCCEEDED(rv) && str2)
+                            nsCAutoString str2;
+                            rv = report->ToString(str2);
+                            if(NS_SUCCEEDED(rv))
                             {
-                                printf(str2);
-                                printf("\n");
-                                nsMemory::Free(str2);
+                                printf("%s\n", str2.get());
                             }                            
                         }                            
                     }                            
