@@ -76,6 +76,7 @@
 class nsMathMLContainerFrame : public nsHTMLContainerFrame,
                                public nsMathMLFrame {
 public:
+  nsMathMLContainerFrame(nsStyleContext* aContext) : nsHTMLContainerFrame(aContext) {}
 
   NS_DECL_ISUPPORTS_INHERITED
 
@@ -83,82 +84,58 @@ public:
   // Overloaded nsMathMLFrame methods -- see documentation in nsIMathMLFrame.h
 
   NS_IMETHOD
-  Stretch(nsPresContext*      aPresContext,
-          nsIRenderingContext& aRenderingContext,
+  Stretch(nsIRenderingContext& aRenderingContext,
           nsStretchDirection   aStretchDirection,
           nsBoundingMetrics&   aContainerSize,
           nsHTMLReflowMetrics& aDesiredStretchSize);
 
   NS_IMETHOD
-  Place(nsPresContext*      aPresContext,
-        nsIRenderingContext& aRenderingContext,
+  Place(nsIRenderingContext& aRenderingContext,
         PRBool               aPlaceOrigin,
         nsHTMLReflowMetrics& aDesiredSize);
 
   NS_IMETHOD
-  UpdatePresentationDataFromChildAt(nsPresContext* aPresContext,
-                                    PRInt32         aFirstIndex,
+  UpdatePresentationDataFromChildAt(PRInt32         aFirstIndex,
                                     PRInt32         aLastIndex,
                                     PRInt32         aScriptLevelIncrement,
                                     PRUint32        aFlagsValues,
                                     PRUint32        aFlagsToUpdate)
   {
-    PropagatePresentationDataFromChildAt(aPresContext, this,
-      aFirstIndex, aLastIndex, aScriptLevelIncrement, aFlagsValues, aFlagsToUpdate);
+    PropagatePresentationDataFromChildAt(this, aFirstIndex, aLastIndex,
+      aScriptLevelIncrement, aFlagsValues, aFlagsToUpdate);
     return NS_OK;
   }
 
   NS_IMETHOD
-  ReResolveScriptStyle(nsPresContext* aPresContext,
-                       PRInt32         aParentScriptLevel)
+  ReResolveScriptStyle(PRInt32 aParentScriptLevel)
   {
-    PropagateScriptStyleFor(aPresContext, this, aParentScriptLevel);
+    PropagateScriptStyleFor(this, aParentScriptLevel);
     return NS_OK;
   }
 
   // --------------------------------------------------------------------------
   // Overloaded nsHTMLContainerFrame methods -- see documentation in nsIFrame.h
 
-  virtual nsIAtom* GetType() const;
+  virtual PRBool IsFrameOfType(PRUint32 aFlags) const;
 
   NS_IMETHOD
-  Init(nsPresContext*  aPresContext,
-       nsIContent*      aContent,
+  Init(nsIContent*      aContent,
        nsIFrame*        aParent,
-       nsStyleContext*  aContext,
        nsIFrame*        aPrevInFlow);
 
   NS_IMETHOD
-  SetInitialChildList(nsPresContext* aPresContext,
-                      nsIAtom*        aListName,
-                      nsIFrame*       aChildList);
-
-  NS_IMETHOD
-  AppendFrames(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
+  AppendFrames(nsIAtom*        aListName,
                nsIFrame*       aFrameList);
 
   NS_IMETHOD
-  InsertFrames(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
+  InsertFrames(nsIAtom*        aListName,
                nsIFrame*       aPrevFrame,
                nsIFrame*       aFrameList);
 
   NS_IMETHOD
-  RemoveFrame(nsPresContext* aPresContext,
-              nsIPresShell&   aPresShell,
-              nsIAtom*        aListName,
+  RemoveFrame(nsIAtom*        aListName,
               nsIFrame*       aOldFrame);
 
-  NS_IMETHOD
-  ReplaceFrame(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
-               nsIFrame*       aOldFrame,
-               nsIFrame*       aNewFrame);
-
   NS_IMETHODIMP
   ReflowDirtyChild(nsIPresShell* aPresShell, 
                    nsIFrame*     aChild);
@@ -169,6 +146,13 @@ public:
          const nsHTMLReflowState& aReflowState,
          nsReflowStatus&          aStatus);
 
+  NS_IMETHOD
+  WillReflow(nsPresContext* aPresContext)
+  {
+    mPresentationData.flags &= ~NS_MATHML_ERROR;
+    return nsHTMLContainerFrame::WillReflow(aPresContext);
+  }
+
   NS_IMETHOD
   DidReflow(nsPresContext*           aPresContext,
             const nsHTMLReflowState*  aReflowState,
@@ -179,12 +163,9 @@ public:
     return nsHTMLContainerFrame::DidReflow(aPresContext, aReflowState, aStatus);
   }
 
-  NS_IMETHOD 
-  Paint(nsPresContext*      aPresContext,
-        nsIRenderingContext& aRenderingContext,
-        const nsRect&        aDirtyRect,
-        nsFramePaintLayer    aWhichLayer,
-        PRUint32             aFlags = 0);
+  NS_IMETHOD BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                              const nsRect&           aDirtyRect,
+                              const nsDisplayListSet& aLists);
 
   // Notification when an attribute is changed. The MathML module uses the
   // following paradigm:
@@ -204,9 +185,7 @@ public:
   //        Therefore, there is an overhead here in that our siblings are re-laid
   //        too (e.g., this happens with <mstyle>, <munder>, <mover>, <munderover>). 
   NS_IMETHOD
-  AttributeChanged(nsPresContext* aPresContext,
-                   nsIContent*     aChild,
-                   PRInt32         aNameSpaceID,
+  AttributeChanged(PRInt32         aNameSpaceID,
                    nsIAtom*        aAttribute,
                    PRInt32         aModType);
 
@@ -216,19 +195,12 @@ public:
   // helper to re-sync the automatic data in our children and notify our parent to
   // reflow us when changes (e.g., append/insert/remove) happen in our child list
   virtual nsresult
-  ChildListChanged(nsPresContext* aPresContext,
-                   PRInt32         aModType);
-
-  // helper to wrap non-MathML frames so that foreign elements (e.g., html:img)
-  // can mix better with other surrounding MathML markups
-  virtual nsresult
-  WrapForeignFrames(nsPresContext* aPresContext);
+  ChildListChanged(PRInt32 aModType);
 
   // helper to get the preferred size that a container frame should use to fire
   // the stretch on its stretchy child frames.
   virtual void
-  GetPreferredStretchSize(nsPresContext*      aPresContext,
-                          nsIRenderingContext& aRenderingContext,
+  GetPreferredStretchSize(nsIRenderingContext& aRenderingContext,
                           PRUint32             aOptions,
                           nsStretchDirection   aStretchDirection,
                           nsBoundingMetrics&   aPreferredStretchSize);
@@ -236,14 +208,8 @@ public:
   // error handlers to provide a visual feedback to the user when an error
   // (typically invalid markup) was encountered during reflow.
   virtual nsresult
-  ReflowError(nsPresContext*      aPresContext,
-              nsIRenderingContext& aRenderingContext,
+  ReflowError(nsIRenderingContext& aRenderingContext,
               nsHTMLReflowMetrics& aDesiredSize);
-  virtual nsresult
-  PaintError(nsPresContext*      aPresContext,
-             nsIRenderingContext& aRenderingContext,
-             const nsRect&        aDirtyRect,
-             nsFramePaintLayer    aWhichLayer);
 
   // helper method to reflow a child frame. We are inline frames, and we don't
   // know our positions until reflow is finished. That's why we ask the
@@ -253,11 +219,14 @@ public:
               nsPresContext*          aPresContext,
               nsHTMLReflowMetrics&     aDesiredSize,
               const nsHTMLReflowState& aReflowState,
-              nsReflowStatus&          aStatus)
-  {
-    return nsHTMLContainerFrame::ReflowChild(aKidFrame, aPresContext, aDesiredSize, aReflowState,
-                                             0, 0, NS_FRAME_NO_MOVE_FRAME, aStatus);
-  }
+              nsReflowStatus&          aStatus);
+
+  nsresult 
+  ReflowForeignChild(nsIFrame*                aKidFrame,
+                     nsPresContext*           aPresContext,
+                     nsHTMLReflowMetrics&     aDesiredSize,
+                     const nsHTMLReflowState& aReflowState,
+                     nsReflowStatus&          aStatus);
 
   // helper to add the inter-spacing when <math> is the immediate parent.
   // Since we don't (yet) handle the root <math> element ourselves, we need to
@@ -266,45 +235,43 @@ public:
   // left to right on the childframes of <math>, and by so doing it will
   // emulate the spacing that would have been done by a <mrow> container.
   // e.g., it fixes <math> <mi>f</mi> <mo>q</mo> <mi>f</mi> <mo>I</mo> </math>
-  virtual nsresult
-  FixInterFrameSpacing(nsPresContext*      aPresContext,
-                       nsHTMLReflowMetrics& aDesiredSize);
+  virtual nscoord
+  FixInterFrameSpacing(nsHTMLReflowMetrics& aDesiredSize);
 
   // helper method to complete the post-reflow hook and ensure that embellished
   // operators don't terminate their Reflow without receiving a Stretch command.
   virtual nsresult
-  FinalizeReflow(nsPresContext*      aPresContext,
-                 nsIRenderingContext& aRenderingContext,
+  FinalizeReflow(nsIRenderingContext& aRenderingContext,
                  nsHTMLReflowMetrics& aDesiredSize);
 
-  // helper method to facilitate getting the reflow and bounding metrics
+  // helper method to facilitate getting the reflow and bounding metrics.
+  // The argument aMathMLFrameType, when non null, will return the 'type' of
+  // the frame, which is used to determine the inter-frame spacing.
   // IMPORTANT: This function is only meant to be called in Place() methods 
   // where it is assumed that the frame's rect is still acting as place holder
   // for the frame's ascent and descent information
   static void
   GetReflowAndBoundingMetricsFor(nsIFrame*            aFrame,
                                  nsHTMLReflowMetrics& aReflowMetrics,
-                                 nsBoundingMetrics&   aBoundingMetrics);
+                                 nsBoundingMetrics&   aBoundingMetrics,
+                                 eMathMLFrameType*    aMathMLFrameType = nsnull);
 
   // helper to let the scriptstyle re-resolution pass through
   // a subtree that may contain non-MathML container frames
   static void
-  PropagateScriptStyleFor(nsPresContext* aPresContext,
-                          nsIFrame*       aFrame,
+  PropagateScriptStyleFor(nsIFrame*       aFrame,
                           PRInt32         aParentScriptLevel);
 
   // helper to let the update of presentation data pass through
   // a subtree that may contain non-MathML container frames
   static void
-  PropagatePresentationDataFor(nsPresContext* aPresContext,
-                               nsIFrame*       aFrame,
+  PropagatePresentationDataFor(nsIFrame*       aFrame,
                                PRInt32         aScriptLevelIncrement,
                                PRUint32        aFlagsValues,
                                PRUint32        aFlagsToUpdate);
 
   static void
-  PropagatePresentationDataFromChildAt(nsPresContext* aPresContext,
-                                       nsIFrame*       aParentFrame,
+  PropagatePresentationDataFromChildAt(nsIFrame*       aParentFrame,
                                        PRInt32         aFirstChildIndex,
                                        PRInt32         aLastChildIndex,
                                        PRInt32         aScriptLevelIncrement,
@@ -324,8 +291,7 @@ public:
   // has to request the re-build from its parent. Unfortunately, the extra cost
   // for this is that it will re-sync in the siblings of the child as well.
   static void
-  RebuildAutomaticDataForChildren(nsPresContext* aPresContext,
-                                  nsIFrame*       aParentFrame);
+  RebuildAutomaticDataForChildren(nsIFrame* aParentFrame);
 
   // helper to blow away the automatic data cached in a frame's subtree and
   // re-layout its subtree to reflect changes that may have happen. In the
@@ -336,8 +302,7 @@ public:
   // frame itself (except for those particular operations that the parent frame
   // may do do its TransmitAutomaticData()). @see RebuildAutomaticDataForChildren
   static nsresult
-  ReLayoutChildren(nsPresContext* aPresContext,
-                   nsIFrame*       aParentFrame);
+  ReLayoutChildren(nsIFrame* aParentFrame);
 
 protected:
   virtual PRIntn GetSkipSides() const { return 0; }
@@ -354,19 +319,18 @@ protected:
 // Issues: If/when mathml becomes a pluggable component, the separation will be needed.
 class nsMathMLmathBlockFrame : public nsBlockFrame {
 public:
-  friend nsresult NS_NewMathMLmathBlockFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame);
+  friend nsIFrame* NS_NewMathMLmathBlockFrame(nsIPresShell* aPresShell, nsStyleContext* aContext);
 
   // beware, mFrames is not set by nsBlockFrame
   // cannot use mFrames{.FirstChild()|.etc} since the block code doesn't set mFrames
   NS_IMETHOD
-  SetInitialChildList(nsPresContext* aPresContext,
-                      nsIAtom*        aListName,
+  SetInitialChildList(nsIAtom*        aListName,
                       nsIFrame*       aChildList)
   {
-    nsresult rv = nsBlockFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+    nsresult rv = nsBlockFrame::SetInitialChildList(aListName, aChildList);
     // re-resolve our subtree to set any mathml-expected data
-    nsMathMLContainerFrame::MapAttributesIntoCSS(aPresContext, this);
-    nsMathMLContainerFrame::RebuildAutomaticDataForChildren(aPresContext, this);
+    nsMathMLContainerFrame::MapCommonAttributesIntoCSS(GetPresContext(), this);
+    nsMathMLContainerFrame::RebuildAutomaticDataForChildren(this);
     return rv;
   }
 
@@ -378,66 +342,55 @@ public:
   {
     if (mScriptStyleChanged) {
       mScriptStyleChanged = PR_FALSE;
-      nsMathMLContainerFrame::PropagateScriptStyleFor(aPresContext, this, 0);
+      nsMathMLContainerFrame::PropagateScriptStyleFor(this, 0);
     }
     return nsBlockFrame::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
   }
 
   NS_IMETHOD
-  AppendFrames(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
+  AppendFrames(nsIAtom*        aListName,
                nsIFrame*       aFrameList)
   {
-    nsresult rv = nsBlockFrame::AppendFrames(aPresContext, aPresShell, aListName, aFrameList);
-    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    NS_ASSERTION(!aListName, "internal error");
+    nsresult rv = nsBlockFrame::AppendFrames(aListName, aFrameList);
+    nsMathMLContainerFrame::ReLayoutChildren(this);
     return rv;
   }
 
   NS_IMETHOD
-  InsertFrames(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
+  InsertFrames(nsIAtom*        aListName,
                nsIFrame*       aPrevFrame,
                nsIFrame*       aFrameList)
   {
-    nsresult rv = nsBlockFrame::InsertFrames(aPresContext, aPresShell, aListName, aPrevFrame, aFrameList);
-    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    NS_ASSERTION(!aListName, "internal error");
+    nsresult rv = nsBlockFrame::InsertFrames(aListName, aPrevFrame, aFrameList);
+    nsMathMLContainerFrame::ReLayoutChildren(this);
     return rv;
   }
 
   NS_IMETHOD
-  ReplaceFrame(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
-               nsIFrame*       aOldFrame,
-               nsIFrame*       aNewFrame)
-  {
-    nsresult rv = nsBlockFrame::ReplaceFrame(aPresContext, aPresShell, aListName, aOldFrame, aNewFrame);
-    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
-    return rv;
-  }
-
-  NS_IMETHOD
-  RemoveFrame(nsPresContext* aPresContext,
-              nsIPresShell&   aPresShell,
-              nsIAtom*        aListName,
+  RemoveFrame(nsIAtom*        aListName,
               nsIFrame*       aOldFrame)
   {
-    nsresult rv = nsBlockFrame::RemoveFrame(aPresContext, aPresShell, aListName, aOldFrame);
-    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    NS_ASSERTION(!aListName, "internal error");
+    nsresult rv = nsBlockFrame::RemoveFrame(aListName, aOldFrame);
+    nsMathMLContainerFrame::ReLayoutChildren(this);
     return rv;
   }
 
 protected:
-  nsMathMLmathBlockFrame() {}
+  nsMathMLmathBlockFrame(nsStyleContext* aContext) : nsBlockFrame(aContext) {
+    // We should always have a space manager.  Not that things can really try
+    // to float out of us anyway, but we need one for line layout.
+    AddStateBits(NS_BLOCK_SPACE_MGR);
+  }
   virtual ~nsMathMLmathBlockFrame() {}
 
   NS_IMETHOD
-  DidSetStyleContext(nsPresContext* aPresContext)
+  DidSetStyleContext()
   {
     mScriptStyleChanged = PR_TRUE;
-    return nsBlockFrame::DidSetStyleContext(aPresContext);
+    return nsBlockFrame::DidSetStyleContext();
   }
 
   PRBool mScriptStyleChanged;
@@ -447,17 +400,16 @@ protected:
 
 class nsMathMLmathInlineFrame : public nsInlineFrame {
 public:
-  friend nsresult NS_NewMathMLmathInlineFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame);
+  friend nsIFrame* NS_NewMathMLmathInlineFrame(nsIPresShell* aPresShell, nsStyleContext* aContext);
 
   NS_IMETHOD
-  SetInitialChildList(nsPresContext* aPresContext,
-                      nsIAtom*        aListName,
+  SetInitialChildList(nsIAtom*        aListName,
                       nsIFrame*       aChildList)
   {
-    nsresult rv = nsInlineFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+    nsresult rv = nsInlineFrame::SetInitialChildList(aListName, aChildList);
     // re-resolve our subtree to set any mathml-expected data
-    nsMathMLContainerFrame::MapAttributesIntoCSS(aPresContext, this);
-    nsMathMLContainerFrame::RebuildAutomaticDataForChildren(aPresContext, this);
+    nsMathMLContainerFrame::MapCommonAttributesIntoCSS(GetPresContext(), this);
+    nsMathMLContainerFrame::RebuildAutomaticDataForChildren(this);
     return rv;
   }
 
@@ -469,66 +421,51 @@ public:
   {
     if (mScriptStyleChanged) {
       mScriptStyleChanged = PR_FALSE;
-      nsMathMLContainerFrame::PropagateScriptStyleFor(aPresContext, this, 0);
+      nsMathMLContainerFrame::PropagateScriptStyleFor(this, 0);
     }
     return nsInlineFrame::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
   }
 
   NS_IMETHOD
-  AppendFrames(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
+  AppendFrames(nsIAtom*        aListName,
                nsIFrame*       aFrameList)
   {
-    nsresult rv = nsInlineFrame::AppendFrames(aPresContext, aPresShell, aListName, aFrameList);
-    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    NS_ASSERTION(!aListName, "internal error");
+    nsresult rv = nsInlineFrame::AppendFrames(aListName, aFrameList);
+    nsMathMLContainerFrame::ReLayoutChildren(this);
     return rv;
   }
 
   NS_IMETHOD
-  InsertFrames(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
+  InsertFrames(nsIAtom*        aListName,
                nsIFrame*       aPrevFrame,
                nsIFrame*       aFrameList)
   {
-    nsresult rv = nsInlineFrame::InsertFrames(aPresContext, aPresShell, aListName, aPrevFrame, aFrameList);
-    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    NS_ASSERTION(!aListName, "internal error");
+    nsresult rv = nsInlineFrame::InsertFrames(aListName, aPrevFrame, aFrameList);
+    nsMathMLContainerFrame::ReLayoutChildren(this);
     return rv;
   }
 
   NS_IMETHOD
-  ReplaceFrame(nsPresContext* aPresContext,
-               nsIPresShell&   aPresShell,
-               nsIAtom*        aListName,
-               nsIFrame*       aOldFrame,
-               nsIFrame*       aNewFrame)
-  {
-    nsresult rv = nsInlineFrame::ReplaceFrame(aPresContext, aPresShell, aListName, aOldFrame, aNewFrame);
-    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
-    return rv;
-  }
-
-  NS_IMETHOD
-  RemoveFrame(nsPresContext* aPresContext,
-              nsIPresShell&   aPresShell,
-              nsIAtom*        aListName,
+  RemoveFrame(nsIAtom*        aListName,
               nsIFrame*       aOldFrame)
   {
-    nsresult rv = nsInlineFrame::RemoveFrame(aPresContext, aPresShell, aListName, aOldFrame);
-    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    NS_ASSERTION(!aListName, "internal error");
+    nsresult rv = nsInlineFrame::RemoveFrame(aListName, aOldFrame);
+    nsMathMLContainerFrame::ReLayoutChildren(this);
     return rv;
   }
 
 protected:
-  nsMathMLmathInlineFrame() {}
+  nsMathMLmathInlineFrame(nsStyleContext* aContext) : nsInlineFrame(aContext) {}
   virtual ~nsMathMLmathInlineFrame() {}
 
   NS_IMETHOD
-  DidSetStyleContext(nsPresContext* aPresContext)
+  DidSetStyleContext()
   {
     mScriptStyleChanged = PR_TRUE;
-    return nsInlineFrame::DidSetStyleContext(aPresContext);
+    return nsInlineFrame::DidSetStyleContext();
   }
 
   PRBool mScriptStyleChanged;
