@@ -53,23 +53,10 @@
 // <mfenced> -- surround content with a pair of fences
 //
 
-nsresult
-NS_NewMathMLmfencedFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
-{
-  NS_PRECONDITION(aNewFrame, "null OUT ptr");
-  if (nsnull == aNewFrame) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  nsMathMLmfencedFrame* it = new (aPresShell) nsMathMLmfencedFrame;
-  if (nsnull == it) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  *aNewFrame = it;  
-  return NS_OK;
-}
-
-nsMathMLmfencedFrame::nsMathMLmfencedFrame()
+nsIFrame*
+NS_NewMathMLmfencedFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
+  return new (aPresShell) nsMathMLmfencedFrame(aContext);
 }
 
 nsMathMLmfencedFrame::~nsMathMLmfencedFrame()
@@ -78,11 +65,10 @@ nsMathMLmfencedFrame::~nsMathMLmfencedFrame()
 }
 
 NS_IMETHODIMP
-nsMathMLmfencedFrame::InheritAutomaticData(nsPresContext* aPresContext,
-                                           nsIFrame*       aParent)
+nsMathMLmfencedFrame::InheritAutomaticData(nsIFrame* aParent)
 {
   // let the base class get the default from our parent
-  nsMathMLContainerFrame::InheritAutomaticData(aPresContext, aParent);
+  nsMathMLContainerFrame::InheritAutomaticData(aParent);
 
   mPresentationData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_VERTICALLY;
 
@@ -90,43 +76,38 @@ nsMathMLmfencedFrame::InheritAutomaticData(nsPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsMathMLmfencedFrame::SetInitialChildList(nsPresContext* aPresContext,
-                                          nsIAtom*        aListName,
+nsMathMLmfencedFrame::SetInitialChildList(nsIAtom*        aListName,
                                           nsIFrame*       aChildList)
 {
   // First, let the base class do its work
-  nsresult rv = nsMathMLContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+  nsresult rv = nsMathMLContainerFrame::SetInitialChildList(aListName, aChildList);
   if (NS_FAILED(rv)) return rv;
 
   // No need to tract the style contexts given to our MathML chars. 
   // The Style System will use Get/SetAdditionalStyleContext() to keep them
   // up-to-date if dynamic changes arise.
-  return CreateFencesAndSeparators(aPresContext);
+  return CreateFencesAndSeparators(GetPresContext());
 }
 
 NS_IMETHODIMP
-nsMathMLmfencedFrame::AttributeChanged(nsPresContext* aPresContext,
-                                       nsIContent*     aContent,
-                                       PRInt32         aNameSpaceID,
+nsMathMLmfencedFrame::AttributeChanged(PRInt32         aNameSpaceID,
                                        nsIAtom*        aAttribute,
                                        PRInt32         aModType)
 {
   RemoveFencesAndSeparators();
-  CreateFencesAndSeparators(aPresContext);
+  CreateFencesAndSeparators(GetPresContext());
 
   return nsMathMLContainerFrame::
-         AttributeChanged(aPresContext, aContent, aNameSpaceID,
-                          aAttribute, aModType);
+         AttributeChanged(aNameSpaceID, aAttribute, aModType);
 }
 
 nsresult
-nsMathMLmfencedFrame::ChildListChanged(nsPresContext* aPresContext,
-                                       PRInt32         aModType)
+nsMathMLmfencedFrame::ChildListChanged(PRInt32 aModType)
 {
   RemoveFencesAndSeparators();
-  CreateFencesAndSeparators(aPresContext);
+  CreateFencesAndSeparators(GetPresContext());
 
-  return nsMathMLContainerFrame::ChildListChanged(aPresContext, aModType);
+  return nsMathMLContainerFrame::ChildListChanged(aModType);
 }
 
 void
@@ -145,66 +126,56 @@ nsMathMLmfencedFrame::RemoveFencesAndSeparators()
 nsresult
 nsMathMLmfencedFrame::CreateFencesAndSeparators(nsPresContext* aPresContext)
 {
-  nsresult rv;
-  nsAutoString value, data;
+  nsAutoString value;
   PRBool isMutable = PR_FALSE;
 
   //////////////  
   // see if the opening fence is there ...
-  rv = GetAttribute(mContent, mPresentationData.mstyle,
-                    nsMathMLAtoms::open_, value);
-  if (NS_CONTENT_ATTR_HAS_VALUE == rv) {
+  if (!GetAttribute(mContent, mPresentationData.mstyle, nsMathMLAtoms::open,
+                    value)) {
+    value = PRUnichar('('); // default as per the MathML REC
+  }
+  else {
     value.Trim(" ");
-    data = value;
   }
-  else if (NS_CONTENT_ATTR_NOT_THERE == rv)
-    data = PRUnichar('('); // default as per the MathML REC
-  else
-    data.Truncate();
 
-  if (!data.IsEmpty()) {
+  if (!value.IsEmpty()) {
     mOpenChar = new nsMathMLChar;
     if (!mOpenChar) return NS_ERROR_OUT_OF_MEMORY;
-    mOpenChar->SetData(aPresContext, data);
-    isMutable = nsMathMLOperators::IsMutableOperator(data);
+    mOpenChar->SetData(aPresContext, value);
+    isMutable = nsMathMLOperators::IsMutableOperator(value);
     ResolveMathMLCharStyle(aPresContext, mContent, mStyleContext, mOpenChar, isMutable);
   }
 
   //////////////
   // see if the closing fence is there ...
-  rv = GetAttribute(mContent, mPresentationData.mstyle,
-                    nsMathMLAtoms::close_, value);
-  if (NS_CONTENT_ATTR_HAS_VALUE == rv) {
+  if(!GetAttribute(mContent, mPresentationData.mstyle,
+                    nsMathMLAtoms::close, value)) {
+    value = PRUnichar(')'); // default as per the MathML REC
+  }
+  else {
     value.Trim(" ");
-    data = value;
   }
-  else if (NS_CONTENT_ATTR_NOT_THERE == rv)
-    data = PRUnichar(')'); // default as per the MathML REC
-  else
-    data.Truncate();
 
-  if (!data.IsEmpty()) {
+  if (!value.IsEmpty()) {
     mCloseChar = new nsMathMLChar;
     if (!mCloseChar) return NS_ERROR_OUT_OF_MEMORY;
-    mCloseChar->SetData(aPresContext, data);
-    isMutable = nsMathMLOperators::IsMutableOperator(data);
+    mCloseChar->SetData(aPresContext, value);
+    isMutable = nsMathMLOperators::IsMutableOperator(value);
     ResolveMathMLCharStyle(aPresContext, mContent, mStyleContext, mCloseChar, isMutable);
   }
 
   //////////////
   // see if separators are there ...
-  rv = GetAttribute(mContent, mPresentationData.mstyle, 
-                    nsMathMLAtoms::separators_, value);
-  if (NS_CONTENT_ATTR_HAS_VALUE == rv) {
+  if(!GetAttribute(mContent, mPresentationData.mstyle, 
+                   nsMathMLAtoms::separators_, value)) {
+    value = PRUnichar(','); // default as per the MathML REC
+  }
+  else {
     value.Trim(" ");
-    data = value;
   }
-  else if (NS_CONTENT_ATTR_NOT_THERE == rv)
-    data = PRUnichar(','); // default as per the MathML REC
-  else
-    data.Truncate();
 
-  mSeparatorsCount = data.Length();
+  mSeparatorsCount = value.Length();
   if (0 < mSeparatorsCount) {
     PRInt32 sepCount = mFrames.GetLength() - 1;
     if (0 < sepCount) {
@@ -213,11 +184,11 @@ nsMathMLmfencedFrame::CreateFencesAndSeparators(nsPresContext* aPresContext)
       nsAutoString sepChar;
       for (PRInt32 i = 0; i < sepCount; i++) {
         if (i < mSeparatorsCount) {
-          sepChar = data[i];
+          sepChar = value[i];
           isMutable = nsMathMLOperators::IsMutableOperator(sepChar);
         }
         else {
-          sepChar = data[mSeparatorsCount-1];
+          sepChar = value[mSeparatorsCount-1];
           // keep the value of isMutable that was set earlier
         }
         mSeparatorsChar[i].SetData(aPresContext, sepChar);
@@ -230,29 +201,32 @@ nsMathMLmfencedFrame::CreateFencesAndSeparators(nsPresContext* aPresContext)
 }
 
 NS_IMETHODIMP
-nsMathMLmfencedFrame::Paint(nsPresContext*      aPresContext,
-                            nsIRenderingContext& aRenderingContext,
-                            const nsRect&        aDirtyRect,
-                            nsFramePaintLayer    aWhichLayer,
-                            PRUint32             aFlags)
+nsMathMLmfencedFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                       const nsRect&           aDirtyRect,
+                                       const nsDisplayListSet& aLists)
 {
   /////////////
-  // paint the content
-  nsresult rv = nsMathMLContainerFrame::Paint(aPresContext, aRenderingContext, 
-                                              aDirtyRect, aWhichLayer);
+  // display the content
+  nsresult rv = nsMathMLContainerFrame::BuildDisplayList(aBuilder, aDirtyRect, aLists);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
   ////////////
-  // paint fences and separators
-  if (NS_SUCCEEDED(rv)) {
-    if (mOpenChar) mOpenChar->Paint(aPresContext, aRenderingContext,
-                                    aDirtyRect, aWhichLayer, this);
-    if (mCloseChar) mCloseChar->Paint(aPresContext, aRenderingContext,
-                                      aDirtyRect, aWhichLayer, this);
-    for (PRInt32 i = 0; i < mSeparatorsCount; i++) {
-      mSeparatorsChar[i].Paint(aPresContext, aRenderingContext,
-                               aDirtyRect, aWhichLayer, this);
-    }
+  // display fences and separators
+  if (mOpenChar) {
+    rv = mOpenChar->Display(aBuilder, this, aLists);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
-  return rv;
+  
+  if (mCloseChar) {
+    rv = mCloseChar->Display(aBuilder, this, aLists);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+  
+  for (PRInt32 i = 0; i < mSeparatorsCount; i++) {
+    rv = mSeparatorsChar[i].Display(aBuilder, this, aLists);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -323,8 +297,10 @@ nsMathMLmfencedFrame::doReflow(nsPresContext*          aPresContext,
     fm->GetMaxDescent(aDesiredSize.descent);
   }
   while (childFrame) {
+    nsReflowReason reason = (childFrame->GetStateBits() & NS_FRAME_FIRST_REFLOW)
+      ? eReflowReason_Initial : aReflowState.reason;
     nsHTMLReflowState childReflowState(aPresContext, aReflowState,
-                                       childFrame, availSize);
+                                       childFrame, availSize, reason);
     rv = mathMLFrame->ReflowChild(childFrame, aPresContext, childDesiredSize,
                                   childReflowState, childStatus);
     //NS_ASSERTION(NS_FRAME_IS_COMPLETE(childStatus), "bad status");
@@ -363,7 +339,7 @@ nsMathMLmfencedFrame::doReflow(nsPresContext*          aPresContext,
   }
   else {
     // case when the call is made for mfenced
-    mathMLFrame->GetPreferredStretchSize(aPresContext, *aReflowState.rendContext,
+    mathMLFrame->GetPreferredStretchSize(*aReflowState.rendContext,
                                          0, /* i.e., without embellishments */
                                          stretchDir, containerSize);
     childFrame = firstChild;
@@ -374,7 +350,7 @@ nsMathMLmfencedFrame::doReflow(nsPresContext*          aPresContext,
         // retrieve the metrics that was stored at the previous pass
         GetReflowAndBoundingMetricsFor(childFrame, childDesiredSize, childDesiredSize.mBoundingMetrics);
 
-        mathmlChild->Stretch(aPresContext, *aReflowState.rendContext, 
+        mathmlChild->Stretch(*aReflowState.rendContext, 
                              stretchDir, containerSize, childDesiredSize);
         // store the updated metrics
         childFrame->SetRect(nsRect(childDesiredSize.descent, childDesiredSize.ascent,
@@ -388,7 +364,7 @@ nsMathMLmfencedFrame::doReflow(nsPresContext*          aPresContext,
       childFrame = childFrame->GetNextSibling();
     }
     // bug 121748: for surrounding fences & separators, use a size that covers everything
-    mathMLFrame->GetPreferredStretchSize(aPresContext, *aReflowState.rendContext,
+    mathMLFrame->GetPreferredStretchSize(*aReflowState.rendContext,
                                          STRETCH_CONSIDER_EMBELLISHMENTS,
                                          stretchDir, containerSize);
   }
@@ -482,6 +458,9 @@ nsMathMLmfencedFrame::doReflow(nsPresContext*          aPresContext,
   mathMLFrame->SetBoundingMetrics(aDesiredSize.mBoundingMetrics);
   mathMLFrame->SetReference(nsPoint(0, aDesiredSize.ascent));
 
+  // see if we should fix the spacing
+  mathMLFrame->FixInterFrameSpacing(aDesiredSize);
+
   aStatus = NS_FRAME_COMPLETE;
   NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
@@ -592,6 +571,31 @@ nsMathMLmfencedFrame::PlaceChar(nsMathMLChar*      aMathMLChar,
   dx += rect.width;
 }
 
+nscoord
+nsMathMLmfencedFrame::FixInterFrameSpacing(nsHTMLReflowMetrics& aDesiredSize)
+{
+  nscoord gap = nsMathMLContainerFrame::FixInterFrameSpacing(aDesiredSize);
+  if (!gap) return 0;
+
+  nsRect rect;
+  if (mOpenChar) {
+    mOpenChar->GetRect(rect);
+    rect.MoveBy(gap, 0);
+    mOpenChar->SetRect(rect);
+  }
+  if (mCloseChar) {
+    mCloseChar->GetRect(rect);
+    rect.MoveBy(gap, 0);
+    mCloseChar->SetRect(rect);
+  }
+  for (PRInt32 i = 0; i < mSeparatorsCount; i++) {
+    mSeparatorsChar[i].GetRect(rect);
+    rect.MoveBy(gap, 0);
+    mSeparatorsChar[i].SetRect(rect);
+  }
+  return gap;
+}
+
 // ----------------------
 // the Style System will use these to pass the proper style context to our MathMLChar
 nsStyleContext*
