@@ -56,23 +56,10 @@
 // <munderover> -- attach an underscript-overscript pair to a base - implementation
 //
 
-nsresult
-NS_NewMathMLmunderoverFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
-{
-  NS_PRECONDITION(aNewFrame, "null OUT ptr");
-  if (nsnull == aNewFrame) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  nsMathMLmunderoverFrame* it = new (aPresShell) nsMathMLmunderoverFrame;
-  if (nsnull == it) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-  *aNewFrame = it;
-  return NS_OK;
-}
-
-nsMathMLmunderoverFrame::nsMathMLmunderoverFrame()
+nsIFrame*
+NS_NewMathMLmunderoverFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
+  return new (aPresShell) nsMathMLmunderoverFrame(aContext);
 }
 
 nsMathMLmunderoverFrame::~nsMathMLmunderoverFrame()
@@ -80,9 +67,7 @@ nsMathMLmunderoverFrame::~nsMathMLmunderoverFrame()
 }
 
 NS_IMETHODIMP
-nsMathMLmunderoverFrame::AttributeChanged(nsPresContext* aPresContext,
-                                          nsIContent*     aContent,
-                                          PRInt32         aNameSpaceID,
+nsMathMLmunderoverFrame::AttributeChanged(PRInt32         aNameSpaceID,
                                           nsIAtom*        aAttribute,
                                           PRInt32         aModType)
 {
@@ -90,22 +75,20 @@ nsMathMLmunderoverFrame::AttributeChanged(nsPresContext* aPresContext,
       nsMathMLAtoms::accentunder_ == aAttribute) {
     // When we have automatic data to update within ourselves, we ask our
     // parent to re-layout its children
-    return ReLayoutChildren(aPresContext, mParent);
+    return ReLayoutChildren(mParent);
   }
 
   return nsMathMLContainerFrame::
-         AttributeChanged(aPresContext, aContent, aNameSpaceID,
-                          aAttribute, aModType);
+         AttributeChanged(aNameSpaceID, aAttribute, aModType);
 }
 
 NS_IMETHODIMP
-nsMathMLmunderoverFrame::UpdatePresentationData(nsPresContext* aPresContext,
-                                                PRInt32         aScriptLevelIncrement,
+nsMathMLmunderoverFrame::UpdatePresentationData(PRInt32         aScriptLevelIncrement,
                                                 PRUint32        aFlagsValues,
                                                 PRUint32        aFlagsToUpdate)
 {
-  nsMathMLContainerFrame::UpdatePresentationData(aPresContext,
-    aScriptLevelIncrement, aFlagsValues, aFlagsToUpdate);
+  nsMathMLContainerFrame::UpdatePresentationData(aScriptLevelIncrement,
+    aFlagsValues, aFlagsToUpdate);
   // disable the stretch-all flag if we are going to act like a subscript-superscript pair
   if ( NS_MATHML_EMBELLISH_IS_MOVABLELIMITS(mEmbellishData.flags) &&
       !NS_MATHML_IS_DISPLAYSTYLE(mPresentationData.flags)) {
@@ -118,8 +101,7 @@ nsMathMLmunderoverFrame::UpdatePresentationData(nsPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsMathMLmunderoverFrame::UpdatePresentationDataFromChildAt(nsPresContext* aPresContext,
-                                                           PRInt32         aFirstIndex,
+nsMathMLmunderoverFrame::UpdatePresentationDataFromChildAt(PRInt32         aFirstIndex,
                                                            PRInt32         aLastIndex,
                                                            PRInt32         aScriptLevelIncrement,
                                                            PRUint32        aFlagsValues,
@@ -150,7 +132,7 @@ nsMathMLmunderoverFrame::UpdatePresentationDataFromChildAt(nsPresContext* aPresC
         aFlagsToUpdate &= ~NS_MATHML_DISPLAYSTYLE;
         aFlagsValues &= ~NS_MATHML_DISPLAYSTYLE;
       }
-      PropagatePresentationDataFor(aPresContext, childFrame,
+      PropagatePresentationDataFor(childFrame,
         aScriptLevelIncrement, aFlagsValues, aFlagsToUpdate);
     }
     index++;
@@ -163,11 +145,10 @@ nsMathMLmunderoverFrame::UpdatePresentationDataFromChildAt(nsPresContext* aPresC
 }
 
 NS_IMETHODIMP
-nsMathMLmunderoverFrame::InheritAutomaticData(nsPresContext* aPresContext,
-                                              nsIFrame*       aParent)
+nsMathMLmunderoverFrame::InheritAutomaticData(nsIFrame* aParent)
 {
   // let the base class get the default from our parent
-  nsMathMLContainerFrame::InheritAutomaticData(aPresContext, aParent);
+  nsMathMLContainerFrame::InheritAutomaticData(aParent);
 
   mPresentationData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
 
@@ -175,7 +156,7 @@ nsMathMLmunderoverFrame::InheritAutomaticData(nsPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsMathMLmunderoverFrame::TransmitAutomaticData(nsPresContext* aPresContext)
+nsMathMLmunderoverFrame::TransmitAutomaticData()
 {
   // At this stage, all our children are in sync and we can fully
   // resolve our own mEmbellishData struct
@@ -199,17 +180,12 @@ nsMathMLmunderoverFrame::TransmitAutomaticData(nsPresContext* aPresContext)
     underscriptFrame = baseFrame->GetNextSibling();
   if (underscriptFrame)
     overscriptFrame = underscriptFrame->GetNextSibling();
-  if (!baseFrame || !underscriptFrame || !overscriptFrame)
-    return NS_OK; // a visual error indicator will be reported later during layout
 
   // if our base is an embellished operator, let its state bubble to us (in particular,
   // this is where we get the flag for NS_MATHML_EMBELLISH_MOVABLELIMITS). Our flags
   // are reset to the default values of false if the base frame isn't embellished.
+  mPresentationData.baseFrame = baseFrame;
   GetEmbellishDataFrom(baseFrame, mEmbellishData);
-  if (NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags))
-    mEmbellishData.nextFrame = baseFrame;
-
-  nsAutoString value;
 
   // The default value of accentunder is false, unless the underscript is embellished
   // and its core <mo> is an accent
@@ -220,13 +196,14 @@ nsMathMLmunderoverFrame::TransmitAutomaticData(nsPresContext* aPresContext)
   else
     mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTUNDER;
 
+  static nsIContent::AttrValuesArray strings[] =
+    {&nsMathMLAtoms::_true, &nsMathMLAtoms::_false, nsnull};
+
   // if we have an accentunder attribute, it overrides what the underscript said
-  if (NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, 
-                   nsMathMLAtoms::accentunder_, value)) {
-    if (value.EqualsLiteral("true"))
-      mEmbellishData.flags |= NS_MATHML_EMBELLISH_ACCENTUNDER;
-    else if (value.EqualsLiteral("false")) 
-      mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTUNDER;
+  switch (mContent->FindAttrValueIn(kNameSpaceID_None, nsMathMLAtoms::accentunder_,
+                                    strings, eCaseMatters)) {
+    case 0: mEmbellishData.flags |= NS_MATHML_EMBELLISH_ACCENTUNDER; break;
+    case 1: mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTUNDER; break;
   }
 
   // The default value of accent is false, unless the overscript is embellished
@@ -238,12 +215,10 @@ nsMathMLmunderoverFrame::TransmitAutomaticData(nsPresContext* aPresContext)
     mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTOVER;
 
   // if we have an accent attribute, it overrides what the overscript said
-  if (NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, 
-                   nsMathMLAtoms::accent_, value)) {
-    if (value.EqualsLiteral("true"))
-      mEmbellishData.flags |= NS_MATHML_EMBELLISH_ACCENTOVER;
-    else if (value.EqualsLiteral("false")) 
-      mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTOVER;
+  switch (mContent->FindAttrValueIn(kNameSpaceID_None, nsMathMLAtoms::accent_,
+                                    strings, eCaseMatters)) {
+    case 0: mEmbellishData.flags |= NS_MATHML_EMBELLISH_ACCENTOVER; break;
+    case 1: mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTOVER; break;
   }
 
   // disable the stretch-all flag if we are going to act like a superscript
@@ -271,7 +246,7 @@ nsMathMLmunderoverFrame::TransmitAutomaticData(nsPresContext* aPresContext)
     ? 0 : 1;
   PRUint32 compress = NS_MATHML_EMBELLISH_IS_ACCENTOVER(mEmbellishData.flags)
     ? NS_MATHML_COMPRESSED : 0;
-  PropagatePresentationDataFor(aPresContext, overscriptFrame, increment,
+  PropagatePresentationDataFor(overscriptFrame, increment,
     ~NS_MATHML_DISPLAYSTYLE | compress,
      NS_MATHML_DISPLAYSTYLE | compress);
 
@@ -281,7 +256,7 @@ nsMathMLmunderoverFrame::TransmitAutomaticData(nsPresContext* aPresContext)
   */
   increment = NS_MATHML_EMBELLISH_IS_ACCENTUNDER(mEmbellishData.flags)
     ? 0 : 1;
-  PropagatePresentationDataFor(aPresContext, underscriptFrame, increment,
+  PropagatePresentationDataFor(underscriptFrame, increment,
     ~NS_MATHML_DISPLAYSTYLE | NS_MATHML_COMPRESSED,
      NS_MATHML_DISPLAYSTYLE | NS_MATHML_COMPRESSED);
 
@@ -308,15 +283,14 @@ i.e.,:
 */
 
 NS_IMETHODIMP
-nsMathMLmunderoverFrame::Place(nsPresContext*      aPresContext,
-                               nsIRenderingContext& aRenderingContext,
+nsMathMLmunderoverFrame::Place(nsIRenderingContext& aRenderingContext,
                                PRBool               aPlaceOrigin,
                                nsHTMLReflowMetrics& aDesiredSize)
 {
   if ( NS_MATHML_EMBELLISH_IS_MOVABLELIMITS(mEmbellishData.flags) &&
       !NS_MATHML_IS_DISPLAYSTYLE(mPresentationData.flags)) {
     // place like sub-superscript pair
-    return nsMathMLmsubsupFrame::PlaceSubSupScript(aPresContext,
+    return nsMathMLmsubsupFrame::PlaceSubSupScript(GetPresContext(),
                                                    aRenderingContext,
                                                    aPlaceOrigin,
                                                    aDesiredSize,
@@ -340,13 +314,13 @@ nsMathMLmunderoverFrame::Place(nsPresContext*      aPresContext,
   if (!baseFrame || !underFrame || !overFrame || overFrame->GetNextSibling()) {
     // report an error, encourage people to get their markups in order
     NS_WARNING("invalid markup");
-    return ReflowError(aPresContext, aRenderingContext, aDesiredSize);
+    return ReflowError(aRenderingContext, aDesiredSize);
   }
   GetReflowAndBoundingMetricsFor(baseFrame, baseSize, bmBase);
   GetReflowAndBoundingMetricsFor(underFrame, underSize, bmUnder);
   GetReflowAndBoundingMetricsFor(overFrame, overSize, bmOver);
 
-  nscoord onePixel = aPresContext->IntScaledPixelsToTwips(1);
+  nscoord onePixel = GetPresContext()->IntScaledPixelsToTwips(1);
 
   ////////////////////
   // Place Children
@@ -504,13 +478,13 @@ nsMathMLmunderoverFrame::Place(nsPresContext*      aPresContext,
     nscoord dy;
     // place overscript
     dy = aDesiredSize.ascent - mBoundingMetrics.ascent + bmOver.ascent - overSize.ascent;
-    FinishReflowChild (overFrame, aPresContext, nsnull, overSize, dxOver, dy, 0);
+    FinishReflowChild (overFrame, GetPresContext(), nsnull, overSize, dxOver, dy, 0);
     // place base
     dy = aDesiredSize.ascent - baseSize.ascent;
-    FinishReflowChild (baseFrame, aPresContext, nsnull, baseSize, dxBase, dy, 0);
+    FinishReflowChild (baseFrame, GetPresContext(), nsnull, baseSize, dxBase, dy, 0);
     // place underscript
     dy = aDesiredSize.ascent + mBoundingMetrics.descent - bmUnder.descent - underSize.ascent;
-    FinishReflowChild (underFrame, aPresContext, nsnull, underSize, dxUnder, dy, 0);
+    FinishReflowChild (underFrame, GetPresContext(), nsnull, underSize, dxUnder, dy, 0);
   }
   return NS_OK;
 }
