@@ -212,7 +212,7 @@ nsGrid::RebuildIfNeeded()
   //   </row>
   // </rows>
   //
-  // computedColumnCount = 2 // for the 2 buttons in the the row tag
+  // computedColumnCount = 2 // for the 2 buttons in the row tag
   // computedRowCount = 0 // there is nothing in the  column tag
   // mColumnCount = 1 // one column defined
   // mRowCount = 1 // one row defined
@@ -220,13 +220,17 @@ nsGrid::RebuildIfNeeded()
   // So in this case we need to make 1 extra column.
   //
 
+  // Make sure to update mExtraColumnCount no matter what, since it might
+  // happen that we now have as many columns as are defined, and we wouldn't
+  // want to have a positive mExtraColumnCount hanging about in that case!
+  mExtraColumnCount = computedColumnCount - columnCount;
   if (computedColumnCount > columnCount) {
-     mExtraColumnCount = computedColumnCount - columnCount;
      columnCount = computedColumnCount;
   }
 
+  // Same for rows.
+  mExtraRowCount = computedRowCount - rowCount;
   if (computedRowCount > rowCount) {
-     mExtraRowCount    = computedRowCount - rowCount;
      rowCount = computedRowCount;
   }
 
@@ -536,10 +540,10 @@ nsGrid::GetRowAt(PRInt32 aIndex, PRBool aIsHorizontal)
   RebuildIfNeeded();
 
   if (aIsHorizontal) {
-    NS_ASSERTION(aIndex < mRowCount || aIndex >= 0, "Index out of range");
+    NS_ASSERTION(aIndex < mRowCount && aIndex >= 0, "Index out of range");
     return &mRows[aIndex];
   } else {
-    NS_ASSERTION(aIndex < mColumnCount || aIndex >= 0, "Index out of range");
+    NS_ASSERTION(aIndex < mColumnCount && aIndex >= 0, "Index out of range");
     return &mColumns[aIndex];
   }
 }
@@ -741,7 +745,7 @@ nsGrid::GetRowOffsets(nsBoxLayoutState& aState, PRInt32 aIndex, nscoord& aTop, n
   nsMargin totalBorderPadding(0,0,0,0);
   nsMargin totalMargin(0,0,0,0);
 
-  // if there is a box and its not bogus take its
+  // if there is a box and it's not bogus take its
   // borders padding and insets into account
   if (box && !row->mIsBogus)
   {
@@ -793,10 +797,10 @@ nsGrid::GetRowOffsets(nsBoxLayoutState& aState, PRInt32 aIndex, nscoord& aTop, n
   // of each columns. 
 
   // If we are the first row then get the largest top border/padding in 
-  // our columns. If thats larger than the rows top border/padding use it.
+  // our columns. If that's larger than the rows top border/padding use it.
 
   // If we are the last row then get the largest bottom border/padding in 
-  // our columns. If thats larger than the rows bottom border/padding use it.
+  // our columns. If that's larger than the rows bottom border/padding use it.
   PRInt32 firstIndex = 0;
   PRInt32 lastIndex = 0;
   nsGridRow* firstRow = nsnull;
@@ -1210,7 +1214,7 @@ nsGrid::IsGrid(nsIBox* aBox)
 }
 
 /**
- * This get the flexibilty of the row at aIndex. Its not trivial. There are a few
+ * This get the flexibilty of the row at aIndex. It's not trivial. There are a few
  * things we need to look at. Specifically we need to see if any <rows> or <columns>
  * tags are around us. Their flexibilty will affect ours.
  */
@@ -1462,7 +1466,7 @@ nsGrid::GetScrolledBox(nsIBox* aChild)
       if (scrollFrame) {
          nsIFrame* scrolledFrame = scrollFrame->GetScrolledFrame();
          NS_ASSERTION(scrolledFrame,"Error no scroll frame!!");
-         return scrolledFrame->IsBoxFrame() ? scrolledFrame : nsnull;
+         return scrolledFrame;
       }
 
       return aChild;
@@ -1475,6 +1479,9 @@ nsGrid::GetScrolledBox(nsIBox* aChild)
 nsIBox*
 nsGrid::GetScrollBox(nsIBox* aChild)
 {
+  if (!aChild)
+    return nsnull;
+
   // get parent
   nsIBox* parent = nsnull;
   nsCOMPtr<nsIBoxLayout> layout;
@@ -1483,8 +1490,8 @@ nsGrid::GetScrollBox(nsIBox* aChild)
   aChild->GetParentBox(&parent);
 
   // walk up until we find a scrollframe or a part
-  // if its a scrollframe return it.
-  // if its a parent then the child passed does not
+  // if it's a scrollframe return it.
+  // if it's a parent then the child passed does not
   // have a scroll frame immediately wrapped around it.
   while (parent) {
     nsCOMPtr<nsIScrollableFrame> scrollFrame = do_QueryInterface(parent);
