@@ -134,7 +134,7 @@ nsBox::DumpBox(FILE* aFile)
 {
   nsAutoString s;
   ListBox(s);
-  fprintf(aFile, "%s", NS_LossyConvertUCS2toASCII(s).get());
+  fprintf(aFile, "%s", NS_LossyConvertUTF16toASCII(s).get());
   return NS_OK;
 }
 
@@ -178,7 +178,7 @@ nsBox::BeginLayout(nsBoxLayoutState& aState)
 
   nsBoxAddIndents();
 
-  nsAutoString reason;
+  nsCAutoString reason;
   switch(aState.LayoutReason())
     {
     case nsBoxLayoutState::Dirty:
@@ -192,9 +192,7 @@ nsBox::BeginLayout(nsBoxLayoutState& aState)
       break;
     }
 
-  char ch[100];
-  reason.ToCString(ch,100);
-  printf("%s Layout: ", ch);
+  printf("%s Layout: ", reason.get());
   DumpBox(stdout);
   printf("\n");
   gIndent++;
@@ -240,8 +238,6 @@ void Coelesced()
 PRBool nsBox::gGotTheme = PR_FALSE;
 nsITheme* nsBox::gTheme = nsnull;
 
-MOZ_DECL_CTOR_COUNTER(nsBox)
-
 nsBox::nsBox()
 {
   MOZ_COUNT_CTOR(nsBox);
@@ -331,8 +327,8 @@ nsBox::MarkStyleChange(nsBoxLayoutState& aState)
      return parent->RelayoutDirtyChild(aState, this);
   else {
     /*
-    nsFrame::CreateAndPostReflowCommand(aState.PresShell(), this, 
-      nsHTMLReflowCommand::StyleChange, nsnull, nsnull, nsnull);
+    aState.PresShell()->AppendReflowCommand(this, eReflowType_StyleChanged,
+                                            nsnull);
     return NS_OK;
     */
     return GetParent()->ReflowDirtyChild(aState.PresShell(), this);
@@ -396,9 +392,8 @@ nsBox::RelayoutDirtyChild(nsBoxLayoutState& aState, nsIBox* aChild)
       AddStateBits(NS_FRAME_HAS_DIRTY_CHILDREN);
 
       if (GetStateBits() & NS_FRAME_REFLOW_ROOT) {
-        nsFrame::CreateAndPostReflowCommand(aState.PresShell(), this,
-                                            eReflowType_ReflowDirty,
-                                            nsnull, nsnull, nsnull);
+        aState.PresShell()->AppendReflowCommand(this, eReflowType_ReflowDirty,
+                                                nsnull);
         return NS_OK;
       }
 
@@ -480,8 +475,6 @@ nsBox::SetBounds(nsBoxLayoutState& aState, const nsRect& aRect, PRBool aRemoveOv
 
     nsRect rect(mRect);
 
-    nsPresContext* presContext = aState.PresContext();
-
     PRUint32 flags = 0;
     GetLayoutFlags(flags);
 
@@ -505,9 +498,9 @@ nsBox::SetBounds(nsBoxLayoutState& aState, const nsRect& aRect, PRBool aRemoveOv
 
     if (!(flags & NS_FRAME_NO_MOVE_VIEW))
     {
-      nsContainerFrame::PositionFrameView(presContext, this);
+      nsContainerFrame::PositionFrameView(this);
       if ((rect.x != aRect.x) || (rect.y != aRect.y))
-        nsContainerFrame::PositionChildViews(presContext, this);
+        nsContainerFrame::PositionChildViews(this);
     }
   
 
@@ -574,7 +567,7 @@ nsBox::GetBorder(nsMargin& aMargin)
     }
   }
 
-  GetStyleBorder()->GetBorder(aMargin);
+  aMargin = GetStyleBorder()->GetBorder();
 
   return NS_OK;
 }
@@ -807,6 +800,16 @@ nsIFrame::Layout(nsBoxLayoutState& aState)
   return NS_OK;
 }
 
+PRBool
+nsBox::DoesClipChildren()
+{
+  const nsStyleDisplay* display = GetStyleDisplay();
+  NS_ASSERTION((display->mOverflowY == NS_STYLE_OVERFLOW_CLIP) ==
+               (display->mOverflowX == NS_STYLE_OVERFLOW_CLIP),
+               "If one overflow is clip, the other should be too");
+  return display->mOverflowX == NS_STYLE_OVERFLOW_CLIP;
+}
+
 nsresult
 nsBox::SyncLayout(nsBoxLayoutState& aState)
 {
@@ -840,11 +843,10 @@ nsBox::SyncLayout(nsBoxLayoutState& aState)
   nsRect rect(nsPoint(0, 0), GetSize());
 
   if (ComputesOwnOverflowArea()) {
-    if (GetStateBits() & NS_FRAME_OUTSIDE_CHILDREN) {
-      nsRect* overflow = GetOverflowAreaProperty();
-      NS_ASSERTION(overflow, "should have overflow area property");
+    nsRect* overflow = GetOverflowAreaProperty();
+    if (overflow)
       rect = *overflow;
-    }
+
   } else {
     if (!DoesClipChildren()) {
       // See if our child frames caused us to overflow after being laid
@@ -854,20 +856,26 @@ nsBox::SyncLayout(nsBoxLayoutState& aState)
       // frames in HTML inside XUL).
       nsIFrame* box;
       GetChildBox(&box);
-      while (box)
-        {
-          nsRect bounds;
-          if (box->GetStateBits() & NS_FRAME_OUTSIDE_CHILDREN) {
-            nsRect* overflowArea = box->GetOverflowAreaProperty();
-            NS_ASSERTION(overflowArea, "Should have created property for overflowing frame");
-            bounds = *overflowArea + box->GetPosition();
-          } else {
-            bounds = box->GetRect();
-          }
-          rect.UnionRect(rect, bounds);
+      while (box) {
+        nsRect* overflowArea = box->GetOverflowAreaProperty();
+        nsRect bounds = overflowArea ? *overflowArea + box->GetPosition()
+                                     : box->GetRect();
+        rect.UnionRect(rect, bounds);
 
-          box->GetNextBox(&box);
+        box->GetNextBox(&box);
+      }
+    }
+
+    const nsStyleDisplay* disp = GetStyleDisplay();
+    if (disp->mAppearance && gTheme) {
+      // Add in the theme's desired overflow
+      if (gTheme->ThemeSupportsWidget(presContext, this, disp->mAppearance)) {
+        nsRect r;
+        if (gTheme->GetWidgetOverflow(presContext->DeviceContext(), this,
+                                      disp->mAppearance, &r)) {
+          rect.UnionRect(rect, r);
         }
+      }
     }
 
     FinishAndStoreOverflow(&rect, GetSize());
@@ -940,14 +948,14 @@ nsIBox::AddCSSPrefSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
     // ignore 'height' and 'width' attributes if the actual element is not XUL
     // For example, we might be magic XUL frames whose primary content is an HTML
     // <select>
-    if (content && content->IsContentOfType(nsIContent::eXUL)) {
+    if (content && content->IsNodeOfType(nsINode::eXUL)) {
         nsPresContext* presContext = aState.PresContext();
 
         nsAutoString value;
         PRInt32 error;
 
-        if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::width, value))
-        {
+        content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::width, value);
+        if (!value.IsEmpty()) {
             value.Trim("%");
 
             aSize.width =
@@ -955,8 +963,8 @@ nsIBox::AddCSSPrefSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
             widthSet = PR_TRUE;
         }
 
-        if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::height, value))
-        {
+        content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::height, value);
+        if (!value.IsEmpty()) {
             value.Trim("%");
 
             aSize.height = presContext->IntScaledPixelsToTwips(value.ToInteger(&error));
@@ -1037,7 +1045,8 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
         nsAutoString value;
         PRInt32 error;
 
-        if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, nsXULAtoms::minwidth, value))
+        content->GetAttr(kNameSpaceID_None, nsXULAtoms::minwidth, value);
+        if (!value.IsEmpty())
         {
             value.Trim("%");
 
@@ -1048,7 +1057,8 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
             widthSet = PR_TRUE;
         }
 
-        if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, nsXULAtoms::minheight, value))
+        content->GetAttr(kNameSpaceID_None, nsXULAtoms::minheight, value);
+        if (!value.IsEmpty())
         {
             value.Trim("%");
 
@@ -1094,8 +1104,8 @@ nsIBox::AddCSSMaxSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
         nsAutoString value;
         PRInt32 error;
 
-        if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, nsXULAtoms::maxwidth, value))
-        {
+        content->GetAttr(kNameSpaceID_None, nsXULAtoms::maxwidth, value);
+        if (!value.IsEmpty()) {
             value.Trim("%");
 
             nscoord val =
@@ -1104,8 +1114,8 @@ nsIBox::AddCSSMaxSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
             widthSet = PR_TRUE;
         }
 
-        if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, nsXULAtoms::maxheight, value))
-        {
+        content->GetAttr(kNameSpaceID_None, nsXULAtoms::maxheight, value);
+        if (!value.IsEmpty()) {
             value.Trim("%");
 
             nscoord val =
@@ -1130,8 +1140,8 @@ nsIBox::AddCSSFlex(nsBoxLayoutState& aState, nsIBox* aBox, nscoord& aFlex)
         PRInt32 error;
         nsAutoString value;
 
-        if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, nsXULAtoms::flex, value))
-        {
+        content->GetAttr(kNameSpaceID_None, nsXULAtoms::flex, value);
+        if (!value.IsEmpty()) {
             value.Trim("%");
             aFlex = value.ToInteger(&error);
             flexSet = PR_TRUE;
@@ -1147,6 +1157,11 @@ nsIBox::AddCSSFlex(nsBoxLayoutState& aState, nsIBox* aBox, nscoord& aFlex)
         }
     }
 
+    if (aFlex < 0)
+      aFlex = 0;
+    if (aFlex >= nscoord_MAX)
+      aFlex = nscoord_MAX - 1;
+
     return flexSet;
 }
 
@@ -1169,7 +1184,8 @@ nsIBox::AddCSSOrdinal(nsBoxLayoutState& aState, nsIBox* aBox, PRUint32& aOrdinal
     PRInt32 error;
     nsAutoString value;
 
-    if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, nsXULAtoms::ordinal, value)) {
+    content->GetAttr(kNameSpaceID_None, nsXULAtoms::ordinal, value);
+    if (!value.IsEmpty()) {
       aOrdinal = value.ToInteger(&error);
       ordinalSet = PR_TRUE;
     }
@@ -1277,7 +1293,8 @@ NS_IMETHODIMP
 nsBox::GetDebugBoxAt( const nsPoint& aPoint,
                       nsIBox**     aBox)
 {
-  if (!mRect.Contains(aPoint))
+  nsRect thisRect(nsPoint(0,0), GetSize());
+  if (!thisRect.Contains(aPoint))
     return NS_ERROR_FAILURE;
 
   nsIBox* child = nsnull;
@@ -1285,10 +1302,8 @@ nsBox::GetDebugBoxAt( const nsPoint& aPoint,
   GetChildBox(&child);
 
   *aBox = nsnull;
-  nsPoint tmp;
-  tmp.MoveTo(aPoint.x - mRect.x, aPoint.y - mRect.y);
   while (nsnull != child) {
-    nsresult rv = child->GetDebugBoxAt(tmp, &hit);
+    nsresult rv = child->GetDebugBoxAt(aPoint - child->GetOffsetTo(this), &hit);
 
     if (NS_SUCCEEDED(rv) && hit) {
       *aBox = hit;
@@ -1305,7 +1320,7 @@ nsBox::GetDebugBoxAt( const nsPoint& aPoint,
   nsMargin m;
   GetBorderAndPadding(m);
 
-  nsRect rect(mRect);
+  nsRect rect(thisRect);
   rect.Deflate(m);
   if (rect.Contains(aPoint)) {
     GetInset(m);
