@@ -51,7 +51,6 @@
 #include "nsStyleConsts.h"
 #include "nsCOMPtr.h"
 #include "nsPresContext.h"
-#include "nsButtonFrameRenderer.h"
 #include "nsBoxLayoutState.h"
 
 #include "nsHTMLParts.h"
@@ -84,79 +83,61 @@
 #include "nsIDOMHTMLMapElement.h"
 #include "nsBoxLayoutState.h"
 #include "nsIDOMDocument.h"
-#include "nsIEventQueueService.h"
 #include "nsTransform2D.h"
 #include "nsITheme.h"
 
 #include "nsIServiceManager.h"
 #include "nsIURI.h"
 #include "nsNetUtil.h"
+#include "nsThreadUtils.h"
 #include "nsGUIEvent.h"
+#include "nsEventDispatcher.h"
+#include "nsDisplayList.h"
 
-#include "nsFormControlHelper.h"
 #include "nsContentUtils.h"
 
 #define ONLOAD_CALLED_TOO_EARLY 1
 
-static void PR_CALLBACK
-HandleImagePLEvent(nsIContent *aContent, PRUint32 aMessage, PRUint32 aFlags)
+class nsImageBoxFrameEvent : public nsRunnable
 {
-  if (!aContent) {
-    NS_ERROR("null node passed to HandleImagePLEvent!");
+public:
+  nsImageBoxFrameEvent(nsIContent *content, PRUint32 message)
+    : mContent(content), mMessage(message) {}
 
-    return;
-  }
+  NS_IMETHOD Run();
 
-  nsIDocument* doc = aContent->GetOwnerDoc();
+private:
+  nsCOMPtr<nsIContent> mContent;
+  PRUint32 mMessage;
+};
 
+NS_IMETHODIMP
+nsImageBoxFrameEvent::Run()
+{
+  nsIDocument* doc = mContent->GetOwnerDoc();
   if (!doc) {
-    return;
+    return NS_OK;
   }
 
   nsIPresShell *pres_shell = doc->GetShellAt(0);
   if (!pres_shell) {
-    return;
+    return NS_OK;
   }
 
   nsCOMPtr<nsPresContext> pres_context = pres_shell->GetPresContext();
   if (!pres_context) {
-    return;
+    return NS_OK;
   }
 
   nsEventStatus status = nsEventStatus_eIgnore;
-  nsEvent event(aMessage);
+  nsEvent event(PR_TRUE, mMessage);
 
-  aContent->HandleDOMEvent(pres_context, &event, nsnull, aFlags, &status);
+  event.flags |= NS_EVENT_FLAG_CANT_BUBBLE;
+  nsEventDispatcher::Dispatch(mContent, pres_context, &event, nsnull, &status);
+  return NS_OK;
 }
 
-static void PR_CALLBACK
-HandleImageOnloadPLEvent(PLEvent *aEvent)
-{
-  nsIContent *content = (nsIContent *)PL_GetEventOwner(aEvent);
-
-  HandleImagePLEvent(content, NS_IMAGE_LOAD,
-                     NS_EVENT_FLAG_INIT | NS_EVENT_FLAG_CANT_BUBBLE);
-
-  NS_RELEASE(content);
-}
-
-static void PR_CALLBACK
-HandleImageOnerrorPLEvent(PLEvent *aEvent)
-{
-  nsIContent *content = (nsIContent *)PL_GetEventOwner(aEvent);
-
-  HandleImagePLEvent(content, NS_IMAGE_ERROR, NS_EVENT_FLAG_INIT);
-
-  NS_RELEASE(content);
-}
-
-static void PR_CALLBACK
-DestroyImagePLEvent(PLEvent* aEvent)
-{
-  delete aEvent;
-}
-
-// Fire off a PLEvent that'll asynchronously call the image elements
+// Fire off an event that'll asynchronously call the image elements
 // onload handler once handled. This is needed since the image library
 // can't decide if it wants to call it's observer methods
 // synchronously or asynchronously. If an image is loaded from the
@@ -165,99 +146,38 @@ DestroyImagePLEvent(PLEvent* aEvent)
 // asynchronously.
 
 void
-FireDOMEvent(nsIContent* aContent, PRUint32 aMessage)
+FireImageDOMEvent(nsIContent* aContent, PRUint32 aMessage)
 {
-  static NS_DEFINE_CID(kEventQueueServiceCID,   NS_EVENTQUEUESERVICE_CID);
+  NS_ASSERTION(aMessage == NS_LOAD || aMessage == NS_LOAD_ERROR,
+               "invalid message");
 
-  nsCOMPtr<nsIEventQueueService> event_service =
-    do_GetService(kEventQueueServiceCID);
-
-  if (!event_service) {
-    NS_WARNING("Failed to get event queue service");
-
-    return;
-  }
-
-  nsCOMPtr<nsIEventQueue> event_queue;
-
-  event_service->GetThreadEventQueue(NS_CURRENT_THREAD,
-                                     getter_AddRefs(event_queue));
-
-  if (!event_queue) {
-    NS_WARNING("Failed to get event queue from service");
-
-    return;
-  }
-
-  PLEvent *event = new PLEvent;
-
-  if (!event) {
-    // Out of memory, but none of our callers care, so just warn and
-    // don't fire the event
-
-    NS_WARNING("Out of memory?");
-
-    return;
-  }
-
-  PLHandleEventProc f;
-
-  switch (aMessage) {
-  case NS_IMAGE_LOAD :
-    f = (PLHandleEventProc)::HandleImageOnloadPLEvent;
-
-    break;
-  case NS_IMAGE_ERROR :
-    f = (PLHandleEventProc)::HandleImageOnerrorPLEvent;
-
-    break;
-  default:
-    NS_WARNING("Huh, I don't know how to fire this type of event?!");
-
-    return;
-  }
-
-  PL_InitEvent(event, aContent, f, (PLDestroyEventProc)::DestroyImagePLEvent);
-
-  // The event owns the content pointer now.
-  NS_ADDREF(aContent);
-
-  event_queue->PostEvent(event);
+  nsCOMPtr<nsIRunnable> event = new nsImageBoxFrameEvent(aContent, aMessage);
+  if (NS_FAILED(NS_DispatchToCurrentThread(event)))
+    NS_WARNING("failed to dispatch image event");
 }
 
 //
 // NS_NewImageBoxFrame
 //
-// Creates a new image frame and returns it in |aNewFrame|
+// Creates a new image frame and returns it
 //
-nsresult
-NS_NewImageBoxFrame ( nsIPresShell* aPresShell, nsIFrame** aNewFrame )
+nsIFrame*
+NS_NewImageBoxFrame (nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
-  NS_PRECONDITION(aNewFrame, "null OUT ptr");
-  if (nsnull == aNewFrame) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  nsImageBoxFrame* it = new (aPresShell) nsImageBoxFrame (aPresShell);
-  if (nsnull == it)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  *aNewFrame = it;
-  return NS_OK;
-  
+  return new (aPresShell) nsImageBoxFrame (aPresShell, aContext);
 } // NS_NewTitledButtonFrame
 
 NS_IMETHODIMP
-nsImageBoxFrame::AttributeChanged(nsPresContext* aPresContext,
-                               nsIContent* aChild,
-                               PRInt32 aNameSpaceID,
-                               nsIAtom* aAttribute,
-                               PRInt32 aModType)
+nsImageBoxFrame::AttributeChanged(PRInt32 aNameSpaceID,
+                                  nsIAtom* aAttribute,
+                                  PRInt32 aModType)
 {
-  nsresult rv = nsLeafBoxFrame::AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType);
+  nsresult rv = nsLeafBoxFrame::AttributeChanged(aNameSpaceID, aAttribute,
+                                                 aModType);
 
   if (aAttribute == nsHTMLAtoms::src) {
     UpdateImage();
-    nsBoxLayoutState state(aPresContext);
+    nsBoxLayoutState state(GetPresContext());
     MarkDirty(state);
   }
   else if (aAttribute == nsXULAtoms::validate)
@@ -266,8 +186,8 @@ nsImageBoxFrame::AttributeChanged(nsPresContext* aPresContext,
   return rv;
 }
 
-nsImageBoxFrame::nsImageBoxFrame(nsIPresShell* aShell) :
-  nsLeafBoxFrame(aShell),
+nsImageBoxFrame::nsImageBoxFrame(nsIPresShell* aShell, nsStyleContext* aContext):
+  nsLeafBoxFrame(aShell, aContext),
   mUseSrcAttr(PR_FALSE),
   mSuppressStyleCheck(PR_FALSE),
   mIntrinsicSize(0,0),
@@ -288,8 +208,8 @@ nsImageBoxFrame::NeedsRecalc()
   return NS_OK;
 }
 
-NS_METHOD
-nsImageBoxFrame::Destroy(nsPresContext* aPresContext)
+void
+nsImageBoxFrame::Destroy()
 {
   // Release image loader first so that it's refcnt can go to zero
   if (mImageRequest)
@@ -298,16 +218,14 @@ nsImageBoxFrame::Destroy(nsPresContext* aPresContext)
   if (mListener)
     NS_REINTERPRET_CAST(nsImageBoxListener*, mListener.get())->SetFrame(nsnull); // set the frame to null so we don't send messages to a dead object.
 
-  return nsLeafBoxFrame::Destroy(aPresContext);
+  nsLeafBoxFrame::Destroy();
 }
 
 
 NS_IMETHODIMP
-nsImageBoxFrame::Init(nsPresContext*  aPresContext,
-                          nsIContent*      aContent,
-                          nsIFrame*        aParent,
-                          nsStyleContext*  aContext,
-                          nsIFrame*        aPrevInFlow)
+nsImageBoxFrame::Init(nsIContent*      aContent,
+                      nsIFrame*        aParent,
+                      nsIFrame*        aPrevInFlow)
 {
   if (!mListener) {
     nsImageBoxListener *listener;
@@ -319,7 +237,7 @@ nsImageBoxFrame::Init(nsPresContext*  aPresContext,
   }
 
   mSuppressStyleCheck = PR_TRUE;
-  nsresult  rv = nsLeafBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
+  nsresult rv = nsLeafBoxFrame::Init(aContent, aParent, aPrevInFlow);
   mSuppressStyleCheck = PR_FALSE;
 
   UpdateLoadFlags();
@@ -334,7 +252,6 @@ nsImageBoxFrame::UpdateImage()
   if (mImageRequest) {
     mImageRequest->Cancel(NS_ERROR_FAILURE);
     mImageRequest = nsnull;
-    mIntrinsicSize.SizeTo(0, 0);
   }
 
   // get the new image src
@@ -372,56 +289,86 @@ nsImageBoxFrame::UpdateImage()
       }
     }
   }
+
+  if (!mImageRequest) {
+    // We have no image, so size to 0
+    mIntrinsicSize.SizeTo(0, 0);
+  }
 }
 
 void
 nsImageBoxFrame::UpdateLoadFlags()
 {
-  nsAutoString loadPolicy;
-  mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::validate, loadPolicy);
-  if (loadPolicy.EqualsLiteral("always"))
-    mLoadFlags = nsIRequest::VALIDATE_ALWAYS;
-  else if (loadPolicy.EqualsLiteral("never"))
-    mLoadFlags = nsIRequest::VALIDATE_NEVER|nsIRequest::LOAD_FROM_CACHE; 
-  else
-    mLoadFlags = nsIRequest::LOAD_NORMAL;
+  static nsIContent::AttrValuesArray strings[] =
+    {&nsXULAtoms::always, &nsXULAtoms::never, nsnull};
+  switch (mContent->FindAttrValueIn(kNameSpaceID_None, nsXULAtoms::validate,
+                                    strings, eCaseMatters)) {
+    case 0:
+      mLoadFlags = nsIRequest::VALIDATE_ALWAYS;
+      break;
+    case 1:
+      mLoadFlags = nsIRequest::VALIDATE_NEVER|nsIRequest::LOAD_FROM_CACHE;
+      break;
+    default:
+      mLoadFlags = nsIRequest::LOAD_NORMAL;
+      break;
+  }
+}
+
+class nsDisplayXULImage : public nsDisplayItem {
+public:
+  nsDisplayXULImage(nsImageBoxFrame* aFrame) : nsDisplayItem(aFrame) {
+    MOZ_COUNT_CTOR(nsDisplayXULImage);
+  }
+#ifdef NS_BUILD_REFCNT_LOGGING
+  virtual ~nsDisplayXULImage() {
+    MOZ_COUNT_DTOR(nsDisplayXULImage);
+  }
+#endif
+
+  // Doesn't handle HitTest because nsLeafBoxFrame already creates an
+  // event receiver for us
+  virtual void Paint(nsDisplayListBuilder* aBuilder, nsIRenderingContext* aCtx,
+     const nsRect& aDirtyRect);
+  NS_DISPLAY_DECL_NAME("XULImage")
+};
+
+void nsDisplayXULImage::Paint(nsDisplayListBuilder* aBuilder,
+     nsIRenderingContext* aCtx, const nsRect& aDirtyRect)
+{
+  NS_STATIC_CAST(nsImageBoxFrame*, mFrame)->
+    PaintImage(*aCtx, aDirtyRect, aBuilder->ToReferenceFrame(mFrame));
 }
 
 NS_IMETHODIMP
-nsImageBoxFrame::Paint(nsPresContext*      aPresContext,
-                       nsIRenderingContext& aRenderingContext,
-                       const nsRect&        aDirtyRect,
-                       nsFramePaintLayer    aWhichLayer,
-                       PRUint32             aFlags)
-{	
-  if (!GetStyleVisibility()->IsVisible())
-    return NS_OK;
+nsImageBoxFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                  const nsRect&           aDirtyRect,
+                                  const nsDisplayListSet& aLists)
+{       
+  nsresult rv = nsLeafBoxFrame::BuildDisplayList(aBuilder, aDirtyRect, aLists);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  nsresult rv = nsLeafBoxFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
-
-  PaintImage(aRenderingContext, aDirtyRect, aWhichLayer);
-
-  return rv;
-}
-
-
-void
-nsImageBoxFrame::PaintImage(nsIRenderingContext& aRenderingContext,
-                            const nsRect& aDirtyRect,
-                            nsFramePaintLayer aWhichLayer)
-{
   if ((0 == mRect.width) || (0 == mRect.height)) {
     // Do not render when given a zero area. This avoids some useless
     // scaling work while we wait for our image dimensions to arrive
     // asynchronously.
-    return;
+    return NS_OK;
   }
 
+  if (!IsVisibleForPainting(aBuilder))
+    return NS_OK;
+
+  return aLists.Content()->AppendNewToTop(new (aBuilder) nsDisplayXULImage(this));
+}
+
+void
+nsImageBoxFrame::PaintImage(nsIRenderingContext& aRenderingContext,
+                            const nsRect& aDirtyRect, nsPoint aPt)
+{
   nsRect rect;
   GetClientRect(rect);
 
-  if (NS_FRAME_PAINT_LAYER_FOREGROUND != aWhichLayer)
-    return;
+  rect += aPt;
 
   if (!mImageRequest)
     return;
@@ -469,7 +416,7 @@ nsImageBoxFrame::PaintImage(nsIRenderingContext& aRenderingContext,
 // When the style context changes, make sure that all of our image is up to date.
 //
 NS_IMETHODIMP
-nsImageBoxFrame::DidSetStyleContext( nsPresContext* aPresContext )
+nsImageBoxFrame::DidSetStyleContext()
 {
   // Fetch our subrect.
   const nsStyleList* myList = GetStyleList();
@@ -613,10 +560,15 @@ NS_IMETHODIMP nsImageBoxFrame::OnStopDecode(imgIRequest *request,
                                             const PRUnichar *statusArg)
 {
   if (NS_SUCCEEDED(aStatus))
+    // Fire an onload DOM event.
+    FireImageDOMEvent(mContent, NS_LOAD);
+  else {
     // Fire an onerror DOM event.
-    FireDOMEvent(mContent, NS_IMAGE_LOAD);
-  else // Fire an onload DOM event.
-    FireDOMEvent(mContent, NS_IMAGE_ERROR);
+    mIntrinsicSize.SizeTo(0, 0);
+    nsBoxLayoutState state(GetPresContext());
+    MarkDirty(state);
+    FireImageDOMEvent(mContent, NS_LOAD_ERROR);
+  }
 
   return NS_OK;
 }
@@ -641,11 +593,6 @@ nsImageBoxListener::~nsImageBoxListener()
 {
 }
 
-NS_IMETHODIMP nsImageBoxListener::OnStartDecode(imgIRequest *request)
-{
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsImageBoxListener::OnStartContainer(imgIRequest *request,
                                                    imgIContainer *image)
 {
@@ -655,25 +602,6 @@ NS_IMETHODIMP nsImageBoxListener::OnStartContainer(imgIRequest *request,
   return mFrame->OnStartContainer(request, image);
 }
 
-NS_IMETHODIMP nsImageBoxListener::OnStartFrame(imgIRequest *request,
-                                               gfxIImageFrame *frame)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsImageBoxListener::OnDataAvailable(imgIRequest *request,
-                                                  gfxIImageFrame *frame,
-                                                  const nsRect * rect)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsImageBoxListener::OnStopFrame(imgIRequest *request,
-                                              gfxIImageFrame *frame)
-{
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsImageBoxListener::OnStopContainer(imgIRequest *request,
                                                   imgIContainer *image)
 {
