@@ -48,7 +48,6 @@
 #include "nsXULAtoms.h"
 #include "nsHTMLAtoms.h"
 #include "nsIContent.h"
-#include "nsITextContent.h"
 #include "nsINameSpaceManager.h"
 #include "nsIDocument.h"
 #include "nsIBindingManager.h"
@@ -70,8 +69,9 @@
 #include "nsIDOMNSDocument.h"
 #include "nsPIBoxObject.h"
 #include "nsINodeInfo.h"
-#include "nsScrollPortFrame.h"
 #include "nsLayoutUtils.h"
+#include "nsPIListBoxObject.h"
+#include "nsContentUtils.h"
 
 /////////////// nsListScrollSmoother //////////////////
 
@@ -91,7 +91,7 @@
 // notices it as a delay.
 #define USER_TIME_THRESHOLD 150000
 
-// how long it takes to layout a single row inital value.
+// how long it takes to layout a single row initial value.
 // we will time this after we scroll a few rows.
 #define TIME_PER_ROW_INITAL  50000
 
@@ -170,8 +170,11 @@ NS_IMPL_ISUPPORTS1(nsListScrollSmoother, nsITimerCallback)
 
 /////////////// nsListBoxBodyFrame //////////////////
 
-nsListBoxBodyFrame::nsListBoxBodyFrame(nsIPresShell* aPresShell, PRBool aIsRoot, nsIBoxLayout* aLayoutManager)
-  : nsBoxFrame(aPresShell, aIsRoot, aLayoutManager),
+nsListBoxBodyFrame::nsListBoxBodyFrame(nsIPresShell* aPresShell,
+                                       nsStyleContext* aContext,
+                                       PRBool aIsRoot,
+                                       nsIBoxLayout* aLayoutManager)
+  : nsBoxFrame(aPresShell, aContext, aIsRoot, aLayoutManager),
     mRowCount(-1),
     mRowHeight(0),
     mRowHeightWasSet(PR_FALSE),
@@ -230,18 +233,17 @@ NS_INTERFACE_MAP_END_INHERITING(nsBoxFrame)
 ////////// nsIFrame /////////////////
 
 NS_IMETHODIMP
-nsListBoxBodyFrame::Init(nsPresContext* aPresContext, nsIContent* aContent,
-                         nsIFrame* aParent, nsStyleContext* aContext, nsIFrame* aPrevInFlow)
+nsListBoxBodyFrame::Init(nsIContent*     aContent,
+                         nsIFrame*       aParent, 
+                         nsIFrame*       aPrevInFlow)
 {
-  nsresult rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
+  nsresult rv = nsBoxFrame::Init(aContent, aParent, aPrevInFlow);
 
-  mOnePixel = aPresContext->IntScaledPixelsToTwips(1);
+  nsPresContext *presContext = GetPresContext();
   
-  nsIFrame* box = aParent->GetParent();
-  if (!box)
-    return rv;
-
-  nsCOMPtr<nsIScrollableFrame> scrollFrame(do_QueryInterface(box));
+  mOnePixel = presContext->IntScaledPixelsToTwips(1);
+  
+  nsIScrollableFrame* scrollFrame = nsLayoutUtils::GetScrollableFrameFor(this);
   if (!scrollFrame)
     return rv;
 
@@ -257,23 +259,23 @@ nsListBoxBodyFrame::Init(nsPresContext* aPresContext, nsIContent* aContent,
   nsCOMPtr<nsIScrollbarFrame> scrollbarFrame(do_QueryInterface(verticalScrollbar));
   scrollbarFrame->SetScrollbarMediator(this);
 
-  nsBoxLayoutState boxLayoutState(aPresContext);
+  nsBoxLayoutState boxLayoutState(presContext);
 
   nsCOMPtr<nsIFontMetrics> fm;
-  aPresContext->DeviceContext()->GetMetricsFor(aContext->GetStyleFont()->mFont,
-                                               *getter_AddRefs(fm));
+  presContext->DeviceContext()->GetMetricsFor(
+    GetStyleContext()->GetStyleFont()->mFont, *getter_AddRefs(fm)
+    );
   fm->GetHeight(mRowHeight);
 
   return rv;
-
 }
 
-NS_IMETHODIMP
-nsListBoxBodyFrame::Destroy(nsPresContext* aPresContext)
+void
+nsListBoxBodyFrame::Destroy()
 {
   // make sure we cancel any posted callbacks.
   if (mReflowCallbackPosted)
-     aPresContext->PresShell()->CancelReflowCallback(this);
+     GetPresContext()->PresShell()->CancelReflowCallback(this);
 
   // Make sure we tell our listbox's box object we're being destroyed.
   for (nsIFrame *a = mParent; a; a = a->GetParent()) {
@@ -281,8 +283,7 @@ nsListBoxBodyFrame::Destroy(nsPresContext* aPresContext)
     nsIDocument *doc;
 
     if (content &&
-        content->GetNodeInfo()->Equals(nsXULAtoms::listbox,
-                                       kNameSpaceID_XUL) &&
+        content->NodeInfo()->Equals(nsXULAtoms::listbox, kNameSpaceID_XUL) &&
         (doc = content->GetDocument())) {
       nsCOMPtr<nsIDOMElement> e(do_QueryInterface(content));
       nsCOMPtr<nsIDOMNSDocument> nsdoc(do_QueryInterface(doc));
@@ -290,23 +291,20 @@ nsListBoxBodyFrame::Destroy(nsPresContext* aPresContext)
       nsCOMPtr<nsIBoxObject> box;
       nsdoc->GetBoxObjectFor(e, getter_AddRefs(box));
 
-      nsCOMPtr<nsPIBoxObject> pibox(do_QueryInterface(box));
-
-      if (pibox) {
-        pibox->InvalidatePresentationStuff();
+      nsCOMPtr<nsPIBoxObject> piBox = do_QueryInterface(box);
+      if (piBox) {
+        piBox->ClearCachedValues();
       }
 
       break;
     }
   }
 
-  return nsBoxFrame::Destroy(aPresContext);
+  nsBoxFrame::Destroy();
 }
 
 NS_IMETHODIMP
-nsListBoxBodyFrame::AttributeChanged(nsPresContext* aPresContext,
-                                     nsIContent* aChild,
-                                     PRInt32 aNameSpaceID,
+nsListBoxBodyFrame::AttributeChanged(PRInt32 aNameSpaceID,
                                      nsIAtom* aAttribute, 
                                      PRInt32 aModType)
 {
@@ -319,20 +317,21 @@ nsListBoxBodyFrame::AttributeChanged(nsPresContext* aPresContext,
     if (!rows.IsEmpty()) {
       PRInt32 dummy;
       PRInt32 count = rows.ToInteger(&dummy);
+      nsPresContext* presContext = GetPresContext();
       float t2p;
-      t2p = aPresContext->TwipsToPixels();
+      t2p = presContext->TwipsToPixels();
       PRInt32 rowHeight = GetRowHeightTwips();
       rowHeight = NSTwipsToIntPixels(rowHeight, t2p);
       nsAutoString value;
       value.AppendInt(rowHeight*count);
       mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::minheight, value, PR_FALSE);
 
-      nsBoxLayoutState state(aPresContext);
+      nsBoxLayoutState state(presContext);
       MarkDirty(state);
     }
   }
   else
-    rv = nsBoxFrame::AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType);
+    rv = nsBoxFrame::AttributeChanged(aNameSpaceID, aAttribute, aModType);
 
   return rv;
  
@@ -372,9 +371,8 @@ nsSize
 nsListBoxBodyFrame::GetMinSizeForScrollArea(nsBoxLayoutState& aBoxLayoutState)
 {
   nsSize result(0, 0);
-  nsAutoString sizeMode;
-  GetContent()->GetAttr(kNameSpaceID_None, nsXULAtoms::sizemode, sizeMode);
-  if (!sizeMode.IsEmpty()) {  
+  if (nsContentUtils::HasNonEmptyAttr(GetContent(), kNameSpaceID_None,
+                                      nsXULAtoms::sizemode)) {
     GetPrefSize(aBoxLayoutState, result);
     result.height = 0;
     nsIScrollableFrame* scrollFrame = nsLayoutUtils::GetScrollableFrameFor(this);
@@ -394,6 +392,8 @@ nsListBoxBodyFrame::GetPrefSize(nsBoxLayoutState& aBoxLayoutState, nsSize& aSize
   nsresult rv = nsBoxFrame::GetPrefSize(aBoxLayoutState, aSize);
 
   PRInt32 size = GetFixedRowSize();
+  nsIBox* box = nsnull;
+  GetChildBox(&box);
   if (size > -1)
     aSize.height = size*GetRowHeightTwips();
    
@@ -472,7 +472,7 @@ nsListBoxBodyFrame::VisibilityChanged(nsISupports* aScrollbar, PRBool aVisible)
   PRInt32 delta = mCurrentIndex - lastPageTopRow;
   if (delta > 0) {
     mCurrentIndex = lastPageTopRow;
-    InternalPositionChanged(PR_TRUE, delta, PR_FALSE);
+    InternalPositionChanged(PR_TRUE, delta);
   }
 
   return NS_OK;
@@ -500,10 +500,8 @@ nsListBoxBodyFrame::ScrollbarButtonPressed(nsISupports* aScrollbar, PRInt32 aOld
 NS_IMETHODIMP
 nsListBoxBodyFrame::ReflowFinished(nsIPresShell* aPresShell, PRBool* aFlushFlag)
 {
-  nsBoxLayoutState state(mPresContext);
-
   // now create or destroy any rows as needed
-  CreateRows(state);
+  CreateRows();
 
   // keep scrollbar in sync
   if (mAdjustScroll) {
@@ -514,6 +512,7 @@ nsListBoxBodyFrame::ReflowFinished(nsIPresShell* aPresShell, PRBool* aFlushFlag)
   // if the row height changed then mark everything as a style change. 
   // That will dirty the entire listbox
   if (mRowHeightWasSet) {
+     nsBoxLayoutState state(GetPresContext());
      MarkStyleChange(state);
      PRInt32 pos = mCurrentIndex * mRowHeight;
      if (mYPosition != pos) 
@@ -594,12 +593,6 @@ nsListBoxBodyFrame::EnsureIndexIsVisible(PRInt32 aRowIndex)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsListBoxBodyFrame::ScrollToIndex(PRInt32 aRowIndex)
-{
-  return DoScrollToIndex(aRowIndex);
-}
-
 NS_IMETHODIMP
 nsListBoxBodyFrame::ScrollByLines(PRInt32 aNumLines)
 {
@@ -626,7 +619,7 @@ nsListBoxBodyFrame::ScrollByLines(PRInt32 aNumLines)
     
   // I'd use Composite here, but it doesn't always work.
   // vm->Composite();
-  mPresContext->GetViewManager()->ForceUpdate();
+  GetPresContext()->GetViewManager()->ForceUpdate();
 
   return NS_OK;
 }
@@ -733,7 +726,7 @@ nsListBoxBodyFrame::SetRowHeight(nscoord aRowHeight)
       PRInt32 dummy;
       PRInt32 count = rows.ToInteger(&dummy);
       float t2p;
-      t2p = mPresContext->TwipsToPixels();
+      t2p = GetPresContext()->TwipsToPixels();
       PRInt32 rowHeight = NSTwipsToIntPixels(aRowHeight, t2p);
       nsAutoString value;
       value.AppendInt(rowHeight*count);
@@ -745,20 +738,16 @@ nsListBoxBodyFrame::SetRowHeight(nscoord aRowHeight)
     // so we can create or destory rows as needed
     mRowHeightWasSet = PR_TRUE;
     PostReflowCallback();
-  } 
+  }
 }
 
 nscoord
 nsListBoxBodyFrame::GetAvailableHeight()
 {
-  nsIBox* box;
-  GetParentBox(&box);
-  if (!box)
-    return 0;
-
-  nsRect contentRect;
-  box->GetContentRect(contentRect);
-  return contentRect.height;
+  nsIScrollableFrame* scrollFrame
+    = nsLayoutUtils::GetScrollableFrameFor(this);
+  nsIScrollableView* scrollView = scrollFrame->GetScrollableView();
+  return scrollView->View()->GetBounds().height;
 }
 
 nscoord
@@ -812,10 +801,8 @@ nsListBoxBodyFrame::ComputeIntrinsicWidth(nsBoxLayoutState& aBoxLayoutState)
           nsAutoString value;
           PRUint32 textCount = child->GetChildCount();
           for (PRUint32 j = 0; j < textCount; ++j) {
-            nsCOMPtr<nsITextContent> text =
-              do_QueryInterface(child->GetChildAt(j));
-
-            if (text && text->IsContentOfType(nsIContent::eTEXT)) {
+            nsIContent* text = child->GetChildAt(j);
+            if (text && text->IsNodeOfType(nsINode::eTEXT)) {
               text->AppendTextTo(value);
             }
           }
@@ -859,14 +846,14 @@ nsListBoxBodyFrame::PostReflowCallback()
 {
   if (!mReflowCallbackPosted) {
     mReflowCallbackPosted = PR_TRUE;
-    mPresContext->PresShell()->PostReflowCallback(this);
+    GetPresContext()->PresShell()->PostReflowCallback(this);
   }
 }
 
 ////////// scrolling
 
 NS_IMETHODIMP
-nsListBoxBodyFrame::DoScrollToIndex(PRInt32 aRowIndex, PRBool aForceDestruct)
+nsListBoxBodyFrame::ScrollToIndex(PRInt32 aRowIndex)
 {
   if (( aRowIndex < 0 ) || (mRowHeight == 0))
     return NS_OK;
@@ -884,7 +871,7 @@ nsListBoxBodyFrame::DoScrollToIndex(PRInt32 aRowIndex, PRBool aForceDestruct)
     return NS_OK;
 
   mCurrentIndex = newIndex;
-  InternalPositionChanged(up, delta, aForceDestruct);
+  InternalPositionChanged(up, delta);
 
   // This change has to happen immediately.
   // Flush any pending reflow commands.
@@ -911,12 +898,13 @@ nsListBoxBodyFrame::InternalPositionChangedCallback()
 }
 
 NS_IMETHODIMP
-nsListBoxBodyFrame::InternalPositionChanged(PRBool aUp, PRInt32 aDelta, PRBool aForceDestruct)
+nsListBoxBodyFrame::InternalPositionChanged(PRBool aUp, PRInt32 aDelta)
 {  
   if (aDelta == 0)
     return NS_OK;
 
-  nsBoxLayoutState state(mPresContext);
+  nsPresContext *presContext = GetPresContext();
+  nsBoxLayoutState state(presContext);
 
   // begin timing how long it takes to scroll a row
   PRTime start = PR_Now();
@@ -925,9 +913,9 @@ nsListBoxBodyFrame::InternalPositionChanged(PRBool aUp, PRInt32 aDelta, PRBool a
 
   PRInt32 visibleRows = 0;
   if (mRowHeight)
-	  visibleRows = GetAvailableHeight()/mRowHeight;
+    visibleRows = GetAvailableHeight()/mRowHeight;
   
-  if (aDelta < visibleRows && !aForceDestruct) {
+  if (aDelta < visibleRows) {
     PRInt32 loseRows = aDelta;
     if (aUp) {
       // scrolling up, destroy rows from the bottom downwards
@@ -960,14 +948,11 @@ nsListBoxBodyFrame::InternalPositionChanged(PRBool aUp, PRInt32 aDelta, PRBool a
   MarkDirtyChildren(state);
   // Flush calls CreateRows
   // XXXbz there has to be a better way to do this than flushing!
-  mPresContext->PresShell()->FlushPendingNotifications(Flush_OnlyReflow);
+  presContext->PresShell()->FlushPendingNotifications(Flush_OnlyReflow);
   mScrolling = PR_FALSE;
   
   VerticalScroll(mYPosition);
 
-  if (aForceDestruct)
-    Redraw(state, nsnull, PR_FALSE);
-  
   PRTime end = PR_Now();
 
   PRTime difTime;
@@ -998,18 +983,8 @@ nsListBoxBodyFrame::GetSmoother()
 void
 nsListBoxBodyFrame::VerticalScroll(PRInt32 aPosition)
 {
-  nsIBox* box;
-  GetParentBox(&box);
-  if (!box)
-    return;
-
-  box->GetParentBox(&box);
-  if (!box)
-    return;
-
-  nsCOMPtr<nsIScrollableFrame> scrollFrame(do_QueryInterface(box));
-  if (!scrollFrame)
-    return;
+  nsIScrollableFrame* scrollFrame
+    = nsLayoutUtils::GetScrollableFrameFor(this);
 
   nsPoint scrollPosition = scrollFrame->GetScrollPosition();
  
@@ -1036,7 +1011,7 @@ nsListBoxBodyFrame::GetLastFrame()
 ////////// lazy row creation and destruction
 
 void
-nsListBoxBodyFrame::CreateRows(nsBoxLayoutState& aState)
+nsListBoxBodyFrame::CreateRows()
 {
   // Get our client rect.
   nsRect clientRect;
@@ -1087,7 +1062,7 @@ nsListBoxBodyFrame::DestroyRows(PRInt32& aRowsToLose)
   // We need to destroy frames until our row count has been properly
   // reduced.  A reflow will then pick up and create the new frames.
   nsIFrame* childFrame = GetFirstFrame();
-  nsBoxLayoutState state(mPresContext);
+  nsBoxLayoutState state(GetPresContext());
 
   while (childFrame && aRowsToLose > 0) {
     --aRowsToLose;
@@ -1107,7 +1082,7 @@ nsListBoxBodyFrame::ReverseDestroyRows(PRInt32& aRowsToLose)
   // We need to destroy frames until our row count has been properly
   // reduced.  A reflow will then pick up and create the new frames.
   nsIFrame* childFrame = GetLastFrame();
-  nsBoxLayoutState state(mPresContext);
+  nsBoxLayoutState state(GetPresContext());
 
   while (childFrame && aRowsToLose > 0) {
     --aRowsToLose;
@@ -1173,8 +1148,11 @@ nsListBoxBodyFrame::GetFirstItemBox(PRInt32 aOffset, PRBool* aCreated)
     // XXX check here if frame was even created, it may not have been if
     //     display: none was on listitem content
     PRBool isAppend = mRowsToPrepend <= 0;
-    mFrameConstructor->CreateListBoxContent(mPresContext, this, nsnull, startContent,
-                                            &mTopFrame, isAppend, PR_FALSE, nsnull);
+    
+    nsPresContext* presContext = GetPresContext();
+    nsCSSFrameConstructor* fc = presContext->PresShell()->FrameConstructor();
+    fc->CreateListBoxContent(presContext, this, nsnull, startContent,
+                             &mTopFrame, isAppend, PR_FALSE, nsnull);
     
     if (mTopFrame) {
       if (aCreated)
@@ -1218,9 +1196,11 @@ nsListBoxBodyFrame::GetNextItemBox(nsIBox* aBox, PRInt32 aOffset,
       // Either append the new frame, or insert it after the current frame
       PRBool isAppend = result != mLinkupFrame && mRowsToPrepend <= 0;
       nsIFrame* prevFrame = isAppend ? nsnull : aBox;
-      mFrameConstructor->CreateListBoxContent(mPresContext, this, prevFrame,
-                                              nextContent, &result, isAppend,
-                                              PR_FALSE, nsnull);
+      
+      nsPresContext* presContext = GetPresContext();
+      nsCSSFrameConstructor* fc = presContext->PresShell()->FrameConstructor();
+      fc->CreateListBoxContent(presContext, this, prevFrame, nextContent,
+                               &result, isAppend, PR_FALSE, nsnull);
 
       if (result) {
         if (aCreated)
@@ -1243,6 +1223,13 @@ nsListBoxBodyFrame::GetNextItemBox(nsIBox* aBox, PRInt32 aOffset,
 PRBool
 nsListBoxBodyFrame::ContinueReflow(nscoord height) 
 { 
+  nsPresContext* presContext = GetPresContext();
+  if (presContext->PresShell()->IsAccessibilityActive()) {
+    // Create all the frames at once so screen readers and
+    // onscreen keyboards can see the full list right away
+    return PR_TRUE;
+  }
+
   if (height <= 0) {
     nsIFrame* lastChild = GetLastFrame();
     nsIFrame* startingPoint = mBottomFrame;
@@ -1255,7 +1242,7 @@ nsListBoxBodyFrame::ContinueReflow(nscoord height)
       // We have some hangers on (probably caused by shrinking the size of the window).
       // Nuke them.
       nsIFrame* currFrame = startingPoint->GetNextSibling();
-      nsBoxLayoutState state(mPresContext);
+      nsBoxLayoutState state(GetPresContext());
 
       while (currFrame) {
         nsIFrame* nextFrame = currFrame->GetNextSibling();
@@ -1275,7 +1262,7 @@ NS_IMETHODIMP
 nsListBoxBodyFrame::ListBoxAppendFrames(nsIFrame* aFrameList)
 {
   // append them after
-  nsBoxLayoutState state(mPresContext);
+  nsBoxLayoutState state(GetPresContext());
   mFrames.AppendFrames(nsnull, aFrameList);
   if (mLayoutManager)
     mLayoutManager->ChildrenAppended(this, state, aFrameList);
@@ -1288,7 +1275,7 @@ NS_IMETHODIMP
 nsListBoxBodyFrame::ListBoxInsertFrames(nsIFrame* aPrevFrame, nsIFrame* aFrameList)
 {
   // insert the frames to our info list
-  nsBoxLayoutState state(mPresContext);
+  nsBoxLayoutState state(GetPresContext());
   mFrames.InsertFrames(nsnull, aPrevFrame, aFrameList);
   if (mLayoutManager)
     mLayoutManager->ChildrenInserted(this, state, aPrevFrame, aFrameList);
@@ -1312,8 +1299,7 @@ nsListBoxBodyFrame::OnContentInserted(nsPresContext* aPresContext, nsIContent* a
   // to CreateRows will create all the frames, but OnContentInserted will
   // still be called again for each content node - so we need to make sure
   // that the frame for each content node hasn't already been created.
-  nsIFrame* childFrame = nsnull;
-  shell->GetPrimaryFrameFor(aChildContent, &childFrame);
+  nsIFrame* childFrame = shell->GetPrimaryFrameFor(aChildContent);
   if (childFrame)
     return;
 
@@ -1328,14 +1314,13 @@ nsListBoxBodyFrame::OnContentInserted(nsPresContext* aPresContext, nsIContent* a
     mRowsToPrepend = 1;
   } else if (nextSiblingContent) {
     // we may be inserting before a frame that is on screen
-    nsIFrame* nextSiblingFrame = nsnull;
-    shell->GetPrimaryFrameFor(nextSiblingContent, &nextSiblingFrame);
+    nsIFrame* nextSiblingFrame = shell->GetPrimaryFrameFor(nextSiblingContent);
     mLinkupFrame = nextSiblingFrame;
   }
   
+  CreateRows();
   nsBoxLayoutState state(aPresContext);
   MarkDirtyChildren(state);
-  shell->FlushPendingNotifications(Flush_OnlyReflow);
 }
 
 // 
@@ -1344,6 +1329,9 @@ nsListBoxBodyFrame::OnContentInserted(nsPresContext* aPresContext, nsIContent* a
 void
 nsListBoxBodyFrame::OnContentRemoved(nsPresContext* aPresContext, nsIFrame* aChildFrame, PRInt32 aIndex)
 {
+  NS_ASSERTION(!aChildFrame || aChildFrame->GetParent() == this,
+               "Removing frame that's not our child... Not good");
+  
   if (mRowCount >= 0)
     --mRowCount;
 
@@ -1379,9 +1367,8 @@ nsListBoxBodyFrame::OnContentRemoved(nsPresContext* aPresContext, nsIFrame* aChi
     PRUint32 childCount = listBoxContent->GetChildCount();
     if (childCount > 0) {
       nsIContent *lastChild = listBoxContent->GetChildAt(childCount - 1);
-      nsIFrame* lastChildFrame = nsnull;
-      aPresContext->PresShell()->GetPrimaryFrameFor(lastChild,
-                                                    &lastChildFrame);
+      nsIFrame* lastChildFrame = 
+        aPresContext->PresShell()->GetPrimaryFrameFor(lastChild);
     
       if (lastChildFrame) {
         mTopFrame = nsnull;
@@ -1404,7 +1391,6 @@ nsListBoxBodyFrame::OnContentRemoved(nsPresContext* aPresContext, nsIFrame* aChi
   }
 
   MarkDirtyChildren(state);
-  aPresContext->PresShell()->FlushPendingNotifications(Flush_OnlyReflow);
 }
 
 void
@@ -1460,29 +1446,27 @@ void
 nsListBoxBodyFrame::RemoveChildFrame(nsBoxLayoutState &aState,
                                      nsIFrame         *aFrame)
 {
-  mFrameConstructor->RemoveMappingsForFrameSubtree(mPresContext, aFrame, nsnull);
+  nsPresContext* presContext = GetPresContext();
+  nsCSSFrameConstructor* fc = presContext->PresShell()->FrameConstructor();
+  fc->RemoveMappingsForFrameSubtree(aFrame);
 
-  mFrames.RemoveFrame(aFrame);
+#ifdef DEBUG
+  PRBool removed =
+#endif
+    mFrames.RemoveFrame(aFrame);
+  NS_ASSERTION(removed,
+               "Going to destroy a frame we didn't remove.  Prepare to crash");
   if (mLayoutManager)
     mLayoutManager->ChildrenRemoved(this, aState, aFrame);
-  aFrame->Destroy(mPresContext);
+  aFrame->Destroy();
 }
 
 // Creation Routines ///////////////////////////////////////////////////////////////////////
 
-nsresult
-NS_NewListBoxBodyFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame, PRBool aIsRoot, 
-                       nsIBoxLayout* aLayoutManager)
+nsIFrame*
+NS_NewListBoxBodyFrame(nsIPresShell* aPresShell, nsStyleContext* aContext,
+                       PRBool aIsRoot, nsIBoxLayout* aLayoutManager)
 {
-  NS_PRECONDITION(aNewFrame, "null OUT ptr");
-  if (nsnull == aNewFrame) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  nsListBoxBodyFrame* it = new (aPresShell) nsListBoxBodyFrame(aPresShell, aIsRoot, aLayoutManager);
-  if (!it)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  *aNewFrame = it;
-  return NS_OK;
-  
+  return 
+    new (aPresShell) nsListBoxBodyFrame(aPresShell, aContext, aIsRoot, aLayoutManager);
 }
