@@ -76,7 +76,8 @@
 #include "nsIServiceManager.h"
 #include "nsGUIEvent.h"
 #include "nsContentUtils.h"
-#include "nsIPrivateDOMEvent.h"
+#include "nsLayoutUtils.h"
+#include "nsDisplayList.h"
 
 PRBool nsSliderFrame::gMiddlePref = PR_FALSE;
 PRInt32 nsSliderFrame::gSnapMultiplier = 6;
@@ -92,24 +93,18 @@ GetContentOfBox(nsIBox *aBox)
   return content;
 }
 
-nsresult
-NS_NewSliderFrame ( nsIPresShell* aPresShell, nsIFrame** aNewFrame)
+nsIFrame*
+NS_NewSliderFrame (nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
-  NS_PRECONDITION(aNewFrame, "null OUT ptr");
-  if (nsnull == aNewFrame) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  nsSliderFrame* it = new (aPresShell) nsSliderFrame(aPresShell);
-  if (nsnull == it)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  *aNewFrame = it;
-  return NS_OK;
-
+  return new (aPresShell) nsSliderFrame(aPresShell, aContext);
 } // NS_NewSliderFrame
 
-nsSliderFrame::nsSliderFrame(nsIPresShell* aPresShell):nsBoxFrame(aPresShell),
- mCurPos(0), mScrollbarListener(nsnull),mChange(0), mMediator(nsnull)
+nsSliderFrame::nsSliderFrame(nsIPresShell* aPresShell, nsStyleContext* aContext):
+  nsBoxFrame(aPresShell, aContext),
+  mCurPos(0),
+  mScrollbarListener(nsnull),
+  mChange(0),
+  mMediator(nsnull)
 {
 }
 
@@ -120,14 +115,11 @@ nsSliderFrame::~nsSliderFrame()
 }
 
 NS_IMETHODIMP
-nsSliderFrame::Init(nsPresContext*  aPresContext,
-                    nsIContent*      aContent,
+nsSliderFrame::Init(nsIContent*      aContent,
                     nsIFrame*        aParent,
-                    nsStyleContext*  aContext,
                     nsIFrame*        aPrevInFlow)
 {
-  nsresult  rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aContext,
-                                  aPrevInFlow);
+  nsresult rv = nsBoxFrame::Init(aContent, aParent, aPrevInFlow);
 
   static PRBool gotPrefs = PR_FALSE;
   if (!gotPrefs) {
@@ -137,19 +129,17 @@ nsSliderFrame::Init(nsPresContext*  aPresContext,
     gSnapMultiplier = nsContentUtils::GetIntPref("slider.snapMultiplier");
   }
 
-  CreateViewForFrame(aPresContext,this,aContext,PR_TRUE);
+  CreateViewForFrame(GetPresContext(), this, GetStyleContext(), PR_TRUE);
   nsIView* view = GetView();
   view->GetViewManager()->SetViewContentTransparency(view, PR_TRUE);
   return rv;
 }
 
 NS_IMETHODIMP
-nsSliderFrame::RemoveFrame(nsPresContext* aPresContext,
-                           nsIPresShell& aPresShell,
-                           nsIAtom* aListName,
-                           nsIFrame* aOldFrame)
+nsSliderFrame::RemoveFrame(nsIAtom*        aListName,
+                           nsIFrame*       aOldFrame)
 {
-  nsresult rv = nsBoxFrame::RemoveFrame(aPresContext, aPresShell, aListName, aOldFrame);
+  nsresult rv = nsBoxFrame::RemoveFrame(aListName, aOldFrame);
   PRInt32 start = GetChildCount();
   if (start == 0)
     RemoveListener();
@@ -158,14 +148,12 @@ nsSliderFrame::RemoveFrame(nsPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsSliderFrame::InsertFrames(nsPresContext* aPresContext,
-                            nsIPresShell& aPresShell,
-                            nsIAtom* aListName,
-                            nsIFrame* aPrevFrame,
-                            nsIFrame* aFrameList)
+nsSliderFrame::InsertFrames(nsIAtom*        aListName,
+                            nsIFrame*       aPrevFrame,
+                            nsIFrame*       aFrameList)
 {
   PRInt32 start = GetChildCount();
-  nsresult rv = nsBoxFrame::InsertFrames(aPresContext, aPresShell, aListName, aPrevFrame, aFrameList);
+  nsresult rv = nsBoxFrame::InsertFrames(aListName, aPrevFrame, aFrameList);
   if (start == 0)
     AddListener();
 
@@ -173,15 +161,13 @@ nsSliderFrame::InsertFrames(nsPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsSliderFrame::AppendFrames(nsPresContext* aPresContext,
-                           nsIPresShell&   aPresShell,
-                           nsIAtom*        aListName,
-                           nsIFrame*       aFrameList)
+nsSliderFrame::AppendFrames(nsIAtom*        aListName,
+                            nsIFrame*       aFrameList)
 {
   // if we have no children and on was added then make sure we add the
   // listener
   PRInt32 start = GetChildCount();
-  nsresult rv = nsBoxFrame::AppendFrames(aPresContext, aPresShell, aListName, aFrameList);
+  nsresult rv = nsBoxFrame::AppendFrames(aListName, aFrameList);
   if (start == 0)
     AddListener();
 
@@ -194,6 +180,12 @@ nsSliderFrame::GetCurrentPosition(nsIContent* content)
   return GetIntegerAttribute(content, nsXULAtoms::curpos, 0);
 }
 
+PRInt32
+nsSliderFrame::GetMinPosition(nsIContent* content)
+{
+  return GetIntegerAttribute(content, nsXULAtoms::minpos, 0);
+}
+
 PRInt32
 nsSliderFrame::GetMaxPosition(nsIContent* content)
 {
@@ -217,8 +209,8 @@ PRInt32
 nsSliderFrame::GetIntegerAttribute(nsIContent* content, nsIAtom* atom, PRInt32 defaultValue)
 {
     nsAutoString value;
-    if (NS_CONTENT_ATTR_HAS_VALUE == content->GetAttr(kNameSpaceID_None, atom, value))
-    {
+    content->GetAttr(kNameSpaceID_None, atom, value);
+    if (!value.IsEmpty()) {
       PRInt32 error;
 
       // convert it to an integer
@@ -230,33 +222,32 @@ nsSliderFrame::GetIntegerAttribute(nsIContent* content, nsIAtom* atom, PRInt32 d
 
 
 NS_IMETHODIMP
-nsSliderFrame::AttributeChanged(nsPresContext* aPresContext,
-                               nsIContent* aChild,
-                               PRInt32 aNameSpaceID,
-                               nsIAtom* aAttribute,
-                               PRInt32 aModType)
+nsSliderFrame::AttributeChanged(PRInt32 aNameSpaceID,
+                                nsIAtom* aAttribute,
+                                PRInt32 aModType)
 {
-  nsresult rv = nsBoxFrame::AttributeChanged(aPresContext, aChild,
-                                             aNameSpaceID, aAttribute,
+  nsresult rv = nsBoxFrame::AttributeChanged(aNameSpaceID, aAttribute,
                                              aModType);
   // if the current position changes
   if (aAttribute == nsXULAtoms::curpos) {
-     rv = CurrentPositionChanged(aPresContext);
+     rv = CurrentPositionChanged(GetPresContext());
      NS_ASSERTION(NS_SUCCEEDED(rv), "failed to change position");
      if (NS_FAILED(rv))
         return rv;
-  } else if (aAttribute == nsXULAtoms::maxpos) {
+  } else if (aAttribute == nsXULAtoms::minpos ||
+             aAttribute == nsXULAtoms::maxpos) {
       // bounds check it.
 
       nsIBox* scrollbarBox = GetScrollbar();
       nsCOMPtr<nsIContent> scrollbar;
       scrollbar = GetContentOfBox(scrollbarBox);
       PRInt32 current = GetCurrentPosition(scrollbar);
+      PRInt32 min = GetMinPosition(scrollbar);
       PRInt32 max = GetMaxPosition(scrollbar);
-      if (current < 0 || current > max)
+      if (current < min || current > max)
       {
-        if (current < 0)
-            current = 0;
+        if (current < min || max < min)
+            current = min;
         else if (current > max)
             current = max;
 
@@ -276,11 +267,12 @@ nsSliderFrame::AttributeChanged(nsPresContext* aPresContext,
       }
   }
 
-  if (aAttribute == nsXULAtoms::maxpos ||
+  if (aAttribute == nsXULAtoms::minpos ||
+      aAttribute == nsXULAtoms::maxpos ||
       aAttribute == nsXULAtoms::pageincrement ||
       aAttribute == nsXULAtoms::increment) {
 
-      nsBoxLayoutState state(aPresContext);
+      nsBoxLayoutState state(GetPresContext());
       MarkDirtyChildren(state);
   }
 
@@ -288,11 +280,24 @@ nsSliderFrame::AttributeChanged(nsPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsSliderFrame::Paint(nsPresContext*      aPresContext,
-                     nsIRenderingContext& aRenderingContext,
-                     const nsRect&        aDirtyRect,
-                     nsFramePaintLayer    aWhichLayer,
-                     PRUint32             aFlags)
+nsSliderFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                const nsRect&           aDirtyRect,
+                                const nsDisplayListSet& aLists)
+{
+  if (aBuilder->IsForEventDelivery() && isDraggingThumb()) {
+    // This is EVIL, we shouldn't be messing with event delivery just to get
+    // thumb mouse drag events to arrive at the slider!
+    return aLists.Outlines()->AppendNewToTop(new (aBuilder)
+        nsDisplayEventReceiver(this));
+  }
+  
+  return nsBoxFrame::BuildDisplayList(aBuilder, aDirtyRect, aLists);
+}
+
+NS_IMETHODIMP
+nsSliderFrame::BuildDisplayListForChildren(nsDisplayListBuilder*   aBuilder,
+                                           const nsRect&           aDirtyRect,
+                                           const nsDisplayListSet& aLists)
 {
   // if we are too small to have a thumb don't paint it.
   nsIBox* thumb;
@@ -308,15 +313,10 @@ nsSliderFrame::Paint(nsPresContext*      aPresContext,
     GetClientRect(crect);
 
     if (crect.width < thumbRect.width || crect.height < thumbRect.height)
-    {
-      if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
-        PaintSelf(aPresContext, aRenderingContext, aDirtyRect);
-      }
       return NS_OK;
-    }
   }
-
-  return nsBoxFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
+  
+  return nsBoxFrame::BuildDisplayListForChildren(aBuilder, aDirtyRect, aLists);
 }
 
 NS_IMETHODIMP
@@ -363,18 +363,22 @@ nsSliderFrame::DoLayout(nsBoxLayoutState& aState)
 
   // get our current position and max position from our content node
   PRInt32 curpospx = GetCurrentPosition(scrollbar);
+  PRInt32 minpospx = GetMinPosition(scrollbar);
   PRInt32 maxpospx = GetMaxPosition(scrollbar);
   PRInt32 pageIncrement = GetPageIncrement(scrollbar);
 
-  if (curpospx < 0)
-     curpospx = 0;
+  if (maxpospx < minpospx)
+    maxpospx = minpospx;
+
+  if (curpospx < minpospx)
+     curpospx = minpospx;
   else if (curpospx > maxpospx)
      curpospx = maxpospx;
 
   nscoord onePixel = aState.PresContext()->IntScaledPixelsToTwips(1);
 
   // get max pos in twips
-  nscoord maxpos = maxpospx*onePixel;
+  nscoord maxpos = (maxpospx - minpospx) * onePixel;
 
   // get our maxpos in twips. This is the space we have left over in the scrollbar
   // after the height of the thumb has been removed
@@ -385,7 +389,7 @@ nsSliderFrame::DoLayout(nsBoxLayoutState& aState)
 
   mRatio = 1;
 
-  if ((pageIncrement + maxpospx) != 0)
+  if ((pageIncrement + maxpospx - minpospx) > 0)
   {
     // if the thumb is flexible make the thumb bigger.
     nscoord flex = 0;
@@ -393,7 +397,7 @@ nsSliderFrame::DoLayout(nsBoxLayoutState& aState)
 
     if (flex > 0)
     {
-      mRatio = float(pageIncrement) / float(maxpospx + pageIncrement);
+      mRatio = float(pageIncrement) / float(maxpospx - minpospx + pageIncrement);
       nscoord thumbsize = NSToCoordRound(ourmaxpos * mRatio);
 
       // if there is more room than the thumb needs stretch the thumb
@@ -404,9 +408,9 @@ nsSliderFrame::DoLayout(nsBoxLayoutState& aState)
 
   ourmaxpos -= thumbcoord;
   if (float(maxpos) != 0)
-    mRatio = float(ourmaxpos)/float(maxpos);
+    mRatio = float(ourmaxpos) / float(maxpos);
 
-  nscoord curpos = curpospx*onePixel;
+  nscoord curpos = (curpospx - minpospx) * onePixel;
 
   // set the thumbs y coord to be the current pos * the ratio.
   nscoord pos = nscoord(float(curpos)*mRatio);
@@ -425,8 +429,9 @@ nsSliderFrame::DoLayout(nsBoxLayoutState& aState)
 
 #ifdef DEBUG_SLIDER
   PRInt32 c = GetCurrentPosition(scrollbar);
-  PRInt32 m = GetMaxPosition(scrollbar);
-  printf("Current=%d, max=%d\n", c, m);
+  PRInt32 min = GetMinPosition(scrollbar);
+  PRInt32 max = GetMaxPosition(scrollbar);
+  printf("Current=%d, min=%d, max=%d\n", c, min, max);
 #endif
 
   // redraw only if thumb changed size.
@@ -455,28 +460,43 @@ nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
 
     switch (aEvent->message) {
     case NS_MOUSE_MOVE: {
-       // convert coord to pixels
-      nscoord pos = isHorizontal ? aEvent->point.x : aEvent->point.y;
+      nsPoint eventPoint = nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent,
+                                                                         this);
+      if (mChange) {
+        // We're in the process of moving the thumb to the mouse,
+        // but the mouse just moved.  Make sure to update our
+        // destination point.
+        mDestinationPoint = eventPoint;
+        nsRepeatService::GetInstance()->Stop();
+        nsRepeatService::GetInstance()->Start(mMediator);
+        break;
+      }
+
+       nscoord pos = isHorizontal ? eventPoint.x : eventPoint.y;
 
        nscoord onePixel = aPresContext->IntScaledPixelsToTwips(1);
 
        nsIFrame* thumbFrame = mFrames.FirstChild();
 
-       // take our current position and substract the start location
+       // take our current position and subtract the start location
        pos -= mDragStart;
        PRBool isMouseOutsideThumb = PR_FALSE;
        if (gSnapMultiplier) {
          nsSize thumbSize = thumbFrame->GetSize();
          if (isHorizontal) {
            // horizontal scrollbar - check if mouse is above or below thumb
-           if (aEvent->point.y < (-gSnapMultiplier * thumbSize.height) || 
-               aEvent->point.y > thumbSize.height + (gSnapMultiplier * thumbSize.height))
+           // XXXbz what about looking at the .y of the thumb's rect?  Is that
+           // always zero here?
+           if (eventPoint.y < -gSnapMultiplier * thumbSize.height ||
+               eventPoint.y > thumbSize.height +
+                                gSnapMultiplier * thumbSize.height)
              isMouseOutsideThumb = PR_TRUE;
          }
          else {
            // vertical scrollbar - check if mouse is left or right of thumb
-           if (aEvent->point.x < (-gSnapMultiplier * thumbSize.width) || 
-               aEvent->point.x > thumbSize.width + (gSnapMultiplier * thumbSize.width))
+           if (eventPoint.x < -gSnapMultiplier * thumbSize.width ||
+               eventPoint.x > thumbSize.width +
+                                gSnapMultiplier * thumbSize.width)
              isMouseOutsideThumb = PR_TRUE;
          }
        }
@@ -494,6 +514,14 @@ nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
        // convert to our internal coordinate system
        pospx = nscoord(pospx/mRatio);
 
+       // if snap="true", then the slider may only be set to min + (increment * x).
+       // Otherwise, the slider may be set to any positive integer.
+       if (mContent->AttrValueIs(kNameSpaceID_None, nsXULAtoms::snap,
+                                 nsXULAtoms::_true, eCaseMatters)) {
+         PRInt32 increment = GetIncrement(scrollbar);
+         pospx = NSToCoordRound(pospx / (float)increment) * increment;
+       }
+
        // set it
        SetCurrentPosition(scrollbar, thumbFrame, pospx, PR_FALSE);
 
@@ -509,6 +537,10 @@ nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
       //printf("stop capturing\n");
       AddListener();
       DragThumb(PR_FALSE);
+      if (mChange) {
+        nsRepeatService::GetInstance()->Stop();
+        mChange = 0;
+      }
       mRedrawImmediate = PR_FALSE;//we MUST call nsFrame HandleEvent for mouse ups to maintain the selection state and capture state.
       return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
     }
@@ -523,7 +555,10 @@ nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
   else if ((aEvent->message == NS_MOUSE_LEFT_BUTTON_DOWN && ((nsMouseEvent *)aEvent)->isShift)
       || (gMiddlePref && aEvent->message == NS_MOUSE_MIDDLE_BUTTON_DOWN)) {
     // convert coord from twips to pixels
-    nscoord pos = isHorizontal ? aEvent->point.x : aEvent->point.y;
+    nsPoint eventPoint = nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent,
+                                                                      this);
+    nscoord pos = isHorizontal ? eventPoint.x : eventPoint.y;
+
     nscoord onePixel = aPresContext->IntScaledPixelsToTwips(1);
     nscoord pospx = pos/onePixel;
 
@@ -550,18 +585,12 @@ nsSliderFrame::HandleEvent(nsPresContext* aPresContext,
 
     mDragStart = pos - mThumbStart;
   }
-  else if (mChange != 0 && aEvent->message == NS_MOUSE_MOVE) {
-    // We're in the process of moving the thumb to the mouse,
-    // but the mouse just moved.  Make sure to update our
-    // destination point.
-    mDestinationPoint = aEvent->point;
-  }
 
   // XXX hack until handle release is actually called in nsframe.
 //  if (aEvent->message == NS_MOUSE_EXIT_SYNTH || aEvent->message == NS_MOUSE_RIGHT_BUTTON_UP || aEvent->message == NS_MOUSE_LEFT_BUTTON_UP)
   //   HandleRelease(aPresContext, aEvent, aEventStatus);
 
-  if (aEvent->message == NS_MOUSE_EXIT_SYNTH || aEvent->message == NS_MOUSE_LEFT_BUTTON_UP)
+  if (aEvent->message == NS_MOUSE_EXIT_SYNTH && mChange)
      HandleRelease(aPresContext, aEvent, aEventStatus);
 
   return nsFrame::HandleEvent(aPresContext, aEvent, aEventStatus);
@@ -598,7 +627,8 @@ nsSliderFrame::PageUpDown(nsIFrame* aThumbFrame, nscoord change)
 
   nscoord pageIncrement = GetPageIncrement(scrollbar);
   PRInt32 curpos = GetCurrentPosition(scrollbar);
-  SetCurrentPosition(scrollbar, aThumbFrame, curpos + change*pageIncrement, PR_TRUE);
+  PRInt32 minpos = GetMinPosition(scrollbar);
+  SetCurrentPosition(scrollbar, aThumbFrame, curpos - minpos + change*pageIncrement, PR_TRUE);
 }
 
 // called when the current position changed and we need to update the thumb's location
@@ -618,17 +648,18 @@ nsSliderFrame::CurrentPositionChanged(nsPresContext* aPresContext)
     if (mCurPos == curpos)
         return NS_OK;
 
-    // get our current position and max position from our content node
+    // get our current min and max position from our content node
+    PRInt32 minpos = GetMinPosition(scrollbar);
     PRInt32 maxpos = GetMaxPosition(scrollbar);
 
-    if (curpos < 0)
-      curpos = 0;
-         else if (curpos > maxpos)
+    if (curpos < minpos || maxpos < minpos)
+      curpos = minpos;
+    else if (curpos > maxpos)
       curpos = maxpos;
 
     // convert to pixels
     nscoord onePixel = aPresContext->IntScaledPixelsToTwips(1);
-    nscoord curpospx = curpos*onePixel;
+    nscoord curpospx = (curpos - minpos) * onePixel;
 
     // get the thumb's rect
     nsIFrame* thumbFrame = mFrames.FirstChild();
@@ -651,9 +682,11 @@ nsSliderFrame::CurrentPositionChanged(nsPresContext* aPresContext)
     // set the rect
     thumbFrame->SetRect(newThumbRect);
 
-    // figure out the union of the rect so we know what to redraw
+    // Figure out the union of the rect so we know what to redraw.
+    // Combine the old and new thumb overflow areas.
     nsRect changeRect;
-    changeRect.UnionRect(thumbRect, newThumbRect);
+    changeRect.UnionRect(thumbFrame->GetOverflowRect() + thumbRect.TopLeft(),
+                         thumbFrame->GetOverflowRect() + newThumbRect.TopLeft());
 
     // redraw just the change
     Invalidate(changeRect, mRedrawImmediate);
@@ -679,23 +712,27 @@ static void UpdateAttribute(nsIContent* aScrollbar, nscoord aNewPos, PRBool aNot
   }
 }
 
+// newpos should be passed to this function as a position as if the minpos is 0.
+// That is, the minpos will be added to the position by this function.
 void
 nsSliderFrame::SetCurrentPosition(nsIContent* scrollbar, nsIFrame* aThumbFrame, nscoord newpos, PRBool aIsSmooth)
 {
 
-   // get our current position and max position from our content node
+   // get our current, min and max position from our content node
+  PRInt32 minpos = GetMinPosition(scrollbar);
   PRInt32 maxpos = GetMaxPosition(scrollbar);
 
+  newpos += minpos;
+
   // get the new position and make sure it is in bounds
-  if (newpos > maxpos)
+  if (newpos < minpos || maxpos < minpos)
+      newpos = minpos;
+  else if (newpos > maxpos)
       newpos = maxpos;
-  else if (newpos < 0)
-      newpos = 0;
 
   nsIBox* scrollbarBox = GetScrollbar();
   nsCOMPtr<nsIScrollbarFrame> scrollbarFrame(do_QueryInterface(scrollbarBox));
 
-
   if (scrollbarFrame) {
     // See if we have a mediator.
     nsCOMPtr<nsIScrollbarMediator> mediator;
@@ -716,40 +753,11 @@ nsSliderFrame::SetCurrentPosition(nsIContent* scrollbar, nsIFrame* aThumbFrame,
 
 }
 
-NS_IMETHODIMP  nsSliderFrame::GetFrameForPoint(nsPresContext* aPresContext,
-                                             const nsPoint& aPoint,
-                                             nsFramePaintLayer aWhichLayer,
-                                             nsIFrame**     aFrame)
-{
-  // This is EVIL, we shouldn't be messing with GetFrameForPoint just to get
-  // thumb mouse drag events to arrive at the slider!
-  if (isDraggingThumb())
-  {
-    // XXX I assume it's better not to test for visibility here.
-    *aFrame = this;
-    return NS_OK;
-  }
-
-  if (NS_SUCCEEDED(nsBoxFrame::GetFrameForPoint(aPresContext, aPoint, aWhichLayer, aFrame)))
-    return NS_OK;
-
-  // always return us (if visible)
-  if (mRect.Contains(aPoint) && GetStyleVisibility()->IsVisible()) {
-    *aFrame = this;
-    return NS_OK;
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-
-
 NS_IMETHODIMP
-nsSliderFrame::SetInitialChildList(nsPresContext* aPresContext,
-                                              nsIAtom*        aListName,
-                                              nsIFrame*       aChildList)
+nsSliderFrame::SetInitialChildList(nsIAtom*        aListName,
+                                   nsIFrame*       aChildList)
 {
-  nsresult r = nsBoxFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+  nsresult r = nsBoxFrame::SetInitialChildList(aListName, aChildList);
 
   AddListener();
 
@@ -781,6 +789,10 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
 {
   //printf("Begin dragging\n");
 
+  if (mContent->AttrValueIs(kNameSpaceID_None, nsHTMLAtoms::disabled,
+                            nsXULAtoms::_true, eCaseMatters))
+    return NS_OK;
+
   PRBool isHorizontal = IsHorizontal();
 
   nsCOMPtr<nsIDOMMouseEvent> mouseEvent(do_QueryInterface(aMouseEvent));
@@ -795,10 +807,18 @@ nsSliderFrame::MouseDown(nsIDOMEvent* aMouseEvent)
     scrollToClick = PR_TRUE;
   }
 
-  nsCOMPtr<nsIPrivateDOMEvent> domEvent(do_QueryInterface(aMouseEvent));
-  nsEvent *event;
-  domEvent->GetInternalNSEvent(&event);
-  nscoord pos = isHorizontal ? event->point.x : event->point.y;
+  PRInt32 clientPosPx;
+  nsIntRect screenRect = GetScreenRect();
+  if (isHorizontal) {
+    mouseEvent->GetScreenX(&clientPosPx);
+    clientPosPx -= screenRect.x;
+  } else {
+    mouseEvent->GetScreenY(&clientPosPx);
+    clientPosPx -= screenRect.y;
+  }
+
+  nsPresContext* presContext = GetPresContext();
+  nscoord pos = presContext->IntScaledPixelsToTwips(clientPosPx);
 
   // If shift click or middle button, first
   // place the middle of the slider thumb under the click
@@ -931,16 +951,23 @@ nsSliderFrame::HandlePress(nsPresContext* aPresContext,
   nsIFrame* thumbFrame = mFrames.FirstChild();
   if (!thumbFrame) // display:none?
     return NS_OK;
+
+  if (mContent->AttrValueIs(kNameSpaceID_None, nsHTMLAtoms::disabled,
+                            nsXULAtoms::_true, eCaseMatters))
+    return NS_OK;
   
   nsRect thumbRect = thumbFrame->GetRect();
   
   nscoord change = 1;
-  if (IsHorizontal() ? aEvent->point.x < thumbRect.x 
-                     : aEvent->point.y < thumbRect.y)
+  nsPoint eventPoint = nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent,
+                                                                    this);
+  if (IsHorizontal() ? eventPoint.x < thumbRect.x 
+                     : eventPoint.y < thumbRect.y)
     change = -1;
 
   mChange = change;
-  mDestinationPoint = aEvent->point;
+  DragThumb(PR_TRUE);
+  mDestinationPoint = eventPoint;
   PageUpDown(thumbFrame, change);
   
   nsRepeatService::GetInstance()->Start(mMediator);
@@ -955,13 +982,11 @@ nsSliderFrame::HandleRelease(nsPresContext* aPresContext,
 {
   nsRepeatService::GetInstance()->Stop();
 
-  mChange = 0;
-
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsSliderFrame::Destroy(nsPresContext* aPresContext)
+void
+nsSliderFrame::Destroy()
 {
   // tell our mediator if we have one we are gone.
   if (mMediator) {
@@ -971,7 +996,7 @@ nsSliderFrame::Destroy(nsPresContext* aPresContext)
   }
 
   // call base class Destroy()
-  return nsBoxFrame::Destroy(aPresContext);
+  nsBoxFrame::Destroy();
 }
 
 NS_IMETHODIMP
