@@ -71,13 +71,14 @@
 #include "nsIXBLService.h"
 #include "nsIServiceManager.h"
 #include "nsHTMLContainerFrame.h"
-#include "nsINodeInfo.h"
 #include "nsGUIEvent.h"
 #include "nsAutoPtr.h"
 #include "nsContentCID.h"
 #include "nsStyleSet.h"
+#include "nsLayoutUtils.h"
+#include "nsDisplayList.h"
+#include "nsContentUtils.h"
 
-const PRInt32 kMaxZ = 0x7fffffff; //XXX: Shouldn't there be a define somewhere for MaxInt for PRInt32
 // was used in nsSplitterFrame::Init but now commented out
 //static NS_DEFINE_IID(kLookAndFeelCID,  NS_LOOKANDFEEL_CID);
 PRInt32 realTimeDrag;
@@ -88,7 +89,7 @@ public:
   nscoord max;
   nscoord current;
   nscoord changed;
-  nsIBox* child;
+  nsCOMPtr<nsIContent> childElem;
   PRInt32 flex;
   PRInt32 index;
 };
@@ -144,8 +145,8 @@ public:
   void RemoveListener();
 
   enum ResizeType { Closest, Farthest, Grow };
-  enum State { Open, Collapsed, Dragging };
-  enum CollapseDirection { Before, After, None };
+  enum State { Open, CollapsedBefore, CollapsedAfter, Dragging };
+  enum CollapseDirection { Before, After };
 
   ResizeType GetResizeBefore();
   ResizeType GetResizeAfter();
@@ -154,7 +155,7 @@ public:
   //nsresult CaptureMouse(nsPresContext* aPresContext, PRBool aGrabMouseEvents);
   //PRBool IsMouseCaptured(nsPresContext* aPresContext);
   void Reverse(nsSplitterInfo*& aIndexes, PRInt32 aCount);
-  CollapseDirection GetCollapseDirection();
+  PRBool SupportsCollapseDirection(CollapseDirection aDirection);
 
   void MoveSplitterBy(nsPresContext* aPresContext, nscoord aDiff);
   void EnsureOrient();
@@ -162,7 +163,7 @@ public:
 
   nsSplitterFrame* mOuter;
   PRBool mDidDrag;
-  nscoord mDragStartPx;
+  nscoord mDragStart;
   nscoord mCurrentPos;
   nsIBox* mParentBox;
   PRBool mPressed;
@@ -183,10 +184,9 @@ NS_IMPL_ISUPPORTS2(nsSplitterFrameInner, nsIDOMMouseListener, nsIDOMMouseMotionL
 nsSplitterFrameInner::ResizeType
 nsSplitterFrameInner::GetResizeBefore()
 {
-  nsString value;
-  mOuter->GetContent()->GetAttr(kNameSpaceID_None,
-                                nsXULAtoms::resizebefore, value);
-  if (value.EqualsLiteral("farthest"))
+  if (mOuter->GetContent()->
+        AttrValueIs(kNameSpaceID_None, nsXULAtoms::resizebefore,
+                    NS_LITERAL_STRING("farthest"), eCaseMatters))
     return Farthest;
   return Closest;
 }
@@ -200,52 +200,57 @@ nsSplitterFrameInner::~nsSplitterFrameInner()
 nsSplitterFrameInner::ResizeType
 nsSplitterFrameInner::GetResizeAfter()
 {
-  nsString value;
-  mOuter->GetContent()->GetAttr(kNameSpaceID_None,
-                                nsXULAtoms::resizeafter, value);
-  if (value.EqualsLiteral("farthest"))
-    return Farthest;
-  if (value.EqualsLiteral("grow"))
-    return Grow;
+  static nsIContent::AttrValuesArray strings[] =
+    {&nsXULAtoms::farthest, &nsXULAtoms::grow, nsnull};
+  switch (mOuter->GetContent()->FindAttrValueIn(kNameSpaceID_None,
+                                                nsXULAtoms::resizeafter,
+                                                strings, eCaseMatters)) {
+    case 0: return Farthest;
+    case 1: return Grow;
+  }
   return Closest;
 }
 
 nsSplitterFrameInner::State
 nsSplitterFrameInner::GetState()
 {
-  nsString value;
-  mOuter->GetContent()->GetAttr(kNameSpaceID_None,
-                                nsXULAtoms::state, value);
-  if (value.EqualsLiteral("dragging"))
-    return Dragging;
-  if (value.EqualsLiteral("collapsed"))
-    return Collapsed;
+  static nsIContent::AttrValuesArray strings[] =
+    {&nsXULAtoms::dragging, &nsXULAtoms::collapsed, nsnull};
+  static nsIContent::AttrValuesArray strings_substate[] =
+    {&nsXULAtoms::before, &nsXULAtoms::after, nsnull};
+  switch (mOuter->GetContent()->FindAttrValueIn(kNameSpaceID_None,
+                                                nsXULAtoms::state,
+                                                strings, eCaseMatters)) {
+    case 0: return Dragging;
+    case 1:
+      switch (mOuter->GetContent()->FindAttrValueIn(kNameSpaceID_None,
+                                                    nsXULAtoms::substate,
+                                                    strings_substate,
+                                                    eCaseMatters)) {
+        case 0: return CollapsedBefore;
+        case 1: return CollapsedAfter;
+        default:
+          if (SupportsCollapseDirection(After))
+            return CollapsedAfter;
+          return CollapsedBefore;
+      }
+  }
   return Open;
 }
 
 //
 // NS_NewSplitterFrame
 //
-// Creates a new Toolbar frame and returns it in |aNewFrame|
+// Creates a new Toolbar frame and returns it
 //
-nsresult
-NS_NewSplitterFrame ( nsIPresShell* aPresShell, nsIFrame** aNewFrame )
+nsIFrame*
+NS_NewSplitterFrame (nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
-  NS_PRECONDITION(aNewFrame, "null OUT ptr");
-  if (nsnull == aNewFrame) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  nsSplitterFrame* it = new (aPresShell) nsSplitterFrame(aPresShell);
-  if (nsnull == it)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  *aNewFrame = it;
-  return NS_OK;
-  
+  return new (aPresShell) nsSplitterFrame(aPresShell, aContext);
 } // NS_NewSplitterFrame
 
-nsSplitterFrame::nsSplitterFrame(nsIPresShell* aPresShell)
-: nsBoxFrame(aPresShell),
+nsSplitterFrame::nsSplitterFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
+: nsBoxFrame(aPresShell, aContext),
   mInner(0)
 {
 }
@@ -267,11 +272,10 @@ NS_INTERFACE_MAP_END_INHERITING(nsBoxFrame)
 
 
 NS_IMETHODIMP
-nsSplitterFrame::GetCursor(nsPresContext* aPresContext,
-                                     nsPoint&        aPoint,
-                                     PRInt32&        aCursor)
+nsSplitterFrame::GetCursor(const nsPoint&    aPoint,
+                           nsIFrame::Cursor& aCursor)
 {
-  return nsBoxFrame::GetCursor(aPresContext, aPoint, aCursor);
+  return nsBoxFrame::GetCursor(aPoint, aCursor);
 
   /*
     if (IsHorizontal())
@@ -284,23 +288,20 @@ nsSplitterFrame::GetCursor(nsPresContext* aPresContext,
 }
 
 NS_IMETHODIMP
-nsSplitterFrame::AttributeChanged(nsPresContext* aPresContext,
-                                  nsIContent* aChild,
-                                  PRInt32 aNameSpaceID,
+nsSplitterFrame::AttributeChanged(PRInt32 aNameSpaceID,
                                   nsIAtom* aAttribute,
                                   PRInt32 aModType)
 {
-  nsresult rv = nsBoxFrame::AttributeChanged(aPresContext, aChild,
-                                              aNameSpaceID, aAttribute,
-                                              aModType);
+  nsresult rv = nsBoxFrame::AttributeChanged(aNameSpaceID, aAttribute,
+                                             aModType);
   // if the alignment changed. Let the grippy know
   if (aAttribute == nsHTMLAtoms::align) {
     // tell the slider its attribute changed so it can 
     // update itself
     nsIFrame* grippy = nsnull;
-    nsScrollbarButtonFrame::GetChildWithTag(aPresContext, nsXULAtoms::grippy, this, grippy);
+    nsScrollbarButtonFrame::GetChildWithTag(GetPresContext(), nsXULAtoms::grippy, this, grippy);
     if (grippy)
-      grippy->AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType);
+      grippy->AttributeChanged(aNameSpaceID, aAttribute, aModType);
   } else if (aAttribute == nsXULAtoms::state) {
     mInner->UpdateState();
   }
@@ -312,11 +313,9 @@ nsSplitterFrame::AttributeChanged(nsPresContext* aPresContext,
  * Initialize us. If we are in a box get our alignment so we know what direction we are
  */
 NS_IMETHODIMP
-nsSplitterFrame::Init(nsPresContext*  aPresContext,
-              nsIContent*      aContent,
-              nsIFrame*        aParent,
-              nsStyleContext*  aContext,
-              nsIFrame*        aPrevInFlow)
+nsSplitterFrame::Init(nsIContent*      aContent,
+                      nsIFrame*        aParent,
+                      nsIFrame*        aPrevInFlow)
 {
   NS_ENSURE_FALSE(mInner, NS_ERROR_ALREADY_INITIALIZED);
   mInner = new nsSplitterFrameInner(this);
@@ -349,30 +348,25 @@ nsSplitterFrame::Init(nsPresContext*  aPresContext,
     PRBool isHorizontal;
     aParent->GetOrientation(isHorizontal);
     if (!isHorizontal) {
-      nsAutoString str;
-      aContent->GetAttr(kNameSpaceID_None, nsXULAtoms::orient, str);
-      if (str.IsEmpty()) {
+      if (!nsContentUtils::HasNonEmptyAttr(aContent, kNameSpaceID_None,
+                                           nsXULAtoms::orient)) {
         aContent->SetAttr(kNameSpaceID_None, nsXULAtoms::orient,
                           NS_LITERAL_STRING("vertical"), PR_FALSE);
-        nsStyleContext* parent = aContext->GetParent();
-        newContext = aPresContext->StyleSet()->
-          ResolveStyleFor(aContent, parent);
-        aContext = newContext;
+        nsStyleContext* parentStyleContext = aParent->GetStyleContext();
+        newContext = GetStyleContext()->GetRuleNode()->GetPresContext()->StyleSet()->
+          ResolveStyleFor(aContent, parentStyleContext);
+        SetStyleContextWithoutNotification(newContext);
       }
     }
   }
 
-  nsresult  rv = nsBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
-
-  // XXX Hack because we need the pres context in some of the event handling functions...
-  mPresContext = aPresContext; 
+  nsresult  rv = nsBoxFrame::Init(aContent, aParent, aPrevInFlow);
 
   nsHTMLContainerFrame::CreateViewForFrame(this, nsnull, PR_TRUE);
   nsIView* view = GetView();
 
   nsIViewManager* viewManager = view->GetViewManager();
   viewManager->SetViewContentTransparency(view, PR_TRUE);
-  viewManager->SetViewZIndex(view, PR_FALSE, kMaxZ);
 
   if (!realTimeDrag) {
     // currently this only works on win32 and mac
@@ -383,7 +377,7 @@ nsSplitterFrame::Init(nsPresContext*  aPresContext,
   }
 
   mInner->mState = nsSplitterFrameInner::Open;
-  mInner->AddListener(aPresContext);
+  mInner->AddListener(GetPresContext());
   mInner->mParentBox = nsnull;
   return rv;
 }
@@ -447,29 +441,23 @@ nsSplitterFrame::HandleRelease(nsPresContext* aPresContext,
   return NS_OK;
 }
 
-NS_IMETHODIMP  nsSplitterFrame::GetFrameForPoint(nsPresContext* aPresContext,
-                                             const nsPoint& aPoint, 
-                                             nsFramePaintLayer aWhichLayer,    
-                                             nsIFrame**     aFrame)
-{   
+NS_IMETHODIMP
+nsSplitterFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                  const nsRect&           aDirtyRect,
+                                  const nsDisplayListSet& aLists)
+{
+  nsresult rv = nsBoxFrame::BuildDisplayList(aBuilder, aDirtyRect, aLists);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
   // if the mouse is captured always return us as the frame.
   if (mInner->mDragging)
   {
     // XXX It's probably better not to check visibility here, right?
-    *aFrame = this;
-    return NS_OK;
+    return aLists.Outlines()->AppendNewToTop(new (aBuilder)
+        nsDisplayEventReceiver(this));
   }
 
-  nsresult rv = nsBoxFrame::GetFrameForPoint(aPresContext, aPoint, aWhichLayer, aFrame);
-
-  if (NS_FAILED(rv) &&
-      aWhichLayer == NS_FRAME_PAINT_LAYER_FOREGROUND &&
-      mRect.Contains(aPoint)) {
-    *aFrame = this;
-    rv = NS_OK;
-  } 
-
-  return rv;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -513,6 +501,11 @@ nsSplitterFrameInner::MouseUp(nsPresContext* aPresContext, nsGUIEvent* aEvent)
 
     //printf("MouseUp\n");
   }
+
+  delete[] mChildInfosBefore;
+  delete[] mChildInfosAfter;
+  mChildInfosBefore = nsnull;
+  mChildInfosAfter = nsnull;
 }
 
 void
@@ -524,41 +517,14 @@ nsSplitterFrameInner::MouseDrag(nsPresContext* aPresContext, nsGUIEvent* aEvent)
 
     PRBool isHorizontal = !mOuter->IsHorizontal();
     // convert coord to pixels
-    nscoord pos = isHorizontal ? aEvent->point.x : aEvent->point.y;
+    nsPoint pt = nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent,
+                                                              mParentBox);
+    nscoord pos = isHorizontal ? pt.x : pt.y;
 
-    // mDragStartPx is in pixels and is in our client areas coordinate system. 
-    // so we need to first convert it so twips and then get it into our coordinate system.
+    // mDragStart is in frame coordinates
+    nscoord start = mDragStart;
 
-    // convert start to twips
-    nscoord startpx = mDragStartPx;
-
-    nscoord onePixel = aPresContext->IntScaledPixelsToTwips(1);
-    nscoord start = startpx*onePixel;
-
-    // get it into our coordintate system by subtracting our parents offsets.
-    nsIFrame* parent = mOuter;
-    while(parent != nsnull)
-    {
-      // if we hit a scrollable view make sure we take into account
-      // how much we are scrolled.
-      nsIView* view = parent->GetView();
-      if (view) {
-        nsIScrollableView* scrollingView;
-        nsresult result = CallQueryInterface(view, &scrollingView);
-        if (NS_SUCCEEDED(result)) {
-          nscoord xoff = 0;
-          nscoord yoff = 0;
-          scrollingView->GetScrollPosition(xoff, yoff);
-          isHorizontal ? start += xoff : start += yoff;
-        }
-      }
-
-      nsRect r = parent->GetRect();
-      isHorizontal ? start -= r.x : start -= r.y;
-      parent = parent->GetParent();
-    }
-
-    // take our current position and substract the start location
+    // take our current position and subtract the start location
     pos -= start;
 
     //printf("Diff=%d\n", pos);
@@ -584,27 +550,41 @@ nsSplitterFrameInner::MouseDrag(nsPresContext* aPresContext, nsGUIEvent* aEvent)
     ResizeChildTo(aPresContext, pos, mChildInfosBefore, mChildInfosAfter, mChildInfosBeforeCount, mChildInfosAfterCount, bounded);
 
     State currentState = GetState();
-    CollapseDirection dir = GetCollapseDirection();
+    PRBool supportsBefore = SupportsCollapseDirection(Before);
+    PRBool supportsAfter = SupportsCollapseDirection(After);
 
     // if we are in a collapsed position
-    if (realTimeDrag && ((oldPos > 0 && oldPos > pos && dir == After) || (oldPos < 0 && oldPos < pos && dir == Before)))
+    if (realTimeDrag && ((oldPos > 0 && oldPos > pos && supportsAfter) ||
+                         (oldPos < 0 && oldPos < pos && supportsBefore)))
     {
       // and we are not collapsed then collapse
       if (currentState == Dragging) {
         if (oldPos > 0 && oldPos > pos)
         {
           //printf("Collapse right\n");
-          if (GetCollapseDirection() == After) 
+          if (supportsAfter) 
           {
-            mOuter->mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::state, NS_LITERAL_STRING("collapsed"), PR_TRUE);
+            nsCOMPtr<nsIContent> outer = mOuter->mContent;
+            outer->SetAttr(kNameSpaceID_None, nsXULAtoms::substate,
+                           NS_LITERAL_STRING("after"),
+                           PR_TRUE);
+            outer->SetAttr(kNameSpaceID_None, nsXULAtoms::state,
+                           NS_LITERAL_STRING("collapsed"),
+                           PR_TRUE);
           }
 
         } else if (oldPos < 0 && oldPos < pos)
         {
           //printf("Collapse left\n");
-          if (GetCollapseDirection() == Before) 
+          if (supportsBefore)
           {
-            mOuter->mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::state, NS_LITERAL_STRING("collapsed"), PR_TRUE);
+            nsCOMPtr<nsIContent> outer = mOuter->mContent;
+            outer->SetAttr(kNameSpaceID_None, nsXULAtoms::substate,
+                           NS_LITERAL_STRING("before"),
+                           PR_TRUE);
+            outer->SetAttr(kNameSpaceID_None, nsXULAtoms::state,
+                           NS_LITERAL_STRING("collapsed"),
+                           PR_TRUE);
           }
         }
       }
@@ -630,11 +610,8 @@ nsSplitterFrameInner::MouseDrag(nsPresContext* aPresContext, nsGUIEvent* aEvent)
     /*
       nsIPresShell *shell = aPresContext->PresShell();
 
-      nsCOMPtr<nsHTMLReflowCommand> reflowCmd;
-      nsresult rv = NS_NewHTMLReflowCommand(getter_AddRefs(reflowCmd), mOuter->mParent,
-                                            eReflowType_StyleChanged);
-      if (NS_SUCCEEDED(rv)) 
-        shell->AppendReflowCommand(reflowCmd);
+      shell->AppendReflowCommand(mOuter->mParent, eReflowType_StyleChanged,
+                                 nsnull);
      
       mOuter->mState |= NS_FRAME_IS_DIRTY;
       mOuter->mParent->ReflowDirtyChild(shell, mOuter);
@@ -717,7 +694,7 @@ nsSplitterFrameInner::MouseUp(nsIDOMEvent* aMouseEvent)
 {  
   mPressed = PR_FALSE;
 
-  mOuter->CaptureMouse(mOuter->mPresContext, PR_FALSE);
+  mOuter->CaptureMouse(mOuter->GetPresContext(), PR_FALSE);
 
   return NS_OK;
 }
@@ -734,30 +711,30 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
   if (button != 0)
      return NS_OK;
 
-  nsAutoString disabled;
-  mOuter->GetContent()->GetAttr(kNameSpaceID_None,
-                                nsHTMLAtoms::disabled, disabled);
-  if (disabled.EqualsLiteral("true"))
+  if (mOuter->GetContent()->
+        AttrValueIs(kNameSpaceID_None, nsHTMLAtoms::disabled,
+                    nsXULAtoms::_true, eCaseMatters))
     return NS_OK;
 
-  nsBoxLayoutState state(mOuter->mPresContext);
+  mOuter->GetParentBox(&mParentBox);
+  if (!mParentBox)
+    return NS_OK;
+  
+  // get our index
+  nsPresContext* outerPresContext = mOuter->GetPresContext();
+  nscoord childIndex = nsFrameNavigator::IndexOf(outerPresContext, mParentBox, mOuter);
+  PRInt32 childCount = nsFrameNavigator::CountFrames(outerPresContext, mParentBox);
+
+  // if it's 0 or the last index then stop right here.
+  if (childIndex == 0 || childIndex == childCount - 1)
+    return NS_OK;
+
+  nsBoxLayoutState state(outerPresContext);
   mCurrentPos = 0;
   mPressed = PR_TRUE;
 
   mDidDrag = PR_FALSE;
 
-  mOuter->GetParentBox(&mParentBox);
-  
-  // get our index
-  nscoord childIndex = nsFrameNavigator::IndexOf(mOuter->mPresContext, mParentBox, mOuter);
-  PRInt32 childCount = nsFrameNavigator::CountFrames(mOuter->mPresContext, mParentBox);
-
-  // if its 0 or the last index then stop right here.
-  if (childIndex == 0 || childIndex == childCount-1) {
-    mPressed = PR_FALSE;
-    return NS_OK;
-  }
-
   EnsureOrient();
   PRBool isHorizontal = !mOuter->IsHorizontal();
   
@@ -766,7 +743,6 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
 
   delete[] mChildInfosBefore;
   delete[] mChildInfosAfter;
-
   mChildInfosBefore = new nsSplitterInfo[childCount];
   mChildInfosAfter  = new nsSplitterInfo[childCount];
 
@@ -815,16 +791,14 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
         nsRect r(childBox->GetRect());
         r.Inflate(margin);
 
-        nsAutoString fixed, hidden;
-        content->GetAttr(kNameSpaceID_None, nsXULAtoms::fixed, fixed);
         // We need to check for hidden attribute too, since treecols with
         // the hidden="true" attribute are not really hidden, just collapsed
-        content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::hidden, hidden);
-
-        NS_NAMED_LITERAL_STRING(attrTrue, "true");
-        if (!attrTrue.Equals(fixed) && !attrTrue.Equals(hidden)) {
+        if (!content->AttrValueIs(kNameSpaceID_None, nsXULAtoms::fixed,
+                                  nsXULAtoms::_true, eCaseMatters) &&
+            !content->AttrValueIs(kNameSpaceID_None, nsHTMLAtoms::hidden,
+                                  nsXULAtoms::_true, eCaseMatters)) {
             if (count < childIndex) {
-                mChildInfosBefore[mChildInfosBeforeCount].child   = childBox;
+                mChildInfosBefore[mChildInfosBeforeCount].childElem = content;
                 mChildInfosBefore[mChildInfosBeforeCount].min     = isHorizontal ? minSize.width : minSize.height;
                 mChildInfosBefore[mChildInfosBeforeCount].max     = isHorizontal ? maxSize.width : maxSize.height;
                 mChildInfosBefore[mChildInfosBeforeCount].current = isHorizontal ? r.width : r.height;
@@ -833,7 +807,7 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
                 mChildInfosBefore[mChildInfosBeforeCount].changed = mChildInfosBefore[mChildInfosBeforeCount].current;
                 mChildInfosBeforeCount++;
             } else if (count > childIndex) {
-                mChildInfosAfter[mChildInfosAfterCount].child   = childBox;
+                mChildInfosAfter[mChildInfosAfterCount].childElem = content;
                 mChildInfosAfter[mChildInfosAfterCount].min     = isHorizontal ? minSize.width : minSize.height;
                 mChildInfosAfter[mChildInfosAfterCount].max     = isHorizontal ? maxSize.width : maxSize.height;
                 mChildInfosAfter[mChildInfosAfterCount].current = isHorizontal ? r.width : r.height;
@@ -884,22 +858,24 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
 
   nsRect vr = mOuter->GetView()->GetBounds();
 
-  PRInt32 c = 0;
+  PRInt32 c;
+  nsPoint pt = nsLayoutUtils::GetDOMEventCoordinatesRelativeTo(mouseEvent,
+                                                               mParentBox);
   if (isHorizontal) {
-     mouseEvent->GetClientX(&c);
+     c = pt.x;
      mSplitterPos = mOuter->mRect.x;
      mSplitterViewPos = vr.x;
   } else {
-     mouseEvent->GetClientY(&c);
+     c = pt.y;
      mSplitterPos = mOuter->mRect.y;
      mSplitterViewPos = vr.y;
   }
 
-  mDragStartPx = c;
-    
-  //printf("Pressed mDragStartPx=%d\n",mDragStartPx);
+  mDragStart = c;
 
-  mOuter->CaptureMouse(mOuter->mPresContext, PR_TRUE);
+  //printf("Pressed mDragStart=%d\n",mDragStart);
+
+  mOuter->CaptureMouse(outerPresContext, PR_TRUE);
 
   return NS_OK;
 }
@@ -910,8 +886,8 @@ nsSplitterFrameInner::MouseMove(nsIDOMEvent* aMouseEvent)
   //printf("Mouse move\n");
 
   if (!mPressed)
-      return NS_OK;
-  
+    return NS_OK;
+
   if (mDragging)
     return NS_OK;
 
@@ -935,33 +911,43 @@ nsSplitterFrameInner::Reverse(nsSplitterInfo*& aChildInfos, PRInt32 aCount)
     aChildInfos = infos;
 }
 
-nsSplitterFrameInner::CollapseDirection
-nsSplitterFrameInner::GetCollapseDirection()
+PRBool
+nsSplitterFrameInner::SupportsCollapseDirection
+(
+  nsSplitterFrameInner::CollapseDirection aDirection
+)
 {
-    nsString value;
-    if (NS_CONTENT_ATTR_HAS_VALUE == mOuter->mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::collapse, value))
-    {
-     if (value.EqualsLiteral("before"))
-         return Before;
-     else if (value.EqualsLiteral("after"))
-         return After;
-     else 
-       return None;
-    } else {
-        return None;
-    }
+  static nsIContent::AttrValuesArray strings[] =
+    {&nsXULAtoms::before, &nsXULAtoms::after, &nsXULAtoms::both, nsnull};
+
+  switch (mOuter->mContent->FindAttrValueIn(kNameSpaceID_None,
+                                            nsXULAtoms::collapse,
+                                            strings, eCaseMatters)) {
+    case 0:
+      return (aDirection == Before);
+    case 1:
+      return (aDirection == After);
+    case 2:
+      return PR_TRUE;
+  }
+
+  return PR_FALSE;
 }
 
 void
 nsSplitterFrameInner::UpdateState()
 {
   // State Transitions:
-  //   Open      -> Dragging
-  //   Open      -> Collapsed
-  //   Collapsed -> Open
-  //   Collapsed -> Dragging
-  //   Dragging  -> Open
-  //   Dragging  -> Collapsed (auto collapse)
+  //   Open            -> Dragging
+  //   Open            -> CollapsedBefore
+  //   Open            -> CollapsedAfter
+  //   CollapsedBefore -> Open
+  //   CollapsedBefore -> Dragging
+  //   CollapsedAfter  -> Open
+  //   CollapsedAfter  -> Dragging
+  //   Dragging        -> Open
+  //   Dragging        -> CollapsedBefore (auto collapse)
+  //   Dragging        -> CollapsedAfter (auto collapse)
 
   State newState = GetState(); 
 
@@ -970,25 +956,28 @@ nsSplitterFrameInner::UpdateState()
     return;
   }
 
-  CollapseDirection direction = GetCollapseDirection();
-  if (direction != None) {
+  if (SupportsCollapseDirection(Before) || SupportsCollapseDirection(After)) {
     nsIBox* splitter = mOuter;
     // Find the splitter's immediate sibling.
     nsIBox* splitterSibling =
-      nsFrameNavigator::GetChildBeforeAfter(mOuter->mPresContext, splitter,
-                                            (direction == Before));
+      nsFrameNavigator::GetChildBeforeAfter(mOuter->GetPresContext(), splitter,
+                                            (newState == CollapsedBefore ||
+                                             mState == CollapsedBefore));
     if (splitterSibling) {
-      nsIContent* sibling = splitterSibling->GetContent();
+      nsCOMPtr<nsIContent> sibling = splitterSibling->GetContent();
       if (sibling) {
-        if (mState == Collapsed) {
-          // Collapsed -> Open
-          // Collapsed -> Dragging
+        if (mState == CollapsedBefore || mState == CollapsedAfter) {
+          // CollapsedBefore -> Open
+          // CollapsedBefore -> Dragging
+          // CollapsedAfter -> Open
+          // CollapsedAfter -> Dragging
           sibling->UnsetAttr(kNameSpaceID_None, nsXULAtoms::collapsed,
                              PR_TRUE);
-        } else if ((mState == Open || mState == Dragging) 
-                   && newState == Collapsed) {
-          // Open -> Collapsed
-          // Dragging -> Collapsed
+        } else if ((mState == Open || mState == Dragging)
+                   && (newState == CollapsedBefore ||
+                       newState == CollapsedAfter)) {
+          // Open -> CollapsedBefore / CollapsedAfter
+          // Dragging -> CollapsedBefore / CollapsedAfter
           sibling->SetAttr(kNameSpaceID_None, nsXULAtoms::collapsed,
                            NS_LITERAL_STRING("true"), PR_TRUE);
         }
@@ -1022,13 +1011,6 @@ nsSplitterFrameInner::AdjustChildren(nsPresContext* aPresContext)
 
    
   if (realTimeDrag) {
-    nsIView* view = mParentBox->GetView();
-
-    if (!view) {
-        nsPoint   offset;
-        mParentBox->GetOffsetFromView(aPresContext, offset, &view);
-        NS_ASSERTION(nsnull != view, "no view");
-    }
     aPresContext->PresShell()->FlushPendingNotifications(Flush_Display);
   }
   else {
@@ -1037,6 +1019,20 @@ nsSplitterFrameInner::AdjustChildren(nsPresContext* aPresContext)
   }
 }
 
+static nsIBox* GetChildBoxForContent(nsIBox* aParentBox, nsIContent* aContent)
+{
+  nsIBox* childBox = nsnull;
+  aParentBox->GetChildBox(&childBox); 
+
+  while (nsnull != childBox) {
+    if (childBox->GetContent() == aContent) {
+      return childBox;
+    }
+    childBox->GetNextBox(&childBox);
+  }
+  return nsnull;
+}
+
 void
 nsSplitterFrameInner::AdjustChildren(nsPresContext* aPresContext, nsSplitterInfo* aChildInfos, PRInt32 aCount, PRBool aIsHorizontal)
 {
@@ -1059,9 +1055,11 @@ nsSplitterFrameInner::AdjustChildren(nsPresContext* aPresContext, nsSplitterInfo
   for (int i=0; i < aCount; i++) 
   {
     nscoord   pref       = aChildInfos[i].changed;
-    nsIBox* childBox     = aChildInfos[i].child;
+    nsIBox* childBox     = GetChildBoxForContent(mParentBox, aChildInfos[i].childElem);
 
-    SetPreferredSize(state, childBox, onePixel, aIsHorizontal, &pref);
+    if (childBox) {
+      SetPreferredSize(state, childBox, onePixel, aIsHorizontal, &pref);
+    }
   }
 }
 
@@ -1108,12 +1106,13 @@ nsSplitterFrameInner::SetPreferredSize(nsBoxLayoutState& aState, nsIBox* aChildB
   // set its preferred size.
   nsAutoString prefValue;
   prefValue.AppendInt(pref/aOnePixel);
-  nsAutoString oldValue;
-  content->GetAttr(kNameSpaceID_None, attribute, oldValue);
-  if (oldValue.Equals(prefValue))
+  if (content->AttrValueIs(kNameSpaceID_None, attribute,
+                           prefValue, eCaseMatters))
      return;
 
+  nsWeakFrame weakBox(aChildBox);
   content->SetAttr(kNameSpaceID_None, attribute, prefValue, PR_TRUE);
+  ENSURE_TRUE(weakBox.IsAlive());
   aChildBox->MarkDirty(aState);
 }
 
