@@ -582,6 +582,19 @@ nsSprocketLayout::Layout(nsIBox* aBox, nsBoxLayoutState& aState)
             originalClientRect.height = clientRect.height;
         }
 
+        if (!(frameState & NS_STATE_IS_DIRECTION_NORMAL)) {
+          // Our childRect had its XMost() or YMost() (depending on our layout
+          // direction), positioned at a certain point.  Ensure that the
+          // newChildRect satisfies the same constraint.  Note that this is
+          // just equivalent to adjusting the x/y by the difference in
+          // width/height between childRect and newChildRect.  So we don't need
+          // to reaccount for the left and right of the box layout state again.
+          if (frameState & NS_STATE_IS_HORIZONTAL)
+            newChildRect.x = childRect.XMost() - newChildRect.width;
+          else
+            newChildRect.y = childRect.YMost() - newChildRect.height;
+        }
+
         // If the child resized then recompute its position.
         ComputeChildsNextPosition(aBox, child, 
                                   x, 
@@ -790,8 +803,12 @@ nsSprocketLayout::PopulateBoxSizes(nsIBox* aBox, nsBoxLayoutState& aState, nsBox
   currentBox = aBoxSizes;
   nsBoxSize* last = nsnull;
 
+  nscoord maxFlex = 0;
+  PRInt32 childCount = 0;
+
   while(child)
   {
+    ++childCount;
     nsSize pref(0,0);
     nsSize min(0,0);
     nsSize max(NS_INTRINSICSIZE,NS_INTRINSICSIZE);
@@ -851,12 +868,17 @@ nsSprocketLayout::PopulateBoxSizes(nsIBox* aBox, nsBoxLayoutState& aState, nsBox
       child->GetFlex(aState, flex);
 
       // set them if you collapsed you are not flexible.
-      if (collapsed)
-         currentBox->flex = 0;
-      else
-         currentBox->flex = flex;
+      if (collapsed) {
+        currentBox->flex = 0;
+      }
+      else {
+        if (flex > maxFlex) {
+          maxFlex = flex;
+        }
+        currentBox->flex = flex;
+      }
 
-      // we we specified all our children are equal size;
+      // we specified all our children are equal size;
       if (frameState & NS_STATE_EQUAL_SIZE) {
 
         if (prefWidth > biggestPrefWidth) 
@@ -903,7 +925,25 @@ nsSprocketLayout::PopulateBoxSizes(nsIBox* aBox, nsBoxLayoutState& aState, nsBox
 
   }
 
-  // we we specified all our children are equal size;
+  if (childCount > 0) {
+    nscoord maxAllowedFlex = nscoord_MAX / childCount;
+  
+    if (NS_UNLIKELY(maxFlex > maxAllowedFlex)) {
+      // clamp all the flexes
+      currentBox = aBoxSizes;
+      while (currentBox) {
+        currentBox->flex = PR_MIN(currentBox->flex, maxAllowedFlex);
+        currentBox = currentBox->next;      
+      }
+    }
+  }
+#ifdef DEBUG
+  else {
+    NS_ASSERTION(maxFlex == 0, "How did that happen?");
+  }
+#endif
+
+  // we specified all our children are equal size;
   if (frameState & NS_STATE_EQUAL_SIZE) {
     currentBox = aBoxSizes;
 
@@ -1033,7 +1073,7 @@ nsSprocketLayout::ChildResized(nsIBox* aBox,
 
       // if we are a horizontal box see if the child will fit inside us.
       if ( childActualHeight > containingHeight) {
-            // if we are a horizontal box and the the child it bigger than our height
+            // if we are a horizontal box and the child is bigger than our height
 
             // ok if the height changed then we need to reflow everyone but us at the new height
             // so we will set the changed index to be us. And signal that we need a new pass.
