@@ -22,6 +22,7 @@
  * Contributor(s):
  *   Peter Annema <disttsc@bart.nl>
  *   Dean Tessman <dean_tessman@hotmail.com>
+ *   Masayuki Nakano <masayuki@d-toybox.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -68,6 +69,7 @@
 #include "nsITheme.h"
 #include "nsUnicharUtils.h"
 #include "nsContentUtils.h"
+#include "nsDisplayList.h"
 
 #ifdef IBMBIDI
 #include "nsBidiUtils.h"
@@ -93,36 +95,25 @@ public:
 };
 
 
-PRBool nsTextBoxFrame::gAlwaysAppendAccessKey       = PR_FALSE;
-PRBool nsTextBoxFrame::gAccessKeyPrefInitialized    = PR_FALSE;
+PRBool nsTextBoxFrame::gAlwaysAppendAccessKey          = PR_FALSE;
+PRBool nsTextBoxFrame::gAccessKeyPrefInitialized       = PR_FALSE;
+PRBool nsTextBoxFrame::gInsertSeparatorBeforeAccessKey = PR_FALSE;
+PRBool nsTextBoxFrame::gInsertSeparatorPrefInitialized = PR_FALSE;
 
 //
 // NS_NewToolbarFrame
 //
-// Creates a new Toolbar frame and returns it in |aNewFrame|
+// Creates a new Toolbar frame and returns it
 //
-nsresult
-NS_NewTextBoxFrame ( nsIPresShell* aPresShell, nsIFrame** aNewFrame )
+nsIFrame*
+NS_NewTextBoxFrame (nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
-    NS_PRECONDITION(aNewFrame, "null OUT ptr");
-    if (nsnull == aNewFrame) {
-        return NS_ERROR_NULL_POINTER;
-    }
-    nsTextBoxFrame* it = new (aPresShell) nsTextBoxFrame (aPresShell);
-    if (nsnull == it)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    // it->SetFlags(aFlags);
-    *aNewFrame = it;
-    return NS_OK;
-
+    return new (aPresShell) nsTextBoxFrame (aPresShell, aContext);
 } // NS_NewTextFrame
 
 
 NS_IMETHODIMP
-nsTextBoxFrame::AttributeChanged(nsPresContext* aPresContext,
-                                 nsIContent*     aChild,
-                                 PRInt32         aNameSpaceID,
+nsTextBoxFrame::AttributeChanged(PRInt32         aNameSpaceID,
                                  nsIAtom*        aAttribute,
                                  PRInt32         aModType)
 {
@@ -130,25 +121,26 @@ nsTextBoxFrame::AttributeChanged(nsPresContext* aPresContext,
     PRBool aResize;
     PRBool aRedraw;
 
-    UpdateAttributes(aPresContext, aAttribute, aResize, aRedraw);
+    UpdateAttributes(aAttribute, aResize, aRedraw);
 
     if (aResize) {
-        nsBoxLayoutState state(aPresContext);
+        nsBoxLayoutState state(GetPresContext());
         MarkDirty(state);
     } else if (aRedraw) {
-        nsBoxLayoutState state(aPresContext);
+        nsBoxLayoutState state(GetPresContext());
         Redraw(state);
     }
 
     // If the accesskey changed, register for the new value
     // The old value has been unregistered in nsXULElement::SetAttr
     if (aAttribute == nsXULAtoms::accesskey || aAttribute == nsXULAtoms::control)
-        RegUnregAccessKey(aPresContext, PR_TRUE);
+        RegUnregAccessKey(PR_TRUE);
 
     return NS_OK;
 }
 
-nsTextBoxFrame::nsTextBoxFrame(nsIPresShell* aShell):nsLeafBoxFrame(aShell), mCropType(CropRight),mAccessKeyInfo(nsnull)
+nsTextBoxFrame::nsTextBoxFrame(nsIPresShell* aShell, nsStyleContext* aContext):
+  nsLeafBoxFrame(aShell, aContext), mCropType(CropRight),mAccessKeyInfo(nsnull)
 {
     mState |= NS_STATE_NEED_LAYOUT;
     NeedsRecalc();
@@ -161,34 +153,29 @@ nsTextBoxFrame::~nsTextBoxFrame()
 
 
 NS_IMETHODIMP
-nsTextBoxFrame::Init(nsPresContext*  aPresContext,
-                     nsIContent*      aContent,
+nsTextBoxFrame::Init(nsIContent*      aContent,
                      nsIFrame*        aParent,
-                     nsStyleContext*  aContext,
                      nsIFrame*        aPrevInFlow)
 {
-    nsresult rv = nsTextBoxFrameSuper::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
-    if (NS_FAILED(rv))
-        return rv;
+    nsTextBoxFrameSuper::Init(aContent, aParent, aPrevInFlow);
 
     mState |= NS_STATE_NEED_LAYOUT;
     PRBool aResize;
     PRBool aRedraw;
-    UpdateAttributes(aPresContext, nsnull, aResize, aRedraw); /* update all */
+    UpdateAttributes(nsnull, aResize, aRedraw); /* update all */
 
     // register access key
-    RegUnregAccessKey(aPresContext, PR_TRUE);
+    RegUnregAccessKey(PR_TRUE);
 
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsTextBoxFrame::Destroy(nsPresContext* aPresContext)
+void
+nsTextBoxFrame::Destroy()
 {
     // unregister access key
-    RegUnregAccessKey(aPresContext, PR_FALSE);
-
-    return nsTextBoxFrameSuper::Destroy(aPresContext);
+    RegUnregAccessKey(PR_FALSE);
+    nsTextBoxFrameSuper::Destroy();
 }
 
 PRBool
@@ -198,16 +185,29 @@ nsTextBoxFrame::AlwaysAppendAccessKey()
   {
     gAccessKeyPrefInitialized = PR_TRUE;
 
-    gAlwaysAppendAccessKey =
-        nsContentUtils::GetCharPref("intl.menuitems.alwaysappendaccesskeys").Equals("true");
+    const char* prefName = "intl.menuitems.alwaysappendaccesskeys";
+    nsAdoptingString val = nsContentUtils::GetLocalizedStringPref(prefName);
+    gAlwaysAppendAccessKey = val.Equals(NS_LITERAL_STRING("true"));
   }
   return gAlwaysAppendAccessKey;
 }
- 
+
+PRBool
+nsTextBoxFrame::InsertSeparatorBeforeAccessKey()
+{
+  if (!gInsertSeparatorPrefInitialized)
+  {
+    gInsertSeparatorPrefInitialized = PR_TRUE;
+
+    const char* prefName = "intl.menuitems.insertseparatorbeforeaccesskeys";
+    nsAdoptingString val = nsContentUtils::GetLocalizedStringPref(prefName);
+    gInsertSeparatorBeforeAccessKey = val.EqualsLiteral("true");
+  }
+  return gInsertSeparatorBeforeAccessKey;
+}
 
 void
-nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
-                                 nsIAtom*         aAttribute,
+nsTextBoxFrame::UpdateAttributes(nsIAtom*         aAttribute,
                                  PRBool&          aResize,
                                  PRBool&          aRedraw)
 {
@@ -216,20 +216,27 @@ nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
     aRedraw = PR_FALSE;
 
     if (aAttribute == nsnull || aAttribute == nsXULAtoms::crop) {
-        nsAutoString value;
-        mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::crop, value);
+        static nsIContent::AttrValuesArray strings[] =
+          {&nsXULAtoms::left, &nsXULAtoms::start, &nsXULAtoms::center,
+           &nsXULAtoms::right, &nsXULAtoms::end, nsnull};
         CroppingStyle cropType;
-
-        if (value.EqualsASCII(CROP_LEFT) ||
-            value.EqualsASCII(CROP_START))
+        switch (mContent->FindAttrValueIn(kNameSpaceID_None, nsXULAtoms::crop,
+                                          strings, eCaseMatters)) {
+          case 0:
+          case 1:
             cropType = CropLeft;
-        else if (value.EqualsASCII(CROP_CENTER))
+            break;
+          case 2:
             cropType = CropCenter;
-        else if (value.EqualsASCII(CROP_RIGHT) ||
-                 value.EqualsASCII(CROP_END))
+            break;
+          case 3:
+          case 4:
             cropType = CropRight;
-        else
+            break;
+          default:
             cropType = CropNone;
+            break;
+        }
 
         if (cropType != mCropType) {
             aResize = PR_TRUE;
@@ -238,12 +245,8 @@ nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
     }
 
     if (aAttribute == nsnull || aAttribute == nsHTMLAtoms::value) {
-        nsAutoString value;
-        mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, value);
-        if (!value.Equals(mTitle)) {
-            mTitle = value;
-            doUpdateTitle = PR_TRUE;
-        }
+        mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, mTitle);
+        doUpdateTitle = PR_TRUE;
     }
 
     if (aAttribute == nsnull || aAttribute == nsXULAtoms::accesskey) {
@@ -258,9 +261,7 @@ nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
         if (!accesskey.Equals(mAccessKey)) {
             if (!doUpdateTitle) {
                 // Need to get clean mTitle and didn't already
-                nsAutoString value;
-                mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, value);
-                mTitle = value;
+                mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, mTitle);
                 doUpdateTitle = PR_TRUE;
             }
             mAccessKey = accesskey;
@@ -274,47 +275,66 @@ nsTextBoxFrame::UpdateAttributes(nsPresContext*  aPresContext,
 
 }
 
-NS_IMETHODIMP
-nsTextBoxFrame::Paint(nsPresContext*      aPresContext,
-                      nsIRenderingContext& aRenderingContext,
-                      const nsRect&        aDirtyRect,
-                      nsFramePaintLayer    aWhichLayer,
-                      PRUint32             aFlags)
+class nsDisplayXULTextBox : public nsDisplayItem {
+public:
+  nsDisplayXULTextBox(nsTextBoxFrame* aFrame) : nsDisplayItem(aFrame) {
+      MOZ_COUNT_CTOR(nsDisplayXULTextBox);
+  }
+#ifdef NS_BUILD_REFCNT_LOGGING
+  virtual ~nsDisplayXULTextBox() {
+      MOZ_COUNT_DTOR(nsDisplayXULTextBox);
+  }
+#endif
+
+  virtual void Paint(nsDisplayListBuilder* aBuilder, nsIRenderingContext* aCtx,
+     const nsRect& aDirtyRect);
+  NS_DISPLAY_DECL_NAME("XULTextBox")
+};
+
+void nsDisplayXULTextBox::Paint(nsDisplayListBuilder* aBuilder,
+     nsIRenderingContext* aCtx, const nsRect& aDirtyRect)
 {
-    if (!GetStyleVisibility()->IsVisible())
-        return NS_OK;
-
-    if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer) {
-
-        // remove the border and padding
-        nsStyleBorderPadding  bPad;
-        mStyleContext->GetBorderPaddingFor(bPad);
-        nsMargin border(0,0,0,0);
-        bPad.GetBorderPadding(border);
-
-        nsRect textRect(0,0,mRect.width, mRect.height);
-        textRect.Deflate(border);
-
-        PaintTitle(aPresContext, aRenderingContext, aDirtyRect, textRect);
-    }
-
-    return nsLeafFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
+  NS_STATIC_CAST(nsTextBoxFrame*, mFrame)->
+    PaintTitle(*aCtx, aDirtyRect, aBuilder->ToReferenceFrame(mFrame));
 }
 
 NS_IMETHODIMP
-nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
-                           nsIRenderingContext& aRenderingContext,
+nsTextBoxFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                                 const nsRect&           aDirtyRect,
+                                 const nsDisplayListSet& aLists)
+{
+    if (!IsVisibleForPainting(aBuilder))
+      return NS_OK;
+
+    nsresult rv = nsLeafBoxFrame::BuildDisplayList(aBuilder, aDirtyRect, aLists);
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    return aLists.Content()->AppendNewToTop(new (aBuilder)
+        nsDisplayXULTextBox(this));
+}
+
+void
+nsTextBoxFrame::PaintTitle(nsIRenderingContext& aRenderingContext,
                            const nsRect&        aDirtyRect,
-                           const nsRect&        aRect)
+                           nsPoint              aPt)
 {
     if (mTitle.IsEmpty())
-        return NS_OK;
+        return;
+
+    nsStyleBorderPadding  bPad;
+    mStyleContext->GetBorderPaddingFor(bPad);
+    nsMargin border(0,0,0,0);
+    bPad.GetBorderPadding(border);
+
+    nsRect textRect(aPt, GetSize());
+    textRect.Deflate(border);
 
     // determine (cropped) title and underline position
-    LayoutTitle(aPresContext, aRenderingContext, aRect);
+    nsPresContext* presContext = GetPresContext();
+    LayoutTitle(presContext, aRenderingContext, textRect);
 
     // make the rect as small as our (cropped) text.
-    nsRect textRect(aRect);
+    nscoord outerWidth = textRect.width;
     textRect.width = mTitleWidth;
 
     // Align our text within the overall rect by checking our text-align property.
@@ -322,19 +342,19 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
     const nsStyleText* textStyle = GetStyleText();
 
     if (textStyle->mTextAlign == NS_STYLE_TEXT_ALIGN_CENTER)
-      textRect.x += (aRect.width - textRect.width)/2;
+      textRect.x += (outerWidth - textRect.width)/2;
     else if (textStyle->mTextAlign == NS_STYLE_TEXT_ALIGN_RIGHT) {
       if (vis->mDirection == NS_STYLE_DIRECTION_LTR)
-        textRect.x += (aRect.width - textRect.width);
+        textRect.x += (outerWidth - textRect.width);
     }
     else {
       if (vis->mDirection == NS_STYLE_DIRECTION_RTL)
-        textRect.x += (aRect.width - textRect.width);
+        textRect.x += (outerWidth - textRect.width);
     }
 
     // don't draw if the title is not dirty
     if (PR_FALSE == aDirtyRect.Intersects(textRect))
-        return NS_OK;
+        return;
 
     // paint the title
     nscolor overColor;
@@ -383,8 +403,8 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
     nscoord size;
     nscoord baseline;
     nsCOMPtr<nsIFontMetrics> fontMet;
-    aPresContext->DeviceContext()->GetMetricsFor(fontStyle->mFont,
-                                                 *getter_AddRefs(fontMet));
+    presContext->DeviceContext()->GetMetricsFor(fontStyle->mFont,
+                                                *getter_AddRefs(fontMet));
     fontMet->GetMaxAscent(baseline);
 
     if (decorations & (NS_FONT_DECORATION_OVERLINE | NS_FONT_DECORATION_UNDERLINE)) {
@@ -414,8 +434,8 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
     nsresult rv = NS_ERROR_FAILURE;
 
     if (mState & NS_FRAME_IS_BIDI) {
-      aPresContext->SetBidiEnabled(PR_TRUE);
-      nsBidiPresUtils* bidiUtils = aPresContext->GetBidiUtils();
+      presContext->SetBidiEnabled(PR_TRUE);
+      nsBidiPresUtils* bidiUtils = presContext->GetBidiUtils();
 
       if (bidiUtils) {
         const nsStyleVisibility* vis = GetStyleVisibility();
@@ -426,7 +446,7 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
            nsBidiPositionResolve posResolve;
            posResolve.logicalIndex = mAccessKeyInfo->mAccesskeyIndex;
            rv = bidiUtils->RenderText(mCroppedTitle.get(), mCroppedTitle.Length(), direction,
-                                      aPresContext, aRenderingContext,
+                                      presContext, aRenderingContext,
                                       textRect.x, textRect.y + baseline,
                                       &posResolve,
                                       1);
@@ -435,7 +455,7 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
         else
         {
            rv = bidiUtils->RenderText(mCroppedTitle.get(), mCroppedTitle.Length(), direction,
-                                      aPresContext, aRenderingContext,
+                                      presContext, aRenderingContext,
                                       textRect.x, textRect.y + baseline);
         }
       }
@@ -463,8 +483,6 @@ nsTextBoxFrame::PaintTitle(nsPresContext*      aPresContext,
                                    mAccessKeyInfo->mAccessWidth,
                                    mAccessKeyInfo->mAccessUnderlineSize);
     }
-
-    return NS_OK;
 }
 
 void
@@ -699,35 +717,48 @@ nsTextBoxFrame::CalculateTitleForWidth(nsPresContext*      aPresContext,
 void
 nsTextBoxFrame::UpdateAccessTitle()
 {
+    /*
+     * Note that if you change appending access key label spec,
+     * you need to maintain same logic in following methods. See bug 324159.
+     * toolkit/content/commonDialog.js (setLabelForNode)
+     * toolkit/content/widgets/text.xml (formatAccessKey)
+     * xpfe/global/resources/content/commonDialog.js (setLabelForNode)
+     * xpfe/global/resources/content/bindings/text.xml (formatAccessKey)
+     */
     PRInt32 menuAccessKey;
     nsMenuBarListener::GetMenuAccessKey(&menuAccessKey);
-    if (menuAccessKey) {
-        if (!mAccessKey.IsEmpty()) {
-            if (( !FindInReadable(mAccessKey, mTitle, nsCaseInsensitiveStringComparator()))
-                || AlwaysAppendAccessKey()) 
-            {
-                nsAutoString tmpstring;
-                if (!mTitle.IsEmpty() && !isspace(mTitle.Last())) {
-                  tmpstring += ' ';
-                }
-                tmpstring += '(';
-                tmpstring += mAccessKey;
-                ToUpperCase(tmpstring);
-                tmpstring.Append(NS_LITERAL_STRING(")"));
-                PRInt32 offset = mTitle.RFind("...");
-                if (offset != kNotFound) {
-                    mTitle.Insert(tmpstring,NS_STATIC_CAST(PRUint32, offset));
-                } else {
-                    PRUint32 l = mTitle.Length();
-                    if((l > 0) && (PRUnichar(':')==mTitle[l-1])) {
-                      mTitle.Insert(tmpstring,l-1);
-                    } else {
-                      mTitle += tmpstring;
-                    }
-                }
-            }
-        }
+    if (!menuAccessKey || mAccessKey.IsEmpty())
+        return;
+
+    if (!AlwaysAppendAccessKey() &&
+        FindInReadable(mAccessKey, mTitle, nsCaseInsensitiveStringComparator()))
+        return;
+
+    nsAutoString accessKeyLabel;
+    accessKeyLabel += '(';
+    accessKeyLabel += mAccessKey;
+    ToUpperCase(accessKeyLabel);
+    accessKeyLabel += ')';
+
+    if (mTitle.IsEmpty()) {
+        mTitle = accessKeyLabel;
+        return;
     }
+
+    PRInt32 offset = mTitle.RFind("...");
+    if (offset == kNotFound) {
+        offset = (PRInt32)mTitle.Length();
+        if (mTitle.Last() == PRUnichar(':'))
+            offset--;
+    }
+
+    if (InsertSeparatorBeforeAccessKey() && offset > 0 &&
+        !NS_IS_SPACE(mTitle[offset - 1])) {
+        mTitle.Insert(' ', (PRUint32)offset);
+        offset++;
+    }
+
+    mTitle.Insert(accessKeyLabel, (PRUint32)offset);
 }
 
 void
@@ -889,7 +920,7 @@ nsTextBoxFrame::GetAscent(nsBoxLayoutState& aBoxLayoutState, nscoord& aAscent)
 NS_IMETHODIMP
 nsTextBoxFrame::GetFrameName(nsAString& aResult) const
 {
-    MakeFrameName(NS_LITERAL_STRING("Text"), aResult);
+    MakeFrameName(NS_LITERAL_STRING("TextBox"), aResult);
     aResult += NS_LITERAL_STRING("[value=") + mTitle + NS_LITERAL_STRING("]");
     return NS_OK;
 }
@@ -898,8 +929,7 @@ nsTextBoxFrame::GetFrameName(nsAString& aResult) const
 // If you make changes to this function, check its counterparts 
 // in nsBoxFrame and nsAreaFrame
 nsresult
-nsTextBoxFrame::RegUnregAccessKey(nsPresContext* aPresContext,
-                                  PRBool          aDoReg)
+nsTextBoxFrame::RegUnregAccessKey(PRBool aDoReg)
 {
     // if we have no content, we can't do anything
     if (!mContent)
@@ -927,7 +957,7 @@ nsTextBoxFrame::RegUnregAccessKey(nsPresContext* aPresContext,
 
     // With a valid PresContext we can get the ESM 
     // and (un)register the access key
-    nsIEventStateManager *esm = aPresContext->EventStateManager();
+    nsIEventStateManager *esm = GetPresContext()->EventStateManager();
 
     PRUint32 key = accessKey.First();
     if (aDoReg)
