@@ -46,18 +46,20 @@
 #include "nsXULAtoms.h"
 #include "nsIPresShell.h"
 #include "nsIFrame.h"
+#include "nsIMenuFrame.h"
 #include "nsIPopupBoxObject.h"
 #include "nsIServiceManager.h"
 #ifdef MOZ_XUL
+#include "nsIDOMNSDocument.h"
 #include "nsITreeView.h"
 #endif
 #include "nsGUIEvent.h"
 #include "nsIPrivateDOMEvent.h"
 #include "nsPresContext.h"
 #include "nsIScriptContext.h"
-#include "nsIScriptGlobalObject.h"
-#include "nsIDOMWindowInternal.h"
+#include "nsPIDOMWindow.h"
 #include "nsContentUtils.h"
+#include "nsIRootBox.h"
 
 //////////////////////////////////////////////////////////////////////////
 //// nsISupports
@@ -108,6 +110,14 @@ nsXULTooltipListener::MouseDown(nsIDOMEvent* aMouseEvent)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsXULTooltipListener::MouseUp(nsIDOMEvent* aMouseEvent)
+{
+  HideTooltip();
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsXULTooltipListener::MouseOut(nsIDOMEvent* aMouseEvent)
 {
@@ -138,7 +148,7 @@ nsXULTooltipListener::MouseOut(nsIDOMEvent* aMouseEvent)
     if (!xulDoc)     // remotely possible someone could have 
       return NS_OK;  // removed tooltip from dom while it was open
     nsCOMPtr<nsIDOMNode> tooltipNode;
-    xulDoc->GetTooltipNode (getter_AddRefs(tooltipNode));
+    xulDoc->TrustedGetTooltipNode (getter_AddRefs(tooltipNode));
 
     // if they're the same, the mouse left the node the tooltip appeared on,
     // close the tooltip.
@@ -265,9 +275,8 @@ PRBool nsXULTooltipListener::sShowTooltips = PR_FALSE;
 
 // XXX: This could all be done in the ctor.
 nsresult
-nsXULTooltipListener::Init(nsIContent* aSourceNode, nsIRootBox* aRootBox)
+nsXULTooltipListener::Init(nsIContent* aSourceNode)
 {
-  mRootBox = aRootBox;
   mSourceNode = aSourceNode;
   AddTooltipSupport(aSourceNode);
   
@@ -295,8 +304,8 @@ nsXULTooltipListener::AddTooltipSupport(nsIContent* aNode)
     return NS_ERROR_NULL_POINTER;
 
   nsCOMPtr<nsIDOMEventTarget> evtTarget(do_QueryInterface(aNode));
-  evtTarget->AddEventListener(NS_LITERAL_STRING("mouseout"), (nsIDOMMouseListener*)this, PR_FALSE);
-  evtTarget->AddEventListener(NS_LITERAL_STRING("mousemove"), (nsIDOMMouseListener*)this, PR_FALSE);
+  evtTarget->AddEventListener(NS_LITERAL_STRING("mouseout"), (nsIDOMMouseListener*)this, PR_TRUE);
+  evtTarget->AddEventListener(NS_LITERAL_STRING("mousemove"), (nsIDOMMouseListener*)this, PR_TRUE);
   
   return NS_OK;
 }
@@ -308,8 +317,8 @@ nsXULTooltipListener::RemoveTooltipSupport(nsIContent* aNode)
     return NS_ERROR_NULL_POINTER;
 
   nsCOMPtr<nsIDOMEventTarget> evtTarget(do_QueryInterface(aNode));
-  evtTarget->RemoveEventListener(NS_LITERAL_STRING("mouseout"), (nsIDOMMouseListener*)this, PR_FALSE);
-  evtTarget->RemoveEventListener(NS_LITERAL_STRING("mousemove"), (nsIDOMMouseListener*)this, PR_FALSE);
+  evtTarget->RemoveEventListener(NS_LITERAL_STRING("mouseout"), (nsIDOMMouseListener*)this, PR_TRUE);
+  evtTarget->RemoveEventListener(NS_LITERAL_STRING("mousemove"), (nsIDOMMouseListener*)this, PR_TRUE);
 
   return NS_OK;
 }
@@ -318,24 +327,45 @@ nsXULTooltipListener::RemoveTooltipSupport(nsIContent* aNode)
 void
 nsXULTooltipListener::CheckTreeBodyMove(nsIDOMMouseEvent* aMouseEvent)
 {
+  if (!mSourceNode)
+    return;
+
+  // get the boxObject of the documentElement of the document the tree is in
+  nsCOMPtr<nsIBoxObject> bx;
+  nsCOMPtr<nsIDOMDocument> doc(do_QueryInterface(mSourceNode->GetDocument()));
+  if (doc) {
+    nsCOMPtr<nsIDOMNSDocument> nsDoc(do_QueryInterface(doc));
+    nsCOMPtr<nsIDOMElement> docElement;
+    doc->GetDocumentElement(getter_AddRefs(docElement));
+    if (nsDoc && docElement) {
+      nsDoc->GetBoxObjectFor(docElement, getter_AddRefs(bx));
+    }
+  }
+
   nsCOMPtr<nsITreeBoxObject> obx;
   GetSourceTreeBoxObject(getter_AddRefs(obx));
-  if (obx) {
+  if (bx && obx) {
     PRInt32 x, y;
-    aMouseEvent->GetClientX(&x);
-    aMouseEvent->GetClientY(&y);
+    aMouseEvent->GetScreenX(&x);
+    aMouseEvent->GetScreenY(&y);
+
     PRInt32 row;
     nsCOMPtr<nsITreeColumn> col;
     nsCAutoString obj;
 
+    // subtract off the documentElement's boxObject
+    PRInt32 boxX, boxY;
+    bx->GetScreenX(&boxX);
+    bx->GetScreenY(&boxY);
+    x -= boxX;
+    y -= boxY;
+
     obx->GetCellAt(x, y, &row, getter_AddRefs(col), obj);
 
     // determine if we are going to need a titletip
     // XXX check the disabletitletips attribute on the tree content
     mNeedTitletip = PR_FALSE;
     if (row >= 0 && obj.EqualsLiteral("text")) {
-      nsCOMPtr<nsITreeView> view;
-      obx->GetView(getter_AddRefs(view));
       PRBool isCropped;
       obx->IsCellCropped(row, col, &isCropped);
       mNeedTitletip = isCropped;
@@ -394,7 +424,7 @@ nsXULTooltipListener::ShowTooltip()
       evtTarget->AddEventListener(NS_LITERAL_STRING("popuphiding"), 
                                   (nsIDOMMouseListener*)this, PR_FALSE);
 
-      // listen for mousedown,keydown, and DOMMouseScroll events at document level
+      // listen for mousedown, mouseup, keydown, and DOMMouseScroll events at document level
       nsIDocument* doc = mSourceNode->GetDocument();
       if (doc) {
         evtTarget = do_QueryInterface(doc);
@@ -402,6 +432,8 @@ nsXULTooltipListener::ShowTooltip()
                                     (nsIDOMMouseListener*)this, PR_TRUE);
         evtTarget->AddEventListener(NS_LITERAL_STRING("mousedown"), 
                                     (nsIDOMMouseListener*)this, PR_TRUE);
+        evtTarget->AddEventListener(NS_LITERAL_STRING("mouseup"), 
+                                    (nsIDOMMouseListener*)this, PR_TRUE);                                    
         evtTarget->AddEventListener(NS_LITERAL_STRING("keydown"), 
                                     (nsIDOMMouseListener*)this, PR_TRUE);
       }
@@ -420,7 +452,7 @@ GetTreeCellCoords(nsITreeBoxObject* aTreeBox, nsIContent* aSourceNode,
                   PRInt32 aRow, nsITreeColumn* aCol, PRInt32* aX, PRInt32* aY)
 {
   PRInt32 junk;
-  aTreeBox->GetCoordsForCellItem(aRow, aCol, NS_LITERAL_CSTRING(""), aX, aY, &junk, &junk);
+  aTreeBox->GetCoordsForCellItem(aRow, aCol, EmptyCString(), aX, aY, &junk, &junk);
   nsCOMPtr<nsIDOMXULElement> xulEl(do_QueryInterface(aSourceNode));
   nsCOMPtr<nsIBoxObject> bx;
   xulEl->GetBoxObject(getter_AddRefs(bx));
@@ -526,91 +558,116 @@ GetImmediateChild(nsIContent* aContent, nsIAtom *aTag, nsIContent** aResult)
 }
 
 nsresult
-nsXULTooltipListener::GetTooltipFor(nsIContent* aTarget, nsIContent** aTooltip)
+nsXULTooltipListener::FindTooltip(nsIContent* aTarget, nsIContent** aTooltip)
 {
   if (!aTarget)
     return NS_ERROR_NULL_POINTER;
 
-  nsCOMPtr<nsIDOMElement> targetEl(do_QueryInterface(aTarget));
-  if (!targetEl)
-    return NS_ERROR_FAILURE; // could be a text node or something
-
   // before we go on, make sure that target node still has a window
   nsCOMPtr<nsIDocument> document = aTarget->GetDocument();
   if (!document) {
-    NS_ERROR("Unable to retrieve the tooltip node document.");
+    NS_WARNING("Unable to retrieve the tooltip node document.");
     return NS_ERROR_FAILURE;
   }
-  nsIScriptGlobalObject *global = document->GetScriptGlobalObject();
-  if (global) {
-    nsIScriptContext *context = global->GetContext();
+  nsCOMPtr<nsPIDOMWindow> window = document->GetWindow();
+  if (!window) {
+    return NS_OK;
+  }
 
-    if (context) {
-      nsCOMPtr<nsIDOMWindowInternal> domWindow = do_QueryInterface(global);
-      if (!domWindow)
-        return NS_ERROR_FAILURE;
-      else {
-        PRBool needTooltip;
-        targetEl->HasAttribute(NS_LITERAL_STRING("tooltiptext"), &needTooltip);
-        if (needTooltip) {
-          // specifying tooltiptext means we will always use the default tooltip
-           mRootBox->GetDefaultTooltip(aTooltip);
-           NS_IF_ADDREF(*aTooltip);
-           return NS_OK;
-        } else {
-          nsAutoString tooltipId;
-          targetEl->GetAttribute(NS_LITERAL_STRING("tooltip"), tooltipId);
+  PRBool closed;
+  window->GetClosed(&closed);
 
-          // if tooltip == _child, look for first <tooltip> child
-          if (tooltipId.EqualsLiteral("_child")) {
-            GetImmediateChild(aTarget, nsXULAtoms::tooltip, aTooltip);
-            return NS_OK;
-          } else {
-            if (!tooltipId.IsEmpty()) {
-              // tooltip must be an id, use getElementById to find it
-              nsCOMPtr<nsIDOMDocument> domDocument =
-                do_QueryInterface(document);
-              if (!domDocument) {
-                return NS_ERROR_FAILURE;
-              }
+  if (closed) {
+    return NS_OK;
+  }
 
-              nsCOMPtr<nsIDOMElement> tooltipEl;
-              domDocument->GetElementById(tooltipId,
-                                          getter_AddRefs(tooltipEl));
+  nsAutoString tooltipText;
+  aTarget->GetAttr(kNameSpaceID_None, nsXULAtoms::tooltiptext, tooltipText);
+  if (!tooltipText.IsEmpty()) {
+    // specifying tooltiptext means we will always use the default tooltip
+    nsIRootBox* rootBox = nsIRootBox::GetRootBox(document->GetShellAt(0));
+    NS_ENSURE_STATE(rootBox);
+    *aTooltip = rootBox->GetDefaultTooltip();
+    if (*aTooltip) {
+      NS_ADDREF(*aTooltip);
+      (*aTooltip)->SetAttr(kNameSpaceID_None, nsXULAtoms::label, tooltipText, PR_TRUE);
+    }
+    return NS_OK;
+  }
 
-              if (tooltipEl) {
+  nsAutoString tooltipId;
+  aTarget->GetAttr(kNameSpaceID_None, nsXULAtoms::tooltip, tooltipId);
+
+  // if tooltip == _child, look for first <tooltip> child
+  if (tooltipId.EqualsLiteral("_child")) {
+    GetImmediateChild(aTarget, nsXULAtoms::tooltip, aTooltip);
+    return NS_OK;
+  }
+
+  if (!tooltipId.IsEmpty()) {
+    // tooltip must be an id, use getElementById to find it
+    nsCOMPtr<nsIDOMDocument> domDocument =
+      do_QueryInterface(document);
+    if (!domDocument) {
+      return NS_ERROR_FAILURE;
+    }
+
+    nsCOMPtr<nsIDOMElement> tooltipEl;
+    domDocument->GetElementById(tooltipId, getter_AddRefs(tooltipEl));
+
+    if (tooltipEl) {
 #ifdef MOZ_XUL
-                mNeedTitletip = PR_FALSE;
+      mNeedTitletip = PR_FALSE;
 #endif
-
-                nsCOMPtr<nsIContent> tooltipContent(do_QueryInterface(tooltipEl));
-                *aTooltip = tooltipContent;
-                NS_IF_ADDREF(*aTooltip);
-
-                return NS_OK;
-              }
-            }
-          }
-        }
-
-#ifdef MOZ_XUL
-        // titletips should just use the default tooltip
-        if (mIsSourceTree && mNeedTitletip) {
-          mRootBox->GetDefaultTooltip(aTooltip);
-          NS_IF_ADDREF(*aTooltip);
-          return NS_OK;
-        }
-#endif
-      }
+      CallQueryInterface(tooltipEl, aTooltip);
+      return NS_OK;
     }
   }
 
+#ifdef MOZ_XUL
+  // titletips should just use the default tooltip
+  if (mIsSourceTree && mNeedTitletip) {
+    nsIRootBox* rootBox = nsIRootBox::GetRootBox(document->GetShellAt(0));
+    NS_ENSURE_STATE(rootBox);
+    NS_IF_ADDREF(*aTooltip = rootBox->GetDefaultTooltip());
+  }
+#endif
+
   return NS_OK;
 }
 
+
+nsresult
+nsXULTooltipListener::GetTooltipFor(nsIContent* aTarget, nsIContent** aTooltip)
+{
+  *aTooltip = nsnull;
+  nsCOMPtr<nsIContent> tooltip;
+  nsresult rv = FindTooltip(aTarget, getter_AddRefs(tooltip));
+  if (NS_FAILED(rv) || !tooltip) {
+    return rv;
+  }
+
+  // Submenus can't be used as tooltips, see bug 288763.
+  nsIContent* parent = tooltip->GetParent();
+  if (parent) {
+    nsIDocument* doc = parent->GetCurrentDoc();
+    nsIPresShell* presShell = doc ? doc->GetShellAt(0) : nsnull;
+    nsIFrame* frame = presShell ? presShell->GetPrimaryFrameFor(parent) : nsnull;
+    if (frame) {
+      nsIMenuFrame* menu = nsnull;
+      CallQueryInterface(frame, &menu);
+      NS_ENSURE_FALSE(menu, NS_ERROR_FAILURE);
+    }
+  }
+
+  tooltip.swap(*aTooltip);
+  return rv;
+}
+
 nsresult
 nsXULTooltipListener::DestroyTooltip()
 {
+  nsCOMPtr<nsIDOMMouseListener> kungFuDeathGrip(this);
   if (mCurrentTooltip) {
     // clear out the tooltip node on the document
     nsCOMPtr<nsIDocument> doc = mCurrentTooltip->GetDocument();
@@ -623,6 +680,7 @@ nsXULTooltipListener::DestroyTooltip()
       nsCOMPtr<nsIDOMEventTarget> evtTarget(do_QueryInterface(doc));
       evtTarget->RemoveEventListener(NS_LITERAL_STRING("DOMMouseScroll"), (nsIDOMMouseListener*)this, PR_TRUE);
       evtTarget->RemoveEventListener(NS_LITERAL_STRING("mousedown"), (nsIDOMMouseListener*)this, PR_TRUE);
+      evtTarget->RemoveEventListener(NS_LITERAL_STRING("mouseup"), (nsIDOMMouseListener*)this, PR_TRUE);
       evtTarget->RemoveEventListener(NS_LITERAL_STRING("keydown"), (nsIDOMMouseListener*)this, PR_TRUE);
     }
 
@@ -682,8 +740,10 @@ void
 nsXULTooltipListener::sAutoHideCallback(nsITimer *aTimer, void* aListener)
 {
   nsXULTooltipListener* self = NS_STATIC_CAST(nsXULTooltipListener*, aListener);
-  if (self)
+  if (self) {
+    nsCOMPtr<nsIDOMMouseListener> kungFuDeathGrip(self);
     self->HideTooltip();
+  }
 }
 
 #ifdef MOZ_XUL
