@@ -24,6 +24,8 @@
  *   Dean Tessman <dean_tessman@hotmail.com>
  *   Brian Ryner <bryner@brianryner.com>
  *   Jan Varga <varga@ku.sk>
+ *   Nate Nielsen <nielsen@memberwebs.com>
+ *   Mark Banner <mark@standard8.demon.co.uk>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -40,7 +42,6 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsLeafBoxFrame.h"
-#include "nsITreeBoxObject.h"
 #include "nsITreeView.h"
 #include "nsICSSPseudoComparator.h"
 #include "nsIScrollbarMediator.h"
@@ -53,6 +54,20 @@
 #include "nsTreeColumns.h"
 #include "nsTreeImageListener.h"
 #include "nsAutoPtr.h"
+#include "nsDataHashtable.h"
+#include "imgIRequest.h"
+#include "imgIDecoderObserver.h"
+
+// An entry in the tree's image cache
+struct nsTreeImageCacheEntry
+{
+  nsTreeImageCacheEntry() {}
+  nsTreeImageCacheEntry(imgIRequest *aRequest, imgIDecoderObserver *aListener)
+    : request(aRequest), listener(aListener) {}
+
+  nsCOMPtr<imgIRequest> request;
+  nsCOMPtr<imgIDecoderObserver> listener;
+};
 
 // The actual frame that paints the cells and rows.
 class nsTreeBodyFrame : public nsLeafBoxFrame,
@@ -62,7 +77,7 @@ class nsTreeBodyFrame : public nsLeafBoxFrame,
                         public nsIReflowCallback
 {
 public:
-  nsTreeBodyFrame(nsIPresShell* aPresShell);
+  nsTreeBodyFrame(nsIPresShell* aPresShell, nsStyleContext* aContext);
   virtual ~nsTreeBodyFrame();
 
   NS_DECL_ISUPPORTS
@@ -85,25 +100,24 @@ public:
   NS_IMETHOD VisibilityChanged(nsISupports* aScrollbar, PRBool aVisible) { Invalidate(); return NS_OK; };
 
   // Overridden from nsIFrame to cache our pres context.
-  NS_IMETHOD Init(nsPresContext* aPresContext, nsIContent* aContent,
-                  nsIFrame* aParent, nsStyleContext* aContext, nsIFrame* aPrevInFlow);
-  NS_IMETHOD Destroy(nsPresContext* aPresContext);
-  NS_IMETHOD GetCursor(nsPresContext* aPresContext,
-                       nsPoint& aPoint,
-                       PRInt32& aCursor);
+  NS_IMETHOD Init(nsIContent*     aContent,
+                  nsIFrame*       aParent,
+                  nsIFrame*       aPrevInFlow);
+  virtual void Destroy();
+
+  NS_IMETHOD GetCursor(const nsPoint& aPoint,
+                       nsIFrame::Cursor& aCursor);
 
   NS_IMETHOD HandleEvent(nsPresContext* aPresContext,
                          nsGUIEvent* aEvent,
                          nsEventStatus* aEventStatus);
 
-  // Painting methods.
-  // Paint is the generic nsIFrame paint method.  We override this method
-  // to paint our contents (our rows and cells).
-  NS_IMETHOD Paint(nsPresContext*      aPresContext,
-                   nsIRenderingContext& aRenderingContext,
-                   const nsRect&        aDirtyRect,
-                   nsFramePaintLayer    aWhichLayer,
-                   PRUint32             aFlags = 0);
+  NS_IMETHOD BuildDisplayList(nsDisplayListBuilder*   aBuilder,
+                              const nsRect&           aDirtyRect,
+                              const nsDisplayListSet& aLists);
+                              
+  void PaintTreeBody(nsIRenderingContext& aRenderingContext,
+                     const nsRect& aDirtyRect, nsPoint aPt);
 
   // This method paints a specific column background of the tree.
   void PaintColumn(nsTreeColumn*        aColumn,
@@ -115,9 +129,10 @@ public:
   // This method paints a single row in the tree.
   void PaintRow(PRInt32              aRowIndex,
                 const nsRect&        aRowRect,
-                nsPresContext*      aPresContext,
+                nsPresContext*       aPresContext,
                 nsIRenderingContext& aRenderingContext,
-                const nsRect&        aDirtyRect);
+                const nsRect&        aDirtyRect,
+                nsPoint              aPt);
 
   // This method paints a single separator in the tree.
   void PaintSeparator(PRInt32              aRowIndex,
@@ -130,10 +145,11 @@ public:
   void PaintCell(PRInt32              aRowIndex, 
                  nsTreeColumn*        aColumn,
                  const nsRect&        aCellRect,
-                 nsPresContext*      aPresContext,
+                 nsPresContext*       aPresContext,
                  nsIRenderingContext& aRenderingContext,
                  const nsRect&        aDirtyRect,
-                 nscoord&             aCurrX);
+                 nscoord&             aCurrX,
+                 nsPoint              aPt);
 
   // This method paints the twisty inside a cell in the primary column of an tree.
   void PaintTwisty(PRInt32              aRowIndex,
@@ -194,25 +210,48 @@ public:
                             const nsRect&        aRect,
                             const nsRect&        aDirtyRect);
 
-  friend nsresult NS_NewTreeBodyFrame(nsIPresShell* aPresShell, 
-                                          nsIFrame** aNewFrame);
+  friend nsIFrame* NS_NewTreeBodyFrame(nsIPresShell* aPresShell);
+
+  struct ScrollParts {
+    nsIScrollbarFrame* mVScrollbar;
+    nsIContent*        mVScrollbarContent;
+    nsIScrollbarFrame* mHScrollbar;
+    nsIContent*        mHScrollbarContent;
+    nsIScrollableView* mColumnsScrollableView;
+  };
 
 protected:
   PRInt32 GetLastVisibleRow() {
     return mTopRowIndex + mPageLength;
   };
 
-  // An internal hit test.
-  PRInt32 GetRowAt(PRInt32 aX, PRInt32 aY);
+  // An internal hit test.  aX and aY are expected to be in twips in the
+  // coordinate system of this frame.
+  PRInt32 GetRowAt(nscoord aX, nscoord aY);
+
+  void AdjustForCellText(nsAutoString& aText,
+                         PRInt32 aRowIndex,  nsTreeColumn* aColumn,
+                         nsIRenderingContext& aRenderingContext,
+                         nsRect& aTextRect);
 
   // A helper used when hit testing.
-  nsIAtom* GetItemWithinCellAt(PRInt32 aX, const nsRect& aCellRect, PRInt32 aRowIndex,
-                               nsTreeColumn* aColumn);
+  nsIAtom* GetItemWithinCellAt(nscoord aX, const nsRect& aCellRect,
+                               PRInt32 aRowIndex, nsTreeColumn* aColumn);
 
-  // An internal hit test.
-  void GetCellAt(PRInt32 aX, PRInt32 aY, PRInt32* aRow, nsTreeColumn** aCol,
+  // An internal hit test.  aX and aY are expected to be in twips in the
+  // coordinate system of this frame.
+  void GetCellAt(nscoord aX, nscoord aY, PRInt32* aRow, nsTreeColumn** aCol,
                  nsIAtom** aChildElt);
 
+  // Retrieve the area for the twisty for a cell.
+  nsITheme* GetTwistyRect(PRInt32 aRowIndex,
+                          nsTreeColumn* aColumn,
+                          nsRect& aImageRect,
+                          nsRect& aTwistyRect,
+                          nsPresContext* aPresContext,
+                          nsIRenderingContext& aRenderingContext,
+                          nsStyleContext* aTwistyContext);
+
   // Fetch an image from the image cache.
   nsresult GetImage(PRInt32 aRowIndex, nsTreeColumn* aCol, PRBool aUseContext,
                     nsStyleContext* aStyleContext, PRBool& aAllowImageRegions, imgIContainer** aResult);
@@ -221,6 +260,12 @@ protected:
   // padding.  It does not include margins.
   nsRect GetImageSize(PRInt32 aRowIndex, nsTreeColumn* aCol, PRBool aUseContext, nsStyleContext* aStyleContext);
 
+  // Returns the destination size of the image, not including borders and padding.
+  nsSize GetImageDestSize(nsStyleContext* aStyleContext, PRBool useImageRegion, imgIContainer* image);
+
+  // Returns the source rectangle of the image to be displayed.
+  nsRect GetImageSourceRect(nsStyleContext* aStyleContext, PRBool useImageRegion, imgIContainer* image);
+
   // Returns the height of rows in the tree.
   PRInt32 GetRowHeight();
 
@@ -230,31 +275,43 @@ protected:
   // Calculates our width/height once border and padding have been removed.
   void CalcInnerBox();
 
+  // Calculate the total width of our scrollable portion
+  nscoord CalcHorzWidth(const ScrollParts& aParts);
+
   // Looks up a style context in the style cache.  On a cache miss we resolve
   // the pseudo-styles passed in and place them into the cache.
   nsStyleContext* GetPseudoStyleContext(nsIAtom* aPseudoElement);
 
-  // Makes |mScrollbar| non-null if at all possible, and returns it.
-  nsIFrame* EnsureScrollbar();
+  // Retrieves the scrollbars and scrollview relevant to this treebody. We
+  // traverse the frame tree under our base element, in frame order, looking
+  // for the first relevant vertical scrollbar, horizontal scrollbar, and
+  // scrollable frame (with associated content and scrollable view). These
+  // are all volatile and should not be retained.
+  ScrollParts GetScrollParts();
 
   // Update the curpos of the scrollbar.
-  void UpdateScrollbar();
+  void UpdateScrollbars(const ScrollParts& aParts);
 
   // Update the maxpos of the scrollbar.
-  void InvalidateScrollbar();
+  void InvalidateScrollbars(const ScrollParts& aParts);
 
-  // Check vertical overflow.
-  void CheckVerticalOverflow();
+  // Check overflow and generate events.
+  void CheckOverflow(const ScrollParts& aParts);
 
   // Use to auto-fill some of the common properties without the view having to do it.
   // Examples include container, open, selected, and focus.
   void PrefillPropertyArray(PRInt32 aRowIndex, nsTreeColumn* aCol);
 
   // Our internal scroll method, used by all the public scroll methods.
-  nsresult ScrollInternal(PRInt32 aRow);
+  nsresult ScrollInternal(const ScrollParts& aParts, PRInt32 aRow);
+  nsresult ScrollToRowInternal(const ScrollParts& aParts, PRInt32 aRow);
+  nsresult ScrollToColumnInternal(const ScrollParts& aParts, nsITreeColumn* aCol);
+  nsresult ScrollHorzInternal(const ScrollParts& aParts, PRInt32 aPosition);
+  nsresult EnsureRowIsVisibleInternal(const ScrollParts& aParts, PRInt32 aRow);
   
-  // Convert pixels, probably from an event, into twips in our coordinate space.
-  void AdjustEventCoordsToBoxCoordSpace(PRInt32 aX, PRInt32 aY, PRInt32* aResultX, PRInt32* aResultY);
+  // Convert client pixels into twips in our coordinate space.
+  void AdjustClientCoordsToBoxCoordSpace(PRInt32 aX, PRInt32 aY,
+                                         nscoord* aResultX, nscoord* aResultY);
 
   // Convert a border style into line style.
   nsLineStyle ConvertBorderStyleToLineStyle(PRUint8 aBorderStyle);
@@ -272,6 +329,17 @@ protected:
                     nscoord& aDesiredSize, nscoord& aCurrentSize);
   nscoord CalcMaxRowWidth();
 
+  // Translate the given rect horizontally from tree coordinates into the
+  // coordinate system of our nsTreeBodyFrame.  If clip is true, then clip the
+  // rect to its intersection with mInnerBox in the horizontal direction.
+  // Return whether the result has a nonempty intersection with mInnerBox
+  // after projecting both onto the horizontal coordinate axis.
+  PRBool OffsetForHorzScroll(nsRect& rect, PRBool clip);
+
+  // Fills in rect with column args and x/width coords. Does some extra 
+  // calculations for the last column
+  void CalcColumnRect(nsRect& rect, nsTreeColumn* aCol, nscoord y, nscoord height);
+
   PRBool CanAutoScroll(PRInt32 aRowIndex);
 
   // Calc the row and above/below/on status given where the mouse currently is hovering.
@@ -307,9 +375,6 @@ protected:
   static void ScrollCallback(nsITimer *aTimer, void *aClosure);
 
 protected: // Data Members
-  // Our cached pres context.
-  nsPresContext* mPresContext;
-
   // The cached box object parent.
   nsCOMPtr<nsITreeBoxObject> mTreeBoxObject;
 
@@ -326,14 +391,11 @@ protected: // Data Members
   // (the power set of all row properties).
   nsTreeStyleCache mStyleCache;
 
-  // A hashtable that maps from URLs to image requests.  The URL is provided
-  // by the view or by the style context. The style context represents
-  // a resolved :-moz-tree-cell-image (or twisty) pseudo-element.
+  // A hashtable that maps from URLs to image request/listener pairs.  The URL
+  // is provided by the view or by the style context. The style context
+  // represents a resolved :-moz-tree-cell-image (or twisty) pseudo-element.
   // It maps directly to an imgIRequest.
-  nsSupportsHashtable* mImageCache;
-
-  // Our vertical scrollbar.
-  nsIFrame* mScrollbar;
+  nsDataHashtable<nsStringHashKey, nsTreeImageCacheEntry> mImageCache;
 
   // The index of the first visible row and the # of rows visible onscreen.  
   // The tree only examines onscreen rows, starting from
@@ -341,6 +403,12 @@ protected: // Data Members
   PRInt32 mTopRowIndex;
   PRInt32 mPageLength;
 
+  // The horizontal scroll position
+  nscoord mHorzPosition;
+  // Our desired horizontal width (the width for which we actually have tree
+  // columns).
+  nscoord mHorzWidth;
+
   // Cached heights and indent info.
   nsRect mInnerBox;
   PRInt32 mRowHeight;
@@ -357,6 +425,7 @@ protected: // Data Members
   PRPackedBool mHasFixedRowCount;
 
   PRPackedBool mVerticalOverflow;
+  PRPackedBool mHorizontalOverflow;
 
   PRPackedBool mReflowCallbackPosted;
 
@@ -391,6 +460,9 @@ protected: // Data Members
       // Number of lines to be scrolled.
       PRInt16                  mScrollLines;
 
+      // The drag action that was received for this slot
+      PRUint32                 mDragAction;
+
       nsCOMPtr<nsIDragSession> mDragSession;
 
       // Timer for opening/closing spring loaded folders or scrolling the tree.
