@@ -53,6 +53,9 @@
 #include "nsGUIEvent.h"
 #include "nsINameSpaceManager.h"
 #include "nsXULAtoms.h"
+#include "nsPLDOMEvent.h"
+#include "nsEventDispatcher.h"
+#include "nsAutoPtr.h"
 
 // A helper class for managing our ranges of selection.
 struct nsTreeRange
@@ -212,7 +215,8 @@ struct nsTreeRange
   };
 
   void Invalidate() {
-    mSelection->mTree->InvalidateRange(mMin, mMax);
+    if (mSelection->mTree)
+      mSelection->mTree->InvalidateRange(mMin, mMax);
     if (mNext)
       mNext->Invalidate();
   };
@@ -253,12 +257,12 @@ struct nsTreeRange
 };
 
 nsTreeSelection::nsTreeSelection(nsITreeBoxObject* aTree)
+  : mTree(aTree),
+    mSuppressed(PR_FALSE),
+    mCurrentIndex(-1),
+    mShiftSelectPivot(-1),
+    mFirstRange(nsnull)
 {
-  mTree = aTree;
-  mSuppressed = PR_FALSE;
-  mFirstRange = nsnull;
-  mShiftSelectPivot = -1;
-  mCurrentIndex = -1;
 }
 
 nsTreeSelection::~nsTreeSelection()
@@ -296,12 +300,19 @@ NS_IMETHODIMP nsTreeSelection::SetTree(nsITreeBoxObject * aTree)
 NS_IMETHODIMP nsTreeSelection::GetSingle(PRBool* aSingle)
 {
   nsCOMPtr<nsIBoxObject> boxObject = do_QueryInterface(mTree);
+
   nsCOMPtr<nsIDOMElement> element;
   boxObject->GetElement(getter_AddRefs(element));
+
   nsCOMPtr<nsIContent> content = do_QueryInterface(element);
-  nsAutoString seltype;
-  content->GetAttr(kNameSpaceID_None, nsXULAtoms::seltype, seltype);
-  *aSingle = seltype.EqualsLiteral("single");
+
+  static nsIContent::AttrValuesArray strings[] =
+    {&nsXULAtoms::single, &nsXULAtoms::cell, &nsXULAtoms::text, nsnull};
+
+  *aSingle = content->FindAttrValueIn(kNameSpaceID_None,
+                                      nsXULAtoms::seltype,
+                                      strings, eCaseMatters) >= 0;
+
   return NS_OK;
 }
 
@@ -344,7 +355,9 @@ NS_IMETHODIMP nsTreeSelection::Select(PRInt32 aIndex)
 {
   mShiftSelectPivot = -1;
 
-  SetCurrentIndex(aIndex);
+  nsresult rv = SetCurrentIndex(aIndex);
+  if (NS_FAILED(rv))
+    return rv;
 
   if (mFirstRange) {
     PRBool alreadySelected = mFirstRange->Contains(aIndex);
@@ -388,9 +401,10 @@ NS_IMETHODIMP nsTreeSelection::ToggleSelect(PRInt32 aIndex)
   // (5) The addition of the item causes two ranges to be merged.
   // (6) The removal of the item causes two ranges to be split.
   mShiftSelectPivot = -1;
-  SetCurrentIndex(aIndex);
+  nsresult rv = SetCurrentIndex(aIndex);
+  if (NS_FAILED(rv))
+    return rv;
 
-  nsresult rv = NS_OK;
   if (!mFirstRange)
     Select(aIndex);
   else {
@@ -403,7 +417,8 @@ NS_IMETHODIMP nsTreeSelection::ToggleSelect(PRInt32 aIndex)
     else
       rv = mFirstRange->Remove(aIndex);
     if (NS_SUCCEEDED(rv)) {
-      mTree->InvalidateRow(aIndex);
+      if (mTree)
+        mTree->InvalidateRow(aIndex);
 
       FireOnSelectHandler();
     }
@@ -434,7 +449,9 @@ NS_IMETHODIMP nsTreeSelection::RangedSelect(PRInt32 aStartIndex, PRInt32 aEndInd
   }
 
   mShiftSelectPivot = aStartIndex;
-  SetCurrentIndex(aEndIndex);
+  nsresult rv = SetCurrentIndex(aEndIndex);
+  if (NS_FAILED(rv))
+    return rv;
   
   PRInt32 start = aStartIndex < aEndIndex ? aStartIndex : aEndIndex;
   PRInt32 end = aStartIndex < aEndIndex ? aEndIndex : aStartIndex;
@@ -465,7 +482,9 @@ NS_IMETHODIMP nsTreeSelection::RangedSelect(PRInt32 aStartIndex, PRInt32 aEndInd
 
 NS_IMETHODIMP nsTreeSelection::ClearRange(PRInt32 aStartIndex, PRInt32 aEndIndex)
 {
-  SetCurrentIndex(aEndIndex);
+  nsresult rv = SetCurrentIndex(aEndIndex);
+  if (NS_FAILED(rv))
+    return rv;
 
   if (mFirstRange) {
     PRInt32 start = aStartIndex < aEndIndex ? aStartIndex : aEndIndex;
@@ -473,7 +492,8 @@ NS_IMETHODIMP nsTreeSelection::ClearRange(PRInt32 aStartIndex, PRInt32 aEndIndex
 
     mFirstRange->RemoveRange(start, end);
 
-    mTree->InvalidateRange(start, end);
+    if (mTree)
+      mTree->InvalidateRange(start, end);
   }
   
   return NS_OK;
@@ -500,6 +520,9 @@ NS_IMETHODIMP nsTreeSelection::InvertSelection()
 
 NS_IMETHODIMP nsTreeSelection::SelectAll()
 {
+  if (!mTree)
+    return NS_OK;
+
   nsCOMPtr<nsITreeView> view;
   mTree->GetView(getter_AddRefs(view));
   if (!view)
@@ -589,14 +612,69 @@ NS_IMETHODIMP nsTreeSelection::GetCurrentIndex(PRInt32 *aCurrentIndex)
 
 NS_IMETHODIMP nsTreeSelection::SetCurrentIndex(PRInt32 aIndex)
 {
-  if (mCurrentIndex != -1)
+  if (!mTree) {
+    return NS_ERROR_UNEXPECTED;
+  }
+  if (mCurrentIndex == aIndex) {
+    return NS_OK;
+  }
+  if (mCurrentIndex != -1 && mTree)
     mTree->InvalidateRow(mCurrentIndex);
   
   mCurrentIndex = aIndex;
+  if (!mTree)
+    return NS_OK;
   
   if (aIndex != -1)
     mTree->InvalidateRow(aIndex);
 
+  // Fire DOMMenuItemActive event for tree
+  nsCOMPtr<nsIBoxObject> boxObject = do_QueryInterface(mTree);
+  NS_ASSERTION(boxObject, "no box object!");
+  if (!boxObject)
+    return NS_ERROR_UNEXPECTED;
+  nsCOMPtr<nsIDOMElement> treeElt;
+  boxObject->GetElement(getter_AddRefs(treeElt));
+
+  nsCOMPtr<nsIDOMNode> treeDOMNode(do_QueryInterface(treeElt));
+  NS_ENSURE_TRUE(treeDOMNode, NS_ERROR_UNEXPECTED);
+
+  nsRefPtr<nsPLDOMEvent> event = new nsPLDOMEvent(treeDOMNode,
+                                         NS_LITERAL_STRING("DOMMenuItemActive"));
+  if (!event)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  return event->PostDOMEvent();
+}
+
+NS_IMETHODIMP nsTreeSelection::GetCurrentColumn(nsITreeColumn** aCurrentColumn)
+{
+  NS_IF_ADDREF(*aCurrentColumn = mCurrentColumn);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsTreeSelection::SetCurrentColumn(nsITreeColumn* aCurrentColumn)
+{
+  if (mCurrentColumn == aCurrentColumn) {
+    return NS_OK;
+  }
+
+  if (mCurrentColumn) {
+    if (mFirstRange)
+      mTree->InvalidateCell(mFirstRange->mMin, mCurrentColumn);
+    if (mCurrentIndex != -1)
+      mTree->InvalidateCell(mCurrentIndex, mCurrentColumn);
+  }
+  
+  mCurrentColumn = aCurrentColumn;
+  
+  if (mCurrentColumn) {
+    if (mFirstRange)
+      mTree->InvalidateCell(mFirstRange->mMin, mCurrentColumn);
+    if (mCurrentIndex != -1)
+      mTree->InvalidateCell(mCurrentIndex, mCurrentColumn);
+  }
+
   return NS_OK;
 }
 
@@ -729,7 +807,7 @@ nsTreeSelection::GetShiftSelectPivot(PRInt32* aIndex)
 nsresult
 nsTreeSelection::FireOnSelectHandler()
 {
-  if (mSuppressed)
+  if (mSuppressed || !mTree)
     return NS_OK;
 
   nsCOMPtr<nsIBoxObject> boxObject = do_QueryInterface(mTree);
@@ -753,10 +831,9 @@ nsTreeSelection::FireOnSelectHandler()
     nsCOMPtr<nsPresContext> aPresContext = shell->GetPresContext();
 
     nsEventStatus status = nsEventStatus_eIgnore;
-    nsEvent event(NS_FORM_SELECTED);
+    nsEvent event(PR_TRUE, NS_FORM_SELECTED);
 
-    content->HandleDOMEvent(aPresContext, &event, nsnull, NS_EVENT_FLAG_INIT,
-                            &status);
+    nsEventDispatcher::Dispatch(content, aPresContext, &event, nsnull, &status);
   }
 
   return NS_OK;
