@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Created by: Paul Sandoz   <paul.sandoz@sun.com>
+ *   Dan Mosedale <dan.mosedale@oracle.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -206,6 +207,21 @@ NS_IMETHODIMP nsAbDirectoryQueryArguments::GetReturnProperties(PRUint32* returnP
     return NS_OK;
 }
 
+NS_IMETHODIMP nsAbDirectoryQueryArguments::GetTypeSpecificArg(nsISupports** aArg)
+{
+    NS_ENSURE_ARG_POINTER(aArg);
+
+    NS_IF_ADDREF(*aArg = mTypeSpecificArg);
+    return NS_OK;
+}
+
+NS_IMETHODIMP nsAbDirectoryQueryArguments::SetTypeSpecificArg(nsISupports* aArg)
+{
+    mTypeSpecificArg = aArg;
+    return NS_OK;
+}
+
+
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsAbDirectoryQueryPropertyValue, nsIAbDirectoryQueryPropertyValue)
 
 nsAbDirectoryQueryPropertyValue::nsAbDirectoryQueryPropertyValue()
@@ -233,15 +249,9 @@ nsAbDirectoryQueryPropertyValue::~nsAbDirectoryQueryPropertyValue()
 /* read only attribute string name; */
 NS_IMETHODIMP nsAbDirectoryQueryPropertyValue::GetName(char*  *aName)
 {
-    const char* value = mName.get ();
-
-    if (!value)
-        *aName = 0;
-    else
-        *aName = ToNewCString(mName);
+    *aName = mName.IsEmpty() ? 0 : ToNewCString(mName);
 
     return NS_OK;
-
 }
 
 /* read only attribute wstring value; */
@@ -372,12 +382,10 @@ nsresult nsAbDirectoryQuery::query (nsIAbDirectory* directory,
     nsIAbDirectoryQueryResultListener* listener,
     PRInt32* resultLimit)
 {
-    nsresult rv = NS_OK;
-
     if (*resultLimit == 0)
-        return rv;
+        return NS_OK;
 
-    rv = queryCards (directory, arguments, listener, resultLimit);
+    nsresult rv = queryCards (directory, arguments, listener, resultLimit);
     NS_ENSURE_SUCCESS(rv, rv);
 
     PRBool doSubDirectories;
@@ -425,7 +433,7 @@ nsresult nsAbDirectoryQuery::queryCards (nsIAbDirectory* directory,
 {
     nsresult rv = NS_OK;
 
-    nsCOMPtr<nsIEnumerator> cards;
+    nsCOMPtr<nsISimpleEnumerator> cards;
     rv = directory->GetChildCards(getter_AddRefs(cards));
     if (NS_FAILED(rv))
     {
@@ -438,14 +446,11 @@ nsresult nsAbDirectoryQuery::queryCards (nsIAbDirectory* directory,
     if (!cards)
         return NS_OK;
 
-    rv = cards->First();
-    if (NS_FAILED(rv))
-        return NS_OK;
-
-    do
+    PRBool more;
+    while (NS_SUCCEEDED(cards->HasMoreElements(&more)) && more)
     {
         nsCOMPtr<nsISupports> item;
-        rv = cards->CurrentItem(getter_AddRefs(item));
+        rv = cards->GetNext(getter_AddRefs(item));
         NS_ENSURE_SUCCESS(rv, rv);
 
         nsCOMPtr<nsIAbCard> card(do_QueryInterface(item, &rv));
@@ -456,10 +461,7 @@ nsresult nsAbDirectoryQuery::queryCards (nsIAbDirectory* directory,
 
         if (*resultLimit == 0)
             return NS_OK;
-
-        rv = cards->Next ();
     }
-    while (rv == NS_OK);
 
     return NS_OK;
 }
@@ -469,10 +471,8 @@ nsresult nsAbDirectoryQuery::matchCard (nsIAbCard* card,
     nsIAbDirectoryQueryResultListener* listener,
     PRInt32* resultLimit)
 {
-    nsresult rv = NS_OK;
-
     nsCOMPtr<nsISupports> supportsExpression;
-    rv = arguments->GetExpression (getter_AddRefs (supportsExpression));
+    nsresult rv = arguments->GetExpression (getter_AddRefs (supportsExpression));
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIAbBooleanExpression> expression(do_QueryInterface(supportsExpression, &rv));
@@ -496,10 +496,8 @@ nsresult nsAbDirectoryQuery::matchCardExpression (nsIAbCard* card,
     nsIAbBooleanExpression* expression,
     PRBool* result)
 {
-    nsresult rv = NS_OK;
-
     nsAbBooleanOperationType operation;
-    rv = expression->GetOperation (&operation);
+    nsresult rv = expression->GetOperation (&operation);
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsISupportsArray> childExpressions;
@@ -556,10 +554,8 @@ nsresult nsAbDirectoryQuery::matchCardCondition (nsIAbCard* card,
     nsIAbBooleanConditionString* condition,
     PRBool* matchFound)
 {
-    nsresult rv = NS_OK;
-
     nsAbBooleanConditionType conditionType;
-    rv = condition->GetCondition (&conditionType);
+    nsresult rv = condition->GetCondition (&conditionType);
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsXPIDLCString name;
@@ -568,26 +564,19 @@ nsresult nsAbDirectoryQuery::matchCardCondition (nsIAbCard* card,
 
     if (name.Equals ("card:nsIAbCard"))
     {
-        if (conditionType == nsIAbBooleanConditionTypes::Exists)
-            *matchFound = PR_TRUE;
-        else
-            *matchFound = PR_FALSE;
-        return NS_OK;
+      *matchFound = (conditionType == nsIAbBooleanConditionTypes::Exists);
+      return NS_OK;
     }
 
-    nsXPIDLString _value;
-    rv = card->GetCardValue (name.get (), getter_Copies (_value));
+    nsXPIDLString value;
+    rv = card->GetCardValue (name.get (), getter_Copies (value));
     NS_ENSURE_SUCCESS(rv, rv);
-    nsString value (_value.get ());
 
-    if (!value.get () || value.Length () == 0)
+    if (value.IsEmpty())
     {
-        if (conditionType == nsIAbBooleanConditionTypes::DoesNotExist)
-            *matchFound = PR_TRUE;
-        else
-            *matchFound = PR_FALSE;
-
-        return NS_OK;
+      *matchFound = (conditionType == nsIAbBooleanConditionTypes::DoesNotExist) ?
+          PR_TRUE : PR_FALSE;
+      return NS_OK;
     }
 
     nsXPIDLString matchValue;
@@ -709,8 +698,7 @@ nsresult nsAbDirectoryQuery::queryMatch (nsIAbCard* card,
         return NS_ERROR_OUT_OF_MEMORY;
     queryResult = _queryResult;
 
-    rv = listener->OnQueryItem (queryResult);
-    return rv;
+    return listener->OnQueryItem (queryResult);
 }
 
 nsresult nsAbDirectoryQuery::queryFinished (nsIAbDirectoryQueryArguments* arguments,
