@@ -23,6 +23,7 @@
  *   Seth Spitzer <sspitzer@netscape.com>
  *   Dan Mosedale <dmose@netscape.com>
  *   Paul Sandoz <paul.sandoz@sun.com>
+ *   Mark Banner <mark@standard8.demon.co.uk>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -40,9 +41,10 @@
 
 #include "nsAbLDAPDirectoryQuery.h"
 #include "nsAbBoolExprToLDAPFilter.h"
-#include "nsAbLDAPProperties.h"
+#include "nsILDAPMessage.h"
 #include "nsILDAPErrors.h"
 #include "nsILDAPOperation.h"
+#include "nsIAbLDAPAttributeMap.h"
 #include "nsAbUtils.h"
 
 #include "nsIAuthPrompt.h"
@@ -51,10 +53,13 @@
 #include "nsAutoLock.h"
 #include "nsIProxyObjectManager.h"
 #include "prprf.h"
+#include "nsCRT.h"
 #include "nsIWindowWatcher.h"
 #include "nsIDOMWindow.h"
 #include "nsICategoryManager.h"
+#include "nsIServiceManager.h"
 #include "nsCategoryManagerUtils.h"
+#include "nsAbLDAPDirectory.h"
 
 class nsAbQueryLDAPMessageListener : public nsILDAPMessageListener
 {
@@ -136,14 +141,13 @@ nsAbQueryLDAPMessageListener::nsAbQueryLDAPMessageListener (
     mLock(0)
 
 {
-    NS_ADDREF(mDirectoryQuery);
+  NS_ADDREF(mDirectoryQuery);
 }
 
 nsAbQueryLDAPMessageListener::~nsAbQueryLDAPMessageListener ()
 {
     if (mLock)
         PR_DestroyLock (mLock);
-
     NS_RELEASE(mDirectoryQuery);
 }
 
@@ -224,8 +228,8 @@ NS_IMETHODIMP nsAbQueryLDAPMessageListener::OnLDAPMessage(nsILDAPMessage *aMessa
         case nsILDAPMessage::RES_SEARCH_ENTRY:
             if (!mFinished && !mWaitingForPrevQueryToFinish)
             {
-            rv = OnLDAPMessageSearchEntry (aMessage, getter_AddRefs (queryResult));
-            NS_ENSURE_SUCCESS(rv, rv);
+                rv = OnLDAPMessageSearchEntry (aMessage, 
+                                               getter_AddRefs (queryResult));
             }
             break;
         case nsILDAPMessage::RES_SEARCH_RESULT:
@@ -239,7 +243,7 @@ NS_IMETHODIMP nsAbQueryLDAPMessageListener::OnLDAPMessage(nsILDAPMessage *aMessa
     else
     {
         if (mSearchOperation)
-            rv = mSearchOperation->Abandon ();
+            rv = mSearchOperation->AbandonExt ();
 
         rv = QueryResultStatus (nsnull, getter_AddRefs (queryResult), nsIAbDirectoryQueryResult::queryResultStopped);
         // reset because we might re-use this listener...except don't do this
@@ -330,7 +334,7 @@ NS_IMETHODIMP nsAbQueryLDAPMessageListener::OnLDAPInit(nsILDAPConnection *aConn,
         // bug in egcs 1.1.2 (the version of gcc that comes with Red Hat 6.2),
         // which is the default compiler for Mozilla on linux at the moment.
         //
-        NS_ConvertASCIItoUCS2 hostTemp(host);
+        NS_ConvertASCIItoUTF16 hostTemp(host);
         const PRUnichar *hostArray[1] = { hostTemp.get() };
 
         // format the hostname into the authprompt text string
@@ -386,7 +390,7 @@ NS_IMETHODIMP nsAbQueryLDAPMessageListener::OnLDAPInit(nsILDAPConnection *aConn,
         //
         rv = authPrompter->PromptPassword(
             authPromptTitle.get(), authPromptText.get(),
-            NS_ConvertUTF8toUCS2(spec).get(),
+            NS_ConvertUTF8toUTF16(spec).get(),
             nsIAuthPrompt::SAVE_PASSWORD_PERMANENTLY, getter_Copies(passwd),
             &status);
         if (NS_FAILED(rv) || !status) {
@@ -400,17 +404,17 @@ NS_IMETHODIMP nsAbQueryLDAPMessageListener::OnLDAPInit(nsILDAPConnection *aConn,
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsILDAPMessageListener> proxyListener;
-    rv = NS_GetProxyForObject(NS_UI_THREAD_EVENTQ,
+    rv = NS_GetProxyForObject(NS_PROXY_TO_MAIN_THREAD,
                      NS_GET_IID(nsILDAPMessageListener),
                      NS_STATIC_CAST(nsILDAPMessageListener *, this),
-                     PROXY_SYNC | PROXY_ALWAYS,
+                     NS_PROXY_SYNC | NS_PROXY_ALWAYS,
                      getter_AddRefs(proxyListener));
 
     rv = ldapOperation->Init(mConnection, proxyListener, nsnull);
     NS_ENSURE_SUCCESS(rv, rv);
 
     // Bind
-    rv = ldapOperation->SimpleBind(NS_ConvertUCS2toUTF8(passwd));
+    rv = ldapOperation->SimpleBind(NS_ConvertUTF16toUTF8(passwd));
     NS_ENSURE_SUCCESS(rv, rv);
 
     return rv;
@@ -469,13 +473,11 @@ nsresult nsAbQueryLDAPMessageListener::OnLDAPMessageBind (nsILDAPMessage *aMessa
     mSearchOperation = do_CreateInstance(NS_LDAPOPERATION_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsIProxyObjectManager> proxyMgr = 
-	    do_GetService(NS_XPCOMPROXY_CONTRACTID, &rv);
-	  NS_ENSURE_SUCCESS(rv, rv);
-    
     nsCOMPtr<nsILDAPMessageListener> proxyListener;
-    rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsILDAPMessageListener),
-									this, PROXY_SYNC | PROXY_ALWAYS, getter_AddRefs(proxyListener));
+    rv = NS_GetProxyForObject( NS_PROXY_TO_MAIN_THREAD,
+                               NS_GET_IID(nsILDAPMessageListener),
+                               this, NS_PROXY_SYNC | NS_PROXY_ALWAYS,
+                               getter_AddRefs(proxyListener));
     NS_ENSURE_SUCCESS(rv, rv);
 
     rv = mSearchOperation->Init (mConnection, proxyListener, nsnull);
@@ -497,6 +499,22 @@ nsresult nsAbQueryLDAPMessageListener::OnLDAPMessageBind (nsILDAPMessage *aMessa
     rv = mUrl->GetAttributes (attributes.GetSizeAddr (), attributes.GetArrayAddr ());
     NS_ENSURE_SUCCESS(rv, rv);
 
+    // I don't _think_ it's ever actually possible to get here without having
+    // an nsAbLDAPDirectory object, but, just in case, I'll do a QI instead
+    // of just static casts...
+    nsCOMPtr<nsIAbLDAPDirectory> nsIAbDir = 
+        do_QueryInterface(mDirectoryQuery, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    nsAbLDAPDirectory *dir = 
+        NS_STATIC_CAST(nsAbLDAPDirectory *,
+                       NS_STATIC_CAST(nsIAbLDAPDirectory *, nsIAbDir.get()));
+
+    rv = mSearchOperation->SetServerControls(dir->mSearchServerControls.get());
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = mSearchOperation->SetClientControls(dir->mSearchClientControls.get());
+    NS_ENSURE_SUCCESS(rv, rv);
+
     rv = mSearchOperation->SearchExt (dn, scope, filter,
             attributes.GetSize (), attributes.GetArray (),
             mTimeOut, mResultLimit);
@@ -507,106 +525,82 @@ nsresult nsAbQueryLDAPMessageListener::OnLDAPMessageSearchEntry (nsILDAPMessage
         nsIAbDirectoryQueryResult** result)
 {
     nsresult rv;
-    nsCOMPtr<nsISupportsArray> propertyValues;
 
+    // the address book fields that we'll be asking for
     CharPtrArrayGuard properties;
     rv = mQueryArguments->GetReturnProperties (properties.GetSizeAddr(), properties.GetArrayAddr());
     NS_ENSURE_SUCCESS(rv, rv);
 
-    CharPtrArrayGuard attrs;
-    rv = aMessage->GetAttributes(attrs.GetSizeAddr(), attrs.GetArrayAddr());
+    // the map for translating between LDAP attrs <-> addrbook fields
+    nsCOMPtr<nsISupports> iSupportsMap;
+    rv = mQueryArguments->GetTypeSpecificArg(getter_AddRefs(iSupportsMap));
+    NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr<nsIAbLDAPAttributeMap> map = do_QueryInterface(iSupportsMap, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCAutoString propertyName;
-    for (PRUint32 i = 0; i < properties.GetSize(); i++)
-    {
-        propertyName.Assign (properties[i]);
+    // set up variables for handling the property values to be returned
+    nsCOMPtr<nsISupportsArray> propertyValues;
+    nsCOMPtr<nsIAbDirectoryQueryPropertyValue> propertyValue;
+    rv = NS_NewISupportsArray(getter_AddRefs(propertyValues));
+    NS_ENSURE_SUCCESS(rv, rv);
+        
+    if (!strcmp(properties[0], "card:nsIAbCard")) {
+        // Meta property
+        nsCAutoString dn;
+        rv = aMessage->GetDn (dn);
+        NS_ENSURE_SUCCESS(rv, rv);
 
-        nsAbDirectoryQueryPropertyValue* _propertyValue = 0;
-        if (propertyName.Equals("card:nsIAbCard"))
-        {
-            // Meta property
-            nsCAutoString dn;
-            rv = aMessage->GetDn (dn);
-            NS_ENSURE_SUCCESS(rv, rv);
+        nsCOMPtr<nsIAbCard> card;
+        rv = mDirectoryQuery->CreateCard (mUrl, dn.get(), getter_AddRefs (card));
+        NS_ENSURE_SUCCESS(rv, rv);
 
-            nsCOMPtr<nsIAbCard> card;
-            rv = mDirectoryQuery->CreateCard (mUrl, dn.get(), getter_AddRefs (card));
-            NS_ENSURE_SUCCESS(rv, rv);
+        rv = map->SetCardPropertiesFromLDAPMessage(aMessage, card);
+        NS_ENSURE_SUCCESS(rv, rv);
 
-            PRBool hasSetCardProperty = PR_FALSE;
-            rv = MozillaLdapPropertyRelator::createCardPropertyFromLDAPMessage (aMessage,
-                    card,
-                    &hasSetCardProperty);
-            NS_ENSURE_SUCCESS(rv, rv);
+        propertyValue = new nsAbDirectoryQueryPropertyValue(properties[0], card);
+        if (!propertyValue) 
+            return NS_ERROR_OUT_OF_MEMORY;
 
-            if (!hasSetCardProperty)
-                continue;
+        rv = propertyValues->AppendElement(propertyValue);
+        NS_ENSURE_SUCCESS(rv, rv);
+    } else {
 
-            _propertyValue = new nsAbDirectoryQueryPropertyValue(propertyName.get (), card);
-            if (_propertyValue == NULL)
-                return NS_ERROR_OUT_OF_MEMORY;
-
-        }
-        else
-        {
-            if (!MozillaLdapPropertyRelator::findLdapPropertyFromMozilla (propertyName.get ()))
-                continue;
-
-            const MozillaLdapPropertyRelation* relation ;
-
-            for (PRUint32 j = 0; j < attrs.GetSize(); j++)
+        for (PRUint32 i = 0; i < properties.GetSize(); i++)
             {
-                relation = MozillaLdapPropertyRelator::findMozillaPropertyFromLdap (attrs[j]);
-                if (!relation)
+                // this is the precedence order list of attrs for this property
+                CharPtrArrayGuard attrs;
+                rv = map->GetAttributes(nsDependentCString(properties[i]),
+                                        attrs.GetSizeAddr(),
+                                        attrs.GetArrayAddr());
+
+                // if there are no attrs for this property, just move on
+                if (NS_FAILED(rv) || !strlen(attrs[0])) {
                     continue;
-                /*
-                 * This change is necessary due to a side effect of #124022. The list of
-                 * requested attributes is created in reverse order than how they appear
-                 * in nsAbLDAPProperties.cpp. Thus while "surname" and "sn" both map to
-                 * LastName, "sn" will be returned by findLdapPropertyFromMozilla() as it
-                 * appears first in the hash table but "surname" will be returned by the 
-                 * request. 
-                 *
-                 * Rather than simply reversing the order, an alternative is to compare
-                 * the mapped Mozilla attributes where a one to one match exists.
-                */
+                }
 
-                if (nsCRT::strcasecmp (relation->mozillaProperty, propertyName.get()) == 0)
-                {
+                // iterate through list, until first property found
+                for (PRUint32 j=0; j < attrs.GetSize(); j++) {
+
+                    // try and get the values for this ldap attribute
                     PRUnicharPtrArrayGuard vals;
-                    rv = aMessage->GetValues (attrs[j], vals.GetSizeAddr(), vals.GetArrayAddr());
-                    NS_ENSURE_SUCCESS(rv, rv);
+                    rv = aMessage->GetValues(attrs[j], vals.GetSizeAddr(),
+                                             vals.GetArrayAddr());
 
-                    if (vals.GetSize() == 0)
+                    if (NS_SUCCEEDED(rv) && vals.GetSize()) {
+                        propertyValue = new nsAbDirectoryQueryPropertyValue(
+                            properties[i], vals[0]);
+                        if (!propertyValue) {
+                            return NS_ERROR_OUT_OF_MEMORY;
+                        }
+
+                        (void)propertyValues->AppendElement (propertyValue);
                         break;
-
-                    _propertyValue = new nsAbDirectoryQueryPropertyValue(propertyName.get (), vals[0]);
-                    if (_propertyValue == NULL)
-                        return NS_ERROR_OUT_OF_MEMORY;
-                    break;
+                    }
                 }
             }
-        }
-
-        if (_propertyValue)
-        {
-            nsCOMPtr<nsIAbDirectoryQueryPropertyValue> propertyValue;
-            propertyValue = _propertyValue;
-
-            if (!propertyValues)
-            {
-                NS_NewISupportsArray(getter_AddRefs(propertyValues));
-            }
-
-            propertyValues->AppendElement (propertyValue);
-        }
     }
 
-    if (!propertyValues)
-        return NS_OK;
-
-    return QueryResultStatus (propertyValues, result,nsIAbDirectoryQueryResult::queryResultMatch);
+    return QueryResultStatus (propertyValues, result, nsIAbDirectoryQueryResult::queryResultMatch);
 }
 nsresult nsAbQueryLDAPMessageListener::OnLDAPMessageSearchResult (nsILDAPMessage *aMessage,
         nsIAbDirectoryQueryResult** result)
@@ -703,8 +697,18 @@ NS_IMETHODIMP nsAbLDAPDirectoryQuery::DoQuery(nsIAbDirectoryQueryArguments* argu
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsIAbBooleanExpression> expression (do_QueryInterface (supportsExpression, &rv));
-    nsCString filter;
-    rv = nsAbBoolExprToLDAPFilter::Convert (expression, filter);
+    nsCAutoString filter;
+
+    // figure out how we map attribute names to addressbook fields for this
+    // query
+    nsCOMPtr<nsISupports> iSupportsMap;
+    rv = arguments->GetTypeSpecificArg(getter_AddRefs(iSupportsMap));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsIAbLDAPAttributeMap> map = do_QueryInterface(iSupportsMap, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = nsAbBoolExprToLDAPFilter::Convert (map, expression, filter);
     NS_ENSURE_SUCCESS(rv, rv);
 
     /*
@@ -722,7 +726,7 @@ NS_IMETHODIMP nsAbLDAPDirectoryQuery::DoQuery(nsIAbDirectoryQueryArguments* argu
     */
     if(filter.IsEmpty())
     {
-        filter += NS_LITERAL_CSTRING("(objectclass=inetorgperson)");
+        filter.AssignLiteral("(objectclass=inetorgperson)");
     }
 
     // Set up the search ldap url
@@ -863,33 +867,51 @@ nsresult nsAbLDAPDirectoryQuery::getLdapReturnAttributes (
     nsresult rv;
 
     CharPtrArrayGuard properties;
-    rv = arguments->GetReturnProperties (properties.GetSizeAddr(), properties.GetArrayAddr());
+    rv = arguments->GetReturnProperties(properties.GetSizeAddr(),
+                                        properties.GetArrayAddr());
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCAutoString propertyName;
+    // figure out how we map attribute names to addressbook fields for this
+    // query
+    nsCOMPtr<nsISupports> iSupportsMap;
+    rv = arguments->GetTypeSpecificArg(getter_AddRefs(iSupportsMap));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsIAbLDAPAttributeMap> map = do_QueryInterface(iSupportsMap, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    if (!strcmp(properties[0], "card:nsIAbCard")) {
+        // Meta property
+        // require all attributes
+        //
+        rv = map->GetAllCardAttributes(returnAttributes);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "GetAllSupportedAttributes failed");
+        return rv;
+    }
+
+    PRBool needComma = PR_FALSE;
     for (PRUint32 i = 0; i < properties.GetSize(); i++)
     {
-        propertyName.Assign (properties[i]);
+        nsCAutoString attrs;
 
-        if (propertyName.Equals("card:nsIAbCard"))
-        {
-            // Meta property
-            // require all attributes
-            //
-            rv = MozillaLdapPropertyRelator::GetAllSupportedLDAPAttributes(returnAttributes);
-            NS_ASSERTION(NS_SUCCEEDED(rv), "GetAllSupportedLDAPAttributes failed");
-            break;
+        // get all the attributes for this property
+        rv = map->GetAttributeList(nsDependentCString(properties[i]), attrs);
+
+        // if there weren't any attrs, just keep going
+        if (NS_FAILED(rv) || attrs.IsEmpty()) {
+            continue;
         }
 
-        const MozillaLdapPropertyRelation* tableEntry=
-            MozillaLdapPropertyRelator::findLdapPropertyFromMozilla (propertyName.get ());
-        if (!tableEntry)
-            continue;
+        // add a comma, if necessary
+        if (needComma) {
+            returnAttributes.Append(PRUnichar (','));
+        }
 
-        if (i)
-            returnAttributes.Append (",");
+        returnAttributes.Append(attrs);
 
-        returnAttributes.Append (tableEntry->ldapProperty);
+        // since we've added attrs, we definitely need a comma next time
+        // we're here
+        needComma = PR_TRUE;
     }
 
     return rv;
