@@ -15,12 +15,12 @@
  * The Original Code is mozilla.org code.
  *
  * The Initial Developer of the Original Code is
- * <rdayal@netscape.com>
- *
+ * Rajiv Dayal <rdayal@netscape.com>
  * Portions created by the Initial Developer are Copyright (C) 2002
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *  Dan Mosedale <dan.mosedale@oracle.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,22 +36,19 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-
+#include "nsILDAPMessage.h"
 #include "nsAbLDAPReplicationData.h"
-#include "nsLDAP.h"
 #include "nsIAbCard.h"
 #include "nsIAddrBookSession.h"
 #include "nsAbBaseCID.h"
 #include "nsAbUtils.h"
-#include "nsAbMDBCard.h"
-#include "nsAbLDAPCard.h"
-#include "nsFileSpec.h"
-#include "nsAbLDAPProperties.h"
+#include "nsIAbMDBCard.h"
 #include "nsAbLDAPReplicationQuery.h"
 #include "nsProxiedService.h"
 #include "nsCPasswordManager.h"
 #include "nsIRDFService.h"
 #include "nsIRDFResource.h"
+#include "nsILDAPErrors.h"
 
 // once bug # 101252 gets fixed, this should be reverted back to be non threadsafe
 // implementation is not really thread safe since each object should exist 
@@ -63,8 +60,7 @@ nsAbLDAPProcessReplicationData::nsAbLDAPProcessReplicationData()
    mProtocol(-1),
    mCount(0),
    mDBOpen(PR_FALSE),
-   mInitialized(PR_FALSE),
-   mDirServerInfo(nsnull)
+   mInitialized(PR_FALSE)
 {
 }
 
@@ -81,14 +77,37 @@ NS_IMETHODIMP nsAbLDAPProcessReplicationData::Init(nsIAbLDAPReplicationQuery *qu
 
    mQuery = query;
 
-   nsresult rv = mQuery->GetReplicationServerInfo(&mDirServerInfo);
+   nsresult rv = mQuery->GetLDAPDirectory(getter_AddRefs(mDirectory));
    if(NS_FAILED(rv)) {
        mQuery = nsnull;
        return rv;   
    }
-   if(!mDirServerInfo) {
-       mQuery = nsnull;
-       return NS_ERROR_FAILURE;   
+
+   nsCOMPtr<nsIAbLDAPAttributeMapService> mapSvc = 
+       do_GetService("@mozilla.org/addressbook/ldap-attribute-map-service;1",
+                     &rv);
+   if (NS_FAILED(rv)) {
+     mQuery = nsnull;
+     return rv;
+   }
+
+   nsCOMPtr<nsIAbDirectory> abDirectory(do_QueryInterface(mDirectory, &rv));
+   if (NS_FAILED(rv)) {
+     mQuery = nsnull;
+     return rv;
+   }
+
+   nsXPIDLCString prefBaseName;
+   rv = abDirectory->GetDirPrefId(prefBaseName);
+   if (NS_FAILED(rv)) {
+     mQuery = nsnull;
+     return rv;
+   }
+
+   rv = mapSvc->GetMapForPrefBranch(prefBaseName, getter_AddRefs(mAttrMap));
+   if (NS_FAILED(rv)) {
+     mQuery = nsnull;
+     return rv;
    }
 
    mListener = progressListener;
@@ -125,10 +144,10 @@ NS_IMETHODIMP nsAbLDAPProcessReplicationData::OnLDAPInit(nsILDAPConnection *aCon
     }
 
     nsCOMPtr<nsILDAPMessageListener> listener;
-    nsresult rv = NS_GetProxyForObject(NS_CURRENT_EVENTQ,
+    nsresult rv = NS_GetProxyForObject(NS_PROXY_TO_CURRENT_THREAD,
                   NS_GET_IID(nsILDAPMessageListener), 
                   NS_STATIC_CAST(nsILDAPMessageListener*, this),
-                  PROXY_SYNC | PROXY_ALWAYS, 
+                  NS_PROXY_SYNC | NS_PROXY_ALWAYS, 
                   getter_AddRefs(listener));
     if(NS_FAILED(rv)) {
         Done(PR_FALSE);
@@ -215,7 +234,7 @@ NS_IMETHODIMP nsAbLDAPProcessReplicationData::Abort()
     nsresult rv = mQuery->GetOperation(getter_AddRefs(operation));
     if(operation && mState != kIdle)
     {
-        rv = operation->Abandon();
+        rv = operation->AbandonExt();
         if(NS_SUCCEEDED(rv))
             mState = kIdle;
     }
@@ -227,10 +246,12 @@ NS_IMETHODIMP nsAbLDAPProcessReplicationData::Abort()
         // delete the unsaved replication file
         if(mReplicationFile) {
             rv = mReplicationFile->Remove(PR_FALSE);
-            if(NS_SUCCEEDED(rv)) {
+            if(NS_SUCCEEDED(rv) && mDirectory) {
+                nsCAutoString fileName;
+                rv = mDirectory->GetReplicationFileName(fileName);
                 // now put back the backed up replicated file if aborted
-                if(mBackupReplicationFile && mDirServerInfo->replInfo) 
-                    rv = mBackupReplicationFile->MoveToNative(nsnull, nsDependentCString(mDirServerInfo->replInfo->fileName));
+                if(NS_SUCCEEDED(rv) && mBackupReplicationFile) 
+                    rv = mBackupReplicationFile->MoveToNative(nsnull, fileName);
             }
         }
     }
@@ -244,9 +265,15 @@ NS_IMETHODIMP nsAbLDAPProcessReplicationData::Abort()
 // this should get the authDN from prefs and password from PswdMgr
 NS_IMETHODIMP nsAbLDAPProcessReplicationData::PopulateAuthData()
 {
-    mAuthDN.Assign(mDirServerInfo->authDn);
+  if (!mDirectory)
+    return NS_ERROR_NOT_INITIALIZED;
+
+  nsCAutoString authDn;
+  nsresult rv = mDirectory->GetAuthDn(authDn);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  mAuthDN.Assign(authDn);
 
-    nsresult rv = NS_OK;
     nsCOMPtr <nsIPasswordManagerInternal> passwordMgrInt = do_GetService(NS_PASSWORDMANAGER_CONTRACTID, &rv);
     if(NS_SUCCEEDED(rv) && passwordMgrInt) {
         // Get the current server URI
@@ -326,20 +353,28 @@ nsresult nsAbLDAPProcessReplicationData::OnLDAPSearchEntry(nsILDAPMessage *aMess
     if(!mReplicationDB || !mDBOpen) 
         return NS_ERROR_FAILURE;
 
-    nsAbLDAPCard card;
-    PRBool hasSetCardProperty = PR_FALSE;
+  nsresult rv = NS_OK;
 
-    nsresult rv = MozillaLdapPropertyRelator::createCardPropertyFromLDAPMessage(aMessage,
-                                                                     &card, &hasSetCardProperty);
-    if(NS_FAILED(rv)) {
-        Abort();
-        return rv;
-    }
+  // Although we would may naturally create an nsIAbLDAPCard here, we don't
+  // need to as we are writing this straight to the database, so just create
+  // the database version instead.
+  nsCOMPtr<nsIAbMDBCard> dbCard(do_CreateInstance(NS_ABMDBCARD_CONTRACTID, &rv));
+  if(NS_FAILED(rv)) {
+    Abort();
+    return rv;
+  }
 
-    if(hasSetCardProperty == PR_FALSE)
+  nsCOMPtr<nsIAbCard> newCard(do_QueryInterface(dbCard, &rv));
+  if(NS_FAILED(rv)) {
+    Abort();
+    return rv;
+  }
+
+    rv = mAttrMap->SetCardPropertiesFromLDAPMessage(aMessage, newCard);
+    if (NS_FAILED(rv))
     {
         NS_WARNING("nsAbLDAPProcessReplicationData::OnLDAPSearchEntry"
-           "No card Properties found and set");
+           "No card properties could be set");
         // if some entries are bogus for us, continue with next one
         return NS_OK;
     }
@@ -358,26 +393,6 @@ nsresult nsAbLDAPProcessReplicationData::OnLDAPSearchEntry(nsILDAPMessage *aMess
         printf("\n LDAPReplication :: got card #: %d, name: %s \n", mCount, name.get());
 #endif
 
-    nsCOMPtr<nsIAbMDBCard> dbCard;
-    dbCard = do_CreateInstance(NS_ABMDBCARD_CONTRACTID, &rv);
-    if(NS_FAILED(rv)) {
-        Abort();
-        return rv;
-    }
-
-    nsCOMPtr<nsIAbCard> newCard;
-    newCard = do_QueryInterface(dbCard, &rv);
-    if(NS_FAILED(rv)) {
-        Abort();
-        return rv;
-    }
-
-    rv = newCard->Copy(&card);
-    if(NS_FAILED(rv)) {
-        Abort();
-        return rv;
-    }
-
     rv = mReplicationDB->CreateNewCardAndAddToDB(newCard, PR_FALSE);
     if(NS_FAILED(rv)) {
         Abort();
@@ -390,7 +405,7 @@ nsresult nsAbLDAPProcessReplicationData::OnLDAPSearchEntry(nsILDAPMessage *aMess
     if(NS_SUCCEEDED(rv) && !authDN.IsEmpty())
     {
         dbCard->SetAbDatabase(mReplicationDB);
-        dbCard->SetStringAttribute("_DN", NS_ConvertUTF8toUCS2(authDN).get());
+        dbCard->SetStringAttribute("_DN", NS_ConvertUTF8toUTF16(authDN).get());
     }
 
     newCard = do_QueryInterface(dbCard, &rv);
@@ -465,10 +480,15 @@ nsresult nsAbLDAPProcessReplicationData::OnLDAPSearchResult(nsILDAPMessage *aMes
             NS_ASSERTION(NS_SUCCEEDED(rv), "Replication File Remove on Failure failed");
             if(NS_SUCCEEDED(rv)) {
                 // now put back the backed up replicated file
-                if(mBackupReplicationFile && mDirServerInfo->replInfo) 
+                if(mBackupReplicationFile && mDirectory) 
                 {
-                    rv = mBackupReplicationFile->MoveToNative(nsnull, nsDependentCString(mDirServerInfo->replInfo->fileName));
+                  nsCAutoString fileName;
+                  rv = mDirectory->GetReplicationFileName(fileName);
+                  if (NS_SUCCEEDED(rv) && !fileName.IsEmpty())
+                  {
+                    rv = mBackupReplicationFile->MoveToNative(nsnull, fileName);
                     NS_ASSERTION(NS_SUCCEEDED(rv), "Replication Backup File Move back on Failure failed");
+                  }
                 }
             }
         }
@@ -480,7 +500,7 @@ nsresult nsAbLDAPProcessReplicationData::OnLDAPSearchResult(nsILDAPMessage *aMes
 
 nsresult nsAbLDAPProcessReplicationData::OpenABForReplicatedDir(PRBool aCreate)
 {
-    if(!mInitialized) 
+    if (!mInitialized)
         return NS_ERROR_NOT_INITIALIZED;
 
     nsresult rv = NS_OK;
@@ -491,30 +511,31 @@ nsresult nsAbLDAPProcessReplicationData::OpenABForReplicatedDir(PRBool aCreate)
         return rv;
     }
 
-    if(!mDirServerInfo->replInfo->fileName) {
-        Done(PR_FALSE);
-        return NS_ERROR_FAILURE;
-    }
+  nsCAutoString fileName;
+  rv = mDirectory->GetReplicationFileName(fileName);
+  if (NS_FAILED(rv) || fileName.IsEmpty())
+  {
+     Done(PR_FALSE);
+     return NS_ERROR_FAILURE;
+  }
 
-    nsFileSpec* dbPath;
-    rv = abSession->GetUserProfileDirectory(&dbPath);
+    rv = abSession->GetUserProfileDirectory(getter_AddRefs(mReplicationFile));
     if(NS_FAILED(rv)) {
         Done(PR_FALSE);
         return rv;
     }
 
-    (*dbPath) += mDirServerInfo->replInfo->fileName;
+    rv = mReplicationFile->AppendNative(fileName);
+    if(NS_FAILED(rv)) {
+        Done(PR_FALSE);
+        return rv;
+    }
 
     // if the AB DB already exists backup existing one, 
     // in case if the user cancels or Abort put back the backed up file
-    if(dbPath->Exists()) {
-        // get nsIFile for nsFileSpec from abSession, why use a obsolete class if not required!
-        rv = NS_FileSpecToIFile(dbPath, getter_AddRefs(mReplicationFile));
-        if(NS_FAILED(rv))  {
-            delete dbPath;
-            Done(PR_FALSE);
-            return rv;
-        }
+    PRBool fileExists;
+    rv = mReplicationFile->Exists(&fileExists);
+    if(NS_SUCCEEDED(rv) && fileExists) {
         // create the backup file object same as the Replication file object.
         // we create a backup file here since we need to cleanup the existing file
         // for create and then commit so instead of deleting existing cards we just
@@ -524,40 +545,36 @@ nsresult nsAbLDAPProcessReplicationData::OpenABForReplicatedDir(PRBool aCreate)
         nsCOMPtr<nsIFile> clone;
         rv = mReplicationFile->Clone(getter_AddRefs(clone));
         if(NS_FAILED(rv))  {
-            delete dbPath;
             Done(PR_FALSE);
             return rv;
         }
         mBackupReplicationFile = do_QueryInterface(clone, &rv);
         if(NS_FAILED(rv))  {
-            delete dbPath;
             Done(PR_FALSE);
             return rv;
         }
         rv = mBackupReplicationFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0777);
         if(NS_FAILED(rv))  {
-            delete dbPath;
             Done(PR_FALSE);
             return rv;
         }
         nsAutoString backupFileLeafName;
         rv = mBackupReplicationFile->GetLeafName(backupFileLeafName);
         if(NS_FAILED(rv))  {
-            delete dbPath;
             Done(PR_FALSE);
             return rv;
         }
         // remove the newly created unique backup file so that move and copy succeeds.
         rv = mBackupReplicationFile->Remove(PR_FALSE);
         if(NS_FAILED(rv))  {
-            delete dbPath;
             Done(PR_FALSE);
             return rv;
         }
 
         if(aCreate) {
             // set backup file to existing replication file for move
-            mBackupReplicationFile->SetNativeLeafName(nsDependentCString(mDirServerInfo->replInfo->fileName));
+            mBackupReplicationFile->SetNativeLeafName(fileName);
+
             rv = mBackupReplicationFile->MoveTo(nsnull, backupFileLeafName);
             // set the backup file leaf name now
             if (NS_SUCCEEDED(rv))
@@ -565,7 +582,8 @@ nsresult nsAbLDAPProcessReplicationData::OpenABForReplicatedDir(PRBool aCreate)
         }
         else {
             // set backup file to existing replication file for copy
-            mBackupReplicationFile->SetNativeLeafName(nsDependentCString(mDirServerInfo->replInfo->fileName));
+            mBackupReplicationFile->SetNativeLeafName(fileName);
+
             // specify the parent here specifically, 
             // passing nsnull to copy to the same dir actually renames existing file
             // instead of making another copy of the existing file.
@@ -578,7 +596,6 @@ nsresult nsAbLDAPProcessReplicationData::OpenABForReplicatedDir(PRBool aCreate)
                 mBackupReplicationFile->SetLeafName(backupFileLeafName);
         }
         if(NS_FAILED(rv))  {
-            delete dbPath;
             Done(PR_FALSE);
             return rv;
         }
@@ -587,15 +604,13 @@ nsresult nsAbLDAPProcessReplicationData::OpenABForReplicatedDir(PRBool aCreate)
     nsCOMPtr<nsIAddrDatabase> addrDBFactory = 
              do_GetService(NS_ADDRDATABASE_CONTRACTID, &rv);
     if(NS_FAILED(rv)) {
-        delete dbPath;
         if (mBackupReplicationFile)
             mBackupReplicationFile->Remove(PR_FALSE);
         Done(PR_FALSE);
         return rv;
     }
     
-    rv = addrDBFactory->Open(dbPath, aCreate, getter_AddRefs(mReplicationDB), PR_TRUE);
-    delete dbPath;
+    rv = addrDBFactory->Open(mReplicationFile, aCreate, PR_TRUE, getter_AddRefs(mReplicationDB));
     if(NS_FAILED(rv)) {
         Done(PR_FALSE);
         if (mBackupReplicationFile)
@@ -627,7 +642,7 @@ void nsAbLDAPProcessReplicationData::Done(PRBool aSuccess)
 nsresult nsAbLDAPProcessReplicationData::DeleteCard(nsString & aDn)
 {
     nsCOMPtr<nsIAbCard> cardToDelete;
-    mReplicationDB->GetCardFromAttribute(nsnull, "_DN", NS_ConvertUCS2toUTF8(aDn).get(),
+    mReplicationDB->GetCardFromAttribute(nsnull, "_DN", NS_ConvertUTF16toUTF8(aDn).get(),
                                          PR_FALSE, getter_AddRefs(cardToDelete));
     return mReplicationDB->DeleteCard(cardToDelete, PR_FALSE);
 }
