@@ -56,7 +56,7 @@
 #include "nsAbQueryStringToExpression.h"
 #include "nsArrayEnumerator.h"
 #include "nsAbMDBCardProperty.h"
-
+#include "nsEnumeratorUtils.h"
 #include "mdb.h"
 #include "prprf.h"
 
@@ -64,10 +64,9 @@
 // fix this -1,0,1 crap, use an enum or #define
 
 nsAbMDBDirectory::nsAbMDBDirectory(void):
-     nsRDFResource(),
+     nsAbDirectoryRDFResource(),
      mInitialized(PR_FALSE),
      mIsMailingList(-1),
-     mIsQueryURI(PR_FALSE),
      mPerformingQuery(PR_FALSE)
 {
 }
@@ -79,7 +78,7 @@ nsAbMDBDirectory::~nsAbMDBDirectory(void)
   }
 }
 
-NS_IMPL_ISUPPORTS_INHERITED4(nsAbMDBDirectory, nsRDFResource,
+NS_IMPL_ISUPPORTS_INHERITED4(nsAbMDBDirectory, nsAbDirectoryRDFResource,
                              nsIAbDirectory,
                              nsIAbMDBDirectory,
                              nsIAbDirectorySearch,
@@ -92,6 +91,24 @@ nsresult nsAbMDBDirectory::RemoveCardFromAddressList(nsIAbCard* card)
   nsresult rv = NS_OK;
   PRUint32 listTotal;
   PRInt32 i, j;
+
+  // These checks ensure we don't run into null pointers
+  // as we did when we caused bug 280463.
+  if (!mDatabase)
+  {
+    rv = GetAbDatabase();
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+
+  if (!m_AddressList)
+  {
+    rv = mDatabase->GetMailingListsFromDB(this);
+    NS_ENSURE_SUCCESS(rv, rv);
+    // Ensure that the previous call did set the address list pointer
+    if (!m_AddressList)
+      return NS_ERROR_NULL_POINTER;
+  }
+
   rv = m_AddressList->Count(&listTotal);
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -100,6 +117,10 @@ nsresult nsAbMDBDirectory::RemoveCardFromAddressList(nsIAbCard* card)
     nsCOMPtr<nsIAbDirectory> listDir(do_QueryElementAt(m_AddressList, i, &rv));
     if (listDir)
     {
+      // First remove the instance in the database
+      mDatabase->DeleteCardFromMailList(listDir, card, PR_FALSE);
+
+      // Now remove the instance in any lists we hold.
       nsCOMPtr <nsISupportsArray> pAddressLists;
       listDir->GetAddressLists(getter_AddRefs(pAddressLists));
       if (pAddressLists)
@@ -208,53 +229,6 @@ nsresult nsAbMDBDirectory::NotifyItemDeleted(nsISupports *item)
   return NS_OK;
 }
 
-
-// nsIRDFResource methods
-
-NS_IMETHODIMP nsAbMDBDirectory::Init(const char* aURI)
-{
-  nsresult rv;
-
-  rv = nsRDFResource::Init(aURI);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  mURINoQuery = aURI;
-
-  nsCOMPtr<nsIURI> uri = do_CreateInstance (NS_STANDARDURL_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = uri->SetSpec(nsDependentCString(aURI));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  mIsValidURI = PR_TRUE;
-
-    nsCOMPtr<nsIURL> url = do_QueryInterface(uri);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCAutoString queryString;
-  rv = url->GetQuery (queryString);
-
-  nsCAutoString path;
-  rv = url->GetPath (path);
-  mPath = path;
-
-  if (!queryString.IsEmpty())
-  {
-    mPath.Truncate(path.Length() - queryString.Length() - 1);
-
-    mURINoQuery.Truncate(mURINoQuery.Length() - queryString.Length() - 1);
-
-    mQueryString = queryString;
-
-    mIsQueryURI = PR_TRUE;
-  }
-  else
-    mIsQueryURI = PR_FALSE;
-
-  return rv;
-}
-
-
 // nsIAbMDBDirectory methods
 
 NS_IMETHODIMP nsAbMDBDirectory::ClearDatabase()
@@ -343,6 +317,21 @@ NS_IMETHODIMP nsAbMDBDirectory::GetDirUri(char **uri)
 
 // nsIAbDirectory methods
 
+NS_IMETHODIMP nsAbMDBDirectory::GetURI(nsACString &aURI)
+{
+  nsresult rv = GetStringValue("uri", EmptyCString(), aURI);
+
+  if (aURI.IsEmpty())
+  {
+    rv = GetFileName(aURI);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    aURI.Insert(kMDBDirectoryRoot, 0);
+  }
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsAbMDBDirectory::GetChildNodes(nsISimpleEnumerator* *aResult)
 {
   if (mIsQueryURI)
@@ -364,7 +353,7 @@ PR_STATIC_CALLBACK(PRBool) enumerateSearchCache(nsHashKey *aKey, void *aData, vo
         return PR_TRUE;
 }
 
-NS_IMETHODIMP nsAbMDBDirectory::GetChildCards(nsIEnumerator* *result)
+NS_IMETHODIMP nsAbMDBDirectory::GetChildCards(nsISimpleEnumerator* *result)
 {
   if (mIsQueryURI)
   {
@@ -378,7 +367,7 @@ NS_IMETHODIMP nsAbMDBDirectory::GetChildCards(nsIEnumerator* *result)
     nsCOMPtr<nsISupportsArray> array;
     NS_NewISupportsArray(getter_AddRefs(array));
     mSearchCache.Enumerate(enumerateSearchCache, (void*)array);
-    return array->Enumerate(result);
+    return NS_NewArrayEnumerator(result, array);
   }
 
   NS_ASSERTION(!mURI.IsEmpty(), "Not Initialized?");
@@ -466,9 +455,10 @@ NS_IMETHODIMP nsAbMDBDirectory::DeleteCards(nsISupportsArray *cards)
         {
           mDatabase->DeleteCardFromMailList(this, card, PR_TRUE);
 
-          PRUint32 cardTotal;
+          PRUint32 cardTotal = 0;
           PRInt32 i;
-          rv = m_AddressList->Count(&cardTotal);
+          if (m_AddressList)
+            rv = m_AddressList->Count(&cardTotal);
           for (i = cardTotal - 1; i >= 0; i--)
           {            
             nsCOMPtr<nsIAbMDBCard> dbarrayCard(do_QueryElementAt(m_AddressList, i, &rv));
@@ -494,8 +484,10 @@ NS_IMETHODIMP nsAbMDBDirectory::DeleteCards(nsISupportsArray *cards)
             //to do, get mailing list dir side uri and notify rdf to remove it
             PRUint32 rowID;
             dbcard->GetDbRowID(&rowID);
-            char *listUri = PR_smprintf("%s/MailList%ld", mURI.get(), rowID);
-            if (listUri)
+            nsCAutoString listUri(mURI);
+            listUri.AppendLiteral("/MailList");
+            listUri.AppendInt(rowID);
+            if (!listUri.IsEmpty())
             {
               nsresult rv = NS_OK;
               nsCOMPtr<nsIRDFService> rdfService = 
@@ -504,7 +496,7 @@ NS_IMETHODIMP nsAbMDBDirectory::DeleteCards(nsISupportsArray *cards)
               if(NS_SUCCEEDED(rv))
                 {
                 nsCOMPtr<nsIRDFResource> listResource;
-                rv = rdfService->GetResource(nsDependentCString(listUri),
+                rv = rdfService->GetResource(listUri,
                                              getter_AddRefs(listResource));
                 nsCOMPtr<nsIAbDirectory> listDir = do_QueryInterface(listResource, &rv);
                 if(NS_SUCCEEDED(rv))
@@ -515,17 +507,14 @@ NS_IMETHODIMP nsAbMDBDirectory::DeleteCards(nsISupportsArray *cards)
 
                   if (listDir)
                     NotifyItemDeleted(listDir);
-                  PR_smprintf_free(listUri);
                   }
                 else 
                   {
-                  PR_smprintf_free(listUri);
                   return rv;
                   }
                 }
               else
                 {
-                PR_smprintf_free(listUri);
                 return rv;
                 }
             }
@@ -611,17 +600,7 @@ NS_IMETHODIMP nsAbMDBDirectory::CreateDirectoryByURI(const PRUnichar *dirName, c
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP nsAbMDBDirectory::AddMailListWithKey(nsIAbDirectory *list, PRUint32 *key)
-{
-  return(InternalAddMailList(list, key));
-}
-
 NS_IMETHODIMP nsAbMDBDirectory::AddMailList(nsIAbDirectory *list)
-{
-  return(InternalAddMailList(list, nsnull));
-}
-
-nsresult nsAbMDBDirectory::InternalAddMailList(nsIAbDirectory *list, PRUint32 *key)
 {
   if (mIsQueryURI)
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -636,28 +615,25 @@ nsresult nsAbMDBDirectory::InternalAddMailList(nsIAbDirectory *list, PRUint32 *k
   nsCOMPtr<nsIAbMDBDirectory> dblist(do_QueryInterface(list, &rv));
   if (NS_FAILED(rv))
   {
-    // XXX fix this.
-    nsAbMDBDirProperty* dblistproperty = new nsAbMDBDirProperty ();
-    if (!dblistproperty)
+    nsCOMPtr<nsIAbDirectory> newlist(new nsAbMDBDirProperty);
+    if (!newlist)
       return NS_ERROR_OUT_OF_MEMORY;
-    NS_ADDREF(dblistproperty);
-    nsCOMPtr<nsIAbDirectory> newlist = getter_AddRefs(NS_STATIC_CAST(nsIAbDirectory*, dblistproperty));
-    newlist->CopyMailList(list);
-    list = newlist;
-    dblist = do_QueryInterface(list, &rv);
+
+    rv = newlist->CopyMailList(list);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    dblist = do_QueryInterface(newlist, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  if (!key)
-    mDatabase->CreateMailListAndAddToDB(list, PR_TRUE);
-  else
-    mDatabase->CreateMailListAndAddToDBWithKey(list, PR_TRUE, key);
+  mDatabase->CreateMailListAndAddToDB(list, PR_TRUE);
   mDatabase->Commit(nsAddrDBCommitType::kLargeCommit);
 
   PRUint32 dbRowID;
   dblist->GetDbRowID(&dbRowID);
 
-  nsCAutoString listUri;
-  listUri = mURI + NS_LITERAL_CSTRING("/MailList");
+  nsCAutoString listUri(mURI);
+  listUri.AppendLiteral("/MailList");
   listUri.AppendInt(dbRowID);
 
   nsCOMPtr<nsIAbDirectory> newList;
@@ -729,12 +705,6 @@ NS_IMETHODIMP nsAbMDBDirectory::DropCard(nsIAbCard* aCard, PRBool needToCopyCard
 
   nsresult rv = NS_OK;
 
-  // Don't add the card if it's not a normal one (ie, they can't be added as members).
-  PRBool isNormal;
-  rv = aCard->GetIsANormalCard(&isNormal);
-  if (!isNormal)
-    return NS_OK;
-
   NS_ASSERTION(!mURI.IsEmpty(), "Not initialized?");
   if (mIsMailingList == -1)
   {
@@ -744,10 +714,7 @@ NS_IMETHODIMP nsAbMDBDirectory::DropCard(nsIAbCard* aCard, PRBool needToCopyCard
      * moz-abmdbdirectory://foo/bar
      */
     NS_ENSURE_TRUE(mURI.Length() > kMDBDirectoryRootLen, NS_ERROR_UNEXPECTED);
-    if (strchr(mURI.get() + kMDBDirectoryRootLen, '/'))
-      mIsMailingList = 1;
-    else
-      mIsMailingList = 0;
+    mIsMailingList = (strchr(mURI.get() + kMDBDirectoryRootLen, '/')) ? 1 : 0;
   }
   if (!mDatabase)
     rv = GetAbDatabase();
@@ -778,8 +745,14 @@ NS_IMETHODIMP nsAbMDBDirectory::DropCard(nsIAbCard* aCard, PRBool needToCopyCard
 
   if (mIsMailingList == 1) {
     if (needToCopyCard) {
-      // first, add the card to the directory that contains the mailing list.
-      mDatabase->CreateNewCardAndAddToDB(newCard, PR_TRUE /* notify */);
+      nsCOMPtr <nsIMdbRow> cardRow;
+      // if card doesn't exist in db, add the card to the directory that 
+      // contains the mailing list.
+      mDatabase->FindRowByCard(newCard, getter_AddRefs(cardRow));
+      if (!cardRow)
+        mDatabase->CreateNewCardAndAddToDB(newCard, PR_TRUE /* notify */);
+      else
+        mDatabase->InitCardFromRow(newCard, cardRow);
     }
     // since we didn't copy the card, we don't have to notify that it was inserted
     mDatabase->CreateNewListCardAndAddToDB(this, m_dbRowID, newCard, PR_FALSE /* notify */);
@@ -819,13 +792,13 @@ NS_IMETHODIMP nsAbMDBDirectory::EditMailListToDatabase(const char *uri, nsIAbCar
 
 // nsIAddrDBListener methods
 
-NS_IMETHODIMP nsAbMDBDirectory::OnCardAttribChange(PRUint32 abCode, nsIAddrDBListener *instigator)
+NS_IMETHODIMP nsAbMDBDirectory::OnCardAttribChange(PRUint32 abCode)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP nsAbMDBDirectory::OnCardEntryChange
-(PRUint32 abCode, nsIAbCard *card, nsIAddrDBListener *instigator)
+(PRUint32 abCode, nsIAbCard *card)
 {
   NS_ENSURE_ARG_POINTER(card);
   nsCOMPtr<nsISupports> cardSupports(do_QueryInterface(card));
@@ -851,7 +824,7 @@ NS_IMETHODIMP nsAbMDBDirectory::OnCardEntryChange
 }
 
 NS_IMETHODIMP nsAbMDBDirectory::OnListEntryChange
-(PRUint32 abCode, nsIAbDirectory *list, nsIAddrDBListener *instigator)
+(PRUint32 abCode, nsIAbDirectory *list)
 {
   nsresult rv = NS_OK;
   
@@ -876,7 +849,7 @@ NS_IMETHODIMP nsAbMDBDirectory::OnListEntryChange
   return rv;
 }
 
-NS_IMETHODIMP nsAbMDBDirectory::OnAnnouncerGoingAway(nsIAddrDBAnnouncer *instigator)
+NS_IMETHODIMP nsAbMDBDirectory::OnAnnouncerGoingAway()
 {
   if (mDatabase)
       mDatabase->RemoveListener(this);
@@ -980,22 +953,17 @@ nsresult nsAbMDBDirectory::GetAbDatabase()
 {
   NS_ASSERTION(!mURI.IsEmpty(), "Not initialized?");
 
-  if (!mDatabase) {
-    nsresult rv;
+  nsresult rv = NS_OK;
 
+  if (!mDatabase) {
     nsCOMPtr<nsIAddressBook> addressBook = do_GetService(NS_ADDRESSBOOK_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
 
     rv = addressBook->GetAbDatabaseFromURI(mURI.get(), getter_AddRefs(mDatabase));
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = mDatabase->AddListener(this);
-    NS_ENSURE_SUCCESS(rv, rv);
+    if (NS_SUCCEEDED(rv))
+      rv = mDatabase->AddListener(this);
   }
-
-  if (!mDatabase)
-    return NS_ERROR_NULL_POINTER;
-  return NS_OK;
+  return rv;
 }
 
 NS_IMETHODIMP nsAbMDBDirectory::HasCardForEmailAddress(const char * aEmailAddress, PRBool * aCardExists)
@@ -1005,6 +973,11 @@ NS_IMETHODIMP nsAbMDBDirectory::HasCardForEmailAddress(const char * aEmailAddres
 
   if (!mDatabase)
     rv = GetAbDatabase();
+  if (rv == NS_ERROR_FILE_NOT_FOUND)
+  {
+    // If file wasn't found, the card cannot exist.
+    return NS_OK;
+  }
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIAbCard> card; 
