@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Seth Spitzer <sspitzer@netscape.com>
+ *   Mark Banner <mark@standard8.demon.co.uk>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -43,30 +44,15 @@
 #include "mdb.h"
 #include "nsVoidArray.h"
 #include "nsString.h"
-#include "nsFileSpec.h"
 #include "nsIAddrDBListener.h"
 #include "nsISupportsArray.h"
 #include "nsCOMPtr.h"
 
-#define  CARD_ATTRIB_PALMID "PalmRecId"
-#define  CARD_ATTRIB_DISPLAY "DisplayName"
-
 typedef enum 
 {
 	AB_NotifyInserted,
 	AB_NotifyDeleted,
-	AB_NotifyPropertyChanged,
-
-	AB_NotifyAll,		   /* contents of the have totally changed. Listener must totally
-							  forget anything they knew about the object. */
-	/* pane notifications (i.e. not tied to a particular entry */
-	AB_NotifyScramble,     /* same contents, but the view indices have all changed 
-						      i.e the object was sorted on a different attribute */
-	AB_NotifyLDAPTotalContentChanged,
-	AB_NotifyNewTopIndex,
-	AB_NotifyStartSearching,
-	AB_NotifyStopSearching
-
+	AB_NotifyPropertyChanged
 } AB_NOTIFY_CODE;
 
 class nsAddrDatabase : public nsIAddrDatabase 
@@ -77,24 +63,21 @@ public:
 	//////////////////////////////////////////////////////////////////////////////
 	// nsIAddrDatabase methods:
 
-	NS_IMETHOD GetDbPath(nsFileSpec * *aDbPath);
-	NS_IMETHOD SetDbPath(nsFileSpec * aDbPath);
-	NS_IMETHOD Open(nsFileSpec *aMabFile, PRBool aCreate, nsIAddrDatabase **pCardDB, PRBool upgrading);
-        NS_IMETHOD OpenWithIFile(nsIFile *dbFile, PRBool create, PRBool upgrading, nsIAddrDatabase **_retval);
+  NS_IMETHOD GetDbPath(nsIFile * *aDbPath);
+  NS_IMETHOD SetDbPath(nsIFile * aDbPath);
+	NS_IMETHOD Open(nsIFile *aMabFile, PRBool aCreate, PRBool upgrading, nsIAddrDatabase **pCardDB);
 	NS_IMETHOD Close(PRBool forceCommit);
-	NS_IMETHOD OpenMDB(nsFileSpec *dbName, PRBool create);
+  NS_IMETHOD OpenMDB(nsIFile *dbName, PRBool create);
 	NS_IMETHOD CloseMDB(PRBool commit);
 	NS_IMETHOD Commit(PRUint32 commitType);
 	NS_IMETHOD ForceClosed();
 
 	NS_IMETHOD CreateNewCardAndAddToDB(nsIAbCard *newCard, PRBool notify);
-	NS_IMETHOD CreateNewCardAndAddToDBWithKey(nsIAbCard *newCard, PRBool notify, PRUint32 *key);	
 	NS_IMETHOD CreateNewListCardAndAddToDB(nsIAbDirectory *list, PRUint32 listRowID, nsIAbCard *newCard, PRBool notify);
 	NS_IMETHOD CreateMailListAndAddToDB(nsIAbDirectory *newList, PRBool notify);
-  NS_IMETHOD CreateMailListAndAddToDBWithKey(nsIAbDirectory *newList, PRBool notify, PRUint32 *key);
-	NS_IMETHOD EnumerateCards(nsIAbDirectory *directory, nsIEnumerator **result);
+	NS_IMETHOD EnumerateCards(nsIAbDirectory *directory, nsISimpleEnumerator **result);
 	NS_IMETHOD GetMailingListsFromDB(nsIAbDirectory *parentDir);
-	NS_IMETHOD EnumerateListAddresses(nsIAbDirectory *directory, nsIEnumerator **result);
+	NS_IMETHOD EnumerateListAddresses(nsIAbDirectory *directory, nsISimpleEnumerator **result);
 	NS_IMETHOD DeleteCard(nsIAbCard *newCard, PRBool notify);
 	NS_IMETHOD EditCard(nsIAbCard *card, PRBool notify);
 	NS_IMETHOD ContainsCard(nsIAbCard *card, PRBool *hasCard);
@@ -135,15 +118,12 @@ public:
 	NS_IMETHOD Add2ndEmail(nsIMdbRow * row, const char * value)
 	{ return AddCharStringColumn(row, m_2ndEmailColumnToken, value); }
 
-  NS_IMETHOD AddDefaultEmail(nsIMdbRow * row, const char * value)
-  { return AddCharStringColumn(row, m_DefaultEmailColumnToken, value); }
-
-  NS_IMETHOD AddCardType(nsIMdbRow * row, const char * value)
-  { return AddCharStringColumn(row, m_CardTypeColumnToken, value); }
-
 	NS_IMETHOD AddPreferMailFormat(nsIMdbRow * row, PRUint32 value)
 	{ return AddIntColumn(row, m_MailFormatColumnToken, value); }
 
+	NS_IMETHOD AddPopularityIndex(nsIMdbRow * row, PRUint32 value)
+	{ return AddIntColumn(row, m_PopularityIndexColumnToken, value); }
+
 	NS_IMETHOD AddWorkPhone(nsIMdbRow * row, const char * value)
 	{ return AddCharStringColumn(row, m_WorkPhoneColumnToken, value); }
 
@@ -291,8 +271,6 @@ public:
 
   NS_IMETHOD SetCardValue(nsIAbCard *card, const char *name, const PRUnichar *value, PRBool notify);
   NS_IMETHOD GetCardValue(nsIAbCard *card, const char *name, PRUnichar **value);
-  NS_IMETHOD AddRowValue(nsIMdbRow *aRow, const nsACString & aLDIFAttributeName, const nsAString & aColValue);
-
 	// nsAddrDatabase methods:
 
 	nsAddrDatabase();
@@ -300,12 +278,11 @@ public:
 
 	nsIMdbFactory	*GetMDBFactory();
 	nsIMdbEnv		*GetEnv() {return m_mdbEnv;}
-	nsIMdbStore		*GetStore() {return m_mdbStore;}
 	PRUint32		GetCurVersion();
 	nsIMdbTableRowCursor *GetTableRowCursor();
 	nsIMdbTable		*GetPabTable() {return m_mdbPabTable;}
 
-	static nsAddrDatabase*	FindInCache(nsFileSpec *dbName);
+  static nsAddrDatabase* FindInCache(nsIFile *dbName);
 
 	static void		CleanupCache();
 
@@ -322,9 +299,6 @@ public:
 	PRUint32 GetListAddressTotal(nsIMdbRow* listRow);
 	nsresult GetAddressRowByPos(nsIMdbRow* listRow, PRUint16 pos, nsIMdbRow** cardRow);
 
-  static void PRTime2Seconds(PRTime prTime, PRUint32 *seconds);
-  
-
     NS_IMETHOD AddListCardColumnsToRow(nsIAbCard *aPCard, nsIMdbRow *aPListRow, PRUint32 aPos, nsIAbCard** aPNewCard, PRBool aInMailingList);
     NS_IMETHOD InitCardFromRow(nsIAbCard *aNewCard, nsIMdbRow* aCardRow);
     NS_IMETHOD SetListAddressTotal(nsIMdbRow* aListRow, PRUint32 aTotal);
@@ -335,12 +309,7 @@ protected:
   static void		AddToCache(nsAddrDatabase* pAddrDB) {GetDBCache()->AppendElement(pAddrDB);}
 	static void		RemoveFromCache(nsAddrDatabase* pAddrDB);
 	static PRInt32	FindInCache(nsAddrDatabase* pAddrDB);
-	PRBool			MatchDbName(nsFileSpec *dbName);	// returns TRUE if they match
-
-#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_MAC)	// this should go away when we can provide our own file stream to MDB/Mork
-	static void		UnixToNative(char*& ioPath);
-#endif
-
+  PRBool MatchDbName(nsIFile *dbName); // returns TRUE if they match
 
 	void YarnToUInt32(struct mdbYarn *yarn, PRUint32 *pResult);
 	void GetCharStringYarn(char* str, struct mdbYarn* strYarn);
@@ -363,7 +332,7 @@ protected:
 	nsresult CreateCardFromDeletedCardsTable(nsIMdbRow* cardRow, mdb_id listRowID, nsIAbCard **result);
 	nsresult DeleteCardFromListRow(nsIMdbRow* pListRow, mdb_id cardRowID);
 	void DeleteCardFromAllMailLists(mdb_id cardRowID);
-	nsresult NotifyListEntryChange(PRUint32 abCode, nsIAbDirectory *dir, nsIAddrDBListener *instigator);
+	nsresult NotifyListEntryChange(PRUint32 abCode, nsIAbDirectory *dir);
 
 	nsresult AddLowercaseColumn(nsIMdbRow * row, mdb_token columnToken, const char* utf8String);
   nsresult GetRowFromAttribute(const char *aName, const char *aUTF8Value, PRBool aCaseInsensitive, nsIMdbRow	**aCardRow);
@@ -392,11 +361,11 @@ protected:
 
 	nsresult			DeleteRow(nsIMdbTable* dbTable, nsIMdbRow* dbRow);
 
-	nsIMdbEnv		    *m_mdbEnv;	// to be used in all the db calls.
-	nsIMdbStore	 	    *m_mdbStore;
-	nsIMdbTable		    *m_mdbPabTable;
-	nsIMdbTable                 *m_mdbDeletedCardsTable;
-	nsFileSpec		    m_dbName;
+  nsIMdbEnv   *m_mdbEnv;	// to be used in all the db calls.
+  nsIMdbStore *m_mdbStore;
+  nsIMdbTable *m_mdbPabTable;
+  nsIMdbTable *m_mdbDeletedCardsTable;
+  nsCOMPtr<nsIFile> m_dbName;
 	PRBool				m_mdbTokensInitialized;
     nsVoidArray /*<nsIAddrDBListener>*/ *m_ChangeListeners;
 
@@ -466,6 +435,7 @@ protected:
 	mdb_token			m_LowerPriEmailColumnToken;
 
 	mdb_token			m_MailFormatColumnToken;
+	mdb_token     m_PopularityIndexColumnToken;
 						
 	mdb_token			m_AddressCharSetColumnToken;
 	mdb_token			m_LastRecordKeyColumnToken;
@@ -482,8 +452,11 @@ protected:
 private:
   nsresult GetRowForCharColumn(const PRUnichar *unicodeStr, mdb_column findColumn, PRBool bIsCard, nsIMdbRow **findRow);
   PRBool HasRowButDeletedForCharColumn(const PRUnichar *unicodeStr, mdb_column findColumn, PRBool aIsCard, nsIMdbRow **aFindRow);
-  nsresult OpenInternal(nsFileSpec *aMabFile, PRBool aCreate, nsIAddrDatabase **pCardDB);
+  nsresult OpenInternal(nsIFile *aMabFile, PRBool aCreate, nsIAddrDatabase **pCardDB);
   nsresult AlertAboutCorruptMabFile(const PRUnichar *aOldFileName, const PRUnichar *aNewFileName);
+  nsresult AlertAboutLockedMabFile(const PRUnichar *aFileName);
+  nsresult DisplayAlert(const PRUnichar *titleName, const PRUnichar *alertStringName, 
+                        const PRUnichar **formatStrings, PRInt32 numFormatStrings);
 };
 
 #endif
