@@ -176,6 +176,8 @@ function ChangeFolderByURI(uri, viewType, viewFlags, sortType, sortOrder)
   if (uri == gCurrentLoadingFolderURI)
     return;
 
+  SetUpToolbarButtons(uri);
+
   // hook for extra toolbar items
   var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
   observerService.notifyObservers(window, "mail:setupToolbarItems", uri);
@@ -211,16 +213,7 @@ function ChangeFolderByURI(uri, viewType, viewFlags, sortType, sortOrder)
   }
 
   // If the user clicks on msgfolder, time to display thread pane and message pane.
-  // Hide AccountCentral page
-  if (gAccountCentralLoaded)
-  {
-      HideAccountCentral();
-  }
-
-  if (gFakeAccountPageLoaded)
-  {
-      HideFakeAccountPage();
-  }
+  ShowThreadPane();
 
   gCurrentLoadingFolderURI = uri;
   gNextMessageAfterDelete = null; // forget what message to select, if any
@@ -363,9 +356,9 @@ function RerootFolder(uri, newFolder, viewType, viewFlags, sortType, sortOrder)
   // that should have initialized gDBView, now re-root the thread pane
   RerootThreadPane();
 
-  SetUpToolbarButtons(uri);
+  UpdateLocationBar(gMsgFolderSelected);
 
-  UpdateStatusMessageCounts(newFolder);
+  UpdateStatusMessageCounts(gMsgFolderSelected);
   
   UpdateMailToolbar("reroot folder in 3 pane");
   // hook for extra toolbar items
@@ -510,7 +503,11 @@ function UpdateStatusMessageCounts(folder)
   var totalElement = GetTotalCountElement();
   if(folder && unreadElement && totalElement)
   {
-    var numUnread =
+    var numSelected = GetNumSelectedMessages();
+
+    var numUnread = (numSelected > 1) ?
+            gMessengerBundle.getFormattedString("selectedMsgStatus",
+                                                [numSelected]) :
             gMessengerBundle.getFormattedString("unreadMsgStatus",
                                                 [ folder.getNumUnread(false)]);
     var numTotal =
@@ -567,8 +564,8 @@ function ConvertColumnIDToSortType(columnID)
     case "threadCol":
       sortKey = nsMsgViewSortType.byThread;
       break;
-    case "labelCol":
-      sortKey = nsMsgViewSortType.byLabel;
+    case "tagsCol":
+      sortKey = nsMsgViewSortType.byTags;
       break;
     case "junkStatusCol":
       sortKey = nsMsgViewSortType.byJunkStatus;
@@ -576,6 +573,9 @@ function ConvertColumnIDToSortType(columnID)
     case "idCol":
       sortKey = nsMsgViewSortType.byId;
       break;
+    case "attachmentCol":
+      sortKey = nsMsgViewSortType.byAttachments;
+      break;
     default:
       dump("unsupported sort column: " + columnID + "\n");
       sortKey = 0;
@@ -617,8 +617,8 @@ function ConvertSortTypeToColumnID(sortKey)
     case nsMsgViewSortType.byStatus:
       columnID = "statusCol";
       break;
-    case nsMsgViewSortType.byLabel:
-      columnID = "labelCol";
+    case nsMsgViewSortType.byTags:
+      columnID = "tagsCol";
       break;
     case nsMsgViewSortType.bySize:
       columnID = "sizeCol";
@@ -638,6 +638,9 @@ function ConvertSortTypeToColumnID(sortKey)
     case nsMsgViewSortType.byJunkStatus:
       columnID = "junkStatusCol";
       break;
+    case nsMsgViewSortType.byAttachments:
+      columnID = "attachmentCol";
+      break;
     default:
       dump("unsupported sort key: " + sortKey + "\n");
       columnID = null;
@@ -671,6 +674,9 @@ function CreateBareDBView(originalView, msgFolder, viewType, viewFlags, sortType
       case nsMsgViewType.eShowQuickSearchResults:
           dbviewContractId += "quicksearch";
           break;
+      case nsMsgViewType.eShowSearch:
+          dbviewContractId += "search";
+          break;
       case nsMsgViewType.eShowThreadsWithUnread:
           dbviewContractId += "threadswithunread";
           break;
@@ -758,14 +764,19 @@ function GetSelectedFolderResource()
 function ChangeMessagePaneVisibility(now_hidden)
 {
   // we also have to hide the File/Attachments menuitem
-  node = document.getElementById("fileAttachmentMenu");
+  var node = document.getElementById("fileAttachmentMenu");
   if (node)
     node.hidden = now_hidden;
 
   if (gDBView) {
+    // clear the subject, collapsing won't automatically do this
+    setTitleFromFolder(GetThreadPaneFolder(), null);
     // the collapsed state is the state after we released the mouse 
     // so we take it as it is
     gDBView.suppressMsgDisplay = now_hidden;
+    // set the subject, uncollapsing won't automatically do this
+    gDBView.loadMessageByUrl("about:blank");
+    gDBView.selectionChanged();
   }
   var event = document.createEvent('Events');
   if (now_hidden) {
@@ -777,20 +788,11 @@ function ChangeMessagePaneVisibility(now_hidden)
   document.getElementById("messengerWindow").dispatchEvent(event);
 }
 
-function OnMouseUpThreadAndMessagePaneSplitter()
+function OnClickThreadAndMessagePaneSplitter()
 {
-  // the collapsed state is the state after we released the mouse 
-  // so we take it as it is
   ChangeMessagePaneVisibility(IsMessagePaneCollapsed());
 }
 
-function OnClickThreadAndMessagePaneSplitterGrippy()
-{
-  // the collapsed state is the state when we clicked on the grippy
-  // not when afterwards, so we need to reverse this value
-  ChangeMessagePaneVisibility(!IsMessagePaneCollapsed());
-}
-
 function FolderPaneSelectionChange()
 {
     var folderTree = GetFolderTree();
@@ -827,6 +829,7 @@ function FolderPaneSelectionChange()
            return;
         gPrevSelectedFolder = gMsgFolderSelected;
         gMsgFolderSelected = msgFolder;
+        UpdateLocationBar(gMsgFolderSelected);
         var folderFlags = msgFolder.flags;
         // if this is same folder, and we're not showing a virtual folder
         // then do nothing.
@@ -838,6 +841,7 @@ function FolderPaneSelectionChange()
         }
         else
         {
+            OnLeavingFolder(gPrevSelectedFolder);  // mark all read in last folder
             var sortType = 0;
             var sortOrder = 0;
             var viewFlags = 0;
@@ -856,9 +860,12 @@ function FolderPaneSelectionChange()
                 var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
                 if (msgDatabase)
                 {
+                  gSearchSession = null;
                   var dbFolderInfo = msgDatabase.dBFolderInfo;
                   sortType = dbFolderInfo.sortType;
                   sortOrder = dbFolderInfo.sortOrder;
+                  viewType = dbFolderInfo.viewType;
+                  viewFlags = dbFolderInfo.viewFlags;
                   if (folderFlags & MSG_FOLDER_FLAG_VIRTUAL)
                   {
                     viewType = nsMsgViewType.eShowQuickSearchResults;
@@ -883,7 +890,6 @@ function FolderPaneSelectionChange()
                     }
                     else
                     {
-                      gSearchSession = null;
                       uriToLoad = srchFolderUri;
                       // we need to load the db for the actual folder so that many hdrs to download
                       // will return false...
@@ -896,12 +902,6 @@ function FolderPaneSelectionChange()
 //                      gSearchInput.showingSearchCriteria = false;
                     }
                   }
-                  else
-                  {
-                    gSearchSession = null;
-                    viewFlags = dbFolderInfo.viewFlags;
-                    viewType = dbFolderInfo.viewType;
-                  }
                   msgDatabase = null;
                   dbFolderInfo = null;
                 }
@@ -1008,15 +1008,6 @@ function GetSelectTrashUri(folder)
     return null;
 }
 
-function Undo()
-{
-    messenger.Undo(msgWindow);
-}
-
-function Redo()
-{
-    messenger.Redo(msgWindow);
-}
 var mailOfflineObserver = {
   observe: function(subject, topic, state) {
     // sanity checks
@@ -1037,21 +1028,6 @@ function RemoveMailOfflineObserver()
   observerService.removeObserver(mailOfflineObserver,"network:offline-status-changed");
 }
 
-function getViewName(okCallbackFunc, defaultViewName) 
-{
-  var preselectedURI = GetSelectedFolderURI();
-  var folderTree = GetFolderTree();
-
-  var name = GetFolderNameFromUri(preselectedURI, folderTree);
-  name += defaultViewName + "-view";
-  var dialog = window.openDialog(
-                          "chrome://messenger/content/virtualFolderName.xul",
-                          "newFolder",
-                          "chrome,titlebar,modal",
-                          {siblingFolderURI: preselectedURI, searchFolderURIs: preselectedURI, 
-                          okCallback: okCallbackFunc, name: name});
-}
-
 function getSearchTermString(searchTerms)
 {
   var searchIndex;
@@ -1064,6 +1040,11 @@ function getSearchTermString(searchTerms)
     if (condition.length > 1)
       condition += ' ';
     
+    if (term.matchAll)
+    {
+        condition = "ALL";
+        break;
+    }
     condition += (term.booleanAnd) ? "AND (" : "OR (";
     condition += term.termAsString + ')';
   }
@@ -1172,6 +1153,7 @@ function CreateGroupedSearchTerms(searchTermsArray)
     var searchTermForQS = searchSession.createTerm();
     searchTermForQS.value = searchTerm.value;
     searchTermForQS.attrib = searchTerm.attrib;
+    searchTermForQS.arbitraryHeader = searchTerm.arbitraryHeader
     searchTermForQS.op = searchTerm.op;
 
     // mark the first node as a group
@@ -1188,6 +1170,21 @@ function CreateGroupedSearchTerms(searchTermsArray)
   return searchTermsArrayForQS;
 }
 
+function OnLeavingFolder(aFolder)
+{
+  try
+  {
+    // Mark all messages of aFolder as read:
+    // We can't use the command controller, because it is already tuned in to the
+    // new folder, so we just mimic its behaviour wrt goDoCommand('cmd_markAllRead').
+    if (gDBView && gPrefBranch.getBoolPref("mailnews.mark_message_read." + aFolder.server.type))
+    {
+      gDBView.doCommand(nsMsgViewCommandType.markAllRead);
+    }
+  }
+  catch(e){/* ignore */}
+}
+
 var gViewDebug = false;
 
 function viewDebug(str)
