@@ -210,15 +210,11 @@ var DefaultController =
 			case "cmd_markAsJunk":
 			case "cmd_markAsNotJunk":
       case "cmd_recalculateJunkScore":
+      case "cmd_markAsShowRemote":
+      case "cmd_markAsNotPhish":
       case "cmd_applyFilters":
       case "cmd_runJunkControls":
       case "cmd_deleteJunk":
-      case "cmd_label0":
-      case "cmd_label1":
-      case "cmd_label2":
-      case "cmd_label3":
-      case "cmd_label4":
-      case "cmd_label5":
       case "button_file":
 			case "cmd_file":
 			case "cmd_emptyTrash":
@@ -227,6 +223,7 @@ var DefaultController =
       case "cmd_close":
       case "cmd_selectAll":
       case "cmd_selectThread":
+      case "cmd_selectFlagged":
 				return true;
       case "cmd_downloadFlagged":
       case "cmd_downloadSelected":
@@ -321,12 +318,19 @@ var DefaultController =
       case "cmd_markAsFlagged":
       case "button_file":
       case "cmd_file":
-        return (GetNumSelectedMessages() > 0 );
+        return (GetNumSelectedMessages() > 0);
       case "cmd_markAsJunk":
       case "cmd_markAsNotJunk":
-      case "cmd_recalculateJunkScore":
         // can't do news on junk yet.
         return (GetNumSelectedMessages() > 0 && !isNewsURI(GetFirstSelectedMessage()));
+      case "cmd_recalculateJunkScore":
+        if (GetNumSelectedMessages() > 0)
+          gDBView.getCommandStatus(nsMsgViewCommandType.runJunkControls, enabled, checkStatus);
+        return enabled.value;
+      case "cmd_markAsShowRemote":
+        return (GetNumSelectedMessages() > 0 && checkMsgHdrPropertyIsNot("remoteContentPolicy", kAllowRemoteContent));
+      case "cmd_markAsNotPhish":
+        return (GetNumSelectedMessages() > 0 && checkMsgHdrPropertyIsNot("notAPhishMessage", kNotAPhishMessage));
       case "cmd_applyFilters":
         if (gDBView)
           gDBView.getCommandStatus(nsMsgViewCommandType.applyFilters, enabled, checkStatus);
@@ -342,12 +346,6 @@ var DefaultController =
       case "button_mark":
       case "cmd_markAsRead":
       case "cmd_markThreadAsRead":
-      case "cmd_label0":
-      case "cmd_label1":
-      case "cmd_label2":
-      case "cmd_label3":
-      case "cmd_label4":
-      case "cmd_label5":
         return GetNumSelectedMessages() > 0;
       case "button_next":
         return IsViewNavigationItemEnabled();
@@ -370,6 +368,7 @@ var DefaultController =
       case "cmd_search":
         return IsCanSearchMessagesEnabled();
       case "cmd_selectAll":
+      case "cmd_selectFlagged":
         return gDBView != null;
       // these are enabled on when we are in threaded mode
       case "cmd_selectThread":
@@ -477,7 +476,8 @@ var DefaultController =
         MsgCreateFilter();
         break;   
       case "cmd_createFilterFromPopup":
-        break;// This does nothing because the createfilter is invoked from the popupnode oncommand.
+        CreateFilter(document.popupNode);
+        break;
 			case "button_delete":
 			case "cmd_delete":
         // if the user deletes a message before its mark as read timer goes off, we should mark it as read
@@ -630,6 +630,12 @@ var DefaultController =
       case "cmd_recalculateJunkScore":
         analyzeMessagesForJunk();
         return;
+      case "cmd_markAsShowRemote":
+        LoadMsgWithRemoteContent();
+        return;
+      case "cmd_markAsNotPhish":
+        MsgIsNotAScam();
+        return;
       case "cmd_applyFilters":
         MsgApplyFilters(null);
         return;
@@ -639,24 +645,6 @@ var DefaultController =
       case "cmd_deleteJunk":
         deleteJunkInFolder();
         return;
-      case "cmd_label0":
-        gDBView.doCommand(nsMsgViewCommandType.label0);
- 				return;
-      case "cmd_label1":
-        gDBView.doCommand(nsMsgViewCommandType.label1);
-        return; 
-      case "cmd_label2":
-        gDBView.doCommand(nsMsgViewCommandType.label2);
-        return; 
-      case "cmd_label3":
-        gDBView.doCommand(nsMsgViewCommandType.label3);
-        return; 
-      case "cmd_label4":
-        gDBView.doCommand(nsMsgViewCommandType.label4);
-        return; 
-      case "cmd_label5":
-        gDBView.doCommand(nsMsgViewCommandType.label5);
-        return; 
 			case "cmd_emptyTrash":
 				MsgEmptyTrash();
 				return;
@@ -688,6 +676,9 @@ var DefaultController =
             case "cmd_selectThread":
                 gDBView.doCommand(nsMsgViewCommandType.selectThread);
                 break;
+      case "cmd_selectFlagged":
+        gDBView.doCommand(nsMsgViewCommandType.selectFlagged);
+        break;
 		}
 	},
 	
@@ -724,11 +715,10 @@ function FocusRingUpdate_Mail()
   // we really only care about nsFocusController::Focus() happens, 
   // which calls nsFocusController::SetFocusedElement(element)
   var currentFocusedElement = WhichPaneHasFocus();
-  if (!currentFocusedElement)
-    return;
       
 	if (currentFocusedElement != gLastFocusedElement) {
-    currentFocusedElement.setAttribute("focusring", "true");
+    if (currentFocusedElement)
+      currentFocusedElement.setAttribute("focusring", "true");
     
     if (gLastFocusedElement)
       gLastFocusedElement.removeAttribute("focusring");
@@ -746,7 +736,6 @@ function FocusRingUpdate_Mail()
 function WhichPaneHasFocus()
 {
   var threadTree = GetThreadTree();
-  var searchInput = GetSearchInput();
   var folderTree = GetFolderTree();
   var messagePane = GetMessagePane();
     
@@ -756,7 +745,6 @@ function WhichPaneHasFocus()
 	var currentNode = top.document.commandDispatcher.focusedElement;	
 	while (currentNode) {
     if (currentNode === threadTree ||
-        currentNode === searchInput || 
         currentNode === folderTree ||
         currentNode === messagePane)
       return currentNode;
@@ -842,9 +830,14 @@ function IsRenameFolderEnabled()
 function IsCanSearchMessagesEnabled()
 {
   var folderURI = GetSelectedFolderURI();
-  var server = GetServer(folderURI);
-  return server.canSearchMessages;
+  if (!folderURI)
+    return false;
+
+  var folder = GetMsgFolderFromUri(folderURI, false);
+  var isVirtualFolder = folder.flags & MSG_FOLDER_FLAG_VIRTUAL;
+  return folder.server.canSearchMessages && !isVirtualFolder;
 }
+
 function IsFolderCharsetEnabled()
 {
   return IsFolderSelected();
@@ -1044,7 +1037,8 @@ function SearchBarToggled()
   }
 
   for (var currentNode = top.document.commandDispatcher.focusedElement; currentNode; currentNode = currentNode.parentNode) {
-    if (currentNode.getAttribute("hidden") == "true") {
+    // But skip the last node, which is a XULDocument.
+    if ((currentNode instanceof XULElement) && currentNode.hidden) {
       SetFocusThreadPane();
       return;
     }
@@ -1053,33 +1047,35 @@ function SearchBarToggled()
 
 function SwitchPaneFocus(event)
 {
-  var focusedElement = WhichPaneHasFocus();
   var folderTree = GetFolderTree();
   var threadTree = GetThreadTree();
-  var searchInput = GetSearchInput();
   var messagePane = GetMessagePane();
 
+  // Although internally this is actually a four-pane window, it is presented as
+  // a three-pane -- the search pane is more of a toolbar.  So, shift among the
+  // three main panes.
+
+  var focusedElement = WhichPaneHasFocus();
+  if (focusedElement == null)       // focus not on one of the main three panes?
+    focusedElement = threadTree;    // treat as if on thread tree
+
   if (event && event.shiftKey)
   {
-    if (focusedElement == threadTree && searchInput.parentNode.getAttribute('hidden') != 'true')
-      searchInput.focus();
-    else if ((focusedElement == threadTree || focusedElement == searchInput) && !IsFolderPaneCollapsed())
+    // Reverse traversal: Message -> Thread -> Folder -> Message
+    if (focusedElement == threadTree && !IsFolderPaneCollapsed())
       folderTree.focus();
     else if (focusedElement != messagePane && !IsMessagePaneCollapsed())
       SetFocusMessagePane();
-    else 
+    else
       threadTree.focus();
   }
   else
   {
-    if (focusedElement == searchInput)
-      threadTree.focus();
-    else if (focusedElement == threadTree && !IsMessagePaneCollapsed())
+    // Forward traversal: Folder -> Thread -> Message -> Folder
+    if (focusedElement == threadTree && !IsMessagePaneCollapsed())
       SetFocusMessagePane();
     else if (focusedElement != folderTree && !IsFolderPaneCollapsed())
       folderTree.focus();
-    else if (searchInput.parentNode.getAttribute('hidden') != 'true')
-      searchInput.focus();
     else
       threadTree.focus();
   }
@@ -1099,8 +1095,10 @@ function SetFocusThreadPane()
 
 function SetFocusMessagePane()
 {
-    var messagePaneFrame = GetMessagePaneFrame();
-    messagePaneFrame.focus();
+    // XXX hack: to clear the focus on the previous element first focus
+    // on the message pane element then focus on the main content window
+    GetMessagePane().focus();
+    GetMessagePaneFrame().focus();
 }
 
 function is_collapsed(element) 
