@@ -23,10 +23,12 @@
  * Contributor(s):
  *   timeless
  *   slucy@objectivesw.co.uk
- *   Hkan Waara <hwaara@chello.se>
+ *   Håkan Waara <hwaara@chello.se>
  *   Jan Varga <varga@ku.sk>
  *   Seth Spitzer <sspitzer@netscape.com>
  *   David Bienvenu <bienvenu@netscape.com>
+ *   Ian Neal <bugzilla@arlen.demon.co.uk>
+ *   Karsten Düsterloh <mnyromyr@tprac.de>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -49,6 +51,25 @@ const MSG_FLAG_MDN_REPORT_SENT   = 0x800000;
 const MDN_DISPOSE_TYPE_DISPLAYED = 0;
 const MSG_DB_LARGE_COMMIT        = 1;
 
+const kClassicMailLayout = 0;
+const kWideMailLayout = 1;
+const kVerticalMailLayout = 2;
+
+// Per message header flags to keep track of whether the user is allowing remote
+// content for a particular message. 
+// if you change or add more values to these constants, be sure to modify
+// the corresponding definitions in nsMsgContentPolicy.cpp
+const kNoRemoteContentPolicy = 0;
+const kBlockRemoteContent = 1;
+const kAllowRemoteContent = 2;
+
+const kIsAPhishMessage = 0;
+const kNotAPhishMessage = 1;
+
+const kMsgNotificationPhishingBar = 1;
+const kMsgNotificationJunkBar = 2;
+const kMsgNotificationRemoteImages = 3;
+
 var gMessengerBundle;
 var gPromptService;
 var gOfflinePromptsBundle;
@@ -106,16 +127,11 @@ function menu_new_init()
                          .getService(Components.interfaces.nsIIOService);
   var showNew = ((serverType != 'nntp') && canCreateNew) || isInbox;
   ShowMenuItem("menu_newFolder", showNew);
+  ShowMenuItem("menu_newVirtualFolder", showNew);
   EnableMenuItem("menu_newFolder", !isIMAPFolder || !ioService.offline);
+  EnableMenuItem("menu_newVirtualFolder", true);
   if (showNew)
-  {
-    if (isServer || isInbox)
-      SetMenuItemLabel("menu_NewFolder",
-        gMessengerBundle.getString("newFolder"));
-    else
-      SetMenuItemLabel("menu_NewFolder",
-        gMessengerBundle.getString("newSubfolder"));
-  }
+    SetMenuItemLabel("menu_newFolder", gMessengerBundle.getString((isServer || isInbox) ? "newFolderMenuItem" : "newSubfolderMenuItem"));
 }
 
 function goUpdateMailMenuItems(commandset)
@@ -153,16 +169,17 @@ function view_init()
 {
   if (!gMessengerBundle)
       gMessengerBundle = document.getElementById("bundle_messenger");
-  var message_menuitem=document.getElementById('menu_showMessage');
 
-  if (message_menuitem)
-  {
-      var message_menuitem_hidden = message_menuitem.getAttribute("hidden");
-      if(message_menuitem_hidden != "true"){
-          message_menuitem.setAttribute('checked', !IsMessagePaneCollapsed());
-      }
+  var message_menuitem = document.getElementById('menu_showMessagePane');
+  if (message_menuitem && !message_menuitem.hidden) {
+      message_menuitem.setAttribute('checked', !IsMessagePaneCollapsed());
+      message_menuitem.setAttribute('disabled', gAccountCentralLoaded);
   }
 
+  var folderPane_menuitem = document.getElementById('menu_showFolderPane');
+  if (folderPane_menuitem && !folderPane_menuitem.hidden)
+    folderPane_menuitem.setAttribute('checked', !IsFolderPaneCollapsed());
+
   // Disable some menus if account manager is showing
   var sort_menuitem = document.getElementById('viewSortMenu');
   if (sort_menuitem) {
@@ -184,6 +201,14 @@ function view_init()
   document.commandDispatcher.updateCommands('create-menu-view');
 }
 
+function InitViewLayoutStyleMenu(event)
+{
+  var paneConfig = GetMailPaneConfig();
+  var layoutStyleMenuitem = event.target.childNodes[paneConfig];
+  if (layoutStyleMenuitem)
+    layoutStyleMenuitem.setAttribute("checked", "true"); 
+}
+
 function setSortByMenuItemCheckState(id, value)
 {
     var menuitem = document.getElementById(id);
@@ -204,16 +229,18 @@ function InitViewSortByMenu()
     setSortByMenuItemCheckState("sortByStatusMenuitem", (sortType == nsMsgViewSortType.byStatus));
     setSortByMenuItemCheckState("sortBySubjectMenuitem", (sortType == nsMsgViewSortType.bySubject));
     setSortByMenuItemCheckState("sortByUnreadMenuitem", (sortType == nsMsgViewSortType.byUnread));
-    setSortByMenuItemCheckState("sortByLabelMenuitem", (sortType == nsMsgViewSortType.byLabel));
+    setSortByMenuItemCheckState("sortByTagsMenuitem", (sortType == nsMsgViewSortType.byTags));
     setSortByMenuItemCheckState("sortByJunkStatusMenuitem", (sortType == nsMsgViewSortType.byJunkStatus));
     setSortByMenuItemCheckState("sortBySenderMenuitem", (sortType == nsMsgViewSortType.byAuthor));
     setSortByMenuItemCheckState("sortByRecipientMenuitem", (sortType == nsMsgViewSortType.byRecipient));
+    setSortByMenuItemCheckState("sortByAttachmentsMenuitem", (sortType == nsMsgViewSortType.byAttachments));
 
     var sortOrder = gDBView.sortOrder;
     var sortTypeSupportsGrouping = (sortType == nsMsgViewSortType.byAuthor 
         || sortType == nsMsgViewSortType.byDate || sortType == nsMsgViewSortType.byPriority
-        || sortType == nsMsgViewSortType.bySubject || sortType == nsMsgViewSortType.byLabel
-        || sortType == nsMsgViewSortType.byRecipient);
+        || sortType == nsMsgViewSortType.bySubject || sortType == nsMsgViewSortType.byTags
+        || sortType == nsMsgViewSortType.byRecipient|| sortType == nsMsgViewSortType.byFlagged
+        || sortType == nsMsgViewSortType.byAttachments);
 
     setSortByMenuItemCheckState("sortAscending", (sortOrder == nsMsgViewSortOrder.ascending));
     setSortByMenuItemCheckState("sortDescending", (sortOrder == nsMsgViewSortOrder.descending));
@@ -292,9 +319,10 @@ function viewRefreshCustomMailViews(aCurrentViewValue)
   // remove any existing entries...
   var menupopupNode = document.getElementById('viewMessageViewPopup');
   var userDefinedItems = menupopupNode.getElementsByAttribute("userdefined","true");
-  for (var i=0; i<userDefinedItems.length; i++)
+  for (var i=0; userDefinedItems.item(i); )
   {
-    menupopupNode.removeChild(userDefinedItems[i]);
+    if (!menupopupNode.removeChild(userDefinedItems[i]))
+      ++i;
   }
   
   // now rebuild the list
@@ -304,7 +332,7 @@ function viewRefreshCustomMailViews(aCurrentViewValue)
   for (i = 0; i < numItems; i++)
   {
     var newMenuItem = document.createElement("menuitem");
-    newMenuItem.setAttribute("label", mailViewList.getMailViewAt(i).mailViewName);
+    newMenuItem.setAttribute("label", mailViewList.getMailViewAt(i).prettyName);
     newMenuItem.setAttribute("userdefined", "true");
     var oncommandStr = "ViewMessagesBy('userdefinedview" + (kLastDefaultViewIndex + i) + "');";  
     newMenuItem.setAttribute("oncommand", oncommandStr);
@@ -386,14 +414,14 @@ function InitMessageMenu()
   if(copyMenu)
       copyMenu.setAttribute("disabled", !aMessage);
 
-  // Disable Forward as/Label menu items if no message is selected
+  // Disable Forward as/tag menu items if no message is selected
   var forwardAsMenu = document.getElementById("forwardAsMenu");
   if(forwardAsMenu)
       forwardAsMenu.setAttribute("disabled", !aMessage);
 
-  var labelMenu = document.getElementById("labelMenu");
-  if(labelMenu)
-      labelMenu.setAttribute("disabled", !aMessage);
+  var tagMenu = document.getElementById("tagMenu");
+  if(tagMenu)
+      tagMenu.setAttribute("disabled", !aMessage);
 
   // Disable mark menu when we're not in a folder
   var markMenu = document.getElementById("markMenu");
@@ -493,89 +521,175 @@ function IsImapMessage(messageUri)
 
 function SetMenuItemLabel(menuItemId, customLabel)
 {
-    var menuItem = document.getElementById(menuItemId);
-
-    if(menuItem)
-        menuItem.setAttribute('label', customLabel);
+  var menuItem = document.getElementById(menuItemId);
+  if (menuItem)
+    menuItem.setAttribute('label', customLabel);
 }
 
-function InitMessageLabel(menuType)
+function RemoveAllMessageTags()
 {
-    /* this code gets the label strings and changes the menu labels */
-    var color;
+  var selectedMsgUris = GetSelectedMessages();
+  if (!selectedMsgUris.length)
+    return;
+    
+  for (var i = 0; i < selectedMsgUris.length; ++i)
+  {
+    // remove all tags by removing all their tag keys at once
+    // (using a msgHdr's setStringProperty won't notify the threadPane!)
+    var msgHdr = messenger.msgHdrFromURI(selectedMsgUris[i]);
+    msgHdr.label = 0; // remove legacy label
+    msgHdr.folder.getMsgDatabase(msgWindow)
+          .setStringProperty(msgHdr.messageKey, "keywords", "");
+  }
+}
 
-    try
+function ToggleMessageTagKey(index)
+{
+  if (GetNumSelectedMessages() < 1)
+    return;
+  // toggle the tag state based upon that of the first selected message
+  var msgHdr = gDBView.hdrForFirstSelectedMessage;
+  var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
+                             .getService(Components.interfaces.nsIMsgTagService);
+  var tagArray = tagService.getAllTags({});
+  for (var i = 0; i < tagArray.length; ++i)
+  {
+    var key = tagArray[i].key;
+    if (!--index) 
     {
-        var msgFolder = GetLoadedMsgFolder();
-        var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
-        var numSelected = GetNumSelectedMessages();
-        var indices = GetSelectedIndices(gDBView);
-        var isChecked = true;
-        var checkedLabel;
-        var msgKey;
-
-        if (numSelected > 0) {
-            msgKey = gDBView.getKeyAt(indices[0]);
-            checkedLabel = msgDatabase.GetMsgHdrForKey(msgKey).label;
-            if (numSelected > 1) {
-                for (var i = 1; i < indices.length; i++)
-                {
-                    msgKey = gDBView.getKeyAt(indices[i]);
-                    if (msgDatabase.GetMsgHdrForKey(msgKey).label == checkedLabel) {
-                        continue;
-                    }
-                    isChecked = false;
-                    break;
-                }
-            }
-        }
-        else {
-            isChecked = false;
-        }
+      // found the key, now toggle its state
+      var curKeys = msgHdr.getStringProperty("keywords");
+      if (msgHdr.label)
+        curKeys += " $label" + msgHdr.label;
+      var addKey  = (" " + curKeys + " ").indexOf(" " + key + " ") < 0;
+      ToggleMessageTag(key, addKey);
+      return;
     }
-    catch(ex)
+  }
+}
+
+function ToggleMessageTagMenu(target)
+{
+  var key    = target.getAttribute("value");
+  var addKey = target.getAttribute("checked") == "true";
+  ToggleMessageTag(key, addKey);
+}
+
+function ToggleMessageTag(key, addKey)
+{
+  var messages = Components.classes["@mozilla.org/supports-array;1"]
+                           .createInstance(Components.interfaces.nsISupportsArray);
+  var msg = Components.classes["@mozilla.org/supports-array;1"]
+                          .createInstance(Components.interfaces.nsISupportsArray);
+  var selectedMsgUris = GetSelectedMessages();
+  var toggler = addKey ? "addKeywordToMessages" : "removeKeywordFromMessages";
+  var prevHdrFolder = null;
+  // this crudely handles cross-folder virtual folders with selected messages
+  // that spans folders, by coalescing consecutive msgs in the selection
+  // that happen to be in the same folder. nsMsgSearchDBView does this
+  // better, but nsIMsgDBView doesn't handle commands with arguments,
+  // and (un)tag takes a key argument.
+  for (var i = 0; i < selectedMsgUris.length; ++i)
+  {
+    var msgHdr = messenger.msgHdrFromURI(selectedMsgUris[i]);
+    if (msgHdr.label)
     {
-        isChecked = false;
+      // Since we touch all these messages anyway, migrate the label now.
+      // If we don't, the thread tree won't always show the correct tag state,
+      // because resetting a label doesn't update the tree anymore...
+      msg.Clear();
+      msg.AppendElement(msgHdr);
+      msgHdr.folder.addKeywordToMessages(msg, "$label" + msgHdr.label);
+      msgHdr.label = 0; // remove legacy label
     }
-
-    for (var label = 0; label <= 5; label++)
+    if (prevHdrFolder != msgHdr.folder)
     {
-        try
-        {
-            var prefString = gPrefBranch.getComplexValue("mailnews.labels.description." + label,
-                                                         Components.interfaces.nsIPrefLocalizedString);
-            var formattedPrefString = gMessengerBundle.getFormattedString("labelMenuItemFormat" + label,
-                                                                          [prefString], 1); 
-            var menuItemId = menuType + "-labelMenuItem" + label;
-            var menuItem = document.getElementById(menuItemId);
-
-            SetMenuItemLabel(menuItemId, formattedPrefString);
-            if (isChecked && label == checkedLabel)
-              menuItem.setAttribute("checked", "true");
-            else
-              menuItem.setAttribute("checked", "false");
-
-            // commented out for now until UE decides on how to show the Labels menu items.
-            // This code will color either the text or background for the Labels menu items.
-            /*****
-            if (label != 0)
-            {
-                color = gPrefBranch.getCharPref("mailnews.labels.color." + label);
-                // this colors the text of the menuitem only.
-                //menuItem.setAttribute("style", ("color: " + color));
-
-                // this colors the background of the menuitem and
-                // when selected, text becomes white.
-                //menuItem.setAttribute("style", ("color: #FFFFFF"));
-                //menuItem.setAttribute("style", ("background-color: " + color));
-            }
-            ****/
-        }
-        catch(ex)
-        {
-        }
+      if (prevHdrFolder)
+        prevHdrFolder[toggler](messages, key);
+      messages.Clear();
+      prevHdrFolder = msgHdr.folder;
     }
-    document.commandDispatcher.updateCommands('create-menu-label');
+    messages.AppendElement(msgHdr);
+  }
+  if (prevHdrFolder)
+    prevHdrFolder[toggler](messages, key);
+}
+
+function AddTag()
+{
+  var args = {result: "", okCallback: AddTagCallback};
+  var dialog = window.openDialog("chrome://messenger/content/newTagDialog.xul",
+                                 "",
+                                 "chrome,titlebar,modal",
+                                 args);
+}
+
+function AddTagCallback(name, color)
+{
+  var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
+                             .getService(Components.interfaces.nsIMsgTagService);
+  tagService.addTag(name, color, '');
+  try
+  {
+    ToggleMessageTag(tagService.getKeyForTag(name), true);
+  }
+  catch(ex)
+  {
+    return false;
+  }
+  return true;
+}
+
+function SetMessageTagLabel(menuitem, index, name)
+{
+  // if a <key> is defined for this tag, use its key as the accesskey
+  // (the key for the tag at index n needs to have the id key_tag<n>)
+  var shortcutkey = document.getElementById("key_tag" + index);
+  var accesskey = shortcutkey ? shortcutkey.getAttribute("key") : "";
+  if (accesskey)
+    menuitem.setAttribute("accesskey", accesskey);
+  var label = gMessengerBundle.getFormattedString("mailnews.tags.format", 
+                                                  [accesskey, name]);
+  menuitem.setAttribute("label", label);
+}
+
+function InitMessageTags(menuPopup)
+{
+  var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
+                             .getService(Components.interfaces.nsIMsgTagService);
+  var tagArray = tagService.getAllTags({});
+  var tagCount = tagArray.length;
+
+  // remove any existing non-static entries...
+  var menuseparator = menuPopup.lastChild.previousSibling;
+  for (var i = menuPopup.childNodes.length; i > 4; --i)
+    menuPopup.removeChild(menuseparator.previousSibling);
+
+  // hide double menuseparator
+  menuseparator.previousSibling.hidden = !tagCount;
+
+  // create label and accesskey for the static remove item
+  var tagRemoveLabel = gMessengerBundle.getString("mailnews.tags.remove");
+  SetMessageTagLabel(menuPopup.firstChild, 0, tagRemoveLabel);
+
+  // now rebuild the list
+  var msgHdr = gDBView.hdrForFirstSelectedMessage;
+  var curKeys = msgHdr.getStringProperty("keywords");
+  if (msgHdr.label)
+    curKeys += " $label" + msgHdr.label;
+  for (var i = 0; i < tagCount; ++i)
+  {
+    var taginfo = tagArray[i];
+    // TODO we want to either remove or "check" the tags that already exist
+    var newMenuItem = document.createElement("menuitem");
+    SetMessageTagLabel(newMenuItem, i + 1, taginfo.tag);
+    newMenuItem.setAttribute("value", taginfo.key);
+    newMenuItem.setAttribute("type", "checkbox");
+    var removeKey = (" " + curKeys + " ").indexOf(" " + taginfo.key + " ") > -1;
+    newMenuItem.setAttribute('checked', removeKey);
+    newMenuItem.setAttribute('oncommand', 'ToggleMessageTagMenu(event.target);');
+    menuPopup.insertBefore(newMenuItem, menuseparator);
+  }
 }
 
 function InitMessageMark()
@@ -732,13 +846,14 @@ function MsgGetMessagesForAllServers(defaultServer)
     try
     {
         var allServers = accountManager.allServers;
+        var i;
         // array of isupportsarrays of servers for a particular folder
-        var pop3DownloadServersArray = new Array; 
+        var pop3DownloadServersArray = new Array();
         // parallel isupports array of folders to download to...
         var localFoldersToDownloadTo = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
         var pop3Server;
 
-        for (var i=0;i<allServers.Count();i++)
+        for (i = 0; i < allServers.Count(); ++i)
         {
             var currentServer = allServers.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
             var protocolinfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + currentServer.type].getService(Components.interfaces.nsIMsgProtocolInfo);
@@ -746,24 +861,23 @@ function MsgGetMessagesForAllServers(defaultServer)
             {
                 if (defaultServer && defaultServer.equals(currentServer) && 
                   !defaultServer.isDeferredTo &&
-                  defaultServer.msgFolder == defaultServer.rootMsgFolder)
+                  defaultServer.rootFolder == defaultServer.rootMsgFolder)
                 {
                     dump(currentServer.serverURI + "...skipping, already opened\n");
                 }
+                else if (currentServer.type == "pop3" && currentServer.downloadOnBiff)
+                {
+                    CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadServersArray, localFoldersToDownloadTo);
+                    pop3Server = currentServer.QueryInterface(Components.interfaces.nsIPop3IncomingServer);
+                }
                 else
                 {
-                    if (currentServer.type == "pop3" && currentServer.downloadOnBiff)
-                    {
-                      CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadServersArray, localFoldersToDownloadTo);
-                      pop3Server = currentServer.QueryInterface(Components.interfaces.nsIPop3IncomingServer);
-                    }
-                    else
                     // Check to see if there are new messages on the server
                     currentServer.PerformBiff(msgWindow);
                 }
             }
         }
-        for (var i = 0; i < pop3DownloadServersArray.length; i++)
+        for (i = 0; i < pop3DownloadServersArray.length; ++i)
         {
           // any ol' pop3Server will do - the serversArray specifies which servers to download from
           pop3Server.downloadMailFromServers(pop3DownloadServersArray[i], msgWindow, localFoldersToDownloadTo.GetElementAt(i), null);
@@ -896,12 +1010,17 @@ function MsgNewMessage(event)
 function MsgReplyMessage(event)
 {
   var loadedFolder = GetLoadedMsgFolder();
-  var server = loadedFolder.server;
+  if (loadedFolder)
+  {
+    var server = loadedFolder.server;
 
-  if(server && server.type == "nntp")
-    MsgReplyGroup(event);
-  else
-    MsgReplySender(event);
+    if (server && server.type == "nntp")
+    {
+      MsgReplyGroup(event);
+      return;
+    }
+  }
+  MsgReplySender(event);
 }
 
 function MsgReplySender(event)
@@ -909,10 +1028,9 @@ function MsgReplySender(event)
   var loadedFolder = GetLoadedMsgFolder();
   var messageArray = GetSelectedMessages();
 
-  if (event && event.shiftKey)
-    ComposeMessage(msgComposeType.ReplyToSender, msgComposeFormat.OppositeOfDefault, loadedFolder, messageArray);
-  else
-    ComposeMessage(msgComposeType.ReplyToSender, msgComposeFormat.Default, loadedFolder, messageArray);
+  ComposeMessage(msgComposeType.ReplyToSender,
+    (event && event.shiftKey) ? msgComposeFormat.OppositeOfDefault : msgComposeFormat.Default,
+    loadedFolder, messageArray);
 }
 
 function MsgReplyGroup(event)
@@ -996,8 +1114,23 @@ function MsgCreateFilter()
   var msgHdr = gDBView.hdrForFirstSelectedMessage;
   var headerParser = Components.classes["@mozilla.org/messenger/headerparser;1"].getService(Components.interfaces.nsIMsgHeaderParser);
   var emailAddress = headerParser.extractHeaderAddressMailboxes(null, msgHdr.author);
-  if (emailAddress)
-    top.MsgFilters(emailAddress);
+  var accountKey = msgHdr.accountKey;
+  var folder;
+  if (accountKey.length > 0)
+  {
+    var account = accountManager.getAccount(accountKey);
+    if (account)
+    {
+      server = account.incomingServer;
+      if (server)
+        folder = server.rootFolder;
+    }
+  }
+  if (!folder)
+    folder = GetFirstSelectedMsgFolder();
+  
+    if (emailAddress)
+     top.MsgFilters(emailAddress, folder);
 }
 
 function MsgHome(url)
@@ -1140,9 +1273,9 @@ function MsgOpenFromFile()
    }
 
   var uri = fp.fileURL;
-  uri.query = "type=x-message-display";
+  uri.query = "type=application/x-message-display";
 
-  MsgOpenNewWindowForMessage(uri, null);
+  window.openDialog( "chrome://messenger/content/messageWindow.xul", "_blank", "all,chrome,dialog=no,status,toolbar", uri, null, null );
 }
 
 function MsgOpenNewWindowForMsgHdr(hdr)
@@ -1363,9 +1496,10 @@ function MsgCanFindAgain()
   return canFindAgainInPage();
 }
 
-function MsgFilters(emailAddress)
+function MsgFilters(emailAddress, folder)
 {
-    var preselectedFolder = GetFirstSelectedMsgFolder();
+    if (!folder)
+      folder = GetFirstSelectedMsgFolder();
     var args;
     if (emailAddress)
     {
@@ -1373,7 +1507,7 @@ function MsgFilters(emailAddress)
          launch the filterEditor dialog
          and prefill that with the emailAddress */
          
-      var curFilterList = preselectedFolder.getFilterList(msgWindow);
+      var curFilterList = folder.getFilterList(msgWindow);
       args = {filterList: curFilterList};
       args.filterName = emailAddress;
       window.openDialog("chrome://messenger/content/FilterEditor.xul", "", 
@@ -1384,13 +1518,13 @@ function MsgFilters(emailAddress)
          we check this here in args to show filterList dialog */
       if ("refresh" in args && args.refresh)
       {
-         args = { refresh: true, folder: preselectedFolder };
+         args = { refresh: true, folder: folder };
          MsgFilterList(args);
       }
     }
     else  // just launch filterList dialog
     {
-      args = { refresh: false, folder: preselectedFolder };
+      args = { refresh: false, folder: folder };
       MsgFilterList(args);
     }
 }
@@ -1427,6 +1561,11 @@ function MsgApplyFilters()
   filterService.applyFiltersToFolders(tempFilterList, selectedFolders, msgWindow);
 }
 
+function ChangeMailLayout(newLayout)
+{
+  gPrefBranch.setIntPref("mail.pane_config.dynamic", newLayout);
+}
+
 function MsgViewAllHeaders()
 {
     gPrefBranch.setIntPref("mail.show_headers",2);
@@ -1916,7 +2055,6 @@ function CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadSe
 {
   var outNumFolders = new Object();
   var inboxFolder = currentServer.rootMsgFolder.getFoldersWithFlag(0x1000, 1, outNumFolders); 
-  pop3Server = currentServer.QueryInterface(Components.interfaces.nsIPop3IncomingServer);
   // coalesce the servers that download into the same folder...
   var index = localFoldersToDownloadTo.GetIndexOf(inboxFolder);
   if (index == -1)
@@ -1929,12 +2067,8 @@ function CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadSe
     localFoldersToDownloadTo.AppendElement(inboxFolder);
     index = pop3DownloadServersArray.length
     pop3DownloadServersArray[index] = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
-    pop3DownloadServersArray[index].AppendElement(currentServer);
-  }
-  else
-  {
-    pop3DownloadServersArray[index].AppendElement(currentServer);
   }
+  pop3DownloadServersArray[index].AppendElement(currentServer);
 }
 
 function GetMessagesForAllAuthenticatedAccounts()
@@ -1943,14 +2077,14 @@ function GetMessagesForAllAuthenticatedAccounts()
   try
   {
     var allServers = accountManager.allServers;
-
+    var i;
     // array of isupportsarrays of servers for a particular folder
-    var pop3DownloadServersArray = new Array; 
+    var pop3DownloadServersArray = new Array();
     // parallel isupports array of folders to download to...
     var localFoldersToDownloadTo = Components.classes["@mozilla.org/supports-array;1"].createInstance(Components.interfaces.nsISupportsArray);
     var pop3Server;
 
-    for (var i=0;i<allServers.Count();i++)
+    for (i = 0; i < allServers.Count(); ++i)
     {
       var currentServer = allServers.GetElementAt(i).QueryInterface(Components.interfaces.nsIMsgIncomingServer);
       var protocolinfo = Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + currentServer.type].getService(Components.interfaces.nsIMsgProtocolInfo);
@@ -1966,7 +2100,7 @@ function GetMessagesForAllAuthenticatedAccounts()
           GetMessagesForInboxOnServer(currentServer);
       }
     }
-    for (var i = 0; i < pop3DownloadServersArray.length; i++)
+    for (i = 0; i < pop3DownloadServersArray.length; ++i)
     {
       // any ol' pop3Server will do - the serversArray specifies which servers to download from
       pop3Server.downloadMailFromServers(pop3DownloadServersArray[i], msgWindow, localFoldersToDownloadTo.GetElementAt(i), null);
@@ -1983,7 +2117,7 @@ function GetMessagesForAccount(aEvent)
   var uri = aEvent.target.id;
   var server = GetServer(uri);
   GetMessagesForInboxOnServer(server);
-  aEvent.preventBubble();
+  aEvent.stopPropagation();
 }
 
 
@@ -2053,42 +2187,134 @@ function HandleJunkStatusChanged(folder)
   // this might be the stand alone window, open to a message that was
   // and attachment (or on disk), in which case, we want to ignore it.
   var loadedMessage = GetLoadedMessage();
-  if (loadedMessage && (!(/type=x-message-display/.test(loadedMessage))) && IsCurrentLoadedFolder(folder))
+  if (loadedMessage && (!(/type=application\/x-message-display/.test(loadedMessage))) && IsCurrentLoadedFolder(folder))
   {
-    var messageURI = GetLoadedMessage();
-    // if multiple message are selected
-    // and we change the junk status
-    // we don't want to show the junk bar
-    // (since the message pane is blank)
-    if (messageURI && (GetNumSelectedMessages() == 1))
-      SetUpJunkBar(messenger.messageServiceFromURI(messageURI).messageURIToMsgHdr(messageURI));
-    else
-      SetUpJunkBar(null);
+    // if multiple message are selected and we change the junk status
+    // we don't want to show the junk bar (since the message pane is blank)
+    var msgHdr = null;
+    if (GetNumSelectedMessages() == 1)
+      msgHdr = messenger.msgHdrFromURI(loadedMessage);
+    gMessageNotificationBar.setJunkMsg(msgHdr);
   }
 }
 
-function SetUpJunkBar(aMsgHdr)
+var gMessageNotificationBar = 
 {
-  // XXX todo
-  // should this happen on the start, or at the end?
-  // if at the end, we might keep the "this message is junk" up for a while, until a big message is loaded
-  // or do we need to wait until here, to make sure the message is fully analyzed
-  // what about almost hiding it on the start, and then showing here?
+  mBarStatus: 0,
+  // flag bit values for mBarStatus, indexed by kMsgNotificationXXX
+  mBarFlagValues: [
+                    0, // for no msgNotificationBar
+                    1, // 1 << (kMsgNotificationPhishingBar - 1)
+                    2, // 1 << (kMsgNotificationJunkBar - 1)
+                    4  // 1 << (kMsgNotificationRemoteImages - 1)
+                  ],
 
-  var isJunk = false;
+  mMsgNotificationBar: document.getElementById('msgNotificationBar'),
+
+  setJunkMsg: function(aMsgHdr)
+  {
+    var isJunk = false;
   
-  if (aMsgHdr) {
-    var junkScore = aMsgHdr.getStringProperty("junkscore"); 
-    isJunk = ((junkScore != "") && (junkScore != "0"));
+    if (aMsgHdr) 
+    {
+      var junkScore = aMsgHdr.getStringProperty("junkscore"); 
+      isJunk = ((junkScore != "") && (junkScore != "0"));
+    }
+
+    this.updateMsgNotificationBar(kMsgNotificationJunkBar, isJunk);
+
+    goUpdateCommand('button_junk');
+  },
+
+  setRemoteContentMsg: function(aMsgHdr)
+  {  
+    var blockRemote = aMsgHdr &&
+                      aMsgHdr.getUint32Property("remoteContentPolicy") == kBlockRemoteContent;
+    this.updateMsgNotificationBar(kMsgNotificationRemoteImages, blockRemote);
+  },
+
+  // aUrl is the nsIURI for the message currently loaded in the message pane
+  setPhishingMsg: function(aUrl)
+  {
+    // if we've explicitly marked this message as not being an email scam, then don't
+    // bother checking it with the phishing detector.
+    var phishingMsg = false;
+    if (!checkMsgHdrPropertyIsNot("notAPhishMessage", kIsAPhishMessage))
+      phishingMsg = isMsgEmailScam(aUrl);
+    this.updateMsgNotificationBar(kMsgNotificationPhishingBar, phishingMsg);
+  },
+
+  clearMsgNotifications: function()
+  {
+    this.mBarStatus = 0;
+    this.mMsgNotificationBar.selectedIndex = 0;
+    this.mMsgNotificationBar.collapsed = true;
+  },
+
+  // private method used to set our message notification deck to the correct value...
+  updateMsgNotificationBar: function(aIndex, aSet)
+  {
+    var chunk = this.mBarFlagValues[aIndex];
+    var status = aSet ? this.mBarStatus | chunk : this.mBarStatus & ~chunk;
+    this.mBarStatus = status;
+
+    // the phishing message takes precedence over the junk message
+    // which takes precedence over the remote content message
+    this.mMsgNotificationBar.selectedIndex = this.mBarFlagValues.indexOf(status & -status);
+
+    this.mMsgNotificationBar.collapsed = !status;
   }
-  
-  var junkBar = document.getElementById("junkBar");
-  if (isJunk)
-    junkBar.removeAttribute("collapsed");
-  else
-    junkBar.setAttribute("collapsed","true");
- 
-  goUpdateCommand('button_junk');
+};
+
+function LoadMsgWithRemoteContent()
+{
+  // we want to get the msg hdr for the currently selected message
+  // change the "remoteContentBar" property on it
+  // then reload the message
+
+  setMsgHdrPropertyAndReload("remoteContentPolicy", kAllowRemoteContent);
+}
+
+function MsgIsNotAScam()
+{
+  // we want to get the msg hdr for the currently selected message
+  // change the "isPhishingMsg" property on it
+  // then reload the message
+
+  setMsgHdrPropertyAndReload("notAPhishMessage", kNotAPhishMessage);
+}
+
+function setMsgHdrPropertyAndReload(aProperty, aValue)
+{
+  // we want to get the msg hdr for the currently selected message
+  // change the appropiate property on it then reload the message
+
+  var msgURI = GetLoadedMessage();
+
+  if (msgURI && !(/type=application\/x-message-display/.test(msgURI)))
+  {
+    var msgHdr = messenger.msgHdrFromURI(msgURI);
+    if (msgHdr)
+    {
+      msgHdr.setUint32Property(aProperty, aValue);
+      MsgReload();
+    }
+  }
+}
+
+function checkMsgHdrPropertyIsNot(aProperty, aValue)
+{
+  // we want to get the msg hdr for the currently selected message,
+  // get the appropiate property on it and then test against value.
+
+  var msgURI = GetLoadedMessage();
+    
+  if (msgURI && !(/type=application\/x-message-display/.test(msgURI)))
+  {
+    var msgHdr = messenger.msgHdrFromURI(msgURI);
+    return (msgHdr && msgHdr.getUint32Property(aProperty) != aValue);
+  }
+  return false;
 }
 
 function MarkCurrentMessageAsRead()
@@ -2105,6 +2331,11 @@ function ClearPendingReadTimer()
   }
 }
 
+function OnMsgParsed(aUrl)
+{
+  gMessageNotificationBar.setPhishingMsg(aUrl);
+}
+
 function OnMsgLoaded(aUrl)
 {
     if (!aUrl)
@@ -2123,20 +2354,17 @@ function OnMsgLoaded(aUrl)
     // SetNextMessageAfterDelete() when the operation completes (bug 243532).
     gNextMessageViewIndexAfterDelete = -2;
 
-    if (/type=x-message-display/.test(msgURI))
-      SetUpJunkBar(null);
-    else
-    {
-      msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI);
-      SetUpJunkBar(msgHdr);
-    }
-    
+    if (!(/type=application\/x-message-display/.test(msgURI)))
+      msgHdr = messenger.msgHdrFromURI(msgURI);
+
+    gMessageNotificationBar.setJunkMsg(msgHdr);
+
     // we just finished loading a message. set a timer to actually mark the message as read after n seconds
     // where n can be configured by the user.
     var markReadOnADelay = gPrefBranch.getBoolPref("mailnews.mark_message_read.delay");
     if (msgHdr && !msgHdr.isRead)
     {
-      var wintype = document.firstChild.getAttribute('windowtype');
+      var wintype = document.documentElement.getAttribute('windowtype');
       if (markReadOnADelay && wintype == "mail:3pane") // only use the timer if viewing using the 3-pane preview pane and the user has set the pref
         gMarkViewedMessageAsReadTimer = setTimeout(MarkCurrentMessageAsRead, gPrefBranch.getIntPref("mailnews.mark_message_read.delay.interval") * 1000);
       else
@@ -2205,9 +2433,20 @@ function HandleMDNResponse(aUrl)
 	if (IsNewsMessage(msgURI))
 		return;
 
-  var msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI);
-  var mimeHdr = aUrl.mimeHeaders;
-    
+  // if the message is marked as junk, do NOT attempt to process a return receipt
+  // in order to better protect the user
+  if (SelectedMessagesAreJunk())
+    return;
+
+  var msgHdr = messenger.msgHdrFromURI(msgURI);
+  var mimeHdr;
+
+  try {
+    mimeHdr = aUrl.mimeHeaders;
+  } catch (ex) {
+    return;
+  }
+
   // If we didn't get the message id when we downloaded the message header,
   // we cons up an md5: message id. If we've done that, we'll try to extract
   // the message id out of the mime headers for the whole message.
@@ -2255,18 +2494,6 @@ function MsgSearchMessages()
   OpenOrFocusWindow(args, "mailnews:search", "chrome://messenger/content/SearchDialog.xul");
 }
 
-function MsgJunkMail()
-{
-  MsgJunkMailInfo(true);
-  var preselectedFolder = null;
-  if ("GetFirstSelectedMsgFolder" in window)
-    preselectedFolder = GetFirstSelectedMsgFolder();
-
-  var args = { folder: preselectedFolder };
-  window.openDialog("chrome://messenger/content/junkMail.xul", "",
-                    "chrome,resizable=no,modal,dialog", args);
-}
-
 function MsgJunkMailInfo(aCheckFirstUse)
 {
   if (aCheckFirstUse) {
@@ -2319,5 +2546,3 @@ function OpenOrFocusWindow(args, windowType, chromeURL)
   else
     window.openDialog(chromeURL, "", "chrome,resizable,status,centerscreen,dialog=no", args);
 }
-
-
