@@ -53,7 +53,6 @@ var msgHeaderParserContractID		   = "@mozilla.org/messenger/headerparser;1";
 var abAddressCollectorContractID	 = "@mozilla.org/addressbook/services/addressCollecter;1";
 
 var gViewAllHeaders = false;
-var gNumAddressesToShow = 3;
 var gShowOrganization = false;
 var gShowUserAgent = false;
 var gCollectIncoming = false;
@@ -70,13 +69,17 @@ var gOpenLabel;
 var gOpenLabelAccesskey;
 var gSaveLabel;
 var gSaveLabelAccesskey;
+var gDetachLabel;
+var gDetachLabelAccesskey;
+var gDeleteLabel;
+var gDeleteLabelAccesskey;
 var gMessengerBundle;
 var gProfileDirURL;
 var gIOService;
 var gFileHandler;
 
 var msgHeaderParser = Components.classes[msgHeaderParserContractID].getService(Components.interfaces.nsIMsgHeaderParser);
-var abAddressCollector = Components.classes[abAddressCollectorContractID].getService(Components.interfaces.nsIAbAddressCollecter);
+var abAddressCollector = null;
 
 // other components may listen to on start header & on end header notifications for each message we display
 // to do that you need to add yourself to our gMessageListeners array with object that has two properties:
@@ -105,13 +108,14 @@ var gMessageListeners = new Array;
 // Our first view is the collapsed view. This is very light weight view of the data. We only show a couple
 // fields.
 var gCollapsedHeaderList = [ {name:"subject", outputFunction:updateHeaderValueInTextNode},
-                             {name:"from", useShortView:true, outputFunction:OutputEmailAddresses},
+                             {name:"from", useToggle:true, useShortView:true, outputFunction:OutputEmailAddresses},
                              {name:"date", outputFunction:updateHeaderValueInTextNode}];
 
 // We also have an expanded header view. This shows many of your more common (and useful) headers.
 var gExpandedHeaderList = [ {name:"subject"}, 
-                            {name:"from", outputFunction:OutputEmailAddresses},
-                            {name:"reply-to", outputFunction:OutputEmailAddresses},
+                            {name:"from", useToggle:true, outputFunction:OutputEmailAddresses},
+                            {name:"sender", outputFunction:OutputEmailAddresses},
+                            {name:"reply-to", useToggle:true, outputFunction:OutputEmailAddresses},
                             {name:"date"},
                             {name:"to", useToggle:true, outputFunction:OutputEmailAddresses},
                             {name:"cc", useToggle:true, outputFunction:OutputEmailAddresses},
@@ -138,7 +142,7 @@ var currentHeaderData = {};
 // .contentType --> the content type of the attachment
 // url --> an imap, or mailbox url which can be used to fetch the message
 // uri --> an RDF URI which refers to the message containig the attachment
-// notDownloaded --> boolean flag stating whether the attachment is downloaded or not.
+// isExternalAttachment --> boolean flag stating whether the attachment is external or not.
 var currentAttachments = new Array();
 
 // createHeaderEntry --> our constructor method which creates a header Entry 
@@ -149,11 +153,21 @@ var currentAttachments = new Array();
 // headerListInfo --> entry from a header list.
 function createHeaderEntry(prefix, headerListInfo)
 {
+  var useShortView = false;
   var partialIDName = prefix + headerListInfo.name;
   this.enclosingBox = document.getElementById(partialIDName + 'Box');
   this.textNode = document.getElementById(partialIDName + 'Value');
   this.isValid = false;
 
+  if ("useShortView" in headerListInfo)
+  {
+    useShortView = headerListInfo.useShortView;
+    if (useShortView)
+      this.enclosingBox = this.textNode;
+    else
+      this.enclosingBox.emailAddressNode = this.textNode;
+  }
+
   if ("useToggle" in headerListInfo)
   {
     this.useToggle = headerListInfo.useToggle;
@@ -167,13 +181,8 @@ function createHeaderEntry(prefix, headerListInfo)
   else
    this.useToggle = false;
 
-  if ("useShortView" in headerListInfo)
-  {
-    this.useShortView = headerListInfo.useShortView;
-    this.enclosingBox.emailAddressNode = this.textNode;
-  }
-  else
-    this.useShortView = false;
+  if (this.textNode)
+    this.textNode.useShortView = useShortView;
 
   if ("outputFunction" in headerListInfo)
     this.outputFunction = headerListInfo.outputFunction;
@@ -220,7 +229,6 @@ function OnLoadMsgHeaderPane()
   
   // load any preferences that at are global with regards to 
   // displaying a message...
-  gNumAddressesToShow = pref.getIntPref("mailnews.max_header_display_length");
   gCollectIncoming = pref.getBoolPref("mail.collect_email_address_incoming");
   gCollectNewsgroup = pref.getBoolPref("mail.collect_email_address_newsgroup");
   gCollectOutgoing = pref.getBoolPref("mail.collect_email_address_outgoing");
@@ -255,7 +263,7 @@ function OnUnloadMsgHeaderPane()
 var messageHeaderSink = {
     onStartHeaders: function()
     {
-      mSaveHdr = null;
+      this.mSaveHdr = null;
       // clear out any pending collected address timers...
       if (gCollectAddressTimer)
       {
@@ -289,8 +297,9 @@ var messageHeaderSink = {
       gBuildAttachmentPopupForCurrentMsg = true;
       ClearAttachmentList();
       ClearEditMessageButton();
+      gMessageNotificationBar.clearMsgNotifications();
 
-      for (index in gMessageListeners)
+      for (var index in gMessageListeners)
         gMessageListeners[index].onStartHeaders();
     },
 
@@ -309,12 +318,17 @@ var messageHeaderSink = {
       UpdateMessageHeaders();
       if (gIsEditableMsgFolder)
         ShowEditMessageButton();
+
+      for (var index in gMessageListeners)
+        gMessageListeners[index].onEndHeaders();
     },
 
     processHeaders: function(headerNameEnumerator, headerValueEnumerator, dontCollectAddress)
     {
       this.onStartHeaders(); 
 
+      const kMailboxSeparator = ", ";
+      var index = 0;
       while (headerNameEnumerator.hasMore()) 
       {
         var header = new Object;        
@@ -325,11 +339,29 @@ var messageHeaderSink = {
         // we don't have to worry about looking for: Cc and CC, etc.
         var lowerCaseHeaderName = header.headerName.toLowerCase();
 
-        // if we have an x-mailer or x-mimeole string, put it in the user-agent slot which we know how to handle
-        // already. 
-        if (lowerCaseHeaderName == "x-mailer" || lowerCaseHeaderName == "x-mimeole")
+        // if we have an x-mailer, x-newsreader, or x-mimeole string,
+        // put it in the user-agent slot which we know how to handle already.
+        if (lowerCaseHeaderName == "x-mailer" ||
+            lowerCaseHeaderName == "x-newsreader" ||
+            lowerCaseHeaderName == "x-mimeole")
           lowerCaseHeaderName = "user-agent";          
         
+        if (this.mDummyMsgHeader)
+        {
+          if (lowerCaseHeaderName == "from")
+            this.mDummyMsgHeader.author = header.headerValue;
+          else if (lowerCaseHeaderName == "to")
+            this.mDummyMsgHeader.recipients = header.headerValue;
+          else if (lowerCaseHeaderName == "cc")
+            this.mDummyMsgHeader.ccList = header.headerValue;
+          else if (lowerCaseHeaderName == "subject")
+            this.mDummyMsgHeader.subject = header.headerValue;
+          else if (lowerCaseHeaderName == "reply-to")
+            this.mDummyMsgHeader.replyTo = header.headerValue;
+          else if (lowerCaseHeaderName == "message-id")
+            this.mDummyMsgHeader.messageId = header.headerValue;
+
+        }
         // according to RFC 2822, certain headers
         // can occur "unlimited" times
         if (lowerCaseHeaderName in currentHeaderData)
@@ -350,45 +382,61 @@ var messageHeaderSink = {
 
         if (lowerCaseHeaderName == "from")
         {
-          if (header.value && abAddressCollector) 
+          if (header.headerValue)
           {
-            if ((gCollectIncoming && !dontCollectAddress) || 
-                (gCollectNewsgroup && dontCollectAddress))
+            try
             {
-              gCollectAddress = header.headerValue;
-              // collect, and add card if doesn't exist, unknown preferred send format
-              gCollectAddressTimer = setTimeout('abAddressCollector.collectUnicodeAddress(gCollectAddress, true, Components.interfaces.nsIAbPreferMailFormat.unknown);', 2000);
-            }
-            else if (gCollectOutgoing) 
-            {
-              // collect, but only update existing cards, unknown preferred send format
-              gCollectAddress = header.headerValue;
-              gCollectAddressTimer = setTimeout('abAddressCollector.collectUnicodeAddress(gCollectAddress, false, Components.interfaces.nsIAbPreferMailFormat.unknown);', 2000);
+              var createCard = (gCollectIncoming && !dontCollectAddress) || (gCollectNewsgroup && dontCollectAddress);
+              if (createCard || gCollectOutgoing)
+              {
+                if (!abAddressCollector)
+                  abAddressCollector = Components.classes[abAddressCollectorContractID]
+                                                 .getService(Components.interfaces.nsIAbAddressCollecter);
+
+                gCollectAddress = header.headerValue;
+                // collect, add card if doesn't exist and gCollectOutgoing is set, 
+                // otherwise only update existing cards, unknown preferred send format
+                gCollectAddressTimer = setTimeout('abAddressCollector.collectUnicodeAddress(gCollectAddress, ' + createCard + ', Components.interfaces.nsIAbPreferMailFormat.unknown);', 2000);
+              }
             }
+            catch(ex) {}
           }
         } // if lowerCaseHeaderName == "from"
       } // while we have more headers to parse
 
+      if (("from" in currentHeaderData) && ("sender" in currentHeaderData) && msgHeaderParser)
+      {
+        var senderMailbox = kMailboxSeparator + msgHeaderParser.extractHeaderAddressMailboxes(null,
+                            currentHeaderData.sender.headerValue) + kMailboxSeparator;
+        var fromMailboxes = kMailboxSeparator + msgHeaderParser.extractHeaderAddressMailboxes(null,
+                            currentHeaderData.from.headerValue) + kMailboxSeparator;
+        if (fromMailboxes.indexOf(senderMailbox) >= 0)
+          delete currentHeaderData.sender;
+      }
+
       this.onEndHeaders();
     },
 
-    handleAttachment: function(contentType, url, displayName, uri, notDownloaded) 
+    handleAttachment: function(contentType, url, displayName, uri, isExternalAttachment) 
     {
       // presentation level change....don't show vcards as external attachments in the UI.
       // libmime already renders them inline.
 
+      try
+      {
+        if (!this.mSaveHdr)
+          this.mSaveHdr = messenger.msgHdrFromURI(uri);
+      }
+      catch (ex) {}
       if (contentType == "text/x-vcard")
       {
         var inlineAttachments = pref.getBoolPref("mail.inline_attachments");
         var displayHtmlAs = pref.getIntPref("mailnews.display.html_as");
         if (inlineAttachments && !displayHtmlAs)
-        {
-          mSaveHdr = messenger.messageServiceFromURI(uri).messageURIToMsgHdr(uri);
           return;
-        }
       }
 
-      currentAttachments.push (new createNewAttachmentInfo(contentType, url, displayName, uri, notDownloaded));
+      currentAttachments.push (new createNewAttachmentInfo(contentType, url, displayName, uri, isExternalAttachment));
       // if we have an attachment, set the MSG_FLAG_ATTACH flag on the hdr
       // this will cause the "message with attachment" icon to show up
       // in the thread pane
@@ -402,8 +450,7 @@ var messageHeaderSink = {
 
         try {
           // convert the uri into a hdr
-          var hdr = messenger.messageServiceFromURI(uri).messageURIToMsgHdr(uri);
-          hdr.markHasAttachments(true);
+          this.mSaveHdr.markHasAttachments(true);
         }
         catch (ex) {
           dump("ex = " + ex + "\n");
@@ -413,11 +460,6 @@ var messageHeaderSink = {
     
     onEndAllAttachments: function()
     {
-      // if we only got a v-card, turn off the attachments flag
-      if (!currentAttachments.length && mSaveHdr)
-        mSaveHdr.markHasAttachments(false);
-      mSaveHdr = null;
-      displayAttachmentsForExpandedView();
       // AddSaveAllAttachmentsMenu();
       if (gCollapsedHeaderViewMode)
         displayAttachmentsForCollapsedView();
@@ -427,6 +469,20 @@ var messageHeaderSink = {
 
     onEndMsgDownload: function(url)
     {
+      // if we don't have any attachments, turn off the attachments flag
+      if (!this.mSaveHdr)
+      {
+        var messageUrl = url.QueryInterface(Components.interfaces.nsIMsgMessageUrl);
+        try
+        {
+          this.mSaveHdr = messenger.msgHdrFromURI(messageUrl.uri);
+        }
+        catch (ex) {}
+
+      }
+      if (!currentAttachments.length && this.mSaveHdr)
+        this.mSaveHdr.markHasAttachments(false);
+      OnMsgParsed(url);
     },
 
     onEndMsgHeaders: function(url)
@@ -436,6 +492,7 @@ var messageHeaderSink = {
     
     onMsgHasRemoteContent: function(aMsgHdr)
     {
+      gMessageNotificationBar.setRemoteContentMsg(aMsgHdr);
     },
 
     mSecurityInfo  : null,
@@ -447,7 +504,24 @@ var messageHeaderSink = {
     setSecurityInfo: function(aSecurityInfo)
     {
       this.mSecurityInfo = aSecurityInfo;
-    }
+    },
+
+    mDummyMsgHeader: null,
+
+    getDummyMsgHeader: function()
+    {
+      if (!this.mDummyMsgHeader)
+        this.mDummyMsgHeader = new nsDummyMsgHeader();
+      return this.mDummyMsgHeader;
+    },
+    mProperties: null,
+    getProperties: function()
+      {
+      if (!this.mProperties)
+        this.mProperties = Components.classes["@mozilla.org/hash-property-bag;1"].
+          createInstance(Components.interfaces.nsIWritablePropertyBag2);
+      return this.mProperties;
+      }
 };
 
 function EnsureSubjectValue()
@@ -592,7 +666,6 @@ function createNewHeaderView(headerName)
   this.enclosingBox = newHeader
   this.isValid = false;
   this.useToggle = false;
-  this.useShortView = false;
   this.outputFunction = updateHeaderValue;
 }
 
@@ -604,6 +677,7 @@ function UpdateMessageHeaders()
   // iterate over each header we received and see if we have a matching entry in each
   // header view table...
 
+  var headerName;
   for (headerName in currentHeaderData)
   {
     var headerField = currentHeaderData[headerName];
@@ -717,17 +791,12 @@ function OutputNewsgroups(headerEntry, headerValue)
 // extracts them one by one, linkifying each email address into a mailto url. 
 // Then we add the link'ified email address to the parentDiv passed in.
 // 
-// defaultParentDiv --> the div to add the link-ified email addresses into. 
 // emailAddresses --> comma separated list of the addresses for this header field
-// includeShortLongToggle --> true if you want to include the ability to toggle between short/long
-// address views for this header field. If true, then pass in a another div which is the div the long
-// view will be added too...
-// useShortView --> if true, we'll only generate the Name of the email address field instead of
-//                        showing the name + the email address.
 
 function OutputEmailAddresses(headerEntry, emailAddresses)
 {
-  if ( !emailAddresses ) return;
+  if (!emailAddresses)
+    return;
 
   if (msgHeaderParser)
   {
@@ -743,19 +812,14 @@ function OutputEmailAddresses(headerEntry, emailAddresses)
       // if we want to include short/long toggle views and we have a long view, always add it.
       // if we aren't including a short/long view OR if we are and we haven't parsed enough
       // addresses to reach the cutoff valve yet then add it to the default (short) div.
+      var address = {};
+      address.emailAddress = addresses.value[index];
+      address.fullAddress = fullNames.value[index];
+      address.displayName = names.value[index];
       if (headerEntry.useToggle)
-      {
-        var address = {};
-        address.emailAddress = addresses.value[index];
-        address.fullAddress = fullNames.value[index];
-        address.displayName = names.value[index];
         headerEntry.enclosingBox.addAddressView(address);
-      }
       else
-      {
-        updateEmailAddressNode(headerEntry.enclosingBox.emailAddressNode, addresses.value[index], 
-                               fullNames.value[index], names.value[index], headerEntry.useShortView);
-      }
+        updateEmailAddressNode(headerEntry.enclosingBox.emailAddressNode, address);
 
       if (headerEntry.enclosingBox.getAttribute("id") == "expandedfromBox") {
         setFromBuddyIcon(addresses.value[index]);
@@ -765,7 +829,7 @@ function OutputEmailAddresses(headerEntry, emailAddresses)
     }
     
     if (headerEntry.useToggle)
-      headerEntry.enclosingBox.buildViews(gNumAddressesToShow);
+      headerEntry.enclosingBox.buildViews();
   } // if msgheader parser
 }
 
@@ -777,6 +841,9 @@ function setFromBuddyIcon(email)
      // better to cache this?
      var myScreenName = pref.getCharPref("aim.session.screenname");
 
+     if (!abAddressCollector)
+       abAddressCollector = Components.classes[abAddressCollectorContractID].getService(Components.interfaces.nsIAbAddressCollecter);
+
      var card = abAddressCollector.getCardFromAttribute("PrimaryEmail", email);
 
      if (myScreenName && card && card.aimScreenName) {
@@ -785,7 +852,7 @@ function setFromBuddyIcon(email)
          gIOService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
          gFileHandler = gIOService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
          
-         var dirService = Components.classes["@mozilla.org/directory_service;1"]
+         var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
              .getService(Components.interfaces.nsIProperties);
          var profileDir = dirService.get("ProfD", Components.interfaces.nsIFile);
          gProfileDirURL = gIOService.newFileURI(profileDir);
@@ -810,100 +877,52 @@ function setFromBuddyIcon(email)
    fromBuddyIcon.setAttribute("src", "");
 }
 
-function updateEmailAddressNode(emailAddressNode, emailAddress, fullAddress, displayName, useShortView)
+function updateEmailAddressNode(emailAddressNode, address)
 {
-  if (useShortView && displayName) {
-    emailAddressNode.setAttribute("label", displayName);
-    emailAddressNode.setAttribute("tooltiptext", emailAddress);
-  } else {
-    emailAddressNode.setAttribute("label", fullAddress);
-    emailAddressNode.removeAttribute("tooltiptext");
-  }
-  emailAddressNode.setTextAttribute("emailAddress", emailAddress);
-  emailAddressNode.setTextAttribute("fullAddress", fullAddress);  
-  emailAddressNode.setTextAttribute("displayName", displayName);  
+  emailAddressNode.setAttribute("label", address.fullAddress || address.displayName);
+  emailAddressNode.removeAttribute("tooltiptext");
+  emailAddressNode.setAttribute("emailAddress", address.emailAddress);
+  emailAddressNode.setAttribute("fullAddress", address.fullAddress);
+  emailAddressNode.setAttribute("displayName", address.displayName);
   
   if ("AddExtraAddressProcessing" in this)
-    AddExtraAddressProcessing(emailAddress, emailAddressNode);
-}
-
-function AddNodeToAddressBook (emailAddressNode)
-{
-  if (emailAddressNode)
-  {
-    var primaryEmail = emailAddressNode.getAttribute("emailAddress");
-    var displayName = emailAddressNode.getAttribute("displayName");
-    window.openDialog("chrome://messenger/content/addressbook/abNewCardDialog.xul",
-                      "",
-                      "chrome,resizable=no,titlebar,modal,centerscreen", 
-                      {primaryEmail:primaryEmail, displayName:displayName });
-  }
-}
-
-// SendMailToNode takes the email address title button, extracts
-// the email address we stored in there and opens a compose window
-// with that address
-function SendMailToNode(emailAddressNode)
-{
-  var fields = Components.classes["@mozilla.org/messengercompose/composefields;1"].createInstance(Components.interfaces.nsIMsgCompFields);
-  var params = Components.classes["@mozilla.org/messengercompose/composeparams;1"].createInstance(Components.interfaces.nsIMsgComposeParams);
-  if (emailAddressNode && fields && params)
-  {
-    fields.to = emailAddressNode.getAttribute("fullAddress");
-    params.type = Components.interfaces.nsIMsgCompType.New;
-    params.format = Components.interfaces.nsIMsgCompFormat.Default;
-    params.identity = accountManager.getFirstIdentityForServer(GetLoadedMsgFolder().server);
-    params.composeFields = fields;
-    msgComposeService.OpenComposeWindowWithParams(null, params);
-  }
-}
-
-// CopyEmailAddress takes the email address title button, extracts
-// the email address we stored in there and copies it to the clipboard
-function CopyEmailAddress(emailAddressNode)
-{
-  if (emailAddressNode)
-  {
-    var emailAddress = emailAddressNode.getAttribute("emailAddress");
-
-    var contractid = "@mozilla.org/widget/clipboardhelper;1";
-    var iid = Components.interfaces.nsIClipboardHelper;
-    var clipboard = Components.classes[contractid].getService(iid);
-    clipboard.copyString(emailAddress);
-  }
-}
-
-// CreateFilter opens the Message Filters and Filter Rules dialogs.
-//The Filter Rules dialog has focus. The window is prefilled with filtername <email address>
-//Sender condition is selected and the value is prefilled <email address>
-function CreateFilter(emailAddressNode)
-{
-  if (emailAddressNode)
-  {
-     var emailAddress = emailAddressNode.getAttribute("emailAddress");
-     if (emailAddress){
-         top.MsgFilters(emailAddress);
-     }
-  }
+    AddExtraAddressProcessing(address.emailAddress, emailAddressNode);
 }
 
 // createnewAttachmentInfo --> constructor method for creating new attachment object which goes into the
 // data attachment array.
-function createNewAttachmentInfo(contentType, url, displayName, uri, notDownloaded)
+function createNewAttachmentInfo(contentType, url, displayName, uri, isExternalAttachment)
 {
   this.contentType = contentType;
   this.url = url;
   this.displayName = displayName;
   this.uri = uri;
-  this.notDownloaded = notDownloaded;
+  this.isExternalAttachment = isExternalAttachment;
+  this.attachment = this;
 }
 
 createNewAttachmentInfo.prototype.saveAttachment = function saveAttachment()
 {
-  messenger.saveAttachment(this.contentType, 
-                           this.url, 
-                           encodeURIComponent(this.displayName), 
-                           this.uri);
+  if (this.isExternalAttachment)
+    internalSave(this.url, null,
+                 this.displayName, null,
+                 this.contentType, false,
+                 "SaveAttachmentTitle", null, null);
+  else
+    messenger.saveAttachment(this.contentType, 
+                             this.url, 
+                             encodeURIComponent(this.displayName), 
+                             this.uri,
+                             false);
+}
+
+createNewAttachmentInfo.prototype.viewAttachment = function viewAttachment()
+{
+  var url = this.url;
+  if (!this.isExternalAttachment)
+    url += "&filename=" + encodeURIComponent(this.displayName);
+  openDialog("chrome://navigator/content/viewSource.xul",
+             "_blank", "all,dialog=no", url);
 }
 
 createNewAttachmentInfo.prototype.openAttachment = function openAttachment()
@@ -911,7 +930,8 @@ createNewAttachmentInfo.prototype.openAttachment = function openAttachment()
   messenger.openAttachment(this.contentType, 
                            this.url, 
                            encodeURIComponent(this.displayName), 
-                           this.uri);
+                           this.uri,
+                           this.isExternalAttachment);
 }
 
 createNewAttachmentInfo.prototype.printAttachment = function printAttachment()
@@ -924,23 +944,84 @@ createNewAttachmentInfo.prototype.printAttachment = function printAttachment()
   */
 }
 
+createNewAttachmentInfo.prototype.deleteAttachment = function deleteAttachment()
+{
+  messenger.detachAttachment(this.contentType,
+                             this.url,
+                             encodeURIComponent(this.displayName),
+                             this.uri,
+                             false);
+}
+
+createNewAttachmentInfo.prototype.detachAttachment = function detachAttachment()
+{
+  messenger.detachAttachment(this.contentType,
+                             this.url,
+                             encodeURIComponent(this.displayName),
+                             this.uri,
+                             true);
+}
+
+function CanDetachAttachments()
+{
+  var uri = GetLoadedMessage();
+  return !IsNewsMessage(uri) && (!IsImapMessage(uri) || CheckOnline());
+}
+
 function onShowAttachmentContextMenu()
 {
   // if no attachments are selected, disable the Open and Save...
   var attachmentList = document.getElementById('attachmentList');
   var selectedAttachments = attachmentList.selectedItems;
   var openMenu = document.getElementById('context-openAttachment');
+  var viewMenu = document.getElementById('context-viewAttachment');
   var saveMenu = document.getElementById('context-saveAttachment');
-  if (selectedAttachments.length > 0)
+  var detachMenu = document.getElementById('context-detachAttachment');
+  var deleteMenu = document.getElementById('context-deleteAttachment');
+  var detachAllMenu = document.getElementById('context-detachAllAttachments');
+  var deleteAllMenu = document.getElementById('context-deleteAllAttachments');
+  var canDetach = CanDetachAttachments();
+  var canOpen = false;
+  for (var i = 0; i < selectedAttachments.length && !canOpen; i++)
+    canOpen = selectedAttachments[i].attachment.contentType != 'text/x-moz-deleted';
+  if (canOpen && selectedAttachments.length == 1)
   {
     openMenu.removeAttribute('disabled');
-    saveMenu.removeAttribute('disabled');
+    viewMenu.removeAttribute('disabled');
   }
   else
   {
     openMenu.setAttribute('disabled', true);
+    viewMenu.setAttribute('disabled', true);
+  }
+  if (canOpen)
+  {
+    saveMenu.removeAttribute('disabled');
+  }
+  else
+  {
     saveMenu.setAttribute('disabled', true);
   }
+  if (canDetach && canOpen)
+  {
+    detachMenu.removeAttribute('disabled');
+    deleteMenu.removeAttribute('disabled');
+  }
+  else
+  {
+    detachMenu.setAttribute('disabled', 'true');
+    deleteMenu.setAttribute('disabled', 'true');
+  }
+  if (canDetach)
+  {
+    detachAllMenu.removeAttribute('disabled');
+    deleteAllMenu.removeAttribute('disabled');
+  }
+  else
+  {
+    detachAllMenu.setAttribute('disabled', 'true');
+    deleteAllMenu.setAttribute('disabled', 'true');
+  }
 }
 
 // this is our onclick handler for the attachment list. 
@@ -967,9 +1048,10 @@ function handleAttachmentSelection(commandPrefix)
 {
   var attachmentList = document.getElementById('attachmentList');
   var selectedAttachments = attachmentList.selectedItems;
-  var listItem = selectedAttachments[0];
-
-  listItem.attachment[commandPrefix]();
+  if (selectedAttachments.length > 1)
+    HandleMultipleAttachments(commandPrefix, selectedAttachments);
+  else
+    selectedAttachments[0].attachment[commandPrefix]();
 }
 
 function displayAttachmentsForExpandedView()
@@ -986,13 +1068,16 @@ function displayAttachmentsForExpandedView()
       // we need to create a listitem to insert the attachment
       // into the attachment list..
       var item = attachmentList.appendItem(attachment.displayName,"");
-      item.setAttribute("class", "listitem-iconic"); 
+      item.setAttribute("class", "listitem-iconic attachment-item"); 
       item.setAttribute("tooltip", "attachmentListTooltip");
       item.attachment = attachment;
       item.setAttribute("attachmentUrl", attachment.url);
       item.setAttribute("attachmentContentType", attachment.contentType);
       item.setAttribute("attachmentUri", attachment.uri);
-      setApplicationIconForAttachment(attachment, item);
+      if (attachment.contentType == "text/x-moz-deleted")
+        item.setAttribute('disabled', 'true');
+      else
+        setApplicationIconForAttachment(attachment, item);
     } // for each attachment   
 
     gBuildAttachmentsForCurrentMsg = true;
@@ -1048,6 +1133,18 @@ function FillAttachmentListPopup(popup)
 
   gBuildAttachmentPopupForCurrentMsg = false;
 
+  var detachAllMenu = document.getElementById('file-detachAllAttachments');
+  var deleteAllMenu = document.getElementById('file-deleteAllAttachments');
+  if (CanDetachAttachments())
+  {
+    detachAllMenu.removeAttribute('disabled');
+    deleteAllMenu.removeAttribute('disabled');
+  }
+  else
+  {
+    detachAllMenu.setAttribute('disabled', 'true');
+    deleteAllMenu.setAttribute('disabled', 'true');
+  }
 }
 
 // Public method used to clear the file attachment menu
@@ -1055,7 +1152,7 @@ function ClearAttachmentMenu(popup)
 { 
   if ( popup ) 
   { 
-     while ( popup.childNodes.length > 2 ) 
+     while ( popup.childNodes[0].localName == 'menu' )
        popup.removeChild(popup.childNodes[0]); 
   } 
 }
@@ -1077,11 +1174,9 @@ function addAttachmentToPopup(popup, attachment, attachmentIndex)
       if (!gMessengerBundle)
         gMessengerBundle = document.getElementById("bundle_messenger");
 
-      // insert the item just before the separator...the separator is the 2nd to last element in the popup.
-      item.setAttribute('class', 'menu-iconic');
-      setApplicationIconForAttachment(attachment,item);
-      var numItemsInPopup = popup.childNodes.length;
-      item = popup.insertBefore(item, popup.childNodes[numItemsInPopup-2]);
+      // insert the item just before the separator
+      item = popup.insertBefore(item, popup.childNodes[attachmentIndex - 1]);
+      item.setAttribute('class', 'menu-iconic attachment-item');
 
       var formattedDisplayNameString = gMessengerBundle.getFormattedString("attachmentDisplayNameFormat",
                                        [attachmentIndex, attachment.displayName]);
@@ -1091,39 +1186,52 @@ function addAttachmentToPopup(popup, attachment, attachmentIndex)
 
       var openpopup = document.createElement('menupopup');
       openpopup = item.appendChild(openpopup);
+      if (attachment.contentType == "text/x-moz-deleted") {
+        item.setAttribute('disabled', 'true');
+        return;
+      }
+      openpopup.attachment = attachment;
+      openpopup.addEventListener('popupshowing', FillAttachmentItemPopup, false);
+      setApplicationIconForAttachment(attachment, item);
+    }
+  }
+}
 
-      var menuitementry = document.createElement('menuitem');     
+function FillAttachmentItemPopup(event)
+{
+  var canDetach = CanDetachAttachments();
+  var openpopup = event.target;
+  openpopup.removeEventListener('popupshowing', FillAttachmentItemPopup, false);
 
-      menuitementry.attachment = attachment;
-      menuitementry.setAttribute('oncommand', 'this.attachment.openAttachment()'); 
+  var menuitementry = document.getElementById("context-openAttachment").cloneNode(false);
+  menuitementry.setAttribute('oncommand', 'this.parentNode.attachment.openAttachment();'); 
+  menuitementry = openpopup.appendChild(menuitementry);
 
-      if (!gSaveLabel)
-        gSaveLabel = gMessengerBundle.getString("saveLabel");
-      if (!gSaveLabelAccesskey)
-        gSaveLabelAccesskey = gMessengerBundle.getString("saveLabelAccesskey");
-      if (!gOpenLabel)
-        gOpenLabel = gMessengerBundle.getString("openLabel");
-      if (!gOpenLabelAccesskey)
-        gOpenLabelAccesskey = gMessengerBundle.getString("openLabelAccesskey");
+  menuitementry = document.getElementById("context-viewAttachment").cloneNode(false);
+  menuitementry.setAttribute('oncommand', 'this.parentNode.attachment.viewAttachment();');
+  menuitementry = openpopup.appendChild(menuitementry);
 
-      menuitementry.setAttribute('label', gOpenLabel); 
-      menuitementry.setAttribute('accesskey', gOpenLabelAccesskey); 
-      menuitementry = openpopup.appendChild(menuitementry);
+  menuitementry = document.getElementById("context-saveAttachment").cloneNode(false);
+  menuitementry.setAttribute('oncommand', 'this.parentNode.attachment.saveAttachment()'); 
+  menuitementry = openpopup.appendChild(menuitementry);
 
-      var menuseparator = document.createElement('menuseparator');
-      openpopup.appendChild(menuseparator);
-      
-      menuitementry = document.createElement('menuitem');
-      menuitementry.attachment = attachment;
-      menuitementry.setAttribute('oncommand', 'this.attachment.saveAttachment()'); 
-      menuitementry.setAttribute('label', gSaveLabel); 
-      menuitementry.setAttribute('accesskey', gSaveLabelAccesskey); 
-      menuitementry = openpopup.appendChild(menuitementry);
-    }  // if we created a menu item for this attachment...
-  } // if we have a popup
+  var menuseparator = document.createElement('menuseparator');
+  openpopup.appendChild(menuseparator);
+
+  menuitementry = document.getElementById("context-detachAttachment").cloneNode(false);
+  menuitementry.setAttribute('oncommand', 'this.parentNode.attachment.detachAttachment()'); 
+  if (!canDetach)
+    menuitementry.setAttribute('disabled', 'true');
+  menuitementry = openpopup.appendChild(menuitementry);
+
+  menuitementry = document.getElementById("context-deleteAttachment").cloneNode(false);
+  menuitementry.setAttribute('oncommand', 'this.attachment.deleteAttachment()'); 
+  if (!canDetach)
+    menuitementry.setAttribute('disabled', 'true');
+  menuitementry = openpopup.appendChild(menuitementry);
 } 
 
-function SaveAllAttachments()
+function HandleMultipleAttachments(commandPrefix, selectedAttachments)
 {
  try 
  {
@@ -1134,30 +1242,55 @@ function SaveAllAttachments()
    var attachmentMessageUriArray = new Array();
 
    // populate these arrays..
-   for (index in currentAttachments)
+   for (index in selectedAttachments)
    {
-     var attachment = currentAttachments[index];
+     var attachment = selectedAttachments[index].attachment;
      attachmentContentTypeArray[index] = attachment.contentType;
      attachmentUrlArray[index] = attachment.url;
      attachmentDisplayNameArray[index] = encodeURI(attachment.displayName);
      attachmentMessageUriArray[index] = attachment.uri;
    }
 
-   // okay the list has been built...now call our save all attachments code...
-   messenger.saveAllAttachments(attachmentContentTypeArray.length,
-                                attachmentContentTypeArray, attachmentUrlArray,
-                                attachmentDisplayNameArray, attachmentMessageUriArray);
+   // okay the list has been built...now call our action code...
+   switch (commandPrefix)
+   {
+     case "saveAttachment":
+       messenger.saveAllAttachments(attachmentContentTypeArray.length,
+                                    attachmentContentTypeArray,
+                                    attachmentUrlArray,
+                                    attachmentDisplayNameArray,
+                                    attachmentMessageUriArray);
+       break;
+     case "detachAttachment":
+       messenger.detachAllAttachments(attachmentContentTypeArray.length,
+                                      attachmentContentTypeArray,
+                                      attachmentUrlArray,
+                                      attachmentDisplayNameArray,
+                                      attachmentMessageUriArray,
+                                      true /* save */);
+       break;
+     case "deleteAttachment":
+       messenger.detachAllAttachments(attachmentContentTypeArray.length,
+                                      attachmentContentTypeArray,
+                                      attachmentUrlArray,
+                                      attachmentDisplayNameArray,
+                                      attachmentMessageUriArray,
+                                      false /* don't save */);
+       break;
+     default:
+       dump (commandPrefix + "** unknown handle all attachments action **\n");
+   }
  }
  catch (ex)
  {
-   dump ("** failed to save all attachments **\n");
+   dump ("** failed to handle all attachments **\n");
  }
 }
 
 function ClearAttachmentList() 
 { 
   // we also have to disable the File/Attachments menuitem
-  node = document.getElementById("fileAttachmentMenu");
+  var node = document.getElementById("fileAttachmentMenu");
   if (node)
     node.setAttribute("disabled", "true");
 
@@ -1165,8 +1298,8 @@ function ClearAttachmentList()
   var list = document.getElementById('attachmentList');
   list.clearSelection();
 
-  while (list.childNodes.length) 
-    list.removeItemAt(list.childNodes.length - 1);
+  while (list.hasChildNodes()) 
+    list.removeChild(list.lastChild);
 }
 
 function ShowEditMessageButton() 
@@ -1264,5 +1397,21 @@ nsFlavorDataProvider.prototype =
 
 }
 
+function nsDummyMsgHeader()
+{
+}
 
-
+nsDummyMsgHeader.prototype =
+{
+  mProperties : new Array,
+  getStringProperty : function(property) {return this.mProperties[property];},
+  setStringProperty : function(property, val) {this.mProperties[property] = val;},
+  messageSize : 0,
+  recipients : null,
+  from : null,
+  subject : null,
+  ccList : null,
+  messageId : null,
+  accountKey : "",
+  folder : null
+};
