@@ -119,9 +119,10 @@ var folderListener = {
     OnItemPropertyChanged: function(item, property, oldValue, newValue) { },
 
     OnItemIntPropertyChanged: function(item, property, oldValue, newValue) {
-      if (item == msgWindow.openFolder) {
+      if (item == gMsgFolderSelected) {
         if(property.toString() == "TotalMessages" || property.toString() == "TotalUnreadMessages") {
-          UpdateStatusMessageCounts(msgWindow.openFolder);
+          UpdateStatusMessageCounts(gMsgFolderSelected);
+          UpdateLocationBar(item);
         }      
       }
     },
@@ -135,13 +136,13 @@ var folderListener = {
        var eventType = event.toString();
        if (eventType == "FolderLoaded") {
          if (folder) {
+           var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
            var uri = folder.URI;
            var rerootingFolder = (uri == gCurrentFolderToReroot);
            if (rerootingFolder) {
             viewDebug("uri = gCurrentFolderToReroot, setting gQSViewIsDirty\n");
              gQSViewIsDirty = true;
              gCurrentFolderToReroot = null;
-             var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
              if(msgFolder) {
                msgFolder.endFolderLoading();
                // suppress command updating when rerooting the folder
@@ -203,7 +204,6 @@ var folderListener = {
                // if we failed to scroll to a new message,
                // reselect the last selected message
                var lastMessageLoaded = msgFolder.lastMessageLoaded;
-                 
                if (lastMessageLoaded != nsMsgKey_None) {
                  scrolled = SelectAndScrollToKey(lastMessageLoaded);
                }
@@ -260,21 +260,17 @@ var folderListener = {
              else if (gVirtualFolderTerms)
              {
                 gDefaultSearchViewTerms = null;
-                ViewChangeByValue(-1); // override current view
                 viewDebug("searching gVirtualFolderTerms\n");
-                Search("");
-  //              gVirtualFolderTerms = null;
+                loadVirtualFolder();
              }
              else if (gMsgFolderSelected.flags & MSG_FOLDER_FLAG_VIRTUAL)
              {
                 viewDebug("selected folder is virtual\n");
                 gDefaultSearchViewTerms = null;
-                ViewChangeByValue(-1); // override current view
              }
              else
              {               
                // get the view value from the folder
-               var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
                if (msgFolder)
                {
                  var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
@@ -650,25 +646,63 @@ var gThreePaneIncomingServerListener = {
     }
 }
 
+function GetMailPaneConfig() {
+  try {
+    return pref.getIntPref("mail.pane_config.dynamic");
+  } catch (e) {
+    return pref.getIntPref("mail.pane_config");
+  }
+}
+
+function UpdateMailPaneConfig() {
+  const dynamicIds = ["messagesBox", "mailContent", "threadPaneBox"]
+  var desiredId = dynamicIds[GetMailPaneConfig()];
+  var messagePane = GetMessagePane();
+  if (messagePane.parentNode.id != desiredId) {
+    ClearAttachmentList();
+    var messagePaneSplitter = GetThreadAndMessagePaneSplitter();
+    var desiredParent = document.getElementById(desiredId);
+    desiredParent.appendChild(messagePaneSplitter);
+    desiredParent.appendChild(messagePane);
+    messagePaneSplitter.orient = desiredParent.orient;
+    messenger.SetWindow(null, null);
+    messenger.SetWindow(window, msgWindow);
+    if (gDBView && GetNumSelectedMessages() == 1)
+      gDBView.reloadMessage();
+  }
+}
+
+const MailPaneConfigObserver = {
+  observe: function observe(subject, topic, prefName) {
+    // verify that we're changing the mail pane config pref
+    if (topic == "nsPref:changed")
+      UpdateMailPaneConfig();
+  }
+};
 
 /* Functions related to startup */
 function OnLoadMessenger()
 {
   AddMailOfflineObserver();
   CreateMailWindowGlobals();
+  pref.QueryInterface(Components.interfaces.nsIPrefBranch2);
+  pref.addObserver("mail.pane_config.dynamic", MailPaneConfigObserver, false);
+  UpdateMailPaneConfig();
   Create3PaneGlobals();
   AddToolBarPrefListener();
   ShowHideToolBarButtons();
   verifyAccounts(null);
     
-  HideAccountCentral();
   InitMsgWindow();
   messenger.SetWindow(window, msgWindow);
 
   InitializeDataSources();
   InitPanes();
+  
+  MigrateJunkMailSettings();
 
   accountManager.setSpecialFolders();
+  accountManager.loadVirtualFolders();
   accountManager.addIncomingServerListener(gThreePaneIncomingServerListener);
 
   AddToSession();
@@ -696,20 +730,15 @@ function OnLoadMessenger()
 
   gNotifyDefaultInboxLoadedOnStartup = true;
 
-  // fix for #168937.  now that we don't have a sidebar
-  // users who haven't moved the splitter will
-  // see it jump around
-  var messengerBox = document.getElementById("messengerBox");
-  if (!messengerBox.getAttribute("width")) {
-    messengerBox.setAttribute("width","500px");
-  }
-
   //Set focus to the Thread Pane the first time the window is opened.
   SetFocusThreadPane();
 }
 
 function OnUnloadMessenger()
 {
+  pref.removeObserver("mail.pane_config.dynamic", MailPaneConfigObserver, false);
+
+  OnLeavingFolder(gMsgFolderSelected);  // mark all read in current folder
   accountManager.removeIncomingServerListener(gThreePaneIncomingServerListener);
   RemoveToolBarPrefListener();
   // FIX ME - later we will be able to use onload from the overlay
@@ -727,21 +756,12 @@ function NotifyObservers(aSubject, aTopic, aData)
 
 function Create3PaneGlobals()
 {
+  accountCentralBox = document.getElementById("accountCentralBox");
+  gSearchBox = document.getElementById("searchBox");
+  gSearchBox.collapsed = true;
+  GetMessagePane().collapsed = true;
 }
  
-// because the "open" state persists, we'll call
-// PerformExpand() for all servers that are open at startup.            
-function PerformExpandForAllOpenServers()
-{
-    var servers = accountManager.allServers;
-    for (var i = 0; i < servers.Count(); i++)
-    {
-        var server = servers.QueryElementAt(i, Components.interfaces.nsIMsgIncomingServer);
-        if (server.type != "imap" && !server.rootMsgFolder.getFlag(MSG_FOLDER_FLAG_ELIDED))
-            server.performExpand(msgWindow);
-    }
-}
-
 function loadStartFolder(initialUri)
 {
     var folderTree = GetFolderTree();
@@ -785,33 +805,19 @@ function loadStartFolder(initialUri)
 
                 startFolderResource = inboxFolder.QueryInterface(Components.interfaces.nsIRDFResource);
             }
-            else
-            {
-                // set the startFolderResource to the server, so we select it
-                // so we'll get account central
-                startFolderResource = RDF.GetResource(defaultServer.serverURI);
-            }
         }
 
         var startFolder = startFolderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
 
-        // Perform biff on the server to check for new mail, except for imap
-        // or a pop3 account that is deferred or deferred to,
+        // Perform biff on the server to check for new mail, except for 
+        // a pop3 account that is deferred or deferred to,
         // or the case where initialUri is non-null (non-startup)
         if (!initialUri && isLoginAtStartUpEnabled && gLoadStartFolder
-            && defaultServer.type != "imap" && !defaultServer.isDeferredTo &&
-            defaultServer.msgFolder == defaultServer.rootMsgFolder)
+            && !defaultServer.isDeferredTo &&
+            defaultServer.rootFolder == defaultServer.rootMsgFolder)
           defaultServer.PerformBiff(msgWindow);        
 
         SelectFolder(startFolder.URI);
-
-        // because the "open" state persists, we'll call
-        // PerformExpand() for all servers that are open at startup.
-        // note, because of the "news.persist_server_open_state_in_folderpane" pref
-        // we don't persist the "open" state of news servers across sessions, 
-        // but we do within a session, so if you open another 3 pane
-        // and a news server is "open", we'll update the unread counts.
-        PerformExpandForAllOpenServers();
     }
     catch(ex)
     {
@@ -829,7 +835,8 @@ function loadStartFolder(initialUri)
             InitPrompts();
             InitServices();
 
-            if (gPromptService) 
+            var sendUnsentWhenGoingOnlinePref = pref.getIntPref("offline.send.unsent_messages");
+            if(gPromptService && sendUnsentWhenGoingOnlinePref == 0) // pref is "ask"
             {
                 var buttonPressed = gPromptService.confirmEx(window, 
                                     gOfflinePromptsBundle.getString('sendMessagesOfflineWindowTitle'), 
@@ -839,9 +846,11 @@ function loadStartFolder(initialUri)
                                     gOfflinePromptsBundle.getString('sendMessagesSendButtonLabel'),
                                     gOfflinePromptsBundle.getString('sendMessagesNoSendButtonLabel'),
                                     null, null, {value:0});
-                if (buttonPressed == 0) 
+                if (buttonPressed == 0)
                     SendUnsentMessages();
             }
+            else if(sendUnsentWhenGoingOnlinePref == 1) // pref is "yes"
+                SendUnsentMessages();
         }
     }
 }
@@ -862,6 +871,7 @@ function AddToSession()
 function InitPanes()
 {
     OnLoadFolderPane();
+    OnLoadLocationTree();
     OnLoadThreadPane();
     SetupCommandUpdateHandlers();
 }
@@ -871,6 +881,8 @@ function InitializeDataSources()
 	//Setup common mailwindow stuff.
 	AddDataSources();
 
+	SetupMoveCopyMenus('goMenu', accountManagerDataSource, folderDataSource);
+
 	//To threadpane move context menu
 	SetupMoveCopyMenus('threadPaneContext-moveMenu', accountManagerDataSource, folderDataSource);
 
@@ -888,28 +900,17 @@ function OnFolderUnreadColAttrModified(event)
     }
 }
 
-// builds prior to 8-14-2001 did not have the unread and total columns
-// in the folder pane.  so if a user ran an old build, and then
-// upgraded, they get the new columns, and this causes problems
-// because it looks like all the folder names are gone (see bug #96979)
-// to work around this, we hide those columns once, using the 
-// "mail.ui.folderpane.version" pref.
+function OnAttachmentColAttrModified(event)
+{
+  if (event.attrName == "hidden")
+    UpdateAttachmentCol(false);
+}
+
 function UpgradeFolderPaneUI()
 {
-  var folderPaneUIVersion = pref.getIntPref("mail.ui.folderpane.version");
-
-  if (folderPaneUIVersion == 1) {
-    var folderUnreadCol = document.getElementById("folderUnreadCol");
-    folderUnreadCol.setAttribute("hidden", "true");
-    var folderTotalCol = document.getElementById("folderTotalCol");
-    folderTotalCol.setAttribute("hidden", "true");
-    pref.setIntPref("mail.ui.folderpane.version", 2);
-  } // we fall through to the == 2 case so we'll upgrade v 1 profiles correctly
-  if (folderPaneUIVersion <= 2) {
-    var folderSizeCol = document.getElementById("folderSizeCol");
-    folderSizeCol.setAttribute("hidden", "true");
-    pref.setIntPref("mail.ui.folderpane.version", 3);
-  }
+  // placeholder in case any new columns get added to the folder pane
+  // note that this function fails to notice a pane layout switch
+  // var folderPaneUIVersion = pref.getIntPref("mail.ui.folderpane.version");
 }
 
 function OnLoadFolderPane()
@@ -949,44 +950,39 @@ function OnLoadFolderPane()
 // "mailnews.ui.threadpane.version" pref.
 function UpgradeThreadPaneUI()
 {
-  var labelCol;
-  var threadPaneUIVersion;
-
   try {
-    threadPaneUIVersion = pref.getIntPref("mailnews.ui.threadpane.version");
-    if (threadPaneUIVersion < 4) {
-      var threadTree = document.getElementById("threadTree");
-      var junkCol = document.getElementById("junkStatusCol");
-      var beforeCol;
-
-      if (threadPaneUIVersion < 3) {
-        var subjectCol = document.getElementById("subjectCol");
-      
+    var threadTree = document.getElementById("threadTree");
+    var junkCol = document.getElementById("junkStatusCol");
+    var subjectCol = document.getElementById("subjectCol");
+    var beforeCol;
+    switch (pref.getIntPref("mailnews.ui.threadpane.version")) {
+      case 1: // upgrade from 1 to 2
+      case 2: // upgrade from 2 to 3
         beforeCol = subjectCol.boxObject.nextSibling.boxObject.nextSibling;
         if (beforeCol)
           threadTree._reorderColumn(junkCol, beforeCol, true);
         else // subjectCol was the last column, put it after
           threadTree._reorderColumn(junkCol, subjectCol, false);
 
-        if (threadPaneUIVersion == 1) {
-          labelCol = document.getElementById("labelCol");
-          labelCol.setAttribute("hidden", "true");
-        }
-      }
+      case 3: // upgrade from 3 to 4
+        var senderCol = document.getElementById("senderCol");
+        var recipientCol = document.getElementById("recipientCol");
+        beforeCol = junkCol.boxObject.nextSibling.boxObject.nextSibling;
+        if (beforeCol)
+          threadTree._reorderColumn(recipientCol, beforeCol, true);
+        else // junkCol was the last column, put it after
+          threadTree._reorderColumn(recipientCol, junkCol, false);
+        threadTree._reorderColumn(senderCol, recipientCol, true);
 
-      var senderCol = document.getElementById("senderCol");
-      var recipientCol = document.getElementById("recipientCol");
-    
-      beforeCol = junkCol.boxObject.nextSibling.boxObject.nextSibling;
-      if (beforeCol)
-        threadTree._reorderColumn(recipientCol, beforeCol, true);
-      else // junkCol was the last column, put it after
-        threadTree._reorderColumn(recipientCol, junkCol, false);
-      threadTree._reorderColumn(senderCol, recipientCol, true);
+      case 4: // upgrade from 4 to 5
+        var attachmentCol = document.getElementById("attachmentCol");
+        threadTree._reorderColumn(attachmentCol, subjectCol, true);
+        pref.setIntPref("mailnews.ui.threadpane.version", 5);
 
-      pref.setIntPref("mailnews.ui.threadpane.version", 4);
+      default: // already upgraded
+        break;
     }
-	}
+  }
   catch (ex) {
     dump("UpgradeThreadPane: ex = " + ex + "\n");
   }
@@ -995,6 +991,75 @@ function UpgradeThreadPaneUI()
 function OnLoadThreadPane()
 {
     UpgradeThreadPaneUI();
+    UpdateAttachmentCol(true);
+}
+
+function UpdateAttachmentCol(aFirstTimeFlag)
+{
+  var attachmentCol = document.getElementById("attachmentCol");
+  var threadTree = GetThreadTree();
+  threadTree.setAttribute("noattachcol", attachmentCol.getAttribute("hidden"));
+  if (aFirstTimeFlag)
+    attachmentCol.addEventListener("DOMAttrModified", OnAttachmentColAttrModified, false);
+  else
+    threadTree.treeBoxObject.clearStyleAndImageCaches();
+}
+
+function OnLocationToolbarAttrModified(event)
+{
+    if (!/collapsed/.test(event.attrName))
+        return;
+    var searchBox = document.getElementById("searchBox");
+    var desiredParent = document.getElementById("msgLocationToolbar");
+    if (desiredParent.getAttribute("collapsed") == "true" ||
+        desiredParent.getAttribute("moz-collapsed") == "true")
+        desiredParent = document.getElementById("searchBoxHolder");
+    if (searchBox.parentNode != desiredParent)
+        desiredParent.appendChild(searchBox);
+}
+
+function OnLoadLocationTree()
+{
+    var locationTree = document.getElementById('locationPopup').tree;
+    locationTree.database.AddDataSource(accountManagerDataSource);
+    locationTree.database.AddDataSource(folderDataSource);
+    locationTree.setAttribute("ref", "msgaccounts:/");
+    var toolbar = document.getElementById("msgLocationToolbar");
+    toolbar.addEventListener("DOMAttrModified", OnLocationToolbarAttrModified, false);
+    OnLocationToolbarAttrModified({attrName:"collapsed"});
+}
+
+function OnLocationTreeSelect(menulist)
+{
+    SelectFolder(menulist.getAttribute('uri'));
+}
+
+function UpdateLocationBar(resource)
+{
+    var tree = GetFolderTree();
+    var folders = document.getElementById('locationFolders');
+    var icon = document.getElementById('locationIcon');
+    var names = ['BiffState', 'NewMessages', 'HasUnreadMessages',
+        'SpecialFolder', 'IsServer', 'IsSecure', 'ServerType', 'NoSelect'];
+    var label = GetFolderAttribute(tree, resource, 'FolderTreeName');
+    folders.setAttribute("label", label);
+    for (var i in names) {
+        var name = names[i];
+        var value = GetFolderAttribute(tree, resource, name);
+        folders.setAttribute(name, value);
+        icon.setAttribute(name, value);
+    }
+    folders.setAttribute('uri', resource.Value);
+}
+
+function goToggleLocationToolbar(toggle)
+{
+    // XXX hidden doesn't work, use collapsed instead
+    var menu = document.getElementById("menu_showLocationToolbar");
+    var toolbar = document.getElementById("msgLocationToolbar");
+    var checked = toolbar.collapsed;
+    menu.setAttribute('checked', checked);
+    toolbar.setAttribute('collapsed', !checked);
 }
 
 function GetFolderDatasource()
@@ -1071,20 +1136,12 @@ function GetTotalCountElement()
 
 function IsMessagePaneCollapsed()
 {
-  var messagePane = GetMessagePane();
-  try {
-    return (messagePane.getAttribute("collapsed") == "true");
-  }
-  catch (ex) {
-    return false;
-  }
+  return GetMessagePane().collapsed;
 }
 
 function IsFolderPaneCollapsed()
 {
-  var folderPaneBox = GetFolderTree().parentNode;
-  return folderPaneBox.getAttribute("collapsed") == "true"
-    || folderPaneBox.getAttribute("hidden") == "true";
+  return GetFolderTree().parentNode.collapsed;
 }
 
 function FindMessenger()
@@ -1113,13 +1170,12 @@ function ClearMessagePane()
   {
     gHaveLoadedMessage = false;
     gCurrentDisplayedMessage = null;
-    if (GetMessagePaneFrame().location != "about:blank")
-        GetMessagePaneFrame().location = "about:blank";
+    if (GetMessagePaneFrame().location.href != "about:blank")
+        GetMessagePaneFrame().location.href = "about:blank";
+
     // hide the message header view AND the message pane...
     HideMessageHeaderPane();
-
-    // hide the junk bar
-    SetUpJunkBar(null);
+    gMessageNotificationBar.clearMsgNotifications();
   }
 }
 
@@ -1156,7 +1212,7 @@ function ChangeSelectionWithoutContentLoad(event, tree)
         if(tree.id == "threadTree")
           gThreadPaneCurrentSelectedIndex = row;
     }
-    event.preventBubble();
+    event.stopPropagation();
 }
 
 function TreeOnMouseDown(event)
@@ -1187,20 +1243,20 @@ function FolderPaneOnClick(event)
     if (row.value == -1) {
       if (event.originalTarget.localName == "treecol")
         // clicking on the name column in the folder pane should not sort
-        event.preventBubble();
+        event.stopPropagation();
       return;
     }
 
     if (elt.value == "twisty")
     {
-        var folderResource = GetFolderResource(folderTree, row.value);
-        var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-
         if (!(folderTree.treeBoxObject.view.isContainerOpen(row.value)))
         {
+            var folderResource = GetFolderResource(folderTree, row.value);
+
             var isServer = GetFolderAttribute(folderTree, folderResource, "IsServer");
             if (isServer == "true")
             {
+                var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
                 var server = msgFolder.server;
                 server.performExpand(msgWindow);
             }
@@ -1217,7 +1273,7 @@ function FolderPaneOnClick(event)
     }
     else if ((event.originalTarget.localName == "slider") ||
              (event.originalTarget.localName == "scrollbarbutton")) {
-      event.preventBubble();
+      event.stopPropagation();
     }
     else if (event.detail == 2) {
       FolderPaneDoubleClick(row.value, event);
@@ -1229,18 +1285,18 @@ function FolderPaneDoubleClick(folderIndex, event)
 {
     var folderTree = GetFolderTree();
     var folderResource = GetFolderResource(folderTree, folderIndex);
-    var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-    var isServer = GetFolderAttribute(folderTree, folderResource, "IsServer");
 
+    var isServer = GetFolderAttribute(folderTree, folderResource, "IsServer");
     if (isServer == "true")
     {
       if (!(folderTree.treeBoxObject.view.isContainerOpen(folderIndex)))
       {
+        var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
         var server = msgFolder.server;
         server.performExpand(msgWindow);
       }
     }
-    else 
+    else if (!pref.getBoolPref("mailnews.reuse_thread_window2"))
     {
       // Open a new msg window only if we are double clicking on 
       // folders or newsgroups.
@@ -1249,7 +1305,7 @@ function FolderPaneDoubleClick(folderIndex, event)
       // double clicking should not toggle the open / close state of the
       // folder.  this will happen if we don't prevent the event from
       // bubbling to the default handler in tree.xml
-      event.preventBubble();
+      event.stopPropagation();
     }
 }
 
@@ -1301,7 +1357,6 @@ function GetSelectedMsgFolders()
           var folderResource = GetFolderResource(folderTree, j); 
           if (folderResource.Value != "http://home.netscape.com/NC-rdf#PageTitleFakeAccount") {
             var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-
             if(msgFolder)
               folderArray[k++] = msgFolder;
           }
@@ -1452,7 +1507,7 @@ function SelectFolder(folderUri)
 
 function SelectMessage(messageUri)
 {
-  var msgHdr = messenger.messageServiceFromURI(messageUri).messageURIToMsgHdr(messageUri);
+  var msgHdr = messenger.msgHdrFromURI(messageUri);
   if (msgHdr)
     gDBView.selectMsgByKey(msgHdr.messageKey);
 }
@@ -1509,3 +1564,31 @@ function GetFolderAttribute(tree, source, attribute)
     return target;
 }
 
+// Some of the per account junk mail settings have been
+// converted to global prefs. Let's try to migrate some
+// of those settings from the default account.
+function MigrateJunkMailSettings()
+{
+  var junkMailSettingsVersion = pref.getIntPref("mail.spam.version");
+  if (!junkMailSettingsVersion)
+  {
+    // get the default account, check to see if we have values for our 
+    // globally migrated prefs.
+    var defaultAccount = accountManager.defaultAccount;
+    if (defaultAccount && defaultAccount.incomingServer)
+    {
+      // we only care about
+      var prefix = "mail.server." + defaultAccount.incomingServer.key + ".";
+      if (pref.prefHasUserValue(prefix + "manualMark"))
+        pref.setBoolPref("mail.spam.manualMark", pref.getBoolPref(prefix + "manualMark"));
+      if (pref.prefHasUserValue(prefix + "manualMarkMode"))
+        pref.setIntPref("mail.spam.manualMarkMode", pref.getIntPref(prefix + "manualMarkMode"));
+      if (pref.prefHasUserValue(prefix + "spamLoggingEnabled"))
+        pref.setBoolPref("mail.spam.logging.enabled", pref.getBoolPref(prefix + "spamLoggingEnabled"));
+      if (pref.prefHasUserValue(prefix + "markAsReadOnSpam"))
+        pref.setBoolPref("mail.spam.markAsReadOnSpam", pref.getBoolPref(prefix + "markAsReadOnSpam"));
+    }  
+    // bump the version so we don't bother doing this again.
+    pref.setIntPref("mail.spam.version", 1);
+  }
+}
