@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -22,6 +22,7 @@
  * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
  *   Seth Spitzer <sspitzer@netscape.com>
+ *   Howard Chu <hyc@symas.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -49,7 +50,6 @@
 #include "nsMsgLocalSearch.h"
 #include "nsMsgSearchTerm.h"
 #include "nsXPIDLString.h"
-#include "nsMsgSearchScopeTerm.h"
 #include "nsIMsgAccountManager.h"
 #include "nsIMsgIncomingServer.h"
 #include "nsMsgSearchValue.h"
@@ -59,7 +59,8 @@
 #include "nsIImportService.h"
 #include "nsISupportsObsolete.h"
 #include "nsIOutputStream.h"
-#include "nsEscape.h"
+#include "nsIStringBundle.h"
+#include "nsDateTimeFormatCID.h"
 
 static const char *kImapPrefix = "//imap:";
 static const char *kWhitespace = "\b\t\r\n ";
@@ -116,7 +117,8 @@ NS_IMETHODIMP
 nsMsgRuleAction::SetTargetFolderUri(const char *aUri)
 {
   NS_ENSURE_ARG_POINTER(aUri);
-  NS_ENSURE_TRUE(m_type == nsMsgFilterAction::MoveToFolder,
+  NS_ENSURE_TRUE(m_type == nsMsgFilterAction::MoveToFolder ||
+                 m_type == nsMsgFilterAction::CopyToFolder,
                  NS_ERROR_ILLEGAL_VALUE);
   m_folderUri = aUri;
   return NS_OK;
@@ -126,7 +128,8 @@ NS_IMETHODIMP
 nsMsgRuleAction::GetTargetFolderUri(char** aResult)
 {
   NS_ENSURE_ARG_POINTER(aResult);
-  NS_ENSURE_TRUE(m_type == nsMsgFilterAction::MoveToFolder,
+  NS_ENSURE_TRUE(m_type == nsMsgFilterAction::MoveToFolder ||
+                 m_type == nsMsgFilterAction::CopyToFolder,
                  NS_ERROR_ILLEGAL_VALUE);
   *aResult = ToNewCString(m_folderUri);
   return NS_OK;
@@ -150,10 +153,26 @@ nsMsgRuleAction::GetJunkScore(PRInt32 *aResult)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgRuleAction::SetStrValue(const char *aStrValue)
+{
+  m_strValue = aStrValue;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgRuleAction::GetStrValue(char **aStrValue)
+{
+  NS_ENSURE_ARG_POINTER(aStrValue);
+  *aStrValue = ToNewCString(m_strValue);
+  return NS_OK;
+}
+
 nsMsgFilter::nsMsgFilter():
     m_temporary(PR_FALSE),
     m_unparseable(PR_FALSE),
-    m_filterList(nsnull)
+    m_filterList(nsnull),
+    m_expressionTree(nsnull)
 {
   NS_NewISupportsArray(getter_AddRefs(m_termList));
   NS_NewISupportsArray(getter_AddRefs(m_actionList));
@@ -163,6 +182,7 @@ nsMsgFilter::nsMsgFilter():
 
 nsMsgFilter::~nsMsgFilter()
 {
+  delete m_expressionTree;
 }
 
 NS_IMPL_ISUPPORTS1(nsMsgFilter, nsIMsgFilter)
@@ -226,7 +246,8 @@ NS_IMETHODIMP nsMsgFilter::AddTerm(
 NS_IMETHODIMP nsMsgFilter::AppendTerm(nsIMsgSearchTerm * aTerm)
 {
     NS_ENSURE_TRUE(aTerm, NS_ERROR_NULL_POINTER);
-    
+    // invalidate expression tree if we're changing the terms
+    delete m_expressionTree;
     return m_termList->AppendElement(NS_STATIC_CAST(nsISupports*,aTerm));
 }
 
@@ -260,6 +281,7 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
   PRUint32 numActions;
   nsresult err = m_actionList->Count(&numActions);
   NS_ENSURE_SUCCESS(err, err);
+  PRBool insertedFinalAction = PR_FALSE;
   PRUint32 front = 0;
 
   for (PRUint32 index =0; index < numActions; index++)
@@ -271,8 +293,32 @@ nsMsgFilter::GetSortedActionList(nsISupportsArray *actionList)
  
     nsMsgRuleActionType actionType;
     action->GetType(&actionType);
-    if (actionType == nsMsgFilterAction::MoveToFolder)  //we always want MoveToFolder action to be last
-      actionList->AppendElement(action);
+    
+    //we always want MoveToFolder action to be last (or delete to trash)
+    if (actionType == nsMsgFilterAction::MoveToFolder || actionType == nsMsgFilterAction::Delete)
+    {
+      err = actionList->AppendElement(action);
+      NS_ENSURE_SUCCESS(err, err);
+      insertedFinalAction = PR_TRUE;
+    }
+    // Copy is always last, except for move/delete
+    else if (actionType == nsMsgFilterAction::CopyToFolder)
+    {
+      if (!insertedFinalAction)
+      {
+        err = actionList->AppendElement(action);
+        NS_ENSURE_SUCCESS(err, err);
+      }
+      else
+      {
+        // If we already have a move/delete action in place, we want to
+        // place ourselves just before that final action.
+        PRUint32 count;
+        actionList->Count(&count);
+        err = actionList->InsertElementAt(action, count - 2);
+        NS_ENSURE_SUCCESS(err, err);
+      }
+    }
     else
     {
       actionList->InsertElementAt(action,front);
@@ -349,6 +395,7 @@ NS_IMETHODIMP nsMsgFilter::GetSearchTerms(nsISupportsArray **aResult)
 
 NS_IMETHODIMP nsMsgFilter::SetSearchTerms(nsISupportsArray *aSearchList)
 {
+    delete m_expressionTree;
     m_termList = aSearchList;
     return NS_OK;
 }
@@ -373,62 +420,101 @@ NS_IMETHODIMP nsMsgFilter::GetScope(nsIMsgSearchScopeTerm **aResult)
 
 NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBHdr *aMsgHdr)
 {
+    NS_ENSURE_TRUE(m_filterList, NS_OK);
     nsCOMPtr <nsIOutputStream> logStream;
     nsresult rv = m_filterList->GetLogStream(getter_AddRefs(logStream));
     NS_ENSURE_SUCCESS(rv,rv);
   
     PRTime date;
-    char dateStr[40];	/* 30 probably not enough */
     nsMsgRuleActionType actionType;
     
-    nsXPIDLCString author;
-    nsXPIDLCString subject;
+    nsXPIDLString authorValue;
+    nsXPIDLString subjectValue;
     nsXPIDLString filterName;
+    nsXPIDLString dateValue;
 
     GetFilterName(getter_Copies(filterName));
     aFilterAction->GetType(&actionType);
-    rv = aMsgHdr->GetDate(&date);
+    (void)aMsgHdr->GetDate(&date);
     PRExplodedTime exploded;
     PR_ExplodeTime(date, PR_LocalTimeParameters, &exploded);
-    // XXX Temporary until logging is fully localized
-    PR_FormatTimeUSEnglish(dateStr, sizeof(dateStr), "%Y-%m-%d %H:%M:%S", &exploded);
 
-    aMsgHdr->GetAuthor(getter_Copies(author));
-    aMsgHdr->GetSubject(getter_Copies(subject));
+    if (!mDateFormatter)
+    {
+      mDateFormatter = do_CreateInstance(NS_DATETIMEFORMAT_CONTRACTID, &rv);
+      NS_ENSURE_SUCCESS(rv, rv);
+      if (!mDateFormatter)
+      {
+        return NS_ERROR_FAILURE;
+      }
+    }
+    mDateFormatter->FormatPRExplodedTime(nsnull, kDateFormatShort,
+                                         kTimeFormatSeconds, &exploded, 
+                                         dateValue);
+  
+    (void)aMsgHdr->GetMime2DecodedAuthor(getter_Copies(authorValue));
+    (void)aMsgHdr->GetMime2DecodedSubject(getter_Copies(subjectValue));
 
     nsCString buffer;
     // this is big enough to hold a log entry.  
     // do this so we avoid growing and copying as we append to the log.
     buffer.SetCapacity(512);  
     
-    buffer = "Applied filter \"";
-    AppendUTF16toUTF8(filterName, buffer);
-    buffer +=  "\" to message from ";
-    buffer +=  (const char*)author;
-    buffer +=  " - ";
-    buffer +=  (const char *)subject;
-    buffer +=  " at ";
-    buffer +=  dateStr;
+    nsCOMPtr<nsIStringBundleService> bundleService =
+      do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    nsCOMPtr<nsIStringBundle> bundle;
+    rv = bundleService->CreateBundle("chrome://messenger/locale/filter.properties",
+      getter_AddRefs(bundle));
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    const PRUnichar *filterLogDetectFormatStrings[4] = { filterName.get(), authorValue.get(), subjectValue.get(), dateValue.get() };
+    nsXPIDLString filterLogDetectStr;
+    rv = bundle->FormatStringFromName(
+      NS_LITERAL_STRING("filterLogDetectStr").get(),
+      filterLogDetectFormatStrings, 4,
+      getter_Copies(filterLogDetectStr));
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    buffer += NS_ConvertUTF16toUTF8(filterLogDetectStr);
     buffer +=  "\n";
-    const char *actionStr = GetActionStr(actionType);
-    buffer +=  "Action = ";
-    buffer +=  actionStr;
-    buffer +=  " ";
         
-    if (actionType == nsMsgFilterAction::MoveToFolder) {
+    if (actionType == nsMsgFilterAction::MoveToFolder ||
+        actionType == nsMsgFilterAction::CopyToFolder)
+    {
       nsXPIDLCString actionFolderUri;
       aFilterAction->GetTargetFolderUri(getter_Copies(actionFolderUri));
-      buffer += actionFolderUri.get();
-    } 
+      NS_ConvertASCIItoUTF16 actionFolderUriValue(actionFolderUri);
          
-    buffer += "\n";
-    if (actionType == nsMsgFilterAction::MoveToFolder) {
       nsXPIDLCString msgId;
       aMsgHdr->GetMessageId(getter_Copies(msgId));
-      buffer += " id = ";
-      buffer += (const char*)msgId;
-      buffer += "\n";
+      NS_ConvertASCIItoUTF16 msgIdValue(msgId);
+
+      const PRUnichar *logMoveFormatStrings[2] = { msgIdValue.get(), actionFolderUriValue.get() };
+      nsXPIDLString logMoveStr;
+      rv = bundle->FormatStringFromName(
+        (actionType == nsMsgFilterAction::MoveToFolder) ?
+          NS_LITERAL_STRING("logMoveStr").get() :
+          NS_LITERAL_STRING("logCopyStr").get(),
+        logMoveFormatStrings, 2,
+        getter_Copies(logMoveStr));
+      NS_ENSURE_SUCCESS(rv, rv);
+      
+      buffer += NS_ConvertUTF16toUTF8(logMoveStr);
     }
+    else 
+    {
+      nsXPIDLString actionValue;
+      nsAutoString filterActionID;
+      filterActionID = NS_LITERAL_STRING("filterAction");
+      filterActionID.AppendInt(actionType);
+      rv = bundle->GetStringFromName(filterActionID.get(), getter_Copies(actionValue));
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      buffer += NS_ConvertUTF16toUTF8(actionValue);
+    }
+    buffer += "\n";
     
     PRUint32 writeCount;
 
@@ -455,19 +541,17 @@ NS_IMETHODIMP nsMsgFilter::LogRuleHit(nsIMsgRuleAction *aFilterAction, nsIMsgDBH
     return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgFilter::MatchHdr(nsIMsgDBHdr	*msgHdr, nsIMsgFolder *folder, nsIMsgDatabase *db, 
-									const char *headers, PRUint32 headersSize, PRBool *pResult)
+NS_IMETHODIMP 
+nsMsgFilter::MatchHdr(nsIMsgDBHdr *msgHdr, nsIMsgFolder *folder, 
+                      nsIMsgDatabase *db, const char *headers, 
+                      PRUint32 headersSize, PRBool *pResult)
 {
   NS_ENSURE_ARG_POINTER(folder);
   // use offlineMail because
-  nsMsgSearchScopeTerm* scope = new nsMsgSearchScopeTerm(nsnull, nsMsgSearchScope::offlineMail, folder);
-  if (!scope) return NS_ERROR_OUT_OF_MEMORY;
-  
   nsXPIDLCString folderCharset;
   folder->GetCharset(getter_Copies(folderCharset));
   nsresult rv = nsMsgSearchOfflineMail::MatchTermsForFilter(msgHdr, m_termList,
-                  folderCharset.get(),  scope,  db,  headers,  headersSize, pResult);
-  delete scope;
+                  folderCharset.get(),  m_scope,  db,  headers,  headersSize, &m_expressionTree, pResult);
   return rv;
 }
 
@@ -491,7 +575,7 @@ void nsMsgFilter::SetFilterScript(nsCString *fileName)
   m_scriptFileName = *fileName;
 }
 
-nsresult nsMsgFilter::ConvertMoveToFolderValue(nsIMsgRuleAction *filterAction, nsCString &moveValue)
+nsresult nsMsgFilter::ConvertMoveOrCopyToFolderValue(nsIMsgRuleAction *filterAction, nsCString &moveValue)
 {
   NS_ENSURE_ARG_POINTER(filterAction);
   PRInt16 filterVersion = kFileVersion;
@@ -651,6 +735,7 @@ nsresult nsMsgFilter::SaveRule(nsIOFileStream *aStream)
     switch(actionType)
     {
       case nsMsgFilterAction::MoveToFolder:
+      case nsMsgFilterAction::CopyToFolder:
       {
         nsXPIDLCString imapTargetString;
         action->GetTargetFolderUri(getter_Copies(imapTargetString));
@@ -681,6 +766,17 @@ nsresult nsMsgFilter::SaveRule(nsIOFileStream *aStream)
         action->GetJunkScore(&junkScore);
         err = filterList->WriteIntAttr(nsIMsgFilterList::attribActionValue, junkScore, aStream);
       }
+      break;
+      case nsMsgFilterAction::AddTag:
+      case nsMsgFilterAction::Reply:
+      case nsMsgFilterAction::Forward:
+      {
+        nsXPIDLCString strValue;
+        action->GetStrValue(getter_Copies(strValue));
+        // strValue is e-mail address
+        err = filterList->WriteStrAttr(nsIMsgFilterList::attribActionValue, strValue.get(), aStream);
+      }
+      break;
       default:
         break;
     }
@@ -691,33 +787,40 @@ nsresult nsMsgFilter::SaveRule(nsIOFileStream *aStream)
   PRUint32 count;
   m_termList->Count(&count);
   for (searchIndex = 0; searchIndex < count && NS_SUCCEEDED(err);
-  searchIndex++)
+        searchIndex++)
   {
     nsCAutoString	stream;
-    
+  
     nsCOMPtr<nsIMsgSearchTerm> term;
     m_termList->QueryElementAt(searchIndex, NS_GET_IID(nsIMsgSearchTerm),
       (void **)getter_AddRefs(term));
     if (!term)
       continue;
-    
+  
     if (condition.Length() > 1)
       condition += ' ';
-    
+  
     PRBool booleanAnd;
+    PRBool matchAll;
     term->GetBooleanAnd(&booleanAnd);
-    if (booleanAnd)
+    term->GetMatchAll(&matchAll);
+    if (matchAll)
+    {
+      condition += "ALL";
+      break;
+    }
+    else if (booleanAnd)
       condition += "AND (";
     else
       condition += "OR (";
-    
+  
     nsresult searchError = term->GetTermAsString(stream);
     if (NS_FAILED(searchError))
     {
       err = searchError;
       break;
     }
-    
+  
     condition += stream;
     condition += ')';
   }
@@ -738,6 +841,7 @@ struct RuleActionsTableEntry
 static struct RuleActionsTableEntry ruleActionsTable[] =
 {
   { nsMsgFilterAction::MoveToFolder,    nsMsgFilterType::Inbox, 0,  "Move to folder"},
+  { nsMsgFilterAction::CopyToFolder,    nsMsgFilterType::Inbox, 0,  "Copy to folder"},
   { nsMsgFilterAction::ChangePriority,  nsMsgFilterType::Inbox, 0,  "Change priority"},
   { nsMsgFilterAction::Delete,          nsMsgFilterType::All,   0,  "Delete"},
   { nsMsgFilterAction::MarkRead,        nsMsgFilterType::All,   0,  "Mark read"},
@@ -752,6 +856,7 @@ static struct RuleActionsTableEntry ruleActionsTable[] =
   { nsMsgFilterAction::LeaveOnPop3Server, nsMsgFilterType::Inbox,   0, "Leave on Pop3 server"},
   { nsMsgFilterAction::JunkScore, nsMsgFilterType::All,   0, "JunkScore"},
   { nsMsgFilterAction::FetchBodyFromPop3Server, nsMsgFilterType::Inbox,   0, "Fetch body from Pop3Server"},
+  { nsMsgFilterAction::AddTag,          nsMsgFilterType::All,   0,  "AddTag"},
 };
 
 const char *nsMsgFilter::GetActionStr(nsMsgRuleActionType action)
@@ -806,7 +911,7 @@ nsMsgFilter::GetVersion()
 void nsMsgFilter::Dump()
 {
   nsCAutoString s;
-  CopyUCS2toASCII(m_filterName, s);
+  LossyCopyUTF16toASCII(m_filterName, s);
   printf("filter %s type = %c desc = %s\n", s.get(), m_type + '0', m_description.get());
 }
 #endif
