@@ -53,6 +53,7 @@
 #include "nsIImportService.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgFilterService.h"
+#include "nsMsgSearchScopeTerm.h"
 #include "nsISupportsObsolete.h"
 #include "nsNetUtil.h"
 
@@ -73,6 +74,7 @@ nsMsgFilterList::nsMsgFilterList() :
 
   m_loggingEnabled = PR_FALSE;
   m_startWritingToBuffer = PR_FALSE; 
+  m_temporaryList = PR_FALSE;
   m_curFilter = nsnull;
   m_arbitraryHeaders.SetLength(0);
 }
@@ -121,14 +123,14 @@ NS_IMETHODIMP nsMsgFilterList::SaveToFile(nsIOFileStream *stream)
 
 NS_IMETHODIMP nsMsgFilterList::EnsureLogFile()
 {
-  nsCOMPtr <nsIFileSpec> file;
+  nsCOMPtr <nsILocalFile> file;
   nsresult rv = GetLogFileSpec(getter_AddRefs(file));
   NS_ENSURE_SUCCESS(rv,rv);
 
   PRBool exists;
   rv = file->Exists(&exists);
   if (NS_SUCCEEDED(rv) && !exists) {
-    rv = file->Touch();
+    rv = file->Create(nsIFile::NORMAL_FILE_TYPE, 0644);
     NS_ENSURE_SUCCESS(rv,rv);
   }
   return NS_OK;
@@ -140,11 +142,12 @@ nsresult nsMsgFilterList::TruncateLog()
   nsresult rv = SetLogStream(nsnull);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCOMPtr <nsIFileSpec> file;
+  nsCOMPtr <nsILocalFile> file;
   rv = GetLogFileSpec(getter_AddRefs(file));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  rv = file->Truncate(0);
+  file->Remove(PR_FALSE);
+  rv = file->Create(nsIFile::NORMAL_FILE_TYPE, 0644);
   NS_ENSURE_SUCCESS(rv,rv);
   return rv;
 }
@@ -164,9 +167,9 @@ NS_IMETHODIMP nsMsgFilterList::ClearLog()
 }
 
 nsresult 
-nsMsgFilterList::GetLogFileSpec(nsIFileSpec **aFileSpec)
+nsMsgFilterList::GetLogFileSpec(nsILocalFile **aFile)
 {
-  NS_ENSURE_ARG_POINTER(aFileSpec);
+  NS_ENSURE_ARG_POINTER(aFile);
 
   // XXX todo
   // the path to the log file won't change
@@ -199,31 +202,41 @@ nsMsgFilterList::GetLogFileSpec(nsIFileSpec **aFileSpec)
     rv = m_folder->GetPath(getter_AddRefs(thisFolder));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr <nsIFileSpec> filterLogFile = do_CreateInstance(NS_FILESPEC_CONTRACTID, &rv);
+    nsFileSpec spec;
+    rv = thisFolder->GetFileSpec(&spec);
     NS_ENSURE_SUCCESS(rv, rv);
-    
-    rv = filterLogFile->FromFileSpec(thisFolder);
+
+    nsCOMPtr<nsILocalFile> filterLogFile;
+    rv = NS_FileSpecToIFile(&spec, getter_AddRefs(filterLogFile));
     NS_ENSURE_SUCCESS(rv, rv);
-    
+
     // NOTE:
     // we don't we need to call NS_MsgHashIfNecessary()
     // it's already been hashed, if necessary
-    nsXPIDLCString filterLogName;
-    rv = filterLogFile->GetLeafName(getter_Copies(filterLogName));
-    NS_ENSURE_SUCCESS(rv,rv);
-    
-    filterLogName.Append(".htm");
-    
-    rv = filterLogFile->SetLeafName(filterLogName.get());
+    nsAutoString filterLogName;
+    rv = filterLogFile->GetLeafName(filterLogName);
     NS_ENSURE_SUCCESS(rv,rv);
 
-    NS_IF_ADDREF(*aFileSpec = filterLogFile);
+    filterLogName.Append(NS_LITERAL_STRING(".htm"));
+
+    rv = filterLogFile->SetLeafName(filterLogName);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    NS_IF_ADDREF(*aFile = filterLogFile);
   }
   else {
-    rv = server->GetLocalPath(aFileSpec);
+    nsCOMPtr<nsIFileSpec> fileSpec;
+    rv = server->GetLocalPath(getter_AddRefs(fileSpec));
     NS_ENSURE_SUCCESS(rv,rv);
-    
-    rv = (*aFileSpec)->AppendRelativeUnixPath("filterlog.html");
+
+    nsFileSpec spec;
+    rv = fileSpec->GetFileSpec(&spec);
+    NS_ENSURE_SUCCESS(rv,rv);
+ 
+    rv = NS_FileSpecToIFile(&spec, aFile);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = (*aFile)->AppendNative(NS_LITERAL_CSTRING("filterlog.html"));
     NS_ENSURE_SUCCESS(rv,rv);
   }
   return NS_OK;
@@ -234,13 +247,17 @@ nsMsgFilterList::GetLogURL(char **aLogURL)
 {
   NS_ENSURE_ARG_POINTER(aLogURL);
 
-  nsCOMPtr <nsIFileSpec> file;
+  nsCOMPtr <nsILocalFile> file;
   nsresult rv = GetLogFileSpec(getter_AddRefs(file));
   NS_ENSURE_SUCCESS(rv,rv);
   
-  rv = file->GetURLString(aLogURL);
+  nsCString url;
+  rv = NS_GetURLSpecFromFile(file, url);
   NS_ENSURE_SUCCESS(rv,rv);
-  return NS_OK;
+
+  *aLogURL = ToNewCString(url);
+
+  return *aLogURL ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
 }
 
 NS_IMETHODIMP
@@ -257,6 +274,9 @@ nsMsgFilterList::SetLogStream(nsIOutputStream *aLogStream)
   return NS_OK;
 }
 
+#define LOG_HEADER "<head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"></head>"
+#define LOG_HEADER_LEN (strlen(LOG_HEADER))
+
 NS_IMETHODIMP
 nsMsgFilterList::GetLogStream(nsIOutputStream **aLogStream)
 {
@@ -265,18 +285,8 @@ nsMsgFilterList::GetLogStream(nsIOutputStream **aLogStream)
   nsresult rv;
 
   if (!m_logStream) {
-    nsCOMPtr <nsIFileSpec> file;
-    rv = GetLogFileSpec(getter_AddRefs(file));
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsXPIDLCString nativePath;
-    rv = file->GetNativePath(getter_Copies(nativePath));
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsCOMPtr <nsILocalFile> logFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = logFile->InitWithNativePath(nsDependentCString(nativePath));
+    nsCOMPtr <nsILocalFile> logFile;
+    rv = GetLogFileSpec(getter_AddRefs(logFile));
     NS_ENSURE_SUCCESS(rv,rv);
 
     // append to the end of the log file
@@ -288,6 +298,22 @@ nsMsgFilterList::GetLogStream(nsIOutputStream **aLogStream)
 
     if (!m_logStream)
       return NS_ERROR_FAILURE;
+
+    PRInt64 fileSize;
+    rv = logFile->GetFileSize(&fileSize);
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    PRUint32 fileLen;;
+    LL_L2UI(fileLen, fileSize);
+    // write the header at the start
+    if (fileLen == 0)
+    {
+      PRUint32 writeCount;
+      
+      rv = m_logStream->Write(LOG_HEADER, LOG_HEADER_LEN, &writeCount);
+      NS_ENSURE_SUCCESS(rv, rv);
+      NS_ASSERTION(writeCount == LOG_HEADER_LEN, "failed to write out log header");
+    }
   }
  
   NS_ADDREF(*aLogStream = m_logStream);
@@ -302,13 +328,20 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
                                    const char *headers,
                                    PRUint32 headersSize,
                                    nsIMsgFilterHitNotify *listener,
-                                   nsIMsgWindow *msgWindow)
+                                   nsIMsgWindow *msgWindow,
+                                   nsILocalFile *aMessageFile)
 {
   nsCOMPtr <nsIMsgFilter>	filter;
   PRUint32		filterCount = 0;
   nsresult		rv = GetFilterCount(&filterCount);
   NS_ENSURE_SUCCESS(rv,rv);
   
+  nsMsgSearchScopeTerm* scope = new nsMsgSearchScopeTerm(nsnull, nsMsgSearchScope::offlineMail, folder);
+  scope->AddRef();
+  if (!scope) return NS_ERROR_OUT_OF_MEMORY;
+  if (aMessageFile)
+    scope->m_localFile = aMessageFile;
+  
   for (PRUint32 filterIndex = 0; filterIndex < filterCount; filterIndex++)
   {
     if (NS_SUCCEEDED(GetFilterAt(filterIndex, getter_AddRefs(filter))))
@@ -326,7 +359,9 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
         nsresult matchTermStatus = NS_OK;
         PRBool result;
         
+        filter->SetScope(scope);
         matchTermStatus = filter->MatchHdr(msgHdr, folder, db, headers, headersSize, &result);
+        filter->SetScope(nsnull);
         if (NS_SUCCEEDED(matchTermStatus) && result && listener)
         {
           PRBool applyMore = PR_TRUE;
@@ -338,6 +373,7 @@ nsMsgFilterList::ApplyFiltersToHdr(nsMsgFilterTypeType filterType,
       }
     }
   }
+  scope->Release();
   return rv;
 }
 
@@ -517,7 +553,7 @@ nsresult nsMsgFilterList::LoadValue(nsCString &value, nsIOFileStream *aStream)
 
 nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
 {
-	nsresult	err = NS_OK;
+  nsresult	err = NS_OK;
   nsMsgFilterFileAttribValue attrib;
   nsCOMPtr<nsIImportService> impSvc;
   nsCOMPtr<nsIMsgRuleAction> currentFilterAction;
@@ -527,8 +563,8 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
   {
     nsCAutoString	value;
     PRInt32 intToStringResult;
-
-		char curChar;
+    
+    char curChar;
     curChar = LoadAttrib(attrib, aStream);
     if (curChar == (char) -1)  //reached eof
       break;
@@ -537,28 +573,28 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
       break;
     switch(attrib)
     {
-      case nsIMsgFilterList::attribNone:
-        m_curFilter->SetUnparseable(PR_TRUE);
-        break;
-      case nsIMsgFilterList::attribVersion:
-        m_fileVersion = value.ToInteger(&intToStringResult, 10);
-			  if (intToStringResult != 0)
-        {
-          attrib = nsIMsgFilterList::attribNone;
-          NS_ASSERTION(PR_FALSE, "error parsing filter file version");
-        }
-        if (m_fileVersion == k45Version)
-        {
-          impSvc = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
-          NS_ASSERTION(impSvc, "cannot get importService");
-        }
-        break;
-      case nsIMsgFilterList::attribLogging:
-        m_loggingEnabled = StrToBool(value);
-        m_unparsedFilterBuffer.Truncate(); //we are going to buffer each filter as we read them, make sure no garbage is there
-        m_startWritingToBuffer = PR_TRUE; //filters begin now
-        break;
-		  case nsIMsgFilterList::attribName:  //every filter starts w/ a name
+    case nsIMsgFilterList::attribNone:
+      m_curFilter->SetUnparseable(PR_TRUE);
+      break;
+    case nsIMsgFilterList::attribVersion:
+      m_fileVersion = value.ToInteger(&intToStringResult, 10);
+      if (intToStringResult != 0)
+      {
+        attrib = nsIMsgFilterList::attribNone;
+        NS_ASSERTION(PR_FALSE, "error parsing filter file version");
+      }
+      if (m_fileVersion == k45Version)
+      {
+        impSvc = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
+        NS_ASSERTION(impSvc, "cannot get importService");
+      }
+      break;
+    case nsIMsgFilterList::attribLogging:
+      m_loggingEnabled = StrToBool(value);
+      m_unparsedFilterBuffer.Truncate(); //we are going to buffer each filter as we read them, make sure no garbage is there
+      m_startWritingToBuffer = PR_TRUE; //filters begin now
+      break;
+    case nsIMsgFilterList::attribName:  //every filter starts w/ a name
       {
         if (m_curFilter)
         {
@@ -566,7 +602,7 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
           nsCAutoString nextFilterPart;
           nextFilterPart = Substring(m_unparsedFilterBuffer, nextFilterStartPos, m_unparsedFilterBuffer.Length());
           m_unparsedFilterBuffer.Truncate(nextFilterStartPos);
-        
+          
           PRBool unparseableFilter;
           m_curFilter->GetUnparseable(&unparseableFilter);
           if (unparseableFilter)
@@ -576,13 +612,13 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
           }
           m_unparsedFilterBuffer = nextFilterPart;
         }
-			  nsMsgFilter *filter = new nsMsgFilter;
-			  if (filter == nsnull)
+        nsMsgFilter *filter = new nsMsgFilter;
+        if (filter == nsnull)
         {
-				  err = NS_ERROR_OUT_OF_MEMORY;
-				  break;
+          err = NS_ERROR_OUT_OF_MEMORY;
+          break;
         }
-			  filter->SetFilterList(NS_STATIC_CAST(nsIMsgFilterList*,this));
+        filter->SetFilterList(NS_STATIC_CAST(nsIMsgFilterList*,this));
         if (m_fileVersion == k45Version && impSvc)
         {
           nsAutoString unicodeStr;
@@ -592,97 +628,109 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
         else
         {  
           PRUnichar *unicodeString =
-          nsTextFormatter::smprintf(unicodeFormatter, value.get());
-			    filter->SetFilterName(unicodeString);
+            nsTextFormatter::smprintf(unicodeFormatter, value.get());
+          filter->SetFilterName(unicodeString);
           nsTextFormatter::smprintf_free(unicodeString);
         }
-			  m_curFilter = filter;
-			  m_filters->AppendElement(NS_STATIC_CAST(nsISupports*,filter));
+        m_curFilter = filter;
+        m_filters->AppendElement(NS_STATIC_CAST(nsISupports*,filter));
       }
-			break;
-		  case nsIMsgFilterList::attribEnabled:
-			  if (m_curFilter)
-				  m_curFilter->SetEnabled(StrToBool(value));
-			break;
-		  case nsIMsgFilterList::attribDescription:
-			  if (m_curFilter)
-			   	m_curFilter->SetFilterDesc(value.get());
-			break;
-		  case nsIMsgFilterList::attribType:
-			  if (m_curFilter)
-				   m_curFilter->SetType((nsMsgFilterTypeType) value.ToInteger(&intToStringResult, 10));
-			break;
-		  case nsIMsgFilterList::attribScriptFile:
-			  if (m_curFilter)
-				  m_curFilter->SetFilterScript(&value);
-			break;
-		  case nsIMsgFilterList::attribAction:
-        if (m_curFilter)
-        {
-          nsMsgRuleActionType actionType = nsMsgFilter::GetActionForFilingStr(value);
-          if (actionType == nsMsgFilterAction::None)
-            m_curFilter->SetUnparseable(PR_TRUE);
-          else
-          {
-            err = m_curFilter->CreateAction(getter_AddRefs(currentFilterAction));
-            NS_ENSURE_SUCCESS(err, err);
-            currentFilterAction->SetType(actionType);
-            m_curFilter->AppendAction(currentFilterAction);
-          }
-        }
       break;
-      case nsIMsgFilterList::attribActionValue:
-        if (m_curFilter && currentFilterAction)
+    case nsIMsgFilterList::attribEnabled:
+      if (m_curFilter)
+        m_curFilter->SetEnabled(StrToBool(value));
+      break;
+    case nsIMsgFilterList::attribDescription:
+      if (m_curFilter)
+        m_curFilter->SetFilterDesc(value.get());
+      break;
+    case nsIMsgFilterList::attribType:
+      if (m_curFilter)
+        m_curFilter->SetType((nsMsgFilterTypeType) value.ToInteger(&intToStringResult, 10));
+      break;
+    case nsIMsgFilterList::attribScriptFile:
+      if (m_curFilter)
+        m_curFilter->SetFilterScript(&value);
+      break;
+    case nsIMsgFilterList::attribAction:
+      if (m_curFilter)
+      {
+        nsMsgRuleActionType actionType = nsMsgFilter::GetActionForFilingStr(value);
+        if (actionType == nsMsgFilterAction::None)
+          m_curFilter->SetUnparseable(PR_TRUE);
+        else
         {
-          nsMsgRuleActionType type;
-          currentFilterAction->GetType(&type);
-          if (type == nsMsgFilterAction::MoveToFolder)
-            err = m_curFilter->ConvertMoveToFolderValue(currentFilterAction, value);
-          else if (type == nsMsgFilterAction::ChangePriority)
+          err = m_curFilter->CreateAction(getter_AddRefs(currentFilterAction));
+          NS_ENSURE_SUCCESS(err, err);
+          currentFilterAction->SetType(actionType);
+          m_curFilter->AppendAction(currentFilterAction);
+        }
+      }
+      break;
+    case nsIMsgFilterList::attribActionValue:
+      if (m_curFilter && currentFilterAction)
+      {
+        nsMsgRuleActionType type;
+        currentFilterAction->GetType(&type);
+        if (type == nsMsgFilterAction::MoveToFolder ||
+              type == nsMsgFilterAction::CopyToFolder)
+          err = m_curFilter->ConvertMoveOrCopyToFolderValue(currentFilterAction, value);
+        else if (type == nsMsgFilterAction::ChangePriority)
+        {
+          nsMsgPriorityValue outPriority;
+          nsresult res = NS_MsgGetPriorityFromString(value.get(), &outPriority);
+          if (NS_SUCCEEDED(res))
+            currentFilterAction->SetPriority(outPriority);
+          else
+            NS_ASSERTION(PR_FALSE, "invalid priority in filter file");
+        }
+        else if (type == nsMsgFilterAction::Label)
+        {
+          // upgrade label to corresponding tag/keyword
+          PRInt32 res;
+          PRInt32 labelInt = value.ToInteger(&res, 10);
+          if (res == 0)
           {
-            nsMsgPriorityValue outPriority;
-            nsresult res = NS_MsgGetPriorityFromString(value.get(), &outPriority);
-            if (NS_SUCCEEDED(res))
-              currentFilterAction->SetPriority(outPriority);
-            else
-              NS_ASSERTION(PR_FALSE, "invalid priority in filter file");
-          }
-          else if (type == nsMsgFilterAction::Label)
-          {
-            PRInt32 res;
-            PRInt32 labelInt = value.ToInteger(&res, 10);
-            if (res == 0)
-              currentFilterAction->SetLabel((nsMsgLabelValue) labelInt);
-          }
-          else if (type == nsMsgFilterAction::JunkScore)
-          {
-            PRInt32 res;
-            PRInt32 junkScore = value.ToInteger(&res, 10);
-            if (!res)
-              currentFilterAction->SetJunkScore(junkScore);
+            nsCAutoString keyword("$label");
+            keyword.Append('0' + labelInt);
+            currentFilterAction->SetType(nsMsgFilterAction::AddTag);
+            currentFilterAction->SetStrValue(keyword.get());
           }
         }
-        break;
-		case nsIMsgFilterList::attribCondition:
-            if (m_curFilter)
-            {
-              if ( m_fileVersion == k45Version && impSvc)
-              {
-                nsAutoString unicodeStr;
-                impSvc->SystemStringToUnicode(value.get(), unicodeStr);
-                char *utf8 = ToNewUTF8String(unicodeStr);
-                value.Assign(utf8);
-                nsMemory::Free(utf8);
-              }
-              err = ParseCondition(m_curFilter, value.get());
-              if (err == NS_ERROR_INVALID_ARG)
-                err = m_curFilter->SetUnparseable(PR_TRUE);
-              NS_ENSURE_SUCCESS(err, err);
-            }
-            break;
-		}
-	} while (!aStream->eof());
-
+        else if (type == nsMsgFilterAction::JunkScore)
+        {
+          PRInt32 res;
+          PRInt32 junkScore = value.ToInteger(&res, 10);
+          if (!res)
+            currentFilterAction->SetJunkScore(junkScore);
+        }
+        else if (type == nsMsgFilterAction::Forward || type == nsMsgFilterAction::Reply
+          || type == nsMsgFilterAction::AddTag)
+        {
+          currentFilterAction->SetStrValue(value.get());
+        }
+      }
+      break;
+    case nsIMsgFilterList::attribCondition:
+      if (m_curFilter)
+      {
+        if ( m_fileVersion == k45Version && impSvc)
+        {
+          nsAutoString unicodeStr;
+          impSvc->SystemStringToUnicode(value.get(), unicodeStr);
+          char *utf8 = ToNewUTF8String(unicodeStr);
+          value.Assign(utf8);
+          nsMemory::Free(utf8);
+        }
+        err = ParseCondition(m_curFilter, value.get());
+        if (err == NS_ERROR_INVALID_ARG)
+          err = m_curFilter->SetUnparseable(PR_TRUE);
+        NS_ENSURE_SUCCESS(err, err);
+      }
+      break;
+    }
+  } while (!aStream->eof());
+  
   if (m_curFilter)
   {
     PRBool unparseableFilter;
@@ -693,8 +741,8 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
       m_curFilter->SetEnabled(PR_FALSE);  //disable the filter because we don't know how to apply it
     }
   }
-
-	return err;
+  
+  return err;
 }
 
 // parse condition like "(subject, contains, fred) AND (body, isn't, "foo)")"
@@ -702,11 +750,24 @@ nsresult nsMsgFilterList::LoadTextFilters(nsIOFileStream *aStream)
 // what about values with close parens and quotes? e.g., (body, isn't, "foo")")
 // I guess interior quotes will need to be escaped - ("foo\")")
 // which will get written out as (\"foo\\")\") and read in as ("foo\")"
+// ALL means match all messages.
 NS_IMETHODIMP nsMsgFilterList::ParseCondition(nsIMsgFilter *aFilter, const char *aCondition)
 {
   PRBool	done = PR_FALSE;
   nsresult	err = NS_OK;
   const char *curPtr = aCondition;
+  if (!strcmp(aCondition, "ALL"))
+  {
+    nsMsgSearchTerm *newTerm = new nsMsgSearchTerm;
+
+    if (newTerm) 
+    {
+      newTerm->m_matchAll = PR_TRUE;
+      aFilter->AppendTerm(newTerm);
+    }
+    return (newTerm) ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  }
+
   while (!done)
   {
     // insert code to save the boolean operator if there is one for this search term....
@@ -819,7 +880,7 @@ nsresult
 nsMsgFilterList::WriteWstrAttr(nsMsgFilterFileAttribValue attrib,
                                const PRUnichar *aFilterName, nsIOFileStream *aStream)
 {
-    WriteStrAttr(attrib, NS_ConvertUCS2toUTF8(aFilterName).get(), aStream);
+    WriteStrAttr(attrib, NS_ConvertUTF16toUTF8(aFilterName).get(), aStream);
     return NS_OK;
 }
 
@@ -958,7 +1019,8 @@ nsMsgFilterList::RemoveFilter(nsIMsgFilter *aFilter)
 nsresult nsMsgFilterList::InsertFilterAt(PRUint32 filterIndex, nsIMsgFilter *aFilter)
 {
   nsMsgFilter *filter = NS_STATIC_CAST(nsMsgFilter *, aFilter);
-  filter->SetFilterList(this);
+  if (!m_temporaryList)
+    filter->SetFilterList(this);
   m_filters->InsertElementAt(aFilter, filterIndex);
   return NS_OK;
 }
@@ -1048,7 +1110,8 @@ NS_IMETHODIMP nsMsgFilterList::MatchOrChangeFilterTarget(const char *oldFolderUr
       else
         continue;
 
-      if (actionType == nsMsgFilterAction::MoveToFolder)
+      if (actionType == nsMsgFilterAction::MoveToFolder ||
+          actionType == nsMsgFilterAction::CopyToFolder)
       {
         rv = filterAction->GetTargetFolderUri(getter_Copies(folderUri));
         if (NS_SUCCEEDED(rv) && folderUri)
