@@ -43,7 +43,7 @@
 #include "nsMsgFilterService.h"
 #include "nsFileStream.h"
 #include "nsMsgFilterList.h"
-#include "nsSpecialSystemDirectory.h"
+#include "nsDirectoryServiceDefs.h"
 #include "nsIPrompt.h"
 #include "nsIDocShell.h"
 #include "nsIMsgWindow.h"
@@ -61,6 +61,9 @@
 #include "nsMsgBaseCID.h"
 #include "nsIMsgCopyService.h"
 #include "nsIOutputStream.h"
+#include "nsIMsgComposeService.h"
+#include "nsMsgCompCID.h"
+#include "nsMsgUtils.h"
 
 NS_IMPL_ISUPPORTS1(nsMsgFilterService, nsIMsgFilterService)
 
@@ -137,15 +140,17 @@ NS_IMETHODIMP nsMsgFilterService::CloseFilterList(nsIMsgFilterList *filterList)
 /* save without deleting */
 NS_IMETHODIMP	nsMsgFilterService::SaveFilterList(nsIMsgFilterList *filterList, nsIFileSpec *filterFile)
 {
-	nsresult ret = NS_OK;
+  NS_ENSURE_ARG_POINTER(filterFile);
+  NS_ENSURE_ARG_POINTER(filterList);
+
+  nsresult ret = NS_OK;
   nsCOMPtr <nsIFileSpec> tmpFiltersFile;
   nsCOMPtr <nsIFileSpec> realFiltersFile;
   nsCOMPtr <nsIFileSpec> parentDir;
 
-  nsSpecialSystemDirectory tmpFile(nsSpecialSystemDirectory::OS_TemporaryDirectory);
-  tmpFile += "tmprules.dat";
-
-  ret = NS_NewFileSpecWithSpec(tmpFile, getter_AddRefs(tmpFiltersFile));
+  ret = GetSpecialDirectoryWithFileName(NS_OS_TEMP_DIR,
+                                        "tmprules.dat",
+                                        getter_AddRefs(tmpFiltersFile));
 
   NS_ASSERTION(NS_SUCCEEDED(ret),"writing filters file: failed to append filename");
   if (NS_FAILED(ret)) 
@@ -250,7 +255,7 @@ nsMsgFilterService::GetStringFromBundle(const char *aMsgName, PRUnichar **aResul
   nsCOMPtr <nsIStringBundle> bundle;
   rv = GetFilterStringBundle(getter_AddRefs(bundle));
   if (NS_SUCCEEDED(rv) && bundle)
-    rv=bundle->GetStringFromName(NS_ConvertASCIItoUCS2(aMsgName).get(), aResult);
+    rv=bundle->GetStringFromName(NS_ConvertASCIItoUTF16(aMsgName).get(), aResult);
   return rv;
   
 }
@@ -388,6 +393,8 @@ nsresult nsMsgFilterAfterTheFact::RunNextFilter()
     m_searchSession->UnregisterListener(this);
   m_searchSession = do_CreateInstance(NS_MSGSEARCHSESSION_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
+
+  nsMsgSearchScopeValue searchScope = nsMsgSearchScope::offlineMail;
   PRUint32 termCount;
   searchTerms->Count(&termCount);
   for (PRUint32 termIndex = 0; termIndex < termCount; termIndex++)
@@ -399,13 +406,7 @@ nsresult nsMsgFilterAfterTheFact::RunNextFilter()
     NS_ENSURE_SUCCESS(rv, rv);
   }
   m_searchSession->RegisterListener(this);
-  // get incoming server for folder and get search scope from there.
-  nsCOMPtr <nsIMsgIncomingServer> incomingServer;
-  rv = m_curFolder->GetServer(getter_AddRefs(incomingServer));
-  NS_ENSURE_SUCCESS(rv, rv);
-  nsMsgSearchScopeValue searchScope;
 
-  incomingServer->GetSearchScope(&searchScope);
   rv = m_searchSession->AddScopeTerm(searchScope, m_curFolder);
   NS_ENSURE_SUCCESS(rv, rv);
   // it's possible that this error handling will need to be rearranged when mscott lands the UI for
@@ -508,7 +509,8 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter()
         continue;
       
       nsXPIDLCString actionTargetFolderUri;
-      if (actionType == nsMsgFilterAction::MoveToFolder)
+      if (actionType == nsMsgFilterAction::MoveToFolder ||
+          actionType == nsMsgFilterAction::CopyToFolder)
       {
         filterAction->GetTargetFolderUri(getter_Copies(actionTargetFolderUri));
         if (actionTargetFolderUri.IsEmpty())
@@ -544,8 +546,9 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter()
         applyMoreActions = PR_FALSE;
         break;
       case nsMsgFilterAction::MoveToFolder:
+      case nsMsgFilterAction::CopyToFolder:
       {
-        // if moving to a different file, do it.
+        // if moving or copying to a different file, do it.
         nsXPIDLCString uri;
         rv = m_curFolder->GetURI(getter_Copies(uri));
 
@@ -569,6 +572,8 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter()
           {
             m_curFilter->SetEnabled(PR_FALSE);
             destIFolder->ThrowAlertMsg("filterDisabled",m_msgWindow);
+            // we need to explicitly save the filter file.
+            m_filters->SaveToDefaultFile();
             // In the case of applying multiple filters
             // we might want to remove the filter from the list, but 
             // that's a bit evil since we really don't know that we own
@@ -580,10 +585,11 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter()
           }
           nsCOMPtr<nsIMsgCopyService> copyService = do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
           if (copyService)
-            return copyService->CopyMessages(m_curFolder, m_searchHitHdrs, destIFolder, PR_TRUE, this, m_msgWindow, PR_FALSE);
+            return copyService->CopyMessages(m_curFolder, m_searchHitHdrs, destIFolder, actionType == nsMsgFilterAction::MoveToFolder, this, m_msgWindow, PR_FALSE);
         }
         //we have already moved the hdrs so we can't apply more actions
-        applyMoreActions = PR_FALSE;
+        if (actionType == nsMsgFilterAction::MoveToFolder)
+          applyMoreActions = PR_FALSE;
       }
         
         break;
@@ -639,6 +645,135 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter()
             filterAction->GetLabel(&filterLabel);
             m_curFolder->SetLabelForMessages(m_searchHitHdrs, filterLabel);
         }
+        break;
+      case nsMsgFilterAction::AddTag:
+        {
+            nsXPIDLCString keyword;
+            filterAction->GetStrValue(getter_Copies(keyword));
+            m_curFolder->AddKeywordToMessages(m_searchHitHdrs, keyword.get());
+        }
+        break;
+      case nsMsgFilterAction::JunkScore:
+      {
+        nsCAutoString junkScoreStr;
+        PRInt32 junkScore;
+        filterAction->GetJunkScore(&junkScore);
+        junkScoreStr.AppendInt(junkScore);
+        m_curFolder->SetJunkScoreForMessages(m_searchHitHdrs, junkScoreStr.get());
+        break;
+      }
+      case nsMsgFilterAction::Forward:
+        {
+          nsXPIDLCString forwardTo;
+          filterAction->GetStrValue(getter_Copies(forwardTo));
+          nsCOMPtr <nsIMsgIncomingServer> server;
+          rv = m_curFolder->GetServer(getter_AddRefs(server));
+          NS_ENSURE_SUCCESS(rv, rv);
+          if (!forwardTo.IsEmpty())
+          {
+            nsCOMPtr <nsIMsgComposeService> compService = do_GetService (NS_MSGCOMPOSESERVICE_CONTRACTID) ;
+            if (compService)
+            {
+              for (PRUint32 msgIndex = 0; msgIndex < m_searchHits.GetSize(); msgIndex++)
+              {
+                nsCOMPtr <nsIMsgDBHdr> msgHdr;
+                m_searchHitHdrs->QueryElementAt(msgIndex, NS_GET_IID(nsIMsgDBHdr), getter_AddRefs(msgHdr));
+                if (msgHdr)
+                {
+                  nsAutoString forwardStr;
+                  forwardStr.AssignWithConversion(forwardTo.get());
+                  rv = compService->ForwardMessage(forwardStr, msgHdr, m_msgWindow, server);
+                }
+              }
+            }
+          }
+        }
+        break;
+      case nsMsgFilterAction::Reply:
+        {
+          nsXPIDLCString replyTemplateUri;
+          filterAction->GetStrValue(getter_Copies(replyTemplateUri));
+          nsCOMPtr <nsIMsgIncomingServer> server;
+          rv = m_curFolder->GetServer(getter_AddRefs(server));
+          NS_ENSURE_SUCCESS(rv, rv);
+          if (!replyTemplateUri.IsEmpty())
+          {
+            nsCOMPtr <nsIMsgComposeService> compService = do_GetService (NS_MSGCOMPOSESERVICE_CONTRACTID) ;
+            if (compService)
+            {
+              for (PRUint32 msgIndex = 0; msgIndex < m_searchHits.GetSize(); msgIndex++)
+              {
+                nsCOMPtr <nsIMsgDBHdr> msgHdr;
+                m_searchHitHdrs->QueryElementAt(msgIndex, NS_GET_IID(nsIMsgDBHdr), getter_AddRefs(msgHdr));
+                if (msgHdr)
+                  rv = compService->ReplyWithTemplate(msgHdr, replyTemplateUri, m_msgWindow, server);
+              }
+            }
+          }
+        }
+        break;
+      case nsMsgFilterAction::DeleteFromPop3Server:
+        {
+          nsCOMPtr <nsIMsgLocalMailFolder> localFolder = do_QueryInterface(m_curFolder);
+          if (localFolder)
+          {
+            // This action ignores the deleteMailLeftOnServer preference
+            localFolder->MarkMsgsOnPop3Server(m_searchHitHdrs, POP3_FORCE_DEL);
+
+            nsCOMPtr <nsISupportsArray> partialMsgs;
+            // Delete the partial headers. They're useless now
+            // that the server copy is being deleted.
+            for (PRUint32 msgIndex = 0; msgIndex < m_searchHits.GetSize(); msgIndex++)
+            {
+              nsCOMPtr <nsIMsgDBHdr> msgHdr;
+              m_searchHitHdrs->QueryElementAt(msgIndex, NS_GET_IID(nsIMsgDBHdr), getter_AddRefs(msgHdr));
+              if (msgHdr)
+              {
+                PRUint32 flags;
+                msgHdr->GetFlags(&flags);
+                if (flags & MSG_FLAG_PARTIAL)
+                {
+                  if (!partialMsgs)
+                    partialMsgs = do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv);
+                  NS_ENSURE_SUCCESS(rv, rv);
+                  partialMsgs->AppendElement(msgHdr);
+                }
+              }
+            }
+            if (partialMsgs)
+              m_curFolder->DeleteMessages(partialMsgs, m_msgWindow, PR_TRUE, PR_FALSE, nsnull, PR_FALSE);
+          }
+        }
+        break;
+      case nsMsgFilterAction::FetchBodyFromPop3Server:
+        {
+          nsCOMPtr <nsIMsgLocalMailFolder> localFolder = do_QueryInterface(m_curFolder);
+          if (localFolder)
+          {
+            nsCOMPtr<nsISupportsArray> messages = do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv);
+            NS_ENSURE_SUCCESS(rv, rv);
+            for (PRUint32 msgIndex = 0; msgIndex < m_searchHits.GetSize(); msgIndex++)
+            {
+              nsCOMPtr <nsIMsgDBHdr> msgHdr;
+              m_searchHitHdrs->QueryElementAt(msgIndex, NS_GET_IID(nsIMsgDBHdr), getter_AddRefs(msgHdr));
+              if (msgHdr)
+              {
+                PRUint32 flags = 0;
+                msgHdr->GetFlags(&flags);
+                if (flags & MSG_FLAG_PARTIAL)
+                {
+                  nsCOMPtr<nsISupports> iSupports = do_QueryInterface(msgHdr);
+                  messages->AppendElement(iSupports);
+                }
+              }
+            }
+            PRUint32 msgsToFetch;
+            messages->Count(&msgsToFetch);
+            if (msgsToFetch > 0)
+              m_curFolder->DownloadMessagesForOffline(messages, m_msgWindow);
+          }
+        }
+        break;
       default:
         break;
       }
@@ -650,14 +785,12 @@ nsresult nsMsgFilterAfterTheFact::ApplyFilter()
 NS_IMETHODIMP nsMsgFilterService::GetTempFilterList(nsIMsgFolder *aFolder, nsIMsgFilterList **aFilterList)
 {
   NS_ENSURE_ARG_POINTER(aFilterList);
-  *aFilterList = new nsMsgFilterList;
-  if (*aFilterList)
-  {
-    (*aFilterList)->SetFolder(aFolder);
-    NS_ADDREF(*aFilterList);
-    return NS_OK;
-  }
-  return NS_ERROR_OUT_OF_MEMORY;
+  nsMsgFilterList *filterList = new nsMsgFilterList;
+  NS_ENSURE_TRUE(filterList, NS_ERROR_OUT_OF_MEMORY);
+  NS_ADDREF(*aFilterList = filterList);
+  (*aFilterList)->SetFolder(aFolder);
+  filterList->m_temporaryList = PR_TRUE;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgFilterService::ApplyFiltersToFolders(nsIMsgFilterList *aFilterList, nsISupportsArray *aFolders, nsIMsgWindow *aMsgWindow)
