@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Howard Chu <hyc@symas.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -41,7 +42,9 @@
 #include "nsMsgSearchAdapter.h"
 #include "nsMsgSearchScopeTerm.h"
 #include "nsMsgI18N.h"
-#include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefLocalizedString.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "nsMsgSearchTerm.h"
@@ -78,6 +81,12 @@ const char *nsMsgSearchAdapter::m_kImapAnswered = " ANSWERED ";
 const char *nsMsgSearchAdapter::m_kImapNotSeen = " UNSEEN ";
 const char *nsMsgSearchAdapter::m_kImapNotAnswered = " UNANSWERED ";
 const char *nsMsgSearchAdapter::m_kImapCharset = " CHARSET ";
+const char *nsMsgSearchAdapter::m_kImapSizeSmaller = " SMALLER ";
+const char *nsMsgSearchAdapter::m_kImapSizeLarger = " LARGER ";
+const char *nsMsgSearchAdapter::m_kImapNew = " NEW "; 
+const char *nsMsgSearchAdapter::m_kImapNotNew = " OLD SEEN "; 
+const char *nsMsgSearchAdapter::m_kImapFlagged = " FLAGGED ";
+const char *nsMsgSearchAdapter::m_kImapNotFlagged = " UNFLAGGED ";
 
 #define PREF_CUSTOM_HEADERS "mailnews.customHeaders"
 
@@ -162,7 +171,7 @@ nsMsgSearchAdapter::GetImapCharsetParam(const PRUnichar *destCharset)
 
 	// Specify a character set unless we happen to be US-ASCII.
   if (nsCRT::strcmp(destCharset, NS_LITERAL_STRING("us-ascii").get()))
-	    result = PR_smprintf("%s%s", nsMsgSearchAdapter::m_kImapCharset, NS_ConvertUCS2toUTF8(destCharset).get());
+	    result = PR_smprintf("%s%s", nsMsgSearchAdapter::m_kImapCharset, NS_ConvertUTF16toUTF8(destCharset).get());
 
 	return result;
 }
@@ -320,22 +329,27 @@ nsresult
 nsMsgSearchAdapter::GetSearchCharsets(PRUnichar **srcCharset, PRUnichar **dstCharset)
 {
   nsresult rv;
+  nsAutoString destination;
   NS_ENSURE_ARG(srcCharset);
   NS_ENSURE_ARG(dstCharset);
   
   if (m_defaultCharset.IsEmpty())
   {
     m_forceAsciiSearch = PR_FALSE;  // set the default value in case of error
-    nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
+    nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
     if (NS_SUCCEEDED(rv))
     {
-      rv = prefs->GetLocalizedUnicharPref("mailnews.view_default_charset", getter_Copies(m_defaultCharset));
-      rv = prefs->GetBoolPref("mailnews.force_ascii_search", &m_forceAsciiSearch);
+      nsCOMPtr<nsIPrefLocalizedString> localizedstr;
+      rv = prefs->GetComplexValue("mailnews.view_default_charset", NS_GET_IID(nsIPrefLocalizedString),
+                                  getter_AddRefs(localizedstr)); 
+      if (NS_SUCCEEDED(rv))
+        localizedstr->GetData(getter_Copies(m_defaultCharset));
+
+      prefs->GetBoolPref("mailnews.force_ascii_search", &m_forceAsciiSearch);
     }
   }
-  *srcCharset = nsCRT::strdup(m_defaultCharset.IsEmpty() ?
-    NS_LITERAL_STRING("ISO-8859-1").get() : m_defaultCharset.get());
-  *dstCharset = nsCRT::strdup(*srcCharset);
+  *srcCharset = m_defaultCharset.IsEmpty() ? 
+    ToNewUnicode(NS_LITERAL_STRING("ISO-8859-1")) : ToNewUnicode(m_defaultCharset);
   
   if (m_scope)
   {
@@ -349,10 +363,13 @@ nsMsgSearchAdapter::GetSearchCharsets(PRUnichar **srcCharset, PRUnichar **dstCha
     {
       nsXPIDLCString folderCharset;
       folder->GetCharset(getter_Copies(folderCharset));
-      PR_Free(*dstCharset);
-      *dstCharset = ToNewUnicode(folderCharset);
+      AppendASCIItoUTF16(folderCharset, destination);
     }
   }
+  else
+  {
+    destination.Assign(*srcCharset);
+  }
   
   
   // If 
@@ -360,10 +377,9 @@ nsMsgSearchAdapter::GetSearchCharsets(PRUnichar **srcCharset, PRUnichar **dstCha
   // the source. (CS_DEFAULT is an indication that the charset
   // was undefined or unavailable.)
   // ### well, it's not really anymore. Is there an equivalent?
-  if (!nsCRT::strcmp(*dstCharset, m_defaultCharset.get()))
+  if (destination.Equals(m_defaultCharset))
   {
-    PR_Free(*dstCharset);
-    *dstCharset = nsCRT::strdup(*srcCharset);
+    destination.Assign(*srcCharset);
   }
   
   if (m_forceAsciiSearch)
@@ -375,9 +391,10 @@ nsMsgSearchAdapter::GetSearchCharsets(PRUnichar **srcCharset, PRUnichar **dstCha
     // If the dest csid is ISO Latin 1 or MacRoman, attempt to convert the 
     // source text to US-ASCII. (Not for now.)
     // if ((dst_csid == CS_LATIN1) || (dst_csid == CS_MAC_ROMAN))
-    PR_Free(*dstCharset);
-    *dstCharset = nsCRT::strdup(NS_LITERAL_STRING("us-ascii").get());
+    destination.AssignLiteral("us-ascii");
   }
+
+  *dstCharset = ToNewUnicode(destination);
   return NS_OK;
 }
 
@@ -386,29 +403,28 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool real
   nsresult err = NS_OK;
   PRBool useNot = PR_FALSE;
   PRBool useQuotes = PR_FALSE;
-  PRBool excludeHeader = PR_FALSE;
   PRBool ignoreValue = PR_FALSE;
-  char *arbitraryHeader = nsnull;
+  nsCAutoString arbitraryHeader;
   const char *whichMnemonic = nsnull;
   const char *orHeaderMnemonic = nsnull;
-  
+
   *ppOutTerm = nsnull;
-  
+
   nsCOMPtr <nsIMsgSearchValue> searchValue;
   nsresult rv = term->GetValue(getter_AddRefs(searchValue));
-  
+
   if (NS_FAILED(rv))
     return rv;
-  
+
   nsMsgSearchOpValue op;
   term->GetOp(&op);
-  
+
   if (op == nsMsgSearchOp::DoesntContain || op == nsMsgSearchOp::Isnt)
     useNot = PR_TRUE;
-  
+
   nsMsgSearchAttribValue attrib;
   term->GetAttrib(&attrib);
-  
+
   switch (attrib)
   {
   case nsMsgSearchAttrib::ToOrCC:
@@ -428,7 +444,6 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool real
     break;
   case nsMsgSearchAttrib::Body:
     whichMnemonic = m_kImapBody;
-    excludeHeader = PR_TRUE;
     break;
   case nsMsgSearchAttrib::AgeInDays:  // added for searching online for age in days...
     // for AgeInDays, we are actually going to perform a search by date, so convert the operations for age
@@ -448,7 +463,20 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool real
       NS_ASSERTION(PR_FALSE, "invalid search operator");
       return NS_ERROR_INVALID_ARG;
     }
-    excludeHeader = PR_TRUE;
+    break;
+  case nsMsgSearchAttrib::Size:
+    switch (op)
+    {
+    case nsMsgSearchOp::IsGreaterThan:
+      whichMnemonic = m_kImapSizeLarger;
+      break;
+    case nsMsgSearchOp::IsLessThan:
+      whichMnemonic = m_kImapSizeSmaller;
+      break;
+    default:
+      NS_ASSERTION(PR_FALSE, "invalid search operator");
+      return NS_ERROR_INVALID_ARG;
+    }
     break;
     case nsMsgSearchAttrib::Date:
       switch (op)
@@ -467,224 +495,245 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool real
         NS_ASSERTION(PR_FALSE, "invalid search operator");
         return NS_ERROR_INVALID_ARG;
       }
-      excludeHeader = PR_TRUE;
       break;
-      case nsMsgSearchAttrib::AnyText:
-        whichMnemonic = m_kImapAnyText;
-        excludeHeader = PR_TRUE;
+    case nsMsgSearchAttrib::AnyText:
+      whichMnemonic = m_kImapAnyText;
+      break;
+    case nsMsgSearchAttrib::Keywords:
+      whichMnemonic = m_kImapKeyword;
+      break;
+    case nsMsgSearchAttrib::MsgStatus:
+      useNot = PR_FALSE; // bizarrely, NOT SEEN is wrong, but UNSEEN is right.
+      ignoreValue = PR_TRUE; // the mnemonic is all we need
+      PRUint32 status;
+      searchValue->GetStatus(&status);
+
+      switch (status)
+      {
+      case MSG_FLAG_READ:
+        whichMnemonic = op == nsMsgSearchOp::Is ? m_kImapSeen : m_kImapNotSeen;
         break;
-      case nsMsgSearchAttrib::Keywords:
-        whichMnemonic = m_kNntpKeywords;
+      case MSG_FLAG_REPLIED:
+        whichMnemonic = op == nsMsgSearchOp::Is ? m_kImapAnswered : m_kImapNotAnswered;
         break;
-      case nsMsgSearchAttrib::MsgStatus:
-        useNot = PR_FALSE; // bizarrely, NOT SEEN is wrong, but UNSEEN is right.
-        ignoreValue = PR_TRUE; // the mnemonic is all we need
-        excludeHeader = PR_TRUE;
-        PRUint32 status;
-        searchValue->GetStatus(&status);
-        
-        switch (status)
+      case MSG_FLAG_NEW:                         
+        whichMnemonic = op == nsMsgSearchOp::Is ? m_kImapNew : m_kImapNotNew;
+        break; 
+      case MSG_FLAG_MARKED:
+        whichMnemonic = op == nsMsgSearchOp::Is ? m_kImapFlagged : m_kImapNotFlagged;
+        break;
+      default:
+        NS_ASSERTION(PR_FALSE, "invalid search operator");
+        return NS_ERROR_INVALID_ARG;
+      }
+      break;
+    default:
+      if ( attrib > nsMsgSearchAttrib::OtherHeader && attrib < nsMsgSearchAttrib::kNumMsgSearchAttributes)
+      {
+        nsXPIDLCString arbitraryHeaderTerm;
+        term->GetArbitraryHeader(getter_Copies(arbitraryHeaderTerm));
+        if (!arbitraryHeaderTerm.IsEmpty())
         {
-        case MSG_FLAG_READ:
-          whichMnemonic = op == nsMsgSearchOp::Is ? m_kImapSeen : m_kImapNotSeen;
-          break;
-        case MSG_FLAG_REPLIED:
-          whichMnemonic = op == nsMsgSearchOp::Is ? m_kImapAnswered : m_kImapNotAnswered;
-          break;
-        default:
-          NS_ASSERTION(PR_FALSE, "invalid search operator");
-          return NS_ERROR_INVALID_ARG;
-        }
-        break;
-        default:
-          if ( attrib > nsMsgSearchAttrib::OtherHeader && attrib < nsMsgSearchAttrib::kNumMsgSearchAttributes)
-          {
-            nsXPIDLCString arbitraryHeaderTerm;
-            term->GetArbitraryHeader(getter_Copies(arbitraryHeaderTerm));
-            if (!arbitraryHeaderTerm.IsEmpty())
-            {
-              arbitraryHeader = new char [strlen((const char *)arbitraryHeaderTerm) + 6];  // 6 bytes for SPACE \" .... \" SPACE
-              if (!arbitraryHeader)
-                return NS_ERROR_OUT_OF_MEMORY;
-              arbitraryHeader[0] = '\0';
-              PL_strcat(arbitraryHeader, " \"");
-              PL_strcat(arbitraryHeader, (const char *)arbitraryHeaderTerm);
-              PL_strcat(arbitraryHeader, "\" ");
-              whichMnemonic = arbitraryHeader;
-            }
-            else
-              return NS_ERROR_FAILURE;
-          }
-          else
-          {
-            NS_ASSERTION(PR_FALSE, "invalid search operator");
-            return NS_ERROR_INVALID_ARG;
-          }
-        }
-        
-        char *value = "";
-        char dateBuf[100];
-        dateBuf[0] = '\0';
-        
-        PRBool valueWasAllocated = PR_FALSE;
-        if (attrib == nsMsgSearchAttrib::Date)
-        {
-          // note that there used to be code here that encoded an RFC822 date for imap searches.
-          // The IMAP RFC 2060 is misleading to the point that it looks like it requires an RFC822
-          // date but really it expects dd-mmm-yyyy, like dredd, and refers to the RFC822 date only in that the
-          // dd-mmm-yyyy date will match the RFC822 date within the message.
-          
-          PRTime adjustedDate;
-          searchValue->GetDate(&adjustedDate);
-          if (whichMnemonic == m_kImapSince)
-          {
-            // it looks like the IMAP server searches on Since includes the date in question...
-            // our UI presents Is, IsGreater and IsLessThan. For the IsGreater case (m_kImapSince)
-            // we need to adjust the date so we get greater than and not greater than or equal to which
-            // is what the IMAP server wants to search on
-            // won't work on Mac.
-            // ack, is this right? is PRTime seconds or microseconds?
-            PRInt64 microSecondsPerSecond, secondsInDay, microSecondsInDay;
-            
-            LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
-            LL_UI2L(secondsInDay, 60 * 60 * 24);
-            LL_MUL(microSecondsInDay, secondsInDay, microSecondsPerSecond);
-            LL_ADD(adjustedDate, adjustedDate, microSecondsInDay); // bump up to the day after this one...
-          }
-          
-          PRExplodedTime exploded;
-          PR_ExplodeTime(adjustedDate, PR_LocalTimeParameters, &exploded);
-          PR_FormatTimeUSEnglish(dateBuf, sizeof(dateBuf), "%d-%b-%Y", &exploded);
-          //		strftime (dateBuf, sizeof(dateBuf), "%d-%b-%Y", localtime (/* &term->m_value.u.date */ &adjustedDate));
-          value = dateBuf;
+          arbitraryHeader.AssignLiteral(" \"");
+          arbitraryHeader.Append(arbitraryHeaderTerm);
+          arbitraryHeader.AppendLiteral("\" ");
+          whichMnemonic = arbitraryHeader.get();
         }
         else
-        {
-          if (attrib == nsMsgSearchAttrib::AgeInDays)
-          {
-            // okay, take the current date, subtract off the age in days, then do an appropriate Date search on 
-            // the resulting day.
-            PRUint32 ageInDays;
-            
-            searchValue->GetAge(&ageInDays);
-            
-            PRTime now = PR_Now();
-            PRTime matchDay;
-            
-            PRInt64 microSecondsPerSecond, secondsInDays, microSecondsInDay;
-            
-            LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
-            LL_UI2L(secondsInDays, 60 * 60 * 24 * ageInDays);
-            LL_MUL(microSecondsInDay, secondsInDays, microSecondsPerSecond);
-            
-            LL_SUB(matchDay, now, microSecondsInDay); // = now - term->m_value.u.age * 60 * 60 * 24; 
-            PRExplodedTime exploded;
-            PR_ExplodeTime(matchDay, PR_LocalTimeParameters, &exploded);
-            PR_FormatTimeUSEnglish(dateBuf, sizeof(dateBuf), "%d-%b-%Y", &exploded);
-            //			strftime (dateBuf, sizeof(dateBuf), "%d-%b-%Y", localtime (&matchDay));
-            value = dateBuf;
-          }
-          else
-            
-            if (IsStringAttribute(attrib))
-            {
-              PRUnichar *convertedValue; // = reallyDredd ? MSG_EscapeSearchUrl (term->m_value.u.string) : msg_EscapeImapSearchProtocol(term->m_value.u.string);
-              nsXPIDLString searchTermValue;
-              searchValue->GetStr(getter_Copies(searchTermValue));
-              // Ugly switch for Korean mail/news charsets.
-              // We want to do this here because here is where
-              // we know what charset we want to use.
+          return NS_ERROR_FAILURE;
+      }
+      else
+      {
+        NS_ASSERTION(PR_FALSE, "invalid search operator");
+        return NS_ERROR_INVALID_ARG;
+      }
+    }
+
+    char *value = nsnull;
+    char dateBuf[100];
+    dateBuf[0] = '\0';
+
+    PRBool valueWasAllocated = PR_FALSE;
+    if (attrib == nsMsgSearchAttrib::Date)
+    {
+      // note that there used to be code here that encoded an RFC822 date for imap searches.
+      // The IMAP RFC 2060 is misleading to the point that it looks like it requires an RFC822
+      // date but really it expects dd-mmm-yyyy, like dredd, and refers to the RFC822 date only in that the
+      // dd-mmm-yyyy date will match the RFC822 date within the message.
+
+      PRTime adjustedDate;
+      searchValue->GetDate(&adjustedDate);
+      if (whichMnemonic == m_kImapSince)
+      {
+        // it looks like the IMAP server searches on Since includes the date in question...
+        // our UI presents Is, IsGreater and IsLessThan. For the IsGreater case (m_kImapSince)
+        // we need to adjust the date so we get greater than and not greater than or equal to which
+        // is what the IMAP server wants to search on
+        // won't work on Mac.
+        // ack, is this right? is PRTime seconds or microseconds?
+        PRInt64 microSecondsPerSecond, secondsInDay, microSecondsInDay;
+
+        LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
+        LL_UI2L(secondsInDay, 60 * 60 * 24);
+        LL_MUL(microSecondsInDay, secondsInDay, microSecondsPerSecond);
+        LL_ADD(adjustedDate, adjustedDate, microSecondsInDay); // bump up to the day after this one...
+      }
+
+      PRExplodedTime exploded;
+      PR_ExplodeTime(adjustedDate, PR_LocalTimeParameters, &exploded);
+      PR_FormatTimeUSEnglish(dateBuf, sizeof(dateBuf), "%d-%b-%Y", &exploded);
+      //		strftime (dateBuf, sizeof(dateBuf), "%d-%b-%Y", localtime (/* &term->m_value.u.date */ &adjustedDate));
+      value = dateBuf;
+    }
+    else
+    {
+      if (attrib == nsMsgSearchAttrib::AgeInDays)
+      {
+        // okay, take the current date, subtract off the age in days, then do an appropriate Date search on 
+        // the resulting day.
+        PRUint32 ageInDays;
+
+        searchValue->GetAge(&ageInDays);
+
+        PRTime now = PR_Now();
+        PRTime matchDay;
+
+        PRInt64 microSecondsPerSecond, secondsInDays, microSecondsInDay;
+
+        LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
+        LL_UI2L(secondsInDays, 60 * 60 * 24 * ageInDays);
+        LL_MUL(microSecondsInDay, secondsInDays, microSecondsPerSecond);
+
+        LL_SUB(matchDay, now, microSecondsInDay); // = now - term->m_value.u.age * 60 * 60 * 24; 
+        PRExplodedTime exploded;
+        PR_ExplodeTime(matchDay, PR_LocalTimeParameters, &exploded);
+        PR_FormatTimeUSEnglish(dateBuf, sizeof(dateBuf), "%d-%b-%Y", &exploded);
+        //			strftime (dateBuf, sizeof(dateBuf), "%d-%b-%Y", localtime (&matchDay));
+        value = dateBuf;
+      }
+      else if (attrib == nsMsgSearchAttrib::Size)
+      {
+        PRUint32 sizeValue;
+        nsCAutoString searchTermValue;
+        searchValue->GetSize(&sizeValue);
+
+        // Multiply by 1024 to get into kb resolution
+        sizeValue *= 1024;
+
+        // Ensure that greater than is really greater than
+        // in kb resolution.
+        if (op == nsMsgSearchOp::IsGreaterThan)
+          sizeValue += 1024;
+
+        searchTermValue.AppendInt(sizeValue);
+
+        value = nsCRT::strdup(searchTermValue.get());
+        valueWasAllocated = PR_TRUE;
+      }
+      else
+        
+      if (IsStringAttribute(attrib))
+      {
+        PRUnichar *convertedValue; // = reallyDredd ? MSG_EscapeSearchUrl (term->m_value.u.string) : msg_EscapeImapSearchProtocol(term->m_value.u.string);
+        nsXPIDLString searchTermValue;
+        searchValue->GetStr(getter_Copies(searchTermValue));
+        // Ugly switch for Korean mail/news charsets.
+        // We want to do this here because here is where
+        // we know what charset we want to use.
 #ifdef DOING_CHARSET
-              if (reallyDredd)
-                dest_csid = INTL_DefaultNewsCharSetID(dest_csid);
-              else
-                dest_csid = INTL_DefaultMailCharSetID(dest_csid);
+        if (reallyDredd)
+          dest_csid = INTL_DefaultNewsCharSetID(dest_csid);
+        else
+          dest_csid = INTL_DefaultMailCharSetID(dest_csid);
 #endif
-              
-              // do all sorts of crazy escaping
-              convertedValue = reallyDredd ? EscapeSearchUrl (searchTermValue) :
-              EscapeImapSearchProtocol(searchTermValue);
-              useQuotes = !reallyDredd || 
-                (nsDependentString(convertedValue).FindChar(PRUnichar(' ')) != -1);
-              // now convert to char* and escape quoted_specials
-              nsCAutoString valueStr;
-              nsresult rv = ConvertFromUnicode(NS_LossyConvertUTF16toASCII(destCharset).get(),
-                nsDependentString(convertedValue), valueStr);
-              if (NS_SUCCEEDED(rv))
-              {
-                const char *vptr = valueStr.get();
-                // max escaped length is one extra character for every character in the cmd.
-                nsAutoArrayPtr<char> newValue(new char[2*strlen(vptr) + 1]);
-                if (newValue)
-                {
-                  char *p = newValue;
-                  while (1)
-                  {
-                    char ch = *vptr++;
-                    if (!ch)
-                      break;
-                    if ((useQuotes ? ch == '"' : 0) || ch == '\\')
-                      *p++ = '\\';
-                    *p++ = ch;
-                  }
-                  *p = '\0';
-                  value = nsCRT::strdup(newValue); // realloc down to smaller size
-                }
-              }
-              else
-                value = nsCRT::strdup("");
-              nsCRT::free(convertedValue);
-              valueWasAllocated = PR_TRUE;
-              
-            }
-        }
-        
-        // this should be rewritten to use nsCString
-        int len = strlen(whichMnemonic) + strlen(value) + (useNot ? strlen(m_kImapNot) : 0) + 
-          (useQuotes ? 2 : 0) + strlen(m_kImapHeader) + 
-          (orHeaderMnemonic ? (strlen(m_kImapHeader) + strlen(m_kImapOr) + (useNot ? strlen(m_kImapNot) : 0) + 
-          strlen(orHeaderMnemonic) + strlen(value) + 2 /*""*/) : 0) + 10; // add slough for imap string literals
-        char *encoding = new char[len];
-        if (encoding)
+
+        // do all sorts of crazy escaping
+        convertedValue = reallyDredd ? EscapeSearchUrl (searchTermValue) :
+        EscapeImapSearchProtocol(searchTermValue);
+        useQuotes = !reallyDredd || 
+          (nsDependentString(convertedValue).FindChar(PRUnichar(' ')) != -1);
+        // now convert to char* and escape quoted_specials
+        nsCAutoString valueStr;
+        nsresult rv = ConvertFromUnicode(NS_LossyConvertUTF16toASCII(destCharset).get(),
+          nsDependentString(convertedValue), valueStr);
+        if (NS_SUCCEEDED(rv))
         {
-          encoding[0] = '\0';
-          // Remember: if ToOrCC and useNot then the expression becomes NOT To AND Not CC as opposed to (NOT TO) || (NOT CC)
-          if (orHeaderMnemonic && !useNot)
-            PL_strcat(encoding, m_kImapOr);
-          if (useNot)
-            PL_strcat (encoding, m_kImapNot);
-          if (!excludeHeader)
-            PL_strcat (encoding, m_kImapHeader);
-          PL_strcat (encoding, whichMnemonic);
-          if (!ignoreValue)
-            err = EncodeImapValue(encoding, value, useQuotes, reallyDredd);
-          
-          if (orHeaderMnemonic)
+          const char *vptr = valueStr.get();
+          // max escaped length is one extra character for every character in the cmd.
+          nsAutoArrayPtr<char> newValue(new char[2*strlen(vptr) + 1]);
+          if (newValue)
           {
-            if (useNot)
-              PL_strcat(encoding, m_kImapNot);
-            
-            PL_strcat (encoding, m_kImapHeader);
-            
-            PL_strcat (encoding, orHeaderMnemonic);
-            if (!ignoreValue)
-              err = EncodeImapValue(encoding, value, useQuotes, reallyDredd);
+            char *p = newValue;
+            while (1)
+            {
+              char ch = *vptr++;
+              if (!ch)
+                break;
+              if ((useQuotes ? ch == '"' : 0) || ch == '\\')
+                *p++ = '\\';
+              *p++ = ch;
+            }
+            *p = '\0';
+            value = nsCRT::strdup(newValue); // realloc down to smaller size
           }
-          
-          // kmcentee, don't let the encoding end with whitespace, 
-          // this throws off later url STRCMP
-          if (*encoding && *(encoding + strlen(encoding) - 1) == ' ')
-            *(encoding + strlen(encoding) - 1) = '\0';
         }
-        
-        if (value && valueWasAllocated)
-          PR_Free (value);
-        if (arbitraryHeader)
-          delete arbitraryHeader;
-        
-        *ppOutTerm = encoding;
-        
-        return err;
+        else
+          value = nsCRT::strdup("");
+        nsCRT::free(convertedValue);
+        valueWasAllocated = PR_TRUE;
+
+      }
+    }
+
+    // this should be rewritten to use nsCString
+    int subLen =
+      (value ? strlen(value) : 0) +
+      (useNot ? strlen(m_kImapNot) : 0) +
+      strlen(m_kImapHeader);
+    int len = strlen(whichMnemonic) + subLen + (useQuotes ? 2 : 0) +
+      (orHeaderMnemonic
+       ? (subLen + strlen(m_kImapOr) + strlen(orHeaderMnemonic) + 2 /*""*/)
+       : 0) +
+      10; // add slough for imap string literals
+    char *encoding = new char[len];
+    if (encoding)
+    {
+      encoding[0] = '\0';
+      // Remember: if ToOrCC and useNot then the expression becomes NOT To AND Not CC as opposed to (NOT TO) || (NOT CC)
+      if (orHeaderMnemonic && !useNot)
+        PL_strcat(encoding, m_kImapOr);
+      if (useNot)
+        PL_strcat (encoding, m_kImapNot);
+      if (!arbitraryHeader.IsEmpty())
+        PL_strcat (encoding, m_kImapHeader);
+      PL_strcat (encoding, whichMnemonic);
+      if (!ignoreValue)
+        err = EncodeImapValue(encoding, value, useQuotes, reallyDredd);
+
+      if (orHeaderMnemonic)
+      {
+        if (useNot)
+          PL_strcat(encoding, m_kImapNot);
+
+        PL_strcat (encoding, m_kImapHeader);
+
+        PL_strcat (encoding, orHeaderMnemonic);
+        if (!ignoreValue)
+          err = EncodeImapValue(encoding, value, useQuotes, reallyDredd);
+      }
+
+      // kmcentee, don't let the encoding end with whitespace, 
+      // this throws off later url STRCMP
+      if (*encoding && *(encoding + strlen(encoding) - 1) == ' ')
+        *(encoding + strlen(encoding) - 1) = '\0';
+    }
+
+    if (value && valueWasAllocated)
+      nsCRT::free (value);
+
+    *ppOutTerm = encoding;
+
+    return err;
 }
 
 nsresult nsMsgSearchAdapter::EncodeImapValue(char *encoding, const char *value, PRBool useQuotes, PRBool reallyDredd)
@@ -728,12 +777,7 @@ nsresult nsMsgSearchAdapter::EncodeImap (char **ppOutEncoding, nsISupportsArray
   PRUint32 termCount;
   searchTerms->Count(&termCount);
   PRUint32 i = 0;
-  int encodingLength = 0;
   
-  // Build up an array of encodings, one per query term
-  char **termEncodings = new char *[termCount];
-  if (!termEncodings)
-    return NS_ERROR_OUT_OF_MEMORY;
   // create our expression
   nsMsgSearchBoolExpression * expression = new nsMsgSearchBoolExpression();
   if (!expression)
@@ -741,60 +785,31 @@ nsresult nsMsgSearchAdapter::EncodeImap (char **ppOutEncoding, nsISupportsArray
   
   for (i = 0; i < termCount && NS_SUCCEEDED(err); i++)
   {
+    char *termEncoding;
     nsCOMPtr<nsIMsgSearchTerm> pTerm;
     searchTerms->QueryElementAt(i, NS_GET_IID(nsIMsgSearchTerm),
       (void **)getter_AddRefs(pTerm));
-    err = EncodeImapTerm (pTerm, reallyDredd, srcCharset, destCharset, &termEncodings[i]);
-    if (NS_SUCCEEDED(err) && nsnull != termEncodings[i])
+    err = EncodeImapTerm (pTerm, reallyDredd, srcCharset, destCharset, &termEncoding);
+    if (NS_SUCCEEDED(err) && nsnull != termEncoding)
     {
-      encodingLength += strlen(termEncodings[i]) + 1;
-      expression = nsMsgSearchBoolExpression::AddSearchTermWithEncoding(expression, pTerm,termEncodings[i]);
+      expression = nsMsgSearchBoolExpression::AddSearchTerm(expression, pTerm, termEncoding);
+      delete [] termEncoding;
     }
   }
   
   if (NS_SUCCEEDED(err)) 
   {
     // Catenate the intermediate encodings together into a big string
-    char *totalEncoding = new char [encodingLength + (!reallyDredd ? strlen(m_kImapUnDeleted) : 0) + 1];
-    nsCString encodingBuff;
+    nsCAutoString encodingBuff;
     
-    if (totalEncoding)
-    {
-      totalEncoding[0] = '\0';
-      
-      int offset = 0;       // offset into starting place for the buffer
-      if (!reallyDredd)
-        PL_strcat(totalEncoding, m_kImapUnDeleted);
-      
-      if (!reallyDredd)
-      {
-        encodingBuff.Append(m_kImapUnDeleted);
-        offset = strlen(m_kImapUnDeleted);
-      }
-      
-      expression->GenerateEncodeStr(&encodingBuff);
-      
-      for (i = 0; i < termCount; i++)
-      {
-        if (termEncodings[i])
-        {
-          PL_strcat (totalEncoding, termEncodings[i]);
-          delete [] termEncodings[i];
-        }
-      }
-    }
-    else
-      err = NS_ERROR_OUT_OF_MEMORY;
-    
-    delete totalEncoding;
-    delete expression;
-    
-    // Set output parameter if we encoded the query successfully
-    if (NS_SUCCEEDED(err))
-      *ppOutEncoding = ToNewCString(encodingBuff);
+    if (!reallyDredd)
+      encodingBuff.Append(m_kImapUnDeleted);
+
+    expression->GenerateEncodeStr(&encodingBuff);
+    *ppOutEncoding = ToNewCString(encodingBuff);
   }
   
-  delete [] termEncodings;
+  delete expression;
   
   return err;
 }
@@ -1038,12 +1053,12 @@ NS_IMETHODIMP nsMsgSearchValidityManager::GetTable (int whichTable, nsIMsgSearch
 {
   NS_ENSURE_ARG_POINTER(ppOutTable);
   
-  nsresult rv = NS_OK;
+  nsresult rv;
   *ppOutTable = nsnull;
   
-  nsCOMPtr<nsIPref> pref = do_GetService(NS_PREF_CONTRACTID, &rv);
+  nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
   nsXPIDLCString customHeaders;
-  if (NS_SUCCEEDED(rv) && pref)
+  if (NS_SUCCEEDED(rv))
     pref->GetCharPref(PREF_CUSTOM_HEADERS, getter_Copies(customHeaders));
   
   switch (whichTable)
@@ -1093,13 +1108,6 @@ NS_IMETHODIMP nsMsgSearchValidityManager::GetTable (int whichTable, nsIMsgSearch
       rv = SetOtherHeadersInTable(m_localNewsTable, customHeaders.get());
     *ppOutTable = m_localNewsTable;
     break;
-#ifdef DOING_EXNEWSSEARCH
-  case nsMsgSearchScope::newsEx:
-    if (!m_newsExTable)
-      rv = InitNewsExTable ();
-    *ppOutTable = m_newsExTable;
-    break;
-#endif
   case nsMsgSearchScope::LDAP:
     if (!m_ldapTable)
       rv = InitLdapTable ();
