@@ -21,6 +21,8 @@
  *
  * Contributor(s):
  *   Seth Spitzer <sspitzer@netscape.com>
+ *   Jungshik Shin <jshin@mailaps.org>
+ *   David Bienvenu <bienvenu@nventure.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -65,6 +67,7 @@
 #include "nsISupportsObsolete.h"
 #include "nsNetCID.h"
 #include "nsIFileStreams.h"
+#include "nsUnicharUtils.h"
 //---------------------------------------------------------------------------
 // nsMsgSearchTerm specifies one criterion, e.g. name contains phil
 //---------------------------------------------------------------------------
@@ -94,6 +97,8 @@ nsMsgSearchAttribEntry SearchAttribEntryTable[] =
     {nsMsgSearchAttrib::ToOrCC,     "to or cc"},
     {nsMsgSearchAttrib::AgeInDays,  "age in days"},
     {nsMsgSearchAttrib::Label,      "label"},
+    {nsMsgSearchAttrib::Keywords,   "tag"},
+    {nsMsgSearchAttrib::Size,       "size"},
     // this used to be nsMsgSearchAttrib::SenderInAddressBook
     // we used to have two Sender menuitems
     // for backward compatability, we can still parse
@@ -110,17 +115,17 @@ nsresult NS_MsgGetAttributeFromString(const char *string, PRInt16 *attrib)
   NS_ENSURE_ARG_POINTER(string);
   NS_ENSURE_ARG_POINTER(attrib);
   
-	PRBool found = PR_FALSE;
-	for (int idxAttrib = 0; idxAttrib < (int)(sizeof(SearchAttribEntryTable) / sizeof(nsMsgSearchAttribEntry)); idxAttrib++)
-	{
-		if (!PL_strcasecmp(string, SearchAttribEntryTable[idxAttrib].attribName))
-		{
-			found = PR_TRUE;
-			*attrib = SearchAttribEntryTable[idxAttrib].attrib;
-			break;
-		}
-	}  
-	if (!found)
+  PRBool found = PR_FALSE;
+  for (int idxAttrib = 0; idxAttrib < (int)(sizeof(SearchAttribEntryTable) / sizeof(nsMsgSearchAttribEntry)); idxAttrib++)
+  {
+    if (!PL_strcasecmp(string, SearchAttribEntryTable[idxAttrib].attribName))
+    {
+      found = PR_TRUE;
+      *attrib = SearchAttribEntryTable[idxAttrib].attrib;
+      break;
+    }
+  }  
+  if (!found)
   {
     nsresult rv;
     PRBool goodHdr;
@@ -164,7 +169,14 @@ nsresult NS_MsgGetAttributeFromString(const char *string, PRInt16 *attrib)
       }
     }
   }
-	return (found) ? NS_OK : NS_ERROR_INVALID_ARG;
+  // If we didn't find the header in MAILNEWS_CUSTOM_HEADERS, we're 
+  // going to return NS_OK and an attrib of nsMsgSearchAttrib::OtherHeader+1.
+  // in case it's a client side spam filter description filter, 
+  // which doesn't add its headers to mailnews.customMailHeaders.
+  // We've already checked that it's a valid header and returned
+  // an error if so.
+
+  return NS_OK;
 }
 
 nsresult NS_MsgGetStringForAttribute(PRInt16 attrib, const char **string)
@@ -259,7 +271,7 @@ nsresult NS_MsgGetStringForOperator(PRInt16 op, const char **string)
 void NS_MsgGetUntranslatedStatusName (uint32 s, nsCString *outName)
 {
 	const char *tmpOutName = NULL;
-#define MSG_STATUS_MASK (MSG_FLAG_READ | MSG_FLAG_REPLIED | MSG_FLAG_FORWARDED | MSG_FLAG_NEW)
+#define MSG_STATUS_MASK (MSG_FLAG_READ | MSG_FLAG_REPLIED | MSG_FLAG_FORWARDED | MSG_FLAG_NEW | MSG_FLAG_MARKED)
 	PRUint32 maskOut = (s & MSG_STATUS_MASK);
 
 	// diddle the flags to pay attention to the most important ones first, if multiple
@@ -291,6 +303,9 @@ void NS_MsgGetUntranslatedStatusName (uint32 s, nsCString *outName)
 	case MSG_FLAG_NEW:
 		tmpOutName = "new";
 		break;
+        case MSG_FLAG_MARKED:
+                tmpOutName = "flagged";
+                break;
 	default:
 		// This is fine, status may be "unread" for example
         break;
@@ -313,6 +328,8 @@ PRInt32 NS_MsgGetStatusValueFromName(char *name)
     return MSG_FLAG_FORWARDED|MSG_FLAG_REPLIED;
   if (!strcmp("new", name))
     return MSG_FLAG_NEW;
+  if (!strcmp("flagged", name))
+    return MSG_FLAG_MARKED;
   return 0;
 }
 
@@ -327,6 +344,7 @@ nsMsgSearchTerm::nsMsgSearchTerm()
     m_attribute = nsMsgSearchAttrib::Default;
     mBeginsGrouping = PR_FALSE;
     mEndsGrouping = PR_FALSE;
+    m_matchAll = PR_FALSE;
 }
 
 nsMsgSearchTerm::nsMsgSearchTerm (
@@ -342,6 +360,7 @@ nsMsgSearchTerm::nsMsgSearchTerm (
   if (attrib > nsMsgSearchAttrib::OtherHeader  && attrib < nsMsgSearchAttrib::kNumMsgSearchAttributes && arbitraryHeader)
     m_arbitraryHeader = arbitraryHeader;
   nsMsgResultElement::AssignValues (val, &m_value);
+  m_matchAll = PR_FALSE;
 }
 
 
@@ -461,6 +480,11 @@ nsresult nsMsgSearchTerm::OutputValue(nsCString &outputStr)
         outputStr.Append("true");  // don't need anything here, really
         break;
       }
+    case nsMsgSearchAttrib::Size:
+      {
+        outputStr.AppendInt(m_value.u.size);
+        break;
+      }
     default:
       NS_ASSERTION(PR_FALSE, "trying to output invalid attribute");
       break;
@@ -475,6 +499,11 @@ NS_IMETHODIMP nsMsgSearchTerm::GetTermAsString (nsACString &outStream)
   nsCAutoString	outputStr;
   nsresult	ret;
   
+  if (m_matchAll)
+  {
+    outStream = "ALL";
+    return NS_OK;
+  }
   ret = NS_MsgGetStringForAttribute(m_attribute, &attrib);
   if (ret != NS_OK)
     return ret;
@@ -549,6 +578,9 @@ nsresult nsMsgSearchTerm::ParseValue(char *inStream)
     case nsMsgSearchAttrib::HasAttachmentStatus:
       m_value.u.msgStatus = MSG_FLAG_ATTACHMENT;
       break; // this should always be true.
+    case nsMsgSearchAttrib::Size:
+      m_value.u.size = atoi(inStream);
+      break;
     default:
       NS_ASSERTION(PR_FALSE, "invalid attribute parsing search term value");
       break;
@@ -618,18 +650,31 @@ nsMsgSearchTerm::ParseAttribute(char *inStream, nsMsgSearchAttribValue *attrib)
 
 nsresult nsMsgSearchTerm::DeStreamNew (char *inStream, PRInt16 /*length*/)
 {
-	char *commaSep = PL_strchr(inStream, ',');
-	nsresult rv = ParseAttribute(inStream, &m_attribute);  // will allocate space for arbitrary header if necessary
+  if (!strcmp(inStream, "ALL"))
+  {
+    m_matchAll = PR_TRUE;
+    return NS_OK;
+  }
+  char *commaSep = PL_strchr(inStream, ',');
+  nsresult rv = ParseAttribute(inStream, &m_attribute);  // will allocate space for arbitrary header if necessary
   NS_ENSURE_SUCCESS(rv, rv);
-	if (!commaSep)
-		return NS_ERROR_INVALID_ARG;
-	char *secondCommaSep = PL_strchr(commaSep + 1, ',');
-	if (commaSep)
-		rv = ParseOperator(commaSep + 1, &m_operator);
+  if (!commaSep)
+    return NS_ERROR_INVALID_ARG;
+  char *secondCommaSep = PL_strchr(commaSep + 1, ',');
+  if (commaSep)
+    rv = ParseOperator(commaSep + 1, &m_operator);
   NS_ENSURE_SUCCESS(rv, rv);
-	if (secondCommaSep)
-		ParseValue(secondCommaSep + 1);
-	return NS_OK;
+  // convert label filters and saved searches to keyword equivalents
+  if (secondCommaSep)
+    ParseValue(secondCommaSep + 1);
+  if (m_attribute == nsMsgSearchAttrib::Label)
+  {
+    nsCAutoString keyword("$label");
+    m_value.attribute = m_attribute = nsMsgSearchAttrib::Keywords;
+    keyword.Append('0' + m_value.u.label);
+    m_value.string = PL_strdup(keyword.get());
+  }
+  return NS_OK;
 }
 
 
@@ -719,14 +764,25 @@ nsresult nsMsgSearchTerm::MatchArbitraryHeader (nsIMsgSearchScopeTerm *scope,
   GetMatchAllBeforeDeciding(&result);
   
   nsCAutoString buf;
+  nsCAutoString curMsgHeader;
   PRBool searchingHeaders = PR_TRUE;
   while (searchingHeaders && (bodyHandler->GetNextLine(buf) >=0))
   {
     char * buf_end = (char *) (buf.get() + buf.Length());
     int headerLength = m_arbitraryHeader.Length();
-    if (!PL_strncasecmp(buf.get(), m_arbitraryHeader.get(),headerLength))
+    PRBool isContinuationHeader = NS_IsAsciiWhitespace(buf.CharAt(0));
+    // this handles wrapped header lines, which start with whitespace. 
+    // If the line starts with whitespace, then we use the current header.
+    if (!isContinuationHeader)
     {
-      const char * headerValue = buf.get() + headerLength; // value occurs after the header name...
+      PRUint32 colonPos = buf.FindChar(':');
+      buf.Left(curMsgHeader, colonPos);
+    }
+
+    if (curMsgHeader.Equals(m_arbitraryHeader, nsCaseInsensitiveCStringComparator()))
+    {
+      // value occurs after the header name or whitespace continuation char.
+      const char * headerValue = buf.get() + (isContinuationHeader ? 1 : headerLength); 
       if (headerValue < buf_end && headerValue[0] == ':')  // + 1 to account for the colon which is MANDATORY
         headerValue++; 
       
@@ -797,9 +853,9 @@ nsresult nsMsgSearchTerm::MatchBody (nsIMsgSearchScopeTerm *scope, PRUint32 offs
   // since we don't have a MIME parser handy, and we want to err on the
   // side of too many hits rather than not enough, we'll assume in that
   // general direction. Blech. ### FIX ME 
-  // bug fix #88935: for stateful csids like JIS, we don't want to decode
-  // quoted printable since it contains '='.
-  PRBool isQuotedPrintable =  /*!(mail_csid & STATEFUL) && */
+  // bug fix #314637: for stateful charsets like ISO-2022-JP, we don't
+  // want to decode quoted printable since it contains '='.
+  PRBool isQuotedPrintable = !nsMsgI18Nstateful_charset(folderCharset) &&
     (PL_strchr (m_value.string, '=') == nsnull);
   
   while (!endOfFile && result == boolContinueLoop)
@@ -921,95 +977,65 @@ nsresult nsMsgSearchTerm::MatchString (const char *stringToMatch,
   PRBool result = PR_FALSE;
   
   nsresult err = NS_OK;
-  nsCAutoString n_str;
-  const char *utf8 = stringToMatch;
-  if(nsMsgSearchOp::IsEmpty != m_operator)	// Save some performance for opIsEmpty
+  nsAutoString utf16StrToMatch;
+  nsAutoString needle; 
+
+  // Save some performance for opIsEmpty
+  if(nsMsgSearchOp::IsEmpty != m_operator)  
   {
-    n_str = m_value.string;
+    NS_ASSERTION(IsUTF8(nsDependentCString(m_value.string)),
+                        "m_value.string is not UTF-8");
+    CopyUTF8toUTF16(m_value.string, needle);
+
     if (charset != nsnull)
     {
-      nsAutoString out;
-      ConvertToUnicode(charset, stringToMatch ? stringToMatch : "", out);
-      utf8 = ToNewUTF8String(out);
+      ConvertToUnicode(charset, stringToMatch ? stringToMatch : "",
+                       utf16StrToMatch);
+    }
+    else { 
+      NS_ASSERTION(IsUTF8(nsDependentCString(stringToMatch)),
+                   "stringToMatch is not UTF-8");
+      CopyUTF8toUTF16(stringToMatch, utf16StrToMatch);
     }
   }
   
   switch (m_operator)
   {
   case nsMsgSearchOp::Contains:
-    if ((nsnull != utf8) && ((n_str.get())[0]) && /* INTL_StrContains(csid, n_header, n_str) */
-      PL_strcasestr(utf8, n_str.get()))
+    if (CaseInsensitiveFindInReadable(needle, utf16StrToMatch))
       result = PR_TRUE;
     break;
   case nsMsgSearchOp::DoesntContain:
-    if ((nsnull != utf8) && ((n_str.get())[0]) &&  /* !INTL_StrContains(csid, n_header, n_str) */
-      !PL_strcasestr(utf8, n_str.get()))
+    if(!CaseInsensitiveFindInReadable(needle, utf16StrToMatch))
       result = PR_TRUE;
     break;
   case nsMsgSearchOp::Is:
-    if(utf8)
-    {
-      if ((n_str.get())[0])
-      {
-        if (n_str.EqualsIgnoreCase(utf8) /* INTL_StrIs(csid, n_header, n_str)*/ )
-          result = PR_TRUE;
-      }
-      else if (utf8[0] == '\0') // Special case for "is <the empty string>"
-        result = PR_TRUE;
-    }
+    if(needle.Equals(utf16StrToMatch, nsCaseInsensitiveStringComparator()))
+      result = PR_TRUE;
     break;
   case nsMsgSearchOp::Isnt:
-    if(utf8)
-    {
-      if ((n_str.get())[0])
-      {
-        if (!n_str.EqualsIgnoreCase(utf8)/* INTL_StrIs(csid, n_header, n_str)*/ )
-          result = PR_TRUE;
-      }
-      else if (utf8[0] != '\0') // Special case for "isn't <the empty string>"
-        result = PR_TRUE;
-    }
+    if(!needle.Equals(utf16StrToMatch, nsCaseInsensitiveStringComparator()))
+      result = PR_TRUE;
     break;
   case nsMsgSearchOp::IsEmpty:
-    if (!PL_strlen(utf8))
+    // For IsEmpty, we didn't copy stringToMatch to utf16StrToMatch.
+    if (!PL_strlen(stringToMatch))
       result = PR_TRUE;
     break;
   case nsMsgSearchOp::BeginsWith:
-#ifdef DO_I18N_YET
-    if((nsnull != n_str) && (nsnull != utf8) && INTL_StrBeginWith(csid, utf8, n_str))
+    if (StringBeginsWith(utf16StrToMatch, needle,
+                         nsCaseInsensitiveStringComparator()))
       result = PR_TRUE;
-#else
-    // ### DMB - not the  most efficient way to do this.
-    if (PL_strncmp(utf8, n_str.get(), n_str.Length()) == 0)
-      result = PR_TRUE;
-#endif
     break;
   case nsMsgSearchOp::EndsWith: 
-    {
-      PRUint32 searchStrLen = (PRUint32) PL_strlen(utf8);
-      if (n_str.Length() <= searchStrLen)
-      {
-        PRInt32 sourceStrOffset = searchStrLen - n_str.Length();
-        if (PL_strcmp(utf8 + sourceStrOffset, n_str.get()) == 0)
-          result = PR_TRUE;
-      }
-    }
-#ifdef DO_I18N_YET
-    {
-      if((nsnull != n_str) && (nsnull != utf8) && INTL_StrEndWith(csid, utf8, n_str))
-        result = PR_TRUE;
-    }
-#else
-    
-#endif
+    if (StringEndsWith(utf16StrToMatch, needle,
+                       nsCaseInsensitiveStringComparator()))
+      result = PR_TRUE;
     break;
   default:
     NS_ASSERTION(PR_FALSE, "invalid operator matching search results");
   }
   
-  if (utf8 != nsnull && utf8 != stringToMatch)
-    free((void *)utf8);
-  
   *pResult = result;
   return err;
 }
@@ -1195,28 +1221,38 @@ nsresult nsMsgSearchTerm::MatchAge (PRTime msgDate, PRBool *pResult)
 
 nsresult nsMsgSearchTerm::MatchSize (PRUint32 sizeToMatch, PRBool *pResult)
 {
-	NS_ENSURE_ARG_POINTER(pResult);
+  NS_ENSURE_ARG_POINTER(pResult);
 
-	PRBool result = PR_FALSE;
-	switch (m_operator)
-	{
-	case nsMsgSearchOp::IsGreaterThan:
-		if (sizeToMatch > m_value.u.size)
-			result = PR_TRUE;
-		break;
-	case nsMsgSearchOp::IsLessThan:
-		if (sizeToMatch < m_value.u.size)
-			result = PR_TRUE;
-		break;
-	case nsMsgSearchOp::Is:
-		if (sizeToMatch == m_value.u.size)
-			result = PR_TRUE;
-		break;
-	default:
-		break;
-	}
-	*pResult = result;
-	return NS_OK;
+  PRBool result = PR_FALSE;
+  // We reduce the sizeToMatch rather than supplied size
+  // as then we can do an exact match on the displayed value
+  // which will be less confusing to the user.
+  PRUint32 sizeToMatchKB = sizeToMatch;
+
+  if (sizeToMatchKB < 1024)
+    sizeToMatchKB = 1024;
+
+  sizeToMatchKB /= 1024;
+
+  switch (m_operator)
+  {
+  case nsMsgSearchOp::IsGreaterThan:
+    if (sizeToMatchKB > m_value.u.size)
+      result = PR_TRUE;
+    break;
+  case nsMsgSearchOp::IsLessThan:
+    if (sizeToMatchKB < m_value.u.size)
+      result = PR_TRUE;
+    break;
+  case nsMsgSearchOp::Is:
+    if (sizeToMatchKB == m_value.u.size)
+      result = PR_TRUE;
+    break;
+  default:
+    break;
+  }
+  *pResult = result;
+  return NS_OK;
 }
 
 nsresult nsMsgSearchTerm::MatchJunkStatus(const char *aJunkScore, PRBool *pResult)
@@ -1242,22 +1278,22 @@ nsresult nsMsgSearchTerm::MatchJunkStatus(const char *aJunkScore, PRBool *pResul
   }
 
   nsresult rv = NS_OK;
-	PRBool matches = (junkStatus == m_value.u.junkStatus);
+  PRBool matches = (junkStatus == m_value.u.junkStatus);
 
-	switch (m_operator)
-	{
-	case nsMsgSearchOp::Is:
-		break;
-	case nsMsgSearchOp::Isnt:
-		matches = !matches;
-		break;
-	default:
-		rv = NS_ERROR_FAILURE;
-		NS_ASSERTION(PR_FALSE, "invalid compare op for junk status");
-	}
+  switch (m_operator)
+  {
+    case nsMsgSearchOp::Is:
+      break;
+    case nsMsgSearchOp::Isnt:
+      matches = !matches;
+      break;
+    default:
+      rv = NS_ERROR_FAILURE;
+      NS_ASSERTION(PR_FALSE, "invalid compare op for junk status");
+  }
 
   *pResult = matches;
-	return rv;	
+  return rv;	
 }
 
 nsresult nsMsgSearchTerm::MatchLabel(nsMsgLabelValue aLabelValue, PRBool *pResult)
@@ -1282,62 +1318,109 @@ nsresult nsMsgSearchTerm::MatchLabel(nsMsgLabelValue aLabelValue, PRBool *pResul
 
 nsresult nsMsgSearchTerm::MatchStatus(PRUint32 statusToMatch, PRBool *pResult)
 {
-	NS_ENSURE_ARG_POINTER(pResult);
+  NS_ENSURE_ARG_POINTER(pResult);
 
-	nsresult rv = NS_OK;
-	PRBool matches = (statusToMatch & m_value.u.msgStatus);
+  nsresult rv = NS_OK;
+  PRBool matches = (statusToMatch & m_value.u.msgStatus);
 
-	switch (m_operator)
-	{
-	case nsMsgSearchOp::Is:
-		break;
-	case nsMsgSearchOp::Isnt:
-		matches = !matches;
-		break;
-	default:
-		rv = NS_ERROR_FAILURE;
+  switch (m_operator)
+  {
+  case nsMsgSearchOp::Is:
+    break;
+  case nsMsgSearchOp::Isnt:
+    matches = !matches;
+    break;
+  default:
+    rv = NS_ERROR_FAILURE;
     NS_ERROR("invalid compare op for msg status");
-	}
+  }
 
   *pResult = matches;
-	return rv;	
+  return rv;	
+}
+
+nsresult nsMsgSearchTerm::MatchKeyword(const char *keyword, PRBool *pResult)
+{
+  NS_ENSURE_ARG_POINTER(pResult);
+
+  nsresult rv = NS_OK;
+  nsCAutoString keys;
+
+  PRBool matches = PR_FALSE;
+
+  switch (m_operator)
+  {
+  case nsMsgSearchOp::Is:
+    matches = !strcmp(keyword, m_value.string);
+    break;
+  case nsMsgSearchOp::Isnt:
+    matches = strcmp(keyword, m_value.string);
+    break;
+  case nsMsgSearchOp::DoesntContain:
+  case nsMsgSearchOp::Contains:
+    {
+      const char *keywordLoc = PL_strstr(keyword, m_value.string);
+      const char *startOfKeyword = keyword;
+      PRUint32 keywordLen = strlen(m_value.string);
+      while (keywordLoc)
+      {
+        // if the keyword is at the beginning of the string, then it's a match if 
+        // it is either the whole string, or is followed by a space, it's a match.
+        if (keywordLoc == startOfKeyword || (keywordLoc[-1] == ' '))
+        {
+          matches = keywordLen == strlen(keywordLoc) || (keywordLoc[keywordLen] == ' ');
+          if (matches)
+            break;
+        }
+        startOfKeyword = keywordLoc + keywordLen;
+        keywordLoc = PL_strstr(keyword, keywordLoc + keywordLen + 1);
+      }
+    }
+    break;
+  default:
+    rv = NS_ERROR_FAILURE;
+    NS_ERROR("invalid compare op for msg status");
+  }
+
+  *pResult = (m_operator == nsMsgSearchOp::DoesntContain) ? !matches : matches;
+  return rv;	
 }
 
 nsresult
 nsMsgSearchTerm::MatchPriority (nsMsgPriorityValue priorityToMatch,
                                 PRBool *pResult)
 {
-	NS_ENSURE_ARG_POINTER(pResult);
+  NS_ENSURE_ARG_POINTER(pResult);
 
-	nsresult err = NS_OK;
-	PRBool result=NS_OK;
+  nsresult err = NS_OK;
+  PRBool result=NS_OK;
 
-	// Use this ugly little hack to get around the fact that enums don't have
-	// integer compare operators
-	int p1 = (priorityToMatch == nsMsgPriority::none) ? (int) nsMsgPriority::normal : (int) priorityToMatch;
-	int p2 = (int) m_value.u.priority;
+  // Use this ugly little hack to get around the fact that enums don't have
+  // integer compare operators
+  int p1 = (priorityToMatch == nsMsgPriority::none) ? (int) nsMsgPriority::normal : (int) priorityToMatch;
+  int p2 = (int) m_value.u.priority;
 
-	switch (m_operator)
-	{
-	case nsMsgSearchOp::IsHigherThan:
-		if (p1 > p2)
-			result = PR_TRUE;
-		break;
-	case nsMsgSearchOp::IsLowerThan:
-		if (p1 < p2)
-			result = PR_TRUE;
-		break;
-	case nsMsgSearchOp::Is:
-		if (p1 == p2)
-			result = PR_TRUE;
-		break;
-	default:
-		result = PR_FALSE;
-		err = NS_ERROR_FAILURE;
-		NS_ASSERTION(PR_FALSE, "invalid match operator");
-	}
-	*pResult = result;
-	return err;
+  switch (m_operator)
+  {
+  case nsMsgSearchOp::IsHigherThan:
+    if (p1 > p2)
+      result = PR_TRUE;
+    break;
+  case nsMsgSearchOp::IsLowerThan:
+    if (p1 < p2)
+      result = PR_TRUE;
+    break;
+  case nsMsgSearchOp::Is:
+    if (p1 == p2)
+      result = PR_TRUE;
+    break;
+  default:
+    result = PR_FALSE;
+    err = NS_ERROR_FAILURE;
+    NS_ASSERTION(PR_FALSE, "invalid match operator");
+  }
+  *pResult = result;
+  return err;
 }
 
 // Lazily initialize the rfc822 header parser we're going to use to do
@@ -1355,7 +1438,7 @@ nsresult nsMsgSearchTerm::InitHeaderAddressParser()
 
 NS_IMPL_GETSET(nsMsgSearchTerm, Attrib, nsMsgSearchAttribValue, m_attribute)
 NS_IMPL_GETSET(nsMsgSearchTerm, Op, nsMsgSearchOpValue, m_operator)
-
+NS_IMPL_GETSET(nsMsgSearchTerm, MatchAll, PRBool, m_matchAll)
 
 NS_IMETHODIMP
 nsMsgSearchTerm::GetValue(nsIMsgSearchValue **aResult)
@@ -1455,15 +1538,23 @@ nsMsgSearchScopeTerm::GetSearchSession(nsIMsgSearchSession** aResult)
 NS_IMETHODIMP nsMsgSearchScopeTerm::GetMailFile(nsILocalFile **aLocalFile)
 {
   NS_ENSURE_ARG_POINTER(aLocalFile);
- if (!m_folder)
-   return NS_ERROR_NULL_POINTER;
+  if (!m_localFile)
+  {
+    if (!m_folder)
+     return NS_ERROR_NULL_POINTER;
 
-  nsCOMPtr <nsIFileSpec> fileSpec;
-  m_folder->GetPath(getter_AddRefs(fileSpec));
-  nsFileSpec realSpec;
-  fileSpec->GetFileSpec(&realSpec);
-  NS_FileSpecToIFile(&realSpec, aLocalFile);
-  return (*aLocalFile) ? NS_OK : NS_ERROR_FAILURE;
+    nsCOMPtr <nsIFileSpec> fileSpec;
+    m_folder->GetPath(getter_AddRefs(fileSpec));
+    nsFileSpec realSpec;
+    fileSpec->GetFileSpec(&realSpec);
+    NS_FileSpecToIFile(&realSpec, getter_AddRefs(m_localFile));
+  }
+  if (m_localFile)
+  {
+    NS_ADDREF(*aLocalFile = m_localFile);
+    return NS_OK;
+  }
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP nsMsgSearchScopeTerm::GetInputStream(nsIInputStream **aInputStream)
@@ -1515,21 +1606,12 @@ nsresult nsMsgSearchScopeTerm::InitializeAdapter (nsISupportsArray *termList)
         m_adapter = new nsMsgSearchOfflineMail (this, termList);
       break;
     case nsMsgSearchScope::newsEx:
-#ifdef DOING_EXNEWSSEARCH
-        if (m_folder->KnowsSearchNntpExtension())
-          m_adapter = new nsMsgSearchNewsEx (this, termList);
-        else
-          m_adapter = new nsMsgSearchNews(this, termList);
-#endif
       NS_ASSERTION(PR_FALSE, "not supporting newsEx yet");
       break;
     case nsMsgSearchScope::news:
           m_adapter = new nsMsgSearchNews (this, termList);
         break;
     case nsMsgSearchScope::allSearchableGroups:
-#ifdef DOING_EXNEWSSEARCH
-      m_adapter = new msMsgSearchNewsEx (this, termList);
-#endif
       NS_ASSERTION(PR_FALSE, "not supporting allSearchableGroups yet");
       break;
     case nsMsgSearchScope::LDAP:
@@ -1621,6 +1703,9 @@ nsresult nsMsgResultElement::AssignValues (nsIMsgSearchValue *src, nsMsgSearchVa
   case nsMsgSearchAttrib::JunkStatus:
     err = src->GetJunkStatus(&dst->u.junkStatus);
     break;
+  case nsMsgSearchAttrib::Size:
+    err = src->GetSize(&dst->u.size);
+    break;
   default:
     if (dst->attribute < nsMsgSearchAttrib::kNumMsgSearchAttributes)
     {
@@ -1664,256 +1749,16 @@ nsresult nsMsgResultElement::GetValue (nsMsgSearchAttribValue attrib,
         err = NS_ERROR_OUT_OF_MEMORY;
     }
   }
-#ifdef HAVE_SEARCH_PORT
-  // No need to store the folderInfo separately; we can always get it if/when
-  // we need it. This code is to support "view thread context" in the search dialog
-  if (SearchError_ScopeAgreement == err && attrib == nsMsgSearchAttrib::FolderInfo)
-  {
-    nsMsgFolderInfo *targetFolder = m_adapter->FindTargetFolder (this);
-    if (targetFolder)
-    {
-      *outValue = new nsMsgSearchValue;
-      if (*outValue)
-      {
-        (*outValue)->u.folder = targetFolder;
-        (*outValue)->attribute = nsMsgSearchAttrib::FolderInfo;
-        err = NS_OK;
-      }
-    }
-  }
-#endif
   return err;
 }
 
-
-nsresult
-nsMsgResultElement::GetValueRef (nsMsgSearchAttribValue attrib,
-                                 nsIMsgSearchValue* *aResult) const 
-{
-  nsCOMPtr<nsIMsgSearchValue> value;
-  PRUint32 count;
-  m_valueList->Count(&count);
-  nsresult rv;
-  for (PRUint32 i = 0; i < count; i++)
-  {
-    rv = m_valueList->QueryElementAt(i, NS_GET_IID(nsIMsgSearchValue),
-      getter_AddRefs(value));
-    NS_ASSERTION(NS_SUCCEEDED(rv), "bad element of array");
-    if (NS_FAILED(rv)) continue;
-    
-    nsMsgSearchAttribValue valueAttribute;
-    value->GetAttrib(&valueAttribute);
-    if (attrib == valueAttribute) {
-      *aResult = value;
-      NS_ADDREF(*aResult);
-    }
-  }
-  return NS_ERROR_FAILURE;
-}
-
-
 nsresult nsMsgResultElement::GetPrettyName (nsMsgSearchValue **value)
 {
-  nsresult err = GetValue (nsMsgSearchAttrib::Location, value);
-#ifdef HAVE_SEARCH_PORT
-  if (NS_OK == err)
-  {
-    nsMsgFolderInfo *folder = m_adapter->m_scope->m_folder;
-    nsMsgNewsHost *host = NULL;
-    if (folder)
-    {
-      // Find the news host because only the host knows whether pretty
-      // names are supported. 
-      if (FOLDER_CONTAINERONLY == folder->GetType())
-        host = ((nsMsgNewsFolderInfoContainer*) folder)->GetHost();
-      else if (folder->IsNews())
-        host = folder->GetNewsFolderInfo()->GetHost();
-      
-      // Ask the host whether it knows pretty names. It isn't strictly
-      // necessary to avoid calling folder->GetPrettiestName() since it
-      // does the right thing. But we do have to find the folder from the host.
-      if (host && host->QueryExtension ("LISTPNAMES"))
-      {
-        folder = host->FindGroup ((*value)->u.string);
-        if (folder)
-        {
-          char *tmp = nsCRT::strdup (folder->GetPrettiestName());
-          if (tmp)
-          {
-            XP_FREE ((*value)->u.string);
-            (*value)->u.utf8SstringZ = tmp;
-          }
-        }
-      }
-    }
-  }
-#endif // HAVE_SEARCH_PORT
-  return err;
+  return GetValue (nsMsgSearchAttrib::Location, value);
 }
 
-int nsMsgResultElement::CompareByFolderInfoPtrs (const void *e1, const void *e2)
-{
-#ifdef HAVE_SEARCH_PORT
-  nsMsgResultElement * re1 = *(nsMsgResultElement **) e1;
-  nsMsgResultElement * re2 = *(nsMsgResultElement **) e2;
-  
-  // get the src folder for each one
-  
-  const nsMsgSearchValue * v1 = re1->GetValueRef(attribFolderInfo);
-  const nsMsgSearchValue * v2 = re2->GetValueRef(attribFolderInfo);
-  
-  if (!v1 || !v2)
-    return 0;
-  
-  return (v1->u.folder - v2->u.folder);
-#else
-  return -1;
-#endif // HAVE_SEARCH_PORT
-}
-
-
-
-int nsMsgResultElement::Compare (const void *e1, const void *e2)
-{
-  int ret = 0;
-#ifdef HAVE_SEARCH_PORT
-  // Bad karma to cast away const, but they're my objects anway.
-  // Maybe if we go through and const everything this should be a const ptr.
-  nsMsgResultElement *re1 = *(nsMsgResultElement**) e1;
-  nsMsgResultElement *re2 = *(nsMsgResultElement**) e2;
-  
-  NS_ASSERTION(re1->IsValid(), "invalid result element1 in resultElement::Compare");
-  NS_ASSERTION(re2->IsValid(), "invalid result element2 in resultElement::Compare");
-  
-  nsMsgSearchAttribute attrib = re1->m_adapter->m_scope->m_frame->m_sortAttribute;
-  
-  const nsMsgSearchValue *v1 = re1->GetValueRef (attrib);
-  const nsMsgSearchValue *v2 = re2->GetValueRef (attrib);
-  
-  if (!v1 || !v2)
-    return ret; // search result doesn't contain the attrib we want to sort on
-  
-  switch (attrib)
-  {
-  case nsMsgSearchAttrib::Date:
-    {
-      // on Win 3.1, the requisite 'int' return type is a short, so use a 
-      // real time_t for comparison
-      time_t date = v1->u.date - v2->u.date;
-      if (date)
-        ret = ((long)date) < 0 ? -1 : 1;
-      else
-        ret = 0;
-    }
-    break;
-  case nsMsgSearchAttrib::Priority:
-    ret = v1->u.priority - v2->u.priority;
-    break;
-  case nsMsgSearchAttrib::MsgStatus:
-    {
-      // Here's a totally arbitrary sorting protocol for msg status
-      uint32 s1, s2;
-      
-      s1 = v1->u.msgStatus & ~MSG_FLAG_REPLIED;
-      s2 = v2->u.msgStatus & ~MSG_FLAG_REPLIED;
-      if (s1 || s2)
-        ret = s1 - s2;
-      else
-      {
-        s1 = v1->u.msgStatus & ~MSG_FLAG_FORWARDED;
-        s2 = v2->u.msgStatus & ~MSG_FLAG_FORWARDED;
-        if (s1 || s2)
-          ret = s1 - s2;
-        else
-        {
-          s1 = v1->u.msgStatus & ~MSG_FLAG_READ;
-          s2 = v2->u.msgStatus & ~MSG_FLAG_READ;
-          if (s1 || s2)
-            ret = s1 - s2;
-          else
-            // statuses don't contain any flags we're interested in, 
-            // so they're equal as far as we care
-            ret = 0;
-        }
-      }
-    }
-    break;
-  default:
-    if (attrib == nsMsgSearchAttrib::Subject)
-    {
-      // Special case for subjects, so "Re:foo" sorts under 'f' not 'r'
-      const char *s1 = v1->u.string;
-      const char *s2 = v2->u.string;
-      NS_MsgStripRE (&s1, NULL);
-      NS_MsgStripRE (&s2, NULL);
-      ret = PL_strcasecomp (s1, s2);
-    }
-    else
-      ret = strcasecomp (v1->u.string, v2->u.string);
-  }
-  // ### need different hack for this.
-  // qsort's default sort order is ascending, so in order to get descending
-  // behavior, we'll tell qsort a lie and reverse the comparison order.
-  if (re1->m_adapter->m_scope->m_frame->m_descending && ret != 0)
-    if (ret < 0)
-      ret = 1;
-    else
-      ret = -1;
-    
-    // <0 --> e1 less than e2
-    // 0  --> e1 equal to e2
-    // >0 --> e1 greater than e2
-#endif
-    return ret;
-}
-
-#ifdef HAVE_SEARCH_PORT
-MWContextType nsMsgResultElement::GetContextType ()
-{
-  MWContextType type=(MWContextType)0;
-  switch (m_adapter->m_scope->m_attribute)
-  {
-  case nsMsgSearchScopeMailFolder:
-    type = MWContextMailMsg;
-    break;
-  case nsMsgSearchScopeOfflineNewsgroup:    // added by mscott could be bug fix...
-  case nsMsgSearchScopeNewsgroup:
-  case nsMsgSearchScopeAllSearchableGroups:
-    type = MWContextNewsMsg;
-    break;
-  case nsMsgSearchScopeLdapDirectory:
-    type = MWContextBrowser;
-    break;
-  default:
-    NS_ASSERTION(PR_FALSE, "invalid scope"); // should never happen
-  }
-  return type;
-}
-
-#endif
 nsresult nsMsgResultElement::Open (void *window)
 {
-#ifdef HAVE_SEARCH_PORT
-  // ###phil this is a little ugly, but I'm not inclined to invest more in it
-  // until the libnet rework is done and I know what kind of context we'll end up with
-  
-  if (window)
-  {
-    if (m_adapter->m_scope->m_attribute != nsMsgSearchScopeLdapDirectory)
-    {
-      msgPane = (MSG_MessagePane *) window; 
-      PR_ASSERT (MSG_MESSAGEPANE == msgPane->GetPaneType());
-      return m_adapter->OpenResultElement (msgPane, this);
-    }
-    else
-    {
-      context = (MWContext*) window;
-      PR_ASSERT (MWContextBrowser == context->type);
-      msg_SearchLdap *thisAdapter = (msg_SearchLdap*) m_adapter;
-      return thisAdapter->OpenResultElement (context, this);
-    }
-  }
-#endif
   return NS_ERROR_NULL_POINTER;
 }
 
