@@ -40,6 +40,7 @@
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
 #include "nsISupportsArray.h"
+#include "nsSupportsPrimitives.h"
 #include "nsNetUtil.h"
 
 #include "prmem.h"
@@ -80,7 +81,6 @@
 #include "nsEscape.h"
 
 #include "nsIUserInfo.h"
-#include "nsIAbUpgrader.h"
 #include "nsIAddressBook.h"
 #include "nsAbBaseCID.h"
 
@@ -96,8 +96,6 @@
 #define DEBUG_MIGRATOR 1
 #endif
 
-static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
-
 #define IMAP_SCHEMA "imap:/"
 #define IMAP_SCHEMA_LENGTH 6
 #define MAILBOX_SCHEMA "mailbox:/"
@@ -236,7 +234,7 @@ static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
   { \
     nsresult macro_rv; \
     nsCOMPtr <nsIFileSpec>macro_spec;	\
-    macro_rv = m_prefs->GetFilePref(PREFNAME, getter_AddRefs(macro_spec)); \
+    macro_rv = m_prefs->GetComplexValue(PREFNAME, NS_GET_IID(nsIFileSpec), getter_AddRefs(macro_spec)); \
     if (NS_SUCCEEDED(macro_rv)) { \
 	char *macro_oldStr = nsnull; \
 	macro_rv = macro_spec->GetUnixStyleFilePath(&macro_oldStr);	\
@@ -256,9 +254,9 @@ static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
     nsresult macro_rv; \
     nsCOMPtr <nsIFileSpec>macro_spec;	\
 	char *macro_val = nsnull; \
-	macro_rv = m_prefs->CopyCharPref(PREFNAME, &macro_val); \
+	macro_rv = m_prefs->GetCharPref(PREFNAME, &macro_val); \
 	if (NS_SUCCEEDED(macro_rv) && macro_val && PL_strlen(macro_val)) { \
-		macro_rv = m_prefs->GetFilePref(PREFNAME, getter_AddRefs(macro_spec)); \
+		macro_rv = m_prefs->GetComplexValue(PREFNAME, NS_GET_IID(nsIFileSpec), getter_AddRefs(macro_spec)); \
 		if (NS_SUCCEEDED(macro_rv)) { \
 			char *macro_oldStr = nsnull; \
 			macro_rv = macro_spec->GetUnixStyleFilePath(&macro_oldStr);	\
@@ -279,9 +277,9 @@ static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
     nsresult macro_rv; \
     nsCOMPtr <nsILocalFile> macro_file; \
     char *macro_oldStr = nsnull; \
-    macro_rv = m_prefs->CopyCharPref(PREFNAME, &macro_oldStr); \
+    macro_rv = m_prefs->GetCharPref(PREFNAME, &macro_oldStr); \
     if (NS_SUCCEEDED(macro_rv) && macro_oldStr && PL_strlen(macro_oldStr)) { \
-      macro_rv = m_prefs->GetFileXPref(PREFNAME, getter_AddRefs(macro_file)); \
+      macro_rv = m_prefs->GetComplexValue(PREFNAME, NS_GET_IID(nsILocalFile), getter_AddRefs(macro_file)); \
       if (NS_SUCCEEDED(macro_rv)) { \
         MACRO_OBJECT->MACRO_METHOD(macro_file); \
       } \
@@ -293,7 +291,7 @@ static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
   { \
     nsresult macro_rv; \
     char *macro_oldStr = nsnull; \
-    macro_rv = m_prefs->CopyCharPref(PREFNAME, &macro_oldStr); \
+    macro_rv = m_prefs->GetCharPref(PREFNAME, &macro_oldStr); \
     if (NS_SUCCEEDED(macro_rv)) { \
       MACRO_OBJECT->MACRO_METHOD(macro_oldStr); \
     } \
@@ -304,8 +302,10 @@ static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
   { \
     nsresult macro_rv; \
     PRUnichar *macro_oldStr = nsnull; \
-    macro_rv = m_prefs->CopyUnicharPref(PREFNAME, &macro_oldStr); \
+    nsCOMPtr<nsISupportsString> macro_tmpstr; \
+    macro_rv = m_prefs->GetComplexValue(PREFNAME, NS_GET_IID(nsISupportsString), getter_AddRefs(macro_tmpstr)); \
     if (NS_SUCCEEDED(macro_rv)) { \
+      macro_tmpstr->ToString(&macro_oldStr); \
       MACRO_OBJECT->MACRO_METHOD(macro_oldStr); \
     } \
     PR_FREEIF(macro_oldStr); \
@@ -337,7 +337,7 @@ static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
     char prefName[BUF_STR_LEN]; \
     char *macro_oldStr = nsnull; \
     PR_snprintf(prefName, BUF_STR_LEN, PREFFORMATSTR, PREFFORMATVALUE); \
-    macro_rv = m_prefs->CopyCharPref(prefName, &macro_oldStr); \
+    macro_rv = m_prefs->GetCharPref(prefName, &macro_oldStr); \
     if (NS_SUCCEEDED(macro_rv)) { \
       INCOMINGSERVERPTR->INCOMINGSERVERMETHOD(macro_oldStr); \
     } \
@@ -416,7 +416,7 @@ nsresult nsMessengerMigrator::Init()
 nsresult 
 nsMessengerMigrator::Shutdown()
 {
-	m_prefs = nsnull;
+  m_prefs = nsnull;
 
   m_haveShutdown = PR_TRUE;
   return NS_OK;
@@ -428,7 +428,7 @@ nsMessengerMigrator::getPrefService()
   nsresult rv = NS_OK;
 
   if (!m_prefs) {
-    m_prefs = do_GetService(kPrefServiceCID, &rv);
+    m_prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
   }
 
   if (NS_FAILED(rv)) return rv;
@@ -489,7 +489,7 @@ nsMessengerMigrator::ProceedWithMigration()
 	) {
     // if they were using pop or movemail, "mail.pop_name" must have been set
     // otherwise, they don't really have anything to migrate
-    rv = m_prefs->CopyCharPref(PREF_4X_MAIL_POP_NAME, &prefvalue);
+    rv = m_prefs->GetCharPref(PREF_4X_MAIL_POP_NAME, &prefvalue);
     if (NS_SUCCEEDED(rv)) {
 	    if (!prefvalue || !*prefvalue) {
 	      rv = NS_ERROR_FAILURE;
@@ -499,7 +499,7 @@ nsMessengerMigrator::ProceedWithMigration()
   else if (m_oldMailType == IMAP_4X_MAIL_TYPE) {
     // if they were using imap, "network.hosts.imap_servers" must have been set
     // otherwise, they don't really have anything to migrate
-    rv = m_prefs->CopyCharPref(PREF_4X_NETWORK_HOSTS_IMAP_SERVER, &prefvalue);
+    rv = m_prefs->GetCharPref(PREF_4X_NETWORK_HOSTS_IMAP_SERVER, &prefvalue);
     if (NS_SUCCEEDED(rv)) {
 	    if (!prefvalue || !*prefvalue) {
 	      rv = NS_ERROR_FAILURE;
@@ -553,7 +553,7 @@ nsMessengerMigrator::CreateLocalMailAccount(PRBool migrating)
   // for a new profile, that pref won't be set.
   if (migrating) {
     nsCOMPtr<nsILocalFile> localFile;
-    rv = m_prefs->GetFileXPref(PREF_MAIL_DIRECTORY, getter_AddRefs(localFile));
+    rv = m_prefs->GetComplexValue(PREF_MAIL_DIRECTORY, NS_GET_IID(nsILocalFile), getter_AddRefs(localFile));
     if (NS_SUCCEEDED(rv))
         mailDir = localFile;
   }
@@ -730,18 +730,25 @@ nsMessengerMigrator::UpgradePrefs()
     rv = MigrateNewsAccounts(identity);
     if (NS_FAILED(rv)) return rv;
 
-    // this will upgrade the ldap prefs
+    // this will migrate the ldap prefs
 #if defined(MOZ_LDAP_XPCOM)
     nsCOMPtr <nsILDAPPrefsService> ldapPrefsService = do_GetService("@mozilla.org/ldapprefs-service;1", &rv);
+
+    rv = ldapPrefsService->MigratePrefsIfNeeded();
+    NS_ENSURE_SUCCESS(rv, rv);
 #endif    
     rv = MigrateAddressBookPrefs();
     NS_ENSURE_SUCCESS(rv,rv);
 
-    rv = MigrateAddressBooks();
-    if (NS_FAILED(rv)) return rv;
+    // XXX: No address book migration for 4.x see bug 35509
+
+    m_prefs->ClearUserPref(PREF_4X_MAIL_POP_PASSWORD); // intentionally ignore the return value
 
     // we're done migrating, let's save the prefs
-    rv = m_prefs->SavePrefFile(nsnull);
+    nsCOMPtr<nsIPrefService> prefService(do_QueryInterface(m_prefs, &rv));
+    if (NS_FAILED(rv)) return rv;
+
+    rv = prefService->SavePrefFile(nsnull);
     if (NS_FAILED(rv)) return rv;
 
 	// remove the temporary identity we used for migration purposes
@@ -757,8 +764,8 @@ nsMessengerMigrator::SetUsernameIfNecessary()
     nsresult rv;
     nsXPIDLCString usernameIn4x;
 
-    rv = m_prefs->CopyCharPref(PREF_4X_MAIL_IDENTITY_USERNAME, getter_Copies(usernameIn4x));
-    if (NS_SUCCEEDED(rv) && ((const char *)usernameIn4x) && (PL_strlen((const char *)usernameIn4x))) {
+    rv = m_prefs->GetCharPref(PREF_4X_MAIL_IDENTITY_USERNAME, getter_Copies(usernameIn4x));
+    if (NS_SUCCEEDED(rv) && !usernameIn4x.IsEmpty()) {
         return NS_OK;
     }
 
@@ -775,7 +782,12 @@ nsMessengerMigrator::SetUsernameIfNecessary()
         return NS_OK;
     }
 
-    rv = m_prefs->SetUnicharPref(PREF_4X_MAIL_IDENTITY_USERNAME, (const PRUnichar *)fullnameFromSystem);
+    nsCOMPtr<nsISupportsString> str(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv));
+    if (NS_SUCCEEDED(rv)) {
+        str->SetData(fullnameFromSystem);
+        rv = m_prefs->SetComplexValue(PREF_4X_MAIL_IDENTITY_USERNAME, NS_GET_IID(nsISupportsString), str);
+    }
+
     return rv;
 }
 
@@ -860,10 +872,10 @@ nsMessengerMigrator::SetNewsCopiesAndFolders(nsIMsgIdentity *identity)
     nsXPIDLCString pop_username;
     nsXPIDLCString pop_hostname;
 
-    rv = m_prefs->CopyCharPref(PREF_4X_MAIL_POP_NAME,getter_Copies(pop_username));
+    rv = m_prefs->GetCharPref(PREF_4X_MAIL_POP_NAME,getter_Copies(pop_username));
     if (NS_FAILED(rv)) return rv;
 
-    rv = m_prefs->CopyCharPref(PREF_4X_NETWORK_HOSTS_POP_SERVER, getter_Copies(pop_hostname));
+    rv = m_prefs->GetCharPref(PREF_4X_NETWORK_HOSTS_POP_SERVER, getter_Copies(pop_hostname));
     if (NS_FAILED(rv)) return rv;
 
 	CONVERT_4X_URI(identity, PR_TRUE /* for news */, pop_username.get(), pop_hostname.get(), DEFAULT_4X_SENT_FOLDER_NAME,GetFccFolder,SetFccFolder,DEFAULT_FCC_FOLDER_PREF_NAME)
@@ -874,7 +886,7 @@ nsMessengerMigrator::SetNewsCopiesAndFolders(nsIMsgIdentity *identity)
   else if (m_oldMailType == MOVEMAIL_4X_MAIL_TYPE) {
     nsXPIDLCString pop_username;
 
-	rv = m_prefs->CopyCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(pop_username));
+	rv = m_prefs->GetCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(pop_username));
     if (NS_FAILED(rv)) return rv;
 
 	CONVERT_4X_URI(identity, PR_TRUE /* for news */, pop_username.get(), MOVEMAIL_FAKE_HOST_NAME, DEFAULT_4X_SENT_FOLDER_NAME,GetFccFolder,SetFccFolder,DEFAULT_FCC_FOLDER_PREF_NAME)
@@ -935,7 +947,7 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
   }
 
   nsXPIDLCString default_pref_value;
-  rv = m_prefs->CopyCharPref(default_pref_name, getter_Copies(default_pref_value));
+  rv = m_prefs->GetCharPref(default_pref_name, getter_Copies(default_pref_value));
   NS_ENSURE_SUCCESS(rv,rv);
 
   // if the old pref value was "", old_uri will be default value (mail.identity.default.fcc_folder)
@@ -998,7 +1010,7 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
 		prefname = PR_smprintf("mail.imap.server.%s.userName", hostname.get());
 		if (!prefname) return NS_ERROR_FAILURE;
 
-		rv = m_prefs->CopyCharPref(prefname, &imap_username);
+		rv = m_prefs->GetCharPref(prefname, &imap_username);
 		PR_FREEIF(prefname);
 		if (NS_FAILED(rv) || !imap_username || !*imap_username) {
 			*new_uri = PR_smprintf("");
@@ -1035,7 +1047,7 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
   char *usernameAtHostname = nsnull;
   nsCOMPtr <nsIFileSpec> mail_dir;
   char *mail_directory_value = nsnull;
-  rv = m_prefs->GetFilePref(PREF_PREMIGRATION_MAIL_DIRECTORY, getter_AddRefs(mail_dir));
+  rv = m_prefs->GetComplexValue(PREF_PREMIGRATION_MAIL_DIRECTORY, NS_GET_IID(nsIFileSpec), getter_AddRefs(mail_dir));
   if (NS_SUCCEEDED(rv)) {
 	rv = mail_dir->GetUnixStyleFilePath(&mail_directory_value);
   }
@@ -1045,7 +1057,7 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
 #endif
     PR_FREEIF(mail_directory_value);
 
-    rv = m_prefs->GetFilePref(PREF_MAIL_DIRECTORY, getter_AddRefs(mail_dir));
+    rv = m_prefs->GetComplexValue(PREF_MAIL_DIRECTORY, NS_GET_IID(nsIFileSpec), getter_AddRefs(mail_dir));
     if (NS_SUCCEEDED(rv)) {
 	rv = mail_dir->GetUnixStyleFilePath(&mail_directory_value);
     } 
@@ -1060,10 +1072,10 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
     nsXPIDLCString pop_username;
     nsXPIDLCString pop_hostname;
 
-    rv = m_prefs->CopyCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(pop_username));
+    rv = m_prefs->GetCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(pop_username));
     if (NS_FAILED(rv)) return rv;
 
-    rv = m_prefs->CopyCharPref(PREF_4X_NETWORK_HOSTS_POP_SERVER, getter_Copies(pop_hostname));
+    rv = m_prefs->GetCharPref(PREF_4X_NETWORK_HOSTS_POP_SERVER, getter_Copies(pop_hostname));
     if (NS_FAILED(rv)) return rv;
 
     // Need to escape hostname in case it contains spaces.
@@ -1083,7 +1095,7 @@ nsMessengerMigrator::Convert4XUri(const char *old_uri, PRBool for_news, const ch
   else if (m_oldMailType == MOVEMAIL_4X_MAIL_TYPE) {
 	nsXPIDLCString movemail_username;
 
-	rv = m_prefs->CopyCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(movemail_username));
+	rv = m_prefs->GetCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(movemail_username));
 	if (NS_FAILED(rv)) return rv; 
 	
     nsXPIDLCString escaped_movemail_username;
@@ -1191,7 +1203,7 @@ nsMessengerMigrator::MigrateLocalMailAccount()
   // if the "mail.directory" pref is set, use that.
   // if they used -installer, this pref will point to where their files got copied
   nsCOMPtr<nsILocalFile> localFile;
-  rv = m_prefs->GetFileXPref(PREF_MAIL_DIRECTORY, getter_AddRefs(localFile));
+  rv = m_prefs->GetComplexValue(PREF_MAIL_DIRECTORY, NS_GET_IID(nsILocalFile), getter_AddRefs(localFile));
   if (NS_SUCCEEDED(rv))
     mailDir = localFile;
   
@@ -1294,7 +1306,7 @@ nsMessengerMigrator::MigrateMovemailAccount(nsIMsgIdentity *identity)
   // get the pop username
   // movemail used the pop username in 4.x
   nsXPIDLCString username;
-  rv = m_prefs->CopyCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(username));
+  rv = m_prefs->GetCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(username));
   if (NS_FAILED(rv)) return rv;
 
   //
@@ -1318,7 +1330,7 @@ nsMessengerMigrator::MigrateMovemailAccount(nsIMsgIdentity *identity)
   if (NS_FAILED(rv)) return rv;
 
   // if they used -installer, this pref will point to where their files got copied
-  rv = m_prefs->GetFilePref(PREF_MAIL_DIRECTORY, getter_AddRefs(mailDir));
+  rv = m_prefs->GetComplexValue(PREF_MAIL_DIRECTORY, NS_GET_IID(nsIFileSpec), getter_AddRefs(mailDir));
   // create the directory structure for old 4.x pop mail
   // under <profile dir>/Mail/movemail or
   // <"mail.directory" pref>/movemail
@@ -1404,12 +1416,12 @@ nsMessengerMigrator::MigratePopAccount(nsIMsgIdentity *identity)
 
   // get the pop username
   nsXPIDLCString username;
-  rv = m_prefs->CopyCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(username));
+  rv = m_prefs->GetCharPref(PREF_4X_MAIL_POP_NAME, getter_Copies(username));
   if (NS_FAILED(rv)) return rv;
 
   // get the hostname and port
   nsXPIDLCString hostAndPort;
-  rv = m_prefs->CopyCharPref(PREF_4X_NETWORK_HOSTS_POP_SERVER,
+  rv = m_prefs->GetCharPref(PREF_4X_NETWORK_HOSTS_POP_SERVER,
                              getter_Copies(hostAndPort));
   if (NS_FAILED(rv)) return rv;
 
@@ -1461,7 +1473,7 @@ nsMessengerMigrator::MigratePopAccount(nsIMsgIdentity *identity)
   // if the "mail.directory" pref is set, use that.
   // if they used -installer, this pref will point to where their files got copied
   nsCOMPtr<nsILocalFile> localFile;
-  rv = m_prefs->GetFileXPref(PREF_MAIL_DIRECTORY, getter_AddRefs(localFile));
+  rv = m_prefs->GetComplexValue(PREF_MAIL_DIRECTORY, NS_GET_IID(nsILocalFile), getter_AddRefs(localFile));
   if (NS_SUCCEEDED(rv))
     mailDir = localFile;
   
@@ -1590,12 +1602,9 @@ nsMessengerMigrator::MigrateOldMailPrefs(nsIMsgIncomingServer * server)
   rv = server->SetRememberPassword(PR_FALSE);
   if (NS_FAILED(rv)) return rv;
   
-#ifdef CAN_UPGRADE_4x_PASSWORDS
-  MIGRATE_SIMPLE_STR_PREF(PREF_4X_MAIL_POP_PASSWORD,server,SetPassword)
-#else
   rv = server->SetPassword(nsnull);
   if (NS_FAILED(rv)) return rv;
-#endif /* CAN_UPGRADE_4x_PASSWORDS */
+
   MIGRATE_SIMPLE_BOOL_PREF(PREF_4X_MAIL_CHECK_NEW_MAIL,server,SetDoBiff)
   MIGRATE_SIMPLE_INT_PREF(PREF_4X_MAIL_CHECK_TIME,server,SetBiffMinutes)
   MIGRATE_SIMPLE_BOOL_PREF(PREF_4X_MAIL_POP3_GETS_NEW_MAIL,server,SetDownloadOnBiff)
@@ -1621,7 +1630,7 @@ nsMessengerMigrator::MigrateImapAccounts(nsIMsgIdentity *identity)
   rv = getPrefService();
   if (NS_FAILED(rv)) return rv;
 
-  rv = m_prefs->CopyCharPref(PREF_4X_NETWORK_HOSTS_IMAP_SERVER, &hostList);
+  rv = m_prefs->GetCharPref(PREF_4X_NETWORK_HOSTS_IMAP_SERVER, &hostList);
   if (NS_FAILED(rv)) return rv;
   
   if (!hostList || !*hostList) return NS_OK;  // NS_ERROR_FAILURE?
@@ -1670,7 +1679,7 @@ nsMessengerMigrator::MigrateImapAccount(nsIMsgIdentity *identity, const char *ho
   nsXPIDLCString username;
   char *imapUsernamePref =
     PR_smprintf("mail.imap.server.%s.userName", hostAndPort);
-  rv = m_prefs->CopyCharPref(imapUsernamePref, getter_Copies(username));
+  rv = m_prefs->GetCharPref(imapUsernamePref, getter_Copies(username));
   PR_FREEIF(imapUsernamePref);
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -1756,7 +1765,7 @@ nsMessengerMigrator::MigrateImapAccount(nsIMsgIdentity *identity, const char *ho
   // if they used -installer, this pref will point to where their files got copied
   // if the "mail.imap.root_dir" pref is set, use that.
   nsCOMPtr<nsILocalFile> localFile;
-  rv = m_prefs->GetFileXPref(PREF_IMAP_DIRECTORY, getter_AddRefs(localFile));
+  rv = m_prefs->GetComplexValue(PREF_IMAP_DIRECTORY, NS_GET_IID(nsILocalFile), getter_AddRefs(localFile));
   if (NS_SUCCEEDED(rv))
     imapMailDir = localFile;
   
@@ -1855,12 +1864,10 @@ nsMessengerMigrator::MigrateOldImapPrefs(nsIMsgIncomingServer *server, const cha
   //MIGRATE_BOOL_PREF("mail.imap.server.%s.remember_password",hostAndPort,server,SetRememberPassword)
   rv = server->SetRememberPassword(PR_FALSE);
   if (NS_FAILED(rv)) return rv;
-#ifdef CAN_UPGRADE_4x_PASSWORDS
-  MIGRATE_STR_PREF("mail.imap.server.%s.password",hostAndPort,server,SetPassword)
-#else 
+
   rv = server->SetPassword(nsnull);
   if (NS_FAILED(rv)) return rv;
-#endif /* CAN_UPGRADE_4x_PASSWORDS */
+
   // upgrade the imap incoming server specific prefs
   MIGRATE_BOOL_PREF("mail.imap.server.%s.check_new_mail",hostAndPort,server,SetDoBiff)
   MIGRATE_INT_PREF("mail.imap.server.%s.check_time",hostAndPort,server,SetBiffMinutes)
@@ -1885,203 +1892,6 @@ nsMessengerMigrator::MigrateOldImapPrefs(nsIMsgIncomingServer *server, const cha
   return NS_OK;
 }
 
-static PRBool charEndsWith(const char *str, const char *endStr)
-{
-    PRUint32 endStrLen = PL_strlen(endStr);
-    PRUint32 strLen = PL_strlen(str);
-   
-    if (strLen < endStrLen) return PR_FALSE;
-
-    PRUint32 pos = strLen - endStrLen;
-    if (PL_strncmp(str + pos, endStr, endStrLen) == 0) {
-        return PR_TRUE;
-    }
-    else {
-        return PR_FALSE;
-    }
-}
-
-#define ADDRESSBOOK_PREF_NAME_ROOT "ldap_2.servers."
-#define ADDRESSBOOK_PREF_NAME_SUFFIX ".filename"
-#define ADDRESSBOOK_PREF_CSID_SUFFIX ".csid"
-#define ADDRESSBOOK_PREF_VALUE_4x_SUFFIX ".na2"
-#define ADDRESSBOOK_PREF_VALUE_5x_SUFFIX ".mab"
-#define TEMP_LDIF_FILE_SUFFIX ".ldif"
-
-#if defined(DEBUG_sspitzer_) || defined(DEBUG_seth_)
-#define DEBUG_AB_MIGRATION 1
-#endif
-
-void
-nsMessengerMigrator::migrateAddressBookPrefEnum(const char *aPref, void *aClosure)
-{
-  nsresult rv = NS_OK;
-  nsIPref *prefs = (nsIPref *)aClosure;
-
-#ifdef DEBUG_AB_MIGRATION
-  printf("investigate pref: %s\n",aPref);
-#endif
-  nsXPIDLCString abFileName;
-  // we only care about ldap_2.servers.*.filename" prefs
-  if (!charEndsWith(aPref, ADDRESSBOOK_PREF_NAME_SUFFIX)) return;
-      
-  // check for pab without a filename and if so, set it to pab.na2.
-  // This is an ugly hack for installations that haven't set 
-  // pab.filename.
-      rv = prefs->CopyCharPref(DEFAULT_PAB_FILENAME_PREF_NAME, getter_Copies(abFileName));
-      if (NS_FAILED(rv))
-      {
-        // pab.filename not set - set it to pab.na2 and pretend aPref is it.
-        prefs->SetCharPref(DEFAULT_PAB_FILENAME_PREF_NAME, "pab.na2");
-        aPref = "ldap_2.servers.pab.filename";
-      }
-
-  // check if this really is an ldap server, in which case, we're not going to migrate
-  // the na2 file for now, since it confuses RDF when the import code adds this
-  // server as a personal address book. I'd like to fix that but it's tricky
-  // and the .na2 file for ldap servers should only be for offline data.
-
-  nsCAutoString serverPrefName(aPref);
-  PRInt32 fileNamePos = serverPrefName.Find(".filename");
-  serverPrefName.Truncate(fileNamePos + 1);
-  serverPrefName.Append("serverName");
-  nsXPIDLCString serverName;
-  rv = prefs->CopyCharPref(serverPrefName.get(), getter_Copies(serverName));
-  if (NS_SUCCEEDED(rv) && !serverName.IsEmpty())
-    return; // skip this - it's an ldap server
-      
-  rv = prefs->CopyCharPref(aPref,getter_Copies(abFileName));
-  NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed: failed to get ab filename");
-  if (NS_FAILED(rv)) return;
-  
-  NS_ASSERTION(((const char *)abFileName), "ERROR:  empty addressbook file name");
-  if (!((const char *)abFileName)) return;
-
-  NS_ASSERTION(PL_strlen((const char *)abFileName),"ERROR:  empty addressbook file name");
-  if (!(PL_strlen((const char *)abFileName))) return;
-
-#ifdef DEBUG_AB_MIGRATION
-  printf("pref value: %s\n",(const char *)abFileName);
-#endif /* DEBUG_AB_MIGRATION */
-  // if this a 5.x addressbook file name, skip it.
-  if (charEndsWith((const char *)abFileName, ADDRESSBOOK_PREF_VALUE_5x_SUFFIX)) return;
-    
-  nsCAutoString abName;
-  abName = (const char *)abFileName;
-  PRInt32 len = abName.Length();
-  PRInt32 suffixLen = PL_strlen(ADDRESSBOOK_PREF_VALUE_4x_SUFFIX);
-  NS_ASSERTION(len > suffixLen, "ERROR: bad length of addressbook filename");
-  if (len <= suffixLen) return;
-  
-  abName.SetLength(len - suffixLen);
-
-  // get 5.0 profile root.
-  nsCOMPtr <nsIFile> ab4xFile;
-  nsCOMPtr <nsIFileSpec> ab4xFileSpec;
-  nsCOMPtr <nsIFile> tmpLDIFFile;
-  nsCOMPtr <nsIFileSpec> tmpLDIFFileSpec;
-
-#ifdef DEBUG_AB_MIGRATION
-  printf("turn %s%s into %s%s\n", (const char *)abName,ADDRESSBOOK_PREF_VALUE_4x_SUFFIX,(const char *)abName,TEMP_LDIF_FILE_SUFFIX);
-#endif /* DEBUG_AB_MIGRATION */
-
-  rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(ab4xFile));
-  NS_ASSERTION(NS_SUCCEEDED(rv) && ab4xFile,"ab migration failed: failed to get profile dir");
-  if (NS_FAILED(rv) || !ab4xFile) return;
-  
-  rv = ab4xFile->AppendNative(nsDependentCString(abFileName));
-  NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed:  failed to append filename");
-  if (NS_FAILED(rv)) return;
-
-  // TODO: Change users of nsIFileSpec to nsIFile and avoid this.
-  rv = NS_NewFileSpecFromIFile(ab4xFile, getter_AddRefs(ab4xFileSpec));
-  if (NS_FAILED(rv)) return;
-
-  rv = NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(tmpLDIFFile));
-  if (NS_FAILED(rv) || !tmpLDIFFile) return;
-
-  // do the migration in a subdirectory of temp, to prevent 
-  // collision (between multiple users), deleting TMPDIR
-  // and privacy issues (where the temp ldif files are readable)
-  rv = tmpLDIFFile->AppendNative(NS_LITERAL_CSTRING("addr-migrate"));
-  if (NS_FAILED(rv) || !tmpLDIFFile) return;
-
-  rv = tmpLDIFFile->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700);
-  if (NS_FAILED(rv) || !tmpLDIFFile) return;
-
-  // TODO: Change users of nsIFileSpec to nsIFile and avoid this.
-  rv = NS_NewFileSpecFromIFile(tmpLDIFFile, getter_AddRefs(tmpLDIFFileSpec));
-  if (NS_FAILED(rv)) return;
-
-  // get the csid from the prefs
-  nsCAutoString csidPrefName;
-  csidPrefName = ADDRESSBOOK_PREF_NAME_ROOT;
-  csidPrefName += abName; 
-  csidPrefName += ADDRESSBOOK_PREF_CSID_SUFFIX;
-  nsXPIDLCString csidPrefValue;
-  rv = prefs->CopyCharPref(csidPrefName.get(),getter_Copies(csidPrefValue));
-  if (NS_FAILED(rv)) { 
-	// if we fail to get the pref value, set it to "", which will
-	// later cause us to use the system charset
-	csidPrefValue.Adopt(nsCRT::strdup(""));
-  }
-
-  nsCOMPtr <nsIAbUpgrader> abUpgrader = do_GetService(NS_AB4xUPGRADER_CONTRACTID, &rv);
-  NS_ASSERTION(NS_SUCCEEDED(rv) && abUpgrader, "failed to get upgrader");
-  if (NS_FAILED(rv) || !abUpgrader) return;
-  rv = abUpgrader->SetCurrentCharset((const char *)csidPrefValue);
-  NS_ASSERTION(NS_SUCCEEDED(rv), "failed to set the current char set");
-  if (NS_FAILED(rv)) return;
-
-// HACK:  I need to rename pab.ldif -> abook.ldif, because a bunch of places are hacked to point to abook.mab, and when I import abook.ldif it will create abook.mab. this is a temporary hack and will go away soon.
-  if (!PL_strcmp(abName.get(),"pab")) {
-	abName = "abook";
-  }
-
-  nsCAutoString ldifFileName;
-  ldifFileName = abName;
-  ldifFileName += TEMP_LDIF_FILE_SUFFIX;
-  rv = tmpLDIFFileSpec->AppendRelativeUnixPath(ldifFileName.get());
-  NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed: failed to append filename");
-  if (NS_FAILED(rv)) return;
-     
-  nsCOMPtr <nsIAddressBook> ab = do_CreateInstance(NS_ADDRESSBOOK_CONTRACTID, &rv);
-  NS_ASSERTION(NS_SUCCEEDED(rv) && ab, "failed to get address book");
-  if (NS_FAILED(rv) || !ab) return;
-
-  rv = ab->ConvertNA2toLDIF(ab4xFileSpec, tmpLDIFFileSpec);
-  NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed: failed to convert na2 to ldif");
-  if (NS_FAILED(rv)) return;
-  
-  rv = ab->ConvertLDIFtoMAB(tmpLDIFFileSpec, PR_TRUE /* migrating */, nsnull, PR_FALSE, PR_FALSE /* Importing Comm4x */);
-  NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration filed:  failed to convert ldif to mab\n");
-  if (NS_FAILED(rv)) return;
- 
-  // this sucks.
-  // ConvertLDIFtoMAB should set this pref value for us. 
-#ifdef DEBUG_AB_MIGRATION
-  printf("set %s the pref to %s%s\n",aPref,(const char *)abName,ADDRESSBOOK_PREF_VALUE_5x_SUFFIX);
-#endif /* DEBUG_AB_MIGRATION */
-  
-  nsCAutoString newPrefValue;
-  newPrefValue = abName;
-  newPrefValue += ADDRESSBOOK_PREF_VALUE_5x_SUFFIX;
-  rv = prefs->SetCharPref(aPref,newPrefValue.get());
-  NS_ASSERTION(NS_SUCCEEDED(rv),"ab migration failed: failed to set pref");
-  if (NS_FAILED(rv)) return;
- 
-#ifdef DEBUG_AB_MIGRATION
-  printf("remove the tmp file\n");
-#endif /* DEBUG_AB_MIGRATION */
-  rv = ab4xFile->Remove(PR_FALSE);
-  NS_ASSERTION(NS_SUCCEEDED(rv),"failed to delete the na2 file");
-  if (NS_FAILED(rv)) return;
-  rv = tmpLDIFFile->Remove(PR_TRUE);
-  NS_ASSERTION(NS_SUCCEEDED(rv),"failed to delete the temp ldif file");
-  
-  return;
-}
-
 nsresult 
 nsMessengerMigrator::MigrateAddressBookPrefs()
 {
@@ -2097,21 +1907,6 @@ nsMessengerMigrator::MigrateAddressBookPrefs()
   return NS_OK;
 }
 
-nsresult
-nsMessengerMigrator::MigrateAddressBooks()
-{
-  nsresult rv = NS_OK;
-
-  nsCOMPtr <nsIAbUpgrader> abUpgrader = do_GetService(NS_AB4xUPGRADER_CONTRACTID, &rv);
-  if (NS_FAILED(rv) || !abUpgrader) {
-    printf("the addressbook migrator is only in the commercial builds.\n");
-    return NS_OK;
-  }
-
-  rv = m_prefs->EnumerateChildren(ADDRESSBOOK_PREF_NAME_ROOT, migrateAddressBookPrefEnum, (void *)m_prefs);
-  return rv;
-}
-
 #ifdef USE_NEWSRC_MAP_FILE
 #define NEWSRC_MAP_FILE_COOKIE "netscape-newsrc-map-file"
 #endif /* USE_NEWSRC_MAP_FILE */
@@ -2136,7 +1931,7 @@ nsMessengerMigrator::MigrateNewsAccounts(nsIMsgIdentity *identity)
 #ifdef USE_NEWSRC_MAP_FILE
     // if they used -installer, this pref will point to where their files got copied
     nsCOMPtr<nsILocalFile> localFile;
-    rv = m_prefs->GetFileXPref(PREF_NEWS_DIRECTORY, getter_AddRefs(localFile));
+    rv = m_prefs->GetComplexValue(PREF_NEWS_DIRECTORY, NS_GET_IID(nsILocalFile), getter_AddRefs(localFile));
     if (NS_SUCCEEDED(rv))
         newsDir = localFile;
 #else
@@ -2173,7 +1968,7 @@ nsMessengerMigrator::MigrateNewsAccounts(nsIMsgIdentity *identity)
 
     // for each news server in the fat file call MigrateNewsAccount();
 	char buffer[512];
-	char psuedo_name[512];
+	char pseudo_name[512];
 	char filename[512];
 	char is_newsgroup[512];
 	PRBool ok;
@@ -2221,8 +2016,8 @@ nsMessengerMigrator::MigrateNewsAccounts(nsIMsgIdentity *identity)
 		is_newsgroup[0]='\0';
 		
 		for (i = 0, p = buffer; *p && *p != '\t' && i < 500; p++, i++)
-			psuedo_name[i] = *p;
-		psuedo_name[i] = '\0';
+			pseudo_name[i] = *p;
+		pseudo_name[i] = '\0';
 		if (*p) 
 		{
 			for (i = 0, p++; *p && *p != '\t' && i < 500; p++, i++)
@@ -2253,29 +2048,29 @@ nsMessengerMigrator::MigrateNewsAccounts(nsIMsgIdentity *identity)
 			rcFile += filename;
 #endif /* NEWS_FAT_STORES_ABSOLUTE_NEWSRC_FILE_PATHS */
 
-			// psuedo-name is of the form newsrc-<host> or snewsrc-<host>.  
-			if (PL_strncmp(PSUEDO_NAME_PREFIX,psuedo_name,PL_strlen(PSUEDO_NAME_PREFIX)) == 0) {
+			// pseudo-name is of the form newsrc-<host> or snewsrc-<host>.  
+			if (PL_strncmp(PSEUDO_NAME_PREFIX,pseudo_name,PL_strlen(PSEUDO_NAME_PREFIX)) == 0) {
                 // check that there is a hostname to get after the "newsrc-" part
-                NS_ASSERTION(PL_strlen(psuedo_name) > PL_strlen(PSUEDO_NAME_PREFIX), "psuedo_name is too short");
-                if (PL_strlen(psuedo_name) <= PL_strlen(PSUEDO_NAME_PREFIX)) {
+                NS_ASSERTION(PL_strlen(pseudo_name) > PL_strlen(PSEUDO_NAME_PREFIX), "pseudo_name is too short");
+                if (PL_strlen(pseudo_name) <= PL_strlen(PSEUDO_NAME_PREFIX)) {
                     return NS_ERROR_FAILURE;
                 }
 
-                char *hostname = psuedo_name + PL_strlen(PSUEDO_NAME_PREFIX);
+                char *hostname = pseudo_name + PL_strlen(PSEUDO_NAME_PREFIX);
                 rv = MigrateNewsAccount(identity, hostname, rcFile, newsHostsDir, PR_FALSE /* isSecure */);
                 if (NS_FAILED(rv)) {
                     // failed to migrate.  bail out
                     return rv;
                 }
 			}
-            else if (PL_strncmp(PSUEDO_SECURE_NAME_PREFIX,psuedo_name,PL_strlen(PSUEDO_SECURE_NAME_PREFIX)) == 0) {
+            else if (PL_strncmp(PSEUDO_SECURE_NAME_PREFIX,pseudo_name,PL_strlen(PSEUDO_SECURE_NAME_PREFIX)) == 0) {
                 // check that there is a hostname to get after the "snewsrc-" part
-                NS_ASSERTION(PL_strlen(psuedo_name) > PL_strlen(PSUEDO_SECURE_NAME_PREFIX), "psuedo_name is too short");
-                if (PL_strlen(psuedo_name) <= PL_strlen(PSUEDO_SECURE_NAME_PREFIX)) {
+                NS_ASSERTION(PL_strlen(pseudo_name) > PL_strlen(PSEUDO_SECURE_NAME_PREFIX), "pseudo_name is too short");
+                if (PL_strlen(pseudo_name) <= PL_strlen(PSEUDO_SECURE_NAME_PREFIX)) {
                     return NS_ERROR_FAILURE;
                 }
 
-                char *hostname = psuedo_name + PL_strlen(PSUEDO_SECURE_NAME_PREFIX);
+                char *hostname = pseudo_name + PL_strlen(PSEUDO_SECURE_NAME_PREFIX);
                 rv = MigrateNewsAccount(identity, hostname, rcFile, newsHostsDir, PR_TRUE /* isSecure */);
                 if (NS_FAILED(rv)) {
                     // failed to migrate.  bail out
@@ -2291,7 +2086,7 @@ nsMessengerMigrator::MigrateNewsAccounts(nsIMsgIdentity *identity)
 	inputStream.close();
 #else /* USE_NEWSRC_MAP_FILE */
     nsCOMPtr<nsILocalFile> prefLocal;
-    rv = m_prefs->GetFileXPref(PREF_NEWS_DIRECTORY, getter_AddRefs(prefLocal));
+    rv = m_prefs->GetComplexValue(PREF_NEWS_DIRECTORY, NS_GET_IID(nsILocalFile), getter_AddRefs(prefLocal));
     if (NS_FAILED(rv)) return rv;
     newsDir = prefLocal;
     
