@@ -22,6 +22,8 @@
  * Contributor(s):
  *   Seth Spitzer <sspitzer@netscape.com>
  *   Bhuvan Racham <racham@netscape.com>
+ *   Howard Chu <hyc@symas.com>
+ *   Jens Bannmann <jens.b@web.de>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -54,22 +56,30 @@
 #include "nsDirectoryServiceDefs.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsIDirectoryService.h"
-
+#include "nsIWindowWatcher.h"
 #include "nsIWindowMediator.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsPIDOMWindow.h"
-#include "nsIScriptGlobalObject.h"
 #include "nsIDocShell.h"
 #include "nsIBaseWindow.h"
 #include "nsIWidget.h"
+#include "nsWidgetsCID.h"
+#include "nsILookAndFeel.h"
 
 #include "nsIMessengerWindowService.h"
 #include "prprf.h"
 #include "nsIWeakReference.h"
 #include "nsIStringBundle.h"
 #include "nsIAlertsService.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+#include "nsISupportsPrimitives.h"
 
-#ifdef MOZ_THUNDERBIRD
+#include "nsNativeCharsetUtils.h"
+
+// XXX test for this as long as there are still non-xul-app suite builds
+#ifdef MOZ_XUL_APP
+#include "nsToolkitCompsCID.h" 
 #define PROFILE_COMMANDLINE_ARG " -profile "
 #else
 #include "nsIProfile.h"
@@ -84,11 +94,12 @@
 #define SHELL32_DLL NS_LITERAL_CSTRING("shell32.dll")
 #define DOUBLE_QUOTE "\""
 #define MAIL_COMMANDLINE_ARG " -mail"
-#define TIMER_INTERVAL_PREF "mail.windows_xp_integration.unread_count_interval"
 #define IDI_MAILBIFF 101
-
+#define UNREAD_UPDATE_INTERVAL	(20 * 1000)	// 20 seconds
+#define ALERT_CHROME_URL "chrome://messenger/content/newmailalert.xul"
 #define NEW_MAIL_ALERT_ICON "chrome://messenger/skin/icons/new-mail-alert.png"
-#define SHOW_ALERT_PREF "mail.biff.show_alert"
+#define SHOW_ALERT_PREF     "mail.biff.show_alert"
+#define SHOW_TRAY_ICON_PREF "mail.biff.show_tray_icon"
 
 // since we are including windows.h in this file, undefine get user name....
 #ifdef GetUserName
@@ -98,12 +109,12 @@
 // begin shameless copying from nsNativeAppSupportWin
 HWND hwndForDOMWindow( nsISupports *window ) 
 {
-  nsCOMPtr<nsIScriptGlobalObject> ppScriptGlobalObj( do_QueryInterface(window) );
-  if ( !ppScriptGlobalObj )
+  nsCOMPtr<nsPIDOMWindow> win( do_QueryInterface(window) );
+  if ( !win )
       return 0;
   
   nsCOMPtr<nsIBaseWindow> ppBaseWindow =
-      do_QueryInterface( ppScriptGlobalObj->GetDocShell() );
+      do_QueryInterface( win->GetDocShell() );
   if (!ppBaseWindow) return 0;
 
   nsCOMPtr<nsIWidget> ppWidget;
@@ -130,28 +141,26 @@ static void activateWindow( nsIDOMWindowInternal *win )
 
 static void openMailWindow(const PRUnichar * aMailWindowName, const char * aFolderUri)
 {
-  nsCOMPtr<nsIWindowMediator> mediator ( do_GetService(NS_WINDOWMEDIATOR_CONTRACTID) );
-  if (!mediator)
+  nsresult rv;
+  nsCOMPtr<nsIMsgMailSession> mailSession ( do_GetService(NS_MSGMAILSESSION_CONTRACTID, &rv));
+  if (NS_FAILED(rv))
     return;
 
-  nsCOMPtr<nsIDOMWindowInternal> domWindow;
-  mediator->GetMostRecentWindow(aMailWindowName, getter_AddRefs(domWindow));
-  if (domWindow)
+  nsCOMPtr<nsIMsgWindow> topMostMsgWindow;
+  rv = mailSession->GetTopmostMsgWindow(getter_AddRefs(topMostMsgWindow));
+  if (topMostMsgWindow)
   {
     if (aFolderUri)
     {
-      nsCOMPtr<nsPIDOMWindow> piDOMWindow(do_QueryInterface(domWindow));
-      if (piDOMWindow)
-      {
-        nsCOMPtr<nsISupports> xpConnectObj;
-        piDOMWindow->GetObjectProperty(NS_LITERAL_STRING("MsgWindowCommands").get(), getter_AddRefs(xpConnectObj));
-        nsCOMPtr<nsIMsgWindowCommands> msgWindowCommands = do_QueryInterface(xpConnectObj);
-        if (msgWindowCommands)
-          msgWindowCommands->SelectFolder(aFolderUri);
-      }
+      nsCOMPtr<nsIMsgWindowCommands> windowCommands;
+      topMostMsgWindow->GetWindowCommands(getter_AddRefs(windowCommands));
+      if (windowCommands)
+        windowCommands->SelectFolder(aFolderUri);
     }
-
-    activateWindow(domWindow);
+    nsCOMPtr<nsIDOMWindowInternal> domWindow;
+    topMostMsgWindow->GetDomWindow(getter_AddRefs(domWindow));
+    if (domWindow)
+      activateWindow(domWindow);
   }
   else
   {
@@ -167,15 +176,45 @@ static void openMailWindow(const PRUnichar * aMailWindowName, const char * aFold
   }
 }
 
+static void CALLBACK delayedSingleClick(HWND msgWindow, UINT msg, INT_PTR idEvent, DWORD dwTime)
+{
+  ::KillTimer(msgWindow, idEvent);
+
+#ifdef MOZ_THUNDERBIRD
+  // single clicks on the biff icon should re-open the alert notification
+  nsresult rv = NS_OK;
+  nsCOMPtr<nsIMessengerOSIntegration> integrationService = 
+    do_GetService(NS_MESSENGEROSINTEGRATION_CONTRACTID, &rv);
+  if (NS_SUCCEEDED(rv))
+  {
+    // we know we are dealing with the windows integration object    
+    nsMessengerWinIntegration * winIntegrationService = NS_STATIC_CAST(nsMessengerWinIntegration*, 
+                                                                       NS_STATIC_CAST(nsIMessengerOSIntegration*, integrationService.get()));
+    winIntegrationService->ShowNewAlertNotification(PR_TRUE);
+  }
+#endif
+}
+
 // Window proc.
 static long CALLBACK MessageWindowProc( HWND msgWindow, UINT msg, WPARAM wp, LPARAM lp ) 
 {
-  if (msg == WM_USER && lp == WM_LBUTTONDBLCLK)
+  if (msg == WM_USER)
   {
-    NS_NAMED_LITERAL_STRING(mailName, "mail:3pane");
-    openMailWindow(mailName.get(), nsnull);
+    if (lp == WM_LBUTTONDOWN)
+    {
+      // the only way to tell a single left click 
+      // from a double left click is to fire a timer which gets cleared if we end up getting
+      // a WM_LBUTTONDBLK event.
+      ::SetTimer(msgWindow, 1, GetDoubleClickTime(), (TIMERPROC) delayedSingleClick);
+    } 
+    else if (lp == WM_LBUTTONDBLCLK)
+    {
+      ::KillTimer(msgWindow, 1);
+      NS_NAMED_LITERAL_STRING(mailName, "mail:3pane");
+      openMailWindow(mailName.get(), nsnull);
+    }
   }
-     
+
   return TRUE;
 }
 
@@ -219,11 +258,10 @@ nsMessengerWinIntegration::nsMessengerWinIntegration()
   mDefaultServerAtom = do_GetAtom("DefaultServer");
   mTotalUnreadMessagesAtom = do_GetAtom("TotalUnreadMessages");
 
-  mFirstTimeFolderUnreadCountChanged = PR_TRUE;
+  mUnreadTimerActive = PR_FALSE;
   mInboxURI = nsnull;
   mEmail = nsnull;
   mStoreUnreadCounts = PR_FALSE; 
-  mIntervalTime = 0;
 
   mBiffStateAtom = do_GetAtom("BiffState");
   mBiffIconVisible = PR_FALSE;
@@ -275,7 +313,7 @@ nsMessengerWinIntegration::ResetCurrent()
   return NS_OK;
 }
 
-NOTIFYICONDATA nsMessengerWinIntegration::mAsciiBiffIconData = { sizeof(NOTIFYICONDATA),
+NOTIFYICONDATA nsMessengerWinIntegration::sNativeBiffIconData = { sizeof(NOTIFYICONDATA),
                                                     0,
                                                     2,
                                                     NIF_ICON | NIF_MESSAGE | NIF_TIP,
@@ -283,7 +321,7 @@ NOTIFYICONDATA nsMessengerWinIntegration::mAsciiBiffIconData = { sizeof(NOTIFYIC
                                                     0,
                                                     0 };
 
-NOTIFYICONDATAW nsMessengerWinIntegration::mWideBiffIconData = { sizeof(NOTIFYICONDATAW),
+NOTIFYICONDATAW nsMessengerWinIntegration::sWideBiffIconData = { sizeof(NOTIFYICONDATAW),
                                                     0,
                                                     2,
                                                     NIF_ICON | NIF_MESSAGE | NIF_TIP,
@@ -291,15 +329,15 @@ NOTIFYICONDATAW nsMessengerWinIntegration::mWideBiffIconData = { sizeof(NOTIFYIC
                                                     0,
                                                     0 };
 
-#ifdef MOZ_THUNDERBIRD
 #ifdef MOZ_STATIC_BUILD
 #define MAIL_DLL_NAME NULL
 #else
+#ifdef MOZ_STATIC_MAIL_BUILD
 #define MAIL_DLL_NAME "mail.dll"
-#endif
 #else
 #define MAIL_DLL_NAME "msgbase.dll"
 #endif
+#endif
 
 void nsMessengerWinIntegration::InitializeBiffStatusIcon()
 {
@@ -308,15 +346,15 @@ void nsMessengerWinIntegration::InitializeBiffStatusIcon()
 
   if (mUseWideCharBiffIcon)
   {
-    mWideBiffIconData.hWnd = (HWND) msgWindow;
-    mWideBiffIconData.hIcon =  ::LoadIcon( ::GetModuleHandle( MAIL_DLL_NAME ), MAKEINTRESOURCE(IDI_MAILBIFF) );
-    mWideBiffIconData.szTip[0] = 0;
+    sWideBiffIconData.hWnd = (HWND) msgWindow;
+    sWideBiffIconData.hIcon =  ::LoadIcon( ::GetModuleHandle( MAIL_DLL_NAME ), MAKEINTRESOURCE(IDI_MAILBIFF) );
+    sWideBiffIconData.szTip[0] = 0;
   }
   else
   {
-    mAsciiBiffIconData.hWnd = (HWND) msgWindow;
-    mAsciiBiffIconData.hIcon =  ::LoadIcon( ::GetModuleHandle( MAIL_DLL_NAME ), MAKEINTRESOURCE(IDI_MAILBIFF) );
-    mAsciiBiffIconData.szTip[0] = 0;
+    sNativeBiffIconData.hWnd = (HWND) msgWindow;
+    sNativeBiffIconData.hIcon =  ::LoadIcon( ::GetModuleHandle( MAIL_DLL_NAME ), MAKEINTRESOURCE(IDI_MAILBIFF) );
+    sNativeBiffIconData.szTip[0] = 0;
   }
 
   mBiffIconInitialized = PR_TRUE;
@@ -327,24 +365,6 @@ nsMessengerWinIntegration::Init()
 {
   nsresult rv;
 
-  // get pref service
-  nsCOMPtr<nsIPref> prefService;
-  prefService = do_GetService(NS_PREF_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  // first check the timer value
-  PRInt32 timerInterval;
-  prefService->GetIntPref(TIMER_INTERVAL_PREF, &timerInterval);
-
-  // return if the timer value is negative or ZERO
-  if (timerInterval > 0) 
-  { 
-    // set the interval for timer. 
-    // multiply the value extracted (in seconds) from prefs 
-    // with 1000 to convert the interval into milliseconds
-    mIntervalTime = timerInterval * 1000; 
-  }
-
   // get directory service to build path for shell dll
   nsCOMPtr<nsIProperties> directoryService = 
     do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
@@ -369,7 +389,7 @@ nsMessengerWinIntegration::Init()
   if (!hModule)
     return NS_OK;
 
-  // get process addresses for the unerad mail count functions
+  // get process addresses for the unread mail count functions
   if (hModule) {
     mSHSetUnreadMailCount = (fnSHSetUnreadMailCount)GetProcAddress(hModule, XP_SHSetUnreadMailCounts);
     mSHEnumerateUnreadMailAccounts = (fnSHEnumerateUnreadMailAccounts)GetProcAddress(hModule, XP_SHEnumerateUnreadMailAccounts);
@@ -380,7 +400,7 @@ nsMessengerWinIntegration::Init()
 
   // if failed to get either of the process addresses, this is not XP platform
   // so we aren't storing unread counts
-  if (mSHSetUnreadMailCount && mSHEnumerateUnreadMailAccounts && (PRUint32) mIntervalTime)
+  if (mSHSetUnreadMailCount && mSHEnumerateUnreadMailAccounts)
     mStoreUnreadCounts = PR_TRUE; 
 
   nsCOMPtr <nsIMsgAccountManager> accountManager = 
@@ -400,7 +420,8 @@ nsMessengerWinIntegration::Init()
 
   if (mStoreUnreadCounts)
   {
-#ifdef MOZ_THUNDERBIRD
+// XXX test for this as long as there are still non-xul-app suite builds
+#ifdef MOZ_XUL_APP
     // get current profile path for the commandliner
     nsCOMPtr<nsIFile> profilePath;
     rv = directoryService->Get(NS_APP_USER_PROFILE_50_DIR, 
@@ -427,8 +448,6 @@ nsMessengerWinIntegration::Init()
 
     rv = ResetCurrent();
     NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = SetupUnreadCountUpdateTimer();
   }
 
   return NS_OK;
@@ -464,6 +483,7 @@ nsresult nsMessengerWinIntegration::GetStringBundle(nsIStringBundle **aBundle)
   return rv;
 }
 
+#ifndef MOZ_THUNDERBIRD
 nsresult nsMessengerWinIntegration::ShowAlertMessage(const PRUnichar * aAlertTitle, const PRUnichar * aAlertText, const char * aFolderURI)
 {
   nsresult rv;
@@ -472,20 +492,20 @@ nsresult nsMessengerWinIntegration::ShowAlertMessage(const PRUnichar * aAlertTit
   if (mAlertInProgress) 
     return NS_OK; 
 
-  nsCOMPtr<nsIPref> prefService;
-  prefService = do_GetService(NS_PREF_CONTRACTID, &rv);  
-  PRBool showAlert = PR_TRUE; 
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+  PRBool showAlert = PR_TRUE;
   
-  if (prefService)
-    prefService->GetBoolPref(SHOW_ALERT_PREF, &showAlert);
+  if (prefBranch)
+    prefBranch->GetBoolPref(SHOW_ALERT_PREF, &showAlert);
   
   if (showAlert)
   {
     nsCOMPtr<nsIAlertsService> alertsService (do_GetService(NS_ALERTSERVICE_CONTRACTID, &rv));
     if (NS_SUCCEEDED(rv))
     {
-      rv = alertsService->ShowAlertNotification(NEW_MAIL_ALERT_ICON, aAlertTitle, aAlertText, PR_TRUE, 
-                                                NS_ConvertASCIItoUCS2(aFolderURI).get(), this); 
+      rv = alertsService->ShowAlertNotification(NS_LITERAL_STRING(NEW_MAIL_ALERT_ICON), nsDependentString(aAlertTitle),
+                                                nsDependentString(aAlertText), PR_TRUE, 
+                                                NS_ConvertASCIItoUTF16(aFolderURI), this); 
       mAlertInProgress = PR_TRUE;
     }
   }
@@ -495,16 +515,104 @@ nsresult nsMessengerWinIntegration::ShowAlertMessage(const PRUnichar * aAlertTit
 
   return rv;
 }
+#else
+// Opening Thunderbird's new mail alert notification window
+// aUserInitiated --> true if we are opening the alert notification in response to a user action
+//                    like clicking on the biff icon
+nsresult nsMessengerWinIntegration::ShowNewAlertNotification(PRBool aUserInitiated)
+{
+  nsresult rv;
+
+  // if we are already in the process of showing an alert, don't try to show another....
+  if (mAlertInProgress) 
+    return NS_OK;
+  
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+  PRBool showAlert = PR_TRUE;
+  
+  if (prefBranch)
+    prefBranch->GetBoolPref(SHOW_ALERT_PREF, &showAlert);
+  
+  if (showAlert)
+  {
+    nsCOMPtr<nsISupportsArray> argsArray;
+    rv = NS_NewISupportsArray(getter_AddRefs(argsArray));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // pass in the array of folders with unread messages
+    nsCOMPtr<nsISupportsInterfacePointer> ifptr = do_CreateInstance(NS_SUPPORTS_INTERFACE_POINTER_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    ifptr->SetData(mFoldersWithNewMail);
+    ifptr->SetDataIID(&NS_GET_IID(nsISupportsArray));
+    rv = argsArray->AppendElement(ifptr);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // pass in the observer
+    ifptr = do_CreateInstance(NS_SUPPORTS_INTERFACE_POINTER_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr <nsISupports> supports = do_QueryInterface(NS_STATIC_CAST(nsIMessengerOSIntegration*, this));
+    ifptr->SetData(supports);
+    ifptr->SetDataIID(&NS_GET_IID(nsIObserver));
+    rv = argsArray->AppendElement(ifptr);
+    NS_ENSURE_SUCCESS(rv, rv);
+    
+    // pass in the animation flag
+    nsCOMPtr<nsISupportsPRBool> scriptableUserInitiated (do_CreateInstance(NS_SUPPORTS_PRBOOL_CONTRACTID, &rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+    scriptableUserInitiated->SetData(aUserInitiated);
+    rv = argsArray->AppendElement(scriptableUserInitiated);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // pass in the alert origin
+    nsCOMPtr<nsISupportsPRUint8> scriptableOrigin (do_CreateInstance(NS_SUPPORTS_PRUINT8_CONTRACTID));
+    NS_ENSURE_TRUE(scriptableOrigin, NS_ERROR_FAILURE);
+    scriptableOrigin->SetData(0);
+    nsCOMPtr<nsILookAndFeel> lookAndFeel = do_GetService("@mozilla.org/widget/lookandfeel;1");
+    if (lookAndFeel)
+    {
+      PRInt32 origin;
+      lookAndFeel->GetMetric(nsILookAndFeel::eMetric_AlertNotificationOrigin,
+                             origin);
+      if (origin && origin >= 0 && origin <= 7)
+        scriptableOrigin->SetData(origin);
+    }
+    rv = argsArray->AppendElement(scriptableOrigin);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
+    nsCOMPtr<nsIDOMWindow> newWindow;
+    rv = wwatch->OpenWindow(0, ALERT_CHROME_URL, "_blank",
+                "chrome,dialog=yes,titlebar=no,popup=yes", argsArray,
+                 getter_AddRefs(newWindow));
+
+    mAlertInProgress = PR_TRUE;
+  }
+
+  // if the user has turned off the mail alert, or  openWindow generated an error, 
+  // then go straight to the system tray.
+  if (!showAlert || NS_FAILED(rv)) 
+    AlertFinished();
+
+  return rv;
+}
+#endif
 
 nsresult nsMessengerWinIntegration::AlertFinished()
 {
-  // okay we are done showing the alert....now put an icon in the system tray
-  if (!mSuppressBiffIcon)
+  // okay, we are done showing the alert
+  // now put an icon in the system tray, if allowed
+  PRBool showTrayIcon = !mSuppressBiffIcon;
+  if (showTrayIcon)
+  {
+    nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+    if (prefBranch)
+      prefBranch->GetBoolPref(SHOW_TRAY_ICON_PREF, &showTrayIcon);
+  }
+  if (showTrayIcon)
   {
     GenericShellNotify(NIM_ADD);
     mBiffIconVisible = PR_TRUE;
   }
-
   mSuppressBiffIcon = PR_FALSE;
   mAlertInProgress = PR_FALSE;
   return NS_OK;
@@ -542,7 +650,6 @@ void nsMessengerWinIntegration::FillToolTipInfo()
   nsXPIDLCString hostName; 
   nsAutoString toolTipText;
   nsAutoString animatedAlertText;
-  nsCOMPtr<nsISupports> supports;
   nsCOMPtr<nsIMsgFolder> folder;
   nsCOMPtr<nsIWeakReference> weakReference;
   PRInt32 numNewMessages = 0;
@@ -553,8 +660,7 @@ void nsMessengerWinIntegration::FillToolTipInfo()
 
   for (PRUint32 index = 0; index < count; index++)
   {
-    supports = getter_AddRefs(mFoldersWithNewMail->ElementAt(index));
-    weakReference = do_QueryInterface(supports);
+    weakReference = do_QueryElementAt(mFoldersWithNewMail, index);
     folder = do_QueryReferent(weakReference);
     if (folder)
     {
@@ -602,7 +708,11 @@ void nsMessengerWinIntegration::FillToolTipInfo()
 
   if (!mBiffIconVisible)
   {
+#ifndef MOZ_THUNDERBIRD
     ShowAlertMessage(accountName, animatedAlertText.get(), "");
+#else
+    ShowNewAlertNotification(PR_FALSE);
+#endif
   }
   else
    GenericShellNotify( NIM_MODIFY);
@@ -615,7 +725,6 @@ nsresult nsMessengerWinIntegration::GetFirstFolderWithNewMail(char ** aFolderURI
   nsresult rv;
   NS_ENSURE_TRUE(mFoldersWithNewMail, NS_ERROR_FAILURE); 
 
-  nsCOMPtr<nsISupports> supports;
   nsCOMPtr<nsIMsgFolder> folder;
   nsCOMPtr<nsIWeakReference> weakReference;
   PRInt32 numNewMessages = 0;
@@ -626,13 +735,11 @@ nsresult nsMessengerWinIntegration::GetFirstFolderWithNewMail(char ** aFolderURI
   if (!count)  // kick out if we don't have any folders with new mail
     return NS_OK;
 
-  supports = getter_AddRefs(mFoldersWithNewMail->ElementAt(0));
-  weakReference = do_QueryInterface(supports);
+  weakReference = do_QueryElementAt(mFoldersWithNewMail, 0);
   folder = do_QueryReferent(weakReference);
   
   if (folder)
   {
-    PRUint32 biffState = nsIMsgFolder::nsMsgBiffState_NoMail; 
     nsCOMPtr<nsIMsgFolder> msgFolder;
     // enumerate over the folders under this root folder till we find one with new mail....
     nsCOMPtr<nsISupportsArray> allFolders;
@@ -680,9 +787,9 @@ void nsMessengerWinIntegration::DestroyBiffIcon()
 PRUint32 nsMessengerWinIntegration::GetToolTipSize()
 {
   if (mUseWideCharBiffIcon)
-    return (sizeof(mWideBiffIconData.szTip)/sizeof(mWideBiffIconData.szTip[0]));
+    return (sizeof(sWideBiffIconData.szTip)/sizeof(sWideBiffIconData.szTip[0]));
   else
-    return (sizeof(mAsciiBiffIconData.szTip));
+    return (sizeof(sNativeBiffIconData.szTip));
 }
 
 void nsMessengerWinIntegration::SetToolTipStringOnIconData(const PRUnichar * aToolTipString)
@@ -693,17 +800,19 @@ void nsMessengerWinIntegration::SetToolTipStringOnIconData(const PRUnichar * aTo
   
   if (mUseWideCharBiffIcon)
   {
-    ::wcsncpy( mWideBiffIconData.szTip, aToolTipString, toolTipBufSize);
+    ::wcsncpy( sWideBiffIconData.szTip, aToolTipString, toolTipBufSize);
     if (wcslen(aToolTipString) >= toolTipBufSize)
-      mWideBiffIconData.szTip[toolTipBufSize - 1] = 0;
+      sWideBiffIconData.szTip[toolTipBufSize - 1] = 0;
   }
   else
   {
-    nsCString asciiToolTip;
-    asciiToolTip.AssignWithConversion(aToolTipString);
-    ::strncpy( mAsciiBiffIconData.szTip, asciiToolTip.get(), GetToolTipSize() );
-    if (asciiToolTip.Length() >= toolTipBufSize)
-      mAsciiBiffIconData.szTip[toolTipBufSize - 1] = 0;
+    nsCAutoString nativeToolTipString;
+    NS_CopyUnicodeToNative(nsDependentString(aToolTipString),
+                           nativeToolTipString);
+    ::strncpy(sNativeBiffIconData.szTip,
+              nativeToolTipString.get(), GetToolTipSize());
+    if (nativeToolTipString.Length() >= toolTipBufSize)
+      sNativeBiffIconData.szTip[toolTipBufSize - 1] = 0;
   }
 }
 
@@ -711,14 +820,14 @@ void nsMessengerWinIntegration::GenericShellNotify(DWORD aMessage)
 {
   if (mUseWideCharBiffIcon)
   {
-    BOOL res = mShellNotifyWideChar( aMessage, &mWideBiffIconData );
+    BOOL res = mShellNotifyWideChar( aMessage, &sWideBiffIconData );
     if (!res)
       RevertToNonUnicodeShellAPI(); // oops we don't really implement the unicode shell apis...fall back.
     else
       return; 
   }
   
-  ::Shell_NotifyIcon( aMessage, &mAsciiBiffIconData ); 
+  ::Shell_NotifyIcon( aMessage, &sNativeBiffIconData ); 
 }
 
 // some flavors of windows define ShellNotifyW but when you actually try to use it,
@@ -732,86 +841,17 @@ void nsMessengerWinIntegration::RevertToNonUnicodeShellAPI()
   InitializeBiffStatusIcon();
 
   // now we need to copy over any left over tool tip strings
-  if (mWideBiffIconData.szTip)
+  if (sWideBiffIconData.szTip)
   {
-    const PRUnichar * oldTooltipString = mWideBiffIconData.szTip;
+    const PRUnichar * oldTooltipString = sWideBiffIconData.szTip;
     SetToolTipStringOnIconData(oldTooltipString);
   }
 }
 
 NS_IMETHODIMP
-nsMessengerWinIntegration::OnItemPropertyFlagChanged(nsISupports *item, nsIAtom *property, PRUint32 oldFlag, PRUint32 newFlag)
+nsMessengerWinIntegration::OnItemPropertyFlagChanged(nsIMsgDBHdr *item, nsIAtom *property, PRUint32 oldFlag, PRUint32 newFlag)
 {
   nsresult rv = NS_OK;
-
-  if (!mBiffIconInitialized)
-    InitializeBiffStatusIcon(); 
-    
-  // if we got new mail show a icon in the system tray
-	if (mBiffStateAtom == property && mFoldersWithNewMail)
-	{
-    nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(item);
-    NS_ENSURE_TRUE(folder, NS_OK);
-
-		if (newFlag == nsIMsgFolder::nsMsgBiffState_NewMail) 
-    {
-      // if the icon is not already visible, only show a system tray icon iff 
-      // we are performing biff (as opposed to the user getting new mail)
-      if (!mBiffIconVisible)
-      {
-        PRBool performingBiff = PR_FALSE;
-        nsCOMPtr<nsIMsgIncomingServer> server;
-        folder->GetServer(getter_AddRefs(server));
-        if (server)
-          server->GetPerformingBiff(&performingBiff);
-        if (!performingBiff) 
-          return NS_OK; // kick out right now...
-      }
-      nsCOMPtr<nsIWeakReference> weakFolder = do_GetWeakReference(folder); 
-
-      // remove the element if it is already in the array....
-      PRUint32 count = 0;
-      PRUint32 index = 0; 
-      mFoldersWithNewMail->Count(&count);
-      nsCOMPtr<nsISupports> supports;
-      nsCOMPtr<nsIMsgFolder> oldFolder;
-      nsCOMPtr<nsIWeakReference> weakReference;
-      for (index = 0; index < count; index++)
-      {
-        supports = getter_AddRefs(mFoldersWithNewMail->ElementAt(index));
-        weakReference = do_QueryInterface(supports);
-        oldFolder = do_QueryReferent(weakReference);
-        if (oldFolder == folder) // if they point to the same folder
-          break;
-        oldFolder = nsnull;
-      }
-
-      if (oldFolder)
-        mFoldersWithNewMail->ReplaceElementAt(weakFolder, index);
-      else
-        mFoldersWithNewMail->AppendElement(weakFolder);
-      // now regenerate the tooltip
-      FillToolTipInfo();    
-    }
-    else if (newFlag == nsIMsgFolder::nsMsgBiffState_NoMail)
-    {
-      // we are always going to remove the icon whenever we get our first no mail
-      // notification. 
-      
-      // avoid a race condition where we are told to remove the icon before we've actually
-      // added it to the system tray. This happens when the user reads a new message before
-      // the animated alert has gone away.
-      if (mAlertInProgress)
-        mSuppressBiffIcon = PR_TRUE;
-
-      mFoldersWithNewMail->Clear(); 
-      if (mBiffIconVisible) 
-      {
-        mBiffIconVisible = PR_FALSE;
-        GenericShellNotify(NIM_DELETE); 
-      }
-    }
-  } // if the biff property changed
   
   return NS_OK;
 }
@@ -861,35 +901,84 @@ nsMessengerWinIntegration::OnItemEvent(nsIMsgFolder *, nsIAtom *)
 NS_IMETHODIMP
 nsMessengerWinIntegration::OnItemIntPropertyChanged(nsIRDFResource *aItem, nsIAtom *aProperty, PRInt32 aOldValue, PRInt32 aNewValue)
 {
-  nsresult rv;
+  // if we got new mail show a icon in the system tray
+  if (mBiffStateAtom == aProperty && mFoldersWithNewMail)
+  {
+    if (!mBiffIconInitialized)
+      InitializeBiffStatusIcon(); 
+
+    nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(aItem);
+    NS_ENSURE_TRUE(folder, NS_OK);
+
+    if (aNewValue == nsIMsgFolder::nsMsgBiffState_NewMail) 
+    {
+      // if the icon is not already visible, only show a system tray icon iff 
+      // we are performing biff (as opposed to the user getting new mail)
+      if (!mBiffIconVisible)
+      {
+        PRBool performingBiff = PR_FALSE;
+        nsCOMPtr<nsIMsgIncomingServer> server;
+        folder->GetServer(getter_AddRefs(server));
+        if (server)
+          server->GetPerformingBiff(&performingBiff);
+        if (!performingBiff) 
+          return NS_OK; // kick out right now...
+      }
+      nsCOMPtr<nsIWeakReference> weakFolder = do_GetWeakReference(folder); 
+
+      if (mFoldersWithNewMail->IndexOf(weakFolder) == -1)
+        mFoldersWithNewMail->AppendElement(weakFolder);
+      // now regenerate the tooltip
+      FillToolTipInfo();    
+    }
+    else if (aNewValue == nsIMsgFolder::nsMsgBiffState_NoMail)
+    {
+      // we are always going to remove the icon whenever we get our first no mail
+      // notification. 
+      
+      // avoid a race condition where we are told to remove the icon before we've actually
+      // added it to the system tray. This happens when the user reads a new message before
+      // the animated alert has gone away.
+      if (mAlertInProgress)
+        mSuppressBiffIcon = PR_TRUE;
+
+      mFoldersWithNewMail->Clear(); 
+      if (mBiffIconVisible) 
+      {
+        mBiffIconVisible = PR_FALSE;
+        GenericShellNotify(NIM_DELETE); 
+      }
+    }
+  } // if the biff property changed
 
   if (!mStoreUnreadCounts) return NS_OK; // don't do anything here if we aren't storing unread counts...
 
   if (aProperty == mTotalUnreadMessagesAtom) {
     const char *itemURI = nsnull;
+    nsresult rv;
     rv = aItem->GetValueConst(&itemURI);
     NS_ENSURE_SUCCESS(rv,rv);
 
-    // Today, if the user brings up the mail app and gets his/her mail and read 
-    // couple of messages and quit the app before the first timer action is 
-    // triggered, app will not get the opportunity to update the registry 
-    // latest unread count thus displaying essentially the previous sessions's data. 
-    // That is not desirable. So, we can avoid that situation by updating the 
-    // registry first time the total unread count is changed. That will update 
-    // the registry to reflect the first user action that alters unread mail count.
-    // As the user reads some of the messages, the latest unread mail count 
-    // is kept track of. Now, if the user quits before the first timer is triggered,
-    // the registry is updated one last time via UpdateRegistryWithCurrent() as we will
-    // have all information needed available to do so.
-    if (mFirstTimeFolderUnreadCountChanged) {
-      mFirstTimeFolderUnreadCountChanged = PR_FALSE;
-
-      rv = UpdateUnreadCount();
-      NS_ENSURE_SUCCESS(rv,rv);
+    if (itemURI && mInboxURI && !strcmp(itemURI, mInboxURI)) {
+      mCurrentUnreadCount = aNewValue;
     }
 
-    if (itemURI && mInboxURI && !nsCRT::strcmp(itemURI, mInboxURI)) {
-      mCurrentUnreadCount = aNewValue;
+    // If the timer isn't running yet, then we immediately update the
+    // registry and then start a one-shot timer. If the Unread counter
+    // has toggled zero / nonzero, we also update immediately.
+    // Otherwise, if the timer is running, defer the update. This means
+    // that all counter updates that occur within the timer interval are
+    // batched into a single registry update, to avoid hitting the
+    // registry too frequently. We also do a final update on shutdown,
+    // regardless of the timer.
+    if (!mUnreadTimerActive ||
+         (!mCurrentUnreadCount && mLastUnreadCountWrittenToRegistry) ||
+         (mCurrentUnreadCount && mLastUnreadCountWrittenToRegistry < 1)) {
+      rv = UpdateUnreadCount();
+      NS_ENSURE_SUCCESS(rv,rv);
+      // If timer wasn't running, start it.
+      if (!mUnreadTimerActive)
+        rv = SetupUnreadCountUpdateTimer();
     }
   }
   return NS_OK;
@@ -900,6 +989,7 @@ nsMessengerWinIntegration::OnUnreadCountUpdateTimer(nsITimer *timer, void *osInt
 {
   nsMessengerWinIntegration *winIntegration = (nsMessengerWinIntegration*)osIntegration;
 
+  winIntegration->mUnreadTimerActive = PR_FALSE;
   nsresult rv = winIntegration->UpdateUnreadCount();
   NS_ASSERTION(NS_SUCCEEDED(rv), "updating unread count failed");
 }
@@ -939,7 +1029,7 @@ nsMessengerWinIntegration::RemoveCurrentFromRegistry()
       if (!deleteKey.IsEmpty()) {
         // delete this key and berak out of the loop
         RegDeleteKey(HKEY_CURRENT_USER, 
-                     NS_ConvertUCS2toUTF8(deleteKey).get());
+                     NS_ConvertUTF16toUTF8(deleteKey).get());
         break;
       }
       else {
@@ -977,7 +1067,8 @@ nsMessengerWinIntegration::UpdateRegistryWithCurrent()
   commandLinerForAppLaunch.Append(mAppName);
   commandLinerForAppLaunch.Append(NS_LITERAL_STRING(DOUBLE_QUOTE));
   commandLinerForAppLaunch.Append(NS_LITERAL_STRING(PROFILE_COMMANDLINE_ARG));
-#ifdef MOZ_THUNDERBIRD
+// XXX test for this as long as there are still non-xul-app suite builds
+#ifdef MOZ_XUL_APP
   commandLinerForAppLaunch.Append(NS_LITERAL_STRING(DOUBLE_QUOTE));
   commandLinerForAppLaunch.Append(mProfilePath);
   commandLinerForAppLaunch.Append(NS_LITERAL_STRING(DOUBLE_QUOTE));
@@ -1057,19 +1148,20 @@ nsMessengerWinIntegration::SetupInbox()
       providerPrefixPref.Append(".unreadMailCountRegistryKeyPrefix");
 
       // get pref service
-      nsCOMPtr<nsIPref> prefService;
-      prefService = do_GetService(NS_PREF_CONTRACTID, &rv);
+      nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
       NS_ENSURE_SUCCESS(rv,rv);
 
-      nsXPIDLString prefixValue;
-      prefService->CopyUnicharPref(providerPrefixPref.get(), getter_Copies(prefixValue));
-      if (prefixValue.get())
-        mEmailPrefix.Assign(prefixValue);
+      nsCOMPtr<nsISupportsString> tmp;
+      rv = prefBranch->GetComplexValue(providerPrefixPref.get(), NS_GET_IID(nsISupportsString),
+                                       getter_AddRefs(tmp));
+
+      if (NS_SUCCEEDED(rv))
+        tmp->GetData(mEmailPrefix);
       else
-        mEmailPrefix.Truncate(0);
+        mEmailPrefix.Truncate();
     }
     else {
-        mEmailPrefix.Truncate(0);
+      mEmailPrefix.Truncate();
     }
 
     // Get user's email address
@@ -1136,14 +1228,17 @@ nsresult
 nsMessengerWinIntegration::SetupUnreadCountUpdateTimer()
 {
   if (!mStoreUnreadCounts) return NS_OK; // don't do anything here if we aren't storing unread counts...
-  PRUint32 timeInMSUint32 = (PRUint32) mIntervalTime;
+  mUnreadTimerActive = PR_TRUE;
   if (mUnreadCountUpdateTimer) {
     mUnreadCountUpdateTimer->Cancel();
   }
+  else
+  {
+    mUnreadCountUpdateTimer = do_CreateInstance("@mozilla.org/timer;1");
+  }
 
-  mUnreadCountUpdateTimer = do_CreateInstance("@mozilla.org/timer;1");
-  mUnreadCountUpdateTimer->InitWithFuncCallback(OnUnreadCountUpdateTimer, (void*)this, timeInMSUint32, 
-                                                nsITimer::TYPE_REPEATING_SLACK);
+  mUnreadCountUpdateTimer->InitWithFuncCallback(OnUnreadCountUpdateTimer,
+    (void *)this, UNREAD_UPDATE_INTERVAL, nsITimer::TYPE_ONE_SHOT);
 
   return NS_OK;
 }
