@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -76,7 +76,6 @@
 #include "nsIMsgPurgeService.h"
 #include "nsIObserverService.h"
 #include "nsIMsgMailSession.h"
-#include "nsIEventQueueService.h"
 #include "nsIDirectoryService.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsMsgFolderFlags.h"
@@ -87,13 +86,14 @@
 #include "nsIMessengerOSIntegration.h"
 #include "nsICategoryManager.h"
 #include "nsISupportsPrimitives.h"
-#include "nsMsgFilterService.h"
+#include "nsIMsgFilterService.h"
 #include "nsIMsgFilter.h"
 #include "nsIMsgSearchSession.h"
 #include "nsIDBChangeListener.h"
 #include "nsIDBFolderInfo.h"
 #include "nsIMsgHdr.h"
 #include "nsILineInputStream.h"
+#include "nsThreadUtils.h"
 #include "nsNetUtil.h"
 #include "nsEscape.h"
 
@@ -112,7 +112,6 @@
 
 static NS_DEFINE_CID(kMsgAccountCID, NS_MSGACCOUNT_CID);
 static NS_DEFINE_CID(kMsgFolderCacheCID, NS_MSGFOLDERCACHE_CID);
-static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 
 // use this to search for all servers with the given hostname/iid and
 // put them in "servers"
@@ -161,7 +160,9 @@ nsMsgAccountManager::nsMsgAccountManager() :
   m_cleanupInboxInProgress(PR_FALSE),
   m_haveShutdown(PR_FALSE),
   m_shutdownInProgress(PR_FALSE),
-  m_userAuthenticated(PR_FALSE)
+  m_userAuthenticated(PR_FALSE),
+  m_loadingVirtualFolders(PR_FALSE),
+  m_virtualFoldersLoaded(PR_FALSE)
 {
 }
 
@@ -225,7 +226,6 @@ nsresult nsMsgAccountManager::Shutdown()
   }
   if(m_msgFolderCache)
     WriteToFolderCache(m_msgFolderCache);
-
   (void)ShutdownServers();
   (void)UnloadAccounts();
   
@@ -782,9 +782,10 @@ nsMsgAccountManager::GetDefaultAccount(nsIMsgAccount * *aDefaultAccount)
     nsXPIDLCString defaultKey;
     rv = m_prefs->GetCharPref(PREF_MAIL_ACCOUNTMANAGER_DEFAULTACCOUNT, getter_Copies(defaultKey));
     
-    if (NS_SUCCEEDED(rv)) {
+    if (NS_SUCCEEDED(rv))
       GetAccount(defaultKey, getter_AddRefs(m_defaultAccount));
-    } else {
+
+    if (!m_defaultAccount) {
       PRUint32 index;
       PRBool foundValidDefaultAccount = PR_FALSE;
       for (index = 0; index < count; index++) {
@@ -810,12 +811,14 @@ nsMsgAccountManager::GetDefaultAccount(nsIMsgAccount * *aDefaultAccount)
           }
         }
       }
+
       if (!foundValidDefaultAccount) {
         // get the first account and use it.
         // we need to fix this scenario.
+        NS_WARNING("No valid default account found, just using first (FIXME)");
         nsCOMPtr<nsIMsgAccount> firstAccount;
-        rv = m_accounts->QueryElementAt(0, NS_GET_IID(nsIMsgAccount),
-                                       (void **)getter_AddRefs(firstAccount));
+        m_accounts->QueryElementAt(0, NS_GET_IID(nsIMsgAccount),
+                                   (void **)getter_AddRefs(firstAccount));
 
         SetDefaultAccount(firstAccount);
       }
@@ -1019,7 +1022,10 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
   server->GetEmptyTrashOnExit(&emptyTrashOnExit);
   nsCOMPtr <nsIImapIncomingServer> imapserver = do_QueryInterface(server);
   if (imapserver)
+  {
     imapserver->GetCleanupInboxOnExit(&cleanupInboxOnExit);
+    imapserver->SetShuttingDown(PR_TRUE);
+  }
   if (emptyTrashOnExit || cleanupInboxOnExit)
   {
     nsCOMPtr<nsIMsgFolder> root;
@@ -1048,12 +1054,6 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
            nsCOMPtr<nsIMsgAccountManager> accountManager = 
                     do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
            if (NS_FAILED(rv)) return rv;
-           nsCOMPtr<nsIEventQueueService> pEventQService = 
-                    do_GetService(kEventQueueServiceCID, &rv);
-           if (NS_FAILED(rv)) return rv;
-           nsCOMPtr<nsIEventQueue> eventQueue;
-           pEventQService->GetThreadEventQueue(NS_CURRENT_THREAD,
-                                    getter_AddRefs(eventQueue)); 
            if (isImap)
              urlListener = do_QueryInterface(accountManager, &rv);
                     
@@ -1090,6 +1090,8 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
                     
            if (isImap && urlListener)
            {
+             nsIThread *thread = NS_GetCurrentThread();
+
              PRBool inProgress = PR_FALSE;
              if (cleanupInboxOnExit)
              {
@@ -1100,8 +1102,7 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
                  PR_CEnterMonitor(folder);
                  PR_CWait(folder, PR_MicrosecondsToInterval(1000UL));
                  PR_CExitMonitor(folder);
-                 if (eventQueue)
-                   eventQueue->ProcessPendingEvents();
+                 NS_ProcessPendingEvents(thread);
                }
              }
              if (emptyTrashOnExit)
@@ -1113,8 +1114,7 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
                  PR_CEnterMonitor(folder);
                  PR_CWait(folder, PR_MicrosecondsToInterval(1000UL));
                  PR_CExitMonitor(folder);
-                 if (eventQueue)
-                   eventQueue->ProcessPendingEvents();
+                 NS_ProcessPendingEvents(thread);
                }
              }
            } 
@@ -1428,38 +1428,40 @@ nsMsgAccountManager::LoadAccounts()
   if (!accountList || !accountList[0])
     return NS_OK;
   
-    /* parse accountList and run loadAccount on each string, comma-separated */   
-    nsCOMPtr<nsIMsgAccount> account;
-    char *newStr;
-    char *rest = accountList.BeginWriting();
-    nsCAutoString str;
-
-    char *token = nsCRT::strtok(rest, ",", &newStr);
-    while (token) {
-      str = token;
-      str.StripWhitespace();
+  /* parse accountList and run loadAccount on each string, comma-separated */   
+  nsCOMPtr<nsIMsgAccount> account;
+  char *newStr;
+  char *rest = accountList.BeginWriting();
+  nsCAutoString str;
+  
+  for (char *token = nsCRT::strtok(rest, ",", &newStr);
+       token;
+       token = nsCRT::strtok(newStr, ",", &newStr))
+  {
+    str = token;
+    str.StripWhitespace();
       
-      if (!str.IsEmpty()) 
-          rv = GetAccount(str.get(), getter_AddRefs(account));
-
-      // force load of accounts (need to find a better way to do this
-      nsCOMPtr<nsISupportsArray> identities;
-      account->GetIdentities(getter_AddRefs(identities));
-      
-      nsCOMPtr<nsIMsgIncomingServer> server;
-      account->GetIncomingServer(getter_AddRefs(server));
-
-      token = nsCRT::strtok(newStr, ",", &newStr);
+    if (str.IsEmpty() ||
+        NS_FAILED(createKeyedAccount(str.get(), getter_AddRefs(account))) ||
+        !account) {
+      NS_WARNING("unexpected entry in account list; prefs corrupt?");
+      continue;
     }
 
+    // force load of accounts (need to find a better way to do this)
+    nsCOMPtr<nsISupportsArray> identities;
+    account->GetIdentities(getter_AddRefs(identities));
     
-    LoadVirtualFolders();
-    nsCOMPtr<nsIMsgMailSession> mailSession = do_GetService(NS_MSGMAILSESSION_CONTRACTID, &rv); 
+    nsCOMPtr<nsIMsgIncomingServer> server;
+    account->GetIncomingServer(getter_AddRefs(server));
+  }
 
-    if (NS_SUCCEEDED(rv))
-      mailSession->AddFolderListener(this, nsIFolderListener::added | nsIFolderListener::removed);
-    /* finished loading accounts */
-    return NS_OK;
+  nsCOMPtr<nsIMsgMailSession> mailSession = do_GetService(NS_MSGMAILSESSION_CONTRACTID, &rv); 
+
+  if (NS_SUCCEEDED(rv))
+    mailSession->AddFolderListener(this, nsIFolderListener::added | nsIFolderListener::removed);
+  /* finished loading accounts */
+  return NS_OK;
 }
 
 PRBool
@@ -1537,8 +1539,13 @@ nsMsgAccountManager::SetSpecialFolders()
       if (folderUri && NS_SUCCEEDED(rdf->GetResource(folderUri, getter_AddRefs(res))))
       {
         folder = do_QueryInterface(res, &rv);
-        if (NS_SUCCEEDED(rv))
-          rv = folder->SetFlag(MSG_FOLDER_FLAG_DRAFTS);
+        nsCOMPtr <nsIMsgFolder> parent;
+        if (folder && NS_SUCCEEDED(rv))
+        {
+          rv = folder->GetParent(getter_AddRefs(parent));
+          if (NS_SUCCEEDED(rv) && parent)
+            rv = folder->SetFlag(MSG_FOLDER_FLAG_DRAFTS);
+        }
       }
       thisIdentity->GetStationeryFolder(getter_Copies(folderUri));
       if (folderUri && NS_SUCCEEDED(rdf->GetResource(folderUri, getter_AddRefs(res))))
@@ -1662,13 +1669,13 @@ nsMsgAccountManager::GetAccount(const char* key,
     
     m_accounts->EnumerateForwards(findAccountByKey, (void *)&findEntry);
 
-    if (findEntry.account) {
+    if (findEntry.account)
         NS_ADDREF(*_retval = findEntry.account);
-        return NS_OK;
-    }
+    else
+        *_retval = nsnull;
 
     // not found, create on demand
-    return createKeyedAccount(key, _retval);
+    return NS_OK;
 }
 
 nsresult
@@ -1896,22 +1903,6 @@ nsMsgAccountManager::InternalFindServer(const char* username,
   nsresult rv;
   nsCOMPtr<nsISupportsArray> servers;
  
-  // If 'useRealSetting' is set then we want to scan all existing accounts
-  // to make sure there's no duplicate including those whose host and/or
-  // user names have been changed.
-  if (!useRealSetting &&
-      (m_lastFindServerHostName.Equals(hostname)) &&
-      (m_lastFindServerUserName.Equals(username)) &&
-      (m_lastFindServerType.Equals(type)) &&
-      m_lastFindServerResult) 
-  {
-    NS_ADDREF(*aResult = m_lastFindServerResult);
-    return NS_OK;
-  }
-
-  rv = GetAllServers(getter_AddRefs(servers));
-  if (NS_FAILED(rv)) return rv;
-
   findServerEntry serverInfo;
 
   // "" acts as the wild card.
@@ -1924,6 +1915,22 @@ nsMsgAccountManager::InternalFindServer(const char* username,
   serverInfo.type = type ? type : "";
   serverInfo.useRealSetting = useRealSetting;
 
+  // If 'useRealSetting' is set then we want to scan all existing accounts
+  // to make sure there's no duplicate including those whose host and/or
+  // user names have been changed.
+  if (!useRealSetting &&
+      (m_lastFindServerHostName.Equals(serverInfo.hostname)) &&
+      (m_lastFindServerUserName.Equals(serverInfo.username)) &&
+      (m_lastFindServerType.Equals(serverInfo.type)) &&
+      m_lastFindServerResult) 
+  {
+    NS_ADDREF(*aResult = m_lastFindServerResult);
+    return NS_OK;
+  }
+
+  rv = GetAllServers(getter_AddRefs(servers));
+  if (NS_FAILED(rv)) return rv;
+
   serverInfo.server = *aResult = nsnull;
   
   servers->EnumerateForwards(findServer, (void *)&serverInfo);
@@ -2627,7 +2634,10 @@ nsresult VirtualFolderChangeListener::Init()
     rv = tempFilter->GetSearchTerms(getter_AddRefs(searchTerms));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    m_searchSession->AddScopeTerm(nsMsgSearchScope::offlineMail, m_folderWatching);
+    // we add the search scope right before we match the header,
+    // because we don't want the search scope caching the body input
+    // stream, because that holds onto the mailbox file, breaking
+    // compaction.
 
     // add each item in termsArray to the search session
     PRUint32 numTerms;
@@ -2654,19 +2664,22 @@ NS_IMETHODIMP VirtualFolderChangeListener::OnHdrChange(nsIMsgDBHdr *aHdrChanged,
 
   nsresult rv = m_folderWatching->GetMsgDatabase(nsnull, getter_AddRefs(msgDB));
   PRBool oldMatch = PR_FALSE, newMatch = PR_FALSE;
+  // we don't want any early returns from this function, until we've 
+  // called ClearScopes 0n the search session.
+  m_searchSession->AddScopeTerm(nsMsgSearchScope::offlineMail, m_folderWatching);
   rv = m_searchSession->MatchHdr(aHdrChanged, msgDB, &newMatch);
-  NS_ENSURE_SUCCESS(rv, rv);
-  if (m_searchOnMsgStatus)
+  if (NS_SUCCEEDED(rv) && m_searchOnMsgStatus)
   {
     // if status is a search criteria, check if the header matched before
     // it changed, in order to determine if we need to bump the counts. 
     aHdrChanged->SetFlags(aOldFlags);
     rv = m_searchSession->MatchHdr(aHdrChanged, msgDB, &oldMatch);
     aHdrChanged->SetFlags(aNewFlags); // restore new flags even on match failure.
-    NS_ENSURE_SUCCESS(rv, rv);
   }
   else
     oldMatch = newMatch;
+  m_searchSession->ClearScopes();
+  NS_ENSURE_SUCCESS(rv, rv);
   // we don't want to change the total counts if this virtual folder is open in a view,
   // because we won't remove the header from view while it's open. On the other hand,
   // it's hard to fix the count when the user clicks away to another folder, w/o re-running
@@ -2712,6 +2725,13 @@ NS_IMETHODIMP VirtualFolderChangeListener::OnHdrChange(nsIMsgDBHdr *aHdrChanged,
       if (numNewMessages == 1)
         m_virtualFolder->SetHasNewMessages(PR_FALSE);
     }
+    if (totalDelta)
+    {
+      nsXPIDLCString searchUri;
+      m_virtualFolder->GetURI(getter_Copies(searchUri));
+      msgDB->UpdateHdrInCache(searchUri, aHdrChanged, totalDelta == 1);
+    }
+
     m_virtualFolder->UpdateSummaryTotals(PR_TRUE); // force update from db.
     virtDatabase->Commit(nsMsgDBCommitType::kLargeCommit);
   }
@@ -2725,13 +2745,15 @@ NS_IMETHODIMP VirtualFolderChangeListener::OnHdrDeleted(nsIMsgDBHdr *aHdrDeleted
   nsresult rv = m_folderWatching->GetMsgDatabase(nsnull, getter_AddRefs(msgDB));
   NS_ENSURE_SUCCESS(rv, rv);
   PRBool match = PR_FALSE;
+  m_searchSession->AddScopeTerm(nsMsgSearchScope::offlineMail, m_folderWatching);
   rv = m_searchSession->MatchHdr(aHdrDeleted, msgDB, &match);
+  m_searchSession->ClearScopes();
   if (match)
   {
     nsCOMPtr <nsIMsgDatabase> virtDatabase;
     nsCOMPtr <nsIDBFolderInfo> dbFolderInfo;
 
-    nsresult rv = m_virtualFolder->GetDBFolderInfoAndDB(getter_AddRefs(dbFolderInfo), getter_AddRefs(virtDatabase));
+    rv = m_virtualFolder->GetDBFolderInfoAndDB(getter_AddRefs(dbFolderInfo), getter_AddRefs(virtDatabase));
     PRBool msgHdrIsRead;
     aHdrDeleted->GetIsRead(&msgHdrIsRead);
     if (!msgHdrIsRead)
@@ -2747,6 +2769,10 @@ NS_IMETHODIMP VirtualFolderChangeListener::OnHdrDeleted(nsIMsgDBHdr *aHdrDeleted
       if (numNewMessages == 1)
         m_virtualFolder->SetHasNewMessages(PR_FALSE);
     }
+    nsXPIDLCString searchUri;
+    m_virtualFolder->GetURI(getter_Copies(searchUri));
+    msgDB->UpdateHdrInCache(searchUri, aHdrDeleted, PR_FALSE);
+
     m_virtualFolder->UpdateSummaryTotals(PR_TRUE); // force update from db.
     virtDatabase->Commit(nsMsgDBCommitType::kLargeCommit);
   }
@@ -2760,7 +2786,9 @@ NS_IMETHODIMP VirtualFolderChangeListener::OnHdrAdded(nsIMsgDBHdr *aNewHdr, nsMs
   nsresult rv = m_folderWatching->GetMsgDatabase(nsnull, getter_AddRefs(msgDB));
   NS_ENSURE_SUCCESS(rv, rv);
   PRBool match = PR_FALSE;
+  m_searchSession->AddScopeTerm(nsMsgSearchScope::offlineMail, m_folderWatching);
   rv = m_searchSession->MatchHdr(aNewHdr, msgDB, &match);
+  m_searchSession->ClearScopes();
   if (match)
   {
     nsCOMPtr <nsIMsgDatabase> virtDatabase;
@@ -2780,6 +2808,9 @@ NS_IMETHODIMP VirtualFolderChangeListener::OnHdrAdded(nsIMsgDBHdr *aNewHdr, nsMs
       m_virtualFolder->SetHasNewMessages(PR_TRUE);
       m_virtualFolder->SetNumNewMessages(numNewMessages + 1);
     }
+    nsXPIDLCString searchUri;
+    m_virtualFolder->GetURI(getter_Copies(searchUri));
+    msgDB->UpdateHdrInCache(searchUri, aNewHdr, PR_TRUE);
     dbFolderInfo->ChangeNumMessages(1);
     m_virtualFolder->UpdateSummaryTotals(true); // force update from db.
     virtDatabase->Commit(nsMsgDBCommitType::kLargeCommit);
@@ -2822,13 +2853,15 @@ nsresult nsMsgAccountManager::GetVirtualFoldersFile(nsCOMPtr<nsILocalFile>& file
   return rv;
 }
 
-nsresult nsMsgAccountManager::LoadVirtualFolders()
+NS_IMETHODIMP nsMsgAccountManager::LoadVirtualFolders()
 {
   nsCOMPtr <nsILocalFile> file;
   GetVirtualFoldersFile(file);
   if (!file)
     return NS_ERROR_FAILURE;
 
+  m_loadingVirtualFolders = PR_TRUE;
+
   nsresult rv;
   nsCOMPtr<nsIMsgDBService> msgDBService = do_GetService(NS_MSGDB_SERVICE_CONTRACTID, &rv);
   if (msgDBService) 
@@ -2872,28 +2905,51 @@ nsresult nsMsgAccountManager::LoadVirtualFolders()
           NS_ENSURE_SUCCESS(rv, rv);
           if (virtualFolder)
           {
-            // need to add the folder as a sub-folder of its parent.
-            PRInt32 lastSlash = buffer.RFindChar('/');
-            nsDependentCSubstring parentUri(buffer, 0, lastSlash);
-            rdf->GetResource(parentUri, getter_AddRefs(resource));
-            nsCOMPtr <nsIMsgFolder> parentFolder = do_QueryInterface(resource);
-            if (parentFolder)
+            nsCOMPtr <nsIMsgFolder> grandParent;
+            nsCOMPtr <nsIMsgFolder> oldParent;
+            nsCOMPtr <nsIMsgFolder> parentFolder;
+            PRBool isServer;
+            do
             {
-              nsAutoString currentFolderNameStr;
-              nsCAutoString currentFolderNameCStr(Substring(buffer, lastSlash + 1, buffer.Length()));
-              nsUnescape(currentFolderNameCStr.BeginWriting());
-              CopyUTF8toUTF16(currentFolderNameCStr, currentFolderNameStr);
-              nsCOMPtr <nsIMsgFolder> childFolder;
-              nsCOMPtr <nsIMsgDatabase> db;
-              virtualFolder->GetMsgDatabase(nsnull, getter_AddRefs(db)); // force db to get created.
-              if (db)
-                rv = db->GetDBFolderInfo(getter_AddRefs(dbFolderInfo));
-              else
-                continue;
-              rv =  parentFolder->AddSubfolder(currentFolderNameStr, getter_AddRefs(childFolder));
+              // need to add the folder as a sub-folder of its parent.
+              PRInt32 lastSlash = buffer.RFindChar('/');
+              if (lastSlash == kNotFound)
+                break;
+              nsDependentCSubstring parentUri(buffer, 0, lastSlash);
+              // hold a reference so it won't get deleted before it's parented.
+              oldParent = parentFolder; 
+              
+              rdf->GetResource(parentUri, getter_AddRefs(resource));
+              parentFolder = do_QueryInterface(resource);
+              if (parentFolder)
+              {
+                nsAutoString currentFolderNameStr;
+                nsCAutoString currentFolderNameCStr(Substring(buffer, lastSlash + 1, buffer.Length()));
+                nsUnescape(currentFolderNameCStr.BeginWriting());
+                CopyUTF8toUTF16(currentFolderNameCStr, currentFolderNameStr);
+                nsCOMPtr <nsIMsgFolder> childFolder;
+                nsCOMPtr <nsIMsgDatabase> db;
+                virtualFolder->GetMsgDatabase(nsnull, getter_AddRefs(db)); // force db to get created.
+                if (db)
+                  rv = db->GetDBFolderInfo(getter_AddRefs(dbFolderInfo));
+                else
+                  break;
 
-              virtualFolder->SetFlag(MSG_FOLDER_FLAG_VIRTUAL);
-            }
+                parentFolder->AddSubfolder(currentFolderNameStr, getter_AddRefs(childFolder));
+                virtualFolder->SetFlag(MSG_FOLDER_FLAG_VIRTUAL);
+                if (childFolder)
+                  parentFolder->NotifyItemAdded(childFolder);
+                // here we make sure if our parent is rooted - if not, we're
+                // going to loop and add our parent as a child of its grandparent
+                // and repeat until we get to the server, or a folder that
+                // has its parent set.
+                parentFolder->GetParent(getter_AddRefs(grandParent));
+                parentFolder->GetIsServer(&isServer);
+                buffer.Truncate(lastSlash);
+              }
+              else
+                break;
+            } while (!grandParent && !isServer);
           }
         }
         else if (dbFolderInfo && Substring(buffer, 0, 6).Equals("scope="))
@@ -2903,20 +2959,11 @@ nsresult nsMsgAccountManager::LoadVirtualFolders()
           // and we have to add a pending listener for each of them.
           if (buffer.Length())
           {
-            nsCStringArray folderUris;
             dbFolderInfo->SetCharPtrProperty("searchFolderUri", buffer.get());
-            folderUris.ParseString(buffer.get(), "|");
-            for (PRInt32 i = 0; i < folderUris.Count(); i++)
-            {
-              rdf->GetResource(*(folderUris[i]), getter_AddRefs(resource));
-              nsCOMPtr <nsIMsgFolder> realFolder = do_QueryInterface(resource);
-              VirtualFolderChangeListener *dbListener = new VirtualFolderChangeListener();
-              m_virtualFolderListeners.AppendObject(dbListener);
-              dbListener->m_virtualFolder = virtualFolder;
-              dbListener->m_folderWatching = realFolder;
-              dbListener->Init();
-              msgDBService->RegisterPendingListener(realFolder, dbListener);
-            }
+            AddVFListenersForVF(virtualFolder, buffer.get(), rdf, msgDBService);
+          }
+          else // this folder is useless
+          {
           }
         }
         else if (dbFolderInfo && Substring(buffer, 0, 6).Equals("terms="))
@@ -2932,6 +2979,9 @@ nsresult nsMsgAccountManager::LoadVirtualFolders()
       }
     }
   }
+
+  m_loadingVirtualFolders = PR_FALSE;
+  m_virtualFoldersLoaded = PR_TRUE;
   return rv;
 }
 
@@ -2939,7 +2989,8 @@ nsresult nsMsgAccountManager::LoadVirtualFolders()
 
 NS_IMETHODIMP nsMsgAccountManager::SaveVirtualFolders()
 {
-
+  if (!m_virtualFoldersLoaded)
+    return NS_OK;
   nsCOMPtr<nsISupportsArray> allServers;
   nsresult rv = GetAllServers(getter_AddRefs(allServers));
   nsCOMPtr <nsILocalFile> file;
@@ -2947,7 +2998,7 @@ NS_IMETHODIMP nsMsgAccountManager::SaveVirtualFolders()
   {
     PRUint32 count = 0;
     allServers->Count(&count);
-    PRInt32 i;
+    PRUint32 i;
     nsCOMPtr <nsIOutputStream> outputStream;
     for (i = 0; i < count; i++) 
     {
@@ -2974,7 +3025,7 @@ NS_IMETHODIMP nsMsgAccountManager::SaveVirtualFolders()
             WriteLineToOutputStream("version=", "1", outputStream);
 
           }
-          for (PRInt32 folderIndex = 0; folderIndex < vfCount; folderIndex++)
+          for (PRUint32 folderIndex = 0; folderIndex < vfCount; folderIndex++)
           {
             nsCOMPtr <nsIRDFResource> folderRes (do_QueryElementAt(virtualFolders, folderIndex));
             nsCOMPtr <nsIMsgFolder> msgFolder = do_QueryInterface(folderRes);
@@ -2991,10 +3042,13 @@ NS_IMETHODIMP nsMsgAccountManager::SaveVirtualFolders()
               dbFolderInfo->GetCharPtrProperty("searchFolderUri", getter_Copies(srchFolderUri));
               dbFolderInfo->GetCharPtrProperty("searchStr", getter_Copies(searchTerms));
               folderRes->GetValueConst(&uri);
-              WriteLineToOutputStream("uri=", uri, outputStream);
-              WriteLineToOutputStream("scope=", srchFolderUri.get(), outputStream);
-              WriteLineToOutputStream("terms=", searchTerms.get(), outputStream);
-              WriteLineToOutputStream("searchOnline=", searchOnline ? "true" : "false", outputStream);
+              if (!srchFolderUri.IsEmpty() && !searchTerms.IsEmpty())
+              {
+                WriteLineToOutputStream("uri=", uri, outputStream);
+                WriteLineToOutputStream("scope=", srchFolderUri.get(), outputStream);
+                WriteLineToOutputStream("terms=", searchTerms.get(), outputStream);
+                WriteLineToOutputStream("searchOnline=", searchOnline ? "true" : "false", outputStream);
+              }
             }
           }
         }
@@ -3015,6 +3069,30 @@ nsresult nsMsgAccountManager::WriteLineToOutputStream(const char *prefix, const
   return NS_OK;
 }
 
+nsresult nsMsgAccountManager::AddVFListenersForVF(nsIMsgFolder *virtualFolder, 
+                                                  const char *srchFolderUris,
+                                                  nsIRDFService *rdf,
+                                                  nsIMsgDBService *msgDBService)
+{
+  nsCStringArray folderUris;
+  folderUris.ParseString(srchFolderUris, "|");
+  nsCOMPtr <nsIRDFResource> resource;
+
+  for (PRInt32 i = 0; i < folderUris.Count(); i++)
+  {
+    rdf->GetResource(*(folderUris[i]), getter_AddRefs(resource));
+    nsCOMPtr <nsIMsgFolder> realFolder = do_QueryInterface(resource);
+    VirtualFolderChangeListener *dbListener = new VirtualFolderChangeListener();
+    NS_ENSURE_TRUE(dbListener, NS_ERROR_OUT_OF_MEMORY);
+    m_virtualFolderListeners.AppendObject(dbListener);
+    dbListener->m_virtualFolder = virtualFolder;
+    dbListener->m_folderWatching = realFolder;
+    dbListener->Init();
+    msgDBService->RegisterPendingListener(realFolder, dbListener);
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsMsgAccountManager::OnItemAdded(nsIRDFResource *parentItem, nsISupports *item)
 {
   nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(item);
@@ -3025,29 +3103,20 @@ NS_IMETHODIMP nsMsgAccountManager::OnItemAdded(nsIRDFResource *parentItem, nsISu
   folder->GetFlags(&folderFlags);
   nsresult rv = NS_OK;
   // need to make sure this isn't happening during loading of virtualfolders.dat
-  if (folderFlags & MSG_FOLDER_FLAG_VIRTUAL)
+  if (folderFlags & MSG_FOLDER_FLAG_VIRTUAL && !m_loadingVirtualFolders)
   {
     // When a new virtual folder is added, need to create a db Listener for it.
     nsCOMPtr<nsIMsgDBService> msgDBService = do_GetService(NS_MSGDB_SERVICE_CONTRACTID, &rv);
     if (msgDBService)
     {
-      VirtualFolderChangeListener *dbListener = new VirtualFolderChangeListener();
-      dbListener->m_virtualFolder = folder;
       nsCOMPtr <nsIMsgDatabase> virtDatabase;
       nsCOMPtr <nsIDBFolderInfo> dbFolderInfo;
-      m_virtualFolderListeners.AppendObject(dbListener);
       rv = folder->GetDBFolderInfoAndDB(getter_AddRefs(dbFolderInfo), getter_AddRefs(virtDatabase));
       NS_ENSURE_SUCCESS(rv, rv);
       nsXPIDLCString srchFolderUri;
       dbFolderInfo->GetCharPtrProperty("searchFolderUri", getter_Copies(srchFolderUri));
-      // if we supported cross folder virtual folders, we'd have a list of folders uris,
-      // and we'd have to add a pending listener for each of them.
-      rv = GetExistingFolder(srchFolderUri.get(), getter_AddRefs(dbListener->m_folderWatching));
-      if (dbListener->m_folderWatching)
-      {
-        dbListener->Init();
-        msgDBService->RegisterPendingListener(dbListener->m_folderWatching, dbListener);
-      }
+      nsCOMPtr<nsIRDFService> rdf(do_GetService("@mozilla.org/rdf/rdf-service;1", &rv));
+      AddVFListenersForVF(folder, srchFolderUri, rdf, msgDBService);
     }
     rv = SaveVirtualFolders();
   }
@@ -3065,7 +3134,8 @@ NS_IMETHODIMP nsMsgAccountManager::OnItemRemoved(nsIRDFResource *parentItem, nsI
   folder->GetFlags(&folderFlags);
   if (folderFlags & MSG_FOLDER_FLAG_VIRTUAL) // if we removed a VF, flush VF list to disk.
     rv = SaveVirtualFolders();
-
+  // clear flags on deleted folder, especially MSG_FOLDER_FLAG_VIRTUAL
+  folder->SetFlags(0);
   // need to check if the underlying folder for a VF was removed, in which case we need to
   // remove the virtual folder.
 
@@ -3093,7 +3163,7 @@ NS_IMETHODIMP nsMsgAccountManager::OnItemUnicharPropertyChanged(nsIRDFResource *
 }
 
 
-NS_IMETHODIMP nsMsgAccountManager::OnItemPropertyFlagChanged(nsISupports *item, nsIAtom *property, PRUint32 oldFlag, PRUint32 newFlag)
+NS_IMETHODIMP nsMsgAccountManager::OnItemPropertyFlagChanged(nsIMsgDBHdr *item, nsIAtom *property, PRUint32 oldFlag, PRUint32 newFlag)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
