@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
+ *   David Bienvenu <bienvenu@nventure.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -63,10 +64,13 @@
 #include "nsIMsgHdr.h"
 #include "nsTraceRefcnt.h"
 #include "nsIMsgFolder.h" // TO include biffState enum. Change to bool later...
-#include "nsArray.h"
+#include "nsIMutableArray.h"
 #include "nsIPop3IncomingServer.h"
+#include "nsINntpIncomingServer.h"
 #include "nsTextFormatter.h"
 #include "nsIStringBundle.h"
+#include "nsIPrompt.h"
+#include "nsIMsgAccountManager.h"
 
 #define MESSENGER_STRING_URL       "chrome://messenger/locale/messenger.properties"
 
@@ -107,6 +111,9 @@ nsIRDFResource* nsMsgFolderDataSource::kNC_ImapShared = nsnull;
 nsIRDFResource* nsMsgFolderDataSource::kNC_Synchronize = nsnull;
 nsIRDFResource* nsMsgFolderDataSource::kNC_SyncDisabled = nsnull;
 nsIRDFResource* nsMsgFolderDataSource::kNC_CanSearchMessages = nsnull;
+nsIRDFResource* nsMsgFolderDataSource::kNC_UnreadFolders = nsnull;
+nsIRDFResource* nsMsgFolderDataSource::kNC_FavoriteFolders = nsnull;
+nsIRDFResource* nsMsgFolderDataSource::kNC_RecentFolders = nsnull;
 
 // commands
 nsIRDFResource* nsMsgFolderDataSource::kNC_Delete= nsnull;
@@ -135,6 +142,7 @@ nsIAtom * nsMsgFolderDataSource::kNameAtom = nsnull;
 nsIAtom * nsMsgFolderDataSource::kSynchronizeAtom = nsnull;
 nsIAtom * nsMsgFolderDataSource::kOpenAtom = nsnull;
 nsIAtom * nsMsgFolderDataSource::kIsDeferredAtom = nsnull;
+nsIAtom * nsMsgFolderDataSource::kIsSecureAtom = nsnull;
 nsIAtom * nsMsgFolderDataSource::kCanFileMessagesAtom = nsnull;
 nsIAtom * nsMsgFolderDataSource::kInVFEditSearchScopeAtom = nsnull;
 
@@ -158,6 +166,10 @@ nsMsgFolderDataSource::nsMsgFolderDataSource()
     rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_NAME),    &kNC_Name);
     rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_OPEN),    &kNC_Open);
     rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREENAME),    &kNC_FolderTreeName);
+    rdf->GetResource(NS_LITERAL_CSTRING("mailnewsunreadfolders:/"),    &kNC_UnreadFolders);
+    rdf->GetResource(NS_LITERAL_CSTRING("mailnewsfavefolders:/"),    &kNC_FavoriteFolders);
+    rdf->GetResource(NS_LITERAL_CSTRING("mailnewsrecentfolders:/"),    &kNC_RecentFolders);
+
     rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREESIMPLENAME),    &kNC_FolderTreeSimpleName);
     rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_NAME_SORT),    &kNC_NameSort);
     rdf->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREENAME_SORT),    &kNC_FolderTreeNameSort);
@@ -216,6 +228,7 @@ nsMsgFolderDataSource::nsMsgFolderDataSource()
     kSynchronizeAtom             = NS_NewAtom("Synchronize");
     kOpenAtom                    = NS_NewAtom("open");
     kIsDeferredAtom              = NS_NewAtom("isDeferred");
+    kIsSecureAtom                = NS_NewAtom("isSecure");
     kCanFileMessagesAtom         = NS_NewAtom("canFileMessages");
     kInVFEditSearchScopeAtom     = NS_NewAtom("inVFEditSearchScope");
 
@@ -305,6 +318,7 @@ nsMsgFolderDataSource::~nsMsgFolderDataSource (void)
     NS_RELEASE(kSynchronizeAtom);
     NS_RELEASE(kOpenAtom);
     NS_RELEASE(kIsDeferredAtom);
+    NS_RELEASE(kIsSecureAtom);
     NS_RELEASE(kCanFileMessagesAtom);
     NS_RELEASE(kInVFEditSearchScopeAtom);
 
@@ -340,8 +354,7 @@ nsresult nsMsgFolderDataSource::Init()
       nsIFolderListener::removed |
       nsIFolderListener::intPropertyChanged |
       nsIFolderListener::boolPropertyChanged |
-      nsIFolderListener::unicharPropertyChanged |
-      nsIFolderListener::propertyFlagChanged);
+      nsIFolderListener::unicharPropertyChanged);
 
   return NS_OK;
 }
@@ -521,13 +534,7 @@ NS_IMETHODIMP nsMsgFolderDataSource::GetTargets(nsIRDFResource* source,
       (kNC_SyncDisabled == property) ||
       (kNC_CanSearchMessages == property))
     {
-      nsSingletonEnumerator* cursor =
-        new nsSingletonEnumerator(property);
-      if (cursor == nsnull)
-        return NS_ERROR_OUT_OF_MEMORY;
-      NS_ADDREF(cursor);
-      *targets = cursor;
-      rv = NS_OK;
+      return NS_NewSingletonEnumerator(targets, property);
     }
   }
   if(!*targets)
@@ -722,9 +729,9 @@ nsMsgFolderDataSource::GetAllCmds(nsIRDFResource* source,
   nsCOMPtr<nsIMsgFolder> folder(do_QueryInterface(source, &rv));
   if (NS_FAILED(rv)) return rv;
 
-  nsCOMPtr<nsIMutableArray> cmds;
-  NS_NewArray(getter_AddRefs(cmds));
-  if (!cmds) return rv;
+  nsCOMPtr<nsIMutableArray> cmds =
+    do_CreateInstance(NS_ARRAY_CONTRACTID);
+  NS_ENSURE_STATE(cmds);
 
   cmds->AppendElement(kNC_Delete, PR_FALSE);
   cmds->AppendElement(kNC_ReallyDelete, PR_FALSE);
@@ -793,6 +800,20 @@ nsMsgFolderDataSource::DoCommand(nsISupportsArray/*<nsIRDFResource>*/* aSources,
 {
   nsresult rv = NS_OK;
   nsCOMPtr<nsISupports> supports;
+  nsCOMPtr<nsIMsgWindow> window;
+
+  // callers can pass in the msgWindow as the last element of the arguments
+  // array. If they do, we'll use that as the msg window for progress, etc.
+  if (aArguments)
+  {
+    PRUint32 numArgs;
+    aArguments->Count(&numArgs);
+    if (numArgs > 1)
+      window = do_QueryElementAt(aArguments, numArgs - 1);
+  }
+  if (!window)
+    window = mWindow;
+
   // XXX need to handle batching of command applied to all sources
 
   PRUint32 cnt = 0;
@@ -808,37 +829,37 @@ nsMsgFolderDataSource::DoCommand(nsISupportsArray/*<nsIRDFResource>*/* aSources,
     {
       if ((aCommand == kNC_Delete))
       {
-        rv = DoDeleteFromFolder(folder, aArguments, mWindow, PR_FALSE);
+        rv = DoDeleteFromFolder(folder, aArguments, window, PR_FALSE);
       }
       if ((aCommand == kNC_ReallyDelete))
       {
-        rv = DoDeleteFromFolder(folder, aArguments, mWindow, PR_TRUE);
+        rv = DoDeleteFromFolder(folder, aArguments, window, PR_TRUE);
       }
       else if((aCommand == kNC_NewFolder)) 
       {
-        rv = DoNewFolder(folder, aArguments);
+        rv = DoNewFolder(folder, aArguments, window);
       }
       else if((aCommand == kNC_GetNewMessages))
       {
         nsCOMPtr<nsIMsgIncomingServer> server = do_QueryElementAt(aArguments, i, &rv);
         NS_ENSURE_SUCCESS(rv, rv);
-        rv = server->GetNewMessages(folder, mWindow, nsnull);
+        rv = server->GetNewMessages(folder, window, nsnull);
       }
       else if((aCommand == kNC_Copy))
       {
-        rv = DoCopyToFolder(folder, aArguments, mWindow, PR_FALSE);
+        rv = DoCopyToFolder(folder, aArguments, window, PR_FALSE);
       }
       else if((aCommand == kNC_Move))
       {
-        rv = DoCopyToFolder(folder, aArguments, mWindow, PR_TRUE);
+        rv = DoCopyToFolder(folder, aArguments, window, PR_TRUE);
       }
       else if((aCommand == kNC_CopyFolder))
       {
-        rv = DoFolderCopyToFolder(folder, aArguments, mWindow, PR_FALSE);
+        rv = DoFolderCopyToFolder(folder, aArguments, window, PR_FALSE);
       }
       else if((aCommand == kNC_MoveFolder))
       {
-        rv = DoFolderCopyToFolder(folder, aArguments, mWindow, PR_TRUE);
+        rv = DoFolderCopyToFolder(folder, aArguments, window, PR_TRUE);
       }
       else if((aCommand == kNC_MarkAllMessagesRead))
       {
@@ -846,15 +867,15 @@ nsMsgFolderDataSource::DoCommand(nsISupportsArray/*<nsIRDFResource>*/* aSources,
       }
       else if ((aCommand == kNC_Compact))
       {
-        rv = folder->Compact(nsnull, mWindow);
+        rv = folder->Compact(nsnull, window);
       }
       else if ((aCommand == kNC_CompactAll))
       {
-        rv = folder->CompactAll(nsnull, mWindow, nsnull, PR_TRUE, nsnull);
+        rv = folder->CompactAll(nsnull, window, nsnull, PR_TRUE, nsnull);
       }
       else if ((aCommand == kNC_EmptyTrash))
       {
-          rv = folder->EmptyTrash(mWindow, nsnull);
+          rv = folder->EmptyTrash(window, nsnull);
       }
       else if ((aCommand == kNC_Rename))
       {
@@ -864,7 +885,7 @@ nsMsgFolderDataSource::DoCommand(nsISupportsArray/*<nsIRDFResource>*/* aSources,
           nsXPIDLString name;
           literal->GetValue(getter_Copies(name));
 
-          rv = folder->Rename(name.get(),mWindow);
+          rv = folder->Rename(name.get(), window);
         }
       }
     }
@@ -888,7 +909,6 @@ NS_IMETHODIMP nsMsgFolderDataSource::OnItemRemoved(nsIRDFResource *parentItem, n
   return OnItemAddedOrRemoved(parentItem, item, PR_FALSE);
 }
 
-
 nsresult nsMsgFolderDataSource::OnItemAddedOrRemoved(nsIRDFResource *parentItem, nsISupports *item, PRBool added)
 {
   nsCOMPtr<nsIRDFNode> itemNode(do_QueryInterface(item));
@@ -921,6 +941,16 @@ nsMsgFolderDataSource::OnItemIntPropertyChanged(nsIRDFResource *resource,
     OnUnreadMessagePropertyChanged(resource, oldValue, newValue);
   else if (kFolderSizeAtom == property)
     OnFolderSizePropertyChanged(resource, oldValue, newValue);
+  else if (kBiffStateAtom == property) {
+    // be careful about skipping if oldValue == newValue
+    // see the comment in nsMsgFolder::SetBiffState() about filters
+
+    nsCOMPtr<nsIRDFNode> biffNode;
+    nsresult rv = createBiffStateNodeFromFlag(newValue, getter_AddRefs(biffNode));
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    NotifyPropertyChanged(resource, kNC_BiffState, biffNode);
+  }
   return NS_OK;
 }
 
@@ -962,6 +992,8 @@ nsMsgFolderDataSource::OnItemBoolPropertyChanged(nsIRDFResource *resource,
       NotifyPropertyChanged(resource, kNC_Open, literalNode);
     else if (kIsDeferredAtom == property) 
       NotifyPropertyChanged(resource, kNC_IsDeferred, literalNode, oldLiteralNode);
+    else if (kIsSecureAtom == property)
+      NotifyPropertyChanged(resource, kNC_IsSecure, literalNode, oldLiteralNode);
     else if (kCanFileMessagesAtom == property)
       NotifyPropertyChanged(resource, kNC_CanFileMessages, literalNode, oldLiteralNode);
     else if (kInVFEditSearchScopeAtom == property)
@@ -972,39 +1004,11 @@ nsMsgFolderDataSource::OnItemBoolPropertyChanged(nsIRDFResource *resource,
 }
 
 NS_IMETHODIMP
-nsMsgFolderDataSource::OnItemPropertyFlagChanged(nsISupports *item,
+nsMsgFolderDataSource::OnItemPropertyFlagChanged(nsIMsgDBHdr *item,
                                                  nsIAtom *property,
                                                  PRUint32 oldFlag,
                                                  PRUint32 newFlag)
 {
-  nsresult rv = NS_OK;
-
-  if (kBiffStateAtom == property) {
-    // for Incoming biff (to turn it on) the item is of type nsIMsgFolder (see nsMsgFolder::SetBiffState)
-    // for clearing the biff the item is of type nsIMsgDBHdr (see nsMsgDBFolder::OnKeyChange)
-    // so check for both of these here
-    nsCOMPtr<nsIMsgFolder> folder(do_QueryInterface(item));
-    if(!folder) {
-      nsCOMPtr<nsIMsgDBHdr> msgHdr  = do_QueryInterface(item);
-      if (msgHdr)
-        rv = msgHdr->GetFolder(getter_AddRefs(folder));
-      if(NS_FAILED(rv))
-        return rv;
-    }
-
-    nsCOMPtr<nsIRDFResource> resource(do_QueryInterface(folder));
-    if(resource) {
-      // be careful about skipping if oldFlag == newFlag
-      // see the comment in nsMsgFolder::SetBiffState() about filters
-
-      nsCOMPtr<nsIRDFNode> biffNode;
-      rv = createBiffStateNodeFromFlag(newFlag, getter_AddRefs(biffNode));
-      NS_ENSURE_SUCCESS(rv,rv);
-
-      NotifyPropertyChanged(resource, kNC_BiffState, biffNode);
-    }
-  }
-
   return NS_OK;
 }
 
@@ -1126,13 +1130,17 @@ nsMsgFolderDataSource::createFolderNameNode(nsIMsgFolder *folder,
   return NS_OK;
 }
 
+nsresult nsMsgFolderDataSource::GetFolderDisplayName(nsIMsgFolder *folder, PRUnichar **folderName)
+{
+  return folder->GetAbbreviatedName(folderName);
+}
 
 nsresult 
 nsMsgFolderDataSource::createFolderTreeNameNode(nsIMsgFolder *folder,
                                                 nsIRDFNode **target)
 {
   nsXPIDLString name;
-  nsresult rv = folder->GetAbbreviatedName(getter_Copies(name));
+  nsresult rv = GetFolderDisplayName(folder, getter_Copies(name));
   if (NS_FAILED(rv)) return rv;
   nsAutoString nameString(name);
   PRInt32 unreadMessages;
@@ -1148,7 +1156,7 @@ nsMsgFolderDataSource::createFolderTreeNameNode(nsIMsgFolder *folder,
 nsresult nsMsgFolderDataSource::createFolderTreeSimpleNameNode(nsIMsgFolder * folder, nsIRDFNode **target)
 {
   nsXPIDLString name;
-  nsresult rv = folder->GetAbbreviatedName(getter_Copies(name));
+  nsresult rv = GetFolderDisplayName(folder, getter_Copies(name));
   if (NS_FAILED(rv)) return rv;
 
   createNode(name.get(), target, getRDFService());
@@ -1191,6 +1199,8 @@ nsMsgFolderDataSource::createFolderSpecialNode(nsIMsgFolder *folder,
     specialFolderString.AssignLiteral("Templates");
   else if (flags & MSG_FOLDER_FLAG_JUNK)
     specialFolderString.AssignLiteral("Junk");
+  else if (flags & MSG_FOLDER_FLAG_VIRTUAL)
+    specialFolderString.AssignLiteral("Virtual");
   else {
     // XXX why do this at all? or just ""
     specialFolderString.AssignLiteral("none");
@@ -1213,7 +1223,7 @@ nsMsgFolderDataSource::createFolderServerTypeNode(nsIMsgFolder* folder,
   rv = server->GetType(getter_Copies(serverType));
   if (NS_FAILED(rv)) return rv;
 
-  createNode(NS_ConvertASCIItoUCS2(serverType).get(), target, getRDFService());
+  createNode(NS_ConvertASCIItoUTF16(serverType).get(), target, getRDFService());
   return NS_OK;
 }
 
@@ -1252,7 +1262,7 @@ nsMsgFolderDataSource::createFolderRedirectorTypeNode(nsIMsgFolder* folder,
   rv = server->GetRedirectorType(getter_Copies(redirectorType));
   if (NS_FAILED(rv)) return rv;
 
-  createNode(NS_ConvertASCIItoUCS2(redirectorType).get(), target, getRDFService());
+  createNode(NS_ConvertASCIItoUTF16(redirectorType).get(), target, getRDFService());
   return NS_OK;
 }
 
@@ -1467,17 +1477,23 @@ nsMsgFolderDataSource::createFolderIsSecureNode(nsIMsgFolder* folder,
                                                   nsIRDFNode **target)
 {
   nsresult rv;
-  PRBool isSecure;
+  PRBool isSecure = PR_FALSE;
 
   nsCOMPtr<nsIMsgIncomingServer> server;
   rv = folder->GetServer(getter_AddRefs(server));
-  if (NS_FAILED(rv) || !server) {
-    // this could be a folder, not a server, so that's ok;
-    isSecure = PR_FALSE;
-  }
-  else {
-    rv = server->GetIsSecure(&isSecure);
-    if (NS_FAILED(rv)) return rv;
+
+  if (NS_SUCCEEDED(rv) && server) {
+    nsCOMPtr<nsINntpIncomingServer> nntpIncomingServer = do_QueryInterface(server);
+
+    if(nntpIncomingServer)  
+      rv = server->GetIsSecure(&isSecure);
+    else {
+      PRInt32 socketType;
+      rv = server->GetSocketType(&socketType);
+      if (NS_SUCCEEDED(rv) && (socketType == nsIMsgIncomingServer::alwaysUseTLS || 
+                              socketType == nsIMsgIncomingServer::useSSL))
+        isSecure = PR_TRUE;
+    }
   }
 
   *target = (isSecure) ? kTrueLiteral : kFalseLiteral;
@@ -1619,7 +1635,7 @@ nsMsgFolderDataSource::createCharsetNode(nsIMsgFolder *folder, nsIRDFNode **targ
   nsXPIDLCString charset;
   nsresult rv = folder->GetCharset(getter_Copies(charset));
   if (NS_SUCCEEDED(rv))
-    createNode(NS_ConvertASCIItoUCS2(charset).get(), target, getRDFService());
+    createNode(NS_ConvertASCIItoUTF16(charset).get(), target, getRDFService());
   else
     createNode(EmptyString().get(), target, getRDFService());
   return NS_OK;
@@ -1733,6 +1749,8 @@ nsMsgFolderDataSource::OnUnreadMessagePropertyChanged(nsIRDFResource *folderReso
     else if(oldValue > 0 && newValue <= 0)
     {
       NotifyPropertyChanged(folderResource, kNC_HasUnreadMessages, kFalseLiteral);
+      // this isn't quite right - parents could still have other children with 
+      // unread messages. NotifyAncestors will have to figure that out...
       NotifyAncestors(folder, kNC_SubfoldersHaveUnreadMessages, kFalseLiteral);
     }
 
@@ -1760,7 +1778,7 @@ nsresult
 nsMsgFolderDataSource::NotifyFolderTreeSimpleNameChanged(nsIMsgFolder* aFolder, nsIRDFResource *folderResource)
 {
   nsXPIDLString abbreviatedName;
-  nsresult rv = aFolder->GetAbbreviatedName(getter_Copies(abbreviatedName));
+  nsresult rv = GetFolderDisplayName(aFolder, getter_Copies(abbreviatedName));
   if (NS_SUCCEEDED(rv)) {
     nsCOMPtr<nsIRDFNode> newNameNode;
     createNode(abbreviatedName.get(), getter_AddRefs(newNameNode), getRDFService());
@@ -1776,7 +1794,7 @@ nsMsgFolderDataSource::NotifyFolderTreeNameChanged(nsIMsgFolder* aFolder,
                                                    PRInt32 aUnreadMessages)
 {
   nsXPIDLString name;
-  nsresult rv = aFolder->GetAbbreviatedName(getter_Copies(name));
+  nsresult rv = GetFolderDisplayName(aFolder, getter_Copies(name));
   if (NS_SUCCEEDED(rv)) {
     nsAutoString newNameString(name);
 			
@@ -1817,6 +1835,14 @@ nsMsgFolderDataSource::NotifyAncestors(nsIMsgFolder *aFolder, nsIRDFResource *aP
   nsCOMPtr<nsIRDFResource> parentFolderResource = do_QueryInterface(parentMsgFolder,&rv);
   NS_ENSURE_SUCCESS(rv,rv);
 
+  // if we're setting the subFoldersHaveUnreadMessages property to false, check
+  // if the folder really doesn't have subfolders with unread messages.
+  if (aPropertyResource == kNC_SubfoldersHaveUnreadMessages && aNode == kFalseLiteral)
+  {
+    nsCOMPtr <nsIRDFNode> unreadMsgsNode;
+    createHasUnreadMessagesNode(parentMsgFolder, PR_TRUE, getter_AddRefs(unreadMsgsNode));
+    aNode = unreadMsgsNode;
+  }
   NotifyPropertyChanged(parentFolderResource, aPropertyResource, aNode);
 
   return NotifyAncestors(parentMsgFolder, aPropertyResource, aNode);
@@ -1912,6 +1938,8 @@ nsMsgFolderDataSource::GetNumMessagesNode(PRInt32 aNumMessages, nsIRDFNode **nod
   return NS_OK;
 }
 
+#define DIVISIONWITHCEIL(num, div) (num/div+((num%div>0)?1:0))
+
 nsresult 
 nsMsgFolderDataSource::GetFolderSizeNode(PRInt32 aFolderSize, nsIRDFNode **aNode)
 {
@@ -1923,15 +1951,15 @@ nsMsgFolderDataSource::GetFolderSizeNode(PRInt32 aFolderSize, nsIRDFNode **aNode
   else
   {
     nsAutoString sizeString;
-    if (folderSize < 1024)
-      folderSize = 1024; // make at least 1 k;
-    folderSize /= 1024;  // normalize into k;
-    PRBool sizeInMB = (folderSize > 1024);
+    // use Round or Ceil - bug #251202
+    folderSize = DIVISIONWITHCEIL(folderSize, 1024);  // normalize into k;
+    PRBool sizeInMB = (folderSize > 999); // 999, not 1024 - bug #251204
+
     // kKiloByteString/kMegaByteString are localized strings that we use
     // to get the right format to add on the "KB"/"MB" or equivalent
     nsTextFormatter::ssprintf(sizeString,
                               (sizeInMB) ? kMegaByteString : kKiloByteString,
-                              (sizeInMB) ? folderSize / 1024 : folderSize);
+                              (sizeInMB) ? DIVISIONWITHCEIL(folderSize, 1024) : folderSize);
     createNode(sizeString.get(), aNode, getRDFService());
   }
   return NS_OK;
@@ -2082,12 +2110,41 @@ nsresult nsMsgFolderDataSource::DoDeleteFromFolder(
   rv = folderArray->Count(&cnt);
   if (NS_FAILED(rv)) return rv;
   if (cnt > 0)
+  {
+    nsCOMPtr<nsIMsgFolder> folderToDelete = do_QueryElementAt(folderArray, 0);
+    PRUint32 folderFlags = 0;
+    if (folderToDelete)
+    {
+      folderToDelete->GetFlags(&folderFlags);
+      if (folderFlags & MSG_FOLDER_FLAG_VIRTUAL)
+      {
+        NS_ENSURE_ARG_POINTER(msgWindow);
+        nsCOMPtr<nsIStringBundleService> sBundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
+        nsCOMPtr<nsIStringBundle> sMessengerStringBundle;
+        nsXPIDLString confirmMsg;
+
+        if (NS_SUCCEEDED(rv) && sBundleService) 
+          rv = sBundleService->CreateBundle(MESSENGER_STRING_URL, getter_AddRefs(sMessengerStringBundle));
+        NS_ENSURE_SUCCESS(rv, rv);
+        sMessengerStringBundle->GetStringFromName(NS_LITERAL_STRING("confirmSavedSearchDeleteMessage").get(), getter_Copies(confirmMsg));
+
+        nsCOMPtr<nsIPrompt> dialog;
+        rv = msgWindow->GetPromptDialog(getter_AddRefs(dialog));
+        if (NS_SUCCEEDED(rv))
+        {
+          PRBool dialogResult;
+          rv = dialog->Confirm(nsnull, confirmMsg, &dialogResult);
+          if (!dialogResult)
+            return NS_OK;
+        }
+      }
+    }
     rv = folder->DeleteSubFolders(folderArray, msgWindow);
-  
+  }
   return rv;
 }
 
-nsresult nsMsgFolderDataSource::DoNewFolder(nsIMsgFolder *folder, nsISupportsArray *arguments)
+nsresult nsMsgFolderDataSource::DoNewFolder(nsIMsgFolder *folder, nsISupportsArray *arguments, nsIMsgWindow *window)
 {
   nsresult rv = NS_OK;
   nsCOMPtr<nsIRDFLiteral> literal = do_QueryElementAt(arguments, 0, &rv);
@@ -2096,7 +2153,7 @@ nsresult nsMsgFolderDataSource::DoNewFolder(nsIMsgFolder *folder, nsISupportsArr
     nsXPIDLString name;
     literal->GetValue(getter_Copies(name));
     
-    rv = folder->CreateSubfolder(name,mWindow);
+    rv = folder->CreateSubfolder(name, window);
     
   }
   return rv;
@@ -2114,7 +2171,7 @@ nsresult nsMsgFolderDataSource::DoFolderAssert(nsIMsgFolder *folder, nsIRDFResou
       const PRUnichar* value;
       rv = literal->GetValueConst(&value);
       if(NS_SUCCEEDED(rv))
-        rv = folder->SetCharset(NS_LossyConvertUCS2toASCII(value).get());
+        rv = folder->SetCharset(NS_LossyConvertUTF16toASCII(value).get());
     }
     else
       rv = NS_ERROR_FAILURE;
@@ -2207,3 +2264,388 @@ nsresult nsMsgFolderDataSource::DoFolderHasAssertion(nsIMsgFolder *folder,
   
   
 }
+
+nsMsgFlatFolderDataSource::nsMsgFlatFolderDataSource()
+{
+}
+
+nsMsgFlatFolderDataSource::~nsMsgFlatFolderDataSource()
+{
+}
+
+nsresult nsMsgFlatFolderDataSource::Init()
+{
+  nsIRDFService* rdf = getRDFService();
+  NS_ENSURE_TRUE(rdf, NS_ERROR_FAILURE);
+  nsCOMPtr<nsIRDFResource> source;
+  nsCAutoString dsUri(m_dsName);
+  dsUri.Append(":/");
+  rdf->GetResource(dsUri, getter_AddRefs(m_rootResource));
+
+  return nsMsgFolderDataSource::Init();
+}
+
+void nsMsgFlatFolderDataSource::Cleanup()
+{
+  m_folders.Clear();
+  nsMsgFolderDataSource::Cleanup();
+}
+
+NS_IMETHODIMP nsMsgFlatFolderDataSource::GetTarget(nsIRDFResource* source,
+                       nsIRDFResource* property,
+                       PRBool tv,
+                       nsIRDFNode** target)
+{
+  return (property == kNC_Child) 
+    ? NS_RDF_NO_VALUE
+    : nsMsgFolderDataSource::GetTarget(source, property, tv, target);
+}
+
+
+NS_IMETHODIMP nsMsgFlatFolderDataSource::GetTargets(nsIRDFResource* source,
+                                                nsIRDFResource* property,    
+                                                PRBool tv,
+                                                nsISimpleEnumerator** targets)
+{
+  if (kNC_Child != property)
+    return nsMsgFolderDataSource::GetTargets(source, property, tv, targets);
+
+  nsresult rv = NS_RDF_NO_VALUE;
+  if(!targets)
+    return NS_ERROR_NULL_POINTER;
+
+  if (ResourceIsOurRoot(source))
+  {
+    // need an enumerator that gives all folders with unread
+    nsCOMPtr <nsIMsgAccountManager> accountManager = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsCOMPtr<nsISupportsArray> allServers;
+    rv = accountManager->GetAllServers(getter_AddRefs(allServers));
+    nsCOMPtr <nsISupportsArray> allFolders = do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv);;
+    if (NS_SUCCEEDED(rv) && allServers)
+    {
+      PRUint32 count = 0;
+      allServers->Count(&count);
+      PRUint32 i;
+      for (i = 0; i < count; i++) 
+      {
+        nsCOMPtr<nsIMsgIncomingServer> server = do_QueryElementAt(allServers, i);
+        if (server)
+        {
+          nsCOMPtr <nsIMsgFolder> rootFolder;
+          server->GetRootFolder(getter_AddRefs(rootFolder));
+          if (rootFolder)
+          {
+            nsCOMPtr<nsIEnumerator> subFolders;
+            rv = rootFolder->GetSubFolders(getter_AddRefs(subFolders));
+
+            PRUint32 lastEntry;
+            allFolders->Count(&lastEntry);
+            rv = rootFolder->ListDescendents(allFolders);
+            PRUint32 newLastEntry;
+            allFolders->Count(&newLastEntry);
+            for (PRUint32 newEntryIndex = lastEntry; newEntryIndex < newLastEntry;)
+            {
+              nsCOMPtr <nsIMsgFolder> curFolder = do_QueryElementAt(allFolders, newEntryIndex);
+              if (!WantsThisFolder(curFolder))
+              {
+                allFolders->RemoveElementAt(newEntryIndex);
+                newLastEntry--;
+              }
+              else
+              {
+                // unfortunately, we need a separate array for this since
+                // ListDescendents takes an nsISupportsArray. But we want
+                // to use an nsCOMArrray for the DS since that's the 
+                // preferred mechanism.
+                m_folders.AppendObject(curFolder);
+                newEntryIndex++;
+              }
+            }
+          }
+        }
+      }
+      return NS_NewArrayEnumerator(targets, allFolders);
+    }
+  }
+  return NS_NewSingletonEnumerator(targets, property);
+}
+
+
+NS_IMETHODIMP nsMsgFlatFolderDataSource::GetURI(char* *aUri)
+{
+  nsCAutoString uri("rdf:");
+  uri.Append(m_dsName);
+  return (*aUri = ToNewCString(uri))
+    ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+}
+
+NS_IMETHODIMP nsMsgFlatFolderDataSource::HasAssertion(nsIRDFResource* source,
+                            nsIRDFResource* property,
+                            nsIRDFNode* target,
+                            PRBool tv,
+                            PRBool* hasAssertion)
+{
+  nsresult rv;
+  
+  nsCOMPtr<nsIMsgFolder> folder(do_QueryInterface(source, &rv));
+  // we need to check if the folder belongs in this datasource.
+  if (NS_SUCCEEDED(rv) && property != kNC_Open && property != kNC_Child)
+  {
+    if (WantsThisFolder(folder) && (kNC_Child != property))
+      return DoFolderHasAssertion(folder, property, target, tv, hasAssertion);
+  }
+  else if (property == kNC_Child && ResourceIsOurRoot(source)) // check if source is us
+  {
+    folder = do_QueryInterface(target);
+    if (folder)
+    {
+      nsCOMPtr<nsIMsgFolder> parentMsgFolder;
+      folder->GetParentMsgFolder(getter_AddRefs(parentMsgFolder));
+      // a folder without a parent must be getting deleted as part of
+      // the rename operation and is thus a folder we are 
+      // no longer interested in
+      if (parentMsgFolder && WantsThisFolder(folder))
+      {
+        *hasAssertion = PR_TRUE;
+        return NS_OK;
+      }
+    }
+  }
+  *hasAssertion = PR_FALSE;
+  return NS_OK;
+}
+
+nsresult nsMsgFlatFolderDataSource::OnItemAddedOrRemoved(nsIRDFResource *parentItem, nsISupports *item, PRBool added)
+{
+  // When a folder is added or removed, parentItem is the parent folder and item is the folder being
+  // added or removed. In a flat data source, there is no arc in the graph between the parent folder
+  // and the folder being added or removed. Our flat data source root (i.e. mailnewsunreadfolders:/) has 
+  // an arc with the child property to every folder in the data source.  We must change parentItem
+  // to be our data source root before calling nsMsgFolderDataSource::OnItemAddedOrRemoved. This ensures
+  // that datasource listeners such as the template builder properly handle add and remove 
+  // notifications on the flat datasource.
+  return nsMsgFolderDataSource::OnItemAddedOrRemoved(m_rootResource, item, added);
+}
+
+PRBool nsMsgFlatFolderDataSource::ResourceIsOurRoot(nsIRDFResource *resource)
+{
+  return m_rootResource.get() == resource;
+}
+
+PRBool nsMsgFlatFolderDataSource::WantsThisFolder(nsIMsgFolder *folder)
+{
+  NS_ASSERTION(PR_FALSE, "must be overridden");
+  return PR_FALSE;
+}
+
+nsresult nsMsgFlatFolderDataSource::GetFolderDisplayName(nsIMsgFolder *folder, PRUnichar **folderName)
+{
+  nsXPIDLString curFolderName;
+  folder->GetName(getter_Copies(curFolderName));
+  PRUint32 foldersCount = m_folders.Count();
+  nsXPIDLString otherFolderName;
+  for (PRUint32 index = 0; index < foldersCount; index++)
+  {
+    if (folder == m_folders[index]) // ignore ourselves.
+      continue;
+    m_folders[index]->GetName(getter_Copies(otherFolderName));
+    if (otherFolderName.Equals(curFolderName))
+    {
+      nsCOMPtr <nsIMsgIncomingServer> server;
+      folder->GetServer(getter_AddRefs(server));
+      if (server)
+      {
+        nsXPIDLString serverName;
+        server->GetPrettyName(getter_Copies(serverName));
+        curFolderName.Append(NS_LITERAL_STRING(" - "));
+        curFolderName.Append(serverName);
+        *folderName = ToNewUnicode(curFolderName);
+        return (*folderName) ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+      }
+    }
+  }
+  // check if folder name is unique - if not, append account name
+  return folder->GetAbbreviatedName(folderName);
+}
+
+
+PRBool nsMsgUnreadFoldersDataSource::WantsThisFolder(nsIMsgFolder *folder)
+{
+  PRInt32 numUnread;
+  folder->GetNumUnread(PR_FALSE, &numUnread);
+  return numUnread > 0;
+}
+
+nsresult nsMsgUnreadFoldersDataSource::NotifyPropertyChanged(nsIRDFResource *resource, 
+                    nsIRDFResource *property, nsIRDFNode *newNode, 
+                    nsIRDFNode *oldNode)
+{
+  // check if it's the has unread property that's changed; if so, see if we need
+  // to add this folder to the view.
+  // Then, call base class.
+  if (kNC_HasUnreadMessages == property)
+  {
+    nsCOMPtr<nsIMsgFolder> folder(do_QueryInterface(resource));
+    if (folder)
+    {
+      PRInt32 numUnread;
+      folder->GetNumUnread(PR_FALSE, &numUnread);
+      if (numUnread > 0)
+      {
+        if (m_folders.IndexOf(folder) == kNotFound)
+          m_folders.AppendObject(folder);
+        NotifyObservers(kNC_UnreadFolders, kNC_Child, resource, nsnull, PR_TRUE, PR_FALSE);
+      }
+    }
+  }
+  return nsMsgFolderDataSource::NotifyPropertyChanged(resource, property, 
+                                                newNode, oldNode);
+}
+
+PRBool nsMsgFavoriteFoldersDataSource::WantsThisFolder(nsIMsgFolder *folder)
+{
+  PRUint32 folderFlags;
+  folder->GetFlags(&folderFlags);
+  return folderFlags & MSG_FOLDER_FLAG_FAVORITE;
+}
+
+
+void nsMsgRecentFoldersDataSource::Cleanup()
+{
+  m_builtRecentFolders = PR_FALSE;
+  m_cutOffDate = 0;
+  nsMsgFlatFolderDataSource::Cleanup();
+}
+
+
+PRBool nsMsgRecentFoldersDataSource::WantsThisFolder(nsIMsgFolder *folder)
+{
+  if (!m_builtRecentFolders)
+  {
+    nsresult rv;
+    nsCOMPtr <nsIMsgAccountManager> accountManager = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsCOMPtr<nsISupportsArray> allServers;
+    rv = accountManager->GetAllServers(getter_AddRefs(allServers));
+    nsCOMPtr <nsISupportsArray> allFolders = do_CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, &rv);;
+    if (NS_SUCCEEDED(rv) && allServers)
+    {
+      PRUint32 count = 0;
+      allServers->Count(&count);
+      PRUint32 i;
+      for (i = 0; i < count; i++) 
+      {
+        nsCOMPtr<nsIMsgIncomingServer> server = do_QueryElementAt(allServers, i);
+        if (server)
+        {
+          nsCOMPtr <nsIMsgFolder> rootFolder;
+          server->GetRootFolder(getter_AddRefs(rootFolder));
+          if (rootFolder)
+          {
+            nsCOMPtr<nsIEnumerator> subFolders;
+            rv = rootFolder->GetSubFolders(getter_AddRefs(subFolders));
+
+            PRUint32 lastEntry;
+            allFolders->Count(&lastEntry);
+            rv = rootFolder->ListDescendents(allFolders);
+            PRUint32 newLastEntry;
+            allFolders->Count(&newLastEntry);
+            for (PRUint32 newEntryIndex = lastEntry; newEntryIndex < newLastEntry; newEntryIndex++)
+            {
+              nsCOMPtr <nsIMsgFolder> curFolder = do_QueryElementAt(allFolders, newEntryIndex);
+              nsXPIDLCString dateStr;
+              PRInt32 err;
+              curFolder->GetStringProperty(MRU_TIME_PROPERTY, getter_Copies(dateStr));
+              PRUint32 curFolderDate = (PRUint32) dateStr.ToInteger(&err);
+              if (err)
+                curFolderDate = 0;
+              if (curFolderDate > m_cutOffDate)
+              {
+                // if m_folders is "full", replace oldest folder with this folder,
+                // and adjust m_cutOffDate so that it's the mrutime 
+                // of the "new" oldest folder.
+                PRUint32 curFaveFoldersCount = m_folders.Count();
+                if (curFaveFoldersCount > m_maxNumFolders)
+                {
+                  PRUint32 indexOfOldestFolder = 0;
+                  PRUint32 oldestFaveDate = 0;
+                  PRUint32 newOldestFaveDate = 0;
+                  for (PRUint32 index = 0; index < curFaveFoldersCount; )
+                  {
+                    nsXPIDLCString curFaveFolderDateStr;
+                    m_folders[index]->GetStringProperty(MRU_TIME_PROPERTY, getter_Copies(curFaveFolderDateStr));
+                    PRUint32 curFaveFolderDate = (PRUint32) curFaveFolderDateStr.ToInteger(&err);
+                    if (!oldestFaveDate || curFaveFolderDate < oldestFaveDate)
+                    {
+                      indexOfOldestFolder = index;
+                      newOldestFaveDate = oldestFaveDate;
+                      oldestFaveDate = curFaveFolderDate;
+                    }
+                    if (!newOldestFaveDate || (index != indexOfOldestFolder
+                                                && curFaveFolderDate < newOldestFaveDate))
+                      newOldestFaveDate = curFaveFolderDate;
+                    index++;
+                  }
+                  if (curFolderDate > oldestFaveDate && m_folders.IndexOf(curFolder) == kNotFound)
+                    m_folders.ReplaceObjectAt(curFolder, indexOfOldestFolder);
+
+                  NS_ASSERTION(newOldestFaveDate >= m_cutOffDate, "cutoff date should be getting bigger");
+                  m_cutOffDate = newOldestFaveDate;
+                }
+                else if (m_folders.IndexOf(curFolder) == kNotFound)
+                  m_folders.AppendObject(curFolder);
+              }
+#ifdef DEBUG_David_Bienvenu
+              else
+              {
+                for (PRUint32 index = 0; index < m_folders.Count(); index++)
+                {
+                  nsXPIDLCString curFaveFolderDateStr;
+                  m_folders[index]->GetStringProperty(MRU_TIME_PROPERTY, getter_Copies(curFaveFolderDateStr));
+                  PRUint32 curFaveFolderDate = (PRUint32) curFaveFolderDateStr.ToInteger(&err);
+                  NS_ASSERTION(curFaveFolderDate > curFolderDate, "folder newer then faves but not added");
+                }
+              }
+#endif
+            }
+          }
+        }
+      }
+    }
+  }
+  m_builtRecentFolders = PR_TRUE;
+  return m_folders.IndexOf(folder) != kNotFound;
+}
+
+
+
+nsresult nsMsgRecentFoldersDataSource::NotifyPropertyChanged(nsIRDFResource *resource, 
+                    nsIRDFResource *property, nsIRDFNode *newNode, 
+                    nsIRDFNode *oldNode)
+{
+  // check if it's the has new property that's changed; if so, see if we need
+  // to add this folder to the view.
+  // Then, call base class.
+  if (kNC_NewMessages == property)
+  {
+    nsCOMPtr<nsIMsgFolder> folder(do_QueryInterface(resource));
+    if (folder)
+    {
+      PRBool hasNewMessages;
+      folder->GetHasNewMessages(&hasNewMessages);
+      if (hasNewMessages > 0)
+      {
+        if (m_folders.IndexOf(folder) == kNotFound)
+        {
+          m_folders.AppendObject(folder);
+          NotifyObservers(kNC_RecentFolders, kNC_Child, resource, nsnull, PR_TRUE, PR_FALSE);
+        }
+      }
+    }
+  }
+  return nsMsgFolderDataSource::NotifyPropertyChanged(resource, property, 
+                                                newNode, oldNode);
+}
