@@ -14,6 +14,11 @@
  *
  * The Original Code is mozilla.org code.
  *
+ * The Initial Developer of the Original Code is
+ * David Bienvenu.
+ * Portions created by the Initial Developer are Copyright (C) 2004
+ * the Initial Developer. All Rights Reserved.
+ *
  * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -43,36 +48,14 @@
 #define MSGHDR_CACHE_MAX_SIZE         8192  // Max msghdr cache entries.
 #define MSGHDR_CACHE_DEFAULT_SIZE     100
 
-PRUnichar * nsMsgGroupView::kTodayString = nsnull;
-PRUnichar * nsMsgGroupView::kYesterdayString = nsnull;
-PRUnichar * nsMsgGroupView::kLastWeekString = nsnull;
-PRUnichar * nsMsgGroupView::kTwoWeeksAgoString = nsnull;
-PRUnichar * nsMsgGroupView::kOldMailString = nsnull;
-
 nsMsgGroupView::nsMsgGroupView()
 {
-  if (!kTodayString) 
-  {
-    // priority strings
-    kTodayString = GetString(NS_LITERAL_STRING("today").get());
-    kYesterdayString = GetString(NS_LITERAL_STRING("yesterday").get());
-    kLastWeekString = GetString(NS_LITERAL_STRING("lastWeek").get());
-    kTwoWeeksAgoString = GetString(NS_LITERAL_STRING("twoWeeksAgo").get());
-    kOldMailString = GetString(NS_LITERAL_STRING("older").get());
-  }
+  m_dayChanged = PR_FALSE;
+  m_lastCurExplodedTime.tm_mday = 0;
 }
 
 nsMsgGroupView::~nsMsgGroupView()
 {
-  // release our global strings
-  if (gInstanceCount <= 1) 
-  {
-    nsCRT::free(kTodayString);
-    nsCRT::free(kYesterdayString);
-    nsCRT::free(kLastWeekString);
-    nsCRT::free(kTwoWeeksAgoString);
-    nsCRT::free(kOldMailString);
-  }
 }
 
 NS_IMETHODIMP nsMsgGroupView::Open(nsIMsgFolder *aFolder, nsMsgViewSortTypeValue aSortType, nsMsgViewSortOrderValue aSortOrder, nsMsgViewFlagsTypeValue aViewFlags, PRInt32 *aCount)
@@ -97,11 +80,44 @@ NS_IMETHODIMP nsMsgGroupView::Open(nsIMsgFolder *aFolder, nsMsgViewSortTypeValue
   return kHashEnumerateNext;
 }
 
+void nsMsgGroupView::InternalClose()
+{
+  if (m_db && m_sortType == nsMsgViewSortType::byDate)
+  {
+    nsCOMPtr <nsIDBFolderInfo> dbFolderInfo;
+    nsresult rv = m_db->GetDBFolderInfo(getter_AddRefs(dbFolderInfo));
+    if (dbFolderInfo)
+    {
+      PRUint32 expandFlags = 0;
+      PRUint32 num = GetSize();
+	  
+      for (PRUint32 i = 0; i < num; i++) 
+      {
+        if (m_flags[i] & MSG_VIEW_FLAG_ISTHREAD && ! (m_flags[i] & MSG_FLAG_ELIDED))
+        {
+          nsCOMPtr <nsIMsgDBHdr> msgHdr;
+          nsresult rv = GetMsgHdrForViewIndex(i, getter_AddRefs(msgHdr));
+          if (msgHdr)
+          {
+             nsHashKey *hashKey = AllocHashKeyForHdr(msgHdr);
+             if (hashKey)
+             {
+               expandFlags |=  1 << ((nsPRUint32Key *)hashKey)->GetValue();
+               delete hashKey;
+             }
+          }
+        }
+      }
+      dbFolderInfo->SetUint32Property("dateGroupFlags", expandFlags);
+    }
+  }
+  // enumerate m_groupsTable releasing the thread objects.
+  m_groupsTable.Reset(ReleaseThread);
+}
 
 NS_IMETHODIMP nsMsgGroupView::Close()
 {
-  // enumerate m_groupsTable releasing the thread objects.
-  m_groupsTable.Enumerate(ReleaseThread);
+  InternalClose();
   return nsMsgThreadedDBView::Close();
 }
 
@@ -109,6 +125,8 @@ nsHashKey *nsMsgGroupView::AllocHashKeyForHdr(nsIMsgDBHdr *msgHdr)
 {
   static nsXPIDLCString cStringKey;
   static nsXPIDLString stringKey;
+  nsresult rv;
+
   switch (m_sortType)
   {
     case nsMsgViewSortType::bySubject:
@@ -116,30 +134,38 @@ nsHashKey *nsMsgGroupView::AllocHashKeyForHdr(nsIMsgDBHdr *msgHdr)
       return new nsCStringKey(cStringKey.get());
       break;
     case nsMsgViewSortType::byAuthor:
-      (void) msgHdr->GetAuthor(getter_Copies(cStringKey));
-      return new nsCStringKey(cStringKey.get());
+      rv = nsMsgDBView::FetchAuthor(msgHdr, getter_Copies(stringKey));
+      return NS_SUCCEEDED(rv) ? new nsStringKey(stringKey.get()) : nsnull;
     case nsMsgViewSortType::byRecipient:
       (void) msgHdr->GetRecipients(getter_Copies(cStringKey));
       return new nsCStringKey(cStringKey.get());
     case nsMsgViewSortType::byAccount:
+    case nsMsgViewSortType::byTags:
       {
         nsCOMPtr <nsIMsgDatabase> dbToUse = m_db;
   
         if (!dbToUse) // probably search view
           GetDBForViewIndex(0, getter_AddRefs(dbToUse));
 
-        (void) FetchAccount(msgHdr, getter_Copies(stringKey));
-        return new nsStringKey (stringKey.get());
+        rv = (m_sortType == nsMsgViewSortType::byAccount) 
+          ? FetchAccount(msgHdr, getter_Copies(stringKey))
+          : FetchTags(msgHdr, getter_Copies(stringKey));
+        return NS_SUCCEEDED(rv) ? new nsStringKey(stringKey.get()) : nsnull;
 
       }
       break;
-    case nsMsgViewSortType::byLabel:
+    case nsMsgViewSortType::byAttachments:
       {
-        nsMsgLabelValue label;
-        msgHdr->GetLabel(&label);
-        return new nsPRUint32Key(label);
+        PRUint32 flags;
+        msgHdr->GetFlags(&flags);
+        return new nsPRUint32Key(flags & MSG_FLAG_ATTACHMENT ? 1 : 0);
+      }
+    case nsMsgViewSortType::byFlagged:
+      {
+        PRUint32 flags;
+        msgHdr->GetFlags(&flags);
+        return new nsPRUint32Key(flags & MSG_FLAG_MARKED ? 1 : 0);
       }
-      break;
     case nsMsgViewSortType::byPriority:
       {
         nsMsgPriorityValue priority;
@@ -158,22 +184,23 @@ nsHashKey *nsMsgGroupView::AllocHashKeyForHdr(nsIMsgDBHdr *msgHdr)
     {
       PRUint32 ageBucket = 1;
       PRTime dateOfMsg;
-      PRUint32 dateOfMsgInSeconds, currentTimeInSeconds;
 	    
       nsresult rv = msgHdr->GetDate(&dateOfMsg);
-      (void) msgHdr->GetDateInSeconds(&dateOfMsgInSeconds);
 
       PRTime currentTime = PR_Now();
-      PRExplodedTime explodedCurrentTime;
-      PR_ExplodeTime(currentTime, PR_LocalTimeParameters, &explodedCurrentTime);
+      PRExplodedTime currentExplodedTime;
+      PR_ExplodeTime(currentTime, PR_LocalTimeParameters, &currentExplodedTime);
       PRExplodedTime explodedMsgTime;
       PR_ExplodeTime(dateOfMsg, PR_LocalTimeParameters, &explodedMsgTime);
 
-      dateOfMsgInSeconds -= explodedMsgTime.tm_params.tp_gmt_offset;
+      if (m_lastCurExplodedTime.tm_mday &&
+         m_lastCurExplodedTime.tm_mday != currentExplodedTime.tm_mday)
+        m_dayChanged = PR_TRUE; // this will cause us to rebuild the view.
 
-      if (explodedCurrentTime.tm_year == explodedMsgTime.tm_year &&
-          explodedCurrentTime.tm_month == explodedMsgTime.tm_month &&
-          explodedCurrentTime.tm_mday == explodedMsgTime.tm_mday)
+      m_lastCurExplodedTime = currentExplodedTime;
+      if (currentExplodedTime.tm_year == explodedMsgTime.tm_year &&
+          currentExplodedTime.tm_month == explodedMsgTime.tm_month &&
+          currentExplodedTime.tm_mday == explodedMsgTime.tm_mday)
       {
         // same day...
         ageBucket = 1;
@@ -184,7 +211,10 @@ nsHashKey *nsMsgGroupView::AllocHashKeyForHdr(nsIMsgDBHdr *msgHdr)
         // some constants for calculation
         static PRInt64 microSecondsPerSecond;
         static PRInt64 microSecondsPerDay;
-
+	static PRInt64 secondsPerDay;
+	static PRInt64 microSecondsPer6Days;
+	static PRInt64 microSecondsPer13Days;
+			
         static PRBool bGotConstants = PR_FALSE;
         if ( !bGotConstants )
         {
@@ -196,41 +226,43 @@ nsHashKey *nsMsgGroupView::AllocHashKeyForHdr(nsIMsgDBHdr *msgHdr)
     
           // derivees
           LL_MUL( microSecondsPerDay,   secondsPerDay,      microSecondsPerSecond );
-          bGotConstants = PR_TRUE;
+          LL_MUL( microSecondsPer6Days, microSecondsPerDay, 6 );
+	  LL_MUL( microSecondsPer13Days, microSecondsPerDay, 13 );
+	  
+	  bGotConstants = PR_TRUE;
         }
 
-        const PRUint32 secondsPerDay = 60 * 60 * 24;
-        const PRUint32 secondsPer6Days = 60 * 60 * 24 * 6;
-        const PRUint32 secondsPer13Days = 60 * 60 * 24 * 13;
-        PRInt64 temp;
-        PRUint32 mostRecentMidnightSeconds;
-
-        LL_DIV(temp, currentTime, microSecondsPerSecond);
-        LL_L2UI(currentTimeInSeconds, temp);
-
-        PRUint32 nowSeconds;
-        LL_L2UI(nowSeconds, temp);
-
-        nowSeconds -= explodedCurrentTime.tm_params.tp_gmt_offset;
-
-        PRUint32 todaysSeconds = (nowSeconds % (60 * 60 * 24));
-        mostRecentMidnightSeconds = currentTimeInSeconds - todaysSeconds;
-        PRUint32 mostRecentWeekSeconds = mostRecentMidnightSeconds - secondsPer6Days;
-        PRUint32 yesterdayInSeconds = mostRecentMidnightSeconds - secondsPerDay;
-
+	// setting the time variables to local time
+        PRInt64 GMTLocalTimeShift;
+        LL_ADD( GMTLocalTimeShift, currentExplodedTime.tm_params.tp_gmt_offset, currentExplodedTime.tm_params.tp_dst_offset );
+        LL_MUL( GMTLocalTimeShift, GMTLocalTimeShift, microSecondsPerSecond );
+        LL_ADD( currentTime, currentTime, GMTLocalTimeShift );
+        LL_ADD( dateOfMsg, dateOfMsg, GMTLocalTimeShift );
+	
+	// the most recent midnight, counting from current time
+	PRInt64 todaysMicroSeconds, mostRecentMidnight;
+	LL_MOD( todaysMicroSeconds, currentTime, microSecondsPerDay );
+	LL_SUB( mostRecentMidnight, currentTime, todaysMicroSeconds );
+	PRInt64 yesterday;
+	LL_SUB( yesterday, mostRecentMidnight, microSecondsPerDay );
+	// most recent midnight minus 6 days
+	PRInt64 mostRecentWeek;
+	LL_SUB( mostRecentWeek, mostRecentMidnight, microSecondsPer6Days );
+	
         // was the message sent yesterday?
-        if (dateOfMsgInSeconds >= yesterdayInSeconds)
+        if ( LL_CMP( dateOfMsg, >=, yesterday ) )
         { // yes ....
           ageBucket = 2;
         }
-        else if (dateOfMsgInSeconds >= mostRecentWeekSeconds)
+        else if ( LL_CMP(dateOfMsg, >=, mostRecentWeek) )
         {
           ageBucket = 3;
         }
         else
         {
-          PRUint32 lastTwoWeeks = mostRecentMidnightSeconds - secondsPer13Days;
-          ageBucket = (dateOfMsgInSeconds >= lastTwoWeeks) ? 4 : 5;
+          PRInt64 lastTwoWeeks;
+	  LL_SUB( lastTwoWeeks, mostRecentMidnight, microSecondsPer13Days);
+	  ageBucket = LL_CMP(dateOfMsg, >=, lastTwoWeeks) ? 4 : 5;
         }
       }
       return new nsPRUint32Key(ageBucket);
@@ -248,6 +280,8 @@ nsMsgGroupThread *nsMsgGroupView::AddHdrToThread(nsIMsgDBHdr *msgHdr, PRBool *pN
   msgHdr->GetMessageKey(&msgKey);
   msgHdr->GetFlags(&msgFlags);
   nsHashKey *hashKey = AllocHashKeyForHdr(msgHdr);
+  if (!hashKey)
+    return nsnull;
 //  if (m_sortType == nsMsgViewSortType::byDate)
 //    msgKey = ((nsPRUint32Key *) hashKey)->GetValue();
   nsMsgGroupThread *foundThread = nsnull;
@@ -286,7 +320,7 @@ nsMsgGroupThread *nsMsgGroupView::AddHdrToThread(nsIMsgDBHdr *msgHdr, PRBool *pN
   }
   delete hashKey;
   if (foundThread)
-    foundThread->AddChild(msgHdr, nsnull, PR_FALSE, nsnull /* announcer */);
+    foundThread->AddChildFromGroupView(msgHdr, this);
   // check if new hdr became thread root
   if (!newThread && foundThread->m_keys[0] == msgKey)
   {
@@ -305,7 +339,7 @@ NS_IMETHODIMP nsMsgGroupView::OpenWithHdrs(nsISimpleEnumerator *aHeaders, nsMsgV
   nsresult rv = NS_OK;
 
   if (aSortType == nsMsgViewSortType::byThread || aSortType == nsMsgViewSortType::byId
-    || aSortType == nsMsgViewSortType::byNone)
+    || aSortType == nsMsgViewSortType::byNone || aSortType == nsMsgViewSortType::bySize)
     return NS_ERROR_INVALID_ARG;
 
   m_sortType = aSortType;
@@ -327,9 +361,18 @@ NS_IMETHODIMP nsMsgGroupView::OpenWithHdrs(nsISimpleEnumerator *aHeaders, nsMsgV
       AddHdrToThread(msgHdr, &notUsed);
     }
   }
-  *aCount = m_keys.GetSize();
+  PRUint32 expandFlags = 0;
   PRUint32 viewFlag = (m_sortType == nsMsgViewSortType::byDate) ? MSG_VIEW_FLAG_DUMMY : 0;
-  //go through the view updating the flags for threads with more than one message...
+  if (viewFlag)
+  {
+    nsCOMPtr <nsIDBFolderInfo> dbFolderInfo;
+    nsresult rv = m_db->GetDBFolderInfo(getter_AddRefs(dbFolderInfo));
+    if (dbFolderInfo)
+      dbFolderInfo->GetUint32Property("dateGroupFlags",  0, &expandFlags);
+
+  }
+  // go through the view updating the flags for threads with more than one message...
+  // and if grouped by date, expanding threads that were expanded before.
   for (PRUint32 viewIndex = 0; viewIndex < m_keys.GetSize(); viewIndex++)
   {
     nsCOMPtr <nsIMsgThread> thread;
@@ -340,13 +383,70 @@ NS_IMETHODIMP nsMsgGroupView::OpenWithHdrs(nsISimpleEnumerator *aHeaders, nsMsgV
       thread->GetNumChildren(&numChildren);
       if (numChildren > 1 || viewFlag)
         OrExtraFlag(viewIndex, viewFlag | MSG_VIEW_FLAG_HASCHILDREN);
+      if (expandFlags)
+      {
+        nsMsgGroupThread *groupThread = NS_STATIC_CAST(nsMsgGroupThread *, (nsIMsgThread *) thread);
+        if (expandFlags & (1 << groupThread->m_threadKey))
+        {
+          PRUint32 numExpanded;
+          ExpandByIndex(viewIndex, &numExpanded);
+          viewIndex += numExpanded;
+        }
+      }
     }
   }
+  *aCount = m_keys.GetSize();
   return rv;
 }
 
+// if the day has changed, we need to close and re-open the view.
+nsresult nsMsgGroupView::HandleDayChange()
+{
+  nsCOMPtr <nsISimpleEnumerator> headers;
+  if (NS_SUCCEEDED(m_db->EnumerateMessages(getter_AddRefs(headers))))
+  {
+    PRInt32 count;
+    m_dayChanged = PR_FALSE;
+    nsMsgKeyArray preservedSelection;
+    nsMsgKey curSelectedKey;
+    SaveAndClearSelection(&curSelectedKey, &preservedSelection);
+    InternalClose();
+    PRInt32 oldSize = GetSize();
+    // this is important, because the tree will ask us for our
+    // row count, which get determine from the number of keys.
+    m_keys.RemoveAll();
+    // be consistent
+    m_flags.RemoveAll();
+    m_levels.RemoveAll();
+
+    // this needs to happen after we remove all the keys, since RowCountChanged() will call our GetRowCount()
+    if (mTree) 
+      mTree->RowCountChanged(0, -oldSize);
+    DisableChangeUpdates();
+    nsresult rv = OpenWithHdrs(headers, m_sortType, m_sortOrder, m_viewFlags, &count);
+    EnableChangeUpdates();
+    if (mTree) 
+      mTree->RowCountChanged(0, GetSize());
+
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    // now, restore our desired selection
+    nsMsgKeyArray keyArray;
+    keyArray.Add(curSelectedKey);
+
+    return RestoreSelection(curSelectedKey, &keyArray);
+  }
+  return NS_OK;
+}
+
 nsresult nsMsgGroupView::OnNewHeader(nsIMsgDBHdr *newHdr, nsMsgKey aParentKey, PRBool /*ensureListed*/)
 {
+
+  // check if we're adding a header, and the current day has changed. If it has, we're just going to 
+  // close and re-open the view so things will be correctly categorized.
+  if (m_dayChanged)
+    return HandleDayChange();
+
   PRBool newThread;
   nsMsgGroupThread *thread = AddHdrToThread(newHdr, &newThread); 
   if (thread)
@@ -372,6 +472,7 @@ nsresult nsMsgGroupView::OnNewHeader(nsIMsgDBHdr *newHdr, nsMsgKey aParentKey, P
       if (! (m_flags[threadIndex] & MSG_FLAG_ELIDED))
       {
         PRUint32 msgIndexInThread = thread->m_keys.IndexOf(msgKey);
+        PRBool insertedAtThreadRoot = !msgIndexInThread;
         if (!msgIndexInThread && GroupViewUsesDummyRow())
           msgIndexInThread++;
 
@@ -408,7 +509,7 @@ nsresult nsMsgGroupView::OnNewHeader(nsIMsgDBHdr *newHdr, nsMsgKey aParentKey, P
         }
         // the call to NoteChange() has to happen after we add the key
         // as NoteChange() will call RowCountChanged() which will call our GetRowCount()
-        NoteChange((newThread && GroupViewUsesDummyRow()) ? threadIndex + msgIndexInThread - 1 : threadIndex + msgIndexInThread,
+        NoteChange((insertedAtThreadRoot && GroupViewUsesDummyRow()) ? threadIndex + msgIndexInThread - 1 : threadIndex + msgIndexInThread,
                       numRowsInserted, nsMsgViewNotificationCode::insertOrDelete);
         numRowsToInvalidate = msgIndexInThread;
       }
@@ -424,6 +525,11 @@ NS_IMETHODIMP nsMsgGroupView::OnHdrChange(nsIMsgDBHdr *aHdrChanged, PRUint32 aOl
 {
   nsCOMPtr <nsIMsgThread> thread;
 
+  // check if we're adding a header, and the current day has changed. If it has, we're just going to 
+  // close and re-open the view so things will be correctly categorized.
+  if (m_dayChanged)
+    return HandleDayChange();
+
   nsresult rv = GetThreadContainingMsgHdr(aHdrChanged, getter_AddRefs(thread));
   NS_ENSURE_SUCCESS(rv, rv);
   PRUint32 deltaFlags = (aOldFlags ^ aNewFlags);
@@ -436,6 +542,11 @@ NS_IMETHODIMP nsMsgGroupView::OnHdrChange(nsIMsgDBHdr *aHdrChanged, PRUint32 aOl
 NS_IMETHODIMP nsMsgGroupView::OnHdrDeleted(nsIMsgDBHdr *aHdrDeleted, nsMsgKey aParentKey, PRInt32 aFlags, 
                             nsIDBChangeListener *aInstigator)
 {
+  // check if we're adding a header, and the current day has changed. If it has, we're just going to 
+  // close and re-open the view so things will be correctly categorized.
+  if (m_dayChanged)
+    return HandleDayChange();
+
   nsCOMPtr <nsIMsgThread> thread;
   nsMsgKey keyDeleted;
   aHdrDeleted->GetMessageKey(&keyDeleted);
@@ -455,7 +566,8 @@ NS_IMETHODIMP nsMsgGroupView::OnHdrDeleted(nsIMsgDBHdr *aHdrDeleted, nsMsgKey aP
     {
       thread->RemoveChildAt(0); // get rid of dummy
       nsMsgDBView::RemoveByIndex(viewIndexOfThread - 1);
-      NoteChange(viewIndexOfThread - 1, -1, nsMsgViewNotificationCode::insertOrDelete); // an example where view is not the listener - D&D messages
+      if (m_deletingRows)
+        mIndicesToNoteChange.Add(viewIndexOfThread - 1);
     }
     else if (rootDeleted)
     {
@@ -467,7 +579,7 @@ NS_IMETHODIMP nsMsgGroupView::OnHdrDeleted(nsIMsgDBHdr *aHdrDeleted, nsMsgKey aP
   {
     nsHashKey *hashKey = AllocHashKeyForHdr(aHdrDeleted);
     if (hashKey)
-    m_groupsTable.Remove(hashKey);
+      m_groupsTable.Remove(hashKey);
     delete hashKey;
   }
   return rv;
@@ -515,19 +627,29 @@ NS_IMETHODIMP nsMsgGroupView::GetCellText(PRInt32 aRow, nsITreeColumn* aCol, nsA
           switch (((nsPRUint32Key *)hashKey)->GetValue())
           {
           case 1:
-            aValue.Assign(kTodayString);
+            if (!m_kTodayString.get())
+              m_kTodayString.Adopt(GetString(NS_LITERAL_STRING("today").get()));
+            aValue.Assign(m_kTodayString);
             break;
           case 2:
-            aValue.Assign(kYesterdayString);
+            if (!m_kYesterdayString.get())
+              m_kYesterdayString.Adopt(GetString(NS_LITERAL_STRING("yesterday").get()));
+            aValue.Assign(m_kYesterdayString);
             break;
           case 3:
-            aValue.Assign(kLastWeekString);
+            if (!m_kLastWeekString.get())
+              m_kLastWeekString.Adopt(GetString(NS_LITERAL_STRING("lastWeek").get()));
+            aValue.Assign(m_kLastWeekString);
             break;
           case 4:
-            aValue.Assign(kTwoWeeksAgoString);
+            if (!m_kTwoWeeksAgoString.get())
+              m_kTwoWeeksAgoString.Adopt(GetString(NS_LITERAL_STRING("twoWeeksAgo").get()));
+            aValue.Assign(m_kTwoWeeksAgoString);
             break;
           case 5:
-            aValue.Assign(kOldMailString);
+            if (!m_kOldMailString.get())
+              m_kOldMailString.Adopt(GetString(NS_LITERAL_STRING("older").get()));
+            aValue.Assign(m_kOldMailString);
             break;
           default:
             NS_ASSERTION(PR_FALSE, "bad age thread");
@@ -545,10 +667,10 @@ NS_IMETHODIMP nsMsgGroupView::GetCellText(PRInt32 aRow, nsITreeColumn* aCol, nsA
             valueText.Adopt(GetString(NS_LITERAL_STRING("messagesWithNoStatus").get()));
           aValue.Assign(valueText);
           break;
-        case nsMsgViewSortType::byLabel:
-          rv = FetchLabel(msgHdr, getter_Copies(valueText));
-          if (!valueText)
-            valueText.Adopt(GetString(NS_LITERAL_STRING("unlabeledMessages").get()));
+        case nsMsgViewSortType::byTags:
+          rv = FetchTags(msgHdr, getter_Copies(valueText));
+          if (valueText.IsEmpty())
+            valueText.Adopt(GetString(NS_LITERAL_STRING("untaggedMessages").get()));
           aValue.Assign(valueText);
           break;
         case nsMsgViewSortType::byPriority:
@@ -565,10 +687,48 @@ NS_IMETHODIMP nsMsgGroupView::GetCellText(PRInt32 aRow, nsITreeColumn* aCol, nsA
           FetchRecipients(msgHdr, getter_Copies(valueText));
           aValue.Assign(valueText);
           break;
+        case nsMsgViewSortType::byAttachments:
+          valueText.Adopt(GetString(((nsPRUint32Key *)hashKey)->GetValue()
+            ? NS_LITERAL_STRING("attachments").get()
+            : NS_LITERAL_STRING("noAttachments").get()));
+          aValue.Assign(valueText);
+          break;
+        case nsMsgViewSortType::byFlagged:
+          valueText.Adopt(GetString(((nsPRUint32Key *)hashKey)->GetValue()
+            ? NS_LITERAL_STRING("groupFlagged").get()
+            : NS_LITERAL_STRING("notFlagged").get()));
+          aValue.Assign(valueText);
+          break;
+
         default:
           NS_ASSERTION(PR_FALSE, "we don't sort by group for this type");
           break;
       }
+     
+      if (groupThread)
+      {
+        // Get number of messages in group
+        nsAutoString formattedCountMsg;
+        PRUint32 numMsg = groupThread->NumRealChildren();
+        formattedCountMsg.AppendInt(numMsg);
+
+        // Get number of unread messages
+        nsAutoString formattedCountUnrMsg;
+        PRUint32 numUnrMsg = 0;
+        groupThread->GetNumUnreadChildren(&numUnrMsg);
+        formattedCountUnrMsg.AppendInt(numUnrMsg);
+
+        // Add text to header
+        aValue.Append(NS_LITERAL_STRING(" ("));
+        if (numUnrMsg)
+        {
+          aValue.Append(formattedCountUnrMsg);
+          aValue.Append(NS_LITERAL_STRING("/"));
+        }
+
+        aValue.Append(formattedCountMsg);
+        aValue.Append(NS_LITERAL_STRING(")"));
+      }
     }
     else if (colID[0] == 't')
     {
@@ -589,9 +749,11 @@ NS_IMETHODIMP nsMsgGroupView::LoadMessageByViewIndex(nsMsgViewIndex aViewIndex)
   if (m_flags[aViewIndex] & MSG_VIEW_FLAG_DUMMY)
   {
     // if we used to have one item selected, and now we have more than one, we should clear the message pane.
-    nsCOMPtr <nsIMsgMessagePaneController> controller;
-    if (mMsgWindow && NS_SUCCEEDED(mMsgWindow->GetMessagePaneController(getter_AddRefs(controller))) && controller)
-      controller->ClearMsgPane();
+    nsCOMPtr <nsIMsgWindowCommands> windowCommands;
+    if (mMsgWindow && NS_SUCCEEDED(mMsgWindow->GetWindowCommands(getter_AddRefs(windowCommands))) && windowCommands)
+      windowCommands->ClearMsgPane();
+    // since we are selecting a dummy row, we should also clear out m_currentlyDisplayedMsgUri
+    m_currentlyDisplayedMsgUri.Truncate();
     return NS_OK;
   }
   else
@@ -603,18 +765,19 @@ nsresult nsMsgGroupView::GetThreadContainingMsgHdr(nsIMsgDBHdr *msgHdr, nsIMsgTh
   nsHashKey *hashKey = AllocHashKeyForHdr(msgHdr);
   if (hashKey)
   {
-  nsMsgGroupThread *groupThread = (nsMsgGroupThread *) m_groupsTable.Get(hashKey);
+    nsMsgGroupThread *groupThread = (nsMsgGroupThread *) m_groupsTable.Get(hashKey);
   
-  if (groupThread)
-    groupThread->QueryInterface(NS_GET_IID(nsIMsgThread), (void **) pThread);
-  delete hashKey;
+    if (groupThread)
+      groupThread->QueryInterface(NS_GET_IID(nsIMsgThread), (void **) pThread);
+    delete hashKey;
   }
   else
     *pThread = nsnull;
   return (*pThread) ? NS_OK : NS_ERROR_FAILURE;
 }
 
-PRInt32 nsMsgGroupView::FindLevelInThread(nsIMsgDBHdr *msgHdr, nsMsgViewIndex startOfThread, nsMsgViewIndex viewIndex)
+PRInt32 nsMsgGroupView::FindLevelInThread(nsIMsgDBHdr *msgHdr,
+                                          nsMsgViewIndex startOfThread, nsMsgViewIndex viewIndex)
 {
   return (startOfThread == viewIndex) ? 0 : 1;
 }
