@@ -48,10 +48,9 @@
 #include "nsEscape.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
-#include "nsIWebShell.h"
 #include "nsIDocShell.h"
 #include "nsIDOMDocument.h"
-#include "nsIDOMWindowInternal.h"
+#include "nsPIDOMWindow.h"
 #include "nsIDocumentViewer.h"
 #include "nsPresContext.h"
 #include "nsIPresShell.h"
@@ -66,17 +65,12 @@
 #include "nsMsgPrintEngine.h"
 #include "nsMsgBaseCID.h"
 #include "nsIDocumentLoader.h"
-#include "nsIWebShellWindow.h"
 #include "nsIWidget.h"
-#include "nsIWebShellWindow.h"
+#include "nsIXULWindow.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-
-// For PLEvents
-#include "plevent.h"
-#include "nsIEventQueue.h"
-#include "nsIEventQueueService.h"
-#include "nsIServiceManager.h"
+#include "nsThreadUtils.h"
+#include "nsAutoPtr.h"
 
 // Interfaces Needed
 #include "nsIBaseWindow.h"
@@ -270,11 +264,11 @@ nsMsgPrintEngine::SetWindow(nsIDOMWindowInternal *aWin)
 
   mWindow = aWin;
 
-  nsCOMPtr<nsIScriptGlobalObject> globalObj( do_QueryInterface(aWin) );
-  NS_ENSURE_TRUE(globalObj, NS_ERROR_FAILURE);
+  nsCOMPtr<nsPIDOMWindow> win( do_QueryInterface(aWin) );
+  NS_ENSURE_TRUE(win, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIDocShellTreeItem> docShellAsItem =
-    do_QueryInterface(globalObj->GetDocShell());
+    do_QueryInterface(win->GetDocShell());
   NS_ENSURE_TRUE(docShellAsItem, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIDocShellTreeItem> rootAsItem;
@@ -284,8 +278,9 @@ nsMsgPrintEngine::SetWindow(nsIDOMWindowInternal *aWin)
   NS_ENSURE_TRUE(rootAsNode, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIDocShellTreeItem> childItem;
-  rootAsNode->FindChildWithName(NS_LITERAL_STRING("content").get(), PR_TRUE, PR_FALSE, nsnull,
-    getter_AddRefs(childItem));
+  rootAsNode->FindChildWithName(NS_LITERAL_STRING("content").get(), PR_TRUE,
+				PR_FALSE, nsnull, nsnull,
+				getter_AddRefs(childItem));
 
   mDocShell = do_QueryInterface(childItem);
 
@@ -310,29 +305,19 @@ nsMsgPrintEngine::ShowWindow(PRBool aShow)
 
   NS_ENSURE_TRUE(mWindow, NS_ERROR_NOT_INITIALIZED);
 
-  nsCOMPtr <nsIScriptGlobalObject> globalScript = do_QueryInterface(mWindow, &rv);
+  nsCOMPtr <nsPIDOMWindow> win = do_QueryInterface(mWindow, &rv);
 
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCOMPtr <nsIWebShell> webShell =
-    do_QueryInterface(globalScript->GetDocShell(), &rv);
+  nsCOMPtr <nsIDocShellTreeItem> treeItem =
+    do_QueryInterface(win->GetDocShell(), &rv);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCOMPtr <nsIWebShellContainer> webShellContainer;
-  rv = webShell->GetContainer(*getter_AddRefs(webShellContainer));
+  nsCOMPtr <nsIDocShellTreeOwner> treeOwner;
+  rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
   NS_ENSURE_SUCCESS(rv,rv);
   
-  if (webShellContainer) {
-    nsCOMPtr <nsIWebShellWindow> webShellWindow = do_QueryInterface(webShellContainer, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsCOMPtr<nsIDocShellTreeItem>  treeItem(do_QueryInterface(webShell, &rv));
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
-    rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
-    NS_ENSURE_SUCCESS(rv,rv);
-
+  if (treeOwner) {
     // disable (enable) the window
     nsCOMPtr<nsIBaseWindow> baseWindow;
     baseWindow = do_QueryInterface(treeOwner, &rv);
@@ -342,7 +327,7 @@ nsMsgPrintEngine::ShowWindow(PRBool aShow)
     NS_ENSURE_SUCCESS(rv,rv);
 
     // hide or show the window
-    rv = webShellWindow->Show(aShow);
+    baseWindow->SetVisibility(aShow);
   }
   return rv;
 }
@@ -561,8 +546,7 @@ nsMsgPrintEngine::FireThatLoadOperation(nsString *uri)
 
   if (NS_SUCCEEDED(rv) && messageService)
   {
-    nsCOMPtr<nsIWebShell> webShell(do_QueryInterface(mDocShell));
-    rv = messageService->DisplayMessageForPrinting(tString, webShell, nsnull, nsnull, nsnull);
+    rv = messageService->DisplayMessageForPrinting(tString, mDocShell, nsnull, nsnull, nsnull);
   }
   //If it's not something we know about, then just load try loading it directly.
   else
@@ -696,6 +680,7 @@ nsMsgPrintEngine::PrintMsgWindow()
       } 
       else 
       {
+        mPrintSettings->SetPrintSilent(mCurrentlyPrintingURI != 0);
         nsCOMPtr<nsIContentViewerFile> contentViewerFile(do_QueryInterface(mWebBrowserPrint));
         if (contentViewerFile && mParentWindow) 
         {
@@ -728,7 +713,7 @@ nsMsgPrintEngine::PrintMsgWindow()
       else
       {
         // Tell the user we started printing...
-        PRUnichar *msg = GetString(NS_ConvertASCIItoUCS2(kMsgKeys[mMsgInx]).get());
+        PRUnichar *msg = GetString(NS_ConvertASCIItoUTF16(kMsgKeys[mMsgInx]).get());
         SetStatusMessage( msg );
         CRTFREEIF(msg)
       }
@@ -737,110 +722,61 @@ nsMsgPrintEngine::PrintMsgWindow()
 }
 
 //---------------------------------------------------------------
-//-- PLEvent Notification
+//-- Event Notification
 //---------------------------------------------------------------
+
+//---------------------------------------------------------------
+class nsPrintMsgWindowEvent : public nsRunnable
+{
+public:
+  nsPrintMsgWindowEvent(nsMsgPrintEngine *mpe)
+    : mMsgPrintEngine(mpe)
+  {}
+
+  NS_IMETHOD Run()
+  {
+    if (mMsgPrintEngine) 
+      mMsgPrintEngine->PrintMsgWindow();
+    return NS_OK;
+  }
+
+private:
+  nsRefPtr<nsMsgPrintEngine> mMsgPrintEngine;
+};
+
 //-----------------------------------------------------------
-PRBool
-FireEvent(nsMsgPrintEngine* aMPE, PLHandleEventProc handler, PLDestroyEventProc destructor)
+class nsStartNextPrintOpEvent : public nsRunnable
 {
-  static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
+public:
+  nsStartNextPrintOpEvent(nsMsgPrintEngine *mpe)
+    : mMsgPrintEngine(mpe)
+  {}
 
-  nsCOMPtr<nsIEventQueueService> event_service = do_GetService(kEventQueueServiceCID);
-
-  if (!event_service) 
+  NS_IMETHOD Run()
   {
-    NS_WARNING("Failed to get event queue service");
-    return PR_FALSE;
+    if (mMsgPrintEngine) 
+      mMsgPrintEngine->StartNextPrintOperation();
+    return NS_OK;
   }
 
-  nsCOMPtr<nsIEventQueue> event_queue;
-
-  event_service->GetThreadEventQueue(NS_CURRENT_THREAD,
-                                     getter_AddRefs(event_queue));
-
-  if (!event_queue) 
-  {
-    NS_WARNING("Failed to get event queue from service");
-    return PR_FALSE;
-  }
-
-  PLEvent *event = new PLEvent;
-
-  if (!event) 
-  {
-    NS_WARNING("Out of memory?");
-    return PR_FALSE;
-  }
-
-  PL_InitEvent(event, aMPE, handler, destructor);
-
-  // The event owns the msgPrintEngine pointer now.
-  NS_ADDREF(aMPE);
-
-  if (NS_FAILED(event_queue->PostEvent(event)))
-  {
-    NS_WARNING("Failed to post event");
-    PL_DestroyEvent(event);
-    return PR_FALSE;
-  }
-
-  return PR_TRUE;
-}
-
-void PR_CALLBACK HandlePLEventPrintMsgWindow(PLEvent* aEvent)
-{
-  nsMsgPrintEngine *msgPrintEngine = (nsMsgPrintEngine*)PL_GetEventOwner(aEvent);
-
-  NS_ASSERTION(msgPrintEngine, "The event owner is null.");
-  if (msgPrintEngine) 
-  {
-    msgPrintEngine->PrintMsgWindow();
-  }
-}
-
-//------------------------------------------------------------------------
-void PR_CALLBACK DestroyPLEventPrintMsgWindow(PLEvent* aEvent)
-{
-  nsMsgPrintEngine *msgPrintEngine = (nsMsgPrintEngine*)PL_GetEventOwner(aEvent);
-  NS_IF_RELEASE(msgPrintEngine);
-
-  delete aEvent;
-}
+private:
+  nsRefPtr<nsMsgPrintEngine> mMsgPrintEngine;
+};
 
 //-----------------------------------------------------------
 PRBool
 nsMsgPrintEngine::FirePrintEvent()
 {
-  return FireEvent(this, (PLHandleEventProc)::HandlePLEventPrintMsgWindow, 
-                         (PLDestroyEventProc)::DestroyPLEventPrintMsgWindow);
-}
-
-void PR_CALLBACK HandlePLEventStartNext(PLEvent* aEvent)
-{
-  nsMsgPrintEngine *msgPrintEngine = (nsMsgPrintEngine*)PL_GetEventOwner(aEvent);
-
-  NS_ASSERTION(msgPrintEngine, "The event owner is null.");
-  if (msgPrintEngine) 
-  {
-    msgPrintEngine->StartNextPrintOperation();
-  }
-}
-
-//------------------------------------------------------------------------
-void PR_CALLBACK DestroyPLEventStartNext(PLEvent* aEvent)
-{
-  nsMsgPrintEngine *msgPrintEngine = (nsMsgPrintEngine*)PL_GetEventOwner(aEvent);
-  NS_IF_RELEASE(msgPrintEngine);
-
-  delete aEvent;
+  nsCOMPtr<nsIRunnable> event = new nsPrintMsgWindowEvent(this);
+  return NS_SUCCEEDED(NS_DispatchToCurrentThread(event));
 }
 
 //-----------------------------------------------------------
 PRBool
 nsMsgPrintEngine::FireStartNextEvent()
 {
-  return FireEvent(this, (PLHandleEventProc)::HandlePLEventStartNext, 
-                         (PLDestroyEventProc)::DestroyPLEventStartNext);
+  nsCOMPtr<nsIRunnable> event = new nsStartNextPrintOpEvent(this);
+  return NS_DispatchToCurrentThread(event);
 }
 
 /* void setStartupPPObserver (in nsIObserver startupPPObs); */
