@@ -48,6 +48,7 @@
 #include "nsICopyMsgStreamListener.h"
 #include "nsMsgUtils.h"
 #include "nsITreeColumns.h"
+#include "nsIMsgMessageService.h"
 
 nsMsgSearchDBView::nsMsgSearchDBView()
 {
@@ -189,6 +190,27 @@ nsresult nsMsgSearchDBView::GetDBForViewIndex(nsMsgViewIndex index, nsIMsgDataba
     return NS_MSG_INVALID_DBVIEW_INDEX;
 }
 
+nsresult nsMsgSearchDBView::AddHdrFromFolder(nsIMsgDBHdr *msgHdr, nsISupports *folder)
+{
+  m_folders->AppendElement(folder);
+  nsMsgKey msgKey;
+  PRUint32 msgFlags;
+  msgHdr->GetMessageKey(&msgKey);
+  // nsMsgKey_None means it's not a valid hdr.
+  if (msgKey != nsMsgKey_None)
+  {
+    msgHdr->GetFlags(&msgFlags);
+    m_keys.Add(msgKey);
+    m_levels.Add(0);
+    m_flags.Add(msgFlags);
+    
+    // this needs to be called after we add the key, since RowCountChanged() will call our GetRowCount()
+    if (mTree)
+      mTree->RowCountChanged(GetSize() - 1, 1);
+  }
+  return NS_OK;
+  }
+
 NS_IMETHODIMP
 nsMsgSearchDBView::OnSearchHit(nsIMsgDBHdr* aMsgHdr, nsIMsgFolder *folder)
 {
@@ -208,20 +230,7 @@ nsMsgSearchDBView::OnSearchHit(nsIMsgDBHdr* aMsgHdr, nsIMsgFolder *folder)
     }
   }
 
-  m_folders->AppendElement(supports);
-  nsMsgKey msgKey;
-  PRUint32 msgFlags;
-  aMsgHdr->GetMessageKey(&msgKey);
-  aMsgHdr->GetFlags(&msgFlags);
-  m_keys.Add(msgKey);
-  m_levels.Add(0);
-  m_flags.Add(msgFlags);
-
-  // this needs to be called after we add the key, since RowCountChanged() will call our GetRowCount()
-  if (mTree)
-    mTree->RowCountChanged(GetSize() - 1, 1);
-
-  return NS_OK;
+  return AddHdrFromFolder(aMsgHdr, supports);
 }
 
 NS_IMETHODIMP
@@ -265,8 +274,13 @@ nsMsgSearchDBView::OnNewSearch()
 
 NS_IMETHODIMP nsMsgSearchDBView::OnAnnouncerGoingAway(nsIDBChangeAnnouncer *instigator)
 {
-  m_dbToUseList.RemoveObject(NS_STATIC_CAST(nsIMsgDatabase *, instigator));
-  return nsMsgDBView::OnAnnouncerGoingAway(instigator);
+  nsIMsgDatabase *db = NS_STATIC_CAST(nsIMsgDatabase *, instigator);
+  if (db)
+  {
+    db->RemoveListener(this);
+    m_dbToUseList.RemoveObject(db);
+  }
+  return NS_OK;
 }
 
 nsresult nsMsgSearchDBView::GetFolders(nsISupportsArray **aFolders)
@@ -276,6 +290,16 @@ nsresult nsMsgSearchDBView::GetFolders(nsISupportsArray **aFolders)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgSearchDBView::GetCommandStatus(nsMsgViewCommandTypeValue command, PRBool *selectable_p, nsMsgViewCommandCheckStateValue *selected_p)
+{
+  if (command != nsMsgViewCommandType::runJunkControls)
+    return nsMsgDBView::GetCommandStatus(command, selectable_p, selected_p);
+
+  *selectable_p = PR_FALSE;
+  return NS_OK;
+}
+
 NS_IMETHODIMP 
 nsMsgSearchDBView::DoCommandWithFolder(nsMsgViewCommandTypeValue command, nsIMsgFolder *destFolder)
 {
@@ -507,7 +531,7 @@ nsresult nsMsgSearchDBView::ProcessRequestsInOneFolder(nsIMsgWindow *window)
 
     // called for delete with trash, copy and move
     if (mCommand == nsMsgViewCommandType::deleteMsg)
-        curFolder->DeleteMessages(messageArray, window, PR_FALSE /* delete storage */, PR_FALSE /* is move*/, this, PR_FALSE /*allowUndo*/);
+        curFolder->DeleteMessages(messageArray, window, PR_FALSE /* delete storage */, PR_FALSE /* is move*/, this, PR_TRUE /*allowUndo*/);
     else 
     {
       NS_ASSERTION(!(curFolder == mDestFolder), "The source folder and the destination folder are the same");
@@ -517,9 +541,9 @@ nsresult nsMsgSearchDBView::ProcessRequestsInOneFolder(nsIMsgWindow *window)
          if (NS_SUCCEEDED(rv))
          {
            if (mCommand == nsMsgViewCommandType::moveMessages)
-             copyService->CopyMessages(curFolder, messageArray, mDestFolder, PR_TRUE /* isMove */, this, window, PR_FALSE /*allowUndo*/);
+             copyService->CopyMessages(curFolder, messageArray, mDestFolder, PR_TRUE /* isMove */, this, window, PR_TRUE /*allowUndo*/);
            else if (mCommand == nsMsgViewCommandType::copyMessages)
-             copyService->CopyMessages(curFolder, messageArray, mDestFolder, PR_FALSE /* isMove */, this, window, PR_FALSE /*allowUndo*/);
+             copyService->CopyMessages(curFolder, messageArray, mDestFolder, PR_FALSE /* isMove */, this, window, PR_TRUE /*allowUndo*/);
          }
       }
     }
