@@ -178,7 +178,8 @@ nsSubscribeDataSource::GetTarget(nsIRDFResource *source,
 
     if (property == kNC_Name.get()) {
         nsCOMPtr<nsIRDFLiteral> name;
-        rv = mRDFService->GetLiteral(NS_ConvertASCIItoUCS2((const char *)relativePath).get(), getter_AddRefs(name));
+        rv = mRDFService->GetLiteral(NS_ConvertUTF8toUTF16(relativePath).get(),
+                                     getter_AddRefs(name));
         NS_ENSURE_SUCCESS(rv,rv);
 
         if (!name) rv = NS_RDF_NO_VALUE;
@@ -187,9 +188,9 @@ nsSubscribeDataSource::GetTarget(nsIRDFResource *source,
     }
     else if (property == kNC_Child.get()) {
         nsXPIDLCString childUri;
-        rv = server->GetFirstChildURI(relativePath, getter_Copies(childUri));
+        rv = server->GetFirstChildURI(relativePath, childUri);
         if (NS_FAILED(rv)) return NS_RDF_NO_VALUE;
-        if (!(const char *)childUri) return NS_RDF_NO_VALUE;
+        if (childUri.IsEmpty()) return NS_RDF_NO_VALUE;
 
         nsCOMPtr <nsIRDFResource> childResource;
         rv = mRDFService->GetResource(childUri, getter_AddRefs(childResource));
@@ -219,7 +220,8 @@ nsSubscribeDataSource::GetTarget(nsIRDFResource *source,
         NS_ENSURE_SUCCESS(rv,rv);
 
         nsCOMPtr<nsIRDFLiteral> serverType;
-        rv = mRDFService->GetLiteral(NS_ConvertASCIItoUCS2((const char *)serverTypeStr).get(), getter_AddRefs(serverType));
+        rv = mRDFService->GetLiteral(NS_ConvertASCIItoUTF16(serverTypeStr).get(),
+                                     getter_AddRefs(serverType));
         NS_ENSURE_SUCCESS(rv,rv);
 
         if (!serverType) rv = NS_RDF_NO_VALUE;
@@ -228,7 +230,7 @@ nsSubscribeDataSource::GetTarget(nsIRDFResource *source,
     }
     else if (property == kNC_LeafName.get()) {
         nsXPIDLString leafNameStr;
-        rv = server->GetLeafName(relativePath, getter_Copies(leafNameStr));
+        rv = server->GetLeafName(relativePath, leafNameStr); 
         NS_ENSURE_SUCCESS(rv,rv);
    
         nsCOMPtr<nsIRDFLiteral> leafName;
@@ -247,28 +249,24 @@ nsSubscribeDataSource::GetTarget(nsIRDFResource *source,
 }
 
 nsresult
-nsSubscribeDataSource::GetChildren(nsISubscribableServer *server, const char *relativePath, nsISimpleEnumerator** aResult)
+nsSubscribeDataSource::GetChildren(nsISubscribableServer *aServer, 
+                                   const nsACString &aRelativePath,
+                                   nsISimpleEnumerator** aResult)
 {
     nsresult rv = NS_OK;
-    NS_ASSERTION(server && aResult, "no server or result");
-    if (!server || !aResult) return NS_ERROR_NULL_POINTER;       
+    NS_ASSERTION(aServer && aResult, "no server or result");
+    if (!aServer || !aResult) return NS_ERROR_NULL_POINTER;       
 
     nsCOMPtr<nsISupportsArray> children;
     rv = NS_NewISupportsArray(getter_AddRefs(children));
     NS_ENSURE_SUCCESS(rv,rv);
     if (!children) return NS_ERROR_FAILURE;
 
-    rv = server->GetChildren(relativePath, children);
+    rv = aServer->GetChildren(aRelativePath, children);
     // GetChildren() can fail if there are no children
     if (NS_FAILED(rv)) return rv;
 
-    nsISimpleEnumerator* result = new nsArrayEnumerator(children);
-    if (!result) return NS_ERROR_OUT_OF_MEMORY;
-
-    NS_ADDREF(result);
-    *aResult = result;
-
-    return NS_OK;
+    return NS_NewArrayEnumerator(aResult, children);
 }
 
 NS_IMETHODIMP
@@ -297,7 +295,7 @@ nsSubscribeDataSource::GetTargets(nsIRDFResource *source,
 	if (!tv) return NS_RDF_NO_VALUE;
 
     nsCOMPtr<nsISubscribableServer> server;
-    nsXPIDLCString relativePath;
+    nsXPIDLCString relativePath;  // UTF-8
 
     rv = GetServerAndRelativePathFromResource(source, getter_AddRefs(server), getter_Copies(relativePath));
     if (NS_FAILED(rv) || !server) {
@@ -313,51 +311,38 @@ nsSubscribeDataSource::GetTargets(nsIRDFResource *source,
     }
     else if (property == kNC_LeafName.get()) {
         nsXPIDLString leafNameStr;
-        rv = server->GetLeafName(relativePath, getter_Copies(leafNameStr));
+        rv = server->GetLeafName(relativePath, leafNameStr);
         NS_ENSURE_SUCCESS(rv,rv);
     
         nsCOMPtr<nsIRDFLiteral> leafName;
         rv = mRDFService->GetLiteral(leafNameStr, getter_AddRefs(leafName));
         NS_ENSURE_SUCCESS(rv,rv);
 
-        nsISimpleEnumerator* result = new nsSingletonEnumerator(leafName);
-        if (!result) return NS_ERROR_OUT_OF_MEMORY;
-
-        NS_IF_ADDREF(*targets = result);
-        return NS_OK;
+        return NS_NewSingletonEnumerator(targets, leafName);
     }
     else if (property == kNC_Subscribed.get()) {
         PRBool isSubscribed;
         rv = server->IsSubscribed(relativePath, &isSubscribed);
         NS_ENSURE_SUCCESS(rv,rv);
 
-        nsISimpleEnumerator* result = new nsSingletonEnumerator(isSubscribed ? kTrueLiteral : kFalseLiteral);
-        if (!result) return NS_ERROR_OUT_OF_MEMORY;
-
-        NS_IF_ADDREF(*targets = result);
-        return NS_OK;
+        return NS_NewSingletonEnumerator(targets,
+                 isSubscribed ? kTrueLiteral : kFalseLiteral);
     }
     else if (property == kNC_Subscribable.get()) {
         PRBool isSubscribable;
         rv = server->IsSubscribable(relativePath, &isSubscribable);
         NS_ENSURE_SUCCESS(rv,rv);
 
-        nsISimpleEnumerator* result = new nsSingletonEnumerator(isSubscribable ? kTrueLiteral : kFalseLiteral);
-        if (!result) return NS_ERROR_OUT_OF_MEMORY;
-
-        NS_IF_ADDREF(*targets = result);
-        return NS_OK;
+        return NS_NewSingletonEnumerator(targets,
+                 isSubscribable ? kTrueLiteral : kFalseLiteral);
     }
     else if (property == kNC_Name.get()) {
         nsCOMPtr<nsIRDFLiteral> name;
-        rv = mRDFService->GetLiteral(NS_ConvertASCIItoUCS2((const char *)relativePath).get(), getter_AddRefs(name));
+        rv = mRDFService->GetLiteral(NS_ConvertUTF8toUTF16(relativePath).get(),
+                                     getter_AddRefs(name));
         NS_ENSURE_SUCCESS(rv,rv);
 
-        nsISimpleEnumerator* result = new nsSingletonEnumerator(name);
-        if (!result) return NS_ERROR_OUT_OF_MEMORY;
-
-        NS_IF_ADDREF(*targets = result);
-        return NS_OK;
+        return NS_NewSingletonEnumerator(targets, name);
     }
     else if (property == kNC_ServerType.get()) {
         nsXPIDLCString serverTypeStr;
@@ -365,14 +350,11 @@ nsSubscribeDataSource::GetTargets(nsIRDFResource *source,
         NS_ENSURE_SUCCESS(rv,rv);
 
         nsCOMPtr<nsIRDFLiteral> serverType;
-        rv = mRDFService->GetLiteral(NS_ConvertASCIItoUCS2((const char *)serverTypeStr).get(), getter_AddRefs(serverType));
+        rv = mRDFService->GetLiteral(NS_ConvertASCIItoUTF16(serverTypeStr).get(),
+                                     getter_AddRefs(serverType));
         NS_ENSURE_SUCCESS(rv,rv);
 
-        nsISimpleEnumerator* result = new nsSingletonEnumerator(serverType);
-        if (!result) return NS_ERROR_OUT_OF_MEMORY;
-
-        NS_IF_ADDREF(*targets = result);
-        return NS_OK;
+        return NS_NewSingletonEnumerator(targets, serverType);
     }
     else {
         // do nothing
@@ -470,6 +452,7 @@ nsSubscribeDataSource::GetServerAndRelativePathFromResource(nsIRDFResource *sour
         *relativePath = nsnull;
     }
     else {
+        // XXX : perhaps, have to unescape before returning 
         *relativePath = nsCRT::strdup(sourceURI + serverURILen + 1);
         NS_ASSERTION(*relativePath,"no relative path");
         if (!*relativePath) return NS_ERROR_OUT_OF_MEMORY;
@@ -519,7 +502,7 @@ nsSubscribeDataSource::HasAssertion(nsIRDFResource *source,
     }
 
         // not everything has children
-        rv = server->HasChildren((const char *)relativePath, hasAssertion);
+        rv = server->HasChildren(relativePath, hasAssertion);
         NS_ENSURE_SUCCESS(rv,rv);
     }
     else if (property == kNC_Name.get()) {
@@ -572,7 +555,7 @@ nsSubscribeDataSource::HasArcOut(nsIRDFResource *source, nsIRDFResource *aArc, P
     }
 
         PRBool hasChildren = PR_FALSE;
-        rv = server->HasChildren((const char *)relativePath, &hasChildren);
+        rv = server->HasChildren(relativePath, &hasChildren);
         NS_ENSURE_SUCCESS(rv,rv);
         *result = hasChildren;
         return NS_OK;
@@ -633,19 +616,14 @@ nsSubscribeDataSource::ArcLabelsOut(nsIRDFResource *source,
     array->AppendElement(kNC_LeafName);
 
     PRBool hasChildren = PR_FALSE;
-    rv = server->HasChildren((const char *)relativePath, &hasChildren);
+    rv = server->HasChildren(relativePath, &hasChildren);
     NS_ENSURE_SUCCESS(rv,rv);
 
     if (hasChildren) {
         array->AppendElement(kNC_Child);
     }
 
-    nsISimpleEnumerator* result = new nsArrayEnumerator(array);
-    if (! result) return NS_ERROR_OUT_OF_MEMORY;
-
-    NS_ADDREF(result);
-    *labels = result;
-    return NS_OK;
+    return NS_NewArrayEnumerator(labels, array);
 }
 
 NS_IMETHODIMP
