@@ -62,14 +62,12 @@
 #include "nsIMsgMailSession.h"
 #include "nsIPrefService.h"
 #include "nsIDocShell.h"
-#include "nsIWebShell.h"
-#include "nsIWebShellWindow.h"
 #include "nsIAuthPrompt.h"
 #include "nsIObserverService.h"
 #include "nsNetUtil.h"
 #include "nsIWindowWatcher.h"
 #include "nsIStringBundle.h"
-
+#include "nsIMsgHdr.h"
 #include "nsIRDFService.h"
 #include "nsRDFCID.h"
 #include "nsIInterfaceRequestor.h"
@@ -86,13 +84,12 @@
 
 #define REL_FILE_PREF_SUFFIX NS_LITERAL_CSTRING("-rel")
 
-MOZ_DECL_CTOR_COUNTER(nsMsgIncomingServer)
-
 nsMsgIncomingServer::nsMsgIncomingServer():
     m_rootFolder(0),
     m_prefBranch(0),
     m_biffState(nsIMsgFolder::nsMsgBiffState_NoMail),
     m_serverBusy(PR_FALSE),
+    m_numMsgsDownloaded(0),
     m_canHaveFilters(PR_TRUE),
     m_displayStartupPage(PR_TRUE),
     mPerformingBiff(PR_FALSE)
@@ -104,7 +101,6 @@ nsMsgIncomingServer::~nsMsgIncomingServer()
     NS_IF_RELEASE(m_prefBranch);
 }
 
-
 NS_IMPL_THREADSAFE_ADDREF(nsMsgIncomingServer)
 NS_IMPL_THREADSAFE_RELEASE(nsMsgIncomingServer)
 NS_INTERFACE_MAP_BEGIN(nsMsgIncomingServer)
@@ -229,6 +225,7 @@ NS_IMETHODIMP
 nsMsgIncomingServer::Shutdown()
 {
   nsresult rv = CloseCachedConnections();
+  mFilterPlugin = nsnull;
   NS_ENSURE_SUCCESS(rv,rv);
 
   if (mFilterList) 
@@ -772,7 +769,7 @@ nsMsgIncomingServer::GetConstructedPrettyName(PRUnichar **retval)
 NS_IMETHODIMP
 nsMsgIncomingServer::ToString(PRUnichar** aResult) {
   *aResult = ToNewUnicode(NS_LITERAL_STRING("[nsIMsgIncomingServer: ") +
-                          NS_ConvertASCIItoUCS2(m_serverKey) +
+                          NS_ConvertASCIItoUTF16(m_serverKey) +
                           NS_LITERAL_STRING("]"));
   NS_ASSERTION(*aResult, "no server name!");
   return NS_OK;
@@ -786,13 +783,6 @@ NS_IMETHODIMP nsMsgIncomingServer::SetPassword(const char * aPassword)
   nsresult rv;
   PRBool rememberPassword = PR_FALSE;
   
-  if (aPassword)
-  {
-    nsCOMPtr<nsIMsgAccountManager> accountManager = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID);
-    if (accountManager)
-      accountManager->SetUserNeedsToAuthenticate(PR_FALSE);
-  }
-
   rv = GetRememberPassword(&rememberPassword);
   if (NS_FAILED(rv)) return rv;
   
@@ -873,9 +863,7 @@ nsMsgIncomingServer::GetPasswordWithUI(const PRUnichar * aPromptMessage, const
       rv = aMsgWindow->GetRootDocShell(getter_AddRefs(docShell));
       if (NS_FAILED(rv)) return rv;
       
-      nsCOMPtr<nsIWebShell> webShell(do_QueryInterface(docShell, &rv));
-      if (NS_FAILED(rv)) return rv;
-      dialog = do_GetInterface(webShell, &rv);
+      dialog = do_GetInterface(docShell, &rv);
       if (NS_FAILED(rv)) return rv;
     }
     else
@@ -887,18 +875,22 @@ nsMsgIncomingServer::GetPasswordWithUI(const PRUnichar * aPromptMessage, const
     }
     if (dialog)
     {
-      nsXPIDLString uniPassword;
       nsXPIDLCString serverUri;
       rv = GetServerURI(getter_Copies(serverUri));
       if (NS_FAILED(rv)) return rv;
       PRBool passwordProtectLocalCache = PR_FALSE;
 
       (void) m_prefBranch->GetBoolPref( "mail.password_protect_local_cache", &passwordProtectLocalCache);
+      PRUnichar *uniPassword = nsnull;
+      if (*aPassword)
+        uniPassword = ToNewUnicode(NS_ConvertASCIItoUTF16(*aPassword));
 
       PRUint32 savePasswordType = (passwordProtectLocalCache) ? nsIAuthPrompt::SAVE_PASSWORD_FOR_SESSION : nsIAuthPrompt::SAVE_PASSWORD_PERMANENTLY;
       rv = dialog->PromptPassword(aPromptTitle, aPromptMessage, 
-        NS_ConvertASCIItoUCS2(serverUri).get(), savePasswordType,
-        getter_Copies(uniPassword), okayValue);
+        NS_ConvertASCIItoUTF16(serverUri).get(), savePasswordType,
+        &uniPassword, okayValue);
+      nsAutoString uniPasswordAdopted;
+      uniPasswordAdopted.Adopt(uniPassword);
       if (NS_FAILED(rv)) return rv;
       
       if (!*okayValue) // if the user pressed cancel, just return NULL;
@@ -908,8 +900,8 @@ nsMsgIncomingServer::GetPasswordWithUI(const PRUnichar * aPromptMessage, const
       }
       
       // we got a password back...so remember it
-      nsCString aCStr; aCStr.AssignWithConversion(uniPassword); 
-      
+      nsCString aCStr; 
+      aCStr.AssignWithConversion(uniPasswordAdopted); 
       rv = SetPassword(aCStr.get());
       if (NS_FAILED(rv)) return rv;
     } // if we got a prompt dialog
@@ -951,35 +943,35 @@ nsMsgIncomingServer::StorePassword()
     rv = CreateServicesForPasswordManager();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = observerService->NotifyObservers(uri, "login-succeeded", NS_ConvertUTF8toUCS2(pwd).get());
+    rv = observerService->NotifyObservers(uri, "login-succeeded", NS_ConvertUTF8toUTF16(pwd).get());
     NS_ENSURE_SUCCESS(rv,rv);
+    nsCOMPtr<nsIMsgAccountManager> accountManager = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID);
+    if (accountManager)
+      accountManager->SetUserNeedsToAuthenticate(PR_FALSE);
     return rv;
 }
 
 NS_IMETHODIMP
 nsMsgIncomingServer::ForgetPassword()
 {
-    nsresult rv;
-
-    nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
     nsXPIDLCString serverSpec;
-    rv = GetServerURI(getter_Copies(serverSpec));
+    nsresult rv = GetServerURI(getter_Copies(serverSpec));
     if (NS_FAILED(rv)) return rv;
 
-    nsCOMPtr<nsIURI> uri;
-    NS_NewURI(getter_AddRefs(uri), serverSpec);
-
     //this is need to make sure wallet service has been created
     rv = CreateServicesForPasswordManager();
     NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr <nsIPasswordManager> passwordMgr = do_GetService(NS_PASSWORDMANAGER_CONTRACTID, &rv);
+    if (NS_SUCCEEDED(rv) && passwordMgr)
+    {
+      // Get the current server URI
+      nsXPIDLCString currServerUri;
+      rv = GetServerURI(getter_Copies(currServerUri));
+      NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = observerService->NotifyObservers(uri, "login-failed", nsnull);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = SetPassword("");
-    return rv;
+      passwordMgr->RemoveUser(currServerUri, EmptyString());
+    }
+    return SetPassword("");
 }
 
 NS_IMETHODIMP
@@ -1027,10 +1019,10 @@ nsMsgIncomingServer::GetLocalPath(nsIFileSpec **aLocalPath)
 	// set the leaf name to "dummy", and then call MakeUnique with a suggested leaf name
     rv = path->AppendRelativeUnixPath("dummy");
     if (NS_FAILED(rv)) return rv;
-	nsXPIDLCString hostname;
+    nsXPIDLCString hostname;
     rv = GetHostName(getter_Copies(hostname));
     if (NS_FAILED(rv)) return rv;
-	rv = path->MakeUniqueWithSuggestedName((const char *)hostname);
+    rv = path->MakeUniqueDirWithSuggestedName((const char *)hostname);
     if (NS_FAILED(rv)) return rv;
 
     rv = SetLocalPath(path);
@@ -1134,6 +1126,7 @@ nsMsgIncomingServer::ClearAllValues()
 {
     nsCAutoString rootPref("mail.server.");
     rootPref += m_serverKey;
+    rootPref += '.';
 
     PRUint32 childCount;
     char**   childArray;
@@ -1682,7 +1675,7 @@ NS_IMETHODIMP nsMsgIncomingServer::DisplayOfflineMsg(nsIMsgWindow *aMsgWindow)
     bundle->GetStringFromName(NS_LITERAL_STRING("nocachedbodybody").get(), getter_Copies(errorMsgBody));
     bundle->GetStringFromName(NS_LITERAL_STRING("nocachedbodytitle").get(),  getter_Copies(errorMsgTitle));
     if (aMsgWindow)
-      return aMsgWindow->DisplayHTMLInMessagePane(errorMsgTitle, errorMsgBody);
+      return aMsgWindow->DisplayHTMLInMessagePane(errorMsgTitle, errorMsgBody, PR_TRUE);
     else
       return NS_ERROR_FAILURE;
 
@@ -1722,10 +1715,28 @@ nsMsgIncomingServer::GetSearchScope(nsMsgSearchScopeValue *searchScope)
    return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgIncomingServer::GetIsSecure(PRBool* aIsSecure)
+{
+  return GetBoolValue("isSecure", aIsSecure);
+}
+
+NS_IMETHODIMP
+nsMsgIncomingServer::SetIsSecure(PRBool aIsSecure)
+{
+  PRBool isSecure;
+  GetBoolValue("isSecure", &isSecure);
+  if (isSecure != aIsSecure) {
+    SetBoolValue("isSecure", aIsSecure);
+    if (m_rootFolder)
+      m_rootFolder->NotifyBoolPropertyChanged(NS_NewAtom("isSecure"), isSecure, aIsSecure);
+  }
+  return NS_OK;
+}
+
 // use the convenience macros to implement the accessors
 NS_IMPL_SERVERPREF_STR(nsMsgIncomingServer, Username, "userName")
 NS_IMPL_SERVERPREF_STR(nsMsgIncomingServer, PrefPassword, "password")
-NS_IMPL_SERVERPREF_BOOL(nsMsgIncomingServer, IsSecure, "isSecure")
 NS_IMPL_SERVERPREF_BOOL(nsMsgIncomingServer, UseSecAuth, "useSecAuth")
 NS_IMPL_SERVERPREF_BOOL(nsMsgIncomingServer, LogonFallback, "logon_fallback")
 NS_IMPL_SERVERPREF_INT(nsMsgIncomingServer, BiffMinutes, "check_time")
@@ -1758,6 +1769,8 @@ NS_IMPL_SERVERPREF_BOOL(nsMsgIncomingServer,
 
 NS_IMPL_SERVERPREF_INT(nsMsgIncomingServer, MaxMessageSize, "max_size")
 
+NS_IMPL_SERVERPREF_INT(nsMsgIncomingServer, IncomingDuplicateAction, "dup_action")
+
 NS_IMETHODIMP nsMsgIncomingServer::SetUnicharAttribute(const char *aName, const PRUnichar *val)
 {
   return SetUnicharValue(aName, val);
@@ -1798,6 +1811,42 @@ NS_IMETHODIMP nsMsgIncomingServer::GetIntAttribute(const char *aName, PRInt32 *v
   return GetIntValue(aName, val);
 }
 
+
+NS_IMETHODIMP nsMsgIncomingServer::GetSocketType(PRInt32 *aSocketType)
+{
+  nsCAutoString fullPrefName;
+  getPrefName(m_serverKey.get(), "socketType", fullPrefName);
+  nsresult rv = m_prefBranch->GetIntPref(fullPrefName.get(), aSocketType);
+
+  // socketType is set to default value. Look at isSecure setting
+  if (NS_FAILED(rv))
+  {
+    PRBool isSecure;
+    rv = GetBoolValue("isSecure", &isSecure);
+    if (NS_SUCCEEDED(rv) && isSecure)
+    {
+       *aSocketType = nsIMsgIncomingServer::useSSL;
+      // don't call virtual method in case overrides call GetSocketType
+      nsMsgIncomingServer::SetSocketType(*aSocketType);
+    }
+    else
+    {
+       getDefaultIntPref("socketType", aSocketType);
+    }
+  }
+  return rv;
+  
+}
+
+NS_IMETHODIMP nsMsgIncomingServer::SetSocketType(PRInt32 aSocketType)
+{
+  nsCAutoString fullPrefName;
+  getPrefName(m_serverKey.get(), "socketType", fullPrefName);
+  nsresult rv = m_prefBranch->SetIntPref(fullPrefName.get(), aSocketType);
+
+  return rv;
+}
+
 // Check if the password is available and return a boolean indicating whether 
 // it is being authenticated or not.
 NS_IMETHODIMP 
@@ -1858,7 +1907,8 @@ nsMsgIncomingServer::GetPasswordPromptRequired(PRBool *aPasswordIsRequired)
 NS_IMETHODIMP nsMsgIncomingServer::ConfigureTemporaryFilters(nsIMsgFilterList *aFilterList)
 {
   nsresult rv = ConfigureTemporaryReturnReceiptsFilter(aFilterList);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_FAILED(rv)) // shut up warnings...
+    return rv;
   return ConfigureTemporaryServerSpamFilters(aFilterList);
 }
 
@@ -1868,6 +1918,15 @@ nsMsgIncomingServer::ConfigureTemporaryServerSpamFilters(nsIMsgFilterList *filte
   nsCOMPtr<nsISpamSettings> spamSettings;
   nsresult rv = GetSpamSettings(getter_AddRefs(spamSettings));
   NS_ENSURE_SUCCESS(rv, rv);
+
+  PRBool useServerFilter;
+  rv = spamSettings->GetUseServerFilter(&useServerFilter);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  // if we aren't configured to use server filters, then return early.
+  if (!useServerFilter)
+    return NS_OK;
+  
   // For performance reasons, we'll handle clearing of filters if the user turns
   // off the server-side filters from the junk mail controls, in the junk mail controls.
   nsCAutoString serverFilterName;
@@ -1943,6 +2002,37 @@ nsMsgIncomingServer::ConfigureTemporaryServerSpamFilters(nsIMsgFilterList *filte
   if (newFilter && serverFilterTrustFlags & nsISpamSettings::TRUST_POSITIVES)
   {
     newFilter->SetTemporary(PR_TRUE);
+    // check if we're supposed to move junk mail to junk folder; if so,
+    // add filter action to do so.
+    PRBool moveOnSpam, markAsReadOnSpam;
+    spamSettings->GetMoveOnSpam(&moveOnSpam);
+    if (moveOnSpam)
+    {
+      nsXPIDLCString spamFolderURI;
+      rv = spamSettings->GetSpamFolderURI(getter_Copies(spamFolderURI));
+      if (NS_SUCCEEDED(rv) && (!spamFolderURI.IsEmpty()))
+      {
+        nsCOMPtr <nsIMsgRuleAction> moveAction;
+        rv = newFilter->CreateAction(getter_AddRefs(moveAction));
+        if (NS_SUCCEEDED(rv))
+        {
+          moveAction->SetType(nsMsgFilterAction::MoveToFolder);
+          moveAction->SetTargetFolderUri(spamFolderURI);
+          newFilter->AppendAction(moveAction);
+        }
+      }
+    }
+    spamSettings->GetMarkAsReadOnSpam(&markAsReadOnSpam);
+    if (markAsReadOnSpam)
+    {
+      nsCOMPtr <nsIMsgRuleAction> markAsReadAction;
+      rv = newFilter->CreateAction(getter_AddRefs(markAsReadAction));
+      if (NS_SUCCEEDED(rv))
+      {
+        markAsReadAction->SetType(nsMsgFilterAction::MarkRead);
+        newFilter->AppendAction(markAsReadAction);
+      }
+    }
     filterList->InsertFilterAt(0, newFilter);
   }
 
@@ -2095,142 +2185,6 @@ nsMsgIncomingServer::GetMsgFolderFromURI(nsIMsgFolder *aFolderResource, const ch
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMsgIncomingServer::SetSpamSettings(nsISpamSettings *aSpamSettings)
-{
-  NS_ENSURE_ARG_POINTER(aSpamSettings);
-  nsresult rv;
-  
-  if (!mSpamSettings) {
-    mSpamSettings = do_CreateInstance(NS_SPAMSETTINGS_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = mSpamSettings->SetServer(this);
-    NS_ENSURE_SUCCESS(rv,rv);
-  }
-
-  nsXPIDLCString oldJunkFolderURI;
-  rv = mSpamSettings->GetSpamFolderURI(getter_Copies(oldJunkFolderURI));
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  if (!oldJunkFolderURI.IsEmpty()) {
-    nsCOMPtr<nsIMsgFolder> oldJunkFolder;
-    rv = GetExistingFolder(oldJunkFolderURI.get(), getter_AddRefs(oldJunkFolder));
-    if (NS_SUCCEEDED(rv) && oldJunkFolder) 
-    {
-      // remove the MSG_FOLDER_FLAG_JUNK on the old junk folder
-      // XXX TODO
-      // JUNK MAIL RELATED
-      // (in ClearFlag?) we need to make sure that this folder
-      // is not a the junk folder for another account
-      // the same goes for set flag.  have fun with all that.
-      oldJunkFolder->ClearFlag(MSG_FOLDER_FLAG_JUNK);
-    }
-  }
-
-  rv = mSpamSettings->Clone(aSpamSettings);
-  NS_ENSURE_SUCCESS(rv,rv);
-  
-  // set the server prefs from the spam settings
-  PRInt32 spamLevel;
-  rv = mSpamSettings->GetLevel(&spamLevel);
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetIntValue("spamLevel", spamLevel);
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  PRBool moveOnSpam;
-  (void)mSpamSettings->GetMoveOnSpam(&moveOnSpam);
-  (void)SetBoolValue("moveOnSpam", moveOnSpam);
-
-  PRBool markAsReadOnSpam;
-  (void)mSpamSettings->GetMarkAsReadOnSpam(&markAsReadOnSpam);
-  (void)SetBoolValue("markAsReadOnSpam", markAsReadOnSpam);
-
-  PRInt32 moveTargetMode;
-  (void)mSpamSettings->GetMoveTargetMode(&moveTargetMode);
-  (void)SetIntValue("moveTargetMode", moveTargetMode);
-
-  PRBool manualMark;
-  (void)mSpamSettings->GetManualMark(&manualMark);
-  (void)SetBoolValue("manualMark", manualMark);
-
-  PRInt32 manualMarkMode;
-  (void)mSpamSettings->GetManualMarkMode(&manualMarkMode);
-  (void)SetIntValue("manualMarkMode", manualMarkMode);
-
-  nsXPIDLCString spamActionTargetAccount;
-  rv = mSpamSettings->GetActionTargetAccount(getter_Copies(spamActionTargetAccount));
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetCharValue("spamActionTargetAccount", spamActionTargetAccount.get());
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  nsXPIDLCString spamActionTargetFolder;
-  rv = mSpamSettings->GetActionTargetFolder(getter_Copies(spamActionTargetFolder));
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetCharValue("spamActionTargetFolder", spamActionTargetFolder.get());
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  nsXPIDLCString newJunkFolderURI;
-  rv = mSpamSettings->GetSpamFolderURI(getter_Copies(newJunkFolderURI));
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  // only try to create the junk folder if we are moving junk
-  // and we have a non-empty uri
-  if (moveOnSpam && !newJunkFolderURI.IsEmpty()) {
-    // as the url listener, the spam settings will set the MSG_FOLDER_FLAG_JUNK folder flag
-    // on the junk mail folder, after it is created
-    nsCOMPtr <nsIUrlListener> listener = do_QueryInterface(mSpamSettings, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = GetOrCreateFolder(newJunkFolderURI, listener);
-    NS_ENSURE_SUCCESS(rv,rv);
-  }
-
-  PRBool useWhiteList;
-  rv = mSpamSettings->GetUseWhiteList(&useWhiteList);
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetBoolValue("useWhiteList", useWhiteList);
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  nsXPIDLCString whiteListAbURI;
-  rv = mSpamSettings->GetWhiteListAbURI(getter_Copies(whiteListAbURI));
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetCharValue("whiteListAbURI", whiteListAbURI.get());
-  NS_ENSURE_SUCCESS(rv,rv);
-  
-  PRBool purgeSpam;
-  rv = mSpamSettings->GetPurge(&purgeSpam);
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetBoolValue("purgeSpam", purgeSpam);
-  NS_ENSURE_SUCCESS(rv,rv);
-  
-  PRInt32 purgeSpamInterval;
-  rv = mSpamSettings->GetPurgeInterval(&purgeSpamInterval);
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetIntValue("purgeSpamInterval", purgeSpamInterval);
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  nsCAutoString serverFilterName;
-  mSpamSettings->GetServerFilterName(serverFilterName);
-  SetCharValue("serverFilterName", serverFilterName.get());
-
-  PRInt32 serverFilterTrustFlags;
-  mSpamSettings->GetServerFilterTrustFlags(&serverFilterTrustFlags);
-  SetIntValue("serverFilterTrustFlags", serverFilterTrustFlags);
-
-  PRInt32 loggingEnabled;
-  rv = mSpamSettings->GetLoggingEnabled(&loggingEnabled);
-  NS_ENSURE_SUCCESS(rv,rv);
-  rv = SetBoolValue("spamLoggingEnabled", loggingEnabled);
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  // flush these pref changes to disk
-  nsCOMPtr<nsIPrefService> prefService = do_QueryInterface(m_prefBranch);
-  NS_ENSURE_TRUE(prefService, NS_ERROR_FAILURE);
-
-  return prefService->SavePrefFile(nsnull);
-}
-
 NS_IMETHODIMP
 nsMsgIncomingServer::GetSpamSettings(nsISpamSettings **aSpamSettings)
 {
@@ -2241,91 +2195,7 @@ nsMsgIncomingServer::GetSpamSettings(nsISpamSettings **aSpamSettings)
 
     mSpamSettings = do_CreateInstance(NS_SPAMSETTINGS_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = mSpamSettings->SetServer(this);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    // set spam settings from server prefs
-    PRInt32 spamLevel;
-    rv = GetIntValue("spamLevel", &spamLevel);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetLevel(spamLevel);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    PRBool moveOnSpam;
-    rv = GetBoolValue("moveOnSpam", &moveOnSpam);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetMoveOnSpam(moveOnSpam);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    PRBool markAsReadOnSpam;
-    rv = GetBoolValue("markAsReadOnSpam", &markAsReadOnSpam);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetMarkAsReadOnSpam(markAsReadOnSpam);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    PRInt32 moveTargetMode;
-    rv = GetIntValue("moveTargetMode", &moveTargetMode);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetMoveTargetMode(moveTargetMode);
-    NS_ENSURE_SUCCESS(rv,rv);
-    
-    PRBool manualMark;
-    (void)GetBoolValue("manualMark", &manualMark);
-    (void)mSpamSettings->SetManualMark(manualMark);
-
-    PRInt32 manualMarkMode;
-    (void)GetIntValue("manualMarkMode", &manualMarkMode);
-    (void)mSpamSettings->SetManualMarkMode(manualMarkMode);
-
-    nsXPIDLCString spamActionTargetAccount;
-    rv = GetCharValue("spamActionTargetAccount", getter_Copies(spamActionTargetAccount));
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetActionTargetAccount(spamActionTargetAccount);    
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsXPIDLCString spamActionTargetFolder;
-    rv = GetCharValue("spamActionTargetFolder", getter_Copies(spamActionTargetFolder));
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetActionTargetFolder(spamActionTargetFolder);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    PRBool useWhiteList;
-    rv = GetBoolValue("useWhiteList", &useWhiteList);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetUseWhiteList(useWhiteList);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsXPIDLCString whiteListAbURI;
-    rv = GetCharValue("whiteListAbURI", getter_Copies(whiteListAbURI));
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetWhiteListAbURI(whiteListAbURI);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    PRBool purgeSpam;
-    rv = GetBoolValue("purgeSpam", &purgeSpam);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetPurge(purgeSpam);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    PRInt32 purgeSpamInterval;
-    rv = GetIntValue("purgeSpamInterval", &purgeSpamInterval);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetPurgeInterval(purgeSpamInterval);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsXPIDLCString serverFilterName;
-    rv = GetCharValue("serverFilterName", getter_Copies(serverFilterName));
-    if (NS_SUCCEEDED(rv))
-      mSpamSettings->SetServerFilterName(serverFilterName);
-    PRInt32 serverFilterTrustFlags = 0;
-    rv = GetIntValue("serverFilterTrustFlags", &serverFilterTrustFlags);
-    mSpamSettings->SetServerFilterTrustFlags(serverFilterTrustFlags);
-
-    PRInt32 loggingEnabled;
-    rv = GetBoolValue("spamLoggingEnabled", &loggingEnabled);
-    NS_ENSURE_SUCCESS(rv,rv);
-    rv = mSpamSettings->SetLoggingEnabled(loggingEnabled);
+    mSpamSettings->Initialize(this);
     NS_ENSURE_SUCCESS(rv,rv);
   }
 
@@ -2434,3 +2304,52 @@ NS_IMETHODIMP nsMsgIncomingServer::GetIsDeferredTo(PRBool *aIsDeferredTo)
   *aIsDeferredTo = PR_FALSE;
   return NS_OK;
 }
+
+const long kMaxDownloadTableSize = 500;
+
+// aData is the server, from that we get the cutoff point, below which we evict
+// element is the arrival index of the msg.
+/* static */PRBool nsMsgIncomingServer::evictOldEntries(nsHashKey *aKey, void *element, void *aData)
+{
+  nsMsgIncomingServer *server = (nsMsgIncomingServer *)aData;
+  if (NS_PTR_TO_INT32(element) < server->m_numMsgsDownloaded - kMaxDownloadTableSize/2)
+    return kHashEnumerateRemove;
+  return server->m_downloadedHdrs.Count() > kMaxDownloadTableSize/2;
+}
+
+// hash the concatenation of the message-id and subject as the hash table key, 
+// and store the arrival index as the value. To limit the size of the hash table,
+// we just throw out ones with a lower ordinal value than the cut-off point.
+NS_IMETHODIMP nsMsgIncomingServer::IsNewHdrDuplicate(nsIMsgDBHdr *aNewHdr, PRBool *aResult)
+{
+  NS_ENSURE_ARG_POINTER(aResult);
+  *aResult = PR_FALSE;
+  nsCAutoString strHashKey;
+  nsXPIDLCString messageId, subject;
+  aNewHdr->GetMessageId(getter_Copies(messageId));
+  strHashKey.Append(messageId);
+  aNewHdr->GetSubject(getter_Copies(subject));
+  // err on the side of caution and ignore messages w/o subject or messageid.
+  if (subject.IsEmpty() || messageId.IsEmpty())
+    return NS_OK;
+  strHashKey.Append(subject);
+  nsCStringKey hashKey(strHashKey);
+  PRInt32 hashValue = NS_PTR_TO_INT32(m_downloadedHdrs.Get(&hashKey));
+  if (hashValue)
+  {
+    *aResult = PR_TRUE;
+  }
+  else
+  {
+    // we store the current size of the hash table as the hash
+    // value - this allows us to delete older entries.
+    m_downloadedHdrs.Put(&hashKey, NS_INT32_TO_PTR(++m_numMsgsDownloaded));
+    // Check if hash table is larger than some reasonable size
+    // and if is it, iterate over hash table deleting messages
+    // with an arrival index < number of msgs downloaded - half the reasonable size.
+    if (m_downloadedHdrs.Count() >= kMaxDownloadTableSize)
+      m_downloadedHdrs.Enumerate(evictOldEntries, this);
+  }
+  return NS_OK;
+}
+
