@@ -43,13 +43,11 @@
 #include "nsISocketTransportService.h"
 #include "nsISocketTransport.h"
 #include "nsXPIDLString.h"
-#include "nsSpecialSystemDirectory.h"
 #include "nsILoadGroup.h"
 #include "nsIIOService.h"
 #include "nsNetUtil.h"
 #include "nsIFileURL.h"
 #include "nsFileStream.h"
-#include "nsIDNSService.h"
 #include "nsIMsgWindow.h"
 #include "nsIMsgStatusFeedback.h"
 #include "nsIWebProgressListener.h"
@@ -60,10 +58,11 @@
 #include "nsIStringBundle.h"
 #include "nsIProtocolProxyService.h"
 #include "nsIProxyInfo.h"
-#include "nsEventQueueUtils.h"
-
-static NS_DEFINE_CID(kSocketTransportServiceCID, NS_SOCKETTRANSPORTSERVICE_CID);
-static NS_DEFINE_CID(kStreamTransportServiceCID, NS_STREAMTRANSPORTSERVICE_CID);
+#include "nsThreadUtils.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
+#include "nsDirectoryServiceDefs.h"
+#include "nsMsgUtils.h"
 
 NS_IMPL_THREADSAFE_ADDREF(nsMsgProtocol)
 NS_IMPL_THREADSAFE_RELEASE(nsMsgProtocol)
@@ -86,9 +85,10 @@ nsMsgProtocol::nsMsgProtocol(nsIURI * aURL)
   m_readCount = 0;
   mLoadFlags = 0;
   m_socketIsOpen = PR_FALSE;
-		
-  m_tempMsgFileSpec = nsSpecialSystemDirectory(nsSpecialSystemDirectory::OS_TemporaryDirectory);
-  m_tempMsgFileSpec += "tempMessage.eml";
+
+  GetSpecialDirectoryWithFileName(NS_OS_TEMP_DIR, "tempMessage.eml",
+                                  getter_AddRefs(m_tempMsgFile));
+
   mSuppressListenerNotifications = PR_FALSE;
   InitFromURI(aURL);
 }
@@ -111,6 +111,10 @@ nsresult nsMsgProtocol::InitFromURI(nsIURI *aUrl)
 nsMsgProtocol::~nsMsgProtocol()
 {}
 
+
+static PRBool gGotTimeoutPref;
+static PRInt32 gSocketTimeout = 60;
+
 nsresult
 nsMsgProtocol::OpenNetworkSocketWithInfo(const char * aHostName,
                                          PRInt32 aGetPort,
@@ -121,7 +125,7 @@ nsMsgProtocol::OpenNetworkSocketWithInfo(const char * aHostName,
   NS_ENSURE_ARG(aHostName);
 
   nsresult rv = NS_OK;
-  nsCOMPtr<nsISocketTransportService> socketService (do_GetService(kSocketTransportServiceCID));
+  nsCOMPtr<nsISocketTransportService> socketService (do_GetService(NS_SOCKETTRANSPORTSERVICE_CONTRACTID));
   NS_ENSURE_TRUE(socketService, NS_ERROR_FAILURE);
   
   // with socket connections we want to read as much data as arrives
@@ -137,13 +141,24 @@ nsMsgProtocol::OpenNetworkSocketWithInfo(const char * aHostName,
   strans->SetSecurityCallbacks(callbacks);
 
   // creates cyclic reference!
-  nsCOMPtr<nsIEventQueue> eventQ;
-  NS_GetCurrentEventQ(getter_AddRefs(eventQ));
-  if (eventQ)
-    strans->SetEventSink(this, eventQ);
+  strans->SetEventSink(this, NS_GetCurrentThread());
 
   m_socketIsOpen = PR_FALSE;
   m_transport = strans;
+
+  if (!gGotTimeoutPref)
+  {
+    nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+    if (prefBranch)
+    {
+      prefBranch->GetIntPref("mailnews.tcptimeout", &gSocketTimeout);
+      gGotTimeoutPref = PR_TRUE;
+    }
+  }
+  strans->SetTimeout(nsISocketTransport::TIMEOUT_CONNECT, gSocketTimeout + 60);  
+  strans->SetTimeout(nsISocketTransport::TIMEOUT_READ_WRITE, gSocketTimeout);  
+
+
   return SetupTransportState();
 }
 
@@ -193,9 +208,14 @@ nsMsgProtocol::OpenNetworkSocket(nsIURI * aURL, const char *connectionType,
           if (NS_SUCCEEDED(rv))
               rv = proxyUri->SetScheme(NS_LITERAL_CSTRING("mailto"));
       }
+      //
+      // XXX(darin): Consider using AsyncResolve instead to avoid blocking
+      //             the calling thread in cases where PAC may call into
+      //             our DNS resolver.
+      //
       if (NS_SUCCEEDED(rv))
-          rv = pps->ExamineForProxy(proxyUri, getter_AddRefs(proxyInfo));
-      NS_ASSERTION(NS_SUCCEEDED(rv), "Couldn't successfully call ExamineForProxy");
+          rv = pps->Resolve(proxyUri, 0, getter_AddRefs(proxyInfo));
+      NS_ASSERTION(NS_SUCCEEDED(rv), "Couldn't successfully resolve a proxy");
       if (NS_FAILED(rv)) proxyInfo = nsnull;
   }
 
@@ -243,10 +263,12 @@ nsresult nsMsgProtocol::OpenFileSocket(nsIURI * aURL, PRUint32 aStartPosition, P
 
   // create input stream transport
   nsCOMPtr<nsIStreamTransportService> sts =
-      do_GetService(kStreamTransportServiceCID, &rv);
+      do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
-  rv = sts->CreateInputTransport(stream, aStartPosition, aReadCount, PR_TRUE, getter_AddRefs(m_transport));
+  rv = sts->CreateInputTransport(stream, nsInt64(aStartPosition),
+                                 nsInt64(aReadCount), PR_TRUE,
+                                 getter_AddRefs(m_transport));
 
   m_socketIsOpen = PR_FALSE;
 	return rv;
@@ -261,7 +283,6 @@ nsresult nsMsgProtocol::SetupTransportState()
     // open buffered, blocking output stream
     rv = m_transport->OpenOutputStream(nsITransport::OPEN_BLOCKING, 0, 0, getter_AddRefs(m_outputStream));
     if (NS_FAILED(rv)) return rv;
-    
     // we want to open the stream 
   } // if m_transport
   
@@ -346,6 +367,11 @@ NS_IMETHODIMP nsMsgProtocol::OnStartRequest(nsIRequest *request, nsISupports *ct
     rv = m_channelListener->OnStartRequest(this, m_channelContext);
   }
   
+  nsCOMPtr<nsISocketTransport> strans = do_QueryInterface(m_transport);
+
+  if (strans)
+    strans->SetTimeout(nsISocketTransport::TIMEOUT_READ_WRITE, gSocketTimeout);  
+
   NS_ENSURE_SUCCESS(rv, rv);
   return rv;
 }
@@ -414,6 +440,15 @@ NS_IMETHODIMP nsMsgProtocol::OnStopRequest(nsIRequest *request, nsISupports *ctx
     } // if we got an error code
   } // if we have a mailnews url.
 
+  // Drop notification callbacks to prevent cycles.
+  mCallbacks = 0;
+  mProgressEventSink = 0;
+  // Call CloseSocket(), in case we got here because the server dropped the 
+  // connection while reading, and we never get a chance to get back into 
+  // the protocol state machine via OnDataAvailable. 
+  if (m_socketIsOpen)
+    CloseSocket();
+
   return rv;
 }
 
@@ -468,7 +503,7 @@ nsresult nsMsgProtocol::LoadUrl(nsIURI * aURL, nsISupports * aConsumer)
         
         nsCOMPtr<nsIInputStreamPump> pump;
         rv = NS_NewInputStreamPump(getter_AddRefs(pump),
-          m_inputStream, -1, m_readCount);
+          m_inputStream, nsInt64(-1), m_readCount);
         if (NS_FAILED(rv)) return rv;
         
         m_request = pump; // keep a reference to the pump so we can cancel it
@@ -651,55 +686,47 @@ NS_IMETHODIMP
 nsMsgProtocol::SetNotificationCallbacks(nsIInterfaceRequestor* aNotificationCallbacks)
 {
   mCallbacks = aNotificationCallbacks;
-
-  // Verify that the event sink is http
-  if (mCallbacks) 
-  {
-      nsCOMPtr<nsIProgressEventSink> progressSink;
-     (void)mCallbacks->GetInterface(NS_GET_IID(nsIProgressEventSink),
-                                   getter_AddRefs(progressSink));
-     // only replace our current progress event sink if we were given a new one..
-     if (progressSink) mProgressEventSink  = progressSink;
-  }
-  
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsMsgProtocol::OnTransportStatus(nsITransport *transport, nsresult status,
-                                 PRUint32 progress, PRUint32 progressMax)
+                                 PRUint64 progress, PRUint64 progressMax)
 {
-  if (mProgressEventSink && !(mLoadFlags & LOAD_BACKGROUND))
+  if ((mLoadFlags & LOAD_BACKGROUND) || !m_url)
+    return NS_OK;
+
+  // these transport events should not generate any status messages
+  if (status == nsISocketTransport::STATUS_RECEIVING_FROM ||
+      status == nsISocketTransport::STATUS_SENDING_TO)
+    return NS_OK;
+
+  if (!mProgressEventSink)
   {
-    // these transport events should not generate any status messages
-    if (status == nsISocketTransport::STATUS_RECEIVING_FROM ||
-        status == nsISocketTransport::STATUS_SENDING_TO)
+    NS_QueryNotificationCallbacks(mCallbacks, m_loadGroup, mProgressEventSink);
+    if (!mProgressEventSink)
+      return NS_OK;
+  }
+
+  nsCAutoString host;
+  m_url->GetHost(host);
+
+  nsCOMPtr<nsIMsgMailNewsUrl> mailnewsUrl = do_QueryInterface(m_url);
+  if (mailnewsUrl) 
+  {
+    nsCOMPtr<nsIMsgIncomingServer> server;
+    mailnewsUrl->GetServer(getter_AddRefs(server));
+    if (server)
     {
-      // do nothing....do NOT report socket transport progress bytes either
+      char *realHostName = nsnull;
+      server->GetRealHostName(&realHostName);
+      if (realHostName)
+        host.Adopt(realHostName);
     }
-    else
-    {
-      nsCAutoString host;
-      if (m_url)
-      {
-        m_url->GetHost(host);
-        nsCOMPtr<nsIMsgMailNewsUrl> mailnewsUrl = do_QueryInterface(m_url);
-        if (mailnewsUrl) 
-        {
-          nsCOMPtr<nsIMsgIncomingServer> server;
-          nsresult rv = mailnewsUrl->GetServer(getter_AddRefs(server));
-          if (NS_SUCCEEDED(rv) && server)
-          {
-            nsXPIDLCString realHostName;
-            rv = server->GetRealHostName(getter_Copies(realHostName));
-            if (NS_SUCCEEDED(rv))
-              host = realHostName;
-          }
-        }
-        mProgressEventSink->OnStatus(this, nsnull, status, NS_ConvertUTF8toUCS2(host).get());
-      }
-    }
-  } 
+  }
+  mProgressEventSink->OnStatus(this, nsnull, status,
+                               NS_ConvertUTF8toUTF16(host).get());
+
   return NS_OK;
 }
 
@@ -709,7 +736,7 @@ nsMsgProtocol::OnTransportStatus(nsITransport *transport, nsresult status,
 
 NS_IMETHODIMP nsMsgProtocol::IsPending(PRBool *result)
 {
-    *result = PR_TRUE;
+    *result = m_channelListener != nsnull;
     return NS_OK; 
 }
 
@@ -859,6 +886,90 @@ nsresult nsMsgProtocol::PostMessage(nsIURI* url, nsIFileSpec *fileSpec)
     return NS_OK;
 }
 
+nsresult nsMsgProtocol::DoGSSAPIStep1(const char *service, const char *username, nsCString &response)
+{
+    nsresult rv;
+
+    // if this fails, then it means that we cannot do GSSAPI SASL.
+    m_authModule = do_CreateInstance(NS_AUTH_MODULE_CONTRACTID_PREFIX "sasl-gssapi", &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    m_authModule->Init(service, nsIAuthModule::REQ_DEFAULT, nsnull, NS_ConvertUTF8toUTF16(username).get(), nsnull);
+
+    void *outBuf;
+    PRUint32 outBufLen;
+    rv = m_authModule->GetNextToken((void *)nsnull, 0, &outBuf, &outBufLen);
+    if (NS_SUCCEEDED(rv) && outBuf)
+    {
+        char *base64Str = PL_Base64Encode((char *)outBuf, outBufLen, nsnull);
+        if (base64Str)
+            response.Adopt(base64Str);
+        else 
+            rv = NS_ERROR_OUT_OF_MEMORY;
+        nsMemory::Free(outBuf);
+    }
+
+    return rv;
+}
+
+nsresult nsMsgProtocol::DoGSSAPIStep2(nsCString &commandResponse, nsCString &response)
+{
+    nsresult rv;
+    void *inBuf, *outBuf;
+    PRUint32 inBufLen, outBufLen;
+    PRUint32 len = commandResponse.Length();
+
+    // Cyrus SASL may send us zero length tokens (grrrr)
+    if (len > 0) {
+        // decode into the input secbuffer
+        inBufLen = (len * 3)/4;      // sufficient size (see plbase64.h)
+        inBuf = nsMemory::Alloc(inBufLen);
+        if (!inBuf)
+            return NS_ERROR_OUT_OF_MEMORY;
+
+        // strip off any padding (see bug 230351)
+        const char *challenge = commandResponse.get();
+        while (challenge[len - 1] == '=')
+            len--;
+
+        // We need to know the exact length of the decoded string to give to
+        // the GSSAPI libraries. But NSPR's base64 routine doesn't seem capable
+        // of telling us that. So, we figure it out for ourselves.
+
+        // For every 4 characters, add 3 to the destination
+        // If there are 3 remaining, add 2
+        // If there are 2 remaining, add 1
+        // 1 remaining is an error
+        inBufLen = (len / 4)*3 + ((len % 4 == 3)?2:0) + ((len % 4 == 2)?1:0);
+
+        rv = (PL_Base64Decode(challenge, len, (char *)inBuf))
+             ? m_authModule->GetNextToken(inBuf, inBufLen, &outBuf, &outBufLen)
+             : NS_ERROR_FAILURE;
+
+        nsMemory::Free(inBuf);
+    }
+    else 
+    {
+        rv = m_authModule->GetNextToken(NULL, 0, &outBuf, &outBufLen);
+    }
+    if (NS_SUCCEEDED(rv)) 
+    {
+        // And in return, we may need to send Cyrus zero length tokens back
+        if (outBuf) 
+        {
+            char *base64Str = PL_Base64Encode((char *)outBuf, outBufLen, nsnull);
+            if (base64Str)
+                response.Adopt(base64Str);
+            else 
+                rv = NS_ERROR_OUT_OF_MEMORY;
+        }
+        else
+            response.Adopt((char *)nsMemory::Clone("",1));
+    }
+
+    return rv;
+}
+
 nsresult nsMsgProtocol::DoNtlmStep1(const char *username, const char *password, nsCString &response)
 {
     nsresult rv;
@@ -868,8 +979,8 @@ nsresult nsMsgProtocol::DoNtlmStep1(const char *username, const char *password,
     if (NS_FAILED(rv) || !m_authModule)
         return rv;
 
-    m_authModule->Init(nsnull, 0, nsnull, NS_ConvertUTF8toUCS2(username).get(),
-                       NS_ConvertUTF8toUCS2(password).get());
+    m_authModule->Init(nsnull, 0, nsnull, NS_ConvertUTF8toUTF16(username).get(),
+                       NS_ConvertUTF8toUTF16(password).get());
 
     void *outBuf;
     PRUint32 outBufLen;
@@ -978,7 +1089,7 @@ public:
 
         // try to write again...
         if (NS_SUCCEEDED(rv))
-          rv = aOutStream->AsyncWait(this, 0, 0, mMsgProtocol->mProviderEventQ);
+          rv = aOutStream->AsyncWait(this, 0, 0, mMsgProtocol->mProviderThread);
 
         NS_ASSERTION(NS_SUCCEEDED(rv) || rv == NS_BINDING_ABORTED, "unexpected error writing stream");
         return NS_OK;
@@ -1077,7 +1188,7 @@ NS_IMETHODIMP nsMsgFilePostHelper::OnDataAvailable(nsIRequest * /* aChannel */,
     // things off again.
     mProtInstance->mSuspendedWrite = PR_FALSE;
     mProtInstance->mAsyncOutStream->AsyncWait(mProtInstance->mProvider, 0, 0,
-                                              mProtInstance->mProviderEventQ);
+                                              mProtInstance->mProviderThread);
   } 
 
   return NS_OK;
@@ -1105,6 +1216,8 @@ nsMsgAsyncWriteProtocol::~nsMsgAsyncWriteProtocol()
 
 NS_IMETHODIMP nsMsgAsyncWriteProtocol::Cancel(nsresult status)
 {
+  mGenerateProgressNotifications = PR_FALSE;
+
   if (m_request)
     m_request->Cancel(status);
 
@@ -1342,7 +1455,7 @@ nsresult nsMsgAsyncWriteProtocol::SetupTransportState()
       PR_TRUE, 
       PR_TRUE);
     
-    rv = NS_GetCurrentEventQ(getter_AddRefs(mProviderEventQ));
+    rv = NS_GetCurrentThread(getter_AddRefs(mProviderThread));
     if (NS_FAILED(rv)) return rv;
     
     nsMsgProtocolStreamProvider *provider;
@@ -1360,7 +1473,7 @@ nsresult nsMsgAsyncWriteProtocol::SetupTransportState()
     if (NS_FAILED(rv)) return rv;
     
     // wait for the output stream to become writable
-    rv = mAsyncOutStream->AsyncWait(mProvider, 0, 0, mProviderEventQ);
+    rv = mAsyncOutStream->AsyncWait(mProvider, 0, 0, mProviderThread);
 	} // if m_transport
 
 	return rv;
@@ -1382,7 +1495,7 @@ nsresult nsMsgAsyncWriteProtocol::CloseSocket()
 
   mAsyncOutStream = 0;
   mProvider = 0;
-  mProviderEventQ = 0;
+  mProviderThread = 0;
 	return rv;
 }
 
@@ -1424,7 +1537,7 @@ PRInt32 nsMsgAsyncWriteProtocol::SendData(nsIURI * aURL, const char * dataBuffer
       // data to write (i.e. the pipe went empty). So resume the channel to kick
       // things off again.
       mSuspendedWrite = PR_FALSE;
-      mAsyncOutStream->AsyncWait(mProvider, 0, 0, mProviderEventQ);
+      mAsyncOutStream->AsyncWait(mProvider, 0, 0, mProviderThread);
     } 
     return NS_OK;
   }
