@@ -53,6 +53,8 @@
 #include "nsMsgBaseCID.h"
 #include "nsMsgImapCID.h"
 #include "nsMsgI18N.h"
+#include "nsNativeCharsetUtils.h"
+#include "nsCharTraits.h"
 #include "prprf.h"
 #include "nsNetCID.h"
 #include "nsIIOService.h"
@@ -60,18 +62,28 @@
 #include "nsIMimeConverter.h"
 #include "nsMsgMimeCID.h"
 #include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+#include "nsISupportsPrimitives.h"
+#include "nsIPrefLocalizedString.h"
 #include "nsIRelativeFilePref.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsICategoryManager.h"
 #include "nsCategoryManagerUtils.h"
 #include "nsISpamSettings.h"
 #include "nsISignatureVerifier.h"
+#include "nsICryptoHash.h"
 #include "nsNativeCharsetUtils.h"
+#include "nsIRssIncomingServer.h"
+#include "nsIMsgFolder.h"
+#include "nsIMsgMessageService.h"
+#include "nsIOutputStream.h"
 
 static NS_DEFINE_CID(kImapUrlCID, NS_IMAPURL_CID);
 static NS_DEFINE_CID(kCMailboxUrl, NS_MAILBOXURL_CID);
 static NS_DEFINE_CID(kCNntpUrlCID, NS_NNTPURL_CID);
 
+#define ILLEGAL_FOLDER_CHARS ";#"
+
 #define NS_PASSWORDMANAGER_CATEGORY "passwordmanager"
 static PRBool gInitPasswordManager = PR_FALSE;
 
@@ -87,6 +99,8 @@ nsresult GetMessageServiceContractIDForURI(const char *uri, nsCString &contractI
   nsCAutoString protocol;
   uriStr.Left(protocol, pos);
 
+  if (protocol.Equals("file") && uriStr.Find("application/x-message-display") != kNotFound)
+    protocol.Assign("mailbox");
   //Build message service contractid
   contractID = "@mozilla.org/messenger/messageservice;1?type=";
   contractID += protocol.get();
@@ -248,6 +262,17 @@ inline PRUint32 StringHash(const nsAutoString& str)
                       str.Length() * 2);
 }
 
+// XXX : this may have other clients, in which case we'd better move it to
+//       xpcom/io/nsNativeCharsetUtils with nsAString in place of nsAutoString
+static PRBool ConvertibleToNative(const nsAutoString& str)
+{
+    nsCAutoString native;
+    nsAutoString roundTripped;
+    NS_CopyUnicodeToNative(str, native);
+    NS_CopyNativeToUnicode(native, roundTripped);
+    return str.Equals(roundTripped);
+}
+
 #if defined(XP_MAC)
   const static PRUint32 MAX_LEN = 25;
 #elif defined(XP_UNIX) || defined(XP_BEOS)
@@ -263,7 +288,7 @@ inline PRUint32 StringHash(const nsAutoString& str)
 nsresult NS_MsgHashIfNecessary(nsCAutoString &name)
 {
   NS_NAMED_LITERAL_CSTRING (illegalChars, 
-                            FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS);
+                            FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS ILLEGAL_FOLDER_CHARS);
   nsCAutoString str(name);
 
   // Given a filename, make it safe for filesystem
@@ -307,36 +332,41 @@ nsresult NS_MsgHashIfNecessary(nsCAutoString &name)
 nsresult NS_MsgHashIfNecessary(nsAutoString &name)
 {
   PRInt32 illegalCharacterIndex = name.FindCharInSet(
-                                  FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS);
+                                  FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS ILLEGAL_FOLDER_CHARS);
 
   char hashedname[9];
-  if (illegalCharacterIndex == kNotFound) 
+  PRInt32 keptLength = -1;
+  if (illegalCharacterIndex != kNotFound) 
+      keptLength = illegalCharacterIndex;
+  else if (!ConvertibleToNative(name))
+      keptLength = 0;
+  else if (name.Length() > MAX_LEN) 
   {
-    if (name.Length() > MAX_LEN) 
-    {
-      PR_snprintf(hashedname, 9, "%08lx", (unsigned long) StringHash(name));
-      name.Truncate(MAX_LEN - 8); 
-      AppendASCIItoUTF16(hashedname, name);
-    }
+    keptLength = MAX_LEN-8; 
+    // To avoid keeping only the high surrogate of a surrogate pair
+    if (IS_HIGH_SURROGATE(name.CharAt(keptLength-1)))
+        --keptLength;
   }
-  else 
-  {
-      PR_snprintf(hashedname, 9, "%08lx",
-                (unsigned long) StringHash(name));
-      CopyASCIItoUTF16(hashedname, name);
+
+  if (keptLength >= 0) {
+    PR_snprintf(hashedname, 9, "%08lx", (unsigned long) StringHash(name));
+    name.Truncate(keptLength);
+    AppendASCIItoUTF16(hashedname, name);
   }
-  
+
   return NS_OK;
 }
 
 
-nsresult NS_MsgCreatePathStringFromFolderURI(const char *folderURI, nsCString& pathCString)
+nsresult NS_MsgCreatePathStringFromFolderURI(const char *aFolderURI,
+                                             nsCString& aPathCString,
+                                             PRBool aIsNewsFolder)
 {
   // A file name has to be in native charset. Here we convert 
   // to UTF-16 and check for 'unsafe' characters before converting 
   // to native charset.
-  NS_ENSURE_TRUE(IsUTF8(nsDependentCString(folderURI)), NS_ERROR_UNEXPECTED); 
-  NS_ConvertUTF8toUTF16 oldPath(folderURI);
+  NS_ENSURE_TRUE(IsUTF8(nsDependentCString(aFolderURI)), NS_ERROR_UNEXPECTED); 
+  NS_ConvertUTF8toUTF16 oldPath(aFolderURI);
 
   nsAutoString pathPiece, path;
 
@@ -350,12 +380,21 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *folderURI, nsCString& p
   while (startSlashPos != -1) {
     oldPath.Mid(pathPiece, startSlashPos + 1, endSlashPos - startSlashPos);
     // skip leading '/' (and other // style things)
-    if (!pathPiece.IsEmpty()) {
+    if (!pathPiece.IsEmpty())
+    {
 
       // add .sbd onto the previous path
-      if (haveFirst) {
+      if (haveFirst)
+      {
         path.AppendLiteral(".sbd/");
       }
+
+      if (aIsNewsFolder)
+      {
+          nsCAutoString tmp;
+          CopyUTF16toMUTF7(pathPiece, tmp); 
+          CopyASCIItoUTF16(tmp, pathPiece);
+      }
         
       NS_MsgHashIfNecessary(pathPiece);
       path += pathPiece;
@@ -373,7 +412,7 @@ nsresult NS_MsgCreatePathStringFromFolderURI(const char *folderURI, nsCString& p
       break;
   }
 
-  return NS_CopyUnicodeToNative(path, pathCString);
+  return NS_CopyUnicodeToNative(path, aPathCString);
 }
 
 /* Given a string and a length, removes any "Re:" strings from the front.
@@ -554,21 +593,23 @@ char * NS_MsgSACat (char **destination, const char *source)
   return *destination;
 }
 
-nsresult NS_MsgEscapeEncodeURLPath(const nsAString& str, nsAFlatCString& result)
+nsresult NS_MsgEscapeEncodeURLPath(const nsAString& aStr, nsAFlatCString& aResult)
 {
-  char *escapedString = nsEscape(NS_ConvertUTF16toUTF8(str).get(), url_Path); 
+  char *escapedString = nsEscape(NS_ConvertUTF16toUTF8(aStr).get(), url_Path); 
   if (!*escapedString)
     return NS_ERROR_OUT_OF_MEMORY;
-  result.Adopt(escapedString);
+  aResult.Adopt(escapedString);
   return NS_OK;
 }
 
-nsresult NS_MsgDecodeUnescapeURLPath(const nsASingleFragmentCString& path, nsAString& result)
+nsresult NS_MsgDecodeUnescapeURLPath(const nsACString& aPath,
+                                     nsAString& aResult)
 {
   nsCAutoString unescapedName;
-  NS_UnescapeURL(path, esc_FileBaseName|esc_Forced|esc_AlwaysCopy,
+  NS_UnescapeURL(PromiseFlatCString(aPath), 
+                 esc_FileBaseName|esc_Forced|esc_AlwaysCopy,
                  unescapedName);
-  CopyUTF8toUTF16(unescapedName, result);
+  CopyUTF8toUTF16(unescapedName, aResult);
   return NS_OK;
 }
 
@@ -627,11 +668,11 @@ PRBool IsAFromSpaceLine(char *start, const char *end)
 // with one or more '>' (ie, ">From", ">>From", etc) in the input buffer
 // (between 'start' and 'end') and prefix them with a ">" .
 //
-nsresult EscapeFromSpaceLine(nsIFileSpec *pDst, char *start, const char *end)
+nsresult EscapeFromSpaceLine(nsIOutputStream *outputStream, char *start, const char *end)
 {
   nsresult rv;
   char *pChar;
-  PRInt32 written;
+  PRUint32 written;
 
   pChar = start;
   while (start < end)
@@ -643,19 +684,19 @@ nsresult EscapeFromSpaceLine(nsIFileSpec *pDst, char *start, const char *end)
     {
       // Found a line so check if it's a qualified "From " line.
       if (IsAFromSpaceLine(start, pChar))
-        rv = pDst->Write(">", 1, &written);
-
-      rv = pDst->Write(start, pChar-start+2, &written);
+        rv = outputStream->Write(">", 1, &written);
+      PRInt32 lineTerminatorCount = (*(pChar + 1) == nsCRT::LF) ? 2 : 1;
+      rv = outputStream->Write(start, pChar - start + lineTerminatorCount, &written);
       NS_ENSURE_SUCCESS(rv,rv);
-      pChar += 2;
+      pChar += lineTerminatorCount;
       start = pChar;
     }
     else if (start < end)
     {
       // Check and flush out the remaining data and we're done.
       if (IsAFromSpaceLine(start, end))
-        rv = pDst->Write(">", 1, &written);
-      rv = pDst->Write(start, end-start, &written);
+        rv = outputStream->Write(">", 1, &written);
+      rv = outputStream->Write(start, end-start, &written);
       NS_ENSURE_SUCCESS(rv,rv);
       break;
     }
@@ -694,6 +735,7 @@ nsresult IsRFC822HeaderFieldName(const char *aHdr, PRBool *aResult)
   return NS_OK;
 }
 
+// Warning, currently this routine only works for the Junk Folder
 nsresult
 GetOrCreateFolder(const nsACString &aURI, nsIUrlListener *aListener)
 {
@@ -745,9 +787,20 @@ GetOrCreateFolder(const nsACString &aURI, nsIUrlListener *aListener)
       folderPath->Exists(&exists);
     if (!exists)
     {
+      // Hack to work around a localization bug with the Junk Folder.
+      // Please see Bug #270261 for more information...
+      nsXPIDLString localizedJunkName; 
+      msgFolder->GetName(getter_Copies(localizedJunkName));
+
+      // force the junk folder name to be Junk so it gets created on disk correctly...
+      msgFolder->SetName(NS_LITERAL_STRING("Junk").get());
+
       rv = msgFolder->CreateStorageIfMissing(aListener);
       NS_ENSURE_SUCCESS(rv,rv);
 
+      // now restore the localized folder name...
+      msgFolder->SetName(localizedJunkName.get());
+
       // XXX TODO
       // JUNK MAIL RELATED
       // ugh, I hate this hack
@@ -782,14 +835,54 @@ GetOrCreateFolder(const nsACString &aURI, nsIUrlListener *aListener)
   return NS_OK;
 }
 
+nsresult IsRSSArticle(nsIURI * aMsgURI, PRBool *aIsRSSArticle)
+{
+  nsresult rv;
+  *aIsRSSArticle = PR_FALSE;
+
+  nsCOMPtr<nsIMsgMessageUrl> msgUrl = do_QueryInterface(aMsgURI, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsXPIDLCString resourceURI;
+  msgUrl->GetUri(getter_Copies(resourceURI));
+  
+  // get the msg service for this URI
+  nsCOMPtr<nsIMsgMessageService> msgService;
+  rv = GetMessageServiceFromURI(resourceURI.get(), getter_AddRefs(msgService));
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  nsCOMPtr<nsIMsgDBHdr> msgHdr;
+  rv = msgService->MessageURIToMsgHdr(resourceURI, getter_AddRefs(msgHdr));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIMsgMailNewsUrl> mailnewsUrl = do_QueryInterface(aMsgURI, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // get the folder and the server from the msghdr
+  nsCOMPtr<nsIRssIncomingServer> rssServer;
+  nsCOMPtr<nsIMsgFolder> folder;
+  rv = msgHdr->GetFolder(getter_AddRefs(folder));
+  if (NS_SUCCEEDED(rv) && folder)
+  {
+    nsCOMPtr<nsIMsgIncomingServer> server;
+    folder->GetServer(getter_AddRefs(server));
+    rssServer = do_QueryInterface(server);
+
+    if (rssServer)
+      *aIsRSSArticle = PR_TRUE;
+  }
+
+  return rv;
+}
+
+
 // digest needs to be a pointer to a DIGEST_LENGTH (16) byte buffer
 nsresult MSGCramMD5(const char *text, PRInt32 text_len, const char *key, PRInt32 key_len, unsigned char *digest)
 {
   nsresult rv;
-  unsigned char result[DIGEST_LENGTH];
-  unsigned char *presult = result;
 
-  nsCOMPtr<nsISignatureVerifier> verifier = do_GetService(SIGNATURE_VERIFIER_CONTRACTID, &rv);
+  nsCAutoString hash;
+  nsCOMPtr<nsICryptoHash> hasher = do_CreateInstance("@mozilla.org/security/hash;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
 
@@ -802,17 +895,16 @@ nsresult MSGCramMD5(const char *text, PRInt32 text_len, const char *key, PRInt32
   if (key_len > 64) 
   {
 
-    HASHContextStr      *tctx;
-    PRUint32 resultLen;
-
-    rv = verifier->HashBegin(nsISignatureVerifier::MD5, &tctx);
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = verifier->HashUpdate(tctx, key, key_len);
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = verifier->HashEnd(tctx, &presult, &resultLen, DIGEST_LENGTH);
+    rv = hasher->Init(nsICryptoHash::MD5);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    key = (const char *) result;
+    rv = hasher->Update((const PRUint8*) key, key_len);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = hasher->Finish(PR_FALSE, hash);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    key = hash.get();
     key_len = DIGEST_LENGTH;
   }
 
@@ -842,21 +934,25 @@ nsresult MSGCramMD5(const char *text, PRInt32 text_len, const char *key, PRInt32
   /*
    * perform inner MD5
    */
-  HASHContextStr      *context;
-  PRUint32 resultLen;
+  nsCAutoString result;
+  rv = hasher->Init(nsICryptoHash::MD5); /* init context for 1st pass */
+  rv = hasher->Update((const PRUint8*)innerPad, 64);       /* start with inner pad */
+  rv = hasher->Update((const PRUint8*)text, text_len);     /* then text of datagram */
+  rv = hasher->Finish(PR_FALSE, result);   /* finish up 1st pass */
 
-  rv = verifier->HashBegin(nsISignatureVerifier::MD5, &context); /* init context for 1st pass */
-  rv = verifier->HashUpdate(context, innerPad, 64);      /* start with inner pad */
-  rv = verifier->HashUpdate(context, text, text_len); /* then text of datagram */
-  rv = verifier->HashEnd(context, &presult, &resultLen, DIGEST_LENGTH);          /* finish up 1st pass */
   /*
    * perform outer MD5
    */
-  verifier->HashBegin(nsISignatureVerifier::MD5, &context);  /* init context for 2nd pass */
-  rv = verifier->HashUpdate(context, outerPad, 64);     /* start with outer pad */
-  rv = verifier->HashUpdate(context, (const char *) result, 16);     /* then results of 1st hash */
-  rv = verifier->HashEnd(context, &presult, &resultLen, DIGEST_LENGTH);  /* finish up 2nd pass */
-  memcpy(digest, result, DIGEST_LENGTH);
+  hasher->Init(nsICryptoHash::MD5);       /* init context for 2nd pass */
+  rv = hasher->Update((const PRUint8*)outerPad, 64);    /* start with outer pad */
+  rv = hasher->Update((const PRUint8*)result.get(), 16);/* then results of 1st hash */
+  rv = hasher->Finish(PR_FALSE, result);    /* finish up 2nd pass */
+
+  if (result.Length() != DIGEST_LENGTH)
+    return NS_ERROR_UNEXPECTED;
+
+  memcpy(digest, result.get(), DIGEST_LENGTH);
+
   return rv;
 
 }
@@ -866,25 +962,27 @@ nsresult MSGCramMD5(const char *text, PRInt32 text_len, const char *key, PRInt32
 nsresult MSGApopMD5(const char *text, PRInt32 text_len, const char *password, PRInt32 password_len, unsigned char *digest)
 {
   nsresult rv;
-  unsigned char result[DIGEST_LENGTH];
-  unsigned char *presult = result;
+  nsCAutoString result;
 
-  nsCOMPtr<nsISignatureVerifier> verifier = do_GetService(SIGNATURE_VERIFIER_CONTRACTID, &rv);
+  nsCOMPtr<nsICryptoHash> hasher = do_CreateInstance("@mozilla.org/security/hash;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
+  rv = hasher->Init(nsICryptoHash::MD5);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  HASHContextStr      *context;
-  PRUint32 resultLen;
+  rv = hasher->Update((const PRUint8*) text, text_len);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = verifier->HashBegin(nsISignatureVerifier::MD5, &context);
+  rv = hasher->Update((const PRUint8*) password, password_len);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = verifier->HashUpdate(context, text, text_len);
+
+  rv = hasher->Finish(PR_FALSE, result);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = verifier->HashUpdate(context, password, password_len);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = verifier->HashEnd(context, &presult, &resultLen, DIGEST_LENGTH);
-  NS_ENSURE_SUCCESS(rv, rv);
-  memcpy(digest, result, DIGEST_LENGTH);
+
+  if (result.Length() != DIGEST_LENGTH)
+    return NS_ERROR_UNEXPECTED;
+
+  memcpy(digest, result.get(), DIGEST_LENGTH);
   return rv;
 }
 
@@ -973,3 +1071,273 @@ NS_MSG_BASE nsresult NS_SetPersistentFile(const char *relPrefName,
     return rv;
 }
 
+NS_MSG_BASE nsresult NS_GetUnicharPreferenceWithDefault(nsIPrefBranch *prefBranch,  //can be null, if so uses the root branch
+                                                        const char *prefName,
+                                                        const nsString& defValue,
+                                                        nsString& prefValue)
+{
+    NS_ENSURE_ARG(prefName);
+
+    nsCOMPtr<nsIPrefBranch> pbr;
+    if(!prefBranch) {
+        pbr = do_GetService(NS_PREFSERVICE_CONTRACTID);
+        prefBranch = pbr;
+    }
+    nsCOMPtr<nsISupportsString> str;
+
+    nsresult rv = prefBranch->GetComplexValue(prefName, NS_GET_IID(nsISupportsString), getter_AddRefs(str));
+    if (NS_SUCCEEDED(rv))
+        return str->GetData(prefValue);
+
+    prefValue = defValue;
+    return NS_OK;
+}
+ 
+NS_MSG_BASE nsresult NS_GetLocalizedUnicharPreferenceWithDefault(nsIPrefBranch *prefBranch,  //can be null, if so uses the root branch
+                                                                 const char *prefName,
+                                                                 const nsString& defValue,
+                                                                 nsXPIDLString& prefValue)
+{
+    NS_ENSURE_ARG(prefName);
+
+    nsCOMPtr<nsIPrefBranch> pbr;
+    if(!prefBranch) {
+        pbr = do_GetService(NS_PREFSERVICE_CONTRACTID);
+        prefBranch = pbr;
+    }
+
+    nsCOMPtr<nsIPrefLocalizedString> str;
+
+    nsresult rv = prefBranch->GetComplexValue(prefName, NS_GET_IID(nsIPrefLocalizedString), getter_AddRefs(str));
+    if (NS_SUCCEEDED(rv))
+        str->ToString(getter_Copies(prefValue));
+    else
+        prefValue = defValue;
+    return NS_OK;
+}
+
+void PRTime2Seconds(PRTime prTime, PRUint32 *seconds)
+{
+  PRInt64 microSecondsPerSecond, intermediateResult;
+  
+  LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
+  LL_DIV(intermediateResult, prTime, microSecondsPerSecond);
+  LL_L2UI((*seconds), intermediateResult);
+}
+
+void PRTime2Seconds(PRTime prTime, PRInt32 *seconds)
+{
+  PRInt64 microSecondsPerSecond, intermediateResult;
+  
+  LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
+  LL_DIV(intermediateResult, prTime, microSecondsPerSecond);
+  LL_L2I((*seconds), intermediateResult);
+}
+
+void Seconds2PRTime(PRUint32 seconds, PRTime *prTime)
+{
+  PRInt64 microSecondsPerSecond, intermediateResult;
+  
+  LL_I2L(microSecondsPerSecond, PR_USEC_PER_SEC);
+  LL_UI2L(intermediateResult, seconds);
+  LL_MUL((*prTime), intermediateResult, microSecondsPerSecond);
+}
+
+nsresult GetSummaryFileLocation(nsIFile* fileLocation, nsIFile** summaryLocation)
+{
+  nsIFile* newSummaryLocation;
+
+  nsresult rv = fileLocation->Clone(&newSummaryLocation);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsXPIDLCString fileName;
+
+  rv = newSummaryLocation->GetNativeLeafName(fileName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  fileName.Append(NS_LITERAL_CSTRING(SUMMARY_SUFFIX));
+
+  rv = newSummaryLocation->SetNativeLeafName(fileName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  *summaryLocation = newSummaryLocation;
+  return NS_OK;
+}
+
+nsresult GetSummaryFileLocation(nsIFileSpec* fileLocation, nsIFileSpec** summaryLocation)
+{
+  nsIFileSpec* newSummaryLocation;
+  nsresult rv = NS_NewFileSpec(&newSummaryLocation);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = newSummaryLocation->FromFileSpec(fileLocation);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsXPIDLCString fileName;
+
+  rv = newSummaryLocation->GetLeafName(getter_Copies(fileName));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  fileName.Append(NS_LITERAL_CSTRING(SUMMARY_SUFFIX));
+  
+  rv = newSummaryLocation->SetLeafName(fileName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  *summaryLocation = newSummaryLocation;
+  return NS_OK;
+}
+
+nsresult GetSummaryFileLocation(nsIFileSpec* fileLocation, nsFileSpec* summaryLocation)
+{
+  nsCOMPtr<nsIFileSpec> summaryIFile;
+  nsresult rv = GetSummaryFileLocation(fileLocation, getter_AddRefs(summaryIFile));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return summaryIFile->GetFileSpec(summaryLocation);
+}
+
+void MsgGenerateNowStr(nsACString &nowStr)
+{
+  char dateBuf[100];
+  dateBuf[0] = '\0';
+  PRExplodedTime exploded;
+  PR_ExplodeTime(PR_Now(), PR_LocalTimeParameters, &exploded);
+  PR_FormatTimeUSEnglish(dateBuf, sizeof(dateBuf), "%a %b %d %H:%M:%S %Y", &exploded);
+  nowStr.Assign(dateBuf);
+}
+
+void GetSummaryFileLocation(nsFileSpec& fileLocation, nsFileSpec* summaryLocation)
+{
+  nsXPIDLCString fileName;
+
+  // First copy all the details across
+  *summaryLocation = fileLocation;
+
+  // Now work out the new file name.
+  fileName.Adopt(fileLocation.GetLeafName());
+
+  fileName.Append(NS_LITERAL_CSTRING(SUMMARY_SUFFIX));
+  
+  summaryLocation->SetLeafName(fileName);
+}
+
+// Gets a special directory and appends the supplied file name onto it.
+nsresult GetSpecialDirectoryWithFileName(const char* specialDirName,
+                                         const char* fileName,
+                                         nsIFile** result)
+{
+  nsresult rv = NS_GetSpecialDirectory(specialDirName, result);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return (*result)->AppendNative(nsDependentCString(fileName));
+}
+
+// XXX This function is provided temporarily whilst we are still working
+// on bug 33451 to remove nsIFileSpec from mailnews.
+nsresult GetSpecialDirectoryWithFileName(const char* specialDirName,
+                                         const char* fileName,
+                                         nsIFileSpec** result)
+{
+  nsCOMPtr<nsIFile> tmpFile;
+  nsresult rv = GetSpecialDirectoryWithFileName(specialDirName, fileName,
+                                                getter_AddRefs(tmpFile));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_NewFileSpecFromIFile(tmpFile, result);
+}
+
+// XXX This function is provided temporarily whilst we are still working
+// on bug 33451 to remove nsIFileSpec from mailnews.
+nsresult GetSpecialDirectoryWithFileName(const char* specialDirName,
+                                         const char* fileName,
+                                         nsFileSpec* result)
+{
+
+
+  nsCOMPtr<nsIFileSpec> tmpFile;
+  nsresult rv = GetSpecialDirectoryWithFileName(specialDirName, fileName,
+                                                getter_AddRefs(tmpFile));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return tmpFile->GetFileSpec(result);
+}
+
+PRBool MsgFindKeyword(const nsACString &keyword, nsACString &keywords, nsACString::const_iterator &start, nsACString::const_iterator &end)
+{
+  keywords.BeginReading(start);
+  keywords.EndReading(end);
+  if (*start == ' ')
+    start++;
+  nsACString::const_iterator saveStart(start), saveEnd(end);
+  while (PR_TRUE)
+  {
+    if (FindInReadable(keyword, start, end))
+    {
+      PRBool beginMatches = start == saveStart;
+      PRBool endMatches = end == saveEnd;
+      nsACString::const_iterator beforeStart(start);
+      beforeStart--;
+      // start and end point to the beginning and end of the match
+      if (beginMatches && (end == saveEnd || *end == ' ')
+        || (endMatches && *beforeStart == ' ')
+        || *beforeStart == ' ' && *end == ' ')
+      {
+        if (*end == ' ')
+          end++;
+        if (*beforeStart == ' ' && endMatches)
+          start--;
+        return PR_TRUE;
+      }
+      else 
+        start = end; // advance past bogus match.
+    }
+    else
+      break;
+  }
+  return PR_FALSE;
+}
+
+PRBool MsgHostDomainIsTrusted(nsCString &host, nsCString &trustedMailDomains)
+{
+  const char *domain, *domainEnd, *end;
+  PRUint32 hostLen, domainLen;
+  PRBool domainIsTrusted = PR_FALSE;
+  
+  domain = trustedMailDomains.BeginReading();
+  domainEnd = trustedMailDomains.EndReading(); 
+  nsACString::const_iterator hostStart;
+  
+  host.BeginReading(hostStart);
+  hostLen = host.Length();
+  
+  do {
+    // skip any whitespace
+    while (*domain == ' ' || *domain == '\t')
+      ++domain;
+    
+    // find end of this domain in the string
+    end = strchr(domain, ',');
+    if (!end)
+      end = domainEnd;
+    
+    // to see if the hostname is in the domain, check if the domain
+    // matches the end of the hostname.
+    domainLen = end - domain;
+    if (domainLen && hostLen >= domainLen) {
+      const char *hostTail = hostStart.get() + hostLen - domainLen;
+      if (PL_strncasecmp(domain, hostTail, domainLen) == 0) 
+      {
+        // now, make sure either that the hostname is a direct match or
+        // that the hostname begins with a dot.
+        if (hostLen == domainLen || *hostTail == '.' || *(hostTail - 1) == '.')
+        {
+          domainIsTrusted = PR_TRUE;
+          break;
+        }
+      }
+    }
+    
+    domain = end + 1;
+  } while (*end);
+  return domainIsTrusted;
+}
