@@ -44,6 +44,7 @@
 
 #include "nsMsgComposeService.h"
 #include "nsMsgCompCID.h"
+#include "nsIMsgSend.h"
 #include "nsISupportsArray.h"
 #include "nsIServiceManager.h"
 #include "nsIObserverService.h"
@@ -60,11 +61,9 @@
 #include "nsIDOMWindow.h"
 #include "nsEscape.h"
 #include "nsIContentViewer.h"
-
+#include "nsIMsgWindow.h"
 #include "nsIDocShell.h"
-#include "nsIScriptGlobalObject.h"
-#include "nsIWebShell.h"
-#include "nsIWebShellWindow.h"
+#include "nsPIDOMWindow.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
 #include "nsIXULWindow.h"
@@ -72,11 +71,15 @@
 #include "nsIDocShellTreeItem.h"
 #include "nsIDocShellTreeOwner.h"
 #include "nsIBaseWindow.h"
-#include "nsIPref.h"
-#include "nsIPrefBranchInternal.h"
-
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefBranch2.h"
 #include "nsMsgBaseCID.h"
 #include "nsIMsgAccountManager.h"
+#include "nsIMimeMiscStatus.h"
+#include "nsXPFEComponentsCID.h"
+
+#include "nsIInterfaceRequestorUtils.h"
 
 #ifdef MSGCOMP_TRACE_PERFORMANCE
 #include "prlog.h"
@@ -91,6 +94,11 @@
 #include "nsIContentSink.h"
 #include "mozISanitizingSerializer.h"
 
+#ifdef MOZ_XUL_APP
+#include "nsICommandLine.h"
+#include "nsIAppStartup.h"
+#endif
+
 #ifdef XP_WIN32
 #include <windows.h>
 #include <shellapi.h>
@@ -98,7 +106,6 @@
 #endif
 
 static NS_DEFINE_CID(kParserCID, NS_PARSER_CID);
-static NS_DEFINE_CID(kNavDTDCID, NS_CNAVDTD_CID);
 // </for>
 
 #ifdef NS_DEBUG
@@ -157,7 +164,11 @@ nsMsgComposeService::nsMsgComposeService()
   mCachedWindows = nsnull;
 }
 
-NS_IMPL_ISUPPORTS4(nsMsgComposeService, nsIMsgComposeService, nsIObserver, nsICmdLineHandler, nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS4(nsMsgComposeService,
+                   nsIMsgComposeService,
+                   nsIObserver,
+                   ICOMMANDLINEHANDLER,
+                   nsISupportsWeakReference)
 
 nsMsgComposeService::~nsMsgComposeService()
 {
@@ -177,12 +188,12 @@ nsresult nsMsgComposeService::Init()
   nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
   if (NS_SUCCEEDED(rv))
   {
-    rv = observerService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_TRUE);
+    rv = observerService->AddObserver(this, "quit-application", PR_TRUE);
     rv = observerService->AddObserver(this, "profile-do-change", PR_TRUE);
   }
 
   // Register some pref observer
-  nsCOMPtr<nsIPrefBranchInternal> pbi = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  nsCOMPtr<nsIPrefBranch2> pbi = do_GetService(NS_PREFSERVICE_CONTRACTID);
   if (pbi)
     rv = pbi->AddObserver(PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS, this, PR_TRUE);
 
@@ -225,7 +236,7 @@ void nsMsgComposeService::DeleteCachedWindows()
   PRInt32 i;
   for (i = 0; i < mMaxRecycledWindows; i ++)
   {
-    CloseWindow(mCachedWindows[i].window);
+    CloseHiddenCachedWindow(mCachedWindows[i].window);
     mCachedWindows[i].Clear();
   }
 }
@@ -296,16 +307,16 @@ nsresult nsMsgComposeService::OpenWindow(const char *chrome, nsIMsgComposeParams
   return rv;
 }
 
-void nsMsgComposeService::CloseWindow(nsIDOMWindowInternal *domWindow)
+void nsMsgComposeService::CloseHiddenCachedWindow(nsIDOMWindowInternal *domWindow)
 {
   if (domWindow)
   {
     nsCOMPtr<nsIDocShell> docshell;
-    nsCOMPtr<nsIScriptGlobalObject> globalObj(do_QueryInterface(domWindow));
-    if (globalObj)
+    nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(domWindow));
+    if (window)
     {
       nsCOMPtr<nsIDocShellTreeItem> treeItem =
-        do_QueryInterface(globalObj->GetDocShell());
+        do_QueryInterface(window->GetDocShell());
 
       if (treeItem)
       {
@@ -315,8 +326,20 @@ void nsMsgComposeService::CloseWindow(nsIDOMWindowInternal *domWindow)
         {
           nsCOMPtr<nsIBaseWindow> baseWindow;
           baseWindow = do_QueryInterface(treeOwner);
-          if (baseWindow)
+          if (baseWindow) {
+#ifdef MOZ_XUL_APP
+            // HACK ALERT: when we hid this window we fired the "xul-window-destroyed"
+            // notification for it. Now that it's being really-destroyed it will fire that
+            // notification *again* for itself. The appstartup code maintains an internal
+            // reference count of windows that block app shutdown: we want to increment that
+            // count without cancelling app shutdown (so don't use "xul-window-registered").
+            nsCOMPtr<nsIAppStartup> appStartup(do_GetService(NS_APPSTARTUP_CONTRACTID));
+            if (appStartup)
+              appStartup->EnterLastWindowClosingSurvivalArea();
+#endif
+
             baseWindow->Destroy();
+          }
         }
       }
     }
@@ -326,7 +349,7 @@ void nsMsgComposeService::CloseWindow(nsIDOMWindowInternal *domWindow)
 NS_IMETHODIMP
 nsMsgComposeService::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
 {
-  if (!strcmp(aTopic,"profile-do-change") || !strcmp(aTopic,NS_XPCOM_SHUTDOWN_OBSERVER_ID))
+  if (!strcmp(aTopic, "profile-do-change") || !strcmp(aTopic, "quit-application"))
   {
     DeleteCachedWindows();
     return NS_OK;
@@ -402,13 +425,16 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, const ch
   /* Actually, the only way to implement forward inline is to simulate a template message. 
      Maybe one day when we will have more time we can change that
   */
-  if (type == nsIMsgCompType::ForwardInline || type == nsIMsgCompType::Draft || type == nsIMsgCompType::Template)
+  if (type == nsIMsgCompType::ForwardInline || type == nsIMsgCompType::Draft || type == nsIMsgCompType::Template
+    || type == nsIMsgCompType::ReplyWithTemplate)
   {
     nsCOMPtr<nsIMsgDraft> pMsgDraft (do_CreateInstance(NS_MSGDRAFT_CONTRACTID, &rv));
     if (NS_SUCCEEDED(rv) && pMsgDraft)
     {
       nsCAutoString uriToOpen(originalMsgURI);
-      uriToOpen.Append("?fetchCompleteMessage=true"); 
+
+      uriToOpen += (uriToOpen.FindChar('?') == kNotFound) ? "?" : "&";
+      uriToOpen.Append("fetchCompleteMessage=true"); 
 
       switch(type)
       {
@@ -453,12 +479,32 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, const ch
           }
           else
             group = originalMsgURI;
-          
-          pMsgCompFields->SetNewsgroups(group.get());
+
+          nsCAutoString unescapedName;
+          NS_UnescapeURL(group, 
+                         esc_FileBaseName|esc_Forced|esc_AlwaysCopy,
+                         unescapedName);
+          pMsgCompFields->SetNewsgroups(NS_ConvertUTF8toUTF16(unescapedName));
           pMsgCompFields->SetNewshost(host.get());
         }
         else
+        {
           pMsgComposeParams->SetOriginalMsgURI(originalMsgURI);
+          if (PL_strstr(originalMsgURI, "type=application/x-message-display"))
+          {
+            nsCOMPtr <nsIMsgDBHdr> msgHdr;
+            if (strncmp(originalMsgURI, "file:", 5))
+              rv = GetMsgDBHdrFromURI(originalMsgURI, getter_AddRefs(msgHdr));
+            if (aMsgWindow && !msgHdr)
+            {
+              nsCOMPtr <nsIMsgHeaderSink> headerSink;
+              rv = aMsgWindow->GetMsgHeaderSink(getter_AddRefs(headerSink));
+              if (headerSink)
+                rv = headerSink->GetDummyMsgHeader(getter_AddRefs(msgHdr));
+            }
+            pMsgComposeParams->SetOrigMsgHdr(msgHdr);
+          }
+        }
       }
 
       pMsgComposeParams->SetComposeFields(pMsgCompFields);
@@ -497,6 +543,7 @@ NS_IMETHODIMP nsMsgComposeService::GetParamsForMailto(nsIURI * aURI, nsIMsgCompo
        nsXPIDLCString aSubjectPart;
        nsXPIDLCString aBodyPart;
        nsXPIDLCString aNewsgroup;
+       nsXPIDLCString aRefPart;
        nsXPIDLCString aHTMLBodyPart;
 
        // we are explictly not allowing attachments to be specified in mailto: urls
@@ -507,7 +554,7 @@ NS_IMETHODIMP nsMsgComposeService::GetParamsForMailto(nsIURI * aURI, nsIMsgCompo
                                     nsnull /* follow */, nsnull /* organization */, 
                                     nsnull /* reply to part */, getter_Copies(aSubjectPart),
                                     getter_Copies(aBodyPart), getter_Copies(aHTMLBodyPart) /* html part */, 
-                                    nsnull /* a ref part */, nsnull /* attachment part, must always null, see #99055 */,
+                                    getter_Copies(aRefPart), nsnull /* attachment part, must always null, see #99055 */,
                                     nsnull /* priority */, getter_Copies(aNewsgroup), nsnull /* host */,
                                     &requestedComposeFormat);
 
@@ -545,17 +592,13 @@ NS_IMETHODIMP nsMsgComposeService::GetParamsForMailto(nsIURI * aURI, nsIMsgCompo
             sanSink->Initialize(&sanitizedBody, 0, NS_ConvertASCIItoUTF16(allowedTags));
 
             parser->SetContentSink(sink);
-            nsCOMPtr<nsIDTD> dtd = do_CreateInstance(kNavDTDCID);
-            if (dtd)
+            rv = parser->Parse(rawBody, 0, NS_LITERAL_CSTRING("text/html"), PR_TRUE);
+            if (NS_FAILED(rv))
             {
-              parser->RegisterDTD(dtd);
-
-              rv = parser->Parse(rawBody, 0, NS_LITERAL_CSTRING("text/html"), PR_FALSE, PR_TRUE);
-              if (NS_FAILED(rv))
-                // Something went horribly wrong with parsing for html format
-                // in the body.  Set composeHTMLFormat to PR_FALSE so we show the
-                // plain text mail compose.
-                composeHTMLFormat = PR_FALSE;
+              // Something went horribly wrong with parsing for html format
+              // in the body.  Set composeHTMLFormat to PR_FALSE so we show the
+              // plain text mail compose.
+              composeHTMLFormat = PR_FALSE;
             }
           }
         }
@@ -575,7 +618,8 @@ NS_IMETHODIMP nsMsgComposeService::GetParamsForMailto(nsIURI * aURI, nsIMsgCompo
           pMsgCompFields->SetTo(NS_ConvertUTF8toUTF16(aToPart));
           pMsgCompFields->SetCc(NS_ConvertUTF8toUTF16(aCcPart));
           pMsgCompFields->SetBcc(NS_ConvertUTF8toUTF16(aBccPart));
-          pMsgCompFields->SetNewsgroups(aNewsgroup);
+          pMsgCompFields->SetNewsgroups(NS_ConvertUTF8toUTF16(aNewsgroup));
+          pMsgCompFields->SetReferences(aRefPart);
           pMsgCompFields->SetSubject(NS_ConvertUTF8toUTF16(aSubjectPart));
           pMsgCompFields->SetBody(composeHTMLFormat ? sanitizedBody : rawBody);
           pMsgComposeParams->SetComposeFields(pMsgCompFields);
@@ -623,12 +667,12 @@ nsresult nsMsgComposeService::OpenComposeWindowWithParams(const char *msgCompose
 // begin shameless copying from nsNativeAppSupportWin
 HWND hwndForComposeDOMWindow( nsISupports *window ) 
 {
-  nsCOMPtr<nsIScriptGlobalObject> ppScriptGlobalObj( do_QueryInterface(window) );
-  if ( !ppScriptGlobalObj )
+  nsCOMPtr<nsPIDOMWindow> win( do_QueryInterface(window) );
+  if ( !win )
       return 0;
 
   nsCOMPtr<nsIBaseWindow> ppBaseWindow =
-    do_QueryInterface( ppScriptGlobalObj->GetDocShell() );
+    do_QueryInterface( win->GetDocShell() );
   if (!ppBaseWindow) return 0;
 
   nsCOMPtr<nsIWidget> ppWidget;
@@ -803,7 +847,7 @@ nsMsgComposeService::CacheWindow(nsIDOMWindowInternal *aWindow, PRBool aComposeH
   */
   if (sameTypeId == -1 && oppositeTypeId != -1)
   {
-    CloseWindow(mCachedWindows[oppositeTypeId].window);
+    CloseHiddenCachedWindow(mCachedWindows[oppositeTypeId].window);
     mCachedWindows[oppositeTypeId].Clear();
     
     rv = ShowCachedComposeWindow(aWindow, PR_FALSE);
@@ -816,24 +860,361 @@ nsMsgComposeService::CacheWindow(nsIDOMWindowInternal *aWindow, PRBool aComposeH
   return NS_ERROR_NOT_AVAILABLE;
 }
 
+class nsMsgTemplateReplyHelper :  public nsIStreamListener, public nsIUrlListener
+{
+public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIURLLISTENER
+  NS_DECL_NSISTREAMLISTENER
+  NS_DECL_NSIREQUESTOBSERVER
+
+  nsMsgTemplateReplyHelper();
+  ~nsMsgTemplateReplyHelper();
+
+  nsCOMPtr <nsIMsgDBHdr> mHdrToReplyTo;
+  nsCOMPtr <nsIMsgDBHdr> mTemplateHdr;
+  nsCOMPtr <nsIMsgWindow> mMsgWindow;
+  nsCOMPtr <nsIMsgIncomingServer> mServer;
+  nsCString mTemplateBody; 
+  PRBool mInMsgBody;
+  char mLastBlockChars[3];
+};
+
+NS_IMPL_ISUPPORTS2(nsMsgTemplateReplyHelper, nsIStreamListener, nsIUrlListener)
+
+nsMsgTemplateReplyHelper::nsMsgTemplateReplyHelper()
+{
+  mInMsgBody = PR_FALSE;
+  memset(mLastBlockChars, 0, sizeof(mLastBlockChars));
+}
+
+nsMsgTemplateReplyHelper::~nsMsgTemplateReplyHelper()
+{
+}
+
+
+NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStartRunningUrl(nsIURI *aUrl)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI *aUrl, nsresult aExitCode)
+{
+
+  NS_ENSURE_SUCCESS(aExitCode, aExitCode);
+  nsresult rv;
+  nsCOMPtr<nsIDOMWindowInternal> parentWindow;
+  if (mMsgWindow)
+  {
+    nsCOMPtr<nsIDocShell> docShell;
+    rv = mMsgWindow->GetRootDocShell(getter_AddRefs(docShell));
+    NS_ENSURE_SUCCESS(rv, rv);
+    parentWindow = do_GetInterface(docShell);
+    NS_ENSURE_TRUE(parentWindow, NS_ERROR_FAILURE);
+  }
+  if ( NS_FAILED(rv) ) return rv ;
+  // get the MsgIdentity for the above key using AccountManager
+  nsCOMPtr <nsIMsgAccountManager> accountManager = do_GetService (NS_MSGACCOUNTMANAGER_CONTRACTID) ;
+  if (NS_FAILED(rv) || (!accountManager) ) return rv ;
+
+  nsCOMPtr <nsIMsgAccount> account;
+  nsCOMPtr <nsIMsgIdentity> identity;
+
+  rv = accountManager->FindAccountForServer(mServer, getter_AddRefs(account));
+  NS_ENSURE_SUCCESS(rv, rv);
+  account->GetDefaultIdentity(getter_AddRefs(identity));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // create the compose params object 
+  nsCOMPtr<nsIMsgComposeParams> pMsgComposeParams (do_CreateInstance(NS_MSGCOMPOSEPARAMS_CONTRACTID, &rv));
+  if (NS_FAILED(rv) || (!pMsgComposeParams) ) return rv ;
+  nsCOMPtr<nsIMsgCompFields> compFields = do_CreateInstance(NS_MSGCOMPFIELDS_CONTRACTID, &rv) ;
+
+  nsXPIDLCString replyTo;
+  mHdrToReplyTo->GetStringProperty("replyTo", getter_Copies(replyTo));
+  if (replyTo.IsEmpty())
+    mHdrToReplyTo->GetAuthor(getter_Copies(replyTo));
+  compFields->SetTo(NS_ConvertUTF8toUTF16(replyTo));
+
+  nsAutoString body;
+  nsXPIDLString templateSubject, replySubject;
+
+  mTemplateHdr->GetMime2DecodedSubject(getter_Copies(templateSubject));
+  mHdrToReplyTo->GetMime2DecodedSubject(getter_Copies(replySubject));
+  if (!replySubject.IsEmpty())
+  {
+    templateSubject.Append(NS_LITERAL_STRING(" (was: "));
+    templateSubject.Append(replySubject);
+    templateSubject.Append(NS_LITERAL_STRING(")"));
+  }
+  compFields->SetSubject(templateSubject);
+  body.AssignWithConversion(mTemplateBody);
+  compFields->SetBody(body);
+  nsXPIDLCString msgUri;
+
+  nsCOMPtr <nsIMsgFolder> folder;
+  mHdrToReplyTo->GetFolder(getter_AddRefs(folder));
+  folder->GetUriForMsg(mHdrToReplyTo, getter_Copies(msgUri));
+  // populate the compose params
+   // we use type new instead of reply - we might need to add a reply with template type.
+  pMsgComposeParams->SetType(nsIMsgCompType::New);
+  pMsgComposeParams->SetFormat(nsIMsgCompFormat::Default);
+  pMsgComposeParams->SetIdentity(identity);
+  pMsgComposeParams->SetComposeFields(compFields); 
+  pMsgComposeParams->SetOriginalMsgURI(msgUri);
+  // create the nsIMsgCompose object to send the object
+  nsCOMPtr<nsIMsgCompose> pMsgCompose (do_CreateInstance(NS_MSGCOMPOSE_CONTRACTID, &rv));
+  if (NS_FAILED(rv) || (!pMsgCompose) ) return rv ;
+
+  /** initialize nsIMsgCompose, Send the message, wait for send completion response **/
+
+  rv = pMsgCompose->Initialize(parentWindow, pMsgComposeParams) ;
+  if (NS_FAILED(rv)) return rv ;
+
+  Release();
+
+  return pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nsnull, nsnull, nsnull) ;
+}
+
+NS_IMETHODIMP
+nsMsgTemplateReplyHelper::OnStartRequest(nsIRequest* request, nsISupports* aSupport)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgTemplateReplyHelper::OnStopRequest(nsIRequest* request, nsISupports* aSupport,
+                                nsresult status)
+{
+  if (NS_SUCCEEDED(status))
+  {
+    // now we've got the message body in mTemplateBody - 
+    // need to set body in compose params and send the reply.
+  }
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgTemplateReplyHelper::OnDataAvailable(nsIRequest* request, 
+                                  nsISupports* aSupport,
+                                  nsIInputStream* inStream, 
+                                  PRUint32 srcOffset,
+                                  PRUint32 count)
+{
+  nsresult rv = NS_OK;
+
+  char readBuf[1024];
+
+  PRUint32 available, readCount;
+  PRUint32 maxReadCount = sizeof(readBuf) - 1;
+
+  rv = inStream->Available(&available);
+  while (NS_SUCCEEDED(rv) && available > 0)
+  {
+    PRInt32 bodyOffset = 0, readOffset = 0;
+    if (!mInMsgBody && mLastBlockChars[0])
+    {
+      memcpy(readBuf, mLastBlockChars, 3);
+      readOffset = 3;
+      maxReadCount -= 3;
+    }
+    if (maxReadCount > available)
+      maxReadCount = available;
+    memset(readBuf, 0, sizeof(readBuf));
+    rv = inStream->Read(readBuf + readOffset, maxReadCount, &readCount);
+    available -= readCount;
+    readCount += readOffset;
+    // we're mainly interested in the msg body, so we need to
+    // find the header/body delimiter of a blank line. A blank line
+    // looks like <CR><CR>, <LF><LF>, or <CRLF><CRLF>
+    if (!mInMsgBody)
+    {
+      for (PRInt32 charIndex = 0; charIndex < readCount && !bodyOffset; charIndex++)
+      {
+        if (readBuf[charIndex] == '\r' || readBuf[charIndex] == '\n')
+        {
+          if (charIndex + 1 < readCount)
+          {
+            if (readBuf[charIndex] == readBuf[charIndex + 1])
+            {
+            // got header+body separator
+              bodyOffset = charIndex + 2;
+              break;
+            }
+            else if ((charIndex + 3 < readCount) && !strncmp(readBuf + charIndex, "\r\n\r\n", 4))
+            {
+              bodyOffset = charIndex + 4;
+              break;
+            }
+          }
+        }
+      }
+      mInMsgBody = bodyOffset != 0;
+      if (!mInMsgBody && readCount > 3) // still in msg hdrs
+        strncpy(mLastBlockChars, readBuf + readCount - 3, 3);
+
+    }
+    mTemplateBody.Append(readBuf + bodyOffset);
+  }
+  return NS_OK;
+}
+
+
+NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(nsIMsgDBHdr *aMsgHdr, const char *templateUri,
+                                             nsIMsgWindow *aMsgWindow, nsIMsgIncomingServer *aServer)
+{
+  // to reply with template, we need the message body of the template 
+  // I think we're going to need to stream the template message to ourselves, 
+  // and construct the body, and call setBody on the compFields.
+  // We need the reply-to header of the msg we're replying to, so
+  // we're going to add that to the db if it's different from "from:"
+  // For imap, we could make adding a reply-to filter append
+  // reply-to to the custom headers...
+
+    nsMsgTemplateReplyHelper *helper = new nsMsgTemplateReplyHelper;
+    if (!helper)
+      return NS_ERROR_OUT_OF_MEMORY;
+
+    NS_ADDREF(helper);
+
+    helper->mHdrToReplyTo = aMsgHdr;
+    helper->mMsgWindow = aMsgWindow;
+    helper->mServer = aServer;
+
+    nsCOMPtr <nsIMsgFolder> templateFolder;
+    nsCOMPtr <nsIMsgDatabase> templateDB;
+    nsXPIDLCString templateMsgHdrUri;
+    const char * query = PL_strstr(templateUri, "?messageId=");
+    if (!query)
+      return NS_ERROR_FAILURE;
+
+    nsCAutoString folderUri(Substring(templateUri, query)); 
+    nsresult rv = GetExistingFolder(folderUri.get(), getter_AddRefs(templateFolder));
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = templateFolder->GetMsgDatabase(aMsgWindow, getter_AddRefs(templateDB));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    const char *subject = PL_strstr(templateUri, "&subject=");
+    if (subject)
+    {
+      const char *subjectEnd = subject + strlen(subject);
+      nsCAutoString messageId(Substring(query + 11, subject));
+      nsCAutoString subjectString(Substring(subject + 9, subjectEnd));
+      templateDB->GetMsgHdrForMessageID(messageId.get(), getter_AddRefs(helper->mTemplateHdr));
+      if (helper->mTemplateHdr)
+        templateFolder->GetUriForMsg(helper->mTemplateHdr, getter_Copies(templateMsgHdrUri));
+      // to use the subject, we'd need to expose a method to find a message by subject,
+      // or painfully iterate through messages...We'll try to make the message-id
+      // not change when saving a template first.
+    }
+    if (templateMsgHdrUri.IsEmpty())
+    {
+      // ### probably want to return a specific error and 
+      // have the calling code disable the filter.
+      NS_ASSERTION(PR_FALSE, "failed to get msg hdr");
+      return NS_ERROR_FAILURE;
+    }
+    // we need to convert the template uri, which is of the form
+    // <folder uri>?messageId=<messageId>&subject=<subject>
+    nsCOMPtr <nsIMsgMessageService> msgService;
+    rv = GetMessageServiceFromURI(templateMsgHdrUri, getter_AddRefs(msgService));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsISupports> listenerSupports;
+    helper->QueryInterface(NS_GET_IID(nsISupports), getter_AddRefs(listenerSupports));
+
+    return msgService->StreamMessage(templateMsgHdrUri, listenerSupports, aMsgWindow,
+						helper, PR_FALSE /* convert data */, 
+                                                "", nsnull);
+}
+
+
+NS_IMETHODIMP nsMsgComposeService::ForwardMessage(const nsAString &forwardTo, nsIMsgDBHdr *aMsgHdr, 
+                                             nsIMsgWindow *aMsgWindow, nsIMsgIncomingServer *aServer)
+{
+
+    nsresult rv;
+    nsCOMPtr<nsIDOMWindowInternal> parentWindow;
+    if (aMsgWindow)
+    {
+      nsCOMPtr<nsIDocShell> docShell;
+      rv = aMsgWindow->GetRootDocShell(getter_AddRefs(docShell));
+      NS_ENSURE_SUCCESS(rv, rv);
+      parentWindow = do_GetInterface(docShell);
+      NS_ENSURE_TRUE(parentWindow, NS_ERROR_FAILURE);
+    }
+    if ( NS_FAILED(rv) ) return rv ;
+    // get the MsgIdentity for the above key using AccountManager
+    nsCOMPtr <nsIMsgAccountManager> accountManager = do_GetService (NS_MSGACCOUNTMANAGER_CONTRACTID) ;
+    if (NS_FAILED(rv) || (!accountManager) ) return rv ;
+
+    nsCOMPtr <nsIMsgAccount> account;
+    nsCOMPtr <nsIMsgIdentity> identity;
+
+    rv = accountManager->FindAccountForServer(aServer, getter_AddRefs(account));
+    NS_ENSURE_SUCCESS(rv, rv);
+    account->GetDefaultIdentity(getter_AddRefs(identity));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // create the compose params object 
+    nsCOMPtr<nsIMsgComposeParams> pMsgComposeParams (do_CreateInstance(NS_MSGCOMPOSEPARAMS_CONTRACTID, &rv));
+    if (NS_FAILED(rv) || (!pMsgComposeParams) ) return rv ;
+    nsCOMPtr<nsIMsgCompFields> compFields = do_CreateInstance(NS_MSGCOMPFIELDS_CONTRACTID, &rv) ;
+
+    compFields->SetTo(forwardTo);
+    nsXPIDLCString msgUri;
+    PRInt32 forwardType = 0; 
+
+    nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+    if (prefBranch)
+      prefBranch->GetIntPref("mail.forward_message_mode", &forwardType);
+
+    nsCOMPtr <nsIMsgFolder> folder;
+    aMsgHdr->GetFolder(getter_AddRefs(folder));
+    folder->GetUriForMsg(aMsgHdr, getter_Copies(msgUri));
+    // populate the compose params
+    // right now, forward inline won't work, since that requires opening a compose window,
+    // and would require major whackage of the compose code.
+    pMsgComposeParams->SetType(/* forwardType ? nsIMsgCompType::ForwardInline : */nsIMsgCompType::ForwardAsAttachment);
+    pMsgComposeParams->SetFormat(nsIMsgCompFormat::Default);
+    pMsgComposeParams->SetIdentity(identity);
+    pMsgComposeParams->SetComposeFields(compFields); 
+    pMsgComposeParams->SetOriginalMsgURI(msgUri);
+    // create the nsIMsgCompose object to send the object
+    nsCOMPtr<nsIMsgCompose> pMsgCompose (do_CreateInstance(NS_MSGCOMPOSE_CONTRACTID, &rv));
+    if (NS_FAILED(rv) || (!pMsgCompose) ) return rv ;
+
+    /** initialize nsIMsgCompose, Send the message, wait for send completion response **/
+
+    rv = pMsgCompose->Initialize(parentWindow, pMsgComposeParams) ;
+    if (NS_FAILED(rv)) return rv ;
+
+    return pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nsnull, nsnull, nsnull) ;
+}
+
+
 nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindowInternal *aComposeWindow, PRBool aShow)
 {
   nsresult rv = NS_OK;
 
-  nsCOMPtr <nsIScriptGlobalObject> globalScript = do_QueryInterface(aComposeWindow, &rv);
+  nsCOMPtr<nsIObserverService> obs
+    (do_GetService("@mozilla.org/observer-service;1", &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr <nsPIDOMWindow> window = do_QueryInterface(aComposeWindow, &rv);
 
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsIDocShell *docShell = globalScript->GetDocShell();
+  nsIDocShell *docShell = window->GetDocShell();
 
-  nsCOMPtr <nsIWebShell> webShell = do_QueryInterface(docShell, &rv);
+  nsCOMPtr <nsIDocShellTreeItem> treeItem = do_QueryInterface(docShell, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCOMPtr <nsIWebShellContainer> webShellContainer;
-  rv = webShell->GetContainer(*getter_AddRefs(webShellContainer));
+  nsCOMPtr <nsIDocShellTreeOwner> treeOwner;
+  rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
   NS_ENSURE_SUCCESS(rv,rv);
   
-  if (webShellContainer) {
+  if (treeOwner) {
 
     // the window need to be sticky before we hide it.
     nsCOMPtr<nsIContentViewer> contentViewer;
@@ -844,16 +1225,6 @@ nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindowInternal *aCom
     NS_ENSURE_SUCCESS(rv,rv);
 
     // disable (enable) the cached window
-    nsCOMPtr <nsIWebShellWindow> webShellWindow = do_QueryInterface(webShellContainer, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsCOMPtr<nsIDocShellTreeItem>  treeItem(do_QueryInterface(docShell, &rv));
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
-    rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
-    NS_ENSURE_SUCCESS(rv,rv);
-
     nsCOMPtr<nsIBaseWindow> baseWindow;
     baseWindow = do_QueryInterface(treeOwner, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
@@ -861,7 +1232,7 @@ nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindowInternal *aCom
     baseWindow->SetEnabled(aShow);
     NS_ENSURE_SUCCESS(rv,rv);
 
-    nsCOMPtr <nsIXULWindow> xulWindow = do_QueryInterface(webShellWindow, &rv);
+    nsCOMPtr <nsIXULWindow> xulWindow = do_GetInterface(treeOwner, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
 
     nsCOMPtr<nsIWindowMediator> windowMediator = 
@@ -872,10 +1243,12 @@ nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindowInternal *aCom
     if (aShow) {
       rv = windowMediator->RegisterWindow(xulWindow);
       NS_ENSURE_SUCCESS(rv,rv);
+
+      obs->NotifyObservers(xulWindow, "xul-window-registered", nsnull);
     }
 
     // hide (show) the cached window
-    rv = webShellWindow->Show(aShow);
+    rv = baseWindow->SetVisibility(aShow);
     NS_ENSURE_SUCCESS(rv,rv);
 
     // if hiding, remove the window from the mediator,
@@ -883,6 +1256,8 @@ nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindowInternal *aCom
     if (!aShow) {
       rv = windowMediator->UnregisterWindow(xulWindow);
       NS_ENSURE_SUCCESS(rv,rv);
+
+      obs->NotifyObservers(xulWindow, "xul-window-destroyed", nsnull);
     }
   }
   else {
@@ -1000,7 +1375,88 @@ nsresult nsMsgComposeService::AddGlobalHtmlDomains()
   return NS_OK;
 }
 
+#ifdef MOZ_XUL_APP
+NS_IMETHODIMP
+nsMsgComposeService::Handle(nsICommandLine* aCmdLine)
+{
+  nsresult rv;
+  PRInt32 found, end, count;
+  nsAutoString uristr;
+
+  rv = aCmdLine->FindFlag(NS_LITERAL_STRING("compose"), PR_FALSE, &found);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+#ifndef MOZ_SUITE
+  // MAC OS X passes in -url mailto:mscott@mozilla.org into the command line
+  // instead of -compose.
+  if (found == -1)
+    rv = aCmdLine->FindFlag(NS_LITERAL_STRING("url"), PR_FALSE, &found);
+#endif
+
+  if (found == -1)
+    return NS_OK;
+
+  end = found;
+
+  rv = aCmdLine->GetLength(&count);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (count > found + 1) {
+    aCmdLine->GetArgument(found + 1, uristr);
+    if (StringBeginsWith(uristr, NS_LITERAL_STRING("mailto:"))  ||
+	StringBeginsWith(uristr, NS_LITERAL_STRING("to="))  ||
+	StringBeginsWith(uristr, NS_LITERAL_STRING("cc="))  ||
+	StringBeginsWith(uristr, NS_LITERAL_STRING("subject="))  ||
+	StringBeginsWith(uristr, NS_LITERAL_STRING("body="))  ||
+	StringBeginsWith(uristr, NS_LITERAL_STRING("attachment="))) {
+      end++;
+      // mailto: URIs are frequently passed with spaces in them. They should be
+      // escaped with %20, but we hack around broken clients. See bug 231032.
+      while (end + 1 < count) {
+        nsAutoString curarg;
+        aCmdLine->GetArgument(end + 1, curarg);
+        if (curarg.First() == '-')
+          break;
+
+        uristr.Append(' ');
+        uristr.Append(curarg);
+        ++end;
+      }
+    }
+    else {
+      uristr.Truncate();
+    }
+  }
+
+  aCmdLine->RemoveArguments(found, end);
+
+  nsCOMPtr<nsIWindowWatcher> wwatch (do_GetService(NS_WINDOWWATCHER_CONTRACTID));
+  NS_ENSURE_TRUE(wwatch, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsISupportsString> arg;
+  if (!uristr.IsEmpty()) {
+    arg = do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID);
+    if (arg)
+      arg->SetData(uristr);
+  }    
+
+  nsCOMPtr<nsIDOMWindow> opened;
+  wwatch->OpenWindow(nsnull, DEFAULT_CHROME, "_blank",
+                     "chrome,dialog=no,all", arg, getter_AddRefs(opened));
+
+  aCmdLine->SetPreventDefault(PR_TRUE);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgComposeService::GetHelpInfo(nsACString& aResult)
+{
+  aResult.Assign(NS_LITERAL_CSTRING("  -compose             Compose a mail or news message.\n"));
+  return NS_OK;
+}
+
+#else
 CMDLINEHANDLER_IMPL(nsMsgComposeService, "-compose", "general.startup.messengercompose", DEFAULT_CHROME,
                     "Start with messenger compose.", NS_MSGCOMPOSESTARTUPHANDLER_CONTRACTID, "Messenger Compose Startup Handler",
                     PR_TRUE, "about:blank", PR_TRUE)
-
+#endif
