@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -41,17 +41,20 @@
 #include "nsDBFolderInfo.h"
 #include "nsMsgLocalFolderHdrs.h"
 #include "nsFileStream.h"
-#include "nsLocalFolderSummarySpec.h"
 #include "nsFileSpec.h"
 #include "nsMsgOfflineImapOperation.h"
 #include "nsMsgFolderFlags.h"
 #include "prlog.h"
 #include "prprf.h"
 #include "nsIFileSpec.h"
+#include "nsMsgUtils.h"
 #ifdef PUTUP_ALERT_ON_INVALID_DB
 #include "nsIPrompt.h"
 #include "nsIWindowWatcher.h"
 #endif
+
+extern PRLogModuleInfo *IMAPOffline;
+
 const char *kOfflineOpsScope = "ns:msg:db:row:scope:ops:all";	// scope for all offine ops table
 const char *kOfflineOpsTableKind = "ns:msg:db:table:kind:ops";
 struct mdbOid gAllOfflineOpsTableOID;
@@ -76,15 +79,27 @@ NS_IMETHODIMP nsMailDatabase::SetFolderStream(nsIOFileStream *aFileStream)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsMailDatabase::GetFolderStream(nsIOFileStream **aFileStream)
+{
+  NS_ENSURE_ARG_POINTER(aFileStream);
+  if (!m_folderStream)
+  {
+    m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
+    m_ownFolderStream = PR_TRUE;
+  }
+  // N.B. - not a ref-counted interface pointer
+  *aFileStream = m_folderStream;
+  return NS_OK;
+}
+
 static PRBool gGotGlobalPrefs = PR_FALSE;
-static PRBool gThreadWithoutRe = PR_TRUE;
 static PRInt32 gTimeStampLeeway;
 
 void nsMailDatabase::GetGlobalPrefs()
 {
   if (!gGotGlobalPrefs)
   {
-    GetBoolPref("mail.thread_without_re", &gThreadWithoutRe);
+    nsMsgDatabase::GetGlobalPrefs();
     GetIntPref("mail.db_timestamp_leeway", &gTimeStampLeeway);
     gGotGlobalPrefs = PR_TRUE;
   }
@@ -121,7 +136,7 @@ nsresult nsMailDatabase::GetAllOfflineOpsTable()
 // cache m_folderStream to make updating mozilla status flags fast
 NS_IMETHODIMP nsMailDatabase::StartBatch()
 {
-  if (!m_folderStream)  //only if we create a stream, set m_ownFolderStream to true.
+  if (!m_folderStream && m_folder)  //only if we create a stream, set m_ownFolderStream to true.
   {
     PRBool isLocked;
     m_folder->GetLocked(&isLocked);
@@ -151,7 +166,6 @@ NS_IMETHODIMP nsMailDatabase::EndBatch()
     m_ownFolderStream = PR_FALSE;
   }
   SetSummaryValid(PR_TRUE);
-  Commit(nsMsgDBCommitType::kLargeCommit);
   return NS_OK;
 }
 
@@ -485,12 +499,14 @@ NS_IMETHODIMP nsMailDatabase::SetSummaryValid(PRBool valid)
       
       m_dbFolderInfo->SetFolderSize(m_folderSpec->GetFileSize());
       m_dbFolderInfo->SetFolderDate(actualFolderTimeStamp);
+      m_dbFolderInfo->SetVersion(GetCurVersion());
     }
     else
     {
       m_dbFolderInfo->SetVersion(0);	// that ought to do the trick.
     }
   }
+  Commit(nsMsgDBCommitType::kLargeCommit);
   return ret;
 }
 
@@ -522,6 +538,8 @@ NS_IMETHODIMP nsMailDatabase::GetOfflineOpForKey(nsMsgKey msgKey, PRBool create,
   mdbOid		rowObjectId;
   mdb_err   err;
   
+  if (!IMAPOffline)
+    IMAPOffline = PR_NewLogModule("IMAPOFFLINE");
   nsresult rv = GetAllOfflineOpsTable();
   NS_ENSURE_SUCCESS(rv, rv);
   
@@ -535,14 +553,14 @@ NS_IMETHODIMP nsMailDatabase::GetOfflineOpForKey(nsMsgKey msgKey, PRBool create,
   err = m_mdbAllOfflineOpsTable->HasOid(GetEnv(), &rowObjectId, &hasOid);
   if (err == NS_OK && m_mdbStore && (hasOid  || create))
   {
-    nsIMdbRow *offlineOpRow;
-    err = m_mdbStore->GetRow(GetEnv(), &rowObjectId, &offlineOpRow);
+    nsCOMPtr <nsIMdbRow> offlineOpRow;
+    err = m_mdbStore->GetRow(GetEnv(), &rowObjectId, getter_AddRefs(offlineOpRow));
     
     if (create)
     {
       if (!offlineOpRow)
       {
-        err  = m_mdbStore->NewRowWithOid(GetEnv(), &rowObjectId, &offlineOpRow);
+        err  = m_mdbStore->NewRowWithOid(GetEnv(), &rowObjectId, getter_AddRefs(offlineOpRow));
         NS_ENSURE_SUCCESS(err, err);
       }
       if (offlineOpRow && !hasOid)
@@ -554,37 +572,19 @@ NS_IMETHODIMP nsMailDatabase::GetOfflineOpForKey(nsMsgKey msgKey, PRBool create,
       *offlineOp = new nsMsgOfflineImapOperation(this, offlineOpRow);
       if (*offlineOp)
         (*offlineOp)->SetMessageKey(msgKey);
-      nsCOMPtr <nsIMsgDBHdr> msgHdr;
-
-      // GetMsgHdrForKey will return an empty header if no header exists,
-      // so we only want to get the hdr if the db contains a hdr for
-      // that key - otherwise, we'll crunch the new flags.
-      PRBool containsKey;
-      rv = ContainsKey(msgKey , &containsKey);
-
-      if (NS_SUCCEEDED(rv) && containsKey)
-        GetMsgHdrForKey(msgKey, getter_AddRefs(msgHdr));
-      if (msgHdr)
-      {
-        imapMessageFlagsType imapFlags = kNoImapMsgFlag;
-        PRUint32 msgHdrFlags;
-        msgHdr->GetFlags(&msgHdrFlags);
-        if (msgHdrFlags & MSG_FLAG_READ)
-          imapFlags |= kImapMsgSeenFlag;
-        if (msgHdrFlags & MSG_FLAG_REPLIED)
-          imapFlags |= kImapMsgAnsweredFlag;
-        if (msgHdrFlags & MSG_FLAG_MARKED)
-          imapFlags |= kImapMsgFlaggedFlag;
-        if (msgHdrFlags & MSG_FLAG_FORWARDED)
-          imapFlags |= kImapMsgForwardedFlag;
-        if (msgHdrFlags & MSG_FLAG_IMAP_DELETED)
-          imapFlags |= kImapMsgDeletedFlag;
-        (*offlineOp)->SetNewFlags(imapFlags);
-      }
       NS_IF_ADDREF(*offlineOp);
     }
     if (!hasOid && m_dbFolderInfo)
     {
+      // set initial value for flags so we don't lose them.
+      nsCOMPtr <nsIMsgDBHdr> msgHdr;
+      GetMsgHdrForKey(msgKey, getter_AddRefs(msgHdr));
+      if (msgHdr)
+      {
+        PRUint32 flags;
+        msgHdr->GetFlags(&flags);
+        (*offlineOp)->SetNewFlags(flags);
+      }
       PRInt32 newFlags;
       m_dbFolderInfo->OrFlags(MSG_FOLDER_FLAG_OFFLINEEVENTS, &newFlags);
     }
@@ -607,6 +607,9 @@ NS_IMETHODIMP nsMailDatabase::ListAllOfflineOpIds(nsMsgKeyArray *offlineOpIds)
   nsresult rv = GetAllOfflineOpsTable();
   NS_ENSURE_SUCCESS(rv, rv);
   nsIMdbTableRowCursor *rowCursor;
+  if (!IMAPOffline)
+    IMAPOffline = PR_NewLogModule("IMAPOFFLINE");
+
   if (m_mdbAllOfflineOpsTable)
   {
     nsresult err = m_mdbAllOfflineOpsTable->GetTableRowCursor(GetEnv(), -1, &rowCursor);
@@ -620,7 +623,22 @@ NS_IMETHODIMP nsMailDatabase::ListAllOfflineOpIds(nsMsgKeyArray *offlineOpIds)
       if (outPos < 0 || outOid.mOid_Id == (mdb_id) -1)	
         break;
       if (err == NS_OK)
+      {
         offlineOpIds->Add(outOid.mOid_Id);
+        if (PR_LOG_TEST(IMAPOffline, PR_LOG_ALWAYS))
+        {
+          nsCOMPtr <nsIMsgOfflineImapOperation> offlineOp;
+          GetOfflineOpForKey(outOid.mOid_Id, PR_FALSE, getter_AddRefs(offlineOp));
+          if (offlineOp)
+          {
+            nsMsgOfflineImapOperation *logOp = NS_STATIC_CAST(nsMsgOfflineImapOperation *, 
+              NS_STATIC_CAST(nsIMsgOfflineImapOperation *, offlineOp.get()));
+            if (logOp)
+              logOp->Log(IMAPOffline);
+
+          }
+        }
+      }
     }
     rv = (err == NS_OK) ? NS_OK : NS_ERROR_FAILURE;
     rowCursor->Release();
@@ -681,10 +699,10 @@ NS_IMETHODIMP nsMailDatabase::ListAllOfflineDeletes(nsMsgKeyArray *offlineDelete
 /* static */
 nsresult nsMailDatabase::SetFolderInfoValid(nsFileSpec *folderName, int num, int numunread)
 {
-  nsLocalFolderSummarySpec	summarySpec(*folderName);
-  nsFileSpec					summaryPath(summarySpec);
-  nsresult		err = NS_OK;
-  PRBool			bOpenedDB = PR_FALSE;
+  nsresult err = NS_OK;
+  PRBool bOpenedDB = PR_FALSE;
+  nsFileSpec summaryPath;
+  GetSummaryFileLocation(*folderName, &summaryPath);
   
   if (!folderName->Exists())
     return NS_MSG_ERROR_FOLDER_SUMMARY_MISSING;
@@ -697,14 +715,14 @@ nsresult nsMailDatabase::SetFolderInfoValid(nsFileSpec *folderName, int num, int
     if(!pMessageDB)
       return NS_ERROR_OUT_OF_MEMORY;
     
-    pMessageDB->m_folderSpec = new nsLocalFolderSummarySpec();
+    pMessageDB->m_folderSpec = new nsFileSpec();
     if(!pMessageDB->m_folderSpec)
     {
       delete pMessageDB;
       return NS_ERROR_OUT_OF_MEMORY;
     }
     
-    *(pMessageDB->m_folderSpec) = summarySpec;
+    *(pMessageDB->m_folderSpec) = summaryPath;
     // ### this does later stuff (marks latered messages unread), which may be a problem
     err = pMessageDB->OpenMDB(summaryPath, PR_FALSE);
     if (err != NS_OK)
@@ -754,14 +772,6 @@ void nsMailDatabase::SetReparse(PRBool reparse)
 }
 
 
-// should we thread messages with common subjects that don't start with Re: together?
-// I imagine we might have separate preferences for mail and news, so this is a virtual method.
-PRBool	nsMailDatabase::ThreadBySubjectWithoutRe()
-{
-  GetGlobalPrefs();
-  return gThreadWithoutRe;
-}
-
 class nsMsgOfflineOpEnumerator : public nsISimpleEnumerator {
 public:
   NS_DECL_ISUPPORTS
