@@ -42,7 +42,7 @@
 #include "nsCOMPtr.h"
 #include "MailNewsTypes2.h"
 
-NS_IMPL_ISUPPORTS1(nsMsgThread, nsMsgThread)
+NS_IMPL_ISUPPORTS1(nsMsgThread, nsIMsgThread)
 
 
 nsMsgThread::nsMsgThread()
@@ -228,7 +228,7 @@ nsresult nsMsgThread::RerootThread(nsIMsgDBHdr *newParentOfOldRoot, nsIMsgDBHdr
   ReparentNonReferenceChildrenOf(oldRoot, newRoot, announcer);
   if (ancestorHdr)
   {
-    nsIMsgHdr *msgHdr = ancestorHdr;
+    nsIMsgDBHdr *msgHdr = ancestorHdr;
     nsMsgHdr* rootMsgHdr = NS_STATIC_CAST(nsMsgHdr*, msgHdr);          // closed system, cast ok
     nsIMdbRow *newRootHdrRow = rootMsgHdr->GetMDBRow();
     // move the  root hdr to pos 0.
@@ -290,54 +290,68 @@ NS_IMETHODIMP nsMsgThread::AddChild(nsIMsgDBHdr *child, nsIMsgDBHdr *inReplyTo,
   
   PRBool hdrMoved = PR_FALSE;
   nsCOMPtr <nsIMsgDBHdr> curHdr;
-  for (childIndex = 0; childIndex < numChildren; childIndex++)
+
+  // This is an ugly but simple fix for a difficult problem. Basically, when we add
+  // a message to a thread, we have to run through the thread to see if the new
+  // message is a parent of an existing message in the thread, and adjust things
+  // accordingly. If you thread by subject, and you have a large folder with
+  // messages w/ all the same subject, this code can take a really long time. So the
+  // pragmatic thing is to say that for threads with more than 1000 messages, it's
+  // simply not worth dealing with the case where the parent comes in after the
+  // child. Threads with more than 1000 messages are pretty unwieldy anyway.
+  // See Bug 90452
+
+  if (numChildren < 1000)
   {
-    nsMsgKey msgKey;
-    
-    ret = GetChildHdrAt(childIndex, getter_AddRefs(curHdr));
-    if (NS_SUCCEEDED(ret) && curHdr)
+    for (childIndex = 0; childIndex < numChildren; childIndex++)
     {
-      if (hdr->IsParentOf(curHdr))
+      nsMsgKey msgKey;
+    
+      ret = GetChildHdrAt(childIndex, getter_AddRefs(curHdr));
+      if (NS_SUCCEEDED(ret) && curHdr)
       {
-        nsMsgKey oldThreadParent;
-        mdb_pos outPos;
-        // move this hdr before the current header.
-        if (!hdrMoved)
+        if (hdr->IsParentOf(curHdr))
         {
-          m_mdbTable->MoveRow(m_mdbDB->GetEnv(), hdrRow, -1, childIndex, &outPos);
-          hdrMoved = PR_TRUE;
-          curHdr->GetThreadParent(&oldThreadParent);
-          curHdr->GetMessageKey(&msgKey);
-          nsCOMPtr <nsIMsgDBHdr> curParent;
-          m_mdbDB->GetMsgHdrForKey(oldThreadParent, getter_AddRefs(curParent));
-          if (curParent && hdr->IsAncestorOf(curParent))
+          nsMsgKey oldThreadParent;
+          mdb_pos outPos;
+          // move this hdr before the current header.
+          if (!hdrMoved)
           {
-            nsMsgKey curParentKey;
-            curParent->GetMessageKey(&curParentKey);
-            if (curParentKey == m_threadRootKey)
+            m_mdbTable->MoveRow(m_mdbDB->GetEnv(), hdrRow, -1, childIndex, &outPos);
+            hdrMoved = PR_TRUE;
+            curHdr->GetThreadParent(&oldThreadParent);
+            curHdr->GetMessageKey(&msgKey);
+            nsCOMPtr <nsIMsgDBHdr> curParent;
+            m_mdbDB->GetMsgHdrForKey(oldThreadParent, getter_AddRefs(curParent));
+            if (curParent && hdr->IsAncestorOf(curParent))
             {
-              m_mdbTable->MoveRow(m_mdbDB->GetEnv(), hdrRow, -1, 0, &outPos);
-              RerootThread(child, curParent, announcer);
+              nsMsgKey curParentKey;
+              curParent->GetMessageKey(&curParentKey);
+              if (curParentKey == m_threadRootKey)
+              {
+                m_mdbTable->MoveRow(m_mdbDB->GetEnv(), hdrRow, -1, 0, &outPos);
+                RerootThread(child, curParent, announcer);
+                parentKeyNeedsSetting = PR_FALSE;
+              }
+            }
+            else if (msgKey == m_threadRootKey)
+            {
+              RerootThread(child, curHdr, announcer);
               parentKeyNeedsSetting = PR_FALSE;
             }
           }
-          else if (msgKey == m_threadRootKey)
-          {
-            RerootThread(child, curHdr, announcer);
+          curHdr->SetThreadParent(newHdrKey);
+          if (msgKey == newHdrKey)
             parentKeyNeedsSetting = PR_FALSE;
-          }
-        }
-        curHdr->SetThreadParent(newHdrKey);
-        if (msgKey == newHdrKey)
-          parentKeyNeedsSetting = PR_FALSE;
         
-        // OK, this is a reparenting - need to send notification
-        if (announcer)
-          announcer->NotifyParentChangedAll(msgKey, oldThreadParent, newHdrKey, nsnull);
-#ifdef DEBUG_bienvenu1
-        if (newHdrKey != m_threadKey)
-          printf("adding second level child\n");
-#endif
+          // OK, this is a reparenting - need to send notification
+          if (announcer)
+            announcer->NotifyParentChangedAll(msgKey, oldThreadParent, newHdrKey, nsnull);
+  #ifdef DEBUG_bienvenu1
+          if (newHdrKey != m_threadKey)
+            printf("adding second level child\n");
+  #endif
+        }
       }
     }
   }
@@ -407,7 +421,6 @@ nsresult nsMsgThread::ReparentNonReferenceChildrenOf(nsIMsgDBHdr *oldTopLevelHdr
     if (NS_SUCCEEDED(ret) && curHdr)
     {
       nsMsgKey oldThreadParent, curHdrKey;
-      nsIMsgDBHdr *curMsgHdr = curHdr;
       nsMsgHdr* oldTopLevelMsgHdr = NS_STATIC_CAST(nsMsgHdr*, oldTopLevelHdr);      // closed system, cast ok
       curHdr->GetThreadParent(&oldThreadParent);
       curHdr->GetMessageKey(&curHdrKey);
@@ -1068,12 +1081,15 @@ NS_IMETHODIMP nsMsgThread::GetFirstUnreadChild(nsIMsgDBHdr **result)
   NS_ENSURE_ARG(result);
   PRUint32 numChildren;
   nsresult rv = NS_OK;
-  
+  PRUint8 minLevel = 0xff;
+
   GetNumChildren(&numChildren);
   
   if ((PRInt32) numChildren < 0)
     numChildren = 0;
-  
+
+  nsCOMPtr <nsIMsgDBHdr> retHdr;
+
   for (PRUint32 childIndex = 0; childIndex < numChildren; childIndex++)
   {
     nsCOMPtr <nsIMsgDBHdr> child;
@@ -1087,13 +1103,36 @@ NS_IMETHODIMP nsMsgThread::GetFirstUnreadChild(nsIMsgDBHdr **result)
       rv = m_mdbDB->IsRead(msgKey, &isRead);
       if (NS_SUCCEEDED(rv) && !isRead)
       {
-        *result = child;
-        NS_ADDREF(*result);
-        break;
+        // this is the root, so it's the best we're going to do.
+        if (msgKey == m_threadRootKey)
+        {
+          retHdr = child;
+          break;
+        }
+        PRUint8 level = 0;
+        nsMsgKey parentId;
+        child->GetThreadParent(&parentId);
+        nsCOMPtr <nsIMsgDBHdr> parent;
+        // count number of ancestors - that's our level
+        while (parentId != nsMsgKey_None)
+        {
+          rv = m_mdbDB->GetMsgHdrForKey(parentId, getter_AddRefs(parent));
+          if (parent)
+          {
+            parent->GetThreadParent(&parentId);
+            level++;
+          }
+        }
+        if (level < minLevel)
+        {
+          minLevel = level;
+          retHdr = child;
+        }
       }
     }
   }
   
+  NS_IF_ADDREF(*result = retHdr);
   return rv;
 }
 
