@@ -85,7 +85,6 @@
 static PRLogModuleInfo *BayesianFilterLogModule = nsnull;
 
 static NS_DEFINE_CID(kParserCID, NS_PARSER_CID);
-static NS_DEFINE_CID(kNavDTDCID, NS_CNAVDTD_CID);
 
 #define kDefaultJunkThreshold .99 // we override this value via a pref
 static const char* kBayesianFilterTokenDelimiters = " \t\n\r\f.";
@@ -274,7 +273,7 @@ static PRBool isDecimalNumber(const char* word)
     if (*p == '-') ++p;
     char c;
     while ((c = *p++)) {
-        if (!isdigit(c))
+        if (!isdigit((unsigned char) c))
             return PR_FALSE;
     }
     return PR_TRUE;
@@ -381,7 +380,6 @@ void Tokenizer::tokenizeHeaders(nsIUTF8StringEnumerator * aHeaderNames, nsIUTF8S
         // This does not compile on linux yet. Need to figure out why. Commenting out for now
         // if (FindInReadable(FORGED_RECEIVED_HEADER_HINT, headerValue))
         //   addTokenForHeader(headerName.get(), FORGED_RECEIVED_HEADER_HINT);
-        // for now 
       }
       
       // leave out reply-to
@@ -443,6 +441,102 @@ void Tokenizer::tokenize_ascii_word(char * aWord)
   } 
 }
 
+// one substract and one conditional jump should be faster than two conditional jump on most recent system.
+#define IN_RANGE(x, low, high)  ((PRUint16)((x)-(low)) <= (high)-(low))
+
+#define IS_JA_HIRAGANA(x)   IN_RANGE(x, 0x3040, 0x309F)
+// swapping the range using xor operation to reduce conditional jump.
+#define IS_JA_KATAKANA(x)	(IN_RANGE(x^0x0004, 0x30A0, 0x30FE)||(IN_RANGE(x, 0xFF66, 0xFF9F)))
+#define IS_JA_KANJI(x)      (IN_RANGE(x, 0x2E80, 0x2FDF)||IN_RANGE(x, 0x4E00, 0x9FAF))
+#define IS_JA_KUTEN(x)      (((x)==0x3001)||((x)==0xFF64)||((x)==0xFF0E))
+#define IS_JA_TOUTEN(x)     (((x)==0x3002)||((x)==0xFF61)||((x)==0xFF0C))
+#define IS_JA_SPACE(x)      ((x)==0x3000)
+#define IS_JA_FWLATAIN(x)   IN_RANGE(x, 0xFF01, 0xFF5E)
+#define IS_JA_FWNUMERAL(x)  IN_RANGE(x, 0xFF10, 0xFF19)
+
+#define IS_JAPANESE_SPECIFIC(x) (IN_RANGE(x, 0x3040, 0x30FF)||IN_RANGE(x, 0xFF01, 0xFF9F))
+
+enum char_class{
+    others = 0,
+    space,
+    hiragana,
+    katakana,
+    kanji,
+    kuten,
+    touten,
+    kigou,
+    fwlatain,
+    ascii
+};
+
+char_class getCharClass(PRUnichar c)
+{
+  char_class charClass = others;
+
+  if(IS_JA_HIRAGANA(c))
+    charClass = hiragana;
+  else if(IS_JA_KATAKANA(c))
+    charClass = katakana;
+  else if(IS_JA_KANJI(c))
+    charClass = kanji;
+  else if(IS_JA_KUTEN(c))
+    charClass = kuten;
+  else if(IS_JA_TOUTEN(c))
+    charClass = touten;
+  else if(IS_JA_FWLATAIN(c))
+    charClass = fwlatain;
+
+  return charClass;
+}
+
+static PRBool isJapanese(const char* word)
+{
+  nsString text = NS_ConvertUTF8toUTF16(word);
+  PRUnichar* p = (PRUnichar*)text.get();
+  PRUnichar c;
+    
+  // it is japanese chunk if it contains any hiragana or katakana.
+  while((c = *p++))
+    if( IS_JAPANESE_SPECIFIC(c)) 
+      return PR_TRUE;
+
+  return PR_FALSE;
+}
+
+PRBool isFWNumeral(const PRUnichar* p1, const PRUnichar* p2)
+{
+  for(;p1<p2;p1++)
+    if(!IS_JA_FWNUMERAL(*p1)) 
+      return PR_FALSE;
+
+  return PR_TRUE;
+}
+
+// The japanese tokenizer was added as part of Bug #277354
+void Tokenizer::tokenize_japanese_word(char* chunk)
+{
+  PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("entering tokenize_japanese_word(%s)", chunk));
+    
+  nsString srcStr = NS_ConvertUTF8toUTF16(chunk);
+  const PRUnichar* p1 = srcStr.get();
+  const PRUnichar* p2 = p1;
+  if(!*p2) return;
+  
+  char_class cc = getCharClass(*p2);
+  while(*(++p2))
+  {
+    if(cc == getCharClass(*p2)) 
+      continue;
+   
+    nsCString token = NS_ConvertUTF16toUTF8(p1, p2-p1);
+    if( (!isDecimalNumber(token.get())) && (!isFWNumeral(p1, p2)))      
+      add(PromiseFlatCString(NS_LITERAL_CSTRING("JA:") + token).get());
+        
+    cc = getCharClass(*p2);
+    p1 = p2;
+  }
+}
+
 nsresult Tokenizer::stripHTML(const nsAString& inString, nsAString& outString)
 {
   nsresult rv = NS_OK;
@@ -464,12 +558,8 @@ nsresult Tokenizer::stripHTML(const nsAString& inString, nsAString& outString)
   textSink->Initialize(&outString, flags, 80);
 
   parser->SetContentSink(sink);
-  nsCOMPtr<nsIDTD> dtd = do_CreateInstance(kNavDTDCID,&rv);
-  NS_ENSURE_SUCCESS(rv, rv);
 
-  parser->RegisterDTD(dtd);
-
-  return parser->Parse(inString, 0, NS_LITERAL_CSTRING("text/html"), PR_FALSE, PR_TRUE);
+  return parser->Parse(inString, 0, NS_LITERAL_CSTRING("text/html"), PR_TRUE);
 }
 
 void Tokenizer::tokenize(char* aText)
@@ -480,11 +570,21 @@ void Tokenizer::tokenize(char* aText)
     // uggh but first we have to blow up our string into UCS2
     // since that's what the document encoder wants. UTF8/UCS2, I wish we all
     // spoke the same language here..
-    nsString text = NS_ConvertUTF8toUCS2(aText);
+    nsString text = NS_ConvertUTF8toUTF16(aText);
     nsString strippedUCS2;
     stripHTML(text, strippedUCS2);
-
-    nsCString strippedStr = NS_ConvertUCS2toUTF8(strippedUCS2);
+    
+    // convert 0x3000(full width space) into 0x0020
+    nsString::iterator substr_start, substr_end;
+    strippedUCS2.BeginWriting(substr_start);
+    strippedUCS2.EndWriting(substr_end);
+    while (substr_start != substr_end) {
+        if (*substr_start == 0x3000)
+            *substr_start = 0x0020;
+        ++substr_start;
+    }
+    
+    nsCString strippedStr = NS_ConvertUTF16toUTF8(strippedUCS2);
     char * strippedText = (char *) strippedStr.get(); // bleh
     PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("tokenize stripped html: %s", strippedText));
 
@@ -495,6 +595,8 @@ void Tokenizer::tokenize(char* aText)
         if (isDecimalNumber(word)) continue;
         if (isASCII(word))
             tokenize_ascii_word(word);
+        else if (isJapanese(word))
+            tokenize_japanese_word(word);
         else {
             nsresult rv;
             // use I18N  scanner to break this word into meaningful semantic units.
@@ -508,7 +610,7 @@ void Tokenizer::tokenize(char* aText)
             if (mScanner) {
                 mScanner->Start("UTF-8");
                 // convert this word from UTF-8 into UCS2.
-                NS_ConvertUTF8toUCS2 uword(word);
+                NS_ConvertUTF8toUTF16 uword(word);
                 ToLowerCase(uword);
                 const PRUnichar* utext = uword.get();
                 PRInt32 len = uword.Length(), pos = 0, begin, end;
@@ -516,7 +618,7 @@ void Tokenizer::tokenize(char* aText)
                 while (pos < len) {
                     rv = mScanner->Next(utext, len, pos, PR_TRUE, &begin, &end, &gotUnit);
                     if (NS_SUCCEEDED(rv) && gotUnit) {
-                        NS_ConvertUCS2toUTF8 utfUnit(utext + begin, end - begin);
+                        NS_ConvertUTF16toUTF8 utfUnit(utext + begin, end - begin);
                         add(utfUnit.get());
                         // advance to end of current unit.
                         pos = end;
@@ -638,9 +740,9 @@ NS_IMETHODIMP TokenStreamListener::ProcessHeaders(nsIUTF8StringEnumerator *aHead
     return NS_OK;
 }
 
-NS_IMETHODIMP TokenStreamListener::HandleAttachment(const char *contentType, const char *url, const PRUnichar *displayName, const char *uri, PRBool aNotDownloaded)
+NS_IMETHODIMP TokenStreamListener::HandleAttachment(const char *contentType, const char *url, const PRUnichar *displayName, const char *uri, PRBool aIsExternalAttachment)
 {
-    mTokenizer.tokenizeAttachment(contentType, NS_ConvertUCS2toUTF8(displayName).get());
+    mTokenizer.tokenizeAttachment(contentType, NS_ConvertUTF16toUTF8(displayName).get());
     return NS_OK;
 }
 
@@ -675,6 +777,16 @@ NS_IMETHODIMP TokenStreamListener::SetSecurityInfo(nsISupports * aSecurityInfo)
     return NS_OK;
 }
 
+NS_IMETHODIMP TokenStreamListener::GetDummyMsgHeader(nsIMsgDBHdr **aMsgDBHdr)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP TokenStreamListener::GetProperties(nsIWritablePropertyBag2 * *aProperties)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 /* void onStartRequest (in nsIRequest aRequest, in nsISupports aContext); */
 NS_IMETHODIMP TokenStreamListener::OnStartRequest(nsIRequest *aRequest, nsISupports *aContext)
 {
@@ -790,12 +902,11 @@ NS_IMETHODIMP TokenStreamListener::OnStopRequest(nsIRequest *aRequest, nsISuppor
 NS_IMPL_ISUPPORTS2(nsBayesianFilter, nsIMsgFilterPlugin, nsIJunkMailPlugin)
 
 nsBayesianFilter::nsBayesianFilter()
-    :   mGoodCount(0), mBadCount(0),
-        mBatchLevel(0), mTrainingDataDirty(PR_FALSE)
+    :   mGoodCount(0), mBadCount(0), mTrainingDataDirty(PR_FALSE)
 {
     if (!BayesianFilterLogModule)
       BayesianFilterLogModule = PR_NewLogModule("BayesianFilter");
-
+    
     PRInt32 junkThreshold = 0;
     nsresult rv;
     nsCOMPtr<nsIPrefBranch> pPrefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
@@ -806,7 +917,9 @@ nsBayesianFilter::nsBayesianFilter()
     if (mJunkProbabilityThreshold == 0 || mJunkProbabilityThreshold >= 1)
       mJunkProbabilityThreshold = kDefaultJunkThreshold;
 
-    PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("junk probabilty threshold: %f", mJunkProbabilityThreshold));
+    PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("junk probability threshold: %f", mJunkProbabilityThreshold));
+
+    getTrainingFile(getter_AddRefs(mTrainingFile));
 
     PRBool ok = (mGoodTokens && mBadTokens);
     NS_ASSERTION(ok, "error allocating tokenizers");
@@ -815,13 +928,49 @@ nsBayesianFilter::nsBayesianFilter()
     else {
       PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("error allocating tokenizers"));
     }
+    
+    // get parameters for training data flushing, from the prefs
+
+    nsCOMPtr<nsIPrefBranch> prefBranch;
+    
+    nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+    NS_ASSERTION(NS_SUCCEEDED(rv),"failed accessing preferences service");
+    rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+    NS_ASSERTION(NS_SUCCEEDED(rv),"failed getting preferences branch");
+
+    rv = prefBranch->GetIntPref("mailnews.bayesian_spam_filter.flush.minimum_interval",&mMinFlushInterval);
+    // it is not a good idea to allow a minimum interval of under 1 second
+    if (NS_FAILED(rv) || (mMinFlushInterval <= 1000) )
+        mMinFlushInterval = DEFAULT_MIN_INTERVAL_BETWEEN_WRITES;
+
+    mTimer = do_CreateInstance(NS_TIMER_CONTRACTID, &rv);
+    NS_ASSERTION(NS_SUCCEEDED(rv), "unable to create a timer; training data will only be written on exit");
+    
+    // the timer is not used on object construction, since for
+    // the time being there are no dirying messages
+    
 }
 
-nsBayesianFilter::~nsBayesianFilter() 
+void
+nsBayesianFilter::TimerCallback(nsITimer* aTimer, void* aClosure)
 {
-  // call shutdown when we are going away in case we need
-  // to flush the training set to disk
-  Shutdown();
+    // we will flush the training data to disk after enough time has passed
+    // since the first time a message has been classified after the last flush
+
+    nsBayesianFilter *filter = NS_STATIC_CAST(nsBayesianFilter *, aClosure);
+    filter->writeTrainingData();
+}
+
+nsBayesianFilter::~nsBayesianFilter()
+{
+    if (mTimer)
+    {
+        mTimer->Cancel();
+        mTimer = nsnull;
+    }
+    // call shutdown when we are going away in case we need
+    // to flush the training set to disk
+    Shutdown();
 }
 
 // this object is used for one call to classifyMessage or classifyMessages(). 
@@ -1062,26 +1211,6 @@ NS_IMETHODIMP nsBayesianFilter::GetShouldDownloadAllHeaders(PRBool *aShouldDownl
     return NS_OK;
 }
 
-NS_IMETHODIMP nsBayesianFilter::StartBatch(void)
-{
-  PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("StartBatch() entered with mBatchLevel=%d", mBatchLevel));
-  ++mBatchLevel;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsBayesianFilter::EndBatch(void)
-{
-    PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("EndBatch() entered with mBatchLevel=%d", mBatchLevel));
-    NS_ASSERTION(mBatchLevel > 0, "nsBayesianFilter::EndBatch() called with"
-                 " mBatchLevel <= 0");
-    --mBatchLevel;
-    
-    if (!mBatchLevel && mTrainingDataDirty)
-        writeTrainingData();
-    
-    return NS_OK;
-}
-
 /* void classifyMessage (in string aMsgURL, in nsIJunkMailClassificationListener aListener); */
 NS_IMETHODIMP nsBayesianFilter::ClassifyMessage(const char *aMessageURL, nsIMsgWindow *aMsgWindow, nsIJunkMailClassificationListener *aListener)
 {
@@ -1156,6 +1285,8 @@ void nsBayesianFilter::observeMessage(Tokenizer& tokenizer, const char* messageU
                                       nsIJunkMailClassificationListener* listener)
 {
     PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("observeMessage(%s) old=%d new=%d", messageURL, oldClassification, newClassification));
+
+    PRBool trainingDataWasDirty = mTrainingDataDirty;
     TokenEnumeration tokens = tokenizer.getTokens();
 
     // Uhoh...if the user is re-training then the message may already be classified and we are classifying it again with the same classification.
@@ -1205,23 +1336,16 @@ void nsBayesianFilter::observeMessage(Tokenizer& tokenizer, const char* messageU
     
     if (listener)
         listener->OnMessageClassified(messageURL, newClassification);
-
-    if (mTrainingDataDirty && !mBatchLevel)
-        writeTrainingData();
-}
-
-static nsresult getTrainingFile(nsCOMPtr<nsILocalFile>& file)
-{
-    // should we cache the profile manager's directory?
-    nsCOMPtr<nsIFile> profileDir;
-
-    nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(profileDir));
-    NS_ENSURE_SUCCESS(rv, rv);
-    rv = profileDir->Append(NS_LITERAL_STRING("training.dat"));
-    if (NS_FAILED(rv)) return rv;
     
-    file = do_QueryInterface(profileDir, &rv);
-    return rv;
+    if (mTrainingDataDirty && !trainingDataWasDirty && ( mTimer != nsnull ))
+    {
+        // if training data became dirty just now, schedule flush
+        // mMinFlushInterval msec from now
+        PR_LOG(
+            BayesianFilterLogModule, PR_LOG_ALWAYS,
+            ("starting training data flush timer %i msec", mMinFlushInterval));
+        mTimer->InitWithFuncCallback(nsBayesianFilter::TimerCallback, this, mMinFlushInterval, nsITimer::TYPE_ONE_SHOT);
+    }
 }
 
 /*
@@ -1294,8 +1418,11 @@ static PRBool readTokens(FILE* stream, Tokenizer& tokenizer)
         if (size >= bufferSize) {
             delete[] buffer;
             PRUint32 newBufferSize = 2 * bufferSize;
-            while (size >= newBufferSize)
+            while (size >= newBufferSize) {
                 newBufferSize *= 2;
+                if(newBufferSize == 0) // see bug #240788
+                    return PR_FALSE;
+            }
             buffer = new char[newBufferSize];
             if (!buffer) return PR_FALSE;
             bufferSize = newBufferSize;
@@ -1311,63 +1438,80 @@ static PRBool readTokens(FILE* stream, Tokenizer& tokenizer)
     return PR_TRUE;
 }
 
+
+nsresult nsBayesianFilter::getTrainingFile(nsILocalFile ** aTrainingFile)
+{
+  // should we cache the profile manager's directory?
+  nsCOMPtr<nsIFile> profileDir;
+
+  nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(profileDir));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = profileDir->Append(NS_LITERAL_STRING("training.dat"));
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  return profileDir->QueryInterface(NS_GET_IID(nsILocalFile), (void **) aTrainingFile);
+}
+
 static const char kMagicCookie[] = { '\xFE', '\xED', '\xFA', '\xCE' };
 
 void nsBayesianFilter::writeTrainingData()
 {
-    PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("writeTrainingData() entered"));
-    nsCOMPtr<nsILocalFile> file;
-    nsresult rv = getTrainingFile(file);
-    if (NS_FAILED(rv)) return;
- 
-    // open the file, and write out training data using fprintf for now.
-    FILE* stream;
-    rv = file->OpenANSIFileDesc("wb", &stream);
-    if (NS_FAILED(rv)) return;
+  PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("writeTrainingData() entered"));
+  if (!mTrainingFile) 
+    return;
 
-    if (!((fwrite(kMagicCookie, sizeof(kMagicCookie), 1, stream) == 1) &&
-          (writeUInt32(stream, mGoodCount) == 1) &&
-          (writeUInt32(stream, mBadCount) == 1) &&
-           writeTokens(stream, mGoodTokens) &&
-           writeTokens(stream, mBadTokens))) {
-        NS_WARNING("failed to write training data.");
-        fclose(stream);
-        // delete the training data file, since it is potentially corrupt.
-        file->Remove(PR_FALSE);
-    } else {
-        fclose(stream);
-        mTrainingDataDirty = PR_FALSE;
-    }
+  // open the file, and write out training data
+  FILE* stream;
+  nsresult rv = mTrainingFile->OpenANSIFileDesc("wb", &stream);
+  if (NS_FAILED(rv)) 
+    return;
+
+  if (!((fwrite(kMagicCookie, sizeof(kMagicCookie), 1, stream) == 1) &&
+        (writeUInt32(stream, mGoodCount) == 1) &&
+        (writeUInt32(stream, mBadCount) == 1) &&
+         writeTokens(stream, mGoodTokens) &&
+         writeTokens(stream, mBadTokens))) 
+  {
+    NS_WARNING("failed to write training data.");
+    fclose(stream);
+    // delete the training data file, since it is potentially corrupt.
+    mTrainingFile->Remove(PR_FALSE);
+  } 
+  else 
+  {
+    fclose(stream);
+    mTrainingDataDirty = PR_FALSE;
+  }
 }
 
 void nsBayesianFilter::readTrainingData()
 {
-    nsCOMPtr<nsILocalFile> file;
-    nsresult rv = getTrainingFile(file);
-    if (NS_FAILED(rv)) return;
-    
-    PRBool exists;
-    rv = file->Exists(&exists);
-    if (NS_FAILED(rv) || !exists) return;
+  if (!mTrainingFile) 
+    return;
+  
+  PRBool exists;
+  nsresult rv = mTrainingFile->Exists(&exists);
+  if (NS_FAILED(rv) || !exists) 
+    return;
 
-    // open the file, and write out training data using fprintf for now.
-    FILE* stream;
-    rv = file->OpenANSIFileDesc("rb", &stream);
-    if (NS_FAILED(rv)) return;
+  FILE* stream;
+  rv = mTrainingFile->OpenANSIFileDesc("rb", &stream);
+  if (NS_FAILED(rv)) 
+    return;
 
-    // FIXME:  should make sure that the tokenizers are empty.
-    char cookie[4];
-    if (!((fread(cookie, sizeof(cookie), 1, stream) == 1) &&
-          (memcmp(cookie, kMagicCookie, sizeof(cookie)) == 0) &&
-          (readUInt32(stream, &mGoodCount) == 1) &&
-          (readUInt32(stream, &mBadCount) == 1) &&
-           readTokens(stream, mGoodTokens) &&
-           readTokens(stream, mBadTokens))) {
-        NS_WARNING("failed to read training data.");
-        PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("failed to read training data."));
-    }
-    
-    fclose(stream);
+  // FIXME:  should make sure that the tokenizers are empty.
+  char cookie[4];
+  if (!((fread(cookie, sizeof(cookie), 1, stream) == 1) &&
+        (memcmp(cookie, kMagicCookie, sizeof(cookie)) == 0) &&
+        (readUInt32(stream, &mGoodCount) == 1) &&
+        (readUInt32(stream, &mBadCount) == 1) &&
+         readTokens(stream, mGoodTokens) &&
+         readTokens(stream, mBadTokens))) {
+      NS_WARNING("failed to read training data.");
+      PR_LOG(BayesianFilterLogModule, PR_LOG_ALWAYS, ("failed to read training data."));
+  }
+  
+  fclose(stream);
 }
 
 NS_IMETHODIMP nsBayesianFilter::GetUserHasClassified(PRBool *aResult)
@@ -1407,10 +1551,9 @@ NS_IMETHODIMP nsBayesianFilter::ResetTrainingData()
   }
 
   // now remove training.dat
-  nsCOMPtr<nsILocalFile> file;
-  nsresult rv = getTrainingFile(file);
-  if (file)
-    file->Remove(PR_FALSE);
+  if (mTrainingFile)
+    mTrainingFile->Remove(PR_FALSE);
 
   return NS_OK;
 }
+
