@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *       Rajiv Dayal <rdayal@netscape.com>
+ *       Mark Banner <mark@standard8.demon.co.uk>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -44,9 +45,11 @@
 #include "nsIRDFService.h"
 #include "nsRDFCID.h"
 #include "nsUnicharUtils.h"
-#include "nsAbMDBCard.h"
+#include "nsIAbMDBCard.h"
+#include "nsIAddressBook.h"
 #include "nsAbCardProperty.h"
 #include "prdtoa.h"
+#include "nsMsgUtils.h"
 
 #define  PREVIOUS_EXTENSION ".prev"
 #define  kPABDirectory  2 // defined in nsDirPrefs.h
@@ -160,7 +163,7 @@ void nsAbPalmHotSync::ConvertAssignPalmIDAttrib(PRUint32 id, nsIAbMDBCard * card
     char buf[128];
     PR_cnvtf(buf, 128, 0, f);
     card->SetAbDatabase(mABDB);
-    card->SetStringAttribute(CARD_ATTRIB_PALMID,NS_ConvertASCIItoUCS2(buf).get());
+    card->SetStringAttribute(CARD_ATTRIB_PALMID,NS_ConvertASCIItoUTF16(buf).get());
 }
 
 nsresult nsAbPalmHotSync::GetABInterface()
@@ -279,14 +282,13 @@ nsresult nsAbPalmHotSync::AddAllRecordsToAB(PRBool existingAB, PRInt32 aCount, l
       rv = OpenABDBForHotSync(PR_FALSE);
       NS_ENSURE_SUCCESS(rv, rv);
       // lets try deleting the db out from under ourselves. 
-      nsFileSpec *abFileSpec;
-      rv = mABDB->GetDbPath(&abFileSpec);
+      nsCOMPtr<nsIFile> abFile;
+      rv = mABDB->GetDbPath(getter_AddRefs(abFile));
       NS_ENSURE_SUCCESS(rv, rv);
       mABDB->ForceClosed();
       mDBOpen = PR_FALSE;
       mABDB = nsnull;
-      abFileSpec->Delete(PR_FALSE);
-      delete abFileSpec;
+      abFile->Remove(PR_FALSE);
     }
     else
     {
@@ -312,7 +314,7 @@ nsresult nsAbPalmHotSync::AddAllRecordsToAB(PRBool existingAB, PRInt32 aCount, l
     if(NS_SUCCEEDED(rv)) {
         mDBOpen = PR_FALSE;
         PRUint32 modTimeInSec;
-        nsAddrDatabase::PRTime2Seconds(PR_Now(), &modTimeInSec);
+        PRTime2Seconds(PR_Now(), &modTimeInSec);
         rv = UpdateABInfo(modTimeInSec, mPalmCategoryIndex);
     }
     else { // get back the previous file
@@ -364,7 +366,7 @@ nsresult nsAbPalmHotSync::GetAllCards(PRInt32 * aCount, lpnsABCOMCardStruct * aC
         return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    nsCOMPtr<nsIEnumerator> cardsEnumerator;
+    nsCOMPtr<nsISimpleEnumerator> cardsEnumerator;
     rv = mABDB->EnumerateCards(mDirectory, getter_AddRefs(cardsEnumerator));
     if (NS_FAILED(rv) || !cardsEnumerator) {
         mABDB->Close(PR_FALSE);
@@ -373,8 +375,9 @@ nsresult nsAbPalmHotSync::GetAllCards(PRInt32 * aCount, lpnsABCOMCardStruct * aC
 
     nsCOMPtr<nsISupports> item;
     nsCOMPtr<nsIAbCard> card;
-    for (rv = cardsEnumerator->First(); NS_SUCCEEDED(rv); rv = cardsEnumerator->Next()) {
-        rv = cardsEnumerator->CurrentItem(getter_AddRefs(item));
+    PRBool more;
+    while (NS_SUCCEEDED(cardsEnumerator->HasMoreElements(&more)) && more) {
+        rv = cardsEnumerator->GetNext(getter_AddRefs(item));
         if (NS_FAILED(rv)) 
             continue;
 
@@ -471,7 +474,7 @@ nsresult nsAbPalmHotSync::DoSyncAndGetUpdatedCards(PRInt32 aPalmCount, lpnsABCOM
         {
             mDBOpen = PR_FALSE;
             PRUint32 modTimeInSec;
-            nsAddrDatabase::PRTime2Seconds(PR_Now(), &modTimeInSec);
+            PRTime2Seconds(PR_Now(), &modTimeInSec);
             rv = UpdateABInfo(modTimeInSec, mPalmCategoryIndex);
         }
         else
@@ -560,7 +563,7 @@ nsresult nsAbPalmHotSync::LoadNewModifiedCardsSinceLastSync()
     if(!mDBOpen || !mABDB || !mInitialized) 
         return NS_ERROR_NOT_INITIALIZED;
 
-    nsCOMPtr<nsIEnumerator> cardsEnumerator;
+    nsCOMPtr<nsISimpleEnumerator> cardsEnumerator;
     nsresult rv = mABDB->EnumerateCards(mDirectory, getter_AddRefs(cardsEnumerator));
     if (NS_FAILED(rv) || !cardsEnumerator) 
         return NS_ERROR_NOT_AVAILABLE; // no cards available
@@ -568,9 +571,10 @@ nsresult nsAbPalmHotSync::LoadNewModifiedCardsSinceLastSync()
     // create the list of cards to be sent to Palm
     nsCOMPtr<nsISupports> item;
     nsCOMPtr<nsIAbCard> card;
-    for (rv = cardsEnumerator->First(); NS_SUCCEEDED(rv); rv = cardsEnumerator->Next()) 
+    PRBool more;
+    while (NS_SUCCEEDED(cardsEnumerator->HasMoreElements(&more)) && more) 
     {
-        rv = cardsEnumerator->CurrentItem(getter_AddRefs(item));
+        rv = cardsEnumerator->GetNext(getter_AddRefs(item));
         if (NS_FAILED(rv)) 
             return rv;
 
@@ -730,31 +734,20 @@ nsresult nsAbPalmHotSync::OpenABDBForHotSync(PRBool aCreate)
     if(NS_FAILED(rv)) 
         return rv;
 
-    nsFileSpec* dbPath;
-    rv = abSession->GetUserProfileDirectory(&dbPath);
+    rv = abSession->GetUserProfileDirectory(getter_AddRefs(mABFile));
     if(NS_FAILED(rv)) 
         return rv;
 
-    (*dbPath) += mFileName.get();
-
-    // get nsIFile for nsFileSpec from abSession, why use a obsolete class if not required!
-    rv = NS_FileSpecToIFile(dbPath, getter_AddRefs(mABFile));
-    if(NS_FAILED(rv))  
-    {
-        delete dbPath;
-        return rv;
-    }
+		mABFile->AppendNative(mFileName);
 
     nsCOMPtr<nsIAddrDatabase> addrDBFactory = 
              do_GetService(NS_ADDRDATABASE_CONTRACTID, &rv);
     if(NS_FAILED(rv)) 
     {
-        delete dbPath;
         return rv;
     }
     
-    rv = addrDBFactory->Open(dbPath, aCreate, getter_AddRefs(mABDB), PR_TRUE);
-    delete dbPath;
+    rv = addrDBFactory->Open(mABFile, aCreate, PR_TRUE, getter_AddRefs(mABDB));
     NS_ENSURE_SUCCESS(rv, rv);
     mDBOpen = PR_TRUE;  // Moz AB DB is now Open
 
@@ -776,14 +769,10 @@ nsresult nsAbPalmHotSync::KeepCurrentStateAsPrevious()
         nsCOMPtr<nsIAddrBookSession> abSession = do_GetService(NS_ADDRBOOKSESSION_CONTRACTID, &rv);
         if(NS_FAILED(rv)) 
             return rv;
-        nsFileSpec* dbPath;
-        rv = abSession->GetUserProfileDirectory(&dbPath); // this still uses nsFileSpec!!!
+        rv = abSession->GetUserProfileDirectory(getter_AddRefs(mPreviousABFile));
         if(NS_SUCCEEDED(rv)) 
         {
-            (*dbPath) += previousLeafName.get();
-            // get nsIFile for nsFileSpec from abSession, why use a obsolete class if not required!
-            rv = NS_FileSpecToIFile(dbPath, getter_AddRefs(mPreviousABFile));
-            delete dbPath;
+            mPreviousABFile->AppendNative(previousLeafName);
             if(NS_FAILED(rv))
                 return rv;
         }
@@ -838,7 +827,7 @@ nsresult nsAbPalmHotSync::UpdateMozABWithPalmRecords()
           {
             nsCOMPtr<nsIAbMDBCard> dbCard = do_QueryInterface(existingCard);
 
-            dbCard->SetStringAttribute(CARD_ATTRIB_PALMID, NS_ConvertASCIItoUCS2(recordIDBuf).get());
+            dbCard->SetStringAttribute(CARD_ATTRIB_PALMID, NS_ConvertASCIItoUTF16(recordIDBuf).get());
             continue;
           }
 
@@ -888,14 +877,14 @@ nsresult nsAbPalmHotSync::UpdateMozABWithPalmRecords()
             || (ipcCard.GetStatus() == ATTR_NONE)) 
         {
             PRUint32 modTimeInSec;
-            nsAddrDatabase::PRTime2Seconds(PR_Now(), &modTimeInSec);
+            PRTime2Seconds(PR_Now(), &modTimeInSec);
             ipcCard.SetLastModifiedDate(modTimeInSec);
             rv = mABDB->CreateNewCardAndAddToDB(newCard, PR_FALSE);
             if(NS_SUCCEEDED(rv)) 
             {
                 // now set the attribute for the PalmRecID in the card in the DB
                 dbCard->SetAbDatabase(mABDB);
-                dbCard->SetStringAttribute(CARD_ATTRIB_PALMID, NS_ConvertASCIItoUCS2(recordIDBuf).get());
+                dbCard->SetStringAttribute(CARD_ATTRIB_PALMID, NS_ConvertASCIItoUTF16(recordIDBuf).get());
                 newCard = do_QueryInterface(dbCard, &rv);
                 if(NS_SUCCEEDED(rv))
                     rv = mABDB->EditCard(newCard, PR_FALSE);
@@ -940,7 +929,7 @@ nsresult nsAbPalmHotSync::Done(PRBool aSuccess, PRInt32 aPalmCatIndex, PRUint32
             {
                 mDBOpen = PR_FALSE;
                 PRUint32 modTimeInSec;
-                nsAddrDatabase::PRTime2Seconds(PR_Now(), &modTimeInSec);
+                PRTime2Seconds(PR_Now(), &modTimeInSec);
                 rv = UpdateABInfo(modTimeInSec, aPalmCatIndex);
             }
         }
@@ -976,7 +965,7 @@ nsresult nsAbPalmHotSync::UpdateSyncInfo(long aCategoryIndex)
   // aCategoryIndex = -1 means that callers want to reset the mod time as well. 
   mDBOpen = PR_FALSE;
   PRUint32 modTimeInSec;
-  nsAddrDatabase::PRTime2Seconds(PR_Now(), &modTimeInSec);
+  PRTime2Seconds(PR_Now(), &modTimeInSec);
   if (aCategoryIndex >= 0)
     return(UpdateABInfo(modTimeInSec, aCategoryIndex));
   else
@@ -1041,7 +1030,7 @@ nsresult nsAbPalmHotSync::RenameAB(long aCategoryIndex, const char * aABUrl)
   rv = properties->SetCategoryId(aCategoryIndex);
   NS_ENSURE_SUCCESS(rv,rv);
   PRUint32 modTimeInSec;
-  nsAddrDatabase::PRTime2Seconds(PR_Now(), &modTimeInSec);
+  PRTime2Seconds(PR_Now(), &modTimeInSec);
   rv = properties->SetSyncTimeStamp(modTimeInSec);
   NS_ENSURE_SUCCESS(rv,rv);
 
