@@ -42,11 +42,10 @@
 #include "nsMsgDBFolder.h"
 #include "nsIImapMailFolderSink.h"
 #include "nsIImapMessageSink.h"
-#include "nsIImapExtensionSink.h"
-#include "nsIImapMiscellaneousSink.h"
 #include "nsICopyMessageListener.h"
 #include "nsIImapService.h"
 #include "nsIUrlListener.h"
+#include "nsAutoPtr.h"
 #include "nsIImapIncomingServer.h" // we need this for its IID
 #include "nsIMsgParseMailMsgState.h"
 #include "nsITransactionManager.h"
@@ -56,12 +55,13 @@
 #include "nsIMsgFilterList.h"
 #include "nsIMsgFilterPlugin.h"
 #include "prmon.h"
-#include "nsIEventQueue.h"
 #include "nsIMsgImapMailFolder.h"
 #include "nsIMsgLocalMailFolder.h"
 #include "nsIImapMailFolderSink.h"
 #include "nsIImapServerSink.h"
 #include "nsIMsgFilterPlugin.h"
+#include "nsIEventTarget.h"
+#include "nsIThread.h"
 class nsImapMoveCoalescer;
 class nsHashtable;
 class nsHashKey;
@@ -76,7 +76,7 @@ class nsHashKey;
 class nsImapMailCopyState: public nsISupports
 {
 public:
-    NS_DEFINE_STATIC_IID_ACCESSOR(NS_IMAPMAILCOPYSTATE_IID)
+    NS_DECLARE_STATIC_IID_ACCESSOR(NS_IMAPMAILCOPYSTATE_IID)
     
     NS_DECL_ISUPPORTS
 
@@ -85,7 +85,7 @@ public:
 
     nsCOMPtr<nsISupports> m_srcSupport; // source file spec or folder
     nsCOMPtr<nsISupportsArray> m_messages; // array of source messages
-    nsCOMPtr<nsMsgTxn> m_undoMsgTxn; // undo object with this copy operation
+    nsRefPtr<nsMsgTxn> m_undoMsgTxn; // undo object with this copy operation
     nsCOMPtr<nsIMsgDBHdr> m_message; // current message to be copied
     nsCOMPtr<nsIMsgCopyServiceListener> m_listener; // listener of this copy
                                                     // operation 
@@ -103,12 +103,16 @@ public:
     PRUint32 m_unreadCount; // num unread messages we're moving
     PRBool m_streamCopy;
     char *m_dataBuffer; // temporary buffer for this copy operation
+    nsCOMPtr<nsIOutputStream> m_msgFileStream;         // temporary file (processed mail)
     PRUint32 m_dataBufferSize;
     PRUint32 m_leftOver;
     PRBool m_allowUndo;
     PRBool m_eatLF;
+    PRBool m_newMsgFlags; // only used if there's no m_message 
 };
 
+NS_DEFINE_STATIC_IID_ACCESSOR(nsImapMailCopyState, NS_IMAPMAILCOPYSTATE_IID)
+
 // ACLs for this folder.
 // Generally, we will try to always query this class when performing
 // an operation on the folder.
@@ -193,8 +197,6 @@ class nsImapMailFolder :  public nsMsgDBFolder,
                           public nsIMsgImapMailFolder,
                           public nsIImapMailFolderSink,
                           public nsIImapMessageSink,
-                          public nsIImapExtensionSink,
-                          public nsIImapMiscellaneousSink,
                           public nsICopyMessageListener,
                           public nsIMsgFilterHitNotify,
                           public nsIJunkMailClassificationListener
@@ -222,6 +224,9 @@ public:
   NS_IMETHOD Compact(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow);
   NS_IMETHOD CompactAll(nsIUrlListener *aListener, nsIMsgWindow *aMsgWindow, nsISupportsArray *aFolderArray, PRBool aCompactOfflineAlso, nsISupportsArray *aOfflineFolderArray);
   NS_IMETHOD EmptyTrash(nsIMsgWindow *msgWindow, nsIUrlListener *aListener);
+  NS_IMETHOD CopyDataToOutputStreamForAppend(nsIInputStream *aIStream,
+                     PRInt32 aLength, nsIOutputStream *outputStream);
+  NS_IMETHOD CopyDataDone();
   NS_IMETHOD Delete ();
   NS_IMETHOD Rename (const PRUnichar *newName, nsIMsgWindow *msgWindow);
   NS_IMETHOD RenameSubFolders(nsIMsgWindow *msgWindow, nsIMsgFolder *oldFolder);
@@ -247,7 +252,7 @@ public:
   NS_IMETHOD MarkMessagesFlagged(nsISupportsArray *messages, PRBool markFlagged);
   NS_IMETHOD MarkThreadRead(nsIMsgThread *thread);
   NS_IMETHOD SetLabelForMessages(nsISupportsArray *aMessages, nsMsgLabelValue aLabel);
-  
+  NS_IMETHOD SetJunkScoreForMessages(nsISupportsArray *aMessages, const char *aJunkScore);
   NS_IMETHOD DeleteSubFolders(nsISupportsArray *folders, nsIMsgWindow *msgWindow);
   NS_IMETHOD ReadFromFolderCacheElem(nsIMsgFolderCacheElement *element);
   NS_IMETHOD WriteToFolderCacheElem(nsIMsgFolderCacheElement *element);
@@ -268,6 +273,7 @@ public:
   NS_IMETHOD CopyFileMessage(nsIFileSpec* fileSpec, 
                               nsIMsgDBHdr* msgToReplace,
                               PRBool isDraftOrTemplate,
+                              PRUint32 aNewMsgFlags,
                               nsIMsgWindow *msgWindow,
                               nsIMsgCopyServiceListener* listener);
   NS_IMETHOD GetNewMessages(nsIMsgWindow *aWindow, nsIUrlListener *aListener);
@@ -282,6 +288,13 @@ public:
   NS_IMETHOD DownloadAllForOffline(nsIUrlListener *listener, nsIMsgWindow *msgWindow);
   NS_IMETHOD GetCanFileMessages(PRBool *aCanFileMessages);
   NS_IMETHOD GetCanDeleteMessages(PRBool *aCanDeleteMessages);
+  NS_IMETHOD FetchMsgPreviewText(nsMsgKey *aKeysToFetch, PRUint32 aNumKeys,
+                                                 PRBool aLocalOnly, nsIUrlListener *aUrlListener, 
+                                                 PRBool *aAsyncResults);
+
+  NS_IMETHOD AddKeywordToMessages(nsISupportsArray *aMessages, const char *aKeyword);
+  NS_IMETHOD RemoveKeywordFromMessages(nsISupportsArray *aMessages, const char *aKeyword);
+
   // nsIMsgImapMailFolder methods
   NS_DECL_NSIMSGIMAPMAILFOLDER
           
@@ -300,32 +313,9 @@ public:
   NS_IMETHOD OnStartRunningUrl(nsIURI * aUrl);
   NS_IMETHOD OnStopRunningUrl(nsIURI * aUrl, nsresult aExitCode);
   
-  // nsIImapExtensionSink methods
-  NS_IMETHOD ClearFolderRights(nsIImapProtocol* aProtocol,
-                                nsIMAPACLRightsInfo* aclRights);
-  
-  NS_IMETHOD SetCopyResponseUid(nsIImapProtocol* aProtocol,
-                                nsMsgKeyArray* keyArray,
-                                const char* msgIdString,
-                                nsIImapUrl * aUrl);
-  NS_IMETHOD SetAppendMsgUid(nsIImapProtocol* aProtocol,
-                              nsMsgKey aKey,
-                              nsIImapUrl * aUrl);
-  NS_IMETHOD GetMessageId(nsIImapProtocol* aProtocol,
-                          nsCString* messageId,
-                          nsIImapUrl * aUrl);
-  
   // nsIImapMiscellaneousSink methods
   NS_IMETHOD AddSearchResult(nsIImapProtocol* aProtocol, 
                               const char* searchHitLine);
-  NS_IMETHOD HeaderFetchCompleted(nsIImapProtocol* aProtocol);
-  // ****
-  NS_IMETHOD SetBiffStateAndUpdate(nsIImapProtocol* aProtocol,
-                                    nsMsgBiffState biffState);
-  NS_IMETHOD ProgressStatus(nsIImapProtocol* aProtocol,
-                            PRUint32 aMsgId, const PRUnichar *extraInfo);
-  NS_IMETHOD PercentProgress(nsIImapProtocol* aProtocol,
-                              ProgressInfo* aInfo);
   NS_IMETHOD MatchName(nsString *name, PRBool *matches);
   
   NS_DECL_NSIMSGFILTERHITNOTIFY
@@ -365,7 +355,7 @@ protected:
   
   nsresult SetupHeaderParseStream(PRUint32 size, const char *content_type, nsIMailboxSpec *boxSpec);
   nsresult  ParseAdoptedHeaderLine(const char *messageLine, PRUint32 msgKey);
-  nsresult  NormalEndHeaderParseStream(nsIImapProtocol *aProtocol);
+  nsresult  NormalEndHeaderParseStream(nsIImapProtocol *aProtocol, nsIImapUrl *imapUrl);
   
   void EndOfflineDownload();
 
@@ -413,6 +403,7 @@ protected:
                           PRBool isMove,
                           PRBool selectedState,
                           PRBool acrossServers,
+                          PRUint32 newMsgFlags,
                           nsIMsgCopyServiceListener* listener,
                           nsIMsgWindow *msgWindow,
                           PRBool allowUndo);
@@ -437,6 +428,7 @@ protected:
   nsresult CopyOfflineMsgBody(nsIMsgFolder *srcFolder, nsIMsgDBHdr *destHdr, nsIMsgDBHdr *origHdr);
   
   void GetTrashFolderName(nsAString &aFolderName);
+  PRBool ShowPreviewText();
   
   PRBool m_initialized;
   PRBool m_haveDiscoveredAllFolders;
@@ -454,12 +446,12 @@ protected:
   PRInt32 m_numStatusRecentMessages; // used to store counts from Status command
   PRInt32 m_numStatusUnseenMessages;
   PRInt32  m_nextMessageByteLength;
-  nsCOMPtr<nsIEventQueue> m_eventQueue;
+  nsCOMPtr<nsIThread> m_thread;
   nsCOMPtr<nsIUrlListener> m_urlListener;
   PRBool m_urlRunning;
   
   // *** jt - undo move/copy trasaction support
-  nsCOMPtr<nsMsgTxn> m_pendingUndoTxn;
+  nsRefPtr<nsMsgTxn> m_pendingUndoTxn;
   nsCOMPtr<nsImapMailCopyState> m_copyState;
   PRMonitor *m_appendMsgMonitor;
   PRUnichar m_hierarchyDelimiter;
