@@ -50,23 +50,12 @@ nsImapMoveCopyMsgTxn::nsImapMoveCopyMsgTxn() :
 {
 }
 
-nsImapMoveCopyMsgTxn::nsImapMoveCopyMsgTxn(
-	nsIMsgFolder* srcFolder, nsMsgKeyArray* srcKeyArray, 
-	const char* srcMsgIdString, nsIMsgFolder* dstFolder,
-	PRBool idsAreUids, PRBool isMove,
-	nsIEventQueue* eventQueue, nsIUrlListener* urlListener) :
-    m_idsAreUids(PR_FALSE), m_isMove(PR_FALSE), m_srcIsPop3(PR_FALSE)
-{
-    Init(srcFolder, srcKeyArray, srcMsgIdString, dstFolder, idsAreUids,
-         isMove, eventQueue, urlListener);
-}
-
 nsresult
 nsImapMoveCopyMsgTxn::Init(
 	nsIMsgFolder* srcFolder, nsMsgKeyArray* srcKeyArray, 
 	const char* srcMsgIdString, nsIMsgFolder* dstFolder,
 	PRBool idsAreUids, PRBool isMove,
-	nsIEventQueue* eventQueue, nsIUrlListener* urlListener)
+	nsIEventTarget* eventTarget, nsIUrlListener* urlListener)
 {
   nsresult rv;
   NS_NewISupportsArray(getter_AddRefs(m_srcHdrs));
@@ -75,7 +64,7 @@ nsImapMoveCopyMsgTxn::Init(
   m_isMove = isMove;
   m_srcFolder = do_GetWeakReference(srcFolder);
   m_dstFolder = do_GetWeakReference(dstFolder);
-  m_eventQueue = do_QueryInterface(eventQueue, &rv);
+  m_eventTarget = eventTarget;
   if (urlListener)
     m_urlListener = do_QueryInterface(urlListener, &rv);
   m_srcKeyArray.CopyArray(srcKeyArray);
@@ -123,7 +112,7 @@ nsImapMoveCopyMsgTxn::Init(
       }
     }
   }
-  return rv;
+  return nsMsgTxn::Init();
 }
 
 nsImapMoveCopyMsgTxn::~nsImapMoveCopyMsgTxn()
@@ -179,7 +168,7 @@ nsImapMoveCopyMsgTxn::UndoTransaction(void)
         return rv;
       // ** make sure we are in the selected state; use lite select
       // folder so we won't hit performance hard
-      rv = imapService->LiteSelectFolder(m_eventQueue, srcFolder,
+      rv = imapService->LiteSelectFolder(m_eventTarget, srcFolder,
         srcListener, nsnull);
       if (NS_FAILED(rv)) 
         return rv;
@@ -198,11 +187,11 @@ nsImapMoveCopyMsgTxn::UndoTransaction(void)
 
       if (deletedMsgs)
         rv = imapService->SubtractMessageFlags(
-             m_eventQueue, srcFolder, srcListener, nsnull,
+             m_eventTarget, srcFolder, srcListener, nsnull,
              m_srcMsgIdString.get(), kImapMsgDeletedFlag,
              m_idsAreUids);
       else
-        rv = imapService->AddMessageFlags(m_eventQueue, srcFolder,
+        rv = imapService->AddMessageFlags(m_eventTarget, srcFolder,
                                           srcListener, nsnull,
                                           m_srcMsgIdString.get(),
                                           kImapMsgDeletedFlag,
@@ -211,13 +200,13 @@ nsImapMoveCopyMsgTxn::UndoTransaction(void)
         return rv;
 
       if (deleteModel != nsMsgImapDeleteModels::IMAPDelete)
-        rv = imapService->GetHeaders(m_eventQueue, srcFolder,
+        rv = imapService->GetHeaders(m_eventTarget, srcFolder,
         srcListener, nsnull,
         m_srcMsgIdString.get(),
         PR_TRUE); 
     }
   }
-  if (m_dstKeyArray.GetSize() > 0)
+  if (!m_dstMsgIdString.IsEmpty())
   {
     nsCOMPtr<nsIMsgFolder> dstFolder = do_QueryReferent(m_dstFolder, &rv);
     if (NS_FAILED(rv) || !dstFolder) return rv;
@@ -228,10 +217,10 @@ nsImapMoveCopyMsgTxn::UndoTransaction(void)
     if (NS_FAILED(rv)) return rv;
     // ** make sure we are in the selected state; use lite select folder
     // so we won't potentially download a bunch of headers.
-    rv = imapService->LiteSelectFolder(m_eventQueue, dstFolder,
+    rv = imapService->LiteSelectFolder(m_eventTarget, dstFolder,
       dstListener, nsnull);
     if (NS_FAILED(rv)) return rv;
-    rv = imapService->AddMessageFlags(m_eventQueue, dstFolder,
+    rv = imapService->AddMessageFlags(m_eventTarget, dstFolder,
       dstListener, nsnull,
       m_dstMsgIdString.get(),
       kImapMsgDeletedFlag,
@@ -279,22 +268,22 @@ nsImapMoveCopyMsgTxn::RedoTransaction(void)
       
       // ** make sire we are in the selected state; use lite select
       // folder so we won't hit preformace hard
-      rv = imapService->LiteSelectFolder(m_eventQueue, srcFolder,
+      rv = imapService->LiteSelectFolder(m_eventTarget, srcFolder,
         srcListener, nsnull);
       if (NS_FAILED(rv)) 
         return rv;
       if (deletedMsgs)
-        rv = imapService->SubtractMessageFlags(m_eventQueue, srcFolder, 
+        rv = imapService->SubtractMessageFlags(m_eventTarget, srcFolder, 
                                                 srcListener, nsnull,
                                                 m_srcMsgIdString.get(), kImapMsgDeletedFlag,
                                                 m_idsAreUids);
       else
-        rv = imapService->AddMessageFlags(m_eventQueue, srcFolder,
+        rv = imapService->AddMessageFlags(m_eventTarget, srcFolder,
                                           srcListener, nsnull, m_srcMsgIdString.get(),
                                           kImapMsgDeletedFlag, m_idsAreUids);
     }
   }
-  if (m_dstKeyArray.GetSize() > 0)
+  if (!m_dstMsgIdString.IsEmpty())
   {
     nsCOMPtr<nsIMsgFolder> dstFolder = do_QueryReferent(m_dstFolder, &rv);
     if (NS_FAILED(rv) || !dstFolder) return rv;
@@ -306,11 +295,11 @@ nsImapMoveCopyMsgTxn::RedoTransaction(void)
       return rv;
     // ** make sure we are in the selected state; use lite select
     // folder so we won't hit preformace hard
-    rv = imapService->LiteSelectFolder(m_eventQueue, dstFolder,
+    rv = imapService->LiteSelectFolder(m_eventTarget, dstFolder,
       dstListener, nsnull);
     if (NS_FAILED(rv)) 
       return rv;
-    rv = imapService->SubtractMessageFlags(m_eventQueue, dstFolder,
+    rv = imapService->SubtractMessageFlags(m_eventTarget, dstFolder,
       dstListener, nsnull,
       m_dstMsgIdString.get(),
       kImapMsgDeletedFlag,
@@ -320,7 +309,7 @@ nsImapMoveCopyMsgTxn::RedoTransaction(void)
     nsMsgImapDeleteModel deleteModel;
     rv = GetImapDeleteModel(dstFolder, &deleteModel);
     if (NS_FAILED(rv) || deleteModel == nsMsgImapDeleteModels::MoveToTrash)
-      rv = imapService->GetHeaders(m_eventQueue, dstFolder,
+      rv = imapService->GetHeaders(m_eventTarget, dstFolder,
       dstListener, nsnull,
       m_dstMsgIdString.get(),
       PR_TRUE);
@@ -329,11 +318,9 @@ nsImapMoveCopyMsgTxn::RedoTransaction(void)
 }
 
 nsresult
-nsImapMoveCopyMsgTxn::SetCopyResponseUid(nsMsgKeyArray* aKeyArray,
-                                         const char* aMsgIdString)
+nsImapMoveCopyMsgTxn::SetCopyResponseUid(const char* aMsgIdString)
 {
-  if (!aKeyArray || !aMsgIdString) return NS_ERROR_NULL_POINTER;
-  m_dstKeyArray.CopyArray(aKeyArray);
+  if (!aMsgIdString) return NS_ERROR_NULL_POINTER;
   m_dstMsgIdString = aMsgIdString;
   if (m_dstMsgIdString.Last() == ']')
   {
@@ -350,17 +337,9 @@ nsImapMoveCopyMsgTxn::GetSrcKeyArray(nsMsgKeyArray& srcKeyArray)
     return NS_OK;
 }
 
-nsresult
-nsImapMoveCopyMsgTxn::GetDstKeyArray(nsMsgKeyArray& dstKeyArray)
-{
-    dstKeyArray.CopyArray(&m_dstKeyArray);
-    return NS_OK;
-}
-
 nsresult
 nsImapMoveCopyMsgTxn::AddDstKey(nsMsgKey aKey)
 {
-    m_dstKeyArray.Add(aKey);
     if (!m_dstMsgIdString.IsEmpty())
         m_dstMsgIdString.Append(",");
     m_dstMsgIdString.AppendInt((PRInt32) aKey);
@@ -408,7 +387,6 @@ nsImapMoveCopyMsgTxn::UndoMailboxDelete()
 		   }
         }
         srcDB->SetSummaryValid(PR_TRUE);
-        srcDB->Commit(nsMsgDBCommitType::kLargeCommit);
         return NS_OK; // always return NS_OK
     }
     else
@@ -433,7 +411,6 @@ nsImapMoveCopyMsgTxn::RedoMailboxDelete()
         {
             srcDB->DeleteMessages(&m_srcKeyArray, nsnull);
             srcDB->SetSummaryValid(PR_TRUE);
-            srcDB->Commit(nsMsgDBCommitType::kLargeCommit);
         }
         return NS_OK; // always return NS_OK
     }
@@ -461,10 +438,10 @@ nsresult nsImapMoveCopyMsgTxn::GetImapDeleteModel(nsIMsgFolder *aFolder, nsMsgIm
 nsImapOfflineTxn::nsImapOfflineTxn(nsIMsgFolder* srcFolder, nsMsgKeyArray* srcKeyArray, 
 	nsIMsgFolder* dstFolder, PRBool isMove, nsOfflineImapOperationType opType,
         nsIMsgDBHdr *srcHdr,
-	nsIEventQueue* eventQueue, nsIUrlListener* urlListener)
+	nsIEventTarget* eventTarget, nsIUrlListener* urlListener)
 {
   Init(srcFolder, srcKeyArray, nsnull, dstFolder, PR_TRUE,
-       isMove, eventQueue, urlListener);
+       isMove, eventTarget, urlListener);
 
   m_opType = opType; 
   m_flags = 0;
