@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   David Bienvenu <bienvenu@nventure.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -38,13 +39,10 @@
 #include "msgCore.h"    // precompiled header...
 #include "nsMsgImapCID.h"
 
-#include "nsIEventQueueService.h"
-
 #include "nsIURL.h"
 #include "nsImapUrl.h"
 #include "nsIMsgMailSession.h"
 #include "nsIIMAPHostSessionList.h"
-#include "nsIMAPGenericParser.h"
 #include "nsString.h"
 #include "prmem.h"
 #include "plstr.h"
@@ -101,8 +99,6 @@ nsImapUrl::nsImapUrl()
   m_fileSpec = nsnull;
   m_imapMailFolderSink = nsnull;
   m_imapMessageSink = nsnull;
-  m_imapExtensionSink = nsnull;
-  m_imapMiscellaneousSink = nsnull;
   m_addDummyEnvelope = PR_FALSE;
   m_canonicalLineEnding = PR_FALSE;
 }
@@ -148,9 +144,9 @@ NS_IMETHODIMP nsImapUrl::SetMsgWindow(nsIMsgWindow *aMsgWindow)
 
 nsImapUrl::~nsImapUrl()
 {
-	PR_FREEIF(m_listOfMessageIds);
-	PR_FREEIF(m_destinationCanonicalFolderPathSubString);
-	PR_FREEIF(m_sourceCanonicalFolderPathSubString);
+  PR_FREEIF(m_listOfMessageIds);
+  PR_FREEIF(m_destinationCanonicalFolderPathSubString);
+  PR_FREEIF(m_sourceCanonicalFolderPathSubString);
   PR_FREEIF(m_searchCriteriaString);
 }
   
@@ -276,44 +272,6 @@ NS_IMETHODIMP nsImapUrl::SetImapServerSink(nsIImapServerSink  * aImapServerSink)
   return rv;
 }
 
-NS_IMETHODIMP nsImapUrl::GetImapExtensionSink(nsIImapExtensionSink ** aImapExtensionSink)
-{
-    NS_ENSURE_ARG_POINTER(aImapExtensionSink);
-    NS_ENSURE_ARG_POINTER(m_imapExtensionSink);
-
-    nsCOMPtr<nsIImapExtensionSink> extensionSink = do_QueryReferent(m_imapExtensionSink);
-    *aImapExtensionSink = extensionSink;
-    NS_IF_ADDREF(*aImapExtensionSink);
-    return NS_OK;
-}
-
-NS_IMETHODIMP nsImapUrl::SetImapExtensionSink(nsIImapExtensionSink  * aImapExtensionSink)
-{
-  nsresult rv;
-  m_imapExtensionSink = do_GetWeakReference(aImapExtensionSink, &rv);
-  return rv;
-}
-
-NS_IMETHODIMP nsImapUrl::GetImapMiscellaneousSink(nsIImapMiscellaneousSink **
-                                              aImapMiscellaneousSink)
-{
-    NS_ENSURE_ARG_POINTER(aImapMiscellaneousSink);
-    NS_ENSURE_ARG_POINTER(m_imapMiscellaneousSink);
-
-    nsCOMPtr<nsIImapMiscellaneousSink> miscSink = do_QueryReferent(m_imapMiscellaneousSink);
-    *aImapMiscellaneousSink = miscSink;
-    NS_IF_ADDREF(*aImapMiscellaneousSink);
-    return NS_OK;
-}
-
-NS_IMETHODIMP nsImapUrl::SetImapMiscellaneousSink(nsIImapMiscellaneousSink  *
-                                              aImapMiscellaneousSink)
-{
-  nsresult rv;
-  m_imapMiscellaneousSink = do_GetWeakReference(aImapMiscellaneousSink, &rv);
-  return rv;
-}
-
         
 ////////////////////////////////////////////////////////////////////////////////////
 // End nsIImapUrl specific support
@@ -323,7 +281,10 @@ NS_IMETHODIMP nsImapUrl::SetSpec(const nsACString &aSpec)
 {
   nsresult rv = nsMsgMailNewsUrl::SetSpec(aSpec);
   if (NS_SUCCEEDED(rv))
+  {
+    m_validUrl = PR_TRUE;	// assume the best.
     rv = ParseUrl();
+  }
   return rv;
 }
 
@@ -451,7 +412,7 @@ NS_IMETHODIMP nsImapUrl::GetCustomSubtractFlags(char **aResult)
 
 NS_IMETHODIMP nsImapUrl::GetImapPartToFetch(char **result) 
 {
-	//  here's the old code....
+  //  here's the old code....
 
   // unforunately an imap part can have the form: /;section= OR
   // it can have the form ?section=. We need to look for both.
@@ -495,25 +456,24 @@ NS_IMETHODIMP nsImapUrl::GetOnlineSubDirSeparator(char* separator)
   }
 }
 
+NS_IMETHODIMP nsImapUrl::GetNumBytesToFetch(PRInt32 *aNumBytesToFetch)
+{
+  NS_ENSURE_ARG_POINTER(aNumBytesToFetch);
+  *aNumBytesToFetch = m_numBytesToFetch;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsImapUrl::SetOnlineSubDirSeparator(char onlineDirSeparator)
 {
-	m_onlineSubDirSeparator = onlineDirSeparator;
+  m_onlineSubDirSeparator = onlineDirSeparator;
   return NS_OK;
 }
 
 // this method is only called from the imap thread
 NS_IMETHODIMP nsImapUrl::MessageIdsAreUids(PRBool *result)
 {
-	*result = m_idsAreUids;
-  return NS_OK;
-}
-
-// this method is only called from the imap thread
-NS_IMETHODIMP 
-nsImapUrl::GetChildDiscoveryDepth(PRInt32* result)
-{
-  *result = m_discoveryDepth;
+  *result = m_idsAreUids;
   return NS_OK;
 }
 
@@ -527,7 +487,7 @@ NS_IMETHODIMP nsImapUrl::GetMsgFlags(imapMessageFlagsType *result)	// kAddMsgFla
 void nsImapUrl::ParseImapPart(char *imapPartOfUrl)
 {
   m_tokenPlaceHolder = imapPartOfUrl;
-  m_urlidSubString = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
+  m_urlidSubString = m_tokenPlaceHolder ? nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
   
   if (!m_urlidSubString)
   {
@@ -560,6 +520,13 @@ void nsImapUrl::ParseImapPart(char *imapPartOfUrl)
       ParseListOfMessageIds();
       ParseCustomMsgFetchAttribute();
     }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "previewBody"))
+    {
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+      ParseNumBytes();
+    }
     else if (!nsCRT::strcasecmp(m_urlidSubString, "deletemsg"))
     {
       m_imapAction = nsImapDeleteMsg;
@@ -691,12 +658,6 @@ void nsImapUrl::ParseImapPart(char *imapPartOfUrl)
       m_imapAction = nsImapDiscoverChildrenUrl;
       ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
     }
-    else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverlevelchildren"))
-    {
-      m_imapAction = nsImapDiscoverLevelChildrenUrl;
-      ParseChildDiscoveryDepth();
-      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-    }
     else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverallboxes"))
     {
       m_imapAction = nsImapDiscoverAllBoxesUrl;
@@ -809,9 +770,16 @@ void nsImapUrl::ParseImapPart(char *imapPartOfUrl)
       ParseUidChoice();
       ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
       ParseListOfMessageIds();
-    	char *flagsPtr = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)nsnull;
-      m_customAddFlags.Assign(flagsPtr);
-      flagsPtr = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)nsnull;
+      PRBool addKeyword = (m_tokenPlaceHolder && *m_tokenPlaceHolder != '>');
+      // if we're not adding a keyword, m_tokenPlaceHolder will now look like >keywordToSubtract>
+      // and strtok will leave flagsPtr pointing to keywordToSubtract. So detect this
+      // case and only set the customSubtractFlags.
+      char *flagsPtr = m_tokenPlaceHolder ? nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)nsnull;
+      if (addKeyword)
+      {
+        m_customAddFlags.Assign(flagsPtr);
+        flagsPtr = m_tokenPlaceHolder ? nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)nsnull;
+      }
       m_customSubtractFlags.Assign(flagsPtr);
     }
     else
@@ -1343,6 +1311,7 @@ NS_IMPL_GETSET(nsImapUrl, CanonicalLineEnding, PRBool, m_canonicalLineEnding)
 NS_IMPL_GETTER(nsImapUrl::GetMsgLoadingFromCache, PRBool, m_msgLoadingFromCache)
 NS_IMPL_GETSET(nsImapUrl, ExternalLinkUrl, PRBool, m_externalLinkUrl)
 NS_IMPL_GETSET(nsImapUrl, RerunningUrl, PRBool, m_rerunningUrl)
+NS_IMPL_GETSET(nsImapUrl, ValidUrl, PRBool, m_validUrl)
 
 NS_IMETHODIMP nsImapUrl::SetMsgLoadingFromCache(PRBool loadingFromCache)
 {
@@ -1432,7 +1401,7 @@ char *nsImapUrl::ReplaceCharsInCopiedString(const char *stringToCopy, char oldCh
 
 void nsImapUrl::ParseFolderPath(char **resultingCanonicalPath)
 {
-	char *resultPath = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
+	char *resultPath = m_tokenPlaceHolder ? nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
 	
 	if (!resultPath)
 	{
@@ -1501,21 +1470,9 @@ void nsImapUrl::ParseSearchCriteriaString()
 }
 
 
-void nsImapUrl::ParseChildDiscoveryDepth()
-{
-	char *discoveryDepth = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
-	if (!discoveryDepth)
-	{
-		m_validUrl = PR_FALSE;
-		m_discoveryDepth = 0;
-		return;
-	}
-	m_discoveryDepth = atoi(discoveryDepth);
-}
-
 void nsImapUrl::ParseUidChoice()
 {
-	char *uidChoiceString = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
+	char *uidChoiceString = m_tokenPlaceHolder ? nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
 	if (!uidChoiceString)
 		m_validUrl = PR_FALSE;
 	else
@@ -1524,7 +1481,7 @@ void nsImapUrl::ParseUidChoice()
 
 void nsImapUrl::ParseMsgFlags()
 {
-	char *flagsPtr = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
+	char *flagsPtr = m_tokenPlaceHolder ? nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
 	if (flagsPtr)
 	{
 		// the url is encodes the flags byte as ascii 
@@ -1537,7 +1494,7 @@ void nsImapUrl::ParseMsgFlags()
 
 void nsImapUrl::ParseListOfMessageIds()
 {
-  m_listOfMessageIds = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
+  m_listOfMessageIds = m_tokenPlaceHolder ? nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
   if (!m_listOfMessageIds)
     m_validUrl = PR_FALSE;
   else
@@ -1558,8 +1515,14 @@ void nsImapUrl::ParseListOfMessageIds()
 
 void nsImapUrl::ParseCustomMsgFetchAttribute()
 {
-  m_msgFetchAttribute = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)nsnull;
+  m_msgFetchAttribute = m_tokenPlaceHolder ? nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)nsnull;
 }
+
+void nsImapUrl::ParseNumBytes()
+{
+  m_numBytesToFetch = m_tokenPlaceHolder ? atoi(nsCRT::strtok(m_tokenPlaceHolder, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder)) : 0;
+}
+
 // nsIMsgI18NUrl support
 
 nsresult nsImapUrl::GetMsgFolder(nsIMsgFolder **msgFolder)
