@@ -50,6 +50,7 @@
 #include "nsIURI.h"
 #include "nsIProxyObjectManager.h"
 #include "nsProxiedService.h"
+#include "nsIOutputStream.h"
 
 #include "nsMsgBaseCID.h"
 #include "nsMsgCompCID.h"
@@ -68,14 +69,10 @@
 #include "EudoraDebugLog.h"
 
 #include "nsMimeTypes.h"
-#include "nsIPref.h"
 #include "nsMsgUtils.h"
 
 static NS_DEFINE_CID( kMsgSendCID, NS_MSGSEND_CID);
 static NS_DEFINE_CID( kMsgCompFieldsCID, NS_MSGCOMPFIELDS_CID); 
-static NS_DEFINE_CID( kIOServiceCID, NS_IOSERVICE_CID);
-static NS_DEFINE_CID( kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
-
 
 // We need to do some calculations to set these numbers to something reasonable!
 // Unless of course, CreateAndSendMessage will NEVER EVER leave us in the lurch
@@ -219,7 +216,7 @@ nsEudoraCompose::~nsEudoraCompose()
         NS_ASSERTION(NS_SUCCEEDED(rv),"failed to clear values");
 		if (NS_FAILED(rv)) return;
 
-		NS_WITH_PROXIED_SERVICE(nsIMsgAccountManager, accMgr, NS_MSGACCOUNTMANAGER_CONTRACTID, NS_UI_THREAD_EVENTQ, &rv);
+		NS_WITH_PROXIED_SERVICE(nsIMsgAccountManager, accMgr, NS_MSGACCOUNTMANAGER_CONTRACTID, NS_PROXY_TO_MAIN_THREAD, &rv);
         NS_ASSERTION(NS_SUCCEEDED(rv) && accMgr,"failed to get account manager");
 		if (NS_FAILED(rv) || !accMgr) return;
 
@@ -238,7 +235,7 @@ nsresult nsEudoraCompose::CreateIdentity( void)
 		return( NS_OK);
 
 	nsresult	rv;
-    NS_WITH_PROXIED_SERVICE(nsIMsgAccountManager, accMgr, NS_MSGACCOUNTMANAGER_CONTRACTID, NS_UI_THREAD_EVENTQ, &rv);
+    NS_WITH_PROXIED_SERVICE(nsIMsgAccountManager, accMgr, NS_MSGACCOUNTMANAGER_CONTRACTID, NS_PROXY_TO_MAIN_THREAD, &rv);
     if (NS_FAILED(rv)) return( rv);
 	rv = accMgr->CreateIdentity( &m_pIdentity);
 	nsString	name(NS_LITERAL_STRING("Import Identity"));
@@ -258,7 +255,7 @@ nsresult nsEudoraCompose::CreateComponents( void)
 	if (!m_pIOService) {
 		IMPORT_LOG0( "Creating nsIOService\n");
 
-		NS_WITH_PROXIED_SERVICE(nsIIOService, service, kIOServiceCID, NS_UI_THREAD_EVENTQ, &rv);
+		NS_WITH_PROXIED_SERVICE(nsIIOService, service, NS_IOSERVICE_CONTRACTID, NS_PROXY_TO_MAIN_THREAD, &rv);
 		if (NS_FAILED(rv)) 
 			return( rv);
 		m_pIOService = service;
@@ -269,16 +266,10 @@ nsresult nsEudoraCompose::CreateComponents( void)
 	if (!m_pMsgSend) {
 		rv = CallCreateInstance( kMsgSendCID, &m_pMsgSend); 
 		if (NS_SUCCEEDED( rv) && m_pMsgSend) {
-			nsCOMPtr<nsIProxyObjectManager> proxyMgr = 
-			         do_GetService(kProxyObjectManagerCID, &rv);
-			if (NS_SUCCEEDED(rv)) {
-				rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsIMsgSend),
-										m_pMsgSend, PROXY_SYNC, (void**)&m_pSendProxy);
-				if (NS_FAILED( rv)) {
-					m_pSendProxy = nsnull;
-				}
-			}
-			if (NS_FAILED( rv)) {
+      rv = NS_GetProxyForObject( NS_PROXY_TO_MAIN_THREAD, NS_GET_IID(nsIMsgSend),
+                  m_pMsgSend, NS_PROXY_SYNC, (void**)&m_pSendProxy);
+      if (NS_FAILED( rv)) {
+        m_pSendProxy = nsnull;
 				NS_RELEASE( m_pMsgSend);
 				m_pMsgSend = nsnull;
 			}
@@ -619,7 +610,7 @@ nsresult nsEudoraCompose::SendTheMessage( nsIFileSpec *pMsg)
 	ExtractCharset( headerVal);
   // Use platform charset as default if the msg doesn't specify one
   // (ie, no 'charset' param in the Content-Type: header). As the last
-  // resort we'll use the mail defaul charset.
+  // resort we'll use the mail default charset.
   if (headerVal.IsEmpty())
   {
     headerVal.AssignWithConversion(nsMsgI18NFileSystemCharset());
@@ -628,22 +619,21 @@ nsresult nsEudoraCompose::SendTheMessage( nsIFileSpec *pMsg)
       if (m_defCharset.IsEmpty())
       {
         nsXPIDLString defaultCharset;
-        nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
-        if (NS_SUCCEEDED(rv))
-          rv = prefs->GetLocalizedUnicharPref("mailnews.view_default_charset", getter_Copies(defaultCharset));
-        m_defCharset.Assign(defaultCharset ? defaultCharset.get() : NS_LITERAL_STRING("ISO-8859-1").get());
+        NS_GetLocalizedUnicharPreferenceWithDefault(nsnull, "mailnews.view_default_charset",
+                                                    NS_LITERAL_STRING("ISO-8859-1"), defaultCharset);
+        m_defCharset = defaultCharset;
       }
       headerVal = m_defCharset;
     }
   }
-  m_pMsgFields->SetCharacterSet( NS_LossyConvertUCS2toASCII(headerVal).get() );
+  m_pMsgFields->SetCharacterSet( NS_LossyConvertUTF16toASCII(headerVal).get() );
   charSet = headerVal;
 	GetHeaderValue( m_pHeaders, m_headerLen, "CC:", headerVal);
 	if (!headerVal.IsEmpty())
 		m_pMsgFields->SetCc( headerVal);
 	GetHeaderValue( m_pHeaders, m_headerLen, "Message-ID:", headerVal);
 	if (!headerVal.IsEmpty())
-		m_pMsgFields->SetMessageId( NS_LossyConvertUCS2toASCII(headerVal).get() );
+		m_pMsgFields->SetMessageId( NS_LossyConvertUTF16toASCII(headerVal).get() );
 	GetHeaderValue( m_pHeaders, m_headerLen, "Reply-To:", headerVal);
 	if (!headerVal.IsEmpty())
 		m_pMsgFields->SetReplyTo( headerVal);
@@ -719,7 +709,9 @@ nsresult nsEudoraCompose::SendTheMessage( nsIFileSpec *pMsg)
 										nsnull,                       // parent window
 										nsnull,                       // progress listener
 										m_pListener,		              // listener
-                    nsnull);                      // password
+										nsnull,                       // password
+										EmptyCString(),               // originalMsgURI
+                    nsnull);                      // message compose type
 
 	}
 	else {
@@ -741,7 +733,9 @@ nsresult nsEudoraCompose::SendTheMessage( nsIFileSpec *pMsg)
 										nsnull,                       // parent window
 										nsnull,                       // progress listener
 										m_pListener,		              // listener
-                    nsnull);                      // password
+										nsnull,                       // password
+										EmptyCString(),               // originalMsgURI
+                    nsnull);                      // message compose type
 
 	}
 
@@ -966,8 +960,10 @@ nsresult nsEudoraCompose::CopyComposedMessage( nsCString& fromLine, nsIFileSpec
 	// so that the following will properly copy the rest of the body
 	char	lastChar = 0;
 
+  nsCOMPtr<nsIOutputStream> outStream;
+  pDst->GetOutputStream (getter_AddRefs (outStream));
 	if (NS_SUCCEEDED( rv)) {
-    rv = EscapeFromSpaceLine(pDst, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
+    rv = EscapeFromSpaceLine(outStream, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
 		if (copy.m_bytesInBuf)
 			lastChar = copy.m_pBuffer[copy.m_bytesInBuf - 1];
     if (NS_SUCCEEDED(rv))
@@ -977,7 +973,7 @@ nsresult nsEudoraCompose::CopyComposedMessage( nsCString& fromLine, nsIFileSpec
 	while ((state.offset < state.size) && NS_SUCCEEDED( rv)) {
 		rv = FillMailBuffer( &state, copy);
 		if (NS_SUCCEEDED( rv)) {
-      rv = EscapeFromSpaceLine(pDst, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
+      rv = EscapeFromSpaceLine(outStream, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
 			lastChar = copy.m_pBuffer[copy.m_bytesInBuf - 1];
 			if (NS_SUCCEEDED( rv))
 		    copy.m_writeOffset = copy.m_bytesInBuf;
