@@ -40,9 +40,10 @@
 #include "nsCOMPtr.h"
 #include "nsReadableUtils.h"
 #include "nsEudoraMailbox.h"
-#include "nsSpecialSystemDirectory.h"
+#include "nsDirectoryServiceDefs.h"
 #include "nsEudoraCompose.h"
 #include "nspr.h"
+#include "nsMsgUtils.h"
 
 #include "EudoraDebugLog.h"
 
@@ -118,23 +119,18 @@ nsEudoraMailbox::~nsEudoraMailbox()
 
 nsresult nsEudoraMailbox::CreateTempFile( nsIFileSpec **ppSpec)
 {
-	*ppSpec = nsnull;
-	
-	nsSpecialSystemDirectory temp(nsSpecialSystemDirectory::OS_TemporaryDirectory);
-	
-	// nsSpecialSystemDirectory temp(nsSpecialSystemDirectory::Mac_DesktopDirectory);
-	
-    temp += "impmail.txt";
-	temp.MakeUnique();
-	nsresult rv = NS_NewFileSpecWithSpec( temp, ppSpec);
-    if (NS_SUCCEEDED(rv)) {
-		if (*ppSpec)
-			return( NS_OK);
-		else
-			return( NS_ERROR_FAILURE);
-	}
+  *ppSpec = nsnull;
 
-	return( rv);
+  nsCOMPtr<nsIFile> tmpFile;
+  nsresult rv = GetSpecialDirectoryWithFileName(NS_OS_TEMP_DIR,
+                                                "impmail.txt",
+                                                getter_AddRefs(tmpFile));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = tmpFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 00600);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_NewFileSpecFromIFile(tmpFile, ppSpec);
 }
 
 nsresult nsEudoraMailbox::DeleteFile( nsIFileSpec *pSpec)
@@ -166,6 +162,7 @@ nsresult nsEudoraMailbox::DeleteFile( nsIFileSpec *pSpec)
 
 
 #define kComposeErrorStr	"X-Eudora-Compose-Error: *****" "\x0D\x0A"
+#define kHTMLTag "<html>"
 
 nsresult nsEudoraMailbox::ImportMailbox( PRUint32 *pBytes, PRBool *pAbort, const PRUnichar *pName, nsIFileSpec *pSrc, nsIFileSpec *pDst, PRInt32 *pMsgCount)
 {
@@ -297,6 +294,12 @@ nsresult nsEudoraMailbox::ImportMailbox( PRUint32 *pBytes, PRBool *pAbort, const
 				*pBytes += (((body.m_writeOffset - 1 + headers.m_writeOffset - 1) / div) * mul);
 			}
 
+      // Unfortunately Eudora stores HTML messages in the sent folder
+      // without any content type header at all. If the first line of the message body is <html>
+      // then mark the message as html internally...See Bug #258489
+      if (body.m_pBuffer && (body.m_writeOffset > strlen(kHTMLTag)) && (strncmp(body.m_pBuffer, kHTMLTag, strlen(kHTMLTag)) == 0 ))
+        bodyType = "text/html"; // ignore whatever body type we were given...force html
+
       compose.SetBody( body.m_pBuffer, body.m_writeOffset - 1, bodyType);
 			compose.SetHeaders( headers.m_pBuffer, headers.m_writeOffset - 1);
 			compose.SetAttachments( &m_attachments);
@@ -1054,12 +1057,14 @@ void nsEudoraMailbox::EmptyAttachments( void)
 
 static char *eudoraAttachLines[] = {
 	"Attachment Converted:",
-	"Attachment converted:"
+	"Attachment converted:",
+  "Pice jointe convertie :"
 };
 
 static PRInt32 eudoraAttachLen[] = {
 	21,
 	21,
+  24,
 	0
 };
 
