@@ -52,6 +52,7 @@
 #include "nsProxiedService.h"
 #include "nsMsgI18N.h"
 #include "nsNativeCharsetUtils.h"
+#include "nsIOutputStream.h"
 
 #include "nsMsgBaseCID.h"
 #include "nsMsgCompCID.h"
@@ -69,14 +70,10 @@
 #include "OutlookDebugLog.h"
 
 #include "nsMimeTypes.h"
-#include "nsIPref.h"
 #include "nsMsgUtils.h"
 
 static NS_DEFINE_CID( kMsgSendCID, NS_MSGSEND_CID);
 static NS_DEFINE_CID( kMsgCompFieldsCID, NS_MSGCOMPFIELDS_CID); 
-static NS_DEFINE_CID( kIOServiceCID, NS_IOSERVICE_CID);
-static NS_DEFINE_CID( kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
-
 
 // We need to do some calculations to set these numbers to something reasonable!
 // Unless of course, CreateAndSendMessage will NEVER EVER leave us in the lurch
@@ -213,7 +210,7 @@ nsOutlookCompose::~nsOutlookCompose()
         NS_ASSERTION(NS_SUCCEEDED(rv),"failed to clear values");
 		if (NS_FAILED(rv)) return;
 
-		NS_WITH_PROXIED_SERVICE(nsIMsgAccountManager, accMgr, NS_MSGACCOUNTMANAGER_CONTRACTID, NS_UI_THREAD_EVENTQ, &rv);
+		NS_WITH_PROXIED_SERVICE(nsIMsgAccountManager, accMgr, NS_MSGACCOUNTMANAGER_CONTRACTID, NS_PROXY_TO_MAIN_THREAD, &rv);
         NS_ASSERTION(NS_SUCCEEDED(rv) && accMgr,"failed to get account manager");
 		if (NS_FAILED(rv) || !accMgr) return;
 
@@ -231,7 +228,7 @@ nsresult nsOutlookCompose::CreateIdentity( void)
 		return( NS_OK);
 
 	nsresult	rv;
-    NS_WITH_PROXIED_SERVICE(nsIMsgAccountManager, accMgr, NS_MSGACCOUNTMANAGER_CONTRACTID, NS_UI_THREAD_EVENTQ, &rv);
+    NS_WITH_PROXIED_SERVICE(nsIMsgAccountManager, accMgr, NS_MSGACCOUNTMANAGER_CONTRACTID, NS_PROXY_TO_MAIN_THREAD, &rv);
     if (NS_FAILED(rv)) return( rv);
 	rv = accMgr->CreateIdentity( &m_pIdentity);
 	nsString	name;
@@ -252,7 +249,7 @@ nsresult nsOutlookCompose::CreateComponents( void)
 	if (!m_pIOService) {
 		IMPORT_LOG0( "Creating nsIOService\n");
 
-		NS_WITH_PROXIED_SERVICE(nsIIOService, service, kIOServiceCID, NS_UI_THREAD_EVENTQ, &rv);
+		NS_WITH_PROXIED_SERVICE(nsIIOService, service, NS_IOSERVICE_CONTRACTID, NS_PROXY_TO_MAIN_THREAD, &rv);
 		if (NS_FAILED(rv)) 
 			return( rv);
 		m_pIOService = service;
@@ -263,18 +260,11 @@ nsresult nsOutlookCompose::CreateComponents( void)
 	if (!m_pMsgSend) {
 		rv = CallCreateInstance( kMsgSendCID, &m_pMsgSend); 
 		if (NS_SUCCEEDED( rv) && m_pMsgSend) {
-			nsCOMPtr<nsIProxyObjectManager> proxyMgr = 
-			         do_GetService(kProxyObjectManagerCID, &rv);
-			if (NS_SUCCEEDED(rv)) {
-				rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsIMsgSend),
-										m_pMsgSend, PROXY_SYNC, (void **)&m_pSendProxy);
-				if (NS_FAILED( rv)) {
-					m_pSendProxy = nsnull;
-				}
-			}
-			if (NS_FAILED( rv)) {
+      rv = NS_GetProxyForObject( NS_PROXY_TO_MAIN_THREAD, NS_GET_IID(nsIMsgSend),
+                  m_pMsgSend, NS_PROXY_SYNC, (void **)&m_pSendProxy);
+      if (NS_FAILED( rv)) {
+        m_pSendProxy = nsnull;
 				NS_RELEASE( m_pMsgSend);
-				m_pMsgSend = nsnull;
 			}
 		}
 	}
@@ -623,17 +613,16 @@ nsresult nsOutlookCompose::SendTheMessage( nsIFileSpec *pMsg, nsMsgDeliverMode m
 	ExtractCharset( headerVal);
   // Use platform charset as default if the msg doesn't specify one
   // (ie, no 'charset' param in the Content-Type: header). As the last
-  // resort we'll use the mail defaul charset.
+  // resort we'll use the mail default charset.
   if (headerVal.IsEmpty())
   {
     headerVal.AssignWithConversion(nsMsgI18NFileSystemCharset());
     if (headerVal.IsEmpty())
     { // last resort
       nsXPIDLString defaultCharset;
-      nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
-      if (NS_SUCCEEDED(rv))
-        rv = prefs->GetLocalizedUnicharPref("mailnews.view_default_charset", getter_Copies(defaultCharset));
-      headerVal.Assign(defaultCharset.IsEmpty() ? NS_LITERAL_STRING("ISO-8859-1").get() : defaultCharset.get());
+      NS_GetLocalizedUnicharPreferenceWithDefault(nsnull, "mailnews.view_default_charset",
+                                                  NS_LITERAL_STRING("ISO-8859-1"), defaultCharset);
+      headerVal = defaultCharset;
     }
   }
   m_pMsgFields->SetCharacterSet( NS_LossyConvertUTF16toASCII(headerVal).get() );
@@ -687,7 +676,9 @@ nsresult nsOutlookCompose::SendTheMessage( nsIFileSpec *pMsg, nsMsgDeliverMode m
 										nsnull,                       // parent window
 										nsnull,                       // progress listener
 										m_pListener,		              // listener
-                    nsnull);                      // password
+										nsnull,                       // password
+										EmptyCString(),               // originalMsgURI
+                    nsnull);                      // message compose type
 
 
 	// IMPORT_LOG0( "Returned from CreateAndSendMessage\n");
@@ -910,8 +901,10 @@ nsresult nsOutlookCompose::CopyComposedMessage( nsCString& fromLine, nsIFileSpec
 	// so that the following will properly copy the rest of the body
 	char	lastChar = 0;
 
+  nsCOMPtr<nsIOutputStream> outStream;
+  pDst->GetOutputStream (getter_AddRefs (outStream));
 	if (NS_SUCCEEDED( rv)) {
-    rv = EscapeFromSpaceLine(pDst, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
+    rv = EscapeFromSpaceLine(outStream, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
 		if (copy.m_bytesInBuf)
 			lastChar = copy.m_pBuffer[copy.m_bytesInBuf - 1];
     if (NS_SUCCEEDED(rv))
@@ -921,7 +914,7 @@ nsresult nsOutlookCompose::CopyComposedMessage( nsCString& fromLine, nsIFileSpec
 	while ((state.offset < state.size) && NS_SUCCEEDED( rv)) {
 		rv = FillMailBuffer( &state, copy);
 		if (NS_SUCCEEDED( rv)) {
-      rv = EscapeFromSpaceLine(pDst, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
+      rv = EscapeFromSpaceLine(outStream, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
 			lastChar = copy.m_pBuffer[copy.m_bytesInBuf - 1];
 			if (NS_SUCCEEDED( rv))
 		    copy.m_writeOffset = copy.m_bytesInBuf;
