@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -63,6 +63,7 @@
 #include "nsAbBaseCID.h"
 #include "nsIAbDirectory.h"
 #include "nsIAddressBook.h"
+#include "nsIStringBundle.h"
 #include "nsImportStringBundle.h"
 #include "nsTextFormatter.h"
 #include "nsIProxyObjectManager.h"
@@ -71,12 +72,6 @@
 #include "ImportDebug.h"
 
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
-static NS_DEFINE_CID(kSupportsWStringCID, NS_SUPPORTS_STRING_CID);
-static NS_DEFINE_CID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
-
-
-////////////////////////////////////////////////////////////////////////
-
 PR_STATIC_CALLBACK( void) ImportAddressThread( void *stuff);
 
 
@@ -120,7 +115,7 @@ private:
 
 public:
 	static void	SetLogs( nsString& success, nsString& error, nsISupportsString *pSuccess, nsISupportsString *pError);
-	static void ReportError( PRUnichar *pName, nsString *pStream);
+  static void ReportError(PRUnichar *pName, nsString *pStream, nsIStringBundle *aBundle);
 
 private:
 	nsIImportAddressBooks *		m_pInterface;
@@ -138,6 +133,7 @@ private:
 	PRBool						m_doImport;
 	AddressThreadData *			m_pThreadData;
 	char *						m_pDestinationUri;
+    nsCOMPtr<nsIStringBundle>   m_stringBundle;
 };
 
 class AddressThreadData {
@@ -155,6 +151,7 @@ public:
 	nsISupportsString *		errorLog;
 	char *						pDestinationUri;
 	PRBool                      bAddrLocInput ;
+    nsIStringBundle*            stringBundle;
 
 	AddressThreadData();
 	~AddressThreadData();
@@ -199,6 +196,8 @@ nsImportGenericAddressBooks::nsImportGenericAddressBooks()
 	m_gotLocation = PR_FALSE;
 	m_found = PR_FALSE;
 	m_userVerify = PR_FALSE;
+
+    nsImportStringBundle::GetStringBundle(IMPORT_MSGS_URL, getter_AddRefs(m_stringBundle));
 }
 
 
@@ -295,7 +294,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::GetData(const char *dataId, nsISuppor
 		}
 		IMPORT_LOG1( "Requesting sample data #: %ld\n", (long)rNum);
 		if (m_pInterface) {
-			nsCOMPtr<nsISupportsString>	data = do_CreateInstance( kSupportsWStringCID, &rv);
+			nsCOMPtr<nsISupportsString>	data = do_CreateInstance( NS_SUPPORTS_STRING_CONTRACTID, &rv);
 			if (NS_FAILED( rv))
 				return( rv);
 			PRUnichar *	pData = nsnull;
@@ -478,8 +477,8 @@ void nsImportGenericAddressBooks::GetDefaultFieldMap( void)
 	rv = m_pFieldMap->GetNumMozFields( &sz);
 	if (NS_SUCCEEDED( rv))
 		rv = m_pFieldMap->DefaultFieldMap( sz);
-	if (NS_SUCCEEDED( rv))
-		rv = m_pInterface->InitFieldMap( m_pLocation, m_pFieldMap);
+    if (NS_SUCCEEDED( rv))
+      rv = m_pInterface->InitFieldMap(m_pFieldMap);
 	if (NS_FAILED( rv)) {
 		IMPORT_LOG0( "*** Error: Unable to initialize field map\n");
 		NS_IF_RELEASE( m_pFieldMap);
@@ -562,13 +561,13 @@ NS_IMETHODIMP nsImportGenericAddressBooks::BeginImport(nsISupportsString *succes
 	
 	if (!m_doImport) {
 		*_retval = PR_TRUE;
-		nsImportStringBundle::GetStringByID( IMPORT_NO_ADDRBOOKS, success);
+    nsImportStringBundle::GetStringByID(IMPORT_NO_ADDRBOOKS, success, m_stringBundle);
 		SetLogs( success, error, successLog, errorLog);
 		return( NS_OK);		
 	}
 	
 	if (!m_pInterface || !m_pBooks) {
-		nsImportStringBundle::GetStringByID( IMPORT_ERROR_AB_NOTINITIALIZED, error);
+    nsImportStringBundle::GetStringByID(IMPORT_ERROR_AB_NOTINITIALIZED, error, m_stringBundle);
 		SetLogs( success, error, successLog, errorLog);
 		*_retval = PR_FALSE;
 		return( NS_OK);
@@ -602,7 +601,9 @@ NS_IMETHODIMP nsImportGenericAddressBooks::BeginImport(nsISupportsString *succes
 	if (m_pDestinationUri)
 		m_pThreadData->pDestinationUri = nsCRT::strdup( m_pDestinationUri);
 	m_pThreadData->bAddrLocInput = isAddrLocHome ;
-				
+
+    NS_IF_ADDREF(m_pThreadData->stringBundle = m_stringBundle);
+
 	PRThread *pThread = PR_CreateThread( PR_USER_THREAD, &ImportAddressThread, m_pThreadData, 
 									PR_PRIORITY_NORMAL, 
 									PR_LOCAL_THREAD, 
@@ -613,7 +614,7 @@ NS_IMETHODIMP nsImportGenericAddressBooks::BeginImport(nsISupportsString *succes
 		m_pThreadData->DriverDelete();
 		m_pThreadData = nsnull;
 		*_retval = PR_FALSE;
-		nsImportStringBundle::GetStringByID( IMPORT_ERROR_AB_NOTHREAD, error);
+    nsImportStringBundle::GetStringByID(IMPORT_ERROR_AB_NOTHREAD, error, m_stringBundle);
 		SetLogs( success, error, successLog, errorLog);
 	}
 	else
@@ -703,18 +704,20 @@ AddressThreadData::AddressThreadData()
 	errorLog = nsnull;
 	pDestinationUri = nsnull;
 	fieldMap = nsnull;
+    stringBundle = nsnull;
 }
 
 AddressThreadData::~AddressThreadData()
 {
-	if (pDestinationUri)
-		nsCRT::free( pDestinationUri);
+  if (pDestinationUri)
+    nsCRT::free(pDestinationUri);
 
-	NS_IF_RELEASE( books);
-	NS_IF_RELEASE( addressImport);
-	NS_IF_RELEASE( errorLog);
-	NS_IF_RELEASE( successLog);
-	NS_IF_RELEASE( fieldMap);
+  NS_IF_RELEASE(books);
+  NS_IF_RELEASE(addressImport);
+  NS_IF_RELEASE(errorLog);
+  NS_IF_RELEASE(successLog);
+  NS_IF_RELEASE(fieldMap);
+  NS_IF_RELEASE(stringBundle);
 }
 
 void AddressThreadData::DriverDelete( void)
@@ -747,7 +750,9 @@ nsIAddrDatabase *GetAddressBookFromUri( const char *pUri)
     nsIAddrDatabase *	pDatabase = nsnull;
     if (pUri) {
         nsresult rv = NS_OK;
-        NS_WITH_PROXIED_SERVICE(nsIAddressBook, addressBook, NS_ADDRESSBOOK_CONTRACTID, NS_UI_THREAD_EVENTQ, &rv); 
+        NS_WITH_PROXIED_SERVICE(nsIAddressBook, addressBook,
+                                NS_ADDRESSBOOK_CONTRACTID,
+                                NS_PROXY_TO_MAIN_THREAD, &rv); 
         if (addressBook)
             rv = addressBook->GetAbDatabaseFromURI(pUri, &pDatabase);
     }
@@ -767,39 +772,38 @@ nsIAddrDatabase *GetAddressBook( const PRUnichar *name, PRBool makeNew)
 	
 	IMPORT_LOG0( "In GetAddressBook\n");
 
-	nsCOMPtr<nsIProxyObjectManager> proxyMgr = 
-	         do_GetService(kProxyObjectManagerCID, &rv);
-	if (NS_FAILED( rv)) {
-		IMPORT_LOG0( "*** Error: Unable to get proxy manager\n");
-		return( nsnull);
-	}
-
 	nsIAddrDatabase *	pDatabase = nsnull;
 
 	/* Get the profile directory */
-	// Note to Candice: This should return an nsIFileSpec, not a nsFileSpec
-	nsFileSpec *					dbPath = nsnull;
+	nsCOMPtr<nsILocalFile> dbPath;
 
-	NS_WITH_PROXIED_SERVICE(nsIAddrBookSession, abSession, NS_ADDRBOOKSESSION_CONTRACTID, NS_UI_THREAD_EVENTQ, &rv); 
+	NS_WITH_PROXIED_SERVICE(nsIAddrBookSession, abSession,
+                          NS_ADDRBOOKSESSION_CONTRACTID,
+                          NS_PROXY_TO_MAIN_THREAD, &rv); 
 	
 	if (NS_SUCCEEDED(rv))
-		abSession->GetUserProfileDirectory(&dbPath);
-	if (dbPath) {
+		rv = abSession->GetUserProfileDirectory(getter_AddRefs(dbPath));
+	if (NS_SUCCEEDED(rv)) {
 		// Create a new address book file - we don't care what the file
 		// name is, as long as it's unique
-		(*dbPath) += "impab.mab";
-		(*dbPath).MakeUnique();
+		rv = dbPath->Append(NS_LITERAL_STRING("impab.mab"));
+        if (NS_SUCCEEDED(rv)) {
+          rv = dbPath->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
 		
-		IMPORT_LOG0( "Getting the address database factory\n");
+          if (NS_SUCCEEDED(rv)) {
+            IMPORT_LOG0( "Getting the address database factory\n");
 
-		NS_WITH_PROXIED_SERVICE(nsIAddrDatabase, addrDBFactory, NS_ADDRDATABASE_CONTRACTID, NS_UI_THREAD_EVENTQ, &rv);
-		if (NS_SUCCEEDED(rv) && addrDBFactory) {
-			
-			IMPORT_LOG0( "Opening the new address book\n");
-			rv = addrDBFactory->Open( dbPath, PR_TRUE, &pDatabase, PR_TRUE);
-		}
+            NS_WITH_PROXIED_SERVICE(nsIAddrDatabase, addrDBFactory,
+                                    NS_ADDRDATABASE_CONTRACTID,
+                                    NS_PROXY_TO_MAIN_THREAD, &rv);
+            if (NS_SUCCEEDED(rv) && addrDBFactory) {
+		  	  IMPORT_LOG0( "Opening the new address book\n");
+			  rv = addrDBFactory->Open( dbPath, PR_TRUE, PR_TRUE, &pDatabase);
+            }
+          }
+        }
 	}
-	else {
+	if (NS_FAILED(rv)) {
 		IMPORT_LOG0( "Failed to get the user profile directory from the address book session\n");
 	}
 
@@ -809,7 +813,8 @@ nsIAddrDatabase *GetAddressBook( const PRUnichar *name, PRBool makeNew)
 		// This is major bogosity again!  Why doesn't the address book
 		// just handle this properly for me?  Uggggg...
 		
-		NS_WITH_PROXIED_SERVICE(nsIRDFService, rdfService, kRDFServiceCID, NS_UI_THREAD_EVENTQ, &rv);
+		NS_WITH_PROXIED_SERVICE(nsIRDFService, rdfService, kRDFServiceCID,
+                            NS_PROXY_TO_MAIN_THREAD, &rv);
 		if (NS_SUCCEEDED(rv)) {
 			nsCOMPtr<nsIRDFResource>	parentResource;
 			rv = rdfService->GetResource(NS_LITERAL_CSTRING(kAllDirectoryRoot),
@@ -826,34 +831,47 @@ nsIAddrDatabase *GetAddressBook( const PRUnichar *name, PRBool makeNew)
 			 * a thread other than the UI thread.
 			 *
 			 */
-			rv = proxyMgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID( nsIAbDirectory),
-				parentResource, PROXY_SYNC | PROXY_ALWAYS, getter_AddRefs( parentDir));
+			rv = NS_GetProxyForObject( NS_PROXY_TO_MAIN_THREAD,
+                                 NS_GET_IID( nsIAbDirectory),
+                                 parentResource,
+                                 NS_PROXY_SYNC | NS_PROXY_ALWAYS,
+                                 getter_AddRefs( parentDir));
 			if (parentDir)
-		       	{
+			{
 				nsCAutoString URI("moz-abmdbdirectory://");
-				URI.Append((*dbPath).GetLeafName());
-				parentDir->CreateDirectoryByURI(name, URI.get (), PR_FALSE);
-
-          delete dbPath;
+				nsCAutoString leafName;
+				rv = dbPath->GetNativeLeafName(leafName);
+				if (NS_FAILED(rv)) {
+				  IMPORT_LOG0( "*** Error: Unable to get name of database file\n");
+				}
+				else {
+				  URI.Append(leafName);
+				  rv = parentDir->CreateDirectoryByURI(name, URI.get (), PR_FALSE);
+				  if (NS_FAILED(rv))
+				    IMPORT_LOG0( "*** Error: Unable to create address book directory\n");
+				}
 			}
 
-			IMPORT_LOG0( "Added new address book to the UI\n");
+			if (NS_SUCCEEDED(rv))
+			  IMPORT_LOG0( "Added new address book to the UI\n");
+			else
+			  IMPORT_LOG0( "*** Error: An error occurred while adding the address book to the UI\n");
 		}
 	}
 	
 	return( pDatabase);
 }
 
-void nsImportGenericAddressBooks::ReportError( PRUnichar *pName, nsString *pStream)
+void nsImportGenericAddressBooks::ReportError( PRUnichar *pName, nsString *pStream, nsIStringBundle* aBundle)
 {
 	if (!pStream)
 		return;
 	// load the error string
-	PRUnichar *pFmt = nsImportStringBundle::GetStringByID( IMPORT_ERROR_GETABOOK);
+  PRUnichar *pFmt = nsImportStringBundle::GetStringByID( IMPORT_ERROR_GETABOOK, aBundle);
 	PRUnichar *pText = nsTextFormatter::smprintf( pFmt, pName);
 	pStream->Append( pText);
 	nsTextFormatter::smprintf_free( pText);
-	nsImportStringBundle::FreeString( pFmt);
+    nsCRT::free(pFmt);
 	pStream->AppendWithConversion( NS_LINEBREAK);
 }
 
@@ -874,6 +892,14 @@ PR_STATIC_CALLBACK( void) ImportAddressThread( void *stuff)
 	nsString					success;
 	nsString					error;
 
+    nsCOMPtr<nsIStringBundle> pBundle;
+    rv = nsImportStringBundle::GetStringBundleProxy(pData->stringBundle, getter_AddRefs(pBundle));
+    if (NS_FAILED(rv))
+    {
+      IMPORT_LOG0("*** ImportMailThread: Unable to obtain proxy string service for the import.");
+      pData->abort = PR_TRUE;
+    }
+
 	for (i = 0; (i < count) && !(pData->abort); i++) {
 		nsCOMPtr<nsIImportABDescriptor> book =
 			do_QueryElementAt(pData->books, i);
@@ -895,10 +921,10 @@ PR_STATIC_CALLBACK( void) ImportAddressThread( void *stuff)
 				}
 
 				nsCOMPtr<nsIAddrDatabase> proxyAddrDatabase;
-				rv = NS_GetProxyForObject(NS_UI_THREAD_EVENTQ,
+				rv = NS_GetProxyForObject(NS_PROXY_TO_MAIN_THREAD,
                                           NS_GET_IID(nsIAddrDatabase),
                                           pDestDB,
-                                          PROXY_SYNC | PROXY_ALWAYS,
+                                          NS_PROXY_SYNC | NS_PROXY_ALWAYS,
                                           getter_AddRefs(proxyAddrDatabase));
 				if (NS_FAILED(rv))
 					return;
@@ -941,7 +967,7 @@ PR_STATIC_CALLBACK( void) ImportAddressThread( void *stuff)
 					}
 				}
 				else {
-					nsImportGenericAddressBooks::ReportError( pName, &error);
+          nsImportGenericAddressBooks::ReportError(pName, &error, pBundle);
 				}
 
 				nsCRT::free( pName);
