@@ -128,12 +128,14 @@ nsresult nsMailboxProtocol::OpenMultipleMsgTransport(PRUint32 offset, PRInt32 si
 {
   nsresult rv;
 
-  NS_DEFINE_CID(kStreamTransportServiceCID, NS_STREAMTRANSPORTSERVICE_CID);
   nsCOMPtr<nsIStreamTransportService> serv =
-      do_GetService(kStreamTransportServiceCID, &rv);
+      do_GetService(NS_STREAMTRANSPORTSERVICE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = serv->CreateInputTransport(m_multipleMsgMoveCopyStream, offset, size, PR_FALSE, getter_AddRefs(m_transport));
+  // XXX 64-bit
+  rv = serv->CreateInputTransport(m_multipleMsgMoveCopyStream, nsInt64(offset),
+                                  nsInt64(size), PR_FALSE,
+                                  getter_AddRefs(m_transport));
 
   return rv;
 }
@@ -220,7 +222,7 @@ nsresult nsMailboxProtocol::Initialize(nsIURI * aURL)
   m_initialState = MAILBOX_READ_FOLDER;
   mCurrentProgress = 0;
   
-  NS_NewFileSpecWithSpec(m_tempMsgFileSpec, getter_AddRefs(m_tempMessageFile));
+  NS_NewFileSpecFromIFile(m_tempMsgFile, getter_AddRefs(m_tempMessageFile));
   return rv;
 }
 
@@ -409,27 +411,31 @@ PRInt32 nsMailboxProtocol::DoneReadingMessage()
 
 PRInt32 nsMailboxProtocol::SetupMessageExtraction()
 {
-	// Determine the number of bytes we are going to need to read out of the 
-	// mailbox url....
-	nsCOMPtr<nsIMsgDBHdr> msgHdr;
-	nsresult rv = NS_OK;
-	
+  // Determine the number of bytes we are going to need to read out of the 
+  // mailbox url....
+  nsCOMPtr<nsIMsgDBHdr> msgHdr;
+  nsresult rv = NS_OK;
+  
   NS_ASSERTION(m_runningUrl, "Not running a url");
   if (m_runningUrl)
   {
-        nsCOMPtr<nsIMsgMessageUrl> msgUrl = do_QueryInterface(m_runningUrl, &rv);
-        NS_ENSURE_SUCCESS(rv,rv);
-	  rv = msgUrl->GetMessageHeader(getter_AddRefs(msgHdr));
-	  if (NS_SUCCEEDED(rv) && msgHdr)
-	  {
-		  PRUint32 messageSize = 0;
-		  msgHdr->GetMessageSize(&messageSize);
-		  m_runningUrl->SetMessageSize(messageSize);
-	  }
-    else
-      NS_ASSERTION(PR_FALSE, "couldn't get message header");
+    PRUint32 messageSize = 0;
+    m_runningUrl->GetMessageSize(&messageSize);
+    if (!messageSize)
+    {
+      nsCOMPtr<nsIMsgMessageUrl> msgUrl = do_QueryInterface(m_runningUrl, &rv);
+      NS_ENSURE_SUCCESS(rv,rv);
+      rv = msgUrl->GetMessageHeader(getter_AddRefs(msgHdr));
+      if (NS_SUCCEEDED(rv) && msgHdr)
+      {
+        msgHdr->GetMessageSize(&messageSize);
+        m_runningUrl->SetMessageSize(messageSize);
+      }
+      else
+        NS_ASSERTION(PR_FALSE, "couldn't get message header");
+    }
   }
-	return rv;
+  return rv;
 }
 
 /////////////////////////////////////////////////////////////////////////////////////////////
@@ -454,6 +460,8 @@ nsresult nsMailboxProtocol::LoadUrl(nsIURI * aURL, nsISupports * aConsumer)
       
       PRBool convertData = PR_FALSE;
 
+      // need to check if we're fetching an rfc822 part in order to
+      // quote a message.
       if (m_mailboxAction == nsIMailboxUrl::ActionFetchMessage)
       {
         nsCOMPtr<nsIMsgMailNewsUrl> msgUrl = do_QueryInterface(m_runningUrl, &rv);
@@ -465,7 +473,7 @@ nsresult nsMailboxProtocol::LoadUrl(nsIURI * aURL, nsISupports * aConsumer)
 
         // check if this is a filter plugin requesting the message.
         // in that case, set up a text converter
-        convertData = (queryStr.Find("header=filter") != kNotFound);
+        convertData = (queryStr.Find("header=filter") != kNotFound || queryStr.Find("header=attach") != kNotFound);
       }
       else if (m_mailboxAction == nsIMailboxUrl::ActionFetchPart)
       {
@@ -656,7 +664,10 @@ PRInt32 nsMailboxProtocol::ReadMessageResponse(nsIInputStream * inputStream, PRU
   {
     PRInt32 contentLength = 0;
     GetContentLength(&contentLength);
-    mProgressEventSink->OnProgress(this, m_channelContext, mCurrentProgress, contentLength);
+    // XXX 64-bit
+    mProgressEventSink->OnProgress(this, m_channelContext,
+                                   nsUint64(mCurrentProgress),
+                                   nsUint64(contentLength));
   }
   
   if (NS_FAILED(rv)) return -1;
