@@ -75,9 +75,20 @@
 #include "nsMsgSearchCore.h"
 #include "nsMailHeaders.h"
 #include "nsIMsgMailSession.h"
+#include "nsIMsgComposeParams.h"
+#include "nsMsgCompCID.h"
+#include "nsIInterfaceRequestor.h"
+#include "nsIInterfaceRequestorUtils.h"
+#include "nsIDocShell.h"
+#include "nsIMsgCompose.h"
+#include "nsIDOMWindowInternal.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
+#include "nsIMsgComposeService.h"
+#include "nsIMsgCopyService.h"
+#include "nsICryptoHash.h"
 
 static NS_DEFINE_CID(kCMailDB, NS_MAILDB_CID);
-static NS_DEFINE_CID(kIOServiceCID,              NS_IOSERVICE_CID);
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 
 /* the following macros actually implement addref, release and query interface for our component. */
@@ -106,7 +117,7 @@ NS_IMETHODIMP nsMsgMailboxParser::OnStartRequest(nsIRequest *request, nsISupport
     // we have an error.
     nsresult rv = NS_OK;
 
-    nsCOMPtr<nsIIOService> ioServ(do_GetService(kIOServiceCID, &rv));
+    nsCOMPtr<nsIIOService> ioServ(do_GetService(NS_IOSERVICE_CONTRACTID, &rv));
 
     nsCOMPtr<nsIMailboxUrl> runningUrl = do_QueryInterface(ctxt, &rv);
 
@@ -307,22 +318,19 @@ int nsMsgMailboxParser::ProcessMailboxInputStream(nsIURI* aURL, nsIInputStream *
 
 void nsMsgMailboxParser::DoneParsingFolder(nsresult status)
 {
-	/* End of file.  Flush out any partial line remaining in the buffer. */
-	FlushLastLine();
-	PublishMsgHeader(nsnull);
+  /* End of file.  Flush out any partial line remaining in the buffer. */
+  FlushLastLine();
+  PublishMsgHeader(nsnull);
 
-	// only mark the db valid if we've succeeded.
-	if (NS_SUCCEEDED(status) && m_mailDB)	// finished parsing, so flush db folder info 
-		UpdateDBFolderInfo();
-	else if (m_mailDB)
-	{
-		m_mailDB->SetSummaryValid(PR_FALSE);
-		m_mailDB->Commit(nsMsgDBCommitType::kLargeCommit);
-	}
+  // only mark the db valid if we've succeeded.
+  if (NS_SUCCEEDED(status) && m_mailDB)	// finished parsing, so flush db folder info 
+    UpdateDBFolderInfo();
+  else if (m_mailDB)
+    m_mailDB->SetSummaryValid(PR_FALSE);
 
-//	if (m_folder != nsnull)
-//		m_folder->SummaryChanged();
-	FreeBuffers();
+  //	if (m_folder != nsnull)
+  //		m_folder->SummaryChanged();
+  FreeBuffers();
 }
 
 void nsMsgMailboxParser::FreeBuffers()
@@ -341,13 +349,7 @@ void nsMsgMailboxParser::UpdateDBFolderInfo()
 // update folder info in db so we know not to reparse.
 void nsMsgMailboxParser::UpdateDBFolderInfo(nsIMsgDatabase *mailDB)
 {
-	mailDB->SetSummaryValid(PR_TRUE);
-	mailDB->Commit(nsMsgDBCommitType::kLargeCommit);
-}
-
-// By default, do nothing
-void nsMsgMailboxParser::FolderTypeSpecificTweakMsgHeader(nsIMsgDBHdr * /* tweakMe */)
-{
+  mailDB->SetSummaryValid(PR_TRUE);
 }
 
 // Tell the world about the message header (add to db, and view, if any)
@@ -356,8 +358,6 @@ PRInt32 nsMsgMailboxParser::PublishMsgHeader(nsIMsgWindow *msgWindow)
   FinishHeader();
   if (m_newMsgHdr)
   {
-    FolderTypeSpecificTweakMsgHeader(m_newMsgHdr);
-    
     PRUint32 flags;
     (void)m_newMsgHdr->GetFlags(&flags);
     if (flags & MSG_FLAG_EXPUNGED)
@@ -508,10 +508,12 @@ NS_IMETHODIMP nsParseMailMessageState::Clear()
   m_envelope_from.length = 0;
   m_envelope_date.length = 0;
   m_priority.length = 0;
+  m_keywords.length = 0;
   m_mdn_dnt.length = 0;
   m_return_path.length = 0;
   m_account_key.length = 0;
   m_in_reply_to.length = 0;
+  m_replyTo.length = 0;
   m_content_type.length = 0;
   m_mdn_original_recipient.length = 0;
   m_body_lines = 0;
@@ -521,6 +523,7 @@ NS_IMETHODIMP nsParseMailMessageState::Clear()
   ClearAggregateHeader (m_ccList);
   m_headers.ResetWritePos();
   m_envelope.ResetWritePos();
+  m_receivedTime = LL_ZERO;
   return NS_OK;
 }
 
@@ -847,7 +850,8 @@ int nsParseMailMessageState::ParseHeaders ()
     char *end;
     char *value = 0;
     struct message_header *header = 0;
-    
+    struct message_header receivedBy;
+
     if (! colon)
       break;
     
@@ -898,6 +902,13 @@ int nsParseMailMessageState::ParseHeaders ()
       // Disposition-Notification-To
       else if (!nsCRT::strncasecmp ("Return-Receipt-To", buf, end - buf))
         header = &m_mdn_dnt;
+      else if (!nsCRT::strncasecmp("Reply-To", buf, end - buf))
+        header = &m_replyTo;
+      else if (!nsCRT::strncasecmp("Received", buf, end - buf))
+      {
+        header = &receivedBy;
+        header->length = 0;
+      }
       break;
     case 'S': case 's':
       if (!nsCRT::strncasecmp ("Subject", buf, end - buf))
@@ -930,6 +941,9 @@ int nsParseMailMessageState::ParseHeaders ()
       else if (!nsCRT::strncasecmp("X-Priority", buf, end - buf)
         || !nsCRT::strncasecmp("Priority", buf, end - buf))
         header = &m_priority;
+      else if (!nsCRT::strncasecmp(HEADER_X_MOZILLA_KEYWORDS, buf, end - buf)
+        && !m_keywords.length)
+        header = &m_keywords;
       break;
     }
     
@@ -986,9 +1000,26 @@ SEARCH_NEWLINE:
       while (header->length > 0 &&
         IS_SPACE (header->value [header->length - 1]))
         ((char *) header->value) [--header->length] = 0;
-    }
+      if (header == &receivedBy && LL_IS_ZERO(m_receivedTime))
+      {
+        // parse Received: header for date.
+        // We trust the first header as that is closest to recipient,
+        // and less likely to be spoofed.
+        nsCAutoString receivedHdr(header->value, header->length);
+        PRInt32 lastSemicolon = receivedHdr.RFindChar(';');
+        if (lastSemicolon != kNotFound)
+        {
+          nsCAutoString receivedDate;
+          receivedHdr.Right(receivedDate, receivedHdr.Length() - lastSemicolon - 1);
+          receivedDate.Trim(" \t\b\r\n");
+          PRTime resultTime;
+          if (PR_ParseTimeString (receivedDate.get(), PR_FALSE, &resultTime) == PR_SUCCESS)
+            m_receivedTime = resultTime;
         }
-        return 0;
+      }
+    }
+  }
+  return 0;
 }
 
 int nsParseMailMessageState::ParseEnvelope (const char *line, PRUint32 line_size)
@@ -1119,18 +1150,19 @@ int nsParseMailMessageState::FinalizeHeaders()
   struct message_header *subject;
   struct message_header *id;
   struct message_header *inReplyTo;
+  struct message_header *replyTo;
   struct message_header *references;
   struct message_header *date;
   struct message_header *statush;
   struct message_header *mozstatus;
   struct message_header *mozstatus2;
   struct message_header *priority;
+  struct message_header *keywords;
   struct message_header *account_key;
   struct message_header *ccList;
   struct message_header *mdn_dnt;
   struct message_header md5_header;
   struct message_header *content_type;
-  unsigned char md5_bin [16];
   char md5_data [50];
   
   const char *s;
@@ -1161,13 +1193,15 @@ int nsParseMailMessageState::FinalizeHeaders()
   references = (m_references.length ? &m_references : 0);
   statush    = (m_status.length     ? &m_status     : 0);
   mozstatus  = (m_mozstatus.length  ? &m_mozstatus  : 0);
-  mozstatus2  = (m_mozstatus2.length  ? &m_mozstatus2  : 0);
+  mozstatus2 = (m_mozstatus2.length  ? &m_mozstatus2  : 0);
   date       = (m_date.length       ? &m_date :
   m_envelope_date.length ? &m_envelope_date :
   0);
   priority   = (m_priority.length   ? &m_priority   : 0);
+  keywords   =  (m_keywords.length   ? &m_keywords  : 0);
   mdn_dnt	   = (m_mdn_dnt.length	  ? &m_mdn_dnt	  : 0);
   inReplyTo = (m_in_reply_to.length ? &m_in_reply_to : 0);
+  replyTo = (m_replyTo.length ? &m_replyTo : 0);
   content_type = (m_content_type.length ? &m_content_type : 0);
   account_key = (m_account_key.length ? &m_account_key :0);
   
@@ -1220,6 +1254,10 @@ int nsParseMailMessageState::FinalizeHeaders()
       if (priorityFlags != nsMsgPriority::notSet)
         m_newMsgHdr->SetPriority(priorityFlags);
   
+      // if we have a reply to header, and it's different from the from: header,
+      // set the "replyTo" attribute on the msg hdr.
+      if (replyTo && (!sender || replyTo->length != sender->length || strncmp(replyTo->value, sender->value, sender->length)))
+        m_newMsgHdr->SetStringProperty("replyTo", replyTo->value);
       // convert the flag values (0xE000000) to label values (0-5)
       if (mozstatus2) // only do this if we have a mozstatus2 header
       {
@@ -1299,13 +1337,17 @@ int nsParseMailMessageState::FinalizeHeaders()
         if (! id)
         {
           // what to do about this? we used to do a hash of all the headers...
-#ifdef SIMPLE_MD5
-          HASH_HashBuf(HASH_AlgMD5, md5_bin, (unsigned char *)m_headers,
-            (int) m_headers_fp);
-#else
-          // ### TODO: is it worth doing something different?
-          memcpy(md5_bin, "dummy message id", sizeof(md5_bin));						
-#endif
+          nsCAutoString hash;
+          const char *md5_bin = "dummy message id";
+          nsresult rv;
+          nsCOMPtr<nsICryptoHash> hasher = do_CreateInstance("@mozilla.org/security/hash;1", &rv);
+          if (NS_SUCCEEDED(rv))
+          {
+            if (NS_SUCCEEDED(hasher->Init(nsICryptoHash::MD5)) && 
+              NS_SUCCEEDED(hasher->Update((const PRUint8*) m_headers.GetBuffer(), m_headers.GetSize())) &&
+              NS_SUCCEEDED(hasher->Finish(PR_FALSE, hash)))
+                  md5_bin = hash.get();
+          }
           PR_snprintf (md5_data, sizeof(md5_data),
             "<md5:"
             "%02X%02X%02X%02X%02X%02X%02X%02X"
@@ -1363,16 +1405,26 @@ int nsParseMailMessageState::FinalizeHeaders()
         else if (inReplyTo != nsnull)
           m_newMsgHdr->SetReferences(inReplyTo->value);
         
-        if (date) {
-          PRTime resultTime;
-          PRStatus timeStatus = PR_ParseTimeString (date->value, PR_FALSE, &resultTime);
-          if (PR_SUCCESS == timeStatus)
-            m_newMsgHdr->SetDate(resultTime);
+        if (!LL_IS_ZERO(m_receivedTime))
+          m_newMsgHdr->SetDate(m_receivedTime);
+        else 
+        {
+          // if there's no date, or it's mal-formed, use now as the time.
+          // PR_ParseTimeString won't touch resultTime unless it succeeds.
+          // (this doesn't affect local messages, because we use the envelope
+          // date if there's no Date: header, but it would affect IMAP msgs
+          // w/o a Date: hdr or Received: headers)
+          PRTime resultTime = PR_Now();
+          if (date) 
+            PR_ParseTimeString (date->value, PR_FALSE, &resultTime);
+          m_newMsgHdr->SetDate(resultTime);
         }
         if (priority)
           m_newMsgHdr->SetPriorityString(priority->value);
         else if (priorityFlags == nsMsgPriority::notSet)
           m_newMsgHdr->SetPriority(nsMsgPriority::none);
+        if (keywords)
+          m_newMsgHdr->SetStringProperty("keywords", keywords->value);
         if (content_type)
         {
           char *substring = PL_strstr(content_type->value, "charset");
@@ -1437,12 +1489,15 @@ nsParseNewMailState::nsParseNewMailState()
   m_ibuffer = nsnull;
   m_ibuffer_size = 0;
   m_ibuffer_fp = 0;
+  m_moveCoalescer = nsnull;
  }
 
 NS_IMPL_ISUPPORTS_INHERITED1(nsParseNewMailState, nsMsgMailboxParser, nsIMsgFilterHitNotify)
 
 nsresult
-nsParseNewMailState::Init(nsIMsgFolder *serverFolder, nsIMsgFolder *downloadFolder, nsFileSpec &folder, nsIOFileStream *inboxFileStream, nsIMsgWindow *aMsgWindow)
+nsParseNewMailState::Init(nsIMsgFolder *serverFolder, nsIMsgFolder *downloadFolder, nsFileSpec &folder, 
+                          nsIOFileStream *inboxFileStream, nsIMsgWindow *aMsgWindow,
+                          PRBool downloadingToTempFile)
 {
   nsresult rv;
   m_position = folder.GetFileSize();
@@ -1451,6 +1506,7 @@ nsParseNewMailState::Init(nsIMsgFolder *serverFolder, nsIMsgFolder *downloadFold
   m_inboxFileStream = inboxFileStream;
   m_msgWindow = aMsgWindow;
   m_downloadFolder = downloadFolder;
+  m_downloadingToTempFile = downloadingToTempFile;
 
   // the new mail parser isn't going to get the stream input, it seems, so we can't use
   // the OnStartRequest mechanism the mailbox parser uses. So, let's open the db right now.
@@ -1477,7 +1533,17 @@ nsParseNewMailState::Init(nsIMsgFolder *serverFolder, nsIMsgFolder *downloadFold
     if (m_filterList) 
     {
       rv = server->ConfigureTemporaryFilters(m_filterList);
-      NS_ASSERTION(NS_SUCCEEDED(rv), "failed to configure temp filters");
+    }
+    // check if this server defers to another server, in which case
+    // we'll use that server's filters as well.
+    nsCOMPtr <nsIMsgFolder> deferredToRootFolder;
+    server->GetRootMsgFolder(getter_AddRefs(deferredToRootFolder));
+    if (rootMsgFolder != deferredToRootFolder)
+    {
+      nsCOMPtr <nsIMsgIncomingServer> deferredToServer;
+      deferredToRootFolder->GetServer(getter_AddRefs(deferredToServer));
+      if (deferredToServer)
+        deferredToServer->GetFilterList(aMsgWindow, getter_AddRefs(m_deferredToServerFilterList));
     }
   }
   m_disableFilters = PR_FALSE;
@@ -1531,12 +1597,73 @@ PRInt32 nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
   
   if (m_newMsgHdr)
   {
-    FolderTypeSpecificTweakMsgHeader(m_newMsgHdr);
     if (!m_disableFilters)
     {
       // flush the inbox because filters will read from disk
       m_inboxFileStream->flush();
-      ApplyFilters(&moved, msgWindow);
+      PRUint32 msgOffset;
+      (void) m_newMsgHdr->GetMessageOffset(&msgOffset);
+      m_curHdrOffset = msgOffset;
+
+      nsCOMPtr<nsIMsgIncomingServer> server;
+      nsresult rv = m_rootFolder->GetServer(getter_AddRefs(server));
+      NS_ENSURE_SUCCESS(rv, 0);
+      PRInt32 duplicateAction;
+      server->GetIncomingDuplicateAction(&duplicateAction);
+      if (duplicateAction != nsIMsgIncomingServer::keepDups)
+      {
+        PRBool isDup;
+        server->IsNewHdrDuplicate(m_newMsgHdr, &isDup);
+        if (isDup)
+        {
+          // we want to do something similar to applying filter hits.
+          // if a dup is marked read, it shouldn't trigger biff.
+          // Same for deleting it or moving it to trash.
+          switch (duplicateAction)
+          {
+            case nsIMsgIncomingServer::deleteDups:
+              {
+                m_inboxFileStream->close();
+
+                nsresult truncRet = m_inboxFileSpec.Truncate(msgOffset);
+                NS_ASSERTION(NS_SUCCEEDED(truncRet), "unable to truncate file");
+                if (NS_FAILED(truncRet))
+                  m_rootFolder->ThrowAlertMsg("dupDeleteFolderTruncateFailed", msgWindow);
+
+                //  need to re-open the inbox file stream.
+                m_inboxFileStream->Open(m_inboxFileSpec, (PR_RDWR | PR_CREATE_FILE));
+                if (m_inboxFileStream)
+                  m_inboxFileStream->seek(m_inboxFileSpec.GetFileSize());
+  
+                m_mailDB->RemoveHeaderMdbRow(m_newMsgHdr);
+                // tell parser we've truncated the inbox.
+                nsParseMailMessageState::Init(msgOffset);
+
+              }
+              break;
+            case nsIMsgIncomingServer::moveDupsToTrash:
+              {
+                nsCOMPtr <nsIMsgFolder> trash;
+                GetTrashFolder(getter_AddRefs(trash));
+                if (trash)
+                  MoveIncorporatedMessage(m_newMsgHdr, m_mailDB, trash,
+                                                          nsnull, msgWindow);
+              }
+              break;
+            case nsIMsgIncomingServer::markDupsRead:
+              MarkFilteredMessageRead(m_newMsgHdr);
+              break;
+          }
+          PRInt32 numNewMessages;
+          m_downloadFolder->GetNumNewMessages(PR_FALSE, &numNewMessages);
+          m_downloadFolder->SetNumNewMessages(numNewMessages - 1);
+
+          m_newMsgHdr = nsnull;
+          return 0;
+        }
+      }
+
+      ApplyFilters(&moved, msgWindow, msgOffset);
     }
     if (!moved)
     {
@@ -1549,7 +1676,7 @@ PRInt32 nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
         
         m_mailDB->AddNewHdrToDB(m_newMsgHdr, PR_TRUE);
       }
-    }		// if it was moved by imap filter, m_parseMsgState->m_newMsgHdr == nsnull
+    } // if it was moved by imap filter, m_parseMsgState->m_newMsgHdr == nsnull
     m_newMsgHdr = nsnull;
   }
   return 0;
@@ -1578,28 +1705,39 @@ nsresult nsParseNewMailState::GetTrashFolder(nsIMsgFolder **pTrashFolder)
   return rv;
 }
 
-void nsParseNewMailState::ApplyFilters(PRBool *pMoved, nsIMsgWindow *msgWindow)
+void nsParseNewMailState::ApplyFilters(PRBool *pMoved, nsIMsgWindow *msgWindow, PRUint32 msgOffset)
 {
   m_msgMovedByFilter = PR_FALSE;
-  
-  nsCOMPtr<nsIMsgDBHdr> msgHdr = m_newMsgHdr;
-  nsCOMPtr<nsIMsgFolder> downloadFolder = m_downloadFolder;
-  nsCOMPtr <nsIMsgFolder> rootMsgFolder = do_QueryInterface(m_rootFolder);
-  if (rootMsgFolder)
+  m_curHdrOffset = msgOffset;
+
+  if (!m_disableFilters)
   {
-    PRUint32 numFolders;
-    if (!downloadFolder)
-      rootMsgFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_INBOX, 1, &numFolders, getter_AddRefs(downloadFolder));
-    if (downloadFolder)
-      downloadFolder->GetURI(getter_Copies(m_inboxUri));
-    char * headers = m_headers.GetBuffer();
-    PRUint32 headersSize = m_headers.GetBufferPos();
-    nsresult matchTermStatus;
-    if (m_filterList)
-      matchTermStatus = m_filterList->ApplyFiltersToHdr(nsMsgFilterType::InboxRule,
-                  msgHdr, downloadFolder, m_mailDB, headers, headersSize, this, msgWindow);
+    nsCOMPtr<nsIMsgDBHdr> msgHdr = m_newMsgHdr;
+    nsCOMPtr<nsIMsgFolder> downloadFolder = m_downloadFolder;
+    nsCOMPtr <nsIMsgFolder> rootMsgFolder = do_QueryInterface(m_rootFolder);
+    if (rootMsgFolder)
+    {
+      PRUint32 numFolders;
+      if (!downloadFolder)
+        rootMsgFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_INBOX, 1, &numFolders, getter_AddRefs(downloadFolder));
+      if (downloadFolder)
+        downloadFolder->GetURI(getter_Copies(m_inboxUri));
+      char * headers = m_headers.GetBuffer();
+      PRUint32 headersSize = m_headers.GetBufferPos();
+      nsresult matchTermStatus;
+      // get nsILocalFile from m_inboxFileSpec
+      nsCOMPtr <nsILocalFile> localFile;
+      NS_FileSpecToIFile(&m_inboxFileSpec, getter_AddRefs(localFile));
+      if (m_filterList)
+        matchTermStatus = m_filterList->ApplyFiltersToHdr(nsMsgFilterType::InboxRule,
+                    msgHdr, downloadFolder, m_mailDB, headers, headersSize, this, msgWindow, localFile);
+      if (!m_msgMovedByFilter && m_deferredToServerFilterList)
+      {
+        matchTermStatus = m_deferredToServerFilterList->ApplyFiltersToHdr(nsMsgFilterType::InboxRule,
+                    msgHdr, downloadFolder, m_mailDB, headers, headersSize, this, msgWindow, localFile);
+      }
+    }
   }
-  
   if (pMoved)
     *pMoved = m_msgMovedByFilter;
 }
@@ -1641,7 +1779,8 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
     
     if (NS_SUCCEEDED(filterAction->GetType(&actionType)))
     {
-      if (actionType == nsMsgFilterAction::MoveToFolder)
+      if (actionType == nsMsgFilterAction::MoveToFolder ||
+          actionType == nsMsgFilterAction::CopyToFolder)
       {
         filterAction->GetTargetFolderUri(getter_Copies(actionTargetFolderUri));
         if (actionTargetFolderUri.IsEmpty())
@@ -1668,19 +1807,78 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
         if ((const char*)actionTargetFolderUri &&
           nsCRT::strcasecmp(m_inboxUri, actionTargetFolderUri))
         {
-          nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, actionTargetFolderUri, filter, msgWindow);
-          if (NS_SUCCEEDED(err))
+          nsresult err;
+          nsCOMPtr<nsIRDFService> rdf(do_GetService(kRDFServiceCID, &err)); 
+          NS_ENSURE_SUCCESS(err, err);
+          nsCOMPtr<nsIRDFResource> res;
+          err = rdf->GetResource(actionTargetFolderUri, getter_AddRefs(res));
+          if (NS_FAILED(err))
+            return err;
+  
+          nsCOMPtr<nsIMsgFolder> destIFolder(do_QueryInterface(res, &err));
+          if (NS_FAILED(err))
+            return err;        
+  
+
+          if (StringBeginsWith(actionTargetFolderUri, NS_LITERAL_CSTRING("imap:")))
           {
+            if (!m_moveCoalescer)
+              m_moveCoalescer = new nsImapMoveCoalescer(m_downloadFolder, m_msgWindow);
+            NS_ENSURE_TRUE(m_moveCoalescer, NS_ERROR_OUT_OF_MEMORY);
+            nsMsgKey msgKey;
+            (void) msgHdr->GetMessageKey(&msgKey);
+            m_moveCoalescer->AddMove(destIFolder , msgKey);
             if (loggingEnabled)
               (void)filter->LogRuleHit(filterAction, msgHdr); 
-            
-            // cleanup after mailHdr in source DB because we moved the message.
-            m_mailDB->RemoveHeaderMdbRow(msgHdr);
-            m_msgMovedByFilter = PR_TRUE;
+            err = NS_OK;
+            msgIsNew = PR_FALSE;
+          }
+          else
+          {
+            err = MoveIncorporatedMessage(msgHdr, m_mailDB, destIFolder, filter, msgWindow);
+            m_msgMovedByFilter = NS_SUCCEEDED(err);
+            if (m_msgMovedByFilter)
+            {
+              if (loggingEnabled)
+                (void)filter->LogRuleHit(filterAction, msgHdr); 
+              // if we're downloading to a temp file, our message key is wrong,
+              // i.e., relative to the temp file and not the original mailbox,
+              // and we need to let nsPop3Sink remove the message hdr after
+              // it fixes the key.
+              if (!m_downloadingToTempFile)
+                m_mailDB->RemoveHeaderMdbRow(msgHdr);
+            }
           }
         }
         *applyMore = PR_FALSE; 
         break;
+        case nsMsgFilterAction::CopyToFolder:
+        {
+          nsXPIDLCString uri;
+          rv = m_rootFolder->GetURI(getter_Copies(uri));
+
+          if (NS_STATIC_CAST(const char*, actionTargetFolderUri) &&
+            strcmp(uri, actionTargetFolderUri))
+          {
+
+            nsCOMPtr<nsISupportsArray> messageArray;
+            NS_NewISupportsArray(getter_AddRefs(messageArray));
+            messageArray->AppendElement(msgHdr);
+
+            nsCOMPtr<nsIMsgFolder> dstFolder;
+            rv = GetExistingFolder(actionTargetFolderUri,
+                                   getter_AddRefs(dstFolder));
+            NS_ENSURE_SUCCESS(rv, rv);
+
+            nsCOMPtr<nsIMsgCopyService> copyService =
+              do_GetService(NS_MSGCOPYSERVICE_CONTRACTID, &rv);
+            NS_ENSURE_SUCCESS(rv, rv);
+            rv = copyService->CopyMessages(m_rootFolder, messageArray, dstFolder,
+                                           PR_FALSE, nsnull, msgWindow, PR_FALSE);
+            NS_ENSURE_SUCCESS(rv, rv);
+          }
+        }
+        break;
       case nsMsgFilterAction::MarkRead:
         msgIsNew = PR_FALSE;
         MarkFilteredMessageRead(msgHdr);
@@ -1700,6 +1898,16 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
         filterAction->GetPriority(&filterPriority);
         msgHdr->SetPriority(filterPriority);
         break;
+      case nsMsgFilterAction::AddTag:
+      {
+        nsXPIDLCString keyword;
+        filterAction->GetStrValue(getter_Copies(keyword));
+        nsCOMPtr<nsISupportsArray> messageArray;
+        NS_NewISupportsArray(getter_AddRefs(messageArray));
+        messageArray->AppendElement(msgHdr);
+        m_downloadFolder->AddKeywordToMessages(messageArray, keyword.get());
+        break;
+      }
       case nsMsgFilterAction::Label:
         nsMsgLabelValue filterLabel;
         filterAction->GetLabel(&filterLabel);
@@ -1713,11 +1921,30 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
         PRInt32 junkScore;
         filterAction->GetJunkScore(&junkScore);
         junkScoreStr.AppendInt(junkScore);
+        if (junkScore > 50)
+          msgIsNew = PR_FALSE;
         nsMsgKey msgKey;
         msgHdr->GetMessageKey(&msgKey);
         m_mailDB->SetStringProperty(msgKey, "junkscore", junkScoreStr.get());
         m_mailDB->SetStringProperty(msgKey, "junkscoreorigin", /* ### should this be plugin? */"plugin");
+        break;
       }
+      case nsMsgFilterAction::Forward:
+        {
+          nsXPIDLCString forwardTo;
+          filterAction->GetStrValue(getter_Copies(forwardTo));
+          m_forwardTo.AppendCString(forwardTo);
+          m_msgToForwardOrReply = msgHdr;
+        }
+        break;
+      case nsMsgFilterAction::Reply:
+        {
+          nsXPIDLCString replyTemplateUri;
+          filterAction->GetStrValue(getter_Copies(replyTemplateUri));
+          m_replyTemplateUri.AppendCString(replyTemplateUri);
+          m_msgToForwardOrReply = msgHdr;
+        }
+        break;
       case nsMsgFilterAction::DeleteFromPop3Server:
         {
           PRUint32 flags = 0;
@@ -1747,7 +1974,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
         break;
       case nsMsgFilterAction::FetchBodyFromPop3Server:
         {
-	  PRUint32 flags = 0;
+      	  PRUint32 flags = 0;
           nsCOMPtr <nsIMsgFolder> downloadFolder;
           msgHdr->GetFolder(getter_AddRefs(downloadFolder));
           nsCOMPtr <nsIMsgLocalMailFolder> localFolder = do_QueryInterface(downloadFolder);
@@ -1760,20 +1987,20 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
             nsCOMPtr<nsISupports> iSupports = do_QueryInterface(msgHdr);
             messages->AppendElement(iSupports);
             localFolder->MarkMsgsOnPop3Server(messages, POP3_FETCH_BODY);
-	    // Don't add this header to the DB, we're going to replace it
-	    // with the full message.
+            // Don't add this header to the DB, we're going to replace it
+            // with the full message.
             m_msgMovedByFilter = PR_TRUE;
             msgIsNew = PR_FALSE;
-	    // Don't do anything else in this filter, wait until we
-	    // have the full message.
-	    *applyMore = PR_FALSE;
+            // Don't do anything else in this filter, wait until we
+            // have the full message.
+            *applyMore = PR_FALSE;
           }
         }
         break;
       default:
         break;
       }
-      if (loggingEnabled && actionType != nsMsgFilterAction::MoveToFolder && actionType != nsMsgFilterAction::Delete)  
+      if (loggingEnabled && actionType != nsMsgFilterAction::MoveToFolder && actionType != nsMsgFilterAction::Delete)
         (void)filter->LogRuleHit(filterAction, msgHdr); 
     }
   }
@@ -1786,6 +2013,52 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
   return rv;
 }
 
+// this gets run in a second pass, after apply filters to a header.
+nsresult nsParseNewMailState::ApplyForwardAndReplyFilter(nsIMsgWindow *msgWindow)
+{
+  nsresult rv = NS_OK;
+  nsCOMPtr <nsIMsgIncomingServer> server;
+
+  PRInt32 i;
+  for (i = 0; i < m_forwardTo.Count(); i++)
+  {
+    if (!m_forwardTo[i]->IsEmpty())
+    {
+      nsAutoString forwardStr;
+      forwardStr.AssignWithConversion(m_forwardTo[i]->get());
+
+      rv = m_rootFolder->GetServer(getter_AddRefs(server));
+      NS_ENSURE_SUCCESS(rv, rv);
+      {
+        nsCOMPtr <nsIMsgComposeService> compService = do_GetService (NS_MSGCOMPOSESERVICE_CONTRACTID) ;
+        if (compService)
+          rv = compService->ForwardMessage(forwardStr, m_msgToForwardOrReply, msgWindow, server);
+      }
+    }
+  }
+  m_forwardTo.Clear();
+
+  for (i = 0; i < m_replyTemplateUri.Count(); i++)
+  {
+    if (!m_replyTemplateUri[i]->IsEmpty())
+    {
+      // copy this and truncate the original, so we don't accidentally re-use it on the next hdr.
+      nsCAutoString replyTemplateUri(*m_replyTemplateUri[i]);
+      rv = m_rootFolder->GetServer(getter_AddRefs(server));
+      if (server && !replyTemplateUri.IsEmpty())
+      {
+        nsCOMPtr <nsIMsgComposeService> compService = do_GetService (NS_MSGCOMPOSESERVICE_CONTRACTID) ;
+        if (compService)
+          rv = compService->ReplyWithTemplate(m_msgToForwardOrReply, replyTemplateUri.get(), msgWindow, server);
+      }
+    }
+  }
+  m_replyTemplateUri.Clear();
+  m_msgToForwardOrReply = nsnull;
+  return rv;
+}
+
+
 int nsParseNewMailState::MarkFilteredMessageRead(nsIMsgDBHdr *msgHdr)
 {
   PRUint32 newFlags;
@@ -1798,6 +2071,9 @@ int nsParseNewMailState::MarkFilteredMessageRead(nsIMsgDBHdr *msgHdr)
 
 nsresult nsParseNewMailState::EndMsgDownload()
 {
+  if (m_moveCoalescer)
+    m_moveCoalescer->PlaybackMoves();
+
   // need to do this for all folders that had messages filtered into them
   PRUint32 serverCount = m_filterTargetFolders.Count();
   nsresult rv;
@@ -1814,7 +2090,12 @@ nsresult nsParseNewMailState::EndMsgDownload()
         PRUint32 folderFlags;
         m_filterTargetFolders[index]->GetFlags(&folderFlags);
         if (! (folderFlags & (MSG_FOLDER_FLAG_TRASH | MSG_FOLDER_FLAG_INBOX)))
-          m_filterTargetFolders[index]->SetMsgDatabase(nsnull);
+        {
+          PRBool filtersRun;
+          m_filterTargetFolders[index]->CallFilterPlugins(nsnull, &filtersRun);
+          if (!filtersRun)
+            m_filterTargetFolders[index]->SetMsgDatabase(nsnull);
+        }
       }
     }
   }
@@ -1822,25 +2103,69 @@ nsresult nsParseNewMailState::EndMsgDownload()
   return rv;
 }
 
+nsresult nsParseNewMailState::AppendMsgFromFile(nsIOFileStream *fileStream, 
+                                                PRInt32 offset, PRUint32 length,
+                                                nsFileSpec &destFileSpec)
+{
+  fileStream->seek(PR_SEEK_SET, offset);
+
+  nsIOFileStream *destFile = new nsIOFileStream(destFileSpec, PR_WRONLY | PR_CREATE_FILE);
+  
+  if (!destFile) 
+  {
+#ifdef DEBUG_bienvenu
+    NS_ASSERTION(PR_FALSE, "out of memory");
+#endif
+    return  NS_MSG_ERROR_WRITING_MAIL_FOLDER;
+  }
+  
+  destFile->seek(PR_SEEK_END, 0);
+  PRInt32 newMsgPos = destFile->tell();
+  
+  if (!m_ibuffer)
+    m_ibuffer_size = 10240;
+  m_ibuffer_fp = 0;
+  
+  while (!m_ibuffer && (m_ibuffer_size >= 512))
+  {
+    m_ibuffer = (char *) PR_Malloc(m_ibuffer_size);
+    if (m_ibuffer == nsnull)
+      m_ibuffer_size /= 2;
+  }
+  NS_ASSERTION(m_ibuffer != nsnull, "couldn't get memory to move msg");
+  while ((length > 0) && m_ibuffer)
+  {
+    PRUint32 nRead = m_inboxFileStream->read (m_ibuffer, length > m_ibuffer_size ? m_ibuffer_size  : length);
+    if (nRead == 0)
+      break;
+    
+    // we must monitor the number of bytes actually written to the file. (mscott)
+    if (destFile->write(m_ibuffer, nRead) != (PRInt32) nRead) 
+    {
+      destFile->close();     
+      
+      // truncate  destination file in case message was partially written
+      // ### how to do this with a stream?
+      destFileSpec.Truncate(newMsgPos);
+      delete destFile;
+      return NS_MSG_ERROR_WRITING_MAIL_FOLDER;
+    }
+    
+    length -= nRead;
+  }
+  
+  NS_ASSERTION(length == 0, "didn't read all of original message in filter move");
+  delete destFile;
+  return NS_OK;
+}
+
 nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr, 
                                                       nsIMsgDatabase *sourceDB, 
-                                                      const nsACString& destFolderUri,
+                                                      nsIMsgFolder *destIFolder,
                                                       nsIMsgFilter *filter,
                                                       nsIMsgWindow *msgWindow)
 {
   nsresult err = 0;
-  nsIOFileStream *destFile;
-  
-  nsCOMPtr<nsIRDFService> rdf(do_GetService(kRDFServiceCID, &err)); 
-  NS_ENSURE_SUCCESS(err, err);
-  nsCOMPtr<nsIRDFResource> res;
-  err = rdf->GetResource(destFolderUri, getter_AddRefs(res));
-  if (NS_FAILED(err))
-    return err;
-  
-  nsCOMPtr<nsIMsgFolder> destIFolder(do_QueryInterface(res, &err));
-  if (NS_FAILED(err))
-    return err;        
   
   // check if the destination is a real folder (by checking for null parent)
   // and if it can file messages (e.g., servers or news folders can't file messages).
@@ -1852,13 +2177,27 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
     destIFolder->GetCanFileMessages(&canFileMessages);
   if (!parentFolder || !canFileMessages)
   {
-    filter->SetEnabled(PR_FALSE);
-    destIFolder->ThrowAlertMsg("filterDisabled", msgWindow);
+    if (filter)
+    {
+      filter->SetEnabled(PR_FALSE);
+      // we need to explicitly save the filter file.
+      if (m_filterList)
+        m_filterList->SaveToDefaultFile();
+      destIFolder->ThrowAlertMsg("filterDisabled", msgWindow);
+    }
     return NS_MSG_NOT_A_MAIL_FOLDER;
   }
   
+  nsCOMPtr <nsIMsgLocalMailFolder> destLocalFolder = do_QueryInterface(destIFolder);
+  if (destLocalFolder)
+  {
+    PRBool destFolderTooBig;
+    destLocalFolder->WarnIfLocalFileTooBig(msgWindow, &destFolderTooBig);
+    if (destFolderTooBig)
+      return NS_MSG_ERROR_WRITING_MAIL_FOLDER;
+  }
   nsCOMPtr <nsIFileSpec> destIFolderSpec;
-  
+
   nsFileSpec destFolderSpec;
   destIFolder->GetPath(getter_AddRefs(destIFolderSpec));
   err = destIFolderSpec->GetFileSpec(&destFolderSpec);
@@ -1888,27 +2227,8 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
     return NS_MSG_FOLDER_UNREADABLE;	// ### dmb
   }
   
-  PRUint32 messageOffset = 0;
-  
-  mailHdr->GetMessageOffset(&messageOffset);
-  m_inboxFileStream->seek(PR_SEEK_SET, messageOffset);
-  int newMsgPos;
-  
-  destFile = new nsIOFileStream(destFolderSpec, PR_WRONLY | PR_CREATE_FILE);
-  
-  if (!destFile) 
-  {
-#ifdef DEBUG_bienvenu
-    NS_ASSERTION(PR_FALSE, "out of memory");
-#endif
-    if (destIFolder)
-      destIFolder->ReleaseSemaphore (myISupports);
-    destIFolder->ThrowAlertMsg("filterFolderWriteFailed", msgWindow);
-    return  NS_MSG_ERROR_WRITING_MAIL_FOLDER;
-  }
-  
-  destFile->seek(PR_SEEK_END, 0);
-  newMsgPos = destFile->tell();
+  m_inboxFileStream->seek(PR_SEEK_SET, m_curHdrOffset);
+  PRUint32 newMsgPos = destFolderSpec.GetFileSize();
   
   nsCOMPtr<nsIMsgLocalMailFolder> localFolder = do_QueryInterface(destIFolder);
   nsCOMPtr<nsIMsgDatabase> destMailDB;
@@ -1920,50 +2240,25 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
   NS_ASSERTION(destMailDB, "failed to open mail db parsing folder");
   // don't force upgrade in place - open the db here before we start writing to the 
   // destination file because XP_Stat can return file size including bytes written...
-  PRUint32 length;
-  mailHdr->GetMessageSize(&length);
-  
-  if (!m_ibuffer)
-    m_ibuffer_size = 10240;
-  m_ibuffer_fp = 0;
-  
-  while (!m_ibuffer && (m_ibuffer_size >= 512))
+
+  PRUint32 messageLength;
+  mailHdr->GetMessageSize(&messageLength);
+  rv = AppendMsgFromFile(m_inboxFileStream, m_curHdrOffset, messageLength, destFolderSpec);
+
+  if (NS_FAILED(rv))
   {
-    m_ibuffer = (char *) PR_Malloc(m_ibuffer_size);
-    if (m_ibuffer == nsnull)
-      m_ibuffer_size /= 2;
-  }
-  NS_ASSERTION(m_ibuffer != nsnull, "couldn't get memory to move msg");
-  while ((length > 0) && m_ibuffer)
-  {
-    PRUint32 nRead = m_inboxFileStream->read (m_ibuffer, length > m_ibuffer_size ? m_ibuffer_size  : length);
-    if (nRead == 0)
-      break;
-    
-    // we must monitor the number of bytes actually written to the file. (mscott)
-    if (destFile->write(m_ibuffer, nRead) != (PRInt32) nRead) 
+    if (destMailDB)
+      destMailDB->Close(PR_TRUE);
+
+    if (destIFolder)
     {
-      destFile->close();     
-      
-      // truncate  destination file in case message was partially written
-      // ### how to do this with a stream?
-      destFolderSpec.Truncate(newMsgPos);
-      
-      if (destIFolder)
-        destIFolder->ReleaseSemaphore(myISupports);
-      
-      if (destMailDB)
-        destMailDB->Close(PR_TRUE);
+      destIFolder->ReleaseSemaphore(myISupports);
       destIFolder->ThrowAlertMsg("filterFolderWriteFailed", msgWindow);
-      return NS_MSG_ERROR_WRITING_MAIL_FOLDER;   // caller (ApplyFilters) currently ignores error conditions
     }
-    
-    length -= nRead;
+    return NS_MSG_ERROR_WRITING_MAIL_FOLDER;
   }
-  
-  NS_ASSERTION(length == 0, "didn't read all of original message in filter move");
-  
-  PRBool movedMsgIsNew = PR_TRUE;
+    
+  PRBool movedMsgIsNew = PR_FALSE;
   // if we have made it this far then the message has successfully been written to the new folder
   // now add the header to the destMailDB.
   if (NS_SUCCEEDED(rv) && destMailDB)
@@ -1979,14 +2274,17 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
       newHdr->GetFlags(&newFlags);
       if (! (newFlags & MSG_FLAG_READ))
       {
-        newHdr->OrFlags(MSG_FLAG_NEW, &newFlags);
-        destMailDB->AddToNewList(newMsgPos);
-      }
-      else
-      {
-        movedMsgIsNew = PR_FALSE;
+        nsXPIDLCString junkScoreStr;
+        (void) newHdr->GetStringProperty("junkscore", getter_Copies(junkScoreStr));
+        if (atoi(junkScoreStr.get()) < 50)
+        {
+          newHdr->OrFlags(MSG_FLAG_NEW, &newFlags);
+          destMailDB->AddToNewList(newMsgPos);
+          movedMsgIsNew = PR_TRUE;
+        }
       }
       destMailDB->AddNewHdrToDB(newHdr, PR_TRUE);
+      m_msgToForwardOrReply = newHdr;
     }
   }
   else
@@ -1996,14 +2294,11 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
   }
   if (movedMsgIsNew)
     destIFolder->SetHasNewMessages(PR_TRUE);
-
-  m_filterTargetFolders.AppendObject(destIFolder);
-
-  destFile->close();
-  delete destFile;
+  if (m_filterTargetFolders.IndexOf(destIFolder) == kNotFound)
+    m_filterTargetFolders.AppendObject(destIFolder);
   m_inboxFileStream->close();
-  // How are we going to do this with a stream?
-  nsresult truncRet = m_inboxFileSpec.Truncate(messageOffset);
+
+  nsresult truncRet = m_inboxFileSpec.Truncate(m_curHdrOffset);
   NS_ASSERTION(NS_SUCCEEDED(truncRet), "unable to truncate file");
   if (NS_FAILED(truncRet))
    destIFolder->ThrowAlertMsg("filterFolderTruncateFailed", msgWindow);
@@ -2017,8 +2312,7 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
     destIFolder->ReleaseSemaphore (myISupports);
   
   // tell parser that we've truncated the Inbox
-  mailHdr->GetMessageOffset(&messageOffset);
-  nsParseMailMessageState::Init(messageOffset);
+  nsParseMailMessageState::Init(m_curHdrOffset);
   
   (void) localFolder->RefreshSizeOnDisk();
   if (destIFolder)
