@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Dan Mosedale <dan.mosedale@oracle.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -71,6 +72,23 @@ pref("mailnews.headers.showUserAgent",       false);
 // is displayed in the message pane or not...
 pref("mailnews.headers.showOrganization",    false);
 
+// default sort order settings (when creating new folder views)
+// sort_order is an int value reflecting nsMsgViewSortOrder values
+//   as defined in nsIMsgDBView.idl (ascending = 1, descending = 2)
+// sort_type is an int value reflecting nsMsgViewSortType values
+//   as defined in nsIMsgDBView.idl (byDate = 18, byThread = 22 etc.)
+pref("mailnews.default_sort_order", 1); // for Mail/RSS/... (nsMsgDatabase)
+pref("mailnews.default_sort_type", 18); //
+pref("mailnews.default_news_sort_order", 1); // for News (nsNewsDatabase)
+pref("mailnews.default_news_sort_type", 22); //
+
+// If true, delete will use the direction of the sort order
+// in determining the next message to select. 
+pref("mail.delete_matches_sort_order", false);
+
+// mailnews tcp read+write timeout in seconds.
+pref("mailnews.tcptimeout", 60);
+
 // Mail server preferences, pop by default
 pref("mail.server_type",	0); 	// 0 pop, 1 imap,
 					// (Unix only:)
@@ -101,16 +119,18 @@ pref("mail.imap.mime_parts_on_demand",      true);
 pref("mail.imap.mime_parts_on_demand_max_depth", 15);
 pref("mail.imap.mime_parts_on_demand_threshold", 30000);
 pref("mail.imap.use_literal_plus",          true);
+pref("mail.imap.expunge_after_delete",      false);
 pref("mail.imap.check_deleted_before_expunge", false);
-pref("mail.thread_without_re",	            true);
+pref("mail.thread_without_re",	            true);  // if false, only thread by subject if Re:
+pref("mail.strict_threading",               false); // if true, don't thread by suject at all
 pref("mail.leave_on_server",                false);
 pref("mail.default_cc",                     "");
 pref("mail.default_fcc",                    ""); // maibox:URL or Imap://Host/OnLineFolderName
 pref("mail.check_new_mail",                 false);
 pref("mail.pop3_gets_new_mail",             false);
-pref("mail.pop3_response_timeout",          45); // in seconds
 pref("mail.check_time",                     10);
 pref("mail.pop_name",                       "");
+pref("mail.pop3.deleteFromServerOnMove",    false);
 pref("mail.remember_password",              false);
 pref("mail.pop_password",                   "");
 pref("mail.fixed_width_messages",           true);
@@ -127,10 +147,18 @@ pref("mail.use_fcc",                        true);
 pref("mail.cc_self",                        false);
 pref("mail.strictly_mime",                  false);
 pref("mail.strictly_mime_headers",          true);
+pref("mail.strictly_mime.parm_folding",     2);  // 0/1 (RFC 2047), 2(RFC 2231)
+pref("mail.label_ascii_only_mail_as_us_ascii", false); 
 pref("mail.file_attach_binary",             false);
 pref("mail.show_headers",                   1); // some
 pref("mail.pane_config",                    0);
 pref("mail.addr_book.mapit_url.format", "chrome://messenger-region/locale/region.properties");
+#ifdef MOZ_SUITE
+pref("mail.addr_book.im.onlineCheckAllowed", false);
+pref("messenger.throbber.url","chrome://messenger-region/locale/region.properties");
+pref("compose.throbber.url","chrome://messenger-region/locale/region.properties");
+pref("addressbook.throbber.url","chrome://messenger-region/locale/region.properties");
+#endif
 
 // the format for "mail.addr_book.quicksearchquery.format" is:
 // @V == the escaped value typed in the quick search bar in the addressbook
@@ -151,6 +179,8 @@ pref("mail.html_compose",                   true);
 // this will show up in the address picker in the compose window
 // examples: "X-Face" or "Approved,X-No-Archive"
 pref("mail.compose.other.header",	    "");
+pref("mail.compose.autosave", true);
+pref("mail.compose.autosaveinterval", 5); // in minutes
 pref("mail.fcc_folder",                     "");
 pref("mail.encrypt_outgoing_mail",          false);
 pref("mail.crypto_sign_outgoing_mail",      false);
@@ -167,6 +197,8 @@ pref("mail.receipt.request_header_type", 0);           // 0: MDN-DNT header  1:
 pref("mail.receipt.request_return_receipt_on", false);
 pref("mail.mdn.report.enabled", true);                 // false: Never send true: Send sometimes
 
+pref("mail.showPreviewText", false);
+
 pref("news.default_cc",                     "");
 pref("news.default_fcc",                    ""); // mailbox:URL or Imap://Host/OnlineFolderName
 pref("news.use_fcc",                        true);
@@ -194,8 +226,11 @@ pref("mailnews.reply_header_separator",     ", ");
 pref("mailnews.reply_header_colon",         ":");
 pref("mailnews.reply_header_originalmessage",   "chrome://messenger/locale/messengercompose/composeMsgs.properties");
 
-pref("mail.purge_threshhold",                100);
-pref("mail.prompt_purge_threshhold",             false);   
+pref("mailnews.reply_to_self_check_all_ident", false);
+
+pref("mail.purge_threshhold",              100);
+pref("mail.prompt_purge_threshhold",       false);
+pref("mail.purge.ask",                     true);
 
 pref("mailnews.offline_sync_mail",         false);
 pref("mailnews.offline_sync_news",         false);
@@ -259,13 +294,54 @@ pref("ldap_2.autoComplete.directoryServer", "");
 pref("ldap_2.servers.pab.position",								1);
 pref("ldap_2.servers.pab.description",							"chrome://messenger/locale/addressbook/addressBook.properties");
 pref("ldap_2.servers.pab.dirType",								2);
+pref("ldap_2.servers.pab.filename",                             "abook.mab");
 pref("ldap_2.servers.pab.isOffline",							false);
 
 pref("ldap_2.servers.history.position",							2);
 pref("ldap_2.servers.history.description",						"chrome://messenger/locale/addressbook/addressBook.properties");
 pref("ldap_2.servers.history.dirType",							2);
+pref("ldap_2.servers.history.filename",                         "history.mab");
 pref("ldap_2.servers.history.isOffline",						false);
 
+// default mapping of addressbook properties to ldap attributes
+pref("ldap_2.servers.default.attrmap.FirstName", "givenName");
+pref("ldap_2.servers.default.attrmap.LastName", "sn,surname");
+pref("ldap_2.servers.default.attrmap.DisplayName", "cn,commonname");
+pref("ldap_2.servers.default.attrmap.NickName", "mozillaNickname,xmozillanickname");
+pref("ldap_2.servers.default.attrmap.PrimaryEmail", "mail");
+pref("ldap_2.servers.default.attrmap.SecondEmail", "mozillaSecondEmail,xmozillasecondemail");
+pref("ldap_2.servers.default.attrmap.WorkPhone", "telephoneNumber");
+pref("ldap_2.servers.default.attrmap.HomePhone", "homePhone");
+pref("ldap_2.servers.default.attrmap.FaxNumber", "fax,facsimiletelephonenumber");
+pref("ldap_2.servers.default.attrmap.PagerNumber", "pager,pagerphone");
+pref("ldap_2.servers.default.attrmap.CellularNumber", "mobile,cellphone,carphone");
+pref("ldap_2.servers.default.attrmap.WorkAddress", "street,streetaddress,postOfficeBox");
+pref("ldap_2.servers.default.attrmap.HomeAddress", "mozillaHomeStreet");
+pref("ldap_2.servers.default.attrmap.WorkAddress2", "mozillaWorkStreet2");
+pref("ldap_2.servers.default.attrmap.HomeAddress2", "mozillaHomeStreet2");
+pref("ldap_2.servers.default.attrmap.WorkCity", "l,locality");
+pref("ldap_2.servers.default.attrmap.HomeCity", "mozillaHomeLocalityName");
+pref("ldap_2.servers.default.attrmap.WorkState", "st,region");
+pref("ldap_2.servers.default.attrmap.HomeState", "mozillaHomeState");
+pref("ldap_2.servers.default.attrmap.WorkZipCode", "postalCode,zip");
+pref("ldap_2.servers.default.attrmap.HomeZipCode", "mozillaHomePostalCode");
+pref("ldap_2.servers.default.attrmap.WorkCountry", "c,countryname");
+pref("ldap_2.servers.default.attrmap.HomeCountry", "mozillaHomeCountryName");
+pref("ldap_2.servers.default.attrmap.JobTitle", "title");
+pref("ldap_2.servers.default.attrmap.Department", "department,departmentnumber,ou,orgunit");
+pref("ldap_2.servers.default.attrmap.Company", "company,o");
+pref("ldap_2.servers.default.attrmap._AimScreenName", "nsAIMid,nscpaimscreenname");
+pref("ldap_2.servers.default.attrmap.WebPage1", "mozillaWorkUrl,workurl");
+pref("ldap_2.servers.default.attrmap.WebPage2", "mozillaHomeUrl,homeurl");
+pref("ldap_2.servers.default.attrmap.BirthYear", "birthyear");
+pref("ldap_2.servers.default.attrmap.Custom1", "mozillaCustom1,custom1");
+pref("ldap_2.servers.default.attrmap.Custom2", "mozillaCustom2,custom2");
+pref("ldap_2.servers.default.attrmap.Custom3", "mozillaCustom3,custom3");
+pref("ldap_2.servers.default.attrmap.Custom4", "mozillaCustom4,custom4");
+pref("ldap_2.servers.default.attrmap.Notes", "description,notes");
+pref("ldap_2.servers.default.attrmap.PreferMailFormat", "mozillaUseHtmlMail,xmozillausehtmlmail");
+pref("ldap_2.servers.default.attrmap.LastModifiedDate", "modifytimestamp");
+
 
 // A position of zero is a special value that indicates the directory is deleted.
 // These entries are provided to keep the (obsolete) Four11 directory and the
@@ -293,6 +369,7 @@ pref("ldap_2.prefs_migrated",      false);
 pref("mailnews.confirm.moveFoldersToTrash", true);
 
 pref("mailnews.reuse_message_window", true);
+pref("mailnews.reuse_thread_window2", false);
 pref("mailnews.open_window_warning", 10); // warn user if they attempt to open more than this many messages at once
 
 pref("mailnews.start_page.url", "chrome://messenger-region/locale/region.properties");
@@ -318,7 +395,8 @@ pref("mail.identity.default.compose_html", true);
 pref("mail.identity.default.valid", true);
 pref("mail.identity.default.fcc",true);
 pref("mail.identity.default.fcc_folder","mailbox://nobody@Local%20Folders/Sent");
-
+pref("mail.identity.default.fcc_reply_follows_parent", false);
+pref("mail.identity.default.autocompleteToMyDomain", false);
 
 // keep these defaults for backwards compatibility and migration
 
@@ -368,6 +446,7 @@ pref("mail.server.default.valid", true);
 pref("mail.server.default.abbreviate",true);
 pref("mail.server.default.isSecure", false);
 pref("mail.server.default.useSecAuth", false);
+pref("mail.server.default.socketType", 0);
 pref("mail.server.default.override_namespaces", true);
 pref("mail.server.default.deferred_to_account", "");
 
@@ -381,6 +460,11 @@ pref("mail.server.default.max_articles", 500);
 pref("mail.server.default.notify.on", true);
 pref("mail.server.default.mark_old_read", false);
 pref("mail.server.default.empty_trash_on_exit", false);
+// 0 = Keep Dupes, leave them alone
+// 1 = delete dupes
+// 2 = Move Dupes to trash
+// 3 = Mark Dupes as Read
+pref("mail.server.default.dup_action", 0);
 
 pref("mail.server.default.using_subscription", true);
 pref("mail.server.default.dual_use_folders", true);
@@ -398,22 +482,28 @@ pref("mail.server.default.use_idle", true);
 // for spam
 pref("mail.server.default.spamLevel",100);  // 0 off, 100 on.  not doing bool since we might have real levels one day.
 pref("mail.server.default.moveOnSpam",false);
-pref("mail.server.default.markAsReadOnSpam",false);
 pref("mail.server.default.moveTargetMode",0); // 0 == "Junk" on server, 1 == specific folder
 pref("mail.server.default.spamActionTargetAccount","");
 pref("mail.server.default.spamActionTargetFolder","");
 pref("mail.server.default.useWhiteList",true);
 pref("mail.server.default.whiteListAbURI","moz-abmdbdirectory://abook.mab");  // the Personal addressbook.
+pref("mail.server.default.useServerFilter", false);
+pref("mail.server.default.serverFilterName", "SpamAssasin");
+pref("mail.server.default.serverFilterTrustFlags", 1); // 1 == trust positives, 2 == trust negatives, 3 == trust both
 pref("mail.server.default.purgeSpam",false);
 pref("mail.server.default.purgeSpamInterval",14); // 14 days
-pref("mail.server.default.spamLoggingEnabled",false);
-pref("mail.server.default.manualMark",false);
-pref("mail.server.default.manualMarkMode",0); // 0 == "move to junk folder", 1 == "delete"
 
 // the probablilty threshold over which messages are classified as junk
 // this number is divided by 100 before it is used. The classifier can be fine tuned
 // by changing this pref. Typical values are .99, .95, .90, .5, etc. 
 pref("mail.adaptivefilters.junk_threshold", 90); 
+pref("mail.spam.version", 0); // used to determine when to migrate global spam settings
+pref("mail.spam.logging.enabled", false);
+pref("mail.spam.manualMark", false);
+pref("mail.spam.markAsReadOnSpam", false);
+pref("mail.spam.manualMarkMode", 0); // 0 == "move to junk folder", 1 == "delete"
+
+pref("mail.autoComplete.highlightNonMatches", true);
 
 // if true, we'll use the password from an incoming server with
 // matching username and domain
@@ -451,12 +541,6 @@ pref("mail.forward_message_mode", 0); // 0=default as attachment 2=forward as in
 
 pref("mail.startup.enabledMailCheckOnce", false);
 
-pref("mailnews.max_header_display_length",3); // number of addresses to show
-
-pref("messenger.throbber.url","chrome://messenger-region/locale/region.properties");
-pref("compose.throbber.url","chrome://messenger-region/locale/region.properties");
-pref("addressbook.throbber.url","chrome://messenger-region/locale/region.properties");
-
 pref("mailnews.send_plaintext_flowed", true); // RFC 2646=======
 pref("mailnews.display.disable_format_flowed_support", false);
 pref("mailnews.nav_crosses_folders", 1); // prompt user when crossing folders
@@ -465,6 +549,8 @@ pref("mailnews.nav_crosses_folders", 1); // prompt user when crossing folders
 pref("news.cancel.confirm",true);
 pref("news.cancel.alert_on_success",true);
 pref("mail.SpellCheckBeforeSend",false);
+pref("mail.spellcheck.inline",true);
+pref("mail.phishing.detection.enabled", true); // enable / disable phishing detection for link clicks
 pref("mail.warn_on_send_accel_key", true);
 pref("mail.enable_autocomplete",true);
 pref("mailnews.html_domains","");
@@ -494,6 +580,8 @@ pref("mail.biff.play_sound.type",0);
 // otherwise, this needs to be a file url
 pref("mail.biff.play_sound.url","");
 pref("mail.biff.show_alert", true);
+pref("mail.biff.show_tray_icon", true); // currently Windows-only
+pref("mail.biff.animate_dock_icon", false);
 
 pref("mail.content_disposition_type", 0);
 
@@ -501,6 +589,7 @@ pref("mailnews.show_send_progress", true); //Will show a progress dialog when sa
 pref("mail.server.default.retainBy", 1);
 
 pref("mailnews.ui.junk.firstuse", true);
+pref("mailnews.ui.junk.manualMarkAsJunkMarksRead", true);
 
 // for manual upgrades of certain UI features.
 // 1 -> 2 is for the folder pane tree landing, to hide the
@@ -535,24 +624,13 @@ pref("mailnews.ui.advanced_directory_search_results.version", 1);
 //If set to a number greater than 0, msg compose windows will be recycled in order to open them quickly
 pref("mail.compose.max_recycled_windows", 1); 
 
-// true makes it so we persist the open state of news server when starting up the 3 pane
-// this is costly, as it might result in network activity.
-// false makes it so we act like 4.x
-// see bug #103010 for details
-pref("news.persist_server_open_state_in_folderpane",false);
-
-// New color prefs for Labels
-pref("mailnews.labels.description.0", "chrome://messenger/locale/messenger.properties");
+// default description and color prefs for tags
+// (we keep the .labels. names for backwards compatibility)
 pref("mailnews.labels.description.1", "chrome://messenger/locale/messenger.properties");
 pref("mailnews.labels.description.2", "chrome://messenger/locale/messenger.properties");
 pref("mailnews.labels.description.3", "chrome://messenger/locale/messenger.properties");
 pref("mailnews.labels.description.4", "chrome://messenger/locale/messenger.properties");
 pref("mailnews.labels.description.5", "chrome://messenger/locale/messenger.properties");
-
-// mailews.labels.color.0 is not defined because there is no color associated with
-// this particular label.  It is defined above (mailnews.lables.description.0) because
-// its description string is required in order to prepend the accesskey '0':
-//   ie: '0 None'.
 pref("mailnews.labels.color.1", "#FF0000"); // default: red
 pref("mailnews.labels.color.2", "#FF9900"); // default: orange
 pref("mailnews.labels.color.3", "#009900"); // default: green
@@ -567,8 +645,10 @@ pref("mailnews.fakeaccount.show", false);
 pref("mailnews.fakeaccount.server", "");
 
 // message display properties
-pref("mailnews.message_display.disable_remote_image", false);
-pref("mailnews.message_display.allow.plugins", true);
+pref("mailnews.message_display.allow.plugins", false);
+pref("mailnews.message_display.disable_remote_image", true);
+pref("mailnews.message_display.disable_remote_images.useWhitelist", true);
+pref("mailnews.message_display.disable_remote_images.whiteListAbURI","moz-abmdbdirectory://abook.mab");
 
 // default msg compose font prefs
 pref("msgcompose.font_face",                "");
@@ -580,6 +660,10 @@ pref("msgcompose.background_color",         "#FFFFFF");
 // to prevent some mail server to disclose the bcc recipients
 pref("mail.compose.add_undisclosed_recipients", true);
 
+// Set this preference to true to tell mail not to attach the source of a link to a local
+// network file (file://///<network name>/<path>/<file name>). Windows only
+pref("mail.compose.dont_attach_source_of_local_network_links", false);
+
 // these prefs (in minutes) are here to help QA test this feature
 // "mail.purge.min_delay", never purge a junk folder more than once every 480 minutes (60 mins/hour * 8 hours)
 // "mail.purge.timer_interval", fire the purge timer every 5 minutes, starting 5 minutes after we load accounts
@@ -596,11 +680,9 @@ pref("mail.password_protect_local_cache", false);
 // overridden by stand alone mail
 pref("mail.standalone", false);
 
-#ifdef XP_WIN
-// Unread mail count timer. Value to be specified in seconds
-// default is 5 minutes, i.e., 5 * 60 seconds = 300
-pref("mail.windows_xp_integration.unread_count_interval", 300);
-#endif
+// import option to skip the first record, recorded so that we can save
+// the users last used preference.
+pref("mailnews.import.text.skipfirstrecord", true);
 
 #ifdef XP_MACOSX
 pref("mail.notification.sound",             "");
@@ -628,8 +710,6 @@ pref("news.show_first_unread", false);
 pref("news.sash_geometry", "");
 pref("mail.signature_date", 0);
 
-// until bug #130581 is fixed, we need to override this on linux
-pref("mail.compose.max_recycled_windows", 0);
 # XP_UNIX
 #endif
 #endif
