@@ -39,8 +39,6 @@
 
 #define MAPI_STARTUP_ARG       "/MAPIStartUp"
 
-#define MAPI_STARTUP_ARG       "/MAPIStartUp"
-
 #ifdef MOZ_LOGGING
 // this has to be before the pre-compiled header
 #define FORCE_PR_LOG /* Allow logging in the release build */
@@ -48,24 +46,24 @@
 #include <mapidefs.h>
 #include <mapi.h>
 #include <tchar.h>
-
+#include <direct.h>
 #include "nsCOMPtr.h"
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
 #include "nsISupports.h"
 #include "nsIPromptService.h"
-#include "nsIAppStartup.h"
+#include "nsIAppStartup.h" 
 #include "nsIAppShellService.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsINativeAppSupport.h"
-#include "nsICmdLineService.h"
 #include "nsIMsgAccountManager.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "nsMsgBaseCID.h"
 #include "nsIStringBundle.h"
-#include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 #include "nsString.h"
 #include "nsUnicharUtils.h"
 #include "nsIMsgAttachment.h"
@@ -73,7 +71,7 @@
 #include "nsIMsgComposeParams.h"
 #include "nsIMsgCompose.h"
 #include "nsMsgCompCID.h"
-#include "nsXPFEComponentsCID.h"
+#include "nsXPFEComponentsCID.h" 
 #include "nsIMsgSend.h"
 #include "nsIProxyObjectManager.h"
 #include "nsIMsgComposeService.h"
@@ -85,10 +83,16 @@
 #include "msgMapiHook.h"
 #include "msgMapiSupport.h"
 #include "msgMapiMain.h"
+#include "nsThreadUtils.h"
 #include "nsNetUtil.h"
 
-extern PRLogModuleInfo *MAPI;
+#include "nsEmbedCID.h"
 
+#ifndef MOZ_XUL_APP
+#include "nsICmdLineService.h"
+#endif
+
+extern PRLogModuleInfo *MAPI;
 
 class nsMAPISendListener : public nsIMsgSendListener
 {
@@ -158,24 +162,22 @@ PRBool nsMapiHook::isMapiService = PR_FALSE;
 
 PRBool nsMapiHook::Initialize()
 {
-#ifndef MOZ_THUNDERBIRD
+// XXX test for this as long as there are still non-xul-app suite builds
+#ifndef MOZ_XUL_APP
     nsresult rv;
     nsCOMPtr<nsINativeAppSupport> native;
-
-    nsCOMPtr<nsICmdLineService> cmdLineArgs (
-      do_GetService(NS_COMMANDLINESERVICE_CONTRACTID, &rv));
-    if (NS_FAILED(rv)) return PR_FALSE;
+    nsCOMPtr<nsICmdLineService> cmdLineArgs (do_GetService(NS_COMMANDLINESERVICE_CONTRACTID, &rv));
+    if (NS_FAILED(rv)) return PR_FALSE; 
 
     nsCOMPtr<nsIAppStartup> appStartup (do_GetService(NS_APPSTARTUP_CONTRACTID, &rv));
-    if (NS_FAILED(rv)) return PR_FALSE;
+    if (NS_FAILED(rv)) return PR_FALSE; 
 
-    rv = appStartup->GetNativeAppSupport(getter_AddRefs(native));
+    rv = appStartup->GetNativeAppSupport(getter_AddRefs(native)); 
     if (NS_FAILED(rv)) return PR_FALSE;
 
     rv = native->EnsureProfile(cmdLineArgs);
     if (NS_FAILED(rv)) return PR_FALSE;
 #endif
-
     return PR_TRUE;
 }
 
@@ -190,8 +192,8 @@ PRBool nsMapiHook::DisplayLoginDialog(PRBool aLogin, PRUnichar **aUsername,
 {
     nsresult rv;
     PRBool btnResult = PR_FALSE;
-
-    nsCOMPtr<nsIPromptService> dlgService(do_GetService("@mozilla.org/embedcomp/prompt-service;1", &rv));
+   
+    nsCOMPtr<nsIPromptService> dlgService(do_GetService(NS_PROMPTSERVICE_CONTRACTID, &rv));
     if (NS_SUCCEEDED(rv) && dlgService)
     {
         nsCOMPtr<nsIStringBundleService> bundleService(do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv));
@@ -203,13 +205,13 @@ PRBool nsMapiHook::DisplayLoginDialog(PRBool aLogin, PRUnichar **aUsername,
 
         nsCOMPtr<nsIStringBundle> brandBundle;
         rv = bundleService->CreateBundle(
-                        "chrome://global/locale/brand.properties",
+                        "chrome://branding/locale/brand.properties",
                         getter_AddRefs(brandBundle));
         if (NS_FAILED(rv)) return PR_FALSE;
 
         nsXPIDLString brandName;
         rv = brandBundle->GetStringFromName(
-                           NS_LITERAL_STRING("brandShortName").get(),
+                           NS_LITERAL_STRING("brandFullName").get(),
                            getter_Copies(brandName));
         if (NS_FAILED(rv)) return PR_FALSE;
         
@@ -286,7 +288,7 @@ PRBool nsMapiHook::VerifyUserName(const PRUnichar *aUsername, char **aIdKey)
             if (index != -1)
                 aEmail.Truncate(index);
 
-			if (nsDependentString(aUsername) == NS_ConvertASCIItoUCS2(aEmail))  // == overloaded
+			if (nsDependentString(aUsername) == NS_ConvertASCIItoUTF16(aEmail))  // == overloaded
                 return NS_SUCCEEDED(thisIdentity->GetKey(aIdKey));
         }
     }
@@ -299,10 +301,10 @@ nsMapiHook::IsBlindSendAllowed()
 {
     PRBool enabled = PR_FALSE;
     PRBool warn = PR_TRUE;
-    nsCOMPtr<nsIPref> prefs = do_GetService(NS_PREF_CONTRACTID);
-    if (prefs) {
-        prefs->GetBoolPref(PREF_MAPI_WARN_PRIOR_TO_BLIND_SEND,&warn);
-            prefs->GetBoolPref(PREF_MAPI_BLIND_SEND_ENABLED,&enabled);
+    nsCOMPtr<nsIPrefBranch> prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID);
+    if (prefBranch) {
+        prefBranch->GetBoolPref(PREF_MAPI_WARN_PRIOR_TO_BLIND_SEND, &warn);
+        prefBranch->GetBoolPref(PREF_MAPI_BLIND_SEND_ENABLED, &enabled);
     } 
     if (!enabled)
         return PR_FALSE;
@@ -328,18 +330,17 @@ nsMapiHook::IsBlindSendAllowed()
                                         getter_Copies(dontShowAgainMessage));
     if (NS_FAILED(rv)) return PR_FALSE;
 
-    nsCOMPtr<nsIPromptService> dlgService(do_GetService("@mozilla.org/embedcomp/prompt-service;1", &rv));
+    nsCOMPtr<nsIPromptService> dlgService(do_GetService(NS_PROMPTSERVICE_CONTRACTID, &rv));
     if (NS_FAILED(rv) || !dlgService) return PR_FALSE;
     
     PRBool continueToWarn = PR_TRUE;
     PRBool okayToContinue = PR_FALSE;
     dlgService->ConfirmCheck(nsnull, nsnull, warningMsg, dontShowAgainMessage, &continueToWarn, &okayToContinue);
         
-    if (!continueToWarn && okayToContinue && prefs)
-        prefs->SetBoolPref(PREF_MAPI_WARN_PRIOR_TO_BLIND_SEND,PR_FALSE);
+    if (!continueToWarn && okayToContinue && prefBranch)
+        prefBranch->SetBoolPref(PREF_MAPI_WARN_PRIOR_TO_BLIND_SEND, PR_FALSE);
     
     return okayToContinue;
-        
 }
 
 // this is used for Send without UI
@@ -387,8 +388,10 @@ nsresult nsMapiHook::BlindSendMail (unsigned long aSession, nsIMsgCompFields * a
     if (NS_FAILED(rv) || (!pMsgComposeParams) ) return rv ;
 
     // populate the compose params
+    PRBool forcePlainText;
+    aCompFields->GetForcePlainText(&forcePlainText);
     pMsgComposeParams->SetType(nsIMsgCompType::New);
-    pMsgComposeParams->SetFormat(nsIMsgCompFormat::Default);
+    pMsgComposeParams->SetFormat(forcePlainText ? nsIMsgCompFormat::PlainText : nsIMsgCompFormat::HTML);
     pMsgComposeParams->SetIdentity(pMsgId);
     pMsgComposeParams->SetComposeFields(aCompFields); 
     pMsgComposeParams->SetSendListener(sendListener) ;
@@ -411,15 +414,13 @@ nsresult nsMapiHook::BlindSendMail (unsigned long aSession, nsIMsgCompFields * a
 
     // we need to wait here to make sure that we return only after send is completed
     // so we will have a event loop here which will process the events till the Send IsDone.
-    nsCOMPtr<nsIEventQueueService> pEventQService = do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID, &rv);
-    nsCOMPtr<nsIEventQueue> eventQueue;
-    pEventQService->GetThreadEventQueue(NS_CURRENT_THREAD,getter_AddRefs(eventQueue));
+    nsIThread *thread = NS_GetCurrentThread();
     while ( !((nsMAPISendListener *) pSendListener)->IsDone() )
     {
         PR_CEnterMonitor(pSendListener);
         PR_CWait(pSendListener, PR_MicrosecondsToInterval(1000UL));
         PR_CExitMonitor(pSendListener);
-        eventQueue->ProcessPendingEvents();
+        NS_ProcessPendingEvents(thread);
     }
 
     return rv ;
@@ -478,7 +479,7 @@ nsresult nsMapiHook::PopulateCompFields(lpnsMapiMessage aMessage,
         }
     }
 
-    PR_LOG(MAPI, PR_LOG_DEBUG, ("to: %s cc: %s bcc: %s \n", NS_ConvertUCS2toUTF8(To).get(), NS_ConvertUCS2toUTF8(Cc).get(), NS_ConvertUCS2toUTF8(Bcc).get()));
+    PR_LOG(MAPI, PR_LOG_DEBUG, ("to: %s cc: %s bcc: %s \n", NS_ConvertUTF16toUTF8(To).get(), NS_ConvertUTF16toUTF8(Cc).get(), NS_ConvertUTF16toUTF8(Bcc).get()));
     // set To, Cc, Bcc
     aCompFields->SetTo (To) ;
     aCompFields->SetCc (Cc) ;
@@ -503,7 +504,11 @@ nsresult nsMapiHook::PopulateCompFields(lpnsMapiMessage aMessage,
         nsString Body;
         Body.AssignWithConversion(aMessage->lpszNoteText);
         if (Body.Last() != nsCRT::LF)
-          Body.AppendLiteral(CRLF);
+          Body.AppendLiteral(CRLF); 
+
+        if (Body.Find("<html>") == kNotFound)
+          aCompFields->SetForcePlainText(PR_TRUE);
+
         rv = aCompFields->SetBody(Body) ;
     }
 
@@ -569,10 +574,17 @@ nsresult nsMapiHook::HandleAttachments (nsIMsgCompFields * aCompFields, PRInt32
             // a value for lpszFileName, use it. Otherwise stick with leafName
             if (aFiles[i].lpszFileName)
             {
+              nsAutoString wholeFileName;
                 if (aIsUnicode)
-                    leafName.Assign(aFiles[i].lpszFileName);
+                    wholeFileName.Assign(aFiles[i].lpszFileName);
                 else
-                    ConvertToUnicode(nsMsgI18NFileSystemCharset(), (char *) aFiles[i].lpszFileName, leafName);
+                    ConvertToUnicode(nsMsgI18NFileSystemCharset(), (char *) aFiles[i].lpszFileName, wholeFileName);
+                // need to find the last '\' and find the leafname from that.
+                PRInt32 lastSlash = wholeFileName.RFindChar(PRUnichar('\\'));
+                if (lastSlash != kNotFound)
+                  wholeFileName.Right(leafName, wholeFileName.Length() - lastSlash - 1);
+                else
+                  leafName.Assign(wholeFileName);
             }
             else 
               pFile->GetLeafName (leafName);
@@ -588,7 +600,6 @@ nsresult nsMapiHook::HandleAttachments (nsIMsgCompFields * aCompFields, PRInt32
              {
                rv = pTempFile->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0777);
                NS_ENSURE_SUCCESS(rv, rv);
-               pTempFile->GetLeafName(leafName);
                pTempFile->Remove(PR_FALSE); // remove so we can copy over it.
              }
             // copy the file to its new location and file name
@@ -676,7 +687,7 @@ nsresult nsMapiHook::PopulateCompFieldsWithConversion(lpnsMapiMessage aMessage,
     aCompFields->SetCc (Cc) ;
     aCompFields->SetBcc (Bcc) ;
 
-    PR_LOG(MAPI, PR_LOG_DEBUG, ("to: %s cc: %s bcc: %s \n", NS_ConvertUCS2toUTF8(To).get(), NS_ConvertUCS2toUTF8(Cc).get(), NS_ConvertUCS2toUTF8(Bcc).get()));
+    PR_LOG(MAPI, PR_LOG_DEBUG, ("to: %s cc: %s bcc: %s \n", NS_ConvertUTF16toUTF8(To).get(), NS_ConvertUTF16toUTF8(Cc).get(), NS_ConvertUTF16toUTF8(Bcc).get()));
 
     nsCAutoString platformCharSet;
     // set subject
@@ -703,7 +714,11 @@ nsresult nsMapiHook::PopulateCompFieldsWithConversion(lpnsMapiMessage aMessage,
         rv = ConvertToUnicode(platformCharSet.get(), (char *) aMessage->lpszNoteText, Body);
         if (NS_FAILED(rv)) return rv ;
         if (Body.Last() != nsCRT::LF)
-          Body.AppendLiteral(CRLF);
+          Body.AppendLiteral(CRLF); 
+
+        if (Body.Find("<html>") == kNotFound)
+          aCompFields->SetForcePlainText(PR_TRUE);
+
         rv = aCompFields->SetBody(Body) ;
     }
 
@@ -767,21 +782,21 @@ nsresult nsMapiHook::PopulateCompFieldsForSendDocs(nsIMsgCompFields * aCompField
         PRUnichar * newFilePaths = (PRUnichar *) strFilePaths.get() ;
         while (offset != kNotFound)
         {
-          //Temp Directory
-          nsCOMPtr <nsIFile> pTempFileDir;
-          NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(pTempFileDir));
+            //Temp Directory
+            nsCOMPtr <nsIFile> pTempFileDir;
+            NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(pTempFileDir));
+            nsCOMPtr <nsILocalFile> pTempDir = do_QueryInterface(pTempFileDir);
 
-          nsCOMPtr <nsILocalFile> pTempDir = do_QueryInterface(pTempFileDir);
+            // if not already existing, create another temp dir for mapi within Win temp dir
+            // this is windows only so we can do "\\"
+            pTempDir->AppendRelativePath (NS_LITERAL_STRING("moz_mapi"));
+            pTempDir->Exists(&bExist) ;
+            if (!bExist)
+            {
+                rv = pTempDir->Create(nsIFile::DIRECTORY_TYPE, 777) ;
+                if (NS_FAILED(rv)) return rv ;
+            }
 
-          // if not already existing, create another temp dir for mapi within Win temp dir
-          // this is windows only so we can do "\\"
-          pTempDir->AppendRelativePath (NS_LITERAL_STRING("moz_mapi"));
-          pTempDir->Exists(&bExist) ;
-          if (!bExist)
-          {
-              rv = pTempDir->Create(nsIFile::DIRECTORY_TYPE, 777) ;
-              if (NS_FAILED(rv)) return rv ;
-          }
             nsString RemainingPaths ;
             RemainingPaths.Assign(newFilePaths) ;
             offset = RemainingPaths.Find (strDelimChars) ;
@@ -795,7 +810,20 @@ nsresult nsMapiHook::PopulateCompFieldsForSendDocs(nsIMsgCompFields * aCompField
                 FilePathsLen -= offset + strDelimChars.Length();
             }
 
+            if (RemainingPaths[1] != ':' && RemainingPaths[1] != '\\') 
+            {
+              char cwd[MAX_PATH];  
+              if (_getdcwd(_getdrive(), cwd, MAX_PATH)) 
+              {
+                nsAutoString cwdStr;
+                CopyASCIItoUTF16(cwd, cwdStr);
+                cwdStr.Append('\\');
+                RemainingPaths.Insert(cwdStr, 0);
+              }
+            }
+
             pFile->InitWithPath (RemainingPaths) ;
+
             rv = pFile->Exists(&bExist) ;
             if (NS_FAILED(rv) || (!bExist) ) return NS_ERROR_FILE_TARGET_DOES_NOT_EXIST ;
 
@@ -852,6 +880,9 @@ nsresult nsMapiHook::ShowComposerWindow (unsigned long aSession, nsIMsgCompField
     nsCOMPtr<nsIMsgComposeParams> pMsgComposeParams (do_CreateInstance(NS_MSGCOMPOSEPARAMS_CONTRACTID, &rv));
     if (NS_FAILED(rv) || (!pMsgComposeParams) ) return rv ;
 
+    PRBool forcePlainText;
+    aCompFields->GetForcePlainText(&forcePlainText);
+    pMsgComposeParams->SetFormat(forcePlainText ? nsIMsgCompFormat::Default : nsIMsgCompFormat::HTML);
     // populate the compose params
     pMsgComposeParams->SetType(nsIMsgCompType::New);
     pMsgComposeParams->SetFormat(nsIMsgCompFormat::Default);
