@@ -64,7 +64,6 @@
 #include "nsMsgMimeCID.h"
 #include "nsDateTimeFormatCID.h"
 
-static NS_DEFINE_CID(kDateTimeFormatCID,    NS_DATETIMEFORMAT_CID);
 #define VIEW_ALL_HEADERS 2
 
 /*
@@ -188,8 +187,10 @@ nsresult nsMimeHtmlDisplayEmitter::BroadcastHeaders(nsIMsgHeaderSink * aHeaderSi
           nsCRT::strcasecmp("bcc", headerInfo->name) && nsCRT::strcasecmp("followup-to", headerInfo->name) &&
           nsCRT::strcasecmp("reply-to", headerInfo->name) && nsCRT::strcasecmp("subject", headerInfo->name) &&
           nsCRT::strcasecmp("organization", headerInfo->name) && nsCRT::strcasecmp("user-agent", headerInfo->name) &&
-          nsCRT::strcasecmp("content-base", headerInfo->name) && 
-          nsCRT::strcasecmp("date", headerInfo->name) && nsCRT::strcasecmp("x-mailer", headerInfo->name))
+          nsCRT::strcasecmp("content-base", headerInfo->name) && nsCRT::strcasecmp("sender", headerInfo->name) &&
+          nsCRT::strcasecmp("date", headerInfo->name) && nsCRT::strcasecmp("x-mailer", headerInfo->name) &&
+          nsCRT::strcasecmp("content-type", headerInfo->name) && nsCRT::strcasecmp("message-id", headerInfo->name) &&
+          nsCRT::strcasecmp("x-newsreader", headerInfo->name) && nsCRT::strcasecmp("x-mimeole", headerInfo->name))
             continue;
     }
 
@@ -272,7 +273,7 @@ nsresult nsMimeHtmlDisplayEmitter::GenerateDateString(const char * dateString, n
   nsresult rv = NS_OK;
 
   if (!mDateFormater) {
-    mDateFormater = do_CreateInstance(kDateTimeFormatCID, &rv);
+    mDateFormater = do_CreateInstance(NS_DATETIMEFORMAT_CONTRACTID, &rv);
     if (NS_FAILED(rv))
       return rv;
   }
@@ -344,6 +345,8 @@ nsMimeHtmlDisplayEmitter::EndHeader()
       {
         PRInt32 bufLen = strlen(subject) + 16;
         char *buf = new char[bufLen];
+        if (!buf)
+          return NS_ERROR_OUT_OF_MEMORY;
         PR_snprintf(buf, bufLen, "<title>%s</title>", subject);
         UtilityWriteCRLF(buf);
         delete [] buf;
@@ -367,7 +370,7 @@ nsresult
 nsMimeHtmlDisplayEmitter::StartAttachment(const char *name,
                                           const char *contentType,
                                           const char *url,
-                                          PRBool aNotDownloaded)
+                                          PRBool aIsExternalAttachment)
 {
   nsresult rv = NS_OK;
   nsCOMPtr<nsIMsgHeaderSink> headerSink; 
@@ -411,7 +414,7 @@ nsMimeHtmlDisplayEmitter::StartAttachment(const char *name,
 
     headerSink->HandleAttachment(contentType, url /* was escapedUrl */,
                                  unicodeHeaderValue, uriString,
-                                 aNotDownloaded);
+                                 aIsExternalAttachment);
 
     nsCRT::free(escapedUrl);
     mSkipAttachment = PR_TRUE;
