@@ -56,7 +56,8 @@
 #include "plstr.h"
 #include "prprf.h"
 #include "prio.h"
-#include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 #include "msgCore.h"
 #include "nsCRT.h"
 #include "nsEscape.h"
@@ -70,7 +71,8 @@
 #include "nsMsgCompCID.h"
 #include "nsIMsgComposeService.h"
 #include "nsMsgI18N.h"
-#include "nsSpecialSystemDirectory.h"
+#include "nsNativeCharsetUtils.h"
+#include "nsDirectoryServiceDefs.h"
 #include "nsIMsgMessageService.h"
 #include "nsMsgUtils.h"
 #include "nsXPIDLString.h"
@@ -85,7 +87,7 @@
 // Header strings...
 //
 #define HEADER_NNTP_POSTING_HOST      "NNTP-Posting-Host"
-#define MIME_HEADER_TABLE             "<TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0>"
+#define MIME_HEADER_TABLE             "<TABLE CELLPADDING=0 CELLSPACING=0 BORDER=0 class=\"moz-email-headers-table\">"
 #define HEADER_START_JUNK             "<TR><TH VALIGN=BASELINE ALIGN=RIGHT NOWRAP>"
 #define HEADER_MIDDLE_JUNK            ": </TH><TD>"
 #define HEADER_END_JUNK               "</TD></TR>"
@@ -107,8 +109,6 @@ static NS_DEFINE_CID(kCMsgComposeServiceCID,  NS_MSGCOMPOSESERVICE_CID);
 // THIS SHOULD ALL MOVE TO ANOTHER FILE AFTER LANDING!
 ////////////////////////////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////////
-// Define CIDs...
-static NS_DEFINE_CID(kPrefCID,                NS_PREF_CID);
 
 // safe filename for all OSes
 #define SAFE_TMP_FILENAME "nsmime.tmp"
@@ -121,19 +121,30 @@ nsFileSpec *
 nsMsgCreateTempFileSpec(const char *tFileName)
 {
   //Calling NS_MsgHashIfNecessary so that when Replies are forwarded - the ':' in the subject line doesn't cause problems
-  nsFileSpec *tmpSpec = new nsFileSpec(nsSpecialSystemDirectory(nsSpecialSystemDirectory::OS_TemporaryDirectory));
-  NS_ASSERTION(tmpSpec, "out of memory");
+  nsCOMPtr<nsIFile> tmpFile;
+  nsresult rv = NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(tmpFile));
+  if (NS_FAILED(rv))
+    return nsnull;
+
+  nsCOMPtr<nsIFileSpec> tmpFileSpec;
+  rv = NS_NewFileSpecFromIFile(tmpFile, getter_AddRefs(tmpFileSpec));
+
+  if (NS_FAILED(rv))
+    return nsnull;
+  
+  nsFileSpec *tmpSpec = new nsFileSpec;
+  tmpFileSpec->GetFileSpec(tmpSpec);
   if (!tmpSpec)
     return nsnull;
 
-  nsresult rv = NS_OK;
   nsCAutoString tempName;
   if ((!tFileName) || (!*tFileName)) {
     tempName = SAFE_TMP_FILENAME;
   }
   else {
     nsAutoString tempNameUni; 
-    if (NS_FAILED(nsMsgI18NCopyNativeToUTF16(tFileName, tempNameUni))) {
+    if (NS_FAILED(NS_CopyNativeToUnicode(nsDependentCString(tFileName),
+                                         tempNameUni))) {
       tempName = SAFE_TMP_FILENAME;
       goto fallback;
     }
@@ -162,7 +173,7 @@ nsMsgCreateTempFileSpec(const char *tFileName)
         }
       }
     } 
-    rv = nsMsgI18NCopyUTF16ToNative(tempNameUni, tempName);
+    rv = NS_CopyUnicodeToNative(tempNameUni, tempName);
     NS_ASSERTION(NS_SUCCEEDED(rv), "UTF-16 to native filename failed"); 
   }
 
@@ -384,8 +395,10 @@ CreateCompositionFields(const char        *from,
   }
   
   if (newsgroups) {
+    // fixme: the newsgroups header had better be decoded using the server-side
+    // character encoding,but this |charset| might be different from it.
     val = MIME_DecodeMimeHeader(newsgroups, charset, PR_FALSE, PR_TRUE);
-    cFields->SetNewsgroups(val ? val : newsgroups);
+    cFields->SetNewsgroups(NS_ConvertUTF8toUTF16(val ? val : newsgroups));
     PR_FREEIF(val);
   }
   
@@ -437,12 +450,12 @@ PRBool
 GetMailXlateionPreference(void)
 {
   nsresult res;
-  PRBool   xlate = PR_FALSE; 
+  PRBool   xlate = PR_FALSE;
+
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &res));
+  if(NS_SUCCEEDED(res))
+    prefBranch->GetBoolPref("mail.unknown", &xlate);
 
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res)); 
-  if (NS_SUCCEEDED(res) && prefs)
-    res = prefs->GetBoolPref("mail.unknown", &xlate);
-  
   return xlate;
 }
 
@@ -454,9 +467,9 @@ GetMailSigningPreference(void)
   nsresult  res;
   PRBool    signit = PR_FALSE;
 
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res)); 
-  if (NS_SUCCEEDED(res) && prefs)
-    res = prefs->GetBoolPref("mail.unknown", &signit);
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &res));
+  if(NS_SUCCEEDED(res))
+    prefBranch->GetBoolPref("mail.unknown", &signit);
 
   return signit;
 }
@@ -598,8 +611,15 @@ mime_draft_process_attachments(mime_draft_data *mdd)
       {
         if (tmpFile->real_name)
           NS_MsgSACopy ( &(tmp->real_name), tmpFile->real_name );
-        else
-          NS_MsgSACopy ( &(tmp->real_name), tmpSpec.get() );
+        else {
+          if (PL_strstr(tmpFile->type, MESSAGE_RFC822))
+            // we have the odd case of processing an e-mail that had an unnamed
+            // eml message attached
+            NS_MsgSACopy( &(tmp->real_name), "ForwardedMessage.eml" );
+
+          else
+            NS_MsgSACopy ( &(tmp->real_name), tmpSpec.get() );
+        }
       }
     }
 
@@ -672,9 +692,9 @@ mime_fix_up_html_address( char **addr)
 static void 
 mime_intl_insert_message_header_1(char        **body, 
                                   char        **hdr_value,
-                                  char        *hdr_str, 
+                                  const char  *hdr_str, 
                                   const char  *html_hdr_str,
-                                  char        *mailcharset,
+                                  const char  *mailcharset,
                                   PRBool      htmlEdit)
 {
   if (!body || !hdr_value || !hdr_str)
@@ -1210,9 +1230,9 @@ mime_insert_forwarded_message_headers(char            **body,
   PRInt32     show_headers = 0;
   nsresult    res;
 
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res)); 
-  if (NS_SUCCEEDED(res) && prefs)
-    res = prefs->GetIntPref("mail.show_headers", &show_headers);
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &res));
+  if (NS_SUCCEEDED(res))
+    prefBranch->GetIntPref("mail.show_headers", &show_headers);
 
   switch (show_headers)
   {
@@ -1389,7 +1409,8 @@ mime_parse_stream_complete (nsMIMESession *stream)
       char *parm = 0;
       parm = MimeHeaders_get_parameter(draftInfo, "vcard", NULL, NULL);
       fields->SetAttachVCard(parm && !nsCRT::strcmp(parm, "1"));
-      
+   
+      fields->SetMessageId(id); // keep same message id for editing template.
       PR_FREEIF(parm);
       parm = MimeHeaders_get_parameter(draftInfo, "receipt", NULL, NULL);
       if (parm && !nsCRT::strcmp(parm, "0"))
@@ -1541,7 +1562,7 @@ mime_parse_stream_complete (nsMIMESession *stream)
           mdd->mailcharset);
       }
       // setting the charset while we are creating the composition fields
-      //fields->SetCharacterSet(NS_ConvertASCIItoUCS2(mdd->mailcharset));
+      //fields->SetCharacterSet(NS_ConvertASCIItoUTF16(mdd->mailcharset));
 
       // convert from UTF-8 to UTF-16
       if (body)
@@ -1560,6 +1581,7 @@ mime_parse_stream_complete (nsMIMESession *stream)
 #ifdef NS_DEBUG
         printf("RICHIE: Time to create the EDITOR with this template - HAS a body!!!!\n");
 #endif
+        fields->SetDraftId(mdd->url_name);
         CreateTheComposeWindow(fields, newAttachData, nsIMsgCompType::Template, composeFormat, mdd->identity, nsnull);
       }
       else
@@ -1652,11 +1674,6 @@ mime_parse_stream_complete (nsMIMESession *stream)
   }
   PR_FREEIF(mdd->mailcharset);
   
-  // Release the prefs service
-  MimeObject *obj = (mdd ? mdd->obj : 0);  
-  if ( (obj) && (obj->options) && (obj->options->prefs) )
-    obj->options->prefs->Release();
-  
   mdd->identity = nsnull;
   PR_Free(mdd->url_name);
   PR_Free(mdd->originalMsgURI);
@@ -1853,16 +1870,6 @@ mime_decompose_file_init_fn ( void *stream_closure, MimeHeaders *headers )
   mdd->curAttachment = newAttachment;  
   newAttachment->type =  MimeHeaders_get ( headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE );
 
-  if (PL_strstr(newAttachment->type, MESSAGE_RFC822))
-  {
-    char *newName = PR_smprintf("%s.eml", newAttachment->real_name);
-    if (newName)
-    {
-      PR_Free(newAttachment->real_name);
-      newAttachment->real_name = newName;
-    }
-  }      
-  
   //
   // This is to handle the degenerated Apple Double attachment.
   //
@@ -1966,7 +1973,13 @@ mime_decompose_file_init_fn ( void *stream_closure, MimeHeaders *headers )
     else if (!nsCRT::strcasecmp(newAttachment->encoding, ENCODING_BASE64))
       fn = &MimeB64DecoderInit;
     else if (!nsCRT::strcasecmp(newAttachment->encoding, ENCODING_QUOTED_PRINTABLE))
-      fn = &MimeQPDecoderInit;
+    {
+      mdd->decoder_data = MimeQPDecoderInit (/* The (nsresult (*) ...) cast is to turn the `void' argument into `MimeObject'. */
+                              ((nsresult (*) (const char *, PRInt32, void *))
+                              dummy_file_write), mdd->tmpFileStream);
+      if (!mdd->decoder_data)
+        return MIME_OUT_OF_MEMORY;
+    }
     else if (!nsCRT::strcasecmp(newAttachment->encoding, ENCODING_UUENCODE) ||
              !nsCRT::strcasecmp(newAttachment->encoding, ENCODING_UUENCODE2) ||
              !nsCRT::strcasecmp(newAttachment->encoding, ENCODING_UUENCODE3) ||
@@ -2084,6 +2097,10 @@ mime_bridge_create_draft_stream(
 
   if (NS_SUCCEEDED(aURL->GetSpec(urlString)))
   {
+    PRInt32 typeIndex = urlString.Find("&type=application/x-message-display");
+    if (typeIndex != kNotFound)
+      urlString.Cut(typeIndex, sizeof("&type=application/x-message-display") - 1);
+
     mdd->url_name = ToNewCString(urlString);
     if (!(mdd->url_name))
       goto FAIL;
@@ -2107,7 +2124,7 @@ mime_bridge_create_draft_stream(
   mdd->options->decompose_file_output_fn = mime_decompose_file_output_fn;
   mdd->options->decompose_file_close_fn = mime_decompose_file_close_fn;
 
-  rv = CallGetService(kPrefCID, &(mdd->options->prefs));
+  mdd->options->m_prefBranch = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
   if (NS_FAILED(rv))
     goto FAIL;
 
