@@ -55,8 +55,8 @@ MimeDefClass(MimeMultipartSigned, MimeMultipartSignedClass,
 static int MimeMultipartSigned_initialize (MimeObject *);
 static int MimeMultipartSigned_create_child (MimeObject *);
 static int MimeMultipartSigned_close_child(MimeObject *);
-static int MimeMultipartSigned_parse_line (char *, PRInt32, MimeObject *);
-static int MimeMultipartSigned_parse_child_line (MimeObject *, char *, PRInt32,
+static int MimeMultipartSigned_parse_line (const char *, PRInt32, MimeObject *);
+static int MimeMultipartSigned_parse_child_line (MimeObject *, const char *, PRInt32,
 												 PRBool);
 static int MimeMultipartSigned_parse_eof (MimeObject *, PRBool);
 static void MimeMultipartSigned_finalize (MimeObject *);
@@ -176,7 +176,7 @@ MimeMultipartSigned_finalize (MimeObject *obj)
 
 
 static int
-MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
+MimeMultipartSigned_parse_line (const char *line, PRInt32 length, MimeObject *obj)
 {
   MimeMultipart *mult = (MimeMultipart *) obj;
   MimeMultipartSigned *sig = (MimeMultipartSigned *) obj;
@@ -395,7 +395,15 @@ MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
     else if (!nsCRT::strcasecmp(encoding.get(), ENCODING_BASE64))
       fn = &MimeB64DecoderInit;
     else if (!nsCRT::strcasecmp(encoding.get(), ENCODING_QUOTED_PRINTABLE))
-      fn = &MimeQPDecoderInit;
+    {
+      sig->sig_decoder_data =
+	MimeQPDecoderInit (((nsresult (*) (const char *, PRInt32, void *))
+		 (((MimeMultipartSignedClass *) obj->clazz)
+		      ->crypto_signature_hash)),
+		sig->crypto_closure);
+      if (!sig->sig_decoder_data)
+	return MIME_OUT_OF_MEMORY;
+    }
     else if (!nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE) ||
              !nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE2) ||
              !nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE3) ||
@@ -498,7 +506,7 @@ MimeMultipartSigned_close_child (MimeObject *obj)
 
 static int
 MimeMultipartSigned_parse_child_line (MimeObject *obj,
-									  char *line, PRInt32 length,
+									  const char *line, PRInt32 length,
 									  PRBool first_line_p)
 {
   MimeMultipartSigned *sig = (MimeMultipartSigned *) obj;
