@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -24,7 +24,7 @@
  *   Scott MacGregor <mscott@netscape.com>
  *   Pierre Phaneuf <pp@ludusdesign.com>
  *   Hkan Waara <hwaara@chello.se>
- *   David Bienvenu < bienvenu@nventure.com>
+ *   David Bienvenu <bienvenu@nventure.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -66,7 +66,6 @@
 #include "nsIMsgAccountManager.h"
 #include "nsIMessengerMigrator.h"
 #include "nsINntpIncomingServer.h"
-#include "nsICmdLineHandler.h"
 #include "nsICategoryManager.h"
 #include "nsIDocShell.h"
 #include "nsIDocShellLoadInfo.h"
@@ -86,6 +85,11 @@
 #include "nsMsgUtils.h"
 #include "nsEscape.h"
 #include "nsNetUtil.h"
+#include "nsIWindowWatcher.h"
+
+#ifdef MOZ_XUL_APP
+#include "nsICommandLine.h"
+#endif
 
 #undef GetPort  // XXX Windows!
 #undef SetPort  // XXX Windows!
@@ -95,8 +99,6 @@
 #define PREF_MAIL_ROOT_NNTP_REL 	"mail.root.nntp-rel"
 
 static NS_DEFINE_CID(kMessengerMigratorCID, NS_MESSENGERMIGRATOR_CID);
-static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
-static NS_DEFINE_CID(kCacheServiceCID, NS_CACHESERVICE_CID);
                     
 nsNntpService::nsNntpService()
 {
@@ -117,7 +119,7 @@ NS_IMPL_QUERY_INTERFACE7(nsNntpService,
                          nsIMsgMessageService,
                          nsIProtocolHandler,
                          nsIMsgProtocolInfo,
-                         nsICmdLineHandler,
+                         ICOMMANDLINEHANDLER,
                          nsIMsgMessageFetchPartService,
                          nsIContentHandler)
 
@@ -652,7 +654,8 @@ nsNntpService::GetFolderFromUri(const char *aUri, nsIMsgFolder **aFolder)
   nsUnescape(unescapedPath);
 
   nsCOMPtr<nsISupports> subFolder;
-  rv = rootFolder->GetChildNamed(NS_ConvertASCIItoUCS2(unescapedPath).get() , getter_AddRefs(subFolder));
+  rv = rootFolder->GetChildNamed(NS_ConvertUTF8toUTF16(unescapedPath).get() ,
+                                 getter_AddRefs(subFolder));
   PL_strfree(unescapedPath);
   NS_ENSURE_SUCCESS(rv,rv);
 
@@ -681,10 +684,10 @@ nsNntpService::CopyMessages(nsMsgKeyArray *keys, nsIMsgFolder *srcFolder, nsIStr
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-typedef struct _findNewsServerEntry {
+struct findNewsServerEntry {
   const char *newsgroup;
   nsINntpIncomingServer *server;
-} findNewsServerEntry;
+};
 
 
 PRBool 
@@ -699,7 +702,10 @@ nsNntpService::findNewsServerWithGroup(nsISupports *aElement, void *data)
   
   PRBool containsGroup = PR_FALSE;
   
-  rv = newsserver->ContainsNewsgroup((const char *)(entry->newsgroup), &containsGroup);
+  NS_ASSERTION(IsUTF8(nsDependentCString(entry->newsgroup)),
+                      "newsgroup is not in UTF-8");
+  rv = newsserver->ContainsNewsgroup(nsDependentCString(entry->newsgroup),
+                                     &containsGroup);
   if (NS_FAILED(rv)) return PR_TRUE;
   
   if (containsGroup) 
@@ -781,9 +787,10 @@ nsNntpService::SetUpNntpUrlForPosting(const char *aAccountKey, char **newsUrlSpe
 
   return NS_OK;
 }
-////////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
 // nsINntpService support
-////////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
+// XXX : may not work with non-ASCII newsgroup names and IDN hostnames
 NS_IMETHODIMP
 nsNntpService::GenerateNewsHeaderValsForPosting(const char *newsgroupsList, char **newsgroupsHeaderVal, char **newshostHeaderVal)
 {
@@ -1236,7 +1243,7 @@ PRBool nsNntpService::WeAreOffline()
 	nsresult rv = NS_OK;
   PRBool offline = PR_FALSE;
 
-  nsCOMPtr<nsIIOService> netService(do_GetService(kIOServiceCID, &rv));
+  nsCOMPtr<nsIIOService> netService(do_GetService(NS_IOSERVICE_CONTRACTID, &rv));
   if (NS_SUCCEEDED(rv) && netService)
     netService->GetOffline(&offline);
 
@@ -1407,11 +1414,12 @@ NS_IMETHODIMP nsNntpService::NewURI(const nsACString &aSpec,
 
 NS_IMETHODIMP nsNntpService::NewChannel(nsIURI *aURI, nsIChannel **_retval)
 {
+  NS_ENSURE_ARG_POINTER(aURI);
   nsresult rv = NS_OK;
   nsCOMPtr <nsINNTPProtocol> nntpProtocol;
   rv = GetProtocolForUri(aURI, nsnull, getter_AddRefs(nntpProtocol));
   if (NS_SUCCEEDED(rv))
-	  rv = nntpProtocol->Initialize(aURI, nsnull);
+    rv = nntpProtocol->Initialize(aURI, nsnull);
   if (NS_FAILED(rv)) return rv;
 
   return CallQueryInterface(nntpProtocol, _retval);
@@ -1499,7 +1507,7 @@ NS_IMETHODIMP
 nsNntpService::GetCanLoginAtStartUp(PRBool *aCanLoginAtStartUp)
 {
   NS_ENSURE_ARG_POINTER(aCanLoginAtStartUp);
-  *aCanLoginAtStartUp = PR_FALSE;
+  *aCanLoginAtStartUp = PR_TRUE;
   return NS_OK;
 }
 
@@ -1690,11 +1698,42 @@ nsNntpService::GetListOfGroupsOnServer(nsINntpIncomingServer *aNntpServer, nsIMs
   return NS_OK;
 }
 
+
+#ifdef MOZ_XUL_APP
+
+NS_IMETHODIMP
+nsNntpService::Handle(nsICommandLine* aCmdLine)
+{
+  nsresult rv;
+  PRBool found;
+
+  rv = aCmdLine->HandleFlag(NS_LITERAL_STRING("news"), PR_FALSE, &found);
+  if (NS_SUCCEEDED(rv) && found) {
+    nsCOMPtr<nsIWindowWatcher> wwatch (do_GetService(NS_WINDOWWATCHER_CONTRACTID));
+    NS_ENSURE_TRUE(wwatch, NS_ERROR_FAILURE);
+
+    nsCOMPtr<nsIDOMWindow> opened;
+    wwatch->OpenWindow(nsnull, "chrome://messenger/content/", "_blank",
+                       "chrome,dialog=no,all", nsnull, getter_AddRefs(opened));
+    aCmdLine->SetPreventDefault(PR_TRUE);
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsNntpService::GetHelpInfo(nsACString& aResult)
+{
+  aResult.Assign(NS_LITERAL_CSTRING("  -news                Open the news client.\n"));
+  return NS_OK;
+}
+
+#else // MOZ_XUL_APP
+
 CMDLINEHANDLER3_IMPL(nsNntpService,"-news","general.startup.news","Start with news.",NS_NEWSSTARTUPHANDLER_CONTRACTID,"News Cmd Line Handler", PR_FALSE,"", PR_TRUE)
 
 NS_IMETHODIMP nsNntpService::GetChromeUrlForTask(char **aChromeUrlForTask) 
 { 
-#ifndef MOZ_THUNDERBIRD
   if (!aChromeUrlForTask) return NS_ERROR_FAILURE; 
   nsresult rv;
   nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
@@ -1715,11 +1754,8 @@ NS_IMETHODIMP nsNntpService::GetChromeUrlForTask(char **aChromeUrlForTask)
   }
   *aChromeUrlForTask = PL_strdup("chrome://messenger/content/messenger.xul"); 
   return NS_OK; 
-#else
-  NS_ENSURE_ARG_POINTER(aChromeUrlForTask);
-  *aChromeUrlForTask = PL_strdup("chrome://messenger/content/"); 
-#endif
 }
+#endif // MOZ_XUL_APP
 
 NS_IMETHODIMP 
 nsNntpService::HandleContent(const char * aContentType, nsIInterfaceRequestor* aWindowContext, nsIRequest *request)
@@ -1802,7 +1838,7 @@ NS_IMETHODIMP nsNntpService::GetCacheSession(nsICacheSession **result)
   nsresult rv = NS_OK;
   if (!mCacheSession)
   {
-    nsCOMPtr<nsICacheService> serv = do_GetService(kCacheServiceCID, &rv);
+    nsCOMPtr<nsICacheService> serv = do_GetService(NS_CACHESERVICE_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
     
     rv = serv->CreateSession("NNTP-memory-only", nsICache::STORE_IN_MEMORY, nsICache::STREAM_BASED, getter_AddRefs(mCacheSession));
