@@ -46,8 +46,12 @@
 #include "nsIZipReader.h"
 #include "nsReadableUtils.h"
 #include "nsAutoPtr.h"
+#include "nsNetCID.h"
+#include "nsIObjectInputStream.h"
+#include "nsIObjectOutputStream.h"
+#include "nsIProgrammingLanguage.h"
 
-static NS_DEFINE_CID(kThisImplCID, NS_THIS_JARURI_IMPL_CID);
+static NS_DEFINE_CID(kJARURICID, NS_JARURI_CID);
 
 ////////////////////////////////////////////////////////////////////////////////
  
@@ -59,6 +63,7 @@ nsJARURI::~nsJARURI()
 {
 }
 
+// XXX Why is this threadsafe?
 NS_IMPL_THREADSAFE_ADDREF(nsJARURI)
 NS_IMPL_THREADSAFE_RELEASE(nsJARURI)
 NS_INTERFACE_MAP_BEGIN(nsJARURI)
@@ -67,9 +72,11 @@ NS_INTERFACE_MAP_BEGIN(nsJARURI)
   NS_INTERFACE_MAP_ENTRY(nsIURL)
   NS_INTERFACE_MAP_ENTRY(nsIJARURI)
   NS_INTERFACE_MAP_ENTRY(nsISerializable)
+  NS_INTERFACE_MAP_ENTRY(nsIClassInfo)
+  NS_INTERFACE_MAP_ENTRY(nsINestedURI)
   // see nsJARURI::Equals
-  if (aIID.Equals(kThisImplCID))
-      foundInterface = NS_STATIC_CAST(nsIJARURI *, this);
+  if (aIID.Equals(NS_GET_IID(nsJARURI)))
+      foundInterface = NS_REINTERPRET_CAST(nsISupports*, this);
   else
 NS_INTERFACE_MAP_END
 
@@ -135,17 +142,98 @@ nsJARURI::CreateEntryURL(const nsACString& entryFilename,
 // nsISerializable methods:
 
 NS_IMETHODIMP
-nsJARURI::Read(nsIObjectInputStream* aStream)
+nsJARURI::Read(nsIObjectInputStream* aInputStream)
 {
-    NS_NOTREACHED("nsJARURI::Read");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    nsresult rv;
+
+    rv = aInputStream->ReadObject(PR_TRUE, getter_AddRefs(mJARFile));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = aInputStream->ReadObject(PR_TRUE, getter_AddRefs(mJAREntry));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = aInputStream->ReadCString(mCharsetHint);
+    return rv;
 }
 
 NS_IMETHODIMP
-nsJARURI::Write(nsIObjectOutputStream* aStream)
+nsJARURI::Write(nsIObjectOutputStream* aOutputStream)
 {
-    NS_NOTREACHED("nsJARURI::Write");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    nsresult rv;
+    
+    rv = aOutputStream->WriteCompoundObject(mJARFile, NS_GET_IID(nsIURI),
+                                            PR_TRUE);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = aOutputStream->WriteCompoundObject(mJAREntry, NS_GET_IID(nsIURL),
+                                            PR_TRUE);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = aOutputStream->WriteStringZ(mCharsetHint.get());
+    return rv;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// nsIClassInfo methods:
+
+NS_IMETHODIMP 
+nsJARURI::GetInterfaces(PRUint32 *count, nsIID * **array)
+{
+    *count = 0;
+    *array = nsnull;
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsJARURI::GetHelperForLanguage(PRUint32 language, nsISupports **_retval)
+{
+    *_retval = nsnull;
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsJARURI::GetContractID(char * *aContractID)
+{
+    *aContractID = nsnull;
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsJARURI::GetClassDescription(char * *aClassDescription)
+{
+    *aClassDescription = nsnull;
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsJARURI::GetClassID(nsCID * *aClassID)
+{
+    *aClassID = (nsCID*) nsMemory::Alloc(sizeof(nsCID));
+    if (!*aClassID)
+        return NS_ERROR_OUT_OF_MEMORY;
+    return GetClassIDNoAlloc(*aClassID);
+}
+
+NS_IMETHODIMP 
+nsJARURI::GetImplementationLanguage(PRUint32 *aImplementationLanguage)
+{
+    *aImplementationLanguage = nsIProgrammingLanguage::CPLUSPLUS;
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsJARURI::GetFlags(PRUint32 *aFlags)
+{
+    // XXX We implement THREADSAFE addref/release, but probably shouldn't.
+    *aFlags = nsIClassInfo::MAIN_THREAD_ONLY;
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsJARURI::GetClassIDNoAlloc(nsCID *aClassIDNoAlloc)
+{
+    *aClassIDNoAlloc = kJARURICID;
+    return NS_OK;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -160,40 +248,79 @@ nsJARURI::GetSpec(nsACString &aSpec)
 }
 
 NS_IMETHODIMP
-nsJARURI::SetSpec(const nsACString &aSpec)
+nsJARURI::SetSpec(const nsACString& aSpec)
+{
+    return SetSpecWithBase(aSpec, nsnull);
+}
+
+nsresult
+nsJARURI::SetSpecWithBase(const nsACString &aSpec, nsIURI* aBaseURL)
 {
     nsresult rv;
+
     nsCOMPtr<nsIIOService> ioServ(do_GetIOService(&rv));
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv, rv);
 
     nsCAutoString scheme;
     rv = ioServ->ExtractScheme(aSpec, scheme);
-    if (NS_FAILED(rv)) return rv;
+    if (NS_FAILED(rv)) {
+        // not an absolute URI
+        if (!aBaseURL)
+            return NS_ERROR_MALFORMED_URI;
 
-    if (strcmp("jar", scheme.get()) != 0)
-        return NS_ERROR_MALFORMED_URI;
+        nsRefPtr<nsJARURI> otherJAR;
+        aBaseURL->QueryInterface(NS_GET_IID(nsJARURI), getter_AddRefs(otherJAR));
+        NS_ENSURE_TRUE(otherJAR, NS_NOINTERFACE);
+
+        mJARFile = otherJAR->mJARFile;
+
+        nsCOMPtr<nsIStandardURL> entry(do_CreateInstance(NS_STANDARDURL_CONTRACTID));
+        if (!entry)
+            return NS_ERROR_OUT_OF_MEMORY;
+
+        rv = entry->Init(nsIStandardURL::URLTYPE_NO_AUTHORITY, -1,
+                         aSpec, mCharsetHint.get(), otherJAR->mJAREntry);
+        if (NS_FAILED(rv))
+            return rv;
+
+        mJAREntry = do_QueryInterface(entry);
+        if (!mJAREntry)
+            return NS_NOINTERFACE;
+
+        return NS_OK;
+    }
+
+    NS_ENSURE_TRUE(scheme.EqualsLiteral("jar"), NS_ERROR_MALFORMED_URI);
+
+    nsACString::const_iterator begin, end;
+    aSpec.BeginReading(begin);
+    aSpec.EndReading(end);
+
+    while (begin != end && *begin != ':')
+        ++begin;
+
+    ++begin; // now we're past the "jar:"
 
     // Search backward from the end for the "!/" delimiter. Remember, jar URLs
     // can nest, e.g.:
     //    jar:jar:http://www.foo.com/bar.jar!/a.jar!/b.html
     // This gets the b.html document from out of the a.jar file, that's 
     // contained within the bar.jar file.
+    // Also, the outermost "inner" URI may be a relative URI:
+    //   jar:../relative.jar!/a.html
 
-    nsACString::const_iterator begin, end, delim_begin, delim_end;
-    aSpec.BeginReading(begin);
-    aSpec.EndReading(end);
-
-    delim_begin = begin;
-    delim_end = end;
+    nsACString::const_iterator delim_begin (begin),
+                               delim_end   (end);
 
     if (!RFindInReadable(NS_JAR_DELIMITER, delim_begin, delim_end))
         return NS_ERROR_MALFORMED_URI;
 
-    begin.advance(4);
-
-    rv = ioServ->NewURI(Substring(begin, delim_begin), mCharsetHint.get(), nsnull, getter_AddRefs(mJARFile));
+    rv = ioServ->NewURI(Substring(begin, delim_begin), mCharsetHint.get(),
+                        aBaseURL, getter_AddRefs(mJARFile));
     if (NS_FAILED(rv)) return rv;
 
+    NS_TryToSetImmutable(mJARFile);
+
     // skip over any extra '/' chars
     while (*delim_end == '/')
         ++delim_end;
@@ -331,14 +458,16 @@ nsJARURI::GetOriginCharset(nsACString &aOriginCharset)
 NS_IMETHODIMP
 nsJARURI::Equals(nsIURI *other, PRBool *result)
 {
+    nsresult rv;
+
     *result = PR_FALSE;
 
     if (other == nsnull)
         return NS_OK;	// not equal
 
     nsRefPtr<nsJARURI> otherJAR;
-    nsresult rv = other->QueryInterface(kThisImplCID, getter_AddRefs(otherJAR));
-    if (NS_FAILED(rv))
+    other->QueryInterface(NS_GET_IID(nsJARURI), getter_AddRefs(otherJAR));
+    if (!otherJAR)
         return NS_OK;   // not equal
 
     PRBool equal;
@@ -374,6 +503,8 @@ nsJARURI::Clone(nsIURI **result)
     rv = mJARFile->Clone(getter_AddRefs(newJARFile));
     if (NS_FAILED(rv)) return rv;
 
+    NS_TryToSetImmutable(newJARFile);
+
     nsCOMPtr<nsIURI> newJAREntryURI;
     rv = mJAREntry->Clone(getter_AddRefs(newJAREntryURI));
     if (NS_FAILED(rv)) return rv;
@@ -387,11 +518,12 @@ nsJARURI::Clone(nsIURI **result)
         uri->mJARFile = newJARFile;
         uri->mJAREntry = newJAREntry;
         *result = uri;
+        rv = NS_OK;
     } else {
         rv = NS_ERROR_OUT_OF_MEMORY;
     }
 
-    return NS_OK;
+    return rv;
 }
 
 NS_IMETHODIMP
@@ -633,16 +765,7 @@ nsJARURI::GetRelativeSpec(nsIURI* uriToCompare, nsACString& relativeSpec)
 NS_IMETHODIMP
 nsJARURI::GetJARFile(nsIURI* *jarFile)
 {
-    *jarFile = mJARFile;
-    NS_ADDREF(*jarFile);
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsJARURI::SetJARFile(nsIURI* jarFile)
-{
-    mJARFile = jarFile;
-    return NS_OK;
+    return GetInnerURI(jarFile);
 }
 
 NS_IMETHODIMP
@@ -664,3 +787,16 @@ nsJARURI::SetJAREntry(const nsACString &entryPath)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+
+NS_IMETHODIMP
+nsJARURI::GetInnerURI(nsIURI **uri)
+{
+    return NS_EnsureSafeToReturn(mJARFile, uri);
+}
+
+NS_IMETHODIMP
+nsJARURI::GetInnermostURI(nsIURI** uri)
+{
+    return NS_ImplGetInnermostURI(this, uri);
+}
+
