@@ -73,7 +73,7 @@ nsBMPDecoder::nsBMPDecoder()
 nsBMPDecoder::~nsBMPDecoder()
 {
     delete[] mColors;
-    delete[] mRow;
+    free(mRow);
     if (mAlpha)
         free(mAlpha);
     if (mDecoded)
@@ -141,30 +141,46 @@ nsresult nsBMPDecoder::SetData()
     nsresult rv = mFrame->SetImageData(mDecoded, mBpr, line * mBpr);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsRect r(0, line, mBIH.width, 1);
+    nsIntRect r(0, line, mBIH.width, 1);
     return mObserver->OnDataAvailable(nsnull, mFrame, &r);
 }
 
 nsresult nsBMPDecoder::WriteRLERows(PRUint32 rows)
 {
-    PRUint32 alpha, cnt, line;
+    PRUint32 cnt, line;
+
+    PRUint32 abpr;
     PRUint8 bit;
     PRUint8* pos = mAlpha;
-
     // First pack the alpha data
-    nsresult rv = mFrame->GetAlphaBytesPerRow(&alpha);
+    nsresult rv = mFrame->GetAlphaBytesPerRow(&abpr);
     NS_ENSURE_SUCCESS(rv, rv);
-    for (cnt = 0; cnt < alpha; cnt++) {
+#ifdef MOZ_CAIRO_GFX
+    gfx_format format;
+    mFrame->GetFormat(&format);
+    if (format == RLE_GFXFORMAT_ALPHA)
+        abpr >>= 2;
+#endif
+    for (cnt = 0; cnt < abpr; cnt++) {
         PRUint8 byte = 0;
         for (bit = 128; bit; bit >>= 1)
             byte |= *pos++ & bit;
         mAlpha[cnt] = byte;
+#ifdef MOZ_CAIRO_GFX
+#ifdef IS_LITTLE_ENDIAN
+        mDecoded[(cnt << 2) + 3] = byte ? 0 : 255;
+#else
+        mDecoded[(cnt << 2)] = byte ? 0 : 255;
+#endif
+#endif
     }
 
     for (cnt = 0; cnt < rows; cnt++) {
         line = (mBIH.height < 0) ? (-mBIH.height - mCurLine--) : --mCurLine;
-        rv = mFrame->SetAlphaData(mAlpha, alpha, line * alpha);
+#ifndef MOZ_CAIRO_GFX
+        rv = mFrame->SetAlphaData(mAlpha, abpr, line * abpr);
         NS_ENSURE_SUCCESS(rv, rv);
+#endif
         rv = mFrame->SetImageData(mDecoded, mBpr, line * mBpr);
         NS_ENSURE_SUCCESS(rv, rv);
         if (cnt == 0) {
@@ -174,7 +190,7 @@ nsresult nsBMPDecoder::WriteRLERows(PRUint32 rows)
     }
 
     line = (mBIH.height < 0) ? (-mBIH.height - mCurLine - rows) : mCurLine;
-    nsRect r(0, line, mBIH.width, rows);
+    nsIntRect r(0, line, mBIH.width, rows);
     return mObserver->OnDataAvailable(nsnull, mFrame, &r);
 }
 
@@ -286,7 +302,7 @@ NS_METHOD nsBMPDecoder::ProcessData(const char* aBuffer, PRUint32 aCount)
         NS_ENSURE_SUCCESS(rv, rv);
         mCurLine = real_height;
 
-        mRow = new PRUint8[(mBIH.width * mBIH.bpp)/8 + 4];
+        mRow = (PRUint8*)malloc((mBIH.width * mBIH.bpp)/8 + 4);
         // +4 because the line is padded to a 4 bit boundary, but I don't want
         // to make exact calculations here, that's unnecessary.
         // Also, it compensates rounding error.
