@@ -71,96 +71,32 @@ or revised. This service is offered free of charge; please provide us with your
 mailing address.
 */
 
+#include <stddef.h>
 #include "prtypes.h"
 #include "prmem.h"
 #include "prlog.h"
 #include "GIF2.h"
-#include "nsCRT.h"
-#include "nsRecyclingAllocator.h"
-#include "nsAutoLock.h"
 
 #include "nsGIFDecoder2.h"
 
-/*******************************************************************************
- * Gif decoder allocator
+/*
+ * GETN(n, s) requests at least 'n' bytes available from 'q', at start of state 's'
  *
- * For every image that gets loaded, we allocate
- *     4097 x 2 : gs->prefix
- *     4097 x 1 : gs->suffix
- *     4097 x 1 : gs->stack
- * for lzw to operate on the data. These are held for a very short interval
- * and freed. This allocator tries to keep one set of these around
- * and reuses them; automatically fails over to use calloc/free when all
- * buckets are full.
+ * Note, the hold will never need to be bigger than 256 bytes to gather up in the hold,
+ * as each GIF block (except colormaps) can never be bigger than 256 bytes.
+ * Colormaps are directly copied in the resp. global_colormap or dynamically allocated local_colormap.
+ * So a fixed buffer in gif_struct is good enough.
+ * This buffer is only needed to copy left-over data from one GifWrite call to the next
  */
-const int kGifAllocatorNBucket = 9;
-nsRecyclingAllocator *gGifAllocator = nsnull;
-
-void nsGifShutdown()
-{
-  // Release cached buffers from zlib allocator
-  delete gGifAllocator;
-  gGifAllocator = nsnull;
-}
-
-#define MAX_HOLD 768        /* for now must be big enough for a cmap */
-
-#define MAX_LZW_BITS          12
-#define MAX_BITS            4097 /* 2^MAX_LZW_BITS+1 */
-#define MINIMUM_DELAY_TIME    10
-
-
-/* Gather n characters from the input stream and then enter state s. */
 #define GETN(n,s)                    \
   PR_BEGIN_MACRO                     \
-    gs->state = gif_gather;          \
-    gs->gather_request_size = (n);   \
-    gs->post_gather_state = s;       \
+    gs->bytes_to_consume = (n);      \
+    gs->state = (s);                 \
   PR_END_MACRO
 
 /* Get a 16-bit value stored in little-endian format */
 #define GETINT16(p)   ((p)[1]<<8|(p)[0])
 
-/* Get a 32-bit value stored in little-endian format */
-#define GETINT32(p)   (((p)[3]<<24) | ((p)[2]<<16) | ((p)[1]<<8) | ((p)[0]))
-
-//******************************************************************************
-/*  binary block Allocate and Concatenate
- *
- *   destination_length  is the length of the existing block
- *   source_length   is the length of the block being added to the
- *   destination block
- */
-static char *il_BACat (char **destination,
-                       size_t destination_length,
-                       const char *source,
-                       size_t source_length)
-{
-  if (source) {
-    if (*destination) {
-      *destination = (char *) PR_REALLOC (*destination,
-                                          destination_length + source_length);
-      if (*destination == nsnull)
-        return (nsnull);
-
-      memmove(*destination + destination_length, source, source_length);
-    }
-    else {
-      *destination = (char *) PR_MALLOC (source_length);
-      if (*destination == nsnull)
-        return (nsnull);
-
-      memcpy(*destination, source, source_length);
-    }
-  }
-
-  return *destination;
-}
-
-#undef BlockAllocCat
-#define BlockAllocCat(dest, dest_length, src, src_length) \
-  il_BACat(&(dest), dest_length, src, src_length)
-
 //******************************************************************************
 // Send the data to the display front-end.
 static void output_row(gif_struct *gs)
@@ -429,28 +365,6 @@ static int do_lzw(gif_struct *gs, const PRUint8 *q)
   return 0;
 }
 
-static inline void *gif_calloc(size_t n, size_t s)
-{
-  if (!gGifAllocator)
-    gGifAllocator = new nsRecyclingAllocator(kGifAllocatorNBucket,
-                                             NS_DEFAULT_RECYCLE_TIMEOUT, "gif");
-  if (gGifAllocator)
-    return gGifAllocator->Calloc(n, s);
-  else
-    return calloc(n, s);
-}
-
-static inline void gif_free(void *ptr)
-{
-  if (!ptr)
-    return;
-  if (gGifAllocator)
-    gGifAllocator->Free(ptr);
-  else
-    free(ptr);
-}
-
-
 /*******************************************************************************
  * setup for gif_struct decoding
  */
@@ -460,31 +374,17 @@ PRBool GIFInit(gif_struct* gs, void* aClientData)
   if (!gs)
     return PR_FALSE;
 
+  // Clear out the structure, excluding the arrays
+  memset(gs, 0, sizeof(gif_struct));
   gs->clientptr = aClientData;
 
-  gs->state = gif_init;
-  gs->post_gather_state = gif_error;
-  gs->gathered = 0;
+  // Start with the version (GIF89a|GIF87a)
+  gs->state = gif_type;
+  gs->bytes_to_consume = 6;
 
   return PR_TRUE;
 }
 
-/* Maximum # of bytes to read ahead while waiting for delay_time to expire.
-   We no longer limit this number to remain within WIN16 malloc limitations
-   of 0xffff */
-
-#define MAX_READ_AHEAD  (0xFFFFFFL)
-
-//******************************************************************************
-PRBool gif_write_ready(const gif_struct* gs)
-{
-  if (!gs)
-    return PR_FALSE;
-
-  PRInt32 max = PR_MAX(MAX_READ_AHEAD, gs->requested_buffer_fullness);
-  return (gs->gathered < max);
-}
-
 /******************************************************************************/
 /*
  * process data arriving from the stream for the gif decoder
@@ -492,20 +392,55 @@ PRBool gif_write_ready(const gif_struct* gs)
 
 PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
 {
-  if (!gs)
+  if (!gs || !len)
     return PR_FAILURE;
 
-  /* If we fail, some upstream data provider ignored the
-     zero return value from il_gif_write_ready() which says not to
-     send any more data to this stream until the delay timeout fires. */
-  if ((len != 0) && (gs->gathered >= MAX_READ_AHEAD))
-    return PR_FAILURE;
+  const PRUint8 *q = buf;
 
-  const PRUint8 *q, *p = buf, *ep = buf + len;
+  // Add what we have sofar to the block
+  // If previous call to me left something in the hold first complete current block
+  // Or if we are filling the colormaps, first complete the colormap
+  PRUint8* p;
+  if (gs->state == gif_global_colormap)
+    p = gs->global_colormap;
+  else if (gs->state == gif_image_colormap)
+    p = gs->local_colormap;
+  else if (gs->bytes_in_hold)
+    p = gs->hold;
+  else
+    p = nsnull;
 
-  q = nsnull;                   /* Initialize to shut up gcc warnings */
+  if (p) {
+    // Add what we have sofar to the block
+    PRUint32 l = PR_MIN(len, gs->bytes_to_consume);
+    memcpy(p+gs->bytes_in_hold, buf, l);
+
+    if (l < gs->bytes_to_consume) {
+      // Not enough in 'buf' to complete current block, get more
+      gs->bytes_in_hold += l;
+      gs->bytes_to_consume -= l;
+      return PR_SUCCESS;
+    }
+    // Reset hold buffer count
+    gs->bytes_in_hold = 0;
+    // Point 'q' to complete block in hold (or in colormap)
+    q = p;
+  }
+
+  // Invariant:
+  //    'q' is start of current to be processed block (hold, colormap or buf)
+  //    'bytes_to_consume' is number of bytes to consume from 'buf'
+  //    'buf' points to the bytes to be consumed from the input buffer
+  //    'len' is number of bytes left in input buffer from position 'buf'.
+  //    At entrance of the for loop will 'buf' will be moved 'bytes_to_consume'
+  //    to point to next buffer, 'len' is adjusted accordingly.
+  //    So that next round in for loop, q gets pointed to the next buffer.
+
+  for (;len >= gs->bytes_to_consume; q=buf) {
+    // Eat the current block from the buffer, q keeps pointed at current block
+    buf += gs->bytes_to_consume;
+    len -= gs->bytes_to_consume;
 
-  while (p <= ep) {
     switch (gs->state)
     {
     case gif_lzw:
@@ -532,17 +467,6 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
       gs->codemask = (1 << gs->codesize) - 1;
 
       gs->datum = gs->bits = 0;
-      if (!gs->prefix)
-        gs->prefix = (PRUint16 *)gif_calloc(sizeof(PRUint16), MAX_BITS);
-      if (!gs->suffix)
-        gs->suffix = ( PRUint8 *)gif_calloc(sizeof(PRUint8),  MAX_BITS);
-      if (!gs->stack)
-        gs->stack  = ( PRUint8 *)gif_calloc(sizeof(PRUint8),  MAX_BITS);
-      if (!gs->prefix || !gs->suffix || !gs->stack) {
-        /* complete from abort will free prefix & suffix */
-        gs->state = gif_oom;
-        break;
-      }
 
       if (gs->clear_code >= MAX_BITS) {
         gs->state = gif_error;
@@ -559,29 +483,12 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
     }
     break;
 
-    /* We're positioned at the very start of the file. */
-    case gif_init:
-    {
-      GETN(3, gif_type);
-      break;
-    }
-
     /* All GIF files begin with "GIF87a" or "GIF89a" */
     case gif_type:
     {
-      if (strncmp((char*)q, "GIF", 3)) {
-        gs->state = gif_error;
-        break;
-      }
-      GETN(3, gif_version);
-    }
-    break;
-
-    case gif_version:
-    {
-      if (!strncmp((char*)q, "89a", 3)) {
+      if (!strncmp((char*)q, "GIF89a", 6)) {
         gs->version = 89;
-      } else if (!strncmp((char*)q, "87a", 3)) {
+      } else if (!strncmp((char*)q, "GIF87a", 6)) {
         gs->version = 87;
       } else {
         gs->state = gif_error;
@@ -614,11 +521,21 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
         gs->screen_height,
         gs->screen_bgcolor);
 
-      if (q[4] & 0x80) /* global map */
-        /* 3 bytes for each entry in the global colormap */
-        GETN(gs->global_colormap_size*3, gif_global_colormap);
-      else
-        GETN(1, gif_image_start);
+      if (q[4] & 0x80) { /* global map */
+        // Get the global colormap
+        const PRUint32 size = 3*gs->global_colormap_size;
+        if (len < size) {
+          // Use 'hold' pattern to get the global colormap
+          GETN(size, gif_global_colormap);
+          break;
+        }
+        // Copy everything and directly go to gif_lzw_start
+        memcpy(gs->global_colormap, buf, size);
+        buf += size;
+        len -= size;
+      }
+
+      GETN(1, gif_image_start);
 
       // q[6] = Pixel Aspect Ratio
       //   Not used
@@ -627,18 +544,8 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
     break;
 
     case gif_global_colormap:
-    {
-      PRUint8* map = (PRUint8*)PR_MALLOC(3 * gs->global_colormap_size);
-      if (!map) {
-        gs->state = gif_oom;
-        break;
-      }
-
-      gs->global_colormap = map;
-      memcpy(map, q, 3 * gs->global_colormap_size);
-
+      // Everything is already copied into global_colormap
       GETN(1, gif_image_start);
-    }
     break;
 
     case gif_image_start:
@@ -727,7 +634,6 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
         gs->is_transparent = PR_FALSE;
         // ignoring gfx control extension
       }
-      gs->control_extension = PR_TRUE;
       gs->disposal_method = (gdispose)(((*q) >> 2) & 0x7);
       // Some specs say 3rd bit (value 4), other specs say value 3
       // Let's choose 3 (the more popular)
@@ -740,9 +646,8 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
 
     case gif_comment_extension:
     {
-      gs->count = *q;
-      if (gs->count)
-        GETN(gs->count, gif_consume_comment);
+      if (*q)
+        GETN(*q, gif_consume_comment);
       else
         GETN(1, gif_image_start);
     }
@@ -787,8 +692,10 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
       }
       /* Wait for specified # of bytes to enter buffer */
       else if (netscape_extension == 2) {
-        gs->requested_buffer_fullness = GETINT32(q + 1);
-        GETN(gs->requested_buffer_fullness, gif_wait_for_buffer_full);
+        // Don't do this, this extension doesn't exist (isn't used at all) 
+        // and doesn't do anything, as our streaming/buffering takes care of it all...
+        // See: http://semmix.pl/color/exgraf/eeg24.htm
+        GETN(1, gif_netscape_extension_block);
       } else
         gs->state = gif_error; // 0,3-7 are yet to be defined netscape
                                // extension codes
@@ -796,11 +703,6 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
       break;
     }
 
-    case gif_wait_for_buffer_full:
-      gs->gathered = gs->requested_buffer_fullness;
-      GETN(1, gif_netscape_extension_block);
-      break;
-
     case gif_image_header:
     {
       PRUintn height, width;
@@ -902,48 +804,51 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
       }
 
       /* Clear state from last image */
-      gs->requested_buffer_fullness = 0;
       gs->irow = 0;
       gs->rows_remaining = gs->height;
       gs->rowend = gs->rowbuf + gs->width;
       gs->rowp = gs->rowbuf;
 
-      /* bits per pixel is 1<<((q[8]&0x07) + 1); */
+      /* bits per pixel is q[8]&0x07 */
 
       if (q[8] & 0x80) /* has a local colormap? */
       {
         int num_colors = 2 << (q[8] & 0x7);
-
-        if (num_colors > gs->local_colormap_size)
-          PR_FREEIF(gs->local_colormap);
-        gs->local_colormap_size = num_colors;
+        const PRUint32 size = 3*num_colors;
+        PRUint8 *map = gs->local_colormap;
+        if (!map || (num_colors > gs->local_colormap_size)) {
+          map = (PRUint8*)PR_REALLOC(map, size);
+          if (!map) {
+            gs->state = gif_oom;
+            break;
+          }
+        }
 
         /* Switch to the new local palette after it loads */
+        gs->local_colormap = map;
+        gs->local_colormap_size = num_colors;
         gs->is_local_colormap_defined = PR_TRUE;
-        GETN(gs->local_colormap_size * 3, gif_image_colormap);
+
+        if (len < size) {
+          // Use 'hold' pattern to get the image colormap
+          GETN(size, gif_image_colormap);
+          break;
+        }
+        // Copy everything and directly go to gif_lzw_start
+        memcpy(gs->local_colormap, buf, size);
+        buf += size;
+        len -= size;
       } else {
         /* Switch back to the global palette */
         gs->is_local_colormap_defined = PR_FALSE;
-        GETN(1, gif_lzw_start);
       }
+      GETN(1, gif_lzw_start);
     }
     break;
 
     case gif_image_colormap:
-    {
-      PRUint8 *map = gs->local_colormap;
-      if (!map) {
-        map = gs->local_colormap = (PRUint8*)PR_MALLOC(3 * gs->local_colormap_size);
-        if (!map) {
-          gs->state = gif_oom;
-          break;
-        }
-      }
-
-      memcpy(map, q, 3 * gs->local_colormap_size);
-
+      // Everything is already copied into local_colormap
       GETN(1, gif_lzw_start);
-    }
     break;
 
     case gif_sub_block:
@@ -974,7 +879,7 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
                                      gs->delay_time);
 
         /* Clear state from this image */
-        gs->control_extension = PR_FALSE;
+        gs->is_local_colormap_defined = PR_FALSE;
         gs->is_transparent = PR_FALSE;
 
         /* An image can specify a delay time before which to display
@@ -992,59 +897,6 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
       return PR_SUCCESS;
       break;
 
-    case gif_delay:
-    case gif_gather:
-    {
-      PRInt32 gather_remaining;
-      PRInt32 request_size = gs->gather_request_size;
-
-      {
-        gather_remaining = request_size - gs->gathered;
-
-        /* Do we already have enough data in the accumulation
-           buffer to satisfy the request ?  (This can happen
-           after we transition from the gif_delay state.) */
-        if (gather_remaining <= 0) {
-          gs->gathered -= request_size;
-          q = gs->gather_head;
-          gs->gather_head += request_size;
-          gs->state = gs->post_gather_state;
-          break;
-        }
-
-        /* Shift remaining data to the head of the buffer */
-        if (gs->gathered && (gs->gather_head != gs->hold)) {
-          memmove(gs->hold, gs->gather_head, gs->gathered);
-          gs->gather_head = gs->hold;
-        }
-
-        /* If we add the data just handed to us by the netlib
-           to what we've already gathered, is there enough to satisfy
-           the current request ? */
-        if ((ep - p) >= gather_remaining) {
-          if (gs->gathered) { /* finish a prior gather */
-            char *hold = (char*)gs->hold;
-            BlockAllocCat(hold, gs->gathered, (char*)p, gather_remaining);
-            gs->hold = (PRUint8*)hold;
-            q = gs->gather_head = gs->hold;
-            gs->gathered = 0;
-          } else
-            q = p;
-
-          p += gather_remaining;
-          gs->state = gs->post_gather_state;
-        } else {
-          char *hold = (char*)gs->hold;
-          BlockAllocCat(hold, gs->gathered, (char*)p, ep - p);
-          gs->hold = (PRUint8*)hold;
-          gs->gather_head = gs->hold;
-          gs->gathered += ep-p;
-          return PR_SUCCESS;
-        }
-      }
-    }
-    break;
-
     // Handle out of memory errors
     case gif_oom:
       return PR_FAILURE;
@@ -1054,15 +906,27 @@ PRStatus gif_write(gif_struct *gs, const PRUint8 *buf, PRUint32 len)
       nsGIFDecoder2::EndGIF(gs->clientptr, gs->loop_count);
       return PR_SUCCESS;
 
-    case gif_stop_animating:
-      return PR_SUCCESS;
-
     // We shouldn't ever get here.
     default:
       break;
     }
   }
 
+  // Copy the leftover into gs->hold
+  gs->bytes_in_hold = len;
+  if (len) {
+    // Add what we have sofar to the block
+    PRUint8* p;
+    if (gs->state == gif_global_colormap)
+      p = gs->global_colormap;
+    else if (gs->state == gif_image_colormap)
+      p = gs->local_colormap;
+    else
+      p = gs->hold;
+    memcpy(p, buf, len);
+    gs->bytes_to_consume -= len;
+  }
+
   return PR_SUCCESS;
 }
 
@@ -1078,13 +942,6 @@ void gif_destroy(gif_struct *gs)
     gs->delay_time = 0;
 
   PR_FREEIF(gs->rowbuf);
-  gif_free(gs->prefix);
-  gif_free(gs->suffix);
-  gif_free(gs->stack);
-  PR_FREEIF(gs->hold);
-
   PR_FREEIF(gs->local_colormap);
-  PR_FREEIF(gs->global_colormap);
-  PR_FREEIF(gs);
 }
 
