@@ -48,6 +48,10 @@
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsMemory.h"
 
+#ifdef MOZ_CAIRO_GFX
+#include "gfxContext.h"
+#endif
+
 NS_IMPL_ISUPPORTS2(imgContainerGIF, imgIContainer, nsITimerCallback)
 
 //******************************************************************************
@@ -74,9 +78,9 @@ imgContainerGIF::~imgContainerGIF()
 }
 
 //******************************************************************************
-/* void init (in nscoord aWidth, in nscoord aHeight,
+/* void init (in PRInt32 aWidth, in PRInt32 aHeight,
               in imgIContainerObserver aObserver); */
-NS_IMETHODIMP imgContainerGIF::Init(nscoord aWidth, nscoord aHeight,
+NS_IMETHODIMP imgContainerGIF::Init(PRInt32 aWidth, PRInt32 aHeight,
                                     imgIContainerObserver *aObserver)
 {
   if (aWidth <= 0 || aHeight <= 0) {
@@ -100,16 +104,16 @@ NS_IMETHODIMP imgContainerGIF::GetPreferredAlphaChannelFormat(gfx_format *aForma
 }
 
 //******************************************************************************
-/* readonly attribute nscoord width; */
-NS_IMETHODIMP imgContainerGIF::GetWidth(nscoord *aWidth)
+/* readonly attribute PRInt32 width; */
+NS_IMETHODIMP imgContainerGIF::GetWidth(PRInt32 *aWidth)
 {
   *aWidth = mSize.width;
   return NS_OK;
 }
 
 //******************************************************************************
-/* readonly attribute nscoord height; */
-NS_IMETHODIMP imgContainerGIF::GetHeight(nscoord *aHeight)
+/* readonly attribute PRInt32 height; */
+NS_IMETHODIMP imgContainerGIF::GetHeight(PRInt32 *aHeight)
 {
   *aHeight = mSize.height;
   return NS_OK;
@@ -139,6 +143,8 @@ NS_IMETHODIMP imgContainerGIF::GetNumFrames(PRUint32 *aNumFrames)
 NS_IMETHODIMP imgContainerGIF::GetFrameAt(PRUint32 index,
                                           gfxIImageFrame **_retval)
 {
+  NS_ENSURE_ARG(index < mFrames.Count());
+
   if (!(*_retval = mFrames[index]))
     return NS_ERROR_FAILURE;
 
@@ -169,7 +175,7 @@ NS_IMETHODIMP imgContainerGIF::AppendFrame(gfxIImageFrame *item)
     // Calculate mFirstFrameRefreshArea
     // Some gifs are huge but only have a small area that they animate
     // We only need to refresh that small area when Frame 0 comes around again
-    nsRect itemRect;
+    nsIntRect itemRect;
     item->GetRect(itemRect);
     mFirstFrameRefreshArea.UnionRect(mFirstFrameRefreshArea, itemRect);
   }
@@ -239,18 +245,19 @@ NS_IMETHODIMP imgContainerGIF::SetAnimationMode(PRUint16 aAnimationMode)
                aAnimationMode == imgIContainer::kLoopOnceAnimMode,
                "Wrong Animation Mode is being set!");
 
-  if (mAnimationMode == kNormalAnimMode &&
-      (aAnimationMode == kDontAnimMode ||
-       aAnimationMode == kLoopOnceAnimMode)) {
-    StopAnimation();
-  } else if (aAnimationMode == kNormalAnimMode &&
-             (mAnimationMode == kDontAnimMode ||
-              mAnimationMode == kLoopOnceAnimMode)) {
-    mAnimationMode = aAnimationMode;
-    StartAnimation();
-    return NS_OK;
+  switch (mAnimationMode = aAnimationMode) {
+    case kDontAnimMode:
+      StopAnimation();
+      break;
+    case kNormalAnimMode:
+      if (mLoopCount != 0 || mCurrentAnimationFrameIndex + 1 < mFrames.Count())
+        StartAnimation();
+      break;
+    case kLoopOnceAnimMode:
+      if (mCurrentAnimationFrameIndex + 1 < mFrames.Count())
+        StartAnimation();
+      break;
   }
-  mAnimationMode = aAnimationMode;
 
   return NS_OK;
 }
@@ -429,7 +436,7 @@ NS_IMETHODIMP imgContainerGIF::Notify(nsITimer *timer)
   else
     StopAnimation();
 
-  nsRect dirtyRect;
+  nsIntRect dirtyRect;
   gfxIImageFrame *frameToUse = nsnull;
 
   if (nextFrameIndex == 0) {
@@ -460,7 +467,7 @@ NS_IMETHODIMP imgContainerGIF::Notify(nsITimer *timer)
 // DoComposite gets called when the timer for animation get fired and we have to
 // update the composited frame of the animation.
 nsresult imgContainerGIF::DoComposite(gfxIImageFrame** aFrameToUse,
-                                      nsRect* aDirtyRect,
+                                      nsIntRect* aDirtyRect,
                                       gfxIImageFrame* aPrevFrame,
                                       gfxIImageFrame* aNextFrame,
                                       PRInt32 aNextFrameIndex)
@@ -485,7 +492,7 @@ nsresult imgContainerGIF::DoComposite(gfxIImageFrame** aFrameToUse,
     return NS_OK;
   }
 
-  nsRect prevFrameRect;
+  nsIntRect prevFrameRect;
   aPrevFrame->GetRect(prevFrameRect);
   PRBool isFullPrevFrame = (prevFrameRect.x == 0 && prevFrameRect.y == 0 &&
                             prevFrameRect.width == mSize.width &&
@@ -500,7 +507,7 @@ nsresult imgContainerGIF::DoComposite(gfxIImageFrame** aFrameToUse,
   }
 
   PRInt32 nextFrameDisposalMethod;
-  nsRect nextFrameRect;
+  nsIntRect nextFrameRect;
   aNextFrame->GetFrameDisposalMethod(&nextFrameDisposalMethod);
   aNextFrame->GetRect(nextFrameRect);
   PRBool isFullNextFrame = (nextFrameRect.x == 0 && nextFrameRect.y == 0 &&
@@ -703,8 +710,7 @@ void imgContainerGIF::BuildCompositeMask(gfxIImageFrame *aCompositingFrame,
     return;
   }
 
-  nscoord widthOverlay;
-  nscoord heightOverlay;
+  PRInt32 widthOverlay, heightOverlay;
   PRInt32 overlayXOffset, overlayYOffset;
   aOverlayFrame->GetWidth(&widthOverlay);
   aOverlayFrame->GetHeight(&heightOverlay);
@@ -726,8 +732,7 @@ void imgContainerGIF::BuildCompositeMask(gfxIImageFrame *aCompositingFrame,
   aOverlayFrame->GetAlphaBytesPerRow(&abprOverlay);
 
   // Only the composite's width & height are needed.  x & y should always be 0.
-  nscoord widthComposite;
-  nscoord heightComposite;
+  PRInt32 widthComposite, heightComposite;
   aCompositingFrame->GetWidth(&widthComposite);
   aCompositingFrame->GetHeight(&heightComposite);
 
@@ -753,15 +758,15 @@ void imgContainerGIF::BuildCompositeMask(gfxIImageFrame *aCompositingFrame,
   const PRUint32 height = PR_MIN(heightOverlay,
                                  heightComposite - overlayYOffset);
 
-// Windows and OS/2 have the funky bottom up data storage we need to account for
-#if defined(XP_WIN) || defined(XP_OS2)
+#ifdef MOZ_PLATFORM_IMAGES_BOTTOM_TO_TOP
+  // Account for bottom-up storage
   PRInt32 offset = ((heightComposite - 1) - overlayYOffset) * abprComposite;
 #else
   PRInt32 offset = overlayYOffset * abprComposite;
 #endif
   PRUint8* alphaLine = compositingAlphaData + offset + (overlayXOffset >> 3);
 
-#if defined(XP_WIN) || defined(XP_OS2)
+#ifdef MOZ_PLATFORM_IMAGES_BOTTOM_TO_TOP
   offset = (heightOverlay - 1) * abprOverlay;
 #else
   offset = 0;
@@ -822,7 +827,7 @@ void imgContainerGIF::BuildCompositeMask(gfxIImageFrame *aCompositingFrame,
       }
     }
 
-#if defined(XP_WIN) || defined(XP_OS2)
+#ifdef MOZ_PLATFORM_IMAGES_BOTTOM_TO_TOP
     alphaLine   -= abprComposite;
     overlayLine -= abprOverlay;
 #else
@@ -845,18 +850,8 @@ void imgContainerGIF::SetMaskVisibility(gfxIImageFrame *aFrame,
   if (!aFrame)
     return;
 
-  nsresult res;
-  PRUint8* alphaData;
-  PRUint32 alphaDataLength;
-  aFrame->LockAlphaData();
-  res = aFrame->GetAlphaData(&alphaData, &alphaDataLength);
-  if (!alphaData || !alphaDataLength || NS_FAILED(res)) {
-    aFrame->UnlockAlphaData();
-    return;
-  }
-
-  nscoord frameWidth;
-  nscoord frameHeight;
+  PRInt32 frameWidth;
+  PRInt32 frameHeight;
   aFrame->GetWidth(&frameWidth);
   aFrame->GetHeight(&frameHeight);
 
@@ -864,6 +859,37 @@ void imgContainerGIF::SetMaskVisibility(gfxIImageFrame *aFrame,
   const PRInt32 height = PR_MIN(aHeight, frameHeight - aY);
 
   if (width <= 0 || height <= 0) {
+    return;
+  }
+
+#ifdef MOZ_CAIRO_GFX
+  PRUint8* alphaData;
+  PRUint32 alphaDataLength;
+  const PRUint8 setMaskTo = aVisible ? 0xFF : 0x00;
+
+  aFrame->LockImageData();
+  nsresult res = aFrame->GetImageData(&alphaData, &alphaDataLength);
+  if (NS_SUCCEEDED(res)) {
+#ifdef IS_LITTLE_ENDIAN
+    alphaData += aY*frameWidth*4 + 3;
+#else
+    alphaData += aY*frameWidth*4;
+#endif
+    for (PRInt32 j = height; j > 0; --j) {
+      for (PRInt32 i = (aX+width-1)*4; i >= aX; i -= 4) {
+        alphaData[i] = setMaskTo;
+      }
+      alphaData += frameWidth*4;
+    }
+  }
+  aFrame->UnlockImageData();
+
+#else
+  PRUint8* alphaData;
+  PRUint32 alphaDataLength;
+  aFrame->LockAlphaData();
+  nsresult res = aFrame->GetAlphaData(&alphaData, &alphaDataLength);
+  if (!alphaData || !alphaDataLength || NS_FAILED(res)) {
     aFrame->UnlockAlphaData();
     return;
   }
@@ -879,8 +905,8 @@ void imgContainerGIF::SetMaskVisibility(gfxIImageFrame *aFrame,
   PRUint32 abpr;
   aFrame->GetAlphaBytesPerRow(&abpr);
 
-// Windows and OS/2 have the funky bottom up data storage we need to account for
-#if defined(XP_WIN) || defined(XP_OS2)
+#ifdef MOZ_PLATFORM_IMAGES_BOTTOM_TO_TOP
+  // Account for bottom-up storage.
   // Start at the bottom (top in memory), go to the top (bottom in memory)
   PRUint8* alphaLine = alphaData + ((frameHeight - aY - height) * abpr) +
                        (aX >> 3);
@@ -943,7 +969,7 @@ void imgContainerGIF::SetMaskVisibility(gfxIImageFrame *aFrame,
   } // if aVisible
 
   aFrame->UnlockAlphaData();
-  return;
+#endif
 }
 
 //******************************************************************************
@@ -956,12 +982,28 @@ void imgContainerGIF::SetMaskVisibility(gfxIImageFrame *aFrame, PRBool aVisible)
   PRUint32 alphaDataLength;
   const PRUint8 setMaskTo = aVisible ? 0xFF : 0x00;
 
+#ifdef MOZ_CAIRO_GFX
+  aFrame->LockImageData();
+  nsresult res = aFrame->GetImageData(&alphaData, &alphaDataLength);
+  if (NS_SUCCEEDED(res)) {
+    for (PRUint32 i = 0; i < alphaDataLength; i+=4) {
+#ifdef IS_LITTLE_ENDIAN
+      alphaData[i+3] = setMaskTo;
+#else
+      alphaData[i] = setMaskTo;
+#endif
+    }
+  }
+  aFrame->UnlockImageData();
+
+#else  
+
   aFrame->LockAlphaData();
   nsresult res = aFrame->GetAlphaData(&alphaData, &alphaDataLength);
   if (NS_SUCCEEDED(res) && alphaData && alphaDataLength)
     memset(alphaData, setMaskTo, alphaDataLength);
   aFrame->UnlockAlphaData();
-  return;
+#endif
 }
 
 //******************************************************************************
@@ -970,29 +1012,20 @@ void imgContainerGIF::BlackenFrame(gfxIImageFrame *aFrame)
 {
   if (!aFrame)
     return;
+#ifdef MOZ_CAIRO_GFX
+  PRInt32 widthFrame;
+  PRInt32 heightFrame;
+  aFrame->GetWidth(&widthFrame);
+  aFrame->GetHeight(&heightFrame);
 
-  aFrame->LockImageData();
+  BlackenFrame(aFrame, 0, 0, widthFrame, heightFrame);
 
-  PRUint8* aData;
+#else
   PRUint32 aDataLength;
 
-  aFrame->GetImageData(&aData, &aDataLength);
-  memset(aData, 0, aDataLength);
-
-  nsCOMPtr<nsIInterfaceRequestor> ireq(do_QueryInterface(aFrame));
-  if (ireq) {
-    nscoord width;
-    nscoord height;
-    aFrame->GetWidth(&width);
-    aFrame->GetHeight(&height);
-
-    nsCOMPtr<nsIImage> img(do_GetInterface(ireq));
-    nsRect r(0, 0, width, height);
-
-    img->ImageUpdated(nsnull, nsImageUpdateFlags_kBitsChanged, &r);
-  }
-
-  aFrame->UnlockImageData();
+  aFrame->GetImageDataLength(&aDataLength);
+  aFrame->SetImageData(nsnull, aDataLength, 0);
+#endif
 }
 
 //******************************************************************************
@@ -1003,10 +1036,26 @@ void imgContainerGIF::BlackenFrame(gfxIImageFrame *aFrame,
   if (!aFrame)
     return;
 
-  aFrame->LockImageData();
+#ifdef MOZ_CAIRO_GFX
+  nsCOMPtr<nsIImage> img(do_GetInterface(aFrame));
+  if (!img)
+    return;
 
-  nscoord widthFrame;
-  nscoord heightFrame;
+  nsRefPtr<gfxASurface> surf;
+  img->GetSurface(getter_AddRefs(surf));
+
+  nsRefPtr<gfxContext> ctx = new gfxContext(surf);
+  ctx->SetColor(gfxRGBA(0, 0, 0));
+  ctx->Rectangle(gfxRect(aX, aY, aWidth, aHeight));
+  ctx->Fill();
+
+  nsIntRect r(aX, aY, aWidth, aHeight);
+  img->ImageUpdated(nsnull, nsImageUpdateFlags_kBitsChanged, &r);
+
+#else // MOZ_CAIRO_GFX
+
+  PRInt32 widthFrame;
+  PRInt32 heightFrame;
   aFrame->GetWidth(&widthFrame);
   aFrame->GetHeight(&heightFrame);
 
@@ -1014,7 +1063,6 @@ void imgContainerGIF::BlackenFrame(gfxIImageFrame *aFrame,
   const PRInt32 height = PR_MIN(aHeight, (heightFrame - aY));
 
   if (width <= 0 || height <= 0) {
-    aFrame->UnlockImageData();
     return;
   }
 
@@ -1029,21 +1077,10 @@ void imgContainerGIF::BlackenFrame(gfxIImageFrame *aFrame,
   const PRUint32 bprToWrite = width * bpp;
   const PRUint32 xOffset = aX * bpp; // offset into row to start writing
 
-  PRUint8* tmpRow = NS_STATIC_CAST(PRUint8*, nsMemory::Alloc(bprToWrite));
-
-  if (!tmpRow) {
-    aFrame->UnlockImageData();
-    return;
-  }
-
-  memset(tmpRow, 0, bprToWrite);
-
   for (PRInt32 y = 0; y < height; y++) {
-    aFrame->SetImageData(tmpRow, bprToWrite, ((y + aY) * bpr) + xOffset);
+    aFrame->SetImageData(nsnull, bprToWrite, ((y + aY) * bpr) + xOffset);
   }
-  nsMemory::Free(tmpRow);
-
-  aFrame->UnlockImageData();
+#endif // MOZ_CAIRO_GFX
 }
 
 
@@ -1074,6 +1111,7 @@ PRBool imgContainerGIF::CopyFrameImage(gfxIImageFrame *aSrcFrame,
   memcpy(aDataDest, aDataSrc, aDataLengthSrc);
   aDstFrame->UnlockImageData();
 
+#ifndef MOZ_CAIRO_GFX
   // Copy Alpha/Mask Over
   // If no mask, lockAlpha will tell us
   if (NS_SUCCEEDED(aDstFrame->LockAlphaData())) {
@@ -1086,6 +1124,7 @@ PRBool imgContainerGIF::CopyFrameImage(gfxIImageFrame *aSrcFrame,
 
     aDstFrame->UnlockAlphaData();
   }
+#endif
 
   // Tell the image that it's data has been updated
   nsCOMPtr<nsIInterfaceRequestor> ireq(do_QueryInterface(aDstFrame));
@@ -1094,7 +1133,7 @@ PRBool imgContainerGIF::CopyFrameImage(gfxIImageFrame *aSrcFrame,
   nsCOMPtr<nsIImage> img(do_GetInterface(ireq));
   if (!img)
     return PR_FALSE;
-  nsRect r;
+  nsIntRect r;
   aDstFrame->GetRect(r);
   img->ImageUpdated(nsnull, nsImageUpdateFlags_kBitsChanged, &r);
 
