@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,26 +14,26 @@
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- * Alec Flett <alecf@netscape.com>
- * Brian Nesse <bnesse@netscape.com>
+ *   Alec Flett <alecf@netscape.com>
+ *   Brian Nesse <bnesse@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -46,13 +46,19 @@
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsXPIDLString.h"
-#include "nsIScriptSecurityManager.h"
 #include "nsIStringBundle.h"
 #include "prefapi.h"
 #include "prmem.h"
 #include "pldhash.h"
+#include "nsPrefsCID.h"
 
+#ifndef MOZ_NO_XPCOM_OBSOLETE
 #include "nsIFileSpec.h"  // this should be removed eventually
+#endif
+
+#include "plstr.h"
+#include "nsCRT.h"
+
 #include "prefapi_private_data.h"
 
 // Definitions
@@ -62,9 +68,9 @@ struct EnumerateData {
 };
 
 struct PrefCallbackData {
-  nsIPrefBranch *pBranch;
-  nsISupports   *pObserver;
-  PRBool        bIsWeakRef;
+  nsPrefBranch     *pBranch;
+  nsIObserver      *pObserver;
+  nsIWeakReference *pWeakRef;
 };
 
 
@@ -112,7 +118,8 @@ NS_IMPL_THREADSAFE_RELEASE(nsPrefBranch)
 NS_INTERFACE_MAP_BEGIN(nsPrefBranch)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIPrefBranch)
   NS_INTERFACE_MAP_ENTRY(nsIPrefBranch)
-  NS_INTERFACE_MAP_ENTRY(nsIPrefBranchInternal)
+  NS_INTERFACE_MAP_ENTRY_CONDITIONAL(nsIPrefBranch2, !mIsDefault)
+  NS_INTERFACE_MAP_ENTRY_CONDITIONAL(nsIPrefBranchInternal, !mIsDefault)
   NS_INTERFACE_MAP_ENTRY(nsISecurityPref)
   NS_INTERFACE_MAP_ENTRY(nsIObserver)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
@@ -255,7 +262,7 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
       } else {
         rv = GetCharPref(aPrefName, getter_Copies(utf8String));
         if (NS_SUCCEEDED(rv)) {
-          rv = theString->SetData(NS_ConvertUTF8toUCS2(utf8String).get());
+          rv = theString->SetData(NS_ConvertUTF8toUTF16(utf8String).get());
         }
       }
       if (NS_SUCCEEDED(rv)) {
@@ -333,7 +340,7 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
     nsCOMPtr<nsISupportsString> theString(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv));
 
     if (NS_SUCCEEDED(rv)) {
-      rv = theString->SetData(NS_ConvertUTF8toUCS2(utf8String));
+      rv = theString->SetData(NS_ConvertUTF8toUTF16(utf8String));
       if (NS_SUCCEEDED(rv)) {
         nsISupportsString *temp = theString;
 
@@ -346,6 +353,7 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
   }
 
   // This is deprecated and you should not be using it
+#ifndef MOZ_NO_XPCOM_OBSOLETE
   if (aType.Equals(NS_GET_IID(nsIFileSpec))) {
     nsCOMPtr<nsIFileSpec> file(do_CreateInstance(NS_FILESPEC_CONTRACTID, &rv));
 
@@ -365,6 +373,7 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
     }
     return rv;
   }
+#endif
 
   NS_WARNING("nsPrefBranch::GetComplexValue - Unsupported interface type");
   return NS_NOINTERFACE;
@@ -426,7 +435,7 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
       rv = theString->GetData(wideString);
       if (NS_SUCCEEDED(rv)) {
-        rv = SetCharPref(aPrefName, NS_ConvertUCS2toUTF8(wideString).get());
+        rv = SetCharPref(aPrefName, NS_ConvertUTF16toUTF8(wideString).get());
       }
     }
     return rv;
@@ -440,12 +449,13 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
       rv = theString->GetData(getter_Copies(wideString));
       if (NS_SUCCEEDED(rv)) {
-        rv = SetCharPref(aPrefName, NS_ConvertUCS2toUTF8(wideString).get());
+        rv = SetCharPref(aPrefName, NS_ConvertUTF16toUTF8(wideString).get());
       }
     }
     return rv;
   }
 
+#ifndef MOZ_NO_XPCOM_OBSOLETE
   // This is deprecated and you should not be using it
   if (aType.Equals(NS_GET_IID(nsIFileSpec))) {
     nsCOMPtr<nsIFileSpec> file = do_QueryInterface(aValue);
@@ -457,6 +467,7 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
     }
     return rv;
   }
+#endif
 
   NS_WARNING("nsPrefBranch::SetComplexValue - Unsupported interface type");
   return NS_NOINTERFACE;
@@ -602,7 +613,7 @@ NS_IMETHODIMP nsPrefBranch::GetChildList(const char *aStartingAt, PRUint32 *aCou
 
 
 /*
- *  nsIPrefBranchInternal methods
+ *  nsIPrefBranch2 methods
  */
 
 NS_IMETHODIMP nsPrefBranch::AddObserver(const char *aDomain, nsIObserver *aObserver, PRBool aHoldWeak)
@@ -623,11 +634,10 @@ NS_IMETHODIMP nsPrefBranch::AddObserver(const char *aDomain, nsIObserver *aObser
   if (nsnull == pCallback)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  pCallback->pBranch = NS_STATIC_CAST(nsIPrefBranch *, this);
-  pCallback->bIsWeakRef = aHoldWeak;
+  pCallback->pBranch = this;
+  pCallback->pObserver = aObserver;
 
   // hold a weak reference to the observer if so requested
-  nsCOMPtr<nsISupports> observerRef;
   if (aHoldWeak) {
     nsCOMPtr<nsISupportsWeakReference> weakRefFactory = do_QueryInterface(aObserver);
     if (!weakRefFactory) {
@@ -635,12 +645,12 @@ NS_IMETHODIMP nsPrefBranch::AddObserver(const char *aDomain, nsIObserver *aObser
       nsMemory::Free(pCallback);
       return NS_ERROR_INVALID_ARG;
     }
-    observerRef = do_GetWeakReference(weakRefFactory);
+    nsCOMPtr<nsIWeakReference> tmp = do_GetWeakReference(weakRefFactory);
+    NS_ADDREF(pCallback->pWeakRef = tmp);
   } else {
-    observerRef = aObserver;
+    pCallback->pWeakRef = nsnull;
+    NS_ADDREF(pCallback->pObserver);
   }
-  pCallback->pObserver = observerRef;
-  NS_ADDREF(pCallback->pObserver);
 
   mObservers->AppendElement(pCallback);
   mObserverDomains.AppendCString(nsCString(aDomain));
@@ -674,16 +684,7 @@ NS_IMETHODIMP nsPrefBranch::RemoveObserver(const char *aDomain, nsIObserver *aOb
   for (i = 0; i < count; i++) {
     pCallback = (PrefCallbackData *)mObservers->ElementAt(i);
     if (pCallback) {
-     nsCOMPtr<nsISupports> observerRef;
-     if (pCallback->bIsWeakRef) {
-       nsCOMPtr<nsISupportsWeakReference> weakRefFactory = do_QueryInterface(aObserver);
-       if (weakRefFactory)
-         observerRef = do_GetWeakReference(aObserver);
-     }
-     if (!observerRef)
-       observerRef = aObserver;
-
-      if (pCallback->pObserver == observerRef) {
+      if (pCallback->pObserver == aObserver) {
         mObserverDomains.CStringAt(i, domain);
         if (domain.Equals(aDomain)) {
           // We must pass a fully qualified preference name to remove the callback
@@ -694,7 +695,11 @@ NS_IMETHODIMP nsPrefBranch::RemoveObserver(const char *aDomain, nsIObserver *aOb
             // what we're trying to remove ourselves right now.
             mObservers->RemoveElementAt(i);
             mObserverDomains.RemoveCStringAt(i);
-            NS_RELEASE(pCallback->pObserver);
+            if (pCallback->pWeakRef) {
+              NS_RELEASE(pCallback->pWeakRef);
+            } else {
+              NS_RELEASE(pCallback->pObserver);
+            }
             nsMemory::Free(pCallback);
           }
           return rv;
@@ -718,31 +723,27 @@ NS_IMETHODIMP nsPrefBranch::Observe(nsISupports *aSubject, const char *aTopic, c
 PR_STATIC_CALLBACK(nsresult) NotifyObserver(const char *newpref, void *data)
 {
   PrefCallbackData *pData = (PrefCallbackData *)data;
-  nsPrefBranch *prefBranch = NS_STATIC_CAST(nsPrefBranch *, pData->pBranch);
 
   // remove any root this string may contain so as to not confuse the observer
   // by passing them something other than what they passed us as a topic
-  PRUint32 len = prefBranch->GetRootLength();
+  PRUint32 len = pData->pBranch->GetRootLength();
   nsCAutoString suffix(newpref + len);  
 
   nsCOMPtr<nsIObserver> observer;
-  if (pData->bIsWeakRef) {
-    nsIWeakReference *weakRef = NS_STATIC_CAST(nsIWeakReference *, pData->pObserver);
-    observer = do_QueryReferent(weakRef);
+  if (pData->pWeakRef) {
+    observer = do_QueryReferent(pData->pWeakRef);
     if (!observer) {
       // this weak referenced observer went away, remove them from the list
-      nsCOMPtr<nsIPrefBranchInternal> pbi = do_QueryInterface(pData->pBranch);
-      if (pbi) {
-        observer = NS_STATIC_CAST(nsIObserver *, pData->pObserver);
-        pbi->RemoveObserver(newpref, observer);
-      }
+      pData->pBranch->RemoveObserver(newpref, pData->pObserver);
       return NS_OK;
     }
-  } else
-    observer = NS_STATIC_CAST(nsIObserver *, pData->pObserver);
+  } else {
+    observer = pData->pObserver;
+  }
 
-  observer->Observe(pData->pBranch, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID,
-                    NS_ConvertASCIItoUCS2(suffix).get());
+  observer->Observe(NS_STATIC_CAST(nsIPrefBranch *, pData->pBranch),
+                    NS_PREFBRANCH_PREFCHANGE_TOPIC_ID,
+                    NS_ConvertASCIItoUTF16(suffix).get());
   return NS_OK;
 }
 
@@ -770,7 +771,11 @@ void nsPrefBranch::freeObserverList(void)
           // what we're trying to remove right now.
           mObservers->ReplaceElementAt(nsnull, i);
           PREF_UnregisterCallback(pref, NotifyObserver, pCallback);
-          NS_RELEASE(pCallback->pObserver);
+          if (pCallback->pWeakRef) {
+            NS_RELEASE(pCallback->pWeakRef);
+          } else {
+            NS_RELEASE(pCallback->pObserver);
+          }
           nsMemory::Free(pCallback);
         }
       }
@@ -807,7 +812,7 @@ nsresult nsPrefBranch::GetDefaultFromPropertiesFile(const char *aPrefName, PRUni
 
   // string names are in unicode
   nsAutoString stringId;
-  stringId.AssignWithConversion(aPrefName);
+  stringId.AssignASCII(aPrefName);
 
   return bundle->GetStringFromName(stringId.get(), return_buf);
 }
@@ -840,14 +845,14 @@ nsresult nsPrefBranch::getValidatedPrefName(const char *aPrefName, const char **
     PL_strncmp(fullPref, capabilityPrefix, sizeof(capabilityPrefix)-1) == 0)
   {
     nsresult rv;
-    nsCOMPtr<nsIScriptSecurityManager> secMan = 
-             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+    nsCOMPtr<nsIPrefSecurityCheck> secCheck = 
+             do_GetService(NS_GLOBAL_PREF_SECURITY_CHECK, &rv);
 
     if (NS_FAILED(rv))
       return NS_ERROR_FAILURE;
 
     PRBool enabled;
-    rv = secMan->IsCapabilityEnabled("CapabilityPreferencesAccess", &enabled);
+    rv = secCheck->CanAccessSecurityPreferences(&enabled);
     if (NS_FAILED(rv) || !enabled)
       return NS_ERROR_FAILURE;
   }
@@ -965,6 +970,8 @@ nsPrefLocalizedString::GetData(PRUnichar** _retval)
 NS_IMETHODIMP
 nsPrefLocalizedString::SetData(const PRUnichar *aData)
 {
+  if (!aData)
+    return SetData(EmptyString());
   return SetData(nsDependentString(aData));
 }
 
@@ -972,6 +979,8 @@ NS_IMETHODIMP
 nsPrefLocalizedString::SetDataWithLength(PRUint32 aLength,
                                          const PRUnichar* aData)
 {
+  if (!aData)
+    return SetData(EmptyString());
   return SetData(Substring(aData, aData + aLength));
 }
 
