@@ -1,5 +1,5 @@
-/* vim:set ts=4 sw=4 cindent et: */
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim:set ts=4 sw=4 cindent et: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -41,10 +41,10 @@
 #include "nsIFileProtocolHandler.h"
 #include "nscore.h"
 #include "nsIServiceManager.h"
-#include "nsIEventQueueService.h"
 #include "nsIURI.h"
 #include "nsIStreamListener.h"
 #include "prprf.h"
+#include "prlog.h"
 #include "nsLoadGroup.h"
 #include "nsInputStreamChannel.h"
 #include "nsXPIDLString.h" 
@@ -53,7 +53,7 @@
 #include "netCore.h"
 #include "nsIObserverService.h"
 #include "nsIPrefService.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "nsIPrefLocalizedString.h"
 #include "nsICategoryManager.h"
 #include "nsXPCOM.h"
@@ -66,6 +66,8 @@
 #include "nsIRecyclingAllocator.h"
 #include "nsISocketTransport.h"
 #include "nsCRT.h"
+#include "nsINestedURI.h"
+#include "nsNetUtil.h"
 
 #define PORT_PREF_PREFIX     "network.security.ports."
 #define PORT_PREF(x)         PORT_PREF_PREFIX x
@@ -77,7 +79,9 @@ static NS_DEFINE_CID(kDNSServiceCID, NS_DNSSERVICE_CID);
 static NS_DEFINE_CID(kErrorServiceCID, NS_ERRORSERVICE_CID);
 static NS_DEFINE_CID(kProtocolProxyServiceCID, NS_PROTOCOLPROXYSERVICE_CID);
 
-// A general port blacklist.  Connections to these ports will not be avoided unless 
+nsIOService* gIOService = nsnull;
+
+// A general port blacklist.  Connections to these ports will not be allowed unless 
 // the protocol overrides.
 //
 // TODO: I am sure that there are more ports to be added.  
@@ -122,6 +126,7 @@ PRInt16 gBadPortList[] = {
   143,  // imap2  
   179,  // BGP
   389,  // ldap        
+  465,  // smtp+ssl
   512,  // print / exec          
   513,  // login         
   514,  // shell         
@@ -132,8 +137,12 @@ PRInt16 gBadPortList[] = {
   532,  // netnews        
   540,  // uucp       
   556,  // remotefs    
+  563,  // nntp+ssl
   587,  //
   601,  //       
+  636,  // ldap+ssl
+  993,  // imap+ssl
+  995,  // pop3+ssl
   2049, // nfs
   4045, // lockd
   6000, // x11        
@@ -149,8 +158,11 @@ nsIMemory* nsIOService::gBufferCache = nsnull;
 ////////////////////////////////////////////////////////////////////////////////
 
 nsIOService::nsIOService()
-    : mOffline(PR_FALSE),
-      mOfflineForProfileChange(PR_FALSE)
+    : mOffline(PR_FALSE)
+    , mOfflineForProfileChange(PR_FALSE)
+    , mManageOfflineStatus(PR_FALSE)
+    , mChannelEventSinks(NS_CHANNEL_EVENT_SINK_CATEGORY)
+    , mContentSniffers(NS_CONTENT_SNIFFER_CATEGORY)
 {
     // Get the allocator ready
     if (!gBufferCache)
@@ -176,17 +188,12 @@ nsIOService::Init()
 {
     nsresult rv;
     
-    // Hold onto the eventQueue service.  We do not want any eventqueues to go away
-    // when we shutdown until we process all remaining transports
-
-    mEventQueueService = do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID, &rv);
-    if (NS_FAILED(rv))
-        NS_WARNING("failed to get event queue service");
-    
     // We need to get references to these services so that we can shut them
     // down later. If we wait until the nsIOService is being shut down,
     // GetService will fail at that point.
 
+    // TODO(darin): Load the Socket and DNS services lazily.
+
     mSocketTransportService = do_GetService(kSocketTransportServiceCID, &rv);
     if (NS_FAILED(rv))
         NS_WARNING("failed to get socket transport service");
@@ -195,10 +202,6 @@ nsIOService::Init()
     if (NS_FAILED(rv))
         NS_WARNING("failed to get DNS service");
 
-    mProxyService = do_GetService(kProtocolProxyServiceCID, &rv);
-    if (NS_FAILED(rv))
-        NS_WARNING("failed to get protocol proxy service");
-
     // XXX hack until xpidl supports error info directly (bug 13423)
     nsCOMPtr<nsIErrorService> errorService = do_GetService(kErrorServiceCID);
     if (errorService) {
@@ -212,7 +215,7 @@ nsIOService::Init()
         mRestrictedPortList.AppendElement(NS_REINTERPRET_CAST(void *, gBadPortList[i]));
 
     // Further modifications to the port list come from prefs
-    nsCOMPtr<nsIPrefBranchInternal> prefBranch;
+    nsCOMPtr<nsIPrefBranch2> prefBranch;
     GetPrefBranch(getter_AddRefs(prefBranch));
     if (prefBranch) {
         prefBranch->AddObserver(PORT_PREF_PREFIX, this, PR_TRUE);
@@ -227,25 +230,82 @@ nsIOService::Init()
         observerService->AddObserver(this, kProfileChangeNetTeardownTopic, PR_TRUE);
         observerService->AddObserver(this, kProfileChangeNetRestoreTopic, PR_TRUE);
         observerService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_TRUE);
+        observerService->AddObserver(this, NS_NETWORK_LINK_TOPIC, PR_TRUE);
     }
     else
         NS_WARNING("failed to get observer service");
+        
+    gIOService = this;
     
+    // go into managed mode if we can
+    mNetworkLinkService = do_GetService(NS_NETWORK_LINK_SERVICE_CONTRACTID);
+    if (mNetworkLinkService) {
+        mManageOfflineStatus = PR_TRUE;
+        TrackNetworkLinkStatusForOffline();
+    }
+
     return NS_OK;
 }
 
 
 nsIOService::~nsIOService()
 {
+    gIOService = nsnull;
 }   
 
-NS_IMPL_THREADSAFE_ISUPPORTS3(nsIOService,
+nsIOService*
+nsIOService::GetInstance() {
+    if (!gIOService) {
+        gIOService = new nsIOService();
+        if (!gIOService)
+            return nsnull;
+        NS_ADDREF(gIOService);
+
+        nsresult rv = gIOService->Init();
+        if (NS_FAILED(rv)) {
+            NS_RELEASE(gIOService);
+            return nsnull;
+        }
+        return gIOService;
+    }
+    NS_ADDREF(gIOService);
+    return gIOService;
+}
+
+NS_IMPL_THREADSAFE_ISUPPORTS5(nsIOService,
                               nsIIOService,
+                              nsIIOService2,
+                              nsINetUtil,
                               nsIObserver,
                               nsISupportsWeakReference)
 
 ////////////////////////////////////////////////////////////////////////////////
 
+nsresult
+nsIOService::OnChannelRedirect(nsIChannel* oldChan, nsIChannel* newChan,
+                               PRUint32 flags)
+{
+    nsCOMPtr<nsIChannelEventSink> sink =
+        do_GetService(NS_GLOBAL_CHANNELEVENTSINK_CONTRACTID);
+    if (sink) {
+        nsresult rv = sink->OnChannelRedirect(oldChan, newChan, flags);
+        if (NS_FAILED(rv))
+            return rv;
+    }
+
+    // Finally, our category
+    const nsCOMArray<nsIChannelEventSink>& entries =
+        mChannelEventSinks.GetEntries();
+    PRInt32 len = entries.Count();
+    for (PRInt32 i = 0; i < len; ++i) {
+        nsresult rv = entries[i]->OnChannelRedirect(oldChan, newChan, flags);
+        if (NS_FAILED(rv))
+            return rv;
+    }
+
+    return NS_OK;
+}
+
 nsresult
 nsIOService::CacheProtocolHandler(const char *scheme, nsIProtocolHandler *handler)
 {
@@ -259,7 +319,7 @@ nsIOService::CacheProtocolHandler(const char *scheme, nsIProtocolHandler *handle
             nsCOMPtr<nsISupportsWeakReference> factoryPtr = do_QueryInterface(handler, &rv);
             if (!factoryPtr)
             {
-                // Dont cache handlers that dont support weak reference as
+                // Don't cache handlers that don't support weak reference as
                 // there is real danger of a circular reference.
 #ifdef DEBUG_dp
                 printf("DEBUG: %s protcol handler doesn't support weak ref. Not cached.\n", scheme);
@@ -310,7 +370,7 @@ nsIOService::GetProtocolHandler(const char* scheme, nsIProtocolHandler* *result)
 
     PRBool externalProtocol = PR_FALSE;
     PRBool listedProtocol   = PR_TRUE;
-    nsCOMPtr<nsIPrefBranchInternal> prefBranch;
+    nsCOMPtr<nsIPrefBranch2> prefBranch;
     GetPrefBranch(getter_AddRefs(prefBranch));
     if (prefBranch) {
         nsCAutoString externalProtocolPref("network.protocol-handler.external.");
@@ -451,34 +511,48 @@ nsIOService::NewChannelFromURI(nsIURI *aURI, nsIChannel **result)
 
     nsCAutoString scheme;
     rv = aURI->GetScheme(scheme);
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsIProxyInfo> pi;
-    if (mProxyService) {
-        rv = mProxyService->ExamineForProxy(aURI, getter_AddRefs(pi));
-        if (NS_FAILED(rv))
-            pi = 0;
-    }
+    if (NS_FAILED(rv))
+        return rv;
 
     nsCOMPtr<nsIProtocolHandler> handler;
+    rv = GetProtocolHandler(scheme.get(), getter_AddRefs(handler));
+    if (NS_FAILED(rv))
+        return rv;
 
-    if (pi && !strcmp(pi->Type(),"http")) {
-        // we are going to proxy this channel using an http proxy
-        rv = GetProtocolHandler("http", getter_AddRefs(handler));
-        if (NS_FAILED(rv)) return rv;
-    } else {
-        rv = GetProtocolHandler(scheme.get(), getter_AddRefs(handler));
-        if (NS_FAILED(rv)) return rv;
+    PRUint32 protoFlags;
+    rv = handler->GetProtocolFlags(&protoFlags);
+    if (NS_FAILED(rv))
+        return rv;
+
+    // Talk to the PPS if the protocol handler allows proxying.  Otherwise,
+    // skip this step.  This allows us to lazily load the PPS at startup.
+    if (protoFlags & nsIProtocolHandler::ALLOWS_PROXY) {
+        nsCOMPtr<nsIProxyInfo> pi;
+        if (!mProxyService) {
+            mProxyService = do_GetService(NS_PROTOCOLPROXYSERVICE_CONTRACTID);
+            if (!mProxyService)
+                NS_WARNING("failed to get protocol proxy service");
+        }
+        if (mProxyService) {
+            rv = mProxyService->Resolve(aURI, 0, getter_AddRefs(pi));
+            if (NS_FAILED(rv))
+                pi = nsnull;
+        }
+        if (pi) {
+            nsCAutoString type;
+            if (NS_SUCCEEDED(pi->GetType(type)) && type.EqualsLiteral("http")) {
+                // we are going to proxy this channel using an http proxy
+                rv = GetProtocolHandler("http", getter_AddRefs(handler));
+                if (NS_FAILED(rv))
+                    return rv;
+            }
+            nsCOMPtr<nsIProxiedProtocolHandler> pph = do_QueryInterface(handler);
+            if (pph)
+                return pph->NewProxiedChannel(aURI, pi, result);
+        }
     }
 
-    nsCOMPtr<nsIProxiedProtocolHandler> pph = do_QueryInterface(handler);
-
-    if (pph)
-        rv = pph->NewProxiedChannel(aURI, pi, result);
-    else
-        rv = handler->NewChannel(aURI, result);
-
-    return rv;
+    return handler->NewChannel(aURI, result);
 }
 
 NS_IMETHODIMP
@@ -506,15 +580,15 @@ nsIOService::SetOffline(PRBool offline)
         do_GetService("@mozilla.org/observer-service;1");
     
     nsresult rv;
-    if (offline) {
-        NS_NAMED_LITERAL_STRING(offlineString, "offline");
+    if (offline && !mOffline) {
+        NS_NAMED_LITERAL_STRING(offlineString, NS_IOSERVICE_OFFLINE);
         mOffline = PR_TRUE; // indicate we're trying to shutdown
 
         // don't care if notification fails
         // this allows users to attempt a little cleanup before dns and socket transport are shut down.
         if (observerService)
             observerService->NotifyObservers(NS_STATIC_CAST(nsIIOService *, this),
-                                             "network:offline-about-to-go-offline",
+                                             NS_IOSERVICE_GOING_OFFLINE_TOPIC,
                                              offlineString.get());
 
         // be sure to try and shutdown both (even if the first fails)...
@@ -531,7 +605,7 @@ nsIOService::SetOffline(PRBool offline)
         // don't care if notification fails
         if (observerService)
             observerService->NotifyObservers(NS_STATIC_CAST(nsIIOService *, this),
-                                             "network:offline-status-changed",
+                                             NS_IOSERVICE_OFFLINE_STATUS_TOPIC,
                                              offlineString.get());
     }
     else if (!offline && mOffline) {
@@ -546,12 +620,16 @@ nsIOService::SetOffline(PRBool offline)
         }
         mOffline = PR_FALSE;    // indicate success only AFTER we've
                                 // brought up the services
+         
+        // trigger a PAC reload when we come back online
+        if (mProxyService)
+            mProxyService->ReloadPAC();
  
         // don't care if notification fails
         if (observerService)
             observerService->NotifyObservers(NS_STATIC_CAST(nsIIOService *, this),
-                                             "network:offline-status-changed",
-                                             NS_LITERAL_STRING("online").get());
+                                             NS_IOSERVICE_OFFLINE_STATUS_TOPIC,
+                                             NS_LITERAL_STRING(NS_IOSERVICE_ONLINE).get());
     }
     return NS_OK;
 }
@@ -658,7 +736,7 @@ nsIOService::ParsePortList(nsIPrefBranch *prefBranch, const char *pref, PRBool r
 }
 
 void
-nsIOService::GetPrefBranch(nsIPrefBranchInternal **result)
+nsIOService::GetPrefBranch(nsIPrefBranch2 **result)
 {
     *result = nsnull;
     CallGetService(NS_PREFSERVICE_CONTRACTID, result);
@@ -673,7 +751,7 @@ nsIOService::Observe(nsISupports *subject,
     if (!strcmp(topic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
         nsCOMPtr<nsIPrefBranch> prefBranch = do_QueryInterface(subject);
         if (prefBranch)
-            PrefsChanged(prefBranch, NS_ConvertUCS2toUTF8(data).get());
+            PrefsChanged(prefBranch, NS_ConvertUTF16toUTF8(data).get());
     }
     else if (!strcmp(topic, kProfileChangeNetTeardownTopic)) {
         if (!mOffline) {
@@ -683,15 +761,161 @@ nsIOService::Observe(nsISupports *subject,
     }
     else if (!strcmp(topic, kProfileChangeNetRestoreTopic)) {
         if (mOfflineForProfileChange) {
-            SetOffline(PR_FALSE);
             mOfflineForProfileChange = PR_FALSE;
-        }    
+            if (!mManageOfflineStatus ||
+                NS_FAILED(TrackNetworkLinkStatusForOffline())) {
+                SetOffline(PR_FALSE);
+            }
+        } 
     }
     else if (!strcmp(topic, NS_XPCOM_SHUTDOWN_OBSERVER_ID)) {
         SetOffline(PR_TRUE);
 
         // Break circular reference.
-        mProxyService = 0;
+        mProxyService = nsnull;
     }
+    else if (!strcmp(topic, NS_NETWORK_LINK_TOPIC)) {
+        if (!mOfflineForProfileChange && mManageOfflineStatus) {
+            TrackNetworkLinkStatusForOffline();
+        }
+    }
+    
     return NS_OK;
 }
+
+// nsINetUtil interface
+NS_IMETHODIMP
+nsIOService::ParseContentType(const nsACString &aTypeHeader,
+                              nsACString &aCharset,
+                              PRBool *aHadCharset,
+                              nsACString &aContentType)
+{
+    net_ParseContentType(aTypeHeader, aContentType, aCharset, aHadCharset);
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsIOService::ProtocolHasFlags(nsIURI   *uri,
+                              PRUint32  flags,
+                              PRBool   *result)
+{
+    NS_ENSURE_ARG(uri);
+
+    *result = PR_FALSE;
+    nsCAutoString scheme;
+    nsresult rv = uri->GetScheme(scheme);
+    NS_ENSURE_SUCCESS(rv, rv);
+  
+    PRUint32 protocolFlags;
+    rv = GetProtocolFlags(scheme.get(), &protocolFlags);
+
+    if (NS_SUCCEEDED(rv)) {
+        *result = (protocolFlags & flags) == flags;
+    }
+  
+    return rv;
+}
+
+NS_IMETHODIMP
+nsIOService::URIChainHasFlags(nsIURI   *uri,
+                              PRUint32  flags,
+                              PRBool   *result)
+{
+    nsresult rv = ProtocolHasFlags(uri, flags, result);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    if (*result) {
+        return rv;
+    }
+
+    // Dig deeper into the chain.  Note that this is not a do/while loop to
+    // avoid the extra addref/release on |uri| in the common (non-nested) case.
+    nsCOMPtr<nsINestedURI> nestedURI = do_QueryInterface(uri);
+    while (nestedURI) {
+        nsCOMPtr<nsIURI> innerURI;
+        rv = nestedURI->GetInnerURI(getter_AddRefs(innerURI));
+        NS_ENSURE_SUCCESS(rv, rv);
+
+        rv = ProtocolHasFlags(innerURI, flags, result);
+
+        if (*result) {
+            return rv;
+        }
+
+        nestedURI = do_QueryInterface(innerURI);
+    }
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsIOService::ToImmutableURI(nsIURI* uri, nsIURI** result)
+{
+    if (!uri) {
+        *result = nsnull;
+        return NS_OK;
+    }
+
+    nsresult rv = NS_EnsureSafeToReturn(uri, result);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    NS_TryToSetImmutable(*result);
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsIOService::SetManageOfflineStatus(PRBool aManage) {
+    PRBool wasManaged = mManageOfflineStatus;
+    mManageOfflineStatus = aManage;
+    if (mManageOfflineStatus && !wasManaged)
+        return TrackNetworkLinkStatusForOffline();
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsIOService::GetManageOfflineStatus(PRBool* aManage) {
+    *aManage = mManageOfflineStatus;
+    return NS_OK;
+}
+
+nsresult
+nsIOService::TrackNetworkLinkStatusForOffline()
+{
+    NS_ASSERTION(mManageOfflineStatus,
+                 "Don't call this unless we're managing the offline status");
+    if (!mNetworkLinkService)
+        return NS_ERROR_FAILURE;
+  
+    // check to make sure this won't collide with Autodial
+    if (mSocketTransportService) {
+        PRBool autodialEnabled = PR_FALSE;
+        mSocketTransportService->GetAutodialEnabled(&autodialEnabled);
+        // If autodialing-on-link-down is enabled, then pretend the link is
+        // always up for the purposes of offline management.
+        if (autodialEnabled)
+            return SetOffline(PR_FALSE);
+    }
+  
+    PRBool isUp;
+    nsresult rv = mNetworkLinkService->GetIsLinkUp(&isUp);
+    NS_ENSURE_SUCCESS(rv, rv);
+    return SetOffline(!isUp);
+}
+
+NS_IMETHODIMP
+nsIOService::EscapeString(const nsACString& aString,
+                          PRUint32 aEscapeType,
+                          nsACString& aResult)
+{
+  NS_ENSURE_ARG_RANGE(aEscapeType, 0, 3);
+
+  nsCAutoString stringCopy(aString);
+  nsCString result;
+
+  if (!NS_Escape(stringCopy, result, (nsEscapeMask) aEscapeType))
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  aResult.Assign(result);
+
+  return NS_OK;
+}
