@@ -21,6 +21,8 @@
  *
  * Contributor(s):
  *   Darin Fisher <darin@netscape.com>
+ *   Malcolm Smith <malsmith@cs.rmit.edu.au>
+ *   Andreas Otte <andreas.otte@debitel.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -45,10 +47,13 @@
 #include "nsStreamUtils.h"
 #include "nsNetSegmentUtils.h"
 #include "nsTransportUtils.h"
+#include "nsProxyInfo.h"
 #include "nsNetCID.h"
 #include "nsAutoLock.h"
+#include "nsAutoPtr.h"
 #include "nsCOMPtr.h"
 #include "netCore.h"
+#include "nsInt64.h"
 #include "prmem.h"
 #include "pratom.h"
 #include "plstr.h"
@@ -61,7 +66,6 @@
 #include "nsISocketProviderService.h"
 #include "nsISocketProvider.h"
 #include "nsISSLSocketControl.h"
-#include "nsIProxyInfo.h"
 #include "nsIPipe.h"
 
 #if defined(XP_WIN)
@@ -75,37 +79,26 @@ static NS_DEFINE_CID(kDNSServiceCID, NS_DNSSERVICE_CID);
 
 //-----------------------------------------------------------------------------
 
-class nsSocketEvent : public PLEvent
+class nsSocketEvent : public nsRunnable
 {
 public:
     nsSocketEvent(nsSocketTransport *transport, PRUint32 type,
                   nsresult status = NS_OK, nsISupports *param = nsnull)
-        : mType(type)
+        : mTransport(transport)
+        , mType(type)
         , mStatus(status)
         , mParam(param)
-    {
-        NS_ADDREF(transport);
-        PL_InitEvent(this, transport, HandleEvent, DestroyEvent);
-    }
+    {}
 
-    PR_STATIC_CALLBACK(void*)
-    HandleEvent(PLEvent *event)
+    NS_IMETHOD Run()
     {
-        nsSocketTransport *trans = (nsSocketTransport *) event->owner; 
-        nsSocketEvent *se = (nsSocketEvent *) event;
-        trans->OnSocketEvent(se->mType, se->mStatus, se->mParam);
-        return nsnull;
-    }
-
-    PR_STATIC_CALLBACK(void)
-    DestroyEvent(PLEvent *event)
-    {
-        nsSocketTransport *trans = (nsSocketTransport *) event->owner; 
-        NS_RELEASE(trans);
-        delete (nsSocketEvent *) event;
+        mTransport->OnSocketEvent(mType, mStatus, mParam);
+        return NS_OK;
     }
 
 private:
+    nsRefPtr<nsSocketTransport> mTransport;
+
     PRUint32              mType;
     nsresult              mStatus;
     nsCOMPtr<nsISupports> mParam;
@@ -120,7 +113,7 @@ static PRErrorCode RandomizeConnectError(PRErrorCode code)
 {
     //
     // To test out these errors, load http://www.yahoo.com/.  It should load
-    // correctly despite the random occurance of these errors.
+    // correctly despite the random occurrence of these errors.
     //
     int n = rand();
     if (n > RAND_MAX/2) {
@@ -171,6 +164,10 @@ ErrorAccordingToNSPR(PRErrorCode errorCode)
     case PR_NETWORK_UNREACHABLE_ERROR: // XXX need new nsresult for this!
     case PR_HOST_UNREACHABLE_ERROR:    // XXX and this!
     case PR_ADDRESS_NOT_AVAILABLE_ERROR:
+    // Treat EACCES as a soft error since (at least on Linux) connect() returns
+    // EACCES when an IPv6 connection is blocked by a firewall. See bug 270784.
+    case PR_ADDRESS_NOT_SUPPORTED_ERROR:
+    case PR_NO_ACCESS_RIGHTS_ERROR:
         rv = NS_ERROR_CONNECTION_REFUSED;
         break;
     case PR_IO_TIMEOUT_ERROR:
@@ -275,7 +272,7 @@ nsSocketInputStream::Available(PRUint32 *avail)
 
         fd = mTransport->GetFD_Locked();
         if (!fd)
-            return NS_BASE_STREAM_WOULD_BLOCK;
+            return NS_OK;
     }
 
     // cannot hold lock while calling NSPR.  (worried about the fact that PSM
@@ -294,7 +291,7 @@ nsSocketInputStream::Available(PRUint32 *avail)
         else {
             PRErrorCode code = PR_GetError();
             if (code == PR_WOULD_BLOCK_ERROR)
-                return NS_BASE_STREAM_WOULD_BLOCK;
+                return NS_OK;
             mCondition = ErrorAccordingToNSPR(code);
         }
         rv = mCondition;
@@ -407,10 +404,14 @@ nsSocketInputStream::AsyncWait(nsIInputStreamCallback *callback,
 {
     LOG(("nsSocketInputStream::AsyncWait [this=%x]\n", this));
 
+    // This variable will be non-null when we want to call the callback
+    // directly from this function, but outside the lock.
+    // (different from callback when target is not null)
+    nsCOMPtr<nsIInputStreamCallback> directCallback;
     {
         nsAutoLock lock(mTransport->mLock);
 
-        if (target) {
+        if (callback && target) {
             //
             // build event proxy
             //
@@ -426,9 +427,16 @@ nsSocketInputStream::AsyncWait(nsIInputStreamCallback *callback,
         else
             mCallback = callback;
 
-        mCallbackFlags = flags;
+        if (NS_FAILED(mCondition))
+            directCallback.swap(mCallback);
+        else
+            mCallbackFlags = flags;
     }
-    mTransport->OnInputPending();
+    if (directCallback)
+        directCallback->OnInputStreamReady(this);
+    else
+        mTransport->OnInputPending();
+
     return NS_OK;
 }
 
@@ -641,7 +649,7 @@ nsSocketOutputStream::AsyncWait(nsIOutputStreamCallback *callback,
     {
         nsAutoLock lock(mTransport->mLock);
 
-        if (target) {
+        if (callback && target) {
             //
             // build event proxy
             //
@@ -673,6 +681,7 @@ nsSocketTransport::nsSocketTransport()
     , mPort(0)
     , mProxyPort(0)
     , mProxyTransparent(PR_FALSE)
+    , mProxyTransparentResolvesHost(PR_FALSE)
     , mState(STATE_CLOSED)
     , mAttached(PR_FALSE)
     , mInputClosed(PR_TRUE)
@@ -688,6 +697,9 @@ nsSocketTransport::nsSocketTransport()
     LOG(("creating nsSocketTransport @%x\n", this));
 
     NS_ADDREF(gSocketTransportService);
+
+    mTimeouts[TIMEOUT_CONNECT]    = PR_UINT16_MAX; // no timeout
+    mTimeouts[TIMEOUT_READ_WRITE] = PR_UINT16_MAX; // no timeout
 }
 
 nsSocketTransport::~nsSocketTransport()
@@ -712,8 +724,17 @@ nsSocketTransport::~nsSocketTransport()
 nsresult
 nsSocketTransport::Init(const char **types, PRUint32 typeCount,
                         const nsACString &host, PRUint16 port,
-                        nsIProxyInfo *proxyInfo)
+                        nsIProxyInfo *givenProxyInfo)
 {
+    if (!mLock)
+        return NS_ERROR_OUT_OF_MEMORY;
+
+    nsCOMPtr<nsProxyInfo> proxyInfo;
+    if (givenProxyInfo) {
+        proxyInfo = do_QueryInterface(givenProxyInfo);
+        NS_ENSURE_ARG(proxyInfo);
+    }
+
     // init socket type info
 
     mPort = port;
@@ -726,7 +747,8 @@ nsSocketTransport::Init(const char **types, PRUint32 typeCount,
         // grab proxy type (looking for "socks" for example)
         proxyType = proxyInfo->Type();
         if (proxyType && (strcmp(proxyType, "http") == 0 ||
-                          strcmp(proxyType, "direct") == 0))
+                          strcmp(proxyType, "direct") == 0 ||
+                          strcmp(proxyType, "unknown") == 0))
             proxyType = nsnull;
     }
 
@@ -735,38 +757,50 @@ nsSocketTransport::Init(const char **types, PRUint32 typeCount,
 
     // include proxy type as a socket type if proxy type is not "http"
     mTypeCount = typeCount + (proxyType != nsnull);
-    if (mTypeCount) {
-        mTypes = (char **) malloc(mTypeCount * sizeof(char *));
-        if (!mTypes)
-            return NS_ERROR_OUT_OF_MEMORY;
+    if (!mTypeCount)
+        return NS_OK;
 
+    // if we have socket types, then the socket provider service had
+    // better exist!
+    nsresult rv;
+    nsCOMPtr<nsISocketProviderService> spserv =
+        do_GetService(kSocketProviderServiceCID, &rv);
+    if (NS_FAILED(rv)) return rv;
+
+    mTypes = (char **) malloc(mTypeCount * sizeof(char *));
+    if (!mTypes)
+        return NS_ERROR_OUT_OF_MEMORY;
+
+    // now verify that each socket type has a registered socket provider.
+    for (PRUint32 i = 0, type = 0; i < mTypeCount; ++i) {
         // store socket types
-        PRUint32 i, type = 0;
-        if (proxyType)
-            mTypes[type++] = PL_strdup(proxyType);
-        for (i=0; i<typeCount; ++i)
-            mTypes[type++] = PL_strdup(types[i]);
+        if (i == 0 && proxyType)
+            mTypes[i] = PL_strdup(proxyType);
+        else
+            mTypes[i] = PL_strdup(types[type++]);
 
-        // if we have socket types, then the socket provider service had
-        // better exist!
-        nsresult rv;
-        nsCOMPtr<nsISocketProviderService> spserv =
-            do_GetService(kSocketProviderServiceCID, &rv);
-        if (NS_FAILED(rv)) return rv;
+        if (!mTypes[i]) {
+            mTypeCount = i;
+            return NS_ERROR_OUT_OF_MEMORY;
+        }
+        nsCOMPtr<nsISocketProvider> provider;
+        rv = spserv->GetSocketProvider(mTypes[i], getter_AddRefs(provider));
+        if (NS_FAILED(rv)) {
+            NS_WARNING("no registered socket provider");
+            return rv;
+        }
 
-        // now verify that each socket type has a registered socket provider.
-        for (i=0; i<mTypeCount; ++i) {
-            nsCOMPtr<nsISocketProvider> provider;
-            rv = spserv->GetSocketProvider(mTypes[i], getter_AddRefs(provider));
-            if (NS_FAILED(rv)) {
-                NS_WARNING("no registered socket provider");
-                return rv;
+        // note if socket type corresponds to a transparent proxy
+        // XXX don't hardcode SOCKS here (use proxy info's flags instead).
+        if ((strcmp(mTypes[i], "socks") == 0) ||
+            (strcmp(mTypes[i], "socks4") == 0)) {
+            mProxyTransparent = PR_TRUE;
+
+            if (proxyInfo->Flags() & nsIProxyInfo::TRANSPARENT_PROXY_RESOLVES_HOST) {
+                // we want the SOCKS layer to send the hostname
+                // and port to the proxy and let it do the DNS.
+                mProxyTransparentResolvesHost = PR_TRUE;
             }
-
-            // note if socket type corresponds to a transparent proxy
-            if ((strcmp(mTypes[i], "socks") == 0) ||
-                (strcmp(mTypes[i], "socks4") == 0))
-                mProxyTransparent = PR_TRUE;
         }
     }
 
@@ -776,6 +810,9 @@ nsSocketTransport::Init(const char **types, PRUint32 typeCount,
 nsresult
 nsSocketTransport::InitWithConnectedSocket(PRFileDesc *fd, const PRNetAddr *addr)
 {
+    if (!mLock)
+        return NS_ERROR_OUT_OF_MEMORY;
+
     NS_ASSERTION(!mFD, "already initialized");
 
     char buf[64];
@@ -792,6 +829,7 @@ nsSocketTransport::InitWithConnectedSocket(PRFileDesc *fd, const PRNetAddr *addr
     memcpy(&mNetAddr, addr, sizeof(PRNetAddr));
 
     mPollFlags = (PR_POLL_READ | PR_POLL_WRITE | PR_POLL_EXCEPT);
+    mPollTimeout = mTimeouts[TIMEOUT_READ_WRITE];
     mState = STATE_TRANSFERRING;
 
     mFD = fd;
@@ -817,15 +855,11 @@ nsSocketTransport::PostEvent(PRUint32 type, nsresult status, nsISupports *param)
     LOG(("nsSocketTransport::PostEvent [this=%p type=%u status=%x param=%p]\n",
         this, type, status, param));
 
-    PLEvent *event = new nsSocketEvent(this, type, status, param);
+    nsCOMPtr<nsIRunnable> event = new nsSocketEvent(this, type, status, param);
     if (!event)
         return NS_ERROR_OUT_OF_MEMORY;
 
-    nsresult rv = gSocketTransportService->PostEvent(event);
-    if (NS_FAILED(rv))
-        PL_DestroyEvent(event);
-
-    return rv;
+    return gSocketTransportService->Dispatch(event, NS_DISPATCH_NORMAL);
 }
 
 void
@@ -834,7 +868,7 @@ nsSocketTransport::SendStatus(nsresult status)
     LOG(("nsSocketTransport::SendStatus [this=%x status=%x]\n", this, status));
 
     nsCOMPtr<nsITransportEventSink> sink;
-    PRUint32 progress;
+    PRUint64 progress;
     {
         nsAutoLock lock(mLock);
         sink = mEventSink;
@@ -851,7 +885,7 @@ nsSocketTransport::SendStatus(nsresult status)
         }
     }
     if (sink)
-        sink->OnTransportStatus(this, status, progress, PR_UINT32_MAX);
+        sink->OnTransportStatus(this, status, progress, LL_MAXUINT);
 }
 
 nsresult
@@ -861,6 +895,27 @@ nsSocketTransport::ResolveHost()
 
     nsresult rv;
 
+    if (!mProxyHost.IsEmpty()) {
+        if (!mProxyTransparent || mProxyTransparentResolvesHost) {
+            // When not resolving mHost locally, we still want to ensure that
+            // it only contains valid characters.  See bug 304904 for details.
+            if (!net_IsValidHostName(mHost))
+                return NS_ERROR_UNKNOWN_HOST;
+        }
+        if (mProxyTransparentResolvesHost) {
+            // Name resolution is done on the server side.  Just pretend
+            // client resolution is complete, this will get picked up later.
+            // since we don't need to do DNS now, we bypass the resolving
+            // step by initializing mNetAddr to an empty address, but we
+            // must keep the port. The SOCKS IO layer will use the hostname
+            // we send it when it's created, rather than the empty address
+            // we send with the connect call.
+            mState = STATE_RESOLVING;
+            PR_SetNetAddr(PR_IpAddrAny, PR_AF_INET, SocketPort(), &mNetAddr);
+            return PostEvent(MSG_DNS_LOOKUP_COMPLETE, NS_OK, nsnull);
+        }
+    }
+
     nsCOMPtr<nsIDNSService> dns = do_GetService(kDNSServiceCID, &rv);
     if (NS_FAILED(rv)) return rv;
 
@@ -899,11 +954,12 @@ nsSocketTransport::BuildSocket(PRFileDesc *&fd, PRBool &proxyTransparent, PRBool
             do_GetService(kSocketProviderServiceCID, &rv);
         if (NS_FAILED(rv)) return rv;
 
-        const char *host      = mHost.get();
-        PRInt32     port      = (PRInt32) mPort;
-        const char *proxyHost = mProxyHost.IsEmpty() ? nsnull : mProxyHost.get();
-        PRInt32     proxyPort = (PRInt32) mProxyPort;
-        
+        const char *host       = mHost.get();
+        PRInt32     port       = (PRInt32) mPort;
+        const char *proxyHost  = mProxyHost.IsEmpty() ? nsnull : mProxyHost.get();
+        PRInt32     proxyPort  = (PRInt32) mProxyPort;
+        PRUint32    proxyFlags = 0;
+
         PRUint32 i;
         for (i=0; i<mTypeCount; ++i) {
             nsCOMPtr<nsISocketProvider> provider;
@@ -914,13 +970,17 @@ nsSocketTransport::BuildSocket(PRFileDesc *&fd, PRBool &proxyTransparent, PRBool
             if (NS_FAILED(rv))
                 break;
 
+            if (mProxyTransparentResolvesHost)
+                proxyFlags |= nsISocketProvider::PROXY_RESOLVES_HOST;
+
             nsCOMPtr<nsISupports> secinfo;
             if (i == 0) {
                 // if this is the first type, we'll want the 
                 // service to allocate a new socket
                 rv = provider->NewSocket(mNetAddr.raw.family,
                                          host, port, proxyHost, proxyPort,
-                                         &fd, getter_AddRefs(secinfo));
+                                         proxyFlags, &fd,
+                                         getter_AddRefs(secinfo));
 
                 if (NS_SUCCEEDED(rv) && !fd) {
                     NS_NOTREACHED("NewSocket succeeded but failed to create a PRFileDesc");
@@ -933,8 +993,10 @@ nsSocketTransport::BuildSocket(PRFileDesc *&fd, PRBool &proxyTransparent, PRBool
                 // to the stack (such as pushing an io layer)
                 rv = provider->AddToSocket(mNetAddr.raw.family,
                                            host, port, proxyHost, proxyPort,
-                                           fd, getter_AddRefs(secinfo));
+                                           proxyFlags, fd,
+                                           getter_AddRefs(secinfo));
             }
+            proxyFlags = 0;
             if (NS_FAILED(rv))
                 break;
 
@@ -996,13 +1058,11 @@ nsSocketTransport::InitiateSocket()
     // 194402 for more info.
     //
     if (!gSocketTransportService->CanAttachSocket()) {
-        PLEvent *event = new nsSocketEvent(this, MSG_RETRY_INIT_SOCKET);
+        nsCOMPtr<nsIRunnable> event =
+                new nsSocketEvent(this, MSG_RETRY_INIT_SOCKET);
         if (!event)
             return NS_ERROR_OUT_OF_MEMORY;
-        rv = gSocketTransportService->NotifyWhenCanAttachSocket(event);
-        if (NS_FAILED(rv))
-            PL_DestroyEvent(event);
-        return rv;
+        return gSocketTransportService->NotifyWhenCanAttachSocket(event);
     }
 
     //
@@ -1056,6 +1116,7 @@ nsSocketTransport::InitiateSocket()
 
     LOG(("  advancing to STATE_CONNECTING\n"));
     mState = STATE_CONNECTING;
+    mPollTimeout = mTimeouts[TIMEOUT_CONNECT];
     SendStatus(STATUS_CONNECTING_TO);
 
 #if defined(PR_LOGGING)
@@ -1243,6 +1304,7 @@ nsSocketTransport::OnSocketConnected()
     LOG(("  advancing to STATE_TRANSFERRING\n"));
 
     mPollFlags = (PR_POLL_READ | PR_POLL_WRITE | PR_POLL_EXCEPT);
+    mPollTimeout = mTimeouts[TIMEOUT_READ_WRITE];
     mState = STATE_TRANSFERRING;
 
     SendStatus(STATUS_CONNECTED_TO);
@@ -1324,8 +1386,13 @@ nsSocketTransport::OnSocketEvent(PRUint32 type, nsresult status, nsISupports *pa
         }
         // status contains DNS lookup status
         if (NS_FAILED(status)) {
-            // fixup error code if proxy was not found
-            if ((status == NS_ERROR_UNKNOWN_HOST) && !mProxyHost.IsEmpty())
+            // When using a HTTP proxy, NS_ERROR_UNKNOWN_HOST means the HTTP 
+            // proxy host is not found, so we fixup the error code.
+            // For SOCKS proxies (mProxyTransparent == true), the socket 
+            // transport resolves the real host here, so there's no fixup 
+            // (see bug 226943).
+            if ((status == NS_ERROR_UNKNOWN_HOST) && !mProxyTransparent &&
+                !mProxyHost.IsEmpty())
                 mCondition = NS_ERROR_UNKNOWN_PROXY_HOST;
             else
                 mCondition = status;
@@ -1357,7 +1424,11 @@ nsSocketTransport::OnSocketEvent(PRUint32 type, nsresult status, nsISupports *pa
         LOG(("  MSG_OUTPUT_PENDING\n"));
         OnMsgOutputPending();
         break;
-
+    case MSG_TIMEOUT_CHANGED:
+        LOG(("  MSG_TIMEOUT_CHANGED\n"));
+        mPollTimeout = mTimeouts[(mState == STATE_TRANSFERRING)
+          ? TIMEOUT_READ_WRITE : TIMEOUT_CONNECT];
+        break;
     default:
         LOG(("  unhandled event!\n"));
     }
@@ -1380,6 +1451,12 @@ nsSocketTransport::OnSocketReady(PRFileDesc *fd, PRInt16 outFlags)
     LOG(("nsSocketTransport::OnSocketReady [this=%x outFlags=%hd]\n",
         this, outFlags));
 
+    if (outFlags == -1) {
+        LOG(("socket timeout expired\n"));
+        mCondition = NS_ERROR_NET_TIMEOUT;
+        return;
+    }
+
     if (mState == STATE_TRANSFERRING) {
         // if waiting to write and socket is writable or hit an exception.
         if ((mPollFlags & PR_POLL_WRITE) && (outFlags & ~PR_POLL_READ)) {
@@ -1395,6 +1472,8 @@ nsSocketTransport::OnSocketReady(PRFileDesc *fd, PRInt16 outFlags)
             mPollFlags &= ~PR_POLL_READ;
             mInput.OnSocketReady(NS_OK);
         }
+        // Update poll timeout in case it was changed
+        mPollTimeout = mTimeouts[TIMEOUT_READ_WRITE];
     }
     else if (mState == STATE_CONNECTING) {
         PRStatus status = PR_ConnectContinue(fd, outFlags);
@@ -1415,6 +1494,8 @@ nsSocketTransport::OnSocketReady(PRFileDesc *fd, PRInt16 outFlags)
             if ((PR_WOULD_BLOCK_ERROR == code) || (PR_IN_PROGRESS_ERROR == code)) {
                 // Set up the select flags for connect...
                 mPollFlags = (PR_POLL_EXCEPT | PR_POLL_WRITE);
+                // Update poll timeout in case it was changed
+                mPollTimeout = mTimeouts[TIMEOUT_CONNECT];
             } 
             else {
                 //
@@ -1457,7 +1538,7 @@ nsSocketTransport::OnSocketDetached(PRFileDesc *fd)
 
         // make sure there isn't any pending DNS request
         if (mDNSRequest) {
-            mDNSRequest->Cancel();
+            mDNSRequest->Cancel(NS_ERROR_ABORT);
             mDNSRequest = 0;
         }
 
@@ -1468,6 +1549,13 @@ nsSocketTransport::OnSocketDetached(PRFileDesc *fd)
         mOutput.OnSocketReady(mCondition);
     }
 
+    // break any potential reference cycle between the security info object
+    // and ourselves by resetting its notification callbacks object.  see
+    // bug 285991 for details.
+    nsCOMPtr<nsISSLSocketControl> secCtrl = do_QueryInterface(mSecInfo);
+    if (secCtrl)
+        secCtrl->SetNotificationCallbacks(nsnull);
+
     // finally, release our reference to the socket (must do this within
     // the transport lock) possibly closing the socket.
     {
@@ -1724,7 +1812,25 @@ nsSocketTransport::GetSelfAddr(PRNetAddr *addr)
 }
 
 NS_IMETHODIMP
-nsSocketTransport::OnLookupComplete(nsIDNSRequest *request,
+nsSocketTransport::GetTimeout(PRUint32 type, PRUint32 *value)
+{
+    NS_ENSURE_ARG_MAX(type, nsISocketTransport::TIMEOUT_READ_WRITE);
+    *value = (PRUint32) mTimeouts[type];
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsSocketTransport::SetTimeout(PRUint32 type, PRUint32 value)
+{
+    NS_ENSURE_ARG_MAX(type, nsISocketTransport::TIMEOUT_READ_WRITE);
+    // truncate overly large timeout values.
+    mTimeouts[type] = (PRUint16) PR_MIN(value, PR_UINT16_MAX);
+    PostEvent(MSG_TIMEOUT_CHANGED);
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsSocketTransport::OnLookupComplete(nsICancelable *request,
                                     nsIDNSRecord  *rec,
                                     nsresult       status)
 {
