@@ -49,19 +49,24 @@
 #include "nsICharsetConverterManager.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "nsIIDNService.h"
 #include "nsNetUtil.h"
 #include "prlog.h"
+#include "nsAutoPtr.h"
+#include "nsIProgrammingLanguage.h"
 
 static NS_DEFINE_CID(kThisImplCID, NS_THIS_STANDARDURL_IMPL_CID);
 static NS_DEFINE_CID(kStandardURLCID, NS_STANDARDURL_CID);
 
-nsIIDNService *nsStandardURL::gIDNService = nsnull;
+nsIIDNService *nsStandardURL::gIDN = nsnull;
 nsICharsetConverterManager *nsStandardURL::gCharsetMgr = nsnull;
 PRBool nsStandardURL::gInitialized = PR_FALSE;
 PRBool nsStandardURL::gEscapeUTF8 = PR_TRUE;
-PRBool nsStandardURL::gAlwaysEncodeInUTF8 = PR_FALSE;
+PRBool nsStandardURL::gAlwaysEncodeInUTF8 = PR_TRUE;
+PRBool nsStandardURL::gEncodeQueryInUTF8 = PR_TRUE;
+PRBool nsStandardURL::gShowPunycode = PR_FALSE;
+nsIPrefBranch *nsStandardURL::gIDNWhitelistPrefBranch = nsnull;
 
 #if defined(PR_LOGGING)
 //
@@ -134,6 +139,9 @@ end:
 #define NS_NET_PREF_ESCAPEUTF8         "network.standard-url.escape-utf8"
 #define NS_NET_PREF_ENABLEIDN          "network.enableIDN"
 #define NS_NET_PREF_ALWAYSENCODEINUTF8 "network.standard-url.encode-utf8"
+#define NS_NET_PREF_ENCODEQUERYINUTF8  "network.standard-url.encode-query-utf8"
+#define NS_NET_PREF_SHOWPUNYCODE       "network.IDN_show_punycode"
+#define NS_NET_PREF_IDNWHITELIST       "network.IDN.whitelist."
 
 NS_IMPL_ISUPPORTS1(nsStandardURL::nsPrefObserver, nsIObserver)
 
@@ -145,28 +153,7 @@ nsPrefObserver::Observe(nsISupports *subject,
     if (!strcmp(topic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
         nsCOMPtr<nsIPrefBranch> prefBranch( do_QueryInterface(subject) );
         if (prefBranch) {
-            if (!nsCRT::strcmp(data, NS_LITERAL_STRING(NS_NET_PREF_ESCAPEUTF8).get())) {
-                PRBool val;
-                if (NS_SUCCEEDED(prefBranch->GetBoolPref(NS_NET_PREF_ESCAPEUTF8, &val)))
-                    gEscapeUTF8 = val;
-                LOG(("escape UTF-8 %s\n", gEscapeUTF8 ? "enabled" : "disabled"));
-            }
-            else if (!nsCRT::strcmp(data, NS_LITERAL_STRING(NS_NET_PREF_ENABLEIDN).get())) {
-                PRBool val;
-                NS_IF_RELEASE(gIDNService);
-                if (NS_SUCCEEDED(prefBranch->GetBoolPref(NS_NET_PREF_ENABLEIDN, &val)) && val) {
-                    nsCOMPtr<nsIIDNService> serv(do_GetService(NS_IDNSERVICE_CONTRACTID));
-                    if (serv)
-                        NS_ADDREF(gIDNService = serv.get());
-                }
-                LOG(("IDN support %s\n", gIDNService ? "enabled" : "disabled"));
-            }
-            else if (!nsCRT::strcmp(data, NS_LITERAL_STRING(NS_NET_PREF_ALWAYSENCODEINUTF8).get())) {
-                PRBool val;
-                if (NS_SUCCEEDED(prefBranch->GetBoolPref(NS_NET_PREF_ALWAYSENCODEINUTF8, &val)))
-                    gAlwaysEncodeInUTF8 = val;
-                LOG(("encode in UTF-8 %s\n", gAlwaysEncodeInUTF8 ? "enabled" : "disabled"));
-            }
+            PrefsChanged(prefBranch, NS_ConvertUTF16toUTF8(data).get());
         } 
     }
     return NS_OK;
@@ -186,8 +173,10 @@ PRInt32 nsStandardURL::
 nsSegmentEncoder::EncodeSegmentCount(const char *str,
                                      const URLSegment &seg,
                                      PRInt16 mask,
-                                     nsAFlatCString &result)
+                                     nsAFlatCString &result,
+                                     PRBool &appended)
 {
+    appended = PR_FALSE;
     if (!str)
         return 0;
     PRInt32 len = 0;
@@ -200,10 +189,10 @@ nsSegmentEncoder::EncodeSegmentCount(const char *str,
         // null or the empty string then the origin charset is UTF-8 and there
         // is nothing to do.
         nsCAutoString encBuf;
-        if (!nsCRT::IsAscii(str + pos, len) && mCharset && *mCharset) {
+        if (mCharset && *mCharset && !nsCRT::IsAscii(str + pos, len)) {
             // we have to encode this segment
             if (mEncoder || InitUnicodeEncoder()) {
-                NS_ConvertUTF8toUCS2 ucsBuf(Substring(str + pos, str + pos + len));
+                NS_ConvertUTF8toUTF16 ucsBuf(Substring(str + pos, str + pos + len));
                 if (NS_SUCCEEDED(EncodeString(mEncoder, ucsBuf, encBuf))) {
                     str = encBuf.get();
                     pos = 0;
@@ -219,11 +208,14 @@ nsSegmentEncoder::EncodeSegmentCount(const char *str,
         PRUint32 initLen = result.Length();
 
         // now perform any required escaping
-        if (NS_EscapeURL(str + pos, len, mask | escapeFlags, result))
+        if (NS_EscapeURL(str + pos, len, mask | escapeFlags, result)) {
             len = result.Length() - initLen;
+            appended = PR_TRUE;
+        }
         else if (str == encBuf.get()) {
             result += encBuf; // append only!!
             len = encBuf.Length();
+            appended = PR_TRUE;
         }
     }
     return len;
@@ -235,13 +227,11 @@ nsSegmentEncoder::EncodeSegment(const nsASingleFragmentCString &str,
                                 nsAFlatCString &result)
 {
     const char *text;
-    PRUint32 resultLen = result.Length();
-    EncodeSegmentCount(str.BeginReading(text), URLSegment(0, str.Length()), mask, result);
-    // since EncodeSegmentCount appends to result, we must check whether its length grew
-    if (result.Length() > resultLen)
+    PRBool encoded;
+    EncodeSegmentCount(str.BeginReading(text), URLSegment(0, str.Length()), mask, result, encoded);
+    if (encoded)
         return result;
-    else
-        return str;
+    return str;
 }
 
 PRBool nsStandardURL::
@@ -268,8 +258,15 @@ nsSegmentEncoder::InitUnicodeEncoder()
     return PR_TRUE;
 }
 
+#define GET_SEGMENT_ENCODER_INTERNAL(name, useUTF8) \
+    nsSegmentEncoder name(useUTF8 ? nsnull : mOriginCharset.get())
+
 #define GET_SEGMENT_ENCODER(name) \
-    nsSegmentEncoder name(mOriginCharset.get())
+    GET_SEGMENT_ENCODER_INTERNAL(name, gAlwaysEncodeInUTF8)
+
+#define GET_QUERY_ENCODER(name) \
+    GET_SEGMENT_ENCODER_INTERNAL(name, gAlwaysEncodeInUTF8 && \
+                                 gEncodeQueryInUTF8)
 
 //----------------------------------------------------------------------------
 // nsStandardURL <public>
@@ -311,24 +308,33 @@ nsStandardURL::~nsStandardURL()
 void
 nsStandardURL::InitGlobalObjects()
 {
-    nsCOMPtr<nsIPrefBranchInternal> prefBranch( do_GetService(NS_PREFSERVICE_CONTRACTID) );
+    nsCOMPtr<nsIPrefBranch2> prefBranch( do_GetService(NS_PREFSERVICE_CONTRACTID) );
     if (prefBranch) {
         nsCOMPtr<nsIObserver> obs( new nsPrefObserver() );
-        prefBranch->AddObserver(NS_NET_PREF_ESCAPEUTF8, obs.get(), PR_FALSE); 
+        prefBranch->AddObserver(NS_NET_PREF_ESCAPEUTF8, obs.get(), PR_FALSE);
         prefBranch->AddObserver(NS_NET_PREF_ALWAYSENCODEINUTF8, obs.get(), PR_FALSE);
-        prefBranch->AddObserver(NS_NET_PREF_ENABLEIDN, obs.get(), PR_FALSE); 
-        // initialize IDN
-        nsCOMPtr<nsIIDNService> serv(do_GetService(NS_IDNSERVICE_CONTRACTID));
-        if (serv)
-            NS_ADDREF(gIDNService = serv.get());
+        prefBranch->AddObserver(NS_NET_PREF_ENCODEQUERYINUTF8, obs.get(), PR_FALSE);
+        prefBranch->AddObserver(NS_NET_PREF_ENABLEIDN, obs.get(), PR_FALSE);
+        prefBranch->AddObserver(NS_NET_PREF_SHOWPUNYCODE, obs.get(), PR_FALSE);
+
+        PrefsChanged(prefBranch, nsnull);
+
+        nsCOMPtr<nsIPrefService> prefs = do_QueryInterface(prefBranch); 
+        if (prefs) {
+            nsCOMPtr<nsIPrefBranch> branch;
+           if (NS_SUCCEEDED(prefs->GetBranch( NS_NET_PREF_IDNWHITELIST,
+                                              getter_AddRefs(branch) )))
+               NS_ADDREF(gIDNWhitelistPrefBranch = branch);
+        }
     }
 }
 
 void
 nsStandardURL::ShutdownGlobalObjects()
 {
-    NS_IF_RELEASE(gIDNService);
+    NS_IF_RELEASE(gIDN);
     NS_IF_RELEASE(gCharsetMgr);
+    NS_IF_RELEASE(gIDNWhitelistPrefBranch);
 }
 
 //----------------------------------------------------------------------------
@@ -401,16 +407,16 @@ nsStandardURL::NormalizeIDN(const nsCSubstring &host, nsCString &result)
 
     if (IsASCII(host)) {
         PRBool isACE;
-        if (gIDNService &&
-            NS_SUCCEEDED(gIDNService->IsACE(host, &isACE)) && isACE &&
-            NS_SUCCEEDED(gIDNService->ConvertACEtoUTF8(host, result))) {
+        if (gIDN &&
+            NS_SUCCEEDED(gIDN->IsACE(host, &isACE)) && isACE &&
+            NS_SUCCEEDED(ACEtoDisplayIDN(host, result))) {
             mHostEncoding = eEncoding_UTF8;
             return PR_TRUE;
         }
     }
     else {
         mHostEncoding = eEncoding_UTF8;
-        if (gIDNService && NS_SUCCEEDED(gIDNService->Normalize(host, result))) {
+        if (gIDN && NS_SUCCEEDED(UTF8toDisplayIDN(host, result))) {
             // normalization could result in an ASCII only hostname
             if (IsASCII(result))
                 mHostEncoding = eEncoding_ASCII;
@@ -437,10 +443,10 @@ nsStandardURL::CoalescePath(netCoalesceFlags coalesceFlag, char *path)
 }
 
 PRUint32
-nsStandardURL::AppendSegmentToBuf(char *buf, PRUint32 i, const char *str, URLSegment &seg, const nsCString *escapedStr)
+nsStandardURL::AppendSegmentToBuf(char *buf, PRUint32 i, const char *str, URLSegment &seg, const nsCString *escapedStr, PRBool useEscaped)
 {
     if (seg.mLen > 0) {
-        if (escapedStr && !escapedStr->IsEmpty()) {
+        if (useEscaped) {
             seg.mLen = escapedStr->Length();
             memcpy(buf + i, escapedStr->get(), seg.mLen);
         }
@@ -472,21 +478,16 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
 
     // buffers for holding escaped url segments (these will remain empty unless
     // escaping is required).
-    nsCAutoString encUsername;
-    nsCAutoString encPassword;
-    nsCAutoString encHost;
-    nsCAutoString encDirectory;
-    nsCAutoString encBasename;
-    nsCAutoString encExtension;
-    nsCAutoString encParam;
-    nsCAutoString encQuery;
-    nsCAutoString encRef;
+    nsCAutoString encUsername, encPassword, encHost, encDirectory,
+      encBasename, encExtension, encParam, encQuery, encRef;
+    PRBool useEncUsername, useEncPassword, useEncHost, useEncDirectory,
+      useEncBasename, useEncExtension, useEncParam, useEncQuery, useEncRef;
 
     //
     // escape each URL segment, if necessary, and calculate approximate normalized
     // spec length.
     //
-    PRInt32 approxLen = 3; // includes room for "://"
+    PRUint32 approxLen = 3; // includes room for "://"
 
     // the scheme is already ASCII
     if (mScheme.mLen > 0)
@@ -497,14 +498,15 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
     // appropriate encoding.
     {
         GET_SEGMENT_ENCODER(encoder);
-        approxLen += encoder.EncodeSegmentCount(spec, mUsername,  esc_Username,      encUsername);
-        approxLen += encoder.EncodeSegmentCount(spec, mPassword,  esc_Password,      encPassword);
-        approxLen += encoder.EncodeSegmentCount(spec, mDirectory, esc_Directory,     encDirectory);
-        approxLen += encoder.EncodeSegmentCount(spec, mBasename,  esc_FileBaseName,  encBasename);
-        approxLen += encoder.EncodeSegmentCount(spec, mExtension, esc_FileExtension, encExtension);
-        approxLen += encoder.EncodeSegmentCount(spec, mParam,     esc_Param,         encParam);
-        approxLen += encoder.EncodeSegmentCount(spec, mQuery,     esc_Query,         encQuery);
-        approxLen += encoder.EncodeSegmentCount(spec, mRef,       esc_Ref,           encRef);
+        GET_QUERY_ENCODER(queryEncoder);
+        approxLen += encoder.EncodeSegmentCount(spec, mUsername,  esc_Username,      encUsername,  useEncUsername);
+        approxLen += encoder.EncodeSegmentCount(spec, mPassword,  esc_Password,      encPassword,  useEncPassword);
+        approxLen += encoder.EncodeSegmentCount(spec, mDirectory, esc_Directory,     encDirectory, useEncDirectory);
+        approxLen += encoder.EncodeSegmentCount(spec, mBasename,  esc_FileBaseName,  encBasename,  useEncBasename);
+        approxLen += encoder.EncodeSegmentCount(spec, mExtension, esc_FileExtension, encExtension, useEncExtension);
+        approxLen += encoder.EncodeSegmentCount(spec, mParam,     esc_Param,         encParam,     useEncParam);
+        approxLen += queryEncoder.EncodeSegmentCount(spec, mQuery, esc_Query,        encQuery,     useEncQuery);
+        approxLen += encoder.EncodeSegmentCount(spec, mRef,       esc_Ref,           encRef,       useEncRef);
     }
 
     // do not escape the hostname, if IPv6 address literal, mHost will
@@ -514,7 +516,7 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
     if (mHost.mLen > 0) {
         const nsCSubstring& tempHost =
             Substring(spec + mHost.mPos, spec + mHost.mPos + mHost.mLen);
-        if (NormalizeIDN(tempHost, encHost))
+        if ((useEncHost = NormalizeIDN(tempHost, encHost)))
             approxLen += encHost.Length();
         else
             approxLen += mHost.mLen;
@@ -523,7 +525,8 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
     //
     // generate the normalized URL string
     //
-    mSpec.SetLength(approxLen + 32);
+    if (!EnsureStringLength(mSpec, approxLen + 32))
+        return NS_ERROR_OUT_OF_MEMORY;
     char *buf;
     mSpec.BeginWriting(buf);
     PRUint32 i = 0;
@@ -539,15 +542,15 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
 
     // append authority
     if (mUsername.mLen > 0) {
-        i = AppendSegmentToBuf(buf, i, spec, mUsername, &encUsername);
+        i = AppendSegmentToBuf(buf, i, spec, mUsername, &encUsername, useEncUsername);
         if (mPassword.mLen >= 0) {
             buf[i++] = ':';
-            i = AppendSegmentToBuf(buf, i, spec, mPassword, &encPassword);
+            i = AppendSegmentToBuf(buf, i, spec, mPassword, &encPassword, useEncPassword);
         }
         buf[i++] = '@';
     }
     if (mHost.mLen > 0) {
-        i = AppendSegmentToBuf(buf, i, spec, mHost, &encHost);
+        i = AppendSegmentToBuf(buf, i, spec, mHost, &encHost, useEncHost);
         net_ToLowerCase(buf + mHost.mPos, mHost.mLen);
         if (mPort != -1 && mPort != mDefaultPort) {
             nsCAutoString portbuf;
@@ -581,7 +584,7 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
         // record corrected (file)path starting position
         mPath.mPos = mFilepath.mPos = i - leadingSlash;
 
-        i = AppendSegmentToBuf(buf, i, spec, mDirectory, &encDirectory);
+        i = AppendSegmentToBuf(buf, i, spec, mDirectory, &encDirectory, useEncDirectory);
 
         // the directory must end with a '/'
         if (buf[i-1] != '/') {
@@ -589,7 +592,7 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
             mDirectory.mLen++;
         }
 
-        i = AppendSegmentToBuf(buf, i, spec, mBasename, &encBasename);
+        i = AppendSegmentToBuf(buf, i, spec, mBasename, &encBasename, useEncBasename);
 
         // make corrections to directory segment if leadingSlash
         if (leadingSlash) {
@@ -602,22 +605,22 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
 
         if (mExtension.mLen >= 0) {
             buf[i++] = '.';
-            i = AppendSegmentToBuf(buf, i, spec, mExtension, &encExtension);
+            i = AppendSegmentToBuf(buf, i, spec, mExtension, &encExtension, useEncExtension);
         }
         // calculate corrected filepath length
         mFilepath.mLen = i - mFilepath.mPos;
 
         if (mParam.mLen >= 0) {
             buf[i++] = ';';
-            i = AppendSegmentToBuf(buf, i, spec, mParam, &encParam);
+            i = AppendSegmentToBuf(buf, i, spec, mParam, &encParam, useEncParam);
         }
         if (mQuery.mLen >= 0) {
             buf[i++] = '?';
-            i = AppendSegmentToBuf(buf, i, spec, mQuery, &encQuery);
+            i = AppendSegmentToBuf(buf, i, spec, mQuery, &encQuery, useEncQuery);
         }
         if (mRef.mLen >= 0) {
             buf[i++] = '#';
-            i = AppendSegmentToBuf(buf, i, spec, mRef, &encRef);
+            i = AppendSegmentToBuf(buf, i, spec, mRef, &encRef, useEncRef);
         }
         // calculate corrected path length
         mPath.mLen = i - mPath.mPos;
@@ -635,28 +638,10 @@ nsStandardURL::BuildNormalizedSpec(const char *spec)
         CoalescePath(coalesceFlag, buf + mDirectory.mPos);
     }
     mSpec.SetLength(strlen(buf));
+    NS_ASSERTION(mSpec.Length() <= approxLen+32, "We've overflowed the mSpec buffer!");
     return NS_OK;
 }
 
-PRBool
-nsStandardURL::HostsAreEquivalent(nsStandardURL *that)
-{
-    // XXX now that hostnames are always normalized this special casing
-    // may be unnecessary.  we should be able to directly compare the
-    // contents of mSpec.
-
-    // optimize for the non-IDN case...
-    if ((this->mHostEncoding == eEncoding_ASCII) &&
-        (that->mHostEncoding == eEncoding_ASCII))
-        return SegmentIs(mHost, that->mSpec.get(), that->mHost);
-
-    nsCAutoString thisHost, thatHost;
-    this->GetAsciiHost(thisHost);
-    that->GetAsciiHost(thatHost);
-    
-    return !nsCRT::strcasecmp(thisHost.get(), thatHost.get());
-}
-
 PRBool
 nsStandardURL::SegmentIs(const URLSegment &seg, const char *val)
 {
@@ -667,7 +652,7 @@ nsStandardURL::SegmentIs(const URLSegment &seg, const char *val)
         return PR_FALSE;
     // if the first |seg.mLen| chars of |val| match, then |val| must
     // also be null terminated at |seg.mLen|.
-    return !nsCRT::strncasecmp(mSpec.get() + seg.mPos, val, seg.mLen)
+    return !strncmp(mSpec.get() + seg.mPos, val, seg.mLen)
         && (val[seg.mLen] == '\0');
 }
 
@@ -681,7 +666,7 @@ nsStandardURL::SegmentIs(const char* spec, const URLSegment &seg, const char *va
         return PR_FALSE;
     // if the first |seg.mLen| chars of |val| match, then |val| must
     // also be null terminated at |seg.mLen|.
-    return !nsCRT::strncasecmp(spec + seg.mPos, val, seg.mLen)
+    return !strncmp(spec + seg.mPos, val, seg.mLen)
         && (val[seg.mLen] == '\0');
 }
 
@@ -692,7 +677,7 @@ nsStandardURL::SegmentIs(const URLSegment &seg1, const char *val, const URLSegme
         return PR_FALSE;
     if (seg1.mLen == -1 || (!val && mSpec.IsEmpty()))
         return PR_TRUE; // both are empty
-    return !nsCRT::strncasecmp(mSpec.get() + seg1.mPos, val + seg2.mPos, seg1.mLen); 
+    return !strncmp(mSpec.get() + seg1.mPos, val + seg2.mPos, seg1.mLen); 
 }
 
 PRInt32
@@ -836,6 +821,106 @@ nsStandardURL::WriteSegment(nsIBinaryOutputStream *stream, const URLSegment &seg
     return NS_OK;
 }
 
+/* static */ void
+nsStandardURL::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
+{
+    PRBool val;
+
+    LOG(("nsStandardURL::PrefsChanged [pref=%s]\n", pref));
+
+#define PREF_CHANGED(p) ((pref == nsnull) || !strcmp(pref, p))
+#define GOT_PREF(p, b) (NS_SUCCEEDED(prefs->GetBoolPref(p, &b)))
+
+    if (PREF_CHANGED(NS_NET_PREF_ENABLEIDN)) {
+        NS_IF_RELEASE(gIDN);
+        if (GOT_PREF(NS_NET_PREF_ENABLEIDN, val) && val) {
+            // initialize IDN
+            nsCOMPtr<nsIIDNService> serv(do_GetService(NS_IDNSERVICE_CONTRACTID));
+            if (serv)
+                NS_ADDREF(gIDN = serv.get());
+        }
+        LOG(("IDN support %s\n", gIDN ? "enabled" : "disabled"));
+    }
+    
+    if (PREF_CHANGED(NS_NET_PREF_ESCAPEUTF8)) {
+        if (GOT_PREF(NS_NET_PREF_ESCAPEUTF8, val))
+            gEscapeUTF8 = val;
+        LOG(("escape UTF-8 %s\n", gEscapeUTF8 ? "enabled" : "disabled"));
+    }
+
+    if (PREF_CHANGED(NS_NET_PREF_ALWAYSENCODEINUTF8)) {
+        if (GOT_PREF(NS_NET_PREF_ALWAYSENCODEINUTF8, val))
+            gAlwaysEncodeInUTF8 = val;
+        LOG(("encode in UTF-8 %s\n", gAlwaysEncodeInUTF8 ? "enabled" : "disabled"));
+    }
+
+    if (PREF_CHANGED(NS_NET_PREF_ENCODEQUERYINUTF8)) {
+        if (GOT_PREF(NS_NET_PREF_ENCODEQUERYINUTF8, val))
+            gEncodeQueryInUTF8 = val;
+        LOG(("encode query in UTF-8 %s\n", gEncodeQueryInUTF8 ? "enabled" : "disabled"));
+    }
+
+    if (PREF_CHANGED(NS_NET_PREF_SHOWPUNYCODE)) {
+        if (GOT_PREF(NS_NET_PREF_SHOWPUNYCODE, val))
+            gShowPunycode = val;
+        LOG(("show punycode %s\n", gShowPunycode ? "enabled" : "disabled"));
+    }
+#undef PREF_CHANGED
+#undef GOT_PREF
+}
+
+/* static */ nsresult
+nsStandardURL::ACEtoDisplayIDN(const nsCSubstring &host, nsCString &result)
+{
+    if (gShowPunycode || !IsInWhitelist(host)) {
+        result = host;
+        return NS_OK;
+    }
+
+    return gIDN->ConvertACEtoUTF8(host, result);
+}
+
+/* static */ nsresult
+nsStandardURL::UTF8toDisplayIDN(const nsCSubstring &host, nsCString &result)
+{
+    // We have to normalize the hostname before testing against the domain
+    // whitelist.  See bug 315411.
+
+    nsCAutoString temp;
+    if (gShowPunycode || NS_FAILED(gIDN->Normalize(host, temp)))
+        return gIDN->ConvertUTF8toACE(host, result);
+
+    PRBool isACE = PR_FALSE;
+    gIDN->IsACE(temp, &isACE);
+
+    // If host is converted to ACE by the normalizer, then the host may contain
+    // unsafe characters.  See bug 283016, bug 301694, and bug 309311.
+ 
+    if (!isACE && !IsInWhitelist(temp))
+        return gIDN->ConvertUTF8toACE(temp, result);
+
+    result = temp;
+    return NS_OK;
+}
+
+/* static */ PRBool
+nsStandardURL::IsInWhitelist(const nsCSubstring &host)
+{
+    PRInt32 pos; 
+    PRBool safe;
+
+    // XXX This code uses strings inefficiently.
+
+    if (gIDNWhitelistPrefBranch && 
+        (pos = nsCAutoString(host).RFind(".")) != kNotFound &&
+        NS_SUCCEEDED(gIDNWhitelistPrefBranch->
+                     GetBoolPref(nsCAutoString(Substring(host, pos + 1)).get(),
+                                 &safe)))
+        return safe;
+
+    return PR_FALSE;
+}
+
 //----------------------------------------------------------------------------
 // nsStandardURL::nsISupports
 //----------------------------------------------------------------------------
@@ -851,6 +936,7 @@ NS_INTERFACE_MAP_BEGIN(nsStandardURL)
     NS_INTERFACE_MAP_ENTRY(nsIStandardURL)
     NS_INTERFACE_MAP_ENTRY(nsISerializable)
     NS_INTERFACE_MAP_ENTRY(nsIClassInfo)
+    NS_INTERFACE_MAP_ENTRY(nsIMutable)
     // see nsStandardURL::Equals
     if (aIID.Equals(kThisImplCID))
         foundInterface = NS_STATIC_CAST(nsIURI *, this);
@@ -993,9 +1079,9 @@ nsStandardURL::GetAsciiHost(nsACString &result)
         return NS_OK;
     }
 
-    if (gIDNService) {
+    if (gIDN) {
         nsresult rv;
-        rv = gIDNService->ConvertUTF8toACE(Host(), result);
+        rv = gIDN->ConvertUTF8toACE(Host(), result);
         if (NS_SUCCEEDED(rv)) {
             mHostA = ToNewCString(result);
             return NS_OK;
@@ -1121,6 +1207,8 @@ nsStandardURL::SetUserPass(const nsACString &input)
     LOG(("nsStandardURL::SetUserPass [userpass=%s]\n", userpass.get()));
 
     if (mURLType == URLTYPE_NO_AUTHORITY) {
+        if (userpass.IsEmpty())
+            return NS_OK;
         NS_ERROR("cannot set user:pass on no-auth url");
         return NS_ERROR_UNEXPECTED;
     }
@@ -1161,18 +1249,20 @@ nsStandardURL::SetUserPass(const nsACString &input)
     nsCAutoString buf;
     if (usernameLen > 0) {
         GET_SEGMENT_ENCODER(encoder);
+        PRBool ignoredOut;
         usernameLen = encoder.EncodeSegmentCount(userpass.get(),
                                                  URLSegment(usernamePos,
                                                             usernameLen),
                                                  esc_Username | esc_AlwaysCopy,
-                                                 buf);
+                                                 buf, ignoredOut);
         if (passwordLen >= 0) {
             buf.Append(':');
             passwordLen = encoder.EncodeSegmentCount(userpass.get(),
                                                      URLSegment(passwordPos,
                                                                 passwordLen),
                                                      esc_Password |
-                                                     esc_AlwaysCopy, buf);
+                                                     esc_AlwaysCopy, buf,
+                                                     ignoredOut);
         }
         if (mUsername.mLen < 0)
             buf.Append('@');
@@ -1218,6 +1308,8 @@ nsStandardURL::SetUsername(const nsACString &input)
     LOG(("nsStandardURL::SetUsername [username=%s]\n", username.get()));
 
     if (mURLType == URLTYPE_NO_AUTHORITY) {
+        if (username.IsEmpty())
+            return NS_OK;
         NS_ERROR("cannot set username on no-auth url");
         return NS_ERROR_UNEXPECTED;
     }
@@ -1261,6 +1353,8 @@ nsStandardURL::SetPassword(const nsACString &input)
     LOG(("nsStandardURL::SetPassword [password=%s]\n", password.get()));
 
     if (mURLType == URLTYPE_NO_AUTHORITY) {
+        if (password.IsEmpty())
+            return NS_OK;
         NS_ERROR("cannot set password on no-auth url");
         return NS_ERROR_UNEXPECTED;
     }
@@ -1327,6 +1421,8 @@ nsStandardURL::SetHost(const nsACString &input)
     LOG(("nsStandardURL::SetHost [host=%s]\n", host));
 
     if (mURLType == URLTYPE_NO_AUTHORITY) {
+        if (flat.IsEmpty())
+            return NS_OK;
         NS_ERROR("cannot set host on no-auth url");
         return NS_ERROR_UNEXPECTED;
     }
@@ -1375,6 +1471,10 @@ nsStandardURL::SetHost(const nsACString &input)
         mAuthority.mLen += shift;
         ShiftFromPath(shift);
     }
+
+    // Now canonicalize the host to lowercase
+    net_ToLowerCase(mSpec.BeginWriting() + mHost.mPos, mHost.mLen);
+
     return NS_OK;
 }
              
@@ -1388,6 +1488,11 @@ nsStandardURL::SetPort(PRInt32 port)
     if ((port == mPort) || (mPort == -1 && port == mDefaultPort))
         return NS_OK;
 
+    if (mURLType == URLTYPE_NO_AUTHORITY) {
+        NS_ERROR("cannot set port on no-auth url");
+        return NS_ERROR_UNEXPECTED;
+    }
+
     InvalidateCache();
 
     if (mPort == -1) {
@@ -1462,30 +1567,82 @@ nsStandardURL::Equals(nsIURI *unknownOther, PRBool *result)
     NS_ENSURE_ARG_POINTER(unknownOther);
     NS_PRECONDITION(result, "null pointer");
 
-    nsStandardURL *other;
-    nsresult rv = unknownOther->QueryInterface(kThisImplCID, (void **) &other);
+    nsRefPtr<nsStandardURL> otherPtr;
+    nsresult rv = unknownOther->QueryInterface(kThisImplCID,
+                                               getter_AddRefs(otherPtr));
+
+    // Hack around issue with MSVC++ not allowing the nsDerivedSafe to access
+    // the private members and not doing the implicit conversion to a raw
+    // pointer.
+    nsStandardURL* other = otherPtr;
     if (NS_FAILED(rv)) {
         *result = PR_FALSE;
         return NS_OK;
     }
 
-    // XXX this needs to allow for case insensitive paths in some cases (e.g.,
-    // file: URLs under windows).
+    // First, check whether one URIs is an nsIFileURL while the other
+    // is not.  If that's the case, they're different.
+    if (mSupportsFileURL != other->mSupportsFileURL) {
+        *result = PR_FALSE;
+        return NS_OK;
+    }
 
-    *result = 
-        SegmentIs(mScheme, other->mSpec.get(), other->mScheme) &&
-        SegmentIs(mDirectory, other->mSpec.get(), other->mDirectory) &&
+    // Next check parts of a URI that, if different, automatically make the
+    // URIs different
+    if (!SegmentIs(mScheme, other->mSpec.get(), other->mScheme) ||
+        // Check for host manually, since conversion to file will
+        // ignore the host!
+        !SegmentIs(mHost, other->mSpec.get(), other->mHost) ||
+        !SegmentIs(mQuery, other->mSpec.get(), other->mQuery) ||
+        !SegmentIs(mRef, other->mSpec.get(), other->mRef) ||
+        !SegmentIs(mUsername, other->mSpec.get(), other->mUsername) ||
+        !SegmentIs(mPassword, other->mSpec.get(), other->mPassword) ||
+        Port() != other->Port() ||
+        !SegmentIs(mParam, other->mSpec.get(), other->mParam)) {
+        // No need to compare files or other URI parts -- these are different
+        // beasties
+        *result = PR_FALSE;
+        return NS_OK;
+    }
+    
+    // Then check for exact identity of URIs.  If we have it, they're equal
+    if (SegmentIs(mDirectory, other->mSpec.get(), other->mDirectory) &&
         SegmentIs(mBasename, other->mSpec.get(), other->mBasename) &&
-        SegmentIs(mExtension, other->mSpec.get(), other->mExtension) &&
-        HostsAreEquivalent(other) &&
-        SegmentIs(mQuery, other->mSpec.get(), other->mQuery) &&
-        SegmentIs(mRef, other->mSpec.get(), other->mRef) &&
-        SegmentIs(mUsername, other->mSpec.get(), other->mUsername) &&
-        SegmentIs(mPassword, other->mSpec.get(), other->mPassword) &&
-        SegmentIs(mParam, other->mSpec.get(), other->mParam) &&
-        (Port() == other->Port());
+        SegmentIs(mExtension, other->mSpec.get(), other->mExtension)) {
+        *result = PR_TRUE;
+        return NS_OK;
+    }
+
+    // At this point, the URIs are not identical, but they only differ in the
+    // directory/filename/extension.  If these are file URLs, then get the
+    // corresponding file objects and compare those, since two filenames that
+    // differ, eg, only in case could still be equal.
+    if (mSupportsFileURL) {
+        // Assume not equal for failure cases... but failures in GetFile are
+        // really failures, more or less, so propagate them to caller.
+        *result = PR_FALSE;
+        
+        rv = EnsureFile();
+        if (NS_FAILED(rv)) {
+            LOG(("nsStandardURL::Equals [this=%p spec=%s] failed to ensure file",
+                this, mSpec.get()));
+            return rv;
+        }
+        NS_ASSERTION(mFile, "EnsureFile() lied!");
+        rv = other->EnsureFile();
+        if (NS_FAILED(rv)) {
+            LOG(("nsStandardURL::Equals [other=%p spec=%s] other failed to ensure file",
+                other, other->mSpec.get()));
+            return rv;
+        }
+        NS_ASSERTION(other->mFile, "EnsureFile() lied!");
+        return mFile->Equals(other->mFile, result);
+    }
+
+    // The URLs are not identical, and they do not correspond to the
+    // same file, so they are different.
+    *result = PR_FALSE;
 
-    NS_RELEASE(other);
     return NS_OK;
 }
 
@@ -1498,11 +1655,18 @@ nsStandardURL::SchemeIs(const char *scheme, PRBool *result)
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsStandardURL::Clone(nsIURI **result)
+/* virtual */ nsStandardURL*
+nsStandardURL::StartClone()
 {
     nsStandardURL *clone;
     NS_NEWXPCOM(clone, nsStandardURL);
+    return clone;
+}
+
+NS_IMETHODIMP
+nsStandardURL::Clone(nsIURI **result)
+{
+    nsStandardURL *clone = StartClone();
     if (!clone)
         return NS_ERROR_OUT_OF_MEMORY;
 
@@ -1720,7 +1884,7 @@ nsStandardURL::GetCommonBaseSpec(nsIURI *uri2, nsACString &aResult)
     nsresult rv = uri2->QueryInterface(kThisImplCID, (void **) &stdurl2);
     isEquals = NS_SUCCEEDED(rv)
             && SegmentIs(mScheme, stdurl2->mSpec.get(), stdurl2->mScheme)    
-            && HostsAreEquivalent(stdurl2)
+            && SegmentIs(mHost, stdurl2->mSpec.get(), stdurl2->mHost)
             && SegmentIs(mUsername, stdurl2->mSpec.get(), stdurl2->mUsername)
             && SegmentIs(mPassword, stdurl2->mSpec.get(), stdurl2->mPassword)
             && (Port() == stdurl2->Port());
@@ -1833,7 +1997,7 @@ nsStandardURL::GetRelativeSpec(nsIURI *uri2, nsACString &aResult)
     while (*thisIndex)
     {
         if (*thisIndex == '/')
-            aResult.Append("../");
+            aResult.AppendLiteral("../");
 
         thisIndex++;
     }
@@ -2049,9 +2213,11 @@ nsStandardURL::SetQuery(const nsACString &input)
 
     // encode query if necessary
     nsCAutoString buf;
-    GET_SEGMENT_ENCODER(encoder);
-    encoder.EncodeSegmentCount(query, URLSegment(0, queryLen), esc_Query, buf);
-    if (!buf.IsEmpty()) {
+    PRBool encoded;
+    GET_QUERY_ENCODER(encoder);
+    encoder.EncodeSegmentCount(query, URLSegment(0, queryLen), esc_Query,
+                               buf, encoded);
+    if (encoded) {
         query = buf.get();
         queryLen = buf.Length();
     }
@@ -2107,9 +2273,11 @@ nsStandardURL::SetRef(const nsACString &input)
 
     // encode ref if necessary
     nsCAutoString buf;
+    PRBool encoded;
     GET_SEGMENT_ENCODER(encoder);
-    encoder.EncodeSegmentCount(ref, URLSegment(0, refLen), esc_Ref, buf);
-    if (!buf.IsEmpty()) {
+    encoder.EncodeSegmentCount(ref, URLSegment(0, refLen), esc_Ref,
+                               buf, encoded);
+    if (encoded) {
         ref = buf.get();
         refLen = buf.Length();
     }
@@ -2177,17 +2345,20 @@ nsStandardURL::SetFileName(const nsACString &input)
         }
         else {
             nsCAutoString newFilename;
+            PRBool ignoredOut;
             GET_SEGMENT_ENCODER(encoder);
             basename.mLen = encoder.EncodeSegmentCount(filename, basename,
                                                        esc_FileBaseName |
                                                        esc_AlwaysCopy,
-                                                       newFilename);
+                                                       newFilename,
+                                                       ignoredOut);
             if (extension.mLen >= 0) {
                 newFilename.Append('.');
                 extension.mLen = encoder.EncodeSegmentCount(filename, extension,
                                                             esc_FileExtension |
                                                             esc_AlwaysCopy,
-                                                            newFilename);
+                                                            newFilename,
+                                                            ignoredOut);
             }
 
             if (mBasename.mLen < 0) {
@@ -2222,39 +2393,71 @@ nsStandardURL::SetFileName(const nsACString &input)
 NS_IMETHODIMP
 nsStandardURL::SetFileBaseName(const nsACString &input)
 {
-    NS_NOTYETIMPLEMENTED("");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    nsCAutoString extension;
+    nsresult rv = GetFileExtension(extension);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCAutoString newFileName(input);
+
+    if (!extension.IsEmpty()) {
+        newFileName.Append('.');
+        newFileName.Append(extension);
+    }
+
+    return SetFileName(newFileName);
 }
 
 NS_IMETHODIMP
 nsStandardURL::SetFileExtension(const nsACString &input)
 {
-    NS_NOTYETIMPLEMENTED("");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    nsCAutoString newFileName;
+    nsresult rv = GetFileBaseName(newFileName);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    if (!input.IsEmpty()) {
+        newFileName.Append('.');
+        newFileName.Append(input);
+    }
+
+    return SetFileName(newFileName);
 }
 
 //----------------------------------------------------------------------------
 // nsStandardURL::nsIFileURL
 //----------------------------------------------------------------------------
 
+nsresult
+nsStandardURL::EnsureFile()
+{
+    NS_PRECONDITION(mSupportsFileURL,
+                    "EnsureFile() called on a URL that doesn't support files!");
+    if (mFile) {
+        // Nothing to do
+        return NS_OK;
+    }
+
+    // Parse the spec if we don't have a cached result
+    if (mSpec.IsEmpty()) {
+        NS_ERROR("url not initialized");
+        return NS_ERROR_NOT_INITIALIZED;
+    }
+
+    if (!SegmentIs(mScheme, "file")) {
+        NS_ERROR("not a file URL");
+        return NS_ERROR_FAILURE;
+    }
+
+    return net_GetFileFromURLSpec(mSpec, getter_AddRefs(mFile));
+}
+
 NS_IMETHODIMP
 nsStandardURL::GetFile(nsIFile **result)
 {
-    // reparse the spec if we don't have a cached result
-    if (!mFile) {
-        if (mSpec.IsEmpty()) {
-            NS_ERROR("url not initialized");
-            return NS_ERROR_NOT_INITIALIZED;
-        }
-
-        if (!SegmentIs(mScheme, "file")) {
-            NS_ERROR("not a file URL");
-            return NS_ERROR_FAILURE;
-        }
-
-        nsresult rv = net_GetFileFromURLSpec(mSpec, getter_AddRefs(mFile));
-        if (NS_FAILED(rv)) return rv;
-    }
+    NS_PRECONDITION(mSupportsFileURL,
+                    "GetFile() called on a URL that doesn't support files!");
+    nsresult rv = EnsureFile();
+    if (NS_FAILED(rv))
+        return rv;
 
 #if defined(PR_LOGGING)
     if (LOG_ENABLED()) {
@@ -2287,7 +2490,9 @@ nsStandardURL::SetFile(nsIFile *file)
     rv = net_GetURLSpecFromFile(file, url);
     if (NS_FAILED(rv)) return rv;
 
-    rv = SetSpec(url);
+    SetSpec(url);
+
+    rv = Init(mURLType, mDefaultPort, url, nsnull, nsnull);
 
     // must clone |file| since its value is not guaranteed to remain constant
     if (NS_SUCCEEDED(rv)) {
@@ -2343,21 +2548,19 @@ nsStandardURL::Init(PRUint32 urlType,
 
     mOriginCharset.Truncate();
 
-    if (!gAlwaysEncodeInUTF8) {
-        if (charset == nsnull || *charset == '\0') {
-            // check if baseURI provides an origin charset and use that.
-            if (baseURI)
-                baseURI->GetOriginCharset(mOriginCharset);
+    if (charset == nsnull || *charset == '\0') {
+        // check if baseURI provides an origin charset and use that.
+        if (baseURI)
+            baseURI->GetOriginCharset(mOriginCharset);
 
-            // URI can't be encoded in UTF-16, UTF-16BE, UTF-16LE, UTF-32,
-            // UTF-32-LE, UTF-32LE, UTF-32BE (yet?). Truncate mOriginCharset if
-            // it starts with "utf" (since an empty mOriginCharset implies
-            // UTF-8, this is safe even if mOriginCharset is UTF-8).
+        // URI can't be encoded in UTF-16, UTF-16BE, UTF-16LE, UTF-32,
+        // UTF-32-LE, UTF-32LE, UTF-32BE (yet?). Truncate mOriginCharset if
+        // it starts with "utf" (since an empty mOriginCharset implies
+        // UTF-8, this is safe even if mOriginCharset is UTF-8).
 
-            if (mOriginCharset.Length() > 3 &&
-                IsUTFCharset(mOriginCharset.get())) {
-                mOriginCharset.Truncate();
-            }
+        if (mOriginCharset.Length() > 3 &&
+            IsUTFCharset(mOriginCharset.get())) {
+            mOriginCharset.Truncate();
         }
     }
     else if (!IsUTFCharset(charset)) {
@@ -2400,6 +2603,8 @@ nsStandardURL::GetMutable(PRBool *value)
 NS_IMETHODIMP
 nsStandardURL::SetMutable(PRBool value)
 {
+    NS_ENSURE_ARG(mMutable || !value);
+
     mMutable = value;
     return NS_OK;
 }
@@ -2483,6 +2688,12 @@ nsStandardURL::Read(nsIObjectInputStream *stream)
     rv = NS_ReadOptionalCString(stream, mOriginCharset);
     if (NS_FAILED(rv)) return rv;
 
+    PRBool isMutable;
+    rv = stream->ReadBoolean(&isMutable);
+    if (NS_FAILED(rv)) return rv;
+
+    mMutable = isMutable;
+
     return NS_OK;
 }
 
@@ -2545,6 +2756,9 @@ nsStandardURL::Write(nsIObjectOutputStream *stream)
     rv = NS_WriteOptionalStringZ(stream, mOriginCharset.get());
     if (NS_FAILED(rv)) return rv;
 
+    rv = stream->WriteBoolean(mMutable);
+    if (NS_FAILED(rv)) return rv;
+
     return NS_OK;
 }
 
@@ -2610,4 +2824,3 @@ nsStandardURL::GetClassIDNoAlloc(nsCID *aClassIDNoAlloc)
     *aClassIDNoAlloc = kStandardURLCID;
     return NS_OK;
 }
-
