@@ -48,7 +48,6 @@
 #include "nsCacheDevice.h"
 #include "nsMemoryCacheDevice.h"
 #include "nsICacheVisitor.h"
-#include "nsCRT.h"
 
 #ifdef NECKO_DISK_CACHE_SQL
 #include "nsDiskCacheDeviceSQL.h"
@@ -56,17 +55,18 @@
 #include "nsDiskCacheDevice.h"
 #endif
 
-#include "nsAutoLock.h"
-#include "nsIEventQueue.h"
 #include "nsIObserverService.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "nsILocalFile.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsAppDirectoryServiceDefs.h"
+#include "nsThreadUtils.h"
+#include "nsProxyRelease.h"
 #include "nsVoidArray.h"
-
+#include "nsDeleteDir.h"
+#include <math.h>  // for log()
 
 
 /******************************************************************************
@@ -85,9 +85,6 @@
 #define MEMORY_CACHE_ENABLE_PREF    "browser.cache.memory.enable"
 #define MEMORY_CACHE_CAPACITY_PREF  "browser.cache.memory.capacity"
 #define MEMORY_CACHE_MAX_ENTRY_SIZE_PREF "browser.cache.memory.max_entry_size"
-#define MEMORY_CACHE_CAPACITY       4096
-
-#define BROWSER_CACHE_MEMORY_CAPACITY  4096
 
 
 class nsCacheProfilePrefObserver : public nsIObserver
@@ -108,7 +105,7 @@ public:
     virtual ~nsCacheProfilePrefObserver() {}
     
     nsresult        Install();
-    nsresult        Remove();
+    void            Remove();
     nsresult        ReadPrefs(nsIPrefBranch* branch);
     
     PRBool          DiskCacheEnabled();
@@ -116,7 +113,7 @@ public:
     nsILocalFile *  DiskCacheParentDirectory()  { return mDiskCacheParentDirectory; }
     
     PRBool          MemoryCacheEnabled();
-    PRInt32         MemoryCacheCapacity()       { return mMemoryCacheCapacity; }
+    PRInt32         MemoryCacheCapacity();
 
 private:
     PRBool                  mHaveProfile;
@@ -155,7 +152,7 @@ nsCacheProfilePrefObserver::Install()
     
     
     // install preferences observer
-    nsCOMPtr<nsIPrefBranchInternal> branch = do_GetService(NS_PREFSERVICE_CONTRACTID);
+    nsCOMPtr<nsIPrefBranch2> branch = do_GetService(NS_PREFSERVICE_CONTRACTID);
     if (!branch) return NS_ERROR_FAILURE;
 
     char * prefList[] = { 
@@ -192,47 +189,31 @@ nsCacheProfilePrefObserver::Install()
 }
 
 
-nsresult
+void
 nsCacheProfilePrefObserver::Remove()
 {
-    nsresult rv, rv2 = NS_OK;
-
     // remove Observer Service observers
-    nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
-    if (NS_FAILED(rv)) return rv;
-    NS_ENSURE_ARG(observerService);
-
-    rv = observerService->RemoveObserver(this, "profile-before-change");
-    if (NS_FAILED(rv)) rv2 = rv;
-    
-    rv = observerService->RemoveObserver(this, "profile-after-change");
-    if (NS_FAILED(rv)) rv2 = rv;
-
-    rv = observerService->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
-    if (NS_FAILED(rv)) rv2 = rv;
-
+    nsCOMPtr<nsIObserverService> obs =
+            do_GetService("@mozilla.org/observer-service;1");
+    if (obs) {
+        obs->RemoveObserver(this, "profile-before-change");
+        obs->RemoveObserver(this, "profile-after-change");
+        obs->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
+    }
 
     // remove Pref Service observers
-    nsCOMPtr<nsIPrefBranchInternal> prefInternal = do_GetService(NS_PREFSERVICE_CONTRACTID);
+    nsCOMPtr<nsIPrefBranch2> prefs =
+           do_GetService(NS_PREFSERVICE_CONTRACTID);
+    if (prefs) {
+        // remove Disk cache pref observers
+        prefs->RemoveObserver(DISK_CACHE_ENABLE_PREF, this);
+        prefs->RemoveObserver(DISK_CACHE_CAPACITY_PREF, this);
+        prefs->RemoveObserver(DISK_CACHE_DIR_PREF, this);
 
-    // remove Disk cache pref observers
-    rv  = prefInternal->RemoveObserver(DISK_CACHE_ENABLE_PREF, this);
-    if (NS_FAILED(rv)) rv2 = rv;
-
-    rv  = prefInternal->RemoveObserver(DISK_CACHE_CAPACITY_PREF, this);
-    if (NS_FAILED(rv)) rv2 = rv;
-
-    rv  = prefInternal->RemoveObserver(DISK_CACHE_DIR_PREF, this);
-    if (NS_FAILED(rv)) rv2 = rv;
-    
-    // remove Memory cache pref observers
-    rv = prefInternal->RemoveObserver(MEMORY_CACHE_ENABLE_PREF, this);
-    if (NS_FAILED(rv)) rv2 = rv;
-
-    rv = prefInternal->RemoveObserver(MEMORY_CACHE_CAPACITY_PREF, this);
-    // if (NS_FAILED(rv)) rv2 = rv;
-
-    return NS_SUCCEEDED(rv) ? rv2 : rv;
+        // remove Memory cache pref observers
+        prefs->RemoveObserver(MEMORY_CACHE_ENABLE_PREF, this);
+        prefs->RemoveObserver(MEMORY_CACHE_CAPACITY_PREF, this);
+    }
 }
 
 
@@ -242,7 +223,7 @@ nsCacheProfilePrefObserver::Observe(nsISupports *     subject,
                                     const PRUnichar * data_unicode)
 {
     nsresult rv;
-    NS_ConvertUCS2toUTF8 data(data_unicode);
+    NS_ConvertUTF16toUTF8 data(data_unicode);
     CACHE_LOG_ALWAYS(("Observe [topic=%s data=%s]\n", topic, data.get()));
 
     if (!strcmp(NS_XPCOM_SHUTDOWN_OBSERVER_ID, topic)) {
@@ -303,13 +284,14 @@ nsCacheProfilePrefObserver::Observe(nsISupports *     subject,
             rv = branch->GetBoolPref(MEMORY_CACHE_ENABLE_PREF,
                                      &mMemoryCacheEnabled);
             if (NS_FAILED(rv))  return rv;
-            nsCacheService::SetMemoryCacheEnabled(MemoryCacheEnabled());
+            nsCacheService::SetMemoryCache();
             
         } else if (!strcmp(MEMORY_CACHE_CAPACITY_PREF, data.get())) {
 
+            mMemoryCacheCapacity = -1;
             (void) branch->GetIntPref(MEMORY_CACHE_CAPACITY_PREF,
                                       &mMemoryCacheCapacity);
-            nsCacheService::SetMemoryCacheCapacity(mMemoryCacheCapacity);
+            nsCacheService::SetMemoryCache();
         }
     }
     
@@ -343,16 +325,34 @@ nsCacheProfilePrefObserver::ReadPrefs(nsIPrefBranch* branch)
                                     getter_AddRefs(directory));
         if (NS_FAILED(rv)) {
             // try to get the profile directory (there may not be a profile yet)
-            rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
-                                        getter_AddRefs(directory));
-#if DEBUG
-            if (NS_FAILED(rv)) {
-                // use current process directory during development
-                rv = NS_GetSpecialDirectory(NS_XPCOM_CURRENT_PROCESS_DIR,
-                                            getter_AddRefs(directory));
+            nsCOMPtr<nsIFile> profDir;
+            NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
+                                   getter_AddRefs(profDir));
+            NS_GetSpecialDirectory(NS_APP_USER_PROFILE_LOCAL_50_DIR,
+                                   getter_AddRefs(directory));
+            if (!directory)
+                directory = profDir;
+            else if (profDir) {
+                PRBool same;
+                if (NS_SUCCEEDED(profDir->Equals(directory, &same)) && !same) {
+                    // We no longer store the cache directory in the main
+                    // profile directory, so we should cleanup the old one.
+                    rv = profDir->AppendNative(NS_LITERAL_CSTRING("Cache"));
+                    if (NS_SUCCEEDED(rv)) {
+                        PRBool exists;
+                        if (NS_SUCCEEDED(profDir->Exists(&exists)) && exists)
+                            DeleteDir(profDir, PR_FALSE, PR_FALSE);
+                    }
+                }
             }
-#endif
         }
+#if DEBUG
+        if (!directory) {
+            // use current process directory during development
+            rv = NS_GetSpecialDirectory(NS_XPCOM_CURRENT_PROCESS_DIR,
+                                        getter_AddRefs(directory));
+        }
+#endif
         if (directory)
             mDiskCacheParentDirectory = do_QueryInterface(directory, &rv);
     }
@@ -361,6 +361,7 @@ nsCacheProfilePrefObserver::ReadPrefs(nsIPrefBranch* branch)
     // read memory cache device prefs
     (void) branch->GetBoolPref(MEMORY_CACHE_ENABLE_PREF, &mMemoryCacheEnabled);
 
+    mMemoryCacheCapacity = -1;
     (void) branch->GetIntPref(MEMORY_CACHE_CAPACITY_PREF,
                               &mMemoryCacheCapacity);
         
@@ -384,6 +385,73 @@ nsCacheProfilePrefObserver::MemoryCacheEnabled()
 }
 
 
+/**
+ * MemoryCacheCapacity
+ *
+ * If the browser.cache.memory.capacity preference is positive, we use that
+ * value for the amount of memory available for the cache.
+ *
+ * If browser.cache.memory.capacity is zero, the memory cache is disabled.
+ * 
+ * If browser.cache.memory.capacity is negative or not present, we use a
+ * formula that grows less than linearly with the amount of system memory, 
+ * with an upper limit on the cache size. No matter how much physical RAM is
+ * present, the default cache size would not exceed 32 MB. This maximum would
+ * apply only to systems with more than 4 GB of RAM (e.g. terminal servers)
+ *
+ *   RAM   Cache
+ *   ---   -----
+ *   32 Mb   2 Mb
+ *   64 Mb   4 Mb
+ *  128 Mb   6 Mb
+ *  256 Mb  10 Mb
+ *  512 Mb  14 Mb
+ * 1024 Mb  18 Mb
+ * 2048 Mb  24 Mb
+ * 4096 Mb  30 Mb
+ *
+ * The equation for this is (for cache size C and memory size K (kbytes)):
+ *  x = log2(K) - 14
+ *  C = x^2/3 + x + 2/3 + 0.1 (0.1 for rounding)
+ *  if (C > 32) C = 32
+ */
+
+PRInt32
+nsCacheProfilePrefObserver::MemoryCacheCapacity()
+{
+    PRInt32 capacity = mMemoryCacheCapacity;
+    if (capacity >= 0)
+        return capacity;
+
+    PRUint64 bytes = PR_GetPhysicalMemorySize();
+
+    if (LL_CMP(bytes, ==, LL_ZERO))
+        return 0;
+
+    // Conversion from unsigned int64 to double doesn't work on all platforms.
+    // We need to truncate the value at LL_MAXINT to make sure we don't
+    // overflow.
+    if (LL_CMP(bytes, >, LL_MAXINT))
+        bytes = LL_MAXINT;
+
+    PRUint64 kbytes;
+    LL_SHR(kbytes, bytes, 10);
+
+    double kBytesD;
+    LL_L2D(kBytesD, (PRInt64) kbytes);
+
+    double x = log(kBytesD)/log(2.0) - 14;
+    if (x > 0) {
+        capacity = (PRInt32)(x * x / 3.0 + x + 2.0 / 3 + 0.1); // 0.1 for rounding
+        if (capacity > 32)
+            capacity = 32;
+        capacity   *= 1024;
+    } else {
+        capacity    = 0;
+    }
+
+    return capacity;
+}
 
 /******************************************************************************
  * nsCacheService
@@ -398,7 +466,7 @@ nsCacheService *   nsCacheService::gService = nsnull;
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsCacheService, nsICacheService)
 
 nsCacheService::nsCacheService()
-    : mCacheServiceLock(nsnull),
+    : mLock(nsnull),
       mInitialized(PR_FALSE),
       mEnableMemoryDevice(PR_TRUE),
       mEnableDiskDevice(PR_TRUE),
@@ -420,7 +488,11 @@ nsCacheService::nsCacheService()
     PR_INIT_CLIST(&mDoomedEntries);
   
     // allocate service lock
-    mCacheServiceLock = PR_NewLock();
+    mLock = PR_NewLock();
+
+#if defined(DEBUG)
+    mLockedThread = nsnull;
+#endif
 }
 
 nsCacheService::~nsCacheService()
@@ -428,7 +500,7 @@ nsCacheService::~nsCacheService()
     if (mInitialized) // Shutdown hasn't been called yet.
         (void) Shutdown();
 
-    PR_DestroyLock(mCacheServiceLock);
+    PR_DestroyLock(mLock);
     gService = nsnull;
 }
 
@@ -440,7 +512,7 @@ nsCacheService::Init()
     if (mInitialized)
         return NS_ERROR_ALREADY_INITIALIZED;
 
-    if (mCacheServiceLock == nsnull)
+    if (mLock == nsnull)
         return NS_ERROR_OUT_OF_MEMORY;
 
     CACHE_LOG_INIT();
@@ -449,13 +521,6 @@ nsCacheService::Init()
     nsresult rv = mActiveEntries.Init();
     if (NS_FAILED(rv)) return rv;
     
-    // get references to services we'll be using frequently
-    mEventQService = do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-    
-    mProxyObjectManager = do_GetService(NS_XPCOMPROXY_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-
     // create profile/preference observer
     mObserver = new nsCacheProfilePrefObserver();
     if (!mObserver)  return NS_ERROR_OUT_OF_MEMORY;
@@ -464,10 +529,6 @@ nsCacheService::Init()
     mObserver->Install();
     mEnableDiskDevice =   mObserver->DiskCacheEnabled();
     mEnableMemoryDevice = mObserver->MemoryCacheEnabled();
-
-    rv = CreateMemoryDevice();
-    if (NS_FAILED(rv) && (rv != NS_ERROR_NOT_AVAILABLE))
-        return rv;
     
     mInitialized = PR_TRUE;
     return NS_OK;
@@ -477,7 +538,7 @@ nsCacheService::Init()
 void
 nsCacheService::Shutdown()
 {
-    nsAutoLock  lock(mCacheServiceLock);
+    nsCacheServiceAutoLock lock;
     NS_ASSERTION(mInitialized, 
                  "can't shutdown nsCacheService unless it has been initialized.");
 
@@ -563,7 +624,27 @@ nsCacheService::EvictEntriesForClient(const char *          clientID,
                                       nsCacheStoragePolicy  storagePolicy)
 {
     if (this == nsnull) return NS_ERROR_NOT_AVAILABLE; // XXX eh?
-    nsAutoLock lock(mCacheServiceLock);
+
+    nsCOMPtr<nsIObserverService> obsSvc =
+        do_GetService("@mozilla.org/observer-service;1");
+    if (obsSvc) {
+        // Proxy to the UI thread since the observer service isn't thredsafe.
+        // We use an async proxy, since this it's not important whether this
+        // notification happens before or after the actual eviction.
+
+        nsCOMPtr<nsIObserverService> obsProxy;
+        NS_GetProxyForObject(NS_PROXY_TO_MAIN_THREAD,
+                             NS_GET_IID(nsIObserverService), obsSvc,
+                             NS_PROXY_ASYNC, getter_AddRefs(obsProxy));
+
+        if (obsProxy) {
+            obsProxy->NotifyObservers(this,
+                                      NS_CACHESERVICE_EMPTYCACHE_TOPIC_ID,
+                                      nsnull);
+        }
+    }
+
+    nsCacheServiceAutoLock lock;
     nsresult rv = NS_OK;
 
 #ifdef NECKO_DISK_CACHE
@@ -584,7 +665,8 @@ nsCacheService::EvictEntriesForClient(const char *          clientID,
     if (storagePolicy == nsICache::STORE_ANYWHERE ||
         storagePolicy == nsICache::STORE_IN_MEMORY) {
 
-        if (mEnableMemoryDevice) {
+        // If there is no memory device, there is no need to evict it...
+        if (mMemoryDevice) {
             rv = mMemoryDevice->EvictEntries(clientID);
             if (NS_FAILED(rv)) return rv;
         }
@@ -599,7 +681,7 @@ nsCacheService::IsStorageEnabledForPolicy(nsCacheStoragePolicy  storagePolicy,
                                           PRBool *              result)
 {
     if (gService == nsnull) return NS_ERROR_NOT_AVAILABLE;
-    nsAutoLock lock(gService->mCacheServiceLock);
+    nsCacheServiceAutoLock lock;
 
     *result = gService->IsStorageEnabledForPolicy_Locked(storagePolicy);
     return NS_OK;
@@ -627,7 +709,9 @@ nsCacheService::IsStorageEnabledForPolicy_Locked(nsCacheStoragePolicy  storagePo
 
 NS_IMETHODIMP nsCacheService::VisitEntries(nsICacheVisitor *visitor)
 {
-    nsAutoLock lock(mCacheServiceLock);
+    NS_ENSURE_ARG_POINTER(visitor);
+
+    nsCacheServiceAutoLock lock;
 
     if (!(mEnableDiskDevice || mEnableMemoryDevice))
         return NS_ERROR_NOT_AVAILABLE;
@@ -636,7 +720,8 @@ NS_IMETHODIMP nsCacheService::VisitEntries(nsICacheVisitor *visitor)
     // XXX i.e. keep list of visitors in progress.
     
     nsresult rv = NS_OK;
-    if (mEnableMemoryDevice) {
+    // If there is no memory device, there are then also no entries to visit...
+    if (mMemoryDevice) {
         rv = mMemoryDevice->Visit(visitor);
         if (NS_FAILED(rv)) return rv;
     }
@@ -712,7 +797,7 @@ nsCacheService::CreateMemoryDevice()
     if (!mMemoryDevice)       return NS_ERROR_OUT_OF_MEMORY;
     
     // set preference
-    mMemoryDevice->SetCapacity(CacheMemoryAvailable());
+    mMemoryDevice->SetCapacity(mObserver->MemoryCacheCapacity());
 
     nsresult rv = mMemoryDevice->Init();
     if (NS_FAILED(rv)) {
@@ -726,18 +811,18 @@ nsCacheService::CreateMemoryDevice()
 
 nsresult
 nsCacheService::CreateRequest(nsCacheSession *   session,
-                              const char *       clientKey,
+                              const nsACString & clientKey,
                               nsCacheAccessMode  accessRequested,
                               PRBool             blockingMode,
                               nsICacheListener * listener,
                               nsCacheRequest **  request)
 {
-    NS_ASSERTION(request, "CreateRequest: request or entry is null");
+    NS_ASSERTION(request, "CreateRequest: request is null");
      
     nsCString * key = new nsCString(*session->ClientID());
     if (!key)
         return NS_ERROR_OUT_OF_MEMORY;
-    key->Append(":");
+    key->Append(':');
     key->Append(clientKey);
 
     if (mMaxKeyLength < key->Length()) mMaxKeyLength = key->Length();
@@ -751,34 +836,70 @@ nsCacheService::CreateRequest(nsCacheSession *   session,
 
     if (!listener)  return NS_OK;  // we're sync, we're done.
 
-    // get the nsIEventQueue for the request's thread
-    (*request)->mThread = PR_GetCurrentThread();
+    // get the request's thread
+    (*request)->mThread = do_GetCurrentThread();
     
     return NS_OK;
 }
 
 
+class nsCacheListenerEvent : public nsRunnable
+{
+public:
+    nsCacheListenerEvent(nsICacheListener *listener,
+                         nsICacheEntryDescriptor *descriptor,
+                         nsCacheAccessMode accessGranted,
+                         nsresult status)
+        : mListener(listener)      // transfers reference
+        , mDescriptor(descriptor)  // transfers reference (may be null)
+        , mAccessGranted(accessGranted)
+        , mStatus(status)
+    {}
+
+    NS_IMETHOD Run()
+    {
+        mListener->OnCacheEntryAvailable(mDescriptor, mAccessGranted, mStatus);
+
+        NS_RELEASE(mListener);
+        NS_IF_RELEASE(mDescriptor);
+        return NS_OK;
+    }
+
+private:
+    // We explicitly leak mListener or mDescriptor if Run is not called
+    // because otherwise we cannot guarantee that they are destroyed on
+    // the right thread.
+
+    nsICacheListener        *mListener;
+    nsICacheEntryDescriptor *mDescriptor;
+    nsCacheAccessMode        mAccessGranted;
+    nsresult                 mStatus;
+};
+
+
 nsresult
 nsCacheService::NotifyListener(nsCacheRequest *          request,
                                nsICacheEntryDescriptor * descriptor,
                                nsCacheAccessMode         accessGranted,
-                               nsresult                  error)
+                               nsresult                  status)
 {
-    nsresult rv;
-
-    nsCOMPtr<nsICacheListener> listenerProxy;
     NS_ASSERTION(request->mThread, "no thread set in async request!");
-    nsCOMPtr<nsIEventQueue> eventQ;
-    mEventQService->GetThreadEventQueue(request->mThread,
-                                        getter_AddRefs(eventQ));
-    rv = mProxyObjectManager->GetProxyForObject(eventQ,
-                                                NS_GET_IID(nsICacheListener),
-                                                request->mListener,
-                                                PROXY_ASYNC|PROXY_ALWAYS,
-                                                getter_AddRefs(listenerProxy));
-    if (NS_FAILED(rv)) return rv;
 
-    return listenerProxy->OnCacheEntryAvailable(descriptor, accessGranted, error);
+    // Swap ownership, and release listener on target thread...
+    nsICacheListener *listener = request->mListener;
+    request->mListener = nsnull;
+
+    nsCOMPtr<nsIRunnable> ev =
+            new nsCacheListenerEvent(listener, descriptor,
+                                     accessGranted, status);
+    if (!ev) {
+        // Better to leak listener and descriptor if we fail because we don't
+        // want to destroy them inside the cache service lock or on potentially
+        // the wrong thread.
+        return NS_ERROR_OUT_OF_MEMORY;
+    }
+
+    return request->mThread->Dispatch(ev, NS_DISPATCH_NORMAL);
 }
 
 
@@ -787,7 +908,7 @@ nsCacheService::ProcessRequest(nsCacheRequest *           request,
                                PRBool                     calledFromOpenCacheEntry,
                                nsICacheEntryDescriptor ** result)
 {
-    // !!! must be called with mCacheServiceLock held !!!
+    // !!! must be called with mLock held !!!
     nsresult           rv;
     nsCacheEntry *     entry = nsnull;
     nsCacheAccessMode  accessGranted = nsICache::ACCESS_NONE;
@@ -807,9 +928,10 @@ nsCacheService::ProcessRequest(nsCacheRequest *           request,
                 return rv;
             
             if (request->IsBlocking()) {
-                PR_Unlock(mCacheServiceLock);
+                // XXX this is probably wrong...
+                Unlock();
                 rv = request->WaitForValidation();
-                PR_Lock(mCacheServiceLock);
+                Lock();
             }
 
             PR_REMOVE_AND_INIT_LINK(request);
@@ -825,10 +947,10 @@ nsCacheService::ProcessRequest(nsCacheRequest *           request,
         // loop back around to look for another entry
     }
 
-    nsCOMPtr<nsICacheEntryDescriptor> descriptor;
+    nsICacheEntryDescriptor *descriptor = nsnull;
     
     if (NS_SUCCEEDED(rv))
-        rv = entry->CreateDescriptor(request, accessGranted, getter_AddRefs(descriptor));
+        rv = entry->CreateDescriptor(request, accessGranted, &descriptor);
 
     if (request->mListener) {  // Asynchronous
     
@@ -841,7 +963,7 @@ nsCacheService::ProcessRequest(nsCacheRequest *           request,
             rv = rv2;  // trigger delete request
         }
     } else {        // Synchronous
-        NS_IF_ADDREF(*result = descriptor);
+        *result = descriptor;
     }
     return rv;
 }
@@ -849,7 +971,7 @@ nsCacheService::ProcessRequest(nsCacheRequest *           request,
 
 nsresult
 nsCacheService::OpenCacheEntry(nsCacheSession *           session,
-                               const char *               key,
+                               const nsACString &         key,
                                nsCacheAccessMode          accessRequested,
                                PRBool                     blockingMode,
                                nsICacheListener *         listener,
@@ -861,7 +983,7 @@ nsCacheService::OpenCacheEntry(nsCacheSession *           session,
 
     nsCacheRequest * request = nsnull;
 
-    nsAutoLock lock(gService->mCacheServiceLock);
+    nsCacheServiceAutoLock lock;
     nsresult rv = gService->CreateRequest(session,
                                           key,
                                           accessRequested,
@@ -967,7 +1089,8 @@ nsCacheService::SearchCacheDevices(nsCString * key, nsCacheStoragePolicy policy)
     nsCacheEntry * entry = nsnull;
 
     if ((policy == nsICache::STORE_ANYWHERE) || (policy == nsICache::STORE_IN_MEMORY)) {
-        if (mEnableMemoryDevice)
+        // If there is no memory device, then there is nothing to search...
+        if (mMemoryDevice)
             entry = mMemoryDevice->FindEntry(key);
     }
 
@@ -996,18 +1119,17 @@ nsCacheService::EnsureEntryHasDevice(nsCacheEntry * entry)
 {
     nsCacheDevice * device = entry->CacheDevice();
     if (device)  return device;
-    nsresult rv = NS_OK;
 
 #ifdef NECKO_DISK_CACHE
     if (entry->IsStreamData() && entry->IsAllowedOnDisk() && mEnableDiskDevice) {
         // this is the default
         if (!mDiskDevice) {
-            rv = CreateDiskDevice();  // ignore the error (check for mDiskDevice instead)
+            (void)CreateDiskDevice();  // ignore the error (check for mDiskDevice instead)
         }
 
         if (mDiskDevice) {
             entry->MarkBinding();  // enter state of binding
-            rv = mDiskDevice->BindEntry(entry);
+            nsresult rv = mDiskDevice->BindEntry(entry);
             entry->ClearBinding(); // exit state of binding
             if (NS_SUCCEEDED(rv))
                 device = mDiskDevice;
@@ -1017,16 +1139,20 @@ nsCacheService::EnsureEntryHasDevice(nsCacheEntry * entry)
      
     // if we can't use mDiskDevice, try mMemoryDevice
     if (!device && mEnableMemoryDevice && entry->IsAllowedInMemory()) {        
-        entry->MarkBinding();  // enter state of binding
-        rv = mMemoryDevice->BindEntry(entry);
-        entry->ClearBinding(); // exit state of binding
-        if (NS_SUCCEEDED(rv))
-            device = mMemoryDevice;
+        if (!mMemoryDevice) {
+            (void)CreateMemoryDevice();  // ignore the error (check for mMemoryDevice instead)
+        }
+        if (mMemoryDevice) {
+            entry->MarkBinding();  // enter state of binding
+            nsresult rv = mMemoryDevice->BindEntry(entry);
+            entry->ClearBinding(); // exit state of binding
+            if (NS_SUCCEEDED(rv))
+                device = mMemoryDevice;
+        }
     }
 
-    if (device == nsnull)  return nsnull;
-
-    entry->SetCacheDevice(device);
+    if (device) 
+        entry->SetCacheDevice(device);
     return device;
 }
 
@@ -1071,50 +1197,11 @@ nsCacheService::DoomEntry_Internal(nsCacheEntry * entry)
 }
 
 
-static void* PR_CALLBACK
-EventHandler(PLEvent *self)
-{
-    nsISupports * object = (nsISupports *)PL_GetEventOwner(self);
-    NS_RELEASE(object);
-    return 0;
-}
-
-
-static void PR_CALLBACK
-DestroyHandler(PLEvent *self)
-{
-    delete self;
-}
-
-
-void
-nsCacheService::ProxyObjectRelease(nsISupports * object, PRThread * thread)
-{
-    NS_ASSERTION(gService, "nsCacheService not initialized");
-    NS_ASSERTION(thread, "no thread");
-    // XXX if thread == current thread, we could avoid posting an event,
-    // XXX by add this object to a queue and release it when the cache service is unlocked.
-    
-    nsCOMPtr<nsIEventQueue> eventQ;
-    gService->mEventQService->GetThreadEventQueue(thread, getter_AddRefs(eventQ));
-    NS_ASSERTION(eventQ, "no event queue for thread");
-    if (!eventQ)  return;
-    
-    PLEvent * event = new PLEvent;
-    if (!event) {
-        NS_WARNING("failed to allocate a PLEvent.");
-        return;
-    }
-    PL_InitEvent(event, object, EventHandler, DestroyHandler);
-    eventQ->PostEvent(event);
-}
-
-
 void
 nsCacheService::OnProfileShutdown(PRBool cleanse)
 {
     if (!gService)  return;
-    nsAutoLock lock(gService->mCacheServiceLock);
+    nsCacheServiceAutoLock lock;
 
     gService->DoomActiveEntries();
     gService->ClearDoomList();
@@ -1132,10 +1219,6 @@ nsCacheService::OnProfileShutdown(PRBool cleanse)
     if (gService->mMemoryDevice) {
         // clear memory cache
         gService->mMemoryDevice->EvictEntries(nsnull);
-#if 0
-        gService->mMemoryDevice->Shutdown();
-        gService->mEnableMemoryDevice = PR_FALSE;
-#endif
     }
 
 }
@@ -1146,23 +1229,18 @@ nsCacheService::OnProfileChanged()
 {
     if (!gService)  return;
  
-    nsresult   rv = NS_OK;
-    nsAutoLock lock(gService->mCacheServiceLock);
+    nsCacheServiceAutoLock lock;
     
     gService->mEnableDiskDevice   = gService->mObserver->DiskCacheEnabled();
     gService->mEnableMemoryDevice = gService->mObserver->MemoryCacheEnabled();
     
-    if (gService->mEnableMemoryDevice && !gService->mMemoryDevice) {
-        (void) gService->CreateMemoryDevice();
-    }
-
 #ifdef NECKO_DISK_CACHE
     if (gService->mDiskDevice) {
         gService->mDiskDevice->SetCacheParentDirectory(gService->mObserver->DiskCacheParentDirectory());
         gService->mDiskDevice->SetCapacity(gService->mObserver->DiskCacheCapacity());
 
         // XXX initialization of mDiskDevice could be made lazily, if mEnableDiskDevice is false
-        rv = gService->mDiskDevice->Init();
+        nsresult rv = gService->mDiskDevice->Init();
         if (NS_FAILED(rv)) {
             NS_ERROR("nsCacheService::OnProfileChanged: Re-initializing disk device failed");
             gService->mEnableDiskDevice = PR_FALSE;
@@ -1171,13 +1249,15 @@ nsCacheService::OnProfileChanged()
     }
 #endif // !NECKO_DISK_CACHE
     
+    // If memoryDevice exists, reset its size to the new profile
     if (gService->mMemoryDevice) {
-        gService->mMemoryDevice->SetCapacity(gService->CacheMemoryAvailable());
-        rv = gService->mMemoryDevice->Init();
-        if (NS_FAILED(rv) && (rv != NS_ERROR_ALREADY_INITIALIZED)) {
-            NS_ERROR("nsCacheService::OnProfileChanged: Re-initializing memory device failed");
-            gService->mEnableMemoryDevice = PR_FALSE;
-            // XXX delete mMemoryDevice?
+        if (gService->mEnableMemoryDevice) {
+            // make sure that capacity is reset to the right value
+            gService->mMemoryDevice->SetCapacity(gService->mObserver->MemoryCacheCapacity());
+        } else {
+            // tell memory device to evict everything
+            gService->mMemoryDevice->SetCapacity(0);
+            // Don't delete memory device, because some entries may be active still...
         }
     }
 }
@@ -1187,7 +1267,7 @@ void
 nsCacheService::SetDiskCacheEnabled(PRBool  enabled)
 {
     if (!gService)  return;
-    nsAutoLock lock(gService->mCacheServiceLock);
+    nsCacheServiceAutoLock lock;
     gService->mEnableDiskDevice = enabled;
 }
 
@@ -1196,7 +1276,7 @@ void
 nsCacheService::SetDiskCacheCapacity(PRInt32  capacity)
 {
     if (!gService)  return;
-    nsAutoLock lock(gService->mCacheServiceLock);
+    nsCacheServiceAutoLock lock;
 
 #ifdef NECKO_DISK_CACHE
     if (gService->mDiskDevice) {
@@ -1209,164 +1289,25 @@ nsCacheService::SetDiskCacheCapacity(PRInt32  capacity)
 
 
 void
-nsCacheService::SetMemoryCacheEnabled(PRBool  enabled)
+nsCacheService::SetMemoryCache()
 {
     if (!gService)  return;
-    nsAutoLock lock(gService->mCacheServiceLock);
-    gService->mEnableMemoryDevice = enabled;
-    (void) gService->CreateMemoryDevice();    // allocate memory device, if necessary
-    
-    if (!enabled && gService->mMemoryDevice) {
-        // tell memory device to evict everything
-        gService->mMemoryDevice->SetCapacity(0);
-    }
-}
+    nsCacheServiceAutoLock lock;
 
-
-void
-nsCacheService::SetMemoryCacheCapacity(PRInt32  capacity)
-{
-    if (!gService)  return;
-    nsAutoLock lock(gService->mCacheServiceLock);
-    
     gService->mEnableMemoryDevice = gService->mObserver->MemoryCacheEnabled();
-    if (gService->mEnableMemoryDevice && !gService->mMemoryDevice) {
-        (void) gService->CreateMemoryDevice();
-    }
 
-    if (gService->mMemoryDevice) {
-        gService->mMemoryDevice->SetCapacity(gService->CacheMemoryAvailable());
-    }
-}
-
-/**
- * CacheMemoryAvailable
- *
- * If the browser.cache.memory.capacity preference is positive, we use that
- * value for the amount of memory available for the cache.
- *
- * If browser.cache.memory.capacity is zero, the memory cache is disabled.
- * 
- * If browser.cache.memory.capacity is negative or not present, we use a
- * formula that grows less than linearly with the amount of system memory.
- *
- *   RAM   Cache
- *   ---   -----
- *   32 Mb   2 Mb
- *   64 Mb   4 Mb
- *  128 Mb   8 Mb
- *  256 Mb  14 Mb
- *  512 Mb  22 Mb
- * 1024 Mb  32 Mb
- * 2048 Mb  44 Mb
- * 4096 Mb  58 Mb
- *
- */
-
-#include <math.h>
-#if defined(__linux) || defined(__sun)
-#include <unistd.h>
-#elif defined(__hpux)
-#include <sys/pstat.h>
-#elif defined(XP_MACOSX)
-extern "C" {
-#include <mach/mach_init.h>
-#include <mach/mach_host.h>
-}
-#elif defined(XP_OS2)
-#define INCL_DOSMISC
-#include <os2.h>
-#elif defined(XP_WIN)
-#include <windows.h>
-#elif defined(_AIX)
-#include <cf.h>
-#include <sys/cfgodm.h>
-#endif
-
-
-PRInt32
-nsCacheService::CacheMemoryAvailable()
-{
-    PRInt32 capacity = mObserver->MemoryCacheCapacity();
-    if (capacity >= 0)
-        return capacity;
-
-    long kbytes  = 0;
-
-#if defined(__linux) || defined(__sun)
-
-    long pageSize  = sysconf(_SC_PAGESIZE);
-    long pageCount = sysconf(_SC_PHYS_PAGES);
-    kbytes         = (pageSize / 1024) * pageCount;
-
-#elif defined(__hpux)
-    
-    struct pst_static  info;
-    int result = pstat_getstatic(&info, sizeof(info), 1, 0);
-    if (result == 1) {
-        kbytes = info.physical_memory * (info.page_size / 1024);
-    }
-    
-#elif defined(XP_MACOSX)
-
-    struct host_basic_info hInfo;
-    mach_msg_type_number_t count;
-
-    int result = host_info(mach_host_self(),
-                           HOST_BASIC_INFO,
-                           (host_info_t) &hInfo,
-                           &count);
-    if (result == KERN_SUCCESS) {
-        kbytes = hInfo.memory_size / 1024;
-    }
-
-#elif defined(XP_WIN)
-    
-    // XXX we should use GlobalMemoryStatusEx on XP and 2000, but
-    // XXX our current build environment doesn't support it.
-    MEMORYSTATUS  memStat;
-    memset(&memStat, 0, sizeof(memStat));
-    GlobalMemoryStatus(&memStat);
-    kbytes = memStat.dwTotalPhys / 1024;
-
-#elif defined(XP_OS2)
-
-    ULONG ulPhysMem;
-    DosQuerySysInfo(QSV_TOTPHYSMEM,
-                    QSV_TOTPHYSMEM,
-                    &ulPhysMem,
-                    sizeof(ulPhysMem));
-    kbytes = (long)(ulPhysMem / 1024);
-      
-#elif defined(_AIX)
-
-    int how_many;
-    struct CuAt *obj;
-    if (odm_initialize() == 0) {
-        obj = getattr("sys0", "realmem", 0, &how_many);
-        if (obj != NULL) {
-            kbytes = atoi(obj->value);
-            free(obj);
+    if (gService->mEnableMemoryDevice) {
+        if (gService->mMemoryDevice) {
+            // make sure that capacity is reset to the right value
+            gService->mMemoryDevice->SetCapacity(gService->mObserver->MemoryCacheCapacity());
         }
-        odm_terminate();
-    }
-
-#else
-    return MEMORY_CACHE_CAPACITY;
-#endif
-
-    if (kbytes == 0)  return 0;
-    if (kbytes < 0)   kbytes = LONG_MAX; // cap overflows
-
-    double x = log((double)kbytes)/log((double)2) - 14;
-    if (x > 0) {
-        capacity    = (PRInt32)(x * x - x + 2.001); // add .001 for rounding
-        capacity   *= 1024;
     } else {
-        capacity    = 0;
+        if (gService->mMemoryDevice) {
+            // tell memory device to evict everything
+            gService->mMemoryDevice->SetCapacity(0);
+            // Don't delete memory device, because some entries may be active still...
+        }
     }
-
-    return capacity;
 }
 
 
@@ -1440,19 +1381,51 @@ nsCacheService::OnDataSizeChange(nsCacheEntry * entry, PRInt32 deltaSize)
     return device->OnDataSizeChange(entry, deltaSize);
 }
 
-
-PRLock *
-nsCacheService::ServiceLock()
+void
+nsCacheService::Lock()
 {
-    NS_ASSERTION(gService, "nsCacheService::gService is null.");
-    return gService->mCacheServiceLock;
+    PR_Lock(gService->mLock);
+
+#if defined(DEBUG)
+    gService->mLockedThread = PR_GetCurrentThread();
+#endif
+}
+
+void
+nsCacheService::Unlock()
+{
+    NS_ASSERTION(gService->mLockedThread == PR_GetCurrentThread(), "oops");
+
+    nsTArray<nsISupports*> doomed;
+    doomed.SwapElements(gService->mDoomedObjects);
+
+#if defined(DEBUG)
+    gService->mLockedThread = nsnull;
+#endif
+    PR_Unlock(gService->mLock);
+
+    for (PRUint32 i = 0; i < doomed.Length(); ++i)
+        doomed[i]->Release();
+}
+
+void
+nsCacheService::ReleaseObject_Locked(nsISupports * obj,
+                                     nsIEventTarget * target)
+{
+    NS_ASSERTION(gService->mLockedThread == PR_GetCurrentThread(), "oops");
+
+    PRBool isCur;
+    if (!target || NS_SUCCEEDED(target->IsOnCurrentThread(&isCur)) && isCur) {
+        gService->mDoomedObjects.AppendElement(obj);
+    } else {
+        NS_ProxyRelease(target, obj);
+    }
 }
 
 
 nsresult
 nsCacheService::SetCacheElement(nsCacheEntry * entry, nsISupports * element)
 {
-    entry->SetThread(PR_GetCurrentThread());
     entry->SetData(element);
     entry->TouchData();
     return NS_OK;
@@ -1591,10 +1564,10 @@ nsCacheService::ProcessPendingRequests(nsCacheEntry * entry)
                 // XXX if (newWriter)  NS_ASSERTION( accessGranted == request->AccessRequested(), "why not?");
 
                 // entry->CreateDescriptor dequeues request, and queues descriptor
-                nsCOMPtr<nsICacheEntryDescriptor> descriptor;
+                nsICacheEntryDescriptor *descriptor = nsnull;
                 rv = entry->CreateDescriptor(request,
                                              accessGranted,
-                                             getter_AddRefs(descriptor));
+                                             &descriptor);
                 
                 // post call to listener to report error or descriptor
                 rv = NotifyListener(request, descriptor, accessGranted, rv);
