@@ -64,8 +64,6 @@
 #include "private/pprio.h"
 #endif
 
-
-
 #include "nsDiskCacheDevice.h"
 #include "nsDiskCacheEntry.h"
 #include "nsDiskCacheMap.h"
@@ -76,6 +74,8 @@
 #include "nsCacheService.h"
 #include "nsCache.h"
 
+#include "nsDeleteDir.h"
+
 #include "nsICacheVisitor.h"
 #include "nsReadableUtils.h"
 #include "nsIInputStream.h"
@@ -102,77 +102,65 @@ static const char DISK_CACHE_DEVICE_ID[] = { "disk" };
 class nsDiskCacheEvictor : public nsDiskCacheRecordVisitor
 {
 public:
-    nsDiskCacheEvictor( nsDiskCacheDevice *   device,
-                        nsDiskCacheMap *      cacheMap,
+    nsDiskCacheEvictor( nsDiskCacheMap *      cacheMap,
                         nsDiskCacheBindery *  cacheBindery,
-                        PRInt32               targetSize,
+                        PRUint32              targetSize,
                         const char *          clientID)
-        : mDevice(device)
-        , mCacheMap(cacheMap)
+        : mCacheMap(cacheMap)
         , mBindery(cacheBindery)
         , mTargetSize(targetSize)
         , mClientID(clientID)
-    {}
+    { 
+        mClientIDSize = clientID ? strlen(clientID) : 0;
+    }
     
     virtual PRInt32  VisitRecord(nsDiskCacheRecord *  mapRecord);
  
 private:
-        nsDiskCacheDevice *  mDevice;
         nsDiskCacheMap *     mCacheMap;
         nsDiskCacheBindery * mBindery;
-        PRInt32              mTargetSize;
+        PRUint32             mTargetSize;
         const char *         mClientID;
+        PRUint32             mClientIDSize;
 };
 
 
 PRInt32
 nsDiskCacheEvictor::VisitRecord(nsDiskCacheRecord *  mapRecord)
 {
-    // read disk cache entry
-    nsDiskCacheEntry *   diskEntry = nsnull;
-    nsDiskCacheBinding * binding   = nsnull;
-    char *               clientID  = nsnull;
-    PRInt32              result    = kVisitNextRecord;
-
+    if (mCacheMap->TotalSize() < mTargetSize)
+        return kStopVisitingRecords;
+    
     if (mClientID) {
-         // we're just evicting records for a specific client
+        // we're just evicting records for a specific client
+        nsDiskCacheEntry *   diskEntry = nsnull;
         nsresult  rv = mCacheMap->ReadDiskCacheEntry(mapRecord, &diskEntry);
-        if (NS_FAILED(rv))  goto exit;  // XXX or delete record?
+        if (NS_FAILED(rv))  
+            return kVisitNextRecord;  // XXX or delete record?
     
-        // XXX FIXME compare clientID's without malloc
-
-        // get client ID from key
-        rv = ClientIDFromCacheKey(nsDependentCString(diskEntry->mKeyStart), &clientID);
-        if (NS_FAILED(rv))  goto exit;
-         
-        if (nsCRT::strcmp(mClientID, clientID) != 0) goto exit;
+        // Compare clientID's without malloc
+        if ((diskEntry->mKeySize <= mClientIDSize) ||
+            (diskEntry->mKeyStart[mClientIDSize] != ':') ||
+            (memcmp(diskEntry->mKeyStart, mClientID, mClientIDSize) != 0)) {
+            delete [] (char *)diskEntry;
+            return kVisitNextRecord;  // clientID doesn't match, skip it
+        }
+        delete [] (char *)diskEntry;
     }
     
-    if (mCacheMap->TotalSize() < mTargetSize) {
-        result = kStopVisitingRecords;
-        goto exit;
-    }
-    
-    binding = mBindery->FindActiveBinding(mapRecord->HashNumber());
+    nsDiskCacheBinding * binding = mBindery->FindActiveBinding(mapRecord->HashNumber());
     if (binding) {
         // We are currently using this entry, so all we can do is doom it.
         // Since we're enumerating the records, we don't want to call
         // DeleteRecord when nsCacheService::DoomEntry() calls us back.
         binding->mDoomed = PR_TRUE;         // mark binding record as 'deleted'
         nsCacheService::DoomEntry(binding->mCacheEntry);
-        result = kDeleteRecordAndContinue;  // this will REALLY delete the record
-
     } else {
         // entry not in use, just delete storage because we're enumerating the records
         (void) mCacheMap->DeleteStorage(mapRecord);
-        result = kDeleteRecordAndContinue;  // this will REALLY delete the record
     }
 
-exit:
-    
-    delete clientID;
-    delete [] (char *)diskEntry;
-    return result;
+    return kDeleteRecordAndContinue;  // this will REALLY delete the record
 }
 
 
@@ -216,7 +204,7 @@ NS_IMETHODIMP nsDiskCacheDeviceInfo::GetUsageReport(char ** usageReport)
     NS_ENSURE_ARG_POINTER(usageReport);
     nsCString buffer;
     
-    buffer.Append("\n<tr>\n<td><b>Cache Directory:</b></td>\n<td><tt> ");
+    buffer.AssignLiteral("\n<tr>\n<td><b>Cache Directory:</b></td>\n<td><tt> ");
     nsCOMPtr<nsILocalFile> cacheDir;
     nsAutoString           path;
     mDevice->getCacheDirectory(getter_AddRefs(cacheDir)); 
@@ -224,9 +212,9 @@ NS_IMETHODIMP nsDiskCacheDeviceInfo::GetUsageReport(char ** usageReport)
     if (NS_SUCCEEDED(rv)) {
         AppendUTF16toUTF8(path, buffer);
     } else {
-        buffer.Append("directory unavailable");
+        buffer.AppendLiteral("directory unavailable");
     }
-    buffer.Append("</tt></td>\n</tr>\n");
+    buffer.AppendLiteral("</tt></td>\n</tr>\n");
     // buffer.Append("<tr><td><b>Files:</b></td><td><tt> XXX</tt></td></tr>");
     *usageReport = ToNewCString(buffer);
     if (!*usageReport) return NS_ERROR_OUT_OF_MEMORY;
@@ -246,7 +234,8 @@ NS_IMETHODIMP nsDiskCacheDeviceInfo::GetEntryCount(PRUint32 *aEntryCount)
 NS_IMETHODIMP nsDiskCacheDeviceInfo::GetTotalSize(PRUint32 *aTotalSize)
 {
     NS_ENSURE_ARG_POINTER(aTotalSize);
-    *aTotalSize = mDevice->getCacheSize();
+    // Returned unit's are in bytes
+    *aTotalSize = mDevice->getCacheSize() * 1024;
     return NS_OK;
 }
 
@@ -254,7 +243,8 @@ NS_IMETHODIMP nsDiskCacheDeviceInfo::GetTotalSize(PRUint32 *aTotalSize)
 NS_IMETHODIMP nsDiskCacheDeviceInfo::GetMaximumSize(PRUint32 *aMaximumSize)
 {
     NS_ENSURE_ARG_POINTER(aMaximumSize);
-    *aMaximumSize = mDevice->getCacheCapacity();
+    // Returned unit's are in bytes
+    *aMaximumSize = mDevice->getCacheCapacity() * 1024;
     return NS_OK;
 }
 
@@ -333,16 +323,13 @@ nsDiskCache::Truncate(PRFileDesc *  fd, PRUint32  newEOF)
 
 nsDiskCacheDevice::nsDiskCacheDevice()
     : mCacheCapacity(0)
-    , mCacheMap(nsnull)
     , mInitialized(PR_FALSE)
-    , mFirstInit(PR_TRUE)
 {
 }
 
 nsDiskCacheDevice::~nsDiskCacheDevice()
 {
     Shutdown();
-    delete mCacheMap;
 }
 
 
@@ -356,28 +343,22 @@ nsDiskCacheDevice::Init()
 
     NS_ENSURE_TRUE(!Initialized(), NS_ERROR_FAILURE);
        
-    if (!mCacheDirectory) return NS_ERROR_FAILURE;
+    if (!mCacheDirectory)
+        return NS_ERROR_FAILURE;
+
     rv = mBindery.Init();
-    if (NS_FAILED(rv)) return rv;
+    if (NS_FAILED(rv))
+        return rv;
     
     // Open Disk Cache
     rv = OpenDiskCache();
     if (NS_FAILED(rv)) {
-        goto error_exit;
+        (void) mCacheMap.Close(PR_FALSE);
+        return rv;
     }
 
     mInitialized = PR_TRUE;
-    mFirstInit   = PR_FALSE;
     return NS_OK;
-
-error_exit:
-    if (mCacheMap)  {
-        (void) mCacheMap->Close(PR_FALSE);
-        delete mCacheMap;
-        mCacheMap = nsnull;
-    }
-
-    return rv;
 }
 
 
@@ -387,21 +368,36 @@ error_exit:
 nsresult
 nsDiskCacheDevice::Shutdown()
 {
-    return Shutdown_Private(PR_TRUE);
+    nsresult rv = Shutdown_Private(PR_TRUE);
+    if (NS_FAILED(rv))
+        return rv;
+
+    if (mCacheDirectory) {
+        // delete any trash files left-over before shutting down.
+        nsCOMPtr<nsIFile> trashDir;
+        GetTrashDir(mCacheDirectory, &trashDir);
+        if (trashDir) {
+            PRBool exists;
+            if (NS_SUCCEEDED(trashDir->Exists(&exists)) && exists)
+                DeleteDir(trashDir, PR_FALSE, PR_TRUE);
+        }
+    }
+
+    return NS_OK;
 }
 
 
 nsresult
 nsDiskCacheDevice::Shutdown_Private(PRBool  flush)
 {
-        if (Initialized()) {
+    CACHE_LOG_DEBUG(("CACHE: disk Shutdown_Private [%u]\n", flush));
+
+    if (Initialized()) {
         // check cache limits in case we need to evict.
-        EvictDiskCacheEntries((PRInt32)mCacheCapacity);
+        EvictDiskCacheEntries(mCacheCapacity);
 
         // write out persistent information about the cache.
-        (void) mCacheMap->Close(flush);
-        delete mCacheMap;
-        mCacheMap = nsnull;
+        (void) mCacheMap.Close(flush);
 
         mBindery.Reset();
 
@@ -445,15 +441,15 @@ nsDiskCacheDevice::FindEntry(nsCString * key)
 #endif
     
     // lookup hash number in cache map
-    rv = mCacheMap->FindRecord(hashNumber, &record);
+    rv = mCacheMap.FindRecord(hashNumber, &record);
     if (NS_FAILED(rv))  return nsnull;  // XXX log error?
     
     nsDiskCacheEntry * diskEntry;
-    rv = mCacheMap->ReadDiskCacheEntry(&record, &diskEntry);
+    rv = mCacheMap.ReadDiskCacheEntry(&record, &diskEntry);
     if (NS_FAILED(rv))  return nsnull;
     
     // compare key to be sure
-    if (nsCRT::strcmp(diskEntry->mKeyStart, key->get()) == 0) {
+    if (strcmp(diskEntry->mKeyStart, key->get()) == 0) {
         entry = diskEntry->CreateCacheEntry(this);
     }
     delete [] (char *)diskEntry;
@@ -482,16 +478,20 @@ nsDiskCacheDevice::DeactivateEntry(nsCacheEntry * entry)
     NS_ASSERTION(binding, "DeactivateEntry: binding == nsnull");
     if (!binding)  return NS_ERROR_UNEXPECTED;
 
+    CACHE_LOG_DEBUG(("CACHE: disk DeactivateEntry [%p %x]\n",
+        entry, binding->mRecord.HashNumber()));
+
     if (entry->IsDoomed()) {
         // delete data, entry, record from disk for entry
-        rv = mCacheMap->DeleteStorage(&binding->mRecord);
+        rv = mCacheMap.DeleteStorage(&binding->mRecord);
 
     } else {
         // save stuff to disk for entry
-        rv = mCacheMap->WriteDiskCacheEntry(binding);
+        rv = mCacheMap.WriteDiskCacheEntry(binding);
         if (NS_FAILED(rv)) {
             // clean up as best we can
-            (void) mCacheMap->DeleteRecordAndStorage(&binding->mRecord);
+            (void) mCacheMap.DeleteStorage(&binding->mRecord);
+            (void) mCacheMap.DeleteRecord(&binding->mRecord);
             binding->mDoomed = PR_TRUE; // record is no longer in cache map
         }
     }
@@ -528,9 +528,12 @@ nsDiskCacheDevice::BindEntry(nsCacheEntry * entry)
     record.SetHashNumber(nsDiskCache::Hash(entry->Key()->get()));
     record.SetEvictionRank(ULONG_MAX - SecondsFromPRTime(PR_Now()));
 
+    CACHE_LOG_DEBUG(("CACHE: disk BindEntry [%p %x]\n",
+        entry, record.HashNumber()));
+
     if (!entry->IsDoomed()) {
         // if entry isn't doomed, add it to the cache map
-        rv = mCacheMap->AddRecord(&record, &oldRecord); // deletes old record, if any
+        rv = mCacheMap.AddRecord(&record, &oldRecord); // deletes old record, if any
         if (NS_FAILED(rv))  return rv;
         
         PRUint32    oldHashNumber = oldRecord.HashNumber();
@@ -548,7 +551,7 @@ nsDiskCacheDevice::BindEntry(nsCacheEntry * entry)
             } else {
                 // delete storage
                 // XXX if debug : compare keys for hashNumber collision
-                rv = mCacheMap->DeleteStorage(&oldRecord);
+                rv = mCacheMap.DeleteStorage(&oldRecord);
                 if (NS_FAILED(rv))  return rv;  // XXX delete record we just added?
             }
         }
@@ -570,14 +573,16 @@ nsDiskCacheDevice::BindEntry(nsCacheEntry * entry)
 void
 nsDiskCacheDevice::DoomEntry(nsCacheEntry * entry)
 {
+    CACHE_LOG_DEBUG(("CACHE: disk DoomEntry [%p]\n", entry));
+
     nsDiskCacheBinding * binding = GetCacheEntryBinding(entry);
     NS_ASSERTION(binding, "DoomEntry: binding == nsnull");
     if (!binding)  return;
 
     if (!binding->mDoomed) {
         // so it can't be seen by FindEntry() ever again.
-        nsresult rv = mCacheMap->DoomRecord(&binding->mRecord);
-        NS_ASSERTION(NS_SUCCEEDED(rv),"DoomRecord failed.");
+        nsresult rv = mCacheMap.DeleteRecord(&binding->mRecord);
+        NS_ASSERTION(NS_SUCCEEDED(rv),"DeleteRecord failed.");
         binding->mDoomed = PR_TRUE; // record in no longer in cache map
     }
 }
@@ -592,6 +597,9 @@ nsDiskCacheDevice::OpenInputStreamForEntry(nsCacheEntry *      entry,
                                            PRUint32            offset,
                                            nsIInputStream **   result)
 {
+    CACHE_LOG_DEBUG(("CACHE: disk OpenInputStreamForEntry [%p %x %u]\n",
+        entry, mode, offset));
+
     NS_ENSURE_ARG_POINTER(entry);
     NS_ENSURE_ARG_POINTER(result);
 
@@ -617,6 +625,9 @@ nsDiskCacheDevice::OpenOutputStreamForEntry(nsCacheEntry *      entry,
                                             PRUint32            offset,
                                             nsIOutputStream **  result)
 {
+    CACHE_LOG_DEBUG(("CACHE: disk OpenOutputStreamForEntry [%p %x %u]\n",
+        entry, mode, offset));
+ 
     NS_ENSURE_ARG_POINTER(entry);
     NS_ENSURE_ARG_POINTER(result);
 
@@ -662,15 +673,15 @@ nsDiskCacheDevice::GetFileForEntry(nsCacheEntry *    entry,
         binding->mRecord.SetDataFileSize(0);    // 1k minimum
         if (!binding->mDoomed) {
             // record stored in cache map, so update it
-            rv = mCacheMap->UpdateRecord(&binding->mRecord);
+            rv = mCacheMap.UpdateRecord(&binding->mRecord);
             if (NS_FAILED(rv))  return rv;
         }
     }
     
     nsCOMPtr<nsIFile>  file;
-    rv = mCacheMap->GetFileForDiskCacheRecord(&binding->mRecord,
-                                              nsDiskCache::kData,
-                                              getter_AddRefs(file));
+    rv = mCacheMap.GetFileForDiskCacheRecord(&binding->mRecord,
+                                             nsDiskCache::kData,
+                                             getter_AddRefs(file));
     if (NS_FAILED(rv))  return rv;
     
     NS_IF_ADDREF(*result = file);
@@ -686,6 +697,13 @@ nsDiskCacheDevice::GetFileForEntry(nsCacheEntry *    entry,
 nsresult
 nsDiskCacheDevice::OnDataSizeChange(nsCacheEntry * entry, PRInt32 deltaSize)
 {
+    CACHE_LOG_DEBUG(("CACHE: disk OnDataSizeChange [%p %d]\n",
+        entry, deltaSize));
+
+    // If passed a negative value, then there's nothing to do.
+    if (deltaSize < 0)
+        return NS_OK;
+
     nsDiskCacheBinding * binding = GetCacheEntryBinding(entry);
     NS_ASSERTION(binding, "OnDataSizeChange: binding == nsnull");
     if (!binding)  return NS_ERROR_UNEXPECTED;
@@ -693,21 +711,25 @@ nsDiskCacheDevice::OnDataSizeChange(nsCacheEntry * entry, PRInt32 deltaSize)
     NS_ASSERTION(binding->mRecord.ValidRecord(), "bad record");
 
     PRUint32  newSize = entry->DataSize() + deltaSize;
-    PRUint32  maxSize = PR_MIN(mCacheCapacity / 2, kMaxDataFileSize);
-    if (newSize > maxSize) {
+    PRUint32  newSizeK =  ((newSize + 0x3FF) >> 10);
+
+    // If the new size is larger than max. file size or larger than
+    // half the cache capacity (which is in KiB's), doom the entry and abort
+    if ((newSize > kMaxDataFileSize) || (newSizeK > mCacheCapacity/2)) {
         nsresult rv = nsCacheService::DoomEntry(entry);
         NS_ASSERTION(NS_SUCCEEDED(rv),"DoomEntry() failed.");
         return NS_ERROR_ABORT;
     }
 
     PRUint32  sizeK = ((entry->DataSize() + 0x03FF) >> 10); // round up to next 1k
-    PRUint32  newSizeK =  ((newSize + 0x3FF) >> 10);
 
     NS_ASSERTION(sizeK < USHRT_MAX, "data size out of range");
     NS_ASSERTION(newSizeK < USHRT_MAX, "data size out of range");
 
     // pre-evict entries to make space for new data
-    PRInt32  targetCapacity = (PRInt32)(mCacheCapacity - ((newSizeK - sizeK) * 1024));
+    PRUint32  targetCapacity = mCacheCapacity > (newSizeK - sizeK)
+                             ? mCacheCapacity - (newSizeK - sizeK)
+                             : 0;
     EvictDiskCacheEntries(targetCapacity);
     
     return NS_OK;
@@ -720,13 +742,10 @@ nsDiskCacheDevice::OnDataSizeChange(nsCacheEntry * entry, PRInt32 deltaSize)
 class EntryInfoVisitor : public nsDiskCacheRecordVisitor
 {
 public:
-    EntryInfoVisitor(nsDiskCacheDevice * device,
-                     nsDiskCacheMap *    cacheMap,
+    EntryInfoVisitor(nsDiskCacheMap *    cacheMap,
                      nsICacheVisitor *   visitor)
-        : mDevice(device)
-        , mCacheMap(cacheMap)
+        : mCacheMap(cacheMap)
         , mVisitor(visitor)
-        , mResult(NS_OK)
     {}
     
     virtual PRInt32  VisitRecord(nsDiskCacheRecord *  mapRecord)
@@ -737,14 +756,12 @@ public:
         nsDiskCacheEntry * diskEntry;
         nsresult rv = mCacheMap->ReadDiskCacheEntry(mapRecord, &diskEntry);
         if (NS_FAILED(rv)) {
-            mResult = rv;
             return kVisitNextRecord;
         }
 
         // create nsICacheEntryInfo
         nsDiskCacheEntryInfo * entryInfo = new nsDiskCacheEntryInfo(DISK_CACHE_DEVICE_ID, diskEntry);
         if (!entryInfo) {
-            mResult = NS_ERROR_OUT_OF_MEMORY;
             return kStopVisitingRecords;
         }
         nsCOMPtr<nsICacheEntryInfo> ref(entryInfo);
@@ -756,10 +773,8 @@ public:
     }
  
 private:
-        nsDiskCacheDevice * mDevice;
         nsDiskCacheMap *    mCacheMap;
         nsICacheVisitor *   mVisitor;
-        nsresult            mResult;
 };
 
 
@@ -775,8 +790,8 @@ nsDiskCacheDevice::Visit(nsICacheVisitor * visitor)
     if (NS_FAILED(rv)) return rv;
     
     if (keepGoing) {
-        EntryInfoVisitor  infoVisitor(this, mCacheMap, visitor);
-        return mCacheMap->VisitRecords(&infoVisitor);
+        EntryInfoVisitor  infoVisitor(&mCacheMap, visitor);
+        return mCacheMap.VisitRecords(&infoVisitor);
     }
 
     return NS_OK;
@@ -786,6 +801,8 @@ nsDiskCacheDevice::Visit(nsICacheVisitor * visitor)
 nsresult
 nsDiskCacheDevice::EvictEntries(const char * clientID)
 {
+    CACHE_LOG_DEBUG(("CACHE: disk EvictEntries [%s]\n", clientID));
+
     if (!Initialized())  return NS_ERROR_NOT_INITIALIZED;
     nsresult  rv;
 
@@ -796,11 +813,11 @@ nsDiskCacheDevice::EvictEntries(const char * clientID)
             return rv;
     }
 
-    nsDiskCacheEvictor  evictor(this, mCacheMap, &mBindery, 0, clientID);
-    rv = mCacheMap->VisitRecords(&evictor);
+    nsDiskCacheEvictor  evictor(&mCacheMap, &mBindery, 0, clientID);
+    rv = mCacheMap.VisitRecords(&evictor);
     
     if (clientID == nsnull)     // we tried to clear the entire cache
-        rv = mCacheMap->Trim(); // so trim cache block files (if possible)
+        rv = mCacheMap.Trim(); // so trim cache block files (if possible)
     return rv;
 }
 
@@ -814,72 +831,54 @@ nsDiskCacheDevice::EvictEntries(const char * clientID)
 #endif
 
 
-// "Cache.Trash" directory is a sibling of the "Cache" directory
-nsresult
-nsDiskCacheDevice::GetCacheTrashDirectory(nsIFile ** result)
-{
-    nsCOMPtr<nsIFile> cacheTrashDir;
-    nsresult rv = mCacheDirectory->Clone(getter_AddRefs(cacheTrashDir));
-    if (NS_FAILED(rv))  return rv;
-    rv = cacheTrashDir->SetNativeLeafName(NS_LITERAL_CSTRING("Cache.Trash"));
-    if (NS_FAILED(rv))  return rv;
-    
-    *result = cacheTrashDir.get();
-    NS_ADDREF(*result);
-    return rv;
-}
-
-
 nsresult
 nsDiskCacheDevice::OpenDiskCache()
 {
-    nsresult  rv;
-
-    // Try opening cache map file.
-    NS_ASSERTION(mCacheMap == nsnull, "leaking mCacheMap");
-    mCacheMap = new nsDiskCacheMap;
-    if (!mCacheMap) {
-        return NS_ERROR_OUT_OF_MEMORY;
-    }
-    
     // if we don't have a cache directory, create one and open it
-    PRBool  cacheDirExists;
-    rv = mCacheDirectory->Exists(&cacheDirExists);
-    if (NS_FAILED(rv))  return rv;
+    PRBool exists;
+    nsresult rv = mCacheDirectory->Exists(&exists);
+    if (NS_FAILED(rv))
+        return rv;
 
-    if (cacheDirExists) {
-        rv = mCacheMap->Open(mCacheDirectory);        
+    PRBool trashing = PR_FALSE;
+    if (exists) {
+        // Try opening cache map file.
+        rv = mCacheMap.Open(mCacheDirectory);        
         // move "corrupt" caches to trash
         if (rv == NS_ERROR_FILE_CORRUPTED) {
-            rv = MoveCacheToTrash(nsnull); // ignore returned dir name
-            if (NS_FAILED(rv))  return rv;
-            cacheDirExists = PR_FALSE;
-
-        } else if (NS_FAILED(rv))  return rv;
+            rv = DeleteDir(mCacheDirectory, PR_TRUE, PR_FALSE);
+            if (NS_FAILED(rv))
+                return rv;
+            exists = PR_FALSE;
+            trashing = PR_TRUE;
+        }
+        else if (NS_FAILED(rv))
+            return rv;
     }
 
     // if we don't have a cache directory, create one and open it
-    if (!cacheDirExists) {
-        rv = InitializeCacheDirectory();
-        if (NS_FAILED(rv))  return rv;
+    if (!exists) {
+        rv = mCacheDirectory->Create(nsIFile::DIRECTORY_TYPE, 0777);
+        CACHE_LOG_PATH(PR_LOG_ALWAYS, "\ncreate cache directory: %s\n", mCacheDirectory);
+        CACHE_LOG_ALWAYS(("mCacheDirectory->Create() = %x\n", rv));
+        if (NS_FAILED(rv))
+            return rv;
+    
+        // reopen the cache map     
+        rv = mCacheMap.Open(mCacheDirectory);
+        if (NS_FAILED(rv))
+            return rv;
     }
 
-    if (! mFirstInit)  return NS_OK;  // we're done
-
-    // Empty Cache Trash
-    PRBool trashDirExists;
-    nsCOMPtr<nsIFile> trashDir;
-    rv = GetCacheTrashDirectory(getter_AddRefs(trashDir));
-    if (NS_FAILED(rv))  return rv;
-    rv = trashDir->Exists(&trashDirExists);
-    if (NS_FAILED(rv))  return rv;
-    if (trashDirExists) {
-        nsCOMArray<nsIFile> * trashList;
-        rv = ListTrashContents(&trashList);
-        if (NS_FAILED(rv))  return rv;
-
-        rv = DeleteFiles(trashList);       // spin up thread to delete contents
-        if (NS_FAILED(rv))  return rv;
+    if (!trashing) {
+        // delete any trash files leftover from a previous run
+        nsCOMPtr<nsIFile> trashDir;
+        GetTrashDir(mCacheDirectory, &trashDir);
+        if (trashDir) {
+            PRBool exists;
+            if (NS_SUCCEEDED(trashDir->Exists(&exists)) && exists)
+                DeleteDir(trashDir, PR_FALSE, PR_FALSE);
+        }
     }
 
     return NS_OK;
@@ -889,223 +888,37 @@ nsDiskCacheDevice::OpenDiskCache()
 nsresult
 nsDiskCacheDevice::ClearDiskCache()
 {
-    if (mBindery.ActiveBindings())  return NS_ERROR_CACHE_IN_USE;
+    if (mBindery.ActiveBindings())
+        return NS_ERROR_CACHE_IN_USE;
 
-    nsresult              rv;
-    nsCOMPtr<nsIFile>     trashDir;
-    nsCOMArray<nsIFile> * deleteList = new nsCOMArray<nsIFile>;
-    if (!deleteList)  return NS_ERROR_OUT_OF_MEMORY;
+    nsresult rv = Shutdown_Private(PR_FALSE);  // false: don't bother flushing
+    if (NS_FAILED(rv))
+        return rv;
 
-    rv = Shutdown_Private(PR_FALSE);  // false = don't bother flushing
-    if (NS_FAILED(rv))  goto error_exit;
+    // If the disk cache directory is already gone, then it's not an error if
+    // we fail to delete it ;-)
+    rv = DeleteDir(mCacheDirectory, PR_TRUE, PR_FALSE);
+    if (NS_FAILED(rv) && rv != NS_ERROR_FILE_TARGET_DOES_NOT_EXIST)
+        return rv;
 
-    rv = MoveCacheToTrash(getter_AddRefs(trashDir));
-    if (NS_FAILED(rv))  goto error_exit;
-    rv = deleteList->AppendObject(trashDir);
-    if (NS_FAILED(rv))  goto error_exit;
-    rv = DeleteFiles(deleteList);
-    if (NS_FAILED(rv))  goto error_exit;
-
-    rv = Init();
-    return rv;
-
- error_exit:
-    delete deleteList;
-    return rv;
+    return Init();
 }
 
 
-// function passed to PR_CreateThread
-static void PR_CALLBACK
-DoDeleteFileList(void *arg)
-{
-    nsCOMArray<nsIFile> * fileList = NS_STATIC_CAST(nsCOMArray<nsIFile> *, arg);
-    nsresult              rv;
-
-    // iterate over items in fileList, recursively deleting each
-    PRInt32  count = fileList->Count();
-    for (PRInt32 i=0; i<count; i++) {
-        nsIFile * item = fileList->ObjectAt(i);
-        CACHE_LOG_PATH(PR_LOG_ALWAYS, "deleting: %s\n", item);
-
-        rv = item->Remove(PR_TRUE);
-        NS_ASSERTION(NS_SUCCEEDED(rv),"failure cleaning up cache");
-    }
-
-    delete fileList; // destroy nsCOMArray
-}
-
-
-#define DEFAULT_STACK_SIZE  0
-
 nsresult
-nsDiskCacheDevice::DeleteFiles(nsCOMArray<nsIFile> * fileList)
+nsDiskCacheDevice::EvictDiskCacheEntries(PRUint32  targetCapacity)
 {
-    // start up another thread to delete deleteDir
-    PRThread  * thread;
-    thread = PR_CreateThread(PR_USER_THREAD,
-                             DoDeleteFileList,
-                             fileList,
-                             PR_PRIORITY_NORMAL,
-                             PR_GLOBAL_THREAD,
-                             PR_UNJOINABLE_THREAD,
-                             DEFAULT_STACK_SIZE);
+    CACHE_LOG_DEBUG(("CACHE: disk EvictDiskCacheEntries [%u]\n",
+        targetCapacity));
 
-    if (!thread)  return NS_ERROR_UNEXPECTED;
-    return NS_OK;
-}
+    NS_ASSERTION(targetCapacity > 0, "oops");
 
-
-/**
- *  ListTrashContents - return pointer to array of nsIFile to delete
- */
-nsresult
-nsDiskCacheDevice::ListTrashContents(nsCOMArray<nsIFile> ** result)
-{
-    nsresult           rv;
-    nsCOMPtr<nsIFile>  trashDir;
-    *result = nsnull;
-    
-    // does "Cache.Trash" directory exist?
-    rv = GetCacheTrashDirectory(getter_AddRefs(trashDir));
-    if (NS_FAILED(rv))  return rv;
-    PRBool exists;
-    rv = trashDir->Exists(&exists);
-    if (NS_FAILED(rv))  return rv;
-    if (!exists) {
+    if (mCacheMap.TotalSize() < targetCapacity)
         return NS_OK;
-    }
 
-    nsCOMArray<nsIFile> * array = new nsCOMArray<nsIFile>;
-    if (!array)  return NS_ERROR_OUT_OF_MEMORY;
-    
-    // iterate over trash directory building array of directory objects
-    nsCOMPtr<nsISimpleEnumerator> dirEntries;
-    nsCOMPtr<nsIFile> item;
-    PRBool            more, success;
-
-    rv = trashDir->GetDirectoryEntries(getter_AddRefs(dirEntries));
-    if (NS_FAILED(rv) || !dirEntries)  goto error_exit; // !dirEntries returns NS_OK
-
-    rv = dirEntries->HasMoreElements(&more);
-    if (NS_FAILED(rv))  goto error_exit;
-
-    while (more) {
-        rv = dirEntries->GetNext(getter_AddRefs(item));
-        if (NS_FAILED(rv))  goto error_exit;
-
-        success = array->AppendObject(item);
-        if (!success) {
-            rv = NS_ERROR_OUT_OF_MEMORY;
-            goto error_exit;
-        }
-
-        rv = dirEntries->HasMoreElements(&more);
-        if (NS_FAILED(rv))  goto error_exit;
-    }
-    
-    // return resulting array
-    *result = array;
-    return NS_OK;
-    
- error_exit:
-    delete array;
-    return rv;
-}
-
-
-// Move 'Cache' dir into unique directory inside 'Cache.Trash', return name of unique directory
-nsresult
-nsDiskCacheDevice::MoveCacheToTrash(nsIFile ** result)
-{
-    nsresult          rv;
-    nsCOMPtr<nsIFile> trashDir;
-
-    if (result) *result = nsnull;
-
-    rv = GetCacheTrashDirectory(getter_AddRefs(trashDir));
-    if (NS_FAILED(rv))  return rv;
-
-    // verify cache.trash exists and is a directory
-    PRBool  exists;
-    rv = trashDir->Exists(&exists);
-    if (NS_FAILED(rv))  return rv;
-    if (exists) {
-        PRBool  isDirectory;
-        rv = trashDir->IsDirectory(&isDirectory);
-        if (NS_FAILED(rv))  return rv;
-        if (!isDirectory) {
-            // delete file or fail
-            rv = trashDir->Remove(PR_FALSE);
-            if (NS_FAILED(rv))  return rv;
-            exists = PR_FALSE;
-        }
-    }
-
-    if (!exists) {
-        // cache.trash doesn't exists, so create it
-        rv = trashDir->Create(nsIFile::DIRECTORY_TYPE, 0777);
-        if (NS_FAILED(rv))  return rv;
-    }
-
-    // create unique directory
-    nsCOMPtr<nsIFile>  uniqueDir;
-    rv = trashDir->Clone(getter_AddRefs(uniqueDir));
-    if (NS_FAILED(rv))  return rv;
-    rv = uniqueDir->AppendNative(NS_LITERAL_CSTRING("Trash"));
-    if (NS_FAILED(rv))  return rv;
-    rv = uniqueDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0777);
-    if (NS_FAILED(rv))  return rv;
-
-    // move cache directory into unique trash directory
-    nsCOMPtr<nsIFile> parentDir;
-
-    rv = mCacheDirectory->GetParent(getter_AddRefs(parentDir));
-    if (NS_FAILED(rv))  return rv;
-    
-    rv = mCacheDirectory->MoveToNative(uniqueDir, EmptyCString());
-    if (NS_FAILED(rv))  return rv;
-    
-    // set mCacheDirectory to point to parentDir/Cache/ again
-    rv = parentDir->AppendNative(NS_LITERAL_CSTRING("Cache"));
-    if (NS_FAILED(rv))  return rv;
-    
-    mCacheDirectory = do_QueryInterface(parentDir);
-
-    // return unique directory, in case caller wants specifically delete it
-    if (result)
-        NS_ADDREF(*result = uniqueDir);
-    return NS_OK;
-}
-
-
-nsresult
-nsDiskCacheDevice::InitializeCacheDirectory()
-{
-    nsresult rv;
-    
-    rv = mCacheDirectory->Create(nsIFile::DIRECTORY_TYPE, 0777);
-    CACHE_LOG_PATH(PR_LOG_ALWAYS, "\ncreate cache directory: %s\n", mCacheDirectory);
-    CACHE_LOG_ALWAYS(("mCacheDirectory->Create() = %x\n", rv));
-    if (NS_FAILED(rv))  return rv;
-
-    // reopen the cache map     
-    rv = mCacheMap->Open(mCacheDirectory);
-    return rv;
-}
-
-
-nsresult
-nsDiskCacheDevice::EvictDiskCacheEntries(PRInt32  targetCapacity)
-{
-    nsresult rv;
-    
-    if (mCacheMap->TotalSize() < targetCapacity)  return NS_OK;
-
-    nsDiskCacheEvictor  evictor(this, mCacheMap, &mBindery, targetCapacity, nsnull);
-    rv = mCacheMap->EvictRecords(&evictor);
-    
-    return rv;
+    // targetCapacity is in KiB's
+    nsDiskCacheEvictor  evictor(&mCacheMap, &mBindery, targetCapacity, nsnull);
+    return mCacheMap.EvictRecords(&evictor);
 }
 
 
@@ -1165,10 +978,11 @@ nsDiskCacheDevice::getCacheDirectory(nsILocalFile ** result)
 void
 nsDiskCacheDevice::SetCapacity(PRUint32  capacity)
 {
-    mCacheCapacity = capacity * 1024;
+    // Units are KiB's
+    mCacheCapacity = capacity;
     if (Initialized()) {
         // start evicting entries if the new size is smaller!
-        EvictDiskCacheEntries((PRInt32)mCacheCapacity);
+        EvictDiskCacheEntries(mCacheCapacity);
     }
 }
 
@@ -1181,11 +995,11 @@ PRUint32 nsDiskCacheDevice::getCacheCapacity()
 
 PRUint32 nsDiskCacheDevice::getCacheSize()
 {
-    return mCacheMap->TotalSize();
+    return mCacheMap.TotalSize();
 }
 
 
 PRUint32 nsDiskCacheDevice::getEntryCount()
 {
-    return mCacheMap->EntryCount();
+    return mCacheMap.EntryCount();
 }
