@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim:set ts=4 sw=4 sts=4 et cin: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -45,12 +46,11 @@
 #include "nsISocketTransport.h"
 #include "nsIServiceManager.h"
 #include "nsISSLSocketControl.h"
-#include "nsIStringStream.h"
+#include "nsStringStream.h"
 #include "netCore.h"
 #include "nsNetCID.h"
 #include "nsAutoLock.h"
 #include "prmem.h"
-#include "plevent.h"
 
 #ifdef DEBUG
 // defined by the socket transport service while active
@@ -67,7 +67,6 @@ nsHttpConnection::nsHttpConnection()
     : mTransaction(nsnull)
     , mConnInfo(nsnull)
     , mLock(nsnull)
-    , mSuspendCount(0)
     , mLastReadTime(0)
     , mIdleTimeout(0)
     , mKeepAlive(PR_TRUE) // assume to keep-alive by default
@@ -283,6 +282,16 @@ nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
     NS_ENSURE_ARG_POINTER(trans);
     NS_ASSERTION(responseHead, "No response head?");
 
+    // If the server issued an explicit timeout, then we need to close down the
+    // socket transport.  We pass an error code of NS_ERROR_NET_RESET to
+    // trigger the transactions 'restart' mechanism.  We tell it to reset its
+    // response headers so that it will be ready to receive the new response.
+    if (responseHead->Status() == 408) {
+        Close(NS_ERROR_NET_RESET);
+        *reset = PR_TRUE;
+        return NS_OK;
+    }
+
     // we won't change our keep-alive policy unless the server has explicitly
     // told us to do so.
 
@@ -579,7 +588,8 @@ nsHttpConnection::OnSocketWritable()
             // here to reflect the fact that we are waiting.  this message will be
             // trumped (overwritten) if the server responds quickly.
             //
-            mTransaction->OnTransportStatus(nsISocketTransport::STATUS_WAITING_FOR, 0);
+            mTransaction->OnTransportStatus(nsISocketTransport::STATUS_WAITING_FOR,
+                                            LL_ZERO);
 
             rv = mSocketIn->AsyncWait(this, 0, 0, nsnull); // start reading
             again = PR_FALSE;
@@ -701,7 +711,7 @@ nsHttpConnection::SetupSSLProxyConnect()
 
     buf.Truncate();
     request.Flatten(buf, PR_FALSE);
-    buf.Append("\r\n");
+    buf.AppendLiteral("\r\n");
 
     return NS_NewCStringInputStream(getter_AddRefs(mSSLProxyConnectStream), buf);
 }
@@ -770,8 +780,8 @@ nsHttpConnection::OnOutputStreamReady(nsIAsyncOutputStream *out)
 NS_IMETHODIMP
 nsHttpConnection::OnTransportStatus(nsITransport *trans,
                                     nsresult status,
-                                    PRUint32 progress,
-                                    PRUint32 progressMax)
+                                    PRUint64 progress,
+                                    PRUint64 progressMax)
 {
     if (mTransaction)
         mTransaction->OnTransportStatus(status, progress);
@@ -786,8 +796,12 @@ nsHttpConnection::OnTransportStatus(nsITransport *trans,
 NS_IMETHODIMP
 nsHttpConnection::GetInterface(const nsIID &iid, void **result)
 {
-    NS_ASSERTION(PR_GetCurrentThread() == gSocketThread, "wrong thread");
-
+    // NOTE: This function is only called on the UI thread via sync proxy from
+    //       the socket transport thread.  If that weren't the case, then we'd
+    //       have to worry about the possibility of mTransaction going away
+    //       part-way through this function call.  See CloseTransaction.
+    NS_ASSERTION(PR_GetCurrentThread() != gSocketThread, "wrong thread");
+ 
     if (mTransaction) {
         nsCOMPtr<nsIInterfaceRequestor> callbacks;
         mTransaction->GetSecurityCallbacks(getter_AddRefs(callbacks));
