@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim:set ts=4 sw=4 sts=4 et cin: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -44,17 +45,19 @@
 #include "nsHttpResponseHead.h"
 #include "nsHttpChunkedDecoder.h"
 #include "nsTransportUtils.h"
+#include "nsProxyRelease.h"
 #include "nsIOService.h"
 #include "nsAutoLock.h"
 #include "pratom.h"
-#include "plevent.h"
 
-#include "nsIStringStream.h"
 #include "nsISeekableStream.h"
 #include "nsISocketTransport.h"
 #include "nsMultiplexInputStream.h"
+#include "nsStringStream.h"
 
 #include "nsComponentManagerUtils.h" // do_CreateInstance
+#include "nsServiceManagerUtils.h"   // do_GetService
+#include "nsIHttpActivityObserver.h"
 
 //-----------------------------------------------------------------------------
 
@@ -125,10 +128,10 @@ nsHttpTransaction::nsHttpTransaction()
     , mContentRead(0)
     , mChunkedDecoder(nsnull)
     , mStatus(NS_OK)
+    , mPriority(0)
     , mRestartCount(0)
     , mCaps(0)
     , mClosed(PR_FALSE)
-    , mDestroying(PR_FALSE)
     , mConnected(PR_FALSE)
     , mHaveStatusLine(PR_FALSE)
     , mHaveAllHeaders(PR_FALSE)
@@ -162,7 +165,7 @@ nsHttpTransaction::Init(PRUint8 caps,
                         nsHttpRequestHead *requestHead,
                         nsIInputStream *requestBody,
                         PRBool requestBodyHasHeaders,
-                        nsIEventQueue *queue,
+                        nsIEventTarget *target,
                         nsIInterfaceRequestor *callbacks,
                         nsITransportEventSink *eventsink,
                         nsIAsyncInputStream **responseBody)
@@ -173,16 +176,36 @@ nsHttpTransaction::Init(PRUint8 caps,
 
     NS_ASSERTION(cinfo, "ouch");
     NS_ASSERTION(requestHead, "ouch");
-    NS_ASSERTION(queue, "ouch");
+    NS_ASSERTION(target, "ouch");
 
     // create transport event sink proxy that coalesces all events
     rv = net_NewTransportEventSinkProxy(getter_AddRefs(mTransportSink),
-                                        eventsink, queue, PR_TRUE);
+                                        eventsink, target, PR_TRUE);
     if (NS_FAILED(rv)) return rv;
 
+    // try to get the nsIHttpActivityObserver distributor
+    mActivityDistributor = do_GetService(NS_HTTPACTIVITYDISTRIBUTOR_CONTRACTID, &rv);
+
+    // mActivityDistributor may not be valid
+    if (NS_SUCCEEDED(rv) && mActivityDistributor) {
+        // the service is valid, now check if it is active
+        PRBool active;
+        rv = mActivityDistributor->GetIsActive(&active);
+        if (NS_SUCCEEDED(rv) && active) {
+            // the service is valid and active, gather nsISupports
+            // for the channel that called Init()
+            mChannel = do_QueryInterface(eventsink);
+            LOG(("nsHttpTransaction::Init() " \
+                 "mActivityDistributor is active " \
+                 "this=%x", this));
+        } else
+            // the interface in valid but not active, so don't use it
+            mActivityDistributor = nsnull;
+    }
+
     NS_ADDREF(mConnInfo = cinfo);
     mCallbacks = callbacks;
-    mConsumerEventQ = queue;
+    mConsumerTarget = target;
     mCaps = caps;
 
     if (requestHead->Method() == nsHttp::Head)
@@ -209,7 +232,16 @@ nsHttpTransaction::Init(PRUint8 caps,
     // If the request body does not include headers or if there is no request
     // body, then we must add the header/body separator manually.
     if (!requestBodyHasHeaders || !requestBody)
-        mReqHeaderBuf.Append("\r\n");
+        mReqHeaderBuf.AppendLiteral("\r\n");
+
+    // report the request header
+    if (mActivityDistributor)
+        mActivityDistributor->ObserveActivity(
+            mChannel,
+            NS_HTTP_ACTIVITY_TYPE_HTTP_TRANSACTION,
+            NS_HTTP_ACTIVITY_SUBTYPE_REQUEST_HEADER,
+            LL_ZERO, LL_ZERO,
+            mReqHeaderBuf);
 
     // Create a string stream for the request header buf (the stream holds
     // a non-owning reference to the request header data, so we MUST keep
@@ -286,9 +318,9 @@ nsHttpTransaction::GetSecurityCallbacks(nsIInterfaceRequestor **cb)
 }
 
 void
-nsHttpTransaction::OnTransportStatus(nsresult status, PRUint32 progress)
+nsHttpTransaction::OnTransportStatus(nsresult status, PRUint64 progress)
 {
-    LOG(("nsHttpTransaction::OnSocketStatus [this=%x status=%x progress=%u]\n",
+    LOG(("nsHttpTransaction::OnSocketStatus [this=%x status=%x progress=%llu]\n",
         this, status, progress));
 
     if (!mTransportSink)
@@ -300,7 +332,27 @@ nsHttpTransaction::OnTransportStatus(nsresult status, PRUint32 progress)
     if (status == nsISocketTransport::STATUS_RECEIVING_FROM)
         return;
 
-    PRUint32 progressMax;
+    if (mActivityDistributor) {
+        // upon STATUS_WAITING_FOR; report request body sent
+        if ((mHasRequestBody) &&
+            (status == nsISocketTransport::STATUS_WAITING_FOR))
+            mActivityDistributor->ObserveActivity(
+                mChannel,
+                NS_HTTP_ACTIVITY_TYPE_HTTP_TRANSACTION,
+                NS_HTTP_ACTIVITY_SUBTYPE_REQUEST_BODY_SENT,
+                LL_ZERO, LL_ZERO, EmptyCString());
+
+        // report the status and progress
+        mActivityDistributor->ObserveActivity(
+            mChannel,
+            NS_HTTP_ACTIVITY_TYPE_SOCKET_TRANSPORT,
+            NS_STATIC_CAST(PRUint32, status),
+            LL_ZERO,
+            progress,
+            EmptyCString());
+    }
+
+    nsUint64 progressMax;
 
     if (status == nsISocketTransport::STATUS_SENDING_TO) {
         // suppress progress when only writing request headers
@@ -309,10 +361,10 @@ nsHttpTransaction::OnTransportStatus(nsresult status, PRUint32 progress)
 
         // when uploading, we include the request headers in the progress
         // notifications.
-        progressMax = mRequestSize;
+        progressMax = mRequestSize; // XXX mRequestSize is 32-bit!
     }
     else {
-        progress = 0;
+        progress = LL_ZERO;
         progressMax = 0;
     }
 
@@ -385,7 +437,7 @@ nsHttpTransaction::ReadSegments(nsAHttpSegmentReader *reader,
                 do_QueryInterface(mRequestStream);
         if (asyncIn) {
             nsCOMPtr<nsIEventTarget> target;
-            gHttpHandler->GetSocketThreadEventTarget(getter_AddRefs(target));
+            gHttpHandler->GetSocketThreadTarget(getter_AddRefs(target));
             if (target)
                 asyncIn->AsyncWait(this, 0, 0, target);
             else {
@@ -449,7 +501,7 @@ nsHttpTransaction::WriteSegments(nsAHttpSegmentWriter *writer,
     // occur on socket thread so we stay synchronized.
     if (rv == NS_BASE_STREAM_WOULD_BLOCK) {
         nsCOMPtr<nsIEventTarget> target;
-        gHttpHandler->GetSocketThreadEventTarget(getter_AddRefs(target));
+        gHttpHandler->GetSocketThreadTarget(getter_AddRefs(target));
         if (target)
             mPipeOut->AsyncWait(this, 0, 0, target);
         else {
@@ -473,6 +525,25 @@ nsHttpTransaction::Close(nsresult reason)
         return;
     }
 
+    if (mActivityDistributor) {
+        // report the reponse is complete if not already reported
+        if (!mResponseIsComplete)
+            mActivityDistributor->ObserveActivity(
+                mChannel,
+                NS_HTTP_ACTIVITY_TYPE_HTTP_TRANSACTION,
+                NS_HTTP_ACTIVITY_SUBTYPE_RESPONSE_COMPLETE,
+                LL_ZERO,
+                NS_STATIC_CAST(PRUint64, mContentRead.mValue),
+                EmptyCString());
+
+        // report that this transaction is closing
+        mActivityDistributor->ObserveActivity(
+            mChannel,
+            NS_HTTP_ACTIVITY_TYPE_HTTP_TRANSACTION,
+            NS_HTTP_ACTIVITY_SUBTYPE_TRANSACTION_CLOSE,
+            LL_ZERO, LL_ZERO, EmptyCString());
+    }
+
     // we must no longer reference the connection!  find out if the 
     // connection was being reused before letting it go.
     PRBool connReused = PR_FALSE;
@@ -508,12 +579,17 @@ nsHttpTransaction::Close(nsresult reason)
 
     PRBool relConn = PR_TRUE;
     if (NS_SUCCEEDED(reason)) {
-        // the server has not sent the final \r\n terminating the header section,
-        // and there is still a header line unparsed.  let's make sure we parse
-        // the remaining header line, and then hopefully, the response will be
-        // usable (see bug 88792).
-        if (!mHaveAllHeaders && !mLineBuf.IsEmpty())
-            ParseLineSegment("\n", 1);
+        // the server has not sent the final \r\n terminating the header
+        // section, and there may still be a header line unparsed.  let's make
+        // sure we parse the remaining header line, and then hopefully, the
+        // response will be usable (see bug 88792).  related to that, we may
+        // also have an empty response containing no headers.  we should treat
+        // that as an empty HTTP/0.9 response (see bug 300613).
+        if (!mHaveAllHeaders) {
+            char data = '\n';
+            PRUint32 unused;
+            ParseHead(&data, 1, &unused);
+        }
 
         // honor the sticky connection flag...
         if (mCaps & NS_HTTP_STICKY_CONNECTION)
@@ -565,7 +641,12 @@ nsHttpTransaction::Restart()
     mSecurityInfo = 0;
     NS_IF_RELEASE(mConnection);
 
-    return gHttpHandler->InitiateTransaction(this);
+    // disable pipelining for the next attempt in case pipelining caused the
+    // reset.  this is being overly cautious since we don't know if pipelining
+    // was the problem here.
+    mCaps &= ~NS_HTTP_ALLOW_PIPELINING;
+
+    return gHttpHandler->InitiateTransaction(this, mPriority);
 }
 
 void
@@ -643,6 +724,14 @@ nsHttpTransaction::ParseHead(char *buf,
         mResponseHead = new nsHttpResponseHead();
         if (!mResponseHead)
             return NS_ERROR_OUT_OF_MEMORY;
+
+        // report that we have a least some of the response
+        if (mActivityDistributor)
+            mActivityDistributor->ObserveActivity(
+                mChannel,
+                NS_HTTP_ACTIVITY_TYPE_HTTP_TRANSACTION,
+                NS_HTTP_ACTIVITY_SUBTYPE_RESPONSE_START,
+                LL_ZERO, LL_ZERO, EmptyCString());
     }
 
     // if we don't have a status line and the line buf is empty, then
@@ -729,6 +818,8 @@ nsHttpTransaction::HandleContentStart()
             LOG(("resetting transaction's response head\n"));
             mHaveAllHeaders = PR_FALSE;
             mHaveStatusLine = PR_FALSE;
+            mReceivedData = PR_FALSE;
+            mSentData = PR_FALSE;
             mResponseHead->Reset();
             // wait to be called again...
             return NS_OK;
@@ -754,8 +845,9 @@ nsHttpTransaction::HandleContentStart()
             // we're done with the socket.  please note that _all_ other
             // decoding is done when the channel receives the content data
             // so as not to block the socket transport thread too much.
-            const char *val = mResponseHead->PeekHeader(nsHttp::Transfer_Encoding);
-            if (PL_strcasestr(val, "chunked")) {
+            // ignore chunked responses from HTTP/1.0 servers and proxies.
+            if (mResponseHead->Version() >= NS_HTTP_VERSION_1_1 &&
+                mResponseHead->HasHeaderValue(nsHttp::Transfer_Encoding, "chunked")) {
                 // we only support the "chunked" transfer encoding right now.
                 mChunkedDecoder = new nsHttpChunkedDecoder();
                 if (!mChunkedDecoder)
@@ -796,7 +888,7 @@ nsHttpTransaction::HandleContent(char *buf,
         if (NS_FAILED(rv)) return rv;
         // Do not write content to the pipe if we haven't started streaming yet
         if (!mDidContentStart)
-          return NS_OK;
+            return NS_OK;
     }
 
     if (mChunkedDecoder) {
@@ -811,18 +903,20 @@ nsHttpTransaction::HandleContent(char *buf,
         // allowances for a possibly invalid Content-Length header. Thus, if
         // NOT persistent, we simply accept everything in |buf|.
         if (mConnection->IsPersistent()) {
-            *contentRead = mContentLength - mContentRead;
-            *contentRead = PR_MIN(count, *contentRead);
+            nsInt64 remaining = mContentLength - mContentRead;
+            nsInt64 count64 = count;
+            *contentRead = PR_MIN(count64, remaining);
+            *contentRemaining = count - *contentRead;
         }
         else {
             *contentRead = count;
             // mContentLength might need to be increased...
-            if (nsInt64(*contentRead) + mContentRead > mContentLength) {
-                mContentLength = nsInt64(*contentRead) + mContentRead;
+            nsInt64 position = mContentRead + nsInt64(count);
+            if (position > mContentLength) {
+                mContentLength = position;
                 //mResponseHead->SetContentLength(mContentLength);
             }
         }
-        *contentRemaining = (count - *contentRead);
     }
     else {
         // when we are just waiting for the server to close the connection...
@@ -833,7 +927,7 @@ nsHttpTransaction::HandleContent(char *buf,
     if (*contentRead) {
         // update count of content bytes read and report progress...
         mContentRead += *contentRead;
-        /*
+        /* when uncommenting, take care of 64-bit integers w/ PR_MAX...
         if (mProgressSink)
             mProgressSink->OnProgress(nsnull, nsnull, mContentRead, PR_MAX(0, mContentLength));
         */
@@ -848,6 +942,16 @@ nsHttpTransaction::HandleContent(char *buf,
         // the transaction is done with a complete response.
         mTransactionDone = PR_TRUE;
         mResponseIsComplete = PR_TRUE;
+
+        // report the entire response has arrived
+        if (mActivityDistributor)
+            mActivityDistributor->ObserveActivity(
+                mChannel,
+                NS_HTTP_ACTIVITY_TYPE_HTTP_TRANSACTION,
+                NS_HTTP_ACTIVITY_SUBTYPE_RESPONSE_COMPLETE,
+                LL_ZERO,
+                NS_STATIC_CAST(PRUint64, mContentRead.mValue),
+                EmptyCString());
     }
 
     return NS_OK;
@@ -874,6 +978,19 @@ nsHttpTransaction::ProcessData(char *buf, PRUint32 count, PRUint32 *countRead)
         // if buf has some content in it, shift bytes to top of buf.
         if (count && bytesConsumed)
             memmove(buf, buf + bytesConsumed, count);
+
+        // report the completed response header
+        if (mActivityDistributor && mResponseHead && mHaveAllHeaders) {
+            nsCAutoString completeResponseHeaders;
+            mResponseHead->Flatten(completeResponseHeaders, PR_FALSE);
+            completeResponseHeaders.AppendLiteral("\r\n");
+            mActivityDistributor->ObserveActivity(
+                mChannel,
+                NS_HTTP_ACTIVITY_TYPE_HTTP_TRANSACTION,
+                NS_HTTP_ACTIVITY_SUBTYPE_RESPONSE_HEADER,
+                LL_ZERO, LL_ZERO,
+                completeResponseHeaders);
+        }
     }
 
     // even though count may be 0, we still want to call HandleContent
@@ -911,57 +1028,37 @@ nsHttpTransaction::ProcessData(char *buf, PRUint32 count, PRUint32 *countRead)
 // nsHttpTransaction deletion event
 //-----------------------------------------------------------------------------
 
+class nsDeleteHttpTransaction : public nsRunnable {
+public:
+    nsDeleteHttpTransaction(nsHttpTransaction *trans)
+        : mTrans(trans)
+    {}
+
+    NS_IMETHOD Run()
+    {
+        delete mTrans;
+        return NS_OK;
+    }
+private:
+    nsHttpTransaction *mTrans;
+};
+
 void
 nsHttpTransaction::DeleteSelfOnConsumerThread()
 {
-    nsCOMPtr<nsIEventQueueService> eqs;
-    nsCOMPtr<nsIEventQueue> currentEventQ;
-
     LOG(("nsHttpTransaction::DeleteSelfOnConsumerThread [this=%x]\n", this));
     
-    NS_ASSERTION(!mDestroying, "deleting self again");
-    mDestroying = PR_TRUE;
-
-    gHttpHandler->GetCurrentEventQ(getter_AddRefs(currentEventQ));
-
-    if (currentEventQ == mConsumerEventQ)
+    PRBool val;
+    if (NS_SUCCEEDED(mConsumerTarget->IsOnCurrentThread(&val)) && val)
         delete this;
     else {
         LOG(("proxying delete to consumer thread...\n"));
-
-        PLEvent *event = new PLEvent;
-        if (!event) {
-            NS_WARNING("out of memory");
-            // probably better to leak |this| than to delete it on this thread.
-            return;
-        }
-
-        PL_InitEvent(event, this, DeleteThis_Handler, DeleteThis_Cleanup);
-
-        nsresult status = mConsumerEventQ->PostEvent(event);
-        if (NS_FAILED(status))
-            NS_ERROR("PostEvent failed");
+        nsCOMPtr<nsIRunnable> event = new nsDeleteHttpTransaction(this);
+        if (NS_FAILED(mConsumerTarget->Dispatch(event, NS_DISPATCH_NORMAL)))
+            NS_WARNING("failed to dispatch nsHttpDeleteTransaction event");
     }
 }
 
-void *PR_CALLBACK
-nsHttpTransaction::DeleteThis_Handler(PLEvent *ev)
-{
-    nsHttpTransaction *trans =
-            NS_STATIC_CAST(nsHttpTransaction *, PL_GetEventOwner(ev));
-
-    LOG(("nsHttpTransaction::DeleteThis_EventHandlerFunc [trans=%x]\n", trans));
-
-    delete trans;
-    return nsnull;
-}
-
-void PR_CALLBACK
-nsHttpTransaction::DeleteThis_Cleanup(PLEvent *ev)
-{
-    delete ev;
-}
-
 //-----------------------------------------------------------------------------
 // nsHttpTransaction::nsISupports
 //-----------------------------------------------------------------------------
