@@ -39,7 +39,8 @@
 
 #include "nsIndexedToHTML.h"
 #include "nsNetUtil.h"
-#include "nsIStringStream.h"
+#include "netCore.h"
+#include "nsStringStream.h"
 #include "nsIFileURL.h"
 #include "nsEscape.h"
 #include "nsIDirIndex.h"
@@ -56,8 +57,6 @@ NS_IMPL_THREADSAFE_ISUPPORTS4(nsIndexedToHTML,
                               nsIRequestObserver,
                               nsIStreamListener)
 
-static NS_DEFINE_CID(kDateTimeFormatCID, NS_DATETIMEFORMAT_CID);
-
 static void ConvertNonAsciiToNCR(const nsAString& in, nsAFlatString& out)
 {
   nsAString::const_iterator start, end;
@@ -100,15 +99,13 @@ nsIndexedToHTML::Init(nsIStreamListener* aListener) {
 
     mListener = aListener;
 
-    mDateTime = do_CreateInstance(kDateTimeFormatCID, &rv);
+    mDateTime = do_CreateInstance(NS_DATETIMEFORMAT_CONTRACTID, &rv);
 
     nsCOMPtr<nsIStringBundleService> sbs =
         do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
-    rv = sbs->CreateBundle("chrome://necko/locale/necko.properties",
-                           getter_AddRefs(mBundle));
+    rv = sbs->CreateBundle(NECKO_MSGS_URL, getter_AddRefs(mBundle));
 
-    mRowCount = 0;
     mExpectAbsLoc = PR_FALSE;
 
     return rv;
@@ -167,6 +164,7 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
     // - bbaetz
 
     PRBool isScheme = PR_FALSE;
+    PRBool isSchemeFile = PR_FALSE;
     if (NS_SUCCEEDED(uri->SchemeIs("ftp", &isScheme)) && isScheme) {
 
         // ftp urls don't always end in a /
@@ -200,11 +198,11 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
                  titleUri.Append('/');
         }
 
-        if (!path.Equals("//") && !path.LowerCaseEqualsLiteral("/%2f")) {
+        if (!path.EqualsLiteral("//") && !path.LowerCaseEqualsLiteral("/%2f")) {
             rv = uri->Resolve(NS_LITERAL_CSTRING(".."),parentStr);
             if (NS_FAILED(rv)) return rv;
         }
-    } else if (NS_SUCCEEDED(uri->SchemeIs("file", &isScheme)) && isScheme) {
+    } else if (NS_SUCCEEDED(uri->SchemeIs("file", &isSchemeFile)) && isSchemeFile) {
         nsCOMPtr<nsIFileURL> fileUrl = do_QueryInterface(uri);
         nsCOMPtr<nsIFile> file;
         rv = fileUrl->GetFile(getter_AddRefs(file));
@@ -227,17 +225,27 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
             parentStr.Assign(url);
         }
 
-        // reset parser's charset to platform's default if this is file url
-        nsCOMPtr<nsIPlatformCharset> platformCharset(do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv));
-        NS_ENSURE_SUCCESS(rv, rv);
-        nsCAutoString charset;
-        rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, charset);
-        NS_ENSURE_SUCCESS(rv, rv);
-        rv = mParser->SetEncoding(charset.get());
+        // Directory index will be always encoded in UTF-8 if this is file url
+        rv = mParser->SetEncoding("UTF-8");
         NS_ENSURE_SUCCESS(rv, rv);
 
     } else if (NS_SUCCEEDED(uri->SchemeIs("gopher", &isScheme)) && isScheme) {
         mExpectAbsLoc = PR_TRUE;
+    } else if (NS_SUCCEEDED(uri->SchemeIs("jar", &isScheme)) && isScheme) {
+        nsCAutoString path;
+        rv = uri->GetPath(path);
+        if (NS_FAILED(rv)) return rv;
+
+        // a top-level jar directory URL is of the form jar:foo.zip!/
+        // path will be of the form foo.zip!/, and its last two characters
+        // will be "!/"
+        //XXX this won't work correctly when the name of the directory being
+        //XXX displayed ends with "!", but then again, jar: URIs don't deal
+        //XXX particularly well with such directories anyway
+        if (!StringEndsWith(path, NS_LITERAL_CSTRING("!/"))) {
+            rv = uri->Resolve(NS_LITERAL_CSTRING(".."), parentStr);
+            if (NS_FAILED(rv)) return rv;
+        }
     }
     else {
         // default behavior for other protocols is to assume the channel's
@@ -250,7 +258,7 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
             path.Append('/');
             uri->SetPath(path);
         }
-        if (!path.Equals("/")) {
+        if (!path.EqualsLiteral("/")) {
             rv = uri->Resolve(NS_LITERAL_CSTRING(".."), parentStr);
             if (NS_FAILED(rv)) return rv;
         }
@@ -271,9 +279,9 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
 
     AppendASCIItoUTF16(encoding, buffer);
 
-    buffer.Append(NS_LITERAL_STRING("\"?>\n") +
-                  NS_LITERAL_STRING("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" ") +
-                  NS_LITERAL_STRING("\"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n"));
+    buffer.AppendLiteral("\"?>\n"
+                         "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" "
+                         "\"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n");
 
     // Anything but a gopher url needs to end in a /,
     // otherwise we end up linking to file:///foo/dirfile
@@ -290,8 +298,22 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
     nsXPIDLString unEscapeSpec;
     rv = mTextToSubURI->UnEscapeAndConvert(encoding, titleUri.get(),
                                            getter_Copies(unEscapeSpec));
+    // unescape may fail because
+    // 1. file URL may be encoded in platform charset for backward compatibility
+    // 2. query part may not be encoded in UTF-8 (see bug 261929)
+    // so try the platform's default if this is file url
+    if (NS_FAILED(rv) && isSchemeFile) {
+        nsCOMPtr<nsIPlatformCharset> platformCharset(do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv));
+        NS_ENSURE_SUCCESS(rv, rv);
+        nsCAutoString charset;
+        rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, charset);
+        NS_ENSURE_SUCCESS(rv, rv);
+
+        rv = mTextToSubURI->UnEscapeAndConvert(charset.get(), titleUri.get(),
+                                               getter_Copies(unEscapeSpec));
+    }
     if (NS_FAILED(rv)) return rv;
-    
+
     nsXPIDLString htmlEscSpec;
     htmlEscSpec.Adopt(nsEscapeHTML2(unEscapeSpec.get(),
                                     unEscapeSpec.Length()));
@@ -314,17 +336,15 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
 
     buffer.AppendLiteral("</title><base href=\"");    
     AppendASCIItoUTF16(baseUri, buffer);
-    buffer.AppendLiteral("\"/>\n");
-
-    buffer.Append(NS_LITERAL_STRING("<style type=\"text/css\">\n") +
-                  NS_LITERAL_STRING("img { border: 0; padding: 0 2px; vertical-align: text-bottom; }\n") +
-                  NS_LITERAL_STRING("td  { font-family: monospace; padding: 2px 3px; text-align: right; vertical-align: bottom; white-space: pre; }\n") +
-                  NS_LITERAL_STRING("td:first-child { text-align: left; padding: 2px 10px 2px 3px; }\n") +
-                  NS_LITERAL_STRING("table { border: 0; }\n") +
-                  NS_LITERAL_STRING("a.symlink { font-style: italic; }\n") +
-                  NS_LITERAL_STRING("</style>\n"));
-
-    buffer.AppendLiteral("</head>\n<body>\n<h1>");
+    buffer.AppendLiteral("\"/>\n"
+                         "<style type=\"text/css\">\n"
+                         "img { border: 0; padding: 0 2px; vertical-align: text-bottom; }\n"
+                         "td  { font-family: monospace; padding: 2px 3px; text-align: right; vertical-align: bottom; white-space: pre; }\n"
+                         "td:first-child { text-align: left; padding: 2px 10px 2px 3px; }\n"
+                         "table { border: 0; }\n"
+                         "a.symlink { font-style: italic; }\n"
+                         "</style>\n"
+                         "</head>\n<body>\n<h1>");
     
     const PRUnichar* formatHeading[] = {
         htmlEscSpec.get()
@@ -351,9 +371,9 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
         ConvertNonAsciiToNCR(parentText, strNCR);
         buffer.AppendLiteral("<tr><td colspan=\"3\"><a href=\"");
         AppendASCIItoUTF16(parentStr, buffer);
-        buffer.Append(NS_LITERAL_STRING("\">") +
-                      strNCR +
-                      NS_LITERAL_STRING("</a></td></tr>\n"));
+        buffer.AppendLiteral("\">");
+        buffer.Append(strNCR);
+        buffer.AppendLiteral("</a></td></tr>\n");
     }
 
     // Push buffer to the listener now, so the initial HTML will not
@@ -362,24 +382,26 @@ nsIndexedToHTML::OnStartRequest(nsIRequest* request, nsISupports *aContext) {
     rv = mListener->OnStartRequest(request, aContext);
     if (NS_FAILED(rv)) return rv;
 
-    rv = FormatInputStream(request, aContext, buffer);
+    // The request may have been canceled, and if that happens, we want to
+    // suppress calls to OnDataAvailable.
+    request->GetStatus(&rv);
+    if (NS_FAILED(rv)) return rv;
 
+    rv = FormatInputStream(request, aContext, buffer);
     return rv;
 }
 
 NS_IMETHODIMP
 nsIndexedToHTML::OnStopRequest(nsIRequest* request, nsISupports *aContext,
                                nsresult aStatus) {
-    nsresult rv = NS_OK;
-    nsString buffer;
-    buffer.AssignLiteral("</table><hr/></body></html>\n");
+    if (NS_SUCCEEDED(aStatus)) {
+        nsString buffer;
+        buffer.AssignLiteral("</table><hr/></body></html>\n");
 
-    rv = FormatInputStream(request, aContext, buffer);
-    if (NS_FAILED(rv)) return rv;
-
-    rv = mParser->OnStopRequest(request, aContext, aStatus);
-    if (NS_FAILED(rv)) return rv;
+        aStatus = FormatInputStream(request, aContext, buffer);
+    }
 
+    mParser->OnStopRequest(request, aContext, aStatus);
     mParser = 0;
     
     return mListener->OnStopRequest(request, aContext, aStatus);
@@ -445,7 +467,7 @@ nsIndexedToHTML::FormatInputStream(nsIRequest* aRequest, nsISupports *aContext,
                                       inputData, 0, dstLength);
     }
     else {
-      NS_ConvertUCS2toUTF8 utf8Buffer(aBuffer);
+      NS_ConvertUTF16toUTF8 utf8Buffer(aBuffer);
       rv = NS_NewCStringInputStream(getter_AddRefs(inputData), utf8Buffer);
       NS_ENSURE_SUCCESS(rv, rv);
       rv = mListener->OnDataAvailable(aRequest, aContext,
@@ -463,10 +485,6 @@ nsIndexedToHTML::OnDataAvailable(nsIRequest *aRequest,
     return mParser->OnDataAvailable(aRequest, aCtxt, aInput, aOffset, aCount);
 }
 
-// This defines the number of rows we are going to have per table
-// splitting this up makes things faster, by helping layout
-#define ROWS_PER_TABLE 250
-
 NS_IMETHODIMP
 nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
                                   nsISupports *aCtxt,
@@ -493,7 +511,7 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
         mTextToSubURI = do_GetService(NS_ITEXTTOSUBURI_CONTRACTID, &rv);
         if (NS_FAILED(rv)) return rv;
     }
-    
+
     nsXPIDLCString encoding;
     rv = mParser->GetEncoding(getter_Copies(encoding));
     if (NS_FAILED(rv)) return rv;
@@ -502,11 +520,11 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
     rv = mTextToSubURI->UnEscapeAndConvert(encoding, loc,
                                            getter_Copies(unEscapeSpec));
     if (NS_FAILED(rv)) return rv;
-  
+
     // need to escape links
     nsCAutoString escapeBuf;
 
-    NS_ConvertUCS2toUTF8 utf8UnEscapeSpec(unEscapeSpec);
+    NS_ConvertUTF16toUTF8 utf8UnEscapeSpec(unEscapeSpec);
 
     // now minimally re-escape the location...
     PRUint32 escFlags;
@@ -520,12 +538,16 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
     }
     else {
         // escape as relative
-        escFlags = esc_Forced | esc_OnlyASCII | esc_AlwaysCopy | esc_FileBaseName | esc_Colon;
+        // esc_Directory is needed for protocols which allow the same name for
+        // both a directory and a file and distinguish between the two by a
+        // trailing '/' -- without it, the trailing '/' will be escaped, and
+        // links from within that directory will be incorrect
+        escFlags = esc_Forced | esc_OnlyASCII | esc_AlwaysCopy | esc_FileBaseName | esc_Colon | esc_Directory;
     }
     NS_EscapeURL(utf8UnEscapeSpec.get(), utf8UnEscapeSpec.Length(), escFlags, escapeBuf);
-  
+
     AppendUTF8toUTF16(escapeBuf, pushBuffer);
-    
+
     pushBuffer.AppendLiteral("\"><img src=\"");
 
     switch (type) {
@@ -551,7 +573,7 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
     PRInt64 size;
     aIndex->GetSize(&size);
     
-    if (LL_NE(size, LL_INIT(0, -1)) &&
+    if (nsUint64(PRUint64(size)) != nsUint64(LL_MAXUINT) &&
         type != nsIDirIndex::TYPE_DIRECTORY &&
         type != nsIDirIndex::TYPE_SYMLINK) {
         nsAutoString  sizeString;
@@ -588,13 +610,6 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
 
     pushBuffer.AppendLiteral("</td>\n</tr>\n");
 
-    // Split this up to avoid slow layout performance with large tables
-    // - bug 85381
-    if (++mRowCount > ROWS_PER_TABLE) {
-        pushBuffer.AppendLiteral("</table>\n<table>\n");
-        mRowCount = 0;
-    }
-    
     return FormatInputStream(aRequest, aCtxt, pushBuffer);
 }
 
@@ -606,17 +621,11 @@ nsIndexedToHTML::OnInformationAvailable(nsIRequest *aRequest,
     PRUnichar* escaped = nsEscapeHTML2(PromiseFlatString(aInfo).get());
     if (!escaped)
         return NS_ERROR_OUT_OF_MEMORY;
-    pushBuffer.Append(NS_LITERAL_STRING("<tr>\n <td>"));
+    pushBuffer.AppendLiteral("<tr>\n <td>");
     pushBuffer.Append(escaped);
     nsMemory::Free(escaped);
-    pushBuffer.Append(NS_LITERAL_STRING("</td>\n <td></td>\n <td></td>\n <td></td>\n</tr>\n"));
+    pushBuffer.AppendLiteral("</td>\n <td></td>\n <td></td>\n <td></td>\n</tr>\n");
     
-    // Split this up to avoid slow layout performance with large tables
-    // - bug 85381
-    if (++mRowCount > ROWS_PER_TABLE) {
-        pushBuffer.Append(NS_LITERAL_STRING("</table>\n<table>\n"));
-        mRowCount = 0;
-    }   
     return FormatInputStream(aRequest, aCtxt, pushBuffer);
 }
 
