@@ -53,6 +53,7 @@
 #include "nsStreamConverterService.h"
 #include "nsIServiceManager.h"
 #include "nsIComponentManager.h"
+#include "nsIComponentRegistrar.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsIAtom.h"
@@ -451,12 +452,12 @@ nsStreamConverterService::FindConverter(const char *aContractID, nsCStringArray
 
         // build out the CONTRACTID.
         nsCAutoString newContractID(ContractIDPrefix);
-        newContractID.Append("?from=");
+        newContractID.AppendLiteral("?from=");
 
         nsCStringKey *predecessorKey = predecessorData->key;
         newContractID.Append(predecessorKey->GetString());
 
-        newContractID.Append("&to=");
+        newContractID.AppendLiteral("&to=");
         newContractID.Append(key->GetString());
     
         // Add this CONTRACTID to the chain.
@@ -472,7 +473,42 @@ nsStreamConverterService::FindConverter(const char *aContractID, nsCStringArray
 
 
 /////////////////////////////////////////////////////
-// nsIStreamConverter methods
+// nsIStreamConverterService methods
+NS_IMETHODIMP
+nsStreamConverterService::CanConvert(const char* aFromType,
+                                     const char* aToType,
+                                     PRBool* _retval) {
+    nsCOMPtr<nsIComponentRegistrar> reg;
+    nsresult rv = NS_GetComponentRegistrar(getter_AddRefs(reg));
+    if (NS_FAILED(rv))
+        return rv;
+
+    nsCAutoString contractID;
+    contractID.AssignLiteral(NS_ISTREAMCONVERTER_KEY "?from=");
+    contractID.Append(aFromType);
+    contractID.AppendLiteral("&to=");
+    contractID.Append(aToType);
+
+    // See if we have a direct match
+    rv = reg->IsContractIDRegistered(contractID.get(), _retval);
+    if (NS_FAILED(rv))
+        return rv;
+    if (*_retval)
+        return NS_OK;
+
+    // Otherwise try the graph.
+    rv = BuildGraph();
+    if (NS_FAILED(rv))
+        return rv;
+
+    nsCStringArray *converterChain = nsnull;
+    rv = FindConverter(contractID.get(), &converterChain);
+    *_retval = NS_SUCCEEDED(rv);
+
+    delete converterChain;
+    return NS_OK;
+}
+
 NS_IMETHODIMP
 nsStreamConverterService::Convert(nsIInputStream *aFromStream,
                                   const char *aFromType, 
@@ -482,13 +518,13 @@ nsStreamConverterService::Convert(nsIInputStream *aFromStream,
     if (!aFromStream || !aFromType || !aToType || !_retval) return NS_ERROR_NULL_POINTER;
     nsresult rv;
 
-    // first determine whether we can even handle this covnversion
+    // first determine whether we can even handle this conversion
     // build a CONTRACTID
-    nsCAutoString contractID(
-        NS_LITERAL_CSTRING(NS_ISTREAMCONVERTER_KEY "?from=") +
-        nsDependentCString(aFromType) +
-        NS_LITERAL_CSTRING("&to=") +
-        nsDependentCString(aToType));
+    nsCAutoString contractID;
+    contractID.AssignLiteral(NS_ISTREAMCONVERTER_KEY "?from=");
+    contractID.Append(aFromType);
+    contractID.AppendLiteral("&to=");
+    contractID.Append(aToType);
     const char *cContractID = contractID.get();
 
     nsCOMPtr<nsIStreamConverter> converter(do_CreateInstance(cContractID, &rv));
@@ -568,13 +604,13 @@ nsStreamConverterService::AsyncConvertData(const char *aFromType,
 
     nsresult rv;
 
-    // first determine whether we can even handle this covnversion
+    // first determine whether we can even handle this conversion
     // build a CONTRACTID
-    nsCAutoString contractID(
-        NS_LITERAL_CSTRING(NS_ISTREAMCONVERTER_KEY "?from=") +
-        nsDependentCString(aFromType) +
-        NS_LITERAL_CSTRING("&to=") +
-        nsDependentCString(aToType));
+    nsCAutoString contractID;
+    contractID.AssignLiteral(NS_ISTREAMCONVERTER_KEY "?from=");
+    contractID.Append(aFromType);
+    contractID.AppendLiteral("&to=");
+    contractID.Append(aToType);
     const char *cContractID = contractID.get();
 
     nsCOMPtr<nsIStreamConverter> listener(do_CreateInstance(cContractID, &rv));
