@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -46,11 +47,6 @@
 #include "nsIServiceManager.h"
 #include "nsUnicharUtils.h"
 
-MOZ_DECL_CTOR_COUNTER(nsEntryStack)
-MOZ_DECL_CTOR_COUNTER(nsDTDContext)
-MOZ_DECL_CTOR_COUNTER(nsTokenAllocator)
-MOZ_DECL_CTOR_COUNTER(CNodeRecycler)
- 
 /**************************************************************************************
   A few notes about how residual style handling is performed:
    
@@ -103,10 +99,10 @@ nsEntryStack::~nsEntryStack() {
 void 
 nsEntryStack::ReleaseAll(nsNodeAllocator* aNodeAllocator)
 {
-  NS_WARN_IF_FALSE(aNodeAllocator,"no allocator? - potential leak!");
+  NS_ASSERTION(aNodeAllocator,"no allocator? - potential leak!");
 
   if(aNodeAllocator) {
-    NS_WARN_IF_FALSE(mCount >= 0,"count should not be negative");
+    NS_ASSERTION(mCount >= 0,"count should not be negative");
     while(mCount > 0) {
       nsCParserNode* node=this->Pop();
       IF_FREE(node,aNodeAllocator);
@@ -171,6 +167,15 @@ void nsEntryStack::Push(nsCParserNode* aNode,
   }
 }
 
+void nsEntryStack::PushTag(eHTMLTags aTag)
+{
+  EnsureCapacityFor(mCount + 1);
+  mEntries[mCount].mTag = aTag;
+  mEntries[mCount].mParent = nsnull;
+  mEntries[mCount].mStyles = nsnull;
+  ++mCount;
+}
+
 
 /**
  * This method inserts the given node onto the front of this stack
@@ -255,24 +260,33 @@ nsCParserNode* nsEntryStack::Remove(PRInt32 anIndex,
       //now we have to tell the residual style stack where this tag
       //originated that it's no longer in use.
       PRUint32 scount = theStyleStack->mCount;
-      PRUint32 sindex = 0;
-      nsTagEntry *theStyleEntry=theStyleStack->mEntries;
-      for (sindex=scount-1;sindex>0;--sindex){            
-        if (theStyleEntry->mTag==aTag) {
-          theStyleEntry->mParent=0;  //this tells us that the style is not open at any level
+#ifdef DEBUG_mrbkap
+      NS_ASSERTION(scount != 0, "RemoveStyles has a bad style stack");
+#endif
+      nsTagEntry *theStyleEntry = theStyleStack->mEntries;
+      for (PRUint32 sindex = scount-1;; --sindex) {            
+        if (theStyleEntry->mTag == aTag) {
+          // This tells us that the style is not open at any level.
+          theStyleEntry->mParent = nsnull;
+          break;
+        }
+        if (sindex == 0) {
+#ifdef DEBUG_mrbkap
+          NS_ERROR("Couldn't find the removed style on its parent stack");
+#endif
           break;
         }
         ++theStyleEntry;
-      } //for
+      }
     }
   }
   return result;
 }
 
 /**
- * 
- * @update  harishd 04/04/99
- * @update  gess 04/21/99
+ * Pops an entry from this style stack. If the entry has a parent stack, it
+ * updates the entry so that we know not to try to remove it from the parent
+ * stack since it's no longer open.
  */
 nsCParserNode* nsEntryStack::Pop(void)
 {
@@ -283,20 +297,34 @@ nsCParserNode* nsEntryStack::Pop(void)
       result->mUseCount--;
     mEntries[mCount].mNode = 0;
     mEntries[mCount].mStyles = 0;
-    nsEntryStack* theStyleStack=mEntries[mCount].mParent;
+    nsEntryStack* theStyleStack = mEntries[mCount].mParent;
     if (theStyleStack) {
       //now we have to tell the residual style stack where this tag
       //originated that it's no longer in use.
       PRUint32 scount = theStyleStack->mCount;
-      PRUint32 sindex = 0;
-      nsTagEntry *theStyleEntry=theStyleStack->mEntries;
-      for (sindex=scount-1;sindex>0;--sindex){            
-        if (theStyleEntry->mTag==mEntries[mCount].mTag) {
-          theStyleEntry->mParent=0;  //this tells us that the style is not open at any level
+
+      // XXX If this NS_ENSURE_TRUE fails, it means that the style stack was
+      //     empty before we were removed.
+#ifdef DEBUG_mrbkap
+      NS_ASSERTION(scount != 0, "preventing a potential crash.");
+#endif
+      NS_ENSURE_TRUE(scount != 0, result);
+
+      nsTagEntry *theStyleEntry = theStyleStack->mEntries;
+      for (PRUint32 sindex = scount - 1;; --sindex) {
+        if (theStyleEntry->mTag == mEntries[mCount].mTag) {
+          // This tells us that the style is not open at any level
+          theStyleEntry->mParent = nsnull;
+          break;
+        }
+        if (sindex == 0) {
+#ifdef DEBUG_mrbkap
+          NS_ERROR("Couldn't find the removed style on its parent stack");
+#endif
           break;
         }
         ++theStyleEntry;
-      } //for
+      }
     }
   }
   return result;
@@ -421,525 +449,28 @@ void nsEntryStack::PushEntry(nsTagEntry* aEntry,
  * 
  * @update	gess 04.21.2000
  */
-nsDTDContext::nsDTDContext() : mStack(), mEntities(0){
-
+nsDTDContext::nsDTDContext() : mStack()
+{
   MOZ_COUNT_CTOR(nsDTDContext);
   mResidualStyleCount=0;
   mContextTopIndex=-1;
-  mTableStates=0;
   mTokenAllocator=0;
   mNodeAllocator=0;
-  mAllBits=0;
 
 #ifdef DEBUG
   memset(mXTags,0,sizeof(mXTags));
-  mCounters = 0;
 #endif
 } 
  
-#ifdef DEBUG
-  class CEntityDeallocator: public nsDequeFunctor{
-  public:
-    virtual void* operator()(void* anObject) {
-      CNamedEntity *theEntity=(CNamedEntity*)anObject;
-      delete theEntity;
-      return 0;
-    }
-  };
-#endif
-
 /**
  * 
  * @update	gess9/10/98
  */
-nsDTDContext::~nsDTDContext() {
+nsDTDContext::~nsDTDContext()
+{
   MOZ_COUNT_DTOR(nsDTDContext);
-
-  while(mTableStates) {
-    //pop the current state and restore it's predecessor, if any...
-    CTableState *theState=mTableStates;      
-    mTableStates=theState->mPrevious;
-    delete theState;
-  } 
-
-#ifdef DEBUG
-  CEntityDeallocator theDeallocator;
-  mEntities.ForEach(theDeallocator);
-  if(mCounters) {
-    delete [] mCounters;
-    mCounters = 0;
-  }
-#endif
-
 }
 
-#ifdef DEBUG
-
-CNamedEntity* nsDTDContext::GetEntity(const nsAString& aName)const {
-  if (aName.Length() > 2) {
-    nsAString::const_iterator start, end;
-    aName.BeginReading(start);
-    aName.EndReading(end);
-    
-    PRUnichar theChar=aName.Last();
-
-    // skip past leading and trailing quotes/etc
-    if(kQuote==*start) {
-      ++start;
-    }
-    
-    if(kSemicolon==theChar ||
-       kQuote == theChar) {
-      --end;
-    }
-    
-  
-    const nsAString& entityName = Substring(start, end);
-    
-    PRInt32 theCount=mEntities.GetSize();
-    PRInt32 theIndex=0;
-    for(theIndex=0;theIndex<theCount;++theIndex) {
-      CNamedEntity *theResult=(CNamedEntity*)mEntities.ObjectAt(theIndex);
-      if(theResult &&
-         theResult->mName.Equals(entityName,
-                                 nsCaseInsensitiveStringComparator())) {
-        return theResult;
-      }
-    }
-  }
-  return 0;
-}
-
-CNamedEntity*  nsDTDContext::RegisterEntity(const nsAString& aName,const nsAString& aValue) {
-  CNamedEntity *theEntity=GetEntity(aName);
-  if(!GetEntity(aName)){
-    theEntity=new CNamedEntity(aName,aValue);
-    mEntities.Push(theEntity);
-  }
-  return theEntity;
-}
-
-/****************************************************************
-  The abacus class is useful today for debug purposes, but it
-  will eventually serve as the implementation for css counters.
-
-  This implementation is fine for static documents, but woefully
-  inadequate for dynamic documents. (This about what happens if
-  someone inserts a new counter using the DOM? -- The other
-  numbers in that "group" should be renumbered.)
-
-  In order to be dynamic, we need a counter "group" manager that
-  is aware of layout (geometry at least) -- and that has a 
-  mechanism for notifying markers that need to be updated, along
-  with the ability to cause incremental reflow to occur in a
-  localized context (so the counters display correctly).
-
- ****************************************************************/
-
-class CAbacus {
-public:
-
-  enum eNumFormat {eUnknown,eAlpha,eDecimal,eRoman,eSpelled,eHex,eBinary,eFootnote,eUserSeries};
-
-  CAbacus(PRInt32 aDefaultValue=0,eNumFormat aFormat=eDecimal) {
-    mUserSeries=0;
-    mFormat=aFormat;
-    mCase=PR_FALSE;
-    mValue=0;
-    mUserBase=0;
-  }
-
-  ~CAbacus() {
-  }
-
-  void  SetValue(int aStartValue) {mValue=aStartValue;}
-  void  SetNumberingStyle(eNumFormat aFormat) {mFormat=aFormat;}
-  void  SetUserSeries(const char* aSeries,int aUserBase) {mUserSeries=aSeries; mUserBase=aUserBase;}
-  void  SetCase(PRBool alwaysUpper) {mCase=alwaysUpper;}
-
-  void  GetNextValueAsString(nsString& aString) {
-    GetFormattedString(mFormat,mValue++,aString,mUserSeries,0,mUserBase);
-  }
-
-  void  GetValueAsString(nsString& aString) {
-    GetFormattedString(mFormat,mValue,aString,mUserSeries,0,mUserBase);
-  }
-
-
-  /**
-   * Get a counter string in the given style for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aFormat -- format of choice
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void  GetFormattedString(eNumFormat aFormat,PRInt32 aValue, nsString& aString,const char* aCharSet, int anOffset, int aBase) {
-	  switch (aFormat) {
-		  case	eDecimal:	    DecimalString(aValue,aString);  break;
-		  case	eHex:			    HexString(aValue,aString);      break;
-		  case	eBinary:		  BinaryString(aValue,aString);   break;
-		  case	eAlpha:		    AlphaString(aValue,aString);    break;
-		  case	eSpelled:		  SpelledString(aValue,aString);  break;
-		  case	eRoman:		    RomanString(aValue,aString);    break;
-		  case	eFootnote:	  FootnoteString(aValue,aString); break;
-		  case	eUserSeries:	SeriesString(aValue,aString,aCharSet,anOffset,aBase); break;
-      default:
-        DecimalString(aValue,aString);  break;
-	  }
-  }
-
-  /**
-   * Compute a counter string in the casted-series style for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void  SeriesString(PRInt32 aValue,nsString& aString,const char* aCharSet, int offset, int base) {
-	  int	ndex=0;
-	  int	root=1;
-	  int	next=base;
-	  int	expn=1;
-
-    aString.Truncate();
-    if(aValue<0)
-      aString.Append(PRUnichar('-'));
-
-	  aValue=abs(aValue);	  // must be positive here...
-    while(next<=aValue)	{	// scale up in baseN; exceed current value.
-		  root=next;
-		  next*=base;
-		  ++expn;
-	  }
-		    
-	  while(expn--) {
-		  ndex = ((root<=aValue) && (root)) ? (aValue/root): 0;
-		  aValue%=root;
-      aString.Append(PRUnichar(aCharSet[ndex+((root>1)*offset)]));
-		  root/=base;
-    }
-  }
-
-  /**
-   * Compute a counter string in the spelled style for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void SpelledString(PRInt32 aValue,nsString& aString) {
-
-    static	char	ones[][12]=   {"zero","one ","two ","three ","four ","five ","six ","seven ","eight ","nine ","ten "};
-    static	char	teens[][12]=	{"ten ","eleven ","twelve ","thirteen ","fourteen ","fifteen ","sixteen ","seventeen ","eighteen ","nineteen "};
-    static	char	tens[][12]=   {"","ten ","twenty ","thirty ","fourty ","fifty ","sixty ","seventy ","eighty ","ninety ","hundred "};
-    static	char	bases[][20]=  {"","hundred ","thousand ","million ","billion ","trillion ","quadrillion ","quintillion ","bajillion "};
-
-    aString.Truncate();
-    if(aValue<0)
-      aString.Append(PRUnichar('-'));
-
-	  PRInt32	root=1000000000;
-	  PRInt32	expn=4;
-	  PRInt32	modu=0;
-    
-	  aValue=abs(aValue);
-    if(0<aValue) {      
-
-	    while(root && aValue) {		
-        PRInt32 temp=aValue/root;
-		    if(temp) {
-          PRInt32 theDiv=temp/100;
-          if (theDiv) {//start with hundreds part
-            aString.AppendWithConversion(ones[theDiv]);
-            aString.AppendWithConversion(bases[1]);
-		      }
-			    modu=(temp%10);
-          theDiv=(temp%100)/10;
-          if (theDiv) {
-				    if (theDiv<2) {
-              aString.AppendWithConversion(teens[modu]);
-						  modu=0;
-					  }
-            else aString.AppendWithConversion(tens[theDiv]);
-          }
-			    if (modu) 
-            aString.AppendWithConversion(ones[modu]); //do remainder
-			    aValue-=(temp*root);
-			    if (expn>1)	
-            aString.AppendWithConversion(bases[expn]);
-		    }
-		    expn--;
-		    root/=1000;
-	    }
-    }
-    else aString.AppendWithConversion(ones[0]);
-  }
-
-  /**
-   * Compute a counter string in the decimal format for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void DecimalString(PRInt32 aValue,nsString& aString) {
-    aString.Truncate();
-    aString.AppendInt(aValue);
-  }
-
-  /**
-   * Compute a counter string in binary format for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void BinaryString(PRInt32 aValue,nsString& aString) {
-	  static char	kBinarySet[]="01";
-
-	  if (aValue<0)
-		  aValue=65536-abs(aValue);
-    SeriesString(aValue,aString,kBinarySet,0,2);
-  }
-
-  /**
-   * Compute a counter string in hex format for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void HexString(PRInt32 aValue,nsString& aString) {
-    static char	kHexSet[]="0123456789ABCDEF";
-
-	  if (aValue<0)
-		  aValue=65536-abs(aValue);
-    SeriesString(aValue,aString,kHexSet,0,16);
-  }
-
-  /**
-   * Compute a counter string in the roman style for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void RomanString(PRInt32 aValue,nsString& aString) {
-	  static PRUnichar digitsA[] = { PRUnichar('i'),
-                                   PRUnichar('x'),
-                                   PRUnichar('c'),
-                                   PRUnichar('m') };
-	  static PRUnichar digitsB[] = { PRUnichar('v'),
-                                   PRUnichar('l'),
-                                   PRUnichar('d'),
-                                   PRUnichar('?') };
-
-    aString.Truncate();
-    if(aValue<0)
-      aString.Append(PRUnichar('-'));
-
-    aValue=abs(aValue);
-	  char	decStr[20];
-	  sprintf(decStr,"%d", aValue);
-
-	  int len=strlen(decStr);
-	  int romanPos=len;
-    int digitPos=0;
-    int n=0;
-
-	  for(digitPos=0;digitPos<len;++digitPos) {
-		  romanPos--;
-		  switch(decStr[digitPos]) {
-			  case	'0':	break;
-			  case	'3':	aString.Append(digitsA[romanPos]);
-			  case	'2':	aString.Append(digitsA[romanPos]);
-			  case	'1':	aString.Append(digitsA[romanPos]);
-				  break;
-			  case	'4':	aString.Append(digitsA[romanPos]);
-			  case 	'5':	case	'6':
-			  case	'7':	case 	'8':	
-          aString.Append(digitsB[romanPos]);
-					for(n=0;n<(decStr[digitPos]-'5');++n)
-            aString.Append(digitsA[romanPos]);
-				  break;
-			  case	'9':	          
-          aString.Append(digitsA[romanPos]);
-          aString.Append(digitsA[romanPos]);
-					break;
-		  }
-	  }
-  }
-
-  /**
-   * Compute a counter string in the alpha style for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void AlphaString(PRInt32 aValue,nsString& aString) {
-	  static const char kAlphaSet[]="abcdefghijklmnopqrstuvwxyz";
-	  
-	  if (0<aValue)
-      SeriesString(aValue-1,aString,kAlphaSet,-1,26);    
-  }
-
-  /**
-   * Compute a counter string in the footnote style for the given value.
-   * 
-   * @update	rickg 6June2000
-   * 
-   * @param   aValue  -- cardinal value of string
-   * @param   aString -- will hold result
-   */
-  static void FootnoteString(PRInt32 aValue,nsString& aString) {
-    static char	kFootnoteSet[]="abcdefg";
-
-	  int			seriesLen = strlen (kFootnoteSet) - 1;
-	  int			count=0;
-    int     repCount=0;
-	  int			modChar=0;
-	  
-    aString.Truncate();
-
-    aValue=abs(aValue);
-	  repCount=((aValue-1)/seriesLen);
-	  modChar=aValue-(repCount*seriesLen);
-	  
-    for(count=0;count<=repCount;++count) {
-      aString.Append(PRUnichar(kFootnoteSet[modChar]));
-    }
-  }
-
-protected:
-
-  const char* mUserSeries;
-  eNumFormat  mFormat;
-  PRBool      mCase;
-  PRInt32     mValue;
-  int         mUserBase;
-};
-
-
-/**
- * 
- * @update	gess 11May2000
- */
-void nsDTDContext::AllocateCounters(void) {
-  if(!mCounters) {
-    mCounters = new PRInt32 [NS_HTML_TAG_MAX]; //in addition to reseting, you may need to allocate.
-    ResetCounters();
-  }
-}
-
-/**
- * 
- * @update	gess 11May2000
- */
-void nsDTDContext::ResetCounters(void) {
-  if(mCounters) {
-    memset(mCounters,0,NS_HTML_TAG_MAX*sizeof(PRInt32));
-  }
-}
-
-/**********************************************************
-  @update: rickg  17May2000
-
-  Call this to handle counter attributes:
-    name="group"
-    value="nnn"
-    noincr="?"
-    format="alpha|dec|footnote|hex|roman|spelled|talk"
-
-  returns the newly incremented value for the (determined) group.
- **********************************************************/
-PRInt32 nsDTDContext::IncrementCounter(eHTMLTags aTag,nsIParserNode& aNode,nsString& aResult) {
-
-  PRInt32     result=0;
-
-  PRInt32       theIndex=0;
-  PRInt32       theNewValue=-1; //-1 is interpreted to mean "don't reset the counter sequence.
-  PRInt32       theIncrValue=1; //this may get set to 0 if we see a "noincr" key.
-  PRInt32       theCount=aNode.GetAttributeCount();
-  CNamedEntity *theEntity=0;
-
-  CAbacus::eNumFormat  theNumFormat=CAbacus::eDecimal;
-
-  for(theIndex=0;theIndex<theCount;++theIndex){
-    const nsAString& theKey=aNode.GetKeyAt(theIndex);
-    const nsAString& theValue=aNode.GetValueAt(theIndex);
-
-    if(theKey.LowerCaseEqualsLiteral("name")){
-      theEntity=GetEntity(theValue);
-      if(!theEntity) {
-        theEntity=RegisterEntity(theValue,theValue);
-        theEntity->mOrdinal=0;
-      }
-      aTag=eHTMLTag_userdefined;
-    }
-    else if(theKey.LowerCaseEqualsLiteral("noincr")){
-      theIncrValue=0;
-    }
-    else if(theKey.LowerCaseEqualsLiteral("format")){
-      nsAString::const_iterator start;
-      
-      PRUnichar theChar=*theValue.BeginReading(start);
-      if('"'==theChar)
-        theChar=*(++start);
-      switch(theChar){
-        case 'A': case 'a': theNumFormat=CAbacus::eAlpha;   break;
-        case 'B': case 'b': theNumFormat=CAbacus::eBinary;  break;
-        case 'D': case 'd': theNumFormat=CAbacus::eDecimal; break;
-        case 'H': case 'h': theNumFormat=CAbacus::eHex;     break;
-        case 'R': case 'r': theNumFormat=CAbacus::eRoman;   break;
-        case 'S': case 's': theNumFormat=CAbacus::eSpelled; break;
-        default:
-          theNumFormat=CAbacus::eDecimal;
-          break;
-      }
-      //determine numbering style
-    }
-    else if(theKey.LowerCaseEqualsLiteral("value")){
-      PRInt32 err=0;
-      theNewValue=atoi(NS_LossyConvertUCS2toASCII(theValue).get());
-      if(!err) {
-
-        theIncrValue=0;
-
-        AllocateCounters();
-        if(mCounters) {
-          mCounters[aTag]=theNewValue;
-        }
-      }
-      else theNewValue=-1;
-    }
-  }
-
-  if(theEntity && (eHTMLTag_userdefined==aTag)) {
-    result=theEntity->mOrdinal+=theIncrValue;
-  }
-  else {
-    AllocateCounters();
-    if(mCounters) {
-      result=mCounters[aTag]+=theIncrValue;
-    }
-    else result=0;
-  }
-  CAbacus::GetFormattedString(theNumFormat,result,aResult,0,0,0);
-
-  return result;
-}
-
-#endif
 
 /**
  * 
@@ -968,6 +499,17 @@ void nsDTDContext::Push(nsCParserNode* aNode,
   }
 }
 
+void nsDTDContext::PushTag(eHTMLTags aTag)
+{
+#ifdef NS_DEBUG
+  if (mStack.mCount < eMaxTags) {
+    mXTags[mStack.mCount] = aTag;
+  }
+#endif
+
+  mStack.PushTag(aTag);
+}
+
 nsTagEntry*
 nsDTDContext::PopEntry()
 {
@@ -1006,7 +548,9 @@ nsDTDContext::MoveEntries(nsDTDContext& aDest,
     while (aCount) {
       aDest.PushEntry(&mStack.mEntries[--mStack.mCount], PR_FALSE);
 #ifdef  NS_DEBUG
-      mXTags[mStack.mCount] = eHTMLTag_unknown;
+      if (mStack.mCount < eMaxTags) {
+        mXTags[mStack.mCount] = eHTMLTag_unknown;
+      }
 #endif
       --aCount;
     }
@@ -1031,9 +575,7 @@ nsCParserNode* nsDTDContext::Pop(nsEntryStack *&aChildStyleStack) {
 
 
     nsTagEntry* theEntry=mStack.EntryAt(mStack.mCount-1);
-    if(theEntry) {
-      aChildStyleStack=theEntry->mStyles;
-    }
+    aChildStyleStack=theEntry->mStyles;
 
     result=mStack.Pop();
     theEntry->mParent=0;
@@ -1485,48 +1027,6 @@ void DebugDumpContainmentRules(nsIDTD& theDTD,const char* aFilename,const char*
 }
 #endif
 
-
-/*************************************************************************
- *  The table lookup technique was adapted from the algorithm described  *
- *  by Avram Perez, Byte-wise CRC Calculations, IEEE Micro 3, 40 (1983). *
- *************************************************************************/
-
-#define POLYNOMIAL 0x04c11db7L
-
-static PRBool crc_table_initialized;
-static PRUint32 crc_table[256];
-
-static void gen_crc_table() {
- /* generate the table of CRC remainders for all possible bytes */
-  int i, j;  
-  PRUint32 crc_accum;
-  for ( i = 0;  i < 256;  i++ ) { 
-    crc_accum = ( (unsigned long) i << 24 );
-    for ( j = 0;  j < 8;  j++ ) { 
-      if ( crc_accum & 0x80000000L )
-        crc_accum = ( crc_accum << 1 ) ^ POLYNOMIAL;
-      else crc_accum = ( crc_accum << 1 ); 
-    }
-    crc_table[i] = crc_accum; 
-  }
-  return; 
-}
-
-PRUint32 AccumulateCRC(PRUint32 crc_accum, char *data_blk_ptr, int data_blk_size)  {
-  if (!crc_table_initialized) {
-    gen_crc_table();
-    crc_table_initialized = PR_TRUE;
-  }
-
- /* update the CRC on the data block one byte at a time */
-  int i, j;
-  for ( j = 0;  j < data_blk_size;  j++ ) { 
-    i = ( (int) ( crc_accum >> 24) ^ *data_blk_ptr++ ) & 0xff;
-    crc_accum = ( crc_accum << 8 ) ^ crc_table[i]; 
-  }
-  return crc_accum; 
-}
-
 /**************************************************************
   This defines the topic object used by the observer service.
   The observerService uses a list of these, 1 per topic when
@@ -1570,7 +1070,7 @@ nsObserverEntry::Notify(nsIParserNode* aNode,
       PRInt32   theCharsetSource;
       nsCAutoString      charset;
       aParser->GetDocumentCharset(charset,theCharsetSource);
-      NS_ConvertASCIItoUCS2 theCharsetValue(charset);
+      NS_ConvertASCIItoUTF16 theCharsetValue(charset);
 
       PRInt32 theAttrCount = aNode->GetAttributeCount(); 
       PRInt32 theObserversCount = theObservers->Count(); 
@@ -1611,6 +1111,13 @@ nsObserverEntry::Notify(nsIParserNode* aNode,
             if (NS_FAILED(result)) {
               break;
             }
+
+            if (result == NS_HTMLPARSER_VALID_META_CHARSET) {
+              // Inform the parser that this meta tag contained a valid
+              // charset. See bug 272815
+              aParser->SetDocumentCharset(charset, kCharsetFromMetaTag);
+              result = NS_OK;
+            }
           }
         } 
       } 
