@@ -79,6 +79,8 @@ static const PRUnichar sHTMLTagUnicodeName_br[] =
   {'b', 'r', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_button[] =
   {'b', 'u', 't', 't', 'o', 'n', '\0'};
+static const PRUnichar sHTMLTagUnicodeName_canvas[] =
+  {'c', 'a', 'n', 'v', 'a', 's', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_caption[] =
   {'c', 'a', 'p', 't', 'i', 'o', 'n', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_center[] =
@@ -111,8 +113,6 @@ static const PRUnichar sHTMLTagUnicodeName_em[] =
   {'e', 'm', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_embed[] =
   {'e', 'm', 'b', 'e', 'd', '\0'};
-static const PRUnichar sHTMLTagUnicodeName_endnote[] =
-  {'e', 'n', 'd', 'n', 'o', 't', 'e', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_fieldset[] =
   {'f', 'i', 'e', 'l', 'd', 's', 'e', 't', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_font[] =
@@ -199,6 +199,8 @@ static const PRUnichar sHTMLTagUnicodeName_p[] =
   {'p', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_param[] =
   {'p', 'a', 'r', 'a', 'm', '\0'};
+static const PRUnichar sHTMLTagUnicodeName_plaintext[] =
+  {'p', 'l', 'a', 'i', 'n', 't', 'e', 'x', 't', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_pre[] =
   {'p', 'r', 'e', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_q[] =
@@ -211,12 +213,8 @@ static const PRUnichar sHTMLTagUnicodeName_script[] =
   {'s', 'c', 'r', 'i', 'p', 't', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_select[] =
   {'s', 'e', 'l', 'e', 'c', 't', '\0'};
-static const PRUnichar sHTMLTagUnicodeName_server[] =
-  {'s', 'e', 'r', 'v', 'e', 'r', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_small[] =
   {'s', 'm', 'a', 'l', 'l', '\0'};
-static const PRUnichar sHTMLTagUnicodeName_sound[] =
-  {'s', 'o', 'u', 'n', 'd', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_spacer[] =
   {'s', 'p', 'a', 'c', 'e', 'r', '\0'};
 static const PRUnichar sHTMLTagUnicodeName_span[] =
@@ -264,7 +262,7 @@ static const PRUnichar sHTMLTagUnicodeName_xmp[] =
 
 // static array of unicode tag names
 #define HTML_TAG(_tag, _classname) sHTMLTagUnicodeName_##_tag,
-#define HTML_OTHER(_tag, _classname)
+#define HTML_OTHER(_tag)
 static const PRUnichar* const kTagUnicodeTable[] = {
 #include "nsHTMLTagList.h"
 };
@@ -388,7 +386,7 @@ nsHTMLTags::CaseSensitiveLookupTag(const PRUnichar* aTagName)
 
   PRUint32 tag = NS_PTR_TO_INT32(PL_HashTableLookupConst(gTagTable, aTagName));
 
-  return (nsHTMLTag)tag;
+  return tag == eHTMLTag_unknown ? eHTMLTag_userdefined : (nsHTMLTag)tag;
 }
 
 // static
@@ -427,34 +425,7 @@ nsHTMLTags::LookupTag(const nsAString& aTagName)
 
   buf[i] = 0;
 
-  nsHTMLTag tag = CaseSensitiveLookupTag(buf);
-
-  // hack: this can come out when rickg provides a way for the editor to ask
-  // CanContain() questions without having to first fetch the parsers
-  // internal enum values for a tag name.
-
-  // Hmm, this hack would be faster if we'd put these strings in the
-  // hash table. But maybe it's not worth it...
-
-  if (tag == eHTMLTag_unknown) {
-    // "__moz_text"
-    static const PRUnichar moz_text[] =
-      {'_', '_', 'm', 'o', 'z', '_', 't', 'e', 'x', 't', PRUnichar(0) };
-
-    // "#text"
-    static const PRUnichar text[] =
-      {'#', 't', 'e', 'x', 't', PRUnichar(0) };
-
-    if (nsCRT::strcmp(buf, moz_text) == 0) {
-      tag = eHTMLTag_text;
-    } else if (nsCRT::strcmp(buf, text) == 0) {
-      tag = eHTMLTag_text;
-    } else {
-      tag = eHTMLTag_userdefined;
-    }
-  }
-
-  return tag;
+  return CaseSensitiveLookupTag(buf);
 }
 
 // static
@@ -479,14 +450,9 @@ nsHTMLTags::GetAtom(nsHTMLTag aEnum)
   return kTagAtomTable[aEnum - 1];
 }
 
-
-#ifdef NS_DEBUG
-
-// tag table verification class.
-
-class nsTestTagTable {
-public:
-   nsTestTagTable() {
+#ifdef DEBUG
+void TestTagTable()
+{
      const PRUnichar *tag;
      nsHTMLTag id;
 
@@ -514,9 +480,6 @@ public:
      NS_ASSERTION(!tag, "found past max enum");
 
      nsHTMLTags::ReleaseTable();
-   }
-};
+}
 
-static const nsTestTagTable validateTagTable;
-
-#endif
+#endif // DEBUG
