@@ -50,8 +50,6 @@
 #include "nsCRT.h"
 #include "nsParser.h"
 
-static NS_DEFINE_CID(kCharsetAliasCID, NS_CHARSETALIAS_CID);
-
 nsReadEndCondition::nsReadEndCondition(const PRUnichar* aTerminateChars) :
   mChars(aTerminateChars), mFilter(PRUnichar(~0)) // All bits set
 {
@@ -71,19 +69,12 @@ nsReadEndCondition::nsReadEndCondition(const PRUnichar* aTerminateChars) :
   }
 }
 
-static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
-
-static const char kBadHTMLText[] ="<H3>Oops...</H3>You just tried to read a non-existent document: <BR>";
-static const char kUnorderedStringError[] = "String argument must be ordered. Don't you read API's?";
-
 #ifdef __INCREMENTAL
 const int   kBufsize=1;
 #else
 const int   kBufsize=64;
 #endif
 
-MOZ_DECL_CTOR_COUNTER(nsScanner)
-
 /**
  *  Use this constructor if you want i/o to be based on 
  *  a single string you hand in during construction.
@@ -201,7 +192,7 @@ nsresult nsScanner::SetDocumentCharset(const nsACString& aCharset , PRInt32 aSou
   if( aSource < mCharsetSource) // priority is lower the the current one , just
     return res;
 
-  nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID, &res));
+  nsCOMPtr<nsICharsetAlias> calias(do_GetService(NS_CHARSETALIAS_CONTRACTID, &res));
   NS_ASSERTION( nsnull != calias, "cannot find charset alias");
   if( NS_SUCCEEDED(res) && (nsnull != calias))
   {
@@ -224,7 +215,7 @@ nsresult nsScanner::SetDocumentCharset(const nsACString& aCharset , PRInt32 aSou
     mCharsetSource = aSource;
 
     nsCOMPtr<nsICharsetConverterManager> ccm = 
-             do_GetService(kCharsetConverterManagerCID, &res);
+             do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &res);
     if(NS_SUCCEEDED(res) && (nsnull != ccm))
     {
       nsIUnicodeDecoder * decoder = nsnull;
@@ -353,16 +344,16 @@ nsresult nsScanner::Append(const char* aBuffer, PRUint32 aLen,
     nsScannerString::Buffer* buffer = nsScannerString::AllocBuffer(unicharBufLen + 1);
     NS_ENSURE_TRUE(buffer,NS_ERROR_OUT_OF_MEMORY);
     start = unichars = buffer->DataStart();
-	  
+
     PRInt32 totalChars = 0;
     PRInt32 unicharLength = unicharBufLen;
     do {
       PRInt32 srcLength = aLen;
-		  res = mUnicodeDecoder->Convert(aBuffer, &srcLength, unichars, &unicharLength);
+      res = mUnicodeDecoder->Convert(aBuffer, &srcLength, unichars, &unicharLength);
 
       totalChars += unicharLength;
       // Continuation of failure case
-		  if(NS_FAILED(res)) {
+      if(NS_FAILED(res)) {
         // if we failed, we consume one byte, replace it with U+FFFD
         // and try the conversion again.
 
@@ -378,7 +369,7 @@ nsresult nsScanner::Append(const char* aBuffer, PRUint32 aLen,
         unichars = unichars + unicharLength;
         unicharLength = unicharBufLen - (++totalChars);
 
-			  mUnicodeDecoder->Reset();
+        mUnicodeDecoder->Reset();
 
         if(((PRUint32) (srcLength + 1)) > aLen) {
           srcLength = aLen;
@@ -389,8 +380,8 @@ nsresult nsScanner::Append(const char* aBuffer, PRUint32 aLen,
 
         aBuffer += srcLength;
         aLen -= srcLength;
-		  }
-	  } while (NS_FAILED(res) && (aLen > 0));
+      }
+    } while (NS_FAILED(res) && (aLen > 0));
 
     buffer->SetDataLength(totalChars);
     AppendToBuffer(buffer, aRequest);
@@ -420,18 +411,6 @@ nsresult nsScanner::FillBuffer(void) {
   nsresult result=NS_OK;
 
   if(!mInputStream) {
-#if 0
-    //This is DEBUG code!!!!!!  XXX DEBUG XXX
-    //If you're here, it means someone tried to load a
-    //non-existent document. So as a favor, we emit a
-    //little bit of HTML explaining the error.
-    if(0==mTotalRead) {
-      mBuffer.Append((const char*)kBadHTMLText);
-      mBuffer.Append(mFilename);
-      mTotalRead+=mBuffer.Length();
-    }
-    else 
-#endif
     result=kEOF;
   }
   else {
@@ -446,7 +425,7 @@ nsresult nsScanner::FillBuffer(void) {
       return kEOF;
     }
 
-    if((0<numread) && (0==result)) {
+    if((0<numread) && NS_SUCCEEDED(result)) {
       AppendASCIItoBuffer(buf, numread, nsnull);
     }
     mTotalRead+=numread;
@@ -455,31 +434,6 @@ nsresult nsScanner::FillBuffer(void) {
   return result;
 }
 
-/**
- *  determine if the scanner has reached EOF
- *  
- *  @update  gess 5/12/98
- *  @param   
- *  @return  0=!eof 1=eof 
- */
-nsresult nsScanner::Eof() {
-  nsresult theError=NS_OK;
-  
-  if (!mSlidingBuffer) {
-    return kEOF;
-  }
-
-  theError=FillBuffer();  
-
-  if(NS_OK==theError) {
-    if (0==(PRUint32)mSlidingBuffer->Length()) {
-      return kEOF;
-    }
-  }
-
-  return theError;
-}
-
 /**
  *  retrieve next char from scanners internal input stream
  *  
@@ -496,7 +450,7 @@ nsresult nsScanner::GetChar(PRUnichar& aChar) {
   }
 
   if (mCurrentPosition == mEndPosition) {
-    result=Eof();
+    result = FillBuffer();
   }
 
   if(NS_OK == result){
@@ -524,13 +478,13 @@ nsresult nsScanner::Peek(PRUnichar& aChar, PRUint32 aOffset) {
   }
 
   if (mCurrentPosition == mEndPosition) {
-    result=Eof();
+    result = FillBuffer();
   }
 
   if(NS_OK == result){
     if (aOffset) {
       while ((NS_OK == result) && (mCountRemaining <= aOffset)) {
-        result = Eof();
+        result = FillBuffer();
       }
 
       if (NS_OK == result) {
@@ -554,7 +508,7 @@ nsresult nsScanner::Peek(nsAString& aStr, PRInt32 aNumChars, PRInt32 aOffset)
   }
 
   if (mCurrentPosition == mEndPosition) {
-    return Eof();
+    return FillBuffer();
   }    
   
   nsScannerIterator start, end;
@@ -599,9 +553,8 @@ nsresult nsScanner::SkipWhitespace(PRInt32& aNewlinesSkipped) {
   PRUnichar theChar = 0;
   nsresult  result = Peek(theChar);
   
-  if (result == kEOF) {
-    // XXX why wouldn't Eof() return kEOF?? --darin
-    return Eof();
+  if (NS_FAILED(result)) {
+    return result;
   }
   
   nsScannerIterator current = mCurrentPosition;
@@ -634,7 +587,7 @@ nsresult nsScanner::SkipWhitespace(PRInt32& aNewlinesSkipped) {
   if (skipped) {
     SetPosition(current);
     if (current == mEndPosition) {
-      result = Eof();
+      result = FillBuffer();
     }
   }
 
@@ -770,7 +723,7 @@ nsresult nsScanner::SkipPast(nsString& aValidSet){
  *  @param   aString - receives new data from stream
  *  @return  error code
  */
-nsresult nsScanner::ReadTagIdentifier(nsString& aString) {
+nsresult nsScanner::ReadTagIdentifier(nsScannerSharedSubstring& aString) {
 
   if (!mSlidingBuffer) {
     return kEOF;
@@ -784,10 +737,11 @@ nsresult nsScanner::ReadTagIdentifier(nsString& aString) {
   current = mCurrentPosition;
   end = mEndPosition;
 
-  while(current != end) {
+  // Loop until we find an illegal character. Everything is then appended
+  // later.
+  while(current != end && !found) {
     theChar=*current;
 
-    found = PR_TRUE;
     switch(theChar) {
       case '\n':
       case '\r':
@@ -800,20 +754,20 @@ nsresult nsScanner::ReadTagIdentifier(nsString& aString) {
       case '>':
       case '/':
       case '\0':
-        found = PR_FALSE;
+        found = PR_TRUE;
         break;
       default:
         break;
     }
 
-    if(!found) {
-      // If the current character isn't a valid character for
-      // the identifier, we're done. Append the results to
-      // the string passed in.
-      AppendUnicodeTo(mCurrentPosition, current, aString);
-      break;
+    if (!found) {
+      ++current;
     }
-    ++current;
+  }
+
+  // Don't bother appending nothing.
+  if (current != mCurrentPosition) {
+    AppendUnicodeTo(mCurrentPosition, current, aString);
   }
 
   // Drop NULs on the floor since nobody really likes them.
@@ -823,7 +777,7 @@ nsresult nsScanner::ReadTagIdentifier(nsString& aString) {
 
   SetPosition(current);  
   if (current == end) {
-    result = Eof();
+    result = FillBuffer();
   }
 
   //DoErrTest(aString);
@@ -883,7 +837,7 @@ nsresult nsScanner::ReadEntityIdentifier(nsString& aString) {
   SetPosition(current);
   if (current == end) {
     AppendUnicodeTo(origin, current, aString);
-    return Eof();
+    return FillBuffer();
   }
 
   //DoErrTest(aString);
@@ -932,7 +886,7 @@ nsresult nsScanner::ReadNumber(nsString& aString,PRInt32 aBase) {
   SetPosition(current);
   if (current == end) {
     AppendUnicodeTo(origin, current, aString);
-    return Eof();
+    return FillBuffer();
   }
 
   //DoErrTest(aString);
@@ -948,8 +902,11 @@ nsresult nsScanner::ReadNumber(nsString& aString,PRInt32 aBase) {
  *  @param   addTerminal tells us whether to append terminal to aString
  *  @return  error code
  */
-nsresult nsScanner::ReadWhitespace(nsString& aString,
-                                   PRInt32& aNewlinesSkipped) {
+nsresult nsScanner::ReadWhitespace(nsScannerSharedSubstring& aString,
+                                   PRInt32& aNewlinesSkipped,
+                                   PRBool& aHaveCR) {
+
+  aHaveCR = PR_FALSE;
 
   if (!mSlidingBuffer) {
     return kEOF;
@@ -958,8 +915,8 @@ nsresult nsScanner::ReadWhitespace(nsString& aString,
   PRUnichar theChar = 0;
   nsresult  result = Peek(theChar);
   
-  if (result == kEOF) {
-    return Eof();
+  if (NS_FAILED(result)) {
+    return result;
   }
   
   nsScannerIterator origin, current, end;
@@ -969,6 +926,8 @@ nsresult nsScanner::ReadWhitespace(nsString& aString,
   current = origin;
   end = mEndPosition;
 
+  PRBool haveCR = PR_FALSE;
+
   while(!done && current != end) {
     switch(theChar) {
       case '\n':
@@ -980,11 +939,13 @@ nsresult nsScanner::ReadWhitespace(nsString& aString,
           if ((thePrevChar == '\r' && theChar == '\n') ||
               (thePrevChar == '\n' && theChar == '\r')) {
             theChar = (++current != end) ? *current : '\0'; // CRLF == LFCR => LF
+            haveCR = PR_TRUE;
           } else if (thePrevChar == '\r') {
             // Lone CR becomes CRLF; callers should know to remove extra CRs
             AppendUnicodeTo(origin, current, aString);
-            aString.Append(PRUnichar('\n'));
+            aString.writable().Append(PRUnichar('\n'));
             origin = current;
+            haveCR = PR_TRUE;
           }
         }
         break;
@@ -1003,9 +964,10 @@ nsresult nsScanner::ReadWhitespace(nsString& aString,
   SetPosition(current);
   if (current == end) {
     AppendUnicodeTo(origin, current, aString);
-    result = Eof();
+    result = FillBuffer();
   }
 
+  aHaveCR = haveCR;
   return result;
 }
 
@@ -1022,8 +984,8 @@ nsresult nsScanner::ReadWhitespace(nsScannerIterator& aStart,
   PRUnichar theChar = 0;
   nsresult  result = Peek(theChar);
   
-  if (result == kEOF) {
-    return Eof();
+  if (NS_FAILED(result)) {
+    return result;
   }
   
   nsScannerIterator origin, current, end;
@@ -1061,7 +1023,7 @@ nsresult nsScanner::ReadWhitespace(nsScannerIterator& aStart,
   if (current == end) {
     aStart = origin;
     aEnd = current;
-    result = Eof();
+    result = FillBuffer();
   }
 
   return result;
@@ -1111,7 +1073,7 @@ nsresult nsScanner::ReadWhile(nsString& aString,
   SetPosition(current);
   if (current == end) {
     AppendUnicodeTo(origin, current, aString);
-    return Eof();
+    return FillBuffer();
   }
 
   //DoErrTest(aString);
@@ -1148,8 +1110,8 @@ nsresult nsScanner::ReadUntil(nsAString& aString,
   PRUnichar         theChar=0;
   nsresult          result=Peek(theChar);
 
-  if (result == kEOF) {
-    return Eof();
+  if (NS_FAILED(result)) {
+    return result;
   }
   
   while (current != mEndPosition) {
@@ -1163,7 +1125,14 @@ nsresult nsScanner::ReadUntil(nsAString& aString,
       setcurrent = setstart;
       while (*setcurrent) {
         if (*setcurrent == theChar) {
-          goto found;
+          if(addTerminal)
+            ++current;
+          AppendUnicodeTo(origin, current, aString);
+          SetPosition(current);
+
+          //DoErrTest(aString);
+
+          return NS_OK;
         }
         ++setcurrent;
       }
@@ -1176,17 +1145,63 @@ nsresult nsScanner::ReadUntil(nsAString& aString,
   // current = mEndPosition
   SetPosition(current);
   AppendUnicodeTo(origin, current, aString);
-  return Eof();
+  return FillBuffer();
+}
 
-found:
-  if(addTerminal)
+nsresult nsScanner::ReadUntil(nsScannerSharedSubstring& aString,
+                              const nsReadEndCondition& aEndCondition,
+                              PRBool addTerminal)
+{  
+  if (!mSlidingBuffer) {
+    return kEOF;
+  }
+
+  nsScannerIterator origin, current;
+  const PRUnichar* setstart = aEndCondition.mChars;
+  const PRUnichar* setcurrent;
+
+  origin = mCurrentPosition;
+  current = origin;
+
+  PRUnichar         theChar=0;
+  nsresult          result=Peek(theChar);
+
+  if (NS_FAILED(result)) {
+    return result;
+  }
+  
+  while (current != mEndPosition) {
+    theChar = *current;
+
+    // Filter out completely wrong characters
+    // Check if all bits are in the required area
+    if(!(theChar & aEndCondition.mFilter)) {
+      // They were. Do a thorough check.
+
+      setcurrent = setstart;
+      while (*setcurrent) {
+        if (*setcurrent == theChar) {
+          if(addTerminal)
+            ++current;
+          AppendUnicodeTo(origin, current, aString);
+          SetPosition(current);
+
+          //DoErrTest(aString);
+
+          return NS_OK;
+        }
+        ++setcurrent;
+      }
+    }
+    
     ++current;
-  AppendUnicodeTo(origin, current, aString);
+  }
+
+  // If we are here, we didn't find any terminator in the string and
+  // current = mEndPosition
   SetPosition(current);
-
-  //DoErrTest(aString);
-
-  return NS_OK;
+  AppendUnicodeTo(origin, current, aString);
+  return FillBuffer();
 }
 
 nsresult nsScanner::ReadUntil(nsScannerIterator& aStart, 
@@ -1208,9 +1223,9 @@ nsresult nsScanner::ReadUntil(nsScannerIterator& aStart,
   PRUnichar         theChar=0;
   nsresult          result=Peek(theChar);
   
-  if (result == kEOF) {
+  if (NS_FAILED(result)) {
     aStart = aEnd = current;
-    return Eof();
+    return result;
   }
   
   while (current != mEndPosition) {
@@ -1221,7 +1236,13 @@ nsresult nsScanner::ReadUntil(nsScannerIterator& aStart,
       setcurrent = setstart;
       while (*setcurrent) {
         if (*setcurrent == theChar) {
-          goto found;
+          if(addTerminal)
+            ++current;
+          aStart = origin;
+          aEnd = current;
+          SetPosition(current);
+
+          return NS_OK;
         }
       ++setcurrent;
       }
@@ -1236,16 +1257,7 @@ nsresult nsScanner::ReadUntil(nsScannerIterator& aStart,
   SetPosition(current);
   aStart = origin;
   aEnd = current;
-  return Eof();
-
- found:
-  if(addTerminal)
-    ++current;
-  aStart = origin;
-  aEnd = current;
-  SetPosition(current);
-
-  return NS_OK; 
+  return FillBuffer();
 }
 
 /**
@@ -1269,8 +1281,12 @@ nsresult nsScanner::ReadUntil(nsAString& aString,
   current = origin;
 
   PRUnichar theChar;
-  Peek(theChar);
-  
+  nsresult result = Peek(theChar);
+
+  if (NS_FAILED(result)) {
+    return result;
+  }
+
   while (current != mEndPosition) {
     if (aTerminalChar == theChar) {
       if(addTerminal)
@@ -1287,7 +1303,7 @@ nsresult nsScanner::ReadUntil(nsAString& aString,
   // current = mEndPosition
   AppendUnicodeTo(origin, current, aString);
   SetPosition(current);
-  return Eof();
+  return FillBuffer();
 
 }
 
@@ -1309,12 +1325,22 @@ void nsScanner::EndReading(nsScannerIterator& aPosition)
 void nsScanner::SetPosition(nsScannerIterator& aPosition, PRBool aTerminate, PRBool aReverse)
 {
   if (mSlidingBuffer) {
+#ifdef DEBUG
+    PRUint32 origRemaining = mCountRemaining;
+#endif
+
     if (aReverse) {
       mCountRemaining += (Distance(aPosition, mCurrentPosition));
     }
     else {
       mCountRemaining -= (Distance(mCurrentPosition, aPosition));
     }
+
+    NS_ASSERTION((mCountRemaining >= origRemaining && aReverse) ||
+                 (mCountRemaining <= origRemaining && !aReverse),
+                 "Improper use of nsScanner::SetPosition. Make sure to set the"
+                 " aReverse parameter correctly");
+
     mCurrentPosition = aPosition;
     if (aTerminate && (mCurrentPosition == mEndPosition)) {
       mMarkPosition = mCurrentPosition;
