@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set sw=2 ts=2 et tw=78: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -39,13 +40,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-/**
- * MODULE NOTES:
- * @update  gess 4/8/98
- * 
- *         
- */
-
 /*
  * Set NS_VIEWSOURCE_TOKENS_PER_BLOCK to 0 to disable multi-block
  * output.  Multi-block output helps reduce the amount of bidi
@@ -84,7 +78,6 @@
 
 #include "nsIServiceManager.h"
 
-#include "COtherDTD.h"
 #include "nsElementTable.h"
 
 #include "prenv.h"  //this is here for debug reasons...
@@ -99,7 +92,6 @@ Stopwatch vsTimer;
 #endif
 
 
-static NS_DEFINE_IID(kClassIID,     NS_VIEWSOURCE_HTML_IID);
 
 // Define this to dump the viewsource stuff to a file
 //#define DUMP_TO_FILE
@@ -116,61 +108,7 @@ static NS_DEFINE_IID(kClassIID,     NS_VIEWSOURCE_HTML_IID);
 static const char kBodyId[] = "viewsource";
 static const char kBodyClassWrap[] = "wrap";
 
-/**
- *  This method gets called as part of our COM-like interfaces.
- *  Its purpose is to create an interface to parser object
- *  of some type.
- *  
- *  @update   gess 4/8/98
- *  @param    nsIID  id of object to discover
- *  @param    aInstancePtr ptr to newly discovered interface
- *  @return   NS_xxx result code
- */
-nsresult CViewSourceHTML::QueryInterface(const nsIID& aIID, void** aInstancePtr)  
-{                                                                        
-  if (NULL == aInstancePtr) {                                            
-    return NS_ERROR_NULL_POINTER;                                        
-  }                                                                      
-
-  if(aIID.Equals(NS_GET_IID(nsISupports)))    {  //do IUnknown...
-    *aInstancePtr = (nsIDTD*)(this);                                        
-  }
-  else if(aIID.Equals(NS_GET_IID(nsIDTD))) {  //do IParser base class...
-    *aInstancePtr = (nsIDTD*)(this);                                        
-  }
-  else if(aIID.Equals(kClassIID)) {  //do this class...
-    *aInstancePtr = (CViewSourceHTML*)(this);                                        
-  }                 
-  else {
-    *aInstancePtr=0;
-    return NS_NOINTERFACE;
-  }
-  NS_ADDREF_THIS();
-  return NS_OK;                                                        
-}
-
-/**
- *  This method is defined in nsIParser. It is used to 
- *  cause the COM-like construction of an nsParser.
- *  
- *  @update  gess 4/8/98
- *  @param   nsIParser** ptr to newly instantiated parser
- *  @return  NS_xxx error result
- */
-nsresult NS_NewViewSourceHTML(nsIDTD** aInstancePtrResult)
-{
-  CViewSourceHTML* it = new CViewSourceHTML();
-
-  if (it == 0) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  return it->QueryInterface(kClassIID, (void **) aInstancePtrResult);
-}
-
-
-NS_IMPL_ADDREF(CViewSourceHTML)
-NS_IMPL_RELEASE(CViewSourceHTML)
+NS_IMPL_ISUPPORTS1(CViewSourceHTML, nsIDTD)
 
 /********************************************
  ********************************************/
@@ -181,15 +119,15 @@ public:
     mIndirectString=0;
   }
   
-  void SetIndirectString(const nsAString& aString) {
+  void SetIndirectString(const nsSubstring& aString) {
     mIndirectString=&aString;
   }
 
-  virtual const nsAString& GetStringValue(void){
-    return (const nsAString&)*mIndirectString;
+  virtual const nsSubstring& GetStringValue(void){
+    return (const nsSubstring&)*mIndirectString;
   }
 
-  const nsAString* mIndirectString;
+  const nsSubstring* mIndirectString;
 };
 
 
@@ -220,19 +158,17 @@ public:
 };
 
 enum {
-  VIEW_SOURCE_START_TAG = 0,
-  VIEW_SOURCE_END_TAG = 1,
-  VIEW_SOURCE_COMMENT = 2,
-  VIEW_SOURCE_CDATA = 3,
-  VIEW_SOURCE_DOCTYPE = 4,
-  VIEW_SOURCE_PI = 5,
-  VIEW_SOURCE_ENTITY = 6,
-  VIEW_SOURCE_TEXT = 7,
-  VIEW_SOURCE_ATTRIBUTE_NAME = 8,
-  VIEW_SOURCE_ATTRIBUTE_VALUE = 9,
-  VIEW_SOURCE_SUMMARY = 10,
-  VIEW_SOURCE_POPUP = 11,
-  VIEW_SOURCE_MARKUPDECLARATION = 12
+  kStartTag = 0,
+  kEndTag,
+  kComment,
+  kCData,
+  kDoctype,
+  kPI,
+  kEntity,
+  kText,
+  kAttributeName,
+  kAttributeValue,
+  kMarkupDecl
 };
 
 static const char* const kElementClasses[] = {
@@ -246,8 +182,6 @@ static const char* const kElementClasses[] = {
   "text",
   "attribute-name",
   "attribute-value",
-  "summary",
-  "popup",
   "markupdeclaration"  
 };
 
@@ -262,8 +196,6 @@ static const char* const kBeforeText[] = {
   "",
   "",
   "=",
-  "",
-  "",
   ""
 };
 
@@ -278,8 +210,6 @@ static const char* const kAfterText[] = {
   "",
   "",
   "",
-  "",
-  "",
   ""
 };
 
@@ -295,8 +225,6 @@ static const char* const kDumpFileBeforeText[] = {
   "",
   "",
   "=",
-  "",
-  "",
   ""
 };
 
@@ -311,8 +239,6 @@ static const char* const kDumpFileAfterText[] = {
   "",
   "",
   "",
-  "",
-  "",
   ""
 };
 #endif // DUMP_TO_FILE
@@ -324,20 +250,8 @@ static const char* const kDumpFileAfterText[] = {
  *  @param   
  *  @return  
  */
-CViewSourceHTML::CViewSourceHTML() : mFilename(), mTags(), mErrors() {
-  mStartTag = VIEW_SOURCE_START_TAG;
-  mEndTag = VIEW_SOURCE_END_TAG;
-  mCommentTag = VIEW_SOURCE_COMMENT;
-  mCDATATag = VIEW_SOURCE_CDATA;
-  mMarkupDeclaration = VIEW_SOURCE_MARKUPDECLARATION;
-  mDocTypeTag = VIEW_SOURCE_DOCTYPE;
-  mPITag = VIEW_SOURCE_PI;
-  mEntityTag = VIEW_SOURCE_ENTITY;
-  mText = VIEW_SOURCE_TEXT;
-  mKey = VIEW_SOURCE_ATTRIBUTE_NAME;
-  mValue = VIEW_SOURCE_ATTRIBUTE_VALUE;
-  mSummaryTag = VIEW_SOURCE_SUMMARY;
-  mPopupTag = VIEW_SOURCE_POPUP;
+CViewSourceHTML::CViewSourceHTML()
+{
   mSyntaxHighlight = PR_FALSE;
   mWrapLongLines = PR_FALSE;
   nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
@@ -380,52 +294,6 @@ CViewSourceHTML::~CViewSourceHTML(){
   mParser=0; //just to prove we destructed...
 }
 
-/**
- * 
- * @update	gess1/8/99
- * @param 
- * @return
- */
-const nsIID& CViewSourceHTML::GetMostDerivedIID(void) const{
-  return kClassIID;
-}
-
-/**
- * Call this method if you want the DTD to construct a fresh 
- * instance of itself. 
- * @update	gess7/23/98
- * @param 
- * @return
- */
-nsresult CViewSourceHTML::CreateNewInstance(nsIDTD** aInstancePtrResult){
-  return NS_NewViewSourceHTML(aInstancePtrResult);
-}
-
-/**
- * This method is called to determine if the given DTD can parse
- * a document in a given source-type. 
- * NOTE: Parsing always assumes that the end result will involve
- *       storing the result in the main content model.
- * @update	gess6/24/98
- * @param   
- * @return  TRUE if this DTD can satisfy the request; FALSE otherwise.
- */
-NS_IMETHODIMP_(eAutoDetectResult)
-CViewSourceHTML::CanParse(CParserContext& aParserContext)
-{
-  if (eViewSource == aParserContext.mParserCommand) {
-    if (aParserContext.mDocType == ePlainText) {
-      return eValidDetect;
-    }
-
-    // We claim to parse XML... now _that_ is funny.
-    return ePrimaryDetect;
-  }
-  
-  return eUnknownDetect;
-}
-
-
 /**
   * The parser uses a code sandwich to wrap the parsing process. Before
   * the process begins, WillBuildModel() is called. Afterwards the parser
@@ -456,16 +324,11 @@ nsresult CViewSourceHTML::WillBuildModel(const CParserContext& aParserContext,
                           12, // The length of "view-source:"
                           contextFilename.Length() - 12);
     
-    mTags.Truncate();
-    mErrors.Assign(NS_LITERAL_STRING(" HTML 4.0 Strict-DTD validation (enabled); [Should use Transitional?].\n"));
-
     mDocType=aParserContext.mDocType;
     mMimeType=aParserContext.mMimeType;
     mDTDMode=aParserContext.mDTDMode;
     mParserCommand=aParserContext.mParserCommand;
     mTokenizer = aTokenizer;
-    mErrorCount=0;
-    mTagCount=0;
 
 #ifdef DUMP_TO_FILE
     if (gDumpFile) {
@@ -474,7 +337,7 @@ nsresult CViewSourceHTML::WillBuildModel(const CParserContext& aParserContext,
       fprintf(gDumpFile, "<head>\n");
       fprintf(gDumpFile, "<title>");
       fprintf(gDumpFile, "Source of: ");
-      fputs(NS_ConvertUCS2toUTF8(mFilename).get(), gDumpFile);
+      fputs(NS_ConvertUTF16toUTF8(mFilename).get(), gDumpFile);
       fprintf(gDumpFile, "</title>\n");
       fprintf(gDumpFile, "<link rel=\"stylesheet\" type=\"text/css\" href=\"resource://gre/res/viewsource.css\">\n");
       fprintf(gDumpFile, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
@@ -533,22 +396,32 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
 
       CStartToken htmlToken(NS_LITERAL_STRING("HTML"), eHTMLTag_html);
       nsCParserNode htmlNode(&htmlToken, 0/*stack token*/);
-      mSink->OpenHTML(htmlNode);
+      mSink->OpenContainer(htmlNode);
 
       CStartToken headToken(NS_LITERAL_STRING("HEAD"), eHTMLTag_head);
       nsCParserNode headNode(&headToken, 0/*stack token*/);
-      mSink->OpenHead(headNode);
+      mSink->OpenContainer(headNode);
+
+      CStartToken titleToken(NS_LITERAL_STRING("TITLE"), eHTMLTag_title);
+      nsCParserNode titleNode(&titleToken, 0/*stack token*/);
+      mSink->OpenContainer(titleNode);
 
       // Note that XUL will automatically add the prefix "Source of: "
       if (StringBeginsWith(mFilename, NS_LITERAL_STRING("data:")) &&
           mFilename.Length() > 50) {
         nsAutoString dataFilename(Substring(mFilename, 0, 50));
         dataFilename.AppendLiteral("...");
-        mSink->SetTitle(dataFilename);
+        CTextToken titleText(dataFilename);
+        nsCParserNode titleTextNode(&titleText, 0/*stack token*/);
+        mSink->AddLeaf(titleTextNode);
       } else {
-        mSink->SetTitle(mFilename);
+        CTextToken titleText(mFilename);
+        nsCParserNode titleTextNode(&titleText, 0/*stack token*/);
+        mSink->AddLeaf(titleTextNode);
       }
 
+      mSink->CloseContainer(eHTMLTag_title);
+
       if (theAllocator) {
         CStartToken* theToken=
           NS_STATIC_CAST(CStartToken*,
@@ -575,9 +448,7 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
         }
       }
 
-      CEndToken endHeadToken(eHTMLTag_head);
-      nsCParserNode endHeadNode(&endHeadToken, 0/*stack token*/);
-      result = mSink->CloseHead();
+      result = mSink->CloseContainer(eHTMLTag_head);
       if(NS_SUCCEEDED(result)) {
         mHasOpenRoot = PR_TRUE;
         if (didBlock) {
@@ -597,14 +468,14 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
 
           AddAttrToNode(bodyNode, theAllocator,
                         NS_LITERAL_STRING("id"),
-                        NS_ConvertASCIItoUCS2(kBodyId));
+                        NS_ConvertASCIItoUTF16(kBodyId));
           
           if (mWrapLongLines) {
             AddAttrToNode(bodyNode, theAllocator,
                           NS_LITERAL_STRING("class"),
-                          NS_ConvertASCIItoUCS2(kBodyClassWrap));
+                          NS_ConvertASCIItoUTF16(kBodyClassWrap));
           }
-          result = mSink->OpenBody(bodyNode);
+          result = mSink->OpenContainer(bodyNode);
           if(NS_SUCCEEDED(result)) mHasOpenBody=PR_TRUE;
         }
         
@@ -654,28 +525,6 @@ NS_IMETHODIMP CViewSourceHTML::BuildModel(nsIParser* aParser,nsITokenizer* aToke
   return result;
 }
 
-
-/**
- * Call this to display an error summary regarding the page.
- * 
- * @update	rickg 6June2000
- * @return  nsresult
- */
-nsresult  CViewSourceHTML::GenerateSummary() {
-  nsresult result=NS_OK;
-
-  if(mErrorCount && mTagCount) {
-
-    mErrors.AppendLiteral("\n\n ");
-    mErrors.AppendInt(mErrorCount);
-    mErrors.Append(NS_LITERAL_STRING(" error(s) detected -- see highlighted portions.\n"));
-
-    result=WriteTag(mSummaryTag,mErrors,0,PR_FALSE);
-  }
-
-  return result;
-}
-
 /**
  * Call this to start a new PRE block.  See bug 86355 for why this
  * makes some pages much faster.
@@ -702,7 +551,7 @@ void CViewSourceHTML::StartNewPreBlock(void){
   nsCParserStartNode startNode(theToken, theAllocator);
   AddAttrToNode(startNode, theAllocator,
                 NS_LITERAL_STRING("id"),
-                NS_ConvertASCIItoUCS2(nsPrintfCString("line%d", mLineNumber)));
+                NS_ConvertASCIItoUTF16(nsPrintfCString("line%d", mLineNumber)));
   mSink->OpenContainer(startNode);
   
 #ifdef DUMP_TO_FILE
@@ -765,17 +614,9 @@ NS_IMETHODIMP CViewSourceHTML::DidBuildModel(nsresult anErrorCode,PRBool aNotify
 #endif // DUMP_TO_FILE
 
       if(ePlainText!=mDocType) {
-        CEndToken theToken(eHTMLTag_pre);
-        nsCParserNode preNode(&theToken, 0/*stack token*/);
         mSink->CloseContainer(eHTMLTag_pre);
-        
-        CEndToken bodyToken(eHTMLTag_body);
-        nsCParserNode bodyNode(&bodyToken, 0/*stack token*/);
-        mSink->CloseBody();
-        
-        CEndToken htmlToken(eHTMLTag_html);
-        nsCParserNode htmlNode(&htmlToken, 0/*stack token*/);
-        mSink->CloseHTML();
+        mSink->CloseContainer(eHTMLTag_body);
+        mSink->CloseContainer(eHTMLTag_html);
       }
       result = mSink->DidBuildModel();
     }
@@ -813,12 +654,6 @@ CViewSourceHTML::GetType() {
   return NS_IPARSER_FLAG_HTML;
 }
 
-NS_IMETHODIMP 
-CViewSourceHTML::CollectSkippedContent(PRInt32 aTag, nsAString& aContent, PRInt32 &aLineNo)
-{
-  return NS_OK;
-}
-
 /**
  * 
  * @update	gess5/18/98
@@ -872,51 +707,6 @@ PRBool CViewSourceHTML::CanContain(PRInt32 aParent,PRInt32 aChild) const{
   return result;
 }
 
-/**
- * Give rest of world access to our tag enums, so that CanContain(), etc,
- * become useful.
- */
-NS_IMETHODIMP
-CViewSourceHTML::StringTagToIntTag(const nsAString &aTag,
-                                   PRInt32* aIntTag) const
-{
-  *aIntTag = nsHTMLTags::LookupTag(aTag);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP_(const PRUnichar *)
-CViewSourceHTML::IntTagToStringTag(PRInt32 aIntTag) const
-{
-  const PRUnichar *str_ptr = nsHTMLTags::GetStringValue((nsHTMLTag)aIntTag);
-
-  NS_ASSERTION(str_ptr, "Bad tag enum passed to COtherDTD::IntTagToStringTag()"
-               "!!");
-
-  return str_ptr;
-}
-
-NS_IMETHODIMP_(nsIAtom *)
-CViewSourceHTML::IntTagToAtom(PRInt32 aIntTag) const
-{
-  nsIAtom *atom = nsHTMLTags::GetAtom((nsHTMLTag)aIntTag);
-
-  NS_ASSERTION(atom, "Bad tag enum passed to COtherDTD::IntTagToAtom()"
-               "!!");
-
-  return atom;
-}
-
-PRBool CViewSourceHTML::IsBlockElement(PRInt32 aTagID,PRInt32 aParentID) const {
-  PRBool result=PR_FALSE;
-  return result;
-}
-
-PRBool CViewSourceHTML::IsInlineElement(PRInt32 aTagID,PRInt32 aParentID) const {
-  PRBool result=PR_FALSE;
-  return result;
-}
-
 /**
  *  This method gets called to determine whether a given 
  *  tag is itself a container
@@ -954,17 +744,17 @@ nsresult CViewSourceHTML::WriteAttributes(PRInt32 attrCount, PRBool aOwnerInErro
           theContext.mTokenNode.AddAttribute(theToken);  //and add it to the node.
 
           CAttributeToken* theAttrToken = (CAttributeToken*)theToken;
-          const nsAString& theKey = theAttrToken->GetKey();
+          const nsSubstring& theKey = theAttrToken->GetKey();
           
           // The attribute is only in error if its owner is NOT in error.
           const PRBool attributeInError =
             !aOwnerInError && theAttrToken->IsInError();
 
-          result = WriteTag(mKey,theKey,0,attributeInError);
-          const nsAString& theValue = theAttrToken->GetValue();
+          result = WriteTag(kAttributeName,theKey,0,attributeInError);
+          const nsSubstring& theValue = theAttrToken->GetValue();
 
           if(!theValue.IsEmpty() || theAttrToken->mHasEqualWithoutValue){
-            result = WriteTag(mValue,theValue,0,attributeInError);
+            result = WriteTag(kAttributeValue,theValue,0,attributeInError);
           }
         } 
       }
@@ -982,7 +772,7 @@ nsresult CViewSourceHTML::WriteAttributes(PRInt32 attrCount, PRBool aOwnerInErro
  *  @param   
  *  @return  result status
  */
-nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAString & aText,PRInt32 attrCount,PRBool aTagInError) {
+nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsSubstring & aText,PRInt32 attrCount,PRBool aTagInError) {
   nsresult result=NS_OK;
 
   // adjust line number to what it will be after we finish writing this tag
@@ -1030,7 +820,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAString & aText,PRIn
     fprintf(gDumpFile, kDumpFileBeforeText[aTagType]);
 #endif // DUMP_TO_FILE
   
-  if (mSyntaxHighlight && aTagType != mText) {
+  if (mSyntaxHighlight && aTagType != kText) {
     CStartToken* theTagToken=
       NS_STATIC_CAST(CStartToken*,
                      theAllocator->CreateTokenOfType(eToken_start,
@@ -1039,7 +829,7 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAString & aText,PRIn
     theContext.mStartNode.Init(theTagToken, theAllocator);
     AddAttrToNode(theContext.mStartNode, theAllocator,
                   NS_LITERAL_STRING("class"),
-                  NS_ConvertASCIItoUCS2(kElementClasses[aTagType]));
+                  NS_ConvertASCIItoUTF16(kElementClasses[aTagType]));
     mSink->OpenContainer(theContext.mStartNode);  //emit <starttag>...
 #ifdef DUMP_TO_FILE
     if (gDumpFile) {
@@ -1058,11 +848,11 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAString & aText,PRIn
   mSink->AddLeaf(theNode);
 #ifdef DUMP_TO_FILE
   if (gDumpFile) {
-    fputs(NS_ConvertUCS2toUTF8(aText).get(), gDumpFile);
+    fputs(NS_ConvertUTF16toUTF8(aText).get(), gDumpFile);
   }
 #endif // DUMP_TO_FILE
 
-  if (mSyntaxHighlight && aTagType != mText) {
+  if (mSyntaxHighlight && aTagType != kText) {
     theContext.mStartNode.ReleaseAll(); 
     CEndToken theEndToken(eHTMLTag_span);
     theContext.mEndNode.Init(&theEndToken, 0/*stack token*/);
@@ -1112,7 +902,8 @@ nsresult CViewSourceHTML::WriteTag(PRInt32 aTagType,const nsAString & aText,PRIn
  *  @param   aToken -- token object to be put into content model
  *  @return  0 if all is well; non-zero is an error
  */
-NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
+NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser)
+{
   nsresult        result=NS_OK;
   CHTMLToken*     theToken= (CHTMLToken*)(aToken);
   eHTMLTokenTypes theType= (eHTMLTokenTypes)theToken->GetTokenType();
@@ -1123,17 +914,12 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
   CSharedVSContext& theContext=CSharedVSContext::GetSharedContext();
   theContext.mTokenNode.Init(theToken, mTokenizer->GetTokenAllocator());
 
-  eHTMLTags theParent=(mTags.Length()) ? (eHTMLTags)mTags.Last() : eHTMLTag_unknown;
-  eHTMLTags theChild=(eHTMLTags)aToken->GetTypeID();
-  
   switch(theType) {
     
     case eToken_start:
       {
-        ++mTagCount;
-
-        const nsAString& startValue = aToken->GetStringValue();
-        result=WriteTag(mStartTag,startValue,aToken->GetAttributeCount(),aToken->IsInError());
+        const nsSubstring& startValue = aToken->GetStringValue();
+        result = WriteTag(kStartTag,startValue,aToken->GetAttributeCount(),aToken->IsInError());
 
         if((ePlainText!=mDocType) && mParser && (NS_OK==result)) {
           result = mSink->NotifyTagObservers(&theContext.mTokenNode);
@@ -1143,12 +929,8 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
 
     case eToken_end:
       {
-        if(theParent==theChild) {
-          mTags.Truncate(mTags.Length()-1);
-        }
-
-        const nsAString& endValue = aToken->GetStringValue();
-        result=WriteTag(mEndTag,endValue,aToken->GetAttributeCount(),aToken->IsInError());
+        const nsSubstring& endValue = aToken->GetStringValue();
+        result = WriteTag(kEndTag,endValue,aToken->GetAttributeCount(),aToken->IsInError());
       }
       break;
 
@@ -1160,7 +942,7 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
         if (!aToken->IsInError()) {
           theStr.AppendLiteral(">");
         }
-        result=WriteTag(mCDATATag,theStr,0,aToken->IsInError());
+        result=WriteTag(kCData,theStr,0,aToken->IsInError());
       }
       break;
 
@@ -1172,7 +954,7 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
         if (!aToken->IsInError()) {
           theStr.AppendLiteral(">");
         }
-        result=WriteTag(mMarkupDeclaration,theStr,0,aToken->IsInError());
+        result=WriteTag(kMarkupDecl,theStr,0,aToken->IsInError());
       }
       break;
 
@@ -1180,32 +962,33 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
       {
         nsAutoString theStr;
         aToken->AppendSourceTo(theStr);
-        result=WriteTag(mCommentTag,theStr,0,aToken->IsInError());
+        result=WriteTag(kComment,theStr,0,aToken->IsInError());
       }
       break;
 
     case eToken_doctypeDecl:
       {
-        const nsAString& doctypeValue = aToken->GetStringValue();
-        result=WriteTag(mDocTypeTag,doctypeValue,0,aToken->IsInError());
+        const nsSubstring& doctypeValue = aToken->GetStringValue();
+        result=WriteTag(kDoctype,doctypeValue,0,aToken->IsInError());
       }
       break;
 
     case eToken_newline:
       {
-        const nsAString& newlineValue = aToken->GetStringValue();
-        result=WriteTag(mText,newlineValue,0,PR_FALSE);
+        const nsSubstring& newlineValue = aToken->GetStringValue();
+        result=WriteTag(kText,newlineValue,0,PR_FALSE);
         ++mTokenCount;
         if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
-            mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK)
+            mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK) {
           StartNewPreBlock();
+        }
       }
       break;
 
     case eToken_whitespace:
       {
-        const nsAString& wsValue = aToken->GetStringValue();
-        result=WriteTag(mText,wsValue,0,PR_FALSE);
+        const nsSubstring& wsValue = aToken->GetStringValue();
+        result=WriteTag(kText,wsValue,0,PR_FALSE);
         ++mTokenCount;
         if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
             mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK &&
@@ -1219,8 +1002,8 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
 
     case eToken_text:
       {
-        const nsAString& str = aToken->GetStringValue();         
-        result=WriteTag(mText,str,aToken->GetAttributeCount(),aToken->IsInError());
+        const nsSubstring& str = aToken->GetStringValue();         
+        result=WriteTag(kText,str,aToken->GetAttributeCount(),aToken->IsInError());
         ++mTokenCount;
         if (NS_VIEWSOURCE_TOKENS_PER_BLOCK > 0 &&
             mTokenCount > NS_VIEWSOURCE_TOKENS_PER_BLOCK && !str.IsEmpty()) {
@@ -1233,11 +1016,12 @@ NS_IMETHODIMP CViewSourceHTML::HandleToken(CToken* aToken,nsIParser* aParser) {
       break;
 
     case eToken_entity:
-      result=WriteTag(mEntityTag,aToken->GetStringValue(),0,aToken->IsInError());
+      result=WriteTag(kEntity,aToken->GetStringValue(),0,aToken->IsInError());
       break;
 
     case eToken_instruction:
-      result=WriteTag(mPITag,aToken->GetStringValue(),0,aToken->IsInError());
+      result=WriteTag(kPI,aToken->GetStringValue(),0,aToken->IsInError());
+      break;
 
     default:
       result=NS_OK;
