@@ -54,6 +54,7 @@
 #include "nsXPIDLString.h"
 #include "nsEscape.h"
 #include "nsIURL.h"
+#include "nsNativeCharsetUtils.h"
 
 #include "prprf.h"
 
@@ -87,16 +88,13 @@
 // Interfaces Needed
 #include "nsIDocShell.h"
 #include "nsIWebBrowserChrome.h"
-
-#include "nsIScriptGlobalObject.h"
-#include "nsIScriptContext.h"
 #include "nsIBaseWindow.h"
 #include "nsIDialogParamBlock.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIWindowWatcher.h"
 #include "jsapi.h"
 #include "nsIJSContextStack.h"
-
+#include "nsEmbedCID.h"
 
 #if defined(XP_MAC) || defined(XP_MACOSX)
 #define OLD_REGISTRY_FILE_NAME "Netscape Registry"
@@ -134,7 +132,7 @@
 
 const char* kDefaultOpenWindowParams = "centerscreen,chrome,modal,titlebar";
 
-const char* kBrandBundleURL = "chrome://global/locale/brand.properties";
+const char* kBrandBundleURL = "chrome://branding/locale/brand.properties";
 const char* kMigrationBundleURL = "chrome://communicator/locale/profile/migration.properties";
 
 // we want everyone to have the debugging info to the console for now
@@ -414,7 +412,7 @@ nsProfile::StartupWithArgs(nsICmdLineService *cmdLineArgs, PRBool canInteract)
         printf(" install new UILocaleName: %s\n", mUILocaleName.get());
 #endif
         rv = chromeRegistry->SelectLocaleForProfile(mUILocaleName,
-                                          NS_ConvertUTF8toUCS2(fileStr).get());
+                                          NS_ConvertUTF8toUTF16(fileStr).get());
         if (NS_FAILED(rv)) return rv;
     }
 
@@ -423,7 +421,7 @@ nsProfile::StartupWithArgs(nsICmdLineService *cmdLineArgs, PRBool canInteract)
         printf(" install new mContentLocaleName: %s\n", mContentLocaleName.get());
 #endif
         rv = chromeRegistry->SelectLocaleForProfile(mContentLocaleName,
-                                          NS_ConvertUTF8toUCS2(fileStr).get());
+                                          NS_ConvertUTF8toUTF16(fileStr).get());
         if (NS_FAILED(rv)) return rv;
     }
 
@@ -446,19 +444,19 @@ nsresult
 nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
 {
     nsresult rv;
-    nsCOMPtr<nsIPrefBranch> prefBranch;
     nsCOMPtr<nsIURI> profileURL;
     PRInt32 numProfiles=0;
   
-    nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-    rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
-    if (NS_FAILED(rv)) return rv;
-    
     GetProfileCount(&numProfiles);
 
     if (profileURLStr.IsEmpty())
     {
+        nsCOMPtr<nsIPrefBranch> prefBranch;
+        nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+        if (NS_FAILED(rv)) return rv;
+        rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+        if (NS_FAILED(rv)) return rv;
+    
         // If this flag is TRUE, it makes the multiple profile case
         // just like the single profile case - the profile will be
         // set to that returned by GetCurrentProfile(). It will prevent
@@ -505,11 +503,11 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
                 // If the profile is locked, we need the UI
                 nsCOMPtr<nsILocalFile> localFile(do_QueryInterface(curProfileDir));
                 nsProfileLock tempLock;
-                rv = tempLock.Lock(localFile);
+                rv = tempLock.Lock(localFile, nsnull);
                 if (NS_FAILED(rv))
                     profileURLStr = PROFILE_MANAGER_URL;
 #endif
-            } 
+            }
         }
         else
             profileURLStr = PROFILE_SELECTION_URL;
@@ -554,44 +552,6 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
         if (NS_FAILED(rv)) return rv;
     }
 
-    nsCOMPtr<nsICategoryManager> catman = 
-             do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
-
-    if(NS_SUCCEEDED(rv) && catman) 
-    {
-        nsCOMPtr<nsISimpleEnumerator> enumItem;
-        rv = catman->EnumerateCategory(NS_PROFILE_STARTUP_CATEGORY, getter_AddRefs(enumItem));
-        if(NS_SUCCEEDED(rv) && enumItem) 
-        {
-           while (PR_TRUE) 
-           {
-               nsCOMPtr<nsISupportsCString> contractid;
-
-               rv = enumItem->GetNext(getter_AddRefs(contractid));
-               if (NS_FAILED(rv) || !contractid) break;
-
-               nsCAutoString contractidString;
-               contractid->GetData(contractidString);
-        
-               nsCOMPtr <nsIProfileStartupListener> listener = do_CreateInstance(contractidString.get(), &rv);
-        
-               if (listener) 
-                   listener->OnProfileStartup(currentProfileStr);
-           }
-        }
-    }
-
-    PRBool prefs_converted = PR_FALSE;
-    (void)prefBranch->GetBoolPref("prefs.converted-to-utf8", &prefs_converted);
-
-    if (!prefs_converted) 
-    {
-        nsCOMPtr <nsIPrefConverter> pPrefConverter = do_CreateInstance(kPrefConverterCID, &rv);
-        if (!pPrefConverter) return NS_ERROR_FAILURE;
-        rv = pPrefConverter->ConvertPrefsToUTF8();
-        if (NS_FAILED(rv)) return rv;
-    }
-    
     return NS_OK;
 }
 
@@ -647,7 +607,7 @@ nsProfile::ConfirmAutoMigration(PRBool canInteract, PRBool *confirmed)
     rv = migrationBundle->GetStringFromName(NS_LITERAL_STRING("manage").get(), getter_Copies(button1Title));
     if (NS_FAILED(rv)) return rv;
     
-    nsCOMPtr<nsIPromptService> promptService(do_GetService("@mozilla.org/embedcomp/prompt-service;1", &rv));
+    nsCOMPtr<nsIPromptService> promptService(do_GetService(NS_PROMPTSERVICE_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
     PRInt32 buttonPressed;
     rv = promptService->ConfirmEx(nsnull, dialogTitle.get(), msgString.get(),
@@ -737,19 +697,8 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
         if (cmdResult) {
             foundProfileCommandArg = PR_TRUE;
             nsAutoString currProfileName; 
-            if (nsCRT::IsAscii(cmdResult))  {
-                currProfileName.AssignWithConversion(cmdResult);
-            }
-            else {
-                // get a platform charset
-                nsCAutoString charSet;
-                rv = GetPlatformCharset(charSet);
-                NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get a platform charset");
-
-                // convert the profile name to Unicode
-                rv = ConvertStringToUnicode(charSet, cmdResult, currProfileName);
-                NS_ASSERTION(NS_SUCCEEDED(rv), "failed to convert ProfileName to unicode");
-            }
+            rv = NS_CopyNativeToUnicode(cmdResult, currProfileName); 
+            NS_ASSERTION(NS_SUCCEEDED(rv), "failed to convert ProfileName to unicode");
  
 #ifdef DEBUG_profile
             printf("ProfileName : %s\n", (const char*)cmdResult);
@@ -806,24 +755,16 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
             foundProfileCommandArg = PR_TRUE;
             nsAutoString currProfileName; 
  
-            if (nsCRT::IsAscii(cmdResult))  {
-                currProfileName.AssignWithConversion(strtok(cmdResult.BeginWriting(), " "));
-            }
-            else {
-                // get a platform charset
-                nsCAutoString charSet;
-                rv = GetPlatformCharset(charSet);
-                NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get a platform charset");
+            char *tmpStr;
+            rv = NS_CopyNativeToUnicode(
+                 nsDependentCString(nsCRT::strtok(cmdResult.BeginWriting(), " ", &tmpStr)),
+                                    currProfileName);
+            NS_ASSERTION(NS_SUCCEEDED(rv), "failed to convert ProfileName to unicode");
 
-                // convert the profile name to Unicode
-                nsCAutoString profileName(strtok(cmdResult.BeginWriting(), " "));
-                rv = ConvertStringToUnicode(charSet, profileName.get(), currProfileName);
-                NS_ASSERTION(NS_SUCCEEDED(rv), "failed to convert ProfileName to unicode");
-            }
-            nsAutoString currProfileDirString; currProfileDirString.AssignWithConversion(strtok(NULL, " "));
-        
-            if (!currProfileDirString.IsEmpty()) {
-                rv = NS_NewLocalFile(currProfileDirString, PR_TRUE, getter_AddRefs(currProfileDir));
+            char *currProfileDirString = nsCRT::strtok(tmpStr, " ", &tmpStr); 
+            if (currProfileDirString && *currProfileDirString) {
+                rv = NS_NewNativeLocalFile(nsDependentCString(currProfileDirString), 
+                     PR_TRUE, getter_AddRefs(currProfileDir));
                 NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
             }
             else {
@@ -847,8 +788,11 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
                 // which assume prefs.js exists after -CreateProfile.
                 nsCOMPtr<nsIFile> newProfileDir;
                 GetProfileDir(currProfileName.get(), getter_AddRefs(newProfileDir));
-                if (newProfileDir) 
-                  gDirServiceProvider->SetProfileDir(newProfileDir);
+                if (newProfileDir) {
+                  nsCOMPtr<nsIFile> localDir;
+                  GetLocalProfileDir(currProfileName.get(), getter_AddRefs(localDir));
+                  gDirServiceProvider->SetProfileDir(newProfileDir, localDir);
+                }
                 rv = LoadNewProfilePrefs();
                 gProfileDataAccess->mProfileDataChanged = PR_TRUE;
                 gProfileDataAccess->UpdateRegistry(nsnull);
@@ -1142,7 +1086,10 @@ nsProfile::GetCurrentProfile(PRUnichar **profileName)
     NS_ENSURE_ARG_POINTER(profileName);
     *profileName = nsnull;
 
-    gProfileDataAccess->GetCurrentProfile(profileName);
+    if (!mCurrentProfileName.IsEmpty()) 
+      *profileName = ToNewUnicode(mCurrentProfileName);
+    else 
+      gProfileDataAccess->GetCurrentProfile(profileName);
     return (*profileName == nsnull) ? NS_ERROR_FAILURE : NS_OK;
 }
 
@@ -1181,7 +1128,7 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
     nsProfileLock localLock;
     nsCOMPtr<nsILocalFile> localProfileDir(do_QueryInterface(profileDir, &rv));
     if (NS_FAILED(rv)) return rv;
-    rv = localLock.Lock(localProfileDir);
+    rv = localLock.Lock(localProfileDir, nsnull);
     if (NS_FAILED(rv))
     {
         NS_ERROR("Could not get profile directory lock.");
@@ -1245,7 +1192,9 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
     // Do the profile switch
     localLock.Unlock(); // gDirServiceProvider will get and hold its own lock
 #endif
-    gDirServiceProvider->SetProfileDir(profileDir);  
+    nsCOMPtr<nsIFile> localDir;
+    GetLocalProfileDir(aCurrentProfile, getter_AddRefs(localDir));
+    gDirServiceProvider->SetProfileDir(profileDir, localDir);
     mCurrentProfileName.Assign(aCurrentProfile);    
     gProfileDataAccess->SetCurrentProfile(aCurrentProfile);
     gProfileDataAccess->mProfileDataChanged = PR_TRUE;
@@ -1297,6 +1246,51 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
     if (mProfileChangeFailed)
       return NS_ERROR_ABORT;
 
+    nsCOMPtr<nsICategoryManager> catman =
+             do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
+
+    if(NS_SUCCEEDED(rv) && catman)
+    {
+        nsCOMPtr<nsISimpleEnumerator> enumItem;
+        rv = catman->EnumerateCategory(NS_PROFILE_STARTUP_CATEGORY, getter_AddRefs(enumItem));
+        if(NS_SUCCEEDED(rv) && enumItem)
+        {
+           while (PR_TRUE)
+           {
+               nsCOMPtr<nsISupportsCString> contractid;
+
+               rv = enumItem->GetNext(getter_AddRefs(contractid));
+               if (NS_FAILED(rv) || !contractid) break;
+
+               nsCAutoString contractidString;
+               contractid->GetData(contractidString);
+        
+               nsCOMPtr <nsIProfileStartupListener> listener = do_GetService(contractidString.get(), &rv);
+        
+               if (listener)
+                   listener->OnProfileStartup(aCurrentProfile);
+           }
+        }
+    }
+
+    nsCOMPtr<nsIPrefBranch> prefBranch;
+  
+    nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+    if (NS_FAILED(rv)) return rv;
+    rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+    if (NS_FAILED(rv)) return rv;
+
+    PRBool prefs_converted = PR_FALSE;
+    (void)prefBranch->GetBoolPref("prefs.converted-to-utf8", &prefs_converted);
+
+    if (!prefs_converted)
+    {
+        nsCOMPtr <nsIPrefConverter> pPrefConverter = do_GetService(kPrefConverterCID, &rv);
+        if (!pPrefConverter) return NS_ERROR_FAILURE;
+        rv = pPrefConverter->ConvertPrefsToUTF8();
+        if (NS_FAILED(rv)) return rv;
+    }
+    
     return NS_OK;
 }
 
@@ -1602,10 +1596,10 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
     {
       printf("ProfileManager : CreateNewProfileWithLocales\n");
 
-      printf("Profile Name: %s\n", NS_LossyConvertUCS2toASCII(profileName).get());
+      printf("Profile Name: %s\n", NS_LossyConvertUTF16toASCII(profileName).get());
 
       if (nativeProfileDir)
-        printf("Profile Dir: %s\n", NS_LossyConvertUCS2toASCII(nativeProfileDir).get());
+        printf("Profile Dir: %s\n", NS_LossyConvertUTF16toASCII(nativeProfileDir).get());
     }
 #endif
 
@@ -1678,8 +1672,9 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
         do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv)) {
 
-        nsCAutoString uiLocale; uiLocale.AssignWithConversion(aUILocale);
-        nsCAutoString contentLocale; contentLocale.AssignWithConversion(aContentLocale);
+        nsCAutoString uiLocale, contentLocale;
+        LossyCopyUTF16toASCII(aUILocale, uiLocale);
+        LossyCopyUTF16toASCII(aContentLocale, contentLocale);
 
         // When aUILocale == null or aContentLocale == null, set those
         // from default values which are from default or from command
@@ -1696,7 +1691,7 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
         // is done in nsAppRunner.cpp::InstallGlobalLocale()
 
         nsCOMPtr<nsIChromeRegistrySea> packageRegistry = do_QueryInterface(chromeRegistry);
-        if ((!aUILocale || !aUILocale[0]) && packageRegistry) {
+        if (uiLocale.IsEmpty() && packageRegistry) {
             nsCAutoString currentUILocaleName;
             rv = packageRegistry->GetSelectedLocale(NS_LITERAL_CSTRING("global"),
                                                     currentUILocaleName);
@@ -1705,7 +1700,7 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
             }
         }
 
-        if (!aContentLocale || !aContentLocale[0]) {
+        if (contentLocale.IsEmpty()) {
             nsCAutoString currentContentLocaleName;
             rv = packageRegistry->GetSelectedLocale(NS_LITERAL_CSTRING("global-region"),
                                                     currentContentLocaleName);
@@ -1729,7 +1724,7 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
 
         if (!uiLocale.IsEmpty()) {
             rv = chromeRegistry->SelectLocaleForProfile(uiLocale, 
-                                                        NS_ConvertUTF8toUCS2(fileStr).get());
+                                                        NS_ConvertUTF8toUTF16(fileStr).get());
             // Remember which profile has been created with the UILocale
             // didn't use gProfileDataAccess because just needed one time
             if (NS_SUCCEEDED(rv)) {
@@ -1743,7 +1738,7 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
         rv = packageRegistry->GetSelectedSkin(NS_LITERAL_CSTRING("global"),currentSkinName);
         if (!currentSkinName.IsEmpty()) {
             rv = chromeRegistry->SelectSkinForProfile(currentSkinName,
-                                         NS_ConvertUTF8toUCS2(fileStr).get());
+                                         NS_ConvertUTF8toUTF16(fileStr).get());
         }
 
         if (!contentLocale.IsEmpty()) {
@@ -1765,7 +1760,7 @@ nsProfile::CreateNewProfileWithLocales(const PRUnichar* profileName,
             }
 
             rv = chromeRegistry->SelectLocaleForProfile(contentLocale, 
-                                                        NS_ConvertUTF8toUCS2(fileStr).get());
+                                                        NS_ConvertUTF8toUTF16(fileStr).get());
             // Remember which profile has been created with the UILocale
             // didn't use gProfileDataAccess because just needed one time
             if (NS_SUCCEEDED(rv)) {
@@ -1819,10 +1814,10 @@ nsProfile::RenameProfile(const PRUnichar* oldName, const PRUnichar* newName)
       printf("ProfileManager : Renaming profile\n");
 
       nsCAutoString temp1; temp1.AssignWithConversion(oldName);
-      printf("Old name:  %s\n", NS_LossyConvertUCS2toASCII(oldName).get());
+      printf("Old name:  %s\n", NS_LossyConvertUTF16toASCII(oldName).get());
 
       nsCAutoString temp2; temp2.AssignWithConversion(newName);
-      printf("New name:  %s\n", NS_LossyConvertUCS2toASCII(newName).get());
+      printf("New name:  %s\n", NS_LossyConvertUTF16toASCII(newName).get());
     }
 #endif
 
@@ -1838,12 +1833,24 @@ nsProfile::RenameProfile(const PRUnichar* oldName, const PRUnichar* newName)
         return NS_ERROR_FAILURE;
     }
 
+    PRBool renamedIsCurrent = mCurrentProfileName.Equals(oldName);
+
     // Copy reg keys
     rv = CopyRegKey(oldName, newName);
     if (NS_FAILED(rv)) return rv;
      
     // Delete old profile entry
     rv = DeleteProfile(oldName, PR_FALSE /* don't delete files */);
+
+    // If the profile being renamed is the current profile,
+    // change the current profile name.
+    if (renamedIsCurrent) {
+        gProfileDataAccess->SetCurrentProfile(newName);
+        gProfileDataAccess->mForgetProfileCalled = PR_FALSE;
+        mCurrentProfileName.Assign(newName);    
+        mCurrentProfileAvailable = PR_TRUE;
+    }
+
     if (NS_FAILED(rv)) return rv;
      
 	/* note, we do not rename the directory on disk to the new name
@@ -1861,9 +1868,6 @@ nsProfile::RenameProfile(const PRUnichar* oldName, const PRUnichar* newName)
 	 *
 	 * bad things would happen if we tried to rename the directory
 	 */
-    
-    rv = ForgetCurrentProfile();
-    if (NS_FAILED(rv)) return rv;
 
     gProfileDataAccess->mProfileDataChanged = PR_TRUE;
     gProfileDataAccess->UpdateRegistry(nsnull);
@@ -2606,3 +2610,25 @@ nsProfile::GetRegStrings(const PRUnichar *aProfileName,
 
      return NS_OK;
 }
+
+nsresult nsProfile::GetLocalProfileDir(const PRUnichar* aProfileName,
+                                       nsIFile** aLocalDir)
+{
+  *aLocalDir = nsnull;
+  nsresult rv;
+  nsCOMPtr<nsIProperties> directoryService = 
+    do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
+  if (NS_FAILED(rv))
+    return rv;
+  nsCOMPtr<nsIFile> localDir;
+  rv = directoryService->Get(NS_APP_USER_PROFILES_LOCAL_ROOT_DIR,
+                             NS_GET_IID(nsIFile),
+                             getter_AddRefs(localDir));
+  if (NS_FAILED(rv))
+    return rv;
+  rv = localDir->Append(nsDependentString(aProfileName));
+  if (NS_FAILED(rv))
+    return rv;
+  localDir.swap(*aLocalDir);
+  return NS_OK;
+}
