@@ -84,7 +84,6 @@
 #include "nsIServiceManager.h"
 #include "nsISupportsArray.h"
 #include "nsCOMArray.h"
-#include "nsAutoLock.h"
 #include "nsEnumeratorUtils.h"
 #include "nsVoidArray.h"  // XXX introduces dependency on raptorbase
 #include "nsCRT.h"
@@ -100,12 +99,8 @@
 #include "prlog.h"
 #include "rdf.h"
 
-
-#if defined(MOZ_THREADSAFE_RDF)
-#define NS_AUTOLOCK(_lock) nsAutoLock _autolock(_lock)
-#else
-#define NS_AUTOLOCK(_lock)
-#endif
+#include "rdfIDataSource.h"
+#include "rdfITripleVisitor.h"
 
 #ifdef PR_LOGGING
 static PRLogModuleInfo* gLog = nsnull;
@@ -300,7 +295,8 @@ class InMemoryResourceEnumeratorImpl;
 class InMemoryDataSource : public nsIRDFDataSource,
                            public nsIRDFInMemoryDataSource,
                            public nsIRDFPropagatableDataSource,
-                           public nsIRDFPurgeableDataSource
+                           public nsIRDFPurgeableDataSource,
+                           public rdfIDataSource
 {
 protected:
     nsFixedSizeAllocator mAllocator;
@@ -316,6 +312,10 @@ protected:
     nsCOMArray<nsIRDFObserver> mObservers;  
     PRUint32                   mNumObservers;
 
+    // VisitFoo needs to block writes, [Un]Assert only allowed
+    // during mReadCount == 0
+    PRUint32 mReadCount;
+
     static PLDHashOperator PR_CALLBACK
     DeleteForwardArcsEntry(PLDHashTable* aTable, PLDHashEntryHdr* aHdr,
                            PRUint32 aNumber, void* aArg);
@@ -362,6 +362,9 @@ public:
     // nsIRDFPurgeableDataSource methods
     NS_DECL_NSIRDFPURGEABLEDATASOURCE
 
+    // rdfIDataSource methods
+    NS_DECL_RDFIDATASOURCE
+
 protected:
     static PLDHashOperator PR_CALLBACK
     SweepForwardArcsEntries(PLDHashTable* aTable, PLDHashEntryHdr* aHdr,
@@ -412,10 +415,6 @@ public:
                  PRBool aTruthValue = PR_TRUE);
 #endif
 
-#ifdef MOZ_THREADSAFE_RDF
-    // This datasource's monitor object.
-    PRLock* mLock;
-#endif
     PRBool  mPropagateChanges;
 };
 
@@ -554,8 +553,6 @@ NS_IMPL_QUERY_INTERFACE1(InMemoryAssertionEnumeratorImpl, nsISimpleEnumerator)
 NS_IMETHODIMP
 InMemoryAssertionEnumeratorImpl::HasMoreElements(PRBool* aResult)
 {
-    NS_AUTOLOCK(mDataSource->mLock);
-
     if (mValue) {
         *aResult = PR_TRUE;
         return NS_OK;
@@ -719,8 +716,6 @@ InMemoryArcsEnumeratorImpl::InMemoryArcsEnumeratorImpl(InMemoryDataSource* aData
             // its our magical HASH_ENTRY forward hash for assertions
             nsresult rv = NS_NewISupportsArray(getter_AddRefs(mHashArcs));
             if (NS_SUCCEEDED(rv)) {
-                NS_AUTOLOCK(mLock);
-
                 PL_DHashTableEnumerate(mAssertion->u.hash.mPropertyHash,
                     ArcEnumerator, mHashArcs.get());
             }
@@ -766,8 +761,6 @@ InMemoryArcsEnumeratorImpl::HasMoreElements(PRBool* aResult)
         return NS_OK;
     }
 
-    NS_AUTOLOCK(mDataSource->mLock);
-
     if (mHashArcs) {
         PRUint32    itemCount;
         nsresult    rv;
@@ -884,7 +877,7 @@ NS_NewRDFInMemoryDataSource(nsISupports* aOuter, const nsIID& aIID, void** aResu
 
 
 InMemoryDataSource::InMemoryDataSource(nsISupports* aOuter)
-    : mNumObservers(0)
+    : mNumObservers(0), mReadCount(0)
 {
     NS_INIT_AGGREGATED(aOuter);
 
@@ -901,9 +894,6 @@ InMemoryDataSource::InMemoryDataSource(nsISupports* aOuter)
 
     mAllocator.Init("nsInMemoryDataSource", kBucketSizes, kNumBuckets, kInitialSize);
 
-#ifdef MOZ_THREADSAFE_RDF
-    mLock = nsnull;
-#endif
     mForwardArcs.ops = nsnull;
     mReverseArcs.ops = nsnull;
     mPropagateChanges = PR_TRUE;
@@ -930,12 +920,6 @@ InMemoryDataSource::Init()
         return NS_ERROR_OUT_OF_MEMORY;
     }
 
-#ifdef MOZ_THREADSAFE_RDF
-    mLock = PR_NewLock();
-    if (! mLock)
-        return NS_ERROR_OUT_OF_MEMORY;
-#endif
-
 #ifdef PR_LOGGING
     if (! gLog)
         gLog = PR_NewLogModule("InMemoryDataSource");
@@ -966,9 +950,6 @@ InMemoryDataSource::~InMemoryDataSource()
     PR_LOG(gLog, PR_LOG_NOTICE,
            ("InMemoryDataSource(%p): destroyed.", this));
 
-#ifdef MOZ_THREADSAFE_RDF
-    PR_DestroyLock(mLock);
-#endif
 }
 
 PLDHashOperator PR_CALLBACK
@@ -994,38 +975,13 @@ InMemoryDataSource::DeleteForwardArcsEntry(PLDHashTable* aTable, PLDHashEntryHdr
 ////////////////////////////////////////////////////////////////////////
 
 NS_IMPL_AGGREGATED(InMemoryDataSource)
-
-NS_IMETHODIMP
-InMemoryDataSource::AggregatedQueryInterface(REFNSIID aIID, void** aResult)
-{
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
-    if (! aResult)
-        return NS_ERROR_NULL_POINTER;
-
-    if (aIID.Equals(NS_GET_IID(nsISupports))) {
-        *aResult = NS_STATIC_CAST(nsISupports*, &fAggregated);
-    }
-    else if (aIID.Equals(NS_GET_IID(nsIRDFDataSource))) {
-        *aResult = NS_STATIC_CAST(nsIRDFDataSource*, this);
-    }
-    else if (aIID.Equals(NS_GET_IID(nsIRDFInMemoryDataSource))) {
-        *aResult = NS_STATIC_CAST(nsIRDFInMemoryDataSource*, this);
-    }
-    else if (aIID.Equals(NS_GET_IID(nsIRDFPropagatableDataSource))) {
-        *aResult = NS_STATIC_CAST(nsIRDFPropagatableDataSource*, this);
-    }
-    else if (aIID.Equals(NS_GET_IID(nsIRDFPurgeableDataSource))) {
-        *aResult = NS_STATIC_CAST(nsIRDFPurgeableDataSource*, this);
-    }
-    else {
-        *aResult = nsnull;
-        return NS_NOINTERFACE;
-    }
-
-    NS_ADDREF(NS_STATIC_CAST(nsISupports*, *aResult));
-    return NS_OK;
-}
-
+NS_INTERFACE_MAP_BEGIN_AGGREGATED(InMemoryDataSource)
+    NS_INTERFACE_MAP_ENTRY(nsIRDFDataSource)
+    NS_INTERFACE_MAP_ENTRY(nsIRDFInMemoryDataSource)
+    NS_INTERFACE_MAP_ENTRY(nsIRDFPropagatableDataSource)
+    NS_INTERFACE_MAP_ENTRY(nsIRDFPurgeableDataSource)
+    NS_INTERFACE_MAP_ENTRY(rdfIDataSource)
+NS_INTERFACE_MAP_END
 
 ////////////////////////////////////////////////////////////////////////
 
@@ -1072,7 +1028,7 @@ InMemoryDataSource::LogOperation(const char* aOperation,
         PR_LogPrint
            ("  -->(\"%s\")\n", valueCStr);
 
-        nsCRT::free(valueCStr);
+        NS_Free(valueCStr);
     }
     else {
         PR_LogPrint
@@ -1111,8 +1067,6 @@ InMemoryDataSource::GetSource(nsIRDFResource* property,
     if (! target)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
-
     for (Assertion* as = GetReverseArcs(target); as; as = as->u.as.mInvNext) {
         if ((property == as->u.as.mProperty) && (tv == as->u.as.mTruthValue)) {
             *source = as->mSource;
@@ -1142,8 +1096,6 @@ InMemoryDataSource::GetTarget(nsIRDFResource* source,
     if (! target)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
-
     Assertion *as = GetForwardArcs(source);
     if (as && as->mHashEntry) {
         PLDHashEntryHdr* hdr = PL_DHashTableOperate(as->u.hash.mPropertyHash, property, PL_DHASH_LOOKUP);
@@ -1190,8 +1142,6 @@ InMemoryDataSource::HasAssertion(nsIRDFResource* source,
     if (! target)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
-
     Assertion *as = GetForwardArcs(source);
     if (as && as->mHashEntry) {
         PLDHashEntryHdr* hdr = PL_DHashTableOperate(as->u.hash.mPropertyHash, property, PL_DHASH_LOOKUP);
@@ -1246,8 +1196,6 @@ InMemoryDataSource::GetSources(nsIRDFResource* aProperty,
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
-
     InMemoryAssertionEnumeratorImpl* result =
         InMemoryAssertionEnumeratorImpl::Create(this, nsnull, aProperty,
                                                   aTarget, aTruthValue);
@@ -1279,8 +1227,6 @@ InMemoryDataSource::GetTargets(nsIRDFResource* aSource,
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
-
     InMemoryAssertionEnumeratorImpl* result =
         InMemoryAssertionEnumeratorImpl::Create(this, aSource, aProperty,
                                                 nsnull, aTruthValue);
@@ -1412,14 +1358,15 @@ InMemoryDataSource::Assert(nsIRDFResource* aSource,
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
-    nsresult rv;
-
-    {
-        NS_AUTOLOCK(mLock);
-        rv = LockedAssert(aSource, aProperty, aTarget, aTruthValue);
-        if (NS_FAILED(rv)) return rv;
+    if (mReadCount) {
+        NS_WARNING("Writing to InMemoryDataSource during read\n");
+        return NS_RDF_ASSERTION_REJECTED;
     }
 
+    nsresult rv;
+    rv = LockedAssert(aSource, aProperty, aTarget, aTruthValue);
+    if (NS_FAILED(rv)) return rv;
+
     // notify observers
     for (PRInt32 i = (PRInt32)mNumObservers - 1; mPropagateChanges && i >= 0; --i) {
         nsIRDFObserver* obs = mObservers[i];
@@ -1570,14 +1517,15 @@ InMemoryDataSource::Unassert(nsIRDFResource* aSource,
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
+    if (mReadCount) {
+        NS_WARNING("Writing to InMemoryDataSource during read\n");
+        return NS_RDF_ASSERTION_REJECTED;
+    }
+
     nsresult rv;
 
-    {
-        NS_AUTOLOCK(mLock);
-
-        rv = LockedUnassert(aSource, aProperty, aTarget);
-        if (NS_FAILED(rv)) return rv;
-    }
+    rv = LockedUnassert(aSource, aProperty, aTarget);
+    if (NS_FAILED(rv)) return rv;
 
     // Notify the world
     for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
@@ -1618,20 +1566,21 @@ InMemoryDataSource::Change(nsIRDFResource* aSource,
     if (! aNewTarget)
         return NS_ERROR_NULL_POINTER;
 
+    if (mReadCount) {
+        NS_WARNING("Writing to InMemoryDataSource during read\n");
+        return NS_RDF_ASSERTION_REJECTED;
+    }
+
     nsresult rv;
 
-    {
-        NS_AUTOLOCK(mLock);
+    // XXX We can implement LockedChange() if we decide that this
+    // is a performance bottleneck.
 
-        // XXX We can implement LockedChange() if we decide that this
-        // is a performance bottleneck.
+    rv = LockedUnassert(aSource, aProperty, aOldTarget);
+    if (NS_FAILED(rv)) return rv;
 
-        rv = LockedUnassert(aSource, aProperty, aOldTarget);
-        if (NS_FAILED(rv)) return rv;
-
-        rv = LockedAssert(aSource, aProperty, aNewTarget, PR_TRUE);
-        if (NS_FAILED(rv)) return rv;
-    }
+    rv = LockedAssert(aSource, aProperty, aNewTarget, PR_TRUE);
+    if (NS_FAILED(rv)) return rv;
 
     // Notify the world
     for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
@@ -1672,20 +1621,21 @@ InMemoryDataSource::Move(nsIRDFResource* aOldSource,
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
+    if (mReadCount) {
+        NS_WARNING("Writing to InMemoryDataSource during read\n");
+        return NS_RDF_ASSERTION_REJECTED;
+    }
+
     nsresult rv;
 
-    {
-        NS_AUTOLOCK(mLock);
+    // XXX We can implement LockedMove() if we decide that this
+    // is a performance bottleneck.
 
-        // XXX We can implement LockedMove() if we decide that this
-        // is a performance bottleneck.
+    rv = LockedUnassert(aOldSource, aProperty, aTarget);
+    if (NS_FAILED(rv)) return rv;
 
-        rv = LockedUnassert(aOldSource, aProperty, aTarget);
-        if (NS_FAILED(rv)) return rv;
-
-        rv = LockedAssert(aNewSource, aProperty, aTarget, PR_TRUE);
-        if (NS_FAILED(rv)) return rv;
-    }
+    rv = LockedAssert(aNewSource, aProperty, aTarget, PR_TRUE);
+    if (NS_FAILED(rv)) return rv;
 
     // Notify the world
     for (PRInt32 i = PRInt32(mNumObservers) - 1; mPropagateChanges && i >= 0; --i) {
@@ -1711,7 +1661,6 @@ InMemoryDataSource::AddObserver(nsIRDFObserver* aObserver)
     if (! aObserver)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
     mObservers.AppendObject(aObserver);
     mNumObservers = mObservers.Count();
 
@@ -1725,7 +1674,6 @@ InMemoryDataSource::RemoveObserver(nsIRDFObserver* aObserver)
     if (! aObserver)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
     mObservers.RemoveObject(aObserver);
     // note: use Count() instead of just decrementing
     // in case aObserver wasn't in list, for example
@@ -1737,8 +1685,6 @@ InMemoryDataSource::RemoveObserver(nsIRDFObserver* aObserver)
 NS_IMETHODIMP 
 InMemoryDataSource::HasArcIn(nsIRDFNode *aNode, nsIRDFResource *aArc, PRBool *result)
 {
-    NS_AUTOLOCK(mDataSource->mLock);
-
     Assertion* ass = GetReverseArcs(aNode);
     while (ass) {
         nsIRDFResource* elbow = ass->u.as.mProperty;
@@ -1755,8 +1701,6 @@ InMemoryDataSource::HasArcIn(nsIRDFNode *aNode, nsIRDFResource *aArc, PRBool *re
 NS_IMETHODIMP 
 InMemoryDataSource::HasArcOut(nsIRDFResource *aSource, nsIRDFResource *aArc, PRBool *result)
 {
-    NS_AUTOLOCK(mDataSource->mLock);
-
     Assertion* ass = GetForwardArcs(aSource);
     if (ass && ass->mHashEntry) {
         PLDHashEntryHdr* hdr = PL_DHashTableOperate(ass->u.hash.mPropertyHash,
@@ -1808,8 +1752,6 @@ InMemoryDataSource::ArcLabelsOut(nsIRDFResource* aSource, nsISimpleEnumerator**
     if (! aSource)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
-
     InMemoryArcsEnumeratorImpl* result =
         InMemoryArcsEnumeratorImpl::Create(this, aSource, nsnull);
 
@@ -1844,17 +1786,10 @@ InMemoryDataSource::GetAllResources(nsISimpleEnumerator** aResult)
     rv = NS_NewISupportsArray(getter_AddRefs(values));
     if (NS_FAILED(rv)) return rv;
 
-    NS_AUTOLOCK(mLock);
-
     // Enumerate all of our entries into an nsISupportsArray.
     PL_DHashTableEnumerate(&mForwardArcs, ResourceEnumerator, values.get());
 
-    *aResult = new nsArrayEnumerator(values);
-    if (! *aResult)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    NS_ADDREF(*aResult);
-    return NS_OK;
+    return NS_NewArrayEnumerator(aResult, values);
 }
 
 NS_IMETHODIMP
@@ -1998,8 +1933,6 @@ InMemoryDataSource::Mark(nsIRDFResource* aSource,
     if (! aTarget)
         return NS_ERROR_NULL_POINTER;
 
-    NS_AUTOLOCK(mLock);
-
     Assertion *as = GetForwardArcs(aSource);
     if (as && as->mHashEntry) {
         PLDHashEntryHdr* hdr = PL_DHashTableOperate(as->u.hash.mPropertyHash,
@@ -2063,13 +1996,10 @@ InMemoryDataSource::Sweep()
 {
     SweepInfo info = { nsnull, &mReverseArcs, &mAllocator};
 
-    {
-        // Remove all the assertions while holding the lock, but don't notify anyone.
-        NS_AUTOLOCK(mLock);
-        PL_DHashTableEnumerate(&mForwardArcs, SweepForwardArcsEntries, &info);
-    }
+    // Remove all the assertions, but don't notify anyone.
+    PL_DHashTableEnumerate(&mForwardArcs, SweepForwardArcsEntries, &info);
 
-    // Now we've left the autolock. Do the notification.
+    // Now do the notification.
     Assertion* as = info.mUnassertList;
     while (as) {
 #ifdef PR_LOGGING
@@ -2187,4 +2117,130 @@ InMemoryDataSource::SweepForwardArcsEntries(PLDHashTable* aTable,
 }
 
 ////////////////////////////////////////////////////////////////////////
+// rdfIDataSource methods
+
+class VisitorClosure
+{
+public:
+    VisitorClosure(rdfITripleVisitor* aVisitor) :
+        mVisitor(aVisitor),
+        mRv(NS_OK)
+    {};
+    rdfITripleVisitor* mVisitor;
+    nsresult mRv;
+};
+
+PLDHashOperator PR_CALLBACK
+SubjectEnumerator(PLDHashTable* aTable, PLDHashEntryHdr* aHdr,
+                  PRUint32 aNumber, void* aArg) {
+    Entry* entry = NS_REINTERPRET_CAST(Entry*, aHdr);
+    VisitorClosure* closure = NS_STATIC_CAST(VisitorClosure*, aArg);
+
+    nsresult rv;
+    nsCOMPtr<nsIRDFNode> subject = do_QueryInterface(entry->mNode, &rv);
+    NS_ENSURE_SUCCESS(rv, PL_DHASH_NEXT);
+
+    closure->mRv = closure->mVisitor->Visit(subject, nsnull, nsnull, PR_TRUE);
+    if (NS_FAILED(closure->mRv) || closure->mRv == NS_RDF_STOP_VISIT)
+        return PL_DHASH_STOP;
+
+    return PL_DHASH_NEXT;
+}
+
+NS_IMETHODIMP
+InMemoryDataSource::VisitAllSubjects(rdfITripleVisitor *aVisitor)
+{
+    // Lock datasource against writes
+    ++mReadCount;
+
+    // Enumerate all of our entries into an nsISupportsArray.
+    VisitorClosure cls(aVisitor);
+    PL_DHashTableEnumerate(&mForwardArcs, SubjectEnumerator, &cls);
+
+    // Unlock datasource
+    --mReadCount;
+
+    return cls.mRv;
+} 
+
+class TriplesInnerClosure
+{
+public:
+    TriplesInnerClosure(nsIRDFNode* aSubject, VisitorClosure* aClosure) :
+        mSubject(aSubject), mOuter(aClosure) {};
+    nsIRDFNode* mSubject;
+    VisitorClosure* mOuter;
+};
+
+PLDHashOperator PR_CALLBACK
+TriplesInnerEnumerator(PLDHashTable* aTable, PLDHashEntryHdr* aHdr,
+                  PRUint32 aNumber, void* aArg) {
+    Entry* entry = NS_REINTERPRET_CAST(Entry*, aHdr);
+    Assertion* assertion = entry->mAssertions;
+    TriplesInnerClosure* closure = 
+        NS_STATIC_CAST(TriplesInnerClosure*, aArg);
+    while (assertion) {
+        NS_ASSERTION(!assertion->mHashEntry, "shouldn't have to hashes");
+        VisitorClosure* cls = closure->mOuter;
+        cls->mRv = cls->mVisitor->Visit(closure->mSubject,
+                                        assertion->u.as.mProperty,
+                                        assertion->u.as.mTarget,
+                                        assertion->u.as.mTruthValue);
+        if (NS_FAILED(cls->mRv) || cls->mRv == NS_RDF_STOP_VISIT) {
+            return PL_DHASH_STOP;
+        }
+        assertion = assertion->mNext;
+    }
+    return PL_DHASH_NEXT;
+}
+PLDHashOperator PR_CALLBACK
+TriplesEnumerator(PLDHashTable* aTable, PLDHashEntryHdr* aHdr,
+                  PRUint32 aNumber, void* aArg) {
+    Entry* entry = NS_REINTERPRET_CAST(Entry*, aHdr);
+    VisitorClosure* closure = NS_STATIC_CAST(VisitorClosure*, aArg);
+
+    nsresult rv;
+    nsCOMPtr<nsIRDFNode> subject = do_QueryInterface(entry->mNode, &rv);
+    NS_ENSURE_SUCCESS(rv, PL_DHASH_NEXT);
+
+    if (entry->mAssertions->mHashEntry) {
+        TriplesInnerClosure cls(subject, closure);
+        PL_DHashTableEnumerate(entry->mAssertions->u.hash.mPropertyHash,
+                               TriplesInnerEnumerator, &cls);
+        if (NS_FAILED(closure->mRv)) {
+            return PL_DHASH_STOP;
+        }
+        return PL_DHASH_NEXT;
+    }
+    Assertion* assertion = entry->mAssertions;
+    while (assertion) {
+        NS_ASSERTION(!assertion->mHashEntry, "shouldn't have to hashes");
+        closure->mRv = closure->mVisitor->Visit(subject,
+                                                assertion->u.as.mProperty,
+                                                assertion->u.as.mTarget,
+                                                assertion->u.as.mTruthValue);
+        if (NS_FAILED(closure->mRv) || closure->mRv == NS_RDF_STOP_VISIT) {
+            return PL_DHASH_STOP;
+        }
+        assertion = assertion->mNext;
+    }
+    return PL_DHASH_NEXT;
+}
+NS_IMETHODIMP
+InMemoryDataSource::VisitAllTriples(rdfITripleVisitor *aVisitor)
+{
+    // Lock datasource against writes
+    ++mReadCount;
+
+    // Enumerate all of our entries into an nsISupportsArray.
+    VisitorClosure cls(aVisitor);
+    PL_DHashTableEnumerate(&mForwardArcs, TriplesEnumerator, &cls);
+
+    // Unlock datasource
+    --mReadCount;
+
+    return cls.mRv;
+} 
+
+////////////////////////////////////////////////////////////////////////
 
