@@ -72,6 +72,7 @@
 */
 
 #include "nsCOMPtr.h"
+#include "nsInterfaceHashtable.h"
 #include "nsIContentSink.h"
 #include "nsIRDFContainer.h"
 #include "nsIRDFContainerUtils.h"
@@ -83,7 +84,6 @@
 #include "nsIURL.h"
 #include "nsIXMLContentSink.h"
 #include "nsRDFCID.h"
-#include "nsRDFParserUtils.h"
 #include "nsVoidArray.h"
 #include "nsXPIDLString.h"
 #include "prlog.h"
@@ -95,13 +95,7 @@
 #include "nsCRT.h"
 #include "nsIAtom.h"
 #include "nsStaticAtom.h"
-
-////////////////////////////////////////////////////////////////////////
-
-static const char kNameSpaceSeparator = ':';
-
-static const char kRDFNameSpaceURI[] = RDF_NAMESPACE_URI;
-static const char kNCNameSpaceURI[] = NC_NAMESPACE_URI;
+#include "nsIScriptError.h"
 
 ////////////////////////////////////////////////////////////////////////
 // XPCOM IIDs
@@ -141,8 +135,6 @@ enum RDFContentSinkParseMode {
     eRDFContentSinkParseMode_Date
 };
 
-MOZ_DECL_CTOR_COUNTER(RDFContentSinkImpl::NameSpaceEntry)
-
 typedef
 NS_STDCALL_FUNCPROTO(nsresult,
                      nsContainerTestFn,
@@ -172,7 +164,7 @@ public:
     NS_IMETHOD WillInterrupt(void);
     NS_IMETHOD WillResume(void);
     NS_IMETHOD SetParser(nsIParser* aParser);  
-    virtual void FlushContent(PRBool aNotify) { }
+    virtual void FlushPendingNotifications(mozFlushType aType) { }
     NS_IMETHOD SetDocumentCharset(nsACString& aCharset) { return NS_OK; }
     virtual nsISupports *GetTarget() { return nsnull; }
 
@@ -194,6 +186,7 @@ public:
 
     static nsIAtom* kAboutAtom;
     static nsIAtom* kIdAtom;
+    static nsIAtom* kNodeIdAtom;
     static nsIAtom* kAboutEachAtom;
     static nsIAtom* kResourceAtom;
     static nsIAtom* kRDFAtom;
@@ -232,51 +225,27 @@ protected:
     nsresult AddProperties(const PRUnichar** aAttributes, nsIRDFResource* aSubject, PRInt32* aCount = nsnull);
     void SetParseMode(const PRUnichar **aAttributes);
 
-    // namespace management
-    nsresult PushNameSpacesFrom(const PRUnichar** aAttributes);
-    PRBool   IsXMLNSDirective(const nsAString& aAttributeKey, nsIAtom** aPrefix = nsnull);
-    nsresult PopNameSpaces();
-
     PRUnichar* mText;
     PRInt32 mTextLength;
     PRInt32 mTextSize;
 
-    struct NameSpaceEntry {
-    public:
-        NameSpaceEntry(nsIAtom* aPrefix, const char* aNameSpaceURI)
-            : mPrefix(aPrefix), mNext(nsnull) {
-            MOZ_COUNT_CTOR(RDFContentSinkImpl::NameSpaceEntry);
-            mNameSpaceURI = PL_strdup(aNameSpaceURI);
-        }
+    /**
+     * From the set of given attributes, this method extracts the 
+     * namespace definitions and feeds them to the datasource.
+     * These can then be suggested to the serializer to be used again.
+     * Hopefully, this will keep namespace definitions intact in a 
+     * parse - serialize cycle.
+     */
+    void RegisterNamespaces(const PRUnichar **aAttributes);
 
-        ~NameSpaceEntry() {
-            MOZ_COUNT_DTOR(RDFContentSinkImpl::NameSpaceEntry);
-            PL_strfree(mNameSpaceURI);
-        }
-
-        nsCOMPtr<nsIAtom> mPrefix;
-        char*             mNameSpaceURI;
-        NameSpaceEntry*   mNext;
-    };
-
-    NameSpaceEntry* mNameSpaceStack;
-    nsAutoVoidArray mNameSpaceScopes;
-
-    nsIAtom*
-    CutNameSpacePrefix(const nsAString& aString, nsIAtom** aTagAtom);
-
-    nsresult
-    GetNameSpaceURI(nsIAtom* aPrefix, const char** aNameSpaceURI);
-
-    nsresult
-    ParseTagString(const PRUnichar* aTagName,
-                   const char** aNameSpaceURI,
-                   nsIAtom** aTag);
-
-    nsresult
-    ParseAttributeString(const nsAString& aAttributeName,
-                         const char** aNameSpaceURI,
-                         nsIAtom** aAttribute);
+    /**
+     * Extracts the localname from aExpatName, the name that the Expat parser
+     * passes us.
+     * aLocalName will contain the localname in aExpatName.
+     * The return value is a dependent string containing just the namespace.
+     */
+    const nsDependentSubstring SplitExpatName(const PRUnichar *aExpatName,
+                                              nsIAtom **aLocalName);
 
     enum eContainerType { eBag, eSeq, eAlt };
     nsresult InitContainer(nsIRDFResource* aContainerType, nsIRDFResource* aContainer);
@@ -285,6 +254,9 @@ protected:
     // The datasource in which we're assigning assertions
     nsCOMPtr<nsIRDFDataSource> mDataSource;
 
+    // A hash of all the node IDs referred to
+    nsInterfaceHashtable<nsStringHashKey, nsIRDFResource> mNodeIDMap;
+
     // The current state of the content sink
     RDFContentSinkState mState;
     RDFContentSinkParseMode mParseMode;
@@ -319,6 +291,7 @@ nsIRDFResource* RDFContentSinkImpl::kRDF_nextVal;
 
 nsIAtom* RDFContentSinkImpl::kAboutAtom;
 nsIAtom* RDFContentSinkImpl::kIdAtom;
+nsIAtom* RDFContentSinkImpl::kNodeIdAtom;
 nsIAtom* RDFContentSinkImpl::kAboutEachAtom;
 nsIAtom* RDFContentSinkImpl::kResourceAtom;
 nsIAtom* RDFContentSinkImpl::kRDFAtom;
@@ -334,6 +307,7 @@ nsIAtom* RDFContentSinkImpl::kParseTypeAtom;
 static const nsStaticAtom rdf_atoms[] = {
     { "about", &RDFContentSinkImpl::kAboutAtom },
     { "ID", &RDFContentSinkImpl::kIdAtom },
+    { "nodeID", &RDFContentSinkImpl::kNodeIdAtom },
     { "aboutEach", &RDFContentSinkImpl::kAboutEachAtom },
     { "resource", &RDFContentSinkImpl::kResourceAtom },
     { "RDF", &RDFContentSinkImpl::kRDFAtom },
@@ -350,7 +324,6 @@ RDFContentSinkImpl::RDFContentSinkImpl()
     : mText(nsnull),
       mTextLength(0),
       mTextSize(0),
-      mNameSpaceStack(nsnull),
       mState(eRDFContentSinkState_InProlog),
       mParseMode(eRDFContentSinkParseMode_Literal),
       mContextStack(nsnull),
@@ -381,6 +354,8 @@ RDFContentSinkImpl::RDFContentSinkImpl()
         NS_RegisterStaticAtoms(rdf_atoms, NS_ARRAY_LENGTH(rdf_atoms));
     }
 
+    mNodeIDMap.Init();
+
 #ifdef PR_LOGGING
     if (! gLog)
         gLog = PR_NewLogModule("nsRDFContentSink");
@@ -397,12 +372,6 @@ RDFContentSinkImpl::~RDFContentSinkImpl()
 
     NS_IF_RELEASE(mDocumentURL);
 
-    if (mNameSpaceStack) {
-        // There shouldn't be any here except in an error condition
-        PRInt32 i = mNameSpaceScopes.Count();
-        while (--i >= 0)
-            PopNameSpaces();
-    }
     if (mContextStack) {
         PR_LOG(gLog, PR_LOG_WARNING,
                ("rdfxml: warning! unclosed tag"));
@@ -488,14 +457,10 @@ RDFContentSinkImpl::HandleStartElement(const PRUnichar *aName,
 {
   FlushText();
 
-  // We must register namespace declarations found in the attribute
-  // list of an element before creating the element. This is because
-  // the namespace prefix for an element might be declared within
-  // the attribute list.
-  PushNameSpacesFrom(aAtts);
-
   nsresult rv = NS_ERROR_UNEXPECTED; // XXX
 
+  RegisterNamespaces(aAtts);
+
   switch (mState) {
   case eRDFContentSinkState_InProlog:
       rv = OpenRDF(aName);
@@ -545,7 +510,7 @@ RDFContentSinkImpl::HandleEndElement(const PRUnichar *aName)
                  ("rdfxml: extra close tag '%s' at line %d",
                   tagCStr, 0/*XXX fix me */);
 
-          nsCRT::free(tagCStr);
+          NS_Free(tagCStr);
       }
 #endif
 
@@ -576,8 +541,6 @@ RDFContentSinkImpl::HandleEndElement(const PRUnichar *aName)
   if (nestLevel == 0)
       mState = eRDFContentSinkState_InEpilog;
 
-  PopNameSpaces();
-    
   NS_IF_RELEASE(resource);
   return NS_OK;
 }
@@ -620,16 +583,23 @@ RDFContentSinkImpl::HandleProcessingInstruction(const PRUnichar *aTarget,
 }
 
 NS_IMETHODIMP 
-RDFContentSinkImpl::HandleXMLDeclaration(const PRUnichar *aData, 
-                                         PRUint32 aLength)
+RDFContentSinkImpl::HandleXMLDeclaration(const PRUnichar *aVersion,
+                                         const PRUnichar *aEncoding,
+                                         PRInt32 aStandalone)
 {
     return NS_OK;
 }
 
 NS_IMETHODIMP
 RDFContentSinkImpl::ReportError(const PRUnichar* aErrorText, 
-                                const PRUnichar* aSourceText)
+                                const PRUnichar* aSourceText,
+                                nsIScriptError *aError,
+                                PRBool *_retval)
 {
+  NS_PRECONDITION(aError && aSourceText && aErrorText, "Check arguments!!!");
+
+  // The expat driver should report the error.
+  *_retval = PR_TRUE;
   return NS_OK;
 }
 
@@ -777,7 +747,7 @@ RDFContentSinkImpl::ParseText(nsIRDFNode **aResult)
 
     case eRDFContentSinkParseMode_Date:
         {
-            PRTime t = rdf_ParseDate(nsDependentCString(NS_LossyConvertUCS2toASCII(value).get(), value.Length()));
+            PRTime t = rdf_ParseDate(nsDependentCString(NS_LossyConvertUTF16toASCII(value).get(), value.Length()));
             nsIRDFDate *result;
             gRDFService->GetDateLiteral(t, &result);
             *aResult = result;
@@ -869,40 +839,12 @@ RDFContentSinkImpl::AddText(const PRUnichar* aText, PRInt32 aLength)
     return NS_OK;
 }
 
-nsresult
-RDFContentSinkImpl::PushNameSpacesFrom(const PRUnichar** aAttributes)
+PRBool
+rdf_RequiresAbsoluteURI(const nsString& uri)
 {
-  // Remember the current top of the stack as the namespace
-  // scope. When popping namespaces, we'll remove stack elements
-  // until we hit this.
-  mNameSpaceScopes.AppendElement(mNameSpaceStack);
-
-  for (; *aAttributes; aAttributes += 2) {
-      nsCOMPtr<nsIAtom> prefix;
-      if (! IsXMLNSDirective(nsDependentString(aAttributes[0]), 
-                             getter_AddRefs(prefix))) {
-          continue;
-      }
-    
-      nsAutoString uri(aAttributes[1]);
-      nsRDFParserUtils::StripAndConvert(uri);
-
-      // Open a local namespace
-      NameSpaceEntry* ns = new NameSpaceEntry(prefix, NS_ConvertUCS2toUTF8(uri).get());
-      if (! ns) {
-          return NS_ERROR_OUT_OF_MEMORY;
-      }
-
-      ns->mNext = mNameSpaceStack;
-      mNameSpaceStack = ns;
-
-      // Add it to the set of namespaces used in the RDF/XML document.
-      nsCOMPtr<nsIRDFXMLSink> sink = do_QueryInterface(mDataSource);
-      if (sink) {
-          sink->AddNameSpace(prefix, uri);
-      }
-  }
-  return NS_OK;
+    // cheap shot at figuring out if this requires an absolute url translation
+    return !(StringBeginsWith(uri, NS_LITERAL_STRING("urn:")) ||
+             StringBeginsWith(uri, NS_LITERAL_STRING("chrome:")));
 }
 
 nsresult
@@ -911,49 +853,45 @@ RDFContentSinkImpl::GetIdAboutAttribute(const PRUnichar** aAttributes,
                                         PRBool* aIsAnonymous)
 {
     // This corresponds to the dirty work of production [6.5]
-    nsresult rv;
+    nsresult rv = NS_OK;
 
-    nsCAutoString docURI;
-    rv = mDocumentURL->GetSpec(docURI);
-    if (NS_FAILED(rv)) return rv;
+    nsAutoString nodeID;
 
+    nsCOMPtr<nsIAtom> localName;
     for (; *aAttributes; aAttributes += 2) {
-        // Get upper-cased key
-        const char* nameSpaceURI;
-        nsCOMPtr<nsIAtom> attr;
-        rv = ParseAttributeString(nsDependentString(aAttributes[0]), 
-                                  &nameSpaceURI, 
-                                  getter_AddRefs(attr));
-        if (NS_FAILED(rv)) return rv;
+        const nsDependentSubstring& nameSpaceURI =
+            SplitExpatName(aAttributes[0], getter_AddRefs(localName));
 
         // We'll accept either `ID' or `rdf:ID' (ibid with `about' or
         // `rdf:about') in the spirit of being liberal towards the
         // input that we receive.
-        if (nameSpaceURI && 0 != PL_strcmp(nameSpaceURI, kRDFNameSpaceURI)) {
+        if (!nameSpaceURI.IsEmpty() &&
+            !nameSpaceURI.EqualsLiteral(RDF_NAMESPACE_URI)) {
           continue;
         }
 
         // XXX you can't specify both, but we'll just pick up the
         // first thing that was specified and ignore the other.
       
-        if (attr.get() == kAboutAtom) {
+        if (localName == kAboutAtom) {
             if (aIsAnonymous)
                 *aIsAnonymous = PR_FALSE;
 
-            nsAutoString uri(aAttributes[1]);
-            nsRDFParserUtils::StripAndConvert(uri);
-
-            rdf_MakeAbsoluteURI(NS_ConvertUTF8toUCS2(docURI), uri);
-
-            return gRDFService->GetUnicodeResource(uri, aResource);
+            nsAutoString relURI(aAttributes[1]);
+            if (rdf_RequiresAbsoluteURI(relURI)) {
+                nsCAutoString uri;
+                rv = mDocumentURL->Resolve(NS_ConvertUTF16toUTF8(aAttributes[1]), uri);
+                if (NS_FAILED(rv)) return rv;
+                
+                return gRDFService->GetResource(uri, 
+                                                aResource);
+            } 
+            return gRDFService->GetResource(NS_ConvertUTF16toUTF8(aAttributes[1]), 
+                                            aResource);
         }
-        else if (attr.get() == kIdAtom) {
+        else if (localName == kIdAtom) {
             if (aIsAnonymous)
                 *aIsAnonymous = PR_FALSE;
-
-            nsAutoString name(aAttributes[1]);
-            nsRDFParserUtils::StripAndConvert(name);
-
             // In the spirit of leniency, we do not bother trying to
             // enforce that this be a valid "XML Name" (see
             // http://www.w3.org/TR/REC-xml#NT-Nmtoken), as per
@@ -962,13 +900,19 @@ RDFContentSinkImpl::GetIdAboutAttribute(const PRUnichar** aAttributes,
             // Construct an in-line resource whose URI is the
             // document's URI plus the XML name specified in the ID
             // attribute.
-            name.Insert(PRUnichar('#'), 0);
-          
-            rdf_MakeAbsoluteURI(NS_ConvertUTF8toUCS2(docURI), name);
+            nsCAutoString name;
+            nsCAutoString ref('#');
+            AppendUTF16toUTF8(aAttributes[1], ref);
 
-            return gRDFService->GetUnicodeResource(name, aResource);
+            rv = mDocumentURL->Resolve(ref, name);
+            if (NS_FAILED(rv)) return rv;
+
+            return gRDFService->GetResource(name, aResource);
         }
-        else if (attr.get() == kAboutEachAtom) {
+        else if (localName == kNodeIdAtom) {
+            nodeID.Assign(aAttributes[1]);
+        }
+        else if (localName == kAboutEachAtom) {
             // XXX we don't deal with aboutEach...
             //PR_LOG(gLog, PR_LOG_WARNING,
             //       ("rdfxml: ignoring aboutEach at line %d",
@@ -980,7 +924,20 @@ RDFContentSinkImpl::GetIdAboutAttribute(const PRUnichar** aAttributes,
     if (aIsAnonymous)
         *aIsAnonymous = PR_TRUE;
 
-    rv = gRDFService->GetAnonymousResource(aResource);
+    // If nodeID is present, check if we already know about it. If we've seen
+    // the nodeID before, use the same resource, otherwise generate a new one.
+    if (!nodeID.IsEmpty()) {
+        mNodeIDMap.Get(nodeID,aResource);
+
+        if (!*aResource) {
+            rv = gRDFService->GetAnonymousResource(aResource);
+            mNodeIDMap.Put(nodeID,*aResource);
+        }
+    }
+    else {
+        rv = gRDFService->GetAnonymousResource(aResource);
+    }
+
     return rv;
 }
 
@@ -988,41 +945,59 @@ nsresult
 RDFContentSinkImpl::GetResourceAttribute(const PRUnichar** aAttributes,
                                          nsIRDFResource** aResource)
 {
-  nsresult rv;
+  nsCOMPtr<nsIAtom> localName;
+
+  nsAutoString nodeID;
 
   for (; *aAttributes; aAttributes += 2) {
-      // Get upper-cased key
-      const char* nameSpaceURI;
-      nsCOMPtr<nsIAtom> attr;
-      rv = ParseAttributeString(nsDependentString(aAttributes[0]), 
-                                &nameSpaceURI, 
-                                getter_AddRefs(attr));
-      if (NS_FAILED(rv)) return rv;
+      const nsDependentSubstring& nameSpaceURI =
+          SplitExpatName(aAttributes[0], getter_AddRefs(localName));
 
       // We'll accept `resource' or `rdf:resource', under the spirit
       // that we should be liberal towards the input that we
       // receive.
-      if (nameSpaceURI && 0 != PL_strcmp(nameSpaceURI, kRDFNameSpaceURI)) {
+      if (!nameSpaceURI.IsEmpty() &&
+          !nameSpaceURI.EqualsLiteral(RDF_NAMESPACE_URI)) {
           continue;
       }
 
       // XXX you can't specify both, but we'll just pick up the
       // first thing that was specified and ignore the other.
 
-      if (attr.get() == kResourceAtom) {
-          nsAutoString uri(aAttributes[1]);
-          nsRDFParserUtils::StripAndConvert(uri);
-
+      if (localName == kResourceAtom) {
           // XXX Take the URI and make it fully qualified by
           // sticking it into the document's URL. This may not be
           // appropriate...
-          nsCAutoString documentURL;
-          mDocumentURL->GetSpec(documentURL);
-          rdf_MakeAbsoluteURI(NS_ConvertUTF8toUCS2(documentURL), uri);
+          nsAutoString relURI(aAttributes[1]);
+          if (rdf_RequiresAbsoluteURI(relURI)) {
+              nsresult rv;
+              nsCAutoString uri;
 
-          return gRDFService->GetUnicodeResource(uri, aResource);
+              rv = mDocumentURL->Resolve(NS_ConvertUTF16toUTF8(aAttributes[1]), uri);
+              if (NS_FAILED(rv)) return rv;
+
+              return gRDFService->GetResource(uri, aResource);
+          } 
+          return gRDFService->GetResource(NS_ConvertUTF16toUTF8(aAttributes[1]), 
+                                          aResource);
+      }
+      else if (localName == kNodeIdAtom) {
+          nodeID.Assign(aAttributes[1]);
       }
   }
+    
+  // If nodeID is present, check if we already know about it. If we've seen
+  // the nodeID before, use the same resource, otherwise generate a new one.
+  if (!nodeID.IsEmpty()) {
+      mNodeIDMap.Get(nodeID,aResource);
+
+      if (!*aResource) {
+          mNodeIDMap.Put(nodeID,*aResource);
+          return gRDFService->GetAnonymousResource(aResource);
+      }
+      return NS_OK;
+  }
+
   return NS_ERROR_FAILURE;
 }
 
@@ -1034,59 +1009,49 @@ RDFContentSinkImpl::AddProperties(const PRUnichar** aAttributes,
   if (aCount)
       *aCount = 0;
 
+  nsCOMPtr<nsIAtom> localName;
   for (; *aAttributes; aAttributes += 2) {
-      // Get upper-cased key
-      const nsDependentString key(aAttributes[0]);
+      const nsDependentSubstring& nameSpaceURI =
+          SplitExpatName(aAttributes[0], getter_AddRefs(localName));
 
       // skip 'xmlns' directives, these are "meta" information
-      if (IsXMLNSDirective(key)) {
+      if (nameSpaceURI.EqualsLiteral("http://www.w3.org/2000/xmlns/")) {
         continue;
       }
 
-      const char* nameSpaceURI;
-      nsCOMPtr<nsIAtom> attr;
-      ParseAttributeString(key, &nameSpaceURI, getter_AddRefs(attr));
-
-      // skip `about', `ID', and `resource' attributes (either with or
+      // skip `about', `ID', `resource', and 'nodeID' attributes (either with or
       // without the `rdf:' prefix); these are all "special" and
       // should've been dealt with by the caller.
-      if (attr == kAboutAtom || attr == kIdAtom || attr == kResourceAtom) {
-          if (!nameSpaceURI || 0 == PL_strcmp(nameSpaceURI, kRDFNameSpaceURI))
+      if (localName == kAboutAtom || localName == kIdAtom ||
+          localName == kResourceAtom || localName == kNodeIdAtom) {
+          if (nameSpaceURI.IsEmpty() ||
+              nameSpaceURI.EqualsLiteral(RDF_NAMESPACE_URI))
               continue;
       }
 
       // Skip `parseType', `RDF:parseType', and `NC:parseType'. This
       // is meta-information that will be handled in SetParseMode.
-      if (attr == kParseTypeAtom) {
-          if (!nameSpaceURI
-              || 0 == PL_strcmp(nameSpaceURI, kRDFNameSpaceURI)
-              || 0 == PL_strcmp(nameSpaceURI, kNCNameSpaceURI)) {
+      if (localName == kParseTypeAtom) {
+          if (nameSpaceURI.IsEmpty() ||
+              nameSpaceURI.EqualsLiteral(RDF_NAMESPACE_URI) ||
+              nameSpaceURI.EqualsLiteral(NC_NAMESPACE_URI)) {
               continue;
           }
       }
 
-      nsAutoString v(aAttributes[1]);
-      nsRDFParserUtils::StripAndConvert(v);
-
       const char* attrName;
-      attr->GetUTF8String(&attrName);
+      localName->GetUTF8String(&attrName);
 
-      nsCAutoString propertyStr;
-    
-      if (nameSpaceURI) {
-        propertyStr.Assign(nsDependentCString(nameSpaceURI) + 
-                           nsDependentCString(attrName));
-      }
-      else {
-        propertyStr.Assign(attrName);
-      }
+      NS_ConvertUTF16toUTF8 propertyStr(nameSpaceURI);    
+      propertyStr.Append(attrName);
 
       // Add the assertion to RDF
       nsCOMPtr<nsIRDFResource> property;
       gRDFService->GetResource(propertyStr, getter_AddRefs(property));
 
       nsCOMPtr<nsIRDFLiteral> target;
-      gRDFService->GetLiteral(v.get(), getter_AddRefs(target));
+      gRDFService->GetLiteral(aAttributes[1], 
+                              getter_AddRefs(target));
 
       mDataSource->Assert(aSubject, property, target, PR_TRUE);
   }
@@ -1096,24 +1061,22 @@ RDFContentSinkImpl::AddProperties(const PRUnichar** aAttributes,
 void
 RDFContentSinkImpl::SetParseMode(const PRUnichar **aAttributes)
 {
+    nsCOMPtr<nsIAtom> localName;
     for (; *aAttributes; aAttributes += 2) {
-        const nsDependentString key(aAttributes[0]);
+        const nsDependentSubstring& nameSpaceURI =
+            SplitExpatName(aAttributes[0], getter_AddRefs(localName));
 
-        const char *nameSpaceURI;
-        nsCOMPtr<nsIAtom> attr;
-        ParseAttributeString(key, &nameSpaceURI, getter_AddRefs(attr));
+        if (localName == kParseTypeAtom) {
+            nsDependentString v(aAttributes[1]);
 
-        if (attr == kParseTypeAtom) {
-            nsAutoString v(aAttributes[1]);
-            nsRDFParserUtils::StripAndConvert(v);
-
-            if (!nameSpaceURI || 0 == PL_strcmp(nameSpaceURI, kRDFNameSpaceURI)) {
+            if (nameSpaceURI.IsEmpty() ||
+                nameSpaceURI.EqualsLiteral(RDF_NAMESPACE_URI)) {
                 if (v.EqualsLiteral("Resource"))
                     mParseMode = eRDFContentSinkParseMode_Resource;
 
                 break;
             }
-            else if (0 == PL_strcmp(nameSpaceURI, kNCNameSpaceURI)) {
+            else if (nameSpaceURI.EqualsLiteral(NC_NAMESPACE_URI)) {
                 if (v.EqualsLiteral("Date"))
                     mParseMode = eRDFContentSinkParseMode_Date;
                 else if (v.EqualsLiteral("Integer"))
@@ -1134,16 +1097,12 @@ RDFContentSinkImpl::OpenRDF(const PRUnichar* aName)
     // ensure that we're actually reading RDF by making sure that the
     // opening tag is <rdf:RDF>, where "rdf:" corresponds to whatever
     // they've declared the standard RDF namespace to be.
-    nsresult rv;
+    nsCOMPtr<nsIAtom> localName;
+    const nsDependentSubstring& nameSpaceURI =
+        SplitExpatName(aName, getter_AddRefs(localName));
 
-    nsCOMPtr<nsIAtom> tag;
-    const char* nameSpaceURI;
-    rv = ParseTagString(aName, &nameSpaceURI, getter_AddRefs(tag));
-    if (NS_FAILED(rv)) return rv;
-
-    if ((nameSpaceURI && 0 != PL_strcmp(nameSpaceURI, kRDFNameSpaceURI))
-        || (tag.get() != kRDFAtom)) {
-       // PR_LOG(gLog, PR_LOG_WARNING,
+    if (!nameSpaceURI.EqualsLiteral(RDF_NAMESPACE_URI) || localName != kRDFAtom) {
+       // PR_LOG(gLog, PR_LOG_ALWAYS,
        //        ("rdfxml: expected RDF:RDF at line %d",
        //         aNode.GetSourceLineNumber()));
 
@@ -1162,11 +1121,9 @@ RDFContentSinkImpl::OpenObject(const PRUnichar* aName,
     // an "object" non-terminal is either a "description", a "typed
     // node", or a "container", so this change the content sink's
     // state appropriately.
-    nsresult rv;
-
-    nsCOMPtr<nsIAtom> tag;
-    const char* nameSpaceURI;
-    ParseTagString(aName, &nameSpaceURI, getter_AddRefs(tag));
+    nsCOMPtr<nsIAtom> localName;
+    const nsDependentSubstring& nameSpaceURI =
+        SplitExpatName(aName, getter_AddRefs(localName));
 
     // Figure out the URI of this object, and create an RDF node for it.
     nsCOMPtr<nsIRDFResource> source;
@@ -1184,24 +1141,24 @@ RDFContentSinkImpl::OpenObject(const PRUnichar* aName,
     // description or a container.
     PRBool isaTypedNode = PR_TRUE;
 
-    if (nameSpaceURI && 0 == PL_strcmp(nameSpaceURI, kRDFNameSpaceURI)) {
+    if (nameSpaceURI.EqualsLiteral(RDF_NAMESPACE_URI)) {
         isaTypedNode = PR_FALSE;
 
-        if (tag.get() == kDescriptionAtom) {
+        if (localName == kDescriptionAtom) {
             // it's a description
             mState = eRDFContentSinkState_InDescriptionElement;
         }
-        else if (tag.get() == kBagAtom) {
+        else if (localName == kBagAtom) {
             // it's a bag container
             InitContainer(kRDF_Bag, source);
             mState = eRDFContentSinkState_InContainerElement;
         }
-        else if (tag.get() == kSeqAtom) {
+        else if (localName == kSeqAtom) {
             // it's a seq container
             InitContainer(kRDF_Seq, source);
             mState = eRDFContentSinkState_InContainerElement;
         }
-        else if (tag.get() == kAltAtom) {
+        else if (localName == kAltAtom) {
             // it's an alt container
             InitContainer(kRDF_Alt, source);
             mState = eRDFContentSinkState_InContainerElement;
@@ -1214,18 +1171,14 @@ RDFContentSinkImpl::OpenObject(const PRUnichar* aName,
     }
 
     if (isaTypedNode) {
-        nsCAutoString typeStr;
-
-        if (nameSpaceURI)
-            typeStr = nameSpaceURI;
-
         const char* attrName;
-        tag->GetUTF8String(&attrName);
+        localName->GetUTF8String(&attrName);
 
-        typeStr += attrName;
+        NS_ConvertUTF16toUTF8 typeStr(nameSpaceURI);
+        typeStr.Append(attrName);
 
         nsCOMPtr<nsIRDFResource> type;
-        rv = gRDFService->GetResource(typeStr, getter_AddRefs(type));
+        nsresult rv = gRDFService->GetResource(typeStr, getter_AddRefs(type));
         if (NS_FAILED(rv)) return rv;
 
         rv = mDataSource->Assert(source, kRDF_type, type, PR_TRUE);
@@ -1246,22 +1199,15 @@ RDFContentSinkImpl::OpenProperty(const PRUnichar* aName, const PRUnichar** aAttr
     // an "object" non-terminal is either a "description", a "typed
     // node", or a "container", so this change the content sink's
     // state appropriately.
-    const char* nameSpaceURI;
-    nsCOMPtr<nsIAtom> tag;
-    ParseTagString(aName, &nameSpaceURI, getter_AddRefs(tag));
-
+    nsCOMPtr<nsIAtom> localName;
+    const nsDependentSubstring& nameSpaceURI =
+        SplitExpatName(aName, getter_AddRefs(localName));
 
     const char* attrName;
-    tag->GetUTF8String(&attrName);
+    localName->GetUTF8String(&attrName);
 
-    nsCAutoString propertyStr;
-    if (nameSpaceURI) {
-        propertyStr.Assign(nsDependentCString(nameSpaceURI) + 
-                           nsDependentCString(attrName));
-    }
-    else {
-        propertyStr.Assign(attrName);
-    }
+    NS_ConvertUTF16toUTF8 propertyStr(nameSpaceURI);
+    propertyStr.Append(attrName);
 
     nsCOMPtr<nsIRDFResource> property;
     rv = gRDFService->GetResource(propertyStr, getter_AddRefs(property));
@@ -1330,12 +1276,13 @@ RDFContentSinkImpl::OpenMember(const PRUnichar* aName,
     // to whatever they've declared the standard RDF namespace to be.
     nsresult rv;
 
-    const char* nameSpaceURI;
-    nsCOMPtr<nsIAtom> tag;
-    ParseTagString(aName, &nameSpaceURI, getter_AddRefs(tag));
+    nsCOMPtr<nsIAtom> localName;
+    const nsDependentSubstring& nameSpaceURI =
+        SplitExpatName(aName, getter_AddRefs(localName));
 
-    if ((0 != PL_strcmp(nameSpaceURI, kRDFNameSpaceURI)) || (tag.get() != kLiAtom)) {
-        PR_LOG(gLog, PR_LOG_WARNING,
+    if (!nameSpaceURI.EqualsLiteral(RDF_NAMESPACE_URI) ||
+        localName != kLiAtom) {
+        PR_LOG(gLog, PR_LOG_ALWAYS,
                ("rdfxml: expected RDF:li at line %d",
                 -1)); // XXX pass in line number
 
@@ -1385,95 +1332,76 @@ RDFContentSinkImpl::OpenValue(const PRUnichar* aName, const PRUnichar** aAttribu
 }
 
 ////////////////////////////////////////////////////////////////////////
-// Qualified name resolution
-
-nsIAtom*
-RDFContentSinkImpl::CutNameSpacePrefix(const nsAString& aString,
-                                       nsIAtom** aTagAtom)
+// namespace resolution
+void
+RDFContentSinkImpl::RegisterNamespaces(const PRUnichar **aAttributes)
 {
-    nsIAtom* prefixAtom = nsnull;
-    NS_PRECONDITION(aTagAtom != nsnull, "null ptr");
-    *aTagAtom = nsnull;
-
-    nsAString::const_iterator start, end;
-    aString.BeginReading(start);
-    aString.EndReading(end);
-    nsAString::const_iterator colon(start);
-
-    if (FindCharInReadable(kNameSpaceSeparator, colon, end)) {
-        prefixAtom = NS_NewAtom(Substring(start, colon));
-        *aTagAtom = NS_NewAtom(Substring(++colon, end));
+    nsCOMPtr<nsIRDFXMLSink> sink = do_QueryInterface(mDataSource);
+    if (!sink) {
+        return;
+    }
+    NS_NAMED_LITERAL_STRING(xmlns, "http://www.w3.org/2000/xmlns/");
+    for (; *aAttributes; aAttributes += 2) {
+        // check the namespace
+        const PRUnichar* attr = aAttributes[0];
+        const PRUnichar* xmlnsP = xmlns.BeginReading();
+        while (*attr ==  *xmlnsP) {
+            ++attr;
+            ++xmlnsP;
+        }
+        if (*attr != 0xFFFF ||
+            xmlnsP != xmlns.EndReading()) {
+            continue;
+        }
+        // get the localname (or "xmlns" for the default namespace)
+        const PRUnichar* endLocal = ++attr;
+        while (*endLocal && *endLocal != 0xFFFF) {
+            ++endLocal;
+        }
+        nsDependentSubstring lname(attr, endLocal);
+        nsCOMPtr<nsIAtom> preferred = do_GetAtom(lname);
+        if (preferred == kXMLNSAtom) {
+            preferred = nsnull;
+        }
+        sink->AddNameSpace(preferred, nsDependentString(aAttributes[1]));
     }
-    else
-        *aTagAtom = NS_NewAtom(aString);
-    return prefixAtom;
 }
 
+////////////////////////////////////////////////////////////////////////
+// Qualified name resolution
 
-nsresult
-RDFContentSinkImpl::GetNameSpaceURI(nsIAtom* aPrefix, const char** aNameSpaceURI)
+const nsDependentSubstring
+RDFContentSinkImpl::SplitExpatName(const PRUnichar *aExpatName,
+                                   nsIAtom **aLocalName)
 {
-    // If we're trying to resolve a namespace ID from a prefix, then
-    // we'd better have some namespaces open. We don't assert here
-    // because the likelihood of bogus files is high, and it doesn't
-    // make sense to drop into the debugger.
-    for (NameSpaceEntry* ns = mNameSpaceStack; ns != nsnull; ns = ns->mNext) {
-        if (ns->mPrefix.get() == aPrefix) {
-            *aNameSpaceURI = ns->mNameSpaceURI;
-            return NS_OK;
+    /**
+     *  Expat can send the following:
+     *    localName
+     *    namespaceURI<separator>localName
+     *    namespaceURI<separator>localName<separator>prefix
+     *
+     *  and we use 0xFFFF for the <separator>.
+     *
+     */
+
+    const PRUnichar *uriEnd = aExpatName;
+    const PRUnichar *nameStart = aExpatName;
+    const PRUnichar *pos;
+    for (pos = aExpatName; *pos; ++pos) {
+        if (*pos == 0xFFFF) {
+            if (uriEnd != aExpatName) {
+                break;
+            }
+
+            uriEnd = pos;
+            nameStart = pos + 1;
         }
     }
 
-    // Couldn't find the namespace, probably because the prefix
-    // was never declared using an 'xmlns' decl.
-    *aNameSpaceURI = nsnull;
-
-#ifdef PR_LOGGING
-    if (PR_LOG_TEST(gLog, PR_LOG_WARNING)) {
-        const char* prefixStr;
-        if (aPrefix)
-            aPrefix->GetUTF8String(&prefixStr);
-
-        PR_LogPrint
-               ("rdfxml: undeclared namespace prefix '%s'",
-                prefixStr);
-
-    }
-#endif
-
-    return NS_ERROR_FAILURE;
-}
-
-nsresult
-RDFContentSinkImpl::ParseTagString(const PRUnichar* aTagName,
-                                   const char** aNameSpaceURI,
-                                   nsIAtom** aTag)
-{
-    // Split the fully-qualified name into a prefix and a tag part.
-    nsDependentString tagStr(aTagName);
-    nsCOMPtr<nsIAtom> prefix = getter_AddRefs(CutNameSpacePrefix(tagStr, aTag));
-
-    GetNameSpaceURI(prefix, aNameSpaceURI);
-    return NS_OK;
-}
-
-
-nsresult
-RDFContentSinkImpl::ParseAttributeString(const nsAString& aAttributeName,
-                                         const char** aNameSpaceURI,
-                                         nsIAtom** aAttribute)
-{
-    // Split the fully-qualified name into a prefix and a tag part.
-    nsCOMPtr<nsIAtom> prefix = getter_AddRefs(CutNameSpacePrefix(aAttributeName,
-                                                                 aAttribute));
-    if (prefix) {
-        GetNameSpaceURI(prefix, aNameSpaceURI);
-    }
-    else {
-        *aNameSpaceURI = nsnull;
-    }
-
-    return NS_OK;
+    const nsDependentSubstring& nameSpaceURI = Substring(aExpatName, uriEnd);
+    *aLocalName = NS_NewAtom(NS_ConvertUTF16toUTF8(nameStart,
+                                                   pos - nameStart));
+    return nameSpaceURI;
 }
 
 nsresult
@@ -1614,62 +1542,6 @@ RDFContentSinkImpl::PopContext(nsIRDFResource         *&aResource,
 }
  
 
-////////////////////////////////////////////////////////////////////////
-// Namespace management
-
-PRBool
-RDFContentSinkImpl::IsXMLNSDirective(const nsAString& aAttributeKey, nsIAtom** aPrefix)
-{
-    NS_NAMED_LITERAL_STRING(kNameSpaceDef, "xmlns");
-    const size_t kNameSpaceDef_Length = sizeof("xmlns") - 1;
-
-    // Look for `xmlns' at the start of the attribute name
-    if (!Substring(aAttributeKey, 0, kNameSpaceDef_Length).Equals(kNameSpaceDef))
-        return PR_FALSE;
-
-    PRInt32 prefixLen = aAttributeKey.Length() - kNameSpaceDef_Length - 1;
-    if (prefixLen <= 0) {
-        // they're setting the default namespace; leave `prefix'
-        // as nsnull.
-    }
-    else {
-        // make sure there's a `:' character
-        nsAString::const_iterator iter;
-        if (*aAttributeKey.BeginReading(iter).advance(kNameSpaceDef_Length)
-                != kNameSpaceSeparator)
-            return PR_FALSE;
-
-        // if the caller wants the prefix back, compute it for them.
-        if (aPrefix)
-            *aPrefix = NS_NewAtom(Substring(aAttributeKey,
-                                            kNameSpaceDef_Length + 1,
-                                            prefixLen));
-    }
-
-    return PR_TRUE;
-}
-
-nsresult
-RDFContentSinkImpl::PopNameSpaces()
-{
-    // Close a namespace scope by removing the topmost entries from
-    // the namespace stack.
-    PRInt32 top = mNameSpaceScopes.Count() - 1;
-    if (top < 0)
-        return NS_ERROR_UNEXPECTED; // XXX huh?
-
-    NameSpaceEntry* ns = NS_STATIC_CAST(NameSpaceEntry*, mNameSpaceScopes[top]);
-    mNameSpaceScopes.RemoveElementAt(top);
-
-    while (mNameSpaceStack && mNameSpaceStack != ns) {
-        NameSpaceEntry* doomed = mNameSpaceStack;
-        mNameSpaceStack = mNameSpaceStack->mNext;
-        delete doomed;
-    }
-
-    return NS_OK;
-}
-
 ////////////////////////////////////////////////////////////////////////
 
 nsresult
