@@ -42,14 +42,14 @@
 */
 
 #include "nsChromeProtocolHandler.h"
+#include "nsChromeRegistry.h"
 #include "nsCOMPtr.h"
 #include "nsContentCID.h"
+#include "nsThreadUtils.h"
 #include "nsCRT.h"
 #include "nsIChannel.h"
 #include "nsIChromeRegistry.h"
 #include "nsIComponentManager.h"
-#include "nsIEventQueue.h"
-#include "nsIEventQueueService.h"
 #include "nsIFastLoadService.h"
 #include "nsIFile.h"
 #include "nsIFileURL.h"
@@ -68,15 +68,14 @@
 #include "nsIXULPrototypeDocument.h"
 #endif
 #include "nsNetCID.h"
+#include "nsNetUtil.h"
+#include "nsAutoPtr.h"
 #include "nsXPIDLString.h"
 #include "nsString.h"
 #include "prlog.h"
 
 //----------------------------------------------------------------------
 
-static NS_DEFINE_CID(kEventQueueServiceCID,      NS_EVENTQUEUESERVICE_CID);
-static NS_DEFINE_CID(kIOServiceCID,              NS_IOSERVICE_CID);
-static NS_DEFINE_CID(kStandardURLCID,            NS_STANDARDURL_CID);
 #ifdef MOZ_XUL
 static NS_DEFINE_CID(kXULPrototypeCacheCID,      NS_XULPROTOTYPECACHE_CID);
 #endif
@@ -113,6 +112,7 @@ protected:
     virtual ~nsCachedChromeChannel();
 
     nsCOMPtr<nsIURI>            mURI;
+    nsCOMPtr<nsIURI>            mOriginalURI;
     nsCOMPtr<nsILoadGroup>      mLoadGroup;
     nsCOMPtr<nsIStreamListener> mListener;
     nsCOMPtr<nsISupports>       mContext;
@@ -120,17 +120,26 @@ protected:
     nsCOMPtr<nsISupports>       mOwner;
     nsresult                    mStatus;
 
-    struct LoadEvent {
-        PLEvent                mEvent;
-        nsCachedChromeChannel* mChannel;
+    typedef void (*LoadEventCallback)(nsCachedChromeChannel*);
+
+    struct LoadEvent : nsRunnable {
+        LoadEvent(nsCachedChromeChannel *chan, LoadEventCallback callback)
+            : mChannel(chan), mCallback(callback) {}
+
+        nsRefPtr<nsCachedChromeChannel> mChannel;
+        LoadEventCallback mCallback;
+
+        NS_IMETHOD Run() {
+            mCallback(mChannel);
+            return NS_OK;
+        }
     };
 
     static nsresult
-    PostLoadEvent(nsCachedChromeChannel* aChannel, PLHandleEventProc aHandler);
+    PostLoadEvent(nsCachedChromeChannel* aChannel, LoadEventCallback aCallback);
 
-    static void* PR_CALLBACK HandleStartLoadEvent(PLEvent* aEvent);
-    static void* PR_CALLBACK HandleStopLoadEvent(PLEvent* aEvent);
-    static void PR_CALLBACK DestroyLoadEvent(PLEvent* aEvent);
+    static void HandleStartLoadEvent(nsCachedChromeChannel* channel);
+    static void HandleStopLoadEvent(nsCachedChromeChannel* channel);
 
 #ifdef PR_LOGGING
     static PRLogModuleInfo* gLog;
@@ -144,7 +153,7 @@ public:
 
     // nsIRequest
     NS_IMETHOD GetName(nsACString &result) { return NS_ERROR_NOT_IMPLEMENTED; }
-    NS_IMETHOD IsPending(PRBool *_retval) { *_retval = PR_TRUE; return NS_OK; }
+    NS_IMETHOD IsPending(PRBool *_retval) { *_retval = (mListener != nsnull); return NS_OK; }
     NS_IMETHOD GetStatus(nsresult *status) { *status = mStatus; return NS_OK; }
     NS_IMETHOD Cancel(nsresult status)  { mStatus = status; return NS_OK; }
     NS_IMETHOD Suspend(void) { return NS_OK; }
@@ -185,7 +194,9 @@ nsCachedChromeChannel::Create(nsIURI* aURI, nsIChannel** aResult)
 
 
 nsCachedChromeChannel::nsCachedChromeChannel(nsIURI* aURI)
-    : mURI(aURI), mLoadGroup(nsnull), mLoadFlags (nsIRequest::LOAD_NORMAL), mStatus(NS_OK)
+    : mURI(aURI), 
+      mLoadFlags(nsIRequest::LOAD_NORMAL), 
+      mStatus(NS_OK)
 {
 #ifdef PR_LOGGING
     if (! gLog)
@@ -207,7 +218,7 @@ nsCachedChromeChannel::~nsCachedChromeChannel()
 NS_IMETHODIMP
 nsCachedChromeChannel::GetOriginalURI(nsIURI* *aOriginalURI)
 {
-    *aOriginalURI = mURI;
+    *aOriginalURI = mOriginalURI ? mOriginalURI : mURI;
     NS_ADDREF(*aOriginalURI);
     return NS_OK;
 }
@@ -215,11 +226,8 @@ nsCachedChromeChannel::GetOriginalURI(nsIURI* *aOriginalURI)
 NS_IMETHODIMP
 nsCachedChromeChannel::SetOriginalURI(nsIURI* aOriginalURI)
 {
-  // don't stp on a uri if we already have one there...this is a work around fix
-  // for Bug #34769.
-  if (!mURI)
-    mURI = aOriginalURI;
-  return NS_OK;
+    mOriginalURI = aOriginalURI;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -392,60 +400,26 @@ nsCachedChromeChannel::SetContentLength(PRInt32 aContentLength)
 
 nsresult
 nsCachedChromeChannel::PostLoadEvent(nsCachedChromeChannel* aChannel,
-                                     PLHandleEventProc aHandler)
+                                     LoadEventCallback aCallback)
 {
-    nsresult rv;
-
-    nsCOMPtr<nsIEventQueueService> svc = do_GetService(kEventQueueServiceCID, &rv);
-    if (NS_FAILED(rv)) return rv;
-
-    if (! svc)
-        return NS_ERROR_UNEXPECTED;
-
-    nsCOMPtr<nsIEventQueue> queue;
-    rv = svc->GetThreadEventQueue(NS_CURRENT_THREAD, getter_AddRefs(queue));
-    if (NS_FAILED(rv)) return rv;
-
-    if (! queue)
-        return NS_ERROR_UNEXPECTED;
-
-    LoadEvent* event = new LoadEvent;
+    nsCOMPtr<nsIRunnable> event = new LoadEvent(aChannel, aCallback);
     if (! event)
         return NS_ERROR_OUT_OF_MEMORY;
 
-    PL_InitEvent(NS_REINTERPRET_CAST(PLEvent*, event),
-                 nsnull,
-                 aHandler,
-                 DestroyLoadEvent);
-
-    event->mChannel = aChannel;
-    NS_ADDREF(event->mChannel);
-
-    rv = queue->EnterMonitor();
-    if (NS_SUCCEEDED(rv)) {
-        (void) queue->PostEvent(NS_REINTERPRET_CAST(PLEvent*, event));
-        (void) queue->ExitMonitor();
-        return NS_OK;
-    }
-
-    // If we get here, something bad happened. Clean up.
-    NS_RELEASE(event->mChannel);
-    delete event;
-    return rv;
+    return NS_DispatchToCurrentThread(event);
 }
 
-void* PR_CALLBACK
-nsCachedChromeChannel::HandleStartLoadEvent(PLEvent* aEvent)
+/*static*/ void
+nsCachedChromeChannel::HandleStartLoadEvent(nsCachedChromeChannel *channel)
 {
     // Fire the OnStartRequest() for the cached chrome channel, then
     // queue another event to trigger the OnStopRequest()...
-    LoadEvent* event = NS_REINTERPRET_CAST(LoadEvent*, aEvent);
-    nsCachedChromeChannel* channel = event->mChannel;
 
     // If the load has been cancelled, then just bail now. We won't
     // send On[Start|Stop]Request().
+    // XXX(darin): this violates the nsIChannel API
     if (NS_FAILED(channel->mStatus))
-      return nsnull;
+      return;
 
     PR_LOG(gLog, PR_LOG_DEBUG,
               ("nsCachedChromeChannel[%p]: firing OnStartRequest for %p",
@@ -453,25 +427,20 @@ nsCachedChromeChannel::HandleStartLoadEvent(PLEvent* aEvent)
 
     (void) channel->mListener->OnStartRequest(channel, channel->mContext);
     (void) PostLoadEvent(channel, HandleStopLoadEvent);
-    return nsnull;
 }
 
 
-void* PR_CALLBACK
-nsCachedChromeChannel::HandleStopLoadEvent(PLEvent* aEvent)
+/*static*/ void
+nsCachedChromeChannel::HandleStopLoadEvent(nsCachedChromeChannel* channel)
 {
     // Fire the OnStopRequest() for the cached chrome channel, and
     // remove it from the load group.
-    LoadEvent* event = NS_REINTERPRET_CAST(LoadEvent*, aEvent);
-    nsCachedChromeChannel* channel = event->mChannel;
-    nsIRequest* request = NS_REINTERPRET_CAST(nsIRequest*, channel);
-
 
     PR_LOG(gLog, PR_LOG_DEBUG,
            ("nsCachedChromeChannel[%p]: firing OnStopRequest for %p",
             channel, channel->mListener.get()));
 
-    (void) channel->mListener->OnStopRequest(request, channel->mContext,
+    (void) channel->mListener->OnStopRequest(channel, channel->mContext,
                                              channel->mStatus);
 
     if (channel->mLoadGroup) {
@@ -479,21 +448,11 @@ nsCachedChromeChannel::HandleStopLoadEvent(PLEvent* aEvent)
                ("nsCachedChromeChannel[%p]: removing self from load group %p",
                 channel, channel->mLoadGroup.get()));
 
-        (void) channel->mLoadGroup->RemoveRequest(request, nsnull, NS_OK);
+        (void) channel->mLoadGroup->RemoveRequest(channel, nsnull, NS_OK);
     }
 
     channel->mListener = nsnull;
     channel->mContext  = nsnull;
-
-    return nsnull;
-}
-
-void PR_CALLBACK
-nsCachedChromeChannel::DestroyLoadEvent(PLEvent* aEvent)
-{
-    LoadEvent* event = NS_REINTERPRET_CAST(LoadEvent*, aEvent);
-    NS_RELEASE(event->mChannel);
-    delete event;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -579,7 +538,7 @@ nsChromeProtocolHandler::NewURI(const nsACString &aSpec,
     // Chrome: URLs (currently) have no additional structure beyond that provided
     // by standard URLs, so there is no "outer" given to CreateInstance
 
-    nsCOMPtr<nsIStandardURL> url(do_CreateInstance(kStandardURLCID, &rv));
+    nsCOMPtr<nsIStandardURL> url(do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv));
     if (NS_FAILED(rv))
         return rv;
 
@@ -594,21 +553,7 @@ nsChromeProtocolHandler::NewURI(const nsACString &aSpec,
     // Canonify the "chrome:" URL; e.g., so that we collapse
     // "chrome://navigator/content/" and "chrome://navigator/content"
     // and "chrome://navigator/content/navigator.xul".
-
-    // Try the global cache first.
-    nsCOMPtr<nsIChromeRegistry> reg = gChromeRegistry;
-
-    // If that fails, the service has not been instantiated yet; let's
-    // do that now.
-    if (!reg) {
-        reg = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
-        if (NS_FAILED(rv))
-            return rv;
-    }
-
-    NS_ASSERTION(reg, "Must have a chrome registry by now");
-    
-    rv = reg->Canonify(uri);
+    rv = nsChromeRegistry::Canonify(uri);
     if (NS_FAILED(rv))
         return rv;
 
@@ -632,7 +577,7 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
         nsCOMPtr<nsIURI> debugClone;
         debug_rv = aURI->Clone(getter_AddRefs(debugClone));
         if (NS_SUCCEEDED(debug_rv)) {
-            debug_rv = debugReg->Canonify(debugClone);
+            debug_rv = nsChromeRegistry::Canonify(debugClone);
             if (NS_SUCCEEDED(debug_rv)) {
                 PRBool same;
                 debug_rv = aURI->Equals(debugClone, &same);
@@ -695,17 +640,13 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
             if (NS_FAILED(rv)) return rv;
         }
 
-        nsCAutoString spec;
-        rv = reg->ConvertChromeURL(aURI, spec);
-        if (NS_FAILED(rv)) return rv;
-
-        nsCOMPtr<nsIIOService> ioServ(do_GetService(kIOServiceCID, &rv));
-        if (NS_FAILED(rv)) return rv;
-
         nsCOMPtr<nsIURI> chromeURI;
-        rv = ioServ->NewURI(spec, nsnull, nsnull, getter_AddRefs(chromeURI));
+        rv = reg->ConvertChromeURL(aURI, getter_AddRefs(chromeURI));
         if (NS_FAILED(rv)) return rv;
 
+        nsCOMPtr<nsIIOService> ioServ (do_GetIOService());
+        if (!ioServ) return NS_ERROR_FAILURE;
+
         rv = ioServ->NewChannelFromURI(chromeURI, getter_AddRefs(result));
         if (NS_FAILED(rv)) return rv;
 
@@ -726,14 +667,12 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
         rv = result->SetOriginalURI(aURI);
         if (NS_FAILED(rv)) return rv;
 
-        // Get a system principal for xul files and set the owner
+        // Get a system principal for content files and set the owner
         // property of the result
         nsCOMPtr<nsIURL> url = do_QueryInterface(aURI);
-        nsCAutoString fileExtension;
-        rv = url->GetFileExtension(fileExtension);
-        if (PL_strcasecmp(fileExtension.get(), "xul") == 0 ||
-            PL_strcasecmp(fileExtension.get(), "html") == 0 ||
-            PL_strcasecmp(fileExtension.get(), "xml") == 0)
+        nsCAutoString path;
+        rv = url->GetPath(path);
+        if (StringBeginsWith(path, NS_LITERAL_CSTRING("/content/")))
         {
             nsCOMPtr<nsIScriptSecurityManager> securityManager =
                      do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
