@@ -55,15 +55,14 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIServiceManager.h"
-#include "nsIScriptGlobalObject.h"
 #include "nsIObserverService.h"
-#include "nsIDocumentLoader.h"
 #include "nsCURILoader.h"
 #include "nsIDocShell.h"
 #include "nsIDocumentViewer.h"
 #include "nsIDocument.h"
+#include "nsIPrincipal.h"
 #include "nsIDOMElement.h"
-#include "nsIDOMWindowInternal.h"
+#include "nsPIDOMWindow.h"
 #include "nsIContent.h"
 #include "nsIWebProgress.h"
 #include "nsIChannel.h"
@@ -78,9 +77,11 @@
 #include "nsIFormSubmitObserver.h"
 #include "nsISecurityWarningDialogs.h"
 #include "nsIProxyObjectManager.h"
+#include "nsThreadUtils.h"
+#include "nsNetUtil.h"
 #include "nsCRT.h"
 
-#define SECURITY_STRING_BUNDLE_URL "chrome://communicator/locale/security.properties"
+#define SECURITY_STRING_BUNDLE_URL "chrome://pipnss/locale/security.properties"
 
 #define IS_SECURE(state) ((state & 0xFFFF) == STATE_IS_SECURE)
 
@@ -176,17 +177,32 @@ NS_IMETHODIMP
 nsSecureBrowserUIImpl::Init(nsIDOMWindow *window)
 {
   PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-         ("SecureUI:%p: Init\n", this));
+         ("SecureUI:%p: Init: mWindow: %p, window: %p\n", this, mWindow.get(),
+          window));
 
+  if (!window) {
+    NS_WARNING("Null window passed to nsSecureBrowserUIImpl::Init()");
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  if (mWindow) {
+    NS_WARNING("Trying to init an nsSecureBrowserUIImpl twice");
+    return NS_ERROR_ALREADY_INITIALIZED;
+  }
+  
   nsresult rv = NS_OK;
   mWindow = window;
 
   nsCOMPtr<nsIStringBundleService> service(do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv));
   if (NS_FAILED(rv)) return rv;
   
-  rv = service->CreateBundle(SECURITY_STRING_BUNDLE_URL,
-                             getter_AddRefs(mStringBundle));
-  if (NS_FAILED(rv)) return rv;
+  // We do not need to test for mStringBundle here...
+  // Anywhere we use it, we will test before using.  Some
+  // embedded users of PSM may want to reuse our
+  // nsSecureBrowserUIImpl implementation without the
+  // bundle.
+  service->CreateBundle(SECURITY_STRING_BUNDLE_URL, getter_AddRefs(mStringBundle));
+  
   
   // hook up to the form post notifications:
   nsCOMPtr<nsIObserverService> svc(do_GetService("@mozilla.org/observer-service;1", &rv));
@@ -194,12 +210,20 @@ nsSecureBrowserUIImpl::Init(nsIDOMWindow *window)
     rv = svc->AddObserver(this, NS_FORMSUBMIT_SUBJECT, PR_TRUE);
   }
   
+  nsCOMPtr<nsPIDOMWindow> piwindow(do_QueryInterface(mWindow));
+  if (!piwindow) return NS_ERROR_FAILURE;
+
+  nsIDocShell *docShell = piwindow->GetDocShell();
+
+  // The Docshell will own the SecureBrowserUI object
+  if (!docShell)
+    return NS_ERROR_FAILURE;
+
+  docShell->SetSecurityUI(this);
+
   /* GetWebProgress(mWindow) */
   // hook up to the webprogress notifications.
-  nsCOMPtr<nsIScriptGlobalObject> sgo(do_QueryInterface(mWindow));
-  if (!sgo) return NS_ERROR_FAILURE;
-  
-  nsCOMPtr<nsIWebProgress> wp(do_GetInterface(sgo->GetDocShell()));
+  nsCOMPtr<nsIWebProgress> wp(do_GetInterface(docShell));
   if (!wp) return NS_ERROR_FAILURE;
   /* end GetWebProgress */
   
@@ -248,7 +272,12 @@ nsSecureBrowserUIImpl::GetState(PRUint32* aState)
 NS_IMETHODIMP
 nsSecureBrowserUIImpl::GetTooltipText(nsAString& aText)
 {
-  if (!mInfoTooltip.IsEmpty())
+  if (mPreviousSecurityState == lis_mixed_security)
+  {
+    GetBundleString(NS_LITERAL_STRING("SecurityButtonMixedContentTooltipText").get(),
+                    aText);
+  }
+  else if (!mInfoTooltip.IsEmpty())
   {
     aText = mInfoTooltip;
   }
@@ -331,10 +360,31 @@ nsSecureBrowserUIImpl::Notify(nsIContent* formNode,
   nsCOMPtr<nsIDocument> document = formNode->GetDocument();
   if (!document) return NS_OK;
 
-  nsIURI *formURL = document->GetBaseURI();
-
-  nsCOMPtr<nsIDOMWindow> postingWindow(do_QueryInterface(document->GetScriptGlobalObject()));
+  nsIPrincipal *principal = formNode->NodePrincipal();
   
+  if (!principal)
+  {
+    *cancelSubmit = PR_TRUE;
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIURI> formURL;
+  if (NS_FAILED(principal->GetURI(getter_AddRefs(formURL))) ||
+      !formURL)
+  {
+    formURL = document->GetDocumentURI();
+  }
+
+  nsCOMPtr<nsIDOMWindow> postingWindow =
+    do_QueryInterface(document->GetWindow());
+  // We can't find this document's window, cancel it.
+  if (!postingWindow)
+  {
+    NS_WARNING("If you see this and can explain why it should be allowed, note in Bug 332324");
+    *cancelSubmit = PR_TRUE;
+    return NS_OK;
+  }
+
   PRBool isChild;
   IsChildOfDomWindow(mWindow, postingWindow, &isChild);
   
@@ -380,6 +430,79 @@ void nsSecureBrowserUIImpl::ResetStateTracking()
                     sizeof(RequestHashEntry), 16);
 }
 
+nsresult
+nsSecureBrowserUIImpl::EvaluateAndUpdateSecurityState(nsIRequest *aRequest)
+{
+  nsCOMPtr<nsIChannel> channel(do_QueryInterface(aRequest));
+
+  if (!channel) {
+    mNewToplevelSecurityState = nsIWebProgressListener::STATE_IS_INSECURE;
+  } else {
+    mNewToplevelSecurityState = GetSecurityStateFromChannel(channel);
+
+    PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+           ("SecureUI:%p: OnStateChange: remember mNewToplevelSecurityState => %x\n", this,
+            mNewToplevelSecurityState));
+
+    // Get SSL Status information if possible
+    nsCOMPtr<nsISupports> info;
+    channel->GetSecurityInfo(getter_AddRefs(info));
+    nsCOMPtr<nsISSLStatusProvider> sp = do_QueryInterface(info);
+    if (sp) {
+      // Ignore result
+      sp->GetSSLStatus(getter_AddRefs(mSSLStatus));
+    }
+
+    if (info) {
+      nsCOMPtr<nsITransportSecurityInfo> secInfo(do_QueryInterface(info));
+      if (secInfo) {
+        secInfo->GetShortSecurityDescription(getter_Copies(mInfoTooltip));
+      }
+    }
+  }
+
+  // assume mNewToplevelSecurityState was set in this scope!
+  // see code that is directly above
+
+  mNewToplevelSecurityStateKnown = PR_TRUE;
+  return UpdateSecurityState(aRequest);
+}
+
+void
+nsSecureBrowserUIImpl::UpdateSubrequestMembers(nsIRequest *aRequest)
+{
+  // For wyciwyg channels in subdocuments we only update our
+  // subrequest state members.
+  PRUint32 reqState = nsIWebProgressListener::STATE_IS_INSECURE;
+  nsCOMPtr<nsIChannel> channel(do_QueryInterface(aRequest));
+
+  if (channel) {
+    reqState = GetSecurityStateFromChannel(channel);
+  }
+
+  if (reqState & STATE_IS_SECURE) {
+    if (reqState & STATE_SECURE_LOW || reqState & STATE_SECURE_MED) {
+      PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+             ("SecureUI:%p: OnStateChange: subreq LOW\n", this));
+      ++mSubRequestsLowSecurity;
+    } else {
+      PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+             ("SecureUI:%p: OnStateChange: subreq HIGH\n", this));
+      ++mSubRequestsHighSecurity;
+    }
+  } else if (reqState & STATE_IS_BROKEN) {
+    PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+           ("SecureUI:%p: OnStateChange: subreq BROKEN\n", this));
+    ++mSubRequestsBrokenSecurity;
+  } else {
+    PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+           ("SecureUI:%p: OnStateChange: subreq INSECURE\n", this));
+    ++mSubRequestsNoSecurity;
+  }
+}
+
+
+
 NS_IMETHODIMP
 nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
                                      nsIRequest* aRequest,
@@ -516,11 +639,13 @@ nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
   }
 
 #ifdef PR_LOGGING
-  nsXPIDLCString reqname;
-  aRequest->GetName(reqname);
-  PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-         ("SecureUI:%p: %p %p OnStateChange %x %s\n", this, aWebProgress, aRequest,
-            aProgressStateFlags, reqname.get()));
+  if (PR_LOG_TEST(gSecureDocLog, PR_LOG_DEBUG)) {
+    nsXPIDLCString reqname;
+    aRequest->GetName(reqname);
+    PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+           ("SecureUI:%p: %p %p OnStateChange %x %s\n", this, aWebProgress,
+            aRequest, aProgressStateFlags, reqname.get()));
+  }
 #endif
 
   nsCOMPtr<nsIChannel> channel(do_QueryInterface(aRequest));
@@ -793,50 +918,11 @@ nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
 
     --mDocumentRequestsInProgress;
 
-    {
-      PRBool MustEvaluate = PR_TRUE;
+    if (requestHasTransferedData) {
+      // Data has been transferred for the single toplevel
+      // request. Evaluate the security state.
 
-      if (!requestHasTransferedData)
-      {
-        // No data has been transfered for the single toplevel request.
-        MustEvaluate = PR_FALSE;
-      }
-
-      if (MustEvaluate)
-      {
-        if (channel) {
-          mNewToplevelSecurityState = GetSecurityStateFromChannel(channel);
-
-          PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-                 ("SecureUI:%p: OnStateChange: remember mNewToplevelSecurityState => %x\n", this,
-                  mNewToplevelSecurityState));
-
-          // Get SSL Status information if possible
-          nsCOMPtr<nsISupports> info;
-          channel->GetSecurityInfo(getter_AddRefs(info));
-          nsCOMPtr<nsISSLStatusProvider> sp = do_QueryInterface(info);
-          if (sp) {
-            // Ignore result
-            sp->GetSSLStatus(getter_AddRefs(mSSLStatus));
-          }
-
-          if (info) {
-            nsCOMPtr<nsITransportSecurityInfo> secInfo(do_QueryInterface(info));
-            if (secInfo) {
-                secInfo->GetShortSecurityDescription(getter_Copies(mInfoTooltip));
-            }
-          }
-        }
-        else {
-          mNewToplevelSecurityState = nsIWebProgressListener::STATE_IS_INSECURE;
-        }
-
-        // assume mNewToplevelSecurityState was set in this scope!
-        // see code that is directly above
-        
-        mNewToplevelSecurityStateKnown = PR_TRUE;
-        return UpdateSecurityState(aRequest);
-      }
+      return EvaluateAndUpdateSecurityState(aRequest);
     }
     
     return NS_OK;
@@ -855,39 +941,7 @@ nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
 
     if (requestHasTransferedData)
     {  
-      PRUint32 reqState = nsIWebProgressListener::STATE_IS_INSECURE;
-
-      if (channel) {
-        reqState = GetSecurityStateFromChannel(channel);
-      }
-
-      if (reqState & STATE_IS_SECURE)
-      {
-        if (reqState & STATE_SECURE_LOW || reqState & STATE_SECURE_MED)
-        {
-          PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-           ("SecureUI:%p: OnStateChange: subreq LOW\n", this));
-          ++mSubRequestsLowSecurity;
-        }
-        else
-        {
-          PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-           ("SecureUI:%p: OnStateChange: subreq HIGH\n", this));
-          ++mSubRequestsHighSecurity;
-        }
-      }
-      else if (reqState & STATE_IS_BROKEN)
-      {
-        PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-         ("SecureUI:%p: OnStateChange: subreq BROKEN\n", this));
-        ++ mSubRequestsBrokenSecurity;
-      }
-      else
-      {
-        PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-         ("SecureUI:%p: OnStateChange: subreq INSECURE\n", this));
-        ++mSubRequestsNoSecurity;
-      }
+      UpdateSubrequestMembers(aRequest);
       
       // Care for the following scenario:
       // A new top level document load might have already started,
@@ -912,19 +966,16 @@ nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
 void nsSecureBrowserUIImpl::ObtainEventSink(nsIChannel *channel)
 {
   if (!mToplevelEventSink)
-  {
-    nsCOMPtr<nsIInterfaceRequestor> requestor;
-    channel->GetNotificationCallbacks(getter_AddRefs(requestor));
-    if (requestor)
-      mToplevelEventSink = do_GetInterface(requestor);
-  }
-
+    NS_QueryNotificationCallbacks(channel, mToplevelEventSink);
 }
 
 nsresult nsSecureBrowserUIImpl::UpdateSecurityState(nsIRequest* aRequest)
 {
   lockIconState newSecurityState;
 
+  PRBool showWarning = PR_FALSE;
+  lockIconState warnSecurityState;
+
   if (mNewToplevelSecurityState & STATE_IS_SECURE)
   {
     if (mNewToplevelSecurityState & STATE_SECURE_LOW
@@ -1018,7 +1069,7 @@ nsresult nsSecureBrowserUIImpl::UpdateSecurityState(nsIRequest* aRequest)
       high        high
     */
 
-    PRBool showWarning = PR_TRUE;
+    showWarning = PR_TRUE;
     
     switch (mPreviousSecurityState)
     {
@@ -1038,30 +1089,12 @@ nsresult nsSecureBrowserUIImpl::UpdateSecurityState(nsIRequest* aRequest)
       default:
         break;
     }
-    
+
     if (showWarning)
     {
-      switch (newSecurityState)
-      {
-        case lis_no_security:
-        case lis_broken_security:
-          ConfirmLeavingSecure();
-          break;
-
-        case lis_mixed_security:
-          ConfirmMixedMode();
-          break;
-
-        case lis_low_security:
-          ConfirmEnteringWeak();
-          break;
-
-        case lis_high_security:
-          ConfirmEnteringSecure();
-          break;
-      }
+      warnSecurityState = newSecurityState;
     }
-
+    
     mPreviousSecurityState = newSecurityState;
 
     if (lis_no_security == newSecurityState)
@@ -1114,6 +1147,29 @@ nsresult nsSecureBrowserUIImpl::UpdateSecurityState(nsIRequest* aRequest)
 
   }
 
+  if (showWarning)
+  {
+    switch (warnSecurityState)
+    {
+      case lis_no_security:
+      case lis_broken_security:
+        ConfirmLeavingSecure();
+        break;
+
+      case lis_mixed_security:
+        ConfirmMixedMode();
+        break;
+
+      case lis_low_security:
+        ConfirmEnteringWeak();
+        break;
+
+      case lis_high_security:
+        ConfirmEnteringSecure();
+        break;
+    }
+  }
+
   return NS_OK; 
 }
 
@@ -1139,6 +1195,44 @@ nsSecureBrowserUIImpl::OnLocationChange(nsIWebProgress* aWebProgress,
 
   mCurrentURI = aLocation;
 
+  // If the location change does not have a corresponding request, then we
+  // assume that it does not impact the security state.
+  if (!aRequest)
+    return NS_OK;
+
+  // The location bar has changed, so we must update the security state.  The
+  // only concern with doing this here is that a page may transition from being
+  // reported as completely secure to being reported as partially secure
+  // (mixed).  This may be confusing for users, and it may bother users who
+  // like seeing security dialogs.  However, it seems prudent given that page
+  // loading may never end in some edge cases (perhaps by a site with malicious
+  // intent).
+
+  nsCOMPtr<nsIDOMWindow> windowForProgress;
+  aWebProgress->GetDOMWindow(getter_AddRefs(windowForProgress));
+
+  if (windowForProgress.get() == mWindow.get()) {
+    // For toplevel channels, update the security state right away.
+    return EvaluateAndUpdateSecurityState(aRequest);
+  }
+
+  // For channels in subdocuments we only update our subrequest state members.
+  UpdateSubrequestMembers(aRequest);
+
+  // Care for the following scenario:
+
+  // A new toplevel document load might have already started, but the security
+  // state of the new toplevel document might not yet be known.
+  // 
+  // At this point, we are learning about the security state of a sub-document.
+  // We must not update the security state based on the sub content, if the new
+  // top level state is not yet known.
+  //
+  // We skip updating the security state in this case.
+
+  if (mNewToplevelSecurityStateKnown)
+    return UpdateSecurityState(aRequest);
+
   return NS_OK;
 }
 
@@ -1181,7 +1275,7 @@ nsSecureBrowserUIImpl::OnSecurityChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP
 nsSecureBrowserUIImpl::GetSSLStatus(nsISupports** _result)
 {
-  NS_ASSERTION(_result, "non-NULL destination required");
+  NS_ENSURE_ARG_POINTER(_result);
 
   *_result = mSSLStatus;
   NS_IF_ADDREF(*_result);
@@ -1192,14 +1286,25 @@ nsSecureBrowserUIImpl::GetSSLStatus(nsISupports** _result)
 nsresult
 nsSecureBrowserUIImpl::IsURLHTTPS(nsIURI* aURL, PRBool* value)
 {
-	*value = PR_FALSE;
+  *value = PR_FALSE;
 
-	if (!aURL)
-		return NS_OK;
+  if (!aURL)
+    return NS_OK;
 
   return aURL->SchemeIs("https", value);
 }
 
+nsresult
+nsSecureBrowserUIImpl::IsURLJavaScript(nsIURI* aURL, PRBool* value)
+{
+  *value = PR_FALSE;
+
+  if (!aURL)
+    return NS_OK;
+
+  return aURL->SchemeIs("javascript", value);
+}
+
 void
 nsSecureBrowserUIImpl::GetBundleString(const PRUnichar* name,
                                        nsAString &outString)
@@ -1211,9 +1316,9 @@ nsSecureBrowserUIImpl::GetBundleString(const PRUnichar* name,
       outString = ptrv;
     else
       outString.SetLength(0);
-    
+
     nsMemory::Free(ptrv);
-    
+
   } else {
     outString.SetLength(0);
   }
@@ -1222,7 +1327,7 @@ nsSecureBrowserUIImpl::GetBundleString(const PRUnichar* name,
 nsresult
 nsSecureBrowserUIImpl::CheckPost(nsIURI *formURL, nsIURI *actionURL, PRBool *okayToPost)
 {
-  PRBool formSecure,actionSecure;
+  PRBool formSecure, actionSecure, actionJavaScript;
   *okayToPost = PR_TRUE;
 
   nsresult rv = IsURLHTTPS(formURL, &formSecure);
@@ -1232,21 +1337,30 @@ nsSecureBrowserUIImpl::CheckPost(nsIURI *formURL, nsIURI *actionURL, PRBool *oka
   rv = IsURLHTTPS(actionURL, &actionSecure);
   if (NS_FAILED(rv))
     return rv;
-  
+
+  rv = IsURLJavaScript(actionURL, &actionJavaScript);
+  if (NS_FAILED(rv))
+    return rv;
+
   // If we are posting to a secure link, all is okay.
   // It doesn't matter whether the currently viewed page is secure or not,
   // because the data will be sent to a secure URL.
   if (actionSecure) {
     return NS_OK;
   }
-    
+
+  // Action is a JavaScript call, not an actual post. That's okay too.
+  if (actionJavaScript) {
+    return NS_OK;
+  }
+
   // posting to insecure webpage from a secure webpage.
   if (formSecure) {
     *okayToPost = ConfirmPostToInsecureFromSecure();
   } else {
     *okayToPost = ConfirmPostToInsecure();
   }
-  
+
   return NS_OK;
 }
 
@@ -1291,6 +1405,10 @@ NS_IMETHODIMP nsUIContext::GetInterface(const nsIID & uuid, void * *result)
 
     rv = internal->GetPrompter(&prompt);
     *result = prompt;
+  } else if (uuid.Equals(NS_GET_IID(nsIDOMWindow))) {
+    *result = mWindow;
+    NS_ADDREF ((nsISupports*) *result);
+    rv = NS_OK;
   } else {
     rv = NS_ERROR_NO_INTERFACE;
   }
@@ -1307,15 +1425,11 @@ GetNSSDialogs(nsISecurityWarningDialogs **result)
   if (NS_FAILED(rv)) 
     return rv;
 
-  nsCOMPtr<nsIProxyObjectManager> proxyman(do_GetService(NS_XPCOMPROXY_CONTRACTID));
-  if (!proxyman)
-    return NS_ERROR_FAILURE;
-
   nsCOMPtr<nsISupports> proxiedResult;
-  proxyman->GetProxyForObject(NS_UI_THREAD_EVENTQ,
-                              NS_GET_IID(nsISecurityWarningDialogs),
-                              my_result, PROXY_SYNC,
-                              getter_AddRefs(proxiedResult));
+  NS_GetProxyForObject(NS_PROXY_TO_MAIN_THREAD,
+                       NS_GET_IID(nsISecurityWarningDialogs),
+                       my_result, NS_PROXY_SYNC,
+                       getter_AddRefs(proxiedResult));
 
   if (!proxiedResult) {
     return NS_ERROR_FAILURE;
