@@ -43,9 +43,14 @@
 #include "nsIX509CertDB.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
+#include "nsUnicharUtils.h"
 #include "nsNSSCertificate.h"
 #include "nsNSSCertHelper.h"
 #include "nsINSSCertCache.h"
+#include "nsIMutableArray.h"
+#include "nsArrayUtils.h"
+#include "nsISupportsPrimitives.h"
+#include "nsXPCOMCID.h"
 
 #include "prlog.h"
 #ifdef PR_LOGGING
@@ -66,6 +71,18 @@ struct treeArrayElStr {
   PRInt32    numChildren; /* number of chidren (certs) for thread */
 };
 
+CompareCacheHashEntryPtr::CompareCacheHashEntryPtr()
+{
+  entry = new CompareCacheHashEntry;
+}
+
+CompareCacheHashEntryPtr::~CompareCacheHashEntryPtr()
+{
+  if (entry) {
+    delete entry;
+  }
+}
+
 CompareCacheHashEntry::CompareCacheHashEntry()
 :key(nsnull)
 {
@@ -77,33 +94,36 @@ CompareCacheHashEntry::CompareCacheHashEntry()
 PR_STATIC_CALLBACK(const void *)
 CompareCacheGetKey(PLDHashTable *table, PLDHashEntryHdr *hdr)
 {
-  CompareCacheHashEntry *entry = NS_STATIC_CAST(CompareCacheHashEntry*, hdr);
-  return entry->key;
+  CompareCacheHashEntryPtr *entryPtr = NS_STATIC_CAST(CompareCacheHashEntryPtr*, hdr);
+  return entryPtr->entry->key;
 }
 
 PR_STATIC_CALLBACK(PRBool)
 CompareCacheMatchEntry(PLDHashTable *table, const PLDHashEntryHdr *hdr,
                          const void *key)
 {
-  const CompareCacheHashEntry *entry = NS_STATIC_CAST(const CompareCacheHashEntry*, hdr);
-  return entry->key == key;
+  const CompareCacheHashEntryPtr *entryPtr = NS_STATIC_CAST(const CompareCacheHashEntryPtr*, hdr);
+  return entryPtr->entry->key == key;
 }
 
 PR_STATIC_CALLBACK(PRBool)
 CompareCacheInitEntry(PLDHashTable *table, PLDHashEntryHdr *hdr,
                      const void *key)
 {
-  new (hdr) CompareCacheHashEntry();
-  CompareCacheHashEntry *entry = NS_STATIC_CAST(CompareCacheHashEntry*, hdr);
-  entry->key = (void*)key;
+  new (hdr) CompareCacheHashEntryPtr();
+  CompareCacheHashEntryPtr *entryPtr = NS_STATIC_CAST(CompareCacheHashEntryPtr*, hdr);
+  if (!entryPtr->entry) {
+    return PR_FALSE;
+  }
+  entryPtr->entry->key = (void*)key;
   return PR_TRUE;
 }
 
 PR_STATIC_CALLBACK(void)
 CompareCacheClearEntry(PLDHashTable *table, PLDHashEntryHdr *hdr)
 {
-  CompareCacheHashEntry *entry = NS_STATIC_CAST(CompareCacheHashEntry*, hdr);
-  entry->~CompareCacheHashEntry();
+  CompareCacheHashEntryPtr *entryPtr = NS_STATIC_CAST(CompareCacheHashEntryPtr*, hdr);
+  entryPtr->~CompareCacheHashEntryPtr();
 }
 
 static PLDHashTableOps gMapOps = {
@@ -126,6 +146,7 @@ nsCertTree::nsCertTree() : mTreeArray(NULL)
 {
   mCompareCache.ops = nsnull;
   mNSSComponent = do_GetService(kNSSComponentCID);
+  mCellText = nsnull;
 }
 
 void nsCertTree::ClearCompareHash()
@@ -140,7 +161,7 @@ nsresult nsCertTree::InitCompareHash()
 {
   ClearCompareHash();
   if (!PL_DHashTableInit(&mCompareCache, &gMapOps, nsnull,
-                         sizeof(CompareCacheHashEntry), 128)) {
+                         sizeof(CompareCacheHashEntryPtr), 128)) {
     mCompareCache.ops = nsnull;
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -176,9 +197,10 @@ CompareCacheHashEntry *
 nsCertTree::getCacheEntry(void *cache, void *aCert)
 {
   PLDHashTable &aCompareCache = *NS_REINTERPRET_CAST(PLDHashTable*, cache);
-  return NS_STATIC_CAST(CompareCacheHashEntry*,
-                        PL_DHashTableOperate(&aCompareCache, aCert,
-                                             PL_DHASH_ADD));
+  CompareCacheHashEntryPtr *entryPtr = 
+    NS_STATIC_CAST(CompareCacheHashEntryPtr*,
+                   PL_DHashTableOperate(&aCompareCache, aCert, PL_DHASH_ADD));
+  return entryPtr ? entryPtr->entry : NULL;
 }
 
 void nsCertTree::RemoveCacheEntry(void *key)
@@ -270,6 +292,7 @@ nsCertTree::nsCertCompareFunc
 nsCertTree::GetCompareFuncFromCertType(PRUint32 aType)
 {
   switch (aType) {
+    case nsIX509Cert2::ANY_CERT:
     case nsIX509Cert::USER_CERT:
       return CmpUserCert;
     case nsIX509Cert::CA_CERT:
@@ -296,24 +319,22 @@ nsCertTree::GetCertsByTypeFromCertList(CERTCertList *aCertList,
   nsresult rv = NS_NewISupportsArray(getter_AddRefs(certarray));
   if (NS_FAILED(rv)) return PR_FALSE;
   CERTCertListNode *node;
-  int i, count = 0;
+  int count = 0;
   for (node = CERT_LIST_HEAD(aCertList);
        !CERT_LIST_END(node, aCertList);
        node = CERT_LIST_NEXT(node)) {
-    if (getCertType(node->cert) == aType) {
+    if (aType == nsIX509Cert2::ANY_CERT || getCertType(node->cert) == aType) {
       nsCOMPtr<nsIX509Cert> pipCert = new nsNSSCertificate(node->cert);
       if (pipCert) {
-        for (i=0; i<count; i++) {
-          nsCOMPtr<nsISupports> isupport = 
-            dont_AddRef(certarray->ElementAt(i));
-          nsCOMPtr<nsIX509Cert> cert = do_QueryInterface(isupport);
+        int i;
+        for (i = 0; i < count; ++i) {
+          nsCOMPtr<nsIX509Cert> cert = do_QueryElementAt(certarray, i);
           if ((*aCertCmpFn)(aCertCmpFnArg, pipCert, cert) < 0) {
-            certarray->InsertElementAt(pipCert, i);
             break;
           }
         }
-        if (i == count) certarray->AppendElement(pipCert);
-          count++;
+        certarray->InsertElementAt(pipCert, i);
+        ++count;
       }
     }
   }
@@ -405,6 +426,8 @@ nsCertTree::UpdateUIContents()
   if (!mTreeArray)
     return NS_ERROR_OUT_OF_MEMORY;
 
+  mCellText = do_CreateInstance(NS_ARRAY_CONTRACTID);
+
   PRUint32 j = 0;
   nsCOMPtr<nsISupports> isupport = dont_AddRef(mCertArray->ElementAt(j));
   nsCOMPtr<nsIX509Cert> orgCert = do_QueryInterface(isupport);
@@ -694,9 +717,23 @@ nsCertTree::GetCellText(PRInt32 row, nsITreeColumn* col,
     if (NS_LITERAL_STRING("certcol").Equals(colID))
       _retval.Assign(el->orgName);
     else
-      _retval.SetCapacity(0);
+      _retval.Truncate();
     return NS_OK;
   }
+  PRInt32 colIndex;
+  col->GetIndex(&colIndex);
+  PRUint32 arrayIndex=row+colIndex*mNumRows;
+  PRUint32 arrayLength=0;
+  if (mCellText) {
+    mCellText->GetLength(&arrayLength);
+  }
+  if (arrayIndex < arrayLength) {
+    nsCOMPtr<nsISupportsString> myString(do_QueryElementAt(mCellText, arrayIndex));
+    if (myString) {
+      myString->GetData(_retval);
+      return NS_OK;
+    }
+  }
   nsCOMPtr<nsIX509Cert> cert = dont_AddRef(GetCertAtIndex(row));
   if (cert == nsnull) return NS_ERROR_FAILURE;
   if (NS_LITERAL_STRING("certcol").Equals(colID)) {
@@ -779,9 +816,41 @@ nsCertTree::GetCellText(PRInt32 row, nsITreeColumn* col,
     }
   } else if (NS_LITERAL_STRING("serialnumcol").Equals(colID)) {
     rv = cert->GetSerialNumber(_retval);
+  } else if (NS_LITERAL_STRING("typecol").Equals(colID)) {
+    nsCOMPtr<nsIX509Cert2> pipCert = do_QueryInterface(cert);
+    PRUint32 type = nsIX509Cert::UNKNOWN_CERT;
+
+    if (pipCert) {
+	rv = pipCert->GetCertType(&type);
+    }
+
+    switch (type) {
+    case nsIX509Cert::USER_CERT:
+        rv = mNSSComponent->GetPIPNSSBundleString("CertUser", _retval);
+	break;
+    case nsIX509Cert::CA_CERT:
+        rv = mNSSComponent->GetPIPNSSBundleString("CertCA", _retval);
+	break;
+    case nsIX509Cert::SERVER_CERT:
+        rv = mNSSComponent->GetPIPNSSBundleString("CertSSL", _retval);
+	break;
+    case nsIX509Cert::EMAIL_CERT:
+        rv = mNSSComponent->GetPIPNSSBundleString("CertEmail", _retval);
+	break;
+    default:
+        rv = mNSSComponent->GetPIPNSSBundleString("CertUnknown", _retval);
+	break;
+    }
+
   } else {
     return NS_ERROR_FAILURE;
   }
+  if (mCellText) {
+    nsCOMPtr<nsISupportsString> text(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+    text->SetData(_retval);
+    mCellText->ReplaceElementAt(text, arrayIndex, PR_FALSE);
+  }
   return rv;
 }
 
@@ -800,10 +869,11 @@ nsCertTree::ToggleOpenState(PRInt32 index)
   if (!mTreeArray)
     return NS_ERROR_NOT_INITIALIZED;
   treeArrayEl *el = GetThreadDescAtIndex(index);
-  if (el) el->open = !el->open;
-  PRInt32 fac = (el->open) ? 1 : -1;
-  if (mTree) mTree->RowCountChanged(index, fac * el->numChildren);
-  mSelection->Select(index);
+  if (el) {
+    el->open = !el->open;
+    PRInt32 newChildren = (el->open) ? el->numChildren : -el->numChildren;
+    if (mTree) mTree->RowCountChanged(index + 1, newChildren);
+  }
   return NS_OK;
 }
 
@@ -836,6 +906,14 @@ nsCertTree::IsEditable(PRInt32 row, nsITreeColumn* col, PRBool *_retval)
   return NS_OK;
 }
 
+/* boolean isSelectable (in long row, in nsITreeColumn col); */
+NS_IMETHODIMP 
+nsCertTree::IsSelectable(PRInt32 row, nsITreeColumn* col, PRBool *_retval)
+{
+  *_retval = PR_FALSE;
+  return NS_OK;
+}
+
 /* void setCellValue (in long row, in nsITreeColumn col, in AString value); */
 NS_IMETHODIMP 
 nsCertTree::SetCellValue(PRInt32 row, nsITreeColumn* col, 
@@ -882,7 +960,7 @@ nsCertTree::dumpMap()
 {
   for (int i=0; i<mNumOrgs; i++) {
     nsAutoString org(mTreeArray[i].orgName);
-    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("ORG[%s]", NS_LossyConvertUCS2toASCII(org).get()));
+    PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("ORG[%s]", NS_LossyConvertUTF16toASCII(org).get()));
     PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("OPEN[%d]", mTreeArray[i].open));
     PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("INDEX[%d]", mTreeArray[i].certIndex));
     PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("NCHILD[%d]", mTreeArray[i].numChildren));
@@ -891,14 +969,14 @@ nsCertTree::dumpMap()
     treeArrayEl *el = GetThreadDescAtIndex(i);
     if (el != nsnull) {
       nsAutoString td(el->orgName);
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("thread desc[%d]: %s", i, NS_LossyConvertUCS2toASCII(td).get()));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("thread desc[%d]: %s", i, NS_LossyConvertUTF16toASCII(td).get()));
     }
     nsCOMPtr<nsIX509Cert> ct = dont_AddRef(GetCertAtIndex(i));
     if (ct != nsnull) {
       PRUnichar *goo;
       ct->GetCommonName(&goo);
       nsAutoString doo(goo);
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("cert [%d]: %s", i, NS_LossyConvertUCS2toASCII(doo).get()));
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("cert [%d]: %s", i, NS_LossyConvertUTF16toASCII(doo).get()));
     }
   }
 }
@@ -936,10 +1014,14 @@ NS_IMETHODIMP nsCertTree::IsSorted(PRBool *_retval)
   return NS_OK;
 }
 
+#define RETURN_NOTHING
+
 void 
 nsCertTree::CmpInitCriterion(nsIX509Cert *cert, CompareCacheHashEntry *entry,
                              sortCriterion crit, PRInt32 level)
 {
+  NS_ENSURE_TRUE( (cert!=0 && entry!=0), RETURN_NOTHING );
+
   entry->mCritInit[level] = PR_TRUE;
   nsXPIDLString &str = entry->mCrit[level];
   
@@ -972,7 +1054,7 @@ nsCertTree::CmpInitCriterion(nsIX509Cert *cert, CompareCacheHashEntry *entry,
           PR_ExplodeTime(notBefore, PR_GMTParameters, &explodedTime);
           char datebuf[20]; // 4 + 2 + 2 + 2 + 2 + 2 + 1 = 15
           if (0 != PR_FormatTime(datebuf, sizeof(datebuf), "%Y%m%d%H%M%S", &explodedTime)) {
-            str = NS_ConvertASCIItoUCS2(nsDependentCString(datebuf));
+            str = NS_ConvertASCIItoUTF16(nsDependentCString(datebuf));
           }
         }
       }
@@ -991,6 +1073,8 @@ nsCertTree::CmpByCrit(nsIX509Cert *a, CompareCacheHashEntry *ace,
                       nsIX509Cert *b, CompareCacheHashEntry *bce, 
                       sortCriterion crit, PRInt32 level)
 {
+  NS_ENSURE_TRUE( (a!=0 && ace!=0 && b!=0 && bce!=0), 0 );
+
   if (!ace->mCritInit[level]) {
     CmpInitCriterion(a, ace, crit, level);
   }
@@ -1004,7 +1088,7 @@ nsCertTree::CmpByCrit(nsIX509Cert *a, CompareCacheHashEntry *ace,
 
   PRInt32 result;
   if (str_a && str_b)
-    result = Compare(str_a, str_b);
+    result = Compare(str_a, str_b, nsCaseInsensitiveStringComparator());
   else
     result = !str_a ? (!str_b ? 0 : -1) : 1;
 
@@ -1018,6 +1102,8 @@ PRInt32
 nsCertTree::CmpBy(void *cache, nsIX509Cert *a, nsIX509Cert *b, 
                   sortCriterion c0, sortCriterion c1, sortCriterion c2)
 {
+  NS_ENSURE_TRUE( (cache!=0 && a!=0 && b!=0), 0 );
+
   if (a == b)
     return 0;
 
