@@ -51,10 +51,15 @@
 #include "nsPK11TokenDB.h"
 #include "nsOCSPResponder.h"
 #include "nsReadableUtils.h"
-#include "nsArray.h"
+#include "nsIMutableArray.h"
+#include "nsArrayUtils.h"
 #include "nsNSSShutDown.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
+#include "nsComponentManagerUtils.h"
+#include "nsIPrompt.h"
+#include "nsIProxyObjectManager.h"
+#include "nsProxiedService.h"
 
 #include "nspr.h"
 extern "C" {
@@ -100,7 +105,7 @@ nsNSSCertificateDB::FindCertByNickname(nsISupports *aToken,
   nsNSSShutDownPreventionLock locker;
   CERTCertificate *cert = NULL;
   char *asciiname = NULL;
-  NS_ConvertUCS2toUTF8 aUtf8Nickname(nickname);
+  NS_ConvertUTF16toUTF8 aUtf8Nickname(nickname);
   asciiname = NS_CONST_CAST(char*, aUtf8Nickname.get());
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("Getting \"%s\"\n", asciiname));
 #if 0
@@ -262,13 +267,13 @@ nsNSSCertificateDB::handleCACertDownload(nsIArray *x509Certs,
   // certs which may or may not be a chained list of certs.  Until
   // the day we can design some solid UI for the general case, we'll
   // code to the > 90% case.  That case is where a CA sends down a
-  // list that is a chain up to its root in either ascending or 
-  // descending order.  What we're gonna do is compare the first 
-  // 2 entries, if the first was signed by the second, we assume
-  // the leaf cert is the first cert and display it.  If the second
-  // cert was signed by the first cert, then we assume the first cert
-  // is the root and the last cert in the array is the leaf.  In this
-  // case we display the last cert.
+  // list that is a hierarchy whose root is either the first or 
+  // the last cert.  What we're gonna do is compare the first 
+  // 2 entries, if the second was signed by the first, we assume
+  // the root cert is the first cert and display it.  Otherwise,
+  // we compare the last 2 entries, if the second to last cert was
+  // signed by the last cert, then we assume the last cert is the
+  // root and display it.
 
   nsNSSShutDownPreventionLock locker;
 
@@ -287,35 +292,37 @@ nsNSSCertificateDB::handleCACertDownload(nsIArray *x509Certs,
     selCertIndex = 0;
     certToShow = do_QueryElementAt(x509Certs, selCertIndex);
   } else {
-    nsCOMPtr<nsIX509Cert> cert0;
-    nsCOMPtr<nsIX509Cert> cert1;
+    nsCOMPtr<nsIX509Cert> cert0;    // first cert
+    nsCOMPtr<nsIX509Cert> cert1;    // second cert
+    nsCOMPtr<nsIX509Cert> certn_2;  // second to last cert
+    nsCOMPtr<nsIX509Cert> certn_1;  // last cert
 
     cert0 = do_QueryElementAt(x509Certs, 0);
     cert1 = do_QueryElementAt(x509Certs, 1);
+    certn_2 = do_QueryElementAt(x509Certs, numCerts-2);
+    certn_1 = do_QueryElementAt(x509Certs, numCerts-1);
 
     nsXPIDLString cert0SubjectName;
-    nsXPIDLString cert0IssuerName;
-    nsXPIDLString cert1SubjectName;
     nsXPIDLString cert1IssuerName;
+    nsXPIDLString certn_2IssuerName;
+    nsXPIDLString certn_1SubjectName;
 
-    cert0->GetIssuerName(cert0IssuerName);
     cert0->GetSubjectName(cert0SubjectName);
-
     cert1->GetIssuerName(cert1IssuerName);
-    cert1->GetSubjectName(cert1SubjectName);
+    certn_2->GetIssuerName(certn_2IssuerName);
+    certn_1->GetSubjectName(certn_1SubjectName);
 
     if (cert1IssuerName.Equals(cert0SubjectName)) {
       // In this case, the first cert in the list signed the second,
-      // so the first cert is the root.  Let's display the last cert 
-      // in the list.
-      selCertIndex = numCerts-1;
-      certToShow = do_QueryElementAt(x509Certs, selCertIndex);
-    } else 
-    if (cert0IssuerName.Equals(cert1SubjectName)) { 
-      // In this case the second cert has signed the first cert.  The 
-      // first cert is the leaf, so let's display it.
+      // so the first cert is the root.  Let's display it. 
       selCertIndex = 0;
       certToShow = cert0;
+    } else 
+    if (certn_2IssuerName.Equals(certn_1SubjectName)) { 
+      // In this case the last cert has signed the second to last cert.
+      // The last cert is the root, so let's display it.
+      selCertIndex = numCerts-1;
+      certToShow = certn_1;
     } else {
       // It's not a chain, so let's just show the first one in the 
       // downloaded list.
@@ -349,6 +356,10 @@ nsNSSCertificateDB::handleCACertDownload(nsIArray *x509Certs,
     tmpCert = CERT_NewTempCertificate(certdb, &der,
                                       nsnull, PR_FALSE, PR_TRUE);
   }
+  nsMemory::Free(der.data);
+  der.data = nsnull;
+  der.len = 0;
+  
   if (!tmpCert) {
     NS_ERROR("Couldn't create cert from DER blob\n");
     return NS_ERROR_FAILURE;
@@ -356,11 +367,13 @@ nsNSSCertificateDB::handleCACertDownload(nsIArray *x509Certs,
 
   CERTCertificateCleaner tmpCertCleaner(tmpCert);
 
+  if (!CERT_IsCACert(tmpCert, NULL)) {
+    DisplayCertificateAlert(ctx, "NotACACert", certToShow);
+    return NS_ERROR_FAILURE;
+  }
+
   if (tmpCert->isperm) {
-    nsPSMUITracker tracker;
-    if (!tracker.isUIForbidden()) {
-      dialogs->NotifyCACertExists(ctx);
-    }
+    DisplayCertificateAlert(ctx, "CaCertExists", certToShow);
     return NS_ERROR_FAILURE;
   }
 
@@ -416,6 +429,10 @@ nsNSSCertificateDB::handleCACertDownload(nsIArray *x509Certs,
     CERTCertificate *tmpCert2 = 
       CERT_NewTempCertificate(certdb, &der, nsnull, PR_FALSE, PR_TRUE);
 
+    nsMemory::Free(der.data);
+    der.data = nsnull;
+    der.len = 0;
+
     if (!tmpCert2) {
       NS_ASSERTION(0, "Couldn't create temp cert from DER blob\n");
       continue;  // Let's try to import the rest of 'em
@@ -450,11 +467,11 @@ nsNSSCertificateDB::ImportCertificates(PRUint8 * data, PRUint32 length,
     PORT_FreeArena(arena, PR_FALSE);
     return NS_ERROR_FAILURE;
   }
-  nsCOMPtr<nsIMutableArray> array;
-  nsresult rv = NS_NewArray(getter_AddRefs(array));
-  if (NS_FAILED(rv)) {
+  nsCOMPtr<nsIMutableArray> array =
+    do_CreateInstance(NS_ARRAY_CONTRACTID, &nsrv);
+  if (NS_FAILED(nsrv)) {
     PORT_FreeArena(arena, PR_FALSE);
-    return rv;
+    return nsrv;
   }
 
   // Now let's create some certs to work with
@@ -555,13 +572,6 @@ nsNSSCertificateDB::ImportEmailCertificate(PRUint8 * data, PRUint32 length,
       CERT_AddCertToListTail(certList, cert);
   }
 
-  /* filter out the certs we don't want */
-  srv = CERT_FilterCertListByUsage(certList, certusage, PR_FALSE);
-  if (srv != SECSuccess) {
-    nsrv = NS_ERROR_FAILURE;
-    goto loser;
-  }
-
   /* go down the remaining list of certs and verify that they have
    * valid chains, then import them.
    */
@@ -569,18 +579,34 @@ nsNSSCertificateDB::ImportEmailCertificate(PRUint8 * data, PRUint32 length,
   for (node = CERT_LIST_HEAD(certList);
        !CERT_LIST_END(node,certList);
        node = CERT_LIST_NEXT(node)) {
+
+    bool alert_and_skip = false;
+
+    if (!node->cert) {
+      continue;
+    }
+
     if (CERT_VerifyCert(certdb, node->cert, 
         PR_TRUE, certusage, now, ctx, NULL) != SECSuccess) {
-      continue;
-    }
-
-    CERTCertificateList *certChain = CERT_CertChainFromCert(node->cert, certusage, PR_FALSE);
-    if (!certChain) {
-      continue;
+      alert_and_skip = true;
     }
 
+    CERTCertificateList *certChain = nsnull;
     CERTCertificateListCleaner chainCleaner(certChain);
 
+    if (!alert_and_skip) {
+      CERT_CertChainFromCert(node->cert, certusage, PR_FALSE);
+      if (!certChain) {
+        alert_and_skip = true;
+      }
+    }
+
+    if (alert_and_skip) {    
+      nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(node->cert);
+      DisplayCertificateAlert(ctx, "NotImportingUnverifiedCert", certToShow);
+      continue;
+    }
+
     /*
      * CertChain returns an array of SECItems, import expects an array of
      * SECItem pointers. Create the SECItem Pointers from the array of
@@ -748,18 +774,30 @@ nsNSSCertificateDB::ImportValidCACertsInList(CERTCertList *certList, nsIInterfac
   for (node = CERT_LIST_HEAD(certList);
        !CERT_LIST_END(node,certList);
        node = CERT_LIST_NEXT(node)) {
+
+    bool alert_and_skip = false;
+
     if (CERT_VerifyCert(CERT_GetDefaultCertDB(), node->cert, 
         PR_TRUE, certUsageVerifyCA, now, ctx, NULL) != SECSuccess) {
-      continue;
-    }
-
-    CERTCertificateList *certChain = CERT_CertChainFromCert(node->cert, certUsageAnyCA, PR_FALSE);
-    if (!certChain) {
-      continue;
+      alert_and_skip = true;
     }
 
+    CERTCertificateList *certChain = nsnull;
     CERTCertificateListCleaner chainCleaner(certChain);
 
+    if (!alert_and_skip) {    
+      certChain = CERT_CertChainFromCert(node->cert, certUsageAnyCA, PR_FALSE);
+      if (!certChain) {
+        alert_and_skip = true;
+      }
+    }
+
+    if (alert_and_skip) {    
+      nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(node->cert);
+      DisplayCertificateAlert(ctx, "NotImportingUnverifiedCert", certToShow);
+      continue;
+    }
+
     /*
      * CertChain returns an array of SECItems, import expects an array of
      * SECItem pointers. Create the SECItem Pointers from the array of
@@ -781,6 +819,50 @@ nsNSSCertificateDB::ImportValidCACertsInList(CERTCertList *certList, nsIInterfac
   return NS_OK;
 }
 
+void nsNSSCertificateDB::DisplayCertificateAlert(nsIInterfaceRequestor *ctx, 
+                                                 const char *stringID, 
+                                                 nsIX509Cert *certToShow)
+{
+  nsPSMUITracker tracker;
+  if (!tracker.isUIForbidden()) {
+
+    // This shall be replaced by embedding ovverridable prompts
+    // as discussed in bug 310446, and should make use of certToShow.
+
+    nsresult rv;
+    nsCOMPtr<nsINSSComponent> nssComponent(do_GetService(kNSSComponentCID, &rv));
+    if (NS_SUCCEEDED(rv)) {
+      nsAutoString tmpMessage;
+      nssComponent->GetPIPNSSBundleString(stringID, tmpMessage);
+
+      // The interface requestor object may not be safe, so proxy the call to get
+      // the nsIPrompt.
+
+      nsCOMPtr<nsIInterfaceRequestor> proxiedCallbacks;
+      NS_GetProxyForObject(NS_PROXY_TO_MAIN_THREAD,
+                           NS_GET_IID(nsIInterfaceRequestor),
+                           ctx,
+                           NS_PROXY_SYNC,
+                           getter_AddRefs(proxiedCallbacks));
+    
+      nsCOMPtr<nsIPrompt> prompt (do_GetInterface(proxiedCallbacks));
+      if (!prompt)
+        return;
+    
+      // Finally, get a proxy for the nsIPrompt
+    
+      nsCOMPtr<nsIPrompt> proxyPrompt;
+      NS_GetProxyForObject(NS_PROXY_TO_MAIN_THREAD,
+                           NS_GET_IID(nsIPrompt),
+                           prompt,
+                           NS_PROXY_SYNC,
+                           getter_AddRefs(proxyPrompt));
+    
+      proxyPrompt->Alert(nsnull, tmpMessage.get());
+    }
+  }
+}
+
 
 NS_IMETHODIMP 
 nsNSSCertificateDB::ImportUserCertificate(PRUint8 *data, PRUint32 length, nsIInterfaceRequestor *ctx)
@@ -813,6 +895,8 @@ nsNSSCertificateDB::ImportUserCertificate(PRUint8 *data, PRUint32 length, nsIInt
 
   slot = PK11_KeyForCertExists(cert, NULL, ctx);
   if ( slot == NULL ) {
+    nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(cert);
+    DisplayCertificateAlert(ctx, "UserCertIgnoredNoPrivateKey", certToShow);
     goto loser;
   }
   PK11_FreeSlot(slot);
@@ -835,6 +919,10 @@ nsNSSCertificateDB::ImportUserCertificate(PRUint8 *data, PRUint32 length, nsIInt
   }
   PK11_FreeSlot(slot);
 
+  {
+    nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(cert);
+    DisplayCertificateAlert(ctx, "UserCertImported", certToShow);
+  }
   rv = NS_OK;
 
   numCACerts = collectArgs->numcerts - 1;
@@ -860,14 +948,14 @@ NS_IMETHODIMP
 nsNSSCertificateDB::DeleteCertificate(nsIX509Cert *aCert)
 {
   nsNSSShutDownPreventionLock locker;
-  nsNSSCertificate *nssCert = NS_STATIC_CAST(nsNSSCertificate*, aCert);
+  nsCOMPtr<nsIX509Cert2> nssCert = do_QueryInterface(aCert);
   CERTCertificate *cert = nssCert->GetCert();
   if (!cert) return NS_ERROR_FAILURE;
   CERTCertificateCleaner certCleaner(cert);
   SECStatus srv = SECSuccess;
 
-  PRUint32 certType = getCertType(cert);
-  nssCert->SetCertType(certType);
+  PRUint32 certType;
+  nssCert->GetCertType(&certType);
   if (NS_FAILED(nssCert->MarkForPermDeletion()))
   {
     return NS_ERROR_FAILURE;
@@ -902,7 +990,7 @@ nsNSSCertificateDB::SetCertTrust(nsIX509Cert *cert,
   nsNSSShutDownPreventionLock locker;
   SECStatus srv;
   nsNSSCertTrust trust;
-  nsNSSCertificate *pipCert = NS_STATIC_CAST(nsNSSCertificate *, cert);
+  nsCOMPtr<nsIX509Cert2> pipCert = do_QueryInterface(cert);
   CERTCertificate *nsscert = pipCert->GetCert();
   CERTCertificateCleaner certCleaner(nsscert);
   if (type == nsIX509Cert::CA_CERT) {
@@ -941,12 +1029,18 @@ nsNSSCertificateDB::IsCertTrusted(nsIX509Cert *cert,
                                   PRUint32 trustType,
                                   PRBool *_isTrusted)
 {
+  NS_ENSURE_ARG_POINTER(_isTrusted);
+  *_isTrusted = PR_FALSE;
+
   nsNSSShutDownPreventionLock locker;
   SECStatus srv;
-  nsNSSCertificate *pipCert = NS_STATIC_CAST(nsNSSCertificate *, cert);
+  nsCOMPtr<nsIX509Cert2> pipCert = do_QueryInterface(cert);
   CERTCertificate *nsscert = pipCert->GetCert();
   CERTCertTrust nsstrust;
   srv = CERT_GetCertTrust(nsscert, &nsstrust);
+  if (srv != SECSuccess)
+    return NS_ERROR_FAILURE;
+
   nsNSSCertTrust trust(&nsstrust);
   CERT_DestroyCertificate(nsscert);
   if (certType == nsIX509Cert::CA_CERT) {
@@ -1112,12 +1206,12 @@ GetOCSPResponders (CERTCertificate *aCert,
   // Get the AIA and nickname //
   serviceURL = CERT_GetOCSPAuthorityInfoAccessLocation(aCert);
   if (serviceURL) {
-    url = ToNewUnicode(NS_ConvertUTF8toUCS2(serviceURL));
+    url = ToNewUnicode(NS_ConvertUTF8toUTF16(serviceURL));
     PORT_Free(serviceURL);
   }
 
   nickname = aCert->nickname;
-  nn = ToNewUnicode(NS_ConvertUTF8toUCS2(nickname));
+  nn = ToNewUnicode(NS_ConvertUTF8toUTF16(nickname));
 
   nsCOMPtr<nsIOCSPResponder> new_entry = new nsOCSPResponder(nn, url);
   nsMemory::Free(nn);
@@ -1150,10 +1244,10 @@ nsNSSCertificateDB::GetOCSPResponders(nsIArray ** aResponders)
 {
   nsNSSShutDownPreventionLock locker;
   SECStatus sec_rv;
-  nsCOMPtr<nsIMutableArray> respondersArray;
-  nsresult rv = NS_NewArray(getter_AddRefs(respondersArray));
-  if (NS_FAILED(rv)) {
-    return rv;
+  nsCOMPtr<nsIMutableArray> respondersArray =
+    do_CreateInstance(NS_ARRAY_CONTRACTID);
+  if (!respondersArray) {
+    return NS_ERROR_OUT_OF_MEMORY;
   }
 
   sec_rv = PK11_TraverseSlotCerts(::GetOCSPResponders,
@@ -1213,16 +1307,19 @@ nsNSSCertificateDB::getCertNames(CERTCertList *certList,
       char *namestr = NULL;
       nsAutoString certstr;
       rv = pipCert.GetDbKey(&dbkey);
-      nsAutoString keystr = NS_ConvertASCIItoUCS2(dbkey);
+      nsAutoString keystr = NS_ConvertASCIItoUTF16(dbkey);
       PR_FREEIF(dbkey);
       if (type == nsIX509Cert::EMAIL_CERT) {
         namestr = node->cert->emailAddr;
       } else {
         namestr = node->cert->nickname;
-        char *sc = strchr(namestr, ':');
-        if (sc) *sc = DELIM;
+        if (namestr) {
+          char *sc = strchr(namestr, ':');
+          if (sc) *sc = DELIM;
+        }
       }
-      nsAutoString certname = NS_ConvertASCIItoUCS2(namestr);
+      if (!namestr) namestr = "";
+      nsAutoString certname = NS_ConvertASCIItoUTF16(namestr);
       certstr.Append(PRUnichar(DELIM));
       certstr += certname;
       certstr.Append(PRUnichar(DELIM));
@@ -1267,7 +1364,7 @@ nsNSSCertificateDB::FindEmailEncryptionCert(const nsAString &aNickname, nsIX509C
   nsCOMPtr<nsIInterfaceRequestor> ctx = new PipUIContext();
   nsNSSCertificate *nssCert = nsnull;
   char *asciiname = NULL;
-  NS_ConvertUCS2toUTF8 aUtf8Nickname(aNickname);
+  NS_ConvertUTF16toUTF8 aUtf8Nickname(aNickname);
   asciiname = NS_CONST_CAST(char*, aUtf8Nickname.get());
 
   /* Find a good cert in the user's database */
@@ -1307,7 +1404,7 @@ nsNSSCertificateDB::FindEmailSigningCert(const nsAString &aNickname, nsIX509Cert
   nsCOMPtr<nsIInterfaceRequestor> ctx = new PipUIContext();
   nsNSSCertificate *nssCert = nsnull;
   char *asciiname = NULL;
-  NS_ConvertUCS2toUTF8 aUtf8Nickname(aNickname);
+  NS_ConvertUTF16toUTF8 aUtf8Nickname(aNickname);
   asciiname = NS_CONST_CAST(char*, aUtf8Nickname.get());
 
   /* Find a good cert in the user's database */
@@ -1572,10 +1669,6 @@ NS_IMETHODIMP nsNSSCertificateDB::AddCertFromBase64(const char *aBase64, const c
   // need to calculate the trust bits from the aTrust string.
   nsresult rv = CERT_DecodeTrustString(trust.GetTrust(), /* this is const, but not declared that way */(char *) aTrust);
   NS_ENSURE_SUCCESS(rv, rv); // if bad trust passed in, return error.
-  trust.SetValidCA();
-  trust.AddCATrust(trust.GetTrust()->sslFlags,
-                   trust.GetTrust()->emailFlags,
-                   trust.GetTrust()->objectSigningFlags);
 
 
   rv = ConstructX509FromBase64(aBase64, getter_AddRefs(newCert));
@@ -1592,6 +1685,9 @@ NS_IMETHODIMP nsNSSCertificateDB::AddCertFromBase64(const char *aBase64, const c
   if (!tmpCert) 
     tmpCert = CERT_NewTempCertificate(certdb, &der,
                                       nsnull, PR_FALSE, PR_TRUE);
+  nsMemory::Free(der.data);
+  der.data = nsnull;
+  der.len = 0;
 
   if (!tmpCert) {
     NS_ASSERTION(0,"Couldn't create cert from DER blob\n");
@@ -1617,3 +1713,22 @@ NS_IMETHODIMP nsNSSCertificateDB::AddCertFromBase64(const char *aBase64, const c
 
   return (srv == SECSuccess) ? NS_OK : NS_ERROR_FAILURE;
 }
+
+NS_IMETHODIMP 
+nsNSSCertificateDB::GetCerts(nsIX509CertList **_retval)
+{
+  CERTCertList *certList;
+
+  nsCOMPtr<nsIInterfaceRequestor> ctx = new PipUIContext();
+  nsCOMPtr<nsIX509CertList> nssCertList;
+  certList = PK11_ListCerts(PK11CertListUnique, ctx);
+
+  // nsNSSCertList 1) adopts certList, and 2) handles the NULL case fine.
+  // (returns an empty list) 
+  nssCertList = new nsNSSCertList(certList, PR_TRUE);
+  if (!nssCertList) { return NS_ERROR_OUT_OF_MEMORY; }
+
+  *_retval = nssCertList;
+  NS_ADDREF(*_retval);
+  return NS_OK;
+}
