@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -29,11 +29,11 @@
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
  
@@ -42,6 +42,8 @@
 #include "nsIRDFLiteral.h"
 #include "rdf.h"
 #include "nsNetUtil.h"
+#include "nsIURI.h"
+#include "nsIURL.h"
 #include "nsIDOMChromeWindow.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMWindowInternal.h"
@@ -64,13 +66,19 @@
 #include "nsVoidArray.h"
 #include "nsEnumeratorUtils.h"
 #include "nsIFileURL.h"
+#include "nsEmbedCID.h"
+#include "nsInt64.h"
+#include "nsAutoPtr.h"
+
+#ifdef XP_WIN
+#include <shlobj.h>
+#endif
 
 /* Outstanding issues/todo:
  * 1. Implement pause/resume.
  */
-  
+
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 static PRBool gStoppingDownloads = PR_FALSE;
 
 #define DOWNLOAD_MANAGER_FE_URL "chrome://mozapps/content/downloads/downloads.xul"
@@ -82,8 +90,11 @@ static PRBool gStoppingDownloads = PR_FALSE;
 #define PREF_BDM_OPENDELAY "browser.download.manager.openDelay"
 #define PREF_BDM_SHOWWHENSTARTING "browser.download.manager.showWhenStarting"
 #define PREF_BDM_FOCUSWHENSTARTING "browser.download.manager.focusWhenStarting"
+#define PREF_BDM_CLOSEWHENDONE "browser.download.manager.closeWhenDone"
 #define PREF_BDM_FLASHCOUNT "browser.download.manager.flashCount"
-#define INTERVAL 500
+#define PREF_BDM_ADDTORECENTDOCS "browser.download.manager.addToRecentDocs"
+
+static const nsInt64 gInterval((PRUint32)(400 * PR_USEC_PER_MSEC));
 
 static nsIRDFResource* gNC_DownloadsRoot = nsnull;
 static nsIRDFResource* gNC_File = nsnull;
@@ -126,11 +137,7 @@ GetFilePathFromURI(nsIURI *aURI, nsAString &aPath)
 ///////////////////////////////////////////////////////////////////////////////
 // nsDownloadManager
 
-#ifdef XP_WIN
-NS_IMPL_ISUPPORTS4(nsDownloadManager, nsIDownloadManager, nsIXPInstallManagerUI, nsIObserver, nsIAlertListener)
-#else
 NS_IMPL_ISUPPORTS3(nsDownloadManager, nsIDownloadManager, nsIXPInstallManagerUI, nsIObserver)
-#endif
 
 nsDownloadManager::nsDownloadManager() : mBatches(0)
 {
@@ -146,9 +153,13 @@ nsDownloadManager::~nsDownloadManager()
 
   gRDFService->UnregisterDataSource(mDataSource);
 
+#if 0
+  // Temporary fix for orange regression from bug 328159 until I
+  // understand new protocol following bug 326491.  See bug 315421.
   gObserverService->RemoveObserver(this, "quit-application");
   gObserverService->RemoveObserver(this, "quit-application-requested");
   gObserverService->RemoveObserver(this, "offline-requested");
+#endif
 
   NS_IF_RELEASE(gNC_DownloadsRoot);                                             
   NS_IF_RELEASE(gNC_File);                                                      
@@ -217,9 +228,12 @@ nsDownloadManager::Init()
     return NS_ERROR_OUT_OF_MEMORY;
 
   rv = NS_STATIC_CAST(nsDownloadsDataSource*, (nsIRDFDataSource*)mDataSource.get())->LoadDataSource();
-  if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) {
+    mDataSource = nsnull; // so we don't UnregisterDataSource on it
+    return rv;
+  }
 
-  nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(kStringBundleServiceCID, &rv);
+  nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
   
   rv = bundleService->CreateBundle(DOWNLOAD_MANAGER_BUNDLE, getter_AddRefs(mBundle));
@@ -231,6 +245,11 @@ nsDownloadManager::Init()
   // completely initialized), but the observerservice would still keep a reference
   // to us and notify us about shutdown, which may cause crashes.
   // failure to add an observer is not critical
+  //
+  // These observers will be cleaned up automatically at app shutdown.  We do
+  // not bother explicitly breaking the observers because we are a singleton
+  // that lives for the duration of the app.
+  //
   gObserverService->AddObserver(this, "quit-application", PR_FALSE);
   gObserverService->AddObserver(this, "quit-application-requested", PR_FALSE);
   gObserverService->AddObserver(this, "offline-requested", PR_FALSE);
@@ -407,7 +426,7 @@ nsDownloadManager::SaveState()
       downloads->GetNext(getter_AddRefs(supports));
       res = do_QueryInterface(supports);
       res->GetValueConst(&uri);
-      AssertProgressInfoFor(NS_ConvertASCIItoUCS2(uri).get());
+      AssertProgressInfoFor(NS_ConvertASCIItoUTF16(uri).get());
       downloads->HasMoreElements(&hasMoreElements);
     }
   }
@@ -423,9 +442,7 @@ nsDownloadManager::AssertProgressInfoFor(const PRUnichar* aPath)
     return NS_ERROR_FAILURE;
  
   nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
-  nsCOMPtr<nsIDownload> download;
-  internalDownload->QueryInterface(NS_GET_IID(nsIDownload), (void**) getter_AddRefs(download));
-  if (!download)
+  if (!internalDownload)
     return NS_ERROR_FAILURE;
   
   nsresult rv;
@@ -451,7 +468,7 @@ nsDownloadManager::AssertProgressInfoFor(const PRUnichar* aPath)
   if (NS_FAILED(rv)) return rv;
 
   // update percentage
-  download->GetPercentComplete(&percentComplete);
+  internalDownload->GetPercentComplete(&percentComplete);
 
   mDataSource->GetTarget(res, gNC_ProgressPercent, PR_TRUE, getter_AddRefs(oldTarget));
   gRDFService->GetIntLiteral(percentComplete, getter_AddRefs(intLiteral));
@@ -463,9 +480,12 @@ nsDownloadManager::AssertProgressInfoFor(const PRUnichar* aPath)
   if (NS_FAILED(rv)) return rv;
 
   // update transferred
-  PRInt32 current = 0;
-  PRInt32 max = 0;
-  internalDownload->GetTransferInformation(&current, &max);
+  nsDownload::TransferInformation transferInfo =
+                                 internalDownload->GetTransferInformation();
+
+  // convert from bytes to kbytes for progress display
+  PRInt64 current = (PRFloat64)transferInfo.mCurrBytes / 1024 + .5;
+  PRInt64 max = (PRFloat64)transferInfo.mMaxBytes / 1024 + .5;
  
   nsAutoString currBytes; currBytes.AppendInt(current);
   nsAutoString maxBytes; maxBytes.AppendInt(max);
@@ -500,11 +520,12 @@ NS_IMETHODIMP
 nsDownloadManager::AddDownload(DownloadType aDownloadType, 
                                nsIURI* aSource,
                                nsIURI* aTarget,
-                               const PRUnichar* aDisplayName,
-                               const PRUnichar* aIconURL, 
+                               const nsAString& aDisplayName,
+                               const nsAString& aIconURL, 
                                nsIMIMEInfo *aMIMEInfo,
-                               PRInt64 aStartTime,
-                               nsIWebBrowserPersist* aPersist,
+                               PRTime aStartTime,
+                               nsILocalFile* aTempFile,
+                               nsICancelable* aCancelable,
                                nsIDownload** aDownload)
 {
   NS_ENSURE_ARG_POINTER(aSource);
@@ -539,6 +560,7 @@ nsDownloadManager::AddDownload(DownloadType aDownloadType,
   internalDownload->SetDownloadManager(this);
   internalDownload->SetTarget(aTarget);
   internalDownload->SetSource(aSource);
+  internalDownload->SetTempFile(aTempFile);
 
   // The path is the uniquifier of the download resource. 
   // XXXben - this is a little risky - really we should be using anonymous
@@ -599,7 +621,7 @@ nsDownloadManager::AddDownload(DownloadType aDownloadType,
   if (displayName.IsEmpty()) {
     targetFile->GetLeafName(displayName);
   }
-  (*aDownload)->SetDisplayName(displayName.get());
+  internalDownload->SetDisplayName(displayName.get());
  
   nsCOMPtr<nsIRDFLiteral> nameLiteral;
   gRDFService->GetLiteral(displayName.get(), getter_AddRefs(nameLiteral));
@@ -615,10 +637,9 @@ nsDownloadManager::AddDownload(DownloadType aDownloadType,
   }
   
   // Assert icon information
-  if (aIconURL) {
+  if (!aIconURL.IsEmpty()) {
     nsCOMPtr<nsIRDFResource> iconURIRes;
-    nsDependentString iconURL(aIconURL);
-    gRDFService->GetUnicodeResource(iconURL, getter_AddRefs(iconURIRes));
+    gRDFService->GetUnicodeResource(aIconURL, getter_AddRefs(iconURIRes));
     mDataSource->GetTarget(downloadRes, gNC_IconURL, PR_TRUE, getter_AddRefs(node));
     if (node)
       rv = mDataSource->Change(downloadRes, gNC_IconURL, node, iconURIRes);
@@ -660,13 +681,8 @@ nsDownloadManager::AddDownload(DownloadType aDownloadType,
     return rv;
   }
 
-  // if a persist object was specified, set the download item as the progress listener
   // this will create a cycle that will be broken in nsDownload::OnStateChange
-  if (aPersist) {
-    internalDownload->SetPersist(aPersist);
-    nsCOMPtr<nsIWebProgressListener> listener = do_QueryInterface(*aDownload);
-    aPersist->SetProgressListener(listener);
-  }
+  internalDownload->SetCancelable(aCancelable);
 
   // If this is an install operation, ensure we have a progress listener for the
   // install and track this download separately. 
@@ -712,11 +728,11 @@ nsDownloadManager::CancelDownload(const PRUnichar* aPath)
   if (!mCurrDownloads.Exists(&key))
     return RemoveDownload(aPath); // XXXBlake for now, to provide a workaround for stuck downloads
   
-  nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
-  nsCOMPtr<nsIDownload> download;
-  CallQueryInterface(internalDownload, NS_STATIC_CAST(nsIDownload**, 
-                                                      getter_AddRefs(download)));
-  if (!download)
+  // Take a strong reference to the download object as we may remove it from
+  // mCurrDownloads in DownloadEnded.
+  nsRefPtr<nsDownload> internalDownload =
+      NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
+  if (!internalDownload)
     return NS_ERROR_FAILURE;
 
   // Don't cancel if download is already finished
@@ -725,38 +741,38 @@ nsDownloadManager::CancelDownload(const PRUnichar* aPath)
 
   internalDownload->SetDownloadState(nsIDownloadManager::DOWNLOAD_CANCELED);
 
-  // if a persist was provided, we can do the cancel ourselves.
-  nsCOMPtr<nsIWebBrowserPersist> persist;
-  download->GetPersist(getter_AddRefs(persist));
-  if (persist) {
-    rv = persist->CancelSave();
-    if (NS_FAILED(rv)) return rv;
-  }
+  // Cancel using the provided object
+  nsCOMPtr<nsICancelable> cancelable;
+  internalDownload->GetCancelable(getter_AddRefs(cancelable));
+  if (cancelable)
+    cancelable->Cancel(NS_BINDING_ABORTED);
 
-  // if an observer was provided, notify that the download was cancelled.
-  // if no persist was provided, this is necessary so that whatever transfer
-  // component being used can cancel the download itself.
-  nsCOMPtr<nsIObserver> observer;
-  download->GetObserver(getter_AddRefs(observer));
-  if (observer) {
-    rv = observer->Observe(download, "oncancel", nsnull);
-    if (NS_FAILED(rv)) return rv;
-  }
- 
   DownloadEnded(aPath, nsnull);
 
-  gObserverService->NotifyObservers(download, "dl-cancel", nsnull);
+  // Dump the temp file.  This should really be done when the transfer
+  // is cancelled, but there are other cancellation causes that shouldn't
+  // remove this. We need to improve those bits.
+  nsCOMPtr<nsILocalFile> tempFile;
+  internalDownload->GetTempFile(getter_AddRefs(tempFile));
+  if (tempFile) {
+    PRBool exists;
+    tempFile->Exists(&exists);
+    if (exists)
+      tempFile->Remove(PR_FALSE);
+  }
+
+  gObserverService->NotifyObservers(internalDownload, "dl-cancel", nsnull);
 
   // if there's a progress dialog open for the item,
   // we have to notify it that we're cancelling
   nsCOMPtr<nsIProgressDialog> dialog;
   internalDownload->GetDialog(getter_AddRefs(dialog));
   if (dialog) {
-    observer = do_QueryInterface(dialog);
-    rv = observer->Observe(download, "oncancel", nsnull);
+    nsCOMPtr<nsIObserver> observer = do_QueryInterface(dialog);
+    rv = observer->Observe(internalDownload, "oncancel", nsnull);
     if (NS_FAILED(rv)) return rv;
   }
-  
+
   return rv;
 }
 
@@ -976,7 +992,7 @@ nsDownloadManager::ValidateDownloadsContainer()
   // Now Remove all the bad downloads. 
   PRUint32 cnt;
   ary->Count(&cnt);
-  for (PRInt32 i = 0; i < cnt; ++i) {
+  for (PRUint32 i = 0; i < cnt; ++i) {
     nsCOMPtr<nsIRDFResource> download(do_QueryElementAt(ary, i));
 
     // Use the internal method because we know what we're doing! (We hope!)
@@ -1037,16 +1053,17 @@ nsDownloadManager::PauseResumeDownload(const PRUnichar* aPath, PRBool aPause)
   if (!mCurrDownloads.Exists(&key))
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIDownload> download;
   nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
-  internalDownload->QueryInterface(NS_GET_IID(nsIDownload), (void**) getter_AddRefs(download));
-  if (!download)
+  if (!internalDownload)
     return NS_ERROR_FAILURE;
 
   // Update download state in the DataSource
   nsCOMPtr<nsIRDFInt> intLiteral;
 
-  gRDFService->GetIntLiteral(aPause ? nsIDownloadManager::DOWNLOAD_PAUSED : nsIDownloadManager::DOWNLOAD_DOWNLOADING, getter_AddRefs(intLiteral));
+  gRDFService->GetIntLiteral(
+    aPause ? 
+    (PRInt32)nsIDownloadManager::DOWNLOAD_PAUSED : 
+    (PRInt32)nsIDownloadManager::DOWNLOAD_DOWNLOADING, getter_AddRefs(intLiteral));
 
   nsCOMPtr<nsIRDFResource> res;
   gRDFService->GetUnicodeResource(nsDependentString(aPath), getter_AddRefs(res));
@@ -1095,10 +1112,8 @@ nsDownloadManager::Open(nsIDOMWindow* aParent, const PRUnichar* aPath)
   if (!mCurrDownloads.Exists(&key))
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIDownload> download;
   nsDownload* internalDownload = NS_STATIC_CAST(nsDownload*, mCurrDownloads.Get(&key));
-  internalDownload->QueryInterface(NS_GET_IID(nsIDownload), (void**) getter_AddRefs(download));
-  if (!download)
+  if (!internalDownload)
     return NS_ERROR_FAILURE;
 
   // 2). Update the DataSource. 
@@ -1119,8 +1134,6 @@ nsDownloadManager::Open(nsIDOMWindow* aParent, const PRUnichar* aPath)
   if (pref)
     pref->GetIntPref(PREF_BDM_OPENDELAY, &delay);
 
-  params->AppendElement((void*)&delay);
-
   // 3). Look for an existing Download Manager window, if we find one we just 
   //     tell it that a new download has begun (we don't focus, that's 
   //     annoying), otherwise we need to open the window. We do this on a timer 
@@ -1138,18 +1151,21 @@ nsDownloadManager::OpenTimerCallback(nsITimer* aTimer, void* aClosure)
   nsVoidArray* params = (nsVoidArray*)aClosure;
   nsIDOMWindow* parent = (nsIDOMWindow*)params->ElementAt(0);
   nsDownload* download = (nsDownload*)params->ElementAt(1);
-  PRInt32 openDelay = *(PRInt32*)params->ElementAt(2);
   
   PRInt32 complete;
   download->GetPercentComplete(&complete);
   
-  // We only show the download window if the download is taking more than a non-tiny
-  // amount of time to complete. 
-  if (!openDelay || complete < 100) {
+  PRBool closeDM = PR_FALSE;
+  nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID));
+  if (pref)
+    pref->GetBoolPref(PREF_BDM_CLOSEWHENDONE, &closeDM);
+
+  // Check closeWhenDone pref before opening download manager
+  if (!closeDM || complete < 100) {
     PRBool focusDM = PR_FALSE;
     PRBool showDM = PR_TRUE;
     PRInt32 flashCount = -1;
-    nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID));
+
     if (pref) {
       pref->GetBoolPref(PREF_BDM_FOCUSWHENSTARTING, &focusDM);
 
@@ -1230,9 +1246,9 @@ nsDownloadManager::Observe(nsISupports* aSubject, const char* aTopic, const PRUn
   PRInt32 currDownloadCount = 0;
 
   if (nsCRT::strcmp(aTopic, "oncancel") == 0) {
-    nsCOMPtr<nsIProgressDialog> dialog = do_QueryInterface(aSubject);
+    nsCOMPtr<nsIDownload> dl = do_QueryInterface(aSubject);
     nsCOMPtr<nsIURI> target;
-    dialog->GetTarget(getter_AddRefs(target));
+    dl->GetTarget(getter_AddRefs(target));
 
     nsAutoString path;
     rv = GetFilePathFromURI(target, path);
@@ -1289,7 +1305,7 @@ nsDownloadManager::Observe(nsISupports* aSubject, const char* aTopic, const PRUn
         // Now Remove all the downloads. 
         PRUint32 cnt;
         ary->Count(&cnt);
-        for (PRInt32 i = 0; i < cnt; ++i) {
+        for (PRUint32 i = 0; i < cnt; ++i) {
           nsCOMPtr<nsIRDFResource> download(do_QueryElementAt(ary, i));
           // Here we use the internal RemoveDownload method, and only here
           // because this is _after_ the download table |mCurrDownloads| has been 
@@ -1340,6 +1356,16 @@ nsDownloadManager::Observe(nsISupports* aSubject, const char* aTopic, const PRUn
       gStoppingDownloads = PR_FALSE;
     }
   }
+  else if (nsCRT::strcmp(aTopic, "alertclickcallback") == 0)
+  {
+    // Attempt to locate a browser window to parent the download manager to
+    nsCOMPtr<nsIWindowMediator> wm = do_GetService(NS_WINDOWMEDIATOR_CONTRACTID, &rv);
+    nsCOMPtr<nsIDOMWindowInternal> browserWindow;
+    if (wm)
+      wm->GetMostRecentWindow(NS_LITERAL_STRING("navigator:browser").get(), getter_AddRefs(browserWindow));
+
+    return OpenDownloadManager(PR_TRUE, -1, nsnull, browserWindow);
+  }
   return NS_OK;
 }
 
@@ -1352,7 +1378,7 @@ nsDownloadManager::ConfirmCancelDownloads(PRInt32 aCount, nsISupportsPRBool* aCa
 {
   nsXPIDLString title, message, quitButton, dontQuitButton;
   
-  nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(kStringBundleServiceCID);
+  nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID);
   nsCOMPtr<nsIStringBundle> bundle;
   if (bundleService)
     bundleService->CreateBundle(DOWNLOAD_MANAGER_BUNDLE, getter_AddRefs(bundle));
@@ -1381,7 +1407,7 @@ nsDownloadManager::ConfirmCancelDownloads(PRInt32 aCount, nsISupportsPRBool* aCa
     wm->GetMostRecentWindow(NS_LITERAL_STRING("Download:Manager").get(), getter_AddRefs(dmWindow));
 
   // Show alert.
-  nsCOMPtr<nsIPromptService> prompter(do_GetService("@mozilla.org/embedcomp/prompt-service;1"));
+  nsCOMPtr<nsIPromptService> prompter(do_GetService(NS_PROMPTSERVICE_CONTRACTID));
   if (prompter) {
     PRInt32 flags = (nsIPromptService::BUTTON_TITLE_IS_STRING * nsIPromptService::BUTTON_POS_0) + (nsIPromptService::BUTTON_TITLE_IS_STRING * nsIPromptService::BUTTON_POS_1);
     PRBool nothing = PR_FALSE;
@@ -1411,31 +1437,6 @@ nsDownloadManager::GetHasActiveXPIOperations(PRBool* aHasOps)
   return NS_OK;
 }
 
-#ifdef XP_WIN 
-///////////////////////////////////////////////////////////////////////////////
-// nsIAlertListener
-NS_IMETHODIMP
-nsDownloadManager::OnAlertFinished(const PRUnichar* aAlertCookie)
-{
-  // Nothing to do here. 
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsDownloadManager::OnAlertClickCallback(const PRUnichar* aAlertCookie)
-{
-  nsresult rv;
-
-  // Attempt to locate a browser window to parent the download manager to
-  nsCOMPtr<nsIWindowMediator> wm = do_GetService(NS_WINDOWMEDIATOR_CONTRACTID, &rv);
-  nsCOMPtr<nsIDOMWindowInternal> browserWindow;
-  if (wm)
-    wm->GetMostRecentWindow(NS_LITERAL_STRING("navigator:browser").get(), getter_AddRefs(browserWindow));
-
-  return OpenDownloadManager(PR_TRUE, -1, nsnull, browserWindow);
-}
-#endif
-
 ///////////////////////////////////////////////////////////////////////////////
 // nsXPIProgressListener
 
@@ -1533,7 +1534,7 @@ nsXPIProgressListener::OnStateChange(PRUint32 aIndex, PRInt16 aState, PRInt32 aV
     if (!gStoppingDownloads) {
       nsCOMPtr<nsIStringBundleService> sbs(do_GetService("@mozilla.org/intl/stringbundle;1"));
       nsCOMPtr<nsIStringBundle> brandBundle, xpinstallBundle;
-      sbs->CreateBundle("chrome://global/locale/brand.properties", getter_AddRefs(brandBundle));
+      sbs->CreateBundle("chrome://branding/locale/brand.properties", getter_AddRefs(brandBundle));
       sbs->CreateBundle("chrome://mozapps/locale/xpinstall/xpinstallConfirm.properties", getter_AddRefs(xpinstallBundle));
 
       nsXPIDLString brandShortName, message, title;
@@ -1542,7 +1543,7 @@ nsXPIProgressListener::OnStateChange(PRUint32 aIndex, PRInt16 aState, PRInt32 aV
       xpinstallBundle->FormatStringFromName(NS_LITERAL_STRING("installComplete").get(), strings, 1, getter_Copies(message));
       xpinstallBundle->GetStringFromName(NS_LITERAL_STRING("installCompleteTitle").get(), getter_Copies(title));
 
-      nsCOMPtr<nsIPromptService> ps(do_GetService("@mozilla.org/embedcomp/prompt-service;1"));
+      nsCOMPtr<nsIPromptService> ps(do_GetService(NS_PROMPTSERVICE_CONTRACTID));
       ps->Alert(nsnull, title, message);
     }
 
@@ -1553,11 +1554,13 @@ nsXPIProgressListener::OnStateChange(PRUint32 aIndex, PRInt16 aState, PRInt32 aV
 }
 
 NS_IMETHODIMP
-nsXPIProgressListener::OnProgress(PRUint32 aIndex, PRUint32 aValue, PRUint32 aMaxValue)
+nsXPIProgressListener::OnProgress(PRUint32 aIndex, PRUint64 aValue, PRUint64 aMaxValue)
 {
   nsCOMPtr<nsIWebProgressListener> wpl(do_QueryElementAt(mDownloads, aIndex));
+  // XXX truncates 64-bit to 32 bit
   if (wpl) 
-    return wpl->OnProgressChange(nsnull, nsnull, 0, 0, aValue, aMaxValue);
+    return wpl->OnProgressChange(nsnull, nsnull, 0, 0, nsUint64(aValue),
+                                 nsUint64(aMaxValue));
   return NS_OK;
 }
 
@@ -1589,8 +1592,9 @@ nsresult
 nsDownloadsDataSource::LoadDataSource()
 {
   nsCOMPtr<nsIFile> downloadsFile;
-  NS_GetSpecialDirectory(NS_APP_DOWNLOADS_50_FILE, getter_AddRefs(downloadsFile));
-    
+  nsresult rv = NS_GetSpecialDirectory(NS_APP_DOWNLOADS_50_FILE, getter_AddRefs(downloadsFile));
+  if (NS_FAILED(rv)) return rv;
+
   nsCAutoString downloadsDB;
   NS_GetURLSpecFromFile(downloadsFile, downloadsDB);
 
@@ -1636,16 +1640,32 @@ nsDownloadsDataSource::GetTarget(nsIRDFResource* aSource, nsIRDFResource* aPrope
         nsCOMPtr<nsIRDFResource> res(do_QueryInterface(target));
         res->GetValue(getter_Copies(path));
 
-        nsCOMPtr<nsILocalFile> lf(do_CreateInstance("@mozilla.org/file/local;1"));
-        lf->InitWithNativePath(path);
-        nsCOMPtr<nsIIOService> ios(do_GetService("@mozilla.org/network/io-service;1"));
-        nsCOMPtr<nsIProtocolHandler> ph;
-        ios->GetProtocolHandler("file", getter_AddRefs(ph));
-        nsCOMPtr<nsIFileProtocolHandler> fph(do_QueryInterface(ph));
-
-        nsCAutoString fileURL;
-        fph->GetURLSpecFromFile(lf, fileURL);
+        // XXXmano: See bug 239948 and bug 335725, we need to do this
+        // until we use real URLs all the time.
+        PRBool alreadyURL = PR_FALSE;
+        nsCOMPtr<nsIURI> fileURI;
+        NS_NewURI(getter_AddRefs(fileURI), path);
+        if (fileURI) {
+          nsCOMPtr<nsIURL> url(do_QueryInterface(fileURI, &rv));
+          if (NS_SUCCEEDED(rv))
+            alreadyURL = PR_TRUE;
+        }
         
+        nsCAutoString fileURL;
+        if (alreadyURL) {
+          fileURL.Assign(path);
+        }
+        else {
+          nsCOMPtr<nsILocalFile> lf(do_CreateInstance("@mozilla.org/file/local;1"));
+          lf->InitWithNativePath(path);
+          nsCOMPtr<nsIIOService> ios(do_GetService("@mozilla.org/network/io-service;1"));
+          nsCOMPtr<nsIProtocolHandler> ph;
+          ios->GetProtocolHandler("file", getter_AddRefs(ph));
+          nsCOMPtr<nsIFileProtocolHandler> fph(do_QueryInterface(ph));
+
+          fph->GetURLSpecFromFile(lf, fileURL);
+        }
+
         nsAutoString iconURL(NS_LITERAL_STRING("moz-icon://"));
         AppendUTF8toUTF16(fileURL, iconURL);
         iconURL.AppendLiteral("?size=32");
@@ -1822,14 +1842,17 @@ nsDownloadsDataSource::FlushTo(const char* aURI)
 ///////////////////////////////////////////////////////////////////////////////
 // nsDownload
 
-NS_IMPL_ISUPPORTS3(nsDownload, nsIDownload, nsITransfer, nsIWebProgressListener)
+NS_IMPL_ISUPPORTS4(nsDownload, nsIDownload, nsITransfer, nsIWebProgressListener,
+                   nsIWebProgressListener2)
 
 nsDownload::nsDownload():mDownloadState(nsIDownloadManager::DOWNLOAD_NOTSTARTED),
                          mPercentComplete(0),
-                         mCurrBytes(0),
-                         mMaxBytes(0),
-                         mStartTime(0),
-                         mLastUpdate(-500)
+                         mCurrBytes(LL_ZERO),
+                         mMaxBytes(LL_ZERO),
+                         mStartTime(LL_ZERO),
+                         mLastUpdate(PR_Now() - (PRUint32)gInterval),
+                         mPaused(PR_FALSE),
+                         mSpeed(0)
 {
 }
 
@@ -1874,6 +1897,21 @@ nsDownload::GetDialog(nsIProgressDialog** aDialog)
   return NS_OK;
 }
 
+nsresult
+nsDownload::SetTempFile(nsILocalFile* aTempFile)
+{
+  mTempFile = aTempFile;
+  return NS_OK;
+}
+
+nsresult
+nsDownload::GetTempFile(nsILocalFile** aTempFile)
+{
+  *aTempFile = mTempFile;
+  NS_IF_ADDREF(*aTempFile);
+  return NS_OK;
+}
+
 DownloadState
 nsDownload::GetDownloadState()
 {
@@ -1899,9 +1937,9 @@ nsDownload::SetDownloadType(DownloadType aType)
 }
 
 nsresult
-nsDownload::SetPersist(nsIWebBrowserPersist* aPersist)
+nsDownload::SetCancelable(nsICancelable* aCancelable)
 {
-  mPersist = aPersist;
+  mCancelable = aCancelable;
   return NS_OK;
 }
 
@@ -1919,18 +1957,40 @@ nsDownload::SetTarget(nsIURI* aTarget)
   return NS_OK;
 }
 
+
 nsresult
-nsDownload::GetTransferInformation(PRInt32* aCurr, PRInt32* aMax)
+nsDownload::SetDisplayName(const PRUnichar* aDisplayName)
 {
-  *aCurr = mCurrBytes;
-  *aMax = mMaxBytes;
+  mDisplayName = aDisplayName;
+
+  nsCOMPtr<nsIRDFDataSource> ds;
+  mDownloadManager->GetDatasource(getter_AddRefs(ds));
+
+  nsCOMPtr<nsIRDFLiteral> nameLiteral;
+  nsCOMPtr<nsIRDFResource> res;
+  nsAutoString path;
+  nsresult rv = GetFilePathFromURI(mTarget, path);
+  if (NS_FAILED(rv)) return rv;
+
+  gRDFService->GetUnicodeResource(path, getter_AddRefs(res));
+  
+  gRDFService->GetLiteral(aDisplayName, getter_AddRefs(nameLiteral));
+  ds->Assert(res, gNC_Name, nameLiteral, PR_TRUE);
+
   return NS_OK;
 }
 
+nsDownload::TransferInformation
+nsDownload::GetTransferInformation()
+{
+  return TransferInformation(mCurrBytes, mMaxBytes);
+}
+
 nsresult
 nsDownload::SetStartTime(PRInt64 aStartTime)
 {
   mStartTime = aStartTime;
+  mLastUpdate = aStartTime;
   return NS_OK;
 }
 
@@ -1942,25 +2002,23 @@ nsDownload::SetMIMEInfo(nsIMIMEInfo *aMIMEInfo)
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-// nsIWebProgressListener
+// nsIWebProgressListener2
 
 NS_IMETHODIMP
-nsDownload::OnProgressChange(nsIWebProgress *aWebProgress,
-                             nsIRequest *aRequest,
-                             PRInt32 aCurSelfProgress,
-                             PRInt32 aMaxSelfProgress,
-                             PRInt32 aCurTotalProgress,
-                             PRInt32 aMaxTotalProgress)
+nsDownload::OnProgressChange64(nsIWebProgress *aWebProgress,
+                               nsIRequest *aRequest,
+                               PRInt64 aCurSelfProgress,
+                               PRInt64 aMaxSelfProgress,
+                               PRInt64 aCurTotalProgress,
+                               PRInt64 aMaxTotalProgress)
 {
-
   if (!mRequest)
     mRequest = aRequest; // used for pause/resume
 
   // filter notifications since they come in so frequently
-  PRTime delta;
   PRTime now = PR_Now();
-  LL_SUB(delta, now, mLastUpdate);
-  if (LL_CMP(delta, <, INTERVAL) &&  aMaxTotalProgress != -1 && aCurTotalProgress < aMaxTotalProgress)
+  nsInt64 delta = now - mLastUpdate;
+  if (delta < gInterval)
     return NS_OK;
 
   mLastUpdate = now;
@@ -1974,13 +2032,28 @@ nsDownload::OnProgressChange(nsIWebProgress *aWebProgress,
     mDownloadManager->DownloadStarted(path.get());
   }
 
+  // Calculate the speed using the elapsed delta time and bytes downloaded
+  // during that time for more accuracy.
+  double elapsedSecs = double(delta) / PR_USEC_PER_SEC;
+  if (elapsedSecs > 0) {
+    nsUint64 curTotalProgress = (PRUint64)aCurTotalProgress;
+    nsUint64 diffBytes = curTotalProgress - nsUint64(mCurrBytes);
+    double speed = double(diffBytes) / elapsedSecs;
+    if (LL_IS_ZERO(mCurrBytes))
+      mSpeed = speed;
+    else {
+      // Calculate 'smoothed average' of 10 readings.
+      mSpeed = mSpeed * 0.9 + speed * 0.1;
+    }
+  }
+
   if (aMaxTotalProgress > 0)
     mPercentComplete = (PRInt32)((PRFloat64)aCurTotalProgress * 100 / aMaxTotalProgress + .5);
   else
     mPercentComplete = -1;
 
-  mCurrBytes = (PRInt32)((PRFloat64)aCurTotalProgress / 1024.0 + .5);
-  mMaxBytes = (PRInt32)((PRFloat64)aMaxTotalProgress / 1024 + .5);
+  mCurrBytes = aCurTotalProgress;
+  mMaxBytes = aMaxTotalProgress;
 
   if (mDownloadManager->NeedsUIUpdate()) {
     nsCOMPtr<nsIDownloadProgressListener> dpl;
@@ -1992,6 +2065,23 @@ nsDownload::OnProgressChange(nsIWebProgress *aWebProgress,
   }
 
   return NS_OK;
+
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// nsIWebProgressListener
+
+NS_IMETHODIMP
+nsDownload::OnProgressChange(nsIWebProgress *aWebProgress,
+                             nsIRequest *aRequest,
+                             PRInt32 aCurSelfProgress,
+                             PRInt32 aMaxSelfProgress,
+                             PRInt32 aCurTotalProgress,
+                             PRInt32 aMaxTotalProgress)
+{
+  return OnProgressChange64(aWebProgress, aRequest,
+                            aCurSelfProgress, aMaxSelfProgress,
+                            aCurTotalProgress, aMaxTotalProgress);
 }
 
 NS_IMETHODIMP
@@ -2018,7 +2108,7 @@ nsDownload::OnStatusChange(nsIWebProgress *aWebProgress,
     // Get title for alert.
     nsXPIDLString title;
     
-    nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(kStringBundleServiceCID, &rv);
+    nsCOMPtr<nsIStringBundleService> bundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
     nsCOMPtr<nsIStringBundle> bundle;
     if (bundleService)
       rv = bundleService->CreateBundle(DOWNLOAD_MANAGER_BUNDLE, getter_AddRefs(bundle));
@@ -2045,10 +2135,11 @@ nsDownload::OnStateChange(nsIWebProgress* aWebProgress,
                           nsIRequest* aRequest, PRUint32 aStateFlags,
                           nsresult aStatus)
 {
-  if (aStateFlags & STATE_START)    
-    mStartTime = PR_Now();  
+  // Record the start time only if it hasn't been set.
+  if (LL_IS_ZERO(mStartTime) && (aStateFlags & STATE_START))
+    SetStartTime(PR_Now());
 
-  // When we break the ref cycle with mPersist, we don't want to lose
+  // When we break the ref cycle with mCancelable, we don't want to lose
   // access to out member vars!
   nsCOMPtr<nsIDownload> kungFuDeathGrip;
   CallQueryInterface(this, NS_STATIC_CAST(nsIDownload**,
@@ -2057,6 +2148,9 @@ nsDownload::OnStateChange(nsIWebProgress* aWebProgress,
   // We need to update mDownloadState before updating the dialog, because
   // that will close and call CancelDownload if it was the last open window.
   nsresult rv = NS_OK;
+
+  nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID));
+
   if (aStateFlags & STATE_STOP) {
     if (nsDownloadManager::IsInFinalStage(mDownloadState)) {
       if (mDownloadState != nsIXPInstallManagerUI::INSTALL_INSTALLING)
@@ -2064,23 +2158,26 @@ nsDownload::OnStateChange(nsIWebProgress* aWebProgress,
       else
         mDownloadState = nsIXPInstallManagerUI::INSTALL_FINISHED;
 
+      // Set file size at the end of a tranfer (for unknown transfer amounts)
+      if (mMaxBytes == -1)
+        mMaxBytes = mCurrBytes;
+
       // Files less than 1Kb shouldn't show up as 0Kb.
-      if (mMaxBytes==0)
-        mMaxBytes = 1;
-      mCurrBytes = mMaxBytes;
+      if (mMaxBytes < 1024) {
+        mCurrBytes = 1024;
+        mMaxBytes  = 1024;
+      }
+
       mPercentComplete = 100;
 
       nsAutoString path;
       rv = GetFilePathFromURI(mTarget, path);
       // can't do an early return; have to break reference cycle below
-      if (NS_SUCCEEDED(rv)) {
+      if (NS_SUCCEEDED(rv))
         mDownloadManager->DownloadEnded(path.get(), nsnull);
-      }
 
-#ifdef XP_WIN
       // Master pref to control this function. 
       PRBool showTaskbarAlert = PR_FALSE;
-      nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID));
       if (pref)
         pref->GetBoolPref(PREF_BDM_SHOWALERTONCOMPLETE, &showTaskbarAlert);
 
@@ -2112,35 +2209,54 @@ nsDownload::OnStateChange(nsIWebProgress* aWebProgress,
             // there's no reason to make the text clickable because if it is, they'll
             // click open the download manager and the items they downloaded will have
             // been removed. 
-            alerts->ShowAlertNotification(DOWNLOAD_MANAGER_ALERT_ICON, title, message, !removeWhenDone, 
-                                          EmptyString().get(), mDownloadManager);
+            alerts->ShowAlertNotification(NS_LITERAL_STRING(DOWNLOAD_MANAGER_ALERT_ICON), title, message, !removeWhenDone, 
+                                          EmptyString(), mDownloadManager);
           }
         }
       }
-#endif
+
+      gObserverService->NotifyObservers(NS_STATIC_CAST(nsIDownload *, this), "dl-done", nsnull);
     }
 
-    gObserverService->NotifyObservers(NS_STATIC_CAST(nsIDownload *, this), "dl-done", nsnull);
+    nsAutoString path;
+    rv = GetFilePathFromURI(mTarget, path);
+    if (NS_FAILED(rv))
+      return rv;
+
+#ifdef XP_WIN
+    PRBool addToRecentDocs = PR_TRUE;
+    if (pref)
+      pref->GetBoolPref(PREF_BDM_ADDTORECENTDOCS, &addToRecentDocs);
+
+    if (addToRecentDocs) {
+      LPSHELLFOLDER lpShellFolder = NULL;
+
+      if (SUCCEEDED(::SHGetDesktopFolder(&lpShellFolder))) {
+        PRUnichar *filePath = ToNewUnicode(path);
+        LPITEMIDLIST lpItemIDList = NULL;
+        if (SUCCEEDED(lpShellFolder->ParseDisplayName(NULL, NULL, filePath, NULL, &lpItemIDList, NULL))) {
+          ::SHAddToRecentDocs(SHARD_PIDL, lpItemIDList);
+          ::CoTaskMemFree(lpItemIDList);
+        }
+        nsMemory::Free(filePath);
+        lpShellFolder->Release();
+      }
+    }
+#endif
 
     // break the cycle we created in AddDownload
-    if (mPersist)
-      mPersist->SetProgressListener(nsnull);
+    mCancelable = nsnull;
 
     // Now remove the download if the user's retention policy is "Remove when Done"
-    if (mDownloadManager->GetRetentionBehavior() == 0) {
-      nsAutoString path;
-      GetFilePathFromURI(mTarget, path);
-
+    if (mDownloadManager->GetRetentionBehavior() == 0)
       mDownloadManager->RemoveDownload(path.get());
-    }
   }
 
   if (mDownloadManager->NeedsUIUpdate()) {
     nsCOMPtr<nsIDownloadProgressListener> dpl;
     mDownloadManager->GetInternalListener(getter_AddRefs(dpl));
-    if (dpl) {
+    if (dpl)
       dpl->OnStateChange(aWebProgress, aRequest, aStateFlags, aStatus, this);
-    }
   }
 
   return rv;
@@ -2159,37 +2275,16 @@ nsDownload::OnSecurityChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP
 nsDownload::Init(nsIURI* aSource,
                  nsIURI* aTarget,
-                 const PRUnichar* aDisplayName,
+                 const nsAString& aDisplayName,
                  nsIMIMEInfo *aMIMEInfo,
-                 PRInt64 aStartTime,
-                 nsIWebBrowserPersist* aPersist)
+                 PRTime aStartTime,
+                 nsILocalFile* aTempFile,
+                 nsICancelable* aCancelable)
 {
   NS_WARNING("Huh...how did we get here?!");
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsDownload::SetDisplayName(const PRUnichar* aDisplayName)
-{
-  mDisplayName = aDisplayName;
-
-  nsCOMPtr<nsIRDFDataSource> ds;
-  mDownloadManager->GetDatasource(getter_AddRefs(ds));
-
-  nsCOMPtr<nsIRDFLiteral> nameLiteral;
-  nsCOMPtr<nsIRDFResource> res;
-  nsAutoString path;
-  nsresult rv = GetFilePathFromURI(mTarget, path);
-  if (NS_FAILED(rv)) return rv;
-
-  gRDFService->GetUnicodeResource(path, getter_AddRefs(res));
-  
-  gRDFService->GetLiteral(aDisplayName, getter_AddRefs(nameLiteral));
-  ds->Assert(res, gNC_Name, nameLiteral, PR_TRUE);
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsDownload::GetDisplayName(PRUnichar** aDisplayName)
 {
@@ -2197,6 +2292,14 @@ nsDownload::GetDisplayName(PRUnichar** aDisplayName)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDownload::GetCancelable(nsICancelable** aCancelable)
+{
+  *aCancelable = mCancelable;
+  NS_IF_ADDREF(*aCancelable);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsDownload::GetTarget(nsIURI** aTarget)
 {
@@ -2213,14 +2316,6 @@ nsDownload::GetSource(nsIURI** aSource)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsDownload::GetPersist(nsIWebBrowserPersist** aPersist)
-{
-  *aPersist = mPersist;
-  NS_IF_ADDREF(*aPersist);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsDownload::GetStartTime(PRInt64* aStartTime)
 {
@@ -2236,30 +2331,16 @@ nsDownload::GetPercentComplete(PRInt32* aPercentComplete)
 }
 
 NS_IMETHODIMP
-nsDownload::SetListener(nsIWebProgressListener* aListener)
+nsDownload::GetAmountTransferred(PRUint64* aAmountTransferred)
 {
+  *aAmountTransferred = mCurrBytes;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsDownload::GetListener(nsIWebProgressListener** aListener)
+nsDownload::GetSize(PRUint64* aSize)
 {
-  *aListener = nsnull;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsDownload::SetObserver(nsIObserver* aObserver)
-{
-  mObserver = aObserver;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsDownload::GetObserver(nsIObserver** aObserver)
-{
-  *aObserver = mObserver;
-  NS_IF_ADDREF(*aObserver);
+  *aSize = mMaxBytes;
   return NS_OK;
 }
 
@@ -2286,6 +2367,13 @@ nsDownload::GetTargetFile(nsILocalFile** aTargetFile)
   return rv;
 }
 
+NS_IMETHODIMP
+nsDownload::GetSpeed(double* aSpeed)
+{
+  *aSpeed = mSpeed;
+  return NS_OK;
+}
+
 void
 nsDownload::Pause(PRBool aPaused)
 {
@@ -2293,10 +2381,12 @@ nsDownload::Pause(PRBool aPaused)
     if (mRequest) {
       if (aPaused) {
         mRequest->Suspend();
+        mPaused = PR_TRUE;
         mDownloadState = nsIDownloadManager::DOWNLOAD_PAUSED;
       }
       else {
         mRequest->Resume();
+        mPaused = PR_FALSE;
         mDownloadState = nsIDownloadManager::DOWNLOAD_DOWNLOADING;
       }
     }
@@ -2308,4 +2398,3 @@ nsDownload::IsPaused()
 {
   return mPaused;
 }
-
