@@ -1,29 +1,46 @@
 # -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 #
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
+# ***** BEGIN LICENSE BLOCK *****
+# Version: MPL 1.1/GPL 2.0/LGPL 2.1
 #
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
+# The contents of this file are subject to the Mozilla Public License Version
+# 1.1 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+# http://www.mozilla.org/MPL/
+#
+# Software distributed under the License is distributed on an "AS IS" basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+# for the specific language governing rights and limitations under the
+# License.
 #
 # The Original Code is mozilla.org code.
 #
-# The Initial Developer of the Original Code is Netscape Communications
-# Corporation.  Portions created by Netscape are
-# Copyright (C) 2000 Netscape Communications Corporation. All
-# Rights Reserved.
+# The Initial Developer of the Original Code is
+# Netscape Communications Corporation.
+# Portions created by the Initial Developer are Copyright (C) 2000
+# the Initial Developer. All Rights Reserved.
 #
-# Contributor(s): 
-#  Stuart Parmenter <pavlov@netscape.com>
-#  Brian Ryner <bryner@brianryner.com>
-#  Jan Varga <varga@ku.sk>
-#  Peter Annema <disttsc@bart.nl>
-#  Johann Petrak <johann@ai.univie.ac.at>
-#  Akkana Peck <akkana@netscape.com>
+# Contributor(s):
+#   Stuart Parmenter <pavlov@netscape.com>
+#   Brian Ryner <bryner@brianryner.com>
+#   Jan Varga <varga@nixcorp.com>
+#   Peter Annema <disttsc@bart.nl>
+#   Johann Petrak <johann@ai.univie.ac.at>
+#   Akkana Peck <akkana@netscape.com>
+#
+# Alternatively, the contents of this file may be used under the terms of
+# either the GNU General Public License Version 2 or later (the "GPL"), or
+# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+# in which case the provisions of the GPL or the LGPL are applicable instead
+# of those above. If you wish to allow use of your version of this file only
+# under the terms of either the GPL or the LGPL, and not to allow others to
+# use your version of this file under the terms of the MPL, indicate your
+# decision by deleting the provisions above and replace them with the notice
+# and other provisions required by the GPL or the LGPL. If you do not delete
+# the provisions above, a recipient may use your version of this file under
+# the terms of any one of the MPL, the GPL or the LGPL.
+#
+# ***** END LICENSE BLOCK *****
 
 const nsIFilePicker       = Components.interfaces.nsIFilePicker;
 const nsIDirectoryServiceProvider = Components.interfaces.nsIDirectoryServiceProvider;
@@ -67,6 +84,7 @@ function filepickerLoad() {
     if (o.displayDirectory) {
       const directory = o.displayDirectory.path;
     }
+
     const initialText = o.defaultString;
     const filterTitles = o.filters.titles;
     const filterTypes = o.filters.types;
@@ -87,6 +105,7 @@ function filepickerLoad() {
   if (filePickerMode == nsIFilePicker.modeGetFolder) {
     var textInputLabel = document.getElementById("textInputLabel");
     textInputLabel.value = gFilePickerBundle.getString("dirTextInputLabel");
+    textInputLabel.accessKey = gFilePickerBundle.getString("dirTextInputAccesskey");
   }
   
   if ((filePickerMode == nsIFilePicker.modeOpen) ||
@@ -121,13 +140,18 @@ function filepickerLoad() {
     treeView.showOnlyDirectories = true;
   }
 
+  // set up the right icon
+  if (filePickerMode == nsIFilePicker.modeSave)
+    okButton.setAttribute("icon","save");
+  else
+    okButton.setAttribute("icon","open");
+
   // start out with a filename sort
   handleColumnClick("FilenameColumn");
 
   document.documentElement.setAttribute("ondialogcancel", "return onCancel();");
   try {
-    var buttonLabel = getOKAction();
-    okButton.setAttribute("label", buttonLabel);
+    setOKAction();
   } catch (exception) {
     // keep it set to "OK"
   }
@@ -496,8 +520,9 @@ function onTreeFocus(event) {
   onFileSelected(treeView.selectedFiles);
 }
 
-function getOKAction(file) {
+function setOKAction(file) {
   var buttonLabel;
+  var buttonIcon = "open"; // used in all but one case
 
   if (file && file.isDirectory() && filePickerMode != nsIFilePicker.modeGetFolder) {
     document.documentElement.setAttribute("ondialogaccept", "return openOnOK();");
@@ -515,11 +540,12 @@ function getOKAction(file) {
       break;
     case nsIFilePicker.modeSave:
       buttonLabel = gFilePickerBundle.getString("saveButtonLabel");
+      buttonIcon = "save";
       break;
     }
   }
-
-  return buttonLabel;
+  okButton.setAttribute("label", buttonLabel);
+  okButton.setAttribute("icon", buttonIcon);
 }
 
 function onSelect(event) {
@@ -558,8 +584,7 @@ function onFileSelected(/* nsIArray */ selectedFileList) {
   }
 
   if (validFileSelected) {
-    var buttonLabel = getOKAction(file);
-    okButton.setAttribute("label", buttonLabel);
+    setOKAction(file);
     okButton.disabled = invalidSelection;
   } else
     okButton.disabled = (textInput.value == "");
@@ -586,8 +611,7 @@ function addToTextFieldValue(path)
 }
 
 function onTextFieldFocus() {
-  var buttonLabel = getOKAction(null);
-  okButton.setAttribute("label", buttonLabel);
+  setOKAction(null);
   doEnabling();
 }
 
@@ -730,6 +754,7 @@ function gotoDirectory(directory) {
     textInput.value = "";
   }
   textInput.focus();
+  textInput.setAttribute("autocompletesearchparam", directory.path);
   sfile = directory;
 }
 
@@ -739,7 +764,7 @@ function toggleShowHidden(event) {
 
 // from the current directory and whatever was entered
 // in the entry field, try to make a new path. This
-// uses "/" as the directory seperator, "~" as a shortcut
+// uses "/" as the directory separator, "~" as a shortcut
 // for the home directory (but only when seen at the start
 // of a path), and ".." to denote the parent directory.
 // returns an array of the files listed,
