@@ -37,22 +37,23 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsPasswordManager.h"
+#include "nsSingleSignonPrompt.h"
 #include "nsIFile.h"
 #include "nsNetUtil.h"
 #include "nsILineInputStream.h"
 #include "plbase64.h"
 #include "nsISecretDecoderRing.h"
-#include "nsIPassword.h"
+#include "nsIPasswordInternal.h"
 #include "nsIPrompt.h"
+#include "nsIPromptService2.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "prmem.h"
 #include "nsIStringBundle.h"
-#include "nsArray.h"
+#include "nsIMutableArray.h"
 #include "nsICategoryManager.h"
 #include "nsIObserverService.h"
-#include "nsIDocumentLoader.h"
 #include "nsIWebProgress.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMWindow.h"
@@ -71,10 +72,14 @@
 #include "nsIAutoCompleteResult.h"
 #include "nsIPK11TokenDB.h"
 #include "nsIPK11Token.h"
+#include "nsUnicharUtils.h"
+#include "nsCOMArray.h"
+#include "nsEmbedCID.h"
 
 static const char kPMPropertiesURL[] = "chrome://passwordmgr/locale/passwordmgr.properties";
 static PRBool sRememberPasswords = PR_FALSE;
 static PRBool sPrefsInitialized = PR_FALSE;
+static PRBool sPasswordsLoaded = PR_FALSE;
 
 static nsIStringBundle* sPMBundle;
 static nsISecretDecoderRing* sDecoderRing;
@@ -106,7 +111,7 @@ public:
   ~SignonHashEntry() { delete head; }
 };
 
-class nsPasswordManager::PasswordEntry : public nsIPassword
+class nsPasswordManager::PasswordEntry : public nsIPasswordInternal
 {
 public:
   PasswordEntry(const nsACString& aKey, SignonDataEntry* aData);
@@ -114,16 +119,20 @@ public:
 
   NS_DECL_ISUPPORTS
   NS_DECL_NSIPASSWORD
+  NS_DECL_NSIPASSWORDINTERNAL
 
 protected:
 
   nsCString mHost;
   nsString  mUser;
+  nsString  mUserField;
   nsString  mPassword;
+  nsString  mPasswordField;
   PRBool    mDecrypted[2];
 };
 
-NS_IMPL_ISUPPORTS1(nsPasswordManager::PasswordEntry, nsIPassword)
+NS_IMPL_ISUPPORTS2(nsPasswordManager::PasswordEntry, nsIPassword,
+                   nsIPasswordInternal)
 
 nsPasswordManager::PasswordEntry::PasswordEntry(const nsACString& aKey,
                                                 SignonDataEntry* aData)
@@ -133,7 +142,9 @@ nsPasswordManager::PasswordEntry::PasswordEntry(const nsACString& aKey,
 
   if (aData) {
     mUser.Assign(aData->userValue);
+    mUserField.Assign(aData->userField);
     mPassword.Assign(aData->passValue);
+    mPasswordField.Assign(aData->passField);
   }
 }
 
@@ -172,7 +183,19 @@ nsPasswordManager::PasswordEntry::GetPassword(nsAString& aPassword)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPasswordManager::PasswordEntry::GetUserFieldName(nsAString& aField)
+{
+  aField.Assign(mUserField);
+  return NS_OK;
+}
 
+NS_IMETHODIMP
+nsPasswordManager::PasswordEntry::GetPasswordFieldName(nsAString& aField)
+{
+  aField.Assign(mPasswordField);
+  return NS_OK;
+}
 
 
 
@@ -186,10 +209,10 @@ NS_INTERFACE_MAP_BEGIN(nsPasswordManager)
   NS_INTERFACE_MAP_ENTRY(nsIFormSubmitObserver)
   NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMFocusListener)
-  NS_INTERFACE_MAP_ENTRY(nsIDOMLoadListener)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMEventListener, nsIDOMFocusListener)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIPasswordManager)
+  NS_INTERFACE_MAP_ENTRY(nsIPromptFactory)
 NS_INTERFACE_MAP_END
 
 nsPasswordManager::nsPasswordManager()
@@ -218,6 +241,11 @@ nsPasswordManager::GetInstance()
     }
   }
 
+  // We fail to load passwords during early initialization
+  // This wrapper function allows us to handle that error and defer
+  // password loading until later
+  sPasswordManager->LoadPasswords();
+
   NS_ADDREF(sPasswordManager);   // addref the return result
   return sPasswordManager;
 }
@@ -239,7 +267,7 @@ nsPasswordManager::Init()
 
   mPrefBranch->GetBoolPref("rememberSignons", &sRememberPasswords);
 
-  nsCOMPtr<nsIPrefBranchInternal> branchInternal = do_QueryInterface(mPrefBranch);
+  nsCOMPtr<nsIPrefBranch2> branchInternal = do_QueryInterface(mPrefBranch);
 
   // Have the pref service hold a weak reference; the service manager
   // will be holding a strong reference.
@@ -253,29 +281,12 @@ nsPasswordManager::Init()
   nsCOMPtr<nsIObserverService> obsService = do_GetService("@mozilla.org/observer-service;1");
   NS_ASSERTION(obsService, "No observer service");
 
-  obsService->AddObserver(this, NS_FORMSUBMIT_SUBJECT, PR_TRUE);
+  obsService->AddObserver(this, NS_EARLYFORMSUBMIT_SUBJECT, PR_TRUE);
 
-  nsCOMPtr<nsIDocumentLoader> docLoaderService = do_GetService(NS_DOCUMENTLOADER_SERVICE_CONTRACTID);
-  NS_ASSERTION(docLoaderService, "No document loader service");
-
-  nsCOMPtr<nsIWebProgress> progress = do_QueryInterface(docLoaderService);
-  NS_ASSERTION(progress, "docloader service does not implement nsIWebProgress");
+  nsCOMPtr<nsIWebProgress> progress = do_GetService(NS_DOCUMENTLOADER_SERVICE_CONTRACTID);
+  NS_ASSERTION(progress, "No web progress service");
 
   progress->AddProgressListener(this, nsIWebProgress::NOTIFY_STATE_DOCUMENT);
-
-  // Now read in the signon file
-  nsXPIDLCString signonFile;
-  mPrefBranch->GetCharPref("SignonFileName", getter_Copies(signonFile));
-  NS_ASSERTION(!signonFile.IsEmpty(), "Fallback for signon filename not present");
-
-  NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mSignonFile));
-  mSignonFile->AppendNative(signonFile);
-
-  nsCAutoString path;
-  mSignonFile->GetNativePath(path);
-
-  ReadPasswords(mSignonFile);
-
   return NS_OK;
 }
 
@@ -313,6 +324,13 @@ nsPasswordManager::Register(nsIComponentManager* aCompMgr,
                            PR_TRUE,
                            getter_Copies(prevEntry));
 
+  catman->AddCategoryEntry("app-startup",
+                           "Password Manager",
+                           NS_PASSWORDMANAGER_CONTRACTID,
+                           PR_TRUE,
+                           PR_TRUE,
+                           getter_Copies(prevEntry));
+
   return NS_OK;
 }
 
@@ -450,8 +468,9 @@ NS_IMETHODIMP
 nsPasswordManager::GetEnumerator(nsISimpleEnumerator** aEnumerator)
 {
   // Build an array out of the hashtable
-  nsCOMPtr<nsIMutableArray> signonArray;
-  NS_NewArray(getter_AddRefs(signonArray));
+  nsCOMPtr<nsIMutableArray> signonArray =
+    do_CreateInstance(NS_ARRAY_CONTRACTID);
+  NS_ENSURE_STATE(signonArray);
 
   mSignonTable.EnumerateRead(BuildArrayEnumerator, signonArray);
 
@@ -475,8 +494,9 @@ NS_IMETHODIMP
 nsPasswordManager::GetRejectEnumerator(nsISimpleEnumerator** aEnumerator)
 {
   // Build an array out of the hashtable
-  nsCOMPtr<nsIMutableArray> rejectArray;
-  NS_NewArray(getter_AddRefs(rejectArray));
+  nsCOMPtr<nsIMutableArray> rejectArray =
+    do_CreateInstance(NS_ARRAY_CONTRACTID);
+  NS_ENSURE_STATE(rejectArray);
 
   mRejectTable.EnumerateRead(BuildRejectArrayEnumerator, rejectArray);
 
@@ -639,7 +659,7 @@ nsPasswordManager::ReadPasswords(nsIFile* aPasswordFile)
   if (NS_FAILED(rv))
     return NS_OK;
 
-  if (!utf8Buffer.Equals(NS_LITERAL_CSTRING("#2c"))) {
+  if (!utf8Buffer.Equals("#2c")) {
     NS_ERROR("Unexpected version header in signon file");
     return NS_OK;
   }
@@ -747,7 +767,13 @@ nsPasswordManager::Observe(nsISupports* aSubject,
     NS_ASSERTION(branch == mPrefBranch, "unexpected pref change notification");
 
     branch->GetBoolPref("rememberSignons", &sRememberPasswords);
-  }
+  } else if (!strcmp(aTopic, "app-startup")) {
+    nsCOMPtr<nsIObserverService> obsService = do_GetService("@mozilla.org/observer-service;1");
+    NS_ASSERTION(obsService, "No observer service");
+
+    obsService->AddObserver(this, "profile-after-change", PR_TRUE);
+  } else if (!strcmp(aTopic, "profile-after-change"))
+    nsCOMPtr<nsIPasswordManager> pm = do_GetService(NS_PASSWORDMANAGER_CONTRACTID);
 
   return NS_OK;
 }
@@ -759,10 +785,9 @@ nsPasswordManager::OnStateChange(nsIWebProgress* aWebProgress,
                                  PRUint32 aStateFlags,
                                  nsresult aStatus)
 {
-  // We're only interested in successful document loads.
-  if (NS_FAILED(aStatus) ||
-      !(aStateFlags & nsIWebProgressListener::STATE_IS_DOCUMENT) ||
-      !(aStateFlags & nsIWebProgressListener::STATE_STOP))
+  if (!(aStateFlags & nsIWebProgressListener::STATE_IS_DOCUMENT) ||
+      !(aStateFlags & nsIWebProgressListener::STATE_TRANSFERRING) ||
+      NS_FAILED(aStatus))
     return NS_OK;
 
   // Don't do anything if the global signon pref is disabled
@@ -777,170 +802,19 @@ nsPasswordManager::OnStateChange(nsIWebProgress* aWebProgress,
   domWin->GetDocument(getter_AddRefs(domDoc));
   NS_ASSERTION(domDoc, "DOM window should always have a document!");
 
-  // For now, only prefill forms in HTML documents.
+  // For now, only prepare to prefill forms in HTML documents.
   nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(domDoc);
   if (!htmlDoc)
     return NS_OK;
 
-  nsCOMPtr<nsIDOMHTMLCollection> forms;
-  htmlDoc->GetForms(getter_AddRefs(forms));
-  
-  nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
+  if (aStateFlags & nsIWebProgressListener::STATE_RESTORING)
+      return FillDocument(domDoc);
 
-  nsCAutoString realm;
-  if (!GetPasswordRealm(doc->GetDocumentURI(), realm))
-    return NS_OK;
-
-  SignonHashEntry* hashEnt;
-  if (!mSignonTable.Get(realm, &hashEnt))
-    return NS_OK;
-
-  PRUint32 formCount;
-  forms->GetLength(&formCount);
-
-  // We can auto-prefill the username and password if there is only
-  // one stored login that matches the username and password field names
-  // on the form in question.  Note that we only need to worry about a
-  // single login per form.
-
-  for (PRUint32 i = 0; i < formCount; ++i) {
-    nsCOMPtr<nsIDOMNode> formNode;
-    forms->Item(i, getter_AddRefs(formNode));
-
-    nsCOMPtr<nsIForm> form = do_QueryInterface(formNode);
-    SignonDataEntry* firstMatch = nsnull;
-    PRBool attachedToInput = PR_FALSE;
-    nsCOMPtr<nsIDOMHTMLInputElement> userField, passField;
-    nsCOMPtr<nsIDOMHTMLInputElement> temp;
-    nsAutoString fieldType;
-
-    for (SignonDataEntry* e = hashEnt->head; e; e = e->next) {
-      
-      nsCOMPtr<nsISupports> foundNode;
-      form->ResolveName(e->userField, getter_AddRefs(foundNode));
-      temp = do_QueryInterface(foundNode);
-
-      nsAutoString oldUserValue;
-
-      if (temp) {
-        temp->GetType(fieldType);
-        if (!fieldType.Equals(NS_LITERAL_STRING("text")))
-          continue;
-
-        temp->GetValue(oldUserValue);
-        userField = temp;
-      } else {
-        continue;
-      }
-
-      if (!(e->passField).IsEmpty()) {
-        form->ResolveName(e->passField, getter_AddRefs(foundNode));
-        temp = do_QueryInterface(foundNode);
-      }
-      else {
-        // No password field name was supplied, try to locate one in the form.
-        nsCOMPtr<nsIFormControl> fc(do_QueryInterface(foundNode));
-        PRInt32 index = -1;
-        form->IndexOfControl(fc, &index);
-        if (index >= 0) {
-          PRUint32 count;
-          form->GetElementCount(&count);
-
-          PRUint32 i;
-          temp = nsnull;
-
-          // Search forwards
-          nsCOMPtr<nsIFormControl> passField;
-          for (i = index + 1; i < count; ++i) {
-            form->GetElementAt(i, getter_AddRefs(passField));
-
-            if (passField && passField->GetType() == NS_FORM_INPUT_PASSWORD) {
-              foundNode = passField;
-              temp = do_QueryInterface(foundNode);
-            }
-          }
-
-          if (!temp && index != 0) {
-            // Search backwards
-            i = index;
-            do {
-              form->GetElementAt(i, getter_AddRefs(passField));
-
-              if (passField && passField->GetType() == NS_FORM_INPUT_PASSWORD) {
-                foundNode = passField;
-                temp = do_QueryInterface(foundNode);
-              }
-            } while (i-- != 0);
-          }
-        }
-      }
-
-      nsAutoString oldPassValue;
-
-      if (temp) {
-        temp->GetType(fieldType);
-        if (!fieldType.Equals(NS_LITERAL_STRING("password")))
-          continue;
-
-        temp->GetValue(oldPassValue);
-        passField = temp;
-      } else {
-        continue;
-      }
-
-      if (!oldUserValue.IsEmpty()) {
-        // The page has prefilled a username.
-        // If it matches any of our saved usernames, prefill the password
-        // for that username.  If there are multiple saved usernames,
-        // we will also attach the autocomplete listener.
-
-        nsAutoString userValue;
-        if (NS_FAILED(DecryptData(e->userValue, userValue)))
-          goto done;
-
-        if (userValue.Equals(oldUserValue)) {
-          nsAutoString passValue;
-          if (NS_FAILED(DecryptData(e->passValue, passValue)))
-            goto done;
-
-          passField->SetValue(passValue);
-        }
-      }
-
-      if (firstMatch && !attachedToInput) {
-        // We've found more than one possible signon for this form.
-
-        // Listen for blur and autocomplete events on the username field so
-        // that we can attempt to prefill the password after the user has
-        // entered the username.
-
-        AttachToInput(userField);
-        attachedToInput = PR_TRUE;
-      } else {
-        firstMatch = e;
-      }
-    }
-
-    if (firstMatch && !attachedToInput) {
-      nsAutoString buffer;
-
-      if (NS_FAILED(DecryptData(firstMatch->userValue, buffer)))
-        goto done;
-
-      userField->SetValue(buffer);
-
-      if (NS_FAILED(DecryptData(firstMatch->passValue, buffer)))
-        goto done;
-
-      passField->SetValue(buffer);
-      AttachToInput(userField);
-    }
-  }
-
- done:
-  nsCOMPtr<nsIDOMEventTarget> targ = do_QueryInterface(domDoc);
-  targ->AddEventListener(NS_LITERAL_STRING("unload"),
-                         NS_STATIC_CAST(nsIDOMLoadListener*, this), PR_FALSE);
+  nsCOMPtr<nsIDOMEventTarget> targDoc = do_QueryInterface(domDoc);
+  nsCOMPtr<nsIDOMEventTarget> targWin = do_QueryInterface(domWin);
+  nsIDOMEventListener* listener = NS_STATIC_CAST(nsIDOMFocusListener*, this);
+  targDoc->AddEventListener(NS_LITERAL_STRING("DOMContentLoaded"), listener, PR_FALSE);
+  targWin->AddEventListener(NS_LITERAL_STRING("pagehide"), listener, PR_FALSE);
 
   return NS_OK;
 }
@@ -992,13 +866,17 @@ nsPasswordManager::Notify(nsIContent* aFormNode,
   // This function must never return a failure code or the form submit
   // will be cancelled.
 
+  NS_ENSURE_TRUE(aWindow, NS_OK);
+
   // Don't do anything if the global signon pref is disabled
   if (!SingleSignonEnabled())
     return NS_OK;
 
   // Check the reject list
   nsCAutoString realm;
-  if (!GetPasswordRealm(aFormNode->GetDocument()->GetDocumentURI(), realm))
+  // XXX bug 281125: GetDocument() could sometimes be null here, hinting
+  // XXX at a problem with document teardown while a modal dialog is posted.
+  if (!GetPasswordRealm(aFormNode->GetOwnerDoc()->GetDocumentURI(), realm))
     return NS_OK;
 
   PRInt32 rejectValue;
@@ -1059,13 +937,13 @@ nsPasswordManager::Notify(nsIContent* aFormNode,
         userFieldElement->GetAttribute(NS_LITERAL_STRING("autocomplete"),
                                        autocomplete);
 
-        if (autocomplete.LowerCaseEqualsLiteral("off"))
+        if (autocomplete.EqualsIgnoreCase("off"))
           return NS_OK;
       }
 
       nsCOMPtr<nsIDOMElement> formDOMEl = do_QueryInterface(aFormNode);
       formDOMEl->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
-      if (autocomplete.LowerCaseEqualsLiteral("off"))
+      if (autocomplete.EqualsIgnoreCase("off"))
         return NS_OK;
 
       nsCOMPtr<nsIDOMElement> passFieldElement = do_QueryInterface(passFields.ObjectAt(0));
@@ -1088,10 +966,8 @@ nsPasswordManager::Notify(nsIContent* aFormNode,
       passFields.ObjectAt(0)->GetValue(passValue);
       passFields.ObjectAt(0)->GetName(passFieldName);
 
-      // If username and password are both empty, there is no reason
-      // to store this login.
-
-      if (userValue.IsEmpty() && passValue.IsEmpty())
+      // If the password is empty, there is no reason to store this login.
+      if (passValue.IsEmpty())
         return NS_OK;
 
       SignonHashEntry* hashEnt;
@@ -1126,21 +1002,47 @@ nsPasswordManager::Notify(nsIContent* aFormNode,
         }
       }
 
-      nsAutoString dialogTitle, dialogText, neverText;
-      GetLocalizedString(NS_LITERAL_STRING("savePasswordTitle"), dialogTitle);
+      nsresult rv;
+      nsCOMPtr<nsIStringBundleService> bundleService =
+        do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
+      nsCOMPtr<nsIStringBundle> brandBundle;
+      rv = bundleService->CreateBundle("chrome://branding/locale/brand.properties",
+                                       getter_AddRefs(brandBundle));
+      NS_ENSURE_SUCCESS(rv, rv);
+      nsXPIDLString brandShortName;
+      rv = brandBundle->GetStringFromName(NS_LITERAL_STRING("brandShortName").get(),
+                                          getter_Copies(brandShortName));
+      NS_ENSURE_SUCCESS(rv, rv);
+      const PRUnichar* formatArgs[1] = { brandShortName.get() };
+
+      nsAutoString dialogText;
       GetLocalizedString(NS_LITERAL_STRING("savePasswordText"),
                          dialogText,
-                         PR_TRUE);
-      GetLocalizedString(NS_LITERAL_STRING("neverForSite"), neverText);
+                         PR_TRUE,
+                         formatArgs,
+                         1);
+
+      nsAutoString dialogTitle, neverButtonText, rememberButtonText,
+                   notNowButtonText;
+      GetLocalizedString(NS_LITERAL_STRING("savePasswordTitle"), dialogTitle);
+
+      GetLocalizedString(NS_LITERAL_STRING("neverForSiteButtonText"), 
+                         neverButtonText);
+      GetLocalizedString(NS_LITERAL_STRING("rememberButtonText"), 
+                         rememberButtonText);
+      GetLocalizedString(NS_LITERAL_STRING("notNowButtonText"),
+                         notNowButtonText);
 
       PRInt32 selection;
       prompt->ConfirmEx(dialogTitle.get(),
                         dialogText.get(),
-                        (nsIPrompt::BUTTON_TITLE_YES * nsIPrompt::BUTTON_POS_0) +
-                        (nsIPrompt::BUTTON_TITLE_NO * nsIPrompt::BUTTON_POS_1) +
+                        nsIPrompt::BUTTON_POS_1_DEFAULT +
+                        (nsIPrompt::BUTTON_TITLE_IS_STRING * nsIPrompt::BUTTON_POS_0) +
+                        (nsIPrompt::BUTTON_TITLE_IS_STRING * nsIPrompt::BUTTON_POS_1) +
                         (nsIPrompt::BUTTON_TITLE_IS_STRING * nsIPrompt::BUTTON_POS_2),
-                        nsnull, nsnull,
-                        neverText.get(),
+                        rememberButtonText.get(),
+                        notNowButtonText.get(),
+                        neverButtonText.get(),
                         nsnull, nsnull,
                         &selection);
 
@@ -1205,8 +1107,11 @@ nsPasswordManager::Notify(nsIContent* aFormNode,
               temp = entry;
 
               for (PRUint32 arg = 0; arg < entryCount; ++arg) {
-                if (NS_FAILED(DecryptData(temp->userValue, ptUsernames[arg])))
+                if (NS_FAILED(DecryptData(temp->userValue, ptUsernames[arg]))) {
+                  delete [] formatArgs;
+                  delete [] ptUsernames;
                   return NS_OK;
+                }
 
                 formatArgs[arg] = ptUsernames[arg].get();
                 temp = temp->next;
@@ -1304,7 +1209,54 @@ nsPasswordManager::Blur(nsIDOMEvent* aEvent)
 NS_IMETHODIMP
 nsPasswordManager::HandleEvent(nsIDOMEvent* aEvent)
 {
-  return FillPassword(aEvent);
+  nsAutoString type;
+  aEvent->GetType(type);
+
+  if (type.EqualsLiteral("DOMAutoComplete"))
+    return FillPassword(aEvent);
+
+  nsCOMPtr<nsIDOMEventTarget> target;
+  aEvent->GetTarget(getter_AddRefs(target));
+
+  nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(target);
+  if (!domDoc)
+    return NS_OK;
+
+  if (type.EqualsLiteral("pagehide"))
+    mAutoCompleteInputs.Enumerate(RemoveForDOMDocumentEnumerator, domDoc);
+  else if (type.EqualsLiteral("DOMContentLoaded"))
+    return FillDocument(domDoc);
+
+  return NS_OK;
+}
+
+
+// nsIPromptFactory implementation
+
+NS_IMETHODIMP
+nsPasswordManager::GetPrompt(nsIDOMWindow* aParent, const nsIID& aIID,
+                             void** _retval)
+{
+  if (!aIID.Equals(NS_GET_IID(nsIAuthPrompt2))) {
+    NS_WARNING("asked for unknown IID");
+    return NS_NOINTERFACE;
+  }
+
+  // NOTE: It is important to return the specific return value here. The
+  // caller cares.
+  nsresult rv;
+  nsCOMPtr<nsIPromptService2> service =
+    do_GetService(NS_PROMPTSERVICE_CONTRACTID, &rv);
+  if (NS_FAILED(rv))
+    return rv;
+
+  nsSingleSignonPrompt2* wrapper = new nsSingleSignonPrompt2(service, aParent);
+  if (!wrapper)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(wrapper);
+  *_retval = NS_STATIC_CAST(nsIAuthPrompt2*, wrapper);
+  return NS_OK;
 }
 
 // Autocomplete implementation
@@ -1444,7 +1396,7 @@ nsPasswordManager::AutoCompleteSearch(const nsAString& aSearchString,
       for (PRInt32 i = result->mArray.Count() - 1; i >= 0; --i) {
         nsDependentString match(NS_STATIC_CAST(PRUnichar*, result->mArray.ElementAt(i)));
         if (aSearchString.Length() > match.Length() ||
-            !StringBeginsWith(match, aSearchString)) {
+            !StringBeginsWith(match, aSearchString, nsCaseInsensitiveStringComparator())) {
           nsMemory::Free(result->mArray.ElementAt(i));
           result->mArray.RemoveElementAt(i);
         }
@@ -1483,8 +1435,10 @@ nsPasswordManager::AutoCompleteSearch(const nsAString& aSearchString,
           return NS_ERROR_FAILURE;
 
         if (aSearchString.Length() <= userValue.Length() &&
-            StringBeginsWith(userValue, aSearchString)) {
-          result->mArray.AppendElement(ToNewUnicode(userValue));
+            StringBeginsWith(userValue, aSearchString, nsCaseInsensitiveStringComparator())) {
+          PRUnichar* data = ToNewUnicode(userValue);
+          if (data)
+            result->mArray.AppendElement(data);
         }
       }
 
@@ -1507,20 +1461,6 @@ nsPasswordManager::AutoCompleteSearch(const nsAString& aSearchString,
   return PR_TRUE;
 }
 
-// nsIDOMLoadListener implementation
-
-NS_IMETHODIMP
-nsPasswordManager::Load(nsIDOMEvent* aEvent)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPasswordManager::BeforeUnload(nsIDOMEvent* aEvent)
-{
-  return NS_OK;
-}
-
  /* static */ PLDHashOperator PR_CALLBACK
 nsPasswordManager::RemoveForDOMDocumentEnumerator(nsISupports* aKey,
                                                   PRInt32& aEntry,
@@ -1536,32 +1476,6 @@ nsPasswordManager::RemoveForDOMDocumentEnumerator(nsISupports* aKey,
   return PL_DHASH_NEXT;
 }
 
-NS_IMETHODIMP
-nsPasswordManager::Unload(nsIDOMEvent* aEvent)
-{
-  nsCOMPtr<nsIDOMEventTarget> target;
-  aEvent->GetTarget(getter_AddRefs(target));
-
-  nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(target);
-  if (domDoc)
-    mAutoCompleteInputs.Enumerate(RemoveForDOMDocumentEnumerator, domDoc);
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPasswordManager::Abort(nsIDOMEvent* aEvent)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPasswordManager::Error(nsIDOMEvent* aEvent)
-{
-  return NS_OK;
-}
-
-
 // internal methods
 
 /*
@@ -1616,20 +1530,20 @@ nsPasswordManager::WriteSignonEntryEnumerator(const nsACString& aKey,
   stream->Write(buffer.get(), buffer.Length(), &bytesWritten);
 
   for (SignonDataEntry* e = aEntry->head; e; e = e->next) {
-    NS_ConvertUCS2toUTF8 userField(e->userField);
+    NS_ConvertUTF16toUTF8 userField(e->userField);
     userField.Append(NS_LINEBREAK);
     stream->Write(userField.get(), userField.Length(), &bytesWritten);
 
-    buffer.Assign(NS_ConvertUCS2toUTF8(e->userValue));
+    buffer.Assign(NS_ConvertUTF16toUTF8(e->userValue));
     buffer.Append(NS_LINEBREAK);
     stream->Write(buffer.get(), buffer.Length(), &bytesWritten);
 
     buffer.Assign("*");
-    buffer.Append(NS_ConvertUCS2toUTF8(e->passField));
+    buffer.Append(NS_ConvertUTF16toUTF8(e->passField));
     buffer.Append(NS_LINEBREAK);
     stream->Write(buffer.get(), buffer.Length(), &bytesWritten);
 
-    buffer.Assign(NS_ConvertUCS2toUTF8(e->passValue));
+    buffer.Assign(NS_ConvertUTF16toUTF8(e->passValue));
     buffer.Append(NS_LINEBREAK);
     stream->Write(buffer.get(), buffer.Length(), &bytesWritten);
   }
@@ -1640,6 +1554,30 @@ nsPasswordManager::WriteSignonEntryEnumerator(const nsACString& aKey,
   return PL_DHASH_NEXT;
 }
 
+// Wrapper function for ReadPasswords
+void
+nsPasswordManager::LoadPasswords()
+{
+  if (sPasswordsLoaded)
+    return;
+
+  nsXPIDLCString signonFile;
+  mPrefBranch->GetCharPref("SignonFileName", getter_Copies(signonFile));
+  NS_ASSERTION(!signonFile.IsEmpty(), "Fallback for signon filename not present");
+
+  NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(mSignonFile));
+  if (!mSignonFile)
+    return;
+
+  mSignonFile->AppendNative(signonFile);
+
+  nsCAutoString path;
+  mSignonFile->GetNativePath(path);
+
+  if (NS_SUCCEEDED(ReadPasswords(mSignonFile)))
+    sPasswordsLoaded = PR_TRUE;
+}
+
 void
 nsPasswordManager::WritePasswords(nsIFile* aPasswordFile)
 {
@@ -1685,7 +1623,7 @@ nsPasswordManager::AddSignonData(const nsACString& aRealm,
 nsPasswordManager::DecryptData(const nsAString& aData,
                                nsAString& aPlaintext)
 {
-  NS_ConvertUCS2toUTF8 flatData(aData);
+  NS_ConvertUTF16toUTF8 flatData(aData);
   char* buffer = nsnull;
 
   if (flatData.CharAt(0) == '~') {
@@ -1710,7 +1648,7 @@ nsPasswordManager::DecryptData(const nsAString& aData,
 
   }
 
-  aPlaintext.Assign(NS_ConvertUTF8toUCS2(buffer));
+  aPlaintext.Assign(NS_ConvertUTF8toUTF16(buffer));
   PR_Free(buffer);
 
   return NS_OK;
@@ -1729,7 +1667,7 @@ nsPasswordManager::EncryptData(const nsAString& aPlaintext,
   NS_ENSURE_TRUE(sDecoderRing, NS_ERROR_FAILURE);
 
   char* buffer;
-  if (NS_FAILED(sDecoderRing->EncryptString(NS_ConvertUCS2toUTF8(aPlaintext).get(), &buffer)))
+  if (NS_FAILED(sDecoderRing->EncryptString(NS_ConvertUTF16toUTF8(aPlaintext).get(), &buffer)))
     return NS_ERROR_FAILURE;
 
   aEncrypted.Assign(buffer);
@@ -1746,7 +1684,7 @@ nsPasswordManager::EncryptDataUCS2(const nsAString& aPlaintext,
   nsresult rv = EncryptData(aPlaintext, buffer);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  aEncrypted.Assign(NS_ConvertUTF8toUCS2(buffer));
+  aEncrypted.Assign(NS_ConvertUTF8toUTF16(buffer));
   return NS_OK;
 }
 
@@ -1833,6 +1771,216 @@ nsPasswordManager::FindPasswordEntryInternal(const SignonDataEntry* aEntry,
   return NS_ERROR_FAILURE;
 }
 
+nsresult
+nsPasswordManager::FillDocument(nsIDOMDocument* aDomDoc)
+{
+  nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(aDomDoc);
+  nsCOMPtr<nsIDOMHTMLCollection> forms;
+  htmlDoc->GetForms(getter_AddRefs(forms));
+  
+  nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDomDoc);
+
+  nsCAutoString realm;
+  if (!GetPasswordRealm(doc->GetDocumentURI(), realm))
+    return NS_OK;
+
+  SignonHashEntry* hashEnt;
+  if (!mSignonTable.Get(realm, &hashEnt))
+    return NS_OK;
+
+  PRUint32 formCount;
+  forms->GetLength(&formCount);
+
+  // We can auto-prefill the username and password if there is only
+  // one stored login that matches the username and password field names
+  // on the form in question.  Note that we only need to worry about a
+  // single login per form.
+
+  for (PRUint32 i = 0; i < formCount; ++i) {
+    nsCOMPtr<nsIDOMNode> formNode;
+    forms->Item(i, getter_AddRefs(formNode));
+
+    nsCOMPtr<nsIForm> form = do_QueryInterface(formNode);
+    SignonDataEntry* firstMatch = nsnull;
+    PRBool attachedToInput = PR_FALSE;
+    PRBool prefilledUser = PR_FALSE;
+    nsCOMPtr<nsIDOMHTMLInputElement> userField, passField;
+    nsCOMPtr<nsIDOMHTMLInputElement> temp;
+    nsAutoString fieldType;
+
+    for (SignonDataEntry* e = hashEnt->head; e; e = e->next) {
+      
+      nsCOMPtr<nsISupports> foundNode;
+      if (!(e->userField).IsEmpty()) {
+        form->ResolveName(e->userField, getter_AddRefs(foundNode));
+        temp = do_QueryInterface(foundNode);
+      }
+
+      nsAutoString oldUserValue;
+      PRBool userFieldFound = PR_FALSE;
+
+      if (temp) {
+        temp->GetType(fieldType);
+        if (!fieldType.Equals(NS_LITERAL_STRING("text")))
+          continue;
+
+        temp->GetValue(oldUserValue);
+        userField = temp;
+        userFieldFound = PR_TRUE;
+      } else if ((e->passField).IsEmpty()) {
+        // Happens sometimes when we import passwords from IE since
+        // their form name match is case insensitive. In this case,
+        // we'll just have to do a case insensitive search for the
+        // userField and hope we get something.
+        PRUint32 count;
+        form->GetElementCount(&count);
+        PRUint32 i;
+        nsCOMPtr<nsIFormControl> formControl;
+        for (i = 0; i < count; i++) {
+          form->GetElementAt(i, getter_AddRefs(formControl));
+
+          if (formControl &&
+              formControl->GetType() == NS_FORM_INPUT_TEXT) {
+            nsCOMPtr<nsIDOMHTMLInputElement> inputField = do_QueryInterface(formControl);
+            nsAutoString name;
+            inputField->GetName(name);
+            if (name.EqualsIgnoreCase(NS_ConvertUTF16toUTF8(e->userField).get())) {
+              inputField->GetValue(oldUserValue);
+              userField = inputField;
+              foundNode = inputField;
+              e->userField.Assign(name);
+              userFieldFound = PR_TRUE;
+              break;
+            }
+          }
+        }
+      }
+
+      // Bail out if we should be seeing a userField but we're not
+      if (!userFieldFound && !(e->userField).IsEmpty())
+        continue;
+
+      if (!(e->passField).IsEmpty()) {
+        form->ResolveName(e->passField, getter_AddRefs(foundNode));
+        temp = do_QueryInterface(foundNode);
+      }
+      else if (userField) {
+        // No password field name was supplied, try to locate one in the form,
+        // but only if we have a username field.
+        nsCOMPtr<nsIFormControl> fc(do_QueryInterface(foundNode));
+        PRInt32 index = -1;
+        form->IndexOfControl(fc, &index);
+        if (index >= 0) {
+          PRUint32 count;
+          form->GetElementCount(&count);
+
+          PRUint32 i;
+          temp = nsnull;
+
+          // Search forwards
+          nsCOMPtr<nsIFormControl> passField;
+          for (i = index + 1; i < count; ++i) {
+            form->GetElementAt(i, getter_AddRefs(passField));
+
+            if (passField && passField->GetType() == NS_FORM_INPUT_PASSWORD) {
+              foundNode = passField;
+              temp = do_QueryInterface(foundNode);
+            }
+          }
+
+          if (!temp && index != 0) {
+            // Search backwards
+            i = index;
+            do {
+              form->GetElementAt(i, getter_AddRefs(passField));
+
+              if (passField && passField->GetType() == NS_FORM_INPUT_PASSWORD) {
+                foundNode = passField;
+                temp = do_QueryInterface(foundNode);
+              }
+            } while (i-- != 0);
+          }
+        }
+      }
+
+      nsAutoString oldPassValue;
+
+      if (temp) {
+        temp->GetType(fieldType);
+        if (!fieldType.Equals(NS_LITERAL_STRING("password")))
+          continue;
+
+        temp->GetValue(oldPassValue);
+        passField = temp;
+        if ((e->passField).IsEmpty())
+          passField->GetName(e->passField);
+      } else {
+        continue;
+      }
+
+      if (!oldUserValue.IsEmpty()) {
+        // The page has prefilled a username.
+        // If it matches any of our saved usernames, prefill the password
+        // for that username.  If there are multiple saved usernames,
+        // we will also attach the autocomplete listener.
+
+        prefilledUser = PR_TRUE;
+        nsAutoString userValue;
+        if (NS_FAILED(DecryptData(e->userValue, userValue)))
+          return NS_OK;
+
+        if (userValue.Equals(oldUserValue)) {
+          nsAutoString passValue;
+          if (NS_FAILED(DecryptData(e->passValue, passValue)))
+            return NS_OK;
+
+          passField->SetValue(passValue);
+        }
+      }
+
+      if (firstMatch && userField && !attachedToInput) {
+        // We've found more than one possible signon for this form.
+
+        // Listen for blur and autocomplete events on the username field so
+        // that we can attempt to prefill the password after the user has
+        // entered the username.
+
+        AttachToInput(userField);
+        attachedToInput = PR_TRUE;
+      } else {
+        firstMatch = e;
+      }
+    }
+
+    // If we found more than one match, attachedToInput will be true,
+    // but if we found just one, we need to attach the autocomplete listener,
+    // and fill in the username and password  only if the HTML didn't prefill
+    // the username.
+    if (firstMatch && !attachedToInput) {
+      if (userField)
+        AttachToInput(userField);
+
+      if (!prefilledUser){
+        nsAutoString buffer;
+
+        if (userField) {
+          if (NS_FAILED(DecryptData(firstMatch->userValue, buffer)))
+            return NS_OK;
+
+          userField->SetValue(buffer);
+        }
+
+        if (NS_FAILED(DecryptData(firstMatch->passValue, buffer)))
+          return NS_OK;
+
+        passField->SetValue(buffer);
+      }
+    }
+  }
+
+  return NS_OK;
+}
+
 nsresult
 nsPasswordManager::FillPassword(nsIDOMEvent* aEvent)
 {
@@ -1919,7 +2067,7 @@ nsPasswordManager::GetPasswordRealm(nsIURI* aURI, nsACString& aRealm)
   aURI->GetScheme(buffer);
 
   aRealm.Append(buffer);
-  aRealm.AppendLiteral("://");
+  aRealm.Append(NS_LITERAL_CSTRING("://"));
 
   aURI->GetHostPort(buffer);
   if (buffer.IsEmpty()) {
