@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,7 +14,7 @@
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -24,29 +24,33 @@
  *   Dean Tessman <dean_tessman@hotmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsFormFillController.h"
 
+#ifdef MOZ_STORAGE_SATCHEL
+#include "nsStorageFormHistory.h"
+#include "nsIAutoCompleteSimpleResult.h"
+#else
 #include "nsFormHistory.h"
 #include "nsIAutoCompleteResultTypes.h"
+#endif
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsIServiceManager.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
-#include "nsIDocument.h"
 #include "nsIDocShellTreeItem.h"
 #include "nsIChromeEventHandler.h"
 #include "nsPIDOMWindow.h"
@@ -54,21 +58,24 @@
 #include "nsIContentViewer.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMKeyEvent.h"
+#include "nsIPrivateDOMEvent.h"
+#include "nsIDOMCompositionListener.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMNSHTMLInputElement.h"
-#include "nsIScriptGlobalObject.h"
+#include "nsIDocument.h"
 #include "nsIContent.h"
 #include "nsIPresShell.h"
-#include "nsPresContext.h"
-#include "nsIView.h"
 #include "nsIFrame.h"
-#include "nsIWidget.h"
 #include "nsRect.h"
 #include "nsIDOMDocumentEvent.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsPasswordManager.h"
+#include "nsSingleSignonPrompt.h"
 #include "nsIDOMMouseEvent.h"
+#include "nsIGenericFactory.h"
+#include "nsToolkitCompsCID.h"
+#include "nsEmbedCID.h"
 
 NS_INTERFACE_MAP_BEGIN(nsFormFillController)
   NS_INTERFACE_MAP_ENTRY(nsIFormFillController)
@@ -79,6 +86,8 @@ NS_INTERFACE_MAP_BEGIN(nsFormFillController)
   NS_INTERFACE_MAP_ENTRY(nsIDOMFormListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMMouseListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMLoadListener)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMCompositionListener)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMContextMenuListener)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIFormFillController)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMEventListener, nsIDOMFocusListener)
 NS_INTERFACE_MAP_END
@@ -92,10 +101,11 @@ nsFormFillController::nsFormFillController() :
   mMaxRows(0),
   mDisableAutoComplete(PR_FALSE), 
   mCompleteDefaultIndex(PR_FALSE),
+  mCompleteSelectedIndex(PR_FALSE),
   mForceComplete(PR_FALSE),
   mSuppressOnInput(PR_FALSE)
 {
-  mController = do_CreateInstance("@mozilla.org/autocomplete/controller;1");
+  mController = do_GetService("@mozilla.org/autocomplete/controller;1");
 
   mDocShells = do_CreateInstance("@mozilla.org/supports-array;1");
   mPopups = do_CreateInstance("@mozilla.org/supports-array;1");
@@ -116,53 +126,27 @@ nsFormFillController::~nsFormFillController()
 
 ////////////////////////////////////////////////////////////////////////
 
-nsRect&
+nsRect
 GetScreenOrigin(nsIDOMElement* aElement)
 {
-  nsRect* rect = new nsRect(0,0,0,0);
-  nsSize size;
- 
+  nsRect rect(0,0,0,0);
+
   nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
   nsCOMPtr<nsIDocument> doc = content->GetDocument();
 
   if (doc) {
     // Get Presentation shell 0
     nsIPresShell* presShell = doc->GetShellAt(0);
-    
+
     if (presShell) {
-      nsPresContext *presContext = presShell->GetPresContext();
-
-      if (presContext) {
-        // Get the scale from that Presentation Context
-        float scale;
-        scale = presContext->TwipsToPixels();
-
-        nsIFrame* frame;
-        nsresult rv = presShell->GetPrimaryFrameFor(content, &frame);
-
-        nsIView* view;
-        nsPoint offset;
-        frame->GetOffsetFromView(presContext, offset, &view);
-        if (view) {
-          nsPoint widgetOffset(0, 0);
-          nsIWidget* widget = view->GetNearestWidget(&widgetOffset);
-          if (widget) {
-            nsRect oldBox(0,0,0,0);
-            widget->WidgetToScreen(oldBox, *rect);
-          }
-          
-          rect->x += NSTwipsToIntPixels(offset.x+widgetOffset.x, scale);
-          rect->y += NSTwipsToIntPixels(offset.y+widgetOffset.y, scale);
-        }
-        
-        size = frame->GetSize();
-        rect->width = NSTwipsToIntPixels(size.width, scale);
-        rect->height = NSTwipsToIntPixels(size.height, scale);
-      }
+      nsIFrame* frame = presShell->GetPrimaryFrameFor(content);
+      if (!frame)
+        return rect;
+      rect = frame->GetScreenRectExternal();
     }
   }
   
-  return *rect;
+  return rect;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -233,12 +217,26 @@ nsFormFillController::SetPopupOpen(PRBool aPopupOpen)
 {
   if (mFocusedPopup) {
     if (aPopupOpen) {
+      // make sure input field is visible before showing popup (bug 320938)
+      nsCOMPtr<nsIContent> content = do_QueryInterface(mFocusedInput);
+      NS_ENSURE_STATE(content);
+      nsCOMPtr<nsIDocShell> docShell = GetDocShellForInput(mFocusedInput);
+      NS_ENSURE_STATE(docShell);
+      nsCOMPtr<nsIPresShell> presShell;
+      docShell->GetPresShell(getter_AddRefs(presShell));
+      NS_ENSURE_STATE(presShell);
+      nsIFrame *frame = presShell->GetPrimaryFrameFor(content.get());
+      if (frame)
+        presShell->ScrollFrameIntoView(frame,
+                                       NS_PRESSHELL_SCROLL_IF_NOT_VISIBLE,
+                                       NS_PRESSHELL_SCROLL_IF_NOT_VISIBLE);
+
       nsRect popupRect = GetScreenOrigin(mFocusedInput);
       mFocusedPopup->OpenPopup(this, popupRect.x, popupRect.y+popupRect.height, popupRect.width);
     } else
       mFocusedPopup->ClosePopup();
   }
-    
+
   return NS_OK;
 }
 
@@ -270,6 +268,20 @@ nsFormFillController::SetCompleteDefaultIndex(PRBool aCompleteDefaultIndex)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsFormFillController::GetCompleteSelectedIndex(PRBool *aCompleteSelectedIndex)
+{
+  *aCompleteSelectedIndex = mCompleteSelectedIndex;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::SetCompleteSelectedIndex(PRBool aCompleteSelectedIndex)
+{
+  mCompleteSelectedIndex = aCompleteSelectedIndex;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsFormFillController::GetForceComplete(PRBool *aForceComplete)
 {
@@ -373,16 +385,22 @@ nsFormFillController::GetSearchAt(PRUint32 index, nsACString & _retval)
 NS_IMETHODIMP
 nsFormFillController::GetTextValue(nsAString & aTextValue)
 {
-  mFocusedInput->GetValue(aTextValue);
+  if (mFocusedInput) {
+    mFocusedInput->GetValue(aTextValue);
+  } else {
+    aTextValue.Truncate();
+  }
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormFillController::SetTextValue(const nsAString & aTextValue)
 {
-  mSuppressOnInput = PR_TRUE;
-  mFocusedInput->SetValue(aTextValue);
-  mSuppressOnInput = PR_FALSE;
+  if (mFocusedInput) {
+    mSuppressOnInput = PR_TRUE;
+    mFocusedInput->SetValue(aTextValue);
+    mSuppressOnInput = PR_FALSE;
+  }
   return NS_OK;
 }
 
@@ -436,10 +454,19 @@ nsFormFillController::OnTextEntered(PRBool *_retval)
   }
 
   event->InitEvent(NS_LITERAL_STRING("DOMAutoComplete"), PR_TRUE, PR_TRUE);
+
+  nsCOMPtr<nsIPrivateDOMEvent> privateEvent(do_QueryInterface(event));
+  if (privateEvent) {
+    // XXXjst: We mark this event as a trusted event, it's up to the
+    // callers of this to ensure that it's only called from trusted
+    // code.
+    privateEvent->SetTrusted(PR_TRUE);
+  }
+
   nsCOMPtr<nsIDOMEventTarget> targ = do_QueryInterface(mFocusedInput);
 
-  PRBool noDefault;
-  targ->DispatchEvent(event, &noDefault);
+  PRBool defaultActionEnabled;
+  targ->DispatchEvent(event, &defaultActionEnabled);
 
   return NS_OK;
 }
@@ -450,6 +477,13 @@ nsFormFillController::OnTextReverted(PRBool *_retval)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsFormFillController::GetConsumeRollupEvent(PRBool *aConsumeRollupEvent)
+{
+  *aConsumeRollupEvent = PR_FALSE;
+  return NS_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////
 //// nsIAutoCompleteSearch
 
@@ -459,25 +493,35 @@ nsFormFillController::StartSearch(const nsAString &aSearchString, const nsAStrin
                                   nsIAutoCompleteResult *aPreviousResult, nsIAutoCompleteObserver *aListener)
 {
   nsCOMPtr<nsIAutoCompleteResult> result;
-  nsCOMPtr<nsIAutoCompleteMdbResult> mdbResult = do_QueryInterface(aPreviousResult);
+
+#ifdef MOZ_STORAGE_SATCHEL
+  // This assumes that FormHistory uses nsIAutoCompleteSimpleResult,
+  // while PasswordManager does not.
+  nsCOMPtr<nsIAutoCompleteSimpleResult> historyResult;
+#else
+  nsCOMPtr<nsIAutoCompleteMdbResult2> historyResult;
+#endif
+  historyResult = do_QueryInterface(aPreviousResult);
 
   nsPasswordManager* passMgr = nsPasswordManager::GetInstance();
+  if (!passMgr)
+    return NS_ERROR_OUT_OF_MEMORY;
 
   // Only hand off a previous result to the password manager if it's
-  // a password manager result (i.e. not an nsIAutoCompleteMdbResult).
+  // a password manager result (i.e. not an nsIAutoCompleteMdb/SimpleResult).
 
   if (!passMgr->AutoCompleteSearch(aSearchString,
-                                   mdbResult ? nsnull : aPreviousResult,
+                                   historyResult ? nsnull : aPreviousResult,
                                    mFocusedInput,
                                    getter_AddRefs(result)))
   {
     nsFormHistory *history = nsFormHistory::GetInstance();
-    history->AutoCompleteSearch(aSearchParam,
-                                aSearchString,
-                                mdbResult,
-                                getter_AddRefs(result));
-
-    NS_RELEASE(history);
+    if (history) {
+      history->AutoCompleteSearch(aSearchParam,
+                                  aSearchString,
+                                  historyResult,
+                                  getter_AddRefs(result));
+    }
   }
   NS_RELEASE(passMgr);
 
@@ -519,18 +563,18 @@ nsFormFillController::Focus(nsIDOMEvent* aEvent)
 
     PRBool isReadOnly = PR_FALSE;
     input->GetReadOnly(&isReadOnly);
-
+                                  
     nsAutoString autocomplete; 
     input->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
-    if (type.EqualsLiteral("text") && !isReadOnly &&
-        !autocomplete.LowerCaseEqualsLiteral("off")) {
+    if (type.Equals(NS_LITERAL_STRING("text")) && !isReadOnly &&
+        !autocomplete.EqualsIgnoreCase("off")) {
 
       nsCOMPtr<nsIDOMHTMLFormElement> form;
       input->GetForm(getter_AddRefs(form));
       if (form)
         form->GetAttribute(NS_LITERAL_STRING("autocomplete"), autocomplete);
 
-      if (!form || !autocomplete.LowerCaseEqualsLiteral("off"))
+      if (!form || !autocomplete.EqualsIgnoreCase("off"))
         StartControllingInput(input);
     }
     
@@ -578,21 +622,26 @@ nsFormFillController::KeyPress(nsIDOMEvent* aEvent)
   keyEvent->GetKeyCode(&k);
   switch (k) {
   case nsIDOMKeyEvent::DOM_VK_DELETE:
+#ifndef XP_MACOSX
+    mController->HandleDelete(&cancel);
+    break;
+  case nsIDOMKeyEvent::DOM_VK_BACK_SPACE:
+    mController->HandleText(PR_FALSE);
+    break;
+#else
+  case nsIDOMKeyEvent::DOM_VK_BACK_SPACE:
     {
       PRBool isShift = PR_FALSE;
       keyEvent->GetShiftKey(&isShift);
 
-      if (isShift) {
+      if (isShift)
         mController->HandleDelete(&cancel);
+      else
+        mController->HandleText(PR_FALSE);
 
-        break;
-      }
-
-      // fall through
+      break;
     }
-  case nsIDOMKeyEvent::DOM_VK_BACK_SPACE:
-    mController->HandleText(PR_FALSE);
-    break;
+#endif
   case nsIDOMKeyEvent::DOM_VK_UP:
     mController->HandleKeyNavigation(nsIAutoCompleteController::KEY_UP, &cancel);
     break;
@@ -631,6 +680,49 @@ nsFormFillController::KeyPress(nsIDOMEvent* aEvent)
   return NS_OK;
 }
 
+////////////////////////////////////////////////////////////////////////
+//// nsIDOMCompositionListener
+
+NS_IMETHODIMP
+nsFormFillController::HandleStartComposition(nsIDOMEvent* aCompositionEvent)
+{
+  NS_ASSERTION(mController, "should have a controller!");
+
+  if (mController && mFocusedInput)
+    mController->HandleStartComposition();
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::HandleEndComposition(nsIDOMEvent* aCompositionEvent)
+{
+  NS_ASSERTION(mController, "should have a controller!");
+
+  if (mController && mFocusedInput)
+    mController->HandleEndComposition();
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::HandleQueryComposition(nsIDOMEvent* aCompositionEvent)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::HandleQueryReconversion(nsIDOMEvent* aCompositionEvent)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::HandleQueryCaretRect(nsIDOMEvent* aCompostionEvent)
+{
+  return NS_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////
 //// nsIDOMFormListener
 
@@ -732,7 +824,7 @@ nsFormFillController::MouseClick(nsIDOMEvent* aMouseEvent)
   input->GetTextValue(value);
   if (value.Length() > 0) {
     // Show the popup with a filtered result set
-    mController->SetSearchString(NS_LITERAL_STRING(""));
+    mController->SetSearchString(EmptyString());
     mController->HandleText(PR_TRUE);
   } else {
     // Show the popup with the complete result set.  Can't use HandleText()
@@ -807,6 +899,14 @@ nsFormFillController::Error(nsIDOMEvent *aLoadEvent)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsFormFillController::ContextMenu(nsIDOMEvent* aContextMenuEvent)
+{
+  if (mFocusedPopup)
+    mFocusedPopup->ClosePopup();
+  return NS_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////
 //// nsFormFillController
 
@@ -817,7 +917,7 @@ nsFormFillController::AddWindowListeners(nsIDOMWindow *aWindow)
     return;
 
   nsCOMPtr<nsPIDOMWindow> privateDOMWindow(do_QueryInterface(aWindow));
-  nsIChromeEventHandler *chromeEventHandler = nsnull;
+  nsIChromeEventHandler* chromeEventHandler = nsnull;
   if (privateDOMWindow)
     chromeEventHandler = privateDOMWindow->GetChromeEventHandler();
 
@@ -826,9 +926,9 @@ nsFormFillController::AddWindowListeners(nsIDOMWindow *aWindow)
   if (!target)
     return;
 
-    target->AddEventListener(NS_LITERAL_STRING("focus"),
-                             NS_STATIC_CAST(nsIDOMFocusListener *, this),
-                             PR_TRUE);
+  target->AddEventListener(NS_LITERAL_STRING("focus"),
+                           NS_STATIC_CAST(nsIDOMFocusListener *, this),
+                           PR_TRUE);
 
   target->AddEventListener(NS_LITERAL_STRING("blur"),
                            NS_STATIC_CAST(nsIDOMFocusListener *, this),
@@ -849,6 +949,22 @@ nsFormFillController::AddWindowListeners(nsIDOMWindow *aWindow)
   target->AddEventListener(NS_LITERAL_STRING("unload"),
                            NS_STATIC_CAST(nsIDOMLoadListener *, this),
                            PR_TRUE);
+
+  target->AddEventListener(NS_LITERAL_STRING("compositionstart"),
+                           NS_STATIC_CAST(nsIDOMCompositionListener *, this),
+                           PR_TRUE);
+
+  target->AddEventListener(NS_LITERAL_STRING("compositionend"),
+                           NS_STATIC_CAST(nsIDOMCompositionListener *, this),
+                           PR_TRUE);
+
+  target->AddEventListener(NS_LITERAL_STRING("contextmenu"),
+                           NS_STATIC_CAST(nsIDOMContextMenuListener *, this),
+                           PR_TRUE);
+
+  target->AddEventListener(NS_LITERAL_STRING("keypress"),
+                           NS_STATIC_CAST(nsIDOMKeyListener *, this),
+                           PR_TRUE);
 }
 
 void
@@ -860,18 +976,18 @@ nsFormFillController::RemoveWindowListeners(nsIDOMWindow *aWindow)
   StopControllingInput();
   
   nsCOMPtr<nsPIDOMWindow> privateDOMWindow(do_QueryInterface(aWindow));
-  nsIChromeEventHandler *chromeEventHandler = nsnull;
+  nsIChromeEventHandler* chromeEventHandler = nsnull;
   if (privateDOMWindow)
     chromeEventHandler = privateDOMWindow->GetChromeEventHandler();
-
+  
   nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(chromeEventHandler));
 
   if (!target)
     return;
 
   target->RemoveEventListener(NS_LITERAL_STRING("focus"),
-                            NS_STATIC_CAST(nsIDOMFocusListener *, this),
-                            PR_TRUE);
+                              NS_STATIC_CAST(nsIDOMFocusListener *, this),
+                              PR_TRUE);
 
   target->RemoveEventListener(NS_LITERAL_STRING("blur"),
                               NS_STATIC_CAST(nsIDOMFocusListener *, this),
@@ -892,32 +1008,22 @@ nsFormFillController::RemoveWindowListeners(nsIDOMWindow *aWindow)
   target->RemoveEventListener(NS_LITERAL_STRING("unload"),
                               NS_STATIC_CAST(nsIDOMLoadListener *, this),
                               PR_TRUE);
-}
 
-void
-nsFormFillController::AddKeyListener(nsIDOMHTMLInputElement *aInput)
-{
-  if (!aInput)
-    return;
-    
-    nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(aInput);
+  target->RemoveEventListener(NS_LITERAL_STRING("compositionstart"),
+                              NS_STATIC_CAST(nsIDOMCompositionListener *, this),
+                              PR_TRUE);
 
-    target->AddEventListener(NS_LITERAL_STRING("keypress"),
-                             NS_STATIC_CAST(nsIDOMKeyListener *, this),
-                             PR_TRUE);
-  }
+  target->RemoveEventListener(NS_LITERAL_STRING("compositionend"),
+                              NS_STATIC_CAST(nsIDOMCompositionListener *, this),
+                              PR_TRUE);
 
-void
-nsFormFillController::RemoveKeyListener()
-{
-  if (!mFocusedInput)
-    return;
+  target->RemoveEventListener(NS_LITERAL_STRING("contextmenu"),
+                              NS_STATIC_CAST(nsIDOMContextMenuListener *, this),
+                              PR_TRUE);
 
-    nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(mFocusedInput);
-
-    target->RemoveEventListener(NS_LITERAL_STRING("keypress"),
-                                NS_STATIC_CAST(nsIDOMKeyListener *, this),
-                                PR_TRUE);
+  target->RemoveEventListener(NS_LITERAL_STRING("keypress"),
+                              NS_STATIC_CAST(nsIDOMKeyListener *, this),
+                              PR_TRUE);
 }
 
 void
@@ -935,7 +1041,6 @@ nsFormFillController::StartControllingInput(nsIDOMHTMLInputElement *aInput)
   // Cache the popup for the focused docShell
   mPopups->GetElementAt(index, getter_AddRefs(mFocusedPopup));
   
-  AddKeyListener(aInput);
   mFocusedInput = aInput;
 
   // Now we are the autocomplete controller's bitch
@@ -945,8 +1050,6 @@ nsFormFillController::StartControllingInput(nsIDOMHTMLInputElement *aInput)
 void
 nsFormFillController::StopControllingInput()
 {
-  RemoveKeyListener();
-
   // Reset the controller's input, but not if it has been switched
   // to another input already, which might happen if the user switches
   // focus by clicking another autocomplete textbox
@@ -965,8 +1068,8 @@ nsFormFillController::GetDocShellForInput(nsIDOMHTMLInputElement *aInput)
   nsCOMPtr<nsIDOMDocument> domDoc;
   aInput->GetOwnerDocument(getter_AddRefs(domDoc));
   nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
-  
-  nsCOMPtr<nsIWebNavigation> webNav = do_GetInterface(doc->GetScriptGlobalObject());
+  NS_ENSURE_TRUE(doc, nsnull);
+  nsCOMPtr<nsIWebNavigation> webNav = do_GetInterface(doc->GetWindow());
   nsCOMPtr<nsIDocShell> docShell = do_QueryInterface(webNav);
   return docShell;
 }
@@ -983,13 +1086,15 @@ nsFormFillController::GetWindowForDocShell(nsIDocShell *aDocShell)
   nsCOMPtr<nsIDocument> doc = do_QueryInterface(domDoc);
   NS_ENSURE_TRUE(doc, nsnull);
 
-  nsCOMPtr<nsIDOMWindow> domWindow = do_QueryInterface(doc->GetScriptGlobalObject());
-  return domWindow;
+  return doc->GetWindow();
 }
 
 PRInt32
 nsFormFillController::GetIndexOfDocShell(nsIDocShell *aDocShell)
 {
+  if (!aDocShell)
+    return -1;
+
   // Loop through our cached docShells looking for the given docShell
   PRUint32 count;
   mDocShells->Count(&count);
@@ -1012,3 +1117,62 @@ nsFormFillController::GetIndexOfDocShell(nsIDocShell *aDocShell)
   return -1;
 }
 
+NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsFormHistory, Init)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsFormFillController)
+NS_GENERIC_FACTORY_SINGLETON_CONSTRUCTOR(nsPasswordManager, nsPasswordManager::GetInstance)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsSingleSignonPrompt)
+#if defined(MOZ_STORAGE_SATCHEL) && defined(MOZ_MORKREADER)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsFormHistoryImporter)
+#endif
+
+static void PR_CALLBACK nsFormHistoryModuleDtor(nsIModule* self)
+{
+  nsPasswordManager::Shutdown();
+}
+
+static const nsModuleComponentInfo components[] =
+{
+  { "Password Manager",
+    NS_PASSWORDMANAGER_CID,
+    NS_PASSWORDMANAGER_CONTRACTID,
+    nsPasswordManagerConstructor,
+    nsPasswordManager::Register,
+    nsPasswordManager::Unregister },
+
+  { "Password Manager",
+    NS_PASSWORDMANAGER_CID,
+    NS_PWMGR_AUTHPROMPTFACTORY,
+    nsPasswordManagerConstructor,
+    nsPasswordManager::Register,
+    nsPasswordManager::Unregister },
+
+  { "Single Signon Prompt",
+    NS_SINGLE_SIGNON_PROMPT_CID,
+    "@mozilla.org/wallet/single-sign-on-prompt;1",
+    nsSingleSignonPromptConstructor },
+
+  { "HTML Form History",
+    NS_FORMHISTORY_CID, 
+    NS_FORMHISTORY_CONTRACTID,
+    nsFormHistoryConstructor },
+
+  { "HTML Form Fill Controller",
+    NS_FORMFILLCONTROLLER_CID, 
+    "@mozilla.org/satchel/form-fill-controller;1",
+    nsFormFillControllerConstructor },
+
+  { "HTML Form History AutoComplete",
+    NS_FORMFILLCONTROLLER_CID, 
+    NS_FORMHISTORYAUTOCOMPLETE_CONTRACTID,
+    nsFormFillControllerConstructor },
+
+#if defined(MOZ_STORAGE_SATCHEL) && defined(MOZ_MORKREADER)
+  { "Form History Importer",
+    NS_FORMHISTORYIMPORTER_CID,
+    NS_FORMHISTORYIMPORTER_CONTRACTID,
+    nsFormHistoryImporterConstructor },
+#endif
+};
+
+NS_IMPL_NSGETMODULE_WITH_DTOR(satchel, components, nsFormHistoryModuleDtor)
+
