@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,7 +14,7 @@
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -23,16 +23,16 @@
  *   Joe Hewitt <hewitt@netscape.com> (Original Author)
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -57,17 +57,26 @@
 #include "nsIDOMHTMLCollection.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "nsVoidArray.h"
 #include "nsCOMArray.h"
 
+static void SwapBytes(PRUnichar* aDest, const PRUnichar* aSrc, PRUint32 aLen)
+{
+  for(PRUint32 i = 0; i < aLen; i++)
+  {
+    PRUnichar aChar = *aSrc++;
+    *aDest++ = (0xff & (aChar >> 8)) | (aChar << 8);
+  }
+}
+
 #define PREF_FORMFILL_BRANCH "browser.formfill."
 #define PREF_FORMFILL_ENABLE "enable"
 
 static const char *kFormHistoryFileName = "formhistory.dat";
 
 NS_INTERFACE_MAP_BEGIN(nsFormHistory)
-  NS_INTERFACE_MAP_ENTRY(nsIFormHistory)
+  NS_INTERFACE_MAP_ENTRY(nsIFormHistory2)
   NS_INTERFACE_MAP_ENTRY(nsIObserver)
   NS_INTERFACE_MAP_ENTRY(nsIFormSubmitObserver)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
@@ -86,69 +95,47 @@ PRBool nsFormHistory::gPrefsInitialized = PR_FALSE;
 nsFormHistory::nsFormHistory() :
   mEnv(nsnull),
   mStore(nsnull),
-  mTable(nsnull)
+  mTable(nsnull),
+  mReverseByteOrder(PR_FALSE)
 {
+  NS_ASSERTION(!gFormHistory, "nsFormHistory must be used as a service");
+  gFormHistory = this;
 }
 
 nsFormHistory::~nsFormHistory()
 {
+  NS_ASSERTION(gFormHistory == this,
+               "nsFormHistory must be used as a service");
   CloseDatabase();
+  gFormHistory = nsnull;
 }
 
 nsresult
 nsFormHistory::Init()
 {
-  gFormHistory = this;
-
   nsCOMPtr<nsIObserverService> service = do_GetService("@mozilla.org/observer-service;1");
   if (service)
-    service->AddObserver(this, NS_FORMSUBMIT_SUBJECT, PR_TRUE);
+    service->AddObserver(this, NS_EARLYFORMSUBMIT_SUBJECT, PR_TRUE);
   
   return NS_OK;
 }
 
 nsFormHistory *nsFormHistory::gFormHistory = nsnull;
 
-nsFormHistory *
-nsFormHistory::GetInstance()
-{
-  if (!gFormHistory) {
-    gFormHistory = new nsFormHistory();
-    if (!gFormHistory)
-      return nsnull;
-
-    NS_ADDREF(gFormHistory);  // addref for the global
-
-    if (NS_FAILED(gFormHistory->Init())) {
-      NS_RELEASE(gFormHistory);
-
-      return nsnull;
-    }  
-  }
-
-  NS_ADDREF(gFormHistory);   // addref for the getter
-
-  return gFormHistory;
-}
-
-
-void
-nsFormHistory::ReleaseInstance()
-{
-  NS_IF_RELEASE(gFormHistory);
-}
-
 /* static */ PRBool
 nsFormHistory::FormHistoryEnabled()
 {
   if (!gPrefsInitialized) {
     nsCOMPtr<nsIPrefService> prefService = do_GetService(NS_PREFSERVICE_CONTRACTID);
-    nsCOMPtr<nsIPrefBranch> branch;
-    prefService->GetBranch(PREF_FORMFILL_BRANCH, getter_AddRefs(branch));
-    branch->GetBoolPref(PREF_FORMFILL_ENABLE, &gFormHistoryEnabled);
 
-    nsCOMPtr<nsIPrefBranchInternal> branchInternal = do_QueryInterface(branch);
-    branchInternal->AddObserver(PREF_FORMFILL_BRANCH, gFormHistory, PR_TRUE);
+    prefService->GetBranch(PREF_FORMFILL_BRANCH,
+                           getter_AddRefs(gFormHistory->mPrefBranch));
+    gFormHistory->mPrefBranch->GetBoolPref(PREF_FORMFILL_ENABLE,
+                                           &gFormHistoryEnabled);
+
+    nsCOMPtr<nsIPrefBranch2> branchInternal =
+      do_QueryInterface(gFormHistory->mPrefBranch);
+    branchInternal->AddObserver(PREF_FORMFILL_ENABLE, gFormHistory, PR_TRUE);
 
     gPrefsInitialized = PR_TRUE;
   }
@@ -158,69 +145,28 @@ nsFormHistory::FormHistoryEnabled()
 
 
 ////////////////////////////////////////////////////////////////////////
-//// nsIFormHistory
+//// nsIFormHistory2
 
 NS_IMETHODIMP
-nsFormHistory::GetRowCount(PRUint32 *aRowCount)
+nsFormHistory::GetHasEntries(PRBool *aHasEntries)
 {
   nsresult rv = OpenDatabase(); // lazily ensure that the database is open
   NS_ENSURE_SUCCESS(rv, rv);
 
-  mdb_err err = mTable->GetCount(mEnv, aRowCount);
+  PRUint32 rowCount;
+  mdb_err err = mTable->GetCount(mEnv, &rowCount);
   NS_ENSURE_TRUE(!err, NS_ERROR_FAILURE);
-  
-  return NS_OK;
-}
 
-NS_IMETHODIMP
-nsFormHistory::GetEntryAt(PRUint32 aIndex, nsAString &aName, nsAString &aValue)
-{
-  nsresult rv = OpenDatabase(); // lazily ensure that the database is open
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIMdbRow> row;
-  mdb_err err = mTable->PosToRow(mEnv, aIndex, getter_AddRefs(row));
-  NS_ENSURE_TRUE(!err, NS_ERROR_FAILURE);
-  
-  GetRowValue(row, kToken_NameColumn, aName);
-  GetRowValue(row, kToken_ValueColumn, aValue);
-    
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsFormHistory::GetNameAt(PRUint32 aIndex, nsAString &aName)
-{
-  nsresult rv = OpenDatabase(); // lazily ensure that the database is open
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIMdbRow> row;
-  mdb_err err = mTable->PosToRow(mEnv, aIndex, getter_AddRefs(row));
-  NS_ENSURE_TRUE(!err, NS_ERROR_FAILURE);
-  
-  GetRowValue(row, kToken_NameColumn, aName);
-    
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsFormHistory::GetValueAt(PRUint32 aIndex, nsAString &aValue)
-{
-  nsresult rv = OpenDatabase(); // lazily ensure that the database is open
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIMdbRow> row;
-  mdb_err err = mTable->PosToRow(mEnv, aIndex, getter_AddRefs(row));
-  NS_ENSURE_TRUE(!err, NS_ERROR_FAILURE);
-  
-  GetRowValue(row, kToken_ValueColumn, aValue);
-    
+  *aHasEntries = rowCount != 0;
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFormHistory::AddEntry(const nsAString &aName, const nsAString &aValue)
 {
+  if (!FormHistoryEnabled())
+    return NS_OK;
+
   nsresult rv = OpenDatabase(); // lazily ensure that the database is open
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -230,105 +176,40 @@ nsFormHistory::AddEntry(const nsAString &aName, const nsAString &aValue)
 }
 
 NS_IMETHODIMP
-nsFormHistory::RemoveEntryAt(PRUint32 index)
+nsFormHistory::EntryExists(const nsAString &aName, const nsAString &aValue, PRBool *_retval)
+{
+  return EntriesExistInternal(&aName, &aValue, _retval);
+}
+
+NS_IMETHODIMP
+nsFormHistory::NameExists(const nsAString &aName, PRBool *_retval)
+{
+  return EntriesExistInternal(&aName, nsnull, _retval);
+}
+
+NS_IMETHODIMP
+nsFormHistory::RemoveEntry(const nsAString &aName, const nsAString &aValue)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsFormHistory::RemoveEntriesForName(const nsAString & name)
+nsFormHistory::RemoveEntriesForName(const nsAString &aName)
 {
-  return NS_ERROR_NOT_IMPLEMENTED;
+  return RemoveEntriesInternal(&aName);
 }
 
 NS_IMETHODIMP
 nsFormHistory::RemoveAllEntries()
 {
-  nsresult rv = OpenDatabase(); // lazily ensure that the database is open
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv = RemoveEntriesInternal(nsnull);
 
-  if (!mTable) return NS_OK;
-
-  mdb_err err;
-  mdb_count count;
-  err = mTable->GetCount(mEnv, &count);
-  if (err != 0) return NS_ERROR_FAILURE;
-
-  // Begin the batch.
-  int marker;
-  err = mTable->StartBatchChangeHint(mEnv, &marker);
-  NS_ASSERTION(err == 0, "unable to start batch");
-  if (err != 0) return NS_ERROR_FAILURE;
-
-  // XXX from here until end batch, no early returns!
-  for (mdb_pos pos = count - 1; pos >= 0; --pos) {
-    nsCOMPtr<nsIMdbRow> row;
-    err = mTable->PosToRow(mEnv, pos, getter_AddRefs(row));
-    NS_ASSERTION(err == 0, "unable to get row");
-    if (err != 0)
-      break;
-
-    NS_ASSERTION(row != nsnull, "no row");
-    if (! row)
-      continue;
-
-    // Officially cut the row *now*, before notifying any observers:
-    // that way, any re-entrant calls won't find the row.
-    err = mTable->CutRow(mEnv, row);
-    NS_ASSERTION(err == 0, "couldn't cut row");
-    if (err != 0)
-      continue;
+  if (NS_SUCCEEDED(rv))
+    rv = InitByteOrder(PR_TRUE);
   
-    // possibly avoid leakage
-    err = row->CutAllColumns(mEnv);
-    NS_ASSERTION(err == 0, "couldn't cut all columns");
-    // we'll notify regardless of whether we could successfully
-    // CutAllColumns or not.
-  }
+  rv |= Flush();
   
-  // Finish the batch.
-  err = mTable->EndBatchChangeHint(mEnv, &marker);
-  NS_ASSERTION(err == 0, "error ending batch");
-
-  return (err == 0) ? NS_OK : NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP
-nsFormHistory::EntryExists(const nsAString &aName, const nsAString &aValue, PRBool *_retval)
-{
-  // Unfortunately we have to do a brute force search through the database
-  // because mork didn't bother to implement any indexing functionality
-  
-  *_retval = PR_FALSE;
-  
-  nsresult rv = OpenDatabase(); // lazily ensure that the database is open
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Get a cursor to iterate through all rows in the database
-  nsCOMPtr<nsIMdbTableRowCursor> rowCursor;
-  mdb_err err = mTable->GetTableRowCursor(mEnv, -1, getter_AddRefs(rowCursor));
-  NS_ENSURE_TRUE(!err, NS_ERROR_FAILURE);
-  
-  nsCOMPtr<nsIMdbRow> row;
-  mdb_pos pos;
-  do {
-    rowCursor->NextRow(mEnv, getter_AddRefs(row), &pos);
-    NS_ENSURE_TRUE(row != nsnull, NS_ERROR_FAILURE);
-
-    // Check if the name and value combination match this row
-    nsAutoString name;
-    GetRowValue(row, kToken_NameColumn, name);
-    if (Compare(name, aName, nsCaseInsensitiveStringComparator()) == 0) {
-      nsAutoString value;
-      GetRowValue(row, kToken_ValueColumn, value);
-      if (Compare(value, aValue, nsCaseInsensitiveStringComparator()) == 0) {
-        *_retval = PR_TRUE;
-        break;
-      }
-    }
-  } while (row);
-  
-  return NS_OK;
+  return rv;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -338,8 +219,7 @@ NS_IMETHODIMP
 nsFormHistory::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *aData) 
 {
   if (!strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
-    nsCOMPtr<nsIPrefBranch> branch = do_QueryInterface(aSubject);
-    branch->GetBoolPref(PREF_FORMFILL_ENABLE, &gFormHistoryEnabled);
+    mPrefBranch->GetBoolPref(PREF_FORMFILL_ENABLE, &gFormHistoryEnabled);
   }
 
   return NS_OK;
@@ -398,6 +278,65 @@ nsFormHistory::Notify(nsIContent* aFormNode, nsIDOMWindowInternal* aWindow, nsIU
 ////////////////////////////////////////////////////////////////////////
 //// Database I/O
 
+class SatchelErrorHook : public nsIMdbErrorHook
+{
+public:
+  NS_DECL_ISUPPORTS
+
+  // nsIMdbErrorHook
+  NS_IMETHOD OnErrorString(nsIMdbEnv* ev, const char* inAscii);
+  NS_IMETHOD OnErrorYarn(nsIMdbEnv* ev, const mdbYarn* inYarn);
+  NS_IMETHOD OnWarningString(nsIMdbEnv* ev, const char* inAscii);
+  NS_IMETHOD OnWarningYarn(nsIMdbEnv* ev, const mdbYarn* inYarn);
+  NS_IMETHOD OnAbortHintString(nsIMdbEnv* ev, const char* inAscii);
+  NS_IMETHOD OnAbortHintYarn(nsIMdbEnv* ev, const mdbYarn* inYarn);
+};
+
+// nsIMdbErrorHook has no IID!
+NS_IMPL_ISUPPORTS0(SatchelErrorHook)
+
+NS_IMETHODIMP
+SatchelErrorHook::OnErrorString(nsIMdbEnv *ev, const char *inAscii)
+{
+  printf("mork error: %s\n", inAscii);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+SatchelErrorHook::OnErrorYarn(nsIMdbEnv *ev, const mdbYarn* inYarn)
+{
+  printf("mork error yarn: %p\n", (void*)inYarn);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+SatchelErrorHook::OnWarningString(nsIMdbEnv *ev, const char *inAscii)
+{
+  printf("mork warning: %s\n", inAscii);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+SatchelErrorHook::OnWarningYarn(nsIMdbEnv *ev, const mdbYarn *inYarn)
+{
+  printf("mork warning yarn: %p\n", (void*)inYarn);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+SatchelErrorHook::OnAbortHintString(nsIMdbEnv *ev, const char *inAscii)
+{
+  printf("mork abort: %s\n", inAscii);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+SatchelErrorHook::OnAbortHintYarn(nsIMdbEnv *ev, const mdbYarn *inYarn)
+{
+  printf("mork abort yarn: %p\n", (void*)inYarn);
+  return NS_OK;
+}
+
 nsresult
 nsFormHistory::OpenDatabase()
 {
@@ -408,12 +347,10 @@ nsFormHistory::OpenDatabase()
   nsCOMPtr <nsIFile> historyFile;
   nsresult rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(historyFile));
   NS_ENSURE_SUCCESS(rv, rv);
-  historyFile->Append(NS_ConvertUTF8toUCS2(kFormHistoryFileName));
+  historyFile->Append(NS_ConvertUTF8toUTF16(kFormHistoryFileName));
 
   // Get an Mdb Factory
-  static NS_DEFINE_CID(kMorkCID, NS_MORK_CID);
-  nsCOMPtr<nsIMdbFactoryFactory> mdbFactory =
-      do_CreateInstance(kMorkCID, &rv);
+  nsCOMPtr<nsIMdbFactoryFactory> mdbFactory = do_CreateInstance(NS_MORK_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = mdbFactory->GetMdbFactory(getter_AddRefs(mMdbFactory));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -423,28 +360,34 @@ nsFormHistory::OpenDatabase()
   NS_ASSERTION(err == 0, "ERROR: Unable to create Form History mdb");
   mEnv->SetAutoClear(PR_TRUE);
   NS_ENSURE_TRUE(!err, NS_ERROR_FAILURE);
+  mEnv->SetErrorHook(new SatchelErrorHook());
 
   nsCAutoString filePath;
   historyFile->GetNativePath(filePath);
   PRBool exists = PR_TRUE;
   historyFile->Exists(&exists);
+
+  PRBool createdNew = PR_FALSE;
   
   if (!exists || NS_FAILED(rv = OpenExistingFile(filePath.get()))) {
     // If the file doesn't exist, or we fail trying to open it,
     // then make sure it is deleted and then create an empty database file
     historyFile->Remove(PR_FALSE);
     rv = CreateNewFile(filePath.get());
+    createdNew = PR_TRUE;
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Get the initial size of the file, needed later for Commit
   historyFile->GetFileSize(&mFileSizeOnDisk);
 
+  rv = InitByteOrder(createdNew);
+
   /* // TESTING: Add a row to the database
   nsAutoString foopy;
-  foopy.AssignLiteral("foopy");
+  foopy.AssignWithConversion("foopy");
   nsAutoString oogly;
-  oogly.AssignLiteral("oogly");
+  oogly.AssignWithConversion("oogly");
   AppendRow(foopy, oogly, nsnull);
   Flush(); */
   
@@ -464,7 +407,7 @@ nsFormHistory::OpenDatabase()
     printf("ROW: %s - %s\n", ToNewCString(name), ToNewCString(value));
   } */
 
-  return NS_OK;
+  return rv;
 }
 
 nsresult
@@ -503,6 +446,10 @@ nsFormHistory::OpenExistingFile(const char *aPath)
     return NS_ERROR_FAILURE;
   }
 
+  err = mTable->GetMetaRow(mEnv, &oid, nsnull, getter_AddRefs(mMetaRow));
+  if (err)
+    NS_WARNING("Could not get meta row");
+
   if (NS_FAILED(thumbErr))
     err = thumbErr;
 
@@ -530,6 +477,13 @@ nsFormHistory::CreateNewFile(const char *aPath)
   err = mStore->NewTable(mEnv, kToken_RowScope, kToken_Kind, PR_TRUE, nsnull, &mTable);
   NS_ENSURE_TRUE(!err && mTable, NS_ERROR_FAILURE);
 
+  mdbOid oid = {kToken_RowScope, 1};
+  err = mTable->GetMetaRow(mEnv, &oid, nsnull, getter_AddRefs(mMetaRow));
+  if (err) {
+    NS_WARNING("Could not get meta row");
+    return NS_ERROR_FAILURE;
+  }
+
    // oldTable will only be set if we detected a corrupt db, and are 
    // trying to restore data from it.
   if (oldTable)
@@ -537,7 +491,7 @@ nsFormHistory::CreateNewFile(const char *aPath)
 
   // Force a commit now to get it written out.
   nsCOMPtr<nsIMdbThumb> thumb;
-  err = mStore->LargeCommit(mEnv, getter_AddRefs(thumb));
+  err = mStore->CompressCommit(mEnv, getter_AddRefs(thumb));
   NS_ENSURE_TRUE(!err, NS_ERROR_FAILURE);
 
   PRBool done;
@@ -551,6 +505,8 @@ nsFormHistory::CloseDatabase()
 {
   Flush();
 
+  mMetaRow = nsnull;
+
   if (mTable)
     mTable->Release();
 
@@ -587,6 +543,9 @@ nsFormHistory::CreateTokens()
   err = mStore->StringToToken(mEnv, "Name", &kToken_NameColumn);
   if (err != 0) return NS_ERROR_FAILURE;
 
+  err = mStore->StringToToken(mEnv, "ByteOrder", &kToken_ByteOrder);
+  if (err != 0) return NS_ERROR_FAILURE;
+
   return NS_OK;
 }
 
@@ -599,7 +558,7 @@ nsFormHistory::Flush()
   mdb_err err;
 
   nsCOMPtr<nsIMdbThumb> thumb;
-  err = mStore->LargeCommit(mEnv, getter_AddRefs(thumb));
+  err = mStore->CompressCommit(mEnv, getter_AddRefs(thumb));
 
   if (err == 0)
     err = UseThumb(thumb, nsnull);
@@ -645,7 +604,7 @@ nsFormHistory::CopyRowsFromTable(nsIMdbTable *sourceTable)
     rowId.mOid_Id = mdb_id(-1);
 
     nsCOMPtr<nsIMdbRow> newRow;
-    mdb_err err = mTable->NewRow(mEnv, &rowId, getter_AddRefs(newRow));
+    mTable->NewRow(mEnv, &rowId, getter_AddRefs(newRow));
     newRow->SetRow(mEnv, row);
     mTable->AddRow(mEnv, newRow);
   } while (row);
@@ -687,9 +646,23 @@ nsresult
 nsFormHistory::SetRowValue(nsIMdbRow *aRow, mdb_column aCol, const nsAString &aValue)
 {
   PRInt32 len = aValue.Length() * sizeof(PRUnichar);
+  PRUnichar *swapval = nsnull;
+  mdbYarn yarn = {nsnull, len, len, 0, 0, nsnull};
+  const nsPromiseFlatString& buffer = PromiseFlatString(aValue);
+
+  if (mReverseByteOrder) {
+    swapval = new PRUnichar[aValue.Length()];
+    if (!swapval)
+      return NS_ERROR_OUT_OF_MEMORY;
+    SwapBytes(swapval, buffer.get(), aValue.Length());
+    yarn.mYarn_Buf = swapval;
+  }
+  else
+    yarn.mYarn_Buf = (void*)buffer.get();
 
-  mdbYarn yarn = {(void *)ToNewUnicode(aValue), len, len, 0, 0, nsnull};
   mdb_err err = aRow->AddColumn(mEnv, aCol, &yarn);
+
+  delete swapval;
   
   return err ? NS_ERROR_FAILURE : NS_OK;
 }
@@ -707,9 +680,20 @@ nsFormHistory::GetRowValue(nsIMdbRow *aRow, mdb_column aCol, nsAString &aValue)
     return NS_OK;
   
   switch (yarn.mYarn_Form) {
-    case 0: // unicode
-      aValue.Assign((const PRUnichar *)yarn.mYarn_Buf, yarn.mYarn_Fill/sizeof(PRUnichar));
+    case 0: { // unicode
+      PRUint32 len = yarn.mYarn_Fill / sizeof(PRUnichar);
+      if (mReverseByteOrder) {
+        PRUnichar *swapval = new PRUnichar[len];
+        if (!swapval)
+          return NS_ERROR_OUT_OF_MEMORY;
+        SwapBytes(swapval, (const PRUnichar*)yarn.mYarn_Buf, len);
+        aValue.Assign(swapval, len);
+        delete swapval;
+      }
+      else
+        aValue.Assign((const PRUnichar *)yarn.mYarn_Buf, len);
       break;
+    }
     default:
       return NS_ERROR_UNEXPECTED;
   }
@@ -720,7 +704,7 @@ nsFormHistory::GetRowValue(nsIMdbRow *aRow, mdb_column aCol, nsAString &aValue)
 nsresult
 nsFormHistory::AutoCompleteSearch(const nsAString &aInputName,
                                   const nsAString &aInputValue,
-                                  nsIAutoCompleteMdbResult *aPrevResult,
+                                  nsIAutoCompleteMdbResult2 *aPrevResult,
                                   nsIAutoCompleteResult **aResult)
 {
   if (!FormHistoryEnabled())
@@ -729,7 +713,7 @@ nsFormHistory::AutoCompleteSearch(const nsAString &aInputName,
   nsresult rv = OpenDatabase(); // lazily ensure that the database is open
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIAutoCompleteMdbResult> result;
+  nsCOMPtr<nsIAutoCompleteMdbResult2> result;
   
   if (aPrevResult) {
     result = aPrevResult;
@@ -748,7 +732,8 @@ nsFormHistory::AutoCompleteSearch(const nsAString &aInputName,
 
     result->SetSearchString(aInputValue);
     result->Init(mEnv, mTable);
-    result->SetTokens(kToken_ValueColumn, nsIAutoCompleteMdbResult::kUnicharType, nsnull, nsIAutoCompleteMdbResult::kUnicharType);
+    result->SetTokens(kToken_ValueColumn, nsIAutoCompleteMdbResult2::kUnicharType, nsnull, nsIAutoCompleteMdbResult2::kUnicharType);
+    result->SetReverseByteOrder(mReverseByteOrder);
 
     // Get a cursor to iterate through all rows in the database
     nsCOMPtr<nsIMdbTableRowCursor> rowCursor;
@@ -791,7 +776,7 @@ nsFormHistory::AutoCompleteSearch(const nsAString &aInputName,
         result->AddRow(matchingRows[items[i]]);
 
         // Free up these strings we owned.
-        delete (PRUnichar *) matchingValues[i];
+        NS_Free(matchingValues[i]);
       }
 
       delete[] items;
@@ -845,3 +830,174 @@ nsFormHistory::RowMatch(nsIMdbRow *aRow, const nsAString &aInputName, const nsAS
   
   return PR_FALSE;
 }
+
+nsresult
+nsFormHistory::EntriesExistInternal(const nsAString *aName, const nsAString *aValue, PRBool *_retval)
+{
+  // Unfortunately we have to do a brute force search through the database
+  // because mork didn't bother to implement any indexing functionality
+  
+  *_retval = PR_FALSE;
+  
+  nsresult rv = OpenDatabase(); // lazily ensure that the database is open
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Get a cursor to iterate through all rows in the database
+  nsCOMPtr<nsIMdbTableRowCursor> rowCursor;
+  mdb_err err = mTable->GetTableRowCursor(mEnv, -1, getter_AddRefs(rowCursor));
+  NS_ENSURE_TRUE(!err, NS_ERROR_FAILURE);
+  
+  nsCOMPtr<nsIMdbRow> row;
+  mdb_pos pos;
+  do {
+    rowCursor->NextRow(mEnv, getter_AddRefs(row), &pos);
+    if (!row)
+      break;
+
+    // Check if the name and value combination match this row
+    nsAutoString name;
+    GetRowValue(row, kToken_NameColumn, name);
+
+    if (Compare(name, *aName, nsCaseInsensitiveStringComparator()) == 0) {
+      nsAutoString value;
+      GetRowValue(row, kToken_ValueColumn, value);
+      if (!aValue || Compare(value, *aValue, nsCaseInsensitiveStringComparator()) == 0) {
+        *_retval = PR_TRUE;
+        break;
+      }
+    }
+  } while (1);
+  
+  return NS_OK;
+}
+
+nsresult
+nsFormHistory::RemoveEntriesInternal(const nsAString *aName)
+{
+  nsresult rv = OpenDatabase(); // lazily ensure that the database is open
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (!mTable) return NS_OK;
+
+  mdb_err err;
+  mdb_count count;
+  nsAutoString name;
+  err = mTable->GetCount(mEnv, &count);
+  if (err != 0) return NS_ERROR_FAILURE;
+
+  // Begin the batch.
+  int marker;
+  err = mTable->StartBatchChangeHint(mEnv, &marker);
+  NS_ASSERTION(err == 0, "unable to start batch");
+  if (err != 0) return NS_ERROR_FAILURE;
+
+  for (mdb_pos pos = count - 1; pos >= 0; --pos) {
+    nsCOMPtr<nsIMdbRow> row;
+    err = mTable->PosToRow(mEnv, pos, getter_AddRefs(row));
+    NS_ASSERTION(err == 0, "unable to get row");
+    if (err != 0)
+      break;
+
+    NS_ASSERTION(row != nsnull, "no row");
+    if (! row)
+      continue;
+
+    // Check if the name matches this row
+    GetRowValue(row, kToken_NameColumn, name);
+    
+    if (!aName || Compare(name, *aName, nsCaseInsensitiveStringComparator()) == 0) {
+
+      // Officially cut the row *now*, before notifying any observers:
+      // that way, any re-entrant calls won't find the row.
+      err = mTable->CutRow(mEnv, row);
+      NS_ASSERTION(err == 0, "couldn't cut row");
+      if (err != 0)
+        continue;
+  
+      // possibly avoid leakage
+      err = row->CutAllColumns(mEnv);
+      NS_ASSERTION(err == 0, "couldn't cut all columns");
+      // we'll notify regardless of whether we could successfully
+      // CutAllColumns or not.
+    }
+
+  }
+  
+  // Finish the batch.
+  err = mTable->EndBatchChangeHint(mEnv, &marker);
+  NS_ASSERTION(err == 0, "error ending batch");
+
+  return (err == 0) ? NS_OK : NS_ERROR_FAILURE;
+
+}
+
+nsresult
+nsFormHistory::InitByteOrder(PRBool aForce)
+{
+  // bigEndian and littleEndian are endianness markers that are stored in
+  // the formhistory db as UTF-16.  Define them to be strings easily
+  // recognized in either endianness.
+  nsAutoString bigEndianByteOrder((PRUnichar*)"BBBB", 2);
+  nsAutoString littleEndianByteOrder((PRUnichar*)"llll", 2);
+#ifdef IS_BIG_ENDIAN
+  nsAutoString nativeByteOrder(bigEndianByteOrder);
+#else
+  nsAutoString nativeByteOrder(littleEndianByteOrder);
+#endif
+
+  nsAutoString fileByteOrder;
+  nsresult rv = NS_OK;
+
+  if (!aForce)
+    rv = GetByteOrder(fileByteOrder);
+
+  if (aForce || NS_FAILED(rv) ||
+      !(fileByteOrder.Equals(bigEndianByteOrder) ||
+        fileByteOrder.Equals(littleEndianByteOrder))) {
+#if defined(XP_MACOSX) && defined(IS_LITTLE_ENDIAN)
+    // The formhistory db did not carry endiannes information until the
+    // initial x86 Mac release.  There are a lot of users out there who
+    // will be switching from ppc versions to x86, and their unmarked
+    // formhistory files are big-endian.  On x86 Macs, unless aForce is set
+    // (indicating formhistory reset or a brand-new db), use big-endian byte
+    // ordering and turn swapping on.
+    if (aForce) {
+      mReverseByteOrder = PR_FALSE;
+      rv = SaveByteOrder(nativeByteOrder);
+    }
+    else {
+      mReverseByteOrder = PR_TRUE;
+      rv = SaveByteOrder(bigEndianByteOrder);
+    }
+#else
+    mReverseByteOrder = PR_FALSE;
+    rv = SaveByteOrder(nativeByteOrder);
+#endif
+  }
+  else
+    mReverseByteOrder = !fileByteOrder.Equals(nativeByteOrder);
+
+  return rv;
+}
+
+nsresult
+nsFormHistory::GetByteOrder(nsAString& aByteOrder)
+{
+  NS_ENSURE_SUCCESS(OpenDatabase(), NS_ERROR_FAILURE);
+
+  mdb_err err = GetRowValue(mMetaRow, kToken_ByteOrder, aByteOrder);
+  NS_ENSURE_FALSE(err, NS_ERROR_FAILURE);
+
+  return NS_OK;
+}
+
+nsresult
+nsFormHistory::SaveByteOrder(const nsAString& aByteOrder)
+{
+  NS_ENSURE_SUCCESS(OpenDatabase(), NS_ERROR_FAILURE);
+
+  mdb_err err = SetRowValue(mMetaRow, kToken_ByteOrder, aByteOrder);
+  NS_ENSURE_FALSE(err, NS_ERROR_FAILURE);
+
+  return NS_OK;
+}
