@@ -1,25 +1,41 @@
 # -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
-# The contents of this file are subject to the Mozilla Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/MPL/
-# 
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-# 
-# The Original Code is mozilla.org code.
-# 
-# The Initial Developer of the Original Code is Netscape
-# Communications Corporation.  Portions created by Netscape are
-# Copyright (C) Netscape Communications Corporation.  All
-# Rights Reserved.
-# 
-# Contributor(s): 
-#    Doron Rosenberg (doronr@naboonline.com) 
-#    Neil Rashbrook (neil@parkwaycc.co.uk)
+# ***** BEGIN LICENSE BLOCK *****
+# Version: MPL 1.1/GPL 2.0/LGPL 2.1
 #
+# The contents of this file are subject to the Mozilla Public License Version
+# 1.1 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+# http://www.mozilla.org/MPL/
+#
+# Software distributed under the License is distributed on an "AS IS" basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+# for the specific language governing rights and limitations under the
+# License.
+#
+# The Original Code is mozilla.org code.
+#
+# The Initial Developer of the Original Code is
+# Netscape Communications Corporation.  Portions created by Netscape are Copyright (C) Netscape Communications Corporation.  All Rights Reserved.
+# Portions created by the Initial Developer are Copyright (C) 2001
+# the Initial Developer. All Rights Reserved.
+#
+# Contributor(s):
+#   Doron Rosenberg (doronr@naboonline.com)
+#   Neil Rashbrook (neil@parkwaycc.co.uk)
+#
+# Alternatively, the contents of this file may be used under the terms of
+# either the GNU General Public License Version 2 or later (the "GPL"), or
+# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+# in which case the provisions of the GPL or the LGPL are applicable instead
+# of those above. If you wish to allow use of your version of this file only
+# under the terms of either the GPL or the LGPL, and not to allow others to
+# use your version of this file under the terms of the MPL, indicate your
+# decision by deleting the provisions above and replace them with the notice
+# and other provisions required by the GPL or the LGPL. If you do not delete
+# the provisions above, a recipient may use your version of this file under
+# the terms of any one of the MPL, the GPL or the LGPL.
+#
+# ***** END LICENSE BLOCK *****
 
 const pageLoaderIface = Components.interfaces.nsIWebPageDescriptor;
 const nsISelectionPrivate = Components.interfaces.nsISelectionPrivate;
@@ -52,6 +68,12 @@ function onLoadViewSource()
 {
   viewSource(window.arguments[0]);
   document.commandDispatcher.focusedWindow = content;
+  gFindBar.initFindBar();
+}
+
+function onUnloadViewSource()
+{
+  gFindBar.uninitFindBar();
 }
 
 function getBrowser()
@@ -106,7 +128,9 @@ function viewSource(url)
           var arrayArgComponents = arg.split('=');
           if (arrayArgComponents) {
             //we should "inherit" the charset menu setting in a new window
-            getMarkupDocumentViewer().defaultCharacterSet = arrayArgComponents[1];
+            var docCharset = getBrowser().docShell.QueryInterface
+                               (Components.interfaces.nsIDocCharset);
+            docCharset.charset = arrayArgComponents[1];
           } 
         }
       } catch (ex) {
@@ -209,13 +233,7 @@ function ViewSourceClose()
   window.close();
 }
 
-function BrowserReload()
-{
-  // Reload will always reload from cache which is probably not what's wanted
-  BrowserReloadSkipCache();
-}
-
-function BrowserReloadSkipCache()
+function ViewSourceReload()
 {
   const webNavigation = getBrowser().webNavigation;
   webNavigation.reload(webNavigation.LOAD_FLAGS_BYPASS_PROXY | webNavigation.LOAD_FLAGS_BYPASS_CACHE);
@@ -364,9 +382,8 @@ function goToLine(line)
 
   gLastLineFound = line;
 
-  //pch: don't update the status bar for now
-  //document.getElementById("statusbar-line-col").label = getViewSourceBundle()
-  //    .getFormattedString("statusBarLineCol", [line, 1]);
+  document.getElementById("statusbar-line-col").label =
+    getViewSourceBundle().getFormattedString("statusBarLineCol", [line, 1]);
 
   return true;
 }
@@ -399,8 +416,6 @@ function updateStatusBar()
   findLocation(null, -1, 
       selection.focusNode, selection.focusOffset, interlinePosition, result);
 
-  //pch no status bar for now
-  return;
   statusBarField.label = getViewSourceBundle()
       .getFormattedString("statusBarLineCol", [result.line, result.col]);
 }
@@ -586,8 +601,3 @@ function BrowserSetForcedDetector(doReload)
     PageLoader.loadPage(PageLoader.currentDescriptor, pageLoaderIface.DISPLAY_NORMAL);
   }
 }
-
-function getMarkupDocumentViewer()
-{
-  return gBrowser.markupDocumentViewer;
-}
