@@ -1,32 +1,44 @@
-# The contents of this file are subject to the Netscape Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/NPL/
-# 
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
-# 
+# ***** BEGIN LICENSE BLOCK *****
+# Version: MPL 1.1/GPL 2.0/LGPL 2.1
+#
+# The contents of this file are subject to the Mozilla Public License Version
+# 1.1 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+# http://www.mozilla.org/MPL/
+#
+# Software distributed under the License is distributed on an "AS IS" basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+# for the specific language governing rights and limitations under the
+# License.
+#
 # The Original Code is Mozilla Communicator client code, released
 # March 31, 1998.
-# 
-# The Initial Developer of the Original Code is David Hyatt. 
-# Portions created by David Hyatt are
-# Copyright (C) 2002 David Hyatt. All
-# Rights Reserved.
-# 
-# Contributor(s): 
+#
+# The Initial Developer of the Original Code is
+# David Hyatt.
+# Portions created by the Initial Developer are Copyright (C) 2002
+# the Initial Developer. All Rights Reserved.
+#
+# Contributor(s):
 #   David Hyatt (hyatt@apple.com)
 #   Blake Ross (blaker@netscape.com)
 #   Joe Hewitt (hewitt@netscape.com)
+#
+# Alternatively, the contents of this file may be used under the terms of
+# either the GNU General Public License Version 2 or later (the "GPL"), or
+# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+# in which case the provisions of the GPL or the LGPL are applicable instead
+# of those above. If you wish to allow use of your version of this file only
+# under the terms of either the GPL or the LGPL, and not to allow others to
+# use your version of this file under the terms of the MPL, indicate your
+# decision by deleting the provisions above and replace them with the notice
+# and other provisions required by the GPL or the LGPL. If you do not delete
+# the provisions above, a recipient may use your version of this file under
+# the terms of any one of the MPL, the GPL or the LGPL.
+#
+# ***** END LICENSE BLOCK *****
 
 const kRowMax = 4;
-const kWindowWidth = 635;
-const kWindowHeight = 400;
-const kAnimateIncrement = 50;
-const kAnimateSteps = kWindowHeight / kAnimateIncrement - 1;
-const kVSizeSlop = 5;
 
 var gToolboxDocument = null;
 var gToolbox = null;
@@ -36,7 +48,13 @@ var gToolboxIconSize = false;
 
 function onLoad()
 {
-  gToolbox = window.arguments[0];
+  InitWithToolbox(window.arguments[0]);
+  repositionDialog();
+}
+
+function InitWithToolbox(aToolbox)
+{
+  gToolbox = aToolbox;
   gToolboxDocument = gToolbox.ownerDocument;
   
   gToolbox.addEventListener("draggesture", onToolbarDragGesture, false);
@@ -44,22 +62,21 @@ function onLoad()
   gToolbox.addEventListener("dragexit", onToolbarDragExit, false);
   gToolbox.addEventListener("dragdrop", onToolbarDragDrop, false);
 
-  repositionDialog();
-  
-  document.documentElement.setAttribute("hidechrome", "true");
-
   initDialog();
 }
 
-function onUnload(aEvent)
+function finishToolbarCustomization()
 {
   removeToolboxListeners();
   unwrapToolbarItems();
   persistCurrentSets();
   
   notifyParentComplete();
-  
-  window.close();
+}
+
+function onUnload(aEvent)
+{
+  finishToolbarCustomization();
 }
 
 function onAccept(aEvent)
@@ -91,17 +108,16 @@ function initDialog()
 function repositionDialog()
 {
   // Position the dialog touching the bottom of the toolbox and centered with 
-  // it. We must resize the window smaller first so that it is positioned 
-  // properly. 
-  var screenX = gToolbox.boxObject.screenX + ((gToolbox.boxObject.width - kWindowWidth) / 2);
+  // it.
+  var width;
+  if (document.documentElement.hasAttribute("width"))
+    width = document.documentElement.getAttribute("width");
+  else
+    width = parseInt(document.documentElement.style.width);
+  var screenX = gToolbox.boxObject.screenX 
+                + ((gToolbox.boxObject.width - width) / 2);
   var screenY = gToolbox.boxObject.screenY + gToolbox.boxObject.height;
 
-  var newHeight = kWindowHeight;
-  if (newHeight >= screen.availHeight - screenY - kVSizeSlop) {
-    newHeight = screen.availHeight - screenY - kVSizeSlop;
-  }
-
-  window.resizeTo(kWindowWidth, newHeight);
   window.moveTo(screenX, screenY);
 }
 
@@ -215,9 +231,6 @@ function unwrapToolbarItems()
     if (paletteItem.hasAttribute("itemcommand"))
       toolbarItem.setAttribute("command", paletteItem.getAttribute("itemcommand"));
 
-    // We need the removeChild here because replaceChild and XBL no workee
-    // together.  See bug 193298.
-    paletteItem.removeChild(toolbarItem);
     paletteItem.parentNode.replaceChild(toolbarItem, paletteItem);
   }
 }
@@ -486,10 +499,11 @@ function setWrapperType(aItem, aWrapper)
 function setDragActive(aItem, aValue)
 {
   var node = aItem;
-  var value = "left";
+  var direction = window.getComputedStyle(aItem, null).direction;
+  var value = direction == "ltr"? "left" : "right";
   if (aItem.localName == "toolbar") {
     node = aItem.lastChild;
-    value = "right";
+    value = direction == "ltr"? "right" : "left";
   }
   
   if (!node)
@@ -511,27 +525,42 @@ function addNewToolbar()
   var stringBundle = document.getElementById("stringBundle");
   var message = stringBundle.getString("enterToolbarName");
   var title = stringBundle.getString("enterToolbarTitle");
-  
+
   var name = {};
-  while (1) {
-    if (!promptService.prompt(window, title, message, name, null, {})) {
+
+  while (true) {
+
+    if (!promptService.prompt(window, title, message, name, null, {}))
       return;
-    } else {
-      // Check for an existing toolbar with the same name and prompt again
-      // if a conflict is found
-      var nameToId = "__customToolbar_" + name.value.replace(" ", "");
-      var existingToolbar = gToolboxDocument.getElementById(nameToId);
-      if (existingToolbar) {
-        message = stringBundle.getFormattedString("enterToolbarDup", [name.value]);
-      } else {
+    
+    if (!name.value) {
+      message = stringBundle.getFormattedString("enterToolbarBlank", [name.value]);
+      continue;
+    }
+
+    var dupeFound = false;
+
+     // Check for an existing toolbar with the same display name
+    for (i = 0; i < gToolbox.childNodes.length; ++i) {
+      var toolbar = gToolbox.childNodes[i];
+      var toolbarName = toolbar.getAttribute("toolbarname");
+
+      if (toolbarName == name.value &&
+          toolbar.getAttribute("type") != "menubar" &&
+          toolbar.nodeName == 'toolbar') {
+        dupeFound = true;
         break;
       }
     }
+
+    if (!dupeFound)
+      break;
+
+    message = stringBundle.getFormattedString("enterToolbarDup", [name.value]);
   }
     
   gToolbox.appendCustomToolbar(name.value, "");
   
-  repositionDialog();
   gToolboxChanged = true;
 }
 
@@ -541,6 +570,10 @@ function addNewToolbar()
  */
 function restoreDefaultSet()
 {
+  // Save disabled/command states, because we're
+  // going to recreate the wrappers and lose this
+  var savedAttributes = saveItemAttributes(["itemdisabled", "itemcommand"]);
+
   // Restore the defaultset for fixed toolbars.
   var toolbar = gToolbox.firstChild;
   while (toolbar) {
@@ -578,10 +611,49 @@ function restoreDefaultSet()
   // Now re-wrap the items on the toolbar.
   wrapToolbarItems();
 
-  repositionDialog();
+  // Restore the disabled and command states
+  restoreItemAttributes(["itemdisabled", "itemcommand"], savedAttributes);
+
   gToolboxChanged = true;
 }
 
+function saveItemAttributes(aAttributeList)
+{
+  var items = [];
+  var paletteItems = gToolbox.getElementsByTagName("toolbarpaletteitem");
+  for (var i = 0; i < paletteItems.length; i++) {
+    var paletteItem = paletteItems.item(i);
+    for (var j = 0; j < aAttributeList.length; j++) {
+      var attr = aAttributeList[j];
+      if (paletteItem.hasAttribute(attr)) {
+        items.push([paletteItem.id, attr, paletteItem.getAttribute(attr)]);
+      }
+    }
+  }
+  return items;
+}
+
+function restoreItemAttributes(aAttributeList, aSavedAttrList)
+{
+  var paletteItems = gToolbox.getElementsByTagName("toolbarpaletteitem");
+
+  for (var i = 0; i < paletteItems.length; i++) {
+    var paletteItem = paletteItems.item(i);
+
+    // if the item is supposed to have this, it'll get
+    // restored from the saved list
+    for (var j = 0; j < aAttributeList.length; j++)
+      paletteItem.removeAttribute(aAttributeList[j]);
+
+    for (var j = 0; j < aSavedAttrList.length; j++) {
+      var savedAttr = aSavedAttrList[j];
+      if (paletteItem.id == savedAttr[0]) {
+        paletteItem.setAttribute(savedAttr[1], savedAttr[2]);
+      }
+    }
+  }
+}
+
 function updateIconSize(aUseSmallIcons)
 {
   gToolboxIconSize = aUseSmallIcons ? "small" : "large";
@@ -596,8 +668,6 @@ function updateIconSize(aUseSmallIcons)
       gToolboxDocument.persist(toolbar.id, "iconsize");
     }
   }
-
-  repositionDialog();
 }
 
 function updateToolbarMode(aModeValue)
@@ -615,8 +685,6 @@ function updateToolbarMode(aModeValue)
 
   var iconSizeCheckbox = document.getElementById("smallicons");
   iconSizeCheckbox.disabled = aModeValue == "text";
-
-  repositionDialog();
 }
 
 
@@ -715,11 +783,16 @@ var toolbarDNDObserver =
     if (dropTarget.localName == "toolbar") {
       gCurrentDragOverItem = dropTarget;
     } else {
-      var dropTargetWidth = dropTarget.boxObject.width;
-      var dropTargetX = dropTarget.boxObject.x;
-
       gCurrentDragOverItem = null;
-      if (aEvent.clientX > (dropTargetX + (dropTargetWidth / 2))) {
+
+      var direction = window.getComputedStyle(dropTarget.parentNode, null).direction;
+      var dropTargetCenter = dropTarget.boxObject.x + (dropTarget.boxObject.width / 2);
+      if (direction == "ltr")
+        dragAfter = aEvent.clientX > dropTargetCenter;
+      else
+        dragAfter = aEvent.clientX < dropTargetCenter;
+        
+      if (dragAfter) {
         gCurrentDragOverItem = dropTarget.nextSibling;
         if (!gCurrentDragOverItem)
           gCurrentDragOverItem = toolbar;
@@ -843,7 +916,6 @@ var toolbarDNDObserver =
     
     gCurrentDragOverItem = null;
 
-    repositionDialog();
     gToolboxChanged = true;
   },
   
@@ -900,7 +972,6 @@ var paletteDNDObserver =
       }
     }
     
-    repositionDialog();
     gToolboxChanged = true;
   },
   
