@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -22,6 +22,7 @@
  * Contributor(s):
  *   Bill Law       law@netscape.com
  *   IBM Corp.
+ *   Rich Walsh     dragtext@e-vertise.com
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -29,11 +30,11 @@
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -48,20 +49,20 @@
 #include "nsAppRunner.h"
 #include "nsXULAppAPI.h"
 #include "nsString.h"
-#include "nsICmdLineService.h"
+#include "nsIBrowserDOMWindow.h"
+#include "nsICommandLineRunner.h"
 #include "nsCOMPtr.h"
 #include "nsXPIDLString.h"
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
-#include "nsICmdLineHandler.h"
-#include "nsIDOMWindow.h"
+#include "nsPIDOMWindow.h"
+#include "nsIDOMChromeWindow.h"
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsISupportsArray.h"
 #include "nsIWindowWatcher.h"
-#include "nsIDOMWindowInternal.h"
-#include "nsIScriptGlobalObject.h"
 #include "nsIDocShell.h"
+#include "nsIDocShellTreeItem.h"
 #include "nsIBaseWindow.h"
 #include "nsIWidget.h"
 #include "nsIAppShellService.h"
@@ -71,16 +72,14 @@
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIPromptService.h"
 #include "nsNetCID.h"
+#include "nsNetUtil.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
-#include "nsXPCOM.h"
-#ifdef MOZ_PHOENIX
-#include "nsIShellService.h"
-#endif
 
 // These are needed to load a URL in a browser window.
 #include "nsIDOMLocation.h"
 #include "nsIJSContextStack.h"
+#include "nsIWebNavigation.h"
 #include "nsIWindowMediator.h"
 
 #include <stdlib.h>
@@ -320,7 +319,7 @@ private:
                                                     HDDEDATA hdata,
                                                     ULONG    dwData1,
                                                     ULONG    dwData2 );
-    static void HandleRequest( LPBYTE request, PRBool newWindow = PR_TRUE );
+    static void HandleCommandLine(const char* aCmdLineString, nsIFile* aWorkingDir, PRUint32 aState);
     static void ParseDDEArg( HSZ args, int index, nsCString& string);
     static void ParseDDEArg( const char* args, int index, nsCString& aString);
     static void ActivateLastWindow();
@@ -328,11 +327,8 @@ private:
     static HDDEDATA CreateDDEData( LPBYTE value, DWORD len );
     static PRBool   InitTopicStrings();
     static int      FindTopic( HSZ topic );
-    static nsresult GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResult );
     static nsresult OpenWindow( const char *urlstr, const char *args );
-    static nsresult OpenBrowserWindow( const char *args, PRBool newWindow = PR_TRUE );
-    static nsresult ReParent( nsISupports *window, HWND newParent );
-    static nsresult GetStartupURL(nsICmdLineService *args, nsCString& taskURL);
+    static nsresult OpenBrowserWindow();
 
     static int   mConversations;
     enum {
@@ -594,7 +590,7 @@ struct MessageWindow {
         if ( !mClassName ) { 
             sprintf( classNameBuffer,
                          "%s%s",
-                         gAppData->appName,
+                         gAppData->name,
                          "MessageWindow" );
             mClassName = classNameBuffer;
         }
@@ -656,42 +652,48 @@ struct MessageWindow {
         return retval;
     }
 
-    // SendRequest: Pass string via WM_COPYDATA to message window.
-    NS_IMETHOD SendRequest( const char *cmd ) {
-        /* Nothing like WM_COPYDATA in OS/2, where the OS allows pointers to be
-         * passed to a different process and automatically accessible by that
-         * process.  So we have to create shared mem on our side and then the
-         * process that gets the WM_COPYDATA message has to do a 
-         * DosGetSharedMem on this pointer to be able to access the data
-         */
+    // SendRequest: Construct a data buffer <commandline>\0<workingdir>\0,
+    // then pass the string via WM_COPYDATA to the message window.
+
+    NS_IMETHOD SendRequest( const char *cmd )
+    {
+    /* Nothing like WM_COPYDATA in OS/2, where the OS allows pointers to be
+     * passed to a different process and automatically accessible by that
+     * process.  So we have to create shared mem on our side and then the
+     * process that gets the WM_COPYDATA message has to do a 
+     * DosGetSharedMem on this pointer to be able to access the data
+    */
 
         COPYDATASTRUCT *pcds;
-        APIRET rc = NO_ERROR;
         PVOID pvData = NULL;
-        ULONG ulSize = sizeof(COPYDATASTRUCT)+strlen(cmd)+1;
-        rc = DosAllocSharedMem( &pvData, NULL, ulSize,
-                                (PAG_COMMIT|PAG_READ|PAG_WRITE|OBJ_GETTABLE) );
 
-        if( rc != NO_ERROR )
-        {
-           /* don't even try doing anything else.  Windows doesn't worry about
-            * errors so I guess that we shouldn't either
-            */
-           return NS_OK;
-        }
+        if (!cmd)
+            return NS_ERROR_FAILURE;
 
-        memset( pvData, '\0', ulSize );
+        ULONG ulSize = sizeof(COPYDATASTRUCT)+strlen(cmd)+1+CCHMAXPATH;
+        if (DosAllocSharedMem( &pvData, NULL, ulSize,
+                               (PAG_COMMIT|PAG_READ|PAG_WRITE|OBJ_GETTABLE)))
+            return NS_ERROR_OUT_OF_MEMORY;
+
+        // We used to set dwData to zero, when we didn't send the
+        // working dir.  Now we're using it as a version number.
+        // The commandline goes in the buffer space immediately after
+        // the COPYDATASTRUCT;  the current directory follows that.
         pcds = (COPYDATASTRUCT *)(pvData);
-        pcds->dwData = 0;
-        pcds->cbData = strlen(cmd)+1;
-        /* put the data in the buffer space immediately after the 
-         * COPYDATASTRUCT
-         */
-        pcds->lpData = &(pcds->chBuff);
-        if( cmd )
-        {
-           strcpy( (char *)pcds->lpData, cmd );
+        pcds->dwData = 1;
+
+        char * ptr = &(pcds->chBuff);
+        pcds->lpData = ptr;
+        strcpy( ptr, cmd);
+        pcds->cbData = strlen( ptr) + 1;
+        ptr += pcds->cbData;
+
+        if (DosQueryPathInfo( ".", FIL_QUERYFULLNAME, ptr, CCHMAXPATH)) {
+            ptr[0] = '.';
+            ptr[1] = '\0';
         }
+        pcds->cbData += strlen( ptr) + 1;
+
         WinSendMsg( mHandle, WM_COPYDATA, 0, (MPARAM)pcds );
         DosFreeMem( pvData );
 
@@ -699,31 +701,36 @@ struct MessageWindow {
     }
 
     // Window proc.
-    static MRESULT EXPENTRY WindowProc( HWND msgWindow, ULONG msg, MPARAM wp, 
-                                        MPARAM lp )
+    static
+    MRESULT EXPENTRY WindowProc( HWND msgWindow, ULONG msg, MPARAM wp, MPARAM lp )
     {
-        MRESULT rc = (MRESULT)TRUE;
+        // We have to return a FALSE from WM_CREATE
+        if ( msg == WM_CREATE )
+            return (MRESULT)FALSE;
 
-        if ( msg == WM_COPYDATA ) {
-            // This is an incoming request.
-            COPYDATASTRUCT *cds = (COPYDATASTRUCT*)lp;
-            DosGetSharedMem( (PVOID)cds, PAG_READ|PAG_WRITE );
-#if MOZ_DEBUG_DDE
-            printf( "Incoming request: %s\n", (const char*)cds->lpData );
-#endif
-            (void)nsNativeAppSupportOS2::HandleRequest( (LPBYTE)cds->lpData );
- }
+        if ( msg != WM_COPYDATA ) 
+            return (MRESULT)TRUE;
 
-    /* We have to return a FALSE from WM_CREATE or this window will never
-     * get off of the ground
-     */
-    else if ( msg == WM_CREATE ) {
-        rc = (MRESULT)FALSE;
+        // This is an incoming request.
+        COPYDATASTRUCT *cds = (COPYDATASTRUCT*)lp;
+        DosGetSharedMem( (PVOID)cds, PAG_READ|PAG_WRITE );
+
+        nsCOMPtr<nsILocalFile> workingDir;
+
+        // a "1" or greater indicates that the other process's working
+        // directory follows the commandline string - locate & convert it
+        if (cds->dwData >= 1) {
+            char* wdpath = strchr( (char*)cds->lpData, 0) + 1;
+            NS_NewNativeLocalFile(nsDependentCString(wdpath),
+                                  PR_FALSE, getter_AddRefs(workingDir));
+        }
+
+        nsNativeAppSupportOS2::HandleCommandLine((char*)cds->lpData,
+                                workingDir, nsICommandLine::STATE_REMOTE_AUTO);
+
+        return (MRESULT)TRUE;
     }
 
-    return rc;
-}
-
 private:
     HWND mHandle;
     USHORT   mMsgWindowAtom;
@@ -748,26 +755,39 @@ nsNativeAppSupportOS2::Start( PRBool *aResult ) {
     NS_ENSURE_ARG( aResult );
     NS_ENSURE_TRUE( mInstance == 0, NS_ERROR_NOT_INITIALIZED );
 
-    if (getenv("MOZ_NO_REMOTE"))
-    {
-        *aResult = PR_TRUE;
-        return NS_OK;
-    }
-
     nsresult rv = NS_ERROR_FAILURE;
     *aResult = PR_FALSE;
 
-    for ( int i = 1; i < gArgc; i++ ) {
-        if ( strcmp( "-dde", gArgv[i] ) == 0 ||
-             strcmp( "/dde", gArgv[i] ) == 0 ) {
+    // see if DDE should be enabled AND remove OS/2-specific
+    // options the app's commandline handler won't recognize
+    // (-console was handled earlier by StartOS2App())
+    for (int i = 1; i < gArgc; i++) {
+        if (stricmp("-dde", gArgv[i]) == 0 ||
+            stricmp("/dde", gArgv[i]) == 0)
             mUseDDE = PR_TRUE;
-        }
+        else
+            if (stricmp("-console", gArgv[i]) != 0 &&
+                stricmp("/console", gArgv[i]) != 0)
+                continue;
+
+        for (int j = i; j < gArgc; j++)
+            gArgv[j] = gArgv[j+1];
+        gArgc--;
+        i--;
+    }
+
+    // if this is a standalone instance, turn off DDE regardless of the
+    // commandline, then skip out before we look for another instance
+    if (getenv("MOZ_NO_REMOTE")) {
+        mUseDDE = PR_FALSE;
+        *aResult = PR_TRUE;
+        return NS_OK;
     }
 
     // Grab mutex first.
 
     // Build mutex name from app name.
-    PR_snprintf( mMutexName, sizeof mMutexName, "%s%s", gAppData->appName, MOZ_STARTUP_MUTEX_NAME );
+    PR_snprintf( mMutexName, sizeof mMutexName, "%s%s", gAppData->name, MOZ_STARTUP_MUTEX_NAME );
     Mutex startupLock = Mutex( mMutexName );
 
     NS_ENSURE_TRUE( startupLock.Lock( MOZ_DDE_START_TIMEOUT ), NS_ERROR_FAILURE );
@@ -781,14 +801,7 @@ nsNativeAppSupportOS2::Start( PRBool *aResult ) {
     HAB hab;
     HMQ hmqCurrent = WinQueryQueueInfo( HMQ_CURRENT, &mqinfo, 
                                         sizeof( MQINFO ) );
-    if( !hmqCurrent )
-    {
-        /* Set our app to be a PM app before attempting Win calls */
-        PPIB ppib;
-        PTIB ptib;
-        DosGetInfoBlocks(&ptib, &ppib);
-        ppib->pib_ultype = 3;
-
+    if( !hmqCurrent ) {
         hab = WinInitialize( 0 );
         hmqCurrent = WinCreateMsgQueue( hab, 0 );
     }
@@ -803,8 +816,9 @@ nsNativeAppSupportOS2::Start( PRBool *aResult ) {
         // We will be server.
         rv = msgWindow.Create();
         if ( NS_SUCCEEDED( rv ) ) {
-            // Start up DDE server.
-            this->StartDDE();
+            // Start up DDE server
+            if (mUseDDE)
+                this->StartDDE();
             // Tell caller to spin message loop.
             *aResult = PR_TRUE;
         }
@@ -878,7 +892,7 @@ nsNativeAppSupportOS2::StartDDE() {
                     NS_ERROR_FAILURE );
 
     // Allocate DDE strings.
-    NS_ENSURE_TRUE( ( mApplication = WinDdeCreateStringHandle( (char*) gAppData->appName, CP_WINANSI ) ) && InitTopicStrings(),
+    NS_ENSURE_TRUE( ( mApplication = WinDdeCreateStringHandle( (char*) gAppData->name, CP_WINANSI ) ) && InitTopicStrings(),
                     NS_ERROR_FAILURE );
 
     // Next step is to register a DDE service.
@@ -1053,6 +1067,7 @@ static void escapeQuotes( nsAString &aString ) {
     return;
 }
 
+
 HDDEDATA APIENTRY
 nsNativeAppSupportOS2::HandleDDENotification( ULONG idInst,     // DDEML instance
                                               USHORT uType,     // transaction type
@@ -1096,28 +1111,27 @@ nsNativeAppSupportOS2::HandleDDENotification( ULONG idInst,     // DDEML instanc
                 case topicOpenURL: {
                     // Open a given URL...
 
-                    // Default is to open in current window.
-                    PRBool new_window = PR_FALSE;
-
                     // Get the URL from the first argument in the command.
                     nsCAutoString url;
                     ParseDDEArg(hsz2, 0, url);
+
                     // Read the 3rd argument in the command to determine if a
                     // new window is to be used.
                     nsCAutoString windowID;
                     ParseDDEArg(hsz2, 2, windowID);
-                    // "0" means to open the URL in a new window.
-                    if ( windowID.Equals( "0" ) ) {
-                        new_window = PR_TRUE;
-                    }
+                    // to open the URL in a new window, the old OS/2 code
+                    // looks for "0" while the new Win32 code looks for "";
+                    // not knowing which is correct, we'll look for both
+                    if (windowID.Equals( "0" ) || windowID.Equals( "" ))
+                        url.Insert("mozilla -new-window ", 0);
+                    else
+                        url.Insert("mozilla -url ", 0);
 
-                    // Make it look like command line args.
-                    url.Insert( "mozilla -url ", 0 );
 #if MOZ_DEBUG_DDE
                     printf( "Handling dde XTYP_REQUEST request: [%s]...\n", url.get() );
 #endif
                     // Now handle it.
-                    HandleRequest( LPBYTE( url.get() ), new_window );
+                    HandleCommandLine(url.get(), nsnull, nsICommandLine::STATE_REMOTE_EXPLICIT);
                     // Return pseudo window ID.
                     result = CreateDDEData( 1 );
                     break;
@@ -1148,7 +1162,7 @@ nsNativeAppSupportOS2::HandleDDENotification( ULONG idInst,     // DDEML instanc
                             break;
                         }
                         // Convert that to internal interface.
-                        nsCOMPtr<nsIDOMWindowInternal> internalContent( do_QueryInterface( content ) );
+                        nsCOMPtr<nsPIDOMWindow> internalContent( do_QueryInterface( content ) );
                         if ( !internalContent ) {
                             break;
                         }
@@ -1166,14 +1180,11 @@ nsNativeAppSupportOS2::HandleDDENotification( ULONG idInst,     // DDEML instanc
                         // Escape any double-quotes.
                         escapeQuotes( url );
 
-                        // Now for the title; first, get the "window" script global object.
-                        nsCOMPtr<nsIScriptGlobalObject> scrGlobalObj( do_QueryInterface( internalContent ) );
-                        if ( !scrGlobalObj ) {
-                            break;
-                        }
-                        // Then from its doc shell get the base window...
+                        // Now for the title...
+
+                        // Get the base window from the doc shell...
                         nsCOMPtr<nsIBaseWindow> baseWindow =
-                          do_QueryInterface( scrGlobalObj->GetDocShell() );
+                          do_QueryInterface( internalContent->GetDocShell() );
                         if ( !baseWindow ) {
                             break;
                         }
@@ -1191,12 +1202,12 @@ nsNativeAppSupportOS2::HandleDDENotification( ULONG idInst,     // DDEML instanc
                         nsCAutoString   outpt( NS_LITERAL_CSTRING("\"") );
                         // Now copy the URL converting the Unicode string
                         // to a single-byte ASCII string
-                        outpt.Append( NS_LossyConvertUCS2toASCII( url ) );
+                        outpt.Append( NS_LossyConvertUTF16toASCII( url ) );
                         // Add the "," used to separate the URL and the page
                         // title
                         outpt.Append( NS_LITERAL_CSTRING("\",\"") );
                         // Now copy the current page title to the return string
-                        outpt.Append( NS_LossyConvertUCS2toASCII( title.get() ));
+                        outpt.Append( NS_LossyConvertUTF16toASCII( title.get() ));
                         // Fill out the return string with the remainin ",""
                         outpt.Append( NS_LITERAL_CSTRING( "\",\"\"" ));
 
@@ -1265,9 +1276,6 @@ nsNativeAppSupportOS2::HandleDDENotification( ULONG idInst,     // DDEML instanc
 #if MOZ_DEBUG_DDE
             printf( "Handling dde request: [%s]...\n", (char*)request );
 #endif
-            // Default is to open in current window.
-            PRBool new_window = PR_FALSE;
-
             nsCAutoString url;
             ParseDDEArg((const char*) request, 0, url);
 
@@ -1276,18 +1284,19 @@ nsNativeAppSupportOS2::HandleDDENotification( ULONG idInst,     // DDEML instanc
             nsCAutoString windowID;
             ParseDDEArg((const char*) request, 2, windowID);
 
-            // "0" means to open the URL in a new window.
-            if ( windowID.Equals( "0" ) ) {
-                new_window = PR_TRUE;
-            }
+            // to open the URL in a new window, the old OS/2 code
+            // looks for "0" while the new Win32 code looks for "";
+            // not knowing which is correct, we'll look for both
+            if (windowID.Equals( "0" ) || windowID.Equals( "" ))
+                url.Insert("mozilla -new-window ", 0);
+            else
+                url.Insert("mozilla -url ", 0);
 
-            // Make it look like command line args.
-            url.Insert( "mozilla -url ", 0 );
 #if MOZ_DEBUG_DDE
             printf( "Handling dde XTYP_REQUEST request: [%s]...\n", url.get() );
 #endif
             // Now handle it.
-            HandleRequest( LPBYTE( url.get() ), new_window );
+            HandleCommandLine(url.get(), nsnull, nsICommandLine::STATE_REMOTE_EXPLICIT);
 
             // Release the data.
 //            DdeUnaccessData( hdata );
@@ -1303,6 +1312,7 @@ nsNativeAppSupportOS2::HandleDDENotification( ULONG idInst,     // DDEML instanc
     return result;
 }
 
+
 // Utility function to advance to end of quoted string.
 // p+offset must point to the comma preceding the arg on entry.
 // On return, p+result points to the closing '"' (or end of the string
@@ -1382,13 +1392,12 @@ void nsNativeAppSupportOS2::ParseDDEArg( HSZ args, int index, nsCString& aString
 void nsNativeAppSupportOS2::ActivateLastWindow() {
     nsCOMPtr<nsIDOMWindowInternal> navWin;
     GetMostRecentWindow( NS_LITERAL_STRING("navigator:browser").get(), getter_AddRefs( navWin ) );
-    if ( navWin ) {
+    if ( navWin )
         // Activate that window.
         activateWindow( navWin );
-    } else {
+    else
         // Need to create a Navigator window, then.
-        OpenBrowserWindow( "about:blank" );
-    }
+        OpenBrowserWindow();
 }
 
 HDDEDATA nsNativeAppSupportOS2::CreateDDEData( DWORD value ) {
@@ -1405,129 +1414,35 @@ HDDEDATA nsNativeAppSupportOS2::CreateDDEData( LPBYTE value, DWORD len ) {
     return result;
 }
 
-// Handle DDE request.  The argument is the command line received by the
-// DDE client process.  We convert that string to an nsICmdLineService
-// object via GetCmdLineArgs.  Then, we look for certain well-known cmd
-// arguments.  This replicates code elsewhere, to some extent,
-// unfortunately (if you can fix that, please do).
+
 void
-nsNativeAppSupportOS2::HandleRequest( LPBYTE request, PRBool newWindow ) {
-
-    // Parse command line.
-
-    nsCOMPtr<nsICmdLineService> args;
+nsNativeAppSupportOS2::HandleCommandLine(const char* aCmdLineString,
+                                         nsIFile* aWorkingDir,
+                                         PRUint32 aState)
+{
     nsresult rv;
 
-    rv = GetCmdLineArgs( request, getter_AddRefs( args ) );
-    if (NS_FAILED(rv)) return;
-
-    // first see if there is a url
-    nsXPIDLCString arg;
-    rv = args->GetURLToLoad(getter_Copies(arg));
-    if (NS_SUCCEEDED(rv) && (const char*)arg ) {
-      // Launch browser.
-#if MOZ_DEBUG_DDE
-      printf( "Launching browser on url [%s]...\n", (const char*)arg );
-#endif
-      (void)OpenBrowserWindow( arg, newWindow );
-      return;
-    }
-
-
-    // ok, let's try the -chrome argument
-    rv = args->GetCmdLineValue("-chrome", getter_Copies(arg));
-    if (NS_SUCCEEDED(rv) && (const char*)arg ) {
-      // Launch chrome.
-#if MOZ_DEBUG_DDE
-      printf( "Launching chrome url [%s]...\n", (const char*)arg );
-#endif
-      (void)OpenWindow( arg, "" );
-      return;
-    }
-
-    // Try standard startup's command-line handling logic from nsAppRunner.cpp...
-
-    // This will tell us whether the command line processing opened a window.
-    PRBool windowOpened = PR_FALSE;
-
-    // If there are no command line arguments, then we want to open windows
-    // based on startup prefs (which say to open navigator and/or mailnews
-    // and/or composer), or, open just a Navigator window.  We do the former
-    // if there are no open windows (i.e., we're in turbo mode), the latter
-    // if there are open windows.  Note that we call DoCommandLines in the
-    // case where there are no command line args but there are windows open
-    // (i.e., with heedStartupPrefs==PR_FALSE) despite the fact that it may
-    // not actually do anything in that case.  That way we're covered if the
-    // logic in DoCommandLines changes.  Note that we cover this case below
-    // by opening a navigator window if DoCommandLines doesn't open one.  We
-    // have to cover that case anyway, because DoCommandLines won't open a
-    // window when given "mozilla -foobar" or the like.
-    PRBool heedStartupPrefs = PR_FALSE;
-    PRInt32 argc = 0;
-    args->GetArgc( &argc );
-    if ( argc <= 1 ) {
-        // Use startup prefs iff there are no windows currently open.
-        nsCOMPtr<nsIDOMWindowInternal> win;
-        GetMostRecentWindow( 0, getter_AddRefs( win ) );
-        if ( !win ) {
-            heedStartupPrefs = PR_TRUE;
-        }
-    }
-
-    // Process command line options.
-    rv = DoCommandLines( args, heedStartupPrefs, &windowOpened );
-
-    // If a window was opened, then we're done.
-    // Note that we keep on trying in the unlikely event of an error.
-    if (rv == NS_ERROR_NOT_AVAILABLE || rv == NS_ERROR_ABORT || windowOpened) {
-      return;
-    }
-
-    // ok, no idea what the param is.
-#if MOZ_DEBUG_DDE
-    printf( "Unknown request [%s]\n", (char*) request );
-#endif
-    // if all else fails, open a browser window
-    const char * const contractID =
-      "@mozilla.org/commandlinehandler/general-startup;1?type=browser";
-    nsCOMPtr<nsICmdLineHandler> handler = do_GetService(contractID, &rv);
-    if (NS_FAILED(rv)) return;
-
-    nsXPIDLString defaultArgs;
-    rv = handler->GetDefaultArgs(getter_Copies(defaultArgs));
-    if (NS_FAILED(rv) || !defaultArgs) return;
-
-    if (defaultArgs) {
-      nsCAutoString url;
-      url.AssignWithConversion( defaultArgs );
-      OpenBrowserWindow(url.get());
-    } else {
-      OpenBrowserWindow("about:blank");
-    }
-}
-
-// Parse command line args according to MS spec
-// (see "Parsing C++ Command-Line Arguments" at
-// http://msdn.microsoft.com/library/devprods/vs6/visualc/vclang/_pluslang_parsing_c.2b2b_.command.2d.line_arguments.htm).
-nsresult
-nsNativeAppSupportOS2::GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResult ) {
-    nsresult rv = NS_OK;
-
     int justCounting = 1;
     char **argv = 0;
     // Flags, etc.
     int init = 1;
     int between, quoted, bSlashCount;
     int argc;
-    char *p;
+    const char *p;
     nsCAutoString arg;
-    nsDependentCString mailtoUrlScheme (kMailtoUrlScheme);
+
+    nsCOMPtr<nsICommandLineRunner> cmdLine
+        (do_CreateInstance("@mozilla.org/toolkit/command-line;1"));
+    if (!cmdLine) {
+        NS_ERROR("Couldn't create command line!");
+        return;
+    }
 
     // We loop if we've not finished the second pass through.
     while ( 1 ) {
         // Initialize if required.
         if ( init ) {
-            p = (char*)request;
+            p = aCmdLineString;
             between = 1;
             argc = quoted = bSlashCount = 0;
 
@@ -1560,8 +1475,12 @@ nsNativeAppSupportOS2::GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResu
         } else {
             // We are processing the contents of an argument.
             // Check for whitespace or end.
-            // if the argument we are parsing is a mailto url then all of the remaining command line data
-            // needs to be part of the mailto url even if it has spaces. See Bug #231032
+
+            // if the argument we are parsing is a mailto url then all
+            // of the remaining command line data needs to be part of
+            // the mailto url even if it has spaces. See Bug #231032
+            nsDependentCString mailtoUrlScheme (kMailtoUrlScheme);
+
             if ( *p == 0 || ( !quoted && isspace( *p ) && !StringBeginsWith(arg, mailtoUrlScheme, nsCaseInsensitiveCStringComparator()) ) ) {
                 // Process pending backslashes (interpret them
                 // literally since they're not followed by a ").
@@ -1645,18 +1564,13 @@ nsNativeAppSupportOS2::GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResu
         }
     }
 
-    // OK, now create nsICmdLineService object from argc/argv.
-    nsCOMPtr<nsIComponentManager> compMgr;
-    NS_GetComponentManager(getter_AddRefs(compMgr));
-    rv = compMgr->CreateInstanceByContractID("@mozilla.org/app-startup/commandLineService;1",
-                                             0, NS_GET_IID( nsICmdLineService ),
-                                             (void**) aResult);
+    rv = cmdLine->Init(argc, argv, aWorkingDir, aState);
 
-    if ( NS_FAILED( rv ) || NS_FAILED( ( rv = (*aResult)->Initialize( argc, argv ) ) ) ) {
-#if MOZ_DEBUG_DDE
-        printf( "Error creating command line service = 0x%08X (argc=%d, argv=0x%08X)\n", (int)rv, (int)argc, (void*)argv );
-#endif
-    }
+    // if these OS/2-specific flags aren't removed,
+    // any URL following them will be ignored
+    PRBool found;
+    cmdLine->HandleFlag(NS_LITERAL_STRING("console"), PR_FALSE, &found);
+    cmdLine->HandleFlag(NS_LITERAL_STRING("dde"), PR_FALSE, &found);
 
     // Cleanup.
     while ( argc ) {
@@ -1664,9 +1578,15 @@ nsNativeAppSupportOS2::GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResu
     }
     delete [] argv;
 
-    return rv;
+    if (NS_FAILED(rv)) {
+        NS_ERROR("Error initializing command line.");
+        return;
+    }
+
+    cmdLine->Run();
 }
 
+
 nsresult
 nsNativeAppSupportOS2::OpenWindow( const char*urlstr, const char *args ) {
 
@@ -1690,13 +1610,13 @@ nsNativeAppSupportOS2::OpenWindow( const char*urlstr, const char *args ) {
 }
 
 HWND hwndForDOMWindow( nsISupports *window ) {
-    nsCOMPtr<nsIScriptGlobalObject> ppScriptGlobalObj( do_QueryInterface(window) );
-    if ( !ppScriptGlobalObj ) {
+    nsCOMPtr<nsPIDOMWindow> pidomwindow( do_QueryInterface(window) );
+    if ( !pidomwindow ) {
         return 0;
     }
 
     nsCOMPtr<nsIBaseWindow> ppBaseWindow =
-        do_QueryInterface( ppScriptGlobalObj->GetDocShell() );
+        do_QueryInterface( pidomwindow->GetDocShell() );
     if ( !ppBaseWindow ) {
         return 0;
     }
@@ -1736,31 +1656,41 @@ SafeJSContext::~SafeJSContext() {
 }
 
 nsresult SafeJSContext::Push() {
-  nsresult rv;
-
   if (mContext) // only once
     return NS_ERROR_FAILURE;
 
   mService = do_GetService(sJSStackContractID);
   if(mService) {
-    rv = mService->GetSafeJSContext(&mContext);
-    if (NS_SUCCEEDED(rv) && mContext) {
-      rv = mService->Push(mContext);
-      if (NS_FAILED(rv))
-        mContext = 0;
+    JSContext *cx;
+    if (NS_SUCCEEDED(mService->GetSafeJSContext(&cx)) &&
+        cx &&
+        NS_SUCCEEDED(mService->Push(cx))) {
+      // Save cx in mContext to indicate need to pop.
+      mContext = cx;
     }
   }
   return mContext ? NS_OK : NS_ERROR_FAILURE;
 }
 
 
+// As of Jan, 2005, most of the code in this method is pointless and
+// will never be used.  It is only called by ActivateLastWindow() and
+// only when there is no existing window.  Consequently, the first test
+// below [if ( !navWin )] will always fail, bypassing everything in the
+// dummy do..while() loop.  Given that this file is largely a port of
+// the Win32 version, it is left in place to maintain concurrency.
+
 nsresult
-nsNativeAppSupportOS2::OpenBrowserWindow( const char *args, PRBool newWindow ) {
+nsNativeAppSupportOS2::OpenBrowserWindow()
+{
     nsresult rv = NS_OK;
+
     // Open the argument URL in the most recently used Navigator window.
     // If there is no Nav window, open a new one.
 
-    // Get most recently used Nav window.
+    // If at all possible, hand the request off to the most recent
+    // browser window.
+
     nsCOMPtr<nsIDOMWindowInternal> navWin;
     GetMostRecentWindow( NS_LITERAL_STRING( "navigator:browser" ).get(), getter_AddRefs( navWin ) );
 
@@ -1768,52 +1698,136 @@ nsNativeAppSupportOS2::OpenBrowserWindow( const char *args, PRBool newWindow ) {
     // out to the OpenWindow call when things go awry.
     do {
         // If caller requires a new window, then don't use an existing one.
-        if ( newWindow ) {
-            break;
-        }
         if ( !navWin ) {
             // Have to open a new one.
             break;
         }
-        // Get content window.
-        nsCOMPtr<nsIDOMWindow> content;
-        navWin->GetContent( getter_AddRefs( content ) );
-        if ( !content ) {
-            break;
+
+        nsCOMPtr<nsIBrowserDOMWindow> bwin;
+        { // scope a bunch of temporary cruft used to generate bwin
+          nsCOMPtr<nsIWebNavigation> navNav( do_GetInterface( navWin ) );
+          nsCOMPtr<nsIDocShellTreeItem> navItem( do_QueryInterface( navNav ) );
+          if ( navItem ) {
+            nsCOMPtr<nsIDocShellTreeItem> rootItem;
+            navItem->GetRootTreeItem( getter_AddRefs( rootItem ) );
+            nsCOMPtr<nsIDOMWindow> rootWin( do_GetInterface( rootItem ) );
+            nsCOMPtr<nsIDOMChromeWindow> chromeWin(do_QueryInterface(rootWin));
+            if ( chromeWin )
+              chromeWin->GetBrowserDOMWindow( getter_AddRefs ( bwin ) );
+          }
         }
-        // Convert that to internal interface.
-        nsCOMPtr<nsIDOMWindowInternal> internalContent( do_QueryInterface( content ) );
-        if ( !internalContent ) {
-            break;
+        if ( bwin ) {
+          nsCOMPtr<nsIURI> uri;
+          NS_NewURI( getter_AddRefs( uri ), NS_LITERAL_CSTRING("about:blank"), 0, 0 );
+          if ( uri ) {
+            nsCOMPtr<nsIDOMWindow> container;
+            rv = bwin->OpenURI( uri, 0,
+                                nsIBrowserDOMWindow::OPEN_DEFAULTWINDOW,
+                                nsIBrowserDOMWindow::OPEN_EXTERNAL,
+                                getter_AddRefs( container ) );
+            if ( NS_SUCCEEDED( rv ) )
+              return NS_OK;
+          }
         }
-        // Get location.
-        nsCOMPtr<nsIDOMLocation> location;
-        internalContent->GetLocation( getter_AddRefs( location ) );
-        if ( !location ) {
-            break;
-        }
-        // Set up environment.
-        SafeJSContext context;
-        if ( NS_FAILED( context.Push() ) ) {
-            break;
-        }
-        // Set href.
-        nsAutoString url; url.AssignWithConversion( args );
-        if ( NS_FAILED( location->SetHref( url ) ) ) {
-            break;
-        }
-        // Finally, if we get here, we're done.
-        return NS_OK;
+
+        NS_ERROR("failed to hand off external URL to extant window\n");
     } while ( PR_FALSE );
 
-    nsCOMPtr<nsICmdLineHandler> handler(do_GetService("@mozilla.org/commandlinehandler/general-startup;1?type=browser", &rv));
-    if (NS_FAILED(rv)) return rv;
+    // open a new window if caller requested it or if anything above failed
 
-    nsXPIDLCString chromeUrlForTask;
-    rv = handler->GetChromeUrlForTask(getter_Copies(chromeUrlForTask));
-    if (NS_FAILED(rv)) return rv;
+    char* argv[] = { 0 };
+    nsCOMPtr<nsICommandLineRunner> cmdLine
+        (do_CreateInstance("@mozilla.org/toolkit/command-line;1"));
+    NS_ENSURE_TRUE(cmdLine, NS_ERROR_FAILURE);
 
-    // Last resort is to open a brand new window.
-    return OpenWindow( chromeUrlForTask, args );
+    rv = cmdLine->Init(0, argv, nsnull, nsICommandLine::STATE_REMOTE_EXPLICIT);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    return cmdLine->Run();
+}
+
+// This is a public function called by nsAppRunner.cpp.  Its primary
+// purpose is to determine if any commandline options require a VIO
+// ("console") window.  If so and one isn't present, it will restart
+// the app in a VIO session.  It is intended to be called as early as
+// possible during startup and before any other commandline processing.
+// It returns TRUE if the current instance should continue running and
+// FALSE if it should terminate upon this function's return.
+
+PRBool     StartOS2App(int aArgc, char **aArgv)
+{
+  PRBool    rv = PR_TRUE;
+  PPIB      ppib;
+  PTIB      ptib;
+
+  DosGetInfoBlocks(&ptib, &ppib);
+
+  // if this isn't a PM session, reset the session type to enable use
+  // of PM functions;  if it is PM, see if a VIO session is required
+  if (ppib->pib_ultype != SSF_TYPE_PM)
+    ppib->pib_ultype = SSF_TYPE_PM;
+  else {
+    for (int i = 1; i < aArgc; i++ ) {
+      char *arg = aArgv[i];
+      if (*arg != '-' && *arg != '/')
+        continue;
+      arg++;
+      if (stricmp("?", arg) == 0 ||
+        stricmp("h", arg) == 0 ||
+        stricmp("v", arg) == 0 ||
+        stricmp("help", arg) == 0 ||
+        stricmp("version", arg) == 0 ||
+        stricmp("console", arg) == 0) {
+        rv = PR_FALSE;
+        break;
+      }
+    }
+  }
+
+  // if the session type is OK, increase the number of 
+  // file handles available to the app, then exit
+  if (rv) {
+    ULONG    ulMaxFH = 0;
+    LONG     ulReqCount = 0;
+
+    DosSetRelMaxFH(&ulReqCount, &ulMaxFH);
+    if (ulMaxFH < 256)
+      DosSetMaxFH(256);
+
+    return rv;
+  }
+
+  // the app has to be restarted in a VIO session
+  char        szErrObj[64] = "";
+  STARTDATA   x;
+
+  memset(&x, 0, sizeof(x));
+  x.Length = sizeof(x);
+  (const char* const)(x.PgmTitle) = gAppData->name;
+  x.InheritOpt = SSF_INHERTOPT_PARENT;
+  x.SessionType = SSF_TYPE_WINDOWABLEVIO;
+  x.PgmControl = SSF_CONTROL_NOAUTOCLOSE;
+  x.ObjectBuffer = szErrObj;
+  x.ObjectBuffLen = sizeof(szErrObj);
+
+  // the f/q exename is the string preceding ppib->pib_pchcmd;
+  // the original commandline is the string following it
+  char * ptr = ppib->pib_pchcmd - 2;
+  while (*ptr)
+    ptr--;
+  x.PgmName = ptr + 1;
+  x.PgmInputs = strchr(ppib->pib_pchcmd, 0) + 1;
+
+  // restart the app;  if this session is in the background, trying
+  // to start in the foreground will produce an error, but the app
+  // will still start; if DosStartSession has a real failure, forget
+  // the console and let the current instance keep running
+  ULONG ulSession;
+  PID   pid;
+  ULONG rc = DosStartSession(&x, &ulSession, &pid);
+  if (rc && rc != ERROR_SMG_START_IN_BACKGROUND)
+    rv = PR_TRUE;
+
+  return rv;
 }
 
