@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -28,11 +28,11 @@
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -41,20 +41,20 @@
 #include "nsAppRunner.h"
 #include "nsXULAppAPI.h"
 #include "nsString.h"
-#include "nsICmdLineService.h"
+#include "nsIBrowserDOMWindow.h"
+#include "nsICommandLineRunner.h"
 #include "nsCOMPtr.h"
 #include "nsXPIDLString.h"
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
-#include "nsICmdLineHandler.h"
-#include "nsIDOMWindow.h"
+#include "nsIDOMChromeWindow.h"
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsISupportsArray.h"
 #include "nsIWindowWatcher.h"
-#include "nsIDOMWindowInternal.h"
-#include "nsIScriptGlobalObject.h"
+#include "nsPIDOMWindow.h"
 #include "nsIDocShell.h"
+#include "nsIDocShellTreeItem.h"
 #include "nsIBaseWindow.h"
 #include "nsIWidget.h"
 #include "nsIAppShellService.h"
@@ -62,20 +62,16 @@
 #include "nsIXULWindow.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
-#include "nsIPref.h"
 #include "nsIPromptService.h"
 #include "nsNetCID.h"
+#include "nsNetUtil.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
-#include "nsXPCOM.h"
-#ifdef MOZ_PHOENIX
-#include "nsIShellService.h"
-#endif
-
-// These are needed to load a URL in a browser window.
 #include "nsIDOMLocation.h"
 #include "nsIJSContextStack.h"
+#include "nsIWebNavigation.h"
 #include "nsIWindowMediator.h"
+#include "nsNativeCharsetUtils.h"
 
 #include <windows.h>
 #include <shellapi.h>
@@ -83,14 +79,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <io.h>
+#include <direct.h>
 #include <fcntl.h>
 
-#define kMailtoUrlScheme "mailto:"
-
-#ifdef MOZ_THUNDERBIRD
-#define MAPI_STARTUP_ARG       "/MAPIStartUp"
-#endif
-
 static HWND hwndForDOMWindow( nsISupports * );
 
 static
@@ -265,6 +256,47 @@ private:
  * whether Mozilla is already running.
  */
 
+/* Update 2006 September
+ *
+ * A change in behavior was implemented in July 2004 which made the
+ * application add on launch and remove on quit the ddexec registry key.
+ * See bug 246078.
+ * Windows Vista has changed the methods used to set an application as default
+ * and the new methods are incompatible with removing the ddeexec registry key.
+ * See bug 353089.
+ *
+ * OS DDE Sequence:
+ * 1. OS checks if the dde name is registered.
+ * 2. If it is registered the OS sends a DDE request with the WWW_OpenURL topic
+ *    and the params as specified in the default value of the ddeexec registry
+ *    key for the verb (e.g. open).
+ * 3. If it isn't registered the OS launches the executable defined in the
+ *    verb's (e.g. open) command registry key.
+ * 4. If the ifexec registry key is not present the OS sends a DDE request with
+ *    the WWW_OpenURL topic and the params as specified in the default value of
+ *    the ddeexec registry key for the verb (e.g. open).
+ * 5. If the ifexec registry key is present the OS sends a DDE request with the
+ *    WWW_OpenURL topic and the params as specified in the ifexec registry key
+ *    for the verb (e.g. open).
+ *
+ * Application DDE Sequence:
+ * 1. If the application is running a DDE request is received DDE with the
+ *    WWW_OpenURL topic and the params as specified in the default value of the
+ *    ddeexec registry key (e.g. "%1",,0,0,,,, where '%1' is the url to open)
+ *    for the verb (e.g. open).
+ * 2. If the application is not running it is launched with the -url argument.
+ *
+ * Note: Typically the application should use the DDE request to open the page
+ *       instead of using a command line argument. The command line argument is
+ *       used to work around when the DDE queue is hung by a different
+ *       application. See bug 53952.
+ *
+ * 3. The application receives a DDE request with the WWW_OpenURL topic and the
+ *    params from the ifexec key without the url (e.g. ,,0,0,,,,). The
+ *    application treats this request as a noop internally and returns the
+ *    correct values for the DDE request to the OS.
+ */
+
 class nsNativeAppSupportWin : public nsNativeAppSupportBase,
                               public nsIObserver
 {
@@ -295,7 +327,7 @@ private:
                                                     HDDEDATA hdata,
                                                     ULONG    dwData1,
                                                     ULONG    dwData2 );
-    static void HandleRequest( LPBYTE request, PRBool newWindow = PR_TRUE );
+    static void HandleCommandLine(const char* aCmdLineString, nsIFile* aWorkingDir, PRUint32 aState);
     static void ParseDDEArg( HSZ args, int index, nsCString& string);
     static void ParseDDEArg( const char* args, int index, nsCString& aString);
     static void ActivateLastWindow();
@@ -303,11 +335,9 @@ private:
     static HDDEDATA CreateDDEData( LPBYTE value, DWORD len );
     static PRBool   InitTopicStrings();
     static int      FindTopic( HSZ topic );
-    static nsresult GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResult );
     static nsresult OpenWindow( const char *urlstr, const char *args );
-    static nsresult OpenBrowserWindow( const char *args, PRBool newWindow = PR_TRUE );
+    static nsresult OpenBrowserWindow();
     static nsresult ReParent( nsISupports *window, HWND newParent );
-    static nsresult GetStartupURL(nsICmdLineService *args, nsCString& taskURL);
     static void     SetupSysTrayIcon();
     static void     RemoveSysTrayIcon();
 
@@ -327,7 +357,6 @@ private:
     static HSZ   mApplication, mTopics[ topicCount ];
     static DWORD mInstance;
     static PRBool mCanHandleRequests;
-    static PRBool mSupportingDDEExec;
     static char mMutexName[];
     friend struct MessageWindow;
 }; // nsNativeAppSupportWin
@@ -341,10 +370,10 @@ NS_IMPL_RELEASE_INHERITED(nsNativeAppSupportWin, nsNativeAppSupportBase)
 
 void
 nsNativeAppSupportWin::CheckConsole() {
-    for ( int i = 1; i < __argc; i++ ) {
-        if ( strcmp( "-console", __argv[i] ) == 0
+    for ( int i = 1; i < gArgc; i++ ) {
+        if ( strcmp( "-console", gArgv[i] ) == 0
              ||
-             strcmp( "/console", __argv[i] ) == 0 ) {
+             strcmp( "/console", gArgv[i] ) == 0 ) {
             // Users wants to make sure we have a console.
             // Try to allocate one.
             BOOL rc = ::AllocConsole();
@@ -359,7 +388,9 @@ nsNativeAppSupportWin::CheckConsole() {
                     FILE *hf = ::_fdopen( hCrt, "w" );
                     if ( hf ) {
                         *stdout = *hf;
+#ifdef DEBUG
                         ::fprintf( stdout, "stdout directed to dynamic console\n" );
+#endif
                     }
                 }
 
@@ -370,7 +401,9 @@ nsNativeAppSupportWin::CheckConsole() {
                     FILE *hf = ::_fdopen( hCrt, "w" );
                     if ( hf ) {
                         *stderr = *hf;
+#ifdef DEBUG
                         ::fprintf( stderr, "stderr directed to dynamic console\n" );
+#endif
                     }
                 }
 
@@ -435,7 +468,6 @@ HSZ   nsNativeAppSupportWin::mApplication   = 0;
 HSZ   nsNativeAppSupportWin::mTopics[nsNativeAppSupportWin::topicCount] = { 0 };
 DWORD nsNativeAppSupportWin::mInstance      = 0;
 PRBool nsNativeAppSupportWin::mCanHandleRequests   = PR_FALSE;
-PRBool nsNativeAppSupportWin::mSupportingDDEExec   = PR_FALSE;
 
 char nsNativeAppSupportWin::mMutexName[ 128 ] = { 0 };
 
@@ -461,7 +493,7 @@ struct MessageWindow {
             ::_snprintf( classNameBuffer,
                          sizeof classNameBuffer,
                          "%s%s",
-                         gAppData->appName,
+                         gAppData->name,
                          "MessageWindow" );
             mClassName = classNameBuffer;
         }
@@ -523,11 +555,31 @@ struct MessageWindow {
 
     // SendRequest: Pass string via WM_COPYDATA to message window.
     NS_IMETHOD SendRequest( const char *cmd ) {
-        COPYDATASTRUCT cds = { 0, ::strlen( cmd ) + 1, (void*)cmd };
+        // Construct a data buffer <commandline>\0<workingdir>\0
+        int cmdlen = strlen(cmd);
+        char* cmdbuf = (char*) malloc(cmdlen + MAX_PATH + 1);
+        if (!cmdbuf)
+            return NS_ERROR_OUT_OF_MEMORY;
+
+        strcpy(cmdbuf, cmd);
+        _getcwd(cmdbuf + cmdlen + 1, MAX_PATH);
+
+        // We used to set dwData to zero, when we didn't send the working dir.
+        // Now we're using it as a version number.
+        COPYDATASTRUCT cds = {
+            1,
+            cmdlen + strlen(cmdbuf + cmdlen + 1) + 2,
+            (void*) cmdbuf
+        };
         HWND newWin = (HWND)::SendMessage( mHandle, WM_COPYDATA, 0, (LPARAM)&cds );
         if ( newWin ) {
+            // Restore the window if it is minimized.
+            if ( ::IsIconic( newWin ) ) {
+                ::ShowWindow( newWin, SW_RESTORE );
+            }
             ::SetForegroundWindow( newWin );
         }
+        free (cmdbuf);
         return NS_OK;
     }
 
@@ -542,39 +594,49 @@ struct MessageWindow {
 #if MOZ_DEBUG_DDE
             printf( "Incoming request: %s\n", (const char*)cds->lpData );
 #endif
-            (void)nsNativeAppSupportWin::HandleRequest( (LPBYTE)cds->lpData );
+            nsCOMPtr<nsILocalFile> workingDir;
+
+            if (1 >= cds->dwData) {
+                char* wdpath = (char*) cds->lpData;
+                // skip the command line, and get the working dir of the
+                // other process, which is after the first null char
+                while (*wdpath)
+                    ++wdpath;
+
+                ++wdpath;
+
+#ifdef MOZ_DEBUG_DDE
+                printf( "Working dir: %s\n", wdpath);
+#endif
+
+                NS_NewNativeLocalFile(nsDependentCString(wdpath),
+                                      PR_FALSE,
+                                      getter_AddRefs(workingDir));
+            }
+            (void)nsNativeAppSupportWin::HandleCommandLine((char*)cds->lpData, workingDir, nsICommandLine::STATE_REMOTE_AUTO);
 
             // Get current window and return its window handle.
             nsCOMPtr<nsIDOMWindowInternal> win;
             GetMostRecentWindow( 0, getter_AddRefs( win ) );
             return win ? (long)hwndForDOMWindow( win ) : 0;
-  } else if ( msg == WM_QUERYENDSESSION ) {
-    if (!nsNativeAppSupportWin::mCanHandleRequests)
-        return 0;
-    // Invoke "-killAll" cmd line handler.  That will close all open windows,
-    // and display dialog asking whether to save/don't save/cancel.  If the
-    // user says cancel, then we pass that indicator along to the system
-    // in order to stop the system shutdown/logoff.
-    nsCOMPtr<nsICmdLineHandler>
-        killAll( do_CreateInstance( "@mozilla.org/commandlinehandler/general-startup;1?type=killAll" ) );
-    if ( killAll ) {
-        nsXPIDLCString unused;
-        // Note: "GetChromeUrlForTask" is a euphemism for
-        //       "ProcessYourCommandLineSwitch".  The interface was written
-        //       presuming a less general-purpose role for command line
-        //       handlers than it ought to have.
-        nsresult rv = killAll->GetChromeUrlForTask( getter_Copies( unused ) );
-        if ( rv == NS_ERROR_ABORT ) {
-            // User cancelled shutdown/logoff.
-            return FALSE;
-        } else {
-            // Shutdown/logoff OK.
-            return TRUE;
+        } else if ( msg == WM_QUERYENDSESSION ) {
+            if (!nsNativeAppSupportWin::mCanHandleRequests)
+                return 0;
+            // Invoke "-killAll" cmd line handler.  That will close all open windows,
+            // and display dialog asking whether to save/don't save/cancel.  If the
+            // user says cancel, then we pass that indicator along to the system
+            // in order to stop the system shutdown/logoff.
+            nsCOMPtr<nsICommandLineRunner> cmdLine
+                (do_CreateInstance("@mozilla.org/toolkit/command-line;1"));
+            char* argv[] = { "-killAll", 0 };
+            if (cmdLine &&
+                NS_SUCCEEDED(cmdLine->Init(1, argv, nsnull,
+                                           nsICommandLine::STATE_REMOTE_AUTO))) {
+                return cmdLine->Run() != NS_ERROR_ABORT;
+            }
         }
+        return DefWindowProc( msgWindow, msg, wp, lp );
     }
-  }
-  return DefWindowProc( msgWindow, msg, wp, lp );
-}
 
 private:
     HWND mHandle;
@@ -610,7 +672,7 @@ nsNativeAppSupportWin::Start( PRBool *aResult ) {
     // Grab mutex first.
 
     // Build mutex name from app name.
-    ::_snprintf( mMutexName, sizeof mMutexName, "%s%s", gAppData->appName, MOZ_STARTUP_MUTEX_NAME );
+    ::_snprintf( mMutexName, sizeof mMutexName, "%s%s", gAppData->name, MOZ_STARTUP_MUTEX_NAME );
     Mutex startupLock = Mutex( mMutexName );
 
     NS_ENSURE_TRUE( startupLock.Lock( MOZ_DDE_START_TIMEOUT ), NS_ERROR_FAILURE );
@@ -657,63 +719,6 @@ nsNativeAppSupportWin::FindTopic( HSZ topic ) {
     return -1;
 }
 
-// Utility function that determines if we're handling http Internet shortcuts.
-static PRBool isDefaultBrowser() 
-{
-#ifdef MOZ_PHOENIX
-  nsCOMPtr<nsIShellService> shell(do_GetService("@mozilla.org/browser/shell-service;1"));
-  PRBool isDefault;
-  shell->IsDefaultBrowser(PR_FALSE, &isDefault);
-  return isDefault;
-#else
-  return FALSE;
-#endif
-}
-
-// Utility function to delete a registry subkey.
-static DWORD deleteKey( HKEY baseKey, const char *keyName ) {
-    // Make sure input subkey isn't null.
-    DWORD rc;
-    if ( keyName && ::strlen(keyName) ) {
-        // Open subkey.
-        HKEY key;
-        rc = ::RegOpenKeyEx( baseKey,
-                             keyName,
-                             0,
-                             KEY_ENUMERATE_SUB_KEYS | DELETE,
-                             &key );
-        // Continue till we get an error or are done.
-        while ( rc == ERROR_SUCCESS ) {
-            char subkeyName[_MAX_PATH];
-            DWORD len = sizeof subkeyName;
-            // Get first subkey name.  Note that we always get the
-            // first one, then delete it.  So we need to get
-            // the first one next time, also.
-            rc = ::RegEnumKeyEx( key,
-                                 0,
-                                 subkeyName,
-                                 &len,
-                                 0,
-                                 0,
-                                 0,
-                                 0 );
-            if ( rc == ERROR_NO_MORE_ITEMS ) {
-                // No more subkeys.  Delete the main one.
-                rc = ::RegDeleteKey( baseKey, keyName );
-                break;
-            } else if ( rc == ERROR_SUCCESS ) {
-                // Another subkey, delete it, recursively.
-                rc = deleteKey( key, subkeyName );
-            }
-        }
-        // Close the key we opened.
-        ::RegCloseKey( key );
-    } else {
-        rc = ERROR_BADKEY;
-    }
-    return rc;
-}
-
 
 // Start DDE server.
 //
@@ -737,7 +742,7 @@ nsNativeAppSupportWin::StartDDE() {
                     NS_ERROR_FAILURE );
 
     // Allocate DDE strings.
-    NS_ENSURE_TRUE( ( mApplication = DdeCreateStringHandle( mInstance, (char*) gAppData->appName, CP_WINANSI ) ) && InitTopicStrings(),
+    NS_ENSURE_TRUE( ( mApplication = DdeCreateStringHandle( mInstance, (char*) gAppData->name, CP_WINANSI ) ) && InitTopicStrings(),
                     NS_ERROR_FAILURE );
 
     // Next step is to register a DDE service.
@@ -810,15 +815,6 @@ nsNativeAppSupportWin::Quit() {
     mw.Destroy();
 
     if ( mInstance ) {
-        // Undo registry setting if we need to.
-        if ( mSupportingDDEExec && isDefaultBrowser() ) {
-            mSupportingDDEExec = PR_FALSE;
-#if MOZ_DEBUG_DDE
-            printf( "Deleting ddexec subkey on exit\n" );
-#endif
-            deleteKey( HKEY_CLASSES_ROOT, "http\\shell\\open\\ddeexec" );
-        }
-
         // Unregister application name.
         DdeNameService( mInstance, mApplication, 0, DNS_UNREGISTER );
         // Clean up strings.
@@ -973,28 +969,30 @@ nsNativeAppSupportWin::HandleDDENotification( UINT uType,       // transaction t
                 case topicOpenURL: {
                     // Open a given URL...
 
-                    // Default is to open in current window.
-                    PRBool new_window = PR_FALSE;
-
                     // Get the URL from the first argument in the command.
                     nsCAutoString url;
                     ParseDDEArg(hsz2, 0, url);
-                    // Read the 3rd argument in the command to determine if a
-                    // new window is to be used.
-                    nsCAutoString windowID;
-                    ParseDDEArg(hsz2, 2, windowID);
-                    // "0" means to open the URL in a new window.
-                    if ( windowID.Equals( "0" ) ) {
-                        new_window = PR_TRUE;
+
+                    if ( !url.Equals( "" ) ) {
+                        // Read the 3rd argument in the command to determine if a
+                        // new window is to be used.
+                        nsCAutoString windowID;
+                        ParseDDEArg(hsz2, 2, windowID);
+                        // "" means to open the URL in a new window.
+                        if ( windowID.Equals( "" ) ) {
+                            url.Insert("mozilla -new-window ", 0);
+                        }
+                        else {
+                            url.Insert("mozilla -url ", 0);
+                        }
+
+#if MOZ_DEBUG_DDE
+                        printf( "Handling dde XTYP_REQUEST request: [%s]...\n", url.get() );
+#endif
+                        // Now handle it.
+                        HandleCommandLine(url.get(), nsnull, nsICommandLine::STATE_REMOTE_EXPLICIT);
                     }
 
-                    // Make it look like command line args.
-                    url.Insert( "mozilla -url ", 0 );
-#if MOZ_DEBUG_DDE
-                    printf( "Handling dde XTYP_REQUEST request: [%s]...\n", url.get() );
-#endif
-                    // Now handle it.
-                    HandleRequest( LPBYTE( url.get() ), new_window );
                     // Return pseudo window ID.
                     result = CreateDDEData( 1 );
                     break;
@@ -1025,7 +1023,7 @@ nsNativeAppSupportWin::HandleDDENotification( UINT uType,       // transaction t
                             break;
                         }
                         // Convert that to internal interface.
-                        nsCOMPtr<nsIDOMWindowInternal> internalContent( do_QueryInterface( content ) );
+                        nsCOMPtr<nsPIDOMWindow> internalContent( do_QueryInterface( content ) );
                         if ( !internalContent ) {
                             break;
                         }
@@ -1043,14 +1041,11 @@ nsNativeAppSupportWin::HandleDDENotification( UINT uType,       // transaction t
                         // Escape any double-quotes.
                         escapeQuotes( url );
 
-                        // Now for the title; first, get the "window" script global object.
-                        nsCOMPtr<nsIScriptGlobalObject> scrGlobalObj( do_QueryInterface( internalContent ) );
-                        if ( !scrGlobalObj ) {
-                            break;
-                        }
-                        // Then from its doc shell get the base window...
+                        // Now for the title...
+
+                        // Get the base window from the doc shell...
                         nsCOMPtr<nsIBaseWindow> baseWindow =
-                            do_QueryInterface( scrGlobalObj->GetDocShell() );
+                          do_QueryInterface( internalContent->GetDocShell() );
                         if ( !baseWindow ) {
                             break;
                         }
@@ -1068,12 +1063,15 @@ nsNativeAppSupportWin::HandleDDENotification( UINT uType,       // transaction t
                         nsCAutoString   outpt( NS_LITERAL_CSTRING("\"") );
                         // Now copy the URL converting the Unicode string
                         // to a single-byte ASCII string
-                        outpt.Append( NS_LossyConvertUCS2toASCII( url ) );
+                        nsCAutoString tmpNativeStr;
+                        NS_CopyUnicodeToNative( url, tmpNativeStr );
+                        outpt.Append( tmpNativeStr );
                         // Add the "," used to separate the URL and the page
                         // title
                         outpt.Append( NS_LITERAL_CSTRING("\",\"") );
                         // Now copy the current page title to the return string
-                        outpt.Append( NS_LossyConvertUCS2toASCII( title.get() ));
+                        NS_CopyUnicodeToNative( title, tmpNativeStr );
+                        outpt.Append( tmpNativeStr );
                         // Fill out the return string with the remainin ",""
                         outpt.Append( NS_LITERAL_CSTRING( "\",\"\"" ));
 
@@ -1148,23 +1146,25 @@ nsNativeAppSupportWin::HandleDDENotification( UINT uType,       // transaction t
             nsCAutoString url;
             ParseDDEArg((const char*) request, 0, url);
 
-            // Read the 3rd argument in the command to determine if a
-            // new window is to be used.
-            nsCAutoString windowID;
-            ParseDDEArg((const char*) request, 2, windowID);
+            if ( !url.Equals( "" ) ) {
+                // Read the 3rd argument in the command to determine if a
+                // new window is to be used.
+                nsCAutoString windowID;
+                ParseDDEArg((const char*) request, 2, windowID);
 
-            // "0" means to open the URL in a new window.
-            if ( windowID.Equals( "0" ) ) {
-                new_window = PR_TRUE;
-            }
-
-            // Make it look like command line args.
-            url.Insert( "mozilla -url ", 0 );
+                // "" means to open the URL in a new window.
+                if ( windowID.Equals( "" ) ) {
+                    url.Insert("mozilla -new-window ", 0);
+                }
+                else {
+                    url.Insert("mozilla -url ", 0);
+                }
 #if MOZ_DEBUG_DDE
-            printf( "Handling dde XTYP_REQUEST request: [%s]...\n", url.get() );
+                printf( "Handling dde XTYP_REQUEST request: [%s]...\n", url.get() );
 #endif
-            // Now handle it.
-            HandleRequest( LPBYTE( url.get() ), new_window );
+                // Now handle it.
+                HandleCommandLine(url.get(), nsnull, nsICommandLine::STATE_REMOTE_EXPLICIT);
+            }
 
             // Release the data.
             DdeUnaccessData( hdata );
@@ -1264,7 +1264,7 @@ void nsNativeAppSupportWin::ActivateLastWindow() {
         activateWindow( navWin );
     } else {
         // Need to create a Navigator window, then.
-        OpenBrowserWindow( "about:blank" );
+        OpenBrowserWindow();
     }
 }
 
@@ -1283,138 +1283,37 @@ HDDEDATA nsNativeAppSupportWin::CreateDDEData( LPBYTE value, DWORD len ) {
     return result;
 }
 
-// Handle DDE request.  The argument is the command line received by the
-// DDE client process.  We convert that string to an nsICmdLineService
-// object via GetCmdLineArgs.  Then, we look for certain well-known cmd
-// arguments.  This replicates code elsewhere, to some extent,
-// unfortunately (if you can fix that, please do).
 void
-nsNativeAppSupportWin::HandleRequest( LPBYTE request, PRBool newWindow ) {
-
-    // Parse command line.
-
-    nsCOMPtr<nsICmdLineService> args;
+nsNativeAppSupportWin::HandleCommandLine(const char* aCmdLineString,
+                                         nsIFile* aWorkingDir,
+                                         PRUint32 aState)
+{
     nsresult rv;
 
-    rv = GetCmdLineArgs( request, getter_AddRefs( args ) );
-    if (NS_FAILED(rv)) return;
-
-    // first see if there is a url
-    nsXPIDLCString arg;
-    rv = args->GetURLToLoad(getter_Copies(arg));
-    if (NS_SUCCEEDED(rv) && (const char*)arg ) {
-      // Launch browser.
-#if MOZ_DEBUG_DDE
-      printf( "Launching browser on url [%s]...\n", (const char*)arg );
-#endif
-      (void)OpenBrowserWindow( arg, newWindow );
-      return;
-    }
-
-
-    // ok, let's try the -chrome argument
-    rv = args->GetCmdLineValue("-chrome", getter_Copies(arg));
-    if (NS_SUCCEEDED(rv) && (const char*)arg ) {
-      // Launch chrome.
-#if MOZ_DEBUG_DDE
-      printf( "Launching chrome url [%s]...\n", (const char*)arg );
-#endif
-      (void)OpenWindow( arg, "" );
-      return;
-    }
-
-#ifdef MOZ_THUNDERBIRD
-    // check wheather it is a MAPI request.  If yes, don't open any new
-    // windows and just return.
-    rv = args->GetCmdLineValue(MAPI_STARTUP_ARG, getter_Copies(arg));
-    if (NS_SUCCEEDED(rv) && (const char*)arg) {
-      return;
-    }
-#endif
-
-    // Try standard startup's command-line handling logic from nsAppRunner.cpp...
-
-    // This will tell us whether the command line processing opened a window.
-    PRBool windowOpened = PR_FALSE;
-
-    // If there are no command line arguments, then we want to open windows
-    // based on startup prefs (which say to open navigator and/or mailnews
-    // and/or composer), or, open just a Navigator window.  We do the former
-    // if there are no open windows (i.e., we're in turbo mode), the latter
-    // if there are open windows.  Note that we call DoCommandLines in the
-    // case where there are no command line args but there are windows open
-    // (i.e., with heedStartupPrefs==PR_FALSE) despite the fact that it may
-    // not actually do anything in that case.  That way we're covered if the
-    // logic in DoCommandLines changes.  Note that we cover this case below
-    // by opening a navigator window if DoCommandLines doesn't open one.  We
-    // have to cover that case anyway, because DoCommandLines won't open a
-    // window when given "mozilla -foobar" or the like.
-    PRBool heedStartupPrefs = PR_FALSE;
-    PRInt32 argc = 0;
-    args->GetArgc( &argc );
-    if ( argc <= 1 ) {
-        // Use startup prefs iff there are no windows currently open.
-        nsCOMPtr<nsIDOMWindowInternal> win;
-        GetMostRecentWindow( 0, getter_AddRefs( win ) );
-        if ( !win ) {
-            heedStartupPrefs = PR_TRUE;
-        }
-    }
-
-    // Process command line options.
-    rv = DoCommandLines( args, heedStartupPrefs, &windowOpened );
-
-    // If a window was opened, then we're done.
-    // Note that we keep on trying in the unlikely event of an error.
-    if (rv == NS_ERROR_NOT_AVAILABLE || rv == NS_ERROR_ABORT || windowOpened) {
-      return;
-    }
-
-    // ok, no idea what the param is.
-#if MOZ_DEBUG_DDE
-    printf( "Unknown request [%s]\n", (char*) request );
-#endif
-    // if all else fails, open a browser window
-    const char * const contractID =
-      "@mozilla.org/commandlinehandler/general-startup;1?type=browser";
-    nsCOMPtr<nsICmdLineHandler> handler = do_GetService(contractID, &rv);
-    if (NS_FAILED(rv)) return;
-
-    nsXPIDLString defaultArgs;
-    rv = handler->GetDefaultArgs(getter_Copies(defaultArgs));
-    if (NS_FAILED(rv) || !defaultArgs) return;
-
-    if (defaultArgs) {
-      nsCAutoString url;
-      url.AssignWithConversion( defaultArgs );
-      OpenBrowserWindow(url.get());
-    } else {
-      OpenBrowserWindow("about:blank");
-    }
-}
-
-// Parse command line args according to MS spec
-// (see "Parsing C++ Command-Line Arguments" at
-// http://msdn.microsoft.com/library/devprods/vs6/visualc/vclang/_pluslang_parsing_c.2b2b_.command.2d.line_arguments.htm).
-nsresult
-nsNativeAppSupportWin::GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResult ) {
-    nsresult rv = NS_OK;
-
     int justCounting = 1;
     char **argv = 0;
     // Flags, etc.
     int init = 1;
     int between, quoted, bSlashCount;
     int argc;
-    char *p;
+    const char *p;
     nsCAutoString arg;
-    nsDependentCString mailtoUrlScheme (kMailtoUrlScheme);
 
+    nsCOMPtr<nsICommandLineRunner> cmdLine
+        (do_CreateInstance("@mozilla.org/toolkit/command-line;1"));
+    if (!cmdLine) {
+        NS_ERROR("Couldn't create command line!");
+        return;
+    }
+
+    // Parse command line args according to MS spec
+    // (see "Parsing C++ Command-Line Arguments" at
+    // http://msdn.microsoft.com/library/devprods/vs6/visualc/vclang/_pluslang_parsing_c.2b2b_.command.2d.line_arguments.htm).
     // We loop if we've not finished the second pass through.
     while ( 1 ) {
         // Initialize if required.
         if ( init ) {
-            p = (char*)request;
+            p = aCmdLineString;
             between = 1;
             argc = quoted = bSlashCount = 0;
 
@@ -1447,9 +1346,7 @@ nsNativeAppSupportWin::GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResu
         } else {
             // We are processing the contents of an argument.
             // Check for whitespace or end.
-            // if the argument we are parsing is a mailto url then all of the remaining command line data
-            // needs to be part of the mailto url even if it has spaces. See Bug #231032
-            if ( *p == 0 || ( !quoted && isspace( *p ) && !StringBeginsWith(arg, mailtoUrlScheme, nsCaseInsensitiveCStringComparator()) ) ) {
+            if ( *p == 0 || ( !quoted && isspace( *p ) ) ) {
                 // Process pending backslashes (interpret them
                 // literally since they're not followed by a ").
                 while( bSlashCount ) {
@@ -1532,18 +1429,7 @@ nsNativeAppSupportWin::GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResu
         }
     }
 
-    // OK, now create nsICmdLineService object from argc/argv.
-    nsCOMPtr<nsIComponentManager> compMgr;
-    NS_GetComponentManager(getter_AddRefs(compMgr));
-    rv = compMgr->CreateInstanceByContractID("@mozilla.org/app-startup/commandLineService;1",
-                                             0, NS_GET_IID( nsICmdLineService ),
-                                             (void**) aResult);
-
-    if ( NS_FAILED( rv ) || NS_FAILED( ( rv = (*aResult)->Initialize( argc, argv ) ) ) ) {
-#if MOZ_DEBUG_DDE
-        printf( "Error creating command line service = 0x%08X (argc=%d, argv=0x%08X)\n", (int)rv, (int)argc, (void*)argv );
-#endif
-    }
+    rv = cmdLine->Init(argc, argv, aWorkingDir, aState);
 
     // Cleanup.
     while ( argc ) {
@@ -1551,7 +1437,12 @@ nsNativeAppSupportWin::GetCmdLineArgs( LPBYTE request, nsICmdLineService **aResu
     }
     delete [] argv;
 
-    return rv;
+    if (NS_FAILED(rv)) {
+        NS_ERROR("Error initializing command line.");
+        return;
+    }
+
+    cmdLine->Run();
 }
 
 nsresult
@@ -1577,13 +1468,13 @@ nsNativeAppSupportWin::OpenWindow( const char*urlstr, const char *args ) {
 }
 
 HWND hwndForDOMWindow( nsISupports *window ) {
-    nsCOMPtr<nsIScriptGlobalObject> ppScriptGlobalObj( do_QueryInterface(window) );
-    if ( !ppScriptGlobalObj ) {
+    nsCOMPtr<nsPIDOMWindow> pidomwindow( do_QueryInterface(window) );
+    if ( !pidomwindow ) {
         return 0;
     }
 
     nsCOMPtr<nsIBaseWindow> ppBaseWindow =
-        do_QueryInterface( ppScriptGlobalObj->GetDocShell() );
+        do_QueryInterface( pidomwindow->GetDocShell() );
     if ( !ppBaseWindow ) {
         return 0;
     }
@@ -1623,18 +1514,17 @@ SafeJSContext::~SafeJSContext() {
 }
 
 nsresult SafeJSContext::Push() {
-  nsresult rv;
-
   if (mContext) // only once
     return NS_ERROR_FAILURE;
 
   mService = do_GetService(sJSStackContractID);
   if(mService) {
-    rv = mService->GetSafeJSContext(&mContext);
-    if (NS_SUCCEEDED(rv) && mContext) {
-      rv = mService->Push(mContext);
-      if (NS_FAILED(rv))
-        mContext = 0;
+    JSContext *cx;
+    if (NS_SUCCEEDED(mService->GetSafeJSContext(&cx)) &&
+        cx &&
+        NS_SUCCEEDED(mService->Push(cx))) {
+      // Save cx in mContext to indicate need to pop.
+      mContext = cx;
     }
   }
   return mContext ? NS_OK : NS_ERROR_FAILURE;
@@ -1642,12 +1532,16 @@ nsresult SafeJSContext::Push() {
 
 
 nsresult
-nsNativeAppSupportWin::OpenBrowserWindow( const char *args, PRBool newWindow ) {
+nsNativeAppSupportWin::OpenBrowserWindow()
+{
     nsresult rv = NS_OK;
+
     // Open the argument URL in the most recently used Navigator window.
     // If there is no Nav window, open a new one.
 
-    // Get most recently used Nav window.
+    // If at all possible, hand the request off to the most recent
+    // browser window.
+
     nsCOMPtr<nsIDOMWindowInternal> navWin;
     GetMostRecentWindow( NS_LITERAL_STRING( "navigator:browser" ).get(), getter_AddRefs( navWin ) );
 
@@ -1655,52 +1549,51 @@ nsNativeAppSupportWin::OpenBrowserWindow( const char *args, PRBool newWindow ) {
     // out to the OpenWindow call when things go awry.
     do {
         // If caller requires a new window, then don't use an existing one.
-        if ( newWindow ) {
-            break;
-        }
         if ( !navWin ) {
             // Have to open a new one.
             break;
         }
-        // Get content window.
-        nsCOMPtr<nsIDOMWindow> content;
-        navWin->GetContent( getter_AddRefs( content ) );
-        if ( !content ) {
-            break;
+
+        nsCOMPtr<nsIBrowserDOMWindow> bwin;
+        { // scope a bunch of temporary cruft used to generate bwin
+          nsCOMPtr<nsIWebNavigation> navNav( do_GetInterface( navWin ) );
+          nsCOMPtr<nsIDocShellTreeItem> navItem( do_QueryInterface( navNav ) );
+          if ( navItem ) {
+            nsCOMPtr<nsIDocShellTreeItem> rootItem;
+            navItem->GetRootTreeItem( getter_AddRefs( rootItem ) );
+            nsCOMPtr<nsIDOMWindow> rootWin( do_GetInterface( rootItem ) );
+            nsCOMPtr<nsIDOMChromeWindow> chromeWin(do_QueryInterface(rootWin));
+            if ( chromeWin )
+              chromeWin->GetBrowserDOMWindow( getter_AddRefs ( bwin ) );
+          }
         }
-        // Convert that to internal interface.
-        nsCOMPtr<nsIDOMWindowInternal> internalContent( do_QueryInterface( content ) );
-        if ( !internalContent ) {
-            break;
+        if ( bwin ) {
+          nsCOMPtr<nsIURI> uri;
+          NS_NewURI( getter_AddRefs( uri ), NS_LITERAL_CSTRING("about:blank"), 0, 0 );
+          if ( uri ) {
+            nsCOMPtr<nsIDOMWindow> container;
+            rv = bwin->OpenURI( uri, 0,
+                                nsIBrowserDOMWindow::OPEN_DEFAULTWINDOW,
+                                nsIBrowserDOMWindow::OPEN_EXTERNAL,
+                                getter_AddRefs( container ) );
+            if ( NS_SUCCEEDED( rv ) )
+              return NS_OK;
+          }
         }
-        // Get location.
-        nsCOMPtr<nsIDOMLocation> location;
-        internalContent->GetLocation( getter_AddRefs( location ) );
-        if ( !location ) {
-            break;
-        }
-        // Set up environment.
-        SafeJSContext context;
-        if ( NS_FAILED( context.Push() ) ) {
-            break;
-        }
-        // Set href.
-        nsAutoString url; url.AssignWithConversion( args );
-        if ( NS_FAILED( location->SetHref( url ) ) ) {
-            break;
-        }
-        // Finally, if we get here, we're done.
-        return NS_OK;
+
+        NS_ERROR("failed to hand off external URL to extant window\n");
     } while ( PR_FALSE );
 
-    nsCOMPtr<nsICmdLineHandler> handler(do_GetService("@mozilla.org/commandlinehandler/general-startup;1?type=browser", &rv));
-    if (NS_FAILED(rv)) return rv;
+    // open a new window if caller requested it or if anything above failed
 
-    nsXPIDLCString chromeUrlForTask;
-    rv = handler->GetChromeUrlForTask(getter_Copies(chromeUrlForTask));
-    if (NS_FAILED(rv)) return rv;
+    char* argv[] = { 0 };
+    nsCOMPtr<nsICommandLineRunner> cmdLine
+        (do_CreateInstance("@mozilla.org/toolkit/command-line;1"));
+    NS_ENSURE_TRUE(cmdLine, NS_ERROR_FAILURE);
 
-    // Last resort is to open a brand new window.
-    return OpenWindow( chromeUrlForTask, args );
+    rv = cmdLine->Init(0, argv, nsnull, nsICommandLine::STATE_REMOTE_EXPLICIT);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    return cmdLine->Run();
 }
 
