@@ -46,9 +46,11 @@
 #include "prlog.h"
 #include "prtime.h"
 
+#include "nsInt64.h"
+
 #include "nsIExternalHelperAppService.h"
 #include "nsIExternalProtocolService.h"
-#include "nsIWebProgressListener.h"
+#include "nsIWebProgressListener2.h"
 #include "nsIHelperAppLauncherDialog.h"
 
 #include "nsIMIMEInfo.h"
@@ -63,20 +65,23 @@
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsILocalFile.h"
 #include "nsIChannel.h"
+#include "nsITimer.h"
 
+#ifdef MOZ_RDF
 #include "nsIRDFDataSource.h"
 #include "nsIRDFResource.h"
+#endif
 #include "nsCOMPtr.h"
 #include "nsIObserver.h"
 #include "nsCOMArray.h"
 #include "nsWeakReference.h"
 #include "nsIPrompt.h"
-#include "nsEventQueueUtils.h"
 
 class nsExternalAppHandler;
 class nsIMIMEInfo;
 class nsIRDFService;
-class nsIDownload;
+class nsITransfer;
+class nsIDOMWindowInternal;
 
 /**
  * The helper app service. Responsible for handling content that Mozilla
@@ -113,26 +118,6 @@ public:
    * this service is first instantiated.
    */
   NS_HIDDEN_(nsresult) Init();
-
-  /**
-   * Create an external app handler and binds it with a mime info object which
-   * represents how we want to dispose of this content.
-   * CreateNewExternalHandler is implemented only by the base class.
-   * @param aMIMEInfo      MIMEInfo object, representing the type of the
-   *                       content that should be handled
-   * @param aFileExtension The extension we need to append to our temp file,
-   *                       INCLUDING the ".". e.g. .mp3
-   * @param aFileName      The filename to use
-   * @param aIsAttachment  Whether the request has Content-Disposition: attachment
-   *                       set
-   * @param aWindowContext Window context, as passed to DoContent
-   */
-  NS_HIDDEN_(nsExternalAppHandler *)
-                         CreateNewExternalHandler(nsIMIMEInfo * aMIMEInfo,
-                                                  const nsCSubstring& aFileExtension,
-                                                  const nsAString& aFileName,
-                                                  PRBool aIsAttachment,
-                                                  nsIInterfaceRequestor* aWindowContext);
  
   /**
    * Given a content type, look up the user override information to see if
@@ -211,6 +196,7 @@ protected:
    * Pointer to the datasource that contains the user override information.
    * @see InitDataSource
    */
+#ifdef MOZ_RDF
   nsCOMPtr<nsIRDFDataSource> mOverRideDataSource;
 
   nsCOMPtr<nsIRDFResource> kNC_Description;
@@ -222,6 +208,7 @@ protected:
   nsCOMPtr<nsIRDFResource> kNC_AlwaysAsk;
   nsCOMPtr<nsIRDFResource> kNC_HandleInternal;
   nsCOMPtr<nsIRDFResource> kNC_PrettyName;
+#endif
 
   /**
    * Whether mOverRideDataSource is initialized
@@ -233,6 +220,7 @@ protected:
    * object for a given content type inside that data source.
    * The content type of the MIME Info will not be changed.
    */
+#ifdef MOZ_RDF
   NS_HIDDEN_(nsresult) FillTopLevelProperties(nsIRDFResource * aContentTypeNodeResource, 
                                               nsIRDFService * aRDFService,
                                               nsIMIMEInfo * aMIMEInfo);
@@ -252,6 +240,7 @@ protected:
   NS_HIDDEN_(nsresult) FillLiteralValueFromTarget(nsIRDFResource * aSource,
                                                   nsIRDFResource * aProperty,
                                                   const PRUnichar ** aLiteralValue);
+#endif
 
   /**
    * Searches the "extra" array of MIMEInfo objects for an object
@@ -301,6 +290,7 @@ protected:
 #endif
   // friend, so that it can access the nspr log module and FixFilePermissions
   friend class nsExternalAppHandler;
+  friend class nsExternalLoadRequest;
 
   /**
    * Functions related to the tempory file cleanup service provided by
@@ -318,9 +308,6 @@ protected:
   virtual NS_HIDDEN_(nsresult) LoadUriInternal(nsIURI * aURL) = 0;
   NS_HIDDEN_(PRBool) isExternalLoadOK(nsIURI* aURI, nsIPrompt* aPrompt);
   NS_HIDDEN_(PRBool) promptForScheme(nsIURI* aURI, nsIPrompt* aPrompt, PRBool *aRemember);
-
-  // friend event handler that accesses the external loading functions
-  static void *PR_CALLBACK handleExternalLoadEvent(PLEvent *event);
 };
 
 /**
@@ -340,30 +327,51 @@ protected:
  */
 class nsExternalAppHandler : public nsIStreamListener,
                              public nsIHelperAppLauncher,
-                             public nsIObserver
+                             public nsITimerCallback
 {
 public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSISTREAMLISTENER
   NS_DECL_NSIREQUESTOBSERVER
   NS_DECL_NSIHELPERAPPLAUNCHER
-  NS_DECL_NSIOBSERVER
+  NS_DECL_NSICANCELABLE
+  NS_DECL_NSITIMERCALLBACK
+
+  /**
+   * @param aMIMEInfo      MIMEInfo object, representing the type of the
+   *                       content that should be handled
+   * @param aFileExtension The extension we need to append to our temp file,
+   *                       INCLUDING the ".". e.g. .mp3
+   * @param aWindowContext Window context, as passed to DoContent
+   * @param aFileName      The filename to use
+   * @param aReason        A constant from nsIHelperAppLauncherDialog indicating
+   *                       why the request is handled by a helper app.
+   */
+  nsExternalAppHandler(nsIMIMEInfo * aMIMEInfo, const nsCSubstring& aFileExtension,
+                       nsIInterfaceRequestor * aWindowContext,
+                       const nsAString& aFilename,
+                       PRUint32 aReason);
 
-  nsExternalAppHandler();
   ~nsExternalAppHandler();
 
-  nsresult Init(nsIMIMEInfo * aMIMEInfo, const nsCSubstring& aFileExtension,
-                nsIInterfaceRequestor * aWindowContext,
-                const nsAString& aFilename,
-                PRBool aIsAttachment);
-
 protected:
   nsCOMPtr<nsIFile> mTempFile;
   nsCOMPtr<nsIURI> mSourceUrl;
   nsString mTempFileExtension;
+  /**
+   * The MIME Info for this load. Will never be null.
+   */
   nsCOMPtr<nsIMIMEInfo> mMimeInfo;
   nsCOMPtr<nsIOutputStream> mOutStream; /**< output stream to the temp file */
-  nsCOMPtr<nsIInterfaceRequestor> mWindowContext; 
+  nsCOMPtr<nsIInterfaceRequestor> mWindowContext;
+
+  /**
+   * Used to close the window on a timer, to avoid any exceptions that are
+   * thrown if we try to close the window before it's fully loaded.
+   */
+  nsCOMPtr<nsIDOMWindowInternal> mWindowToClose;
+  nsCOMPtr<nsITimer> mTimer;
+
   /**
    * The following field is set if we were processing an http channel that had
    * a content disposition header which specified the SUGGESTED file name we
@@ -377,6 +385,13 @@ protected:
    */
   PRPackedBool mCanceled;
 
+  /**
+   * This is set based on whether the channel indicates that a new window
+   * was opened specifically for this download.  If so, then we
+   * close it.
+   */
+  PRPackedBool mShouldCloseWindow;
+
   /**
    * have we received information from the user about how they want to
    * dispose of this content
@@ -384,11 +399,19 @@ protected:
   PRPackedBool mReceivedDispositionInfo;
   PRPackedBool mStopRequestIssued; 
   PRPackedBool mProgressListenerInitialized;
-  /// This is set when handling something with "Content-Disposition: attachment"
-  PRPackedBool mHandlingAttachment;
+
+  PRPackedBool mIsFileChannel;
+
+  /**
+   * One of the REASON_ constants from nsIHelperAppLauncherDialog. Indicates the
+   * reason the dialog was shown (unknown content type, server requested it,
+   * etc).
+   */
+  PRUint32 mReason;
+
   PRTime mTimeDownloadStarted;
-  PRInt32 mContentLength;
-  PRInt32 mProgress; /**< Number of bytes received (for sending progress notifications). */
+  nsInt64 mContentLength;
+  nsInt64 mProgress; /**< Number of bytes received (for sending progress notifications). */
 
   /**
    * When we are told to save the temp file to disk (in a more permament
@@ -399,6 +422,10 @@ protected:
 
   char mDataBuffer[DATA_BUFFER_SIZE];
 
+  /**
+   * Creates the temporary file for the download and an output stream for it.
+   * Upon successful return, both mTempFile and mOutStream will be valid.
+   */
   nsresult SetUpTempFile(nsIChannel * aChannel);
   /**
    * When we download a helper app, we are going to retarget all load
@@ -450,9 +477,9 @@ protected:
   PRBool GetNeverAskFlagFromPref(const char * prefName, const char * aContentType);
 
   /**
-   * Initialize an nsIDownload object for use as a progress object
+   * Initialize an nsITransfer object for use as a progress object
    */
-  nsresult InitializeDownload(nsIDownload*);
+  nsresult InitializeDownload(nsITransfer*);
   
   /**
    * Helper routine to ensure mSuggestedFileName is "correct";
@@ -466,8 +493,15 @@ protected:
    * Utility function to send proper error notification to web progress listener
    */
   void SendStatusChange(ErrorType type, nsresult aStatus, nsIRequest *aRequest, const nsAFlatString &path);
-  
-  nsCOMPtr<nsIWebProgressListener> mWebProgressListener;
+
+  /**
+   * Closes the window context if it does not have a refresh header
+   * and it never displayed content before the external helper app
+   * service was invoked.
+   */
+  nsresult MaybeCloseWindow();
+
+  nsCOMPtr<nsIWebProgressListener2> mWebProgressListener;
   nsCOMPtr<nsIChannel> mOriginalChannel; /**< in the case of a redirect, this will be the pre-redirect channel. */
   nsCOMPtr<nsIHelperAppLauncherDialog> mDialog;
 
