@@ -52,7 +52,7 @@
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
 #include "nsCOMPtr.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 
 #include <math.h>
 
@@ -166,11 +166,6 @@ NS_IMETHODIMP nsScrollPortView::SetWidget(nsIWidget *aWidget)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsScrollPortView::ComputeScrollOffsets(PRBool aAdjustWidgets)
-{
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsScrollPortView::GetContainerSize(nscoord *aWidth, nscoord *aHeight) const
 {
   if (!aWidth || !aHeight)
@@ -228,14 +223,14 @@ static nsresult ClampScrollValues(nscoord& aX, nscoord& aY, nsScrollPortView* aT
   nsView* scrolledView = aThis->GetScrolledView();
   if (!scrolledView) return NS_ERROR_FAILURE;
   
-  nsSize scrolledSize;
-  scrolledView->GetDimensions(scrolledSize);
+  nsRect scrolledRect;
+  scrolledView->GetDimensions(scrolledRect);
   
   nsSize portSize;
   aThis->GetDimensions(portSize);
   
-  nscoord maxX = scrolledSize.width - portSize.width;
-  nscoord maxY = scrolledSize.height - portSize.height;
+  nscoord maxX = scrolledRect.XMost() - portSize.width;
+  nscoord maxY = scrolledRect.YMost() - portSize.height;
   
   if (aX > maxX)
     aX = maxX;
@@ -243,11 +238,11 @@ static nsresult ClampScrollValues(nscoord& aX, nscoord& aY, nsScrollPortView* aT
   if (aY > maxY)
     aY = maxY;
   
-  if (aX < 0)
-    aX = 0;
+  if (aX < scrolledRect.x)
+    aX = scrolledRect.x;
   
-  if (aY < 0)
-    aY = 0;
+  if (aY < scrolledRect.y)
+    aY = scrolledRect.y;
   
   return NS_OK;
 }
@@ -333,13 +328,6 @@ NS_IMETHODIMP nsScrollPortView::ScrollTo(nscoord aDestinationX, nscoord aDestina
   return NS_OK;
 }
 
-NS_IMETHODIMP nsScrollPortView::GetScrollbarVisibility(PRBool *aVerticalVisible,
-                                                       PRBool *aHorizontalVisible) const
-{ 
-  NS_WARNING("Attempt to get scrollbars visibility this is not xp!");
-  return NS_OK;
-}
-
 static void AdjustChildWidgets(nsView *aView,
   nsPoint aWidgetToParentViewOrigin, float aScale, PRBool aInvalidate)
 {
@@ -354,7 +342,14 @@ static void AdjustChildWidgets(nsView *aView,
       widget->Move(NSTwipsToIntPixels(widgetOrigin.x, aScale),
                    NSTwipsToIntPixels(widgetOrigin.y, aScale));
       if (aInvalidate) {
-        widget->Invalidate(PR_FALSE);
+        // Force the widget and everything in it to repaint. We can't
+        // just use Invalidate because the widget might have child
+        // widgets and they wouldn't get updated. We can't call
+        // UpdateView(aView) because the area to be repainted might be
+        // outside aView's clipped bounds. This isn't the greatest way
+        // to achieve this, perhaps, but it works.
+        widget->Show(PR_FALSE);
+        widget->Show(PR_TRUE);
       }
     }
   } else {
@@ -431,20 +426,30 @@ NS_IMETHODIMP nsScrollPortView::ScrollByLines(PRInt32 aNumLinesX, PRInt32 aNumLi
   return ScrollTo(mOffsetX + dx, mOffsetY + dy, NS_VMREFRESH_SMOOTHSCROLL);
 }
 
-NS_IMETHODIMP nsScrollPortView::ScrollByPages(PRInt32 aNumPagesX, PRInt32 aNumPagesY)
+NS_IMETHODIMP nsScrollPortView::GetPageScrollDistances(nsSize *aDistances)
 {
   nsSize size;
   GetDimensions(size);
+
+  // The page increment is the size of the page, minus the smaller of
+  // 10% of the size or 2 lines.
+  aDistances->width  = size.width  - PR_MIN(size.width  / 10, 2 * mLineHeight);
+  aDistances->height = size.height - PR_MIN(size.height / 10, 2 * mLineHeight);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsScrollPortView::ScrollByPages(PRInt32 aNumPagesX, PRInt32 aNumPagesY)
+{
+  nsSize delta;
+  GetPageScrollDistances(&delta);
     
-  // scroll % of the window
-  nscoord dx = nscoord(float(size.width)*PAGE_SCROLL_PERCENT);
-  nscoord dy = nscoord(float(size.height)*PAGE_SCROLL_PERCENT);
-
   // put in the number of pages.
-  dx *= aNumPagesX;
-  dy *= aNumPagesY;
+  delta.width *= aNumPagesX;
+  delta.height *= aNumPagesY;
 
-  return ScrollTo(mOffsetX + dx, mOffsetY + dy, NS_VMREFRESH_SMOOTHSCROLL);
+  return ScrollTo(mOffsetX + delta.width, mOffsetY + delta.height,
+                  NS_VMREFRESH_SMOOTHSCROLL);
 }
 
 NS_IMETHODIMP nsScrollPortView::ScrollByWhole(PRBool aTop)
@@ -458,40 +463,103 @@ NS_IMETHODIMP nsScrollPortView::ScrollByWhole(PRBool aTop)
     newPos = scrolledSize.height;
   }
 
-  ScrollTo(0, newPos, 0);
+  ScrollTo(mOffsetX, newPos, 0);
 
   return NS_OK;
 }
 
-PRBool nsScrollPortView::CannotBitBlt(nsView* aScrolledView)
+NS_IMETHODIMP nsScrollPortView::ScrollByPixels(PRInt32 aNumPixelsX,
+                                               PRInt32 aNumPixelsY)
 {
-  PRUint32  scrolledViewFlags = aScrolledView->GetViewFlags();
+  nsCOMPtr<nsIDeviceContext> dev;
+  mViewManager->GetDeviceContext(*getter_AddRefs(dev));
+  float p2t = dev->DevUnitsToAppUnits(); 
 
-  return (mScrollProperties & NS_SCROLL_PROPERTY_NEVER_BLIT) ||
-         (scrolledViewFlags & NS_VIEW_FLAG_DONT_BITBLT) ||
-         (!(mScrollProperties & NS_SCROLL_PROPERTY_ALWAYS_BLIT) && 
-          !mViewManager->CanScrollWithBitBlt(aScrolledView));
+  nscoord dx = NSIntPixelsToTwips(aNumPixelsX, p2t);
+  nscoord dy = NSIntPixelsToTwips(aNumPixelsY, p2t);
+
+  return ScrollTo(mOffsetX + dx, mOffsetY + dy, 0);
 }
 
+NS_IMETHODIMP nsScrollPortView::CanScroll(PRBool aHorizontal,
+                                          PRBool aForward,
+                                          PRBool &aResult)
+{
+  nscoord offset = aHorizontal ? mOffsetX : mOffsetY;
+
+  nsView* scrolledView = GetScrolledView();
+  if (!scrolledView) {
+    aResult = PR_FALSE;
+    return NS_ERROR_FAILURE;
+  }
+
+  nsRect scrolledRect;
+  scrolledView->GetDimensions(scrolledRect);
+
+  // Can scroll to Top or to Left?
+  if (!aForward) {
+    aResult = offset > (aHorizontal ? scrolledRect.x : scrolledRect.y);
+    return NS_OK;
+  }
+
+  nsSize portSize;
+  GetDimensions(portSize);
+
+  nsCOMPtr<nsIDeviceContext> dev;
+  mViewManager->GetDeviceContext(*getter_AddRefs(dev));
+  float t2p, p2t;
+  t2p = dev->AppUnitsToDevUnits();
+  p2t = dev->DevUnitsToAppUnits();
+
+  nscoord max;
+  if (aHorizontal) {
+    max = scrolledRect.XMost() - portSize.width;
+    // Round by pixel
+    nscoord maxPx = NSTwipsToIntPixels(max, t2p);
+    max = NSIntPixelsToTwips(maxPx, p2t);
+  } else {
+    max = scrolledRect.YMost() - portSize.height;
+    // Round by pixel
+    nscoord maxPx = NSTwipsToIntPixels(max, t2p);
+    max = NSIntPixelsToTwips(maxPx, p2t);
+  }
+
+  // Can scroll to Bottom or to Right?
+  aResult = (offset < max) ? PR_TRUE : PR_FALSE;
+
+  return NS_OK;
+}
 
 void nsScrollPortView::Scroll(nsView *aScrolledView, nsPoint aTwipsDelta, nsPoint aPixDelta,
                               float aT2P)
 {
   if (aTwipsDelta.x != 0 || aTwipsDelta.y != 0)
   {
-    // Since we keep track of the dirty region as absolute coordinates,
-    // we need to offset it by the amount we scrolled.
-    if (HasNonEmptyDirtyRegion()) {
-      nsRegion* rgn = GetDirtyRegion();
-      if (rgn) {
-        rgn->MoveBy(aTwipsDelta);
+    nsIWidget *scrollWidget = GetWidget();
+    nsRegion updateRegion;
+    PRBool canBitBlit = PR_TRUE;
+    if (!scrollWidget) {
+      canBitBlit = PR_FALSE;
+    } else {
+      PRUint32 scrolledViewFlags = aScrolledView->GetViewFlags();
+
+      if ((mScrollProperties & NS_SCROLL_PROPERTY_NEVER_BLIT) ||
+          (scrolledViewFlags & NS_VIEW_FLAG_DONT_BITBLT) ||
+          (!(mScrollProperties & NS_SCROLL_PROPERTY_ALWAYS_BLIT) && 
+           !mViewManager->CanScrollWithBitBlt(aScrolledView, aTwipsDelta, &updateRegion))) {
+        canBitBlit = PR_FALSE;
       }
     }
 
-    nsIWidget *scrollWidget = GetWidget();
- 
+    if (canBitBlit) {
+      // We're going to bit-blit.  Let the viewmanager know so it can
+      // adjust dirty regions appropriately.
+      mViewManager->WillBitBlit(this, aTwipsDelta);
+    }
+    
     if (!scrollWidget)
     {
+      NS_ASSERTION(!canBitBlit, "Someone screwed up");
       nsPoint offsetToWidget;
       GetNearestWidget(&offsetToWidget);
       // We're moving the child widgets because we are scrolling. But
@@ -504,7 +572,7 @@ void nsScrollPortView::Scroll(nsView *aScrolledView, nsPoint aTwipsDelta, nsPoin
       // We should call this after fixing up the widget positions to be
       // consistent with the view hierarchy.
       mViewManager->UpdateView(this, 0);
-    } else if (CannotBitBlt(aScrolledView)) {
+    } else if (!canBitBlit) {
       // We can't blit for some reason.
       // Just update the view and adjust widgets
       // Recall that our widget's origin is at our bounds' top-left
@@ -516,60 +584,24 @@ void nsScrollPortView::Scroll(nsView *aScrolledView, nsPoint aTwipsDelta, nsPoin
       // consistent with the view hierarchy.
       mViewManager->UpdateView(this, 0);
     } else { // if we can blit and have a scrollwidget then scroll.
-      // Scroll the contents of the widget by the specfied amount, and scroll
+      // Scroll the contents of the widget by the specified amount, and scroll
       // the child widgets
       scrollWidget->Scroll(aPixDelta.x, aPixDelta.y, nsnull);
-      mViewManager->UpdateViewAfterScroll(this, aTwipsDelta.x, aTwipsDelta.y);
+      mViewManager->UpdateViewAfterScroll(this, updateRegion);
     }
   }
 }
 
-NS_IMETHODIMP nsScrollPortView::Paint(nsIRenderingContext& rc, const nsRect& rect,
-                                      PRUint32 aPaintFlags, PRBool &aResult)
-{
-  rc.PushState();
-  nsRect bounds;
-  GetDimensions(bounds);
-  bounds.x = bounds.y = 0;
-  rc.SetClipRect(bounds, nsClipCombine_kIntersect);
-
-  nsresult rv = nsView::Paint(rc, rect, aPaintFlags, aResult);
-
-  rc.PopState();
-    
-  return rv;
-}
-
-NS_IMETHODIMP nsScrollPortView::Paint(nsIRenderingContext& aRC, const nsIRegion& aRegion,
-                                      PRUint32 aPaintFlags, PRBool &aResult)
-{
-  aRC.PushState();
-  nsRect bounds;
-  GetDimensions(bounds);
-  bounds.x = bounds.y = 0;
-  aRC.SetClipRect(bounds, nsClipCombine_kIntersect);
-
-  nsresult rv = nsView::Paint(aRC, aRegion, aPaintFlags, aResult);
-
-  aRC.PopState();
-    
-  return rv;
-}
-
 NS_IMETHODIMP nsScrollPortView::ScrollToImpl(nscoord aX, nscoord aY, PRUint32 aUpdateFlags)
 {
   PRInt32           dxPx = 0, dyPx = 0;
-  
-  // convert to pixels
-  nsIDeviceContext  *dev;
-  float             t2p;
-  float             p2t;
 
-  mViewManager->GetDeviceContext(dev);
-  t2p = dev->AppUnitsToDevUnits(); 
+  // convert to pixels
+  nsCOMPtr<nsIDeviceContext> dev;
+  mViewManager->GetDeviceContext(*getter_AddRefs(dev));
+  float t2p, p2t;
+  t2p = dev->AppUnitsToDevUnits();
   p2t = dev->DevUnitsToAppUnits();
-  
-  NS_RELEASE(dev);
 
   // Update the scrolled view's position
   nsresult rv = ClampScrollValues(aX, aY, this);
