@@ -39,14 +39,12 @@
 #include "nsIWidget.h"
 #include "nsViewManager.h"
 #include "nsIWidget.h"
-#include "nsIButton.h"
 #include "nsGUIEvent.h"
 #include "nsIDeviceContext.h"
 #include "nsIComponentManager.h"
 #include "nsIRenderingContext.h"
 #include "nsTransform2D.h"
 #include "nsIScrollableView.h"
-#include "nsVoidArray.h"
 #include "nsGfxCIID.h"
 #include "nsIRegion.h"
 #include "nsIInterfaceRequestor.h"
@@ -62,7 +60,7 @@ static nsEventStatus PR_CALLBACK HandleEvent(nsGUIEvent *aEvent);
 
 // {34297A07-A8FD-d811-87C6-000244212BCB}
 #define VIEW_WRAPPER_IID \
-{ 0x34297a07, 0xa8fd, 0xd811, { 0x87, 0xc6, 0x0, 0x2, 0x44, 0x21, 0x2b, 0xcb } };
+{ 0x34297a07, 0xa8fd, 0xd811, { 0x87, 0xc6, 0x0, 0x2, 0x44, 0x21, 0x2b, 0xcb } }
 
 
 /**
@@ -71,7 +69,7 @@ static nsEventStatus PR_CALLBACK HandleEvent(nsGUIEvent *aEvent);
 class ViewWrapper : public nsIInterfaceRequestor
 {
   public:
-    NS_DEFINE_STATIC_IID_ACCESSOR(VIEW_WRAPPER_IID)
+    NS_DECLARE_STATIC_IID_ACCESSOR(VIEW_WRAPPER_IID)
     NS_DECL_ISUPPORTS
     NS_DECL_NSIINTERFACEREQUESTOR
 
@@ -82,6 +80,8 @@ class ViewWrapper : public nsIInterfaceRequestor
     nsView* mView;
 };
 
+NS_DEFINE_STATIC_IID_ACCESSOR(ViewWrapper, VIEW_WRAPPER_IID)
+
 NS_IMPL_ADDREF(ViewWrapper)
 NS_IMPL_RELEASE(ViewWrapper)
 #ifndef DEBUG
@@ -120,7 +120,15 @@ NS_IMETHODIMP ViewWrapper::QueryInterface(REFNSIID aIID, void** aInstancePtr)
 
 NS_IMETHODIMP ViewWrapper::GetInterface(REFNSIID aIID, void** aInstancePtr)
 {
-  return mView->QueryInterface(aIID, aInstancePtr);
+  if (aIID.Equals(NS_GET_IID(nsIScrollableView))) {
+    *aInstancePtr = mView->ToScrollableView();
+    return NS_OK;
+  }
+  if (aIID.Equals(NS_GET_IID(nsIView))) {
+    *aInstancePtr = mView;
+    return NS_OK;
+  }
+  return QueryInterface(aIID, aInstancePtr);
 }
 
 /**
@@ -166,8 +174,6 @@ nsEventStatus PR_CALLBACK HandleEvent(nsGUIEvent *aEvent)
   return result;
 }
 
-MOZ_DECL_CTOR_COUNTER(nsView)
-
 nsView::nsView(nsViewManager* aViewManager, nsViewVisibility aVisibility)
 {
   MOZ_COUNT_CTOR(nsView);
@@ -209,10 +215,10 @@ nsView::~nsView()
     }
   }
 
-  DropMouseGrabbing();
-  
   if (mViewManager)
   {
+    DropMouseGrabbing();
+  
     nsView *rootView = mViewManager->GetRootView();
     
     if (rootView)
@@ -281,7 +287,7 @@ nsresult nsView::QueryInterface(const nsIID& aIID, void** aInstancePtr)
   return NS_NOINTERFACE;
 }
 
-nsView* nsView::GetViewFor(nsIWidget* aWidget)
+nsIView* nsIView::GetViewFor(nsIWidget* aWidget)
 {           
   NS_PRECONDITION(nsnull != aWidget, "null widget ptr");
 
@@ -296,38 +302,6 @@ void nsIView::Destroy()
   delete this;
 }
 
-NS_IMETHODIMP nsView::Paint(nsIRenderingContext& rc, const nsRect& rect,
-                              PRUint32 aPaintFlags, PRBool &aResult)
-{
-    // Just paint, assume compositor knows what it's doing.
-    if (nsnull != mClientData) {
-      nsCOMPtr<nsIViewObserver> observer;
-      if (NS_OK == mViewManager->GetViewObserver(*getter_AddRefs(observer))) {
-        observer->Paint((nsIView *)this, rc, rect);
-      }
-    }
-        return NS_OK;
-}
-
-NS_IMETHODIMP nsView::Paint(nsIRenderingContext& rc, const nsIRegion& region,
-                              PRUint32 aPaintFlags, PRBool &aResult)
-{
-  // XXX apply region to rc
-  // XXX get bounding rect from region
-  //if (nsnull != mClientData)
-  //{
-  //  nsIViewObserver *obs;
-  //
-  //  if (NS_OK == mViewManager->GetViewObserver(obs))
-  //  {
-  //    obs->Paint((nsIView *)this, rc, rect, aPaintFlags);
-  //    NS_RELEASE(obs);
-  //  }
-  //}
-  aResult = PR_FALSE;
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 void nsView::SetPosition(nscoord aX, nscoord aY)
 {
   mDimBounds.x += aX - mPosX;
@@ -354,70 +328,15 @@ void nsView::SetPositionIgnoringChildWidgets(nscoord aX, nscoord aY)
 void nsView::ResetWidgetBounds(PRBool aRecurse, PRBool aMoveOnly,
                                PRBool aInvalidateChangedSize) {
   if (mWindow) {
-    // If our view manager has refresh disabled, then
-    // do nothing; the view manager will set our position when
-    // refresh is reenabled.
+    // If our view manager has refresh disabled, then do nothing; the view
+    // manager will set our position when refresh is reenabled.  Just let it
+    // know that it has pending updates.
     if (!mViewManager->IsRefreshEnabled()) {
+      mViewManager->PostPendingUpdate();
       return;
     }
 
-    // The geometry of a root view's widget is controlled externally,
-    // NOT by sizing or positioning the view
-    if (mViewManager->GetRootView() == this) {
-      return;
-    }
-
-    nsIDeviceContext  *dx;
-    float             t2p, p2t;
-  
-    mViewManager->GetDeviceContext(dx);
-    t2p = dx->AppUnitsToDevUnits();
-    p2t = dx->DevUnitsToAppUnits();
-    NS_RELEASE(dx);
-
-    nsPoint offset(0, 0);
-    if (GetParent()) {
-      nsIWidget* parentWidget = GetParent()->GetNearestWidget(&offset);
-
-      nsWindowType type;
-      mWindow->GetWindowType(type);
-      if (type == eWindowType_popup) {
-        // put offset into screen coordinates
-        nsRect screenRect(0,0,1,1);
-        parentWidget->WidgetToScreen(screenRect, screenRect);
-        offset += nsPoint(NSIntPixelsToTwips(screenRect.x, p2t),
-                          NSIntPixelsToTwips(screenRect.y, p2t));
-      }
-    }
-
-    nsRect newBounds(NSTwipsToIntPixels((mDimBounds.x + offset.x), t2p),
-                     NSTwipsToIntPixels((mDimBounds.y + offset.y), t2p),
-                     NSTwipsToIntPixels(mDimBounds.width, t2p),
-                     NSTwipsToIntPixels(mDimBounds.height, t2p));
-    
-    PRBool changedPos = PR_TRUE;
-    PRBool changedSize = PR_TRUE;
-    if (!(mVFlags & NS_VIEW_FLAG_HAS_POSITIONED_WIDGET)) {
-      mVFlags |= NS_VIEW_FLAG_HAS_POSITIONED_WIDGET;
-    } else {
-      nsRect curBounds;
-      mWindow->GetBounds(curBounds);
-      changedPos = curBounds.TopLeft() != newBounds.TopLeft();
-      changedSize = curBounds.Size() != newBounds.Size();
-    }
-
-    if (changedPos) {
-      if (changedSize && !aMoveOnly) {
-        mWindow->Resize(newBounds.x, newBounds.y, newBounds.width, newBounds.height,
-                        aInvalidateChangedSize);
-      } else {
-        mWindow->Move(newBounds.x, newBounds.y);
-      }
-    } else {
-      if (changedSize && !aMoveOnly) {
-        mWindow->Resize(newBounds.width, newBounds.height, aInvalidateChangedSize);
-      } // else do nothing!
-    }
+    DoResetWidgetBounds(aMoveOnly, aInvalidateChangedSize);
   } else if (aRecurse) {
     // reposition any widgets under this view
     for (nsView* v = GetFirstChild(); v; v = v->GetNextSibling()) {
@@ -426,12 +345,88 @@ void nsView::ResetWidgetBounds(PRBool aRecurse, PRBool aMoveOnly,
   }
 }
 
+void nsView::DoResetWidgetBounds(PRBool aMoveOnly,
+                                 PRBool aInvalidateChangedSize) {
+  // The geometry of a root view's widget is controlled externally,
+  // NOT by sizing or positioning the view
+  if (mViewManager->GetRootView() == this) {
+    return;
+  }
+  
+  nsRect curBounds;
+  mWindow->GetBounds(curBounds);
+  nsWindowType type;
+  mWindow->GetWindowType(type);
+
+  if (curBounds.IsEmpty() && mDimBounds.IsEmpty() && type == eWindowType_popup) {
+    // Don't manipulate empty popup widgets. For example there's no point
+    // moving hidden comboboxes around, or doing X server roundtrips
+    // to compute their true screen position. This could mean that WidgetToScreen
+    // operations on these widgets don't return up-to-date values, but popup
+    // positions aren't reliable anyway because of correction to be on or off-screen.
+    return;
+  }
+
+  NS_PRECONDITION(mWindow, "Why was this called??");
+  nsIDeviceContext  *dx;
+  float             t2p, p2t;
+  
+  mViewManager->GetDeviceContext(dx);
+  t2p = dx->AppUnitsToDevUnits();
+  p2t = dx->DevUnitsToAppUnits();
+  NS_RELEASE(dx);
+
+  nsPoint offset(0, 0);
+  if (GetParent()) {
+    nsIWidget* parentWidget = GetParent()->GetNearestWidget(&offset);
+    
+    if (type == eWindowType_popup) {
+      // put offset into screen coordinates
+      nsRect screenRect(0,0,1,1);
+      parentWidget->WidgetToScreen(screenRect, screenRect);
+      offset += nsPoint(NSIntPixelsToTwips(screenRect.x, p2t),
+                        NSIntPixelsToTwips(screenRect.y, p2t));
+    }
+  }
+
+  nsRect newBounds(NSTwipsToIntPixels((mDimBounds.x + offset.x), t2p),
+                   NSTwipsToIntPixels((mDimBounds.y + offset.y), t2p),
+                   NSTwipsToIntPixels(mDimBounds.width, t2p),
+                   NSTwipsToIntPixels(mDimBounds.height, t2p));
+    
+  PRBool changedPos = PR_TRUE;
+  PRBool changedSize = PR_TRUE;
+  if (!(mVFlags & NS_VIEW_FLAG_HAS_POSITIONED_WIDGET)) {
+    mVFlags |= NS_VIEW_FLAG_HAS_POSITIONED_WIDGET;
+  } else {
+    changedPos = curBounds.TopLeft() != newBounds.TopLeft();
+    changedSize = curBounds.Size() != newBounds.Size();
+  }
+
+  if (changedPos) {
+    if (changedSize && !aMoveOnly) {
+      mWindow->Resize(newBounds.x, newBounds.y, newBounds.width, newBounds.height,
+                      aInvalidateChangedSize);
+    } else {
+      mWindow->Move(newBounds.x, newBounds.y);
+    }
+  } else {
+    if (changedSize && !aMoveOnly) {
+      mWindow->Resize(newBounds.width, newBounds.height, aInvalidateChangedSize);
+    } // else do nothing!
+  }
+}
+
 void nsView::SetDimensions(const nsRect& aRect, PRBool aPaint, PRBool aResizeWidget)
 {
   nsRect dims = aRect;
   dims.MoveBy(mPosX, mPosY);
 
-  if (mDimBounds == dims) {
+  // Don't use nsRect's operator== here, since it returns true when
+  // both rects are empty even if they have different widths and we
+  // have cases where that sort of thing matters to us.
+  if (mDimBounds.TopLeft() == dims.TopLeft() &&
+      mDimBounds.Size() == dims.Size()) {
     return;
   }
 
@@ -456,7 +451,10 @@ NS_IMETHODIMP nsView::SetVisibility(nsViewVisibility aVisibility)
   {
 #ifndef HIDE_ALL_WIDGETS
     if (mVis == nsViewVisibility_kShow)
+    {
+      DoResetWidgetBounds(PR_FALSE, PR_TRUE);
       mWindow->Show(PR_TRUE);
+    }
     else
 #endif
       mWindow->Show(PR_FALSE);
@@ -482,6 +480,24 @@ NS_IMETHODIMP nsView::SetFloating(PRBool aFloatingView)
 	return NS_OK;
 }
 
+void nsView::InvalidateHierarchy(nsViewManager *aViewManagerParent)
+{
+  if (aViewManagerParent) {
+    // We're removed from the view hierarchy of aRemovalPoint, so make sure
+    // we're not still grabbing mouse events.
+    if (aViewManagerParent->GetMouseEventGrabber() == this) {
+      PRBool res;
+      aViewManagerParent->GrabMouseEvents(nsnull, res);
+    }
+  }
+
+  if (mViewManager->GetRootView() == this)
+    mViewManager->InvalidateHierarchy();
+
+  for (nsView *child = mFirstChild; child; child = child->GetNextSibling())
+    child->InvalidateHierarchy(aViewManagerParent);
+}
+
 void nsView::InsertChild(nsView *aChild, nsView *aSibling)
 {
   NS_PRECONDITION(nsnull != aChild, "null ptr");
@@ -503,6 +519,15 @@ void nsView::InsertChild(nsView *aChild, nsView *aSibling)
       mFirstChild = aChild;
     }
     aChild->SetParent(this);
+
+    // If we just inserted a root view, then update the RootViewManager
+    // on all view managers in the new subtree.
+
+    nsViewManager *vm = aChild->GetViewManager();
+    if (vm->GetRootView() == aChild)
+    {
+      aChild->InvalidateHierarchy(nsnull); // don't care about releasing grabs
+    }
   }
 }
 
@@ -531,6 +556,15 @@ void nsView::RemoveChild(nsView *child)
 	    kid = kid->GetNextSibling();
     }
     NS_ASSERTION(found, "tried to remove non child");
+
+    // If we just removed a root view, then update the RootViewManager
+    // on all view managers in the removed subtree.
+
+    nsViewManager *vm = child->GetViewManager();
+    if (vm->GetRootView() == child)
+    {
+      child->InvalidateHierarchy(GetViewManager());
+    }
   }
 }
 
@@ -550,6 +584,40 @@ NS_IMETHODIMP nsView::SetContentTransparency(PRBool aTransparent)
   return NS_OK;
 }
 
+// Native widgets ultimately just can't deal with the awesome power of
+// CSS2 z-index. However, we set the z-index on the widget anyway
+// because in many simple common cases the widgets do end up in the
+// right order. We set each widget's z-index to the z-index of the
+// nearest ancestor that has non-auto z-index.
+static void UpdateNativeWidgetZIndexes(nsView* aView, PRInt32 aZIndex)
+{
+  if (aView->HasWidget()) {
+    nsIWidget* widget = aView->GetWidget();
+    PRInt32 curZ;
+    widget->GetZIndex(&curZ);
+    if (curZ != aZIndex) {
+      widget->SetZIndex(aZIndex);
+    }
+  } else {
+    for (nsView* v = aView->GetFirstChild(); v; v = v->GetNextSibling()) {
+      if (v->GetZIndexIsAuto()) {
+        UpdateNativeWidgetZIndexes(v, aZIndex);
+      }
+    }
+  }
+}
+
+static PRInt32 FindNonAutoZIndex(nsView* aView)
+{
+  while (aView) {
+    if (!aView->GetZIndexIsAuto()) {
+      return aView->GetZIndex();
+    }
+    aView = aView->GetParent();
+  }
+  return 0;
+}
+
 nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
                                nsWidgetInitData *aWidgetInitData,
                                nsNativeWidget aNative,
@@ -559,12 +627,12 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
 {
   nsIDeviceContext  *dx;
   nsRect            trect = mDimBounds;
-  float             scale;
 
+  NS_ASSERTION(!mWindow, "We already have a window for this view? BAD");
   NS_IF_RELEASE(mWindow);
 
   mViewManager->GetDeviceContext(dx);
-  scale = dx->AppUnitsToDevUnits();
+  float scale = dx->AppUnitsToDevUnits();
 
   trect *= scale;
 
@@ -584,6 +652,7 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
         // Create initData to pass in params
         initDataPassedIn = PR_FALSE;
         initData.clipChildren = PR_TRUE; // Clip child window's children
+        initData.clipSiblings = PR_TRUE; // Clip child window's siblings
         aWidgetInitData = &initData;
       }
       aWidgetInitData->mContentType = aContentType;
@@ -596,9 +665,15 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
           GetParent()->GetViewManager() != mViewManager)
           initData.mListenForResizes = PR_TRUE;
 
-        nsIWidget* parentWidget = GetParent() ? GetParent()->GetNearestWidget(nsnull)
+        nsPoint offset(0, 0);
+        nsIWidget* parentWidget = GetParent() ? GetParent()->GetNearestWidget(&offset)
           : nsnull;
+        trect += offset;
         if (aWidgetInitData->mWindowType == eWindowType_popup) {
+          // Without a parent, we can't make a popup.  This can happen
+          // when printing
+          if (!parentWidget)
+            return NS_ERROR_FAILURE;
           mWindow->Create(parentWidget->GetNativeData(NS_NATIVE_WIDGET), trect,
                           ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
         } else {
@@ -611,7 +686,7 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
       }
       
       // propagate the z-index to the widget.
-      mWindow->SetZIndex(mZIndex);
+      UpdateNativeWidgetZIndexes(v, FindNonAutoZIndex(v));
     }
   }
 
@@ -628,12 +703,13 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
 
 void nsView::SetZIndex(PRBool aAuto, PRInt32 aZIndex, PRBool aTopMost)
 {
+  PRBool oldIsAuto = GetZIndexIsAuto();
   mVFlags = (mVFlags & ~NS_VIEW_FLAG_AUTO_ZINDEX) | (aAuto ? NS_VIEW_FLAG_AUTO_ZINDEX : 0);
   mZIndex = aZIndex;
   SetTopMost(aTopMost);
   
-  if (nsnull != mWindow) {
-    mWindow->SetZIndex(aZIndex);
+  if (HasWidget() || !oldIsAuto || !aAuto) {
+    UpdateNativeWidgetZIndexes(this, FindNonAutoZIndex(this));
   }
 }
 
@@ -659,6 +735,8 @@ NS_IMETHODIMP nsView::SetWidget(nsIWidget *aWidget)
 
   mVFlags &= ~NS_VIEW_FLAG_HAS_POSITIONED_WIDGET;
 
+  UpdateNativeWidgetZIndexes(this, FindNonAutoZIndex(this));
+
   return NS_OK;
 }
 
@@ -682,7 +760,6 @@ nsresult nsView::LoadWidget(const nsCID &aClassIID)
   }
 
   mVFlags &= ~NS_VIEW_FLAG_HAS_POSITIONED_WIDGET;
-
   return rv;
 }
 
@@ -706,8 +783,10 @@ void nsIView::List(FILE* out, PRInt32 aIndent) const
     nonclientBounds *= p2t;
     nsrefcnt widgetRefCnt = mWindow->AddRef() - 1;
     mWindow->Release();
-    fprintf(out, "(widget=%p[%d] pos={%d,%d,%d,%d}) ",
-            (void*)mWindow, widgetRefCnt,
+    PRInt32 Z;
+    mWindow->GetZIndex(&Z);
+    fprintf(out, "(widget=%p[%d] z=%d pos={%d,%d,%d,%d}) ",
+            (void*)mWindow, widgetRefCnt, Z,
             nonclientBounds.x, nonclientBounds.y,
             windowBounds.width, windowBounds.height);
   }
@@ -754,11 +833,30 @@ nsPoint nsIView::GetOffsetTo(const nsIView* aOther) const
   return offset;
 }
 
-nsIWidget* nsIView::GetNearestWidget(nsPoint* aOffset)
+nsIntPoint nsIView::GetScreenPosition() const
+{
+  nsIntRect screenRect(0,0,0,0);  
+  nsPoint toWidgetOffset(0,0);
+  nsIWidget* widget = GetNearestWidget(&toWidgetOffset);
+  if (widget) {
+    nsCOMPtr<nsIDeviceContext> dx;
+    mViewManager->GetDeviceContext(*getter_AddRefs(dx));
+    float t2p = dx->AppUnitsToDevUnits();
+    nsIntRect ourRect(NSTwipsToIntPixels(toWidgetOffset.x, t2p),
+                      NSTwipsToIntPixels(toWidgetOffset.y, t2p),
+                      0,
+                      0);
+    widget->WidgetToScreen(ourRect, screenRect);
+  }
+  
+  return nsIntPoint(screenRect.x, screenRect.y);
+}
+
+nsIWidget* nsIView::GetNearestWidget(nsPoint* aOffset) const
 {
   nsPoint pt(0, 0);
-  nsView* v;
-  for (v = NS_STATIC_CAST(nsView*, this);
+  const nsView* v;
+  for (v = NS_STATIC_CAST(const nsView*, this);
        v && !v->HasWidget(); v = v->GetParent()) {
     pt += v->GetPosition();
   }
@@ -766,7 +864,7 @@ nsIWidget* nsIView::GetNearestWidget(nsPoint* aOffset)
     if (aOffset) {
       *aOffset = pt;
     }
-    return NS_STATIC_CAST(nsView*, this)->GetViewManager()->GetWidget();
+    return NS_STATIC_CAST(const nsView*, this)->GetViewManager()->GetWidget();
   }
 
   // pt is now the offset from v's origin to this's origin
@@ -790,15 +888,20 @@ PRBool nsIView::ExternalIsRoot() const
   return nsIView::IsRoot();
 }
 
-// Returns PR_TRUE iff we found a placeholder parent edge on the way
-static PRBool ApplyClipRect(const nsView* aView, nsRect* aRect, PRBool aFollowPlaceholders)
+/**
+ * @param aStopAtView clipping from ancestors of this view will not be applied
+ * @return PR_TRUE iff we found a placeholder parent edge on the way up the view tree
+ * from aView to aStopAtView
+ */
+static PRBool ApplyClipRect(const nsView* aView, nsRect* aRect, PRBool aFollowPlaceholders,
+                            nsIView* aStopAtView)
 {
   // this records the offset from the origin of the current aView
   // to the origin of the initial aView
   nsPoint offset(0, 0);
   PRBool lastViewIsFloating = aView->GetFloating();
   PRBool foundPlaceholders = PR_FALSE;
-  while (PR_TRUE) {
+  while (NS_STATIC_CAST(const nsIView*, aView) != aStopAtView) {
     const nsView* parentView = aView->GetParent();
     nsPoint offsetFromParent = aView->GetPosition();
 
@@ -808,7 +911,7 @@ static PRBool ApplyClipRect(const nsView* aView, nsRect* aRect, PRBool aFollowPl
       if (aFollowPlaceholders) {
         // correct offsetFromParent to account for the fact that we're
         // switching parentView to ZParent
-        // Note tha the common case is that parentView is an ancestor of
+        // Note that the common case is that parentView is an ancestor of
         // ZParent.
         const nsView* zParentChain;
         for (zParentChain = zParent;
@@ -850,8 +953,13 @@ static PRBool ApplyClipRect(const nsView* aView, nsRect* aRect, PRBool aFollowPl
         break;
       }
     }
+    // The child-clipping rect is applied to all *content* children.
+    // So we apply it if we're in the placeholder-following pass, or
+    // if we're in the first pass and we haven't detected any
+    // placeholders yet, in which case this geometric ancestor is also
+    // a content ancestor.
     const nsRect* r = parentView->GetClipChildrenToRect();
-    if (r && !aFollowPlaceholders) {
+    if (r && (!foundPlaceholders || aFollowPlaceholders)) {
       // Get the parent's clip rect into the initial aView's coordinates
       nsRect clipRect = *r;
       clipRect -= offset;
@@ -868,12 +976,12 @@ static PRBool ApplyClipRect(const nsView* aView, nsRect* aRect, PRBool aFollowPl
   return foundPlaceholders;
 }
 
-nsRect nsView::GetClippedRect()
+nsRect nsView::GetClippedRect(nsIView* aStopAtView)
 {
   nsRect clip = GetDimensions();
-  PRBool foundPlaceholders = ApplyClipRect(this, &clip, PR_FALSE);
+  PRBool foundPlaceholders = ApplyClipRect(this, &clip, PR_FALSE, aStopAtView);
   if (foundPlaceholders && !clip.IsEmpty()) {
-    ApplyClipRect(this, &clip, PR_TRUE);
+    ApplyClipRect(this, &clip, PR_TRUE, aStopAtView);
   }
   return clip;
 }
