@@ -45,11 +45,10 @@
 #include "prtime.h"
 #include "prinrval.h"
 #include "nsVoidArray.h"
+#include "nsThreadUtils.h"
 #include "nsIScrollableView.h"
 #include "nsIRegion.h"
 #include "nsIBlender.h"
-#include "nsIEventQueueService.h"
-#include "nsIEventQueue.h"
 #include "nsView.h"
 
 class nsIRegion;
@@ -145,6 +144,16 @@ protected:
   nsView   *mReparentedView;
 };
 
+class nsViewManagerEvent : public nsRunnable {
+public:
+  nsViewManagerEvent(class nsViewManager *vm) : mViewManager(vm) {
+    NS_ASSERTION(mViewManager, "null parameter");
+  }
+  void Revoke() { mViewManager = nsnull; }
+protected:
+  class nsViewManager *mViewManager;
+};
+
 class nsViewManager : public nsIViewManager {
 public:
   nsViewManager();
@@ -168,8 +177,6 @@ public:
   NS_IMETHOD  GetWindowDimensions(nscoord *width, nscoord *height);
   NS_IMETHOD  SetWindowDimensions(nscoord width, nscoord height);
 
-  NS_IMETHOD  ResetScrolling(void);
-
   NS_IMETHOD  Composite(void);
 
   NS_IMETHOD  UpdateView(nsIView *aView, PRUint32 aUpdateFlags);
@@ -231,6 +238,11 @@ public:
 
   NS_IMETHOD Display(nsIView *aView, nscoord aX, nscoord aY, const nsRect& aClipRect);
 
+  NS_IMETHOD RenderOffscreen(nsIView* aView, nsRect aRect, PRBool aUntrusted,
+                             PRBool aIgnoreViewportScrolling,
+                             nscolor aBackgroundColor,
+                             nsIRenderingContext** aRenderedContext);
+
   NS_IMETHOD AddCompositeListener(nsICompositeListener *aListener);
   NS_IMETHOD RemoveCompositeListener(nsICompositeListener *aListener);
 
@@ -240,7 +252,6 @@ public:
  
   NS_IMETHOD AllowDoubleBuffering(PRBool aDoubleBuffer);
   NS_IMETHOD IsPainting(PRBool& aIsPainting);
-  NS_IMETHOD FlushPendingInvalidates();
   NS_IMETHOD SetDefaultBackgroundColor(nscolor aColor);
   NS_IMETHOD GetDefaultBackgroundColor(nscolor* aColor);
   NS_IMETHOD GetLastUserEventTime(PRUint32& aTime);
@@ -266,11 +277,15 @@ public:
   NS_IMETHOD SynthesizeMouseMove(PRBool aFromScroll);
   void ProcessSynthMouseMoveEvent(PRBool aFromScroll);
 
+  /* Update the cached RootViewManager pointer on this view manager. */
+  void InvalidateHierarchy();
+
 protected:
   virtual ~nsViewManager();
-  void ProcessPendingUpdates(nsView *aView);
 
 private:
+  void FlushPendingInvalidates();
+  void ProcessPendingUpdates(nsView *aView, PRBool aDoInvalidate);
   void ReparentChildWidgets(nsIView* aView, nsIWidget *aNewWidget);
   void ReparentWidgets(nsIView* aView, nsIView *aParent);
   already_AddRefed<nsIRenderingContext> CreateRenderingContext(nsView &aView);
@@ -284,12 +299,13 @@ private:
   /**
    * Refresh aView (which must be non-null) with our default background color
    */
-  void DefaultRefresh(nsView* aView, const nsRect* aRect);
+  void DefaultRefresh(nsView* aView, nsIRenderingContext *aContext, const nsRect* aRect);
   PRBool BuildRenderingDisplayList(nsIView* aRootView,
-    const nsRegion& aRegion, nsVoidArray* aDisplayList, PLArenaPool &aPool);
+    const nsRegion& aRegion, nsVoidArray* aDisplayList, PLArenaPool &aPool,
+    PRBool aIgnoreCoveringWidgets, PRBool aIgnoreOutsideClipping,
+    nsIView* aSuppressScrolling);
   void RenderViews(nsView *aRootView, nsIRenderingContext& aRC,
-                   const nsRegion& aRegion, nsIDrawingSurface* aRCSurface,
-                   const nsVoidArray& aDisplayList);
+                   const nsRegion& aRegion, nsIDrawingSurface* aRCSurface);
 
   void RenderDisplayListElement(DisplayListElement2* element,
                                 nsIRenderingContext* aRC);
@@ -301,14 +317,17 @@ private:
   void InvalidateHorizontalBandDifference(nsView *aView, const nsRect& aRect, const nsRect& aCutOut,
                                           PRUint32 aUpdateFlags, nscoord aY1, nscoord aY2, PRBool aInCutOut);
 
-  BlendingBuffers* CreateBlendingBuffers(nsIRenderingContext *aRC, PRBool aBorrowContext,
-                                         nsIDrawingSurface* aBorrowSurface, PRBool aNeedAlpha,
-                                         const nsRect& aArea);
+  virtual BlendingBuffers* CreateBlendingBuffers(nsIRenderingContext *aRC, PRBool aBorrowContext,
+                                                 nsIDrawingSurface* aBorrowSurface, PRBool aNeedAlpha,
+                                                 const nsRect& aArea);
+  virtual nsIBlender* GetBlender() { return mBlender; }
 
   void ReparentViews(DisplayZTreeNode* aNode, nsHashtable &);
   void BuildDisplayList(nsView* aView, const nsRect& aRect, PRBool aEventProcessing,
-                        PRBool aCaptured, nsVoidArray* aDisplayList, PLArenaPool &aPool);
-  void BuildEventTargetList(nsVoidArray &aTargets, nsView* aView, nsGUIEvent* aEvent, PRBool aCaptured, PLArenaPool &aPool);
+                        PRBool aCaptured, nsIView* aSuppressScrolling,
+                        nsVoidArray* aDisplayList, PLArenaPool &aPool);
+  void BuildEventTargetList(nsVoidArray &aTargets, nsView* aView, nsPoint aPoint,
+                            PRBool aCaptured, PLArenaPool &aPool);
 
   PRBool CreateDisplayList(nsView *aView,
                            DisplayZTreeNode* &aResult,
@@ -316,11 +335,13 @@ private:
                            nsView *aRealView, const nsRect *aDamageRect,
                            nsView *aTopView, nscoord aX, nscoord aY,
                            PRBool aPaintFloats, PRBool aEventProcessing,
+                           nsIView* aSuppressClip,
                            nsHashtable&, PLArenaPool &aPool);
   PRBool AddToDisplayList(nsView *aView,
                           DisplayZTreeNode* &aParent, nsRect &aClipRect,
                           nsRect& aDirtyRect, PRUint32 aFlags, nscoord aAbsX, nscoord aAbsY,
-                          PRBool aAssumeIntersection, PLArenaPool &aPool);
+                          PRBool aAssumeIntersection, PLArenaPool &aPool,
+                          nsIView* aStopClippingAtView);
   void OptimizeDisplayList(const nsVoidArray* aDisplayList, const nsRegion& aDirtyRegion,
                            nsRect& aFinalTransparentRect, nsRegion& aOpaqueRgn,
                            PRBool aTreatUniformAsOpaque);
@@ -449,18 +470,50 @@ public: // NOT in nsIViewManager, so private to the view module
   nsViewManager* RootViewManager() const { return mRootViewManager; }
   PRBool IsRootVM() const { return this == RootViewManager(); }
 
-  nsEventStatus HandleEvent(nsView* aView, nsGUIEvent* aEvent, PRBool aCaptured);
+  nsEventStatus HandleEvent(nsView* aView, nsPoint aPoint, nsGUIEvent* aEvent,
+                            PRBool aCaptured);
 
+  /**
+   * Called to inform the view manager that a view is about to bit-blit.
+   * @param aView the view that will bit-blit
+   * @param aScrollAmount how much aView will scroll by
+   * @return always returns NS_OK
+   * @note
+   * This method used to return void, but MSVC 6.0 SP5 (without the
+   * Processor Pack) and SP6, and the MS eMbedded Visual C++ 4.0 SP4
+   * (for WINCE) hit an internal compiler error when compiling this
+   * method:
+   *
+   * @par
+@verbatim
+       fatal error C1001: INTERNAL COMPILER ERROR
+                   (compiler file 'E:\8966\vc98\p2\src\P2\main.c', line 494)
+@endverbatim
+   *
+   * @par
+   * Making the method return nsresult worked around the internal
+   * compiler error.  See Bugzilla bug 281158.  (The WINCE internal
+   * compiler error was addressed by the patch in bug 291229 comment
+   * 14 although the bug report did not mention the problem.)
+   */
+  nsresult WillBitBlit(nsView* aView, nsPoint aScrollAmount);
+  
   /**
    * Called to inform the view manager that a view has scrolled.
    * The view manager will invalidate any widgets which may need
    * to be rerendered.
-   * @param aView view to paint. should be root view
-   * @param aUpdateFlags see bottom of nsIViewManager.h for description
+   * @param aView view to paint. should be the nsScrollPortView that
+   * got scrolled.
+   * @param aUpdateRegion ensure that this part of the view is repainted
    */
-  void UpdateViewAfterScroll(nsIView *aView, PRInt32 aDX, PRInt32 aDY);
+  void UpdateViewAfterScroll(nsView *aView, const nsRegion& aUpdateRegion);
 
-  PRBool CanScrollWithBitBlt(nsView* aView);
+  /**
+   * Asks whether we can scroll a view using bitblt. If we say 'yes', we
+   * return in aUpdateRegion an area that must be updated (relative to aView
+   * after it has been scrolled).
+   */
+  PRBool CanScrollWithBitBlt(nsView* aView, nsPoint aDelta, nsRegion* aUpdateRegion);
 
   nsresult CreateRegion(nsIRegion* *result);
 
@@ -471,6 +524,11 @@ public: // NOT in nsIViewManager, so private to the view module
 
   PRBool IsRefreshEnabled() { return RootViewManager()->mRefreshEnabled; }
 
+  nsIViewObserver* GetViewObserver() { return mObserver; }
+
+  // Call this when you need to let the viewmanager know that it now has
+  // pending updates.
+  void PostPendingUpdate() { RootViewManager()->mHasPendingUpdates = PR_TRUE; }
 private:
   nsIDeviceContext  *mContext;
   float             mTwipsToPixels;
@@ -489,11 +547,14 @@ private:
   nsISupportsArray  *mCompositeListeners;
   nsCOMPtr<nsIFactory> mRegionFactory;
   nsView            *mRootView;
+  // mRootViewManager is a strong ref unless it equals |this|.  It's
+  // never null (if we have no ancestors, it will be |this|).
   nsViewManager     *mRootViewManager;
-  nsCOMPtr<nsIEventQueueService>  mEventQueueService;
-  nsCOMPtr<nsIEventQueue>         mSynthMouseMoveEventQueue;
   PRPackedBool      mAllowDoubleBuffering;
 
+  nsRevocableEventPtr<nsViewManagerEvent> mSynthMouseMoveEvent;
+  nsRevocableEventPtr<nsViewManagerEvent> mInvalidateEvent;
+
   // The following members should not be accessed directly except by
   // the root view manager.  Some have accessor functions to enforce
   // this, as noted.
@@ -505,13 +566,14 @@ private:
   PRInt32           mUpdateCnt;
   PRInt32           mUpdateBatchCnt;
   PRUint32          mUpdateBatchFlags;
-  nsCOMPtr<nsIEventQueue>         mInvalidateEventQueue;
+  PRInt32           mScrollCnt;
   // Use IsRefreshEnabled() to check the value of mRefreshEnabled.
   PRPackedBool      mRefreshEnabled;
   // Use IsPainting() and SetPainting() to access mPainting.
   PRPackedBool      mPainting;
   PRPackedBool      mRecursiveRefreshPending;
-  PRPackedBool      mHasPendingInvalidates;
+  PRPackedBool      mHasPendingUpdates;
+  PRPackedBool      mInScroll;
 
   //from here to public should be static and locked... MMP
   static PRInt32           mVMCount;        //number of viewmanagers
