@@ -52,6 +52,7 @@
 #ifdef WIN32
 #undef ERROR
 #endif
+#include "nsCOMPtr.h"
 #include "nsIDOMKeyEvent.h"
 
 class nsIRenderingContext;
@@ -61,7 +62,8 @@ class nsIMenuItem;
 class nsIAccessible;
 class nsIContent;
 class nsIURI;
-           
+class nsIDOMEvent;
+
 /**
  * Event Struct Types
  */
@@ -91,9 +93,19 @@ class nsIURI;
 #define NS_POPUPBLOCKED_EVENT             25
 #define NS_BEFORE_PAGE_UNLOAD_EVENT       26
 #define NS_UI_EVENT                       27
+#define NS_QUERYCARETRECT_EVENT           28
+#define NS_PAGETRANSITION_EVENT           29
+#ifdef MOZ_SVG
+#define NS_SVG_EVENT                      30
+#define NS_SVGZOOM_EVENT                  31
+#endif // MOZ_SVG
+#define NS_XUL_COMMAND_EVENT              32
 
+// These flags are sort of a mess. They're sort of shared between event
+// listener flags and event flags, but only some of them. You've been
+// warned!
 #define NS_EVENT_FLAG_NONE                0x0000
-#define NS_EVENT_FLAG_INIT                0x0001
+#define NS_EVENT_FLAG_TRUSTED             0x0001
 #define NS_EVENT_FLAG_BUBBLE              0x0002
 #define NS_EVENT_FLAG_CAPTURE             0x0004
 #define NS_EVENT_FLAG_STOP_DISPATCH       0x0008
@@ -104,19 +116,12 @@ class nsIURI;
 #define NS_EVENT_FLAG_NO_CONTENT_DISPATCH 0x0100
 #define NS_EVENT_FLAG_SYSTEM_EVENT        0x0200
 #define NS_EVENT_FLAG_STOP_DISPATCH_IMMEDIATELY 0x0400 // @see nsIDOM3Event::stopImmediatePropagation()
+#define NS_EVENT_FLAG_DISPATCHING         0x0800
 
-#define NS_EVENT_CAPTURE_MASK             (~(NS_EVENT_FLAG_INIT | NS_EVENT_FLAG_BUBBLE | NS_EVENT_FLAG_NO_CONTENT_DISPATCH))
-#define NS_EVENT_BUBBLE_MASK              (~(NS_EVENT_FLAG_INIT | NS_EVENT_FLAG_CAPTURE | NS_EVENT_FLAG_NO_CONTENT_DISPATCH))
+#define NS_PRIV_EVENT_UNTRUSTED_PERMITTED 0x8000
 
-// Flags for internalAppFlags
-
-#define NS_APP_EVENT_FLAG_NONE      0x00000000
-
-// Similar to NS_EVENT_FLAG_NO_DEFAULT, but it allows focus
-#define NS_APP_EVENT_FLAG_HANDLED   0x00000001
-
-// True if the event came from a trusted source
-#define NS_APP_EVENT_FLAG_TRUSTED   0x00000002
+#define NS_EVENT_CAPTURE_MASK             (~(NS_EVENT_FLAG_BUBBLE | NS_EVENT_FLAG_NO_CONTENT_DISPATCH))
+#define NS_EVENT_BUBBLE_MASK              (~(NS_EVENT_FLAG_CAPTURE | NS_EVENT_FLAG_NO_CONTENT_DISPATCH))
 
 #define NS_EVENT_TYPE_NULL                   0
 
@@ -186,9 +191,6 @@ class nsIURI;
 // ensure that it is drawn using the current system colors.
 #define NS_SYSCOLORCHANGED              (NS_WINDOW_START + 42)
 
-// Indicates a script error has occurred
-#define NS_SCRIPT_ERROR                 (NS_WINDOW_START + 50)
-
 #define NS_RESIZE_EVENT                 (NS_WINDOW_START + 60)
 #define NS_SCROLL_EVENT                 (NS_WINDOW_START + 61)
 
@@ -226,13 +228,12 @@ class nsIURI;
 #define NS_SCROLLBAR_LINE_PREV          (NS_SCROLLBAR_MESSAGE_START + 4)
 
 #define NS_STREAM_EVENT_START           1100
-#define NS_PAGE_LOAD                    (NS_STREAM_EVENT_START)
+#define NS_LOAD                         (NS_STREAM_EVENT_START)
 #define NS_PAGE_UNLOAD                  (NS_STREAM_EVENT_START + 1)
-#define NS_IMAGE_LOAD                   (NS_STREAM_EVENT_START + 2)
 #define NS_IMAGE_ABORT                  (NS_STREAM_EVENT_START + 3)
-#define NS_IMAGE_ERROR                  (NS_STREAM_EVENT_START + 4)
-#define NS_SCRIPT_LOAD                  (NS_STREAM_EVENT_START + 5)
+#define NS_LOAD_ERROR                   (NS_STREAM_EVENT_START + 4)
 #define NS_BEFORE_PAGE_UNLOAD           (NS_STREAM_EVENT_START + 6)
+#define NS_PAGE_RESTORE                 (NS_STREAM_EVENT_START + 7)
  
 #define NS_FORM_EVENT_START             1200
 #define NS_FORM_SUBMIT                  (NS_FORM_EVENT_START)
@@ -262,10 +263,9 @@ class nsIURI;
 #define NS_XUL_POPUP_SHOWN            (NS_XUL_EVENT_START+1)
 #define NS_XUL_POPUP_HIDING           (NS_XUL_EVENT_START+2)
 #define NS_XUL_POPUP_HIDDEN           (NS_XUL_EVENT_START+3)
-#define NS_XUL_COMMAND                (NS_XUL_EVENT_START+4)
+// NS_XUL_COMMAND used to be here     (NS_XUL_EVENT_START+4)
 #define NS_XUL_BROADCAST              (NS_XUL_EVENT_START+5)
 #define NS_XUL_COMMAND_UPDATE         (NS_XUL_EVENT_START+6)
-#define NS_XUL_CLICK                  (NS_XUL_EVENT_START+7)
 //@}
 
 // Scroll events
@@ -317,6 +317,34 @@ class nsIURI;
 #define NS_UI_FOCUSIN              (NS_UI_EVENT_START + 1)
 #define NS_UI_FOCUSOUT             (NS_UI_EVENT_START + 2)
 
+// query caret rect events
+#define NS_QUERYCARETRECT_START    2600
+#define NS_QUERYCARETRECT          (NS_QUERYCARETRECT_START)
+
+// pagetransition events
+#define NS_PAGETRANSITION_START    2700
+#define NS_PAGE_SHOW               (NS_PAGETRANSITION_START + 1)
+#define NS_PAGE_HIDE               (NS_PAGETRANSITION_START + 2)
+
+#ifdef MOZ_SVG
+// SVG events
+#define NS_SVG_EVENT_START              2800
+#define NS_SVG_LOAD                     (NS_SVG_EVENT_START)
+#define NS_SVG_UNLOAD                   (NS_SVG_EVENT_START + 1)
+#define NS_SVG_ABORT                    (NS_SVG_EVENT_START + 2)
+#define NS_SVG_ERROR                    (NS_SVG_EVENT_START + 3)
+#define NS_SVG_RESIZE                   (NS_SVG_EVENT_START + 4)
+#define NS_SVG_SCROLL                   (NS_SVG_EVENT_START + 5)
+
+// SVG Zoom events
+#define NS_SVGZOOM_EVENT_START          2900
+#define NS_SVG_ZOOM                     (NS_SVGZOOM_EVENT_START)
+#endif // MOZ_SVG
+
+// XUL command events
+#define NS_XULCOMMAND_EVENT_START       3000
+#define NS_XUL_COMMAND                  (NS_XULCOMMAND_EVENT_START)
+
 /**
  * Return status for event processors, nsEventStatus, is defined in
  * nsEvent.h.
@@ -344,47 +372,68 @@ enum nsWindowZ {
  * General event
  */
 
-struct nsEvent {
-  nsEvent(PRUint32 msg = 0, PRUint8 structType = NS_EVENT)
+class nsEvent
+{
+protected:
+  nsEvent(PRBool isTrusted, PRUint32 msg, PRUint8 structType)
     : eventStructType(structType),
       message(msg),
-      point(0, 0),
       refPoint(0, 0),
       time(0),
-      flags(0),
-      internalAppFlags(NS_APP_EVENT_FLAG_NONE),
+      flags(isTrusted ? NS_EVENT_FLAG_TRUSTED : NS_EVENT_FLAG_NONE),
       userType(0)
   {
   }
 
-  /// See event struct types
+public:
+  nsEvent(PRBool isTrusted, PRUint32 msg)
+    : eventStructType(NS_EVENT),
+      message(msg),
+      refPoint(0, 0),
+      time(0),
+      flags(isTrusted ? NS_EVENT_FLAG_TRUSTED : NS_EVENT_FLAG_NONE),
+      userType(0)
+  {
+  }
+
+  // See event struct types
   PRUint8     eventStructType;
-  /// See GUI MESSAGES,
-  PRUint32    message;              
-  /// in widget relative coordinates, modified to be relative to current view in layout.
-  nsPoint     point;               
-  // in widget relative coordinates, not modified by layout code.
-  nsPoint     refPoint;               
-  /// elapsed time, in milliseconds, from the time the system was started to the time the message was created
-  PRUint32    time;      
-  // flags to hold event flow stage and capture/bubble cancellation status
+  // See GUI MESSAGES,
+  PRUint32    message;
+  // In widget relative coordinates, not modified by layout code.
+  nsPoint     refPoint;
+  // Elapsed time, in milliseconds, from the time the system was
+  // started to the time the message was created
+  PRUint32    time;
+  // Flags to hold event flow stage and capture/bubble cancellation
+  // status. This is used also to indicate whether the event is trusted.
   PRUint32    flags;
-  // flags for indicating more event state for Mozilla applications.
-  PRUint32    internalAppFlags;
-  // additional type info for user defined events
+  // Additional type info for user defined events
   nsHashKey*  userType;
+  // Event targets, needed by DOM Events
+  // Using nsISupports, not nsIDOMEventTarget because in some cases
+  // nsIDOMEventTarget is implemented as a tearoff.
+  nsCOMPtr<nsISupports> target;
+  nsCOMPtr<nsISupports> currentTarget;
+  nsCOMPtr<nsISupports> originalTarget;
 };
 
 /**
  * General graphic user interface event
  */
 
-struct nsGUIEvent : public nsEvent
+class nsGUIEvent : public nsEvent
 {
-  nsGUIEvent(PRUint32 msg = 0,
-             nsIWidget *w = nsnull,
-             PRUint8 structType = NS_GUI_EVENT)
-    : nsEvent(msg, structType),
+protected:
+  nsGUIEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w, PRUint8 structType)
+    : nsEvent(isTrusted, msg, structType),
+      widget(w), nativeMsg(nsnull)
+  {
+  }
+
+public:
+  nsGUIEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsEvent(isTrusted, msg, NS_GUI_EVENT),
       widget(w), nativeMsg(nsnull)
   {
   }
@@ -399,11 +448,11 @@ struct nsGUIEvent : public nsEvent
  * Script error event
  */
 
-struct nsScriptErrorEvent : public nsEvent
+class nsScriptErrorEvent : public nsEvent
 {
-  nsScriptErrorEvent(PRUint32 msg = 0,
-                     PRUint8 structType = NS_SCRIPT_ERROR_EVENT)
-    : nsEvent(msg, structType),
+public:
+  nsScriptErrorEvent(PRBool isTrusted, PRUint32 msg)
+    : nsEvent(isTrusted, msg, NS_SCRIPT_ERROR_EVENT),
       lineNr(0), errorMsg(nsnull), fileName(nsnull)
   {
   }
@@ -413,10 +462,11 @@ struct nsScriptErrorEvent : public nsEvent
   const PRUnichar*  fileName;
 };
 
-struct nsBeforePageUnloadEvent : public nsEvent
+class nsBeforePageUnloadEvent : public nsEvent
 {
-  nsBeforePageUnloadEvent(PRUint32 msg)
-    : nsEvent(msg, NS_BEFORE_PAGE_UNLOAD_EVENT)
+public:
+  nsBeforePageUnloadEvent(PRBool isTrusted, PRUint32 msg)
+    : nsEvent(isTrusted, msg, NS_BEFORE_PAGE_UNLOAD_EVENT)
   {
   }
 
@@ -427,12 +477,11 @@ struct nsBeforePageUnloadEvent : public nsEvent
  * Window resize event
  */
 
-struct nsSizeEvent : public nsGUIEvent
+class nsSizeEvent : public nsGUIEvent
 {
-  nsSizeEvent(PRUint32 msg = 0,
-              nsIWidget *w = nsnull,
-              PRUint8 structType = NS_SIZE_EVENT)
-    : nsGUIEvent(msg, w, structType),
+public:
+  nsSizeEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_SIZE_EVENT),
       windowSize(nsnull), mWinWidth(0), mWinHeight(0)
   {
   }
@@ -449,12 +498,11 @@ struct nsSizeEvent : public nsGUIEvent
  * Window size mode event
  */
 
-struct nsSizeModeEvent : public nsGUIEvent
+class nsSizeModeEvent : public nsGUIEvent
 {
-  nsSizeModeEvent(PRUint32 msg = 0,
-                  nsIWidget *w = nsnull,
-                  PRUint8 structType = NS_SIZEMODE_EVENT)
-    : nsGUIEvent(msg, w, structType),
+public:
+  nsSizeModeEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_SIZEMODE_EVENT),
       mSizeMode(nsSizeMode_Normal)
   {
   }
@@ -466,12 +514,11 @@ struct nsSizeModeEvent : public nsGUIEvent
  * Window z-level event
  */
 
-struct nsZLevelEvent : public nsGUIEvent
+class nsZLevelEvent : public nsGUIEvent
 {
-  nsZLevelEvent(PRUint32 msg = 0,
-                nsIWidget *w = nsnull,
-                PRUint8 structType = NS_ZLEVEL_EVENT)
-    : nsGUIEvent(msg, w, structType),
+public:
+  nsZLevelEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_ZLEVEL_EVENT),
       mPlacement(nsWindowZTop), mReqBelow(nsnull), mActualBelow(nsnull),
       mImmediate(PR_FALSE), mAdjusted(PR_FALSE)
   {
@@ -488,12 +535,11 @@ struct nsZLevelEvent : public nsGUIEvent
  * Window repaint event
  */
 
-struct nsPaintEvent : public nsGUIEvent
+class nsPaintEvent : public nsGUIEvent
 {
-  nsPaintEvent(PRUint32 msg = 0,
-               nsIWidget *w = nsnull,
-               PRUint8 structType = NS_PAINT_EVENT)
-    : nsGUIEvent(msg, w, structType),
+public:
+  nsPaintEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_PAINT_EVENT),
       renderingContext(nsnull), region(nsnull), rect(nsnull)
   {
   }
@@ -510,12 +556,11 @@ struct nsPaintEvent : public nsGUIEvent
  * Scrollbar event
  */
 
-struct nsScrollbarEvent : public nsGUIEvent
+class nsScrollbarEvent : public nsGUIEvent
 {
-  nsScrollbarEvent(PRUint32 msg = 0,
-                   nsIWidget *w = nsnull,
-                   PRUint8 structType = NS_SCROLLBAR_EVENT)
-    : nsGUIEvent(msg, w, structType),
+public:
+  nsScrollbarEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_SCROLLBAR_EVENT),
       position(0)
   {
   }
@@ -524,18 +569,17 @@ struct nsScrollbarEvent : public nsGUIEvent
   PRUint32        position; 
 };
 
-struct nsScrollPortEvent : public nsGUIEvent
+class nsScrollPortEvent : public nsGUIEvent
 {
+public:
   enum orientType {
     vertical   = 0,
     horizontal = 1,
     both       = 2
   };
 
-  nsScrollPortEvent(PRUint32 msg = 0,
-                    nsIWidget *w = nsnull,
-                    PRUint8 structType = NS_SCROLLPORT_EVENT)
-    : nsGUIEvent(msg, w, structType),
+  nsScrollPortEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_SCROLLPORT_EVENT),
       orient(vertical)
   {
   }
@@ -543,12 +587,19 @@ struct nsScrollPortEvent : public nsGUIEvent
   orientType orient;
 };
 
-struct nsInputEvent : public nsGUIEvent
+class nsInputEvent : public nsGUIEvent
 {
-  nsInputEvent(PRUint32 msg = 0,
-               nsIWidget *w = nsnull,
-               PRUint8 structType = NS_INPUT_EVENT)
-    : nsGUIEvent(msg, w, structType),
+protected:
+  nsInputEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w,
+               PRUint8 structType)
+    : nsGUIEvent(isTrusted, msg, w, structType),
+      isShift(PR_FALSE), isControl(PR_FALSE), isAlt(PR_FALSE), isMeta(PR_FALSE)
+  {
+  }
+
+public:
+  nsInputEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_INPUT_EVENT),
       isShift(PR_FALSE), isControl(PR_FALSE), isAlt(PR_FALSE), isMeta(PR_FALSE)
   {
   }
@@ -567,15 +618,14 @@ struct nsInputEvent : public nsGUIEvent
  * Mouse event
  */
 
-struct nsMouseEvent : public nsInputEvent
+class nsMouseEvent : public nsInputEvent
 {
+public:
   enum reasonType { eReal, eSynthesized };
 
-  nsMouseEvent(PRUint32 msg = 0,
-               nsIWidget *w = nsnull,
-               reasonType aReason = eReal,
-               PRUint8 structType = NS_MOUSE_EVENT)
-    : nsInputEvent(msg, w, structType),
+  nsMouseEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w,
+               reasonType aReason)
+    : nsInputEvent(isTrusted, msg, w, NS_MOUSE_EVENT),
       clickCount(0), acceptActivation(PR_FALSE), reason(aReason)
   {
     if (msg == NS_MOUSE_MOVE) {
@@ -584,40 +634,40 @@ struct nsMouseEvent : public nsInputEvent
   }
 
   /// The number of mouse clicks
-  PRUint32        clickCount;          
+  PRUint32              clickCount;
   /// Special return code for MOUSE_ACTIVATE to signal
   /// if the target accepts activation (1), or denies it (0)
-  PRPackedBool    acceptActivation;           
-  reasonType      reason : 8;
+  PRPackedBool          acceptActivation;
+  reasonType            reason : 8;
+  /// The possible related target
+  nsCOMPtr<nsISupports> relatedTarget;
 };
 
 /**
  * Accessible event
  */
 
-struct nsAccessibleEvent : public nsInputEvent
+class nsAccessibleEvent : public nsInputEvent
 {
-  nsAccessibleEvent(PRUint32 msg = 0,
-                    nsIWidget *w = nsnull,
-                    PRUint8 structType = NS_ACCESSIBLE_EVENT)
-    : nsInputEvent(msg, w, structType),
+public:
+  nsAccessibleEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_ACCESSIBLE_EVENT),
       accessible(nsnull)
   {
   }
 
-  nsIAccessible*     accessible;           
+  nsIAccessible*  accessible;     
 };
 
 /**
  * Keyboard event
  */
 
-struct nsKeyEvent : public nsInputEvent
+class nsKeyEvent : public nsInputEvent
 {
-  nsKeyEvent(PRUint32 msg = 0,
-             nsIWidget *w = nsnull,
-             PRUint8 structType = NS_KEY_EVENT)
-    : nsInputEvent(msg, w, structType),
+public:
+  nsKeyEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_KEY_EVENT),
       keyCode(0), charCode(0), isChar(0)
   {
   }
@@ -633,7 +683,8 @@ struct nsKeyEvent : public nsInputEvent
 /**
  * IME Related Events
  */
-struct nsTextRange {
+struct nsTextRange
+{
   nsTextRange()
     : mStartOffset(0), mEndOffset(0), mRangeType(0)
   {
@@ -646,7 +697,8 @@ struct nsTextRange {
 
 typedef nsTextRange* nsTextRangeArray;
 
-struct nsTextEventReply {
+struct nsTextEventReply
+{
   nsTextEventReply()
     : mCursorIsCollapsed(PR_FALSE)
   {
@@ -658,12 +710,11 @@ struct nsTextEventReply {
 
 typedef struct nsTextEventReply nsTextEventReply;
 
-struct nsTextEvent : public nsInputEvent
+class nsTextEvent : public nsInputEvent
 {
-  nsTextEvent(PRUint32 msg = 0,
-              nsIWidget *w = nsnull,
-              PRUint8 structType = NS_TEXT_EVENT)
-    : nsInputEvent(msg, w, structType),
+public:
+  nsTextEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_TEXT_EVENT),
       theText(nsnull), rangeCount(0), rangeArray(nsnull), isChar(PR_FALSE)
   {
   }
@@ -675,36 +726,37 @@ struct nsTextEvent : public nsInputEvent
   PRBool            isChar;
 };
 
-struct nsCompositionEvent : public nsInputEvent
+class nsCompositionEvent : public nsInputEvent
 {
-  nsCompositionEvent(PRUint32 msg = 0,
-                     nsIWidget *w = nsnull,
-                     PRUint8 structType = NS_COMPOSITION_EVENT)
-    : nsInputEvent(msg, w, structType)
+public:
+  nsCompositionEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_COMPOSITION_EVENT)
   {
   }
 
   nsTextEventReply theReply;
 };
 
-struct nsMouseScrollEvent : public nsInputEvent
+class nsMouseScrollEvent : public nsInputEvent
 {
+public:
   enum nsMouseScrollFlags {
     kIsFullPage =   1 << 0,
     kIsVertical =   1 << 1,
-    kIsHorizontal = 1 << 2
+    kIsHorizontal = 1 << 2,
+    kIsPixels =     1 << 3
   };
 
-  nsMouseScrollEvent(PRUint32 msg = 0,
-                     nsIWidget *w = nsnull,
-                     PRUint8 structType = NS_MOUSE_SCROLL_EVENT)
-    : nsInputEvent(msg, w, structType),
+  nsMouseScrollEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_MOUSE_SCROLL_EVENT),
       scrollFlags(0), delta(0)
   {
   }
 
-  PRInt32 scrollFlags;
-  PRInt32 delta;
+  PRInt32               scrollFlags;
+  PRInt32               delta;
+  /// The possible related target
+  nsCOMPtr<nsISupports> relatedTarget;
 };
 
 struct nsReconversionEventReply {
@@ -716,18 +768,39 @@ struct nsReconversionEventReply {
   PRUnichar *mReconversionString;
 };
 
-struct nsReconversionEvent : public nsInputEvent
+class nsReconversionEvent : public nsInputEvent
 {
-  nsReconversionEvent(PRUint32 msg = 0,
-                      nsIWidget *w = nsnull,
-                      PRUint8 structType = NS_RECONVERSION_EVENT)
-    : nsInputEvent(msg, w, structType)
+public:
+  nsReconversionEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_RECONVERSION_EVENT)
   {
   }
 
   nsReconversionEventReply  theReply;
 };
 
+struct nsQueryCaretRectEventReply
+{
+  nsQueryCaretRectEventReply()
+    : mRectIsValid(PR_FALSE)
+  {
+  }
+
+  PRBool mRectIsValid;
+  nsRect mCaretRect;
+};
+
+class nsQueryCaretRectEvent : public nsInputEvent
+{
+public:
+  nsQueryCaretRectEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_QUERYCARETRECT_EVENT)
+  {
+  }
+
+  nsQueryCaretRectEventReply theReply;
+};
+
 /**
  * MenuItem event
  * 
@@ -735,12 +808,11 @@ struct nsReconversionEvent : public nsInputEvent
  * for the event
  */
 
-struct nsMenuEvent : public nsGUIEvent
+class nsMenuEvent : public nsGUIEvent
 {
-  nsMenuEvent(PRUint32 msg = 0,
-              nsIWidget *w = nsnull,
-              PRUint8 structType = NS_MENU_EVENT)
-    : nsGUIEvent(msg, w, structType),
+public:
+  nsMenuEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_MENU_EVENT),
       mMenuItem(nsnull), mCommand(0)
   {
   }
@@ -756,10 +828,11 @@ struct nsMenuEvent : public nsGUIEvent
  * originator is a weak pointer (does not hold a strong reference).
  */
 
-struct nsFormEvent : public nsEvent
+class nsFormEvent : public nsEvent
 {
-  nsFormEvent(PRUint32 msg = 0, PRUint8 structType = NS_FORM_EVENT)
-    : nsEvent(msg, structType),
+public:
+  nsFormEvent(PRBool isTrusted, PRUint32 msg)
+    : nsEvent(isTrusted, msg, NS_FORM_EVENT),
       originator(nsnull)
   {
   }
@@ -770,12 +843,11 @@ struct nsFormEvent : public nsEvent
 /**
 * Focus event
 */
-struct nsFocusEvent : public nsGUIEvent
+class nsFocusEvent : public nsGUIEvent
 {
-  nsFocusEvent(PRUint32 msg = 0,
-               nsIWidget *w = nsnull,
-               PRUint8 structType = NS_FOCUS_EVENT)
-    : nsGUIEvent(msg, w, structType),
+public:
+  nsFocusEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsGUIEvent(isTrusted, msg, w, NS_FOCUS_EVENT),
       isMozWindowTakingFocus(PR_FALSE)
   {
   }
@@ -789,12 +861,11 @@ struct nsFocusEvent : public nsGUIEvent
  * Custom commands from the operating system.  eg. WM_APPCOMMAND on Windows
  */
 
-struct nsAppCommandEvent : public nsInputEvent
+class nsAppCommandEvent : public nsInputEvent
 {
-  nsAppCommandEvent(PRUint32 msg = 0,
-                    nsIWidget *w = nsnull,
-                    PRUint8 structType = NS_APPCOMMAND_EVENT)
-    : nsInputEvent(msg, w, structType),
+public:
+  nsAppCommandEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_APPCOMMAND_EVENT),
       appCommand(0)
   {
   }
@@ -805,11 +876,11 @@ struct nsAppCommandEvent : public nsInputEvent
 /**
  * blocked popup window event
  */
-struct nsPopupBlockedEvent : public nsEvent
+class nsPopupBlockedEvent : public nsEvent
 {
-  nsPopupBlockedEvent(PRUint32 msg = 0,
-                      PRUint8 structType = NS_POPUPBLOCKED_EVENT)
-    : nsEvent(msg, structType),
+public:
+  nsPopupBlockedEvent(PRBool isTrusted, PRUint32 msg)
+    : nsEvent(isTrusted, msg, NS_POPUPBLOCKED_EVENT),
       mRequestingWindowURI(nsnull), mPopupWindowURI(nsnull)
   {
   }
@@ -817,16 +888,17 @@ struct nsPopupBlockedEvent : public nsEvent
   nsIURI* mRequestingWindowURI; // owning reference
   nsIURI* mPopupWindowURI;      // owning reference
   nsString mPopupWindowFeatures;
+  nsString mPopupWindowName;
 };
 
 /**
  * DOM UIEvent
  */
-struct nsUIEvent : public nsEvent {
-  nsUIEvent(PRUint32 msg = 0,
-               PRInt32 d = 0,
-               PRUint8 structType = NS_UI_EVENT)
-    : nsEvent(msg, structType),
+class nsUIEvent : public nsEvent
+{
+public:
+  nsUIEvent(PRBool isTrusted, PRUint32 msg, PRInt32 d)
+    : nsEvent(isTrusted, msg, NS_UI_EVENT),
       detail(d)
   {
   }
@@ -834,6 +906,35 @@ struct nsUIEvent : public nsEvent {
   PRInt32 detail;
 };
 
+/**
+ * PageTransition event
+ */
+class nsPageTransitionEvent : public nsEvent
+{
+public:
+  nsPageTransitionEvent(PRBool isTrusted, PRUint32 msg, PRBool p)
+    : nsEvent(isTrusted, msg, NS_PAGETRANSITION_EVENT),
+      persisted(p)
+  {
+  }
+
+  PRBool persisted;
+};
+
+/**
+ * XUL command event
+ */
+class nsXULCommandEvent : public nsInputEvent
+{
+public:
+  nsXULCommandEvent(PRBool isTrusted, PRUint32 msg, nsIWidget *w)
+    : nsInputEvent(isTrusted, msg, w, NS_XUL_COMMAND_EVENT)
+  {
+  }
+
+  nsCOMPtr<nsIDOMEvent> sourceEvent;
+};
+
 /**
  * Event status for D&D Event
  */
@@ -862,8 +963,7 @@ enum nsDragDropEventStatus {
         ((evnt)->message == NS_MOUSE_RIGHT_DOUBLECLICK) || \
         ((evnt)->message == NS_MOUSE_ENTER) || \
         ((evnt)->message == NS_MOUSE_EXIT) || \
-        ((evnt)->message == NS_MOUSE_ENTER) || \
-        ((evnt)->message == NS_MOUSE_EXIT) || \
+        ((evnt)->message == NS_MOUSE_ACTIVATE) || \
         ((evnt)->message == NS_MOUSE_ENTER_SYNTH) || \
         ((evnt)->message == NS_MOUSE_EXIT_SYNTH) || \
         ((evnt)->message == NS_MOUSE_MOVE))
@@ -887,6 +987,7 @@ enum nsDragDropEventStatus {
         ((evnt)->message == NS_COMPOSITION_START) ||  \
         ((evnt)->message == NS_COMPOSITION_END) || \
         ((evnt)->message == NS_RECONVERSION_QUERY) || \
+        ((evnt)->message == NS_QUERYCARETRECT) || \
         ((evnt)->message == NS_COMPOSITION_QUERY))
 
 #define NS_IS_FOCUS_EVENT(evnt) \
@@ -896,6 +997,23 @@ enum nsDragDropEventStatus {
         ((evnt)->message == NS_DEACTIVATE) || \
         ((evnt)->message == NS_PLUGIN_ACTIVATE))
 
+#define NS_IS_TRUSTED_EVENT(event) \
+  (((event)->flags & NS_EVENT_FLAG_TRUSTED) != 0)
+
+// Mark an event as being dispatching.
+#define NS_MARK_EVENT_DISPATCH_STARTED(event) \
+  (event)->flags |= NS_EVENT_FLAG_DISPATCHING;
+
+#define NS_IS_EVENT_IN_DISPATCH(event) \
+  (((event)->flags & NS_EVENT_FLAG_DISPATCHING) != 0)
+
+// Mark an event as being done dispatching.
+#define NS_MARK_EVENT_DISPATCH_DONE(event) \
+  NS_ASSERTION(NS_IS_EVENT_IN_DISPATCH(event), \
+               "Event never got marked for dispatch!"); \
+  (event)->flags &= ~NS_EVENT_FLAG_DISPATCHING; \
+  (event)->flags |= NS_EVENT_FLAG_STOP_DISPATCH_IMMEDIATELY;
+
 /*
  * Virtual key bindings for keyboard events.
  * These come from nsIDOMKeyEvent.h, which is generated from MouseKeyEvent.idl.
