@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 40; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -62,8 +62,13 @@ struct  nsFont;
 class   nsIMenuBar;
 class   nsIEventListener;
 class   nsIRollupListener;
-struct  nsGUIEvent;
+class   nsGUIEvent;
 struct  nsColorMap;
+class   imgIContainer;
+
+#ifdef MOZ_CAIRO_GFX
+class   gfxASurface;
+#endif
 
 /**
  * Callback function that processes events.
@@ -89,11 +94,12 @@ typedef nsEventStatus (*PR_CALLBACK EVENT_CALLBACK)(nsGUIEvent *event);
 #define NS_NATIVE_OFFSETY     7
 #define NS_NATIVE_PLUGIN_PORT 8
 #define NS_NATIVE_SCREEN      9
+#define NS_NATIVE_SHELLWIDGET 10      // Get the shell GtkWidget
 
-// {18032AD5-B265-11d1-AA2A-000000000000}
+// d6fec391-e2d8-4841-9d8f-43423b953884
 #define NS_IWIDGET_IID \
-{ 0x18032ad5, 0xb265, 0x11d1, \
-{ 0xaa, 0x2a, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 } }
+{ 0xd6fec391, 0xe2d8, 0x4841, \
+  { 0x9d, 0x8f, 0x43, 0x42, 0x3b, 0x95, 0x38, 0x84 } }
 
 
 // Hide the native window systems real window type so as to avoid
@@ -119,7 +125,9 @@ enum nsWindowType {     // Don't alter previously encoded enum values - 3rd part
   // plugin window
   eWindowType_plugin,
   // java plugin window
-  eWindowType_java
+  eWindowType_java,
+  // MacOSX sheet (special dialog class)
+  eWindowType_sheet
 };
 
 enum nsBorderStyle
@@ -152,9 +160,6 @@ enum nsBorderStyle
   // show the close button
   eBorderStyle_close    = 1 << 7,
 
-  // sheet hint (Mac OS X)
-  eBorderStyle_sheet    = 1 << 8,
-
   // whatever the OS wants... i.e. don't do anything
   eBorderStyle_default  = -1
 };
@@ -209,8 +214,9 @@ enum nsCursor {   ///(normal cursor,       usually rendered as an arrow)
 
 enum nsContentType {
   eContentTypeInherit = -1,
-  eContentTypeUI = 0,         // eContentTypeUI must equal 0
-  eContentTypeContent = 1     // eContentTypeUI must equal 1
+  eContentTypeUI = 0,
+  eContentTypeContent = 1,
+  eContentTypeContentFrame = 2
 };
 
 enum nsTopLevelWidgetZPlacement { // for PlaceBehind()
@@ -254,7 +260,7 @@ class nsIWidget : public nsISupports {
 
   public:
 
-    NS_DEFINE_STATIC_IID_ACCESSOR(NS_IWIDGET_IID)
+    NS_DECLARE_STATIC_IID_ACCESSOR(NS_IWIDGET_IID)
 
     nsIWidget()
       : mLastChild(nsnull)
@@ -645,6 +651,18 @@ class nsIWidget : public nsISupports {
 
     NS_IMETHOD SetCursor(nsCursor aCursor) = 0;
 
+    /**
+     * Sets an image as the cursor for this widget.
+     *
+     * @param aCursor the cursor to set
+     * @param aX the X coordinate of the hotspot (from left).
+     * @param aY the Y coordinate of the hotspot (from top).
+     * @retval NS_ERROR_NOT_IMPLEMENTED if setting images as cursors is not
+     *         supported
+     */
+    NS_IMETHOD SetCursor(imgIContainer* aCursor,
+                         PRUint32 aHotspotX, PRUint32 aHotspotY) = 0;
+
     /** 
      * Get the window type of this widget
      *
@@ -667,6 +685,7 @@ class nsIWidget : public nsISupports {
      * value for all pixels is 1, i.e., opaque.
      * If the window is resized then the alpha channel values for
      * all pixels are reset to 1.
+     * Pixel RGB color values are already premultiplied with alpha channel values.
      * @param aTranslucent true if the window may have translucent
      *   or transparent pixels
      */
@@ -684,6 +703,7 @@ class nsIWidget : public nsISupports {
      * Update the alpha channel for some pixels of the top-level window
      * that contains this widget.
      * The window must have been made translucent using SetWindowTranslucency.
+     * Pixel RGB color values are already premultiplied with alpha channel values.
      * @param aRect the rect to update
      * @param aAlphas the alpha values, in w x h array, row-major order,
      * in units of 1/255. nsBlender::GetAlphas is a good way to compute this array.
@@ -834,7 +854,6 @@ class nsIWidget : public nsISupports {
     virtual void FreeNativeData(void * data, PRUint32 aDataType) = 0;//~~~
     virtual nsIRenderingContext* GetRenderingContext() = 0;
     virtual nsIDeviceContext* GetDeviceContext() = 0;
-    virtual nsIAppShell *GetAppShell() = 0;
     //@}
 
     /**
@@ -936,14 +955,6 @@ class nsIWidget : public nsISupports {
      */
     NS_IMETHOD DispatchEvent(nsGUIEvent* event, nsEventStatus & aStatus) = 0;
 
-
-    /**
-     * For printing and lightweight widgets
-     *
-     */
-    NS_IMETHOD Paint(nsIRenderingContext& aRenderingContext,
-                     const nsRect& aDirtyRect) = 0;
-   
     /**
      * Enables the dropping of files to a widget (XXX this is temporary)
      *
@@ -960,16 +971,9 @@ class nsIWidget : public nsISupports {
     NS_IMETHOD CaptureMouse(PRBool aCapture) = 0;
 
     /**
-     * Gets the window class
-     * implemented in gtk
+     * Classify the window for the window manager. Mostly for X11.
      */
-    NS_IMETHOD GetWindowClass(char *aClass) = 0;
-
-    /**
-     * Sets the window class
-     * implemented in gtk
-     */
-    NS_IMETHOD SetWindowClass(char *aClass) = 0;
+    NS_IMETHOD SetWindowClass(const nsAString& xulWinType) = 0;
 
     /**
      * Enables/Disables system capture of any and all events that would cause a
@@ -1019,6 +1023,12 @@ class nsIWidget : public nsISupports {
      */
     NS_IMETHOD GetLastInputEventTime(PRUint32& aTime) = 0;
 
+#ifdef MOZ_CAIRO_GFX
+    /**
+     * Get the Thebes surface associated with this widget.
+     */
+    virtual gfxASurface *GetThebesSurface() = 0;
+#endif
 
 protected:
     // keep the list of children.  We also keep track of our siblings.
@@ -1033,4 +1043,6 @@ protected:
     nsIWidget* mPrevSibling;
 };
 
+NS_DEFINE_STATIC_IID_ACCESSOR(nsIWidget, NS_IWIDGET_IID)
+
 #endif // nsIWidget_h__
