@@ -96,7 +96,7 @@ PRBool nsWidget::OnKey(nsKeyEvent &aEvent)
     // before we dispatch a key, check if it's the context menu key.
     // If so, send a context menu key event instead.
     if (IsContextMenuKey(aEvent)) {
-      nsMouseEvent contextMenuEvent;
+      nsMouseEvent contextMenuEvent(PR_TRUE, 0, nsnull, nsMouseEvent::eReal);
       ConvertKeyEventToContextMenuEvent(&aEvent, &contextMenuEvent);
       ret = DispatchWindowEvent(&contextMenuEvent);
     }
@@ -424,10 +424,7 @@ nsWidget::OnDestroySignal(GtkWidget* aGtkWidget)
 
 nsIWidget* nsWidget::GetParent(void)
 {
-  nsIWidget *ret;
-  ret = mParent;
-  NS_IF_ADDREF(ret);
-  return ret;
+  return mParent;
 }
 
 //-------------------------------------------------------------------------
@@ -493,6 +490,44 @@ NS_IMETHODIMP nsWidget::CaptureRollupEvents(nsIRollupListener * aListener, PRBoo
   return NS_OK;
 }
 
+GdkWindow* nsWidget::GetLayeringWindow()
+{
+  return mWidget->window;
+}
+
+void nsWidget::ResetZOrder()
+{
+    if (!GetNextSibling()) {
+        GdkWindow* window = GetLayeringWindow();
+        if (window) {
+            gdk_window_raise(window);
+        }
+    } else {
+        // Move this widget and all the widgets above it to the top, in
+        // the right order
+        for (nsWidget* w = this; w;
+             w = NS_STATIC_CAST(nsWidget*, w->GetNextSibling())) {
+            GdkWindow* window = w->GetLayeringWindow();
+            if (window) {
+                gdk_window_raise(window);
+            }
+        }
+    }
+}
+
+NS_IMETHODIMP nsWidget::SetZIndex(PRInt32 aZIndex)
+{
+    nsIWidget* oldPrev = GetPrevSibling();
+
+    nsBaseWidget::SetZIndex(aZIndex);
+
+    if (GetPrevSibling() != oldPrev) {
+        ResetZOrder();
+    }
+
+    return NS_OK;
+}
+
 NS_IMETHODIMP nsWidget::IsVisible(PRBool &aState)
 {
   if (mWidget)
@@ -640,15 +675,15 @@ PRBool nsWidget::OnResize(nsSizeEvent *event)
 
 PRBool nsWidget::OnResize(nsRect &aRect)
 {
-  nsSizeEvent event(NS_SIZE, this);
+  nsSizeEvent event(PR_TRUE, NS_SIZE, this);
 
   InitEvent(event);
 
   nsRect *foo = new nsRect(0, 0, aRect.width, aRect.height);
   event.windowSize = foo;
 
-  event.point.x = 0;
-  event.point.y = 0;
+  event.refPoint.x = 0;
+  event.refPoint.y = 0;
   event.mWinWidth = aRect.width;
   event.mWinHeight = aRect.height;
   
@@ -680,10 +715,10 @@ PRBool nsWidget::OnMove(PRInt32 aX, PRInt32 aY)
 
     ResetInternalVisibility();
 
-    nsGUIEvent event(NS_MOVE, this);
+    nsGUIEvent event(PR_TRUE, NS_MOVE, this);
     InitEvent(event);
-    event.point.x = aX;
-    event.point.y = aY;
+    event.refPoint.x = aX;
+    event.refPoint.y = aY;
     PRBool result = DispatchWindowEvent(&event);
     return result;
 }
@@ -1113,7 +1148,7 @@ NS_IMETHODIMP nsWidget::InvalidateRegion(const nsIRegion *aRegion, PRBool aIsSyn
     }
   }
 
-  // drop the const.. whats the right thing to do here?
+  // drop the const.. what's the right thing to do here?
   ((nsIRegion*)aRegion)->FreeRects(regionRectSet);
 
   return NS_OK;
@@ -1369,40 +1404,24 @@ void nsWidget::InitEvent(nsGUIEvent& event, nsPoint* aPoint)
 
     if (ge != nsnull) {
       //       ::ScreenToClient(mWnd, &cpos);
-      event.point.x = PRInt32(ge->configure.x);
-      event.point.y = PRInt32(ge->configure.y);
+      event.refPoint.x = PRInt32(ge->configure.x);
+      event.refPoint.y = PRInt32(ge->configure.y);
     }  
   }    
   else {                      // use the point override if provided
-    event.point.x = aPoint->x;
-    event.point.y = aPoint->y;
+    event.refPoint.x = aPoint->x;
+    event.refPoint.y = aPoint->y;
   }
 
   event.time = gdk_event_get_time(ge);
 
-  //    mLastPoint.x = event.point.x;
-  //    mLastPoint.y = event.point.y;
+  //    mLastPoint.x = event.refPoint.x;
+  //    mLastPoint.y = event.refPoint.y;
 
   if (ge)
     gdk_event_free(ge);
 }
 
-PRBool nsWidget::ConvertStatus(nsEventStatus aStatus)
-{
-  switch(aStatus) {
-  case nsEventStatus_eIgnore:
-    return(PR_FALSE);
-  case nsEventStatus_eConsumeNoDefault:
-    return(PR_TRUE);
-  case nsEventStatus_eConsumeDoDefault:
-    return(PR_FALSE);
-  default:
-    NS_ASSERTION(0, "Illegal nsEventStatus enumeration value");
-    break;
-  }
-  return PR_FALSE;
-}
-
 PRBool nsWidget::DispatchWindowEvent(nsGUIEvent* event)
 {
   nsEventStatus status;
@@ -1418,7 +1437,7 @@ PRBool nsWidget::DispatchWindowEvent(nsGUIEvent* event)
 
 PRBool nsWidget::DispatchStandardEvent(PRUint32 aMsg)
 {
-  nsGUIEvent event(aMsg, this);
+  nsGUIEvent event(PR_TRUE, aMsg, this);
   InitEvent(event);
   PRBool result = DispatchWindowEvent(&event);
   return result;
@@ -1555,7 +1574,7 @@ PRBool nsWidget::DispatchMouseEvent(nsMouseEvent& aEvent)
 //         result = ConvertStatus(mMouseListener->MouseMoved(aEvent));
 //         nsRect rect;
 //         GetBounds(rect);
-//         if (rect.Contains(event.point.x, event.point.y)) {
+//         if (rect.Contains(event.refPoint.x, event.refPoint.y)) {
 //           if (mCurrentWindow == NULL || mCurrentWindow != this) {
 //             printf("Mouse enter");
 //             mCurrentWindow = this;
@@ -1689,7 +1708,7 @@ nsWidget::OnMotionNotifySignal(GdkEventMotion * aGdkMotionEvent)
   if (mIsDestroying)
     return;
 
-  nsMouseEvent event(NS_MOUSE_MOVE);
+  nsMouseEvent event(PR_TRUE, NS_MOUSE_MOVE, nsnull, nsMouseEvent::eReal);
 
   // If there is a button motion target, use that instead of the
   // current widget
@@ -1718,8 +1737,8 @@ nsWidget::OnMotionNotifySignal(GdkEventMotion * aGdkMotionEvent)
     x = (gint) aGdkMotionEvent->x;
     y = (gint) aGdkMotionEvent->y;
 
-    event.point.x = nscoord(x);
-    event.point.y = nscoord(y);
+    event.refPoint.x = nscoord(x);
+    event.refPoint.y = nscoord(y);
     event.widget = this;
   }
 
@@ -1738,8 +1757,8 @@ nsWidget::OnMotionNotifySignal(GdkEventMotion * aGdkMotionEvent)
       
       // The event coords will be the initial *widget* coords plus the 
       // root difference computed above.
-      event.point.x = nscoord(sButtonMotionWidgetX + diffX);
-      event.point.y = nscoord(sButtonMotionWidgetY + diffY);
+      event.refPoint.x = nscoord(sButtonMotionWidgetX + diffX);
+      event.refPoint.y = nscoord(sButtonMotionWidgetY + diffY);
     }
   }
   else
@@ -1784,12 +1803,12 @@ nsWidget::OnEnterNotifySignal(GdkEventCrossing * aGdkCrossingEvent)
     return;
   }
 
-  nsMouseEvent event(NS_MOUSE_ENTER, this);
+  nsMouseEvent event(PR_TRUE, NS_MOUSE_ENTER, this, nsMouseEvent::eReal);
 
   if (aGdkCrossingEvent != NULL) 
   {
-    event.point.x = nscoord(aGdkCrossingEvent->x);
-    event.point.y = nscoord(aGdkCrossingEvent->y);
+    event.refPoint.x = nscoord(aGdkCrossingEvent->x);
+    event.refPoint.y = nscoord(aGdkCrossingEvent->y);
     event.time = aGdkCrossingEvent->time;
   }
 
@@ -1818,12 +1837,12 @@ nsWidget::OnLeaveNotifySignal(GdkEventCrossing * aGdkCrossingEvent)
     return;
   }
 
-  nsMouseEvent event(NS_MOUSE_EXIT, this);
+  nsMouseEvent event(PR_TRUE, NS_MOUSE_EXIT, this, nsMouseEvent::eReal);
 
   if (aGdkCrossingEvent != NULL) 
   {
-    event.point.x = nscoord(aGdkCrossingEvent->x);
-    event.point.y = nscoord(aGdkCrossingEvent->y);
+    event.refPoint.x = nscoord(aGdkCrossingEvent->x);
+    event.refPoint.y = nscoord(aGdkCrossingEvent->y);
     event.time = aGdkCrossingEvent->time;
   }
 
@@ -1839,7 +1858,7 @@ nsWidget::OnLeaveNotifySignal(GdkEventCrossing * aGdkCrossingEvent)
 /* virtual */ void
 nsWidget::OnButtonPressSignal(GdkEventButton * aGdkButtonEvent)
 {
-  nsMouseScrollEvent scrollEvent(NS_MOUSE_SCROLL, this);
+  nsMouseScrollEvent scrollEvent(PR_TRUE, NS_MOUSE_SCROLL, this);
   PRUint32 eventType = 0;
 
   // If you double click in GDK, it will actually generate a single
@@ -1895,8 +1914,8 @@ nsWidget::OnButtonPressSignal(GdkEventButton * aGdkButtonEvent)
       else
         scrollEvent.delta = 3;
 
-      scrollEvent.point.x = nscoord(aGdkButtonEvent->x);
-      scrollEvent.point.y = nscoord(aGdkButtonEvent->y);
+      scrollEvent.refPoint.x = nscoord(aGdkButtonEvent->x);
+      scrollEvent.refPoint.y = nscoord(aGdkButtonEvent->y);
       
       scrollEvent.isShift = (aGdkButtonEvent->state & GDK_SHIFT_MASK) ? PR_TRUE : PR_FALSE;
       scrollEvent.isControl = (aGdkButtonEvent->state & GDK_CONTROL_MASK) ? PR_TRUE : PR_FALSE;
@@ -1920,7 +1939,7 @@ nsWidget::OnButtonPressSignal(GdkEventButton * aGdkButtonEvent)
     break;
   }
 
-  nsMouseEvent event(eventType, this);
+  nsMouseEvent event(PR_TRUE, eventType, this, nsMouseEvent::eReal);
   InitMouseEvent(aGdkButtonEvent, event);
 
   // Set the button motion target and remeber the widget and root coords
@@ -1946,7 +1965,8 @@ nsWidget::OnButtonPressSignal(GdkEventButton * aGdkButtonEvent)
   // if we're a right-button-down on linux, we're trying to
   // popup a context menu. send that event to gecko also.
   if (eventType == NS_MOUSE_RIGHT_BUTTON_DOWN) {
-    nsMouseEvent contextMenuEvent(NS_CONTEXTMENU, this);
+    nsMouseEvent contextMenuEvent(PR_TRUE, NS_CONTEXTMENU, this,
+                                  nsMouseEvent::eReal);
     InitMouseEvent(aGdkButtonEvent, contextMenuEvent);
     DispatchMouseEvent(contextMenuEvent);
   }
@@ -1987,7 +2007,7 @@ nsWidget::OnButtonReleaseSignal(GdkEventButton * aGdkButtonEvent)
     break;
 	}
 
-  nsMouseEvent event(eventType, this);
+  nsMouseEvent event(PR_TRUE, eventType, this, nsMouseEvent::eReal);
   InitMouseEvent(aGdkButtonEvent, event);
 
   if (sButtonMotionTarget) {
@@ -2000,8 +2020,8 @@ nsWidget::OnButtonReleaseSignal(GdkEventButton * aGdkButtonEvent)
     event.widget = sButtonMotionTarget;
 
     // see comments in nsWidget::OnMotionNotifySignal
-    event.point.x = nscoord(sButtonMotionWidgetX + diffX);
-    event.point.y = nscoord(sButtonMotionWidgetY + diffY);
+    event.refPoint.x = nscoord(sButtonMotionWidgetX + diffX);
+    event.refPoint.y = nscoord(sButtonMotionWidgetY + diffY);
   }
 
   // Drop the motion target before dispatching the event so that we
@@ -2026,7 +2046,7 @@ nsWidget::OnFocusInSignal(GdkEventFocus * aGdkFocusEvent)
 
   GTK_WIDGET_SET_FLAGS(mWidget, GTK_HAS_FOCUS);
 
-  nsFocusEvent event(NS_GOTFOCUS, this);
+  nsFocusEvent event(PR_TRUE, NS_GOTFOCUS, this);
 
 //  event.time = aGdkFocusEvent->time;;
 //  event.time = PR_Now();
@@ -2046,7 +2066,7 @@ nsWidget::OnFocusOutSignal(GdkEventFocus * aGdkFocusEvent)
 
   GTK_WIDGET_UNSET_FLAGS(mWidget, GTK_HAS_FOCUS);
 
-  nsFocusEvent event(NS_LOSTFOCUS, this);
+  nsFocusEvent event(PR_TRUE, NS_LOSTFOCUS, this);
 
 //  event.time = aGdkFocusEvent->time;;
 //  event.time = PR_Now();
@@ -2090,11 +2110,11 @@ nsWidget::InstallSignal(GtkWidget *   aWidget,
 //////////////////////////////////////////////////////////////////
 void 
 nsWidget::InitMouseEvent(GdkEventButton * aGdkButtonEvent,
-						 nsMouseEvent &anEvent)
+                         nsMouseEvent &anEvent)
 {
   if (aGdkButtonEvent != NULL) {
-    anEvent.point.x = nscoord(aGdkButtonEvent->x);
-    anEvent.point.y = nscoord(aGdkButtonEvent->y);
+    anEvent.refPoint.x = nscoord(aGdkButtonEvent->x);
+    anEvent.refPoint.y = nscoord(aGdkButtonEvent->y);
 
     anEvent.isShift = (aGdkButtonEvent->state & GDK_SHIFT_MASK) ? PR_TRUE : PR_FALSE;
     anEvent.isControl = (aGdkButtonEvent->state & GDK_CONTROL_MASK) ? PR_TRUE : PR_FALSE;
@@ -2355,6 +2375,18 @@ NS_IMETHODIMP nsWidget::GetIMEOpenState(PRBool* aState) {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+NS_IMETHODIMP nsWidget::SetIMEEnabled(PRBool aState) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP nsWidget::GetIMEEnabled(PRBool* aState) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP nsWidget::CancelIMEComposition() {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 /* virtual */
 GtkWindow *nsWidget::GetTopLevelWindow(void)
 {
