@@ -133,7 +133,10 @@ nsDragService::Observe(nsISupports *aSubject, const char *aTopic,
   if (!nsCRT::strcmp(aTopic, "quit-application")) {
     PR_LOG(sDragLm, PR_LOG_DEBUG,
            ("nsDragService::Observe(\"quit-application\")"));
-    gtk_widget_destroy(mHiddenWidget);
+    if (mHiddenWidget) {
+      gtk_widget_destroy(mHiddenWidget);
+      mHiddenWidget = 0;
+    }
     TargetResetData();
   } else {
     NS_NOTREACHED("unexpected topic");
@@ -790,7 +793,7 @@ nsDragService::IsTargetContextList(void)
     return retval;
 }
 
-// Maximum time to wait for a "drag_recieved" arrived, in microseconds
+// Maximum time to wait for a "drag_received" arrived, in microseconds
 #define NS_DND_TIMEOUT 500000
 
 void
@@ -847,14 +850,14 @@ nsDragService::GetSourceList(void)
         // gtk's implementation?), we don't advertise all flavours listed
         // in the nsITransferable.
 
-        // the aplication/x-moz-internal-item-list format, which preserves
+        // the application/x-moz-internal-item-list format, which preserves
         // all information for drags within the same mozilla instance.
         GdkAtom listAtom = gdk_atom_intern(gMimeListType, FALSE);
         GtkTargetEntry *listTarget =
             (GtkTargetEntry *)g_malloc(sizeof(GtkTargetEntry));
         listTarget->target = g_strdup(gMimeListType);
         listTarget->flags = 0;
-        listTarget->info = (guint)listAtom;
+        listTarget->info = GPOINTER_TO_UINT(listAtom);
         PR_LOG(sDragLm, PR_LOG_DEBUG,
                ("automatically adding target %s with id %ld\n",
                listTarget->target, listAtom));
@@ -893,7 +896,7 @@ nsDragService::GetSourceList(void)
                              (GtkTargetEntry *)g_malloc(sizeof(GtkTargetEntry));
                             listTarget->target = g_strdup(gTextUriListType);
                             listTarget->flags = 0;
-                            listTarget->info = (guint)listAtom;
+                            listTarget->info = GPOINTER_TO_UINT(listAtom);
                             PR_LOG(sDragLm, PR_LOG_DEBUG,
                                    ("automatically adding target %s with \
                                    id %ld\n", listTarget->target, listAtom));
@@ -930,7 +933,7 @@ nsDragService::GetSourceList(void)
                           (GtkTargetEntry *)g_malloc(sizeof(GtkTargetEntry));
                         target->target = g_strdup(flavorStr);
                         target->flags = 0;
-                        target->info = (guint)atom;
+                        target->info = GPOINTER_TO_UINT(atom);
                         PR_LOG(sDragLm, PR_LOG_DEBUG,
                                ("adding target %s with id %ld\n",
                                target->target, atom));
@@ -947,7 +950,7 @@ nsDragService::GetSourceList(void)
                              (GtkTargetEntry *)g_malloc(sizeof(GtkTargetEntry));
                             plainTarget->target = g_strdup(kTextMime);
                             plainTarget->flags = 0;
-                            plainTarget->info = (guint)plainAtom;
+                            plainTarget->info = GPOINTER_TO_UINT(plainAtom);
                             PR_LOG(sDragLm, PR_LOG_DEBUG,
                                    ("automatically adding target %s with \
                                    id %ld\n", plainTarget->target, plainAtom));
@@ -964,7 +967,7 @@ nsDragService::GetSourceList(void)
                              (GtkTargetEntry *)g_malloc(sizeof(GtkTargetEntry));
                             urlTarget->target = g_strdup(gMozUrlType);
                             urlTarget->flags = 0;
-                            urlTarget->info = (guint)urlAtom;
+                            urlTarget->info = GPOINTER_TO_UINT(urlAtom);
                             PR_LOG(sDragLm, PR_LOG_DEBUG,
                                    ("automatically adding target %s with \
                                    id %ld\n", urlTarget->target, urlAtom));
@@ -1009,6 +1012,9 @@ nsDragService::SourceEndDrag(void)
 {
     // this just releases the list of data items that we provide
     mSourceDataItems = 0;
+
+    // Inform the drag session that we're ending the drag.
+    EndDragSession();
 }
 
 static void
