@@ -41,6 +41,8 @@
 #include "nsLookAndFeel.h"
 #include <gtk/gtkinvisible.h>
 
+#include "gtkdrawing.h"
+
 #define GDK_COLOR_TO_NS_RGB(c) \
     ((nscolor) NS_RGB(c.red>>8, c.green>>8, c.blue>>8))
 
@@ -78,17 +80,16 @@ nsLookAndFeel::~nsLookAndFeel()
 nsresult nsLookAndFeel::NativeGetColor(const nsColorID aID, nscolor& aColor)
 {
     nsresult res = NS_OK;
-    aColor = 0; // default color black
 
     switch (aID) {
         // These colors don't seem to be used for anything anymore in Mozilla
         // (except here at least TextSelectBackground and TextSelectForeground)
         // The CSS2 colors below are used.
     case eColor_WindowBackground:
-        aColor = GDK_COLOR_TO_NS_RGB(mStyle->bg[GTK_STATE_NORMAL]);
+        aColor = GDK_COLOR_TO_NS_RGB(mStyle->base[GTK_STATE_NORMAL]);
         break;
     case eColor_WindowForeground:
-        aColor = GDK_COLOR_TO_NS_RGB(mStyle->fg[GTK_STATE_NORMAL]);
+        aColor = GDK_COLOR_TO_NS_RGB(mStyle->text[GTK_STATE_NORMAL]);
         break;
     case eColor_WidgetBackground:
         aColor = GDK_COLOR_TO_NS_RGB(mStyle->bg[GTK_STATE_NORMAL]);
@@ -117,13 +118,33 @@ nsresult nsLookAndFeel::NativeGetColor(const nsColorID aID, nscolor& aColor)
         aColor = GDK_COLOR_TO_NS_RGB(mStyle->text[GTK_STATE_NORMAL]);
         break;
     case eColor_TextSelectBackground:
+    case eColor_IMESelectedRawTextBackground:
+    case eColor_IMESelectedConvertedTextBackground:
         // still used
         aColor = GDK_COLOR_TO_NS_RGB(mStyle->base[GTK_STATE_SELECTED]);
         break;
     case eColor_TextSelectForeground:
+    case eColor_IMESelectedRawTextForeground:
+    case eColor_IMESelectedConvertedTextForeground:
         // still used
         aColor = GDK_COLOR_TO_NS_RGB(mStyle->text[GTK_STATE_SELECTED]);
         break;
+    case eColor_IMERawInputBackground:
+    case eColor_IMEConvertedTextBackground:
+        aColor = NS_TRANSPARENT;
+        break;
+    case eColor_IMERawInputForeground:
+    case eColor_IMEConvertedTextForeground:
+        aColor = NS_SAME_AS_FOREGROUND_COLOR;
+        break;
+    case eColor_IMERawInputUnderline:
+    case eColor_IMEConvertedTextUnderline:
+        aColor = NS_SAME_AS_FOREGROUND_COLOR;
+        break;
+    case eColor_IMESelectedRawTextUnderline:
+    case eColor_IMESelectedConvertedTextUnderline:
+        aColor = NS_TRANSPARENT;
+        break;
 
         // css2  http://www.w3.org/TR/REC-CSS2/ui.html#system-colors
     case eColor_activeborder:
@@ -285,10 +306,45 @@ nsresult nsLookAndFeel::NativeGetColor(const nsColorID aID, nscolor& aColor)
     return res;
 }
 
+static PRInt32 CheckWidgetStyle(GtkWidget* aWidget, const char* aStyle, PRInt32 aMetric) {
+    gboolean value = PR_FALSE;
+    gtk_widget_style_get(aWidget, aStyle, &value, NULL);
+    return value ? aMetric : 0;
+}
+
+static PRInt32 ConvertGTKStepperStyleToMozillaScrollArrowStyle(GtkWidget* aWidget)
+{
+    if (!aWidget)
+        return nsILookAndFeel::eMetric_ScrollArrowStyleSingle;
+  
+    return
+        CheckWidgetStyle(aWidget, "has-backward-stepper",
+                         nsILookAndFeel::eMetric_ScrollArrowStartBackward) |
+        CheckWidgetStyle(aWidget, "has-forward-stepper",
+                         nsILookAndFeel::eMetric_ScrollArrowEndForward) |
+        CheckWidgetStyle(aWidget, "has-secondary-backward-stepper",
+                         nsILookAndFeel::eMetric_ScrollArrowEndBackward) |
+        CheckWidgetStyle(aWidget, "has-secondary-forward-stepper",
+                         nsILookAndFeel::eMetric_ScrollArrowStartForward);
+}
+
 NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
 {
     nsresult res = NS_OK;
 
+    // Set these before they can get overrided in the nsXPLookAndFeel. 
+    switch (aID) {
+    case eMetric_ScrollButtonLeftMouseButtonAction:
+        aMetric = 0;
+        return NS_OK;
+    case eMetric_ScrollButtonMiddleMouseButtonAction:
+        aMetric = 1;
+        return NS_OK;
+    case eMetric_ScrollButtonRightMouseButtonAction:
+        aMetric = 2;
+        return NS_OK;
+    }
+
     res = nsXPLookAndFeel::GetMetric(aID, aMetric);
     if (NS_SUCCEEDED(res))
         return res;
@@ -363,8 +419,23 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
         aMetric = 1;
         break;
     case eMetric_CaretBlinkTime:
-        aMetric = 500;
-        break;
+        {
+            GtkSettings *settings;
+            gint blink_time;
+            gboolean blink;
+
+            settings = gtk_settings_get_default ();
+            g_object_get (settings,
+                          "gtk-cursor-blink-time", &blink_time,
+                          "gtk-cursor-blink", &blink,
+                          NULL);
+ 
+            if (blink)
+                aMetric = (PRInt32) blink_time;
+            else
+                aMetric = 0;
+            break;
+        }
     case eMetric_CaretWidth:
         aMetric = 1;
         break;
@@ -372,17 +443,46 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
         aMetric = 0;
         break;
     case eMetric_SelectTextfieldsOnKeyFocus:
-        // Select textfield content when focused by kbd
-        // used by nsEventStateManager::sTextfieldSelectModel
-        aMetric = 1;
+        {
+            GtkWidget *entry;
+            GtkSettings *settings;
+            gboolean select_on_focus;
+
+            entry = gtk_entry_new();
+            gtk_widget_ref(entry);
+            gtk_object_sink(GTK_OBJECT(entry));
+            settings = gtk_widget_get_settings(entry);
+            g_object_get(settings, 
+                         "gtk-entry-select-on-focus",
+                         &select_on_focus,
+                         NULL);
+            
+            if(select_on_focus)
+                aMetric = 1;
+            else
+                aMetric = 0;
+
+            gtk_widget_destroy(entry);
+            gtk_widget_unref(entry);
+        }
         break;
     case eMetric_SubmenuDelay:
-        aMetric = 200;
-        break;
+        {
+            GtkSettings *settings;
+            gint delay;
+
+            settings = gtk_settings_get_default ();
+            g_object_get (settings, "gtk-menu-popup-delay", &delay, NULL);
+            aMetric = (PRInt32) delay;
+            break;
+        }
     case eMetric_MenusCanOverlapOSBar:
         // we want XUL popups to be able to overlap the task bar.
         aMetric = 1;
         break;
+    case eMetric_SkipNavigatingDisabledMenuItem:
+        aMetric = 1;
+        break;
     case eMetric_DragFullWindow:
         aMetric = 1;
         break;
@@ -394,16 +494,25 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
             g_object_get(gtk_widget_get_settings(box),
                          "gtk-dnd-drag-threshold", &threshold,
                          NULL);
-            gtk_widget_destroy(box);
+            gtk_object_sink(GTK_OBJECT(box));
             aMetric = threshold;
         }
         break;
     case eMetric_ScrollArrowStyle:
-        aMetric = eMetric_ScrollArrowStyleSingle;
+        aMetric =
+            ConvertGTKStepperStyleToMozillaScrollArrowStyle(moz_gtk_get_scrollbar_widget());
         break;
     case eMetric_ScrollSliderStyle:
         aMetric = eMetric_ScrollThumbStyleProportional;
         break;
+    case eMetric_ScrollbarWidth:
+    case eMetric_ScrollbarHeight:
+        {
+            MozGtkScrollbarMetrics metrics;
+            moz_gtk_get_scrollbar_metrics(&metrics);
+            aMetric = metrics.stepper_size;
+        }
+        break;
     case eMetric_TreeOpenDelay:
         aMetric = 1000;
         break;
@@ -462,6 +571,9 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricFloatID aID,
     case eMetricFloat_ButtonHorizontalInsidePadding:
         aMetric = 0.25f;
         break;
+    case eMetricFloat_IMEUnderlineRelativeSize:
+        aMetric = 1.0f;
+        break;
     default:
         aMetric = -1.0;
         res = NS_ERROR_FAILURE;
