@@ -14,13 +14,14 @@
  * for the specific language governing rights and limitations under the
  * License.
  *
- * The Original Code mozilla.org code.
+ * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code Christopher Blizzard
+ * The Initial Developer of the Original Code is Christopher Blizzard
  * <blizzard@mozilla.org>.  Portions created by the Initial Developer
  * are Copyright (C) 2001 the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Masayuki Nakano <masayuki@d-toybox.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -38,6 +39,8 @@
 
 #ifndef __nsWindow_h__
 
+#include "nsAutoPtr.h"
+
 #include "nsCommonWidget.h"
 
 #include "mozcontainer.h"
@@ -60,9 +63,14 @@
 #ifdef USE_XIM
 #include <gtk/gtkimmulticontext.h>
 #include "pldhash.h"
+#include "nsIKBStateControl.h"
 #endif
 
-class nsWindow : public nsCommonWidget, public nsSupportsWeakReference {
+class nsWindow : public nsCommonWidget, public nsSupportsWeakReference
+#ifdef USE_XIM
+                ,public nsIKBStateControl
+#endif
+{
 public:
     nsWindow();
     virtual ~nsWindow();
@@ -87,6 +95,7 @@ public:
                               nsIToolkit       *aToolkit,
                               nsWidgetInitData *aInitData);
     NS_IMETHOD         Destroy(void);
+    NS_IMETHOD         SetParent(nsIWidget* aNewParent);
     NS_IMETHOD         SetModal(PRBool aModal);
     NS_IMETHOD         IsVisible(PRBool & aState);
     NS_IMETHOD         ConstrainPosition(PRBool aAllowSlop,
@@ -97,6 +106,7 @@ public:
     NS_IMETHOD         PlaceBehind(nsTopLevelWidgetZPlacement  aPlacement,
                                    nsIWidget                  *aWidget,
                                    PRBool                      aActivate);
+    NS_IMETHOD         SetZIndex(PRInt32 aZIndex);
     NS_IMETHOD         SetSizeMode(PRInt32 aMode);
     NS_IMETHOD         Enable(PRBool aState);
     NS_IMETHOD         SetFocus(PRBool aRaise = PR_FALSE);
@@ -106,6 +116,8 @@ public:
     virtual            nsIFontMetrics* GetFont(void);
     NS_IMETHOD         SetFont(const nsFont &aFont);
     NS_IMETHOD         SetCursor(nsCursor aCursor);
+    NS_IMETHOD         SetCursor(imgIContainer* aCursor,
+                                 PRUint32 aHotspotX, PRUint32 aHotspotY);
     NS_IMETHOD         Validate();
     NS_IMETHOD         Invalidate(PRBool aIsSynchronous);
     NS_IMETHOD         Invalidate(const nsRect &aRect,
@@ -126,6 +138,7 @@ public:
     NS_IMETHOD         SetBorderStyle(nsBorderStyle aBorderStyle);
     NS_IMETHOD         SetTitle(const nsAString& aTitle);
     NS_IMETHOD         SetIcon(const nsAString& aIconSpec);
+    NS_IMETHOD         SetWindowClass(const nsAString& xulWinType);
     NS_IMETHOD         SetMenuBar(nsIMenuBar * aMenuBar);
     NS_IMETHOD         ShowMenuBar(PRBool aShow);
     NS_IMETHOD         WidgetToScreen(const nsRect& aOldRect,
@@ -143,6 +156,7 @@ public:
                                            PRBool aDoCapture,
                                            PRBool aConsumeRollupEvent);
     NS_IMETHOD         GetAttention(PRInt32 aCycleCount);
+    NS_IMETHOD         MakeFullScreen(PRBool aFullScreen);
     NS_IMETHOD         HideWindowChrome(PRBool aShouldHide);
 
     // utility methods
@@ -251,8 +265,11 @@ public:
     Window             mOldFocusWindow;
 
     static guint32     mLastButtonPressTime;
+    static guint32     mLastButtonReleaseTime;
 
 #ifdef USE_XIM
+    void               IMEInitData       (void);
+    void               IMEReleaseData    (void);
     void               IMEDestroyContext (void);
     void               IMESetFocus       (void);
     void               IMELoseFocus      (void);
@@ -260,14 +277,65 @@ public:
     void               IMEComposeText    (const PRUnichar *aText,
                                           const PRInt32 aLen,
                                           const gchar *aPreeditString,
+                                          const gint aCursorPos,
                                           const PangoAttrList *aFeedback);
     void               IMEComposeEnd     (void);
     GtkIMContext*      IMEGetContext     (void);
+    nsWindow*          IMEGetOwningWindow(void);
+    PRBool             IMEIsEnabled      (void);
+    nsWindow*          IMEComposingWindow(void);
     void               IMECreateContext  (void);
     PRBool             IMEFilterEvent    (GdkEventKey *aEvent);
 
-    GtkIMContext       *mIMContext;
-    PRBool             mComposingText;
+    /*
+     *  |mIMEData| has all IME data for the window and its children widgets.
+     *  Only stand-alone windows and child windows embedded in non-Mozilla GTK
+     *  containers own IME contexts.
+     *  But this is referred from all children after the widget gets focus.
+     *  The children refers to its owning window's object.
+     */
+    struct nsIMEData {
+        // Actual context. This is used for handling the user's input.
+        GtkIMContext       *mContext;
+        // mDummyContext is a dummy context and will be used in IMESetFocus()
+        // when mEnabled is false. This mDummyContext IM state is always
+        // "off", so it works to switch conversion mode to OFF on IM status
+        // window.
+        GtkIMContext       *mDummyContext;
+        // This mComposingWindow is set in IMEComposeStart(), when user starts
+        // composition, then unset in IMEComposeEnd() when user ends the
+        // composition. We will keep the widget where the actual composition is
+        // started. During the composition, we may get some events like
+        // ResetInputStateInternal() and CancelIMECompositionInternal() by
+        // changing input focus, we will use the original widget of
+        // mComposingWindow to commit or reset the composition.
+        nsWindow           *mComposingWindow;
+        // Owner of this struct.
+        // The owner window must release the contexts at destroying.
+        nsWindow           *mOwner;
+        // The reference counter. When this will be zero by the decrement,
+        // the decrementer must free the instance.
+        PRUint32           mRefCount;
+        // IME enabled state in this window.
+        PRPackedBool       mEnabled;
+        nsIMEData(nsWindow* aOwner) {
+            mContext         = nsnull;
+            mDummyContext    = nsnull;
+            mComposingWindow = nsnull;
+            mOwner           = aOwner;
+            mRefCount        = 1;
+            mEnabled         = PR_TRUE;
+        }
+    };
+    nsIMEData          *mIMEData;
+
+    // nsIKBStateControl interface
+    NS_IMETHOD ResetInputState();
+    NS_IMETHOD SetIMEOpenState(PRBool aState);
+    NS_IMETHOD GetIMEOpenState(PRBool* aState);
+    NS_IMETHOD SetIMEEnabled(PRBool aState);
+    NS_IMETHOD GetIMEEnabled(PRBool* aState);
+    NS_IMETHOD CancelIMEComposition();
 
 #endif
 
@@ -276,9 +344,19 @@ public:
 #ifdef MOZ_XUL
    NS_IMETHOD          SetWindowTranslucency(PRBool aTransparent);
    NS_IMETHOD          GetWindowTranslucency(PRBool& aTransparent);
+   nsresult            UpdateTranslucentWindowAlphaInternal(const nsRect& aRect,
+                                                            PRUint8* aAlphas, PRInt32 aStride);
    NS_IMETHOD          UpdateTranslucentWindowAlpha(const nsRect& aRect, PRUint8* aAlphas);
 #endif
 
+#ifdef MOZ_CAIRO_GFX
+    gfxASurface       *GetThebesSurface();
+#endif
+
+#ifdef ACCESSIBILITY
+    static PRBool      sAccessibilityEnabled;
+#endif
+
 private:
     void               GetToplevelWidget(GtkWidget **aWidget);
     void               GetContainerWindow(nsWindow  **aWindow);
@@ -304,6 +382,10 @@ private:
     PRInt32             mSizeState;
     PluginType          mPluginType;
 
+#ifdef MOZ_CAIRO_GFX
+    nsRefPtr<gfxASurface> mThebesSurface;
+#endif
+
 #ifdef ACCESSIBILITY
     nsCOMPtr<nsIAccessible> mRootAccessible;
     void                CreateRootAccessible();
