@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Mats Palmgren <mats.palmgren@bredband.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -53,14 +54,13 @@
 #include "nsMimeMapper.h"
 #include "nsClipboard.h"
 #include "nsIRegion.h"
-#include "nsVoidArray.h"
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsCOMPtr.h"
 #include "nsCRT.h"
 #include "nsPrimitiveHelpers.h"
-#include "nsWatchTask.h"
 #include "nsLinebreakConverter.h"
+#include "nsIMacUtils.h"
 
 #include "nsIContent.h"
 #include "nsIDOMNode.h"
@@ -72,24 +72,51 @@
 #include "nsRect.h"
 #include "nsPoint.h"
 #include "nsIWidget.h"
+#ifndef MOZ_WIDGET_COCOA
 #include "nsCarbonHelpers.h"
-#include "nsGfxUtils.h"
-
+#endif
 // file save stuff
 #include "nsNetUtil.h"
 #include "nsILocalFileMac.h"
 
-#ifdef MOZ_XUL
-#include "nsIXULContent.h"
-#endif
-
 #include "nsIDOMElement.h"
+#ifndef MOZ_CAIRO_GFX
 #include "nsIImageMac.h"
+#endif
 #include "nsIImage.h"
 #include "nsMacNativeUnicodeConverter.h"
 #include "nsICharsetConverterManager.h"
 #include "nsStylClipboardUtils.h"
 
+//
+// Get the primary frame for a content node in the current document.
+//
+static nsIFrame*
+GetPrimaryFrameFor(nsIContent* aContent)
+{
+  nsIFrame* result = nsnull;
+  nsIDocument* doc = aContent->GetCurrentDoc();
+  if (doc) {
+    nsIPresShell* presShell = doc->GetShellAt(0);
+    if (presShell) {
+      result = presShell->GetPrimaryFrameFor(aContent);
+    }
+  }
+  return result;
+}
+
+static const PRUint32 kPrivateFlavorMask = 0xffff0000;
+static const PRUint32 kPrivateFlavorTag = 'MZ..' & kPrivateFlavorMask;
+
+
+static void
+SetPortToKnownGoodPort()
+{
+    WindowPtr firstWindow = GetWindowList();
+    if (firstWindow)
+      ::SetGWorld(::GetWindowPort(firstWindow), ::GetMainDevice());
+}
+
 
 // we need our own stuff for MacOS because of nsIDragSessionMac.
 NS_IMPL_ADDREF_INHERITED(nsDragService, nsBaseDragService)
@@ -135,14 +162,15 @@ nsDragService::ComputeGlobalRectFromFrame ( nsIDOMNode* aDOMNode, Rect & outScre
 {
   NS_ASSERTION ( aDOMNode, "Oopps, no DOM node" );
 
+  nsCOMPtr<nsIContent> content = do_QueryInterface(aDOMNode);
+
 // this isn't so much of an issue as long as we're just dragging around outlines,
 // but it is when we are showing the text being drawn. Comment it out for now
 // but leave it around when we turn this all back on (pinkerton).
 #if USE_TRANSLUCENT_DRAGGING && defined(MOZ_XUL)
   // until bug 41237 is fixed, only do translucent dragging if the drag is in
   // the chrome or it's a link.
-  nsCOMPtr<nsIXULContent> xulContent ( do_QueryInterface(aDOMNode) );
-  if ( !xulContent ) {
+  if (!content || !content->IsNodeOfType(nsINode::eXUL)) {
     // the link node is the parent of the node we have (which is probably text or image).
     nsCOMPtr<nsIDOMNode> parent;
     aDOMNode->GetParentNode ( getter_AddRefs(parent) );
@@ -159,11 +187,12 @@ nsDragService::ComputeGlobalRectFromFrame ( nsIDOMNode* aDOMNode, Rect & outScre
   
   outScreenRect.left = outScreenRect.right = outScreenRect.top = outScreenRect.bottom = 0;
 
+  if (!content)
+    return PR_FALSE;
+
   // Get the frame for this content node (note: frames are not refcounted)
-  nsIFrame *aFrame = nsnull;
-  nsCOMPtr<nsPresContext> presContext;
-  GetFrameFromNode ( aDOMNode, &aFrame, getter_AddRefs(presContext) );
-  if ( !aFrame || !presContext )
+  nsIFrame *frame = ::GetPrimaryFrameFor(content);
+  if (!frame)
     return PR_FALSE;
   
   //
@@ -171,12 +200,12 @@ nsDragService::ComputeGlobalRectFromFrame ( nsIDOMNode* aDOMNode, Rect & outScre
   // coordinates.
   //
   
-  nsRect rect = aFrame->GetRect();
+  nsRect rect = frame->GetRect();
 
   // Find offset from our view
 	nsIView *containingView = nsnull;
 	nsPoint	viewOffset(0,0);
-	aFrame->GetOffsetFromView(presContext, viewOffset, &containingView);
+	frame->GetOffsetFromView(viewOffset, &containingView);
   NS_ASSERTION(containingView, "No containing view!");
   if ( !containingView )
     return PR_FALSE;
@@ -185,8 +214,7 @@ nsDragService::ComputeGlobalRectFromFrame ( nsIDOMNode* aDOMNode, Rect & outScre
   nsPoint widgetOffset;
   nsIWidget* aWidget = containingView->GetNearestWidget ( &widgetOffset );
 
-  float t2p = 1.0;
-  t2p = presContext->TwipsToPixels();
+  float t2p = frame->GetPresContext()->TwipsToPixels();
 
   // Shift our offset rect by offset into our view, and
   // the view's offset to the closest widget. Then convert that to global coordinates.
@@ -208,7 +236,7 @@ nsDragService::ComputeGlobalRectFromFrame ( nsIDOMNode* aDOMNode, Rect & outScre
 
 
 //
-// StartDragSession
+// InvokeDragSession
 //
 // Do all the work to kick it off.
 //
@@ -216,7 +244,7 @@ NS_IMETHODIMP
 nsDragService::InvokeDragSession (nsIDOMNode *aDOMNode, nsISupportsArray * aTransferableArray, nsIScriptableRegion * aDragRgn, PRUint32 aActionType)
 {
 #ifdef MOZ_WIDGET_COCOA
-  nsGraphicsUtils::SetPortToKnownGoodPort();
+  SetPortToKnownGoodPort();
   GrafPtr port;
   GDHandle handle;
   ::GetGWorld(&port, &handle);
@@ -298,7 +326,7 @@ nsDragService::InvokeDragSession (nsIDOMNode *aDOMNode, nsISupportsArray * aTran
 
   return NS_OK; 
 
-} // StartDragSession
+} // InvokeDragSession
 
 
 //
@@ -319,7 +347,7 @@ nsDragService::BuildDragRegion ( nsIScriptableRegion* inRegion, nsIDOMNode* inNo
     inRegion->GetRegion(getter_AddRefs(geckoRegion));
     
 #ifdef MOZ_WIDGET_COCOA
-  nsGraphicsUtils::SetPortToKnownGoodPort();
+  SetPortToKnownGoodPort();
   GrafPtr port;
   GDHandle handle;
   ::GetGWorld(&port, &handle);
@@ -344,20 +372,6 @@ nsDragService::BuildDragRegion ( nsIScriptableRegion* inRegion, nsIDOMNode* inNo
       Point offsetFromLocalToGlobal = { 0, 0 };
       ::LocalToGlobal ( &offsetFromLocalToGlobal );
       ::OffsetRgn ( ioDragRgn, offsetFromLocalToGlobal.h, offsetFromLocalToGlobal.v );
-
-#ifdef MOZ_WIDGET_COCOA
-      // for cocoa, we have to transform this region into cocoa screen 
-      // coordinates. Only the main screen is important in this caculation
-      // as that's where the 2 coord systems differ.
-      Rect regionBounds;
-      GetRegionBounds(ioDragRgn, &regionBounds);
-      
-      GDHandle  screenDevice = ::GetMainDevice();
-      Rect      screenRect   = (**screenDevice).gdRect;
-      // offset the rect
-      short screenHeight = screenRect.bottom - screenRect.top;
-      ::OffsetRgn(ioDragRgn, 0, screenRect.top + (screenHeight - regionBounds.top) - regionBounds.top);
-#endif
     }
   }
   else {
@@ -373,17 +387,6 @@ nsDragService::BuildDragRegion ( nsIScriptableRegion* inRegion, nsIDOMNode* inNo
     else
       NS_WARNING ( "Can't find anything to get a drag rect from. I'm dyin' out here!" );
 
-#ifdef MOZ_WIDGET_COCOA
-    // for cocoa, we have to transform this region into cocoa screen 
-    // coordinates. Only the main screen is important in this caculation
-    // as that's where the 2 coord systems differ.
-    GDHandle  screenDevice = ::GetMainDevice();
-    Rect      screenRect   = (**screenDevice).gdRect;
-    // offset the rect
-    short screenHeight = screenRect.bottom - screenRect.top;
-    ::OffsetRect(&frameRect, 0, screenRect.top + (screenHeight - frameRect.top) - frameRect.top);
-#endif
-
     if ( ioDragRgn ) {
       RgnHandle frameRgn = ::NewRgn();
       if ( frameRgn ) {
@@ -483,7 +486,7 @@ nsDragService::RegisterDragItemsAndFlavors(nsISupportsArray* inArray, RgnHandle
     if ( mapping && mappingLen ) {
       ::AddDragItemFlavor ( mDragRef, itemIndex, nsMimeMapperMac::MappingFlavor(), 
                                mapping, mappingLen, flags );
-	    nsCRT::free ( mapping );
+	    nsMemory::Free ( mapping );
     
       ::SetDragItemBounds(mDragRef, itemIndex, &dragRgnBounds);
 	  }
@@ -524,7 +527,7 @@ nsDragService::GetData ( nsITransferable * aTransferable, PRUint32 aItemIndex )
   // create a mime mapper to help us out based on data in a special flavor for this item
   char* mappings = LookupMimeMappingsForItem(mDragRef, itemRef);
   nsMimeMapperMac theMapper ( mappings );
-  nsCRT::free ( mappings );
+  nsMemory::Free ( mappings );
   
   // Now walk down the list of flavors. When we find one that is actually present,
   // copy out the data into the transferable in that format. SetTransferData()
@@ -643,7 +646,6 @@ nsDragService::GetData ( nsITransferable * aTransferable, PRUint32 aItemIndex )
           nsLinebreakHelpers::ConvertPlatformToDOMLinebreaks(flavorStr.get(), &dataBuff, NS_REINTERPRET_CAST(int*, &dataSize));
 
           unsigned char *dataPtr = (unsigned char *) dataBuff;
-#if TARGET_CARBON
           // skip BOM (Byte Order Mark to distinguish little or big endian) in 'utxt'
           // 10.2 puts BOM for 'utxt', we need to remove it here
           // for little endian case, we also need to convert the data to big endian
@@ -655,7 +657,6 @@ nsDragService::GetData ( nsITransferable * aTransferable, PRUint32 aItemIndex )
             dataSize -= sizeof(PRUnichar);
             dataPtr += sizeof(PRUnichar);
           }
-#endif
           nsPrimitiveHelpers::CreatePrimitiveForData(flavorStr.get(), (void *) dataPtr, dataSize, getter_AddRefs(genericDataWrapper));
         }
         
@@ -796,7 +797,27 @@ nsDragService::DragSendDataProc(FlavorType inFlavor, void* inRefCon, ItemReferen
     PRUint32 dataSize = 0;
     retVal = dragService->GetDataForFlavor(dragService->mDataItems, inDragRef, inItemRef, inFlavor, &data, &dataSize);
     if ( retVal == noErr ) {      
-        // make the data accessable to the DragManager
+        if ((inFlavor & kPrivateFlavorMask) == kPrivateFlavorTag) {
+          // Byte-swap private flavors if running translated
+          nsCOMPtr<nsIMacUtils> macUtils =
+           do_GetService("@mozilla.org/xpcom/mac-utils;1");
+          PRBool isTranslated;
+          if (macUtils &&
+              NS_SUCCEEDED(macUtils->GetIsTranslated(&isTranslated)) &&
+              isTranslated) {
+            char* swappedData = (char*) nsMemory::Alloc(dataSize);
+            if (!swappedData) {
+              nsMemory::Free(data);
+              return notEnoughMemoryErr;
+            }
+            else {
+              swab(data, swappedData, dataSize);
+              nsMemory::Free(data);
+              data = swappedData;
+            }
+          }
+        }
+        // make the data accessible to the DragManager
         retVal = ::SetDragItemFlavorData ( inDragRef, inItemRef, inFlavor, data, dataSize, 0 );
         NS_ASSERTION ( retVal == noErr, "SDIFD failed in DragSendDataProc" );
     }
@@ -919,6 +940,7 @@ nsDragService::GetDataForFlavor(nsISupportsArray* inDragItems, DragReference inD
     
     nsCOMPtr<nsISupports> primitiveData;
     ptrPrimitive->GetData(getter_AddRefs(primitiveData));
+#ifndef MOZ_CAIRO_GFX
     nsCOMPtr<nsIImageMac> image = do_QueryInterface(primitiveData);
     if (!image) return cantGetFlavorErr; 
       
@@ -940,6 +962,9 @@ nsDragService::GetDataForFlavor(nsISupportsArray* inDragItems, DragReference inD
       retVal = cantGetFlavorErr;
 
     ::KillPicture(picture);
+#else
+    retVal = cantGetFlavorErr;
+#endif
     return retVal;
   }
     
@@ -1045,7 +1070,7 @@ char*
 nsDragService::LookupMimeMappingsForItem ( DragReference inDragRef, ItemReference inItemRef )
 {
   char* mapperData = nsnull;
-  PRInt32 mapperSize = 0;
+  PRUint32 mapperSize = 0;
   ExtractDataFromOS(inDragRef, inItemRef, nsMimeMapperMac::MappingFlavor(),  (void**)&mapperData, &mapperSize);
 
   return mapperData;
@@ -1081,7 +1106,7 @@ nsDragService::LookupMimeMappingsForItem ( DragReference inDragRef, ItemReferenc
 //
 nsresult
 nsDragService::ExtractDataFromOS ( DragReference inDragRef, ItemReference inItemRef, ResType inFlavor, 
-                                        void** outBuffer, PRInt32* outBuffSize )
+                                       void** outBuffer, PRUint32* outBuffSize )
 {
   if ( !outBuffer || !outBuffSize || !inFlavor )
     return NS_ERROR_FAILURE;
@@ -1094,9 +1119,29 @@ nsDragService::ExtractDataFromOS ( DragReference inDragRef, ItemReference inItem
     buff = NS_REINTERPRET_CAST(char*, nsMemory::Alloc(buffSize + 1));
     if ( buff ) {	     
       err = ::GetFlavorData ( inDragRef, inItemRef, inFlavor, buff, &buffSize, 0 );
-      if ( err ) {
+      if (err == noErr) {
+        if ((inFlavor & kPrivateFlavorMask) == kPrivateFlavorTag) {
+          // Byte-swap private flavors if running translated
+          nsCOMPtr<nsIMacUtils> macUtils =
+           do_GetService("@mozilla.org/xpcom/mac-utils;1");
+          PRBool isTranslated;
+          if (macUtils &&
+              NS_SUCCEEDED(macUtils->GetIsTranslated(&isTranslated)) &&
+              isTranslated) {
+            char* swappedData = (char*) nsMemory::Alloc(buffSize);
+            if (!swappedData)
+              retval = NS_ERROR_OUT_OF_MEMORY;
+            else {
+              swab(buff, swappedData, buffSize);
+              nsMemory::Free(buff);
+              buff = swappedData;
+            }
+          }
+        }
+      }
+      else {
         #ifdef NS_DEBUG
-          printf("nsDragService: Error getting data out of drag manager, #%ld\n", err);
+          printf("nsDragService: Error getting data out of drag manager, #%d\n", err);
         #endif
         retval = NS_ERROR_FAILURE;
       }
@@ -1118,30 +1163,6 @@ nsDragService::ExtractDataFromOS ( DragReference inDragRef, ItemReference inItem
 } // ExtractDataFromOS
 
 
-//
-// StartDragSession
-// EndDragSession
-//
-// Override the defaults to disable/enable the watch cursor while we're dragging
-//
-
-nsresult
-nsDragService::StartDragSession ( )
-{
-  nsWatchTask::GetTask().Suspend();
-  
-  return nsBaseDragService::StartDragSession();
-}
-
-nsresult
-nsDragService::EndDragSession ( )
-{
-  nsWatchTask::GetTask().Resume();
-  
-  return nsBaseDragService::EndDragSession();
-}
-
-
 //
 // SetDragAction
 //
