@@ -36,11 +36,17 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsLookAndFeel.h"
-#include "nsCarbonHelpers.h"
 #include "nsIInternetConfigService.h"
 #include "nsIServiceManager.h"
 #include "nsSize.h"
+#include <ControlDefinitions.h>
+#include <MacWindows.h>
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_3
+// This theme brush is available in 10.2 and later, but was not
+// formally documented in the SDK until 10.3.
+#define kThemeBrushAlternatePrimaryHighlightColor -5
+#endif
  
 //-------------------------------------------------------------------------
 //
@@ -140,6 +146,26 @@ nsresult nsLookAndFeel::NativeGetColor(const nsColorID aID, nscolor &aColor)
     		else
     			aColor = NS_DONT_CHANGE_COLOR;
         break;
+    case eColor_IMESelectedRawTextBackground:
+    case eColor_IMESelectedConvertedTextBackground:
+    case eColor_IMERawInputBackground:
+    case eColor_IMEConvertedTextBackground:
+      aColor = NS_TRANSPARENT;
+      break;
+    case eColor_IMESelectedRawTextForeground:
+    case eColor_IMESelectedConvertedTextForeground:
+    case eColor_IMERawInputForeground:
+    case eColor_IMEConvertedTextForeground:
+      aColor = NS_SAME_AS_FOREGROUND_COLOR;
+      break;
+    case eColor_IMERawInputUnderline:
+    case eColor_IMEConvertedTextUnderline:
+      aColor = NS_40PERCENT_FOREGROUND_COLOR;
+      break;
+    case eColor_IMESelectedRawTextUnderline:
+    case eColor_IMESelectedConvertedTextUnderline:
+      aColor = NS_SAME_AS_FOREGROUND_COLOR;
+      break;
 
     //
     // css2 system colors http://www.w3.org/TR/REC-CSS2/ui.html#system-colors
@@ -297,6 +323,9 @@ nsresult nsLookAndFeel::NativeGetColor(const nsColorID aID, nscolor &aColor)
 	case eColor__moz_mac_menushadow:
 		res = GetMacBrushColor(kThemeBrushBevelActiveDark, aColor, NS_RGB(0x88,0x88,0x88));
 	    break;	        
+	case eColor__moz_mac_menutextdisable:
+		res = GetMacTextColor(kThemeTextColorMenuItemDisabled, aColor, NS_RGB(0x99,0x99,0x99));
+	    break;	    
 	case eColor__moz_mac_menutextselect:
 		//default to white, which is what Platinum uses, if not available		
 		res = GetMacTextColor(kThemeTextColorMenuItemSelected, aColor, NS_RGB(0xFF,0xFF,0xFF));
@@ -306,8 +335,8 @@ nsresult nsLookAndFeel::NativeGetColor(const nsColorID aID, nscolor &aColor)
 		res = GetMacAccentColor(eColorOffset_mac_accentlightesthighlight, aColor, NS_RGB(0xEE,0xEE,0xEE));
 	    break;    
     case eColor__moz_mac_accentregularhighlight:
-		//get this colour by querying variation table, ows. default to Platinum/Lavendar
-		res = GetMacAccentColor(eColorOffset_mac_accentregularhighlight, aColor, NS_RGB(0xCC,0xCC,0xFF));
+		//get this colour by querying variation table, ows. default to Aqua normal hilight color
+		res = GetMacAccentColor(eColorOffset_mac_accentregularhighlight, aColor, NS_RGB(0xB5,0xD5,0xFF));
 	    break;        
     case eColor__moz_mac_accentface:
 		//get this colour by querying variation table, ows. default to Platinum/Lavendar
@@ -319,8 +348,8 @@ nsresult nsLookAndFeel::NativeGetColor(const nsColorID aID, nscolor &aColor)
 	    break; 
 	case eColor__moz_mac_menuselect:
 	case eColor__moz_mac_accentregularshadow:
-		//get this colour by querying variation table, ows. default to Platinum/Lavendar
-		res = GetMacAccentColor(eColorOffset_mac_accentregularshadow, aColor, NS_RGB(0x63,0x63,0xCE));
+		//get this colour by querying variation table, ows. default to Aqua's normal menu select color
+		res = GetMacAccentColor(eColorOffset_mac_accentregularshadow, aColor, NS_RGB(0x33,0x6F,0xCB));
 	    break;
     case eColor__moz_mac_accentdarkshadow:
     	//get this colour by querying variation table, ows. default to Platinum/Lavendar
@@ -333,6 +362,17 @@ nsresult nsLookAndFeel::NativeGetColor(const nsColorID aID, nscolor &aColor)
     case eColor__moz_buttondefault:
         res = GetMacBrushColor(kThemeBrushButtonActiveDarkShadow, aColor, NS_RGB(0x77,0x77,0x77));
         break;
+    case eColor__moz_mac_alternateprimaryhighlight:
+        // For proper styling of lists when active, on 10.2+
+        // On older OSs may have to fall back to primary highlight color
+        nscolor fallbackColor;
+        GetMacBrushColor(kThemeBrushPrimaryHighlightColor, fallbackColor, NS_RGB(0x00,0x00,0x00));
+        res = GetMacBrushColor(kThemeBrushAlternatePrimaryHighlightColor, aColor, fallbackColor);
+        break;
+    case eColor__moz_mac_secondaryhighlight:
+    	// For inactive list selection
+        res = GetMacBrushColor(kThemeBrushSecondaryHighlightColor, aColor, NS_RGB(0x00,0x00,0x00));
+        break;
     default:
         NS_WARNING("Someone asked nsILookAndFeel for a color I don't know about");
         aColor = NS_RGB(0xff,0xff,0xff);
@@ -502,6 +542,9 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
         // xul popups are not allowed to overlap the menubar.
         aMetric = 0;
         break;
+    case eMetric_SkipNavigatingDisabledMenuItem:
+        aMetric = 1;
+        break;
     case eMetric_DragFullWindow:
         aMetric = 1;
         break;        
@@ -512,12 +555,30 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
     case eMetric_ScrollArrowStyle:
         ThemeScrollBarArrowStyle arrowStyle;
         ::GetThemeScrollBarArrowStyle ( &arrowStyle );
-        aMetric = arrowStyle;
+        switch (arrowStyle) {
+        case kThemeScrollBarArrowsSingle:
+            aMetric = eMetric_ScrollArrowStyleSingle;
+            break;
+        default:
+            NS_WARNING("Not handling all possible ThemeScrollBarArrowStyle values");
+            // fall through so we default to BothAtBottom
+        case kThemeScrollBarArrowsLowerRight:
+            aMetric = eMetric_ScrollArrowStyleBothAtBottom;
+        }
         break;
     case eMetric_ScrollSliderStyle:
         ThemeScrollBarThumbStyle thumbStyle;
         ::GetThemeScrollBarThumbStyle ( &thumbStyle );
-        aMetric = thumbStyle;
+        switch (thumbStyle) {
+        case kThemeScrollBarThumbNormal:
+            aMetric = eMetric_ScrollThumbStyleNormal;
+            break;
+        default:
+            NS_WARNING("Not handling all possible ThemeScrollBarThumbStyle values");
+            // fall through so we default to Proportional
+        case kThemeScrollBarThumbProportional:
+            aMetric = eMetric_ScrollThumbStyleProportional;
+        }
         break;
     case eMetric_TreeOpenDelay:
         aMetric = 1000;
@@ -534,7 +595,28 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
     case eMetric_TreeScrollLinesMax:
         aMetric = 3;
         break;
-
+    case eMetric_TabFocusModel:
+        {
+          // we should probably cache this
+          CFPropertyListRef fullKeyboardAccessProperty;
+          fullKeyboardAccessProperty = ::CFPreferencesCopyValue(CFSTR("AppleKeyboardUIMode"),
+                                                                 kCFPreferencesAnyApplication,
+                                                                 kCFPreferencesCurrentUser,
+                                                                 kCFPreferencesAnyHost);
+          aMetric = 1;    // default to just textboxes
+          if (fullKeyboardAccessProperty)
+          {
+            PRInt32 fullKeyboardAccessPrefVal;
+            if (::CFNumberGetValue((CFNumberRef) fullKeyboardAccessProperty, kCFNumberIntType, &fullKeyboardAccessPrefVal))
+            {
+              // the second bit means  "Full keyboard access" is on
+              if (fullKeyboardAccessPrefVal & (1 << 1))
+                aMetric = 7; // everything that can be focused
+            }
+            ::CFRelease(fullKeyboardAccessProperty);
+          }
+        }
+        break;
     default:
         aMetric = 0;
         res = NS_ERROR_FAILURE;
@@ -574,6 +656,9 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricFloatID aID, float & aMetri
     case eMetricFloat_ButtonHorizontalInsidePadding:
         aMetric = 0.5f;
         break;
+    case eMetricFloat_IMEUnderlineRelativeSize:
+        aMetric = 2.0f;
+        break;
     default:
         aMetric = -1.0;
         res = NS_ERROR_FAILURE;
