@@ -45,7 +45,6 @@
 
 #include "nsMenuBarX.h"
 #include "nsMenuX.h"
-#include "nsDynamicMDEF.h"
 
 #include "nsISupports.h"
 #include "nsIWidget.h"
@@ -54,30 +53,20 @@
 #include "nsIDocument.h"
 #include "nsIDocShell.h"
 #include "nsIDocumentViewer.h"
-#include "nsIDocumentObserver.h"
+#include "nsIMutationObserver.h"
 
 #include "nsIDOMDocument.h"
 #include "nsWidgetAtoms.h"
 
-#include <Menus.h>
-#include <TextUtils.h>
-#include <Balloons.h>
-#include <Resources.h>
-#include <Appearance.h>
-#include <Gestalt.h>
 #include "nsMacResources.h"
 
 #include "nsGUIEvent.h"
 
-#if !XP_MACOSX
-#include "MenuSharing.h"
-#endif
-
 // CIDs
 #include "nsWidgetsCID.h"
 static NS_DEFINE_CID(kMenuCID, NS_MENU_CID);
 
-NS_IMPL_ISUPPORTS6(nsMenuBarX, nsIMenuBar, nsIMenuListener, nsIDocumentObserver, 
+NS_IMPL_ISUPPORTS6(nsMenuBarX, nsIMenuBar, nsIMenuListener, nsIMutationObserver, 
                     nsIChangeManager, nsIMenuCommandDispatcher, nsISupportsWeakReference)
 
 MenuRef nsMenuBarX::sAppleMenu = nsnull;
@@ -107,8 +96,7 @@ nsMenuBarX::~nsMenuBarX()
 
   // make sure we unregister ourselves as a document observer
   if ( mDocument ) {
-    nsCOMPtr<nsIDocumentObserver> observer ( do_QueryInterface(NS_STATIC_CAST(nsIMenuBar*,this)) );
-    mDocument->RemoveObserver(observer);
+    mDocument->RemoveMutationObserver(this);
   }
 
   if ( mRootMenu )
@@ -124,12 +112,10 @@ nsMenuBarX::MenuItemSelected(const nsMenuEvent & aMenuEvent)
   PRUint32  numItems;
   mMenusArray.Count(&numItems);
   
-  for (PRUint32 i = numItems; i > 0; --i)
-  {
+  for (PRUint32 i = numItems; i > 0; --i) {
     nsCOMPtr<nsISupports>     menuSupports = getter_AddRefs(mMenusArray.ElementAt(i - 1));
     nsCOMPtr<nsIMenuListener> menuListener = do_QueryInterface(menuSupports);
-    if(menuListener)
-    {
+    if (menuListener) {
       eventStatus = menuListener->MenuItemSelected(aMenuEvent);
       if (nsEventStatus_eIgnore != eventStatus)
         return eventStatus;
@@ -145,39 +131,24 @@ nsMenuBarX::MenuSelected(const nsMenuEvent & aMenuEvent)
   // Dispatch event
   nsEventStatus eventStatus = nsEventStatus_eIgnore;
 
-  nsCOMPtr<nsIMenuListener> menuListener;
-  //((nsISupports*)mMenuVoidArray[i-1])->QueryInterface(NS_GET_IID(nsIMenuListener), (void**)&menuListener);
-  //printf("gPreviousMenuStack.Count() = %d \n", gPreviousMenuStack.Count());
-#if !TARGET_CARBON
-  nsCOMPtr<nsIMenu> theMenu;
-  gPreviousMenuStack.GetMenuAt(gPreviousMenuStack.Count() - 1, getter_AddRefs(theMenu));
-  menuListener = do_QueryInterface(theMenu);
-#endif
-  if (menuListener) {
-    //TODO: MenuSelected is the right thing to call...
-    //eventStatus = menuListener->MenuSelected(aMenuEvent);
-    eventStatus = menuListener->MenuItemSelected(aMenuEvent);
-    if (nsEventStatus_eIgnore != eventStatus)
-      return eventStatus;
-  } else {
-    // If it's the help menu, gPreviousMenuStack won't be accurate so we need to get the listener a different way 
-    // We'll do it the old fashioned way of looping through and finding it
-    PRUint32  numItems;
-    mMenusArray.Count(&numItems);
-    for (PRUint32 i = numItems; i > 0; --i)
+  // If it's the help menu, gPreviousMenuStack won't be accurate so we need to get the listener a different way 
+  // We'll do it the old fashioned way of looping through and finding it
+  PRUint32  numItems;
+  mMenusArray.Count(&numItems);
+  for (PRUint32 i = numItems; i > 0; --i)
+  {
+    nsCOMPtr<nsISupports>     menuSupports = getter_AddRefs(mMenusArray.ElementAt(i - 1));
+    nsCOMPtr<nsIMenuListener> thisListener = do_QueryInterface(menuSupports);
+    if (thisListener)
     {
-      nsCOMPtr<nsISupports>     menuSupports = getter_AddRefs(mMenusArray.ElementAt(i - 1));
-      nsCOMPtr<nsIMenuListener> thisListener = do_QueryInterface(menuSupports);
-	    if (thisListener)
-	    {
-        //TODO: MenuSelected is the right thing to call...
-  	    //eventStatus = menuListener->MenuSelected(aMenuEvent);
-  	    eventStatus = thisListener->MenuItemSelected(aMenuEvent);
-  	    if(nsEventStatus_eIgnore != eventStatus)
-  	      return eventStatus;
-      }
+      //TODO: MenuSelected is the right thing to call...
+      //eventStatus = menuListener->MenuSelected(aMenuEvent);
+      eventStatus = thisListener->MenuItemSelected(aMenuEvent);
+      if(nsEventStatus_eIgnore != eventStatus)
+        return eventStatus;
     }
   }
+  
   return eventStatus;
 }
 
@@ -202,14 +173,13 @@ nsMenuBarX::SetRebuild(PRBool aNeedsRebuild)
 }
 
 void
-nsMenuBarX :: GetDocument ( nsIWebShell* inWebShell, nsIDocument** outDocument )
+nsMenuBarX :: GetDocument ( nsIDocShell* inDocShell, nsIDocument** outDocument )
 {
   *outDocument = nsnull;
   
-  nsCOMPtr<nsIDocShell> docShell ( do_QueryInterface(inWebShell) );
-  nsCOMPtr<nsIContentViewer> cv;
-  if ( docShell ) {
-    docShell->GetContentViewer(getter_AddRefs(cv));
+  if ( inDocShell ) {
+    nsCOMPtr<nsIContentViewer> cv;
+    inDocShell->GetContentViewer(getter_AddRefs(cv));
     if (cv) {
       // get the document
       nsCOMPtr<nsIDocumentViewer> docv(do_QueryInterface(cv));
@@ -224,20 +194,17 @@ nsMenuBarX :: GetDocument ( nsIWebShell* inWebShell, nsIDocument** outDocument )
 //
 // RegisterAsDocumentObserver
 //
-// Name says it all.
-//
 void
-nsMenuBarX :: RegisterAsDocumentObserver ( nsIWebShell* inWebShell )
+nsMenuBarX :: RegisterAsDocumentObserver ( nsIDocShell* inDocShell )
 {
   nsCOMPtr<nsIDocument> doc;
-  GetDocument(inWebShell, getter_AddRefs(doc));
+  GetDocument(inDocShell, getter_AddRefs(doc));
   if (!doc)
     return;
 
   // register ourselves
-  nsCOMPtr<nsIDocumentObserver> observer ( do_QueryInterface(NS_STATIC_CAST(nsIMenuBar*,this)) );
-  doc->AddObserver(observer);
-  // also get pointer to doc, just in case webshell goes away
+  doc->AddMutationObserver(this);
+  // also get pointer to doc, just in case docshell goes away
   // we can still remove ourself as doc observer directly from doc
   mDocument = doc;
 } // RegisterAsDocumentObesrver
@@ -256,13 +223,22 @@ nsMenuBarX :: AquifyMenuBar ( )
   nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(mMenuBarContent->GetDocument()));
   if ( domDoc ) {
     // remove quit item and its separator
-    HideItem ( domDoc, NS_LITERAL_STRING("menu_FileQuitSeparator"), nsnull );
-    HideItem ( domDoc, NS_LITERAL_STRING("menu_FileQuitItem"), getter_AddRefs(mQuitItemContent) );
-  
+    HideItem(domDoc, NS_LITERAL_STRING("menu_FileQuitSeparator"), nsnull);
+    HideItem(domDoc, NS_LITERAL_STRING("menu_FileQuitItem"), getter_AddRefs(mQuitItemContent));
+
     // remove prefs item and its separator, but save off the pref content node
     // so we can invoke its command later.
-    HideItem ( domDoc, NS_LITERAL_STRING("menu_PrefsSeparator"), nsnull );
-    HideItem ( domDoc, NS_LITERAL_STRING("menu_preferences"), getter_AddRefs(mPrefItemContent) );
+    HideItem(domDoc, NS_LITERAL_STRING("menu_PrefsSeparator"), nsnull);
+    HideItem(domDoc, NS_LITERAL_STRING("menu_preferences"), getter_AddRefs(mPrefItemContent));
+    if (mPrefItemContent)
+      ::EnableMenuCommand(NULL, kHICommandPreferences);
+
+    // Cocoa menus compatibility - hide items that we use for the Application menu in Cocoa menus.
+    // This way, a menu setup for the Cocoa application menu will also work for Carbon menus.
+    HideItem(domDoc, NS_LITERAL_STRING("menu_mac_services"), nsnull);
+    HideItem(domDoc, NS_LITERAL_STRING("menu_mac_hide_app"), nsnull);
+    HideItem(domDoc, NS_LITERAL_STRING("menu_mac_hide_others"), nsnull);
+    HideItem(domDoc, NS_LITERAL_STRING("menu_mac_show_all"), nsnull);
   }
       
 } // AquifyMenuBar
@@ -283,9 +259,15 @@ nsMenuBarX :: InstallCommandEventHandler ( )
   WindowRef myWindow = NS_REINTERPRET_CAST(WindowRef, mParent->GetNativeData(NS_NATIVE_DISPLAY));
   NS_ASSERTION ( myWindow, "Can't get WindowRef to install command handler!" );
   if ( myWindow && sCommandEventHandler ) {
-    const EventTypeSpec commandEventList[] = { {kEventClassCommand, kEventCommandProcess},
-                                               {kEventClassCommand, kEventCommandUpdateStatus} };
-    err = ::InstallWindowEventHandler ( myWindow, sCommandEventHandler, 2, commandEventList, this, NULL );
+    const EventTypeSpec commandEventList[] = {
+     {kEventClassCommand, kEventCommandProcess},
+    };
+    err = ::InstallWindowEventHandler(myWindow,
+                                      sCommandEventHandler,
+                                      GetEventTypeCount(commandEventList),
+                                      commandEventList,
+                                      this,
+                                      NULL);
     NS_ASSERTION ( err == noErr, "Uh oh, command handler not installed" );
   }
 
@@ -297,7 +279,7 @@ nsMenuBarX :: InstallCommandEventHandler ( )
 //
 // CommandEventHandler
 //
-// Processes Command carbon events from enabling/selecting of items in the menu.
+// Processes Command carbon events from selecting of items in the menu.
 //
 pascal OSStatus
 nsMenuBarX :: CommandEventHandler ( EventHandlerCallRef inHandlerChain, EventRef inEvent, void* userData )
@@ -309,94 +291,61 @@ nsMenuBarX :: CommandEventHandler ( EventHandlerCallRef inHandlerChain, EventRef
                                         NULL, sizeof(HICommand), NULL, &command );	
   if ( err1 )
     return handled;
-    
+
+  NS_ASSERTION(::GetEventKind(inEvent) == kEventCommandProcess,
+               "CommandEventHandler asked to handle unknown event kind");
+
   nsMenuBarX* self = NS_REINTERPRET_CAST(nsMenuBarX*, userData);
-  switch ( ::GetEventKind(inEvent) ) {
-    // user selected a menu item. See if it's one we handle.
-    case kEventCommandProcess:
-    {
-      switch ( command.commandID ) {
-        case kHICommandPreferences:
-        {
-          nsEventStatus status = self->ExecuteCommand(self->mPrefItemContent);
-          if ( status == nsEventStatus_eConsumeNoDefault )    // event handled, no other processing
-            handled = noErr;
-          break;
-        }
-        
-        case kHICommandQuit:
-        {
-          nsEventStatus status = self->ExecuteCommand(self->mQuitItemContent);
-          if ( status == nsEventStatus_eConsumeNoDefault )    // event handled, no other processing
-            handled = noErr;
-          break;
-        }
-        
-        case kHICommandAbout:
-        {
-          // the 'about' command is special because we don't have a
-          // nsIMenu or nsIMenuItem for the apple menu. Grovel for the
-          // content node with an id of "aboutName" and call it
-          // directly.
-          nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(self->mDocument);
-	        if ( domDoc ) {
-      	    nsCOMPtr<nsIDOMElement> domElement;
-      	    domDoc->GetElementById(NS_LITERAL_STRING("aboutName"),
-                                   getter_AddRefs(domElement));
-      	    nsCOMPtr<nsIContent> aboutContent ( do_QueryInterface(domElement) );
-      	    self->ExecuteCommand(aboutContent);
-          }
-          handled = noErr;
-          break;
-        }
-
-#if !XP_MACOSX
-        case 'SHMN':
-        { // Shared menu support
-          MenuItemIndex index = command.menu.menuItemIndex;
-          if (index)
-          {
-            (void)SharedMenuHit(GetMenuID(command.menu.menuRef), command.menu.menuItemIndex);
-            ::HiliteMenu(0);
-            handled = noErr;
-          }
-          break;
-        }
-#endif
-
-        default:
-        {
-          // given the commandID, look it up in our hashtable and dispatch to
-          // that content node. Recall that we store weak pointers to the content
-          // nodes in the hash table.
-          nsPRUint32Key key ( command.commandID );
-          nsIMenuItem* content = NS_REINTERPRET_CAST(nsIMenuItem*, self->mObserverTable.Get(&key));
-          if ( content ) {
-            content->DoCommand();
-            handled = noErr;
-          }        
-          break; 
-        }        
-
-      } // switch on commandID
+  switch (command.commandID) {
+    case kHICommandPreferences: {
+      nsEventStatus status = self->ExecuteCommand(self->mPrefItemContent);
+      if (status == nsEventStatus_eConsumeNoDefault)
+        // event handled, no other processing
+        handled = noErr;
       break;
     }
-    
-    // enable/disable menu id's
-    case kEventCommandUpdateStatus:
-    {
-      // only enable the preferences item in the app menu if we found a pref
-      // item DOM node in this menubar.
-      if ( command.commandID == kHICommandPreferences ) {
-        if ( self->mPrefItemContent )
-          ::EnableMenuCommand ( nsnull, kHICommandPreferences );
-        else
-          ::DisableMenuCommand ( nsnull, kHICommandPreferences );
+
+    case kHICommandQuit: {
+      nsEventStatus status = self->ExecuteCommand(self->mQuitItemContent);
+      if (status == nsEventStatus_eConsumeNoDefault)
+        // event handled, no other processing
+        handled = noErr;
+      break;
+    }
+
+    case kHICommandAbout: {
+      // the 'about' command is special because we don't have a
+      // nsIMenu or nsIMenuItem for the apple menu. Grovel for the
+      // content node with an id of "aboutName" and call it
+      // directly.
+      nsCOMPtr<nsIDOMDocument> domDoc = do_QueryInterface(self->mDocument);
+      if (domDoc) {
+        nsCOMPtr<nsIDOMElement> domElement;
+        domDoc->GetElementById(NS_LITERAL_STRING("aboutName"),
+                               getter_AddRefs(domElement));
+        nsCOMPtr<nsIContent> aboutContent(do_QueryInterface(domElement));
+        nsEventStatus status = self->ExecuteCommand(aboutContent);
+        if (status == nsEventStatus_eConsumeNoDefault)
+          // event handled, no other processing
+          handled = noErr;
+      }
+      break;
+    }
+
+    default: {
+      // given the commandID, look it up in our hashtable and dispatch to
+      // that content node. Recall that we store weak pointers to the content
+      // nodes in the hash table.
+      nsPRUint32Key key(command.commandID);
+      nsIMenuItem* content = NS_REINTERPRET_CAST(nsIMenuItem*,
+                                               self->mObserverTable.Get(&key));
+      if (content) {
+        content->DoCommand();
         handled = noErr;
       }
       break;
     }
-  } // switch on event type
+  }
   
   return handled;
   
@@ -415,7 +364,7 @@ nsMenuBarX :: ExecuteCommand ( nsIContent* inDispatchTo )
   if (!inDispatchTo)
     return nsEventStatus_eIgnore;
 
-  return MenuHelpersX::DispatchCommandTo(mWebShellWeakRef, inDispatchTo);
+  return MenuHelpersX::DispatchCommandTo(mDocShellWeakRef, inDispatchTo);
 } // ExecuteCommand
 
 
@@ -445,9 +394,9 @@ nsMenuBarX :: HideItem ( nsIDOMDocument* inDoc, const nsAString & inID, nsIConte
 
 nsEventStatus
 nsMenuBarX::MenuConstruct( const nsMenuEvent & aMenuEvent, nsIWidget* aParentWindow, 
-                            void * menubarNode, void * aWebShell )
+                            void * menubarNode, void * aDocShell )
 {
-  mWebShellWeakRef = do_GetWeakReference(NS_STATIC_CAST(nsIWebShell*, aWebShell));
+  mDocShellWeakRef = do_GetWeakReference(NS_STATIC_CAST(nsIDocShell*, aDocShell));
   nsIDOMNode* aDOMNode  = NS_STATIC_CAST(nsIDOMNode*, menubarNode);
   mMenuBarContent = do_QueryInterface(aDOMNode);           // strong ref
   NS_ASSERTION ( mMenuBarContent, "No content specified for this menubar" );
@@ -456,18 +405,15 @@ nsMenuBarX::MenuConstruct( const nsMenuEvent & aMenuEvent, nsIWidget* aParentWin
     
   Create(aParentWindow);
   
-  // if we're on X (using aqua UI guidelines for menus), remove quit and prefs
-  // from our menubar.
-  SInt32 result = 0L;
-  OSStatus err = ::Gestalt ( gestaltMenuMgrAttr, &result );
-  if ( !err && (result & gestaltMenuMgrAquaLayoutMask) )
-    AquifyMenuBar();
-  err = InstallCommandEventHandler();
+  // since we're on OSX, remove quit and prefs from our menubar.
+  AquifyMenuBar();
+  
+  OSStatus err = InstallCommandEventHandler();
   if ( err )
     return nsEventStatus_eIgnore;
 
-  nsCOMPtr<nsIWebShell> webShell = do_QueryReferent(mWebShellWeakRef);
-  if (webShell) RegisterAsDocumentObserver(webShell);
+  nsCOMPtr<nsIDocShell> docShell = do_QueryReferent(mDocShellWeakRef);
+  if (docShell) RegisterAsDocumentObserver(docShell);
 
   // set this as a nsMenuListener on aParentWindow
   aParentWindow->AddMenuListener((nsIMenuListener *)this);
@@ -477,7 +423,7 @@ nsMenuBarX::MenuConstruct( const nsMenuEvent & aMenuEvent, nsIWidget* aParentWin
     nsIContent *menu = mMenuBarContent->GetChildAt(i);
     if ( menu ) {
       if (menu->Tag() == nsWidgetAtoms::menu &&
-          menu->IsContentOfType(nsIContent::eXUL)) {
+          menu->IsNodeOfType(nsINode::eXUL)) {
         nsAutoString menuName;
         nsAutoString menuAccessKey(NS_LITERAL_STRING(" "));
         menu->GetAttr(kNameSpaceID_None, nsWidgetAtoms::label, menuName);
@@ -490,20 +436,14 @@ nsMenuBarX::MenuConstruct( const nsMenuEvent & aMenuEvent, nsIWidget* aParentWin
         if ( pnsMenu ) {
           pnsMenu->Create(NS_STATIC_CAST(nsIMenuBar*, this), menuName, menuAccessKey, 
                           NS_STATIC_CAST(nsIChangeManager *, this), 
-                          NS_REINTERPRET_CAST(nsIWebShell*, aWebShell), menu);
+                          NS_REINTERPRET_CAST(nsIDocShell*, aDocShell), menu);
 
           // Make nsMenu a child of nsMenuBar. nsMenuBar takes ownership
           AddMenu(pnsMenu); 
                   
-          nsAutoString menuIDstring;
-          menu->GetAttr(kNameSpaceID_None, nsWidgetAtoms::id, menuIDstring);
-          if ( menuIDstring.EqualsLiteral("menu_Help") ) {
-            nsMenuEvent event;
-            MenuHandle handle = nsnull;
-#if !TARGET_CARBON
-            ::HMGetHelpMenuHandle(&handle);
-#endif
-            event.mCommand = (unsigned int) handle;
+          if (menu->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::id,
+                                NS_LITERAL_STRING("menu_Help"), eCaseMatters)) {
+            nsMenuEvent event(PR_TRUE, 0, nsnull);
             nsCOMPtr<nsIMenuListener> listener(do_QueryInterface(pnsMenu));
             listener->MenuSelected(event);
           }          
@@ -581,22 +521,18 @@ NS_METHOD nsMenuBarX::AddMenu(nsIMenu * aMenu)
   MenuRef menuRef = nsnull;
   aMenu->GetNativeData((void**)&menuRef);
 
-  PRBool helpMenu;
-  aMenu->IsHelpMenu(&helpMenu);
-  if(!helpMenu) {
-    nsCOMPtr<nsIContent> menu;
-    aMenu->GetMenuContent(getter_AddRefs(menu));
-    nsAutoString menuHidden;
-    menu->GetAttr(kNameSpaceID_None, nsWidgetAtoms::hidden, menuHidden);
-    if( !menuHidden.EqualsLiteral("true")) {
-    	// make sure we only increment |mNumMenus| if the menu is visible, since
-    	// we use it as an index of where to insert the next menu.
-      mNumMenus++;
-      
-      ::InsertMenuItem(mRootMenu, "\pPlaceholder", mNumMenus);
-      OSStatus status = ::SetMenuItemHierarchicalMenu(mRootMenu, mNumMenus, menuRef);
-      NS_ASSERTION(status == noErr, "nsMenuBarX::AddMenu: SetMenuItemHierarchicalMenu failed.");
-    }
+  nsCOMPtr<nsIContent> menu;
+  aMenu->GetMenuContent(getter_AddRefs(menu));
+  if(!menu->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::hidden,
+                        nsWidgetAtoms::_true, eCaseMatters) &&
+     menu->GetChildCount() > 0) {
+    // make sure we only increment |mNumMenus| if the menu is visible, since
+    // we use it as an index of where to insert the next menu.
+    mNumMenus++;
+    
+    ::InsertMenuItem(mRootMenu, "\pPlaceholder", mNumMenus);
+    OSStatus status = ::SetMenuItemHierarchicalMenu(mRootMenu, mNumMenus, menuRef);
+    NS_ASSERTION(status == noErr, "nsMenuBarX::AddMenu: SetMenuItemHierarchicalMenu failed.");
   }
 
   return NS_OK;
@@ -670,7 +606,7 @@ NS_METHOD nsMenuBarX::GetMenuAt(const PRUint32 aCount, nsIMenu *& aMenu)
 //-------------------------------------------------------------------------
 NS_METHOD nsMenuBarX::InsertMenuAt(const PRUint32 aCount, nsIMenu *& aMenu)
 {
-  return NS_OK;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 //-------------------------------------------------------------------------
@@ -685,28 +621,20 @@ NS_METHOD nsMenuBarX::RemoveMenu(const PRUint32 aCount)
 //-------------------------------------------------------------------------
 NS_METHOD nsMenuBarX::RemoveAll()
 {
-  NS_ASSERTION(0, "Not implemented!");
-  // mMenusArray.Clear();    // maybe?
-  return NS_OK;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 //-------------------------------------------------------------------------
 NS_METHOD nsMenuBarX::GetNativeData(void *& aData)
 {
-    aData = (void *) mRootMenu;
-    return NS_OK;
+  aData = (void *) mRootMenu;
+  return NS_OK;
 }
 
 //-------------------------------------------------------------------------
 NS_METHOD nsMenuBarX::SetNativeData(void* aData)
 {
-#if 0
-    Handle menubarHandle = (Handle)aData;
-    if (mMacMBarHandle && mMacMBarHandle != menubarHandle)
-        ::DisposeHandle(mMacMBarHandle);
-    mMacMBarHandle = menubarHandle;
-#endif
-    return NS_OK;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 //-------------------------------------------------------------------------
@@ -722,25 +650,10 @@ NS_METHOD nsMenuBarX::Paint()
 #pragma mark -
 
 //
-// nsIDocumentObserver
+// nsIMutationObserver
 // this is needed for menubar changes
 //
 
-NS_IMPL_NSIDOCUMENTOBSERVER_LOAD_STUB(nsMenuBarX)
-NS_IMPL_NSIDOCUMENTOBSERVER_REFLOW_STUB(nsMenuBarX)
-NS_IMPL_NSIDOCUMENTOBSERVER_STATE_STUB(nsMenuBarX)
-NS_IMPL_NSIDOCUMENTOBSERVER_STYLE_STUB(nsMenuBarX)
-
-void
-nsMenuBarX::BeginUpdate( nsIDocument * aDocument, nsUpdateType aUpdateType )
-{
-}
-
-void
-nsMenuBarX::EndUpdate( nsIDocument * aDocument, nsUpdateType aUpdateType )
-{
-}
-
 void
 nsMenuBarX::CharacterDataChanged( nsIDocument * aDocument,
                                   nsIContent * aContent, PRBool aAppend)
@@ -751,28 +664,24 @@ void
 nsMenuBarX::ContentAppended( nsIDocument * aDocument, nsIContent  * aContainer,
                               PRInt32 aNewIndexInContainer)
 {
-  if ( aContainer == mMenuBarContent ) {
-    //Register(aContainer, );
-    //InsertMenu ( aNewIndexInContainer );
-  }
-  else {
+  if (aContainer != mMenuBarContent) {
     nsCOMPtr<nsIChangeObserver> obs;
-    Lookup ( aContainer, getter_AddRefs(obs) );
-    if ( obs )
-      obs->ContentInserted ( aDocument, aContainer, aNewIndexInContainer );
+    Lookup(aContainer, getter_AddRefs(obs));
+    if (obs)
+      obs->ContentInserted(aDocument, aContainer, aNewIndexInContainer);
     else {
       nsCOMPtr<nsIContent> parent = aContainer->GetParent();
-      if(parent) {
-        Lookup ( parent, getter_AddRefs(obs) );
-        if ( obs )
-          obs->ContentInserted ( aDocument, aContainer, aNewIndexInContainer );
+      if (parent) {
+        Lookup(parent, getter_AddRefs(obs));
+        if (obs)
+          obs->ContentInserted(aDocument, aContainer, aNewIndexInContainer);
       }
     }
   }
 }
 
 void
-nsMenuBarX::DocumentWillBeDestroyed( nsIDocument * aDocument )
+nsMenuBarX::NodeWillBeDestroyed(const nsINode* aNode)
 {
   mDocument = nsnull;
 }
@@ -785,9 +694,9 @@ nsMenuBarX::AttributeChanged( nsIDocument * aDocument, nsIContent * aContent,
 {
   // lookup and dispatch to registered thang.
   nsCOMPtr<nsIChangeObserver> obs;
-  Lookup ( aContent, getter_AddRefs(obs) );
-  if ( obs )
-    obs->AttributeChanged ( aDocument, aNameSpaceID, aAttribute );
+  Lookup(aContent, getter_AddRefs(obs));
+  if (obs)
+    obs->AttributeChanged(aDocument, aNameSpaceID, aContent, aAttribute);
 }
 
 void
@@ -818,21 +727,17 @@ void
 nsMenuBarX::ContentInserted( nsIDocument * aDocument, nsIContent * aContainer,
                              nsIContent * aChild, PRInt32 aIndexInContainer )
 {  
-  if ( aContainer == mMenuBarContent ) {
-    //Register(aChild, );
-    //InsertMenu ( aIndexInContainer );
-  }
-  else {
+  if (aContainer != mMenuBarContent) {
     nsCOMPtr<nsIChangeObserver> obs;
-    Lookup ( aContainer, getter_AddRefs(obs) );
-    if ( obs )
-      obs->ContentInserted ( aDocument, aChild, aIndexInContainer );
+    Lookup (aContainer, getter_AddRefs(obs));
+    if (obs)
+      obs->ContentInserted(aDocument, aChild, aIndexInContainer);
     else {
       nsCOMPtr<nsIContent> parent = aContainer->GetParent();
-      if(parent) {
-        Lookup ( parent, getter_AddRefs(obs) );
-        if ( obs )
-          obs->ContentInserted ( aDocument, aChild, aIndexInContainer );
+      if (parent) {
+        Lookup(parent, getter_AddRefs(obs));
+        if (obs)
+          obs->ContentInserted(aDocument, aChild, aIndexInContainer);
       }
     }
   }
@@ -935,25 +840,23 @@ nsMenuBarX :: Unregister ( PRUint32 inCommandID )
 
 
 //
-// WebShellToPresContext
+// DocShellToPresContext
 //
-// Helper to dig out a pres context from a webshell. A common thing to do before
+// Helper to dig out a pres context from a docshell. A common thing to do before
 // sending an event into the dom.
 //
 nsresult
-MenuHelpersX::WebShellToPresContext (nsIWebShell* inWebShell, nsPresContext** outContext )
+MenuHelpersX::DocShellToPresContext (nsIDocShell* inDocShell, nsPresContext** outContext )
 {
   NS_ENSURE_ARG_POINTER(outContext);
   *outContext = nsnull;
-  if (!inWebShell)
+  if (!inDocShell)
     return NS_ERROR_INVALID_ARG;
   
   nsresult retval = NS_OK;
   
-  nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(inWebShell));
-
   nsCOMPtr<nsIContentViewer> contentViewer;
-  docShell->GetContentViewer(getter_AddRefs(contentViewer));
+  inDocShell->GetContentViewer(getter_AddRefs(contentViewer));
   if ( contentViewer ) {
     nsCOMPtr<nsIDocumentViewer> docViewer ( do_QueryInterface(contentViewer) );
     if ( docViewer )
@@ -966,42 +869,26 @@ MenuHelpersX::WebShellToPresContext (nsIWebShell* inWebShell, nsPresContext** ou
   
   return retval;
   
-} // WebShellToPresContext
+} // DocShellToPresContext
 
 nsEventStatus
-MenuHelpersX::DispatchCommandTo(nsIWeakReference* aWebShellWeakRef,
+MenuHelpersX::DispatchCommandTo(nsIWeakReference* aDocShellWeakRef,
                                 nsIContent* aTargetContent)
 {
   NS_PRECONDITION(aTargetContent, "null ptr");
 
-  nsCOMPtr<nsIWebShell> webShell = do_QueryReferent(aWebShellWeakRef);
-  if (!webShell)
+  nsCOMPtr<nsIDocShell> docShell = do_QueryReferent(aDocShellWeakRef);
+  if (!docShell)
     return nsEventStatus_eConsumeNoDefault;
   nsCOMPtr<nsPresContext> presContext;
-  MenuHelpersX::WebShellToPresContext(webShell, getter_AddRefs(presContext));
+  MenuHelpersX::DocShellToPresContext(docShell, getter_AddRefs(presContext));
 
   nsEventStatus status = nsEventStatus_eConsumeNoDefault;
-  nsMouseEvent event(NS_XUL_COMMAND);
+  nsXULCommandEvent event(PR_TRUE, NS_XUL_COMMAND, nsnull);
 
   // FIXME: Should probably figure out how to init this with the actual
   // pressed keys, but this is a big old edge case anyway. -dwh
 
-  // See if we have a command element.  If so, we execute on the
-  // command instead of on our content element.
-  nsAutoString command;
-  aTargetContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::command, command);
-  if (!command.IsEmpty()) {
-    nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(aTargetContent->GetDocument()));
-    nsCOMPtr<nsIDOMElement> commandElt;
-    domDoc->GetElementById(command, getter_AddRefs(commandElt));
-    nsCOMPtr<nsIContent> commandContent(do_QueryInterface(commandElt));
-    if (commandContent)
-      commandContent->HandleDOMEvent(presContext, &event, nsnull,
-                                     NS_EVENT_FLAG_INIT, &status);
-  }
-  else
-    aTargetContent->HandleDOMEvent(presContext, &event, nsnull,
-                                   NS_EVENT_FLAG_INIT, &status);
-
+  aTargetContent->DispatchDOMEvent(&event, nsnull, presContext, &status);
   return status;
 }
