@@ -44,6 +44,7 @@
 #include "nsIComponentManager.h"
 #include "nsIDocShell.h"
 #include "prinrval.h"
+#include "nsIRollupListener.h"
 
 #include "nsMenuX.h"
 #include "nsMenuBarX.h"
@@ -53,6 +54,7 @@
 #include "nsIMenuListener.h"
 #include "nsPresContext.h"
 #include "nsIMenuCommandDispatcher.h"
+#include "nsMenuItemIcon.h"
 
 #include "nsString.h"
 #include "nsReadableUtils.h"
@@ -64,25 +66,17 @@
 #include "nsIXBLService.h"
 #include "nsIServiceManager.h"
 
-#include <Appearance.h>
-#include <ToolUtils.h>
-#include <UnicodeConverter.h>
-
 #include "nsGUIEvent.h"
 
-#include "nsDynamicMDEF.h"
 #include "nsCRT.h"
 
+// externs defined in nsWindow.cpp
+extern nsIRollupListener * gRollupListener;
+extern nsIWidget         * gRollupWidget;
 
 static OSStatus InstallMyMenuEventHandler(MenuRef menuRef, void* userData, EventHandlerRef* outHandler) ;
 
-// keep track of the menuID of the menu the mouse is currently over. Yes, this is ugly,
-// but necessary to work around bugs in Carbon with ::MenuSelect() sometimes returning
-// the wrong menuID.
-static MenuID gCurrentlyTrackedMenuID = 0;
-
-const PRInt16 kMacMenuIDX = nsMenuBarX::kAppleMenuID + 1;
-static PRInt16 gMacMenuIDCountX = kMacMenuIDX;
+static PRInt16 gMacMenuIDCountX = nsMenuBarX::kAppleMenuID + 1;
 static PRBool gConstructingMenu = PR_FALSE;
   
 #if DEBUG
@@ -116,9 +110,8 @@ NS_IMPL_ISUPPORTS4(nsMenuX, nsIMenu, nsIMenuListener, nsIChangeObserver, nsISupp
 // nsMenuX constructor
 //
 nsMenuX::nsMenuX()
-    :   mNumMenuItems(0), mParent(nsnull), mManager(nsnull),
-        mMacMenuID(0), mMacMenuHandle(nsnull), mHelpMenuOSItemsCount(0),
-        mIsHelpMenu(PR_FALSE), mIsEnabled(PR_TRUE), mDestroyHandlerCalled(PR_FALSE),
+    :   mNumMenuItems(0), mParent(nsnull), mManager(nsnull), mMacMenuID(0),
+        mMacMenuHandle(nsnull), mIsEnabled(PR_TRUE), mDestroyHandlerCalled(PR_FALSE),
         mNeedsRebuild(PR_TRUE), mConstructed(PR_FALSE), mVisible(PR_TRUE), mHandler(nsnull)
 {
 #if DEBUG
@@ -154,9 +147,9 @@ nsMenuX::~nsMenuX()
 //
 NS_METHOD 
 nsMenuX::Create(nsISupports * aParent, const nsAString &aLabel, const nsAString &aAccessKey, 
-                nsIChangeManager* aManager, nsIWebShell* aShell, nsIContent* aNode )
+                nsIChangeManager* aManager, nsIDocShell* aShell, nsIContent* aNode )
 {
-  mWebShellWeakRef = do_GetWeakReference(aShell);
+  mDocShellWeakRef = do_GetWeakReference(aShell);
   mMenuContent = aNode;
 
   // register this menu to be notified when changes are made to our content object
@@ -169,23 +162,33 @@ nsMenuX::Create(nsISupports * aParent, const nsAString &aLabel, const nsAString
 
   mParent = aParent;
   // our parent could be either a menu bar (if we're toplevel) or a menu (if we're a submenu)
-  PRBool isValidParent = PR_FALSE;
-  if (aParent) {
-    nsCOMPtr<nsIMenuBar> menubar = do_QueryInterface(aParent);
-    nsCOMPtr<nsIMenu> menu = do_QueryInterface(aParent);
-    isValidParent = (menubar || menu);
-  }
-  NS_ASSERTION(isValidParent, "Menu parent not a menu bar or menu!" );
+  nsCOMPtr<nsIMenuBar> menubar = do_QueryInterface(aParent);
+  nsCOMPtr<nsIMenu> menu = do_QueryInterface(aParent);
+  NS_ASSERTION(menubar || menu, "Menu parent not a menu bar or menu!" );
 
   SetLabel(aLabel);
   SetAccessKey(aAccessKey);
 
-  nsAutoString hiddenValue, collapsedValue;
-  mMenuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::hidden, hiddenValue);
-  mMenuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::collapsed, collapsedValue);
-  if ( hiddenValue.EqualsLiteral("true") || collapsedValue.EqualsLiteral("true") )
+  if (mMenuContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::hidden,
+                                nsWidgetAtoms::_true, eCaseMatters) ||
+      mMenuContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::collapsed,
+                                nsWidgetAtoms::_true, eCaseMatters))
     mVisible = PR_FALSE;
 
+  if (menubar && mMenuContent->GetChildCount() == 0)
+    mVisible = PR_FALSE;
+
+  // We call MenuConstruct here because keyboard commands are dependent upon
+  // native menu items being created. If we only call MenuConstruct when a menu
+  // is actually selected, then we can't access keyboard commands until the
+  // menu gets selected, which is bad.
+  nsMenuEvent fake(PR_TRUE, 0, nsnull);
+  MenuConstruct(fake, nsnull, nsnull, nsnull);
+
+  if (menu)
+    mIcon = new nsMenuItemIcon(NS_STATIC_CAST(nsIMenu*, this),
+                               menu, mMenuContent);
+
   return NS_OK;
 }
 
@@ -222,13 +225,13 @@ NS_METHOD nsMenuX::SetLabel(const nsAString &aText)
 //-------------------------------------------------------------------------
 NS_METHOD nsMenuX::GetAccessKey(nsString &aText)
 {
-  return NS_OK;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 //-------------------------------------------------------------------------
 NS_METHOD nsMenuX::SetAccessKey(const nsAString &aText)
 {
-  return NS_OK;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 //-------------------------------------------------------------------------
@@ -267,11 +270,9 @@ NS_METHOD nsMenuX::AddMenuItem(nsIMenuItem * aMenuItem)
   // I want to be internationalized too!
   nsAutoString keyEquivalent(NS_LITERAL_STRING(" "));
   aMenuItem->GetShortcutChar(keyEquivalent);
-  if (!keyEquivalent.EqualsLiteral(" ")) {
+  if (!keyEquivalent.IsEmpty() && !keyEquivalent.EqualsLiteral(" ")) {
     ToUpperCase(keyEquivalent);
-    char keyStr[2];
-    keyEquivalent.ToCString(keyStr, sizeof(keyStr));
-    short inKey = keyStr[0];
+    short inKey = NS_LossyConvertUTF16toASCII(keyEquivalent).First();
     ::SetItemCmd(mMacMenuHandle, currItemIndex, inKey);
     //::SetMenuItemKeyGlyph(mMacMenuHandle, mNumMenuItems, 0x61);
   }
@@ -407,15 +408,13 @@ NS_METHOD nsMenuX::GetItemAt(const PRUint32 aPos, nsISupports *& aMenuItem)
 //-------------------------------------------------------------------------
 NS_METHOD nsMenuX::InsertItemAt(const PRUint32 aPos, nsISupports * aMenuItem)
 {
-  NS_ASSERTION(0, "Not implemented");
-  return NS_OK;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 //-------------------------------------------------------------------------
 NS_METHOD nsMenuX::RemoveItem(const PRUint32 aPos)
 {
-  NS_WARNING("Not implemented");
-  return NS_OK;
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 //-------------------------------------------------------------------------
@@ -483,19 +482,13 @@ nsEventStatus nsMenuX::MenuItemSelected(const nsMenuEvent & aMenuEvent)
 //-------------------------------------------------------------------------
 nsEventStatus nsMenuX::MenuSelected(const nsMenuEvent & aMenuEvent)
 {
-  //printf("MenuSelected called for %s \n", NS_LossyConvertUCS2toASCII(mLabel).get());
+  //printf("MenuSelected called for %s \n", NS_LossyConvertUTF16toASCII(mLabel).get());
   nsEventStatus eventStatus = nsEventStatus_eIgnore;
 
   // Determine if this is the correct menu to handle the event
   MenuHandle selectedMenuHandle = (MenuHandle) aMenuEvent.mCommand;
 
   if (mMacMenuHandle == selectedMenuHandle) {
-    if (mIsHelpMenu && mConstructed){
-      RemoveAll();
-      mConstructed = false;
-      SetRebuild(PR_TRUE);
-    }
-
     // Open the node.
     mMenuContent->SetAttr(kNameSpaceID_None, nsWidgetAtoms::open, NS_LITERAL_STRING("true"), PR_TRUE);
   
@@ -503,25 +496,21 @@ nsEventStatus nsMenuX::MenuSelected(const nsMenuEvent & aMenuEvent)
     // Fire our oncreate handler. If we're told to stop, don't build the menu at all
     PRBool keepProcessing = OnCreate();
 
-    if (!mIsHelpMenu && !mNeedsRebuild || !keepProcessing)
+    if (!mNeedsRebuild || !keepProcessing)
       return nsEventStatus_eConsumeNoDefault;
 
-    if(!mConstructed || mNeedsRebuild) {
+    if (!mConstructed || mNeedsRebuild) {
       if (mNeedsRebuild)
         RemoveAll();
 
-      nsCOMPtr<nsIWebShell> webShell = do_QueryReferent(mWebShellWeakRef);
-      if (!webShell) {
-        NS_ERROR("No web shell");
+      nsCOMPtr<nsIDocShell> docShell = do_QueryReferent(mDocShellWeakRef);
+      if (!docShell) {
+        NS_ERROR("No doc shell");
         return nsEventStatus_eConsumeNoDefault;
       }
-      if (mIsHelpMenu) {
-        HelpMenuConstruct(aMenuEvent, nsnull /* mParentWindow */, nsnull, webShell);	      
-        mConstructed = true;
-      } else {
-        MenuConstruct(aMenuEvent, nsnull /* mParentWindow */, nsnull, webShell);
-        mConstructed = true;
-      }	
+
+      MenuConstruct(aMenuEvent, nsnull /* mParentWindow */, nsnull, docShell);
+      mConstructed = true;
     } 
 
     OnCreated();  // Now that it's built, fire the popupShown event.
@@ -563,7 +552,7 @@ nsEventStatus nsMenuX::MenuConstruct(
     const nsMenuEvent & aMenuEvent,
     nsIWidget         * aParentWindow, 
     void              * /* menuNode */,
-	  void              * aWebShell)
+	  void              * aDocShell)
 {
   mConstructed = false;
   gConstructingMenu = PR_TRUE;
@@ -571,7 +560,7 @@ nsEventStatus nsMenuX::MenuConstruct(
   // reset destroy handler flag so that we'll know to fire it next time this menu goes away.
   mDestroyHandlerCalled = PR_FALSE;
   
-  //printf("nsMenuX::MenuConstruct called for %s = %d \n", NS_LossyConvertUCS2toASCII(mLabel).get(), mMacMenuHandle);
+  //printf("nsMenuX::MenuConstruct called for %s = %d \n", NS_LossyConvertUTF16toASCII(mLabel).get(), mMacMenuHandle);
   // Begin menuitem inner loop
   
   // Retrieve our menupopup.
@@ -603,51 +592,11 @@ nsEventStatus nsMenuX::MenuConstruct(
   return nsEventStatus_eIgnore;
 }
 
-//-------------------------------------------------------------------------
-nsEventStatus nsMenuX::HelpMenuConstruct(
-    const nsMenuEvent & aMenuEvent,
-    nsIWidget         * aParentWindow, 
-    void              * /* menuNode */,
-    void              * aWebShell)
-{
-  //printf("nsMenuX::MenuConstruct called for %s = %d \n", NS_LossyConvertUCS2toASCII(mLabel).get(), mMacMenuHandle);
- 
-  int numHelpItems = ::CountMenuItems(mMacMenuHandle);
-  for (int i=0; i < numHelpItems; ++i) {
-    mMenuItemsArray.AppendElement(&gDummyMenuItemX);
-  }
-     
-  // Retrieve our menupopup.
-  nsCOMPtr<nsIContent> menuPopup;
-  GetMenuPopupContent(getter_AddRefs(menuPopup));
-  if (!menuPopup)
-    return nsEventStatus_eIgnore;
-      
-  // Iterate over the kids
-  PRUint32 count = menuPopup->GetChildCount();
-  for ( PRUint32 i = 0; i < count; ++i ) {
-    nsIContent *child = menuPopup->GetChildAt(i);
-    if ( child ) {      
-      // depending on the type, create a menu item, separator, or submenu
-      nsIAtom *tag = child->Tag();
-      if ( tag == nsWidgetAtoms::menuitem )
-        LoadMenuItem(this, child);
-      else if ( tag == nsWidgetAtoms::menuseparator )
-        LoadSeparator(child);
-      else if ( tag == nsWidgetAtoms::menu )
-        LoadSubMenu(this, child);
-    }   
-  } // for each menu item
-  
-  //printf("  Done building, mMenuItemVoidArray.Count() = %d \n", mMenuItemVoidArray.Count());
-             
-  return nsEventStatus_eIgnore;
-}
 
 //-------------------------------------------------------------------------
 nsEventStatus nsMenuX::MenuDestruct(const nsMenuEvent & aMenuEvent)
 {
-  //printf("nsMenuX::MenuDestruct() called for %s \n", NS_LossyConvertUCS2toASCII(mLabel).get());
+  //printf("nsMenuX::MenuDestruct() called for %s \n", NS_LossyConvertUTF16toASCII(mLabel).get());
   
   // Fire our ondestroy handler. If we're told to stop, don't destroy the menu
   PRBool keepProcessing = OnDestroy();
@@ -675,11 +624,9 @@ nsEventStatus nsMenuX::CheckRebuild(PRBool & aNeedsRebuild)
 //-------------------------------------------------------------------------
 nsEventStatus nsMenuX::SetRebuild(PRBool aNeedsRebuild)
 {
-  if(!gConstructingMenu) {
+  if (!gConstructingMenu)
     mNeedsRebuild = aNeedsRebuild;
-    //if(mNeedsRebuild)
-    //  RemoveAll();
-  }
+
   return nsEventStatus_eIgnore;
 }
 
@@ -700,7 +647,7 @@ NS_METHOD nsMenuX::SetEnabled(PRBool aIsEnabled)
   return NS_OK;
 }
 
-//-------------------------------------------------------------------------
+
 /**
 * Get enabled state
 *
@@ -712,20 +659,7 @@ NS_METHOD nsMenuX::GetEnabled(PRBool* aIsEnabled)
   return NS_OK;
 }
 
-//-------------------------------------------------------------------------
-/**
-* Query if this is the help menu
-*
-*/
-NS_METHOD nsMenuX::IsHelpMenu(PRBool* aIsHelpMenu)
-{
-  NS_ENSURE_ARG_POINTER(aIsHelpMenu);
-  *aIsHelpMenu = mIsHelpMenu;
-  return NS_OK;
-}
 
-
-//-------------------------------------------------------------------------
 /**
 * Get GetMenuContent
 *
@@ -747,29 +681,44 @@ static pascal OSStatus MyMenuEventHandler(EventHandlerCallRef myHandler, EventRe
   OSStatus result = eventNotHandledErr;
 
   UInt32 kind = ::GetEventKind(event);
-  if (kind == kEventMenuOpening || kind == kEventMenuClosed) {
+  if (kind == kEventMenuTargetItem) {
+    // get the position of the menu item we want
+    nsIMenu* targetMenu = reinterpret_cast<nsIMenu*>(userData);
+    PRUint16 aPos;
+    ::GetEventParameter(event, kEventParamMenuItemIndex, typeMenuItemIndex, NULL, sizeof(MenuItemIndex), NULL, &aPos);
+    aPos--; // subtract 1 from aPos because Carbon menu positions start at 1 not 0
+    
+    nsISupports* aTargetMenuItem;
+    targetMenu->GetItemAt((PRUint32)aPos, aTargetMenuItem);
+    
+    // Send DOM event
+    // If the QI fails, we're over a submenu and we shouldn't send the event
+    nsCOMPtr<nsIMenuItem> bTargetMenuItem(do_QueryInterface(aTargetMenuItem));
+    if (bTargetMenuItem) {
+      PRBool handlerCalledPreventDefault; // but we don't actually care
+      bTargetMenuItem->DispatchDOMEvent(NS_LITERAL_STRING("DOMMenuItemActive"), &handlerCalledPreventDefault);
+    }
+  }
+  else if (kind == kEventMenuOpening || kind == kEventMenuClosed) {
+    if (kind == kEventMenuOpening && gRollupListener != nsnull && gRollupWidget != nsnull) {
+      gRollupListener->Rollup();
+      return userCanceledErr;
+    }
+
     nsISupports* supports = reinterpret_cast<nsISupports*>(userData);
     nsCOMPtr<nsIMenuListener> listener(do_QueryInterface(supports));
     if (listener) {
       MenuRef menuRef;
       ::GetEventParameter(event, kEventParamDirectObject, typeMenuRef, NULL, sizeof(menuRef), NULL, &menuRef);
-      nsMenuEvent menuEvent(NS_MENU_SELECTED);
+      nsMenuEvent menuEvent(PR_TRUE, NS_MENU_SELECTED, nsnull);
       menuEvent.time = PR_IntervalNow();
       menuEvent.mCommand = (PRUint32) menuRef;
-      if (kind == kEventMenuOpening) {
-        gCurrentlyTrackedMenuID = ::GetMenuID(menuRef);    // remember which menu ID we're over for later
+      if (kind == kEventMenuOpening)
         listener->MenuSelected(menuEvent);
-      }
       else
         listener->MenuDeselected(menuEvent);
     }
   }
-  else if ( kind == kEventMenuTargetItem ) {
-    // remember which menu ID we're over for later
-    MenuRef menuRef;
-    ::GetEventParameter(event, kEventParamDirectObject, typeMenuRef, NULL, sizeof(menuRef), NULL, &menuRef);
-    gCurrentlyTrackedMenuID = ::GetMenuID(menuRef);
-  }
   
   return result;
 }
@@ -778,14 +727,9 @@ static OSStatus InstallMyMenuEventHandler(MenuRef menuRef, void* userData, Event
 {
   // install the event handler for the various carbon menu events.
   static EventTypeSpec eventList[] = {
-      { kEventClassMenu, kEventMenuBeginTracking },
-      { kEventClassMenu, kEventMenuEndTracking },
-      { kEventClassMenu, kEventMenuChangeTrackingMode },
       { kEventClassMenu, kEventMenuOpening },
       { kEventClassMenu, kEventMenuClosed },
-      { kEventClassMenu, kEventMenuTargetItem },
-      { kEventClassMenu, kEventMenuMatchKey },
-      { kEventClassMenu, kEventMenuEnableItems }
+      { kEventClassMenu, kEventMenuTargetItem }
   };
   static EventHandlerUPP gMyMenuEventHandlerUPP = NewEventHandlerUPP(&MyMenuEventHandler);
   return ::InstallMenuEventHandler(menuRef, gMyMenuEventHandlerUPP,
@@ -820,25 +764,46 @@ void nsMenuX::LoadMenuItem( nsIMenu* inParentMenu, nsIContent* inMenuItemContent
     return;
 
   // if menu should be hidden, bail
-  nsAutoString hidden;
-  inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::hidden, hidden);
-  if ( hidden.EqualsLiteral("true") )
+  if (inMenuItemContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::hidden,
+                                     nsWidgetAtoms::_true, eCaseMatters))
     return;
+  
+  nsCOMPtr<nsIDOMDocument> domDocument = do_QueryInterface(inMenuItemContent->GetDocument());
+  if (!domDocument)
+    return;
+  
+  // We want the enabled state from the command element, not the menu item element.
+  // The command element is more up-to-date and it matters for keyboard shortcuts
+  // that can be invoked without opening the menu for the item. Sync menu item's
+  // enabled state with its command element now.
+  nsAutoString ourCommand;
+  inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::command, ourCommand);
+  if (!ourCommand.IsEmpty()) {
+    // get the command DOM element
+    nsCOMPtr<nsIDOMElement> commandElt;
+    domDocument->GetElementById(ourCommand, getter_AddRefs(commandElt));
+    if (commandElt) {
+      nsCOMPtr<nsIContent> commandContent = do_QueryInterface(commandElt);
+      nsAutoString menuItemDisabled;
+      nsAutoString commandDisabled;
+      inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::disabled, menuItemDisabled);
+      commandContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::disabled, commandDisabled);
+      if (!commandDisabled.Equals(menuItemDisabled)) {
+        // The menu's disabled state needs to be updated to match the command
+        if (commandDisabled.IsEmpty()) 
+          inMenuItemContent->UnsetAttr(kNameSpaceID_None, nsWidgetAtoms::disabled, PR_TRUE);
+        else
+          inMenuItemContent->SetAttr(kNameSpaceID_None, nsWidgetAtoms::disabled, commandDisabled, PR_TRUE);
+      }
+    }
+  }
 
   // Create nsMenuItem
   nsCOMPtr<nsIMenuItem> pnsMenuItem = do_CreateInstance ( kMenuItemCID ) ;
-  if ( pnsMenuItem ) {
-    nsAutoString disabled;
-    nsAutoString checked;
-    nsAutoString type;
+  if (pnsMenuItem) {
     nsAutoString menuitemName;
-    nsAutoString menuitemCmd;
     
-    inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::disabled, disabled);
-    inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::checked, checked);
-    inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::type, type);
     inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::label, menuitemName);
-    inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::command, menuitemCmd);
 
     // Bug 164155 - Carbon interprets a leading hyphen on a menu item as
     // a separator. We want a real menu item, so "escape" the hyphen by inserting
@@ -846,23 +811,24 @@ void nsMenuX::LoadMenuItem( nsIMenu* inParentMenu, nsIContent* inMenuItemContent
     if ( menuitemName[0] == '-' )
       menuitemName.Assign( NS_LITERAL_STRING("\u200c") + menuitemName );
 
-    //printf("menuitem %s \n", NS_LossyConvertUCS2toASCII(menuitemName).get());
-              
-    PRBool enabled = ! (disabled.EqualsLiteral("true"));
+    //printf("menuitem %s \n", NS_LossyConvertUTF16toASCII(menuitemName).get());
     
+    static nsIContent::AttrValuesArray strings[] =
+      {&nsWidgetAtoms::checkbox, &nsWidgetAtoms::radio, nsnull};
     nsIMenuItem::EMenuItemType itemType = nsIMenuItem::eRegular;
-    if ( type.EqualsLiteral("checkbox") )
-      itemType = nsIMenuItem::eCheckbox;
-    else if ( type.EqualsLiteral("radio") )
-      itemType = nsIMenuItem::eRadio;
+    switch (inMenuItemContent->FindAttrValueIn(kNameSpaceID_None, nsWidgetAtoms::type,
+                                               strings, eCaseMatters)) {
+      case 0: itemType = nsIMenuItem::eCheckbox; break;
+      case 1: itemType = nsIMenuItem::eRadio; break;
+    }
       
-    nsCOMPtr<nsIWebShell>  webShell = do_QueryReferent(mWebShellWeakRef);
-    if (!webShell)
+    nsCOMPtr<nsIDocShell>  docShell = do_QueryReferent(mDocShellWeakRef);
+    if (!docShell)
       return;
 
     // Create the item.
     pnsMenuItem->Create(inParentMenu, menuitemName, PR_FALSE, itemType, 
-                          enabled, mManager, webShell, inMenuItemContent);   
+                        mManager, docShell, inMenuItemContent);   
 
     //
     // Set key shortcut and modifiers
@@ -871,12 +837,7 @@ void nsMenuX::LoadMenuItem( nsIMenu* inParentMenu, nsIContent* inMenuItemContent
     nsAutoString keyValue;
     inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::key, keyValue);
 
-    // Try to find the key node. Get the document so we can do |GetElementByID|
-    nsCOMPtr<nsIDOMDocument> domDocument =
-      do_QueryInterface(inMenuItemContent->GetDocument());
-    if ( !domDocument )
-      return;
-
+    // Try to find the key node.
     nsCOMPtr<nsIDOMElement> keyElement;
     if (!keyValue.IsEmpty())
       domDocument->GetElementById(keyValue, getter_AddRefs(keyElement));
@@ -892,7 +853,7 @@ void nsMenuX::LoadMenuItem( nsIMenu* inParentMenu, nsIContent* inMenuItemContent
       keyContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::modifiers, modifiersStr);
 		  char* str = ToNewCString(modifiersStr);
 		  char* newStr;
-		  char* token = nsCRT::strtok( str, ", ", &newStr );
+		  char* token = nsCRT::strtok( str, ", \t", &newStr );
 		  while( token != NULL ) {
 		    if (PL_strcmp(token, "shift") == 0)
 		      modifiers |= knsMenuItemShiftModifier;
@@ -905,20 +866,23 @@ void nsMenuX::LoadMenuItem( nsIMenu* inParentMenu, nsIContent* inMenuItemContent
           modifiers |= knsMenuItemCommandModifier;
 		    }
 		    
-		    token = nsCRT::strtok( newStr, ", ", &newStr );
+		    token = nsCRT::strtok( newStr, ", \t", &newStr );
 		  }
 		  nsMemory::Free(str);
 
 	    pnsMenuItem->SetModifiers ( modifiers );
     }
 
-    if ( checked.EqualsLiteral("true") )
+    if (inMenuItemContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::checked,
+                                       nsWidgetAtoms::_true, eCaseMatters))
       pnsMenuItem->SetChecked(PR_TRUE);
     else
       pnsMenuItem->SetChecked(PR_FALSE);
       
     nsCOMPtr<nsISupports> supports ( do_QueryInterface(pnsMenuItem) );
     inParentMenu->AddItem(supports);         // Parent now owns menu item
+
+    pnsMenuItem->SetupIcon();
   }
 }
 
@@ -927,29 +891,27 @@ void
 nsMenuX::LoadSubMenu( nsIMenu * pParentMenu, nsIContent* inMenuItemContent )
 {
   // if menu should be hidden, bail
-  nsAutoString hidden; 
-  inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::hidden, hidden);
-  if ( hidden.EqualsLiteral("true") )
+  if (inMenuItemContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::hidden,
+                                     nsWidgetAtoms::_true, eCaseMatters))
     return;
   
   nsAutoString menuName; 
   inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::label, menuName);
-  //printf("Creating Menu [%s] \n", NS_LossyConvertUCS2toASCII(menuName).get());
+  //printf("Creating Menu [%s] \n", NS_LossyConvertUTF16toASCII(menuName).get());
 
   // Create nsMenu
   nsCOMPtr<nsIMenu> pnsMenu ( do_CreateInstance(kMenuCID) );
   if (pnsMenu) {
     // Call Create
-    nsCOMPtr<nsIWebShell> webShell = do_QueryReferent(mWebShellWeakRef);
-    if (!webShell)
+    nsCOMPtr<nsIDocShell> docShell = do_QueryReferent(mDocShellWeakRef);
+    if (!docShell)
         return;
     nsCOMPtr<nsISupports> supports(do_QueryInterface(pParentMenu));
-    pnsMenu->Create(supports, menuName, EmptyString(), mManager, webShell, inMenuItemContent);
+    pnsMenu->Create(supports, menuName, EmptyString(), mManager, docShell, inMenuItemContent);
 
     // set if it's enabled or disabled
-    nsAutoString disabled;
-    inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::disabled, disabled);
-    if ( disabled.EqualsLiteral("true") )
+    if (inMenuItemContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::disabled,
+                                       nsWidgetAtoms::_true, eCaseMatters))
       pnsMenu->SetEnabled ( PR_FALSE );
     else
       pnsMenu->SetEnabled ( PR_TRUE );
@@ -957,6 +919,8 @@ nsMenuX::LoadSubMenu( nsIMenu * pParentMenu, nsIContent* inMenuItemContent )
     // Make nsMenu a child of parent nsMenu. The parent takes ownership
     nsCOMPtr<nsISupports> supports2 ( do_QueryInterface(pnsMenu) );
 	  pParentMenu->AddItem(supports2);
+
+    pnsMenu->SetupIcon();
   }     
 }
 
@@ -965,9 +929,8 @@ void
 nsMenuX::LoadSeparator ( nsIContent* inMenuItemContent ) 
 {
   // if item should be hidden, bail
-  nsAutoString hidden;
-  inMenuItemContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::hidden, hidden);
-  if ( hidden.EqualsLiteral("true") )
+  if (inMenuItemContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::hidden,
+                                     nsWidgetAtoms::_true, eCaseMatters))
     return;
 
   AddSeparator();
@@ -985,22 +948,23 @@ PRBool
 nsMenuX::OnCreate()
 {
   nsEventStatus status = nsEventStatus_eIgnore;
-  nsMouseEvent event(NS_XUL_POPUP_SHOWING);
+  nsMouseEvent event(PR_TRUE, NS_XUL_POPUP_SHOWING, nsnull,
+                     nsMouseEvent::eReal);
   
   nsCOMPtr<nsIContent> popupContent;
   GetMenuPopupContent(getter_AddRefs(popupContent));
 
-  nsCOMPtr<nsIWebShell> webShell = do_QueryReferent(mWebShellWeakRef);
-  if (!webShell) {
-    NS_ERROR("No web shell");
+  nsCOMPtr<nsIDocShell> docShell = do_QueryReferent(mDocShellWeakRef);
+  if (!docShell) {
+    NS_ERROR("No doc shell");
     return PR_FALSE;
   }
   nsCOMPtr<nsPresContext> presContext;
-  MenuHelpersX::WebShellToPresContext(webShell, getter_AddRefs(presContext) );
+  MenuHelpersX::DocShellToPresContext(docShell, getter_AddRefs(presContext) );
   if ( presContext ) {
     nsresult rv = NS_OK;
     nsIContent* dispatchTo = popupContent ? popupContent : mMenuContent;
-    rv = dispatchTo->HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
+    rv = dispatchTo->DispatchDOMEvent(&event, nsnull, presContext, &status);
     if ( NS_FAILED(rv) || status == nsEventStatus_eConsumeNoDefault )
       return PR_FALSE;
  }
@@ -1067,22 +1031,22 @@ PRBool
 nsMenuX::OnCreated()
 {
   nsEventStatus status = nsEventStatus_eIgnore;
-  nsMouseEvent event(NS_XUL_POPUP_SHOWN);
+  nsMouseEvent event(PR_TRUE, NS_XUL_POPUP_SHOWN, nsnull, nsMouseEvent::eReal);
   
   nsCOMPtr<nsIContent> popupContent;
   GetMenuPopupContent(getter_AddRefs(popupContent));
 
-  nsCOMPtr<nsIWebShell> webShell = do_QueryReferent(mWebShellWeakRef);
-  if (!webShell) {
-    NS_ERROR("No web shell");
+  nsCOMPtr<nsIDocShell> docShell = do_QueryReferent(mDocShellWeakRef);
+  if (!docShell) {
+    NS_ERROR("No doc shell");
     return PR_FALSE;
   }
   nsCOMPtr<nsPresContext> presContext;
-  MenuHelpersX::WebShellToPresContext(webShell, getter_AddRefs(presContext) );
+  MenuHelpersX::DocShellToPresContext(docShell, getter_AddRefs(presContext) );
   if ( presContext ) {
     nsresult rv = NS_OK;
     nsIContent* dispatchTo = popupContent ? popupContent : mMenuContent;
-    rv = dispatchTo->HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
+    rv = dispatchTo->DispatchDOMEvent(&event, nsnull, presContext, &status);
     if ( NS_FAILED(rv) || status == nsEventStatus_eConsumeNoDefault )
       return PR_FALSE;
  }
@@ -1103,11 +1067,12 @@ nsMenuX::OnDestroy()
     return PR_TRUE;
 
   nsEventStatus status = nsEventStatus_eIgnore;
-  nsMouseEvent event(NS_XUL_POPUP_HIDING);
+  nsMouseEvent event(PR_TRUE, NS_XUL_POPUP_HIDING, nsnull,
+                     nsMouseEvent::eReal);
   
-  nsCOMPtr<nsIWebShell>  webShell = do_QueryReferent(mWebShellWeakRef);
-  if (!webShell) {
-    NS_WARNING("No web shell so can't run the OnDestroy");
+  nsCOMPtr<nsIDocShell>  docShell = do_QueryReferent(mDocShellWeakRef);
+  if (!docShell) {
+    NS_WARNING("No doc shell so can't run the OnDestroy");
     return PR_FALSE;
   }
 
@@ -1115,11 +1080,11 @@ nsMenuX::OnDestroy()
   GetMenuPopupContent(getter_AddRefs(popupContent));
 
   nsCOMPtr<nsPresContext> presContext;
-  MenuHelpersX::WebShellToPresContext (webShell, getter_AddRefs(presContext) );
+  MenuHelpersX::DocShellToPresContext (docShell, getter_AddRefs(presContext) );
   if (presContext )  {
     nsresult rv = NS_OK;
     nsIContent* dispatchTo = popupContent ? popupContent : mMenuContent;
-    rv = dispatchTo->HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
+    rv = dispatchTo->DispatchDOMEvent(&event, nsnull, presContext, &status);
 
     mDestroyHandlerCalled = PR_TRUE;
     
@@ -1133,11 +1098,12 @@ PRBool
 nsMenuX::OnDestroyed()
 {
   nsEventStatus status = nsEventStatus_eIgnore;
-  nsMouseEvent event(NS_XUL_POPUP_HIDDEN);
+  nsMouseEvent event(PR_TRUE, NS_XUL_POPUP_HIDDEN, nsnull,
+                     nsMouseEvent::eReal);
   
-  nsCOMPtr<nsIWebShell>  webShell = do_QueryReferent(mWebShellWeakRef);
-  if (!webShell) {
-    NS_WARNING("No web shell so can't run the OnDestroy");
+  nsCOMPtr<nsIDocShell>  docShell = do_QueryReferent(mDocShellWeakRef);
+  if (!docShell) {
+    NS_WARNING("No doc shell so can't run the OnDestroy");
     return PR_FALSE;
   }
 
@@ -1145,11 +1111,11 @@ nsMenuX::OnDestroyed()
   GetMenuPopupContent(getter_AddRefs(popupContent));
 
   nsCOMPtr<nsPresContext> presContext;
-  MenuHelpersX::WebShellToPresContext (webShell, getter_AddRefs(presContext) );
+  MenuHelpersX::DocShellToPresContext (docShell, getter_AddRefs(presContext) );
   if (presContext )  {
     nsresult rv = NS_OK;
     nsIContent* dispatchTo = popupContent ? popupContent : mMenuContent;
-    rv = dispatchTo->HandleDOMEvent(presContext, &event, nsnull, NS_EVENT_FLAG_INIT, &status);
+    rv = dispatchTo->DispatchDOMEvent(&event, nsnull, presContext, &status);
 
     mDestroyHandlerCalled = PR_TRUE;
     
@@ -1232,10 +1198,11 @@ nsMenuX :: CountVisibleBefore ( PRUint32* outVisibleBefore )
       nsCOMPtr<nsIContent> menuContent;
       currMenu->GetMenuContent(getter_AddRefs(menuContent));
       if ( menuContent ) {
-        nsAutoString hiddenValue, collapsedValue;
-        menuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::hidden, hiddenValue);
-        menuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::collapsed, collapsedValue);
-        if ( !hiddenValue.EqualsLiteral("true") && !collapsedValue.EqualsLiteral("true"))
+        if ( menuContent->GetChildCount() > 0 ||
+             !menuContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::hidden,
+                                       nsWidgetAtoms::_true, eCaseMatters) &&
+             !menuContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::collapsed,
+                                       nsWidgetAtoms::_true, eCaseMatters))
           ++(*outVisibleBefore);
       }
     }
@@ -1247,6 +1214,49 @@ nsMenuX :: CountVisibleBefore ( PRUint32* outVisibleBefore )
 } // CountVisibleBefore
 
 
+NS_IMETHODIMP
+nsMenuX::ChangeNativeEnabledStatusForMenuItem(nsIMenuItem* aMenuItem,
+                                              PRBool aEnabled)
+{
+  MenuRef menuRef;
+  PRUint16 menuItemIndex;
+  nsresult rv = GetMenuRefAndItemIndexForMenuItem(aMenuItem,
+                                                  (void**)&menuRef,
+                                                  &menuItemIndex);
+  if (NS_FAILED(rv)) return rv;
+
+  if (aEnabled)
+    ::EnableMenuItem(menuRef, menuItemIndex);
+  else
+    ::DisableMenuItem(menuRef, menuItemIndex);
+
+  return NS_OK;
+}
+
+
+NS_IMETHODIMP
+nsMenuX::GetMenuRefAndItemIndexForMenuItem(nsISupports* aMenuItem,
+                                           void**       aMenuRef,
+                                           PRUint16*    aMenuItemIndex)
+{
+  // look for the menu item given
+  PRUint32 menuItemCount;
+  mMenuItemsArray.Count(&menuItemCount);
+
+  for (PRUint32 i = 0; i < menuItemCount; i++) {
+    nsCOMPtr<nsISupports> currItem; 
+    mMenuItemsArray.GetElementAt(i, getter_AddRefs(currItem));
+    if (currItem == aMenuItem) {
+      *aMenuRef = (void*)mMacMenuHandle;
+      *aMenuItemIndex = i + 1;
+      return NS_OK;
+    }
+  }
+
+  return NS_ERROR_FAILURE;
+}
+
+
 #pragma mark -
 
 //
@@ -1255,7 +1265,7 @@ nsMenuX :: CountVisibleBefore ( PRUint32* outVisibleBefore )
 
 
 NS_IMETHODIMP
-nsMenuX::AttributeChanged(nsIDocument *aDocument, PRInt32 aNameSpaceID, nsIAtom *aAttribute)
+nsMenuX::AttributeChanged(nsIDocument *aDocument, PRInt32 aNameSpaceID, nsIContent *aContent, nsIAtom *aAttribute)
 {
   if (gConstructingMenu)
     return NS_OK;
@@ -1266,19 +1276,18 @@ nsMenuX::AttributeChanged(nsIDocument *aDocument, PRInt32 aNameSpaceID, nsIAtom
     
   nsCOMPtr<nsIMenuBar> menubarParent = do_QueryInterface(mParent);
 
-  if(aAttribute == nsWidgetAtoms::disabled) {
+  if (aAttribute == nsWidgetAtoms::disabled) {
     SetRebuild(PR_TRUE);
    
-    nsAutoString valueString;
-    mMenuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::disabled, valueString);
-    if(valueString.EqualsLiteral("true"))
+    if (mMenuContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::disabled,
+                                  nsWidgetAtoms::_true, eCaseMatters))
       SetEnabled(PR_FALSE);
     else
       SetEnabled(PR_TRUE);
       
     ::DrawMenuBar();
   } 
-  else if(aAttribute == nsWidgetAtoms::label) {
+  else if (aAttribute == nsWidgetAtoms::label) {
     SetRebuild(PR_TRUE);
     
     mMenuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::label, mLabel);
@@ -1302,14 +1311,13 @@ nsMenuX::AttributeChanged(nsIDocument *aDocument, PRInt32 aNameSpaceID, nsIAtom
       ::DrawMenuBar();
     }
   }
-  else if(aAttribute == nsWidgetAtoms::hidden || aAttribute == nsWidgetAtoms::collapsed) {
+  else if (aAttribute == nsWidgetAtoms::hidden || aAttribute == nsWidgetAtoms::collapsed) {
     SetRebuild(PR_TRUE);
       
-      nsAutoString hiddenValue, collapsedValue;
-      mMenuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::hidden, hiddenValue);
-      mMenuContent->GetAttr(kNameSpaceID_None, nsWidgetAtoms::collapsed, collapsedValue);
-        
-      if (hiddenValue.EqualsLiteral("true") || collapsedValue.EqualsLiteral("true")) {
+      if (mMenuContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::hidden,
+                                    nsWidgetAtoms::_true, eCaseMatters) ||
+          mMenuContent->AttrValueIs(kNameSpaceID_None, nsWidgetAtoms::collapsed,
+                                    nsWidgetAtoms::_true, eCaseMatters)) {
         if ( mVisible ) {
           if ( menubarParent ) {
             PRUint32 indexToRemove = 0;
@@ -1362,6 +1370,9 @@ nsMenuX::AttributeChanged(nsIDocument *aDocument, PRInt32 aNameSpaceID, nsIAtom
         ::DrawMenuBar();
       }
   }
+  else if (aAttribute == nsWidgetAtoms::image) {
+    SetupIcon();
+  }
 
   return NS_OK;
   
@@ -1395,3 +1406,15 @@ nsMenuX :: ContentInserted(nsIDocument *aDocument, nsIContent *aChild, PRInt32 a
   return NS_OK;
   
 } // ContentInserted
+
+
+NS_IMETHODIMP
+nsMenuX::SetupIcon()
+{
+  // In addition to out-of-memory, menus that are children of the menu bar
+  // will not have mIcon set.
+
+  if (!mIcon) return NS_ERROR_OUT_OF_MEMORY;
+
+  return mIcon->SetupIcon();
+}
