@@ -60,11 +60,13 @@
 #include "nsReadableUtils.h"
 
 #include "nsIPref.h"
+#include "nsClipboard.h"
 
 #include <errno.h>
 #include <photon/PtServer.h>
 
 static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
+static NS_DEFINE_CID(kCClipboardCID, NS_CLIPBOARD_CID);
 
 // BGR, not RGB - REVISIT
 #define NSCOLOR_TO_PHCOLOR(g,n) \
@@ -86,6 +88,7 @@ nsILookAndFeel     *nsWidget::sLookAndFeel = nsnull;
 #ifdef PHOTON_DND
 nsIDragService     *nsWidget::sDragService = nsnull;
 #endif
+nsClipboard        *nsWidget::sClipboard = nsnull;
 PRUint32            nsWidget::sWidgetCount = 0;
 nsWidget*						nsWidget::sFocusWidget = 0;
 
@@ -108,6 +111,14 @@ nsWidget::nsWidget()
 		}
 #endif
 
+	if( !sClipboard ) {
+		nsresult rv;
+		nsCOMPtr<nsClipboard> s;
+		s = do_GetService( kCClipboardCID, &rv );
+		sClipboard = ( nsClipboard * ) s;
+		if( NS_FAILED( rv ) ) sClipboard = 0;
+		}
+
   mWidget = nsnull;
   mParent = nsnull;
   mPreferredWidth  = 0;
@@ -229,6 +240,18 @@ NS_IMETHODIMP nsWidget::GetIMEOpenState(PRBool* aState) {
   return NS_ERROR_NOT_IMPLEMENTED;
 	}
 
+NS_IMETHODIMP nsWidget::SetIMEEnabled(PRBool aState) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+	}
+
+NS_IMETHODIMP nsWidget::GetIMEEnabled(PRBool* aState) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+	}
+
+NS_IMETHODIMP nsWidget::CancelIMEComposition() {
+  return NS_ERROR_NOT_IMPLEMENTED;
+	}
+
 //-------------------------------------------------------------------------
 //
 // Hide or show this component
@@ -358,15 +381,15 @@ PRBool nsWidget::OnResize( nsRect &aRect ) {
 
   // call the event callback
   if( mEventCallback ) {
-		nsSizeEvent event;
+		nsSizeEvent event(PR_TRUE, 0, nsnull);
 
 	  InitEvent(event, NS_SIZE);
 
 		nsRect *foo = new nsRect(0, 0, aRect.width, aRect.height);
 		event.windowSize = foo;
 
-		event.point.x = 0;
-		event.point.y = 0;
+		event.refPoint.x = 0;
+		event.refPoint.y = 0;
 		event.mWinWidth = aRect.width;
 		event.mWinHeight = aRect.height;
   
@@ -383,10 +406,10 @@ PRBool nsWidget::OnResize( nsRect &aRect ) {
 // Move
 //------
 PRBool nsWidget::OnMove( PRInt32 aX, PRInt32 aY ) {
-  nsGUIEvent event;
+  nsGUIEvent event(PR_TRUE, 0, nsnull);
   InitEvent(event, NS_MOVE);
-  event.point.x = aX;
-  event.point.y = aY;
+  event.refPoint.x = aX;
+  event.refPoint.y = aY;
   return DispatchWindowEvent(&event);
 	}
 
@@ -709,8 +732,8 @@ void nsWidget::InitMouseEvent(PhPointerEvent_t *aPhButtonEvent,
     anEvent.isControl = ( aPhButtonEvent->key_mods & Pk_KM_Ctrl )  ? PR_TRUE : PR_FALSE;
     anEvent.isAlt =     ( aPhButtonEvent->key_mods & Pk_KM_Alt )   ? PR_TRUE : PR_FALSE;
 		anEvent.isMeta =		PR_FALSE;
-    anEvent.point.x =   aPhButtonEvent->pos.x; 
-    anEvent.point.y =   aPhButtonEvent->pos.y;
+    anEvent.refPoint.x =   aPhButtonEvent->pos.x; 
+    anEvent.refPoint.y =   aPhButtonEvent->pos.y;
     anEvent.clickCount = aPhButtonEvent->click_count;
   	}
 	}
@@ -878,21 +901,24 @@ PRBool  nsWidget::DispatchKeyEvent( PhKeyEvent_t *aPhKeyEvent ) {
   w->AddRef();
  
   if (aPhKeyEvent->key_flags & Pk_KF_Key_Down) {
-		nsKeyEvent keyDownEvent(NS_KEY_DOWN, w);
+		nsKeyEvent keyDownEvent(PR_TRUE, NS_KEY_DOWN, w);
 		InitKeyEvent(aPhKeyEvent, keyDownEvent);
-		w->OnKey(keyDownEvent);
+		PRBool noDefault = w->OnKey(keyDownEvent);
 
-		nsKeyEvent keyPressEvent(NS_KEY_PRESS, w);
+		nsKeyEvent keyPressEvent(PR_TRUE, NS_KEY_PRESS, w);
 		InitKeyPressEvent(aPhKeyEvent, keyPressEvent);
+		if (noDefault) {  // If prevent default set for keydown, do same for keypress
+			keyPressEvent.flags = NS_EVENT_FLAG_NO_DEFAULT;
+		}
 		w->OnKey(keyPressEvent);
   	}
   else if (aPhKeyEvent->key_flags & Pk_KF_Key_Repeat) {
-		nsKeyEvent keyPressEvent(NS_KEY_PRESS, w);
+		nsKeyEvent keyPressEvent(PR_TRUE, NS_KEY_PRESS, w);
 		InitKeyPressEvent(aPhKeyEvent, keyPressEvent);
 		w->OnKey(keyPressEvent);
   	}
   else if (PkIsKeyDown(aPhKeyEvent->key_flags) == 0) {
-		nsKeyEvent kevent(NS_KEY_UP, w);
+		nsKeyEvent kevent(PR_TRUE, NS_KEY_UP, w);
 		InitKeyEvent(aPhKeyEvent, kevent);
 		w->OnKey(kevent);
   	}
@@ -969,7 +995,7 @@ inline PRBool nsWidget::HandleEvent( PtWidget_t *widget, PtCallbackInfo_t* aCbIn
       case Ph_EV_PTR_MOTION_NOBUTTON:
        	{
 	    	PhPointerEvent_t* ptrev = (PhPointerEvent_t*) PhGetData( event );
-	    	nsMouseEvent   theMouseEvent;
+	    	nsMouseEvent theMouseEvent(PR_TRUE, 0, nsnull, nsMouseEvent::eReal);
 
         if( ptrev ) {
 
@@ -988,7 +1014,7 @@ inline PRBool nsWidget::HandleEvent( PtWidget_t *widget, PtCallbackInfo_t* aCbIn
        {
 
 	    	PhPointerEvent_t* ptrev = (PhPointerEvent_t*) PhGetData( event );
-   		  nsMouseEvent   theMouseEvent;
+   		  nsMouseEvent theMouseEvent(PR_TRUE, 0, nsnull, nsMouseEvent::eReal);
 
 				/* there should be no reason to do this - mozilla should figure out how to call SetFocus */
 				/* this though fixes the problem with the plugins capturing the focus */
@@ -1010,7 +1036,8 @@ inline PRBool nsWidget::HandleEvent( PtWidget_t *widget, PtCallbackInfo_t* aCbIn
 
 					// if we're a right-button-up we're trying to popup a context menu. send that event to gecko also
 					if( ptrev->buttons & Ph_BUTTON_MENU ) {
-						nsMouseEvent contextMenuEvent;
+						nsMouseEvent contextMenuEvent(PR_TRUE, 0, nsnull,
+                                                      nsMouseEvent::eReal);
 						InitMouseEvent( ptrev, this, contextMenuEvent, NS_CONTEXTMENU );
 						result = DispatchMouseEvent( contextMenuEvent );
 						}
@@ -1022,8 +1049,17 @@ inline PRBool nsWidget::HandleEvent( PtWidget_t *widget, PtCallbackInfo_t* aCbIn
 		case Ph_EV_BUT_RELEASE:
 		  {
 			  PhPointerEvent_t* ptrev = (PhPointerEvent_t*) PhGetData( event );
-			  nsMouseEvent      theMouseEvent;
+			  nsMouseEvent theMouseEvent(PR_TRUE, 0, nsnull,
+                                         nsMouseEvent::eReal);
 			  
+			  // Update the current input group for clipboard mouse events
+			  // (mozilla only). Note that for mozserver the mouse based
+			  // (eg. Edit->Copy/Paste menu) events don't come through here.
+			  // They are sent by the voyager client app via libPtWeb.so to
+			  // do_command() in mozserver.cpp.
+			  if (sClipboard)
+			  	sClipboard->SetInputGroup(event->input_group);
+
 			  if (event->subtype==Ph_EV_RELEASE_REAL || event->subtype==Ph_EV_RELEASE_PHANTOM) {
 				  if (ptrev) {
 					  ScreenToWidgetPos( ptrev->pos );
@@ -1048,7 +1084,7 @@ inline PRBool nsWidget::HandleEvent( PtWidget_t *widget, PtCallbackInfo_t* aCbIn
 		case Ph_EV_PTR_MOTION_BUTTON:
       	{
         PhPointerEvent_t* ptrev = (PhPointerEvent_t*) PhGetData( event );
-	    	nsMouseEvent   theMouseEvent;
+	    	nsMouseEvent theMouseEvent(PR_TRUE, 0, nsnull, nsMouseEvent::eReal);
 
         if( ptrev ) {
 
@@ -1071,18 +1107,23 @@ inline PRBool nsWidget::HandleEvent( PtWidget_t *widget, PtCallbackInfo_t* aCbIn
       	break;
 
       case Ph_EV_KEY:
+        // Update the current input group for clipboard key events. This
+        // covers both mozserver and mozilla.
+        if (sClipboard)
+          sClipboard->SetInputGroup(event->input_group);
 				result = DispatchKeyEvent( (PhKeyEvent_t*) PhGetData( event ) );
         break;
 
       case Ph_EV_DRAG:
 	    	{
-          nsMouseEvent   theMouseEvent;
+          nsMouseEvent theMouseEvent(PR_TRUE, 0, nsnull, nsMouseEvent::eReal);
 
           switch(event->subtype) {
 
 		  			case Ph_EV_DRAG_COMPLETE: 
             	{  
- 		      		nsMouseEvent      theMouseEvent;
+ 		      		nsMouseEvent theMouseEvent(PR_TRUE, 0, nsnull,
+                                               nsMouseEvent::eReal);
               PhPointerEvent_t* ptrev2 = (PhPointerEvent_t*) PhGetData( event );
               ScreenToWidgetPos( ptrev2->pos );
               InitMouseEvent(ptrev2, this, theMouseEvent, NS_MOUSE_LEFT_BUTTON_UP );
@@ -1119,7 +1160,8 @@ inline PRBool nsWidget::HandleEvent( PtWidget_t *widget, PtCallbackInfo_t* aCbIn
 
 				if( evtype != 0 ) {
 					PhPointerEvent_t* ptrev = (PhPointerEvent_t*) PhGetData( event );
-					nsMouseEvent theMouseEvent;
+					nsMouseEvent theMouseEvent(PR_TRUE, 0, nsnull,
+                                               nsMouseEvent::eReal);
 					ScreenToWidgetPos( ptrev->pos );
 					InitMouseEvent( ptrev, this, theMouseEvent, evtype );
 					result = DispatchMouseEvent( theMouseEvent );
@@ -1221,12 +1263,12 @@ void nsWidget::ProcessDrag( PhEvent_t *event, PRUint32 aEventType, PhPoint_t *po
 
 void nsWidget::DispatchDragDropEvent( PhEvent_t *phevent, PRUint32 aEventType, PhPoint_t *pos ) {
   nsEventStatus status;
-  nsMouseEvent event;
+  nsMouseEvent event(PR_TRUE, 0, nsnull, nsMouseEvent::eReal);
 
   InitEvent( event, aEventType );
 
-  event.point.x = pos->x;
-  event.point.y = pos->y;
+  event.refPoint.x = pos->x;
+  event.refPoint.y = pos->y;
 
 	PhDndEvent_t *dnd = ( PhDndEvent_t * ) PhGetData( phevent );
   event.isControl = ( dnd->key_mods & Pk_KM_Ctrl ) ? PR_TRUE : PR_FALSE;
@@ -1236,7 +1278,7 @@ void nsWidget::DispatchDragDropEvent( PhEvent_t *phevent, PRUint32 aEventType, P
 
 	event.widget = this;
 
-///* ATENTIE */ printf("DispatchDragDropEvent pos=%d %d widget=%p\n", event.point.x, event.point.y, this );
+///* ATENTIE */ printf("DispatchDragDropEvent pos=%d %d widget=%p\n", event.refPoint.x, event.refPoint.y, this );
 
   DispatchEvent( &event, status );
 	}
