@@ -278,14 +278,15 @@ nsCommonWidget::Initialize(QWidget *widget)
 NS_IMETHODIMP
 nsCommonWidget::Show(PRBool aState)
 {
-    qDebug("MM Show");
     mIsShown = aState;
 
     // Ok, someone called show on a window that isn't sized to a sane
     // value.  Mark this window as needing to have Show() called on it
     // and return.
     if ((aState && !AreBoundsSane()) || !mWidget) {
-        qDebug("XX Bounds are insane or window hasn't been created yet");
+#ifdef DEBUG_WIDGETS
+        qWarning("XX Bounds are insane or window hasn't been created yet");
+#endif
         mNeedsShow = PR_TRUE;
         return NS_OK;
     }
@@ -297,8 +298,10 @@ nsCommonWidget::Show(PRBool aState)
     // If someone is showing this window and it needs a resize then
     // resize the widget.
     if (aState && mNeedsResize) {
+#ifdef DEBUG_WIDGETS
         qDebug("\tresizing [%d, %d, %d, %d]", mBounds.x, mBounds.y,
                mBounds.width, mBounds.height);
+#endif
         NativeResize(mBounds.x, mBounds.y, mBounds.width, mBounds.height,
                      PR_FALSE);
     }
@@ -356,7 +359,9 @@ nsCommonWidget::Move(PRInt32 x, PRInt32 y)
                      !popup))
         return NS_OK;
 
+#ifdef DEBUG_WIDGETS
     qDebug("Move [%d,%d] (%s)", x, y, popup?"popup":"widget");
+#endif
 
     if (!mWidget)
         return NS_OK;
@@ -371,9 +376,11 @@ nsCommonWidget::Move(PRInt32 x, PRInt32 y)
             mParent->WidgetToScreen(oldrect, newrect);
 
             pos = QPoint(newrect.x, newrect.y);
+#ifdef DEBUG_WIDGETS
             qDebug("pos is [%d,%d]", pos.x(), pos.y());
+#endif
         } else {
-            qDebug("NO POSITION!!!!!!!!!!!!");
+            qDebug("Widget within another? (%p)", (void*)mWidget);
         }
     }
 
@@ -462,7 +469,9 @@ NS_IMETHODIMP
 nsCommonWidget::Resize(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight,
                        PRBool aRepaint)
 {
+#ifdef DEBUG_WIDGETS
     qDebug("Resize : [%d,%d,%d,%d]", aX, aY, aWidth, aHeight);
+#endif
     if (!mWidget || (mWidget->x() == aX &&
                      mWidget->y() == aY &&
                      mWidget->height() == aHeight &&
@@ -545,13 +554,18 @@ nsCommonWidget::IsEnabled(PRBool *aState)
 }
 
 NS_IMETHODIMP
-nsCommonWidget::SetFocus(PRBool aSet)
+nsCommonWidget::SetFocus(PRBool aRaise)
 {
+#ifdef DEBUG_WIDGETS
+    qDebug("SetFocus mWidget=%p, mContainer=%p, focuswidget=%p (%d)",
+           (void*)mWidget, (void*)mContainer, (void*)mWidget->focusWidget());
+#endif
     if (mWidget) {
-        if (aSet)
-            mWidget->setFocus();
-        else
-            mWidget->clearFocus();
+        if (aRaise)
+            mWidget->raise();
+        mWidget->setFocus();
+
+        DispatchGotFocusEvent();
     }
 
     return NS_OK;
@@ -560,7 +574,9 @@ nsCommonWidget::SetFocus(PRBool aSet)
 nsIFontMetrics*
 nsCommonWidget::GetFont()
 {
-    qDebug("nsCommonWidget.cpp: GetFont not implemented");
+#ifdef DEBUG_WIDGETS
+    qWarning("nsCommonWidget.cpp: GetFont not implemented");
+#endif
     return nsnull;
 }
 
@@ -573,38 +589,46 @@ nsCommonWidget::SetFont(const nsFont&)
 NS_IMETHODIMP
 nsCommonWidget::Invalidate(PRBool aIsSynchronous)
 {
-    if (mContainer) {
-        qDebug("invalidate 1");
-        if (aIsSynchronous)
-            mContainer->update();
-        else
-            mContainer->repaint();
-    }
+#ifdef DEBUG_WIDGETS
+    qDebug("nsCommonWidget::Invalidate1 container=%p widget=%p", (void*)mContainer, (void*)mWidget);
+#endif
+    if (!mWidget)
+        return NS_OK;
+
+    if (aIsSynchronous)
+        mWidget->repaint();
+    else
+        mWidget->update();
+
     return NS_OK;
 }
 
 NS_IMETHODIMP
 nsCommonWidget::Invalidate(const nsRect & aRect, PRBool aIsSynchronous)
 {
-    if (mContainer) {
-        qDebug("invalidate 2");
-        if (aIsSynchronous)
-            mContainer->update(aRect.x, aRect.y, aRect.width, aRect.height);
-        else
-            mContainer->repaint(aRect.x, aRect.y, aRect.width, aRect.height);
-    }
+#ifdef DEBUG_WIDGETS
+    qDebug("nsCommonWidget::Invalidate2 container=%p widget=%p", (void*)mContainer, (void*)mWidget);
+#endif
+    if (!mWidget)
+        return NS_OK;
+
+    if (aIsSynchronous)
+        mWidget->repaint(aRect.x, aRect.y, aRect.width, aRect.height);
+    else
+        mWidget->update(aRect.x, aRect.y, aRect.width, aRect.height);
+
     return NS_OK;
 }
 
 NS_IMETHODIMP
 nsCommonWidget::Update()
 {
-    //qDebug("Update!!!!!!");
-    if (mContainer) {
-        mContainer->update();
-    } else if (mWidget) {
-        mWidget->update();
-    }
+#ifdef DEBUG_WIDGETS
+    qDebug("nsCommonWidget::Update container=%p widget=%p", (void*)mContainer, (void*)mWidget);
+#endif
+    if (!mWidget)
+        return NS_OK;
+    mWidget->update();
 
     return NS_OK;
 }
@@ -686,14 +710,15 @@ nsCommonWidget::SetTitle(const nsAString &str)
 NS_IMETHODIMP
 nsCommonWidget::SetMenuBar(nsIMenuBar*)
 {
-    qDebug("XXXXX SetMenuBar");
+    qWarning("XXXXX SetMenuBar");
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
 nsCommonWidget::ShowMenuBar(int)
 {
-    qDebug("XXXXX ShowMenuBar");
+
+    qWarning("XXXXX ShowMenuBar");
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -738,14 +763,14 @@ nsCommonWidget::ScreenToWidget(const nsRect &aOldRect, nsRect &aNewRect)
 NS_IMETHODIMP
 nsCommonWidget::BeginResizingChildren()
 {
-    qDebug("XXXXXX BeginResizingChildren");
+    qWarning("XXXXXX BeginResizingChildren");
     return  NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
 nsCommonWidget::EndResizingChildren()
 {
-    qDebug("XXXXXXX EndResizingChildren");
+    qWarning("XXXXXXX EndResizingChildren");
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -764,7 +789,7 @@ nsCommonWidget::GetPreferredSize(PRInt32 &aWidth, PRInt32 &aHeight)
 NS_IMETHODIMP
 nsCommonWidget::SetPreferredSize(int w, int h)
 {
-    qDebug("SetPreferredSize %d %d", w, h);
+    qWarning("XXX SetPreferredSize %d %d", w, h);
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -797,7 +822,7 @@ nsCommonWidget::CaptureRollupEvents(nsIRollupListener*, PRBool, PRBool)
 void
 nsCommonWidget::DispatchGotFocusEvent(void)
 {
-    nsGUIEvent event(NS_GOTFOCUS, this);
+    nsGUIEvent event(PR_TRUE, NS_GOTFOCUS, this);
     nsEventStatus status;
     DispatchEvent(&event, status);
 }
@@ -805,7 +830,7 @@ nsCommonWidget::DispatchGotFocusEvent(void)
 void
 nsCommonWidget::DispatchLostFocusEvent(void)
 {
-    nsGUIEvent event(NS_LOSTFOCUS, this);
+    nsGUIEvent event(PR_TRUE, NS_LOSTFOCUS, this);
     nsEventStatus status;
     DispatchEvent(&event, status);
 }
@@ -813,7 +838,7 @@ nsCommonWidget::DispatchLostFocusEvent(void)
 void
 nsCommonWidget::DispatchActivateEvent(void)
 {
-    nsGUIEvent event(NS_ACTIVATE, this);
+    nsGUIEvent event(PR_TRUE, NS_ACTIVATE, this);
     nsEventStatus status;
     DispatchEvent(&event, status);
 }
@@ -821,7 +846,7 @@ nsCommonWidget::DispatchActivateEvent(void)
 void
 nsCommonWidget::DispatchDeactivateEvent(void)
 {
-    nsGUIEvent event(NS_DEACTIVATE, this);
+    nsGUIEvent event(PR_TRUE, NS_DEACTIVATE, this);
     nsEventStatus status;
     DispatchEvent(&event, status);
 }
@@ -829,11 +854,11 @@ nsCommonWidget::DispatchDeactivateEvent(void)
 void
 nsCommonWidget::DispatchResizeEvent(nsRect &aRect, nsEventStatus &aStatus)
 {
-    nsSizeEvent event(NS_SIZE, this);
+    nsSizeEvent event(PR_TRUE, NS_SIZE, this);
 
     event.windowSize = &aRect;
-    event.point.x = aRect.x;
-    event.point.y = aRect.y;
+    event.refPoint.x = aRect.x;
+    event.refPoint.y = aRect.y;
     event.mWinWidth = aRect.width;
     event.mWinHeight = aRect.height;
 
@@ -859,7 +884,7 @@ nsCommonWidget::mousePressEvent(QMouseEvent *e)
         break;
     }
 
-    nsMouseEvent event(eventType, this);
+    nsMouseEvent event(PR_TRUE, eventType, this, nsMouseEvent::eReal);
 
     InitMouseEvent(&event, e, 1);
 
@@ -868,7 +893,8 @@ nsCommonWidget::mousePressEvent(QMouseEvent *e)
 
     // right menu click on linux should also pop up a context menu
     if (eventType == NS_MOUSE_RIGHT_BUTTON_DOWN) {
-        nsMouseEvent contextMenuEvent(NS_CONTEXTMENU, this);
+        nsMouseEvent contextMenuEvent(PR_TRUE, NS_CONTEXTMENU, this,
+                                      nsMouseEvent::eReal);
         InitMouseEvent(&contextMenuEvent, e, 1);
         DispatchEvent(&contextMenuEvent, status);
     }
@@ -894,7 +920,7 @@ nsCommonWidget::mouseReleaseEvent(QMouseEvent *e)
         break;
     }
 
-    nsMouseEvent event(eventType, this);
+    nsMouseEvent event(PR_TRUE, eventType, this, nsMouseEvent::eReal);
 
     InitMouseEvent(&event, e, 1);
 
@@ -921,7 +947,7 @@ nsCommonWidget::mouseDoubleClickEvent(QMouseEvent *e)
         break;
     }
 
-    nsMouseEvent event(eventType, this);
+    nsMouseEvent event(PR_TRUE, eventType, this, nsMouseEvent::eReal);
 
     InitMouseEvent(&event, e, 2);
     //pressed
@@ -933,7 +959,7 @@ nsCommonWidget::mouseDoubleClickEvent(QMouseEvent *e)
 bool
 nsCommonWidget::mouseMoveEvent(QMouseEvent *e)
 {
-    nsMouseEvent event(NS_MOUSE_MOVE, this);
+    nsMouseEvent event(PR_TRUE, NS_MOUSE_MOVE, this, nsMouseEvent::eReal);
 
     InitMouseEvent(&event, e, 0);
     nsEventStatus status;
@@ -944,7 +970,7 @@ nsCommonWidget::mouseMoveEvent(QMouseEvent *e)
 bool
 nsCommonWidget::wheelEvent(QWheelEvent *e)
 {
-    nsMouseScrollEvent nsEvent(NS_MOUSE_SCROLL, this);
+    nsMouseScrollEvent nsEvent(PR_TRUE, NS_MOUSE_SCROLL, this);
 
     InitMouseWheelEvent(&nsEvent, e);
 
@@ -966,28 +992,31 @@ nsCommonWidget::keyPressEvent(QKeyEvent *e)
     // release.  gtk2 already filters the extra key release events for
     // us.
 
+    nsKeyEvent pressEvent(PR_TRUE, NS_KEY_PRESS, this);
+    InitKeyEvent(&pressEvent, e);
+    pressEvent.charCode = (PRInt32)e->text()[0].unicode();
+
     if (!e->isAutoRepeat()) {
 
         // send the key down event
-        nsKeyEvent downEvent(NS_KEY_DOWN, this);
+        nsKeyEvent downEvent(PR_TRUE, NS_KEY_DOWN, this);
         InitKeyEvent(&downEvent, e);
         DispatchEvent(&downEvent, status);
+        if (ignoreEvent(status)) { // If prevent default on keydown, do same for keypress
+          pressEvent.flags |= NS_EVENT_FLAG_NO_DEFAULT;
+        }
     }
 
-    nsKeyEvent event(NS_KEY_PRESS, this);
-    InitKeyEvent(&event, e);
-    event.charCode = (PRInt32)e->text()[0].unicode();
-
     // before we dispatch a key, check if it's the context menu key.
     // If so, send a context menu key event instead.
-    if (isContextMenuKey(event)) {
-        nsMouseEvent contextMenuEvent;
-        keyEventToContextMenuEvent(&event, &contextMenuEvent);
+    if (isContextMenuKey(pressEvent)) {
+        nsMouseEvent contextMenuEvent(PR_TRUE, 0, nsnull, nsMouseEvent::eReal);
+        keyEventToContextMenuEvent(&pressEvent, &contextMenuEvent);
         DispatchEvent(&contextMenuEvent, status);
     }
     else {
         // send the key press event
-        DispatchEvent(&event, status);
+        DispatchEvent(&pressEvent, status);
     }
 
     return ignoreEvent(status);
@@ -996,7 +1025,7 @@ nsCommonWidget::keyPressEvent(QKeyEvent *e)
 bool
 nsCommonWidget::keyReleaseEvent(QKeyEvent *e)
 {
-    nsKeyEvent event(NS_KEY_UP, this);
+    nsKeyEvent event(PR_TRUE, NS_KEY_UP, this);
 
     InitKeyEvent(&event, e);
 
@@ -1008,35 +1037,43 @@ nsCommonWidget::keyReleaseEvent(QKeyEvent *e)
 bool
 nsCommonWidget::focusInEvent(QFocusEvent *)
 {
-    //qDebug("focusInEvent mWidget=%p", (void*)mWidget);
-    // dispatch a got focus event
+    if (!mWidget)
+        return FALSE;
+
+#ifdef DEBUG_WIDGETS
+    qDebug("focusInEvent mWidget=%p, mContainer=%p", (void*)mWidget, (void*)mContainer);
+#endif
+
     DispatchGotFocusEvent();
 
-    // send the activate event if it wasn't already sent via any
-    // SetFocus() calls that were the result of the GOTFOCUS event
-    // above.
-    if (mContainer)
-        DispatchActivateEvent();
+    DispatchActivateEvent();
+
     return FALSE;
 }
 
 bool
 nsCommonWidget::focusOutEvent(QFocusEvent *)
 {
-    //qDebug("focusOutEvent mWidget=%p", (void*)mWidget);
+#ifdef DEBUG_WIDGETS
+    qDebug("focusOutEvent mWidget=%p,mContainer = %p", (void*)mWidget, (void*)mContainer);
+#endif
+
     DispatchLostFocusEvent();
+    if (mContainer)
+        DispatchDeactivateEvent();
+
     return FALSE;
 }
 
 bool
 nsCommonWidget::enterEvent(QEvent *)
 {
-    nsMouseEvent event(NS_MOUSE_ENTER, this);
+    nsMouseEvent event(PR_TRUE, NS_MOUSE_ENTER, this, nsMouseEvent::eReal);
 
     QPoint pt = QCursor::pos();
 
-    event.point.x = nscoord(pt.x());
-    event.point.y = nscoord(pt.y());
+    event.refPoint.x = nscoord(pt.x());
+    event.refPoint.y = nscoord(pt.y());
 
     nsEventStatus status;
     DispatchEvent(&event, status);
@@ -1046,12 +1083,12 @@ nsCommonWidget::enterEvent(QEvent *)
 bool
 nsCommonWidget::leaveEvent(QEvent *aEvent)
 {
-    nsMouseEvent event(NS_MOUSE_EXIT, this);
+    nsMouseEvent event(PR_TRUE, NS_MOUSE_EXIT, this, nsMouseEvent::eReal);
 
     QPoint pt = QCursor::pos();
 
-    event.point.x = nscoord(pt.x());
-    event.point.y = nscoord(pt.y());
+    event.refPoint.x = nscoord(pt.x());
+    event.refPoint.y = nscoord(pt.y());
 
     nsEventStatus status;
     DispatchEvent(&event, status);
@@ -1074,9 +1111,9 @@ nsCommonWidget::paintEvent(QPaintEvent *e)
     nsCOMPtr<nsIRenderingContext> rc = getter_AddRefs(GetRenderingContext());
 
     // Generate XPFE paint event
-    nsPaintEvent event(NS_PAINT, this);
-    event.point.x = 0;
-    event.point.y = 0;
+    nsPaintEvent event(PR_TRUE, NS_PAINT, this);
+    event.refPoint.x = 0;
+    event.refPoint.y = 0;
     event.rect = &rect;
     // XXX fix this!
     event.region = nsnull;
@@ -1091,15 +1128,19 @@ nsCommonWidget::paintEvent(QPaintEvent *e)
 bool
 nsCommonWidget::moveEvent(QMoveEvent *e)
 {
-    bool shown = mWidget ? mWidget->isShown() : false;
-    bool popup = mWidget ? mWidget->isPopup() : false;
-    qDebug("moveEvent mWidget=%p %d %d (%s, %s)", (void*)mWidget, e->pos().x(), e->pos().y(),
-           shown? "shown": "hidden", popup ? "popup": "widget");
     // can we shortcut?
     if (!mWidget || (mBounds.x == e->pos().x() &&
                      mBounds.y == e->pos().y()))
         return FALSE;
-    qDebug("\tmoving");
+
+#ifdef DEBUG_WIDGETS
+    bool shown = mWidget ? mWidget->isShown() : false;
+    bool popup = mWidget ? mWidget->isPopup() : false;
+
+    qDebug("moveEvent mWidget=%p %d %d (%s, %s)", (void*)mWidget, e->pos().x(), e->pos().y(),
+           shown? "shown": "hidden", popup ? "popup": "widget");
+#endif
+
     // Toplevel windows need to have their bounds set so that we can
     // keep track of our location.  It's not often that the x,y is set
     // by the layout engine.  Width and height are set elsewhere.
@@ -1111,13 +1152,14 @@ nsCommonWidget::moveEvent(QMoveEvent *e)
         WidgetToScreen(oldrect, newrect);
         mBounds.x = newrect.x;
         mBounds.y = newrect.y;
-
+#ifdef DEBUG_WIDGETS
         qDebug("BOUNDS are [%d, %d]", mBounds.x, mBounds.y);
+#endif
     }
 
-    nsGUIEvent event(NS_MOVE, this);
-    event.point.x = pos.x();
-    event.point.y = pos.y();
+    nsGUIEvent event(PR_TRUE, NS_MOVE, this);
+    event.refPoint.x = pos.x();
+    event.refPoint.y = pos.y();
 
     // XXX mozilla will invalidate the entire window after this move
     // complete.  wtf?
@@ -1139,8 +1181,10 @@ nsCommonWidget::resizeEvent(QResizeEvent *e)
     mBounds.width = rect.width;
     mBounds.height = rect.height;
 
+#ifdef DEBUG_WIDGETS
     qDebug("resizeEvent: mWidget=%p, aWidth=%d, aHeight=%d, aX = %d, aY = %d", (void*)mWidget,
            rect.width, rect.height, rect.x, rect.y);
+#endif
 
     nsEventStatus status;
     DispatchResizeEvent(rect, status);
@@ -1150,10 +1194,10 @@ nsCommonWidget::resizeEvent(QResizeEvent *e)
 bool
 nsCommonWidget::closeEvent(QCloseEvent *)
 {
-    nsGUIEvent event(NS_XUL_CLOSE, this);
+    nsGUIEvent event(PR_TRUE, NS_XUL_CLOSE, this);
 
-    event.point.x = 0;
-    event.point.y = 0;
+    event.refPoint.x = 0;
+    event.refPoint.y = 0;
 
     nsEventStatus status;
     DispatchEvent(&event, status);
@@ -1171,65 +1215,67 @@ nsCommonWidget::contextMenuEvent(QContextMenuEvent *)
 bool
 nsCommonWidget::imStartEvent(QIMEvent *)
 {
-    qDebug("imStartEvent");
+    qWarning("XXX imStartEvent");
     return false;
 }
 
 bool
 nsCommonWidget::imComposeEvent(QIMEvent *)
 {
-    qDebug("imComposeEvent");
+    qWarning("XXX imComposeEvent");
     return false;
 }
 
 bool
 nsCommonWidget::imEndEvent(QIMEvent * )
 {
-    qDebug("imComposeEvent");
+    qWarning("XXX imComposeEvent");
     return false;
 }
 
 bool
 nsCommonWidget::dragEnterEvent(QDragEnterEvent *)
 {
-    qDebug("dragEnterEvent");
+    qDebug("XXX dragEnterEvent");
     return false;
 }
 
 bool
 nsCommonWidget::dragMoveEvent(QDragMoveEvent *)
 {
-    qDebug("dragMoveEvent");
+    qDebug("XXX dragMoveEvent");
     return false;
 }
 
 bool
 nsCommonWidget::dragLeaveEvent(QDragLeaveEvent *)
 {
-    qDebug("dragLeaveEvent");
+    qDebug("XXX dragLeaveEvent");
     return false;
 }
 
 bool
 nsCommonWidget::dropEvent(QDropEvent *)
 {
-    qDebug("dropEvent");
+    qDebug("XXX dropEvent");
     return false;
 }
 
 bool
 nsCommonWidget::showEvent(QShowEvent *)
 {
+#ifdef DEBUG_WIDGETS
     qDebug("showEvent mWidget=%p", (void*)mWidget);
+#endif
 
     QRect r = mWidget->rect();
     nsRect rect(r.x(), r.y(), r.width(), r.height());
 
     nsCOMPtr<nsIRenderingContext> rc = getter_AddRefs(GetRenderingContext());
        // Generate XPFE paint event
-    nsPaintEvent event(NS_PAINT, this);
-    event.point.x = 0;
-    event.point.y = 0;
+    nsPaintEvent event(PR_TRUE, NS_PAINT, this);
+    event.refPoint.x = 0;
+    event.refPoint.y = 0;
     event.rect = &rect;
     // XXX fix this!
     event.region = nsnull;
@@ -1245,7 +1291,9 @@ nsCommonWidget::showEvent(QShowEvent *)
 bool
 nsCommonWidget::hideEvent(QHideEvent *)
 {
+#ifdef DEBUG_WIDGETS
     qDebug("hideEvent mWidget=%p", (void*)mWidget);
+#endif
     return false;
 }
 
@@ -1276,8 +1324,8 @@ nsCommonWidget::InitKeyEvent(nsKeyEvent *nsEvent, QKeyEvent *qEvent)
 void
 nsCommonWidget::InitMouseEvent(nsMouseEvent *nsEvent, QMouseEvent *qEvent, int aClickCount)
 {
-    nsEvent->point.x = nscoord(qEvent->x());
-    nsEvent->point.y = nscoord(qEvent->y());
+    nsEvent->refPoint.x = nscoord(qEvent->x());
+    nsEvent->refPoint.y = nscoord(qEvent->y());
 
     nsEvent->isShift         = qEvent->state() & Qt::ShiftButton;
     nsEvent->isControl       = qEvent->state() & Qt::ControlButton;
@@ -1303,8 +1351,8 @@ nsCommonWidget::InitMouseWheelEvent(nsMouseScrollEvent *aEvent,
     }
     aEvent->delta = (int)((qEvent->delta() / WHEEL_DELTA) * -3);
 
-    aEvent->point.x = nscoord(qEvent->x());
-    aEvent->point.y = nscoord(qEvent->y());
+    aEvent->refPoint.x = nscoord(qEvent->x());
+    aEvent->refPoint.y = nscoord(qEvent->y());
 
     aEvent->isShift         = qEvent->state() & Qt::ShiftButton;
     aEvent->isControl       = qEvent->state() & Qt::ControlButton;
@@ -1456,30 +1504,13 @@ NS_METHOD nsCommonWidget::SetCursor(nsCursor aCursor)
     return NS_OK;
 }
 
-bool nsCommonWidget::ignoreEvent(nsEventStatus aStatus) const
-{
-      switch(aStatus) {
-    case nsEventStatus_eIgnore:
-        return(PR_FALSE);
-
-    case nsEventStatus_eConsumeNoDefault:
-        return(PR_TRUE);
-
-    case nsEventStatus_eConsumeDoDefault:
-        return(PR_FALSE);
-
-    default:
-        NS_ASSERTION(0,"Illegal nsEventStatus enumeration value");
-        break;
-    }
-    return(PR_FALSE);
-}
-
 NS_METHOD nsCommonWidget::SetModal(PRBool aModal)
 {
+#ifdef DEBUG_WIDGETS
     qDebug("------------> SetModal mWidget=%p",(void*) mWidget);
+#endif
 
-    MozQWidget *mozWidget = dynamic_cast<MozQWidget*>(mWidget);
+    MozQWidget *mozWidget = qt_cast<MozQWidget*>(mWidget);
     if (mozWidget)
         mozWidget->setModal(aModal);
 
@@ -1504,7 +1535,6 @@ nsCommonWidget::NativeShow(PRBool aState)
          qDebug("nsCommon::Show : widget empty");
          return;
     }
-    qDebug("Show (%p)  XXXXXXXXXXXXXXXXXX", (void*)mWidget);
     mWidget->setShown(aState);
 }
 
@@ -1538,9 +1568,13 @@ nsCommonWidget::NativeResize(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHe
             mParent->WidgetToScreen(oldrect, newrect);
 
             pos = QPoint(newrect.x, newrect.y);
+#ifdef DEBUG_WIDGETS
             qDebug("pos is [%d,%d]", pos.x(), pos.y());
+#endif
         } else {
-            qDebug("NO POSITION!!!!!!!!!!!!");
+#ifdef DEBUG_WIDGETS
+            qDebug("Widget with original position? (%p)", mWidget);
+#endif
         }
     }
 
