@@ -22,6 +22,7 @@
  * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
  *   Sean Echevarria <sean@beatnik.com>
+ *   David Gardiner <david.gardiner@unisa.edu.au>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -56,6 +57,7 @@
 #include "nsISupportsPrimitives.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
+#include "nsUnicharUtils.h"
 #include "nsPrimitiveHelpers.h"
 #include "nsImageClipboard.h"
 #include "nsIWidget.h"
@@ -63,7 +65,6 @@
 #include "nsWidgetsCID.h"
 #include "nsCRT.h"
 
-#include "nsVoidArray.h"
 #include "nsNetUtil.h"
 
 #include "nsIImage.h"
@@ -111,16 +112,13 @@ UINT nsClipboard::GetFormat(const char* aMimeStr)
     format = CF_TEXT;
   else if (strcmp(aMimeStr, kUnicodeMime) == 0)
     format = CF_UNICODETEXT;
+#ifndef WINCE
   else if (strcmp(aMimeStr, kJPEGImageMime) == 0)
     format = CF_DIB;
   else if (strcmp(aMimeStr, kFileMime) == 0 || 
            strcmp(aMimeStr, kFilePromiseMime) == 0)
     format = CF_HDROP;
-  else if (strcmp(aMimeStr, kURLMime) == 0 || 
-           strcmp(aMimeStr, kURLDataMime) == 0 || 
-           strcmp(aMimeStr, kURLDescriptionMime) == 0 || 
-           strcmp(aMimeStr, kFilePromiseURLMime) == 0)
-    format = CF_UNICODETEXT;
+#endif
   else if (strcmp(aMimeStr, kNativeHTMLMime) == 0)
     format = CF_HTML;
   else
@@ -130,7 +128,7 @@ UINT nsClipboard::GetFormat(const char* aMimeStr)
 }
 
 //-------------------------------------------------------------------------
-nsresult nsClipboard::CreateNativeDataObject(nsITransferable * aTransferable, IDataObject ** aDataObj)
+nsresult nsClipboard::CreateNativeDataObject(nsITransferable * aTransferable, IDataObject ** aDataObj, nsIURI * uri)
 {
   if (nsnull == aTransferable) {
     return NS_ERROR_FAILURE;
@@ -138,11 +136,14 @@ nsresult nsClipboard::CreateNativeDataObject(nsITransferable * aTransferable, ID
 
   // Create our native DataObject that implements 
   // the OLE IDataObject interface
-  nsDataObj * dataObj;
-  dataObj = new nsDataObj();
+  nsDataObj * dataObj = new nsDataObj(uri);
+
+  if (!dataObj) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
   dataObj->AddRef();
 
-  // No set it up with all the right data flavors & enums
+  // Now set it up with all the right data flavors & enums
   nsresult res = SetupNativeDataObject(aTransferable, dataObj);
   if (NS_OK == res) {
     *aDataObj = dataObj; 
@@ -215,8 +216,10 @@ nsresult nsClipboard::SetupNativeDataObject(nsITransferable * aTransferable, IDa
         dObj->AddDataFlavor(kURLMime, &shortcutFE);      
         SET_FORMATETC(shortcutFE, ::RegisterClipboardFormat(CFSTR_FILEDESCRIPTORW), 0, DVASPECT_CONTENT, -1, TYMED_HGLOBAL)
         dObj->AddDataFlavor(kURLMime, &shortcutFE);      
+#ifndef WINCE
         SET_FORMATETC(shortcutFE, ::RegisterClipboardFormat(CFSTR_FILECONTENTS), 0, DVASPECT_CONTENT, -1, TYMED_HGLOBAL)
         dObj->AddDataFlavor(kURLMime, &shortcutFE);  
+#endif
         SET_FORMATETC(shortcutFE, ::RegisterClipboardFormat(CFSTR_INETURLA), 0, DVASPECT_CONTENT, -1, TYMED_HGLOBAL)
         dObj->AddDataFlavor(kURLMime, &shortcutFE);      
         SET_FORMATETC(shortcutFE, ::RegisterClipboardFormat(CFSTR_INETURLW), 0, DVASPECT_CONTENT, -1, TYMED_HGLOBAL)
@@ -229,6 +232,7 @@ nsresult nsClipboard::SetupNativeDataObject(nsITransferable * aTransferable, IDa
         SET_FORMATETC(imageFE, CF_DIB, 0, DVASPECT_CONTENT, -1, TYMED_HGLOBAL)
         dObj->AddDataFlavor(flavorStr, &imageFE);      
       }
+#ifndef WINCE
       else if ( strcmp(flavorStr, kFilePromiseMime) == 0 ) {
          // if we're a file promise flavor, also register the 
          // CFSTR_PREFERREDDROPEFFECT format.  The data object
@@ -247,6 +251,7 @@ nsresult nsClipboard::SetupNativeDataObject(nsITransferable * aTransferable, IDa
         SET_FORMATETC(shortcutFE, ::RegisterClipboardFormat(CFSTR_PREFERREDDROPEFFECT), 0, DVASPECT_CONTENT, -1, TYMED_HGLOBAL)
         dObj->AddDataFlavor(kFilePromiseMime, &shortcutFE);
       }
+#endif
     }
   }
 
@@ -266,13 +271,13 @@ NS_IMETHODIMP nsClipboard::SetNativeClipboardData ( PRInt32 aWhichClipboard )
     return NS_ERROR_FAILURE;
   }
 
-  // Clear the native clipboard
-  ::OleSetClipboard(NULL);
-
   IDataObject * dataObj;
-  if ( NS_SUCCEEDED(CreateNativeDataObject(mTransferable, &dataObj)) ) { // this add refs
+  if ( NS_SUCCEEDED(CreateNativeDataObject(mTransferable, &dataObj, NULL)) ) { // this add refs dataObj
     ::OleSetClipboard(dataObj);
     dataObj->Release();
+  } else {
+    // Clear the native clipboard
+    ::OleSetClipboard(NULL);
   }
 
   mIgnoreEmptyNotification = PR_FALSE;
@@ -290,14 +295,14 @@ nsresult nsClipboard::GetGlobalData(HGLOBAL aHGBL, void ** aData, PRUint32 * aLe
   // null them out to ensure that all of our strlen calls will succeed.
   nsresult  result = NS_ERROR_FAILURE;
   if (aHGBL != NULL) {
-    LPSTR lpStr = (LPSTR)::GlobalLock(aHGBL);
-    DWORD allocSize = ::GlobalSize(aHGBL);
+    LPSTR lpStr = (LPSTR) GlobalLock(aHGBL);
+    DWORD allocSize = GlobalSize(aHGBL);
     char* data = NS_STATIC_CAST(char*, nsMemory::Alloc(allocSize + sizeof(PRUnichar)));
     if ( data ) {    
       memcpy ( data, lpStr, allocSize );
       data[allocSize] = data[allocSize + 1] = '\0';     // null terminate for safety
 
-      ::GlobalUnlock(aHGBL);
+      GlobalUnlock(aHGBL);
       *aData = data;
       *aLen = allocSize;
 
@@ -429,7 +434,9 @@ nsresult nsClipboard::GetNativeDataOffClipboard(IDataObject * aDataObject, UINT
   if (S_OK == hres) {
     static CLIPFORMAT fileDescriptorFlavorA = ::RegisterClipboardFormat( CFSTR_FILEDESCRIPTORA ); 
     static CLIPFORMAT fileDescriptorFlavorW = ::RegisterClipboardFormat( CFSTR_FILEDESCRIPTORW ); 
+#ifndef WINCE
     static CLIPFORMAT fileFlavor = ::RegisterClipboardFormat( CFSTR_FILECONTENTS ); 
+#endif
     switch (stm.tymed) {
      case TYMED_HGLOBAL: 
         {
@@ -464,22 +471,23 @@ nsresult nsClipboard::GetNativeDataOffClipboard(IDataObject * aDataObject, UINT
                 }
               } break;
 
+#ifndef WINCE
             case CF_DIB :
               {
-                HGLOBAL hGlobal = stm.hGlobal;
-                BYTE  * pGlobal = (BYTE  *)::GlobalLock (hGlobal) ;
-                BITMAPV4HEADER * header = (BITMAPV4HEADER *)pGlobal;
-
-                nsImageFromClipboard converter ( header );
-                nsIImage* image;
-                converter.GetImage ( &image );   // addrefs for us, don't release
-                if ( image ) {
-                  *aData = image;
-                  *aLen = sizeof(nsIImage*);
-                  result = NS_OK;
+                PRUint32 allocLen = 0;
+                unsigned char * clipboardData;
+                nsresult rv = GetGlobalData(stm.hGlobal, (void **) &clipboardData, &allocLen);
+                if (NS_SUCCEEDED(rv))
+                {
+                  nsImageFromClipboard converter;
+                  nsIInputStream * inputStream;
+                  converter.GetEncodedImageStream (clipboardData,  &inputStream );   // addrefs for us, don't release
+                  if ( inputStream ) {
+                    *aData = inputStream;
+                    *aLen = sizeof(nsIInputStream*);
+                    result = NS_OK;
+                  }
                 }
-
-                ::GlobalUnlock (hGlobal) ;
               } break;
 
             case CF_HDROP : 
@@ -488,32 +496,36 @@ nsresult nsClipboard::GetNativeDataOffClipboard(IDataObject * aDataObject, UINT
                 // single data object. In order to match mozilla's D&D apis, we
                 // just pull out the file at the requested index, pretending as
                 // if there really are multiple drag items.
-                HDROP dropFiles = (HDROP) ::GlobalLock(stm.hGlobal);
+                HDROP dropFiles = (HDROP) GlobalLock(stm.hGlobal);
 
-                UINT numFiles = ::DragQueryFile(dropFiles, 0xFFFFFFFF, NULL, 0);
+                UINT numFiles = ::DragQueryFileW(dropFiles, 0xFFFFFFFF, NULL, 0);
                 NS_ASSERTION ( numFiles > 0, "File drop flavor, but no files...hmmmm" );
                 NS_ASSERTION ( aIndex < numFiles, "Asked for a file index out of range of list" );
                 if (numFiles > 0) {
-                  UINT fileNameLen = ::DragQueryFile(dropFiles, aIndex, nsnull, 0);
-                  char* buffer = NS_REINTERPRET_CAST(char*, nsMemory::Alloc(fileNameLen + 1));
+                  UINT fileNameLen = ::DragQueryFileW(dropFiles, aIndex, nsnull, 0);
+                  PRUnichar* buffer = NS_REINTERPRET_CAST(PRUnichar*, nsMemory::Alloc((fileNameLen + 1) * sizeof(PRUnichar)));
                   if ( buffer ) {
-                    ::DragQueryFile(dropFiles, aIndex, buffer, fileNameLen + 1);
+                    ::DragQueryFileW(dropFiles, aIndex, buffer, fileNameLen + 1);
                     *aData = buffer;
-                    *aLen = fileNameLen;
+                    *aLen = fileNameLen * sizeof(PRUnichar);
                     result = NS_OK;
                   }
                   else
                     result = NS_ERROR_OUT_OF_MEMORY;
                 }
-                ::GlobalUnlock (stm.hGlobal) ;
+                GlobalUnlock (stm.hGlobal) ;
 
               } break;
 
+#endif
             default: {
+#ifndef WINCE
               if ( fe.cfFormat == fileDescriptorFlavorA || fe.cfFormat == fileDescriptorFlavorW || fe.cfFormat == fileFlavor ) {
                 NS_WARNING ( "Mozilla doesn't yet understand how to read this type of file flavor" );
               } 
-              else {
+              else
+#endif
+              {
                 // Get the data out of the global data handle. The size we return
                 // should not include the null because the other platforms don't
                 // use nulls, so just return the length we get back from strlen(),
@@ -619,13 +631,14 @@ nsresult nsClipboard::GetDataFromDataObject(IDataObject     * aDataObject,
       // Hopefully by this point we've found it and can go about our business
       if ( dataFound ) {
         nsCOMPtr<nsISupports> genericDataWrapper;
-	      if ( strcmp(flavorStr, kFileMime) == 0 ) {
-	        // we have a file path in |data|. Create an nsLocalFile object.
-	        nsDependentCString filepath(NS_REINTERPRET_CAST(char*, data));
-	        nsCOMPtr<nsILocalFile> file;
-	        if ( NS_SUCCEEDED(NS_NewNativeLocalFile(filepath, PR_FALSE, getter_AddRefs(file))) )
-	          genericDataWrapper = do_QueryInterface(file);
-	      }
+          if ( strcmp(flavorStr, kFileMime) == 0 ) {
+            // we have a file path in |data|. Create an nsLocalFile object.
+            nsDependentString filepath(NS_REINTERPRET_CAST(PRUnichar*, data));
+            nsCOMPtr<nsILocalFile> file;
+            if ( NS_SUCCEEDED(NS_NewLocalFile(filepath, PR_FALSE, getter_AddRefs(file))) )
+              genericDataWrapper = do_QueryInterface(file);
+            nsMemory::Free(data);
+          }
         else if ( strcmp(flavorStr, kNativeHTMLMime) == 0) {
           // the editor folks want CF_HTML exactly as it's on the clipboard, no conversions,
           // no fancy stuff. Pull it off the clipboard, stuff it into a wrapper and hand
@@ -633,7 +646,16 @@ nsresult nsClipboard::GetDataFromDataObject(IDataObject     * aDataObject,
           if ( FindPlatformHTML(aDataObject, anIndex, &data, &dataLen) )
             nsPrimitiveHelpers::CreatePrimitiveForData ( flavorStr, data, dataLen, getter_AddRefs(genericDataWrapper) );
           else
+          {
+            nsMemory::Free(data);
             continue;     // something wrong with this flavor, keep looking for other data
+          }
+          nsMemory::Free(data);
+        }
+        else if ( strcmp(flavorStr, kJPEGImageMime) == 0) {
+          nsIInputStream * imageStream = NS_REINTERPRET_CAST(nsIInputStream*, data);
+          genericDataWrapper = do_QueryInterface(imageStream);
+          NS_IF_RELEASE(imageStream);
         }
         else {
           // we probably have some form of text. The DOM only wants LF, so convert from Win32 line 
@@ -643,14 +665,13 @@ nsresult nsClipboard::GetDataFromDataObject(IDataObject     * aDataObject,
           dataLen = signedLen;
 
           nsPrimitiveHelpers::CreatePrimitiveForData ( flavorStr, data, dataLen, getter_AddRefs(genericDataWrapper) );
+          nsMemory::Free(data);
         }
         
         NS_ASSERTION ( genericDataWrapper, "About to put null data into the transferable" );
         aTransferable->SetTransferData(flavorStr, genericDataWrapper, dataLen);
-
-        nsMemory::Free ( NS_REINTERPRET_CAST(char*, data) );        
         res = NS_OK;
-        
+
         // we found one, get out of the loop
         break;
       }
@@ -742,9 +763,9 @@ nsClipboard :: FindURLFromLocalFile ( IDataObject* inDataObject, UINT inIndex, v
   nsresult loadResult = GetNativeDataOffClipboard(inDataObject, inIndex, GetFormat(kFileMime), outData, outDataLen);
   if ( NS_SUCCEEDED(loadResult) && *outData ) {
     // we have a file path in |data|. Is it an internet shortcut or a normal file?
-    const char* filepath = NS_REINTERPRET_CAST(char*, *outData);
+    const nsDependentString filepath(NS_STATIC_CAST(PRUnichar*, *outData));
     nsCOMPtr<nsILocalFile> file;
-    nsresult rv = NS_NewNativeLocalFile(nsDependentCString(filepath), PR_TRUE, getter_AddRefs(file));
+    nsresult rv = NS_NewLocalFile(filepath, PR_TRUE, getter_AddRefs(file));
     if (NS_FAILED(rv))
       return dataFound;
 
@@ -803,12 +824,9 @@ nsClipboard :: ResolveShortcut ( nsILocalFile* aFile, nsACString& outURL )
 // A file is an Internet Shortcut if it ends with .URL
 //
 PRBool
-nsClipboard :: IsInternetShortcut ( const char* inFileName ) 
+nsClipboard :: IsInternetShortcut ( const nsAString& inFileName ) 
 {
-  if ( strstr(inFileName, ".URL") || strstr(inFileName, ".url") )
-    return PR_TRUE;
-  
-  return PR_FALSE;
+  return StringEndsWith(inFileName, NS_LITERAL_STRING(".url"), nsCaseInsensitiveStringComparator());
 } // IsInternetShortcut
 
 
@@ -874,7 +892,7 @@ NS_IMETHODIMP nsClipboard::HasDataMatchingFlavors(nsISupportsArray *aFlavorList,
 #endif
 
       UINT format = GetFormat(flavorStr);
-      if (::IsClipboardFormatAvailable(format)) {
+      if (IsClipboardFormatAvailable(format)) {
         *_retval = PR_TRUE;
         break;
       }
@@ -884,7 +902,7 @@ NS_IMETHODIMP nsClipboard::HasDataMatchingFlavors(nsISupportsArray *aFlavorList,
         if ( strcmp(flavorStr, kUnicodeMime) == 0 ) {
           // client asked for unicode and it wasn't present, check if we have CF_TEXT.
           // We'll handle the actual data substitution in the data object.
-          if ( ::IsClipboardFormatAvailable(GetFormat(kTextMime)) )
+          if (IsClipboardFormatAvailable(GetFormat(kTextMime)) )
             *_retval = PR_TRUE;
         }
       }
