@@ -24,6 +24,7 @@
  *   Blake Ross <blaker@netscape.com>
  *   Brodie Thiesfield <brofield@jellycan.com>
  *   Masayuki Nakano <masayuki@d-toybox.com>
+ *   David Gardiner <david.gardiner@unisa.edu.au>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -40,9 +41,6 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include <ole2.h>
-#ifndef __MINGW32__
-#include <urlmon.h>
-#endif
 #include <shlobj.h>
 
 #include "nsDataObj.h"
@@ -67,12 +65,17 @@
 #include "nsCRT.h"
 #include "nsPrintfCString.h"
 #include "nsIStringBundle.h"
+#include "nsEscape.h"
+#include "nsIURL.h"
+#include "nsNetUtil.h"
+#include "nsXPCOMStrings.h"
 
-#include <ole2.h>
-#ifndef __MINGW32__
-#include <urlmon.h>
+// XXX for older version of PSDK where IAsyncOperation and related stuff is not available
+// but this should be removed when parocles config is updated
+// IAsyncOperation interface GUID
+#ifndef __IAsyncOperation_INTERFACE_DEFINED__
+  const IID IID_IAsyncOperation = {0x3D8B0590, 0xF691, 0x11d2, {0x8E, 0xA9, 0x00, 0x60, 0x97, 0xDF, 0x5B, 0xD4}};
 #endif
-#include <shlobj.h>
 
 #if 0
 #define PRNTDEBUG(_x) printf(_x);
@@ -84,6 +87,186 @@
 #define PRNTDEBUG3(_x1, _x2, _x3) // printf(_x1, _x2, _x3);
 #endif
 
+//-----------------------------------------------------------------------------
+// CStream implementation
+nsDataObj::CStream::CStream() : mRefCount(1)
+{
+}
+
+//-----------------------------------------------------------------------------
+nsDataObj::CStream::~CStream()
+{
+}
+
+//-----------------------------------------------------------------------------
+// helper - initializes the stream
+nsresult nsDataObj::CStream::Init(nsIURI *pSourceURI)
+{
+  nsresult rv;
+  rv = NS_NewChannel(getter_AddRefs(mChannel), pSourceURI);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = mChannel->Open(getter_AddRefs(mInputStream));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
+}
+
+//-----------------------------------------------------------------------------
+// IUnknown
+STDMETHODIMP nsDataObj::CStream::QueryInterface(REFIID refiid, void** ppvResult)
+{
+  *ppvResult = NULL;
+  if (IID_IUnknown == refiid ||
+      refiid == IID_IStream)
+
+  {
+    *ppvResult = this;
+  }
+
+  if (NULL != *ppvResult)
+  {
+    ((LPUNKNOWN)*ppvResult)->AddRef();
+    return S_OK;
+  }
+
+  return ResultFromScode(E_NOINTERFACE);
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP_(ULONG) nsDataObj::CStream::AddRef(void)
+{
+  return ++mRefCount;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP_(ULONG) nsDataObj::CStream::Release(void)
+{
+  ULONG nCount = --mRefCount;
+  if (nCount == 0)
+  {
+    delete this;
+    return (ULONG)0;
+  }
+
+  return mRefCount;
+}
+
+//-----------------------------------------------------------------------------
+// IStream
+STDMETHODIMP nsDataObj::CStream::Clone(IStream** ppStream)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::Commit(DWORD dwFrags)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::CopyTo(IStream* pDestStream,
+                                        ULARGE_INTEGER nBytesToCopy,
+                                        ULARGE_INTEGER* nBytesRead,
+                                        ULARGE_INTEGER* nBytesWritten)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::LockRegion(ULARGE_INTEGER nStart,
+                                            ULARGE_INTEGER nBytes,
+                                            DWORD dwFlags)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::Read(void* pvBuffer,
+                                      ULONG nBytesToRead,
+                                      ULONG* nBytesRead)
+{
+  NS_ENSURE_TRUE(mInputStream, E_FAIL);
+
+  nsresult rv;
+  PRUint32 read = 0;
+  rv = mInputStream->Read((char*)pvBuffer, nBytesToRead, &read);
+  *nBytesRead = read;
+  NS_ENSURE_SUCCESS(rv, S_FALSE);
+
+  return S_OK;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::Revert(void)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::Seek(LARGE_INTEGER nMove,
+                                      DWORD dwOrigin,
+                                      ULARGE_INTEGER* nNewPos)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::SetSize(ULARGE_INTEGER nNewSize)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::Stat(STATSTG* statstg, DWORD dwFlags)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::UnlockRegion(ULARGE_INTEGER nStart,
+                                              ULARGE_INTEGER nBytes,
+                                              DWORD dwFlags)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+STDMETHODIMP nsDataObj::CStream::Write(const void* pvBuffer,
+                                       ULONG nBytesToRead,
+                                       ULONG* nBytesRead)
+{
+  return E_NOTIMPL;
+}
+
+//-----------------------------------------------------------------------------
+HRESULT nsDataObj::CreateStream(IStream **outStream)
+{
+  NS_ENSURE_TRUE(outStream, E_INVALIDARG);
+
+  nsresult rv = NS_ERROR_FAILURE;
+  nsAutoString wideFileName;
+  nsCOMPtr<nsIURI> sourceURI;
+
+  rv = GetDownloadDetails(getter_AddRefs(sourceURI),
+                          wideFileName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsDataObj::CStream *pStream = new nsDataObj::CStream();
+  NS_ENSURE_TRUE(pStream, E_OUTOFMEMORY);
+
+  rv = pStream->Init(sourceURI);
+  if (NS_FAILED(rv))
+  {
+    pStream->Release();
+    return E_FAIL;
+  }
+  *outStream = pStream;
+
+  return S_OK;
+}
+
 ULONG nsDataObj::g_cRef = 0;
 
 EXTERN_C GUID CDECL CLSID_nsDataObj =
@@ -104,16 +287,24 @@ EXTERN_C GUID CDECL CLSID_nsDataObj =
 //-----------------------------------------------------
 // construction 
 //-----------------------------------------------------
-nsDataObj::nsDataObj()
+nsDataObj::nsDataObj(nsIURI * uri)
+: m_cRef(0), mTransferable(nsnull)
 {
-	m_cRef	        = 0;
-  mTransferable   = nsnull;
   mDataFlavors    = new nsVoidArray();
+  m_enumFE        = new CEnumFormatEtc(32);
+ 
+  m_enumFE->AddRef();
 
-  m_enumFE = new CEnumFormatEtc(32);
-  m_enumFE->AddRef();  
+  if (uri) {
+
+    // A URI was obtained, so pass this through to the DataObject
+    // so it can create a SourceURL for CF_HTML flavour
+    uri->GetSpec(mSourceURL);
+  }
+
+  mIsAsyncMode = FALSE;
+  mIsInOperation = FALSE;
 }
-
 //-----------------------------------------------------
 // destruction
 //-----------------------------------------------------
@@ -144,8 +335,12 @@ STDMETHODIMP nsDataObj::QueryInterface(REFIID riid, void** ppv)
 	if ( (IID_IUnknown == riid) || (IID_IDataObject	== riid) ) {
 		*ppv = this;
 		AddRef();
-		return NOERROR;
-	}
+		return S_OK;
+  } else if (IID_IAsyncOperation == riid) {
+    *ppv = static_cast<IAsyncOperation*>(this);
+    AddRef();
+    return S_OK;
+  }
 
 	return ResultFromScode(E_NOINTERFACE);
 }
@@ -203,10 +398,12 @@ STDMETHODIMP nsDataObj::GetData(LPFORMATETC pFE, LPSTGMEDIUM pSTM)
 
   static CLIPFORMAT fileDescriptorFlavorA = ::RegisterClipboardFormat( CFSTR_FILEDESCRIPTORA ); 
   static CLIPFORMAT fileDescriptorFlavorW = ::RegisterClipboardFormat( CFSTR_FILEDESCRIPTORW ); 
-  static CLIPFORMAT fileFlavor = ::RegisterClipboardFormat( CFSTR_FILECONTENTS ); 
   static CLIPFORMAT uniformResourceLocatorA = ::RegisterClipboardFormat( CFSTR_INETURLA );
   static CLIPFORMAT uniformResourceLocatorW = ::RegisterClipboardFormat( CFSTR_INETURLW );
+#ifndef WINCE
+  static CLIPFORMAT fileFlavor = ::RegisterClipboardFormat( CFSTR_FILECONTENTS ); 
   static CLIPFORMAT PreferredDropEffect = ::RegisterClipboardFormat( CFSTR_PREFERREDDROPEFFECT );
+#endif
 
   ULONG count;
   FORMATETC fe;
@@ -223,9 +420,6 @@ STDMETHODIMP nsDataObj::GetData(LPFORMATETC pFE, LPSTGMEDIUM pSTM)
         case CF_TEXT:
         case CF_UNICODETEXT:
         return GetText(*df, *pFE, *pSTM);
-                  
-        case CF_HDROP:
-          return GetFile(*df, *pFE, *pSTM);
 
         // Someone is asking for an image
         case CF_DIB:
@@ -242,14 +436,16 @@ STDMETHODIMP nsDataObj::GetData(LPFORMATETC pFE, LPSTGMEDIUM pSTM)
             return GetFileDescriptor ( *pFE, *pSTM, PR_FALSE );
           if ( format == fileDescriptorFlavorW )
             return GetFileDescriptor ( *pFE, *pSTM, PR_TRUE);
-          if ( format == fileFlavor )
-            return GetFileContents ( *pFE, *pSTM );
           if ( format == uniformResourceLocatorA )
             return GetUniformResourceLocator( *pFE, *pSTM, PR_FALSE);
           if ( format == uniformResourceLocatorW )
             return GetUniformResourceLocator( *pFE, *pSTM, PR_TRUE);
+#ifndef WINCE
+          if ( format == fileFlavor )
+            return GetFileContents ( *pFE, *pSTM );
           if ( format == PreferredDropEffect )
             return GetPreferredDropEffect( *pFE, *pSTM );
+#endif
           PRNTDEBUG2("***** nsDataObj::GetData - Unknown format %u\n", format);
           return GetText(*df, *pFE, *pSTM);
         } //switch
@@ -305,6 +501,7 @@ STDMETHODIMP nsDataObj::GetCanonicalFormatEtc
 STDMETHODIMP nsDataObj::SetData(LPFORMATETC pFE, LPSTGMEDIUM pSTM, BOOL fRelease)
 {
   PRNTDEBUG("nsDataObj::SetData\n");
+#ifndef WINCE
   static CLIPFORMAT PerformedDropEffect = ::RegisterClipboardFormat( CFSTR_PERFORMEDDROPEFFECT );  
 
   if (pFE && pFE->cfFormat == PerformedDropEffect) {
@@ -314,6 +511,7 @@ STDMETHODIMP nsDataObj::SetData(LPFORMATETC pFE, LPSTGMEDIUM pSTM, BOOL fRelease
       mCachedTempFile = NULL;
     }
   }
+#endif
 
   if (fRelease) {
     ReleaseStgMedium(pSTM);
@@ -375,6 +573,48 @@ STDMETHODIMP nsDataObj::EnumDAdvise(LPENUMSTATDATA *ppEnum)
 	return ResultFromScode(E_FAIL);
 }
 
+// IAsyncOperation methods
+STDMETHODIMP nsDataObj::EndOperation(HRESULT hResult,
+                                     IBindCtx *pbcReserved,
+                                     DWORD dwEffects)
+{
+  mIsInOperation = FALSE;
+  Release();
+  return S_OK;
+}
+
+STDMETHODIMP nsDataObj::GetAsyncMode(BOOL *pfIsOpAsync)
+{
+  if (!pfIsOpAsync)
+    return E_FAIL;
+
+  *pfIsOpAsync = mIsAsyncMode;
+
+  return S_OK;
+}
+
+STDMETHODIMP nsDataObj::InOperation(BOOL *pfInAsyncOp)
+{
+  if (!pfInAsyncOp)
+    return E_FAIL;
+
+  *pfInAsyncOp = mIsInOperation;
+
+  return S_OK;
+}
+
+STDMETHODIMP nsDataObj::SetAsyncMode(BOOL fDoOpAsync)
+{
+  mIsAsyncMode = fDoOpAsync;
+  return S_OK;
+}
+
+STDMETHODIMP nsDataObj::StartOperation(IBindCtx *pbcReserved)
+{
+  mIsInOperation = TRUE;
+  return S_OK;
+}
+
 //-----------------------------------------------------
 // other methods
 //-----------------------------------------------------
@@ -425,7 +665,7 @@ nsDataObj :: GetDib ( const nsACString& inFlavor, FORMATETC &, STGMEDIUM & aSTG
 {
   PRNTDEBUG("nsDataObj::GetDib\n");
   ULONG result = E_FAIL;
-  
+#ifndef WINCE  
   
   PRUint32 len = 0;
   nsCOMPtr<nsISupports> genericDataWrapper;
@@ -454,8 +694,9 @@ nsDataObj :: GetDib ( const nsACString& inFlavor, FORMATETC &, STGMEDIUM & aSTG
     }
   } // if we have an image
   else  
-    NS_WARNING ( "Definately not an image on clipboard" );
-  
+    NS_WARNING ( "Definitely not an image on clipboard" );
+
+#endif
 	return ResultFromScode(result);
 }
 
@@ -472,12 +713,21 @@ nsDataObj :: GetFileDescriptor ( FORMATETC& aFE, STGMEDIUM& aSTG, PRBool aIsUnic
   
   // How we handle this depends on if we're dealing with an internet
   // shortcut, since those are done under the covers.
-  if ( IsInternetShortcut() ) {
+  if (IsFlavourPresent(kFilePromiseMime) ||
+      IsFlavourPresent(kFileMime))
+  {
+    if (aIsUnicode)
+      return GetFileDescriptor_IStreamW(aFE, aSTG);
+    else
+      return GetFileDescriptor_IStreamA(aFE, aSTG);
+  }
+  else if (IsFlavourPresent(kURLMime))
+  {
     if ( aIsUnicode )
       res = GetFileDescriptorInternetShortcutW ( aFE, aSTG );
     else
       res = GetFileDescriptorInternetShortcutA ( aFE, aSTG );
-  } 
+  }
   else
     NS_WARNING ( "Not yet implemented\n" );
   
@@ -494,8 +744,11 @@ nsDataObj :: GetFileContents ( FORMATETC& aFE, STGMEDIUM& aSTG )
   
   // How we handle this depends on if we're dealing with an internet
   // shortcut, since those are done under the covers.
-  if ( IsInternetShortcut() )  
-    res = GetFileContentsInternetShortcut ( aFE, aSTG );
+  if (IsFlavourPresent(kFilePromiseMime) ||
+      IsFlavourPresent(kFileMime))
+    return GetFileContents_IStream(aFE, aSTG);
+  else if (IsFlavourPresent(kURLMime))
+    return GetFileContentsInternetShortcut ( aFE, aSTG );
   else
     NS_WARNING ( "Not yet implemented\n" );
 
@@ -522,9 +775,9 @@ MangleTextToValidFilename(nsString & aText)
 
   aText.StripChars(FILE_PATH_SEPARATOR  FILE_ILLEGAL_CHARACTERS);
   aText.CompressWhitespace(PR_TRUE, PR_TRUE);
-  int nameLen;
-  for (int n = 0; n < NS_ARRAY_LENGTH(forbiddenNames); ++n) {
-    nameLen = strlen(forbiddenNames[n]);
+  PRUint32 nameLen;
+  for (size_t n = 0; n < NS_ARRAY_LENGTH(forbiddenNames); ++n) {
+    nameLen = (PRUint32) strlen(forbiddenNames[n]);
     if (aText.EqualsIgnoreCase(forbiddenNames[n], nameLen)) {
       // invalid name is either the entire string, or a prefix with a period
       if (aText.Length() == nameLen || aText.CharAt(nameLen) == PRUnichar('.')) {
@@ -596,14 +849,13 @@ CreateFilenameFromTextW(nsString & aText, const wchar_t * aExtension,
   return PR_TRUE;
 }
 
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 #define PAGEINFO_PROPERTIES "chrome://navigator/locale/pageInfo.properties"
 
 static PRBool
 GetLocalizedString(const PRUnichar * aName, nsXPIDLString & aString)
 {
   nsresult rv;
-  nsCOMPtr<nsIStringBundleService> stringService = do_GetService(kStringBundleServiceCID, &rv);
+  nsCOMPtr<nsIStringBundleService> stringService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
   if (NS_FAILED(rv)) 
     return PR_FALSE;
 
@@ -625,6 +877,9 @@ GetLocalizedString(const PRUnichar * aName, nsXPIDLString & aString)
 HRESULT
 nsDataObj :: GetFileDescriptorInternetShortcutA ( FORMATETC& aFE, STGMEDIUM& aSTG )
 {
+#ifdef WINCE
+  return E_FAIL;
+#else
   // get the title of the shortcut
   nsAutoString title;
   if ( NS_FAILED(ExtractShortcutTitle(title)) )
@@ -644,11 +899,11 @@ nsDataObj :: GetFileDescriptorInternetShortcutA ( FORMATETC& aFE, STGMEDIUM& aST
   // get a valid filename in the following order: 1) from the page title, 
   // 2) localized string for an untitled page, 3) just use "Untitled.URL"
   if (!CreateFilenameFromTextA(title, ".URL", 
-          fileGroupDescA->fgd[0].cFileName, NS_MAX_FILEDESCRIPTOR)) {
+                               fileGroupDescA->fgd[0].cFileName, NS_MAX_FILEDESCRIPTOR)) {
     nsXPIDLString untitled;
     if (!GetLocalizedString(NS_LITERAL_STRING("noPageTitle").get(), untitled) ||
         !CreateFilenameFromTextA(untitled, ".URL", 
-            fileGroupDescA->fgd[0].cFileName, NS_MAX_FILEDESCRIPTOR)) {
+                                 fileGroupDescA->fgd[0].cFileName, NS_MAX_FILEDESCRIPTOR)) {
       strcpy(fileGroupDescA->fgd[0].cFileName, "Untitled.URL");
     }
   }
@@ -662,12 +917,15 @@ nsDataObj :: GetFileDescriptorInternetShortcutA ( FORMATETC& aFE, STGMEDIUM& aST
   aSTG.tymed = TYMED_HGLOBAL;
 
   return S_OK;
-
+#endif
 } // GetFileDescriptorInternetShortcutA
 
 HRESULT
 nsDataObj :: GetFileDescriptorInternetShortcutW ( FORMATETC& aFE, STGMEDIUM& aSTG )
 {
+#ifdef WINCE
+  return E_FAIL;
+#else
   // get the title of the shortcut
   nsAutoString title;
   if ( NS_FAILED(ExtractShortcutTitle(title)) )
@@ -687,11 +945,11 @@ nsDataObj :: GetFileDescriptorInternetShortcutW ( FORMATETC& aFE, STGMEDIUM& aST
   // get a valid filename in the following order: 1) from the page title, 
   // 2) localized string for an untitled page, 3) just use "Untitled.URL"
   if (!CreateFilenameFromTextW(title, NS_LITERAL_STRING(".URL").get(), 
-          fileGroupDescW->fgd[0].cFileName, NS_MAX_FILEDESCRIPTOR)) {
+                               fileGroupDescW->fgd[0].cFileName, NS_MAX_FILEDESCRIPTOR)) {
     nsXPIDLString untitled;
     if (!GetLocalizedString(NS_LITERAL_STRING("noPageTitle").get(), untitled) ||
         !CreateFilenameFromTextW(untitled, NS_LITERAL_STRING(".URL").get(), 
-            fileGroupDescW->fgd[0].cFileName, NS_MAX_FILEDESCRIPTOR)) {
+                                 fileGroupDescW->fgd[0].cFileName, NS_MAX_FILEDESCRIPTOR)) {
       wcscpy(fileGroupDescW->fgd[0].cFileName, NS_LITERAL_STRING("Untitled.URL").get());
     }
   }
@@ -705,7 +963,7 @@ nsDataObj :: GetFileDescriptorInternetShortcutW ( FORMATETC& aFE, STGMEDIUM& aST
   aSTG.tymed = TYMED_HGLOBAL;
 
   return S_OK;
-
+#endif
 } // GetFileDescriptorInternetShortcutW
 
 
@@ -718,6 +976,9 @@ nsDataObj :: GetFileDescriptorInternetShortcutW ( FORMATETC& aFE, STGMEDIUM& aST
 HRESULT
 nsDataObj :: GetFileContentsInternetShortcut ( FORMATETC& aFE, STGMEDIUM& aSTG )
 {
+#ifdef WINCE
+  return E_FAIL;
+#else
   nsAutoString url;
   if ( NS_FAILED(ExtractShortcutURL(url)) )
     return E_OUTOFMEMORY;
@@ -752,42 +1013,31 @@ nsDataObj :: GetFileContentsInternetShortcut ( FORMATETC& aFE, STGMEDIUM& aSTG )
   aSTG.tymed = TYMED_HGLOBAL;
 
   return S_OK;
-  
+#endif  
 } // GetFileContentsInternetShortcut
 
-
-//
-// IsInternetShortcut
-//
-// Is the data that we're handling destined for an internet shortcut if
-// dragged to the desktop? We can tell because there will be a mime type
-// of |kURLMime| present in the transferable
-//
-PRBool
-nsDataObj :: IsInternetShortcut ( )
+// check if specified flavour is present in the transferable
+PRBool nsDataObj :: IsFlavourPresent(const char *inFlavour)
 {
   PRBool retval = PR_FALSE;
-  
-  if ( !mTransferable )
-    return PR_FALSE;
+  NS_ENSURE_TRUE(mTransferable, PR_FALSE);
   
   // get the list of flavors available in the transferable
   nsCOMPtr<nsISupportsArray> flavorList;
-  mTransferable->FlavorsTransferableCanExport ( getter_AddRefs(flavorList) );
-  if ( !flavorList )
-    return PR_FALSE;
-  
-  // go spelunking for the url flavor
+  mTransferable->FlavorsTransferableCanExport(getter_AddRefs(flavorList));
+  NS_ENSURE_TRUE(flavorList, PR_FALSE);
+
+  // try to find requested flavour
   PRUint32 cnt;
   flavorList->Count(&cnt);
-  for ( PRUint32 i = 0;i < cnt; ++i ) {
+  for (PRUint32 i = 0; i < cnt; ++i) {
     nsCOMPtr<nsISupports> genericFlavor;
     flavorList->GetElementAt (i, getter_AddRefs(genericFlavor));
     nsCOMPtr<nsISupportsCString> currentFlavor (do_QueryInterface(genericFlavor));
     if (currentFlavor) {
       nsCAutoString flavorStr;
       currentFlavor->GetData(flavorStr);
-      if ( flavorStr.Equals(kURLMime) ) {
+      if (flavorStr.Equals(inFlavour)) {
         retval = PR_TRUE;         // found it!
         break;
       }
@@ -795,8 +1045,7 @@ nsDataObj :: IsInternetShortcut ( )
   } // for each flavor
 
   return retval;
-  
-} // IsInternetShortcut
+}
 
 HRESULT nsDataObj::GetPreferredDropEffect ( FORMATETC& aFE, STGMEDIUM& aSTG )
 {
@@ -806,7 +1055,7 @@ HRESULT nsDataObj::GetPreferredDropEffect ( FORMATETC& aFE, STGMEDIUM& aSTG )
   HGLOBAL hGlobalMemory = NULL;
   hGlobalMemory = ::GlobalAlloc(GMEM_MOVEABLE, sizeof(DWORD));
   if (hGlobalMemory) {
-    DWORD* pdw = (DWORD*) ::GlobalLock(hGlobalMemory);
+    DWORD* pdw = (DWORD*) GlobalLock(hGlobalMemory);
     // The PreferredDropEffect clipboard format is only registered if a drag/drop
     // of an image happens from Mozilla to the desktop.  We want its value
     // to be DROPEFFECT_MOVE in that case so that the file is moved from the
@@ -815,7 +1064,7 @@ HRESULT nsDataObj::GetPreferredDropEffect ( FORMATETC& aFE, STGMEDIUM& aSTG )
     // our IDataObject implementation doesn't implement SetData.  It adds data
     // to the data object lazily only when the drop target asks for it.
     *pdw = (DWORD) DROPEFFECT_MOVE;
-    ::GlobalUnlock(hGlobalMemory);
+    GlobalUnlock(hGlobalMemory);
   }
   else {
     res = E_OUTOFMEMORY;
@@ -881,7 +1130,7 @@ HRESULT nsDataObj::GetText(const nsACString & aDataFlavor, FORMATETC& aFE, STGME
   else if ( aFE.cfFormat == nsClipboard::CF_HTML ) {
     // Someone is asking for win32's HTML flavor. Convert our html fragment
     // from unicode to UTF-8 then put it into a format specified by msft.
-    NS_ConvertUCS2toUTF8 converter ( NS_REINTERPRET_CAST(PRUnichar*, data) );
+    NS_ConvertUTF16toUTF8 converter ( NS_REINTERPRET_CAST(PRUnichar*, data) );
     char* utf8HTML = nsnull;
     nsresult rv = BuildPlatformHTML ( converter.get(), &utf8HTML );      // null terminates
     
@@ -902,14 +1151,14 @@ HRESULT nsDataObj::GetText(const nsACString & aDataFlavor, FORMATETC& aFE, STGME
     allocLen += sizeof(PRUnichar);
   }
 
-  hGlobalMemory = (HGLOBAL)::GlobalAlloc(GMEM_MOVEABLE, allocLen);
+  hGlobalMemory = (HGLOBAL)GlobalAlloc(GMEM_MOVEABLE, allocLen);
 
   // Copy text to Global Memory Area
   if ( hGlobalMemory ) {
-    char* dest = NS_REINTERPRET_CAST(char*, ::GlobalLock(hGlobalMemory));
+    char* dest = NS_REINTERPRET_CAST(char*, GlobalLock(hGlobalMemory));
     char* source = NS_REINTERPRET_CAST(char*, data);
     memcpy ( dest, source, allocLen );                         // copies the null as well
-    ::GlobalUnlock(hGlobalMemory);
+    GlobalUnlock(hGlobalMemory);
   }
   aSTG.hGlobal = hGlobalMemory;
 
@@ -919,79 +1168,6 @@ HRESULT nsDataObj::GetText(const nsACString & aDataFlavor, FORMATETC& aFE, STGME
   return ResultFromScode(S_OK);
 }
 
-//-----------------------------------------------------
-HRESULT nsDataObj::GetFile(const nsACString & aDataFlavor, FORMATETC& aFE, STGMEDIUM& aSTG)
-{
-  HRESULT res = S_OK;
-  if (strcmp(PromiseFlatCString(aDataFlavor).get(), kFilePromiseMime) == 0) {
-    // XXX We are copying the file twice below.  Once from the original location to the temporary
-    // one (here) and then from the temporary location to the drop location (done by the drop target
-    // when this function returns).  If the file being drag-dropped is in the cache, we should be able to
-    // just pass back its file name to the drop target and save a copy.  Need help here!
-    nsresult rv;
-    if (!mCachedTempFile) {
-      // Save the file to a temporary location.      
-      nsCOMPtr<nsILocalFile> dropDirectory;
-      nsCOMPtr<nsIProperties> directoryService = 
-        do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
-      if (!directoryService || NS_FAILED(rv)) return ResultFromScode(E_FAIL);
-      directoryService->Get(NS_OS_TEMP_DIR, 
-                            NS_GET_IID(nsIFile), 
-                            getter_AddRefs(dropDirectory));
-      nsCOMPtr<nsISupports> localFileISupports = do_QueryInterface(dropDirectory);
-      rv = mTransferable->SetTransferData(kFilePromiseDirectoryMime, localFileISupports, sizeof(nsILocalFile*));
-      if (NS_FAILED(rv)) return ResultFromScode(E_FAIL);
-      nsCOMPtr<nsISupports> fileDataPrimitive;
-      PRUint32 dataSize = 0;
-      rv = mTransferable->GetTransferData(kFilePromiseMime, getter_AddRefs(fileDataPrimitive), &dataSize);
-      if (NS_FAILED(rv)) return ResultFromScode(E_FAIL);      
-      
-      mCachedTempFile = do_QueryInterface(fileDataPrimitive);
-      if (!mCachedTempFile) return ResultFromScode(E_FAIL);
-    }
-
-    // Pass the file name back to the drop target so that it can copy to the drop location
-    nsCAutoString path;
-    rv = mCachedTempFile->GetNativePath(path);
-    if (NS_FAILED(rv)) return ResultFromScode(E_FAIL);
-    const char* pFileName = path.get();
-    // Two null characters are needed to terminate the file name list
-    PRUint32 allocLen = path.Length() + 2;
-    HGLOBAL hGlobalMemory = NULL;
-    aSTG.tymed = TYMED_HGLOBAL;
-    aSTG.pUnkForRelease = NULL;    
-    hGlobalMemory = ::GlobalAlloc(GMEM_MOVEABLE, sizeof(DROPFILES) + allocLen);
-    if (hGlobalMemory) {      
-      DROPFILES* pDropFile = NS_REINTERPRET_CAST(DROPFILES*, ::GlobalLock(hGlobalMemory));
-
-      // First, populate drop file structure
-      pDropFile->pFiles = sizeof(DROPFILES);   // offset to start of file name char array
-      pDropFile->fNC = 0;
-      pDropFile->pt.x = 0;
-      pDropFile->pt.y = 0;
-      pDropFile->fWide = 0;
-      
-      // Copy the filename right after the DROPFILES structure
-      char* dest = NS_REINTERPRET_CAST(char*, pDropFile);
-      dest += pDropFile->pFiles;
-      const char* source = pFileName;
-      memcpy (NS_REINTERPRET_CAST(char*, dest), source, allocLen - 1);    // copies the null character in pFileName as well
-
-      // Two null characters are needed at the end of the file name.  
-      // Lookup the CF_HDROP shell clipboard format for more info.
-      // Add the second null character right after the first one.
-      dest[allocLen - 1] = '\0';
-
-      ::GlobalUnlock(hGlobalMemory);
-    }
-    else {
-      res = E_OUTOFMEMORY;
-    }
-    aSTG.hGlobal = hGlobalMemory;
-  }
-  return res;
-}
-
 //-----------------------------------------------------
 HRESULT nsDataObj::GetMetafilePict(FORMATETC&, STGMEDIUM&)
 {
@@ -1055,15 +1231,15 @@ void nsDataObj::AddDataFlavor(const char* aDataFlavor, LPFORMATETC aFE)
   // so we will look up data flavor that corresponds to the FE
   // and then ask the transferable for that type of data
 
-  // Add CF_HDROP to the head of the list so that it is returned first
-  // when the drop target calls EnumFormatEtc to enumerate through the FE list
-  // We use the CF_HDROP format to copy file contents when an image is dragged
-  // from Mozilla to a drop target.
+  // Just ignore the CF_HDROP here
+  // all file drags are now hangled by CFSTR_FileContents format
+#ifndef WINCE
   if (aFE->cfFormat == CF_HDROP) {
-    mDataFlavors->InsertElementAt(new nsCAutoString(aDataFlavor), 0);
-    m_enumFE->InsertFEAt(aFE, 0);
+    return;
   }  
-  else {
+  else 
+#endif
+  {
     mDataFlavors->AppendElement(new nsCAutoString(aDataFlavor));
     m_enumFE->AddFE(aFE);
   }
@@ -1099,7 +1275,7 @@ void nsDataObj::SetTransferable(nsITransferable * aTransferable)
 nsresult
 nsDataObj :: ExtractShortcutURL ( nsString & outURL )
 {
-  NS_ASSERTION ( mTransferable, "We'd don't have a good transferable" );
+  NS_ASSERTION ( mTransferable, "We don't have a good transferable" );
   nsresult rv = NS_ERROR_FAILURE;
   
   PRUint32 len = 0;
@@ -1120,7 +1296,18 @@ nsDataObj :: ExtractShortcutURL ( nsString & outURL )
         rv = NS_OK;    
       }
     }
-  } // if found flavor
+  } else if ( NS_SUCCEEDED(mTransferable->GetTransferData(kURLDataMime, getter_AddRefs(genericURL), &len)) ||
+              NS_SUCCEEDED(mTransferable->GetTransferData(kURLPrivateMime, getter_AddRefs(genericURL), &len)) ) {
+    nsCOMPtr<nsISupportsString> urlObject ( do_QueryInterface(genericURL) );
+    if ( urlObject ) {
+      nsAutoString url;
+      urlObject->GetData ( url );
+      outURL = url;
+
+      rv = NS_OK;    
+    }
+
+  }  // if found flavor
   
   return rv;
 
@@ -1189,8 +1376,18 @@ nsDataObj :: BuildPlatformHTML ( const char* inOurHTML, char** outPlatformHTML )
   const char* const endHTMLPrefix   = "\r\nEndHTML:";
   const char* const startFragPrefix = "\r\nStartFragment:";
   const char* const endFragPrefix   = "\r\nEndFragment:";
+  const char* const startSourceURLPrefix = "\r\nSourceURL:";
   const char* const endFragTrailer  = "\r\n";
 
+  // Do we already have mSourceURL from a drag?
+  if (mSourceURL.IsEmpty()) {
+    nsAutoString url;
+    ExtractShortcutURL(url);
+
+    AppendUTF16toUTF8(url, mSourceURL);
+  }
+
+  const PRInt32 kSourceURLLength    = mSourceURL.Length();
   const PRInt32 kNumberLength       = strlen(numPlaceholder);
 
   const PRInt32 kTotalHeaderLen     = strlen(startHTMLPrefix) +
@@ -1198,6 +1395,8 @@ nsDataObj :: BuildPlatformHTML ( const char* inOurHTML, char** outPlatformHTML )
                                       strlen(startFragPrefix) + 
                                       strlen(endFragPrefix) + 
                                       strlen(endFragTrailer) +
+                                      (kSourceURLLength > 0 ? strlen(startSourceURLPrefix) : 0) +
+                                      kSourceURLLength +
                                       (4 * kNumberLength);
 
   NS_NAMED_LITERAL_CSTRING(htmlHeaderString, "<html><body>\r\n");
@@ -1237,6 +1436,11 @@ nsDataObj :: BuildPlatformHTML ( const char* inOurHTML, char** outPlatformHTML )
   clipboardString.Append(endFragPrefix);
   clipboardString.Append(nsPrintfCString("%08u", endFragOffset));
 
+  if (kSourceURLLength > 0) {
+    clipboardString.Append(startSourceURLPrefix);
+    clipboardString.Append(mSourceURL);
+  }
+
   clipboardString.Append(endFragTrailer);
 
   clipboardString.Append(htmlHeaderString);
@@ -1255,7 +1459,7 @@ HRESULT
 nsDataObj :: GetUniformResourceLocator( FORMATETC& aFE, STGMEDIUM& aSTG, PRBool aIsUnicode )
 {
   HRESULT res = S_OK;
-  if ( IsInternetShortcut() ) {
+  if (IsFlavourPresent(kURLMime)) {
     if ( aIsUnicode )
       res = ExtractUniformResourceLocatorW( aFE, aSTG );
     else
@@ -1277,18 +1481,18 @@ nsDataObj::ExtractUniformResourceLocatorA(FORMATETC& aFE, STGMEDIUM& aSTG )
 
   NS_LossyConvertUTF16toASCII asciiUrl(url);
   const int totalLen = asciiUrl.Length() + 1;
-  HGLOBAL hGlobalMemory = ::GlobalAlloc(GMEM_ZEROINIT|GMEM_SHARE, totalLen);
+  HGLOBAL hGlobalMemory = GlobalAlloc(GMEM_ZEROINIT|GMEM_SHARE, totalLen);
   if (!hGlobalMemory)
     return E_OUTOFMEMORY;
 
-  char* contents = NS_REINTERPRET_CAST(char*, ::GlobalLock(hGlobalMemory));
+  char* contents = NS_REINTERPRET_CAST(char*, GlobalLock(hGlobalMemory));
   if (!contents) {
-    ::GlobalFree(hGlobalMemory);
+    GlobalFree(hGlobalMemory);
     return E_OUTOFMEMORY;
   }
 
   strcpy(contents, asciiUrl.get());
-  ::GlobalUnlock(hGlobalMemory);
+  GlobalUnlock(hGlobalMemory);
   aSTG.hGlobal = hGlobalMemory;
   aSTG.tymed = TYMED_HGLOBAL;
 
@@ -1305,20 +1509,163 @@ nsDataObj::ExtractUniformResourceLocatorW(FORMATETC& aFE, STGMEDIUM& aSTG )
     return E_OUTOFMEMORY;
 
   const int totalLen = (url.Length() + 1) * sizeof(PRUnichar);
-  HGLOBAL hGlobalMemory = ::GlobalAlloc(GMEM_ZEROINIT|GMEM_SHARE, totalLen);
+  HGLOBAL hGlobalMemory = GlobalAlloc(GMEM_ZEROINIT|GMEM_SHARE, totalLen);
   if (!hGlobalMemory)
     return E_OUTOFMEMORY;
 
-  wchar_t* contents = NS_REINTERPRET_CAST(wchar_t*, ::GlobalLock(hGlobalMemory));
+  wchar_t* contents = NS_REINTERPRET_CAST(wchar_t*, GlobalLock(hGlobalMemory));
   if (!contents) {
-    ::GlobalFree(hGlobalMemory);
+    GlobalFree(hGlobalMemory);
     return E_OUTOFMEMORY;
   }
 
   wcscpy(contents, url.get());
-  ::GlobalUnlock(hGlobalMemory);
+  GlobalUnlock(hGlobalMemory);
   aSTG.hGlobal = hGlobalMemory;
   aSTG.tymed = TYMED_HGLOBAL;
 
   return result;
 }
+
+
+// Gets the filename from the kFilePromiseURLMime flavour
+nsresult nsDataObj::GetDownloadDetails(nsIURI **aSourceURI,
+                                       nsAString &aFilename)
+{
+  NS_ENSURE_TRUE(mTransferable, NS_ERROR_FAILURE);
+
+  // get the URI from the kFilePromiseURLMime flavor
+  nsCOMPtr<nsISupports> urlPrimitive;
+  PRUint32 dataSize = 0;
+  mTransferable->GetTransferData(kFilePromiseURLMime, getter_AddRefs(urlPrimitive), &dataSize);
+  nsCOMPtr<nsISupportsString> srcUrlPrimitive = do_QueryInterface(urlPrimitive);
+  NS_ENSURE_TRUE(srcUrlPrimitive, NS_ERROR_FAILURE);
+  
+  // Get data for flavor
+  // The format of the data is (URLSTRING\nFILENAME)
+  nsAutoString strData;
+  srcUrlPrimitive->GetData(strData);
+  if (strData.IsEmpty())
+    return NS_ERROR_FAILURE;
+
+  // Now figure if there is a "\n" delimiter in the data string.
+  // If there is, the string after the "\n" is a filename.
+  // If there is no delimiter then just get the filename from the url.
+  nsCAutoString strFileName;
+  nsCOMPtr<nsIURI> sourceURI;
+  // New line char is used as a delimiter (hardcoded)
+  PRInt32 nPos = strData.FindChar('\n');
+  // Store source uri
+  NS_NewURI(aSourceURI, Substring(strData, 0, nPos));
+  if (nPos != -1) {
+    // if there is delimiter
+    CopyUTF16toUTF8(Substring(strData, nPos + 1, strData.Length()), strFileName);
+  } else {
+    // no filename was supplied - try to get it from a URL
+    nsCOMPtr<nsIURL> sourceURL = do_QueryInterface(*aSourceURI);
+    sourceURL->GetFileName(strFileName);
+  }
+  // check for an error; the URL must point to a file
+  if (strFileName.IsEmpty())
+    return NS_ERROR_FAILURE;
+
+  NS_UnescapeURL(strFileName);
+  NS_ConvertUTF8toUTF16 wideFileName(strFileName);
+
+  // make the name safe for the filesystem
+  MangleTextToValidFilename(wideFileName);
+
+  aFilename = wideFileName;
+
+  return NS_OK;
+}
+
+HRESULT nsDataObj::GetFileDescriptor_IStreamA(FORMATETC& aFE, STGMEDIUM& aSTG)
+{
+  HGLOBAL fileGroupDescHandle = ::GlobalAlloc(GMEM_ZEROINIT|GMEM_SHARE,sizeof(FILEGROUPDESCRIPTORW));
+  NS_ENSURE_TRUE(fileGroupDescHandle, E_OUTOFMEMORY);
+
+  LPFILEGROUPDESCRIPTORA fileGroupDescA = NS_REINTERPRET_CAST(LPFILEGROUPDESCRIPTORA, 
+                                                              ::GlobalLock(fileGroupDescHandle));
+  if (!fileGroupDescA) {
+    ::GlobalFree(fileGroupDescHandle);
+    return E_OUTOFMEMORY;
+  }
+
+  nsAutoString wideFileName;
+  nsresult rv;
+  nsCOMPtr<nsIURI> sourceURI;
+  rv = GetDownloadDetails(getter_AddRefs(sourceURI),
+                          wideFileName);
+  if (NS_FAILED(rv))
+  {
+    ::GlobalFree(fileGroupDescHandle);
+    return E_FAIL;
+  }
+
+  nsCAutoString nativeFileName;
+  NS_UTF16ToCString(wideFileName, NS_CSTRING_ENCODING_NATIVE_FILESYSTEM, nativeFileName);
+  
+  strncpy(fileGroupDescA->fgd[0].cFileName, nativeFileName.get(), NS_MAX_FILEDESCRIPTOR - 1);
+  fileGroupDescA->fgd[0].cFileName[NS_MAX_FILEDESCRIPTOR - 1] = '\0';
+
+  // one file in the file block
+  fileGroupDescA->cItems = 1;
+  fileGroupDescA->fgd[0].dwFlags = FD_PROGRESSUI;
+
+  ::GlobalUnlock( fileGroupDescHandle );
+  aSTG.hGlobal = fileGroupDescHandle;
+  aSTG.tymed = TYMED_HGLOBAL;
+
+  return S_OK;
+}
+
+HRESULT nsDataObj::GetFileDescriptor_IStreamW(FORMATETC& aFE, STGMEDIUM& aSTG)
+{
+  HGLOBAL fileGroupDescHandle = ::GlobalAlloc(GMEM_ZEROINIT|GMEM_SHARE,sizeof(FILEGROUPDESCRIPTORW));
+  NS_ENSURE_TRUE(fileGroupDescHandle, E_OUTOFMEMORY);
+
+  LPFILEGROUPDESCRIPTORW fileGroupDescW = NS_REINTERPRET_CAST(LPFILEGROUPDESCRIPTORW, 
+                                                              ::GlobalLock(fileGroupDescHandle));
+  if (!fileGroupDescW) {
+    ::GlobalFree(fileGroupDescHandle);
+    return E_OUTOFMEMORY;
+  }
+
+  nsAutoString wideFileName;
+  nsresult rv;
+  nsCOMPtr<nsIURI> sourceURI;
+  rv = GetDownloadDetails(getter_AddRefs(sourceURI),
+                          wideFileName);
+  if (NS_FAILED(rv))
+  {
+    ::GlobalFree(fileGroupDescHandle);
+    return E_FAIL;
+  }
+
+  wcsncpy(fileGroupDescW->fgd[0].cFileName, wideFileName.get(), NS_MAX_FILEDESCRIPTOR - 1);
+  fileGroupDescW->fgd[0].cFileName[NS_MAX_FILEDESCRIPTOR - 1] = '\0';
+  // one file in the file block
+  fileGroupDescW->cItems = 1;
+  fileGroupDescW->fgd[0].dwFlags = FD_PROGRESSUI;
+
+  ::GlobalUnlock(fileGroupDescHandle);
+  aSTG.hGlobal = fileGroupDescHandle;
+  aSTG.tymed = TYMED_HGLOBAL;
+
+  return S_OK;
+}
+
+HRESULT nsDataObj::GetFileContents_IStream(FORMATETC& aFE, STGMEDIUM& aSTG)
+{
+  IStream *pStream = NULL;
+
+  nsDataObj::CreateStream(&pStream);
+  NS_ENSURE_TRUE(pStream, E_FAIL);
+
+  aSTG.tymed = TYMED_ISTREAM;
+  aSTG.pstm = pStream;
+  aSTG.pUnkForRelease = pStream;
+
+  return S_OK;
+}
