@@ -57,14 +57,14 @@
 
 #include "nsVoidArray.h"
 
-#include <imm.h>
-
 class nsNativeDragTarget;
 class nsIRollupListener;
 
 class nsIMenuBar;
 class nsIFile;
 
+class imgIContainer;
+
 #ifdef ACCESSIBILITY
 #include "OLEACC.H"
 #include "nsIAccessible.h"
@@ -83,163 +83,17 @@ class nsIFile;
  * For example, changing the window classes
  * could break touchpad scrolling or screen readers.
  */
-const PRUint32 kMaxClassNameLength  = 40;
-const LPCWSTR kWClassNameHidden     = L"MozillaHiddenWindowClass";
-const LPCWSTR kWClassNameUI         = L"MozillaUIWindowClass";
-const LPCWSTR kWClassNameContent    = L"MozillaContentWindowClass";
-const LPCWSTR kWClassNameGeneral    = L"MozillaWindowClass";
-const LPCSTR kClassNameHidden       = "MozillaHiddenWindowClass";
-const LPCSTR kClassNameUI           = "MozillaUIWindowClass";
-const LPCSTR kClassNameContent      = "MozillaContentWindowClass";
-const LPCSTR kClassNameGeneral      = "MozillaWindowClass";
-
-/**
-* Native IMM wrapper
-*/
-class nsIMM
-{
-  //prototypes for DLL function calls...
-  typedef LONG (CALLBACK *GetCompStrPtr)  (HIMC, DWORD, LPVOID, DWORD);
-  typedef LONG (CALLBACK *GetContextPtr)  (HWND);
-  typedef LONG (CALLBACK *RelContextPtr)  (HWND, HIMC);
-  typedef LONG (CALLBACK *NotifyIMEPtr)   (HIMC, DWORD, DWORD, DWORD);
-  typedef LONG (CALLBACK *SetCandWindowPtr)(HIMC, LPCANDIDATEFORM);
-  typedef LONG (CALLBACK *SetCompWindowPtr)(HIMC, LPCOMPOSITIONFORM);
-  typedef LONG (CALLBACK *GetCompWindowPtr)(HIMC, LPCOMPOSITIONFORM);
-  typedef LONG (CALLBACK *GetPropertyPtr)  (HKL, DWORD);
-  typedef LONG (CALLBACK *GetDefaultIMEWndPtr) (HWND);
-  typedef BOOL (CALLBACK *GetOpenStatusPtr)  (HIMC);
-  typedef BOOL (CALLBACK *SetOpenStatusPtr)  (HIMC, BOOL);
-public:
-
-  static nsIMM& LoadModule() {
-    static nsIMM gIMM;
-    return gIMM;
-  }
-
-  nsIMM(const char* aModuleName="IMM32.DLL") {
-    mInstance=::LoadLibrary(aModuleName);
-    NS_ASSERTION(mInstance!=NULL, "nsIMM.LoadLibrary failed.");
-
-    mGetCompositionStringA=(mInstance) ? (GetCompStrPtr)GetProcAddress(mInstance, "ImmGetCompositionStringA") : 0;
-    NS_ASSERTION(mGetCompositionStringA!=NULL, "nsIMM.ImmGetCompositionStringA failed.");
-
-    mGetCompositionStringW=(mInstance) ? (GetCompStrPtr)GetProcAddress(mInstance, "ImmGetCompositionStringW") : 0;
-    NS_ASSERTION(mGetCompositionStringW!=NULL, "nsIMM.ImmGetCompositionStringW failed.");
-
-    mGetContext=(mInstance) ? (GetContextPtr)GetProcAddress(mInstance, "ImmGetContext") : 0;
-    NS_ASSERTION(mGetContext!=NULL, "nsIMM.ImmGetContext failed.");
-
-    mReleaseContext=(mInstance) ? (RelContextPtr)GetProcAddress(mInstance, "ImmReleaseContext") : 0;
-    NS_ASSERTION(mReleaseContext!=NULL, "nsIMM.ImmReleaseContext failed.");
-
-    mNotifyIME=(mInstance) ? (NotifyIMEPtr)GetProcAddress(mInstance, "ImmNotifyIME") : 0;
-    NS_ASSERTION(mNotifyIME!=NULL, "nsIMM.ImmNotifyIME failed.");
-
-    mSetCandiateWindow=(mInstance) ? (SetCandWindowPtr)GetProcAddress(mInstance, "ImmSetCandidateWindow") : 0;
-    NS_ASSERTION(mSetCandiateWindow!=NULL, "nsIMM.ImmSetCandidateWindow failed.");
-
-    mGetCompositionWindow=(mInstance) ? (GetCompWindowPtr)GetProcAddress(mInstance, "ImmGetCompositionWindow") : 0;
-    NS_ASSERTION(mGetCompositionWindow!=NULL, "nsIMM.ImmGetCompositionWindow failed.");
-
-    mSetCompositionWindow=(mInstance) ? (SetCompWindowPtr)GetProcAddress(mInstance, "ImmSetCompositionWindow") : 0;
-    NS_ASSERTION(mSetCompositionWindow!=NULL, "nsIMM.ImmSetCompositionWindow failed.");
-
-    mGetProperty=(mInstance) ? (GetPropertyPtr)GetProcAddress(mInstance, "ImmGetProperty") : 0;
-    NS_ASSERTION(mGetProperty!=NULL, "nsIMM.ImmGetProperty failed.");
-
-    mGetDefaultIMEWnd=(mInstance) ? (GetDefaultIMEWndPtr)GetProcAddress(mInstance, "ImmGetDefaultIMEWnd") : 0;
-    NS_ASSERTION(mGetDefaultIMEWnd!=NULL, "nsIMM.ImmGetDefaultIMEWnd failed.");
-
-    mGetOpenStatus=(mInstance) ? (GetOpenStatusPtr)GetProcAddress(mInstance,"ImmGetOpenStatus") : 0;
-    NS_ASSERTION(mGetOpenStatus!=NULL, "nsIMM.ImmGetOpenStatus failed.");
-
-    mSetOpenStatus=(mInstance) ? (SetOpenStatusPtr)GetProcAddress(mInstance,"ImmSetOpenStatus") : 0;
-    NS_ASSERTION(mSetOpenStatus!=NULL, "nsIMM.ImmSetOpenStatus failed.");
-  }
-
-  ~nsIMM() {
-    if(mInstance) {
-      ::FreeLibrary(mInstance);
-    }
-    mGetCompositionStringA=0;
-    mGetCompositionStringW=0;
-    mGetContext=0;
-    mReleaseContext=0;
-    mNotifyIME=0;
-    mSetCandiateWindow=0;
-    mGetCompositionWindow=0;
-    mSetCompositionWindow=0;
-    mGetProperty=0;
-    mGetDefaultIMEWnd=0;
-    mGetOpenStatus=0;
-    mSetOpenStatus=0;
-  }
-
-  LONG GetCompositionStringA(HIMC h, DWORD d1, LPVOID v, DWORD d2) {
-    return (mGetCompositionStringA) ? mGetCompositionStringA(h, d1, v, d2) : 0L;
-  }
-
-  LONG GetCompositionStringW(HIMC h, DWORD d1, LPVOID v, DWORD d2) {
-    return (mGetCompositionStringW) ? mGetCompositionStringW(h, d1, v, d2) : 0L;
-  }
-
-  LONG GetContext(HWND anHWND) {
-    return (mGetContext) ? mGetContext(anHWND) : 0L;
-  }
-
-  LONG ReleaseContext(HWND anHWND, HIMC anIMC) {
-    return (mReleaseContext) ? mReleaseContext(anHWND, anIMC) : 0L;
-  }
-
-  LONG NotifyIME(HIMC h, DWORD d1, DWORD d2, DWORD d3) {
-    return (mNotifyIME) ? mNotifyIME(h, d1, d2, d3) : 0L;
-  }
-
-  LONG SetCandidateWindow(HIMC h, LPCANDIDATEFORM l) {
-    return (mSetCandiateWindow) ? mSetCandiateWindow(h, l) : 0L;
-  }
-
-  LONG SetCompositionWindow(HIMC h, LPCOMPOSITIONFORM l) {
-    return (mSetCompositionWindow) ? mSetCompositionWindow(h, l) : 0L;
-  }
-
-  LONG GetCompositionWindow(HIMC h,LPCOMPOSITIONFORM l) {
-    return (mGetCompositionWindow) ? mGetCompositionWindow(h, l) : 0L;
-  }
-
-  LONG GetProperty(HKL hKL, DWORD dwIndex) {
-    return (mGetProperty) ? mGetProperty(hKL, dwIndex) : 0L;
-  }
-
-  LONG GetDefaultIMEWnd(HWND hWnd) {
-    return (mGetDefaultIMEWnd) ? mGetDefaultIMEWnd(hWnd) : 0L;
-  }
-
-  BOOL GetOpenStatus(HIMC h) {
-    return (mGetOpenStatus) ? mGetOpenStatus(h) : FALSE;
-  }
-
-  BOOL SetOpenStatus(HIMC h, BOOL b) {
-    return (mSetOpenStatus) ? mSetOpenStatus(h,b) : FALSE;
-  }
-
-private:
-
-  HINSTANCE           mInstance;
-  GetCompStrPtr       mGetCompositionStringA;
-  GetCompStrPtr       mGetCompositionStringW;
-  GetContextPtr       mGetContext;
-  RelContextPtr       mReleaseContext;
-  NotifyIMEPtr        mNotifyIME;
-  SetCandWindowPtr    mSetCandiateWindow;
-  SetCompWindowPtr    mSetCompositionWindow;
-  GetCompWindowPtr    mGetCompositionWindow;
-  GetPropertyPtr      mGetProperty;
-  GetDefaultIMEWndPtr mGetDefaultIMEWnd;
-  GetOpenStatusPtr    mGetOpenStatus;
-  SetOpenStatusPtr    mSetOpenStatus;
-};
+const PRUint32 kMaxClassNameLength    = 40;
+const LPCWSTR kWClassNameHidden       = L"MozillaHiddenWindowClass";
+const LPCWSTR kWClassNameUI           = L"MozillaUIWindowClass";
+const LPCWSTR kWClassNameContent      = L"MozillaContentWindowClass";
+const LPCWSTR kWClassNameContentFrame = L"MozillaContentFrameWindowClass";
+const LPCWSTR kWClassNameGeneral      = L"MozillaWindowClass";
+const LPCSTR kClassNameHidden         = "MozillaHiddenWindowClass";
+const LPCSTR kClassNameUI             = "MozillaUIWindowClass";
+const LPCSTR kClassNameContent        = "MozillaContentWindowClass";
+const LPCSTR kClassNameContentFrame   = "MozillaContentFrameWindowClass";
+const LPCSTR kClassNameGeneral        = "MozillaWindowClass";
 
 /**
  * Native WIN32 window wrapper.
@@ -309,6 +163,8 @@ public:
   virtual nsIFontMetrics* GetFont(void);
   NS_IMETHOD              SetFont(const nsFont &aFont);
   NS_IMETHOD              SetCursor(nsCursor aCursor);
+  NS_IMETHOD              SetCursor(imgIContainer* aCursor,
+                                    PRUint32 aHotspotX, PRUint32 aHotspotY);
   NS_IMETHOD              HideWindowChrome(PRBool aShouldHide);
   NS_IMETHOD              Validate();
   NS_IMETHOD              Invalidate(PRBool aIsSynchronous);
@@ -342,6 +198,11 @@ public:
 
   NS_IMETHOD              GetAttention(PRInt32 aCycleCount);
   NS_IMETHOD              GetLastInputEventTime(PRUint32& aTime);
+  nsWindow*               GetTopLevelWindow();
+
+#ifdef MOZ_CAIRO_GFX
+  gfxASurface             *GetThebesSurface();
+#endif
 
 #ifdef MOZ_XUL
   NS_IMETHOD              SetWindowTranslucency(PRBool aTransparent);
@@ -351,9 +212,10 @@ private:
   nsresult                SetWindowTranslucencyInner(PRBool aTransparent);
   PRBool                  GetWindowTranslucencyInner() { return mIsTranslucent; }
   void                    UpdateTranslucentWindowAlphaInner(const nsRect& aRect, PRUint8* aAlphas);
-  nsIWidget*              GetTopLevelWidget();
-  void                    ResizeTranslucentWindow(PRInt32 aNewWidth, PRInt32 aNewHeight);
+  void                    ResizeTranslucentWindow(PRInt32 aNewWidth, PRInt32 aNewHeight, PRBool force = PR_FALSE);
   nsresult                UpdateTranslucentWindow();
+  nsresult                SetupTranslucentWindowMemoryBitmap(PRBool aTranslucent);
+  void                    SetWindowRegionToAlphaMask();
 public:
 #endif
 
@@ -362,6 +224,9 @@ public:
   NS_IMETHOD ResetInputState();
   NS_IMETHOD SetIMEOpenState(PRBool aState);
   NS_IMETHOD GetIMEOpenState(PRBool* aState);
+  NS_IMETHOD SetIMEEnabled(PRBool aState);
+  NS_IMETHOD GetIMEEnabled(PRBool* aState);
+  NS_IMETHOD CancelIMEComposition();
 
   PRBool IMEMouseHandling(PRUint32 aEventType, PRInt32 aAction, LPARAM lParam);
   PRBool IMECompositionHitTest(PRUint32 aEventType, POINT * ptPos);
@@ -374,11 +239,10 @@ public:
   HWND                    GetWindowHandle() { return mWnd; }
   WNDPROC                 GetPrevWindowProc() { return mPrevWndProc; }
 
-  virtual PRBool          DispatchMouseEvent(PRUint32 aEventType, WPARAM wParam = NULL, nsPoint* aPoint = nsnull);
+  virtual PRBool          DispatchMouseEvent(PRUint32 aEventType, WPARAM wParam, LPARAM lParam);
 #ifdef ACCESSIBILITY
   virtual PRBool          DispatchAccessibleEvent(PRUint32 aEventType, nsIAccessible** aAccessible, nsPoint* aPoint = nsnull);
-  void                    CreateRootAccessible();
-  void                    ClearRootAccessible();
+  already_AddRefed<nsIAccessible> GetRootAccessible();
 #endif
   virtual PRBool          AutoErase();
   nsPoint*                GetLastPoint() { return &mLastPoint; }
@@ -391,6 +255,9 @@ public:
   PRBool                  BlurEventsSuppressed();
 
 protected:
+
+#ifndef WINCE
+
   // special callback hook methods for pop ups
   static LRESULT CALLBACK MozSpecialMsgFilter(int code, WPARAM wParam, LPARAM lParam);
   static LRESULT CALLBACK MozSpecialWndProc(int code, WPARAM wParam, LPARAM lParam);
@@ -401,12 +268,16 @@ protected:
   static void             RegisterSpecialDropdownHooks();
   static void             UnregisterSpecialDropdownHooks();
 
-  static BOOL             DealWithPopups (UINT inMsg, WPARAM inWParam, LPARAM inLParam, LRESULT* outResult);
+  static void             PostSleepWakeNotification(const char* aNotification);
+#endif
+
+  static BOOL             DealWithPopups (HWND inWnd, UINT inMsg, WPARAM inWParam, LPARAM inLParam, LRESULT* outResult);
 
   static PRBool           EventIsInsideWindow(UINT Msg, nsWindow* aWindow);
 
-  static nsWindow *       GetNSWindowPtr(HWND aWnd);
+  static nsWindow*        GetNSWindowPtr(HWND aWnd);
   static BOOL             SetNSWindowPtr(HWND aWnd, nsWindow * ptr);
+  nsWindow*               GetParent(PRBool aStopOnFirstTopLevel);
 
   void                    DispatchPendingEvents();
   virtual PRBool          ProcessMessage(UINT msg, WPARAM wParam, LPARAM lParam, LRESULT *aRetValue);
@@ -430,7 +301,7 @@ protected:
   virtual PRBool          OnPaint(HDC aDC = nsnull);
   virtual PRBool          OnResize(nsRect &aWindowRect);
 
-  BOOL                    OnChar(UINT charCode);
+  BOOL                    OnChar(UINT charCode, PRUint32 aFlags = 0);
 
   BOOL                    OnKeyDown( UINT aVirtualKeyCode, UINT aScanCode, LPARAM aKeyCode);
   BOOL                    OnKeyUp( UINT aVirtualKeyCode, UINT aScanCode, LPARAM aKeyCode);
@@ -443,15 +314,20 @@ protected:
   BOOL                    OnIMECompositionFull();
   BOOL                    OnIMEEndComposition();
   BOOL                    OnIMENotify(WPARAM  aIMN, LPARAM aData, LRESULT *oResult);
-  BOOL                    OnIMERequest(WPARAM  aIMR, LPARAM aData, LRESULT *oResult, PRBool aUseUnicode = PR_FALSE);
+  BOOL                    OnIMERequest(WPARAM  aIMR, LPARAM aData, LRESULT *oResult);
   BOOL                    OnIMESelect(BOOL  aSelected, WORD aLangID);
   BOOL                    OnIMESetContext(BOOL aActive, LPARAM& aISC);
   BOOL                    OnIMEStartComposition();
-  BOOL                    OnIMEReconvert(LPARAM aData, LRESULT *oResult, PRBool aUseUnicode);
+  BOOL                    OnIMEReconvert(LPARAM aData, LRESULT *oResult);
+  BOOL                    OnIMEQueryCharPosition(LPARAM aData, LRESULT *oResult);
 
-  void                    GetCompositionString(HIMC aHIMC, DWORD aIndex, nsString* aStrUnicode, nsCString* aStrAnsi);
+  void                    GetCompositionString(HIMC aHIMC, DWORD aIndex, nsString* aStrUnicode);
+  void                    ResolveIMECaretPos(nsWindow* aClient,
+                                             nsRect&   aEventResult,
+                                             nsRect&   aResult);
 
-  virtual PRBool          DispatchKeyEvent(PRUint32 aEventType, WORD aCharCode, UINT aVirtualCharCode, LPARAM aKeyCode);
+  virtual PRBool          DispatchKeyEvent(PRUint32 aEventType, WORD aCharCode, UINT aVirtualCharCode,
+                                           LPARAM aKeyCode, PRUint32 aFlags = 0);
 
   virtual PRBool          DispatchFocus(PRUint32 aEventType, PRBool isMozWindowTakingFocus);
   virtual PRBool          OnScroll(UINT scrollCode, int cPos);
@@ -460,7 +336,9 @@ protected:
   static LRESULT CALLBACK WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
   static LRESULT CALLBACK DefaultWindowProc(HWND hWns, UINT msg, WPARAM wParam, LPARAM lParam);
 
-  static PRBool ConvertStatus(nsEventStatus aStatus);
+  // Convert nsEventStatus value to a windows boolean
+  static PRBool ConvertStatus(nsEventStatus aStatus)
+                       { return aStatus == nsEventStatus_eConsumeNoDefault; }
 
   PRBool DispatchStandardEvent(PRUint32 aMsg);
   PRBool DispatchAppCommandEvent(PRUint32 aEventCommand);
@@ -474,6 +352,11 @@ protected:
 
   void ConstrainZLevel(HWND *aAfter);
 
+  LPARAM lParamToScreen(LPARAM lParam);
+  LPARAM lParamToClient(LPARAM lParam);
+
+  PRBool CanTakeFocus();
+
 private:
 
 
@@ -503,12 +386,13 @@ protected:
 
   // For describing composing frame
   static RECT*      sIMECompCharPos;
+  static PRInt32    sIMECaretHeight;
 
   nsSize        mLastSize;
   static        nsWindow* gCurrentWindow;
   nsPoint       mLastPoint;
   HWND          mWnd;
-  HWND          mBorderlessParent;
+  HDC           mPaintDC; // only set during painting
 #if 0
   HPALETTE      mPalette;
 #endif
@@ -516,10 +400,13 @@ protected:
   HBRUSH        mBrush;
 
 #ifdef MOZ_XUL
-  HBITMAP       mMemoryBitmap;
+  // use layered windows to support full 256 level alpha translucency
   HDC           mMemoryDC;
+  HBITMAP       mMemoryBitmap;
+  PRUint8*      mMemoryBits;
   PRUint8*      mAlphaMask;
   PRPackedBool  mIsTranslucent;
+  PRPackedBool  mIsTopTranslucent;     // Topmost window itself or any of it's child windows has tranlucency enabled
 #endif
   PRPackedBool  mIsTopWidgetWindow;
   PRPackedBool  mHas3DBorder;
@@ -533,6 +420,7 @@ protected:
   PRPackedBool  mIsInMouseWheelProcessing;
   PRPackedBool  mUnicodeWidget;
 
+  PRPackedBool  mPainting;
   char          mLeadByte;
   PRUint32      mBlurEventSuppressionLevel;
   nsContentType mContentType;
@@ -545,17 +433,22 @@ protected:
 
   PRInt32       mMenuCmdId;
 
-    // Window styles used by this window before chrome was hidden
+  // Window styles used by this window before chrome was hidden
   DWORD         mOldStyle;
   DWORD         mOldExStyle;
 
-  static UINT   gCurrentKeyboardCP;
+  // To enable/disable IME
+  HIMC          mOldIMC;
+
   static HKL    gKeyboardLayout;
+  static PRBool gSwitchKeyboardLayout;
+
+  HKL           mLastKeyboardLayout;
 
   // Drag & Drop
   nsNativeDragTarget * mNativeDragTarget;
 
-  // Enumeration of the methods which are accessable on the "main GUI thread"
+  // Enumeration of the methods which are accessible on the "main GUI thread"
   // via the CallMethod(...) mechanism...
   // see nsSwitchToUIThread
   enum {
@@ -571,18 +464,68 @@ protected:
   static BOOL   sIsPopupClassRegistered;
 
   HDWP mDeferredPositioner;
-  static UINT   uMSH_MOUSEWHEEL;
-
-  // IME special message
-  static UINT   uWM_MSIME_RECONVERT; // reconvert message for MSIME
   static UINT   uWM_MSIME_MOUSE;     // mouse message for MSIME
-  static UINT   uWM_ATOK_RECONVERT;  // reconvert message for ATOK
 
   // Heap dump
   static UINT   uWM_HEAP_DUMP;       // Dump heap to a file
 
+  // Cursor caching
+  static HCURSOR        gHCursor;
+  static imgIContainer* gCursorImgContainer;
+
+  /**
+   * Create a 1 bit mask out of a 8 bit alpha layer.
+   *
+   * @param aAlphaData        8 bit alpha data
+   * @param aAlphaBytesPerRow How many bytes one row of data is
+   * @param aWidth            Width of the alpha data, in pixels
+   * @param aHeight           Height of the alpha data, in pixels
+   *
+   * @return 1 bit mask.  Must be delete[]d. On failure, NULL will be returned.
+   */
+  static PRUint8* Data8BitTo1Bit(PRUint8* aAlphaData, PRUint32 aAlphaBytesPerRow,
+                                 PRUint32 aWidth, PRUint32 aHeight);
+
+  /**
+   * Combine the given image data with a separate alpha channel to image data
+   * with the alpha channel interleaved with the image data (BGRA).
+   *
+   * @return BGRA data. Must be delete[]d. On failure, NULL will be returned.
+   */
+  static PRUint8* DataToAData(PRUint8* aImageData, PRUint32 aImageBytesPerRow,
+                              PRUint8* aAlphaData, PRUint32 aAlphaBytesPerRow,
+                              PRUint32 aWidth, PRUint32 aHeight);
+  /**
+   * Convert the given image data to a HBITMAP. If the requested depth is
+   * 32 bit and the OS supports translucency, a bitmap with an alpha channel
+   * will be returned.
+   *
+   * @param aImageData The image data to convert. Must use the format accepted
+   *                   by CreateDIBitmap.
+   * @param aWidth     With of the bitmap, in pixels.
+   * @param aHeight    Height of the image, in pixels.
+   * @param aDepth     Image depth, in bits. Should be one of 1, 24 and 32.
+   *
+   * @return The HBITMAP representing the image. Caller should call
+   *         DeleteObject when done with the bitmap.
+   *         On failure, NULL will be returned.
+   */
+  static HBITMAP DataToBitmap(PRUint8* aImageData,
+                              PRUint32 aWidth,
+                              PRUint32 aHeight,
+                              PRUint32 aDepth);
+
+  /**
+   * Create a bitmap representing an opaque alpha channel (filled with 0xff).
+   * @param aWidth  Desired with of the bitmap
+   * @param aHeight Desired height of the bitmap
+   * @return        The bitmap. Caller should call DeleteObject when done with
+   *                the bitmap. On failure, NULL will be returned.
+   */
+  static HBITMAP CreateOpaqueAlphaChannel(PRUint32 aWidth, PRUint32 aHeight);
+
+
 #ifdef ACCESSIBILITY
-  nsIAccessible* mRootAccessible;
   static BOOL gIsAccessibilityOn;
   static HINSTANCE gmAccLib;
   static LPFNLRESULTFROMOBJECT gmLresultFromObject;
@@ -595,6 +538,7 @@ protected:
 
 public:
   static void GlobalMsgWindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
+  static HWND GetTopLevelHWND(HWND aWnd, PRBool aStopOnFirstTopLevel = PR_FALSE);
 };
 
 //
@@ -604,7 +548,7 @@ class ChildWindow : public nsWindow {
 
 public:
   ChildWindow() {}
-  PRBool DispatchMouseEvent(PRUint32 aEventType, WPARAM wParam = NULL, nsPoint* aPoint = nsnull);
+  PRBool DispatchMouseEvent(PRUint32 aEventType, WPARAM wParam, LPARAM lParam);
 
 protected:
   virtual DWORD WindowStyle();
