@@ -411,7 +411,7 @@ NS_IMETHODIMP nsAppShell::ListenToEventQueue(nsIEventQueue *aQueue,
                                 HandleQueueXtProc,
                                 (XtPointer)mEventQueue);
 
-/* This hack would not be neccesary if we would have a hashtable function
+/* This hack would not be necessary if we would have a hashtable function
  * which returns success/failure in a separate var ...
  */
 #define NEVER_BE_ZERO_MAGIC (54321) 
@@ -592,11 +592,11 @@ nsAppShell::HandleMotionNotifyEvent(XEvent *event, nsWidget *aWidget)
     HandleDragMotionEvent(event, aWidget);
   }
 
-  nsMouseEvent mevent(NS_MOUSE_MOVE, aWidget);
+  nsMouseEvent mevent(PR_TRUE, NS_MOUSE_MOVE, aWidget, nsMouseEvent::eReal);
   XEvent aEvent;
 
-  mevent.point.x = event->xmotion.x;
-  mevent.point.y = event->xmotion.y;
+  mevent.refPoint.x = event->xmotion.x;
+  mevent.refPoint.y = event->xmotion.y;
 
   mevent.isShift = mShiftDown;
   mevent.isControl = mCtrlDown;
@@ -610,8 +610,8 @@ nsAppShell::HandleMotionNotifyEvent(XEvent *event, nsWidget *aWidget)
                           win,
                           ButtonMotionMask,
                           &aEvent)) {
-    mevent.point.x = aEvent.xmotion.x;
-    mevent.point.y = aEvent.xmotion.y;
+    mevent.refPoint.x = aEvent.xmotion.x;
+    mevent.refPoint.y = aEvent.xmotion.y;
   }
   NS_ADDREF(aWidget);
   aWidget->DispatchMouseEvent(mevent);
@@ -623,7 +623,7 @@ nsAppShell::HandleButtonEvent(XEvent *event, nsWidget *aWidget)
 {
   PRUint32 eventType = 0;
   PRBool currentlyDragging = mDragging;
-  nsMouseScrollEvent scrollEvent(NS_MOUSE_SCROLL, aWidget);
+  nsMouseScrollEvent scrollEvent(PR_TRUE, NS_MOUSE_SCROLL, aWidget);
 
   PR_LOG(XlibWidgetsLM, PR_LOG_DEBUG, ("Button event for window 0x%lx button %d type %s\n",
                                        event->xany.window,
@@ -649,8 +649,8 @@ nsAppShell::HandleButtonEvent(XEvent *event, nsWidget *aWidget)
       scrollEvent.delta = (event->xbutton.button == 4) ? -3 : 3;
       scrollEvent.scrollFlags = nsMouseScrollEvent::kIsVertical;
 
-      scrollEvent.point.x = event->xbutton.x;
-      scrollEvent.point.y = event->xbutton.y;
+      scrollEvent.refPoint.x = event->xbutton.x;
+      scrollEvent.refPoint.y = event->xbutton.y;
 
       scrollEvent.isShift = mShiftDown;
       scrollEvent.isControl = mCtrlDown;
@@ -682,13 +682,13 @@ nsAppShell::HandleButtonEvent(XEvent *event, nsWidget *aWidget)
     break;
   }
 
-  nsMouseEvent mevent(eventType, aWidget);
+  nsMouseEvent mevent(PR_TRUE, eventType, aWidget, nsMouseEvent::eReal);
   mevent.isShift = mShiftDown;
   mevent.isControl = mCtrlDown;
   mevent.isAlt = mAltDown;
   mevent.isMeta = mMetaDown;
-  mevent.point.x = event->xbutton.x;
-  mevent.point.y = event->xbutton.y;
+  mevent.refPoint.x = event->xbutton.x;
+  mevent.refPoint.y = event->xbutton.y;
   mevent.time = PR_Now();
   
   // If we are waiting longer than 1 sec for the second click, this is not a
@@ -775,11 +775,11 @@ nsAppShell::HandleConfigureNotifyEvent(XEvent *event, nsWidget *aWidget)
     }
   }
 
-  nsSizeEvent sevent(NS_SIZE, aWidget);
+  nsSizeEvent sevent(PR_TRUE, NS_SIZE, aWidget);
   sevent.windowSize = new nsRect (event->xconfigure.x, event->xconfigure.y,
                                   event->xconfigure.width, event->xconfigure.height);
-  sevent.point.x = event->xconfigure.x;
-  sevent.point.y = event->xconfigure.y;
+  sevent.refPoint.x = event->xconfigure.x;
+  sevent.refPoint.y = event->xconfigure.y;
   sevent.mWinWidth = event->xconfigure.width;
   sevent.mWinHeight = event->xconfigure.height;
   // XXX fix sevent.time
@@ -869,13 +869,13 @@ nsAppShell::HandleKeyPressEvent(XEvent *event, nsWidget *aWidget)
       break;
   }
 
-  // Dont dispatch events for modifier keys pressed ALONE
+  // Don't dispatch events for modifier keys pressed ALONE
   if (nsKeyCode::KeyCodeIsModifier(event->xkey.keycode))
   {
     return;
   }
 
-  nsKeyEvent keyEvent(NS_KEY_DOWN, focusWidget);
+  nsKeyEvent keyEvent(PR_TRUE, NS_KEY_DOWN, focusWidget);
 
   XComposeStatus compose;
 
@@ -895,9 +895,9 @@ nsAppShell::HandleKeyPressEvent(XEvent *event, nsWidget *aWidget)
   //         event->xkey.keycode,
   //         keyEvent.keyCode);
 
-  focusWidget->DispatchKeyEvent(keyEvent);
+  PRBool noDefault = focusWidget->DispatchKeyEvent(keyEvent);
 
-  nsKeyEvent pressEvent(NS_KEY_PRESS, focusWidget);
+  nsKeyEvent pressEvent(PR_TRUE, NS_KEY_PRESS, focusWidget);
   pressEvent.keyCode = nsKeyCode::ConvertKeySymToVirtualKey(keysym);
   pressEvent.charCode = nsConvertCharCodeToUnicode(&event->xkey);
   pressEvent.time = event->xkey.time;
@@ -905,9 +905,11 @@ nsAppShell::HandleKeyPressEvent(XEvent *event, nsWidget *aWidget)
   pressEvent.isControl = (event->xkey.state & ControlMask) ? 1 : 0;
   pressEvent.isAlt = (event->xkey.state & Mod1Mask) ? 1 : 0;
   pressEvent.isMeta = (event->xkey.state & Mod1Mask) ? 1 : 0;
+  if (noDefault) {   // If default prevented on keydown, do same for keypress
+    pressEvent.flags |= NS_EVENT_FLAG_NO_DEFAULT;
+  }
 
   focusWidget->DispatchKeyEvent(pressEvent);
-
 }
 
 void
@@ -940,13 +942,13 @@ nsAppShell::HandleKeyReleaseEvent(XEvent *event, nsWidget *aWidget)
       break;
   }
 
-  // Dont dispatch events for modifier keys pressed ALONE
+  // Don't dispatch events for modifier keys pressed ALONE
   if (nsKeyCode::KeyCodeIsModifier(event->xkey.keycode))
   {
     return;
   }
 
-  nsKeyEvent keyEvent(NS_KEY_UP, aWidget);
+  nsKeyEvent keyEvent(PR_TRUE, NS_KEY_UP, aWidget);
 
   keyEvent.keyCode = nsKeyCode::ConvertKeySymToVirtualKey(keysym);
   keyEvent.time = event->xkey.time;
@@ -954,8 +956,8 @@ nsAppShell::HandleKeyReleaseEvent(XEvent *event, nsWidget *aWidget)
   keyEvent.isControl = (event->xkey.state & ControlMask) ? 1 : 0;
   keyEvent.isAlt = (event->xkey.state & Mod1Mask) ? 1 : 0;
   keyEvent.isMeta = (event->xkey.state & Mod1Mask) ? 1 : 0;
-  keyEvent.point.x = event->xkey.x;
-  keyEvent.point.y = event->xkey.y;
+  keyEvent.refPoint.x = event->xkey.x;
+  keyEvent.refPoint.y = event->xkey.y;
 
   NS_ADDREF(aWidget);
 
@@ -969,7 +971,7 @@ nsAppShell::HandleFocusInEvent(XEvent *event, nsWidget *aWidget)
 {
   PR_LOG(XlibWidgetsLM, PR_LOG_DEBUG, ("FocusIn event for window 0x%lx\n",
                                        event->xfocus.window));
-  nsFocusEvent focusEvent(NS_GOTFOCUS, aWidget);
+  nsFocusEvent focusEvent(PR_TRUE, NS_GOTFOCUS, aWidget);
   
   NS_ADDREF(aWidget);
   aWidget->DispatchWindowEvent(focusEvent);
@@ -981,7 +983,7 @@ nsAppShell::HandleFocusOutEvent(XEvent *event, nsWidget *aWidget)
 {
   PR_LOG(XlibWidgetsLM, PR_LOG_DEBUG, ("FocusOut event for window 0x%lx\n",
                                        event->xfocus.window));
-  nsFocusEvent focusEvent(NS_LOSTFOCUS, aWidget);
+  nsFocusEvent focusEvent(PR_TRUE, NS_LOSTFOCUS, aWidget);
 
   NS_ADDREF(aWidget);
   aWidget->DispatchWindowEvent(focusEvent);
@@ -1021,11 +1023,12 @@ nsAppShell::HandleEnterEvent(XEvent *event, nsWidget *aWidget)
     HandleDragEnterEvent(event, aWidget);
   }
 
-  nsMouseEvent enterEvent(NS_MOUSE_ENTER, aWidget);
+  nsMouseEvent enterEvent(PR_TRUE, NS_MOUSE_ENTER, aWidget,
+                          nsMouseEvent::eReal);
 
   enterEvent.time = event->xcrossing.time;
-  enterEvent.point.x = nscoord(event->xcrossing.x);
-  enterEvent.point.y = nscoord(event->xcrossing.y);
+  enterEvent.refPoint.x = nscoord(event->xcrossing.x);
+  enterEvent.refPoint.y = nscoord(event->xcrossing.y);
   
   // make sure this is in focus. This will do until I rewrite all the 
   // focus routines. KenF
@@ -1052,11 +1055,12 @@ nsAppShell::HandleLeaveEvent(XEvent *event, nsWidget *aWidget)
     HandleDragLeaveEvent(event, aWidget);
   }
 
-  nsMouseEvent leaveEvent(NS_MOUSE_EXIT, aWidget);
+  nsMouseEvent leaveEvent(PR_TRUE, NS_MOUSE_EXIT, aWidget,
+                          nsMouseEvent::eReal);
 
   leaveEvent.time = event->xcrossing.time;
-  leaveEvent.point.x = nscoord(event->xcrossing.x);
-  leaveEvent.point.y = nscoord(event->xcrossing.y);
+  leaveEvent.refPoint.x = nscoord(event->xcrossing.x);
+  leaveEvent.refPoint.y = nscoord(event->xcrossing.y);
   
   NS_ADDREF(aWidget);
   aWidget->DispatchWindowEvent(leaveEvent);
@@ -1116,7 +1120,7 @@ void nsAppShell::HandleClientMessageEvent(XEvent *event, nsWidget *aWidget)
 
 void nsAppShell::HandleSelectionRequestEvent(XEvent *event, nsWidget *aWidget)
 {
-  nsGUIEvent ev(0, aWidget);
+  nsGUIEvent ev(PR_TRUE, 0, aWidget);
 
   ev.nativeMsg = (void *)event;
 
@@ -1139,9 +1143,10 @@ void nsAppShell::HandleDragMotionEvent(XEvent *event, nsWidget *aWidget) {
   if (currentlyDragging) {
     dragServiceXlib->UpdatePosition(event->xmotion.x, event->xmotion.y);
 
-    nsMouseEvent mevent(NS_DRAGDROP_OVER, aWidget);
-    mevent.point.x = event->xmotion.x;
-    mevent.point.y = event->xmotion.y;
+    nsMouseEvent mevent(PR_TRUE, NS_DRAGDROP_OVER, aWidget,
+                        nsMouseEvent::eReal);
+    mevent.refPoint.x = event->xmotion.x;
+    mevent.refPoint.y = event->xmotion.y;
 
     NS_ADDREF(aWidget);
     aWidget->DispatchMouseEvent(mevent);
@@ -1163,10 +1168,11 @@ void nsAppShell::HandleDragEnterEvent(XEvent *event, nsWidget *aWidget) {
   }
 
   if (currentlyDragging) {
-    nsMouseEvent enterEvent(NS_DRAGDROP_ENTER, aWidget);
+    nsMouseEvent enterEvent(PR_TRUE, NS_DRAGDROP_ENTER, aWidget,
+                            nsMouseEvent::eReal);
   
-    enterEvent.point.x = event->xcrossing.x;
-    enterEvent.point.y = event->xcrossing.y;
+    enterEvent.refPoint.x = event->xcrossing.x;
+    enterEvent.refPoint.y = event->xcrossing.y;
   
     NS_ADDREF(aWidget);
     aWidget->DispatchWindowEvent(enterEvent);
@@ -1190,10 +1196,11 @@ void nsAppShell::HandleDragLeaveEvent(XEvent *event, nsWidget *aWidget) {
   }
 
   if (currentlyDragging) {
-    nsMouseEvent leaveEvent(NS_DRAGDROP_EXIT, aWidget);
+    nsMouseEvent leaveEvent(PR_TRUE, NS_DRAGDROP_EXIT, aWidget,
+                            nsMouseEvent::eReal);
   
-    leaveEvent.point.x = event->xcrossing.x;
-    leaveEvent.point.y = event->xcrossing.y;
+    leaveEvent.refPoint.x = event->xcrossing.x;
+    leaveEvent.refPoint.y = event->xcrossing.y;
   
     NS_ADDREF(aWidget);
     aWidget->DispatchWindowEvent(leaveEvent);
@@ -1207,7 +1214,7 @@ void nsAppShell::HandleDragDropEvent(XEvent *event, nsWidget *aWidget) {
   nsresult rv;
   nsCOMPtr<nsIDragService> dragService( do_GetService(kCDragServiceCID, &rv) );
 
-  // FIXME: Dont think the currentlyDragging check is required. KenF
+  // FIXME: Don't think the currentlyDragging check is required. KenF
   if (NS_SUCCEEDED(rv)) {
     nsCOMPtr<nsIDragSessionXlib> dragServiceXlib;
     dragServiceXlib = do_QueryInterface(dragService);
@@ -1217,9 +1224,10 @@ void nsAppShell::HandleDragDropEvent(XEvent *event, nsWidget *aWidget) {
   }
 
   if (currentlyDragging) {
-    nsMouseEvent mevent(NS_DRAGDROP_DROP, aWidget);
-    mevent.point.x = event->xbutton.x;
-    mevent.point.y = event->xbutton.y;
+    nsMouseEvent mevent(PR_TRUE, NS_DRAGDROP_DROP, aWidget,
+                        nsMouseEvent::eReal);
+    mevent.refPoint.x = event->xbutton.x;
+    mevent.refPoint.y = event->xbutton.y;
   
     NS_IF_ADDREF(aWidget);
     aWidget->DispatchMouseEvent(mevent);
@@ -1231,7 +1239,7 @@ void nsAppShell::HandleDragDropEvent(XEvent *event, nsWidget *aWidget) {
 
 void nsAppShell::ForwardEvent(XEvent *event, nsWidget *aWidget)
 {
-  nsGUIEvent ev(0, aWidget);
+  nsGUIEvent ev(PR_TRUE, 0, aWidget);
   ev.nativeMsg = (void *)event;
 
   aWidget->DispatchWindowEvent(ev);
