@@ -25,6 +25,7 @@
  *   B.J. Rossiter <bj@igelaus.com.au>
  *   Tony Tsui <tony@igelaus.com.au>
  *   Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
+ *   Mats Palmgren <mats.palmgren@bredband.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -50,6 +51,8 @@
 #include "nsIPlatformCharset.h"
 #include "nsIServiceManager.h"
 
+#define NS_WINDOW_TITLE_MAX_LENGTH 4095
+
 // Variables for grabbing
 PRBool   nsWindow::sIsGrabbing = PR_FALSE;
 nsWindow *nsWindow::sGrabWindow = nsnull;
@@ -381,7 +384,7 @@ NS_IMETHODIMP nsWindow::InvalidateRegion(const nsIRegion* aRegion, PRBool aIsSyn
 NS_IMETHODIMP nsWindow::SetFocus(PRBool aRaise)
 {
   nsEventStatus status;
-  nsFocusEvent event(NS_GOTFOCUS, this);
+  nsFocusEvent event(PR_TRUE, NS_GOTFOCUS, this);
   //  nsGUIEvent eventActivate;
 
   if (mBaseWindow)
@@ -396,7 +399,7 @@ NS_IMETHODIMP nsWindow::SetFocus(PRBool aRaise)
   DispatchEvent(&event, status);
   Release();
   
-  nsGUIEvent actEvent(NS_ACTIVATE, this);
+  nsGUIEvent actEvent(PR_TRUE, NS_ACTIVATE, this);
   
   AddRef();
   DispatchWindowEvent(actEvent);
@@ -448,7 +451,7 @@ NS_IMETHODIMP nsWindow::Resize(PRInt32 aWidth,
   }
   nsWidget::Resize(aWidth, aHeight, aRepaint);
 
-  nsSizeEvent sevent(NS_SIZE, this);
+  nsSizeEvent sevent(PR_TRUE, NS_SIZE, this);
   nsRect sevent_windowSize(0, 0, aWidth, aHeight);
   sevent.windowSize = &sevent_windowSize;
   sevent.mWinWidth = aWidth;
@@ -481,7 +484,7 @@ NS_IMETHODIMP nsWindow::Resize(PRInt32 aX,
 
   nsWidget::Resize(aX, aY, aWidth, aHeight, aRepaint);
 
-  nsSizeEvent sevent(NS_SIZE, this);
+  nsSizeEvent sevent(PR_TRUE, NS_SIZE, this);
   nsRect sevent_windowSize(0, 0, aWidth, aHeight);
   sevent.windowSize = &sevent_windowSize;
   sevent.mWinWidth = aWidth;
@@ -548,10 +551,10 @@ nsWindow::DoPaint (PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight,
                    nsIRegion *aClipRegion)
 {
   if (mEventCallback) {
-    nsPaintEvent event(NS_PAINT, this);
+    nsPaintEvent event(PR_TRUE, NS_PAINT, this);
     nsRect rect(aX, aY, aWidth, aHeight);
-    event.point.x = aX;
-    event.point.y = aY; 
+    event.refPoint.x = aX;
+    event.refPoint.y = aY; 
     event.time = PR_Now(); /* No time in EXPOSE events */
     event.rect = &rect;
     
@@ -751,6 +754,10 @@ NS_IMETHODIMP nsWindow::SetTitle(const nsAString& aTitle)
   PRInt32 len = (PRInt32)aTitle.Length();
   encoder->GetMaxLength(title, len, &platformLen);
   if (platformLen) {
+    // Truncate overlong titles (bug 167315).
+    if (platformLen > NS_WINDOW_TITLE_MAX_LENGTH) {
+      platformLen = NS_WINDOW_TITLE_MAX_LENGTH;
+    }
     platformText = NS_REINTERPRET_CAST(char*, nsMemory::Alloc(platformLen + sizeof(char)));
     if (platformText) {
       rv = encoder->Convert(title, &len, platformText, &platformLen);
@@ -783,7 +790,7 @@ NS_IMETHODIMP nsWindow::SetTitle(const nsAString& aTitle)
   }
 
   /* if the stuff above failed, replace multibyte with .... */
-  XStoreName(mDisplay, mBaseWindow, NS_LossyConvertUCS2toASCII(aTitle).get());
+  XStoreName(mDisplay, mBaseWindow, NS_LossyConvertUTF16toASCII(aTitle).get());
 
   return NS_OK;
 }
