@@ -44,7 +44,7 @@
 #include "nsGfxCIID.h"
 #include "nsWidgetsCID.h"
 #include "nsIFullScreen.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 #include "nsIScreenManager.h"
 #include "nsAppDirectoryServiceDefs.h"
 
@@ -167,8 +167,6 @@ void nsBaseWidget::BaseCreate(nsIWidget *aParent,
     
   }
   
-  mAppShell = aAppShell;    // addrefs
-  
   // save the event callback function
   mEventCallback = aHandleEventFunction;
   
@@ -244,7 +242,6 @@ NS_METHOD nsBaseWidget::Destroy()
   nsIWidget *parent = GetParent();
   if (parent) {
     parent->RemoveChild(this);
-    NS_RELEASE(parent);
   }
   // disconnect listeners.
   NS_IF_RELEASE(mMouseListener);
@@ -306,9 +303,7 @@ void nsBaseWidget::AddChild(nsIWidget* aChild)
 //-------------------------------------------------------------------------
 void nsBaseWidget::RemoveChild(nsIWidget* aChild)
 {
-  NS_ASSERTION(nsCOMPtr<nsIWidget>(dont_AddRef(aChild->GetParent())) ==
-                 NS_STATIC_CAST(nsIWidget*, this),
-               "Not one of our kids!");
+  NS_ASSERTION(aChild->GetParent() == this, "Not one of our kids!");
   
   if (mLastChild == aChild) {
     mLastChild = mLastChild->GetPrevSibling();
@@ -340,6 +335,10 @@ void nsBaseWidget::RemoveChild(nsIWidget* aChild)
 //-------------------------------------------------------------------------
 NS_IMETHODIMP nsBaseWidget::SetZIndex(PRInt32 aZIndex)
 {
+  // Hold a ref to ourselves just in case, since we're going to remove
+  // from our parent.
+  nsCOMPtr<nsIWidget> kungFuDeathGrip(this);
+  
   mZIndex = aZIndex;
 
   // reorder this child in its parent's list.
@@ -374,8 +373,6 @@ NS_IMETHODIMP nsBaseWidget::SetZIndex(PRInt32 aZIndex)
     if (!sib) {
       parent->AddChild(this);
     }
-    
-    NS_RELEASE(parent);
   }
   return NS_OK;
 }
@@ -489,6 +486,12 @@ NS_METHOD nsBaseWidget::SetCursor(nsCursor aCursor)
   mCursor = aCursor; 
   return NS_OK;
 }
+
+NS_IMETHODIMP nsBaseWidget::SetCursor(imgIContainer* aCursor,
+                                      PRUint32 aHotspotX, PRUint32 aHotspotY)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
     
 //-------------------------------------------------------------------------
 //
@@ -545,6 +548,11 @@ NS_IMETHODIMP nsBaseWidget::HideWindowChrome(PRBool aShouldHide)
 NS_IMETHODIMP nsBaseWidget::MakeFullScreen(PRBool aFullScreen)
 {
   HideWindowChrome(aFullScreen);
+  return MakeFullScreenInternal(aFullScreen);
+}
+
+nsresult nsBaseWidget::MakeFullScreenInternal(PRBool aFullScreen)
+{
   nsCOMPtr<nsIFullScreen> fullScreen = do_GetService("@mozilla.org/browser/fullscreen;1");
 
   if (aFullScreen) {
@@ -596,9 +604,16 @@ nsIRenderingContext* nsBaseWidget::GetRenderingContext()
   nsresult                      rv;
   nsCOMPtr<nsIRenderingContext> renderingCtx;
 
+  if (mOnDestroyCalled)
+    return nsnull;
+
   rv = mContext->CreateRenderingContextInstance(*getter_AddRefs(renderingCtx));
   if (NS_SUCCEEDED(rv)) {
+#if defined(MOZ_CAIRO_GFX)
+    rv = renderingCtx->Init(mContext, GetThebesSurface());
+#else
     rv = renderingCtx->Init(mContext, this);
+#endif
     if (NS_SUCCEEDED(rv)) {
       nsIRenderingContext *ret = renderingCtx;
       /* Increment object refcount that the |ret| object is still a valid one
@@ -640,19 +655,23 @@ nsIDeviceContext* nsBaseWidget::GetDeviceContext()
   return mContext; 
 }
 
+#ifdef MOZ_CAIRO_GFX
 //-------------------------------------------------------------------------
 //
-// Return the App Shell
+// Get the thebes surface
 //
 //-------------------------------------------------------------------------
-
-nsIAppShell *nsBaseWidget::GetAppShell()
+gfxASurface *nsBaseWidget::GetThebesSurface()
 {
-  nsIAppShell*  theAppShell = mAppShell;
-  NS_IF_ADDREF(theAppShell);
-  return theAppShell;
-}
+  nsIWidget *parent = GetParent();
+  if (!parent)
+    return nsnull;
 
+  // in theory we should get our parent's surface,
+  // clone it, and set a device offset before returning
+  return nsnull;
+}
+#endif
 
 //-------------------------------------------------------------------------
 //
@@ -664,15 +683,9 @@ void nsBaseWidget::OnDestroy()
   // release references to device context, toolkit, and app shell
   NS_IF_RELEASE(mContext);
   NS_IF_RELEASE(mToolkit);
-  mAppShell = nsnull;     // clear out nsCOMPtr
 }
 
-NS_METHOD nsBaseWidget::GetWindowClass(char *aClass)
-{
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_METHOD nsBaseWidget::SetWindowClass(char *aClass)
+NS_METHOD nsBaseWidget::SetWindowClass(const nsAString& xulWinType)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -756,21 +769,6 @@ NS_METHOD nsBaseWidget::GetScreenBounds(nsRect &aRect)
   return GetBounds(aRect);
 }
 
-/**
-* If the implementation of nsWindow supports borders this method MUST be overridden
-*
-**/
-NS_METHOD nsBaseWidget::GetBoundsAppUnits(nsRect &aRect, float aAppUnits)
-{
-  aRect = mBounds;
-  // Convert to twips
-  aRect.x      = nscoord((PRFloat64)aRect.x * aAppUnits);
-  aRect.y      = nscoord((PRFloat64)aRect.y * aAppUnits);
-  aRect.width  = nscoord((PRFloat64)aRect.width * aAppUnits); 
-  aRect.height = nscoord((PRFloat64)aRect.height * aAppUnits);
-  return NS_OK;
-}
-
 /**
 * 
 *
@@ -801,93 +799,6 @@ NS_METHOD nsBaseWidget::GetBorderSize(PRInt32 &aWidth, PRInt32 &aHeight)
   return NS_OK;
 }
 
-
-/**
-* Calculates the border width and height  
-*
-**/
-void nsBaseWidget::DrawScaledRect(nsIRenderingContext& aRenderingContext, const nsRect & aRect, float aScale, float aAppUnits)
-{
-  nsRect rect = aRect;
-
-  float x = (float)rect.x;
-  float y = (float)rect.y;
-  float w = (float)rect.width;
-  float h = (float)rect.height;
-  float twoAppUnits = aAppUnits * 2.0f;
-
-  for (int i=0;i<int(aScale);i++) {
-    rect.x      = nscoord(x);
-    rect.y      = nscoord(y);
-    rect.width  = nscoord(w);
-    rect.height = nscoord(h);
-    aRenderingContext.DrawRect(rect);
-    x += aAppUnits; 
-    y += aAppUnits;
-    w -= twoAppUnits; 
-    h -= twoAppUnits;
-  }
-}
-
-/**
-* Calculates the border width and height  
-*
-**/
-void nsBaseWidget::DrawScaledLine(nsIRenderingContext& aRenderingContext, 
-                                  nscoord aSX, 
-                                  nscoord aSY, 
-                                  nscoord aEX, 
-                                  nscoord aEY, 
-                                  float   aScale, 
-                                  float   aAppUnits,
-                                  PRBool  aIsHorz)
-{
-  float sx = (float)aSX;
-  float sy = (float)aSY;
-  float ex = (float)aEX;
-  float ey = (float)aEY;
-
-  for (int i=0;i<int(aScale);i++) {
-    aSX = nscoord(sx);
-    aSY = nscoord(sy);
-    aEX = nscoord(ex);
-    aEY = nscoord(ey);
-    aRenderingContext.DrawLine(aSX, aSY, aEX, aEY);
-    if (aIsHorz) {
-      sy += aAppUnits; 
-      ey += aAppUnits;
-    } else {
-      sx += aAppUnits; 
-      ex += aAppUnits;
-    }
-  }
-}
-
-/**
-* Paints default border (XXX - this should be done by CSS)
-*
-**/
-NS_METHOD nsBaseWidget::Paint(nsIRenderingContext& aRenderingContext,
-                              const nsRect&        aDirtyRect)
-{
-  nsRect rect;
-  float  appUnits;
-  float  scale;
-  nsIDeviceContext * context;
-  aRenderingContext.GetDeviceContext(context);
-
-  context->GetCanonicalPixelScale(scale);
-  appUnits = context->DevUnitsToAppUnits();
-
-  GetBoundsAppUnits(rect, appUnits);
-  aRenderingContext.SetColor(NS_RGB(0,0,0));
-
-  DrawScaledRect(aRenderingContext, rect, scale, appUnits);
-
-  NS_RELEASE(context);
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsBaseWidget::ScrollWidgets(PRInt32 aDx, PRInt32 aDy)
 {
   return NS_ERROR_FAILURE;
@@ -1039,8 +950,7 @@ case _value: eventName.AssignWithConversion(_name) ; break
     _ASSIGN_eventName(NS_FORM_SUBMIT,"NS_FORM_SUBMIT");
     _ASSIGN_eventName(NS_GOTFOCUS,"NS_GOTFOCUS");
     _ASSIGN_eventName(NS_IMAGE_ABORT,"NS_IMAGE_ABORT");
-    _ASSIGN_eventName(NS_IMAGE_ERROR,"NS_IMAGE_ERROR");
-    _ASSIGN_eventName(NS_IMAGE_LOAD,"NS_IMAGE_LOAD");
+    _ASSIGN_eventName(NS_LOAD_ERROR,"NS_LOAD_ERROR");
     _ASSIGN_eventName(NS_KEY_DOWN,"NS_KEY_DOWN");
     _ASSIGN_eventName(NS_KEY_PRESS,"NS_KEY_PRESS");
     _ASSIGN_eventName(NS_KEY_UP,"NS_KEY_UP");
@@ -1062,7 +972,7 @@ case _value: eventName.AssignWithConversion(_name) ; break
     _ASSIGN_eventName(NS_MOUSE_RIGHT_CLICK,"NS_MOUSE_RIGHT_CLICK");
     _ASSIGN_eventName(NS_MOUSE_RIGHT_DOUBLECLICK,"NS_MOUSE_RIGHT_DBLCLICK");
     _ASSIGN_eventName(NS_MOVE,"NS_MOVE");
-    _ASSIGN_eventName(NS_PAGE_LOAD,"NS_PAGE_LOAD");
+    _ASSIGN_eventName(NS_LOAD,"NS_LOAD");
     _ASSIGN_eventName(NS_PAGE_UNLOAD,"NS_PAGE_UNLOAD");
     _ASSIGN_eventName(NS_PAINT,"NS_PAINT");
     _ASSIGN_eventName(NS_XUL_BROADCAST, "NS_XUL_BROADCAST");
@@ -1163,8 +1073,6 @@ static void debug_SetCachedBoolPref(const char * aPrefName,PRBool aValue)
   NS_ASSERTION(PR_FALSE, "cmon, this code is not reached dude.");
 }
 
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-
 //////////////////////////////////////////////////////////////
 /* static */ int PR_CALLBACK 
 debug_PrefChangedCallback(const char * name,void * closure)
@@ -1172,7 +1080,7 @@ debug_PrefChangedCallback(const char * name,void * closure)
 
   nsIPref * prefs = nsnull;
   
-  nsresult rv = CallGetService(kPrefCID, &prefs);
+  nsresult rv = CallGetService(NS_PREF_CONTRACTID, &prefs);
   
   NS_ASSERTION(NS_SUCCEEDED(rv),"Could not get prefs service.");
   NS_ASSERTION(nsnull != prefs,"Prefs services is null.");
@@ -1202,7 +1110,7 @@ debug_RegisterPrefCallbacks()
 
     nsIPref * prefs = nsnull;
 
-    nsresult rv = CallGetService(kPrefCID, &prefs);
+    nsresult rv = CallGetService(NS_PREF_CONTRACTID, &prefs);
     
     NS_ASSERTION(NS_SUCCEEDED(rv),"Could not get prefs service.");
     NS_ASSERTION(nsnull != prefs,"Prefs services is null.");
@@ -1270,14 +1178,14 @@ nsBaseWidget::debug_DumpEvent(FILE *                aFileOut,
   nsCAutoString tempString; tempString.AssignWithConversion(debug_GuiEventToString(aGuiEvent).get());
   
   fprintf(aFileOut,
-          "%4d %-26s widget=%-8p name=%-12s id=%-8p pos=%d,%d\n",
+          "%4d %-26s widget=%-8p name=%-12s id=%-8p refpt=%d,%d\n",
           _GetPrintCount(),
           tempString.get(),
           (void *) aWidget,
           aWidgetName.get(),
           (void *) (aWindowID ? aWindowID : 0x0),
-          aGuiEvent->point.x,
-          aGuiEvent->point.y);
+          aGuiEvent->refPoint.x,
+          aGuiEvent->refPoint.y);
 }
 //////////////////////////////////////////////////////////////
 /* static */ void
