@@ -79,8 +79,10 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
 
     if (curCapacity != size_type(-1))
       {
-        if (capacity <= curCapacity)
+        if (capacity <= curCapacity) {
+          mFlags &= ~F_VOIDED;  // mutation clears voided flag
           return PR_TRUE;
+        }
 
         if (curCapacity > 0)
           {
@@ -110,22 +112,17 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
     // case #1
     if (mFlags & F_SHARED)
       {
-        nsStringHeader* hdr = nsStringHeader::FromData(mData);
+        nsStringBuffer* hdr = nsStringBuffer::FromData(mData);
         if (!hdr->IsReadonly())
           {
-            nsStringHeader *newHdr = nsStringHeader::Realloc(hdr, storageSize);
-            if (newHdr)
-              {
-                hdr = newHdr;
-                mData = (char_type*) hdr->Data();
-                return PR_TRUE;
-              }
-            hdr->Release();
-            // out of memory!!  put us in a consistent state at least.
-            mData = NS_CONST_CAST(char_type*, char_traits::sEmptyBuffer);
-            mLength = 0;
-            SetDataFlags(F_TERMINATED);
-            return PR_FALSE;
+            nsStringBuffer *newHdr = nsStringBuffer::Realloc(hdr, storageSize);
+            if (!newHdr)
+              return PR_FALSE; // out-of-memory (original header left intact)
+
+            hdr = newHdr;
+            mData = (char_type*) hdr->Data();
+            mFlags &= ~F_VOIDED;  // mutation clears voided flag
+            return PR_TRUE;
           }
       }
 
@@ -145,7 +142,7 @@ nsTSubstring_CharT::MutatePrep( size_type capacity, char_type** oldData, PRUint3
         // make use of our F_OWNED or F_FIXED buffers because they are not
         // large enough.
 
-        nsStringHeader* newHdr = nsStringHeader::Alloc(storageSize);
+        nsStringBuffer* newHdr = nsStringBuffer::Alloc(storageSize);
         if (!newHdr)
           return PR_FALSE; // we are still in a consistent state
 
@@ -175,7 +172,14 @@ nsTSubstring_CharT::Finalize()
     // mData, mLength, and mFlags are purposefully left dangling
   }
 
-void
+#ifndef MOZ_V1_STRING_ABI
+nsTSubstring_CharT::~nsTSubstring_CharT()
+  {
+    Finalize();
+  }
+#endif
+
+PRBool
 nsTSubstring_CharT::ReplacePrep( index_type cutStart, size_type cutLen, size_type fragLen )
   {
     // bound cut length
@@ -186,7 +190,7 @@ nsTSubstring_CharT::ReplacePrep( index_type cutStart, size_type cutLen, size_typ
     char_type* oldData;
     PRUint32 oldFlags;
     if (!MutatePrep(newLen, &oldData, &oldFlags))
-      return; // XXX out-of-memory error occured!
+      return PR_FALSE; // out-of-memory
 
     if (oldData)
       {
@@ -229,6 +233,8 @@ nsTSubstring_CharT::ReplacePrep( index_type cutStart, size_type cutLen, size_typ
     // terminator).
     mData[newLen] = char_type(0);
     mLength = newLen;
+
+    return PR_TRUE;
   }
 
 nsTSubstring_CharT::size_type
@@ -240,7 +246,7 @@ nsTSubstring_CharT::Capacity() const
     if (mFlags & F_SHARED)
       {
         // if the string is readonly, then we pretend that it has no capacity.
-        nsStringHeader* hdr = nsStringHeader::FromData(mData);
+        nsStringBuffer* hdr = nsStringBuffer::FromData(mData);
         if (hdr->IsReadonly())
           capacity = size_type(-1);
         else
@@ -266,20 +272,39 @@ nsTSubstring_CharT::Capacity() const
     return capacity;
   }
 
-void
-nsTSubstring_CharT::EnsureMutable()
+PRBool
+nsTSubstring_CharT::EnsureMutable( size_type newLen )
   {
-    if (mFlags & (F_FIXED | F_OWNED))
-      return;
-    if ((mFlags & F_SHARED) && !nsStringHeader::FromData(mData)->IsReadonly())
-      return;
+    if (newLen == size_type(-1) || newLen == mLength)
+      {
+        if (mFlags & (F_FIXED | F_OWNED))
+          return PR_TRUE;
+        if ((mFlags & F_SHARED) && !nsStringBuffer::FromData(mData)->IsReadonly())
+          return PR_TRUE;
 
-    // promote to a shared string buffer
-    Assign(string_type(mData, mLength));
+        // promote to a shared string buffer
+        char_type* prevData = mData;
+        Assign(string_type(mData, mLength));
+        return mData != prevData;
+      }
+    else
+      {
+        SetLength(newLen);
+        return mLength == newLen;
+      }
   }
 
 // ---------------------------------------------------------------------------
 
+  // This version of Assign is optimized for single-character assignment.
+void
+nsTSubstring_CharT::Assign( char_type c )
+  {
+    if (ReplacePrep(0, mLength, 1))
+      *mData = c;
+  }
+
+
 void
 nsTSubstring_CharT::Assign( const char_type* data, size_type length )
   {
@@ -300,8 +325,8 @@ nsTSubstring_CharT::Assign( const char_type* data, size_type length )
         return;
       }
 
-    ReplacePrep(0, mLength, length);
-    char_traits::copy(mData, data, length);
+    if (ReplacePrep(0, mLength, length))
+      char_traits::copy(mData, data, length);
   }
 
 void
@@ -318,8 +343,8 @@ nsTSubstring_CharT::AssignASCII( const char* data, size_type length )
       }
 #endif
 
-    ReplacePrep(0, mLength, length);
-    char_traits::copyASCII(mData, data, length);
+    if (ReplacePrep(0, mLength, length))
+      char_traits::copyASCII(mData, data, length);
   }
 
 void
@@ -351,7 +376,7 @@ nsTSubstring_CharT::Assign( const self_type& str )
         SetDataFlags(F_TERMINATED | F_SHARED);
 
         // get an owning reference to the mData
-        nsStringHeader::FromData(mData)->AddRef();
+        nsStringBuffer::FromData(mData)->AddRef();
       }
     else if (str.mFlags & F_VOIDED)
       {
@@ -377,12 +402,12 @@ nsTSubstring_CharT::Assign( const substring_tuple_type& tuple )
 
     size_type length = tuple.Length();
 
-    ReplacePrep(0, mLength, length);
-    if (length)
+    if (ReplacePrep(0, mLength, length) && length)
       tuple.WriteTo(mData, length);
   }
 
   // this is non-inline to reduce codesize at the callsite
+#ifdef MOZ_V1_STRING_ABI
 void
 nsTSubstring_CharT::Assign( const abstract_string_type& readable )
   {
@@ -392,6 +417,7 @@ nsTSubstring_CharT::Assign( const abstract_string_type& readable )
     else
       Assign(readable.ToSubstring());
   }
+#endif
 
 
 void
@@ -409,6 +435,11 @@ nsTSubstring_CharT::Adopt( char_type* data, size_type length )
         SetDataFlags(F_TERMINATED | F_OWNED);
 
         STRING_STAT_INCREMENT(Adopt);
+#ifdef NS_BUILD_REFCNT_LOGGING
+        // Treat this as construction of a "StringAdopt" object for leak
+        // tracking purposes.        
+        NS_LogCtor(mData, "StringAdopt", 1);
+#endif // NS_BUILD_REFCNT_LOGGING
       }
     else
       {
@@ -417,6 +448,17 @@ nsTSubstring_CharT::Adopt( char_type* data, size_type length )
   }
 
 
+  // This version of Replace is optimized for single-character replacement.
+void
+nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, char_type c )
+  {
+    cutStart = PR_MIN(cutStart, Length());
+
+    if (ReplacePrep(cutStart, cutLength, 1))
+      mData[cutStart] = c;
+  }
+
+
 void
 nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, const char_type* data, size_type length )
   {
@@ -440,9 +482,7 @@ nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, const cha
 
     cutStart = PR_MIN(cutStart, Length());
 
-    ReplacePrep(cutStart, cutLength, length);
-
-    if (length > 0)
+    if (ReplacePrep(cutStart, cutLength, length) && length > 0)
       char_traits::copy(mData + cutStart, data, length);
   }
 
@@ -465,9 +505,7 @@ nsTSubstring_CharT::ReplaceASCII( index_type cutStart, size_type cutLength, cons
 
     cutStart = PR_MIN(cutStart, Length());
 
-    ReplacePrep(cutStart, cutLength, length);
-
-    if (length > 0)
+    if (ReplacePrep(cutStart, cutLength, length) && length > 0)
       char_traits::copyASCII(mData + cutStart, data, length);
   }
 
@@ -485,17 +523,17 @@ nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, const sub
 
     cutStart = PR_MIN(cutStart, Length());
 
-    ReplacePrep(cutStart, cutLength, length);
-
-    if (length > 0)
+    if (ReplacePrep(cutStart, cutLength, length) && length > 0)
       tuple.WriteTo(mData + cutStart, length);
   }
 
+#ifdef MOZ_V1_STRING_ABI
 void
 nsTSubstring_CharT::Replace( index_type cutStart, size_type cutLength, const abstract_string_type& readable )
   {
     Replace(cutStart, cutLength, readable.ToSubstring());
   }
+#endif
 
 void
 nsTSubstring_CharT::SetCapacity( size_type capacity )
@@ -515,7 +553,7 @@ nsTSubstring_CharT::SetCapacity( size_type capacity )
         char_type* oldData;
         PRUint32 oldFlags;
         if (!MutatePrep(capacity, &oldData, &oldFlags))
-          return; // XXX out-of-memory error occured!
+          return; // out-of-memory
 
         // compute new string length
         size_type newLen = NS_MIN(mLength, capacity);
@@ -542,8 +580,19 @@ nsTSubstring_CharT::SetCapacity( size_type capacity )
 void
 nsTSubstring_CharT::SetLength( size_type length )
   {
+    if (mLength == length) {
+      mFlags &= ~F_VOIDED;  // mutation clears voided flag
+      return;
+    }
+
     SetCapacity(length);
-    mLength = length;
+
+    // XXX(darin): SetCapacity may fail, but it doesn't give us a way to find
+    // out.  We should improve that.  For now we just verify that the capacity
+    // changed as expected as a means of error checking.
+ 
+    if (Capacity() >= length)
+      mLength = length;
   }
 
 void
@@ -572,6 +621,7 @@ nsTSubstring_CharT::Equals( const self_type& str, const comparator_type& comp )
     return mLength == str.mLength && comp(mData, str.mData, mLength) == 0;
   }
 
+#ifdef MOZ_V1_STRING_ABI
 PRBool
 nsTSubstring_CharT::Equals( const abstract_string_type& readable ) const
   {
@@ -589,6 +639,7 @@ nsTSubstring_CharT::Equals( const abstract_string_type& readable, const comparat
 
     return mLength == length && comp(mData, data, mLength) == 0;
   }
+#endif
 
 PRBool
 nsTSubstring_CharT::Equals( const char_type* data ) const
@@ -664,3 +715,27 @@ nsTSubstring_CharT::FindChar( char_type c, index_type offset ) const
       }
     return -1;
   }
+
+void
+nsTSubstring_CharT::StripChar( char_type aChar, PRInt32 aOffset )
+  {
+    if (mLength == 0 || aOffset >= PRInt32(mLength))
+      return;
+
+    EnsureMutable(); // XXX do this lazily?
+
+    // XXX(darin): this code should defer writing until necessary.
+
+    char_type* to   = mData + aOffset;
+    char_type* from = mData + aOffset;
+    char_type* end  = mData + mLength;
+
+    while (from < end)
+      {
+        char_type theChar = *from++;
+        if (aChar != theChar)
+          *to++ = theChar;
+      }
+    *to = char_type(0); // add the null
+    mLength = to - mData;
+  }
