@@ -44,12 +44,15 @@
 // Helper Classes
 #include "nsIGenericFactory.h"
 #include "nsIServiceManager.h"
+#include "nsAutoPtr.h"
 
 // Interfaces needed to be included
 #include "nsIDOMNode.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMNodeList.h"
 #include "nsIDOMWindowInternal.h"
+#include "nsIDOMChromeWindow.h"
+#include "nsIBrowserDOMWindow.h"
 #include "nsIDOMXULElement.h"
 #include "nsIEmbeddingSiteWindow.h"
 #include "nsIEmbeddingSiteWindow2.h"
@@ -57,10 +60,16 @@
 #include "nsIAuthPrompt.h"
 #include "nsIWindowMediator.h"
 #include "nsIXULBrowserWindow.h"
+#include "nsIPrincipal.h"
+#include "nsIURIFixup.h"
+#include "nsCDefaultURIFixup.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+#include "nsIWebNavigation.h"
 
-// Needed for nsIDocument::FlushPendingNotifications(...)
 #include "nsIDOMDocument.h"
-#include "nsIDocument.h"
+#include "nsIScriptObjectPrincipal.h"
+#include "nsIURI.h"
 
 // CIDs
 static NS_DEFINE_CID(kWindowMediatorCID, NS_WINDOWMEDIATOR_CID);
@@ -88,8 +97,7 @@ private:
 //*****************************************************************************
 
 nsContentTreeOwner::nsContentTreeOwner(PRBool fPrimary) : mXULWindow(nsnull), 
-   mPrimary(fPrimary), mContentTitleSetting(PR_FALSE), 
-   mChromeFlags(nsIWebBrowserChrome::CHROME_ALL)
+   mPrimary(fPrimary), mContentTitleSetting(PR_FALSE)
 {
   // note if this fails, QI on nsIEmbeddingSiteWindow(2) will simply fail
   mSiteWindow2 = new nsSiteWindow2(this);
@@ -110,9 +118,20 @@ NS_IMPL_RELEASE(nsContentTreeOwner)
 NS_INTERFACE_MAP_BEGIN(nsContentTreeOwner)
    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDocShellTreeOwner)
    NS_INTERFACE_MAP_ENTRY(nsIDocShellTreeOwner)
+   NS_INTERFACE_MAP_ENTRY(nsIDocShellTreeOwner_MOZILLA_1_8_BRANCH)
    NS_INTERFACE_MAP_ENTRY(nsIBaseWindow)
    NS_INTERFACE_MAP_ENTRY(nsIWebBrowserChrome)
+   NS_INTERFACE_MAP_ENTRY(nsIWebBrowserChrome2)
    NS_INTERFACE_MAP_ENTRY(nsIInterfaceRequestor)
+   NS_INTERFACE_MAP_ENTRY(nsIWindowProvider)
+   // NOTE: This is using aggregation because there are some properties and
+   // method on nsIBaseWindow (which we implement) and on
+   // nsIEmbeddingSiteWindow (which we also implement) that have the same name.
+   // And it just so happens that we want different behavior for these methods
+   // and properties depending on the interface through which they're called
+   // (SetFocus() is a good example here).  If it were not for that, we could
+   // ditch the aggregation and just deal with not being able to use NS_DECL_*
+   // macros for this stuff....
    NS_INTERFACE_MAP_ENTRY_AGGREGATED(nsIEmbeddingSiteWindow, mSiteWindow2)
    NS_INTERFACE_MAP_ENTRY_AGGREGATED(nsIEmbeddingSiteWindow2, mSiteWindow2)
 NS_INTERFACE_MAP_END
@@ -126,9 +145,6 @@ NS_IMETHODIMP nsContentTreeOwner::GetInterface(const nsIID& aIID, void** aSink)
   NS_ENSURE_ARG_POINTER(aSink);
   *aSink = 0;
 
-  if(aIID.Equals(NS_GET_IID(nsIWebBrowserChrome)))
-    return mXULWindow->GetInterface(aIID, aSink);
-
   if(aIID.Equals(NS_GET_IID(nsIPrompt)))
     return mXULWindow->GetInterface(aIID, aSink);
   if(aIID.Equals(NS_GET_IID(nsIAuthPrompt)))
@@ -136,12 +152,8 @@ NS_IMETHODIMP nsContentTreeOwner::GetInterface(const nsIID& aIID, void** aSink)
   if (aIID.Equals(NS_GET_IID(nsIDocShellTreeItem))) {
     nsCOMPtr<nsIDocShell> shell;
     mXULWindow->GetDocShell(getter_AddRefs(shell));
-    if (shell) {
-      nsIDocShellTreeItem *result;
-      CallQueryInterface(shell, &result);
-      *aSink = result;
-      return NS_OK;
-    }
+    if (shell)
+      return shell->QueryInterface(aIID, aSink);
     return NS_ERROR_FAILURE;
   }
 
@@ -167,30 +179,56 @@ NS_IMETHODIMP nsContentTreeOwner::GetInterface(const nsIID& aIID, void** aSink)
 //*****************************************************************************   
 
 NS_IMETHODIMP nsContentTreeOwner::FindItemWithName(const PRUnichar* aName,
-   nsIDocShellTreeItem* aRequestor, nsIDocShellTreeItem** aFoundItem)
+   nsIDocShellTreeItem* aRequestor, nsIDocShellTreeItem* aOriginalRequestor,
+   nsIDocShellTreeItem** aFoundItem)
 {
    NS_ENSURE_ARG_POINTER(aFoundItem);
 
    *aFoundItem = nsnull;
 
-   nsAutoString   name(aName);
-
    PRBool fIs_Content = PR_FALSE;
 
    /* Special Cases */
-   if(name.IsEmpty())
+   if (!aName || !*aName)
       return NS_OK;
-   if(name.LowerCaseEqualsLiteral("_blank"))
+
+   nsDependentString name(aName);
+
+   if (name.LowerCaseEqualsLiteral("_blank"))
       return NS_OK;
    // _main is an IE target which should be case-insensitive but isn't
    // see bug 217886 for details
-   if(name.LowerCaseEqualsLiteral("_content") || name.EqualsLiteral("_main"))
-      {
-      fIs_Content = PR_TRUE;
-      mXULWindow->GetPrimaryContentShell(aFoundItem);
-      if(*aFoundItem)
-         return NS_OK;
-      }
+   if (name.LowerCaseEqualsLiteral("_content") ||
+       name.EqualsLiteral("_main")) {
+     // If we're being called with an aRequestor and it's targetable, just
+     // return it -- _main and _content from inside targetable content shells
+     // should just be that content shell.  Note that we don't have to worry
+     // about the case when it's not targetable because it's primary -- that
+     // will Just Work when we call GetPrimaryContentShell.
+     if (aRequestor) {
+       // This better be the root item!
+#ifdef DEBUG
+       nsCOMPtr<nsIDocShellTreeItem> debugRoot;
+       aRequestor->GetSameTypeRootTreeItem(getter_AddRefs(debugRoot));
+       NS_ASSERTION(SameCOMIdentity(debugRoot, aRequestor),
+                    "Bogus aRequestor");
+#endif
+       PRInt32 count = mXULWindow->mTargetableShells.Count();
+       for (PRInt32 i = 0; i < count; ++i) {
+         nsCOMPtr<nsIDocShellTreeItem> item =
+           do_QueryReferent(mXULWindow->mTargetableShells[i]);
+         if (SameCOMIdentity(item, aRequestor)) {
+           NS_ADDREF(*aFoundItem = aRequestor);
+           return NS_OK;
+         }
+       }
+     }
+     mXULWindow->GetPrimaryContentShell(aFoundItem);
+     if(*aFoundItem)
+       return NS_OK;
+     // Fall through and keep looking...
+     fIs_Content = PR_TRUE;
+   }
 
    nsCOMPtr<nsIWindowMediator> windowMediator(do_GetService(kWindowMediatorCID));
    NS_ENSURE_TRUE(windowMediator, NS_ERROR_FAILURE);
@@ -202,46 +240,70 @@ NS_IMETHODIMP nsContentTreeOwner::FindItemWithName(const PRUnichar* aName,
    PRBool more;
    
    windowEnumerator->HasMoreElements(&more);
-   while(more)
-      {
-      nsCOMPtr<nsISupports> nextWindow = nsnull;
-      windowEnumerator->GetNext(getter_AddRefs(nextWindow));
-      nsCOMPtr<nsIXULWindow> xulWindow(do_QueryInterface(nextWindow));
-      NS_ENSURE_TRUE(xulWindow, NS_ERROR_FAILURE);
+   while(more) {
+     nsCOMPtr<nsISupports> nextWindow = nsnull;
+     windowEnumerator->GetNext(getter_AddRefs(nextWindow));
+     nsCOMPtr<nsIXULWindow> xulWindow(do_QueryInterface(nextWindow));
+     NS_ENSURE_TRUE(xulWindow, NS_ERROR_FAILURE);
 
-      nsCOMPtr<nsIDocShellTreeItem> shellAsTreeItem;
-      xulWindow->GetPrimaryContentShell(getter_AddRefs(shellAsTreeItem));
+     if (fIs_Content) {
+       xulWindow->GetPrimaryContentShell(aFoundItem);
+     } else {
+       // Get all the targetable windows from xulWindow and search them
+       nsRefPtr<nsXULWindow> win;
+       xulWindow->QueryInterface(NS_GET_IID(nsXULWindow), getter_AddRefs(win));
+       if (win) {
+         PRInt32 count = win->mTargetableShells.Count();
+         PRInt32 i;
+         for (i = 0; i < count; ++i) {
+           nsCOMPtr<nsIDocShellTreeItem> shellAsTreeItem =
+             do_QueryReferent(win->mTargetableShells[i]);
+           if (shellAsTreeItem) {
+             // Get the root tree item of same type, since roots are the only
+             // things that call into the treeowner to look for named items.
+             // XXXbz ideally we could guarantee that mTargetableShells only
+             // contains roots, but the current treeowner apis don't allow
+             // that... yet.
+             nsCOMPtr<nsIDocShellTreeItem> root;
+             shellAsTreeItem->GetSameTypeRootTreeItem(getter_AddRefs(root));
+             NS_ASSERTION(root, "Must have root tree item of same type");
+             shellAsTreeItem.swap(root);
+             if (aRequestor != shellAsTreeItem) {
+               // Do this so we can pass in the tree owner as the
+               // requestor so the child knows not to call back up.
+               nsCOMPtr<nsIDocShellTreeOwner> shellOwner;
+               shellAsTreeItem->GetTreeOwner(getter_AddRefs(shellOwner));
+               nsCOMPtr<nsISupports> shellOwnerSupports =
+                 do_QueryInterface(shellOwner);
 
-      if(shellAsTreeItem)
-         {
-         if(fIs_Content)
-		    {
-            *aFoundItem = shellAsTreeItem;
-            NS_ADDREF(*aFoundItem);
-            }
-         else if(aRequestor != shellAsTreeItem.get())
-            {
-            // Do this so we can pass in the tree owner as the requestor so the child knows not
-            // to call back up.
-            nsCOMPtr<nsIDocShellTreeOwner> shellOwner;
-            shellAsTreeItem->GetTreeOwner(getter_AddRefs(shellOwner));
-            nsCOMPtr<nsISupports> shellOwnerSupports(do_QueryInterface(shellOwner));
-
-            shellAsTreeItem->FindItemWithName(aName, shellOwnerSupports, aFoundItem);
-            }
-         if(*aFoundItem)
-            return NS_OK;
+               shellAsTreeItem->FindItemWithName(aName, shellOwnerSupports,
+                                                 aOriginalRequestor,
+                                                 aFoundItem);
+             }
+           }
          }
-      windowEnumerator->HasMoreElements(&more);
-      }
+       }
+     }
+     
+     if (*aFoundItem)
+       return NS_OK;
+
+     windowEnumerator->HasMoreElements(&more);
+   }
    return NS_OK;      
 }
 
 NS_IMETHODIMP nsContentTreeOwner::ContentShellAdded(nsIDocShellTreeItem* aContentShell,
    PRBool aPrimary, const PRUnichar* aID)
 {
-   mXULWindow->ContentShellAdded(aContentShell, aPrimary, aID);
-   return NS_OK;
+   NS_ENSURE_STATE(mXULWindow);
+   if (aID) {
+     return mXULWindow->ContentShellAdded(aContentShell, aPrimary, PR_FALSE,
+                                          nsDependentString(aID));
+   }
+
+   return mXULWindow->ContentShellAdded(aContentShell, aPrimary, PR_FALSE,
+                                        EmptyString());
 }
 
 NS_IMETHODIMP nsContentTreeOwner::GetPrimaryContentShell(nsIDocShellTreeItem** aShell)
@@ -349,33 +411,55 @@ nsContentTreeOwner::GetPersistence(PRBool* aPersistPosition,
 }
 
 //*****************************************************************************
-// nsContentTreeOwner::nsIWebBrowserChrome
+// nsContentTreeOwner::nsIWebBrowserChrome2
 //*****************************************************************************   
 
-NS_IMETHODIMP nsContentTreeOwner::SetStatus(PRUint32 aStatusType, const PRUnichar* aStatus)
+NS_IMETHODIMP nsContentTreeOwner::SetStatusWithContext(PRUint32 aStatusType,
+                                                       const nsAString &aStatusText,
+                                                       nsISupports *aStatusContext)
 {
+  // We only allow the status to be set from the primary content shell
+  if (!mPrimary && aStatusType != STATUS_LINK)
+    return NS_OK;
+  
   nsCOMPtr<nsIXULBrowserWindow> xulBrowserWindow;
   mXULWindow->GetXULBrowserWindow(getter_AddRefs(xulBrowserWindow));
 
-   if (xulBrowserWindow)
-   {
-     switch(aStatusType)
-     {
-     case STATUS_SCRIPT:
-       xulBrowserWindow->SetJSStatus(aStatus);
-       break;
-     case STATUS_SCRIPT_DEFAULT:
-       xulBrowserWindow->SetJSDefaultStatus(aStatus);
-       break;
-     case STATUS_LINK:
-       xulBrowserWindow->SetOverLink(aStatus);
-       break;
-     }
-   }
+  if (xulBrowserWindow)
+  {
+    switch(aStatusType)
+    {
+    case STATUS_SCRIPT:
+      xulBrowserWindow->SetJSStatus(aStatusText);
+      break;
+    case STATUS_SCRIPT_DEFAULT:
+      xulBrowserWindow->SetJSDefaultStatus(aStatusText);
+      break;
+    case STATUS_LINK:
+      {
+        nsCOMPtr<nsIDOMElement> element = do_QueryInterface(aStatusContext);
+        xulBrowserWindow->SetOverLink(aStatusText, element);
+        break;
+      }
+    }
+  }
 
   return NS_OK;
 }
 
+//*****************************************************************************
+// nsContentTreeOwner::nsIWebBrowserChrome
+//*****************************************************************************   
+
+NS_IMETHODIMP nsContentTreeOwner::SetStatus(PRUint32 aStatusType,
+                                            const PRUnichar* aStatus)
+{
+  return SetStatusWithContext(aStatusType,
+      aStatus ? NS_STATIC_CAST(const nsString &, nsDependentString(aStatus))
+              : EmptyString(),
+      nsnull);
+}
+
 NS_IMETHODIMP nsContentTreeOwner::SetWebBrowser(nsIWebBrowser* aWebBrowser)
 {
    NS_ERROR("Haven't Implemented this yet");
@@ -393,34 +477,12 @@ NS_IMETHODIMP nsContentTreeOwner::GetWebBrowser(nsIWebBrowser** aWebBrowser)
 
 NS_IMETHODIMP nsContentTreeOwner::SetChromeFlags(PRUint32 aChromeFlags)
 {
-   mChromeFlags = aChromeFlags;
-   NS_ENSURE_SUCCESS(ApplyChromeFlags(), NS_ERROR_FAILURE);
-
-   return NS_OK;
+   return mXULWindow->SetChromeFlags(aChromeFlags);
 }
 
 NS_IMETHODIMP nsContentTreeOwner::GetChromeFlags(PRUint32* aChromeFlags)
 {
-  NS_ENSURE_ARG_POINTER(aChromeFlags);
-
-  *aChromeFlags = mChromeFlags;
-
-  /* mChromeFlags is kept up to date, except for scrollbar visibility.
-     That can be changed directly by the content DOM window, which
-     doesn't know to update the chrome window. So that we must check
-     separately. */
-
-  // however, it's pointless to ask if the window isn't set up yet
-  if (!mXULWindow->mChromeLoaded)
-    return NS_OK;
-
-  PRBool scrollbarVisibility = mXULWindow->GetContentScrollbarVisibility();
-  if (scrollbarVisibility)
-    *aChromeFlags |= nsIWebBrowserChrome::CHROME_SCROLLBARS;
-  else
-    *aChromeFlags &= ~nsIWebBrowserChrome::CHROME_SCROLLBARS;
-
-  return NS_OK;
+  return mXULWindow->GetChromeFlags(aChromeFlags);
 }
 
 NS_IMETHODIMP nsContentTreeOwner::DestroyBrowserWindow()
@@ -583,9 +645,7 @@ NS_IMETHODIMP nsContentTreeOwner::GetTitle(PRUnichar** aTitle)
 {
    NS_ENSURE_ARG_POINTER(aTitle);
 
-   //XXX First Check In
-   NS_ASSERTION(PR_FALSE, "Not Yet Implemented");
-   return NS_OK;
+   return mXULWindow->GetTitle(aTitle);
 }
 
 NS_IMETHODIMP nsContentTreeOwner::SetTitle(const PRUnichar* aTitle)
@@ -611,76 +671,193 @@ NS_IMETHODIMP nsContentTreeOwner::SetTitle(const PRUnichar* aTitle)
       title = docTitle;
     }
   
-    title += mTitleSeparator + mWindowTitleModifier;
+    if (!mWindowTitleModifier.IsEmpty())
+      title += mTitleSeparator + mWindowTitleModifier;
   }
   else
     title.Assign(mWindowTitleModifier); // Title will just be plain "Mozilla"
 
-  // XXX Don't need to fully qualify this once I remove nsWebShellWindow::SetTitle
-  // return mXULWindow->SetTitle(title.get());
-  return mXULWindow->nsXULWindow::SetTitle(title.get());
+  //
+  // if there is no location bar we modify the title to display at least
+  // the scheme and host (if any) as an anti-spoofing measure.
+  //
+  nsCOMPtr<nsIDOMElement> docShellElement;
+  mXULWindow->GetWindowDOMElement(getter_AddRefs(docShellElement));
+
+  if (docShellElement) {
+    nsAutoString chromeString;
+    docShellElement->GetAttribute(NS_LITERAL_STRING("chromehidden"), chromeString);
+    if (chromeString.Find(NS_LITERAL_STRING("location")) != kNotFound) {
+      //
+      // location bar is turned off, find the browser location
+      //
+      // use the document's nsPrincipal to find the true owner
+      // in case of javascript: or data: documents
+      //
+      nsCOMPtr<nsIDocShellTreeItem> dsitem;
+      GetPrimaryContentShell(getter_AddRefs(dsitem));
+      nsCOMPtr<nsIDOMDocument> domdoc(do_GetInterface(dsitem));
+      nsCOMPtr<nsIScriptObjectPrincipal> doc(do_QueryInterface(domdoc));
+      if (doc) {
+        nsCOMPtr<nsIURI> uri;
+        nsIPrincipal* principal = doc->GetPrincipal();
+        if (principal) {
+          principal->GetURI(getter_AddRefs(uri));
+          if (uri) {
+            //
+            // remove any user:pass information
+            //
+            nsCOMPtr<nsIURIFixup> fixup(do_GetService(NS_URIFIXUP_CONTRACTID));
+            if (fixup) {
+              nsCOMPtr<nsIURI> tmpuri;
+              nsresult rv = fixup->CreateExposableURI(uri,getter_AddRefs(tmpuri));
+              if (NS_SUCCEEDED(rv) && tmpuri) {
+                // (don't bother if there's no host)
+                nsCAutoString host;
+                nsCAutoString prepath;
+                tmpuri->GetHost(host);
+                tmpuri->GetPrePath(prepath);
+                if (!host.IsEmpty()) {
+                  //
+                  // We have a scheme/host, update the title
+                  //
+                  title.Insert(NS_ConvertUTF8toUTF16(prepath) +
+                               mTitleSeparator, 0);
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+
+  return mXULWindow->SetTitle(title.get());
 }
 
 //*****************************************************************************
-// nsContentTreeOwner: Helpers
+// nsContentTreeOwner: nsIWindowProvider
 //*****************************************************************************   
-
-NS_IMETHODIMP nsContentTreeOwner::ApplyChromeFlags()
+NS_IMETHODIMP
+nsContentTreeOwner::ProvideWindow(nsIDOMWindow* aParent,
+                                  PRUint32 aChromeFlags,
+                                  PRBool aPositionSpecified,
+                                  PRBool aSizeSpecified,
+                                  nsIURI* aURI,
+                                  const nsAString& aName,
+                                  const nsACString& aFeatures,
+                                  PRBool* aWindowIsNew,
+                                  nsIDOMWindow** aReturn)
 {
-  if(!mXULWindow->mChromeLoaded)
-    return NS_OK;  // We'll do this later when chrome is loaded
-      
-  nsCOMPtr<nsIDOMElement> window;
-  mXULWindow->GetWindowDOMElement(getter_AddRefs(window));
-  NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
+  NS_ENSURE_ARG_POINTER(aParent);
+  
+  *aReturn = nsnull;
 
-  // menubar has its own special treatment
-  mXULWindow->mWindow->ShowMenuBar(mChromeFlags & 
-                                     nsIWebBrowserChrome::CHROME_MENUBAR ? 
-                                   PR_TRUE : PR_FALSE);
+  if (!mXULWindow) {
+    // Nothing to do here
+    return NS_OK;
+  }
 
-  /* Scrollbars have their own special treatment. (note here we *do* use
-     mChromeFlags directly, without going through the accessor. This method
-     is intended to be used right after setting mChromeFlags, so this is
-     where the content window's scrollbar state is set to match mChromeFlags
-     in the first place. */
-  mXULWindow->SetContentScrollbarVisibility(mChromeFlags &
-                                              nsIWebBrowserChrome::CHROME_SCROLLBARS ?
-                                            PR_TRUE : PR_FALSE);
+#ifdef DEBUG
+  nsCOMPtr<nsIWebNavigation> parentNav = do_GetInterface(aParent);
+  nsCOMPtr<nsIDocShellTreeOwner> parentOwner = do_GetInterface(parentNav);
+  NS_ASSERTION(SameCOMIdentity(parentOwner,
+                               NS_STATIC_CAST(nsIDocShellTreeOwner*, this)),
+               "Parent from wrong docshell tree?");
+#endif
 
-  /* the other flags are handled together. we have style rules
-     in navigator.css that trigger visibility based on
-     the 'chromehidden' attribute of the <window> tag. */
-  nsAutoString newvalue;
+  // First check what our prefs say
+  nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  if (!prefs) {
+    return NS_OK;
+  }
 
-  if (! (mChromeFlags & nsIWebBrowserChrome::CHROME_MENUBAR))
-    newvalue.AppendLiteral("menubar ");
+  nsCOMPtr<nsIPrefBranch> branch;
+  prefs->GetBranch("browser.link.", getter_AddRefs(branch));
+  if (!branch) {
+    return NS_OK;
+  }
 
-  if (! (mChromeFlags & nsIWebBrowserChrome::CHROME_TOOLBAR))
-    newvalue.AppendLiteral("toolbar ");
+  // Where should we open this?
+  PRInt32 containerPref;
+  if (NS_FAILED(branch->GetIntPref("open_newwindow", &containerPref))) {
+    return NS_OK;
+  }
 
-  if (! (mChromeFlags & nsIWebBrowserChrome::CHROME_LOCATIONBAR))
-    newvalue.AppendLiteral("location ");
+  if (containerPref != nsIBrowserDOMWindow::OPEN_NEWTAB &&
+      containerPref != nsIBrowserDOMWindow::OPEN_CURRENTWINDOW) {
+    // Just open a window normally
+    return NS_OK;
+  }
 
-  if (! (mChromeFlags & nsIWebBrowserChrome::CHROME_PERSONAL_TOOLBAR))
-    newvalue.AppendLiteral("directories ");
+  /* Now check our restriction pref.  The restriction pref is a power-user's
+     fine-tuning pref. values:     
+     0: no restrictions - divert everything
+     1: don't divert window.open at all
+     2: don't divert window.open with features
+  */
+  PRInt32 restrictionPref;
+  if (NS_FAILED(branch->GetIntPref("open_newwindow.restriction",
+                                   &restrictionPref)) ||
+      restrictionPref < 0 ||
+      restrictionPref > 2) {
+    restrictionPref = 2; // Sane default behavior
+  }
 
-  if (! (mChromeFlags & nsIWebBrowserChrome::CHROME_STATUSBAR))
-    newvalue.AppendLiteral("status ");
+  if (restrictionPref == 1) {
+    return NS_OK;
+  }
 
-  if (! (mChromeFlags & nsIWebBrowserChrome::CHROME_EXTRA))
-    newvalue.AppendLiteral("extrachrome");
+  if (restrictionPref == 2 &&
+      // Only continue if there are no size/position features and no special
+      // chrome flags.
+      (aChromeFlags != nsIWebBrowserChrome::CHROME_ALL ||
+       aPositionSpecified || aSizeSpecified)) {
+    return NS_OK;
+  }
 
+  nsCOMPtr<nsIDOMWindowInternal> domWin;
+  mXULWindow->GetWindowDOMWindow(getter_AddRefs(domWin));
+  nsCOMPtr<nsIDOMChromeWindow> chromeWin = do_QueryInterface(domWin);
+  if (!chromeWin) {
+    // Really odd... but whatever
+    NS_WARNING("nsXULWindow's DOMWindow is not a chrome window");
+    return NS_OK;
+  }
+  
+  nsCOMPtr<nsIBrowserDOMWindow> browserDOMWin;
+  chromeWin->GetBrowserDOMWindow(getter_AddRefs(browserDOMWin));
+  if (!browserDOMWin) {
+    return NS_OK;
+  }
 
-  // Get the old value, to avoid useless style reflows if we're just
-  // setting stuff to the exact same thing.
-  nsAutoString oldvalue;
-  window->GetAttribute(NS_LITERAL_STRING("chromehidden"), oldvalue);
+  *aWindowIsNew = (containerPref != nsIBrowserDOMWindow::OPEN_CURRENTWINDOW);
+  
+  // Get a new rendering area from the browserDOMWin.  To make this
+  // safe for cases when it'll try to return an existing window or
+  // something, get it with a null URI.
+  return browserDOMWin->OpenURI(nsnull, aParent, containerPref,
+                                nsIBrowserDOMWindow::OPEN_NEW, aReturn);
+}
 
-  if (oldvalue != newvalue)
-    window->SetAttribute(NS_LITERAL_STRING("chromehidden"), newvalue);
+//*****************************************************************************
+// nsContentTreeOwner::nsIDocShellTreeOwner_MOZILLA_1_8_BRANCH
+//*****************************************************************************   
+NS_IMETHODIMP
+nsContentTreeOwner::ContentShellAdded2(nsIDocShellTreeItem* aContentShell,
+                                       PRBool aPrimary, PRBool aTargetable,
+                                       const nsAString& aID)
+{
+  NS_ENSURE_STATE(mXULWindow);
+  return mXULWindow->ContentShellAdded(aContentShell, aPrimary, aTargetable,
+                                       aID);
+}
 
-  return NS_OK;
+NS_IMETHODIMP
+nsContentTreeOwner::ContentShellRemoved(nsIDocShellTreeItem* aContentShell)
+{
+  NS_ENSURE_STATE(mXULWindow);
+  return mXULWindow->ContentShellRemoved(aContentShell);
 }
 
 //*****************************************************************************
@@ -718,9 +895,8 @@ void nsContentTreeOwner::XULWindow(nsXULWindow* aXULWindow)
                 mTitleDefault = mWindowTitleModifier;
                 mWindowTitleModifier.Truncate();
             }
-#else
-            docShellElement->GetAttribute(NS_LITERAL_STRING("titlemenuseparator"), mTitleSeparator);
 #endif
+            docShellElement->GetAttribute(NS_LITERAL_STRING("titlemenuseparator"), mTitleSeparator);
             }
          }
       else
