@@ -21,6 +21,8 @@
  *
  * Contributor(s):
  *   Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
+ *   Fredrik Holmqvist <thesuckiestemail@yahoo.se>
+ *   Masayuki Nakano <masayuki@d-toybox.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -39,26 +41,26 @@
 
 #ifdef XPCOM_GLUE
 #include "nsXPCOMGlue.h"
-#include "nsStringSupport.h"
-#else
-#include "nsString.h"
 #endif
 
+#include "nsStringSupport.h"
+
 #include "nsXPCOM.h"
 #include "nsIServiceManager.h"
-#include "nsIServiceManagerUtils.h"
+#include "nsServiceManagerUtils.h"
 #include "nsIComponentManager.h"
 #include "nsComponentManagerUtils.h"
 #include "nsIGenericFactory.h"
 #include "nsIComponentRegistrar.h"
+#include "nsStaticComponents.h"
 
 #ifdef XP_OS2
 #include "private/pprthred.h"
 #endif
 
-#include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 #include "nsILocaleService.h"
-#include "plevent.h"
 #include "prmem.h"
 #include "prenv.h"
 #include "prnetdb.h"
@@ -82,7 +84,6 @@
 #include "nsICategoryManager.h"
 #include "nsIXULWindow.h"
 #include "nsIChromeRegistrySea.h"
-#include "nsIEventQueueService.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsBuildID.h"
 #include "nsIWindowCreator.h"
@@ -103,9 +104,12 @@
 
 // for X remote support
 #ifdef MOZ_ENABLE_XREMOTE
-#include "nsXRemoteClientCID.h"
-#include "nsIXRemoteClient.h"
-#include "nsIXRemoteService.h"
+#ifdef MOZ_WIDGET_PHOTON
+#include "PhRemoteClient.h"
+#else
+#include "XRemoteClient.h"
+#endif
+#include "nsIRemoteService.h"
 #endif
 
 // see DoOnShutdown()
@@ -115,9 +119,13 @@
 #include "nsTraceMalloc.h"
 #endif
 
-#if defined(DEBUG) && defined(XP_WIN32)
+#ifdef XP_WIN32
+#include <windows.h>
+
+#ifdef DEBUG
 #include <malloc.h>
 #endif
+#endif
 
 #if defined (XP_MACOSX)
 #include <Processes.h>
@@ -206,33 +214,9 @@ extern "C" {
 }
 #endif
 
-#ifdef _BUILD_STATIC_BIN
-#include "nsStaticComponent.h"
-
-#ifdef _MOZCOMPS_SHARED_LIBRARY
-extern "C" nsresult nsMetaModule_nsGetModule(nsIComponentManager *servMgr,
-                                             nsIFile *location,
-                                             nsIModule **result);
-
-static nsStaticModuleInfo staticInfo[] = {
-  {
-    "meta component",
-    nsMetaModule_nsGetModule
-  }
-};
-
-static nsresult PR_CALLBACK
-app_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count) {
-  *info = staticInfo;
-  *count = 1;
-  return NS_OK;
-}
-#else
-
-nsresult PR_CALLBACK
-app_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count);
-
-#endif
+#ifndef _BUILD_STATIC_BIN
+nsStaticModuleInfo const *const kPStaticModules = nsnull;
+PRUint32 const kStaticModuleCount = 0;
 #endif
 
 #if defined(XP_UNIX) || defined(XP_BEOS)
@@ -242,81 +226,9 @@ app_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count);
 #if defined(XP_OS2)
 /* Adding globals that OS/2 doesn't have so we can port the DDE code */
 char **__argv;
-int   __argc;
+int   *__pargc;
 #endif /* XP_OS2 */
 
-#if defined(XP_BEOS)
-
-#include <AppKit.h>
-#include <AppFileInfo.h>
-
-class nsBeOSApp : public BApplication
-{
-public:
-  nsBeOSApp(sem_id sem)
-  : BApplication(GetAppSig()), init(sem)
-  {
-  }
-
-  void ReadyToRun(void)
-  {
-    release_sem(init);
-  }
-
-  static int32 Main(void *args)
-  {
-    nsBeOSApp *app = new nsBeOSApp((sem_id)args);
-    if (nsnull == app)
-      return B_ERROR;
-    return app->Run();
-  }
-
-private:
-  char *GetAppSig(void)
-  {
-    app_info appInfo;
-    BFile file;
-    BAppFileInfo appFileInfo;
-    image_info info;
-    int32 cookie = 0;
-    static char sig[B_MIME_TYPE_LENGTH];
-
-    sig[0] = 0;
-    if (get_next_image_info(0, &cookie, &info) != B_OK ||
-        file.SetTo(info.name, B_READ_ONLY) != B_OK ||
-        appFileInfo.SetTo(&file) != B_OK ||
-        appFileInfo.GetSignature(sig) != B_OK)
-    {
-      return "application/x-vnd.Mozilla";
-    }
-    return sig;
-  }
-
-  sem_id init;
-};
-
-static nsresult InitializeBeOSApp(void)
-{
-  nsresult rv = NS_OK;
-
-  sem_id initsem = create_sem(0, "beapp init");
-  if (initsem < B_OK)
-    return NS_ERROR_FAILURE;
-
-  thread_id tid = spawn_thread(nsBeOSApp::Main, "BApplication", B_NORMAL_PRIORITY, (void *)initsem);
-  if (tid < B_OK || B_OK != resume_thread(tid))
-    rv = NS_ERROR_FAILURE;
-
-  if (B_OK != acquire_sem(initsem))
-    rv = NS_ERROR_FAILURE;
-  if (B_OK != delete_sem(initsem))
-    rv = NS_ERROR_FAILURE;
-
-  return rv;
-}
-
-#endif // XP_BEOS
-
 #if defined(XP_MAC)
 
 #include "macstdlibextras.h"
@@ -368,6 +280,10 @@ static void InitializeMacOSXApp(int argc, char* argv[])
 
 #endif /* XP_MACOSX */
 
+#if defined(MOZ_WIDGET_QT)
+#include <qapplication.h>
+#endif
+
 #ifdef MOZ_X11
 #include <X11/Xlib.h>
 #endif /* MOZ_X11 */
@@ -381,13 +297,13 @@ static void InitializeMacOSXApp(int argc, char* argv[])
 /*********************************************/
 // Default implemenations for nativeAppSupport
 // If your platform implements these functions if def out this code.
-#if !defined(MOZ_WIDGET_COCOA) && !defined(MOZ_WIDGET_PHOTON) && !defined( XP_WIN) && !defined(XP_OS2) && !defined( XP_BEOS ) && !defined(MOZ_WIDGET_GTK) && !defined(MOZ_WIDGET_GTK2)
+#if !defined(MOZ_WIDGET_COCOA) && !defined(MOZ_WIDGET_PHOTON) && !defined( XP_WIN) && !defined(XP_OS2) && !defined(MOZ_WIDGET_GTK) && !defined(MOZ_WIDGET_GTK2)
 
 nsresult NS_CreateSplashScreen(nsISplashScreen **aResult)
 {
     nsresult rv = NS_OK;
     if (aResult) {
-        *aResult = 0;
+        *aResult = nsnull;
     } else {
         rv = NS_ERROR_NULL_POINTER;
     }
@@ -416,13 +332,13 @@ PRBool NS_CanRun()
 //       nsISplashScreen will be removed.
 //
 
-#if !defined(XP_WIN) && !defined(XP_OS2) && !defined(MOZ_WIDGET_GTK) && !defined(MOZ_WIDGET_GTK2) && !defined(XP_MAC) && (!defined(XP_MACOSX) || defined(MOZ_WIDGET_COCOA))
+#if !defined(XP_WIN) && !defined(XP_OS2)&& !defined( XP_BEOS ) && !defined(MOZ_WIDGET_GTK) && !defined(MOZ_WIDGET_GTK2) && !defined(XP_MAC) && (!defined(XP_MACOSX) || defined(MOZ_WIDGET_COCOA))
 
 nsresult NS_CreateNativeAppSupport(nsINativeAppSupport **aResult)
 {
     nsresult rv = NS_OK;
     if (aResult) {
-        *aResult = 0;
+        *aResult = nsnull;
     } else {
         rv = NS_ERROR_NULL_POINTER;
     }
@@ -491,7 +407,7 @@ static nsresult OpenWindow(const nsCString& aChromeURL,
 
 #ifdef DEBUG_CMD_LINE
   printf("OpenWindow(%s, %s, %d, %d)\n", aChromeURL.get(),
-                                         NS_ConvertUCS2toUTF8(aAppArgs).get(),
+                                         NS_ConvertUTF16toUTF8(aAppArgs).get(),
                                          aWidth, aHeight);
 #endif /* DEBUG_CMD_LINE */
 
@@ -530,11 +446,11 @@ static nsresult OpenWindow(const nsCString& aChromeURL,
   nsCAutoString features("chrome,dialog=no,all");
   if (aHeight != nsIAppShellService::SIZE_TO_CONTENT) {
     features.Append(",height=");
-    features.AppendInt(aHeight);
+    AppendIntToString(features, aHeight);
   }
   if (aWidth != nsIAppShellService::SIZE_TO_CONTENT) {
     features.Append(",width=");
-    features.AppendInt(aWidth);
+    AppendIntToString(features, aWidth);
   }
 
 #ifdef DEBUG_CMD_LINE
@@ -568,13 +484,13 @@ static void DumpArbitraryHelp()
         rv = catman->GetCategoryEntry(COMMAND_LINE_ARGUMENT_HANDLERS,
                                       entryString.get(),
                                       getter_Copies(contractidString));
-        if (NS_FAILED(rv) || !((const char *)contractidString)) break;
+        if (NS_FAILED(rv) || contractidString.IsEmpty()) break;
 
 #ifdef DEBUG_CMD_LINE
-        printf("cmd line handler contractid = %s\n", (const char *)contractidString);
+        printf("cmd line handler contractid = %s\n", contractidString.get());
 #endif /* DEBUG_CMD_LINE */
 
-        nsCOMPtr <nsICmdLineHandler> handler(do_GetService((const char *)contractidString, &rv));
+        nsCOMPtr <nsICmdLineHandler> handler(do_GetService(contractidString.get(), &rv));
 
         if (handler) {
           nsXPIDLCString commandLineArg;
@@ -585,16 +501,16 @@ static void DumpArbitraryHelp()
           rv = handler->GetHelpText(getter_Copies(helpText));
           if (NS_FAILED(rv)) continue;
 
-          if ((const char *)commandLineArg) {
-            printf("%s%s", HELP_SPACER_1,(const char *)commandLineArg);
+          if (!commandLineArg.IsEmpty()) {
+            printf("%s%s", HELP_SPACER_1, commandLineArg.get());
 
             PRBool handlesArgs = PR_FALSE;
             rv = handler->GetHandlesArgs(&handlesArgs);
             if (NS_SUCCEEDED(rv) && handlesArgs) {
               printf(" <url>");
             }
-            if ((const char *)helpText) {
-              printf("%s%s\n",HELP_SPACER_2,(const char *)helpText);
+            if (!helpText.IsEmpty()) {
+              printf("%s%s\n", HELP_SPACER_2, helpText.get());
             }
           }
         }
@@ -618,12 +534,8 @@ LaunchApplicationWithArgs(const char *commandLineArg,
 
   nsresult rv;
 
-  nsCOMPtr<nsICmdLineService> cmdLine =
-    do_GetService(NS_COMMANDLINESERVICE_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
-
   nsCOMPtr <nsICmdLineHandler> handler;
-  rv = cmdLine->GetHandlerForParam(aParam, getter_AddRefs(handler));
+  rv = cmdLineArgs->GetHandlerForParam(aParam, getter_AddRefs(handler));
   if (NS_FAILED(rv)) return rv;
 
   if (!handler) return NS_ERROR_FAILURE;
@@ -633,27 +545,28 @@ LaunchApplicationWithArgs(const char *commandLineArg,
   if (NS_FAILED(rv)) return rv;
 
 #ifdef DEBUG_CMD_LINE
-  printf("XXX got this one:\t%s\n\t%s\n\n",commandLineArg,(const char *)chromeUrlForTask);
+  printf("XXX got this one:\t%s\n\t%s\n\n",commandLineArg,chromeUrlForTask.get());
 #endif /* DEBUG_CMD_LINE */
 
   nsXPIDLCString cmdResult;
   rv = cmdLineArgs->GetCmdLineValue(commandLineArg, getter_Copies(cmdResult));
   if (NS_FAILED(rv)) return rv;
 #ifdef DEBUG_CMD_LINE
-  printf("%s, cmdResult = %s\n",commandLineArg,(const char *)cmdResult);
+  printf("%s, cmdResult = %s\n",commandLineArg,cmdResult.get());
 #endif /* DEBUG_CMD_LINE */
 
   PRBool handlesArgs = PR_FALSE;
   rv = handler->GetHandlesArgs(&handlesArgs);
   if (handlesArgs) {
-    if ((const char *)cmdResult) {
-      if (PL_strcmp("1",(const char *)cmdResult)) {
+    if (!cmdResult.IsEmpty()) {
+      if (strcmp("1", cmdResult.get())) {
         PRBool openWindowWithArgs = PR_TRUE;
         rv = handler->GetOpenWindowWithArgs(&openWindowWithArgs);
         if (NS_FAILED(rv)) return rv;
 
         if (openWindowWithArgs) {
-          nsAutoString cmdArgs; cmdArgs.AssignWithConversion(cmdResult);
+          nsAutoString cmdArgs;
+          NS_CopyNativeToUnicode(cmdResult, cmdArgs);
 #ifdef DEBUG_CMD_LINE
           printf("opening %s with %s\n", chromeUrlForTask.get(), "OpenWindow");
 #endif /* DEBUG_CMD_LINE */
@@ -684,8 +597,8 @@ LaunchApplicationWithArgs(const char *commandLineArg,
     }
   }
   else {
-    if (NS_SUCCEEDED(rv) && (const char*)cmdResult) {
-      if (PL_strcmp("1",cmdResult) == 0) {
+    if (NS_SUCCEEDED(rv) && !cmdResult.IsEmpty()) {
+      if (strcmp("1", cmdResult.get()) == 0) {
         rv = OpenWindow(chromeUrlForTask, width, height);
         if (NS_FAILED(rv)) return rv;
       }
@@ -723,7 +636,7 @@ static PRBool IsStartupCommand(const char *arg)
 
 
 // This should be done by app shell enumeration someday
-nsresult DoCommandLines(nsICmdLineService* cmdLineArgs, PRBool heedGeneralStartupPrefs, PRBool *windowOpened)
+nsresult DoCommandLines(nsICmdLineService* cmdLineArgs, PRBool *windowOpened)
 {
   NS_ENSURE_ARG(windowOpened);
   *windowOpened = PR_FALSE;
@@ -744,21 +657,18 @@ nsresult DoCommandLines(nsICmdLineService* cmdLineArgs, PRBool heedGeneralStartu
 	if (NS_SUCCEEDED(rv) && !tempString.IsEmpty())
     PR_sscanf(tempString.get(), "%d", &height);
   
-  if (heedGeneralStartupPrefs) {
-    nsCOMPtr<nsIAppStartup> appStartup(do_GetService(NS_APPSTARTUP_CONTRACTID, &rv));
-    if (NS_FAILED(rv)) return rv;
-    rv = appStartup->CreateStartupState(width, height, windowOpened);
-    if (NS_FAILED(rv)) return rv;
-  }
-  else {
     PRInt32 argc = 0;
     rv = cmdLineArgs->GetArgc(&argc);
-    if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) 
+    return rv;
 
     char **argv = nsnull;
     rv = cmdLineArgs->GetArgv(&argv);
-    if (NS_FAILED(rv)) return rv;
+  if (NS_FAILED(rv)) 
+    return rv;
 
+  // first apply command line args
+  // if there is any window opened, startup preferences are ignored
     PRInt32 i = 0;
     for (i=1;i<argc;i++) {
 #ifdef DEBUG_CMD_LINE
@@ -783,6 +693,37 @@ nsresult DoCommandLines(nsICmdLineService* cmdLineArgs, PRBool heedGeneralStartu
           return rv;
       }
     }
+   // second if an URL is given in the command line
+   // then open a browser window with the URL
+  nsXPIDLCString urlToLoad;
+  rv = cmdLineArgs->GetURLToLoad(getter_Copies(urlToLoad));
+  if (NS_SUCCEEDED(rv) && !urlToLoad.IsEmpty()) {
+    nsCOMPtr<nsICmdLineHandler> handler(
+      do_GetService("@mozilla.org/commandlinehandler/general-startup;1?type=browser"));
+
+    nsXPIDLCString chromeUrlForTask;
+    rv = handler->GetChromeUrlForTask(getter_Copies(chromeUrlForTask));
+
+    if (NS_SUCCEEDED(rv)) {
+      // convert the cmdLine URL to Unicode
+      nsAutoString url;
+      NS_CopyNativeToUnicode(urlToLoad, url);
+      rv = OpenWindow(chromeUrlForTask, url, width, height);
+    }
+
+    if (NS_SUCCEEDED(rv)) {
+      *windowOpened = PR_TRUE;
+    }
+  }
+
+  // third if no window is opened then apply the startup preferences
+  if (!*windowOpened) {
+    nsCOMPtr<nsIAppStartup> appStartup(do_GetService(NS_APPSTARTUP_CONTRACTID, &rv));
+    if (NS_FAILED(rv))
+      return rv;
+    rv = appStartup->CreateStartupState(width, height, windowOpened);
+    if (NS_FAILED(rv))
+      return rv;
   }
   return NS_OK;
 }
@@ -794,7 +735,7 @@ static nsresult DoOnShutdown()
   // save the prefs, in case they weren't saved
   {
     // scoping this in a block to force release
-    nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
+    nsCOMPtr<nsIPrefService> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get prefs, so unable to save them");
     if (NS_SUCCEEDED(rv))
       prefs->SavePrefFile(nsnull);
@@ -819,7 +760,7 @@ static char kMatchOSLocalePref[] = "intl.locale.matchOS";
 nsresult
 getCountry(const nsAString& lc_name, nsAString& aCountry)
 {
-#ifdef XPCOM_GLUE
+#ifndef MOZILLA_INTERNAL_API
   const PRUnichar *begin = lc_name.BeginReading();
   const PRUnichar *end   = lc_name.EndReading();
   while (begin != end) {
@@ -862,7 +803,7 @@ static nsresult InstallGlobalLocale(nsICmdLineService *cmdLineArgs)
     nsresult rv = NS_OK;
 
     // check the pref first
-    nsCOMPtr<nsIPref> prefService(do_GetService(NS_PREF_CONTRACTID));
+    nsCOMPtr<nsIPrefBranch> prefService(do_GetService(NS_PREFSERVICE_CONTRACTID));
     PRBool matchOS = PR_FALSE;
     if (prefService)
       prefService->GetBoolPref(kMatchOSLocalePref, &matchOS);
@@ -878,38 +819,36 @@ static nsresult InstallGlobalLocale(nsICmdLineService *cmdLineArgs)
     nsXPIDLCString cmdUI;
     rv = cmdLineArgs->GetCmdLineValue(UILOCALE_CMD_LINE_ARG, getter_Copies(cmdUI));
     if (NS_SUCCEEDED(rv)){
-        if (cmdUI) {
-            nsCAutoString UILocaleName(cmdUI);
+        if (!cmdUI.IsEmpty()) {
             nsCOMPtr<nsIChromeRegistrySea> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
             if (chromeRegistry)
-                rv = chromeRegistry->SelectLocale(UILocaleName, PR_FALSE);
+                rv = chromeRegistry->SelectLocale(cmdUI, PR_FALSE);
         }
     }
     // match OS when no cmdline override
-    if (!cmdUI && matchOS) {
+    if (cmdUI.IsEmpty() && matchOS) {
       nsCOMPtr<nsIChromeRegistrySea> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
       if (chromeRegistry) {
         chromeRegistry->SetRuntimeProvider(PR_TRUE);
-        rv = chromeRegistry->SelectLocale(NS_ConvertUCS2toUTF8(uiLang), PR_FALSE);
+        rv = chromeRegistry->SelectLocale(NS_ConvertUTF16toUTF8(uiLang), PR_FALSE);
       }
     }
 
     nsXPIDLCString cmdContent;
     rv = cmdLineArgs->GetCmdLineValue(CONTENTLOCALE_CMD_LINE_ARG, getter_Copies(cmdContent));
     if (NS_SUCCEEDED(rv)){
-        if (cmdContent) {
-            nsCAutoString contentLocaleName(cmdContent);
+        if (!cmdContent.IsEmpty()) {
             nsCOMPtr<nsIChromeRegistrySea> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
             if(chromeRegistry)
-                rv = chromeRegistry->SelectLocale(contentLocaleName, PR_FALSE);
+                rv = chromeRegistry->SelectLocale(cmdContent, PR_FALSE);
         }
     }
     // match OS when no cmdline override
-    if (!cmdContent && matchOS) {
+    if (cmdContent.IsEmpty() && matchOS) {
       nsCOMPtr<nsIChromeRegistrySea> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
       if (chromeRegistry) {
         chromeRegistry->SetRuntimeProvider(PR_TRUE);        
-        rv = chromeRegistry->SelectLocale(NS_ConvertUCS2toUTF8(country), PR_FALSE);
+        rv = chromeRegistry->SelectLocale(NS_ConvertUTF16toUTF8(country), PR_FALSE);
       }
     }
 
@@ -949,7 +888,7 @@ static nsresult InitializeProfileService(nsICmdLineService *cmdLineArgs)
     if (shouldShowUI) {
       nsXPIDLCString arg;
       if (NS_SUCCEEDED(cmdLineArgs->GetCmdLineValue("-silent", getter_Copies(arg)))) {
-        if ((const char*)arg) {
+        if (!arg.IsEmpty()) {
           shouldShowUI = PR_FALSE;
         }
       }
@@ -982,7 +921,7 @@ static void ShowOSAlertFromFile(int argc, char **argv, const char *alert_filenam
     if (NS_SUCCEEDED(rv) && fileName) {
       fileName->AppendNative(NS_LITERAL_CSTRING("res"));
       fileName->AppendNative(nsDependentCString(alert_filename));
-      PRFileDesc* fd = 0;
+      PRFileDesc* fd = nsnull;
       fileName->OpenNSPRFileDesc(PR_RDONLY, 0664, &fd);
       if (fd) {
         numRead = PR_Read(fd, message, sizeof(message)-1);
@@ -1093,14 +1032,6 @@ static nsresult main1(int argc, char* argv[], nsISupports *nativeApp )
   fpsetmask(0);
 #endif
 
-  NS_TIMELINE_ENTER("init event service");
-  nsCOMPtr<nsIEventQueueService> eventQService(do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID, &rv));
-  if (NS_SUCCEEDED(rv)) {
-    // XXX: What if this fails?
-    rv = eventQService->CreateThreadEventQueue();
-  }
-  NS_TIMELINE_LEAVE("init event service");
-
   // Setup an autoreg obserer, so that we can update a progress
   // string in the splash screen
   nsCOMPtr<nsIObserverService> obsService(do_GetService("@mozilla.org/observer-service;1"));
@@ -1200,7 +1131,7 @@ static nsresult main1(int argc, char* argv[], nsISupports *nativeApp )
   NS_TIMELINE_ENTER("appStartup->Initialize");
 
   // Create the Application Shell instance...
-  rv = appStartup->Initialize(cmdLineArgs, nativeAppOwner);
+  rv = appStartup->Initialize(nativeAppOwner);
 
   NS_TIMELINE_LEAVE("appStartup->Initialize");
 
@@ -1238,35 +1169,9 @@ static nsresult main1(int argc, char* argv[], nsISupports *nativeApp )
   NS_TIMELINE_LEAVE("appStartup->CreateHiddenWindow");
 
   // This will go away once Components are handling there own commandlines
-  // if we have no command line arguments, we need to heed the
-  // "general.startup.*" prefs
-  // if we had no command line arguments, argc == 1.
 
   PRBool windowOpened = PR_FALSE;
-  PRBool defaultStartup;
-#if defined(XP_MAC) || defined(XP_MACOSX)
-  // On Mac, nsCommandLineServiceMac may have added synthetic
-  // args. Check this adjusted value instead of the raw value.
-  PRInt32 processedArgc;
-  cmdLineArgs->GetArgc(&processedArgc);
-  defaultStartup = (processedArgc == 1);
-#if defined(XP_MACOSX)
-  // On OSX, we get passed two args if double-clicked from the Finder.
-  // The second is our PSN. Check for this and consider it to be default.
-  if (argc == 2 && processedArgc == 2) {
-    ProcessSerialNumber ourPSN;
-    if (::MacGetCurrentProcess(&ourPSN) == noErr) {
-      char argBuf[64];
-      sprintf(argBuf, "-psn_%ld_%ld", ourPSN.highLongOfPSN, ourPSN.lowLongOfPSN);
-      if (!strcmp(argBuf, argv[1]))
-        defaultStartup = PR_TRUE;
-    }
-  }
-#endif /* XP_MACOSX */
-#else
-  defaultStartup = (argc == 1);
-#endif
-  rv = DoCommandLines(cmdLineArgs, defaultStartup, &windowOpened);
+  rv = DoCommandLines(cmdLineArgs, &windowOpened);
   if (NS_FAILED(rv))
   {
     NS_WARNING("failed to process command line");
@@ -1275,7 +1180,7 @@ static nsresult main1(int argc, char* argv[], nsISupports *nativeApp )
   
 	if (obsService)
   {
-    nsAutoString userMessage; userMessage.AssignLiteral("Creating first window...");
+    NS_NAMED_LITERAL_STRING(userMessage, "Creating first window...");
     obsService->NotifyObservers(nsnull, "startup_user_notifcations", userMessage.get());
   }
   
@@ -1295,10 +1200,23 @@ static nsresult main1(int argc, char* argv[], nsISupports *nativeApp )
   // if we have X remote support and we have our one window up and
   // running start listening for requests on the proxy window.
   // It will shut itself down before the event queue stops processing events.
-  nsCOMPtr<nsIXRemoteService> remoteService;
-  remoteService = do_GetService(NS_IXREMOTESERVICE_CONTRACTID);
-  if (remoteService)
-    remoteService->Startup(MOZ_APP_NAME);
+  nsCOMPtr<nsIRemoteService> remoteService
+    (do_GetService("@mozilla.org/toolkit/remote-service;1"));
+  NS_ASSERTION(remoteService, "Couldn't create remote service?");
+  if (remoteService) {
+    nsCAutoString pname;
+
+    nsCOMPtr<nsIProfile> pm (do_GetService(NS_PROFILE_CONTRACTID));
+    if (pm) {
+      nsXPIDLString name;
+      pm->GetCurrentProfile(getter_Copies(name));
+      if (name) {
+        CopyUTF16toUTF8(name, pname);
+      }
+    }
+
+    remoteService->Startup(NS_STRINGIFY(MOZ_APP_NAME), pname.IsEmpty() ? nsnull : pname.get());
+  }
 #endif /* MOZ_ENABLE_XREMOTE */
 
   // remove the nativeApp as an XPCOM autoreg observer
@@ -1379,7 +1297,7 @@ static void DumpHelp(char *appname)
 #ifdef XP_UNIX
   printf("%s--g-fatal-warnings%sMake all warnings fatal\n", HELP_SPACER_1, HELP_SPACER_2);
 
-  printf("\nMozilla options\n");
+  printf("\n%s options\n", NS_STRINGIFY(MOZ_APP_DISPLAYNAME));
 #endif
 
   printf("%s-height <value>%sSet height of startup window to <value>.\n",HELP_SPACER_1,HELP_SPACER_2);
@@ -1394,14 +1312,14 @@ static void DumpHelp(char *appname)
   printf("%s-SelectProfile%sStart with profile selection dialog.\n",HELP_SPACER_1,HELP_SPACER_2);
   printf("%s-UILocale <locale>%sStart with <locale> resources as UI Locale.\n",HELP_SPACER_1,HELP_SPACER_2);
   printf("%s-contentLocale <locale>%sStart with <locale> resources as content Locale.\n",HELP_SPACER_1,HELP_SPACER_2);
-#ifdef XP_WIN32
-  printf("%s-console%sStart Mozilla with a debugging console.\n",HELP_SPACER_1,HELP_SPACER_2);
+#if defined(XP_WIN32) || defined(XP_OS2)
+  printf("%s-console%sStart %s with a debugging console.\n",HELP_SPACER_1,HELP_SPACER_2,NS_STRINGIFY(MOZ_APP_DISPLAYNAME));
 #endif
 #ifdef MOZ_ENABLE_XREMOTE
   printf("%s-remote <command>%sExecute <command> in an already running\n"
-         "%sMozilla process.  For more info, see:\n"
+         "%s%s process.  For more info, see:\n"
          "\n%shttp://www.mozilla.org/unix/remote.html\n\n",
-         HELP_SPACER_1,HELP_SPACER_1,HELP_SPACER_4,HELP_SPACER_2);
+         HELP_SPACER_1,HELP_SPACER_1,HELP_SPACER_4,NS_STRINGIFY(MOZ_APP_DISPLAYNAME),HELP_SPACER_2);
   printf("%s-splash%sEnable splash screen.\n",HELP_SPACER_1,HELP_SPACER_2);
 #else
   printf("%s-nosplash%sDisable splash screen.\n",HELP_SPACER_1,HELP_SPACER_2);
@@ -1422,7 +1340,7 @@ static nsresult DumpVersion(char *appname)
   nsresult rv = NS_OK;
   long buildID = NS_BUILD_ID;  // 10-digit number
 
-  printf("Mozilla %s, Copyright (c) 2003-2004 mozilla.org", MOZILLA_VERSION);
+  printf("%s %s, Copyright (c) 2003-2006 mozilla.org", NS_STRINGIFY(MOZ_APP_DISPLAYNAME), NS_STRINGIFY(MOZ_APP_VERSION));
 
   if(buildID) {
     printf(", build %u\n", (unsigned int)buildID);
@@ -1438,14 +1356,12 @@ static nsresult DumpVersion(char *appname)
 // from main - just to keep types consistent
 static int HandleRemoteArguments(int argc, char* argv[], PRBool *aArgUsed)
 {
-  int i = 0;
+  const char *profile = nsnull;
+  const char *program = nsnull;
+  const char *remote = nsnull;
+  const char *username = nsnull;
 
-  const char *remote = 0;
-  const char *profile = 0;
-  const char *program = 0;
-  const char *username = 0;
-
-  for (i=1; i < argc; i++) {
+  for (int i = 1; i < argc; ++i) {
     if (PL_strcasecmp(argv[i], "-remote") == 0) {
       // someone used a -remote flag
       *aArgUsed = PR_TRUE;
@@ -1491,13 +1407,11 @@ static int HandleRemoteArguments(int argc, char* argv[], PRBool *aArgUsed)
     return 0; // No remote argument == success
 
   // try to get the X remote client
-  nsCOMPtr<nsIXRemoteClient> client (do_CreateInstance(NS_XREMOTECLIENT_CONTRACTID));
-  if (!client)
-    return 1;
+  XRemoteClient client;
 
   nsresult rv;
   // try to init - connects to the X server and stuff
-  rv = client->Init();
+  rv = client.Init();
   if (NS_FAILED(rv)) {
     PR_fprintf(PR_STDERR, "Error: Failed to connect to X server.\n");
     return 1;
@@ -1510,13 +1424,13 @@ static int HandleRemoteArguments(int argc, char* argv[], PRBool *aArgUsed)
 
   // Same with the program name
   if (!program) {
-    program = MOZ_APP_NAME;
+    program = NS_STRINGIFY(MOZ_APP_NAME);
   }
 
   char *response = NULL;
   PRBool success = PR_FALSE;
-  rv = client->SendCommand(program, username, profile, remote,
-                           &response, &success);
+  rv = client.SendCommand(program, username, profile, remote,
+                          &response, &success);
 
   // did the command fail?
   if (NS_FAILED(rv)) {
@@ -1538,7 +1452,6 @@ static int HandleRemoteArguments(int argc, char* argv[], PRBool *aArgUsed)
     return 2;
   }
 
-  client->Shutdown();
   // success
   return 0;
 }
@@ -1609,55 +1522,70 @@ static PRBool GetWantSplashScreen(int argc, char* argv[])
   return dosplash;
 }
 
+#if defined(XP_OS2)
+// because we use early returns, we use a stack-based helper to un-set the OS2 FP handler
+class ScopedFPHandler {
+private:
+  EXCEPTIONREGISTRATIONRECORD excpreg;
+
+public:
+  ScopedFPHandler() { PR_OS2_SetFloatExcpHandler(&excpreg); }
+  ~ScopedFPHandler() { PR_OS2_UnsetFloatExcpHandler(&excpreg); }
+};
+#endif
+
 int main(int argc, char* argv[])
 {
   NS_TIMELINE_MARK("enter main");
-  int i; //Moved here due to portability guideline 20. See bug 258055
 
-#if defined(DEBUG) && defined(XP_WIN32)
+#ifdef XP_WIN32
+  // Suppress the "DLL Foo could not be found" dialog, such that if dependent
+  // libraries (such as GDI+) are not preset, we gracefully fail to load those
+  // XPCOM components, instead of being ungraceful.
+  SetErrorMode(SEM_FAILCRITICALERRORS);
+
+#ifdef DEBUG
   // Disable small heap allocator to get heapwalk() giving us
   // accurate heap numbers. Win2k non-debug does not use small heap allocator.
   // Win2k debug seems to be still using it.
   // http://msdn.microsoft.com/library/default.asp?url=/library/en-us/vclib/html/_crt__set_sbh_threshold.asp
   _set_sbh_threshold(0);
 #endif
+#endif
 
 #if defined(XP_UNIX) || defined(XP_BEOS)
   InstallUnixSignalHandlers(argv[0]);
 #endif
 
+#ifdef MOZ_ACCESSIBILITY_ATK
+  // Reset GTK_MODULES, strip atk-bridge if exists
+  // Mozilla will load libatk-bridge.so later if necessary
+  const char* gtkModules = PR_GetEnv("GTK_MODULES");
+  if (gtkModules && *gtkModules) {
+    nsCString gtkModulesStr(gtkModules);
+    gtkModulesStr.ReplaceSubstring("atk-bridge", "");
+    char* expr = PR_smprintf("GTK_MODULES=%s", gtkModulesStr.get());
+    if (expr)
+      PR_SetEnv(expr);
+    // We intentionally leak |expr| here since it is required by PR_SetEnv.
+  }
+#endif
+
 #if defined(XP_OS2)
-  __argc = argc;
+  __pargc = &argc;
   __argv = argv;
 
-  ULONG    ulMaxFH = 0;
-  LONG     ulReqCount = 0;
-
-  DosSetRelMaxFH(&ulReqCount,
-                 &ulMaxFH);
-
-  if (ulMaxFH < 256) {
-    DosSetMaxFH(256);
-  }
-
-  EXCEPTIONREGISTRATIONRECORD excpreg;
-  PR_OS2_SetFloatExcpHandler(&excpreg);
-#endif /* XP_OS2 */
-
-#if defined(XP_BEOS)
-  if (NS_OK != InitializeBeOSApp())
+  PRBool StartOS2App(int aArgc, char **aArgv);
+  if (!StartOS2App(argc, argv))
     return 1;
-#endif
+
+  ScopedFPHandler handler;
+#endif /* XP_OS2 */
 
 #if defined(XP_MACOSX)
   InitializeMacOSXApp(argc, argv);
 #endif
 
-#ifdef _BUILD_STATIC_BIN
-  // Initialize XPCOM's module info table
-  NSGetStaticModuleInfo = app_getModuleInfo;
-#endif
-
   // Handle -help and -version command line arguments.
   // They should% return quick, so we deal with them here.
   if (HandleDumpArguments(argc, argv))
@@ -1667,22 +1595,26 @@ int main(int argc, char* argv[])
   argc = NS_TraceMallocStartupArgs(argc, argv);
 #endif
 
+#if defined(MOZ_WIDGET_GTK) || defined(MOZ_WIDGET_GTK2) || defined(MOZ_X11)
+  int i;
+#endif
+
 #ifdef MOZ_X11
   /* Init threadsafe mode of Xlib API on demand
    * (currently this is only for testing, future builds may use this by
    * default) */
-  PRBool x11threadsafe = PR_FALSE;
-  for (i=1; i<argc; i++) {
-    if (PL_strcmp(argv[i], "-xinitthreads") == 0) {
-      x11threadsafe = PR_TRUE;
-      break;
-    }
-  }
-
-  if (PR_GetEnv("MOZILLA_X11_XINITTHREADS")) {
+  PRBool x11threadsafe;
+  const char *xInitThreads = PR_GetEnv("MOZILLA_X11_XINITTHREADS");
+  if (xInitThreads && *xInitThreads)
     x11threadsafe = PR_TRUE;
+  else {
+    x11threadsafe = PR_FALSE;
+    for (i = 1; i < argc; ++i)
+      if (PL_strcmp(argv[i], "-xinitthreads") == 0) {
+        x11threadsafe = PR_TRUE;
+        break;
+      }
   }
-  
   if (x11threadsafe) {
     if (XInitThreads() == False) {
       fprintf(stderr, "%s: XInitThreads failure.", argv[0]);
@@ -1712,6 +1644,15 @@ int main(int argc, char* argv[])
   gtk_widget_set_default_visual(gdk_rgb_get_visual());
   gtk_widget_set_default_colormap(gdk_rgb_get_cmap());
 #endif /* MOZ_WIDGET_GTK || MOZ_WIDGET_GTK2 */
+
+#if defined(MOZ_WIDGET_QT)
+  QApplication qapp(argc, argv);
+#endif
+
+// #if defined(MOZ_WIDGET_XLIB)
+// XXXtimeless fix me! Because we don't have external shared libs holding global
+// refs, there isn't an easy way to get a Display from here to the widget component.
+// #endif
     
   // Call the code to install our handler
 #ifdef MOZ_JPROF
@@ -1729,12 +1670,13 @@ int main(int argc, char* argv[])
     return 1;
   }
 #else
-  NS_TIMELINE_MARK("NS_InitXPCOM2...");
-  nsresult rv = NS_InitXPCOM2(nsnull, nsnull, nsnull);
-  NS_TIMELINE_MARK("...NS_InitXPCOM2 done");
+  NS_TIMELINE_MARK("NS_InitXPCOM3...");
+  nsresult rv = NS_InitXPCOM3(nsnull, nsnull, nsnull,
+                              kPStaticModules, kStaticModuleCount);
+  NS_TIMELINE_MARK("...NS_InitXPCOM3 done");
   if (NS_FAILED(rv)) {
     // We should be displaying a dialog here with the reason why we failed.
-    NS_WARNING("NS_InitXPCOM2 failed");
+    NS_WARNING("NS_InitXPCOM3 failed");
     return 1;
   }
 #endif
@@ -1743,7 +1685,7 @@ int main(int argc, char* argv[])
   // Try to allocate "native app support."
   // Note: this object is not released here.  It is passed to main1 which
   //       has responsibility to release it.
-  nsINativeAppSupport *nativeApp = 0;
+  nsINativeAppSupport *nativeApp = nsnull;
   rv = NS_CreateNativeAppSupport(&nativeApp);
 
   // See if we can run.
@@ -1762,7 +1704,7 @@ int main(int argc, char* argv[])
   }
   // Note: this object is not released here.  It is passed to main1 which
   //       has responsibility to release it.
-  nsISplashScreen *splash = 0;
+  nsISplashScreen *splash = nsnull;
   PRBool dosplash = GetWantSplashScreen(argc, argv);
 
   if (dosplash && !nativeApp) {
@@ -1811,10 +1753,6 @@ int main(int argc, char* argv[])
   NS_ASSERTION(NS_SUCCEEDED(rv), "NS_ShutdownXPCOM failed");
 #endif
 
-#ifdef XP_OS2
-  PR_OS2_UnsetFloatExcpHandler(&excpreg);
-#endif
-
   return TranslateReturnValue(mainResult);
 }
 
