@@ -49,14 +49,13 @@
 |   longer term, this code will be restructured to make it more reusable.      |
 ------------------------------------------------------------------------------*/
 
-var gShowBiDi = false;
-
 function nsContextMenu( xulMenu ) {
     this.target         = null;
     this.menu           = null;
     this.popupURL       = null;
     this.onTextInput    = false;
     this.onImage        = false;
+    this.onLoadedImage  = false;
     this.onLink         = false;
     this.onMailtoLink   = false;
     this.onSaveableLink = false;
@@ -68,6 +67,7 @@ function nsContextMenu( xulMenu ) {
     this.isTextSelected = false;
     this.inDirList      = false;
     this.shouldDisplay  = true;
+    this.autoDownload   = false;
 
     // Initialize new menu.
     this.initMenu( xulMenu );
@@ -80,14 +80,14 @@ nsContextMenu.prototype = {
     },
     // Initialize context menu.
     initMenu : function ( popup ) {
+        // Save menu.
+        this.menu = popup;
+
         const xulNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
         if ( document.popupNode.namespaceURI == xulNS ) {
           this.shouldDisplay = false;
           return;
         }
-        // Save menu.
-        this.menu = popup;
-
         // Get contextual info.
         this.setTarget( document.popupNode );
         
@@ -136,17 +136,23 @@ nsContextMenu.prototype = {
         //this.setItemAttrFromNode( "context-stop", "disabled", "canStop" );
     },
     initSaveItems : function () {
-        this.showItem( "context-savepage", 
-                       !( this.inDirList || this.isTextSelected || this.onTextInput || this.onStandaloneImage ||
-                         (this.onLink && this.onImage)));
+        var showSave = !( this.inDirList || this.isTextSelected || this.onTextInput || this.onStandaloneImage ||
+                       ( this.onLink && this.onImage ) );
+        if (showSave)
+          goSetMenuValue( "context-savepage", this.autoDownload ? "valueSave" : "valueSaveAs" );
+        this.showItem( "context-savepage", showSave );
 
         // Save link depends on whether we're in a link.
+        if (this.onSaveableLink)
+          goSetMenuValue( "context-savelink", this.autoDownload ? "valueSave" : "valueSaveAs" );
         this.showItem( "context-savelink", this.onSaveableLink );
 
-        // Save image depends on whether there is one.
-        this.showItem( "context-saveimage", this.onImage || this.onStandaloneImage);
-        
-        this.showItem( "context-sendimage", this.onImage || this.onStandaloneImage);
+        // Save/Send image depends on whether there is one.
+        showSave = this.onLoadedImage || this.onStandaloneImage;
+        if (showSave)
+          goSetMenuValue( "context-saveimage", this.autoDownload ? "valueSave" : "valueSaveAs" );
+        this.showItem( "context-saveimage", showSave );
+        this.showItem( "context-sendimage", showSave );
     },
     initViewItems : function () {
         // View source is always OK, unless in directory listing.
@@ -161,11 +167,11 @@ nsContextMenu.prototype = {
         this.showItem( "context-sep-properties", !( this.inDirList || this.isTextSelected || this.onTextInput ) );
         // Set As Wallpaper depends on whether an image was clicked on, and only works on Windows.
         var isWin = navigator.appVersion.indexOf("Windows") != -1;
-        this.showItem( "context-setWallpaper", isWin && (this.onImage || this.onStandaloneImage));
+        this.showItem( "context-setWallpaper", isWin && (this.onLoadedImage || this.onStandaloneImage));
 
-        this.showItem( "context-sep-image", this.onImage || this.onStandaloneImage);
+        this.showItem( "context-sep-image", this.onLoadedImage || this.onStandaloneImage);
 
-        if( isWin && this.onImage )
+        if( isWin && this.onLoadedImage )
             // Disable the Set As Wallpaper menu item if we're still trying to load the image
           this.setItemAttr( "context-setWallpaper", "disabled", (("complete" in this.target) && !this.target.complete) ? "true" : null );
 
@@ -175,6 +181,8 @@ nsContextMenu.prototype = {
           this.setItemAttr( "context-fitimage", "checked", content.document.imageIsResized ? "true" : null);
         }
 
+        this.showItem( "context-reloadimage", this.onImage);
+
         // View Image depends on whether an image was clicked on.
         this.showItem( "context-viewimage", this.onImage && !this.onStandaloneImage);
 
@@ -190,6 +198,8 @@ nsContextMenu.prototype = {
         this.showItem( "context-searchselect", this.isTextSelected && !this.onTextInput );
         this.showItem( "frame", this.inFrame );
         this.showItem( "frame-sep", this.inFrame );
+        if (this.inFrame)
+          goSetMenuValue( "saveframeas", this.autoDownload ? "valueSave" : "valueSaveAs" );
         var blocking = true;
         if (this.popupURL)
           try {
@@ -256,6 +266,7 @@ nsContextMenu.prototype = {
     setTarget : function ( node ) {
         // Initialize contextual info.
         this.onImage    = false;
+        this.onLoadedImage = false;
         this.onStandaloneImage = false;
         this.onMetaDataItem = false;
         this.onTextInput = false;
@@ -269,22 +280,27 @@ nsContextMenu.prototype = {
         // Remember the node that was clicked.
         this.target = node;
 
+        this.autoDownload = Components.classes["@mozilla.org/preferences-service;1"]
+                                      .getService(Components.interfaces.nsIPrefBranch)
+                                      .getBoolPref("browser.download.autoDownload");
+
         // See if the user clicked on an image.
         if ( this.target.nodeType == Node.ELEMENT_NODE ) {
-             if ( this.target instanceof Components.interfaces.nsIImageLoadingContent &&
-                  this.target.currentURI != null ) {
+            if ( this.target instanceof Components.interfaces.nsIImageLoadingContent && this.target.currentURI  ) {
                 this.onImage = true;
+                var request = this.target.getRequest( Components.interfaces.nsIImageLoadingContent.CURRENT_REQUEST );
+                if (request && (request.imageStatus & request.STATUS_SIZE_AVAILABLE))
+                    this.onLoadedImage = true;
                 this.imageURL = this.target.currentURI.spec;
 
-                var documentType = window.content.document.contentType;
-                if ( documentType.substr(0,6) == "image/" )
-                    this.onStandaloneImage = true;
-             } else if ( this.target.localName.toUpperCase() == "INPUT") {
+                if ( this.target.ownerDocument instanceof ImageDocument )
+                   this.onStandaloneImage = true;
+            } else if ( this.target instanceof HTMLInputElement ) {
                type = this.target.getAttribute("type");
                this.onTextInput = this.isTargetATextBox(this.target);
-            } else if ( this.target.localName.toUpperCase() == "TEXTAREA" ) {
+            } else if ( this.target instanceof HTMLTextAreaElement ) {
                  this.onTextInput = true;
-            } else if ( this.target.localName.toUpperCase() == "HTML" ) {
+            } else if ( this.target instanceof HTMLHtmlElement ) {
                // pages with multiple <body>s are lame. we'll teach them a lesson.
                var bodyElt = this.target.ownerDocument.getElementsByTagName("body")[0];
                if ( bodyElt ) {
@@ -348,16 +364,15 @@ nsContextMenu.prototype = {
         }
         
         // Bubble out, looking for items of interest
+        const XMLNS = "http://www.w3.org/XML/1998/namespace";
         var elem = this.target;
         while ( elem ) {
             if ( elem.nodeType == Node.ELEMENT_NODE ) {
-                var localname = elem.localName.toUpperCase();
-                
                 // Link?
                 if ( !this.onLink && 
-                    ( (localname === "A" && elem.href) ||
-                      localname === "AREA" ||
-                      localname === "LINK" ||
+                    ( (elem instanceof HTMLAnchorElement && elem.href) ||
+                      elem instanceof HTMLAreaElement ||
+                      elem instanceof HTMLLinkElement ||
                       elem.getAttributeNS( "http://www.w3.org/1999/xlink", "type") == "simple" ) ) {
                     // Clicked on a link.
                     this.onLink = true;
@@ -379,14 +394,13 @@ nsContextMenu.prototype = {
                 if ( !this.onMetaDataItem ) {
                     // We currently display metadata on anything which fits
                     // the below test.
-                    if ( ( localname === "BLOCKQUOTE" && 'cite' in elem && elem.cite)  ||
-                         ( localname === "Q" && 'cite' in elem && elem.cite)           ||
-                         ( localname === "TABLE" && 'summary' in elem && elem.summary) ||
-                         ( ( localname === "INS" || localname === "DEL" ) &&
-                           ( ( 'cite' in elem && elem.cite ) ||
-                             ( 'dateTime' in elem && elem.dateTime ) ) )               ||
-                         ( 'title' in elem && elem.title )                             ||
-                         ( 'lang' in elem && elem.lang ) ) {
+                    if ( ( elem instanceof HTMLQuoteElement && elem.cite)    ||
+                         ( elem instanceof HTMLTableElement && elem.summary) ||
+                         ( elem instanceof HTMLModElement &&
+                             ( elem.cite || elem.dateTime ) )                ||
+                         ( elem instanceof HTMLElement &&
+                             ( elem.title || elem.lang ) )                   ||
+                         elem.getAttributeNS(XMLNS, "lang") ) {
                         dump("On metadata item.\n");
                         this.onMetaDataItem = true;
                     }
@@ -433,9 +447,7 @@ nsContextMenu.prototype = {
           // initialize popupURL
           const IOS = Components.classes["@mozilla.org/network/io-service;1"]
                       .getService(CI.nsIIOService);
-          var opener = new XPCNativeWrapper(window.content, "opener").opener;
-          var location = new XPCNativeWrapper(opener, "location").location;
-          this.popupURL = IOS.newURI(location.href, null, null);
+          this.popupURL = IOS.newURI(window.content.opener.location.href, null, null);
 
           // but cancel if it's an unsuitable URL
           const PM = Components.classes["@mozilla.org/PopupWindowManager;1"]
@@ -511,14 +523,14 @@ nsContextMenu.prototype = {
         openNewWindowWith( this.linkURL(), true );
     },
     // Open linked-to URL in a new tab.
-    openLinkInTab : function () {
+    openLinkInTab : function ( reverseBackgroundPref ) {
         // Determine linked-to URL.
-        openNewTabWith( this.linkURL(), true, false );
+        openNewTabWith( this.linkURL(), true, reverseBackgroundPref );
     },
     // Open frame in a new tab.
-    openFrameInTab : function () {
+    openFrameInTab : function ( reverseBackgroundPref ) {
         // Determine linked-to URL.
-        openNewTabWith( this.target.ownerDocument.location.href );
+        openNewTabWith( this.target.ownerDocument.location.href, true, reverseBackgroundPref );
     },
     // Reload clicked-in frame.
     reloadFrame : function () {
@@ -546,7 +558,7 @@ nsContextMenu.prototype = {
         // when there is one
         var reference = null;
         if (context == "selection")
-          reference = focusedWindow.__proto__.getSelection.call(focusedWindow);
+          reference = focusedWindow.getSelection();
         else if (context == "mathml")
           reference = this.target;
         else
@@ -570,12 +582,20 @@ nsContextMenu.prototype = {
     toggleImageSize : function () {
         content.document.toggleImageSize();
     },
+    // Reload image
+    reloadImage : function () {
+        urlSecurityCheck( this.imageURL, document );
+        if (this.target instanceof Components.interfaces.nsIImageLoadingContent)
+          this.target.forceReload();
+    },
     // Change current window to the URL of the image.
     viewImage : function () {
+        urlSecurityCheck( this.imageURL, document );
         openTopWin( this.imageURL );
     },
     // Change current window to the URL of the background image.
     viewBGImage : function () {
+        urlSecurityCheck( this.bgImageURL, document );
         openTopWin( this.bgImageURL );
     },
     setWallpaper: function() {
@@ -605,49 +625,46 @@ nsContextMenu.prototype = {
     },
     // Save URL of clicked-on link.
     saveLink : function () {
-        saveURL( this.linkURL(), this.linkText(), null, true );
+        saveURL( this.linkURL(), this.linkText(), null, true,
+                 getReferrer(document) );
     },
     // Save URL of clicked-on image.
     saveImage : function () {
-        saveURL( this.imageURL, null, "SaveImageTitle", false );
+        // Note: getReferrer wants our chrome document, not the actual
+        // target document; it handles getting that itself.
+        saveImageURL( this.imageURL, null, "SaveImageTitle", false,
+                      getReferrer(document) );
     },
-    // Generate email address and put it on clipboard.
-    copyEmail : function () {
-        // Copy the comma-separated list of email addresses only.
+    // Generate email address.
+    getEmail : function () {
+        // Get the comma-separated list of email addresses only.
         // There are other ways of embedding email addresses in a mailto:
         // link, but such complex parsing is beyond us.
-        var url = this.linkURL();
-        var qmark = url.indexOf( "?" );
         var addresses;
-        
-        if ( qmark > 7 ) {                   // 7 == length of "mailto:"
-            addresses = url.substring( 7, qmark );
-        } else {
-            addresses = url.substr( 7 );
-        }
-
-        // Let's try to unescape it using a character set
-        // in case the address is not ASCII.
         try {
-          var ownerDocument = new XPCNativeWrapper(this.target, "ownerDocument").ownerDocument;
-          var characterSet = new XPCNativeWrapper(ownerDocument, "characterSet").characterSet;
+          // Let's try to unescape it using a character set
+          var characterSet = this.target.ownerDocument.characterSet;
           const textToSubURI = Components.classes["@mozilla.org/intl/texttosuburi;1"]
                                          .getService(Components.interfaces.nsITextToSubURI);
-          addresses = textToSubURI.unEscapeNonAsciiURI(characterSet, addresses);
+          addresses = this.linkURL().match(/^mailto:([^?]+)/)[1];
+          addresses = textToSubURI.unEscapeURIForUI(characterSet, addresses);
         }
         catch(ex) {
           // Do nothing.
         }
-
+        return addresses;
+    },
+    // Copy email to clipboard
+    copyEmail : function () {
         var clipboard = this.getService( "@mozilla.org/widget/clipboardhelper;1",
                                          Components.interfaces.nsIClipboardHelper );
-        clipboard.copyString(addresses);
+        clipboard.copyString(this.getEmail());
     },    
     addBookmark : function() {
       var docshell = document.getElementById( "content" ).webNavigation;
       BookmarksUtils.addBookmark( docshell.currentURI.spec,
                                   docshell.document.title,
-                                  docshell.document.charset,
+                                  docshell.document.characterSet,
                                   false );
     },
     addBookmarkForFrame : function() {
@@ -658,7 +675,7 @@ nsContextMenu.prototype = {
         title = uri;
       BookmarksUtils.addBookmark( uri,
                                   title,
-                                  doc.charset,
+                                  doc.characterSet,
                                   false );
     },
     // Open Metadata window for node
@@ -789,7 +806,7 @@ nsContextMenu.prototype = {
     
     searchSelected : function( charlen ) {
         var focusedWindow = document.commandDispatcher.focusedWindow;
-        var searchStr = focusedWindow.__proto__.getSelection.call(focusedWindow);
+        var searchStr = focusedWindow.getSelection();
         searchStr = searchStr.toString();
         // searching for more than 150 chars makes no sense
         if (!charlen)
@@ -800,7 +817,8 @@ nsContextMenu.prototype = {
             pattern.test(searchStr);
             searchStr = RegExp.lastMatch;
         }
-        searchStr = searchStr.replace(/\s*(.*?)\s*$/, "$1");
+        searchStr = searchStr.replace(/^\s+/, "");
+        searchStr = searchStr.replace(/\s+$/, "");
         searchStr = searchStr.replace(/\s+/g, " ");
         return searchStr;
     },
@@ -824,29 +842,12 @@ nsContextMenu.prototype = {
     },
     isTargetATextBox : function ( node )
     {
-      if (node.nodeType != Node.ELEMENT_NODE)
-        return false;
+      if (node instanceof HTMLInputElement)
+        return (node.type == "text" || node.type == "password")
 
-      if (node.localName.toUpperCase() == "INPUT") {
-        var attrib = "";
-        var type = node.getAttribute("type");
-
-        if (type)
-          attrib = type.toUpperCase();
-
-        return( (attrib != "IMAGE") &&
-                (attrib != "CHECKBOX") &&
-                (attrib != "RADIO") &&
-                (attrib != "SUBMIT") &&
-                (attrib != "RESET") &&
-                (attrib != "HIDDEN") &&
-                (attrib != "RESET") &&
-                (attrib != "BUTTON") );
-      } else  {
-        return(node.localName.toUpperCase() == "TEXTAREA");
-      }
+      return (node instanceof HTMLTextAreaElement);
     },
-    
+
     // Determines whether or not the separator with the specified ID should be 
     // shown or not by determining if there are any non-hidden items between it
     // and the previous separator. 
@@ -877,7 +878,7 @@ function nsDefaultEngine()
         var pb = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefBranch);
         var pbi = pb.QueryInterface(
-                    Components.interfaces.nsIPrefBranchInternal);
+                    Components.interfaces.nsIPrefBranch2);
         pbi.addObserver(this.domain, this, false);
 
         // reuse code by explicitly invoking initial |observe| call
