@@ -39,7 +39,6 @@ var BookmarksMenu = {
   _selection:null,
   _target:null,
   _orientation:null,
-  
 
   //////////////////////////////////////////////////////////////////////////
   // Fill a context menu popup with menuitems appropriate for the current
@@ -47,23 +46,26 @@ var BookmarksMenu = {
   createContextMenu: function (aEvent)
   {
     var target = document.popupNode;
+    if (!this.isBTBookmark(target.id) && target.id != "bookmarks-ptf")
+      return false;
     target.focus() // buttons in the pt have -moz-user-focus: ignore -->
     this._selection   = this.getBTSelection(target);
     this._orientation = this.getBTOrientation(aEvent, target);
     this._target      = this.getBTTarget(target, this._orientation);
     BookmarksCommand.createContextMenu(aEvent, this._selection);
     this.onCommandUpdate();
-    aEvent.target.addEventListener("mousemove", BookmarksMenuController.onMouseMove, false)
+    aEvent.target.addEventListener("mousemove", BookmarksMenuController.onMouseMove, false);
+    return true;
   },
 
   /////////////////////////////////////////////////////////////////////////
   // Clean up after closing the context menu popup
   destroyContextMenu: function (aEvent)
   {
-    if (content) 
-      content.focus()
+    if (content)
+      content.focus();
     BookmarksMenuDNDObserver.onDragRemoveFeedBack(document.popupNode); // needed on cancel
-    aEvent.target.removeEventListener("mousemove", BookmarksMenuController.onMouseMove, false)
+    aEvent.target.removeEventListener("mousemove", BookmarksMenuController.onMouseMove, false);
   },
 
   /////////////////////////////////////////////////////////////////////////////
@@ -99,21 +101,17 @@ var BookmarksMenu = {
   getBTTarget: function (aNode, aOrientation)
   {
     var item, parent, index;
+
     switch (aNode.id) {
-    case "bookmarks-ptf":
-      parent = "NC:PersonalToolbarFolder";
-      item = BookmarksToolbar.getLastVisibleBookmark();
-      break;
     case "BookmarksMenu":
-      parent = "NC:BookmarksRoot";
-      break;
     case "bookmarks-button":
       parent = "NC:BookmarksRoot";
       break;
+    case "bookmarks-ptf":
+      item = BookmarksToolbar.getLastVisibleBookmark();
+      // Continue to next case.
     case "bookmarks-chevron":
       parent = "NC:PersonalToolbarFolder";
-      item = document.getElementById("bookmarks-ptf").lastChild;
-      aOrientation == BookmarksUtils.DROP_AFTER;
       break;
     default:
       if (aOrientation == BookmarksUtils.DROP_ON)
@@ -151,12 +149,11 @@ var BookmarksMenu = {
     parent = parent.id;
     switch (parent) {
     case "BookmarksMenu":
+    case "bookmarks-button":
       return "NC:BookmarksRoot";
     case "PersonalToolbar":
     case "bookmarks-chevron":
       return "NC:PersonalToolbarFolder";
-    case "bookmarks-button":
-      return "NC:BookmarksRoot";
     default:
       return parent;
     }
@@ -198,10 +195,12 @@ var BookmarksMenu = {
     if (target.localName == "menu"                 &&
         target.parentNode.localName != "menupopup")
       return BookmarksUtils.DROP_ON;
-    if (target.id == "bookmarks-ptf" || 
-        target.id == "bookmarks-chevron") {
-      return BookmarksUtils.DROP_ON;
+    if (target.id == "bookmarks-ptf") {
+      return target.hasChildNodes() ?
+          BookmarksUtils.DROP_AFTER : BookmarksUtils.DROP_ON;
     }
+    if (target.id == "bookmarks-chevron")
+      return BookmarksUtils.DROP_ON;
 
     var overButtonBoxObject = target.boxObject.QueryInterface(Components.interfaces.nsIBoxObject);
     var overParentBoxObject = target.parentNode.boxObject.QueryInterface(Components.interfaces.nsIBoxObject);
@@ -235,7 +234,7 @@ var BookmarksMenu = {
         border = size/5;
     else
       border = size/2;
-    
+
     // in the first region?
     if (clientCoordValue-coordValue < border)
       return BookmarksUtils.DROP_BEFORE;
@@ -275,6 +274,21 @@ var BookmarksMenu = {
   {
     if (this.isBTBookmark(aEvent.target.id))
       BookmarksUtils.loadBookmarkBrowser(aEvent, aDS);
+  },
+
+  ////////////////////////////////////////////////
+  // loads a bookmark with the middle mouse button
+  loadBookmarkMiddleClick: function (aEvent, aDS)
+  {
+    if (aEvent.type != "click" || aEvent.button != 1)
+      return;
+    // unlike for command events, we have to close the menus manually
+    for (var node = aEvent.target; node != aEvent.currentTarget;
+         node = node.parentNode) {
+      if (node.nodeType == node.ELEMENT_NODE && node.tagName == "menupopup")
+        node.hidePopup();
+    }
+    this.loadBookmark(aEvent, aDS);
   }
 }
 
@@ -372,7 +386,7 @@ var BookmarksMenuDNDObserver = {
     }
     if (this.isPlatformNotSupported)
       return;
-    if (this.isTimerSupported)
+    if (this.isTimerSupported || !aDragSession.sourceNode)
       return;
     this.onDragOverCheckTimers();
   },
@@ -454,12 +468,22 @@ var BookmarksMenuDNDObserver = {
       var element = document.createElementNS(XUL_NS, "menuseparator");
       menuTarget.insertBefore(element, menuTarget.lastChild);
     }
-
   },
 
   canDrop: function (aEvent, aDragSession)
   {
     var target = aEvent.target;
+
+    if (aDragSession) { // this function gets its API abused by onDragStart
+      var orientation = BookmarksMenu.getBTOrientation(aEvent, target);
+      if (target == aDragSession.sourceNode ||
+          (target == aDragSession.sourceNode.previousSibling &&
+           orientation == BookmarksUtils.DROP_AFTER) ||
+          (target == aDragSession.sourceNode.nextSibling &&
+           orientation == BookmarksUtils.DROP_BEFORE))
+        return false;
+    }
+
     return BookmarksMenu.isBTBookmark(target.id)       && 
            target.id != "NC:SystemBookmarksStaticRoot" &&
            target.id.substring(0,5) != "find:"         ||
@@ -479,8 +503,8 @@ var BookmarksMenuDNDObserver = {
     flavourSet.appendFlavour("application/x-moz-file", "nsIFile");
     flavourSet.appendFlavour("text/unicode");
     return flavourSet;
-  }, 
-  
+  },
+
 
   ////////////////////////////////////
   // Private methods and properties //
@@ -488,6 +512,7 @@ var BookmarksMenuDNDObserver = {
 
   springLoadedMenuDelay: 350, // milliseconds
   isPlatformNotSupported: navigator.platform.indexOf("Mac") != -1, // see bug 136524
+  // Needs to be dynamically overridden (to |true|) in the case of an external drag: see bug 232795.
   isTimerSupported: navigator.platform.indexOf("Win") == -1,
 
   mCurrentDragOverTarget: null,
@@ -538,7 +563,7 @@ var BookmarksMenuDNDObserver = {
         if (children[i] != this.mCurrentDragOverTarget || this.mCurrentDropPosition != BookmarksUtils.DROP_ON)
           children[i].lastChild.hidePopup();
       }
-    } 
+    }
   },
 
   onDragCloseTarget: function ()
@@ -581,7 +606,7 @@ var BookmarksMenuDNDObserver = {
   {
     if (this.isPlatformNotSupported)
       return;
-    if (this.isTimerSupported) {
+    if (this.isTimerSupported || !aDragSession.sourceNode) {
       var targetToBeLoaded = aTarget;
       clearTimeout(this.loadTimer);
       if (aTarget == aDragSession.sourceNode)
@@ -600,7 +625,7 @@ var BookmarksMenuDNDObserver = {
     if (this.isPlatformNotSupported)
       return;
     var This = this;
-    if (this.isTimerSupported) {
+    if (this.isTimerSupported || !aDragSession.sourceNode) {
       clearTimeout(this.closeTimer)
       this.closeTimer=setTimeout(function () {This.onDragCloseTarget()}, This.springLoadedMenuDelay);
     } else {
@@ -609,7 +634,7 @@ var BookmarksMenuDNDObserver = {
       this.closeTarget = aTarget;
       this.loadTimer = null;
 
-      // If user isn't rearranging within the menu, close it
+      // If the user isn't rearranging within the menu, close it
       // To do so, we exploit a Mac bug: timeout set during
       // drag and drop on Windows and Mac are fired only after that the drop is released.
       // timeouts will pile up, we may have a better approach but for the moment, this one
@@ -667,7 +692,7 @@ var BookmarksMenuDNDObserver = {
   },
 
   onDragRemoveFeedBack: function (aTarget)
-  { 
+  {
     var newTarget;
     var bt;
     if (aTarget.id == "PersonalToolbar" || aTarget.id == "bookmarks-ptf") { 
@@ -725,10 +750,10 @@ var BookmarksToolbar =
   },
 
   resizeFunc: function(event) 
-  { 
+  {
     if (!event) // timer callback case
       BookmarksToolbarRDFObserver._overflowTimerInEffect = false;
-    else if (event.target != document)
+    else if (event.target != window)
       return; // only interested in chrome resizes
 
     var buttons = document.getElementById("bookmarks-ptf");
@@ -752,7 +777,7 @@ var BookmarksToolbar =
     for (var i=0; i<buttons.childNodes.length; i++) {
       var button = buttons.childNodes[i];
       button.collapsed = overflowed;
-      
+
       if (i == buttons.childNodes.length - 1)
         chevronWidth = 0;
       remainingWidth -= button.boxObject.width;
@@ -768,33 +793,32 @@ var BookmarksToolbar =
     }
   },
 
-  // Fill in tooltips for personal toolbar
+  // Fill in tooltips for personal toolbar (and Bookmarks menu).
   fillInBTTooltip: function (tipElement)
   {
+    // Don't show a tooltip for non bookmark related elements.
+    if (!/bookmark/.test(tipElement.className))
+      return false;
 
     var title = tipElement.label;
     var url = tipElement.statusText;
 
-    if (!title && !url) {
-      // bail out early if there is nothing to show
+    // Don't show a tooltip without any data.
+    if (!title && !url)
       return false;
-    }
 
-    var tooltipTitle = document.getElementById("btTitleText");
-    var tooltipUrl = document.getElementById("btUrlText"); 
-    if (title && title != url) {
-      tooltipTitle.removeAttribute("hidden");
-      tooltipTitle.setAttribute("value", title);
-    } else  {
-      tooltipTitle.setAttribute("hidden", "true");
-    }
-    if (url) {
-      tooltipUrl.removeAttribute("hidden");
-      tooltipUrl.setAttribute("value", url);
-    } else {
-      tooltipUrl.setAttribute("hidden", "true");
-    }
-    return true; // show tooltip
+    var tooltipElement = document.getElementById("btTitleText");
+    tooltipElement.hidden = !title || (title == url);
+    if (!tooltipElement.hidden)
+      tooltipElement.setAttribute("value", title);
+
+    tooltipElement = document.getElementById("btUrlText");
+    tooltipElement.hidden = !url;
+    if (!tooltipElement.hidden)
+      tooltipElement.setAttribute("value", url);
+
+    // Show the tooltip.
+    return true;
   }
 }
 
@@ -813,13 +837,13 @@ var BookmarksToolbarRDFObserver =
   onChange: function (aDataSource, aSource, aProperty, aOldTarget, aNewTarget) {},
   onMove: function (aDataSource, aOldSource, aNewSource, aProperty, aTarget) {},
   onBeginUpdateBatch: function (aDataSource) {},
-  onEndUpdateBatch:   function (aDataSource) {
+  onEndUpdateBatch: function (aDataSource)
+  {
     if (this._overflowTimerInEffect)
       return;
     this._overflowTimerInEffect = true;
     setTimeout(BookmarksToolbar.resizeFunc, 0, null);
   },
-
   _overflowTimerInEffect: false,
   setOverflowTimeout: function (aSource, aProperty)
   {
