@@ -57,21 +57,21 @@
 #include "nsCOMPtr.h"
 #include "nsCRT.h"
 #include "nsEscape.h"
-#include "nsIDocumentLoader.h"
-#include "nsIDocumentViewer.h"
 #include "nsIEnumerator.h"
+#ifdef MOZ_RDF
 #include "nsIRDFService.h"
+#include "nsRDFCID.h"
+#include "rdf.h"
+#endif
 #include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIServiceManager.h"
 #include "nsISupportsArray.h"
 #include "nsIXPConnect.h"
 #include "nsEnumeratorUtils.h"
-#include "nsRDFCID.h"
 #include "nsString.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
-#include "rdf.h"
 #include "nsITextToSubURI.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
@@ -100,7 +100,9 @@ static const int FORMAT_XUL = 3;
 // Common CIDs
 //
 
+#ifdef MOZ_RDF
 static NS_DEFINE_CID(kRDFServiceCID,             NS_RDFSERVICE_CID);
+#endif
 
 // Various protocols we have to special case
 static const char               kFTPProtocol[] = "ftp://";
@@ -111,6 +113,7 @@ static const char               kGopherProtocol[] = "gopher://";
 // nsHTTPIndex
 //
 
+#ifdef MOZ_RDF
 NS_IMPL_THREADSAFE_ISUPPORTS7(nsHTTPIndex,
                               nsIHTTPIndex,
                               nsIRDFDataSource,
@@ -192,11 +195,11 @@ nsHTTPIndex::OnFTPControlLog(PRBool server, const char *msg)
 
     JSContext* jscontext = NS_REINTERPRET_CAST(JSContext*,
                                                context->GetNativeContext());
+    NS_ENSURE_TRUE(jscontext, NS_OK);
 
     JSObject* global = JS_GetGlobalObject(jscontext);
+    NS_ENSURE_TRUE(global, NS_OK);
 
-    if (!jscontext || !global) return NS_OK;
-    
     jsval params[2];
 
     nsString unicodeMsg;
@@ -319,7 +322,7 @@ nsHTTPIndex::OnStartRequest(nsIRequest *request, nsISupports* aContext)
     nsCOMPtr<nsIRDFResource> entry;
     rv = mDirRDF->GetResource(entryuriC, getter_AddRefs(entry));
     
-    NS_ConvertUTF8toUCS2 uriUnicode(entryuriC);
+    NS_ConvertUTF8toUTF16 uriUnicode(entryuriC);
 
     nsCOMPtr<nsIRDFLiteral> URLVal;
     rv = mDirRDF->GetLiteral(uriUnicode.get(), getter_AddRefs(URLVal));
@@ -366,7 +369,7 @@ nsHTTPIndex::OnStopRequest(nsIRequest *request,
   mParser->GetComment(getter_Copies(commentStr));
 
   nsCOMPtr<nsIRDFLiteral> comment;
-  rv = mDirRDF->GetLiteral(NS_ConvertASCIItoUCS2(commentStr).get(), getter_AddRefs(comment));
+  rv = mDirRDF->GetLiteral(NS_ConvertASCIItoUTF16(commentStr).get(), getter_AddRefs(comment));
   if (NS_FAILED(rv)) return rv;
 
   rv = Assert(mDirectory, kNC_Comment, comment, PR_TRUE);
@@ -436,7 +439,7 @@ nsHTTPIndex::OnIndexAvailable(nsIRequest* aRequest, nsISupports *aContext,
 
   PRBool isDirType = (type == nsIDirIndex::TYPE_DIRECTORY);
 
-  if (isDirType) {
+  if (isDirType && entryuriC.Last() != '/') {
       entryuriC.Append('/');
   }
 
@@ -472,6 +475,8 @@ nsHTTPIndex::OnIndexAvailable(nsIRequest* aRequest, nsISupports *aContext,
       // description
       rv = aIndex->GetDescription(getter_Copies(xpstr));
       if (NS_FAILED(rv)) return rv;
+      if (xpstr.Last() == '/')
+        xpstr.Truncate(xpstr.Length() - 1);
 
       rv = mDirRDF->GetLiteral(xpstr.get(), getter_AddRefs(lit));
       if (NS_FAILED(rv)) return rv;
@@ -482,7 +487,8 @@ nsHTTPIndex::OnIndexAvailable(nsIRequest* aRequest, nsISupports *aContext,
       PRInt64 size;
       rv = aIndex->GetSize(&size);
       if (NS_FAILED(rv)) return rv;
-      if (LL_NE(size, LL_INIT(0, -1))) {
+      PRInt64 minus1 = LL_MAXUINT;
+      if (LL_NE(size, minus1)) {
         PRInt32 intSize;
         LL_L2I(intSize, size);
         // XXX RDF should support 64 bit integers (bug 240160)
@@ -1275,13 +1281,7 @@ nsHTTPIndex::ArcLabelsOut(nsIRDFResource *aSource, nsISimpleEnumerator **_retval
 		}
 	}
 
-	nsISimpleEnumerator* result = new nsArrayEnumerator(array);
-	if (! result)
-	return NS_ERROR_OUT_OF_MEMORY;
-
-	NS_ADDREF(result);
-	*_retval = result;
-	return(NS_OK);
+        return NS_NewArrayEnumerator(_retval, array);
 }
 
 NS_IMETHODIMP
@@ -1342,6 +1342,7 @@ nsHTTPIndex::GetAllCmds(nsIRDFResource *aSource, nsISimpleEnumerator **_retval)
 	return(rv);
 }
 
+#endif /* MOZ_RDF */
 
 
 //----------------------------------------------------------------------
@@ -1379,15 +1380,16 @@ nsDirectoryViewerFactory::CreateInstance(const char *aCommand,
   nsCOMPtr<nsIPrefBranch> prefSrv = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return rv;
 
-  PRBool useXUL = PR_FALSE;
+  PRBool useXUL = PR_FALSE;  
+  PRBool viewSource = (PL_strstr(aContentType,"view-source") != 0);
+  
+#ifdef MOZ_RDF
   PRInt32 dirPref;
   rv = prefSrv->GetIntPref("network.dir.format", &dirPref);
   if (NS_SUCCEEDED(rv) && dirPref == FORMAT_XUL) {
     useXUL = PR_TRUE;
   }
 
-  PRBool viewSource = (PL_strstr(aContentType,"view-source") != 0);
-
   if ((NS_FAILED(rv) || useXUL) && !viewSource) {
     // ... and setup the original channel's content type
     (void)aChannel->SetContentType(NS_LITERAL_CSTRING("application/vnd.mozilla.xul+xml"));
@@ -1446,6 +1448,7 @@ nsDirectoryViewerFactory::CreateInstance(const char *aCommand,
     
     return NS_OK;
   }
+#endif
 
   // setup the original channel's content type
   (void)aChannel->SetContentType(NS_LITERAL_CSTRING("text/html"));
