@@ -60,10 +60,14 @@
 #include "nsIPromptService.h"
 #include "nsIObserverService.h"
 #include "nsIProfileChangeStatus.h"
-#include "nsISound.h"
 #include "nsIPrefService.h"
 #include "nsIFileURL.h"
 #include "nsIAlertsService.h"
+#include "nsEmbedCID.h"
+#include "nsInt64.h"
+#ifdef MOZ_XUL_APP
+#include "nsToolkitCompsCID.h" 
+#endif
 
 /* Outstanding issues/todo:
  * 1. Implement pause/resume.
@@ -71,7 +75,8 @@
   
 #define DOWNLOAD_MANAGER_FE_URL "chrome://communicator/content/downloadmanager/downloadmanager.xul"
 #define DOWNLOAD_MANAGER_BUNDLE "chrome://communicator/locale/downloadmanager/downloadmanager.properties"
-#define INTERVAL 500
+
+static const nsInt64 gInterval((PRUint32)(400 * PR_USEC_PER_MSEC));
 
 static nsIRDFResource* gNC_DownloadsRoot = nsnull;
 static nsIRDFResource* gNC_File = nsnull;
@@ -394,9 +399,13 @@ nsDownloadManager::AssertProgressInfoFor(const nsACString& aTargetPath)
   // update transferred
   nsDownload::TransferInformation transferInfo =
                                  internalDownload->GetTransferInformation();
- 
-  nsAutoString currBytes; currBytes.AppendInt(transferInfo.mCurrBytes);
-  nsAutoString maxBytes; maxBytes.AppendInt(transferInfo.mMaxBytes);
+
+  // convert from bytes to kbytes for progress display
+  PRInt64 current = (PRFloat64)transferInfo.mCurrBytes / 1024 + .5;
+  PRInt64 max = (PRFloat64)transferInfo.mMaxBytes / 1024 + .5;
+
+  nsAutoString currBytes; currBytes.AppendInt(current);
+  nsAutoString maxBytes; maxBytes.AppendInt(max);
   const PRUnichar *strings[] = {
     currBytes.get(),
     maxBytes.get()
@@ -429,10 +438,11 @@ nsDownloadManager::AssertProgressInfoFor(const nsACString& aTargetPath)
 NS_IMETHODIMP
 nsDownloadManager::AddDownload(nsIURI* aSource,
                                nsIURI* aTarget,
-                               const PRUnichar* aDisplayName,
+                               const nsAString& aDisplayName,
                                nsIMIMEInfo *aMIMEInfo,
-                               PRInt64 aStartTime,
-                               nsIWebBrowserPersist* aPersist,
+                               PRTime aStartTime,
+                               nsILocalFile* aTempFile,
+                               nsICancelable* aCancelable,
                                nsIDownload** aDownload)
 {
   NS_ENSURE_ARG_POINTER(aSource);
@@ -443,7 +453,8 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
   nsresult rv = GetDownloadsContainer(getter_AddRefs(downloads));
   if (NS_FAILED(rv)) return rv;
 
-  nsDownload* internalDownload = new nsDownload(this, aTarget, aSource);
+  // this will create a cycle that will be broken in nsDownload::OnStateChange
+  nsDownload* internalDownload = new nsDownload(this, aTarget, aSource, aCancelable);
   if (!internalDownload)
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -484,6 +495,7 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
     targetFile->GetLeafName(displayName);
   }
   internalDownload->SetDisplayName(displayName.get());
+  internalDownload->SetTempFile(aTempFile);
  
   nsCOMPtr<nsIRDFLiteral> nameLiteral;
   gRDFService->GetLiteral(displayName.get(), getter_AddRefs(nameLiteral));
@@ -525,13 +537,6 @@ nsDownloadManager::AddDownload(nsIURI* aSource,
   rv = remote->Flush();
   if (NS_FAILED(rv)) return rv;
 
-  // if a persist object was specified, set the download item as the progress listener
-  // this will create a cycle that will be broken in nsDownload::OnStateChange
-  if (aPersist) {
-    internalDownload->SetPersist(aPersist);
-    aPersist->SetProgressListener(internalDownload);
-  }
-
   mCurrDownloads.Put(utf8Path, internalDownload);
 
   return rv;
@@ -552,45 +557,11 @@ nsDownloadManager::GetDownload(const nsACString & aTargetPath, nsIDownload** aDo
 NS_IMETHODIMP
 nsDownloadManager::CancelDownload(const nsACString & aTargetPath)
 {
-  nsresult rv = NS_OK;
   nsRefPtr<nsDownload> internalDownload = mCurrDownloads.GetWeak(aTargetPath);
   if (!internalDownload)
     return NS_ERROR_FAILURE;
 
-  // Don't cancel if download is already finished
-  if (internalDownload->GetDownloadState() == FINISHED)
-    return NS_OK;
-  
-  internalDownload->SetDownloadState(CANCELED);
-
-  // if a persist was provided, we can do the cancel ourselves.
-  nsCOMPtr<nsIWebBrowserPersist> persist;
-  internalDownload->GetPersist(getter_AddRefs(persist));
-  if (persist) {
-    rv = persist->CancelSave();
-    if (NS_FAILED(rv)) return rv;
-  }
-
-  // if an observer was provided, notify that the download was cancelled.
-  // if no persist was provided, this is necessary so that whatever transfer
-  // component being used can cancel the download itself.
-  nsCOMPtr<nsIObserver> observer;
-  internalDownload->GetObserver(getter_AddRefs(observer));
-  if (observer) {
-    rv = observer->Observe(NS_STATIC_CAST(nsIDownload*, internalDownload), "oncancel", nsnull);
-    if (NS_FAILED(rv)) return rv;
-  }
-  
-  DownloadEnded(aTargetPath, nsnull);
-  
-  // if there's a progress dialog open for the item,
-  // we have to notify it that we're cancelling
-  observer = do_QueryInterface(internalDownload->GetDialog());
-  if (observer) {
-    rv = observer->Observe(NS_STATIC_CAST(nsIDownload*, internalDownload), "oncancel", nsnull);
-  }
-  
-  return rv;
+  return internalDownload->Cancel();
 }
 
 NS_IMETHODIMP
@@ -798,11 +769,12 @@ nsDownloadManager::OpenProgressDialogFor(nsIDownload* aDownload, nsIDOMWindow* a
   nsCOMPtr<nsIMIMEInfo> mimeInfo;
   aDownload->GetMIMEInfo(getter_AddRefs(mimeInfo));
 
-  dialog->Init(source, target, nsnull, mimeInfo, startTime, nsnull); 
+  dialog->Init(source, target, EmptyString(), mimeInfo, startTime, nsnull,
+               nsnull); 
   dialog->SetObserver(internalDownload);
 
   // now set the listener so we forward notifications to the dialog
-  nsCOMPtr<nsIWebProgressListener> listener = do_QueryInterface(dialog);
+  nsCOMPtr<nsIWebProgressListener2> listener = do_QueryInterface(dialog);
   internalDownload->SetDialogListener(listener);
   
   internalDownload->SetDialog(dialog);
@@ -870,7 +842,7 @@ nsDownloadManager::Observe(nsISupports* aSubject, const char* aTopic, const PRUn
                                     getter_Copies(proceed));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsIPromptService> promptService(do_GetService("@mozilla.org/embedcomp/prompt-service;1", &rv));
+    nsCOMPtr<nsIPromptService> promptService(do_GetService(NS_PROMPTSERVICE_CONTRACTID, &rv));
     if (NS_FAILED(rv))
       return rv;
 
@@ -919,20 +891,24 @@ nsDownloadManager::Observe(nsISupports* aSubject, const char* aTopic, const PRUn
 ///////////////////////////////////////////////////////////////////////////////
 // nsDownload
 
-NS_IMPL_ISUPPORTS4(nsDownload, nsIDownload, nsITransfer, nsIWebProgressListener, nsIObserver)
+NS_IMPL_ISUPPORTS5(nsDownload, nsIDownload, nsITransfer, nsIWebProgressListener,
+                   nsIWebProgressListener2, nsIObserver)
 
 nsDownload::nsDownload(nsDownloadManager* aManager,
                        nsIURI* aTarget,
-                       nsIURI* aSource) :
+                       nsIURI* aSource,
+                       nsICancelable* aCancelable) :
                          mDownloadManager(aManager),
                          mTarget(aTarget),
                          mSource(aSource),
+                         mCancelable(aCancelable),
                          mDownloadState(NOTSTARTED),
                          mPercentComplete(0),
-                         mCurrBytes(0),
-                         mMaxBytes(0),
-                         mStartTime(0),
-                         mLastUpdate(-500)
+                         mCurrBytes(LL_ZERO),
+                         mMaxBytes(LL_ZERO),
+                         mStartTime(LL_ZERO),
+                         mSpeed(0),
+                         mLastUpdate(PR_Now() - (PRUint32)gInterval)
 {
 }
 
@@ -953,6 +929,67 @@ nsDownload::Suspend()
   return mRequest->Suspend();
 }
 
+nsresult
+nsDownload::Cancel()
+{
+  // Don't cancel if download is already finished or canceled
+  if (GetDownloadState() == FINISHED || GetDownloadState() == CANCELED)
+    return NS_OK;
+
+  nsresult rv = mCancelable->Cancel(NS_BINDING_ABORTED);
+  if (NS_FAILED(rv))
+    return rv;
+  
+  SetDownloadState(CANCELED);
+
+  nsCAutoString path;
+  rv = GetFilePathUTF8(mTarget, path);
+  if (NS_FAILED(rv))
+    return rv;
+  mDownloadManager->DownloadEnded(path, nsnull);
+  
+  // Dump the temp file.  This should really be done when the transfer
+  // is cancelled, but there are other cancellation causes that shouldn't
+  // remove this. We need to improve those bits.
+  if (mTempFile) {
+    PRBool exists;
+    mTempFile->Exists(&exists);
+    if (exists)
+      mTempFile->Remove(PR_FALSE);
+  }
+
+  // if there's a progress dialog open for the item,
+  // we have to notify it that we're cancelling
+  nsCOMPtr<nsIObserver> observer = do_QueryInterface(GetDialog());
+  if (observer) {
+    rv = observer->Observe(NS_STATIC_CAST(nsIDownload*, this), "oncancel", nsnull);
+  }
+  
+  return rv;
+}
+
+nsresult
+nsDownload::SetDisplayName(const PRUnichar* aDisplayName)
+{
+  mDisplayName = aDisplayName;
+
+  nsCOMPtr<nsIRDFDataSource> ds;
+  mDownloadManager->GetDataSource(getter_AddRefs(ds));
+
+  nsCOMPtr<nsIRDFLiteral> nameLiteral;
+  nsCOMPtr<nsIRDFResource> res;
+  nsCAutoString path;
+  nsresult rv = GetFilePathUTF8(mTarget, path);
+  if (NS_FAILED(rv)) return rv;
+
+  gRDFService->GetResource(path, getter_AddRefs(res));
+  
+  gRDFService->GetLiteral(aDisplayName, getter_AddRefs(nameLiteral));
+  ds->Assert(res, gNC_Name, nameLiteral, PR_TRUE);
+
+  return NS_OK;
+}
+
 nsresult
 nsDownload::Resume()
 {
@@ -975,11 +1012,7 @@ nsDownload::Observe(nsISupports* aSubject, const char* aTopic, const PRUnichar*
   if (strcmp(aTopic, "oncancel") == 0) {
     SetDialog(nsnull);
 
-    nsCAutoString path;
-    nsresult rv = GetFilePathUTF8(mTarget, path);
-    // XXX why can't nsDownload cancel itself without help from the dl manager?
-    if (NS_SUCCEEDED(rv))
-      mDownloadManager->CancelDownload(path);
+    Cancel();
     // Ignoring return value; this function will get called twice,
     // and bad things happen if we return a failure code the second time.
     return NS_OK;
@@ -994,27 +1027,24 @@ nsDownload::Observe(nsISupports* aSubject, const char* aTopic, const PRUnichar*
   return NS_OK;
 }
 
-
 ///////////////////////////////////////////////////////////////////////////////
-// nsIWebProgressListener
+// nsIWebProgressListener2
 
 NS_IMETHODIMP
-nsDownload::OnProgressChange(nsIWebProgress *aWebProgress,
-                             nsIRequest *aRequest,
-                             PRInt32 aCurSelfProgress,
-                             PRInt32 aMaxSelfProgress,
-                             PRInt32 aCurTotalProgress,
-                             PRInt32 aMaxTotalProgress)
+nsDownload::OnProgressChange64(nsIWebProgress *aWebProgress,
+                               nsIRequest *aRequest,
+                               PRInt64 aCurSelfProgress,
+                               PRInt64 aMaxSelfProgress,
+                               PRInt64 aCurTotalProgress,
+                               PRInt64 aMaxTotalProgress)
 {
-
   if (!mRequest)
     mRequest = aRequest; // used for pause/resume
 
   // filter notifications since they come in so frequently
-  PRTime delta;
   PRTime now = PR_Now();
-  LL_SUB(delta, now, mLastUpdate);
-  if (LL_CMP(delta, <, INTERVAL) && aMaxTotalProgress != -1 && aCurTotalProgress < aMaxTotalProgress)
+  nsInt64 delta = now - mLastUpdate;
+  if (delta < gInterval)
     return NS_OK;
 
   mLastUpdate = now;
@@ -1028,43 +1058,67 @@ nsDownload::OnProgressChange(nsIWebProgress *aWebProgress,
     mDownloadManager->DownloadStarted(path);
   }
 
+  // Calculate the speed using the elapsed delta time and bytes downloaded
+  // during that time for more accuracy.
+  double elapsedSecs = double(delta) / PR_USEC_PER_SEC;
+  if (elapsedSecs > 0) {
+    nsUint64 curTotalProgress = (PRUint64)aCurTotalProgress;
+    nsUint64 diffBytes = curTotalProgress - nsUint64(mCurrBytes);
+    double speed = double(diffBytes) / elapsedSecs;
+    if (LL_IS_ZERO(mCurrBytes))
+      mSpeed = speed;
+    else {
+      // Calculate 'smoothed average' of 10 readings.
+      mSpeed = mSpeed * 0.9 + speed * 0.1;
+    }
+  }
+
   if (aMaxTotalProgress > 0)
     mPercentComplete = aCurTotalProgress * 100 / aMaxTotalProgress;
   else
     mPercentComplete = -1;
 
-  mCurrBytes = (PRInt32)((PRFloat64)aCurTotalProgress / 1024.0 + .5);
-  mMaxBytes = (PRInt32)((PRFloat64)aMaxTotalProgress / 1024 + .5);
-
-  if (mListener) {
-    mListener->OnProgressChange(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress,
-                                aCurTotalProgress, aMaxTotalProgress);
-  }
+  mCurrBytes = aCurTotalProgress;
+  mMaxBytes = aMaxTotalProgress;
 
   if (mDownloadManager->MustUpdateUI()) {
     nsCOMPtr<nsIDownloadProgressListener> internalListener;
     mDownloadManager->GetInternalListener(getter_AddRefs(internalListener));
     if (internalListener) {
       internalListener->OnProgressChange(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress,
-                                          aCurTotalProgress, aMaxTotalProgress, this);
+                                         aCurTotalProgress, aMaxTotalProgress, this);
     }
   }
 
   if (mDialogListener) {
-    mDialogListener->OnProgressChange(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress,
-                                          aCurTotalProgress, aMaxTotalProgress);
+    mDialogListener->OnProgressChange64(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress,
+                                        aCurTotalProgress, aMaxTotalProgress);
   }
 
   return NS_OK;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// nsIWebProgressListener
+
+NS_IMETHODIMP
+nsDownload::OnProgressChange(nsIWebProgress *aWebProgress,
+                             nsIRequest *aRequest,
+                             PRInt32 aCurSelfProgress,
+                             PRInt32 aMaxSelfProgress,
+                             PRInt32 aCurTotalProgress,
+                             PRInt32 aMaxTotalProgress)
+{
+  return OnProgressChange64(aWebProgress, aRequest,
+                            aCurSelfProgress, aMaxSelfProgress,
+                            aCurTotalProgress, aMaxTotalProgress);
+}
+
+
 NS_IMETHODIMP
 nsDownload::OnLocationChange(nsIWebProgress *aWebProgress,
                              nsIRequest *aRequest, nsIURI *aLocation)
 {
-  if (mListener)
-    mListener->OnLocationChange(aWebProgress, aRequest, aLocation);
-
   if (mDownloadManager->MustUpdateUI()) {
     nsCOMPtr<nsIDownloadProgressListener> internalListener;
     mDownloadManager->GetInternalListener(getter_AddRefs(internalListener));
@@ -1091,9 +1145,6 @@ nsDownload::OnStatusChange(nsIWebProgress *aWebProgress,
       mDownloadManager->DownloadEnded(path, aMessage);
   }
 
-  if (mListener)
-    mListener->OnStatusChange(aWebProgress, aRequest, aStatus, aMessage);
-
   if (mDownloadManager->MustUpdateUI()) {
     nsCOMPtr<nsIDownloadProgressListener> internalListener;
     mDownloadManager->GetInternalListener(getter_AddRefs(internalListener));
@@ -1123,7 +1174,7 @@ nsDownload::OnStatusChange(nsIWebProgress *aWebProgress,
         wm->GetMostRecentWindow(NS_LITERAL_STRING("Download:Manager").get(), getter_AddRefs(dmWindow));
 
       // Show alert.
-      nsCOMPtr<nsIPromptService> prompter(do_GetService("@mozilla.org/embedcomp/prompt-service;1"));
+      nsCOMPtr<nsIPromptService> prompter(do_GetService(NS_PROMPTSERVICE_CONTRACTID));
       if (prompter)
         prompter->Alert(dmWindow, title, aMessage);
     }
@@ -1162,9 +1213,9 @@ void nsDownload::DisplayDownloadFinishedAlert()
   
   nsCAutoString url;
   mTarget->GetSpec(url);
-  alertsService->ShowAlertNotification(PromiseFlatCString(NS_LITERAL_CSTRING("moz-icon://") + url).get(),
+  alertsService->ShowAlertNotification(NS_LITERAL_STRING("moz-icon://") + NS_ConvertUTF8toUTF16(url),
                                        finishedTitle, finishedText, PR_TRUE,
-                                       NS_LITERAL_STRING("download").get(), this);
+                                       NS_LITERAL_STRING("download"), this);
 }
 
 NS_IMETHODIMP
@@ -1172,26 +1223,31 @@ nsDownload::OnStateChange(nsIWebProgress* aWebProgress,
                           nsIRequest* aRequest, PRUint32 aStateFlags,
                           nsresult aStatus)
 {
-  if (aStateFlags & STATE_START)
-    mStartTime = PR_Now();
+  // Record the start time only if it hasn't been set.
+  if (LL_IS_ZERO(mStartTime) && (aStateFlags & STATE_START))
+    SetStartTime(PR_Now());
 
   // When we break the ref cycle with mPersist, we don't want to lose
   // access to out member vars!
   nsRefPtr<nsDownload> kungFuDeathGrip(this);
   
-  if (mListener)
-    mListener->OnStateChange(aWebProgress, aRequest, aStateFlags, aStatus);
-
   // We need to update mDownloadState before updating the dialog, because
   // that will close and call CancelDownload if it was the last open window.
   nsresult rv = NS_OK;
   if (aStateFlags & STATE_STOP) {
     if (mDownloadState == DOWNLOADING || mDownloadState == NOTSTARTED) {
       mDownloadState = FINISHED;
+
+      // Set file size at the end of a tranfer (for unknown transfer amounts)
+      if (mMaxBytes == -1)
+        mMaxBytes = mCurrBytes;
+
       // Files less than 1Kb shouldn't show up as 0Kb.
-      if (mMaxBytes==0)
-        mMaxBytes = 1;
-      mCurrBytes = mMaxBytes;
+      if (mMaxBytes < 1024) {
+        mCurrBytes = 1024;
+        mMaxBytes  = 1024;
+      }
+
       mPercentComplete = 100;
 
       // Play a sound or show an alert when the download finishes
@@ -1215,16 +1271,18 @@ nsDownload::OnStateChange(nsIWebProgress* aWebProgress,
       }
 
       if (!soundStr.IsEmpty()) {
-        nsCOMPtr<nsISound> snd = do_GetService("@mozilla.org/sound;1");
-        if (snd) {
+        if (!mDownloadManager->mSoundInterface) {
+          mDownloadManager->mSoundInterface = do_CreateInstance("@mozilla.org/sound;1");
+        }
+        if (mDownloadManager->mSoundInterface) {
           nsCOMPtr<nsIURI> soundURI;
           
           NS_NewURI(getter_AddRefs(soundURI), soundStr);
           nsCOMPtr<nsIURL> soundURL(do_QueryInterface(soundURI));
           if (soundURL)
-            snd->Play(soundURL);
+            mDownloadManager->mSoundInterface->Play(soundURL);
           else
-            snd->Beep();
+            mDownloadManager->mSoundInterface->Beep();
         }
       }
       if (showAlert)
@@ -1239,8 +1297,12 @@ nsDownload::OnStateChange(nsIWebProgress* aWebProgress,
     }
 
     // break the cycle we created in AddDownload
-    if (mPersist)
-      mPersist->SetProgressListener(nsnull);
+    mCancelable = nsnull;
+    // and the one with the progress dialog
+    if (mDialog) {
+      mDialog->SetObserver(nsnull);
+      mDialog = nsnull;
+    }
   }
 
   if (mDownloadManager->MustUpdateUI()) {
@@ -1250,8 +1312,13 @@ nsDownload::OnStateChange(nsIWebProgress* aWebProgress,
       internalListener->OnStateChange(aWebProgress, aRequest, aStateFlags, aStatus, this);
   }
 
-  if (mDialogListener)
+  if (mDialogListener) {
     mDialogListener->OnStateChange(aWebProgress, aRequest, aStateFlags, aStatus);
+    if (aStateFlags & STATE_STOP) {
+      // Break this cycle, too
+      mDialogListener = nsnull;
+    }
+  }
 
   return rv;
 }
@@ -1260,9 +1327,6 @@ NS_IMETHODIMP
 nsDownload::OnSecurityChange(nsIWebProgress *aWebProgress,
                              nsIRequest *aRequest, PRUint32 aState)
 {
-  if (mListener)
-    mListener->OnSecurityChange(aWebProgress, aRequest, aState);
-
   if (mDownloadManager->MustUpdateUI()) {
     nsCOMPtr<nsIDownloadProgressListener> internalListener;
     mDownloadManager->GetInternalListener(getter_AddRefs(internalListener));
@@ -1282,34 +1346,13 @@ nsDownload::OnSecurityChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP
 nsDownload::Init(nsIURI* aSource,
                  nsIURI* aTarget,
-                 const PRUnichar* aDisplayName,
+                 const nsAString& aDisplayName,
                  nsIMIMEInfo *aMIMEInfo,
-                 PRInt64 aStartTime,
-                 nsIWebBrowserPersist* aPersist)
+                 PRTime aStartTime,
+                 nsILocalFile* aTempFile,
+                 nsICancelable* aCancelable)
 {
-  NS_WARNING("Huh...how did we get here?!");
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsDownload::SetDisplayName(const PRUnichar* aDisplayName)
-{
-  mDisplayName = aDisplayName;
-
-  nsCOMPtr<nsIRDFDataSource> ds;
-  mDownloadManager->GetDataSource(getter_AddRefs(ds));
-
-  nsCOMPtr<nsIRDFLiteral> nameLiteral;
-  nsCOMPtr<nsIRDFResource> res;
-  nsCAutoString path;
-  nsresult rv = GetFilePathUTF8(mTarget, path);
-  if (NS_FAILED(rv)) return rv;
-
-  gRDFService->GetResource(path, getter_AddRefs(res));
-  
-  gRDFService->GetLiteral(aDisplayName, getter_AddRefs(nameLiteral));
-  ds->Assert(res, gNC_Name, nameLiteral, PR_TRUE);
-
+  NS_NOTREACHED("Huh...how did we get here?!");
   return NS_OK;
 }
 
@@ -1320,6 +1363,14 @@ nsDownload::GetDisplayName(PRUnichar** aDisplayName)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDownload::GetCancelable(nsICancelable** aCancelable)
+{
+  *aCancelable = mCancelable;
+  NS_IF_ADDREF(*aCancelable);
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsDownload::GetTarget(nsIURI** aTarget)
 {
@@ -1336,14 +1387,6 @@ nsDownload::GetSource(nsIURI** aSource)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsDownload::GetPersist(nsIWebBrowserPersist** aPersist)
-{
-  *aPersist = mPersist;
-  NS_IF_ADDREF(*aPersist);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsDownload::GetStartTime(PRInt64* aStartTime)
 {
@@ -1359,32 +1402,16 @@ nsDownload::GetPercentComplete(PRInt32* aPercentComplete)
 }
 
 NS_IMETHODIMP
-nsDownload::SetListener(nsIWebProgressListener* aListener)
+nsDownload::GetAmountTransferred(PRUint64* aAmountTransferred)
 {
-  mListener = aListener;
+  *aAmountTransferred = mCurrBytes;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsDownload::GetListener(nsIWebProgressListener** aListener)
+nsDownload::GetSize(PRUint64* aSize)
 {
-  *aListener = mListener;
-  NS_IF_ADDREF(*aListener);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsDownload::SetObserver(nsIObserver* aObserver)
-{
-  mObserver = aObserver;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsDownload::GetObserver(nsIObserver** aObserver)
-{
-  *aObserver = mObserver;
-  NS_IF_ADDREF(*aObserver);
+  *aSize = mMaxBytes;
   return NS_OK;
 }
 
@@ -1410,3 +1437,17 @@ nsDownload::GetTargetFile(nsILocalFile** aTargetFile)
     rv = CallQueryInterface(file, aTargetFile);
   return rv;
 }
+
+NS_IMETHODIMP
+nsDownload::GetSpeed(double* aSpeed)
+{
+  *aSpeed = mSpeed;
+  return NS_OK;
+}
+
+nsresult
+nsDownload::SetTempFile(nsILocalFile* aTempFile)
+{
+  mTempFile = aTempFile;
+  return NS_OK;
+}
