@@ -49,10 +49,11 @@
 #include "nsICollation.h"
 #include "nsCollationCID.h"
 #include "nsLocaleCID.h"
+#include "nsIGenericFactory.h"
 #include "nsILocaleService.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-#include "nsIPrefBranchInternal.h"
+#include "nsIPrefBranch2.h"
 #include "nsIPrefLocalizedString.h"
 #include "nsICurrentCharsetListener.h"
 #include "nsQuickSort.h"
@@ -69,12 +70,9 @@
 // Global functions and data [declaration]
 
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
-static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 static NS_DEFINE_CID(kRDFInMemoryDataSourceCID, NS_RDFINMEMORYDATASOURCE_CID);
 static NS_DEFINE_CID(kRDFContainerUtilsCID, NS_RDFCONTAINERUTILS_CID);
 static NS_DEFINE_CID(kRDFContainerCID, NS_RDFCONTAINER_CID);
-static NS_DEFINE_CID(kCollationFactoryCID, NS_COLLATIONFACTORY_CID);
-static NS_DEFINE_CID(kLocaleServiceCID, NS_LOCALESERVICE_CID); 
 
 static const char kURINC_BrowserAutodetMenuRoot[] = "NC:BrowserAutodetMenuRoot";
 static const char kURINC_BrowserCharsetMenuRoot[] = "NC:BrowserCharsetMenuRoot";
@@ -142,8 +140,6 @@ public:
   nsAutoString      mTitle;
 };
 
-MOZ_DECL_CTOR_COUNTER(nsMenuEntry)
-
 //----------------------------------------------------------------------------
 // Class nsCharsetMenu [declaration]
 
@@ -506,7 +502,7 @@ nsCharsetMenu::nsCharsetMenu()
   nsresult res = NS_OK;
 
   //get charset manager
-  mCCManager = do_GetService(kCharsetConverterManagerCID, &res);
+  mCCManager = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &res);
 
   //initialize skeleton RDF source
   mRDFService = do_GetService(kRDFServiceCID, &res);
@@ -847,7 +843,7 @@ nsresult nsCharsetMenu::FreeResources()
   nsresult res = NS_OK;
 
   if (mCharsetMenuObserver) {
-    nsCOMPtr<nsIPrefBranchInternal> pbi = do_QueryInterface(mPrefs);
+    nsCOMPtr<nsIPrefBranch2> pbi = do_QueryInterface(mPrefs);
     if (pbi) {
       pbi->RemoveObserver(kBrowserStaticPrefKey, mCharsetMenuObserver);
       pbi->RemoveObserver(kMaileditPrefKey, mCharsetMenuObserver);
@@ -898,7 +894,7 @@ nsresult nsCharsetMenu::InitBrowserMenu()
     NS_ASSERTION(NS_SUCCEEDED(res), "error initializing browser cache charset menu");
 
     // register prefs callback
-    nsCOMPtr<nsIPrefBranchInternal> pbi = do_QueryInterface(mPrefs);
+    nsCOMPtr<nsIPrefBranch2> pbi = do_QueryInterface(mPrefs);
     if (pbi)
       res = pbi->AddObserver(kBrowserStaticPrefKey, mCharsetMenuObserver, PR_FALSE);
   }
@@ -938,7 +934,7 @@ nsresult nsCharsetMenu::InitMaileditMenu()
     NS_ASSERTION(NS_SUCCEEDED(res), "error initializing mailedit charset menu from prefs");
 
     // register prefs callback
-    nsCOMPtr<nsIPrefBranchInternal> pbi = do_QueryInterface(mPrefs);
+    nsCOMPtr<nsIPrefBranch2> pbi = do_QueryInterface(mPrefs);
     if (pbi)
       res = pbi->AddObserver(kMaileditPrefKey, mCharsetMenuObserver, PR_FALSE);
   }
@@ -1469,7 +1465,7 @@ nsresult nsCharsetMenu::AddFromPrefsToMenu(
   if (pls) {
     nsXPIDLString ucsval;
     pls->ToString(getter_Copies(ucsval));
-    NS_ConvertUCS2toUTF8 utf8val(ucsval);
+    NS_ConvertUTF16toUTF8 utf8val(ucsval);
     if (ucsval)
       res = AddFromStringToMenu(utf8val.BeginWriting(), aArray,
                                 aContainer, aDecs, aIDPrefix);
@@ -1808,13 +1804,13 @@ nsresult nsCharsetMenu::GetCollation(nsICollation ** aCollation)
   nsICollationFactory * collationFactory = nsnull;
   
   nsCOMPtr<nsILocaleService> localeServ = 
-           do_GetService(kLocaleServiceCID, &res);
+           do_GetService(NS_LOCALESERVICE_CONTRACTID, &res);
   if (NS_FAILED(res)) return res;
 
   res = localeServ->GetApplicationLocale(getter_AddRefs(locale));
   if (NS_FAILED(res)) return res;
 
-  res = CallCreateInstance(kCollationFactoryCID, &collationFactory);
+  res = CallCreateInstance(NS_COLLATIONFACTORY_CONTRACTID, &collationFactory);
   if (NS_FAILED(res)) return res;
 
   res = collationFactory->CreateCollation(locale, aCollation);
@@ -1831,7 +1827,7 @@ NS_IMETHODIMP nsCharsetMenu::SetCurrentCharset(const PRUnichar * aCharset)
   nsresult res = NS_OK;
 
   if (mBrowserMenuInitialized) {
-    res = AddCharsetToCache(NS_LossyConvertUCS2toASCII(aCharset),
+    res = AddCharsetToCache(NS_LossyConvertUTF16toASCII(aCharset),
                             &mBrowserMenu, kNC_BrowserCharsetMenuRoot, 
                             mBrowserCacheStart, mBrowserCacheSize,
                             mBrowserMenuRDFPosition);
@@ -1857,7 +1853,7 @@ NS_IMETHODIMP nsCharsetMenu::SetCurrentMailCharset(const PRUnichar * aCharset)
   nsresult res = NS_OK;
 
   if (mMailviewMenuInitialized) {
-    res = AddCharsetToCache(NS_LossyConvertUCS2toASCII(aCharset),
+    res = AddCharsetToCache(NS_LossyConvertUTF16toASCII(aCharset),
                             &mMailviewMenu, kNC_MailviewCharsetMenuRoot, 
                             mMailviewCacheStart, mMailviewCacheSize,
                             mMailviewMenuRDFPosition);
@@ -1881,7 +1877,7 @@ NS_IMETHODIMP nsCharsetMenu::SetCurrentComposerCharset(const PRUnichar * aCharse
 
   if (mComposerMenuInitialized) {
 
-    res = AddCharsetToCache(NS_LossyConvertUCS2toASCII(aCharset),
+    res = AddCharsetToCache(NS_LossyConvertUTF16toASCII(aCharset),
                             &mComposerMenu, kNC_ComposerCharsetMenuRoot, 
                             mComposerCacheStart, mComposerCacheSize,
                             mComposerMenuRDFPosition);
@@ -2062,3 +2058,13 @@ NS_IMETHODIMP nsCharsetMenu::EndUpdateBatch()
 {
   return mInner->EndUpdateBatch();
 }
+
+// Module definitions
+
+static const nsModuleComponentInfo components[] = {
+    { "nsCharsetMenu", NS_CHARSETMENU_CID,
+      NS_RDF_DATASOURCE_CONTRACTID_PREFIX NS_CHARSETMENU_PID,
+      NS_NewCharsetMenu },
+};
+
+NS_IMPL_NSGETMODULE(nsXPIntlModule, components)
