@@ -42,15 +42,12 @@
 #include "nsAppStartup.h"
 
 #include "nsIAppShellService.h"
-#include "nsICharsetConverterManager.h"
 #include "nsICloseAllWindows.h"
+#include "nsICmdLineService.h"
 #include "nsIDOMWindowInternal.h"
-#include "nsIEventQueue.h"
-#include "nsIEventQueueService.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsILocalFile.h"
 #include "nsIObserverService.h"
-#include "nsIPlatformCharset.h"
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
 #include "nsIProfileChangeStatus.h"
@@ -59,12 +56,13 @@
 #include "nsIStringBundle.h"
 #include "nsISupportsPrimitives.h"
 #include "nsITimelineService.h"
-#include "nsIUnicodeDecoder.h"
 #include "nsIWebBrowserChrome.h"
-#include "nsIWebShellWindow.h"
 #include "nsIWindowMediator.h"
 #include "nsIWindowWatcher.h"
 #include "nsIXULWindow.h"
+#include "nsNativeCharsetUtils.h"
+#include "nsThreadUtils.h"
+#include "nsAutoPtr.h"
 
 #include "prprf.h"
 #include "nsCRT.h"
@@ -72,12 +70,29 @@
 #include "nsWidgetsCID.h"
 #include "nsAppShellCID.h"
 #include "nsXPFEComponentsCID.h"
+#include "nsEmbedCID.h"
+
+class nsAppStartupExitEvent : public nsRunnable
+{
+public:
+  nsAppStartupExitEvent(nsAppStartup *service)
+    : mService(service) {}
+
+  NS_IMETHOD Run() {
+    // Tell the appshell to exit
+    mService->mAppShell->Exit();
+
+    // We're done "shutting down".
+    mService->mShuttingDown = PR_FALSE;
+    return NS_OK;
+  }
+
+private:
+  nsRefPtr<nsAppStartup> mService;
+};
 
 NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
 
-// Static Function Prototypes
-static nsresult ConvertToUnicode(nsCString& aCharset, const char* inString, nsAString& outString); 
-
 //
 // nsAppStartup
 //
@@ -106,14 +121,10 @@ NS_IMPL_ISUPPORTS5(nsAppStartup,
 //
 
 NS_IMETHODIMP
-nsAppStartup::Initialize(nsICmdLineService *aCmdLineService,
-                         nsISupports *aNativeAppSupportOrSplashScreen)
+nsAppStartup::Initialize(nsISupports *aNativeAppSupportOrSplashScreen)
 {
   nsresult rv;
 
-  // Remember cmd line service.
-  mCmdLineService = aCmdLineService;
-
   // Remember where the native app support lives.
   mNativeAppSupport = do_QueryInterface(aNativeAppSupportOrSplashScreen);
 
@@ -122,26 +133,13 @@ nsAppStartup::Initialize(nsICmdLineService *aCmdLineService,
     mSplashScreen = do_QueryInterface(aNativeAppSupportOrSplashScreen);
 
   // Create widget application shell
-  mAppShell = do_CreateInstance(kAppShellCID, &rv);
+  mAppShell = do_GetService(kAppShellCID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 argc = 0;
-  char** argv = nsnull;
-  aCmdLineService->GetArgc(&argc);
-  aCmdLineService->GetArgv(&argv);
-
-  rv = mAppShell->Create(&argc, argv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // listen to EventQueues' comings and goings. do this after the appshell
-  // has been created, but after the event queue has been created. that
-  // latter bit is unfortunate, but we deal with it.
   nsCOMPtr<nsIObserverService> os
     (do_GetService("@mozilla.org/observer-service;1", &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  os->AddObserver(this, "nsIEventQueueActivated", PR_TRUE);
-  os->AddObserver(this, "nsIEventQueueDestroyed", PR_TRUE);
   os->AddObserver(this, "skin-selected", PR_TRUE);
   os->AddObserver(this, "locale-selected", PR_TRUE);
   os->AddObserver(this, "xpinstall-restart", PR_TRUE);
@@ -369,34 +367,16 @@ nsAppStartup::Quit(PRUint32 aFerocity)
     // no matter what, make sure we send the exit event.  If
     // worst comes to worst, we'll do a leaky shutdown but we WILL
     // shut down. Well, assuming that all *this* stuff works ;-).
-    nsCOMPtr<nsIEventQueueService> svc = do_GetService(NS_EVENTQUEUESERVICE_CONTRACTID, &rv);
-    if (NS_SUCCEEDED(rv)) {
-
-      nsCOMPtr<nsIEventQueue> queue;
-      rv = svc->GetThreadEventQueue(NS_CURRENT_THREAD, getter_AddRefs(queue));
+    nsCOMPtr<nsIRunnable> event = new nsAppStartupExitEvent(this);
+    if (event) {
+      rv = NS_DispatchToCurrentThread(event);
       if (NS_SUCCEEDED(rv)) {
-
-        PLEvent* event = new PLEvent;
-        if (event) {
-          NS_ADDREF_THIS();
-          PL_InitEvent(event,
-                       this,
-                       HandleExitEvent,
-                       DestroyExitEvent);
-
-          rv = queue->PostEvent(event);
-          if (NS_SUCCEEDED(rv)) {
-            postedExitEvent = PR_TRUE;
-          }
-          else {
-            PL_DestroyEvent(event);
-          }
-        }
-        else {
-          rv = NS_ERROR_OUT_OF_MEMORY;
-        }
+        postedExitEvent = PR_TRUE;
       }
     }
+    else {
+      rv = NS_ERROR_OUT_OF_MEMORY;
+    }
   }
 
   // turn off the reentrancy check flag, but not if we have
@@ -527,7 +507,7 @@ nsAppStartup::CheckAndRemigrateDefunctProfile()
   rv = stringBundleService->CreateBundle("chrome://communicator/locale/profile/migration.properties", getter_AddRefs(migrationBundle));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  rv = stringBundleService->CreateBundle("chrome://global/locale/brand.properties", getter_AddRefs(brandBundle));
+  rv = stringBundleService->CreateBundle("chrome://branding/locale/brand.properties", getter_AddRefs(brandBundle));
   NS_ENSURE_SUCCESS(rv,rv);
 
   nsXPIDLString brandName;
@@ -538,7 +518,7 @@ nsAppStartup::CheckAndRemigrateDefunctProfile()
   rv = migrationBundle->GetStringFromName(NS_LITERAL_STRING("confirmRemigration").get(), getter_Copies(dialogText));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCOMPtr<nsIPromptService> promptService(do_GetService("@mozilla.org/embedcomp/prompt-service;1", &rv));
+  nsCOMPtr<nsIPromptService> promptService(do_GetService(NS_PROMPTSERVICE_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv,rv);
   PRInt32 buttonPressed;
   rv = promptService->ConfirmEx(nsnull, brandName.get(),
@@ -598,8 +578,25 @@ nsAppStartup::CreateStartupState(PRInt32 aWindowWidth, PRInt32 aWindowHeight,
                                  PRBool *_retval)
 {
   NS_ENSURE_ARG_POINTER(_retval);
+  *_retval = PR_FALSE;
   nsresult rv;
   
+  // If starting up in server mode, then we do things differently.
+  nsCOMPtr<nsINativeAppSupport> nativeApp;
+  rv = GetNativeAppSupport(getter_AddRefs(nativeApp));
+  if (NS_SUCCEEDED(rv)) {
+      PRBool isServerMode = PR_FALSE;
+      nativeApp->GetIsServerMode(&isServerMode);
+      if (isServerMode) {
+          nativeApp->StartServerMode();
+      }
+      PRBool shouldShowUI = PR_TRUE;
+      nativeApp->GetShouldShowUI(&shouldShowUI);
+      if (!shouldShowUI) {
+          return NS_OK;
+      }
+  }  
+
   nsCOMPtr<nsIPrefService> prefService(do_GetService(NS_PREFSERVICE_CONTRACTID));
   if (!prefService)
     return NS_ERROR_FAILURE;
@@ -616,8 +613,8 @@ nsAppStartup::CreateStartupState(PRInt32 aWindowWidth, PRInt32 aWindowHeight,
     
   for (PRUint32 i = 0; i < childCount; i++) {
     PRBool prefValue;
-    startupBranch->GetBoolPref(childArray[i], &prefValue);
-    if (prefValue) {
+    rv = startupBranch->GetBoolPref(childArray[i], &prefValue);
+    if (NS_SUCCEEDED(rv) && prefValue) {
       PRBool windowOpened;
       rv = LaunchTask(childArray[i], aWindowHeight, aWindowWidth, &windowOpened);
       if (NS_SUCCEEDED(rv) && windowOpened)
@@ -635,23 +632,7 @@ NS_IMETHODIMP
 nsAppStartup::Ensure1Window(nsICmdLineService *aCmdLineService)
 {
   nsresult rv;
-
-  // If starting up in server mode, then we do things differently.
-  nsCOMPtr<nsINativeAppSupport> nativeApp;
-  rv = GetNativeAppSupport(getter_AddRefs(nativeApp));
-  if (NS_SUCCEEDED(rv)) {
-      PRBool isServerMode = PR_FALSE;
-      nativeApp->GetIsServerMode(&isServerMode);
-      if (isServerMode) {
-          nativeApp->StartServerMode();
-      }
-      PRBool shouldShowUI = PR_TRUE;
-      nativeApp->GetShouldShowUI(&shouldShowUI);
-      if (!shouldShowUI) {
-          return NS_OK;
-      }
-  }
-  
+ 
   nsCOMPtr<nsIWindowMediator> windowMediator(do_GetService(NS_WINDOWMEDIATOR_CONTRACTID, &rv));
   if (NS_FAILED(rv))
     return rv;
@@ -684,32 +665,6 @@ nsAppStartup::Ensure1Window(nsICmdLineService *aCmdLineService)
   return rv;
 }
 
-
-void* PR_CALLBACK
-nsAppStartup::HandleExitEvent(PLEvent* aEvent)
-{
-  nsAppStartup *service =
-    NS_REINTERPRET_CAST(nsAppStartup*, aEvent->owner);
-
-  // Tell the appshell to exit
-  service->mAppShell->Exit();
-
-  // We're done "shutting down".
-  service->mShuttingDown = PR_FALSE;
-
-  return nsnull;
-}
-
-void PR_CALLBACK
-nsAppStartup::DestroyExitEvent(PLEvent* aEvent)
-{
-  nsAppStartup *service =
-    NS_REINTERPRET_CAST(nsAppStartup*, aEvent->owner);
-  NS_RELEASE(service);
-  delete aEvent;
-}
-
-
 nsresult
 nsAppStartup::LaunchTask(const char *aParam, PRInt32 height, PRInt32 width, PRBool *windowOpened)
 {
@@ -819,8 +774,6 @@ nsAppStartup::OpenBrowserWindow(PRInt32 height, PRInt32 width)
 
   nsXPIDLCString urlToLoad;
   rv = cmdLine->GetURLToLoad(getter_Copies(urlToLoad));
-  if (NS_FAILED(rv)) return rv;
-
   if (!urlToLoad.IsEmpty()) {
 
 #ifdef DEBUG_CMD_LINE
@@ -828,30 +781,11 @@ nsAppStartup::OpenBrowserWindow(PRInt32 height, PRInt32 width)
 #endif /* DEBUG_CMD_LINE */
 
     nsAutoString url; 
-    if (nsCRT::IsAscii(urlToLoad))  {
-      CopyASCIItoUTF16(urlToLoad, url);
-    }
-    else {
-      // get a platform charset
-      nsCAutoString charSet;
-      nsCOMPtr <nsIPlatformCharset> platformCharset(do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv));
-      if (NS_FAILED(rv)) {
-        NS_ERROR("Failed to get a platform charset");
-        return rv;
-      }
-
-      rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, charSet);
-      if (NS_FAILED(rv)) {
-        NS_ERROR("Failed to get a charset");
-        return rv;
-      }
-
-      // convert the cmdLine URL to Unicode
-      rv = ConvertToUnicode(charSet, urlToLoad, url);
-      if (NS_FAILED(rv)) {
-        NS_ASSERTION(0, "Failed to convert commandline url to unicode");
-        return rv;
-      }
+    // convert the cmdLine URL to Unicode
+    rv = NS_CopyNativeToUnicode(nsDependentCString(urlToLoad), url);
+    if (NS_FAILED(rv)) {
+      NS_ERROR("Failed to convert commandline url to unicode");
+      return rv;
     }
     rv = OpenWindow(chromeUrlForTask, url, width, height);
 
@@ -924,9 +858,10 @@ nsAppStartup::CreateChromeWindow2(nsIWebBrowserChrome *aParent,
     if (!appShell)
       return NS_ERROR_FAILURE;
     
-    appShell->CreateTopLevelWindow(0, 0, PR_FALSE, PR_FALSE,
-      aChromeFlags, nsIAppShellService::SIZE_TO_CONTENT,
-      nsIAppShellService::SIZE_TO_CONTENT, mAppShell, getter_AddRefs(newWindow));
+    appShell->CreateTopLevelWindow(0, 0, aChromeFlags,
+                                   nsIAppShellService::SIZE_TO_CONTENT,
+                                   nsIAppShellService::SIZE_TO_CONTENT,
+                                   mAppShell, getter_AddRefs(newWindow));
   }
 
   // if anybody gave us anything to work with, use it
@@ -950,27 +885,9 @@ nsAppStartup::Observe(nsISupports *aSubject,
                       const char *aTopic, const PRUnichar *aData)
 {
   NS_ASSERTION(mAppShell, "appshell service notified before appshell built");
-  if (!strcmp(aTopic, "nsIEventQueueActivated")) {
-    nsCOMPtr<nsIEventQueue> eq(do_QueryInterface(aSubject));
-    if (eq) {
-      PRBool isNative = PR_TRUE;
-      // we only add native event queues to the appshell
-      eq->IsQueueNative(&isNative);
-      if (isNative)
-        mAppShell->ListenToEventQueue(eq, PR_TRUE);
-    }
-  } else if (!strcmp(aTopic, "nsIEventQueueDestroyed")) {
-    nsCOMPtr<nsIEventQueue> eq(do_QueryInterface(aSubject));
-    if (eq) {
-      PRBool isNative = PR_TRUE;
-      // we only remove native event queues from the appshell
-      eq->IsQueueNative(&isNative);
-      if (isNative)
-        mAppShell->ListenToEventQueue(eq, PR_FALSE);
-    }
-  } else if (!strcmp(aTopic, "skin-selected") ||
-             !strcmp(aTopic, "locale-selected") ||
-             !strcmp(aTopic, "xpinstall-restart")) {
+  if (!strcmp(aTopic, "skin-selected") ||
+      !strcmp(aTopic, "locale-selected") ||
+      !strcmp(aTopic, "xpinstall-restart")) {
     if (mNativeAppSupport)
       mNativeAppSupport->SetIsServerMode(PR_FALSE);
   } else if (!strcmp(aTopic, "profile-change-teardown")) {
@@ -1018,37 +935,3 @@ nsAppStartup::Observe(nsISupports *aSubject,
   return NS_OK;
 }
 
-static nsresult
-ConvertToUnicode(nsCString& aCharset, const char* inString, nsAString& outString)
-{
-  nsresult rv;
-
-  // convert result to unicode
-  nsCOMPtr<nsICharsetConverterManager> ccm(do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID , &rv));
-  if (NS_FAILED(rv))
-    return rv;
-
-  nsCOMPtr <nsIUnicodeDecoder> decoder; 
-  rv = ccm->GetUnicodeDecoderRaw(aCharset.get(), getter_AddRefs(decoder));
-  if (NS_FAILED(rv))
-    return rv;
-
-  PRInt32 uniLength = 0;
-  PRInt32 srcLength = strlen(inString);
-  rv = decoder->GetMaxLength(inString, srcLength, &uniLength);
-  if (NS_FAILED(rv))
-    return rv;
-
-  outString.SetLength(uniLength);
-  nsWritingIterator<PRUnichar> unichars;
-  outString.BeginWriting(unichars);
-
-  // convert to unicode
-  rv = decoder->Convert(inString, &srcLength, unichars.get(), &uniLength);
-  if (NS_SUCCEEDED(rv)) {
-    // Pass back the unicode string
-    outString.Assign(unichars.get(), uniLength);
-  }
-
-  return rv;
-}
