@@ -51,12 +51,10 @@
 #include "nsIPromptService.h"
 #include "nsIStringBundle.h"
 #include "nsIAllocator.h"
-#include "nsICmdLineService.h"
 #include "nsXPIDLString.h"
 #include "nsString.h"
 #include "nsMemory.h"
 #include "nsNetUtil.h"
-#include "nsWindowsHooksUtil.cpp"
 #include "nsWindowsHooks.h"
 #include <windows.h>
 #include <shlobj.h>
@@ -73,44 +71,19 @@
 #include "imgIRequest.h"
 #include "imgIContainer.h"
 #include "gfxIImageFrame.h"
+#include "nsDirectoryServiceUtils.h"
+#include "nsAppDirectoryServiceDefs.h"
+
+#ifndef MOZ_XUL_APP
+#include "nsICmdLineService.h"
+#endif
 
 #include "nsXPFEComponentsCID.h"
+#include "nsEmbedCID.h"
 
 #define RUNKEY "Software\\Microsoft\\Windows\\CurrentVersion\\Run"
 
-// Objects that describe the Windows registry entries that we need to tweak.
-static ProtocolRegistryEntry
-    http( "http" ),
-    https( "https" ),
-    ftp( "ftp" ),
-    chrome( "chrome" ),
-    gopher( "gopher" );
-const char *jpgExts[]  = { ".jpg", ".jpe", ".jpeg", ".jfif", ".pjpeg", ".pjp", 0 };
-const char *gifExts[]  = { ".gif", 0 };
-const char *pngExts[]  = { ".png", 0 };
-const char *mngExts[]  = { ".mng", 0 };
-const char *xbmExts[]  = { ".xbm", 0 };
-const char *bmpExts[]  = { ".bmp", ".rle", ".dib", 0 };
-const char *icoExts[]  = { ".ico", 0 };
-const char *xmlExts[]  = { ".xml", 0 };
-const char *xhtmExts[] = { ".xht", ".xhtml", 0 };
-const char *xulExts[]  = { ".xul", 0 };
-const char *htmExts[]  = { ".htm", ".html", ".shtml", 0 };
-
-static FileTypeRegistryEntry
-    jpg(   jpgExts,  "MozillaJPEG",  "JPEG Image",           "jpegfile", "jpeg-file.ico"),
-    gif(   gifExts,  "MozillaGIF",   "GIF Image",            "giffile",  "gif-file.ico"),
-    png(   pngExts,  "MozillaPNG",   "PNG Image",            "pngfile",  "image-file.ico"),
-    mng(   mngExts,  "MozillaMNG",   "MNG Image",            "",         "image-file.ico"),
-    xbm(   xbmExts,  "MozillaXBM",   "XBM Image",            "xbmfile",  "image-file.ico"),
-    bmp(   bmpExts,  "MozillaBMP",   "BMP Image",            "",         "image-file.ico"),
-    ico(   icoExts,  "MozillaICO",   "Icon",                 "icofile",  "%1"),
-    xml(   xmlExts,  "MozillaXML",   "XML Document",         "xmlfile",  "xml-file.ico"),
-    xhtml( xhtmExts, "MozillaXHTML", "XHTML Document",       "",         "misc-file.ico"),
-    xul(   xulExts,  "MozillaXUL",   "Mozilla XUL Document", "",         "xul-file.ico");
-
-static EditableFileTypeRegistryEntry
-    mozillaMarkup( htmExts, "MozillaHTML", "HTML Document", "htmlfile",  "html-file.ico");
+static nsWindowsHooks *gWindowsHooks = nsnull;
 
 // Implementation of the nsIWindowsHooksSettings interface.
 // Use standard implementation of nsISupports stuff.
@@ -173,12 +146,45 @@ DEFINE_GETTER_AND_SETTER( HaveBeenSet,      mHaveBeenSet  )
 
 // Implementation of the nsIWindowsHooks interface.
 // Use standard implementation of nsISupports stuff.
-NS_IMPL_ISUPPORTS2( nsWindowsHooks, nsIWindowsHooks, nsIWindowsRegistry )
+NS_IMPL_ISUPPORTS1( nsWindowsHooks, nsIWindowsHooks )
 
-nsWindowsHooks::nsWindowsHooks() {
+static const char *jpgExts[]  = { ".jpg", ".jpe", ".jpeg", ".jfif", ".pjpeg", ".pjp", 0 };
+static const char *gifExts[]  = { ".gif", 0 };
+static const char *pngExts[]  = { ".png", 0 };
+static const char *mngExts[]  = { ".mng", 0 };
+static const char *xbmExts[]  = { ".xbm", 0 };
+static const char *bmpExts[]  = { ".bmp", ".rle", ".dib", 0 };
+static const char *icoExts[]  = { ".ico", 0 };
+static const char *xmlExts[]  = { ".xml", 0 };
+static const char *xhtmExts[] = { ".xht", ".xhtml", 0 };
+static const char *xulExts[]  = { ".xul", 0 };
+static const char *htmExts[]  = { ".htm", ".html", ".shtml", 0 };
+
+nsWindowsHooks::nsWindowsHooks()
+  : http( "http" ),
+    https( "https" ),
+    ftp( "ftp" ),
+    chrome( "chrome" ),
+    gopher( "gopher" ),
+
+    jpg(   jpgExts,  "MozillaJPEG",  "JPEG Image",           "jpegfile", "jpeg-file.ico"),
+    gif(   gifExts,  "MozillaGIF",   "GIF Image",            "giffile",  "gif-file.ico"),
+    png(   pngExts,  "MozillaPNG",   "PNG Image",            "pngfile",  "image-file.ico"),
+    mng(   mngExts,  "MozillaMNG",   "MNG Image",            "",         "image-file.ico"),
+    xbm(   xbmExts,  "MozillaXBM",   "XBM Image",            "xbmfile",  "image-file.ico"),
+    bmp(   bmpExts,  "MozillaBMP",   "BMP Image",            "",         "image-file.ico"),
+    ico(   icoExts,  "MozillaICO",   "Icon",                 "icofile",  "%1"),
+    xml(   xmlExts,  "MozillaXML",   "XML Document",         "xmlfile",  "xml-file.ico"),
+    xhtml( xhtmExts, "MozillaXHTML", "XHTML Document",       "",         "misc-file.ico"),
+    xul(   xulExts,  "MozillaXUL",   "Mozilla XUL Document", "",         "xul-file.ico"),
+
+    mozillaMarkup( htmExts, "MozillaHTML", "HTML Document", "htmlfile",  "html-file.ico")
+{
+    gWindowsHooks = this;
 }
 
 nsWindowsHooks::~nsWindowsHooks() {
+    gWindowsHooks = nsnull;
 }
 
 // Internal GetPreferences.
@@ -217,7 +223,7 @@ nsWindowsHooks::GetSettings( nsWindowsHooksSettings **result ) {
     prefs->mHaveBeenSet  = BoolRegistryEntry( "haveBeenSet"      );
 
 #ifdef DEBUG_law
-NS_WARN_IF_FALSE( NS_SUCCEEDED( rv ), "GetPreferences failed" );
+NS_ASSERTION( NS_SUCCEEDED( rv ), "GetPreferences failed" );
 #endif
 
     return rv;
@@ -299,37 +305,37 @@ nsWindowsHooksSettings::GetRegistryMatches( PRBool *_retval ) {
     NS_ENSURE_ARG( _retval );
     *_retval = PR_TRUE;
     // Test registry for all selected attributes.
-    if ( misMatch( mHandleHTTP,   http )
+    if ( misMatch( mHandleHTTP,   gWindowsHooks->http )
          ||
-         misMatch( mHandleHTTPS,  https )
+         misMatch( mHandleHTTPS,  gWindowsHooks->https )
          ||
-         misMatch( mHandleFTP,    ftp )
+         misMatch( mHandleFTP,    gWindowsHooks->ftp )
          ||
-         misMatch( mHandleCHROME, chrome )
+         misMatch( mHandleCHROME, gWindowsHooks->chrome )
          ||
-         misMatch( mHandleGOPHER, gopher )
+         misMatch( mHandleGOPHER, gWindowsHooks->gopher )
          ||
-         misMatch( mHandleHTML,   mozillaMarkup )
+         misMatch( mHandleHTML,   gWindowsHooks->mozillaMarkup )
          ||
-         misMatch( mHandleJPEG,   jpg )
+         misMatch( mHandleJPEG,   gWindowsHooks->jpg )
          ||
-         misMatch( mHandleGIF,    gif )
+         misMatch( mHandleGIF,    gWindowsHooks->gif )
          ||
-         misMatch( mHandlePNG,    png )
+         misMatch( mHandlePNG,    gWindowsHooks->png )
          ||
-         misMatch( mHandleMNG,    mng )
+         misMatch( mHandleMNG,    gWindowsHooks->mng )
          ||
-         misMatch( mHandleXBM,    xbm )
+         misMatch( mHandleXBM,    gWindowsHooks->xbm )
          ||
-         misMatch( mHandleBMP,    bmp )
+         misMatch( mHandleBMP,    gWindowsHooks->bmp )
          ||
-         misMatch( mHandleICO,    ico )
+         misMatch( mHandleICO,    gWindowsHooks->ico )
          ||
-         misMatch( mHandleXML,    xml )
+         misMatch( mHandleXML,    gWindowsHooks->xml )
          ||
-         misMatch( mHandleXHTML,  xhtml )
+         misMatch( mHandleXHTML,  gWindowsHooks->xhtml )
          ||
-         misMatch( mHandleXUL,    xul ) ) {
+         misMatch( mHandleXUL,    gWindowsHooks->xul ) ) {
         // Registry is out of synch.
         *_retval = PR_FALSE;
     }
@@ -370,22 +376,23 @@ nsWindowsHooks::CheckSettings( nsIDOMWindowInternal *aParent,
             settings->mHandleCHROME = PR_TRUE;
             settings->mHandleGOPHER = PR_TRUE;
             settings->mHandleHTML   = PR_TRUE;
-            settings->mHandleJPEG   = PR_TRUE;
-            settings->mHandleGIF    = PR_TRUE;
-            settings->mHandlePNG    = PR_TRUE;
-            settings->mHandleMNG    = PR_TRUE;
-            settings->mHandleXBM    = PR_TRUE;
+            settings->mHandleJPEG   = PR_FALSE;
+            settings->mHandleGIF    = PR_FALSE;
+            settings->mHandlePNG    = PR_FALSE;
+            settings->mHandleMNG    = PR_FALSE;
+            settings->mHandleXBM    = PR_FALSE;
             settings->mHandleBMP    = PR_FALSE;
             settings->mHandleICO    = PR_FALSE;
             settings->mHandleXML    = PR_TRUE;
             settings->mHandleXHTML  = PR_TRUE;
             settings->mHandleXUL    = PR_TRUE;
 
-            settings->mShowDialog       = PR_TRUE;
+            settings->mShowDialog   = PR_TRUE;
         }
 
         // If launched with "-installer" then override mShowDialog.
         PRBool installing = PR_FALSE;
+#ifndef MOZ_XUL_APP
         if ( !settings->mShowDialog ) {
             // Get command line service.
             nsCOMPtr<nsICmdLineService> cmdLineArgs
@@ -399,6 +406,7 @@ nsWindowsHooks::CheckSettings( nsIDOMWindowInternal *aParent,
                 }
             }
         }
+#endif
 
         // First, make sure the user cares.
         if ( settings->mShowDialog || installing ) {
@@ -411,9 +419,8 @@ nsWindowsHooks::CheckSettings( nsIDOMWindowInternal *aParent,
                 //   o We need the common dialog service to show the dialog.
                 //   o We need the string bundle service to fetch the appropriate
                 //     dialog text.
-                nsCID bundleCID = NS_STRINGBUNDLESERVICE_CID;
-                nsCOMPtr<nsIPromptService> promptService( do_GetService("@mozilla.org/embedcomp/prompt-service;1"));
-                nsCOMPtr<nsIStringBundleService> bundleService( do_GetService( bundleCID, &rv ) );
+                nsCOMPtr<nsIPromptService> promptService( do_GetService(NS_PROMPTSERVICE_CONTRACTID));
+                nsCOMPtr<nsIStringBundleService> bundleService( do_GetService( NS_STRINGBUNDLE_CONTRACTID, &rv ) );
 
                 if ( promptService && bundleService ) {
                     // Next, get bundle that provides text for dialog.
@@ -421,7 +428,7 @@ nsWindowsHooks::CheckSettings( nsIDOMWindowInternal *aParent,
                     nsCOMPtr<nsIStringBundle> brandBundle;
                     rv = bundleService->CreateBundle( "chrome://global-platform/locale/nsWindowsHooks.properties",
                                                       getter_AddRefs( bundle ) );
-                    rv = bundleService->CreateBundle( "chrome://global/locale/brand.properties",
+                    rv = bundleService->CreateBundle( "chrome://branding/locale/brand.properties",
                                                       getter_AddRefs( brandBundle ) );
                     if ( NS_SUCCEEDED( rv ) && bundle && brandBundle ) {
                         nsXPIDLString text, label, shortName;
@@ -677,40 +684,6 @@ nsWindowsHooks::SetRegistry() {
     return NS_OK;
 }
 
-NS_IMETHODIMP nsWindowsHooks::GetRegistryEntry( PRInt32 aHKEYConstant, const char *aSubKeyName, const char *aValueName, char **aResult ) {
-    NS_ENSURE_ARG( aResult );
-    *aResult = 0;
-    // Calculate HKEY_* starting point based on input nsIWindowsHooks constant.
-    HKEY hKey;
-    switch ( aHKEYConstant ) {
-        case HKCR:
-            hKey = HKEY_CLASSES_ROOT;
-            break;
-        case HKCC:
-            hKey = HKEY_CURRENT_CONFIG;
-            break;
-        case HKCU:
-            hKey = HKEY_CURRENT_USER;
-            break;
-        case HKLM:
-            hKey = HKEY_LOCAL_MACHINE;
-            break;
-        case HKU:
-            hKey = HKEY_USERS;
-            break;
-        default:
-            return NS_ERROR_INVALID_ARG;
-    }
-
-    // Get requested registry entry.
-    nsCAutoString entry( RegistryEntry( hKey, aSubKeyName, aValueName, 0 ).currentSetting() );
-
-    // Copy to result.
-    *aResult = PL_strdup( entry.get() );
-
-    return *aResult ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
-}
-
 // nsIWindowsHooks.idl for documentation
 
 /*
@@ -866,7 +839,7 @@ NS_IMETHODIMP nsWindowsHooks::StartupRemoveOption(const char* option) {
 }
 
 nsresult
-WriteBitmap(nsString& aPath, gfxIImageFrame* aImage)
+WriteBitmap(nsIFile* aFile, gfxIImageFrame* aImage)
 {
   PRInt32 width, height;
   aImage->GetWidth(&width);
@@ -874,45 +847,44 @@ WriteBitmap(nsString& aPath, gfxIImageFrame* aImage)
   
   PRUint8* bits;
   PRUint32 length;
+  aImage->LockImageData();
   aImage->GetImageData(&bits, &length);
-  if (!bits) return NS_ERROR_FAILURE;
-  
+  if (!bits) {
+      aImage->UnlockImageData();
+      return NS_ERROR_FAILURE;
+  }
+ 
   PRUint32 bpr;
   aImage->GetImageBytesPerRow(&bpr);
   PRInt32 bitCount = bpr/width;
   
   // initialize these bitmap structs which we will later
   // serialize directly to the head of the bitmap file
-  LPBITMAPINFOHEADER bmi = (LPBITMAPINFOHEADER)new BITMAPINFO;
-  bmi->biSize = sizeof(BITMAPINFOHEADER);
-  bmi->biWidth = width;
-  bmi->biHeight = height;
-  bmi->biPlanes = 1;
-  bmi->biBitCount = (WORD)bitCount*8;
-  bmi->biCompression = BI_RGB;
-  bmi->biSizeImage = 0; // don't need to set this if bmp is uncompressed
-  bmi->biXPelsPerMeter = 0;
-  bmi->biYPelsPerMeter = 0;
-  bmi->biClrUsed = 0;
-  bmi->biClrImportant = 0;
+  BITMAPINFOHEADER bmi;
+  bmi.biSize = sizeof(BITMAPINFOHEADER);
+  bmi.biWidth = width;
+  bmi.biHeight = height;
+  bmi.biPlanes = 1;
+  bmi.biBitCount = (WORD)bitCount*8;
+  bmi.biCompression = BI_RGB;
+  bmi.biSizeImage = length;
+  bmi.biXPelsPerMeter = 0;
+  bmi.biYPelsPerMeter = 0;
+  bmi.biClrUsed = 0;
+  bmi.biClrImportant = 0;
   
   BITMAPFILEHEADER bf;
   bf.bfType = 0x4D42; // 'BM'
   bf.bfReserved1 = 0;
   bf.bfReserved2 = 0;
   bf.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
-  bf.bfSize = bf.bfOffBits + bmi->biSizeImage;
+  bf.bfSize = bf.bfOffBits + bmi.biSizeImage;
 
   // get a file output stream
   nsresult rv;
-  nsCOMPtr<nsILocalFile> path;
-  rv = NS_NewLocalFile(aPath, PR_TRUE, getter_AddRefs(path));
-  
-  if (NS_FAILED(rv))
-    return rv;
-
   nsCOMPtr<nsIOutputStream> stream;
-  NS_NewLocalFileOutputStream(getter_AddRefs(stream), path);
+  rv = NS_NewLocalFileOutputStream(getter_AddRefs(stream), aFile);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // write the bitmap headers and rgb pixel data to the file
   rv = NS_ERROR_FAILURE;
@@ -920,17 +892,34 @@ WriteBitmap(nsString& aPath, gfxIImageFrame* aImage)
     PRUint32 written;
     stream->Write((const char*)&bf, sizeof(BITMAPFILEHEADER), &written);
     if (written == sizeof(BITMAPFILEHEADER)) {
-      stream->Write((const char*)bmi, sizeof(BITMAPINFOHEADER), &written);
+      stream->Write((const char*)&bmi, sizeof(BITMAPINFOHEADER), &written);
       if (written == sizeof(BITMAPINFOHEADER)) {
+#ifndef MOZ_CAIRO_GFX
         stream->Write((const char*)bits, length, &written);
         if (written == length)
           rv = NS_OK;
+#else
+        // write out the image data backwards because the desktop won't
+        // show bitmaps with negative heights for top-to-bottom
+        PRUint32 i = length;
+        rv = NS_OK;
+        do {
+          i -= bpr;
+
+          stream->Write(((const char*)bits) + i, bpr, &written);
+          if (written != bpr) {
+            rv = NS_ERROR_FAILURE;
+            break;
+          }
+        } while (i != 0);
+#endif
       }
     }
   
     stream->Close();
   }
-  
+
+  aImage->UnlockImageData(); 
   return rv;
 }
 
@@ -963,38 +952,56 @@ nsWindowsHooks::SetImageAsWallpaper(nsIDOMElement* aElement, PRBool aUseBackgrou
   if (!gfxFrame)
     return NS_ERROR_FAILURE;
 
-  // get the windows directory ('c:\windows' usually)
-  char winDir[256];
-  ::GetWindowsDirectory(winDir, sizeof(winDir));
-  nsAutoString winPath;
-  winPath.AssignWithConversion(winDir);
+  // get the profile root directory
+  nsCOMPtr<nsIFile> file;
+  rv = NS_GetSpecialDirectory(NS_APP_APPLICATION_REGISTRY_DIR,
+                              getter_AddRefs(file));
+  NS_ENSURE_SUCCESS(rv, rv);
   
   // get the product brand name from localized strings
-  nsXPIDLString brandName;
-  nsCID bundleCID = NS_STRINGBUNDLESERVICE_CID;
-  nsCOMPtr<nsIStringBundleService> bundleService(do_GetService(bundleCID));
-  if (bundleService) {
-    nsCOMPtr<nsIStringBundle> brandBundle;
-    rv = bundleService->CreateBundle("chrome://global/locale/brand.properties",
-                                     getter_AddRefs(brandBundle));
-    if (NS_SUCCEEDED(rv) && brandBundle) {
-      if (NS_FAILED(rv = brandBundle->GetStringFromName(NS_LITERAL_STRING("brandShortName").get(),
-                            getter_Copies(brandName))))
-        return rv;                              
-    }
-  }
-  
-  // build the file name
-  winPath.Append(NS_LITERAL_STRING("\\").get());
-  winPath.Append(brandName);
-  winPath.Append(NS_LITERAL_STRING(" Wallpaper.bmp").get());
-  
-  // write the bitmap to a file in the windows dir
-  rv = WriteBitmap(winPath, gfxFrame);
+  nsXPIDLString brandName, fileLeafName;
+  nsCOMPtr<nsIStringBundleService> bundleService =
+           do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
+  NS_ENSURE_TRUE(bundleService, rv);
 
-  // if the file was written successfully, set it as the system wallpaper
-  if (NS_SUCCEEDED(rv))
-    ::SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, ToNewCString(winPath), SPIF_UPDATEINIFILE | SPIF_SENDWININICHANGE);
+  // get the product brand name from localized strings
+  nsCOMPtr<nsIStringBundle> bundle;
+  rv = bundleService->CreateBundle("chrome://branding/locale/brand.properties",
+                                   getter_AddRefs(bundle));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = bundle->GetStringFromName(NS_LITERAL_STRING("brandShortName").get(),
+                                 getter_Copies(brandName));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // get the file leaf name from localized strings (e.g. "%S Wallpaper.bmp")
+  rv = bundleService->CreateBundle("chrome://global-platform/locale/nsWindowsHooks.properties",
+                                   getter_AddRefs(bundle));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  const PRUnichar* stringArray[] = { brandName.get() };
+
+  rv = bundle->FormatStringFromName(NS_LITERAL_STRING("wallpaperFile").get(),
+                                    stringArray, NS_ARRAY_LENGTH(stringArray),
+                                    getter_Copies(fileLeafName));
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  // eventually, the path is %APPDATA%\Mozilla\Mozilla Wallpaper.bmp
+  fileLeafName.AppendLiteral(".bmp");
+  rv = file->Append(fileLeafName);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // write the bitmap to the target file
+  rv = WriteBitmap(file, gfxFrame);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // set it as the system wallpaper
+  nsCAutoString nativePath;
+  rv = file->GetNativePath(nativePath);
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  ::SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, (PVOID) nativePath.get(),
+                         SPIF_UPDATEINIFILE | SPIF_SENDWININICHANGE);
 
   return rv;
 }
