@@ -45,60 +45,18 @@
 #include "nsDirectoryService.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsNativeCharsetUtils.h"
+#include "nsWindowsHooksUtil.h"
 
 #ifdef MOZ_XUL_APP
 #include "nsNativeAppSupportWin.h"
 #else
 #include "nsINativeAppSupportWin.h"
+#include "nsICmdLineHandler.h"
 #endif
 
 #define MOZ_HWND_BROADCAST_MSG_TIMEOUT 5000
 #define MOZ_CLIENT_BROWSER_KEY "Software\\Clients\\StartMenuInternet"
 
-// Where Mozilla stores its own registry values.
-const char * const mozillaKeyName = "Software\\Mozilla\\Desktop";
-
-static const char shortcutSuffix[] = " -url \"%1\"";
-static const char chromeSuffix[] = " -chrome \"%1\"";
-static const char iconSuffix[] = ",0";
-
-// Returns the (fully-qualified) name of this executable.
-static nsCString thisApplication() {
-    static nsCAutoString result;
-
-    if ( result.IsEmpty() ) {
-        char buffer[MAX_PATH] = { 0 };
-    	DWORD len = ::GetModuleFileName( NULL, buffer, sizeof buffer );
-        len = ::GetShortPathName( buffer, buffer, sizeof buffer );
-    
-        result = buffer;
-        ToUpperCase(result);
-    }
-
-    return result;
-}
-
-// Returns the "short" name of this application (in upper case).  This is for
-// use as a StartMenuInternet value.
-static nsCString shortAppName() {
-    static nsCAutoString result;
-    
-    if ( result.IsEmpty() ) { 
-        // Find last backslash in thisApplication().
-        nsCAutoString thisApp( thisApplication() );
-        PRInt32 n = thisApp.RFind( "\\" );
-        if ( n != kNotFound ) {
-            // Use what comes after the last backslash.
-            result = (const char*)thisApp.get() + n + 1;
-        } else {
-            // Use entire string.
-            result = thisApp;
-        }
-    }
-
-    return result;
-}
-
 static PRBool IsNT()
 {
     static PRBool sInitialized = PR_FALSE;
@@ -117,186 +75,6 @@ static PRBool IsNT()
     return sIsNT;
 }
 
-// RegistryEntry
-//
-// Generic registry entry (no saving of previous values).  Each is comprised of:
-//      o A base HKEY
-//      o A subkey name.
-//      o An optional value name (empty for the "default" value).
-//      o The registry setting we'd like this entry to have when set.
-struct RegistryEntry {
-    HKEY        baseKey;   // e.g., HKEY_CURRENT_USER
-    PRBool      isNull;    // i.e., should use ::RegDeleteValue
-    nsCString   keyName;   // Key name.
-    nsCString   valueName; // Value name (can be empty, which implies NULL).
-    nsCString   setting;   // What we set it to (UTF-8). This had better be 
-                           // nsString to avoid extra string copies, but
-                           // this code is not performance-critical. 
-
-    RegistryEntry( HKEY baseKey, const char* keyName, const char* valueName, const char* setting )
-        : baseKey( baseKey ), isNull( setting == 0 ), keyName( keyName ), valueName( valueName ), setting( setting ? setting : "" ) {
-    }
-
-    PRBool     isAlreadySet() const;
-    nsresult   set();
-    nsresult   reset();
-    nsCString  currentSetting( PRBool *currentUndefined = 0 ) const;
-
-    // Return value name in proper form for passing to ::Reg functions
-    // (i.e., emptry string is converted to a NULL pointer).
-    const char* valueNameArg() const {
-        return valueName.IsEmpty() ? NULL : valueName.get();
-    }
-
-    nsCString  fullName() const;
-};
-
-// BoolRegistryEntry
-// 
-// These are used to store the "windows integration" preferences.
-// You can query the value via operator void* (i.e., if ( boolPref )... ).
-// These are stored under HKEY_LOCAL_MACHINE\Software\Mozilla\Desktop.
-// Set sets the stored value to "1".  Reset deletes it (which implies 0).
-struct BoolRegistryEntry : public RegistryEntry {
-    BoolRegistryEntry( const char *name )
-        : RegistryEntry( HKEY_LOCAL_MACHINE, mozillaKeyName, name, "1" ) {
-    }
-    operator PRBool();
-};
-
-// SavedRegistryEntry
-//
-// Like a plain RegistryEntry, but set/reset save/restore the
-// it had before we set it.
-struct SavedRegistryEntry : public RegistryEntry {
-    SavedRegistryEntry( HKEY baseKey, const char *keyName, const char *valueName, const char *setting )
-        : RegistryEntry( baseKey, keyName, valueName, setting ) {
-    }
-    nsresult set();
-    nsresult reset();
-};
-
-// ProtocolRegistryEntry
-//
-// For setting entries for a given Internet Shortcut protocol.
-// The key name is calculated as
-// HKEY_LOCAL_MACHINE\Software\Classes\protocol\shell\open\command.
-// The setting is this executable (with appropriate suffix).
-// Set/reset are trickier in this case.
-struct ProtocolRegistryEntry : public SavedRegistryEntry {
-    nsCString protocol;
-    ProtocolRegistryEntry( const char* protocol )
-        : SavedRegistryEntry( HKEY_LOCAL_MACHINE, "", "", thisApplication().get() ),
-          protocol( protocol ) {
-        keyName = "Software\\Classes\\";
-        keyName += protocol;
-        keyName += "\\shell\\open\\command";
-
-        // Append appropriate suffix to setting.
-        if ( this->protocol.Equals( "chrome" ) || this->protocol.Equals( "MozillaXUL" ) ) {
-            // Use "-chrome" command line flag.
-            setting += chromeSuffix;
-        } else {
-            // Use standard "-url" command line flag.
-            setting += shortcutSuffix;
-        }
-    }
-    nsresult set();
-    nsresult reset();
-};
-
-// ProtocolIconRegistryEntry
-//
-// For setting icon entries for a given Internet Shortcut protocol.
-// The key name is calculated as
-// HKEY_LOCAL_MACHINE\Software\Classes\protocol\DefaultIcon.
-// The setting is this executable (with appropriate suffix).
-struct ProtocolIconRegistryEntry : public SavedRegistryEntry {
-    nsCString protocol;
-    ProtocolIconRegistryEntry( const char* aprotocol )
-        : SavedRegistryEntry( HKEY_LOCAL_MACHINE, "", "", thisApplication().get() ),
-          protocol( aprotocol ) {
-        keyName = NS_LITERAL_CSTRING("Software\\Classes\\") + protocol + NS_LITERAL_CSTRING("\\DefaultIcon");
-
-        // Append appropriate suffix to setting.
-        setting += iconSuffix;
-    }
-};
-
-// DDERegistryEntry
-//
-// Like a protocol registry entry, but for the shell\open\ddeexec subkey.
-//
-// We need to remove this subkey entirely to ensure we work properly with
-// various programs on various platforms (see Bugzilla bugs 59078, 58770, etc.).
-//
-// We don't try to save everything, though.  We do save the known useful info
-// under the ddeexec subkey:
-//     ddeexec\@
-//     ddeexec\NoActivateHandler
-//     ddeexec\Application\@
-//     ddeexec\Topic\@
-//
-// set/reset save/restore these values and remove/restore the ddeexec subkey
-struct DDERegistryEntry : public SavedRegistryEntry {
-    DDERegistryEntry( const char *protocol )
-        : SavedRegistryEntry( HKEY_LOCAL_MACHINE, "", "", 0 ),
-          activate( HKEY_LOCAL_MACHINE, "", "NoActivateHandler", 0 ),
-          app( HKEY_LOCAL_MACHINE, "", "", 0 ),
-          topic( HKEY_LOCAL_MACHINE, "", "", 0 ) {
-        // Derive keyName from protocol.
-        keyName = "Software\\Classes\\";
-        keyName += protocol;
-        keyName += "\\shell\\open\\ddeexec";
-        // Set subkey names.
-        activate.keyName = keyName;
-        app.keyName = keyName;
-        app.keyName += "\\Application";
-        topic.keyName = keyName;
-        topic.keyName += "\\Topic";
-    }
-    nsresult set();
-    nsresult reset();
-    SavedRegistryEntry activate, app, topic;
-};
-
-// FileTypeRegistryEntry
-//
-// For setting entries relating to a file extension (or extensions).
-// This object itself is for the "file type" associated with the extension.
-// Set/reset manage the mapping from extension to the file type, as well.
-// The description is taken from defDescKey if available. Otherwise desc 
-// is used.
-struct FileTypeRegistryEntry : public ProtocolRegistryEntry {
-    nsCString fileType;
-    const char **ext;
-    nsCString desc;
-    nsCString defDescKey;
-    nsCString iconFile;
-    FileTypeRegistryEntry ( const char **ext, const char *fileType, 
-        const char *desc, const char *defDescKey, const char *iconFile )
-        : ProtocolRegistryEntry( fileType ),
-          fileType( fileType ),
-          ext( ext ),
-          desc( desc ),
-          defDescKey(defDescKey),
-          iconFile(iconFile) {
-    }
-    nsresult set();
-    nsresult reset();
-};
-
-// EditableFileTypeRegistryEntry
-//
-// Extends FileTypeRegistryEntry by setting an additional handler for an "edit" command.
-struct EditableFileTypeRegistryEntry : public FileTypeRegistryEntry {
-    EditableFileTypeRegistryEntry( const char **ext, const char *fileType, 
-        const char *desc, const char *defDescKey, const char *iconFile )
-        : FileTypeRegistryEntry( ext, fileType, desc, defDescKey, iconFile ) {
-    }
-    nsresult set();
-};
-
 // Generate the "full" name of this registry entry.
 nsCString RegistryEntry::fullName() const {
     nsCString result;
@@ -364,7 +142,7 @@ else printf( "Setting %s=%s\n", fullName().get(), setting.get() );
                    rc = ::RegSetValueExA( key, valueNameArg(), 0, REG_SZ,
                         (LPBYTE)cSetting.get(), cSetting.Length() );
 #ifdef DEBUG_law
-                   NS_WARN_IF_FALSE( rc == ERROR_SUCCESS, fullName().get() );
+                   NS_ASSERTION( rc == ERROR_SUCCESS, fullName().get() );
 #endif
                    if ( rc == ERROR_SUCCESS ) {
                       result = NS_OK;
@@ -396,7 +174,7 @@ else printf( "Setting %s=%s\n", fullName().get(), setting.get() );
         ::RegCloseKey( key );
     } else {
 #ifdef DEBUG_law
-NS_WARN_IF_FALSE( rc == ERROR_SUCCESS, fullName().get() );
+NS_ASSERTION( rc == ERROR_SUCCESS, fullName().get() );
 #endif
     }
     return result;
@@ -441,80 +219,79 @@ static void setWindowsXP() {
         // set registered as "Start Menu internet apps."  These entries go
         // under the subkey MOZILLA.EXE (or whatever the name of this executable is),
         // that subkey name is generated by the utility function shortAppName.
-        if ( rc == ERROR_SUCCESS ) {
-            // The next 3 go under this subkey.
-            nsCAutoString subkey( baseKey + NS_LITERAL_CSTRING( "\\" ) + shortAppName() );
-            // Pretty name.  This goes into the LocalizedString value.  It is the
-            // name of the executable (preceded by '@'), followed by ",-nnn" where
-            // nnn is the resource identifier of the string in the .exe.  That value
-            // comes from nsINativeAppSupportWin.h.
-            char buffer[ _MAX_PATH + 8 ]; // Path, plus '@', comma, minus, and digits (5)
-            _snprintf( buffer, sizeof buffer, "@%s,-%d", thisApplication().get(), IDS_STARTMENU_APPNAME );
-            RegistryEntry tmp_entry1( HKEY_LOCAL_MACHINE, 
-                           subkey.get(),
-                           "LocalizedString", 
-                           buffer );
-            tmp_entry1.set();
-            // Default icon (from .exe resource).
-            RegistryEntry tmp_entry2( HKEY_LOCAL_MACHINE, 
-                           nsCAutoString( subkey + NS_LITERAL_CSTRING( "\\DefaultIcon" ) ).get(),
+        ::RegCloseKey(key);
+        // The next 3 go under this subkey.
+        nsCAutoString subkey( baseKey + NS_LITERAL_CSTRING( "\\" ) + shortAppName() );
+        // Pretty name.  This goes into the LocalizedString value.  It is the
+        // name of the executable (preceded by '@'), followed by ",-nnn" where
+        // nnn is the resource identifier of the string in the .exe.  That value
+        // comes from nsINativeAppSupportWin.h.
+        char buffer[ _MAX_PATH + 8 ]; // Path, plus '@', comma, minus, and digits (5)
+        _snprintf( buffer, sizeof buffer, "@%s,-%d", thisApplication().get(), IDS_STARTMENU_APPNAME );
+        RegistryEntry tmp_entry1( HKEY_LOCAL_MACHINE, 
+                       subkey.get(),
+                       "LocalizedString", 
+                       buffer );
+        tmp_entry1.set();
+        // Default icon (from .exe resource).
+        RegistryEntry tmp_entry2( HKEY_LOCAL_MACHINE, 
+                       nsCAutoString( subkey + NS_LITERAL_CSTRING( "\\DefaultIcon" ) ).get(),
+                       "", 
+                       nsCAutoString( thisApplication() + NS_LITERAL_CSTRING( ",0" ) ).get() );
+        tmp_entry2.set();
+        // Command to open.
+        RegistryEntry tmp_entry3( HKEY_LOCAL_MACHINE,
+                       nsCAutoString( subkey + NS_LITERAL_CSTRING( "\\shell\\open\\command" ) ).get(),
+                       "", 
+                       thisApplication().get() );
+        tmp_entry3.set();
+        // "Properties" verb.  The default value is the text that will appear in the menu.
+        // The default value under the command subkey is the name of this application, with
+        // arguments to cause the Preferences window to appear.
+        nsCOMPtr<nsIStringBundleService> bundleService( do_GetService( "@mozilla.org/intl/stringbundle;1" ) );
+        nsCOMPtr<nsIStringBundle> bundle;
+        nsXPIDLString label;
+        if ( bundleService &&
+             NS_SUCCEEDED( bundleService->CreateBundle( "chrome://global-platform/locale/nsWindowsHooks.properties",
+                                                   getter_AddRefs( bundle ) ) ) &&
+             NS_SUCCEEDED( bundle->GetStringFromName( NS_LITERAL_STRING( "prefsLabel" ).get(), getter_Copies( label ) ) ) ) {
+            // Set the label that will appear in the start menu context menu.
+            RegistryEntry tmp_entry4( HKEY_LOCAL_MACHINE,
+                           nsCAutoString( subkey + NS_LITERAL_CSTRING( "\\shell\\properties" ) ).get(),
                            "", 
-                           nsCAutoString( thisApplication() + NS_LITERAL_CSTRING( ",0" ) ).get() );
-            tmp_entry2.set();
-            // Command to open.
-            RegistryEntry tmp_entry3( HKEY_LOCAL_MACHINE,
-                           nsCAutoString( subkey + NS_LITERAL_CSTRING( "\\shell\\open\\command" ) ).get(),
-                           "", 
-                           thisApplication().get() );
-            tmp_entry3.set();
-            // "Properties" verb.  The default value is the text that will appear in the menu.
-            // The default value under the command subkey is the name of this application, with
-            // arguments to cause the Preferences window to appear.
-            nsCOMPtr<nsIStringBundleService> bundleService( do_GetService( "@mozilla.org/intl/stringbundle;1" ) );
-            nsCOMPtr<nsIStringBundle> bundle;
-            nsXPIDLString label;
-            if ( bundleService &&
-                 NS_SUCCEEDED( bundleService->CreateBundle( "chrome://global-platform/locale/nsWindowsHooks.properties",
-                                                       getter_AddRefs( bundle ) ) ) &&
-                 NS_SUCCEEDED( bundle->GetStringFromName( NS_LITERAL_STRING( "prefsLabel" ).get(), getter_Copies( label ) ) ) ) {
-                // Set the label that will appear in the start menu context menu.
-                RegistryEntry tmp_entry4( HKEY_LOCAL_MACHINE,
-                               nsCAutoString( subkey + NS_LITERAL_CSTRING( "\\shell\\properties" ) ).get(),
-                               "", 
-                               NS_ConvertUCS2toUTF8( label ).get() );
-                tmp_entry4.set();
-            }
-            RegistryEntry tmp_entry5( HKEY_LOCAL_MACHINE,
-                           nsCAutoString( subkey + NS_LITERAL_CSTRING( "\\shell\\properties\\command" ) ).get(),
-                           "", nsCAutoString( thisApplication() + 
-                                               NS_LITERAL_CSTRING(" -chrome \"chrome://communicator/content/pref/pref.xul\"") ).get()
-                          );
-            tmp_entry5.set();
-
-            // Now we need to select our application as the default start menu internet application.
-            // This is accomplished by first trying to store our subkey name in 
-            // HKLM\Software\Clients\StartMenuInternet's default value.  See
-            // http://support.microsoft.com/directory/article.asp?ID=KB;EN-US;Q297878 for detail.
-            SavedRegistryEntry hklmAppEntry( HKEY_LOCAL_MACHINE, baseKey.get(), "", shortAppName().get() );
-            hklmAppEntry.set();
-            // That may or may not have worked (depending on whether we have sufficient access).
-            if ( hklmAppEntry.currentSetting() == hklmAppEntry.setting ) {
-                // We've set the hklm entry, so we can delete the one under hkcu.
-                SavedRegistryEntry tmp_entry6( HKEY_CURRENT_USER, baseKey.get(), "", 0 );
-                tmp_entry6.set();
-            } else {
-                // All we can do is set the default start menu internet app for this user.
-                SavedRegistryEntry tmp_entry7( HKEY_CURRENT_USER, baseKey.get(), "", shortAppName().get() );
-            }
-            // Notify the system of the changes.
-            ::SendMessageTimeout( HWND_BROADCAST,
-                                  WM_SETTINGCHANGE,
-                                  0,
-                                  (LPARAM)MOZ_CLIENT_BROWSER_KEY,
-                                  SMTO_NORMAL|SMTO_ABORTIFHUNG,
-                                  MOZ_HWND_BROADCAST_MSG_TIMEOUT,
-                                  NULL);
+                           NS_ConvertUTF16toUTF8( label ).get() );
+            tmp_entry4.set();
         }
+        RegistryEntry tmp_entry5( HKEY_LOCAL_MACHINE,
+                       nsCAutoString( subkey + NS_LITERAL_CSTRING( "\\shell\\properties\\command" ) ).get(),
+                       "", nsCAutoString( thisApplication() + 
+                                           NS_LITERAL_CSTRING(" -chrome \"chrome://communicator/content/pref/pref.xul\"") ).get()
+                      );
+        tmp_entry5.set();
+
+        // Now we need to select our application as the default start menu internet application.
+        // This is accomplished by first trying to store our subkey name in 
+        // HKLM\Software\Clients\StartMenuInternet's default value.  See
+        // http://support.microsoft.com/directory/article.asp?ID=KB;EN-US;Q297878 for detail.
+        SavedRegistryEntry hklmAppEntry( HKEY_LOCAL_MACHINE, baseKey.get(), "", shortAppName().get() );
+        hklmAppEntry.set();
+        // That may or may not have worked (depending on whether we have sufficient access).
+        if ( hklmAppEntry.currentSetting() == hklmAppEntry.setting ) {
+            // We've set the hklm entry, so we can delete the one under hkcu.
+            SavedRegistryEntry tmp_entry6( HKEY_CURRENT_USER, baseKey.get(), "", 0 );
+            tmp_entry6.set();
+        } else {
+            // All we can do is set the default start menu internet app for this user.
+            SavedRegistryEntry tmp_entry7( HKEY_CURRENT_USER, baseKey.get(), "", shortAppName().get() );
+        }
+        // Notify the system of the changes.
+        ::SendMessageTimeout( HWND_BROADCAST,
+                              WM_SETTINGCHANGE,
+                              0,
+                              (LPARAM)MOZ_CLIENT_BROWSER_KEY,
+                              SMTO_NORMAL|SMTO_ABORTIFHUNG,
+                              MOZ_HWND_BROADCAST_MSG_TIMEOUT,
+                              NULL);
     }
 }
 
@@ -546,6 +323,7 @@ nsresult RegistryEntry::reset() {
     LONG rc = ::RegOpenKey( baseKey, keyName.get(), &key );
     if ( rc == ERROR_SUCCESS ) {
         rc = ::RegDeleteValue( key, valueNameArg() );
+        ::RegCloseKey(key);
 #ifdef DEBUG_law
 if ( rc == ERROR_SUCCESS ) printf( "Deleting key=%s\n", (const char*)fullName().get() );
 #endif
@@ -871,6 +649,7 @@ nsresult FileTypeRegistryEntry::reset() {
 // "edit" entry a SavedRegistryEntry).
 nsresult EditableFileTypeRegistryEntry::set() {
     nsresult rv = FileTypeRegistryEntry::set();
+#ifndef MOZ_XUL_APP
     if ( NS_SUCCEEDED( rv ) ) {
         // only set this if we support "-edit" on the command-line
         nsCOMPtr<nsICmdLineHandler> editorService =
@@ -884,6 +663,7 @@ nsresult EditableFileTypeRegistryEntry::set() {
             rv = RegistryEntry( HKEY_LOCAL_MACHINE, editKey.get(), "", editor.get() ).set();
         }
     }
+#endif
     return rv;
 }
 
