@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -21,11 +20,12 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *   Daniel Veditz <dveditz@netscape.com>
- *   Douglas Turner <dougt@netscape.com>
- *   Pierre Phaneuf <pp@ludusdesign.com>
- *   Sean Su <ssu@netscape.com>
- *   Samir Gehani <sgehani@netscape.com>
+ *     Daniel Veditz <dveditz@netscape.com>
+ *     Douglas Turner <dougt@netscape.com>
+ *     Jens Bannmann <jens.b@web.de>
+ *     Pierre Phaneuf <pp@ludusdesign.com>
+ *     Sean Su <ssu@netscape.com>
+ *     Samir Gehani <sgehani@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -41,12 +41,11 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-
-
 #include "nscore.h"
 #include "nsIFactory.h"
 #include "nsISupports.h"
 #include "nsNativeCharsetUtils.h"
+#include "nsStringEnumerator.h"
 
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
@@ -55,6 +54,11 @@
 #include "nsIFileChannel.h"
 #include "nsDirectoryService.h"
 #include "nsDirectoryServiceDefs.h"
+#include "nsAppDirectoryServiceDefs.h"
+#include "nsDirectoryServiceUtils.h"
+#include "nsThreadUtils.h"
+
+#include "nsNetUtil.h"
 
 #include "nsIPrefBranch.h"
 #include "nsIPrefService.h"
@@ -108,10 +112,10 @@
 #include <windows.h>
 #endif
 
-static NS_DEFINE_IID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
-static NS_DEFINE_IID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
-
-static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
+#if defined(XP_OS2)
+#define INCL_DOSMISC
+#include <os2.h>
+#endif
 
 #define kInstallLocaleProperties "chrome://global/locale/commonDialogs.properties"
 
@@ -140,7 +144,19 @@ NS_SoftwareUpdateRequestAutoReg()
   }
 
   file->AppendNative(nsDependentCString(".autoreg"));
+#ifdef DEBUG_timeless
+  PRBool condition;
+  if (NS_SUCCEEDED(file->IsDirectory(&condition)) && condition) {
+    /* someone did this intentionally, no point in complaining */
+    return;
+  }
+#endif
 
+  // Remove and recreate the file to update its timestamp.
+  // .autoreg must be newer than compreg.dat for component registration
+  // to occur.
+
+  file->Remove(PR_FALSE);
   rv = file->Create(nsIFile::NORMAL_FILE_TYPE, 0666);
 
   if (NS_FAILED(rv)) {
@@ -151,16 +167,13 @@ NS_SoftwareUpdateRequestAutoReg()
 
 
 
-MOZ_DECL_CTOR_COUNTER(nsInstallInfo)
-
 nsInstallInfo::nsInstallInfo(PRUint32           aInstallType,
                              nsIFile*           aFile,
                              const PRUnichar*   aURL,
                              const PRUnichar*   aArgs,
                              nsIPrincipal*      aPrincipal,
                              PRUint32           flags,
-                             nsIXPIListener*    aListener,
-                             nsIXULChromeRegistry* aChromeRegistry)
+                             nsIXPIListener*    aListener)
 : mPrincipal(aPrincipal),
   mError(0),
   mType(aInstallType),
@@ -168,10 +181,47 @@ nsInstallInfo::nsInstallInfo(PRUint32           aInstallType,
   mURL(aURL),
   mArgs(aArgs),
   mFile(aFile),
-  mListener(aListener),
-  mChromeRegistry(aChromeRegistry)
+  mListener(aListener)
 {
     MOZ_COUNT_CTOR(nsInstallInfo);
+
+    nsresult rv;
+
+    // Failure is an option, and will occur in the stub installer.
+
+    nsCOMPtr<nsIThread> thread = do_GetMainThread();
+
+    NS_WITH_ALWAYS_PROXIED_SERVICE(CHROMEREG_IFACE, cr,
+                                   NS_CHROMEREGISTRY_CONTRACTID,
+                                   thread, &rv);
+    if (NS_SUCCEEDED(rv)) {
+      mChromeRegistry = cr;
+
+      nsCAutoString spec;
+      rv = NS_GetURLSpecFromFile(aFile, spec);
+      if (NS_SUCCEEDED(rv)) {
+        spec.Insert(NS_LITERAL_CSTRING("jar:"), 0);
+        spec.AppendLiteral("!/");
+#ifdef MOZ_XUL_APP
+        NS_NewURI(getter_AddRefs(mFileJARURL), spec);
+#else
+        mFileJARSpec.Assign(spec);
+#endif
+      }
+    }
+
+#ifdef MOZ_XUL_APP
+    NS_WITH_ALWAYS_PROXIED_SERVICE(nsIExtensionManager, em,
+                                   "@mozilla.org/extensions/manager;1",
+                                   thread, &rv);
+    if (NS_SUCCEEDED(rv))
+      mExtensionManager = em;
+
+    nsCOMPtr<nsIFile> manifest;
+    rv = NS_GetSpecialDirectory(NS_APP_CHROME_DIR, getter_AddRefs(manifest));
+    if (NS_SUCCEEDED(rv))
+      NS_NewFileURI(getter_AddRefs(mManifestURL), manifest);
+#endif
 }
 
 
@@ -183,8 +233,6 @@ nsInstallInfo::~nsInstallInfo()
 static NS_DEFINE_IID(kSoftwareUpdateCID,  NS_SoftwareUpdate_CID);
 
 
-MOZ_DECL_CTOR_COUNTER(nsInstall)
-
 nsInstall::nsInstall(nsIZipReader * theJARFile)
 {
     MOZ_COUNT_CTOR(nsInstall);
@@ -216,12 +264,14 @@ nsInstall::nsInstall(nsIZipReader * theJARFile)
 
     su->Release();
 
+    nsCOMPtr<nsIThread> thread = do_GetMainThread();
+
     // get the resourced xpinstall string bundle
     mStringBundle = nsnull;
     NS_WITH_PROXIED_SERVICE( nsIStringBundleService,
                              service,
-                             kStringBundleServiceCID,
-                             NS_UI_THREAD_EVENTQ,
+                             NS_STRINGBUNDLE_CONTRACTID,
+                             thread,
                              &rv );
 
     if (NS_SUCCEEDED(rv) && service)
@@ -415,20 +465,14 @@ nsInstall::AddDirectory(const nsString& aRegName,
         return NS_OK;
     }
 
+    // Default subName = location in jar file
     nsString qualifiedRegName;
+    result = GetQualifiedRegName( aRegName.IsEmpty() ? aJarSource : aRegName,
+                                  qualifiedRegName);
 
-    if ( aRegName.IsEmpty())
-    {
-        // Default subName = location in jar file
-        *aReturn = GetQualifiedRegName( aJarSource, qualifiedRegName);
-    }
-    else
-    {
-        *aReturn = GetQualifiedRegName( aRegName, qualifiedRegName );
-    }
-
-    if (*aReturn != SUCCESS)
+    if (result != nsInstall::SUCCESS)
     {
+        *aReturn = SaveError( result );
         return NS_OK;
     }
 
@@ -445,47 +489,47 @@ nsInstall::AddDirectory(const nsString& aRegName,
         }
     }
 
-    nsString subdirectory(aSubdir);
-
+    nsAutoString subdirectory(aSubdir);
     if (!subdirectory.IsEmpty())
-    {
         subdirectory.AppendLiteral("/");
-    }
 
+    nsAutoString prefix(aJarSource + NS_LITERAL_STRING("/"));
+    const PRInt32 prefix_length = prefix.Length();
+    NS_LossyConvertUTF16toASCII pattern(prefix + NS_LITERAL_STRING("*"));
 
-    nsVoidArray *paths = new nsVoidArray();
-
-    if (paths == nsnull)
+    nsCOMPtr<nsIUTF8StringEnumerator> jarEnum;
+    nsresult rv = mJarFileData->FindEntries(pattern.get(),
+                                            getter_AddRefs(jarEnum) );
+    if (NS_FAILED(rv) || !jarEnum)
     {
-        *aReturn = SaveError(nsInstall::OUT_OF_MEMORY);
+        *aReturn = SaveError( nsInstall::EXTRACTION_FAILED );
         return NS_OK;
     }
 
     PRInt32 count = 0;
-    result = ExtractDirEntries(aJarSource, paths);
-    if (result == nsInstall::SUCCESS)
+    PRBool bMore = PR_FALSE;
+    while (NS_SUCCEEDED(jarEnum->HasMore(&bMore)) && bMore)
     {
-        count = paths->Count();
-        if (count == 0)
-            result = nsInstall::DOES_NOT_EXIST;
-    }
-
-    for (PRInt32 i=0; i < count && result == nsInstall::SUCCESS; i++)
-    {
-        nsString *thisPath = (nsString *)paths->ElementAt(i);
-
-        nsString newJarSource = aJarSource;
-        newJarSource.AppendLiteral("/");
-        newJarSource += *thisPath;
-
-        nsString newSubDir;
-
-        if (!subdirectory.IsEmpty())
+        nsCAutoString name;
+        rv = jarEnum->GetNext(name);
+        if (NS_FAILED(rv))
         {
-            newSubDir = subdirectory;
+            result = nsInstall::EXTRACTION_FAILED;
+            break;
         }
 
-        newSubDir += *thisPath;
+        if ( name.Last() == '/' )
+        {
+            // Skip the directory entries
+            continue;
+        }
+        const PRInt32 namelen = name.Length();
+        NS_ASSERTION( prefix_length <= namelen, "Match must be longer than pattern!" );
+        NS_ConvertASCIItoUTF16 fileName(Substring(name,
+                                                  prefix_length,
+                                                  namelen - prefix_length));
+        nsAutoString newJarSource(prefix + fileName);
+        nsAutoString newSubDir(subdirectory + fileName);
 
         ie = new nsInstallFile( this,
                                 qualifiedRegName,
@@ -494,7 +538,7 @@ nsInstall::AddDirectory(const nsString& aRegName,
                                 aFolder,
                                 newSubDir,
                                 aMode,
-                                (i == 0), // register the first one only
+                                (count == 0), // register the first one only
                                 &result);
 
         if (ie == nsnull)
@@ -508,10 +552,12 @@ nsInstall::AddDirectory(const nsString& aRegName,
         else
         {
             result = ScheduleForInstall( ie );
+            count++;
         }
     }
 
-    DeleteVector(paths);
+    if (result == nsInstall::SUCCESS && count == 0) 
+        result = nsInstall::DOES_NOT_EXIST;
 
     *aReturn = SaveError( result );
     return NS_OK;
@@ -613,19 +659,13 @@ nsInstall::AddSubcomponent(const nsString& aRegName,
     if (qualifiedVersion.IsEmpty())
         qualifiedVersion.AssignLiteral("0.0.0.0");
 
+    // Default subName = location in jar file
+    result = GetQualifiedRegName( aRegName.IsEmpty() ? aJarSource : aRegName,
+                                  qualifiedRegName);
 
-    if ( aRegName.IsEmpty() )
-    {
-        // Default subName = location in jar file
-        *aReturn = GetQualifiedRegName( aJarSource, qualifiedRegName);
-    }
-    else
-    {
-        *aReturn = GetQualifiedRegName( aRegName, qualifiedRegName );
-    }
-
-    if (*aReturn != SUCCESS)
+    if (result != nsInstall::SUCCESS)
     {
+        *aReturn = SaveError( result );
         return NS_OK;
     }
 
@@ -643,7 +683,7 @@ nsInstall::AddSubcomponent(const nsString& aRegName,
     if (ie == nsnull)
     {
         *aReturn = SaveError(nsInstall::OUT_OF_MEMORY);
-            return NS_OK;
+        return NS_OK;
     }
 
     if (errcode == nsInstall::SUCCESS)
@@ -812,8 +852,8 @@ nsInstall::FinalizeInstall(PRInt32* aReturn)
     {
         if ( mUninstallPackage )
         {
-            VR_UninstallCreateNode( NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(mRegistryPackageName).get()),
-                                    NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(mUIName).get()));
+            VR_UninstallCreateNode( NS_CONST_CAST(char *, NS_ConvertUTF16toUTF8(mRegistryPackageName).get()),
+                                    NS_CONST_CAST(char *, NS_ConvertUTF16toUTF8(mUIName).get()));
         }
 
         // Install the Component into the Version Registry.
@@ -829,7 +869,7 @@ nsInstall::FinalizeInstall(PRInt32* aReturn)
             if (mPackageFolder)
                 mPackageFolder->GetDirectoryPath(path);
 
-            VR_Install( NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(mRegistryPackageName).get()),
+            VR_Install( NS_CONST_CAST(char *, NS_ConvertUTF16toUTF8(mRegistryPackageName).get()),
                         NS_CONST_CAST(char *, path.get()),
                         NS_CONST_CAST(char *, versionCString.get()),
                         PR_TRUE );
@@ -850,7 +890,7 @@ nsInstall::FinalizeInstall(PRInt32* aReturn)
                 if (objString)
                 {
                     mListener->OnFinalizeProgress(
-                                    NS_ConvertASCIItoUCS2(objString).get(),
+                                    NS_ConvertASCIItoUTF16(objString).get(),
                                     (i+1), mInstalledFiles->Count());
                     delete [] objString;
                 }
@@ -978,7 +1018,7 @@ nsInstall::GetComponentFolder(const nsString& aComponentName, const nsString& aS
         return NS_OK;
     }
 
-    NS_ConvertUCS2toUTF8 componentCString(tempString);
+    NS_ConvertUTF16toUTF8 componentCString(tempString);
 
     if((err = VR_GetDefaultDirectory( NS_CONST_CAST(char *, componentCString.get()), sizeof(dir), dir )) != REGERR_OK)
     {
@@ -1163,7 +1203,7 @@ PRInt32
 nsInstall::LoadResources(JSContext* cx, const nsString& aBaseName, jsval* aReturn)
 {
     *aReturn = JSVAL_NULL;
-
+ 
     if (SanityCheck() != nsInstall::SUCCESS)
     {
         return NS_OK;
@@ -1172,7 +1212,6 @@ nsInstall::LoadResources(JSContext* cx, const nsString& aBaseName, jsval* aRetur
     nsCOMPtr<nsIFile> resFile;
     nsIURI *url = nsnull;
     nsIStringBundleService* service = nsnull;
-    nsIEventQueueService* pEventQueueService = nsnull;
     nsIStringBundle* bundle = nsnull;
     nsCOMPtr<nsISimpleEnumerator> propEnum;
     jsval v = JSVAL_NULL;
@@ -1196,14 +1235,7 @@ nsInstall::LoadResources(JSContext* cx, const nsString& aBaseName, jsval* aRetur
     }
 
     // initialize string bundle and related services
-    ret = CallGetService(kStringBundleServiceCID, &service);
-    if (NS_FAILED(ret))
-        goto cleanup;
-    ret = CallGetService(kEventQueueServiceCID, &pEventQueueService);
-    if (NS_FAILED(ret))
-        goto cleanup;
-    ret = pEventQueueService->CreateThreadEventQueue();
-    NS_RELEASE(pEventQueueService);
+    ret = CallGetService(NS_STRINGBUNDLE_CONTRACTID, &service);
     if (NS_FAILED(ret))
         goto cleanup;
 
@@ -1358,14 +1390,11 @@ nsPIXPIProxy* nsInstall::GetUIThreadProxy()
     if (!mUIThreadProxy)
     {
         nsresult rv;
-        nsCOMPtr<nsIProxyObjectManager> pmgr =
-                 do_GetService(kProxyObjectManagerCID, &rv);
-        if (NS_SUCCEEDED(rv))
-        {
-            nsCOMPtr<nsPIXPIProxy> tmp(do_QueryInterface(new nsXPIProxy()));
-            rv = pmgr->GetProxyForObject( NS_UI_THREAD_EVENTQ, NS_GET_IID(nsPIXPIProxy),
-                    tmp, PROXY_SYNC | PROXY_ALWAYS, getter_AddRefs(mUIThreadProxy) );
-        }
+        nsCOMPtr<nsPIXPIProxy> tmp(do_QueryInterface(new nsXPIProxy()));
+        rv = NS_GetProxyForObject( NS_PROXY_TO_MAIN_THREAD,
+                                   NS_GET_IID(nsPIXPIProxy), tmp,
+                                   NS_PROXY_SYNC | NS_PROXY_ALWAYS,
+                                   getter_AddRefs(mUIThreadProxy) );
     }
 
     return mUIThreadProxy;
@@ -1468,7 +1497,7 @@ nsInstall::StartInstall(const nsString& aUserPackageName, const nsString& aRegis
 
     mPackageFolder = nsnull;
     if(REGERR_OK == VR_GetDefaultDirectory(
-                        NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(mRegistryPackageName).get()),
+                        NS_CONST_CAST(char *, NS_ConvertUTF16toUTF8(mRegistryPackageName).get()),
                         sizeof(szRegPackagePath), szRegPackagePath))
     {
         // found one saved in the registry
@@ -1565,9 +1594,9 @@ nsInstall::GetPatch(nsHashKey *aKey, nsIFile** fileName)
     else
         *fileName = nsnull;
 
-    if (mPatchList != nsnull)
+    if (mPatchList)
     {
-        *fileName = (nsIFile*) mPatchList->Get(aKey);
+        NS_IF_ADDREF(*fileName = (nsIFile*) mPatchList->Get(aKey));
     }
 }
 
@@ -1620,7 +1649,7 @@ nsInstall::FileOpDirGetParent(nsInstallFolder& aTarget, nsInstallFolder** thePar
     {
         return NS_ERROR_OUT_OF_MEMORY;
     }
-      folder->Init(parent,EmptyString());
+      folder->Init(parent, EmptyString());
       *theParentFolder = folder;
   }
   else
@@ -2260,7 +2289,7 @@ nsInstall::ScheduleForInstall(nsInstallObject* ob)
     // flash current item
 
     if (mListener)
-        mListener->OnItemScheduled(NS_ConvertASCIItoUCS2(objString).get());
+        mListener->OnItemScheduled(NS_ConvertASCIItoUTF16(objString).get());
 
 
     // do any unpacking or other set-up
@@ -2533,7 +2562,7 @@ void nsInstall::SetInstallURL(const nsString& url)  { mInstallURL = url; }
 //-----------------------------------------------------------------------------
 PRUnichar *GetTranslatedString(const PRUnichar* aString)
 {
-    nsCOMPtr<nsIStringBundleService> stringService = do_GetService(kStringBundleServiceCID);
+    nsCOMPtr<nsIStringBundleService> stringService = do_GetService(NS_STRINGBUNDLE_CONTRACTID);
     nsCOMPtr<nsIStringBundle> stringBundle;
     PRUnichar* translatedString;
 
@@ -2553,22 +2582,55 @@ nsInstall::Alert(nsString& string)
     if (!ui)
         return UNEXPECTED_ERROR;
 
-    return ui->Alert( GetTranslatedString(NS_LITERAL_STRING("Alert").get()),
-                      string.get());
+    nsAutoString title;
+    title.AssignLiteral("Alert");
+    if (!mUIName.IsEmpty())
+    {
+        title = mUIName;
+    }
+    else
+    {
+        PRUnichar *t = GetTranslatedString(title.get());
+        if (t)
+            title.Adopt(t);
+    }
+    return ui->Alert( title.get(), string.get());
 }
 
 PRInt32
-nsInstall::Confirm(nsString& string, PRBool* aReturn)
+nsInstall::ConfirmEx(nsString& aDialogTitle, nsString& aText, PRUint32 aButtonFlags, nsString& aButton0Title, nsString& aButton1Title, nsString& aButton2Title, nsString& aCheckMsg, PRBool* aCheckState, PRInt32* aReturn)
 {
-    *aReturn = PR_FALSE; /* default value */
+    *aReturn = -1; /* default value */
 
     nsPIXPIProxy *ui = GetUIThreadProxy();
     if (!ui)
         return UNEXPECTED_ERROR;
 
-    return ui->Confirm( GetTranslatedString(NS_LITERAL_STRING("Confirm").get()),
-                        string.get(),
-                        aReturn);
+    nsAutoString title;
+    title.AssignLiteral("Confirm");
+    if (!aDialogTitle.IsEmpty())
+    {
+        title = aDialogTitle;
+    }
+    else if (!mUIName.IsEmpty())
+    {
+        title = mUIName;
+    }
+    else
+    {
+        PRUnichar *t = GetTranslatedString(title.get());
+        if (t)
+            title.Adopt(t);
+    }
+    return ui->ConfirmEx( title.get(),
+                          aText.get(),
+                          aButtonFlags,
+                          aButton0Title.get(),
+                          aButton1Title.get(),
+                          aButton2Title.get(),
+                          aCheckMsg.get(),
+                          aCheckState,
+                          aReturn);
 }
 
 
@@ -2647,7 +2709,7 @@ nsInstall::ExtractFileFromJar(const nsString& aJarfile, nsIFile* aSuggestedName,
         extractHereSpec = temp;
     }
 
-    rv = mJarFileData->Extract(NS_LossyConvertUCS2toASCII(aJarfile).get(),
+    rv = mJarFileData->Extract(NS_LossyConvertUTF16toASCII(aJarfile).get(),
                                extractHereSpec);
     if (NS_FAILED(rv))
     {
@@ -2675,10 +2737,10 @@ nsInstall::ExtractFileFromJar(const nsString& aJarfile, nsIFile* aSuggestedName,
         if (asd)
             decodeErr = asd->Decode();
 
-        delete asd;
-
         if (decodeErr != noErr)
         {
+            if (asd)
+                delete asd;
             return EXTRACTION_FAILED;
         }
 
@@ -2726,82 +2788,5 @@ nsInstall::GetResourcedString(const nsAString& aResName)
     ** than nothing due to failure above: always the case for the Install Wizards.
     */
     return nsCRT::strdup(nsInstallResources::GetDefaultVal(
-                    NS_LossyConvertUCS2toASCII(aResName).get()));
-}
-
-
-PRInt32
-nsInstall::ExtractDirEntries(const nsString& directory, nsVoidArray *paths)
-{
-    char                *buf;
-    nsISimpleEnumerator *jarEnum = nsnull;
-    nsIZipEntry         *currZipEntry = nsnull;
-
-    if ( paths )
-    {
-        nsString pattern(directory + NS_LITERAL_STRING("/*"));
-        PRInt32 prefix_length = directory.Length()+1; // account for slash
-
-        nsresult rv = mJarFileData->FindEntries(
-                          NS_LossyConvertUCS2toASCII(pattern).get(),
-                          &jarEnum );
-        if (NS_FAILED(rv) || !jarEnum)
-            goto handle_err;
-
-        PRBool bMore;
-        rv = jarEnum->HasMoreElements(&bMore);
-        while (bMore && NS_SUCCEEDED(rv))
-        {
-            rv = jarEnum->GetNext( (nsISupports**) &currZipEntry );
-            if (currZipEntry)
-            {
-                // expensive 'buf' callee malloc per iteration!
-                rv = currZipEntry->GetName(&buf);
-                if (NS_FAILED(rv))
-                    goto handle_err;
-                if (buf)
-                {
-                    PRInt32 namelen = PL_strlen(buf);
-                    NS_ASSERTION( prefix_length <= namelen, "Match must be longer than pattern!" );
-
-                    if ( buf[namelen-1] != '/' )
-                    {
-                        // XXX manipulation should be in caller
-                        nsString* tempString = new nsString; tempString->AssignWithConversion(buf+prefix_length);
-                        paths->AppendElement(tempString);
-                    }
-
-                    PR_FREEIF( buf );
-                }
-                NS_IF_RELEASE(currZipEntry);
-            }
-            rv = jarEnum->HasMoreElements(&bMore);
-        }
-    }
-
-    NS_IF_RELEASE(jarEnum);
-    return SUCCESS;
-
-handle_err:
-    NS_IF_RELEASE(jarEnum);
-    NS_IF_RELEASE(currZipEntry);
-    return EXTRACTION_FAILED;
-}
-
-void
-nsInstall::DeleteVector(nsVoidArray* vector)
-{
-    if (vector != nsnull)
-    {
-        for (PRInt32 i=0; i < vector->Count(); i++)
-        {
-            nsString* element = (nsString*)vector->ElementAt(i);
-            if (element != nsnull)
-                delete element;
-        }
-
-        vector->Clear();
-        delete (vector);
-        vector = nsnull;
-    }
+                    NS_LossyConvertUTF16toASCII(aResName).get()));
 }
