@@ -88,8 +88,6 @@ static int32   gdiff_validateFile( pDIFFDATA dd, int file );
 static PRBool  su_unbind(char* oldsrc, char* newsrc);
 #endif
 
-MOZ_DECL_CTOR_COUNTER(nsInstallPatch)
-
 nsInstallPatch::nsInstallPatch( nsInstall* inInstall,
                                 const nsString& inVRName,
                                 const nsString& inVInfo,
@@ -102,7 +100,7 @@ nsInstallPatch::nsInstallPatch( nsInstall* inInstall,
 
     char tempTargetFile[MAXREGPATHLEN];
 
-    PRInt32 err = VR_GetPath( NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(inVRName).get()),
+    PRInt32 err = VR_GetPath( NS_CONST_CAST(char *, NS_ConvertUTF16toUTF8(inVRName).get()),
                               sizeof(tempTargetFile), tempTargetFile );
     
     if (err != REGERR_OK)
@@ -330,9 +328,9 @@ PRInt32 nsInstallPatch::Complete()
             // DO NOT propogate version registry errors, it will abort 
             // FinalizeInstall() leaving things hosed. These piddly errors
             // aren't worth that.
-            VR_Install( NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(*mRegistryName).get()),
+            VR_Install( NS_CONST_CAST(char *, NS_ConvertUTF16toUTF8(*mRegistryName).get()),
                         NS_CONST_CAST(char *, tempPath.get()),
-                        NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(tempVersionString).get()),
+                        NS_CONST_CAST(char *, NS_ConvertUTF16toUTF8(tempVersionString).get()),
                         PR_FALSE );
         }
         else
@@ -411,7 +409,6 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
   nsresult rv;
   DIFFDATA	  *dd;
 	PRInt32		  status		   = GDIFF_ERR_MEM;
-	char 		    *tmpurl		   = NULL;
 	//nsFileSpec  *outFileSpec = new nsFileSpec; 
   //nsFileSpec  *tempSrcFile = new nsFileSpec;   // TODO: do you need to free?
   nsCOMPtr<nsIFile> outFileSpec;
@@ -438,7 +435,8 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
 		dd->bufsize = BUFSIZE;
 
 		// validate patch header & check for special instructions
-    patchFileLocal->OpenNSPRFileDesc(PR_RDONLY, 0666, &dd->fDiff);
+    // we're just reading, the 0400 is an annotation.
+    patchFileLocal->OpenNSPRFileDesc(PR_RDONLY, 0400, &dd->fDiff);
 
 		if (dd->fDiff != NULL)
 		{
@@ -555,8 +553,9 @@ nsInstallPatch::NativePatch(nsIFile *sourceFile, nsIFile *patchFile, nsIFile **n
     nsCOMPtr<nsILocalFile> realFileLocal = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);;
     realFileLocal->InitWithNativePath(realfile);
 
-    realFileLocal->OpenNSPRFileDesc(PR_RDONLY, 0666, &dd->fSrc);
-    outFileLocal->OpenNSPRFileDesc(PR_RDWR|PR_CREATE_FILE|PR_TRUNCATE, 0666, &dd->fOut);
+    // it's ok for people in the group to modify these files later, but it wouldn't be good for just anyone.
+    realFileLocal->OpenNSPRFileDesc(PR_RDONLY, 0664, &dd->fSrc);
+    outFileLocal->OpenNSPRFileDesc(PR_RDWR|PR_CREATE_FILE|PR_TRUNCATE, 0664, &dd->fOut);
 
     if (dd->fSrc != NULL && dd->fOut != NULL)
     {
@@ -662,12 +661,6 @@ cleanup:
         PR_DELETE(dd);
     }
 
-    if ( tmpurl != NULL ) {
-        //XP_FileRemove( tmpurl, xpURL );
-        tmpurl = NULL;
-        PR_DELETE( tmpurl );
-    }
-
     if (tempSrcFile)
     {
         tempSrcFile->Exists(&flagExists);
@@ -944,7 +937,7 @@ int32 gdiff_valCRC32( pDIFFDATA dd, PRFileDesc* fh, uint32 chksum )
  *  opened, GDIFF header read, and all other setup complete
  *
  *  The GDIFF patch is processed sequentially which random
- *  access is neccessary for the source file.
+ *  access is necessary for the source file.
  *---------------------------------------------------------
  */
 static
