@@ -1,4 +1,3 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -12,7 +11,8 @@
  * for the specific language governing rights and limitations under the
  * License.
  *
- * The Original Code is Mozilla Communicator client code.
+ * The Original Code is Mozilla Communicator client code, released
+ * March 31, 1998.
  *
  * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
@@ -35,6 +35,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+
 #include "nsSoftwareUpdate.h"
 #include "nsXPInstallManager.h"
 #include "nsInstallTrigger.h"
@@ -45,6 +46,7 @@
 #include "netCore.h"
 #include "nsIFactory.h"
 #include "nsISupports.h"
+#include "nsPIDOMWindow.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptGlobalObjectOwner.h"
 
@@ -56,6 +58,8 @@
 #include "nsIDOMDocument.h"
 #include "nsIDocument.h"
 #include "nsIPrincipal.h"
+#include "nsIObserverService.h"
+#include "nsIPropertyBag2.h"
 
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
@@ -147,21 +151,29 @@ nsInstallTrigger::HandleContent(const char * aContentType,
 
 
     // Save the referrer if any, for permission checks
-    static const char kReferrerProperty[] = "docshell.internalReferrer";
+    NS_NAMED_LITERAL_STRING(referrerProperty, "docshell.internalReferrer");
     PRBool useReferrer = PR_FALSE;
     nsCOMPtr<nsIURI> referringURI;
-    nsCOMPtr<nsIProperties> channelprops(do_QueryInterface(channel));
+    nsCOMPtr<nsIPropertyBag2> channelprops(do_QueryInterface(channel));
 
-    if (channelprops && 
-        NS_SUCCEEDED(channelprops->Has(kReferrerProperty, &useReferrer)) &&
-        useReferrer)
+    if (channelprops)
     {
-        // channel may have the property but set to null. In that case we know
-        // it was a typed URL or bookmark and can bypass site whitelisting, as
-        // opposed to not knowing the origin and going with the fallback plan.
-        channelprops->Get(kReferrerProperty,
-                          NS_GET_IID(nsIURI),
-                          getter_AddRefs(referringURI));
+        // Get the referrer from the channel properties if we can (not all
+        // channels support our internal-referrer property).
+        //
+        // It's possible docshell explicitly set a null referrer in the case
+        // of typed, pasted, or bookmarked URLs and the like. In such a case
+        // we get a success return value with null pointer.
+        //
+        // A null referrer is automatically whitelisted as an explicit user
+        // action (though they'll still get the confirmation dialog). For a
+        // missing referrer we go to our fall-back plan of using the XPI
+        // location for whitelisting purposes.
+        rv = channelprops->GetPropertyAsInterface(referrerProperty,
+                                                  NS_GET_IID(nsIURI),
+                                                  getter_AddRefs(referringURI));
+        if (NS_SUCCEEDED(rv))
+            useReferrer = PR_TRUE;
     }
 
     // Cancel the current request. nsXPInstallManager restarts the download
@@ -170,13 +182,10 @@ nsInstallTrigger::HandleContent(const char * aContentType,
 
 
     // Get the global object of the target window for StartSoftwareUpdate
-    nsIScriptGlobalObject* globalObject = nsnull;
     nsCOMPtr<nsIScriptGlobalObjectOwner> globalObjectOwner = 
                                          do_QueryInterface(aWindowContext);
-    if ( globalObjectOwner )
-    {
-        globalObject = globalObjectOwner->GetScriptGlobalObject();
-    }
+    nsIScriptGlobalObject* globalObject =
+      globalObjectOwner ? globalObjectOwner->GetScriptGlobalObject() : nsnull;
     if ( !globalObject )
         return NS_ERROR_INVALID_ARG;
 
@@ -244,10 +253,16 @@ nsInstallTrigger::HandleContent(const char * aContentType,
     }
     else
     {
-        // TODO: fire event signaling blocked Install attempt
+        nsCOMPtr<nsPIDOMWindow> win(do_QueryInterface(globalObject));
+        nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+        if (os) {
+            os->NotifyObservers(win->GetDocShell(),
+                                "xpinstall-install-blocked", 
+                                NS_LITERAL_STRING("install-chrome").get());
+        }
         rv = NS_ERROR_ABORT;
     }
-
+    
     return rv;
 }
 
@@ -333,6 +348,9 @@ nsInstallTrigger::AllowInstall(nsIURI* aLaunchURI)
             updatePermissions( XPINSTALL_WHITELIST_ADD,
                                nsIPermissionManager::ALLOW_ACTION,
                                permissionMgr, prefBranch );
+            updatePermissions( XPINSTALL_WHITELIST_ADD_103,
+                               nsIPermissionManager::ALLOW_ACTION,
+                               permissionMgr, prefBranch );
             updatePermissions( XPINSTALL_BLACKLIST_ADD,
                                nsIPermissionManager::DENY_ACTION,
                                permissionMgr, prefBranch );
@@ -394,7 +412,6 @@ nsInstallTrigger::UpdateEnabled(nsIScriptGlobalObject* aGlobalObject, PRBool aUs
 }
 
 
-
 NS_IMETHODIMP
 nsInstallTrigger::Install(nsIScriptGlobalObject* aGlobalObject, nsXPITriggerInfo* aTrigger, PRBool* aReturn)
 {
@@ -518,7 +535,7 @@ nsInstallTrigger::CompareVersion(const nsString& aRegName, nsIDOMInstallVersion*
     *aReturn = NOT_FOUND;  // assume failure.
 
     VERSION              cVersion;
-    NS_ConvertUCS2toUTF8 regName(aRegName);
+    NS_ConvertUTF16toUTF8 regName(aRegName);
     REGERR               status;
     nsInstallVersion     regNameVersion;
 
@@ -545,7 +562,7 @@ NS_IMETHODIMP
 nsInstallTrigger::GetVersion(const nsString& component, nsString& version)
 {
     VERSION              cVersion;
-    NS_ConvertUCS2toUTF8 regName(component);
+    NS_ConvertUTF16toUTF8 regName(component);
     REGERR               status;
 
     status = VR_GetVersion( NS_CONST_CAST(char *, regName.get()), &cVersion );
