@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Jens Bannmann <jens.b@web.de>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -46,6 +47,7 @@
 #include "nsInstall.h"
 #include "nsInstallFile.h"
 #include "nsInstallTrigger.h"
+#include "nsIPromptService.h"
 
 #include "nsIDOMInstallVersion.h"
 
@@ -64,6 +66,32 @@ extern JSClass FileSpecObjectClass;
 
 extern JSClass FileOpClass;
 
+JS_STATIC_DLL_CALLBACK(JSBool)
+GetInstallProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+
+JS_STATIC_DLL_CALLBACK(JSBool)
+SetInstallProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+
+JS_STATIC_DLL_CALLBACK(void)
+FinalizeInstall(JSContext *cx, JSObject *obj);
+
+/***********************************************************************/
+//
+// class for Install
+//
+JSClass InstallClass = {
+  "Install",
+  JSCLASS_HAS_PRIVATE | JSCLASS_GLOBAL_FLAGS,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  GetInstallProperty,
+  SetInstallProperty,
+  JS_EnumerateStub,
+  JS_ResolveStub,
+  JS_ConvertStub,
+  FinalizeInstall
+};
+
 //
 // Install property ids
 //
@@ -92,7 +120,7 @@ JSObject *gFileOpObject = nsnull;
 //
 // Install Properties Getter
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 GetInstallProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 {
   nsInstall *a = (nsInstall*)JS_GetPrivate(cx, obj);
@@ -185,7 +213,7 @@ GetInstallProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 //
 // Install Properties Setter
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 SetInstallProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 {
   nsInstall *a = (nsInstall*)JS_GetPrivate(cx, obj);
@@ -210,7 +238,7 @@ SetInstallProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 }
 
 
-static void PR_CALLBACK FinalizeInstall(JSContext *cx, JSObject *obj)
+static void JS_DLL_CALLBACK FinalizeInstall(JSContext *cx, JSObject *obj)
 {
     nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
     delete nativeThis;
@@ -265,36 +293,50 @@ PRBool ConvertJSValToBool(PRBool* aProp,
 
 PRBool ConvertJSValToObj(nsISupports** aSupports,
                         REFNSIID aIID,
-                        const nsString& aTypeName,
+                        JSClass* aClass,
                         JSContext* aContext,
                         jsval aValue)
 {
   if (JSVAL_IS_NULL(aValue)) {
     *aSupports = nsnull;
+    return JS_TRUE;
   }
-  else if (JSVAL_IS_OBJECT(aValue)) {
-    JSObject* jsobj = JSVAL_TO_OBJECT(aValue);
-    JSClass* jsclass = JS_GetClass(aContext, jsobj);
-    if ((nsnull != jsclass) && (jsclass->flags & JSCLASS_HAS_PRIVATE)) {
-      nsISupports *supports = (nsISupports *)JS_GetPrivate(aContext, jsobj);
-      if (NS_OK != supports->QueryInterface(aIID, (void **)aSupports)) {
-        char buf[128];
-        char typeName[128];
-        aTypeName.ToCString(typeName, sizeof(typeName));
-        sprintf(buf, "Parameter must of type %s", typeName);
-        JS_ReportError(aContext, buf);
-        return JS_FALSE;
-      }
-    }
-    else {
-      JS_ReportError(aContext, "Parameter isn't an object");
-      return JS_FALSE;
-    }
-  }
-  else {
+  if (!JSVAL_IS_OBJECT(aValue)) {
     JS_ReportError(aContext, "Parameter must be an object");
     return JS_FALSE;
   }
+  JSObject* jsobj = JSVAL_TO_OBJECT(aValue);
+  JSClass* jsclass = JS_GetClass(aContext, jsobj);
+  if (!jsclass ||
+      !(jsclass->flags & JSCLASS_HAS_PRIVATE) ||
+      !(jsclass->flags & JSCLASS_PRIVATE_IS_NSISUPPORTS)) {
+    JS_ReportError(aContext,
+                   "Parameter isn't an object that implements nsISupports");
+    return JS_FALSE;
+  }
+  if (!JS_InstanceOf(aContext,
+                     jsobj,
+                     aClass,
+                     nsnull))
+  {
+    char buf[128];
+    sprintf(buf, "Parameter isn't an instance of type %s", aClass->name);
+    JS_ReportError(aContext, buf);
+    return JS_FALSE;
+  }
+  nsISupports *supports = (nsISupports *)JS_GetPrivate(aContext, jsobj);
+  if (!supports) {
+    NS_ERROR("This should not happen.");
+    JS_ReportError(aContext,
+                   "JSObject has JSCLASS_PRIVATE_IS_NSISUPPORTS flag but has a null private slot, please file a bug.");
+    return JS_FALSE;
+  }
+  if (NS_FAILED(supports->QueryInterface(aIID, (void **)aSupports))) {
+    char buf[128];
+    sprintf(buf, "Parameter must be of type %s", aClass->name);
+    JS_ReportError(aContext, buf);
+    return JS_FALSE;
+  }
 
   return JS_TRUE;
 }
@@ -329,10 +371,12 @@ void ConvertJSvalToVersionString(nsString& versionString, JSContext* cx, jsval a
 //
 // Native method AbortInstall
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallAbortInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   int32   b0;
 
   *rval = JSVAL_VOID;
@@ -367,10 +411,12 @@ InstallAbortInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval
 //
 // Native method AddDirectory
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallAddDirectory(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
   nsAutoString b0;
   nsAutoString b1;
@@ -519,10 +565,12 @@ InstallAddDirectory(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval
 //
 // Native method AddSubcomponent
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallAddSubcomponent(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
   nsAutoString b0;
   nsAutoString b1;
@@ -680,7 +728,7 @@ InstallAddSubcomponent(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, js
 //
 // Native method DeleteComponent
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallDeleteComponent(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
   // this function was once documented but never supported. Return an error,
@@ -692,10 +740,12 @@ InstallDeleteComponent(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, js
 //
 // Native method Execute
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallExecute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
   nsAutoString b0;
   nsAutoString b1;
@@ -757,10 +807,12 @@ InstallExecute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rva
 //
 // Native method FinalizeInstall
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallFinalizeInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
 
   *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
@@ -786,10 +838,12 @@ InstallFinalizeInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, js
 //
 // Native method Gestalt
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallGestalt(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
   nsAutoString b0;
 
@@ -827,10 +881,12 @@ InstallGestalt(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rva
 //
 // Native method GetComponentFolder
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallGetComponentFolder(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   nsInstallFolder* folder;
   nsAutoString b0;
   nsAutoString b1;
@@ -899,10 +955,12 @@ InstallGetComponentFolder(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
 //
 // Native method GetFolder
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallGetFolder(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   JSObject *jsObj;
   nsAutoString b0;
   nsAutoString b1;
@@ -1000,10 +1058,12 @@ InstallGetFolder(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *r
 //
 // Native method GetLastError
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallGetLastError(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
 
 
@@ -1027,22 +1087,20 @@ InstallGetLastError(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval
 //
 // Native method GetWinProfile
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallGetWinProfile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
   *rval = JSVAL_NULL;
 
 #ifdef _WINDOWS
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsAutoString b0;
   nsAutoString b1;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 2)
   {
     //  public int GetWinProfile (Object folder,
@@ -1070,20 +1128,17 @@ InstallGetWinProfile(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsva
 //
 // Native method GetWinRegistry
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallGetWinRegistry(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
   *rval = JSVAL_NULL;
 
 #ifdef _WINDOWS
 
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
-
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
 
   //  public int GetWinRegistry (void);
   if(NS_OK != nativeThis->GetWinRegistry(cx, &WinRegClass, rval))
@@ -1100,44 +1155,48 @@ InstallGetWinRegistry(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsv
 //
 // Native method LoadResources
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallLoadResources(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-    nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
-    nsAutoString b0;
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
 
-    *rval = JSVAL_NULL;
+  nsAutoString b0;
 
-    // If there's no private data, this must be the prototype, so ignore
-    if (nsnull == nativeThis) {
-        return JS_TRUE;
-    }
-
-    if (argc >= 1)
-    {
-        ConvertJSValToStr(b0, cx, argv[0]);
-        if (NS_OK != nativeThis->LoadResources(cx, b0, rval))
-        {
-            return JS_FALSE;
-        }
-    }
-    else
-    {
-        JS_ReportError(cx, "Function LoadResources requires 1 parameter");
-        return JS_FALSE;
-    }
+  *rval = JSVAL_NULL;
 
+  // If there's no private data, this must be the prototype, so ignore
+  if (nsnull == nativeThis) {
     return JS_TRUE;
+  }
+
+  if (argc >= 1)
+  {
+    ConvertJSValToStr(b0, cx, argv[0]);
+    if (NS_OK != nativeThis->LoadResources(cx, b0, rval))
+    {
+      return JS_FALSE;
+    }
+  }
+  else
+  {
+    JS_ReportError(cx, "Function LoadResources requires 1 parameter");
+    return JS_FALSE;
+  }
+
+  return JS_TRUE;
 }
 
 
 //
 // Native method Patch
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallPatch(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
   nsAutoString b0;
   nsAutoString b1;
@@ -1245,11 +1304,13 @@ InstallPatch(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 //         int  type,
 //         FileSpecObject chrome,
 //         String extraPath)
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallRegisterChrome(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
   // If there's no private data, this must be the prototype, so ignore
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   if (nsnull == nativeThis) {
     *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
     return JS_TRUE;
@@ -1280,7 +1341,7 @@ InstallRegisterChrome(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsv
     ConvertJSValToStr(path, cx, argv[2]);
   }
 
-  *rval = INT_TO_JSVAL(nativeThis->RegisterChrome(chrome, chromeType, NS_ConvertUCS2toUTF8(path).get()));
+  *rval = INT_TO_JSVAL(nativeThis->RegisterChrome(chrome, chromeType, NS_ConvertUTF16toUTF8(path).get()));
 
   return JS_TRUE;
 }
@@ -1289,11 +1350,13 @@ InstallRegisterChrome(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsv
 //
 // Native method RefreshPlugins
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallRefreshPlugins(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   // If there's no private data, this must be the prototype, so ignore
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
   if (!nativeThis)
   {
     *rval = INT_TO_JSVAL(nsInstall::UNEXPECTED_ERROR);
@@ -1313,10 +1376,11 @@ InstallRefreshPlugins(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsv
 //
 // Native method ResetError
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallResetError(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
 
   *rval = JSVAL_VOID;
 
@@ -1339,10 +1403,12 @@ InstallResetError(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *
 //
 // Native method SetPackageFolder
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallSetPackageFolder(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   nsAutoString b0;
   JSObject *jsObj;
   nsInstallFolder *folder;
@@ -1397,10 +1463,12 @@ InstallSetPackageFolder(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, j
 //
 // Native method StartInstall
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallStartInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
   nsAutoString b0;
   nsAutoString b1;
@@ -1449,10 +1517,12 @@ InstallStartInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval
 //
 // Native method Uninstall
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallUninstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+
   PRInt32 nativeRet;
   nsAutoString b0;
 
@@ -1488,7 +1558,7 @@ InstallUninstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *r
 
 /*START HACK FOR DEBUGGING UNTIL ALERTS WORK*/
 
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallTRACE(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
   nsAutoString b0;
@@ -1510,20 +1580,18 @@ InstallTRACE(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 //
 // Native method LogComment
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallLogComment(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsAutoString b0;
 
   *rval = JSVAL_NULL;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc >= 1)
   {
     //  public int LogComment (String aComment);
@@ -1545,20 +1613,18 @@ InstallLogComment(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *
 //
 // Native method InstallAlert
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallAlert(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsAutoString b0;
 
   *rval = JSVAL_NULL;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
-  {
-    return JS_TRUE;
-  }
-
   if(argc == 1)
   {
     //  public int InstallAlert (String aComment);
@@ -1573,7 +1639,7 @@ InstallAlert(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
   }
   else
   {
-    JS_ReportError(cx, "Function LogComment requires 1 parameter");
+    JS_ReportError(cx, "Function Alert requires 1 parameter");
     return JS_FALSE;
   }
 
@@ -1583,61 +1649,106 @@ InstallAlert(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 //
 // Native method InstallConfirm
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallConfirm(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsInstall *nativeThis = (nsInstall*)JS_GetPrivate(cx, obj);
-  nsAutoString b0;
+  nsInstall *nativeThis =
+    (nsInstall*)JS_GetInstancePrivate(cx, obj, &InstallClass, argv);
+  if (!nativeThis)
+    return JS_FALSE;
+
+  nsAutoString text;
+  nsAutoString title;
+  PRUint32 buttonFlags = nsIPromptService::STD_OK_CANCEL_BUTTONS;
+  nsAutoString button0Title;
+  nsAutoString button1Title;
+  nsAutoString button2Title;
+  nsAutoString checkMsg;
+  JSObject* checkObj = 0;
+  jsval jsCheckState = 0;
+  PRBool checkState = PR_FALSE;
   PRInt32 nativeRet;
 
-  *rval = JSVAL_NULL;
+  *rval = JSVAL_FALSE;
 
-  // If there's no private data, this must be the prototype, so ignore
-  if(nsnull == nativeThis)
+  if(argc == 0)
   {
-    return JS_TRUE;
-  }
-
-  if(argc == 1)
-  {
-    //  public int InstallConfirm (String aComment);
-
-    ConvertJSValToStr(b0, cx, argv[0]);
-
-    jsrefcount saveDepth = JS_SuspendRequest(cx);//Need to suspend use of thread or deadlock occurs
-
-    nativeThis->Confirm(b0, &nativeRet);
-
-    JS_ResumeRequest(cx, saveDepth);
-
-    *rval = INT_TO_JSVAL(nativeRet);
-  }
-  else
-  {
-    JS_ReportError(cx, "Function LogComment requires 1 parameter");
+    JS_ReportError(cx, "Function Confirm requires at least 1 parameter");
     return JS_FALSE;
   }
 
+  //  public int InstallConfirm (String aComment);
+  ConvertJSValToStr(text, cx, argv[0]);
+
+  //  Any additional parameters are optional.
+  //  public int InstallConfirm (String aText,
+  //                             String aDialogTitle,
+  //                             Number aButtonFlags,
+  //                             String aButton0Title,
+  //                             String aButton1Title,
+  //                             String aButton2Title,
+  //                             String aCheckMsg,
+  //                             Object aCheckState);
+  if (argc > 1)
+    ConvertJSValToStr(title, cx, argv[1]);
+
+  if(argc > 2)
+  {
+    if(!JSVAL_IS_INT(argv[2]))
+    {
+      JS_ReportError(cx, "Parameter 'aButtonFlags' must be a number");
+      return JS_FALSE;
+    }
+    buttonFlags = JSVAL_TO_INT(argv[2]);
+  }
+
+  if (argc > 3)
+    ConvertJSValToStr(button0Title, cx, argv[3]);
+  if (argc > 4)
+    ConvertJSValToStr(button1Title, cx, argv[4]);
+  if (argc > 5)
+    ConvertJSValToStr(button2Title, cx, argv[5]);
+  if (argc > 6)
+    ConvertJSValToStr(checkMsg, cx, argv[6]);
+
+  if (argc > 7 && !JSVAL_IS_PRIMITIVE(argv[7])) 
+  {
+    checkObj = JSVAL_TO_OBJECT(argv[7]);
+    if (!JS_GetProperty(cx, checkObj, "value", &jsCheckState) ||
+      !JSVAL_IS_BOOLEAN(jsCheckState))
+    {
+      JS_ReportError(cx, "Parameter 'aCheckState' must have a boolean 'value' property");
+      return JS_FALSE;
+    }
+    JS_ValueToBoolean(cx, jsCheckState, &checkState);
+  }
+
+  jsrefcount saveDepth = JS_SuspendRequest(cx); //Need to suspend use of thread or deadlock occurs
+
+  nativeThis->ConfirmEx(title, text, buttonFlags, button0Title, button1Title, button2Title, checkMsg, &checkState, &nativeRet);
+
+  JS_ResumeRequest(cx, saveDepth);
+
+  // Only save back checkState when an object was passed as 8th parameter
+  if (checkObj)
+  {
+    jsCheckState = BOOLEAN_TO_JSVAL(checkState);
+    JS_SetProperty(cx, checkObj, "value", &jsCheckState);
+  }
+
+  // To maintain compatibility with the classic one-arg confirm that returned 1
+  // for OK and 0 for cancel/close, we have to swap 0 and 1 here, as confirmEx
+  // always returns 0 for the first (i.e. OK) button.
+  if (nativeRet == 0)
+    nativeRet = 1; // 0 = first button = OK => true(1)
+  else if (nativeRet == 1)
+    nativeRet = 0; // 1 = second button or window closed = Cancel => false(0)
+
+  *rval = INT_TO_JSVAL(nativeRet);
+
   return JS_TRUE;
 }
 
-/***********************************************************************/
-//
-// class for Install
-//
-JSClass InstallClass = {
-  "Install",
-  JSCLASS_HAS_PRIVATE,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  GetInstallProperty,
-  SetInstallProperty,
-  JS_EnumerateStub,
-  JS_ResolveStub,
-  JS_ConvertStub,
-  FinalizeInstall
-};
-
 //
 // Install class properties
 //
@@ -1710,6 +1821,27 @@ static JSConstDoubleSpec install_constants[] =
     { nsInstall::SUCCESS,                    "SUCCESS"                      },
     { nsInstall::REBOOT_NEEDED,              "REBOOT_NEEDED"                },
     { nsInstall::INVALID_SIGNATURE,          "INVALID_SIGNATURE"            },
+    { nsInstall::INVALID_HASH,               "INVALID_HASH"                 },
+    { nsInstall::INVALID_HASH_TYPE,          "INVALID_HASH_TYPE"            },
+
+    // these are flags supported by confirm
+    { nsIPromptService::BUTTON_POS_0,            "BUTTON_POS_0"             },
+    { nsIPromptService::BUTTON_POS_1,            "BUTTON_POS_1"             },
+    { nsIPromptService::BUTTON_POS_2,            "BUTTON_POS_2"             },
+    { nsIPromptService::BUTTON_TITLE_OK,         "BUTTON_TITLE_OK"          },
+    { nsIPromptService::BUTTON_TITLE_CANCEL,     "BUTTON_TITLE_CANCEL"      },
+    { nsIPromptService::BUTTON_TITLE_YES,        "BUTTON_TITLE_YES"         },
+    { nsIPromptService::BUTTON_TITLE_NO,         "BUTTON_TITLE_NO"          },
+    { nsIPromptService::BUTTON_TITLE_SAVE,       "BUTTON_TITLE_SAVE"        },
+    { nsIPromptService::BUTTON_TITLE_DONT_SAVE,  "BUTTON_TITLE_DONT_SAVE"   },
+    { nsIPromptService::BUTTON_TITLE_REVERT,     "BUTTON_TITLE_REVERT"      },
+    { nsIPromptService::BUTTON_TITLE_IS_STRING,  "BUTTON_TITLE_IS_STRING"   },
+    { nsIPromptService::BUTTON_POS_0_DEFAULT,    "BUTTON_POS_0_DEFAULT"     },
+    { nsIPromptService::BUTTON_POS_1_DEFAULT,    "BUTTON_POS_1_DEFAULT"     },
+    { nsIPromptService::BUTTON_POS_2_DEFAULT,    "BUTTON_POS_2_DEFAULT"     },
+    { nsIPromptService::BUTTON_DELAY_ENABLE,     "BUTTON_DELAY_ENABLE"      },
+    { nsIPromptService::STD_OK_CANCEL_BUTTONS,   "STD_OK_CANCEL_BUTTONS"    },
+    { nsIPromptService::STD_YES_NO_BUTTONS,      "STD_YES_NO_BUTTONS"       },
 
     // these are bitwise values supported by addFile
     { DO_NOT_UNINSTALL,                      "DO_NOT_UNINSTALL"             },
@@ -1735,109 +1867,99 @@ static JSConstDoubleSpec install_constants[] =
 static JSFunctionSpec InstallMethods[] =
 {
 /*START HACK FOR DEBUGGING UNTIL ALERTS WORK*/
-  {"TRACE",                     InstallTRACE,                   1},
+  {"TRACE",                     InstallTRACE,                   1,0,0},
 /*END HACK FOR DEBUGGING UNTIL ALERTS WORK*/
   // -- new forms that match prevailing javascript style --
-  {"addDirectory",              InstallAddDirectory,            6},
-  {"addFile",                   InstallAddSubcomponent,         6},
-  {"alert",                     InstallAlert,                   1},
-  {"cancelInstall",             InstallAbortInstall,            1},
-  {"confirm",                   InstallConfirm,                 2},
-  {"execute",                   InstallExecute,                 2},
-  {"gestalt",                   InstallGestalt,                 1},
-  {"getComponentFolder",        InstallGetComponentFolder,      2},
-  {"getFolder",                 InstallGetFolder,               2},
-  {"getLastError",              InstallGetLastError,            0},
-  {"getWinProfile",             InstallGetWinProfile,           2},
-  {"getWinRegistry",            InstallGetWinRegistry,          0},
-  {"initInstall",               InstallStartInstall,            4},
-  {"loadResources",             InstallLoadResources,           1},
-  {"logComment",                InstallLogComment,              1},
-  {"patch",                     InstallPatch,                   5},
-  {"performInstall",            InstallFinalizeInstall,         0},
-  {"registerChrome",            InstallRegisterChrome,          2},
-  {"refreshPlugins",            InstallRefreshPlugins,          1},
-  {"resetError",                InstallResetError,              1},
-//  {"selectChrome",              InstallSelectChrome,            2},
-  {"setPackageFolder",          InstallSetPackageFolder,        1},
-  {"uninstall",                 InstallUninstall,               1},
+  {"addDirectory",              InstallAddDirectory,            6,0,0},
+  {"addFile",                   InstallAddSubcomponent,         6,0,0},
+  {"alert",                     InstallAlert,                   1,0,0},
+  {"cancelInstall",             InstallAbortInstall,            1,0,0},
+  {"confirm",                   InstallConfirm,                 8,0,0},
+  {"execute",                   InstallExecute,                 2,0,0},
+  {"gestalt",                   InstallGestalt,                 1,0,0},
+  {"getComponentFolder",        InstallGetComponentFolder,      2,0,0},
+  {"getFolder",                 InstallGetFolder,               2,0,0},
+  {"getLastError",              InstallGetLastError,            0,0,0},
+  {"getWinProfile",             InstallGetWinProfile,           2,0,0},
+  {"getWinRegistry",            InstallGetWinRegistry,          0,0,0},
+  {"initInstall",               InstallStartInstall,            4,0,0},
+  {"loadResources",             InstallLoadResources,           1,0,0},
+  {"logComment",                InstallLogComment,              1,0,0},
+  {"patch",                     InstallPatch,                   5,0,0},
+  {"performInstall",            InstallFinalizeInstall,         0,0,0},
+  {"registerChrome",            InstallRegisterChrome,          2,0,0},
+  {"refreshPlugins",            InstallRefreshPlugins,          1,0,0},
+  {"resetError",                InstallResetError,              1,0,0},
+//  {"selectChrome",              InstallSelectChrome,            2,0,0},
+  {"setPackageFolder",          InstallSetPackageFolder,        1,0,0},
+  {"uninstall",                 InstallUninstall,               1,0,0},
 
   // the raw file methods are deprecated, use the File object instead
-  {"dirCreate",                 InstallFileOpDirCreate,                1},
-  {"dirGetParent",              InstallFileOpDirGetParent,             1},
-  {"dirRemove",                 InstallFileOpDirRemove,                2},
-  {"dirRename",                 InstallFileOpDirRename,                2},
-  {"fileCopy",                  InstallFileOpFileCopy,                 2},
-  {"fileDelete",                InstallFileOpFileRemove,               1},
-  {"fileExists",                InstallFileOpFileExists,               1},
-  {"fileExecute",               InstallFileOpFileExecute,              2},
-  {"fileGetNativeVersion",      InstallFileOpFileGetNativeVersion,     1},
-  {"fileGetDiskSpaceAvailable", InstallFileOpFileGetDiskSpaceAvailable,1},
-  {"fileGetModDate",            InstallFileOpFileGetModDate,           1},
-  {"fileGetSize",               InstallFileOpFileGetSize,              1},
-  {"fileIsDirectory",           InstallFileOpFileIsDirectory,          1},
-  {"fileIsWritable",            InstallFileOpFileIsWritable,           1},
-  {"fileIsFile",                InstallFileOpFileIsFile,               1},
-  {"fileModDateChanged",        InstallFileOpFileModDateChanged,       2},
-  {"fileMove",                  InstallFileOpFileMove,                 2},
-  {"fileRename",                InstallFileOpFileRename,               2},
-  {"fileWindowsShortcut",       InstallFileOpFileWindowsShortcut,      7},
-  {"fileMacAlias",              InstallFileOpFileMacAlias,             2},
-  {"fileUnixLink",              InstallFileOpFileUnixLink,             2},
+  {"dirCreate",                 InstallFileOpDirCreate,                1,0,0},
+  {"dirGetParent",              InstallFileOpDirGetParent,             1,0,0},
+  {"dirRemove",                 InstallFileOpDirRemove,                2,0,0},
+  {"dirRename",                 InstallFileOpDirRename,                2,0,0},
+  {"fileCopy",                  InstallFileOpFileCopy,                 2,0,0},
+  {"fileDelete",                InstallFileOpFileRemove,               1,0,0},
+  {"fileExists",                InstallFileOpFileExists,               1,0,0},
+  {"fileExecute",               InstallFileOpFileExecute,              2,0,0},
+  {"fileGetNativeVersion",      InstallFileOpFileGetNativeVersion,     1,0,0},
+  {"fileGetDiskSpaceAvailable", InstallFileOpFileGetDiskSpaceAvailable,1,0,0},
+  {"fileGetModDate",            InstallFileOpFileGetModDate,           1,0,0},
+  {"fileGetSize",               InstallFileOpFileGetSize,              1,0,0},
+  {"fileIsDirectory",           InstallFileOpFileIsDirectory,          1,0,0},
+  {"fileIsWritable",            InstallFileOpFileIsWritable,           1,0,0},
+  {"fileIsFile",                InstallFileOpFileIsFile,               1,0,0},
+  {"fileModDateChanged",        InstallFileOpFileModDateChanged,       2,0,0},
+  {"fileMove",                  InstallFileOpFileMove,                 2,0,0},
+  {"fileRename",                InstallFileOpFileRename,               2,0,0},
+  {"fileWindowsShortcut",       InstallFileOpFileWindowsShortcut,      7,0,0},
+  {"fileMacAlias",              InstallFileOpFileMacAlias,             2,0,0},
+  {"fileUnixLink",              InstallFileOpFileUnixLink,             2,0,0},
 
   // -- documented but never supported --
-  {"deleteRegisteredFile",      InstallDeleteComponent,         1},
+  {"deleteRegisteredFile",      InstallDeleteComponent,         1,0,0},
 
   // -- obsolete forms for temporary compatibility --
-  {"abortInstall",              InstallAbortInstall,            1},
-  {"finalizeInstall",           InstallFinalizeInstall,         0},
-  {"startInstall",              InstallStartInstall,            4},
-  {0}
+  {"abortInstall",              InstallAbortInstall,            1,0,0},
+  {"finalizeInstall",           InstallFinalizeInstall,         0,0,0},
+  {"startInstall",              InstallStartInstall,            4,0,0},
+  {nsnull,nsnull,0,0,0}
 };
 
 
-
-
-
 JSObject * InitXPInstallObjects(JSContext *jscontext,
-                             JSObject *global,
                              nsIFile* jarfile,
                              const PRUnichar* url,
                              const PRUnichar* args,
                              PRUint32 flags,
-                             nsIXULChromeRegistry* reg,
+                             CHROMEREG_IFACE* reg,
                              nsIZipReader * theJARFile)
 {
-  JSObject *installObject  = nsnull;
+  JSObject *installObject;
   nsInstall *nativeInstallObject;
 
-    if (global == nsnull)
-    {
-        //we are the global
-        // new global object
-        global = JS_NewObject(jscontext, &InstallClass, nsnull, nsnull);
-    }
+  // new global object
+  installObject = JS_NewObject(jscontext, &InstallClass, nsnull, nsnull);
+  if (!installObject)
+    return nsnull;
 
-  installObject  = JS_InitClass( jscontext,         // context
-                                 global,            // global object
-                                 nsnull,            // parent proto
-                                 &InstallClass,     // JSClass
-                                 nsnull,            // JSNative ctor
-                                 0,                 // ctor args
-                                 nsnull,            // proto props
-                                 nsnull,            // proto funcs
-                                 InstallProperties, // ctor props (static)
-                                 InstallMethods);   // ctor funcs (static)
+  if (!JS_DefineProperty(jscontext, installObject, InstallClass.name,
+                         OBJECT_TO_JSVAL(installObject), NULL, NULL, 0))
+    return nsnull;
 
-  if (nsnull == installObject)
-  {
-      return nsnull;
-  }
+  if (!JS_DefineProperties(jscontext, installObject, InstallProperties))
+    return nsnull;
 
-  if ( PR_FALSE == JS_DefineConstDoubles(jscontext, installObject, install_constants) )
-            return nsnull;
+  if (!JS_DefineFunctions(jscontext, installObject, InstallMethods))
+    return nsnull;
+
+  if (!JS_DefineConstDoubles(jscontext, installObject, install_constants))
+    return nsnull;
 
   nativeInstallObject = new nsInstall(theJARFile);
+  if (!nativeInstallObject)
+    return nsnull;
 
   nativeInstallObject->SetJarFileLocation(jarfile);
   nativeInstallObject->SetInstallArguments(nsAutoString(args));
@@ -1852,20 +1974,25 @@ JSObject * InitXPInstallObjects(JSContext *jscontext,
   //
   // Initialize and create the FileOp object
   //
-  if(NS_OK != InitXPFileOpObjectPrototype(jscontext, global, &gFileOpProto))
+  if(NS_OK != InitXPFileOpObjectPrototype(jscontext, installObject, &gFileOpProto))
   {
     return nsnull;
   }
 
   gFileOpObject = JS_NewObject(jscontext, &FileOpClass, gFileOpProto, nsnull);
   if (gFileOpObject == nsnull)
-      return nsnull;
+    return nsnull;
 
   JS_SetPrivate(jscontext, gFileOpObject, nativeInstallObject);
 
-  JS_DefineProperty (jscontext, installObject, "File", OBJECT_TO_JSVAL(gFileOpObject),
-                     JS_PropertyStub, JS_PropertyStub, JSPROP_READONLY | JSPROP_PERMANENT);
-
+  if (!JS_DefineProperty (jscontext,
+                          installObject,
+                          "File", 
+                          OBJECT_TO_JSVAL(gFileOpObject),
+                          JS_PropertyStub, 
+                          JS_PropertyStub, 
+                          JSPROP_READONLY | JSPROP_PERMANENT))
+    return nsnull;
 
   //
   // Initialize the FileSpecObject
@@ -1883,15 +2010,15 @@ JSObject * InitXPInstallObjects(JSContext *jscontext,
   JSObject *winRegPrototype     = nsnull;
   JSObject *winProfilePrototype = nsnull;
 
-  if(NS_OK != InitWinRegPrototype(jscontext, global, &winRegPrototype))
+  if(NS_OK != InitWinRegPrototype(jscontext, installObject, &winRegPrototype))
   {
-      return nsnull;
+    return nsnull;
   }
   nativeInstallObject->SaveWinRegPrototype(winRegPrototype);
 
-  if(NS_OK != InitWinProfilePrototype(jscontext, global, &winProfilePrototype))
+  if(NS_OK != InitWinProfilePrototype(jscontext, installObject, &winProfilePrototype))
   {
-      return nsnull;
+    return nsnull;
   }
   nativeInstallObject->SaveWinProfilePrototype(winProfilePrototype);
 #endif
