@@ -44,8 +44,14 @@
 #include "nsString.h"
 #include "nsIDOMInstallVersion.h"
 #include "nsIDOMInstallTriggerGlobal.h"
+#include "nsPIDOMWindow.h"
+#include "nsIDOMDocument.h"
+#include "nsIDocument.h"
+#include "nsIDocShell.h"
+#include "nsIObserverService.h"
 #include "nsInstallTrigger.h"
 #include "nsXPITriggerInfo.h"
+#include "nsDOMJSUtils.h"
 
 #include "nsIComponentManager.h"
 #include "nsNetUtil.h"
@@ -65,16 +71,31 @@ extern PRBool ConvertJSValToBool(PRBool* aProp,
                                 JSContext* aContext,
                                 jsval aValue);
 
-extern PRBool ConvertJSValToObj(nsISupports** aSupports,
-                               REFNSIID aIID,
-                               const nsString& aTypeName,
-                               JSContext* aContext,
-                               jsval aValue);
+PR_STATIC_CALLBACK(void)
+FinalizeInstallTriggerGlobal(JSContext *cx, JSObject *obj);
+
+/***********************************************************************/
+//
+// class for InstallTriggerGlobal
+//
+JSClass InstallTriggerGlobalClass = {
+  "InstallTrigger",
+  JSCLASS_HAS_PRIVATE,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_PropertyStub,
+  JS_EnumerateStub,
+  JS_ResolveStub,
+  JS_ConvertStub,
+  FinalizeInstallTriggerGlobal
+};
+
 
 //
 // InstallTriggerGlobal finalizer
 //
-PR_STATIC_CALLBACK(void)
+JS_STATIC_DLL_CALLBACK(void)
 FinalizeInstallTriggerGlobal(JSContext *cx, JSObject *obj)
 {
   nsISupports *nativeThis = (nsISupports*)JS_GetPrivate(cx, obj);
@@ -123,19 +144,68 @@ static JSBool CreateNativeObject(JSContext *cx, JSObject *obj, nsIDOMInstallTrig
     return JS_TRUE;
 }
 
+//
+// Helper function for URI verification
+//
+static nsresult
+InstallTriggerCheckLoadURIFromScript(JSContext *cx, const nsAString& uriStr)
+{
+    nsresult rv;
+    nsCOMPtr<nsIScriptSecurityManager> secman(
+        do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID,&rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // get the script principal
+    nsCOMPtr<nsIPrincipal> principal;
+    rv = secman->GetSubjectPrincipal(getter_AddRefs(principal));
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (!principal)
+        return NS_ERROR_FAILURE;
+
+    // convert the requested URL string to a URI
+    // Note that we use a null base URI here, since that's what we use when we
+    // actually convert the string into a URI to load.
+    nsCOMPtr<nsIURI> uri;
+    rv = NS_NewURI(getter_AddRefs(uri), uriStr);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // are we allowed to load this one?
+    rv = secman->CheckLoadURIWithPrincipal(principal, uri,
+                    nsIScriptSecurityManager::DISALLOW_SCRIPT_OR_DATA);
+    return rv;
+}
+
+//
+// Helper function to get native object
+//
+// This is our own version of JS_GetInstancePrivate() that in addition
+// performs the delayed creation of the native InstallTrigger if necessary
+//
+static nsIDOMInstallTriggerGlobal* getTriggerNative(JSContext *cx, JSObject *obj)
+{
+  if (!JS_InstanceOf(cx, obj, &InstallTriggerGlobalClass, nsnull))
+    return nsnull;
+
+  nsIDOMInstallTriggerGlobal *native = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  if (!native) {
+    // xpinstall script contexts delay creation of the native.
+    CreateNativeObject(cx, obj, &native);
+  }
+  return native;
+}
+
 //
 // Native method UpdateEnabled
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallTriggerGlobalUpdateEnabled(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = getTriggerNative(cx, obj);
+  if (!nativeThis)
+    return JS_FALSE;
 
   *rval = JSVAL_FALSE;
 
-  if (nsnull == nativeThis  &&  (JS_FALSE == CreateNativeObject(cx, obj, &nativeThis)) )
-    return JS_TRUE;
-
   nsIScriptGlobalObject *globalObject = nsnull;
   nsIScriptContext *scriptContext = GetScriptContextFromJSContext(cx);
   if (scriptContext)
@@ -149,32 +219,55 @@ InstallTriggerGlobalUpdateEnabled(JSContext *cx, JSObject *obj, uintN argc, jsva
   return JS_TRUE;
 }
 
+
 //
 // Native method Install
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
-{
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+{ 
+  nsIDOMInstallTriggerGlobal *nativeThis = getTriggerNative(cx, obj);
+  if (!nativeThis)
+    return JS_FALSE;
 
   *rval = JSVAL_FALSE;
 
-  if (nsnull == nativeThis  &&  (JS_FALSE == CreateNativeObject(cx, obj, &nativeThis)) )
-    return JS_TRUE;
-
-
   // make sure XPInstall is enabled, return false if not
   nsIScriptGlobalObject *globalObject = nsnull;
   nsIScriptContext *scriptContext = GetScriptContextFromJSContext(cx);
   if (scriptContext)
-      globalObject = scriptContext->GetGlobalObject();
+    globalObject = scriptContext->GetGlobalObject();
 
   PRBool enabled = PR_FALSE;
   nativeThis->UpdateEnabled(globalObject, XPI_WHITELIST, &enabled);
   if (!enabled || !globalObject)
-      return JS_TRUE;
+  {
+    nsCOMPtr<nsPIDOMWindow> win(do_QueryInterface(globalObject));
+    nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+    if (os)
+    {
+      os->NotifyObservers(win->GetDocShell(), "xpinstall-install-blocked", 
+                          NS_LITERAL_STRING("install").get());
+    }
+    return JS_TRUE;
+  }
 
 
+  nsCOMPtr<nsIScriptSecurityManager> secman(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
+  if (!secman)
+  {
+    JS_ReportError(cx, "Could not the script security manager service.");
+    return JS_FALSE;
+  }
+  // get the principal.  if it doesn't exist, die.
+  nsCOMPtr<nsIPrincipal> principal;
+  secman->GetSubjectPrincipal(getter_AddRefs(principal));
+  if (!principal)
+  {
+    JS_ReportError(cx, "Could not get the Subject Principal during InstallTrigger.Install()");
+    return JS_FALSE;
+  }
+
   // get window.location to construct relative URLs
   nsCOMPtr<nsIURI> baseURL;
   JSObject* global = JS_GetGlobalObject(cx);
@@ -189,48 +282,58 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
     }
   }
 
-  // if we can't create a security manager we might be in the wizard, allow
   PRBool abortLoad = PR_FALSE;
-  nsCOMPtr<nsIScriptSecurityManager> secman(
-      do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-
 
   // parse associative array of installs
-  if ( argc >= 1 && JSVAL_IS_OBJECT(argv[0]) )
+  if ( argc >= 1 && JSVAL_IS_OBJECT(argv[0]) && JSVAL_TO_OBJECT(argv[0]) )
   {
     nsXPITriggerInfo *trigger = new nsXPITriggerInfo();
     if (!trigger)
       return JS_FALSE;
 
+    trigger->SetPrincipal(principal);
+
     JSIdArray *ida = JS_Enumerate( cx, JSVAL_TO_OBJECT(argv[0]) );
     if ( ida )
     {
       jsval v;
       const PRUnichar *name, *URL;
       const PRUnichar *iconURL = nsnull;
+      const char *hash;
 
       for (int i = 0; i < ida->length && !abortLoad; i++ )
       {
         JS_IdToValue( cx, ida->vector[i], &v );
-        name = NS_REINTERPRET_CAST(const PRUnichar*, JS_GetStringChars( JS_ValueToString( cx, v ) ));
+        JSString * str = JS_ValueToString( cx, v );
+        if (!str)
+        {
+          abortLoad = PR_TRUE;
+          break;
+        }
+
+        name = NS_REINTERPRET_CAST(const PRUnichar*, JS_GetStringChars( str ));
 
         URL = iconURL = nsnull;
+        hash = nsnull;
         JS_GetUCProperty( cx, JSVAL_TO_OBJECT(argv[0]), NS_REINTERPRET_CAST(const jschar*, name), nsCRT::strlen(name), &v );
-        if ( JSVAL_IS_OBJECT(v) ) 
+        if ( JSVAL_IS_OBJECT(v) && JSVAL_TO_OBJECT(v) ) 
         {
           jsval v2;
-          if (JS_GetProperty( cx, JSVAL_TO_OBJECT(v), "URL", &v2 ))
+          if (JS_GetProperty( cx, JSVAL_TO_OBJECT(v), "URL", &v2 ) && !JSVAL_IS_VOID(v2))
             URL = NS_REINTERPRET_CAST(const PRUnichar*, JS_GetStringChars( JS_ValueToString( cx, v2 ) ));
 
-          if (JS_GetProperty( cx, JSVAL_TO_OBJECT(v), "IconURL", &v2 ))
+          if (JS_GetProperty( cx, JSVAL_TO_OBJECT(v), "IconURL", &v2 ) && !JSVAL_IS_VOID(v2))
             iconURL = NS_REINTERPRET_CAST(const PRUnichar*, JS_GetStringChars( JS_ValueToString( cx, v2 ) ));
+
+          if (JS_GetProperty( cx, JSVAL_TO_OBJECT(v), "Hash", &v2) && !JSVAL_IS_VOID(v2))
+            hash = NS_REINTERPRET_CAST(const char*, JS_GetStringBytes( JS_ValueToString( cx, v2 ) ));
         }
         else
         {
           URL = NS_REINTERPRET_CAST(const PRUnichar*, JS_GetStringChars( JS_ValueToString( cx, v ) ));
         }
 
-        if ( name && URL )
+        if ( URL )
         {
             // Get relative URL to load
             nsAutoString xpiURL(URL);
@@ -241,19 +344,6 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
                 xpiURL = NS_ConvertUTF8toUTF16(resolvedURL);
             }
 
-            // Make sure we're allowed to load this URL
-            if (secman)
-            {
-                nsCOMPtr<nsIURI> uri;
-                nsresult rv = NS_NewURI(getter_AddRefs(uri), xpiURL);
-                if (NS_SUCCEEDED(rv))
-                {
-                    rv = secman->CheckLoadURIFromScript(cx, uri);
-                    if (NS_FAILED(rv))
-                        abortLoad = PR_TRUE;
-                }
-            }
-
             nsAutoString icon(iconURL);
             if (iconURL && baseURL)
             {
@@ -262,13 +352,30 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
                 icon = NS_ConvertUTF8toUTF16(resolvedIcon);
             }
 
-            nsXPITriggerItem *item = new nsXPITriggerItem( name, xpiURL.get(), icon.get() );
-            if ( item )
-            {
-                trigger->Add( item );
-            }
-            else
+            // Make sure we're allowed to load this URL and the icon URL
+            nsresult rv = InstallTriggerCheckLoadURIFromScript(cx, xpiURL);
+            if (NS_FAILED(rv))
                 abortLoad = PR_TRUE;
+
+            if (!abortLoad && iconURL)
+            {
+                rv = InstallTriggerCheckLoadURIFromScript(cx, icon);
+                if (NS_FAILED(rv))
+                    abortLoad = PR_TRUE;
+            }
+
+            if (!abortLoad)
+            {
+                // Add the install item to the trigger collection
+                nsXPITriggerItem *item =
+                    new nsXPITriggerItem( name, xpiURL.get(), icon.get(), hash );
+                if ( item )
+                {
+                    trigger->Add( item );
+                }
+                else
+                    abortLoad = PR_TRUE;
+            }
         }
         else
             abortLoad = PR_TRUE;
@@ -304,20 +411,19 @@ InstallTriggerGlobalInstall(JSContext *cx, JSObject *obj, uintN argc, jsval *arg
 //
 // Native method InstallChrome
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallTriggerGlobalInstallChrome(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = getTriggerNative(cx, obj);
+  if (!nativeThis)
+    return JS_FALSE;
+
   uint32       chromeType = NOT_CHROME;
   nsAutoString sourceURL;
   nsAutoString name;
 
   *rval = JSVAL_FALSE;
 
-  if (nsnull == nativeThis  &&  (JS_FALSE == CreateNativeObject(cx, obj, &nativeThis)) ) {
-    return JS_TRUE;
-  }
-
   // get chromeType first, the update enabled check for skins skips whitelisting
   if (argc >=1)
       JS_ValueToECMAUint32(cx, argv[0], &chromeType);
@@ -329,10 +435,18 @@ InstallTriggerGlobalInstallChrome(JSContext *cx, JSObject *obj, uintN argc, jsva
       globalObject = scriptContext->GetGlobalObject();
 
   PRBool enabled = PR_FALSE;
-  PRBool useWhitelist = ( chromeType != CHROME_SKIN );
-  nativeThis->UpdateEnabled(globalObject, useWhitelist, &enabled);
+  nativeThis->UpdateEnabled(globalObject, XPI_WHITELIST, &enabled);
   if (!enabled || !globalObject)
-      return JS_TRUE;
+  {
+    nsCOMPtr<nsPIDOMWindow> win(do_QueryInterface(globalObject));
+    nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+    if (os)
+    {
+      os->NotifyObservers(win->GetDocShell(), "xpinstall-install-blocked", 
+                          NS_LITERAL_STRING("install").get());
+    }
+    return JS_TRUE;
+  }
 
 
   // get window.location to construct relative URLs
@@ -363,20 +477,9 @@ InstallTriggerGlobalInstallChrome(JSContext *cx, JSObject *obj, uintN argc, jsva
     }
 
     // Make sure caller is allowed to load this url.
-    // if we can't create a security manager we might be in the wizard, allow
-    nsCOMPtr<nsIScriptSecurityManager> secman(
-        do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-    if (secman)
-    {
-        nsCOMPtr<nsIURI> uri;
-        nsresult rv = NS_NewURI(getter_AddRefs(uri), sourceURL);
-        if (NS_SUCCEEDED(rv))
-        {
-            rv = secman->CheckLoadURIFromScript(cx, uri);
-            if (NS_FAILED(rv))
-                return JS_FALSE;
-        }
-    }
+    nsresult rv = InstallTriggerCheckLoadURIFromScript(cx, sourceURL);
+    if (NS_FAILED(rv))
+        return JS_FALSE;
 
     if ( chromeType & CHROME_ALL )
     {
@@ -397,18 +500,18 @@ InstallTriggerGlobalInstallChrome(JSContext *cx, JSObject *obj, uintN argc, jsva
 //
 // Native method StartSoftwareUpdate
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallTriggerGlobalStartSoftwareUpdate(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = getTriggerNative(cx, obj);
+  if (!nativeThis)
+    return JS_FALSE;
+
   PRBool       nativeRet;
   PRInt32      flags = 0;
 
   *rval = JSVAL_FALSE;
 
-  if (nsnull == nativeThis  &&  (JS_FALSE == CreateNativeObject(cx, obj, &nativeThis)) )
-    return JS_TRUE;
-
   // make sure XPInstall is enabled, return if not
   nsIScriptGlobalObject *globalObject = nsnull;
   nsIScriptContext *scriptContext = GetScriptContextFromJSContext(cx);
@@ -418,7 +521,16 @@ InstallTriggerGlobalStartSoftwareUpdate(JSContext *cx, JSObject *obj, uintN argc
   PRBool enabled = PR_FALSE;
   nativeThis->UpdateEnabled(globalObject, XPI_WHITELIST, &enabled);
   if (!enabled || !globalObject)
-      return JS_TRUE;
+  {
+    nsCOMPtr<nsPIDOMWindow> win(do_QueryInterface(globalObject));
+    nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+    if (os)
+    {
+      os->NotifyObservers(win->GetDocShell(), "xpinstall-install-blocked", 
+                          NS_LITERAL_STRING("install").get());
+    }
+    return JS_TRUE;
+  }
 
   // get window.location to construct relative URLs
   nsCOMPtr<nsIURI> baseURL;
@@ -447,20 +559,9 @@ InstallTriggerGlobalStartSoftwareUpdate(JSContext *cx, JSObject *obj, uintN argc
     }
 
     // Make sure caller is allowed to load this url.
-    // if we can't create a security manager we might be in the wizard, allow
-    nsCOMPtr<nsIScriptSecurityManager> secman(
-        do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-    if (secman)
-    {
-        nsCOMPtr<nsIURI> uri;
-        nsresult rv = NS_NewURI(getter_AddRefs(uri), xpiURL);
-        if (NS_SUCCEEDED(rv))
-        {
-            rv = secman->CheckLoadURIFromScript(cx, uri);
-            if (NS_FAILED(rv))
-                return JS_FALSE;
-        }
-    }
+    nsresult rv = InstallTriggerCheckLoadURIFromScript(cx, xpiURL);
+    if (NS_FAILED(rv))
+        return JS_FALSE;
 
     if (argc >= 2 && !JS_ValueToInt32(cx, argv[1], (int32 *)&flags))
     {
@@ -486,10 +587,13 @@ InstallTriggerGlobalStartSoftwareUpdate(JSContext *cx, JSObject *obj, uintN argc
 //
 // Native method CompareVersion
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallTriggerGlobalCompareVersion(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = getTriggerNative(cx, obj);
+  if (!nativeThis)
+    return JS_FALSE;
+
   nsAutoString regname;
   nsAutoString version;
   int32        major,minor,release,build;
@@ -498,10 +602,6 @@ InstallTriggerGlobalCompareVersion(JSContext *cx, JSObject *obj, uintN argc, jsv
   PRInt32 nativeRet = nsIDOMInstallTriggerGlobal::NOT_FOUND;
   *rval = INT_TO_JSVAL(nativeRet);
 
-  if (nsnull == nativeThis  &&  (JS_FALSE == CreateNativeObject(cx, obj, &nativeThis)) )
-    return JS_TRUE;
-
-
   // make sure XPInstall is enabled, return if not
   nsIScriptGlobalObject *globalObject = nsnull;
   nsIScriptContext *scriptContext = GetScriptContextFromJSContext(cx);
@@ -585,10 +685,12 @@ InstallTriggerGlobalCompareVersion(JSContext *cx, JSObject *obj, uintN argc, jsv
 //
 // Native method GetVersion
 //
-PR_STATIC_CALLBACK(JSBool)
+JS_STATIC_DLL_CALLBACK(JSBool)
 InstallTriggerGlobalGetVersion(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-  nsIDOMInstallTriggerGlobal *nativeThis = (nsIDOMInstallTriggerGlobal*)JS_GetPrivate(cx, obj);
+  nsIDOMInstallTriggerGlobal *nativeThis = getTriggerNative(cx, obj);
+  if (!nativeThis)
+    return JS_FALSE;
 
   nsAutoString regname;
   nsAutoString version;
@@ -596,10 +698,6 @@ InstallTriggerGlobalGetVersion(JSContext *cx, JSObject *obj, uintN argc, jsval *
   // In case of error return a null value
   *rval = JSVAL_NULL;
 
-  if (nsnull == nativeThis  &&  (JS_FALSE == CreateNativeObject(cx, obj, &nativeThis)) )
-      return JS_TRUE;
-
-
   // make sure XPInstall is enabled, return if not
   nsIScriptGlobalObject *globalObject = nsnull;
   nsIScriptContext *scriptContext = GetScriptContextFromJSContext(cx);
@@ -623,42 +721,25 @@ InstallTriggerGlobalGetVersion(JSContext *cx, JSObject *obj, uintN argc, jsval *
   return JS_TRUE;
 }
 
-/***********************************************************************/
-//
-// class for InstallTriggerGlobal
-//
-JSClass InstallTriggerGlobalClass = {
-  "InstallTrigger",
-  JSCLASS_HAS_PRIVATE,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_PropertyStub,
-  JS_EnumerateStub,
-  JS_ResolveStub,
-  JS_ConvertStub,
-  FinalizeInstallTriggerGlobal
-};
-
 //
 // InstallTriggerGlobal class methods
 //
 static JSFunctionSpec InstallTriggerGlobalMethods[] =
 {
   // -- obsolete forms, do not document. Kept for 4.x compatibility
-  {"UpdateEnabled",             InstallTriggerGlobalUpdateEnabled,             0},
-  {"StartSoftwareUpdate",       InstallTriggerGlobalStartSoftwareUpdate,       2},
-  {"CompareVersion",            InstallTriggerGlobalCompareVersion,            5},
-  {"GetVersion",                InstallTriggerGlobalGetVersion,                2},
-  {"updateEnabled",             InstallTriggerGlobalUpdateEnabled,             0},
+  {"UpdateEnabled",         InstallTriggerGlobalUpdateEnabled,         0,0,0},
+  {"StartSoftwareUpdate",   InstallTriggerGlobalStartSoftwareUpdate,   2,0,0},
+  {"CompareVersion",        InstallTriggerGlobalCompareVersion,        5,0,0},
+  {"GetVersion",            InstallTriggerGlobalGetVersion,            2,0,0},
+  {"updateEnabled",         InstallTriggerGlobalUpdateEnabled,         0,0,0},
   // -- new forms to match JS style --
-  {"enabled",                   InstallTriggerGlobalUpdateEnabled,             0},
-  {"install",                   InstallTriggerGlobalInstall,                   2},
-  {"installChrome",             InstallTriggerGlobalInstallChrome,             2},
-  {"startSoftwareUpdate",       InstallTriggerGlobalStartSoftwareUpdate,       2},
-  {"compareVersion",            InstallTriggerGlobalCompareVersion,            5},
-  {"getVersion",                InstallTriggerGlobalGetVersion,                2},
-  {0}
+  {"enabled",               InstallTriggerGlobalUpdateEnabled,         0,0,0},
+  {"install",               InstallTriggerGlobalInstall,               2,0,0},
+  {"installChrome",         InstallTriggerGlobalInstallChrome,         2,0,0},
+  {"startSoftwareUpdate",   InstallTriggerGlobalStartSoftwareUpdate,   2,0,0},
+  {"compareVersion",        InstallTriggerGlobalCompareVersion,        5,0,0},
+  {"getVersion",            InstallTriggerGlobalGetVersion,            2,0,0},
+  {nsnull,nsnull,0,0,0}
 };
 
 
@@ -674,7 +755,7 @@ static JSConstDoubleSpec diff_constants[] =
     { CHROME_LOCALE,                             "LOCALE"     },
     { CHROME_CONTENT,                            "CONTENT"    },
     { CHROME_ALL,                                "PACKAGE"    },
-    {0}
+    {0,nsnull}
 };
 
 
