@@ -1,4 +1,5 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set ts=4 sw=4 et tw=80: */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -40,59 +41,45 @@
 
 #include "nsSoftwareUpdate.h"
 #include "nsSoftwareUpdateRun.h"
-#include "nsSoftwareUpdateIIDs.h"
 
 #include "nsInstall.h"
 
-#include "nsIComponentManager.h"
-#include "nsIServiceManager.h"
-
-#include "nsProxiedService.h"
-#include "nsIURI.h"
-#include "nsIFileURL.h"
 #include "nsNetUtil.h"
 
 #include "nspr.h"
+#include "plstr.h"
 #include "jsapi.h"
 
-#include "nsIEventQueueService.h"
-#include "nsIEnumerator.h"
 #include "nsIZipReader.h"
-#include "nsIJSRuntimeService.h"
-#include "nsCOMPtr.h"
-#include "nsXPIDLString.h"
-#include "nsReadableUtils.h"
-#include "nsILocalFile.h"
-#include "nsIChromeRegistry.h"
 #include "nsInstallTrigger.h"
 #include "nsIConsoleService.h"
 #include "nsIScriptError.h"
+#include "nsStringEnumerator.h"
 
 #include "nsIJAR.h"
 #include "nsIPrincipal.h"
 
-#ifndef MOZ_XUL_APP
-#include "nsIChromeRegistrySea.h"
-#endif
+#include "nsIExtensionManager.h"
 
 static NS_DEFINE_CID(kSoftwareUpdateCID,  NS_SoftwareUpdate_CID);
-static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
 
-extern JSObject *InitXPInstallObjects(JSContext *jscontext, JSObject *global, 
+extern JSObject *InitXPInstallObjects(JSContext *jscontext, 
                                       nsIFile* jarfile, const PRUnichar* url, 
                                       const PRUnichar* args, PRUint32 flags, 
-                                      nsIXULChromeRegistry* registry, 
+                                      CHROMEREG_IFACE* registry, 
                                       nsIZipReader* hZip);
 extern nsresult InitInstallVersionClass(JSContext *jscontext, JSObject *global, void** prototype);
 extern nsresult InitInstallTriggerGlobalClass(JSContext *jscontext, JSObject *global, void** prototype);
 
 // Defined in this file:
-PR_STATIC_CALLBACK(void) XPInstallErrorReporter(JSContext *cx, const char *message, JSErrorReport *report);
-static PRInt32  GetInstallScriptFromJarfile(nsIZipReader* hZip, nsIFile* jarFile, nsIPrincipal* aPrincipal, char** scriptBuffer, PRUint32 *scriptLength);
-static nsresult SetupInstallContext(nsIZipReader* hZip, nsIFile* jarFile, const PRUnichar* url, const PRUnichar* args, 
-                                    PRUint32 flags, nsIXULChromeRegistry* reg, JSRuntime *jsRT, JSContext **jsCX, JSObject **jsGlob);
+JS_STATIC_DLL_CALLBACK(void) XPInstallErrorReporter(JSContext *cx, const char *message, JSErrorReport *report);
+static PRInt32  GetInstallScriptFromJarfile(nsIZipReader* hZip, char** scriptBuffer, PRUint32 *scriptLength);
+static PRInt32  OpenAndValidateArchive(nsIZipReader* hZip, nsIFile* jarFile, nsIPrincipal* aPrincipal);
 
-extern "C" void RunInstallOnThread(void *data);
+static nsresult SetupInstallContext(nsIZipReader* hZip, nsIFile* jarFile, const PRUnichar* url, const PRUnichar* args, 
+                                    PRUint32 flags, CHROMEREG_IFACE* reg, JSRuntime *jsRT, JSContext **jsCX, JSObject **jsGlob);
+
+extern "C" void PR_CALLBACK RunInstallOnThread(void *data);
 
 
 nsresult VerifySigning(nsIZipReader* hZip, nsIPrincipal* aPrincipal)
@@ -118,32 +105,32 @@ nsresult VerifySigning(nsIZipReader* hZip, nsIPrincipal* aPrincipal)
     PRUint32 entryCount = 0;
 
     // first verify all files in the jar are also in the manifest.
-    nsCOMPtr<nsISimpleEnumerator> entries;
-    rv = hZip->FindEntries("*", getter_AddRefs(entries));
+    nsCOMPtr<nsIUTF8StringEnumerator> entries;
+    rv = hZip->FindEntries(nsnull, getter_AddRefs(entries));
     if (NS_FAILED(rv))
         return rv;
 
     PRBool more;
-    nsXPIDLCString name;
-    while (NS_SUCCEEDED(entries->HasMoreElements(&more)) && more)
+    nsCAutoString name;
+    while (NS_SUCCEEDED(entries->HasMore(&more)) && more)
     {
-        nsCOMPtr<nsIZipEntry> file;
-        rv = entries->GetNext(getter_AddRefs(file));
+        rv = entries->GetNext(name);
         if (NS_FAILED(rv)) return rv;
         
-        file->GetName(getter_Copies(name));
-
-        if ( PL_strncasecmp("META-INF/", name.get(), 9) == 0)
+        // Do not verify the directory entries or 
+        // entries which are in the meta-inf directory
+        if ((name.Last() == '/') || 
+            (PL_strncasecmp("META-INF/", name.get(), 9) == 0))
             continue;
 
-        // we only count the entries not in the meta-inf directory
+        // Count the entries to be verified
         entryCount++;
 
         // Each entry must be signed
-        PRBool equal;
-        rv = jar->GetCertificatePrincipal(name, getter_AddRefs(principal));
+        rv = jar->GetCertificatePrincipal(name.get(), getter_AddRefs(principal));
         if (NS_FAILED(rv) || !principal) return NS_ERROR_FAILURE;
 
+        PRBool equal;
         rv = principal->Equals(aPrincipal, &equal);
         if (NS_FAILED(rv) || !equal) return NS_ERROR_FAILURE;
     }
@@ -193,7 +180,7 @@ XPInstallErrorReporter(JSContext *cx, const char *message, JSErrorReport *report
         PRUint32 column = report->uctokenptr - report->uclinebuf;
 
         rv = errorObject->Init(NS_REINTERPRET_CAST(const PRUnichar*, report->ucmessage),
-                               NS_ConvertASCIItoUCS2(report->filename).get(),
+                               NS_ConvertASCIItoUTF16(report->filename).get(),
                                NS_REINTERPRET_CAST(const PRUnichar*, report->uclinebuf),
                                report->lineno, column, report->flags,
                                "XPInstall JavaScript");
@@ -241,58 +228,67 @@ XPInstallErrorReporter(JSContext *cx, const char *message, JSErrorReport *report
 }
 
 
-
-
-
 ///////////////////////////////////////////////////////////////////////////////////////////////
-// Function name    : GetInstallScriptFromJarfile
-// Description      : Extracts and reads in a install.js file from a passed jar file.
-// Return type      : static PRInt32
-// Argument         : const char* jarFile     - **NSPR** filepath
+// Function name    : OpenAndValidateArchive
+// Description      : Opens install archive and validates contents
+// Return type      : PRInt32
+// Argument         : nsIZipReader* hZip       - the zip reader
+// Argument         : nsIFile* jarFile         - the .xpi file
 // Argument         : nsIPrincipal* aPrincipal - a principal, if any, displayed to the user 
 //                    regarding the cert used to sign this install
-// Argument         : char** scriptBuffer     - must be deleted via delete []
-// Argument         : PRUint32 *scriptLength
 ///////////////////////////////////////////////////////////////////////////////////////////////
 
 static PRInt32
-GetInstallScriptFromJarfile(nsIZipReader* hZip, nsIFile* jarFile, nsIPrincipal* aPrincipal, char** scriptBuffer, PRUint32 *scriptLength)
+OpenAndValidateArchive(nsIZipReader* hZip, nsIFile* jarFile, nsIPrincipal* aPrincipal)
 {
-    PRInt32 result = NS_OK;
+    if (!jarFile)
+        return nsInstall::DOWNLOAD_ERROR;
 
-    *scriptBuffer = nsnull;
-    *scriptLength = 0;
-
-    nsIFile* jFile;
-    nsresult rv =jarFile->Clone(&jFile);
+    nsCOMPtr<nsIFile> jFile;
+    nsresult rv =jarFile->Clone(getter_AddRefs(jFile));
     if (NS_SUCCEEDED(rv))
-      rv = hZip->Init(jFile);
+        rv = hZip->Open(jFile);
 
     if (NS_FAILED(rv))
         return nsInstall::CANT_READ_ARCHIVE;
 
-    rv = hZip->Open();
-    if (NS_FAILED(rv))
-        return nsInstall::CANT_READ_ARCHIVE;
-
     // CRC check the integrity of all items in this archive
     rv = hZip->Test(nsnull);
     if (NS_FAILED(rv))
     {
-        NS_ASSERTION(0, "CRC check of archive failed!");
+        NS_WARNING("CRC check of archive failed!");
         return nsInstall::CANT_READ_ARCHIVE;
     }
 
     rv = VerifySigning(hZip, aPrincipal);
     if (NS_FAILED(rv))
     {
-        NS_ASSERTION(0, "Signing check of archive failed!");
+        NS_WARNING("Signing check of archive failed!");
         return nsInstall::INVALID_SIGNATURE;
     }
+ 
+    return nsInstall::SUCCESS;
+}
+
+///////////////////////////////////////////////////////////////////////////////////////////////
+// Function name    : GetInstallScriptFromJarfile
+// Description      : Extracts and reads in a install.js file from a passed jar file.
+// Return type      : static PRInt32
+// Argument         : char** scriptBuffer     - must be deleted via delete []
+// Argument         : PRUint32 *scriptLength
+///////////////////////////////////////////////////////////////////////////////////////////////
+
+static PRInt32
+GetInstallScriptFromJarfile(nsIZipReader* hZip, char** scriptBuffer, PRUint32 *scriptLength)
+{
+    PRInt32 result = NS_OK;
+
+    *scriptBuffer = nsnull;
+    *scriptLength = 0;
 
     // Extract the install.js file.
     nsCOMPtr<nsIInputStream> instream;
-    rv = hZip->GetInputStream("install.js", getter_AddRefs(instream));
+    nsresult rv = hZip->GetInputStream("install.js", getter_AddRefs(instream));
     if ( NS_SUCCEEDED(rv) )
     {
         // Read it into a buffer
@@ -351,7 +347,7 @@ static nsresult SetupInstallContext(nsIZipReader* hZip,
                                     const PRUnichar* url,
                                     const PRUnichar* args,
                                     PRUint32 flags,
-                                    nsIXULChromeRegistry* reg,
+                                    CHROMEREG_IFACE* reg,
                                     JSRuntime *rt,
                                     JSContext **jsCX,
                                     JSObject **jsGlob)
@@ -373,15 +369,21 @@ static nsresult SetupInstallContext(nsIZipReader* hZip,
 
     JS_SetErrorReporter(cx, XPInstallErrorReporter);
 
+    JS_BeginRequest(cx);
+    glob = InitXPInstallObjects(cx, jarFile, url, args, flags, reg, hZip);
+    if (!glob)
+    {
+        JS_DestroyContext(cx);
+        return NS_ERROR_OUT_OF_MEMORY;
+    }
 
-    glob = InitXPInstallObjects(cx, nsnull, jarFile, url, args, flags, reg, hZip);
     // Init standard classes
     JS_InitStandardClasses(cx, glob);
 
     // Add our Install class to this context
     InitInstallVersionClass(cx, glob, nsnull);
     InitInstallTriggerGlobalClass(cx, glob, nsnull);
-
+    JS_EndRequest(cx);
     *jsCX   = cx;
     *jsGlob = glob;
 
@@ -425,7 +427,7 @@ PRInt32 RunInstall(nsInstallInfo *installInfo)
 // Return type      : extern "C"
 // Argument         : void *data
 ///////////////////////////////////////////////////////////////////////////////////////////////
-extern "C" void RunInstallOnThread(void *data)
+extern "C" void PR_CALLBACK RunInstallOnThread(void *data)
 {
     nsInstallInfo *installInfo = (nsInstallInfo*)data;
 
@@ -449,16 +451,6 @@ extern "C" void RunInstallOnThread(void *data)
 
     nsCOMPtr<nsIXPIListener> listener;
 
-    // lets set up an eventQ so that our xpcom/proxies will not have to:
-    nsCOMPtr<nsIEventQueue> eventQ;
-    nsCOMPtr<nsIEventQueueService> eventQService =
-             do_GetService(kEventQueueServiceCID, &rv);
-    if (NS_SUCCEEDED(rv))
-    {
-        eventQService->CreateMonitoredThreadEventQueue();
-        eventQService->GetThreadEventQueue(NS_CURRENT_THREAD, getter_AddRefs(eventQ));
-    }
-
     nsCOMPtr<nsISoftwareUpdate> softwareUpdate =
              do_GetService(kSoftwareUpdateCID, &rv);
 
@@ -476,104 +468,113 @@ extern "C" void RunInstallOnThread(void *data)
 
     nsCOMPtr<nsIFile> jarpath = installInfo->GetFile();
 
-    if (NS_SUCCEEDED(rv))
+    finalStatus = OpenAndValidateArchive( hZip,
+                                          jarpath,
+                                          installInfo->mPrincipal);
+
+    if (finalStatus == nsInstall::SUCCESS)
     {
-        finalStatus = GetInstallScriptFromJarfile( hZip,
-                                                   jarpath,
-                                                   installInfo->mPrincipal,
-                                                   &scriptBuffer,
-                                                   &scriptLength);
-
-        if ( finalStatus == NS_OK && scriptBuffer )
+#ifdef MOZ_XUL_APP
+        if (NS_SUCCEEDED(hZip->Test("install.rdf")) && !(nsSoftwareUpdate::GetProgramDirectory()))
         {
-            PRBool ownRuntime = PR_FALSE;
-
-            nsCOMPtr<nsIJSRuntimeService> rtsvc =
-                     do_GetService("@mozilla.org/js/xpc/RuntimeService;1", &rv);
-            if(NS_FAILED(rv) || NS_FAILED(rtsvc->GetRuntime(&rt)))
+            hZip->Close();
+            // appears to be an Extension Manager install
+            nsIExtensionManager* em = installInfo->GetExtensionManager();
+            if (em)
             {
-                // service not available (wizard context?)
-                // create our own runtime
-                ownRuntime = PR_TRUE;
-                rt = JS_Init(4L * 1024L * 1024L);
+                rv = em->InstallItemFromFile(jarpath, 
+                                             NS_INSTALL_LOCATION_APPPROFILE);
+                if (NS_FAILED(rv))
+                    finalStatus = nsInstall::EXECUTION_ERROR;
+            } else {
+                finalStatus = nsInstall::UNEXPECTED_ERROR;
             }
-
-            rv = SetupInstallContext( hZip, jarpath,
-                                      installInfo->GetURL(),
-                                      installInfo->GetArguments(),
-                                      installInfo->GetFlags(),
-                                      installInfo->GetChromeRegistry(),
-                                      rt, &cx, &glob);
-
-            if (NS_SUCCEEDED(rv))
+            // If install.rdf exists, but the install failed, we don't want
+            // to try an install.js install.
+        } else
+#endif
+        {
+            // If we're the suite, or there is no install.rdf,
+            // try original XPInstall
+            finalStatus = GetInstallScriptFromJarfile( hZip,
+                                                       &scriptBuffer,
+                                                       &scriptLength);
+            if ( finalStatus == NS_OK && scriptBuffer )
             {
-                // Go ahead and run!!
-                jsval rval;
-                jsval installedFiles;
-                JS_BeginRequest(cx); //Increment JS thread counter associated
-                                     //with this context
-                PRBool ok = JS_EvaluateScript(  cx,
-                                                glob,
-                                                scriptBuffer,
-                                                scriptLength,
-                                                nsnull,
-                                                0,
-                                                &rval);
+                // create our runtime
+                rt = JS_NewRuntime(4L * 1024L * 1024L);
 
+                rv = SetupInstallContext( hZip, jarpath,
+                                          installInfo->GetURL(),
+                                          installInfo->GetArguments(),
+                                          installInfo->GetFlags(),
+                                          installInfo->GetChromeRegistry(),
+                                          rt, &cx, &glob);
 
-                if(!ok)
+                if (NS_SUCCEEDED(rv))
                 {
-                    // problem compiling or running script -- a true SCRIPT_ERROR
-                    if(JS_GetProperty(cx, glob, "_installedFiles", &installedFiles) &&
-                       JSVAL_TO_BOOLEAN(installedFiles))
-                    {
-                        nsInstall *a = (nsInstall*)JS_GetPrivate(cx, glob);
-                        a->InternalAbort(nsInstall::SCRIPT_ERROR);
-                    }
+                    // Go ahead and run!!
+                    jsval rval;
+                    jsval installedFiles;
+                    JS_BeginRequest(cx); //Increment JS thread counter associated
+                                        //with this context
+                    PRBool ok = JS_EvaluateScript(  cx,
+                                                    glob,
+                                                    scriptBuffer,
+                                                    scriptLength,
+                                                    nsnull,
+                                                    0,
+                                                    &rval);
 
-                    finalStatus = nsInstall::SCRIPT_ERROR;
+
+                    if(!ok)
+                    {
+                        // problem compiling or running script -- a true SCRIPT_ERROR
+                        if(JS_GetProperty(cx, glob, "_installedFiles", &installedFiles) &&
+                          JSVAL_TO_BOOLEAN(installedFiles))
+                        {
+                            nsInstall *a = (nsInstall*)JS_GetPrivate(cx, glob);
+                            a->InternalAbort(nsInstall::SCRIPT_ERROR);
+                        }
+
+                        finalStatus = nsInstall::SCRIPT_ERROR;
+                    }
+                    else
+                    {
+                        // check to make sure the script sent back a status -- if
+                        // not the install may have been syntactically correct but
+                        // left the init/(perform|cancel) transaction open
+
+                        if(JS_GetProperty(cx, glob, "_installedFiles", &installedFiles) &&
+                          JSVAL_TO_BOOLEAN(installedFiles))
+                        {
+                            // install items remain in queue, must clean up!
+                            nsInstall *a = (nsInstall*)JS_GetPrivate(cx, glob);
+                            a->InternalAbort(nsInstall::MALFORMED_INSTALL);
+                        }
+
+                        jsval sent;
+                        if ( JS_GetProperty( cx, glob, "_finalStatus", &sent ) )
+                            finalStatus = JSVAL_TO_INT(sent);
+                        else
+                            finalStatus = nsInstall::UNEXPECTED_ERROR;
+                    }
+                    JS_EndRequest(cx); //Decrement JS thread counter
+                    JS_DestroyContextMaybeGC(cx);
                 }
                 else
                 {
-                    // check to make sure the script sent back a status -- if
-                    // not the install may have been syntactically correct but
-                    // left the init/(perform|cancel) transaction open
-
-                    if(JS_GetProperty(cx, glob, "_installedFiles", &installedFiles) &&
-                       JSVAL_TO_BOOLEAN(installedFiles))
-                    {
-                        // install items remain in queue, must clean up!
-                        nsInstall *a = (nsInstall*)JS_GetPrivate(cx, glob);
-                        a->InternalAbort(nsInstall::MALFORMED_INSTALL);
-                    }
-
-                    jsval sent;
-                    if ( JS_GetProperty( cx, glob, "_finalStatus", &sent ) )
-                        finalStatus = JSVAL_TO_INT(sent);
-                    else
-                        finalStatus = nsInstall::UNEXPECTED_ERROR;
+                    // couldn't initialize install context
+                    finalStatus = nsInstall::UNEXPECTED_ERROR;
                 }
-                JS_EndRequest(cx); //Decrement JS thread counter
-                JS_DestroyContextMaybeGC(cx);
-            }
-            else
-            {
-                // couldn't initialize install context
-                finalStatus = nsInstall::UNEXPECTED_ERROR;
-            }
 
-            // clean up Runtime if we created it ourselves
-            if ( ownRuntime )
+                // Clean up after ourselves.
                 JS_DestroyRuntime(rt);
+            }
         }
         // force zip archive closed before other cleanup
         hZip = 0;
     }
-    else
-    {
-        // no path to local jar archive
-        finalStatus = nsInstall::DOWNLOAD_ERROR;
-    }
 
     if(listener)
         listener->OnInstallDone( installInfo->GetURL(), finalStatus );
@@ -594,7 +595,7 @@ extern "C" void RunInstallOnThread(void *data)
 // How do we get it there? Maybe just alerts on errors, could also dump to
 // the new console service.
 //-----------------------------------------------------------------------------
-extern "C" void RunChromeInstallOnThread(void *data)
+extern "C" void PR_CALLBACK RunChromeInstallOnThread(void *data)
 {
     nsresult rv;
 
@@ -606,73 +607,75 @@ extern "C" void RunChromeInstallOnThread(void *data)
         listener->OnInstallStart(info->GetURL());
 
     // make sure we've got a chrome registry -- can't proceed if not
-    nsIXULChromeRegistry* reg = info->GetChromeRegistry();
-#ifndef MOZ_XUL_APP
-    nsCOMPtr<nsIChromeRegistrySea> cr = do_QueryInterface(reg);
-#endif
+    CHROMEREG_IFACE* reg = info->GetChromeRegistry();
+    NS_ASSERTION(reg, "We shouldn't get here without a chrome registry.");
+
     if (reg)
     {
-        // build up jar: URL
-        nsCString spec;
-        spec.SetCapacity(200);
-        spec = "jar:";
+#ifdef MOZ_XUL_APP
+        if (info->GetType() == CHROME_SKIN) {
+            static NS_DEFINE_CID(kZipReaderCID,  NS_ZIPREADER_CID);
+            nsCOMPtr<nsIZipReader> hZip = do_CreateInstance(kZipReaderCID, &rv);
+            if (NS_SUCCEEDED(rv) && hZip) {
+                rv = hZip->Open(info->GetFile());
+                if (NS_SUCCEEDED(rv))
+                {
+                    rv = hZip->Test("install.rdf");
+                    nsIExtensionManager* em = info->GetExtensionManager();
+                    if (NS_SUCCEEDED(rv) && em) {
+                        rv = em->InstallItemFromFile(info->GetFile(), 
+                                                     NS_INSTALL_LOCATION_APPPROFILE);
+                    }
+                }
+                hZip->Close();
+            }
+            // Extension Manager copies the theme .jar file to 
+            // a different location, so remove the temporary file.
+            info->GetFile()->Remove(PR_FALSE);
+        }
+#else
+        PRBool isSkin    = (info->GetType() & CHROME_SKIN);
+        PRBool isLocale  = (info->GetType() & CHROME_LOCALE);
+        PRBool isContent = (info->GetType() & CHROME_CONTENT);
+        PRBool selected  = (info->GetFlags() != 0);
 
-        nsCAutoString localURL;
-        rv = NS_GetURLSpecFromFile(info->GetFile(), localURL);
-        if (NS_SUCCEEDED(rv)) {
-            spec.Append(localURL);
-            spec.Append("!/");
+        const nsCString& spec = info->GetFileJARSpec();
+
+        if ( isContent )
+            rv = reg->InstallPackage(spec.get(), PR_TRUE);
+
+        if ( isSkin )
+        {
+            rv = reg->InstallSkin(spec.get(), PR_TRUE, PR_FALSE);
+                
+            if (NS_SUCCEEDED(rv) && selected)
+            {
+                NS_ConvertUTF16toUTF8 utf8Args(info->GetArguments());
+                rv = reg->SelectSkin(utf8Args, PR_TRUE);
+            }
         }
 
-        // Now register the new chrome
-        if (NS_SUCCEEDED(rv))
+        if ( isLocale )
         {
-            PRBool isSkin    = (info->GetType() & CHROME_SKIN);
-            PRBool isLocale  = (info->GetType() & CHROME_LOCALE);
-            PRBool isContent = (info->GetType() & CHROME_CONTENT);
-            PRBool selected  = (info->GetFlags() != 0);
+            rv = reg->InstallLocale(spec.get(), PR_TRUE);
 
-            if ( isContent )
+            if (NS_SUCCEEDED(rv) && selected)
             {
-                rv = reg->InstallPackage(spec.get(), PR_TRUE);
+                NS_ConvertUTF16toUTF8 utf8Args(info->GetArguments());
+                rv = reg->SelectLocale(utf8Args, PR_TRUE);
             }
+        }
 
-            if ( isSkin )
-            {
-              rv = reg->InstallSkin(spec.get(), PR_TRUE, PR_FALSE);
-
-#ifndef MOZ_XUL_APP
-              if (NS_SUCCEEDED(rv) && selected && cr)
-              {
-                  NS_ConvertUCS2toUTF8 utf8Args(info->GetArguments());
-                  cr->SelectSkin(utf8Args, PR_TRUE);
-              }
-#endif
-            }
-
-            if ( isLocale )
-            {
-                rv = reg->InstallLocale(spec.get(), PR_TRUE);
-
-#ifndef MOZ_XUL_APP
-                if (NS_SUCCEEDED(rv) && selected && cr)
-                {
-                    NS_ConvertUCS2toUTF8 utf8Args(info->GetArguments());
-                    cr->SelectLocale(utf8Args, PR_TRUE);
-                }
-#endif
-            }
-
-            // now that all types are registered try to activate
-            if ( isSkin && selected )
-                reg->RefreshSkins();
+        // now that all types are registered try to activate
+        if ( isSkin && selected )
+            reg->RefreshSkins();
 
 #ifdef RELOAD_CHROME_WORKS
 // XXX ReloadChrome() crashes right now
             if ( isContent || (isLocale && selected) )
                 reg->ReloadChrome();
 #endif
-        }
+#endif
     }
 
     if (listener)
