@@ -1,4 +1,3 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -17,12 +16,11 @@
  *
  * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998-1999
+ * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *   Daniel Veditz <dveditz@netscape.com>
- *   Pierre Phaneuf <pp@ludusdesign.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -41,6 +39,8 @@
 #include "nscore.h"
 #include "pratom.h"
 #include "prmem.h"
+#include "prprf.h"
+#include "nsInt64.h"
 
 #include "nsISupports.h"
 #include "nsIServiceManager.h"
@@ -54,6 +54,7 @@
 #include "nsIInputStream.h"
 #include "nsIFileStreams.h"
 #include "nsIStreamListener.h"
+#include "nsICryptoHash.h"
 
 #include "nsISoftwareUpdate.h"
 #include "nsSoftwareUpdateIIDs.h"
@@ -66,7 +67,6 @@
 #include "nsIWindowWatcher.h"
 #include "nsIWindowMediator.h"
 #include "nsIDOMWindowInternal.h"
-#include "nsPIDOMWindow.h"
 #include "nsDirectoryService.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsAppDirectoryServiceDefs.h"
@@ -84,14 +84,16 @@
 
 #include "CertReader.h"
 
-static NS_DEFINE_IID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
+#include "nsEmbedCID.h"
+
 static NS_DEFINE_IID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 
-#include "nsIEventQueueService.h"
-
-#define PREF_XPINSTALL_CONFIRM_DLG "xpinstall.dialog.confirm"
-#define PREF_XPINSTALL_STATUS_DLG "xpinstall.dialog.progress"
-#define PREF_XPINSTALL_STATUS_DLG_TYPE "xpinstall.dialog.progress.type"
+#define PREF_XPINSTALL_ENABLED                "xpinstall.enabled"
+#define PREF_XPINSTALL_CONFIRM_DLG            "xpinstall.dialog.confirm"
+#define PREF_XPINSTALL_STATUS_DLG_SKIN        "xpinstall.dialog.progress.skin"
+#define PREF_XPINSTALL_STATUS_DLG_CHROME      "xpinstall.dialog.progress.chrome"
+#define PREF_XPINSTALL_STATUS_DLG_TYPE_SKIN   "xpinstall.dialog.progress.type.skin"
+#define PREF_XPINSTALL_STATUS_DLG_TYPE_CHROME "xpinstall.dialog.progress.type.chrome"
 
 #define UPDATE_DLG(x)  (((x) - mLastUpdate) > 250000)
 
@@ -107,7 +109,7 @@ inline PRBool nsXPInstallManager::TimeToUpdate(PRTime now)
 nsXPInstallManager::nsXPInstallManager()
   : mTriggers(0), mItem(0), mNextItem(0), mNumJars(0), mChromeType(NOT_CHROME),
     mContentLength(0), mDialogOpen(PR_FALSE), mCancelled(PR_FALSE),
-    mSelectChrome(PR_TRUE), mNeedsShutdown(PR_FALSE)
+    mSelectChrome(PR_FALSE), mNeedsShutdown(PR_FALSE)
 {
     // we need to own ourself because we have a longer
     // lifetime than the scriptlet that created us.
@@ -124,18 +126,15 @@ nsXPInstallManager::nsXPInstallManager()
 
 nsXPInstallManager::~nsXPInstallManager()
 {
-    nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
-    if (os)
-        os->RemoveObserver(this, XPI_PROGRESS_TOPIC);
- 
     if (mTriggers)
         delete mTriggers;
 }
 
 
-NS_IMPL_THREADSAFE_ISUPPORTS8( nsXPInstallManager,
+NS_IMPL_THREADSAFE_ISUPPORTS9( nsXPInstallManager,
                                nsIXPIListener,
                                nsIXPIDialogService,
+                               nsIXPInstallManager,
                                nsIObserver,
                                nsIStreamListener,
                                nsIProgressEventSink,
@@ -143,6 +142,66 @@ NS_IMPL_THREADSAFE_ISUPPORTS8( nsXPInstallManager,
                                nsPICertNotification,
                                nsISupportsWeakReference)
 
+NS_IMETHODIMP
+nsXPInstallManager::InitManagerFromChrome(const PRUnichar **aURLs,
+                                          PRUint32 aURLCount,
+                                          nsIXPIProgressDialog* aListener)
+{
+    return InitManagerWithHashes(aURLs, nsnull, aURLCount, aListener);
+}
+
+NS_IMETHODIMP
+nsXPInstallManager::InitManagerWithHashes(const PRUnichar **aURLs,
+                                          const char **aHashes,
+                                          PRUint32 aURLCount,
+                                          nsIXPIProgressDialog* aListener)
+{
+    // If Software Installation is not enabled, we don't want to proceed with
+    // update. 
+    PRBool xpinstallEnabled = PR_TRUE;
+    nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID));
+    if (pref) 
+        pref->GetBoolPref(PREF_XPINSTALL_ENABLED, &xpinstallEnabled);
+
+    if (!xpinstallEnabled)
+        return NS_OK;
+  
+    mTriggers = new nsXPITriggerInfo();
+    if (!mTriggers)
+        return NS_ERROR_OUT_OF_MEMORY;
+
+    mNeedsShutdown = PR_TRUE;
+
+    for (PRUint32 i = 0; i < aURLCount; ++i) 
+    {
+        nsXPITriggerItem* item = new nsXPITriggerItem(0, aURLs[i], nsnull,
+                                                      aHashes ? aHashes[i] : nsnull);
+        if (!item)
+        {
+            delete mTriggers; // nsXPITriggerInfo frees any alloc'ed nsXPITriggerItems
+            mTriggers = nsnull;
+            Shutdown();
+            return NS_ERROR_OUT_OF_MEMORY;
+        }
+        mTriggers->Add(item);
+    }
+
+    nsresult rv;
+    mInstallSvc = do_GetService(nsSoftwareUpdate::GetCID(), &rv);
+    if (NS_FAILED(rv)) 
+    {
+        delete mTriggers;
+        mTriggers = nsnull;
+        Shutdown();
+        return rv;
+    }
+
+    rv = Observe(aListener, XPI_PROGRESS_TOPIC, NS_LITERAL_STRING("open").get());
+    if (NS_FAILED(rv))
+        Shutdown();
+    return rv;
+}
+
 
 NS_IMETHODIMP
 nsXPInstallManager::InitManager(nsIScriptGlobalObject* aGlobalObject, nsXPITriggerInfo* aTriggers, PRUint32 aChromeType)
@@ -162,25 +221,18 @@ nsXPInstallManager::InitManager(nsIScriptGlobalObject* aGlobalObject, nsXPITrigg
 
     mParentWindow = do_QueryInterface(aGlobalObject);
 
-    // Don't launch installs while page is still loading
-    nsCOMPtr<nsPIDOMWindow> piWindow = do_QueryInterface(mParentWindow);
-    if (piWindow && piWindow->IsLoadingOrRunningTimeout())
-        rv = NS_ERROR_FAILURE;
+    // Start downloading initial chunks looking for signatures,
+    mOutstandingCertLoads = mTriggers->Size();
+
+    nsXPITriggerItem *item = mTriggers->Get(--mOutstandingCertLoads);
+
+    nsCOMPtr<nsIURI> uri;
+    NS_NewURI(getter_AddRefs(uri), NS_ConvertUTF16toUTF8(item->mURL));
+    nsCOMPtr<nsIStreamListener> listener = new CertReader(uri, nsnull, this);
+    if (listener)
+        rv = NS_OpenURI(listener, nsnull, uri);
     else
-    {
-        // Start downloading initial chunks looking for signatures,
-        mOutstandingCertLoads = mTriggers->Size();
-
-        nsXPITriggerItem *item = mTriggers->Get(--mOutstandingCertLoads);
-
-        nsCOMPtr<nsIURI> uri;
-        NS_NewURI(getter_AddRefs(uri), NS_ConvertUCS2toUTF8(item->mURL));
-        nsCOMPtr<nsIStreamListener> listener = new CertReader(uri, nsnull, this);
-        if (listener)
-            rv = NS_OpenURI(listener, nsnull, uri);
-        else
-            rv = NS_ERROR_OUT_OF_MEMORY;
-    }
+        rv = NS_ERROR_OUT_OF_MEMORY;
 
     if (NS_FAILED(rv)) {
         Shutdown();
@@ -222,7 +274,7 @@ nsXPInstallManager::InitManagerInternal()
         {
             nsXPITriggerItem *item = mTriggers->Get(i);
             packageList[j++] = item->mName.get();
-            packageList[j++] = item->mURL.get();
+            packageList[j++] = item->GetSafeURLString();
             packageList[j++] = item->mIconURL.get();
             packageList[j++] = item->mCertName.get();
         }
@@ -231,21 +283,28 @@ nsXPInstallManager::InitManagerInternal()
         // Get permission to install
         //-----------------------------------------------------
 
+#ifdef ENABLE_SKIN_SIMPLE_INSTALLATION_UI
         if ( mChromeType == CHROME_SKIN )
         {
+            // We may want to enable the simple installation UI once
+            // bug 343037 is fixed
+
             // skins get a simpler/friendlier dialog
             // XXX currently not embeddable
             OKtoInstall = ConfirmChromeInstall( mParentWindow, packageList );
         }
         else
         {
+#endif
             rv = dlgSvc->ConfirmInstall( mParentWindow,
                                          packageList,
                                          numStrings,
                                          &OKtoInstall );
             if (NS_FAILED(rv))
                 OKtoInstall = PR_FALSE;
+#ifdef ENABLE_SKIN_SIMPLE_INSTALLATION_UI
         }
+#endif
 
         if (OKtoInstall)
         {
@@ -309,15 +368,16 @@ nsXPInstallManager::ConfirmInstall(nsIDOMWindow *aParent, const PRUnichar **aPac
         ifptr->SetDataIID(&NS_GET_IID(nsIDialogParamBlock));
 
         char* confirmDialogURL;
-        nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID));
-        if (pref) {
-          rv = pref->GetCharPref(PREF_XPINSTALL_CONFIRM_DLG, &confirmDialogURL);
-          NS_ASSERTION(NS_SUCCEEDED(rv), "Can't invoke XPInstall FE without a FE URL! Set xpinstall.dialog.confirm");
-          if (NS_FAILED(rv))
-            return rv;
-        }
+        nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+        if (!pref)
+          return rv;
+          
+        rv = pref->GetCharPref(PREF_XPINSTALL_CONFIRM_DLG, &confirmDialogURL);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Can't invoke XPInstall FE without a FE URL! Set xpinstall.dialog.confirm");
+        if (NS_FAILED(rv))
+          return rv;
 
-        rv = parentWindow->OpenDialog(NS_ConvertASCIItoUCS2(confirmDialogURL),
+        rv = parentWindow->OpenDialog(NS_ConvertASCIItoUTF16(confirmDialogURL),
                                       NS_LITERAL_STRING("_blank"),
                                       NS_LITERAL_STRING("chrome,centerscreen,modal,titlebar"),
                                       ifptr,
@@ -335,6 +395,7 @@ nsXPInstallManager::ConfirmInstall(nsIDOMWindow *aParent, const PRUnichar **aPac
     return rv;
 }
 
+#ifdef ENABLE_SKIN_SIMPLE_INSTALLATION_UI
 PRBool nsXPInstallManager::ConfirmChromeInstall(nsIDOMWindowInternal* aParentWindow, const PRUnichar **aPackage)
 {
     // get the dialog strings
@@ -380,21 +441,19 @@ PRBool nsXPInstallManager::ConfirmChromeInstall(nsIDOMWindowInternal* aParentWin
 
     // confirmation dialog
     PRBool bInstall = PR_FALSE;
-    nsCOMPtr<nsIPromptService> dlgService(do_GetService("@mozilla.org/embedcomp/prompt-service;1"));
+    nsCOMPtr<nsIPromptService> dlgService(do_GetService(NS_PROMPTSERVICE_CONTRACTID));
     if (dlgService)
     {
-        dlgService->ConfirmCheck(
+        dlgService->Confirm(
             aParentWindow,
             nsnull,
             confirmText,
-            applyNowText,
-            &mSelectChrome,
             &bInstall );
     }
 
     return bInstall;
 }
-
+#endif
 
 NS_IMETHODIMP
 nsXPInstallManager::OpenProgressDialog(const PRUnichar **aPackageList, PRUint32 aCount, nsIObserver *aObserver)
@@ -427,42 +486,47 @@ nsXPInstallManager::OpenProgressDialog(const PRUnichar **aPackageList, PRUint32
 
     // --- open the window
     nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv));
-    if (wwatch) {
-        char *statusDialogURL, *statusDialogType;
-        nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID));
-        if (pref) {
-          rv = pref->GetCharPref(PREF_XPINSTALL_STATUS_DLG, &statusDialogURL);
-          NS_ASSERTION(NS_SUCCEEDED(rv), "Can't invoke XPInstall FE without a FE URL! Set xpinstall.dialog.status");
-          if (NS_FAILED(rv))
-            return rv;
+    if (!wwatch)
+        return rv;
+    
+    char *statusDialogURL, *statusDialogType;
+    nsCOMPtr<nsIPrefBranch> pref(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
+    if (!pref)
+        return rv;
+	const char* statusDlg = mChromeType == CHROME_SKIN ? PREF_XPINSTALL_STATUS_DLG_SKIN
+                                                         : PREF_XPINSTALL_STATUS_DLG_CHROME;
+	rv = pref->GetCharPref(statusDlg, &statusDialogURL);
+	NS_ASSERTION(NS_SUCCEEDED(rv), "Can't invoke XPInstall FE without a FE URL! Set xpinstall.dialog.status");
+	if (NS_FAILED(rv))
+		return rv;
 
-          rv = pref->GetCharPref(PREF_XPINSTALL_STATUS_DLG_TYPE, &statusDialogType);
-          nsAutoString type; 
-          type.AssignWithConversion(statusDialogType);
-          if (NS_SUCCEEDED(rv) && !type.IsEmpty()) {
-            nsCOMPtr<nsIWindowMediator> wm = do_GetService(NS_WINDOWMEDIATOR_CONTRACTID);
-          
-            nsCOMPtr<nsIDOMWindowInternal> recentWindow;
-            wm->GetMostRecentWindow(type.get(), getter_AddRefs(recentWindow));
-            if (recentWindow) {
-              nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
-              os->NotifyObservers(params, "xpinstall-download-started", nsnull);
+    const char* statusType = mChromeType == CHROME_SKIN ? PREF_XPINSTALL_STATUS_DLG_TYPE_SKIN
+                                                        : PREF_XPINSTALL_STATUS_DLG_TYPE_CHROME;
+    rv = pref->GetCharPref(statusType, &statusDialogType);
+    nsAutoString type; 
+    type.AssignWithConversion(statusDialogType);
+    if (NS_SUCCEEDED(rv) && !type.IsEmpty()) {
+        nsCOMPtr<nsIWindowMediator> wm = do_GetService(NS_WINDOWMEDIATOR_CONTRACTID);
+      
+        nsCOMPtr<nsIDOMWindowInternal> recentWindow;
+        wm->GetMostRecentWindow(type.get(), getter_AddRefs(recentWindow));
+        if (recentWindow) {
+            nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+            os->NotifyObservers(params, "xpinstall-download-started", nsnull);
 
-              recentWindow->Focus();
-              return NS_OK;
-            }
-          }
+            recentWindow->Focus();
+            return NS_OK;
         }
-
-        nsCOMPtr<nsIDOMWindow> newWindow;
-        rv = wwatch->OpenWindow(0, 
-                                statusDialogURL,
-                                "_blank", 
-                                "chrome,centerscreen,titlebar,resizable",
-                                params, 
-                                getter_AddRefs(newWindow));
     }
 
+    nsCOMPtr<nsIDOMWindow> newWindow;
+    rv = wwatch->OpenWindow(0, 
+                            statusDialogURL,
+                            "_blank", 
+                            "chrome,centerscreen,titlebar,dialog=no,resizable",
+                            params, 
+                            getter_AddRefs(newWindow));
+
     return rv;
 }
 
@@ -497,16 +561,11 @@ NS_IMETHODIMP nsXPInstallManager::Observe( nsISupports *aSubject,
             if (dlg)
             {
                 // --- create and save a proxy for the dialog
-                nsCOMPtr<nsIProxyObjectManager> pmgr =
-                            do_GetService(kProxyObjectManagerCID, &rv);
-                if (pmgr)
-                {
-                    rv = pmgr->GetProxyForObject( NS_UI_THREAD_EVENTQ,
-                                                  NS_GET_IID(nsIXPIProgressDialog),
-                                                  dlg,
-                                                  PROXY_SYNC | PROXY_ALWAYS,
-                                                  getter_AddRefs(mDlg) );
-                }
+                NS_GetProxyForObject( NS_PROXY_TO_MAIN_THREAD,
+                                      NS_GET_IID(nsIXPIProgressDialog),
+                                      dlg,
+                                      NS_PROXY_SYNC | NS_PROXY_ALWAYS,
+                                      getter_AddRefs(mDlg) );
             }
 
             // -- get the ball rolling
@@ -659,6 +718,31 @@ NS_IMETHODIMP nsXPInstallManager::DownloadNext()
                 continue;
             }
 
+            // If there was hash info in the trigger, but
+            // there wasn't a hash object created, then the
+            // algorithm used isn't known.
+
+            if (mItem->mHashFound && !mItem->mHasher)
+            {
+                // report failure
+                mTriggers->SendStatus( mItem->mURL.get(), nsInstall::INVALID_HASH_TYPE );
+                if (mDlg)
+                    mDlg->OnStateChange( i, nsIXPIProgressDialog::INSTALL_DONE,
+                                         nsInstall::INVALID_HASH_TYPE );
+                continue;
+            }
+
+            // Don't install if we can't verify the hash (if specified)
+            if (mItem->mHasher && !VerifyHash(mItem))
+            {
+                // report failure
+                mTriggers->SendStatus( mItem->mURL.get(), nsInstall::INVALID_HASH );
+                if (mDlg)
+                    mDlg->OnStateChange( i, nsIXPIProgressDialog::INSTALL_DONE,
+                                         nsInstall::INVALID_HASH );
+                continue;
+            }
+
             // We've got one to install; increment count first so we
             // don't have to worry about thread timing.
             PR_AtomicIncrement(&mNumJars);
@@ -704,6 +788,45 @@ NS_IMETHODIMP nsXPInstallManager::DownloadNext()
 }
 
 
+//-------------------------------------------------------------------
+// VerifyHash
+//
+// Returns true if the file hash matches the expected value (or if
+// the item has no hash value). False if we can't verify the hash
+// for any reason
+//
+PRBool nsXPInstallManager::VerifyHash(nsXPITriggerItem* aItem)
+{
+    NS_ASSERTION(aItem, "Null nsXPITriggerItem passed to VerifyHash");
+
+    nsresult rv;
+    if (!aItem->mHasher)
+      return PR_FALSE;
+    
+    nsCOMPtr<nsIInputStream> stream;
+    rv = NS_NewLocalFileInputStream(getter_AddRefs(stream), aItem->mFile);
+    if (NS_FAILED(rv)) return PR_FALSE;
+
+    rv = aItem->mHasher->UpdateFromStream(stream, PR_UINT32_MAX);
+    if (NS_FAILED(rv)) return PR_FALSE;
+
+    nsCAutoString binaryHash;
+    rv = aItem->mHasher->Finish(PR_FALSE, binaryHash);
+    if (NS_FAILED(rv)) return PR_FALSE;
+
+    char* hash = nsnull;
+    for (PRUint32 i=0; i < binaryHash.Length(); ++i)
+    {
+        hash = PR_sprintf_append(hash,"%.2x", (PRUint8)binaryHash[i]);
+    }
+
+    PRBool result = aItem->mHash.EqualsIgnoreCase(hash);
+
+    PR_smprintf_free(hash);
+    return result;
+}
+
+
 void nsXPInstallManager::Shutdown()
 {
     if (mDlg)
@@ -742,6 +865,20 @@ void nsXPInstallManager::Shutdown()
             }
         }
 
+        nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1"));
+        if (os)
+        {
+            nsresult rv;
+            nsCOMPtr<nsIObserverService> pos;
+            rv = NS_GetProxyForObject( NS_PROXY_TO_MAIN_THREAD,
+                                       NS_GET_IID(nsIObserverService),
+                                       os,
+                                       NS_PROXY_SYNC | NS_PROXY_ALWAYS,
+                                       getter_AddRefs(pos) );
+            if (NS_SUCCEEDED(rv))
+                pos->RemoveObserver(this, XPI_PROGRESS_TOPIC);
+        }
+
         NS_RELEASE_THIS();
     }
 }
@@ -749,9 +886,8 @@ void nsXPInstallManager::Shutdown()
 NS_IMETHODIMP
 nsXPInstallManager::LoadParams(PRUint32 aCount, const PRUnichar** aPackageList, nsIDialogParamBlock** aParams)
 {
-    nsIDialogParamBlock* paramBlock;
-    nsresult rv = CallCreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID, &paramBlock);
-
+    nsresult rv;
+    nsCOMPtr<nsIDialogParamBlock> paramBlock = do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv))
     {
         // set OK and Cancel buttons
@@ -764,7 +900,7 @@ nsXPInstallManager::LoadParams(PRUint32 aCount, const PRUnichar** aPackageList,
             paramBlock->SetString( i, aPackageList[i] );
     }
 
-    *aParams = paramBlock;
+    NS_IF_ADDREF(*aParams = paramBlock);
     return rv;
 }
 
@@ -795,7 +931,7 @@ nsXPInstallManager::GetDestinationFile(nsString& url, nsILocalFile* *file)
             if (NS_SUCCEEDED(rv))
             { 
                 temp->AppendNative(NS_LITERAL_CSTRING("tmp.xpi"));
-                temp->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0644);
+                temp->CreateUnique(nsIFile::NORMAL_FILE_TYPE, 0600);
                 *file = temp;
                 NS_IF_ADDREF(*file);
             }
@@ -845,6 +981,18 @@ nsXPInstallManager::OnStartRequest(nsIRequest* request, nsISupports *ctxt)
 {
     nsresult rv = NS_ERROR_FAILURE;
 
+    // If we are dealing with a HTTP request, then treat HTTP error pages as
+    // download failures.
+    nsCOMPtr<nsIHttpChannel> httpChan = do_QueryInterface(request);
+    if (httpChan) {
+        PRBool succeeded;
+        if (NS_SUCCEEDED(httpChan->GetRequestSucceeded(&succeeded)) && !succeeded) {
+            // HTTP response is not a 2xx!
+            request->Cancel(NS_BINDING_ABORTED);
+            return NS_OK;
+        }
+    }
+
     NS_ASSERTION( mItem && mItem->mFile, "XPIMgr::OnStartRequest bad state");
     if ( mItem && mItem->mFile )
     {
@@ -853,7 +1001,7 @@ nsXPInstallManager::OnStartRequest(nsIRequest* request, nsISupports *ctxt)
         rv = NS_NewLocalFileOutputStream(getter_AddRefs(mItem->mOutStream),
                                          mItem->mFile,
                                          PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE,
-                                         0664);
+                                         0600);
     }
     return rv;
 }
@@ -932,9 +1080,10 @@ nsXPInstallManager::OnDataAvailable(nsIRequest* request, nsISupports *ctxt,
                                     PRUint32 sourceOffset,
                                     PRUint32 length)
 {
-    PRUint32 amt;
+#define XPI_ODA_BUFFER_SIZE 8*1024
+    PRUint32 amt = PR_MIN(XPI_ODA_BUFFER_SIZE, length);
     nsresult err;
-    char buffer[8*1024];
+    char buffer[XPI_ODA_BUFFER_SIZE];
     PRUint32 writeCount;
 
     if (mCancelled)
@@ -947,19 +1096,20 @@ nsXPInstallManager::OnDataAvailable(nsIRequest* request, nsISupports *ctxt,
 
     do
     {
-        err = pIStream->Read(buffer, sizeof(buffer), &amt);
+        err = pIStream->Read(buffer, amt, &amt);
+
         if (amt == 0) break;
-        if (NS_FAILED(err))
-        {
-            //printf("pIStream->Read Failed!  %d", err);
-            return err;
-        }
+        if (NS_FAILED(err)) return err;
+
         err = mItem->mOutStream->Write( buffer, amt, &writeCount);
         if (NS_FAILED(err) || writeCount != amt)
         {
             return NS_ERROR_FAILURE;
         }
         length -= amt;
+
+        amt = PR_MIN(XPI_ODA_BUFFER_SIZE, length);
+
     } while (length > 0);
 
     return NS_OK;
@@ -967,7 +1117,7 @@ nsXPInstallManager::OnDataAvailable(nsIRequest* request, nsISupports *ctxt,
 
 
 NS_IMETHODIMP
-nsXPInstallManager::OnProgress(nsIRequest* request, nsISupports *ctxt, PRUint32 aProgress, PRUint32 aProgressMax)
+nsXPInstallManager::OnProgress(nsIRequest* request, nsISupports *ctxt, PRUint64 aProgress, PRUint64 aProgressMax)
 {
     nsresult rv = NS_OK;
 
@@ -982,7 +1132,8 @@ nsXPInstallManager::OnProgress(nsIRequest* request, nsISupports *ctxt, PRUint32
             if (NS_FAILED(rv)) return rv;
         }
         mLastUpdate = now;
-        rv = mDlg->OnProgress( mNextItem-1, aProgress, mContentLength );
+        // XXX once channels support that, use 64-bit contentlength
+        rv = mDlg->OnProgress( mNextItem-1, aProgress, nsUint64(mContentLength) );
     }
 
     return rv;
@@ -1108,7 +1259,7 @@ nsXPInstallManager::OnCertAvailable(nsIURI *aURI,
     item = mTriggers->Get(--mOutstandingCertLoads);
 
     nsCOMPtr<nsIURI> uri;
-    NS_NewURI(getter_AddRefs(uri), NS_ConvertUCS2toUTF8(item->mURL.get()).get());
+    NS_NewURI(getter_AddRefs(uri), NS_ConvertUTF16toUTF8(item->mURL.get()).get());
 
     if (!uri || mChromeType != NOT_CHROME)
         return OnCertAvailable(uri, context, NS_ERROR_FAILURE, nsnull);
